//
//  BestResponse.java
//  BoundedDegreeNetwork
//
//  Created by Jason Alexander on Tue Sep 10 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//
import java.util.*;

public class BestResponse implements LearningRule {

    public int nextStrategy(Agent agent) {
		
        int N = agent.getPayoffMatrix().getNumberOfStrategies();

        double expected_scores[] = new double[N];

        Iterator itr = agent.constructUpdateNeighborhood().iterator();
        int i; // loop index
        
        // Now loop over the update_neighborhood calculating the
        // expected scores of each strategy, assuming that the people
        // in the update neighborhood don't change their strategy
        while (itr.hasNext()) {
            Agent n = (Agent)itr.next();
            for (i=0; i<N; i++) {
                expected_scores[i] += (
									   0.5*agent.getPayoffMatrix().getPayoff(i, n.getStrategy(), PayoffMatrix.ROW)
									   +
									   0.5*agent.getPayoffMatrix().getPayoff(n.getStrategy(), i, PayoffMatrix.COLUMN)
									   );
            }
        }

        // Construct the list of top scoring strategies and randomly select
        // one of the best.
        double max_score = Double.NEGATIVE_INFINITY;
        for (i=0; i<N; i++) {
            if (expected_scores[i] > max_score)
                max_score = expected_scores[i];
        }

        LinkedList top_strategies = new LinkedList();
        for (i=0; i<N; i++) {
            if (expected_scores[i] == max_score)
                top_strategies.add(new Integer(i));
        }
        
		int size = top_strategies.size();
		int randomInt = agent.parentModel.random.nextInt( size );
        return ((Integer)top_strategies.get(randomInt)).intValue();
    }
    
}
