import java.lang.reflect.*;
import java.util.*;
import com.wolfram.jlink.*;

public class CombinatoricaGraph {
    public ArrayList vertexList = new ArrayList();
    public ArrayList edgeList = new ArrayList();
	
    public CombinatoricaGraph() { }
	
    public void addVertices(float positions[][]) {
		int i;
		int size = Array.getLength(positions);
		vertexList = new ArrayList(size);
		
		for (i=0; i < size; i++) {
			vertexList.add(new Vertex(positions[i][0], positions[i][1]));
		}
    }
	
    public void addEdges(int edges[][]) {
		int i;
		int size = Array.getLength(edges);
		edgeList = new ArrayList(size);
		for (i=0; i < size; i++) {
			edgeList.add(new Edge(edges[i][0], edges[i][1]));
		}
    }
	
    public void changeEdgeWeightStatus(boolean weighted[]) {
        int i;
		for (i=0; i < Array.getLength(weighted); i++) {
			Edge e = (Edge) edgeList.get(i);
			e.weighted = weighted[i];
		}
    }
	
    public void changeEdgeColors(float colors[][]) {
        int i;
		for (i=0; i < Array.getLength(colors); i++) {
			Edge e = (Edge) edgeList.get(i);
			e.red = colors[i][0];
			e.green = colors[i][1];
			e.blue = colors[i][2];
		}
    }	
	
	
    public void changeEdgeWeights(double weights[]) {
		int i;
		for (i=0; i < Array.getLength(weights); i++) {
			Edge e = (Edge) edgeList.get(i);
			e.weight = weights[i];
		}
    }
	
    public void changeEdgeDirectionStatus(boolean directed[]) {
		int i;
		for (i=0; i < Array.getLength(directed); i++) {
			Edge e = (Edge) edgeList.get(i);
			e.directed = directed[i];
		}
    }
	
    public Expr toExpr() {
		Expr edges = getEdgeList();
		Expr vertices = getVertexList();
		Expr graph = ExprFactory.expr("Graph", edges, vertices );
		edges.dispose();
		vertices.dispose();
		return graph;
    }
	
	public void sendToMathematica(KernelLink kl) throws MathLinkException {
		kl.putFunction("Graph", 2);
		// Write out edges
		kl.putFunction("List", edgeList.size());
		for (int i=0; i < edgeList.size(); i++) {
			Edge e = (Edge) edgeList.get(i);
			e.sendToMathematica(kl);
		}
		// Write out vertices
		kl.putFunction("List", vertexList.size());
		for(int i=0; i < vertexList.size(); i++) {
			Vertex v = (Vertex) vertexList.get(i);
			kl.putFunction("List", 1);
			kl.putFunction("List", 2);
			kl.put(v.x); 
			kl.put(v.y);
		}		
	}
	
    public Expr getVertexList() {
		int size = vertexList.size();
		Expr vertices[] = new Expr[size];
		for(int i=0; i<size; i++) {
			Vertex v = (Vertex) vertexList.get(i);
			vertices[i] = v.toExpr();
		}
		Expr l = ExprFactory.list( vertices );
		ExprFactory.disposeAll( vertices );
		return l;
    }
	
    public Expr getEdgeList () {
		int size = edgeList.size();
		Expr edges[] = new Expr[size];
		int i;
		for (i=0; i<size; i++) {
			Edge e = (Edge) edgeList.get(i);
			edges[i] = e.toExpr();
		}
		Expr l = ExprFactory.list( edges );
		ExprFactory.disposeAll( edges );
		return l;
    }
	
    public static class Vertex {
		float x,y;
		public Vertex(float x, float y) {
			this.x = x;
			this.y = y;
		}
		
		public Expr toExpr() {
			return ExprFactory.list(
									ExprFactory.list(x,y)
									);
		}

    }
	
    protected static class Edge {
		int one,two;
		double weight=0.0;
		float red,green,blue;
		boolean weighted=false;
		boolean directed=false;
		
		public Edge(int one, int two) {
			this.one = one;
			this.two = two;
		}
		
		public Edge(int one, int two, double w) {
			this.one = one;
			this.two = two;
			this.weight = w;
			this.weighted = true;
		}
		
		public void sendToMathematica(KernelLink kl) throws MathLinkException {
			int thingsToTransfer=1; // For vertex list;
			if (weighted)
				thingsToTransfer++; // Need to transfer EdgeWeight->W
			if (directed)
				thingsToTransfer++; // Need to transfer EdgeDirection->True
			if (red != 0.0 || green != 0.0 || blue != 0.0)
				thingsToTransfer++; // Only transfer EdgeColor->... if not black
			
			kl.putFunction("List", thingsToTransfer);
			kl.putFunction("List",2); 
			{
				kl.put(one);
				kl.put(two);
			}
			if (weighted) {
				kl.putFunction("Rule", 2);
				{
					kl.putSymbol("EdgeWeight");
					kl.put(weight);
				}
			}
			if (directed) {
				kl.putFunction("Rule", 2);
				{
					kl.putSymbol("EdgeDirection");
					kl.putSymbol("True");
				}
			}
			if (red != 0.0 || green != 0.0 || blue != 0.0) {
				kl.putFunction("Rule", 2);
				{
					kl.putSymbol("EdgeColor");
					kl.putFunction("RGBColor", 3);
					{
						kl.put(red);
						kl.put(green);
						kl.put(blue);
					}
				}
			}
		}
		
		public Expr toExpr() {
			// the vertices
			Expr e1 = ExprFactory.list( one, two );
			
			Expr e2 = null;
			if (weighted) 
				e2 = ExprFactory.rule("EdgeWeight", weight);
			
			Expr e3 = null;
			if (directed)
				e3 = ExprFactory.rule("EdgeDirection", true);
			
			Expr rgbColors = ExprFactory.rgbColor(red,green,blue);
			
			Expr e4 = ExprFactory.rule( "EdgeColor", rgbColors );
			
			rgbColors.dispose();	    
			
			Expr e = ExprFactory.list( e1 );
			
			Expr newE;
			if (e2 != null) {
				newE = e.insert(e2, -1);
				e.dispose();
				e = newE;
				e2.dispose();
			}
			
			if (e3 != null) {
				newE = e.insert(e3, -1);
				e.dispose();
				e = newE;
				e3.dispose();
			}
			
			newE = e.insert(e4, -1);
			e.dispose();
			
			return newE;
		}
		 
    }
	
}