import com.wolfram.jlink.*;
import java.lang.reflect.*;

public class ExprFactory {
    public final static Expr True = new Expr(Expr.SYMBOL, "True");
    public final static Expr False = new Expr(Expr.SYMBOL, "False");
    public final static Expr List = new Expr(Expr.SYMBOL, "List");
    public final static Expr Rule = new Expr(Expr.SYMBOL, "Rule");
	
    public static void disposeAll(Expr list[]) {
		int size = Array.getLength(list);
		for (int i=0; i<size; i++) {
			list[i].dispose();
		}
    }
	
    public static Expr expr(String head, Expr one) {
        return new Expr(
						new Expr(Expr.SYMBOL, head),
						new Expr[] { one }
						);
    }
	
    public static Expr expr(String head, Expr one, Expr two) {
        return new Expr(
						new Expr(Expr.SYMBOL, head),
						new Expr[] { one, two }
						);
    }
	
    public static Expr expr(String head, Expr one, Expr two, Expr three) {
        return new Expr(
						new Expr(Expr.SYMBOL, head),
						new Expr[] { one, two, three }
						);
    }
	
	
    public static Expr expr(String head, Expr body[]) {
        return new Expr(
						new Expr(Expr.SYMBOL, head),
						body
						);
    }
	
    public static Expr list(int one, int two) {
		return new Expr ( new int[] { one, two } );
    }
	
    public static Expr list(double one, double two) {
		return new Expr ( new double [] { one, two } );
    }
	
    public static Expr list(Expr e) {
		return new Expr( List, new Expr[] { e } );
    }
	
    public static Expr list(Expr l[]) {
		return new Expr( List, l );
    }
	
    public static Expr rule(String symbol, double value) {
		return new Expr(
						Rule,
						new Expr[] {
							new Expr(Expr.SYMBOL, symbol),
							new Expr(value)
						});
    }
    
    public static Expr rule(String symbol, boolean b) {
		if (b)
			return new Expr(
							Rule,
							new Expr[] {
								new Expr(Expr.SYMBOL, symbol),
								True
							});
		else
			return new Expr(
							Rule,
							new Expr[] {
								new Expr(Expr.SYMBOL, symbol),
								False
							});
    }
	
    public static Expr rule(String symbol, Expr e) {
		return new Expr(
						Rule,
						new Expr[] {
							new Expr( Expr.SYMBOL, symbol ),
							e
						});
    }
	
    public static Expr rgbColor(double r, double g, double b) {
		return new Expr(
						new Expr( Expr.SYMBOL, "RGBColor"),
						new Expr[] {
							new Expr(r), new Expr(g), new Expr(b)
						});
    }
	
}