//file: NashView.java
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.math.*;
import com.wolfram.jlink.*;
import java.util.Iterator;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

class View extends MathJFrame implements ChangeListener {
    Model model;
	
	MathGraphicsJPanel mathCanvas;
	DrawPanel drawPanel;
	
	String showGraphHead;
	
	final static int MATHEMATICA_GRAPHICS = 0;
	final static int FAST_JAVA_GRAPHICS   = 1;
	int type;
	
    public View (Model m, int t) {
		super();
		type = t;
		model = m;
		showGraphHead = "ShowGraph";
	}
	
	public void layoutView() {
		getContentPane().setLayout(new BorderLayout() );
		if (type == MATHEMATICA_GRAPHICS) {
			mathCanvas = new MathGraphicsJPanel();
			mathCanvas.setUsesFE(true);
			mathCanvas.setPreferredSize( new Dimension(450,450) );
			getContentPane().add("Center", mathCanvas);
		}
		else {
			drawPanel = new DrawPanel();
			drawPanel.showGraph( model.getDisplayGraph() );

			drawPanel.setPreferredSize( new Dimension(450,450) );
			drawPanel.setStatus("movevertex");
			addMouseListener( drawPanel );
			getContentPane().add("Center", drawPanel);
		}
		pack();
	}
	
	public String getNodeColors() {
		StringBuffer sb = new StringBuffer(1000);
		Agent agents[] = model.getAgents();
		int i;
		for (i=0; i<agents.length; i++) {
			Color c = agents[i].getColor();
			sb.append("{" + (i+1) + ", VertexColor->RGBColor["
			          + (c.getRed()/255.0) + "," + (c.getGreen()/255.0) + "," + (c.getBlue()/255.0) + "]}");
			if (i != agents.length-1)
				sb.append(",");
		}
		return sb.toString();
	}
	
	public Expr getGraphAsDisplayed() {
		KernelLink kl = StdLink.getLink();
		if (type == MATHEMATICA_GRAPHICS) {
			try {
				kl.evaluate( 
							 "SetGraphOptions[" + model.getDisplayGraph().toString()
							 + ",{" + getNodeColors() + "}]"
							 );
				kl.waitForAnswer();
				Expr e = kl.getExpr();
				kl.newPacket();
				return e;
			}
			catch (MathLinkException e) {
				return new Expr("Null");
			}
		}
		else {
			return drawPanel.graphToExpr();
		}
	}
	
	public void updateGraphDisplay() {
		if (type == MATHEMATICA_GRAPHICS) {
			mathCanvas.setMathCommand( showGraphHead + "[SetGraphOptions[" + model.getDisplayGraph() 
							+ ",{" + getNodeColors() + "}]]" 
							);
			mathCanvas.recompute();
		}
		else {
			Agent agents[] = model.getAgents();
			Iterator itr = drawPanel.getVertices().iterator();
			int i=0;
			while (itr.hasNext()) {
				Color c = agents[i++].getColor();
				GEVertex vertex = (GEVertex) itr.next();
				vertex.color = c;
			}
		}
		repaint();
	}
	
	public void showVertexNumbers() {
		if (type == MATHEMATICA_GRAPHICS) {
			showGraphHead = "ShowLabeledGraph";
		}
		else {
			drawPanel.setShowVertexNumbers(true);
		}
	}
	
	public void hideVertexNumbers() {
		if (type == MATHEMATICA_GRAPHICS) {
			showGraphHead = "ShowGraph";
		}
		else {
			drawPanel.setShowVertexNumbers(false);
		}
	}

	public void stateChanged(ChangeEvent e) {
		updateGraphDisplay();
	}
	
}
