(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5792,        174]*)
(*NotebookOutlinePosition[      7509,        223]*)
(*  CellTagsIndexPosition[      7402,        216]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Evolve", "ObjectName",
  CellTags->{"b:6", "LocalInteractions:13"}],

Cell[TextData[{
  "\[FilledSmallSquare] ",
  StyleBox["Evolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        StyleBox["model",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox["]",
    FontFamily->"Courier"],
  " steps ",
  StyleBox["model",
    FontSlant->"Italic"],
  " forward one generation.\n",
  "\[FilledSmallSquare] ",
  StyleBox["Evolve[",
    FontFamily->"Courier"],
  Cell[BoxData[
      FormBox[
        StyleBox["model",
          FontSlant->"Italic"], TraditionalForm]]],
  ", ",
  StyleBox["options",
    FontSlant->"Italic"],
  StyleBox["]",
    FontFamily->"Courier"],
  " steps ",
  StyleBox["model",
    FontSlant->"Italic"],
  " forward according to the specified ",
  StyleBox["options",
    FontSlant->"Italic"],
  "."
}], "Usage",
  CellTags->{"b:6", "LocalInteractions:13"}],

Cell["\<\
\[FilledSmallSquare] The following options can be given:\
\>", \
"Notes",
  CellTags->{"b:6", "LocalInteractions:13"}],

Cell[TextData[Cell[BoxData[GridBox[{
        {"UpdateDisplay", "True", 
          StyleBox[\(refresh\ the\ display\ after\ each\ generation\),
            FontFamily->"Times"]},
        {"Generations", "1", 
          StyleBox[
            RowBox[{
              RowBox[{
                StyleBox["the",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["number",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["of",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["generations",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["to",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["step",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["model",
                  FontFamily->"Times",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Times",
                  FontSlant->"Plain"], 
                StyleBox["forward",
                  FontFamily->"Times",
                  FontSlant->"Plain"]}], 
              StyleBox[",",
                FontFamily->"Times",
                FontSlant->"Plain"], 
              StyleBox[" ",
                FontFamily->"Times",
                FontSlant->"Plain"], 
              StyleBox[\(refreshing\ the\ display\ after\ each\ generation\),
                FontFamily->"Times",
                FontSlant->"Plain"]}],
            FontFamily->"Times"]},
        {"DisplayPause", "0", 
          
          StyleBox[\(the\ number\ of\ seconds\ \((or\ fraction\ of\ a\ \
second)\)\ to\ pause\ after\ completing\ a\ generation\ before\ starting\ the\
\ next\ \(\(one\)\(.\)\)\),
            FontFamily->"Times"]}
        }]]]], "3ColumnTable",
  CellTags->{"b:6", "LocalInteractions:13"}],

Cell[TextData[{
  "\[FilledSmallSquare] Complex models with a lot of vertices or edges will \
often take sufficiently long to draw that the Java graphics engine will not \
have finished rendering the display by the time the next display update \
request is queued.  When this happens, the display will abort the current \
render and will start again.  If several aborted attempts occur, this will \
cause the model to evolve in \"jerks\" as multiple generations lapse between \
display refreshes.  Setting ",
  StyleBox["DisplayPause",
    FontFamily->"Courier"],
  " to an appropriate value (which depends on the model) allows for smooth \
updating of graphics across generations."
}], "Notes",
  CellTags->{"b:6", "LocalInteractions:13"}]
}, Open  ]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1152}, {0, 701}},
WindowSize->{520, 578},
WindowMargins->{{Automatic, 90}, {Automatic, 36}},
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "b:6"->{
    Cell[1776, 53, 73, 1, 38, "ObjectName",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[1852, 56, 835, 34, 54, "Usage",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[2690, 92, 128, 4, 24, "Notes",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[2821, 98, 2212, 58, 149, "3ColumnTable",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[5036, 158, 740, 13, 120, "Notes",
      CellTags->{"b:6", "LocalInteractions:13"}]},
  "LocalInteractions:13"->{
    Cell[1776, 53, 73, 1, 38, "ObjectName",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[1852, 56, 835, 34, 54, "Usage",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[2690, 92, 128, 4, 24, "Notes",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[2821, 98, 2212, 58, 149, "3ColumnTable",
      CellTags->{"b:6", "LocalInteractions:13"}],
    Cell[5036, 158, 740, 13, 120, "Notes",
      CellTags->{"b:6", "LocalInteractions:13"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"b:6", 6411, 190},
  {"LocalInteractions:13", 6908, 201}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 73, 1, 38, "ObjectName",
  CellTags->{"b:6", "LocalInteractions:13"}],
Cell[1852, 56, 835, 34, 54, "Usage",
  CellTags->{"b:6", "LocalInteractions:13"}],
Cell[2690, 92, 128, 4, 24, "Notes",
  CellTags->{"b:6", "LocalInteractions:13"}],
Cell[2821, 98, 2212, 58, 149, "3ColumnTable",
  CellTags->{"b:6", "LocalInteractions:13"}],
Cell[5036, 158, 740, 13, 120, "Notes",
  CellTags->{"b:6", "LocalInteractions:13"}]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

