(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   6473551,     174880]*)
(*NotebookOutlinePosition[   6528112,     176009]*)
(*  CellTagsIndexPosition[   6528011,     176003]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["LocalInteractions`LocalInteractions`", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  StyleBox["LocalInteractions`LocalInteractions`",
    FontFamily->"Courier"],
  " extends ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by over 50 functions for the construction of local interaction models of \
evolutionary games. It uses a special Java engine, hidden from the user, so \
that the evolutionary models can run as quickly as possible once they have \
been configured. This combination of Java and ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides for maximum flexibility and speed in the design and analysis of \
local interaction models of evolutionary games."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
I encourage you to visit the web site, \
http://evolve.lse.ac.uk/localinteractions/, where you will find the latest \
release of the package and addition documentation, as it becomes available.\
\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Acknowledgements", "Subsubsection"]], "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
I would like to thank Lloyd Levon and Steven Skiena (SUNY Stony \
Brook) for allowing me to incorporate a modified version of the Java Graph \
Editor in this software.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell["Getting started", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell["We'll begin with a number of examples. ", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "If the package has been installed correctly, it is loaded into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " as follows:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(<< LocalInteractions`LocalInteractions`;\)\), "\[IndentingNewLine]", 
    \(\(<< LocalInteractions`Graphs`;\)\)}], "Input",
  CellLabel->"In[1]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The second package provides a number of common topologies for local \
interaction models.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Let's start with ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  ", a two-strategy asymmetric game whose payoff matrix is the following:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"battleOfSexes", "=", 
        RowBox[{"(", GridBox[{
              {\({1, 2}\), \({0, 0}\)},
              {\({0, 0}\), \({2, 1}\)}
              }], ")"}]}], ";"}]], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "This payoff matrix is interpreted in the standard way: strategies are \
enumerated from 1 to ",
  StyleBox["N",
    FontSlant->"Italic"],
  " (in this case, ",
  StyleBox["N",
    FontSlant->"Italic"],
  "=2) beginning with the top row and leftmost column, working down or to the \
right."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "In order to indicate which agent follows which strategy, we need to \
specify a color for each strategy. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " comes with a number of colours predefined, so we can write"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(bosColors = {\ Red, Green};\)\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Alternatively, you could enter", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(bosColors = {RGBColor[1, 0, 0], RGBColor[0, 1, 0]};\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The second notation is more flexible because you have greater \
control over the colouring of the strategies.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "You might want to create a legend specifying what the colors mean. We know \
strategy 1 is red and strategy 2 is green - but usually strategies have more \
descriptive names. In ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  ", strategy 1 is typically called \"Boxing\" and strategy 2 is \"Theatre\". \
 We can give more descriptive labels, matching them to the colours, with the \
",
  StyleBox["StrategyLegend", "Input"],
  " command. The form of this command used below takes two arguments: the \
list of colors, followed by a list of labels for each color:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(StrategyLegend[
        bosColors, {"\<Watch boxing\>", "\<Go to theatre\>"}];\)\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 0, 0],
                Background->RGBColor[1, 0, 0]],
              BoxMargins->False], \(Watch\ boxing\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 1, 0],
                Background->RGBColor[0, 1, 0]],
              BoxMargins->False], \(Go\ to\ theatre\)}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[6]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Now that we have entered the payoff matrix, we need to pick a basic \
topology. The simplest interesting model on which to create a local \
interaction model is the ring. Rings are created using the ",
  StyleBox["Cycle", "Input"],
  " command from the ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  " package (which comes in the standard installation of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ")."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g = Cycle[30];\)\), "\n", 
    \(\(ShowGraph[g];\)\)}], "Input",
  CellLabel->"In[7]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0431795 0.913641 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.94461 .59498 m
.91525 .68581 L
s
.94461 .59498 m
.95455 .5 L
s
.91525 .68581 m
.86773 .76851 L
s
.86773 .76851 m
.80415 .83948 L
s
.80415 .83948 m
.72727 .89562 L
s
.72727 .89562 m
.64046 .93446 L
s
.64046 .93446 m
.54751 .95432 L
s
.54751 .95432 m
.45249 .95432 L
s
.45249 .95432 m
.35954 .93446 L
s
.35954 .93446 m
.27273 .89562 L
s
.27273 .89562 m
.19585 .83948 L
s
.19585 .83948 m
.13227 .76851 L
s
.13227 .76851 m
.08475 .68581 L
s
.08475 .68581 m
.05539 .59498 L
s
.05539 .59498 m
.04545 .5 L
s
.04545 .5 m
.05539 .40502 L
s
.05539 .40502 m
.08475 .31419 L
s
.08475 .31419 m
.13227 .23149 L
s
.13227 .23149 m
.19585 .16052 L
s
.19585 .16052 m
.27273 .10438 L
s
.27273 .10438 m
.35954 .06554 L
s
.35954 .06554 m
.45249 .04568 L
s
.45249 .04568 m
.54751 .04568 L
s
.54751 .04568 m
.64046 .06554 L
s
.64046 .06554 m
.72727 .10438 L
s
.72727 .10438 m
.80415 .16052 L
s
.80415 .16052 m
.86773 .23149 L
s
.86773 .23149 m
.91525 .31419 L
s
.91525 .31419 m
.94461 .40502 L
s
.94461 .40502 m
.95455 .5 L
s
.025 w
.94461 .59498 Mdot
.91525 .68581 Mdot
.86773 .76851 Mdot
.80415 .83948 Mdot
.72727 .89562 Mdot
.64046 .93446 Mdot
.54751 .95432 Mdot
.45249 .95432 Mdot
.35954 .93446 Mdot
.27273 .89562 Mdot
.19585 .83948 Mdot
.13227 .76851 Mdot
.08475 .68581 Mdot
.05539 .59498 Mdot
.04545 .5 Mdot
.05539 .40502 Mdot
.08475 .31419 Mdot
.13227 .23149 Mdot
.19585 .16052 Mdot
.27273 .10438 Mdot
.35954 .06554 Mdot
.45249 .04568 Mdot
.54751 .04568 Mdot
.64046 .06554 Mdot
.72727 .10438 Mdot
.80415 .16052 Mdot
.86773 .23149 Mdot
.91525 .31419 Mdot
.94461 .40502 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[7]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00P?ooool3o`0001Wooooo0ol00021ooooo`00Ooooool5o`0001Oooooo1Ol00020ooooo`00
O_ooool7o`0001Gooooo1ol0001oooooo`00O_oooolSo`0007oooooo001iooooo`co00005Oooool<
o`0007[ooooo001Vooooo`?o00002oooool5o`0000Kooooo1Ol0000Gooooo`Go00001_ooool5o`00
00_ooooo0ol0001Wooooo`00IOooool5o`0000Gooooo1Ol0000<ooooo`?o00006Oooool3o`0000co
oooo1Ol00005ooooo`Go0000I_ooool006Cooooo2ol00011ooooo`_o0000IOooool006Cooooo1ol0
0019ooooo`Oo0000IOooool006Cooooo1ol00019ooooo`Oo0000IOooool0067ooooo0ol000000ooo
oooo0000o`000003o`0004_ooooo1Ol000001?oooooo0000o`000?l0001Rooooo`00Goooool2o`00
00Gooooo0ol0001=ooooo`?o00001Oooool2o`00063ooooo001Mooooo`;o0000HOooool2o`0005ko
oooo001Jooooo`?o0000IOooool3o`0005_ooooo001Hooooo`;o0000Joooool2o`0005Wooooo001F
ooooo`;o0000Koooool2o`0005Oooooo001Dooooo`;o0000Loooool2o`0005Gooooo001=ooooo`?o
000000Coooooo`000?l0003o0000Moooool3o`000004ooooool0003o0000o`0004kooooo001<oooo
o`Go0000OOooool5o`0004gooooo001;ooooo`Oo0000Noooool7o`0004cooooo001;ooooo`Oo0000
Noooool7o`0004cooooo001;ooooo`Oo0000Noooool7o`0004cooooo0019ooooo`;o000000?ooooo
o`000?l000000ol0001mooooo`Go000000?oooooo`000?l00000B_ooool004Sooooo00?o0000oooo
ooooool00_ooool3o`0007oooooo0ol00004ooooo`03o`000?oooooooooo04Oooooo0017ooooo`03
o`000?oooooooooo08gooooo00?o0000ooooooooool0A_ooool004Kooooo00?o0000ooooooooool0
Soooool00ol0003oooooooooo`15ooooo`00A?ooool2o`0009?ooooo0_l00015ooooo`00@oooool0
0ol0003oooooooooo`2Eooooo`03o`000?oooooooooo04;ooooo0012ooooo`03o`000?oooooooooo
09Oooooo00?o0000ooooooooool0@Oooool0043ooooo0_l0002Kooooo`;o0000@Oooool003oooooo
00?o0000ooooooooool0WOooool00ol0003oooooooooo`0nooooo`00?_ooool00ol0003ooooooooo
o`2Oooooo`03o`000?oooooooooo03gooooo000mooooo`03o`000?oooooooooo0:7ooooo00?o0000
ooooooooool0??ooool003Oooooo0ol000000ooooooo0000o`00002Uooooo`;o000000Coooooo`00
0?l0003o0000>?ooool003Kooooo1Ol0002Yooooo`Go0000=oooool003Gooooo1ol0002Wooooo`Oo
0000=_ooool003Gooooo1ol0002Wooooo`Oo0000=_ooool003Gooooo1ol0002Wooooo`Oo0000=_oo
ool003Kooooo1Ol0002Yooooo`Go0000=oooool003Kooooo1?l0002[ooooo`Co0000=oooool003Go
oooo00?o0000ooooooooool0/Oooool00ol0003oooooooooo`0dooooo`00=?ooool00ol0003ooooo
ooooo`2cooooo`03o`000?oooooooooo03?ooooo000cooooo`03o`000?oooooooooo0;Gooooo00?o
0000ooooooooool0<_ooool003;ooooo00?o0000ooooooooool0]oooool00ol0003oooooooooo`0a
ooooo`00<Oooool00ol0003oooooooooo`2iooooo`03o`000?oooooooooo033ooooo000`ooooo`03
o`000?oooooooooo0;_ooooo00?o0000ooooooooool0;oooool002oooooo00?o0000ooooooooool0
_Oooool00ol0003oooooooooo`0^ooooo`00;_ooool00ol0003oooooooooo`2oooooo`03o`000?oo
oooooooo02gooooo000]ooooo`03o`000?oooooooooo0<7ooooo00?o0000ooooooooool0;?ooool0
02cooooo00?o0000ooooooooool0`oooool00ol0003oooooooooo`0[ooooo`00:oooool00ol0003o
ooooooooo`35ooooo`03o`000?oooooooooo02[ooooo000Zooooo`03o`000?oooooooooo0<Oooooo
00?o0000ooooooooool0:Oooool002Wooooo00?o0000ooooooooool0bOooool00ol0003ooooooooo
o`0Xooooo`009?ooool3o`000003ooooool0003ooooo0<cooooo1?l0000Vooooo`008oooool5o`00
0<kooooo1Ol0000Uooooo`008_ooool7o`000<cooooo1ol0000Tooooo`008_ooool7o`000<cooooo
1ol0000Tooooo`008_ooool7o`000<cooooo1ol0000Tooooo`008oooool5o`000<kooooo1Ol0000U
ooooo`009?ooool3o`000=3ooooo1?l0000Uooooo`008oooool00ol0003oooooooooo`3Eooooo`03
o`000?oooooooooo02;ooooo000Sooooo`03o`000?oooooooooo0=Gooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool0eoooool00ol0003oooooooooo`0Qooooo`008Oooool0
0ol0003oooooooooo`3Iooooo`03o`000?oooooooooo023ooooo000Qooooo`03o`000?oooooooooo
0=Wooooo00?o0000ooooooooool08?ooool0023ooooo00?o0000ooooooooool0foooool00ol0003o
ooooooooo`0Oooooo`008?ooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo
000Oooooo`03o`000?oooooooooo0=gooooo00?o0000ooooooooool07_ooool001oooooo00?o0000
ooooooooool0gOooool00ol0003oooooooooo`0Nooooo`007_ooool00ol0003oooooooooo`3Ooooo
o`03o`000?oooooooooo01gooooo000Nooooo`03o`000?oooooooooo0=oooooo00?o0000oooooooo
ool07Oooool001gooooo00?o0000ooooooooool0hOooool00ol0003oooooooooo`0Looooo`007Ooo
ool00ol0003oooooooooo`3Qooooo`03o`000?oooooooooo01cooooo000Looooo`03o`000?oooooo
oooo0>?ooooo00?o0000ooooooooool06oooool001cooooo00?o0000ooooooooool0hoooool00ol0
003oooooooooo`0Kooooo`006oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooooooo01[o
oooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o0000ooooooooool06Oooool001Oooooo1?l0
003Yooooo`Co00006?ooool001Kooooo1Ol0003Yooooo`Go00005oooool001Gooooo1ol0003Woooo
o`Oo00005_ooool001Gooooo1ol0003Wooooo`Oo00005_ooool001Gooooo1ol0003Wooooo`Oo0000
5_ooool001Kooooo1Ol0003Yooooo`Go00005oooool001Oooooo0ol0003[ooooo`?o00006?ooool0
01Oooooo00?o0000ooooooooool0kOooool00ol0003oooooooooo`0Fooooo`005_ooool00ol0003o
ooooooooo`3_ooooo`03o`000?oooooooooo01Gooooo000Fooooo`03o`000?oooooooooo0>oooooo
00?o0000ooooooooool05Oooool001Kooooo00?o0000ooooooooool0koooool00ol0003ooooooooo
o`0Eooooo`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo01Cooooo000Eoooo
o`03o`000?oooooooooo0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000oooooooo
ool0lOooool00ol0003oooooooooo`0Dooooo`005?ooool00ol0003oooooooooo`3cooooo`03o`00
0?oooooooooo01?ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool04ooo
ool001Cooooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`004oooool00ol0
003oooooooooo`3eooooo`03o`000?oooooooooo01;ooooo000Cooooo`03o`000?oooooooooo0?Go
oooo00?o0000ooooooooool04_ooool001?ooooo00?o0000ooooooooool0mOooool00ol0003ooooo
ooooo`0Booooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000B
ooooo`03o`000?oooooooooo0?Oooooo00?o0000ooooooooool04Oooool001;ooooo00?o0000oooo
ooooool0moooool00ol0003oooooooooo`0Aooooo`004Oooool00ol0003oooooooooo`3iooooo`03
o`000?oooooooooo013ooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool0
4?ooool0017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`003_ooool3
o`000?gooooo0ol0000?ooooo`003Oooool5o`000?_ooooo1Ol0000>ooooo`003?ooool7o`000?Wo
oooo1ol0000=ooooo`003?ooool7o`000?Wooooo1ol0000=ooooo`003?ooool7o`000?Wooooo1ol0
000=ooooo`003Oooool5o`000?_ooooo1Ol0000>ooooo`003_ooool3o`000?gooooo0ol0000?oooo
o`003oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0oOoo
ool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`3mooooo`03o`000?oooooo
oooo00kooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000ko
oooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooo
ooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o
0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03
o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0
ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oo
oooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool0
00kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003o
ooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo
00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000_o
oooo0ol0003oooooo`?ooooo0ol0000=ooooo`002_ooool5o`000?oooooo0Oooool5o`0000cooooo
0009ooooo`Oo0000o_ooool7o`0000_ooooo0009ooooo`Oo0000o_ooool7o`0000_ooooo0009oooo
o`Oo0000o_ooool7o`0000_ooooo000:ooooo`Go0000ooooool1ooooo`Go00003?ooool000_ooooo
0ol0003oooooo`?ooooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000kooooo
00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000
ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`00
0?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0oooo
ool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooo
oooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000ko
oooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooo
ooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o
0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=
ooooo`003oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0
oOooool00ol0003oooooooooo`0>ooooo`003_ooool3o`000?gooooo0ol0000?ooooo`003Oooool5
o`000?_ooooo1Ol0000>ooooo`003?ooool7o`000?Wooooo1ol0000=ooooo`003?ooool7o`000?Wo
oooo1ol0000=ooooo`003?ooool7o`000?Wooooo1ol0000=ooooo`003Oooool5o`000?_ooooo1Ol0
000>ooooo`003_ooool3o`000?gooooo0ol0000?ooooo`004?ooool00ol0003oooooooooo`3koooo
o`03o`000?oooooooooo00oooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000oooooooo
ool04?ooool0017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`004Ooo
ool00ol0003oooooooooo`3iooooo`03o`000?oooooooooo013ooooo000Booooo`03o`000?oooooo
oooo0?Oooooo00?o0000ooooooooool04Oooool001;ooooo00?o0000ooooooooool0moooool00ol0
003oooooooooo`0Aooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017o
oooo000Cooooo`03o`000?oooooooooo0?Gooooo00?o0000ooooooooool04_ooool001?ooooo00?o
0000ooooooooool0mOooool00ol0003oooooooooo`0Booooo`004oooool00ol0003oooooooooo`3e
ooooo`03o`000?oooooooooo01;ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000oooo
ooooool04oooool001Cooooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`00
5?ooool00ol0003oooooooooo`3cooooo`03o`000?oooooooooo01?ooooo000Eooooo`03o`000?oo
oooooooo0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000ooooooooool0lOooool0
0ol0003oooooooooo`0Dooooo`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo
01Cooooo000Fooooo`03o`000?oooooooooo0>oooooo00?o0000ooooooooool05Oooool001Kooooo
00?o0000ooooooooool0koooool00ol0003oooooooooo`0Eooooo`005_ooool00ol0003ooooooooo
o`3_ooooo`03o`000?oooooooooo01Gooooo000Gooooo`?o0000joooool3o`0001Sooooo000Foooo
o`Go0000jOooool5o`0001Oooooo000Eooooo`Oo0000ioooool7o`0001Kooooo000Eooooo`Oo0000
ioooool7o`0001Kooooo000Eooooo`Oo0000ioooool7o`0001Kooooo000Fooooo`Go0000jOooool5
o`0001Oooooo000Gooooo`?o0000joooool3o`0001Sooooo000Jooooo`03o`000?oooooooooo0>Oo
oooo00?o0000ooooooooool06Oooool001[ooooo00?o0000ooooooooool0ioooool00ol0003ooooo
ooooo`0Iooooo`006oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooooooo01[ooooo000L
ooooo`03o`000?oooooooooo0>?ooooo00?o0000ooooooooool06oooool001cooooo00?o0000oooo
ooooool0hoooool00ol0003oooooooooo`0Kooooo`007Oooool00ol0003oooooooooo`3Qooooo`03
o`000?oooooooooo01cooooo000Mooooo`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0
7?ooool001kooooo00?o0000ooooooooool0goooool00ol0003oooooooooo`0Mooooo`007_ooool0
0ol0003oooooooooo`3Oooooo`03o`000?oooooooooo01gooooo000Oooooo`03o`000?oooooooooo
0=gooooo00?o0000ooooooooool07_ooool001oooooo00?o0000ooooooooool0gOooool00ol0003o
ooooooooo`0Nooooo`008?ooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo
000Pooooo`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool07oooool0027ooooo00?o0000
ooooooooool0fOooool00ol0003oooooooooo`0Pooooo`008Oooool00ol0003oooooooooo`3Ioooo
o`03o`000?oooooooooo023ooooo000Rooooo`03o`000?oooooooooo0=Oooooo00?o0000oooooooo
ool08Oooool002?ooooo00?o0000ooooooooool0eOooool00ol0003oooooooooo`0Rooooo`008ooo
ool4o`000=3ooooo0ol000000ooooooo0000ooooo`0Sooooo`008oooool5o`000<kooooo1Ol0000U
ooooo`008_ooool7o`000<cooooo1ol0000Tooooo`008_ooool7o`000<cooooo1ol0000Tooooo`00
8_ooool7o`000<cooooo1ol0000Tooooo`008oooool5o`000<kooooo1Ol0000Uooooo`009?ooool4
o`000<oooooo0ol0000Vooooo`00:?ooool00ol0003oooooooooo`3;ooooo`03o`000?oooooooooo
02Oooooo000Yooooo`03o`000?oooooooooo0<Wooooo00?o0000ooooooooool0:?ooool002[ooooo
00?o0000ooooooooool0aoooool00ol0003oooooooooo`0Yooooo`00:oooool00ol0003ooooooooo
o`35ooooo`03o`000?oooooooooo02[ooooo000/ooooo`03o`000?oooooooooo0<?ooooo00?o0000
ooooooooool0:oooool002gooooo00?o0000ooooooooool0`Oooool00ol0003oooooooooo`0/oooo
o`00;_ooool00ol0003oooooooooo`2oooooo`03o`000?oooooooooo02gooooo000_ooooo`03o`00
0?oooooooooo0;gooooo00?o0000ooooooooool0;_ooool0033ooooo00?o0000ooooooooool0^ooo
ool00ol0003oooooooooo`0_ooooo`00<Oooool00ol0003oooooooooo`2iooooo`03o`000?oooooo
oooo033ooooo000booooo`03o`000?oooooooooo0;Oooooo00?o0000ooooooooool0<Oooool003?o
oooo00?o0000ooooooooool0]Oooool00ol0003oooooooooo`0booooo`00=?ooool00ol0003ooooo
ooooo`2cooooo`03o`000?oooooooooo03?ooooo000eooooo`03o`000?oooooo000000;o0000Zooo
ool3o`000003ooooool0003ooooo03Gooooo000fooooo`Go0000ZOooool5o`0003Oooooo000eoooo
o`Oo0000Yoooool7o`0003Kooooo000eooooo`Oo0000Yoooool7o`0003Kooooo000eooooo`Oo0000
Yoooool7o`0003Kooooo000fooooo`Ko0000Yoooool6o`0003Oooooo000gooooo`?o00000_ooool2
o`000:?ooooo0_l00002ooooo`?o0000>?ooool003kooooo00?o0000ooooooooool0Woooool00ol0
003oooooooooo`0mooooo`00?oooool00ol0003oooooooooo`2Mooooo`03o`000?oooooooooo03ko
oooo0010ooooo`03o`000?oooooooooo09_ooooo00?o0000ooooooooool0?oooool0047ooooo0_l0
002Iooooo`;o0000@_ooool004?ooooo00?o0000ooooooooool0UOooool00ol0003oooooooooo`12
ooooo`00A?ooool00ol0003oooooooooo`2Cooooo`03o`000?oooooooooo04?ooooo0015ooooo`;o
0000TOooool2o`0004Kooooo0017ooooo`03o`000?oooooooooo08gooooo00?o0000ooooooooool0
A_ooool004Sooooo00?o0000ooooooooool0Roooool00ol0003oooooooooo`17ooooo`00BOooool0
0ol0003oooooooooo`29ooooo`03o`000?oooooooooo04Sooooo001:ooooo`;o000000Coooooo`00
0?l0003o0000Ooooool3o`000003ooooool0003o000004_ooooo001<ooooo`Go0000OOooool5o`00
04gooooo001;ooooo`Oo0000Noooool7o`0004cooooo001;ooooo`Oo0000Noooool7o`0004cooooo
001;ooooo`Oo0000Noooool7o`0004cooooo001<ooooo`Go000000Coooooo`000?l0003o0000MOoo
ool3o`000003ooooool0003o000000?o0000COooool004gooooo0ol00005ooooo`;o0000LOooool2
o`0000Gooooo0ol0001>ooooo`00Eoooool2o`0006gooooo0_l0001Hooooo`00FOooool3o`0006Oo
oooo0ol0001Jooooo`00G?ooool2o`0006?ooooo0_l0001Mooooo`00G_ooool2o`0005oooooo0_l0
001Oooooo`00H?ooool2o`0000Cooooo0ol0001=ooooo`?o00001?ooool2o`00067ooooo001Roooo
o`So0000Boooool8o`0006?ooooo001Tooooo`Oo0000BOooool7o`0006Gooooo001Tooooo`Oo0000
BOooool7o`0006Gooooo001Tooooo`co0000?oooool<o`0006Gooooo001Uooooo`Go00001_ooool5
o`0000_ooooo0ol0000Iooooo`?o00002oooool5o`0000Kooooo1Ol0001Vooooo`00I_ooool3o`00
00cooooo1Ol00005ooooo`Go00005oooool5o`0000Gooooo1Ol0000<ooooo`?o0000Ioooool007[o
oooo2ol0000Eooooo`_o0000Noooool007kooooo8ol0001oooooo`00O_ooool7o`0001Gooooo1ol0
001oooooo`00Ooooool5o`0001Oooooo1Ol00020ooooo`00P?ooool3o`0001Wooooo0ol00021oooo
o`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQoooo
o`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQoooo
o`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0472664, \
0.00383279, 0.0038137}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Now that we have an underlying topology, a payoff matrix, and colors for \
representing each strategy, we have everything we need to construct a simple \
local interaction model. This is done by calling ",
  StyleBox["LocalInteractionModel", "Input"],
  " as follows:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(bos1 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] g, \[IndentingNewLine]PayoffMatrix \[Rule] 
            battleOfSexes, \[IndentingNewLine]StrategyColors \[Rule] 
            bosColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
After evaluating the above command, a separate window displaying \
the model will appear on your display. This shows the current state of the \
model.  It should look something like this:\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0419283 0.913641 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.94461 .59498 m
.91525 .68581 L
s
.94461 .59498 m
.95455 .5 L
s
.91525 .68581 m
.86774 .76851 L
s
.86774 .76851 m
.80415 .83948 L
s
.80415 .83948 m
.72727 .89562 L
s
.72727 .89562 m
.64046 .93446 L
s
.64046 .93446 m
.54751 .95432 L
s
.54751 .95432 m
.45249 .95432 L
s
.45249 .95432 m
.35954 .93446 L
s
.35954 .93446 m
.27273 .89562 L
s
.27273 .89562 m
.19585 .83948 L
s
.19585 .83948 m
.13226 .76851 L
s
.13226 .76851 m
.08475 .68581 L
s
.08475 .68581 m
.05539 .59498 L
s
.05539 .59498 m
.04545 .5 L
s
.04545 .5 m
.05539 .40502 L
s
.05539 .40502 m
.08475 .31419 L
s
.08475 .31419 m
.13226 .23149 L
s
.13226 .23149 m
.19585 .16052 L
s
.19585 .16052 m
.27273 .10438 L
s
.27273 .10438 m
.35954 .06554 L
s
.35954 .06554 m
.45249 .04568 L
s
.45249 .04568 m
.54751 .04568 L
s
.54751 .04568 m
.64046 .06554 L
s
.64046 .06554 m
.72727 .10438 L
s
.72727 .10438 m
.80415 .16052 L
s
.80415 .16052 m
.86774 .23149 L
s
.86774 .23149 m
.91525 .31419 L
s
.91525 .31419 m
.94461 .40502 L
s
.94461 .40502 m
.95455 .5 L
s
0 1 0 r
.025 w
.94461 .59498 Mdot
1 0 0 r
.91525 .68581 Mdot
.86774 .76851 Mdot
0 1 0 r
.80415 .83948 Mdot
1 0 0 r
.72727 .89562 Mdot
.64046 .93446 Mdot
0 1 0 r
.54751 .95432 Mdot
1 0 0 r
.45249 .95432 Mdot
0 1 0 r
.35954 .93446 Mdot
.27273 .89562 Mdot
.19585 .83948 Mdot
1 0 0 r
.13226 .76851 Mdot
.08475 .68581 Mdot
.05539 .59498 Mdot
0 1 0 r
.04545 .5 Mdot
1 0 0 r
.05539 .40502 Mdot
0 1 0 r
.08475 .31419 Mdot
.13226 .23149 Mdot
.19585 .16052 Mdot
1 0 0 r
.27273 .10438 Mdot
0 1 0 r
.35954 .06554 Mdot
.45249 .04568 Mdot
1 0 0 r
.54751 .04568 Mdot
.64046 .06554 Mdot
.72727 .10438 Mdot
.80415 .16052 Mdot
0 1 0 r
.86774 .23149 Mdot
1 0 0 r
.91525 .31419 Mdot
0 1 0 r
.94461 .40502 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  TextAlignment->Left,
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00P?ooool3o`3o01Wooooo0ooo0021ooooo`00Ooooool5o`3o01Oooooo1Ooo0020ooooo`00
O_ooool7o`3o01Gooooo1ooo001oooooo`00O_ooool7o`3o01Go00001ooo001oooooo`00NOooool5
o`0000Oo0?l05Oooool7ool000Go0000N_ooool006Kooooo0ol0o`0;ooooo`Go00001_ooool5o`3o
01Oooooo1Ooo0006ooooo`Go00002oooool3ool006Oooooo001Uooooo`Go0?l01Oooool5o`0000co
oooo0ol0o`0Iooooo`?oo`003?ooool5o`0000Gooooo1Ooo001Vooooo`00I?ooool7o`3o00Co0000
@Oooool4o`0000Ooo`00IOooool006Cooooo1ol0o`19ooooo`Ooo`00IOooool006Cooooo1ol0o`19
ooooo`Ooo`00IOooool0067ooooo0ol000000ooooooo0?l0o`3o0003o`3o04_ooooo1Ooo00001?oo
oooo0000o`000?l0001Rooooo`00Goooool2o`0000Gooooo0ol0o`1=ooooo`?oo`001Oooool2o`00
063ooooo001Mooooo`;o0000HOooool2o`0005kooooo001Jooooo`?o0000IOooool3o`0005_ooooo
001Hooooo`;o0000Joooool2o`0005Wooooo001Fooooo`;o0000Koooool2o`0005Oooooo001Doooo
o`;o0000Loooool2o`0005Gooooo001=ooooo`?oo`0000Coooooo`000?l0003o0000Moooool3o`00
0004oooooooo003oo`00ool004kooooo001<ooooo`Goo`00OOooool5ool004gooooo001;ooooo`Oo
o`00Noooool7ool004cooooo001;ooooo`Ooo`00Noooool7ool004cooooo001;ooooo`Ooo`00Nooo
ool7ool004cooooo0019ooooo`;o000000?oooooool00?oo00000ooo001mooooo`Goo`0000?ooooo
o`000?l00000B_ooool004Sooooo00?o0000ooooooooool00_ooool3ool007oooooo0ooo0004oooo
o`03o`000?oooooooooo04Oooooo0017ooooo`03o`000?oooooooooo08gooooo00?o0000oooooooo
ool0A_ooool004Kooooo00?o0000ooooooooool0Soooool00ol0003oooooooooo`15ooooo`00A?oo
ool2o`0009?ooooo0_l00015ooooo`00@oooool00ol0003oooooooooo`2Eooooo`03o`000?oooooo
oooo04;ooooo0012ooooo`03o`000?oooooooooo09Oooooo00?o0000ooooooooool0@Oooool0043o
oooo0_l0002Kooooo`;o0000@Oooool003oooooo00?o0000ooooooooool0WOooool00ol0003ooooo
ooooo`0nooooo`00?_ooool00ol0003oooooooooo`2Oooooo`03o`000?oooooooooo03gooooo000m
ooooo`03o`000?oooooooooo0:7ooooo00?o0000ooooooooool0??ooool003Oooooo0ol0o`000ooo
oooo0000o`00002Uooooo`;o000000Coooooool00?oo003oo`00>?ooool003Kooooo1Ol0o`2Yoooo
o`Goo`00=oooool003Gooooo1ol0o`2Wooooo`Ooo`00=_ooool003Gooooo1ol0o`2Wooooo`Ooo`00
=_ooool003Gooooo1ol0o`2Wooooo`Ooo`00=_ooool003Kooooo1Ol0o`2Yooooo`Goo`00=oooool0
03Kooooo00Co0000o`3o0?l0o`3o0?l0Zoooool3ool00003o`000?oooooooooo03Gooooo000eoooo
o`03o`000?oooooooooo0;7ooooo00?o0000ooooooooool0=?ooool003Cooooo00?o0000oooooooo
ool0/oooool00ol0003oooooooooo`0cooooo`00<oooool00ol0003oooooooooo`2eooooo`03o`00
0?oooooooooo03;ooooo000booooo`03o`000?oooooooooo0;Oooooo00?o0000ooooooooool0<Ooo
ool0037ooooo00?o0000ooooooooool0^Oooool00ol0003oooooooooo`0`ooooo`00<?ooool00ol0
003oooooooooo`2kooooo`03o`000?oooooooooo02oooooo000_ooooo`03o`000?oooooooooo0;go
oooo00?o0000ooooooooool0;_ooool002kooooo00?o0000ooooooooool0_oooool00ol0003ooooo
ooooo`0]ooooo`00;Oooool00ol0003oooooooooo`31ooooo`03o`000?oooooooooo02cooooo000/
ooooo`03o`000?oooooooooo0<?ooooo00?o0000ooooooooool0:oooool002_ooooo00?o0000oooo
ooooool0aOooool00ol0003oooooooooo`0Zooooo`00:_ooool00ol0003oooooooooo`37ooooo`03
o`000?oooooooooo02Wooooo000Yooooo`03o`000?oooooooooo0<Wooooo00?o0000ooooooooool0
:?ooool002Cooooo0ol0o`000ooooooo0000ooooo`3<ooooo`04o`000?l0o`3o0?l0o`3o02Kooooo
000Sooooo`Go0?l0c_ooool5o`3o02Gooooo000Rooooo`Oo0?l0c?ooool7o`3o02Cooooo000Roooo
o`Oo0?l0c?ooool7o`3o02Cooooo000Rooooo`Oo0?l0c?ooool7o`3o02Cooooo000Sooooo`Go0?l0
c_ooool5o`3o02Gooooo000Tooooo`?o0?l0d?ooool3o`3o0003o`000?oooooooooo02?ooooo000S
ooooo`03o`000?oooooooooo0=Gooooo00?o0000ooooooooool08_ooool002?ooooo00?o0000oooo
ooooool0eOooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`3Gooooo`03
o`000?oooooooooo027ooooo000Qooooo`03o`000?oooooooooo0=Wooooo00?o0000ooooooooool0
8?ooool0027ooooo00?o0000ooooooooool0fOooool00ol0003oooooooooo`0Pooooo`008?ooool0
0ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo000Pooooo`03o`000?oooooooooo
0=_ooooo00?o0000ooooooooool07oooool001oooooo00?o0000ooooooooool0gOooool00ol0003o
ooooooooo`0Nooooo`007oooool00ol0003oooooooooo`3Mooooo`03o`000?oooooooooo01kooooo
000Nooooo`03o`000?oooooooooo0=oooooo00?o0000ooooooooool07Oooool001kooooo00?o0000
ooooooooool0goooool00ol0003oooooooooo`0Mooooo`007Oooool00ol0003oooooooooo`3Qoooo
o`03o`000?oooooooooo01cooooo000Mooooo`03o`000?oooooooooo0>7ooooo00?o0000oooooooo
ool07?ooool001cooooo00?o0000ooooooooool0hoooool00ol0003oooooooooo`0Kooooo`007?oo
ool00ol0003oooooooooo`3Sooooo`03o`000?oooooooooo01_ooooo000Kooooo`03o`000?oooooo
oooo0>Gooooo00?o0000ooooooooool06_ooool001[ooooo00?o0000ooooooooool0ioooool00ol0
003oooooooooo`0Iooooo`005oooool3o`3o0003o`000?oooooooooo0>Oooooo00Co0000ool00?oo
003oo`006?ooool001Kooooo1Ol0o`3Yooooo`Goo`005oooool001Gooooo1ol0o`3Wooooo`Ooo`00
5_ooool001Gooooo1ol0o`3Wooooo`Ooo`005_ooool001Gooooo1ol0o`3Wooooo`Ooo`005_ooool0
01Kooooo1Ol0o`3Yooooo`Goo`005oooool001Oooooo0ol0o`3[ooooo`?oo`006?ooool001Oooooo
00?o0000ooooooooool0kOooool00ol0003oooooooooo`0Fooooo`005_ooool00ol0003ooooooooo
o`3_ooooo`03o`000?oooooooooo01Gooooo000Fooooo`03o`000?oooooooooo0>oooooo00?o0000
ooooooooool05Oooool001Kooooo00?o0000ooooooooool0koooool00ol0003oooooooooo`0Eoooo
o`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo01Cooooo000Eooooo`03o`00
0?oooooooooo0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000ooooooooool0lOoo
ool00ol0003oooooooooo`0Dooooo`005?ooool00ol0003oooooooooo`3cooooo`03o`000?oooooo
oooo01?ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool04oooool001Co
oooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`004oooool00ol0003ooooo
ooooo`3eooooo`03o`000?oooooooooo01;ooooo000Cooooo`03o`000?oooooooooo0?Gooooo00?o
0000ooooooooool04_ooool001?ooooo00?o0000ooooooooool0mOooool00ol0003oooooooooo`0B
ooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000Booooo`03
o`000?oooooooooo0?Oooooo00?o0000ooooooooool04Oooool001;ooooo00?o0000ooooooooool0
moooool00ol0003oooooooooo`0Aooooo`004Oooool00ol0003oooooooooo`3iooooo`03o`000?oo
oooooooo013ooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool04?ooool0
017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`003_ooool3ool00?go
oooo0ol0o`0?ooooo`003Oooool5ool00?_ooooo1Ol0o`0>ooooo`003?ooool7ool00?Wooooo1ol0
o`0=ooooo`003?ooool7ool00?Wooooo1ol0o`0=ooooo`003?ooool7ool00?Wooooo1ol0o`0=oooo
o`003Oooool5ool00?_ooooo1Ol0o`0>ooooo`003_ooool3ool00?gooooo0ol0o`0?ooooo`003ooo
ool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooo
oooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0oOooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00ko
oooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o
0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3o
ooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooo
ooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oo
oooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool0
0ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo
00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo
00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000
ooooooooool03Oooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000_ooooo0ol0
o`3oooooo`?ooooo0ol0o`0=ooooo`002_ooool5o`3o0?oooooo0Oooool5o`3o00cooooo0009oooo
o`Oo0?l0o_ooool7o`3o00_ooooo0009ooooo`Oo0?l0o_ooool7o`3o00_ooooo0009ooooo`Oo0?l0
o_ooool7o`3o00_ooooo000:ooooo`Go0?l0ooooool1ooooo`Go0?l03?ooool000_ooooo0ol0o`3o
ooooo`?ooooo0ol0o`0=ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000kooooo00?o0000
ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3ooooo
o`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooooooo
ool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_oo
ool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooo
oooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0
003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00go
oooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o
0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3o
ooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooo
ooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`00
3oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oo
oooooooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0oOooool0
0ol0003oooooooooo`0>ooooo`003_ooool3ool00?gooooo0ol0o`0?ooooo`003Oooool5ool00?_o
oooo1Ol0o`0>ooooo`003?ooool7ool00?Wooooo1ol0o`0=ooooo`003?ooool7ool00?Wooooo1ol0
o`0=ooooo`003?ooool7ool00?Wooooo1ol0o`0=ooooo`003Oooool5ool00?_ooooo1Ol0o`0>oooo
o`003_ooool3ool00?gooooo0ol0o`0?ooooo`004?ooool00ol0003oooooooooo`3kooooo`03o`00
0?oooooooooo00oooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool04?oo
ool0017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`004Oooool00ol0
003oooooooooo`3iooooo`03o`000?oooooooooo013ooooo000Booooo`03o`000?oooooooooo0?Oo
oooo00?o0000ooooooooool04Oooool001;ooooo00?o0000ooooooooool0moooool00ol0003ooooo
ooooo`0Aooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000C
ooooo`03o`000?oooooooooo0?Gooooo00?o0000ooooooooool04_ooool001?ooooo00?o0000oooo
ooooool0mOooool00ol0003oooooooooo`0Booooo`004oooool00ol0003oooooooooo`3eooooo`03
o`000?oooooooooo01;ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool0
4oooool001Cooooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`005?ooool0
0ol0003oooooooooo`3cooooo`03o`000?oooooooooo01?ooooo000Eooooo`03o`000?oooooooooo
0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000ooooooooool0lOooool00ol0003o
ooooooooo`0Dooooo`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo01Cooooo
000Fooooo`03o`000?oooooooooo0>oooooo00?o0000ooooooooool05Oooool001Kooooo00?o0000
ooooooooool0koooool00ol0003oooooooooo`0Eooooo`005_ooool00ol0003oooooooooo`3_oooo
o`03o`000?oooooooooo01Gooooo000Gooooo`?oo`00joooool3ool001Sooooo000Fooooo`Goo`00
jOooool5ool001Oooooo000Eooooo`Ooo`00ioooool7ool001Kooooo000Eooooo`Ooo`00ioooool7
ool001Kooooo000Eooooo`Ooo`00ioooool7ool001Kooooo000Fooooo`Goo`00jOooool5ool001Oo
oooo000Gooooo`?oo`00joooool3ool001Sooooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o
0000ooooooooool06Oooool001[ooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0I
ooooo`006oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooooooo01[ooooo000Looooo`03
o`000?oooooooooo0>?ooooo00?o0000ooooooooool06oooool001cooooo00?o0000ooooooooool0
hoooool00ol0003oooooooooo`0Kooooo`007Oooool00ol0003oooooooooo`3Qooooo`03o`000?oo
oooooooo01cooooo000Mooooo`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool07?ooool0
01kooooo00?o0000ooooooooool0goooool00ol0003oooooooooo`0Mooooo`007_ooool00ol0003o
ooooooooo`3Oooooo`03o`000?oooooooooo01gooooo000Oooooo`03o`000?oooooooooo0=gooooo
00?o0000ooooooooool07_ooool001oooooo00?o0000ooooooooool0gOooool00ol0003ooooooooo
o`0Nooooo`008?ooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo000Poooo
o`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool07oooool0027ooooo00?o0000oooooooo
ool0fOooool00ol0003oooooooooo`0Pooooo`008Oooool00ol0003oooooooooo`3Iooooo`03o`00
0?oooooooooo023ooooo000Rooooo`03o`000?oooooooooo0=Oooooo00?o0000ooooooooool08Ooo
ool002?ooooo00?o0000ooooooooool0eOooool00ol0003oooooooooo`0Rooooo`008oooool01?l0
003oo`00ool00?oo003@ooooo`?oo`0000?oooooo`000?ooool08oooool002?ooooo1Ooo003>oooo
o`Goo`009Oooool002;ooooo1ooo003<ooooo`Ooo`009?ooool002;ooooo1ooo003<ooooo`Ooo`00
9?ooool002;ooooo1ooo003<ooooo`Ooo`009?ooool002?ooooo1Ooo003>ooooo`Goo`009Oooool0
02Cooooo0ooo00000ol0003oooooooooo`3=ooooo`?oo`009_ooool002Sooooo00?o0000oooooooo
ool0boooool00ol0003oooooooooo`0Wooooo`00:Oooool00ol0003oooooooooo`39ooooo`03o`00
0?oooooooooo02Sooooo000Zooooo`03o`000?oooooooooo0<Oooooo00?o0000ooooooooool0:Ooo
ool002_ooooo00?o0000ooooooooool0aOooool00ol0003oooooooooo`0Zooooo`00;?ooool00ol0
003oooooooooo`33ooooo`03o`000?oooooooooo02_ooooo000]ooooo`03o`000?oooooooooo0<7o
oooo00?o0000ooooooooool0;?ooool002kooooo00?o0000ooooooooool0_oooool00ol0003ooooo
ooooo`0]ooooo`00;oooool00ol0003oooooooooo`2mooooo`03o`000?oooooooooo02kooooo000`
ooooo`03o`000?oooooooooo0;_ooooo00?o0000ooooooooool0;oooool0037ooooo00?o0000oooo
ooooool0^Oooool00ol0003oooooooooo`0`ooooo`00<_ooool00ol0003oooooooooo`2gooooo`03
o`000?oooooooooo037ooooo000cooooo`03o`000?oooooooooo0;Gooooo00?o0000ooooooooool0
<_ooool003Cooooo00?o0000ooooooooool0/oooool00ol0003oooooooooo`0cooooo`00=Oooool0
0ol0003oooooo`3o0002o`3o0:_ooooo0ol0o`000ooooooo0000ooooo`0eooooo`00=_ooool5o`3o
0:Wooooo1Ol0o`0gooooo`00=Oooool7o`3o0:Oooooo1ol0o`0fooooo`00=Oooool7o`3o0:Oooooo
1ol0o`0fooooo`00=Oooool7o`3o0:Oooooo1ol0o`0fooooo`00=_ooool5o`3o0003o`000?oooooo
oooo0:Gooooo00?o0000o`3o0?l0o`000ol0o`0gooooo`00=oooool3o`3o00;ooooo0_l0002Soooo
o`;o00000_ooool3o`3o03Sooooo000nooooo`03o`000?oooooooooo09oooooo00?o0000oooooooo
ool0?Oooool003oooooo00?o0000ooooooooool0WOooool00ol0003oooooooooo`0nooooo`00@?oo
ool00ol0003oooooooooo`2Kooooo`03o`000?oooooooooo03oooooo0011ooooo`;o0000VOooool2
o`0004;ooooo0013ooooo`03o`000?oooooooooo09Gooooo00?o0000ooooooooool0@_ooool004Co
oooo00?o0000ooooooooool0Toooool00ol0003oooooooooo`13ooooo`00AOooool2o`00097ooooo
0_l00016ooooo`00Aoooool00ol0003oooooooooo`2=ooooo`03o`000?oooooooooo04Kooooo0018
ooooo`03o`000?oooooooooo08_ooooo00?o0000ooooooooool0Aoooool004Wooooo00?o0000oooo
ooooool0ROooool00ol0003oooooooooo`18ooooo`00B_ooool2o`000004ooooool0o`3o0?l0o`3o
07oooooo0ooo00000ooooooo0000o`00001;ooooo`00C?ooool5o`3o07gooooo1Ooo001=ooooo`00
Boooool7o`3o07_ooooo1ooo001<ooooo`00Boooool7o`3o07_ooooo1ooo001<ooooo`00Boooool7
o`3o07_ooooo1ooo001<ooooo`00C?ooool5o`3o0004ooooool0003o0000o`0007Gooooo0ol00000
0oooooooo`00ool00003ool004gooooo001=ooooo`?o0?l01Oooool2o`00077ooooo0_l00005oooo
o`?oo`00C_ooool005Oooooo0_l0001]ooooo`;o0000F?ooool005Wooooo0ol0001Wooooo`?o0000
F_ooool005cooooo0_l0001Sooooo`;o0000GOooool005kooooo0_l0001Oooooo`;o0000Goooool0
063ooooo0_l00004ooooo`?o0?l0COooool3ool000Cooooo0_l0001Qooooo`00H_ooool3o`0000Go
0?l0Boooool5ool000?o0000Hoooool006Cooooo1ol0o`19ooooo`Ooo`00IOooool006Cooooo1ol0
o`19ooooo`Ooo`00IOooool006Cooooo1ol0o`05o`0003oooooo1Ol00007ool006Gooooo001Uoooo
o`Go0?l01_ooool5o`0000_ooooo0ooo000Iooooo`?o0?l02oooool5o`0000Kooooo1Ooo001Voooo
o`00I_ooool3o`3o00cooooo1Ol00005ooooo`Goo`005oooool5o`3o00Gooooo1Ol0000<ooooo`?o
o`00Ioooool007[ooooo1?l00007ool001Gooooo1ol0o`04o`0007_ooooo001nooooo`Ooo`005Ol0
0007o`3o07oooooo001nooooo`Ooo`005Oooool7o`3o07oooooo001oooooo`Goo`005oooool5o`3o
083ooooo0020ooooo`?oo`006Oooool3o`3o087ooooo003oooooob7ooooo003oooooob7ooooo003o
ooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003o
ooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{73.4375, 302.812}, {390.438, 161.062}} -> {-0.402189, \
0.722662, 0.00383655, 0.00381744}},
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The nodes in the window can be rearranged, if you prefer a \
different embedding of the graph. Click on a node to drag it around \
\>", \
"Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "To run the model forward a number of generations, use the ",
  StyleBox["Evolve", "Input"],
  " command. This steps the model one iteration forward."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(Evolve[bos1]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The window displaying the model should now look something like the \
following:\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0419283 0.913641 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.94461 .59498 m
.91525 .68581 L
s
.94461 .59498 m
.95455 .5 L
s
.91525 .68581 m
.86774 .76851 L
s
.86774 .76851 m
.80415 .83948 L
s
.80415 .83948 m
.72727 .89562 L
s
.72727 .89562 m
.64046 .93446 L
s
.64046 .93446 m
.54751 .95432 L
s
.54751 .95432 m
.45249 .95432 L
s
.45249 .95432 m
.35954 .93446 L
s
.35954 .93446 m
.27273 .89562 L
s
.27273 .89562 m
.19585 .83948 L
s
.19585 .83948 m
.13226 .76851 L
s
.13226 .76851 m
.08475 .68581 L
s
.08475 .68581 m
.05539 .59498 L
s
.05539 .59498 m
.04545 .5 L
s
.04545 .5 m
.05539 .40502 L
s
.05539 .40502 m
.08475 .31419 L
s
.08475 .31419 m
.13226 .23149 L
s
.13226 .23149 m
.19585 .16052 L
s
.19585 .16052 m
.27273 .10438 L
s
.27273 .10438 m
.35954 .06554 L
s
.35954 .06554 m
.45249 .04568 L
s
.45249 .04568 m
.54751 .04568 L
s
.54751 .04568 m
.64046 .06554 L
s
.64046 .06554 m
.72727 .10438 L
s
.72727 .10438 m
.80415 .16052 L
s
.80415 .16052 m
.86774 .23149 L
s
.86774 .23149 m
.91525 .31419 L
s
.91525 .31419 m
.94461 .40502 L
s
.94461 .40502 m
.95455 .5 L
s
0 1 0 r
.025 w
.94461 .59498 Mdot
1 0 0 r
.91525 .68581 Mdot
.86774 .76851 Mdot
.80415 .83948 Mdot
.72727 .89562 Mdot
.64046 .93446 Mdot
.54751 .95432 Mdot
0 1 0 r
.45249 .95432 Mdot
.35954 .93446 Mdot
.27273 .89562 Mdot
.19585 .83948 Mdot
1 0 0 r
.13226 .76851 Mdot
.08475 .68581 Mdot
.05539 .59498 Mdot
.04545 .5 Mdot
0 1 0 r
.05539 .40502 Mdot
.08475 .31419 Mdot
.13226 .23149 Mdot
.19585 .16052 Mdot
.27273 .10438 Mdot
.35954 .06554 Mdot
.45249 .04568 Mdot
1 0 0 r
.54751 .04568 Mdot
.64046 .06554 Mdot
.72727 .10438 Mdot
.80415 .16052 Mdot
.86774 .23149 Mdot
0 1 0 r
.91525 .31419 Mdot
.94461 .40502 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00P?ooool3o`3o01Wooooo0ooo0021ooooo`00Ooooool5o`3o01Oooooo1Ooo0020ooooo`00
O_ooool7o`3o01Gooooo1ooo001oooooo`00O_ooool7o`3o01Go00001ooo001oooooo`00NOooool5
o`0000Oo0?l05Oooool7ool000Go0000N_ooool006Kooooo0ol0o`0;ooooo`Go00001_ooool5o`3o
01Oooooo1Ooo0006ooooo`Go00002oooool3ool006Oooooo001Uooooo`Go0?l01Oooool5o`0000co
oooo0ol0o`0Iooooo`?oo`003?ooool5o`0000Gooooo1Ooo001Vooooo`00I?ooool7o`3o00Co0000
@Oooool4o`0000Ooo`00IOooool006Cooooo1ol0o`19ooooo`Ooo`00IOooool006Cooooo1ol0o`19
ooooo`Ooo`00IOooool0067ooooo0ol000000ooooooo0?l0o`3o0003o`3o04_ooooo1Ooo00001?oo
oooo0000o`000?l0001Rooooo`00Goooool2o`0000Gooooo0ol0o`1=ooooo`?oo`001Oooool2o`00
063ooooo001Mooooo`;o0000HOooool2o`0005kooooo001Jooooo`?o0000IOooool3o`0005_ooooo
001Hooooo`;o0000Joooool2o`0005Wooooo001Fooooo`;o0000Koooool2o`0005Oooooo001Doooo
o`;o0000Loooool2o`0005Gooooo001=ooooo`?o0?l000Coooooo`000?l0003o0000Moooool3o`00
0004oooooooo003oo`00ool004kooooo001<ooooo`Go0?l0OOooool5ool004gooooo001;ooooo`Oo
0?l0Noooool7ool004cooooo001;ooooo`Oo0?l0Noooool7ool004cooooo001;ooooo`Oo0?l0Nooo
ool7ool004cooooo0019ooooo`;o000000?oooooo`3o0?l0o`000ol0o`1mooooo`Goo`0000?ooooo
o`000?l00000B_ooool004Sooooo00?o0000ooooooooool00_ooool3o`3o07oooooo0ooo0004oooo
o`03o`000?oooooooooo04Oooooo0017ooooo`03o`000?oooooooooo08gooooo00?o0000oooooooo
ool0A_ooool004Kooooo00?o0000ooooooooool0Soooool00ol0003oooooooooo`15ooooo`00A?oo
ool2o`0009?ooooo0_l00015ooooo`00@oooool00ol0003oooooooooo`2Eooooo`03o`000?oooooo
oooo04;ooooo0012ooooo`03o`000?oooooooooo09Oooooo00?o0000ooooooooool0@Oooool0043o
oooo0_l0002Kooooo`;o0000@Oooool003oooooo00?o0000ooooooooool0WOooool00ol0003ooooo
ooooo`0nooooo`00?_ooool00ol0003oooooooooo`2Oooooo`03o`000?oooooooooo03gooooo000m
ooooo`03o`000?oooooooooo0:7ooooo00?o0000ooooooooool0??ooool003Oooooo0ol0o`000ooo
oooo0000o`00002Uooooo`;o000000Coooooool00?oo003oo`00>?ooool003Kooooo1Ol0o`2Yoooo
o`Goo`00=oooool003Gooooo1ol0o`2Wooooo`Ooo`00=_ooool003Gooooo1ol0o`2Wooooo`Ooo`00
=_ooool003Gooooo1ol0o`2Wooooo`Ooo`00=_ooool003Kooooo1Ol0o`2Yooooo`Goo`00=oooool0
03Kooooo00Co0000o`3o0?l0o`3o0?l0Zoooool3ool00003o`000?oooooooooo03Gooooo000eoooo
o`03o`000?oooooooooo0;7ooooo00?o0000ooooooooool0=?ooool003Cooooo00?o0000oooooooo
ool0/oooool00ol0003oooooooooo`0cooooo`00<oooool00ol0003oooooooooo`2eooooo`03o`00
0?oooooooooo03;ooooo000booooo`03o`000?oooooooooo0;Oooooo00?o0000ooooooooool0<Ooo
ool0037ooooo00?o0000ooooooooool0^Oooool00ol0003oooooooooo`0`ooooo`00<?ooool00ol0
003oooooooooo`2kooooo`03o`000?oooooooooo02oooooo000_ooooo`03o`000?oooooooooo0;go
oooo00?o0000ooooooooool0;_ooool002kooooo00?o0000ooooooooool0_oooool00ol0003ooooo
ooooo`0]ooooo`00;Oooool00ol0003oooooooooo`31ooooo`03o`000?oooooooooo02cooooo000/
ooooo`03o`000?oooooooooo0<?ooooo00?o0000ooooooooool0:oooool002_ooooo00?o0000oooo
ooooool0aOooool00ol0003oooooooooo`0Zooooo`00:_ooool00ol0003oooooooooo`37ooooo`03
o`000?oooooooooo02Wooooo000Yooooo`03o`000?oooooooooo0<Wooooo00?o0000ooooooooool0
:?ooool002Cooooo0ol0o`000ooooooo0000ooooo`3<ooooo`04o`000?oo003oo`00ool002Kooooo
000Sooooo`Go0?l0c_ooool5ool002Gooooo000Rooooo`Oo0?l0c?ooool7ool002Cooooo000Roooo
o`Oo0?l0c?ooool7ool002Cooooo000Rooooo`Oo0?l0c?ooool7ool002Cooooo000Sooooo`Go0?l0
c_ooool5ool002Gooooo000Tooooo`?o0?l0d?ooool3ool00003o`000?oooooooooo02?ooooo000S
ooooo`03o`000?oooooooooo0=Gooooo00?o0000ooooooooool08_ooool002?ooooo00?o0000oooo
ooooool0eOooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`3Gooooo`03
o`000?oooooooooo027ooooo000Qooooo`03o`000?oooooooooo0=Wooooo00?o0000ooooooooool0
8?ooool0027ooooo00?o0000ooooooooool0fOooool00ol0003oooooooooo`0Pooooo`008?ooool0
0ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo000Pooooo`03o`000?oooooooooo
0=_ooooo00?o0000ooooooooool07oooool001oooooo00?o0000ooooooooool0gOooool00ol0003o
ooooooooo`0Nooooo`007oooool00ol0003oooooooooo`3Mooooo`03o`000?oooooooooo01kooooo
000Nooooo`03o`000?oooooooooo0=oooooo00?o0000ooooooooool07Oooool001kooooo00?o0000
ooooooooool0goooool00ol0003oooooooooo`0Mooooo`007Oooool00ol0003oooooooooo`3Qoooo
o`03o`000?oooooooooo01cooooo000Mooooo`03o`000?oooooooooo0>7ooooo00?o0000oooooooo
ool07?ooool001cooooo00?o0000ooooooooool0hoooool00ol0003oooooooooo`0Kooooo`007?oo
ool00ol0003oooooooooo`3Sooooo`03o`000?oooooooooo01_ooooo000Kooooo`03o`000?oooooo
oooo0>Gooooo00?o0000ooooooooool06_ooool001[ooooo00?o0000ooooooooool0ioooool00ol0
003oooooooooo`0Iooooo`005oooool3o`3o0003o`000?oooooooooo0>Oooooo00Co0000o`3o0?l0
o`3o0?l06?ooool001Kooooo1Ol0o`3Yooooo`Go0?l05oooool001Gooooo1ol0o`3Wooooo`Oo0?l0
5_ooool001Gooooo1ol0o`3Wooooo`Oo0?l05_ooool001Gooooo1ol0o`3Wooooo`Oo0?l05_ooool0
01Kooooo1Ol0o`3Yooooo`Go0?l05oooool001Oooooo0ol0o`3[ooooo`?o0?l06?ooool001Oooooo
00?o0000ooooooooool0kOooool00ol0003oooooooooo`0Fooooo`005_ooool00ol0003ooooooooo
o`3_ooooo`03o`000?oooooooooo01Gooooo000Fooooo`03o`000?oooooooooo0>oooooo00?o0000
ooooooooool05Oooool001Kooooo00?o0000ooooooooool0koooool00ol0003oooooooooo`0Eoooo
o`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo01Cooooo000Eooooo`03o`00
0?oooooooooo0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000ooooooooool0lOoo
ool00ol0003oooooooooo`0Dooooo`005?ooool00ol0003oooooooooo`3cooooo`03o`000?oooooo
oooo01?ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool04oooool001Co
oooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`004oooool00ol0003ooooo
ooooo`3eooooo`03o`000?oooooooooo01;ooooo000Cooooo`03o`000?oooooooooo0?Gooooo00?o
0000ooooooooool04_ooool001?ooooo00?o0000ooooooooool0mOooool00ol0003oooooooooo`0B
ooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000Booooo`03
o`000?oooooooooo0?Oooooo00?o0000ooooooooool04Oooool001;ooooo00?o0000ooooooooool0
moooool00ol0003oooooooooo`0Aooooo`004Oooool00ol0003oooooooooo`3iooooo`03o`000?oo
oooooooo013ooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool04?ooool0
017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`003_ooool3o`3o0?go
oooo0ol0o`0?ooooo`003Oooool5o`3o0?_ooooo1Ol0o`0>ooooo`003?ooool7o`3o0?Wooooo1ol0
o`0=ooooo`003?ooool7o`3o0?Wooooo1ol0o`0=ooooo`003?ooool7o`3o0?Wooooo1ol0o`0=oooo
o`003Oooool5o`3o0?_ooooo1Ol0o`0>ooooo`003_ooool3o`3o0?gooooo0ol0o`0?ooooo`003ooo
ool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooo
oooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0oOooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00ko
oooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o
0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3o
ooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooo
ooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oo
oooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool0
0ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo
00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo
00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000
ooooooooool03Oooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000_ooooo0ooo
003oooooo`?ooooo0ol0o`0=ooooo`002_ooool5ool00?oooooo0Oooool5o`3o00cooooo0009oooo
o`Ooo`00o_ooool7o`3o00_ooooo0009ooooo`Ooo`00o_ooool7o`3o00_ooooo0009ooooo`Ooo`00
o_ooool7o`3o00_ooooo000:ooooo`Goo`00ooooool1ooooo`Go0?l03?ooool000_ooooo0ooo003o
ooooo`?ooooo0ol0o`0=ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000kooooo00?o0000
ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3ooooo
o`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooooooo
ool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_oo
ool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooo
oooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0
003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00go
oooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o
0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3o
ooooo`03o`000?oooooooooo00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000oooo
ooooool03Oooool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`00
3oooool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oo
oooooooo0?gooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0oOooool0
0ol0003oooooooooo`0>ooooo`003_ooool3ool00?gooooo0ol0o`0?ooooo`003Oooool5ool00?_o
oooo1Ol0o`0>ooooo`003?ooool7ool00?Wooooo1ol0o`0=ooooo`003?ooool7ool00?Wooooo1ol0
o`0=ooooo`003?ooool7ool00?Wooooo1ol0o`0=ooooo`003Oooool5ool00?_ooooo1Ol0o`0>oooo
o`003_ooool3ool00?gooooo0ol0o`0?ooooo`004?ooool00ol0003oooooooooo`3kooooo`03o`00
0?oooooooooo00oooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool04?oo
ool0017ooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0@ooooo`004Oooool00ol0
003oooooooooo`3iooooo`03o`000?oooooooooo013ooooo000Booooo`03o`000?oooooooooo0?Oo
oooo00?o0000ooooooooool04Oooool001;ooooo00?o0000ooooooooool0moooool00ol0003ooooo
ooooo`0Aooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000C
ooooo`03o`000?oooooooooo0?Gooooo00?o0000ooooooooool04_ooool001?ooooo00?o0000oooo
ooooool0mOooool00ol0003oooooooooo`0Booooo`004oooool00ol0003oooooooooo`3eooooo`03
o`000?oooooooooo01;ooooo000Dooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool0
4oooool001Cooooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Cooooo`005?ooool0
0ol0003oooooooooo`3cooooo`03o`000?oooooooooo01?ooooo000Eooooo`03o`000?oooooooooo
0?7ooooo00?o0000ooooooooool05?ooool001Gooooo00?o0000ooooooooool0lOooool00ol0003o
ooooooooo`0Dooooo`005Oooool00ol0003oooooooooo`3aooooo`03o`000?oooooooooo01Cooooo
000Fooooo`03o`000?oooooooooo0>oooooo00?o0000ooooooooool05Oooool001Kooooo00?o0000
ooooooooool0koooool00ol0003oooooooooo`0Eooooo`005_ooool00ol0003oooooooooo`3_oooo
o`03o`000?oooooooooo01Gooooo000Gooooo`?oo`00joooool3ool001Sooooo000Fooooo`Goo`00
jOooool5ool001Oooooo000Eooooo`Ooo`00ioooool7ool001Kooooo000Eooooo`Ooo`00ioooool7
ool001Kooooo000Eooooo`Ooo`00ioooool7ool001Kooooo000Fooooo`Goo`00jOooool5ool001Oo
oooo000Gooooo`?oo`00joooool3ool001Sooooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o
0000ooooooooool06Oooool001[ooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0I
ooooo`006oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooooooo01[ooooo000Looooo`03
o`000?oooooooooo0>?ooooo00?o0000ooooooooool06oooool001cooooo00?o0000ooooooooool0
hoooool00ol0003oooooooooo`0Kooooo`007Oooool00ol0003oooooooooo`3Qooooo`03o`000?oo
oooooooo01cooooo000Mooooo`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool07?ooool0
01kooooo00?o0000ooooooooool0goooool00ol0003oooooooooo`0Mooooo`007_ooool00ol0003o
ooooooooo`3Oooooo`03o`000?oooooooooo01gooooo000Oooooo`03o`000?oooooooooo0=gooooo
00?o0000ooooooooool07_ooool001oooooo00?o0000ooooooooool0gOooool00ol0003ooooooooo
o`0Nooooo`008?ooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo01oooooo000Poooo
o`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool07oooool0027ooooo00?o0000oooooooo
ool0fOooool00ol0003oooooooooo`0Pooooo`008Oooool00ol0003oooooooooo`3Iooooo`03o`00
0?oooooooooo023ooooo000Rooooo`03o`000?oooooooooo0=Oooooo00?o0000ooooooooool08Ooo
ool002?ooooo00?o0000ooooooooool0eOooool00ol0003oooooooooo`0Rooooo`008oooool01?l0
003oo`00ool00?oo003@ooooo`?oo`0000?oooooo`000?ooool08oooool002?ooooo1Ooo003>oooo
o`Goo`009Oooool002;ooooo1ooo003<ooooo`Ooo`009?ooool002;ooooo1ooo003<ooooo`Ooo`00
9?ooool002;ooooo1ooo003<ooooo`Ooo`009?ooool002?ooooo1Ooo003>ooooo`Goo`009Oooool0
02Cooooo0ooo00000ol0003oooooooooo`3=ooooo`?oo`009_ooool002Sooooo00?o0000oooooooo
ool0boooool00ol0003oooooooooo`0Wooooo`00:Oooool00ol0003oooooooooo`39ooooo`03o`00
0?oooooooooo02Sooooo000Zooooo`03o`000?oooooooooo0<Oooooo00?o0000ooooooooool0:Ooo
ool002_ooooo00?o0000ooooooooool0aOooool00ol0003oooooooooo`0Zooooo`00;?ooool00ol0
003oooooooooo`33ooooo`03o`000?oooooooooo02_ooooo000]ooooo`03o`000?oooooooooo0<7o
oooo00?o0000ooooooooool0;?ooool002kooooo00?o0000ooooooooool0_oooool00ol0003ooooo
ooooo`0]ooooo`00;oooool00ol0003oooooooooo`2mooooo`03o`000?oooooooooo02kooooo000`
ooooo`03o`000?oooooooooo0;_ooooo00?o0000ooooooooool0;oooool0037ooooo00?o0000oooo
ooooool0^Oooool00ol0003oooooooooo`0`ooooo`00<_ooool00ol0003oooooooooo`2gooooo`03
o`000?oooooooooo037ooooo000cooooo`03o`000?oooooooooo0;Gooooo00?o0000ooooooooool0
<_ooool003Cooooo00?o0000ooooooooool0/oooool00ol0003oooooooooo`0cooooo`00=Oooool0
0ol0003oooooo`3o0002o`3o0:_ooooo0ooo00000ooooooo0000ooooo`0eooooo`00=_ooool5o`3o
0:Wooooo1Ooo000gooooo`00=Oooool7o`3o0:Oooooo1ooo000fooooo`00=Oooool7o`3o0:Oooooo
1ooo000fooooo`00=Oooool7o`3o0:Oooooo1ooo000fooooo`00=_ooool5o`3o0003o`000?oooooo
oooo0:Gooooo00?o0000ool00?oo00000ooo000gooooo`00=oooool3o`3o00;ooooo0_l0002Soooo
o`;o00000_ooool3ool003Sooooo000nooooo`03o`000?oooooooooo09oooooo00?o0000oooooooo
ool0?Oooool003oooooo00?o0000ooooooooool0WOooool00ol0003oooooooooo`0nooooo`00@?oo
ool00ol0003oooooooooo`2Kooooo`03o`000?oooooooooo03oooooo0011ooooo`;o0000VOooool2
o`0004;ooooo0013ooooo`03o`000?oooooooooo09Gooooo00?o0000ooooooooool0@_ooool004Co
oooo00?o0000ooooooooool0Toooool00ol0003oooooooooo`13ooooo`00AOooool2o`00097ooooo
0_l00016ooooo`00Aoooool00ol0003oooooooooo`2=ooooo`03o`000?oooooooooo04Kooooo0018
ooooo`03o`000?oooooooooo08_ooooo00?o0000ooooooooool0Aoooool004Wooooo00?o0000oooo
ooooool0ROooool00ol0003oooooooooo`18ooooo`00B_ooool2o`000004ooooool0o`3o0?l0o`3o
07oooooo0ooo00000ooooooo0000o`00001;ooooo`00C?ooool5o`3o07gooooo1Ooo001=ooooo`00
Boooool7o`3o07_ooooo1ooo001<ooooo`00Boooool7o`3o07_ooooo1ooo001<ooooo`00Boooool7
o`3o07_ooooo1ooo001<ooooo`00C?ooool5o`3o0004ooooool0003o0000o`0007Gooooo0ol00000
0oooooooo`00ool00003ool004gooooo001=ooooo`?o0?l01Oooool2o`00077ooooo0_l00005oooo
o`?oo`00C_ooool005Oooooo0_l0001]ooooo`;o0000F?ooool005Wooooo0ol0001Wooooo`?o0000
F_ooool005cooooo0_l0001Sooooo`;o0000GOooool005kooooo0_l0001Oooooo`;o0000Goooool0
063ooooo0_l00004ooooo`?o0?l0COooool3ool000Cooooo0_l0001Qooooo`00H_ooool3o`0000Go
0?l0Boooool5ool000?o0000Hoooool006Cooooo1ol0o`19ooooo`Ooo`00IOooool006Cooooo1ol0
o`19ooooo`Ooo`00IOooool006Cooooo1ol0o`05o`0003oooooo1Ol00007ool006Gooooo001Uoooo
o`Go0?l01_ooool5o`0000_ooooo0ol0o`0Iooooo`?oo`002oooool5o`0000Kooooo1Ooo001Voooo
o`00I_ooool3o`3o00cooooo1Ol00005ooooo`Go0?l05oooool5ool000Gooooo1Ol0000<ooooo`?o
o`00Ioooool007[ooooo1?l00007o`3o01Gooooo1ooo0004o`0007_ooooo001nooooo`Oo0?l05Ol0
0007ool007oooooo001nooooo`Oo0?l05Oooool7ool007oooooo001oooooo`Go0?l05oooool5ool0
083ooooo0020ooooo`?o0?l06Oooool3ool0087ooooo003oooooob7ooooo003oooooob7ooooo003o
ooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003o
ooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{73.4375, 302.812}, {286.438, 57.0625}} -> {-0.402189, \
0.226394, 0.00383655, 0.00381744}},
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "If you want to run the model forward a number of iterations, specify the \
number of iterations to the ",
  StyleBox["Generations", "Input"],
  " option, like so:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(Evolve[bos1, Generations \[Rule] 10]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Very quickly the model should arrive at a fixed state (it may have \
done so after the first iteration).\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "When you are done with a model, you should tell ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to close the window and release all of the resources it consumes using \
the ",
  StyleBox["DeleteModel", "Input"],
  " command."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[bos1]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["What just happened?", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "In the above, we created a local interaction model for the ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  " game and played it on a ring consisting of 30 agents (each vertex in the \
graph represents an agent). Each iteration, every agent played the game with \
his ",
  StyleBox["neighbors ",
    FontSlant->"Italic"],
  "(two agents are neighbors if there is an edge connecting them in the \
graph). When two agents play the game, a coin flip determines who is row and \
who is column. A coin is tossed for each pairwise interaction, so the same \
agent may be row in one game and column in another. Each agent receives a \
total payoff equalling the sum of the individual payoffs from each game they \
play. On a ring, each agent plays two games, with this neighbors on either \
side."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Once everyone has played the game, the ",
  "learning",
  " phase begins. The default learning rule is ",
  StyleBox["Imitate the Best",
    FontSlant->"Italic"],
  ". According to this rule, each agent inspects the score and strategy used \
by every agent in his ",
  StyleBox["update neighborhood",
    FontSlant->"Italic"],
  ".  When no update neighborhood is explicitly specified (as in the game \
above), the default is to use the same neighborhood as for the interactions. \
According to ",
  StyleBox["Imitate the Best",
    FontSlant->"Italic"],
  ", each agent looks at the who received the maximal score in his update \
neighborhood. If this maximal payoff ",
  StyleBox["strictly exceeds",
    FontSlant->"Italic"],
  " his own score, the agent will adopt the strategy conferring this maximal \
score; if more than one strategy earned the maximal score in his update \
neighborhood, then the agent tosses a coin to decide which to adopt. Once \
everyone has decided what strategy they will adopt, everyone switches in \
unison. Scores are reset to 0 and we are ready to begin the next interaction \
phase."
}], "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["More advanced features", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell["Basic topological variations", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Many of the parameters can be varied. For example, here's how to create a \
model for ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  " on a grid."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g = GridGraph[10, 10];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[g];\)\)}], "Input",
  CellLabel->"In[13]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.025 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[13]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`002oooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Joooo
o`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo000:ooooo`Go
00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go
00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00003?ooool000Wooooo1ol0000Fooooo`Oo0000
5_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo0000
5_ooool7o`0001Kooooo1ol0000;ooooo`002Oooooooo`0000go00002oooool000Wooooo1ol0000F
ooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000F
ooooo`Oo00005_ooool7o`0001Kooooo1ol0000;ooooo`002_ooool5o`0001Sooooo1Ol0000Hoooo
o`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hoooo
o`Go00006?ooool5o`0000cooooo000;ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o
00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o
00003Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000_ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3
o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=ooooo`002_ooool5o`00
01Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`00
01Sooooo1Ol0000Hooooo`Go00006?ooool5o`0000cooooo0009ooooo`Oo00005_ooool7o`0001Ko
oooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Ko
oooo1ol0000Fooooo`Oo00002oooool000Woooooool0000=o`0000_ooooo0009ooooo`Oo00005_oo
ool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_oo
ool7o`0001Kooooo1ol0000Fooooo`Oo00002oooool000[ooooo1Ol0000Hooooo`Go00006?ooool5
o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5
o`0001Sooooo1Ol0000<ooooo`002oooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`00
01[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`00
00gooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000;ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0
000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Oooool000[ooooo1Ol0000H
ooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000H
ooooo`Go00006?ooool5o`0001Sooooo1Ol0000<ooooo`002Oooool7o`0001Kooooo1ol0000Foooo
o`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Foooo
o`Oo00005_ooool7o`0000_ooooo0009oooooooo00003Ol0000;ooooo`002Oooool7o`0001Kooooo
1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo
1ol0000Fooooo`Oo00005_ooool7o`0000_ooooo000:ooooo`Go00006?ooool5o`0001Sooooo1Ol0
000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0
000Hooooo`Go00003?ooool000_ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000J
ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
2oooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o0000
6_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo000:ooooo`Go00006?oo
ool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?oo
ool5o`0001Sooooo1Ol0000Hooooo`Go00003?ooool000Wooooo1ol0000Fooooo`Oo00005_ooool7
o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7
o`0001Kooooo1ol0000;ooooo`002Oooooooo`0000go00002oooool000Wooooo1ol0000Fooooo`Oo
00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo
00005_ooool7o`0001Kooooo1ol0000;ooooo`002_ooool5o`0001Sooooo1Ol0000Hooooo`Go0000
6?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go0000
6?ooool5o`0000cooooo000;ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_oo
ool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Ooo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000_o
oooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[o
oooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=ooooo`002_ooool5o`0001Sooooo
1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo
1Ol0000Hooooo`Go00006?ooool5o`0000cooooo0009ooooo`Oo00005_ooool7o`0001Kooooo1ol0
000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0
000Fooooo`Oo00002oooool000Woooooool0000=o`0000_ooooo0009ooooo`Oo00005_ooool7o`00
01Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`00
01Kooooo1ol0000Fooooo`Oo00002oooool000[ooooo1Ol0000Hooooo`Go00006?ooool5o`0001So
oooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001So
oooo1Ol0000<ooooo`002oooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo
0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000;oooo
o`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Joooo
o`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Oooool000[ooooo1Ol0000Hooooo`Go
00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go
00006?ooool5o`0001Sooooo1Ol0000<ooooo`002Oooooooo`0000go00002oooool000Wooooo1ol0
000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0
000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000;ooooo`002Oooool7o`0001Kooooo1ol0000F
ooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000F
ooooo`Oo00005_ooool7o`0000_ooooo000:ooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hoooo
o`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hoooo
o`Go00003?ooool000_ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o
00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`002oooool3
o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3
o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo000:ooooo`Go00006?ooool5o`00
01Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`00
01Sooooo1Ol0000Hooooo`Go00003?ooool000Woooooool0000=o`0000_ooooo0009ooooo`Oo0000
5_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo0000
5_ooool7o`0001Kooooo1ol0000Fooooo`Oo00002oooool000Wooooo1ol0000Fooooo`Oo00005_oo
ool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_oo
ool7o`0001Kooooo1ol0000;ooooo`002_ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5
o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5
o`0000cooooo000;ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`00
01[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Oooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000_ooooo0ol0
000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0
000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=ooooo`002_ooool5o`0001Sooooo1Ol0000H
ooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000H
ooooo`Go00006?ooool5o`0000cooooo0009oooooooo00003Ol0000;ooooo`002Oooool7o`0001Ko
oooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Ko
oooo1ol0000Fooooo`Oo00005_ooool7o`0000_ooooo0009ooooo`Oo00005_ooool7o`0001Kooooo
1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo
1ol0000Fooooo`Oo00002oooool000[ooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0
000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0
000<ooooo`002oooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000J
ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000;ooooo`?o0000
6_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o0000
6_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Oooool000[ooooo1Ol0000Hooooo`Go00006?oo
ool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?oo
ool5o`0001Sooooo1Ol0000<ooooo`002Oooooooo`0000go00002oooool000Wooooo1ol0000Foooo
o`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Foooo
o`Oo00005_ooool7o`0001Kooooo1ol0000;ooooo`002Oooool7o`0001Kooooo1ol0000Fooooo`Oo
00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo
00005_ooool7o`0000_ooooo000:ooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go0000
6?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go0000
3?ooool000_ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_oo
ool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000=ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`002oooool3o`0001[o
oooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[o
oooo0ol0000Jooooo`?o00006_ooool3o`0000gooooo000:ooooo`Go00006?ooool5o`0001Sooooo
1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo
1Ol0000Hooooo`Go00003?ooool000Woooooool0000=o`0000_ooooo0009ooooo`Oo00005_ooool7
o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7
o`0001Kooooo1ol0000Fooooo`Oo00002oooool000Wooooo1ol0000Fooooo`Oo00005_ooool7o`00
01Kooooo1ol0000Fooooo`Oo00005_ooool7o`0001Kooooo1ol0000Fooooo`Oo00005_ooool7o`00
01Kooooo1ol0000;ooooo`002_ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0001So
oooo1Ol0000Hooooo`Go00006?ooool5o`0001Sooooo1Ol0000Hooooo`Go00006?ooool5o`0000co
oooo000;ooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00006_ooool3o`0001[ooooo
0ol0000Jooooo`?o00006_ooool3o`0001[ooooo0ol0000Jooooo`?o00003Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(bos2 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] g, \[IndentingNewLine]PayoffMatrix \[Rule] 
            battleOfSexes, \[IndentingNewLine]StrategyColors \[Rule] 
            bosColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "To put a copy of the current display of a model into a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook, use ",
  StyleBox["ExtractGraph", "Input"],
  ", passing the model as the argument. The extracted graph is a standard \
graph object that is understood by ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  ", so you can display it using ",
  StyleBox["ShowGraph", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ExtractGraph[bos2] // ShowGraph;\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.025 w
.04545 .04545 Mdot
.14646 .04545 Mdot
1 0 0 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 0 r
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
0 1 0 r
.75253 .04545 Mdot
1 0 0 r
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 0 0 r
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
0 1 0 r
.14646 .24747 Mdot
.24747 .24747 Mdot
1 0 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
0 1 0 r
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
0 1 0 r
.75253 .24747 Mdot
1 0 0 r
.85354 .24747 Mdot
0 1 0 r
.95455 .24747 Mdot
.04545 .34848 Mdot
1 0 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
0 1 0 r
.85354 .34848 Mdot
1 0 0 r
.95455 .34848 Mdot
.04545 .44949 Mdot
0 1 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
0 1 0 r
.95455 .44949 Mdot
1 0 0 r
.04545 .55051 Mdot
0 1 0 r
.14646 .55051 Mdot
1 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 0 r
.44949 .55051 Mdot
1 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 1 0 r
.95455 .55051 Mdot
1 0 0 r
.04545 .65152 Mdot
0 1 0 r
.14646 .65152 Mdot
1 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
0 1 0 r
.44949 .65152 Mdot
.55051 .65152 Mdot
1 0 0 r
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
1 0 0 r
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 1 0 r
.75253 .85354 Mdot
1 0 0 r
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 1 0 r
.55051 .95455 Mdot
1 0 0 r
.65152 .95455 Mdot
0 1 0 r
.75253 .95455 Mdot
1 0 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[16]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`002oooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ooo000Joooo
o`?o0?l06_ooool3ool001[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool000gooooo000:ooooo`Go
0?l06?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Go
o`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`003?ooool000Wooooo1ol0o`0Fooooo`Oo0?l0
5_ooool7ool001Kooooo1ooo000Fooooo`Ooo`005_ooool7o`3o01Kooooo1ooo000Fooooo`Oo0?l0
5_ooool7ool001Kooooo1ooo000;ooooo`002Oooool7o`3o01Ko00001ol0o`0Fo`0000Ooo`005_l0
0007ool001Ko00001ooo000Fo`0000Oo0?l05_l00007ool001Ko00001ol0o`0Fo`0000Ooo`005_l0
0007ool000_ooooo0009ooooo`Oo0?l05_ooool7o`3o01Kooooo1ooo000Fooooo`Ooo`005_ooool7
ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o01Kooooo1ooo000Fooooo`Ooo`002oooool0
00[ooooo1Ol0o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Goo`006?ooool5o`3o
01Sooooo1Ooo000Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000<ooooo`002oooool3o`3o01[o
oooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool001[o
oooo0ol0o`0Jooooo`?oo`006_ooool3ool000gooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000;ooooo`?o0?l06_ooool3o`3o01[ooooo0ol0o`0J
ooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ooo000J
ooooo`?oo`003Oooool000[ooooo1Ol0o`0Hooooo`Go0?l06?ooool5o`3o01Sooooo1Ooo000Hoooo
o`Go0?l06?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`006?ooool5ool001Sooooo1Ooo000<oooo
o`002Oooool7o`3o01Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Kooooo1ol0o`0Fooooo`Oo
0?l05_ooool7ool001Kooooo1ooo000Fooooo`Ooo`005_ooool7ool000_ooooo0009ooooo`Oo0?l0
5_l00007o`3o01Ko00001ol0o`0Fo`0000Ooo`005_l00007o`3o01Ko00001ol0o`0Fo`0000Ooo`00
5_l00007ool001Ko00001ooo000Fo`0000Ooo`002oooool000Wooooo1ol0o`0Fooooo`Oo0?l05_oo
ool7o`3o01Kooooo1ooo000Fooooo`Oo0?l05_ooool7o`3o01Kooooo1ooo000Fooooo`Ooo`005_oo
ool7ool001Kooooo1ooo000;ooooo`002_ooool5o`3o01Sooooo1Ol0o`0Hooooo`Go0?l06?ooool5
ool001Sooooo1Ol0o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Goo`006?ooool5
ool000cooooo000;ooooo`?o0?l06_ooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3o`3o
01[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ooo000Jooooo`?oo`003Oooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000_ooooo0ooo
000Jooooo`?o0?l06_ooool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3o`3o01[ooooo0ooo
000Jooooo`?o0?l06_ooool3ool001[ooooo0ol0o`0=ooooo`002_ooool5ool001Sooooo1Ol0o`0H
ooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Go0?l06?ooool5ool001Sooooo1Ol0o`0H
ooooo`Goo`006?ooool5o`3o00cooooo0009ooooo`Ooo`005_ooool7o`3o01Kooooo1ol0o`0Foooo
o`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o01Kooooo1ooo000Foooo
o`Oo0?l02oooool000Wooooo1ooo000Fo`0000Oo0?l05_l00007o`3o01Ko00001ooo000Fo`0000Oo
o`005_l00007o`3o01Ko00001ooo000Fo`0000Oo0?l05_l00007ool001Ko00001ol0o`0;ooooo`00
2Oooool7ool001Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Kooooo1ooo000Fooooo`Oo0?l0
5_ooool7ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o00_ooooo000:ooooo`Goo`006?oo
ool5o`3o01Sooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Goo`006?oo
ool5o`3o01Sooooo1Ooo000Hooooo`Go0?l03?ooool000_ooooo0ooo000Jooooo`?o0?l06_ooool3
o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3o`3o01[ooooo0ooo000Jooooo`?o0?l06_ooool3
ool001[ooooo0ol0o`0=ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`002oooool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3o`3o01[ooooo
0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool000gooooo
000:ooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Go0?l06?ooool5o`3o01Sooooo1Ooo
000Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Goo`003?ooool000Wooooo1ol0o`0F
ooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Kooooo1ooo000F
ooooo`Ooo`005_ooool7o`3o01Kooooo1ooo000;ooooo`002Oooool7o`3o01Ko00001ooo000Fo`00
00Ooo`005_l00007o`3o01Ko00001ol0o`0Fo`0000Ooo`005_l00007ool001Ko00001ooo000Fo`00
00Oo0?l05_l00007ool000_ooooo0009ooooo`Oo0?l05_ooool7ool001Kooooo1ooo000Fooooo`Oo
0?l05_ooool7o`3o01Kooooo1ooo000Fooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Oo
o`002oooool000[ooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Go0?l0
6?ooool5ool001Sooooo1Ooo000Hooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000<ooooo`002ooo
ool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_oo
ool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool000gooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000;ooooo`?oo`006_ooool3o`3o01[o
oooo0ol0o`0Jooooo`?o0?l06_ooool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3ool001[o
oooo0ooo000Jooooo`?o0?l03Oooool000[ooooo1Ooo000Hooooo`Go0?l06?ooool5o`3o01Sooooo
1Ol0o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Goo`006?ooool5ool001Sooooo
1Ol0o`0<ooooo`002Oooool7ool001Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7o`3o01Kooooo1ol0
o`0Fooooo`Ooo`005_ooool7ool001Kooooo1ooo000Fooooo`Ooo`005_ooool7o`3o00_ooooo0009
ooooo`Ooo`005_l00007o`3o01Ko00001ol0o`0Fo`0000Oo0?l05_l00007o`3o01Ko00001ooo000F
o`0000Ooo`005_l00007ool001Ko00001ooo000Fo`0000Oo0?l02oooool000Wooooo1ooo000Foooo
o`Oo0?l05_ooool7o`3o01Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Kooooo1ooo000Foooo
o`Ooo`005_ooool7ool001Kooooo1ol0o`0;ooooo`002_ooool5ool001Sooooo1Ol0o`0Hooooo`Go
0?l06?ooool5o`3o01Sooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo1Ooo000Hooooo`Go
o`006?ooool5o`3o00cooooo000;ooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0Jooooo`?o0?l0
6_ooool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3ool001[ooooo0ooo000Jooooo`?o0?l0
3Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00_ooooo0ooo000Jooooo`?o0?l06_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool0
01[ooooo0ooo000Jooooo`?oo`006_ooool3ool001[ooooo0ol0o`0=ooooo`002_ooool5ool001So
oooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001So
oooo1Ooo000Hooooo`Goo`006?ooool5o`3o00cooooo0009ooooo`Ooo`005_l00007o`3o01Ko0000
1ooo000Fo`0000Ooo`005_l00007o`3o01Ko00001ooo000Fo`0000Ooo`005_l00007ool001Ko0000
1ooo000Fo`0000Oo0?l02oooool000Wooooo1ooo000Fooooo`Oo0?l05_ooool7ool001Kooooo1ooo
000Fooooo`Oo0?l05_ooool7ool001Kooooo1ooo000Fooooo`Ooo`005_ooool7ool001Kooooo1ol0
o`0;ooooo`002Oooool7ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0F
ooooo`Ooo`005_ooool7ool001Kooooo1ooo000Fooooo`Ooo`005_ooool7o`3o00_ooooo000:oooo
o`Goo`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000Hoooo
o`Goo`006?ooool5ool001Sooooo1Ooo000Hooooo`Go0?l03?ooool000_ooooo0ooo000Jooooo`?o
0?l06_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool001[ooooo0ooo000Jooooo`?o
o`006_ooool3ool001[ooooo0ol0o`0=ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`002oooool3ool001[ooooo0ol0o`0Jooooo`?oo`006_ooool3
ool001[ooooo0ol0o`0Jooooo`?o0?l06_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3
o`3o00gooooo000:ooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`006?ooool5o`3o
01Sooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo1Ol0o`0Hooooo`Go0?l03?ooool000Wo
oooo1ooo000Fo`0000Oo0?l05_l00007ool001Ko00001ooo000Fo`0000Oo0?l05_l00007o`3o01Ko
00001ooo000Fo`0000Ooo`005_l00007o`3o01Ko00001ol0o`0;ooooo`002Oooool7ool001Kooooo
1ol0o`0Fooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Kooooo
1ooo000Fooooo`Oo0?l05_ooool7o`3o00_ooooo0009ooooo`Ooo`005_ooool7o`3o01Kooooo1ooo
000Fooooo`Ooo`005_ooool7o`3o01Kooooo1ol0o`0Fooooo`Ooo`005_ooool7ool001Kooooo1ol0
o`0Fooooo`Oo0?l02oooool000[ooooo1Ooo000Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000H
ooooo`Go0?l06?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`006?ooool5o`3o01Sooooo1Ol0o`0<
ooooo`002oooool3ool001[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo0ol0o`0Joooo
o`?o0?l06_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3o`3o00gooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000;ooooo`?o0?l06_oo
ool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_oo
ool3o`3o01[ooooo0ol0o`0Jooooo`?o0?l03Oooool000[ooooo1Ol0o`0Hooooo`Go0?l06?ooool5
o`3o01Sooooo1Ooo000Hooooo`Go0?l06?ooool5o`3o01Sooooo1Ooo000Hooooo`Go0?l06?ooool5
o`3o01Sooooo1Ol0o`0<ooooo`002Oooool7o`3o01Ko00001ol0o`0Fo`0000Oo0?l05_l00007ool0
01Ko00001ol0o`0Fo`0000Oo0?l05_l00007ool001Ko00001ol0o`0Fo`0000Oo0?l05_l00007o`3o
00_ooooo0009ooooo`Oo0?l05_ooool7o`3o01Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o01Ko
oooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o01Kooooo1ol0o`0Fooooo`Oo0?l02oooool000Wooooo
1ol0o`0Fooooo`Oo0?l05_ooool7o`3o01Kooooo1ooo000Fooooo`Oo0?l05_ooool7o`3o01Kooooo
1ooo000Fooooo`Oo0?l05_ooool7o`3o01Kooooo1ol0o`0;ooooo`002_ooool5o`3o01Sooooo1Ol0
o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ol0o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ol0
o`0Hooooo`Go0?l06?ooool5o`3o00cooooo000;ooooo`?o0?l06_ooool3o`3o01[ooooo0ol0o`0J
ooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3o`3o01[ooooo0ol0o`0J
ooooo`?o0?l03Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000_ooooo0ol0o`0Jooooo`?o0?l06_ooool3o`3o01[ooooo0ooo000Jooooo`?oo`00
6_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool001[ooooo0ol0o`0=ooooo`002_oo
ool5o`3o01Sooooo1Ol0o`0Hooooo`Go0?l06?ooool5ool001Sooooo1Ooo000Hooooo`Goo`006?oo
ool5ool001Sooooo1Ol0o`0Hooooo`Goo`006?ooool5o`3o00cooooo0009ooooo`Oo0?l05_l00007
o`3o01Ko00001ol0o`0Fo`0000Ooo`005_l00007ool001Ko00001ooo000Fo`0000Ooo`005_l00007
o`3o01Ko00001ooo000Fo`0000Oo0?l02oooool000Wooooo1ol0o`0Fooooo`Oo0?l05_ooool7o`3o
01Kooooo1ooo000Fooooo`Ooo`005_ooool7ool001Kooooo1ooo000Fooooo`Oo0?l05_ooool7ool0
01Kooooo1ol0o`0;ooooo`002Oooool7o`3o01Kooooo1ol0o`0Fooooo`Oo0?l05_ooool7ool001Ko
oooo1ooo000Fooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o00_o
oooo000:ooooo`Go0?l06?ooool5o`3o01Sooooo1Ol0o`0Hooooo`Goo`006?ooool5ool001Sooooo
1Ooo000Hooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Go0?l03?ooool000_ooooo0ol0
o`0Jooooo`?o0?l06_ooool3o`3o01[ooooo0ooo000Jooooo`?oo`006_ooool3ool001[ooooo0ooo
000Jooooo`?o0?l06_ooool3ool001[ooooo0ol0o`0=ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`002oooool3o`3o01[ooooo0ol0o`0Jooooo`?o
o`006_ooool3ool001[ooooo0ooo000Jooooo`?o0?l06_ooool3ool001[ooooo0ol0o`0Jooooo`?o
o`006_ooool3ool000gooooo000:ooooo`Go0?l06?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`00
6?ooool5ool001Sooooo1Ol0o`0Hooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Goo`00
3?ooool000Wooooo1ol0o`0Fo`0000Oo0?l05_l00007ool001Ko00001ooo000Fo`0000Ooo`005_l0
0007o`3o01Ko00001ooo000Fo`0000Oo0?l05_l00007ool001Ko00001ooo000;ooooo`002Oooool7
o`3o01Kooooo1ol0o`0Fooooo`Ooo`005_ooool7ool001Kooooo1ooo000Fooooo`Oo0?l05_ooool7
ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7ool000_ooooo0009ooooo`Oo0?l05_ooool7o`3o
01Kooooo1ooo000Fooooo`Ooo`005_ooool7ool001Kooooo1ol0o`0Fooooo`Ooo`005_ooool7o`3o
01Kooooo1ooo000Fooooo`Ooo`002oooool000[ooooo1Ol0o`0Hooooo`Go0?l06?ooool5ool001So
oooo1Ooo000Hooooo`Goo`006?ooool5o`3o01Sooooo1Ooo000Hooooo`Go0?l06?ooool5ool001So
oooo1Ooo000<ooooo`002oooool3o`3o01[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool001[ooooo
0ooo000Jooooo`?o0?l06_ooool3ool001[ooooo0ol0o`0Jooooo`?oo`006_ooool3ool000gooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "We can create a table of 16 generations by stepping the model forward 16 \
generations, extracting the graph after each generation and saving it in a \
list, and then generate the table using ",
  StyleBox["ShowGraphArray", "Input"],
  ". Stepping the model forward and saving the graph is pretty \
straightforward: "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(Do[\[IndentingNewLine]AppendTo[l, 
        ExtractGraph[bos2]]; \[IndentingNewLine]Evolve[
        bos2], \ {16}\[IndentingNewLine]]\)}], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "To display the table, partition the list ",
  StyleBox["l", "Input"],
  " into 4 rows of 4 columns each using the ",
  StyleBox["Partition", "Input"],
  " command, and then call ",
  StyleBox["ShowGraphArray", "Input"],
  " on this 4x4 array of graphs.  (The ",
  StyleBox["VertexStyle", "Input"],
  " option is part of the ",
  StyleBox["ShowGraph", "Input"],
  " command. We just use it here to make the disks a little bigger so that \
you can see what strategy each agent follows even though the graphs are \
rather small.)"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[\[IndentingNewLine]Partition[l, 4], 
        VertexStyle \[Rule] Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.0238095 0.238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.245293 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
1 0 0 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
1 0 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.0238095 0.488926 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
1 0 0 r
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
1 0 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 0 r
.04545 .85354 Mdot
1 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.0238095 0.732558 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
1 0 0 r
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
1 0 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 0 r
.04545 .85354 Mdot
1 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
1 0 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.0238095 0.97619 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
1 0 0 r
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
1 0 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 0 r
.04545 .85354 Mdot
1 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
1 0 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.267442 0.245293 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.267442 0.488926 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.267442 0.732558 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
1 0 0 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
0 1 0 r
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.267442 0.97619 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
1 0 0 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.511074 0.245293 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 0 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.511074 0.488926 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 0 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.511074 0.732558 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 0 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.511074 0.97619 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
1 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.754707 0.245293 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
1 0 0 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 0 r
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
0 1 0 r
.75253 .04545 Mdot
1 0 0 r
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 0 0 r
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
0 1 0 r
.14646 .24747 Mdot
.24747 .24747 Mdot
1 0 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
0 1 0 r
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
0 1 0 r
.75253 .24747 Mdot
1 0 0 r
.85354 .24747 Mdot
0 1 0 r
.95455 .24747 Mdot
.04545 .34848 Mdot
1 0 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
0 1 0 r
.85354 .34848 Mdot
1 0 0 r
.95455 .34848 Mdot
.04545 .44949 Mdot
0 1 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
0 1 0 r
.95455 .44949 Mdot
1 0 0 r
.04545 .55051 Mdot
0 1 0 r
.14646 .55051 Mdot
1 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 0 r
.44949 .55051 Mdot
1 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 1 0 r
.95455 .55051 Mdot
1 0 0 r
.04545 .65152 Mdot
0 1 0 r
.14646 .65152 Mdot
1 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
0 1 0 r
.44949 .65152 Mdot
.55051 .65152 Mdot
1 0 0 r
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
1 0 0 r
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 1 0 r
.75253 .85354 Mdot
1 0 0 r
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 1 0 r
.55051 .95455 Mdot
1 0 0 r
.65152 .95455 Mdot
0 1 0 r
.75253 .95455 Mdot
1 0 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.754707 0.488926 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
1 0 0 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 0 r
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 0 0 r
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
.34848 .24747 Mdot
0 1 0 r
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
0 1 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
1 0 0 r
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 1 0 r
.95455 .34848 Mdot
1 0 0 r
.04545 .44949 Mdot
0 1 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
0 1 0 r
.95455 .44949 Mdot
1 0 0 r
.04545 .55051 Mdot
0 1 0 r
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
0 1 0 r
.44949 .55051 Mdot
1 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 1 0 r
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
1 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
1 0 0 r
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 1 0 r
.75253 .85354 Mdot
1 0 0 r
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.754707 0.732558 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
1 0 0 r
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 0 0 r
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
0 1 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
1 0 0 r
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 1 0 r
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
1 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
1 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.754707 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
1 0 0 r
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
1 0 0 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
1 0 0 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
1 0 0 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 1 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
1 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
1 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 0 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
1 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 0 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[19]:=",
  ImageSize->{462, 462},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007>0001cR000`400?l00000o`00003oooooooo?
ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?
ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?
ooooo`00ooooooo?ooooo`00ooooooo?ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l0
1oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l0
1oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l0
1oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`00
1oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l0
1oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_oo
ool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_oo
ool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o
00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007ool0
00?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000go
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo
1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo
1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo
1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo
1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo
1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0
o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0
o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0
o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo
0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0
o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001ooo
ool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001ooo
ool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5
o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Kooooo1Ooo000>ooooo`003?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o
00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool0
00cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool0
00?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool0
00gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?o
oooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo
1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo
1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo
0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo
000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo
0005ooooo`Goo`001_ooool5ool000kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007
ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Oooooo0ooo0008
ooooo`?oo`003oooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?oo
ool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Ooo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?oo
ool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Sooooo0ooo000?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5
ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o
00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o
00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o
00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool0
00?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000co
oooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o
00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo000<ooooo`Oo0?l01?l00007o`3o00?o
00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o
00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o
00001ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000go
oooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007ool000?o
00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo
1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo
1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo
1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo
1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo
1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo
1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo
0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo
0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo
0006ooooo`Goo`003_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07
ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@
ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07
ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07
ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007
ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07
ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?
ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001ooo
ool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001ooo
ool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?oo
ool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5
o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5
ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool0
00cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool0
00gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o00?o
00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o
00001ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o00?o
00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o
00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o
00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o
00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o
00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo
1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo
1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo
1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo
1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo
1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo
1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo
000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo
0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo
0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo
000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08
ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08
ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008
ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08
ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007
ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08
ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?oo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003ooo
ool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5
ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`003?ooool7o`3o
00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o
00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o
00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<ooooo`Oo0?l00ol00007o`3o00Co
00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co
00001ooo0003o`0000Ooo`003?ooool7o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co
00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000go
oooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co
00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00Co
00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000?o
00001ooo0004o`0000Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo
1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo
1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo
1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo
0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000kooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo
0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo
0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000>
ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007
ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000A
ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3
o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3
ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3
ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3
o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3
ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3
o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo000?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool0
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool0
00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Co
oooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?o
oooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo
1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007ool000?o0000
1ooo0003o`0000Ooo`001?l00007ool000?o00001ooo000<ooooo`Oo0?l01?l00007o`3o00?o0000
1ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o0000
1ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o0000
1ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000gooooo
1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o0000
1ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo1ol0
o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo
0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0
o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo
000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo
0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0
o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo
0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05
ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005
ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005
ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05
ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005
ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05
ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`003_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@oooo
o`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07oooo
o`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?oooo
o`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3
ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3
ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3
ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3
o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3
ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3
ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3
ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o
00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000co
oooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?o
oooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000go
oooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o00?o0000
1ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o0000
1ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o00?o0000
1ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o0000
1ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o0000
1ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o0000
1ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o0000
1ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0
o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo
0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo
0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0
o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo
0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0
o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo
0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>
ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05
ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005
ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>
ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008oooo
o`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08oooo
o`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008oooo
o`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08oooo
o`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007oooo
o`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008oooo
o`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3
o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool0
00gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00oooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`003?ooool7o`3o00?o
oooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?o
oooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<ooooo`Oo0?l00ol00007o`3o00Co0000
1ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co0000
1ooo0003o`0000Ooo`003?ooool7o`3o00Co00001ol0o`03o`0000Ooo`000ol00007ool000Co0000
1ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000gooooo
1ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co0000
1ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00Co0000
1ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000?o0000
1ooo0004o`0000Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo
0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo
000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo
0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0
o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo
0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo
0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo
000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06
ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005
ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000>oooo
o`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07oooo
o`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aoooo
o`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3ool0
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00Oooooo0ooo000@ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?oo`002?ooool3ool0
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0
00Sooooo0ooo000?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000ko
oooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000ko
oooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo
1ooo000=ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo
1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo1ol0
o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo
0003o`0000Ooo`001?l00007ool000?o00001ooo000<ooooo`Oo0?l01?l00007ool000?o00001ooo
0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo
0003o`0000Ooo`003Oooool7o`3o00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo
0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000gooooo1ol0
o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo
0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo1ol0o`04
ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004
ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004
ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo000=
ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004
ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ooo0004
ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003
ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005oooo
o`Goo`003_ooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006oooo
o`Goo`003_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o
0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
o`001oooool3ool0017ooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`00
3oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0
013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0
00Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Ko
oooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Go
oooo1Ooo000?ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000cooooo
1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo
1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo
1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o00?o00001ol0
o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo
0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo
0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo
000=ooooo`Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo
0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ooo
0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo
0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003
ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003
ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003
ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03
ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003
ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003
ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004
ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>oooo
o`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>oooo
o`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o
0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o
o`004Oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo003ooooooloooooo
003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo
003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo
003ooooooloooooo003ooooooloooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Oooooo0ooo0008oooo
o`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
o`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`00
3?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l0
0ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l0
0ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`00
0ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l0
0ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`00
3Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l0
0ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool0
00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o00So
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000So
oooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oo
oooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07oooo
o`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@oooo
o`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07oooo
o`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07oooo
o`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?oooo
o`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Go
o`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go
o`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000<ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l0
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo1ol0o`03o`0000Oo0?l01?l0
0007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l0
0007ool000?o00001ooo000<ooooo`Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l0
0007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`003Ooo
ool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l0
0007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l01?l0
0007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`000ol0
0007ool000Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7
o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7
o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7
o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7
o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o
00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000ko
oooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017o
oooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>
ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08oooo
o`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08oooo
o`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008oooo
o`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08oooo
o`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007oooo
o`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08oooo
o`?o0?l01oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000=ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Go
o`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Go
o`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000cooooo1ol0o`03ooooo`Oo0?l0
1?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
1?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l0
1?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
3Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l0
1?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l0
1?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`00
0oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol0
0007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol0
0007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol0
0007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol0
0007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol0
0007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol0
0007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0003o`0000Ooo`001?l0
0007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7
o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7
o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7
o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7
o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool0
00gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o
00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o
00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o
00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oo
oooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oo
oooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oo
oooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oo
oooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oo
oooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oo
oooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oo
oooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo
000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000>oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aoooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`001_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
o`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`001Oooool5ool000Kooooo1Ooo000>ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`00
0oooool7ool000cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l0
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`00
1?ooool7ool000gooooo000<ooooo`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol0
0007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`003?oo
ool7o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol0
0007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l00ol0
0007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol0
0007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol0
0007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`003Ooo
ool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000>ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000So
oooo0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017o
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000So
oooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oo
oooo0ooo0008ooooo`?oo`003oooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o
0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?o
0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
o`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`00
3Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l0
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`00
3oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000<ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007
o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007
ool000?o00001ooo000<ooooo`Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007
ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`003Oooool7
o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007
ool000?o00001ooo0003o`0000Ooo`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l01?l00007
o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007
ool000Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o
00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Ko
oooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Ko
oooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Go
oooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000kooooo
0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo
0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo
0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo
0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo
0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>oooo
o`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o
0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o
o`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo000=ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l0
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000cooooo1ol0o`03ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Ooo
ool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007
ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007
ool000cooooo1ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`000ol00007
ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007
o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007
ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007
ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007
ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o
00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool0
00?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000go
oooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Go
oooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oo
oooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oooooo
0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo
0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo
0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o
0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o
o`001oooool3ool000Sooooo0ooo0007ooooo`?o0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?o
0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o
o`001oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?o
o`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aooooo`?o
0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000Gooooo1Ol0o`0>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`00
1_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l0
3oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo000>ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00ooo
ool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000gooooo000<ooooo`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007
ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo0?l03?ooool7
o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007
ool000Co00001ooo0003o`0000Ooo`000ol00007o`3o00gooooo1ol0o`03o`0000Oo0?l00ol00007
o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007
ool000?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007
ool000Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`003Oooool0
00cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`0<ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00?ooooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7o`3o
00gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Go
oooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o00Go
oooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo
0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo
0ooo0007ooooo`?o0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo
0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo
0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo
0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo
0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`003oooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`0@ooooo`?o0?l0
2?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000Oooooo0ol0o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l0
2?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`00
1oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003Ooo
ool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Go0?l01Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Ooo
ool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l03ooo
ool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000kooooo000<ooooo`Oo0?l00oooool7
o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ol0o`03ooooo`Oo0?l03?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7
o`3o00gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7
o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o
00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007o`3o
00?o00001ol0o`0<ooooo`Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool0
00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ol0o`03o`0000Oo0?l03Oooool7o`3o
00?o00001ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool0
00?o00001ooo0003o`0000Ooo`000ol00007o`3o00gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o
00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool0
00Co00001ooo000=ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7o`3o00co
oooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?o
oooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000go
oooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Go0?l03_ooool5o`3o00Kooooo
1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo
1Ooo0005ooooo`Go0?l01Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo
1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000kooooo0ol0
o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0
o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ol0
o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0
o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`00
3_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3o`3o013ooooo0ol0o`08ooooo`?o0?l0
1oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o0?l0
4Oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o00oooooo000=ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ol0o`05ooooo`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Go0?l03_ooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
o`3o00?ooooo1ol0o`0<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7
o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`000oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ol0o`0=ooooo`003?ooool7o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo0?l00ol00007o`3o
00cooooo1ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo0003o`0000Oo0?l03Oooool7o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007o`3o
00gooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Oo0?l03?ooool7o`3o00Co
oooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?o
oooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo0?l03Oooool000gooooo
1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001_ooool5o`3o00Gooooo1Ol0o`0>ooooo`Go0?l01_ooool5o`3o00Gooooo
1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00oooooo
1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`0>ooooo`003_ooool3o`3o00Oooooo0ol0
o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?o0?l01oooool3o`3o013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0
o`0Aooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0
o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3o`3o00oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l0
1oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`00
1oooool3ool000Sooooo0ooo0007ooooo`?o0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?oo`00
1oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`00
1oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`00
1oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aooooo`?o0?l0
1oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`00
1oooool3ool000Oooooo0ooo0008ooooo`?o0?l03oooool000gooooo1Ol0o`05ooooo`Go0?l01_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000Gooooo1Ol0o`0>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l03ooo
ool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ol0o`0>ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
o`3o00cooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7
o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
o`3o00gooooo000<ooooo`Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool0
00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo0?l03?ooool7o`3o
00Co00001ol0o`03o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool0
00Co00001ooo0003o`0000Ooo`000ol00007o`3o00gooooo1ol0o`03o`0000Oo0?l00ol00007ool0
00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool0
00?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool0
00Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo0?l03Oooool000co
oooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`0<ooooo`Oo0?l01?ooool7o`3o00?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7o`3o00go
oooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?o
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`0=ooooo`003Oooool5o`3o00Gooooo
1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo
1Ooo0006ooooo`Goo`001Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo
1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo
1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo
1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o00Gooooo
1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo
1Ooo0005ooooo`Goo`001_ooool5o`3o00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo
0007ooooo`?o0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0
o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?o0?l03oooool00?oooooocoooool00?oooooocoooool00?oooooocoooool00?oooooo
coooool00?oooooocoooool00?oooooocoooool00?oooooocoooool00?oooooocoooool00?oooooo
coooool00?oooooocoooool00?oooooocoooool00?oooooocoooool00?oooooocoooool000kooooo
0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo
0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo
0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo
0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo
0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo
0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo
0006ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo
000?ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0
o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0
o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo
0005ooooo`Goo`001_ooool5ool000kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03
ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003
ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03
ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03
ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003
ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03
ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000?ooooo1ooo0004
ooooo`Ooo`003Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`00
00Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo000<oooo
o`Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`00
00Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`03o`00
00Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`00
00Ooo`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`00
00Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=oooo
o`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo
0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo
0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Oo
o`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo
0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Oo
o`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo
0?l00oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l0
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000kooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?oo
ool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Ooo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?oo
ool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?oo
ool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Sooooo0ooo000?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo
0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo
0ooo0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo
0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo
0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo
0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo
0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo
0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0
o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Gooooo1Ooo
0006ooooo`Goo`003_ooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03
ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<
ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03
ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03
ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003
ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03
ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=
ooooo`003?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`00
00Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`00
00Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`00
00Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`00
00Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0003o`00
00Ooo`003Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`00
00Oo0?l00ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo
0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo
o`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo
0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Oo
o`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo
0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo
0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo
o`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`00
3oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l0
1_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001ooo
ool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001ooo
ool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01ooo
ool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0007ooooo`?oo`002?oo
ool3ool000oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo
0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo
0ooo0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo
0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo
0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo
0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo
0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo
000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0
o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo
0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0
o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo
000>ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04
ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04
ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004
ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04
ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003
ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04
ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<oooo
o`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`00
00Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`003?ooool7o`3o00Co00001ol0o`03o`00
00Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`00
00Ooo`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`00
00Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=oooo
o`Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`00
00Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Oo
o`001?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo
o`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo
0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Oo
o`000oooool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`00
1Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?oo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001ooo
ool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01ooo
ool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003ooo
ool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0
o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0
o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo
0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0
o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0
o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06
ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06
ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?
ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006
ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06
ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03oooo
o`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003oooo
o`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03oooo
o`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03oooo
o`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003oooo
o`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03oooo
o`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004oooo
o`Ooo`003Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo
0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo000<ooooo`Oo
0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Oo
o`001?l00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`03o`0000Oo
0?l01?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Oo
o`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo
0?l01?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`00
3?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`00
0oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`00
3Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Ooo
ool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`003_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3
o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3
ool000Oooooo0ooo000@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3
o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3
o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3
ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3
o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo000?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0
o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0
o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
000Aooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0
o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05
ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005
ooooo`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005
ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05
ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005
ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05
ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006
ooooo`Goo`003_ooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03oooo
o`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo000<oooo
o`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003oooo
o`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03oooo
o`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003oooo
o`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03oooo
o`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=oooo
o`003?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Oo
o`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo
0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo
o`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Oo
0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Oo
o`003Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Oo
o`000ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_oo
ool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003ooo
ool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_oo
ool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3
o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3
ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3
ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3
o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3
ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3
o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3
ool000oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00ko
oooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0
o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo
0007ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0
o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo
0007ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0
o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>
ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05
ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005
ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005
ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>
ooooo`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004oooo
o`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04oooo
o`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004oooo
o`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04oooo
o`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003oooo
o`Ooo`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004oooo
o`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<ooooo`Oo
0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo
o`000ol00007ool000Co00001ooo0003o`0000Ooo`003?ooool7o`3o00Co00001ol0o`03o`0000Oo
0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo
o`000ol00007ool000gooooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Oo
o`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo
0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo
o`000ol00007ool000?o00001ooo0004o`0000Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo0?l0
1?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
1?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`00
1?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
3Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
1?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l0
1?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
0oooool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Ooo
ool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_oo
ool5ool000kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3
o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3
o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3
ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3
ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool0
00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07
ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007
ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07
ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ol0o`07
ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007
ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07
ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007
ooooo`?oo`001oooool3ool000Sooooo0ol0o`0?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06oooo
o`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006oooo
o`Go0?l01Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006oooo
o`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`0?oooo
o`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006oooo
o`Goo`001Oooool5ool000Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06oooo
o`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5o`3o00kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo
o`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Oo
0?l03?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Oo
o`000oooool7ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00gooooo1ol0o`03ooooo`Oo
0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Oo
o`000oooool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo
o`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo
0?l03Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`00
1?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007o`3o00?o00001ol0o`0<ooooo`Oo0?l0
1?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`00
1?l00007ool000?o00001ol0o`03o`0000Oo0?l03Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l0
1?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Oo0?l0
0ol00007o`3o00gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`00
1?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ol0o`0=ooooo`003?oo
ool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00ooo
ool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ol0o`03ooooo`Oo0?l03Ooo
ool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7o`3o00gooooo000=ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Kooooo1Ol0o`05ooooo`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5
ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Go0?l01Oooool5
o`3o00oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0006ooooo`Go0?l03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o
00Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool0
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ol0o`07ooooo`?o0?l04Oooool3o`3o
00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00Oooooo0ooo0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o
00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0
00Sooooo0ol0o`0?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08
ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`08
ooooo`?o0?l01oooool3o`3o013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008
ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`0A
ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008
ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`08
ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`07
ooooo`?o0?l02?ooool3o`3o00oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Kooooo1Ol0o`05oooo
o`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00oooooo1Ol0o`05oooo
o`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005oooo
o`Go0?l01Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05oooo
o`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`06oooo
o`Go0?l03_ooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Oo
o`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`0<ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo
o`001?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo
0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`03ooooo`Oo
0?l00oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Oo
o`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`0=ooooo`00
3?ooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`00
0ol00007ool000?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00cooooo1ol0o`04o`0000Oo0?l0
0ol00007o`3o00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo0?l0
0ol00007o`3o00?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`00
0ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007o`3o00?o00001ol0o`03o`0000Oo0?l0
3Oooool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`00
0ol00007ool000?o00001ol0o`03o`0000Oo0?l01?l00007o`3o00gooooo000<ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l03?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Oo0?l00ooo
ool7o`3o00gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l03Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ol0o`0>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l03oooool5
o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000Gooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01_ooool5
o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l01Oooool5
o`3o00Kooooo1Ol0o`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o
013ooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool0
00Oooooo0ooo0008ooooo`?o0?l01oooool3o`3o00Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o
00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3o`3o
00Oooooo0ol0o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool0
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o
00oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo
000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ol0o`07
ooooo`?o0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07
ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007
ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`08
ooooo`?o0?l03oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5o`3o00Gooooo1Ol0o`0>oooo
o`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`05oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005oooo
o`Go0?l01Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Kooooo1Ol0o`0>oooo
o`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo
o`000oooool7ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7o`3o00cooooo1ol0o`04ooooo`Oo
0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo
o`000oooool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Oo
o`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ol0o`03ooooo`Oo
0?l03Oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo
o`000oooool7ool000?ooooo1ooo0003ooooo`Oo0?l01?ooool7o`3o00gooooo000<ooooo`Oo0?l0
0ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`00
0ol00007ool000Co00001ol0o`03o`0000Oo0?l03?ooool7o`3o00Co00001ol0o`03o`0000Ooo`00
0ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Oo0?l0
0ol00007o`3o00gooooo1ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`00
0ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007o`3o00?o00001ol0o`0=ooooo`Oo0?l0
0ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`00
0ol00007ool000?o00001ol0o`04o`0000Oo0?l03Oooool000cooooo1ol0o`03ooooo`Oo0?l01?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7o`3o00?ooooo1ol0o`0<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`03ooooo`Oo0?l03Ooo
ool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l01?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000ooo
ool7o`3o00Cooooo1ol0o`0=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Go0?l01Oooool5
o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5
ool000Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l01_ooool5
o`3o00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool0
00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ol0o`07ooooo`?o0?l04?ooool3o`3o
00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0
00Sooooo0ooo0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l01oooool3ool0
00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3o`3o
00Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool0
00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ol0o`08ooooo`?o0?l03oooool000oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07oooo
o`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?oo`002?ooool3ool000Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?o0?l04Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007oooo
o`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07oooo
o`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ol0o`0?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Go
o`001Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go
o`001_ooool5o`3o00kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Oo0?l0
3?ooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo0?l0
3Oooool000cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l0
0007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ol0o`0<ooooo`Oo0?l01?l0
0007o`3o00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l0
0007ool000?o00001ooo0003o`0000Oo0?l03Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l0
0007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol0
0007o`3o00gooooo1ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l0
0007ool000?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ol0o`0=ooooo`003?ooool7
o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`000oooool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00oooool7
o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7
o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7o`3o00gooooo000=ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo0005ooooo`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o
00oooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0006ooooo`Go0?l03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oo
oooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o0?l04Oooool3o`3o00Oo
oooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000So
oooo0ol0o`0?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?oooo
o`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?oooo
o`00ooooooo?ooooo`00ooooooo?ooooo`00ooooooo?ooooo`003_ooool3o`3o00Oooooo0ol0o`08
ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3ool000Oooooo0ol0o`08
ooooo`?oo`001oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008
ooooo`?oo`001oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000A
ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`08
ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08
ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0007
ooooo`?oo`002?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Goo`001Oooool5o`3o00Kooooo1Ooo0005oooo
o`Goo`003_ooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005oooo
o`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05oooo
o`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005oooo
o`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005oooo
o`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006oooo
o`Goo`003_ooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Oo
o`001?ooool7o`3o00?ooooo1ooo0003ooooo`Oo0?l01?ooool7ool000?ooooo1ooo000<ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`03ooooo`Oo
o`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo
o`001?ooool7ool000?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Oo
o`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ol0o`03ooooo`Oo
0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`00
3?ooool7o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo0?l0
0ol00007ool000?o00001ol0o`04o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l0
0ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007o`3o00?o00001ooo0004o`0000Ooo`00
0ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`00
0ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`00
3Oooool7o`3o00?o00001ol0o`04o`0000Ooo`000ol00007o`3o00?o00001ol0o`04o`0000Oo0?l0
0ol00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000gooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5o`3o00Gooooo1Ooo0005ooooo`Go0?l01_ooool5
ool000Gooooo1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5
o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ol0o`05ooooo`Go0?l01_ooool5
ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5
ool000Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001Oooool5
ool000Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool0
00Oooooo0ooo0008ooooo`?o0?l01oooool3ool000Oooooo0ol0o`08ooooo`?oo`001oooool3ool0
013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3o`3o
00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o
00Oooooo0ooo0008ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool0
00Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3o`3o
00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool0
00oooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo
000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?oo`001oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007
ooooo`?oo`004?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ol0o`07
ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007
ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07
ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Oooooo0ooo0008
ooooo`?oo`003oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005oooo
o`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo000>oooo
o`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5o`3o00Gooooo1Ol0o`05oooo
o`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`05oooo
o`Go0?l01_ooool5ool000Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005oooo
o`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05oooo
o`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>oooo
o`003?ooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ol0o`04ooooo`Oo
0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000cooooo1ol0o`04ooooo`Oo
0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Oo
o`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo
o`000oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Oo
o`003Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo
0?l00oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<ooooo`Oo0?l0
0ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007o`3o00Co00001ol0o`03o`0000Ooo`00
0ol00007ool000Co00001ooo0003o`0000Ooo`003?ooool7o`3o00Co00001ol0o`03o`0000Oo0?l0
0ol00007ool000Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`00
0ol00007ool000gooooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Oo0?l0
0ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l0
0ol00007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`00
0ol00007ool000?o00001ooo0004o`0000Ooo`003Oooool000cooooo1ol0o`03ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ooo0003ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?oo
ool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`003Ooo
ool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ol0o`03ooooo`Oo0?l01?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo000=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5
ool000Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5
ool000kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5
ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5
o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5
ool000kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3o`3o
00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`004?ooool3o`3o
00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ol0o`07ooooo`?o0?l01oooool3ool0
00Sooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o
00Sooooo0ooo0007ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool0
00Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o
00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ooo0007oooo
o`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o00Oooooo0ooo0007oooo
o`?o0?l02?ooool3ool000Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007oooo
o`?oo`002?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007oooo
o`?oo`004Oooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`07oooo
o`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07oooo
o`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ooo000?ooooo`003Oooool5ool000Gooooo1Ol0o`06ooooo`Go
0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Go0?l01Oooool5ool000Gooooo1Ol0o`06ooooo`Go
o`001Oooool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
o`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go
0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0005ooooo`Go
o`001_ooool5ool000kooooo000<ooooo`Ooo`000oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`00
0oooool7ool000Cooooo1ol0o`03ooooo`Ooo`000oooool7o`3o00Cooooo1ooo0003ooooo`Oo0?l0
3?ooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Oo0?l0
0oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l0
0oooool7ool000Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`00
0oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Oo0?l0
0oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`00
3Oooool000cooooo1ooo0003o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l0
0007o`3o00?o00001ooo0003o`0000Oo0?l01?l00007ool000?o00001ol0o`0<ooooo`Oo0?l01?l0
0007o`3o00?o00001ooo0003o`0000Ooo`001?l00007o`3o00?o00001ol0o`03o`0000Ooo`001?l0
0007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o00?o00001ol0o`03o`0000Ooo`001?l0
0007o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`000ol0
0007ool000gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ol0o`03o`0000Oo0?l01?l0
0007o`3o00?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`003?ooool7
ool000?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7
ool000?ooooo1ol0o`04ooooo`Ooo`000oooool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00oooool7
ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7
ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Oo0?l00oooool7
o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7
o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Goo`001Oooool5o`3o
00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Goo`001Oooool5o`3o
00Kooooo1Ooo0005ooooo`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool0
00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00oooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o
00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0006ooooo`Goo`003_ooool000kooooo0ooo0007ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ooo0007ooooo`?oo`002?ooool3o`3o00Oooooo0ooo0007ooooo`?o0?l02?ooool3ool000Oo
oooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o00Oo
oooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oo
oooo0ol0o`07ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oo
oooo0ooo0007ooooo`?oo`001oooool3ool0017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oo
oooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`001oooool3ool000So
oooo0ooo000?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0>ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ooo0008oooo
o`?oo`001oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008oooo
o`?o0?l01oooool3ool0013ooooo0ooo0008ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008oooo
o`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aoooo
o`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3o`3o00Oooooo0ooo0008oooo
o`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08oooo
o`?o0?l01oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007oooo
o`?oo`002?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Go
o`003_ooool5ool000Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo1Ol0o`05ooooo`Go
o`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00oooooo1Ol0o`05ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Go0?l01Oooool5ool000Kooooo1Ooo0005ooooo`Go
o`001Oooool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Go
0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Go
o`003_ooool000cooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ol0o`03ooooo`Oo0?l0
1?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7o`3o00?ooooo1ooo000<ooooo`Ooo`00
1?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`00
1?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l0
1?ooool7ool000?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`00
0oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l0
1?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?oo
ool7o`3o00?o00001ooo0004o`0000Ooo`000ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol0
0007ool000?o00001ooo0004o`0000Oo0?l00ol00007ool000cooooo1ooo0004o`0000Oo0?l00ol0
0007o`3o00?o00001ooo0004o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol0
0007ool000?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol0
0007o`3o00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Ooo
ool7o`3o00?o00001ol0o`04o`0000Oo0?l00ol00007o`3o00?o00001ol0o`04o`0000Ooo`000ol0
0007ool000?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7
ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ol0o`03ooooo`Ooo`003?ooool7ool000Cooooo1ol0o`03ooooo`Oo0?l00oooool7
ool000Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7
o`3o00gooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Oo0?l00oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7
o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000?ooooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Goo`001_ooool5ool0
00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5o`3o
00Gooooo1Ooo000>ooooo`Goo`001_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5o`3o
00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o
00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool0
00Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o
00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool0
00Kooooo1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3o`3o00Oo
oooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3ool0013o
oooo0ooo0008ooooo`?o0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?o0?l01oooool3ool000Oo
oooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Oo
oooo0ol0o`08ooooo`?oo`001oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oo
oooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3o`3o00Oo
oooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oo
oooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00kooooo000>ooooo`?oo`001oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?o0?l04?ooool3ool000Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07oooo
o`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007oooo
o`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07oooo
o`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008oooo
o`?oo`003oooool000gooooo1Ooo0005ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ol0o`0>ooooo`Go
o`001_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`001_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`05ooooo`Go
0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Go
o`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go
o`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo000>ooooo`00
3?ooool7ool000?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7o`3o00cooooo1ooo0004ooooo`Oo0?l0
0oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`00
0oooool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l0
0oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Ooo`00
3Oooool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?ooooo1ooo0004ooooo`Ooo`00
0oooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000<ooooo`Ooo`000ol0
0007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol0
0007ool000Co00001ooo0003o`0000Oo0?l03?ooool7ool000Co00001ol0o`03o`0000Oo0?l00ol0
0007o`3o00Co00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol0
0007o`3o00gooooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol0
0007ool000Co00001ooo0003o`0000Ooo`000ol00007ool000?o00001ooo000=ooooo`Oo0?l00ol0
0007o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007ool000Co00001ooo0003o`0000Ooo`000ol0
0007ool000?o00001ooo0004o`0000Ooo`003Oooool000cooooo1ooo0003ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ol0o`0<ooooo`Ooo`001?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7
o`3o00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ooo000=ooooo`003Oooool5ool000Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5o`3o
00kooooo1Ooo0006ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool0
00Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o
00Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool0
00kooooo000>ooooo`?oo`001oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o0?l04?ooool3ool000So
oooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000So
oooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00So
oooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3ool000Oo
oooo0ooo000Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000So
oooo0ooo0007ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`003oooool000oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ooo0007ooooo`?o
0?l02?ooool3ool000Oooooo0ooo0007ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ol0o`0@ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`07ooooo`?o
o`002?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
0?l04Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?o
o`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07ooooo`?o
0?l02?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o
o`001oooool3ool000Sooooo0ooo000?ooooo`003Oooool5ool000Gooooo1Ol0o`06ooooo`Goo`00
1Oooool5ool000Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`00
1Oooool5o`3o00kooooo1Ooo0006ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Go0?l0
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l0
1Oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`00
1Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l0
1Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000kooooo000<ooooo`Ooo`000oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Oo0?l03?oo
ool7ool000Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l00ooo
ool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000ooo
ool7ool000?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000ooo
ool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`003Ooo
ool000cooooo1ooo0003o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Oo0?l01?l00007
ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ol0o`0<ooooo`Ooo`001?l00007
o`3o00?o00001ol0o`03o`0000Ooo`001?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007
ool000?o00001ooo0003o`0000Oo0?l03Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007
ool000?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`000ol00007
o`3o00gooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007
ool000?o00001ooo0003o`0000Ooo`000ol00007ool000Co00001ooo000=ooooo`003?ooool7ool0
00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ol0o`04ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0004ooooo`Ooo`000oooool7o`3o00cooooo1ooo0004ooooo`Oo0?l00oooool7o`3o
00?ooooo1ooo0004ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0003ooooo`Oo0?l03Oooool7o`3o
00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool0
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000gooooo000=ooooo`Goo`001Oooool5o`3o00Ko
oooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Go0?l03_ooool5ool000Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Ko
oooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00oo
oooo1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Ko
oooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Ko
oooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`003_ooool000kooooo0ooo0007ooooo`?o0?l02?ooool3ool000Oooooo
0ooo0007ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo
0ol0o`0@ooooo`?oo`002?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3o`3o00Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o0?l04Oooool3o`3o00Oooooo
0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo
0ooo0007ooooo`?oo`001oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo
0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo
0ooo000?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3ool000Oooooo0ol0o`08ooooo`?o
o`001oooool3ool000Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o
0?l01oooool3o`3o013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`0Aooooo`?o
0?l01oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?o
o`001oooool3ool000Oooooo0ol0o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`08ooooo`?o
o`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o
0?l02?ooool3o`3o00oooooo000=ooooo`Goo`001Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`00
1Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Go0?l0
3_ooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo0005ooooo`Go0?l01Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l0
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`00
1Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`00
1Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`06ooooo`Go0?l0
3_ooool000cooooo1ooo0003ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7o`3o00?ooooo1ol0o`0<ooooo`Oo0?l01?oo
ool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ol0o`03ooooo`Oo0?l03Oooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l00ooo
ool7o`3o00gooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?oo
ool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7o`3o00Cooooo1ol0o`0=ooooo`003?ooool7
ool000?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ol0o`04o`0000Oo0?l00ol00007
ool000?o00001ooo0004o`0000Oo0?l00ol00007o`3o00cooooo1ol0o`04o`0000Oo0?l00ol00007
ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007
o`3o00?o00001ol0o`0=ooooo`Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`000ol00007
ool000?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ol0o`03o`0000Oo0?l03Oooool7
o`3o00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007
ool000?o00001ooo0003o`0000Oo0?l01?l00007o`3o00gooooo000<ooooo`Ooo`000oooool7o`3o
00Cooooo1ooo0003ooooo`Ooo`000oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool0
00Cooooo1ol0o`03ooooo`Oo0?l03?ooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Oo0?l00oooool7o`3o
00gooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool0
00Cooooo1ooo0003ooooo`Ooo`000oooool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o
00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool0
00?ooooo1ol0o`04ooooo`Oo0?l03Oooool000gooooo1Ooo0005ooooo`Go0?l01_ooool5ool000Go
oooo1Ooo0005ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5o`3o00Go
oooo1Ol0o`0>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Go
oooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Go
oooo1Ooo0005ooooo`Go0?l01Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Go
oooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Ko
oooo1Ol0o`0>ooooo`003_ooool3ool000Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo
0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3o`3o013ooooo
0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Oooooo
0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo
0ol0o`07ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo
0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?o0?l02?ooool3o`3o00oooooo
000?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00kooooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o
o`001oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3o`3o00Sooooo0ol0o`07ooooo`?o
0?l04?ooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?o
0?l01oooool3ool000Sooooo0ol0o`07ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o
0?l01oooool3o`3o00Sooooo0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o
0?l01oooool3o`3o00Oooooo0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?o
o`001oooool3ool000Sooooo0ooo0007ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`08ooooo`?o
0?l03oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ooo0005ooooo`Go0?l01_ooool5o`3o00Gooooo1Ol0o`0>ooooo`Go0?l0
1_ooool5o`3o00Gooooo1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ol0o`05ooooo`Goo`00
1_ooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`05ooooo`Go0?l0
1_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ol0o`05ooooo`Go0?l0
1Oooool5o`3o00oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`00
1_ooool5ool000Gooooo1Ooo0005ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`0>ooooo`003?oo
ool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ol0o`04ooooo`Oo0?l00ooo
ool7ool000?ooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00ooo
ool7o`3o00?ooooo1ooo0004ooooo`Ooo`000oooool7o`3o00?ooooo1ooo0004ooooo`Oo0?l00ooo
ool7o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000ooo
ool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l03Ooo
ool7o`3o00?ooooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000ooo
ool7ool000?ooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00gooooo000<ooooo`Oo0?l00ol00007
o`3o00Co00001ol0o`03o`0000Ooo`000ol00007o`3o00Co00001ol0o`03o`0000Ooo`000ol00007
o`3o00Co00001ol0o`03o`0000Oo0?l03?ooool7o`3o00Co00001ol0o`03o`0000Oo0?l00ol00007
ool000Co00001ooo0003o`0000Oo0?l00ol00007ool000Co00001ol0o`03o`0000Oo0?l00ol00007
o`3o00gooooo1ol0o`03o`0000Oo0?l00ol00007o`3o00Co00001ooo0003o`0000Ooo`000ol00007
ool000Co00001ooo0003o`0000Oo0?l00ol00007o`3o00?o00001ol0o`0=ooooo`Oo0?l00ol00007
o`3o00Co00001ol0o`03o`0000Ooo`000ol00007ool000Co00001ooo0003o`0000Ooo`000ol00007
o`3o00?o00001ol0o`04o`0000Oo0?l03Oooool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ooo0003ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ol0o`0<ooooo`Oo0?l01?ooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7ool0
00?ooooo1ol0o`03ooooo`Ooo`001?ooool7o`3o00?ooooo1ol0o`03ooooo`Oo0?l03Oooool7o`3o
00?ooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool0
00?ooooo1ol0o`03ooooo`Oo0?l00oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o
00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Oo0?l00oooool7o`3o
00Cooooo1ol0o`0=ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Go
oooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Gooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00ko
oooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Oooool5o`3o00Go
oooo1Ooo0006ooooo`Go0?l01Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Go
oooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5o`3o00Go
oooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Oooool5ool000Go
oooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00ko
oooo000>ooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3o`3o00Sooooo
0ol0o`07ooooo`?oo`001oooool3o`3o00Sooooo0ol0o`07ooooo`?o0?l04?ooool3o`3o00Sooooo
0ol0o`07ooooo`?o0?l01oooool3ool000Sooooo0ooo0007ooooo`?o0?l01oooool3ool000Sooooo
0ol0o`07ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l01oooool3o`3o00Sooooo
0ooo0007ooooo`?oo`001oooool3ool000Sooooo0ooo0007ooooo`?o0?l01oooool3o`3o00Oooooo
0ol0o`0Aooooo`?o0?l01oooool3o`3o00Sooooo0ol0o`07ooooo`?oo`001oooool3ool000Sooooo
0ooo0007ooooo`?oo`001oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l03oooool000oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool03_ooool000kooooo0ol0o`07ooooo`?o0?l0
2?ooool3o`3o00Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o0?l0
2?ooool3ool000Oooooo0ol0o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`00
2?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3o`3o00Oooooo0ooo0007ooooo`?o0?l0
4Oooool3o`3o00Oooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000Oooooo0ooo0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l0
2?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`00
1oooool3o`3o00Sooooo0ol0o`0?ooooo`003Oooool5o`3o00Gooooo1Ol0o`06ooooo`Go0?l01Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ol0o`06ooooo`Goo`001Ooo
ool5o`3o00kooooo1Ol0o`06ooooo`Go0?l01Oooool5o`3o00Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Go0?l01Oooool5ool000Gooooo1Ol0o`0?ooooo`Go0?l01Ooo
ool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`001Ooo
ool5ool000Gooooo1Ol0o`05ooooo`Go0?l03oooool5o`3o00Gooooo1Ol0o`06ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo0006ooooo`Goo`001Oooool5ool000Gooooo1Ooo0005ooooo`Go0?l01_oo
ool5o`3o00kooooo000<ooooo`Oo0?l00oooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7o`3o00Cooooo1ooo0003ooooo`Oo0?l03?ooool7
o`3o00Cooooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ol0o`03ooooo`Ooo`000oooool7o`3o00gooooo1ol0o`03ooooo`Oo0?l00oooool7
o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7
o`3o00?ooooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00Cooooo1ooo0003ooooo`Ooo`000oooool7
ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ol0o`04ooooo`Oo0?l03Oooool0
00cooooo1ol0o`03o`0000Oo0?l01?l00007o`3o00?o00001ooo0003o`0000Ooo`001?l00007ool0
00?o00001ooo0003o`0000Oo0?l01?l00007ool000?o00001ol0o`0<ooooo`Oo0?l01?l00007o`3o
00?o00001ol0o`03o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007o`3o
00?o00001ooo0003o`0000Oo0?l03Oooool7o`3o00?o00001ol0o`03o`0000Oo0?l01?l00007ool0
00?o00001ooo0003o`0000Ooo`001?l00007ool000?o00001ooo0003o`0000Oo0?l00ol00007o`3o
00gooooo1ol0o`03o`0000Oo0?l01?l00007ool000?o00001ooo0003o`0000Ooo`001?l00007ool0
00?o00001ooo0003o`0000Ooo`000ol00007o`3o00Co00001ol0o`0=ooooo`003?ooool7o`3o00?o
oooo1ol0o`04ooooo`Oo0?l00oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ol0o`04ooooo`Ooo`000oooool7o`3o00cooooo1ol0o`04ooooo`Oo0?l00oooool7o`3o00?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Oo0?l00oooool7ool000?o
oooo1ol0o`0=ooooo`Oo0?l00oooool7o`3o00?ooooo1ol0o`04ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`000oooool7ool000?ooooo1ol0o`03ooooo`Oo0?l03Oooool7o`3o00?o
oooo1ol0o`04ooooo`Ooo`000oooool7ool000?ooooo1ooo0004ooooo`Ooo`000oooool7ool000?o
oooo1ooo0003ooooo`Oo0?l01?ooool7o`3o00gooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo
1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5o`3o00Kooooo
1Ooo0005ooooo`Go0?l03_ooool5o`3o00Kooooo1Ol0o`05ooooo`Go0?l01Oooool5ool000Kooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ol0o`05ooooo`Goo`001Oooool5o`3o00oooooo
1Ol0o`05ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo
1Ooo0005ooooo`Goo`001Oooool5o`3o00Gooooo1Ol0o`0?ooooo`Go0?l01Oooool5o`3o00Kooooo
1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo
1Ol0o`06ooooo`Go0?l03_ooool000kooooo0ol0o`07ooooo`?o0?l02?ooool3o`3o00Oooooo0ooo
0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?o0?l02?ooool3ool000Oooooo0ol0
o`0@ooooo`?o0?l02?ooool3o`3o00Oooooo0ol0o`07ooooo`?oo`002?ooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3o`3o00Oooooo0ooo0007ooooo`?o0?l04Oooool3o`3o00Oooooo0ol0
o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000Oooooo0ooo
0007ooooo`?o0?l01oooool3o`3o017ooooo0ol0o`07ooooo`?o0?l02?ooool3ool000Oooooo0ooo
0007ooooo`?oo`002?ooool3ool000Oooooo0ooo0007ooooo`?oo`001oooool3o`3o00Sooooo0ol0
o`0?ooooo`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`00
1oooool3ool000Oooooo0ooo0008ooooo`?o0?l01oooool3ool000Oooooo0ol0o`08ooooo`?oo`00
1oooool3ool0013ooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l0
1oooool3o`3o00Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`00
1oooool3ool000Oooooo0ooo0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l0
1oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`00
2?ooool3ool000oooooo000=ooooo`Go0?l01Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ol0o`05ooooo`Goo`001Oooool5o`3o00Kooooo1Ooo0005ooooo`Goo`003_oo
ool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo1Ooo0005ooooo`Goo`001Ooo
ool5ool000Gooooo1Ooo000?ooooo`Go0?l01Oooool5o`3o00Kooooo1Ol0o`05ooooo`Goo`001Ooo
ool5ool000Kooooo1Ooo0005ooooo`Goo`001Oooool5ool000Gooooo1Ooo0006ooooo`Goo`003_oo
ool000cooooo1ol0o`03ooooo`Oo0?l01?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
o`3o00?ooooo1ooo0003ooooo`Oo0?l01?ooool7ool000?ooooo1ooo000<ooooo`Oo0?l01?ooool7
o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`003Oooool7o`3o00?ooooo1ol0o`03ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`001?ooool7ool000?ooooo1ooo0003ooooo`Ooo`000oooool7
ool000gooooo1ol0o`03ooooo`Oo0?l01?ooool7o`3o00?ooooo1ooo0003ooooo`Ooo`001?ooool7
ool000?ooooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo000=ooooo`003?ooool7o`3o
00?o00001ol0o`04o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Oo0?l00ol00007ool0
00?o00001ol0o`04o`0000Ooo`000ol00007ool000cooooo1ol0o`04o`0000Oo0?l00ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo000=ooooo`Oo0?l00ol00007o`3o00?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo0004o`0000Ooo`000ol00007ool000?o00001ooo0003o`0000Ooo`003Oooool7o`3o
00?o00001ol0o`04o`0000Oo0?l00ol00007ool000?o00001ooo0004o`0000Ooo`000ol00007ool0
00?o00001ooo0003o`0000Ooo`001?l00007ool000gooooo000<ooooo`Oo0?l00oooool7o`3o00Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ol0o`03ooooo`Ooo`000oooool7o`3o00Co
oooo1ooo0003ooooo`Ooo`003?ooool7o`3o00Cooooo1ol0o`03ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000go
oooo1ol0o`03ooooo`Oo0?l00oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000Co
oooo1ooo0003ooooo`Ooo`000oooool7ool000?ooooo1ooo000=ooooo`Oo0?l00oooool7o`3o00Co
oooo1ol0o`03ooooo`Ooo`000oooool7ool000Cooooo1ooo0003ooooo`Ooo`000oooool7ool000?o
oooo1ooo0004ooooo`Ooo`003Oooool000gooooo1Ol0o`05ooooo`Go0?l01_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001_ooool5o`3o00Gooooo1Ooo0005ooooo`Go0?l01_ooool5ool000Gooooo
1Ooo000>ooooo`Go0?l01_ooool5o`3o00Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`003oooool5o`3o00Gooooo
1Ol0o`05ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo
1Ooo0005ooooo`Goo`001Oooool5ool000oooooo1Ol0o`05ooooo`Go0?l01_ooool5o`3o00Gooooo
1Ooo0005ooooo`Goo`001_ooool5ool000Gooooo1Ooo0005ooooo`Goo`001Oooool5ool000Kooooo
1Ooo000>ooooo`003_ooool3o`3o00Oooooo0ol0o`08ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?o0?l01oooool3ool000Oooooo0ol0o`08ooooo`?oo`001oooool3ool0013ooooo0ol0
o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo000Aooooo`?o0?l01oooool3o`3o00Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo0008ooooo`?oo`001oooool3ool000Oooooo0ooo
0007ooooo`?oo`004Oooool3o`3o00Oooooo0ol0o`08ooooo`?o0?l01oooool3ool000Oooooo0ooo
0008ooooo`?oo`001oooool3ool000Oooooo0ooo0007ooooo`?oo`002?ooool3ool000oooooo003o
oooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003o
oooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003ooooooloooooo003o
oooooloooooo003ooooooloooooo0000\
\>"],
  ImageRangeCache->{{{0, 461}, {461, 0}} -> {-0.100021, -0.100021, \
0.00911073, 0.00911073}, {{10.9375, 113.062}, {450, 347.875}} -> {-0.167816, \
-0.168489, 0.0107712, 0.0107712}, {{123.25, 225.375}, {450, 347.875}} -> \
{-1.37756, -0.168489, 0.0107712, 0.0107712}, {{235.562, 337.688}, {450, \
347.875}} -> {-2.5873, -0.168489, 0.0107712, 0.0107712}, {{347.875, 450}, \
{450, 347.875}} -> {-3.79704, -0.168489, 0.0107712, 0.0107712}, {{10.9375, \
113.062}, {337.688, 235.562}} -> {-0.167816, -1.37823, 0.0107712, 0.0107712}, \
{{123.25, 225.375}, {337.688, 235.562}} -> {-1.37756, -1.37823, 0.0107712, \
0.0107712}, {{235.562, 337.688}, {337.688, 235.562}} -> {-2.5873, -1.37823, \
0.0107712, 0.0107712}, {{347.875, 450}, {337.688, 235.562}} -> {-3.79704, \
-1.37823, 0.0107712, 0.0107712}, {{10.9375, 113.062}, {225.375, 123.25}} -> \
{-0.167816, -2.58797, 0.0107712, 0.0107712}, {{123.25, 225.375}, {225.375, \
123.25}} -> {-1.37756, -2.58797, 0.0107712, 0.0107712}, {{235.562, 337.688}, \
{225.375, 123.25}} -> {-2.5873, -2.58797, 0.0107712, 0.0107712}, {{347.875, \
450}, {225.375, 123.25}} -> {-3.79704, -2.58797, 0.0107712, 0.0107712}, \
{{10.9375, 113.062}, {113.062, 10.9375}} -> {-0.167816, -3.79772, 0.0107712, \
0.0107712}, {{123.25, 225.375}, {113.062, 10.9375}} -> {-1.37756, -3.79772, \
0.0107712, 0.0107712}, {{235.562, 337.688}, {113.062, 10.9375}} -> {-2.5873, \
-3.79772, 0.0107712, 0.0107712}, {{347.875, 450}, {113.062, 10.9375}} -> \
{-3.79704, -3.79772, 0.0107712, 0.0107712}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Now let's delete this model and create another one.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[bos2]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Up to now I've been creating the graph for the interaction neighborhood \
ahead of time and storing it in a variable. There's no reason why one needs \
to do this: the ",
  StyleBox["InteractionNeighborhood", "Input"],
  " option expects to receive a ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  " ",
  StyleBox["Graph", "Input"],
  " object as its value. What this means is that ",
  StyleBox["any",
    FontSlant->"Italic"],
  " of the graph generating functions provided by ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  " can be used to generate graphs for the local interaction models... and \
these graphs can be passed directly to the options for ",
  StyleBox["LocalInteractionModel", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Here's ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  " played on the Petersen graph",
  ":"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(bos3 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] PetersenGraph, \[IndentingNewLine]PayoffMatrix \[Rule] 
            battleOfSexes, \[IndentingNewLine]StrategyColors \[Rule] 
            bosColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Note that we used R",
  StyleBox["andomStrategy[2]", "Input"],
  " to generate a random strategy. ",
  StyleBox["RandomStrategy[i]", "Input"],
  " is defined to be ",
  StyleBox["Table[Random[Integer,{1,i}]],", "Input"],
  " which is usually easier to read."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos3]];\)\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.52948 .72701 m
.25021 .3597 L
s
.52948 .72701 m
.52948 .27299 L
s
.52948 .72701 m
.60667 .95401 L
s
.25021 .6403 m
.52948 .27299 L
s
.25021 .6403 m
.70208 .5 L
s
.25021 .6403 m
.04812 .7806 L
s
.25021 .3597 m
.70208 .5 L
s
.25021 .3597 m
.04812 .2194 L
s
.52948 .27299 m
.60667 .04599 L
s
.70208 .5 m
.95188 .5 L
s
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
0 1 0 r
.025 w
.52948 .72701 Mdot
1 0 0 r
.25021 .6403 Mdot
.25021 .3597 Mdot
.52948 .27299 Mdot
.70208 .5 Mdot
.60667 .95401 Mdot
.04812 .7806 Mdot
0 1 0 r
.04812 .2194 Mdot
.60667 .04599 Mdot
1 0 0 r
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[22]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3o`3o073ooooo002/ooooo`Go0?l0Koooool00:_ooooo1ol0o`1^ooooo`00Zooo
ool7o`3o06kooooo002Yooooo`;o00001ol0o`1^ooooo`00Y_ooool3o`0000?ooooo1Ol0o`1_oooo
o`00X_ooool4o`0000Oooooo0ol0o`000ol0003oooooooooo`1]ooooo`00Woooool3o`0000_ooooo
00Go0000ooooooooooooooooo`00001^ooooo`00W?ooool3o`0000gooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`00013ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`1[ooooo`00UOooool4o`0001?ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`1Zooooo`00T_ooool3o`0001Kooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`1Yooooo`00Soooool3o`0001Wooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`1Yooooo`00Roooool4o`0001cooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`1Xooooo`00R?ooool3o`0001oooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`1Wooooo`00QOooool3o`0002;ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1V
ooooo`00P_ooool3o`0002Gooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`1Uoooo
o`00O_ooool4o`0002Oooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1Uooooo`00
Noooool3o`0002_ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`1Tooooo`00N?oo
ool3o`0002kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Sooooo`00M?ooool4
o`00033ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Rooooo`00LOooool3o`00
03Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Rooooo`00K_ooool3o`0003Ko
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`1Qooooo`00Joooool3o`0003Wooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`1Pooooo`00Ioooool4o`0003cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`1Oooooo`00I?ooool3o`0003oooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`1Oooooo`00HOooool3o`0004;ooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0004Gooooo00?o0000oooooooo
ool05oooool00ol0003oooooooooo`1Mooooo`00F_ooool3o`0004Sooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`1Looooo`00Eoooool3o`0004_ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`1Looooo`00E?ooool3o`0004kooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`1Kooooo`00D?ooool4o`00053ooooo00?o0000ooooooooool07?ooool00ol0
003oooooooooo`1Jooooo`00COooool3o`0005Cooooo00?o0000ooooooooool07Oooool00ol0003o
ooooooooo`1Iooooo`00B_ooool3o`0005Oooooo00?o0000ooooooooool07Oooool00ol0003ooooo
ooooo`1Iooooo`00A_ooool4o`0005Wooooo00?o0000ooooooooool07oooool00ol0003ooooooooo
o`1Hooooo`00@oooool3o`0005gooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`1G
ooooo`00@?ooool3o`0005oooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`1Foooo
o`00?Oooool3o`0006;ooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`1Eooooo`00
>Oooool4o`0006Gooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`1Eooooo`00=_oo
ool3o`0006Sooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1Dooooo`00<oooool3
o`0006_ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`1Cooooo`00;oooool4o`00
06kooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`1Booooo`00;?ooool3o`00077o
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`1Booooo`00:Oooool3o`0007Cooooo
00?o0000ooooooooool0:Oooool00ol0003oooooooooo`1Aooooo`009_ooool3o`0007Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`0007Wooooo00?o0000
ooooooooool0;?ooool00ol0003oooooooooo`1?ooooo`007oooool3o`0007gooooo00?o0000oooo
ooooool0;?ooool00ol0003oooooooooo`1?ooooo`007?ooool3o`00083ooooo00?o0000oooooooo
ool0;Oooool00ol0003oooooooooo`1>ooooo`006?ooool4o`0008;ooooo00?o0000ooooooooool0
;oooool00ol0003oooooooooo`1=ooooo`005Oooool3o`0008Kooooo00?o0000ooooooooool0<?oo
ool00ol0003oooooooooo`1<ooooo`003?ooool3o`3o00?ooooo0ol00029ooooo`03o`000?oooooo
oooo033ooooo00?o0000ooooooooool0C?ooool000_ooooo1Ol0o`02o`0008_ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`1;ooooo`002_ooool7o`3o08cooooo00?o0000oooooooo
ool0<oooool00ol0003oooooooooo`1:ooooo`002_ooool7o`3o08_ooooo00?o0000ooooooooool0
=Oooool00ol0003oooooooooo`19ooooo`002_ooool7o`3o0003o`000?oooooooooo08Sooooo00?o
0000ooooooooool0=Oooool00ol0003oooooooooo`19ooooo`002oooool5o`3o00;ooooo00?o0000
ooooooooool0Qoooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo04Sooooo000<oooo
o`?o0?l01?ooool00ol0003oooooooooo`25ooooo`03o`000?oooooooooo03Sooooo00?o0000oooo
ooooool0Aoooool000gooooo00?o0000ooooooooool01?ooool2o`0008Gooooo00?o0000oooooooo
ool0>Oooool00ol0003oooooooooo`16ooooo`003Oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo08;ooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`15ooooo`003Ooo
ool00ol0003oooooooooo`07ooooo`;o0000POooool00ol0003oooooooooo`0kooooo`03o`000?oo
oooooooo04Gooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0O_ooool0
0ol0003oooooooooo`0looooo`03o`000?oooooooooo04Cooooo000=ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool0OOooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo
04?ooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0001looooo`03o`000?oooooooooo03oo
oooo00?o0000ooooooooool0@_ooool000gooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`1iooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0@_ooool000gooooo00?o
0000ooooooooool03_ooool2o`0007Wooooo00?o0000ooooooooool0@?ooool00ol0003ooooooooo
o`11ooooo`003Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo07?ooooo0ooo0014
ooooo`03o`000?oooooooooo043ooooo000=ooooo`03o`000?oooooooooo017ooooo0_l0001boooo
o`Goo`00A?ooool00ol0003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo06kooooo1ooo0013ooooo`03o`000?oooooooooo03oooooo000=ooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool0KOooool7ool004Cooooo00?o0000ooooooooool0
?_ooool000gooooo00?o0000ooooooooool05Oooool2o`0006gooooo1ooo0015ooooo`03o`000?oo
oooooooo03gooooo000=ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0Joooool5
ool004Oooooo00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool06?ooool2o`00
06_ooooo00Co0000ool00?oo003oo`00B?ooool00ol0003oooooooooo`0looooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo06Oooooo00Co0000oooooooooooo0000B_ooool0
0ol0003oooooooooo`0kooooo`003Oooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo
06Kooooo00Co0000oooooooooooo0000Boooool00ol0003oooooooooo`0jooooo`003Oooool00ol0
003oooooooooo`0Looooo`;o0000IOooool01Ol0003ooooooooooooooooo000004cooooo00?o0000
ooooooooool0>Oooool000gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`1Qoooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0ioooo
o`003Oooool00ol0003oooooooooo`0Oooooo`;o0000H?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo04_ooooo00?o0000ooooooooool0>?ooool000gooooo00?o0000ooooooooool0
8Oooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
C?ooool00ol0003oooooooooo`0gooooo`003Oooool00ol0003oooooooooo`0Rooooo`03o`000?oo
oooooooo05_ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`1=ooooo`03o`000?oo
oooooooo03Kooooo000=ooooo`03o`000?oooooooooo02?ooooo0_l0001Jooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`0fooooo`003Oooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`1>ooooo`03o`000?oooooooooo03Gooooo000=ooooo`03o`000?oooooooooo02Ko
oooo0_l0001Eooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0Coooool00ol0003o
ooooooooo`0dooooo`003Oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo05;ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo03?ooooo
000=ooooo`03o`000?oooooooooo02Wooooo0_l0001Aooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool0DOooool00ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`0[
ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1A
ooooo`03o`000?oooooooooo03;ooooo000=ooooo`03o`000?oooooooooo02cooooo00?o0000oooo
ooooool0Boooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo05;ooooo00?o0000oooo
ooooool0<Oooool000gooooo00?o0000ooooooooool0;Oooool2o`0004_ooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo033ooooo000=ooooo`03o`00
0?oooooooooo02oooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo05Cooooo00?o0000ooooooooool0;oooool000gooooo00?o0000ooooooooool0<?oo
ool2o`0004Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1Dooooo`03o`000?oo
oooooooo02oooooo000=ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0@_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool0;_ooool0
00gooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`03ool003kooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo02gooooo000=ooooo`03
o`000?oooooooooo03Cooooo00?o0000ool00?oo00000ooo000looooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool0Eoooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003ooooo
ooooo`0dooooo`Ooo`00>_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo05Oooooo
00?o0000ooooooooool0;?ooool000gooooo00?o0000ooooooooool0=?ooool7ool003Wooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooooooo02_ooooo000=
ooooo`03o`000?oooooooooo03Cooooo1ooo0002o`0003Oooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`1Iooooo`03o`000?oooooooooo02[ooooo000=ooooo`03o`000?oooooooooo
03Gooooo1Ooo0003ooooo`?o0000<oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
05[ooooo00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0=_ooool3ool00003
o`000?oooooooooo00Cooooo0ol0000_ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool0F_ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003oooooooooo`0jooooo`03o`00
0?oooooooooo00Kooooo1?l0000Zooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
Foooool00ol0003oooooooooo`0Xooooo`003Oooool00ol0003oooooooooo`0jooooo`03o`000?oo
oooooooo00[ooooo0ol0000Wooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0G?oo
ool00ol0003oooooooooo`0Wooooo`003Oooool00ol0003oooooooooo`0kooooo`03o`000?oooooo
oooo00cooooo0ol0000Sooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0GOooool0
0ol0003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo
00kooooo0ol0000Oooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0GOooool00ol0
003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo013o
oooo1?l0000Jooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0G_ooool00ol0003o
ooooooooo`0Uooooo`003Oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo01Cooooo
0ol0000Fooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0Goooool00ol0003ooooo
ooooo`0Tooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo01Kooooo0ol0
000Cooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0H?ooool00ol0003ooooooooo
o`0Sooooo`003Oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo01Sooooo0ol0000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0HOooool00ol0003oooooooooo`0R
ooooo`003Oooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo01[ooooo1?l0000:oooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0HOooool00ol0003oooooooooo`0Roooo
o`003Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01gooooo0ol00006ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool0H_ooool00ol0003oooooooooo`0Qooooo`00
3Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo023ooooo0ol00003ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool0Hoooool00ol0003oooooooooo`0Pooooo`003Ooo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo02;ooooo0ol0000Mooooo`03o`000?oo
oooooooo06Cooooo00?o0000ooooooooool07oooool000gooooo00?o0000ooooooooool0@oooool0
0ol0003oooooooooo`0Rooooo`03o`000?oooooo000000?o00006Oooool00ol0003oooooooooo`1T
ooooo`03o`000?oooooooooo01oooooo000=ooooo`03o`000?oooooooooo04Cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`04ooooo`?o00005_ooool00ol0003oooooooooo`1Uoooo
o`03o`000?oooooooooo01kooooo000=ooooo`03o`000?oooooooooo04Cooooo00?o0000oooooooo
ool08?ooool00ol0003oooooooooo`07ooooo`?o00004oooool00ol0003oooooooooo`1Vooooo`03
o`000?oooooooooo01gooooo000=ooooo`03o`000?oooooooooo04Gooooo00?o0000ooooooooool0
7_ooool00ol0003oooooooooo`0;ooooo`?o00004?ooool00ol0003oooooooooo`1Wooooo`03o`00
0?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo04Kooooo00?o0000ooooooooool07?oo
ool00ol0003oooooooooo`0?ooooo`Co00003?ooool00ol0003oooooooooo`1Wooooo`03o`000?oo
oooooooo01cooooo000=ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Dooooo`?o00002Oooool00ol0003oooooooooo`1Xooooo`03o`000?oooooo
oooo01_ooooo000=ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Gooooo`?o00001_ooool00ol0003oooooooooo`1Yooooo`03o`000?oooooooooo
01[ooooo000=ooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Kooooo`?o00000oooool00ol0003oooooooooo`1Zooooo`03o`000?oooooooooo01Wo
oooo000=ooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0Oooooo`Co0000K?ooool00ol0003oooooooooo`0Iooooo`003Oooool00ol0003ooooooooo
o`1:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool08oooool4o`0006[ooooo00?o
0000ooooooooool06?ooool000gooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo02?ooooo00Co0000oooooooooooooooo0ol0001Xooooo`03o`000?oo
oooooooo01Oooooo000=ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00Cooooo0ol0001Vooooo`03o`000?oooooo
oooo01Kooooo000=ooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo00Oooooo1?l0001Rooooo`03o`000?oooooooooo
01Kooooo000=ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo00_ooooo0ol0001Pooooo`03o`000?oooooooooo01Go
oooo000=ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Wooooo`03o`000?oooooooooo00kooooo0ol0001Nooooo`03o`000?oooooooooo01Cooooo
000=ooooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`0Wooooo`03o`000?oooooooooo017ooooo0ol0001Looooo`03o`000?oooooooooo01?ooooo000=
ooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0X
ooooo`03o`000?oooooooooo01Cooooo1?l0001Iooooo`03o`000?oooooooooo01;ooooo000=oooo
o`03o`000?oooooooooo053ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Yoooo
o`03o`000?oooooooooo01Sooooo0ol0001Fooooo`03o`000?oooooooooo01;ooooo000=ooooo`03
o`000?oooooooooo057ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Zooooo`03
o`000?oooooooooo01_ooooo0ol0001Dooooo`03o`000?oooooooooo017ooooo000=ooooo`03o`00
0?oooooooooo057ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo01kooooo0ol0001Booooo`03o`000?oooooooooo013ooooo000=ooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0[ooooo`03o`000?oo
oooooooo027ooooo1?l0001?ooooo`03o`000?oooooooooo00oooooo000=ooooo`03o`000?oooooo
oooo05?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooo
oooo02Gooooo0ol00006ooooo`?oo`00@oooool00ol0003oooooool00002ool000gooooo000=oooo
o`03o`000?oooooooooo05Cooooo00Co0000oooooooooooo0000;oooool00ol0003oooooooooo`0X
ooooo`?o00000_ooool5ool004?ooooo1Ooo000<ooooo`003Oooool00ol0003oooooooooo`1Doooo
o`04o`000?ooooooooooo`0002oooooo00?o0000ooooooooool0:oooool00ol0003oo`00ool00005
ool0047ooooo1ooo000;ooooo`003Oooool00ol0003oooooooooo`1Eooooo`;o0000<?ooool00ol0
003oooooooooo`0/ooooo`Ooo`00@Ol00007ool000_ooooo000=ooooo`03o`000?oooooooooo05Go
oooo0_l0000`ooooo`03o`000?oooooooooo02[ooooo0_l00007ool0047ooooo1ooo000;ooooo`00
3Oooool00ol0003oooooooooo`1Dooooo`04o`000?ooooooooooo`0002oooooo00?o0000oooooooo
ool09oooool3o`0000?ooooo1Ooo0013ooooo`Goo`003?ooool000gooooo00?o0000ooooooooool0
E?ooool01?l0003oooooooooool0000_ooooo`03o`000?oooooooooo02Cooooo0ol00007ooooo`?o
o`00A?ooool01?l0003oo`00ool00?oo000=ooooo`003Oooool00ol0003oooooooooo`1Cooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Pooooo`Co
0000D?ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`1Booooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0Mooooo`?o0000
E?ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`1Aooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Jooooo`?o0000E_oo
ool00ol0003oooooooooo`0@ooooo`003Oooool00ol0003oooooooooo`1Aooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Gooooo`?o0000F?ooool0
0ol0003oooooooooo`0Aooooo`003Oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Cooooo`Co0000F_ooool00ol0
003oooooooooo`0Booooo`003Oooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo00[o
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0@ooooo`?o0000G_ooool00ol0003o
ooooooooo`0Booooo`003Oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool09oooool00ol0003oooooooooo`0=ooooo`?o0000H?ooool00ol0003ooooo
ooooo`0Cooooo`003Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool09oooool00ol0003oooooooooo`0:ooooo`?o0000H_ooool00ol0003ooooooooo
o`0Dooooo`003Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`06ooooo`Co0000I?ooool00ol0003oooooooooo`0E
ooooo`003Oooool00ol0003oooooooooo`1<ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`03ooooo`?o0000Ioooool00ol0003oooooooooo`0Foooo
o`003Oooool00ol0003oooooooooo`1;ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool09?ooool00ol0003oooooooooo`03o`0006[ooooo00?o0000ooooooooool05_ooool000gooooo
00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo02?ooooo
0ol0001/ooooo`03o`000?oooooooooo01Oooooo000=ooooo`03o`000?oooooooooo04[ooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Oooooo`Go0000KOooool00ol0003ooooooooo
o`0Hooooo`003Oooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo01Kooooo00?o0000
ooooooooool06oooool3o`0000Cooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`0I
ooooo`003Oooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05oooool3o`0000Oooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`0Ioooo
o`003Oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool04oooool3o`0000[ooooo00?o0000ooooooooool0JOooool00ol0003oooooooooo`0Jooooo`00
3Oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3oooool4o`0000gooooo00?o0000ooooooooool0J?ooool00ol0003oooooooooo`0Kooooo`003Ooo
ool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool02ooo
ool3o`00017ooooo00?o0000ooooooooool0Ioooool00ol0003oooooooooo`0Looooo`003Oooool0
0ol0003oooooooooo`15ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool01oooool3
o`0001Cooooo00?o0000ooooooooool0Ioooool00ol0003oooooooooo`0Looooo`003Oooool00ol0
003oooooooooo`14ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool00oooool3o`00
01Oooooo00?o0000ooooooooool0I_ooool00ol0003oooooooooo`0Mooooo`003Oooool00ol0003o
ooooooooo`14ooooo`03o`000?oooooooooo023ooooo00?o0000ooooool000000ol0000Jooooo`03
o`000?oooooooooo06Gooooo00?o0000ooooooooool07_ooool000gooooo00?o0000ooooooooool0
@oooool00ol0003oooooooooo`0Pooooo`?o00007_ooool00ol0003oooooooooo`1Tooooo`03o`00
0?oooooooooo01oooooo000=ooooo`03o`000?oooooooooo04;ooooo00?o0000ooooooooool07_oo
ool3o`0000?ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`1Tooooo`03o`000?oo
oooooooo01oooooo000=ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool07?ooool3
o`0000Oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`1Sooooo`03o`000?oooooo
oooo023ooooo000=ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool06?ooool4o`00
00[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`1Rooooo`03o`000?oooooooooo
027ooooo000=ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool05_ooool3o`0000oo
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo02;o
oooo000=ooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool05?ooool3o`0001?ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo02;ooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool04_ooool3o`0001Oooooo00?o
0000ooooooooool05oooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo02?ooooo000=
ooooo`03o`000?oooooooooo03gooooo00?o0000ooooooooool03oooool4o`0001[ooooo00?o0000
ooooooooool05oooool00ol0003oooooooooo`1Oooooo`03o`000?oooooooooo02Cooooo000=oooo
o`03o`000?oooooooooo03gooooo00?o0000ooooooooool03?ooool3o`0001oooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`1Nooooo`03o`000?oooooooooo02Gooooo000=ooooo`03
o`000?oooooooooo03cooooo00?o0000ooooooooool02_ooool3o`0002?ooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo02Kooooo000=ooooo`03o`00
0?oooooooooo03_ooooo00?o0000ooooooooool02?ooool3o`0002Oooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo02Kooooo000=ooooo`03o`000?oo
oooooooo03[ooooo00?o0000ooooooooool01Oooool4o`0002_ooooo00?o0000ooooooooool04ooo
ool00ol0003oooooooooo`1Looooo`03o`000?oooooooooo02Oooooo000=ooooo`03o`000?oooooo
oooo03Kooooo0ooo00000ooooooo0000ooooo`03ooooo`?o0000;oooool00ol0003oooooooooo`0C
ooooo`03o`000?oooooooooo05_ooooo00?o0000ooooooooool0:?ooool000gooooo00?o0000oooo
ooooool0=Oooool5ool000;ooooo0ol0000cooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool0F_ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003oooooooooo`0dooooo`Oo
o`0000?o0000ooooooooool0=Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo05[o
oooo00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0=?ooool7ool003Wooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo02[ooooo
000=ooooo`03o`000?oooooooooo03Cooooo1ooo000iooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool0F?ooool00ol0003oooooooooo`0[ooooo`003Oooool00ol0003oooooooooo`0c
ooooo`;o00001Ooo000kooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0Eoooool0
0ol0003oooooooooo`0/ooooo`003Oooool00ol0003oooooooooo`0booooo`04o`000?oooooooooo
ooooo`?oo`00?Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05Oooooo00?o0000
ooooooooool0;?ooool000gooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`13oooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0]oooo
o`003Oooool00ol0003oooooooooo`0_ooooo`;o0000A_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo05Gooooo00?o0000ooooooooool0;_ooool000gooooo00?o0000ooooooooool0
;_ooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0
E?ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0003oooooooooo`0/ooooo`;o0000Booo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo05Cooooo00?o0000ooooooooool0;ooo
ool000gooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1<ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`0`ooooo`003Oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`1Booooo`03o`000?oooooooooo037ooooo000=ooooo`03o`000?oooooooooo02So
oooo0_l0001Aooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0DOooool00ol0003o
ooooooooo`0booooo`003Oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo05;ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`1Aooooo`03o`000?oooooooooo03;ooooo
000=ooooo`03o`000?oooooooooo02Gooooo0_l0001Fooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0D?ooool00ol0003oooooooooo`0cooooo`003Oooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`1?
ooooo`03o`000?oooooooooo03Cooooo000=ooooo`03o`000?oooooooooo02;ooooo0_l0001Joooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0eoooo
o`003Oooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05_ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`00
0?oooooooooo023ooooo00?o0000ooooooooool0GOooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo04gooooo00?o0000ooooooooool0=_ooool000gooooo00?o0000ooooooooool07_oo
ool2o`00067ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1<ooooo`03o`000?oo
oooooooo03Oooooo000=ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0HOooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool0>?ooool0
00gooooo00?o0000ooooooooool06oooool2o`0006Gooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`1:ooooo`03o`000?oooooooooo03Wooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool0I_ooool01Ol0003ooooooooooooooooo000004cooooo00?o0000oooo
ooooool0>Oooool000gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1Xooooo`04
o`000?ooooooooooo`0004_ooooo00?o0000ooooooooool0>_ooool000gooooo00?o0000oooooooo
ool05oooool2o`0006_ooooo00?o0000ooooool0o`000_l0o`19ooooo`03o`000?oooooooooo03_o
oooo000=ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0K?ooool5o`3o04Oooooo
00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool05?ooool2o`0006kooooo1ol0
o`16ooooo`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo01?ooooo00?o0000
ooooooooool0K_ooool7o`3o04Gooooo00?o0000ooooooooool0?Oooool000gooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`1_ooooo`Oo0?l0A?ooool00ol0003oooooooooo`0noooo
o`003Oooool00ol0003oooooooooo`0@ooooo`;o0000Loooool5o`3o04Cooooo00?o0000oooooooo
ool0?oooool000gooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`1dooooo`?o0?l0
AOooool00ol0003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`0=ooooo`;o0000NOoo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo043ooooo000=ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool0N_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooo
oooo047ooooo000=ooooo`03o`000?oooooooooo00[ooooo0_l0001mooooo`03o`000?oooooooooo
03oooooo00?o0000ooooooooool0@_ooool000gooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`1mooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0@_ooool000gooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`1oooooo`03o`000?oooooooooo03gooooo
00?o0000ooooooooool0@oooool000gooooo00?o0000ooooooooool01_ooool2o`0008;ooooo00?o
0000ooooooooool0??ooool00ol0003oooooooooo`14ooooo`003Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo08;ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`15
ooooo`003Oooool00ol0003oooooooooo`03ooooo`;o0000Q_ooool00ol0003oooooooooo`0joooo
o`03o`000?oooooooooo04Gooooo000=ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool0Q_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo04Kooooo000<ooooo`?oo`00
0_ooool00ol0003oooooooooo`27ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0
Aoooool000_ooooo1Ooo00000ol0003oooooooooo`29ooooo`03o`000?oooooooooo03Kooooo00?o
0000ooooooooool0B?ooool000[ooooo1ooo002;ooooo`03o`000?oooooooooo03Gooooo00?o0000
ooooooooool0BOooool000[ooooo1ooo002;ooooo`03o`000?oooooooooo03Gooooo00?o0000oooo
ooooool0BOooool000[ooooo1ooo0002o`0008[ooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`1:ooooo`002oooool5ool000?ooooo0ol00027ooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0Boooool000cooooo0ooo0007ooooo`Co0000Q?ooool00ol0003ooooooooo
o`0`ooooo`03o`000?oooooooooo04cooooo000Jooooo`?o0000POooool00ol0003oooooooooo`0`
ooooo`03o`000?oooooooooo04cooooo000Mooooo`?o0000O_ooool00ol0003oooooooooo`0_oooo
o`03o`000?oooooooooo04gooooo000Pooooo`?o0000O?ooool00ol0003oooooooooo`0]ooooo`03
o`000?oooooooooo04kooooo000Sooooo`Co0000N?ooool00ol0003oooooooooo`0/ooooo`03o`00
0?oooooooooo04oooooo000Wooooo`?o0000MOooool00ol0003oooooooooo`0/ooooo`03o`000?oo
oooooooo04oooooo000Zooooo`?o0000Loooool00ol0003oooooooooo`0Zooooo`03o`000?oooooo
oooo053ooooo000]ooooo`Co0000Koooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo
057ooooo000aooooo`?o0000K?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo05;o
oooo000dooooo`?o0000J_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo05;ooooo
000gooooo`?o0000Ioooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo05?ooooo000j
ooooo`Co0000Hoooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05Cooooo000noooo
o`?o0000HOooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo05Gooooo0011ooooo`?o
0000G_ooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo05Gooooo0014ooooo`Co0000
Foooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05Kooooo0018ooooo`?o0000F?oo
ool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo05Oooooo001;ooooo`?o0000EOooool0
0ol0003oooooooooo`0Oooooo`03o`000?oooooooooo05Sooooo001>ooooo`?o0000Doooool00ol0
003oooooooooo`0Mooooo`03o`000?oooooooooo05Wooooo001Aooooo`Co0000Coooool00ol0003o
ooooooooo`0Mooooo`03o`000?oooooooooo05Wooooo001Eooooo`?o0000C?ooool00ol0003ooooo
ooooo`0Looooo`03o`000?oooooooooo05[ooooo001Hooooo`?o0000B_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo05_ooooo001Kooooo`Co0000A_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo05cooooo001Oooooo`?o0000@oooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo05cooooo001Rooooo`?o0000@Oooool00ol0003oooooooooo`0Gooooo`03
o`000?oooooooooo05gooooo001Uooooo`?o0000?_ooool00ol0003oooooooooo`0Fooooo`03o`00
0?oooooooooo05kooooo001Xooooo`Co0000>_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo05oooooo001/ooooo`?o0000>?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo05oooooo001_ooooo`?o0000=Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
063ooooo001booooo`Co0000<Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo067o
oooo001fooooo`?o0000;oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo06;ooooo
001iooooo`?o0000;?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo06;ooooo001l
ooooo`?o0000:_ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo06?ooooo001ooooo
o`Co00009_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo06Cooooo0023ooooo`?o
00008oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo06Gooooo0026ooooo`?o0000
8Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo06Gooooo0029ooooo`Co00007Ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Kooooo002=ooooo`?o00006_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo06Oooooo002@ooooo`?o00006?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo06Sooooo002Cooooo`?o00005Oooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo06Wooooo002Fooooo`Co00004Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo06Wooooo002Jooooo`?o00003oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo06[ooooo002Mooooo`?o00003?ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo06_ooooo002Pooooo`Co00002?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo06cooooo002Tooooo`?o00001_ooool3ool00003ooooool0003ooooo06go
oooo002Wooooo`?o00000_ooool5ool006oooooo002Zooooo`03o`000?oo003oo`0000Goo`00K_oo
ool00:_ooooo1ooo001^ooooo`00Zoooool7ool006kooooo002/ooooo`Goo`00Koooool00:gooooo
0ooo001`ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(Evolve[bos3, Generations \[Rule] 5]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos3]];\)\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.52948 .72701 m
.25021 .3597 L
s
.52948 .72701 m
.52948 .27299 L
s
.52948 .72701 m
.60667 .95401 L
s
.25021 .6403 m
.52948 .27299 L
s
.25021 .6403 m
.70208 .5 L
s
.25021 .6403 m
.04812 .7806 L
s
.25021 .3597 m
.70208 .5 L
s
.25021 .3597 m
.04812 .2194 L
s
.52948 .27299 m
.60667 .04599 L
s
.70208 .5 m
.95188 .5 L
s
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
1 0 0 r
.025 w
.52948 .72701 Mdot
.25021 .6403 Mdot
.25021 .3597 Mdot
.52948 .27299 Mdot
.70208 .5 Mdot
.60667 .95401 Mdot
.04812 .7806 Mdot
.04812 .2194 Mdot
.60667 .04599 Mdot
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[24]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3ool0073ooooo002/ooooo`Goo`00Koooool00:_ooooo1ooo001^ooooo`00Zooo
ool7ool006kooooo002Yooooo`;o00001ooo001^ooooo`00Y_ooool3o`0000?ooooo1Ooo001_oooo
o`00X_ooool4o`0000Oooooo0ooo00000ol0003oooooooooo`1]ooooo`00Woooool3o`0000_ooooo
00Go0000ooooooooooooooooo`00001^ooooo`00W?ooool3o`0000gooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`00013ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`1[ooooo`00UOooool4o`0001?ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`1Zooooo`00T_ooool3o`0001Kooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`1Yooooo`00Soooool3o`0001Wooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`1Yooooo`00Roooool4o`0001cooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`1Xooooo`00R?ooool3o`0001oooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`1Wooooo`00QOooool3o`0002;ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1V
ooooo`00P_ooool3o`0002Gooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`1Uoooo
o`00O_ooool4o`0002Oooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1Uooooo`00
Noooool3o`0002_ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`1Tooooo`00N?oo
ool3o`0002kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Sooooo`00M?ooool4
o`00033ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Rooooo`00LOooool3o`00
03Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Rooooo`00K_ooool3o`0003Ko
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`1Qooooo`00Joooool3o`0003Wooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`1Pooooo`00Ioooool4o`0003cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`1Oooooo`00I?ooool3o`0003oooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`1Oooooo`00HOooool3o`0004;ooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0004Gooooo00?o0000oooooooo
ool05oooool00ol0003oooooooooo`1Mooooo`00F_ooool3o`0004Sooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`1Looooo`00Eoooool3o`0004_ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`1Looooo`00E?ooool3o`0004kooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`1Kooooo`00D?ooool4o`00053ooooo00?o0000ooooooooool07?ooool00ol0
003oooooooooo`1Jooooo`00COooool3o`0005Cooooo00?o0000ooooooooool07Oooool00ol0003o
ooooooooo`1Iooooo`00B_ooool3o`0005Oooooo00?o0000ooooooooool07Oooool00ol0003ooooo
ooooo`1Iooooo`00A_ooool4o`0005Wooooo00?o0000ooooooooool07oooool00ol0003ooooooooo
o`1Hooooo`00@oooool3o`0005gooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`1G
ooooo`00@?ooool3o`0005oooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`1Foooo
o`00?Oooool3o`0006;ooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`1Eooooo`00
>Oooool4o`0006Gooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`1Eooooo`00=_oo
ool3o`0006Sooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1Dooooo`00<oooool3
o`0006_ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`1Cooooo`00;oooool4o`00
06kooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`1Booooo`00;?ooool3o`00077o
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`1Booooo`00:Oooool3o`0007Cooooo
00?o0000ooooooooool0:Oooool00ol0003oooooooooo`1Aooooo`009_ooool3o`0007Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`0007Wooooo00?o0000
ooooooooool0;?ooool00ol0003oooooooooo`1?ooooo`007oooool3o`0007gooooo00?o0000oooo
ooooool0;?ooool00ol0003oooooooooo`1?ooooo`007?ooool3o`00083ooooo00?o0000oooooooo
ool0;Oooool00ol0003oooooooooo`1>ooooo`006?ooool4o`0008;ooooo00?o0000ooooooooool0
;oooool00ol0003oooooooooo`1=ooooo`005Oooool3o`0008Kooooo00?o0000ooooooooool0<?oo
ool00ol0003oooooooooo`1<ooooo`003?ooool3ool000?ooooo0ol00029ooooo`03o`000?oooooo
oooo033ooooo00?o0000ooooooooool0C?ooool000_ooooo1Ooo0002o`0008_ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`1;ooooo`002_ooool7ool008cooooo00?o0000oooooooo
ool0<oooool00ol0003oooooooooo`1:ooooo`002_ooool7ool008_ooooo00?o0000ooooooooool0
=Oooool00ol0003oooooooooo`19ooooo`002_ooool7ool00003o`000?oooooooooo08Sooooo00?o
0000ooooooooool0=Oooool00ol0003oooooooooo`19ooooo`002oooool5ool000;ooooo00?o0000
ooooooooool0Qoooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo04Sooooo000<oooo
o`?oo`001?ooool00ol0003oooooooooo`25ooooo`03o`000?oooooooooo03Sooooo00?o0000oooo
ooooool0Aoooool000gooooo00?o0000ooooooooool01?ooool2o`0008Gooooo00?o0000oooooooo
ool0>Oooool00ol0003oooooooooo`16ooooo`003Oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo08;ooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`15ooooo`003Ooo
ool00ol0003oooooooooo`07ooooo`;o0000POooool00ol0003oooooooooo`0kooooo`03o`000?oo
oooooooo04Gooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0O_ooool0
0ol0003oooooooooo`0looooo`03o`000?oooooooooo04Cooooo000=ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool0OOooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo
04?ooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0001looooo`03o`000?oooooooooo03oo
oooo00?o0000ooooooooool0@_ooool000gooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`1iooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0@_ooool000gooooo00?o
0000ooooooooool03_ooool2o`0007Wooooo00?o0000ooooooooool0@?ooool00ol0003ooooooooo
o`11ooooo`003Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo07?ooooo0ooo0014
ooooo`03o`000?oooooooooo043ooooo000=ooooo`03o`000?oooooooooo017ooooo0_l0001boooo
o`Goo`00A?ooool00ol0003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo06kooooo1ooo0013ooooo`03o`000?oooooooooo03oooooo000=ooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool0KOooool7ool004Cooooo00?o0000ooooooooool0
?_ooool000gooooo00?o0000ooooooooool05Oooool2o`0006gooooo1ooo0015ooooo`03o`000?oo
oooooooo03gooooo000=ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0Joooool5
ool004Oooooo00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool06?ooool2o`00
06_ooooo00Co0000ool00?oo003oo`00B?ooool00ol0003oooooooooo`0looooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo06Oooooo00Co0000oooooooooooo0000B_ooool0
0ol0003oooooooooo`0kooooo`003Oooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo
06Kooooo00Co0000oooooooooooo0000Boooool00ol0003oooooooooo`0jooooo`003Oooool00ol0
003oooooooooo`0Looooo`;o0000IOooool01Ol0003ooooooooooooooooo000004cooooo00?o0000
ooooooooool0>Oooool000gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`1Qoooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0ioooo
o`003Oooool00ol0003oooooooooo`0Oooooo`;o0000H?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo04_ooooo00?o0000ooooooooool0>?ooool000gooooo00?o0000ooooooooool0
8Oooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
C?ooool00ol0003oooooooooo`0gooooo`003Oooool00ol0003oooooooooo`0Rooooo`03o`000?oo
oooooooo05_ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`1=ooooo`03o`000?oo
oooooooo03Kooooo000=ooooo`03o`000?oooooooooo02?ooooo0_l0001Jooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`0fooooo`003Oooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`1>ooooo`03o`000?oooooooooo03Gooooo000=ooooo`03o`000?oooooooooo02Ko
oooo0_l0001Eooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0Coooool00ol0003o
ooooooooo`0dooooo`003Oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo05;ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo03?ooooo
000=ooooo`03o`000?oooooooooo02Wooooo0_l0001Aooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool0DOooool00ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`0[
ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1A
ooooo`03o`000?oooooooooo03;ooooo000=ooooo`03o`000?oooooooooo02cooooo00?o0000oooo
ooooool0Boooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo05;ooooo00?o0000oooo
ooooool0<Oooool000gooooo00?o0000ooooooooool0;Oooool2o`0004_ooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo033ooooo000=ooooo`03o`00
0?oooooooooo02oooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo05Cooooo00?o0000ooooooooool0;oooool000gooooo00?o0000ooooooooool0<?oo
ool2o`0004Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1Dooooo`03o`000?oo
oooooooo02oooooo000=ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0@_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool0;_ooool0
00gooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`03ool003kooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo02gooooo000=ooooo`03
o`000?oooooooooo03Cooooo00?o0000ool00?oo00000ooo000looooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool0Eoooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003ooooo
ooooo`0dooooo`Ooo`00>_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo05Oooooo
00?o0000ooooooooool0;?ooool000gooooo00?o0000ooooooooool0=?ooool7ool003Wooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooooooo02_ooooo000=
ooooo`03o`000?oooooooooo03Cooooo1ooo0002o`0003Oooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`1Iooooo`03o`000?oooooooooo02[ooooo000=ooooo`03o`000?oooooooooo
03Gooooo1Ooo0003ooooo`?o0000<oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
05[ooooo00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0=_ooool3ool00003
o`000?oooooooooo00Cooooo0ol0000_ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool0F_ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003oooooooooo`0jooooo`03o`00
0?oooooooooo00Kooooo1?l0000Zooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
Foooool00ol0003oooooooooo`0Xooooo`003Oooool00ol0003oooooooooo`0jooooo`03o`000?oo
oooooooo00[ooooo0ol0000Wooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0G?oo
ool00ol0003oooooooooo`0Wooooo`003Oooool00ol0003oooooooooo`0kooooo`03o`000?oooooo
oooo00cooooo0ol0000Sooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0GOooool0
0ol0003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo
00kooooo0ol0000Oooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0GOooool00ol0
003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo013o
oooo1?l0000Jooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0G_ooool00ol0003o
ooooooooo`0Uooooo`003Oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo01Cooooo
0ol0000Fooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0Goooool00ol0003ooooo
ooooo`0Tooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo01Kooooo0ol0
000Cooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0H?ooool00ol0003ooooooooo
o`0Sooooo`003Oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo01Sooooo0ol0000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0HOooool00ol0003oooooooooo`0R
ooooo`003Oooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo01[ooooo1?l0000:oooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0HOooool00ol0003oooooooooo`0Roooo
o`003Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01gooooo0ol00006ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool0H_ooool00ol0003oooooooooo`0Qooooo`00
3Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo023ooooo0ol00003ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool0Hoooool00ol0003oooooooooo`0Pooooo`003Ooo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo02;ooooo0ol0000Mooooo`03o`000?oo
oooooooo06Cooooo00?o0000ooooooooool07oooool000gooooo00?o0000ooooooooool0@oooool0
0ol0003oooooooooo`0Rooooo`03o`000?oooooo000000?o00006Oooool00ol0003oooooooooo`1T
ooooo`03o`000?oooooooooo01oooooo000=ooooo`03o`000?oooooooooo04Cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`04ooooo`?o00005_ooool00ol0003oooooooooo`1Uoooo
o`03o`000?oooooooooo01kooooo000=ooooo`03o`000?oooooooooo04Cooooo00?o0000oooooooo
ool08?ooool00ol0003oooooooooo`07ooooo`?o00004oooool00ol0003oooooooooo`1Vooooo`03
o`000?oooooooooo01gooooo000=ooooo`03o`000?oooooooooo04Gooooo00?o0000ooooooooool0
7_ooool00ol0003oooooooooo`0;ooooo`?o00004?ooool00ol0003oooooooooo`1Wooooo`03o`00
0?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo04Kooooo00?o0000ooooooooool07?oo
ool00ol0003oooooooooo`0?ooooo`Co00003?ooool00ol0003oooooooooo`1Wooooo`03o`000?oo
oooooooo01cooooo000=ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Dooooo`?o00002Oooool00ol0003oooooooooo`1Xooooo`03o`000?oooooo
oooo01_ooooo000=ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Gooooo`?o00001_ooool00ol0003oooooooooo`1Yooooo`03o`000?oooooooooo
01[ooooo000=ooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Kooooo`?o00000oooool00ol0003oooooooooo`1Zooooo`03o`000?oooooooooo01Wo
oooo000=ooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0Oooooo`Co0000K?ooool00ol0003oooooooooo`0Iooooo`003Oooool00ol0003ooooooooo
o`1:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool08oooool4o`0006[ooooo00?o
0000ooooooooool06?ooool000gooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo02?ooooo00Co0000oooooooooooooooo0ol0001Xooooo`03o`000?oo
oooooooo01Oooooo000=ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00Cooooo0ol0001Vooooo`03o`000?oooooo
oooo01Kooooo000=ooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo00Oooooo1?l0001Rooooo`03o`000?oooooooooo
01Kooooo000=ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo00_ooooo0ol0001Pooooo`03o`000?oooooooooo01Go
oooo000=ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Wooooo`03o`000?oooooooooo00kooooo0ol0001Nooooo`03o`000?oooooooooo01Cooooo
000=ooooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`0Wooooo`03o`000?oooooooooo017ooooo0ol0001Looooo`03o`000?oooooooooo01?ooooo000=
ooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0X
ooooo`03o`000?oooooooooo01Cooooo1?l0001Iooooo`03o`000?oooooooooo01;ooooo000=oooo
o`03o`000?oooooooooo053ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Yoooo
o`03o`000?oooooooooo01Sooooo0ol0001Fooooo`03o`000?oooooooooo01;ooooo000=ooooo`03
o`000?oooooooooo057ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Zooooo`03
o`000?oooooooooo01_ooooo0ol0001Dooooo`03o`000?oooooooooo017ooooo000=ooooo`03o`00
0?oooooooooo057ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo01kooooo0ol0001Booooo`03o`000?oooooooooo013ooooo000=ooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0[ooooo`03o`000?oo
oooooooo027ooooo1?l0001?ooooo`03o`000?oooooooooo00oooooo000=ooooo`03o`000?oooooo
oooo05?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooo
oooo02Gooooo0ol00006ooooo`?oo`00@oooool00ol0003oooooool00002ool000gooooo000=oooo
o`03o`000?oooooooooo05Cooooo00Co0000oooooooooooo0000;oooool00ol0003oooooooooo`0X
ooooo`?o00000_ooool5ool004?ooooo1Ooo000<ooooo`003Oooool00ol0003oooooooooo`1Doooo
o`04o`000?ooooooooooo`0002oooooo00?o0000ooooooooool0:oooool00ol0003oo`00ool00005
ool0047ooooo1ooo000;ooooo`003Oooool00ol0003oooooooooo`1Eooooo`;o0000<?ooool00ol0
003oooooooooo`0/ooooo`Ooo`00@Ol00007ool000_ooooo000=ooooo`03o`000?oooooooooo05Go
oooo0_l0000`ooooo`03o`000?oooooooooo02[ooooo0_l00007ool0047ooooo1ooo000;ooooo`00
3Oooool00ol0003oooooooooo`1Dooooo`04o`000?ooooooooooo`0002oooooo00?o0000oooooooo
ool09oooool3o`0000?ooooo1Ooo0013ooooo`Goo`003?ooool000gooooo00?o0000ooooooooool0
E?ooool01?l0003oooooooooool0000_ooooo`03o`000?oooooooooo02Cooooo0ol00007ooooo`?o
o`00A?ooool01?l0003oo`00ool00?oo000=ooooo`003Oooool00ol0003oooooooooo`1Cooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Pooooo`Co
0000D?ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`1Booooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0Mooooo`?o0000
E?ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`1Aooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Jooooo`?o0000E_oo
ool00ol0003oooooooooo`0@ooooo`003Oooool00ol0003oooooooooo`1Aooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Gooooo`?o0000F?ooool0
0ol0003oooooooooo`0Aooooo`003Oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Cooooo`Co0000F_ooool00ol0
003oooooooooo`0Booooo`003Oooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo00[o
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0@ooooo`?o0000G_ooool00ol0003o
ooooooooo`0Booooo`003Oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool09oooool00ol0003oooooooooo`0=ooooo`?o0000H?ooool00ol0003ooooo
ooooo`0Cooooo`003Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool09oooool00ol0003oooooooooo`0:ooooo`?o0000H_ooool00ol0003ooooooooo
o`0Dooooo`003Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`06ooooo`Co0000I?ooool00ol0003oooooooooo`0E
ooooo`003Oooool00ol0003oooooooooo`1<ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`03ooooo`?o0000Ioooool00ol0003oooooooooo`0Foooo
o`003Oooool00ol0003oooooooooo`1;ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool09?ooool00ol0003oooooooooo`03o`0006[ooooo00?o0000ooooooooool05_ooool000gooooo
00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo02?ooooo
0ol0001/ooooo`03o`000?oooooooooo01Oooooo000=ooooo`03o`000?oooooooooo04[ooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Oooooo`Go0000KOooool00ol0003ooooooooo
o`0Hooooo`003Oooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo01Kooooo00?o0000
ooooooooool06oooool3o`0000Cooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`0I
ooooo`003Oooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05oooool3o`0000Oooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`0Ioooo
o`003Oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool04oooool3o`0000[ooooo00?o0000ooooooooool0JOooool00ol0003oooooooooo`0Jooooo`00
3Oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3oooool4o`0000gooooo00?o0000ooooooooool0J?ooool00ol0003oooooooooo`0Kooooo`003Ooo
ool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool02ooo
ool3o`00017ooooo00?o0000ooooooooool0Ioooool00ol0003oooooooooo`0Looooo`003Oooool0
0ol0003oooooooooo`15ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool01oooool3
o`0001Cooooo00?o0000ooooooooool0Ioooool00ol0003oooooooooo`0Looooo`003Oooool00ol0
003oooooooooo`14ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool00oooool3o`00
01Oooooo00?o0000ooooooooool0I_ooool00ol0003oooooooooo`0Mooooo`003Oooool00ol0003o
ooooooooo`14ooooo`03o`000?oooooooooo023ooooo00?o0000ooooool000000ol0000Jooooo`03
o`000?oooooooooo06Gooooo00?o0000ooooooooool07_ooool000gooooo00?o0000ooooooooool0
@oooool00ol0003oooooooooo`0Pooooo`?o00007_ooool00ol0003oooooooooo`1Tooooo`03o`00
0?oooooooooo01oooooo000=ooooo`03o`000?oooooooooo04;ooooo00?o0000ooooooooool07_oo
ool3o`0000?ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`1Tooooo`03o`000?oo
oooooooo01oooooo000=ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool07?ooool3
o`0000Oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`1Sooooo`03o`000?oooooo
oooo023ooooo000=ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool06?ooool4o`00
00[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`1Rooooo`03o`000?oooooooooo
027ooooo000=ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool05_ooool3o`0000oo
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo02;o
oooo000=ooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool05?ooool3o`0001?ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo02;ooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool04_ooool3o`0001Oooooo00?o
0000ooooooooool05oooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo02?ooooo000=
ooooo`03o`000?oooooooooo03gooooo00?o0000ooooooooool03oooool4o`0001[ooooo00?o0000
ooooooooool05oooool00ol0003oooooooooo`1Oooooo`03o`000?oooooooooo02Cooooo000=oooo
o`03o`000?oooooooooo03gooooo00?o0000ooooooooool03?ooool3o`0001oooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`1Nooooo`03o`000?oooooooooo02Gooooo000=ooooo`03
o`000?oooooooooo03cooooo00?o0000ooooooooool02_ooool3o`0002?ooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo02Kooooo000=ooooo`03o`00
0?oooooooooo03_ooooo00?o0000ooooooooool02?ooool3o`0002Oooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo02Kooooo000=ooooo`03o`000?oo
oooooooo03[ooooo00?o0000ooooooooool01Oooool4o`0002_ooooo00?o0000ooooooooool04ooo
ool00ol0003oooooooooo`1Looooo`03o`000?oooooooooo02Oooooo000=ooooo`03o`000?oooooo
oooo03Kooooo0ooo00000ooooooo0000ooooo`03ooooo`?o0000;oooool00ol0003oooooooooo`0C
ooooo`03o`000?oooooooooo05_ooooo00?o0000ooooooooool0:?ooool000gooooo00?o0000oooo
ooooool0=Oooool5ool000;ooooo0ol0000cooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool0F_ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003oooooooooo`0dooooo`Oo
o`0000?o0000ooooooooool0=Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo05[o
oooo00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0=?ooool7ool003Wooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo02[ooooo
000=ooooo`03o`000?oooooooooo03Cooooo1ooo000iooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool0F?ooool00ol0003oooooooooo`0[ooooo`003Oooool00ol0003oooooooooo`0c
ooooo`;o00001Ooo000kooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0Eoooool0
0ol0003oooooooooo`0/ooooo`003Oooool00ol0003oooooooooo`0booooo`04o`000?oooooooooo
ooooo`?oo`00?Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05Oooooo00?o0000
ooooooooool0;?ooool000gooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`13oooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0]oooo
o`003Oooool00ol0003oooooooooo`0_ooooo`;o0000A_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo05Gooooo00?o0000ooooooooool0;_ooool000gooooo00?o0000ooooooooool0
;_ooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0
E?ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0003oooooooooo`0/ooooo`;o0000Booo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo05Cooooo00?o0000ooooooooool0;ooo
ool000gooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1<ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`0`ooooo`003Oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`1Booooo`03o`000?oooooooooo037ooooo000=ooooo`03o`000?oooooooooo02So
oooo0_l0001Aooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0DOooool00ol0003o
ooooooooo`0booooo`003Oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo05;ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`1Aooooo`03o`000?oooooooooo03;ooooo
000=ooooo`03o`000?oooooooooo02Gooooo0_l0001Fooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0D?ooool00ol0003oooooooooo`0cooooo`003Oooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`1?
ooooo`03o`000?oooooooooo03Cooooo000=ooooo`03o`000?oooooooooo02;ooooo0_l0001Joooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0eoooo
o`003Oooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05_ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`00
0?oooooooooo023ooooo00?o0000ooooooooool0GOooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo04gooooo00?o0000ooooooooool0=_ooool000gooooo00?o0000ooooooooool07_oo
ool2o`00067ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1<ooooo`03o`000?oo
oooooooo03Oooooo000=ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0HOooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool0>?ooool0
00gooooo00?o0000ooooooooool06oooool2o`0006Gooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`1:ooooo`03o`000?oooooooooo03Wooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool0I_ooool01Ol0003ooooooooooooooooo000004cooooo00?o0000oooo
ooooool0>Oooool000gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1Xooooo`04
o`000?ooooooooooo`0004_ooooo00?o0000ooooooooool0>_ooool000gooooo00?o0000oooooooo
ool05oooool2o`0006_ooooo00?o0000oooooooo00000_oo0019ooooo`03o`000?oooooooooo03_o
oooo000=ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0K?ooool5ool004Oooooo
00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool05?ooool2o`0006kooooo1ooo
0016ooooo`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo01?ooooo00?o0000
ooooooooool0K_ooool7ool004Gooooo00?o0000ooooooooool0?Oooool000gooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`1_ooooo`Ooo`00A?ooool00ol0003oooooooooo`0noooo
o`003Oooool00ol0003oooooooooo`0@ooooo`;o0000Loooool5ool004Cooooo00?o0000oooooooo
ool0?oooool000gooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`1dooooo`?oo`00
AOooool00ol0003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`0=ooooo`;o0000NOoo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo043ooooo000=ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool0N_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooo
oooo047ooooo000=ooooo`03o`000?oooooooooo00[ooooo0_l0001mooooo`03o`000?oooooooooo
03oooooo00?o0000ooooooooool0@_ooool000gooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`1mooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0@_ooool000gooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`1oooooo`03o`000?oooooooooo03gooooo
00?o0000ooooooooool0@oooool000gooooo00?o0000ooooooooool01_ooool2o`0008;ooooo00?o
0000ooooooooool0??ooool00ol0003oooooooooo`14ooooo`003Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo08;ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`15
ooooo`003Oooool00ol0003oooooooooo`03ooooo`;o0000Q_ooool00ol0003oooooooooo`0joooo
o`03o`000?oooooooooo04Gooooo000=ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool0Q_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo04Kooooo000<ooooo`?oo`00
0_ooool00ol0003oooooooooo`27ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0
Aoooool000_ooooo1Ooo00000ol0003oooooooooo`29ooooo`03o`000?oooooooooo03Kooooo00?o
0000ooooooooool0B?ooool000[ooooo1ooo002;ooooo`03o`000?oooooooooo03Gooooo00?o0000
ooooooooool0BOooool000[ooooo1ooo002;ooooo`03o`000?oooooooooo03Gooooo00?o0000oooo
ooooool0BOooool000[ooooo1ooo0002o`0008[ooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`1:ooooo`002oooool5ool000?ooooo0ol00027ooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0Boooool000cooooo0ooo0007ooooo`Co0000Q?ooool00ol0003ooooooooo
o`0`ooooo`03o`000?oooooooooo04cooooo000Jooooo`?o0000POooool00ol0003oooooooooo`0`
ooooo`03o`000?oooooooooo04cooooo000Mooooo`?o0000O_ooool00ol0003oooooooooo`0_oooo
o`03o`000?oooooooooo04gooooo000Pooooo`?o0000O?ooool00ol0003oooooooooo`0]ooooo`03
o`000?oooooooooo04kooooo000Sooooo`Co0000N?ooool00ol0003oooooooooo`0/ooooo`03o`00
0?oooooooooo04oooooo000Wooooo`?o0000MOooool00ol0003oooooooooo`0/ooooo`03o`000?oo
oooooooo04oooooo000Zooooo`?o0000Loooool00ol0003oooooooooo`0Zooooo`03o`000?oooooo
oooo053ooooo000]ooooo`Co0000Koooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo
057ooooo000aooooo`?o0000K?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo05;o
oooo000dooooo`?o0000J_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo05;ooooo
000gooooo`?o0000Ioooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo05?ooooo000j
ooooo`Co0000Hoooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05Cooooo000noooo
o`?o0000HOooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo05Gooooo0011ooooo`?o
0000G_ooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo05Gooooo0014ooooo`Co0000
Foooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05Kooooo0018ooooo`?o0000F?oo
ool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo05Oooooo001;ooooo`?o0000EOooool0
0ol0003oooooooooo`0Oooooo`03o`000?oooooooooo05Sooooo001>ooooo`?o0000Doooool00ol0
003oooooooooo`0Mooooo`03o`000?oooooooooo05Wooooo001Aooooo`Co0000Coooool00ol0003o
ooooooooo`0Mooooo`03o`000?oooooooooo05Wooooo001Eooooo`?o0000C?ooool00ol0003ooooo
ooooo`0Looooo`03o`000?oooooooooo05[ooooo001Hooooo`?o0000B_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo05_ooooo001Kooooo`Co0000A_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo05cooooo001Oooooo`?o0000@oooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo05cooooo001Rooooo`?o0000@Oooool00ol0003oooooooooo`0Gooooo`03
o`000?oooooooooo05gooooo001Uooooo`?o0000?_ooool00ol0003oooooooooo`0Fooooo`03o`00
0?oooooooooo05kooooo001Xooooo`Co0000>_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo05oooooo001/ooooo`?o0000>?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo05oooooo001_ooooo`?o0000=Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
063ooooo001booooo`Co0000<Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo067o
oooo001fooooo`?o0000;oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo06;ooooo
001iooooo`?o0000;?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo06;ooooo001l
ooooo`?o0000:_ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo06?ooooo001ooooo
o`Co00009_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo06Cooooo0023ooooo`?o
00008oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo06Gooooo0026ooooo`?o0000
8Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo06Gooooo0029ooooo`Co00007Ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Kooooo002=ooooo`?o00006_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo06Oooooo002@ooooo`?o00006?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo06Sooooo002Cooooo`?o00005Oooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo06Wooooo002Fooooo`Co00004Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo06Wooooo002Jooooo`?o00003oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo06[ooooo002Mooooo`?o00003?ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo06_ooooo002Pooooo`Co00002?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo06cooooo002Tooooo`?o00001_ooool3ool00003ooooool0003ooooo06go
oooo002Wooooo`?o00000_ooool5ool006oooooo002Zooooo`03o`000?oo003oo`0000Goo`00K_oo
ool00:_ooooo1ooo001^ooooo`00Zoooool7ool006kooooo002/ooooo`Goo`00Koooool00:gooooo
0ooo001`ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[bos3]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Here's ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  " on a random tree of 250 vertices."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g3 = RandomTree[50];\)\), "\[IndentingNewLine]", 
    \(\(ShowLabeledGraph[g3];\)\)}], "Input",
  CellLabel->"In[26]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.199448 1.14233 0.0471852 0.943705 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.28477 .44721 m
.28477 .5324 L
s
.28477 .44721 m
.28477 .36201 L
s
.28477 .5324 m
.35769 .59264 L
s
.28477 .5324 m
.21185 .59264 L
s
.36369 .2046 m
.40316 .12589 L
s
.36369 .2046 m
.32423 .28331 L
s
.84372 .73335 m
.94288 .75674 L
s
.84372 .73335 m
.74457 .70996 L
s
.49939 .81764 m
.43661 .75005 L
s
.49939 .81764 m
.56217 .88523 L
s
.44262 .04719 m
.40316 .12589 L
s
.47608 .82876 m
.43661 .75005 L
s
.47608 .82876 m
.51554 .90747 L
s
.64588 .68523 m
.74457 .70996 L
s
.64588 .68523 m
.54823 .65784 L
s
.64588 .68523 m
.7424 .71523 L
s
.5552 .63636 m
.45296 .62524 L
s
.28477 .36201 m
.18949 .32941 L
s
.28477 .36201 m
.2453 .28331 L
s
.28477 .36201 m
.32423 .28331 L
s
.28477 .36201 m
.38004 .32941 L
s
.88098 .23583 m
.77984 .25245 L
s
.35769 .59264 m
.45296 .62524 L
s
.35769 .59264 m
.39715 .67135 L
s
.45296 .62524 m
.54823 .65784 L
s
.45296 .62524 m
.53477 .6771 L
s
.18516 .67493 m
.21185 .59264 L
s
.43661 .75005 m
.45007 .83452 L
s
.43661 .75005 m
.39715 .67135 L
s
.64072 .69552 m
.54823 .65784 L
s
.74457 .70996 m
.84274 .73602 L
s
.07615 .72047 m
.13893 .65288 L
s
.2453 .28331 m
.20584 .2046 L
s
.6787 .26907 m
.57755 .28569 L
s
.6787 .26907 m
.77984 .25245 L
s
.56217 .88523 m
.62494 .95281 L
s
.47531 .29681 m
.55713 .24495 L
s
.47531 .29681 m
.57059 .26421 L
s
.47531 .29681 m
.57755 .28569 L
s
.47531 .29681 m
.38004 .32941 L
s
.14855 .13377 m
.20584 .2046 L
s
.14855 .13377 m
.09126 .06293 L
s
.13893 .65288 m
.21185 .59264 L
s
.13893 .65288 m
.05712 .70474 L
s
.55713 .24495 m
.63894 .19309 L
s
.20584 .2046 m
.18572 .12104 L
s
.68046 .28012 m
.57755 .28569 L
s
.11224 .61469 m
.21185 .59264 L
s
.63894 .19309 m
.72075 .14123 L
s
.025 w
.28477 .44721 Mdot
[(1)] .26477 .42721 1 0 Mshowa
.28477 .5324 Mdot
[(2)] .26477 .5124 1 0 Mshowa
.36369 .2046 Mdot
[(3)] .34369 .1846 1 0 Mshowa
.84372 .73335 Mdot
[(4)] .82372 .71335 1 0 Mshowa
.49939 .81764 Mdot
[(5)] .47939 .79764 1 0 Mshowa
.44262 .04719 Mdot
[(6)] .42262 .02719 1 0 Mshowa
.47608 .82876 Mdot
[(7)] .45608 .80876 1 0 Mshowa
.64588 .68523 Mdot
[(8)] .62588 .66523 1 0 Mshowa
.5552 .63636 Mdot
[(9)] .5352 .61636 1 0 Mshowa
.28477 .36201 Mdot
[(10)] .26477 .34201 1 0 Mshowa
.88098 .23583 Mdot
[(11)] .86098 .21583 1 0 Mshowa
.35769 .59264 Mdot
[(12)] .33769 .57264 1 0 Mshowa
.45296 .62524 Mdot
[(13)] .43296 .60524 1 0 Mshowa
.18516 .67493 Mdot
[(14)] .16516 .65493 1 0 Mshowa
.43661 .75005 Mdot
[(15)] .41661 .73005 1 0 Mshowa
.64072 .69552 Mdot
[(16)] .62072 .67552 1 0 Mshowa
.45007 .83452 Mdot
[(17)] .43007 .81452 1 0 Mshowa
.94288 .75674 Mdot
[(18)] .92288 .73674 1 0 Mshowa
.74457 .70996 Mdot
[(19)] .72457 .68996 1 0 Mshowa
.18949 .32941 Mdot
[(20)] .16949 .30941 1 0 Mshowa
.40316 .12589 Mdot
[(21)] .38316 .10589 1 0 Mshowa
.07615 .72047 Mdot
[(22)] .05615 .70047 1 0 Mshowa
.2453 .28331 Mdot
[(23)] .2253 .26331 1 0 Mshowa
.6787 .26907 Mdot
[(24)] .6587 .24907 1 0 Mshowa
.39715 .67135 Mdot
[(25)] .37715 .65135 1 0 Mshowa
.56217 .88523 Mdot
[(26)] .54217 .86523 1 0 Mshowa
.47531 .29681 Mdot
[(27)] .45531 .27681 1 0 Mshowa
.14855 .13377 Mdot
[(28)] .12855 .11377 1 0 Mshowa
.13893 .65288 Mdot
[(29)] .11893 .63288 1 0 Mshowa
.55713 .24495 Mdot
[(30)] .53713 .22495 1 0 Mshowa
.20584 .2046 Mdot
[(31)] .18584 .1846 1 0 Mshowa
.68046 .28012 Mdot
[(32)] .66046 .26012 1 0 Mshowa
.84274 .73602 Mdot
[(33)] .82274 .71602 1 0 Mshowa
.54823 .65784 Mdot
[(34)] .52823 .63784 1 0 Mshowa
.11224 .61469 Mdot
[(35)] .09224 .59469 1 0 Mshowa
.21185 .59264 Mdot
[(36)] .19185 .57264 1 0 Mshowa
.57059 .26421 Mdot
[(37)] .55059 .24421 1 0 Mshowa
.7424 .71523 Mdot
[(38)] .7224 .69523 1 0 Mshowa
.63894 .19309 Mdot
[(39)] .61894 .17309 1 0 Mshowa
.18572 .12104 Mdot
[(40)] .16572 .10104 1 0 Mshowa
.51554 .90747 Mdot
[(41)] .49554 .88747 1 0 Mshowa
.32423 .28331 Mdot
[(42)] .30423 .26331 1 0 Mshowa
.57755 .28569 Mdot
[(43)] .55755 .26569 1 0 Mshowa
.09126 .06293 Mdot
[(44)] .07126 .04293 1 0 Mshowa
.05712 .70474 Mdot
[(45)] .03712 .68474 1 0 Mshowa
.77984 .25245 Mdot
[(46)] .75984 .23245 1 0 Mshowa
.72075 .14123 Mdot
[(47)] .70075 .12123 1 0 Mshowa
.62494 .95281 Mdot
[(48)] .60494 .93281 1 0 Mshowa
.38004 .32941 Mdot
[(49)] .36004 .30941 1 0 Mshowa
.53477 .6771 Mdot
[(50)] .51477 .6571 1 0 Mshowa
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[26]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00M?ooool3o`000:Wooooo001cooooo`05o`000?ooooooooooooooool00000Z?ooool007?o
oooo00Go0000ooooooooooooooooo`00002Xooooo`00Loooool4o`000:Wooooo000:ooooo`?o0000
0oooool3o`00063ooooo00?o0000ooooooooool0Z_ooool000_ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo00Kooooo0ol0002Pooooo`002?ooool5
o`000003ooooool0003o000000?o0000H_ooool2o`0000Gooooo1Ol0002Oooooo`002?ooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0006Wooooo1ol0002Nooooo`002Oooool0
0ol0003oooooo`000003ooooo`03o`000?oooooo000006Wooooo1ol0002Nooooo`002_ooool2o`00
00Cooooo0_l0001Yooooo`Oo0000W_ooool000_ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`05ooooo`?o0000H?ooool5o`0009oooooo000Hooooo`Go0000H?ooool3o`000:3ooooo
000Gooooo`Oo0000Goooool00ol0003oooooooooo`2Pooooo`005oooool7o`0005oooooo00?o0000
ooooooooool0X?ooool001Oooooo1ol0001Nooooo`03o`000?oooooooooo0:7ooooo000Hooooo`Go
0000Goooool00ol0003oooooooooo`2Qooooo`006Oooool4o`0005kooooo00?o0000ooooooooool0
X_ooool001gooooo00?o0000ooooooooool0Foooool00ol0003oooooooooo`2Rooooo`007_ooool0
0ol0003oooooooooo`1Iooooo`03o`000?oooooooooo0:?ooooo000Nooooo`03o`000?oooooooooo
05Wooooo00?o0000ooooooooool0Xoooool001oooooo00?o0000ooooooooool0Eoooool00ol0003o
ooooooooo`2Tooooo`008?ooool00ol0003oooooooooo`02ooooo`?o00000oooool2o`0004_ooooo
00?o0000ooooooooool0YOooool0027ooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooo
oooooooo000000;ooooo00?o0000ooooooooool0<Oooool5o`0000;ooooo0ol0000=ooooo`03o`00
0?oooooooooo0:Gooooo000Rooooo`Ko00000_ooool01?l0003oooooooooool0000dooooo`04o`00
0?ooooooooooo`0000?ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`2Vooooo`00
8_ooool2o`0000;ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo03?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo0:Ko
oooo000Hooooo`Go00000_ooool3o`000005ooooool0003o0000ooooool000000oooool01?l0003o
ooooooooool0000fooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`2Wooooo`006Oooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000
ooooool000000_l00003ooooo`04o`000?ooooooooooo`0000Kooooo0ol0000Zooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo0:Oooooo
000Jooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`02o`0000Cooooo
0_l00006ooooo`Go0000:Oooool01Ol0003ooooooooooooooooo000000;ooooo0_l0000;ooooo`03
o`000?oooooooooo04Oooooo0ol00003ooooo`03o`000?oooooooooo05Sooooo000Kooooo`04o`00
0?ooooooooooooooo`?o00001?ooool00ol0003oooooooooo`09ooooo`Oo0000:Oooool3o`0000Co
oooo00?o0000ooooooooool01_ooool4o`0004[ooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`1Hooooo`006?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool01Ol0
003ooooooooooooooooo000000_ooooo1ol0000hooooo`Go0000Aoooool5o`0000Cooooo00?o0000
ooooooooool0Eoooool001Sooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000002o`0000Kooooo1ol0000gooooo`Oo0000A_oo
ool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo05Oooooo000Iooooo`?o00000ooo
ool3o`0000Kooooo1Ol00006ooooo`Go0000>?ooool7o`0004Oooooo00?o0000ooooool000001_oo
ool00ol0003oooooooooo`1Fooooo`009oooool7o`0000Kooooo0ol0000iooooo`Oo0000B?ooool2
o`0000;ooooo00Go0000ooooooooooooooooo`000005ooooo`?o0000D?ooool002Oooooo1ol00008
ooooo`03o`000?oooooooooo03Sooooo1Ol0001:ooooo`03o`000?oooooooooo00Go00001?ooool5
o`0004oooooo000Wooooo`Oo00002?ooool00ol0003oooooooooo`0iooooo`?o0000E_ooool7o`00
04kooooo000Xooooo`Go00002Oooool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo05Oo
oooo1ol0001>ooooo`00:Oooool4o`0000[ooooo00?o0000ooooooooool0=oooool00ol0003ooooo
ooooo`1Gooooo`Oo0000C_ooool002gooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0fooooo`03o`000?oooooooooo05Oooooo1ol0001?ooooo`00;_ooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0E_ooool01?l0003ooooooooooooo
ool3o`00053ooooo000_ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0=Oooool0
0ol0003oooooooooo`1Eooooo`;o0000Eoooool0033ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0dooooo`03o`000?oooooooooo05Cooooo00?o0000ooooooooool0Eoooool0033o
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo05?o
oooo0_l0001Jooooo`00<Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0=Oooool3o`0000?ooooo0_l0000Dooooo`;o0000G?ooool003;ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03Gooooo00Go
0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
G?ooool003?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0aooooo`03o`000?oo
oooooooo03Wooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0=ooooo`;o0000Gooo
ool002[ooooo0ol00003ooooo`?o000000?oooooo`000?ooool00oooool00ol0003oooooooooo`0Q
ooooo`?o00003?ooool00ol0003oooooooooo`0hooooo`;o00000oooool4o`0000kooooo00?o0000
ooooooooool0Goooool002Wooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`0Pooooo`05o`000?ooooooooooooooool00000
2oooool00ol0003oooooooooo`0jooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool0
2_ooool2o`0006;ooooo000]ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooo
oooooooooooo000002Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0gooooo`07
o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oooooooooo00Cooooo0ol00002
ooooo`03o`000?oooooooooo06;ooooo000[ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`05o`000?ooooooooooooooool000008oooool2o`0000_ooooo00?o0000ooooooooool0>?ooool3
o`0000?ooooo0ol00006ooooo`Ko0000IOooool002gooooo00Go0000ooooooooooooooooo`000005
ooooo`04o`000?ooooooooooo`0002Gooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`17ooooo`Oo0000IOooool002Wooooo00Go0000ooooooooooooooooo`000002ooooo`;o00001_oo
ool00ol0003oooooo`000002o`0001oooooo00Go0000ooooooooooooooooo`000006ooooo`Co0000
BOooool7o`0006Gooooo000Zooooo`?o00001?ooool00ol0003oooooooooo`05ooooo`Go00007ooo
ool3o`0000Kooooo1Ol00019ooooo`Oo0000IOooool003Sooooo1ol0000Vooooo`Oo0000Aoooool7
o`0006Kooooo000hooooo`Oo00009_ooool7o`0004Gooooo0_l00003ooooo`?o0000Ioooool003So
oooo1ol0000Vooooo`Oo0000A?ooool00ol0003oooooooooo`0iooooo`?o00000oooool3o`0002_o
oooo000iooooo`Go0000:?ooool5o`0004?ooooo0_l0000mooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool0:_ooool003[ooooo0ol0000Zooooo`?o0000@oooool00ol0003ooooooooo
o`0mooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0:_ooool003gooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`?o00001?ooool2o`0001?ooooo0_l00010
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0:_ooool003gooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Uooooo`05o`000?ooooooooooooooool000000_ooool0
1?l0003oooooooooool0000@ooooo`;o0000@_ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo02[ooooo000nooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0:_ooool0
1?l0003oooooooooool00002ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool09Ooo
ool3o`0000;ooooo0ol0000Dooooo`;o00001?ooool2o`0000Oooooo0ol0000Rooooo`00?_ooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo02Sooooo0_l00003ooooo`04o`000?oooooo
ooooo`0000gooooo0_l0000Yooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo1Ol0000Qooooo`00?ooo
ool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo02_ooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo02Kooooo1Ol000000ooooooo0000
ooooo`02ooooo`03o`000?oooooooooo01kooooo1ol0000Pooooo`00?oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo02Oooooo00Co0000oooooooooooooooo1?l00002ooooo`03o`00
0?oooooo000000?ooooo0ol00002ooooo`;o0000:Oooool01?l0003oooooooooool00002ooooo`Co
00008Oooool7o`00023ooooo0010ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0
:Oooool4o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo00Ko00003_ooool5o`0000?ooooo
0ol0000Cooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool07?ooool=o`00023ooooo
0010ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0<?ooool00ol0003ooooooooo
o`02ooooo`So00003oooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo01?ooooo
0_l00003ooooo`03o`000?oooooooooo00Kooooo0ol0000<ooooo`Ko00001oooool5o`00027ooooo
0011ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0;oooool2o`0000Gooooo2?l0
000@ooooo`04o`000?ooooooooooooooo`Go00005Oooool00ol0003oooooooooo`02ooooo`;o0000
1Oooool5o`0000Kooooo1Ol0000>ooooo`?o00008_ooool0047ooooo00?o0000ooooooooool07_oo
ool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo00?ooooo1ol0000Aooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool07oooool<o`0003Sooooo0011ooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool0;Oooool01ol0003ooooooooooooooooo0000ooooool0
00000_ooool:o`0000cooooo1Ol000000ooooooo0000o`000003o`00023ooooo1ol0000mooooo`00
=?ooool5o`0000;ooooo0ol00004ooooo`03o`000?oooooooooo00Sooooo0ol000000ooooooo0000
o`000003o`0000_ooooo00?o0000ooooooooool0;ooooolCo`0000_ooooo00Go0000oooooooooooo
ooooo`000002ooooo`Co00006_ooool=o`0003gooooo000eooooo`06o`000?ooooooooooo`000?oo
oooo00000oooool01Ol0003ooooooooooooooooo000000_ooooo00Go0000ooooooooooooooooo`00
0002ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0<oooool4o`0000?ooooo00?o
0000ooooool000001_l0000;ooooo`Co00000oooool2o`0000Kooooo0ol0000<ooooo`Ko00001ooo
ool5o`0003kooooo000fooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`05ooooo`Go00000oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo037ooooo1Ol00005ooooo`03o`000?oooooo000000Ko00003?ooool2o`0000Gooooo00?o0000
ooooooooool00oooool5o`0000Kooooo1Ol0000>ooooo`?o0000?oooool003Oooooo00?o0000oooo
ooooool00_ooool2o`0000Gooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00005
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool5o`0000?ooooo00?o0000
ooooooooool04oooool4o`0000Cooooo0_l000000ooooooo0000o`000006o`0000kooooo00?o0000
ooooool000000oooool01?l0003oooooooooooooool<o`0005Gooooo000dooooo`05o`000?oooooo
ooooooooool000001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooool000000_ooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooooooool0
5Oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo01;ooooo0ol000000ooooooo
0000ooooo`03ooooo`?o00000_ooool5o`0000_ooooo00Oo0000ooooooooooooooooo`000?oooooo
000000?ooooo00?o0000ooooooooool02?l0001Jooooo`00=?ooool01ol0003ooooooooooooooooo
0000ooooool000000oooool00ol0003oooooooooo`03ooooo`Co00001Oooool2o`0000;ooooo00Go
0000ooooooooooooooooo`000005ooooo`?o000000?oooooo`000?ooool05oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo013ooooo00Co0000oooooooooooooooo0_l00002ooooo`;o
00000_ooool01?l0003oooooooooooooool3o`0000gooooo0ol00002ooooo`Oo000000?oooooo`00
0?l000001Ol0001Jooooo`00=Oooool3o`0000?ooooo0ol00006ooooo`Go00001Oooool01?l0003o
ooooooooooooool3o`0000Gooooo1Ol0000Jooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool03_ooool2o`0000Gooooo0ol000001?oooooo0000o`000?l00006ooooo`?o00002Oooool7
o`0000Wooooo1ol0001Iooooo`00@oooool7o`0000oooooo1ol0000Fooooo`05o`000?oooooooooo
ooooool000001Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00;ooooo0ol0000<
ooooo`Go000000?oooooo`000?l000001Ol00004oooooa?o0000FOooool004?ooooo1ol0000?oooo
o`Oo00005_ooool01ol0003ooooooooooooooooo0000ooooool000000oooool00ol0003ooooooooo
o`0:ooooo`;o00000_ooool3o`0000gooooo4ol0000<ooooo`Oo0000FOooool004?ooooo1ol0000?
ooooo`Oo00005oooool3o`0000;ooooo1Ol00006ooooo`?o00000_ooool00ol0003oooooooooo`02
o`0000Wooooo1ol000000ooooooo0000o`000005o`0001Sooooo1Ol0001Jooooo`00A?ooool5o`00
017ooooo1Ol0000Wooooo`Wo00001?ooool7o`0000Sooooo1ol0000Iooooo`?o0000Foooool004Go
oooo0ol0000Cooooo`?o00009oooool>o`00013ooooo1Ol0001hooooo`00B?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo02Oooooo1ol0000Hooooo`?o0000NOooool002Cooooo1Ol0
0003ooooo`;o00006_ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo000000;o00000_oo
ool2o`00023ooooo2?l0002Dooooo`009Oooool01?l0003oooooooooool00002ooooo`04o`000?oo
ooooooooo`0001[ooooo00?o0000ooooooooool03_ooool01Ol0003ooooooooooooooooo000000Go
oooo00?o0000ooooooooool06_ooool3o`0000;ooooo1Ol0002Eooooo`009_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0001[ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooo`000004o`0000Gooooo00?o0000ooooooooool05_ooool3o`0000Kooooo0ol0002Fooooo`00
9oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooo000000;ooooo00Co0000oooooooooooooooo1?l0000Eoooo
o`?o0000X_ooool002Cooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oooooooooo
o`0001_ooooo00?o0000ooooooooool02oooool01ol0003ooooooooooooooooo0000ooooool00000
0_ooool01Ol0003ooooooooooooooooo000001;ooooo0ol0002Uooooo`009?ooool01Ol0003ooooo
oooooooooooo000000;ooooo00Co0000oooooooooooo00001_ooool3o`0001?ooooo00?o0000oooo
ooooool02_ooool00ol0003oooooooooo`02ooooo`;o00000_ooool01Ol0003ooooooooooooooooo
000000Kooooo0ol00006ooooo`?o0000Z?ooool002Gooooo0ol00004ooooo`;o00001_ooool5o`00
01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`04ooooo`04o`000?oooooooooo
ooooo`?o00001_ooool5o`0000;ooooo0ol0002[ooooo`00<oooool7o`0001;ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0@ooooo`So0000[_ooool003?ooooo1ol0000Booooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool7o`000:oooooo000cooooo`Wo0000
1?ooool3o`0000Cooooo0_l00004ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
3oooool9o`000:oooooo000dooooo`Go00000oooool3o`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
3Oooool3o`0000?ooooo1Ol0002`ooooo`00=Oooool3o`0000Oooooo0ol00004ooooo`04o`000?oo
ooooooooo`0000Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0:ooooo`?o0000
1oooool3o`000;7ooooo0011ooooo`Co000000Goooooo`000?ooooooooooo`000004ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool02?ooool3o`000;kooooo0011ooooo`04o`000?oo
ooooooooooooo`?o000000?oooooo`000?ooool01?ooool01Ol0003ooooooooooooooooo000000So
oooo0ol00031ooooo`00@?ooool2o`0000Cooooo00?o0000ooooool000000_l00004ooooo`Go0000
1Oooool3o`000<Cooooo0011ooooo`03o`000?oooooooooo00?ooooo0_l00002ooooo`?o000000?o
ooooo`000?l000000ol00002ooooo`?o0000aoooool004kooooo2?l0003:ooooo`00C_ooool7o`00
0<_ooooo001>ooooo`Oo0000boooool004oooooo1Ol0003<ooooo`00D?ooool3o`000<gooooo001A
ooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03
o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oo
oooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo
0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo
001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aoooo
o`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo0016ooooo`?o0000
2?ooool00ol0003oooooooooo`3<ooooo`00Aoooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo0<cooooo0017ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0c?ooool0
04Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`3<ooooo`00Aoooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo0<cooooo0016ooooo`;o00002?ooool3o`000<gooooo
0017ooooo`03o`000?oooooooooo00Gooooo1Ol0003<ooooo`00C_ooool7o`000<_ooooo001>oooo
o`Oo0000boooool004kooooo1ol0003;ooooo`00Coooool5o`000<cooooo001@ooooo`?o0000cOoo
ool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057o
oooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o
0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000oooo
ooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0
c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0
057ooooo00?o0000ooooooooool0c?ooool004Gooooo1Ol00007ooooo`03o`000?oooooooooo0<co
oooo0016ooooo`04o`000?ooooooooooo`0000Oooooo00?o0000ooooooooool0c?ooool004Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`3<ooooo`00B?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo0<cooooo0015ooooo`05o`000?ooooooooooooooool000001ooo
ool00ol0003oooooooooo`3<ooooo`00AOooool01Ol0003ooooooooooooooooo000000Kooooo0ol0
003=ooooo`00A_ooool3o`0000Kooooo1Ol0003<ooooo`00C_ooool7o`000<_ooooo001>ooooo`Oo
0000boooool004kooooo1ol0003;ooooo`00C_ooool7o`000<_ooooo001=ooooo`03o`000?oooooo
oooo00?o00000_ooool00ol0003oooooooooo`38ooooo`00C?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo0<Oooooo001;ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool0a_ooool004Wooooo0_l0000=ooooo`;o0000a_ooool004Sooooo00?o0000ooooooooool03ooo
ool00ol0003oooooooooo`33ooooo`00Aoooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo0<;ooooo000/ooooo`?o00000oooool3o`00017ooooo00?o0000ooooooooool03?ooool3o`00
00;ooooo1Ol00031ooooo`00:oooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool00_ooool0
1?l0003oooooo`000?l00031ooooo`00;oooool00ol0003oooooo`000003ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
00000<7ooooo000]ooooo`;o00000_ooool4o`0000gooooo0_l0000Booooo`03o`000?oooooooooo
00Cooooo00Co0000ooooool0003o0000_oooool002oooooo00?o0000ooooool000003oooool00ol0
003oooooooooo`0Booooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
002nooooo`00:oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0Booooo`;o00000oooool01Ol0003ooooooooooooooooo000000?ooooo
00?o0000ooooooooool0^oooool002cooooo0ol00004ooooo`;o00001_ooool3o`000003ooooool0
003ooooo01Gooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000ooooool000000_l0
002hooooo`003oooool3o`0000?ooooo0ol0000Rooooo`Go00009Oooool5o`000;Oooooo000>oooo
o`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oooooooooo01kooooo1ol0
000Sooooo`Oo0000]_ooool001;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0N
ooooo`Oo00008oooool7o`000;Kooooo000@ooooo`;o00001_ooool00ol0003oooooooooo`0Joooo
o`_o00008oooool9o`0000Gooooo0ol00003ooooo`?o0000Y_ooool001;ooooo00?o0000ooooool0
00000ol0000Hooooo`Go00001?ooool6o`0002Gooooo1Ol00003ooooo`?o00000oooool01Ol0003o
oooooooooooooooo000000?ooooo00?o0000ooooooooool0Xoooool000kooooo00Oo0000oooooooo
ooooooooo`000?oooooo000000[ooooo0ol0000:ooooo`Co00002?ooool00ol0003oooooooooo`03
o`0002Oooooo0ol00007ooooo`Co00001oooool00ol0003oooooooooo`2Sooooo`003oooool3o`00
00;ooooo1Ol00005ooooo`Go00001?ooool5o`0000[ooooo0_l00003ooooo`03o`000?oooooooooo
02[ooooo00?o0000ooooooooool01oooool3o`0000?ooooo0_l0000Jooooo`;o0000R_ooool001go
oooo2_l0000>ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0:_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00?o00000_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo08Oooooo000Mooooo`Oo00004?ooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo02cooooo00?o0000ooooooooool01Oooool2o`0000?ooooo00?o0000ooooool00000
0_l00005ooooo`?o00005?ooool00ol0003oooooooooo`26ooooo`007Oooool7o`0000oooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool00_ooool3o`000005ooooool0003o0000o`000?ooool01Ol0000@ooooo`Co0000
R?ooool001kooooo1Ol0000>ooooo`;o00001oooool00ol0003oooooooooo`0]ooooo`03o`000?oo
oooooooo017ooooo1ol0000>ooooo`05o`000?ooooooooooooooool00000R?ooool001Kooooo1Ol0
0002ooooo`Go00003_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo02kooooo00?o
0000ooooooooool04Oooool:o`0000_ooooo00Go0000ooooooooooooooooo`000006ooooo`?o0000
Ooooool001Oooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0@oooooaCo0000
00?oooooo`000?l000000_l00006ooooo`Go0000O_ooool001Sooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0;ooo
ool00ol0003oooooooooo`0Aooooo`Go000000Coooooo`000?l0003ooooo1?l00003ooooo`03o`00
0?oooooooooo00Wo000000?oooooo`000?l000001Ol0001mooooo`006Oooool01?l0003ooooooooo
ooooool4o`0000cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0aooooo`03o`00
0?oooooooooo017ooooo0ol00004ooooo`04o`000?ooooooooooooooo`?o000000?oooooo`000?oo
ool01Ol00005ooooo`So0000OOooool001Kooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00?o0000ooooooooool02?ooool2o`0000_ooooo00?o0000ooooooooool0<Oooool00ol0
003oooooooooo`0Iooooo`;o00000oooool5o`0000;ooooo00?o0000ooooooooool01Oooool7o`00
07gooooo000Fooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oooooo
oooo00?ooooo0ol000000ooooooo0000ooooo`0<ooooo`03o`000?oooooooooo02Gooooo1Ol00002
ooooo`?o00000oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooool000000ol00008ooooo`Go0000O_ooool001Oooooo0ol00003ooooo`?o00001?ooool6o`00
0004ooooool0003o0000o`0000[ooooo00?o0000ooooooooool09_ooool01_l0003oooooooooool0
003oooooo`0000?ooooo00Co0000oooooooooooo00007Oooool2o`0000;ooooo00Co0000oooooooo
oooooooo1?l00005ooooo`Go0000Ooooool002Cooooo1ol000000ooooooo0000ooooo`09ooooo`03
o`000?oooooooooo02Sooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo0000
01Wooooo0ol000000ooooooo0000o`000003o`0000?ooooo00?o0000ooooooooool00ol000000ooo
oooo0000o`000003o`00083ooooo000Tooooo`[o00002Oooool00ol0003oooooooooo`0Yooooo`03
o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`00000Hooooo`05o`000?oooooooooo
ooooool000000_ooool2o`000003ooooool0003ooooo00Oooooo1ol0001oooooo`009?ooool7o`00
0003ooooool0003ooooo00Wooooo00?o0000ooooooooool09_ooool01_l0003ooooooooooooooooo
0000ooooo`Co00001Oooool00ol0003oooooooooo`0Iooooo`05o`000?ooooooooooo`000?ooool0
0_l00008ooooo`Oo00003Oooool3o`0006oooooo000Sooooo`So000000?oooooo`000?ooool02?oo
ool00ol0003oooooooooo`0Wooooo`07o`000?ooooooooooooooool0003oooooo`000008ooooo`Co
00006?ooool01?l0003oooooooooool00002ooooo`;o00001oooool9o`0000[ooooo00Go0000oooo
ooooooooooooo`00001^ooooo`008_ooool00ol0003oooooo`000004o`0000;ooooo0_l00007oooo
o`?o0000:_ooool3o`0000;ooooo1Ol00004ooooo`Go00004oooool4o`0000?ooooo00Go0000oooo
oooooooo0000ooooo`02o`0000Kooooo1Ol00002ooooo`Go00001_ooool01Ol0003ooooooooooooo
oooo000006kooooo000Pooooo`;o00000_ooool2o`0000Kooooo00?o0000ooooooooool01?ooool5
o`0003Kooooo1ol0000Booooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000ooo
ool00ol0003oooooooooo`03o`000004ooooool0003o0000o`0000Gooooo0_l000001?oooooo0000
o`000?l00003ooooo`Co0000Koooool001oooooo00Go0000ooooooooooooooooo`00000>ooooo`Oo
0000=Oooool7o`0001;ooooo1Ol00003ooooo`;o00001Oooool6o`0000[ooooo0ol000000ooooooo
0000o`000002o`0000;ooooo0_l0001^ooooo`007Oooool2o`0000?ooooo00?o0000ooooooooool0
3Oooool7o`0003Gooooo1ol0000Qooooo`Oo00002oooool3o`000003ooooool0003o000000Co0000
1_ooool3o`0006Gooooo0003o`0000;ooooo0ol0000Cooooo`;o00001?ooool00ol0003ooooooooo
o`0>ooooo`Oo0000=_ooool5o`0002;ooooo1ol0000;ooooo`03o`000?oooooooooo00[o00000_oo
ool5o`0006Cooooo00001Ooooooo0000oooooooooooo000000?ooooo00?o0000ooooooooool03ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo013ooooo1Ol0000hooooo`?o00008ooo
ool7o`0000_ooooo00Co0000oooooooooooooooo1?l00005ooooo`So00001oooool3o`0000?ooooo
0_l0001Dooooo`000ol00005ooooo`03o`000?oooooooooo00gooooo0_l00005ooooo`03o`000?oo
oooooooo01;ooooo0ol0000looooo`03o`000?oooooooooo027ooooo1Ol0000;ooooo`;o00001?oo
ool00ol0003oooooooooo`02o`0000Cooooo1ol00008ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool0DOooool00003ooooool0003ooooo00Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool08_ooool3
o`0000gooooo00?o0000ooooooooool00oooool2o`0000;ooooo0_l000000ooooooo0000o`000009
o`0000Cooooo0ol00003ooooo`Co0000D_ooool00004ooooool0003oooooooooo`Go000000?ooooo
o`000?l000000ol00006ooooo`;o00001_ooool00ol0003oooooooooo`1Dooooo`03o`000?oooooo
oooo03gooooo2?l00003ooooo`Go000000Goooooo`000?oooooo0000ooooo`05o`0005;ooooo0002
o`0000;ooooo0_l00002ooooo`04o`000?ooooooooooo`0000;ooooo1?l00002ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`0mooooo`Oo00001ooo
ool6o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool0D?ooool00004ooooool0003o
ooooooooo`Go00000oooool00ol0003oooooo`000005o`0000Oooooo00?o0000ooooooooool0Eooo
ool00ol0003oooooooooo`0looooo`Oo00002Oooool:o`0000Kooooo0ol00019ooooo`001oooool0
0ol0003oooooooooo`03ooooo`Oo00001_ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooo
oooo03gooooo1Ol0000;ooooo`03o`000?oooooo000000So00000oooool5o`0004Sooooo0004oooo
o`07o`000?ooooooooooooooool0003oooooo`000002ooooo`Oo00001Oooool00ol0003ooooooooo
o`1Jooooo`03o`000?oooooooooo03gooooo0ol0000:ooooo`08o`000?ooooooooooooooool0003o
ooooo`000?ooool3o`0000;ooooo2Ol00008ooooo`?o00000oooool4o`0003Gooooo0004ooooo`07
o`000?ooooooooooooooool0003oooooo`000002ooooo`[o000000?oooooo`000?ooool0G?ooool0
0ol0003oooooooooo`1;ooooo`?o00000oooool3o`000005ooooool0003o0000o`000?ooool01ol0
0007ooooo`07o`000?ooooooooooooooool0003oooooo`000002ooooo`;o0000=Oooool000Gooooo
0ol00003ooooo`go0000Goooool00ol0003oooooooooo`1Gooooo`_o00002?ooool00ol0003ooooo
o`000004o`0003Gooooo000?ooooo`[o0000Goooool00ol0003oooooooooo`1Gooooo`Oo00000_oo
ool5o`0000;ooooo0_l00002ooooo`04o`000?oooooo0000o`0003Kooooo000Booooo`Oo0000Gooo
ool00ol0003oooooooooo`1Gooooo`Oo00001_ooool6o`0000;ooooo00Co0000ooooool0003o0000
=Oooool001;ooooo1ol0001Cooooo`?o00000oooool3o`0000Cooooo00?o0000ooooooooool0Eooo
ool5o`0000Sooooo00?o0000ooooool000002?l00006ooooo`?o0000;?ooool001?ooooo1Ol0001E
ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo000005[ooooo
0ol0000:ooooo`?o00000_ooool7o`0000?ooooo1Ol00008ooooo`?o00000oooool3o`0001[ooooo
000Dooooo`?o0000E_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool0K?ooool=o`0000Sooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo01Oooooo001]ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`1`ooooo`Wo00002?ooool01Ol0003ooooooooooooooooo000000?ooooo
00?o0000ooooooooool05oooool006gooooo00Co0000oooooooooooooooo1?l00006ooooo`03o`00
0?oooooooooo073ooooo2ol00005ooooo`03o`000?oooooooooo00;ooooo0ol0000Jooooo`00K?oo
ool2o`0000?ooooo00?o0000ooooooooool01oooool4o`0006oooooo1ol00004ooooo`Co000000?o
ooooo`000?ooool00_ooool01Ol0003ooooooooooooooooo000001Wooooo001]ooooo`04o`000?oo
ooooooooooooo`Go00001Oooool5o`0006oooooo1Ol00009ooooo`Co000000?oooooo`000?ooool0
0_ooool00ol0003oooooooooo`0Gooooo`00N_ooool7o`0006oooooo0ol0000;ooooo`03o`000?oo
oooooooo00Go00001oooool3o`00013ooooo001jooooo`Oo0000Q?ooool4o`0000?ooooo1Ol0000?
ooooo`00N_ooool7o`0008Sooooo2Ol0000>ooooo`00Noooool5o`0008_ooooo1ol0000>ooooo`00
O?ooool4o`0008_ooooo1ol0000>ooooo`00O_ooool3o`0008_ooooo1Ol0000?ooooo`00O_ooool2
o`000003ooooool0003ooooo08[ooooo0ol0000@ooooo`00O_ooool01Ol0003oooooo`000?oooooo
000009gooooo001nooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool0V_ooool007ko
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`2Iooooo`00O_ooool01?l0003o
ooooooooool00003ooooo`03o`000?oooooooooo09Sooooo001oooooo`04o`000?ooooooooooo`00
00?ooooo00?o0000ooooooooool0Uoooool007oooooo00Go0000oooooooooooo0000ooooo`04o`00
09Sooooo001oooooo`05o`000?ooooooooooooooool000000oooool2o`0009Oooooo001oooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooo`00002Fooooo`00O_ooool2o`0000Co
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`2Cooooo`00O_ooool2o`0000?o
oooo1?l00004ooooo`03o`000?oooooooooo09;ooooo001`ooooo`?o00001?ooool00ol0003ooooo
ooooo`05ooooo`;o00000_ooool00ol0003oooooo`000006ooooo`03o`000?oooooo000000;o0000
Soooool0077ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`;o00000_oo
ool5o`0000Gooooo1Ol0002>ooooo`00LOooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`Oo0000SOooool0
077ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00oooool4o`0000;ooooo1ol0002=ooooo`00LOooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Go00001Oooool5o`000003ooooool0003o000000Go0000SOooool0073o
oooo0_l00003ooooo`05o`000?ooooooooooooooool000001_ooool3o`0000;ooooo1ol000000ooo
oooo0000o`000003o`0008kooooo001aooooo`04o`000?ooooooooooooooo`Go00001Oooool5o`00
0003ooooool0003o000000Go00000_ooool4o`0008kooooo001nooooo`ko00001_ooool00ol0003o
ooooooooo`2;ooooo`00O_ooool7o`000003ooooool0003o000000?o00002?ooool00ol0003ooooo
ooooo`2:ooooo`00O_ooool7o`0000;ooooo0ol0000:ooooo`03o`000?oooooooooo08Wooooo001o
ooooo`Go00001_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo08Sooooo0020oooo
o`?o00001oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo08Oooooo002;ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool0Q_ooool008_ooooo00?o0000ooooooooool0
2_ooool00ol0003oooooooooo`25ooooo`00S?ooool00ol0003oooooooooo`05o`0000;ooooo0ol0
0027ooooo`00S?ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool000000ooo
ool00ol0003oooooooooo`24ooooo`00SOooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooooooo0_l00025ooooo`00SOooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00Co00000_ooool00ol0003oooooooooo`22ooooo`00S_ooool2o`0000?ooooo00?o0000
ooooool000001_ooool00ol0003oooooooooo`21ooooo`00S_ooool2o`0000?ooooo00Co0000oooo
oooooooo00001_ooool00ol0003oooooooooo`20ooooo`00S_ooool00ol0003oooooo`000002o`00
00Cooooo0_l00005ooooo`03o`000?oooooo000000;o0000OOooool008Cooooo0ol00002ooooo`?o
00000oooool00ol0003oooooooooo`0=ooooo`Go0000O?ooool008Gooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo1ol0001kooooo`00P_oo
ool5o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0;ooooo`Oo0000Nooo
ool008;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00_ooooo1ol0001kooooo`00Poooool00ol0003oooooo`000004ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool02oooool5o`0007cooooo0024ooooo`;o00000oooool2o`00
00Kooooo1?l0000;ooooo`Co0000O?ooool008Gooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`04ooooo`Go00003_ooool00ol0003oooooooooo`1iooooo`00T?ooool7o`0000kooooo
00?o0000ooooooooool0N?ooool0093ooooo1ol0000?ooooo`03o`000?oooooooooo07Oooooo002@
ooooo`Oo00004?ooool00ol0003oooooooooo`1fooooo`00TOooool5o`0001;ooooo00?o0000oooo
ooooool0MOooool009;ooooo0ol0000Dooooo`03o`000?oooooooooo07Cooooo002Zooooo`03o`00
0?oooooooooo07?ooooo002Sooooo`?o00000_ooool4o`0007Cooooo002Tooooo`04o`000?oooooo
ooooo`0000?ooooo0_l0001cooooo`00XOooool5o`000003ooooool0003ooooo00;ooooo00?o0000
ooooool00000L_ooool00:7ooooo00Co0000oooooooooooo00000oooool3o`0000?ooooo00?o0000
ooooooooool0Koooool00:;ooooo00?o0000ooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00?o0000ooooooooool0K_ooool00:?ooooo0_l00002ooooo`05o`000?oooooooooo
ooooool000001?ooool00ol0003oooooo`000002o`0006_ooooo002Tooooo`04o`000?oooooooooo
ooooo`?o00001_ooool5o`0006[ooooo002`ooooo`Oo0000JOooool00;3ooooo1ol0001Yooooo`00
/?ooool7o`0006Wooooo002aooooo`Go0000J_ooool00;;ooooo0ol0001[ooooo`00oooooolQoooo
o`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQoooo
o`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.174593, -0.0500052, \
0.00305022, 0.00369221}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(bos4 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] g3, \[IndentingNewLine]PayoffMatrix \[Rule] 
            battleOfSexes, \[IndentingNewLine]StrategyColors \[Rule] 
            bosColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos4]];\)\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.177966 1.12187 0.0463215 0.926431 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.28427 .44711 m
.28427 .53246 L
s
.28427 .44711 m
.28427 .36175 L
s
.28427 .53246 m
.35736 .59281 L
s
.28427 .53246 m
.21118 .59281 L
s
.36338 .20404 m
.40293 .12518 L
s
.36338 .20404 m
.32382 .28289 L
s
.84452 .73379 m
.94391 .75723 L
s
.84452 .73379 m
.74513 .71036 L
s
.49939 .81825 m
.43647 .75053 L
s
.49939 .81825 m
.56231 .88596 L
s
.44249 .04632 m
.40293 .12518 L
s
.47602 .82939 m
.43647 .75053 L
s
.47602 .82939 m
.51558 .90824 L
s
.64622 .68558 m
.74513 .71036 L
s
.64622 .68558 m
.54834 .65814 L
s
.64622 .68558 m
.74296 .71564 L
s
.55533 .63662 m
.45285 .62548 L
s
.28427 .36175 m
.18878 .32909 L
s
.28427 .36175 m
.24472 .28289 L
s
.28427 .36175 m
.32382 .28289 L
s
.28427 .36175 m
.37976 .32909 L
s
.88186 .23533 m
.78048 .25198 L
s
.35736 .59281 m
.45285 .62548 L
s
.35736 .59281 m
.39691 .67167 L
s
.45285 .62548 m
.54834 .65814 L
s
.45285 .62548 m
.53485 .67744 L
s
.18443 .67526 m
.21118 .59281 L
s
.43647 .75053 m
.44996 .83515 L
s
.43647 .75053 m
.39691 .67167 L
s
.64105 .69589 m
.54834 .65814 L
s
.74513 .71036 m
.84354 .73647 L
s
.07517 .72089 m
.1381 .65317 L
s
.24472 .28289 m
.20516 .20404 L
s
.67911 .26863 m
.57773 .28528 L
s
.67911 .26863 m
.78048 .25198 L
s
.56231 .88596 m
.62523 .95368 L
s
.47526 .29642 m
.55726 .24446 L
s
.47526 .29642 m
.57075 .26376 L
s
.47526 .29642 m
.57773 .28528 L
s
.47526 .29642 m
.37976 .32909 L
s
.14774 .13307 m
.20516 .20404 L
s
.14774 .13307 m
.09031 .0621 L
s
.1381 .65317 m
.21118 .59281 L
s
.1381 .65317 m
.05609 .70513 L
s
.55726 .24446 m
.63926 .1925 L
s
.20516 .20404 m
.185 .12032 L
s
.68087 .2797 m
.57773 .28528 L
s
.11134 .61491 m
.21118 .59281 L
s
.63926 .1925 m
.72126 .14054 L
s
0 1 0 r
.025 w
.28427 .44711 Mdot
.28427 .53246 Mdot
1 0 0 r
.36338 .20404 Mdot
0 1 0 r
.84452 .73379 Mdot
.49939 .81825 Mdot
1 0 0 r
.44249 .04632 Mdot
.47602 .82939 Mdot
0 1 0 r
.64622 .68558 Mdot
1 0 0 r
.55533 .63662 Mdot
.28427 .36175 Mdot
0 1 0 r
.88186 .23533 Mdot
1 0 0 r
.35736 .59281 Mdot
.45285 .62548 Mdot
.18443 .67526 Mdot
0 1 0 r
.43647 .75053 Mdot
.64105 .69589 Mdot
.44996 .83515 Mdot
1 0 0 r
.94391 .75723 Mdot
.74513 .71036 Mdot
0 1 0 r
.18878 .32909 Mdot
.40293 .12518 Mdot
.07517 .72089 Mdot
.24472 .28289 Mdot
.67911 .26863 Mdot
.39691 .67167 Mdot
.56231 .88596 Mdot
1 0 0 r
.47526 .29642 Mdot
.14774 .13307 Mdot
.1381 .65317 Mdot
0 1 0 r
.55726 .24446 Mdot
.20516 .20404 Mdot
1 0 0 r
.68087 .2797 Mdot
0 1 0 r
.84354 .73647 Mdot
1 0 0 r
.54834 .65814 Mdot
.11134 .61491 Mdot
0 1 0 r
.21118 .59281 Mdot
1 0 0 r
.57075 .26376 Mdot
0 1 0 r
.74296 .71564 Mdot
1 0 0 r
.63926 .1925 Mdot
0 1 0 r
.185 .12032 Mdot
.51558 .90824 Mdot
1 0 0 r
.32382 .28289 Mdot
.57773 .28528 Mdot
.09031 .0621 Mdot
.05609 .70513 Mdot
0 1 0 r
.78048 .25198 Mdot
1 0 0 r
.72126 .14054 Mdot
.62523 .95368 Mdot
.37976 .32909 Mdot
0 1 0 r
.53485 .67744 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[29]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00OOooool3ool00:3ooooo001looooo`Goo`00Woooool007_ooooo1ooo002Nooooo`00Nooo
ool7ool009kooooo000Hooooo`?oo`00H?ooool7ool009kooooo000Gooooo`Goo`00H?ooool5ool0
09oooooo000Fooooo`Ooo`00H?ooool3ool00:3ooooo000Fooooo`Ooo`00Goooool00ol0003ooooo
ooooo`2Qooooo`005_ooool7ool005oooooo00?o0000ooooooooool0XOooool001Oooooo1Ooo001O
ooooo`03o`000?oooooooooo0:;ooooo000Hooooo`?oo`0000?o0000ooooooooool0GOooool00ol0
003oooooooooo`2Rooooo`007?ooool00ol0003oooooooooo`1Kooooo`03o`000?oooooooooo0:?o
oooo000Mooooo`03o`000?oooooooooo05[ooooo00?o0000ooooooooool0Xoooool001gooooo00?o
0000ooooooooool0FOooool00ol0003oooooooooo`2Tooooo`007_ooool00ol0003oooooooooo`1H
ooooo`03o`000?oooooooooo0:Cooooo000Oooooo`03o`000?oooooooooo05Kooooo00?o0000oooo
ooooool0YOooool0023ooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`2Uooooo`00
8Oooool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo0:Kooooo000Rooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool0Y_ooool002;ooooo00?o0000ooooooooool0DOooool0
0ol0003oooooooooo`2Wooooo`008oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo
0:Oooooo000Tooooo`03o`000?oooooooooo00gooooo0ol0o`0nooooo`03o`000?oooooooooo0:So
oooo000Uooooo`03o`000?oooooooooo00_ooooo1Ol0o`0jooooo`?o0?l000?o0000ooooooooool0
Z?ooool002Kooooo00?o0000ooooooooool02Oooool7o`3o03Sooooo1Ol0o`2Zooooo`009_ooool0
0ol0003oooooooooo`09ooooo`Oo0?l0=oooool7o`3o0:Wooooo000Wooooo`03o`000?ooooooo`00
00;oo`001_ooool7o`3o03Oooooo1ol0o`2Yooooo`00:?ooool5ool000Kooooo1Ol0o`0hooooo`Oo
0?l0ZOooool002Oooooo1ooo0006ooooo`?o0?l0>_ooool5o`3o05Oooooo0ooo001@ooooo`009ooo
ool7ool000Sooooo00?o0000ooooooooool0>Oooool3o`3o05Oooooo1Ooo001?ooooo`009oooool7
ool000Sooooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`1Gooooo`Ooo`00C_ooool0
02Sooooo1Ooo0009ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0Eoooool7ool0
04kooooo000Yooooo`?oo`0000?o0000ooooooooool01oooool00ol0003oooooooooo`0gooooo`03
o`000?oooooooooo05Sooooo1ooo001>ooooo`00;Oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo03Kooooo00?o0000ooooooooool0F?ooool00ol0003oo`00ool00003ool004oooooo
000^ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0=Oooool00ol0003ooooooooo
o`1Gooooo`;o00000_ooool3ool0053ooooo000^ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool0=Oooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo05Gooooo000_oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`1Eoooo
o`;o0000F?ooool0033ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0dooooo`03
o`000?oooooooooo05Cooooo00?o0000ooooooooool0F?ooool0037ooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo05?ooooo0_l0001Kooooo`00<_oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0DOoo
ool2o`0005gooooo000booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`1@ooooo`03o`000?oooooooooo05gooooo000cooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`1?ooooo`;o0000H?ooool003Co
oooo00Go0000ooooooooooooooooo`00000cooooo`03o`000?oooooooooo04kooooo00?o0000oooo
ooooool0H?ooool003Gooooo00Go0000ooooooooooooooooo`00000aooooo`03o`000?oooooooooo
04Sooooo0ooo0002ooooo`;o0000Hoooool003Kooooo00Co0000oooooooooooo0000<Oooool00ol0
003oooooooooo`17ooooo`Goo`0000?o0000ooooooooool0Hoooool003Kooooo00Co0000oooooooo
oooo0000<?ooool00ol0003oooooooooo`17ooooo`Ooo`00IOooool003Oooooo00?o0000ooooool0
o`000_l0o`0[ooooo`?oo`0000?o0000ooooooooool0Aoooool7ool006Gooooo000hooooo`Go0?l0
:Oooool5ool004Wooooo1ooo001Uooooo`00=oooool7o`3o02Oooooo1ooo0017ooooo`;o00001Ooo
001Vooooo`00=oooool7o`3o02Oooooo1ooo0015ooooo`;o00000oooool3ool006Oooooo000goooo
o`Oo0?l09oooool7ool004Cooooo00?o0000ooooooooool0KOooool003Sooooo1Ol0o`0Yooooo`Go
o`00@oooool2o`00073ooooo000iooooo`?o0?l0:oooool3ool004?ooooo00?o0000ooooooooool0
L?ooool003cooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`12ooooo`;o0000Looo
ool003cooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`10ooooo`;o0000MOooool0
03gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo
057ooooo0ol0o`0Qooooo`00?Oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03ko
oooo0_l0001Cooooo`Go0?l08?ooool003kooooo00?o0000ooooooooool08oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo05;ooooo1ol0o`0Oooooo`00?_ooool00ol0003ooooooooo
o`0Sooooo`03o`000?oooooooooo03Oooooo0ol0o`02ooooo`;o0000EOooool7o`3o01oooooo000o
ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0=oooool5o`3o0003o`000?oooooo
oooo04oooooo1_l00007o`3o01oooooo000oooooo`03o`000?oooooooooo027ooooo00?o0000oooo
ooooool0=_ooool7o`3o03_ooooo0ol0o`0=ooooo`Ko00001oooool5o`3o023ooooo0010ooooo`03
o`000?oooooooooo01oooooo00?o0000ooooooooool0=oooool7o`3o03[ooooo1Ol0o`06ooooo`Ko
00003_ooool3o`3o027ooooo0011ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0
>?ooool7o`3o03Wooooo1ol0o`05o`0003Sooooo0011ooooo`03o`000?oooooooooo01gooooo00?o
0000ooooooooool0=oooool2o`0000Go0?l00ooo000gooooo`Oo0?l0?Oooool004;ooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`0gooooo`04o`000?ooooooooooooooo`?o0?l01Ooo
000`ooooo`Ko00001ol0o`0mooooo`00@_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooo
oooo03Gooooo0_l00006ooooo`Ooo`006_ooool3o`3o00cooooo1_l00007ooooo`Go0?l0?_ooool0
04?ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo
00Kooooo1ooo000Iooooo`Go0?l01_ooool5o`0000kooooo0ol0o`0oooooo`00@oooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo03?ooooo0_l00008ooooo`03o`000?oo003oo`0000Go
o`006?ooool7o`3o00Go0000EOooool004Cooooo00?o0000ooooooooool05oooool00ol0003ooooo
ooooo`0booooo`;o00001oooool3o`0000;ooooo1Ooo000Hooooo`04o`000?l0o`3o0?l0o`3o00?o
o`0000?o0?l0ooooooooool0F?ooool004Cooooo00Co0000o`3o0?l0o`3o0?l04oooool3ool00003
o`000?oooooooooo037ooooo00?o0000ooooooooool01Oooool2o`0000Kooooo1Ooo000@ooooo`Oo
000000?oooooo`3o0?l0o`001Ooo001Jooooo`00A?ooool5o`3o017ooooo1Ooo000aooooo`;o0000
1Oooool3o`0000Wooooo1Ooo0008ooooo`Oo00002Oooool7ool005Wooooo0013ooooo`Oo0?l03ooo
ool7ool002oooooo00?o0000ooooooooool00_ooool3o`0000_ooooo1ooo0007o`0000Cooooo3?l0
0007ool005Wooooo0013ooooo`Oo0?l03oooool7ool002gooooo0_l00002ooooo`?o00003Oooool0
0ol0003oo`00ool00005ool000_o00003?ooool7ool005Wooooo0013ooooo`Oo0?l03oooool7ool0
02Oooooo0ooo0002ooooo`03o`000?oooooooooo00;o00002Oooool7o`000003oooooooo003oo`00
00Goo`006?ooool5ool005[ooooo0014ooooo`Go0?l04Oooool5ool002Oooooo1Ooo0004o`0000Co
oooo1ol00009ooooo`Goo`006_ooool3ool005_ooooo0015ooooo`?o0?l04oooool3ool002Oooooo
1ooo0007o`00017ooooo0ooo001iooooo`00B?ooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo02Sooooo1ooo002Dooooo`00B?ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooo
oooo02Kooooo0_l00007ool009Cooooo0019ooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool09?ooool3o`0000?ooooo1Ooo002Eooooo`00BOooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo027ooooo0ol00007ooooo`?oo`00U_ooool004[ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0Oooooo`?o0000Xoooool004[ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0Looooo`?o0000Y_ooool003Gooooo0ol0o`0Cooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool04Oooool3ool000Kooooo0ol0002Yooooo`00=?ooool5o`3o
01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0@ooooo`Goo`000_ooool3o`00
0:cooooo000cooooo`Oo0?l04_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013o
oooo1ooo00000ol0003oooooooooo`2]ooooo`00<oooool7o`3o01;ooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0@ooooo`Ooo`00/?ooool003?ooooo1ol0o`02o`00017ooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo003oo`0000Goo`00/?oo
ool003Cooooo1Ol0o`03ooooo`?o00003_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00gooooo0ol00002ooooo`Goo`00/Oooool003Gooooo0ol0o`07ooooo`?o00003?ooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00_ooooo0ol00006ooooo`?oo`00/_ooool004;o
oooo0ol00009ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?ooool3o`000;ko
oooo0015ooooo`?o00001oooool01Ol0003ooooooooooooooooo000000Sooooo0ol00031ooooo`00
B?ooool3o`0000Cooooo00Go0000ool00?oo003oo`00o`000005ooooo`?o0000a?ooool004_ooooo
0ol000000oooooooo`00ool00003ool000;ooooo0ol00037ooooo`00C_ooool7ool00003o`000?oo
oooooooo0<Sooooo001>ooooo`Ooo`00boooool004kooooo1ooo003;ooooo`00Coooool5ool00<co
oooo001@ooooo`?oo`00cOooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000
ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000oooooooo
ool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?oo
ool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057o
oooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o
0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000oooo
ooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0
c?ooool0057ooooo00?o0000ooooooooool0c?ooool0057ooooo00?o0000ooooooooool0c?ooool0
057ooooo00?o0000ooooooooool0c?ooool0053ooooo0ol0o`3=ooooo`00Coooool5o`3o0<cooooo
001>ooooo`Oo0?l0boooool004kooooo1ol0o`3;ooooo`00C_ooool7o`3o0<_ooooo001?ooooo`Go
0?l0c?ooool0053ooooo0ol0o`3=ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool0
0ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003o
ooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003ooooooooo
o`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<oooo
o`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOoo
ool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0
003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003ooooo
ooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<
ooooo`00D?ooool3o`3o0<gooooo001?ooooo`Go0?l0c?ooool004kooooo1ol0o`3;ooooo`00C_oo
ool7o`3o0<_ooooo001>ooooo`Oo0?l0boooool004kooooo00?o0000o`3o0?l0o`000ol0o`000ol0
003oooooooooo`39ooooo`00COooool00ol0003oooooooooo`03o`3o00;ooooo00?o0000oooooooo
ool0b?ooool004cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`37ooooo`00Booo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo0<Kooooo0019ooooo`;o00003Oooool2
o`000<Kooooo0018ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0`oooool004Oo
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`32ooooo`00A_ooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo0<7ooooo0015ooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool0`?ooool004Cooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`2o
ooooo`00@_ooool2o`0001_ooooo0_l0002oooooo`00@Oooool00ol0003oooooooooo`0Mooooo`03
o`000?oooooooooo0;cooooo0010ooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0
^oooool003_ooooo0ol0o`000ooooooo0000ooooo`0Rooooo`03o`000?ooooooo`0000;oo`00^?oo
ool003[ooooo1Ol0o`0Uooooo`Goo`00]oooool003Wooooo1ol0o`0Sooooo`Ooo`00]_ooool003Wo
oooo1ol0o`0Sooooo`Ooo`00]_ooool003Gooooo1?l00007o`3o02?ooooo1ooo0002o`000;Cooooo
000`ooooo`Go00001?ooool00ol0003o0?l0o`3o0003o`3o02Gooooo1Ooo0003ooooo`?o0000/Ooo
ool001kooooo0ooo000:ooooo`Go00002?ooool00ol0003oooooooooo`03o`3o02Oooooo0ooo0007
ooooo`?o0000[_ooool001gooooo1Ooo0004ooooo`Go00002oooool2o`0000?ooooo00?o0000oooo
ooooool0:_ooool00ol0003oooooooooo`07ooooo`?o0000Zoooool001cooooo1ooo0003o`0000oo
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo00[o
oooo0ol0002Xooooo`007?ooool7ool0017ooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`0/ooooo`03o`000?oooooooooo00cooooo0ol00005ooooo`?oo`00WOooool001cooooo1ooo
000@ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0;?ooool00ol0003ooooooooo
o`0?ooooo`?o000000?oooooool00?oo00000ooo002Looooo`007Oooool5ool000oooooo0_l00006
ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool04Oooool7ool009_ooooo000Noooo
o`?oo`003oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo02kooooo00?o0000oooo
ooooool04Oooool7ool000;o00005oooool3ool007oooooo000_ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0@ooooo`Ooo`003?l0000<ooooo`Go
o`00O_ooool002kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo017ooooo1Ooo0002o`000004ooooool0003o0000o`0000Oooooo2_l000000ooooooo
o`00ool00005ool007gooooo000]ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
<Oooool00ol0003oooooooooo`0Aooooo`?oo`000oooool2o`0000;ooooo0ol0000>ooooo`03o`00
0?oo003oo`0000Goo`00OOooool002_ooooo0_l0000;ooooo`03o`000?oooooooooo037ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`02ooooo`?o00003?ooool7ool007gooooo000V
ooooo`?oo`0000?oooooo`000?ooool02oooool00ol0003oooooooooo`0cooooo`03o`000?oooooo
oooo01Wooooo0_l00005ooooo`?o00002_ooool5ool007kooooo000Uooooo`Goo`003Oooool00ol0
003oooooooooo`0cooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool01Oooool3o`00
00Kooooo1Ooo001oooooo`009?ooool7ool000cooooo00?o0000ooooooooool0=?ooool00ol0003o
ooooooooo`0Kooooo`;o00002?ooool3o`0000;ooooo1Ooo0020ooooo`009?ooool7ool000_ooooo
00?o0000ooooooooool0=Oooool00ol0003oooooooooo`0Mooooo`;o00002Oooool00ol0003oo`00
ool00005ool007oooooo000Tooooo`Ooo`002oooool00ol0003oooooooooo`0fooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool01oooool7ool007oooooo000Sooooo`;o00001Ooo000<
ooooo`03o`000?oooooooooo03Kooooo00Co0000o`3o0?l0o`3o0?l07_ooool2o`0000Oooooo1ooo
0002o`0007gooooo000Rooooo`04o`000?ooooooooooo`0000?oo`002_ooool3ool003Wooooo1Ol0
o`0Oooooo`03o`000?oooooooooo00Gooooo1Ooo0002ooooo`Go0000NOooool0023ooooo0_l00002
ooooo`03o`000?oooooooooo00_ooooo1Ooo000gooooo`Oo0?l07oooool2o`0000;ooooo0ol0o`00
1?ooooooo`00ool00?oo0005ooooo`;o000000Coooooo`000?l0003o0000M_ooool001oooooo00Go
0000ooooooooooooooooo`00000=ooooo`Ooo`00=_ooool7o`3o027ooooo00?o0000o`3o0?l0o`00
0ol0o`0:ooooo`?o000000?oooooo`000?l000000_l0001booooo`007Oooool2o`0000?ooooo00?o
0000ooooooooool03?ooool7ool003Kooooo1ol0o`0Qooooo`Oo0?l03?ooool2o`0000?ooooo0ol0
0007ooooo`?o0?l0IOooool001_ooooo0_l00004ooooo`03o`000?oooooooooo00gooooo1ooo000g
ooooo`Go0?l08_ooool7o`3o00kooooo0_l00004ooooo`Co00000_ooool5o`3o06Cooooo000Joooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03oooool5ool003Wooooo0ol0o`0Soooo
o`Oo0?l04?ooool3o`0000Gooooo00?o0000o`3o0?l0o`001Ol0o`1Sooooo`006?ooool2o`0000Go
oooo00?o0000ooooooooool04Oooool3ool003gooooo00?o0000ooooooooool08Oooool5o`3o01Co
oooo0_l00004ooooo`Oo0?l0Hoooool001Oooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`1Booooo`03o`000?oooooooooo02;ooooo0ol0o`0Gooooo`;o000000?oooooo`3o0?l0o`00
1_l0o`03o`00063ooooo000Eooooo`;o00001_ooool00ol0003oooooooooo`1Dooooo`03o`000?oo
oooooooo03gooooo2?l0o`03ooooo`Go0000G?ooool000oooooo0ooo0002ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`0mooooo`Oo0?l01oooool6
o`0005Sooooo000>ooooo`Goo`0000?o0000ooooooooool01Oooool00ol0003oooooooooo`1Goooo
o`03o`000?oooooooooo03cooooo1ol0o`0:ooooo`Oo00002?ooool3ool004Wooooo000=ooooo`Oo
o`001_ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooooooo03gooooo1Ol0o`0>ooooo`?o
000000?oooooo`000?l000000_l00003ooooo`Goo`00B?ooool000gooooo1ooo0005ooooo`03o`00
0?oooooooooo05[ooooo00?o0000ooooooooool0?Oooool3o`3o01;ooooo0ol00002ooooo`;o0000
0_oo0003o`3o00;oo`00Aoooool000gooooo1ooo0003o`3o0003ooooool0003ooooo05cooooo00?o
0000ooooooooool0EOooool3o`000003oooooooo003o0?l000Co0?l000?oo`00ooooooooool0AOoo
ool000kooooo1Ooo0005o`3o05oooooo00?o0000ooooooooool0Eoooool00ol0003o0?l0o`3o0005
o`3o00?o0000A?ooool000oooooo0ooo0007o`3o05oooooo00?o0000ooooooooool0Eoooool7o`3o
00;ooooo1Ol00010ooooo`004_ooool7o`3o05oooooo00?o0000ooooooooool0Eoooool7o`3o00Ko
oooo1_l0000kooooo`004_ooool7o`3o063ooooo00?o0000ooooooooool0Eoooool5o`3o00_ooooo
1_l00008ooooo`?o0?l0;?ooool001?ooooo1Ol0o`1Qooooo`03o`000?oooooooooo05Sooooo0ol0
o`0?ooooo`Oo00000oooool5o`3o02_ooooo000Dooooo`?o0?l0Hoooool00ol0003oooooooooo`1]
ooooo`Go00001ol0o`0Zooooo`00N_ooool00ol0003oooooooooo`1aooooo`03o`000?l0o`3o0?l0
00Go0?l0:_ooool007_ooooo00?o0000ooooooooool0LOooool7o`3o00?o00009oooool007_ooooo
00Co0000o`3o0?l0o`3o0?l0L?ooool7o`3o00?ooooo1?l0000Sooooo`00Noooool5o`3o073ooooo
1Ol0o`08ooooo`Co00007oooool007[ooooo1ol0o`1`ooooo`?o0?l03Oooool4o`0000Sooooo0ooo
000@ooooo`00N_ooool7o`3o08Cooooo1?l00003ooooo`Goo`003oooool007[ooooo1ol0o`28oooo
o`;o00001ooo000>ooooo`00Noooool5o`3o08_ooooo1ooo000>ooooo`00O?ooool3o`3o0003o`00
0?oooooooooo08Wooooo1ooo000>ooooo`00O_ooool3o`0008_ooooo1Ooo000?ooooo`00O_ooool2
o`000003ooooool0003ooooo08[ooooo0ooo000@ooooo`00O_ooool01Ol0003oooooo`000?oooooo
000009gooooo001nooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool0V_ooool007ko
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`2Iooooo`00O_ooool01?l0003o
ooooooooool00003ooooo`03o`000?oooooooooo09Sooooo001oooooo`04o`000?ooooooooooo`00
00?ooooo00?o0000ooooooooool0Uoooool007oooooo00Co0000oooooooooooo00001?ooool00ol0
003oooooooooo`2Fooooo`00Ooooool01Ol0003ooooooooooooooooo000000Cooooo00?o0000oooo
ooooool0UOooool007oooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo
09Cooooo001oooooo`05o`000?ooooooooooooooool000001_ooool00ol0003oooooooooo`2Coooo
o`00Ooooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool0T_ooool0083ooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooo0?l000;o
0?l0Soooool0083ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`Go0?l0
S_ooool0083ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`Oo0?l0SOoo
ool0083ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`Oo0?l0SOooool0
083ooooo00?o0000ooooooooool00oooool01?l0003oo`00ool00?oo0002ooooo`Oo0?l0SOooool0
083ooooo0ol0o`03ooooo`Goo`000_ooool5o`3o08kooooo001oooooo`Go0?l000?oooooool00?oo
00001Ooo0002ooooo`?o0?l000?o0000ooooooooool0S?ooool007kooooo1ol0o`07ool000Kooooo
00?o0000ooooooooool0Roooool007kooooo1ol0o`07ool000Oooooo00?o0000ooooooooool0R_oo
ool007kooooo1ol0o`000oooooooo`00ool00003ool000Wooooo00?o0000ooooooooool0ROooool0
07oooooo1Ol0o`03ooooo`?oo`002oooool00ol0003oooooooooo`28ooooo`00P?ooool3o`3o00Oo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`27ooooo`00R_ooool00ol0003ooooo
ooooo`0:ooooo`03o`000?oooooooooo08Kooooo002;ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool0QOooool008_ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`25
ooooo`00S?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo08Cooooo002<ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool0Poooool008gooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`22ooooo`00SOooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo087ooooo002>ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0P?ooool0
08kooooo00?o0000ooooooooool03Oooool00ol0003oooooo`3o0002o`3o07gooooo002?ooooo`03
o`000?oooooooooo00gooooo1Ol0o`1looooo`00Soooool00ol0003oooooooooo`0<ooooo`Oo0?l0
Noooool0093ooooo00?o0000ooooooooool02oooool7o`3o07_ooooo002@ooooo`03o`000?oooooo
oooo00_ooooo1ol0o`1kooooo`00TOooool00ol0003oooooooooo`0;ooooo`Go0?l0O?ooool0097o
oooo00Co0000o`3o0?l0o`3o0?l02oooool3o`3o0003o`000?oooooooooo07[ooooo002Aooooo`Go
0?l03_ooool00ol0003oooooooooo`1iooooo`00T?ooool7o`3o00kooooo00?o0000ooooooooool0
N?ooool0093ooooo1ol0o`0?ooooo`03o`000?oooooooooo07Oooooo002@ooooo`Oo0?l04?ooool0
0ol0003oooooooooo`1fooooo`00TOooool5o`3o01;ooooo00?o0000ooooooooool0MOooool009;o
oooo0ol0o`0Dooooo`03o`000?oooooooooo07Cooooo002Zooooo`03o`000?oooooooooo07?ooooo
002[ooooo`03o`000?oooooooooo07;ooooo002/ooooo`03o`000?oooooooooo077ooooo002]oooo
o`03o`000?oooooooooo073ooooo002^ooooo`03o`000?oooooooooo06oooooo002_ooooo`03o`00
0?oooooooooo06kooooo002`ooooo`03o`000?ooooooo`0000;oo`00Joooool00;7ooooo1Ooo001Z
ooooo`00/?ooool7ool006Wooooo002`ooooo`Ooo`00JOooool00;3ooooo1ooo001Yooooo`00/Ooo
ool5ool006[ooooo002booooo`?oo`00Joooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo
8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo
8Oooool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.158629, -0.0500053, \
0.00310585, 0.00376105}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(Evolve[bos4, Generations \[Rule] 20]\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["The following graph is a fixed point: ", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos4]];\)\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.177966 1.12187 0.0463215 0.926431 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.28427 .44711 m
.28427 .53246 L
s
.28427 .44711 m
.28427 .36175 L
s
.28427 .53246 m
.35736 .59281 L
s
.28427 .53246 m
.21118 .59281 L
s
.36338 .20404 m
.40293 .12518 L
s
.36338 .20404 m
.32382 .28289 L
s
.84452 .73379 m
.94391 .75723 L
s
.84452 .73379 m
.74513 .71036 L
s
.49939 .81825 m
.43647 .75053 L
s
.49939 .81825 m
.56231 .88596 L
s
.44249 .04632 m
.40293 .12518 L
s
.47602 .82939 m
.43647 .75053 L
s
.47602 .82939 m
.51558 .90824 L
s
.64622 .68558 m
.74513 .71036 L
s
.64622 .68558 m
.54834 .65814 L
s
.64622 .68558 m
.74296 .71564 L
s
.55533 .63662 m
.45285 .62548 L
s
.28427 .36175 m
.18878 .32909 L
s
.28427 .36175 m
.24472 .28289 L
s
.28427 .36175 m
.32382 .28289 L
s
.28427 .36175 m
.37976 .32909 L
s
.88186 .23533 m
.78048 .25198 L
s
.35736 .59281 m
.45285 .62548 L
s
.35736 .59281 m
.39691 .67167 L
s
.45285 .62548 m
.54834 .65814 L
s
.45285 .62548 m
.53485 .67744 L
s
.18443 .67526 m
.21118 .59281 L
s
.43647 .75053 m
.44996 .83515 L
s
.43647 .75053 m
.39691 .67167 L
s
.64105 .69589 m
.54834 .65814 L
s
.74513 .71036 m
.84354 .73647 L
s
.07517 .72089 m
.1381 .65317 L
s
.24472 .28289 m
.20516 .20404 L
s
.67911 .26863 m
.57773 .28528 L
s
.67911 .26863 m
.78048 .25198 L
s
.56231 .88596 m
.62523 .95368 L
s
.47526 .29642 m
.55726 .24446 L
s
.47526 .29642 m
.57075 .26376 L
s
.47526 .29642 m
.57773 .28528 L
s
.47526 .29642 m
.37976 .32909 L
s
.14774 .13307 m
.20516 .20404 L
s
.14774 .13307 m
.09031 .0621 L
s
.1381 .65317 m
.21118 .59281 L
s
.1381 .65317 m
.05609 .70513 L
s
.55726 .24446 m
.63926 .1925 L
s
.20516 .20404 m
.185 .12032 L
s
.68087 .2797 m
.57773 .28528 L
s
.11134 .61491 m
.21118 .59281 L
s
.63926 .1925 m
.72126 .14054 L
s
1 0 0 r
.025 w
.28427 .44711 Mdot
0 1 0 r
.28427 .53246 Mdot
1 0 0 r
.36338 .20404 Mdot
0 1 0 r
.84452 .73379 Mdot
.49939 .81825 Mdot
1 0 0 r
.44249 .04632 Mdot
0 1 0 r
.47602 .82939 Mdot
.64622 .68558 Mdot
1 0 0 r
.55533 .63662 Mdot
.28427 .36175 Mdot
.88186 .23533 Mdot
.35736 .59281 Mdot
.45285 .62548 Mdot
0 1 0 r
.18443 .67526 Mdot
.43647 .75053 Mdot
1 0 0 r
.64105 .69589 Mdot
0 1 0 r
.44996 .83515 Mdot
.94391 .75723 Mdot
.74513 .71036 Mdot
1 0 0 r
.18878 .32909 Mdot
.40293 .12518 Mdot
0 1 0 r
.07517 .72089 Mdot
1 0 0 r
.24472 .28289 Mdot
.67911 .26863 Mdot
0 1 0 r
.39691 .67167 Mdot
.56231 .88596 Mdot
1 0 0 r
.47526 .29642 Mdot
0 1 0 r
.14774 .13307 Mdot
.1381 .65317 Mdot
1 0 0 r
.55726 .24446 Mdot
0 1 0 r
.20516 .20404 Mdot
1 0 0 r
.68087 .2797 Mdot
0 1 0 r
.84354 .73647 Mdot
1 0 0 r
.54834 .65814 Mdot
0 1 0 r
.11134 .61491 Mdot
.21118 .59281 Mdot
1 0 0 r
.57075 .26376 Mdot
0 1 0 r
.74296 .71564 Mdot
1 0 0 r
.63926 .1925 Mdot
0 1 0 r
.185 .12032 Mdot
.51558 .90824 Mdot
1 0 0 r
.32382 .28289 Mdot
.57773 .28528 Mdot
0 1 0 r
.09031 .0621 Mdot
.05609 .70513 Mdot
1 0 0 r
.78048 .25198 Mdot
.72126 .14054 Mdot
0 1 0 r
.62523 .95368 Mdot
1 0 0 r
.37976 .32909 Mdot
.53485 .67744 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[31]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00OOooool3ool00:3ooooo001looooo`Goo`00Woooool007_ooooo1ooo002Nooooo`00Nooo
ool7ool009kooooo000Hooooo`?o0?l0H?ooool7ool009kooooo000Gooooo`Go0?l0H?ooool5ool0
09oooooo000Fooooo`Oo0?l0H?ooool3ool00:3ooooo000Fooooo`Oo0?l0Goooool00ol0003ooooo
ooooo`2Qooooo`005_ooool7o`3o05oooooo00?o0000ooooooooool0XOooool001Oooooo1Ol0o`1O
ooooo`03o`000?oooooooooo0:;ooooo000Hooooo`?o0?l000?o0000ooooooooool0GOooool00ol0
003oooooooooo`2Rooooo`007?ooool00ol0003oooooooooo`1Kooooo`03o`000?oooooooooo0:?o
oooo000Mooooo`03o`000?oooooooooo05[ooooo00?o0000ooooooooool0Xoooool001gooooo00?o
0000ooooooooool0FOooool00ol0003oooooooooo`2Tooooo`007_ooool00ol0003oooooooooo`1H
ooooo`03o`000?oooooooooo0:Cooooo000Oooooo`03o`000?oooooooooo05Kooooo00?o0000oooo
ooooool0YOooool0023ooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`2Uooooo`00
8Oooool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo0:Kooooo000Rooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool0Y_ooool002;ooooo00?o0000ooooooooool0DOooool0
0ol0003oooooooooo`2Wooooo`008oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo
0:Oooooo000Tooooo`03o`000?oooooooooo00gooooo0ol0o`0nooooo`03o`000?oooooooooo0:So
oooo000Uooooo`03o`000?oooooooooo00_ooooo1Ol0o`0jooooo`?oo`0000?o0000ooooooooool0
Z?ooool002Kooooo00?o0000ooooooooool02Oooool7o`3o03Sooooo1Ooo002Zooooo`009_ooool0
0ol0003oooooooooo`09ooooo`Oo0?l0=oooool7ool00:Wooooo000Wooooo`03o`000?oooooo0?l0
00;o0?l01_ooool7o`3o03Oooooo1ooo002Yooooo`00:?ooool5o`3o00Kooooo1Ol0o`0hooooo`Oo
o`00ZOooool002Oooooo1ol0o`06ooooo`?o0?l0>_ooool5ool005Oooooo0ooo001@ooooo`009ooo
ool7o`3o00Sooooo00?o0000ooooooooool0>Oooool3ool005Oooooo1Ooo001?ooooo`009oooool7
o`3o00Sooooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`1Gooooo`Ooo`00C_ooool0
02Sooooo1Ol0o`09ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0Eoooool7ool0
04kooooo000Yooooo`?o0?l000?o0000ooooooooool01oooool00ol0003oooooooooo`0gooooo`03
o`000?oooooooooo05Sooooo1ooo001>ooooo`00;Oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo03Kooooo00?o0000ooooooooool0F?ooool00ol0003oo`00ool00003ool004oooooo
000^ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0=Oooool00ol0003ooooooooo
o`1Gooooo`;o00000_ooool3ool0053ooooo000^ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool0=Oooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo05Gooooo000_oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`1Eoooo
o`;o0000F?ooool0033ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0dooooo`03
o`000?oooooooooo05Cooooo00?o0000ooooooooool0F?ooool0037ooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo05?ooooo0_l0001Kooooo`00<_oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0DOoo
ool2o`0005gooooo000booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`1@ooooo`03o`000?oooooooooo05gooooo000cooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`1?ooooo`;o0000H?ooool003Co
oooo00Go0000ooooooooooooooooo`00000cooooo`03o`000?oooooooooo04kooooo00?o0000oooo
ooooool0H?ooool003Gooooo00Go0000ooooooooooooooooo`00000aooooo`03o`000?oooooooooo
04Sooooo0ooo0002ooooo`;o0000Hoooool003Kooooo00Co0000oooooooooooo0000<Oooool00ol0
003oooooooooo`17ooooo`Goo`0000?o0000ooooooooool0Hoooool003Kooooo00Co0000oooooooo
oooo0000<?ooool00ol0003oooooooooo`17ooooo`Ooo`00IOooool003Oooooo00?o0000ooooool0
o`000_l0o`0[ooooo`?oo`0000?o0000ooooooooool0Aoooool7ool006Gooooo000hooooo`Go0?l0
:Oooool5ool004Wooooo1ooo001Uooooo`00=oooool7o`3o02Oooooo1ooo0017ooooo`;o00001Ooo
001Vooooo`00=oooool7o`3o02Oooooo1ooo0015ooooo`;o00000oooool3ool006Oooooo000goooo
o`Oo0?l09oooool7ool004Cooooo00?o0000ooooooooool0KOooool003Sooooo1Ol0o`0Yooooo`Go
o`00@oooool2o`00073ooooo000iooooo`?o0?l0:oooool3ool004?ooooo00?o0000ooooooooool0
L?ooool003cooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`12ooooo`;o0000Looo
ool003cooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`10ooooo`;o0000MOooool0
03gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo
057ooooo0ooo000Qooooo`00?Oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03ko
oooo0_l0001Cooooo`Goo`008?ooool003kooooo00?o0000ooooooooool08oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo05;ooooo1ooo000Oooooo`00?_ooool00ol0003ooooooooo
o`0Sooooo`03o`000?oooooooooo03Oooooo0ooo0002ooooo`;o0000EOooool7ool001oooooo000o
ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0=oooool5ool00003o`000?oooooo
oooo04oooooo1_l00007ool001oooooo000oooooo`03o`000?oooooooooo027ooooo00?o0000oooo
ooooool0=_ooool7ool003_ooooo0ooo000=ooooo`Ko00001oooool5ool0023ooooo0010ooooo`03
o`000?oooooooooo01oooooo00?o0000ooooooooool0=oooool7ool003[ooooo1Ooo0006ooooo`Ko
00003_ooool3ool0027ooooo0011ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0
>?ooool7ool003Wooooo1ooo0005o`0003Sooooo0011ooooo`03o`000?oooooooooo01gooooo00?o
0000ooooooooool0=oooool2o`0000Soo`00=oooool7ool003gooooo0012ooooo`03o`000?oooooo
oooo01_ooooo00?o0000ooooooooool0=oooool01?l0003oooooooooooooool8ool0033ooooo1_l0
0007ool003gooooo0012ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool0=Oooool2
o`0000Kooooo1ooo000Jooooo`?oo`003?ooool6o`0000Oooooo1Ooo000nooooo`00@oooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo03Gooooo00?o0000ooooooooool01_ooool7ool0
01Wooooo1Ooo0006ooooo`Go00003_ooool3ool003oooooo0013ooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool0<oooool2o`0000Sooooo00?o0000ool00?oo00001Ooo000Hooooo`Oo
o`001Ol0001Eooooo`00A?ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo03;ooooo
0_l00007ooooo`?o00000_ooool5ool001Sooooo00?o0000ool00?oo00001Ooo001Jooooo`00A?oo
ool01?l0003oo`00ool00?oo000Cooooo`?oo`0000?o0000ooooooooool0<Oooool00ol0003ooooo
ooooo`05ooooo`;o00001_ooool5ool0013ooooo1ol000000oooooooo`00ool00005ool005[ooooo
0014ooooo`Goo`004Oooool5ool0037ooooo0_l00005ooooo`?o00002Oooool5ool000Sooooo1ol0
0009ooooo`Ooo`00FOooool004?ooooo1ooo000?ooooo`Ooo`00;oooool00ol0003oooooooooo`02
ooooo`?o00002oooool7ool000Oo00001?ooool<o`0000Ooo`00FOooool004?ooooo1ooo000?oooo
o`Ooo`00;Oooool2o`0000;ooooo0ol0000=ooooo`03o`000?oo003oo`0000Goo`002ol0000<oooo
o`Ooo`00FOooool004?ooooo1ooo000?ooooo`Ooo`009oooool3ool000;ooooo00?o0000oooooooo
ool00_l00009ooooo`Oo000000?oooooool00?oo00001Ooo000Hooooo`Goo`00F_ooool004Cooooo
1Ooo000Aooooo`Goo`009oooool5ool000Co00001?ooool7o`0000Wooooo1Ooo000Jooooo`?oo`00
Foooool004Gooooo0ooo000Cooooo`?oo`009oooool7ool000Oo00004Oooool3ool007Wooooo0018
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0:?ooool7ool009Cooooo0018oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool09_ooool2o`0000Ooo`00U?ooool004Wo
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Tooooo`?o00000oooool5ool009Go
oooo0019ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool08Oooool3o`0000Oooooo
0ooo002Fooooo`00B_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01oooooo0ol0
002Sooooo`00B_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01cooooo0ol0002V
ooooo`00=Oooool3ool001?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Aoooo
o`?oo`001_ooool3o`000:Wooooo000dooooo`Goo`004_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo013ooooo1Ooo0002ooooo`?o0000[?ooool003?ooooo1ooo000Booooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool04?ooool7ool00003o`000?oooooooooo0:gooooo
000cooooo`Ooo`004_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo013ooooo1ooo
002`ooooo`00<oooool7ool000;o00004Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo013ooooo00?o0000ool00?oo00001Ooo002`ooooo`00=?ooool5ool000?ooooo0ol0000>oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03Oooool3o`0000;ooooo1Ooo002aoooo
o`00=Oooool3ool000Oooooo0ol0000<ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool02oooool3o`0000Kooooo0ooo002booooo`00@_ooool3o`0000Wooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`08ooooo`?o0000__ooool004Gooooo0ol00007ooooo`05o`000?oo
ooooooooooooool000002?ooool3o`000<7ooooo0018ooooo`?o00001?ooool01Ol0003oo`00ool0
0?oo003o000000Gooooo0ol00034ooooo`00Boooool3o`000003oooooooo003oo`0000?oo`000_oo
ool3o`000<Oooooo001>ooooo`Ooo`0000?o0000ooooooooool0b?ooool004kooooo1ooo003;oooo
o`00C_ooool7ool00<_ooooo001?ooooo`Goo`00c?ooool0053ooooo0ooo003=ooooo`00DOooool0
0ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003o
ooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003ooooooooo
o`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<oooo
o`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOoo
ool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0
003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003ooooo
ooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<
ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00DOooool00ol0003oooooooooo`3<ooooo`00
D?ooool3ool00<gooooo001?ooooo`Goo`00c?ooool004kooooo1ooo003;ooooo`00C_ooool7ool0
0<_ooooo001>ooooo`Ooo`00boooool004oooooo1Ooo003<ooooo`00D?ooool3ool00<gooooo001A
ooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03
o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oo
oooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo
0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo
001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aoooo
o`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`00
0?oooooooooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001Aooooo`03o`000?oooooo
oooo0<cooooo001Aooooo`03o`000?oooooooooo0<cooooo001@ooooo`?o0?l0cOooool004oooooo
1Ol0o`3<ooooo`00C_ooool7o`3o0<_ooooo001>ooooo`Oo0?l0boooool004kooooo1ol0o`3;oooo
o`00C_ooool00ol0003o0?l0o`3o0003o`3o0003o`000?oooooooooo0<Wooooo001=ooooo`03o`00
0?oooooooooo00?o0?l00_ooool00ol0003oooooooooo`38ooooo`00C?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo0<Oooooo001;ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool0a_ooool004Wooooo0_l0000=ooooo`;o0000a_ooool004Sooooo00?o0000oooooooo
ool03oooool00ol0003oooooooooo`33ooooo`00Aoooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo0<;ooooo0016ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0`Ooo
ool004Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`30ooooo`00A?ooool00ol0
003oooooooooo`0Gooooo`03o`000?oooooooooo0;oooooo0012ooooo`;o00006oooool2o`000;oo
oooo0011ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0_?ooool0043ooooo00?o
0000ooooooooool07oooool00ol0003oooooooooo`2kooooo`00>oooool3o`3o0003ooooool0003o
oooo02;ooooo00?o0000oooooooo00000_oo002hooooo`00>_ooool5o`3o02Gooooo1Ooo002goooo
o`00>Oooool7o`3o02?ooooo1ooo002fooooo`00>Oooool7o`3o02?ooooo1ooo002fooooo`00=Ooo
ool4o`0000Oo0?l08oooool7ool000;o0000]?ooool0033ooooo1Ol00004ooooo`03o`000?l0o`3o
0?l000?o0?l09Oooool5ool000?ooooo0ol0002aooooo`007_ooool3o`3o00[ooooo1Ol00008oooo
o`03o`000?oooooooooo00?o0?l09oooool3ool000Oooooo0ol0002^ooooo`007Oooool5o`3o00Co
oooo1Ol0000;ooooo`;o00000oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00Oo
oooo0ol0002[ooooo`007?ooool7o`3o00?o00003oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo02_ooooo00?o0000ooooooooool02_ooool3o`000:Sooooo000Looooo`Oo0?l04Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool03?oo
ool3o`0000Gooooo0ooo002Mooooo`007?ooool7o`3o013ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00oooooo0ol000000oooooooo`00ool00003
ool009cooooo000Mooooo`Go0?l03oooool2o`0000Kooooo00?o0000ooooooooool0;_ooool00ol0
003oooooooooo`0Aooooo`Ooo`00Voooool001kooooo0ol0o`0?ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0Aooooo`Ooo`000_l0000Gooooo`?o
o`00Ooooool002oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0_ooooo`03o`00
0?oooooooooo013ooooo1ooo000<o`0000cooooo1Ooo001nooooo`00;_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool04Oooool5ool000;o000000Co
ooooo`000?l0003o00001oooool:o`000003oooooooo003oo`0000Goo`00OOooool002gooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo017ooooo0ooo
0003ooooo`;o00000_ooool3o`0000kooooo00?o0000ool00?oo00001Ooo001mooooo`00:oooool2
o`0000_ooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo00;ooooo0ol0000<ooooo`Ooo`00OOooool002Kooooo0ol0o`000ooooooo0000ooooo`0;oooo
o`03o`000?oooooooooo03?ooooo00?o0000ooooooooool06Oooool2o`0000Gooooo0ol0000:oooo
o`Goo`00O_ooool002Gooooo1Ol0o`0=ooooo`03o`000?oooooooooo03?ooooo00?o0000oooooooo
ool06oooool00ol0003oooooooooo`05ooooo`?o00001_ooool5ool007oooooo000Tooooo`Oo0?l0
3?ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo01_ooooo0_l00008ooooo`?o0000
0_ooool5ool0083ooooo000Tooooo`Oo0?l02oooool00ol0003oooooooooo`0eooooo`03o`000?oo
oooooooo01gooooo0_l00009ooooo`03o`000?oo003oo`0000Goo`00Ooooool002Cooooo1ol0o`0;
ooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`07
ooooo`Ooo`00Ooooool002?ooooo0_l00005o`3o00cooooo00?o0000ooooooooool0=_ooool01?l0
003o0?l0o`3o0?l0o`0Nooooo`;o00001oooool7ool000;o0000OOooool002;ooooo00Co0000oooo
oooooooo00000ol0o`0:ooooo`?o0?l0>Oooool5o`3o01oooooo00?o0000ooooooooool01Oooool5
ool000;ooooo1Ol0001iooooo`008?ooool2o`0000;ooooo00?o0000ooooooooool02oooool5o`3o
03Oooooo1ol0o`0Oooooo`;o00000_ooool3ool00004oooooooo003oo`00ool000Gooooo0_l00000
1?oooooo0000o`000?l0001fooooo`007oooool01Ol0003ooooooooooooooooo000000gooooo1ol0
o`0fooooo`Oo0?l08Oooool00ol0003oo`00ool00003ool000[ooooo0ol000000ooooooo0000o`00
0002o`0007;ooooo000Mooooo`;o00000oooool00ol0003oooooooooo`0<ooooo`Oo0?l0=_ooool7
o`3o027ooooo1ooo000<ooooo`;o00000oooool3o`0000Oooooo0ol0o`1Uooooo`006oooool2o`00
00Cooooo00?o0000ooooooooool03Oooool7o`3o03Oooooo1Ol0o`0Rooooo`Ooo`003_ooool2o`00
00Cooooo1?l00002ooooo`Go0?l0I?ooool001[ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`0?ooooo`Go0?l0>Oooool3o`3o02?ooooo1ooo000@ooooo`?o00001Oooool00ol0003o
0?l0o`3o0005o`3o06?ooooo000Hooooo`;o00001Oooool00ol0003oooooooooo`0Aooooo`?o0?l0
?Oooool00ol0003oooooooooo`0Qooooo`Goo`005?ooool2o`0000Cooooo0ooo0004o`3o06?ooooo
000Gooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0D_ooool00ol0003ooooooooo
o`0Rooooo`?oo`005oooool2o`000003oooooooo003oo`0000?oo`000ol0o`03o`00063ooooo000E
ooooo`;o00001_ooool00ol0003oooooooooo`1Dooooo`03o`000?oooooooooo03gooooo1ooo0000
1?l0o`3oooooooooooooool5o`0005cooooo000?ooooo`?o0?l00_ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool0?Oooool7ool000Oooooo1_l0001H
ooooo`003_ooool5o`3o0003o`000?oooooooooo00Gooooo00?o0000ooooooooool0Eoooool00ol0
003oooooooooo`0looooo`Ooo`002_ooool7o`0000Sooooo0ol0o`19ooooo`003Oooool7o`3o00Ko
oooo00?o0000ooooooooool0F?ooool00ol0003oooooooooo`0mooooo`Goo`003_ooool3o`000003
ooooool0003o000000;o00000oooool5o`3o04Sooooo000=ooooo`Oo0?l01Oooool00ol0003ooooo
ooooo`1Jooooo`03o`000?oooooooooo03gooooo0ooo000Booooo`?o00000_ooool2o`0000Oo0?l0
Aoooool000gooooo2_l0o`000ooooooo0000ooooo`1Looooo`03o`000?oooooooooo05Gooooo0ol0
00000ooooooo0?l0o`3o0005o`3o04Oooooo000>ooooo`[o0?l0Goooool00ol0003oooooooooo`1G
ooooo`03o`000?l0o`3o0?l000Go0?l00ol00014ooooo`003oooool:o`3o05oooooo00?o0000oooo
ooooool0Eoooool7o`3o00;ooooo1Ol00010ooooo`004_ooool7o`3o05oooooo00?o0000oooooooo
ool0Eoooool7o`3o00Kooooo1_l0000kooooo`004_ooool7o`3o063ooooo00?o0000ooooooooool0
Eoooool5o`3o00_ooooo1_l00008ooooo`?o0?l0;?ooool001?ooooo1Ol0o`1Qooooo`03o`000?oo
oooooooo05Sooooo0ol0o`0?ooooo`Oo00000oooool5o`3o02_ooooo000Dooooo`?o0?l0Hoooool0
0ol0003oooooooooo`1]ooooo`Go00001ol0o`0Zooooo`00N_ooool00ol0003oooooooooo`1aoooo
o`03o`000?l0o`3o0?l000Go0?l0:_ooool007_ooooo00?o0000ooooooooool0LOooool7o`3o00?o
00009oooool007_ooooo00Co0000o`3o0?l0o`3o0?l0L?ooool7o`3o00?ooooo1?l0000Sooooo`00
Noooool5o`3o073ooooo1Ol0o`08ooooo`Co00007oooool007[ooooo1ol0o`1`ooooo`?o0?l03Ooo
ool4o`0000Sooooo0ol0o`0@ooooo`00N_ooool7o`3o08Cooooo1?l00003ooooo`Go0?l03oooool0
07[ooooo1ol0o`28ooooo`;o00001ol0o`0>ooooo`00Noooool5o`3o08_ooooo1ol0o`0>ooooo`00
O?ooool3o`3o0003o`000?oooooooooo08Wooooo1ol0o`0>ooooo`00O_ooool3o`0008_ooooo1Ol0
o`0?ooooo`00O_ooool2o`000003ooooool0003ooooo08[ooooo0ol0o`0@ooooo`00O_ooool01Ol0
003oooooo`000?oooooo000009gooooo001nooooo`03o`000?oooooo000000;ooooo00?o0000oooo
ooooool0V_ooool007kooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`2Ioooo
o`00O_ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo09Sooooo001oooooo`04
o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool0Uoooool007oooooo00Co0000oooooooo
oooo00001?ooool00ol0003oooooooooo`2Fooooo`00Ooooool01Ol0003ooooooooooooooooo0000
00Cooooo00?o0000ooooooooool0UOooool007oooooo00Go0000ooooooooooooooooo`000005oooo
o`03o`000?oooooooooo09Cooooo001oooooo`05o`000?ooooooooooooooool000001_ooool00ol0
003oooooooooo`2Cooooo`00Ooooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool0T_ooool0083ooooo00Go0000ooooooooooooooooo`000007ooooo`03
o`000?oooooo0?l000;o0?l0Soooool0083ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`05ooooo`Go0?l0S_ooool0083ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`04ooooo`Oo0?l0SOooool0083ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03
ooooo`Oo0?l0SOooool0083ooooo00?o0000ooooooooool00oooool01?l0003o0?l0o`3o0?l0o`02
ooooo`Oo0?l0SOooool0083ooooo0ol0o`03ooooo`Go0?l00_ooool5o`3o08kooooo001oooooo`Go
0?l000?oooooo`3o0?l0o`001Ol0o`02ooooo`?o0?l000?o0000ooooooooool0S?ooool007kooooo
3_l0o`06ooooo`03o`000?oooooooooo08_ooooo001nooooo`ko0?l01oooool00ol0003ooooooooo
o`2:ooooo`00O_ooool7o`3o0003ooooool0o`3o0?l000?o0?l02Oooool00ol0003oooooooooo`29
ooooo`00Ooooool5o`3o00?ooooo0ol0o`0;ooooo`03o`000?oooooooooo08Sooooo0020ooooo`?o
0?l01oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo08Oooooo002:ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool0Q_ooool008_ooooo00?o0000ooooooooool02_oo
ool00ol0003oooooooooo`25ooooo`00Roooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo08Gooooo002<ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0Q?ooool008co
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`23ooooo`00SOooool00ol0003ooooo
ooooo`0;ooooo`03o`000?oooooooooo08;ooooo002=ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool0POooool008kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`20
ooooo`00S_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo0?l000;o0?l0OOooool008oo
oooo00?o0000ooooooooool03Oooool5o`3o07cooooo002?ooooo`03o`000?oooooooooo00cooooo
1ol0o`1kooooo`00T?ooool00ol0003oooooooooo`0;ooooo`Oo0?l0Noooool0093ooooo00?o0000
ooooooooool02oooool7o`3o07_ooooo002Aooooo`03o`000?oooooooooo00_ooooo1Ol0o`1loooo
o`00TOooool01?l0003o0?l0o`3o0?l0o`0;ooooo`?o0?l000?o0000ooooooooool0N_ooool0097o
oooo1Ol0o`0>ooooo`03o`000?oooooooooo07Wooooo002@ooooo`Oo0?l03_ooool00ol0003ooooo
ooooo`1hooooo`00T?ooool7o`3o00oooooo00?o0000ooooooooool0Moooool0093ooooo1ol0o`0@
ooooo`03o`000?oooooooooo07Kooooo002Aooooo`Go0?l04_ooool00ol0003oooooooooo`1eoooo
o`00T_ooool3o`3o01Cooooo00?o0000ooooooooool0M?ooool00:[ooooo00?o0000ooooooooool0
Loooool00:_ooooo00?o0000ooooooooool0L_ooool00:cooooo00?o0000ooooooooool0LOooool0
0:gooooo00?o0000ooooooooool0L?ooool00:kooooo00?o0000ooooooooool0Koooool00:oooooo
00?o0000ooooooooool0K_ooool00;3ooooo00?o0000ooooool0o`000_l0o`1[ooooo`00/Oooool5
o`3o06[ooooo002`ooooo`Oo0?l0JOooool00;3ooooo1ol0o`1Yooooo`00/?ooool7o`3o06Wooooo
002aooooo`Go0?l0J_ooool00;;ooooo0ol0o`1[ooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.158629, -0.0500053, \
0.00310585, 0.00376105}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(DeleteModel[bos4];\)\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Nonisomorphic interaction and update neighborhoods", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "If ",
  Cell[BoxData[
      \(TraditionalForm\`G\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`G\_2\)]],
  " are two different graphs with the same number of nodes, we can use one of \
these graphs to specify the interaction neighborhood and the other the update \
neighborhood."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  StyleBox["GraphPower[", "Input"],
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  StyleBox[",", "Input"],
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  StyleBox["]", "Input"],
  " gives the ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  "th power of graph ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  ". This is the graph whose vertex set is identical to the vertex set of ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " and that contains an edge between vertices ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  " if ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " contains a path between ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  " of length, at most, ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  ". We can use this to create an update neighborhood in which agents have an \
update radius twice that of their interaction radius."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(intNbhd = Cycle[50];\)\), "\[IndentingNewLine]", 
    \(\(updNbhd = GraphPower[Cycle[50], 2];\)\)}], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "However, plotting ",
  StyleBox["updNbhd", "Input"],
  " reveals something you might not expect:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[updNbhd, PlotRange \[Rule] All];\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0330204 0.889485 0.0321582 0.891127 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 0 r
.005 w
[ ] 0 setdash
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.93 17.6423 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
49.4117 28.9325 1 0 365.73 arc
setmatrix s
.919 .53357 m
.90853 .58853 L
s
.919 .53357 m
.89127 .64174 L
s
.919 .53357 m
.919 .42188 L
s
.919 .53357 m
.92251 .47772 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.5971 19.387 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
48.8573 31.8404 1 0 365.73 arc
setmatrix s
.90853 .58853 m
.89127 .64174 L
s
.90853 .58853 m
.86749 .69237 L
s
.90853 .58853 m
.92251 .47772 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.0482 21.0759 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
47.9426 34.6552 1 0 365.73 arc
setmatrix s
.89127 .64174 m
.86749 .69237 L
s
.89127 .64174 m
.83757 .73962 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
28.2921 22.6829 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
46.6823 37.3335 1 0 365.73 arc
setmatrix s
.86749 .69237 m
.83757 .73962 L
s
.86749 .69237 m
.80197 .78273 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
27.3403 24.1826 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
45.0961 39.8325 1 0 365.73 arc
setmatrix s
.83757 .73962 m
.80197 .78273 L
s
.83757 .73962 m
.76125 .82103 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
26.2083 25.5509 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
43.2094 42.1135 1 0 365.73 arc
setmatrix s
.80197 .78273 m
.76125 .82103 L
s
.80197 .78273 m
.71607 .85392 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
24.9138 26.7666 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
41.0513 44.1396 1 0 365.73 arc
setmatrix s
.76125 .82103 m
.71607 .85392 L
s
.76125 .82103 m
.66712 .88088 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
23.4768 27.8105 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
38.6569 45.8795 1 0 365.73 arc
setmatrix s
.71607 .85392 m
.66712 .88088 L
s
.71607 .85392 m
.6152 .90148 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
21.9206 28.6662 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
36.0632 47.3056 1 0 365.73 arc
setmatrix s
.66712 .88088 m
.6152 .90148 L
s
.66712 .88088 m
.5611 .91539 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
20.2693 29.32 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
33.311 48.3949 1 0 365.73 arc
setmatrix s
.6152 .90148 m
.5611 .91539 L
s
.6152 .90148 m
.50569 .92241 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
18.5493 29.7615 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
30.4439 49.1312 1 0 365.73 arc
setmatrix s
.5611 .91539 m
.50569 .92241 L
s
.5611 .91539 m
.44984 .92241 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
16.7874 29.984 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
27.5073 49.502 1 0 365.73 arc
setmatrix s
.50569 .92241 m
.44984 .92241 L
s
.50569 .92241 m
.39443 .91539 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
15.0111 29.984 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
24.5474 49.502 1 0 365.73 arc
setmatrix s
.44984 .92241 m
.39443 .91539 L
s
.44984 .92241 m
.34033 .90148 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
13.2492 29.7615 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
21.6108 49.1312 1 0 365.73 arc
setmatrix s
.39443 .91539 m
.34033 .90148 L
s
.39443 .91539 m
.2884 .88088 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
11.5292 29.32 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
18.7436 48.3949 1 0 365.73 arc
setmatrix s
.34033 .90148 m
.2884 .88088 L
s
.34033 .90148 m
.23946 .85392 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
9.87788 28.6662 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
15.9915 47.3056 1 0 365.73 arc
setmatrix s
.2884 .88088 m
.23946 .85392 L
s
.2884 .88088 m
.19427 .82103 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
8.32166 27.8105 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
13.3978 45.8795 1 0 365.73 arc
setmatrix s
.23946 .85392 m
.19427 .82103 L
s
.23946 .85392 m
.15356 .78273 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
6.88469 26.7666 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
11.0033 44.1396 1 0 365.73 arc
setmatrix s
.19427 .82103 m
.15356 .78273 L
s
.19427 .82103 m
.11796 .73962 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
5.59017 25.5509 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
8.84528 42.1135 1 0 365.73 arc
setmatrix s
.15356 .78273 m
.11796 .73962 L
s
.15356 .78273 m
.08803 .69237 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
4.45815 24.1826 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
6.95857 39.8325 1 0 365.73 arc
setmatrix s
.11796 .73962 m
.08803 .69237 L
s
.11796 .73962 m
.06425 .64174 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
3.50642 22.6829 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
5.37235 37.3335 1 0 365.73 arc
setmatrix s
.08803 .69237 m
.06425 .64174 L
s
.08803 .69237 m
.04699 .58853 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
2.75025 21.0759 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
4.11207 34.6552 1 0 365.73 arc
setmatrix s
.06425 .64174 m
.04699 .58853 L
s
.06425 .64174 m
.03653 .53357 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
2.20141 19.387 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
3.19787 31.8404 1 0 365.73 arc
setmatrix s
.04699 .58853 m
.03653 .53357 L
s
.04699 .58853 m
.03302 .47772 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
1.86848 17.6423 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
2.64298 28.9325 1 0 365.73 arc
setmatrix s
.03653 .53357 m
.03302 .47772 L
s
.03653 .53357 m
.03653 .42188 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
1.75718 15.8699 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
2.45696 25.9786 1 0 365.73 arc
setmatrix s
.03302 .47772 m
.03653 .42188 L
s
.03302 .47772 m
.04699 .36691 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
1.86848 14.0976 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
2.64298 23.0241 1 0 365.73 arc
setmatrix s
.03653 .42188 m
.04699 .36691 L
s
.03653 .42188 m
.06425 .3137 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
2.20141 12.3528 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
3.19787 20.1167 1 0 365.73 arc
setmatrix s
.04699 .36691 m
.06425 .3137 L
s
.04699 .36691 m
.08803 .26307 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
2.75025 10.664 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
4.11207 17.3019 1 0 365.73 arc
setmatrix s
.06425 .3137 m
.08803 .26307 L
s
.06425 .3137 m
.11796 .21583 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
3.50642 9.05698 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
5.37235 14.6236 1 0 365.73 arc
setmatrix s
.08803 .26307 m
.11796 .21583 L
s
.08803 .26307 m
.15356 .17271 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
4.45815 7.55727 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
6.95857 12.1241 1 0 365.73 arc
setmatrix s
.11796 .21583 m
.15356 .17271 L
s
.11796 .21583 m
.19427 .13441 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
5.59017 6.18896 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
8.84528 9.8436 1 0 365.73 arc
setmatrix s
.15356 .17271 m
.19427 .13441 L
s
.15356 .17271 m
.23946 .10152 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
6.88469 4.97332 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
11.0033 7.81753 1 0 365.73 arc
setmatrix s
.19427 .13441 m
.23946 .10152 L
s
.19427 .13441 m
.2884 .07456 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
8.32166 3.9294 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
13.3978 6.07766 1 0 365.73 arc
setmatrix s
.23946 .10152 m
.2884 .07456 L
s
.23946 .10152 m
.34033 .05397 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
9.87788 3.07369 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
15.9915 4.65148 1 0 365.73 arc
setmatrix s
.2884 .07456 m
.34033 .05397 L
s
.2884 .07456 m
.39443 .04005 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
11.5292 2.41985 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
18.7436 3.56174 1 0 365.73 arc
setmatrix s
.34033 .05397 m
.39443 .04005 L
s
.34033 .05397 m
.44984 .03304 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
13.2492 1.97835 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
21.6108 2.82591 1 0 365.73 arc
setmatrix s
.39443 .04005 m
.44984 .03304 L
s
.39443 .04005 m
.50569 .03304 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
15.0111 1.75585 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
24.5474 2.45455 1 0 365.73 arc
setmatrix s
.44984 .03304 m
.50569 .03304 L
s
.44984 .03304 m
.5611 .04005 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
16.7874 1.75585 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
27.5073 2.45455 1 0 365.73 arc
setmatrix s
.50569 .03304 m
.5611 .04005 L
s
.50569 .03304 m
.6152 .05397 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
18.5493 1.97835 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
30.4439 2.82591 1 0 365.73 arc
setmatrix s
.5611 .04005 m
.6152 .05397 L
s
.5611 .04005 m
.66712 .07456 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
20.2693 2.41985 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
33.311 3.56174 1 0 365.73 arc
setmatrix s
.6152 .05397 m
.66712 .07456 L
s
.6152 .05397 m
.71607 .10152 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
21.9206 3.07369 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
36.0632 4.65148 1 0 365.73 arc
setmatrix s
.66712 .07456 m
.71607 .10152 L
s
.66712 .07456 m
.76125 .13441 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
23.4768 3.9294 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
38.6569 6.07766 1 0 365.73 arc
setmatrix s
.71607 .10152 m
.76125 .13441 L
s
.71607 .10152 m
.80197 .17271 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
24.9138 4.97332 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
41.0513 7.81753 1 0 365.73 arc
setmatrix s
.76125 .13441 m
.80197 .17271 L
s
.76125 .13441 m
.83757 .21583 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
26.2083 6.18896 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
43.2094 9.8436 1 0 365.73 arc
setmatrix s
.80197 .17271 m
.83757 .21583 L
s
.80197 .17271 m
.86749 .26307 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
27.3403 7.55727 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
45.0961 12.1241 1 0 365.73 arc
setmatrix s
.83757 .21583 m
.86749 .26307 L
s
.83757 .21583 m
.89127 .3137 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
28.2921 9.05698 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
46.6823 14.6236 1 0 365.73 arc
setmatrix s
.86749 .26307 m
.89127 .3137 L
s
.86749 .26307 m
.90853 .36691 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.0482 10.664 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
47.9426 17.3019 1 0 365.73 arc
setmatrix s
.89127 .3137 m
.90853 .36691 L
s
.89127 .3137 m
.919 .42188 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.5971 12.3528 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
48.8573 20.1167 1 0 365.73 arc
setmatrix s
.90853 .36691 m
.919 .42188 L
s
.90853 .36691 m
.92251 .47772 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
29.93 14.0976 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
49.4117 23.0241 1 0 365.73 arc
setmatrix s
.919 .42188 m
.92251 .47772 L
s
newpath
matrix currentmatrix
0.031448 0.0315061 scale
30.0413 15.8699 1 0 365.73 arc
setmatrix s
newpath
matrix currentmatrix
0.0188688 0.0189037 scale
49.5977 25.9786 1 0 365.73 arc
setmatrix s
.025 w
.919 .53357 Mdot
.90853 .58853 Mdot
.89127 .64174 Mdot
.86749 .69237 Mdot
.83757 .73962 Mdot
.80197 .78273 Mdot
.76125 .82103 Mdot
.71607 .85392 Mdot
.66712 .88088 Mdot
.6152 .90148 Mdot
.5611 .91539 Mdot
.50569 .92241 Mdot
.44984 .92241 Mdot
.39443 .91539 Mdot
.34033 .90148 Mdot
.2884 .88088 Mdot
.23946 .85392 Mdot
.19427 .82103 Mdot
.15356 .78273 Mdot
.11796 .73962 Mdot
.08803 .69237 Mdot
.06425 .64174 Mdot
.04699 .58853 Mdot
.03653 .53357 Mdot
.03302 .47772 Mdot
.03653 .42188 Mdot
.04699 .36691 Mdot
.06425 .3137 Mdot
.08803 .26307 Mdot
.11796 .21583 Mdot
.15356 .17271 Mdot
.19427 .13441 Mdot
.23946 .10152 Mdot
.2884 .07456 Mdot
.34033 .05397 Mdot
.39443 .04005 Mdot
.44984 .03304 Mdot
.50569 .03304 Mdot
.5611 .04005 Mdot
.6152 .05397 Mdot
.66712 .07456 Mdot
.71607 .10152 Mdot
.76125 .13441 Mdot
.80197 .17271 Mdot
.83757 .21583 Mdot
.86749 .26307 Mdot
.89127 .3137 Mdot
.90853 .36691 Mdot
.919 .42188 Mdot
.92251 .47772 Mdot
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[5]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00P?ooool3o`0000;ooooo1?l00007ooooo`?o00000_ooool4o`0008Oo
oooo001oooooo`So00000_ooool3o`0000?ooooo2?l00002ooooo`?o0000Q?ooool0073ooooo0ol0
0002ooooo`Go00001?ooool7o`0000;ooooo0_l00003ooooo`03o`000?oooooo000000Ko00000_oo
ool2o`0000?ooooo00?o0000ooooooooool00ol00003ooooo`Co0000Moooool006oooooo2?l00003
ooooo`;o000000?oooooo`000?l000005ol00003ooooo`04o`000?ooooooooooooooo`[o00000_oo
ool3o`0007Cooooo001^ooooocKo00000oooool2o`0000?ooooo00?o0000ooooooooool0LOooool0
06kooooo3?l00003ooooo`03o`000?oooooo000000Co00001_ooool00ol0003oooooooooo`02oooo
o`Go00001_ooool:o`0000Gooooo00Go0000ooooooooooooooooo`00001booooo`00H?ooool3o`00
00?ooooo1?l000000ooooooo0000o`000008o`0000Cooooo00?o0000ooooooooool00_ooool5o`00
00Oooooo00?o0000ooooooooool00_ooool4o`0000Oooooo00?o0000ooooooooool02ol000000ooo
oooo0000ooooo`03ooooo`Co00000oooool4o`0006Oooooo001Oooooo`oo000000?oooooo`000?l0
00000ol00006ooooo`04o`000?ooooooooooooooo`;o00002_ooool01?l0003oooooooooooooool3
o`0000Wooooo00Co0000oooooooooooooooo1Ol000000ooooooo0000o`000005o`0000?ooooo2?l0
0003ooooo`?o0000I?ooool005kooooo2ol00003ooooo`03o`000?oooooooooo00Co00001oooool0
1?l0003oooooooooooooool2o`0000[ooooo00Co0000oooooooooooooooo0ol00009ooooo`03o`00
0?oooooooooo00;ooooo0ol00007ooooo`_o00000_ooool2o`0000?ooooo00?o0000ooooooooool0
H_ooool005kooooo2?l00003ooooo`04o`000?ooooooooooooooo`?o00002_ooool01?l0003ooooo
ooooooooool3o`0000Sooooo00?o0000ooooooooool00_ooool3o`0000Sooooo00?o0000oooooooo
ool00_ooool3o`0000Wooooo00?o0000ooooooooool01ol00004ooooo`05o`000?oooooooooooooo
ool00000Hoooool005gooooo2?l00004ooooo`03o`000?oooooooooo00;ooooo0_l0000:ooooo`04
o`000?ooooooooooooooo`?o00002?ooool00ol0003oooooooooo`02ooooo`?o00002?ooool00ol0
003oooooooooo`02ooooo`?o00002Oooool00ol0003oooooooooo`08o`0000?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`1Pooooo`00Foooool9o`0000Kooooo00Co0000oooooooo
oooooooo0_l0000:ooooo`06o`000?ooooooooooooooool0003ooooo0_l00005ooooo`;o00001Ooo
ool01?l0003oooooo`000?l00005ooooo`;o00001Oooool3o`0000Sooooo00?o0000ooooooooool0
0_ooool9o`0000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`1Oooooo`00DOoo
ool3o`0000?ooooo1_l00002ooooo`Co00001oooool01?l0003oooooooooooooool3o`0000Sooooo
00?o0000ooooooooool00oooool2o`000003ooooool0003o000000?o00002?ooool2o`000003oooo
ool0003o000000?o00001oooool3o`0000Sooooo00?o0000ooooooooool00_ooool4o`0000Cooooo
1?l00004ooooo`?o00000_ooool5o`0005Sooooo001@ooooo`[o000000?oooooo`000?l000001?l0
0009ooooo`06o`000?ooooooooooooooool0003ooooo0_l00006ooooo`03o`000?oooooooooo00Co
oooo0_l0000>ooooo`03o`000?oooooooooo00cooooo1?l00005ooooo`;o00001Oooool00ol0003o
ooooo`000007ooooo`Co000000?oooooo`000?l000001_l00003ooooo`?o0000EOooool004oooooo
2Ol000000ooooooo0000ooooo`04ooooo`;o00002Oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooo000000Ko00001oooool2o`0000kooooo00?o0000ooooooooool03?ooool2o`0000;ooooo
1Ol00007ooooo`03o`000?oooooo000000Sooooo2_l00002ooooo`;o00001?ooool00ol0003ooooo
ooooo`1Booooo`00Coooool7o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Wooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`00000=ooooo`03o`000?oooooooooo00cooooo0_l0
000>ooooo`03o`000?oooooooooo00cooooo00?o0000ooooool000002Oooool00ol0003ooooooooo
o`07o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`1Aooooo`00Coooool7
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Wooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`00000=ooooo`?o00002_ooool2o`0000;ooooo0_l0000:ooooo`;o000000?ooooo
o`000?ooool03?ooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo00Oo00001?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo057ooooo001>ooooo`Oo00001_ooool01_l0
003ooooooooooooooooo0000ooooo`;o00001Oooool2o`0000Gooooo0_l0000=ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo
00Go0000ooooooooooooooooo`00000>ooooo`04o`000?oooooo0000o`0000Gooooo0_l00004oooo
o`Oo00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo053ooooo001<ooooo`?o
00000_ooool3o`0000Oooooo00So0000ooooooooooooooooo`000?oooooo0000ooooo`Go00002?oo
ool2o`0000[ooooo0_l00005ooooo`So00002?ooool8o`0000Gooooo0_l0000:ooooo`?o00000ooo
ool5o`0000Kooooo1?l00002ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo053ooooo001:ooooo`?o00000oooool2o`0000Wooooo0_l00002ooooo`03o`000?oooooo
000000kooooo00?o0000ooooool000002?ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooo
oooo00Kooooo00Co0000oooooooooooo00003_ooool00ol0003oooooo`000005ooooo`Co00001Ooo
ool00ol0003oooooooooo`1?ooooo`00@oooool3o`000003ooooool0003o000000Ko000000?ooooo
o`000?l000002_ooool01ol0003ooooooooooooooooo0000ooooool000003Oooool00ol0003ooooo
ooooo`08o`0002Sooooo2?l00004ooooo`03o`000?oooooooooo00_ooooo0ol00007ooooo`?o0000
0_ooool3o`000003ooooool0003o000000Ko0000B?ooool004;ooooo2Ol00004ooooo`?o00002Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool0?Ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooo000000Wooooo2ol00005ooooo`03o`000?oo
oooooooo04Gooooo0011ooooo`Oo00000oooool00ol0003oooooooooo`02ooooo`?o00002?ooool0
0ol0003oooooooooo`03ooooo`;o00002_ooool2o`00047ooooo0_l0000:ooooo`;o000000?ooooo
o`000?ooool01oooool00ol0003oooooo`000006o`0000;ooooo0_l00004ooooo`03o`000?oooooo
oooo04Cooooo0011ooooo`Oo00001?ooool01_l0003ooooooooooooooooo0000ooooo`;o00001Ooo
ool2o`0000Oooooo0_l00008ooooo`03o`000?oooooooooo04?ooooo00?o0000ooooooooool01_oo
ool2o`0000?ooooo00?o0000ooooooooool00oooool2o`0000;ooooo1ol00004ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool0@oooool0047ooooo1ol00004ooooo`08o`000?oooooo
ooooooooool0003oooooo`000?ooool5o`0000Wooooo00?o0000ooooool000001ol00017ooooo`So
00000_ooool01?l0003oooooooooooooool5o`0000Cooooo1ol00004ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool0@oooool0043ooooo1ol00006ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo
00_ooooo1ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0@_ooool003oo
oooo0_l000000ooooooo0000o`000002o`0000Oooooo00Oo0000ooooooooooooooooo`000?oooooo
000000gooooo00?o0000ooooooooool0F_ooool00ol0003oooooooooo`0:ooooo`Co00000_ooool2
o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`11ooooo`00?Oooool3o`00
00;ooooo0_l00009ooooo`03o`000?oooooooooo00;ooooo0_l0000<ooooo`03o`000?oooooooooo
05cooooo00?o0000ooooooooool02Oooool00ol0003oooooo`000004ooooo`?o000000?oooooo`00
0?ooool01?ooool00ol0003oooooooooo`11ooooo`00??ooool3o`0000?ooooo0_l00009ooooo`03
o`000?oooooooooo00?ooooo0_l00009ooooo`;o0000H?ooool00ol0003oooooooooo`08ooooo`;o
00001oooool3o`0000Gooooo00?o0000ooooooooool0@Oooool003Kooooo0ol00002ooooo`Go0000
0_ooool2o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo067ooooo00?o0000ooooooooool01_ooool00ol0003oooooo`000008ooooo`;o0000
0_ooool4o`000003ooooool0003o000000?o0000?Oooool003Gooooo1ol00004ooooo`?o000000?o
ooooo`000?l000001_ooool00ol0003oooooooooo`05ooooo`Wo0000IOooool9o`0000;ooooo0_l0
0006ooooo`Wo00001Oooool2o`0003_ooooo000dooooo`_o00000oooool2o`0000;ooooo0_l00003
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0M?ooool00ol0003oooooooooo`02
ooooo`;o00000oooool00ol0003oooooooooo`0;o`0000?ooooo00?o0000ooooooooool0>?ooool0
03Cooooo1ol00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool3o`0000Sooooo00?o
0000ooooooooool0MOooool00ol0003oooooooooo`03ooooo`?o00000oooool7o`0000Cooooo00Go
0000ooooooooooooooooo`00000iooooo`00=?ooool7o`0000Cooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo07Gooooo00?o0000ooooooooool0
2Oooool7o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0fooooo`00=Ooo
ool5o`0000Kooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo07Kooooo00?o0000ooooooooool02Oooool6o`0000Gooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`0eooooo`00=?ooool5o`0000Oooooo00Co0000oooooooooooooooo
0ol0000;ooooo`03o`000?oooooooooo07Sooooo0_l00009ooooo`Oo00001?ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo03Gooooo000cooooo`Co00002Oooool2o`0000Cooooo00?o
0000ooooooooool01oooool2o`0007gooooo00?o0000ooooooooool01_ooool3o`0000;ooooo0ol0
0003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0=?ooool003;ooooo0_l00000
0ooooooo0000o`00000:ooooo`03o`000?oooooooooo00;ooooo0ol00004ooooo`?o0000P?ooool0
0ol0003oooooooooo`03ooooo`?o00001Oooool3o`0000;ooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`0dooooo`00<Oooool2o`0000;ooooo0_l00009ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooool000000ol00024ooooo`Go000000Coooooo`000?oooooo00001Oooool3
o`000003ooooool0003ooooo00Cooooo00?o0000ooooooooool0=?ooool0033ooooo0_l00003oooo
o`;o00002Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo08gooooo00?o0000oooo
ool000001_ooool4o`0000Gooooo00?o0000ooooooooool0=?ooool002_ooooo0ol000000ooooooo
0000o`000003o`000003ooooool0003ooooo00;o00001Oooool2o`0000Kooooo00?o0000oooooooo
ool0S_ooool00ol0003oooooooooo`02o`0000Gooooo0ol000001Ooooooo0000o`000?l0003ooooo
00Go0000<_ooool002[ooooo2?l00002ooooo`?o00000_ooool5o`0000Sooooo00?o0000oooooooo
ool0S_ooool00ol0003oooooooooo`02ooooo`Go000000?oooooo`000?ooool02?l00002ooooo`?o
0000;oooool002Wooooo1ol00002ooooo`;o000000Coooooo`000?oooooo00003_ooool00ol0003o
ooooooooo`2?ooooo`03o`000?oooooooooo00Sooooo1ol00002ooooo`;o00000oooool00ol0003o
ooooooooo`0/ooooo`00:Oooool7o`0000Cooooo00Go0000ooooool0003oooooo`00000=ooooo`03
o`000?oooooooooo093ooooo00?o0000ooooooooool01oooool7o`0000Cooooo00Go0000oooooooo
ooooooooo`00000]ooooo`00:Oooool7o`0000Cooooo00?o0000ooooool000000_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo097ooooo00?o0000ooooooooool01oooool7o`0000Co
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Zooooo`00:_ooool5o`0000Kooooo
00Ko0000ooooool0003o0000ooooool00008ooooo`;o0000UOooool2o`0000Sooooo1Ol00006oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0:Oooool002[ooooo1?l00007ooooo`04
o`000?ooooooooooooooo`;o00001oooool00ol0003oooooooooo`2Gooooo`03o`000?oooooooooo
00Gooooo1Ol00006ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0:Oooool002Wo
oooo0_l0000:ooooo`03o`000?oooooooooo00;ooooo0ol000000ooooooo0000o`000002o`0009_o
oooo0ol000000ooooooo0000o`000002o`0000?ooooo0_l00005ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool0:?ooool002Sooooo0ol0000:ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooool00000X_ooool00ol0003oooooooooo`02o`0000Cooooo0_l00004ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool0:?ooool002Oooooo1?l00009ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool0YOooool2o`0000Gooooo0_l00002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool0:?ooool002Kooooo0_l000000ooooooo0000o`000009
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0YOooool00ol0003oooooo`000005
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0:?ooool002Gooooo00Ko0000
ooooool0003oooooo`000?ooool2o`0000Gooooo0_l00007ooooo`03o`000?oooooooooo0:Gooooo
00Co0000oooooooooooo00001?ooool3o`0000Oooooo00?o0000ooooooooool0:?ooool002Cooooo
00?o0000ooooool000000_ooool01?l0003oooooooooooooool5o`0000Wooooo00?o0000oooooooo
ool0YOooool01?l0003oooooooooooooool7o`0000Oooooo00?o0000ooooooooool0:?ooool0023o
oooo3?l0000>ooooo`03o`000?oooooooooo0:Oooooo00?o0000ooooooooool01_ooool5o`000003
ooooool0003o000000Go00009_ooool001oooooo2_l000000ooooooo0000ooooo`02o`0000cooooo
00?o0000ooooooooool0Yoooool00ol0003oooooooooo`06ooooo`[o00000oooool2o`0002Cooooo
000Nooooo`Oo00000oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool0Z?ooool00ol0003oooooooooo`06ooooo`Oo00000oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo027ooooo000Nooooo`Oo00001?ooool01?l0003oooooo`000?l0
0002ooooo`03o`000?oooooooooo00Gooooo0_l0002/ooooo`;o00001_ooool7o`0000Cooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0Pooooo`007_ooool7o`0000Cooooo00Oo0000
ooooooooooooooooo`000?oooooo000000Kooooo00?o0000ooooooooool0[_ooool00ol0003ooooo
ooooo`03ooooo`Oo00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo023ooooo
000Oooooo`Go00001_ooool01?l0003oooooooooooooool4o`000004ooooool0003o0000o`000;;o
oooo1?l000000ooooooo0000o`000004o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`0Oooooo`007oooool4o`0000Oooooo0_l00004ooooo`03o`000?oooooo00000;Wooooo
00?o0000ooooool000001?l00006ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
7_ooool001kooooo0ol0000:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0^ooo
ool00ol0003oooooooooo`03o`0000Gooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`0Nooooo`007_ooool3o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`2i
ooooo`03o`000?oooooo000000;ooooo0_l00005ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool07_ooool001gooooo1?l00009ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool0^Oooool00ol0003oooooo`000003ooooo`;o00000oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo01kooooo000Mooooo`?o000000?oooooo`000?l000001Oooool2o`0000Oo
oooo00?o0000ooooooooool0^Oooool01?l0003oooooooooool00003ooooo`04o`000?oooooooooo
o`0000Oooooo00?o0000ooooooooool07_ooool001cooooo0_l000000ooooooo0000ooooo`02oooo
o`;o000000?oooooo`000?l000002Oooool00ol0003oooooooooo`2iooooo`04o`000?oooooooooo
ooooo`Ko00002?ooool00ol0003oooooooooo`0Nooooo`007?ooool01?l0003oooooooooool00005
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0^oooool00ol0003oooooooooo`03
ooooo`;o00002Oooool00ol0003oooooooooo`0Nooooo`006?ooool:o`0000kooooo00?o0000oooo
ooooool0^oooool00ol0003oooooooooo`04ooooo`Go00000oooool4o`0001oooooo000Gooooo`So
000000?oooooo`000?ooool00_l0000;ooooo`03o`000?oooooooooo0;gooooo00?o0000oooooooo
ool01?ooool9o`0000;ooooo0ol0000Looooo`005_ooool7o`0000;ooooo1?l000000ooooooo0000
ooooo`07ooooo`;o0000`Oooool2o`0000?ooooo2Ol000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooooooo01Wooooo000Fooooo`Oo00001?ooool01Ol0003oooooo`000?oooooo000000Kooooo
00?o0000ooooooooool0`oooool00ol0003oooooooooo`07o`0000Cooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`0Hooooo`005_ooool7o`0000Cooooo00?o0000ooooooooool00ol0
0003ooooo`;o0000aoooool00ol0003oooooo`000006o`0000Cooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0Gooooo`005oooool5o`0000Kooooo00Co0000oooooooooooooooo1?l0
003:ooooo`Oo00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo01Kooooo000G
ooooo`Co00001oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo0<gooooo0ol00007
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool05_ooool001Kooooo0ol00009oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0boooool2o`000003ooooool0003o0000
00Oooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Fooooo`005_ooool2o`0000[o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3;ooooo`;o00000_ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool05Oooool001Kooooo
0ol00009ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0boooool2o`0000;ooooo
0_l00005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool05Oooool001Kooooo00?o
0000ooooool000002?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo0<_ooooo0_l0
0003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Eooooo`005Oooool2o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo0<_ooooo00Go0000ooooool0003o0000ooooo`02o`0000;ooooo0_l0
0007ooooo`03o`000?oooooooooo01Gooooo000Eooooo`?o00000_ooool6o`0000Sooooo00?o0000
ooooooooool0boooool01?l0003oooooooooooooool5o`0000Wooooo00?o0000ooooooooool05Ooo
ool001Gooooo00?o0000ooooool000003oooool00ol0003oooooooooo`3<ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Fooooo`005?ooool2o`00
0003ooooool0003ooooo00kooooo00?o0000ooooooooool0cOooool00ol0003oooooooooo`02oooo
o`;o00002Oooool00ol0003oooooooooo`0Fooooo`004Oooool;o`0000[ooooo00?o0000oooooooo
ool0c_ooool00ol0003oooooooooo`03ooooo`Co000000?oooooo`000?l000001_l0000Fooooo`00
4?ooool8o`000004ooooool0003oooooooooo`;o00001_ooool2o`000=;ooooo0_l00003ooooo`Wo
00001?ooool2o`0001Cooooo000?ooooo`Oo00000_ooool3o`0000?ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`3Dooooo`03o`000?oooooo000000Oo00000_ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo017ooooo000?ooooo`Oo00001?ooool9o`000=Sooooo2?l0
0004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool04?ooool000oooooo1ol00004
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0f_ooool7o`0000Cooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`0@ooooo`004?ooool5o`0000Kooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`3Jooooo`Go00001_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00oooooo000@ooooo`Co00001oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo0=[ooooo0ol00007ooooo`;o00001?ooool00ol0003oooooooooo`0?ooooo`00
4?ooool3o`0000Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3Iooooo`Co0000
2?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00oooooo000?ooooo`?o00002Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0=Wooooo0_l000000ooooooo0000o`00
0006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03_ooool000oooooo0ol00009
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0fOooool2o`000003ooooool0003o
000000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0>ooooo`003oooool2o`00
0003ooooool0003o000000Gooooo0_l00006ooooo`03o`000?oooooooooo0=Wooooo00?o0000oooo
ool000000_l00004ooooo`;o00001oooool00ol0003oooooooooo`0>ooooo`003oooool2o`0000?o
oooo0_l000000ooooooo0000o`000008ooooo`03o`000?oooooooooo0=Wooooo00Co0000oooooooo
oooooooo0_l000000ooooooo0000o`000009ooooo`03o`000?oooooooooo00kooooo000?ooooo`;o
00001Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo0=Wooooo00Co0000oooooooo
oooooooo0ol0000:ooooo`03o`000?oooooooooo00oooooo000?ooooo`;o00004?ooool00ol0003o
ooooooooo`3Iooooo`04o`000?ooooooooooooooo`;o00002oooool00ol0003oooooooooo`0?oooo
o`003_ooool2o`000003ooooool0003ooooo00gooooo00?o0000ooooooooool0foooool01?l0003o
ooooooooooooool2o`0000Wooooo00?o0000ooooooooool04?ooool000cooooo1?l000000ooooooo
0000o`000004o`0000Oooooo0_l0003Oooooo`;o000000?oooooo`000?l000000_l00003ooooo`Ko
00004Oooool000_ooooo2?l000001?oooooo0000ooooooooool2o`0000Cooooo00?o0000oooooooo
ool0hOooool:o`0000Cooooo0_l0000?ooooo`002_ooool7o`0000;ooooo0ol00003ooooo`04o`00
0?oooooo0000o`000>Cooooo2Ol000000ooooooo0000ooooo`04ooooo`03o`000?oooooooooo00co
oooo000:ooooo`Oo00001?ooool6o`000>Kooooo1ol00004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool02oooool000[ooooo1ol00004ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool0i?ooool7o`0000Cooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`0;ooooo`002oooool5o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`3T
ooooo`Go00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo000;oooo
o`Co00001oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo0>Gooooo0ol00007oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_ooool000_ooooo0ol00008ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool0hoooool4o`0000Oooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`0:ooooo`002oooool3o`0000Sooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`3Sooooo`Co00001oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`04o`000?oooooo0000o`0000Sooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`3Sooooo`Co00001oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`04o`000?oooooo0000o`0000Sooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`3Sooooo`04o`000?oooooo0000o`0000Oooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`09ooooo`002_ooool01Ol0003oooooo`000?oooooo000000Go
oooo0_l00006ooooo`03o`000?oooooooooo0>?ooooo00Co0000oooooooooooo00001Oooool2o`00
00Kooooo00?o0000ooooooooool02_ooool000_ooooo0_l00002ooooo`Go00002?ooool00ol0003o
ooooooooo`3Sooooo`03o`000?oooooooooo00Ko00002?ooool00ol0003oooooooooo`0:ooooo`00
2oooool2o`0000oooooo00?o0000ooooooooool0hoooool00ol0003oooooooooo`02o`0000cooooo
00?o0000ooooooooool02_ooool000_ooooo0_l0000?ooooo`03o`000?oooooooooo0>Cooooo00Co
0000ooooool0003o00003?ooool00ol0003oooooooooo`0:ooooo`002oooool4o`0000cooooo00?o
0000ooooooooool0i_ooool3o`0000_ooooo00?o0000ooooooooool02oooool000Wooooo0ol00000
0ooooooo0000o`000005o`0000Gooooo0_l0003[ooooo`?o000000?oooooo`000?l000001_l0000>
ooooo`002?ooool9o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo0>[ooooo2Ol000001?oo
oooo0000ooooooooool2o`0000cooooo0007ooooo`Oo00000_ooool8o`000>cooooo2ol00005oooo
o`03o`000?oooooooooo00Wooooo0007ooooo`Oo00001?ooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo0>[ooooo1ol00004ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool02?ooool000Oooooo1ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
j_ooool7o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`002?oo
ool5o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`3Zooooo`Go00001_oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo0008ooooo`Co00001oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo0>[ooooo0ol00007ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01oooool000Sooooo00Co0000ooooool0003o00001oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo0>Wooooo0ol00008ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01oooool000Sooooo0ol00008ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0jOooool3o`0000Sooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`06ooooo`001oooool01?l0003oooooo`000?l00008ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0jOooool3o`0000Sooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`06ooooo`001oooool00ol0003oooooooooo`02o`0000Gooooo0_l00006ooooo`03
o`000?oooooooooo0>Wooooo00Co0000ooooool0003o00001Oooool2o`0000Oooooo00?o0000oooo
ooooool01_ooool000Oooooo00Go0000oooooooooooo0000ooooo`02o`000003ooooool0003o0000
00Sooooo00?o0000ooooooooool0jOooool01?l0003oooooo`000?ooool2o`000003ooooool0003o
000000Sooooo00?o0000ooooooooool01oooool000Sooooo0ol00003ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool0jOooool01?l0003oooooo`000?l00002ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01oooool000Sooooo0ol0000>ooooo`03o`000?oooooooooo
0>Wooooo00Co0000ooooool0003o00003Oooool00ol0003oooooooooo`07ooooo`002Oooool2o`00
00gooooo00?o0000ooooooooool0joooool3o`0000cooooo00?o0000ooooooooool02?ooool000Wo
oooo0ol0000:ooooo`;o0000koooool2o`0000[ooooo0_l0000;ooooo`002?ooool3o`000003oooo
ool0003o000000Go00000_ooool00ol0003oooooooooo`3_ooooo`?o000000?oooooo`000?l00000
1_l0000=ooooo`001oooool8o`0000Cooooo0_l0003aooooo`Wo000000Coooooo`000?l0003ooooo
0_l0000;ooooo`001_ooool7o`000003ooooool0003o000000?o00000_ooool00ol0003ooooooooo
o`3]ooooo`_o00001Oooool00ol0003oooooooooo`08ooooo`001_ooool7o`0000Cooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`3/ooooo`Oo00001?ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00Oooooo0006ooooo`Oo00001?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo0>cooooo1ol00004ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01oooool000Oooooo1Ol00006ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool0k?ooool5o`0000Kooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`06ooooo`00
1oooool4o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3/ooooo`?o0000
1oooool2o`0000Cooooo00?o0000ooooooooool01_ooool000Oooooo00Co0000ooooool0003o0000
1oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0>_ooooo0ol00008ooooo`;o0000
1?ooool00ol0003oooooooooo`06ooooo`001oooool4o`0000Oooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`3[ooooo`?o00002?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo0006ooooo`03o`000?oooooo000000;o00001oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo0>_ooooo0ol00008ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool000Kooooo00?o0000ooooooooool00_l00005ooooo`;o00001_ooool00ol0
003oooooooooo`3[ooooo`Co00001Oooool2o`0000Oooooo00?o0000ooooooooool01Oooool000Ko
oooo00?o0000ooooooooool01?l000000ooooooo0000o`000008ooooo`03o`000?oooooooooo0>_o
oooo0ol000001?oooooo0000o`000?ooool2o`0000Sooooo00?o0000ooooooooool01_ooool000Oo
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
0>_ooooo0_l00004ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01_ooool000Oo
oooo00Co0000oooooooooooo00003Oooool00ol0003oooooooooo`3[ooooo`;o00003oooool00ol0
003oooooooooo`06ooooo`002?ooool00ol0003oooooo`00000<ooooo`03o`000?oooooooooo0>go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`07ooooo`002Oooool2o`0000[ooooo
0_l0003`ooooo`?o00002_ooool2o`0000[ooooo0009ooooo`?o000000?oooooo`000?l000001_l0
003aooooo`?o000000?oooooo`000?l000001_l000000ooooooo0000ooooo`0;ooooo`002?ooool8
o`0000;ooooo1?l0003_ooooo`Wo00000oooool3o`0000cooooo0007ooooo`_o00001?ooool00ol0
003oooooooooo`3[ooooo`Oo00000_ooool4o`0000?ooooo00?o0000ooooooooool02Oooool000Oo
oooo1ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0j_ooool7o`0000Co
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`001oooool7o`0000Cooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`3Zooooo`Oo00001?ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Sooooo0008ooooo`Go00001_ooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo0>[ooooo1Ol00006ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool01oooool000Sooooo1?l00007ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool0j_ooool3o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07
ooooo`002?ooool00ol0003oooooooooo`02o`0000Kooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`3Yooooo`;o00002Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Oooooo0008ooooo`;o000000?oooooo`000?l000001_ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo0>Wooooo0_l00009ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01_ooool000Oooooo00Co0000ooooool0003ooooo0_l00006ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool0jOooool2o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`06ooooo`001oooool00ol0003oooooooooo`03o`0000Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`3Yooooo`Co00001_ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Kooooo0007ooooo`03o`000?oooooooooo00;ooooo0_l00003ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool0jOooool01?l0003oooooooooooooool2o`0000?ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`07ooooo`002?ooool01_l0003ooooooooo
oooooooo0000ooooo`?o00002?ooool00ol0003oooooooooo`3Yooooo`03o`000?oooooooooo00?o
oooo0ol00008ooooo`03o`000?oooooooooo00Oooooo0008ooooo`05o`000?ooooooooooooooool0
00003?ooool00ol0003oooooooooo`3Yooooo`03o`000?oooooooooo00kooooo00?o0000oooooooo
ool01oooool000Wooooo00?o0000ooooooooool00_l0000:ooooo`03o`000?oooooooooo0>Wooooo
0ol0000>ooooo`03o`000?oooooooooo00Sooooo000:ooooo`Go00001oooool2o`000>_ooooo0ol0
00000ooooooo0000o`00000:ooooo`;o00002oooool000_ooooo3?l0003[ooooo`go00000oooool0
0ol0003oooooooooo`0;ooooo`002_ooool;o`0000;ooooo0_l0003Xooooo`_o00000oooool3o`00
00kooooo000:ooooo`Wo000000?oooooo`000?ooool00oooool00ol0003oooooooooo`3Uooooo`Oo
00000oooool5o`000003ooooool0003ooooo00gooooo000:ooooo`Oo00001?ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo0>Cooooo1ol00004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool02oooool000_ooooo1Ol00005ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool0i?ooool5o`0000Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0:ooooo`002oooool5o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`3T
ooooo`Co00001oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo000;oooo
o`03o`000?oooooooooo00;o00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
0>?ooooo0_l00009ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_ooool000_o
oooo00?o0000ooooooooool00_l00006ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0hoooool2o`0000Wooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0:ooooo`00
2_ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`3Sooooo`;o00002Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooo000000;ooooo0_l00005ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool0h_ooool3o`0000Wooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`04o`0000?ooooo0_l00006
ooooo`03o`000?oooooooooo0>;ooooo0_l000000ooooooo0000o`000005ooooo`;o00001oooool0
0ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`02ooooo`;o000000?oooooo`00
0?l000002?ooool00ol0003oooooooooo`3Rooooo`;o00000oooool2o`000003ooooool0003o0000
00Sooooo00?o0000ooooooooool02_ooool000_ooooo00?o0000ooooooooool00_ooool2o`0000[o
oooo00?o0000ooooooooool0hOooool3o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0:ooooo`002oooool00ol0003oooooooooo`02ooooo`;o00002_ooool00ol0003ooooo
ooooo`3Qooooo`?o00004?ooool00ol0003oooooooooo`0:ooooo`003?ooool00ol0003ooooooooo
o`02ooooo`;o00002?ooool00ol0003oooooooooo`3Qooooo`;o00000_ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00_ooooo000=ooooo`;o00000_ooool3o`000003ooooool0003o
000000Go0000h_ooool3o`000003ooooool0003o000000Ko00001_ooool2o`0000kooooo000?oooo
o`[o00000oooool2o`000=oooooo2_l00003ooooo`;o00000oooool00ol0003oooooooooo`0>oooo
o`003oooool;o`0000Cooooo00?o0000ooooooooool0foooool7o`0000?ooooo0ol00003ooooo`?o
00004Oooool000oooooo1ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
f_ooool7o`0000Cooooo00?o0000ooooool000001?l0000Booooo`003oooool7o`0000Cooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`3Jooooo`Oo00001?ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo013ooooo000@ooooo`Go00001_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo0=[ooooo1Ol00006ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool03oooool0013ooooo1Ol00006ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool0fOooool4o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0?oooo
o`004?ooool01?l0003oooooooooooooool2o`0000Gooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`3Hooooo`?o00002_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00oooooo000@ooooo`;o00000_ooool2o`0000Gooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`3Hooooo`?o00002_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00ko
oooo000?ooooo`03o`000?oooooo000000?ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool0eoooool4o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0>ooooo`003oooool00ol0003oooooooooo`02o`000003ooooool0003o000000?ooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`3Gooooo`?o000000?oooooo`000?l000001_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooo
oooo00;ooooo0ol00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0eoooool0
0ol0003oooooo`000003ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00kooooo000@ooooo`03o`000?oooooooooo00?ooooo0ol00008ooooo`03o`000?oooooooooo
0=Kooooo0_l000000ooooooo0000ooooo`04ooooo`?o00002Oooool00ol0003oooooooooo`0>oooo
o`004?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool0e_ooool01?l0003oooooooooool0000@ooooo`03o`000?oooooooooo00oooooo000Aooooo`03
o`000?oooooooooo00?ooooo1?l00006ooooo`03o`000?oooooooooo0=?ooooo1?l00003ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool03oooool001;ooooo0_l00003ooooo`co0000
doooool=o`0000[ooooo00?o0000ooooooooool04?ooool001Cooooo00?o0000ooooool000002Ol0
0003ooooo`;o0000d?ooool;o`000003ooooool0003ooooo00;o00001_ooool2o`0001?ooooo000E
ooooo`Wo00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo0<gooooo1ol00003
ooooo`03o`000?oooooooooo00;o000000Coooooo`000?oooooooooo0ol0000Eooooo`005_ooool7
o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`3<ooooo`Oo00001?ooool0
1?l0003oooooooooooooool4o`0001Sooooo000Gooooo`Go00001_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo0<cooooo1Ol00006ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool05_ooool001Oooooo1Ol00006ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool0c?ooool4o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0Foooo
o`005_ooool2o`0000?ooooo0_l00005ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0boooool2o`0000[ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Fooooo`00
5_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo0<[ooooo0ol0000:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
5_ooool001Kooooo00?o0000ooooool000000oooool2o`0000Cooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`39ooooo`Co00002Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo01Gooooo000Fooooo`03o`000?oooooo000000Cooooo0_l00002ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool0b?ooool2o`000003ooooool0003o000000Wooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0Eooooo`005_ooool01?l0003oooooooooool00003
ooooo`;o000000?oooooo`000?ooool01_ooool00ol0003oooooooooo`38ooooo`;o000000?ooooo
o`000?ooool00_l00005ooooo`;o00001oooool00ol0003oooooooooo`0Eooooo`005_ooool01?l0
003oooooooooooooool6o`0000Sooooo00?o0000ooooooooool0aoooool2o`0000;ooooo00Co0000
oooooooooooooooo1Ol00009ooooo`03o`000?oooooooooo01Gooooo000Gooooo`03o`000?oooooo
oooo00Cooooo0ol00006ooooo`03o`000?oooooooooo0<Oooooo0ol00002ooooo`03o`000?oooooo
oooo00kooooo00?o0000ooooooooool05_ooool001Oooooo00?o0000ooooooooool01Oooool4o`00
00?ooooo1_l00033ooooo`Co000000?oooooo`000?l000001?l0000=ooooo`03o`000?oooooooooo
01Kooooo000Hooooo`03o`000?oooooooooo00Cooooo2Ol00004ooooo`;o0000`?ooool8o`000004
ooooool0003oooooooooo`;o00002_ooool00ol0003oooooooooo`0Gooooo`006Oooool00ol0003o
ooooooooo`02ooooo`So00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0;co
oooo1ol00002ooooo`Co00000_ooool00ol0003oooooooooo`05ooooo`;o00006_ooool001[ooooo
00Co0000oooooooooooooooo1ol00004ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0^oooool7o`0000Cooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo01[ooooo000Kooooo`[o00001?ooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo0;_ooooo1ol00004ooooo`03o`000?oooooooooo00Co00000_ooool2o`0001gooooo
000Oooooo`Ko00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0;[ooooo1_l0
0006ooooo`03o`000?oooooooooo00;ooooo0ol0000Oooooo`008?ooool4o`000003ooooool0003o
oooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`2jooooo`Go00001oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo01kooooo000Oooooo`;o00000oooool00ol0
003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0^Oooool3o`00
00[ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0Nooooo`007oooool2o`0000Co
oooo00?o0000ooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0;Oo
oooo0_l000000ooooooo0000o`000009ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool07_ooool001oooooo0_l00005ooooo`;o00000_ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo0;Kooooo0_l000001?oooooo0000ooooool00008ooooo`;o00001Oooool00ol0003o
ooooooooo`0Nooooo`007oooool00ol0003oooooo`000005ooooo`?o00001oooool00ol0003ooooo
ooooo`2fooooo`;o000000Goooooo`000?ooooooooooo`000006ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool07_ooool001oooooo00Co0000oooooooooooo00001Oooool2o`0000Oo
oooo00?o0000ooooooooool0]Oooool2o`0000;ooooo00Co0000oooooooooooo00001_ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo01kooooo000Oooooo`04o`000?oooooooooooooo
o`Go000000?oooooo`000?ooool00ol00003ooooo`03o`000?oooooooooo0;7ooooo1Ol00004oooo
o`03o`000?oooooooooo00Ko00002?ooool00ol0003oooooooooo`0Nooooo`007oooool00ol0003o
ooooooooo`08ooooo`go0000[Oooool=o`0000kooooo00?o0000ooooooooool07_ooool0023ooooo
00?o0000ooooooooool01_ooool;o`0000?ooooo00?o0000ooooooooool0ZOooool;o`000004oooo
ool0003oooooo`0000gooooo00?o0000ooooooooool07_ooool0027ooooo0_l00006ooooo`Oo0000
1?ooool01Ol0003ooooooooooooooooo00000:[ooooo1ol00004ooooo`03o`000?oooooo000000;o
00002oooool00ol0003oooooooooo`0Oooooo`008oooool00ol0003oooooooooo`03ooooo`Oo0000
1?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo0:Oooooo1ol00004ooooo`04o`00
0?ooooooooooooooo`;o00002?ooool2o`0002;ooooo000Tooooo`;o00001?ooool6o`0000Gooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`2Vooooo`Ko00001_ooool01?l0003ooooo
ooooooooool2o`0000Kooooo00?o0000ooooooooool08_ooool002Kooooo2?l000000ooooooo0000
o`000004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0YOooool00ol0003ooooo
o`000003o`0000Oooooo00?o0000ooooooooool00_ooool7o`0002Gooooo000Yooooo`;o00001Ooo
ool2o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`2Sooooo`04o`000?oo
oooo0000o`0000[ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0Xooooo`00:Ooo
ool2o`0000Kooooo0_l00002ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0XOoo
ool2o`0000;ooooo0_l0000:ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0:?oo
ool002Wooooo0_l00007ooooo`?o00001_ooool00ol0003oooooooooo`2Pooooo`;o00000oooool2
o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Xooooo`00:Oooool2o`00
00Sooooo0ol00005ooooo`03o`000?oooooooooo09oooooo0_l00004ooooo`03o`000?oooooo0000
00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Xooooo`00:Oooool01?l0003o
ooooo`000?l00005ooooo`?o000000Coooooo`000?l0003o00000_ooool00ol0003oooooooooo`2K
ooooo`Go00001Oooool01?l0003oooooooooool00005ooooo`;o00001oooool00ol0003ooooooooo
o`0Xooooo`00:Oooool01?l0003oooooooooooooool5o`0000?ooooo3_l0002Eooooo`go00000_oo
ool5o`0000Wooooo00?o0000ooooooooool0:?ooool002Wooooo00?o0000ooooooooool02?ooool;
o`0000Cooooo00?o0000ooooooooool0TOooool;o`0000;ooooo0_l0000>ooooo`03o`000?oooooo
oooo02Wooooo000Zooooo`03o`000?oooooooooo00Oooooo1ol00004ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool0T?ooool7o`0000Cooooo00Go0000ooooool0003oooooo`00000=
ooooo`03o`000?oooooooooo02Wooooo000Zooooo`03o`000?oooooooooo00Oooooo1ol00004oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0Soooool7o`0000Cooooo00?o0000oooo
ool000000_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo02[ooooo000[ooooo`;o
00002?ooool7o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`2=ooooo`Oo
00001_ooool01_l0003oooooo`000?l0003oooooo`0000Sooooo0_l0000]ooooo`00;Oooool00ol0
003oooooooooo`06ooooo`?o00000_ooool3o`0000;ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`2;ooooo`?o000000?oooooo`000?l000000_l00007ooooo`04o`000?oooooooooo
ooooo`;o00001oooool00ol0003oooooooooo`0]ooooo`00;_ooool4o`000003ooooool0003o0000
00;o00001_ooool2o`000003ooooool0003o000000Cooooo00?o0000ooooooooool0R_ooool2o`00
00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`Co000000Coooooo`00
0?l0003o0000<?ooool003;ooooo00?o0000ooooooooool00_l00007ooooo`Co00001?ooool00ol0
003oooooooooo`28ooooo`?o00001?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00?ooooo00Co0000oooooooooooo0000<oooool003Gooooo0_l00008ooooo`?o00001Oooool00ol0
003oooooooooo`26ooooo`?o00001?ooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0=?ooool003Gooooo0_l00009ooooo`Ko000000?oooooo`000?l0
00001_l0001mooooo`co000000?oooooo`000?ooool02?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo03Cooooo000eooooo`04o`000?oooooo0000o`0000Gooooo0_l00002ooooo`Wo
00001?ooool2o`0007[ooooo2?l00004ooooo`;o000000?oooooo`000?l000001Oooool2o`0000Ko
oooo00?o0000ooooooooool0=?ooool003Gooooo00Co0000oooooooooooooooo1Ol00003ooooo`Oo
00000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo07Kooooo1ol00002ooooo`;o
00000_ooool00ol0003oooooo`000002ooooo`Go00002?ooool00ol0003oooooooooo`0dooooo`00
=Oooool00ol0003oooooooooo`09ooooo`So00000oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo07Cooooo2?l00004ooooo`06o`000?ooooooooooo`000?oooooo00003_ooool00ol0
003oooooooooo`0dooooo`00=_ooool00ol0003oooooooooo`08ooooo`Oo000000Goooooo`000?oo
ooooooooo`000005ooooo`03o`000?oooooooooo07?ooooo00?o0000ooooool000001_l00004oooo
o`04o`000?ooooooooooooooo`;o00003_ooool00ol0003oooooooooo`0dooooo`00=_ooool00ol0
003oooooooooo`09ooooo`Go000000?oooooo`000?l000000_l000000ooooooo0000ooooo`04oooo
o`03o`000?oooooooooo073ooooo1?l000000ooooooo0000o`000003o`0000Kooooo00Ko0000oooo
oooooooo0000ooooool0000<ooooo`03o`000?oooooooooo03Gooooo000gooooo`;o00002Oooool4
o`0000Cooooo1?l00005ooooo`03o`000?oooooooooo06oooooo0ol00003ooooo`Co00001oooool0
1?l0003oooooooooooooool3o`0000Wooooo0_l0000hooooo`00>Oooool00ol0003oooooooooo`06
ooooo`;o00002?ooool3o`0000;ooooo0ol0001Zooooo`?o00000_ooool3o`0000Gooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool0>?ooool003[ooooo0ol000000ooooooo0000o`000004o`0000Wooooo4?l0001Poooo
o`co00000_ooool2o`0000Wooooo00?o0000ooooooooool00oooool4o`000003ooooool0003o0000
00;o0000>oooool003gooooo00?o0000ooooooooool00_ooool2o`0000Wooooo00?o0000ooooool0
00002_l00003ooooo`;o0000GOooool:o`0000?ooooo0_l000000ooooooo0000ooooo`08ooooo`03
o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`00000oooooo`00@_ooool01?l0003o
ooooo`000?l00005ooooo`;o00000_ooool7o`0000?ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`1Jooooo`Oo00000oooool01?l0003oooooooooooooool2o`000003ooooool0003o
000000Gooooo0_l00006ooooo`03o`000?oooooooooo047ooooo0012ooooo`04o`000?oooooooooo
ooooo`;o000000?oooooo`000?l000001?ooool9o`0000;ooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`1Gooooo`Wo00001?ooool01_l0003oooooooooool0003oooooo`0000;ooooo
0_l000000ooooooo0000o`000008ooooo`03o`000?oooooooooo047ooooo0012ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool5o`0000;ooooo1?l00006ooooo`03o`000?oo
oooooooo05Cooooo0ol00002ooooo`Go00001Oooool01ol0003ooooooooooooooooo0000ooooool0
00000oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo047ooooo0012ooooo`03o`00
0?oooooooooo00cooooo0ol00005ooooo`Go00000_ooool3o`0004cooooo0ol00003ooooo`Co0000
1?ooool4o`0000Oooooo00Ko0000oooooooooooo0000ooooool0000>ooooo`03o`000?oooooooooo
047ooooo0013ooooo`03o`000?oooooooooo00[ooooo0ol00008oooooa7o0000@_ooool=o`0000;o
oooo00?o0000ooooooooool02?ooool01ol0003ooooooooooooooooo0000ooooool000003?ooool0
0ol0003oooooooooo`12ooooo`00A?ooool2o`0000[ooooo0_l00009ooooo`;o000000?oooooo`00
0?l000002?l00004ooooo`03o`000?oooooooooo03kooooo2_l00004ooooo`03o`000?oooooo0000
00[ooooo00?o0000ooooooooool00_ooool2o`0000[ooooo0_l00015ooooo`00A_ooool00ol0003o
ooooooooo`06ooooo`?o00002Oooool00ol0003oooooooooo`09o`000003ooooool0003ooooo00?o
oooo00?o0000ooooooooool0??ooool8o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003o
ooooo`000009ooooo`03o`000?oooooooooo00?ooooo0_l00008ooooo`03o`000?oooooooooo04Go
oooo0017ooooo`;o00001?ooool2o`000003ooooool0003o000000Wooooo00?o0000ooooooooool0
3?l00004ooooo`03o`000?oooooo000000;o0000;?ooool3o`0000Kooooo3?l00004ooooo`04o`00
0?ooooooooooooooo`?o00002Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000Go
oooo0_l00018ooooo`00BOooool4o`0000?ooooo00Co0000ooooool0003o00001Oooool2o`0000Co
oooo1Ol00004oooooa?o00008oooool=o`0000?ooooo1Ol00006ooooo`05o`000?ooooooooooo`00
0?ooool00ol00005ooooo`;o00001_ooool00ol0003oooooooooo`05o`0004[ooooo001@ooooo`04
o`000?ooooooooooooooo`;o000000?oooooo`000?l000001_ooool4o`0000Oooooo00?o0000oooo
ool000002ol00003ooooo`;o00000oooool3o`0000gooooo0ol0000:ooooo`_o00000oooool2o`00
0003ooooool0003o000000;o00001oooool01_l0003oooooooooool0003oooooo`0000;ooooo0_l0
00000ooooooo0000o`000008ooooo`03o`000?oooooooooo04oooooo001@ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01_ooool3o`0000Sooooo00Co0000oooooooooooooooo3?l0
0003ooooo`03o`000?oooooo000000_o00001?oooolEo`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000[ooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`1?ooooo`00DOooool00ol0003oooooooooo`0;ooooo`?o0000
2?ooool01?l0003oooooooooooooool7o`000003ooooool0003o000002co00001Oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool2o`0000ko
oooo00?o0000ooooooooool0Coooool0057ooooo00?o0000ooooooooool02oooool2o`0000Wooooo
00?o0000ooooooooool00_ooool5o`0000Kooooo00Co0000oooooooooooooooo5ol00003ooooo`03
o`000?oooooooooo00;ooooo1_l00007ooooo`06o`000?ooooooooooo`000?oooooo00002Oooool0
0ol0003oooooooooo`02ooooo`;o00003Oooool00ol0003oooooooooo`1@ooooo`00D_ooool2o`00
00[ooooo0ol00009ooooo`03o`000?oooooooooo00;ooooo1?l00007ooooo`04o`000?oooooooooo
ooooo`Oo00001?ooool00ol0003oooooooooo`02ooooo`Oo00001?ooool00ol0003oooooooooo`02
ooooo`Co00002?ooool01_l0003oooooooooool0003oooooo`0000Wooooo00?o0000ooooooooool0
0oooool2o`0000[ooooo0_l0001Cooooo`00E?ooool00ol0003oooooooooo`06ooooo`04o`000?oo
oooo0000ooooo`;o00001Oooool2o`0000Cooooo1?l00008ooooo`03o`000?oooooooooo00;ooooo
1Ol00006ooooo`03o`000?oooooooooo00;ooooo1Ol00006ooooo`04o`000?ooooooooooooooo`;o
00002_ooool01_l0003ooooooooooooooooo0000ooooo`;o00001Oooool2o`0000Kooooo00?o0000
ooooool000002?ooool00ol0003oooooooooo`1Cooooo`00EOooool2o`0000Cooooo0_l00002oooo
o`04o`000?ooooooooooooooo`Go00001_ooool3o`0000Wooooo00?o0000ooooooooool00_ooool4
o`0000Oooooo00?o0000ooooooooool00_ooool4o`0000Oooooo00Co0000oooooooooooooooo0_l0
000:ooooo`08o`000?ooooooooooooooool0003oooooo`000?ooool5o`0000Sooooo00Co0000oooo
oooooooo00001Oooool2o`0005Kooooo001Gooooo`Co00001?ooool00ol0003oooooooooo`0<oooo
o`?o00002Oooool01?l0003oooooooooooooool3o`0000Wooooo00Co0000oooooooooooooooo0ol0
0009ooooo`04o`000?ooooooooooooooo`?o00002?ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooo000000kooooo00Co0000oooooooooooooooo1Ol0001Hooooo`00H?ooool00ol0003ooooo
ooooo`0;ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool00_ooool3o`0000Wooooo
00Co0000oooooooooooooooo0ol00009ooooo`04o`000?ooooooooooooooo`?o00002?ooool00ol0
003oooooooooo`03ooooo`;o00003_ooool00ol0003oooooooooo`1Nooooo`00H?ooool00ol0003o
ooooooooo`0;ooooo`;o000000?oooooo`000?ooool01Oooool00ol0003oooooooooo`03ooooo`?o
00002?ooool00ol0003oooooooooo`02ooooo`?o00002?ooool00ol0003oooooooooo`02ooooo`04
o`000?oooooo0000o`0000Gooooo0_l00006ooooo`;o00003Oooool00ol0003oooooooooo`1Ooooo
o`00HOooool2o`0000[ooooo0_l00003ooooo`Ko00001_ooool3o`0000Sooooo00?o0000oooooooo
ool00_ooool3o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooo`000?ooool5o`00
00Wooooo0_l0000:ooooo`;o0000H_ooool006?ooooo00?o0000ooooooooool01_ooool01?l0003o
ooooooooool0000>ooooo`Co00001Oooool2o`0000Gooooo00Co0000ooooool0003o00001Oooool2
o`0000Gooooo00?o0000ooooool000003_ooool01?l0003oooooooooool00008ooooo`03o`000?oo
oooooooo06;ooooo001Tooooo`?o00000oooool2o`0000?ooooo00?o0000ooooooooool03?ooool2
o`0000;ooooo1Ol00007ooooo`04o`000?oooooo0000ooooo`Go00002?ooool2o`0000kooooo00Co
0000oooooooooooooooo0ol00003ooooo`;o0000IOooool006Oooooo0ol00005ooooo`03o`000?oo
oooooooo00cooooo0_l0000?ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`?o0000Ioooool0073ooooo0_l0000:ooooo`;o000000?ooooo
o`000?ooool03_ooool00ol0003oooooooooo`0=ooooo`?o00002_ooool2o`0007;ooooo001boooo
o`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`00000>ooooo`;o00003_ooool0
1Ol0003ooooooooooooooooo000000Sooooo00?o0000ooooooooool0L_ooool007?ooooo1?l00003
ooooo`03o`000?oooooooooo00?ooooo0_l0000:ooooo`;o00000_ooool2o`0000[ooooo0_l00005
ooooo`Co00000_ooool2o`0007Gooooo001gooooo`?o00002?ooool00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`09oooo
o`;o0000Moooool008?ooooo1?l00003ooooo`03o`000?oooooooooo00Kooooo1?l00003ooooo`03
o`000?oooooooooo08?ooooo0027ooooo`?o00003Oooool3o`0008Kooooo003oooooob7ooooo003o
ooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0371287, -0.0360927, \
0.00391727, 0.00391005}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Because every vertex is connected to itself by a path of length 0, ",
  StyleBox["GraphPower", "Input"],
  " returns a graph that has self-loops. We can eliminate self-loops by using \
",
  StyleBox["MakeSimple", "Input"],
  " to reduce the graph to one without multiple edges (a multi-graph) or \
self-loops."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(updNbhd = MakeSimple[updNbhd];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[updNbhd];\)\)}], "Input",
  CellLabel->"In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0446377 0.910725 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95096 .55707 m
.94027 .61324 L
s
.95096 .55707 m
.92263 .66763 L
s
.95096 .55707 m
.95096 .44293 L
s
.95096 .55707 m
.95455 .5 L
s
.94027 .61324 m
.92263 .66763 L
s
.94027 .61324 m
.89832 .71937 L
s
.94027 .61324 m
.95455 .5 L
s
.92263 .66763 m
.89832 .71937 L
s
.92263 .66763 m
.86773 .76766 L
s
.89832 .71937 m
.86773 .76766 L
s
.89832 .71937 m
.83135 .81172 L
s
.86773 .76766 m
.83135 .81172 L
s
.86773 .76766 m
.78974 .85086 L
s
.83135 .81172 m
.78974 .85086 L
s
.83135 .81172 m
.74356 .88448 L
s
.78974 .85086 m
.74356 .88448 L
s
.78974 .85086 m
.69354 .91202 L
s
.74356 .88448 m
.69354 .91202 L
s
.74356 .88448 m
.64046 .93308 L
s
.69354 .91202 m
.64046 .93308 L
s
.69354 .91202 m
.58517 .9473 L
s
.64046 .93308 m
.58517 .9473 L
s
.64046 .93308 m
.52854 .95446 L
s
.58517 .9473 m
.52854 .95446 L
s
.58517 .9473 m
.47146 .95446 L
s
.52854 .95446 m
.47146 .95446 L
s
.52854 .95446 m
.41483 .9473 L
s
.47146 .95446 m
.41483 .9473 L
s
.47146 .95446 m
.35954 .93308 L
s
.41483 .9473 m
.35954 .93308 L
s
.41483 .9473 m
.30646 .91202 L
s
.35954 .93308 m
.30646 .91202 L
s
.35954 .93308 m
.25644 .88448 L
s
.30646 .91202 m
.25644 .88448 L
s
.30646 .91202 m
.21026 .85086 L
s
.25644 .88448 m
.21026 .85086 L
s
.25644 .88448 m
.16865 .81172 L
s
.21026 .85086 m
.16865 .81172 L
s
.21026 .85086 m
.13227 .76766 L
s
.16865 .81172 m
.13227 .76766 L
s
.16865 .81172 m
.10168 .71937 L
s
.13227 .76766 m
.10168 .71937 L
s
.13227 .76766 m
.07737 .66763 L
s
.10168 .71937 m
.07737 .66763 L
s
.10168 .71937 m
.05973 .61324 L
s
.07737 .66763 m
.05973 .61324 L
s
.07737 .66763 m
.04904 .55707 L
s
.05973 .61324 m
.04904 .55707 L
s
.05973 .61324 m
.04545 .5 L
s
.04904 .55707 m
.04545 .5 L
s
.04904 .55707 m
.04904 .44293 L
s
.04545 .5 m
.04904 .44293 L
s
.04545 .5 m
.05973 .38676 L
s
.04904 .44293 m
.05973 .38676 L
s
.04904 .44293 m
.07737 .33237 L
s
.05973 .38676 m
.07737 .33237 L
s
.05973 .38676 m
.10168 .28063 L
s
.07737 .33237 m
.10168 .28063 L
s
.07737 .33237 m
.13227 .23234 L
s
.10168 .28063 m
.13227 .23234 L
s
.10168 .28063 m
.16865 .18828 L
s
.13227 .23234 m
.16865 .18828 L
s
.13227 .23234 m
.21026 .14914 L
s
.16865 .18828 m
.21026 .14914 L
s
.16865 .18828 m
.25644 .11552 L
s
.21026 .14914 m
.25644 .11552 L
s
.21026 .14914 m
.30646 .08798 L
s
.25644 .11552 m
.30646 .08798 L
s
.25644 .11552 m
.35954 .06692 L
s
.30646 .08798 m
.35954 .06692 L
s
.30646 .08798 m
.41483 .0527 L
s
.35954 .06692 m
.41483 .0527 L
s
.35954 .06692 m
.47146 .04554 L
s
.41483 .0527 m
.47146 .04554 L
s
.41483 .0527 m
.52854 .04554 L
s
.47146 .04554 m
.52854 .04554 L
s
.47146 .04554 m
.58517 .0527 L
s
.52854 .04554 m
.58517 .0527 L
s
.52854 .04554 m
.64046 .06692 L
s
.58517 .0527 m
.64046 .06692 L
s
.58517 .0527 m
.69354 .08798 L
s
.64046 .06692 m
.69354 .08798 L
s
.64046 .06692 m
.74356 .11552 L
s
.69354 .08798 m
.74356 .11552 L
s
.69354 .08798 m
.78974 .14914 L
s
.74356 .11552 m
.78974 .14914 L
s
.74356 .11552 m
.83135 .18828 L
s
.78974 .14914 m
.83135 .18828 L
s
.78974 .14914 m
.86773 .23234 L
s
.83135 .18828 m
.86773 .23234 L
s
.83135 .18828 m
.89832 .28063 L
s
.86773 .23234 m
.89832 .28063 L
s
.86773 .23234 m
.92263 .33237 L
s
.89832 .28063 m
.92263 .33237 L
s
.89832 .28063 m
.94027 .38676 L
s
.92263 .33237 m
.94027 .38676 L
s
.92263 .33237 m
.95096 .44293 L
s
.94027 .38676 m
.95096 .44293 L
s
.94027 .38676 m
.95455 .5 L
s
.95096 .44293 m
.95455 .5 L
s
.025 w
.95096 .55707 Mdot
.94027 .61324 Mdot
.92263 .66763 Mdot
.89832 .71937 Mdot
.86773 .76766 Mdot
.83135 .81172 Mdot
.78974 .85086 Mdot
.74356 .88448 Mdot
.69354 .91202 Mdot
.64046 .93308 Mdot
.58517 .9473 Mdot
.52854 .95446 Mdot
.47146 .95446 Mdot
.41483 .9473 Mdot
.35954 .93308 Mdot
.30646 .91202 Mdot
.25644 .88448 Mdot
.21026 .85086 Mdot
.16865 .81172 Mdot
.13227 .76766 Mdot
.10168 .71937 Mdot
.07737 .66763 Mdot
.05973 .61324 Mdot
.04904 .55707 Mdot
.04545 .5 Mdot
.04904 .44293 Mdot
.05973 .38676 Mdot
.07737 .33237 Mdot
.10168 .28063 Mdot
.13227 .23234 Mdot
.16865 .18828 Mdot
.21026 .14914 Mdot
.25644 .11552 Mdot
.30646 .08798 Mdot
.35954 .06692 Mdot
.41483 .0527 Mdot
.47146 .04554 Mdot
.52854 .04554 Mdot
.58517 .0527 Mdot
.64046 .06692 Mdot
.69354 .08798 Mdot
.74356 .11552 Mdot
.78974 .14914 Mdot
.83135 .18828 Mdot
.86773 .23234 Mdot
.89832 .28063 Mdot
.92263 .33237 Mdot
.94027 .38676 Mdot
.95096 .44293 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[6]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00Q_ooool3o`0000gooooo0ol00027ooooo`00QOooool5o`0000_ooooo1Ol00026ooooo`00
MOooool3o`0000cooooo1ol00009ooooo`Oo00002oooool3o`0007Oooooo001dooooo`Go00002_oo
oolIo`0000Wooooo1Ol0001fooooo`00Loooool7o`000003ooooool0003o000001;o000000?ooooo
o`000?l000006Ol0001eooooo`00Loooool<o`0000Kooooo1Ol0000;ooooo`Go00001_ooool;o`00
07Gooooo001Vooooo`?o00002?ooool9o`0000cooooo0ol0000=ooooo`?o00002oooool:o`0000So
oooo0ol0001Wooooo`00IOooool5o`0000?ooooo3?l0000/ooooo`Go000000?oooooo`000?l00000
1Ol00003ooooo`Go0000I_ooool006Cooooo2ol00006ooooo`?o0000;_ooool3o`0000Oooooo2ol0
001Uooooo`00I?ooool8o`0004Oooooo2?l0001Uooooo`00Hoooool8o`0004Wooooo2?l0001Toooo
o`00H?ooool:o`0004_ooooo2_l0001Qooooo`00E_ooool3o`0000Gooooo1?l00004ooooo`?o0000
COooool3o`0000Cooooo1?l00004ooooo`?o0000F?ooool005Gooooo1Ol000000ooooooo0000o`00
0002o`00067ooooo2Ol0001Gooooo`00E?ooool9o`0006Gooooo2?l0001Fooooo`00E?ooool7o`00
06Sooooo1ol0001Fooooo`00E?ooool7o`0006Sooooo2?l0001Eooooo`00Doooool7o`0006[ooooo
2?l0001Dooooo`00DOooool4o`000004ooooool0003o0000o`0006cooooo0ol00002ooooo`Co0000
D_ooool004oooooo1?l0001iooooo`Co0000D?ooool004Sooooo0ol00002ooooo`?o0000Ooooool3
o`0000;ooooo0ol00019ooooo`00Aoooool7o`0008?ooooo1ol00018ooooo`00A_ooool7o`0008Go
oooo1ol00017ooooo`00A_ooool7o`0008Gooooo1ol00017ooooo`00A_ooool7o`0008Gooooo1ol0
0017ooooo`00AOooool7o`0008Oooooo1ol00016ooooo`00A?ooool2o`0000;ooooo0ol00029oooo
o`?o00000_ooool2o`0004Gooooo0012ooooo`?o0000UOooool3o`0004?ooooo0011ooooo`?o0000
Uoooool3o`0004;ooooo000kooooo`?o00000_ooool2o`0009_ooooo0_l00002ooooo`?o0000??oo
ool003[ooooo1ol0002Mooooo`Oo0000>oooool003Wooooo1ol0002Oooooo`Oo0000>_ooool003Wo
oooo1ol0002Oooooo`Oo0000>_ooool003Wooooo1ol0002Oooooo`Oo0000>_ooool003[ooooo1Ol0
002Qooooo`Go0000>oooool003Wooooo1Ol0002Sooooo`Go0000>_ooool003Sooooo0_l0002[oooo
o`;o0000>Oooool003Oooooo0_l0002]ooooo`;o0000>?ooool003Kooooo0_l0002_ooooo`;o0000
=oooool003Gooooo0_l0002aooooo`;o0000=_ooool003Cooooo0_l0002cooooo`;o0000=Oooool0
02oooooo0ol000000ooooooo0000o`00002eooooo`;o000000Coooooo`000?l0003o0000<?ooool0
02kooooo1_l0002gooooo`Ko0000;oooool002gooooo1ol0002gooooo`Oo0000;_ooool002gooooo
1ol0002gooooo`Oo0000;_ooool002gooooo1ol0002gooooo`Oo0000;_ooool002kooooo1Ol0002i
ooooo`Go0000;oooool002kooooo1?l0002kooooo`Co0000;oooool002gooooo00?o0000oooooooo
ool0`Oooool00ol0003oooooooooo`0/ooooo`00;?ooool2o`000<?ooooo0_l0000]ooooo`00:ooo
ool2o`000<Gooooo0_l0000/ooooo`00:_ooool2o`000<Oooooo0_l0000[ooooo`00:Oooool00ol0
003oooooo`000037ooooo`03o`000?oooooo000002[ooooo000Tooooo`?o000000Coooooo`000?oo
oooo0000bOooool00ol0003oooooo`000003o`0002Kooooo000Sooooo`Go000000?oooooo`000?oo
ool0b_ooool6o`0002Gooooo000Rooooo`Oo0000c?ooool7o`0002Cooooo000Rooooo`Oo0000c?oo
ool7o`0002Cooooo000Rooooo`Oo0000c?ooool7o`0002Cooooo000Sooooo`Go0000c_ooool5o`00
02Gooooo000Tooooo`?o0000d?ooool4o`0002Gooooo000Sooooo`03o`000?oooooo00000=?ooooo
00?o0000ooooool000009?ooool002;ooooo0ol0003Eooooo`?o00008oooool002;ooooo0_l0003G
ooooo`;o00008oooool0027ooooo0_l0003Iooooo`;o00008_ooool0027ooooo0_l0003Iooooo`;o
00008_ooool0023ooooo0_l0003Kooooo`;o00008Oooool0023ooooo00?o0000ooooooooool0fooo
ool00ol0003oooooooooo`0Oooooo`007?ooool5o`000=gooooo1Ol0000Mooooo`006oooool5o`00
0=oooooo1Ol0000Looooo`006_ooool7o`000=gooooo1ol0000Kooooo`006_ooool7o`000=gooooo
1ol0000Kooooo`006_ooool7o`000=gooooo1ol0000Kooooo`006oooool5o`000=oooooo1Ol0000L
ooooo`007?ooool3o`000>7ooooo0ol0000Mooooo`006oooool2o`000>Gooooo0_l0000Looooo`00
6oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooooooo01[ooooo000Jooooo`;o0000iooo
ool2o`0001_ooooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o0000ooooooooool06Oooool0
01Wooooo0_l0003Yooooo`;o00006_ooool001Wooooo0_l0003Yooooo`;o00006_ooool001Wooooo
00?o0000ooooooooool0jOooool00ol0003oooooooooo`0Hooooo`006?ooool2o`000>_ooooo0_l0
000Iooooo`005Oooool4o`000>gooooo1?l0000Fooooo`005?ooool5o`000>gooooo1Ol0000Eoooo
o`004oooool7o`000>_ooooo1ol0000Dooooo`004oooool7o`000>_ooooo1ol0000Dooooo`004ooo
ool7o`000>_ooooo1ol0000Dooooo`005?ooool5o`000>gooooo1Ol0000Eooooo`005Oooool3o`00
0>oooooo0ol0000Fooooo`005Oooool2o`000?7ooooo0_l0000Fooooo`005?ooool2o`000??ooooo
0_l0000Eooooo`005?ooool2o`000??ooooo0_l0000Eooooo`005?ooool00ol0003oooooooooo`3c
ooooo`03o`000?oooooooooo01?ooooo000Cooooo`;o0000mOooool2o`0001Cooooo000Cooooo`;o
0000mOooool2o`0001Cooooo000Cooooo`;o0000mOooool2o`0001Cooooo000Booooo`;o0000mooo
ool2o`0001?ooooo000Booooo`;o0000moooool2o`0001?ooooo000@ooooo`Co0000moooool4o`00
017ooooo000?ooooo`Go0000moooool5o`00013ooooo000>ooooo`Oo0000mOooool7o`0000oooooo
000>ooooo`Oo0000mOooool7o`0000oooooo000>ooooo`Oo0000mOooool7o`0000oooooo000?oooo
o`Go0000moooool5o`00013ooooo000@ooooo`?o0000nOooool3o`00017ooooo000@ooooo`;o0000
noooool2o`00017ooooo000@ooooo`;o0000noooool2o`00017ooooo000@ooooo`;o0000noooool2
o`00017ooooo000@ooooo`03o`000?oooooooooo0?_ooooo00?o0000ooooooooool03oooool000oo
oooo0_l0003mooooo`;o00004?ooool000oooooo0_l0003mooooo`;o00004?ooool000oooooo0_l0
003mooooo`;o00004?ooool000oooooo0_l0003mooooo`;o00004?ooool000oooooo00?o0000oooo
ooooool0oOooool00ol0003oooooooooo`0>ooooo`003Oooool3o`000?oooooo0ol0000>ooooo`00
3?ooool5o`000?gooooo1Ol0000=ooooo`002oooool7o`000?_ooooo1ol0000<ooooo`002oooool7
o`000?_ooooo1ol0000<ooooo`002oooool7o`000?_ooooo1ol0000<ooooo`003?ooool5o`000?go
oooo1Ol0000=ooooo`003Oooool3o`000?oooooo0ol0000>ooooo`003_ooool2o`000?oooooo0_l0
000?ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo00gooooo000>oooo
o`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo00?o0000oooooooo
ool0ooooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`00
0?oooooooooo00gooooo000=ooooo`;o0000ooooool2ooooo`;o00003_ooool000gooooo0_l0003o
ooooo`;ooooo0_l0000>ooooo`003Oooool2o`000?oooooo0_ooool2o`0000kooooo000=ooooo`;o
0000ooooool2ooooo`;o00003_ooool000_ooooo1?l0003oooooo`;ooooo0ol0000=ooooo`002_oo
ool5o`000?oooooo0Oooool5o`0000cooooo0009ooooo`Oo0000o_ooool7o`0000_ooooo0009oooo
o`Oo0000o_ooool7o`0000_ooooo0009ooooo`Oo0000o_ooool7o`0000_ooooo000:ooooo`Go0000
ooooool1ooooo`Go00003?ooool000_ooooo1?l0003oooooo`;ooooo0ol0000=ooooo`003Oooool2
o`000?oooooo0_ooool2o`0000kooooo000=ooooo`;o0000ooooool2ooooo`;o00003_ooool000go
oooo0_l0003oooooo`;ooooo0_l0000>ooooo`003Oooool2o`000?oooooo0_ooool2o`0000kooooo
000=ooooo`;o0000ooooool2ooooo`;o00003_ooool000kooooo00?o0000ooooooooool0ooooool0
0ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oooooooooo
00gooooo000>ooooo`03o`000?oooooooooo0?oooooo00?o0000ooooooooool03Oooool000kooooo
00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003Oooool3o`000?oooooo0ol0
000>ooooo`003?ooool5o`000?gooooo1Ol0000=ooooo`002oooool7o`000?_ooooo1ol0000<oooo
o`002oooool7o`000?_ooooo1ol0000<ooooo`002oooool7o`000?_ooooo1ol0000<ooooo`003?oo
ool5o`000?gooooo1Ol0000=ooooo`003Oooool3o`000?oooooo0ol0000>ooooo`003oooool00ol0
003oooooooooo`3mooooo`03o`000?oooooooooo00kooooo000?ooooo`;o0000oOooool2o`00013o
oooo000?ooooo`;o0000oOooool2o`00013ooooo000?ooooo`;o0000oOooool2o`00013ooooo000?
ooooo`;o0000oOooool2o`00013ooooo000@ooooo`03o`000?oooooooooo0?_ooooo00?o0000oooo
ooooool03oooool0013ooooo00?o0000ooooooooool0noooool00ol0003oooooooooo`0?ooooo`00
4?ooool2o`000?_ooooo0_l0000Aooooo`004?ooool2o`000?_ooooo0_l0000Aooooo`004?ooool2
o`000?_ooooo0_l0000Aooooo`004?ooool3o`000?Wooooo0ol0000Aooooo`003oooool5o`000?Oo
oooo1Ol0000@ooooo`003_ooool7o`000?Gooooo1ol0000?ooooo`003_ooool7o`000?Gooooo1ol0
000?ooooo`003_ooool7o`000?Gooooo1ol0000?ooooo`003oooool5o`000?Oooooo1Ol0000@oooo
o`004?ooool4o`000?Oooooo1?l0000Aooooo`004_ooool2o`000?Oooooo0_l0000Cooooo`004ooo
ool00ol0003oooooooooo`3eooooo`03o`000?oooooooooo01;ooooo000Cooooo`;o0000mOooool2
o`0001Cooooo000Cooooo`;o0000mOooool2o`0001Cooooo000Dooooo`03o`000?oooooooooo0??o
oooo00?o0000ooooooooool04oooool001Cooooo0_l0003cooooo`;o00005Oooool001Cooooo0_l0
003cooooo`;o00005Oooool001Gooooo00?o0000ooooooooool0lOooool00ol0003oooooooooo`0D
ooooo`005Oooool3o`000>oooooo0ol0000Fooooo`005?ooool5o`000>gooooo1Ol0000Eooooo`00
4oooool7o`000>_ooooo1ol0000Dooooo`004oooool7o`000>_ooooo1ol0000Dooooo`004oooool7
o`000>_ooooo1ol0000Dooooo`005?ooool5o`000>gooooo1Ol0000Eooooo`005Oooool4o`000>go
oooo1?l0000Fooooo`006?ooool2o`000>_ooooo0_l0000Iooooo`006?ooool2o`000>_ooooo0_l0
000Iooooo`006Oooool00ol0003oooooooooo`3Yooooo`03o`000?oooooooooo01Sooooo000Ioooo
o`;o0000jOooool2o`0001[ooooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o0000oooooooo
ool06Oooool001[ooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0Iooooo`006_oo
ool2o`000>Oooooo0_l0000Kooooo`006oooool00ol0003oooooooooo`3Uooooo`03o`000?oooooo
oooo01[ooooo000Kooooo`Co0000hOooool4o`0001cooooo000Kooooo`Go0000goooool5o`0001co
oooo000Jooooo`Oo0000gOooool7o`0001_ooooo000Jooooo`Oo0000gOooool7o`0001_ooooo000J
ooooo`Oo0000gOooool7o`0001_ooooo000Kooooo`Go0000goooool5o`0001cooooo000Looooo`Co
0000goooool4o`0001gooooo000Oooooo`;o0000gOooool2o`00023ooooo000Pooooo`03o`000?oo
oooooooo0=_ooooo00?o0000ooooooooool07oooool0023ooooo0_l0003Kooooo`;o00008Oooool0
027ooooo0_l0003Iooooo`;o00008_ooool0027ooooo0_l0003Iooooo`;o00008_ooool002;ooooo
0_l0003Gooooo`;o00008oooool002;ooooo0ol0003Eooooo`?o00008oooool002?ooooo1?l0003@
ooooo`?o000000?oooooo`000?ooool08oooool002?ooooo1Ol0003>ooooo`Go00009Oooool002;o
oooo1ol0003<ooooo`Oo00009?ooool002;ooooo1ol0003<ooooo`Oo00009?ooool002;ooooo1ol0
003<ooooo`Oo00009?ooool002?ooooo1_l0003=ooooo`Go00009Oooool002Cooooo1?l000000ooo
oooo0000ooooo`3:ooooo`03o`000?oooooo000000;o00009_ooool002Sooooo00?o0000ooooool0
0000bOooool00ol0003oooooo`00000Yooooo`00:Oooool00ol0003oooooo`000037ooooo`03o`00
0?oooooo000002[ooooo000Zooooo`;o0000aoooool2o`0002_ooooo000[ooooo`;o0000aOooool2
o`0002cooooo000/ooooo`;o0000`oooool2o`0002gooooo000]ooooo`03o`000?oooooooooo0<7o
oooo00?o0000ooooooooool0;?ooool002kooooo1?l0002kooooo`Co0000;oooool002kooooo1Ol0
002iooooo`Go0000;oooool002gooooo1ol0002gooooo`Oo0000;_ooool002gooooo1ol0002goooo
o`Oo0000;_ooool002gooooo1ol0002gooooo`Oo0000;_ooool002kooooo1ol0002eooooo`Oo0000
;oooool002oooooo0ol00002ooooo`;o0000/oooool2o`0000;ooooo0ol0000`ooooo`00=Oooool3
o`000:oooooo0ol0000fooooo`00=_ooool00ol0003oooooo`00002]ooooo`03o`000?oooooo0000
03Oooooo000gooooo`03o`000?oooooo00000:_ooooo00?o0000ooooool00000>?ooool003Sooooo
00?o0000ooooool00000ZOooool00ol0003oooooo`00000iooooo`00>Oooool00ol0003oooooo`00
0002o`000:?ooooo0ol000000ooooooo0000ooooo`0iooooo`00>_ooool5o`000:7ooooo1Ol0000k
ooooo`00>Oooool7o`0009oooooo1ol0000jooooo`00>Oooool7o`0009oooooo1ol0000jooooo`00
>Oooool8o`0009gooooo2?l0000jooooo`00>_ooool8o`0009_ooooo2?l0000kooooo`00>oooool3
o`0000?ooooo0_l0002Iooooo`;o00000oooool3o`0003cooooo0012ooooo`?o0000UOooool3o`00
04?ooooo0014ooooo`;o0000Toooool2o`0004Gooooo0015ooooo`;o000000Coooooo`000?l0003o
0000ROooool3o`000003ooooool0003o000004Kooooo0016ooooo`Ko0000Qoooool6o`0004Oooooo
0016ooooo`Oo0000QOooool7o`0004Oooooo0016ooooo`Oo0000QOooool7o`0004Oooooo0016oooo
o`So0000Poooool8o`0004Oooooo0017ooooo`Go000000?oooooo`000?l00000POooool2o`000003
ooooool0003o000000?o0000B?ooool004Sooooo0ol00003ooooo`Co0000Noooool4o`0000?ooooo
0ol00019ooooo`00D?ooool4o`0007Oooooo1?l0001Aooooo`00D_ooool7o`0006cooooo0ol00000
0ooooooo0000o`000002o`0005?ooooo001Dooooo`Ko0000J_ooool7o`0005Gooooo001Dooooo`Oo
0000J?ooool7o`0005Kooooo001Dooooo`So0000Ioooool7o`0005Kooooo001Dooooo`[o0000Hooo
ool9o`0005Kooooo001Eooooo`Go00000_ooool5o`0005gooooo1Ol000000ooooooo0000o`000003
o`0005Oooooo001Fooooo`?o00001_ooool4o`0000?ooooo0ol0001=ooooo`?o00000oooool4o`00
00Gooooo0ol0001Hooooo`00HOooool9o`0004_ooooo2Ol0001Rooooo`00I?ooool7o`0004Wooooo
1ol0001Uooooo`00I?ooool9o`0004Gooooo2Ol0001Uooooo`00I?ooool<o`0000Gooooo0ol0000^
ooooo`?o00001_ooool<o`0006Gooooo001Uooooo`Go00001?ooool;o`0002cooooo3?l00004oooo
o`Go0000I_ooool006Kooooo0ol00009ooooo`Wo00002oooool3o`0000gooooo0ol0000;ooooo`Wo
00002Oooool3o`0006Oooooo001cooooo`go00001Oooool5o`0000_ooooo1Ol00005ooooo`co0000
MOooool007?ooooo1ol00002ooooobOo000000?oooooo`000?l000001Ol0001eooooo`00M?ooool5
o`0000_ooooo5ol0000:ooooo`Go0000M_ooool007Gooooo0ol0000<ooooo`Oo00002Oooool7o`00
00_ooooo0ol0001gooooo`00QOooool5o`0000_ooooo1Ol00026ooooo`00Q_ooool3o`0000gooooo
0ol00027ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0490189, \
0.00383279, 0.00382591}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Although it is hard to see, each vertex lies on more than two edges\
\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "For something different, let's try the ",
  StyleBox["Prisoner's Dilemma",
    FontSlant->"Italic"],
  " in this case",
  " "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"PrisonersDilemma", "=", 
        RowBox[{"(", GridBox[{
              {\({2.1, 2.1}\), \({0, 3}\)},
              {\({3, 0}\), \({1, 1}\)}
              }], ")"}]}], 
      ";"}], "\n", \(pdColors = {LightGray, Black};\), "\n", \(StrategyLegend[
        pdColors, {"\<Cooperate\>", "\<Defect\>"}];\)}], "Input",
  CellLabel->"In[8]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.666667],
                Background->GrayLevel[0.666667]],
              BoxMargins->False], "Cooperate"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0],
                Background->GrayLevel[0]],
              BoxMargins->False], "Defect"}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[8]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(pd1 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] intNbhd, \[IndentingNewLine]UpdateNeighborhood \[Rule] 
            updNbhd, \[IndentingNewLine]PayoffMatrix \[Rule] 
            PrisonersDilemma, \[IndentingNewLine]StrategyColors \[Rule] 
            pdColors\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(\(SetStrategies[Agents[pd1, {1, 2, 3, 4, 5}], 1];\)\)}], "Input",
  CellLabel->"In[11]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
When both an interaction neighborhood and an update neighborhood \
are specified, the default is to draw the edges in the interaction \
neighborhood black and the edges in the update neighborhood blue. (This can \
be changed if one wishes.) In the default embedding, the difference between \
the interaction and update neighborhoods is barely visible.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ExtractGraph[pd1] // ShowGraph;\)\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0441884 0.910725 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95096 .55707 m
.94374 .5848 L
.94027 .61324 L
s
0 0 1 r
.95096 .55707 m
.94748 .58552 L
.94027 .61324 L
s
.95096 .55707 m
.94561 .58516 L
.94027 .61324 L
s
.95096 .55707 m
.93311 .6114 L
.92263 .66763 L
s
.95096 .55707 m
.94047 .6133 L
.92263 .66763 L
s
.95096 .55707 m
.94716 .5 L
.95096 .44293 L
s
.95096 .55707 m
.95476 .5 L
.95096 .44293 L
s
0 0 0 r
.95096 .55707 m
.95085 .52842 L
.95455 .5 L
s
0 0 1 r
.95096 .55707 m
.95465 .52866 L
.95455 .5 L
s
.95096 .55707 m
.95275 .52854 L
.95455 .5 L
s
0 0 0 r
.94027 .61324 m
.92964 .63985 L
.92263 .66763 L
s
0 0 1 r
.94027 .61324 m
.93325 .64103 L
.92263 .66763 L
s
.94027 .61324 m
.93145 .64044 L
.92263 .66763 L
s
.94027 .61324 m
.91576 .66491 L
.89832 .71937 L
s
.94027 .61324 m
.92282 .66771 L
.89832 .71937 L
s
.94027 .61324 m
.94364 .55615 L
.95455 .5 L
s
.94027 .61324 m
.95117 .5571 L
.95455 .5 L
s
0 0 0 r
.92263 .66763 m
.90875 .69269 L
.89832 .71937 L
s
0 0 1 r
.92263 .66763 m
.9122 .69431 L
.89832 .71937 L
s
.92263 .66763 m
.91047 .6935 L
.89832 .71937 L
s
.92263 .66763 m
.89185 .71581 L
.86773 .76766 L
s
.92263 .66763 m
.89851 .71948 L
.86773 .76766 L
s
0 0 0 r
.89832 .71937 m
.88142 .74249 L
.86773 .76766 L
s
0 0 1 r
.89832 .71937 m
.88463 .74454 L
.86773 .76766 L
s
.89832 .71937 m
.88303 .74351 L
.86773 .76766 L
s
.89832 .71937 m
.86176 .76331 L
.83135 .81172 L
s
.89832 .71937 m
.86791 .76778 L
.83135 .81172 L
s
0 0 0 r
.86773 .76766 m
.84808 .78847 L
.83135 .81172 L
s
0 0 1 r
.86773 .76766 m
.85101 .7909 L
.83135 .81172 L
s
.86773 .76766 m
.84954 .78969 L
.83135 .81172 L
s
.86773 .76766 m
.82597 .80665 L
.78974 .85086 L
s
.86773 .76766 m
.83151 .81186 L
.78974 .85086 L
s
0 0 0 r
.83135 .81172 m
.80924 .8299 L
.78974 .85086 L
s
0 0 1 r
.83135 .81172 m
.81185 .83268 L
.78974 .85086 L
s
.83135 .81172 m
.81054 .83129 L
.78974 .85086 L
s
.83135 .81172 m
.78503 .84516 L
.74356 .88448 L
s
.83135 .81172 m
.78987 .85103 L
.74356 .88448 L
s
0 0 0 r
.78974 .85086 m
.76553 .86613 L
.74356 .88448 L
s
0 0 1 r
.78974 .85086 m
.76777 .86921 L
.74356 .88448 L
s
.78974 .85086 m
.76665 .86767 L
.74356 .88448 L
s
.78974 .85086 m
.7396 .87823 L
.69354 .91202 L
s
.78974 .85086 m
.74367 .88466 L
.69354 .91202 L
s
0 0 0 r
.74356 .88448 m
.71763 .89658 L
.69354 .91202 L
s
0 0 1 r
.74356 .88448 m
.71946 .89992 L
.69354 .91202 L
s
.74356 .88448 m
.71855 .89825 L
.69354 .91202 L
s
.74356 .88448 m
.69039 .90533 L
.64046 .93308 L
s
.74356 .88448 m
.69363 .91222 L
.64046 .93308 L
s
0 0 0 r
.69354 .91202 m
.6663 .92078 L
.64046 .93308 L
s
0 0 1 r
.69354 .91202 m
.6677 .92432 L
.64046 .93308 L
s
.69354 .91202 m
.667 .92255 L
.64046 .93308 L
s
.69354 .91202 m
.63818 .92604 L
.58517 .9473 L
s
.69354 .91202 m
.64053 .93328 L
.58517 .9473 L
s
0 0 0 r
.64046 .93308 m
.61234 .93834 L
.58517 .9473 L
s
0 0 1 r
.64046 .93308 m
.61329 .94203 L
.58517 .9473 L
s
.64046 .93308 m
.61282 .94019 L
.58517 .9473 L
s
.64046 .93308 m
.58379 .94003 L
.52854 .95446 L
s
.64046 .93308 m
.58521 .94751 L
.52854 .95446 L
s
0 0 0 r
.58517 .9473 m
.55662 .94899 L
.52854 .95446 L
s
0 0 1 r
.58517 .9473 m
.5571 .95277 L
.52854 .95446 L
s
.58517 .9473 m
.55686 .95088 L
.52854 .95446 L
s
.58517 .9473 m
.52808 .94708 L
.47146 .95446 L
s
.58517 .9473 m
.52855 .95468 L
.47146 .95446 L
s
0 0 0 r
.52854 .95446 m
.5 .95637 L
.47146 .95446 L
s
0 0 1 r
.52854 .95446 m
.5 .95256 L
.47146 .95446 L
s
.52854 .95446 m
.5 .95446 L
.47146 .95446 L
s
.52854 .95446 m
.47145 .95468 L
.41483 .9473 L
s
.52854 .95446 m
.47192 .94708 L
.41483 .9473 L
s
0 0 0 r
.47146 .95446 m
.4429 .95277 L
.41483 .9473 L
s
0 0 1 r
.47146 .95446 m
.44338 .94899 L
.41483 .9473 L
s
.47146 .95446 m
.44314 .95088 L
.41483 .9473 L
s
.47146 .95446 m
.41479 .94751 L
.35954 .93308 L
s
.47146 .95446 m
.41621 .94003 L
.35954 .93308 L
s
0 0 0 r
.41483 .9473 m
.38671 .94203 L
.35954 .93308 L
s
0 0 1 r
.41483 .9473 m
.38766 .93834 L
.35954 .93308 L
s
.41483 .9473 m
.38718 .94019 L
.35954 .93308 L
s
.41483 .9473 m
.35947 .93328 L
.30646 .91202 L
s
.41483 .9473 m
.36182 .92604 L
.30646 .91202 L
s
0 0 0 r
.35954 .93308 m
.3323 .92432 L
.30646 .91202 L
s
0 0 1 r
.35954 .93308 m
.3337 .92078 L
.30646 .91202 L
s
.35954 .93308 m
.333 .92255 L
.30646 .91202 L
s
.35954 .93308 m
.30637 .91222 L
.25644 .88448 L
s
.35954 .93308 m
.30961 .90533 L
.25644 .88448 L
s
0 0 0 r
.30646 .91202 m
.28054 .89992 L
.25644 .88448 L
s
0 0 1 r
.30646 .91202 m
.28237 .89658 L
.25644 .88448 L
s
.30646 .91202 m
.28145 .89825 L
.25644 .88448 L
s
.30646 .91202 m
.25633 .88466 L
.21026 .85086 L
s
.30646 .91202 m
.2604 .87823 L
.21026 .85086 L
s
0 0 0 r
.25644 .88448 m
.23223 .86921 L
.21026 .85086 L
s
0 0 1 r
.25644 .88448 m
.23447 .86613 L
.21026 .85086 L
s
.25644 .88448 m
.23335 .86767 L
.21026 .85086 L
s
.25644 .88448 m
.21013 .85103 L
.16865 .81172 L
s
.25644 .88448 m
.21497 .84516 L
.16865 .81172 L
s
0 0 0 r
.21026 .85086 m
.18815 .83268 L
.16865 .81172 L
s
0 0 1 r
.21026 .85086 m
.19076 .8299 L
.16865 .81172 L
s
.21026 .85086 m
.18946 .83129 L
.16865 .81172 L
s
.21026 .85086 m
.16849 .81186 L
.13227 .76766 L
s
.21026 .85086 m
.17403 .80665 L
.13227 .76766 L
s
0 0 0 r
.16865 .81172 m
.14899 .7909 L
.13227 .76766 L
s
0 0 1 r
.16865 .81172 m
.15192 .78847 L
.13227 .76766 L
s
.16865 .81172 m
.15046 .78969 L
.13227 .76766 L
s
.16865 .81172 m
.13209 .76778 L
.10168 .71937 L
s
.16865 .81172 m
.13824 .76331 L
.10168 .71937 L
s
0 0 0 r
.13227 .76766 m
.11537 .74454 L
.10168 .71937 L
s
0 0 1 r
.13227 .76766 m
.11858 .74249 L
.10168 .71937 L
s
.13227 .76766 m
.11697 .74351 L
.10168 .71937 L
s
.13227 .76766 m
.10149 .71948 L
.07737 .66763 L
s
.13227 .76766 m
.10815 .71581 L
.07737 .66763 L
s
0 0 0 r
.10168 .71937 m
.0878 .69431 L
.07737 .66763 L
s
0 0 1 r
.10168 .71937 m
.09125 .69269 L
.07737 .66763 L
s
.10168 .71937 m
.08953 .6935 L
.07737 .66763 L
s
.10168 .71937 m
.07718 .66771 L
.05973 .61324 L
s
.10168 .71937 m
.08424 .66491 L
.05973 .61324 L
s
0 0 0 r
.07737 .66763 m
.06675 .64103 L
.05973 .61324 L
s
0 0 1 r
.07737 .66763 m
.07036 .63985 L
.05973 .61324 L
s
.07737 .66763 m
.06855 .64044 L
.05973 .61324 L
s
.07737 .66763 m
.05953 .6133 L
.04904 .55707 L
s
.07737 .66763 m
.06689 .6114 L
.04904 .55707 L
s
0 0 0 r
.05973 .61324 m
.05252 .58552 L
.04904 .55707 L
s
0 0 1 r
.05973 .61324 m
.05626 .5848 L
.04904 .55707 L
s
.05973 .61324 m
.05439 .58516 L
.04904 .55707 L
s
.05973 .61324 m
.04883 .5571 L
.04545 .5 L
s
.05973 .61324 m
.05636 .55615 L
.04545 .5 L
s
0 0 0 r
.04904 .55707 m
.04535 .52866 L
.04545 .5 L
s
0 0 1 r
.04904 .55707 m
.04915 .52842 L
.04545 .5 L
s
.04904 .55707 m
.04725 .52854 L
.04545 .5 L
s
.04904 .55707 m
.04524 .5 L
.04904 .44293 L
s
.04904 .55707 m
.05284 .5 L
.04904 .44293 L
s
0 0 0 r
.04545 .5 m
.04535 .47134 L
.04904 .44293 L
s
0 0 1 r
.04545 .5 m
.04915 .47158 L
.04904 .44293 L
s
.04545 .5 m
.04725 .47146 L
.04904 .44293 L
s
.04545 .5 m
.04883 .4429 L
.05973 .38676 L
s
.04545 .5 m
.05636 .44385 L
.05973 .38676 L
s
0 0 0 r
.04904 .44293 m
.05252 .41448 L
.05973 .38676 L
s
0 0 1 r
.04904 .44293 m
.05626 .4152 L
.05973 .38676 L
s
.04904 .44293 m
.05439 .41484 L
.05973 .38676 L
s
.04904 .44293 m
.05953 .3867 L
.07737 .33237 L
s
.04904 .44293 m
.06689 .3886 L
.07737 .33237 L
s
0 0 0 r
.05973 .38676 m
.06675 .35897 L
.07737 .33237 L
s
0 0 1 r
.05973 .38676 m
.07036 .36015 L
.07737 .33237 L
s
.05973 .38676 m
.06855 .35956 L
.07737 .33237 L
s
.05973 .38676 m
.07718 .33229 L
.10168 .28063 L
s
.05973 .38676 m
.08424 .33509 L
.10168 .28063 L
s
0 0 0 r
.07737 .33237 m
.0878 .30569 L
.10168 .28063 L
s
0 0 1 r
.07737 .33237 m
.09125 .30731 L
.10168 .28063 L
s
.07737 .33237 m
.08953 .3065 L
.10168 .28063 L
s
.07737 .33237 m
.10149 .28052 L
.13227 .23234 L
s
.07737 .33237 m
.10815 .28419 L
.13227 .23234 L
s
0 0 0 r
.10168 .28063 m
.11537 .25546 L
.13227 .23234 L
s
0 0 1 r
.10168 .28063 m
.11858 .25751 L
.13227 .23234 L
s
.10168 .28063 m
.11697 .25649 L
.13227 .23234 L
s
.10168 .28063 m
.13209 .23222 L
.16865 .18828 L
s
.10168 .28063 m
.13824 .23669 L
.16865 .18828 L
s
0 0 0 r
.13227 .23234 m
.14899 .2091 L
.16865 .18828 L
s
0 0 1 r
.13227 .23234 m
.15192 .21153 L
.16865 .18828 L
s
.13227 .23234 m
.15046 .21031 L
.16865 .18828 L
s
.13227 .23234 m
.16849 .18814 L
.21026 .14914 L
s
.13227 .23234 m
.17403 .19335 L
.21026 .14914 L
s
0 0 0 r
.16865 .18828 m
.18815 .16732 L
.21026 .14914 L
s
0 0 1 r
.16865 .18828 m
.19076 .1701 L
.21026 .14914 L
s
.16865 .18828 m
.18946 .16871 L
.21026 .14914 L
s
.16865 .18828 m
.21013 .14897 L
.25644 .11552 L
s
.16865 .18828 m
.21497 .15484 L
.25644 .11552 L
s
0 0 0 r
.21026 .14914 m
.23223 .13079 L
.25644 .11552 L
s
0 0 1 r
.21026 .14914 m
.23447 .13387 L
.25644 .11552 L
s
.21026 .14914 m
.23335 .13233 L
.25644 .11552 L
s
.21026 .14914 m
.25633 .11534 L
.30646 .08798 L
s
.21026 .14914 m
.2604 .12177 L
.30646 .08798 L
s
0 0 0 r
.25644 .11552 m
.28054 .10008 L
.30646 .08798 L
s
0 0 1 r
.25644 .11552 m
.28237 .10342 L
.30646 .08798 L
s
.25644 .11552 m
.28145 .10175 L
.30646 .08798 L
s
.25644 .11552 m
.30637 .08778 L
.35954 .06692 L
s
.25644 .11552 m
.30961 .09467 L
.35954 .06692 L
s
0 0 0 r
.30646 .08798 m
.3323 .07568 L
.35954 .06692 L
s
0 0 1 r
.30646 .08798 m
.3337 .07922 L
.35954 .06692 L
s
.30646 .08798 m
.333 .07745 L
.35954 .06692 L
s
.30646 .08798 m
.35947 .06672 L
.41483 .0527 L
s
.30646 .08798 m
.36182 .07396 L
.41483 .0527 L
s
0 0 0 r
.35954 .06692 m
.38671 .05797 L
.41483 .0527 L
s
0 0 1 r
.35954 .06692 m
.38766 .06166 L
.41483 .0527 L
s
.35954 .06692 m
.38718 .05981 L
.41483 .0527 L
s
.35954 .06692 m
.41479 .05249 L
.47146 .04554 L
s
.35954 .06692 m
.41621 .05997 L
.47146 .04554 L
s
0 0 0 r
.41483 .0527 m
.4429 .04723 L
.47146 .04554 L
s
0 0 1 r
.41483 .0527 m
.44338 .05101 L
.47146 .04554 L
s
.41483 .0527 m
.44314 .04912 L
.47146 .04554 L
s
.41483 .0527 m
.47145 .04532 L
.52854 .04554 L
s
.41483 .0527 m
.47192 .05292 L
.52854 .04554 L
s
0 0 0 r
.47146 .04554 m
.5 .04744 L
.52854 .04554 L
s
0 0 1 r
.47146 .04554 m
.5 .04363 L
.52854 .04554 L
s
.47146 .04554 m
.5 .04554 L
.52854 .04554 L
s
.47146 .04554 m
.52808 .05292 L
.58517 .0527 L
s
.47146 .04554 m
.52855 .04532 L
.58517 .0527 L
s
0 0 0 r
.52854 .04554 m
.55662 .05101 L
.58517 .0527 L
s
0 0 1 r
.52854 .04554 m
.5571 .04723 L
.58517 .0527 L
s
.52854 .04554 m
.55686 .04912 L
.58517 .0527 L
s
.52854 .04554 m
.58379 .05997 L
.64046 .06692 L
s
.52854 .04554 m
.58521 .05249 L
.64046 .06692 L
s
0 0 0 r
.58517 .0527 m
.61234 .06166 L
.64046 .06692 L
s
0 0 1 r
.58517 .0527 m
.61329 .05797 L
.64046 .06692 L
s
.58517 .0527 m
.61282 .05981 L
.64046 .06692 L
s
.58517 .0527 m
.63818 .07396 L
.69354 .08798 L
s
.58517 .0527 m
.64053 .06672 L
.69354 .08798 L
s
0 0 0 r
.64046 .06692 m
.6663 .07922 L
.69354 .08798 L
s
0 0 1 r
.64046 .06692 m
.6677 .07568 L
.69354 .08798 L
s
.64046 .06692 m
.667 .07745 L
.69354 .08798 L
s
.64046 .06692 m
.69039 .09467 L
.74356 .11552 L
s
.64046 .06692 m
.69363 .08778 L
.74356 .11552 L
s
0 0 0 r
.69354 .08798 m
.71763 .10342 L
.74356 .11552 L
s
0 0 1 r
.69354 .08798 m
.71946 .10008 L
.74356 .11552 L
s
.69354 .08798 m
.71855 .10175 L
.74356 .11552 L
s
.69354 .08798 m
.7396 .12177 L
.78974 .14914 L
s
.69354 .08798 m
.74367 .11534 L
.78974 .14914 L
s
0 0 0 r
.74356 .11552 m
.76553 .13387 L
.78974 .14914 L
s
0 0 1 r
.74356 .11552 m
.76777 .13079 L
.78974 .14914 L
s
.74356 .11552 m
.76665 .13233 L
.78974 .14914 L
s
.74356 .11552 m
.78503 .15484 L
.83135 .18828 L
s
.74356 .11552 m
.78987 .14897 L
.83135 .18828 L
s
0 0 0 r
.78974 .14914 m
.80924 .1701 L
.83135 .18828 L
s
0 0 1 r
.78974 .14914 m
.81185 .16732 L
.83135 .18828 L
s
.78974 .14914 m
.81054 .16871 L
.83135 .18828 L
s
.78974 .14914 m
.82597 .19335 L
.86773 .23234 L
s
.78974 .14914 m
.83151 .18814 L
.86773 .23234 L
s
0 0 0 r
.83135 .18828 m
.84808 .21153 L
.86773 .23234 L
s
0 0 1 r
.83135 .18828 m
.85101 .2091 L
.86773 .23234 L
s
.83135 .18828 m
.84954 .21031 L
.86773 .23234 L
s
.83135 .18828 m
.86176 .23669 L
.89832 .28063 L
s
.83135 .18828 m
.86791 .23222 L
.89832 .28063 L
s
0 0 0 r
.86773 .23234 m
.88142 .25751 L
.89832 .28063 L
s
0 0 1 r
.86773 .23234 m
.88463 .25546 L
.89832 .28063 L
s
.86773 .23234 m
.88303 .25649 L
.89832 .28063 L
s
.86773 .23234 m
.89185 .28419 L
.92263 .33237 L
s
.86773 .23234 m
.89851 .28052 L
.92263 .33237 L
s
0 0 0 r
.89832 .28063 m
.90875 .30731 L
.92263 .33237 L
s
0 0 1 r
.89832 .28063 m
.9122 .30569 L
.92263 .33237 L
s
.89832 .28063 m
.91047 .3065 L
.92263 .33237 L
s
.89832 .28063 m
.91576 .33509 L
.94027 .38676 L
s
.89832 .28063 m
.92282 .33229 L
.94027 .38676 L
s
0 0 0 r
.92263 .33237 m
.92964 .36015 L
.94027 .38676 L
s
0 0 1 r
.92263 .33237 m
.93325 .35897 L
.94027 .38676 L
s
.92263 .33237 m
.93145 .35956 L
.94027 .38676 L
s
.92263 .33237 m
.93311 .3886 L
.95096 .44293 L
s
.92263 .33237 m
.94047 .3867 L
.95096 .44293 L
s
0 0 0 r
.94027 .38676 m
.94374 .4152 L
.95096 .44293 L
s
0 0 1 r
.94027 .38676 m
.94748 .41448 L
.95096 .44293 L
s
.94027 .38676 m
.94561 .41484 L
.95096 .44293 L
s
.94027 .38676 m
.94364 .44385 L
.95455 .5 L
s
.94027 .38676 m
.95117 .4429 L
.95455 .5 L
s
0 0 0 r
.95096 .44293 m
.95085 .47158 L
.95455 .5 L
s
0 0 1 r
.95096 .44293 m
.95465 .47134 L
.95455 .5 L
s
.95096 .44293 m
.95275 .47146 L
.95455 .5 L
s
0 0 0 r
.025 w
.95096 .55707 Mdot
.94027 .61324 Mdot
.92263 .66763 Mdot
.89832 .71937 Mdot
.663 .663 .663 r
.86773 .76766 Mdot
0 0 0 r
.83135 .81172 Mdot
.78974 .85086 Mdot
.74356 .88448 Mdot
.663 .663 .663 r
.69354 .91202 Mdot
.64046 .93308 Mdot
.58517 .9473 Mdot
.52854 .95446 Mdot
0 0 0 r
.47146 .95446 Mdot
.663 .663 .663 r
.41483 .9473 Mdot
0 0 0 r
.35954 .93308 Mdot
.30646 .91202 Mdot
.663 .663 .663 r
.25644 .88448 Mdot
0 0 0 r
.21026 .85086 Mdot
.16865 .81172 Mdot
.663 .663 .663 r
.13227 .76766 Mdot
.10168 .71937 Mdot
0 0 0 r
.07737 .66763 Mdot
.05973 .61324 Mdot
.663 .663 .663 r
.04904 .55707 Mdot
0 0 0 r
.04545 .5 Mdot
.04904 .44293 Mdot
.05973 .38676 Mdot
.07737 .33237 Mdot
.663 .663 .663 r
.10168 .28063 Mdot
.13227 .23234 Mdot
.16865 .18828 Mdot
0 0 0 r
.21026 .14914 Mdot
.25644 .11552 Mdot
.30646 .08798 Mdot
.35954 .06692 Mdot
.41483 .0527 Mdot
.47146 .04554 Mdot
.663 .663 .663 r
.52854 .04554 Mdot
.58517 .0527 Mdot
.64046 .06692 Mdot
.69354 .08798 Mdot
0 0 0 r
.74356 .11552 Mdot
.663 .663 .663 r
.78974 .14914 Mdot
0 0 0 r
.83135 .18828 Mdot
.86773 .23234 Mdot
.89832 .28063 Mdot
.663 .663 .663 r
.92263 .33237 Mdot
.94027 .38676 Mdot
.95096 .44293 Mdot
0 0 0 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[14]:=",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00Q_ooool3o`0000gooooo0onYZJV7ooooo`00QOooool5o`0000_ooooo1OnYZJV6ooooo`00
MOooool3o`0000cooooo1ol00009o`00o`OoZJVY2oooool3ojVYZGOooooo001dooooo`Go00001?oo
ool6o`000003o`00ool0003o000000Go00002Ol00?l7ojVYZ@Oo003o0oooool5ojVYZGKooooo001c
ooooo`So00002Ol00?l7o`0000Wo003o1onYZJT9o`00o`OoZJVYMOooool007?ooooo1ol0000;o`00
o`Go000000?o003oooooooooool01oooool00ol00?ooZJVYojVYZ@03ojVYZ@[o003o1onYZJT2o`00
og?ooooo001Vooooo`?o00001?ooool4o`0000;o003o1ol00003o`00o`Wooooo0ol0000=ooooo`?o
ZJVY2Oooool2o`00o`OoZJVY1ol00?l4ooooo`?oZJVYIoooool006Gooooo2?l00007o`00o`Go0000
;?ooool5ojVYZ@_o003o1OnYZJUVooooo`00I?ooool7o`0000So003o0_ooool3o`0002kooooo0onY
ZJT3ooooo`So003o1onYZJUUooooo`00I?ooool7o`0000?o003o@oooool3o`00o`OoZJVYIOooool0
067ooooo0_l000000ol00?oo0000o`000005o`0004Wooooo1onYZJT3o`00of;ooooo001Nooooo`;o
00001?l00?l00ooooooo0000o`000003o`0004_ooooo1OnYZJT00ooooooo003oo`00o`04o`00oeoo
oooo001Fooooo`?o00000oooool2o`0000Oo003o00Coooooo`000?l0003o0000COooool3ojVYZ@03
ooooool00?oo003o00Oo003o0_ooool3ojVYZESooooo001Eooooo`Ko00001_l00?mMooooo`Ko003o
1OnYZJUGooooo`00E?ooool7o`0000Co003oHOooool3o`00o`OoZJVYE_ooool005Cooooo1ol00002
o`00ofGooooo00?o003oojVYZOnYZJT01OnYZJUFooooo`00Doooool8o`0006Sooooo1onYZJT2o`00
oeCooooo001Aooooo`;o00000_l00?l5o`0006[ooooo1OnYZJT00ooooooo003oo`00o`02o`00oe;o
oooo001?ooooo`;o00000_l00?l00ooooooo003oo`00o`03o`0006cooooo0onYZJT3o`00o`03oooo
ool00?oo003o00;o003oD?ooool004kooooo00?o0000o`00ool00?l00ol00?mgooooo`Ko003oCooo
ool004Sooooo0ol000000ooooooo0000o`00o`04o`00og_ooooo1_l00?l01?oooooo0000o`000?l0
0019ooooo`00Aoooool5o`0000?o003oPOooool00ol00?oo0000o`00o`05o`0004Sooooo0016oooo
o`Oo000000?o003oooooooooool0POooool00ol00?oo0000o`000005o`0004Oooooo0016ooooo`Oo
0000QOooool7o`0004Oooooo0016ooooo`Oo0000QOooool7o`0004Oooooo0015ooooo`;o003o1Ol0
0027ooooo`Go00000_l00?m6ooooo`00@oooool4o`00o`04ooooool0003o0000o`0008Wooooo0ol0
00000ooooooo003oo`00o`02o`00odCooooo0011ooooo`Go003oToooool5o`00od;ooooo0010oooo
o`Co003oUoooool01?l00?oo0000o`00ool00?m1ooooo`00>oooool3o`000003ooooool00?oo003o
00;o003oVOooool01Ol00?oo0000o`00ool00?oooooo00?oZJVY??ooool003[ooooo1Ol000000ol0
0?ooooooo`00o`2Kooooo`03o`00oooooooo003o00GoZJVY>oooool003Wooooo1ol000000ol00?oo
ooooooooo`2Kooooo`03o`00oonYZJWoZJVY00GoZJVY>_ooool003Wooooo1ol0002Oooooo`OoZJVY
>_ooool003Wooooo1ol0002Oooooo`OoZJVY>_ooool003Wooooo00?o003oo`000?l000000ol0002Q
ooooo`GoZJVY00?o003oooooooooool0>?ooool003Sooooo0ol00?l3o`000:?ooooo0onYZJT3o`00
ocWooooo000gooooo`Co003oZOooool4o`00ocSooooo000fooooo`Co003oZoooool4o`00ocOooooo
000eooooo`?o003o[oooool3o`00ocKooooo000dooooo`?o003o/Oooool3o`00ocGooooo000coooo
o`Co003o/Oooool4o`00ocCooooo000_ooooo`?oZJVY1?l00?ncooooo`Co003o0ol0000`ooooo`00
;_ooool5ojVYZ@03ooooool00?oooooo0;Cooooo00?o003oooooool000001?l0000_ooooo`00;Ooo
ool7ojVYZKOooooo1ol0000^ooooo`00;Oooool7ojVYZKOooooo1ol0000^ooooo`00;Oooool7ojVY
ZKOooooo1ol0000^ooooo`00;_ooool5ojVYZKWooooo1Ol0000_ooooo`00;Oooool2o`00o`?oZJVY
^oooool3o`0000;o003o;_ooool002cooooo0ol00?o1ooooo`?o003o;Oooool002_ooooo0ol00?o3
ooooo`?o003o;?ooool002[ooooo0ol00?o5ooooo`?o003o:oooool002[ooooo0_l00?o7ooooo`;o
003o:oooool002Wooooo00?o003oooooool00?l0aoooool00ol00?ooooooo`00o`0Zooooo`009?oo
ool3ojVYZ@04ooooool00?ooooooo`00olWooooo00?o003oooooool00?l00ol0000Vooooo`008ooo
ool5ojVYZ@03ooooool00?oooooo0<[ooooo00?o003oo`000?l000000ol0000Uooooo`008_ooool7
ojVYZ@03o`00oooooooooooo0<Wooooo1ol0000Tooooo`008_ooool7ojVYZLcooooo1ol0000Toooo
o`008_ooool7ojVYZLcooooo1ol0000Tooooo`008oooool5ojVYZLkooooo1Ol0000Uooooo`009?oo
ool3ojVYZM3ooooo0ol000000ol00?ooooooooooo`0Sooooo`008oooool3o`00om?ooooo0ol00?lT
ooooo`008_ooool3o`00omGooooo0ol00?lSooooo`008_ooool3o`00omGooooo0ol00?lSooooo`00
8Oooool3o`00omOooooo0ol00?lRooooo`008Oooool2o`00omWooooo0_l00?lRooooo`008?ooool3
o`00omWooooo00?o003oo`000?l00?l08Oooool0023ooooo0_l00?oKooooo`;o003o8Oooool001co
oooo0onYZJT3o`00om_ooooo0ol00?l3o`0001gooooo000Kooooo`GoZJVY00?o003oooooooooool0
foooool00ol00?oo0000o`000003o`0001cooooo000Jooooo`OoZJVYgOooool7o`0001_ooooo000J
ooooo`OoZJVYgOooool7o`0001_ooooo000Jooooo`OoZJVYgOooool7o`0001_ooooo000Kooooo`Go
ZJVYgoooool5o`0001cooooo000Looooo`?oZJVYhOooool3o`0001gooooo000Kooooo`?o003ohooo
ool3o`00oacooooo000Jooooo`Co003ohoooool4o`00oa_ooooo000Jooooo`?o003oiOooool3o`00
oa_ooooo000Iooooo`?o003oioooool3o`00oa[ooooo000Iooooo`?o003oioooool00ol00?oo0000
o`00o`0Jooooo`006?ooool3o`00onWooooo0ol00?lIooooo`006?ooool3o`00onWooooo0ol00?lI
ooooo`006?ooool3o`00onWooooo00?o003oo`000?l00?l06Oooool001Gooooo0ol00002o`00on_o
oooo0_l00?l3ojVYZAKooooo000Dooooo`Go000000?o003oooooooooool0jOooool00ol00?ooZJVY
ojVYZ@03ojVYZAGooooo000Cooooo`Oo0000joooool7ojVYZACooooo000Cooooo`Oo0000joooool7
ojVYZACooooo000Cooooo`Oo0000joooool7ojVYZACooooo000Dooooo`Go0000kOooool5ojVYZAGo
oooo000Eooooo`?o0000koooool3ojVYZAKooooo000Eooooo`03o`00oooooooo003o0>oooooo00?o
003oooooool00?l05_ooool001Cooooo0ol00?oaooooo`?o003o5Oooool001Cooooo0ol00?oaoooo
o`?o003o5Oooool001Cooooo0_l00?ocooooo`;o003o5Oooool001?ooooo0ol00?ocooooo`03o`00
ool0003o003o01Cooooo000Cooooo`;o003omOooool2o`00oaCooooo000Cooooo`;o003omOooool2
o`00oaCooooo000Booooo`?o003omOooool00ol00?oo0000o`00o`0Cooooo`004_ooool3o`00ooGo
oooo0ol00?lCooooo`004?ooool3o`000003ooooool00?oooooo0?Cooooo00?o003ooooooonYZJT0
0_nYZJTAooooo`003oooool5o`000?Oooooo1OnYZJT@ooooo`003_ooool7o`000?Gooooo1onYZJT?
ooooo`003_ooool7o`000?Gooooo1onYZJT?ooooo`003_ooool7o`000?Gooooo1onYZJT?ooooo`00
3oooool5o`000?Oooooo1OnYZJT@ooooo`004?ooool3o`000?Wooooo0onYZJTAooooo`004?ooool3
o`00ooWooooo0ol00?lAooooo`004?ooool3o`00ooWooooo0ol00?lAooooo`003oooool00ol0003o
003oo`00o`3kooooo`?o003o4?ooool000oooooo0ol00?okooooo`?o003o4?ooool000oooooo0ol0
0?okooooo`?o003o4?ooool000oooooo0_l00?omooooo`;o003o4?ooool000oooooo0_l00?omoooo
o`;o003o4?ooool000oooooo0_l00?omooooo`;o003o4?ooool000kooooo00?o0000o`00ool00?l0
oOooool3o`00o`oooooo000=ooooo`?o000000?o003oooooooooool0noooool01?l00?ooZJVYojVY
ZOnYZJT>ooooo`003?ooool5o`000?gooooo1OnYZJT=ooooo`002oooool7o`000?_ooooo1onYZJT<
ooooo`002oooool7o`000?_ooooo1onYZJT<ooooo`002oooool7o`000?_ooooo1onYZJT<ooooo`00
3?ooool5o`000?gooooo1OnYZJT=ooooo`003Oooool3o`000003o`00oooooooooooo0?_ooooo00Co
003oojVYZOnYZJWoZJVY3_ooool000kooooo0_l00?ooooooo`;o003o3oooool000gooooo0ol00?oo
ooooo`03o`00ool0003o003o00kooooo000=ooooo`?o003oooooool00ol00?oo0000o`00o`0>oooo
o`003Oooool3o`00oooooooo00?o003oo`000?l00?l03_ooool000gooooo0ol00?ooooooo`03o`00
ool0003o003o00kooooo000=ooooo`?o003oooooool00ol00?oo0000o`00o`0>ooooo`003Oooool3
o`00oooooooo00?o003oo`000?l00?l03_ooool000gooooo0ol00?ooooooo`03o`00ool0003o003o
00kooooo000=ooooo`?o003oooooool00ol00?oo0000o`00o`0>ooooo`002oooool3o`0000;o003o
ooooool01?l00?oo0000o`000?l0000=ooooo`002_ooool5o`000003o`00oooooooooooo0?gooooo
1Ol0000<ooooo`002Oooool7o`000?kooooo1ol0000;ooooo`002Oooool7o`000?kooooo1ol0000;
ooooo`002Oooool7o`000?kooooo1ol0000;ooooo`002_ooool5o`000003o`00oooooooooooo0?go
oooo1Ol0000<ooooo`002oooool3o`000003ooooool00?oooooo0?kooooo00Co003oo`000?l0003o
00003Oooool000cooooo1?l00?ooooooo`Co003o3Oooool000gooooo0ol00?ooooooo`?o003o3_oo
ool000gooooo0ol00?ooooooo`?o003o3_ooool000gooooo0ol00?ooooooo`?o003o3_ooool000go
oooo0ol00?ooooooo`?o003o3_ooool000gooooo0ol00?ooooooo`?o003o3_ooool000gooooo0ol0
0?ooooooo`?o003o3_ooool000gooooo0ol00?ooooooo`?o003o3_ooool000gooooo0ol00?oooooo
o`?o003o3_ooool000gooooo0onYZJWoooooo`?o00003_ooool000cooooo1OnYZJWmooooo`Go0000
3Oooool000_ooooo1onYZJWkooooo`Oo00003?ooool000_ooooo1onYZJWkooooo`Oo00003?ooool0
00_ooooo1onYZJWkooooo`Oo00003?ooool000cooooo1OnYZJWmooooo`Go00003Oooool000gooooo
0onYZJT00ol00?ooooooooooo`3kooooo`04o`00ool0003o0000o`0000kooooo000>ooooo`?o003o
oOooool3o`00o`oooooo000?ooooo`;o003ooOooool2o`00oa3ooooo000?ooooo`;o003ooOooool2
o`00oa3ooooo000?ooooo`;o003ooOooool2o`00oa3ooooo000?ooooo`;o003ooOooool2o`00oa3o
oooo000?ooooo`?o003onoooool3o`00oa3ooooo000?ooooo`?o003onoooool3o`00oa3ooooo000?
ooooo`03o`000?l00?oo003o0?_ooooo0ol00?l@ooooo`004?ooool3o`00ooWooooo0ol00?lAoooo
o`004?ooool3o`00ooWooooo0ol00?lAooooo`004?ooool3o`000?Wooooo0ol0000Aooooo`003ooo
ool5o`000?Oooooo1Ol0000@ooooo`003_ooool7o`000?Gooooo1ol0000?ooooo`003_ooool7o`00
0?Gooooo1ol0000?ooooo`003_ooool7o`000?Gooooo1ol0000?ooooo`003oooool5o`000?Oooooo
1Ol0000@ooooo`004?ooool3o`0000;o003omOooool2o`00o`?o00004Oooool001;ooooo0ol00?oe
ooooo`?o003o4oooool001;ooooo0ol00?oeooooo`?o003o4oooool001?ooooo0_l00?oeooooo`;o
003o5?ooool001?ooooo0ol00?ocooooo`03o`00ool0003o003o01Cooooo000Cooooo`?o003olooo
ool3o`00oaCooooo000Dooooo`;o003oloooool2o`00oaGooooo000Dooooo`?o003olOooool00ol0
0?oo0000o`00o`0Eooooo`005?ooool3o`00oo7ooooo0ol00?lEooooo`005Oooool3o`000>oooooo
0ol0000Fooooo`005?ooool5o`000>gooooo1Ol0000Eooooo`004oooool7o`000>_ooooo1ol0000D
ooooo`004oooool7o`000>_ooooo1ol0000Dooooo`004oooool7o`000>_ooooo1ol0000Dooooo`00
5?ooool5o`000003o`00oooooooooooo0>Wooooo00?o003oo`000?l000000ol0000Eooooo`005Ooo
ool3o`0000;o003ojoooool2o`00o`?o00005_ooool001Oooooo0ol00?o[ooooo`?o003o6?ooool0
01Sooooo0ol00?oYooooo`?o003o6Oooool001Sooooo0ol00?oYooooo`?o003o6Oooool001Sooooo
0ol00?oYooooo`?o003o6Oooool001Wooooo0ol00?oWooooo`?o003o6_ooool001Wooooo0ol00?oW
ooooo`?o003o6_ooool001[ooooo0ol00?oUooooo`03o`00ool0003o003o01_ooooo000Jooooo`Co
003ohoooool4o`00oa_ooooo000Kooooo`04o`00oonYZJWoZJVYojVYZN7ooooo0ol000000ol00?oo
ooooooooo`0Jooooo`006oooool5ojVYZMoooooo1Ol0000Looooo`006_ooool7ojVYZMgooooo1ol0
000Kooooo`006_ooool7ojVYZMgooooo1ol0000Kooooo`006_ooool7ojVYZMgooooo1ol0000Koooo
o`006oooool5ojVYZ@03o`00oooooooooooo0=_ooooo00?o003oo`000?l000000ol0000Looooo`00
7?ooool3ojVYZ@;o003ogOooool2o`00o`?o00007Oooool001oooooo0ol00?oKooooo`?o003o8?oo
ool0023ooooo0_l00?oKooooo`;o003o8Oooool0023ooooo0ol00?oIooooo`?o003o8Oooool0027o
oooo0_l00?oIooooo`;o003o8_ooool0027ooooo0ol00?oGooooo`03o`00ool0003o003o02;ooooo
000Rooooo`?o003oeOooool00ol00?oo0000o`00o`0Sooooo`008_ooool3o`00omGooooo00?o003o
o`000?l00?l08oooool002?ooooo00Co003oojVYZOnYZJWoZJVYd?ooool3ojVYZ@03o`000?l00?oo
oooo02?ooooo000Sooooo`GoZJVYc_ooool5ojVYZBGooooo000Rooooo`OoZJVYc?ooool7ojVYZBCo
oooo000Rooooo`OoZJVYc?ooool7ojVYZBCooooo000Rooooo`OoZJVYc?ooool7ojVYZBCooooo000S
ooooo`GoZJVY0_l00?o;ooooo`03o`00oonYZJWoZJVY00?oZJVY9Oooool002Cooooo0onYZJT3o`00
ol_ooooo0_l00?l3ojVYZBKooooo000Xooooo`?o003obOooool3o`00obWooooo000Yooooo`?o003o
aoooool3o`00ob[ooooo000Zooooo`?o003oaOooool3o`00ob_ooooo000[ooooo`?o003o`oooool3
o`00obcooooo000[ooooo`Co003o`Oooool4o`00obcooooo000/ooooo`Co003o_oooool4o`00obgo
oooo000]ooooo`;o003o0ol0002kooooo`?o00000_l00?l^ooooo`00;_ooool5o`000;Wooooo1Ol0
000_ooooo`00;Oooool7o`000;Oooooo1ol0000^ooooo`00;Oooool7o`000;Oooooo1ol0000^oooo
o`00;Oooool7o`000;Oooooo1ol0000^ooooo`00;_ooool5o`0000?o003o/oooool3o`00o`Go0000
;oooool002oooooo0ol00002ooooo`?o003o/Oooool4o`00o`04ooooool0003o0000o`00033ooooo
000eooooo`?o003o[oooool4o`00ocGooooo000fooooo`?o003o[Oooool4o`00ocKooooo000goooo
o`04o`00oooooooo003oo`00ojWooooo0_l00?l00ooooooo003oooooo`0gooooo`00>?ooool01?l0
0?ooooooooooool00?nWooooo`04o`00ooooooooooooo`00ocWooooo000iooooo`03o`00oooooooo
000000;o0000Xoooool3o`000003ooooool00?oooooo03Wooooo000jooooo`Go0000XOooool5o`00
03_ooooo000iooooo`Oo0000Woooool7o`0003[ooooo000iooooo`Oo000000?o003oooooooooool0
Voooool00ol00?oo0000o`000005o`0003[ooooo000iooooo`Oo000000?oooooo`00ooooool0V_oo
ool00ol00?ooooooo`000006o`0003[ooooo000jooooo`Go00000_l00?l00ooooooo003oooooo`2H
ooooo`04o`00oooooooo003oo`00o`Go0000>oooool003_ooooo0ol00002ooooo`Co003oUoooool4
o`00o`;ooooo0ol0000looooo`00@Oooool4o`00oiGooooo1?l00?m2ooooo`00@_ooool5o`00oi7o
oooo1Ol00?m3ooooo`00A?ooool4o`00o`?oZJVYROooool3o`000004o`00ool0003o003oo`00odGo
oooo0016ooooo`03o`00oonYZJWoZJVY00?oZJVYQoooool5o`000003o`00oooooooooooo04Gooooo
0016ooooo`OoZJVYQOooool7o`0004Oooooo0016ooooo`OoZJVYQOooool7o`0004Oooooo0016oooo
o`OoZJVY0_l00?n1ooooo`;o003o1ol00017ooooo`00Aoooool5ojVYZ@04o`000?l00?oo003oo`00
ogoooooo1?l00?l5o`0004Sooooo0018ooooo`?oZJVY0oooool5o`00ogWooooo1Ol00?l3ooooo`?o
0000BOooool004oooooo00?o0000o`00ool00?l00ol00?meooooo`;o003o00Co0000o`00ool00?oo
003oD?ooool0053ooooo0_l00002o`00o`03ooooool00?oo000000;o0000K?ooool3ojVYZ@;o003o
00?oooooo`00ool00?l00_l00?mAooooo`00D_ooool2o`000003o`00ool0003o000000?o0000J_oo
ool5ojVYZ@04o`000?l00?oo003oo`00oe?ooooo001Dooooo`Oo000000?o003oooooooooool0IOoo
ool7ojVYZEKooooo001Dooooo`Oo000000?oooooo`00ool00?l0Hoooool2o`00o`OoZJVYE_ooool0
05Cooooo1ol00005o`00oeoooooo1?l00?l7ojVYZEKooooo001Eooooo`Go000000?oooooo`000?l0
0?l01Ol00?mKooooo`Oo003o1OnYZJUGooooo`00E_ooool3o`0000Cooooo0_l00007o`00o`?o0000
COooool3ojVYZ@Wo003o0oooool3ojVYZESooooo001Oooooo`;o00001?l00?l5o`0004_ooooo1OnY
ZJT6o`00of3ooooo001Rooooo`Wo000000?o003oooooooooool0AOooool00ol00?ooZJVYojVYZ@05
ojVYZ@;o003oHoooool006Cooooo1ol00004o`00od7ooooo1?l00?l7ojVYZFGooooo001Tooooo`Oo
00002Ol00?l01?ooooooZJVYojVYZOnYZJT^ooooo`?oZJVY0_ooool9o`00o`OoZJVYIOooool006Go
oooo1Ol00002ooooo`;o00001_l00?l5ojVYZ@03o`00oooooooooooo02Wooooo1OnYZJT9o`00o`;o
oooo1OnYZJUVooooo`00I_ooool3o`0000Gooooo1?l000000ol00?ooZJVYojVYZ@05ojVYZ@Co003o
2?ooool3o`0000gooooo0onYZJT8ooooo`?o003o1onYZJT6o`00o`Gooooo0onYZJUWooooo`00Looo
ool7ojVYZ@_o003o1Ol00002o`00o`Oooooo0_l00?l5ojVYZ@[o003o1onYZJT00ol00?oooooooooo
o`1booooo`00Loooool7ojVYZ@;o00002?l00?l7o`0000Wo003o1onYZJT9o`00o`OoZJVYMOooool0
07Cooooo1OnYZJT5ooooo`go00002Ol00?l7ojVYZ@Ko003o1?ooool5ojVYZGKooooo001eooooo`?o
ZJVY3?ooool7o`000003ooooool0003o000000Go000000?oooooojVYZOnYZJT01OnYZJT;ooooo`?o
ZJVYMoooool008Gooooo1Ol0000;ooooo`GoZJVYQ_ooool008Kooooo0ol0000=ooooo`?oZJVYQooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0485255, \
0.00383279, 0.00382591}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
However, if you drag one vertex out from its initial position, you \
will see the difference between the interaction and update neighborhoods. \
Trying dragging the vertex located at 3 o'clock before evaluating the next \
cell. \
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g = ExtractGraph[pd1];\)\), "\[IndentingNewLine]", 
    \(\(ShowLabeledGraph[g, 
        PlotRange \[Rule] Zoom[{1, 2, 3, 4, 48, 49, 50}]];\)\)}], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.94021 2.80944 -0.701187 2.14868 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.85953 .50946 m
.83721 .57491 L
.82646 .64205 L
s
0 0 1 r
.85953 .50946 m
.84877 .5766 L
.82646 .64205 L
s
.85953 .50946 m
.84299 .57576 L
.82646 .64205 L
s
.85953 .50946 m
.80435 .63771 L
.77192 .77043 L
s
.85953 .50946 m
.8271 .64218 L
.77192 .77043 L
s
.85953 .50946 m
.84779 .37474 L
.85953 .24003 L
s
.85953 .50946 m
.87127 .37474 L
.85953 .24003 L
s
0 0 0 r
.85953 .50946 m
.49498 .47023 L
.14047 .39434 L
s
0 0 1 r
.85953 .50946 m
.50502 .43357 L
.14047 .39434 L
s
.85953 .50946 m
.5 .4519 L
.14047 .39434 L
s
0 0 0 r
.82646 .64205 m
.79359 .70485 L
.77192 .77043 L
s
0 0 1 r
.82646 .64205 m
.80478 .70763 L
.77192 .77043 L
s
.82646 .64205 m
.79919 .70624 L
.77192 .77043 L
s
.82646 .64205 m
.7507 .764 L
.69677 .89257 L
s
.82646 .64205 m
.77253 .77062 L
.69677 .89257 L
s
.82646 .64205 m
.47267 .53569 L
.14047 .39434 L
s
.82646 .64205 m
.49426 .50071 L
.14047 .39434 L
s
0 0 0 r
.77192 .77043 m
.72902 .82958 L
.69677 .89257 L
s
0 0 1 r
.77192 .77043 m
.73967 .83341 L
.69677 .89257 L
s
.77192 .77043 m
.73434 .8315 L
.69677 .89257 L
s
.77192 .77043 m
.67677 .88416 L
s
.67677 .88416 m
.60618 1 L
s
.77192 .77043 m
.69735 .89281 L
s
.69735 .89281 m
.60767 1 L
s
0 0 0 r
.69677 .89257 m
.64452 .94714 L
s
.64452 .94714 m
.60686 1 L
s
0 0 1 r
.69677 .89257 m
.65445 .95196 L
s
.65445 .95196 m
.60846 1 L
s
.69677 .89257 m
.64949 .94955 L
s
.64949 .94955 m
.60762 1 L
s
.69677 .89257 m
.58373 .99628 L
s
.58373 .99628 m
.58067 1 L
s
.69677 .89257 m
0 0 rlineto s
.69677 .89257 m
.60836 1 L
s
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
.75971 0 m
.82646 .10743 L
s
.78139 0 m
.82646 .10743 L
s
0 0 0 r
.77884 0 m
.79359 .04464 L
.82646 .10743 L
s
0 0 1 r
.78288 0 m
.80478 .04186 L
.82646 .10743 L
s
.78081 0 m
.79919 .04325 L
.82646 .10743 L
s
.77704 0 m
.80435 .11178 L
.85953 .24003 L
s
.78093 0 m
.8271 .10731 L
.85953 .24003 L
s
0 0 0 r
.82646 .10743 m
.83721 .17457 L
.85953 .24003 L
s
0 0 1 r
.82646 .10743 m
.84877 .17289 L
.85953 .24003 L
s
.82646 .10743 m
.84299 .17373 L
.85953 .24003 L
s
.82646 .10743 m
.47096 .2334 L
.14047 .39434 L
s
.82646 .10743 m
.49597 .26838 L
.14047 .39434 L
s
0 0 0 r
.85953 .24003 m
.49327 .29885 L
.14047 .39434 L
s
0 0 1 r
.85953 .24003 m
.50673 .33552 L
.14047 .39434 L
s
.85953 .24003 m
.5 .31718 L
.14047 .39434 L
s
0 0 0 r
.025 w
.85953 .50946 Mdot
[(1)] .83953 .48946 1 0 Mshowa
.82646 .64205 Mdot
[(2)] .80646 .62205 1 0 Mshowa
.77192 .77043 Mdot
[(3)] .75192 .75043 1 0 Mshowa
.69677 .89257 Mdot
[(4)] .67677 .87257 1 0 Mshowa
[(5)] .5822 .98654 1 0 Mshowa
[(6)] .4697 1.09054 1 0 Mshowa
[(7)] .34104 1.18295 1 0 Mshowa
[(8)] .19826 1.26229 1 0 Mshowa
[(9)] .04359 1.32732 1 0 Mshowa
[(10)] -0.1205 1.37701 1 0 Mshowa
[(11)] -0.29145 1.41057 1 0 Mshowa
[(12)] -0.46655 1.42749 1 0 Mshowa
[(13)] -0.64305 1.42749 1 0 Mshowa
[(14)] -0.81815 1.41057 1 0 Mshowa
[(15)] -0.9891 1.37701 1 0 Mshowa
[(16)] -1.1532 1.32732 1 0 Mshowa
[(17)] -1.30786 1.26229 1 0 Mshowa
[(18)] -1.45064 1.18295 1 0 Mshowa
[(19)] -1.5793 1.09054 1 0 Mshowa
[(20)] -1.6918 .98654 1 0 Mshowa
[(21)] -1.78637 .87257 1 0 Mshowa
[(22)] -1.86152 .75043 1 0 Mshowa
[(23)] -1.91606 .62205 1 0 Mshowa
[(24)] -1.94913 .48946 1 0 Mshowa
[(25)] -1.96021 .35474 1 0 Mshowa
[(26)] -1.94913 .22003 1 0 Mshowa
[(27)] -1.91606 .08743 1 0 Mshowa
[(28)] -1.86152 -0.04094 1 0 Mshowa
[(29)] -1.78637 -0.16308 1 0 Mshowa
[(30)] -1.6918 -0.27705 1 0 Mshowa
[(31)] -1.5793 -0.38106 1 0 Mshowa
[(32)] -1.45064 -0.47346 1 0 Mshowa
[(33)] -1.30786 -0.5528 1 0 Mshowa
[(34)] -1.1532 -0.61783 1 0 Mshowa
[(35)] -0.9891 -0.66752 1 0 Mshowa
[(36)] -0.81815 -0.70109 1 0 Mshowa
[(37)] -0.64305 -0.71801 1 0 Mshowa
[(38)] -0.46655 -0.71801 1 0 Mshowa
[(39)] -0.29145 -0.70109 1 0 Mshowa
[(40)] -0.1205 -0.66752 1 0 Mshowa
[(41)] .04359 -0.61783 1 0 Mshowa
[(42)] .19826 -0.5528 1 0 Mshowa
[(43)] .34104 -0.47346 1 0 Mshowa
[(44)] .4697 -0.38106 1 0 Mshowa
[(45)] .5822 -0.27705 1 0 Mshowa
[(46)] .67677 -0.16308 1 0 Mshowa
[(47)] .75192 -0.04094 1 0 Mshowa
.663 .663 .663 r
.82646 .10743 Mdot
0 0 0 r
[(48)] .80646 .08743 1 0 Mshowa
.663 .663 .663 r
.85953 .24003 Mdot
0 0 0 r
[(49)] .83953 .22003 1 0 Mshowa
.14047 .39434 Mdot
[(50)] .12047 .37434 1 0 Mshowa
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[15]:=",
  ImageSize->{397, 397},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006=0001SB000`400?l00000o`00003ooooooon>
ooooo`00ooooool]ooooo`03o`00oooooooooooo00Cooooo00Co003oo`000?l00?oo003oE_ooool0
0?oooooo;Oooool00ol00?ooooooooooo`04ooooo`04o`00ool0003o003oo`00oeKooooo003ooooo
obkooooo00?o003oooooooooool00oooool00ol00?ooooooo`000002o`00oeGooooo003ooooooboo
oooo00?o003oooooooooool00oooool01?l00?oo0000o`00ool00?mEooooo`00ooooool_ooooo`03
o`00oooooooooooo00?ooooo00?o003oo`000?l00?l00_l00?mDooooo`00ooooool`ooooo`03o`00
oooooooooooo00;ooooo00?o003oooooool000000_l00?mDooooo`00oooooolaooooo`0:o`00oooo
ooooooooooooool00?ooooooo`000?l00?ooooooo`00oe?ooooo003ooooooc7ooooo00?o003ooooo
ooooool00_ooool00ol00?oo0000ooooo`02o`00oe?ooooo003ooooooc;ooooo00[o003ooooooooo
ooooooooo`00oooooooo0000o`00oooooooo003oD_ooool00?oooooo<_ooool02?l00?oooooooooo
oooooooo003oooooool0003ooooo0_l00?mBooooo`00oooooolcooooo`0:o`00ooooooooooooo`00
oooooooo0000ooooool00?ooooooo`00oe7ooooo003oooooocCooooo00Ko003ooooooooooooo003o
ooooool00003o`00oe7ooooo003oooooocCooooo00[o003ooooooooooooo003oooooool0003ooooo
o`00oooooooo003oD?ooool00?oooooo=Oooool02Ol00?ooooooo`00oooooooo0000ooooool00?oo
ooooo`00o`1@ooooo`00oooooolfooooo`;o003o0_ooool01Ol0003oooooo`00oooooooo003o04oo
oooo003oooooocKooooo00Wo003oooooool00?ooooooo`000?oooooo003oooooool00?l0Coooool0
0?oooooo=oooool2o`00o`03ooooool0003ooooo00;o003o00?oooooo`00ooooool0COooool00?oo
oooo>?ooool02?l00?ooooooooooool0003oooooo`00oooooooo003oC_ooool00?oooooo>?ooool0
1Ol00?ooooooooooool0003ooooo00;o003o00?oooooo`00ooooool0C?ooool00?oooooo>Oooool0
2?l00?ooooooooooool0003oooooo`00oooooooo003oCOooool00?oooooo>Oooool2o`00o`06oooo
ool0003oooooo`00oooooooo003oCOooool00?oooooo>Oooool2o`00o`;ooooo00Go0000ooooool0
0?ooooooo`00o`1<ooooo`00ooooooliooooo`09o`00oooooooo003oooooool0003oooooo`00oooo
oooo003o04cooooo003ooooooc[ooooo0_l00?l2ooooo`04o`000?l00?oo003oo`00odcooooo003o
oooooc[ooooo00Wo003oooooool00?ooooooo`000?oooooo003oooooool00?l0Boooool00?oooooo
>_ooool01_l00?ooooooooooool00?ooooooo`0000?o003oBoooool00?oooooo>_ooool01ol00?oo
ooooooooool00?ooooooo`000?ooool00_l00?m;ooooo`00oooooolkooooo`09o`00oooooooooooo
o`00oooooooo0000o`00oooooooo003o04[ooooo003ooooooc_ooooo00Oo003ooooooooooooooooo
o`00ool0003ooooo00;o003oB_ooool00?oooooo>oooool01ol00?oooooooooooooooooo003ooooo
ool000000_l00?m:ooooo`00oooooolkooooo`03o`00oooooooooooo00;ooooo00?o003oooooool0
00000_l00?m9ooooo`00oooooollooooo`03o`00oooooooooooo00;ooooo00Co003oo`000?l00?oo
003oBOooool00?oooooo=_ooool3o`0000;ooooo0ol00003ooooo`04o`00oooooooo003oo`00odWo
oooo003oooooocOooooo00So0000oooooooooooo0000ooooool00?ooooooo`0000?ooooo00Co003o
o`000?l00?oo003oB?ooool00?oooooo=?ooool5o`000006ooooool0003oooooo`00oooooooo0000
1?ooool3o`00odSooooo003oooooocCooooo00Co0000oooooooooooo00000oooool3o`0000Gooooo
00?o003oo`000?l00?l0B?ooool00?oooooo=Oooool00ol0003oooooo`000002ooooo`05o`000?oo
ooooooooo`00ool000001Oooool3o`00odOooooo003oooooocKooooo0_l00002ooooo`05o`000?oo
ooooooooo`00ool000001Oooool00ol00?oo0000o`00o`17ooooo`00oooooolgooooo`04o`000?oo
ooooooooooooo`?o00001oooool5ojVYZDCooooo003oooooockooooo00?o003oooooooooool00ooo
ool7ojVYZD?ooooo003oooooockooooo00?o003oooooooooool00_ooool9ojVYZD;ooooo003ooooo
ockooooo00?o003oooooooooool00_ooool9ojVYZD;ooooo003oooooockooooo00?o003ooooooooo
ool00_ooool9ojVYZD;ooooo003oooooocoooooo00Co003ooooooooooooooooo2OnYZJU2ooooo`00
ooooooloooooo`04o`00oooooooo003oo`00o`WoZJVY@_ooool00?oooooo?_ooool5o`00o`03oooo
oonYZJWoZJVY00GoZJVY@oooool00?oooooo>oooool3o`00o`03ooooool00?oo003o00Cooooo1OnY
ZJU4ooooo`00oooooolhooooo`?o003o0_ooool2o`00o`03ooooool00?oooooo00Oooooo0_l00?m3
ooooo`00oooooolfooooo`;o003o0oooool2o`00o`Cooooo00?o003oooooooooool01Oooool2o`00
od?ooooo003ooooooc?ooooo0ol00?l3ooooo`;o003o1_ooool00ol00?ooooooooooo`05ooooo`03
o`000?l00?oooooo04;ooooo003ooooooc3ooooo0ol00?l4ooooo`;o003o2?ooool00ol00?oooooo
ooooo`05ooooo`03o`000?l00?oo003o04;ooooo003oooooobgooooo0ol00?l5ooooo`;o003o2ooo
ool00ol00?ooooooooooo`04ooooo`03o`000?l00?oo003o04;ooooo003oooooob[ooooo0ol00?l6
ooooo`;o003o3Oooool00ol00?ooooooooooo`04ooooo`03o`000?l00?oo003o04;ooooo003ooooo
obSooooo0_l00?l6ooooo`?o003o4?ooool00ol00?ooooooooooo`04ooooo`?o003o@Oooool00?oo
oooo9Oooool3o`00o`Kooooo0_l00?lCooooo`03o`00oooooooooooo00Cooooo00?o0000o`00ool0
0?l0@Oooool00?oooooo8_ooool3o`00o`Oooooo0_l00?lFooooo`03o`00oooooooooooo00?ooooo
00?o0000o`00ool00?l0@Oooool00?oooooo7oooool3o`00o`Sooooo0_l00?lHooooo`03o`00oooo
oooooooo00?ooooo00Co0000o`00oooooooo003o@?ooool00?oooooo7?ooool3o`00o`Wooooo0_l0
0?lJooooo`03o`00oooooooooooo00?ooooo00Co0000o`00ool00?oo003o@?ooool00?oooooo6Ooo
ool3o`00o`[ooooo0_l00?lMooooo`03o`00oooooooooooo00;ooooo00Co0000ooooool00?oo003o
@?ooool00?oooooo5oooool2o`00o`_ooooo0_l00?lOooooo`03o`00oooooooooooo00;ooooo00Go
0000ooooool00?ooooooo`00o`0oooooo`00oooooolDooooo`?o003o2oooool2o`00ob;ooooo00?o
003oooooooooool00_ooool01?l0003o003oooooool00?loooooo`00oooooolAooooo`?o003o3?oo
ool2o`00obCooooo00?o003oooooooooool00_ooool01?l0003o003oo`00ool00?loooooo`00oooo
ool>ooooo`?o003o3Oooool2o`00obOooooo00Wo003oooooooooooooooooo`000?oooooo003ooooo
ool00?l0?_ooool00?oooooo2oooool3o`00o`kooooo0_l00?lYooooo`09o`00oooooooooooooooo
ool0003oooooo`00oooooooo003o03kooooo003oooooo`Wooooo0_l00?l?ooooo`;o003o:oooool0
2Ol00?oooooooooooooooooo0000ooooool00?ooooooo`00o`0nooooo`00ooooool6ooooo`?o003o
3oooool2o`00obkooooo00Go003ooooooooooooo0000ooooo`02o`00o`03ooooool00?oooooo03co
oooo003oooooo`?ooooo0ol00?l@ooooo`;o003o<?ooool01Ol00?ooooooooooool0003ooooo00;o
003o00?oooooo`00ooooool0??ooool00?oooooo0ol00?lAooooo`;o003o<oooool02?l00?oooooo
ooooool0003oooooo`00oooooooo003o?Oooool00?cooooo0ol00?lBooooo`;o003o=Oooool01_l0
0?ooooooooooool0003oooooo`00o`;ooooo00?o003oooooooooool0>_ooool00?Wooooo0ol00?lC
ooooo`;o003o>?ooool01?l00?ooooooo`000?ooool2o`00o`03ooooool00?oooooo03_ooooo003g
ooooo`;o003o5?ooool2o`00oc[ooooo00Co003oooooool0003ooooo0_l00?l00ooooooo003ooooo
o`0kooooo`00m?ooool3o`00oaCooooo0_l00?llooooo`08o`00ooooooooooooo`000?oooooo003o
ooooool00?llooooo`00lOooool3o`00oaGooooo0_l00?loooooo`05o`00oooooooo0000ooooool0
0?l00_ooool00ol00?ooooooooooo`0iooooo`00k_ooool3o`00oaKooooo0_l00?m1ooooo`05o`00
oooooooo0000ooooool00?l00_ooool00ol00?ooooooooooo`0iooooo`00joooool3o`00oaKooooo
0ol00?m4ooooo`03o`00oooooooo000000;o003o00?oooooo`00ooooool0>_ooool00>Wooooo0_l0
0?lGooooo`;o003oAoooool01ol00?ooooooo`000?oooooo003oooooool00?l0>oooool00>Kooooo
0ol00?lGooooo`;o003oB_ooool01_l00?oo0000ooooool00?ooooooo`00oc_ooooo003Sooooo`?o
003o6?ooool2o`00odcooooo00Ko003oooooool0003o003oooooool00?lkooooo`00h?ooool3o`00
oaWooooo0_l00?m>ooooo`03o`00oooooooo000000?o003o>oooool00=gooooo0ol00?lJooooo`;o
003oDOooool01_l00?oo0000ooooool00?ooooooo`00oc[ooooo003Jooooo`?o003o6oooool2o`00
oe?ooooo00Ko003oooooool0003o003oooooool00?ljooooo`00f?ooool2o`00oacooooo0_l00?mF
ooooo`05o`00ool0003o003oooooool00?l0>_ooool00=Gooooo0ol00?lLooooo`;o003oF?ooool0
0ol00?oo0000ooooo`02o`00oc[ooooo003Booooo`?o003o7Oooool2o`00oe_ooooo00Co003oo`00
0?l00?oo003o>_ooool00<oooooo0ol00?lNooooo`;o003oD?ooool3o`0000;ooooo0_l00006oooo
o`04o`00ool0003o003oo`00oc[ooooo003<ooooo`?o003o7oooool2o`00oe?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`03ooooo`04o`00ool0003o003oo`00oc[ooooo003:oooo
o`;o003o8?ooool2o`00oe;ooooo1Ol00005ooooo`03o`000?oooooooooo00?ooooo00Co003oo`00
0?l00?oo003o>Oooool00<Oooooo0ol00?lPooooo`;o003oE?ooool01?l0003oooooooooool00003
ooooo`Co00001Oooool01?l00?oo0000o`00ool00?liooooo`00a?ooool3o`00ob7ooooo0_l00?mG
ooooo`03o`000?oooooo000000;ooooo00Go0000ooooooooooooooooo`000006ooooo`03o`000?l0
0?oo003o03Wooooo0031ooooo`?o003o8_ooool2o`00oe[ooooo0_l00002ooooo`05o`000?oooooo
ooooooooool000001_ooool3o`00ocWooooo002nooooo`?o003o8oooool2o`00oegooooo00Co0000
oooooooooooooooo0ol00008ooooo`03o`000?l00?oooooo03Sooooo002kooooo`?o003o9?ooool2
o`00ofgooooo1OnYZJTgooooo`00^Oooool2o`00obGooooo0_l00?m^ooooo`OoZJVY=_ooool00;Oo
oooo0_l00?lUooooo`;o003oKoooool9ojVYZCGooooo002eooooo`;o003o9Oooool2o`00og7ooooo
2OnYZJTeooooo`00/oooool2o`00obCooooo0ol00?mcooooo`WoZJVY=Oooool00;7ooooo0_l00?lT
ooooo`;o003oL_ooool2o`0000;o003o2OnYZJTeooooo`00[oooool2o`00obCooooo0_l00?m]oooo
o`Co00001_l00?l00oooooooZJVYojVYZ@07ojVYZCGooooo002]ooooo`;o003o9?ooool2o`00ofWo
oooo1Ol00008o`00o`Gooooo1onYZJTfooooo`00Zoooool2o`00obCooooo0_l00?mUooooo`Ko0000
00?oooooo`00ool00?l01_l00?l:ooooo`GoZJVY=oooool00:Wooooo0_l00?lTooooo`;o003oH?oo
ool7o`0000;ooooo2Ol00?lAooooo`03o`00oooooooooooo03Kooooo002Wooooo`;o003o9?ooool2
o`00oecooooo1_l00004ooooo`Go003o00?oooooo`00ool00?l00_l00?lDooooo`03o`00oooooooo
003o03Oooooo002Uooooo`;o003o9?ooool2o`00oeSooooo1_l00005ooooo`Go003o0oooool3o`00
oaSooooo00?o003oooooool00?l0=oooool00:;ooooo0ol00?lTooooo`;o003oDoooool7o`0000Oo
oooo1?l00?l4ooooo`Co003o6oooool00ol00?ooooooo`00o`0gooooo`00X?ooool2o`00obGooooo
0_l00?m?ooooo`Ko00002Oooool5o`00o`Cooooo1?l00?lOooooo`03o`00oooooooo003o03Oooooo
002Nooooo`;o003o9Oooool2o`00od_ooooo1_l0000:ooooo`Go003o1Oooool4o`00ob?ooooo00?o
003oooooool00?l0=oooool009cooooo0_l00?lUooooo`;o003oA_ooool7o`0000cooooo1?l00?l6
ooooo`Co003o9oooool00ol00?ooooooo`00o`0gooooo`00V_ooool2o`00obCooooo0ol00?m2oooo
o`Ko00003_ooool5o`00o`Oooooo0ol00?l[ooooo`03o`00oooooooo003o03Oooooo002Hooooo`;o
003o8oooool3o`00ocoooooo1_l0000?ooooo`Go003o2?ooool4o`00obkooooo00?o003oooooool0
0?l0=oooool009Kooooo0_l00?lRooooo`?o003o??ooool6o`00013ooooo1Ol00?l9ooooo`Co003o
<_ooool00ol00?ooooooo`00o`0gooooo`00U?ooool2o`00ob7ooooo0ol00?lhooooo`Oo00004_oo
ool4o`00o`[ooooo1?l00?lfooooo`03o`00oooooooo003o03Oooooo002Booooo`;o003o8?ooool3
o`00ocGooooo1_l0000Dooooo`Go003o2oooool3o`00oc[ooooo00?o003oooooool00?l0=oooool0
093ooooo0_l00?lPooooo`;o003o<_ooool6o`0001Gooooo1Ol00?l<ooooo`Co003o??ooool01Ol0
0?oooooooooooooooooo003o03Kooooo002>ooooo`;o003o7oooool3o`00obgooooo1ol0000Foooo
o`Go003o3Oooool4o`00od3ooooo00Go003oooooooooooooooooo`00o`0fooooo`00S?ooool2o`00
oakooooo0ol00?lZooooo`Ko00006Oooool4o`00o`kooooo1?l00?m4ooooo`05o`00oooooooooooo
ooooool00?l0=_ooool008[ooooo0_l00?lMooooo`?o003o9oooool6o`0001[ooooo1Ol00?l>oooo
o`Co003oB?ooool01Ol00?oooooooooooooooooo003o03Kooooo0028ooooo`;o003o7?ooool3o`00
ob?ooooo1ol0000Kooooo`Go003o4?ooool3o`00odcooooo00Go003oooooooooooooooooo`00o`0f
ooooo`00Q_ooool2o`00oacooooo0_l00?lPooooo`Ko00007Oooool5o`00oa7ooooo1?l00?m?oooo
o`05o`00ooooooooooooooooool00?l0=_ooool008Cooooo0_l00?lKooooo`?o003o7Oooool5o`00
01oooooo1?l00?lBooooo`Co003oDoooool01Ol00?oooooooooooooooooo003o03Kooooo0022oooo
o`;o003o6_ooool3o`00oacooooo1?l0000Oooooo`Go003o4_ooool4o`00oeOooooo00Go003ooooo
ooooooooooooo`00o`0fooooo`00P?ooool2o`00oaWooooo0ol00?lLooooo`?o00007_ooool5o`00
oaCooooo0ol00?mKooooo`05o`00ooooooooooooooooool00?l0=_ooool007kooooo0_l00?lHoooo
o`?o003o6oooool4o`0001gooooo1?l00?lEooooo`Co003oG_ooool01Ol00?oooooooooooooooooo
003o03Kooooo001looooo`;o003o5oooool3o`00oa[ooooo1?l0000Looooo`Go003o5Oooool4o`00
of7ooooo00?o003oooooooooool00oooool00ol00?ooooooooooo`0cooooo`00N_ooool2o`00oaOo
oooo0_l00?lJooooo`?o00006oooool5o`00oaKooooo1?l00?mUooooo`03o`00oooooooooooo00?o
oooo00?o003oooooooooool0<oooool007Oooooo0ol00?lFooooo`?o003o6?ooool4o`0001Wooooo
1Ol00?lGooooo`Co003oJOooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo03?ooooo
001eooooo`;o003o5_ooool3o`00oaOooooo1?l0000Iooooo`Co003o6Oooool3o`00ofgooooo00?o
003oooooooooool00oooool00ol00?ooooooooooo`0cooooo`00Loooool2o`00oaGooooo0ol00?lG
ooooo`?o00006?ooool5o`00oaWooooo1?l00?m`ooooo`03o`00oooooooooooo00?ooooo00?o003o
ooooooooool0<oooool0077ooooo0_l00?lDooooo`?o003o5_ooool4o`0001Kooooo1Ol00?lJoooo
o`Co003oM?ooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo03?ooooo001_ooooo`;o
003o5?ooool2o`00oaGooooo1?l0000Fooooo`Co003o6oooool4o`00ogSooooo00?o003ooooooooo
ool00oooool00ol00?ooooooooooo`0cooooo`00KOooool2o`00oa?ooooo0ol00?lDooooo`?o0000
5Oooool5o`00oa_ooooo1?l00?mlooooo`03o`00oooooooooooo00?ooooo00?o003oooooooooool0
<oooool006_ooooo0_l00?lBooooo`?o003o4oooool4o`0001Cooooo1?l00?lMooooo`?o003oP?oo
ool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo03?ooooo001Yooooo`;o003o4Oooool3
o`00oa;ooooo1?l0000Cooooo`Go003o7Oooool4o`00oh?ooooo00?o003oooooooooool00oooool0
0ol00?ooooooooooo`0cooooo`00Ioooool2o`00oa3ooooo0ol00?lBooooo`?o00004_ooool5o`00
oagooooo1Ol00?n7ooooo`03o`00oooooooooooo00?ooooo00?o003oooooooooool0<oooool006Go
oooo0_l00?l?ooooo`?o003o4Oooool4o`00017ooooo1?l00?lLooooo`Ko003oRoooool00ol00?oo
ooooooooo`05ooooo`03o`00oooooooooooo03;ooooo001Sooooo`;o003o3oooool2o`00oa3ooooo
1?l0000@ooooo`Go003o6_ooool6o`00oi7ooooo00?o003oooooooooool01Oooool00ol00?oooooo
ooooo`0booooo`00HOooool2o`00o`kooooo0ol00?l?ooooo`?o00004?ooool4o`00oaWooooo1_l0
0?nGooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool0<_ooool005oooooo0_l00?l=
ooooo`?o003o3_ooool4o`0000kooooo1Ol00?lGooooo`Ko003oWOooool00ol00?ooooooooooo`05
ooooo`03o`00oooooooooooo03;ooooo001Mooooo`;o003o3?ooool3o`00o`gooooo1?l0000=oooo
o`Go003o5_ooool6o`00oj?ooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0boooo
o`00Foooool2o`00o`_ooooo0ol00?l<ooooo`Co00003Oooool4o`00oaGooooo1_l00?nYooooo`03
o`00oooooooooooo00Gooooo00?o003oooooooooool0<_ooool005Wooooo0_l00?l;ooooo`;o003o
3?ooool3o`0000cooooo1Ol00?lCooooo`Ko003o[oooool00ol00?ooooooooooo`05ooooo`03o`00
oooooooooooo03;ooooo001Gooooo`;o003o2_ooool3o`00o`[ooooo1?l0000:ooooo`Go003o4_oo
ool6o`00okGooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0booooo`00EOooool2
o`00o`Wooooo0ol00?l9ooooo`Co00002_ooool4o`00oa7ooooo1_l00?nkooooo`03o`00oooooooo
oooo00Gooooo00?o003oooooooooool0<_ooool005?ooooo0_l00?l8ooooo`?o003o2Oooool3o`00
00Wooooo1Ol00?l?ooooo`Ko003o`Oooool00ol00?ooooooooooo`05ooooo`03o`00oooooooooooo
03;ooooo001Aooooo`;o003o1oooool3o`00o`Sooooo1?l00008ooooo`Co003o3_ooool6o`00olOo
oooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0booooo`00Coooool2o`00o`Kooooo
0ol00?l7ooooo`Co00001oooool5o`00o`_ooooo1ol00?o<ooooo`03o`00oooooooooooo00Oooooo
00?o003oooooooooool0<Oooool002Cooooo0ol00004ooooo`;o00007oooool3o`00o`Kooooo0_l0
0?l7ooooo`?o00001_ooool5o`00o`[ooooo1_l00?oCooooo`03o`00oooooooooooo00Oooooo00?o
003oooooooooool0<Oooool002?ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oo
ooooooooo`0001cooooo0_l00?l6ooooo`?o003o1Oooool4o`0000Gooooo1?l00?l9ooooo`Ko003o
fOooool00ol00?ooooooooooo`07ooooo`03o`00oooooooooooo037ooooo000Wooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool06?ooool2o`00o`Gooooo0ol00?l4ooooo`Co0000
1?ooool5o`00o`Oooooo1_l00?oOooooo`03o`00oooooooooooo00Oooooo00?o003oooooooooool0
<Oooool002Oooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0Fooooo`;o003o
1?ooool3o`00o`Cooooo0ol00004ooooo`Co003o1_ooool6o`00onGooooo00?o003oooooooooool0
1oooool00ol00?ooooooooooo`0aooooo`008oooool4o`0000?ooooo00Co0000oooooooooooo0000
5_ooool2o`00o`?ooooo0ol00?l3ooooo`Co00000_ooool5o`00o`Cooooo1_l00?o[ooooo`03o`00
oooooooooooo00Oooooo00?o003oooooooooool0<Oooool002?ooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooool0000Dooooo`;o003o0oooool2o`00o`;ooooo1?l000000ooooooo003o
o`00o`03o`00o`?ooooo1_l00?oaooooo`03o`00oooooooooooo00Oooooo00?o003oooooooooool0
<Oooool002?ooooo1Ol00003ooooo`;o00004oooool2o`00o`;ooooo0ol00?l01Ooooooo0000o`00
0?l0003ooooo00Co003o0_ooool6o`00ooOooooo00?o003oooooooooool01oooool00ol00?oooooo
ooooo`0aooooo`00=Oooool5o`0000Cooooo0_l00?l01?oooooo003oo`00ool00?l3o`0000_o003o
oOooool00ol00?ooooooooooo`07ooooo`03o`00oooooooooooo037ooooo000dooooo`Oo000000?o
ooooo`00ool00?l00_l00?l3o`0000Wo003oooooool4ooooo`03o`00oooooooooooo00Oooooo00?o
003oooooooooool0<Oooool003?ooooo2Ol000000ol00?oo0000o`00o`07o`00oooooooo2oooool0
0ol00?ooooooooooo`05ooooo`03o`00oooooooooooo03;ooooo000cooooo`Wo00001?l00?oooooo
oa7ooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0booooo`00<oooool9o`000003
o`00oooooooooooo0?oooooo4_ooool00ol00?ooooooooooo`05ooooo`03o`00oooooooooooo03;o
oooo000cooooo`Wo00002_l00?ooooooo`_ooooo00?o003oooooooooool01Oooool00ol00?oooooo
ooooo`0booooo`00<oooool9o`0000Co003o0_l0000>o`00oooooooo0Oooool00ol00?oooooooooo
o`05ooooo`03o`00oooooooooooo03;ooooo000dooooo`Oo00000oooool6o`00o`Co00001ol00?l0
0ooooooo003oo`00o`08o`00ooKooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0b
ooooo`00=Oooool5o`0000Kooooo0ol00?l01Ooooooo003oo`00ool00?oooooo00Go00000_ooool6
o`00o`Gooooo2Ol00?o]ooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool0<_ooool0
04?ooooo0_l00?l2ooooo`?o003o0oooool5o`0000?ooooo1ol00?l7ooooo`[o003ohoooool00ol0
0?ooooooooooo`05ooooo`03o`00oooooooooooo03;ooooo0015ooooo`;o003o0oooool4o`00o`Co
oooo1Ol00005ooooo`Ko003o2oooool9o`00om[ooooo00?o003oooooooooool01Oooool00ol00?oo
ooooooooo`0booooo`00Aoooool3o`00o`Cooooo0ol00?l6ooooo`Co00001oooool7o`00o`gooooo
2_l00?o@ooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool0<_ooool004[ooooo0_l0
0?l5ooooo`?o003o1oooool5o`0000Wooooo1_l00?lAooooo`[o003oa_ooool00ol00?oooooooooo
o`05ooooo`03o`00oooooooooooo03;ooooo001<ooooo`;o003o1_ooool4o`00o`Sooooo1Ol0000:
ooooo`Oo003o5?ooool9o`00okkooooo00?o003oooooooooool00oooool00ol00?ooooooooooo`0c
ooooo`00C_ooool3o`00o`Oooooo0ol00?l:ooooo`Co00003Oooool6o`00oaOooooo2_l00?ndoooo
o`03o`00oooooooooooo00?ooooo00?o003oooooooooool0<oooool0057ooooo0_l00?l8ooooo`?o
003o2oooool5o`0000kooooo1ol00?lJooooo`Wo003oZoooool00ol00?ooooooooooo`03ooooo`03
o`00oooooooooooo03?ooooo001Cooooo`;o003o2Oooool4o`00o`cooooo1Ol0000@ooooo`Ko003o
7Oooool:o`00oj7ooooo00?o003oooooooooool00oooool00ol00?ooooooooooo`0cooooo`00EOoo
ool3o`00o`[ooooo0ol00?l>ooooo`Co00004_ooool7o`00ob3ooooo2_l00?nGooooo`03o`00oooo
oooooooo00?ooooo00?o003oooooooooool0<oooool005Sooooo0_l00?l;ooooo`?o003o3oooool5
o`0001Cooooo1_l00?lTooooo`Wo003oS_ooool00ol00?ooooooooooo`03ooooo`03o`00oooooooo
oooo03?ooooo001Jooooo`?o003o2oooool4o`00oa3ooooo1Ol0000Eooooo`Oo003o9_ooool7o`00
ohOooooo00?o003oooooooooool00oooool00ol00?ooooooooooo`0cooooo`00GOooool2o`00o`go
oooo0ol00?lBooooo`Go00005oooool6o`00obOooooo1Ol00?n2ooooo`03o`00oooooooooooo00?o
oooo00?o003oooooooooool0<oooool005oooooo0_l00?l>ooooo`?o003o5?ooool4o`0001Wooooo
1ol00?lUooooo`Go003oOOooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo03?ooooo
001Qooooo`?o003o3_ooool4o`00oaCooooo1Ol0000Kooooo`Ko003o9?ooool4o`00ogWooooo00?o
003oooooooooool00oooool00ol00?ooooooooooo`0cooooo`00I?ooool2o`00oa3ooooo0ol00?lF
ooooo`Go00007?ooool7o`00ob7ooooo1Ol00?meooooo`05o`00ooooooooooooooooool00?l0=_oo
ool006Kooooo0_l00?lAooooo`?o003o6?ooool4o`0001oooooo1_l00?lPooooo`Go003oL?ooool0
1Ol00?oooooooooooooooooo003o03Kooooo001Xooooo`?o003o4Oooool4o`00oaSooooo1Ol0000P
ooooo`Oo003o7_ooool5o`00of_ooooo00Go003oooooooooooooooooo`00o`0fooooo`00Joooool2
o`00oa?ooooo0ol00?lJooooo`Go00008_ooool6o`00oagooooo1?l00?mWooooo`05o`00oooooooo
ooooooooool00?l0=_ooool006gooooo0_l00?lDooooo`?o003o7?ooool4o`0002Cooooo1_l00?lK
ooooo`Go003oH_ooool01Ol00?oooooooooooooooooo003o03Kooooo001_ooooo`?o003o5?ooool4
o`00oacooooo1Ol0000Uooooo`Ko003o6_ooool5o`00oegooooo00Go003oooooooooooooooooo`00
o`0fooooo`00L_ooool2o`00oaKooooo0ol00?lNooooo`Go00009_ooool7o`00oaSooooo1?l00?mI
ooooo`05o`00ooooooooooooooooool00?l0=_ooool007Cooooo0_l00?lGooooo`?o003o8?ooool5
o`0002Sooooo1_l00?lFooooo`Go003oE?ooool01Ol00?oooooooooooooooooo003o03Kooooo001f
ooooo`?o003o5oooool4o`00ob7ooooo1?l0000Zooooo`Ko003o5Oooool5o`00odoooooo00Go003o
ooooooooooooooooo`00o`0fooooo`00NOooool2o`00oaWooooo0ol00?lRooooo`Go0000:oooool6
o`00oaCooooo1?l00?m;ooooo`05o`00ooooooooooooooooool00?l0=_ooool007_ooooo0ol00?lI
ooooo`?o003o9?ooool5o`0002cooooo1_l00?lBooooo`Go003oA_ooool01Ol00?oooooooooooooo
oooo003o03Kooooo001nooooo`;o003o6_ooool4o`00obGooooo1ol0000[ooooo`Ko003o4Oooool5
o`00od;ooooo00?o003oooooool00?l0=oooool0083ooooo0_l00?lLooooo`?o003o:Oooool9o`00
02Sooooo1ol00?l?ooooo`Go003o?Oooool00ol00?ooooooo`00o`0gooooo`00P_ooool3o`00oaco
oooo0ol00?l_ooooo`[o00009Oooool6o`00o`kooooo1?l00?liooooo`03o`00oooooooo003o03Oo
oooo0025ooooo`;o003o7Oooool4o`00ocGooooo2_l0000Qooooo`Ko003o3?ooool5o`00ocCooooo
00?o003oooooool00?l0=oooool008Oooooo0_l00?lOooooo`?o003o??ooool9o`0001kooooo1_l0
0?l;ooooo`Go003o;oooool00ol00?ooooooo`00o`0gooooo`00ROooool3o`00oaoooooo0ol00?m2
ooooo`[o00006_ooool6o`00o`[ooooo1?l00?lOooooo`?o00002Oooool00ol00?ooooooo`00o`0g
ooooo`00S?ooool2o`00ob3ooooo1?l00?m8ooooo`Wo00005oooool7o`00o`Oooooo1Ol00?lKoooo
o`03o`000?oooooooooo00Sooooo00?o003oooooool00?l0=oooool008kooooo0_l00?lRooooo`?o
003oC_ooool:o`0001Cooooo1_l00?l6ooooo`Go003o5_ooool00ol0003oooooooooo`08ooooo`03
o`00oooooooo003o03Oooooo002@ooooo`?o003o8_ooool3o`00oeGooooo2_l0000@ooooo`Ko003o
1Oooool4o`00oa;ooooo00?o0000ooooooooool02?ooool00ol00?ooooooo`00o`0gooooo`00Tooo
ool2o`00ob?ooooo1?l00?mKooooo`Wo00003Oooool6o`00o`?ooooo1Ol00?l=ooooo`03o`000?oo
oooooooo00Sooooo00?o003oooooool00?l0=oooool009Gooooo0_l00?lUooooo`?o003oHOooool:
o`0000Wooooo1_l00?l2ooooo`Go003o1oooool2o`0000[ooooo00?o003oooooool00?l0=oooool0
09Oooooo0ol00?lUooooo`?o003oJ?ooool9o`0000Kooooo1_l00?l00ooooooo003oo`00o`03o`00
o`?ooooo00?o0000ooooooooool02Oooool00ol00?ooooooooooo`0fooooo`00V_ooool2o`00obKo
oooo0ol00?m^ooooo`[o00000_ooool:o`00o`Sooooo1Ol0000gooooo`00W?ooool3o`00obKooooo
0ol00?meooooo`Wo00002?l00?l2ooooo`Oo0000=_ooool009oooooo0_l00?lWooooo`;o003oOOoo
ool5o`0000?o003o2Ol0000eooooo`00XOooool2o`00obOooooo0_l00?n3ooooo`Wo0000=Oooool0
0:?ooooo0ol00?lVooooo`?o003oP?ooool9o`0003Gooooo002Vooooo`;o003o9oooool2o`00ogko
oooo2Ol0000eooooo`00Z?ooool2o`00obOooooo0_l00?mlooooo`Wo0000=Oooool00:[ooooo0ol0
0?lVooooo`?o003oN_ooool7o`0003Kooooo002]ooooo`;o003o9oooool2o`00ogWooooo1Ol0000g
ooooo`00[oooool2o`00obOooooo0_l00?mhooooo`;o003o>Oooool00;7ooooo0ol00?lVooooo`?o
003oM?ooool3o`00ocWooooo002dooooo`;o003o9oooool2o`00og;ooooo0ol00?liooooo`00]_oo
ool2o`00obOooooo0_l00?m_ooooo`04o`00ool0003o003oo`00ocWooooo002hooooo`?o003o9_oo
ool3o`00ofcooooo1?l00?liooooo`00^oooool3o`00obKooooo0_l00?mYooooo`04o`00ool0003o
003oo`00oc[ooooo002nooooo`?o003o9Oooool3o`00ofKooooo00Co003oo`000?l00?oo003o>_oo
ool00<7ooooo0ol00?lUooooo`;o003oI?ooool01?l00?oo0000o`00ool00?ljooooo`00a?ooool4
o`00ob?ooooo0_l00?mQooooo`03o`00ool0003o003o00;o003o>_ooool00<Sooooo0ol00?lRoooo
o`?o003oG_ooool01Ol00?oo0000o`00oooooooo003o03[ooooo003;ooooo`?o003o8_ooool2o`00
oe_ooooo00Ko003oooooool0003o003oooooool00?ljooooo`00c_ooool4o`00ob3ooooo0_l00?mI
ooooo`06o`00ool0003oooooo`00oooooooo003o>_ooool00=;ooooo0ol00?lOooooo`?o003oEOoo
ool00ol00?ooooooo`000003o`00oc_ooooo003Eooooo`?o003o7oooool2o`00oe?ooooo00Ko003o
ooooool0003o003oooooool00?lkooooo`00f?ooool4o`00oagooooo0_l00?mAooooo`06o`00ool0
003oooooo`00oooooooo003o>oooool00=cooooo0ol00?lLooooo`?o003oCOooool00ol00?oooooo
o`000002o`00o`03ooooool00?oooooo03[ooooo003Oooooo`?o003o7?ooool2o`00od_ooooo00?o
003oooooool000000_l00?l00ooooooo003oooooo`0jooooo`00h_ooool4o`00oa[ooooo0_l00?m8
ooooo`05o`00oooooooo0000ooooool00?l00_ooool00ol00?ooooooooooo`0iooooo`00i_ooool3
o`00oaWooooo0ol00?m5ooooo`05o`00oooooooo0000ooooool00?l00_ooool00ol00?oooooooooo
o`0iooooo`00jOooool3o`00oaWooooo0_l00?m2ooooo`04o`00ooooooooooooo`0000;o003o00?o
ooooo`00ooooool0>oooool00>cooooo1?l00?lGooooo`?o003o?oooool01?l00?ooooooo`000?oo
ool2o`00o`03ooooool00?oooooo03_ooooo003`ooooo`?o003o5oooool2o`00ocgooooo00Go003o
ooooool0003oooooo`00o`02ooooo`03o`00oooooooooooo03[ooooo003cooooo`?o003o5_ooool2
o`00oc[ooooo00Ko003ooooooooooooo0000ooooool00?l2ooooo`03o`00oooooooooooo03[ooooo
003fooooo`Co003o5?ooool3o`00ocOooooo00Co003ooooooooooooo00000_l00?l00ooooooo003o
ooooo`0looooo`00n_ooool3o`00oaCooooo0_l00?ldooooo`05o`00ooooooooooooo`000?ooool0
0_l00?l00ooooooo003oooooo`0looooo`00oOooool3o`00oa?ooooo0_l00?lbooooo`05o`00oooo
ooooooooo`000?ooool00_l00?l00ooooooo003oooooo`0looooo`00ooooool1ooooo`Co003o4Ooo
ool3o`00obkooooo00Wo003oooooooooooooooooo`000?oooooo003oooooool00?l0?_ooool00?oo
oooo1Oooool3o`00oa7ooooo0_l00?l/ooooo`09o`00ooooooooooooooooool0003oooooo`00oooo
oooo003o03kooooo003oooooo`Sooooo0ol00?l@ooooo`;o003o:_ooool01Ol00?oooooooooooooo
oooo000000;o003o00?oooooo`00ooooool0?Oooool00?oooooo2oooool4o`00o`kooooo0ol00?lV
ooooo`03o`00oooooooooooo00;ooooo00Co0000o`00ool00?oo003o?oooool00?oooooo3oooool3
o`00o`kooooo0_l00?lTooooo`03o`00oooooooooooo00;ooooo00Co0000o`00oooooooo003o?ooo
ool00?oooooo4_ooool3o`00o`gooooo0_l00?lQooooo`03o`00oooooooooooo00;ooooo00Go0000
ooooool00?ooooooo`00o`0oooooo`00oooooolEooooo`Co003o2oooool3o`00oakooooo00?o003o
ooooooooool00_ooool01?l0003o003oo`00ool00?m0ooooo`00oooooolIooooo`?o003o2oooool2
o`00oa_ooooo00?o003oooooooooool00oooool01?l0003o003oo`00ool00?m0ooooo`00oooooolL
ooooo`?o003o2_ooool3o`00oaSooooo00?o003oooooooooool00oooool01?l0003o003oooooool0
0?m0ooooo`00oooooolOooooo`Co003o2Oooool2o`00oaKooooo00?o003oooooooooool00oooool0
0ol0003o003oo`00o`11ooooo`00oooooolSooooo`?o003o2?ooool2o`00o`[ooooo1Ol00004oooo
o`03o`00oooooooooooo00Cooooo00?o0000o`00ool00?l0@Oooool00?oooooo9_ooool3o`00o`Oo
oooo0ol00?l8ooooo`04o`000?ooooooooooo`0000Cooooo00?o003oooooooooool01?ooool3o`00
od7ooooo003oooooobWooooo1?l00?l6ooooo`;o003o1oooool00ol0003oooooooooo`03ooooo`03
o`00oooooooooooo00Cooooo00?o0000o`00ool00?l0@_ooool00?oooooo;Oooool3o`00o`Gooooo
0_l00?l6ooooo`03o`000?oooooooooo00;ooooo00?o003oooooooooool01?ooool00ol0003o003o
o`00o`12ooooo`00ooooool`ooooo`?o003o1?ooool3o`00o`05o`000?ooooooooooooooool00000
0_ooool00ol00?ooooooooooo`05ooooo`03o`000?l00?oo003o04;ooooo003ooooooc?ooooo1?l0
0?l3ooooo`05o`000?l00?ooooooooooool000000_ooool00ol00?ooooooooooo`05ooooo`;o003o
@oooool00?oooooo=oooool3o`00o`04ooooool0003o0000o`0000?ooooo00?o003oooooooooool0
1Oooool2o`00od?ooooo003ooooooc[ooooo0ol00?l01?oooooo003oo`00ool00?l4ooooo`Go0000
00?o003oooooooooool0@Oooool00?oooooo?Oooool6o`00o`03ooooool0003o000000Go0000@ooo
ool00?oooooo?oooool01?l00?ooooooo`00ool00?l9o`0004;ooooo003oooooocoooooo00Co003o
oooooooooooooooo2Ol00012ooooo`00ooooooloooooo`04o`00ooooooooooooooooo`Wo0000@_oo
ool00?oooooo?_ooool00ol00?ooooooooooo`02ooooo`Wo0000@_ooool00?oooooo?_ooool00ol0
0?ooooooooooo`02ooooo`Wo0000@_ooool00?oooooo?_ooool00ol00?ooooooooooo`03ooooo`Oo
0000@oooool00?oooooo?_ooool00ol00?ooooooooooo`04ooooo`Go0000A?ooool00?oooooo?Ooo
ool00ol00?ooooooooooo`05ooooo`;o003oAoooool00?oooooo?Oooool00ol00?ooooooooooo`05
ooooo`;o003oAoooool00?oooooo?Oooool00ol00?ooooooooooo`04ooooo`?o003oAoooool00?oo
oooo?Oooool00ol00?ooooooooooo`03ooooo`03o`00ool0003o003o04Sooooo003ooooooccooooo
00?o003oooooooooool01?ooool3o`00odSooooo003ooooooccooooo00?o003oooooooooool00ooo
ool01?l00?oo0000o`00ool00?m8ooooo`00oooooollooooo`03o`00oooooooooooo00;ooooo00Co
003oooooool00?oo003oBOooool00?oooooo??ooool00ol00?ooooooooooo`02ooooo`04o`00ool0
003o003oo`00odWooooo003ooooooc_ooooo00?o003oooooooooool00_ooool00ol00?oo0000o`00
o`02o`00odWooooo003ooooooc_ooooo00?o003oooooooooool00_ooool01?l00?oo0000o`00ool0
0?m:ooooo`00oooooolkooooo`07o`00ooooooooooooooooool00?oo0000ooooo`02o`00od[ooooo
003ooooooc_ooooo00Wo003ooooooooooooo003oooooool0003o003oooooool00?l0B_ooool00?oo
oooo>_ooool01ol00?oooooooooooooooooo003oo`000?ooool00_l00?m;ooooo`00ooooooljoooo
o`09o`00ooooooooooooo`00oooooooo0000o`00oooooooo003o04_ooooo003ooooooc[ooooo00Wo
003oooooool00?ooooooo`000?oooooo003oooooool00?l0Boooool00?oooooo>_ooool01Ol00?oo
ooooo`00oooooooo000000?o003oC?ooool00?oooooo>Oooool02Ol00?ooooooo`00oooooooo0000
ooooool00?ooooooo`00o`1<ooooo`00ooooooliooooo`09o`00oooooooo003oooooool0003ooooo
o`00oooooooo003o04cooooo003oooooocWooooo0_l00?l01_oooooo0000ooooool00?ooooooo`00
odgooooo003oooooocWooooo00So003ooooooooooooo0000ooooool00?ooooooo`00odgooooo003o
ooooocSooooo0_l00?l01?oooooo0000ooooool00?l2ooooo`03o`00oooooooooooo04_ooooo003o
ooooocSooooo00So003ooooooooooooo0000ooooool00?ooooooo`00odkooooo003oooooocOooooo
0_l00?l00ooooooo0000ooooo`02o`00o`03ooooool00?oooooo04gooooo003oooooocOooooo0_l0
0?l01_oooooo0000ooooool00?ooooooo`00odoooooo003oooooocKooooo00Wo003oooooool00?oo
ooooo`000?oooooo003oooooool00?l0Coooool00?oooooo=_ooool2o`00o`06ooooool0003ooooo
o`00oooooooo003oD?ooool00?oooooo=Oooool02Ol00?ooooooo`00oooooooo0000ooooool00?oo
ooooo`00o`1@ooooo`00ooooooldooooo`09o`00ooooooooooooo`00oooooooo0000o`00oooooooo
003o057ooooo003oooooocCooooo00Wo003ooooooooooooo003oo`000?oooooo003oooooool00?l0
DOooool00?oooooo<oooool01_l00?ooooooooooool00?ooooooo`0000?o003oD_ooool00?oooooo
<_ooool02_l00?oooooooooooooooooo003oooooool0003o003oooooool00?mBooooo`00oooooolb
ooooo`07o`00ooooooooooooooooool00?oo0000ooooo`02o`00oe?ooooo003ooooooc7ooooo00?o
003oooooooooool00_ooool01Ol00?oo0000o`00oooooooo003o05?ooooo003ooooooc7ooooo00Oo
003oooooooooooooooooo`00oooooooo000000;o003oE?ooool00?oooooo<?ooool00ol00?oooooo
ooooo`02ooooo`04o`00ool0003o003oo`00oeGooooo003oooooobGooooo0ol00007ooooo`03o`00
oooooooooooo00?ooooo00Co003oo`000?l00?oo003oEOooool00?oooooo9?ooool01Ol0003ooooo
oooooooooooo000000Kooooo00?o003oooooooooool00oooool3o`00oeKooooo003oooooobSooooo
00?o0000ooooooooool00oooool00ol00?ooooooooooo`03ooooo`04o`00ool0003o003oo`00oeKo
oooo003oooooobKooooo0_l00005ooooo`03o`00oooooooooooo00Cooooo00?o003oo`000?l00?l0
Eoooool00?oooooo:?ooool00ol0003oooooooooo`02ooooo`03o`00oooooooooooo00Cooooo0ol0
0?mGooooo`00oooooolTooooo`05o`000?ooooooooooooooool000000oooool00ol00?oooooooooo
o`05ooooo`03o`000?l00?oooooo05Oooooo003oooooobGooooo0ol00004ooooo`03o`00oooooooo
oooo00Cooooo0ol00?mHooooo`00ooooool[ooooo`03o`00oooooooooooo00;ooooo1Ol0001Ioooo
o`00oooooolZooooo`03o`00oooooooooooo00;ooooo1ol0001Hooooo`00oooooolZooooo`04o`00
ooooooooooooooooo`Wo0000Eoooool00?oooooo:Oooool00ol00?ooooooooooo`02ooooo`Wo0000
Eoooool00?oooooo:Oooool00ol00?ooooooooooo`02ooooo`Wo0000Eoooool00?oooooo:?ooool0
0ol00?ooooooooooo`03ooooo`Wo0000Eoooool00?oooooo:?ooool00ol00?ooooooooooo`03oooo
o`Wo0000Eoooool00?oooooo:?ooool00ol00?ooooooooooo`04ooooo`Oo0000F?ooool00?oooooo
9oooool00ol00?ooooooooooo`05ooooo`03o`00ool0003o000000?o0000FOooool00?oooooo9ooo
ool00ol00?ooooooooooo`04ooooo`?o003oGOooool00?oooooo9_ooool00ol00?ooooooooooo`04
ooooo`03o`00ool0003o003o05kooooo003oooooobKooooo00?o003oooooooooool01?ooool3o`00
oekooooo003oooooobGooooo00?o003oooooooooool01?ooool3o`00oeoooooo003oooooobGooooo
00?o003oooooooooool00oooool01?l00?oo0000o`00ool00?mOooooo`00oooooolUooooo`03o`00
oooooooooooo00;ooooo00Co003oo`000?l00?oo003oH?ooool00?oooooo9?ooool00ol00?oooooo
ooooo`02ooooo`03o`00ool0003o003o00;o003oH?ooool00?oooooo9?ooool00ol00?oooooooooo
o`02ooooo`04o`00oooooooo003oo`00of7ooooo003oooooob?ooooo00?o003oooooooooool00_oo
ool01?l00?oo0000o`00ool00?mRooooo`00oooooolSooooo`07o`00ooooooooooooooooool00?oo
0000ooooo`02o`00of;ooooo003oooooob;ooooo00Oo003oooooooooooooooooo`00ool0003ooooo
00;o003oHoooool00?oooooo8_ooool01_l00?ooooooooooool00?ooooooo`0000?o003oHoooool0
0?oooooo8_ooool01_l00?ooooooo`00oooooooo0000ooooo`;o003oI?ooool00?oooooo8Oooool0
2Ol00?ooooooooooool00?oo0000ooooool00?ooooooo`00o`1Tooooo`00oooooolQooooo`05o`00
oooooooo003oo`000?ooool00ol00?mUooooo`00oooooolPooooo`09o`00oooooooo003oooooool0
003oooooo`00oooooooo003o06Gooooo003oooooob3ooooo0_l00?l00ooooooo0000ooooo`03o`00
ofKooooo003ooooooaoooooo0_l00?l01ooooooo0000oooooooooooo003oooooool00?l0I_ooool0
0?oooooo7oooool00ol00?ooooooo`000002ooooo`03o`00oooooooo003o06Oooooo003ooooooaoo
oooo00Co003oooooool0003ooooo0_l00?l00ooooooo003oooooo`1Vooooo`00oooooolNooooo`03
o`00oooooooo000000;ooooo00?o003oooooool00?l0J?ooool00?oooooo7Oooool2o`00o`03oooo
ool0003ooooo00;o003o00?oooooo`00ooooool0Ioooool00?oooooo7?ooool2o`00o`03ooooool0
003ooooo00;o003o00?oooooo`00ooooool0J?ooool00?oooooo6oooool02Ol00?ooooooo`00oooo
oooo0000ooooool00?ooooooo`00o`1Zooooo`00oooooolKooooo`;o003o00?oooooo`000?ooool0
0ol00?m[ooooo`00oooooolJooooo`09o`00oooooooo003oooooool0003oooooo`00oooooooo003o
06_ooooo003ooooooaWooooo00Ko003ooooooooooooo003oo`000?ooool3o`00ofcooooo003ooooo
oaSooooo00Ko003ooooooooooooo003oooooool00003o`00ofgooooo003ooooooaOooooo00Oo003o
ooooooooooooooooo`00ool0003ooooo00;o003oK_ooool00?oooooo5_ooool01ol00?oooooooooo
oooooooo003oooooool000000ol00?m^ooooo`00oooooolFooooo`06o`00ooooooooooooooooool0
0?oo00000ol00?m_ooooo`00oooooolEooooo`07o`00ooooooooooooooooool00?oo0000ooooo`02
o`00og3ooooo003ooooooaCooooo00?o003oooooooooool00_ooool01?l00?oo0000o`00ool00?ma
ooooo`00ooooool9ooooo`?o00001oooool00ol00?ooooooooooo`03ooooo`04o`000?oooooo003o
o`00og7ooooo003oooooo`[ooooo00?o0000ooooooooool01Oooool00ol00?ooooooooooo`03oooo
o`04o`00ool0003o003oo`00og;ooooo003oooooo`Oooooo1Ol00006ooooo`03o`00oooooooooooo
00?ooooo00?o0000o`00ool00?l0Loooool00?oooooo1oooool01?l0003oooooooooool00006oooo
o`03o`00oooooooooooo00?ooooo00Co003oo`000?l00?oo003oLoooool00?oooooo2?ooool00ol0
003oooooo`000005ooooo`03o`00oooooooooooo00Cooooo00?o0000o`00ool00?l0M?ooool00?oo
oooo2Oooool2o`0000Cooooo00?o003oooooooooool01?ooool3o`00ogGooooo003oooooo`[ooooo
00Go0000ooooooooooooooooo`00o`03ooooo`Go00000_l00?meooooo`00ooooool=ooooo`04o`00
ooooooooooooooooo`Oo0000M_ooool00?oooooo3Oooool00ol00?ooooooooooo`09o`0007Gooooo
003oooooo`cooooo00Co003ooooooooooooooooo2Ol0001eooooo`00ooooool;ooooo`03o`00oooo
oooooooo00;ooooo2Ol0001eooooo`00ooooool;ooooo`03o`00oooooooooooo00;ooooo2Ol0001e
ooooo`00ooooool:ooooo`03o`00oooooooooooo00?ooooo2Ol0001eooooo`00ooooool:ooooo`03
o`00oooooooooooo00?ooooo00?o003oo`000?l000001Ol0001fooooo`00ooooool9ooooo`03o`00
oooooooooooo00?ooooo00?o003oooooool00?l01Ol0001gooooo`00ooooool8ooooo`03o`00oooo
oooooooo00?ooooo00Co003oooooool00?oo003oO?ooool00?oooooo2?ooool00ol00?oooooooooo
o`02ooooo`04o`00oooooooo003oo`00oggooooo003oooooo`Oooooo00?o003oooooooooool00_oo
ool01?l00?ooooooo`000?l00?mnooooo`00ooooool6ooooo`03o`00oooooooooooo00;ooooo00?o
003oooooool000000_l00?mnooooo`00ooooool6ooooo`07o`00ooooooooooooooooool00?oooooo
o`000002o`00ogoooooo003oooooo`Gooooo00Oo003oooooooooooooooooo`00oooooooo000000;o
003oP?ooool00?oooooo1Oooool01_l00?ooooooooooool00?ooooooo`0000?o003oP?ooool00?oo
oooo1?ooool01?l00?ooooooooooool00?l2ooooo`?o003oPOooool00?oooooo0oooool01?l00?oo
ooooooooool00?l2ooooo`03o`000?l00?oo003o08;ooooo003oooooo`?ooooo0ol00?l2ooooo`04
o`000?l00?ooooooo`00oh;ooooo003oooooo`;ooooo0_l00?l3ooooo`04o`000?l00?oo003oo`00
oh?ooooo003oooooo`7ooooo0_l00?l3ooooo`04o`000?l00?oo003oo`00ohCooooo003oooooo`7o
oooo00Go003oooooooooooooooooo`000002o`00o`03ooooool00?oooooo08?ooooo003oooooo`05
o`00ooooooooooooooooool000000_l00?l00ooooooo003oooooo`24ooooo`00o_ooool2o`00o`;o
oooo00?o0000ooooool00?l00_l00?n6ooooo`00oOooool2o`00o`;ooooo00?o0000ooooool00?l0
0_l00?n7ooooo`00o?ooool2o`00o`;ooooo00Ko0000ooooool00?oo003oooooool00?n7ooooo`00
noooool01_l00?ooooooo`00oooooooo0000ooooo`;o003o00?oooooo`00ooooool0Qoooool00?[o
oooo00?o003oooooool00?l00_ooool01Ol0003o003oo`00oooooooo003o08Wooooo003iooooo`06
o`00ooooooooooooo`00oooooooo00001?l00?n:ooooo`00n?ooool01ol00?ooooooooooool00?oo
ooooo`000?ooool00ol00?n;ooooo`00moooool01ol00?ooooooooooool00?ooooooo`000?ooool0
0ol00?n<ooooo`00mOooool2o`00o`?ooooo00?o003oooooool000000ol00?n=ooooo`00m?ooool0
0ol00?ooooooooooo`02ooooo`03o`00oooooooo000000?o003oS_ooool00??ooooo00?o003ooooo
ooooool00_ooool00ol00?ooooooo`000003o`00ohoooooo003booooo`03o`00oooooooooooo00?o
oooo00?o003oo`000?l00?l00_l00?n@ooooo`00lOooool00ol00?ooooooooooo`03ooooo`03o`00
oooooooo000000;o003oTOooool00?3ooooo00?o003oooooooooool01?ooool01?l00?oo0000o`00
ool00?nBooooo`00koooool00ol00?ooooooooooo`04ooooo`04o`00ool0003o003oo`00oi?ooooo
003^ooooo`03o`00oooooooooooo00Cooooo00Co003oo`000?l00?oo003oU?ooool00>7ooooo0ol0
0009ooooo`03o`00oooooooooooo00Gooooo0ol00?nEooooo`00h?ooool01Ol0003ooooooooooooo
oooo000000Oooooo00?o003oooooooooool01Oooool3o`00oiKooooo003Tooooo`03o`000?oooooo
oooo00Cooooo00?o003oooooooooool01Oooool00ol00?oo0000o`00o`2Gooooo`00i?ooool00ol0
003oooooooooo`03ooooo`03o`00oooooooooooo00Kooooo00?o0000o`00ooooool0Uoooool00>3o
oooo1?l00005ooooo`03o`00oooooooooooo00Kooooo0_l00?nIooooo`00h?ooool00ol0003ooooo
ooooo`05ooooo`03o`00oooooooooooo00Oooooo00?o003oooooooooool0V?ooool00>3ooooo1Ol0
00000ooooooo003oo`00o`09ooooo`03o`00oooooooooooo09Wooooo003Uooooo`03o`00oooooooo
oooo00Sooooo00?o003oooooooooool0V_ooool00001\
\>"],
  ImageRangeCache->{{{0, 396}, {396, 0}} -> {0.690602, 0.326332, 0.000898855, \
0.00117527}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Let's save a run of 6 generations...", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[pd1]]; \[IndentingNewLine]Evolve[
          pd1], {6}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["...and then display it.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[\[IndentingNewLine]Partition[l, 3], 
        VertexStyle \[Rule] Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .65625 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.015625 0.31746 [
[ 0 0 0 0 ]
[ 1 .65625 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .65625 L
0 .65625 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.015625 0.321429 0.313244 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
.663 .663 .663 r
.07 w
.9544 .55707 Mdot
.94367 .61325 Mdot
0 0 0 r
.92596 .66764 Mdot
.90157 .71938 Mdot
.87087 .76767 Mdot
.663 .663 .663 r
.83435 .81173 Mdot
.79259 .85088 Mdot
.74624 .88449 Mdot
.69604 .91204 Mdot
.64277 .93309 Mdot
.58728 .94732 Mdot
.53044 .95448 Mdot
0 0 0 r
.47315 .95448 Mdot
.41632 .94732 Mdot
.36082 .93309 Mdot
.30756 .91204 Mdot
.25735 .88449 Mdot
.21101 .85088 Mdot
.16924 .81173 Mdot
.13272 .76767 Mdot
.10203 .71938 Mdot
.07763 .66764 Mdot
.05993 .61325 Mdot
.04919 .55707 Mdot
.0456 .5 Mdot
.663 .663 .663 r
.04919 .44293 Mdot
.05993 .38675 Mdot
.07763 .33236 Mdot
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
.21101 .14912 Mdot
.25735 .11551 Mdot
.30756 .08796 Mdot
.36082 .06691 Mdot
.41632 .05268 Mdot
.47315 .04552 Mdot
.53044 .04552 Mdot
.58728 .05268 Mdot
.64277 .06691 Mdot
.69604 .08796 Mdot
0 0 0 r
.74624 .11551 Mdot
.79259 .14912 Mdot
.663 .663 .663 r
.83435 .18827 Mdot
.87087 .23233 Mdot
.90157 .28062 Mdot
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.015625 0.64881 0.313244 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
.663 .663 .663 r
.07 w
.9544 .55707 Mdot
.94367 .61325 Mdot
.92596 .66764 Mdot
0 0 0 r
.90157 .71938 Mdot
.663 .663 .663 r
.87087 .76767 Mdot
.83435 .81173 Mdot
.79259 .85088 Mdot
.74624 .88449 Mdot
.69604 .91204 Mdot
.64277 .93309 Mdot
.58728 .94732 Mdot
.53044 .95448 Mdot
.47315 .95448 Mdot
0 0 0 r
.41632 .94732 Mdot
.36082 .93309 Mdot
.30756 .91204 Mdot
.25735 .88449 Mdot
.21101 .85088 Mdot
.16924 .81173 Mdot
.13272 .76767 Mdot
.10203 .71938 Mdot
.07763 .66764 Mdot
.05993 .61325 Mdot
.04919 .55707 Mdot
.663 .663 .663 r
.0456 .5 Mdot
.04919 .44293 Mdot
.05993 .38675 Mdot
.07763 .33236 Mdot
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
.21101 .14912 Mdot
.25735 .11551 Mdot
.30756 .08796 Mdot
.36082 .06691 Mdot
.41632 .05268 Mdot
.47315 .04552 Mdot
.53044 .04552 Mdot
.58728 .05268 Mdot
.64277 .06691 Mdot
.69604 .08796 Mdot
.74624 .11551 Mdot
.79259 .14912 Mdot
.83435 .18827 Mdot
.87087 .23233 Mdot
.90157 .28062 Mdot
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.015625 0.97619 0.313244 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
.663 .663 .663 r
.07 w
.9544 .55707 Mdot
0 0 0 r
.94367 .61325 Mdot
.92596 .66764 Mdot
.90157 .71938 Mdot
.87087 .76767 Mdot
.83435 .81173 Mdot
.663 .663 .663 r
.79259 .85088 Mdot
.74624 .88449 Mdot
.69604 .91204 Mdot
.64277 .93309 Mdot
.58728 .94732 Mdot
.53044 .95448 Mdot
.47315 .95448 Mdot
.41632 .94732 Mdot
0 0 0 r
.36082 .93309 Mdot
.30756 .91204 Mdot
.25735 .88449 Mdot
.21101 .85088 Mdot
.16924 .81173 Mdot
.13272 .76767 Mdot
.10203 .71938 Mdot
.07763 .66764 Mdot
.05993 .61325 Mdot
.663 .663 .663 r
.04919 .55707 Mdot
.0456 .5 Mdot
.04919 .44293 Mdot
.05993 .38675 Mdot
.07763 .33236 Mdot
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
.21101 .14912 Mdot
.25735 .11551 Mdot
.30756 .08796 Mdot
.36082 .06691 Mdot
.41632 .05268 Mdot
.47315 .04552 Mdot
.53044 .04552 Mdot
.58728 .05268 Mdot
.64277 .06691 Mdot
.69604 .08796 Mdot
.74624 .11551 Mdot
.79259 .14912 Mdot
.83435 .18827 Mdot
.87087 .23233 Mdot
.90157 .28062 Mdot
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.343006 0.321429 0.640625 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
0 0 0 r
.07 w
.9544 .55707 Mdot
.94367 .61325 Mdot
.92596 .66764 Mdot
.90157 .71938 Mdot
.663 .663 .663 r
.87087 .76767 Mdot
0 0 0 r
.83435 .81173 Mdot
.79259 .85088 Mdot
.74624 .88449 Mdot
.663 .663 .663 r
.69604 .91204 Mdot
.64277 .93309 Mdot
.58728 .94732 Mdot
.53044 .95448 Mdot
0 0 0 r
.47315 .95448 Mdot
.663 .663 .663 r
.41632 .94732 Mdot
0 0 0 r
.36082 .93309 Mdot
.30756 .91204 Mdot
.663 .663 .663 r
.25735 .88449 Mdot
0 0 0 r
.21101 .85088 Mdot
.16924 .81173 Mdot
.663 .663 .663 r
.13272 .76767 Mdot
.10203 .71938 Mdot
0 0 0 r
.07763 .66764 Mdot
.05993 .61325 Mdot
.663 .663 .663 r
.04919 .55707 Mdot
0 0 0 r
.0456 .5 Mdot
.04919 .44293 Mdot
.05993 .38675 Mdot
.07763 .33236 Mdot
.663 .663 .663 r
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
0 0 0 r
.21101 .14912 Mdot
.25735 .11551 Mdot
.30756 .08796 Mdot
.36082 .06691 Mdot
.41632 .05268 Mdot
.47315 .04552 Mdot
.663 .663 .663 r
.53044 .04552 Mdot
.58728 .05268 Mdot
.64277 .06691 Mdot
.69604 .08796 Mdot
0 0 0 r
.74624 .11551 Mdot
.663 .663 .663 r
.79259 .14912 Mdot
0 0 0 r
.83435 .18827 Mdot
.87087 .23233 Mdot
.90157 .28062 Mdot
.663 .663 .663 r
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
0 0 0 r
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.343006 0.64881 0.640625 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
0 0 0 r
.07 w
.9544 .55707 Mdot
.94367 .61325 Mdot
.92596 .66764 Mdot
.90157 .71938 Mdot
.87087 .76767 Mdot
.83435 .81173 Mdot
.79259 .85088 Mdot
.663 .663 .663 r
.74624 .88449 Mdot
.69604 .91204 Mdot
.64277 .93309 Mdot
0 0 0 r
.58728 .94732 Mdot
.53044 .95448 Mdot
.47315 .95448 Mdot
.41632 .94732 Mdot
.36082 .93309 Mdot
.30756 .91204 Mdot
.25735 .88449 Mdot
.21101 .85088 Mdot
.16924 .81173 Mdot
.13272 .76767 Mdot
.10203 .71938 Mdot
.07763 .66764 Mdot
.05993 .61325 Mdot
.04919 .55707 Mdot
.0456 .5 Mdot
.04919 .44293 Mdot
.05993 .38675 Mdot
.663 .663 .663 r
.07763 .33236 Mdot
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
.21101 .14912 Mdot
0 0 0 r
.25735 .11551 Mdot
.30756 .08796 Mdot
.36082 .06691 Mdot
.41632 .05268 Mdot
.663 .663 .663 r
.47315 .04552 Mdot
.53044 .04552 Mdot
.58728 .05268 Mdot
0 0 0 r
.64277 .06691 Mdot
.69604 .08796 Mdot
.74624 .11551 Mdot
.79259 .14912 Mdot
.83435 .18827 Mdot
.87087 .23233 Mdot
.663 .663 .663 r
.90157 .28062 Mdot
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.343006 0.97619 0.640625 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0455976 0.911952 0.0441679 0.910316 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.9544 .55707 m
.94716 .5848 L
.94367 .61325 L
s
0 0 1 r
.9544 .55707 m
.95091 .58552 L
.94367 .61325 L
s
.9544 .55707 m
.94903 .58516 L
.94367 .61325 L
s
.9544 .55707 m
.93649 .61141 L
.92596 .66764 L
s
.9544 .55707 m
.94388 .6133 L
.92596 .66764 L
s
.9544 .55707 m
.95059 .5 L
.9544 .44293 L
s
.9544 .55707 m
.95821 .5 L
.9544 .44293 L
s
0 0 0 r
.9544 .55707 m
.83607 .54046 L
.72099 .5083 L
s
0 0 1 r
.9544 .55707 m
.83933 .52492 L
.72099 .5083 L
s
.9544 .55707 m
.8377 .53269 L
.72099 .5083 L
s
0 0 0 r
.94367 .61325 m
.933 .63985 L
.92596 .66764 L
s
0 0 1 r
.94367 .61325 m
.93663 .64103 L
.92596 .66764 L
s
.94367 .61325 m
.93482 .64044 L
.92596 .66764 L
s
.94367 .61325 m
.91907 .66492 L
.90157 .71938 L
s
.94367 .61325 m
.92616 .66772 L
.90157 .71938 L
s
.94367 .61325 m
.82883 .56819 L
.72099 .5083 L
s
.94367 .61325 m
.83584 .55337 L
.72099 .5083 L
s
0 0 0 r
.92596 .66764 m
.91204 .6927 L
.90157 .71938 L
s
0 0 1 r
.92596 .66764 m
.9155 .69432 L
.90157 .71938 L
s
.92596 .66764 m
.91377 .69351 L
.90157 .71938 L
s
.92596 .66764 m
.89508 .71582 L
.87087 .76767 L
s
.92596 .66764 m
.90176 .71949 L
.87087 .76767 L
s
0 0 0 r
.90157 .71938 m
.88461 .7425 L
.87087 .76767 L
s
0 0 1 r
.90157 .71938 m
.88783 .74455 L
.87087 .76767 L
s
.90157 .71938 m
.88622 .74352 L
.87087 .76767 L
s
.90157 .71938 m
.86488 .76332 L
.83435 .81173 L
s
.90157 .71938 m
.87105 .76779 L
.83435 .81173 L
s
0 0 0 r
.87087 .76767 m
.85114 .78848 L
.83435 .81173 L
s
0 0 1 r
.87087 .76767 m
.85409 .79091 L
.83435 .81173 L
s
.87087 .76767 m
.85261 .7897 L
.83435 .81173 L
s
.87087 .76767 m
.82895 .80667 L
.79259 .85088 L
s
.87087 .76767 m
.83451 .81188 L
.79259 .85088 L
s
0 0 0 r
.83435 .81173 m
.81217 .82992 L
.79259 .85088 L
s
0 0 1 r
.83435 .81173 m
.81478 .83269 L
.79259 .85088 L
s
.83435 .81173 m
.81347 .8313 L
.79259 .85088 L
s
.83435 .81173 m
.78787 .84518 L
.74624 .88449 L
s
.83435 .81173 m
.79273 .85104 L
.74624 .88449 L
s
0 0 0 r
.79259 .85088 m
.7683 .86614 L
.74624 .88449 L
s
0 0 1 r
.79259 .85088 m
.77054 .86923 L
.74624 .88449 L
s
.79259 .85088 m
.76942 .86769 L
.74624 .88449 L
s
.79259 .85088 m
.74227 .87825 L
.69604 .91204 L
s
.79259 .85088 m
.74636 .88467 L
.69604 .91204 L
s
0 0 0 r
.74624 .88449 m
.72022 .8966 L
.69604 .91204 L
s
0 0 1 r
.74624 .88449 m
.72206 .89994 L
.69604 .91204 L
s
.74624 .88449 m
.72114 .89827 L
.69604 .91204 L
s
.74624 .88449 m
.69288 .90535 L
.64277 .93309 L
s
.74624 .88449 m
.69613 .91224 L
.64277 .93309 L
s
0 0 0 r
.69604 .91204 m
.6687 .9208 L
.64277 .93309 L
s
0 0 1 r
.69604 .91204 m
.67011 .92434 L
.64277 .93309 L
s
.69604 .91204 m
.66941 .92257 L
.64277 .93309 L
s
.69604 .91204 m
.64048 .92606 L
.58728 .94732 L
s
.69604 .91204 m
.64284 .9333 L
.58728 .94732 L
s
0 0 0 r
.64277 .93309 m
.61455 .93836 L
.58728 .94732 L
s
0 0 1 r
.64277 .93309 m
.6155 .94205 L
.58728 .94732 L
s
.64277 .93309 m
.61503 .94021 L
.58728 .94732 L
s
.64277 .93309 m
.58589 .94005 L
.53044 .95448 L
s
.64277 .93309 m
.58732 .94753 L
.53044 .95448 L
s
0 0 0 r
.58728 .94732 m
.55862 .94901 L
.53044 .95448 L
s
0 0 1 r
.58728 .94732 m
.5591 .95279 L
.53044 .95448 L
s
.58728 .94732 m
.55886 .9509 L
.53044 .95448 L
s
.58728 .94732 m
.52998 .9471 L
.47315 .95448 L
s
.58728 .94732 m
.53046 .9547 L
.47315 .95448 L
s
0 0 0 r
.53044 .95448 m
.5018 .95639 L
.47315 .95448 L
s
0 0 1 r
.53044 .95448 m
.5018 .95258 L
.47315 .95448 L
s
.53044 .95448 m
.5018 .95448 L
.47315 .95448 L
s
.53044 .95448 m
.47314 .9547 L
.41632 .94732 L
s
.53044 .95448 m
.47362 .9471 L
.41632 .94732 L
s
0 0 0 r
.47315 .95448 m
.44449 .95279 L
.41632 .94732 L
s
0 0 1 r
.47315 .95448 m
.44497 .94901 L
.41632 .94732 L
s
.47315 .95448 m
.44473 .9509 L
.41632 .94732 L
s
.47315 .95448 m
.41627 .94753 L
.36082 .93309 L
s
.47315 .95448 m
.4177 .94005 L
.36082 .93309 L
s
0 0 0 r
.41632 .94732 m
.3881 .94205 L
.36082 .93309 L
s
0 0 1 r
.41632 .94732 m
.38904 .93836 L
.36082 .93309 L
s
.41632 .94732 m
.38857 .94021 L
.36082 .93309 L
s
.41632 .94732 m
.36076 .9333 L
.30756 .91204 L
s
.41632 .94732 m
.36311 .92606 L
.30756 .91204 L
s
0 0 0 r
.36082 .93309 m
.33349 .92434 L
.30756 .91204 L
s
0 0 1 r
.36082 .93309 m
.33489 .9208 L
.30756 .91204 L
s
.36082 .93309 m
.33419 .92257 L
.30756 .91204 L
s
.36082 .93309 m
.30747 .91224 L
.25735 .88449 L
s
.36082 .93309 m
.31071 .90535 L
.25735 .88449 L
s
0 0 0 r
.30756 .91204 m
.28154 .89994 L
.25735 .88449 L
s
0 0 1 r
.30756 .91204 m
.28338 .8966 L
.25735 .88449 L
s
.30756 .91204 m
.28246 .89827 L
.25735 .88449 L
s
.30756 .91204 m
.25724 .88467 L
.21101 .85088 L
s
.30756 .91204 m
.26132 .87825 L
.21101 .85088 L
s
0 0 0 r
.25735 .88449 m
.23306 .86923 L
.21101 .85088 L
s
0 0 1 r
.25735 .88449 m
.2353 .86614 L
.21101 .85088 L
s
.25735 .88449 m
.23418 .86769 L
.21101 .85088 L
s
.25735 .88449 m
.21087 .85104 L
.16924 .81173 L
s
.25735 .88449 m
.21573 .84518 L
.16924 .81173 L
s
0 0 0 r
.21101 .85088 m
.18882 .83269 L
.16924 .81173 L
s
0 0 1 r
.21101 .85088 m
.19143 .82992 L
.16924 .81173 L
s
.21101 .85088 m
.19012 .8313 L
.16924 .81173 L
s
.21101 .85088 m
.16909 .81188 L
.13272 .76767 L
s
.21101 .85088 m
.17464 .80667 L
.13272 .76767 L
s
0 0 0 r
.16924 .81173 m
.14951 .79091 L
.13272 .76767 L
s
0 0 1 r
.16924 .81173 m
.15245 .78848 L
.13272 .76767 L
s
.16924 .81173 m
.15098 .7897 L
.13272 .76767 L
s
.16924 .81173 m
.13255 .76779 L
.10203 .71938 L
s
.16924 .81173 m
.13872 .76332 L
.10203 .71938 L
s
0 0 0 r
.13272 .76767 m
.11576 .74455 L
.10203 .71938 L
s
0 0 1 r
.13272 .76767 m
.11899 .7425 L
.10203 .71938 L
s
.13272 .76767 m
.11738 .74352 L
.10203 .71938 L
s
.13272 .76767 m
.10184 .71949 L
.07763 .66764 L
s
.13272 .76767 m
.10852 .71582 L
.07763 .66764 L
s
0 0 0 r
.10203 .71938 m
.0881 .69432 L
.07763 .66764 L
s
0 0 1 r
.10203 .71938 m
.09156 .6927 L
.07763 .66764 L
s
.10203 .71938 m
.08983 .69351 L
.07763 .66764 L
s
.10203 .71938 m
.07743 .66772 L
.05993 .61325 L
s
.10203 .71938 m
.08452 .66492 L
.05993 .61325 L
s
0 0 0 r
.07763 .66764 m
.06697 .64103 L
.05993 .61325 L
s
0 0 1 r
.07763 .66764 m
.0706 .63985 L
.05993 .61325 L
s
.07763 .66764 m
.06878 .64044 L
.05993 .61325 L
s
.07763 .66764 m
.05972 .6133 L
.04919 .55707 L
s
.07763 .66764 m
.06711 .61141 L
.04919 .55707 L
s
0 0 0 r
.05993 .61325 m
.05269 .58552 L
.04919 .55707 L
s
0 0 1 r
.05993 .61325 m
.05644 .5848 L
.04919 .55707 L
s
.05993 .61325 m
.05456 .58516 L
.04919 .55707 L
s
.05993 .61325 m
.04898 .5571 L
.0456 .5 L
s
.05993 .61325 m
.05655 .55615 L
.0456 .5 L
s
0 0 0 r
.04919 .55707 m
.04549 .52866 L
.0456 .5 L
s
0 0 1 r
.04919 .55707 m
.0493 .52842 L
.0456 .5 L
s
.04919 .55707 m
.0474 .52854 L
.0456 .5 L
s
.04919 .55707 m
.04538 .5 L
.04919 .44293 L
s
.04919 .55707 m
.05301 .5 L
.04919 .44293 L
s
0 0 0 r
.0456 .5 m
.04549 .47134 L
.04919 .44293 L
s
0 0 1 r
.0456 .5 m
.0493 .47158 L
.04919 .44293 L
s
.0456 .5 m
.0474 .47146 L
.04919 .44293 L
s
.0456 .5 m
.04898 .4429 L
.05993 .38675 L
s
.0456 .5 m
.05655 .44385 L
.05993 .38675 L
s
0 0 0 r
.04919 .44293 m
.05269 .41448 L
.05993 .38675 L
s
0 0 1 r
.04919 .44293 m
.05644 .4152 L
.05993 .38675 L
s
.04919 .44293 m
.05456 .41484 L
.05993 .38675 L
s
.04919 .44293 m
.05972 .3867 L
.07763 .33236 L
s
.04919 .44293 m
.06711 .38859 L
.07763 .33236 L
s
0 0 0 r
.05993 .38675 m
.06697 .35897 L
.07763 .33236 L
s
0 0 1 r
.05993 .38675 m
.0706 .36015 L
.07763 .33236 L
s
.05993 .38675 m
.06878 .35956 L
.07763 .33236 L
s
.05993 .38675 m
.07743 .33228 L
.10203 .28062 L
s
.05993 .38675 m
.08452 .33508 L
.10203 .28062 L
s
0 0 0 r
.07763 .33236 m
.0881 .30568 L
.10203 .28062 L
s
0 0 1 r
.07763 .33236 m
.09156 .3073 L
.10203 .28062 L
s
.07763 .33236 m
.08983 .30649 L
.10203 .28062 L
s
.07763 .33236 m
.10184 .28051 L
.13272 .23233 L
s
.07763 .33236 m
.10852 .28418 L
.13272 .23233 L
s
0 0 0 r
.10203 .28062 m
.11576 .25545 L
.13272 .23233 L
s
0 0 1 r
.10203 .28062 m
.11899 .2575 L
.13272 .23233 L
s
.10203 .28062 m
.11738 .25648 L
.13272 .23233 L
s
.10203 .28062 m
.13255 .23221 L
.16924 .18827 L
s
.10203 .28062 m
.13872 .23668 L
.16924 .18827 L
s
0 0 0 r
.13272 .23233 m
.14951 .20909 L
.16924 .18827 L
s
0 0 1 r
.13272 .23233 m
.15245 .21152 L
.16924 .18827 L
s
.13272 .23233 m
.15098 .2103 L
.16924 .18827 L
s
.13272 .23233 m
.16909 .18812 L
.21101 .14912 L
s
.13272 .23233 m
.17464 .19333 L
.21101 .14912 L
s
0 0 0 r
.16924 .18827 m
.18882 .16731 L
.21101 .14912 L
s
0 0 1 r
.16924 .18827 m
.19143 .17008 L
.21101 .14912 L
s
.16924 .18827 m
.19012 .1687 L
.21101 .14912 L
s
.16924 .18827 m
.21087 .14896 L
.25735 .11551 L
s
.16924 .18827 m
.21573 .15482 L
.25735 .11551 L
s
0 0 0 r
.21101 .14912 m
.23306 .13077 L
.25735 .11551 L
s
0 0 1 r
.21101 .14912 m
.2353 .13386 L
.25735 .11551 L
s
.21101 .14912 m
.23418 .13231 L
.25735 .11551 L
s
.21101 .14912 m
.25724 .11533 L
.30756 .08796 L
s
.21101 .14912 m
.26132 .12175 L
.30756 .08796 L
s
0 0 0 r
.25735 .11551 m
.28154 .10006 L
.30756 .08796 L
s
0 0 1 r
.25735 .11551 m
.28338 .1034 L
.30756 .08796 L
s
.25735 .11551 m
.28246 .10173 L
.30756 .08796 L
s
.25735 .11551 m
.30747 .08776 L
.36082 .06691 L
s
.25735 .11551 m
.31071 .09465 L
.36082 .06691 L
s
0 0 0 r
.30756 .08796 m
.33349 .07566 L
.36082 .06691 L
s
0 0 1 r
.30756 .08796 m
.33489 .0792 L
.36082 .06691 L
s
.30756 .08796 m
.33419 .07743 L
.36082 .06691 L
s
.30756 .08796 m
.36076 .0667 L
.41632 .05268 L
s
.30756 .08796 m
.36311 .07394 L
.41632 .05268 L
s
0 0 0 r
.36082 .06691 m
.3881 .05795 L
.41632 .05268 L
s
0 0 1 r
.36082 .06691 m
.38904 .06164 L
.41632 .05268 L
s
.36082 .06691 m
.38857 .05979 L
.41632 .05268 L
s
.36082 .06691 m
.41627 .05247 L
.47315 .04552 L
s
.36082 .06691 m
.4177 .05995 L
.47315 .04552 L
s
0 0 0 r
.41632 .05268 m
.44449 .04721 L
.47315 .04552 L
s
0 0 1 r
.41632 .05268 m
.44497 .05099 L
.47315 .04552 L
s
.41632 .05268 m
.44473 .0491 L
.47315 .04552 L
s
.41632 .05268 m
.47314 .0453 L
.53044 .04552 L
s
.41632 .05268 m
.47362 .0529 L
.53044 .04552 L
s
0 0 0 r
.47315 .04552 m
.5018 .04742 L
.53044 .04552 L
s
0 0 1 r
.47315 .04552 m
.5018 .04361 L
.53044 .04552 L
s
.47315 .04552 m
.5018 .04552 L
.53044 .04552 L
s
.47315 .04552 m
.52998 .0529 L
.58728 .05268 L
s
.47315 .04552 m
.53046 .0453 L
.58728 .05268 L
s
0 0 0 r
.53044 .04552 m
.55862 .05099 L
.58728 .05268 L
s
0 0 1 r
.53044 .04552 m
.5591 .04721 L
.58728 .05268 L
s
.53044 .04552 m
.55886 .0491 L
.58728 .05268 L
s
.53044 .04552 m
.58589 .05995 L
.64277 .06691 L
s
.53044 .04552 m
.58732 .05247 L
.64277 .06691 L
s
0 0 0 r
.58728 .05268 m
.61455 .06164 L
.64277 .06691 L
s
0 0 1 r
.58728 .05268 m
.6155 .05795 L
.64277 .06691 L
s
.58728 .05268 m
.61503 .05979 L
.64277 .06691 L
s
.58728 .05268 m
.64048 .07394 L
.69604 .08796 L
s
.58728 .05268 m
.64284 .0667 L
.69604 .08796 L
s
0 0 0 r
.64277 .06691 m
.6687 .0792 L
.69604 .08796 L
s
0 0 1 r
.64277 .06691 m
.67011 .07566 L
.69604 .08796 L
s
.64277 .06691 m
.66941 .07743 L
.69604 .08796 L
s
.64277 .06691 m
.69288 .09465 L
.74624 .11551 L
s
.64277 .06691 m
.69613 .08776 L
.74624 .11551 L
s
0 0 0 r
.69604 .08796 m
.72022 .1034 L
.74624 .11551 L
s
0 0 1 r
.69604 .08796 m
.72206 .10006 L
.74624 .11551 L
s
.69604 .08796 m
.72114 .10173 L
.74624 .11551 L
s
.69604 .08796 m
.74227 .12175 L
.79259 .14912 L
s
.69604 .08796 m
.74636 .11533 L
.79259 .14912 L
s
0 0 0 r
.74624 .11551 m
.7683 .13386 L
.79259 .14912 L
s
0 0 1 r
.74624 .11551 m
.77054 .13077 L
.79259 .14912 L
s
.74624 .11551 m
.76942 .13231 L
.79259 .14912 L
s
.74624 .11551 m
.78787 .15482 L
.83435 .18827 L
s
.74624 .11551 m
.79273 .14896 L
.83435 .18827 L
s
0 0 0 r
.79259 .14912 m
.81217 .17008 L
.83435 .18827 L
s
0 0 1 r
.79259 .14912 m
.81478 .16731 L
.83435 .18827 L
s
.79259 .14912 m
.81347 .1687 L
.83435 .18827 L
s
.79259 .14912 m
.82895 .19333 L
.87087 .23233 L
s
.79259 .14912 m
.83451 .18812 L
.87087 .23233 L
s
0 0 0 r
.83435 .18827 m
.85114 .21152 L
.87087 .23233 L
s
0 0 1 r
.83435 .18827 m
.85409 .20909 L
.87087 .23233 L
s
.83435 .18827 m
.85261 .2103 L
.87087 .23233 L
s
.83435 .18827 m
.86488 .23668 L
.90157 .28062 L
s
.83435 .18827 m
.87105 .23221 L
.90157 .28062 L
s
0 0 0 r
.87087 .23233 m
.88461 .2575 L
.90157 .28062 L
s
0 0 1 r
.87087 .23233 m
.88783 .25545 L
.90157 .28062 L
s
.87087 .23233 m
.88622 .25648 L
.90157 .28062 L
s
.87087 .23233 m
.89508 .28418 L
.92596 .33236 L
s
.87087 .23233 m
.90176 .28051 L
.92596 .33236 L
s
0 0 0 r
.90157 .28062 m
.91204 .3073 L
.92596 .33236 L
s
0 0 1 r
.90157 .28062 m
.9155 .30568 L
.92596 .33236 L
s
.90157 .28062 m
.91377 .30649 L
.92596 .33236 L
s
.90157 .28062 m
.91907 .33508 L
.94367 .38675 L
s
.90157 .28062 m
.92616 .33228 L
.94367 .38675 L
s
0 0 0 r
.92596 .33236 m
.933 .36015 L
.94367 .38675 L
s
0 0 1 r
.92596 .33236 m
.93663 .35897 L
.94367 .38675 L
s
.92596 .33236 m
.93482 .35956 L
.94367 .38675 L
s
.92596 .33236 m
.93649 .38859 L
.9544 .44293 L
s
.92596 .33236 m
.94388 .3867 L
.9544 .44293 L
s
0 0 0 r
.94367 .38675 m
.94716 .4152 L
.9544 .44293 L
s
0 0 1 r
.94367 .38675 m
.95091 .41448 L
.9544 .44293 L
s
.94367 .38675 m
.94903 .41484 L
.9544 .44293 L
s
.94367 .38675 m
.82827 .44012 L
.72099 .5083 L
s
.94367 .38675 m
.83639 .45494 L
.72099 .5083 L
s
0 0 0 r
.9544 .44293 m
.83552 .46785 L
.72099 .5083 L
s
0 0 1 r
.9544 .44293 m
.83988 .48338 L
.72099 .5083 L
s
.9544 .44293 m
.8377 .47561 L
.72099 .5083 L
s
.663 .663 .663 r
.07 w
.9544 .55707 Mdot
0 0 0 r
.94367 .61325 Mdot
.92596 .66764 Mdot
.90157 .71938 Mdot
.87087 .76767 Mdot
.83435 .81173 Mdot
.663 .663 .663 r
.79259 .85088 Mdot
.74624 .88449 Mdot
.69604 .91204 Mdot
.64277 .93309 Mdot
.58728 .94732 Mdot
0 0 0 r
.53044 .95448 Mdot
.47315 .95448 Mdot
.41632 .94732 Mdot
.36082 .93309 Mdot
.30756 .91204 Mdot
.25735 .88449 Mdot
.21101 .85088 Mdot
.16924 .81173 Mdot
.13272 .76767 Mdot
.10203 .71938 Mdot
.07763 .66764 Mdot
.05993 .61325 Mdot
.04919 .55707 Mdot
.0456 .5 Mdot
.04919 .44293 Mdot
.663 .663 .663 r
.05993 .38675 Mdot
.07763 .33236 Mdot
.10203 .28062 Mdot
.13272 .23233 Mdot
.16924 .18827 Mdot
.21101 .14912 Mdot
.25735 .11551 Mdot
0 0 0 r
.30756 .08796 Mdot
.36082 .06691 Mdot
.663 .663 .663 r
.41632 .05268 Mdot
.47315 .04552 Mdot
.53044 .04552 Mdot
.58728 .05268 Mdot
.64277 .06691 Mdot
0 0 0 r
.69604 .08796 Mdot
.74624 .11551 Mdot
.79259 .14912 Mdot
.83435 .18827 Mdot
.663 .663 .663 r
.87087 .23233 Mdot
.90157 .28062 Mdot
.92596 .33236 Mdot
.94367 .38675 Mdot
.9544 .44293 Mdot
.72099 .5083 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[19]:=",
  ImageSize->{438, 287.438},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006f00017b000`400?l00000o`00003ooooooong
ooooo`00oooooongooooo`00oooooongooooo`00oooooongooooo`00oooooongooooo`00oooooong
ooooo`00oooooongooooo`00oooooongooooo`00oooooongooooo`00A_ooool5ojVYZ@;ooooo1OnY
ZJV2ooooo`GoZJVY0oooool5ojVYZH;ooooo1OnYZJT3ooooo`GoZJVYA_ooool003kooooo1OnYZJT2
ooooo`koZJVY00?oooooojVYZOnYZJT00onYZJUdooooo`GoZJVY00?oooooojVYZOnYZJT01OnYZJT0
0oooooooZJVYojVYZ@05ojVYZ@03oooooonYZJWoZJVY00?oZJVYM?ooool5ojVYZ@03oooooonYZJWo
ZJVY00GoZJVY00?oooooojVYZOnYZJT01OnYZJT00oooooooZJVYojVYZ@03ojVYZCoooooo000moooo
oagoZJVYL_oooolMojVYZG;ooooo7OnYZJTnooooo`00=ooooolTojVYZ@03oooooonYZJWoZJVY00?o
ZJVYIOoooolTojVYZ@03oooooonYZJWoZJVY00?oZJVYIOoooolYojVYZCSooooo000fooooobcoZJVY
Hoooool/ojVYZF?ooooo:onYZJTgooooo`00<?oooolhojVYZEOooooo=onYZJUHooooocOoZJVY<Ooo
ool002oooooo>_nYZJUEooooocWoZJVYE_ooooliojVYZC3ooooo000^oooooccoZJVYDooooolMojVY
Z@03oooooonYZJWoZJVY01_oZJVYE?oooolMojVYZ@03oooooonYZJWoZJVY01_oZJVY;oooool002ko
oooo5_nYZJT2ooooo`GoZJVY0_ooool5ojVYZ@03oooooonYZJWoZJVY01GoZJVYDooooolFojVYZ@03
oooooonYZJWoZJVY00?oZJVY0oooool5ojVYZ@03oooooonYZJWoZJVY01CoZJVYE?oooolFojVYZ@03
oooooonYZJWoZJVY00?oZJVY0oooool5ojVYZ@03oooooonYZJWoZJVY01CoZJVY;oooool002[ooooo
6OnYZJTAooooo`GoZJVY00?oooooojVYZOnYZJT03OnYZJT5o`0004[ooooo6_nYZJTAooooo`GoZJVY
00?oooooojVYZOnYZJT04_nYZJU:oooooa[oZJVY4OoooolJojVYZB[ooooo000YoooooaCoZJVY7_oo
ool=ojVYZ@Oo0000B?oooolEojVYZAkooooo5?nYZJU8oooooaGoZJVY7OoooolEojVYZBWooooo000X
oooooaCoZJVY8?ooool;ojVYZ@Wo0000A_oooolEojVYZB3ooooo5?nYZJU6oooooaGoZJVY7ooooolE
ojVYZBSooooo000Xooooo`koZJVY;?ooool5ojVYZ@Wo0000A_ooool?ojVYZB_ooooo3onYZJU6oooo
o`ooZJVY:oooool?ojVYZBSooooo000Toooooa7oZJVY;_ooool4ojVYZ@go0000?_oooolBojVYZBgo
oooo4_nYZJTnoooooa;oZJVY;OoooolBojVYZBCooooo000Sooooo`koZJVY=_ooool>o`0003cooooo
3_nYZJTgooooo`koZJVY??ooool>ojVYZCOooooo3_nYZJTSooooo`008_ooool?ojVYZCKooooo3ol0
000jooooo`ooZJVY=oooool?ojVYZC[ooooo3onYZJTgooooo`ooZJVY8_ooool002;ooooo3_nYZJTh
ooooo`ko0000>_ooool>ojVYZCWooooo3_nYZJTjooooo`koZJVY>Oooool>ojVYZB;ooooo000Roooo
o`goZJVY>_ooool=o`0003[ooooo3OnYZJTkooooo`goZJVY>_ooool=ojVYZC_ooooo3OnYZJTRoooo
o`007_ooool=ojVYZD;ooooo1ol00005ojVYZCCooooo3?nYZJU3ooooo`coZJVY=?ooool<ojVYZD?o
oooo3?nYZJTOooooo`007Oooool>ojVYZD;ooooo1_l00007ojVYZC;ooooo3OnYZJU3ooooo`goZJVY
<_ooool=ojVYZD?ooooo3OnYZJTNooooo`007?ooool>ojVYZDCooooo1?l00009ojVYZC3ooooo3OnY
ZJU5ooooo`goZJVY<?ooool=ojVYZDGooooo3OnYZJTMooooo`007?ooool=ojVYZDKooooo0ol00009
ojVYZC3ooooo3?nYZJU7ooooo`coZJVY<?ooool<ojVYZDOooooo3?nYZJTMooooo`007?ooool9ojVY
ZDgooooo2OnYZJT`ooooo`WoZJVYCOooool9ojVYZC3ooooo2OnYZJU=ooooo`WoZJVY7Oooool001co
oooo2OnYZJU=ooooo`WoZJVY<?ooool9ojVYZDgooooo2OnYZJT`ooooo`WoZJVYCOooool9ojVYZAgo
oooo000Iooooo`coZJVYCOooool<ojVYZB[ooooo3?nYZJU=ooooo`coZJVY:_ooool<ojVYZDgooooo
3?nYZJTJooooo`006?ooool<ojVYZDoooooo3?nYZJTXooooo`coZJVYCoooool<ojVYZBSooooo3?nY
ZJU?ooooo`coZJVY6Oooool001Oooooo3?nYZJUAooooo`coZJVY9_ooool<ojVYZE7ooooo3?nYZJTV
ooooo`coZJVYDOooool<ojVYZASooooo000Gooooo`WoZJVYEoooool9ojVYZBKooooo2OnYZJUGoooo
o`WoZJVY9_ooool9ojVYZEOooooo2OnYZJTHooooo`005oooool9ojVYZEOooooo2OnYZJTVooooo`Wo
ZJVYEoooool9ojVYZBKooooo2OnYZJUGooooo`WoZJVY6?ooool001Oooooo2OnYZJUGooooo`WoZJVY
9_ooool9ojVYZEOooooo2OnYZJTVooooo`WoZJVYEoooool9ojVYZASooooo000Eooooo`_oZJVYEooo
ool;ojVYZB;ooooo2onYZJUGooooo`_oZJVY8_ooool;ojVYZEOooooo2onYZJTFooooo`005?ooool;
ojVYZEWooooo2onYZJTPooooo`_oZJVYFOooool;ojVYZB3ooooo2onYZJUIooooo`_oZJVY5Oooool0
01?ooooo2onYZJUKooooo`_oZJVY7_ooool;ojVYZE_ooooo2onYZJTNooooo`_oZJVYFoooool;ojVY
ZACooooo000Cooooo`WoZJVYGoooool9ojVYZAkooooo2OnYZJUOooooo`WoZJVY7_ooool9ojVYZEoo
oooo2OnYZJTDooooo`004oooool9ojVYZEoooooo2OnYZJTNooooo`WoZJVYGoooool9ojVYZAkooooo
2OnYZJUOooooo`WoZJVY5?ooool001?ooooo2OnYZJUOooooo`WoZJVY7_ooool9ojVYZEoooooo2OnY
ZJTNooooo`WoZJVYGoooool9ojVYZACooooo000Cooooo`WoZJVYGoooool9ojVYZAkooooo2OnYZJUO
ooooo`WoZJVY7_ooool9ojVYZEoooooo2OnYZJTDooooo`004_ooool9ojVYZF7ooooo2OnYZJTLoooo
o`WoZJVYHOooool9ojVYZAcooooo2OnYZJUQooooo`WoZJVY4oooool0017ooooo2OnYZJUSooooo`Wo
ZJVY6_ooool9ojVYZF?ooooo2OnYZJTJooooo`WoZJVYHoooool9ojVYZA;ooooo000@ooooo`WoZJVY
IOooool9ojVYZASooooo2OnYZJUUooooo`WoZJVY6?ooool9ojVYZFGooooo2OnYZJTAooooo`004?oo
ool9ojVYZFGooooo2OnYZJTHooooo`WoZJVYIOooool9ojVYZASooooo2OnYZJUUooooo`WoZJVY4Ooo
ool0013ooooo2OnYZJUUooooo`WoZJVY6?ooool9ojVYZFGooooo2OnYZJTHooooo`WoZJVYIOooool9
ojVYZA7ooooo000@ooooo`WoZJVYIOooool9ojVYZASooooo2OnYZJUUooooo`WoZJVY6?ooool9ojVY
ZFGooooo2OnYZJTAooooo`004?ooool9ojVYZFGooooo2OnYZJTHooooo`WoZJVYIOooool9ojVYZASo
oooo2OnYZJUUooooo`WoZJVY4Oooool0013ooooo2?nYZJUWooooo`WoZJVY5oooool8ojVYZFOooooo
2OnYZJTGooooo`SoZJVYIoooool8ojVYZA7ooooo000?ooooo`SoZJVYJOooool9ojVYZAGooooo2?nY
ZJUYooooo`WoZJVY5Oooool8ojVYZFWooooo2?nYZJT@ooooo`003_ooool9ojVYZF[ooooo2OnYZJTC
ooooo`WoZJVYJ_ooool9ojVYZA?ooooo2OnYZJUYooooo`WoZJVY3oooool000kooooo2OnYZJUZoooo
o`WoZJVY4oooool9ojVYZF[ooooo2OnYZJTCooooo`WoZJVYJOooool9ojVYZ@oooooo000>ooooo`Wo
ZJVYJ_ooool9ojVYZA?ooooo2OnYZJUZooooo`WoZJVY4oooool9ojVYZFWooooo2OnYZJT?ooooo`00
3_ooool9ojVYZF[ooooo2OnYZJTCooooo`WoZJVYJ_ooool9ojVYZA?ooooo2OnYZJUYooooo`WoZJVY
3oooool000kooooo2OnYZJUYooooo`03o`00oonYZJWoZJVY00OoZJVY4oooool9ojVYZFWooooo00?o
003oojVYZOnYZJT01onYZJTCooooo`WoZJVYJ?ooool00ol00?ooZJVYojVYZ@07ojVYZ@oooooo000?
ooooo`OoZJVYIoooool4o`00o`03oooooonYZJWoZJVY00GoZJVY5?ooool8ojVYZFOooooo1?l00?l0
0oooooooZJVYojVYZ@05ojVYZACooooo2?nYZJUWooooo`?o003o00?oooooojVYZOnYZJT01_nYZJT?
ooooo`003_ooool7ojVYZFKooooo0_l00?l00ooooooo003oo`00o`03ooooo`OoZJVY4_ooool8ojVY
ZFKooooo0_l00?l00ooooooo003oo`00o`03ooooo`OoZJVY4_ooool8ojVYZFKooooo1?l00?l3oooo
o`SoZJVY3_ooool000gooooo2OnYZJUSooooo`;o003o00?oooooo`00ool00?l01?ooool9ojVYZA3o
oooo2OnYZJUTooooo`;o003o00?oooooo`00ool00?l01?ooool9ojVYZA3ooooo2OnYZJUTooooo`;o
003o00?oooooo`00ooooool01?ooool9ojVYZ@gooooo000=ooooo`WoZJVYHOooool2o`00o`;ooooo
00?o003oooooooooool01?ooool9ojVYZA3ooooo2OnYZJURooooo`;o003o0_ooool00ol00?oooooo
ooooo`04ooooo`WoZJVY4?ooool9ojVYZF;ooooo0_l00?l2ooooo`03o`00oooooooooooo00Cooooo
2OnYZJT=ooooo`003Oooool9ojVYZEoooooo0_l00?l2ooooo`;o003o1oooool9ojVYZA3ooooo2OnY
ZJUPooooo`;o003o0_ooool2o`00o`Oooooo2OnYZJT@ooooo`WoZJVYH?ooool2o`00o`;ooooo0_l0
0?l7ooooo`WoZJVY3Oooool000gooooo2OnYZJUNooooo`05o`00ooooooooooooooooool00?l01ooo
ool2o`0000WoZJVY4?ooool9ojVYZEoooooo00Go003oooooooooooooooooo`00o`07ooooo`;o0000
2OnYZJT@ooooo`WoZJVYGoooool01Ol00?oooooooooooooooooo003o00Oooooo0_l00009ojVYZ@go
oooo000=ooooo`WoZJVYG?ooool2o`00o`;ooooo0_l00?l5ooooo`;o00000ol00?l9ojVYZA3ooooo
2OnYZJUMooooo`;o003o0_ooool2o`00o`Gooooo0_l00003o`00o`WoZJVY4?ooool9ojVYZEgooooo
0_l00?l2ooooo`;o003o1?ooool3o`0000?o003o2OnYZJT=ooooo`003_ooool7ojVYZEcooooo00?o
003oooooooooool00_l00?l3ooooo`?o00001?l00?l3ooooo`OoZJVY4_ooool7ojVYZEgooooo00?o
003oooooooooool00_l00?l3ooooo`?o00001?l00?l3ooooo`OoZJVY4_ooool7ojVYZEgooooo00?o
003oooooooooool00_l00?l2ooooo`Co00001?l00?l3ooooo`OoZJVY3_ooool000kooooo00?o0000
ojVYZOnYZJT00onYZJUKooooo`;o003o00?oooooo`00ool00?l00_ooool3o`0000Co003o1oooool5
ojVYZACooooo1OnYZJULooooo`;o003o00?oooooo`00ool00?l00_ooool3o`0000Co003o1oooool5
ojVYZACooooo1OnYZJULooooo`;o003o00Coooooo`00ool00?oooooo0ol000000ooooooo003oo`00
o`02o`00o`Oooooo1OnYZJT?ooooo`003Oooool7o`0005;ooooo1OnYZJT2ooooo`Go003o00?ooooo
o`000?l000001_l00?l;ooooo`;o003o5?ooool7ojVYZE;ooooo1OnYZJT2ooooo`Go003o00?ooooo
o`000?l000001_l00?l:ooooo`;o003o5Oooool7ojVYZE;ooooo1OnYZJT2ooooo`Go003o0ol00006
o`00o`[ooooo0_l00?lAooooo`003?ooool9o`00053ooooo1onYZJT3o`00o`03ooooool0003o003o
00Ko003o3_ooool2o`00oa?ooooo2OnYZJU@ooooo`OoZJVY0ol00?l00ooooooo0000o`00o`06o`00
o`gooooo0_l00?lDooooo`WoZJVYD?ooool7ojVYZ@?o003o0_l00007o`00o`gooooo0_l00?lAoooo
o`003?ooool9o`0004oooooo2OnYZJT00ol0003o003oo`00o`05o`00oa7ooooo0_l00?lCooooo`Wo
ZJVYCoooool9ojVYZ@03o`000?l00?oo003o00Go003o4?ooool2o`00oaCooooo2OnYZJU?ooooo`Wo
ZJVY00?o0000o`00ool00?l01Ol00?l@ooooo`;o003o4Oooool000cooooo2Ol0001?ooooo`WoZJVY
0ol00?lFooooo`;o003o4oooool9ojVYZDoooooo2OnYZJT3o`00oaGooooo0_l00?lDooooo`WoZJVY
Coooool9ojVYZ@?o003o5Oooool2o`00oa7ooooo000<ooooo`Wo0000Coooool9ojVYZAWooooo0_l0
0?lCooooo`WoZJVYCoooool9ojVYZASooooo0_l00?lDooooo`WoZJVYCoooool9ojVYZASooooo0_l0
0?lAooooo`003?ooool9o`0004oooooo2OnYZJT7o`00oa;ooooo0_l00?lCooooo`WoZJVYCoooool9
ojVYZ@Oo003o4Oooool2o`00oaCooooo2OnYZJU?ooooo`WoZJVY1ol00?lAooooo`;o003o4Oooool0
00gooooo1ol0001@ooooo`WoZJVY0_l00?l2o`0000Wo003o2_ooool5ojVYZA?ooooo1onYZJU@oooo
o`WoZJVY0_l00?l2o`0000Wo003o2_ooool5ojVYZA?ooooo1onYZJU@ooooo`WoZJVY0_l00?l2o`00
00Wo003o2_ooool5ojVYZ@oooooo000>ooooo`Oo0000D?ooool7ojVYZ@03ooooool00?oo003o00?o
003o1?l00008o`00o`Gooooo1onYZJTBooooo`03o`000?nYZJWoZJVY00?oZJVY00?o0000oooooooo
ool0Coooool7ojVYZ@03ooooool00?oo003o00?o003o1?l00008o`00o`Gooooo1onYZJTBooooo`Oo
ZJVYDOooool7ojVYZ@03ooooool00?oo003o00;o003o00?oooooo`000?l000000_l00008o`00o`Go
oooo1onYZJT>ooooo`003Oooool9o`00053ooooo1OnYZJT4ooooo`;o003o00?oooooo`00ool00?l0
0_ooool5o`0000Ko003o00?oooooojVYZOnYZJT01onYZJT@ooooo`Wo0000DOooool5ojVYZ@Cooooo
0_l00?l00ooooooo003oo`00o`02ooooo`Go00001_l00?l00oooooooZJVYojVYZ@07ojVYZA3ooooo
2OnYZJUAooooo`GoZJVY1?ooool5o`00o`;ooooo1Ol00006o`00o`03oooooonYZJWoZJVY00OoZJVY
3Oooool000gooooo2Ol0001Kooooo`;o003o00Coooooo`00ool00?oo003o1Oooool4o`0000;o003o
2OnYZJT@ooooo`Wo0000G?ooool2o`00o`04ooooool00?oo003oo`00o`Gooooo1?l00002o`00o`Wo
ZJVY4?ooool9ojVYZEcooooo0_l00?l00ooooooo003oo`00o`06ooooo`Co00000_l00?l9ojVYZ@go
oooo000=ooooo`Wo0000GOooool01?l00?ooooooooooooooool2o`00o`Wooooo2OnYZJT@ooooo`Wo
0000G_ooool01?l00?ooooooooooooooool2o`00o`Wooooo2OnYZJT@ooooo`WoZJVYG_ooool00ol0
0?ooooooooooo`02o`00o`[ooooo2OnYZJT=ooooo`003Oooool9o`0005kooooo0_l00?l3ooooo`;o
003o1oooool9ojVYZA3ooooo2Ol0001Oooooo`;o003o0oooool2o`00o`Oooooo2OnYZJT@ooooo`Wo
ZJVYGoooool2o`00o`;ooooo0_l00?l8ooooo`WoZJVY3Oooool000gooooo2Ol0001Pooooo`;o003o
0oooool00ol00?ooooooooooo`04ooooo`WoZJVY4?ooool9o`00067ooooo0_l00?l3ooooo`03o`00
oooooooooooo00Cooooo2OnYZJT@ooooo`WoZJVYHOooool2o`00o`;ooooo00?o003oooooooooool0
1Oooool9ojVYZ@gooooo000>ooooo`Oo0000Hoooool3o`00o`03ooooool00?oo003o00Gooooo1onY
ZJTBooooo`Oo0000I?ooool3o`00o`03ooooool00?oo003o00Gooooo1onYZJTBooooo`OoZJVYI?oo
ool5o`00o`Kooooo1onYZJT>ooooo`003oooool6o`0006Kooooo0_l00?l00ooooooo003oo`00o`03
ooooo`KoZJVY5?ooool7o`0006Kooooo0_l00?l00ooooooo003oo`00o`03ooooo`KoZJVY5?ooool5
ojVYZ@;o0000I_ooool4o`00o`?ooooo1Ol00002ojVYZ@oooooo000?ooooo`Oo0000Ioooool5o`00
o`OoZJVY5Oooool7o`0006Oooooo1Ol00?l7ojVYZAGooooo1ol0001Wooooo`Co003o1ol0000@oooo
o`003_ooool9o`0006Wooooo00?o003oojVYZOnYZJT01onYZJTCooooo`Wo0000JOooool00ol00?oo
ZJVYojVYZ@07ojVYZA?ooooo2Ol0001Xooooo`03o`00ool0003o000000Oo00003oooool000kooooo
2Ol0001Zooooo`WoZJVY4oooool9o`0006[ooooo2OnYZJTCooooo`Wo0000JOooool9o`0000oooooo
000>ooooo`Wo0000J_ooool9ojVYZA?ooooo2Ol0001Zooooo`WoZJVY4oooool9o`0006Wooooo2Ol0
000?ooooo`003_ooool9o`0006[ooooo2OnYZJTCooooo`Wo0000J_ooool9ojVYZA?ooooo2Ol0001Y
ooooo`Wo00003oooool000kooooo2Ol0001Zooooo`WoZJVY4oooool9o`0006[ooooo2OnYZJTCoooo
o`Wo0000JOooool9o`0000oooooo000?ooooo`So0000JOooool5o`0000CoZJVY5Oooool8o`0006Wo
oooo2OnYZJTEooooo`So0000JOooool8o`00013ooooo000@ooooo`So0000Ioooool7o`0000;oZJVY
5oooool8o`0006Oooooo2OnYZJTGooooo`So0000Ioooool8o`00017ooooo000@ooooo`Wo0000IOoo
ool9o`0001Sooooo2Ol0001Uooooo`WoZJVY6?ooool9o`0006Gooooo2Ol0000Aooooo`004?ooool9
o`0006Gooooo2Ol0000Hooooo`Wo0000IOooool9ojVYZASooooo2Ol0001Uooooo`Wo00004Oooool0
013ooooo2Ol0001Uooooo`Wo00006?ooool9o`0006Gooooo2OnYZJTHooooo`Wo0000IOooool9o`00
017ooooo000@ooooo`Wo0000IOooool9o`0001Sooooo2Ol0001Uooooo`WoZJVY6?ooool9o`0006Go
oooo2Ol0000Aooooo`004?ooool:o`0006?ooooo2_l0000Hooooo`[o0000Hoooool5o`0000GoZJVY
6?ooool:o`0006?ooooo2_l0000Aooooo`004Oooool:o`00067ooooo2_l0000Jooooo`[o0000HOoo
ool7o`0000?oZJVY6_ooool:o`00067ooooo2_l0000Booooo`004_ooool:o`0005oooooo2_l0000L
ooooo`[o0000Goooool9o`000003ojVYZOoooooooooo01[ooooo2_l0001Oooooo`[o00004oooool0
01?ooooo2Ol0001Oooooo`Wo00007_ooool9o`0005oooooo2Ol0000Nooooo`Wo0000Goooool9o`00
01Cooooo000Cooooo`Wo0000Goooool9o`0001kooooo2Ol0001Oooooo`Wo00007_ooool9o`0005oo
oooo2Ol0000Dooooo`004oooool9o`0005oooooo2Ol0000Nooooo`Wo0000Goooool9o`0001kooooo
2Ol0001Oooooo`Wo00005?ooool001?ooooo2Ol0001Oooooo`Wo00007_ooool9o`0005oooooo2Ol0
000Nooooo`Wo0000Goooool9o`0001Cooooo000Dooooo`[o0000Foooool:o`00023ooooo2_l0001K
ooooo`GoZJVY1Ol0000Pooooo`[o0000Foooool:o`0001Gooooo000Eooooo`[o0000FOooool:o`00
02;ooooo2_l0001Iooooo`OoZJVY0ol0000Rooooo`[o0000FOooool:o`0001Kooooo000Gooooo`Wo
0000Eoooool9o`0002Kooooo2Ol0001Gooooo`WoZJVY9_ooool9o`0005Oooooo2Ol0000Hooooo`00
5oooool9o`0005Oooooo2Ol0000Vooooo`Wo0000Eoooool9ojVYZBKooooo2Ol0001Gooooo`Wo0000
6?ooool001Oooooo2Ol0001Gooooo`Wo00009_ooool9o`0005Oooooo2OnYZJTVooooo`Wo0000Eooo
ool9o`0001Sooooo000Gooooo`co0000DOooool5ojVYZ@Oo00009_ooool<o`00057ooooo3?nYZJTV
ooooo`co0000DOooool<o`0001Sooooo000Gooooo`go0000Coooool7ojVYZ@Ko00009_ooool=o`00
04oooooo3OnYZJTVooooo`go0000Coooool=o`0001Sooooo000Hooooo`go0000COooool9ojVYZ@Co
0000:?ooool=o`0004gooooo3OnYZJTXooooo`go0000COooool=o`0001Wooooo000Iooooo`co0000
COooool9ojVYZ@?o0000:_ooool<o`0004gooooo3?nYZJTZooooo`co0000COooool<o`0001[ooooo
000Looooo`Wo0000COooool9ojVYZC3ooooo2Ol0001=ooooo`WoZJVY<?ooool9o`0004gooooo2Ol0
000Mooooo`007?ooool=o`0004Kooooo3?nYZJT`ooooo`co0000Aoooool<ojVYZC3ooooo3?l00017
ooooo`GoZJVY1ol0000Mooooo`007?ooool>o`0004Cooooo3OnYZJT`ooooo`go0000AOooool=ojVY
ZC3ooooo3Ol00015ooooo`OoZJVY1_l0000Mooooo`007Oooool>o`0004;ooooo3OnYZJTbooooo`go
0000@oooool=ojVYZC;ooooo3Ol00013ooooo`WoZJVY1?l0000Nooooo`007_ooool=o`0004;ooooo
3?nYZJTdooooo`co0000@oooool<ojVYZCCooooo3?l00013ooooo`WoZJVY0ol0000Oooooo`008_oo
ool9o`0004;ooooo2OnYZJTjooooo`Wo0000@oooool9ojVYZC[ooooo2Ol00013ooooo`WoZJVY8_oo
ool002;ooooo3Ol0000jooooo`goZJVY>_ooool=o`0003_ooooo3OnYZJTjooooo`go0000>oooool=
ojVYZB;ooooo000Rooooo`ko0000>?ooool>ojVYZC[ooooo3_l0000iooooo`koZJVY>_ooool>o`00
03Wooooo3_nYZJTRooooo`008oooool>o`0003Kooooo3_nYZJTlooooo`ko0000=oooool>ojVYZCco
oooo3_l0000gooooo`koZJVY8oooool002Cooooo4Ol0000^oooooa7oZJVY?_oooolBo`0002gooooo
4_nYZJTnoooooa;o0000;OoooolBojVYZBCooooo000Xooooo`ko0000;?ooool>ojVYZDKooooo3ol0
000[ooooo`ooZJVYA_ooool?o`0002_ooooo3onYZJTXooooo`00:?ooool?o`0002[ooooo3onYZJU6
oooooa3o0000:Oooool@ojVYZDKooooo4?l0000Yoooooa3oZJVY:?ooool002Sooooo5?l0000Poooo
oaCoZJVYA_oooolEo`00023ooooo5?nYZJU6oooooaGo00007ooooolEojVYZBSooooo000YoooooaCo
00007_oooolDojVYZDSooooo5Ol0000NoooooaCoZJVYB?oooolEo`0001gooooo5OnYZJTYooooo`00
:_oooolIo`00017ooooo1OnYZJT00oooooooZJVYojVYZ@0BojVYZD[ooooo6_l0000Aooooo`GoZJVY
00?oooooojVYZOnYZJT04_nYZJU:oooooaGo00001OnYZJTAoooooa[oZJVY:_ooool002kooooo5_l0
0002ooooo`Go00000_ooool5ojVYZ@03oooooonYZJWoZJVY01GoZJVYDooooolFo`000003oooooonY
ZJWoZJVY00?oZJVY0oooool5ojVYZ@03oooooonYZJWoZJVY01CoZJVYE?ooool@o`0000KoZJVY00?o
ooooojVYZOnYZJT00onYZJT3ooooo`GoZJVY00?oooooojVYZOnYZJT05?nYZJT_ooooo`00;ooooolM
o`0001goZJVYEOoooolFo`0000KoZJVY00?oooooojVYZOnYZJT06_nYZJUFooooo`oo00003OnYZJT0
0oooooooZJVYojVYZ@0JojVYZC3ooooo000`oooooago00006onYZJUGoooooaGo00008_nYZJUHoooo
o`ko0000:OnYZJTaooooo`00=OoooolHo`0001KoZJVYHOooool@o`0001koZJVYHOooool9o`0002Co
ZJVY=_ooool003Kooooo5ol0000EojVYZF?ooooo3ol0000MojVYZF?ooooo1ol0000TojVYZCOooooo
000goooooaKo00003_nYZJT00oooooooZJVYojVYZ@03ojVYZFGooooo3_l0000FojVYZ@03oooooonY
ZJWoZJVY00?oZJVYIOooool5o`0002CoZJVY>?ooool003gooooo4?l0000=ojVYZG;ooooo1ol0000F
ojVYZG;ooooo7OnYZJTnooooo`00?_ooool5o`0000;ooooo1ol00007ojVYZ@03oooooonYZJWoZJVY
00?oZJVYM?ooool5o`000003oooooonYZJWoZJVY00GoZJVY00?oooooojVYZOnYZJT01OnYZJT00ooo
ooooZJVYojVYZ@03ojVYZGCooooo1OnYZJT00oooooooZJVYojVYZ@05ojVYZ@03oooooonYZJWoZJVY
00GoZJVY00?oooooojVYZOnYZJT00onYZJToooooo`00A_ooool5o`0000;ooooo1OnYZJV2ooooo`Go
ZJVY0oooool5ojVYZH;ooooo1OnYZJT3ooooo`GoZJVYA_ooool00?oooooo]oooool00?oooooo]ooo
ool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]ooo
ool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]ooo
ool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool004Kooooo1Ol0
0002ooooo`GoZJVYP_ooool5ojVYZ@?ooooo1OnYZJV2ooooo`GoZJVY0oooool5ojVYZDKooooo000n
ooooo`Go00000_ooool7o`0000OoZJVY00?oooooojVYZOnYZJT00onYZJUdooooo`Go000000?ooooo
ojVYZOnYZJT01OnYZJT00oooooooZJVYojVYZ@05ojVYZ@03oooooonYZJWoZJVY00?oZJVYM?ooool5
ojVYZ@03oooooonYZJWoZJVY00GoZJVY00?oooooojVYZOnYZJT01OnYZJT00oooooooZJVYojVYZ@03
ojVYZCoooooo000mooooo`ko00003onYZJUbooooo`Ko00005onYZJUboooooagoZJVY?_ooool003Oo
oooo5?l0000@ojVYZ@03oooooonYZJWoZJVY00?oZJVYIOooool<o`0001SoZJVY00?oooooo`000?l0
00000ol0001Uooooo`Go00009?nYZJThooooo`00=_oooolEo`0001OoZJVYHoooool=o`0001SoZJVY
1ol0001Sooooo`Ko00009OnYZJTgooooo`00<?oooolKo`0001goZJVYEooooolCo`0001OoZJVY3Ol0
001Hooooo`co00009_nYZJT5o`00037ooooo000_oooooaco00007_nYZJUEoooooaCo00005onYZJT>
o`0005Kooooo3Ol0000UojVYZ@Oo0000<?ooool002kooooo7_l0000NojVYZE?ooooo5_l00007ojVY
Z@03oooooonYZJWoZJVY00coZJVY3ol0001Dooooo`ko00003onYZJT00oooooooZJVYojVYZ@0BojVY
Z@Wo0000;oooool002kooooo5_l00002ooooo`Go00000_ooool5ojVYZ@03oooooonYZJWoZJVY01Go
ZJVYDooooolFo`000003oooooonYZJWoZJVY00?oZJVY0oooool5ojVYZ@03oooooonYZJWoZJVY00Go
ZJVY3ol0001Dooooo`oo00001onYZJT00oooooooZJVYojVYZ@03ojVYZ@?ooooo1OnYZJT00ooooooo
ZJVYojVYZ@0;ojVYZ@Wo0000;oooool002[ooooo6Ol0000Aooooo`GoZJVY00?oooooojVYZOnYZJT0
3OnYZJT5o`0004[ooooo6_l0000Aooooo`GoZJVY00?oooooo`000?l000004_l0001:ooooo`GoZJVY
4?l00005ojVYZA7ooooo3?nYZJT>o`0002[ooooo000YoooooaCo00007_ooool=ojVYZ@Oo0000B?oo
oolEo`0001kooooo5?l00018ooooo`KoZJVY3ol0000Mooooo`KoZJVY3ol0000Yooooo`00:?oooolD
o`00023ooooo2onYZJT9o`0004Kooooo5Ol0000PoooooaCo0000A_ooool7ojVYZ@ko00007oooool5
ojVYZA3o0000:?ooool002Sooooo3_l0000/ooooo`GoZJVY2Ol00016ooooo`oo0000:oooool?o`00
04Kooooo2?nYZJT7o`0002_ooooo3ol0000Xooooo`009?oooolAo`0002kooooo1?nYZJT8o`0000Go
ZJVY?_ooool4ojVYZ@ko0000;OoooolBo`0003kooooo3OnYZJT5o`0002gooooo4_l0000Tooooo`00
8oooool>o`0003Kooooo1ol00007ojVYZCcooooo1OnYZJT9o`0003Oooooo3_l0000looooo`koZJVY
=oooool>o`0002?ooooo000Rooooo`oo0000=_ooool6o`0000WoZJVY>_ooool6ojVYZ@Wo0000=ooo
ool?o`0003[ooooo3onYZJTgooooo`oo00008_ooool002;ooooo3_l0000hooooo`Go00002OnYZJTj
ooooo`OoZJVY1ol0000iooooo`ko0000>_ooool>ojVYZCWooooo3_l0000Rooooo`008_ooool=o`00
03[ooooo1?l00009ojVYZC[ooooo2?nYZJT5o`0003_ooooo3Ol0000jooooo`goZJVY>oooool=o`00
02;ooooo000Nooooo`CoZJVY2Ol00012ooooo`OoZJVY1Ol0000dooooo`coZJVY@oooool<o`0003Co
oooo3?nYZJU3ooooo`co00007oooool001gooooo1OnYZJT9o`0004;ooooo1_nYZJT7o`0003;ooooo
3OnYZJU3ooooo`go0000<_ooool=ojVYZD?ooooo3Ol0000Nooooo`007?ooool7ojVYZ@Oo0000A?oo
ool4ojVYZ@Wo0000<?ooool=ojVYZDGooooo3Ol0000`ooooo`goZJVYAOooool=o`0001gooooo000L
ooooo`SoZJVY1Ol00016ooooo`?oZJVY2Ol0000`ooooo`coZJVYAoooool<o`00033ooooo3?nYZJU7
ooooo`co00007Oooool001cooooo2OnYZJU=ooooo`Wo0000<?ooool9ojVYZDgooooo2Ol0000`oooo
o`WoZJVYCOooool9o`0001gooooo000Looooo`WoZJVYCOooool9o`00033ooooo2OnYZJU=ooooo`Wo
0000<?ooool9ojVYZDgooooo2Ol0000Mooooo`006Oooool<ojVYZDgooooo3?l0000Zooooo`coZJVY
COooool<o`0002[ooooo3?nYZJU=ooooo`Oo00001OnYZJTJooooo`006?ooool<ojVYZDoooooo3?l0
000Xooooo`coZJVYCoooool<o`0002Sooooo3?nYZJU?ooooo`Go00001onYZJTIooooo`005oooool<
ojVYZE7ooooo3?l0000Vooooo`coZJVYDOooool<o`0002Kooooo3?nYZJUAooooo`?o00002OnYZJTH
ooooo`005oooool9ojVYZEOooooo2Ol0000Vooooo`WoZJVYEoooool9o`0002Kooooo2OnYZJUGoooo
o`WoZJVY6?ooool001Oooooo2OnYZJUGooooo`Wo00009_ooool9ojVYZEOooooo2Ol0000Vooooo`Wo
ZJVYEoooool9ojVYZASooooo000Gooooo`WoZJVYEoooool9o`0002Kooooo2OnYZJUGooooo`Wo0000
9_ooool9ojVYZEOooooo2OnYZJTHooooo`005Oooool;ojVYZEOooooo2ol0000Rooooo`_oZJVYEooo
ool6o`0000GoZJVY8_ooool;ojVYZEOooooo2onYZJTFooooo`005?ooool;ojVYZEWooooo2ol0000P
ooooo`_oZJVYFOooool4o`0000OoZJVY8?ooool;ojVYZEWooooo2onYZJTEooooo`004oooool;ojVY
ZE_ooooo2ol0000Nooooo`_oZJVYFoooool2o`0000WoZJVY7_ooool;ojVYZE_ooooo2onYZJTDoooo
o`004oooool9ojVYZEoooooo2Ol0000Nooooo`WoZJVYGoooool9ojVYZAkooooo2OnYZJUOooooo`Wo
ZJVY5?ooool001?ooooo2OnYZJUOooooo`Wo00007_ooool9ojVYZEoooooo2OnYZJTNooooo`WoZJVY
Goooool9ojVYZACooooo000Cooooo`WoZJVYGoooool9o`0001kooooo2OnYZJUOooooo`WoZJVY7_oo
ool9ojVYZEoooooo2OnYZJTDooooo`004oooool9ojVYZEoooooo2Ol0000Nooooo`WoZJVYGoooool9
ojVYZAkooooo2OnYZJUOooooo`WoZJVY5?ooool001;ooooo0_l00007ojVYZF7ooooo1?l00005ojVY
ZAcooooo2OnYZJUQooooo`WoZJVY7?ooool9ojVYZF7ooooo2OnYZJTCooooo`004Oooool4o`0000Go
ZJVYHoooool2o`0000OoZJVY6_ooool9ojVYZF?ooooo2OnYZJTJooooo`WoZJVYHoooool9ojVYZA;o
oooo000@ooooo`Wo0000IOooool9ojVYZASooooo2OnYZJUUooooo`WoZJVY6?ooool9ojVYZFGooooo
2OnYZJTAooooo`004?ooool9o`0006Gooooo2OnYZJTHooooo`WoZJVYIOooool9ojVYZASooooo2OnY
ZJUUooooo`WoZJVY4Oooool0013ooooo2Ol0001Uooooo`WoZJVY6?ooool9ojVYZFGooooo2OnYZJTH
ooooo`WoZJVYIOooool9ojVYZA7ooooo000@ooooo`Wo0000IOooool9ojVYZASooooo2OnYZJUUoooo
o`WoZJVY6?ooool9ojVYZFGooooo2OnYZJTAooooo`004?ooool9o`0006Gooooo2OnYZJTHooooo`Wo
ZJVYIOooool9ojVYZASooooo2OnYZJUUooooo`WoZJVY4Oooool0013ooooo2?l0001Wooooo`WoZJVY
5oooool00ol0003oZJVYojVYZ@05ojVYZFOooooo2OnYZJTGooooo`SoZJVYIoooool8ojVYZA7ooooo
000?ooooo`So0000JOooool9ojVYZAGooooo0ol00005ojVYZFWooooo2OnYZJTEooooo`SoZJVYJOoo
ool8ojVYZA3ooooo000>ooooo`Wo0000J_ooool9ojVYZA?ooooo2Ol0001Zooooo`WoZJVY4oooool9
ojVYZFWooooo2OnYZJT?ooooo`003_ooool9o`0006[ooooo2OnYZJTCooooo`Wo0000J_ooool9ojVY
ZA?ooooo2OnYZJUYooooo`WoZJVY3oooool000kooooo2Ol0001Zooooo`WoZJVY4oooool9o`0006[o
oooo2OnYZJTCooooo`WoZJVYJOooool9ojVYZ@oooooo000>ooooo`Wo0000J_ooool9ojVYZA?ooooo
2Ol0001Zooooo`WoZJVY4oooool9ojVYZFWooooo2OnYZJT?ooooo`003_ooool9o`0006Wooooo00?o
003oojVYZOnYZJT01onYZJTCooooo`Wo0000JOooool00ol00?ooZJVYojVYZ@07ojVYZA?ooooo2OnY
ZJUXooooo`03o`00oonYZJWoZJVY00OoZJVY3oooool000oooooo1ol0001Wooooo`Co003o00?ooooo
ojVYZOnYZJT01OnYZJTDooooo`So0000Ioooool4o`00o`03oooooonYZJWoZJVY00GoZJVY5?ooool0
0ol0003oZJVYojVYZ@05ojVYZFOooooo0ol00?l00oooooooZJVYojVYZ@06ojVYZ@oooooo000>oooo
o`Oo0000I_ooool2o`00o`03ooooool00?oo003o00?ooooo1onYZJTBooooo`So0000I_ooool2o`00
o`03ooooool00?oo003o00?ooooo1onYZJTBooooo`?o00001OnYZJUVooooo`Co003o0oooool8ojVY
Z@kooooo000=ooooo`Wo0000Hoooool2o`00o`03ooooool00?oo003o00Cooooo2OnYZJT@ooooo`Wo
0000I?ooool2o`00o`03ooooool00?oo003o00Cooooo2OnYZJT@ooooo`Wo0000I?ooool2o`00o`03
ooooool00?oooooo00Cooooo2OnYZJT=ooooo`003Oooool9o`00067ooooo0_l00?l2ooooo`03o`00
oooooooooooo00Cooooo2OnYZJT@ooooo`Wo0000H_ooool2o`00o`;ooooo00?o003oooooooooool0
1?ooool9ojVYZA3ooooo2Ol0001Rooooo`;o003o0_ooool00ol00?ooooooooooo`04ooooo`WoZJVY
3Oooool000gooooo2Ol0001Oooooo`;o003o0_ooool2o`00o`Oooooo2OnYZJT@ooooo`Wo0000H?oo
ool2o`00o`;ooooo0_l00?l7ooooo`WoZJVY4?ooool9o`00063ooooo0_l00?l2ooooo`;o003o1ooo
ool9ojVYZ@gooooo000=ooooo`Wo0000G_ooool01Ol00?oooooooooooooooooo003o00Oooooo0_l0
0009ojVYZA3ooooo2Ol0001Oooooo`05o`00ooooooooooooooooool00?l01oooool2o`0000WoZJVY
4?ooool9o`0005oooooo00Go003oooooooooooooooooo`00o`07ooooo`;o00002OnYZJT=ooooo`00
3Oooool9o`0005cooooo0_l00?l2ooooo`;o003o1Oooool2o`0000?o003o2OnYZJT@ooooo`Wo0000
GOooool2o`00o`;ooooo0_l00?l5ooooo`;o00000ol00?l9ojVYZA3ooooo2Ol0001Mooooo`;o003o
0_ooool2o`00o`Cooooo0ol00003o`00o`WoZJVY3Oooool000kooooo1ol0001Looooo`03o`00oooo
oooooooo00;o003o0oooool3o`0000Co003o0oooool7ojVYZA;ooooo1ol0001Mooooo`03o`00oooo
oooooooo00;o003o0oooool3o`0000Co003o0oooool7ojVYZA;ooooo1ol0001Mooooo`03o`00oooo
oooooooo00;o003o0_ooool4o`0000Co003o0oooool7ojVYZ@kooooo000>ooooo`Ko0000Foooool2
o`00o`03ooooool00?oo003o00;ooooo0ol00004o`00o`Oooooo1OnYZJTDooooo`Go0000G?ooool2
o`00o`03ooooool00?oo003o00;ooooo0ol00004o`00o`Oooooo1OnYZJTDooooo`Go0000G?ooool2
o`00o`04ooooool00?oo003oooooo`?o000000?oooooo`00ool00?l00_l00?l7ooooo`GoZJVY3ooo
ool000gooooo1ol0001Booooo`Go00000_ooool5o`00o`03ooooool0003o000000Ko003o2oooool2
o`00oaCooooo1ol0001Booooo`GoZJVY0_ooool5o`00o`03ooooool0003o000000Ko003o2_ooool2
o`00oaGooooo1ol0001Booooo`GoZJVY0_ooool5o`00o`?o00001_l00?l:ooooo`;o003o4Oooool0
00cooooo2Ol0001@ooooo`Oo00000ol00?l00ooooooo0000o`00o`06o`00o`kooooo0_l00?lCoooo
o`Wo0000D?ooool7ojVYZ@?o003o00?oooooo`000?l00?l01_l00?l=ooooo`;o003o5?ooool9o`00
053ooooo1onYZJT3o`00o`;o00001ol00?l=ooooo`;o003o4Oooool000cooooo2Ol0001?ooooo`[o
00001ol00?lAooooo`;o003o4oooool9o`0004oooooo2OnYZJT00ol0003o003oo`00o`05o`00oa3o
oooo0_l00?lDooooo`Wo0000Coooool9ojVYZ@03o`000?l00?oo003o00Go003o4?ooool2o`00oa7o
oooo000<ooooo`Wo0000Coooool9o`0000?o003o5_ooool2o`00oa?ooooo2Ol0001?ooooo`WoZJVY
0ol00?lEooooo`;o003o5?ooool9o`0004oooooo2OnYZJT3o`00oaGooooo0_l00?lAooooo`003?oo
ool9o`0004oooooo2Ol0000Iooooo`;o003o4oooool9o`0004oooooo2OnYZJTHooooo`;o003o5?oo
ool9o`0004oooooo2OnYZJTHooooo`;o003o4Oooool000cooooo2Ol0001?ooooo`Wo00001ol00?lB
ooooo`;o003o4oooool9o`0004oooooo2OnYZJT7o`00oa7ooooo0_l00?lDooooo`Wo0000Coooool9
ojVYZ@Oo003o4Oooool2o`00oa7ooooo000=ooooo`Oo0000D?ooool9o`0000;o003o0_l00009o`00
o`[ooooo1Ol0000Cooooo`Oo0000D?ooool9ojVYZ@;o003o0_l00009o`00o`[ooooo1Ol0000Coooo
o`Oo0000D?ooool9ojVYZ@;o003o0_l00009o`00o`[ooooo1OnYZJT?ooooo`003_ooool5o`0000;o
ZJVYD?ooool7o`000003ooooool00?oo003o00?o003o1?l00008o`00o`Gooooo1ol0000Booooo`Oo
0000DOooool7ojVYZ@03ooooool00?oo003o00?o003o1?l00008o`00o`Gooooo1ol0000Booooo`Oo
0000DOooool7ojVYZ@03ooooool00?oo003o00;o003o00?oooooo`000?l000000_l00008o`00o`Go
oooo1onYZJT>ooooo`003Oooool9ojVYZE3ooooo1Ol00004ooooo`;o003o00?oooooo`00ool00?l0
0_ooool5o`0000Ko003o00?oooooo`000?l000001ol0000@ooooo`Wo0000DOooool5ojVYZ@Cooooo
0_l00?l00ooooooo003oo`00o`02ooooo`Go00001_l00?l00ooooooo0000o`000007o`00013ooooo
2Ol0001Aooooo`GoZJVY1?ooool5o`00o`;ooooo1Ol00006o`00o`03oooooonYZJWoZJVY00OoZJVY
3Oooool000gooooo2OnYZJUKooooo`;o003o00Coooooo`00ool00?oo003o1Oooool4o`0000;o003o
2Ol0000@ooooo`Wo0000G?ooool2o`00o`04ooooool00?oo003oo`00o`Gooooo1?l00002o`00o`Wo
00004?ooool9o`0005cooooo0_l00?l00ooooooo003oo`00o`06ooooo`Co00000_l00?l9ojVYZ@go
oooo000=ooooo`WoZJVYGOooool01?l00?ooooooooooooooool2o`00o`Wooooo2Ol0000@ooooo`Wo
0000G_ooool01?l00?ooooooooooooooool2o`00o`Wooooo2Ol0000@ooooo`Wo0000G_ooool00ol0
0?ooooooooooo`02o`00o`[ooooo2OnYZJT=ooooo`003Oooool9ojVYZEkooooo0_l00?l3ooooo`;o
003o1oooool9o`00013ooooo2Ol0001Oooooo`;o003o0oooool2o`00o`Oooooo2Ol0000@ooooo`Wo
0000Goooool2o`00o`;ooooo0_l00?l8ooooo`WoZJVY3Oooool000gooooo2OnYZJUPooooo`?o003o
0_ooool2o`00o`Gooooo2Ol0000@ooooo`Wo0000HOooool3o`00o`;ooooo0_l00?l5ooooo`Wo0000
4?ooool9o`00067ooooo0_l00?l2ooooo`;o003o1_ooool9ojVYZ@gooooo000>ooooo`OoZJVYI?oo
ool3o`00o`03ooooool00?oo003o00Cooooo1ol0000Booooo`So0000I?ooool3o`00o`03ooooool0
0?oo003o00Cooooo1ol0000Booooo`So0000Hoooool3o`00o`03ooooool00?oo003o00Cooooo1Ol0
0003ojVYZ@kooooo000?ooooo`GoZJVY0_l0001Vooooo`Go003o00?oooooo`000?l000001Ol0000D
ooooo`So0000I_ooool5o`00o`03ooooool0003o000000Go00005?ooool8o`0006Gooooo1Ol00?l0
0ooooooo0000o`000005o`000003ojVYZOoooooooooo00gooooo000>ooooo`Wo0000J?ooool2o`00
o`Wo00004oooool9o`0006Sooooo0_l00?l9o`0001?ooooo2Ol0001Wooooo`;o003o2Ol0000?oooo
o`003_ooool9o`0006[ooooo2Ol0000Cooooo`Wo0000J_ooool9o`0001?ooooo2Ol0001Yooooo`Wo
00003oooool000kooooo2Ol0001Zooooo`Wo00004oooool9o`0006[ooooo2Ol0000Cooooo`Wo0000
JOooool9o`0000oooooo000>ooooo`Wo0000J_ooool9o`0001?ooooo2Ol0001Zooooo`Wo00004ooo
ool9o`0006Wooooo2Ol0000?ooooo`003_ooool9o`0006[ooooo2Ol0000Cooooo`Wo0000J_ooool9
o`0001?ooooo2Ol0001Yooooo`Wo00003oooool000oooooo1ol0001/ooooo`Oo00005Oooool7o`00
06cooooo1ol0000Eooooo`Oo0000Joooool7o`00013ooooo000@ooooo`Oo0000JOooool8o`0001Oo
oooo1ol0001Yooooo`So00005oooool7o`0006Wooooo1ol0000Aooooo`004Oooool7o`0006Oooooo
1ol0000Jooooo`Oo0000Ioooool7o`0001[ooooo1ol0001Wooooo`Oo00004_ooool0013ooooo2Ol0
001Uooooo`Wo00006?ooool9o`0006Gooooo2Ol0000Hooooo`Wo0000IOooool9o`00017ooooo000@
ooooo`Wo0000IOooool9o`0001Sooooo2Ol0001Uooooo`Wo00006?ooool9o`0006Gooooo2Ol0000A
ooooo`004?ooool9o`0006Gooooo2Ol0000Hooooo`Wo0000IOooool9o`0001Sooooo2Ol0001Uoooo
o`Wo00004Oooool0013ooooo2Ol0001Uooooo`Wo00006?ooool9o`0006Gooooo2Ol0000Hooooo`Wo
0000IOooool9o`00017ooooo000@ooooo`Wo000000?oZJVYooooooooool0HOooool:o`0001Sooooo
2_l0001Sooooo`[o00006?ooool:o`0006?ooooo2_l0000Aooooo`004Oooool7o`0000?oZJVYHOoo
ool:o`0001[ooooo2_l0001Qooooo`[o00006_ooool:o`00067ooooo2_l0000Booooo`004_ooool5
o`0000GoZJVYGoooool:o`0001cooooo2_l0001Oooooo`[o00007?ooool:o`0005oooooo2_l0000C
ooooo`004oooool9ojVYZEoooooo2Ol0000Nooooo`Wo0000Goooool9o`0001kooooo2Ol0001Ooooo
o`Wo00005?ooool001?ooooo2OnYZJUOooooo`Wo00007_ooool9o`0005oooooo2Ol0000Nooooo`Wo
0000Goooool9o`0001Cooooo000Cooooo`WoZJVYGoooool9o`0001kooooo2Ol0001Oooooo`Wo0000
7_ooool9o`0005oooooo2Ol0000Dooooo`004oooool9ojVYZEoooooo2Ol0000Nooooo`Wo0000Gooo
ool9o`0001kooooo2Ol0001Oooooo`Wo00005?ooool001Cooooo2_nYZJUKooooo`GoZJVY1Ol0000P
ooooo`[o0000Foooool:o`00023ooooo2_l0001Kooooo`[o00005Oooool001Gooooo2_nYZJUIoooo
o`OoZJVY0ol0000Rooooo`[o0000FOooool:o`0002;ooooo2_l0001Iooooo`[o00005_ooool001Oo
oooo2OnYZJUGooooo`WoZJVY9_ooool9o`0005Oooooo2Ol0000Vooooo`Wo0000Eoooool9o`0001So
oooo000Gooooo`WoZJVYEoooool9ojVYZBKooooo2Ol0001Gooooo`Wo00009_ooool9o`0005Oooooo
2Ol0000Hooooo`005oooool9ojVYZEOooooo2OnYZJTVooooo`Wo0000Eoooool9o`0002Kooooo2Ol0
001Gooooo`Wo00006?ooool001Oooooo2OnYZJT3o`00057ooooo1Ol00007ojVYZBKooooo3?l0001A
ooooo`co00009_ooool<o`00057ooooo3?l0000Hooooo`005oooool9ojVYZ@Co0000Coooool7o`00
00KoZJVY9_ooool=o`0004oooooo3Ol0000Vooooo`go0000Coooool=o`0001Sooooo000Hooooo`Oo
ZJVY1_l0001=ooooo`Wo00001?nYZJTXooooo`go0000COooool=o`0002Sooooo3Ol0001=ooooo`go
00006Oooool001Wooooo1OnYZJT7o`0004gooooo2Ol00003ojVYZB[ooooo3?l0001=ooooo`co0000
:_ooool<o`0004gooooo3?l0000Jooooo`007?ooool9o`0004gooooo2Ol0000`ooooo`Wo0000COoo
ool9o`00033ooooo2Ol0001=ooooo`Wo00007Oooool001cooooo3Ol00016ooooo`co0000<?ooool<
o`0004Oooooo3?l0000`ooooo`co0000Aoooool5ojVYZ@Oo00007Oooool001cooooo3_l00014oooo
o`go0000<?ooool=o`0004Gooooo3Ol0000`ooooo`go0000AOooool7ojVYZ@Ko00007Oooool001go
oooo3_l00012ooooo`go0000<_ooool=o`0004?ooooo3Ol0000booooo`go0000@oooool9ojVYZ@Co
00007_ooool001kooooo3Ol00012ooooo`co0000=?ooool<o`0004?ooooo3?l0000dooooo`co0000
@oooool9ojVYZ@?o00007oooool002;ooooo2Ol00012ooooo`Wo0000>_ooool9o`0004?ooooo2Ol0
000jooooo`Wo0000@oooool9ojVYZB;ooooo000Rooooo`Wo00001?nYZJTjooooo`go0000>_ooool=
o`0003_ooooo1OnYZJT8o`0003[ooooo3Ol0000kooooo`goZJVY8_ooool002;ooooo2Ol00005ojVY
ZCSooooo3_l0000jooooo`ko0000>Oooool7ojVYZ@Oo0000>_ooool>o`0003Wooooo3_nYZJTRoooo
o`008oooool7o`0000OoZJVY=_ooool>o`0003cooooo3_l0000gooooo`WoZJVY1Ol0000looooo`ko
0000=oooool>ojVYZB?ooooo000Tooooo`Go00002?nYZJT4o`0002kooooo1OnYZJT<o`0003kooooo
4_l0000]ooooo`koZJVY1?l0000noooooa;o0000;OoooolBojVYZBCooooo000Xooooo`WoZJVY1Ol0
000/ooooo`OoZJVY1ol00016ooooo`oo0000:oooool?ojVYZDKooooo3ol0000[ooooo`ooZJVY:?oo
ool002Sooooo2OnYZJT6o`0002[ooooo2OnYZJT6o`0004Kooooo4?l0000Yoooooa3oZJVYA_ooool@
o`0002Wooooo4?nYZJTXooooo`00:?ooool9ojVYZ@_o00008?ooool>ojVYZ@Ko0000A_oooolEo`00
023ooooo5?nYZJU6oooooaGo00007ooooolEojVYZBSooooo000Yooooo`OoZJVY3Ol0000Nooooo`oo
ZJVY1Ol00018oooooaGo00007_oooolDojVYZDSooooo5Ol0000MoooooaGoZJVY:Oooool002[ooooo
1OnYZJT?o`0000GoZJVY4Oooool5ojVYZ@03oooooonYZJWoZJVY00koZJVY1?l0001:oooooa[o0000
4Oooool5o`000003oooooonYZJWoZJVY01;oZJVYB_oooolJo`00017ooooo6_nYZJTZooooo`00;_oo
ool@o`0000KoZJVY0_ooool5o`0000;ooooo1OnYZJT00oooooooZJVYojVYZ@0EojVYZE?ooooo5_l0
00000ooooooo0000o`000003o`0000?ooooo1Ol000000ooooooo0000o`000005o`0000ooZJVYE?oo
oolFo`000003ooooool0003o000000?o00000oooool5o`000003oooooonYZJWoZJVY01CoZJVY;ooo
ool002oooooo3ol00007ojVYZ@Oo00007OnYZJUEoooooaco000000?oooooo`000?l000003Ol0000=
ojVYZEKooooo7?l000000ooooooo0000o`000005o`0001GoZJVY<?ooool0033ooooo3_l00007ojVY
Z@So00006onYZJUGooooob_o00003?nYZJUHooooobCo00004onYZJTaooooo`00=Oooool9o`0000Oo
ZJVY2?l0000FojVYZF7ooooo9_l00008ojVYZF7ooooo7ol0000>ojVYZCKooooo000fooooo`Oo0000
2?nYZJT8o`0001GoZJVYHooooolUo`0000OoZJVYHooooolNo`0000goZJVY=oooool003Oooooo1Ol0
0009ojVYZ@So00003_nYZJT00oooooooZJVYojVYZ@03ojVYZFGooooo9?l000000oooooooZJVYojVY
Z@03ojVYZFGooooo7Ol0000<ojVYZCSooooo000mooooo`OoZJVY2Ol0000=ojVYZG;ooooo7Ol0001b
oooooaOo00001_nYZJTnooooo`00?_ooool5ojVYZ@;ooooo1ol00007ojVYZ@03oooooonYZJWoZJVY
00?oZJVYM?ooool5o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol000000ooo
oooo0000o`000003o`0007Cooooo1Ol000000ooooooo0000o`000005o`000003ooooool0003o0000
00Go000000?oooooojVYZOnYZJT00onYZJToooooo`00A_ooool5o`0000;ooooo1OnYZJV2ooooo`Go
00000oooool5o`0008;ooooo1Ol00003ooooo`Go0000A_ooool00?oooooo]oooool00?oooooo]ooo
ool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]oooool00?oooooo]ooo
ool00?oooooo]oooool00001\
\>"],
  ImageRangeCache->{{{0, 437}, {286.438, 0}} -> {-0.0769059, -0.0492291, \
0.00721697, 0.00721697}, {{10.625, 140.5}, {279.562, 149.688}} -> {-0.139714, \
-0.106676, 0.00844319, 0.00845837}, {{153.5, 283.438}, {279.562, 149.688}} -> \
{-1.3463, -0.106676, 0.00844319, 0.00845837}, {{296.438, 426.312}, {279.562, \
149.688}} -> {-2.55289, -0.106676, 0.00844319, 0.00845837}, {{10.625, 140.5}, \
{136.688, 6.8125}} -> {-0.139714, -1.31517, 0.00844319, 0.00845837}, {{153.5, \
283.438}, {136.688, 6.8125}} -> {-1.3463, -1.31517, 0.00844319, 0.00845837}, \
{{296.438, 426.312}, {136.688, 6.8125}} -> {-2.55289, -1.31517, 0.00844319, \
0.00845837}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Cooperation spreads!", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[pd1]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Approximations of small-world networks", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Watts (1999) defines a small-world network as one in which the mean \
characteristic path length is low even though the graph is sparse. We can \
approximate a small-world network by taking a cycle as the main graph and \
then adding a handful of \"bridge\" edges which serve to radically lower the \
mean characteristic path length.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g = Cycle[200];\)\), "\[IndentingNewLine]", 
    \(\(e = {{5, 98}, {41, 160}, {106, 150}, {82, 172}, {45, 
            118}};\)\), "\[IndentingNewLine]", 
    \(\(g = AddEdges[g, e];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[g, VertexStyle \[Rule] Disk[Small]];\)\)}], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.04635 .52854 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.64046 .0677 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.11621 .25644 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.78974 .14977 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.5 .04545 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{278, 278},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F00015R000`400?l00000o`00003ooooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00POooool3o`000005ooooool0003o0000o`000?ooool00ol000001Ooo
oooo0000o`000?l0003ooooo00?o0000P_ooool007Wooooo0ol000000ooooooo0000o`00000Io`00
0004ooooool0003o0000o`0007[ooooo001aooooo`?o000000?oooooo`000?l000002Ol000001Ooo
oooo0000o`000?l0003ooooo00Oo000000Goooooo`000?l0003o0000ooooo`03o`000003ooooool0
003o000000co0000Loooool006gooooo0ol000000ooooooo0000o`000005o`000005ooooool0003o
0000o`000?ooool00ol00007ooooo`03o`000?oooooooooo00_ooooo0ol000001Ooooooo0000o`00
0?l0003ooooo00[o0000Koooool006[ooooo2_l000001?oooooo0000o`000?l0000>ooooo`03o`00
0?oooooooooo01Cooooo1_l000000ooooooo0000o`000005o`0006_ooooo001Vooooo`[o00005Ooo
ool00ol0003oooooooooo`0Looooo`?o000000?oooooo`000?l000001Ol0001Wooooo`00H_ooool7
o`000004ooooool0003o0000o`0001Kooooo0_l0000Sooooo`?o000000?oooooo`000?l000001Ol0
001Sooooo`00H_ooool3o`000004ooooool0003o0000o`0001Wooooo00?o0000ooooooooool09ooo
ool3o`000004ooooool0003o0000o`0006?ooooo001Nooooo`Oo00007?ooool00ol0003ooooooooo
o`0/ooooo`Oo0000Goooool005_ooooo1_l0000Oooooo`03o`000?oooooooooo02kooooo00?o0000
ooooooooool01_l0001Looooo`00F_ooool7o`0001gooooo0_l0000aooooo`03o`000?oooooooooo
00Oo0000Foooool005Oooooo0ol000001?oooooo0000o`000?l0000Oooooo`03o`000?oooooooooo
037ooooo00?o0000ooooooooool00oooool3o`000004ooooool0003o0000o`0005Sooooo001Coooo
o`Oo00008_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Oooooo1_l0001Eoooo
o`00Doooool3o`000004ooooool0003o0000o`00027ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`07ooooo`Oo0000E?ooool0053ooooo1_l0000Tooooo`03o`000?oooooooooo03Co
oooo00?o0000ooooooooool02_ooool3o`000004ooooool0003o0000o`00057ooooo001@ooooo`?o
00009Oooool2o`0003Oooooo00?o0000ooooooooool03_ooool3o`00057ooooo001<ooooo`Oo0000
9?ooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo00kooooo1ol0001=ooooo`00C?oo
ool3o`0002Oooooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`0Booooo`?o0000COoo
ool004Wooooo1_l0000Vooooo`03o`000?oooooooooo03Wooooo00?o0000ooooooooool04_ooool6
o`0004[ooooo0019ooooo`?o0000:?ooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo
01Gooooo0ol0001:ooooo`00AOooool7o`0002Kooooo0_l0000mooooo`03o`000?oooooooooo01Go
oooo1_l00017ooooo`00AOooool3o`0002Wooooo00?o0000ooooooooool0?Oooool00ol0003ooooo
ooooo`0Hooooo`?o0000Aoooool004;ooooo1_l0000Xooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool06?ooool7o`0004?ooooo0012ooooo`?o0000:_ooool00ol0003oooooooooo`0o
ooooo`03o`000?oooooooooo01cooooo0ol00013ooooo`00?oooool6o`0002Sooooo0_l00012oooo
o`03o`000?oooooooooo01cooooo1_l00010ooooo`00?oooool3o`0002[ooooo00?o0000oooooooo
ool0@_ooool00ol0003oooooooooo`0Oooooo`?o0000@?ooool003oooooo0ol0000Yooooo`03o`00
0?oooooooooo04?ooooo00?o0000ooooooooool07oooool3o`00043ooooo000looooo`?o0000:ooo
ool00ol0003oooooooooo`14ooooo`03o`000?oooooooooo02;ooooo0ol0000mooooo`00??ooool3
o`0002[ooooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`0Rooooo`?o0000?Oooool0
03Wooooo1_l0000Xooooo`;o0000B?ooool00ol0003oooooooooo`0Rooooo`Ko0000>_ooool003Wo
oooo0ol0000Zooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool09Oooool3o`0003[o
oooo000iooooo`?o0000:Oooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo02Cooooo
1?l0000jooooo`00=_ooool3o`0002_ooooo00?o0000ooooooooool0B_ooool00ol0003ooooooooo
o`0Sooooo`03o`000?oooooooooo00;ooooo0ol0000gooooo`00=_ooool3o`0002Wooooo0_l0001=
ooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool00oooool3o`0003Oooooo000coooo
o`Ko0000:?ooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo027ooooo00?o0000oooo
ooooool01?ooool6o`0003Cooooo000cooooo`?o0000:_ooool00ol0003oooooooooo`1>ooooo`03
o`000?oooooooooo023ooooo00?o0000ooooooooool02?ooool3o`0003Cooooo000cooooo`?o0000
:Oooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0
2Oooool3o`0003Cooooo000`ooooo`?o0000:oooool00ol0003oooooooooo`1@ooooo`03o`000?oo
oooooooo01gooooo0_l0000?ooooo`?o0000<Oooool0033ooooo0ol0000Yooooo`;o0000D_ooool0
0ol0003oooooooooo`0Mooooo`03o`000?oooooooooo00oooooo0ol0000aooooo`00<?ooool3o`00
02Sooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
013ooooo0ol0000aooooo`00;Oooool3o`0002[ooooo00?o0000ooooooooool0Doooool00ol0003o
ooooooooo`0Kooooo`03o`000?oooooooooo01Cooooo0ol0000^ooooo`00;Oooool3o`0002Wooooo
00?o0000ooooooooool0E?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Gooooo
0ol0000^ooooo`00;Oooool3o`0002Sooooo00?o0000ooooooooool0EOooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01Kooooo0ol0000^ooooo`00:_ooool4o`0002Sooooo0_l0001H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool3o`0002cooooo000Zoooo
o`?o0000:?ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooooooo01Oooooo00?o0000oooo
ooooool06_ooool3o`0002cooooo000Zooooo`?o00009oooool00ol0003oooooooooo`1Iooooo`03
o`000?oooooooooo01Gooooo0_l0000Mooooo`?o0000;?ooool002Sooooo0ol0000Xooooo`03o`00
0?oooooooooo05[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Pooooo`?o0000
:Oooool002Sooooo0ol0000Vooooo`;o0000GOooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo027ooooo0ol0000Yooooo`00:?ooool3o`0002Gooooo00?o0000ooooooooool0GOooool0
0ol0003oooooooooo`0Booooo`03o`000?oooooooooo02;ooooo0ol0000Yooooo`009Oooool4o`00
02Kooooo00?o0000ooooooooool0G_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
02Gooooo0ol0000Wooooo`009Oooool3o`0002Kooooo00?o0000ooooooooool0Goooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo02Kooooo0ol0000Wooooo`009Oooool3o`0002Gooooo
00?o0000ooooooooool0H?ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo02Oooooo
0ol0000Wooooo`008oooool3o`0002Gooooo0_l0001Sooooo`03o`000?oooooooooo00gooooo0_l0
000]ooooo`?o00009?ooool002?ooooo0ol0000Tooooo`03o`000?oooooooooo06?ooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`0]ooooo`?o00009?ooool002?ooooo0ol0000Soooo
o`03o`000?oooooooooo06Cooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0^oooo
o`?o00009?ooool0027ooooo0ol0000Tooooo`03o`000?oooooooooo06Gooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0aooooo`?o00008_ooool0027ooooo0ol0000Sooooo`03o`00
0?oooooooooo06Kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0booooo`?o0000
8_ooool0027ooooo0ol0000Qooooo`;o0000JOooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo03?ooooo0ol0000Rooooo`008Oooool00ol0003oooooooooo`0Pooooo`03o`000?oooooo
oooo06Wooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0fooooo`03o`000?oooooo
oooo023ooooo000Oooooo`?o00008Oooool00ol0003oooooooooo`1Zooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool0=oooool3o`00023ooooo000Oooooo`?o00008?ooool00ol0003o
ooooooooo`1[ooooo`03o`000?oooooooooo00Cooooo0_l0000jooooo`?o00008?ooool001oooooo
0ol0000Nooooo`;o0000K_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo03[ooooo
0ol0000Pooooo`007Oooool3o`000003ooooool0003ooooo01cooooo00?o0000ooooooooool0K_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo03gooooo0ol0000Nooooo`007Oooool3
o`0000;ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1_ooooo`05o`000?oooooo
ooooooooool00000@?ooool3o`0001kooooo000Mooooo`?o00000_ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo073ooooo00Co0000oooooooooooo0000@Oooool3o`0001kooooo000K
ooooo`?o00001Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo077ooooo00?o0000
ooooool00000A?ooool3o`0001cooooo000Kooooo`?o00001_ooool00ol0003oooooooooo`0Coooo
o`;o0000M?ooool2o`0004Gooooo0ol0000Looooo`006oooool3o`0000Kooooo00?o0000oooooooo
ool04_ooool00ol0003oooooooooo`1cooooo`;o0000A_ooool3o`0001cooooo000Kooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1cooooo`03o`00
0?oooooo000004Sooooo00?o0000ooooooooool06_ooool001Wooooo0ol0000:ooooo`03o`000?oo
oooooooo00kooooo00?o0000ooooooooool0Loooool01?l0003oooooooooool00018ooooo`?o0000
6_ooool001Wooooo0ol0000:ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0Looo
ool01Ol0003ooooooooooooooooo000004Sooooo0ol0000Jooooo`006Oooool3o`0000_ooooo00?o
0000ooooooooool02_ooool2o`0007Gooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`16ooooo`?o00006_ooool001Wooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo07Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`18
ooooo`03o`000?oooooooooo01Sooooo000Gooooo`?o00003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo07Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`18oooo
o`?o00006?ooool001Oooooo0ol0000?ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool0M?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo04Sooooo0ol0000Hooooo`00
5oooool3o`00013ooooo00?o0000ooooooooool00_ooool2o`0007Gooooo0_l00008ooooo`03o`00
0?oooooooooo04Sooooo0ol0000Hooooo`005Oooool3o`0001;ooooo00Go0000oooooooooooooooo
o`00001fooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0BOooool3o`0001Oooooo
000Eooooo`?o00004oooool00ol0003oooooo`00001fooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool0BOooool3o`0001Oooooo000Eooooo`?o00005?ooool00ol0003oooooooooo`1d
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0BOooool3o`0001Oooooo000Foooo
o`03o`000?oooooooooo01;ooooo0_l0001eooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool0Boooool00ol0003oooooooooo`0Eooooo`005?ooool3o`0001;ooooo0_l00002ooooo`03
o`000?oooooooooo077ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1;ooooo`?o
00005Oooool001Cooooo0ol0000Aooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
Koooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo04_ooooo0ol0000Eooooo`005?oo
ool3o`00013ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`1]ooooo`;o00004?oo
ool00ol0003oooooooooo`1;ooooo`?o00005Oooool001;ooooo0ol0000Aooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0Joooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo04cooooo0ol0000Dooooo`004_ooool3o`0000oooooo0_l0000:ooooo`03o`000?oooooooooo
06Wooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1<ooooo`?o00005?ooool001;o
oooo0ol0000>ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0J?ooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo04cooooo0ol0000Dooooo`004oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0I_ooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool04_ooool0017ooooo0ol0
000=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0IOooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo04kooooo0ol0000Booooo`004Oooool3o`0000cooooo00?o0000
ooooooooool03oooool00ol0003oooooooooo`1Sooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool0C_ooool3o`0001;ooooo000Aooooo`?o00002_ooool2o`0001?ooooo00?o0000oooo
ooooool0HOooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo04kooooo0ol0000Boooo
o`004_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000oooooooo
ool0Goooool2o`0001Wooooo00?o0000ooooooooool0Coooool00ol0003oooooooooo`0Aooooo`00
4?ooool3o`0000Wooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`1Mooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool0Coooool3o`00017ooooo000@ooooo`?o00002?oo
ool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo05_ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`1?ooooo`?o00004Oooool0013ooooo0ol00007ooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool0F_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooo
oooo04oooooo0ol0000Aooooo`004Oooool00ol0003oooooooooo`04ooooo`;o00007?ooool00ol0
003oooooooooo`1Hooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0D?ooool00ol0
003oooooooooo`0@ooooo`003oooool3o`0000Gooooo00?o0000ooooooooool07Oooool00ol0003o
ooooooooo`1Fooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0D?ooool3o`00013o
oooo000?ooooo`?o00001?ooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo05Gooooo
00?o0000ooooooooool07_ooool00ol0003oooooooooo`1@ooooo`?o00004?ooool000oooooo0ol0
0003ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0D_ooool2o`00027ooooo00?o
0000ooooooooool0D?ooool3o`00013ooooo000@ooooo`03o`000?oooooooooo00;o00009?ooool0
0ol0003oooooooooo`1@ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0D_ooool0
0ol0003oooooooooo`0?ooooo`003_ooool3o`000003ooooool0003ooooo02Gooooo00?o0000oooo
ooooool0Coooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05;ooooo0ol0000?oooo
o`003_ooool4o`0002Sooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`0Rooooo`03
o`000?oooooooooo05;ooooo0ol0000?ooooo`003_ooool3o`0002[ooooo00?o0000ooooooooool0
Boooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo05;ooooo0ol0000?ooooo`003_oo
ool2o`0002_ooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo05Cooooo00?o0000ooooooooool03Oooool000gooooo0ol0000/ooooo`03o`000?oooooo
oooo04Sooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1Cooooo`?o00003_ooool0
00gooooo0ol0000]ooooo`03o`000?oooooooooo04Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`1Cooooo`?o00003_ooool000gooooo0ol0000]ooooo`03o`000?oooooooooo04Co
oooo0_l0000Yooooo`03o`000?oooooooooo05?ooooo0ol0000>ooooo`003Oooool3o`0002kooooo
00?o0000ooooooooool0@_ooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo05?ooooo
0ol0000>ooooo`003Oooool3o`0002oooooo00?o0000ooooooooool0@?ooool00ol0003ooooooooo
o`0Zooooo`03o`000?oooooooooo05?ooooo0ol0000>ooooo`003Oooool3o`0002oooooo00?o0000
ooooooooool0?oooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo05?ooooo0ol0000>
ooooo`003_ooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo03gooooo00?o0000oooo
ooooool0;?ooool00ol0003oooooooooo`1Dooooo`03o`000?oooooooooo00gooooo000<ooooo`?o
0000<_ooool00ol0003oooooooooo`0kooooo`03o`000?oooooooooo02gooooo00?o0000oooooooo
ool0E?ooool3o`0000gooooo000<ooooo`?o0000<_ooool00ol0003oooooooooo`0jooooo`03o`00
0?oooooooooo02kooooo00?o0000ooooooooool0E?ooool3o`0000gooooo000<ooooo`?o0000<ooo
ool00ol0003oooooooooo`0gooooo`;o0000<Oooool00ol0003oooooooooo`1Dooooo`?o00003Ooo
ool000gooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0eooooo`03o`000?oooooo
oooo037ooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`0<ooooo`003?ooool3o`00
03Cooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
05Cooooo0ol0000=ooooo`003?ooool3o`0003Gooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0cooooo`03o`000?oooooooooo05Cooooo0ol0000=ooooo`003?ooool3o`0003Kooooo
00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo05Cooooo
0ol0000=ooooo`003Oooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo02oooooo00?o
0000ooooooooool0=Oooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo00cooooo000;
ooooo`?o0000>?ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo03Kooooo00?o0000
ooooooooool0EOooool3o`0000cooooo000;ooooo`?o0000>Oooool00ol0003oooooooooo`0Zoooo
o`;o0000>Oooool00ol0003oooooooooo`1Eooooo`?o00003?ooool000_ooooo0ol0000iooooo`03
o`000?oooooooooo02Wooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`1Eooooo`?o
00003?ooool000cooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`0Wooooo`03o`00
0?oooooooooo03[ooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0;ooooo`002ooo
ool3o`0003_ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0kooooo`03o`000?oo
oooooooo05Gooooo0ol0000<ooooo`002oooool3o`0003_ooooo00?o0000ooooooooool09?ooool0
0ol0003oooooooooo`0looooo`03o`000?oooooooooo05Gooooo0ol0000<ooooo`002oooool3o`00
03cooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo
05Gooooo0ol0000<ooooo`003?ooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo023o
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo00_o
oooo000;ooooo`?o0000?Oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo03oooooo
00?o0000ooooooooool0EOooool3o`0000cooooo000;ooooo`?o0000?_ooool00ol0003ooooooooo
o`0Looooo`;o0000@_ooool00ol0003oooooooooo`1Eooooo`?o00003?ooool000_ooooo0ol0000o
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`1E
ooooo`?o00003?ooool000cooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo04?ooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0;oooo
o`002oooool3o`00043ooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`14ooooo`03
o`000?oooooooooo05Gooooo0ol0000<ooooo`002oooool3o`00047ooooo00?o0000ooooooooool0
5Oooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo05Gooooo0ol0000<ooooo`002ooo
ool3o`00047ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`16ooooo`03o`000?oo
oooooooo05Gooooo0ol0000<ooooo`003?ooool00ol0003oooooooooo`11ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`1Fooooo`03o`000?oooooo
oooo00_ooooo000;ooooo`?o0000@_ooool00ol0003oooooooooo`0@ooooo`;o0000B_ooool00ol0
003oooooooooo`1Eooooo`?o00003?ooool000_ooooo3?l0000jooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`1Eooooo`?o00003?ooool000_ooooo
0ol00009oooooaGo00009_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo04_ooooo
00?o0000ooooooooool0EOooool3o`0000cooooo000<ooooo`03o`000?oooooooooo01gooooo5Ol0
000Aooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0C?ooool00ol0003ooooooooo
o`1Fooooo`03o`000?oooooooooo00_ooooo000<ooooo`?o0000<_oooolEo`0000Wooooo00?o0000
ooooooooool0COooool00ol0003oooooooooo`1Dooooo`?o00003Oooool000cooooo0ol00015oooo
o`03o`000?oooooo000001Co0000A?ooool00ol0003oooooooooo`1Dooooo`?o00003Oooool000co
oooo0ol00015ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02ooooolEo`0002oo
oooo00?o0000ooooooooool0E?ooool3o`0000gooooo000=ooooo`03o`000?oooooooooo04Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0QoooooaCo00006oooool00ol0003ooooo
ooooo`1Eooooo`03o`000?oooooooooo00cooooo000<ooooo`?o0000Aoooool01?l0003ooooooooo
ooooool2o`0003Sooooo5Ol00006ooooo`03o`000?oooooooooo05Cooooo0ol0000=ooooo`003?oo
ool3o`0004Oooooo00Co0000oooooooooooo0000CooooolEo`0004Sooooo0ol0000=ooooo`003?oo
ool3o`0004Sooooo0_l0001Fooooo`03o`000?oooooooooo00cooooo5Ol0000cooooo`?o00003Ooo
ool000gooooo00?o0000ooooooooool0Aoooool2o`0005Kooooo00?o0000ooooooooool08OoooolE
o`0001oooooo00?o0000ooooooooool03?ooool000gooooo0ol00016ooooo`03o`000?oooooo0000
05Kooooo00?o0000ooooooooool0=_oooolEo`0000Sooooo0ol0000>ooooo`003Oooool3o`0004Go
oooo00Go0000ooooooooooooooooo`00001Eooooo`03o`000?oooooooooo04_ooooo2ol0000>oooo
o`003Oooool3o`0004Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1Booooo`03
o`000?oooooooooo05?ooooo0ol0000>ooooo`003_ooool00ol0003oooooooooo`11ooooo`;o0000
1_ooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo05Cooooo00?o0000ooooooooool0
3Oooool000gooooo0ol00011ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0DOoo
ool00ol0003oooooooooo`1Cooooo`?o00003_ooool000gooooo0ol00010ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool0D?ooool00ol0003oooooooooo`1Cooooo`?o00003_ooool0
00gooooo0ol0000oooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0D?ooool00ol0
003oooooooooo`1Cooooo`?o00003_ooool000oooooo00?o0000ooooooooool0??ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool0Doooool00ol0003o
ooooooooo`0>ooooo`003_ooool3o`0003cooooo00?o0000ooooooooool03_ooool00ol0003ooooo
ooooo`1>ooooo`03o`000?oooooooooo05;ooooo0ol0000?ooooo`003_ooool3o`0003_ooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo05;ooooo0ol0
000?ooooo`003_ooool3o`0003[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1=
ooooo`03o`000?oooooooooo05;ooooo0ol0000?ooooo`003oooool3o`0003Oooooo0_l0000Eoooo
o`03o`000?oooooooooo04cooooo00?o0000ooooooooool0DOooool3o`00013ooooo000?ooooo`?o
0000=_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04cooooo00?o0000oooooooo
ool0DOooool3o`00013ooooo000?ooooo`?o0000=Oooool00ol0003oooooooooo`0Gooooo`03o`00
0?oooooooooo04[ooooo00?o0000ooooooooool0D_ooool3o`00013ooooo000@ooooo`03o`000?oo
oooooooo03?ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`19ooooo`03o`000?oo
oooooooo05?ooooo00?o0000ooooooooool03oooool0013ooooo0ol0000booooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool0BOooool00ol0003oooooooooo`1Aooooo`?o00004Oooool0
013ooooo0ol0000aooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0B?ooool00ol0
003oooooooooo`1Aooooo`?o00004Oooool0013ooooo0ol0000`ooooo`03o`000?oooooooooo01ko
oooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`1Aooooo`?o00004Oooool0017ooooo
00?o0000ooooooooool0;Oooool2o`00027ooooo00?o0000ooooooooool0Aoooool00ol0003ooooo
ooooo`1Booooo`03o`000?oooooooooo013ooooo000Aooooo`?o0000;?ooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo04Kooooo00?o0000ooooooooool0D?ooool3o`0001;ooooo000A
ooooo`?o0000:oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo04Gooooo00?o0000
ooooooooool0D?ooool3o`0001;ooooo000Aooooo`?o0000:_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo04Gooooo00?o0000ooooooooool0D?ooool3o`0001;ooooo000Cooooo`03
o`000?oooooooooo02Oooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`14ooooo`03
o`000?oooooooooo053ooooo00?o0000ooooooooool04_ooool001;ooooo0ol0000Wooooo`03o`00
0?oooooooooo02Wooooo00?o0000ooooooooool0@oooool00ol0003oooooooooo`1>ooooo`?o0000
5?ooool001;ooooo0ol0000Vooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0@ooo
ool00ol0003oooooooooo`1>ooooo`?o00005?ooool001;ooooo0ol0000Uooooo`03o`000?oooooo
oooo02cooooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`1>ooooo`?o00005?ooool0
01Cooooo0ol0000Qooooo`;o0000<?ooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo
04gooooo0ol0000Eooooo`005?ooool3o`00023ooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`11ooooo`03o`000?oooooooooo04gooooo0ol0000Eooooo`005?ooool3o`0001oooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo04gooooo
0ol0000Eooooo`005Oooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo03Cooooo00?o
0000ooooooooool0?oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo01Cooooo000E
ooooo`?o00007?ooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo03oooooo00?o0000
ooooooooool0Boooool3o`0001Oooooo000Eooooo`?o00006oooool00ol0003oooooooooo`0goooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0Boooool3o`0001Oooooo000Eooooo`?o
00006_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo03gooooo00?o0000oooooooo
ool0Boooool3o`0001Oooooo000Gooooo`03o`000?oooooooooo01Kooooo0_l0000looooo`03o`00
0?oooooooooo03gooooo00?o0000ooooooooool0C?ooool00ol0003oooooooooo`0Fooooo`005ooo
ool3o`0001Gooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`0looooo`03o`000?oo
oooooooo04[ooooo0ol0000Hooooo`005oooool3o`0001Cooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0looooo`03o`000?oooooooooo04[ooooo0ol0000Hooooo`005oooool3o`00
01?ooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`0kooooo`03o`000?oooooooooo
04[ooooo0ol0000Hooooo`006Oooool3o`00013ooooo00?o0000ooooooooool0@_ooool00ol0003o
ooooooooo`0jooooo`03o`000?oooooooooo04Sooooo0ol0000Jooooo`006Oooool3o`0000oooooo
00?o0000ooooooooool0@oooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo04Sooooo
0ol0000Jooooo`006Oooool3o`0000kooooo00?o0000ooooooooool0AOooool00ol0003ooooooooo
o`0iooooo`03o`000?oooooooooo04Sooooo0ol0000Jooooo`006oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`18
ooooo`03o`000?oooooooooo01[ooooo000Kooooo`?o00002Oooool2o`0004[ooooo00?o0000oooo
ooooool0>?ooool00ol0003oooooooooo`16ooooo`?o00007?ooool001_ooooo0ol00008ooooo`03
o`000?oooooooooo04_ooooo00?o0000ooooooooool0=oooool00ol0003oooooooooo`16ooooo`?o
00007?ooool001_ooooo0ol00007ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0
=_ooool00ol0003oooooooooo`16ooooo`?o00007?ooool001gooooo0ol00004ooooo`03o`000?oo
oooooooo04kooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`14ooooo`?o00007_oo
ool001gooooo0ol00003ooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool0=Oooool0
0ol0003oooooooooo`14ooooo`?o00007_ooool001gooooo0ol00002ooooo`03o`000?oooooooooo
05;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`14ooooo`?o00007_ooool001oo
oooo00?o0000ooooool00000EOooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo04Co
oooo00?o0000ooooooooool07_ooool001oooooo0ol0001Fooooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0@_ooool3o`00023ooooo000Oooooo`?o0000Eoooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo04;ooooo0ol0000Pooooo`007oooool3o`0005Oooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`12ooooo`?o00008?ooool0027ooooo0ol0001Foooo
o`03o`000?oooooooooo037ooooo00?o0000ooooooooool0@?ooool3o`0002;ooooo000Qooooo`?o
0000Eoooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo043ooooo0ol0000Rooooo`00
8Oooool3o`0005Oooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`10ooooo`?o0000
8_ooool002?ooooo0ol0001Fooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool0?_oo
ool3o`0002Cooooo000Sooooo`?o0000Eoooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo03kooooo0ol0000Tooooo`008oooool3o`0005Oooooo00?o0000ooooooooool0;_ooool00ol0
003oooooooooo`0nooooo`?o00009?ooool002Gooooo0ol0001Fooooo`03o`000?oooooooooo02go
oooo00?o0000ooooooooool0>oooool3o`0002Oooooo000Uooooo`?o0000Eoooool00ol0003ooooo
ooooo`0/ooooo`03o`000?oooooooooo03_ooooo0ol0000Wooooo`009Oooool3o`0005Oooooo00?o
0000ooooooooool0;?ooool00ol0003oooooooooo`0kooooo`?o00009oooool002Sooooo0ol0001E
ooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool0>Oooool3o`0002Wooooo000Xoooo
o`?o0000E_ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo03Wooooo0ol0000Yoooo
o`00:?ooool3o`0005Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0iooooo`?o
0000:Oooool002[ooooo0ol0001Eooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool0
=_ooool3o`0002cooooo000Zooooo`?o0000E_ooool00ol0003oooooooooo`0Xooooo`03o`000?oo
oooooooo03Kooooo0ol0000/ooooo`00:_ooool3o`0005Kooooo00?o0000ooooooooool0:?ooool0
0ol0003oooooooooo`0fooooo`?o0000;?ooool002gooooo0ol0001Dooooo`03o`000?oooooooooo
02Oooooo00?o0000ooooooooool0=?ooool3o`0002kooooo000]ooooo`?o0000EOooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo03Cooooo0ol0000^ooooo`00;Oooool3o`0005Gooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0dooooo`?o0000;_ooool0033ooooo0ol0
001Cooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0<Oooool3o`00037ooooo000`
ooooo`?o0000E?ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo037ooooo0ol0000a
ooooo`00<?ooool3o`0005Cooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0aoooo
o`?o0000<Oooool003?ooooo0ol0001Booooo`03o`000?oooooooooo02?ooooo00?o0000oooooooo
ool0;_ooool3o`0003Cooooo000cooooo`?o0000Doooool00ol0003oooooooooo`0Qooooo`03o`00
0?oooooooooo02oooooo0ol0000dooooo`00<oooool4o`0005;ooooo00?o0000ooooooooool08Ooo
ool00ol0003oooooooooo`0^ooooo`Co0000=?ooool003Kooooo0ol0001Aooooo`03o`000?oooooo
oooo023ooooo00?o0000ooooooooool0;?ooool3o`0003Oooooo000fooooo`?o0000D_ooool00ol0
003oooooooooo`0Oooooo`03o`000?oooooooooo02cooooo0ol0000gooooo`00=_ooool6o`0004oo
oooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0Yooooo`Ko0000=oooool003Wooooo
0ol0001@ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0:Oooool3o`0003[ooooo
000iooooo`Co0000D?ooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo02Sooooo1?l0
000jooooo`00??ooool3o`0004kooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0V
ooooo`?o0000?Oooool003cooooo0ol0001?ooooo`03o`000?oooooooooo01cooooo00?o0000oooo
ooooool09_ooool3o`0003gooooo000looooo`Ko0000C?ooool00ol0003oooooooooo`0Looooo`03
o`000?oooooooooo02?ooooo1_l0000mooooo`00?oooool3o`0004gooooo00?o0000ooooooooool0
6oooool00ol0003oooooooooo`0Sooooo`?o0000@?ooool003oooooo1_l0001;ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool08?ooool6o`00043ooooo0012ooooo`?o0000Boooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo023ooooo0ol00013ooooo`00@_ooool6o`00
04Wooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Looooo`Oo0000@oooool004Go
oooo1?l00019ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool07?ooool3o`0004Oo
oooo0015ooooo`?o000000Coooooo`000?l0003o0000A_ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Wooooo1_l00017ooooo`00BOooool3o`0004Oooooo00?o0000ooooooooool0
5oooool00ol0003oooooooooo`0Iooooo`?o0000B_ooool004Wooooo1_l00015ooooo`03o`000?oo
oooooooo01Kooooo00?o0000ooooooooool05_ooool6o`0004[ooooo001<ooooo`?o0000AOooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo0ol0001=ooooo`00C?ooool7o`00
04;ooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Booooo`Oo0000COooool0053o
oooo0ol00013ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool04_ooool3o`00057o
oooo001@ooooo`Ko0000@?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00kooooo
1ol0001Aooooo`00Doooool3o`00047ooooo00?o0000ooooooooool04oooool00ol0003ooooooooo
o`0>ooooo`?o0000EOooool005?ooooo1ol0000nooooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool02oooool6o`0005Gooooo001Gooooo`Oo0000>_ooool00ol0003oooooooooo`0Boooo
o`03o`000?oooooooooo00Oooooo1ol0001Hooooo`00Eoooool3o`000003ooooool0003o000000;o
0000>_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Oooooo0ol000001?oooooo
0000o`000?l0001Hooooo`00Foooool6o`0003Wooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`04ooooo`Ko0000G?ooool005kooooo1ol0000eooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool01ol0001Oooooo`00G_ooool3o`000003ooooool0003o000000Go0000<_oo
ool00ol0003oooooooooo`0>ooooo`Oo000000Coooooo`000?l0003o0000Goooool006;ooooo0ol0
00000ooooooo0000o`000005o`0002oooooo00?o0000ooooooooool02Oooool7o`000004ooooool0
003o0000o`0006?ooooo001Vooooo`?o000000?oooooo`000?l000001?l0000/ooooo`03o`000?oo
oooooooo00Gooooo1ol000001?oooooo0000o`000?l0001Wooooo`00J_ooool:o`000004ooooool0
003o0000o`0002Gooooo1_l000001Ooooooo0000o`000?l0003ooooo00?o0000Joooool006gooooo
0ol000000ooooooo0000o`000009o`000004ooooool0003o0000o`0001Gooooo0ol000000ooooooo
0000o`00000<o`0006oooooo001aooooo`?o000000Goooooo`000?l0003o0000ooooo`0;o`000005
ooooool0003o0000o`000?ooool00ol000001Ooooooo0000o`000?l0003ooooo00_o000000?ooooo
o`000?l000001?l0001cooooo`00NOooool3o`000005ooooool0003o0000o`000?ooool04ol00000
1Ooooooo0000o`000?l0003ooooo00?o0000N_ooool0087ooooo0ol000001Ooooooo0000o`000?l0
003ooooo00?o000000Goooooo`000?l0003o0000ooooo`03o`0008;ooooo003ooooooaOooooo003o
oooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003o
oooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo0000
\
\>"],
  ImageRangeCache->{{{0, 277}, {277, 0}} -> {-0.0500054, -0.0500054, \
0.00397116, 0.00397116}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Construct an expanded update neighborhood.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g2 = GraphPower[g, 2] // MakeSimple;\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[g2, VertexStyle \[Rule] Disk[Small]];\)\)}], "Input",
  CellLabel->"In[7]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95253 .54278 L
s
.95432 .51428 m
.95432 .48572 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95365 .52854 m
.95096 .55697 L
s
.95365 .52854 m
.95455 .5 L
s
.95253 .54278 m
.95096 .55697 L
s
.95253 .54278 m
.94895 .57111 L
s
.95096 .55697 m
.94895 .57111 L
s
.95096 .55697 m
.94649 .58517 L
s
.95096 .55697 m
.04635 .52854 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.9436 .59916 L
s
.94895 .57111 m
.04747 .54278 L
s
.94895 .57111 m
.04635 .52854 L
s
.94895 .57111 m
.04568 .51428 L
s
.94649 .58517 m
.9436 .59916 L
s
.94649 .58517 m
.94027 .61304 L
s
.94649 .58517 m
.04635 .52854 L
s
.9436 .59916 m
.94027 .61304 L
s
.9436 .59916 m
.9365 .62681 L
s
.94027 .61304 m
.9365 .62681 L
s
.94027 .61304 m
.9323 .64046 L
s
.9365 .62681 m
.9323 .64046 L
s
.9365 .62681 m
.92767 .65397 L
s
.9323 .64046 m
.92767 .65397 L
s
.9323 .64046 m
.92263 .66733 L
s
.92767 .65397 m
.92263 .66733 L
s
.92767 .65397 m
.91716 .68052 L
s
.92263 .66733 m
.91716 .68052 L
s
.92263 .66733 m
.91129 .69354 L
s
.91716 .68052 m
.91129 .69354 L
s
.91716 .68052 m
.905 .70636 L
s
.91129 .69354 m
.905 .70636 L
s
.91129 .69354 m
.89832 .71898 L
s
.905 .70636 m
.89832 .71898 L
s
.905 .70636 m
.89125 .73138 L
s
.89832 .71898 m
.89125 .73138 L
s
.89832 .71898 m
.88379 .74356 L
s
.89125 .73138 m
.88379 .74356 L
s
.89125 .73138 m
.87595 .75549 L
s
.88379 .74356 m
.87595 .75549 L
s
.88379 .74356 m
.86773 .76718 L
s
.87595 .75549 m
.86773 .76718 L
s
.87595 .75549 m
.85916 .77859 L
s
.86773 .76718 m
.85916 .77859 L
s
.86773 .76718 m
.85023 .78974 L
s
.85916 .77859 m
.85023 .78974 L
s
.85916 .77859 m
.84096 .8006 L
s
.85023 .78974 m
.84096 .8006 L
s
.85023 .78974 m
.83135 .81116 L
s
.84096 .8006 m
.83135 .81116 L
s
.84096 .8006 m
.82141 .82141 L
s
.83135 .81116 m
.82141 .82141 L
s
.83135 .81116 m
.81116 .83135 L
s
.82141 .82141 m
.81116 .83135 L
s
.82141 .82141 m
.8006 .84096 L
s
.81116 .83135 m
.8006 .84096 L
s
.81116 .83135 m
.78974 .85023 L
s
.8006 .84096 m
.78974 .85023 L
s
.8006 .84096 m
.77859 .85916 L
s
.78974 .85023 m
.77859 .85916 L
s
.78974 .85023 m
.76718 .86773 L
s
.77859 .85916 m
.76718 .86773 L
s
.77859 .85916 m
.75549 .87595 L
s
.76718 .86773 m
.75549 .87595 L
s
.76718 .86773 m
.74356 .88379 L
s
.75549 .87595 m
.74356 .88379 L
s
.75549 .87595 m
.73138 .89125 L
s
.74356 .88379 m
.73138 .89125 L
s
.74356 .88379 m
.71898 .89832 L
s
.73138 .89125 m
.71898 .89832 L
s
.73138 .89125 m
.70636 .905 L
s
.71898 .89832 m
.70636 .905 L
s
.71898 .89832 m
.69354 .91129 L
s
.70636 .905 m
.69354 .91129 L
s
.70636 .905 m
.68052 .91716 L
s
.69354 .91129 m
.68052 .91716 L
s
.69354 .91129 m
.66733 .92263 L
s
.68052 .91716 m
.66733 .92263 L
s
.68052 .91716 m
.65397 .92767 L
s
.66733 .92263 m
.65397 .92767 L
s
.66733 .92263 m
.64046 .9323 L
s
.65397 .92767 m
.64046 .9323 L
s
.65397 .92767 m
.62681 .9365 L
s
.64046 .9323 m
.62681 .9365 L
s
.64046 .9323 m
.61304 .94027 L
s
.64046 .9323 m
.64046 .0677 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.59916 .9436 L
s
.62681 .9365 m
.62681 .0635 L
s
.62681 .9365 m
.64046 .0677 L
s
.62681 .9365 m
.65397 .07233 L
s
.61304 .94027 m
.59916 .9436 L
s
.61304 .94027 m
.58517 .94649 L
s
.61304 .94027 m
.64046 .0677 L
s
.59916 .9436 m
.58517 .94649 L
s
.59916 .9436 m
.57111 .94895 L
s
.58517 .94649 m
.57111 .94895 L
s
.58517 .94649 m
.55697 .95096 L
s
.58517 .94649 m
.11621 .25644 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.54278 .95253 L
s
.57111 .94895 m
.10875 .26862 L
s
.57111 .94895 m
.11621 .25644 L
s
.57111 .94895 m
.12405 .24451 L
s
.55697 .95096 m
.54278 .95253 L
s
.55697 .95096 m
.52854 .95365 L
s
.55697 .95096 m
.11621 .25644 L
s
.54278 .95253 m
.52854 .95365 L
s
.54278 .95253 m
.51428 .95432 L
s
.52854 .95365 m
.51428 .95432 L
s
.52854 .95365 m
.5 .95455 L
s
.51428 .95432 m
.5 .95455 L
s
.51428 .95432 m
.48572 .95432 L
s
.5 .95455 m
.48572 .95432 L
s
.5 .95455 m
.47146 .95365 L
s
.48572 .95432 m
.47146 .95365 L
s
.48572 .95432 m
.45722 .95253 L
s
.47146 .95365 m
.45722 .95253 L
s
.47146 .95365 m
.44303 .95096 L
s
.45722 .95253 m
.44303 .95096 L
s
.45722 .95253 m
.42889 .94895 L
s
.44303 .95096 m
.42889 .94895 L
s
.44303 .95096 m
.41483 .94649 L
s
.42889 .94895 m
.41483 .94649 L
s
.42889 .94895 m
.40084 .9436 L
s
.41483 .94649 m
.40084 .9436 L
s
.41483 .94649 m
.38696 .94027 L
s
.40084 .9436 m
.38696 .94027 L
s
.40084 .9436 m
.37319 .9365 L
s
.38696 .94027 m
.37319 .9365 L
s
.38696 .94027 m
.35954 .9323 L
s
.37319 .9365 m
.35954 .9323 L
s
.37319 .9365 m
.34603 .92767 L
s
.35954 .9323 m
.34603 .92767 L
s
.35954 .9323 m
.33267 .92263 L
s
.34603 .92767 m
.33267 .92263 L
s
.34603 .92767 m
.31948 .91716 L
s
.33267 .92263 m
.31948 .91716 L
s
.33267 .92263 m
.30646 .91129 L
s
.31948 .91716 m
.30646 .91129 L
s
.31948 .91716 m
.29364 .905 L
s
.30646 .91129 m
.29364 .905 L
s
.30646 .91129 m
.28102 .89832 L
s
.29364 .905 m
.28102 .89832 L
s
.29364 .905 m
.26862 .89125 L
s
.28102 .89832 m
.26862 .89125 L
s
.28102 .89832 m
.25644 .88379 L
s
.26862 .89125 m
.25644 .88379 L
s
.26862 .89125 m
.24451 .87595 L
s
.25644 .88379 m
.24451 .87595 L
s
.25644 .88379 m
.23282 .86773 L
s
.24451 .87595 m
.23282 .86773 L
s
.24451 .87595 m
.22141 .85916 L
s
.23282 .86773 m
.22141 .85916 L
s
.23282 .86773 m
.21026 .85023 L
s
.22141 .85916 m
.21026 .85023 L
s
.22141 .85916 m
.1994 .84096 L
s
.21026 .85023 m
.1994 .84096 L
s
.21026 .85023 m
.18884 .83135 L
s
.1994 .84096 m
.18884 .83135 L
s
.1994 .84096 m
.17859 .82141 L
s
.18884 .83135 m
.17859 .82141 L
s
.18884 .83135 m
.16865 .81116 L
s
.17859 .82141 m
.16865 .81116 L
s
.17859 .82141 m
.15904 .8006 L
s
.16865 .81116 m
.15904 .8006 L
s
.16865 .81116 m
.14977 .78974 L
s
.15904 .8006 m
.14977 .78974 L
s
.15904 .8006 m
.14084 .77859 L
s
.14977 .78974 m
.14084 .77859 L
s
.14977 .78974 m
.13227 .76718 L
s
.14084 .77859 m
.13227 .76718 L
s
.14084 .77859 m
.12405 .75549 L
s
.13227 .76718 m
.12405 .75549 L
s
.13227 .76718 m
.11621 .74356 L
s
.12405 .75549 m
.11621 .74356 L
s
.12405 .75549 m
.10875 .73138 L
s
.12405 .75549 m
.78974 .14977 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.10168 .71898 L
s
.11621 .74356 m
.77859 .14084 L
s
.11621 .74356 m
.78974 .14977 L
s
.11621 .74356 m
.8006 .15904 L
s
.10875 .73138 m
.10168 .71898 L
s
.10875 .73138 m
.095 .70636 L
s
.10875 .73138 m
.78974 .14977 L
s
.10168 .71898 m
.095 .70636 L
s
.10168 .71898 m
.08871 .69354 L
s
.095 .70636 m
.08871 .69354 L
s
.095 .70636 m
.08284 .68052 L
s
.08871 .69354 m
.08284 .68052 L
s
.08871 .69354 m
.07737 .66733 L
s
.08284 .68052 m
.07737 .66733 L
s
.08284 .68052 m
.07233 .65397 L
s
.07737 .66733 m
.07233 .65397 L
s
.07737 .66733 m
.0677 .64046 L
s
.07233 .65397 m
.0677 .64046 L
s
.07233 .65397 m
.0635 .62681 L
s
.0677 .64046 m
.0635 .62681 L
s
.0677 .64046 m
.05973 .61304 L
s
.0635 .62681 m
.05973 .61304 L
s
.0635 .62681 m
.0564 .59916 L
s
.05973 .61304 m
.0564 .59916 L
s
.05973 .61304 m
.05351 .58517 L
s
.0564 .59916 m
.05351 .58517 L
s
.0564 .59916 m
.05105 .57111 L
s
.05351 .58517 m
.05105 .57111 L
s
.05351 .58517 m
.04904 .55697 L
s
.05105 .57111 m
.04904 .55697 L
s
.05105 .57111 m
.04747 .54278 L
s
.04904 .55697 m
.04747 .54278 L
s
.04904 .55697 m
.04635 .52854 L
s
.04747 .54278 m
.04635 .52854 L
s
.04747 .54278 m
.04568 .51428 L
s
.04635 .52854 m
.04568 .51428 L
s
.04635 .52854 m
.04545 .5 L
s
.04568 .51428 m
.04545 .5 L
s
.04568 .51428 m
.04568 .48572 L
s
.04545 .5 m
.04568 .48572 L
s
.04545 .5 m
.04635 .47146 L
s
.04568 .48572 m
.04635 .47146 L
s
.04568 .48572 m
.04747 .45722 L
s
.04635 .47146 m
.04747 .45722 L
s
.04635 .47146 m
.04904 .44303 L
s
.04747 .45722 m
.04904 .44303 L
s
.04747 .45722 m
.05105 .42889 L
s
.04904 .44303 m
.05105 .42889 L
s
.04904 .44303 m
.05351 .41483 L
s
.05105 .42889 m
.05351 .41483 L
s
.05105 .42889 m
.0564 .40084 L
s
.05105 .42889 m
.5 .04545 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.05973 .38696 L
s
.05351 .41483 m
.48572 .04568 L
s
.05351 .41483 m
.5 .04545 L
s
.05351 .41483 m
.51428 .04568 L
s
.0564 .40084 m
.05973 .38696 L
s
.0564 .40084 m
.0635 .37319 L
s
.0564 .40084 m
.5 .04545 L
s
.05973 .38696 m
.0635 .37319 L
s
.05973 .38696 m
.0677 .35954 L
s
.0635 .37319 m
.0677 .35954 L
s
.0635 .37319 m
.07233 .34603 L
s
.0677 .35954 m
.07233 .34603 L
s
.0677 .35954 m
.07737 .33267 L
s
.07233 .34603 m
.07737 .33267 L
s
.07233 .34603 m
.08284 .31948 L
s
.07737 .33267 m
.08284 .31948 L
s
.07737 .33267 m
.08871 .30646 L
s
.08284 .31948 m
.08871 .30646 L
s
.08284 .31948 m
.095 .29364 L
s
.08871 .30646 m
.095 .29364 L
s
.08871 .30646 m
.10168 .28102 L
s
.095 .29364 m
.10168 .28102 L
s
.095 .29364 m
.10875 .26862 L
s
.10168 .28102 m
.10875 .26862 L
s
.10168 .28102 m
.11621 .25644 L
s
.10875 .26862 m
.11621 .25644 L
s
.10875 .26862 m
.12405 .24451 L
s
.11621 .25644 m
.12405 .24451 L
s
.11621 .25644 m
.13227 .23282 L
s
.12405 .24451 m
.13227 .23282 L
s
.12405 .24451 m
.14084 .22141 L
s
.13227 .23282 m
.14084 .22141 L
s
.13227 .23282 m
.14977 .21026 L
s
.14084 .22141 m
.14977 .21026 L
s
.14084 .22141 m
.15904 .1994 L
s
.14977 .21026 m
.15904 .1994 L
s
.14977 .21026 m
.16865 .18884 L
s
.15904 .1994 m
.16865 .18884 L
s
.15904 .1994 m
.17859 .17859 L
s
.16865 .18884 m
.17859 .17859 L
s
.16865 .18884 m
.18884 .16865 L
s
.17859 .17859 m
.18884 .16865 L
s
.17859 .17859 m
.1994 .15904 L
s
.18884 .16865 m
.1994 .15904 L
s
.18884 .16865 m
.21026 .14977 L
s
.1994 .15904 m
.21026 .14977 L
s
.1994 .15904 m
.22141 .14084 L
s
.21026 .14977 m
.22141 .14084 L
s
.21026 .14977 m
.23282 .13227 L
s
.22141 .14084 m
.23282 .13227 L
s
.22141 .14084 m
.24451 .12405 L
s
.23282 .13227 m
.24451 .12405 L
s
.23282 .13227 m
.25644 .11621 L
s
.24451 .12405 m
.25644 .11621 L
s
.24451 .12405 m
.26862 .10875 L
s
.25644 .11621 m
.26862 .10875 L
s
.25644 .11621 m
.28102 .10168 L
s
.26862 .10875 m
.28102 .10168 L
s
.26862 .10875 m
.29364 .095 L
s
.28102 .10168 m
.29364 .095 L
s
.28102 .10168 m
.30646 .08871 L
s
.29364 .095 m
.30646 .08871 L
s
.29364 .095 m
.31948 .08284 L
s
.30646 .08871 m
.31948 .08284 L
s
.30646 .08871 m
.33267 .07737 L
s
.31948 .08284 m
.33267 .07737 L
s
.31948 .08284 m
.34603 .07233 L
s
.33267 .07737 m
.34603 .07233 L
s
.33267 .07737 m
.35954 .0677 L
s
.34603 .07233 m
.35954 .0677 L
s
.34603 .07233 m
.37319 .0635 L
s
.35954 .0677 m
.37319 .0635 L
s
.35954 .0677 m
.38696 .05973 L
s
.37319 .0635 m
.38696 .05973 L
s
.37319 .0635 m
.40084 .0564 L
s
.38696 .05973 m
.40084 .0564 L
s
.38696 .05973 m
.41483 .05351 L
s
.40084 .0564 m
.41483 .05351 L
s
.40084 .0564 m
.42889 .05105 L
s
.41483 .05351 m
.42889 .05105 L
s
.41483 .05351 m
.44303 .04904 L
s
.42889 .05105 m
.44303 .04904 L
s
.42889 .05105 m
.45722 .04747 L
s
.44303 .04904 m
.45722 .04747 L
s
.44303 .04904 m
.47146 .04635 L
s
.45722 .04747 m
.47146 .04635 L
s
.45722 .04747 m
.48572 .04568 L
s
.47146 .04635 m
.48572 .04568 L
s
.47146 .04635 m
.5 .04545 L
s
.48572 .04568 m
.5 .04545 L
s
.48572 .04568 m
.51428 .04568 L
s
.5 .04545 m
.51428 .04568 L
s
.5 .04545 m
.52854 .04635 L
s
.51428 .04568 m
.52854 .04635 L
s
.51428 .04568 m
.54278 .04747 L
s
.52854 .04635 m
.54278 .04747 L
s
.52854 .04635 m
.55697 .04904 L
s
.54278 .04747 m
.55697 .04904 L
s
.54278 .04747 m
.57111 .05105 L
s
.55697 .04904 m
.57111 .05105 L
s
.55697 .04904 m
.58517 .05351 L
s
.57111 .05105 m
.58517 .05351 L
s
.57111 .05105 m
.59916 .0564 L
s
.58517 .05351 m
.59916 .0564 L
s
.58517 .05351 m
.61304 .05973 L
s
.59916 .0564 m
.61304 .05973 L
s
.59916 .0564 m
.62681 .0635 L
s
.61304 .05973 m
.62681 .0635 L
s
.61304 .05973 m
.64046 .0677 L
s
.62681 .0635 m
.64046 .0677 L
s
.62681 .0635 m
.65397 .07233 L
s
.64046 .0677 m
.65397 .07233 L
s
.64046 .0677 m
.66733 .07737 L
s
.65397 .07233 m
.66733 .07737 L
s
.65397 .07233 m
.68052 .08284 L
s
.66733 .07737 m
.68052 .08284 L
s
.66733 .07737 m
.69354 .08871 L
s
.68052 .08284 m
.69354 .08871 L
s
.68052 .08284 m
.70636 .095 L
s
.69354 .08871 m
.70636 .095 L
s
.69354 .08871 m
.71898 .10168 L
s
.70636 .095 m
.71898 .10168 L
s
.70636 .095 m
.73138 .10875 L
s
.71898 .10168 m
.73138 .10875 L
s
.71898 .10168 m
.74356 .11621 L
s
.73138 .10875 m
.74356 .11621 L
s
.73138 .10875 m
.75549 .12405 L
s
.74356 .11621 m
.75549 .12405 L
s
.74356 .11621 m
.76718 .13227 L
s
.75549 .12405 m
.76718 .13227 L
s
.75549 .12405 m
.77859 .14084 L
s
.76718 .13227 m
.77859 .14084 L
s
.76718 .13227 m
.78974 .14977 L
s
.77859 .14084 m
.78974 .14977 L
s
.77859 .14084 m
.8006 .15904 L
s
.78974 .14977 m
.8006 .15904 L
s
.78974 .14977 m
.81116 .16865 L
s
.8006 .15904 m
.81116 .16865 L
s
.8006 .15904 m
.82141 .17859 L
s
.81116 .16865 m
.82141 .17859 L
s
.81116 .16865 m
.83135 .18884 L
s
.82141 .17859 m
.83135 .18884 L
s
.82141 .17859 m
.84096 .1994 L
s
.83135 .18884 m
.84096 .1994 L
s
.83135 .18884 m
.85023 .21026 L
s
.84096 .1994 m
.85023 .21026 L
s
.84096 .1994 m
.85916 .22141 L
s
.85023 .21026 m
.85916 .22141 L
s
.85023 .21026 m
.86773 .23282 L
s
.85916 .22141 m
.86773 .23282 L
s
.85916 .22141 m
.87595 .24451 L
s
.86773 .23282 m
.87595 .24451 L
s
.86773 .23282 m
.88379 .25644 L
s
.87595 .24451 m
.88379 .25644 L
s
.87595 .24451 m
.89125 .26862 L
s
.88379 .25644 m
.89125 .26862 L
s
.88379 .25644 m
.89832 .28102 L
s
.89125 .26862 m
.89832 .28102 L
s
.89125 .26862 m
.905 .29364 L
s
.89832 .28102 m
.905 .29364 L
s
.89832 .28102 m
.91129 .30646 L
s
.905 .29364 m
.91129 .30646 L
s
.905 .29364 m
.91716 .31948 L
s
.91129 .30646 m
.91716 .31948 L
s
.91129 .30646 m
.92263 .33267 L
s
.91716 .31948 m
.92263 .33267 L
s
.91716 .31948 m
.92767 .34603 L
s
.92263 .33267 m
.92767 .34603 L
s
.92263 .33267 m
.9323 .35954 L
s
.92767 .34603 m
.9323 .35954 L
s
.92767 .34603 m
.9365 .37319 L
s
.9323 .35954 m
.9365 .37319 L
s
.9323 .35954 m
.94027 .38696 L
s
.9365 .37319 m
.94027 .38696 L
s
.9365 .37319 m
.9436 .40084 L
s
.94027 .38696 m
.9436 .40084 L
s
.94027 .38696 m
.94649 .41483 L
s
.9436 .40084 m
.94649 .41483 L
s
.9436 .40084 m
.94895 .42889 L
s
.94649 .41483 m
.94895 .42889 L
s
.94649 .41483 m
.95096 .44303 L
s
.94895 .42889 m
.95096 .44303 L
s
.94895 .42889 m
.95253 .45722 L
s
.95096 .44303 m
.95253 .45722 L
s
.95096 .44303 m
.95365 .47146 L
s
.95253 .45722 m
.95365 .47146 L
s
.95253 .45722 m
.95432 .48572 L
s
.95365 .47146 m
.95432 .48572 L
s
.95365 .47146 m
.95455 .5 L
s
.95432 .48572 m
.95455 .5 L
s
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[7]:=",
  ImageSize->{278, 278},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F00015R000`400?l00000o`00003ooooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00POooool3o`000005ooooool0003o0000o`000?ooool00ol000001Ooo
oooo0000o`000?l0003ooooo00?o0000P_ooool007Wooooo0ol000000ooooooo0000o`00000Io`00
0004ooooool0003o0000o`0007[ooooo001aooooo`?o000000?oooooo`000?l000006Ol000000ooo
oooo0000o`00000@o`0007?ooooo001]ooooo`oo000000Coooooo`000?l0003o00000oooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01oooool3o`000003ooooool0003o
000000co0000Koooool006[ooooo2_l000001?oooooo0000o`000?l0000:ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`0@ooooo`ko0000Joooool006Kooooo2_l0000A
ooooo`03o`000?oooooooooo00;o00000_ooool2o`0001[ooooo0ol000000ooooooo0000o`000005
o`0006Oooooo001Rooooo`Oo000000Coooooo`000?l0003o00004oooool00ol0003oooooooooo`02
o`0000;ooooo00?o0000ooooooooool07_ooool3o`000003ooooool0003o000000Go0000Hoooool0
06;ooooo1ol0000Eooooo`;o00000_ooool2o`0000;ooooo00?o0000ooooooooool08oooool7o`00
06?ooooo001Nooooo`Oo00006?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`07o`0005oooooo001Kooooo`Ko00006oooool00ol0003o
ooooooooo`02o`0000;ooooo0_l0000Xooooo`05o`000?ooooooooooooooool000000_ooool6o`00
05cooooo001Jooooo`Oo00006_ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000oooooooo
ool0:?ooool01Ol0003ooooooooooooooooo000000;ooooo1ol0001Kooooo`00Eoooool7o`0001co
oooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo02Wooooo00Go0000oooooooo
ooooooooo`000003ooooo`03o`000?oooooo000000Ko0000F?ooool005?ooooo1ol0000Nooooo`;o
00000_ooool2o`0000;ooooo00?o0000ooooooooool0:_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool6o`0005Gooooo001Cooooo`?o000000Coooooo`000?l0
003o00007Oooool00ol0003oooooooooo`03o`000003ooooool0003o000002gooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo1ol0001Dooooo`00D?ooool6o`00
023ooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo02gooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00Kooooo1ol0001Aooooo`00D?ooool3o`00
02;ooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo02kooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00[ooooo0ol0001Aooooo`00C?ooool7o`00
027ooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo02oooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00[ooooo1ol0001=ooooo`00C?ooool3o`00
02Cooooo00?o0000ooooool000000_l000000ooooooo0000o`00000booooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooooooo`0>ooooo`?o0000COooool004Wooooo1_l0000Roooo
o`;o000000Koooooo`000?l0003o0000ooooool0000dooooo`05o`000?ooooooooooooooool00000
0oooool00ol0003oooooooooo`0>ooooo`Ko0000B_ooool004Wooooo0ol0000Tooooo`03o`000?oo
oooooooo00?o000000?oooooo`000?ooool0=?ooool01Ol0003ooooooooooooooooo000000?ooooo
00?o0000ooooooooool04Oooool3o`0004[ooooo0015ooooo`Oo00008oooool00ol0003ooooooooo
o`02o`0000;ooooo00?o0000ooooooooool0=?ooool01Ol0003ooooooooooooooooo000000?ooooo
00?o0000ooooooooool04Oooool6o`0004Oooooo0015ooooo`?o00009_ooool00ol0003ooooooooo
o`02o`000003ooooool0003o000003Oooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo01Cooooo0ol00017ooooo`00@_ooool6o`0002Gooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`0hooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooo
ooooo`0Dooooo`Oo0000@oooool004;ooooo0ol0000Wooooo`03o`000?oooooo000000;o000000?o
ooooo`000?ooool0>Oooool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool06?oo
ool3o`0004?ooooo000oooooo`Ko00009Oooool2o`000006ooooool0003o0000o`000?oooooo0000
>oooool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool06?ooool6o`00043ooooo
000oooooo`?o00009oooool00ol0003oooooooooo`05o`0003cooooo00?o0000ooooooooool00_l0
0002ooooo`03o`000?oooooooooo01cooooo0ol00010ooooo`00?_ooool4o`0002Kooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`0mooooo`03o`000?oooooooooo00;o00000_ooool0
0ol0003oooooooooo`0Looooo`Co0000?oooool003cooooo0ol0000Xooooo`03o`000?oooooo0000
00;o000000?oooooo`000?ooool0?_ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000oooo
ooooool07oooool3o`0003gooooo000looooo`?o00009oooool00ol0003oooooo`000004o`00043o
oooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo01oooooo0ol0000mooooo`00
>Oooool6o`0002Gooooo0_l000000ooooooo0000o`000002o`0004;ooooo00?o0000ooooooooool0
0_l00002ooooo`03o`000?oooooooooo01kooooo1ol0000jooooo`00>Oooool3o`0002Oooooo00?o
0000ooooool000001?l00013ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003ooooooooo
o`0Mooooo`03o`000?oooooooooo00;ooooo0ol0000jooooo`00>?ooool4o`0002Kooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`13ooooo`03o`000?oooooooooo00;o00000_ooool0
0ol0003oooooooooo`0Looooo`03o`000?oooooooooo00;ooooo1Ol0000iooooo`00=_ooool3o`00
02Sooooo00?o0000ooooool000001?l00015ooooo`03o`000?oooooooooo00;o00000_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool3o`00
03Oooooo000fooooo`?o00009oooool00ol0003oooooo`000003o`0004Oooooo00?o0000oooooooo
ool00_l00002ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`03ooooo`?o0000=oooool003?ooooo1_l0000Vooooo`?o000000?oooooo`000?l00000
B?ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00?ooooo1ol0000dooooo`00<oooool3o`0002Oooooo1?l0
00000ooooooo0000o`000019ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`02ooooo`?o0000=?ooool003?ooooo0ol0000Vooooo`03o`000?oooooo000000Co0000B_ooool0
0ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool05oooool01?l0003ooooooooooooo
ool2o`0000Gooooo00?o0000ooooooooool00oooool3o`0003Cooooo000`ooooo`?o0000:?ooool0
0ol0003oooooo`000003o`0004cooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooo
oooo01Kooooo00Co0000oooooooooooooooo0_l00005ooooo`03o`000?oooooooooo00Oooooo0ol0
000aooooo`00<?ooool3o`0002Oooooo1_l0001=ooooo`03o`000?oooooooooo00;o00000_ooool0
0ol0003oooooooooo`0Dooooo`;o00000oooool2o`0000Gooooo00?o0000ooooooooool02?ooool3
o`00037ooooo000`ooooo`?o00009_ooool3o`000003ooooool0003o000004kooooo00?o0000oooo
ooooool00_l00002ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo0_l0000;ooooo`?o0000<Oooool002gooooo1?l0000Wooooo`?o000000?oooooo`000?l0
0000Coooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool04_ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0=ooooo`Co0000;_ooool002gooooo
0ol0000Vooooo`Co000000?oooooo`000?ooool0D?ooool00ol0003oooooooooo`02o`0000;ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool03oooool3o`0002kooooo000]ooooo`?o00009Oooool6o`0005;ooooo00?o
0000ooooooooool00_l00002ooooo`03o`000?oooooooooo013ooooo00Co0000oooooooooooooooo
0_l00005ooooo`03o`000?oooooooooo013ooooo0ol0000^ooooo`00:_ooool4o`0002Kooooo1_l0
001Cooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0?ooooo`04o`000?oo
ooooooooooooo`;o00001Oooool00ol0003oooooooooo`0Cooooo`?o0000;?ooool002[ooooo0ol0
000Vooooo`?o000000?oooooo`000?l00000E?ooool00ol0003oooooooooo`02o`0000;ooooo00?o
0000ooooooooool03_ooool01?l0003oooooooooooooool2o`0000Cooooo0_l0000Fooooo`?o0000
;?ooool002[ooooo0ol0000Uooooo`?o000000?oooooo`000?ooool0EOooool00ol0003ooooooooo
o`02o`0000;ooooo00?o0000ooooooooool03Oooool01?l0003oooooooooooooool2o`0000Cooooo
00?o0000ooooooooool05_ooool3o`0002cooooo000Xooooo`Co00009?ooool4o`000003ooooool0
003ooooo05Kooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo00cooooo00Co
0000oooooooooooooooo0_l00004ooooo`03o`000?oooooooooo01Wooooo1?l0000Yooooo`00:?oo
ool3o`0002Cooooo1?l000000ooooooo0000ooooo`1Gooooo`03o`000?oooooooooo00;o00000_oo
ool00ol0003oooooooooo`0;ooooo`04o`000?ooooooooooooooo`;o00001?ooool00ol0003ooooo
ooooo`0Kooooo`?o0000:Oooool002Sooooo0ol0000Sooooo`?o000000?oooooo`000?l00000FOoo
ool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool02Oooool2o`0000;ooooo0ol0
0004ooooo`03o`000?oooooooooo01cooooo0ol0000Yooooo`009Oooool4o`0002Cooooo0ol00000
0ooooooo0000ooooo`1Jooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`08
ooooo`04o`000?ooooooooooooooo`?o00001?ooool00ol0003oooooooooo`0Oooooo`?o00009ooo
ool002Gooooo0ol0000Tooooo`?o000000?oooooo`000?ooool0Foooool00ol0003oooooooooo`02
o`0000;ooooo00?o0000ooooooooool01oooool01?l0003oooooooooooooool3o`0000?ooooo0_l0
000Rooooo`?o00009oooool002Gooooo0ol0000Sooooo`05o`000?oooooo0000ooooool00000GOoo
ool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool01_ooool01?l0003ooooooooo
ooooool3o`0000?ooooo00?o0000ooooooooool08_ooool3o`0002Oooooo000Sooooo`Co00008_oo
ool6o`0005kooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo00Gooooo00Co
0000oooooooooooooooo0_l00004ooooo`03o`000?oooooooooo02Gooooo1?l0000Tooooo`008ooo
ool3o`0002;ooooo0ol000000ooooooo0000o`00001Oooooo`03o`000?oooooo000000;o00000_oo
ool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`;o00001?ooool00ol0003ooooo
ooooo`0Wooooo`?o00009?ooool002?ooooo0ol0000Qooooo`?o000000?oooooo`000?l00000H?oo
ool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`03o`0000Cooooo00?o0000ooooooooool0:?ooool3o`0002Cooooo000Qooooo`?o00008_ooool3
o`000003ooooool0003ooooo067ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`04
ooooo`03o`000?oooooooooo00?o00001?ooool00ol0003oooooooooo`0[ooooo`?o00008_ooool0
027ooooo0ol0000Qooooo`03o`000?oooooo000000;o0000Hoooool00ol0003oooooo`000002o`00
0003ooooool0003ooooo00?ooooo00?o0000ooooooooool00ol00003ooooo`;o0000;_ooool3o`00
02;ooooo000Qooooo`?o00008?ooool5o`0006Cooooo00?o0000ooooool000000_l000001?oooooo
0000ooooooooool2o`0000;ooooo0ol00003ooooo`03o`000?oooooooooo02kooooo0ol0000Roooo
o`008Oooool00ol0003oooooo`00000Nooooo`?o000000?oooooo`000?l00000IOooool00ol0003o
ooooo`000002o`000004ooooool0003oooooo`0000?ooooo0ol00003ooooo`03o`000?oooooooooo
037ooooo00?o0000ooooooooool08?ooool001oooooo0ol00002ooooo`03o`000?oooooooooo01[o
oooo0ol000000ooooooo0000o`00001Vooooo`03o`000?oooooo000000;o000000?oooooo`000?l0
00000oooool3o`0000?ooooo00?o0000ooooooooool0<_ooool3o`00023ooooo000Oooooo`?o0000
0_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo000000?o0000Ioooool00ol0003ooooo
o`000002o`000004ooooool0003oooooooooo`Co00000oooool00ol0003oooooooooo`0cooooo`?o
00008?ooool001oooooo0ol00003ooooo`03o`000?oooooooooo01Oooooo1Ol0001Yooooo`03o`00
0?oooooo000000Co000000?oooooo`000?l000000_l00003ooooo`03o`000?oooooooooo03Cooooo
0ol0000Pooooo`007Oooool3o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool05_oo
ool2o`000003ooooool0003o000006[ooooo00?o0000ooooool000000_l000000ooooooo0000o`00
0002o`0000?ooooo0_l0000iooooo`?o00007_ooool001gooooo0ol00002ooooo`05o`000?oooooo
ooooooooool000005Oooool3o`000003ooooool0003o000006_ooooo00?o0000ooooool000000_l0
00001?oooooo0000o`000?l00003ooooo`03o`000?oooooooooo03Wooooo0ol0000Nooooo`007Ooo
ool3o`0000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooo000000?o0000K?ooool00ol0003oooooo`000005o`0000?ooooo00?o0000ooooooooool0>_oo
ool3o`0001kooooo000Kooooo`?o000000?oooooo`000?ooool00_ooool01Ol0003ooooooooooooo
oooo000001;ooooo00Ko0000ooooool0003o0000ooooool0001]ooooo`Oo00000oooool00ol0003o
ooooooooo`0mooooo`?o00007?ooool001_ooooo0ol00002ooooo`03o`000?oooooooooo00;o0000
0oooool00ol0003oooooooooo`0>ooooo`Ko0000K_ooool00ol0003oooooo`000004o`0000;ooooo
00?o0000ooooooooool0?_ooool3o`0001cooooo000Kooooo`?o00000_ooool01Ol0003ooooooooo
oooooooo000000Cooooo00?o0000ooooooooool02oooool3o`000003ooooool0003o000006kooooo
1ol000001?oooooo0000o`000?l00011ooooo`?o00007?ooool001_ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool02_ooool6o`0006ko
oooo00?o0000ooooooooool01Ol000000ooooooo0000ooooo`14ooooo`03o`000?oooooooooo01[o
oooo000Iooooo`?o00001_ooool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool0
2?ooool4o`000003ooooool0003ooooo06gooooo00?o0000ooooooooool00_l000000ooooooo0000
o`000003o`0004Gooooo0ol0000Jooooo`006Oooool3o`0000Kooooo00Go0000oooooooooooooooo
o`000004ooooo`03o`000?oooooooooo00Kooooo00Ko0000ooooool0003o0000ooooool0001^oooo
o`03o`000?oooooooooo00Oo000000?oooooo`000?ooool0A?ooool3o`0001[ooooo000Iooooo`?o
00001oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01?ooool5o`00
0003ooooool0003ooooo06gooooo00?o0000ooooooooool00_l000000ooooooo0000ooooo`03o`00
0003ooooool0003ooooo04Cooooo0ol0000Jooooo`006Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00;o00000oooool00ol0003oooooooooo`02ooooo`Oo0000K_ooool00ol0003o
ooooo`000005o`000006ooooool0003o0000o`000?oooooo0000Aoooool00ol0003oooooooooo`0H
ooooo`005oooool3o`0000[ooooo00Co0000oooooooooooooooo0_l00002ooooo`04o`000?oooooo
ooooooooo`Co000000?oooooo`000?ooool0K_ooool01?l0003oooooo`000?ooool2o`0000;ooooo
1Ol000000ooooooo0000ooooo`16ooooo`?o00006?ooool001Oooooo0ol0000;ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooo`000003o`000003ooooool0003ooooo06kooooo
00Co0000ooooool0003ooooo0_l00002ooooo`;o000000Koooooo`000?l0003o0000ooooool00017
ooooo`?o00006?ooool001Oooooo0ol0000<ooooo`03o`000?oooooooooo00;o00000_ooool5o`00
0003ooooool0003ooooo06kooooo00?o0000ooooool000000ol00002ooooo`04o`000?oooooo0000
ooooo`?o000000?oooooo`000?ooool0A_ooool3o`0001Sooooo000Eooooo`?o00003oooool00ol0
003oooooooooo`07o`000003ooooool0003ooooo06kooooo00Ko0000ooooool0003o0000ooooool0
0002ooooo`05o`000?ooooooooooo`000?ooool00ol000000ooooooo0000ooooo`17ooooo`?o0000
5oooool001Gooooo0ol0000?ooooo`03o`000?oooooooooo00Ko000000?oooooo`000?ooool0KOoo
ool2o`000005ooooool0003o0000ooooool000000_ooool01_l0003ooooooooooooooooo0000oooo
o`?o000000?oooooo`000?ooool0Aoooool3o`0001Oooooo000Eooooo`?o00004?ooool00ol0003o
ooooo`000003o`000003ooooool0003o000006kooooo00?o0000ooooool000000_l000001Ooooooo
0000oooooooooooo000000Cooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`17oooo
o`?o00005oooool001Kooooo00?o0000ooooooooool03oooool7o`000003ooooool0003ooooo06co
oooo00So0000ooooool0003oooooo`000?oooooo0000ooooo`;o00001Oooool00ol0003oooooo`00
0002o`000003ooooool0003ooooo04Wooooo00?o0000ooooooooool05Oooool001Cooooo0ol0000@
ooooo`Co000000?oooooo`000?l000000oooool00ol0003oooooooooo`1Yooooo`09o`000?oooooo
0000ooooool0003oooooo`000?oooooo000000Oooooo00?o0000ooooool000000_l000000ooooooo
0000ooooo`19ooooo`?o00005Oooool001Cooooo0ol0000?ooooo`Go000000Coooooo`000?l0003o
00000oooool00ol0003oooooooooo`1Wooooo`09o`000?oooooo0000ooooool0003oooooo`000?oo
oooo000000Sooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`19ooooo`?o00005Ooo
ool001Cooooo0ol0000>ooooo`Co00000_ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000
ooooooooool0I_ooool00ol0003oooooo`000004o`000003ooooool0003ooooo00Sooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`19ooooo`?o00005Oooool001;ooooo0ol0000>oooo
o`Go000000Coooooo`000?l0003o00000_ooool2o`0000?ooooo00?o0000ooooooooool0I?ooool0
1_l0003oooooo`000?l0003oooooo`0000;ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo04[ooooo0ol0000Dooooo`004_ooool3o`0000gooooo00Co
0000ooooool0003o00000_ooool01Ol0003ooooooooooooooooo000000;ooooo0_l00002ooooo`03
o`000?oooooooooo06?ooooo1?l000001Ooooooo0000oooooooooooo000000_ooooo0_l000001Ooo
oooo0000o`000?oooooo000004_ooooo0ol0000Dooooo`004_ooool3o`0000cooooo00Co0000oooo
ool0003o00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o00000_ooool0
0ol0003oooooooooo`1Qooooo`;o000000Goooooo`000?oooooo0000ooooo`02o`0000cooooo0_l0
00001?oooooo0000o`000?l0001<ooooo`?o00005?ooool001?ooooo00?o0000ooooooooool02_oo
ool4o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02o`0000?ooooo00?o
0000ooooooooool0Goooool2o`000006ooooool0003oooooo`000?oooooo00003_ooool2o`000004
ooooool0003o0000o`0004kooooo00?o0000ooooooooool04_ooool0017ooooo0ol0000;ooooo`Co
00000_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003o
ooooooooo`1Mooooo`?o000000Koooooo`000?oooooo0000ooooool0000?ooooo`;o000000Cooooo
o`000?l0003o0000C_ooool3o`0001;ooooo000Aooooo`?o00002Oooool4o`0000?ooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool0Foooool0
2Ol0003oooooo`000?oooooo0000ooooool0003oooooo`00000@ooooo`;o000000Coooooo`000?l0
003o0000C_ooool3o`0001;ooooo000Aooooo`?o00002?ooool01?l0003oooooo`000?l00002oooo
o`;o00002oooool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool0FOooool02Ol0
003oooooo`000?oooooo0000ooooool0003oooooo`00000Aooooo`;o000000Coooooo`000?l0003o
0000C_ooool3o`0001;ooooo000Booooo`03o`000?oooooooooo00Kooooo00Co0000ooooool0003o
00000_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003o
ooooooooo`1Hooooo`Go000000Coooooo`000?l0003o00004_ooool2o`000004ooooool0003o0000
o`0004oooooo00?o0000ooooooooool04Oooool0013ooooo0ol00007ooooo`04o`000?oooooo0000
o`0000;ooooo00?o0000ooooooooool03_ooool00ol0003oooooo`000002o`0000;ooooo00?o0000
ooooooooool0E_ooool2o`000004ooooool0003oooooooooo`;o00005?ooool2o`000004ooooool0
003o0000o`0004oooooo0ol0000Aooooo`004?ooool3o`0000Gooooo1?l00003ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool00_l00003ooooo`03o`000?oooooooooo05Cooooo0_l0
00001?oooooo0000ooooooooool2o`0001Gooooo0_l000001?oooooo0000o`000?l0001?ooooo`?o
00004Oooool0013ooooo0ol00004ooooo`04o`000?oooooo0000o`0000?ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool0Doooool2o`000003
ooooool0003ooooo00?o00005_ooool2o`000004ooooool0003o0000o`0004oooooo0ol0000Aoooo
o`004Oooool00ol0003oooooooooo`02ooooo`04o`000?oooooo0000o`0000?ooooo00?o0000oooo
ooooool04oooool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool0DOooool2o`00
0006ooooool0003oooooo`000?oooooo00005oooool2o`000004ooooool0003o0000o`00053ooooo
00?o0000ooooooooool04?ooool000oooooo0ol00003ooooo`04o`000?oooooo0000o`0000;ooooo
0_l0000Fooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`1@ooooo`;o0000
00Koooooo`000?oooooo0000ooooool0000Hooooo`;o000000Coooooo`000?l0003o0000D?ooool3
o`00013ooooo000?ooooo`?o000000Goooooo`000?l0003oooooo`000003ooooo`03o`000?oooooo
oooo01Oooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo04gooooo0ol00000
0ooooooo0000ooooo`03o`0001Wooooo0_l000001?oooooo0000o`000?l0001@ooooo`?o00004?oo
ool000oooooo1?l000000ooooooo0000o`000003ooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooool000000_l00002ooooo`03o`000?oooooooooo04_ooooo0_l000001?oooooo0000o`000?oo
ool2o`0001_ooooo0_l000001?oooooo0000o`000?l0001@ooooo`?o00004?ooool0013ooooo0_l0
00000ooooooo0000o`000003ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool00ol0
00000ooooooo0000ooooo`1;ooooo`;o000000Coooooo`000?oooooooooo0_l0000Looooo`?o0000
00?oooooo`000?l00000DOooool00ol0003oooooooooo`0?ooooo`003_ooool3o`000003ooooool0
003o000000?ooooo00?o0000ooooooooool07?ooool00ol0003oooooo`000002o`0000;ooooo00?o
0000ooooooooool0B?ooool2o`000004ooooool0003oooooooooo`;o00007Oooool3o`000003oooo
ool0003o0000057ooooo0ol0000?ooooo`003_ooool4o`0000Cooooo00?o0000ooooooooool07_oo
ool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool0A_ooool2o`000004ooooool0
003oooooooooo`;o00007_ooool3o`000003ooooool0003o0000057ooooo0ol0000?ooooo`003_oo
ool3o`0000?ooooo0_l0000Rooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0A_oo
ool2o`000004ooooool0003oooooooooo`;o00007oooool3o`000003ooooool0003o0000057ooooo
0ol0000?ooooo`003_ooool2o`0000?ooooo00?o0000ooooooooool08_ooool00ol0003oooooo`00
0002o`0000;ooooo00?o0000ooooooooool0@oooool2o`000004ooooool0003oooooooooo`;o0000
8?ooool3o`000003ooooool0003o000005;ooooo0_l0000?ooooo`003Oooool3o`0000;ooooo00?o
0000ooooooooool09?ooool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool0@Ooo
ool01Ol0003oooooooooool0003ooooo00;o00008_ooool3o`000003ooooool0003o000005;ooooo
0ol0000>ooooo`003Oooool3o`000003ooooool0003ooooo02Oooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`11ooooo`05o`000?ooooooooooo`000?ooool00_l0000Sooooo`?o0000
00?oooooo`000?l00000D_ooool3o`0000kooooo000=ooooo`Co0000:Oooool00ol0003oooooo`00
0002o`0000;ooooo00?o0000ooooooooool0?_ooool01Ol0003oooooo`000?l0003ooooo00;o0000
9?ooool3o`000003ooooool0003o000005;ooooo0ol0000>ooooo`003Oooool3o`0002_ooooo00?o
0000ooooool000000_l000000ooooooo0000ooooo`0mooooo`;o000000Coooooo`000?oooooooooo
0_l0000Uooooo`?o000000?oooooo`000?l00000D_ooool3o`0000kooooo000=ooooo`?o0000;?oo
ool00ol0003oooooo`000002o`000003ooooool0003ooooo03_ooooo0_l000001?oooooo0000oooo
ooooool2o`0002Kooooo0ol000000ooooooo0000o`00001Booooo`?o00003_ooool000gooooo0ol0
000/ooooo`03o`000?oooooo000000;o00000_ooool00ol0003oooooooooo`0hooooo`;o000000Co
ooooo`000?oooooooooo0_l0000Wooooo`?o000000?oooooo`000?l00000D_ooool3o`0000kooooo
000=ooooo`;o0000;_ooool00ol0003oooooo`000002o`000003ooooool0003ooooo03Sooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`0Wooooo`?o000000?oooooo`000?l00000
Doooool2o`0000kooooo000<ooooo`?o0000;oooool2o`000005ooooool0003o0000ooooool00000
=oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo02Sooooo0ol000000ooooooo
0000o`00001Cooooo`?o00003Oooool000cooooo0ol0000_ooooo`03o`000?oooooo000000;o0000
0_ooool00ol0003oooooooooo`0cooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooooooo
ool0:Oooool3o`000003ooooool0003o000005?ooooo0ol0000=ooooo`003?ooool3o`00033ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`0cooooo`04o`000?oooooo0000o`0000;o
oooo00?o0000ooooooooool0:_ooool3o`000003ooooool0003o000005?ooooo0ol0000=ooooo`00
3Oooool00ol0003oooooooooo`0`ooooo`;o000000Goooooo`000?l0003oooooo`00000aooooo`;o
000000Coooooo`000?oooooooooo0_l0000]ooooo`?o000000?oooooo`000?l00000E?ooool00ol0
003oooooooooo`0<ooooo`003?ooool3o`00037ooooo00?o0000ooooool000000_l00002ooooo`03
o`000?oooooooooo02gooooo0_l000001?oooooo0000ooooooooool2o`0002kooooo00Co0000oooo
ool0003ooooo0_l0001Cooooo`?o00003Oooool000cooooo0ol0000booooo`03o`000?oooooo0000
00;o000000?oooooo`000?ooool0;Oooool2o`000005ooooool0003oooooooooool00000<?ooool0
1Ol0003oooooo`000?oooooo000005Cooooo0ol0000=ooooo`003?ooool3o`0003?ooooo0_l00000
1Ooooooo0000o`000?oooooo000002cooooo0_l000001Ooooooo0000oooooooooooo0000037ooooo
00Go0000ooooool0003oooooo`00001Dooooo`?o00003Oooool000cooooo0_l0000dooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool0:_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo033ooooo00Go0000ooooool0003oooooo`00001Eooooo`;o00003Oooool000_o
oooo0ol0000eooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0:?ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo037ooooo00Go0000ooooool0003oooooo`00001E
ooooo`?o00003?ooool000_ooooo0ol0000fooooo`;o000000Goooooo`000?l0003oooooo`00000V
ooooo`;o000000Coooooo`000?l0003ooooo0_l0000dooooo`05o`000?oooooo0000ooooool00000
EOooool3o`0000cooooo000;ooooo`?o0000=_ooool00ol0003oooooo`000002o`000003ooooool0
003ooooo02Cooooo0_l000001?oooooo0000ooooooooool2o`0003Gooooo00Go0000ooooool0003o
ooooo`00001Eooooo`?o00003?ooool000cooooo00?o0000ooooooooool0=_ooool2o`000005oooo
ool0003o0000ooooool000008oooool2o`000004ooooool0003oooooooooo`;o0000=_ooool01Ol0
003oooooo`000?oooooo000005Kooooo00?o0000ooooooooool02oooool000_ooooo0ol0000hoooo
o`;o000000Goooooo`000?l0003oooooo`00000Qooooo`;o000000Coooooo`000?oooooooooo0_l0
000gooooo`05o`000?oooooo0000ooooool00000EOooool3o`0000cooooo000;ooooo`?o0000>Ooo
ool3o`000003ooooool0003o0000023ooooo0_l000001?oooooo0000ooooooooool2o`0003Sooooo
00Go0000ooooool0003oooooo`00001Eooooo`?o00003?ooool000_ooooo0ol0000iooooo`;o0000
00Goooooo`000?l0003oooooo`00000Nooooo`;o000000Coooooo`000?oooooooooo0_l0000ioooo
o`05o`000?oooooo0000ooooool00000EOooool3o`0000cooooo000<ooooo`03o`000?oooooooooo
03Wooooo0_l000001Ooooooo0000o`000?oooooo000001_ooooo0ol000000ooooooo0000ooooo`02
o`0003_ooooo00Go0000ooooool0003oooooo`00001Fooooo`03o`000?oooooooooo00_ooooo000;
ooooo`?o0000>oooool3o`000003ooooool0003o000001[ooooo0ol000000ooooooo0000ooooo`02
o`0003cooooo00Go0000ooooool0003oooooo`00001Eooooo`?o00003?ooool000_ooooo0ol0000k
ooooo`;o000000Goooooo`000?l0003oooooo`00000Hooooo`;o000000Coooooo`000?l0003ooooo
0_l0000mooooo`05o`000?oooooo0000ooooool00000EOooool3o`0000cooooo000;ooooo`?o0000
??ooool2o`000004ooooool0003o0000o`0001Oooooo0_l000001?oooooo0000ooooooooool2o`00
03kooooo00Go0000ooooool0003oooooo`00001Eooooo`?o00003?ooool000cooooo00?o0000oooo
ooooool0??ooool3o`000003ooooool0003o000001Gooooo0_l000001?oooooo0000ooooooooool2
o`0003oooooo00Go0000ooooool0003oooooo`00001Fooooo`03o`000?oooooooooo00_ooooo000;
ooooo`?o0000?Oooool2o`000005ooooool0003o0000ooooool000004oooool2o`000004ooooool0
003oooooooooo`;o0000@?ooool01Ol0003oooooo`000?oooooo000005Gooooo0ol0000<ooooo`00
2oooool:o`0003Oooooo0_l000001?oooooo0000o`000?l0000Aooooo`?o000000?oooooo`000?oo
ool00ol00011ooooo`05o`000?oooooo0000ooooool00000EOooool3o`0000cooooo000;ooooo`?o
00001oooool?o`0002Wooooo00Co0000ooooool0003ooooo0_l0000?ooooo`09o`000?oooooo0000
ooooool0003oooooo`000?oooooo000004;ooooo00Go0000ooooool0003oooooo`00001Eooooo`?o
00003?ooool000cooooo00?o0000ooooooooool05Oooool@o`0001Wooooo0_l000001Ooooooo0000
o`000?oooooo000000gooooo00Wo0000ooooool0003oooooo`000?oooooo0000ooooool00000@ooo
ool01Ol0003oooooo`000?oooooo000005Kooooo00?o0000ooooooooool02oooool000_ooooo0ol0
000Voooooa3o00002_ooool3o`000003ooooool0003o000000cooooo00?o0000ooooool000000_l0
00001?oooooo0000ooooool00014ooooo`05o`000?oooooo0000ooooool00000EOooool3o`0000co
oooo000;oooooa;o00009ooooolAo`0000[ooooo1?l00002ooooo`?o0000AOooool01Ol0003ooooo
o`000?oooooo000005Gooooo0ol0000<ooooo`002oooool3o`0000Oooooo9ol0000Cooooo`;o0000
00?oooooo`000?l000004_l00017ooooo`05o`000?oooooo0000ooooool00000EOooool3o`0000co
oooo000<ooooo`;o00005_oooolgo`000007ooooool0003o0000o`000?oooooo0000ooooo`0Ao`00
03[ooooo00Go0000ooooool0003oooooo`00001Eooooo`;o00003Oooool000cooooo0ol0000Uoooo
ob?o00001?oooolWo`0002_ooooo00Go0000ooooool0003oooooo`00001Dooooo`?o00003Oooool0
00cooooo4_l0000Vooooo`oo000000?oooooo`000?l000005Ol0000@oooooaoo00004oooool01Ol0
003oooooo`000?oooooo000005Cooooo0ol0000=ooooo`003?ooool3o`0000oooooo7ol0000Eoooo
oa7o00002?oooolEo`0001;ooooo:?l0001<ooooo`?o00003Oooool000gooooo00?o0000oooooooo
ool0;OoooolPo`000004ooooool0003oooooo`0000;ooooo4?l0000=oooooaCo00003OoooolAo`00
00Sooooo7ol0000^ooooo`03o`000?oooooooooo00cooooo000<ooooo`?o0000A?ooool2o`000003
ooooool0003o000002[o00004_oooolEo`0000Cooooo00Co0000ooooool0003ooooo4?l0000Hoooo
oaoo00003_ooool3o`0000gooooo000<ooooo`?o0000AOooool3o`000005ooooool0003o0000oooo
ool000000_ooool00ol0003oooooooooo`0Joooooaoo00003ooooolEo`0000?ooooo3ol0000Xoooo
oa7o00003Oooool000cooooo0ol00015ooooo`;o000000?oooooo`000?l000000_l00002ooooo`03
o`000?oooooooooo03;ooooo9ol00005ooooob;o00009_ooool3o`0000gooooo000=ooooo`;o0000
A?ooool00ol0003oooooo`000007o`0004Cooooo4Ol00007ooooocOo00005_ooool2o`0000kooooo
000=ooooo`?o0000@_ooool02?l0003oooooo`000?oooooo0000ooooool0003ooooo0_l0001@oooo
o`;o000000?oooooo`000?l000004?l0000GooooobOo00001_ooool3o`0000kooooo000=ooooo`?o
0000@Oooool02_l0003oooooo`000?oooooo0000ooooool0003oooooo`000?ooool2o`0004oooooo
0_l000001?oooooo0000o`000?l0000?ooooo`oo00009ooooolAo`0000kooooo000=ooooo`?o0000
@?ooool01?l0003oooooo`000?ooool3o`000006ooooool0003oooooo`000?oooooo0000Coooool2
o`000004ooooool0003o0000o`0001kooooo4?l0000Uooooo`?o00003_ooool000kooooo00?o0000
ooooooooool0?Oooool2o`000003ooooool0003o000000;o00000_ooool2o`000003ooooool0003o
oooo00;o0000C_ooool2o`000004ooooool0003o0000o`0002kooooo4?l0000Fooooo`03o`000?oo
oooooooo00gooooo000=ooooo`?o0000?Oooool00ol0003oooooooooo`02o`000005ooooool0003o
ooooooooool000000_ooool01Ol0003oooooo`000?oooooo000004kooooo0_l000001?oooooo0000
o`000?l0000nooooo`oo00001_ooool3o`0000kooooo000=ooooo`?o0000??ooool00ol0003ooooo
ooooo`02o`000005ooooool0003oooooooooool000001?ooool01Ol0003oooooo`000?oooooo0000
04gooooo0_l000001?oooooo0000o`000?l0001=ooooo`Wo00003_ooool000gooooo0ol0000koooo
o`03o`000?oooooooooo00;o000000Goooooo`000?ooooooooooo`000005ooooo`04o`000?oooooo
0000ooooo`;o0000C?ooool2o`000004ooooool0003o0000o`0005?ooooo0ol0000>ooooo`003_oo
ool2o`0003[ooooo00?o0000ooooooooool00_l000001Ooooooo0000oooooooooooo000000Oooooo
00Go0000ooooool0003oooooo`00001<ooooo`;o000000Coooooo`000?l0003o0000Doooool2o`00
00oooooo000>ooooo`?o0000>?ooool00ol0003oooooooooo`04o`0000;ooooo00?o0000oooooooo
ool01_ooool2o`000004ooooool0003oooooo`0004_ooooo0_l000001?oooooo0000o`000?l0001B
ooooo`?o00003oooool000kooooo0ol0000fooooo`;o000000?oooooo`000?l000000_l00003oooo
o`03o`000?oooooooooo00Sooooo00Co0000ooooool0003ooooo0_l0001:ooooo`;o000000Cooooo
o`000?l0003o0000D_ooool3o`0000oooooo000>ooooo`?o0000=Oooool01Ol0003oooooooooool0
003ooooo00;o00000_ooool2o`0000cooooo00Go0000ooooool0003oooooo`00001:ooooo`;o0000
00Coooooo`000?l0003o0000D_ooool3o`0000oooooo000?ooooo`?o0000<oooool00ol0003ooooo
ooooo`04o`0000;ooooo00?o0000ooooooooool03?ooool2o`000004ooooool0003oooooo`0004Wo
oooo0_l000001?oooooo0000o`000?l0001Aooooo`?o00004?ooool000oooooo0ol0000booooo`03
o`000?oooooooooo00;o000000Goooooo`000?ooooooooooo`00000@ooooo`04o`000?oooooo0000
ooooo`;o0000B?ooool2o`000004ooooool0003o0000o`00057ooooo0ol0000@ooooo`003oooool3
o`00037ooooo00?o0000ooooooooool01?l00002ooooo`03o`000?oooooooooo013ooooo00Go0000
ooooool0003oooooo`000018ooooo`?o000000?oooooo`000?l00000DOooool3o`00013ooooo000@
ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool00ol00003ooooo`03o`000?oooooo
oooo017ooooo0_l000001?oooooo0000ooooool00017ooooo`?o000000?oooooo`000?l00000D_oo
ool00ol0003oooooooooo`0?ooooo`004?ooool3o`0002gooooo0_l00002ooooo`?o00000oooool0
0ol0003oooooooooo`0Cooooo`05o`000?oooooo0000ooooool00000Aoooool3o`000003ooooool0
003o0000053ooooo0ol0000Aooooo`004?ooool3o`0002cooooo00Co0000oooooooooooooooo0ol0
0003ooooo`03o`000?oooooooooo01Cooooo0_l000001?oooooo0000ooooool00016ooooo`?o0000
00?oooooo`000?l00000D?ooool3o`00017ooooo000@ooooo`?o0000:oooool01?l0003ooooooooo
ooooool3o`0000?ooooo00?o0000ooooooooool05_ooool2o`000004ooooool0003oooooo`0004Go
oooo0ol000000ooooooo0000o`00001@ooooo`?o00004Oooool0017ooooo0_l0000Zooooo`04o`00
0?ooooooooooooooo`?o00000oooool00ol0003oooooooooo`0Hooooo`05o`000?oooooo0000oooo
ool00000AOooool3o`000003ooooool0003o0000053ooooo0_l0000Booooo`004Oooool3o`0002So
oooo00?o0000ooooooooool01?l00003ooooo`03o`000?oooooooooo01Wooooo0_l000001?oooooo
0000ooooool00014ooooo`?o000000?oooooo`000?l00000Coooool3o`0001;ooooo000Aooooo`?o
00009oooool00ol0003oooooooooo`03o`0000?ooooo0_l0000Mooooo`;o000000Coooooo`000?oo
oooo0000@oooool3o`000003ooooool0003o000004oooooo0ol0000Booooo`004Oooool3o`0002Go
oooo0_l00002ooooo`?o00000oooool00ol0003oooooooooo`0Nooooo`05o`000?oooooo0000oooo
ool00000@oooool3o`000003ooooool0003o000004oooooo0ol0000Booooo`004oooool00ol0003o
ooooooooo`0Rooooo`04o`000?ooooooooooooooo`?o00000oooool00ol0003oooooooooo`0Ooooo
o`;o000000Coooooo`000?oooooo0000@_ooool3o`000003ooooool0003o000004oooooo00?o0000
ooooooooool04_ooool001;ooooo0ol0000Rooooo`04o`000?ooooooooooooooo`?o00000oooool0
0ol0003oooooooooo`0Qooooo`;o000000Coooooo`000?oooooo0000@Oooool3o`000003ooooool0
003o000004gooooo0ol0000Dooooo`004_ooool3o`00027ooooo00Co0000oooooooooooooooo0ol0
0003ooooo`03o`000?oooooooooo02?ooooo0ol000000ooooooo0000o`000010ooooo`?o000000?o
ooooo`000?l00000COooool3o`0001Cooooo000Booooo`?o00008?ooool01?l0003ooooooooooooo
ool3o`0000?ooooo00?o0000ooooooooool09?ooool2o`000004ooooool0003oooooo`00043ooooo
0ol000000ooooooo0000o`00001=ooooo`?o00005?ooool001Cooooo0ol0000Mooooo`04o`000?oo
ooooooooooooo`;o00001?ooool00ol0003oooooooooo`0Vooooo`;o000000Coooooo`000?l0003o
0000?oooool3o`000003ooooool0003o000004cooooo0ol0000Eooooo`005?ooool3o`0001_ooooo
0_l00003ooooo`;o00001?ooool00ol0003oooooooooo`0Wooooo`05o`000?oooooo0000o`000?oo
ool00_l0000nooooo`?o000000?oooooo`000?l00000C?ooool3o`0001Gooooo000Dooooo`?o0000
6_ooool01?l0003oooooooooooooool3o`0000Cooooo00?o0000ooooooooool0:Oooool2o`000004
ooooool0003oooooo`0003gooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`1;oooo
o`?o00005Oooool001Gooooo0_l0000Iooooo`04o`000?ooooooooooooooo`?o00001?ooool00ol0
003oooooooooo`0[ooooo`;o000000Coooooo`000?l0003o0000??ooool00ol0003oooooo`000002
o`000003ooooool0003ooooo04_ooooo0_l0000Fooooo`005Oooool3o`0001Oooooo00Co0000oooo
oooooooooooo0ol00004ooooo`03o`000?oooooooooo02cooooo00Go0000ooooool0003o0000oooo
o`02o`0003_ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`19ooooo`?o00005ooo
ool001Gooooo0ol0000Fooooo`04o`000?ooooooooooooooo`?o00000oooool2o`00033ooooo0_l0
00001?oooooo0000ooooool0000kooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0
BOooool3o`0001Oooooo000Eooooo`?o00005Oooool01?l0003oooooooooooooool2o`0000Cooooo
00?o0000ooooooooool0<Oooool2o`000004ooooool0003o0000o`0003[ooooo00?o0000ooooool0
00000_l000000ooooooo0000ooooo`19ooooo`?o00005oooool001Oooooo00?o0000ooooooooool0
4Oooool2o`0000?ooooo0_l00004ooooo`03o`000?oooooooooo03;ooooo00Go0000ooooool0003o
0000ooooo`02o`0003Wooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`1:ooooo`03
o`000?oooooooooo01Kooooo000Gooooo`?o00004?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`0dooooo`;o000000Coooooo`000?oooooo0000>Oooool00ol0003o
ooooo`000002o`000003ooooool0003ooooo04Sooooo0ol0000Hooooo`005oooool3o`0000oooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0=_ooool3o`000003oooo
ool0003o000003Sooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`18ooooo`?o0000
6?ooool001Oooooo0ol0000>ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oo
oooooooo03Oooooo00Go0000ooooool0003o0000ooooo`02o`0003Oooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`18ooooo`?o00006?ooool001Wooooo0ol0000;ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0iooooo`;o000000Co
ooooo`000?l0003o0000=oooool00ol0003oooooo`000002o`000003ooooool0003ooooo04Kooooo
0ol0000Jooooo`006Oooool3o`0000[ooooo00Co0000oooooooooooooooo0_l00005ooooo`03o`00
0?oooooooooo03[ooooo00Go0000ooooool0003o0000ooooo`02o`0003Kooooo00?o0000ooooool0
00000_l000000ooooooo0000ooooo`16ooooo`?o00006_ooool001Wooooo0ol00008ooooo`;o0000
0oooool2o`0000Gooooo00?o0000ooooooooool0??ooool01Ol0003oooooo`000?l0003ooooo00;o
0000=Oooool00ol0003oooooo`000002o`000003ooooool0003ooooo04Kooooo0ol0000Jooooo`00
6oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0_l00005ooooo`03o`00
0?oooooooooo03kooooo0_l000001?oooooo0000o`000?l0000eooooo`03o`000?oooooo000000;o
000000?oooooo`000?ooool0A_ooool00ol0003oooooooooo`0Jooooo`006oooool3o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo0_l00011ooooo`05o`000?oooooo0000o`000?oo
ool00_l0000dooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0A?ooool3o`0001co
oooo000Kooooo`?o00000oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`12ooooo`05o`000?oooooo0000o`000?ooool00_l0000cooooo`03o`000?oooooo000000;o
000000?oooooo`000?ooool0A?ooool3o`0001cooooo000Kooooo`?o00000_ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0A?ooool2o`000004oooo
ool0003o0000o`0003?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`14ooooo`?o
00007?ooool001gooooo0ol00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
AOooool01Ol0003oooooo`000?l0003ooooo00;o0000<_ooool00ol0003oooooo`000002o`000003
ooooool0003ooooo04;ooooo0ol0000Nooooo`007Oooool3o`0000?ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`17ooooo`05o`000?oooooo0000o`000?ooool00_l0000aooooo`03
o`000?oooooo000000;o000000?oooooo`000?ooool0@_ooool3o`0001kooooo000Mooooo`?o0000
0_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo04Wooooo0_l000001Ooooooo0000
o`000?oooooo0000033ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`12ooooo`?o
00007_ooool001oooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`1:ooooo`05o`00
0?oooooo0000o`000?ooool00_l0000`ooooo`03o`000?oooooo000000;o000000?oooooo`000?oo
ool0@_ooool00ol0003oooooooooo`0Nooooo`007oooool3o`0000Cooooo00?o0000ooooooooool0
C?ooool00ol0003oooooo`000002o`000003ooooool0003ooooo02kooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`10ooooo`?o00008?ooool001oooooo0ol00003ooooo`03o`000?oo
oooooooo04gooooo00?o0000ooooooooool00ol000000ooooooo0000ooooo`0]ooooo`03o`000?oo
oooo000000;o000000?oooooo`000?ooool0@?ooool3o`00023ooooo000Oooooo`?o00000_ooool0
0ol0003oooooooooo`1?ooooo`05o`000?oooooo0000o`000?ooool00_l0000^ooooo`03o`000?oo
oooo000000;o000000?oooooo`000?ooool0@?ooool3o`00023ooooo000Qooooo`?o0000Doooool0
0ol0003oooooo`000002o`000003ooooool0003ooooo02cooooo00?o0000ooooool000000_l00000
0ooooooo0000ooooo`0nooooo`?o00008_ooool0027ooooo0ol0001Cooooo`03o`000?oooooooooo
00?o000000?oooooo`000?ooool0:oooool00ol0003oooooo`000002o`000003ooooool0003ooooo
03kooooo0ol0000Rooooo`008Oooool3o`0005Cooooo00Go0000ooooool0003o0000ooooo`02o`00
02cooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`0nooooo`?o00008_ooool002?o
oooo0ol0001Cooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0:_ooool00ol0003o
ooooo`000002o`000003ooooool0003ooooo03cooooo0ol0000Tooooo`008oooool3o`0005?ooooo
00?o0000ooooooooool00ol000000ooooooo0000ooooo`0Yooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool0??ooool3o`0002Cooooo000Sooooo`?o0000E?ooool01Ol0003oooooo`00
0?l0003ooooo00;o0000:_ooool00ol0003oooooo`000002o`000003ooooool0003ooooo03cooooo
0ol0000Tooooo`009Oooool3o`0005?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`0Xooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0>Oooool4o`0002Kooooo000U
ooooo`?o0000Doooool00ol0003oooooooooo`03o`000003ooooool0003ooooo02Oooooo00Co0000
ooooool0003o00000_ooool00ol0003oooooooooo`0hooooo`?o00009oooool002Gooooo0ol0001D
ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool09_ooool00ol0003oooooooooo`02
o`0000;ooooo00?o0000ooooooooool0>?ooool3o`0002Oooooo000Xooooo`?o0000DOooool00ol0
003oooooooooo`03o`000003ooooool0003ooooo02Gooooo00?o0000ooooooooool00_l00002oooo
o`03o`000?oooooooooo03Kooooo0ol0000Yooooo`00:?ooool3o`0005;ooooo00?o0000oooooooo
ool00ol000000ooooooo0000ooooo`0Tooooo`03o`000?oooooooooo00;o00000_ooool00ol0003o
ooooooooo`0fooooo`?o0000:Oooool002Sooooo0ol0001Cooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool09?ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool0
=_ooool3o`0002Wooooo000Zooooo`?o0000DOooool00ol0003oooooooooo`03o`000003ooooool0
003ooooo02?ooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo03?ooooo1?l0
000[ooooo`00:_ooool3o`0005;ooooo00?o0000ooooooooool00ol000000ooooooo0000ooooo`0R
ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0cooooo`?o0000;?ooool0
02[ooooo0ol0001Cooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool08_ooool00ol0
003oooooooooo`02o`0000;ooooo00?o0000ooooooooool0<oooool3o`0002cooooo000]ooooo`?o
0000D?ooool00ol0003oooooooooo`03o`000003ooooool0003ooooo027ooooo00?o0000oooooooo
ool00_l00002ooooo`03o`000?oooooooooo037ooooo0ol0000^ooooo`00;Oooool3o`00057ooooo
00?o0000ooooooooool00ol000000ooooooo0000ooooo`0Pooooo`03o`000?oooooooooo00;o0000
0_ooool00ol0003oooooooooo`0aooooo`?o0000;_ooool002gooooo0ol0001Booooo`03o`000?oo
oooo000000;o000000?oooooo`000?ooool08?ooool00ol0003oooooooooo`02o`0000;ooooo00?o
0000ooooooooool0<Oooool3o`0002kooooo000`ooooo`?o0000Coooool00ol0003oooooooooo`03
o`000003ooooool0003ooooo01oooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooo
oooo02kooooo0ol0000aooooo`00<?ooool3o`00053ooooo00?o0000ooooooooool00_l00002oooo
o`03o`000?oooooooooo01gooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo
02kooooo0ol0000aooooo`00<?ooool4o`0004oooooo00Co0000oooooooooooooooo0_l00002oooo
o`03o`000?oooooooooo01cooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo
02gooooo1?l0000aooooo`00<oooool3o`0004kooooo00?o0000ooooooooool00ol000000ooooooo
0000ooooo`0Mooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0[ooooo`?o
0000=?ooool003?ooooo0ol0001?ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003ooooo
ooooo`0Kooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0[ooooo`?o0000
=?ooool003?ooooo1?l0001>ooooo`04o`000?ooooooooooooooo`;o00000_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0Zooooo`Co0000
=?ooool003Kooooo0ol0001=ooooo`03o`000?oooooooooo00?o000000?oooooo`000?ooool06ooo
ool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool0:?ooool3o`0003Oooooo000f
ooooo`Co0000COooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool09oooool4o`0003Oooooo000foooo
o`Ko0000Boooool01?l0003oooooooooooooool2o`0000;ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool09Oooool6o`0003Oooooo000ioooo
o`?o0000C?ooool00ol0003oooooooooo`03o`000003ooooool0003ooooo01Wooooo00?o0000oooo
ooooool00_l00002ooooo`03o`000?oooooooooo02Gooooo0ol0000jooooo`00>Oooool4o`0004co
oooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo01Oooooo00?o0000oooooooo
ool00_l00002ooooo`03o`000?oooooooooo02Cooooo1?l0000jooooo`00??ooool3o`0004[ooooo
00Co0000oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool00_l00002ooooo`03o`000?oooooooooo02;ooooo0ol0000mooooo`00??ooool4o`0004[ooooo
00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0
0_l00002ooooo`03o`000?oooooooooo027ooooo1?l0000mooooo`00??ooool6o`0004Sooooo00Co
0000oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0
0_l00002ooooo`03o`000?oooooooooo01oooooo1_l0000mooooo`00?oooool3o`0004Wooooo00Co
0000oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
0_l00002ooooo`03o`000?oooooooooo01oooooo0ol00010ooooo`00?oooool6o`0004Oooooo00?o
0000ooooooooool00_l00002ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool00_l0
0002ooooo`03o`000?oooooooooo01cooooo1_l00010ooooo`00@_ooool3o`0004Oooooo00Co0000
oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool00_l0
0002ooooo`03o`000?oooooooooo01cooooo0ol00013ooooo`00@_ooool6o`0004Gooooo00Co0000
oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool00_l0
0002ooooo`03o`000?oooooooooo01Sooooo1ol00013ooooo`00AOooool4o`0004Gooooo00?o0000
ooooooooool00_l00002ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool00_l00002
ooooo`03o`000?oooooooooo01Sooooo0ol00017ooooo`00AOooool7o`0004;ooooo00Co0000oooo
oooooooooooo0_l00002ooooo`03o`000?oooooooooo017ooooo00Co0000oooooooooooo00000ooo
ool00ol0003oooooooooo`0Eooooo`Ko0000Aoooool004Wooooo0ol00013ooooo`04o`000?oooooo
ooooooooo`;o00000_ooool00ol0003oooooooooo`0?ooooo`05o`000?ooooooooooooooool00000
0oooool00ol0003oooooooooo`0Eooooo`?o0000B_ooool004Wooooo1_l00011ooooo`03o`000?oo
oooooooo00;o00000_ooool00ol0003oooooooooo`0?ooooo`05o`000?ooooooooooooooool00000
0oooool00ol0003oooooooooo`0Booooo`Ko0000B_ooool004cooooo0ol00011ooooo`04o`000?oo
ooooooooooooo`;o00000_ooool00ol0003oooooooooo`0>ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`0Booooo`?o0000COooool004cooooo1ol0000nooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooooooo`0=ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`0>ooooo`Oo0000COooool0053ooooo0ol0000nooooo`04
o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooooooo`0=ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooooooo`0>ooooo`?o0000DOooool0053ooooo1_l0000loooo
o`04o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooooooo`0<ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`0:ooooo`Oo0000DOooool005?ooooo0ol0000m
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0;ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`0:ooooo`?o0000EOooool005?ooooo1ol0
000iooooo`03o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`00000<ooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooooooo`07ooooo`Ko0000EOooool005Oooooo
1ol0000fooooo`04o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooooooo`0:ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`Oo0000F?ooool005Oo
oooo0ol000000ooooooo0000o`000002o`0003Kooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00Wooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
oooo00?ooooo0ol000001?oooooo0000o`000?l0001Hooooo`00Foooool6o`0003Cooooo00?o0000
ooooooooool00_ooool01Ol0003ooooooooooooooooo000000[ooooo00Go0000oooooooooooooooo
o`000003ooooo`03o`000?oooooooooo00Ko0000G?ooool005kooooo1ol0000aooooo`04o`000?oo
ooooooooooooo`;o00000_ooool00ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool0
00000_ooool7o`0005oooooo001Nooooo`?o000000?oooooo`000?l000001Ol0000^ooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
000000Coooooo`000?l0003o0000Goooool006;ooooo0ol000000ooooooo0000o`000005o`0002[o
oooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000Oooooo1ol000001?oo
oooo0000o`000?l0001Sooooo`00I_ooool3o`000003ooooool0003o000000Co0000:?ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooooooo1ol000001?oooooo0000o`00
0?l0001Wooooo`00J_ooool:o`000004ooooool0003o0000o`00023ooooo00?o0000ooooooooool0
0_ooool:o`000004ooooool0003o0000o`0006_ooooo001]ooooo`?o000000?oooooo`000?l00000
2Ol000001?oooooo0000o`000?l0000Eooooo`?o000000?oooooo`000?l000003?l0001_ooooo`00
LOooool3o`000003ooooool0003o000000go000000Goooooo`000?l0003o0000ooooo`03o`000005
ooooool0003o0000o`000?ooool04_l0001cooooo`00NOooool3o`000003ooooool0003o000001Wo
000000Coooooo`000?l0003o0000N_ooool0087ooooo0ol000001Ooooooo0000o`000?l0003ooooo
00?o000000Goooooo`000?l0003o0000ooooo`03o`0008;ooooo003ooooooaOooooo003ooooooaOo
oooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOo
oooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo003ooooooaOooooo0000\
\>"],
  ImageRangeCache->{{{0, 277}, {277, 0}} -> {-0.0500054, -0.0500054, \
0.00397116, 0.00397116}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Since the update neighborhood bears such close topological similarity to \
the interaction neighborhood, displaying both neighborhoods introduces \
unnecessary clutter. We can suppress displaying the update neighborhood by \
setting the ",
  StyleBox["NeighborhoodsToDraw", "Input"],
  " option to ",
  StyleBox["{Interaction}", "Input"],
  ". (The default is ",
  StyleBox["Automatic", "Input"],
  ", which draws both the interaction and update neighborhoods.)"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(pd2 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] g, \[IndentingNewLine]UpdateNeighborhood \[Rule] 
            g2, \[IndentingNewLine]PayoffMatrix \[Rule] 
            PrisonersDilemma[3.0, 2.0, 1.0, 
              0.0], \[IndentingNewLine]NeighborhoodsToDraw \[Rule] \
{Interaction}, \[IndentingNewLine]StrategyColors \[Rule] {LightGray, 
              Black}\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(SetStrategies[Agents[pd2, Table[i, {i, 104, 108}]], 1]\), "\n", 
    \(SetStrategies[Agents[pd2, Table[i, {i, 148, 152}]], 1]\)}], "Input",
  CellLabel->"In[10]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[\[IndentingNewLine]ExtractGraph[pd2], \ 
        VertexStyle \[Rule] Disk[Small]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.04635 .52854 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.64046 .0677 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.11621 .25644 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.78974 .14977 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.5 .04545 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.663 .663 .663 r
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
0 0 0 r
.95253 .54278 Mdot
.663 .663 .663 r
.95096 .55697 Mdot
.94895 .57111 Mdot
0 0 0 r
.94649 .58517 Mdot
.9436 .59916 Mdot
.663 .663 .663 r
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
0 0 0 r
.92767 .65397 Mdot
.663 .663 .663 r
.92263 .66733 Mdot
0 0 0 r
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.663 .663 .663 r
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
0 0 0 r
.86773 .76718 Mdot
.663 .663 .663 r
.85916 .77859 Mdot
.85023 .78974 Mdot
0 0 0 r
.84096 .8006 Mdot
.83135 .81116 Mdot
.663 .663 .663 r
.82141 .82141 Mdot
.81116 .83135 Mdot
0 0 0 r
.8006 .84096 Mdot
.663 .663 .663 r
.78974 .85023 Mdot
.77859 .85916 Mdot
0 0 0 r
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.663 .663 .663 r
.70636 .905 Mdot
0 0 0 r
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.663 .663 .663 r
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
0 0 0 r
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.663 .663 .663 r
.51428 .95432 Mdot
0 0 0 r
.5 .95455 Mdot
.663 .663 .663 r
.48572 .95432 Mdot
0 0 0 r
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.663 .663 .663 r
.41483 .94649 Mdot
0 0 0 r
.40084 .9436 Mdot
.663 .663 .663 r
.38696 .94027 Mdot
0 0 0 r
.37319 .9365 Mdot
.35954 .9323 Mdot
.663 .663 .663 r
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
0 0 0 r
.29364 .905 Mdot
.28102 .89832 Mdot
.663 .663 .663 r
.26862 .89125 Mdot
.25644 .88379 Mdot
0 0 0 r
.24451 .87595 Mdot
.663 .663 .663 r
.23282 .86773 Mdot
.22141 .85916 Mdot
0 0 0 r
.21026 .85023 Mdot
.663 .663 .663 r
.1994 .84096 Mdot
0 0 0 r
.18884 .83135 Mdot
.663 .663 .663 r
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
0 0 0 r
.14084 .77859 Mdot
.13227 .76718 Mdot
.663 .663 .663 r
.12405 .75549 Mdot
0 0 0 r
.11621 .74356 Mdot
.663 .663 .663 r
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
0 0 0 r
.08871 .69354 Mdot
.08284 .68052 Mdot
.663 .663 .663 r
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
0 0 0 r
.05973 .61304 Mdot
.663 .663 .663 r
.0564 .59916 Mdot
.05351 .58517 Mdot
0 0 0 r
.05105 .57111 Mdot
.04904 .55697 Mdot
.663 .663 .663 r
.04747 .54278 Mdot
0 0 0 r
.04635 .52854 Mdot
.663 .663 .663 r
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
0 0 0 r
.04635 .47146 Mdot
.04747 .45722 Mdot
.663 .663 .663 r
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
0 0 0 r
.0635 .37319 Mdot
.663 .663 .663 r
.0677 .35954 Mdot
0 0 0 r
.07233 .34603 Mdot
.07737 .33267 Mdot
.663 .663 .663 r
.08284 .31948 Mdot
.08871 .30646 Mdot
0 0 0 r
.095 .29364 Mdot
.663 .663 .663 r
.10168 .28102 Mdot
0 0 0 r
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.663 .663 .663 r
.14977 .21026 Mdot
0 0 0 r
.15904 .1994 Mdot
.16865 .18884 Mdot
.663 .663 .663 r
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
0 0 0 r
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.663 .663 .663 r
.25644 .11621 Mdot
0 0 0 r
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.663 .663 .663 r
.30646 .08871 Mdot
0 0 0 r
.31948 .08284 Mdot
.33267 .07737 Mdot
.663 .663 .663 r
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
0 0 0 r
.40084 .0564 Mdot
.41483 .05351 Mdot
.663 .663 .663 r
.42889 .05105 Mdot
.44303 .04904 Mdot
0 0 0 r
.45722 .04747 Mdot
.663 .663 .663 r
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
0 0 0 r
.55697 .04904 Mdot
.663 .663 .663 r
.57111 .05105 Mdot
0 0 0 r
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.663 .663 .663 r
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
0 0 0 r
.71898 .10168 Mdot
.663 .663 .663 r
.73138 .10875 Mdot
0 0 0 r
.74356 .11621 Mdot
.663 .663 .663 r
.75549 .12405 Mdot
0 0 0 r
.76718 .13227 Mdot
.663 .663 .663 r
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
0 0 0 r
.81116 .16865 Mdot
.663 .663 .663 r
.82141 .17859 Mdot
0 0 0 r
.83135 .18884 Mdot
.663 .663 .663 r
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
0 0 0 r
.89125 .26862 Mdot
.663 .663 .663 r
.89832 .28102 Mdot
.905 .29364 Mdot
0 0 0 r
.91129 .30646 Mdot
.91716 .31948 Mdot
.663 .663 .663 r
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
0 0 0 r
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.663 .663 .663 r
.95096 .44303 Mdot
0 0 0 r
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.663 .663 .663 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[12]:=",
  ImageSize->{278, 278},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F00015R000`400?l00000o`00003ooooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00POooool3ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onYZJT01Ooo
ooooZJVYojVYZOnYZJWooooo00?oZJVYP_ooool007Wooooo0onYZJT00ooooooo0000o`000002o`00
00?oZJVY00Go0000ojVYZOnYZJWoZJVYo`000003ojVYZ@05o`000?nYZJWoZJVYojVYZOl000000onY
ZJT01Ol0003oZJVYojVYZOnYZJWooooo00?o0000N_ooool0077ooooo0ol000001OooooooZJVYojVY
ZOnYZJWo000000?oZJVY1?l000001OooooooZJVYojVYZOnYZJWooooo00?oZJVY00Go0000ojVYZOnY
ZJWoZJVYooooo`03ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onYZJT5o`0000?oZJVY0ol0001c
ooooo`00KOooool3o`000003ooooool0003o000000;o00000onYZJT01OooooooZJVYojVYZOnYZJWo
oooo00?o00001oooool00ol0003oooooooooo`0;ooooo`?oZJVY00Goooooo`000?l0003o0000oooo
o`03ojVYZ@Oo0000Koooool006[ooooo0onYZJT7o`000004oooooonYZJWoZJVYojVYZ@kooooo00?o
0000ooooooooool05?ooool3ojVYZ@?o000000?oooooo`000?l000001Ol0001[ooooo`00I_ooool3
ojVYZ@04o`000?nYZJWoZJVYojVYZ@?o00005Oooool00ol0003oooooooooo`0Looooo`?o000000?o
ooooo`000?l000001Ol0001Wooooo`00H_ooool3ojVYZ@05o`000?nYZJWoZJVYojVYZOooool00onY
ZJTFooooo`;o00008oooool3o`000003ooooool0003o000000Go0000Hoooool006;ooooo0onYZJT0
1?ooooooZJVYojVYZOnYZJTIooooo`03o`000?oooooooooo02Oooooo0ol000001?oooooo0000o`00
0?l0001Sooooo`00G_ooool3ojVYZ@04o`000?nYZJWoZJVYojVYZAcooooo00?o0000ooooooooool0
;?ooool4o`0000?oZJVYGoooool005_ooooo0ol00003ojVYZAoooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`06ojVYZEcooooo001Jooooo`Co00000onYZJTMooooo`;o0000<Oooool0
0ol0003oooooooooo`06ojVYZ@03o`000?oooooooooo05Wooooo001Gooooo`?o000000Coooooo`00
0?l0003o00007oooool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo00?ooooo0onYZJT0
1?ooooooZJVYojVYZOnYZJUHooooo`00Doooool3ojVYZ@Co00008_ooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00Oooooo1_nYZJUEooooo`00Doooool3ojVYZ@04ooooool0003o0000
o`00027ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`07ooooo`KoZJVY00?o0000
ooooooooool0D_ooool0053ooooo0ol00003ojVYZBCooooo00?o0000ooooooooool0=?ooool00ol0
003oooooooooo`0:ooooo`?oZJVY00CoooooojVYZOnYZJWoZJVYDOooool0053ooooo0ol0000Uoooo
o`;o0000=oooool00ol0003oooooooooo`0>ooooo`?oZJVYDOooool004cooooo1ol0000Tooooo`03
o`000?oooooooooo03Oooooo00?o0000ooooooooool03_ooool3ojVYZ@Co0000COooool004cooooo
0ol0000Wooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool04_ooool3o`0004gooooo
0019ooooo`Ko00009_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo01;ooooo0ol0
0003ojVYZD[ooooo0019ooooo`?o0000:?ooool00ol0003oooooooooo`0jooooo`03o`000?oooooo
oooo01Gooooo0onYZJU:ooooo`00AOooool3ojVYZ@Co00009_ooool2o`0003gooooo00?o0000oooo
ooooool05Oooool3ojVYZ@?o0000Aoooool004Gooooo0onYZJTYooooo`03o`000?oooooooooo03go
oooo00?o0000ooooooooool06?ooool3o`0004Oooooo0012ooooo`?o00000onYZJTXooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool06?ooool4o`0000?oZJVY@oooool004;ooooo0ol0
000Zooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool07?ooool3ojVYZD?ooooo000o
ooooo`Ko0000:?ooool2o`0004;ooooo00?o0000ooooooooool07?ooool3ojVYZ@?o0000@?ooool0
03oooooo0ol0000Zooooo`03o`000?oooooooooo04;ooooo00?o0000ooooooooool07oooool3o`00
043ooooo000oooooo`?o0000:Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo01oo
oooo0ol00010ooooo`00??ooool3o`0002_ooooo00?o0000ooooooooool0A?ooool00ol0003ooooo
ooooo`0Rooooo`?oZJVY?Oooool003cooooo0ol0000Zooooo`03o`000?oooooooooo04Gooooo00?o
0000ooooooooool08_ooool3ojVYZCgooooo000iooooo`Ko0000:?ooool2o`0004Sooooo00?o0000
ooooooooool08_ooool6ojVYZC[ooooo000iooooo`?o0000:_ooool00ol0003oooooooooo`18oooo
o`03o`000?oooooooooo02Gooooo0onYZJTjooooo`00>Oooool3o`0002Wooooo00?o0000oooooooo
ool0BOooool00ol0003oooooooooo`0Tooooo`04o`000?nYZJWoZJVYojVYZC[ooooo000fooooo`?o
ZJVY:oooool00ol0003oooooooooo`1:ooooo`03o`000?oooooooooo02?ooooo00?o0000oooooooo
ool00_ooool3ojVYZCOooooo000fooooo`?oZJVY:Oooool2o`0004gooooo00?o0000ooooooooool0
8_ooool00ol0003oooooooooo`03ooooo`?oZJVY=oooool003?ooooo1_nYZJTXooooo`03o`000?oo
oooooooo04gooooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`04ooooo`?oZJVY0ol0
000dooooo`00<oooool3ojVYZB[ooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0P
ooooo`03o`000?oooooooooo00Sooooo0ol0000dooooo`00<oooool3ojVYZBWooooo00?o0000oooo
ooooool0Coooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo00Wooooo0ol0000doooo
o`00<?ooool3ojVYZB_ooooo00?o0000ooooooooool0D?ooool00ol0003oooooooooo`0Mooooo`;o
00003oooool3ojVYZC7ooooo000`ooooo`?oZJVY:Oooool2o`0005;ooooo00?o0000ooooooooool0
7Oooool00ol0003oooooooooo`0?ooooo`?oZJVY<Oooool0033ooooo0onYZJTXooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0@ooooo`?oZJVY<Ooo
ool002gooooo0ol0000Zooooo`03o`000?oooooooooo05?ooooo00?o0000ooooooooool06oooool0
0ol0003oooooooooo`0Dooooo`?o0000;_ooool002gooooo0ol0000Yooooo`03o`000?oooooooooo
05Cooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Eooooo`?o0000;_ooool002go
oooo0ol0000Xooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Fooooo`?o0000;_ooool002[ooooo1?l0000Xooooo`;o0000F?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo0onYZJT/ooooo`00:_ooool3o`0002Sooooo00?o
0000ooooooooool0F?ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01[ooooo0onY
ZJT/ooooo`00:_ooool3o`0002Oooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0E
ooooo`;o00007Oooool3ojVYZBcooooo000Xooooo`?oZJVY:?ooool00ol0003oooooooooo`1Joooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool08?ooool3ojVYZBWooooo000Xooooo`?o
ZJVY9_ooool2o`0005gooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Qooooo`?o
ZJVY:Oooool002Sooooo0onYZJTUooooo`03o`000?oooooooooo05gooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`0Rooooo`?oZJVY:Oooool002Gooooo1?l0000Vooooo`03o`000?oo
oooooooo05kooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Uooooo`?oZJVY9ooo
ool002Gooooo0ol0000Vooooo`03o`000?oooooooooo05oooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Vooooo`?oZJVY9oooool002Gooooo0ol0000Uooooo`03o`000?oooooooooo
063ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Wooooo`?oZJVY9oooool002?o
oooo0ol0000Uooooo`;o0000Hoooool00ol0003oooooooooo`0=ooooo`;o0000;Oooool3ojVYZBCo
oooo000Sooooo`?o00009?ooool00ol0003oooooooooo`1Sooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool0;Oooool3ojVYZBCooooo000Sooooo`?o00008oooool00ol0003ooooooooo
o`1Tooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0;_ooool3ojVYZBCooooo000Q
ooooo`?o00009?ooool00ol0003oooooooooo`1Uooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool0<Oooool3ojVYZB;ooooo000Qooooo`?o00008oooool00ol0003oooooooooo`1Voooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0<_ooool3ojVYZB;ooooo000Qooooo`?o
00008Oooool2o`0006Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0cooooo`?o
ZJVY8_ooool0027ooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`1Yooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`0Pooooo`007ooo
ool3o`00027ooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo03Oooooo0onYZJTPooooo`007oooool3o`00023ooooo00?o0000ooooooooool0Joooool0
0ol0003oooooooooo`04ooooo`;o0000>_ooool3ojVYZB3ooooo000Oooooo`?o00007_ooool2o`00
06kooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0jooooo`?oZJVY8?ooool001go
oooo0ol000000ooooooo0000ooooo`0Looooo`03o`000?oooooooooo06kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0mooooo`?o00007_ooool001gooooo0ol00002ooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool0Koooool01Ol0003ooooooooooooooooo0000043o
oooo0ol0000Nooooo`007Oooool3o`0000;ooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`1`ooooo`04o`000?ooooooooooo`00047ooooo0ol0000Nooooo`006oooool3ojVYZ@Gooooo
00?o0000ooooooooool05_ooool00ol0003oooooooooo`1aooooo`03o`000?oooooo000004Cooooo
0onYZJTLooooo`006oooool3ojVYZ@Kooooo00?o0000ooooooooool04oooool2o`0007Cooooo0_l0
0015ooooo`?oZJVY7?ooool001_ooooo0onYZJT6ooooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool0Loooool2o`0004Kooooo0onYZJTLooooo`006oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool0Loooool00ol0003oooooo`000018oooo
o`03o`000?oooooooooo01[ooooo000Iooooo`?o00002_ooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo07?ooooo00Co0000oooooooooooo0000B?ooool3ojVYZA[ooooo000Iooooo`?o
00002_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo07?ooooo00Go0000oooooooo
ooooooooo`000018ooooo`?oZJVY6_ooool001Wooooo0ol0000;ooooo`03o`000?oooooooooo00[o
oooo0_l0001eooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0A_ooool3ojVYZA[o
oooo000Iooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`1dooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0B?ooool00ol0003ooooo
ooooo`0Hooooo`005oooool3ojVYZ@kooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`1dooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0B?ooool3o`0001Sooooo000G
ooooo`?oZJVY3oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo07Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`18ooooo`?o00006?ooool001Oooooo0onYZJT@oooo
o`03o`000?oooooooooo00;ooooo0_l0001eooooo`;o00002?ooool00ol0003oooooooooo`18oooo
o`?o00006?ooool001Gooooo0onYZJTBooooo`05o`000?ooooooooooooooool00000M_ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo04Wooooo0ol0000Gooooo`005Oooool3ojVYZA?o
oooo00?o0000ooooool00000M_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo04Wo
oooo0ol0000Gooooo`005Oooool3ojVYZACooooo00?o0000ooooooooool0M?ooool00ol0003ooooo
ooooo`0:ooooo`03o`000?oooooooooo04Wooooo0ol0000Gooooo`005_ooool00ol0003ooooooooo
o`0Booooo`;o0000MOooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo04_ooooo00?o
0000ooooooooool05Oooool001Cooooo0ol0000Booooo`;o00000_ooool00ol0003oooooooooo`1a
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0Boooool3ojVYZAGooooo000Doooo
o`?o00004Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo06oooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`1;ooooo`?oZJVY5Oooool001Cooooo0ol0000@ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool0KOooool2o`00013ooooo00?o0000oooooooo
ool0Boooool3ojVYZAGooooo000Booooo`?o00004Oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo06_ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1<ooooo`?oZJVY
5?ooool001;ooooo0ol0000?ooooo`;o00002_ooool00ol0003oooooooooo`1Yooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool0C?ooool3ojVYZACooooo000Booooo`?o00003_ooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Sooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`1<ooooo`?oZJVY5?ooool001?ooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo06Kooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`1>ooooo`03o`000?oooooooooo01;ooooo000Aooooo`?oZJVY3Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo06Gooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`1>ooooo`?oZJVY4_ooool0017ooooo0onYZJT<ooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool0Hoooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04kooooo
0onYZJTBooooo`004Oooool3ojVYZ@[ooooo0_l0000Cooooo`03o`000?oooooooooo067ooooo00?o
0000ooooooooool05_ooool00ol0003oooooooooo`1>ooooo`?oZJVY4_ooool001;ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo05oooooo0_l0000I
ooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool04Oooool0013ooooo0ol00009oooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0GOooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo04oooooo0onYZJTAooooo`004?ooool3o`0000Sooooo00?o0000oooooooo
ool05oooool00ol0003oooooooooo`1Kooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool0Coooool3ojVYZA7ooooo000@ooooo`?o00001oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo05[ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`1?ooooo`?oZJVY
4Oooool0017ooooo00?o0000ooooooooool01?ooool2o`0001cooooo00?o0000ooooooooool0F?oo
ool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool04?oo
ool000oooooo0onYZJT5ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0E_ooool0
0ol0003oooooooooo`0Mooooo`03o`000?oooooooooo053ooooo0ol0000@ooooo`003oooool3ojVY
Z@Cooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo
01kooooo00?o0000ooooooooool0D?ooool3o`00013ooooo000?ooooo`?oZJVY0oooool00ol0003o
ooooooooo`0Pooooo`03o`000?oooooooooo05;ooooo0_l0000Qooooo`03o`000?oooooooooo053o
oooo0ol0000@ooooo`004?ooool00ol0003oooooooooo`02o`0002Cooooo00?o0000ooooooooool0
D?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool0
3oooool000kooooo0onYZJT00ooooooo0000ooooo`0Uooooo`03o`000?oooooooooo04oooooo00?o
0000ooooooooool08Oooool00ol0003oooooooooo`1Booooo`?o00003oooool000kooooo0onYZJT0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool08_ooool0
0ol0003oooooooooo`1Booooo`?o00003oooool000kooooo0onYZJTZooooo`03o`000?oooooooooo
04_ooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`1Booooo`?o00003oooool000ko
oooo0_l0000[ooooo`03o`000?oooooooooo04[ooooo00?o0000ooooooooool09?ooool00ol0003o
ooooooooo`1Dooooo`03o`000?oooooooooo00gooooo000=ooooo`?oZJVY;?ooool00ol0003ooooo
ooooo`18ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0Doooool3o`0000kooooo
000=ooooo`?oZJVY;Oooool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool0Doooool3o`0000kooooo000=ooooo`?oZJVY;Oooool00ol0003oooooooooo`14
ooooo`;o0000:Oooool00ol0003oooooooooo`1Cooooo`?o00003_ooool000gooooo0onYZJT^oooo
o`03o`000?oooooooooo04;ooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`1Coooo
o`?o00003_ooool000gooooo0onYZJT_ooooo`03o`000?oooooooooo043ooooo00?o0000oooooooo
ool0:_ooool00ol0003oooooooooo`1Cooooo`?o00003_ooool000gooooo0onYZJT_ooooo`03o`00
0?oooooooooo03oooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1Cooooo`?o0000
3_ooool000kooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0mooooo`03o`000?oo
oooooooo02cooooo00?o0000ooooooooool0E?ooool00ol0003oooooooooo`0=ooooo`003?ooool3
ojVYZC;ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`0]ooooo`03o`000?oooooo
oooo05Cooooo0onYZJT=ooooo`003?ooool3ojVYZC;ooooo00?o0000ooooooooool0>_ooool00ol0
003oooooooooo`0^ooooo`03o`000?oooooooooo05Cooooo0onYZJT=ooooo`003?ooool3ojVYZC?o
oooo00?o0000ooooooooool0=oooool2o`00037ooooo00?o0000ooooooooool0E?ooool3ojVYZ@go
oooo000=ooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0=Oooool00ol0003ooooo
ooooo`0aooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool03?ooool000cooooo0ol0
000dooooo`03o`000?oooooooooo03Cooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`1Dooooo`?o00003Oooool000cooooo0ol0000eooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<oooool00ol0003oooooooooo`1Dooooo`?o00003Oooool000cooooo0ol0000foooo
o`03o`000?oooooooooo033ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`1Doooo
o`?o00003Oooool000gooooo00?o0000ooooooooool0=Oooool00ol0003oooooooooo`0_ooooo`03
o`000?oooooooooo03Gooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`0<ooooo`00
2oooool3o`0003Sooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0fooooo`03o`00
0?oooooooooo05Gooooo0ol0000<ooooo`002oooool3o`0003Wooooo00?o0000ooooooooool0:_oo
ool2o`0003Wooooo00?o0000ooooooooool0EOooool3o`0000cooooo000;ooooo`?o0000>Oooool0
0ol0003oooooooooo`0Yooooo`03o`000?oooooooooo03Wooooo00?o0000ooooooooool0EOooool3
o`0000cooooo000<ooooo`03o`000?oooooooooo03Wooooo00?o0000ooooooooool09oooool00ol0
003oooooooooo`0jooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool02oooool000_o
oooo0onYZJTkooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>oooool00ol0003o
ooooooooo`1Eooooo`?o00003?ooool000_ooooo0onYZJTkooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool0??ooool00ol0003oooooooooo`1Eooooo`?o00003?ooool000_ooooo0onY
ZJTlooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0?Oooool00ol0003ooooooooo
o`1Eooooo`?o00003?ooool000cooooo00?o0000ooooooooool0??ooool00ol0003oooooooooo`0P
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0;
ooooo`002oooool3ojVYZCgooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0ooooo
o`03o`000?oooooooooo05Gooooo0onYZJT<ooooo`002oooool3ojVYZCkooooo00?o0000oooooooo
ool07?ooool2o`0004;ooooo00?o0000ooooooooool0EOooool3ojVYZ@cooooo000;ooooo`?oZJVY
?oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo04;ooooo00?o0000ooooooooool0
EOooool3ojVYZ@cooooo000<ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool02ooo
ool000_ooooo0onYZJU0ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0A?ooool0
0ol0003oooooooooo`1Eooooo`?oZJVY3?ooool000_ooooo0onYZJU1ooooo`03o`000?oooooooooo
01Gooooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`1Eooooo`?oZJVY3?ooool000_o
oooo0onYZJU1ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0A_ooool00ol0003o
ooooooooo`1Eooooo`?oZJVY3?ooool000cooooo00?o0000ooooooooool0@Oooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool0E_ooool00ol0003ooooo
ooooo`0;ooooo`002oooool3o`0004;ooooo00?o0000ooooooooool04?ooool2o`0004[ooooo00?o
0000ooooooooool0EOooool3ojVYZ@cooooo000;ooooo`co0000>_ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo04[ooooo00?o0000ooooooooool0EOooool3ojVYZ@cooooo000;oooo
o`?o00002OoooolEo`0002Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1;oooo
o`03o`000?oooooooooo05Gooooo0onYZJT<ooooo`003?ooool00ol0003oooooooooo`0MoooooaGo
00004Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo04cooooo00?o0000oooooooo
ool0E_ooool00ol0003oooooooooo`0;ooooo`003?ooool3ojVYZC;ooooo5Ol00009ooooo`03o`00
0?oooooooooo04gooooo00?o0000ooooooooool0E?ooool3o`0000gooooo000<ooooo`?oZJVYAOoo
ool00ol0003oooooo`00000Do`0004Cooooo00?o0000ooooooooool0E?ooool3o`0000gooooo000<
ooooo`?oZJVYAOooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo5Ol0000_
ooooo`03o`000?oooooooooo05Cooooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`15oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool08OoooolDo`0001_ooooo00?o0000oooo
ooooool0EOooool00ol0003oooooooooo`0<ooooo`003?ooool3o`0004Oooooo00Co0000oooooooo
oooooooo0_l0000hoooooaGo00001_ooool00ol0003oooooooooo`1Dooooo`?oZJVY3Oooool000co
oooo0ol00017ooooo`04o`000?ooooooooooo`0004oooooo5Ol00018ooooo`?oZJVY3Oooool000co
oooo0ol00018ooooo`;o0000E_ooool00ol0003oooooooooo`0<oooooaGo0000<oooool3ojVYZ@go
oooo000=ooooo`03o`000?oooooooooo04Oooooo0_l0001Fooooo`03o`000?oooooooooo027ooooo
5Ol0000Oooooo`03o`000?oooooooooo00cooooo000=ooooo`?o0000A_ooool00ol0003oooooo`00
001Fooooo`03o`000?oooooooooo03Kooooo5Ol00008ooooo`?oZJVY3_ooool000gooooo0ol00015
ooooo`05o`000?ooooooooooooooool00000EOooool00ol0003oooooooooo`1;ooooo`So00000onY
ZJT>ooooo`003Oooool3o`0004Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1B
ooooo`03o`000?oooooooooo05?ooooo0onYZJT>ooooo`003_ooool00ol0003oooooooooo`11oooo
o`;o00001_ooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo05Cooooo00?o0000oooo
ooooool03Oooool000gooooo0onYZJU1ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool0DOooool00ol0003oooooooooo`1Cooooo`?o00003_ooool000gooooo0onYZJU0ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool0D?ooool00ol0003oooooooooo`1Cooooo`?o0000
3_ooool000gooooo0onYZJToooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0D?oo
ool00ol0003oooooooooo`1Cooooo`?o00003_ooool000oooooo00?o0000ooooooooool0??ooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool0Doooool0
0ol0003oooooooooo`0>ooooo`003_ooool3ojVYZCcooooo00?o0000ooooooooool03_ooool00ol0
003oooooooooo`1>ooooo`03o`000?oooooooooo05;ooooo0ol0000?ooooo`003_ooool3ojVYZC_o
oooo00?o0000ooooooooool03oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo05;o
oooo0ol0000?ooooo`003_ooool3ojVYZC[ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`1=ooooo`03o`000?oooooooooo05;ooooo0ol0000?ooooo`003oooool3o`0003Oooooo0_l0
000Eooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool0DOooool3ojVYZA3ooooo000?
ooooo`?o0000=_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04cooooo00?o0000
ooooooooool0DOooool3ojVYZA3ooooo000?ooooo`?o0000=Oooool00ol0003oooooooooo`0Goooo
o`03o`000?oooooooooo04[ooooo00?o0000ooooooooool0D_ooool3ojVYZA3ooooo000@ooooo`03
o`000?oooooooooo03?ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`19ooooo`03
o`000?oooooooooo05?ooooo00?o0000ooooooooool03oooool0013ooooo0onYZJTbooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool0BOooool00ol0003oooooooooo`1Aooooo`?oZJVY
4Oooool0013ooooo0onYZJTaooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0B?oo
ool00ol0003oooooooooo`1Aooooo`?oZJVY4Oooool0013ooooo0onYZJT`ooooo`03o`000?oooooo
oooo01kooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`1Aooooo`?oZJVY4Oooool0
017ooooo00?o0000ooooooooool0;Oooool2o`00027ooooo00?o0000ooooooooool0Aoooool00ol0
003oooooooooo`1Booooo`03o`000?oooooooooo013ooooo000Aooooo`?oZJVY;?ooool00ol0003o
ooooooooo`0Rooooo`03o`000?oooooooooo04Kooooo00?o0000ooooooooool0D?ooool3ojVYZA;o
oooo000Aooooo`?oZJVY:oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo04Gooooo
00?o0000ooooooooool0D?ooool3ojVYZA;ooooo000Aooooo`?oZJVY:_ooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo04Gooooo00?o0000ooooooooool0D?ooool3ojVYZA;ooooo000C
ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`14
ooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool04_ooool001;ooooo0onYZJTWoooo
o`03o`000?oooooooooo02Wooooo00?o0000ooooooooool0@oooool00ol0003oooooooooo`1>oooo
o`?o00005?ooool001;ooooo0onYZJTVooooo`03o`000?oooooooooo02[ooooo00?o0000oooooooo
ool0@oooool00ol0003oooooooooo`1>ooooo`?o00005?ooool001;ooooo0onYZJTUooooo`03o`00
0?oooooooooo02cooooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`1>ooooo`?o0000
5?ooool001Cooooo0onYZJTQooooo`;o0000<?ooool00ol0003oooooooooo`11ooooo`03o`000?oo
oooooooo04gooooo0onYZJTEooooo`005?ooool3ojVYZB3ooooo00?o0000ooooooooool0<?ooool0
0ol0003oooooooooo`11ooooo`03o`000?oooooooooo04gooooo0onYZJTEooooo`005?ooool3ojVY
ZAoooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo
04gooooo0onYZJTEooooo`005Oooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo03Co
oooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo01Co
oooo000Eooooo`?o00007?ooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo03oooooo
00?o0000ooooooooool0Boooool3o`0001Oooooo000Eooooo`?o00006oooool00ol0003ooooooooo
o`0gooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0Boooool3o`0001Oooooo000E
ooooo`?o00006_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo03gooooo00?o0000
ooooooooool0Boooool3o`0001Oooooo000Gooooo`03o`000?oooooooooo01Kooooo0_l0000loooo
o`03o`000?oooooooooo03gooooo00?o0000ooooooooool0C?ooool00ol0003oooooooooo`0Foooo
o`005oooool3o`0001Gooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`0looooo`03
o`000?oooooooooo04[ooooo0ol0000Hooooo`005oooool3o`0001Cooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo04[ooooo0ol0000Hooooo`005ooo
ool3o`0001?ooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`0kooooo`03o`000?oo
oooooooo04[ooooo0ol0000Hooooo`006Oooool3ojVYZA3ooooo00?o0000ooooooooool0@_ooool0
0ol0003oooooooooo`0jooooo`03o`000?oooooooooo04Sooooo0ol0000Jooooo`006Oooool3ojVY
Z@oooooo00?o0000ooooooooool0@oooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo
04Sooooo0ol0000Jooooo`006Oooool3ojVYZ@kooooo00?o0000ooooooooool0AOooool00ol0003o
ooooooooo`0iooooo`03o`000?oooooooooo04Sooooo0ol0000Jooooo`006oooool00ol0003ooooo
ooooo`0;ooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool0>?ooool00ol0003ooooo
ooooo`18ooooo`03o`000?oooooooooo01[ooooo000Kooooo`?oZJVY2Oooool2o`0004[ooooo00?o
0000ooooooooool0>?ooool00ol0003oooooooooo`16ooooo`?o00007?ooool001_ooooo0onYZJT8
ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool0=oooool00ol0003oooooooooo`16
ooooo`?o00007?ooool001_ooooo0onYZJT7ooooo`03o`000?oooooooooo04gooooo00?o0000oooo
ooooool0=_ooool00ol0003oooooooooo`16ooooo`?o00007?ooool001gooooo0onYZJT4ooooo`03
o`000?oooooooooo04kooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`14ooooo`?o
ZJVY7_ooool001gooooo0onYZJT3ooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool0
=Oooool00ol0003oooooooooo`14ooooo`?oZJVY7_ooool001gooooo0onYZJT2ooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`14ooooo`?oZJVY7_oo
ool001oooooo00?o0000ooooool00000EOooool00ol0003oooooooooo`0dooooo`03o`000?oooooo
oooo04Cooooo00?o0000ooooooooool07_ooool001oooooo0ol0001Fooooo`03o`000?oooooooooo
03?ooooo00?o0000ooooooooool0@_ooool3ojVYZB3ooooo000Oooooo`?o0000Eoooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo04;ooooo0onYZJTPooooo`007oooool3o`0005Oooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`12ooooo`?oZJVY8?ooool0027ooooo0onY
ZJUFooooo`03o`000?oooooooooo037ooooo00?o0000ooooooooool0@?ooool3ojVYZB;ooooo000Q
ooooo`?oZJVYEoooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo043ooooo0onYZJTR
ooooo`008Oooool3ojVYZEOooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`10oooo
o`?oZJVY8_ooool002?ooooo0ol0001Fooooo`03o`000?oooooooooo02oooooo00?o0000oooooooo
ool0?_ooool3o`0002Cooooo000Sooooo`?o0000Eoooool00ol0003oooooooooo`0^ooooo`03o`00
0?oooooooooo03kooooo0ol0000Tooooo`008oooool3o`0005Oooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`0nooooo`?o00009?ooool002Gooooo0ol0001Fooooo`03o`000?oooooo
oooo02gooooo00?o0000ooooooooool0>oooool3ojVYZBOooooo000Uooooo`?o0000Eoooool00ol0
003oooooooooo`0/ooooo`03o`000?oooooooooo03_ooooo0onYZJTWooooo`009Oooool3o`0005Oo
oooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0kooooo`?oZJVY9oooool002Sooooo
0onYZJUEooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool0>Oooool3ojVYZBWooooo
000Xooooo`?oZJVYE_ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo03Wooooo0onY
ZJTYooooo`00:?ooool3ojVYZEKooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0i
ooooo`?oZJVY:Oooool002[ooooo0onYZJUEooooo`03o`000?oooooooooo02Wooooo00?o0000oooo
ooooool0=_ooool3o`0002cooooo000Zooooo`?oZJVYE_ooool00ol0003oooooooooo`0Xooooo`03
o`000?oooooooooo03Kooooo0ol0000/ooooo`00:_ooool3ojVYZEKooooo00?o0000ooooooooool0
:?ooool00ol0003oooooooooo`0fooooo`?o0000;?ooool002gooooo0onYZJUDooooo`03o`000?oo
oooooooo02Oooooo00?o0000ooooooooool0=?ooool3o`0002kooooo000]ooooo`?oZJVYEOooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo03Cooooo0ol0000^ooooo`00;Oooool3ojVY
ZEGooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0dooooo`?o0000;_ooool0033o
oooo0onYZJUCooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0<Oooool3ojVYZC7o
oooo000`ooooo`?oZJVYE?ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo037ooooo
0onYZJTaooooo`00<?ooool3ojVYZECooooo00?o0000ooooooooool09?ooool00ol0003ooooooooo
o`0aooooo`?oZJVY<Oooool003?ooooo0ol0001Booooo`03o`000?oooooooooo02?ooooo00?o0000
ooooooooool0;_ooool3ojVYZCCooooo000cooooo`?o0000Doooool00ol0003oooooooooo`0Qoooo
o`03o`000?oooooooooo02oooooo0onYZJTdooooo`00<oooool4o`0005;ooooo00?o0000oooooooo
ool08Oooool00ol0003oooooooooo`0^ooooo`04o`000?nYZJWoZJVYojVYZCCooooo000fooooo`?o
ZJVYDOooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo02cooooo0ol0000gooooo`00
=_ooool3ojVYZE;ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0/ooooo`?o0000
=oooool003Kooooo0onYZJT3o`0004oooooo00?o0000ooooooooool07oooool00ol0003ooooooooo
o`0Yooooo`?oZJVY0ol0000gooooo`00>Oooool3o`00053ooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0Yooooo`?oZJVY>_ooool003Wooooo1?l0001@ooooo`03o`000?oooooooooo
01gooooo00?o0000ooooooooool0:?ooool01?l0003oZJVYojVYZOnYZJTjooooo`00??ooool3ojVY
ZDkooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0Vooooo`?oZJVY?Oooool003co
oooo0onYZJU?ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool09_ooool3ojVYZCgo
oooo000looooo`KoZJVYC?ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo02?ooooo
0ol00003ojVYZCgooooo000oooooo`?oZJVYCOooool00ol0003oooooooooo`0Kooooo`03o`000?oo
oooooooo02?ooooo0ol00010ooooo`00?oooool3ojVYZ@?o0000Boooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo023ooooo1_l00010ooooo`00@_ooool3o`0004_ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Pooooo`?o0000@oooool004;ooooo0ol00003ojVYZDWo
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Looooo`Oo0000@oooool004Gooooo
0onYZJT00ol0003oooooooooo`17ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
7?ooool3o`0004Oooooo0015ooooo`?oZJVY00CoooooojVYZOnYZJWoZJVYA_ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo1_l00017ooooo`00BOooool3ojVYZDOooooo00?o
0000ooooooooool05oooool00ol0003oooooooooo`0Iooooo`?o0000B_ooool004Wooooo0onYZJT3
o`0004Gooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Fooooo`Ko0000B_ooool0
04cooooo0ol00015ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool05_ooool3o`00
04gooooo001<ooooo`Oo0000@_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01;o
oooo0onYZJT4o`0004gooooo001@ooooo`?o0000@oooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01;ooooo0onYZJUAooooo`00D?ooool3o`0000?oZJVY@?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo00kooooo1?l00003ojVYZE7ooooo001Cooooo`?oZJVY@Oooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00kooooo0ol0001Eooooo`00Doooool3ojVY
Z@04o`000?nYZJWoZJVYojVYZCkooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0;
ooooo`Ko0000EOooool005Oooooo0onYZJT01?l0003oZJVYojVYZOnYZJTjooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool01oooool7o`0005Sooooo001Gooooo`?oZJVY00GoooooojVY
ZOnYZJWoZJVYo`00000jooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01oooool3
o`000004ooooool0003o0000o`0005Sooooo001Kooooo`KoZJVY>Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo00Cooooo0onYZJT3o`0005cooooo001Nooooo`?oZJVY1?l0000eoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool00onYZJT01?l0003oZJVYojVYZOnYZJUO
ooooo`00G_ooool3ojVYZ@03ooooool0003o000000Go0000<_ooool00ol0003oooooooooo`0>oooo
o`?oZJVY00Go0000ojVYZOnYZJWoZJVYooooo`03ojVYZEoooooo001Rooooo`?o000000?oooooo`00
0?l000000_l00003ojVYZBoooooo00?o0000ooooooooool02Oooool3ojVYZ@05o`000?nYZJWoZJVY
ojVYZOooool00onYZJUSooooo`00I_ooool3o`000004oooooonYZJWoZJVYojVYZ@?o0000;?ooool0
0ol0003oooooooooo`05ooooo`?oZJVY00Go0000ojVYZOnYZJWoZJVYooooo`03ojVYZFOooooo001Z
ooooo`?oZJVY1?l00003ojVYZ@04ooooool0003o0000o`0002Gooooo1_nYZJT01OooooooZJVYojVY
ZOnYZJWooooo00?oZJVYJoooool006gooooo0ol000001?ooooooZJVYojVYZOnYZJT8o`000004oooo
ool0003o0000o`0001Gooooo0ol000000ooooooo0000o`000002o`0000KoZJVY00Co0000ojVYZOnY
ZJWoZJVYKoooool0077ooooo0onYZJT01Ooooooo0000o`000?l0003ooooo00_o000000GoooooojVY
ZOnYZJWoZJVYooooo`03o`000005oooooonYZJWoZJVYojVYZOooool02ol000000oooooooZJVYojVY
Z@04ojVYZG?ooooo001iooooo`?o000000Goooooo`000?l0003o0000ooooo`04o`0000?oZJVY1Ol0
0003ojVYZ@Co000000Goooooo`000?l0003o0000ooooo`03o`0007[ooooo0021ooooo`?o000000Go
ooooojVYZOnYZJWoZJVYooooo`03o`000005oooooonYZJWoZJVYojVYZOooool00ol00022ooooo`00
oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00
oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00
oooooolGooooo`00\
\>"],
  ImageRangeCache->{{{0, 277}, {277, 0}} -> {-0.0500054, -0.0500054, \
0.00397116, 0.00397116}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(Evolve[pd2, Generations \[Rule] 10];\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Some cooperators survive:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[\[IndentingNewLine]ExtractGraph[pd2], \ 
        VertexStyle \[Rule] Disk[Small]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.04635 .52854 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.64046 .0677 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.11621 .25644 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.78974 .14977 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.5 .04545 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.663 .663 .663 r
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
0 0 0 r
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.663 .663 .663 r
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
0 0 0 r
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[14]:=",
  ImageSize->{278, 278},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F00015R000`400?l00000o`00003ooooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolGooooo`00oooooolG
ooooo`00oooooolGooooo`00POooool3ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onYZJT01Ooo
ooooZJVYojVYZOnYZJWooooo00?oZJVYP_ooool007Wooooo0ol000000ooooooo0000o`000002o`00
00?oZJVY00Go0000ojVYZOnYZJWoZJVYo`000003ojVYZ@05o`000?nYZJWoZJVYojVYZOl000000onY
ZJT4o`000004ooooool0003o0000o`0007[ooooo001aooooo`?o000000?oooooo`000?l000002Ol0
00001OooooooZJVYojVYZOnYZJWooooo00?oZJVY00Go0000ojVYZOnYZJWoZJVYooooo`03ojVYZ@05
oooooonYZJWoZJVYojVYZOooool03_l0001cooooo`00KOooool3o`000003ooooool0003o000000Go
000000Goooooo`000?l0003o0000ooooo`03o`0000Oooooo00?o0000ooooooooool02oooool3o`00
0005ooooool0003o0000o`000?ooool02_l0001_ooooo`00J_ooool:o`000004ooooool0003o0000
o`0000kooooo00?o0000ooooooooool05?ooool6o`000003ooooool0003o000000Go0000Joooool0
06Kooooo2_l0000Eooooo`03o`000?oooooooooo01cooooo0ol000000ooooooo0000o`000005o`00
06Oooooo001Rooooo`Oo000000Coooooo`000?l0003o00005_ooool2o`0002?ooooo0ol000000ooo
oooo0000o`000005o`0006?ooooo001Rooooo`?o000000Coooooo`000?l0003o00006Oooool00ol0
003oooooooooo`0Wooooo`?o000000Coooooo`000?l0003o0000Hoooool005kooooo1ol0000Loooo
o`03o`000?oooooooooo02cooooo1ol0001Oooooo`00Foooool6o`0001oooooo00?o0000oooooooo
ool0;_ooool00ol0003oooooooooo`06o`0005cooooo001Jooooo`Oo00007Oooool2o`00037ooooo
00?o0000ooooooooool01ol0001Kooooo`00Eoooool3o`000004ooooool0003o0000o`0001oooooo
00?o0000ooooooooool0<Oooool00ol0003oooooooooo`03ooooo`?o000000Coooooo`000?l0003o
0000F?ooool005?ooooo1ol0000Rooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
1oooool6o`0005Gooooo001Cooooo`?o000000Coooooo`000?l0003o00008Oooool00ol0003ooooo
ooooo`0cooooo`03o`000?oooooooooo00Oooooo1ol0001Dooooo`00D?ooool6o`0002Cooooo00?o
0000ooooooooool0=?ooool00ol0003oooooooooo`0:ooooo`?o000000Coooooo`000?l0003o0000
DOooool0053ooooo0ol0000Uooooo`;o0000=oooool00ol0003oooooooooo`0>ooooo`?o0000DOoo
ool004cooooo1ol0000Tooooo`03o`000?oooooooooo03Oooooo00?o0000ooooooooool03_ooool7
o`0004gooooo001<ooooo`?o00009oooool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo
01;ooooo0ol0001=ooooo`00BOooool6o`0002Kooooo00?o0000ooooooooool0>Oooool00ol0003o
ooooooooo`0Booooo`Ko0000B_ooool004Wooooo0ol0000Xooooo`03o`000?oooooooooo03[ooooo
00?o0000ooooooooool05Oooool3o`0004[ooooo0015ooooo`Oo00009_ooool2o`0003gooooo00?o
0000ooooooooool05Oooool6o`0004Oooooo0015ooooo`?o0000:Oooool00ol0003oooooooooo`0m
ooooo`03o`000?oooooooooo01Sooooo0ol00017ooooo`00@_ooool6o`0002Sooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0Hooooo`Oo0000@oooool004;ooooo0ol0000Zooooo`03
o`000?oooooooooo03oooooo00?o0000ooooooooool07?ooool3o`0004?ooooo000oooooo`Ko0000
:?ooool2o`0004;ooooo00?o0000ooooooooool07?ooool6o`00043ooooo000oooooo`?o0000:_oo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo01oooooo0ol00010ooooo`00?oooool3
o`0002Wooooo00?o0000ooooooooool0@oooool00ol0003oooooooooo`0Oooooo`?o0000@?ooool0
03cooooo0ol0000[ooooo`03o`000?oooooooooo04Cooooo00?o0000ooooooooool08_ooool3o`00
03gooooo000looooo`?o0000:_ooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo02;o
oooo0ol0000mooooo`00>Oooool6o`0002Sooooo0_l00018ooooo`03o`000?oooooooooo02;ooooo
1_l0000jooooo`00>Oooool3o`0002[ooooo00?o0000ooooooooool0B?ooool00ol0003ooooooooo
o`0Uooooo`?o0000>_ooool003Wooooo0ol0000Yooooo`03o`000?oooooooooo04Wooooo00?o0000
ooooooooool09?ooool4o`0003[ooooo000fooooo`?o0000:oooool00ol0003oooooooooo`1:oooo
o`03o`000?oooooooooo02?ooooo00?o0000ooooooooool00_ooool3o`0003Oooooo000fooooo`?o
0000:Oooool2o`0004gooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`03ooooo`?o
0000=oooool003?ooooo1_l0000Xooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0
8Oooool00ol0003oooooooooo`04ooooo`Ko0000=?ooool003?ooooo0ol0000Zooooo`03o`000?oo
oooooooo04kooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`08ooooo`?o0000=?oo
ool003?ooooo0ol0000Yooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool07oooool0
0ol0003oooooooooo`09ooooo`?o0000=?ooool0033ooooo0ol0000[ooooo`03o`000?oooooooooo
053ooooo00?o0000ooooooooool07Oooool2o`0000oooooo0ol0000aooooo`00<?ooool3o`0002Wo
oooo0_l0001Booooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool03oooool3o`00037o
oooo000`ooooo`?o0000:?ooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo01cooooo
00?o0000ooooooooool04?ooool3o`00037ooooo000]ooooo`?o0000:_ooool00ol0003ooooooooo
o`1Cooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool05?ooool3o`0002kooooo000]
ooooo`?o0000:Oooool00ol0003oooooooooo`1Dooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool05Oooool3o`0002kooooo000]ooooo`?o0000:?ooool00ol0003oooooooooo`1Eoooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool05_ooool3o`0002kooooo000Zooooo`Co
0000:?ooool2o`0005Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`?o
0000;?ooool002[ooooo0ol0000Xooooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool0
5oooool00ol0003oooooooooo`0Jooooo`?o0000;?ooool002[ooooo0ol0000Wooooo`03o`000?oo
oooooooo05Wooooo00?o0000ooooooooool05Oooool2o`0001gooooo0ol0000/ooooo`00:?ooool3
o`0002Sooooo00?o0000ooooooooool0F_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo023ooooo0ol0000Yooooo`00:?ooool3o`0002Kooooo0_l0001Mooooo`03o`000?oooooooooo
01?ooooo00?o0000ooooooooool08Oooool3o`0002Wooooo000Xooooo`?o00009Oooool00ol0003o
ooooooooo`1Mooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool08_ooool3o`0002Wo
oooo000Uooooo`Co00009_ooool00ol0003oooooooooo`1Nooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool09Oooool3o`0002Oooooo000Uooooo`?o00009_ooool00ol0003ooooooooo
o`1Oooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool09_ooool3o`0002Oooooo000U
ooooo`?o00009Oooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool09oooool3o`0002Oooooo000Sooooo`?o00009Oooool2o`0006?ooooo00?o0000oooo
ooooool03Oooool2o`0002gooooo0ol0000Tooooo`008oooool3o`0002Cooooo00?o0000oooooooo
ool0Hoooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo02gooooo0ol0000Tooooo`00
8oooool3o`0002?ooooo00?o0000ooooooooool0I?ooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo02kooooo0ol0000Tooooo`008Oooool3o`0002Cooooo00?o0000ooooooooool0IOoo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo037ooooo0ol0000Rooooo`008Oooool3
o`0002?ooooo00?o0000ooooooooool0I_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo03;ooooo0ol0000Rooooo`008Oooool3o`00027ooooo0_l0001Yooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool0<oooool3o`0002;ooooo000Qooooo`03o`000?oooooooooo023o
oooo00?o0000ooooooooool0JOooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo03Ko
oooo00?o0000ooooooooool08?ooool001oooooo0ol0000Qooooo`03o`000?oooooooooo06[ooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0gooooo`?o00008?ooool001oooooo0ol0
000Pooooo`03o`000?oooooooooo06_ooooo00?o0000ooooooooool01?ooool2o`0003[ooooo0ol0
000Pooooo`007oooool3o`0001kooooo0_l0001^ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool0>_ooool3o`00023ooooo000Mooooo`?o000000?oooooo`000?ooool07?ooool00ol0
003oooooooooo`1^ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0?Oooool3o`00
01kooooo000Mooooo`?o00000_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo06oo
oooo00Go0000ooooooooooooooooo`000010ooooo`?o00007_ooool001gooooo0ol00002ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool0L?ooool01?l0003oooooooooool00011oooo
o`?o00007_ooool001_ooooo0ol00005ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool0LOooool00ol0003oooooo`000014ooooo`?o00007?ooool001_ooooo0ol00006ooooo`03o`00
0?oooooooooo01?ooooo0_l0001dooooo`;o0000AOooool3o`0001cooooo000Kooooo`?o00001_oo
ool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo07?ooooo0_l00016ooooo`?o00007?oo
ool001_ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo07?ooooo00?o0000ooooool00000B?ooool00ol0003oooooooooo`0Jooooo`006Oooool3o`00
00[ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1cooooo`04o`000?oooooooooo
o`0004Sooooo0ol0000Jooooo`006Oooool3o`0000[ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`1cooooo`05o`000?ooooooooooooooool00000B?ooool3o`0001[ooooo000Ioooo
o`?o00002oooool00ol0003oooooooooo`0:ooooo`;o0000MOooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo04Kooooo0ol0000Jooooo`006Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool0M?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo04Sooooo00?o0000ooooooooool06?ooool001Oooooo0ol0000>ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0M?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo04Sooooo0ol0000Hooooo`005oooool3o`0000oooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`1dooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0B?oo
ool3o`0001Sooooo000Gooooo`?o00004?ooool00ol0003oooooooooo`02ooooo`;o0000MOooool2
o`0000Sooooo00?o0000ooooooooool0B?ooool3o`0001Sooooo000Eooooo`?o00004_ooool01Ol0
003ooooooooooooooooo000007Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`19
ooooo`?o00005oooool001Gooooo0ol0000Cooooo`03o`000?oooooo000007Kooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`19ooooo`?o00005oooool001Gooooo0ol0000Dooooo`03
o`000?oooooooooo07Cooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`19ooooo`?o
00005oooool001Kooooo00?o0000ooooooooool04_ooool2o`0007Gooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`1;ooooo`03o`000?oooooooooo01Gooooo000Dooooo`?o00004_oo
ool2o`0000;ooooo00?o0000ooooooooool0LOooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo04_ooooo0ol0000Eooooo`005?ooool3o`00017ooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`1_ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0Boooool3
o`0001Gooooo000Dooooo`?o00004?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
06gooooo0_l0000@ooooo`03o`000?oooooooooo04_ooooo0ol0000Eooooo`004_ooool3o`00017o
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`1[ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool0C?ooool3o`0001Cooooo000Booooo`?o00003oooool2o`0000[ooooo
00?o0000ooooooooool0JOooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo04cooooo
0ol0000Dooooo`004_ooool3o`0000kooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`1Xooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0C?ooool3o`0001Cooooo000C
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1V
ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0B
ooooo`004Oooool3o`0000gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`1Uoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0C_ooool3o`0001;ooooo000Aooooo`?o
00003?ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo06?ooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`1>ooooo`?o00004_ooool0017ooooo0ol0000:ooooo`;o0000
4oooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0
C_ooool3o`0001;ooooo000Booooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04ooo
ool00ol0003oooooooooo`1Oooooo`;o00006Oooool00ol0003oooooooooo`1?ooooo`03o`000?oo
oooooooo017ooooo000@ooooo`?o00002Oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo05gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`1?ooooo`?o00004Oooool0
013ooooo0ol00008ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0Foooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo04oooooo0ol0000Aooooo`004?ooool3o`0000Oo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`1Jooooo`03o`000?oooooooooo01_o
oooo00?o0000ooooooooool0Coooool3o`00017ooooo000Aooooo`03o`000?oooooooooo00Cooooo
0_l0000Looooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool07?ooool00ol0003ooooo
ooooo`1@ooooo`03o`000?oooooooooo013ooooo000?ooooo`?oZJVY1Oooool00ol0003ooooooooo
o`0Mooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool07Oooool00ol0003ooooooooo
o`1@ooooo`?o00004?ooool000oooooo0onYZJT4ooooo`03o`000?oooooooooo01kooooo00?o0000
ooooooooool0EOooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo053ooooo0ol0000@
ooooo`003oooool3ojVYZ@?ooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`1Boooo
o`;o00008Oooool00ol0003oooooooooo`1@ooooo`?o00004?ooool0013ooooo00?o0000oooooooo
ool00_l0000Tooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool08?ooool00ol0003o
ooooooooo`1Booooo`03o`000?oooooooooo00oooooo000>ooooo`?oZJVY00?oooooo`000?ooool0
9Oooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0
D_ooool3o`0000oooooo000>ooooo`?oZJVY00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`1=ooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0D_ooool3o`0000oooooo000>
ooooo`?oZJVY:_ooool00ol0003oooooooooo`1;ooooo`03o`000?oooooooooo02?ooooo00?o0000
ooooooooool0D_ooool3o`0000oooooo000>ooooo`;o0000:oooool00ol0003oooooooooo`1:oooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0E?ooool00ol0003oooooooooo`0=oooo
o`003Oooool3ojVYZBcooooo00?o0000ooooooooool0B?ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo05?ooooo0ol0000>ooooo`003Oooool3ojVYZBgooooo00?o0000ooooooooool0
A_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo05?ooooo0ol0000>ooooo`003Ooo
ool3ojVYZBgooooo00?o0000ooooooooool0A?ooool2o`0002Wooooo00?o0000ooooooooool0Dooo
ool3o`0000kooooo000=ooooo`?oZJVY;_ooool00ol0003oooooooooo`12ooooo`03o`000?oooooo
oooo02Wooooo00?o0000ooooooooool0Doooool3o`0000kooooo000=ooooo`?oZJVY;oooool00ol0
003oooooooooo`10ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0Doooool3o`00
00kooooo000=ooooo`?oZJVY;oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo02_o
oooo00?o0000ooooooooool0Doooool3o`0000kooooo000>ooooo`03o`000?oooooooooo02oooooo
00?o0000ooooooooool0?Oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo05Cooooo
00?o0000ooooooooool03Oooool000cooooo0onYZJTbooooo`03o`000?oooooooooo03_ooooo00?o
0000ooooooooool0;Oooool00ol0003oooooooooo`1Dooooo`?o00003Oooool000cooooo0onYZJTb
ooooo`03o`000?oooooooooo03[ooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`1D
ooooo`?o00003Oooool000cooooo0onYZJTcooooo`03o`000?oooooooooo03Oooooo0_l0000aoooo
o`03o`000?oooooooooo05Cooooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`0cooooo`03
o`000?oooooooooo03Gooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`1Eooooo`03
o`000?oooooooooo00cooooo000<ooooo`?o0000=?ooool00ol0003oooooooooo`0dooooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool0E?ooool3o`0000gooooo000<ooooo`?o0000=Ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0E?oo
ool3o`0000gooooo000<ooooo`?o0000=_ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooo
oooo03Cooooo00?o0000ooooooooool0E?ooool3o`0000gooooo000=ooooo`03o`000?oooooooooo
03Gooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo
05Gooooo00?o0000ooooooooool03?ooool000_ooooo0ol0000hooooo`03o`000?oooooooooo02go
oooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`1Eooooo`?o00003?ooool000_ooooo
0ol0000iooooo`03o`000?oooooooooo02[ooooo0_l0000iooooo`03o`000?oooooooooo05Gooooo
0ol0000<ooooo`002oooool3o`0003Wooooo00?o0000ooooooooool0:Oooool00ol0003ooooooooo
o`0iooooo`03o`000?oooooooooo05Gooooo0ol0000<ooooo`003?ooool00ol0003oooooooooo`0i
ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`1F
ooooo`03o`000?oooooooooo00_ooooo000;ooooo`?o0000>oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0EOooool3o`0000cooooo000;ooooo`?o
0000>oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo03cooooo00?o0000oooooooo
ool0EOooool3o`0000cooooo000;ooooo`?o0000??ooool00ol0003oooooooooo`0Rooooo`03o`00
0?oooooooooo03gooooo00?o0000ooooooooool0EOooool3o`0000cooooo000<ooooo`03o`000?oo
oooooooo03cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo05Kooooo00?o0000ooooooooool02oooool000_ooooo0ol0000mooooo`03o`000?oooooo
oooo01oooooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`1Eooooo`?o00003?ooool0
00_ooooo0ol0000nooooo`03o`000?oooooooooo01cooooo0_l00012ooooo`03o`000?oooooooooo
05Gooooo0ol0000<ooooo`002oooool3o`0003oooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`12ooooo`03o`000?oooooooooo05Gooooo0ol0000<ooooo`003?ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0@oooool00ol0003ooooo
ooooo`1Fooooo`03o`000?oooooooooo00_ooooo000;ooooo`?o0000@?ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo04Cooooo00?o0000ooooooooool0EOooool3o`0000cooooo000;
ooooo`?o0000@Oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04Gooooo00?o0000
ooooooooool0EOooool3o`0000cooooo000;ooooo`?o0000@Oooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo04Kooooo00?o0000ooooooooool0EOooool3o`0000cooooo000<ooooo`03
o`000?oooooooooo047ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`17ooooo`03
o`000?oooooooooo05Kooooo00?o0000ooooooooool02oooool000_ooooo0ol00012ooooo`03o`00
0?oooooooooo013ooooo0_l0001:ooooo`03o`000?oooooooooo05Gooooo0ol0000<ooooo`002ooo
ool<o`0003[ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1:ooooo`03o`000?oo
oooooooo05Gooooo0ol0000<ooooo`002oooool3o`0000Wooooo5Ol0000Vooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool0Boooool00ol0003oooooooooo`1Eooooo`?o00003?ooool0
00cooooo00?o0000ooooooooool07OoooolEo`00017ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`1<ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool02oooool000co
oooo0ol0000boooooaGo00002Oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo05Co
oooo0ol0000=ooooo`003?ooool3o`0004Gooooo00?o0000ooooool000005?l00014ooooo`03o`00
0?oooooooooo05Cooooo0ol0000=ooooo`003?ooool3o`0004Gooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0;oooooaGo0000;oooool00ol0003oooooooooo`1Dooooo`?o00003Ooo
ool000gooooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo027ooooo5?l0000Kooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool03?ooool0
00cooooo0ol00017ooooo`04o`000?ooooooooooooooo`;o0000>?oooolEo`0000Kooooo00?o0000
ooooooooool0E?ooool3o`0000gooooo000<ooooo`?o0000Aoooool01?l0003oooooooooool0001?
oooooaGo0000B?ooool3o`0000gooooo000<ooooo`?o0000B?ooool2o`0005Kooooo00?o0000oooo
ooooool03?oooolEo`0003?ooooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`17ooooo`;o
0000E_ooool00ol0003oooooooooo`0QoooooaGo00007oooool00ol0003oooooooooo`0<ooooo`00
3Oooool3o`0004Kooooo00?o0000ooooool00000E_ooool00ol0003oooooooooo`0foooooaGo0000
2?ooool3o`0000kooooo000=ooooo`?o0000AOooool01Ol0003ooooooooooooooooo000005Gooooo
00?o0000ooooooooool0Boooool;o`0000kooooo000=ooooo`?o0000A?ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool0Doooool3o`0000kooooo000>
ooooo`03o`000?oooooooooo047ooooo0_l00006ooooo`03o`000?oooooooooo05;ooooo00?o0000
ooooooooool0E?ooool00ol0003oooooooooo`0=ooooo`003Oooool3o`00047ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`1Aooooo`03o`000?oooooooooo05?ooooo0ol0000>oooo
o`003Oooool3o`00043ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1@ooooo`03
o`000?oooooooooo05?ooooo0ol0000>ooooo`003Oooool3o`0003oooooo00?o0000ooooooooool0
2_ooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo05?ooooo0ol0000>ooooo`003ooo
ool00ol0003oooooooooo`0looooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0Cooo
ool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo00kooooo000>ooooo`?o0000??ooool0
0ol0003oooooooooo`0>ooooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool0D_ooool3
o`0000oooooo000>ooooo`?o0000>oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo
04kooooo00?o0000ooooooooool0D_ooool3o`0000oooooo000>ooooo`?o0000>_ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0D_ooool3o`0000oo
oooo000?ooooo`?o0000=oooool2o`0001Gooooo00?o0000ooooooooool0C?ooool00ol0003ooooo
ooooo`1Aooooo`?o00004?ooool000oooooo0ol0000fooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool0C?ooool00ol0003oooooooooo`1Aooooo`?o00004?ooool000oooooo0ol0000e
ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`1B
ooooo`?o00004?ooool0013ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo04Wooooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`0?oooo
o`004?ooool3o`0003;ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`19ooooo`03
o`000?oooooooooo057ooooo0ol0000Aooooo`004?ooool3o`00037ooooo00?o0000ooooooooool0
7?ooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo057ooooo0ol0000Aooooo`004?oo
ool3o`00033ooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`17ooooo`03o`000?oo
oooooooo057ooooo0ol0000Aooooo`004Oooool00ol0003oooooooooo`0]ooooo`;o00008Oooool0
0ol0003oooooooooo`17ooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool04?ooool0
017ooooo0ol0000/ooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0A_ooool00ol0
003oooooooooo`1@ooooo`?o00004_ooool0017ooooo0ol0000[ooooo`03o`000?oooooooooo02Co
oooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`1@ooooo`?o00004_ooool0017ooooo
0ol0000Zooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0AOooool00ol0003ooooo
ooooo`1@ooooo`?o00004_ooool001?ooooo00?o0000ooooooooool09oooool00ol0003ooooooooo
o`0Wooooo`03o`000?oooooooooo04Cooooo00?o0000ooooooooool0D?ooool00ol0003ooooooooo
o`0Booooo`004_ooool3o`0002Oooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`13
ooooo`03o`000?oooooooooo04kooooo0ol0000Dooooo`004_ooool3o`0002Kooooo00?o0000oooo
ooooool0:_ooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo04kooooo0ol0000Doooo
o`004_ooool3o`0002Gooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`12ooooo`03
o`000?oooooooooo04kooooo0ol0000Dooooo`005?ooool3o`00027ooooo0_l0000`ooooo`03o`00
0?oooooooooo047ooooo00?o0000ooooooooool0COooool3o`0001Gooooo000Dooooo`?o00008?oo
ool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool0COoo
ool3o`0001Gooooo000Dooooo`?o00007oooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo043ooooo00?o0000ooooooooool0COooool3o`0001Gooooo000Eooooo`03o`000?oooooooooo
01gooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo
04kooooo00?o0000ooooooooool05?ooool001Gooooo0ol0000Looooo`03o`000?oooooooooo03Go
oooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`1;ooooo`?o00005oooool001Gooooo
0ol0000Kooooo`03o`000?oooooooooo03Oooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`1;ooooo`?o00005oooool001Gooooo0ol0000Jooooo`03o`000?oooooooooo03Wooooo00?o
0000ooooooooool0?Oooool00ol0003oooooooooo`1;ooooo`?o00005oooool001Oooooo00?o0000
ooooooooool05_ooool2o`0003cooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`1<
ooooo`03o`000?oooooooooo01Kooooo000Gooooo`?o00005Oooool00ol0003oooooooooo`0moooo
o`03o`000?oooooooooo03cooooo00?o0000ooooooooool0B_ooool3o`0001Sooooo000Gooooo`?o
00005?ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03cooooo00?o0000oooooooo
ool0B_ooool3o`0001Sooooo000Gooooo`?o00004oooool00ol0003oooooooooo`10ooooo`03o`00
0?oooooooooo03_ooooo00?o0000ooooooooool0B_ooool3o`0001Sooooo000Iooooo`?o00004?oo
ool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo03[ooooo00?o0000ooooooooool0B?oo
ool3o`0001[ooooo000Iooooo`?o00003oooool00ol0003oooooooooo`13ooooo`03o`000?oooooo
oooo03[ooooo00?o0000ooooooooool0B?ooool3o`0001[ooooo000Iooooo`?o00003_ooool00ol0
003oooooooooo`15ooooo`03o`000?oooooooooo03Wooooo00?o0000ooooooooool0B?ooool3o`00
01[ooooo000Kooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0Aoooool00ol0003o
ooooooooo`0hooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool06_ooool001_ooooo
0ol00009ooooo`;o0000B_ooool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo04Kooooo
0ol0000Looooo`006oooool3o`0000Sooooo00?o0000ooooooooool0Boooool00ol0003ooooooooo
o`0gooooo`03o`000?oooooooooo04Kooooo0ol0000Looooo`006oooool3o`0000Oooooo00?o0000
ooooooooool0COooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo04Kooooo0ol0000L
ooooo`007Oooool3o`0000Cooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0foooo
o`03o`000?oooooooooo04Cooooo0ol0000Nooooo`007Oooool3o`0000?ooooo00?o0000oooooooo
ool0D?ooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo04Cooooo0ol0000Nooooo`00
7Oooool3o`0000;ooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0dooooo`03o`00
0?oooooooooo04Cooooo0ol0000Nooooo`007oooool00ol0003oooooo`00001Eooooo`03o`000?oo
oooooooo03Cooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`0Nooooo`007oooool3
o`0005Kooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`12ooooo`?o00008?ooool0
01oooooo0ol0001Gooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0@_ooool3o`00
023ooooo000Oooooo`?o0000Eoooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo04;o
oooo0ol0000Pooooo`008Oooool3o`0005Kooooo00?o0000ooooooooool0<Oooool00ol0003ooooo
ooooo`10ooooo`?o00008_ooool0027ooooo0ol0001Gooooo`03o`000?oooooooooo033ooooo00?o
0000ooooooooool0@?ooool3o`0002;ooooo000Qooooo`?o0000Eoooool00ol0003oooooooooo`0`
ooooo`03o`000?oooooooooo043ooooo0ol0000Rooooo`008oooool3o`0005Kooooo00?o0000oooo
ooooool0;oooool00ol0003oooooooooo`0nooooo`?o00009?ooool002?ooooo0ol0001Gooooo`03
o`000?oooooooooo02kooooo00?o0000ooooooooool0?_ooool3o`0002Cooooo000Sooooo`?o0000
Eoooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo03kooooo0ol0000Tooooo`009Ooo
ool3o`0005Kooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0kooooo`?o00009ooo
ool002Gooooo0ol0001Gooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool0>oooool3
o`0002Oooooo000Uooooo`?o0000Eoooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo
03_ooooo0ol0000Wooooo`00:?ooool3o`0005Gooooo00?o0000ooooooooool0:oooool00ol0003o
ooooooooo`0iooooo`?o0000:Oooool002Sooooo0ol0001Fooooo`03o`000?oooooooooo02[ooooo
00?o0000ooooooooool0>Oooool3o`0002Wooooo000Xooooo`?o0000E_ooool00ol0003ooooooooo
o`0Zooooo`03o`000?oooooooooo03Wooooo0ol0000Yooooo`00:_ooool3o`0005Gooooo00?o0000
ooooooooool0:Oooool00ol0003oooooooooo`0fooooo`?o0000;?ooool002[ooooo0ol0001Foooo
o`03o`000?oooooooooo02Sooooo00?o0000ooooooooool0=_ooool3o`0002cooooo000Zooooo`?o
0000E_ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo03Kooooo0ol0000/ooooo`00
;Oooool3o`0005Cooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0dooooo`?o0000
;_ooool002gooooo0ol0001Eooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0=?oo
ool3o`0002kooooo000]ooooo`?o0000EOooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo03Cooooo0ol0000^ooooo`00<?ooool3o`0005?ooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0aooooo`?o0000<Oooool0033ooooo0ol0001Dooooo`03o`000?oooooooooo02Co
oooo00?o0000ooooooooool0<Oooool3o`00037ooooo000`ooooo`?o0000E?ooool00ol0003ooooo
ooooo`0Tooooo`03o`000?oooooooooo037ooooo0ol0000aooooo`00<oooool3o`0005;ooooo00?o
0000ooooooooool08oooool00ol0003oooooooooo`0^ooooo`?o0000=?ooool003?ooooo0ol0001C
ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0;oooool3o`0003Cooooo000coooo
o`Co0000D_ooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo02kooooo1?l0000doooo
o`00=_ooool3o`00057ooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0/ooooo`?o
0000=oooool003Kooooo0ol0001Booooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0
;?ooool3o`0003Oooooo000fooooo`Ko0000Coooool00ol0003oooooooooo`0Oooooo`03o`000?oo
oooooooo02Wooooo1_l0000gooooo`00>Oooool3o`00053ooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0Yooooo`?o0000>_ooool003Wooooo1?l0001@ooooo`03o`000?oooooooooo
01gooooo00?o0000ooooooooool0:?ooool4o`0003[ooooo000looooo`?o0000C_ooool00ol0003o
ooooooooo`0Mooooo`03o`000?oooooooooo02Kooooo0ol0000mooooo`00??ooool3o`0004oooooo
00?o0000ooooooooool07?ooool00ol0003oooooooooo`0Vooooo`?o0000?Oooool003cooooo1_l0
001<ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool08oooool6o`0003gooooo000o
ooooo`?o0000COooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo02?ooooo0ol00010
ooooo`00?oooool6o`0004_ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Poooo
o`Ko0000@?ooool004;ooooo0ol0001;ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool08?ooool3o`0004?ooooo0012ooooo`Ko0000BOooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01cooooo1ol00013ooooo`00AOooool4o`0004Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Looooo`?o0000Aoooool004Gooooo0ol000001?oooooo0000o`000?l0
0016ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool6o`0004Oooooo0019
ooooo`?o0000Aoooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01Wooooo0ol0001:
ooooo`00BOooool6o`0004Gooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Foooo
o`Ko0000B_ooool004cooooo0ol00015ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool05_ooool3o`0004gooooo001<ooooo`Oo0000@_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01;ooooo1ol0001=ooooo`00D?ooool3o`0004?ooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Booooo`?o0000DOooool0053ooooo1_l00010ooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool03_ooool7o`00057ooooo001Cooooo`?o0000@Oooool00ol0
003oooooooooo`0Cooooo`03o`000?oooooooooo00kooooo0ol0001Eooooo`00Doooool7o`0003ko
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0;ooooo`Ko0000EOooool005Oooooo
1ol0000jooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool01oooool7o`0005Sooooo
001Gooooo`?o000000?oooooo`000?l000000_l0000jooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool01oooool3o`000004ooooool0003o0000o`0005Sooooo001Kooooo`Ko0000>Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Cooooo1_l0001Looooo`00G_ooool7
o`0003Gooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07o`0005oooooo001Noooo
o`?o000000?oooooo`000?l000001Ol0000booooo`03o`000?oooooooooo00kooooo1ol000001?oo
oooo0000o`000?l0001Oooooo`00H_ooool3o`000003ooooool0003o000000Go0000;oooool00ol0
003oooooooooo`09ooooo`Oo000000Coooooo`000?l0003o0000Hoooool006Kooooo0ol000000ooo
oooo0000o`000004o`0002cooooo00?o0000ooooooooool01Oooool7o`000004ooooool0003o0000
o`0006Oooooo001Zooooo`[o000000Coooooo`000?l0003o00009Oooool6o`000005ooooool0003o
0000o`000?ooool00ol0001[ooooo`00KOooool3o`000003ooooool0003o000000Wo000000Cooooo
o`000?l0003o00005Oooool3o`000003ooooool0003o000000co0000Koooool0077ooooo0ol00000
1Ooooooo0000o`000?l0003ooooo00_o000000Goooooo`000?l0003o0000ooooo`03o`000005oooo
ool0003o0000o`000?ooool02ol000000ooooooo0000o`000004o`0007?ooooo001iooooo`?o0000
00Goooooo`000?l0003o0000ooooo`0Co`000005ooooool0003o0000o`000?ooool00ol0001joooo
o`00POooool3o`000005ooooool0003o0000o`000?ooool00ol000001Ooooooo0000o`000?l0003o
oooo00?o0000P_ooool00?oooooo5oooool00?oooooo5oooool00?oooooo5oooool00?oooooo5ooo
ool00?oooooo5oooool00?oooooo5oooool00?oooooo5oooool00?oooooo5oooool00?oooooo5ooo
ool00?oooooo5oooool00?oooooo5oooool00001\
\>"],
  ImageRangeCache->{{{0, 277}, {277, 0}} -> {-0.0500054, -0.0500054, \
0.00397116, 0.00397116}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Let's take a closer look at what's going on.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(gr = ExtractGraph[pd2];\)\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Although the edges of the update neighborhood are coloured automatically \
by the ",
  StyleBox["LocalInteractionModel", "Input"],
  " command if you tell it to display the update neighborhood. However, since \
we suppressed them, but now want to include them, we have to add the edges of \
the update neighborhood to ",
  StyleBox["gr", "Input"],
  " by hand. This isn't hard. All we have to do is make sure that each edge \
we want to add includes the right graphics information."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  " represents edges by an ordered pair ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  " with an extra set of braces around it: ",
  Cell[BoxData[
      \(TraditionalForm\`{{x, y}}\)]],
  ". The extra set of braces is so that additional information about the edge \
can be included, like its weight, thickness, or color. To color an edge blue, \
we need to add ",
  StyleBox["EdgeColor\[Rule]Blue", "Input"],
  " after the (inner) ordered pair ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  ", i.e., {",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  ", ",
  StyleBox["EdgeColor\[Rule]Blue", "Input"],
  "}. The way to do this is to take the Cartesian product of the edge set \
with the singleton set ",
  StyleBox["{EdgeColor\[Rule]Blue}", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(e1 = 
        CartesianProduct[Edges[g2], {EdgeColor \[Rule] Blue}];\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["To verify, here's what the first edge looks like:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1\[LeftDoubleBracket]1\[RightDoubleBracket]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{1, 2}, EdgeColor \[Rule] RGBColor[0.`, 0.`, 1.`]}\)], "Output",
  CellLabel->"Out[17]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gr = AddEdges[gr, e1];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[gr, VertexStyle \[Rule] Disk[Small]];\)\)}], "Input",
  CellLabel->"In[18]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95351 .52139 L
.95365 .52854 L
s
0 0 1 r
.95432 .51428 m
.95446 .52143 L
.95365 .52854 L
s
.95432 .51428 m
.95253 .54278 L
s
.95432 .51428 m
.95432 .48572 L
s
0 0 0 r
.95432 .51428 m
.95396 .50713 L
.95455 .5 L
s
0 0 1 r
.95432 .51428 m
.95491 .50715 L
.95455 .5 L
s
0 0 0 r
.95365 .52854 m
.95261 .53562 L
.95253 .54278 L
s
0 0 1 r
.95365 .52854 m
.95356 .5357 L
.95253 .54278 L
s
.95365 .52854 m
.95096 .55697 L
s
.95365 .52854 m
.95455 .5 L
s
0 0 0 r
.95253 .54278 m
.95127 .54982 L
.95096 .55697 L
s
0 0 1 r
.95253 .54278 m
.95222 .54993 L
.95096 .55697 L
s
.95253 .54278 m
.94895 .57111 L
s
0 0 0 r
.95096 .55697 m
.94948 .56397 L
.94895 .57111 L
s
0 0 1 r
.95096 .55697 m
.95043 .56411 L
.94895 .57111 L
s
.95096 .55697 m
.94649 .58517 L
s
.95096 .55697 m
.04635 .52854 L
s
0 0 0 r
.94895 .57111 m
.94725 .57806 L
.94649 .58517 L
s
0 0 1 r
.94895 .57111 m
.94819 .57822 L
.94649 .58517 L
s
.94895 .57111 m
.9436 .59916 L
s
.94895 .57111 m
.04747 .54278 L
s
0 0 0 r
.94895 .57111 m
.49623 .57991 L
.04635 .52854 L
s
0 0 1 r
.94895 .57111 m
.49907 .51974 L
.04635 .52854 L
s
.94895 .57111 m
.04568 .51428 L
s
0 0 0 r
.94649 .58517 m
.94458 .59207 L
.9436 .59916 L
s
0 0 1 r
.94649 .58517 m
.94551 .59226 L
.9436 .59916 L
s
.94649 .58517 m
.94027 .61304 L
s
.94649 .58517 m
.04635 .52854 L
s
0 0 0 r
.9436 .59916 m
.94147 .60599 L
.94027 .61304 L
s
0 0 1 r
.9436 .59916 m
.94239 .60621 L
.94027 .61304 L
s
.9436 .59916 m
.9365 .62681 L
s
0 0 0 r
.94027 .61304 m
.93792 .6198 L
.9365 .62681 L
s
0 0 1 r
.94027 .61304 m
.93884 .62005 L
.9365 .62681 L
s
.94027 .61304 m
.9323 .64046 L
s
0 0 0 r
.9365 .62681 m
.93394 .6335 L
.9323 .64046 L
s
0 0 1 r
.9365 .62681 m
.93485 .63378 L
.9323 .64046 L
s
.9365 .62681 m
.92767 .65397 L
s
0 0 0 r
.9323 .64046 m
.92954 .64706 L
.92767 .65397 L
s
0 0 1 r
.9323 .64046 m
.93044 .64737 L
.92767 .65397 L
s
.9323 .64046 m
.92263 .66733 L
s
0 0 0 r
.92767 .65397 m
.9247 .66048 L
.92263 .66733 L
s
0 0 1 r
.92767 .65397 m
.92559 .66082 L
.92263 .66733 L
s
.92767 .65397 m
.91716 .68052 L
s
0 0 0 r
.92263 .66733 m
.91945 .67374 L
.91716 .68052 L
s
0 0 1 r
.92263 .66733 m
.92033 .67411 L
.91716 .68052 L
s
.92263 .66733 m
.91129 .69354 L
s
0 0 0 r
.91716 .68052 m
.91379 .68683 L
.91129 .69354 L
s
0 0 1 r
.91716 .68052 m
.91466 .68722 L
.91129 .69354 L
s
.91716 .68052 m
.905 .70636 L
s
0 0 0 r
.91129 .69354 m
.90772 .69974 L
.905 .70636 L
s
0 0 1 r
.91129 .69354 m
.90857 .70016 L
.905 .70636 L
s
.91129 .69354 m
.89832 .71898 L
s
0 0 0 r
.905 .70636 m
.90124 .71245 L
.89832 .71898 L
s
0 0 1 r
.905 .70636 m
.90208 .71289 L
.89832 .71898 L
s
.905 .70636 m
.89125 .73138 L
s
0 0 0 r
.89832 .71898 m
.89437 .72494 L
.89125 .73138 L
s
0 0 1 r
.89832 .71898 m
.8952 .72542 L
.89125 .73138 L
s
.89832 .71898 m
.88379 .74356 L
s
0 0 0 r
.89125 .73138 m
.88711 .73722 L
.88379 .74356 L
s
0 0 1 r
.89125 .73138 m
.88792 .73772 L
.88379 .74356 L
s
.89125 .73138 m
.87595 .75549 L
s
0 0 0 r
.88379 .74356 m
.87947 .74926 L
.87595 .75549 L
s
0 0 1 r
.88379 .74356 m
.88026 .74979 L
.87595 .75549 L
s
.88379 .74356 m
.86773 .76718 L
s
0 0 0 r
.87595 .75549 m
.87145 .76106 L
.86773 .76718 L
s
0 0 1 r
.87595 .75549 m
.87223 .76161 L
.86773 .76718 L
s
.87595 .75549 m
.85916 .77859 L
s
0 0 0 r
.86773 .76718 m
.86307 .7726 L
.85916 .77859 L
s
0 0 1 r
.86773 .76718 m
.86383 .77317 L
.85916 .77859 L
s
.86773 .76718 m
.85023 .78974 L
s
0 0 0 r
.85916 .77859 m
.85433 .78387 L
.85023 .78974 L
s
0 0 1 r
.85916 .77859 m
.85507 .78446 L
.85023 .78974 L
s
.85916 .77859 m
.84096 .8006 L
s
0 0 0 r
.85023 .78974 m
.84523 .79486 L
.84096 .8006 L
s
0 0 1 r
.85023 .78974 m
.84596 .79548 L
.84096 .8006 L
s
.85023 .78974 m
.83135 .81116 L
s
0 0 0 r
.84096 .8006 m
.8358 .80556 L
.83135 .81116 L
s
0 0 1 r
.84096 .8006 m
.83651 .8062 L
.83135 .81116 L
s
.84096 .8006 m
.82141 .82141 L
s
0 0 0 r
.83135 .81116 m
.82604 .81595 L
.82141 .82141 L
s
0 0 1 r
.83135 .81116 m
.82672 .81662 L
.82141 .82141 L
s
.83135 .81116 m
.81116 .83135 L
s
0 0 0 r
.82141 .82141 m
.81595 .82604 L
.81116 .83135 L
s
0 0 1 r
.82141 .82141 m
.81662 .82672 L
.81116 .83135 L
s
.82141 .82141 m
.8006 .84096 L
s
0 0 0 r
.81116 .83135 m
.80556 .8358 L
.8006 .84096 L
s
0 0 1 r
.81116 .83135 m
.8062 .83651 L
.8006 .84096 L
s
.81116 .83135 m
.78974 .85023 L
s
0 0 0 r
.8006 .84096 m
.79486 .84523 L
.78974 .85023 L
s
0 0 1 r
.8006 .84096 m
.79548 .84596 L
.78974 .85023 L
s
.8006 .84096 m
.77859 .85916 L
s
0 0 0 r
.78974 .85023 m
.78387 .85433 L
.77859 .85916 L
s
0 0 1 r
.78974 .85023 m
.78446 .85507 L
.77859 .85916 L
s
.78974 .85023 m
.76718 .86773 L
s
0 0 0 r
.77859 .85916 m
.7726 .86307 L
.76718 .86773 L
s
0 0 1 r
.77859 .85916 m
.77317 .86383 L
.76718 .86773 L
s
.77859 .85916 m
.75549 .87595 L
s
0 0 0 r
.76718 .86773 m
.76106 .87145 L
.75549 .87595 L
s
0 0 1 r
.76718 .86773 m
.76161 .87223 L
.75549 .87595 L
s
.76718 .86773 m
.74356 .88379 L
s
0 0 0 r
.75549 .87595 m
.74926 .87947 L
.74356 .88379 L
s
0 0 1 r
.75549 .87595 m
.74979 .88026 L
.74356 .88379 L
s
.75549 .87595 m
.73138 .89125 L
s
0 0 0 r
.74356 .88379 m
.73722 .88711 L
.73138 .89125 L
s
0 0 1 r
.74356 .88379 m
.73772 .88792 L
.73138 .89125 L
s
.74356 .88379 m
.71898 .89832 L
s
0 0 0 r
.73138 .89125 m
.72494 .89437 L
.71898 .89832 L
s
0 0 1 r
.73138 .89125 m
.72542 .8952 L
.71898 .89832 L
s
.73138 .89125 m
.70636 .905 L
s
0 0 0 r
.71898 .89832 m
.71245 .90124 L
.70636 .905 L
s
0 0 1 r
.71898 .89832 m
.71289 .90208 L
.70636 .905 L
s
.71898 .89832 m
.69354 .91129 L
s
0 0 0 r
.70636 .905 m
.69974 .90772 L
.69354 .91129 L
s
0 0 1 r
.70636 .905 m
.70016 .90857 L
.69354 .91129 L
s
.70636 .905 m
.68052 .91716 L
s
0 0 0 r
.69354 .91129 m
.68683 .91379 L
.68052 .91716 L
s
0 0 1 r
.69354 .91129 m
.68722 .91466 L
.68052 .91716 L
s
.69354 .91129 m
.66733 .92263 L
s
0 0 0 r
.68052 .91716 m
.67374 .91945 L
.66733 .92263 L
s
0 0 1 r
.68052 .91716 m
.67411 .92033 L
.66733 .92263 L
s
.68052 .91716 m
.65397 .92767 L
s
0 0 0 r
.66733 .92263 m
.66048 .9247 L
.65397 .92767 L
s
0 0 1 r
.66733 .92263 m
.66082 .92559 L
.65397 .92767 L
s
.66733 .92263 m
.64046 .9323 L
s
0 0 0 r
.65397 .92767 m
.64706 .92954 L
.64046 .9323 L
s
0 0 1 r
.65397 .92767 m
.64737 .93044 L
.64046 .9323 L
s
.65397 .92767 m
.62681 .9365 L
s
0 0 0 r
.64046 .9323 m
.6335 .93394 L
.62681 .9365 L
s
0 0 1 r
.64046 .9323 m
.63378 .93485 L
.62681 .9365 L
s
.64046 .9323 m
.61304 .94027 L
s
.64046 .9323 m
.64046 .0677 L
s
0 0 0 r
.62681 .9365 m
.6198 .93792 L
.61304 .94027 L
s
0 0 1 r
.62681 .9365 m
.62005 .93884 L
.61304 .94027 L
s
.62681 .9365 m
.59916 .9436 L
s
.62681 .9365 m
.62681 .0635 L
s
0 0 0 r
.62681 .9365 m
.60468 .50164 L
.64046 .0677 L
s
0 0 1 r
.62681 .9365 m
.6626 .50255 L
.64046 .0677 L
s
.62681 .9365 m
.65397 .07233 L
s
0 0 0 r
.61304 .94027 m
.60599 .94147 L
.59916 .9436 L
s
0 0 1 r
.61304 .94027 m
.60621 .94239 L
.59916 .9436 L
s
.61304 .94027 m
.58517 .94649 L
s
.61304 .94027 m
.64046 .0677 L
s
0 0 0 r
.59916 .9436 m
.59207 .94458 L
.58517 .94649 L
s
0 0 1 r
.59916 .9436 m
.59226 .94551 L
.58517 .94649 L
s
.59916 .9436 m
.57111 .94895 L
s
0 0 0 r
.58517 .94649 m
.57806 .94725 L
.57111 .94895 L
s
0 0 1 r
.58517 .94649 m
.57822 .94819 L
.57111 .94895 L
s
.58517 .94649 m
.55697 .95096 L
s
.58517 .94649 m
.11621 .25644 L
s
0 0 0 r
.57111 .94895 m
.56397 .94948 L
.55697 .95096 L
s
0 0 1 r
.57111 .94895 m
.56411 .95043 L
.55697 .95096 L
s
.57111 .94895 m
.54278 .95253 L
s
.57111 .94895 m
.10875 .26862 L
s
0 0 0 r
.57111 .94895 m
.32058 .61786 L
.11621 .25644 L
s
0 0 1 r
.57111 .94895 m
.36674 .58753 L
.11621 .25644 L
s
.57111 .94895 m
.12405 .24451 L
s
0 0 0 r
.55697 .95096 m
.54982 .95127 L
.54278 .95253 L
s
0 0 1 r
.55697 .95096 m
.54993 .95222 L
.54278 .95253 L
s
.55697 .95096 m
.52854 .95365 L
s
.55697 .95096 m
.11621 .25644 L
s
0 0 0 r
.54278 .95253 m
.53562 .95261 L
.52854 .95365 L
s
0 0 1 r
.54278 .95253 m
.5357 .95356 L
.52854 .95365 L
s
.54278 .95253 m
.51428 .95432 L
s
0 0 0 r
.52854 .95365 m
.52139 .95351 L
.51428 .95432 L
s
0 0 1 r
.52854 .95365 m
.52143 .95446 L
.51428 .95432 L
s
.52854 .95365 m
.5 .95455 L
s
0 0 0 r
.51428 .95432 m
.50713 .95396 L
.5 .95455 L
s
0 0 1 r
.51428 .95432 m
.50715 .95491 L
.5 .95455 L
s
.51428 .95432 m
.48572 .95432 L
s
0 0 0 r
.5 .95455 m
.49285 .95491 L
.48572 .95432 L
s
0 0 1 r
.5 .95455 m
.49287 .95396 L
.48572 .95432 L
s
.5 .95455 m
.47146 .95365 L
s
0 0 0 r
.48572 .95432 m
.47857 .95446 L
.47146 .95365 L
s
0 0 1 r
.48572 .95432 m
.47861 .95351 L
.47146 .95365 L
s
.48572 .95432 m
.45722 .95253 L
s
0 0 0 r
.47146 .95365 m
.4643 .95356 L
.45722 .95253 L
s
0 0 1 r
.47146 .95365 m
.46438 .95261 L
.45722 .95253 L
s
.47146 .95365 m
.44303 .95096 L
s
0 0 0 r
.45722 .95253 m
.45007 .95222 L
.44303 .95096 L
s
0 0 1 r
.45722 .95253 m
.45018 .95127 L
.44303 .95096 L
s
.45722 .95253 m
.42889 .94895 L
s
0 0 0 r
.44303 .95096 m
.43589 .95043 L
.42889 .94895 L
s
0 0 1 r
.44303 .95096 m
.43603 .94948 L
.42889 .94895 L
s
.44303 .95096 m
.41483 .94649 L
s
0 0 0 r
.42889 .94895 m
.42178 .94819 L
.41483 .94649 L
s
0 0 1 r
.42889 .94895 m
.42194 .94725 L
.41483 .94649 L
s
.42889 .94895 m
.40084 .9436 L
s
0 0 0 r
.41483 .94649 m
.40774 .94551 L
.40084 .9436 L
s
0 0 1 r
.41483 .94649 m
.40793 .94458 L
.40084 .9436 L
s
.41483 .94649 m
.38696 .94027 L
s
0 0 0 r
.40084 .9436 m
.39379 .94239 L
.38696 .94027 L
s
0 0 1 r
.40084 .9436 m
.39401 .94147 L
.38696 .94027 L
s
.40084 .9436 m
.37319 .9365 L
s
0 0 0 r
.38696 .94027 m
.37995 .93884 L
.37319 .9365 L
s
0 0 1 r
.38696 .94027 m
.3802 .93792 L
.37319 .9365 L
s
.38696 .94027 m
.35954 .9323 L
s
0 0 0 r
.37319 .9365 m
.36622 .93485 L
.35954 .9323 L
s
0 0 1 r
.37319 .9365 m
.3665 .93394 L
.35954 .9323 L
s
.37319 .9365 m
.34603 .92767 L
s
0 0 0 r
.35954 .9323 m
.35263 .93044 L
.34603 .92767 L
s
0 0 1 r
.35954 .9323 m
.35294 .92954 L
.34603 .92767 L
s
.35954 .9323 m
.33267 .92263 L
s
0 0 0 r
.34603 .92767 m
.33918 .92559 L
.33267 .92263 L
s
0 0 1 r
.34603 .92767 m
.33952 .9247 L
.33267 .92263 L
s
.34603 .92767 m
.31948 .91716 L
s
0 0 0 r
.33267 .92263 m
.32589 .92033 L
.31948 .91716 L
s
0 0 1 r
.33267 .92263 m
.32626 .91945 L
.31948 .91716 L
s
.33267 .92263 m
.30646 .91129 L
s
0 0 0 r
.31948 .91716 m
.31278 .91466 L
.30646 .91129 L
s
0 0 1 r
.31948 .91716 m
.31317 .91379 L
.30646 .91129 L
s
.31948 .91716 m
.29364 .905 L
s
0 0 0 r
.30646 .91129 m
.29984 .90857 L
.29364 .905 L
s
0 0 1 r
.30646 .91129 m
.30026 .90772 L
.29364 .905 L
s
.30646 .91129 m
.28102 .89832 L
s
0 0 0 r
.29364 .905 m
.28711 .90208 L
.28102 .89832 L
s
0 0 1 r
.29364 .905 m
.28755 .90124 L
.28102 .89832 L
s
.29364 .905 m
.26862 .89125 L
s
0 0 0 r
.28102 .89832 m
.27458 .8952 L
.26862 .89125 L
s
0 0 1 r
.28102 .89832 m
.27506 .89437 L
.26862 .89125 L
s
.28102 .89832 m
.25644 .88379 L
s
0 0 0 r
.26862 .89125 m
.26228 .88792 L
.25644 .88379 L
s
0 0 1 r
.26862 .89125 m
.26278 .88711 L
.25644 .88379 L
s
.26862 .89125 m
.24451 .87595 L
s
0 0 0 r
.25644 .88379 m
.25021 .88026 L
.24451 .87595 L
s
0 0 1 r
.25644 .88379 m
.25074 .87947 L
.24451 .87595 L
s
.25644 .88379 m
.23282 .86773 L
s
0 0 0 r
.24451 .87595 m
.23839 .87223 L
.23282 .86773 L
s
0 0 1 r
.24451 .87595 m
.23894 .87145 L
.23282 .86773 L
s
.24451 .87595 m
.22141 .85916 L
s
0 0 0 r
.23282 .86773 m
.22683 .86383 L
.22141 .85916 L
s
0 0 1 r
.23282 .86773 m
.2274 .86307 L
.22141 .85916 L
s
.23282 .86773 m
.21026 .85023 L
s
0 0 0 r
.22141 .85916 m
.21554 .85507 L
.21026 .85023 L
s
0 0 1 r
.22141 .85916 m
.21613 .85433 L
.21026 .85023 L
s
.22141 .85916 m
.1994 .84096 L
s
0 0 0 r
.21026 .85023 m
.20452 .84596 L
.1994 .84096 L
s
0 0 1 r
.21026 .85023 m
.20514 .84523 L
.1994 .84096 L
s
.21026 .85023 m
.18884 .83135 L
s
0 0 0 r
.1994 .84096 m
.1938 .83651 L
.18884 .83135 L
s
0 0 1 r
.1994 .84096 m
.19444 .8358 L
.18884 .83135 L
s
.1994 .84096 m
.17859 .82141 L
s
0 0 0 r
.18884 .83135 m
.18338 .82672 L
.17859 .82141 L
s
0 0 1 r
.18884 .83135 m
.18405 .82604 L
.17859 .82141 L
s
.18884 .83135 m
.16865 .81116 L
s
0 0 0 r
.17859 .82141 m
.17328 .81662 L
.16865 .81116 L
s
0 0 1 r
.17859 .82141 m
.17396 .81595 L
.16865 .81116 L
s
.17859 .82141 m
.15904 .8006 L
s
0 0 0 r
.16865 .81116 m
.16349 .8062 L
.15904 .8006 L
s
0 0 1 r
.16865 .81116 m
.1642 .80556 L
.15904 .8006 L
s
.16865 .81116 m
.14977 .78974 L
s
0 0 0 r
.15904 .8006 m
.15404 .79548 L
.14977 .78974 L
s
0 0 1 r
.15904 .8006 m
.15477 .79486 L
.14977 .78974 L
s
.15904 .8006 m
.14084 .77859 L
s
0 0 0 r
.14977 .78974 m
.14493 .78446 L
.14084 .77859 L
s
0 0 1 r
.14977 .78974 m
.14567 .78387 L
.14084 .77859 L
s
.14977 .78974 m
.13227 .76718 L
s
0 0 0 r
.14084 .77859 m
.13617 .77317 L
.13227 .76718 L
s
0 0 1 r
.14084 .77859 m
.13693 .7726 L
.13227 .76718 L
s
.14084 .77859 m
.12405 .75549 L
s
0 0 0 r
.13227 .76718 m
.12777 .76161 L
.12405 .75549 L
s
0 0 1 r
.13227 .76718 m
.12855 .76106 L
.12405 .75549 L
s
.13227 .76718 m
.11621 .74356 L
s
0 0 0 r
.12405 .75549 m
.11974 .74979 L
.11621 .74356 L
s
0 0 1 r
.12405 .75549 m
.12053 .74926 L
.11621 .74356 L
s
.12405 .75549 m
.10875 .73138 L
s
.12405 .75549 m
.78974 .14977 L
s
0 0 0 r
.11621 .74356 m
.11208 .73772 L
.10875 .73138 L
s
0 0 1 r
.11621 .74356 m
.11289 .73722 L
.10875 .73138 L
s
.11621 .74356 m
.10168 .71898 L
s
.11621 .74356 m
.77859 .14084 L
s
0 0 0 r
.11621 .74356 m
.43318 .42421 L
.78974 .14977 L
s
0 0 1 r
.11621 .74356 m
.47277 .46911 L
.78974 .14977 L
s
.11621 .74356 m
.8006 .15904 L
s
0 0 0 r
.10875 .73138 m
.1048 .72542 L
.10168 .71898 L
s
0 0 1 r
.10875 .73138 m
.10563 .72494 L
.10168 .71898 L
s
.10875 .73138 m
.095 .70636 L
s
.10875 .73138 m
.78974 .14977 L
s
0 0 0 r
.10168 .71898 m
.09792 .71289 L
.095 .70636 L
s
0 0 1 r
.10168 .71898 m
.09876 .71245 L
.095 .70636 L
s
.10168 .71898 m
.08871 .69354 L
s
0 0 0 r
.095 .70636 m
.09143 .70016 L
.08871 .69354 L
s
0 0 1 r
.095 .70636 m
.09228 .69974 L
.08871 .69354 L
s
.095 .70636 m
.08284 .68052 L
s
0 0 0 r
.08871 .69354 m
.08534 .68722 L
.08284 .68052 L
s
0 0 1 r
.08871 .69354 m
.08621 .68683 L
.08284 .68052 L
s
.08871 .69354 m
.07737 .66733 L
s
0 0 0 r
.08284 .68052 m
.07967 .67411 L
.07737 .66733 L
s
0 0 1 r
.08284 .68052 m
.08055 .67374 L
.07737 .66733 L
s
.08284 .68052 m
.07233 .65397 L
s
0 0 0 r
.07737 .66733 m
.07441 .66082 L
.07233 .65397 L
s
0 0 1 r
.07737 .66733 m
.0753 .66048 L
.07233 .65397 L
s
.07737 .66733 m
.0677 .64046 L
s
0 0 0 r
.07233 .65397 m
.06956 .64737 L
.0677 .64046 L
s
0 0 1 r
.07233 .65397 m
.07046 .64706 L
.0677 .64046 L
s
.07233 .65397 m
.0635 .62681 L
s
0 0 0 r
.0677 .64046 m
.06515 .63378 L
.0635 .62681 L
s
0 0 1 r
.0677 .64046 m
.06606 .6335 L
.0635 .62681 L
s
.0677 .64046 m
.05973 .61304 L
s
0 0 0 r
.0635 .62681 m
.06116 .62005 L
.05973 .61304 L
s
0 0 1 r
.0635 .62681 m
.06208 .6198 L
.05973 .61304 L
s
.0635 .62681 m
.0564 .59916 L
s
0 0 0 r
.05973 .61304 m
.05761 .60621 L
.0564 .59916 L
s
0 0 1 r
.05973 .61304 m
.05853 .60599 L
.0564 .59916 L
s
.05973 .61304 m
.05351 .58517 L
s
0 0 0 r
.0564 .59916 m
.05449 .59226 L
.05351 .58517 L
s
0 0 1 r
.0564 .59916 m
.05542 .59207 L
.05351 .58517 L
s
.0564 .59916 m
.05105 .57111 L
s
0 0 0 r
.05351 .58517 m
.05181 .57822 L
.05105 .57111 L
s
0 0 1 r
.05351 .58517 m
.05275 .57806 L
.05105 .57111 L
s
.05351 .58517 m
.04904 .55697 L
s
0 0 0 r
.05105 .57111 m
.04957 .56411 L
.04904 .55697 L
s
0 0 1 r
.05105 .57111 m
.05052 .56397 L
.04904 .55697 L
s
.05105 .57111 m
.04747 .54278 L
s
0 0 0 r
.04904 .55697 m
.04778 .54993 L
.04747 .54278 L
s
0 0 1 r
.04904 .55697 m
.04873 .54982 L
.04747 .54278 L
s
.04904 .55697 m
.04635 .52854 L
s
0 0 0 r
.04747 .54278 m
.04644 .5357 L
.04635 .52854 L
s
0 0 1 r
.04747 .54278 m
.04739 .53562 L
.04635 .52854 L
s
.04747 .54278 m
.04568 .51428 L
s
0 0 0 r
.04635 .52854 m
.04554 .52143 L
.04568 .51428 L
s
0 0 1 r
.04635 .52854 m
.04649 .52139 L
.04568 .51428 L
s
.04635 .52854 m
.04545 .5 L
s
0 0 0 r
.04568 .51428 m
.04509 .50715 L
.04545 .5 L
s
0 0 1 r
.04568 .51428 m
.04604 .50713 L
.04545 .5 L
s
.04568 .51428 m
.04568 .48572 L
s
0 0 0 r
.04545 .5 m
.04509 .49285 L
.04568 .48572 L
s
0 0 1 r
.04545 .5 m
.04604 .49287 L
.04568 .48572 L
s
.04545 .5 m
.04635 .47146 L
s
0 0 0 r
.04568 .48572 m
.04554 .47857 L
.04635 .47146 L
s
0 0 1 r
.04568 .48572 m
.04649 .47861 L
.04635 .47146 L
s
.04568 .48572 m
.04747 .45722 L
s
0 0 0 r
.04635 .47146 m
.04644 .4643 L
.04747 .45722 L
s
0 0 1 r
.04635 .47146 m
.04739 .46438 L
.04747 .45722 L
s
.04635 .47146 m
.04904 .44303 L
s
0 0 0 r
.04747 .45722 m
.04778 .45007 L
.04904 .44303 L
s
0 0 1 r
.04747 .45722 m
.04873 .45018 L
.04904 .44303 L
s
.04747 .45722 m
.05105 .42889 L
s
0 0 0 r
.04904 .44303 m
.04957 .43589 L
.05105 .42889 L
s
0 0 1 r
.04904 .44303 m
.05052 .43603 L
.05105 .42889 L
s
.04904 .44303 m
.05351 .41483 L
s
0 0 0 r
.05105 .42889 m
.05181 .42178 L
.05351 .41483 L
s
0 0 1 r
.05105 .42889 m
.05275 .42194 L
.05351 .41483 L
s
.05105 .42889 m
.0564 .40084 L
s
.05105 .42889 m
.5 .04545 L
s
0 0 0 r
.05351 .41483 m
.05449 .40774 L
.0564 .40084 L
s
0 0 1 r
.05351 .41483 m
.05542 .40793 L
.0564 .40084 L
s
.05351 .41483 m
.05973 .38696 L
s
.05351 .41483 m
.48572 .04568 L
s
0 0 0 r
.05351 .41483 m
.26444 .21526 L
.5 .04545 L
s
0 0 1 r
.05351 .41483 m
.28907 .24502 L
.5 .04545 L
s
.05351 .41483 m
.51428 .04568 L
s
0 0 0 r
.0564 .40084 m
.05761 .39379 L
.05973 .38696 L
s
0 0 1 r
.0564 .40084 m
.05853 .39401 L
.05973 .38696 L
s
.0564 .40084 m
.0635 .37319 L
s
.0564 .40084 m
.5 .04545 L
s
0 0 0 r
.05973 .38696 m
.06116 .37995 L
.0635 .37319 L
s
0 0 1 r
.05973 .38696 m
.06208 .3802 L
.0635 .37319 L
s
.05973 .38696 m
.0677 .35954 L
s
0 0 0 r
.0635 .37319 m
.06515 .36622 L
.0677 .35954 L
s
0 0 1 r
.0635 .37319 m
.06606 .3665 L
.0677 .35954 L
s
.0635 .37319 m
.07233 .34603 L
s
0 0 0 r
.0677 .35954 m
.06956 .35263 L
.07233 .34603 L
s
0 0 1 r
.0677 .35954 m
.07046 .35294 L
.07233 .34603 L
s
.0677 .35954 m
.07737 .33267 L
s
0 0 0 r
.07233 .34603 m
.07441 .33918 L
.07737 .33267 L
s
0 0 1 r
.07233 .34603 m
.0753 .33952 L
.07737 .33267 L
s
.07233 .34603 m
.08284 .31948 L
s
0 0 0 r
.07737 .33267 m
.07967 .32589 L
.08284 .31948 L
s
0 0 1 r
.07737 .33267 m
.08055 .32626 L
.08284 .31948 L
s
.07737 .33267 m
.08871 .30646 L
s
0 0 0 r
.08284 .31948 m
.08534 .31278 L
.08871 .30646 L
s
0 0 1 r
.08284 .31948 m
.08621 .31317 L
.08871 .30646 L
s
.08284 .31948 m
.095 .29364 L
s
0 0 0 r
.08871 .30646 m
.09143 .29984 L
.095 .29364 L
s
0 0 1 r
.08871 .30646 m
.09228 .30026 L
.095 .29364 L
s
.08871 .30646 m
.10168 .28102 L
s
0 0 0 r
.095 .29364 m
.09792 .28711 L
.10168 .28102 L
s
0 0 1 r
.095 .29364 m
.09876 .28755 L
.10168 .28102 L
s
.095 .29364 m
.10875 .26862 L
s
0 0 0 r
.10168 .28102 m
.1048 .27458 L
.10875 .26862 L
s
0 0 1 r
.10168 .28102 m
.10563 .27506 L
.10875 .26862 L
s
.10168 .28102 m
.11621 .25644 L
s
0 0 0 r
.10875 .26862 m
.11208 .26228 L
.11621 .25644 L
s
0 0 1 r
.10875 .26862 m
.11289 .26278 L
.11621 .25644 L
s
.10875 .26862 m
.12405 .24451 L
s
0 0 0 r
.11621 .25644 m
.11974 .25021 L
.12405 .24451 L
s
0 0 1 r
.11621 .25644 m
.12053 .25074 L
.12405 .24451 L
s
.11621 .25644 m
.13227 .23282 L
s
0 0 0 r
.12405 .24451 m
.12777 .23839 L
.13227 .23282 L
s
0 0 1 r
.12405 .24451 m
.12855 .23894 L
.13227 .23282 L
s
.12405 .24451 m
.14084 .22141 L
s
0 0 0 r
.13227 .23282 m
.13617 .22683 L
.14084 .22141 L
s
0 0 1 r
.13227 .23282 m
.13693 .2274 L
.14084 .22141 L
s
.13227 .23282 m
.14977 .21026 L
s
0 0 0 r
.14084 .22141 m
.14493 .21554 L
.14977 .21026 L
s
0 0 1 r
.14084 .22141 m
.14567 .21613 L
.14977 .21026 L
s
.14084 .22141 m
.15904 .1994 L
s
0 0 0 r
.14977 .21026 m
.15404 .20452 L
.15904 .1994 L
s
0 0 1 r
.14977 .21026 m
.15477 .20514 L
.15904 .1994 L
s
.14977 .21026 m
.16865 .18884 L
s
0 0 0 r
.15904 .1994 m
.16349 .1938 L
.16865 .18884 L
s
0 0 1 r
.15904 .1994 m
.1642 .19444 L
.16865 .18884 L
s
.15904 .1994 m
.17859 .17859 L
s
0 0 0 r
.16865 .18884 m
.17328 .18338 L
.17859 .17859 L
s
0 0 1 r
.16865 .18884 m
.17396 .18405 L
.17859 .17859 L
s
.16865 .18884 m
.18884 .16865 L
s
0 0 0 r
.17859 .17859 m
.18338 .17328 L
.18884 .16865 L
s
0 0 1 r
.17859 .17859 m
.18405 .17396 L
.18884 .16865 L
s
.17859 .17859 m
.1994 .15904 L
s
0 0 0 r
.18884 .16865 m
.1938 .16349 L
.1994 .15904 L
s
0 0 1 r
.18884 .16865 m
.19444 .1642 L
.1994 .15904 L
s
.18884 .16865 m
.21026 .14977 L
s
0 0 0 r
.1994 .15904 m
.20452 .15404 L
.21026 .14977 L
s
0 0 1 r
.1994 .15904 m
.20514 .15477 L
.21026 .14977 L
s
.1994 .15904 m
.22141 .14084 L
s
0 0 0 r
.21026 .14977 m
.21554 .14493 L
.22141 .14084 L
s
0 0 1 r
.21026 .14977 m
.21613 .14567 L
.22141 .14084 L
s
.21026 .14977 m
.23282 .13227 L
s
0 0 0 r
.22141 .14084 m
.22683 .13617 L
.23282 .13227 L
s
0 0 1 r
.22141 .14084 m
.2274 .13693 L
.23282 .13227 L
s
.22141 .14084 m
.24451 .12405 L
s
0 0 0 r
.23282 .13227 m
.23839 .12777 L
.24451 .12405 L
s
0 0 1 r
.23282 .13227 m
.23894 .12855 L
.24451 .12405 L
s
.23282 .13227 m
.25644 .11621 L
s
0 0 0 r
.24451 .12405 m
.25021 .11974 L
.25644 .11621 L
s
0 0 1 r
.24451 .12405 m
.25074 .12053 L
.25644 .11621 L
s
.24451 .12405 m
.26862 .10875 L
s
0 0 0 r
.25644 .11621 m
.26228 .11208 L
.26862 .10875 L
s
0 0 1 r
.25644 .11621 m
.26278 .11289 L
.26862 .10875 L
s
.25644 .11621 m
.28102 .10168 L
s
0 0 0 r
.26862 .10875 m
.27458 .1048 L
.28102 .10168 L
s
0 0 1 r
.26862 .10875 m
.27506 .10563 L
.28102 .10168 L
s
.26862 .10875 m
.29364 .095 L
s
0 0 0 r
.28102 .10168 m
.28711 .09792 L
.29364 .095 L
s
0 0 1 r
.28102 .10168 m
.28755 .09876 L
.29364 .095 L
s
.28102 .10168 m
.30646 .08871 L
s
0 0 0 r
.29364 .095 m
.29984 .09143 L
.30646 .08871 L
s
0 0 1 r
.29364 .095 m
.30026 .09228 L
.30646 .08871 L
s
.29364 .095 m
.31948 .08284 L
s
0 0 0 r
.30646 .08871 m
.31278 .08534 L
.31948 .08284 L
s
0 0 1 r
.30646 .08871 m
.31317 .08621 L
.31948 .08284 L
s
.30646 .08871 m
.33267 .07737 L
s
0 0 0 r
.31948 .08284 m
.32589 .07967 L
.33267 .07737 L
s
0 0 1 r
.31948 .08284 m
.32626 .08055 L
.33267 .07737 L
s
.31948 .08284 m
.34603 .07233 L
s
0 0 0 r
.33267 .07737 m
.33918 .07441 L
.34603 .07233 L
s
0 0 1 r
.33267 .07737 m
.33952 .0753 L
.34603 .07233 L
s
.33267 .07737 m
.35954 .0677 L
s
0 0 0 r
.34603 .07233 m
.35263 .06956 L
.35954 .0677 L
s
0 0 1 r
.34603 .07233 m
.35294 .07046 L
.35954 .0677 L
s
.34603 .07233 m
.37319 .0635 L
s
0 0 0 r
.35954 .0677 m
.36622 .06515 L
.37319 .0635 L
s
0 0 1 r
.35954 .0677 m
.3665 .06606 L
.37319 .0635 L
s
.35954 .0677 m
.38696 .05973 L
s
0 0 0 r
.37319 .0635 m
.37995 .06116 L
.38696 .05973 L
s
0 0 1 r
.37319 .0635 m
.3802 .06208 L
.38696 .05973 L
s
.37319 .0635 m
.40084 .0564 L
s
0 0 0 r
.38696 .05973 m
.39379 .05761 L
.40084 .0564 L
s
0 0 1 r
.38696 .05973 m
.39401 .05853 L
.40084 .0564 L
s
.38696 .05973 m
.41483 .05351 L
s
0 0 0 r
.40084 .0564 m
.40774 .05449 L
.41483 .05351 L
s
0 0 1 r
.40084 .0564 m
.40793 .05542 L
.41483 .05351 L
s
.40084 .0564 m
.42889 .05105 L
s
0 0 0 r
.41483 .05351 m
.42178 .05181 L
.42889 .05105 L
s
0 0 1 r
.41483 .05351 m
.42194 .05275 L
.42889 .05105 L
s
.41483 .05351 m
.44303 .04904 L
s
0 0 0 r
.42889 .05105 m
.43589 .04957 L
.44303 .04904 L
s
0 0 1 r
.42889 .05105 m
.43603 .05052 L
.44303 .04904 L
s
.42889 .05105 m
.45722 .04747 L
s
0 0 0 r
.44303 .04904 m
.45007 .04778 L
.45722 .04747 L
s
0 0 1 r
.44303 .04904 m
.45018 .04873 L
.45722 .04747 L
s
.44303 .04904 m
.47146 .04635 L
s
0 0 0 r
.45722 .04747 m
.4643 .04644 L
.47146 .04635 L
s
0 0 1 r
.45722 .04747 m
.46438 .04739 L
.47146 .04635 L
s
.45722 .04747 m
.48572 .04568 L
s
0 0 0 r
.47146 .04635 m
.47857 .04554 L
.48572 .04568 L
s
0 0 1 r
.47146 .04635 m
.47861 .04649 L
.48572 .04568 L
s
.47146 .04635 m
.5 .04545 L
s
0 0 0 r
.48572 .04568 m
.49285 .04509 L
.5 .04545 L
s
0 0 1 r
.48572 .04568 m
.49287 .04604 L
.5 .04545 L
s
.48572 .04568 m
.51428 .04568 L
s
0 0 0 r
.5 .04545 m
.50713 .04604 L
.51428 .04568 L
s
0 0 1 r
.5 .04545 m
.50715 .04509 L
.51428 .04568 L
s
.5 .04545 m
.52854 .04635 L
s
0 0 0 r
.51428 .04568 m
.52139 .04649 L
.52854 .04635 L
s
0 0 1 r
.51428 .04568 m
.52143 .04554 L
.52854 .04635 L
s
.51428 .04568 m
.54278 .04747 L
s
0 0 0 r
.52854 .04635 m
.53562 .04739 L
.54278 .04747 L
s
0 0 1 r
.52854 .04635 m
.5357 .04644 L
.54278 .04747 L
s
.52854 .04635 m
.55697 .04904 L
s
0 0 0 r
.54278 .04747 m
.54982 .04873 L
.55697 .04904 L
s
0 0 1 r
.54278 .04747 m
.54993 .04778 L
.55697 .04904 L
s
.54278 .04747 m
.57111 .05105 L
s
0 0 0 r
.55697 .04904 m
.56397 .05052 L
.57111 .05105 L
s
0 0 1 r
.55697 .04904 m
.56411 .04957 L
.57111 .05105 L
s
.55697 .04904 m
.58517 .05351 L
s
0 0 0 r
.57111 .05105 m
.57806 .05275 L
.58517 .05351 L
s
0 0 1 r
.57111 .05105 m
.57822 .05181 L
.58517 .05351 L
s
.57111 .05105 m
.59916 .0564 L
s
0 0 0 r
.58517 .05351 m
.59207 .05542 L
.59916 .0564 L
s
0 0 1 r
.58517 .05351 m
.59226 .05449 L
.59916 .0564 L
s
.58517 .05351 m
.61304 .05973 L
s
0 0 0 r
.59916 .0564 m
.60599 .05853 L
.61304 .05973 L
s
0 0 1 r
.59916 .0564 m
.60621 .05761 L
.61304 .05973 L
s
.59916 .0564 m
.62681 .0635 L
s
0 0 0 r
.61304 .05973 m
.6198 .06208 L
.62681 .0635 L
s
0 0 1 r
.61304 .05973 m
.62005 .06116 L
.62681 .0635 L
s
.61304 .05973 m
.64046 .0677 L
s
0 0 0 r
.62681 .0635 m
.6335 .06606 L
.64046 .0677 L
s
0 0 1 r
.62681 .0635 m
.63378 .06515 L
.64046 .0677 L
s
.62681 .0635 m
.65397 .07233 L
s
0 0 0 r
.64046 .0677 m
.64706 .07046 L
.65397 .07233 L
s
0 0 1 r
.64046 .0677 m
.64737 .06956 L
.65397 .07233 L
s
.64046 .0677 m
.66733 .07737 L
s
0 0 0 r
.65397 .07233 m
.66048 .0753 L
.66733 .07737 L
s
0 0 1 r
.65397 .07233 m
.66082 .07441 L
.66733 .07737 L
s
.65397 .07233 m
.68052 .08284 L
s
0 0 0 r
.66733 .07737 m
.67374 .08055 L
.68052 .08284 L
s
0 0 1 r
.66733 .07737 m
.67411 .07967 L
.68052 .08284 L
s
.66733 .07737 m
.69354 .08871 L
s
0 0 0 r
.68052 .08284 m
.68683 .08621 L
.69354 .08871 L
s
0 0 1 r
.68052 .08284 m
.68722 .08534 L
.69354 .08871 L
s
.68052 .08284 m
.70636 .095 L
s
0 0 0 r
.69354 .08871 m
.69974 .09228 L
.70636 .095 L
s
0 0 1 r
.69354 .08871 m
.70016 .09143 L
.70636 .095 L
s
.69354 .08871 m
.71898 .10168 L
s
0 0 0 r
.70636 .095 m
.71245 .09876 L
.71898 .10168 L
s
0 0 1 r
.70636 .095 m
.71289 .09792 L
.71898 .10168 L
s
.70636 .095 m
.73138 .10875 L
s
0 0 0 r
.71898 .10168 m
.72494 .10563 L
.73138 .10875 L
s
0 0 1 r
.71898 .10168 m
.72542 .1048 L
.73138 .10875 L
s
.71898 .10168 m
.74356 .11621 L
s
0 0 0 r
.73138 .10875 m
.73722 .11289 L
.74356 .11621 L
s
0 0 1 r
.73138 .10875 m
.73772 .11208 L
.74356 .11621 L
s
.73138 .10875 m
.75549 .12405 L
s
0 0 0 r
.74356 .11621 m
.74926 .12053 L
.75549 .12405 L
s
0 0 1 r
.74356 .11621 m
.74979 .11974 L
.75549 .12405 L
s
.74356 .11621 m
.76718 .13227 L
s
0 0 0 r
.75549 .12405 m
.76106 .12855 L
.76718 .13227 L
s
0 0 1 r
.75549 .12405 m
.76161 .12777 L
.76718 .13227 L
s
.75549 .12405 m
.77859 .14084 L
s
0 0 0 r
.76718 .13227 m
.7726 .13693 L
.77859 .14084 L
s
0 0 1 r
.76718 .13227 m
.77317 .13617 L
.77859 .14084 L
s
.76718 .13227 m
.78974 .14977 L
s
0 0 0 r
.77859 .14084 m
.78387 .14567 L
.78974 .14977 L
s
0 0 1 r
.77859 .14084 m
.78446 .14493 L
.78974 .14977 L
s
.77859 .14084 m
.8006 .15904 L
s
0 0 0 r
.78974 .14977 m
.79486 .15477 L
.8006 .15904 L
s
0 0 1 r
.78974 .14977 m
.79548 .15404 L
.8006 .15904 L
s
.78974 .14977 m
.81116 .16865 L
s
0 0 0 r
.8006 .15904 m
.80556 .1642 L
.81116 .16865 L
s
0 0 1 r
.8006 .15904 m
.8062 .16349 L
.81116 .16865 L
s
.8006 .15904 m
.82141 .17859 L
s
0 0 0 r
.81116 .16865 m
.81595 .17396 L
.82141 .17859 L
s
0 0 1 r
.81116 .16865 m
.81662 .17328 L
.82141 .17859 L
s
.81116 .16865 m
.83135 .18884 L
s
0 0 0 r
.82141 .17859 m
.82604 .18405 L
.83135 .18884 L
s
0 0 1 r
.82141 .17859 m
.82672 .18338 L
.83135 .18884 L
s
.82141 .17859 m
.84096 .1994 L
s
0 0 0 r
.83135 .18884 m
.8358 .19444 L
.84096 .1994 L
s
0 0 1 r
.83135 .18884 m
.83651 .1938 L
.84096 .1994 L
s
.83135 .18884 m
.85023 .21026 L
s
0 0 0 r
.84096 .1994 m
.84523 .20514 L
.85023 .21026 L
s
0 0 1 r
.84096 .1994 m
.84596 .20452 L
.85023 .21026 L
s
.84096 .1994 m
.85916 .22141 L
s
0 0 0 r
.85023 .21026 m
.85433 .21613 L
.85916 .22141 L
s
0 0 1 r
.85023 .21026 m
.85507 .21554 L
.85916 .22141 L
s
.85023 .21026 m
.86773 .23282 L
s
0 0 0 r
.85916 .22141 m
.86307 .2274 L
.86773 .23282 L
s
0 0 1 r
.85916 .22141 m
.86383 .22683 L
.86773 .23282 L
s
.85916 .22141 m
.87595 .24451 L
s
0 0 0 r
.86773 .23282 m
.87145 .23894 L
.87595 .24451 L
s
0 0 1 r
.86773 .23282 m
.87223 .23839 L
.87595 .24451 L
s
.86773 .23282 m
.88379 .25644 L
s
0 0 0 r
.87595 .24451 m
.87947 .25074 L
.88379 .25644 L
s
0 0 1 r
.87595 .24451 m
.88026 .25021 L
.88379 .25644 L
s
.87595 .24451 m
.89125 .26862 L
s
0 0 0 r
.88379 .25644 m
.88711 .26278 L
.89125 .26862 L
s
0 0 1 r
.88379 .25644 m
.88792 .26228 L
.89125 .26862 L
s
.88379 .25644 m
.89832 .28102 L
s
0 0 0 r
.89125 .26862 m
.89437 .27506 L
.89832 .28102 L
s
0 0 1 r
.89125 .26862 m
.8952 .27458 L
.89832 .28102 L
s
.89125 .26862 m
.905 .29364 L
s
0 0 0 r
.89832 .28102 m
.90124 .28755 L
.905 .29364 L
s
0 0 1 r
.89832 .28102 m
.90208 .28711 L
.905 .29364 L
s
.89832 .28102 m
.91129 .30646 L
s
0 0 0 r
.905 .29364 m
.90772 .30026 L
.91129 .30646 L
s
0 0 1 r
.905 .29364 m
.90857 .29984 L
.91129 .30646 L
s
.905 .29364 m
.91716 .31948 L
s
0 0 0 r
.91129 .30646 m
.91379 .31317 L
.91716 .31948 L
s
0 0 1 r
.91129 .30646 m
.91466 .31278 L
.91716 .31948 L
s
.91129 .30646 m
.92263 .33267 L
s
0 0 0 r
.91716 .31948 m
.91945 .32626 L
.92263 .33267 L
s
0 0 1 r
.91716 .31948 m
.92033 .32589 L
.92263 .33267 L
s
.91716 .31948 m
.92767 .34603 L
s
0 0 0 r
.92263 .33267 m
.9247 .33952 L
.92767 .34603 L
s
0 0 1 r
.92263 .33267 m
.92559 .33918 L
.92767 .34603 L
s
.92263 .33267 m
.9323 .35954 L
s
0 0 0 r
.92767 .34603 m
.92954 .35294 L
.9323 .35954 L
s
0 0 1 r
.92767 .34603 m
.93044 .35263 L
.9323 .35954 L
s
.92767 .34603 m
.9365 .37319 L
s
0 0 0 r
.9323 .35954 m
.93394 .3665 L
.9365 .37319 L
s
0 0 1 r
.9323 .35954 m
.93485 .36622 L
.9365 .37319 L
s
.9323 .35954 m
.94027 .38696 L
s
0 0 0 r
.9365 .37319 m
.93792 .3802 L
.94027 .38696 L
s
0 0 1 r
.9365 .37319 m
.93884 .37995 L
.94027 .38696 L
s
.9365 .37319 m
.9436 .40084 L
s
0 0 0 r
.94027 .38696 m
.94147 .39401 L
.9436 .40084 L
s
0 0 1 r
.94027 .38696 m
.94239 .39379 L
.9436 .40084 L
s
.94027 .38696 m
.94649 .41483 L
s
0 0 0 r
.9436 .40084 m
.94458 .40793 L
.94649 .41483 L
s
0 0 1 r
.9436 .40084 m
.94551 .40774 L
.94649 .41483 L
s
.9436 .40084 m
.94895 .42889 L
s
0 0 0 r
.94649 .41483 m
.94725 .42194 L
.94895 .42889 L
s
0 0 1 r
.94649 .41483 m
.94819 .42178 L
.94895 .42889 L
s
.94649 .41483 m
.95096 .44303 L
s
0 0 0 r
.94895 .42889 m
.94948 .43603 L
.95096 .44303 L
s
0 0 1 r
.94895 .42889 m
.95043 .43589 L
.95096 .44303 L
s
.94895 .42889 m
.95253 .45722 L
s
0 0 0 r
.95096 .44303 m
.95127 .45018 L
.95253 .45722 L
s
0 0 1 r
.95096 .44303 m
.95222 .45007 L
.95253 .45722 L
s
.95096 .44303 m
.95365 .47146 L
s
0 0 0 r
.95253 .45722 m
.95261 .46438 L
.95365 .47146 L
s
0 0 1 r
.95253 .45722 m
.95356 .4643 L
.95365 .47146 L
s
.95253 .45722 m
.95432 .48572 L
s
0 0 0 r
.95365 .47146 m
.95351 .47861 L
.95432 .48572 L
s
0 0 1 r
.95365 .47146 m
.95446 .47857 L
.95432 .48572 L
s
.95365 .47146 m
.95455 .5 L
s
0 0 0 r
.95432 .48572 m
.95396 .49287 L
.95455 .5 L
s
0 0 1 r
.95432 .48572 m
.95491 .49285 L
.95455 .5 L
s
0 0 0 r
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.663 .663 .663 r
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
0 0 0 r
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.663 .663 .663 r
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
0 0 0 r
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[18]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00P_ooool3o`000005oooooonYZJWoZJVYojVYZOoo
ool00onYZJT01Ol0003oZJVYojVYZOnYZJWo003o00?oZJVY00GoooooojVYZOnYZJWoZJVYooooo`03
o`0008?ooooo001iooooo`?o000000Coooooo`000?l0003o00000_l00?l3o`000005o`00oonYZJWo
ZJVYojVYZOl00?l00onYZJT01Ol00?ooZJVYojVYZOnYZJWo003o00?oZJVY00Go003oojVYZOnYZJWo
ZJVYo`00o`03o`000005o`00ool0003o0000o`000?ooool00ol0001kooooo`00MOooool7o`000004
o`00ool0003o0000o`0000;o003o0ol000001OooooooZJVYojVYZOnYZJWo003o00?oZJVY00Go003o
ojVYZOnYZJWoZJVYo`00o`03ojVYZ@05oooooonYZJWoZJVYojVYZOooool00ol000001Ol00?oo0000
o`000?l0003o003o00?o000000Co003oo`000?l0003o0000Moooool0077ooooo0ol000001Ol00?oo
0000o`000?l0003o003o00?o000000Coooooo`000?l0003o00002?ooool01Ol00?ooooooooooool0
003o003o00?ooooo00?o003oooooooooool02oooool3o`000003ooooool0003o000000Go000000Co
003oo`000?l0003o0000Loooool006gooooo0ol000001Ol00?oo0000o`000?l0003o003o00?o0000
3oooool01Ol00?ooooooooooool0003o003o00?ooooo00?o003oooooooooool05?ooool3o`000005
ooooool0003o0000o`000?l00?l00ol0001_ooooo`00JOooool3o`000005o`00ool0003o0000o`00
0?l00?l00ol0000Booooo`03o`00oooooooooooo00;o003o0_ooool2o`00oa_ooooo0ol000001Ooo
oooo0000o`000?l0003o003o00?o0000Joooool006Kooooo1_l000001?l00?oo0000o`000?l0000E
ooooo`03o`00oooooooo000000;o003o0_ooool00ol00?ooooooooooo`0Oooooo`?o000000Gooooo
o`000?l0003o0000o`00o`03o`0006Oooooo001Rooooo`?o000000?o003oo`000?l000001?l0000G
ooooo`;o003o00Coooooo`000?l00?oo003o0_ooool00ol00?ooooooooooo`0Tooooo`?o000000Go
ooooo`000?l0003o0000o`00o`03o`0006?ooooo001Rooooo`?o000000Co003oo`000?l0003o0000
6Oooool01?l00?ooooooo`000?l00002o`00o`;ooooo00?o003oooooooooool09_ooool00ol00?oo
ooooooooo`03o`000004o`00ool0003o0000o`0006?ooooo001Nooooo`?o000000Co003oo`000?l0
003o00007?ooool00ol00?ooooooo`000003o`00o`03ooooool00?oo003o02Wooooo00Go003ooooo
ooooooooooooo`00o`02ooooo`?o000000Co003oo`000?l0003o0000Goooool005[ooooo0ol00000
1?l00?oo0000o`000?l0000Oooooo`03o`00oooooooo000000?o003o00?oooooo`00ooooool0:_oo
ool01Ol00?oooooooooooooooooo003o00?ooooo00?o003oooooooooool00ol000001?l00?oo0000
o`000?l0001Kooooo`00F_ooool3o`000004o`00ool0003o0000o`0001kooooo00?o003oo`000?l0
00000ol00?l00ooooooo003oooooo`0[ooooo`05o`00ooooooooooooooooool00?l00oooool00ol0
0?ooooooooooo`03o`000004o`00ool0003o0000o`0005_ooooo001Fooooo`?o000000Co003oo`00
0?l0003o00008Oooool00ol00?oo0000ooooo`03o`00o`03ooooool00?oooooo02cooooo00Go003o
ooooooooooooooooo`00o`03ooooo`03o`00oooooooooooo00Cooooo1_l0001Hooooo`00E_ooool3
o`0002?ooooo00?o003oo`000?l000001_l00?l^ooooo`05o`00ooooooooooooooooool00?l00ooo
ool00ol00?ooooooooooo`07ooooo`?o000000?o003oooooooooool0EOooool005?ooooo1_l0000R
ooooo`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l0<?ooool01Ol00?ooooooooooool0
003o003o00?ooooo00?o003oooooooooool01oooool3o`000004o`00ool0003o0000o`0005Cooooo
001Cooooo`?o00009?ooool2o`000003ooooool00?oo003o00?o003o<Oooool01Ol00?oooooooooo
ool0003o003o00?ooooo00?o003oooooooooool02oooool3o`0005Cooooo001?ooooo`?o000000Co
003oo`000?l0003o00008oooool00ol0003oooooooooo`03o`00o`03ooooool00?oooooo037ooooo
00Go003ooooooooooooo0000o`00o`03ooooo`03o`00oooooooooooo00_ooooo0ol000001?l00?oo
0000o`000?l0001@ooooo`00Coooool3o`0002Kooooo00?o0000ooooool00?l01Ol00?lcooooo`04
o`00ooooooooooooo`0000;o003o0_ooool00ol00?ooooooooooo`0?ooooo`?o0000D?ooool004_o
oooo0ol000001?l00?oo0000o`000?l0000Tooooo`;o000000?oooooo`00ool00?l00ol00?leoooo
o`04o`00ooooooooooooo`0000;o003o0_ooool00ol00?ooooooooooo`0?ooooo`Ko0000COooool0
04_ooooo0ol0000Wooooo`03o`000?l00?oooooo00;o003o00?oooooo`00ool00?l0=_ooool01?l0
0?ooooooooooool00002o`00o`;ooooo00?o003oooooooooool04_ooool3o`0004gooooo0018oooo
o`Ko00009Oooool2o`0000;ooooo0_l00?l00ooooooo003oo`00o`0gooooo`04o`00oooooooooooo
o`0000;o003o0_ooool00ol00?ooooooooooo`0Booooo`?o000000Co003oo`000?l0003o0000BOoo
ool004Sooooo0ol0000Wooooo`03o`000?l00?oooooo00?o003o00?oooooo`00ool00?l0>?ooool0
1?l00?ooooooooooool00002o`00o`;ooooo00?o003oooooooooool05_ooool3o`0004Wooooo0014
ooooo`?o000000Co003oo`000?l0003o00009_ooool02?l0003o003oooooool00?ooooooo`00oooo
oooo003o>_ooool01?l00?ooooooooooool00002o`00o`;ooooo00?o003oooooooooool05_ooool6
o`0004Kooooo0014ooooo`?o0000:?ooool2o`000007o`00oooooooo003oooooool00?ooooooo`00
o`0kooooo`04o`00ooooooooooooo`0000;o003o0_ooool00ol00?ooooooooooo`0Iooooo`?o0000
A_ooool004?ooooo00Co003oo`000?l0003o00009oooool01?l0003oooooo`00ooooool3o`00o`03
ooooool00?oooooo03_ooooo00?o003oooooooooool00ol00?l00ooooooo003oooooo`0Kooooo`?o
00000_l00?m4ooooo`00@Oooool3o`0002Sooooo0_l00005o`00o`03ooooool00?oo003o03gooooo
00?o003oooooooooool00ol00?l00ooooooo003oooooo`0Nooooo`?o000000?o003oooooooooool0
@?ooool0043ooooo00Co003oo`000?l0003o00009oooool02?l0003oooooo`00oooooooo003ooooo
ool00?oooooo0_l00?lnooooo`03o`00oooooooooooo00?o003o00?oooooo`00ooooool07_ooool3
o`0000;o003o@Oooool003kooooo1_l0000Uooooo`;o000000Ooooooo`00oooooooo003oooooool0
0?oooooo00;o003o?oooool00ol00?ooooooo`000003o`00o`03ooooool00?oooooo01kooooo0ol0
00001?l00?oo0000o`000?l0000oooooo`00?_ooool3o`0002Oooooo00Wo0000oooooooooooo003o
ooooool00?ooooooo`00ooooool00_l00?m0ooooo`03o`00oooooooo000000?o003o00?oooooo`00
ooooool08_ooool3o`0003oooooo000mooooo`04o`00ool0003o0000o`0002Kooooo00Wo0000oooo
oooooooo003oooooool00?ooooooo`00ooooool00_l00?m1ooooo`03o`00oooooooo000000?o003o
00?oooooo`00ooooool08Oooool01Ol00?oo0000o`000?l0003o003o03kooooo000kooooo`?o0000
9oooool2o`000003ooooool00?oo003o00Ko003o@oooool00ol00?ooooooo`000003o`00o`03oooo
ool00?oooooo023ooooo00?o003oooooooooool00_ooool3o`0003cooooo000kooooo`?o00009_oo
ool02Ol0003oooooooooool00?ooooooo`00oooooooo003oooooo`02o`00odCooooo00?o003ooooo
ool000000ol00?l00ooooooo003oooooo`0Oooooo`03o`00oooooooooooo00;ooooo00Co003oo`00
0?l0003o0000??ooool003Sooooo1_l0000Tooooo`;o00000_ooool01_l00?ooooooo`00oooooooo
003oooooo`;o003oAOooool00ol00?ooooooo`000003o`00o`03ooooool00?oooooo01kooooo00?o
003oooooooooool00_ooool00ol00?ooooooo`000005o`0003Wooooo000hooooo`?o00009_ooool0
2_l0003ooooooooooooooooo003oooooool00?ooooooo`00ooooool2o`00odKooooo00?o003ooooo
ool000000ol00?l00ooooooo003oooooo`0Mooooo`08o`00ooooooooooooooooool0003o003ooooo
ool00?l3ooooo`?o0000>Oooool003Oooooo1?l0000Uooooo`04o`000?ooooooooooooooo`?o003o
00?oooooo`00ool00?l00_l00?m7ooooo`03o`00oooooooo000000?o003o00?oooooo`00ooooool0
7?ooool02?l00?oooooooooooooooooo0000o`00oooooooo003o0oooool01Ol00?oo0000o`000?l0
003o003o03Sooooo000eooooo`?o00009_ooool2o`0000?ooooo0_l00?l2ooooo`;o003o00?ooooo
o`00ooooool0Aoooool00ol00?ooooooo`000002o`00o`03ooooool00?oo003o01cooooo00So003o
ooooooooooooooooo`000?l00?ooooooo`00o`?ooooo00?o003oooooooooool00_ooool3o`0003Ko
oooo000eooooo`?o00009Oooool01?l0003oooooooooooooool3o`00o`06ooooool00?oo003oo`00
oooooooo003oBOooool00ol00?ooooooo`000002o`00o`03ooooool00?oo003o01_ooooo00Co003o
oooooooooooooooo0_l00?l00ooooooo003oooooo`02ooooo`03o`00oooooooooooo00?ooooo0ol0
000fooooo`00=?ooool01?l00?oo0000o`000?l0000Sooooo`;o00000oooool02Ol00?ooooooo`00
oooooooo003oooooool00?ooooooo`00o`1:ooooo`03o`00oooooooo000000;o003o00?oooooo`00
ool00?l06_ooool01?l00?ooooooooooool00002o`00o`03ooooool00?oooooo00;ooooo00?o003o
ooooooooool01?ooool3o`000003o`00oooooooooooo03?ooooo000booooo`?o00009Oooool00ol0
003oooooooooo`02ooooo`?o003o00Koooooo`00ool00?oo003oooooool00?m;ooooo`03o`00oooo
oooo000000;o003o00?oooooo`00ool00?l06?ooool2o`00o`;ooooo00Go0000o`00ool00?oooooo
o`00o`03ooooo`03o`00oooooooooooo00Sooooo0ol0000cooooo`00<_ooool3o`0002?ooooo0_l0
0004ooooo`;o003o0_ooool2o`00o`;ooooo00?o003oooooooooool0B_ooool00ol00?oo0000oooo
o`02o`00o`03ooooool00?oo003o01Oooooo00Go003oooooooooooooooooo`000002o`00o`04oooo
ool00?ooooooooooo`;o003o2oooool3o`0003?ooooo000aooooo`04o`00ool0003o0000o`0002;o
oooo00?o0000ooooooooool00oooool2o`00o`;ooooo0_l00?l2ooooo`03o`00oooooooooooo04_o
oooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0Fooooo`03o`00oooooooooooo00;o
00000_l00?l01Ooooooo003ooooooooooooo003o00gooooo0ol000000ol00?ooooooooooo`0`oooo
o`00;oooool3o`0002Cooooo00?o0000ooooooooool00oooool2o`00o`;ooooo0_l00?l2ooooo`03
o`00oooooooooooo04cooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0Eooooo`06
o`00ooooooooooooo`000?oooooo003o0_ooool01?l00?ooooooooooool00?lAooooo`?o0000<?oo
ool002oooooo0ol0000Rooooo`;o00001?ooool3o`00o`04ooooool00?oo003oo`00o`;ooooo00?o
003oooooooooool0COooool00ol00?oo0000ooooo`02o`00o`03ooooool00?oo003o01Cooooo00Co
003ooooooooooooo00000_l00?l2ooooo`04o`00ooooooooooooo`00oa;ooooo0ol0000`ooooo`00
;_ooool4o`00027ooooo00?o0000ooooooooool00oooool2o`00o`;ooooo0_l00?l3ooooo`03o`00
oooooooooooo04kooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0Cooooo`04o`00
ooooooooooooo`0000;o003o0_ooool01?l00?ooooooooooool00?lCooooo`?o000000?o003ooooo
ooooool0;Oooool002cooooo0ol0000Rooooo`;o00001Oooool2o`00o`;ooooo0_l00?l2ooooo`;o
003oDOooool00ol00?oo0000ooooo`02o`00o`03ooooool00?oo003o01;ooooo00Co003oooooool0
003o00000_l00?l2ooooo`04o`00ooooooooooooo`00oaOooooo0ol0000]ooooo`00;?ooool3o`00
027ooooo00?o0000ooooooooool01?ooool2o`00o`;ooooo0_l00?l2ooooo`03o`00oooooooooooo
057ooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0Aooooo`04o`00oooooooo0000
ooooo`;o003o0_ooool01?l00?ooooooo`00ool00?lHooooo`?o0000;Oooool002cooooo0ol0000P
ooooo`03o`000?oooooooooo00Cooooo0_l00?l2ooooo`;o003o0_ooool00ol00?ooooooooooo`1B
ooooo`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l04?ooool01?l00?ooooooo`000?oo
ool2o`00o`;ooooo00?o003oooooool00?l06_ooool3o`0002gooooo000Yooooo`?o000000?o003o
ooooooooool07_ooool2o`0000Kooooo00?o003oooooooooool00_l00?l3ooooo`03o`00oooooooo
oooo05?ooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0?ooooo`04o`00ool0003o
0000ooooo`;o003o0_ooool00ol00?ooooooo`00o`0Mooooo`?o0000:oooool002Wooooo0ol0000P
ooooo`03o`000?oooooooooo00Cooooo0_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1D
ooooo`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l03Oooool2o`00o`03o`000?oooooo
003o00;o003o0_ooool00ol00?ooooooo`00o`0Nooooo`?o0000:oooool002Wooooo0ol0000Noooo
o`;o00001_ooool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo05Gooooo00?o003oo`00
0?ooool00_l00?l00ooooooo003oo`00o`0<ooooo`07o`00oooooooo0000ooooool00?ooooooo`00
o`02ooooo`03o`00oooooooo003o01oooooo0ol000000ol00?ooooooooooo`0Xooooo`009oooool3
o`0001oooooo00?o0000ooooooooool01Oooool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooo
oooo05Kooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0;ooooo`04o`00oooooooo
0000ooooo`?o003o0_ooool00ol00?ooooooo`00o`0Sooooo`?o0000:?ooool002Oooooo0ol0000N
ooooo`03o`000?oooooooooo00Gooooo00Go003oooooooooooooooooo`00o`04ooooo`03o`00oooo
oooooooo05Oooooo00?o0000ooooooooool00_l00?l00ooooooo003oo`00o`0:ooooo`04o`00ool0
003o0000ooooo`;o003o0oooool3o`00obCooooo0ol0000Xooooo`009oooool3o`0001gooooo00?o
0000ooooooooool01Oooool01Ol00?oooooooooooooooooo003o00Cooooo00?o003oooooooooool0
F?ooool00ol0003oooooooooo`02o`00o`03ooooool00?oo003o00Wooooo00Co003oo`000?oooooo
oooo0_l00?l3ooooo`;o003o9_ooool3o`0002Sooooo000Vooooo`;o003o7_ooool00ol0003ooooo
ooooo`05ooooo`03o`00oooooooooooo00;o003o1?ooool00ol00?ooooooooooo`1Iooooo`03o`00
0?oooooooooo00;o003o0_ooool00ol00?ooooooooooo`06ooooo`04o`00ool0003oooooooooo`;o
003o0oooool2o`00obWooooo0_l00?lWooooo`009?ooool3o`0001kooooo00?o0000ooooooooool0
1?ooool2o`00o`;ooooo0_l00?l4ooooo`03o`00oooooooooooo05[ooooo00?o0000ooooooooool0
0_l00?l2ooooo`03o`00oooooooooooo00Gooooo0_l000001?oooooo003oo`00ool00?l3ooooo`;o
003o:_ooool3o`0002Kooooo000Tooooo`?o00007Oooool00ol0003oooooooooo`04ooooo`05o`00
ooooooooooooooooool00?l01Oooool00ol00?ooooooooooo`1Kooooo`06o`000?oooooo003ooooo
ool00?oooooo0_l00?l6ooooo`06o`000?ooooooooooo`00oooooooo003o0oooool2o`00ob_ooooo
0ol0000Vooooo`009?ooool3o`0001cooooo00?o0000ooooooooool01?ooool01Ol00?oooooooooo
oooooooo003o00Gooooo00?o003oooooooooool0G?ooool01_l0003oooooo`00oooooooo003ooooo
o`;o003o1Oooool01_l0003oooooooooool00?ooooooo`00o`?ooooo0_l00?l/ooooo`?o000000?o
003oooooooooool08oooool002;ooooo0ol0000Mooooo`03o`000?oooooooooo00Cooooo00Go003o
ooooooooooooooooo`00o`05ooooo`03o`00oooooooooooo05gooooo00Ko0000ooooool00?oooooo
o`00ooooool2o`00o`?ooooo0_l00002ooooo`03o`00oooooooo003o00?ooooo0_l00?l`ooooo`?o
00008oooool002;ooooo0ol0000Looooo`03o`000?oooooooooo00?ooooo0_l00?l2ooooo`;o003o
1?ooool2o`00of3ooooo00Ko0000ooooool00?ooooooo`00ooooool2o`00o`;ooooo00Oo0000o`00
ooooooooooooo`00oooooooo003o00?ooooo00?o003oooooooooool0<?ooool3o`0002?ooooo000R
ooooo`?o00006oooool00ol0003oooooooooo`03ooooo`;o003o0_ooool2o`00o`Cooooo00?o003o
ooooooooool0H?ooool01_l0003oooooo`00oooooooo003oooooo`;o003o00?oooooo`000?l00?l0
0_ooool00ol00?ooooooo`00o`03ooooo`03o`00oooooooooooo037ooooo0ol0000Sooooo`008?oo
ool3o`000003ooooool00?oooooo01Wooooo00?o0000ooooooooool00oooool2o`00o`;ooooo00?o
003oooooooooool00oooool00ol00?ooooooooooo`1Qooooo`06o`000?oooooo003oooooool00?oo
oooo0_l00?l00ol0003oooooooooo`02o`00o`03ooooool00?oooooo00;ooooo00?o003ooooooooo
ool0=?ooool3o`00027ooooo000Pooooo`?o00000_ooool00ol00?ooooooooooo`0Fooooo`03o`00
0?oooooooooo00?ooooo00Go003oooooooooooooooooo`00o`04ooooo`;o003oI?ooool01_l0003o
ooooo`00oooooooo003oooooo`;o00000_ooool01?l00?ooooooo`00ool00?l3ooooo`03o`00oooo
oooooooo03Gooooo0ol0000Qooooo`008?ooool3o`0000;ooooo00?o003oooooooooool05Oooool0
0ol0003oooooooooo`02ooooo`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool0I?ooool0
1_l0003oooooo`00oooooooo003oo`0000;o003o00Coooooo`00oooooooo003o1?ooool00ol00?oo
ooooooooo`0fooooo`?o00008Oooool0023ooooo00?o003oooooool00?l00oooool00ol00?oooooo
ooooo`0Cooooo`03o`000?oooooooooo00;ooooo0_l00?l2ooooo`;o003o0oooool2o`00ofOooooo
00Go0000ooooool00?ooooooo`000004o`00o`03ooooool00?oooooo00;ooooo0_l00?lkooooo`03
o`00oooooooooooo01oooooo000Nooooo`?o000000?oooooo`000?l00?l00_ooool00ol00?oooooo
ooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00?l2ooooo`;o003o0oooool00ol00?oooooo
ooooo`1Vooooo`03o`000?l00?oooooo00;o000000Ko003oooooool00?oo003oooooool00?l3oooo
o`;o003o??ooool3o`0001oooooo000Nooooo`?o00000_ooool2o`00o`;ooooo00?o003ooooooooo
ool04?ooool00ol0003oooooooooo`02ooooo`;o003o0_ooool2o`00o`?ooooo00?o003ooooooooo
ool0Ioooool00ol0003o003oo`000003o`00o`04ooooool00?oo003oo`00o`?ooooo0_l00?lmoooo
o`?o00007oooool001kooooo0ol00003ooooo`05o`00ooooooooooooooooool00?l04?ooool01?l0
003oooooooooooooool3o`00o`03ooooool00?oo003o00?ooooo0_l00?mZooooo`;o000000?ooooo
o`00ooooool01?l00?l3ooooo`;o003o?_ooool3o`0001oooooo000Kooooo`?o000000?o003ooooo
ool00?l00oooool2o`00o`;ooooo00?o003oooooooooool03Oooool01?l0003oooooooooooooool2
o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo06[ooooo00?o0000o`00ool00?l00ol00?l0
0ooooooo003oo`00o`02ooooo`;o003o@Oooool3o`0001gooooo000Kooooo`?o000000Co003ooooo
oooooooo003o0oooool2o`00o`;ooooo00?o003oooooooooool02oooool01?l0003ooooooooooooo
ool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo06Wooooo0ol000001?l00?ooooooo`00
ooooool4o`00o`03ooooool00?oo003o04;ooooo0ol0000Mooooo`006oooool3o`0000?ooooo00Go
003oooooooooooooooooo`000002o`00o`;ooooo00?o003oooooooooool02Oooool01?l0003ooooo
ooooooooool2o`00o`;ooooo0_l00?l2ooooo`;o003oJoooool00ol0003oooooo`00o`04o`00o`03
ooooool00?oooooo00Co003o@oooool3o`0001gooooo000Looooo`03o`00oooooooooooo00?ooooo
00Co003ooooooooooooooooo0_l00?l2ooooo`03o`00oooooooooooo00Sooooo00?o0000oooooooo
ool00ol00?l01?oooooo003oo`00ool00?l2ooooo`03o`00oooooooooooo06[ooooo00Oo0000oooo
ool00?ooooooo`000?l00?oooooo00?o003o00Coooooo`00oooooooo003oA_ooool00ol00?oooooo
ooooo`0Kooooo`006_ooool3o`0000Kooooo00Co003ooooooooooooo00000_l00?l2ooooo`03o`00
oooooooooooo00Kooooo00Oo0000oooooooooooo003oooooool00?oooooo00;o003o0oooool00ol0
0?ooooooooooo`1Yooooo`;o000000Coooooo`00oooooooooooo1?l00?l00ooooooo003oo`00o`03
o`00odKooooo0ol0000Kooooo`006_ooool3o`0000Kooooo00Go003oooooooooooooooooo`000002
o`00o`;ooooo00?o003oooooooooool01?ooool00ol0003oooooooooo`03o`00o`03ooooool00?oo
003o00;ooooo0_l00?m[ooooo`05o`000?ooooooooooo`00ooooool00_l00?l02_l0003o003ooooo
ool00?ooooooo`00oooooooo003oooooool00?m6ooooo`?o00006oooool001[ooooo0ol00007oooo
o`04o`00ooooooooooooooooo`?o003o00?oooooo`00ooooool01?ooool00ol0003oooooooooo`02
o`00o`;ooooo0_l00?l2ooooo`03o`00oooooooooooo06[ooooo00Ko0000oooooooooooo003ooooo
ool00?l2ooooo`;o003o00?oooooo`00ool00?l00ol00?l00ooooooo003oooooo`15ooooo`?o0000
6oooool001[ooooo00?o003oooooooooool02?ooool01?l00?ooooooooooool00002o`00o`;ooooo
00Ko003oooooooooooooooooo`000?ooool3o`00o`05ooooool00?oo003oo`00ooooool00_l00?m/
ooooo`06o`000?ooooooooooo`00oooooooo003o0_ooool02ol00?oo0000o`00oooooooo003ooooo
ool00?ooooooo`00oooooooo003o04Sooooo00?o003oooooooooool06Oooool001Sooooo0ol0000:
ooooo`05o`00ooooooooooooooooool000000_l00?l03ooooooo003ooooooooooooo0000ooooool0
0?ooooooo`00oooooooo003oooooool00?ooooooo`00o`1/ooooo`;o00000_ooool00ol00?oooooo
o`00o`02ooooo`03o`00oooooooo000000Go003o00Coooooo`00oooooooo003oB?ooool3o`0001Wo
oooo000Hooooo`?o00002oooool01?l00?ooooooooooool00003o`00o`08ooooool00?oo0000oooo
ool00?ooooooo`00ooooool3o`00o`03ooooool00?oooooo06_ooooo00Oo0000oooooooooooooooo
o`00oooooooo003o00;ooooo00?o003oooooool000000_l00?l02?oooooo003oooooool00?oooooo
o`00oooooooo003oB?ooool3o`0001Wooooo000Hooooo`?o00003?ooool02?l00?ooooooooooool0
003o003oooooool00?oo00000_l00?l00ooooooo003oooooo`02o`00o`03ooooool00?oo003o06co
oooo00Oo0000ooooooooooooooooo`00oooooooo003o00;ooooo00Go003ooooooooooooo003ooooo
o`03o`00o`06ooooool00?ooooooo`00oooooooo003oB?ooool3o`0001Wooooo000Fooooo`?o0000
3_ooool01Ol00?ooooooooooool0003o003o00;o00001ol00?l00ooooooo003oooooo`1[ooooo`;o
00000oooool3o`00o`;ooooo00oo003ooooooooooooo003oo`000?oooooo003oooooool00?oooooo
o`00oooooooo003oooooool00?l0B_ooool3o`0001Oooooo000Fooooo`?o00003oooool01?l00?oo
ooooooooool00006o`00o`04ooooool00?ooooooo`00ofcooooo00Co0000oooooooooooooooo0ol0
0?l3ooooo`06o`00ooooooooooooo`00oooooooo00000_l00?l02?oooooo003oooooool00?oooooo
o`00oooooooo003oB_ooool3o`0001Oooooo000Fooooo`?o00004?ooool00ol00?oo0000o`000009
o`00ofcooooo00Oo0000ooooooooooooooooo`00oooooooo003o00?ooooo00Co003ooooooooooooo
003o0_ooool02ol00?ooooooo`00oooooooo003oooooool00?ooooooo`00oooooooo003o04[ooooo
0ol0000Gooooo`005_ooool00ol0003o003oooooo`0@ooooo`04o`000?oooooo003oo`0000Ko003o
KOooool01ol0003ooooooooooooooooo003oooooool00?l00_ooool2o`00o`03ooooool00?oo003o
00;ooooo00co003oo`000?oooooo003oooooool00?ooooooo`00oooooooo003oooooool00?m;oooo
o`03o`000?l00?oooooo01Kooooo000Eooooo`?o00004?ooool01_l0003oooooo`00oooooooo003o
o`0000Go003oJoooool2o`0000?ooooo00?o003oooooool00?l00_ooool01?l00?ooooooooooool0
0?l3ooooo`0=o`00oooooooo0000ooooool00?ooooooo`00oooooooo003oooooool00?ooooooo`00
o`1;ooooo`?o00005_ooool001Gooooo0ol0000?ooooo`03o`000?l00?oo003o00So003o00?ooooo
o`00ooooool0J?ooool00ol0003oooooooooo`02ooooo`03o`00oooooooo003o00;ooooo00Co003o
oooooooooooo003o0oooool03_l00?ooooooooooool0003oooooo`00oooooooo003oooooool00?oo
ooooo`00oooooooo003oBoooool3o`0001Kooooo000Eooooo`?o00003_ooool01Ol0003o003ooooo
ool00?oooooo00Ko003o00?oooooo`00ool00?l0J?ooool00ol0003oooooooooo`02ooooo`?o003o
0_ooool01?l00?ooooooooooool00?l3ooooo`0?o`00ooooooooooooooooool0003oooooo`00oooo
oooo003oooooool00?ooooooo`00oooooooo003o04_ooooo0ol0000Fooooo`005Oooool00ol00?oo
ooooooooo`0=ooooo`07o`000?l00?ooooooo`00oooooooo003oooooo`02o`00o`05o`000?oooooo
003oo`00ooooool00_l00?mUooooo`;o00001?ooool2o`00o`?ooooo00Co003ooooooooooooo003o
0oooool00ol00?ooooooooooo`02ooooo`0;o`000?oooooo003oooooool00?ooooooo`00oooooooo
003oooooool00?l0COooool00ol00?ooooooooooo`0Dooooo`004oooool3o`0000kooooo00go0000
o`00oooooooo003oooooool00?ooooooo`00oooooooo003oooooool0003ooooo00;o003o00?ooooo
o`00ooooool0Hoooool00ol0003oooooooooo`03ooooo`;o003o0oooool01?l00?ooooooooooool0
0?l3ooooo`03o`00oooooooooooo00?ooooo00Co0000ooooool00?oo003o0_ooool2o`00o`;ooooo
00?o003oooooooooool0Boooool3o`0001Cooooo000Cooooo`?o00003Oooool2o`00o`06ooooool0
0?oo003oo`00oooooooo003o0oooool00ol00?oo0000ooooo`02o`00o`03ooooool00?oo003o06;o
oooo00?o0000ooooooooool00oooool2o`00o`?ooooo00Co003oooooool00?oo003o0oooool00ol0
0?ooooooooooo`04ooooo`04o`000?oooooo003oo`00o`;ooooo0_l00?l2ooooo`03o`00oooooooo
oooo04_ooooo0ol0000Dooooo`004oooool3o`0000cooooo00?o003oooooooooool00_l00?l00ooo
oooo003oo`00o`04ooooo`04o`00oooooooo0000ooooo`;o003o00?oooooo`00ool00?l0Goooool2
o`0000Gooooo0_l00?l3ooooo`03o`00oooooooo003o00Cooooo00?o003oooooooooool01Oooool0
1?l0003oooooo`00ool00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1:ooooo`?o00005?oo
ool001?ooooo00?o0000o`00ooooool02oooool00ol00?ooooooo`00o`02o`00o`03ooooool00?oo
oooo00Kooooo00So003oooooool0003o003oooooool00?ooooooo`00oekooooo00?o0000oooooooo
ool00oooool3o`00o`?ooooo00?o003oooooool00?l01?ooool00ol00?ooooooooooo`05ooooo`03
o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo00?o003oooooooooool0Boooool2o`00oaCo
oooo000Booooo`?o00002oooool01ol00?ooooooo`00oooooooo003oooooool00?l02Oooool00ol0
0?oo0000ooooo`02o`00o`;ooooo00?o003oooooooooool0F_ooool00ol0003oooooooooo`03oooo
o`?o003o0oooool00ol00?ooooooo`00o`04ooooo`03o`00oooooooooooo00Kooooo00?o0000oooo
ooooool00_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1;ooooo`?o00004oooool001;o
oooo0ol00009ooooo`;o003o00?oooooo`00ool00?l00ol00?l:ooooo`04o`00oooooooo0000oooo
o`;o003o00?oooooo`00ool00?l0F_ooool00ol0003oooooooooo`03ooooo`;o003o0oooool2o`00
o`03ooooool00?oooooo00?ooooo00?o003oooooooooool01oooool00ol0003oooooooooo`02o`00
o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo04_ooooo0ol0000Cooooo`004_ooool3o`0000So
oooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0<ooooo`08o`00ool0003oooooo`00
oooooooo003oooooool00?mHooooo`;o00001Oooool2o`00o`?ooooo00Co003ooooooooooooo003o
1?ooool00ol00?ooooooooooo`08ooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo
00?o003oooooooooool0Boooool3o`0001?ooooo000Booooo`;o003o2?ooool00ol00?oo0000oooo
o`05o`00o`kooooo00?o003oo`000?ooool00_l00?l2ooooo`03o`00oooooooooooo05Cooooo00?o
0000ooooooooool01?ooool2o`00o`?ooooo00Co003oooooool00?oo003o1?ooool00ol00?oooooo
ooooo`09ooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo00?o003oooooooooool0
C?ooool2o`00oa?ooooo000Aooooo`?o00001oooool00ol00?oo0000ooooo`05o`00o`oooooo00Co
003oo`000?oooooooooo0_l00?l00ooooooo003oo`00o`1Dooooo`03o`000?oooooooooo00Cooooo
0_l00?l3ooooo`03o`00oooooooo003o00Gooooo00?o003oooooooooool02_ooool00ol0003ooooo
ooooo`02o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo04cooooo0ol0000Booooo`004Ooo
ool3o`0000Gooooo0_l00?l00ol0003o003oo`00o`03o`00oa;ooooo00So003oo`000?oooooo003o
ooooool00?ooooooo`00oe;ooooo0_l00006ooooo`;o003o0oooool00ol00?ooooooo`00o`05oooo
o`03o`00oooooooooooo00_ooooo00?o0000ooooooooool00_l00?l2ooooo`;o003o0oooool00ol0
0?ooooooooooo`1<ooooo`?o00004_ooool0017ooooo0ol00004ooooo`03o`00oooooooo000000Go
003o5?ooool00ol00?oo0000ooooo`02o`00o`;ooooo00?o003oooooooooool0C_ooool00ol0003o
ooooooooo`05ooooo`;o003o0oooool00ol00?ooooooo`00o`05ooooo`03o`00oooooooooooo00co
oooo00?o0000ooooooooool00_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1<ooooo`?o
00004_ooool0017ooooo0_l00?l4ooooo`03o`00oooooooo000000?o003o00?oooooo`00ooooool0
5?ooool01ol00?oo0000ooooool00?ooooooo`00ooooool00_l00?m>ooooo`03o`000?oooooooooo
00Gooooo00?o003oooooooooool00_ooool00ol00?ooooooo`00o`05ooooo`03o`00oooooooooooo
00gooooo00?o0000ooooooooool00_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1=oooo
o`;o003o4_ooool000oooooo0onYZJT4ooooo`03o`00oooooooo000000?o003o00?oooooo`00oooo
ool05_ooool01ol00?oo0000ooooool00?ooooooo`00ooooool00_l00?m<ooooo`03o`000?oooooo
oooo00Gooooo00?o003oooooooooool00_ooool00ol00?ooooooo`00o`05ooooo`03o`00oooooooo
oooo00kooooo00?o0000ooooooooool00_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1=
ooooo`?o00004Oooool000oooooo0onYZJT2ooooo`;o003o00?oooooo`000?l00?l01?l00?lIoooo
o`03o`000?oooooooooo00;o003o0_ooool00ol00?ooooooooooo`18ooooo`;o00001_ooool2o`00
o`Cooooo0ol00?l5ooooo`03o`00oooooooooooo00oooooo00?o0000ooooooooool00_l00?l2oooo
o`;o003o0oooool00ol00?ooooooooooo`1=ooooo`?o00004Oooool000oooooo0onYZJT01Ooooooo
003ooooooooooooo000000;o003o00?oooooo`00ooooool06_ooool01ol00?oo0000ooooool00?oo
ooooo`00ooooool00_l00?m8ooooo`03o`000?oooooooooo00Gooooo0_l00?l4ooooo`;o003o1_oo
ool00ol00?ooooooooooo`0@ooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo00?o
003oooooooooool0COooool3o`00017ooooo000@ooooo`?o003o0_ooool01Ol0003o003oo`00oooo
oooo003o01gooooo00Oo0000oooooooooooo003oooooool00?oooooo00;o003oA_ooool00ol0003o
ooooooooo`05ooooo`;o003o1?ooool2o`00o`Kooooo00?o003oooooooooool04?ooool01?l0003o
ooooooooooooool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo04kooooo0_l00?lAoooo
o`003oooool3ojVYZ@;ooooo00Go0000o`00ool00?ooooooo`00o`0Oooooo`03o`000?oooooooooo
00;o003o0_ooool2o`00od?ooooo0_l00007ooooo`;o003o0oooool3o`00o`Kooooo00?o003ooooo
ooooool04Oooool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooo
oooo04kooooo0ol0000@ooooo`003oooool3ojVYZ@03ooooool00?oo003o00;ooooo00?o003ooooo
ooooool07_ooool01ol00?oo0000ooooool00?ooooooo`00ooooool00_l00?m2ooooo`03o`000?oo
oooooooo00Kooooo00?o003oooooooooool00_ooool00ol00?ooooooo`00o`06ooooo`03o`00oooo
oooooooo01;ooooo00Co0000oooooooooooooooo0_l00?l2ooooo`;o003o0oooool00ol00?oooooo
ooooo`1>ooooo`?o00004?ooool000oooooo0onYZJT2o`00o`;ooooo00?o003oooooooooool08?oo
ool01ol0003oooooooooool00?ooooooo`00ooooool00_l00?m0ooooo`03o`000?oooooooooo00Ko
oooo00?o003oooooooooool00_ooool00ol00?ooooooo`00o`06ooooo`03o`00oooooooooooo01?o
oooo00Co0000oooooooooooooooo0_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1>oooo
o`?o00004?ooool000oooooo0ol00?l2ooooo`;o003o9?ooool00ol0003oooooooooo`02o`00o`04
ooooool00?ooooooo`00ockooooo00?o0000ooooooooool01_ooool00ol00?ooooooooooo`02oooo
o`?o003o1_ooool00ol00?ooooooooooo`0Dooooo`04o`000?ooooooooooooooo`;o003o0_ooool2
o`00o`?ooooo00?o003oooooooooool0Coooool2o`00oa3ooooo000>ooooo`?oZJVY0_ooool00ol0
0?ooooooooooo`0Tooooo`0:o`000?ooooooooooo`00oooooooo003oooooool00?ooooooo`00oc_o
oooo0_l00008ooooo`03o`00oooooooooooo00;ooooo0_l00?l7ooooo`03o`00oooooooooooo01Go
oooo00Co0000oooooooooooooooo0_l00?l2ooooo`;o003o1?ooool00ol00?ooooooooooo`1>oooo
o`?o00003oooool000kooooo0onYZJT00ooooooo003oooooo`0Wooooo`07o`000?ooooooooooo`00
oooooooo003oooooo`02o`00oc[ooooo00?o0000ooooooooool01oooool00ol00?ooooooooooo`02
ooooo`;o003o1oooool00ol00?ooooooooooo`0Fooooo`04o`000?ooooooooooooooo`;o003o0_oo
ool2o`00o`Cooooo00?o003oooooooooool0C_ooool3o`0000oooooo000>ooooo`?oZJVY00?o003o
ooooooooool09oooool02_l0003o003oooooool00?ooooooo`00oooooooo003oooooool00?lhoooo
o`03o`000?oooooooooo00Kooooo0_l00?l4ooooo`;o003o1oooool00ol00?ooooooooooo`0Goooo
o`04o`000?ooooooooooooooo`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool0C_ooool3
o`0000oooooo000>ooooo`03o`000?l00?oo003o02_ooooo00[o0000oooooooooooo003oooooool0
0?ooooooo`00oooooooo003o=_ooool00ol0003oooooooooo`06ooooo`;o003o1?ooool2o`00o`Oo
oooo00?o003oooooooooool06?ooool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l4oooo
o`03o`00oooooooooooo04oooooo0_l00?l?ooooo`003Oooool3ojVYZBgooooo00[o0000oooooooo
oooo003oooooool00?ooooooo`00oooooooo003o=Oooool00ol0003oooooooooo`05ooooo`03o`00
oooooooooooo00?ooooo0_l00?l7ooooo`03o`00oooooooooooo01Wooooo00Co0000oooooooooooo
oooo0_l00?l2ooooo`;o003o1?ooool00ol00?ooooooooooo`1?ooooo`?o00003_ooool000gooooo
0onYZJT]ooooo`04o`000?l00?ooooooo`00o`;ooooo0_l00?l2ooooo`03o`00oooooooooooo037o
oooo00?o0000ooooooooool01Oooool00ol00?ooooooooooo`03ooooo`;o003o1oooool00ol00?oo
ooooooooo`0Jooooo`04o`000?ooooooooooooooo`;o003o0_ooool2o`00o`Cooooo00?o003ooooo
ooooool0Coooool3o`0000kooooo000=ooooo`?oZJVY;_ooool02_l0003oooooooooool00?oooooo
o`00oooooooo003oooooool00?lbooooo`03o`000?oooooooooo00Gooooo00?o003oooooooooool0
0_ooool2o`00o`Sooooo00?o003oooooooooool06oooool01?l0003oooooooooooooool2o`00o`;o
oooo0_l00?l4ooooo`03o`00oooooooooooo04oooooo0ol0000>ooooo`003_ooool00ol00?oooooo
ooooo`0]ooooo`0;o`000?l00?ooooooooooool00?ooooooo`00oooooooo003oooooool00?l0<?oo
ool00ol0003oooooooooo`04ooooo`;o003o1?ooool2o`00o`Sooooo00?o003oooooooooool06ooo
ool00ol0003oooooooooo`02ooooo`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool0D?oo
ool00ol00?ooooooooooo`0=ooooo`003?ooool3ojVYZC3ooooo00Co0000o`00oooooooo003o0_oo
ool2o`00o`;ooooo00?o003oooooooooool0;?ooool00ol0003oooooooooo`04ooooo`;o003o1?oo
ool2o`00o`Sooooo00?o003oooooooooool07?ooool00ol0003oooooooooo`02ooooo`;o003o0_oo
ool2o`00o`Cooooo00?o003oooooooooool0Coooool3o`0000kooooo000<ooooo`?oZJVY<?ooool0
2Ol0003oooooo`00oooooooo003oooooool00?ooooooo`00o`02ooooo`03o`00oooooooooooo02[o
oooo00?o0000ooooooooool01?ooool2o`00o`Cooooo0_l00?l8ooooo`03o`00oooooooooooo01go
oooo00?o0000ooooooooool00_ooool2o`00o`;ooooo0_l00?l4ooooo`03o`00oooooooooooo04oo
oooo0ol0000>ooooo`003?ooool3ojVYZC7ooooo00Ko0000o`00ooooooooooooo`00ooooool2o`00
o`;ooooo00?o003oooooooooool0:Oooool00ol0003oooooooooo`04ooooo`;o003o1?ooool2o`00
o`Sooooo00?o003oooooooooool07_ooool00ol0003oooooooooo`02ooooo`;o003o0_ooool2o`00
o`Cooooo00?o003oooooooooool0Coooool3o`0000kooooo000=ooooo`;o003o<Oooool01Ol0003o
ooooo`00oooooooo003o00;ooooo0_l00?l2ooooo`03o`00oooooooooooo02Oooooo00?o0000oooo
ooooool01?ooool2o`00o`Cooooo0_l00?l8ooooo`03o`00oooooooooooo01oooooo00?o0000oooo
ooooool00_ooool01Ol00?oooooooooooooooooo003o00Gooooo00?o003oooooooooool0D?ooool2
o`00o`kooooo000<ooooo`?o0000<_ooool02Ol0003oooooo`00oooooooo003oooooool00?oooooo
o`00o`02ooooo`03o`00oooooooooooo02Gooooo00?o0000ooooooooool01?ooool2o`00o`Cooooo
00?o003oooooooooool01oooool00ol00?ooooooooooo`0Pooooo`03o`000?oooooooooo00;ooooo
00Go003oooooooooooooooooo`00o`05ooooo`03o`00oooooooooooo053ooooo0ol0000=ooooo`00
3?ooool3o`0003?ooooo00Co0000o`00oooooooo003o0_ooool2o`00o`?ooooo00?o003ooooooooo
ool08oooool00ol0003oooooooooo`03ooooo`?o003o1?ooool00ol00?ooooooooooo`07ooooo`03
o`00oooooooooooo027ooooo00?o0000ooooooooool00_ooool01Ol00?oooooooooooooooooo003o
00Gooooo00?o003oooooooooool0D?ooool3o`0000gooooo000<ooooo`?o0000<oooool01Ol0003o
ooooo`00oooooooo003o00;ooooo0_l00?l2ooooo`03o`00oooooooooooo02;ooooo00?o0000oooo
ooooool00oooool3o`00o`Cooooo00?o003oooooooooool01oooool00ol00?ooooooooooo`0Roooo
o`03o`000?oooooooooo00;ooooo00Go003oooooooooooooooooo`00o`05ooooo`03o`00oooooooo
oooo053ooooo0ol0000=ooooo`003Oooool00ol00?ooooooooooo`0cooooo`09o`000?oooooo003o
ooooool00?ooooooo`00oooooooo003o00;ooooo00?o003oooooooooool08?ooool00ol0003ooooo
ooooo`03ooooo`;o003o1Oooool00ol00?ooooooooooo`07ooooo`03o`00oooooooooooo02?ooooo
00?o0000ooooooooool00_ooool01Ol00?oooooooooooooooooo003o00Gooooo00?o003ooooooooo
ool0DOooool00ol00?ooooooooooo`0<ooooo`003?ooool3o`0003Cooooo00Go0000ooooool00?oo
ooooo`00o`02ooooo`;o003o0oooool00ol00?ooooooooooo`0Nooooo`03o`000?oooooooooo00?o
oooo0_l00?l4ooooo`;o003o2?ooool2o`00obKooooo00?o0000ooooooooool00_ooool01Ol00?oo
oooooooooooooooo003o00Gooooo00?o003oooooooooool0D?ooool3o`0000gooooo000<ooooo`?o
0000=Oooool01Ol0003oooooo`00oooooooo003o00;ooooo0_l00?l3ooooo`03o`00oooooooooooo
01cooooo00?o0000ooooooooool00oooool2o`00o`Cooooo0_l00?l8ooooo`03o`00oooooooooooo
02Kooooo00?o0000ooooooooool00_ooool01Ol00?oooooooooooooooooo003o00Gooooo00?o003o
ooooooooool0D?ooool3o`0000gooooo000<ooooo`?o0000=Oooool02_l0003oooooooooool00?oo
ooooo`00oooooooo003oooooool00?l2ooooo`03o`00oooooooooooo01_ooooo00?o0000oooooooo
ool00oooool2o`00o`Cooooo00?o003oooooooooool01oooool00ol00?ooooooooooo`0Wooooo`03
o`000?oooooooooo00;ooooo00Go003oooooooooooooooooo`00o`06ooooo`03o`00oooooooooooo
04oooooo0ol0000=ooooo`003Oooool00ol00?ooooooooooo`0eooooo`05o`000?oooooo003ooooo
ool00?l00_ooool2o`00o`?ooooo00?o003oooooooooool06Oooool00ol0003oooooooooo`02oooo
o`?o003o1?ooool00ol00?ooooooooooo`06ooooo`;o003o:_ooool00ol0003oooooooooo`02oooo
o`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`1@ooooo`03o`00oooooooo
oooo00cooooo000;ooooo`?o0000>?ooool01Ol0003oooooo`00oooooooo003o00;ooooo0_l00?l3
ooooo`03o`00oooooooooooo01Oooooo00?o0000ooooooooool00_ooool00ol00?ooooooo`00o`04
ooooo`03o`00oooooooooooo00Kooooo00?o003oooooooooool0:Oooool00ol0003oooooooooo`03
ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`1?ooooo`?o00003Ooo
ool000_ooooo0ol0000hooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`Cooooo00?o003o
ooooooooool05Oooool00ol0003oooooooooo`02ooooo`03o`00oooooooo003o00Cooooo00?o003o
ooooooooool01_ooool00ol00?ooooooooooo`0Zooooo`03o`000?oooooooooo00?ooooo00Go003o
ooooooooooooooooo`00o`06ooooo`03o`00oooooooooooo04oooooo0ol0000=ooooo`002oooool3
o`0003Wooooo00Go0000ooooool00?ooooooo`00o`02ooooo`;o003o0oooool00ol00?oooooooooo
o`0Dooooo`03o`000?oooooooooo00;ooooo00?o003oooooool00?l01?ooool00ol00?oooooooooo
o`06ooooo`03o`00oooooooooooo02_ooooo00?o0000ooooooooool00oooool01Ol00?oooooooooo
oooooooo003o00Kooooo00?o003oooooooooool0Coooool3o`0000gooooo000<ooooo`03o`000?l0
0?oooooo03Sooooo00Ko0000oooooooooooo003oooooool00?l2ooooo`;o003o0oooool00ol00?oo
ooooooooo`0Booooo`03o`000?oooooooooo00;ooooo00?o003oooooool00?l00oooool2o`00o`Oo
oooo0_l00?l^ooooo`03o`000?oooooooooo00?ooooo00Go003oooooooooooooooooo`00o`06oooo
o`03o`00oooooooooooo053ooooo00?o0000o`00ooooool03?ooool000_ooooo0ol0000jooooo`03
o`000?oooooooooo00;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool04?ooool00ol0003o
ooooooooo`02ooooo`?o003o0oooool2o`00o`Oooooo00?o003oooooooooool0;_ooool00ol0003o
ooooooooo`03ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`1?oooo
o`?o00003Oooool000_ooooo0ol0000jooooo`06o`000?ooooooooooo`00oooooooo003o0_ooool2
o`00o`Cooooo00?o003oooooooooool03_ooool01?l0003oooooooooooooool3o`00o`Cooooo0_l0
0?l7ooooo`03o`00oooooooooooo02oooooo00?o0000ooooooooool00oooool01Ol00?oooooooooo
oooooooo003o00Kooooo00?o003oooooooooool0Coooool3o`0000gooooo000;ooooo`?o0000>ooo
ool01_l0003oooooooooool00?ooooooo`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo00go
oooo00Oo0000ooooooooooooooooo`00oooooooo003o00Cooooo0_l00?l6ooooo`;o003o<_ooool0
0ol0003oooooooooo`03ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?oooooooooo
o`1?ooooo`?o00003Oooool000cooooo00?o0000o`00ooooool0>oooool00ol0003oooooooooo`02
o`00o`;ooooo0_l00?l4ooooo`03o`00oooooooooooo00_ooooo00Oo0000ooooooooooooooooo`00
oooooooo003o00Cooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`0booooo`03o`00
0?oooooooooo00?ooooo00Go003oooooooooooooooooo`00o`06ooooo`03o`00oooooooooooo053o
oooo0_l00?l=ooooo`002oooool3o`0003cooooo00Ko0000oooooooooooo003oooooool00?l2oooo
o`;o003o1?ooool00ol00?ooooooooooo`09ooooo`07o`000?ooooooooooooooool00?ooooooo`00
o`04ooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool0<oooool00ol0003ooooooooo
o`03ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`1?ooooo`?o0000
3Oooool000_ooooo0ol00008o`00ocGooooo00Ko0000oooooooooooo003oooooool00?l2ooooo`03
o`00oooooooooooo00?ooooo00?o003oooooooooool01oooool01ol0003ooooooooooooooooo003o
ooooool00?l00oooool2o`00o`Oooooo00?o003oooooooooool0=?ooool00ol0003oooooooooo`03
ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`1?ooooo`?o00003Ooo
ool000_ooooo0ol00008oooooa3o003o9Oooool01?l0003oooooooooooooool2o`00o`;ooooo0_l0
0?l4ooooo`03o`00oooooooooooo00Kooooo00Oo0000ooooooooooooooooo`00oooooooo003o00?o
oooo0_l00?l6ooooo`;o003o=oooool00ol0003oooooooooo`03ooooo`05o`00oooooooooooooooo
ool00?l01_ooool00ol00?ooooooooooo`1?ooooo`?o00003Oooool000gooooo00?o003ooooooooo
ool05_ooool@o`00oaKooooo00Ko0000oooooooooooo003oooooool00?l2ooooo`;o003o1?ooool0
0ol00?ooooooooooo`04ooooo`03o`000?oooooooooo00;o003o00?oooooo`00ooooool00_ooool2
o`00o`;ooooo9_l00?lHooooo`03o`000?oooooooooo00?ooooo00Go003oooooooooooooooooo`00
o`06ooooo`03o`00oooooooooooo053ooooo00?o003oooooooooool03?ooool000cooooo0ol0000O
oooooako003o00?o0000o`00ool00?l04Ol00?l00ol0003o003oo`00o`0<o`00o`?ooooo00?o003o
ooooooooool08?ooool9o`00o`oooooo00?o0000ooooooooool00oooool01Ol00?oooooooooooooo
oooo003o00Kooooo00?o003oooooooooool0Coooool3o`0000gooooo000<ooooo`?o00007ol00?lI
ooooo`Ko003o00?o0000o`00ool00?l01ol00?l5ooooo`04o`00ooooooooooooo`0000;ooooo00Co
003ooooooooooooo003o0oooool2o`00o`Gooooo0_l00?l/ooooo`Wo003o1_ooool00ol0003ooooo
ooooo`03ooooo`05o`00ooooooooooooooooool00?l01Oooool00ol00?ooooooooooo`1@ooooo`?o
00003Oooool000cooooo0ol00003ooooo`Co0000:Ol00?l?ooooo`06o`000?ooooooooooo`00oooo
oooo003o0_ooool8o`00o`03o`000?l00?oo003o00Ko003o00?oooooo`00ool00?l01Oooool00ol0
0?ooooooooooo`0eooooo`Ko003o00?o0000o`00ool00?l00oooool01Ol00?oooooooooooooooooo
003o00Gooooo00?o003oooooooooool0D?ooool3o`0000gooooo000=ooooo`03o`00oooooooooooo
00_ooooo2Ol00002oooooa3o003o2Oooool?o`00o`03o`000?l00?oo003o00co003o00Ko0000o`00
oooooooo003oooooool00?l3oooooa7o003o=_ooool00ol0003oooooooooo`09o`00o`Cooooo00?o
003oooooooooool0DOooool00ol00?ooooooooooo`0<ooooo`003?ooool3o`0001Gooooo2?l0000:
oooooa3o003o2Oooool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l3ooooo`03o`000?oo
oooo003o02So003o9oooool00ol0003oooooooooo`02ooooo`06o`00ooooooooooooooooool00?oo
oooo2_l00?m=ooooo`?o00003Oooool000cooooo0ol0000?o`00o`kooooo2Ol0000Aooooo`Wo003o
00?o0000o`00ool00?l01Ol00?l01_oooooo003oo`00oooooooo0000ooooo`Co003o0oooool3o`00
o`Cooooo0_l00?lBooooob7o003o3_ooool00ol0003oooooooooo`02ooooo`05o`00oooooooooooo
ooooool00?l01Oooool00ol00?ooooooooooo`03ooooo`Wo003oA?ooool3o`0000gooooo000<oooo
o`?o00003ooooolGo`00o`So00000_l00?lAooooo`04o`000?ooooooooooooooo`;o003o00Gooooo
o`00ool00?oo003oo`00000<o`00o`?ooooo00?o003oooooooooool0:ooooolFo`00o`03o`000?l0
0?oo003o00oo003o3?ooool9o`00oc_ooooo0ol0000=ooooo`003Oooool2o`00obkooooo2Ol0000:
o`00o`03o`000?l00?oo003o00Ko003o00?o0000o`00ool00?l00ol00?l00ooooooo003oooooo`02
o`00o`?ooooo4?l00?laooooo`Ko003o00?o0000o`00ool00?l01ol00?l5ooooo`03o`00oooooooo
oooo023o003o<?ooool2o`00o`kooooo000=ooooo`;o003o=oooool9o`0000;ooooo00[o0000oooo
ooooooooooooo`00oooooooo003oo`000?oooooo003o0_ooool00ol00?ooooooo`00o`0Wo`00obOo
oooo00?o0000ooooooooool00_ooool01?l00?oooooooooooooooolAo`00oaKooooo8ol00?l?oooo
o`;o003o3_ooool000cooooo0ol00010ooooo`Oo000000Go003oo`000?oooooo003oooooo`02o`00
o`;ooooo1?l00?l3ooooo`03o`00oooooooooooo01Kooooo8Ol00?l>ooooo`03o`000?oooooooooo
00;ooooo00Go003oooooooooooooooooo`00o`05ooooo`03o`00oooooooooooo00Sooooo4?l00?l@
ooooo`Wo003o4?ooool>o`00o`?o00003_ooool000cooooo0ol00013ooooo`05o`000?oooooooooo
ooooool000000_l00?l2o`0000;o003o00Co0000o`00ool0003o003o0_ooool00ol00?ooooooo`00
o`0aoooooaOo003o00?o0000o`00ool00?l03ol00?lHoooooa7o003o2?ooool9o`00oaGooooo0ol0
000>ooooo`003?ooool3o`0004Cooooo00?o0000ooooool000000ol00?l00ooooooo003oooooo`02
o`00o`03ooooool00?oo000000;o00000_l00?l3o`0003oooooo1ol00?l00ol0003o003oo`00o`07
o`00o`Gooooo00?o003oooooooooool08?l00?l9oooooa3o003o00?oooooo`00ool00?l01ol00?l<
ooooo`?o00003_ooool000kooooo00?o003oooooooooool0@_ooool2o`000003o`00oooooooooooo
00;o003o0_ooool3o`00o`?ooooo00?o003oooooool00?l00oooool9o`0003gooooo00Co0000oooo
oooooooooooo0_l00?l2oooooa7o003o6Oooool/o`00o`Cooooo00?o003oooooooooool03Oooool0
00gooooo0ol00013ooooo`06o`000?l00?ooooooooooool00?oooooo0ol00?l00ooooooo003oo`00
o`02ooooo`05o`00ooooooooooooooooool00?l02oooool9o`0003Cooooo00Co0000oooooooooooo
oooo0_l00?l2ooooo`;o003o1?ooool00ol00?ooooooooooo`08oooooa7o003o8_ooool?o`0000Ko
003o0ol0000>ooooo`003Oooool3o`0004;ooooo00Go0000o`00ool0003oooooo`00o`03ooooo`07
o`00oooooooo003oooooool00?ooooooo`00o`05ooooo`03o`00oooooooooooo017ooooo2?l0000/
ooooo`04o`000?ooooooooooooooo`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool04?oo
ool9o`00013o003o4_l0000Eooooo`?o00003_ooool000gooooo0ol00011ooooo`05o`000?l00?oo
ooooooooool00?l00_ooool4o`00o`;ooooo0_l00?l6ooooo`03o`00oooooooooooo01Wooooo2Ol0
000Iooooo`ko00000_l00?l2o`0000;o003o1?l000000ol00?oo0000o`00000@o`0001Wooooo4?l0
0?lGooooo`?o00003_ooool000kooooo00?o0000o`00ooooool0?oooool02ol0003o003ooooooooo
oooo003oo`000?oooooo003oooooool00?oooooo00?o003o0_ooool00ol00?ooooooooooo`04oooo
o`03o`00oooooooooooo027ooooo6Ol0000:ooooo`04o`000?ooooooooooooooo`;o003o0_ooool2
o`00o`Cooooo00?o003oooooooooool0>Oooool@o`00o`Oooooo0_l00?l?ooooo`003_ooool3o`00
03kooooo0_l00?l2ooooo`06o`00ooooooooooooo`00oooooooo003o0oooool01Ol00?oooooooooo
oooooooo003o00Oooooo00?o003oooooooooool0@oooool01?l0003oooooooooooooool2o`00o`;o
oooo0_l00?l4ooooo`03o`00oooooooooooo04Wooooo1_l00?l3o`0000oooooo000>ooooo`?o0000
?Oooool01Ol00?oooooooooooooooooo003o00;ooooo00?o003oooooool00?l00_ooool4o`00o`?o
oooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`12ooooo`04o`000?oooooooooooooo
o`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool0Coooool3o`0000oooooo000>ooooo`?o
0000??ooool01Ol00?oooooooooooooooooo003o00;ooooo00Ko003oooooool00?oo0000ooooool0
0?l3ooooo`04o`00ooooooooooooooooo`;o003o1_ooool00ol00?ooooooooooo`12ooooo`04o`00
0?ooooooooooooooo`;o003o0_ooool2o`00o`Cooooo00?o003oooooooooool0Coooool3o`0000oo
oooo000?ooooo`;o003o>oooool01Ol00?oooooooooooooooooo003o00;ooooo00?o003oooooool0
0?l00_ooool00ol00?ooooooooooo`03ooooo`05o`00ooooooooooooooooool00?l01oooool00ol0
0?ooooooooooo`11ooooo`04o`000?ooooooooooooooo`;o003o0_ooool2o`00o`Cooooo00?o003o
ooooooooool0Coooool2o`00oa3ooooo000?ooooo`?o0000>Oooool01_l00?oooooooooooooooooo
003oooooo`;o003o00Koooooo`00ooooooooooooo`00ool00005ooooo`;o003o0oooool00ol00?oo
ooooooooo`04ooooo`03o`00oooooooooooo047ooooo00Co0000oooooooooooooooo0_l00?l2oooo
o`;o003o1?ooool00ol00?ooooooooooo`1>ooooo`?o00004?ooool000oooooo0ol0000hooooo`03
o`00oooooooooooo00;o003o00Koooooo`00ooooooooooooo`00ooooool2o`00o`;ooooo00?o0000
ooooooooool00oooool01?l00?ooooooooooooooool2o`00o`Kooooo00?o003oooooooooool0@?oo
ool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l4ooooo`03o`00oooooooooooo04kooooo
0ol0000@ooooo`003oooool3o`0003Kooooo0_l00?l2ooooo`04o`00ooooooooooooo`00o`;ooooo
00?o003oooooool00?l01?ooool00ol0003oooooooooo`04ooooo`05o`00ooooooooooooooooool0
0?l01oooool00ol00?ooooooooooo`0oooooo`04o`000?ooooooooooooooo`;o003o0_ooool2o`00
o`?ooooo00?o003oooooooooool0Coooool3o`00013ooooo000@ooooo`;o003o=Oooool01Ol00?oo
0000oooooooooooo003o00;ooooo00Ko003ooooooooooooo003oooooool00?l6ooooo`03o`000?oo
oooooooo00?ooooo0_l00?l3ooooo`03o`00oooooooooooo00Cooooo00?o003oooooooooool0?ooo
ool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo04oooooo
0_l00?lAooooo`003oooool3o`0003Cooooo00Go003oo`000?ooooooooooo`00o`02ooooo`03o`00
oooooooo003o00?o003o2?ooool00ol0003oooooooooo`03ooooo`04o`00ooooooooooooooooo`;o
003o1_ooool00ol00?ooooooooooo`0oooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?o
oooo00?o003oooooooooool0C_ooool3o`00017ooooo000?ooooo`?o0000<oooool01Ol00?oo0000
oooooooooooo003o00;ooooo00Go003oooooool00?ooooooo`00o`0:ooooo`03o`000?oooooooooo
00Cooooo00Go003oooooooooooooooooo`00o`06ooooo`03o`00oooooooooooo03oooooo00?o0000
ooooooooool00_l00?l2ooooo`;o003o0oooool00ol00?ooooooooooo`1>ooooo`?o00004Oooool0
00oooooo0ol0000booooo`06o`00ool0003oooooooooool00?oooooo0_l00?l01?oooooo003ooooo
ool00?l<ooooo`03o`000?oooooooooo00?ooooo0_l00?l2ooooo`;o003o1_ooool00ol00?oooooo
ooooo`0nooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo00?o003oooooooooool0
C_ooool3o`00017ooooo000Aooooo`;o003o<?ooool01ol00?oo0000oooooooooooo003oooooool0
0?l00_ooool00ol00?ooooooo`00o`0=ooooo`03o`000?oooooooooo00Cooooo00Co003ooooooooo
oooooooo0_l00?l5ooooo`03o`00oooooooooooo03kooooo00?o0000ooooooooool00_l00?l2oooo
o`;o003o0oooool00ol00?ooooooooooo`1>ooooo`;o003o4_ooool0017ooooo0ol0000]ooooo`;o
003o00Ko0000oooooooooooo003oooooool00?l2ooooo`?o003o3oooool00ol0003oooooooooo`04
ooooo`05o`00ooooooooooooooooool00?l01_ooool00ol00?ooooooooooo`0mooooo`03o`000?oo
oooooooo00;o003o0_ooool2o`00o`?ooooo00?o003oooooooooool0COooool3o`0001;ooooo000A
ooooo`?o0000;?ooool00ol00?ooooooo`000002ooooo`03o`00oooooooo003o00;ooooo0_l00?lB
ooooo`03o`000?oooooooooo00?ooooo00Co003ooooooooooooooooo0_l00?l5ooooo`03o`00oooo
oooooooo03gooooo00?o0000ooooooooool00_l00?l2ooooo`;o003o0oooool00ol00?oooooooooo
o`1=ooooo`?o00004_ooool0017ooooo0ol0000[ooooo`03o`00oooooooo000000;ooooo00?o003o
ooooool00?l00_ooool2o`00oaCooooo00?o0000ooooooooool00oooool01?l00?oooooooooooooo
ool2o`00o`Gooooo00?o003oooooooooool0??ooool00ol0003oooooooooo`02o`00o`;ooooo0_l0
0?l3ooooo`03o`00oooooooooooo04gooooo0ol0000Booooo`004_ooool3o`0002Wooooo00Co003o
ooooool0003ooooo0_l00?l01?oooooo003oooooooooool2o`00oaGooooo00?o0000ooooooooool0
1?ooool01Ol00?oooooooooooooooooo003o00Kooooo00?o003oooooooooool0>oooool00ol0003o
ooooooooo`02o`00o`;ooooo0_l00?l3ooooo`03o`00oooooooooooo04cooooo0ol0000Cooooo`00
4_ooool3o`0002Sooooo00Ko003oooooool0003oooooo`00ooooool2o`00o`;ooooo00?o003ooooo
ooooool05_ooool00ol0003oooooooooo`03ooooo`04o`00ooooooooooooooooo`;o003o1Oooool0
0ol00?ooooooooooo`0kooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`?ooooo00?o003o
ooooooooool0C?ooool3o`0001?ooooo000Booooo`?o00009oooool01ol00?ooooooo`000?oooooo
003oooooool00?l00oooool00ol00?ooooooooooo`0Hooooo`03o`000?oooooooooo00?ooooo00Co
003ooooooooooooooooo0_l00?l5ooooo`03o`00oooooooooooo03[ooooo00?o0000ooooooooool0
0_l00?l2ooooo`;o003o0_ooool00ol00?ooooooooooo`1=ooooo`?o00004oooool001?ooooo0_l0
0?lUooooo`;o003o00Koooooo`000?oooooo003oooooool00?l2ooooo`;o003o7?ooool00ol0003o
ooooooooo`03ooooo`05o`00ooooooooooooooooool00?l01Oooool00ol00?ooooooooooo`0joooo
o`03o`000?oooooooooo00;o003o0_ooool2o`00o`;ooooo00?o003oooooooooool0COooool2o`00
oaCooooo000Cooooo`?o00008oooool02?l00?ooooooooooool0003oooooo`00oooooooo003o0_oo
ool2o`00oagooooo00?o0000ooooooooool00oooool01?l00?ooooooooooooooool2o`00o`Gooooo
00?o003oooooooooool0>Oooool00ol0003oooooooooo`02o`00o`;ooooo0_l00?l2ooooo`03o`00
oooooooooooo04cooooo0ol0000Dooooo`004oooool3o`0002;ooooo00So003ooooooooooooo0000
ooooool00?ooooooo`00o`;ooooo00?o003oooooooooool07_ooool00ol0003oooooooooo`03oooo
o`04o`00ooooooooooooooooo`;o003o1?ooool00ol00?ooooooooooo`0iooooo`03o`000?oooooo
oooo00;o003o0_ooool2o`00o`;ooooo00?o003oooooooooool0C?ooool3o`0001Cooooo000Coooo
o`?o00008Oooool02?l00?ooooooooooool0003oooooo`00oooooooo003o0_ooool00ol00?oooooo
ooooo`0Pooooo`03o`000?oooooooooo00;ooooo0_l00?l2ooooo`;o003o1Oooool00ol00?oooooo
ooooo`0hooooo`03o`000?oooooooooo00;o003o0_ooool2o`00o`;ooooo00?o003oooooooooool0
C?ooool3o`0001Cooooo000Eooooo`03o`00oooooooooooo01kooooo00Go003ooooooooooooo0000
ooooo`03o`00o`03ooooool00?oo003o02Cooooo00?o0000ooooooooool00_ooool01?l00?oooooo
ooooooooool2o`00o`Cooooo00?o003oooooooooool0>?ooool00ol0003oooooooooo`02o`00o`;o
oooo0_l00?l2ooooo`03o`00oooooooooooo04cooooo00?o003oooooooooool05?ooool001Gooooo
0ol0000Mooooo`05o`00ooooooooooooo`000?ooool00_l00?l2ooooo`;o003o9Oooool00ol0003o
ooooooooo`03ooooo`04o`00ooooooooooooooooo`;o003o1?ooool00ol00?ooooooooooo`0goooo
o`03o`000?oooooooooo00;o003o0_ooool2o`00o`;ooooo00?o003oooooooooool0B_ooool3o`00
01Kooooo000Eooooo`?o00006oooool2o`00o`;ooooo00Co0000ooooool00?oo003o0_ooool2o`00
obOooooo00?o0000ooooooooool00_ooool2o`00o`;ooooo00?o003oooooool00?l01?ooool00ol0
0?ooooooooooo`0fooooo`03o`000?oooooooooo00;o003o00Coooooo`00oooooooo003o0_ooool0
0ol00?ooooooooooo`1:ooooo`?o00005_ooool001Gooooo0ol0000Jooooo`06o`00oooooooooooo
ooooool0003ooooo0_l00?l01?oooooo003oo`00ool00?lYooooo`03o`000?oooooooooo00;ooooo
0_l00?l2ooooo`;o003o1?ooool00ol00?ooooooooooo`0fooooo`09o`000?oooooo003oooooool0
0?ooooooo`00oooooooo003o00;ooooo00?o003oooooooooool0B_ooool3o`0001Kooooo000Foooo
o`;o003o6Oooool01Ol00?oooooooooooooooooo000000?o003o00Coooooo`00oooooooo003o:ooo
ool00ol0003oooooooooo`02ooooo`04o`00ooooooooooooooooo`;o003o1?ooool00ol00?oooooo
ooooo`0eooooo`09o`000?oooooo003oooooool00?ooooooo`00oooooooo003o00;ooooo00?o003o
ooooooooool0B_ooool2o`00oaOooooo000Fooooo`?o00005oooool03?l00?oooooooooooooooooo
0000o`00oooooooo003oooooool00?ooooooo`00obcooooo00?o0000ooooooooool00_ooool2o`00
o`;ooooo00?o003oooooool00?l00oooool00ol00?ooooooooooo`0eooooo`09o`000?oooooo003o
ooooool00?ooooooo`00oooooooo003o00;ooooo00?o003oooooooooool0BOooool3o`0001Oooooo
000Fooooo`?o00005_ooool01Ol00?oooooooooooooooooo000000?o003o00Coooooo`00oooooooo
003o;_ooool00ol0003oooooooooo`02ooooo`;o003o0_ooool2o`00o`Cooooo00?o003ooooooooo
ool0=?ooool02Ol0003oooooo`00oooooooo003oooooool00?ooooooo`00o`02ooooo`03o`00oooo
oooooooo04Wooooo0ol0000Gooooo`005_ooool3o`0001Gooooo00Go003oooooooooooooooooo`00
0002o`00o`03ooooool00?oo003o00;o003o<?ooool00ol0003oooooooooo`02ooooo`04o`00oooo
ooooooooooooo`;o003o0oooool00ol00?ooooooooooo`0eooooo`0:o`000?l00?ooooooo`00oooo
oooo003oooooool00?ooooooo`00odcooooo0ol0000Gooooo`006?ooool00ol00?ooooooooooo`0A
ooooo`;o003o0oooool01ol0003o003oo`00oooooooo003oooooool00?l0<oooool01?l0003ooooo
ooooooooool2o`00o`;ooooo00?o003oooooool00?l00oooool00ol00?ooooooooooo`0dooooo`0:
o`000?l00?ooooooo`00oooooooo003oooooool00?ooooooo`00odcooooo00?o003oooooooooool0
5oooool001Sooooo0ol0000@ooooo`03o`00oooooooooooo00;ooooo00Oo0000o`00ool00?oooooo
o`00oooooooo003o03Cooooo00?o0000ooooooooool00_ooool2o`00o`;ooooo0_l00?l3ooooo`03
o`00oooooooooooo03Cooooo00[o0000o`00oooooooo003oooooool00?ooooooo`00oooooooo003o
B_ooool3o`0001Wooooo000Hooooo`?o00003oooool00ol00?ooooooooooo`02ooooo`03o`000?l0
0?oo003o00;o003o00?oooooo`00ooooool0=Oooool01?l0003oooooooooooooool2o`00o`;ooooo
00?o003oooooool00?l00oooool00ol00?ooooooooooo`0cooooo`0:o`000?l00?ooooooo`00oooo
oooo003oooooool00?ooooooo`00od[ooooo0ol0000Iooooo`006?ooool3o`0000kooooo00?o003o
ooooooooool00_ooool01?l0003o003oo`00ool00?l2ooooo`03o`00oooooooooooo03Kooooo00Co
0000oooooooooooooooo0_l00?l2ooooo`03o`00oooooooo003o00?ooooo00?o003oooooooooool0
<_ooool02_l0003o003oooooool00?ooooooo`00oooooooo003oooooool00?m:ooooo`?o00006Ooo
ool001[ooooo0ol0000;ooooo`03o`00oooooooooooo00;ooooo00Co0000o`00ool00?oo003o0_oo
ool00ol00?ooooooooooo`0hooooo`04o`000?ooooooooooooooo`;o003o0_ooool2o`00o`?ooooo
00?o003oooooooooool0<_ooool02_l0003o003oooooool00?ooooooo`00oooooooo003oooooool0
0?m8ooooo`?o00006oooool001[ooooo0ol0000:ooooo`03o`00oooooooooooo00;ooooo00Co0000
o`00oooooooo003o0_ooool00ol00?ooooooooooo`0iooooo`04o`000?ooooooooooooooo`;o003o
0_ooool00ol00?ooooooo`00o`03ooooo`03o`00oooooooooooo037ooooo00[o0000o`00oooooooo
003oooooool00?ooooooo`00oooooooo003oB?ooool3o`0001_ooooo000Jooooo`?o00002?ooool2
o`00o`Cooooo00Co0000o`00ool00?oo003o0_ooool00ol00?ooooooooooo`0kooooo`04o`000?oo
ooooooooooooo`;o003o0_ooool00ol00?ooooooo`00o`02ooooo`03o`00oooooooooooo037ooooo
00[o0000o`00oooooooo003oooooool00?ooooooo`00oooooooo003oB?ooool3o`0001_ooooo000L
ooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool00oooool00ol0003o003oo`00o`03
ooooo`03o`00oooooooooooo03gooooo00Co0000oooooooooooooooo0_l00?l2ooooo`;o003o0ooo
ool00ol00?ooooooooooo`0`ooooo`0:o`000?l00?ooooooo`00oooooooo003oooooool00?oooooo
o`00odSooooo00?o003oooooooooool06oooool001_ooooo0ol00005ooooo`03o`00oooooooooooo
00?ooooo00?o0000o`00ool00?l00oooool00ol00?ooooooooooo`0oooooo`03o`000?oooooooooo
00;o003o0_ooool00ol00?ooooooo`00o`02ooooo`03o`00oooooooooooo033ooooo00[o0000o`00
oooooooo003oooooool00?ooooooo`00oooooooo003oA_ooool3o`0001gooooo000Kooooo`?o0000
1?ooool00ol00?ooooooooooo`03ooooo`03o`000?l00?oo003o00;ooooo0_l00?m2ooooo`04o`00
0?ooooooooooooooo`;o003o0_ooool00ol00?ooooooo`00o`02ooooo`03o`00oooooooooooo02oo
oooo00[o0000o`00oooooooo003oooooool00?ooooooo`00oooooooo003oA_ooool3o`0001gooooo
000Kooooo`?o000000Co003ooooooooooooo003o1Oooool2o`00o`?ooooo00?o003oooooooooool0
@oooool01?l0003oooooooooooooool2o`00o`04ooooool00?ooooooo`00o`;ooooo00?o003ooooo
ooooool0;oooool01ol0003o003oooooool00?ooooooo`00ooooool00_l00?m7ooooo`?o00007Ooo
ool001kooooo00?o003oooooool00?l01Oooool2o`00o`?ooooo00?o003oooooooooool0AOooool0
0ol0003oooooooooo`02o`00o`;ooooo00?o003oooooool00?l00_ooool00ol00?ooooooooooo`0^
ooooo`07o`000?l00?ooooooo`00oooooooo003oooooo`02o`00odOooooo00?o003oooooooooool0
7Oooool001kooooo0ol00004ooooo`;o003o0oooool00ol00?ooooooooooo`17ooooo`03o`000?oo
oooooooo00;o003o0_ooool00ol00?ooooooo`00o`02ooooo`03o`00oooooooooooo02gooooo00Oo
0000o`00oooooooo003oooooool00?oooooo00;o003oAOooool3o`0001oooooo000Nooooo`?o0000
0oooool2o`00o`?ooooo00?o003oooooooooool0B?ooool02_l0003oooooooooool00?ooooooo`00
oooooooo003oooooool00?l2ooooo`03o`00oooooooooooo02gooooo00Oo0000o`00oooooooo003o
ooooool00?oooooo00;o003oAOooool3o`0001oooooo000Nooooo`?o00000_ooool00ol00?oooooo
ooooo`02ooooo`03o`00oooooooooooo04[ooooo00?o0000ooooooooool00_l00?l2ooooo`03o`00
oooooooo003o00;ooooo00?o003oooooooooool0;?ooool01ol0003o003oooooool00?ooooooo`00
ooooool00_l00?m5ooooo`?o00007oooool0023ooooo0ol00004ooooo`03o`00oooooooooooo04co
oooo00?o0000ooooooooool00_l00?l2ooooo`05o`00oooooooo003oooooool00?l0;_ooool01ol0
003o003oooooool00?ooooooo`00ooooool00_l00?m3ooooo`?o00008Oooool0023ooooo0ol00003
ooooo`03o`00oooooooooooo04kooooo00Wo0000ooooool00?ooooooo`00oooooooo003oooooool0
0?l00_ooool00ol00?ooooooooooo`0[ooooo`07o`000?l00?ooooooo`00oooooooo003oooooo`02
o`00od?ooooo0ol0000Qooooo`008?ooool3o`0000;ooooo00?o003oooooooooool0Coooool00ol0
003oooooooooo`02o`00o`;ooooo00Go003oooooool00?ooooooo`00o`0]ooooo`07o`000?l00?oo
ooooo`00oooooooo003oooooo`02o`00od?ooooo0ol0000Qooooo`008_ooool00ol00?ooooooo`00
o`1Cooooo`03o`000?oooooooooo00;o003o0_ooool01Ol00?ooooooo`00oooooooo003o02cooooo
00Oo0000o`00oooooooo003oooooool00?oooooo00;o003o@oooool00ol00?ooooooooooo`0Qoooo
o`008_ooool3o`0005Cooooo00_o0000ooooool00?ooooooo`00oooooooo003oooooool00?oooooo
o`00o`0/ooooo`07o`000?l00?ooooooo`00oooooooo003oooooo`02o`00od7ooooo0ol0000Soooo
o`008_ooool3o`0005Gooooo00Co0000ooooool00?oo003o0_ooool01Ol00?ooooooo`00oooooooo
003o02cooooo00Ko003oooooool00?ooooooo`00ooooool2o`00od7ooooo0ol0000Sooooo`008_oo
ool3o`0005Gooooo00Oo0000ooooool00?ooooooo`00oooooooo003o00;ooooo00?o003oooooool0
0?l0:oooool01ol00?ooooooo`00oooooooo003oooooool00?l0@_ooool3o`0002?ooooo000Toooo
o`?o0000E?ooool02ol0003oooooo`00oooooooo003oooooool00?ooooooo`00oooooooo003o02_o
oooo00Oo003oooooool00?ooooooo`00oooooooo003o03oooooo0ol000000ol00?ooooooooooo`0S
ooooo`009?ooool3o`0005Gooooo00Co0000ooooool00?oo003o0_ooool01Ol00?ooooooo`00oooo
oooo003o02[ooooo00Oo003oooooool00?ooooooo`00oooooooo003o03oooooo0ol0000Vooooo`00
9?ooool3o`000003o`00oooooooooooo05?ooooo00Ko0000o`00oooooooo003oooooool00?l2oooo
o`;o003o:_ooool01ol00?ooooooo`00oooooooo003oooooool00?l0?oooool3o`0002Kooooo000V
ooooo`04o`00ool0003o0000o`0005?ooooo00Oo0000ooooool00?ooooooo`00oooooooo003o00;o
oooo0_l00?lYooooo`04o`00oooooooo003oo`00o`;ooooo00?o003oooooooooool0>oooool3o`00
0003o`00oooooooooooo02Gooooo000Wooooo`?o0000E?ooool01?l0003oooooo`00ool00?l2oooo
o`04o`00oooooooo003oo`00obSooooo00?o003oo`000?ooool00_l00?l2ooooo`03o`00oooooooo
oooo03_ooooo0ol0000Xooooo`009oooool3o`0005Gooooo00Ko0000o`00oooooooo003oooooool0
0?l2ooooo`;o003o9oooool00ol00?oo0000ooooo`02o`00o`;ooooo00?o003oooooooooool0>ooo
ool3o`0002Sooooo000Yooooo`?o0000E?ooool01_l0003o003oooooool00?ooooooo`00o`;ooooo
0_l00?lVooooo`03o`00ool0003ooooo00;o003o0_ooool00ol00?ooooooooooo`0hooooo`?o0000
00?o003oooooooooool0:?ooool002Wooooo0ol0001Dooooo`08o`000?oooooo003oooooool00?oo
ooooo`00ooooool2o`00obKooooo00?o003oo`000?ooool00_l00?l2ooooo`03o`00oooooooooooo
03Sooooo0ol0000[ooooo`00:Oooool3o`0005Gooooo00Ko0000o`00oooooooo003oooooool00?l2
ooooo`;o003o9Oooool00ol00?oo0000ooooo`02o`00o`;ooooo00?o003oooooooooool0>?ooool3
o`0002_ooooo000/ooooo`?o0000Doooool01_l0003o003oooooool00?ooooooo`00o`;ooooo00?o
003oooooooooool08oooool00ol00?oo0000ooooo`02o`00o`;ooooo00?o003oooooooooool0=_oo
ool3o`0002gooooo000/ooooo`?o0000Doooool01?l0003o003oooooooooool2o`00o`;ooooo0_l0
0?lTooooo`03o`00ool0003ooooo00;o003o0_ooool00ol00?ooooooooooo`0fooooo`?o0000;Ooo
ool002cooooo0ol0001Dooooo`06o`000?l00?ooooooo`00oooooooo003o0_ooool00ol00?oooooo
ooooo`0Rooooo`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l0>?ooool3o`0002gooooo
000^ooooo`Co0000D_ooool01_l0003o003oooooool00?ooooooo`00o`;ooooo00?o003ooooooooo
ool08Oooool00ol00?oo0000ooooo`02o`00o`03ooooool00?oo003o03Gooooo0ol000000ol00?oo
ooooooooo`0]ooooo`00;oooool3o`0005?ooooo00?o003oooooooooool00_l00?l2ooooo`03o`00
oooooooooooo027ooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0eooooo`?o0000
<?ooool002oooooo0ol0001Cooooo`06o`000?l00?ooooooo`00oooooooo003o0_ooool00ol00?oo
ooooooooo`0Pooooo`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l0=Oooool3o`00033o
oooo000aooooo`04o`00ool0003o0000o`00057ooooo00Ko0000o`00oooooooo003oooooool00?l2
ooooo`03o`00oooooooooooo01oooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00o`0b
ooooo`?o000000?o003oooooooooool0<?ooool003;ooooo0ol0001Booooo`03o`00oooooooooooo
00;o003o0_ooool00ol00?ooooooooooo`0Oooooo`03o`00ool0003ooooo00;o003o00?oooooo`00
ool00?l0<_ooool3o`0003?ooooo000booooo`?o0000Doooool01Ol00?ooooooo`00oooooooo003o
00;ooooo00?o003oooooooooool07_ooool00ol00?oo0000ooooo`02o`00o`03ooooool00?oo003o
03;ooooo0ol0000cooooo`00=?ooool01?l00?oo0000o`000?l0001@ooooo`07o`000?l00?oooooo
o`00oooooooo003oooooo`02o`00oaoooooo00?o003oo`000?ooool00_l00?l00ooooooo003oo`00
o`0_ooooo`?o000000?o003oooooooooool0<oooool003Gooooo0ol0001Aooooo`03o`00oooooooo
oooo00;o003o0_ooool00ol00?ooooooooooo`0Mooooo`03o`00oooooooo000000;o003o00?ooooo
o`00ool00?l0;oooool3o`0003Kooooo000eooooo`?o0000D_ooool01_l00?ooooooo`00oooooooo
003oooooo`;o003o7_ooool00ol00?ooooooo`000002o`00o`03ooooool00?oo003o02oooooo0ol0
000fooooo`00=oooool4o`0004oooooo00?o003oo`000?ooool00_l00?l2ooooo`;o003o7Oooool0
0ol00?ooooooo`000002o`00o`03ooooool00?oo003o02cooooo0ol000000ol00?ooooooooooo`0f
ooooo`00>?ooool3o`00053ooooo00?o003oooooooooool00_l00?l00ooooooo003oo`00o`0Moooo
o`03o`00oooooooo000000?o003o00?oooooo`00ooooool0:oooool3o`0003Wooooo000hooooo`?o
000000?o003oooooooooool0C_ooool01_l00?ooooooo`00oooooooo003oooooo`;o003o7?ooool0
0ol00?ooooooo`000003o`00o`03ooooool00?oooooo02[ooooo00Co003oo`000?l0003o0000>Ooo
ool003_ooooo0ol0001>ooooo`03o`00ool0003ooooo00;o003o0_ooool2o`00oa_ooooo00?o003o
ooooool000000ol00?l00ooooooo003oooooo`0Xooooo`?o0000??ooool003_ooooo0ol0001?oooo
o`03o`00ool0003ooooo00;o003o00?oooooo`00ool00?l06oooool00ol00?ooooooo`000003o`00
o`03ooooool00?oooooo02Sooooo0ol0000looooo`00>oooool6o`0004gooooo00Ko003oooooool0
0?ooooooo`00ooooool2o`00oa[ooooo00?o003oooooool000000ol00?l00ooooooo003oooooo`0U
ooooo`Ko0000??ooool003kooooo0ol0001=ooooo`03o`00ool0003ooooo00;o003o00Coooooo`00
oooooooo003o6Oooool00ol00?ooooooo`000003o`00o`03ooooool00?oooooo02Gooooo0ol0000o
ooooo`00?_ooool3o`000003o`00oooooooooooo04_ooooo00?o003oo`000?ooool00_l00?l00ooo
oooo003oo`00o`0Iooooo`03o`00oooooooo000000?o003o00?oooooo`00ooooool09?ooool01?l0
0?oo0000o`000?l0000oooooo`00@Oooool3o`0004_ooooo00?o003oooooool000000_l00?l01?oo
oooo003oooooool00?lHooooo`03o`00oooooooo000000?o003o00?oooooo`00ooooool08Oooool3
o`000003o`00oooooooooooo043ooooo0011ooooo`?o000000?o003oooooooooool0BOooool00ol0
0?oo0000ooooo`02o`00o`04ooooool00?ooooooo`00oaOooooo00?o003oooooool000000ol00?l0
0ooooooo003oooooo`0Qooooo`?o0000@oooool0047ooooo1_l0001:ooooo`03o`00ool0003ooooo
00?o003o00?oooooo`00ooooool05_ooool00ol00?ooooooo`000003o`00o`03ooooool00?oooooo
01kooooo1_l00013ooooo`00A?ooool3o`000003o`00oooooooooooo04Oooooo00?o003oooooool0
00000_l00?l01?oooooo003oooooool00?lFooooo`03o`00oooooooo000000?o003o00?oooooo`00
ooooool07_ooool3o`0004Kooooo0014ooooo`?o000000Co003oo`000?l0003o0000Aoooool00ol0
0?ooooooo`000002o`00o`04ooooool00?ooooooo`00oaGooooo00?o003oooooool000000_l00?l2
ooooo`03o`00oooooooooooo01[ooooo1_l00016ooooo`00B?ooool3o`0004Sooooo00?o003oo`00
0?ooool00ol00?l00ooooooo003oooooo`0Dooooo`03o`00oooooooo000000;o003o0_ooool00ol0
0?ooooooooooo`0Jooooo`?o0000BOooool004Sooooo1_l00015ooooo`03o`00oooooooo000000;o
003o00Coooooo`00oooooooo003o4oooool01?l00?ooooooooooool00002o`00o`;ooooo00?o003o
ooooooooool05_ooool3o`000004o`00ool0003o0000o`0004Wooooo001;ooooo`?o000000?o003o
ooooooooool0@oooool00ol00?ooooooo`000003o`00o`;ooooo00?o003oooooooooool04?ooool0
1?l00?ooooooooooool00002o`00o`;ooooo00?o003oooooooooool05_ooool3o`0004gooooo001;
ooooo`?o00000ol00?m4ooooo`03o`00oooooooo000000?o003o0_ooool00ol00?ooooooooooo`0?
ooooo`04o`00ooooooooooooo`0000;o003o0_ooool00ol00?ooooooooooo`0Dooooo`;o003o0ol0
001=ooooo`00Coooool3o`0004?ooooo00Co003oooooool0003ooooo0_l00?l2ooooo`03o`00oooo
oooooooo00oooooo00Co003ooooooooooooo00000_l00?l2ooooo`03o`00oooooooooooo01?ooooo
0ol0001@ooooo`00Coooool3o`000004o`00ool0003o0000o`00043ooooo00?o003oooooool00000
0ol00?l2ooooo`03o`00oooooooooooo00kooooo00Co003ooooooooooooo00000_l00?l2ooooo`03
o`00oooooooooooo00oooooo0ol000001?l00?oo0000o`000?l0001@ooooo`00Coooool3o`000004
ooooool0003o0000o`00043ooooo00Co003ooooooooooooo00000_l00?l3ooooo`03o`00oooooooo
oooo00gooooo00Co003ooooooooooooooooo0_l00?l2ooooo`03o`00oooooooooooo00oooooo0ol0
00001?oooooo0000o`000?l0001@ooooo`00Doooool6o`0003kooooo00Co003ooooooooooooo0000
0_l00?l2ooooo`03o`00oooooooooooo00gooooo00Co003ooooooooooooooooo0_l00?l2ooooo`03
o`00oooooooooooo00_ooooo0ol000001?l00?oo0000o`000?l0001Dooooo`00E_ooool3o`0003oo
oooo00?o003oooooool000000ol00?l2ooooo`03o`00oooooooooooo00cooooo00Co003ooooooooo
oooooooo0_l00?l2ooooo`03o`00oooooooooooo00_ooooo0ol0001Hooooo`00E_ooool3o`000004
o`00ool0003o0000o`0003_ooooo00Co003ooooooooooooo00000_l00?l3ooooo`03o`00oooooooo
oooo00_ooooo00Co003ooooooooooooooooo0_l00?l2ooooo`03o`00oooooooooooo00Sooooo1_l0
001Hooooo`00F_ooool3o`000004o`00ool0003o0000o`0003Sooooo00Co003ooooooooooooo0000
0_l00?l2ooooo`03o`00oooooooooooo00_ooooo00Go003oooooooooooooooooo`00o`03ooooo`03
o`00oooooooooooo00Cooooo1ol0001Kooooo`00F_ooool7o`0003Wooooo00Go003ooooooooooooo
0000o`00o`03ooooo`03o`00oooooooooooo00[ooooo00Go003oooooooooooooooooo`00o`03oooo
o`03o`00oooooooooooo00Cooooo0ol000001?l00?oo0000o`000?l0001Kooooo`00G_ooool3o`00
0004o`00ool0003o0000o`0003Gooooo00Co003ooooooooooooo00000_l00?l3ooooo`03o`00oooo
oooooooo00Wooooo00Go003oooooooooooooooooo`00o`03ooooo`03o`00oooooooooooo00?o0000
00Co003oo`000?l0003o0000Goooool006;ooooo0ol000001?l00?oo0000o`000?l0000booooo`05
o`00ooooooooooooo`000?l00?l00oooool00ol00?ooooooooooo`09ooooo`05o`00oooooooooooo
ooooool00?l00_ooool3o`000004o`00ool0003o0000o`0006?ooooo001Rooooo`[o0000<?ooool0
1Ol00?ooooooooooool0003o003o00?ooooo00?o003oooooooooool02?ooool00ol00?oooooooooo
o`03o`000005o`00ool0003o0000o`000?l00?l00ol0001Sooooo`00I_ooool6o`0000;o003o;_oo
ool01Ol00?oooooooooooooooooo003o00Cooooo00?o003oooooooooool01oooool00ol00?oooooo
o`00o`03o`000004o`00ool0003o0000o`0006Oooooo001Yooooo`?o000000Goooooo`000?l0003o
0000o`00o`03o`0002Wooooo00Go003ooooooooooooo0000o`00o`03ooooo`03o`00oooooooooooo
00;ooooo0ol000001Ol00?oo0000o`000?l0003o003o00?o0000Joooool006gooooo0ol000001Ooo
oooo0000o`000?l0003o003o00?o00009Oooool01?l00?ooooooooooooooool2o`00o`?ooooo0ol0
00001Ol00?oo0000o`000?l0003o003o00?o0000Koooool006gooooo0ol000001Ooooooo0000o`00
0?l0003ooooo00?o000000Co003oo`000?l0003o00008_ooool01?l00?ooooooooooooooool3o`00
0005o`00ool0003o0000o`000?l00?l00ol000001?oooooo0000o`000?l0001_ooooo`00MOooool3
o`000005ooooool0003o0000o`000?l00?l00ol00002o`00o`?o000000Coooooo`000?l0003o0000
3Oooool3o`000005ooooool0003o0000o`000?l00?l00ol000001Ol00?oo0000o`000?l0003o003o
00?o0000Moooool007Wooooo0ol000001?oooooo0000o`000?l00002o`00o`?o000000Go003oo`00
0?l0003o0000o`00o`03o`000005o`00ool0003o0000o`000?l00?l00ol000001Ol00?oo0000o`00
0?l0003o003o00?o000000Go003oo`000?l0003o0000ooooo`03o`0007_ooooo001mooooo`?o0000
0_ooool3o`000005ooooool0003o0000o`000?ooool01ol000001Ol00?oo0000o`000?l0003ooooo
00?o000000Goooooo`000?l0003o0000ooooo`03o`0007oooooo002:ooooo`?o000000Goooooo`00
0?l0003o0000ooooo`03o`0008_ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Now let's zoom in on the vertices of interest. We can do this with ",
  StyleBox["ShowGraphArray ",
    FontWeight->"Bold"],
  "and standard functions for manipulating graphs from ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  ". "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[\[IndentingNewLine]{SetGraphOptions[
            gr, \[IndentingNewLine]PlotRange \[Rule] 
              Zoom[{105, 106, 
                  107}], \[IndentingNewLine]VertexNumberPosition \[Rule] \
{\(-0.05\), 0.0}\[IndentingNewLine]], 
          SetGraphOptions[
            gr, \[IndentingNewLine]PlotRange \[Rule] 
              Zoom[{149, 150, 
                  151}], \[IndentingNewLine]VertexNumberPosition \[Rule] \
{0.05, \(-0.075\)}\[IndentingNewLine]]}, \[IndentingNewLine]VertexStyle \
\[Rule] Disk[Large], \[IndentingNewLine]VertexNumber \[Rule] 
          True\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .47619 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0113379 0.47619 [
[ 0 0 0 0 ]
[ 1 .47619 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .47619 L
0 .47619 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0113379 0.477324 0.464853 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.41407 9.44414 -2.6054 7.64208 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
.42245 1 m
.42339 .97571 L
s
0 0 0 r
.41996 1 m
.42339 .97571 L
s
0 0 1 r
.42397 1 m
.42339 .97571 L
s
.42383 1 m
.43503 .85605 L
s
0 0 0 r
.42339 .97571 m
.42428 .91557 L
.43503 .85605 L
s
0 0 1 r
.42339 .97571 m
.43414 .91619 L
.43503 .85605 L
s
.42339 .97571 m
.45131 .73673 L
s
0 0 0 r
.43503 .85605 m
.43825 .79595 L
.45131 .73673 L
s
0 0 1 r
.43503 .85605 m
.44808 .79683 L
.45131 .73673 L
s
.43503 .85605 m
.47221 .6179 L
s
0 0 0 r
.45131 .73673 m
.45686 .67675 L
.47221 .6179 L
s
0 0 1 r
.45131 .73673 m
.46665 .67788 L
.47221 .6179 L
s
.45131 .73673 m
.49771 .49965 L
s
0 0 0 r
.47221 .6179 m
.48009 .55808 L
.49771 .49965 L
s
0 0 1 r
.47221 .6179 m
.48983 .55946 L
.49771 .49965 L
s
.47221 .6179 m
.52779 .3821 L
s
.47221 .6179 m
0 0 rlineto s
.47221 .6179 m
1 .25313 L
s
0 0 0 r
.49771 .49965 m
.50791 .44006 L
.52779 .3821 L
s
0 0 1 r
.49771 .49965 m
.51759 .44169 L
.52779 .3821 L
s
.49771 .49965 m
.56242 .26538 L
s
.49771 .49965 m
0 0 rlineto s
.49771 .49965 m
1 .15251 L
s
0 0 0 r
.49771 .49965 m
0 0 rlineto s
.49771 .49965 m
1 .1151 L
s
0 0 1 r
.49771 .49965 m
0 0 rlineto s
.49771 .49965 m
1 .20666 L
s
.49771 .49965 m
0 0 rlineto s
.49771 .49965 m
1 .17402 L
s
0 0 0 r
.52779 .3821 m
.5403 .32281 L
.56242 .26538 L
s
0 0 1 r
.52779 .3821 m
.54992 .32468 L
.56242 .26538 L
s
.52779 .3821 m
.60157 .1496 L
s
.52779 .3821 m
0 0 rlineto s
.52779 .3821 m
1 .07598 L
s
0 0 0 r
.56242 .26538 m
.57723 .20644 L
.60157 .1496 L
s
0 0 1 r
.56242 .26538 m
.58676 .20855 L
.60157 .1496 L
s
.56242 .26538 m
.64518 .03487 L
s
0 0 0 r
.60157 .1496 m
.61865 .09106 L
.64518 .03487 L
s
0 0 1 r
.60157 .1496 m
.6281 .09341 L
.64518 .03487 L
s
.60157 .1496 m
0 0 rlineto s
.60157 .1496 m
.66164 0 L
s
0 0 0 r
.64518 .03487 m
0 0 rlineto s
.64518 .03487 m
.6568 0 L
s
0 0 1 r
.64518 .03487 m
0 0 rlineto s
.64518 .03487 m
.66322 0 L
s
.64518 .03487 m
0 0 rlineto s
.64518 .03487 m
.6607 0 L
s
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
[(1)] 9.80588 1.33566 1 0 Mshowa
[(2)] 9.79889 1.45556 1 0 Mshowa
[(3)] 9.78725 1.57523 1 0 Mshowa
[(4)] 9.77097 1.69454 1 0 Mshowa
[(5)] 9.75007 1.81338 1 0 Mshowa
[(6)] 9.72457 1.93163 1 0 Mshowa
[(7)] 9.69449 2.04917 1 0 Mshowa
[(8)] 9.65986 2.16589 1 0 Mshowa
[(9)] 9.62071 2.28167 1 0 Mshowa
[(10)] 9.57709 2.39641 1 0 Mshowa
[(11)] 9.52904 2.50997 1 0 Mshowa
[(12)] 9.47661 2.62226 1 0 Mshowa
[(13)] 9.41984 2.73316 1 0 Mshowa
[(14)] 9.3588 2.84256 1 0 Mshowa
[(15)] 9.29353 2.95035 1 0 Mshowa
[(16)] 9.22412 3.05644 1 0 Mshowa
[(17)] 9.15062 3.16071 1 0 Mshowa
[(18)] 9.07311 3.26305 1 0 Mshowa
[(19)] 8.99167 3.36338 1 0 Mshowa
[(20)] 8.90637 3.46159 1 0 Mshowa
[(21)] 8.81731 3.55758 1 0 Mshowa
[(22)] 8.72456 3.65126 1 0 Mshowa
[(23)] 8.62822 3.74254 1 0 Mshowa
[(24)] 8.52838 3.83132 1 0 Mshowa
[(25)] 8.42515 3.91752 1 0 Mshowa
[(26)] 8.31862 4.00106 1 0 Mshowa
[(27)] 8.2089 4.08184 1 0 Mshowa
[(28)] 8.0961 4.1598 1 0 Mshowa
[(29)] 7.98033 4.23485 1 0 Mshowa
[(30)] 7.8617 4.30693 1 0 Mshowa
[(31)] 7.74034 4.37595 1 0 Mshowa
[(32)] 7.61635 4.44185 1 0 Mshowa
[(33)] 7.48987 4.50457 1 0 Mshowa
[(34)] 7.36101 4.56404 1 0 Mshowa
[(35)] 7.22991 4.62021 1 0 Mshowa
[(36)] 7.0967 4.67302 1 0 Mshowa
[(37)] 6.9615 4.72242 1 0 Mshowa
[(38)] 6.82445 4.76835 1 0 Mshowa
[(39)] 6.68568 4.81078 1 0 Mshowa
[(40)] 6.54534 4.84966 1 0 Mshowa
[(41)] 6.40355 4.88496 1 0 Mshowa
[(42)] 6.26047 4.91663 1 0 Mshowa
[(43)] 6.11623 4.94466 1 0 Mshowa
[(44)] 5.97097 4.969 1 0 Mshowa
[(45)] 5.82483 4.98964 1 0 Mshowa
[(46)] 5.67797 5.00655 1 0 Mshowa
[(47)] 5.53053 5.01972 1 0 Mshowa
[(48)] 5.38264 5.02914 1 0 Mshowa
[(49)] 5.23446 5.03479 1 0 Mshowa
[(50)] 5.08614 5.03668 1 0 Mshowa
[(51)] 4.93782 5.03479 1 0 Mshowa
[(52)] 4.78964 5.02914 1 0 Mshowa
[(53)] 4.64175 5.01972 1 0 Mshowa
[(54)] 4.49431 5.00655 1 0 Mshowa
[(55)] 4.34745 4.98964 1 0 Mshowa
[(56)] 4.20131 4.969 1 0 Mshowa
[(57)] 4.05605 4.94466 1 0 Mshowa
[(58)] 3.91181 4.91663 1 0 Mshowa
[(59)] 3.76872 4.88496 1 0 Mshowa
[(60)] 3.62694 4.84966 1 0 Mshowa
[(61)] 3.4866 4.81078 1 0 Mshowa
[(62)] 3.34783 4.76835 1 0 Mshowa
[(63)] 3.21078 4.72242 1 0 Mshowa
[(64)] 3.07558 4.67302 1 0 Mshowa
[(65)] 2.94237 4.62021 1 0 Mshowa
[(66)] 2.81127 4.56404 1 0 Mshowa
[(67)] 2.68241 4.50457 1 0 Mshowa
[(68)] 2.55593 4.44185 1 0 Mshowa
[(69)] 2.43194 4.37595 1 0 Mshowa
[(70)] 2.31058 4.30693 1 0 Mshowa
[(71)] 2.19195 4.23485 1 0 Mshowa
[(72)] 2.07618 4.1598 1 0 Mshowa
[(73)] 1.96338 4.08184 1 0 Mshowa
[(74)] 1.85366 4.00106 1 0 Mshowa
[(75)] 1.74713 3.91752 1 0 Mshowa
[(76)] 1.6439 3.83132 1 0 Mshowa
[(77)] 1.54406 3.74254 1 0 Mshowa
[(78)] 1.44772 3.65126 1 0 Mshowa
[(79)] 1.35497 3.55758 1 0 Mshowa
[(80)] 1.26591 3.46159 1 0 Mshowa
[(81)] 1.18061 3.36338 1 0 Mshowa
[(82)] 1.09916 3.26305 1 0 Mshowa
[(83)] 1.02166 3.16071 1 0 Mshowa
[(84)] .94816 3.05644 1 0 Mshowa
[(85)] .87875 2.95035 1 0 Mshowa
[(86)] .81348 2.84256 1 0 Mshowa
[(87)] .75244 2.73316 1 0 Mshowa
[(88)] .69567 2.62226 1 0 Mshowa
[(89)] .64324 2.50997 1 0 Mshowa
[(90)] .59518 2.39641 1 0 Mshowa
[(91)] .55157 2.28167 1 0 Mshowa
[(92)] .51242 2.16589 1 0 Mshowa
[(93)] .47779 2.04917 1 0 Mshowa
[(94)] .44771 1.93163 1 0 Mshowa
[(95)] .42221 1.81338 1 0 Mshowa
[(96)] .40131 1.69454 1 0 Mshowa
[(97)] .38503 1.57523 1 0 Mshowa
[(98)] .37339 1.45556 1 0 Mshowa
[(99)] .3664 1.33566 1 0 Mshowa
[(100)] .36407 1.21564 1 0 Mshowa
[(101)] .3664 1.09562 1 0 Mshowa
.07 w
.42339 .97571 Mdot
[(102)] .37339 .97571 1 0 Mshowa
.43503 .85605 Mdot
[(103)] .38503 .85605 1 0 Mshowa
.663 .663 .663 r
.45131 .73673 Mdot
0 0 0 r
[(104)] .40131 .73673 1 0 Mshowa
.663 .663 .663 r
.47221 .6179 Mdot
0 0 0 r
[(105)] .42221 .6179 1 0 Mshowa
.663 .663 .663 r
.49771 .49965 Mdot
0 0 0 r
[(106)] .44771 .49965 1 0 Mshowa
.663 .663 .663 r
.52779 .3821 Mdot
0 0 0 r
[(107)] .47779 .3821 1 0 Mshowa
.663 .663 .663 r
.56242 .26538 Mdot
0 0 0 r
[(108)] .51242 .26538 1 0 Mshowa
.60157 .1496 Mdot
[(109)] .55157 .1496 1 0 Mshowa
.64518 .03487 Mdot
[(110)] .59518 .03487 1 0 Mshowa
[(111)] .64324 -0.07869 1 0 Mshowa
[(112)] .69567 -0.19098 1 0 Mshowa
[(113)] .75244 -0.30188 1 0 Mshowa
[(114)] .81348 -0.41128 1 0 Mshowa
[(115)] .87875 -0.51908 1 0 Mshowa
[(116)] .94816 -0.62516 1 0 Mshowa
[(117)] 1.02166 -0.72943 1 0 Mshowa
[(118)] 1.09916 -0.83178 1 0 Mshowa
[(119)] 1.18061 -0.93211 1 0 Mshowa
[(120)] 1.26591 -1.03031 1 0 Mshowa
[(121)] 1.35497 -1.1263 1 0 Mshowa
[(122)] 1.44772 -1.21998 1 0 Mshowa
[(123)] 1.54406 -1.31126 1 0 Mshowa
[(124)] 1.6439 -1.40004 1 0 Mshowa
[(125)] 1.74713 -1.48625 1 0 Mshowa
[(126)] 1.85366 -1.56978 1 0 Mshowa
[(127)] 1.96338 -1.65057 1 0 Mshowa
[(128)] 2.07618 -1.72852 1 0 Mshowa
[(129)] 2.19195 -1.80358 1 0 Mshowa
[(130)] 2.31058 -1.87565 1 0 Mshowa
[(131)] 2.43194 -1.94467 1 0 Mshowa
[(132)] 2.55593 -2.01057 1 0 Mshowa
[(133)] 2.68241 -2.07329 1 0 Mshowa
[(134)] 2.81127 -2.13277 1 0 Mshowa
[(135)] 2.94237 -2.18893 1 0 Mshowa
[(136)] 3.07558 -2.24174 1 0 Mshowa
[(137)] 3.21078 -2.29114 1 0 Mshowa
[(138)] 3.34783 -2.33708 1 0 Mshowa
[(139)] 3.4866 -2.37951 1 0 Mshowa
[(140)] 3.62694 -2.41839 1 0 Mshowa
[(141)] 3.76872 -2.45368 1 0 Mshowa
[(142)] 3.91181 -2.48536 1 0 Mshowa
[(143)] 4.05605 -2.51338 1 0 Mshowa
[(144)] 4.20131 -2.53772 1 0 Mshowa
[(145)] 4.34745 -2.55836 1 0 Mshowa
[(146)] 4.49431 -2.57527 1 0 Mshowa
[(147)] 4.64175 -2.58844 1 0 Mshowa
[(148)] 4.78964 -2.59786 1 0 Mshowa
[(149)] 4.93782 -2.60352 1 0 Mshowa
[(150)] 5.08614 -2.6054 1 0 Mshowa
[(151)] 5.23446 -2.60352 1 0 Mshowa
[(152)] 5.38264 -2.59786 1 0 Mshowa
[(153)] 5.53053 -2.58844 1 0 Mshowa
[(154)] 5.67797 -2.57527 1 0 Mshowa
[(155)] 5.82483 -2.55836 1 0 Mshowa
[(156)] 5.97097 -2.53772 1 0 Mshowa
[(157)] 6.11623 -2.51338 1 0 Mshowa
[(158)] 6.26047 -2.48536 1 0 Mshowa
[(159)] 6.40355 -2.45368 1 0 Mshowa
[(160)] 6.54534 -2.41839 1 0 Mshowa
[(161)] 6.68568 -2.37951 1 0 Mshowa
[(162)] 6.82445 -2.33708 1 0 Mshowa
[(163)] 6.9615 -2.29114 1 0 Mshowa
[(164)] 7.0967 -2.24174 1 0 Mshowa
[(165)] 7.22991 -2.18893 1 0 Mshowa
[(166)] 7.36101 -2.13277 1 0 Mshowa
[(167)] 7.48987 -2.07329 1 0 Mshowa
[(168)] 7.61635 -2.01057 1 0 Mshowa
[(169)] 7.74034 -1.94467 1 0 Mshowa
[(170)] 7.8617 -1.87565 1 0 Mshowa
[(171)] 7.98033 -1.80358 1 0 Mshowa
[(172)] 8.0961 -1.72852 1 0 Mshowa
[(173)] 8.2089 -1.65057 1 0 Mshowa
[(174)] 8.31862 -1.56978 1 0 Mshowa
[(175)] 8.42515 -1.48625 1 0 Mshowa
[(176)] 8.52838 -1.40004 1 0 Mshowa
[(177)] 8.62822 -1.31126 1 0 Mshowa
[(178)] 8.72456 -1.21998 1 0 Mshowa
[(179)] 8.81731 -1.1263 1 0 Mshowa
[(180)] 8.90637 -1.03031 1 0 Mshowa
[(181)] 8.99167 -0.93211 1 0 Mshowa
[(182)] 9.07311 -0.83178 1 0 Mshowa
[(183)] 9.15062 -0.72943 1 0 Mshowa
[(184)] 9.22412 -0.62516 1 0 Mshowa
[(185)] 9.29353 -0.51908 1 0 Mshowa
[(186)] 9.3588 -0.41128 1 0 Mshowa
[(187)] 9.41984 -0.30188 1 0 Mshowa
[(188)] 9.47661 -0.19098 1 0 Mshowa
[(189)] 9.52904 -0.07869 1 0 Mshowa
[(190)] 9.57709 .03487 1 0 Mshowa
[(191)] 9.62071 .1496 1 0 Mshowa
[(192)] 9.65986 .26538 1 0 Mshowa
[(193)] 9.69449 .3821 1 0 Mshowa
[(194)] 9.72457 .49965 1 0 Mshowa
[(195)] 9.75007 .6179 1 0 Mshowa
[(196)] 9.77097 .73673 1 0 Mshowa
[(197)] 9.78725 .85605 1 0 Mshowa
[(198)] 9.79889 .97571 1 0 Mshowa
[(199)] 9.80588 1.09562 1 0 Mshowa
[(200)] 9.80821 1.21564 1 0 Mshowa
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.0113379 0.97619 0.464853 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-3.30486 7.60973 0.498769 9.97539 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
.05231 1 m
.5 .49877 L
s
0 0 0 r
0 0 1 r
0 .92722 m
.38049 .50123 L
s
0 0 0 r
0 .97124 m
.5 .49877 L
s
0 0 1 r
.09586 1 m
.5 .49877 L
s
.14459 1 m
.61951 .50123 L
s
0 0 0 r
0 0 1 r
.02273 1 m
.5 .49877 L
s
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 .5429 m
.02312 .5381 L
s
0 0 0 r
0 .54037 m
.02312 .5381 L
s
0 0 1 r
0 .54449 m
.02312 .5381 L
s
0 .54441 m
.14193 .52091 L
s
0 0 0 r
.02312 .5381 m
.08209 .52431 L
.14193 .52091 L
s
0 0 1 r
.02312 .5381 m
.08296 .53469 L
.14193 .52091 L
s
.02312 .5381 m
.26109 .50861 L
s
0 0 0 r
.14193 .52091 m
.2012 .50955 L
.26109 .50861 L
s
0 0 1 r
.14193 .52091 m
.20182 .51997 L
.26109 .50861 L
s
.14193 .52091 m
.38049 .50123 L
s
0 0 0 r
.26109 .50861 m
.3206 .4997 L
.38049 .50123 L
s
0 0 1 r
.26109 .50861 m
.32098 .51014 L
.38049 .50123 L
s
.26109 .50861 m
.5 .49877 L
s
0 0 0 r
.38049 .50123 m
.44018 .49478 L
.5 .49877 L
s
0 0 1 r
.38049 .50123 m
.44031 .50522 L
.5 .49877 L
s
.38049 .50123 m
.61951 .50123 L
s
0 0 0 r
.5 .49877 m
.55969 .50522 L
.61951 .50123 L
s
0 0 1 r
.5 .49877 m
.55982 .49478 L
.61951 .50123 L
s
.5 .49877 m
.73891 .50861 L
s
0 0 0 r
.61951 .50123 m
.67902 .51014 L
.73891 .50861 L
s
0 0 1 r
.61951 .50123 m
.6794 .4997 L
.73891 .50861 L
s
.61951 .50123 m
.85807 .52091 L
s
0 0 0 r
.73891 .50861 m
.79818 .51997 L
.85807 .52091 L
s
0 0 1 r
.73891 .50861 m
.7988 .50955 L
.85807 .52091 L
s
.73891 .50861 m
.97688 .5381 L
s
0 0 0 r
.85807 .52091 m
.91704 .53469 L
.97688 .5381 L
s
0 0 1 r
.85807 .52091 m
.91791 .52431 L
.97688 .5381 L
s
.85807 .52091 m
0 0 rlineto s
.85807 .52091 m
1 .54441 L
s
0 0 0 r
.97688 .5381 m
0 0 rlineto s
.97688 .5381 m
1 .54449 L
s
0 0 1 r
.97688 .5381 m
0 0 rlineto s
.97688 .5381 m
1 .54037 L
s
.97688 .5381 m
0 0 rlineto s
.97688 .5381 m
1 .5429 L
s
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
0 0 1 r
0 0 0 r
[(1)] 4.35299 5.56813 1 0 Mshowa
[(2)] 4.34736 5.72464 1 0 Mshowa
[(3)] 4.33798 5.88085 1 0 Mshowa
[(4)] 4.32486 6.03659 1 0 Mshowa
[(5)] 4.30802 6.19171 1 0 Mshowa
[(6)] 4.28747 6.34606 1 0 Mshowa
[(7)] 4.26323 6.49949 1 0 Mshowa
[(8)] 4.23533 6.65185 1 0 Mshowa
[(9)] 4.20379 6.80299 1 0 Mshowa
[(10)] 4.16864 6.95275 1 0 Mshowa
[(11)] 4.12992 7.10098 1 0 Mshowa
[(12)] 4.08767 7.24756 1 0 Mshowa
[(13)] 4.04193 7.39232 1 0 Mshowa
[(14)] 3.99274 7.53512 1 0 Mshowa
[(15)] 3.94016 7.67583 1 0 Mshowa
[(16)] 3.88423 7.8143 1 0 Mshowa
[(17)] 3.82501 7.95041 1 0 Mshowa
[(18)] 3.76255 8.084 1 0 Mshowa
[(19)] 3.69693 8.21496 1 0 Mshowa
[(20)] 3.6282 8.34316 1 0 Mshowa
[(21)] 3.55643 8.46846 1 0 Mshowa
[(22)] 3.4817 8.59074 1 0 Mshowa
[(23)] 3.40407 8.70988 1 0 Mshowa
[(24)] 3.32363 8.82577 1 0 Mshowa
[(25)] 3.24044 8.9383 1 0 Mshowa
[(26)] 3.15461 9.04734 1 0 Mshowa
[(27)] 3.0662 9.15279 1 0 Mshowa
[(28)] 2.97531 9.25455 1 0 Mshowa
[(29)] 2.88203 9.35251 1 0 Mshowa
[(30)] 2.78644 9.44659 1 0 Mshowa
[(31)] 2.68865 9.53669 1 0 Mshowa
[(32)] 2.58875 9.62271 1 0 Mshowa
[(33)] 2.48683 9.70458 1 0 Mshowa
[(34)] 2.38301 9.78221 1 0 Mshowa
[(35)] 2.27737 9.85553 1 0 Mshowa
[(36)] 2.17003 9.92446 1 0 Mshowa
[(37)] 2.06109 9.98894 1 0 Mshowa
[(38)] 1.95066 10.0489 1 0 Mshowa
[(39)] 1.83885 10.1043 1 0 Mshowa
[(40)] 1.72577 10.155 1 0 Mshowa
[(41)] 1.61152 10.2011 1 0 Mshowa
[(42)] 1.49623 10.2425 1 0 Mshowa
[(43)] 1.38001 10.279 1 0 Mshowa
[(44)] 1.26296 10.3108 1 0 Mshowa
[(45)] 1.14521 10.3377 1 0 Mshowa
[(46)] 1.02688 10.3598 1 0 Mshowa
[(47)] .90807 10.377 1 0 Mshowa
[(48)] .78891 10.3893 1 0 Mshowa
[(49)] .66951 10.3967 1 0 Mshowa
[(50)] .55 10.3992 1 0 Mshowa
[(51)] .43049 10.3967 1 0 Mshowa
[(52)] .31109 10.3893 1 0 Mshowa
[(53)] .19193 10.377 1 0 Mshowa
[(54)] .07312 10.3598 1 0 Mshowa
[(55)] -0.04521 10.3377 1 0 Mshowa
[(56)] -0.16296 10.3108 1 0 Mshowa
[(57)] -0.28001 10.279 1 0 Mshowa
[(58)] -0.39623 10.2425 1 0 Mshowa
[(59)] -0.51152 10.2011 1 0 Mshowa
[(60)] -0.62577 10.155 1 0 Mshowa
[(61)] -0.73885 10.1043 1 0 Mshowa
[(62)] -0.85066 10.0489 1 0 Mshowa
[(63)] -0.96109 9.98894 1 0 Mshowa
[(64)] -1.07003 9.92446 1 0 Mshowa
[(65)] -1.17737 9.85553 1 0 Mshowa
[(66)] -1.28301 9.78221 1 0 Mshowa
[(67)] -1.38683 9.70458 1 0 Mshowa
[(68)] -1.48875 9.62271 1 0 Mshowa
[(69)] -1.58865 9.53669 1 0 Mshowa
[(70)] -1.68644 9.44659 1 0 Mshowa
[(71)] -1.78203 9.35251 1 0 Mshowa
[(72)] -1.87531 9.25455 1 0 Mshowa
[(73)] -1.9662 9.15279 1 0 Mshowa
[(74)] -2.05461 9.04734 1 0 Mshowa
[(75)] -2.14044 8.9383 1 0 Mshowa
[(76)] -2.22363 8.82577 1 0 Mshowa
[(77)] -2.30407 8.70988 1 0 Mshowa
[(78)] -2.3817 8.59074 1 0 Mshowa
[(79)] -2.45643 8.46846 1 0 Mshowa
[(80)] -2.5282 8.34316 1 0 Mshowa
[(81)] -2.59693 8.21496 1 0 Mshowa
[(82)] -2.66255 8.084 1 0 Mshowa
[(83)] -2.72501 7.95041 1 0 Mshowa
[(84)] -2.78423 7.8143 1 0 Mshowa
[(85)] -2.84016 7.67583 1 0 Mshowa
[(86)] -2.89274 7.53512 1 0 Mshowa
[(87)] -2.94193 7.39232 1 0 Mshowa
[(88)] -2.98767 7.24756 1 0 Mshowa
[(89)] -3.02992 7.10098 1 0 Mshowa
[(90)] -3.06864 6.95275 1 0 Mshowa
[(91)] -3.10379 6.80299 1 0 Mshowa
[(92)] -3.13533 6.65185 1 0 Mshowa
[(93)] -3.16323 6.49949 1 0 Mshowa
[(94)] -3.18747 6.34606 1 0 Mshowa
[(95)] -3.20802 6.19171 1 0 Mshowa
[(96)] -3.22486 6.03659 1 0 Mshowa
[(97)] -3.23798 5.88085 1 0 Mshowa
[(98)] -3.24736 5.72464 1 0 Mshowa
[(99)] -3.25299 5.56813 1 0 Mshowa
[(100)] -3.25486 5.41146 1 0 Mshowa
[(101)] -3.25299 5.2548 1 0 Mshowa
[(102)] -3.24736 5.09828 1 0 Mshowa
[(103)] -3.23798 4.94208 1 0 Mshowa
[(104)] -3.22486 4.78634 1 0 Mshowa
[(105)] -3.20802 4.63122 1 0 Mshowa
[(106)] -3.18747 4.47686 1 0 Mshowa
[(107)] -3.16323 4.32343 1 0 Mshowa
[(108)] -3.13533 4.17107 1 0 Mshowa
[(109)] -3.10379 4.01994 1 0 Mshowa
[(110)] -3.06864 3.87018 1 0 Mshowa
[(111)] -3.02992 3.72194 1 0 Mshowa
[(112)] -2.98767 3.57537 1 0 Mshowa
[(113)] -2.94193 3.43061 1 0 Mshowa
[(114)] -2.89274 3.28781 1 0 Mshowa
[(115)] -2.84016 3.1471 1 0 Mshowa
[(116)] -2.78423 3.00862 1 0 Mshowa
[(117)] -2.72501 2.87252 1 0 Mshowa
[(118)] -2.66255 2.73892 1 0 Mshowa
[(119)] -2.59693 2.60796 1 0 Mshowa
[(120)] -2.5282 2.47977 1 0 Mshowa
[(121)] -2.45643 2.35447 1 0 Mshowa
[(122)] -2.3817 2.23219 1 0 Mshowa
[(123)] -2.30407 2.11304 1 0 Mshowa
[(124)] -2.22363 1.99715 1 0 Mshowa
[(125)] -2.14044 1.88463 1 0 Mshowa
[(126)] -2.05461 1.77559 1 0 Mshowa
[(127)] -1.9662 1.67014 1 0 Mshowa
[(128)] -1.87531 1.56838 1 0 Mshowa
[(129)] -1.78203 1.47041 1 0 Mshowa
[(130)] -1.68644 1.37633 1 0 Mshowa
[(131)] -1.58865 1.28624 1 0 Mshowa
[(132)] -1.48875 1.20021 1 0 Mshowa
[(133)] -1.38683 1.11835 1 0 Mshowa
[(134)] -1.28301 1.04071 1 0 Mshowa
[(135)] -1.17737 .9674 1 0 Mshowa
[(136)] -1.07003 .89846 1 0 Mshowa
[(137)] -0.96109 .83398 1 0 Mshowa
[(138)] -0.85066 .77402 1 0 Mshowa
[(139)] -0.73885 .71864 1 0 Mshowa
[(140)] -0.62577 .66788 1 0 Mshowa
[(141)] -0.51152 .62181 1 0 Mshowa
[(142)] -0.39623 .58047 1 0 Mshowa
[(143)] -0.28001 .54389 1 0 Mshowa
[(144)] -0.16296 .51212 1 0 Mshowa
[(145)] -0.04521 .48518 1 0 Mshowa
.07 w
.02312 .5381 Mdot
[(146)] .07312 .4631 1 0 Mshowa
.14193 .52091 Mdot
[(147)] .19193 .44591 1 0 Mshowa
.663 .663 .663 r
.26109 .50861 Mdot
0 0 0 r
[(148)] .31109 .43361 1 0 Mshowa
.663 .663 .663 r
.38049 .50123 Mdot
0 0 0 r
[(149)] .43049 .42623 1 0 Mshowa
.663 .663 .663 r
.5 .49877 Mdot
0 0 0 r
[(150)] .55 .42377 1 0 Mshowa
.663 .663 .663 r
.61951 .50123 Mdot
0 0 0 r
[(151)] .66951 .42623 1 0 Mshowa
.663 .663 .663 r
.73891 .50861 Mdot
0 0 0 r
[(152)] .78891 .43361 1 0 Mshowa
.85807 .52091 Mdot
[(153)] .90807 .44591 1 0 Mshowa
.97688 .5381 Mdot
[(154)] 1.02688 .4631 1 0 Mshowa
[(155)] 1.14521 .48518 1 0 Mshowa
[(156)] 1.26296 .51212 1 0 Mshowa
[(157)] 1.38001 .54389 1 0 Mshowa
[(158)] 1.49623 .58047 1 0 Mshowa
[(159)] 1.61152 .62181 1 0 Mshowa
[(160)] 1.72577 .66788 1 0 Mshowa
[(161)] 1.83885 .71864 1 0 Mshowa
[(162)] 1.95066 .77402 1 0 Mshowa
[(163)] 2.06109 .83398 1 0 Mshowa
[(164)] 2.17003 .89846 1 0 Mshowa
[(165)] 2.27737 .9674 1 0 Mshowa
[(166)] 2.38301 1.04071 1 0 Mshowa
[(167)] 2.48683 1.11835 1 0 Mshowa
[(168)] 2.58875 1.20021 1 0 Mshowa
[(169)] 2.68865 1.28624 1 0 Mshowa
[(170)] 2.78644 1.37633 1 0 Mshowa
[(171)] 2.88203 1.47041 1 0 Mshowa
[(172)] 2.97531 1.56838 1 0 Mshowa
[(173)] 3.0662 1.67014 1 0 Mshowa
[(174)] 3.15461 1.77559 1 0 Mshowa
[(175)] 3.24044 1.88463 1 0 Mshowa
[(176)] 3.32363 1.99715 1 0 Mshowa
[(177)] 3.40407 2.11304 1 0 Mshowa
[(178)] 3.4817 2.23219 1 0 Mshowa
[(179)] 3.55643 2.35447 1 0 Mshowa
[(180)] 3.6282 2.47977 1 0 Mshowa
[(181)] 3.69693 2.60796 1 0 Mshowa
[(182)] 3.76255 2.73892 1 0 Mshowa
[(183)] 3.82501 2.87252 1 0 Mshowa
[(184)] 3.88423 3.00862 1 0 Mshowa
[(185)] 3.94016 3.1471 1 0 Mshowa
[(186)] 3.99274 3.28781 1 0 Mshowa
[(187)] 4.04193 3.43061 1 0 Mshowa
[(188)] 4.08767 3.57537 1 0 Mshowa
[(189)] 4.12992 3.72194 1 0 Mshowa
[(190)] 4.16864 3.87018 1 0 Mshowa
[(191)] 4.20379 4.01994 1 0 Mshowa
[(192)] 4.23533 4.17107 1 0 Mshowa
[(193)] 4.26323 4.32343 1 0 Mshowa
[(194)] 4.28747 4.47686 1 0 Mshowa
[(195)] 4.30802 4.63122 1 0 Mshowa
[(196)] 4.32486 4.78634 1 0 Mshowa
[(197)] 4.33798 4.94208 1 0 Mshowa
[(198)] 4.34736 5.09828 1 0 Mshowa
[(199)] 4.35299 5.2548 1 0 Mshowa
[(200)] 4.35486 5.41146 1 0 Mshowa
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[20]:=",
  ImageSize->{439.812, 209.375},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006g0000dB000`400?l00000o`00003ooooooonh
ooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00SOooool0
0ol0003o003oooooo`3oooooobSooooo0028ooooo`Ko000000?o003oooooooooool0oooooolWoooo
o`00Q_ooool9o`000?oooooo:Oooool008Gooooo2ol0003oooooobSooooo0025ooooo`_o0000oooo
oolXooooo`00L?ooool3o`0000?ooooo0ol00004ooooo`;o00001Oooool=o`000?oooooo9oooool0
077ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
o`0000Cooooo3Ol0003oooooobOooooo001aooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooool00004ooooo`go0000oooooolWooooo`00LOooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001?ooool=
o`000?oooooo9oooool0077ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000Cooooo3Ol0003oooooobOooooo001`ooooo`;o00001?ooool2o`00
00Cooooo00Co0000oooooooooooo00001Oooool;o`000?oooooo:?ooool0077ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`03ooooo`;o00001_ooool;o`000?oooooo:?ooool008Ko
oooo2Ol0003oooooobWooooo0028ooooo`Go0000ooooool[ooooo`00Qoooool00ol0003oooooo`00
o`3oooooobkooooo0027ooooo`03o`000?l00?oooooo04Kooooo0_l00?oUooooo`00Q_ooool00ol0
003oooooo`00o`16ooooo`03o`00oooooooooooo0>Gooooo0026ooooo`03o`000?l00?oo003o04Co
oooo0_l00?oXooooo`00QOooool00ol0003oooooo`00o`14ooooo`03o`00oooooooooooo0>Sooooo
0025ooooo`03o`000?oooooo003o04;ooooo0_l00?o[ooooo`00QOooool00ol0003o003oo`00o`11
ooooo`03o`00oooooooooooo0>_ooooo0024ooooo`03o`000?oooooo003o043ooooo0_l00?o^oooo
o`00Q?ooool00ol0003o003oo`00o`0nooooo`;o003ol?ooool008Cooooo00?o0000o`00ooooool0
?Oooool00ol00?ooooooooooo`09ooooo`;o0000iOooool007oooooo1_l000000ol00?oooooooooo
o`0jooooo`;o003o2oooool00ol0003oooooooooo`3Uooooo`00OOooool9o`0003_ooooo00?o003o
ooooooooool02_ooool00ol0003oooooooooo`3Vooooo`00O?ooool;o`0003Sooooo0_l00?l;oooo
o`;o0000jOooool006Oooooo0ol00004ooooo`;o00000oooool2o`0000Oooooo2ol0000gooooo`03
o`00oooooooooooo00[ooooo00?o0000ooooooooool0jOooool006Sooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00?ooooo3Ol0000dooooo`;o
003o3?ooool00ol0003oooooooooo`3Zooooo`00J?ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000Gooooo00?o0000ooooooooool00_ooool=o`0003?ooooo00?o003ooooooooo
ool02_ooool2o`000>gooooo001Xooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00000_ooool4o`0000Cooooo3Ol0000aooooo`;o003o3?ooool00ol0003oooooooooo`06ooooo`;o
003oiOooool006Sooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003oooooo`00
00?ooooo00?o0000ooooooooool00_ooool=o`00033ooooo00?o003oooooooooool02oooool00ol0
003oooooooooo`06ooooo`03o`00oooooooooooo0>Gooooo001Wooooo`;o00001?ooool01_l0003o
ooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool=o`0002kooooo0_l00?l<
ooooo`;o00001oooool2o`00onSooooo001Xooooo`03o`000?oooooooooo00?ooooo0_l00003oooo
o`?o00001_ooool;o`0002gooooo0_l00?l=ooooo`03o`000?oooooooooo00Kooooo00?o003ooooo
ooooool0j?ooool007cooooo2ol0000/ooooo`03o`00oooooooooooo00cooooo00?o0000oooooooo
ool01Oooool2o`00on_ooooo001mooooo`Wo0000:oooool2o`00o`kooooo00?o0000ooooooooool0
1Oooool00ol00?ooooooooooo`04ooooo`;o003oiOooool007oooooo1Ol0000/ooooo`03o`00oooo
oooooooo00cooooo0_l00007ooooo`03o`00oooooooooooo00Cooooo00?o003oooooooooool0iOoo
ool007oooooo00?o0000o`00ooooool0;?ooool2o`00o`kooooo00?o0000ooooooooool01Oooool2
o`00o`Gooooo0_l00?oXooooo`00Ooooool00ol0003o003oooooo`0[ooooo`03o`00oooooooooooo
00gooooo00?o0000ooooooooool01Oooool00ol00?ooooooooooo`04ooooo`03o`00oooooooooooo
0>Sooooo001nooooo`03o`000?l00?oo003o02[ooooo0_l00?l>ooooo`;o00001_ooool2o`00o`Go
oooo0_l00?o[ooooo`00O_ooool00ol0003o003oo`00o`0Yooooo`03o`00oooooooooooo00gooooo
00?o0000ooooooooool01Oooool00ol00?ooooooooooo`03ooooo`;o003okOooool007gooooo00?o
0000ooooool00?l0:?ooool2o`00o`oooooo00?o0000ooooooooool01?ooool2o`00o`Gooooo00?o
003oooooooooool01_ooool2o`00onGooooo001mooooo`03o`000?l00?oo003o02Oooooo00?o003o
ooooooooool03Oooool2o`0000Kooooo00?o003oooooooooool00oooool2o`00o`Oooooo0_l00?oW
ooooo`00OOooool00ol0003o003oo`00o`0Uooooo`;o003o3oooool00ol0003oooooooooo`04oooo
o`;o003o1Oooool00ol00?ooooooooooo`06ooooo`03o`00oooooooooooo0>Oooooo001looooo`03
o`000?oooooo003o02Gooooo00?o003oooooooooool03_ooool00ol0003oooooooooo`04ooooo`03
o`00oooooooooooo00?ooooo0_l00?l7ooooo`;o003oj_ooool007cooooo00?o0000o`00ool00?l0
8oooool2o`00o`oooooo0_l00006ooooo`03o`00oooooooooooo00?ooooo00?o003oooooooooool0
1Oooool2o`00oncooooo001looooo`03o`000?l00?oooooo027ooooo0_l00?l@ooooo`03o`000?oo
oooooooo00Cooooo0_l00?l4ooooo`;o003o1oooool00ol00?ooooooooooo`3/ooooo`00N?ooool5
ojVYZ@03o`00oooooooooooo01oooooo00?o003oooooooooool03oooool00ol0003oooooooooo`04
ooooo`03o`00oooooooooooo00;ooooo0_l00?l7ooooo`;o003okoooool007Kooooo2OnYZJTNoooo
o`;o003o4?ooool2o`0000Gooooo0_l00?l4ooooo`03o`00oooooooooooo00Gooooo0_l00?oaoooo
o`00MOooool;ojVYZAcooooo00?o003oooooooooool03oooool00ol0003oooooooooo`04ooooo`03
o`00oooooooooooo00;ooooo0_l00?l6ooooo`;o003oloooool005oooooo0ol00004ooooo`;o0000
0oooool3o`0000Oooooo2onYZJTJooooo`;o003o4Oooool00ol0003oooooooooo`03ooooo`;o003o
1?ooool00ol00?ooooooooooo`05ooooo`03o`00oooooooooooo00cooooo0_l00?oUooooo`00H?oo
ool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003o
ooooooooo`03ooooo`goZJVY6?ooool00ol00?ooooooooooo`0?ooooo`;o00001Oooool00ol00?oo
ooooooooo`02ooooo`;o003o1_ooool2o`00o`kooooo00?o003oooooooooool0iOooool0063ooooo
00?o0000ooooooooool00_ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000oooo
ooooool00oooool=ojVYZAKooooo0_l00?lAooooo`03o`000?oooooooooo00?ooooo0_l00?l4oooo
o`03o`00oooooooooooo00Cooooo0_l00?l>ooooo`;o003oj?ooool0063ooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooool00002ooooo`?o00001_ooool=ojVYZAGooooo00?o003ooooo
ooooool04?ooool00ol0003oooooooooo`03ooooo`03o`00oooooooooooo00;ooooo0_l00?l5oooo
o`;o003o3oooool00ol00?ooooooooooo`3Xooooo`00H?ooool00ol0003oooooooooo`02ooooo`06
o`000?ooooooooooo`000?oooooo00000oooool00ol0003oooooooooo`03ooooo`goZJVY4oooool2
o`00oa;ooooo00?o0000ooooooooool00_ooool2o`00o`Cooooo00?o003oooooooooool01?ooool0
0ol00?ooooooooooo`0>ooooo`03o`00oooooooooooo0>Wooooo001Oooooo`;o00001?ooool01_l0
003oooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool00oooool=ojVYZA;ooooo00?o
003oooooooooool04?ooool2o`0000Cooooo00?o003oooooooooool00_ooool2o`00o`Gooooo0_l0
0?l?ooooo`;o003ok?ooool0063ooooo00?o0000ooooooooool00oooool2o`0000?ooooo0ol00007
ooooo`_oZJVY4Oooool2o`00oa;ooooo00?o0000ooooooooool00oooool01?l00?oooooooooooooo
ool2o`00o`Gooooo0_l00?l@ooooo`03o`00oooooooooooo0>cooooo001eooooo`_oZJVY4?ooool0
0ol00?ooooooooooo`0Aooooo`03o`000?oooooooooo00;ooooo0_l00?l3ooooo`03o`00oooooooo
oooo00Cooooo00?o003oooooooooool03_ooool2o`00onoooooo001fooooo`WoZJVY3oooool2o`00
oa;ooooo0_l00004ooooo`04o`00ooooooooooooooooo`;o003o1Oooool2o`00oa3ooooo00?o003o
ooooooooool0koooool007Sooooo1OnYZJT?ooooo`;o003o4oooool00ol0003oooooooooo`02oooo
o`;o003o0oooool00ol00?ooooooooooo`03ooooo`;o003o4?ooool2o`00oo;ooooo001hooooo`;o
003o4Oooool00ol00?ooooooooooo`0Booooo`03o`000?oooooooooo00;ooooo00Co003ooooooooo
oooooooo0_l00?l4ooooo`;o003o4Oooool00ol00?ooooooooooo`3booooo`00Moooool00ol0003o
003oo`00o`0?ooooo`;o003o4oooool2o`0000?ooooo0_l00?l3ooooo`03o`00oooooooooooo00?o
oooo00?o003oooooooooool03oooool2o`00ooGooooo001gooooo`03o`000?l00?oo003o00kooooo
00?o003oooooooooool04_ooool00ol0003oooooooooo`02ooooo`04o`00ooooooooooooooooo`;o
003o1?ooool2o`00oa7ooooo00?o003oooooooooool0mOooool007Oooooo0_l00?l=ooooo`;o003o
5?ooool01?l0003oooooooooooooool2o`00o`;ooooo0_l00?l4ooooo`;o003o4_ooool00ol00?oo
ooooooooo`3fooooo`00M_ooool00ol0003o003oo`00o`0<ooooo`03o`00oooooooooooo01;ooooo
0_l00003ooooo`05o`00ooooooooooooooooool00?l01?ooool2o`00oa;ooooo0_l00?oiooooo`00
M_ooool00ol0003o003oo`00o`0:ooooo`;o003o5?ooool00ol0003oooooooooo`02ooooo`03o`00
oooooooooooo00;o003o1?ooool00ol00?ooooooooooo`0Aooooo`03o`00oooooooooooo0?Wooooo
001fooooo`03o`00oooooooo003o00Wooooo00?o003oooooooooool04oooool01?l0003ooooooooo
ooooool2o`00o`;ooooo00?o003oooooooooool00_ooool2o`00oa;ooooo0_l00?olooooo`00MOoo
ool00ol0003o003oo`00o`08ooooo`;o003o5?ooool2o`0000?ooooo00?o003oooooooooool00_l0
0?l3ooooo`;o003o4oooool00ol00?ooooooooooo`3looooo`00MOooool00ol0003o003oo`00o`07
ooooo`03o`00oooooooooooo01?ooooo00Co0000oooooooooooooooo0_l00?l2ooooo`03o`00oooo
oooooooo00;ooooo00?o003oooooooooool04Oooool2o`00oooooooo001eooooo`;o003o1_ooool2
o`00oaGooooo00Go0000ooooooooooooooooo`00o`02ooooo`;o003o0oooool2o`00oa?ooooo00?o
003oooooooooool0ooooool0077ooooo1OnYZJT00ol00?ooooooooooo`02ooooo`;o003o5Oooool2
o`0000;ooooo0_l00?l2ooooo`04o`00ooooooooooooooooo`;o003o5?ooool00ol00?oooooooooo
o`3oooooo`7ooooo001_ooooo`WoZJVY0_ooool00ol00?ooooooooooo`0Dooooo`05o`000?oooooo
ooooooooool00?l00_ooool2o`00o`;ooooo0_l00?lDooooo`;o003oooooool4ooooo`00K_ooool;
ojVYZ@03o`00oooooooooooo01Cooooo00?o0000ooooooooool00_l00?l00ooooooo003oo`00o`03
ooooo`03o`00oooooooooooo01?ooooo00?o003oooooooooool0ooooool4ooooo`00F?ooool3o`00
00Cooooo0_l00004ooooo`03o`000?oooooooooo00Kooooo2onYZJTFooooo`04o`000?oooooooooo
o`00o`;ooooo00Co003ooooooooooooooooo0_l00?lDooooo`;o003oooooool7ooooo`00FOooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool01Ooo
ool=ojVYZA?ooooo0_l00002ooooo`04o`00oooooooo003oo`00o`;ooooo0_l00?lEooooo`03o`00
oooooooooooo0?oooooo1oooool005Wooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool00004ooooo`03o`000?oooooooooo00Cooooo3OnYZJTBooooo`03o`000?oooooooooo00;o003o
00?oooooo`00ooooool00_ooool00ol00?ooooooooooo`0Cooooo`;o003oooooool:ooooo`00FOoo
ool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool0
1?ooool=ojVYZA7ooooo00Go0000oooooooooooo003oooooo`02o`00o`;ooooo0_l00?lEooooo`03
o`00oooooooooooo0?oooooo2_ooool005Wooooo00?o0000ooooooooool00_ooool01?l0003ooooo
ooooool00005ooooo`03o`000?oooooooooo00?ooooo3OnYZJT?ooooo`;o000000Goooooo`00ool0
0?ooooooo`00o`02ooooo`;o003o5Oooool2o`00oooooooo3Oooool005Sooooo0_l00004ooooo`06
o`000?ooooooooooo`000?oooooo00000oooool00ol0003oooooooooo`03ooooo`goZJVY3_ooool0
1Ol0003oooooooooool00?oooooo00;o003o00?oooooo`00ool00?l05_ooool00ol00?oooooooooo
o`3oooooo`gooooo001Iooooo`03o`000?oooooooooo00?ooooo0_l00002ooooo`Go00001_ooool;
ojVYZ@kooooo00?o0000ooooool00?l00ol00?l2ooooo`03o`00oooooooooooo01Gooooo00?o003o
ooooooooool0ooooool>ooooo`00K_ooool;ojVYZ@cooooo0_l000001?oooooo003oooooool00?l2
ooooo`;o003o5_ooool2o`00oooooooo4Oooool006oooooo2OnYZJT<ooooo`03o`000?oooooo003o
00?o003o00?oooooo`00ool00?l05oooool00ol00?ooooooooooo`3ooooooa7ooooo001aooooo`Go
ZJVY3Oooool01_l0003oooooo`00oooooooo003oooooo`;o003o5oooool2o`00ohGooooo0ol00004
ooooo`?o00000_ooool2o`0000[ooooo0ol00003ooooo`?o00001?ooool2o`0000Wooooo0ol00003
ooooo`?o00000oooool3o`0004oooooo001aooooo`03o`000?l00?oooooo00gooooo0_l00004o`00
o`03ooooool00?oooooo01Oooooo00?o003oooooooooool0Q_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`07ooooo`05o`000?oo
ooooooooooooool000000oooool01Ol0003ooooooooooooooooo0000053ooooo001`ooooo`03o`00
0?l00?oo003o00gooooo00Ko0000ooooool00?ooooooo`00ooooool2o`00oaOooooo0_l00?maoooo
o`?o00001?ooool3o`0000;ooooo0ol00009ooooo`04o`000?ooooooooooooooo`Go00001Oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00000_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Gooooo00Go0000oooooooooooooooo
o`000009ooooo`?o00000oooool3o`0000;ooooo1Ol0000gooooo`00L?ooool00ol0003o003oo`00
o`0<ooooo`03o`000?oooooo003o00Co003o6?ooool00ol00?ooooooooooo`1booooo`03o`000?oo
oooooooo00Cooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`06ooooo`05o`00
0?ooooooooooooooool000000_ooool01?l0003oooooooooooooool4o`0000Wooooo00?o0000oooo
ooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01Oooool01Ol0003ooooooooooooooooo000000[ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0=Oooool0073ooooo
00?o0000o`00ooooool02_ooool2o`0000?o003o00?oooooo`00ooooool05oooool2o`00oecooooo
0ol00004ooooo`?o00000oooool00ol0003oooooooooo`09ooooo`04o`000?ooooooooooooooo`Go
000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooool000000_ooool01Ol0003ooooooooooooooooo000000Wooooo00Co0000oooo
oooooooooooo1?l00003ooooo`04o`000?ooooooooooo`0000Wooooo00Co0000oooooooooooooooo
1?l00004ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01Oooool01Ol0003ooooo
oooooooooooo000000[ooooo0ol00003ooooo`?o00000oooool3o`00023ooooo001_ooooo`03o`00
0?oooooo003o00[ooooo00?o0000ooooool00?l01?l00?lHooooo`03o`00oooooooooooo05gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo
00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooooooo`?o00002?ooool2o`00
00Gooooo0_l00002ooooo`05o`000?ooooooooooooooool000002?ooool2o`0000?ooooo00?o0000
ooooooooool01?ooool01?l0003oooooooooool00008ooooo`;o00000oooool00ol0003ooooooooo
o`04ooooo`;o00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool00000
0oooool00ol0003oooooooooo`0Mooooo`00Koooool00ol0003o003oo`00o`09ooooo`03o`000?l0
0?oo003o00?o003o6Oooool00ol00?ooooooooooo`1Nooooo`04o`000?ooooooooooooooo`Go0000
1?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
0_ooool01Ol0003ooooooooooooooooo000000Sooooo00?o0000ooooooooool01?ooool01?l0003o
ooooooooooooool3o`0000[ooooo00Co0000oooooooooooooooo1Ol00003ooooo`;o00002_ooool0
1?l0003oooooooooooooool5o`0000?ooooo00?o0000ooooooooool02?ooool01?l0003ooooooooo
ooooool4o`0000;ooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0Mooooo`00Koooool00ol0003o003o
o`00o`07ooooo`;o00001?l00?lIooooo`;o003oHOooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`;o00001Oooool2o`0000;o
oooo00Go0000ooooooooooooooooo`00001>ooooo`;o00000oooool00ol0003oooooooooo`03oooo
o`05o`000?ooooooooooooooool000002Oooool00ol0003oooooooooo`05ooooo`04o`000?oooooo
ooooooooo`;o00008?ooool006oooooo00?o0000o`00ooooool01_ooool00ol0003o003oo`00o`02
o`00oa[ooooo00?o003oooooooooool0Coooool3o`0000;ooooo0ol0000:ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Cooooo00Co0000oooooooooooooooo0ol0001@ooooo`04o`000?ooooooooooooooo`Go0000
0_ooool3o`0000[ooooo00Co0000oooooooooooooooo1?l00006ooooo`03o`000?oooooooooo00Ko
oooo0ol00003ooooo`?o00003_ooool006kooooo00?o0000o`00ool00?l01_ooool00ol0003o003o
o`00o`02o`00oaWooooo0_l00?mCooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooooooo
ool01_ooool2o`0000Gooooo0_l00002ooooo`05o`000?ooooooooooooooool00000O_ooool2o`00
00?ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000
ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00_ooooo001^ooooo`03o`000?l00?oo
003o00Gooooo00Co0000o`00ool00?oo003o6_ooool00ol00?ooooooooooo`1@ooooo`Go000000?o
ooooo`000?ooool00_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Ol0001oooooo`04o`000?ooooooooooooooo`Go00000_ooool3o`0000[ooooo
00?o0000ooooooooool01Oooool00ol0003oooooo`00000;ooooo`00K_ooool00ol0003o003ooooo
o`03ooooo`;o00000_l00?lJooooo`;o003oDoooool01?l0003oooooooooool00002ooooo`Co0000
/?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000_ooooo001[ooooo`GoZJVY0ooo
ool00ol0003o003oo`00o`0Kooooo`03o`00oooooooooooo05Cooooo00?o0000ooooool000000_oo
ool00ol0003oooooooooo`0nooooo`GoZJVY4oooool5ojVYZA;ooooo1OnYZJToooooo`04o`000?oo
ooooooooooooo`Co00003_ooool006Wooooo2OnYZJT3o`00oa_ooooo00?o003oooooooooool0E_oo
ool2o`0000?ooooo00?o0000ooooooooool09Oooool5ojVYZA7ooooo2OnYZJT?ooooo`WoZJVY3_oo
ool9ojVYZA7ooooo1OnYZJTVooooo`;o00000oooool00ol0003oooooooooo`0?ooooo`00J?ooool;
ojVYZA_ooooo0_l00?mJooooo`03o`000?oooooooooo00;ooooo0_l0000Sooooo`WoZJVY3_ooool;
ojVYZ@gooooo2onYZJT<ooooo`_oZJVY3_ooool9ojVYZBGooooo00Co0000oooooooooooooooo1Ol0
000=ooooo`00D_ooool3o`0000Cooooo0_l00003ooooo`?o00001oooool;ojVYZA[ooooo00?o003o
ooooooooool0Poooool;ojVYZ@gooooo2onYZJT=ooooo`_oZJVY3?ooool;ojVYZ@gooooo2onYZJTj
ooooo`00Doooool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooo`000?oooooo00000ooo
ool00ol0003oooooooooo`03ooooo`goZJVY5oooool2o`00og;ooooo1Ol0000?ooooo`_oZJVY3?oo
ool=ojVYZ@;ooooo2Ol0000=ojVYZ@Wo003o00?oooooojVYZOnYZJT02onYZJT<ooooo`_oZJVY4?oo
ool5o`0002Gooooo001Cooooo`03o`000?oooooooooo00;ooooo00Ko0000oooooooooooo0000oooo
ool00003ooooo`03o`000?oooooooooo00?ooooo3OnYZJTFooooo`03o`00oooooooooooo073ooooo
2Ol0000<ooooo`goZJVY0_ooool9o`0000goZJVY0_l00003ooooo`Ko003o3OnYZJT6o`00o`?ooooo
00?o003oojVYZOnYZJT02onYZJT:o`00o`03oooooonYZJWoZJVY00_oZJVY3Oooool9o`0002?ooooo
001Cooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo0000ooooo`04o`0000Kooooo
3OnYZJTDooooo`;o003oL_ooool;o`0000_ooooo3OnYZJT2o`0000Cooooo1Ol00?l=ojVYZ@_o003o
3OnYZJT6o`0000Co003o3OnYZJT6o`00o`Cooooo00?o003oojVYZOnYZJT02onYZJT<ooooo`_o0000
8_ooool005?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003oooooo`0000Wo
oooo3OnYZJTCooooo`03o`00oooooooooooo05gooooo1Ol0000@ooooo`_o00000oooool8o`0000go
ZJVY2ol00?l=ojVYZ@_ooooo3OnYZJT:ooooo`goZJVY00?oooooo`000?l000002?l00?l=ojVYZ@Wo
003o0oooool;o`0000oooooo1Ol0000>ooooo`00D_ooool2o`0000Cooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`06ooooo`goZJVY4Oooool2o`00oekooooo2Ol0000=ooooo`ko
00002Ol00?l=ojVYZ@_ooooo3OnYZJT;ooooo`goZJVY2_ooool=ojVYZ@_ooooo3OnYZJT;o`00o`go
00003?ooool9o`0000cooooo001Cooooo`03o`000?oooooooooo00?ooooo0_l00004ooooo`;o0000
1oooool;ojVYZA7ooooo00?o003oooooooooool0G_ooool:o`0000cooooo3Ol00008o`00o`;ooooo
3OnYZJT<ooooo`_oZJVY3Oooool;ojVYZ@cooooo2onYZJT<ooooo`goZJVY0oooool3o`0000Go003o
3Ol0000;ooooo`_o00002oooool006Sooooo2onYZJT@ooooo`03o`00oooooooooooo05oooooo2_l0
0004oooooaGo00002oooool;ojVYZ@gooooo2onYZJT<ooooo`03o`000?nYZJWoZJVY00WoZJVY3?oo
ool;ojVYZ@gooooo2onYZJT<ooooo`go00002?l00?l3ooooo`_o00002oooool006Wooooo2OnYZJT?
ooooo`;o003oH_ooool>o`0000So003o3Ol0000;ooooo`_oZJVY3_ooool9ojVYZ@cooooo00?o0000
ooooool00?l02OnYZJT>ooooo`WoZJVY3_ooool;ojVYZ@cooooo3ol00008o`00o`co00002oooool0
06_ooooo1OnYZJT@ooooo`03o`00oooooooooooo06;ooooo2ol00007o`00o`Cooooo3Ol0000<oooo
o`WoZJVY3_ooool00ol00?ooooooooooo`05ojVYZ@gooooo00Co0000ooooool00?oo003o0_ooool5
ojVYZA3ooooo00?o003ooooooonYZJT01?nYZJTAooooo`WoZJVY3Oooool=o`0000Cooooo1?l00002
o`00o`co00002oooool006_ooooo00?o0000o`00ooooool04?ooool2o`00ofGooooo2ol0000<oooo
o`_o00003oooool5ojVYZ@oooooo00?o003oooooooooool04_ooool01?l0003oooooo`00ool00?lG
ooooo`03o`00oooooooooooo01Sooooo1OnYZJT@ooooo`_o00002oooool<o`0000_ooooo001[oooo
o`03o`000?l00?oooooo00oooooo00?o003oooooooooool0IOooool;o`0000cooooo2ol0000Roooo
o`03o`00oooooooooooo01;ooooo00Co0000ooooool00?oo003o5oooool00ol00?ooooooooooo`0^
ooooo`_o00002oooool<o`0000_ooooo001[ooooo`;o003o3_ooool2o`00ofSooooo2ol0000=oooo
o`Wo00008_ooool00ol00?ooooooooooo`0Booooo`04o`000?oooooo003oo`00oaOooooo00?o003o
ooooooooool0<?ooool9o`0000cooooo3?l0000;ooooo`00J_ooool00ol0003o003oo`00o`0=oooo
o`03o`00oooooooooooo06Sooooo2_l0000@ooooo`Go00009?ooool00ol00?ooooooooooo`0Aoooo
o`03o`000?oooooo003o00;o003o5_ooool00ol00?ooooooooooo`0cooooo`Go00003oooool;o`00
00_ooooo001Zooooo`03o`000?l00?oo003o00cooooo00?o003oooooooooool0JOooool:o`0003So
oooo00?o003oooooooooool04Oooool00ol0003oooooo`00o`02o`00oaKooooo00?o003ooooooooo
ool0B?ooool;o`0000_ooooo001Zooooo`03o`000?l00?oo003o00[ooooo0_l00?m/ooooo`Wo0000
>?ooool00ol00?ooooooooooo`0Aooooo`03o`000?oooooo003o00;o003o5_ooool00ol00?oooooo
ooooo`1:ooooo`Wo00003?ooool006[ooooo0_l00?l:ooooo`03o`00oooooooooooo06kooooo1Ol0
000iooooo`03o`00oooooooooooo017ooooo00?o0000ooooool00?l00_l00?lFooooo`03o`00oooo
oooooooo04gooooo1Ol0000>ooooo`00J_ooool2o`00o`Sooooo0_l00?n^ooooo`03o`00oooooooo
oooo017ooooo00?o0000ooooool00?l00_l00?lFooooo`03o`00oooooooooooo067ooooo001Yoooo
o`03o`000?l00?oo003o00Oooooo00?o003oooooooooool0[Oooool00ol00?ooooooooooo`0Aoooo
o`06o`000?oooooo003oo`00oooooooo003o5Oooool00ol00?ooooooooooo`1Rooooo`00JOooool0
0ol0003o003oooooo`05ooooo`;o003o[oooool00ol00?ooooooooooo`0Aooooo`06o`000?oooooo
003oo`00oooooooo003o5Oooool00ol00?ooooooooooo`1Sooooo`00I_ooool5ojVYZ@Gooooo00?o
003oooooooooool0[_ooool00ol00?ooooooooooo`0Aooooo`06o`000?oooooo003oo`00oooooooo
003o5Oooool00ol00?ooooooooooo`1Tooooo`00I?ooool9ojVYZ@03ooooool00?oo003o0;3ooooo
00?o003oooooooooool04?ooool2o`000003ooooool00?oooooo00;o003o5Oooool00ol00?oooooo
ooooo`1Uooooo`00Hoooool;ojVYZK7ooooo00?o003oooooooooool04?ooool01Ol0003ooooooooo
ool00?oooooo00;o003o5Oooool00ol00?ooooooooooo`1Vooooo`00COooool3o`0000Cooooo0_l0
0003ooooo`?o00001oooool;ojVYZK7ooooo00?o003oooooooooool03oooool01?l0003ooooooooo
ooooool2o`00o`03ooooool00?oooooo01?ooooo00?o003oooooooooool0Ioooool004kooooo00?o
0000ooooooooool00_ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000oooooooo
ool00oooool=ojVYZJoooooo00?o003oooooooooool03oooool01?l0003oooooooooooooool2o`00
o`03ooooool00?oooooo01Cooooo00?o003oooooooooool0Ioooool004kooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo3OnYZJV^oooo
o`03o`00oooooooooooo00oooooo00Co0000oooooooooooooooo0_l00?l00ooooooo003oooooo`0D
ooooo`03o`00oooooooooooo06Sooooo001>ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00001Oooool00ol0003oooooooooo`03ooooo`goZJVY[Oooool00ol00?ooooooooooo`0?
ooooo`04o`000?ooooooooooooooo`;o003o00?oooooo`00ooooool05?ooool00ol00?oooooooooo
o`1Yooooo`00C_ooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooo`000?ooool01?l0
0006ooooo`goZJVY[?ooool00ol00?ooooooooooo`0?ooooo`04o`000?ooooooooooooooo`;o003o
00?oooooo`00ooooool05?ooool00ol00?ooooooooooo`1Zooooo`00COooool2o`0000Cooooo00Ko
0000oooooooooooo0000ooooool00009ooooo`goZJVYZoooool00ol00?ooooooooooo`0?ooooo`04
o`000?ooooooooooooooo`;o003o0_ooool00ol00?ooooooooooo`0Booooo`03o`00oooooooooooo
06_ooooo001>ooooo`03o`000?oooooooooo00?ooooo0_l00002ooooo`Go00001_ooool;ojVYZJ_o
oooo00?o003oooooooooool03oooool01?l0003oooooooooooooool2o`00o`;ooooo00?o003ooooo
ooooool04_ooool00ol00?ooooooooooo`1/ooooo`00Hoooool;ojVYZJ[ooooo00?o003ooooooooo
ool03oooool02Ol0003ooooooooooooooooo003oooooool00?ooooooo`00o`0Dooooo`03o`00oooo
oooooooo06gooooo001Tooooo`WoZJVYZ_ooool00ol00?ooooooooooo`0?ooooo`09o`000?oooooo
ooooooooool00?ooooooo`00oooooooo003o01Cooooo00?o003oooooooooool0K_ooool006Kooooo
1OnYZJV/ooooo`03o`00oooooooooooo00kooooo00Wo0000ooooooooooooooooo`00oooooooo003o
ooooool00?l05?ooool00ol00?ooooooooooo`1_ooooo`00I_ooool00ol0003o003oooooo`2]oooo
o`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooooooo003o00;ooooo00?o
003oooooooooool04Oooool00ol00?ooooooooooo`1`ooooo`00I_ooool00ol0003o003oooooo`2/
ooooo`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooooooo003o00;ooooo
00?o003oooooooooool04Oooool00ol00?ooooooooooo`1aooooo`00I_ooool00ol0003o003ooooo
o`2[ooooo`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooooooo003o00;o
oooo00?o003oooooooooool04Oooool00ol00?ooooooooooo`1booooo`00IOooool00ol0003ooooo
o`00o`2[ooooo`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooooooo003o
00;ooooo00?o003oooooooooool04Oooool00ol00?ooooooooooo`1cooooo`00IOooool00ol0003o
003oo`00o`2Zooooo`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooooooo
003o00;ooooo00?o003oooooooooool04Oooool00ol00?ooooooooooo`1dooooo`00IOooool00ol0
003o003oo`00o`2Yooooo`03o`00oooooooooooo00kooooo00Oo0000ooooooooooooooooo`00oooo
oooo003o00?ooooo00?o003oooooooooool04?ooool00ol00?ooooooooooo`1eooooo`00IOooool0
0ol0003o003oo`00o`2Xooooo`03o`00oooooooooooo00kooooo00Go0000ooooooooooooooooo`00
o`02ooooo`04o`00ooooooooooooo`00oa;ooooo00?o003oooooooooool0M_ooool006Gooooo00?o
0000o`00ool00?l0Yoooool00ol00?ooooooooooo`0=ooooo`;o00000oooool01?l00?oooooooooo
ool00?l2ooooo`03o`00oooooooooooo013ooooo00?o003oooooooooool0Moooool006Gooooo00?o
0000o`00ooooool0Yoooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00;ooooo00Co
003ooooooooooooo003o0_ooool00ol00?ooooooooooo`0@ooooo`03o`00oooooooooooo07Sooooo
001Uooooo`;o003oYoooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00;ooooo00Co
003ooooooooooooo003o0_ooool00ol00?ooooooooooo`0@ooooo`03o`00oooooooooooo07Wooooo
001Uooooo`;o003oY_ooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00?ooooo00?o
003oooooool00?l00oooool00ol00?ooooooooooo`0?ooooo`03o`00oooooooooooo07[ooooo001R
ooooo`GoZJVYYOooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00?ooooo00?o003o
ooooool00?l00oooool00ol00?ooooooooooo`0@ooooo`03o`00oooooooooooo07[ooooo001Poooo
o`WoZJVYX_ooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00?ooooo00?o003ooooo
ool00?l00oooool00ol00?ooooooooooo`0@ooooo`03o`00oooooooooooo07_ooooo001Oooooo`_o
ZJVYX?ooool00ol00?ooooooooooo`0<ooooo`03o`000?oooooooooo00?ooooo00?o003oooooool0
0?l00oooool00ol00?ooooooooooo`0@ooooo`03o`00oooooooooooo07cooooo0019ooooo`?o0000
1?ooool2o`0000Cooooo0ol00006ooooo`_oZJVYWoooool00ol00?ooooooooooo`0<ooooo`03o`00
0?oooooooooo00?ooooo00?o003oooooool00?l00oooool00ol00?ooooooooooo`0@ooooo`03o`00
oooooooooooo07gooooo001:ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo0000
1?ooool00ol0003oooooooooo`04ooooo`goZJVYWOooool00ol00?ooooooooooo`0<ooooo`03o`00
0?oooooooooo00?ooooo00Co003ooooooooooooo003o0oooool00ol00?ooooooooooo`0?ooooo`03
o`00oooooooooooo07kooooo001:ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo
0000ooooo`05o`0000Gooooo3OnYZJVLooooo`03o`00oooooooooooo00cooooo00?o0000oooooooo
ool00oooool01?l00?ooooooooooool00?l3ooooo`03o`00oooooooooooo00oooooo00?o003ooooo
ooooool0Ooooool004[ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003ooooo
o`0000;ooooo00?o0000ooooooooool01?ooool=ojVYZI_ooooo00?o003oooooooooool03?ooool0
0ol0003oooooooooo`03ooooo`04o`00ooooooooooooo`00o`?ooooo00?o003oooooooooool03ooo
ool00ol00?ooooooooooo`20ooooo`00B_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooool000001_ooool=ojVYZI_ooooo00?o003oooooooooool02ooo
ool00ol0003oooooooooo`03ooooo`04o`00ooooooooooooo`00o`?ooooo00?o003oooooooooool0
3oooool00ol00?ooooooooooo`21ooooo`00BOooool2o`0000Cooooo00Co0000oooooooooooo0000
0oooool2o`0000Kooooo3OnYZJVJooooo`03o`00oooooooooooo00_ooooo00?o0000ooooooooool0
0oooool01?l00?ooooooooooool00?l3ooooo`03o`00oooooooooooo00oooooo00?o003ooooooooo
ool0P_ooool004[ooooo00?o0000ooooooooool00oooool2o`0000Gooooo00?o0000ooooooooool0
1Oooool;ojVYZI[ooooo00?o003oooooooooool02oooool00ol0003oooooooooo`03ooooo`04o`00
ooooooooooooo`00o`Cooooo00?o003oooooooooool03_ooool00ol00?ooooooooooo`23ooooo`00
Goooool;ojVYZIWooooo00?o003oooooooooool02oooool00ol0003oooooooooo`03ooooo`04o`00
ooooooooooooo`00o`Cooooo00?o003oooooooooool03_ooool00ol00?ooooooooooo`24ooooo`00
H?ooool9ojVYZIWooooo00?o003oooooooooool02oooool00ol0003oooooooooo`03ooooo`04o`00
ooooooooooooo`00o`Cooooo00?o003oooooooooool03_ooool00ol00?ooooooooooo`25ooooo`00
H_ooool5ojVYZI[ooooo00?o003oooooooooool02oooool00ol0003oooooooooo`03ooooo`04o`00
ooooooooooooo`00o`Cooooo00?o003oooooooooool03_ooool00ol00?ooooooooooo`26ooooo`00
H_ooool00ol0003o003oooooo`2Kooooo`03o`00oooooooooooo00_ooooo00?o0000ooooooooool0
0oooool01?l00?ooooooooooool00?l4ooooo`03o`00oooooooooooo00kooooo00?o003ooooooooo
ool0Qoooool006;ooooo00?o0000o`00ooooool0V_ooool00ol00?ooooooooooo`0:ooooo`;o0000
1Oooool01Ol00?oooooooooooooooooo003o00Cooooo00?o003oooooooooool03Oooool00ol00?oo
ooooooooo`28ooooo`00H_ooool00ol0003o003oooooo`2Iooooo`03o`00oooooooooooo00[ooooo
00?o0000ooooooooool01?ooool01Ol00?oooooooooooooooooo003o00Cooooo00?o003ooooooooo
ool03Oooool00ol00?ooooooooooo`29ooooo`00HOooool00ol0003oooooo`00o`2Jooooo`03o`00
oooooooooooo00Wooooo00?o0000ooooooooool01?ooool01Ol00?oooooooooooooooooo003o00Co
oooo00?o003oooooooooool03Oooool00ol00?ooooooooooo`2:ooooo`00HOooool00ol0003ooooo
o`00o`2Iooooo`03o`00oooooooooooo00Wooooo00?o0000ooooooooool01?ooool01Ol00?oooooo
oooooooooooo003o00Cooooo00?o003oooooooooool03Oooool00ol00?ooooooooooo`2;ooooo`00
HOooool00ol0003oooooo`00o`2Hooooo`03o`00oooooooooooo00Wooooo00?o0000ooooooooool0
1?ooool01Ol00?oooooooooooooooooo003o00Cooooo00?o003oooooooooool03Oooool00ol00?oo
ooooooooo`2<ooooo`00HOooool00ol0003oooooo`00o`2Gooooo`03o`00oooooooooooo00Wooooo
00?o0000ooooooooool01Oooool01?l00?ooooooooooool00?l5ooooo`03o`00oooooooooooo00go
oooo00?o003oooooooooool0S?ooool0067ooooo00?o0000o`00ool00?l0U_ooool00ol00?oooooo
ooooo`09ooooo`03o`000?oooooooooo00Gooooo00Co003ooooooooooooo003o1Oooool00ol00?oo
ooooooooo`0=ooooo`03o`00oooooooooooo08gooooo001Qooooo`03o`000?l00?oooooo09Gooooo
00?o003oooooooooool02Oooool00ol0003oooooooooo`05ooooo`04o`00ooooooooooooo`00o`Go
oooo00?o003oooooooooool03Oooool00ol00?ooooooooooo`2>ooooo`00HOooool00ol0003o003o
ooooo`2Dooooo`03o`00oooooooooooo00Wooooo00?o0000ooooooooool01Oooool01?l00?oooooo
ooooool00?l5ooooo`03o`00oooooooooooo00gooooo00?o003oooooooooool0Soooool005kooooo
1Ol0002Dooooo`03o`00oooooooooooo00Wooooo00?o0000ooooooooool01Oooool01Ol00?oooooo
oooooooooooo003o00Cooooo00?o003oooooooooool03Oooool00ol00?ooooooooooo`2@ooooo`00
G?ooool9o`00097ooooo00?o003oooooooooool02Oooool00ol0003oooooooooo`05ooooo`05o`00
ooooooooooooooooool00?l01Oooool00ol00?ooooooooooo`0<ooooo`03o`00oooooooooooo097o
oooo001Kooooo`_o0000T?ooool00ol00?ooooooooooo`08ooooo`03o`000?oooooooooo00Gooooo
00Go003oooooooooooooooooo`00o`05ooooo`03o`00oooooooooooo00cooooo00?o003ooooooooo
ool0T_ooool004Kooooo0ol00004ooooo`;o00000oooool3o`0000Kooooo2ol0002?ooooo`03o`00
oooooooooooo00Sooooo00?o0000ooooooooool01Oooool01Ol00?oooooooooooooooooo003o00Go
oooo00?o003oooooooooool03?ooool00ol00?ooooooooooo`2Cooooo`00Aoooool00ol0003ooooo
ooooo`02ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003oooooooooo`02oooo
o`go0000SOooool00ol00?ooooooooooo`08ooooo`03o`000?oooooooooo00Gooooo00Go003ooooo
ooooooooooooo`00o`05ooooo`03o`00oooooooooooo00cooooo00?o003oooooooooool0U?ooool0
04Oooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooo
oooo00;ooooo3Ol0002<ooooo`03o`00oooooooooooo00Sooooo00?o0000ooooooooool01Oooool0
1Ol00?oooooooooooooooooo003o00Gooooo00?o003oooooooooool03?ooool00ol00?oooooooooo
o`2Eooooo`00Aoooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000?ooooo0_l0
0005ooooo`go0000Roooool00ol00?ooooooooooo`08ooooo`03o`000?oooooooooo00Gooooo00Go
003oooooooooooooooooo`00o`06ooooo`03o`00oooooooooooo00_ooooo00?o003oooooooooool0
U_ooool004Oooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00;ooooo3Ol0002:ooooo`03o`00oooooooooooo00Sooooo00?o0000ooooooooool0
1Oooool01Ol00?oooooooooooooooooo003o00Kooooo00?o003oooooooooool02oooool00ol00?oo
ooooooooo`2Gooooo`00A_ooool2o`0000Cooooo00Ko0000oooooooooooo0000ooooool00003oooo
o`03o`000?oooooooooo00;ooooo3Ol00029ooooo`03o`00oooooooooooo00Sooooo00?o0000oooo
ooooool01Oooool01Ol00?oooooooooooooooooo003o00Kooooo00?o003oooooooooool02oooool0
0ol00?ooooooooooo`2Hooooo`00Aoooool00ol0003oooooooooo`03ooooo`;o00000oooool3o`00
00Kooooo2ol00029ooooo`03o`00oooooooooooo00Sooooo00?o0000ooooooooool01Oooool00ol0
0?ooooooooooo`02ooooo`03o`00oooooooooooo00?ooooo00?o003oooooooooool02oooool00ol0
0?ooooooooooo`2Iooooo`00Foooool;o`0008Sooooo00?o003oooooooooool01oooool2o`0000Oo
oooo00?o003oooooooooool00_ooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo00_o
oooo00?o003oooooooooool0V_ooool005cooooo2Ol00028ooooo`03o`00oooooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol00?ooooooooooo`02ooooo`03o`00oooooooooooo00Cooooo
00?o003oooooooooool02_ooool00ol00?ooooooooooo`2Kooooo`00G_ooool5o`0008[ooooo00?o
003oooooooooool01_ooool00ol0003oooooooooo`06ooooo`03o`00oooooooooooo00;ooooo00?o
003oooooooooool01?ooool00ol00?ooooooooooo`0:ooooo`03o`00oooooooooooo09cooooo001P
ooooo`03o`000?l00?oooooo08Wooooo00?o003oooooooooool01_ooool00ol0003oooooooooo`06
ooooo`03o`00oooooooooooo00;ooooo00?o003oooooooooool01?ooool00ol00?ooooooooooo`0:
ooooo`03o`00oooooooooooo09gooooo001Pooooo`03o`000?l00?oooooo08Sooooo00?o003ooooo
ooooool01_ooool00ol0003oooooooooo`06ooooo`03o`00oooooooooooo00;ooooo00?o003ooooo
ooooool01?ooool00ol00?ooooooooooo`0:ooooo`03o`00oooooooooooo09kooooo001Pooooo`03
o`000?l00?oooooo08Oooooo00?o003oooooooooool01_ooool00ol0003oooooooooo`06ooooo`03
o`00oooooooooooo00;ooooo00?o003oooooooooool01?ooool00ol00?ooooooooooo`0:ooooo`03
o`00oooooooooooo09oooooo001Oooooo`03o`000?l00?oo003o08Oooooo00?o003oooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`05o`00ooooooooooooooooool00?l01oooool00ol00?oo
ooooooooo`0:ooooo`03o`00oooooooooooo09oooooo001Oooooo`03o`000?l00?oo003o08Kooooo
00?o003oooooooooool01_ooool00ol0003oooooooooo`07ooooo`05o`00ooooooooooooooooool0
0?l01oooool00ol00?ooooooooooo`0:ooooo`03o`00oooooooooooo0:3ooooo001Oooooo`03o`00
0?l00?oo003o08Gooooo00?o003oooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
oooooooooooo00;ooooo00?o003oooooooooool01?ooool00ol00?ooooooooooo`0:ooooo`03o`00
oooooooooooo0:7ooooo001Oooooo`03o`000?l00?oo003o08Cooooo00?o003oooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`00oooooooooooo00;ooooo00?o003oooooooooool01?oo
ool00ol00?ooooooooooo`0:ooooo`03o`00oooooooooooo0:;ooooo001Oooooo`03o`000?l00?oo
003o08?ooooo00?o003oooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00oooooooo
oooo00;ooooo00?o003oooooooooool01?ooool00ol00?ooooooooooo`0:ooooo`03o`00oooooooo
oooo0:?ooooo001Oooooo`03o`000?l00?oo003o08_ooooo00?o0000ooooooooool01oooool00ol0
0?ooooooooooo`02ooooo`03o`00oooooooooooo00Gooooo00?o003oooooooooool02Oooool00ol0
0?ooooooooooo`2Tooooo`00Goooool00ol0003o003oooooo`2:ooooo`03o`000?oooooooooo00Oo
oooo00?o003oooooooooool00_ooool00ol00?ooooooooooo`05ooooo`03o`00oooooooooooo00Wo
oooo00?o003oooooooooool0YOooool005oooooo00?o0000o`00ooooool0ROooool00ol0003ooooo
ooooo`07ooooo`03o`00oooooooooooo00;ooooo00?o003oooooooooool01Oooool00ol00?oooooo
ooooo`09ooooo`03o`00oooooooooooo0:Kooooo001Looooo`Go0000ROooool00ol0003ooooooooo
o`07ooooo`03o`00oooooooooooo00;ooooo00?o003oooooooooool01Oooool00ol00?oooooooooo
o`09ooooo`03o`00oooooooooooo0:Oooooo001Jooooo`Wo0000Q_ooool00ol0003oooooooooo`07
ooooo`03o`00oooooooooooo00;ooooo00?o003oooooooooool01Oooool00ol00?ooooooooooo`09
ooooo`03o`00oooooooooooo0:Sooooo001Iooooo`_o0000Q?ooool00ol0003oooooooooo`07oooo
o`03o`00oooooooooooo00;ooooo00?o003oooooooooool01_ooool00ol00?ooooooooooo`08oooo
o`03o`00oooooooooooo0:Wooooo0014ooooo`?o00001?ooool2o`0000;ooooo1Ol00005ooooo`_o
0000Poooool00ol0003oooooooooo`07ooooo`03o`00oooooooooooo00?ooooo00?o003ooooooooo
ool01Oooool00ol00?ooooooooooo`08ooooo`03o`00oooooooooooo0:[ooooo0015ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00004oooo
o`go0000POooool00ol0003oooooooooo`07ooooo`03o`00oooooooooooo00?ooooo00?o003ooooo
ooooool01Oooool00ol00?ooooooooooo`08ooooo`03o`00oooooooooooo0:_ooooo0015ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`04oooo
o`go0000P?ooool00ol0003oooooooooo`07ooooo`03o`00oooooooooooo00?ooooo00?o003ooooo
ooooool01Oooool00ol00?ooooooooooo`08ooooo`03o`00oooooooooooo0:cooooo0015ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`03oooo
o`go0000ROooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo00Gooooo00?o003ooooo
ooooool02?ooool00ol00?ooooooooooo`2]ooooo`00AOooool00ol0003oooooooooo`02ooooo`06
o`000?ooooooooooo`000?oooooo00000oooool00ol0003oooooooooo`02ooooo`go0000R?ooool0
0ol00?ooooooooooo`03ooooo`03o`00oooooooooooo00Kooooo00?o003oooooooooool01oooool0
0ol00?ooooooooooo`2^ooooo`00A?ooool2o`0000Cooooo00Ko0000oooooooooooo0000ooooool0
0003ooooo`03o`000?oooooooooo00;ooooo3Ol00027ooooo`03o`00oooooooooooo00?ooooo00?o
003oooooooooool01_ooool00ol00?ooooooooooo`07ooooo`03o`00oooooooooooo0:oooooo0015
ooooo`03o`000?oooooooooo00?ooooo0_l00003ooooo`?o00001_ooool;o`0008Oooooo00?o003o
ooooooooool00oooool00ol00?ooooooooooo`06ooooo`03o`00oooooooooooo00Oooooo00?o003o
ooooooooool0/?ooool005Wooooo2ol00026ooooo`03o`00oooooooooooo00?ooooo00?o003ooooo
ooooool01_ooool00ol00?ooooooooooo`07ooooo`03o`00oooooooooooo0;7ooooo001Jooooo`Wo
0000Q_ooool00ol00?ooooooooooo`03ooooo`03o`00oooooooooooo00Kooooo00?o003ooooooooo
ool01oooool00ol00?ooooooooooo`2booooo`00oooooonhooooo`00oooooonhooooo`00oooooonh
ooooo`00oooooonhooooo`00\
\>"],
  ImageRangeCache->{{{0, 438.812}, {208.375, 0}} -> {-0.0529489, -0.0238146, \
0.00479909, 0.00479909}, {{11, 209.438}, {203.375, 4.9375}} -> {-0.0497143, \
0.33763, 0.000533603, 0.00065943}, {{229.312, 427.75}, {203.375, 4.9375}} -> \
{0.282435, -0.0525264, 0.000662234, 0.000505185}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "(How did I know what the correct vertex numbers were to pass to the ",
  StyleBox["Zoom",
    FontWeight->"Bold"],
  " option? I checked beforehand by using ",
  StyleBox["ShowLabeledGraph",
    FontWeight->"Bold"],
  " to display ",
  StyleBox["gr",
    FontWeight->"Bold"],
  " with every vertex numbered. I manually resized the graph so that I could \
see the numbers for particular vertices and made a mental note. I then \
deleted the display produced by ",
  StyleBox["ShowLabeledGraph",
    FontWeight->"Bold"],
  ". Somewhat messy, but it works. Alternatively, one could consult the \
vertices in the list of edges we added to create the small-world network )"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "The reason for the surviving Cooperators becomes clear once we've zoomed \
in on the relevant portions of the graph. Each cooperator incident on a \
bridge edge has one more member in his interaction neighborhood than the \
typical member of the network. Hence, 106 and 150 receive a total score of ",
  Cell[BoxData[
      \(TraditionalForm\`3\[Cross]R\)]],
  " whereas typical cooperators not incident on a bridge edge receive a total \
score of only ",
  Cell[BoxData[
      \(TraditionalForm\`2\[Cross]R\)]],
  ". Due to how we set up the payoff matrix, ",
  Cell[BoxData[
      \(TraditionalForm\`3\[Cross]R > 2\[Cross]P\)]],
  ", so individuals whose update neighbor includes 106 or 150 will choose \
Cooperate rather than Defect, preventing every agent in the model from \
switching to Defect. "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Let's see what happens with a different set of initial conditions. The ",
  StyleBox["SetStrategies",
    FontWeight->"Bold"],
  " command takes a model as its first argument and then a list of strategies \
as its second. (It can also take a list of agents, which is an alternate form \
used by the next 3 lines.) We can construct a list of random strategies using \
normal ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " commands. But, this time, let's not use a uniform distribution."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(SetStrategies[
        pd2, \[IndentingNewLine]Table[
          If[Random[] <  .7, 1, 2], {200}]\[IndentingNewLine]];\)\), "\n", 
    \(\(SetStrategies[Agents[pd2, Table[i, {i, 80, 100}]], 1];\)\), "\n", 
    \(\(SetStrategies[Agents[pd2, Table[i, {i, 3, 8}]], 1];\)\), "\n", 
    \(\(SetStrategies[Agents[pd2, Table[i, {i, 171, 173}]], 1];\)\)}], "Input",\

  CellLabel->"In[21]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[\[IndentingNewLine]ExtractGraph[pd2], 
        VertexStyle \[Rule] Disk[Small]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.04635 .52854 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.64046 .0677 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.11621 .25644 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.78974 .14977 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.5 .04545 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.663 .663 .663 r
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
0 0 0 r
.92263 .66733 Mdot
.663 .663 .663 r
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
0 0 0 r
.89832 .71898 Mdot
.663 .663 .663 r
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
0 0 0 r
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.663 .663 .663 r
.82141 .82141 Mdot
0 0 0 r
.81116 .83135 Mdot
.663 .663 .663 r
.8006 .84096 Mdot
.78974 .85023 Mdot
0 0 0 r
.77859 .85916 Mdot
.663 .663 .663 r
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
0 0 0 r
.70636 .905 Mdot
.663 .663 .663 r
.69354 .91129 Mdot
.68052 .91716 Mdot
0 0 0 r
.66733 .92263 Mdot
.65397 .92767 Mdot
.663 .663 .663 r
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
0 0 0 r
.58517 .94649 Mdot
.663 .663 .663 r
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
0 0 0 r
.52854 .95365 Mdot
.663 .663 .663 r
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
0 0 0 r
.45722 .95253 Mdot
.663 .663 .663 r
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
0 0 0 r
.40084 .9436 Mdot
.663 .663 .663 r
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
0 0 0 r
.31948 .91716 Mdot
.663 .663 .663 r
.30646 .91129 Mdot
.29364 .905 Mdot
0 0 0 r
.28102 .89832 Mdot
.26862 .89125 Mdot
.663 .663 .663 r
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
0 0 0 r
.21026 .85023 Mdot
.663 .663 .663 r
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
0 0 0 r
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.663 .663 .663 r
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
0 0 0 r
.095 .29364 Mdot
.663 .663 .663 r
.10168 .28102 Mdot
.10875 .26862 Mdot
0 0 0 r
.11621 .25644 Mdot
.12405 .24451 Mdot
.663 .663 .663 r
.13227 .23282 Mdot
0 0 0 r
.14084 .22141 Mdot
.663 .663 .663 r
.14977 .21026 Mdot
.15904 .1994 Mdot
0 0 0 r
.16865 .18884 Mdot
.663 .663 .663 r
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
0 0 0 r
.29364 .095 Mdot
.663 .663 .663 r
.30646 .08871 Mdot
0 0 0 r
.31948 .08284 Mdot
.663 .663 .663 r
.33267 .07737 Mdot
.34603 .07233 Mdot
0 0 0 r
.35954 .0677 Mdot
.37319 .0635 Mdot
.663 .663 .663 r
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
0 0 0 r
.45722 .04747 Mdot
.663 .663 .663 r
.47146 .04635 Mdot
0 0 0 r
.48572 .04568 Mdot
.663 .663 .663 r
.5 .04545 Mdot
0 0 0 r
.51428 .04568 Mdot
.52854 .04635 Mdot
.663 .663 .663 r
.54278 .04747 Mdot
0 0 0 r
.55697 .04904 Mdot
.663 .663 .663 r
.57111 .05105 Mdot
0 0 0 r
.58517 .05351 Mdot
.663 .663 .663 r
.59916 .0564 Mdot
.61304 .05973 Mdot
0 0 0 r
.62681 .0635 Mdot
.663 .663 .663 r
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
0 0 0 r
.68052 .08284 Mdot
.69354 .08871 Mdot
.663 .663 .663 r
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
0 0 0 r
.82141 .17859 Mdot
.663 .663 .663 r
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
0 0 0 r
.87595 .24451 Mdot
.88379 .25644 Mdot
.663 .663 .663 r
.89125 .26862 Mdot
0 0 0 r
.89832 .28102 Mdot
.663 .663 .663 r
.905 .29364 Mdot
0 0 0 r
.91129 .30646 Mdot
.663 .663 .663 r
.91716 .31948 Mdot
.92263 .33267 Mdot
0 0 0 r
.92767 .34603 Mdot
.663 .663 .663 r
.9323 .35954 Mdot
0 0 0 r
.9365 .37319 Mdot
.663 .663 .663 r
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
0 0 0 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[25]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00P_ooool3o`000005oooooonYZJWoZJVYojVYZOoo
ool00ol000001OooooooZJVYojVYZOnYZJWooooo00?o000000Goooooo`000?l0003o0000ooooo`03
ojVYZH?ooooo001iooooo`?oZJVY00GoooooojVYZOnYZJWoZJVYooooo`05o`0000?oZJVY1Ol00003
ojVYZ@Wo00000onYZJT4o`000004oooooonYZJWoZJVYojVYZG_ooooo001eooooo`?oZJVY00Gooooo
ojVYZOnYZJWoZJVYo`000003ojVYZ@03o`000?oooooo000000;o000000GoooooojVYZOnYZJWoZJVY
ooooo`04o`0000?oZJVY00Goooooo`000?l0003o0000ooooo`03o`000005oooooonYZJWoZJVYojVY
ZOooool01?l00003ojVYZ@Co0000Moooool0077ooooo0onYZJT01OooooooZJVYojVYZOnYZJWo0000
00?oZJVY00CoooooojVYZOnYZJWoZJVY3?ooool00ol0003oooooooooo`0?ooooo`?o000000Gooooo
ojVYZOnYZJWoZJVYooooo`04o`0000?oZJVYLoooool006gooooo0onYZJT01OooooooZJVYojVYZOnY
ZJWo000000?oZJVY4oooool00ol0003oooooooooo`0Hooooo`?o000000GoooooojVYZOnYZJWoZJVY
o`000003ojVYZFoooooo001Yooooo`?o000000GoooooojVYZOnYZJWoZJVYo`000003ojVYZAKooooo
00?o0000ooooooooool07Oooool3ojVYZ@04oooooonYZJWoZJVYojVYZ@Co0000Joooool006Kooooo
1ol00003ojVYZASooooo0_l0000Tooooo`?oZJVY00?oooooo`000?l000000_l00003ojVYZFOooooo
001Rooooo`?oZJVY1ol0000Kooooo`03o`000?oooooooooo02Sooooo0ol000001OooooooZJVYojVY
ZOnYZJWo000000?oZJVYHoooool006;ooooo0onYZJT01?oooooo0000o`000?l0000Mooooo`03o`00
0?oooooooooo02gooooo0onYZJT01?ooooooZJVYojVYZOnYZJUSooooo`00G_ooool3ojVYZ@04o`00
0?nYZJWoZJVYojVYZB3ooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`03ojVYZ@04
o`000?nYZJWoZJVYojVYZEoooooo001Jooooo`Co00000onYZJTSooooo`03o`000?oooooooooo033o
oooo00?o0000ooooooooool01?ooool3ojVYZ@Co0000Foooool005[ooooo0ol000001?ooooooZJVY
ojVYZOnYZJTQooooo`;o0000<oooool00ol0003oooooooooo`04ooooo`?oZJVY00Coooooo`000?l0
003o0000Foooool005Kooooo0onYZJT4o`0002Cooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`08ooooo`Ko0000F?ooool005Kooooo0onYZJTWooooo`03o`000?oooooooooo03Cooooo
00?o0000ooooooooool02oooool4o`0005Oooooo001Cooooo`?o00000onYZJTVooooo`03o`000?oo
oooooooo03Gooooo00?o0000ooooooooool02oooool3o`000004oooooonYZJWoZJVYojVYZECooooo
001Cooooo`?o00009oooool2o`0003Sooooo00?o0000ooooooooool03oooool3ojVYZECooooo001?
ooooo`?oZJVY1?l0000Vooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool03oooool3
ojVYZ@04o`000?nYZJWoZJVYojVYZE3ooooo001?ooooo`?oZJVY:Oooool00ol0003oooooooooo`0i
ooooo`03o`000?oooooooooo01?ooooo0onYZJU@ooooo`00Boooool3ojVYZ@04o`000?nYZJWoZJVY
ojVYZBSooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`0Cooooo`KoZJVYCOooool0
04_ooooo0onYZJT[ooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool05_ooool3ojVY
ZDgooooo0018ooooo`KoZJVY:Oooool2o`0003kooooo00?o0000ooooooooool05_ooool3ojVYZ@04
o`000?nYZJWoZJVYojVYZDWooooo0018ooooo`?oZJVY:oooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo01[ooooo0onYZJU9ooooo`00A?ooool3ojVYZ@04o`000?nYZJWoZJVYojVYZB[o
oooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`0Jooooo`KoZJVYA_ooool004Cooooo
0onYZJT]ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool07Oooool3ojVYZDKooooo
0013ooooo`04o`000?nYZJWoZJVYojVYZBcooooo00?o0000ooooooooool0@Oooool00ol0003ooooo
ooooo`0Mooooo`?oZJVY0_l00014ooooo`00@Oooool3ojVYZBgooooo0_l00014ooooo`03o`000?oo
oooooooo023ooooo0onYZJU3ooooo`00@Oooool3ojVYZBcooooo00?o0000ooooooooool0A?ooool0
0ol0003oooooooooo`0Pooooo`?oZJVY00?o0000ooooooooool0@?ooool003kooooo1_nYZJT[oooo
o`03o`000?oooooooooo04Gooooo00?o0000ooooooooool08?ooool3ojVYZ@04oooooonYZJWoZJVY
ojVYZCoooooo000nooooo`?oZJVY;Oooool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo
02Cooooo0onYZJToooooo`00?Oooool01?l0003oZJVYojVYZOnYZJT/ooooo`03o`000?oooooooooo
04Oooooo00?o0000ooooooooool09?ooool3ojVYZ@03o`000?oooooooooo03cooooo000kooooo`?o
ZJVY;Oooool2o`0004[ooooo00?o0000ooooooooool09oooool3ojVYZCcooooo000kooooo`?oZJVY
;?ooool00ol0003oooooooooo`1:ooooo`03o`000?oooooooooo02Kooooo00Co0000ojVYZOnYZJWo
ZJVY??ooool003Sooooo1_nYZJT[ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool0
9Oooool00ol0003oooooojVYZ@05ojVYZCWooooo000hooooo`?oZJVY;Oooool00ol0003ooooooooo
o`1<ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool00oooool3ojVYZCWooooo000h
ooooo`?oZJVY;?ooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo02?ooooo00?o0000
ooooooooool01?ooool3ojVYZCWooooo000eooooo`?oZJVY;Oooool2o`00053ooooo00?o0000oooo
ooooool08_ooool00ol0003oooooooooo`08ooooo`?oZJVY=_ooool003Gooooo0onYZJT/ooooo`03
o`000?oooooooooo053ooooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`09ooooo`?o
ZJVY=_ooool003Gooooo0onYZJT[ooooo`03o`000?oooooooooo057ooooo00?o0000ooooooooool0
8?ooool00ol0003oooooooooo`0:ooooo`?oZJVY=_ooool003;ooooo0onYZJT]ooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool07_ooool2o`00013ooooo0ol0000cooooo`00<_ooool3
ojVYZBcooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0Nooooo`03o`000?oooooo
oooo013ooooo0ol0000cooooo`00<_ooool3ojVYZB[ooooo0_l0001Eooooo`03o`000?oooooooooo
01gooooo00?o0000ooooooooool04Oooool3o`0003?ooooo000_ooooo`?o0000;?ooool00ol0003o
ooooooooo`1Eooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool05Oooool3ojVYZC3o
oooo000_ooooo`?o0000:oooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo01_ooooo
00?o0000ooooooooool05_ooool3ojVYZC3ooooo000_ooooo`?o0000:_ooool00ol0003ooooooooo
o`1Gooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool05oooool3ojVYZC3ooooo000/
ooooo`?oZJVY:oooool2o`0005[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0K
ooooo`?oZJVY;Oooool002cooooo0onYZJTZooooo`03o`000?oooooooooo05[ooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Looooo`?oZJVY;Oooool002cooooo0onYZJTYooooo`03
o`000?oooooooooo05_ooooo00?o0000ooooooooool05_ooool2o`0001oooooo0onYZJT]ooooo`00
:Oooool3ojVYZB_ooooo00?o0000ooooooooool0G?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo027ooooo0onYZJT[ooooo`00:Oooool3ojVYZB[ooooo00?o0000ooooooooool0GOoo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo02;ooooo0onYZJT[ooooo`00:Oooool3
ojVYZBSooooo0_l0001Pooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool08oooool3
ojVYZB_ooooo000Wooooo`?o0000:Oooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo
01;ooooo00?o0000ooooooooool09oooool3ojVYZBSooooo000Wooooo`?o0000:?ooool00ol0003o
ooooooooo`1Qooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0:?ooool3ojVYZBSo
oooo000Wooooo`?o00009oooool00ol0003oooooooooo`1Rooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool0:Oooool3ojVYZBSooooo000Vooooo`03o`000?oooooooooo02Oooooo00?o
0000ooooooooool0Hoooool00ol0003oooooooooo`0>ooooo`;o0000;oooool00ol0003ooooooooo
o`0Uooooo`009?ooool3ojVYZBOooooo0_l0001Vooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool0;_ooool3ojVYZBKooooo000Tooooo`?oZJVY9_ooool00ol0003oooooooooo`1Voooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool0;oooool3ojVYZBKooooo000Tooooo`?o
ZJVY9Oooool00ol0003oooooooooo`1Wooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool0<?ooool3ojVYZ@03o`000?oooooooooo02?ooooo000Rooooo`?o00009_ooool00ol0003ooooo
ooooo`1Xooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0=?ooool3o`0002?ooooo
000Rooooo`?o00009Oooool00ol0003oooooooooo`1Yooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool0=Oooool3o`0002?ooooo000Rooooo`?o00008oooool2o`0006cooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0fooooo`?o00008oooool0023ooooo0ol0000Toooo
o`03o`000?oooooooooo06cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0ioooo
o`?o00008Oooool0023ooooo0ol0000Sooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool01Oooool2o`0003cooooo0ol0000Qooooo`008?ooool3o`0002;ooooo00?o0000ooooooooool0
K_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo03cooooo0ol0000Qooooo`008?oo
ool00ol0003oooooo`00000Qooooo`03o`000?oooooooooo06oooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo01oooooo000Nooooo`?oZJVY0_ooool0
0ol0003oooooooooo`0Looooo`;o0000L_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo043ooooo0onYZJTOooooo`007_ooool3ojVYZ@;ooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`1booooo`05o`000?ooooooooooooooool00000@oooool3ojVYZAoooooo000Noooo
o`?oZJVY0oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo07?ooooo00Co0000oooo
oooooooo0000A?ooool3ojVYZAoooooo000Kooooo`?oZJVY00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0Gooooo`03o`000?oooooooooo07Cooooo00?o0000ooooool00000Aoooool3o`00
01gooooo000Kooooo`?oZJVY1oooool00ol0003oooooooooo`0Eooooo`;o0000Moooool2o`0004So
oooo0ol0000Mooooo`006oooool3ojVYZ@Sooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`1fooooo`;o0000BOooool3o`0001gooooo000Looooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`1fooooo`03o`000?oooooo000004_ooooo00?o
0000ooooooooool06oooool001[ooooo0ol0000:ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool0M_ooool01?l0003oooooooooool0001;ooooo`?oZJVY6oooool001[ooooo0ol0000;
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0M_ooool01Ol0003ooooooooooooo
oooo000004_ooooo0onYZJTKooooo`006_ooool3o`0000cooooo00?o0000ooooooooool02oooool2
o`0007Sooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`19ooooo`?oZJVY6oooool0
01[ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo
07Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1;ooooo`03o`000?oooooooooo
01Wooooo000Hooooo`?oZJVY3oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo07Oo
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`1;ooooo`?o00006Oooool001Sooooo
0onYZJT?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0Moooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo04_ooooo0ol0000Iooooo`006?ooool3ojVYZA3ooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`1fooooo`;o00002?ooool00ol0003ooooooooo
o`1;ooooo`?o00006Oooool001Kooooo0onYZJTCooooo`03o`000?oooooooooo00;ooooo0_l0001h
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0COooool3ojVYZAOooooo000Foooo
o`?oZJVY4oooool01Ol0003ooooooooooooooooo000007Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`1=ooooo`?oZJVY5oooool001Kooooo0onYZJTDooooo`03o`000?oooooo0000
07Wooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1=ooooo`?oZJVY5oooool001Oo
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`1gooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0Fooooo`005Oooool3ojVYZAGooooo
0_l0001hooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0C_ooool3ojVYZAKooooo
000Eooooo`?oZJVY4oooool2o`0000;ooooo00?o0000ooooooooool0M?ooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo04kooooo0onYZJTFooooo`005Oooool3ojVYZA;ooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`1aooooo`;o00004?ooool00ol0003oooooooooo`1>
ooooo`?oZJVY5_ooool001Gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo073ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1@oooo
o`03o`000?oooooooooo01Cooooo000Cooooo`?oZJVY4_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo06kooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1@ooooo`?o
00005?ooool001?ooooo0onYZJTAooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
K?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo053ooooo0ol0000Dooooo`004ooo
ool3ojVYZ@oooooo0_l0000;ooooo`03o`000?oooooooooo06_ooooo00?o0000ooooooooool04ooo
ool00ol0003oooooooooo`1@ooooo`?o00005?ooool001Cooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo06Wooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`1Aooooo`03o`000?oooooooooo01?ooooo000Booooo`?oZJVY3_ooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo06Oooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`1Aooooo`?oZJVY4oooool001;ooooo0onYZJT=ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool0I_ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo057o
oooo0onYZJTCooooo`004_ooool3ojVYZ@cooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`1Sooooo`;o00006Oooool00ol0003oooooooooo`1Aooooo`?oZJVY4oooool001?ooooo00?o
0000ooooooooool02Oooool2o`0001Gooooo00?o0000ooooooooool0HOooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool04_ooool0017ooooo0onYZJT:
ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0H?ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo05;ooooo0ol0000Booooo`004Oooool3ojVYZ@Wooooo00?o0000oooo
ooooool05oooool00ol0003oooooooooo`1Nooooo`03o`000?oooooooooo01_ooooo00?o0000oooo
ooooool0D_ooool3o`0001;ooooo000Aooooo`?oZJVY2?ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo05cooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`1Booooo`?o
00004_ooool001;ooooo00?o0000ooooooooool01Oooool2o`0001cooooo00?o0000ooooooooool0
Foooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo05?ooooo00?o0000ooooooooool0
4Oooool000oooooo0onYZJT7ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0FOoo
ool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo05?ooooo0onYZJTAooooo`003oooool3
ojVYZ@Kooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooo
oooo01oooooo00?o0000ooooooooool0Doooool3ojVYZA7ooooo000?ooooo`?oZJVY1Oooool00ol0
003oooooooooo`0Pooooo`03o`000?oooooooooo05Gooooo0_l0000Rooooo`03o`000?oooooooooo
05?ooooo0onYZJTAooooo`004Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo02;o
oooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo05Go
oooo00?o0000ooooooooool04?ooool000oooooo0ol00002ooooo`;o00009Oooool00ol0003ooooo
ooooo`1Booooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0EOooool3ojVYZA3ooooo
000?ooooo`?o000000?oooooo`000?ooool09oooool00ol0003oooooooooo`1@ooooo`03o`000?oo
oooooooo02?ooooo00?o0000ooooooooool0EOooool3ojVYZA3ooooo000?ooooo`Co0000:_ooool0
0ol0003oooooooooo`1>ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0EOooool3
ojVYZA3ooooo000@ooooo`;o0000:oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0?ooooo`003_ooool3o`0002go
oooo00?o0000ooooooooool0Boooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo05Ko
oooo0onYZJT?ooooo`003_ooool3o`0002kooooo00?o0000ooooooooool0BOooool00ol0003ooooo
ooooo`0Wooooo`03o`000?oooooooooo05Kooooo0onYZJT?ooooo`003_ooool3o`0002kooooo00?o
0000ooooooooool0Aoooool2o`0002[ooooo00?o0000ooooooooool0E_ooool3ojVYZ@oooooo000?
ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`0Z
ooooo`03o`000?oooooooooo05Oooooo00?o0000ooooooooool03_ooool000gooooo0ol0000aoooo
o`03o`000?oooooooooo04?ooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1Goooo
o`?oZJVY3_ooool000gooooo0ol0000aooooo`03o`000?oooooooooo04;ooooo00?o0000oooooooo
ool0;?ooool00ol0003oooooooooo`1Gooooo`?oZJVY3_ooool000gooooo0ol0000booooo`03o`00
0?oooooooooo043ooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`1Gooooo`?oZJVY
3_ooool000kooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo02kooooo00?o0000ooooooooool0F?ooool00ol0003oooooooooo`0=ooooo`003?ooool3
ojVYZCCooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`0_ooooo`03o`000?oooooo
oooo05Oooooo0onYZJT>ooooo`003?ooool3ojVYZCGooooo00?o0000ooooooooool0>_ooool2o`00
03;ooooo00?o0000ooooooooool0Eoooool3ojVYZ@kooooo000<ooooo`?oZJVY=_ooool00ol0003o
ooooooooo`0hooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0Eoooool3ojVYZ@ko
oooo000>ooooo`03o`000?oooooooooo03Cooooo00?o0000ooooooooool0=oooool00ol0003ooooo
ooooo`0cooooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool03Oooool000cooooo0onY
ZJTgooooo`03o`000?oooooooooo03Gooooo00?o0000ooooooooool0=?ooool00ol0003ooooooooo
o`1Hooooo`?oZJVY3Oooool000cooooo0onYZJThooooo`03o`000?oooooooooo03?ooooo00?o0000
ooooooooool0=Oooool00ol0003oooooooooo`1Hooooo`?oZJVY3Oooool000cooooo0onYZJThoooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`1Hoooo
o`?oZJVY3Oooool000gooooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`0`ooooo`03
o`000?oooooooooo03Oooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0<ooooo`00
3?ooool3ojVYZC[ooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0hooooo`03o`00
0?oooooooooo05Sooooo0onYZJT=ooooo`003?ooool3ojVYZC[ooooo00?o0000ooooooooool0;?oo
ool2o`0003_ooooo00?o0000ooooooooool0F?ooool3ojVYZ@gooooo000<ooooo`?oZJVY>oooool0
0ol0003oooooooooo`0Zooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0F?ooool3
ojVYZ@gooooo000=ooooo`03o`000?oooooooooo03[ooooo00?o0000ooooooooool0:Oooool00ol0
003oooooooooo`0looooo`03o`000?oooooooooo05Wooooo00?o0000ooooooooool03?ooool000_o
oooo0onYZJTmooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool0?Oooool00ol0003o
ooooooooo`1Hooooo`?oZJVY3Oooool000_ooooo0onYZJTnooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`1Hooooo`?oZJVY3Oooool000_ooooo0onY
ZJTnooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0?oooool00ol0003ooooooooo
o`1Hooooo`?oZJVY3Oooool000gooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`0R
ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0<
ooooo`002oooool3ojVYZD3ooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`11oooo
o`03o`000?oooooooooo05Sooooo0ol0000=ooooo`002oooool3ojVYZD3ooooo00?o0000oooooooo
ool07_ooool2o`0004Cooooo00?o0000ooooooooool0F?ooool3o`0000gooooo000;ooooo`?oZJVY
@Oooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo04Cooooo00?o0000ooooooooool0
F?ooool3o`0000gooooo000=ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo05Wooooo00?o0000ooooooooool03?oo
ool000_ooooo0onYZJU2ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0A_ooool0
0ol0003oooooooooo`1Hooooo`?o00003Oooool000_ooooo0onYZJU3ooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`1Hooooo`?o00003Oooool000_o
oooo0onYZJU4ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0B?ooool00ol0003o
ooooooooo`1Hooooo`?o00003Oooool000gooooo00?o0000ooooooooool0@_ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool0FOooool00ol0003ooooo
ooooo`0<ooooo`003?ooool3ojVYZDCooooo00?o0000ooooooooool04_ooool00ol0003ooooooooo
o`1:ooooo`03o`000?oooooooooo05Sooooo0ol0000=ooooo`003?ooool3ojVYZ@[o0000>oooool0
0ol0003oooooooooo`0?ooooo`;o0000COooool00ol0003oooooooooo`1Hooooo`?o00003Oooool0
00cooooo0onYZJT:oooooaGo00009_ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
04gooooo00?o0000ooooooooool0F?ooool3o`0000gooooo000=ooooo`03o`000?oooooooooo01ko
oooo5_l0000Aooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0C_ooool00ol0003o
ooooooooo`1Iooooo`03o`000?oooooooooo00cooooo000<ooooo`?oZJVY=OoooolFo`0000Wooooo
00?o0000ooooooooool0Coooool00ol0003oooooooooo`1Hooooo`?oZJVY3Oooool000cooooo0onY
ZJU7ooooo`04o`000?oooooooooooooooaGo0000A_ooool00ol0003oooooooooo`1Hooooo`?oZJVY
3Oooool000cooooo0onYZJU8ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02ooo
oolFo`00033ooooo00?o0000ooooooooool0F?ooool3ojVYZ@gooooo000=ooooo`03o`000?oooooo
oooo04Sooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0RoooooaGo00006oooool0
0ol0003oooooooooo`1Iooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo
04Oooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0hoooooaKo00001Oooool00ol0
003oooooooooo`1Hooooo`03o`000?oooooooooo00gooooo000<ooooo`?oZJVYB_ooool01?l0003o
ooooooooooooool2o`00057ooooo5Ol0001:ooooo`?oZJVY3_ooool000cooooo0onYZJU;ooooo`03
o`000?oooooo000005Sooooo00?o0000ooooooooool03OoooolFo`0003Cooooo0onYZJT>ooooo`00
3?ooool3ojVYZD_ooooo0_l0001Iooooo`03o`000?oooooooooo02?ooooo5_l0000Nooooo`?oZJVY
3_ooool000kooooo00?o0000ooooooooool0BOooool2o`0005Wooooo00?o0000ooooooooool0>Ooo
oolEo`0000[ooooo00?o0000ooooooooool03Oooool000gooooo0onYZJU9ooooo`03o`000?oooooo
000005Wooooo00?o0000ooooooooool0C_ooool9o`0000?oZJVY3_ooool000gooooo0onYZJU8oooo
o`05o`000?ooooooooooooooool00000F?ooool00ol0003oooooooooo`1Gooooo`?oZJVY3_ooool0
00gooooo0onYZJU7ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0EOooool00ol0
003oooooooooo`1Gooooo`?oZJVY3_ooool000oooooo00?o0000ooooooooool0@oooool2o`0000Ko
oooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`1Gooooo`03o`000?oooooooooo00ko
oooo000>ooooo`?oZJVY@oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo05Cooooo
00?o0000ooooooooool0E_ooool3ojVYZ@oooooo000>ooooo`?oZJVY@_ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo05?ooooo00?o0000ooooooooool0E_ooool3ojVYZ@oooooo000>
ooooo`?oZJVY@Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo05?ooooo00?o0000
ooooooooool0E_ooool3ojVYZ@oooooo000@ooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo05Kooooo00?o0000oooo
ooooool03oooool000oooooo0onYZJTnooooo`03o`000?oooooooooo00kooooo00?o0000oooooooo
ool0DOooool00ol0003oooooooooo`1Eooooo`?oZJVY4?ooool000oooooo0onYZJTmooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool0DOooool00ol0003oooooooooo`1Eooooo`?oZJVY
4?ooool000oooooo0onYZJTlooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0D?oo
ool00ol0003oooooooooo`1Eooooo`?oZJVY4?ooool0017ooooo00?o0000ooooooooool0>?ooool2
o`0001Gooooo00?o0000ooooooooool0Coooool00ol0003oooooooooo`1Eooooo`03o`000?oooooo
oooo013ooooo000?ooooo`?oZJVY>Oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo
04oooooo00?o0000ooooooooool0E?ooool3ojVYZA7ooooo000?ooooo`?oZJVY>?ooool00ol0003o
ooooooooo`0Gooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0EOooool3ojVYZA7o
oooo000?ooooo`?oZJVY=oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo04cooooo
00?o0000ooooooooool0EOooool3ojVYZA7ooooo000Booooo`03o`000?oooooooooo03?ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`1<ooooo`03o`000?oooooooooo05Gooooo00?o
0000ooooooooool04Oooool0017ooooo0onYZJTcooooo`03o`000?oooooooooo01cooooo00?o0000
ooooooooool0Boooool00ol0003oooooooooo`1Dooooo`?oZJVY4_ooool0017ooooo0onYZJTboooo
o`03o`000?oooooooooo01kooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`1Doooo
o`?oZJVY4_ooool0017ooooo0onYZJTaooooo`03o`000?oooooooooo01oooooo00?o0000oooooooo
ool0B_ooool00ol0003oooooooooo`1Dooooo`?oZJVY4_ooool001;ooooo0onYZJT^ooooo`;o0000
8oooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo05?ooooo0onYZJTCooooo`004_oo
ool3ojVYZBgooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`18ooooo`03o`000?oo
oooooooo05?ooooo0onYZJTCooooo`004_ooool3ojVYZBcooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`18ooooo`03o`000?oooooooooo05?ooooo0onYZJTCooooo`004oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool0Aoooool00ol0
003oooooooooo`1Dooooo`03o`000?oooooooooo01;ooooo000Cooooo`?oZJVY:Oooool00ol0003o
ooooooooo`0Xooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool0D_ooool3ojVYZACo
oooo000Cooooo`?oZJVY:?ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo04Kooooo
00?o0000ooooooooool0D_ooool3ojVYZACooooo000Cooooo`?oZJVY9oooool00ol0003ooooooooo
o`0/ooooo`03o`000?oooooooooo04Gooooo00?o0000ooooooooool0D_ooool3ojVYZACooooo000E
ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`15
ooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool05?ooool001Gooooo0onYZJTRoooo
o`;o0000<Oooool00ol0003oooooooooo`14ooooo`03o`000?oooooooooo053ooooo0ol0000Foooo
o`005Oooool3ojVYZB7ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`13ooooo`03
o`000?oooooooooo053ooooo0ol0000Fooooo`005Oooool3ojVYZB3ooooo00?o0000ooooooooool0
<oooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo053ooooo0ol0000Fooooo`005_oo
ool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo03Gooooo00?o0000ooooooooool0@_oo
ool00ol0003oooooooooo`1Aooooo`03o`000?oooooooooo01Gooooo000Fooooo`?oZJVY7Oooool0
0ol0003oooooooooo`0gooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool0Coooool3
ojVYZAOooooo000Fooooo`?oZJVY7?ooool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo
047ooooo00?o0000ooooooooool0Coooool3ojVYZAOooooo000Fooooo`?oZJVY6oooool00ol0003o
ooooooooo`0jooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool0Coooool3ojVYZAOo
oooo000Hooooo`03o`000?oooooooooo01Oooooo0_l0000nooooo`03o`000?oooooooooo03oooooo
00?o0000ooooooooool0Coooool00ol0003oooooooooo`0Gooooo`006?ooool3ojVYZAKooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo04gooooo0onY
ZJTIooooo`006?ooool3ojVYZAGooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo04gooooo0onYZJTIooooo`006?ooool3ojVYZACooooo00?o0000oooo
ooooool0@_ooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo04gooooo0onYZJTIoooo
o`006_ooool3ojVYZA7ooooo00?o0000ooooooooool0@oooool00ol0003oooooooooo`0mooooo`03
o`000?oooooooooo04_ooooo0onYZJTKooooo`006_ooool3ojVYZA3ooooo00?o0000ooooooooool0
AOooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo04_ooooo0onYZJTKooooo`006_oo
ool3ojVYZ@oooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`0kooooo`03o`000?oo
oooooooo04_ooooo0onYZJTKooooo`007?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo04Sooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`1;ooooo`03o`000?oooooo
oooo01_ooooo000Kooooo`?oZJVY2oooool2o`0004cooooo00?o0000ooooooooool0>_ooool00ol0
003oooooooooo`19ooooo`?o00007Oooool001_ooooo0onYZJT:ooooo`03o`000?oooooooooo04go
oooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`19ooooo`?o00007Oooool001_ooooo
0onYZJT9ooooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool0>Oooool00ol0003ooooo
ooooo`19ooooo`?o00007Oooool001kooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`1@ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0BOooool00ol0003ooooooooo
o`0Mooooo`007_ooool3ojVYZ@Cooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0g
ooooo`03o`000?oooooooooo04Oooooo0onYZJTOooooo`007_ooool3ojVYZ@?ooooo00?o0000oooo
ooooool0Doooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo04Oooooo0onYZJTOoooo
o`007_ooool3ojVYZ@;ooooo00?o0000ooooooooool0EOooool00ol0003oooooooooo`0fooooo`03
o`000?oooooooooo04Oooooo0onYZJTOooooo`008?ooool3ojVYZESooooo00?o0000ooooooooool0
=_ooool00ol0003oooooooooo`15ooooo`?oZJVY8Oooool0023ooooo0onYZJUIooooo`03o`000?oo
oooooooo03Gooooo00?o0000ooooooooool0AOooool3ojVYZB7ooooo000Pooooo`?oZJVYF_ooool0
0ol0003oooooooooo`0dooooo`03o`000?oooooooooo04Gooooo0onYZJTQooooo`008_ooool00ol0
003oooooooooo`1Hooooo`03o`000?oooooooooo03Cooooo00?o0000ooooooooool0AOooool00ol0
003oooooooooo`0Qooooo`008_ooool3ojVYZEWooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`13ooooo`?oZJVY8oooool002;ooooo0onYZJUJooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0@oooool3ojVYZB?ooooo000Rooooo`?oZJVYF_ooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo04?ooooo0onYZJTSooooo`009?ooool3ojVYZEWooooo00?o0000
ooooooooool0<Oooool00ol0003oooooooooo`10ooooo`?oZJVY00?o0000ooooooooool08oooool0
02Cooooo0onYZJUJooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool0@?ooool3ojVY
ZBKooooo000Tooooo`?oZJVYF_ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo043o
oooo0onYZJTVooooo`009oooool3ojVYZESooooo00?o0000ooooooooool0;oooool00ol0003ooooo
ooooo`0nooooo`?o0000:?ooool002Oooooo0onYZJUIooooo`03o`000?oooooooooo02kooooo00?o
0000ooooooooool0?_ooool3o`0002Sooooo000Wooooo`?oZJVYFOooool00ol0003oooooooooo`0^
ooooo`03o`000?oooooooooo03kooooo0ol0000Xooooo`00:Oooool3ojVYZESooooo00?o0000oooo
ooooool0;Oooool00ol0003oooooooooo`0kooooo`Co0000:_ooool002Wooooo0onYZJUIooooo`03
o`000?oooooooooo02cooooo00?o0000ooooooooool0>oooool3o`0002_ooooo000Yooooo`?oZJVY
FOooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo03_ooooo0ol0000[ooooo`00;?oo
ool3ojVYZEOooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0iooooo`?o0000;Ooo
ool002cooooo0onYZJUHooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0>Oooool3
o`0002gooooo000/ooooo`?oZJVYF?ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo
03Wooooo0ol0000]ooooo`00;oooool3ojVYZEKooooo00?o0000ooooooooool0:Oooool00ol0003o
ooooooooo`0fooooo`?o0000<?ooool002oooooo0onYZJUGooooo`03o`000?oooooooooo02Sooooo
00?o0000ooooooooool0=_ooool3o`00033ooooo000_ooooo`?oZJVYEoooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo03Kooooo0ol0000`ooooo`00<_ooool3ojVYZEGooooo00?o0000
ooooooooool09oooool00ol0003oooooooooo`0cooooo`?oZJVY<oooool003;ooooo0onYZJUFoooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0<oooool3ojVYZC?ooooo000booooo`?o
ZJVYE_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo03?ooooo0onYZJTcooooo`00
=Oooool3ojVYZECooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0`ooooo`?o0000
=_ooool003Gooooo0onYZJUDooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0<Ooo
ool3o`0003Kooooo000eooooo`?oZJVYEOooool00ol0003oooooooooo`0Sooooo`03o`000?oooooo
oooo037ooooo0ol0000fooooo`00>?ooool3ojVYZE?ooooo00?o0000ooooooooool08_ooool00ol0
003oooooooooo`0^ooooo`?oZJVY>Oooool003Sooooo0onYZJUCooooo`03o`000?oooooooooo02;o
oooo00?o0000ooooooooool0;_ooool3ojVYZCWooooo000hooooo`?oZJVYE?ooool00ol0003ooooo
ooooo`0Qooooo`03o`000?oooooooooo02kooooo0onYZJTiooooo`00>oooool3o`0005;ooooo00?o
0000ooooooooool08?ooool00ol0003oooooooooo`0[ooooo`?oZJVY??ooool003_ooooo0ol0001B
ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0:oooool3ojVYZCcooooo000koooo
o`?o00000onYZJU@ooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0:?ooool3o`00
00?oZJVY??ooool003kooooo0onYZJUAooooo`03o`000?oooooooooo01kooooo00?o0000oooooooo
ool0:?ooool3o`0003oooooo000nooooo`?oZJVYDOooool00ol0003oooooooooo`0Nooooo`03o`00
0?oooooooooo02Sooooo0ol0000oooooo`00@Oooool3ojVYZDoooooo00?o0000ooooooooool07Ooo
ool00ol0003oooooooooo`0Tooooo`?oZJVY00?o0000ooooooooool0@?ooool0047ooooo0onYZJU@
ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool09?ooool3ojVYZD?ooooo0011oooo
o`KoZJVYCOooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo027ooooo1_nYZJU3oooo
o`00A?ooool3ojVYZ@03o`000?oooooooooo04_ooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`0Qooooo`?oZJVYA_ooool004Cooooo0onYZJT01?ooooooZJVYojVYZOnYZJU;ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool07_ooool6ojVYZDKooooo0018ooooo`?oZJVY
Boooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01kooooo0onYZJU9ooooo`00B?oo
ool3ojVYZ@?o0000BOooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo0onY
ZJT01?l0003oZJVYojVYZOnYZJU9ooooo`00Boooool4o`0004Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Jooooo`?oZJVYCOooool004_ooooo0ol000000ooooooo0000ooooo`17
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool01?l0003oZJVYojVYZOnY
ZJU=ooooo`00Coooool3o`0004Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0G
ooooo`?oZJVYD?ooool004oooooo1?l00003ojVYZDCooooo00?o0000ooooooooool05_ooool00ol0
003oooooooooo`0Cooooo`Co00000onYZJU@ooooo`00Coooool3o`000004oooooonYZJWoZJVYojVY
ZDCooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Cooooo`?o000000CoooooojVY
ZOnYZJWoZJVYD?ooool005?ooooo1_nYZJU2ooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool03oooool3ojVYZ@Co0000E?ooool005Kooooo0onYZJU2ooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool03oooool3ojVYZESooooo001Fooooo`?oZJVY1?l0000oooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool03?ooool6ojVYZESooooo001Jooooo`Co00000onY
ZJTlooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02?ooool4o`0000?oZJVYFooo
ool005[ooooo0ol000001?ooooooZJVYojVYZOnYZJTlooooo`03o`000?oooooooooo01?ooooo00?o
0000ooooooooool02?ooool3o`000004oooooonYZJWoZJVYojVYZE_ooooo001Nooooo`?oZJVY00Co
0000ojVYZOnYZJWoZJVY>Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Cooooo
1ol0001Oooooo`00H_ooool3ojVYZ@04o`000?nYZJWoZJVYojVYZCKooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`03ojVYZ@Co0000Hoooool006;ooooo0onYZJT00oooooooZJVYojVY
Z@04ojVYZC?ooooo00?o0000ooooooooool04?ooool3ojVYZ@05o`000?nYZJWoZJVYojVYZOooool0
0ol0001Sooooo`00I_ooool6ojVYZ@;o0000<_ooool00ol0003oooooooooo`0>ooooo`05o`000?nY
ZJWoZJVYojVYZOooool00onYZJUWooooo`00JOooool3ojVYZ@04oooooonYZJWoZJVYojVYZ@Co0000
;Oooool00ol0003oooooooooo`06ooooo`?oZJVY00Go0000ojVYZOnYZJWoZJVYooooo`03ojVYZF_o
oooo001]ooooo`?oZJVY00?oooooo`000?l000000_l00003ojVYZBWooooo00?o0000ooooooooool0
0_ooool4o`0000?oZJVY00CoooooojVYZOnYZJWoZJVYKoooool006gooooo0onYZJT01Ooooooo0000
o`000?l0003ooooo00?oZJVY00Co0000ojVYZOnYZJWoZJVY9_ooool3ojVYZ@Co000000GoooooojVY
ZOnYZJWoZJVYooooo`03ojVYZFoooooo001eooooo`?oZJVY00GoooooojVYZOnYZJWoZJVYo`000003
ojVYZ@Go000000CoooooojVYZOnYZJWoZJVY3Oooool3o`000005oooooonYZJWoZJVYojVYZOl00000
0onYZJT01Ol0003oZJVYojVYZOnYZJWooooo00?o0000Moooool007Wooooo0onYZJT01?ooooooZJVY
ojVYZOnYZJT2ooooo`Co00000onYZJT01Ol0003oZJVYojVYZOnYZJWo000000?oZJVY00Co0000ojVY
ZOnYZJWoZJVY1Ol00003ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onYZJUkooooo`00OOooool3
ojVYZ@;ooooo0ol000001OooooooZJVYojVYZOnYZJWooooo00?oZJVY00GoooooojVYZOnYZJWoZJVY
ooooo`03ojVYZ@05ooooool0003o0000o`000?ooool00onYZJT01?ooooooZJVYojVYZOnYZJUooooo
o`00R_ooool3ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onYZJV;ooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(Evolve[pd2, Generations \[Rule] 50]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[pd2], 
        VertexStyle \[Rule] Disk[Small]];\)\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94895 .57111 m
.04635 .52854 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.62681 .9365 m
.64046 .0677 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.57111 .94895 m
.11621 .25644 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.11621 .74356 m
.78974 .14977 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.05351 .41483 m
.5 .04545 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.012 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.663 .663 .663 r
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
0 0 0 r
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.663 .663 .663 r
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
0 0 0 r
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.663 .663 .663 r
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
0 0 0 r
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[27]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00P_ooool3o`000005ooooool0003o0000o`000?oo
ool00ol000001Ooooooo0000o`000?l0003ooooo00?o000000Goooooo`000?l0003o0000ooooo`03
o`0008?ooooo001iooooo`?o000000Goooooo`000?l0003o0000ooooo`0Po`000004ooooool0003o
0000o`0007_ooooo001eooooo`?o000000?oooooo`000?l000001_l000001Ooooooo0000o`000?l0
003ooooo00?o000000?oooooo`000?l000001Ol000001Ooooooo0000o`000?l0003ooooo00?o0000
00Goooooo`000?l0003o0000ooooo`0;o`0007Oooooo001aooooo`?o000000?oooooo`000?l00000
1Ol000001?oooooo0000o`000?l0000<ooooo`03o`000?oooooooooo00oooooo0ol000001Ooooooo
0000o`000?l0003ooooo00Oo0000Loooool006gooooo0ol000000ooooooo0000o`000005o`0001?o
oooo00?o0000ooooooooool06?ooool3o`000003ooooool0003o000000Go0000Koooool006Wooooo
0ol000000ooooooo0000o`000005o`0001Kooooo00?o0000ooooooooool07Oooool3o`000003oooo
ool0003o000000Go0000Joooool006Kooooo2_l0000Hooooo`;o00009?ooool3o`000003ooooool0
003o000000Go0000Ioooool006;ooooo2_l0000Kooooo`03o`000?oooooooooo02Sooooo0ol00000
0ooooooo0000o`000005o`0006?ooooo001Rooooo`?o000000Coooooo`000?l0003o00007Oooool0
0ol0003oooooooooo`0]ooooo`?o000000Coooooo`000?l0003o0000Hoooool005kooooo1ol0000P
ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool01ol0001Oooooo`00F_ooool7o`00
02?ooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`Oo0000Foooool005[o
oooo0ol000001?oooooo0000o`000?l0000Qooooo`;o0000<oooool00ol0003oooooooooo`04oooo
o`?o000000Coooooo`000?l0003o0000Foooool005Kooooo1ol0000Tooooo`03o`000?oooooooooo
03?ooooo00?o0000ooooooooool02?ooool6o`0005Sooooo001Fooooo`?o00009oooool00ol0003o
ooooooooo`0dooooo`03o`000?oooooooooo00_ooooo1?l0001Gooooo`00Doooool6o`0002Kooooo
00?o0000ooooooooool0=Oooool00ol0003oooooooooo`0;ooooo`?o000000Coooooo`000?l0003o
0000E?ooool005?ooooo0ol0000Wooooo`;o0000>?ooool00ol0003oooooooooo`0?ooooo`?o0000
E?ooool004oooooo1ol0000Vooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool03ooo
ool7o`00053ooooo001?ooooo`?o0000:Oooool00ol0003oooooooooo`0iooooo`03o`000?oooooo
oooo01?ooooo0ol0001@ooooo`00Boooool7o`0002Sooooo00?o0000ooooooooool0>_ooool00ol0
003oooooooooo`0Cooooo`Ko0000COooool004_ooooo0ol0000[ooooo`03o`000?oooooooooo03_o
oooo00?o0000ooooooooool05_ooool3o`0004gooooo0018ooooo`Ko0000:Oooool2o`0003kooooo
00?o0000ooooooooool05_ooool7o`0004Wooooo0018ooooo`?o0000:oooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo01[ooooo0ol00019ooooo`00A?ooool7o`0002[ooooo00?o0000
ooooooooool0?oooool00ol0003oooooooooo`0Jooooo`Ko0000A_ooool004Cooooo0ol0000]oooo
o`03o`000?oooooooooo043ooooo00?o0000ooooooooool07Oooool3o`0004Kooooo0013ooooo`Co
0000;?ooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01gooooo1Ol00014ooooo`00
@Oooool3o`0002gooooo0_l00014ooooo`03o`000?oooooooooo023ooooo0onYZJU3ooooo`00@Ooo
ool3o`0002cooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`0Pooooo`?oZJVY00?o
0000ooooooooool0@?ooool003kooooo1_l0000[ooooo`03o`000?oooooooooo04Gooooo00?o0000
ooooooooool08?ooool3ojVYZ@04oooooonYZJWoZJVYojVYZCoooooo000nooooo`?o0000;Oooool0
0ol0003oooooooooo`16ooooo`03o`000?oooooooooo02Cooooo0onYZJToooooo`00?Oooool4o`00
02cooooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`0Tooooo`?oZJVY00?o0000oooo
ooooool0??ooool003_ooooo0ol0000]ooooo`;o0000B_ooool00ol0003oooooooooo`0Wooooo`?o
ZJVY??ooool003_ooooo0ol0000/ooooo`03o`000?oooooooooo04[ooooo00?o0000ooooooooool0
9_ooool01?l0003oZJVYojVYZOnYZJTlooooo`00>?ooool6o`0002_ooooo00?o0000ooooooooool0
Boooool00ol0003oooooooooo`0Uooooo`03o`000?ooooooZJVY00GoZJVY>Oooool003Sooooo0ol0
000]ooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool09?ooool00ol0003ooooooooo
o`03ooooo`?oZJVY>Oooool003Sooooo0ol0000/ooooo`03o`000?oooooooooo04gooooo00?o0000
ooooooooool08oooool00ol0003oooooooooo`04ooooo`?oZJVY>Oooool003Gooooo0ol0000]oooo
o`;o0000D?ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00Sooooo0onYZJTfoooo
o`00=Oooool3o`0002cooooo00?o0000ooooooooool0D?ooool00ol0003oooooooooo`0Qooooo`03
o`000?oooooooooo00Wooooo0onYZJTfooooo`00=Oooool3o`0002_ooooo00?o0000ooooooooool0
DOooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00[ooooo0onYZJTfooooo`00<_oo
ool3o`0002gooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0Nooooo`;o00004?oo
ool3o`0003?ooooo000booooo`?o0000;?ooool00ol0003oooooooooo`1Booooo`03o`000?oooooo
oooo01kooooo00?o0000ooooooooool04?ooool3o`0003?ooooo000booooo`?o0000:_ooool2o`00
05Gooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0Aooooo`?o0000<oooool002oo
oooo0ol0000/ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool07?ooool00ol0003o
ooooooooo`0Eooooo`?o0000<?ooool002oooooo0ol0000[ooooo`03o`000?oooooooooo05Kooooo
00?o0000ooooooooool06oooool00ol0003oooooooooo`0Fooooo`?o0000<?ooool002oooooo0ol0
000Zooooo`03o`000?oooooooooo05Oooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Gooooo`?o0000<?ooool002cooooo0ol0000[ooooo`;o0000F_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01_ooooo0ol0000]ooooo`00;?ooool3o`0002[ooooo00?o0000oooo
ooooool0F_ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01cooooo0ol0000]oooo
o`00;?ooool3o`0002Wooooo00?o0000ooooooooool0Foooool00ol0003oooooooooo`0Fooooo`;o
00007oooool3o`0002gooooo000Yooooo`?o0000:oooool00ol0003oooooooooo`1Looooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool08Oooool3o`0002_ooooo000Yooooo`?o0000:_oo
ool00ol0003oooooooooo`1Mooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool08_oo
ool3o`0002_ooooo000Yooooo`?o0000:?ooool2o`00063ooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`0Sooooo`?o0000:oooool002Oooooo0ol0000Yooooo`03o`000?oooooooooo
063ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Wooooo`?o0000:?ooool002Oo
oooo0ol0000Xooooo`03o`000?oooooooooo067ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0Xooooo`?o0000:?ooool002Oooooo0ol0000Wooooo`03o`000?oooooooooo06;ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Yooooo`?o0000:?ooool002Kooooo00?o
0000ooooooooool09oooool00ol0003oooooooooo`1Sooooo`03o`000?oooooooooo00kooooo0_l0
000_ooooo`03o`000?oooooooooo02Gooooo000Tooooo`?o00009oooool2o`0006Kooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0^ooooo`?o00009_ooool002Cooooo0ol0000Voooo
o`03o`000?oooooooooo06Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0_oooo
o`?o00009_ooool002Cooooo0ol0000Uooooo`03o`000?oooooooooo06Oooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0`ooooo`Co00009Oooool002;ooooo0ol0000Vooooo`03o`00
0?oooooooooo06Sooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0dooooo`?o0000
8oooool002;ooooo0ol0000Uooooo`03o`000?oooooooooo06Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0eooooo`?o00008oooool002;ooooo0ol0000Sooooo`;o0000K?ooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo03Kooooo0ol0000Sooooo`008?ooool3o`00
02Cooooo00?o0000ooooooooool0K?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
03Wooooo0ol0000Qooooo`008?ooool3o`0002?ooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`05ooooo`;o0000??ooool3o`00027ooooo000Pooooo`?o00008_ooool00ol0003ooooo
ooooo`1^ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0??ooool3o`00027ooooo
000Pooooo`03o`000?oooooo0000027ooooo00?o0000ooooooooool0Koooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool07oooool001kooooo0ol00002
ooooo`03o`000?oooooooooo01cooooo0_l0001booooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool0@?ooool3o`0001oooooo000Nooooo`?o00000_ooool00ol0003oooooooooo`0Koooo
o`03o`000?oooooooooo07;ooooo00Go0000ooooooooooooooooo`000013ooooo`?o00007oooool0
01kooooo0ol00003ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0Loooool01?l0
003oooooooooool00014ooooo`?o00007oooool001_ooooo1?l00006ooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool0M?ooool00ol0003oooooo`000017ooooo`?o00007Oooool001_o
oooo0ol00007ooooo`03o`000?oooooooooo01Gooooo0_l0001gooooo`;o0000B?ooool3o`0001go
oooo000Kooooo`?o00002?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo07Kooooo
0_l00019ooooo`?o00007Oooool001cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo07Kooooo00?o0000ooooool00000Boooool00ol0003ooooooooo
o`0Kooooo`006_ooool3o`0000[ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1f
ooooo`04o`000?ooooooooooo`0004_ooooo0ol0000Kooooo`006_ooool3o`0000_ooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`1fooooo`05o`000?ooooooooooooooool00000Booo
ool3o`0001_ooooo000Jooooo`?o00003?ooool00ol0003oooooooooo`0;ooooo`;o0000N?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo04Wooooo0ol0000Kooooo`006_ooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0Moooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool06Oooool001So
oooo0ol0000?ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0Moooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo04_ooooo0ol0000Iooooo`006?ooool3o`0000oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`1gooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0Boooool3o`0001Wooooo000Hooooo`?o00004?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo07Kooooo0_l00008ooooo`03o`000?oooooooooo04_ooooo0ol0
000Iooooo`005_ooool3o`0001?ooooo00?o0000ooooooooool00_ooool2o`0007Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`1=ooooo`?o00005oooool001Kooooo0ol0000Coooo
o`05o`000?ooooooooooooooool00000NOooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo04gooooo0ol0000Gooooo`005_ooool3o`0001Cooooo00?o0000ooooool00000NOooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo04gooooo0ol0000Gooooo`005oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo07Oooooo00?o0000ooooooooool02oooool00ol0003o
ooooooooo`1>ooooo`03o`000?oooooooooo01Kooooo000Eooooo`?o00005Oooool2o`0007Sooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`1>ooooo`?o00005_ooool001Gooooo0ol0
000Cooooo`;o00000_ooool00ol0003oooooooooo`1dooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool0C_ooool3o`0001Kooooo000Eooooo`?o00004_ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo077ooooo0_l0000@ooooo`03o`000?oooooooooo04kooooo0ol0000F
ooooo`005Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool0L?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo053ooooo00?o0000oooo
ooooool05?ooool001?ooooo0ol0000Booooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0K_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo053ooooo0ol0000Dooooo`00
4oooool3o`00017ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`1/ooooo`03o`00
0?oooooooooo01;ooooo00?o0000ooooooooool0D?ooool3o`0001Cooooo000Cooooo`?o00003ooo
ool2o`0000_ooooo00?o0000ooooooooool0Joooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo053ooooo0ol0000Dooooo`005?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool0JOooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo057ooooo00?o0000ooooooooool04oooool001;ooooo0ol0000>ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool0Ioooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo
057ooooo0ol0000Cooooo`004_ooool3o`0000gooooo00?o0000ooooooooool03oooool00ol0003o
ooooooooo`1Vooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0DOooool3o`0001?o
oooo000Booooo`?o00003?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo06?ooooo
0_l0000Iooooo`03o`000?oooooooooo057ooooo0ol0000Cooooo`004oooool00ol0003ooooooooo
o`09ooooo`;o00005Oooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool0D_ooool00ol0003oooooooooo`0Booooo`004Oooool3o`0000[ooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool0D_ooool3o`0001;ooooo000Aooooo`?o00002Oooool00ol0003oooooooooo`0Goooo
o`03o`000?oooooooooo05kooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`1Boooo
o`?o00004_ooool0017ooooo0ol00008ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool0G?ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo05;ooooo0ol0000Booooo`00
4_ooool00ol0003oooooooooo`05ooooo`;o00007?ooool00ol0003oooooooooo`1Kooooo`03o`00
0?oooooooooo01gooooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`0Aooooo`003ooo
ool3o`0000Oooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`1Iooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool0Doooool3o`00017ooooo000?ooooo`?o00001_ooool0
0ol0003oooooooooo`0Nooooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool07oooool0
0ol0003oooooooooo`1Cooooo`?o00004Oooool000oooooo0ol00005ooooo`03o`000?oooooooooo
023ooooo00?o0000ooooooooool0EOooool2o`0002;ooooo00?o0000ooooooooool0Doooool3o`00
017ooooo000Aooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool08_ooool00ol0003o
ooooooooo`1Cooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0EOooool00ol0003o
ooooooooo`0@ooooo`003oooool3o`0000;ooooo0_l0000Uooooo`03o`000?oooooooooo05;ooooo
00?o0000ooooooooool08_ooool00ol0003oooooooooo`1Eooooo`?o00004?ooool000oooooo0ol0
00000ooooooo0000ooooo`0Wooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool08ooo
ool00ol0003oooooooooo`1Eooooo`?o00004?ooool000oooooo1?l0000Zooooo`03o`000?oooooo
oooo04kooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`1Eooooo`?o00004?ooool0
013ooooo0_l0000[ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1Fooooo`03o`000?oooooooooo00oooooo000>ooooo`?o0000;Oooool00ol0003o
ooooooooo`1;ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0E_ooool3o`0000oo
oooo000>ooooo`?o0000;_ooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo02Oooooo
00?o0000ooooooooool0E_ooool3o`0000oooooo000>ooooo`?o0000;_ooool00ol0003ooooooooo
o`17ooooo`;o0000:_ooool00ol0003oooooooooo`1Fooooo`?o00003oooool000oooooo00?o0000
ooooooooool0;_ooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo02[ooooo00?o0000
ooooooooool0Eoooool00ol0003oooooooooo`0>ooooo`003Oooool3o`00037ooooo00?o0000oooo
ooooool0@oooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo05Oooooo0ol0000>oooo
o`003Oooool3o`00037ooooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`0/ooooo`03
o`000?oooooooooo05Oooooo0ol0000>ooooo`003Oooool3o`0003;ooooo00?o0000ooooooooool0
@?ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo05Oooooo0ol0000>ooooo`003_oo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`1Hooooo`03o`000?oooooooooo00gooooo000<ooooo`?o0000=?ooool0
0ol0003oooooooooo`0mooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool0Eoooool3
o`0000kooooo000<ooooo`?o0000=Oooool00ol0003oooooooooo`0jooooo`;o0000<_ooool00ol0
003oooooooooo`1Gooooo`?o00003_ooool000cooooo0ol0000fooooo`03o`000?oooooooooo03So
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`1Gooooo`?o00003_ooool000kooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0F?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`0003Oooooo00?o
0000ooooooooool0=Oooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo05Sooooo0ol0
000=ooooo`003?ooool3o`0003Sooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0e
ooooo`03o`000?oooooooooo05Sooooo0ol0000=ooooo`003?ooool3o`0003Sooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo05Sooooo0ol0000=oooo
o`003Oooool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo033ooooo00?o0000oooooooo
ool0=oooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo00cooooo000<ooooo`?o0000
>_ooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0
F?ooool3o`0000gooooo000<ooooo`?o0000>_ooool00ol0003oooooooooo`0/ooooo`;o0000>ooo
ool00ol0003oooooooooo`1Hooooo`?o00003Oooool000cooooo0ol0000kooooo`03o`000?oooooo
oooo02[ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`1Hooooo`?o00003Oooool0
00gooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo
03cooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0<ooooo`002oooool3o`0003go
oooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo05So
oooo0ol0000=ooooo`002oooool3o`0003kooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo05Sooooo0ol0000=ooooo`002oooool3o`0003kooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo05Sooooo0ol0
000=ooooo`003Oooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo02;ooooo00?o0000
ooooooooool0@?ooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo00cooooo000;oooo
o`?oZJVY@?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo047ooooo00?o0000oooo
ooooool0F?ooool3o`0000gooooo000;ooooo`?oZJVY@?ooool00ol0003oooooooooo`0Nooooo`;o
0000A?ooool00ol0003oooooooooo`1Hooooo`?o00003Oooool000_ooooo0onYZJU1ooooo`03o`00
0?oooooooooo01cooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`1Hooooo`?o0000
3Oooool000gooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo04Gooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0<ooooo`002oooool3
ojVYZD;ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`16ooooo`03o`000?oooooo
oooo05Sooooo0ol0000=ooooo`002oooool3ojVYZD?ooooo00?o0000ooooooooool05oooool00ol0
003oooooooooo`17ooooo`03o`000?oooooooooo05Sooooo0ol0000=ooooo`002oooool3ojVYZDCo
oooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo05So
oooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool0BOooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo00cooooo
000<ooooo`?oZJVYA?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo04[ooooo00?o
0000ooooooooool0F?ooool3o`0000gooooo000<ooooo`?oZJVY2_l0000kooooo`03o`000?oooooo
oooo00oooooo0_l0001=ooooo`03o`000?oooooooooo05Sooooo0ol0000=ooooo`003?ooool3ojVY
Z@[ooooo5Ol0000Vooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0COooool00ol0
003oooooooooo`1Hooooo`?o00003Oooool000gooooo00?o0000ooooooooool07_oooolFo`00017o
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo05Wo
oooo00?o0000ooooooooool03?ooool000cooooo0onYZJTeoooooaKo00002Oooool00ol0003ooooo
ooooo`1?ooooo`03o`000?oooooooooo05Sooooo0onYZJT=ooooo`003?ooool3ojVYZDOooooo00Co
0000oooooooooooooooo5Ol00016ooooo`03o`000?oooooooooo05Sooooo0onYZJT=ooooo`003?oo
ool3ojVYZDSooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0;oooooaKo0000<?oo
ool00ol0003oooooooooo`1Hooooo`?oZJVY3Oooool000gooooo00?o0000ooooooooool0B?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo02;ooooo5Ol0000Kooooo`03o`000?oooooo
oooo05Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool0Aoooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo03Sooooo5_l00005ooooo`03o`000?oooooooooo
05Sooooo00?o0000ooooooooool03Oooool000cooooo0onYZJU:ooooo`04o`000?oooooooooooooo
o`;o0000DOoooolEo`0004[ooooo0onYZJT>ooooo`003?ooool3ojVYZD_ooooo00?o0000ooooool0
0000F?ooool00ol0003oooooooooo`0=oooooaKo0000=?ooool3ojVYZ@kooooo000<ooooo`?oZJVY
Boooool2o`0005Wooooo00?o0000ooooooooool08ooooolFo`0001kooooo0onYZJT>ooooo`003_oo
ool00ol0003oooooooooo`19ooooo`;o0000FOooool00ol0003oooooooooo`0ioooooaGo00002_oo
ool00ol0003oooooooooo`0=ooooo`003Oooool3ojVYZDWooooo00?o0000ooooool00000FOooool0
0ol0003oooooooooo`1>ooooo`Wo00000onYZJT>ooooo`003Oooool3ojVYZDSooooo00Go0000oooo
ooooooooooooo`00001Hooooo`03o`000?oooooooooo05Oooooo0onYZJT>ooooo`003Oooool3ojVY
ZDOooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo
05Oooooo0onYZJT>ooooo`003oooool00ol0003oooooooooo`13ooooo`;o00001_ooool00ol0003o
ooooooooo`1Eooooo`03o`000?oooooooooo05Oooooo00?o0000ooooooooool03_ooool000kooooo
0onYZJU3ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0E?ooool00ol0003ooooo
ooooo`1Fooooo`?oZJVY3oooool000kooooo0onYZJU2ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool0Doooool00ol0003oooooooooo`1Fooooo`?oZJVY3oooool000kooooo0onYZJU1
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`1F
ooooo`?oZJVY3oooool0013ooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo05;ooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0?oooo
o`003oooool3ojVYZCkooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Aooooo`03
o`000?oooooooooo05Gooooo0onYZJT@ooooo`003oooool3ojVYZCgooooo00?o0000ooooooooool0
3oooool00ol0003oooooooooo`1Aooooo`03o`000?oooooooooo05Gooooo0onYZJT@ooooo`003ooo
ool3ojVYZCcooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1@ooooo`03o`000?oo
oooooooo05Gooooo0onYZJT@ooooo`004Oooool00ol0003oooooooooo`0hooooo`;o00005Oooool0
0ol0003oooooooooo`1?ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool04?ooool0
00oooooo0onYZJTiooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0Coooool00ol0
003oooooooooo`1Dooooo`?o00004Oooool000oooooo0onYZJThooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool0COooool00ol0003oooooooooo`1Eooooo`?o00004Oooool000oooooo
0onYZJTgooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0C?ooool00ol0003ooooo
ooooo`1Eooooo`?o00004Oooool001;ooooo00?o0000ooooooooool0<oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool0EOooool00ol0003ooooooooo
o`0Aooooo`004Oooool3ojVYZC?ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`1;
ooooo`03o`000?oooooooooo05Cooooo0ol0000Booooo`004Oooool3ojVYZC;ooooo00?o0000oooo
ooooool07_ooool00ol0003oooooooooo`1:ooooo`03o`000?oooooooooo05Cooooo0ol0000Boooo
o`004Oooool3ojVYZC7ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`1:ooooo`03
o`000?oooooooooo05Cooooo0ol0000Booooo`004_ooool3ojVYZBkooooo0_l0000Sooooo`03o`00
0?oooooooooo04Wooooo00?o0000ooooooooool0Doooool3o`0001?ooooo000Booooo`?oZJVY;Ooo
ool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool0Dooo
ool3o`0001?ooooo000Booooo`?oZJVY;?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo04Sooooo00?o0000ooooooooool0Doooool3o`0001?ooooo000Cooooo`03o`000?oooooooooo
02[ooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo
05Cooooo00?o0000ooooooooool04_ooool001?ooooo0onYZJTYooooo`03o`000?oooooooooo02So
oooo00?o0000ooooooooool0Aoooool00ol0003oooooooooo`1Booooo`?o00005?ooool001?ooooo
0onYZJTXooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0A_ooool00ol0003ooooo
ooooo`1Booooo`?o00005?ooool001?ooooo0onYZJTWooooo`03o`000?oooooooooo02cooooo00?o
0000ooooooooool0AOooool00ol0003oooooooooo`1Booooo`?o00005?ooool001Gooooo00?o0000
ooooooooool09?ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo04Gooooo00?o0000
ooooooooool0D_ooool00ol0003oooooooooo`0Dooooo`005Oooool3ojVYZB;ooooo0_l0000aoooo
o`03o`000?oooooooooo04Cooooo00?o0000ooooooooool0D?ooool3o`0001Kooooo000Eooooo`?o
ZJVY8Oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo04?ooooo00?o0000oooooooo
ool0D?ooool3o`0001Kooooo000Eooooo`?oZJVY8?ooool00ol0003oooooooooo`0cooooo`03o`00
0?oooooooooo04?ooooo00?o0000ooooooooool0D?ooool3o`0001Kooooo000Fooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool0=Oooool00ol0003oooooooooo`12ooooo`03o`000?oo
oooooooo057ooooo00?o0000ooooooooool05Oooool001Kooooo0onYZJTMooooo`03o`000?oooooo
oooo03Oooooo00?o0000ooooooooool0@Oooool00ol0003oooooooooo`1?ooooo`?o00005oooool0
01Kooooo0onYZJTLooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0@Oooool00ol0
003oooooooooo`1?ooooo`?o00005oooool001Kooooo0onYZJTKooooo`03o`000?oooooooooo03[o
oooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`1?ooooo`?o00005oooool001Sooooo
00?o0000ooooooooool05oooool2o`0003kooooo00?o0000ooooooooool0?oooool00ol0003ooooo
ooooo`1?ooooo`03o`000?oooooooooo01Oooooo000Hooooo`?oZJVY5_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0COooool3o`0001Wooooo000H
ooooo`?oZJVY5Oooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0COooool3o`0001Wooooo000Hooooo`?oZJVY5?ooool00ol0003oooooooooo`12oooo
o`03o`000?oooooooooo03gooooo00?o0000ooooooooool0COooool3o`0001Wooooo000Jooooo`?o
ZJVY4Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo03gooooo00?o0000oooooooo
ool0Boooool3o`0001_ooooo000Jooooo`?oZJVY4?ooool00ol0003oooooooooo`15ooooo`03o`00
0?oooooooooo03cooooo00?o0000ooooooooool0Boooool3o`0001_ooooo000Jooooo`?oZJVY3ooo
ool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0Booo
ool3o`0001_ooooo000Looooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0B?ooool0
0ol0003oooooooooo`0kooooo`03o`000?oooooooooo04_ooooo00?o0000ooooooooool06oooool0
01_ooooo0onYZJT;ooooo`;o0000C?ooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo
04Wooooo0ol0000Mooooo`006oooool3ojVYZ@[ooooo00?o0000ooooooooool0COooool00ol0003o
ooooooooo`0iooooo`03o`000?oooooooooo04Wooooo0ol0000Mooooo`006oooool3ojVYZ@Wooooo
00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo04Wooooo
0ol0000Mooooo`007_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo053ooooo00?o
0000ooooooooool0>?ooool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo01gooooo000N
ooooo`?oZJVY1?ooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo03Oooooo00?o0000
ooooooooool0Aoooool3o`0001oooooo000Nooooo`?oZJVY0oooool00ol0003oooooooooo`1Coooo
o`03o`000?oooooooooo03Oooooo00?o0000ooooooooool0Aoooool3o`0001oooooo000Nooooo`?o
ZJVY0_ooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo03Kooooo00?o0000oooooooo
ool0Aoooool3o`0001oooooo000Pooooo`?oZJVYF?ooool00ol0003oooooooooo`0fooooo`03o`00
0?oooooooooo04Gooooo0ol0000Qooooo`008?ooool3ojVYZEWooooo00?o0000ooooooooool0=Ooo
ool00ol0003oooooooooo`15ooooo`?o00008Oooool0023ooooo0onYZJUJooooo`03o`000?oooooo
oooo03Cooooo00?o0000ooooooooool0AOooool3o`00027ooooo000Rooooo`03o`000?oooooooooo
05Sooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo
027ooooo000Rooooo`?oZJVYFOooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo04?o
oooo0ol0000Sooooo`008_ooool3ojVYZE[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`13ooooo`?o00008oooool002;ooooo0onYZJUJooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0@oooool3o`0002?ooooo000Tooooo`?oZJVYFOooool00ol0003oooooooooo`0a
ooooo`03o`000?oooooooooo043ooooo1?l0000Uooooo`009?ooool3ojVYZE[ooooo00?o0000oooo
ooooool0<?ooool00ol0003oooooooooo`10ooooo`?o00009_ooool002Cooooo0onYZJUJooooo`03
o`000?oooooooooo033ooooo00?o0000ooooooooool0@?ooool3o`0002Kooooo000Wooooo`?o0000
F?ooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo03kooooo0ol0000Xooooo`009ooo
ool3o`0005Wooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0nooooo`?o0000:?oo
ool002Oooooo0ol0001Iooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool0?_ooool3
o`0002Sooooo000Yooooo`?o0000F?ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo
03_ooooo1?l0000Zooooo`00:Oooool3o`0005Wooooo00?o0000ooooooooool0;?ooool00ol0003o
ooooooooo`0kooooo`?o0000:oooool002Wooooo0ol0001Iooooo`03o`000?oooooooooo02cooooo
00?o0000ooooooooool0>oooool3o`0002_ooooo000/ooooo`?o0000Eoooool00ol0003ooooooooo
o`0[ooooo`03o`000?oooooooooo03Wooooo0ol0000]ooooo`00;?ooool3o`0005Sooooo00?o0000
ooooooooool0:_ooool00ol0003oooooooooo`0iooooo`?o0000;Oooool002cooooo0ol0001Hoooo
o`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0>Oooool3o`0002gooooo000_ooooo`?o
0000E_ooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo03Kooooo0ol0000`ooooo`00
;oooool3o`0005Oooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0fooooo`?o0000
<?ooool002oooooo0ol0001Gooooo`03o`000?oooooooooo02Sooooo00?o0000ooooooooool0=_oo
ool3o`00033ooooo000booooo`?o0000EOooool00ol0003oooooooooo`0Wooooo`03o`000?oooooo
oooo03?ooooo0ol0000cooooo`00<_ooool3o`0005Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0cooooo`?o0000<oooool003;ooooo0ol0001Fooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool0<oooool3o`0003?ooooo000eooooo`?o0000E?ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo033ooooo0ol0000fooooo`00=Oooool3o`0005Cooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0aooooo`?o0000=_ooool003Gooooo0ol0001E
ooooo`03o`000?oooooooooo02?ooooo00?o0000ooooooooool0<Oooool3o`0003Kooooo000hoooo
o`?o0000Doooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo02kooooo0ol0000ioooo
o`00>?ooool3o`0005?ooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0^ooooo`?o
0000>Oooool003Sooooo0ol0001Dooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0
;_ooool3o`0003Wooooo000kooooo`?o0000D_ooool00ol0003oooooooooo`0Pooooo`03o`000?oo
oooooooo02_ooooo0ol0000looooo`00>oooool3o`0005;ooooo00?o0000ooooooooool08?ooool0
0ol0003oooooooooo`0[ooooo`?o0000??ooool003_ooooo1_l0001@ooooo`03o`000?oooooooooo
01oooooo00?o0000ooooooooool0:?ooool6o`0003cooooo000nooooo`?o0000DOooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo02Sooooo0ol0000oooooo`00?_ooool3o`00057ooooo
00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Xooooo`?o0000?oooool0047ooooo0ol0
001?ooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool09?ooool4o`0004;ooooo0011
ooooo`?o0000D?ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo02Cooooo0ol00013
ooooo`00@Oooool6o`0004gooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0Qoooo
o`Ko0000@oooool004Cooooo1?l0001=ooooo`03o`000?oooooooooo01_ooooo00?o0000oooooooo
ool08Oooool3o`0004Kooooo0014ooooo`?o000000Coooooo`000?l0003o0000Boooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01kooooo1_l00016ooooo`00B?ooool3o`0004_ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Nooooo`?o0000BOooool004Sooooo1_l0
0019ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool7o`0004Wooooo001;
ooooo`Co0000BOooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01[ooooo0ol0001=
ooooo`00Boooool3o`000003ooooool0003ooooo04Oooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Iooooo`Co0000COooool004oooooo0ol00017ooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool05oooool3o`00053ooooo001?ooooo`Oo0000A?ooool00ol0003ooooo
ooooo`0Fooooo`03o`000?oooooooooo01?ooooo1ol0001@ooooo`00Coooool3o`000004ooooool0
003o0000o`0004Cooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Cooooo`?o0000
00Coooooo`000?l0003o0000D?ooool005?ooooo1_l00012ooooo`03o`000?oooooooooo01Gooooo
00?o0000ooooooooool03oooool7o`0005Cooooo001Fooooo`?o0000@_ooool00ol0003ooooooooo
o`0Eooooo`03o`000?oooooooooo00oooooo0ol0001Hooooo`00E_ooool7o`0003oooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0<ooooo`Ko0000F?ooool005[ooooo1ol0000loooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02?ooool7o`0005_ooooo001Jooooo`?o
000000Coooooo`000?l0003o0000??ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Sooooo0ol000001?oooooo0000o`000?l0001Kooooo`00G_ooool7o`0003Wooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`04ooooo`Oo0000Goooool006;ooooo1ol0000fooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool01ol0001Sooooo`00H_ooool3o`000003oooo
ool0003o000000Co0000<oooool00ol0003oooooooooo`0@ooooo`Oo000000Coooooo`000?l0003o
0000Hoooool006Kooooo2?l0000booooo`03o`000?oooooooooo00kooooo1?l000001?oooooo0000
o`000?l0001Wooooo`00JOooool3o`000003ooooool0003o000000Go0000;Oooool00ol0003ooooo
ooooo`06ooooo`Oo000000Coooooo`000?l0003o0000Joooool006gooooo0ol000000ooooooo0000
o`000005o`0002Wooooo00?o0000ooooooooool00_ooool7o`000004ooooool0003o0000o`0006oo
oooo001]ooooo`?o000000Goooooo`000?l0003o0000ooooo`07o`0002Kooooo1ol000001Ooooooo
0000o`000?l0003ooooo00?o0000Koooool007Gooooo0ol000000ooooooo0000o`00000:o`000004
ooooool0003o0000o`0000gooooo0ol000000ooooooo0000o`000009o`000004ooooool0003o0000
o`0007Oooooo001iooooo`?o000000Coooooo`000?l0003o00000_oooolKo`000005ooooool0003o
0000o`000?ooool00ol0001kooooo`00OOooool3o`0000;ooooo0ol000001Ooooooo0000o`000?l0
003ooooo00?o000000Goooooo`000?l0003o0000ooooo`03o`000005ooooool0003o0000o`000?oo
ool00ol000001?oooooo0000o`000?l0001oooooo`00R_ooool3o`000005ooooool0003o0000o`00
0?ooool00ol0002;ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "This graph is a fixed point. Notice something very interesting happened: \
Defect was able to expand but did not spread to take over the entire network. \
More interestingly, whereas before we saw how Cooperators straddling both \
sides of a bridge edge could persist, in this case there is an entire region \
of Cooperators that survive. Let's take a closer look at what happened, \
rotating the graph ",
  Cell[BoxData[
      \(TraditionalForm\`90  \[Degree]\)]],
  " clockwise so the region we want to zoom in fits better."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gr = ExtractGraph[pd2];\)\), "\[IndentingNewLine]", 
    \(\(gr = 
        ChangeVertices[gr, 
          RotateVertices[
            Vertices[gr], \(-\(Pi\/2\)\)]];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[
        gr, \[IndentingNewLine]AspectRatio \[Rule] 
          Automatic, \[IndentingNewLine]PlotRange \[Rule] 
          Zoom[Table[i, {i, 82, 118}]]\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[28]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .27969 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.286378 1.57276 -1.3717 1.57276 [
[ 0 0 0 0 ]
[ 1 .27969 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .27969 L
0 .27969 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.55879 0 m
.54938 .1995 L
s
.19835 0 m
.07864 .07864 L
s
1 .0216 m
.98198 .03604 L
s
.98198 .03604 m
.96222 .05087 L
s
.96222 .05087 m
.94201 .06507 L
s
.94201 .06507 m
.92136 .07864 L
s
.92136 .07864 m
.9003 .09155 L
s
.92136 .07864 m
0 0 rlineto s
.92136 .07864 m
.85203 0 L
s
.9003 .09155 m
.87884 .10379 L
s
.87884 .10379 m
.85701 .11534 L
s
.85701 .11534 m
.83482 .12621 L
s
.83482 .12621 m
.81231 .13638 L
s
.81231 .13638 m
.78948 .14583 L
s
.78948 .14583 m
.76638 .15456 L
s
.76638 .15456 m
.743 .16257 L
s
.743 .16257 m
.71939 .16983 L
s
.71939 .16983 m
.69556 .17635 L
s
.69556 .17635 m
.67154 .18212 L
s
.67154 .18212 m
.64735 .18713 L
s
.64735 .18713 m
.62302 .19137 L
s
.62302 .19137 m
.59856 .19485 L
s
.59856 .19485 m
.574 .19756 L
s
.574 .19756 m
.54938 .1995 L
s
.54938 .1995 m
.5247 .20067 L
s
.5247 .20067 m
.5 .20105 L
s
.5 .20105 m
.4753 .20067 L
s
.4753 .20067 m
.45062 .1995 L
s
.45062 .1995 m
.426 .19756 L
s
.426 .19756 m
.40144 .19485 L
s
.40144 .19485 m
.37698 .19137 L
s
.37698 .19137 m
.35265 .18713 L
s
.35265 .18713 m
.32846 .18212 L
s
.35265 .18713 m
0 0 rlineto s
.35265 .18713 m
.19784 0 L
s
.32846 .18212 m
.30444 .17635 L
s
.30444 .17635 m
.28061 .16983 L
s
.28061 .16983 m
.257 .16257 L
s
.257 .16257 m
.23362 .15456 L
s
.23362 .15456 m
.21052 .14583 L
s
.21052 .14583 m
.18769 .13638 L
s
.18769 .13638 m
.16518 .12621 L
s
.16518 .12621 m
.14299 .11534 L
s
.14299 .11534 m
.12116 .10379 L
s
.12116 .10379 m
.0997 .09155 L
s
.0997 .09155 m
.07864 .07864 L
s
.07864 .07864 m
.05799 .06507 L
s
.05799 .06507 m
.03778 .05087 L
s
.03778 .05087 m
.01802 .03604 L
s
.01802 .03604 m
0 0 rlineto s
.01802 .03604 m
0 .0216 L
s
.025 w
.98198 .03604 Mdot
.663 .663 .663 r
.96222 .05087 Mdot
.94201 .06507 Mdot
.92136 .07864 Mdot
.9003 .09155 Mdot
.87884 .10379 Mdot
.85701 .11534 Mdot
.83482 .12621 Mdot
.81231 .13638 Mdot
.78948 .14583 Mdot
.76638 .15456 Mdot
.743 .16257 Mdot
.71939 .16983 Mdot
.69556 .17635 Mdot
.67154 .18212 Mdot
.64735 .18713 Mdot
.62302 .19137 Mdot
.59856 .19485 Mdot
.574 .19756 Mdot
.54938 .1995 Mdot
.5247 .20067 Mdot
.5 .20105 Mdot
0 0 0 r
.4753 .20067 Mdot
.45062 .1995 Mdot
.426 .19756 Mdot
.40144 .19485 Mdot
.37698 .19137 Mdot
.35265 .18713 Mdot
.32846 .18212 Mdot
.30444 .17635 Mdot
.28061 .16983 Mdot
.257 .16257 Mdot
.23362 .15456 Mdot
.21052 .14583 Mdot
.18769 .13638 Mdot
.16518 .12621 Mdot
.14299 .11534 Mdot
.12116 .10379 Mdot
.0997 .09155 Mdot
.07864 .07864 Mdot
.05799 .06507 Mdot
.03778 .05087 Mdot
.01802 .03604 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[28]:=",
  ImageSize->{468.375, 131},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007D0000Pb000`400?l00000o`00003oFoooool3
o`000:Kooooo0_l00026ooooo`;o0000A_ooool005[ooooo1?l0002Vooooo`;o0000Q_ooool2o`00
04Kooooo001Hooooo`Oo0000YOooool2o`0008Kooooo0ol00015ooooo`00Eoooool4o`0000;ooooo
0ol0002Tooooo`;o0000Qoooool3o`0004Cooooo001Eooooo`Co00001Oooool3o`000:?ooooo0_l0
0028ooooo`?o0000@oooool005Cooooo1?l00007ooooo`?o0000X_ooool2o`0008Wooooo0ol00012
ooooo`00D_ooool4o`0000[ooooo0ol0002Qooooo`;o0000R_ooool3o`00047ooooo001Aooooo`Co
00003?ooool2o`000:7ooooo0_l0002;ooooo`?o0000@?ooool004oooooo1?l0000>ooooo`?o0000
X?ooool2o`0008cooooo0_l00010ooooo`00C_ooool4o`00013ooooo0ol0002Oooooo`;o0000S?oo
ool3o`0003oooooo001<ooooo`Co00004oooool3o`0009gooooo0ol0002=ooooo`?o0000?_ooool0
0003ooooool0003o000004Sooooo1?l0000Eooooo`?o0000W?ooool2o`0008oooooo0ol0000ioooo
o`?o00000Oooool00004ooooool0003o0000o`0000?ooooo1Ol0000mooooo`Co00006?ooool3o`00
09_ooooo0_l0002@ooooo`?o0000;oooool5o`0000?ooooo1?l00001ooooo`000_ooool3o`000003
ooooool0003o000000Go0000>oooool4o`0001[ooooo0_l0002Kooooo`;o0000TOooool3o`0002go
oooo2ol00003ooooo`000oooool;o`0003Sooooo1?l0000Looooo`?o0000V_ooool2o`0009;ooooo
0ol0000[ooooo`_o00001?ooool000Cooooo2ol0000fooooo`Co00007_ooool3o`0009Wooooo0_l0
002Cooooo`?o0000:Oooool;o`0000Gooooo0004ooooo`_o0000=?ooool4o`00027ooooo0ol0002H
ooooo`;o0000U?ooool2o`0002Wooooo2ol00005ooooo`001?ooool;o`0003?ooooo1?l0000Soooo
o`?o0000Uoooool2o`0009Cooooo0ol0000Xooooo`_o00001Oooool000Cooooo2ol0000aooooo`Co
00009_ooool2o`0009Oooooo0_l0002Eooooo`?o00009oooool;o`0000Gooooo0004ooooo`_o0000
00?oooooo`000?l000000ol0000Yooooo`Go00009oooool3o`0009Kooooo0_l0002Fooooo`?o0000
8?ooool5ojVYZ@03ooooool0003o000000Wo00001Oooool000Gooooo4Ol0000Wooooo`Co0000:_oo
ool3o`0009Gooooo0_l0002Gooooo`?o00007_ooool7ojVYZ@[o00001_ooool000Kooooo4Ol0000T
ooooo`Co0000;Oooool3o`0009Cooooo0_l0002Hooooo`?o00007?ooool9ojVYZ@So00001oooool0
00Oooooo1Ol000000ooooooo0000o`000009o`0002;ooooo1?l0000_ooooo`?o0000Toooool2o`00
09Wooooo0ol0000Jooooo`_oZJVY00?oooooo`000?l000000ol00008ooooo`003Oooool;o`00023o
oooo1?l0000booooo`?o0000T_ooool2o`0009[ooooo0_l0000Jooooo`_oZJVY3_ooool000gooooo
2ol0000Oooooo`Co0000=?ooool2o`0009;ooooo0_l0002Jooooo`?o00006Oooool;ojVYZ@kooooo
000=ooooo`_o00007Oooool4o`0003Kooooo0ol0002Aooooo`;o0000Voooool3o`0001Sooooo2onY
ZJT>ooooo`003Oooool;o`000003ooooool0003o000000?o00005_ooool4o`0003Sooooo0ol0002@
ooooo`;o0000W?ooool3o`00013ooooo1OnYZJT2ooooo`_oZJVY3_ooool000kooooo4Ol0000Coooo
o`Co0000>oooool3o`0008oooooo0_l0002Mooooo`?o00003_ooool7ojVYZ@;o00002OnYZJT?oooo
o`003ooooolAo`00017ooooo1?l0000mooooo`?o0000S_ooool2o`0009kooooo0ol0000<ooooo`Wo
ZJVY0_l00007ojVYZA3ooooo000@ooooo`Go000000?oooooo`000?l000002Ol0000>ooooo`Co0000
@?ooool3o`0008cooooo0ol0002Oooooo`?o00002_ooool;ojVYZ@;ooooo1OnYZJTAooooo`005_oo
ool;o`0000gooooo1?l00012ooooo`;o0000S?ooool2o`000:7ooooo0ol00009ooooo`_oZJVY6?oo
ool001Kooooo2ol0000;ooooo`Co0000A?ooool3o`0008_ooooo0_l0002Rooooo`;o00002Oooool;
ojVYZASooooo000Fooooo`_o00000_ooool5o`0000?ooooo1?l00016ooooo`?o0000R_ooool2o`00
0:;ooooo0ol00002ooooo`GoZJVY00?oooooojVYZOnYZJT02OnYZJTHooooo`005_ooool;o`000003
ooooool0003o000000Wo0000BOooool3o`0008Wooooo0_l0002Sooooo`?o00004_nYZJTHooooo`00
5ooooolEo`0004_ooooo0ol00028ooooo`;o0000Y?ooool00ol0003oZJVYojVYZ@0@ojVYZAWooooo
000Hooooo`Oo000000?oooooo`000?l000002Ol0001=ooooo`?o0000Qoooool2o`000:Cooooo4_nY
ZJTJooooo`006Oooool5o`0000;ooooo2ol0001>ooooo`;o0000Qoooool2o`000:Cooooo2onYZJT0
0oooooooZJVYojVYZ@03ojVYZA_ooooo000Pooooo`_o0000C_ooool3o`0008Kooooo0_l0002Toooo
o`_oZJVY8Oooool0023ooooo2ol00002ooooo`Go0000B?ooool3o`0008Gooooo0_l0002Mooooo`Go
ZJVY0_ooool;ojVYZB7ooooo000Pooooo`_o000000?oooooo`000?l000001Ol00018ooooo`?o0000
Q?ooool2o`0009cooooo1onYZJT00oooooooZJVYojVYZ@09ojVYZB7ooooo000Qoooooa?o0000B?oo
ool3o`0008?ooooo0_l0002Kooooo`WoZJVY00?o0000ojVYZOnYZJT01onYZJTRooooo`008_oooolC
o`0004Sooooo0ol00022ooooo`;o0000V_ooool;ojVYZ@03oooooonYZJWoZJVY00GoZJVY8oooool0
02?ooooo1Ol00002ooooo`_o0000BOooool2o`0008;ooooo0_l0002Jooooo`_oZJVY0_ooool5ojVY
ZBCooooo000Zooooo`_o0000BOooool3o`00087ooooo0_l0002Jooooo`_oZJVY:oooool002[ooooo
2ol00002ooooo`Go0000@oooool3o`00083ooooo0_l0002Cooooo`GoZJVY0_ooool;ojVYZB_ooooo
000Zooooo`_o000000?oooooo`000?l000001Ol00013ooooo`?o0000Ooooool2o`0009;ooooo1onY
ZJT00oooooooZJVYojVYZ@09ojVYZB_ooooo000[oooooa?o0000@oooool3o`0007kooooo0_l0002A
ooooo`WoZJVY00?o0000ojVYZOnYZJT01onYZJT/ooooo`00;?oooolCo`0004?ooooo0_l0001moooo
o`?o0000T?ooool;ojVYZ@03oooooonYZJWoZJVY00GoZJVY;Oooool002gooooo1Ol00002ooooo`_o
0000@oooool3o`0007cooooo0_l0002Aooooo`_oZJVY0_ooool5ojVYZBkooooo000dooooo`_o0000
0_ooool5o`0003gooooo0ol0001kooooo`;o0000R_ooool5ojVYZ@;ooooo2onYZJTeooooo`00=?oo
ool;o`000003ooooool0003o000000Go0000?Oooool3o`0007[ooooo0_l00029ooooo`OoZJVY00?o
ooooojVYZOnYZJT02OnYZJTeooooo`00=?oooolDo`0003gooooo0ol0001iooooo`;o0000R?oooolD
ojVYZCGooooo000eoooooaCo0000?Oooool3o`0007Sooooo0_l00027oooooaCoZJVY=_ooool003Ko
oooo1ol000000ooooooo0000o`000009o`0003kooooo0_l0001hooooo`;o0000Qoooool;ojVYZ@03
oooooonYZJWoZJVY00GoZJVY=oooool003Oooooo1Ol00002ooooo`_o00000_ooool5o`0003Oooooo
0ol0001gooooo`;o0000P?ooool5ojVYZ@;ooooo2onYZJT2ooooo`GoZJVY>?ooool003kooooo2ol0
00000ooooooo0000o`000005o`0003Oooooo0ol0001fooooo`;o0000Ooooool7ojVYZ@03oooooonY
ZJWoZJVY00WoZJVY?oooool003kooooo5?l0000gooooo`?o0000MOooool2o`0007kooooo5?nYZJTo
ooooo`00?ooooolDo`0003Oooooo0ol0001dooooo`;o0000OOoooolDojVYZD3ooooo0010ooooo`Oo
000000?oooooo`000?l000002Ol0000hooooo`?o0000Loooool2o`0007gooooo2onYZJT00ooooooo
ZJVYojVYZ@05ojVYZD7ooooo0011ooooo`Go00000_ooool;o`0000?ooooo1Ol0000aooooo`;o0000
Loooool2o`0007Gooooo1OnYZJT3ooooo`_oZJVY0_ooool5ojVYZD;ooooo0018ooooo`_o00000_oo
ool7o`00033ooooo0ol0001booooo`;o0000M?ooool7ojVYZ@;ooooo2onYZJU9ooooo`00B?oooolE
o`00033ooooo0ol0001aooooo`;o0000Loooool9ojVYZ@03o`000?nYZJWoZJVY00WoZJVYBOooool0
04Wooooo5Ol0000`ooooo`?o0000L?ooool2o`0007;ooooo2onYZJT00ol0003oZJVYojVYZ@07ojVY
ZD[ooooo001:ooooo`Oo00000_ooool;o`00037ooooo0ol0001_ooooo`;o0000L_ooool;ojVYZ@;o
oooo1onYZJU;ooooo`00Boooool5o`0000?ooooo2ol00002ooooo`Go0000:oooool3o`0006kooooo
0_l0001Zooooo`GoZJVY0oooool;ojVYZ@?ooooo1OnYZJU<ooooo`00Doooool;o`000003ooooool0
003o000000Go0000:oooool2o`0006kooooo0_l0001Yooooo`OoZJVY0_ooool;ojVYZECooooo001C
oooooaCo0000:_ooool3o`0006cooooo0ol0001Xooooo`WoZJVY00?o0000ojVYZOnYZJT02OnYZJUD
ooooo`00E?oooolDo`0002[ooooo0ol0001[ooooo`;o0000J?ooool;ojVYZ@03o`000?nYZJWoZJVY
00OoZJVYEOooool005Gooooo1ol000000ooooooo0000o`000009o`0000?ooooo1Ol0000Sooooo`?o
0000J_ooool2o`00067ooooo1OnYZJT2ooooo`_oZJVY0_ooool7ojVYZEKooooo001Fooooo`Go0000
0_ooool;o`0000;ooooo1ol0000Sooooo`?o0000JOooool2o`00063ooooo1onYZJT00oooooooZJVY
ojVYZ@09ojVYZ@?ooooo1OnYZJUGooooo`00GOoooolEo`0002?ooooo0ol0001Xooooo`;o0000Gooo
oolDojVYZEoooooo001MoooooaKo00000oooool5o`0001_ooooo0_l0001Xooooo`;o0000E_ooool5
ojVYZ@?ooooo5OnYZJUOooooo`00G_ooool9o`000003ooooool0003o000000Wo00000_ooool7o`00
01[ooooo0ol0001Wooooo`;o0000EOooool7ojVYZ@;ooooo5?nYZJUPooooo`00Goooool7o`0000;o
oooo2ol000000ooooooo0000o`000007o`0001[ooooo0ol0001Vooooo`;o0000E?ooool9ojVYZ@03
oooooonYZJWoZJVY00WoZJVY00?oooooojVYZOnYZJT01OnYZJUQooooo`00H?ooool5o`0000?ooooo
5_l0000Jooooo`?o0000IOooool2o`0005?ooooo5_nYZJT2ooooo`GoZJVYH_ooool006Sooooo5_l0
0003ooooo`Go00004oooool3o`0006Cooooo0_l0001;ooooo`GoZJVY0ooooolFojVYZFWooooo001Y
ooooo`Wo000000?oooooo`000?l000002Ol00002ooooo`Oo00004oooool2o`0006Cooooo0_l0001:
ooooo`OoZJVY0_ooool;ojVYZ@03oooooonYZJWoZJVY00OoZJVYJ_ooool006[ooooo1ol00002oooo
oaGo00004_ooool3o`0006?ooooo0_l00019ooooo`WoZJVY00?o0000ojVYZOnYZJT02OnYZJT2oooo
o`OoZJVYJoooool006_ooooo1Ol00003oooooaKo00000oooool5o`0000[ooooo0ol0001Rooooo`;o
0000@?ooool5ojVYZ@?ooooo5_nYZJT3ooooo`GoZJVYK?ooool007Cooooo2Ol000000ooooooo0000
o`000009o`0000;ooooo1ol0000:ooooo`?o0000HOooool2o`0003oooooo1onYZJT2ooooo`_oZJVY
00?oooooojVYZOnYZJT01onYZJUeooooo`00MOooool7o`0000;ooooo5Ol00004ooooo`Go000000Co
ooooo`000?l0003o0000H?ooool2o`0003Gooooo1OnYZJT4ooooo`WoZJVY00?o0000ojVYZOnYZJT0
2OnYZJT2ooooo`OoZJVYM_ooool007Kooooo1Ol00003oooooaKo00000_ooool7o`000004ooooool0
003o0000o`0005oooooo0_l0000dooooo`OoZJVY0_oooolFojVYZ@?ooooo1OnYZJUgooooo`00O_oo
oolFo`000003ooooool0003o000000Oo000000?oooooo`000?l00000Goooool2o`0003?ooooo2OnY
ZJT00oooooooZJVYojVYZ@0DojVYZGoooooo001oooooo`Wo000000?oooooo`000?l000007?l0001I
ooooo`;o0000:Oooool5ojVYZ@Cooooo5_nYZJT00oooooooZJVYojVYZ@07ojVYZH3ooooo0020oooo
o`Oo00000_oooolFo`000003ooooool0003o000000Ko0000F?ooool2o`0002Sooooo1onYZJT3oooo
oaKoZJVY0_ooool7ojVYZH7ooooo0021ooooo`Go00000ooooolFo`000003ooooool0003o000000Oo
00001Oooool5o`0004cooooo0ol0000Nooooo`GoZJVY1?ooool9ojVYZ@;ooooo5_nYZJT3ooooo`Go
ZJVYP_ooool008[ooooo2Ol000000ooooooo0000o`00000Do`0000?ooooo1ol00005ooooo`Go0000
@Oooool2o`0001Cooooo1OnYZJT5ooooo`OoZJVY0_ooool;ojVYZ@03o`000?nYZJWoZJVY00WoZJVY
00?oooooojVYZOnYZJT01onYZJV;ooooo`00Roooool7o`0000;ooooo5_l00002ooooo`Wo00000ooo
ool7o`00043ooooo0_l0000Cooooo`OoZJVY0oooool9ojVYZ@03oooooonYZJWoZJVY00WoZJVY00?o
0000ojVYZOnYZJT02OnYZJT2ooooo`OoZJVYS?ooool008cooooo1Ol00004ooooo`Wo000000?ooooo
o`000?l000005Ol000000ooooooo0000o`000007o`0000Gooooo1Ol00006ooooo`Go0000:Oooool5
ojVYZ@Kooooo1OnYZJT5ooooo`WoZJVY00?oooooojVYZOnYZJT05?nYZJT2ooooo`WoZJVY1?ooool5
ojVYZHgooooo002Fooooo`Oo00000_oooolRo`0000?ooooo1ol00004ooooo`Oo00001Oooool5o`00
00Oooooo1OnYZJT6ooooo`GoZJVY1_ooool7ojVYZ@Cooooo1onYZJT3ooooob7oZJVY0oooool7ojVY
ZIOooooo002Gooooo`Go00000oooool;o`000003ooooool0003o000001Co00000_ooool9o`0000;o
oooo2Ol00003ooooo`Oo00001Oooool7ojVYZ@Cooooo1onYZJT4ooooo`WoZJVY0_ooool9ojVYZ@;o
oooo8OnYZJT4ooooo`GoZJVYV?ooool00:3ooooo2Ol00002ooooobgo000000?oooooo`000?l00000
1ol00003ooooo`WoZJVY0_ooool9ojVYZ@;ooooo5_nYZJT00ol0003oZJVYojVYZ@0DojVYZ@03oooo
oonYZJWoZJVY00OoZJVYX_ooool00:7ooooo1ol00003ooooocSo000000?oooooojVYZOnYZJT05?nY
ZJT00oooooooZJVYojVYZ@0DojVYZ@03o`000?nYZJWoZJVY01CoZJVY0_ooool7ojVYZJ?ooooo002R
ooooo`Go00001Oooool9o`000003ooooool0003o000002_o000000?oooooojVYZOnYZJT05?nYZJT0
0ol0003oZJVYojVYZ@0DojVYZ@03o`000?nYZJWoZJVY00WoZJVY00?oooooojVYZOnYZJT01onYZJT4
ooooo`GoZJVYY?ooool00:gooooo1ol00003ooooo`Wo00000_oooolRo`0001KoZJVY00?o0000ojVY
ZOnYZJT05?nYZJT2ooooo`WoZJVY0oooool7ojVYZJkooooo002^ooooo`Go00001Oooool7o`0000?o
oooo8_l0000FojVYZ@03o`000?nYZJWoZJVY01CoZJVY0oooool7ojVYZ@Gooooo1OnYZJV_ooooo`00
^Oooool5o`0000Gooooo2Ol00002ooooo`Wo000000?oooooo`000?l000002Ol000000oooooooZJVY
ojVYZ@0DojVYZ@;ooooo2OnYZJT2ooooo`WoZJVY1Oooool5ojVYZK[ooooo0034ooooo`Oo00001?oo
ool7o`0000?ooooo2Ol00003ooooo`WoZJVY0_ooool9ojVYZ@Cooooo1onYZJT4ooooo`OoZJVYaOoo
ool00<Gooooo1Ol00006ooooo`Go00001Oooool7o`0000Gooooo1onYZJT4ooooo`OoZJVY1_ooool5
ojVYZ@Kooooo1OnYZJW6ooooo`00foooool5o`0000Oooooo1OnYZJT6ooooo`GoZJVYgOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?ooooooeOooool00?oo
ooooeOooool00001\
\>"],
  ImageRangeCache->{{{0, 467.375}, {130, 0}} -> {0.180322, 0.87216, \
0.00136797, 0.00136797}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
The above graph does not include the update neighborhood, but we \
can infer what happened. The relatively low number of Defects initially \
present meant that in many regions of the graph, Defect was able to spread.  \
However, as we saw previously, though, the spread of Defect could be blocked \
by clusters of Cooperators forming around bridge vertices. If a region of \
all-Cooperate is surrounded on either side by Cooperators occupying bridge \
vertices, this prevents the spread of Defect into those regions lying between \
the bridge edges. Indeed, that is what we see in the graphic above.\
\>", \
"Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(DeleteModel[pd2];\)\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Collecting and showing statistics", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Suppose you want to plot the time evolution of the number of strategies. \
The ",
  StyleBox["StrategyHistogram",
    FontWeight->"Bold"],
  " command, which takes a model as its sole argument, returns a list \
containing the total number of individuals following each strategy. The \
output from this command can be saved in a list for further analysis."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Here is an example with ",
  StyleBox["Battle of the Sexes",
    FontSlant->"Italic"],
  " played on a ",
  Cell[BoxData[
      \(TraditionalForm\`10\[Cross]10\)]],
  " grid. Notice that one doesn't need to create the graph, payoff matrix, \
and list of strategy colors ahead of time like we've been doing. (Although \
doing so is often useful, as we've seen.)"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"bos4", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] GridGraph[10, 10]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({1, 2}\), \({0, 0}\)},
                    {\({0, 0}\), \({2, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {\ Red, 
                Green}\)}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos4], 
        VertexStyle \[Rule] Box[Large]];\)\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 0 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
1 0 0 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 1 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
1 0 0 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
0 1 0 r
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
1 0 0 r
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
0 1 0 r
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
1 0 0 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 1 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
1 0 0 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 1 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
1 0 0 r
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
0 1 0 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
1 0 0 r
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
0 1 0 r
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
1 0 0 r
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
0 1 0 r
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
1 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
0 1 0 r
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
1 0 0 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 1 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
1 0 0 r
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
0 1 0 r
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
1 0 0 r
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
0 1 0 r
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
1 0 0 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
0 1 0 r
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
1 0 0 r
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
0 1 0 r
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
1 0 0 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 1 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
1 0 0 r
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
0 1 0 r
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
1 0 0 r
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
0 1 0 r
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
1 0 0 r
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
0 1 0 r
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
1 0 0 r
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
0 1 0 r
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
1 0 0 r
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
0 1 0 r
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
1 0 0 r
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
0 1 0 r
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
1 0 0 r
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[4]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`000ooooolC
o`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolC
o`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool0
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o
00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[o
oooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo
0003oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:
oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
o`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooo
oolCo`3o00[o00004ooo000:o`0001?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?o0?l02_l0
000Co`3o00[o00004ol0o`0:o`0001?oo`002_l0000Cool000Gooooo0003oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolC
o`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool0
00[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Go
oooo0003oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?oo`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`00
0ooooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Ooo
ool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00Gooooo0003oooooa?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?oo`00
2_l0000Cool000[o00004ol0o`0:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:o`0001?o0?l0
1Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oo
oolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolC
o`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[o
oooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo
0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0
o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
o`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l01Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`001Oooool0
00?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo0005ooooo`000ooooolCool000[o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000Gooooo0003oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`001Oooool000?ooooo4ooo000:o`0001?o
o`002_l0000Cool000[o00004ooo000:o`0001?oo`002_l0000Co`3o00[o00004ooo000:o`0001?o
0?l02_l0000Co`3o00[o00004ooo0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`001Ooo
ool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo0005ooooo`000ooooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000Gooooo0003oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000Gooooo0003
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`001Oooool000?ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo0005ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`00
2_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`00
2_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oo
oolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oo
oolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolC
o`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolC
o`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?o
oooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo
4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0
o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05
ooooo`000ooooolCool000[o00004ooo000:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:o`00
01?o0?l02_l0000Cool000[o00004ol0o`0:o`0001?oo`002_l0000Co`3o00Gooooo0003oooooa?o
o`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oo
oolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool0
00?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[o
oooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`00
0ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o
00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_l0000Co`3o00[o00004ooo000:
o`0001?oo`002_l0000Cool000[o00004ooo000:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:
o`0001?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05oooo
o`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
o`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?o0?l01Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
o`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Go
oooo0003oooooa?oo`002_l0000Cool000[o00004ol0o`0:o`0001?oo`002_l0000Cool000[o0000
4ooo000:o`0001?oo`002_l0000Cool000[o00004ooo000:o`0001?o0?l01Oooool000?ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`00
0ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
1Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolC
o`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:o`0001?oo`002_l0000Cool000[o
00004ooo000:o`0001?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?oo`002_l0000Co`3o00[o
00004ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo
0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0
o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Go
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0
o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Ooo
ool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o
00[o00004ol0o`0:o`0001?o0?l02_l0000Cool000[o00004ooo000:o`0001?oo`002_l0000Cool0
00[o00004ooo000:o`0001?oo`002_l0000Co`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003
oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l0
1Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolC
o`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?o0?l01Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo
4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05
ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oo
oolCo`3o00Gooooo0003oooooa?o0?l02_l0000Co`3o00[o00004ol0o`0:o`0001?o0?l02_l0000C
o`3o00[o00004ol0o`0:o`0001?oo`002_l0000Co`3o00[o00004ooo000:o`0001?o0?l01Oooool0
00?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`00
2_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Ooo
ool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolC
o`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005oooo
o`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ooo000:o`0001?oo`002_l0
000Cool000[o00004ooo000:o`0001?o0?l02_l0000Cool000[o00004ol0o`0:o`0001?o0?l02_l0
000Cool000[o00004ooo0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo0005
ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o
o`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000Gooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(StrategyHistogram[bos4]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({54, 46}\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Now let's run the model for 120 generations and keep track of the \
strategy totals.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          StrategyHistogram[bos4]]; \[IndentingNewLine]Evolve[
          bos4], {120}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[l]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(120\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "This list can be plotted using ",
  StyleBox["MultipleListPlot",
    FontWeight->"Bold"],
  ". But, first, take a close look at its definition... "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(<< Graphics`MultipleListPlot`\), "\n", 
    \(\(?MultipleListPlot\)\)}], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(Star::"shdw"\), \(\(:\)\(\ \)\), "\<\"Symbol \\!\\(\\\"Star\\\"\
\\) appears in multiple contexts \\!\\({\\\"Graphics`MultipleListPlot`\\\", \
\\\"DiscreteMath`Combinatorica`\\\"}\\); definitions in context \
\\!\\(\\\"Graphics`MultipleListPlot`\\\"\\) may shadow or be shadowed by \
other definitions. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::shdw\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[3]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{"\<\"MultipleListPlot[data, (opts)] plots the data, using symbols \
determined by the options. MultipleListPlot[data1, data2,...] plots multiple \
sets of data. Data are either a list of numbers (whose x coordinates are \
implicitly the positions in the list), or a list of {x, y} coordinate pairs. \
Each data point can be accompanied by an error, indicated by an ErrorBar \
object giving the error as positive and negative offsets (see ErrorBar for \
more information). \"\>", " ", 
      ButtonBox[
        StyleBox["More\[Ellipsis]",
          "SR"],
        ButtonData:>"Graphics`MultipleListPlot`",
        Active->True,
        ButtonStyle->"AddOnsLink"]}]], "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "When plotting multiple lists, ",
  StyleBox["MultipleListPlot",
    FontWeight->"Bold"],
  " expects to be given lists of time-series data, each list corresponding to \
the time-series data for a given variable. We don't have that, exactly. What \
we have is a list, ",
  StyleBox["l", "Input"],
  ", of time-series data for ",
  StyleBox["pairs",
    FontSlant->"Italic"],
  " of variables: strategy 1 and strategy 2. We need to convert this to two \
lists of the proper form before using ",
  StyleBox["MultipleListPlot",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(strategy1 = Map[First, l];\)\), "\[IndentingNewLine]", 
    \(\(strategy2 = Map[Last, l];\)\)}], "Input",
  CellLabel->"In[11]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Verify that ",
  StyleBox["strategy1", "Input"],
  " and ",
  StyleBox["strategy2", "Input"],
  " really do contain the right data:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{\(Take[l, 10]\), " ", 
      StyleBox[\( (*Return\ the\ first\ 10\ items\ of\ the\ list*) \),
        FontColor->RGBColor[0, 0, 1]]}]], "Input",
  CellLabel->"In[13]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{54, 46}, {62, 38}, {66, 34}, {69, 31}, {70, 30}, {70, 30}, {71, 
        29}, {71, 29}, {75, 25}, {77, 23}}\)], "Output",
  CellLabel->"Out[13]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Use this list to check that ",
  StyleBox["strategy1", "Input"],
  " only contains the first element of each ordered pair and that ",
  StyleBox["strategy2", "Input"],
  " only contains the second element of each ordered pair."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Take[strategy1, 10]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({54, 62, 66, 69, 70, 70, 71, 71, 75, 77}\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Take[strategy2, 10]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({46, 38, 34, 31, 30, 30, 29, 29, 25, 23}\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Now plot the data.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(MultipleListPlot[strategy1, 
        strategy2, \[IndentingNewLine]SymbolShape \[Rule] 
          None, \[IndentingNewLine]PlotJoined \[Rule] 
          True\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00793651 0.0147151 0.00588604 [
[.18254 .00222 -6 -9 ]
[.18254 .00222 6 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.65873 .00222 -6 -9 ]
[.65873 .00222 6 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .13244 -12 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -12 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -12 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -12 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -18 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(20)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(40)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(80)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(100)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(120)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.06349 .01472 m
.06349 .01847 L
s
.10317 .01472 m
.10317 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.22222 .01472 m
.22222 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.30159 .01472 m
.30159 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.42063 .01472 m
.42063 .01847 L
s
.46032 .01472 m
.46032 .01847 L
s
.53968 .01472 m
.53968 .01847 L
s
.57937 .01472 m
.57937 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.69841 .01472 m
.69841 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.77778 .01472 m
.77778 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.89683 .01472 m
.89683 .01847 L
s
.93651 .01472 m
.93651 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(20)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(40)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(60)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(80)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(100)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.001 w
.03175 .33256 m
.03968 .37965 L
.04762 .40319 L
.05556 .42085 L
.06349 .42674 L
.07143 .42674 L
.07937 .43262 L
.0873 .43262 L
.09524 .45617 L
.10317 .46794 L
.11111 .47971 L
.11905 .47971 L
.12698 .4856 L
.13492 .4856 L
.14286 .4856 L
.15079 .49148 L
.15873 .49148 L
.16667 .49148 L
.1746 .49148 L
.18254 .49148 L
.19048 .49148 L
.19841 .49737 L
.20635 .49737 L
.21429 .50326 L
.22222 .49737 L
.23016 .50326 L
.2381 .50326 L
.24603 .50326 L
.25397 .50326 L
.2619 .50326 L
.26984 .50914 L
.27778 .51503 L
.28571 .52091 L
.29365 .52091 L
.30159 .51503 L
.30952 .51503 L
.31746 .5268 L
.3254 .52091 L
.33333 .52091 L
.34127 .52091 L
.34921 .5268 L
.35714 .53857 L
.36508 .55034 L
.37302 .55623 L
.38095 .55623 L
.38889 .56212 L
.39683 .568 L
.40476 .57389 L
.4127 .59155 L
.42063 .60332 L
Mistroke
.42857 .60332 L
.43651 .60332 L
.44444 .60332 L
.45238 .60332 L
.46032 .60332 L
.46825 .60332 L
.47619 .60332 L
.48413 .60332 L
.49206 .60332 L
.5 .60332 L
.50794 .60332 L
.51587 .60332 L
.52381 .60332 L
.53175 .60332 L
.53968 .60332 L
.54762 .60332 L
.55556 .60332 L
.56349 .60332 L
.57143 .60332 L
.57937 .60332 L
.5873 .60332 L
.59524 .60332 L
.60317 .60332 L
.61111 .60332 L
.61905 .60332 L
.62698 .60332 L
.63492 .60332 L
.64286 .60332 L
.65079 .60332 L
.65873 .60332 L
.66667 .60332 L
.6746 .60332 L
.68254 .60332 L
.69048 .60332 L
.69841 .60332 L
.70635 .60332 L
.71429 .60332 L
.72222 .60332 L
.73016 .60332 L
.7381 .60332 L
.74603 .60332 L
.75397 .60332 L
.7619 .60332 L
.76984 .60332 L
.77778 .60332 L
.78571 .60332 L
.79365 .60332 L
.80159 .60332 L
.80952 .60332 L
.81746 .60332 L
Mistroke
.8254 .60332 L
.83333 .60332 L
.84127 .60332 L
.84921 .60332 L
.85714 .60332 L
.86508 .60332 L
.87302 .60332 L
.88095 .60332 L
.88889 .60332 L
.89683 .60332 L
.90476 .60332 L
.9127 .60332 L
.92063 .60332 L
.92857 .60332 L
.93651 .60332 L
.94444 .60332 L
.95238 .60332 L
.96032 .60332 L
.96825 .60332 L
.97619 .60332 L
Mfstroke
[ .5 3 ] 0 Mabsdash
.03175 .28547 m
.03968 .23838 L
.04762 .21484 L
.05556 .19718 L
.06349 .1913 L
.07143 .1913 L
.07937 .18541 L
.0873 .18541 L
.09524 .16187 L
.10317 .15009 L
.11111 .13832 L
.11905 .13832 L
.12698 .13244 L
.13492 .13244 L
.14286 .13244 L
.15079 .12655 L
.15873 .12655 L
.16667 .12655 L
.1746 .12655 L
.18254 .12655 L
.19048 .12655 L
.19841 .12066 L
.20635 .12066 L
.21429 .11478 L
.22222 .12066 L
.23016 .11478 L
.2381 .11478 L
.24603 .11478 L
.25397 .11478 L
.2619 .11478 L
.26984 .10889 L
.27778 .10301 L
.28571 .09712 L
.29365 .09712 L
.30159 .10301 L
.30952 .10301 L
.31746 .09123 L
.3254 .09712 L
.33333 .09712 L
.34127 .09712 L
.34921 .09123 L
.35714 .07946 L
.36508 .06769 L
.37302 .0618 L
.38095 .0618 L
.38889 .05592 L
.39683 .05003 L
.40476 .04415 L
.4127 .02649 L
.42063 .01472 L
Mistroke
.42857 .01472 L
.43651 .01472 L
.44444 .01472 L
.45238 .01472 L
.46032 .01472 L
.46825 .01472 L
.47619 .01472 L
.48413 .01472 L
.49206 .01472 L
.5 .01472 L
.50794 .01472 L
.51587 .01472 L
.52381 .01472 L
.53175 .01472 L
.53968 .01472 L
.54762 .01472 L
.55556 .01472 L
.56349 .01472 L
.57143 .01472 L
.57937 .01472 L
.5873 .01472 L
.59524 .01472 L
.60317 .01472 L
.61111 .01472 L
.61905 .01472 L
.62698 .01472 L
.63492 .01472 L
.64286 .01472 L
.65079 .01472 L
.65873 .01472 L
.66667 .01472 L
.6746 .01472 L
.68254 .01472 L
.69048 .01472 L
.69841 .01472 L
.70635 .01472 L
.71429 .01472 L
.72222 .01472 L
.73016 .01472 L
.7381 .01472 L
.74603 .01472 L
.75397 .01472 L
.7619 .01472 L
.76984 .01472 L
.77778 .01472 L
.78571 .01472 L
.79365 .01472 L
.80159 .01472 L
.80952 .01472 L
.81746 .01472 L
Mistroke
.8254 .01472 L
.83333 .01472 L
.84127 .01472 L
.84921 .01472 L
.85714 .01472 L
.86508 .01472 L
.87302 .01472 L
.88095 .01472 L
.88889 .01472 L
.89683 .01472 L
.90476 .01472 L
.9127 .01472 L
.92063 .01472 L
.92857 .01472 L
.93651 .01472 L
.94444 .01472 L
.95238 .01472 L
.96032 .01472 L
.96825 .01472 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[16]:=",
  ImageSize->{405.812, 250.812},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006E0000nR000`400?l00000o`00003ooooooonF
ooooo`00Coooool5o`0000?ooooo0_l0000fooooo`?o00000oooool2o`0003Gooooo0ol00004oooo
o`;o0000=Oooool3o`0000Cooooo0_l0000booooo`?o00001?ooool2o`0000Cooooo0_l0000_oooo
o`?o00000_ooool5o`0000?ooooo0_l00003ooooo`00D?ooool01?l0003oooooooooool00002oooo
o`04o`000?ooooooooooo`0003Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo037ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0003?o
oooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0003;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0002oooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`00
00;ooooo001Aooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo0000<oooool5o`00
00;ooooo00Co0000oooooooooooo0000<oooool01Ol0003ooooooooooooooooo000000;ooooo00Co
0000oooooooooooo0000<oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooo
oooo0000<_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000
oooooooooooo0000;oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool005;ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`00
0?oooooooooo037ooooo00Co0000oooooooooooo00000oooool01?l0003oooooooooool0000coooo
o`Co00000oooool01?l0003oooooooooool0000dooooo`?o00000oooool01?l0003oooooooooool0
000booooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003ooooo
ooooool0000_ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000002oooo
o`7o00000Oooool1ooooo`00Coooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooo
oooooooo0000=?ooool00ol0003oooooo`000003ooooo`04o`000?ooooooooooo`0003?ooooo00?o
0000ooooooooool01?ooool01?l0003oooooooooool0000cooooo`05o`000?ooooooooooooooool0
00000_ooool01?l0003oooooooooool0000booooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000_ooool01?l0003oooooooooool0000_ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00002ooooo`7o00000Oooool1ooooo`00Coooool01Ol0003ooooo
oooooooooooo000000;ooooo00Co0000oooooooooooo0000=Oooool2o`0000?ooooo00Co0000oooo
oooooooo0000=?ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0003?ooooo00Go
0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`00037ooooo0_l00004ooooo`04
o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo0000;_ooool2o`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000;ooooo001@ooooo`?o00001?oo
ool2o`0003Oooooo00?o0000ooooooooool00_ooool2o`0003Kooooo0_l00004ooooo`;o0000=Ooo
ool3o`0000Cooooo0_l0000cooooo`03o`000?oooooooooo00?ooooo0_l00004ooooo`;o0000<?oo
ool00ol0003oooooooooo`02ooooo`?o00001?ooool2o`0000?ooooo000Fooooo`03o`000?oooooo
oooo0?ooooooOOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003o
ooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o
0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooc;ooooo00?o0000
ooooooooool0B?ooool000goooooool00029o`0000005_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0<ooooo`;o
00003_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00Oooooo000Fooooo`03o`00
0?oooooooooo03cooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01kooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`0kooooo`03o`00
0?oooooooooo03_ooooo00?o0000ooooooooool01oooool001Kooooo00?o0000ooooooooool0oooo
oommooooo`005_ooool00ol0003oooooooooo`2Fooooo`03o`000?oooooooooo0>?ooooo000Foooo
o`03o`000?oooooooooo09Gooooo00?o0000ooooooooool0i?ooool001Kooooo00?o0000oooooooo
ool0oooooommooooo`005_ooool00ol0003oooooooooo`2Dooooo`03o`000?oooooooooo0>Gooooo
000Fooooo`03o`000?oooooooooo09Cooooo00?o0000ooooooooool0iOooool001Kooooo00?o0000
ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`?o
0000T_ooool2o`000>Sooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o
0000ooooooooool0T?ooool00ol0003oooooooooo`3Yooooo`005_ooool00ol0003oooooooooo`3o
oooooggooooo000Fooooo`03o`000?oooooooooo08gooooo00?o0000ooooooooool0k?ooool001Ko
oooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`2:ooooo`03o`00
0?oooooooooo0>oooooo000Fooooo`03o`000?oooooooooo08Oooooo00?o0000ooooooooool0l_oo
ool001Kooooo00?o0000ooooooooool0Q?ooool00ol0003oooooooooo`3eooooo`005_ooool00ol0
003oooooooooo`24ooooo`03o`000?oooooooooo0?Gooooo000Fooooo`03o`000?oooooooooo0?oo
ooooOOooool001Kooooo00?o0000ooooooooool0P_ooool00ol0003oooooooooo`3gooooo`005_oo
ool3o`000?ooooooOOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0
003oooooooooo`20ooooo`03o`000?oooooooooo0?Wooooo000Fooooo`03o`000?oooooooooo0?oo
ooooOOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003ooooooooo
o`1nooooo`03o`000?oooooooooo0?_ooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool0
01Kooooo00?o0000ooooooooool0O?ooool00ol0003oooooooooo`3mooooo`005_ooool00ol0003o
ooooooooo`1_ooooo`;o000000?oooooo`000?ooool01oooool00ol0003oooooooooo`3nooooo`00
5_ooool00ol0003oooooooooo`1Tooooo`03o`000?oooooooooo00;o00003?ooool00ol0003ooooo
ooooo`02o`000?oooooo0oooool001Kooooo00?o0000ooooooooool0K_ooool00ol0003ooooooooo
o`3oooooo`cooooo000Fooooo`?o0000HOooool00ol0003oooooooooo`06ooooo`;o000000?ooooo
o`000?ooool0ooooool>ooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03
o`000?oooooooooo05kooooo0_l0003ooooooagooooo000Fooooo`03o`000?oooooooooo0?oooooo
OOooool001Kooooo00?o0000ooooooooool0Foooool2o`000?oooooo8?ooool001Kooooo00?o0000
ooooooooool0B?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o00000_ooool0
0ol0003oooooooooo`02o`000?oooooo8oooool001Kooooo00?o0000ooooooooool0A_ooool00ol0
003oooooooooo`3oooooocCooooo000Fooooo`03o`000?oooooooooo04;ooooo00Co0000oooooooo
oooo00001Oooool00ol0003oooooooooo`3ooooooboooooo000Fooooo`03o`000?oooooooooo0?oo
ooooOOooool000Kooooo1Ol00003ooooo`;o00001_ooool00ol0003oooooooooo`0aooooo`;o0000
0_ooool00ol0003oooooooooo`02o`0000;ooooo00Co0000oooooooooooo0000oooooolmooooo`00
1oooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000oooo
ooooool0;_ooool00ol0003oooooooooo`3oooooodcooooo0008ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`3ooooooggooooo0009ooooo`05
o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`03ooooo`Co00009?ooool01?l0
003oooooo`000?l00002ooooo`03o`000?oooooooooo0?ooooooCoooool000Kooooo00Go0000oooo
ooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool09?oo
ool00ol0003oooooooooo`3ooooooeKooooo0006ooooo`05o`000?ooooooooooooooool000000_oo
ool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo027ooooo0_l0003ooooooe[ooooo
0007ooooo`?o00001?ooool2o`0000Kooooo00?o0000ooooooooool07oooool00ol0003ooooooooo
o`3ooooooe_ooooo000Fooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0oooooomL
ooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo01go
oooo00?o0000ooooooooool0oooooomMooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo
000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o0000ooooooooool06oooool0
0ol0003oooooooooo`3ooooooeoooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Ko
oooo0ol0003ooooooggooooo000Fooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
oooooomQooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooo
oooo0?ooooooOOooool001Kooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`3ooooo
of;ooooo000Fooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0oooooomRooooo`00
5_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOoo
ool001Kooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`3oooooof?ooooo000Foooo
o`03o`000?oooooooooo01Cooooo0_l0003oooooofOooooo000Fooooo`03o`000?oooooooooo0?oo
ooooOOooool001Kooooo00?o0000ooooooooool04Oooool2o`000?ooooooJ_ooool001Kooooo0ol0
000>ooooo`03o`000?oooooooooo0?ooooooK?ooool001Kooooo00?o0000ooooooooool02oooool0
0ol0003oooooooooo`3oooooofoooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Ko
oooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo0?ooooooLOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool0
0ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool0oooooombooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`00
0?oooooooooo0?ooooooOOooool001Kooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`3oooooogCooooo000Fooooo`?o0000oooooommooooo`005_ooool00ol0003oooooooooo`3ooooo
oggooooo000Fooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0oooooomeooooo`00
5_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo0?ooooooMOooool001Kooooo00?o
0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Foooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0oooooomfooooo`005_ooool00ol0003o
ooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool000Sooooo0ol0
0003ooooo`;o00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0?ooooooMooo
ool000Wooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool0oooooommooooo`001_ooool5o`0000;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`3ooooooggooooo0006ooooo`04o`000?ooooooooooo`0000?ooooo00Co
0000oooooooooooo00001Oooool4o`0000;ooooo00?o0000ooooooooool0oooooomgooooo`001ooo
ool00ol0003oooooo`000003ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`3oooooogOooooo0008ooooo`;o00000oooool01?l0003ooooooooo
ool00005ooooo`03o`000?oooooooooo0?ooooooOOooool000Wooooo00?o0000ooooooooool00_oo
ool2o`0000Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo0?ooooooN?ooool001Kooooo00?o0000ooooooooool0oooooommoooo
o`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0oooooomhooooo`005_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo0?ooooooN?ooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0
003oooooooooo`3ooooooggooooo000Fooooo`?o000000?oooooo`000?ooool0oooooomjooooo`00
5_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOoo
ool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooo
oggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o0000ooooooooool0
oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooo
oooo0?ooooooOOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool00ol0003o
ooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo0ol0
003ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o0000oooo
ooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`00
0?oooooooooo0?ooooooOOooool001Kooooo00?o0000ooooooooool0oooooommooooo`005_ooool0
0ol0003oooooooooo`3ooooooggooooo000Fooooo`03o`000?oooooooooo0?ooooooOOooool001Ko
oooo00?o0000ooooooooool0oooooommooooo`005_ooool01Ol0003ooooooooooooooooo00000?oo
ooooNoooool001Kooooo00Go0000ooooooooooooooooo`00003oooooog_ooooo000Fooooo`?o0000
00?oooooo`000?ooool0oooooomjooooo`005_ooool01Ol0003ooooooooooooooooo00000?oooooo
Noooool001Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`3oooooogSooooo000F
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0oooooomhooooo`005_ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo0?ooooooN?ooool001Kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`3oooooogSooooo000Fooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0oooooomhooooo`005_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo0?ooooooN?ooool001Kooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3o
ooooogOooooo0007ooooo`?o00001?ooool2o`0000Kooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`3oooooogOooooo0006ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0oooooomg
ooooo`001_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo0?ooooooMoooool000Kooooo1?l00003
ooooo`04o`000?ooooooooooo`0000Gooooo1?l00002ooooo`03o`000?oooooooooo0?ooooooMooo
ool000Kooooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00005ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool0oooooomgooooo`001oooool00ol0003oooooooooo`03
ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`3oooooogKooooo0008ooooo`;o00001?ooool2o`0000Kooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`3oooooogKooooo000Fooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool0oooooomfooooo`005_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo0?oo
ooooM_ooool001Kooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`3oooooogGooooo
000Fooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0oooooomeooooo`005_ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo0?ooooooMOooool001Kooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`3oooooogCooooo000Fooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool0oooooomdooooo`005_ooool3o`0000Kooooo00?o0000ooooooooool0
oooooomdooooo`005_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo0?ooooooLooo
ool001Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`3oooooog?ooooo000Foooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0oooooomcooooo`005_ooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo0?ooooooL_ooool001Kooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`3oooooog;ooooo000Fooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool0oooooomaooooo`005_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo0?ooooooLOooool001Kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`3ooooo
og7ooooo000Fooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0oooooom`ooooo`00
5_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo0?ooooooKoooool001Kooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`3oooooofkooooo000Fooooo`?o00003Oooool5
o`000?ooooooJoooool001Kooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`3ooooo
ofSooooo000Fooooo`03o`000?oooooooooo01?ooooo1?l0003oooooofKooooo000Fooooo`03o`00
0?oooooooooo01Kooooo00?o0000ooooooooool0oooooomTooooo`005_ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo0?ooooooHoooool001Kooooo00?o0000ooooooooool05oooool0
0ol0003oooooooooo`3oooooof?ooooo000Fooooo`03o`000?oooooooooo01Oooooo00?o0000oooo
ooooool0oooooomSooooo`005_ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo0?oo
ooooH_ooool001Kooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`3oooooof;ooooo
000Fooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0oooooomRooooo`005_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo0?ooooooHOooool001Kooooo0ol0000Ioooo
o`03o`000?oooooooooo0?ooooooHOooool001Kooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`3oooooof7ooooo000Fooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
oooooomPooooo`005_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo0?ooooooGooo
ool001Kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`3ooooooekooooo000Foooo
o`03o`000?oooooooooo01gooooo00?o0000ooooooooool0oooooomMooooo`005_ooool00ol0003o
ooooooooo`0Mooooo`03o`000?oooooooooo0?ooooooGOooool001Kooooo00?o0000ooooooooool0
7_ooool00ol0003oooooooooo`3ooooooecooooo000Fooooo`03o`000?oooooooooo01oooooo00?o
0000ooooooooool0oooooomKooooo`001oooool3o`0000Cooooo0_l00006ooooo`03o`000?oooooo
oooo023ooooo1?l0003ooooooeWooooo0006ooooo`05o`000?ooooooooooooooool000000_ooool0
1?l0003oooooooooool00005ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0oooo
oomFooooo`001_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo0000
1Oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo0?ooooooEOooool000Oooooo0ol0
0003ooooo`04o`000?ooooooooooo`0000Gooooo1?l0000Uooooo`So0000oooooom?ooooo`001_oo
ool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00001Oooool00ol0003o
ooooooooo`0^ooooo`03o`000?oooooooooo0?ooooooC?ooool000Kooooo00Go0000oooooooooooo
ooooo`000002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0;ooooolBo`00
0?oooooo??ooool000Oooooo0ol00004ooooo`;o00001_ooool00ol0003oooooooooo`11ooooo`03
o`000?oooooooooo0?oooooo>Oooool001Kooooo00?o0000ooooooooool0@_ooool4o`0000Gooooo
00?o0000ooooooooool0ooooool_ooooo`005_ooool00ol0003oooooooooo`16ooooo`07o`000?oo
ooooooooooooool0003oooooo`00003ooooooc3ooooo000Fooooo`03o`000?oooooooooo04Oooooo
00?o0000ooooool000000oooool00ol0003oooooooooo`3oooooobgooooo000Fooooo`03o`000?oo
oooooooo04Sooooo00?o0000ooooooooool00oooool?o`000?oooooo8?ooool001Kooooo00?o0000
ooooooooool0GOooool00ol0003oooooooooo`3ooooooagooooo000Fooooo`03o`000?oooooooooo
05kooooo0_l0003ooooooagooooo000Fooooo`03o`000?oooooooooo063ooooo00?o0000oooooooo
ool0oooooolJooooo`005_ooool3o`00067ooooo00?o0000ooooooooool01_ooool4o`000?oooooo
3oooool001Kooooo00?o0000ooooooooool0H_ooool00ol0003oooooooooo`04ooooo`05o`000?oo
ooooooooooooool00000ooooool?ooooo`005_ooool00ol0003oooooooooo`1Sooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3oooooo`cooooo000F
ooooo`03o`000?oooooooooo06Cooooo1?l00007ooooo`03o`000?oooooooooo00[o0000ooooool1
ooooo`005_ooool00ol0003oooooooooo`1_ooooo`03o`000?oooooo000000[ooooo00?o0000oooo
ooooool0oOooool001Kooooo00?o0000ooooooooool0L?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo0?cooooo000Fooooo`03o`000?oooooooooo07gooooo00?o0000ooooooooool0
o?ooool001Kooooo00?o0000ooooooooool0O_ooool00ol0003oooooooooo`3kooooo`005_ooool0
0ol0003oooooooooo`1oooooo`03o`000?oooooooooo0?[ooooo000Fooooo`03o`000?oooooooooo
083ooooo00?o0000ooooooooool0nOooool001Kooooo00?o0000ooooooooool0P?ooool00ol0003o
ooooooooo`3iooooo`005_ooool00ol0003oooooooooo`21ooooo`03o`000?oooooooooo0?Sooooo
000Fooooo`?o0000P_ooool00ol0003oooooooooo`3gooooo`005_ooool00ol0003oooooooooo`22
ooooo`03o`000?oooooooooo0?Oooooo000Fooooo`03o`000?oooooooooo08?ooooo0_l0003goooo
o`005_ooool00ol0003oooooooooo`25ooooo`03o`000?oooooooooo0?Cooooo000Fooooo`03o`00
0?oooooooooo08Kooooo1?l0003booooo`005_ooool00ol0003oooooooooo`2:ooooo`03o`000?oo
oooooooo0>oooooo000Fooooo`03o`000?oooooooooo08_ooooo00?o0000ooooooooool0k_ooool0
01Kooooo00?o0000ooooooooool0S?ooool2o`000>kooooo000Fooooo`03o`000?oooooooooo08ko
oooo00?o0000ooooooooool0joooool001Kooooo00?o0000ooooooooool0Soooool2o`000>_ooooo
000Fooooo`03o`000?oooooooooo097ooooo00?o0000ooooooooool0j?ooool001Kooooo0ol0002B
ooooo`03o`000?oooooooooo0>Oooooo000Fooooo`03o`000?oooooooooo09;ooooo00?o0000oooo
ooooool0ioooool001Kooooo00?o0000ooooooooool0Toooool00ol0003oooooooooo`3Vooooo`00
5_ooool00ol0003oooooooooo`2Cooooo`03o`000?oooooooooo0>Kooooo000Fooooo`03o`000?oo
oooooooo09Cooooo00?o0000ooooooooool0iOooool001Kooooo00?o0000ooooooooool0UOooool0
0ol0003oooooooooo`3Tooooo`005_ooool00ol0003oooooooooo`2Eooooo`03o`000?oooooooooo
0>Cooooo000Fooooo`03o`000?oooooooooo09Kooooo00?o0000ooooooooool0hoooool001Kooooo
00?o0000ooooooooool0U_ooool00ol0003oooooooooo`3Sooooo`0000Coooooo`000?l0003o0000
1?ooool2o`0000Cooooo0_l00006ooooo`03o`000?oooooooooo09Oooooo00?o0000ooooooooool0
h_ooool000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`00
0?ooooooooooo`0000Gooooo00?o0000ooooooooool0V?ooool00ol0003oooooooooo`3Qooooo`00
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooo
oooo00001Oooool00ol0003oooooooooo`2Hooooo`03o`000?oooooooooo0>7ooooo0002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00005
ooooo`Co0000V?oooooJo`0000Wooooo0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo0?ooooooOOoo
ool00003ooooool0003o000000Cooooo00Co0000oooooooooooo00000_ooool01?l0003ooooooooo
ool00005ooooo`03o`000?oooooooooo0?ooooooOOooool000;ooooo00?o0000ooooooooool00ooo
ool2o`0000Cooooo0_l00006ooooo`03o`000?oooooooooo0?ooooooOOooool001Kooooo00?o0000
ooooooooool0oooooommooooo`005_ooool00ol0003oooooooooo`3ooooooggooooo0000\
\>"],
  ImageRangeCache->{{{0, 404.812}, {249.812, 0}} -> {-7.40523, -6.03349, \
0.322213, 0.434459}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Well, that's nice. It would be better if we drew the lines using \
the same color as we used for representing the strategies.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(MultipleListPlot[strategy1, 
        strategy2, \[IndentingNewLine]SymbolShape \[Rule] 
          None, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]PlotStyle \[Rule] {Red, 
            Green}\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00793651 0.0147151 0.00588604 [
[.18254 .00222 -6 -9 ]
[.18254 .00222 6 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.65873 .00222 -6 -9 ]
[.65873 .00222 6 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .13244 -12 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -12 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -12 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -12 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -18 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(20)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(40)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(80)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(100)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(120)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.06349 .01472 m
.06349 .01847 L
s
.10317 .01472 m
.10317 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.22222 .01472 m
.22222 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.30159 .01472 m
.30159 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.42063 .01472 m
.42063 .01847 L
s
.46032 .01472 m
.46032 .01847 L
s
.53968 .01472 m
.53968 .01847 L
s
.57937 .01472 m
.57937 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.69841 .01472 m
.69841 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.77778 .01472 m
.77778 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.89683 .01472 m
.89683 .01847 L
s
.93651 .01472 m
.93651 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(20)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(40)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(60)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(80)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(100)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.001 w
.03175 .33256 m
.03968 .37965 L
.04762 .40319 L
.05556 .42085 L
.06349 .42674 L
.07143 .42674 L
.07937 .43262 L
.0873 .43262 L
.09524 .45617 L
.10317 .46794 L
.11111 .47971 L
.11905 .47971 L
.12698 .4856 L
.13492 .4856 L
.14286 .4856 L
.15079 .49148 L
.15873 .49148 L
.16667 .49148 L
.1746 .49148 L
.18254 .49148 L
.19048 .49148 L
.19841 .49737 L
.20635 .49737 L
.21429 .50326 L
.22222 .49737 L
.23016 .50326 L
.2381 .50326 L
.24603 .50326 L
.25397 .50326 L
.2619 .50326 L
.26984 .50914 L
.27778 .51503 L
.28571 .52091 L
.29365 .52091 L
.30159 .51503 L
.30952 .51503 L
.31746 .5268 L
.3254 .52091 L
.33333 .52091 L
.34127 .52091 L
.34921 .5268 L
.35714 .53857 L
.36508 .55034 L
.37302 .55623 L
.38095 .55623 L
.38889 .56212 L
.39683 .568 L
.40476 .57389 L
.4127 .59155 L
.42063 .60332 L
Mistroke
.42857 .60332 L
.43651 .60332 L
.44444 .60332 L
.45238 .60332 L
.46032 .60332 L
.46825 .60332 L
.47619 .60332 L
.48413 .60332 L
.49206 .60332 L
.5 .60332 L
.50794 .60332 L
.51587 .60332 L
.52381 .60332 L
.53175 .60332 L
.53968 .60332 L
.54762 .60332 L
.55556 .60332 L
.56349 .60332 L
.57143 .60332 L
.57937 .60332 L
.5873 .60332 L
.59524 .60332 L
.60317 .60332 L
.61111 .60332 L
.61905 .60332 L
.62698 .60332 L
.63492 .60332 L
.64286 .60332 L
.65079 .60332 L
.65873 .60332 L
.66667 .60332 L
.6746 .60332 L
.68254 .60332 L
.69048 .60332 L
.69841 .60332 L
.70635 .60332 L
.71429 .60332 L
.72222 .60332 L
.73016 .60332 L
.7381 .60332 L
.74603 .60332 L
.75397 .60332 L
.7619 .60332 L
.76984 .60332 L
.77778 .60332 L
.78571 .60332 L
.79365 .60332 L
.80159 .60332 L
.80952 .60332 L
.81746 .60332 L
Mistroke
.8254 .60332 L
.83333 .60332 L
.84127 .60332 L
.84921 .60332 L
.85714 .60332 L
.86508 .60332 L
.87302 .60332 L
.88095 .60332 L
.88889 .60332 L
.89683 .60332 L
.90476 .60332 L
.9127 .60332 L
.92063 .60332 L
.92857 .60332 L
.93651 .60332 L
.94444 .60332 L
.95238 .60332 L
.96032 .60332 L
.96825 .60332 L
.97619 .60332 L
Mfstroke
0 1 0 r
.03175 .28547 m
.03968 .23838 L
.04762 .21484 L
.05556 .19718 L
.06349 .1913 L
.07143 .1913 L
.07937 .18541 L
.0873 .18541 L
.09524 .16187 L
.10317 .15009 L
.11111 .13832 L
.11905 .13832 L
.12698 .13244 L
.13492 .13244 L
.14286 .13244 L
.15079 .12655 L
.15873 .12655 L
.16667 .12655 L
.1746 .12655 L
.18254 .12655 L
.19048 .12655 L
.19841 .12066 L
.20635 .12066 L
.21429 .11478 L
.22222 .12066 L
.23016 .11478 L
.2381 .11478 L
.24603 .11478 L
.25397 .11478 L
.2619 .11478 L
.26984 .10889 L
.27778 .10301 L
.28571 .09712 L
.29365 .09712 L
.30159 .10301 L
.30952 .10301 L
.31746 .09123 L
.3254 .09712 L
.33333 .09712 L
.34127 .09712 L
.34921 .09123 L
.35714 .07946 L
.36508 .06769 L
.37302 .0618 L
.38095 .0618 L
.38889 .05592 L
.39683 .05003 L
.40476 .04415 L
.4127 .02649 L
.42063 .01472 L
Mistroke
.42857 .01472 L
.43651 .01472 L
.44444 .01472 L
.45238 .01472 L
.46032 .01472 L
.46825 .01472 L
.47619 .01472 L
.48413 .01472 L
.49206 .01472 L
.5 .01472 L
.50794 .01472 L
.51587 .01472 L
.52381 .01472 L
.53175 .01472 L
.53968 .01472 L
.54762 .01472 L
.55556 .01472 L
.56349 .01472 L
.57143 .01472 L
.57937 .01472 L
.5873 .01472 L
.59524 .01472 L
.60317 .01472 L
.61111 .01472 L
.61905 .01472 L
.62698 .01472 L
.63492 .01472 L
.64286 .01472 L
.65079 .01472 L
.65873 .01472 L
.66667 .01472 L
.6746 .01472 L
.68254 .01472 L
.69048 .01472 L
.69841 .01472 L
.70635 .01472 L
.71429 .01472 L
.72222 .01472 L
.73016 .01472 L
.7381 .01472 L
.74603 .01472 L
.75397 .01472 L
.7619 .01472 L
.76984 .01472 L
.77778 .01472 L
.78571 .01472 L
.79365 .01472 L
.80159 .01472 L
.80952 .01472 L
.81746 .01472 L
Mistroke
.8254 .01472 L
.83333 .01472 L
.84127 .01472 L
.84921 .01472 L
.85714 .01472 L
.86508 .01472 L
.87302 .01472 L
.88095 .01472 L
.88889 .01472 L
.89683 .01472 L
.90476 .01472 L
.9127 .01472 L
.92063 .01472 L
.92857 .01472 L
.93651 .01472 L
.94444 .01472 L
.95238 .01472 L
.96032 .01472 L
.96825 .01472 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[17]:=",
  ImageSize->{402.125, 248.562},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006B0000n2000`400?l00000o`00003ooooooonC
ooooo`00C_ooool5o`0000?ooooo0_l0000eooooo`?o00000oooool2o`0003Gooooo0ol00004oooo
o`;o0000=?ooool3o`0000Cooooo0_l0000booooo`?o00001?ooool2o`0000Cooooo0_l0000^oooo
o`?o00000_ooool5o`0000?ooooo0_l00004ooooo`00Coooool01?l0003oooooooooool00002oooo
o`04o`000?ooooooooooo`0003Gooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo037ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0003;o
oooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0003;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0002kooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`00
00?ooooo001@ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo0000<_ooool5o`00
00;ooooo00Co0000oooooooooooo0000<oooool01Ol0003ooooooooooooooooo000000;ooooo00Co
0000oooooooooooo0000<_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooo
oooo0000<_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000
oooooooooooo0000;_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000oooool0057ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`00
0?oooooooooo033ooooo00Co0000oooooooooooo00000oooool01?l0003oooooooooool0000coooo
o`Co00000oooool01?l0003oooooooooool0000cooooo`?o00000oooool01?l0003oooooooooool0
000booooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003ooooo
ooooool0000^ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000002oooo
o`03o`000?oooooooooo007ooooo001>ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000cooooo`03o`000?oooooo000000?ooooo00Co0000oooooooooooo0000<ooo
ool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0003;ooooo00Go0000oooooooooooo
ooooo`000002ooooo`04o`000?ooooooooooo`0003;ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0002kooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00Oooool004kooooo
00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0003Cooooo0_l00003oooo
o`04o`000?ooooooooooo`0003Cooooo00?o0000ooooooooool00oooool01?l0003oooooooooool0
000booooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool0000aooooo`;o
00001?ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0002gooooo0_l00003
ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003ooooo`00Cooo
ool3o`0000Cooooo0_l0000fooooo`03o`000?oooooooooo00;ooooo0_l0000fooooo`;o00001?oo
ool2o`0003Cooooo0ol00004ooooo`;o0000<oooool00ol0003oooooooooo`03ooooo`;o00001?oo
ool2o`0002oooooo00?o0000ooooooooool00_ooool3o`0000Cooooo0_l00004ooooo`005_ooool0
0ol0003oooooooooo`3oooooog[ooooo000Fooooo`03o`000?oooooooooo0?ooooooN_ooool001Ko
oooo00?o0000ooooooooool0oooooomjooooo`005_ooool00ol0003oooooooooo`3oooooog[ooooo
000Fooooo`03o`000?oooooooooo0?ooooooN_ooool001Kooooo00?o0000ooooooooool0oooooomj
ooooo`003OoooonSo`000=So0?l02_l0000001Kooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool00ol0o`3o
ooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool02?ooool001Kooooo
00?o0000ooooooooool0>oooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo01_ooooo
00?o0?l0ooooooooool07Oooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo03_ooooo
00?o0000ooooooooool0>_ooool00ol0003oooooooooo`08ooooo`005_ooool00ol0003ooooooooo
o`2Eooooo`03o`3o0?oooooooooo0>7ooooo000Fooooo`03o`000?oooooooooo09Cooooo00?o0?l0
ooooooooool0h_ooool001Kooooo00?o0000ooooooooool0U?ooool00ol0o`3oooooooooo`3Roooo
o`005_ooool00ol0003oooooooooo`2Cooooo`03o`3o0?oooooooooo0>?ooooo000Fooooo`03o`00
0?oooooooooo09?ooooo00?o0?l0ooooooooool0hoooool001Kooooo00?o0000ooooooooool0T_oo
ool00ol0o`3oooooooooo`3Tooooo`005_ooool00ol0003oooooooooo`2Booooo`03o`3o0?oooooo
oooo0>Cooooo000Fooooo`03o`000?oooooooooo09;ooooo00?o0?l0ooooooooool0i?ooool001Ko
oooo0ol0002@ooooo`;o0?l0ioooool001Kooooo00?o0000ooooooooool0Soooool00ol0o`3ooooo
ooooo`3Wooooo`005_ooool00ol0003oooooooooo`2>ooooo`03o`3o0?oooooooooo0>Sooooo000F
ooooo`03o`000?oooooooooo08gooooo00?o0?l0ooooooooool0jOooool001Kooooo00?o0000oooo
ooooool0S?ooool00ol0o`3oooooooooo`3Zooooo`005_ooool00ol0003oooooooooo`2:ooooo`;o
0?l0kOooool001Kooooo00?o0000ooooooooool0ROooool00ol0o`3oooooooooo`3]ooooo`005_oo
ool00ol0003oooooooooo`24ooooo`Go0?l0l?ooool001Kooooo00?o0000ooooooooool0Poooool0
0ol0o`3oooooooooo`3cooooo`005_ooool00ol0003oooooooooo`22ooooo`03o`3o0?oooooooooo
0?Cooooo000Fooooo`03o`000?oooooooooo08;ooooo00?o0?l0ooooooooool0m?ooool001Kooooo
00?o0000ooooooooool0POooool00ol0o`3oooooooooo`3eooooo`005_ooool3o`00083ooooo00?o
0?l0ooooooooool0m_ooool001Kooooo00?o0000ooooooooool0P?ooool00ol0o`3oooooooooo`3f
ooooo`005_ooool00ol0003oooooooooo`1oooooo`03o`3o0?oooooooooo0?Oooooo000Fooooo`03
o`000?oooooooooo07kooooo00?o0?l0ooooooooool0n?ooool001Kooooo00?o0000ooooooooool0
OOooool00ol0o`3oooooooooo`3iooooo`005_ooool00ol0003oooooooooo`1looooo`03o`3o0?oo
oooooooo0?[ooooo000Fooooo`03o`000?oooooooooo06oooooo00?o0?l0ooooooooool02Oooool0
0ol0o`3oooooooooo`3kooooo`005_ooool00ol0003oooooooooo`1_ooooo`;o0?l02Oooool00ol0
o`3oooooooooo`3looooo`005_ooool00ol0003oooooooooo`1^ooooo`04o`3o0?ooooooooooo`3o
00Oooooo00?o0?l0ooooooooool0oOooool001Kooooo00?o0000ooooooooool0H_ooool6o`3o00Go
oooo00?o0?l0ooooooooool00_ooool7o`3o0?oooooo0Oooool001Kooooo00?o0000ooooooooool0
HOooool00ol0o`3oooooooooo`04ooooo`03o`3o0?oooooooooo00;ooooo00?o0?l0ooooooooool0
ooooool:ooooo`005_ooool3o`0005oooooo0_l0o`08ooooo`Co0?l0ooooool=ooooo`005_ooool0
0ol0003oooooooooo`1Nooooo`03o`3o0?oooooooooo0?oooooo6Oooool001Kooooo00?o0000oooo
ooooool0GOooool00ol0o`3oooooooooo`3ooooooa[ooooo000Fooooo`03o`000?oooooooooo05co
oooo00?o0?l0ooooooooool0oooooolKooooo`005_ooool00ol0003oooooooooo`1Kooooo`03o`3o
0?oooooooooo0?oooooo7?ooool001Kooooo00?o0000ooooooooool0A_ooool3o`3o00?ooooo3ol0
o`3ooooooaoooooo000Fooooo`03o`000?oooooooooo04Gooooo00Oo0?l0ooooooooooooooooo`3o
0?oooooo0?l00?oooooo;_ooool001Kooooo00?o0000ooooooooool0@?ooool5o`3o00Gooooo00?o
0?l0ooooooooool0ooooool]ooooo`005_ooool00ol0003oooooooooo`0oooooo`03o`3o0?oooooo
oooo0?oooooo>?ooool000Kooooo1Ol00003ooooo`;o00001_ooool00ol0003oooooooooo`0^oooo
oa7o0?l0oooooolkooooo`001oooool01?l0003oooooooooool00002ooooo`04o`000?oooooooooo
o`0000Gooooo00?o0000ooooooooool0;Oooool00ol0o`3oooooooooo`3ooooood[ooooo0008oooo
o`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Oooool4o`0002Cooooo2?l0o`3o
ooooodgooooo0009ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo02Cooooo00?o0?l0ooooooooool0oooooomCooooo`001_ooool01Ol0
003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00001Oooool00ol0003ooooooooo
o`0Sooooo`03o`3o0?oooooooooo0?ooooooE?ooool000Kooooo00Go0000ooooooooooooooooo`00
0002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool07oooool4o`3o0?oooooo
Eoooool000Oooooo0ol00004ooooo`;o00001_ooool00ol0003oooooooooo`0Oooooo`03o`3o0?oo
oooooooo0?ooooooF?ooool001Kooooo00?o0000ooooooooool07_ooool00ol0o`3oooooooooo`3o
oooooeWooooo000Fooooo`03o`000?oooooooooo01gooooo00?o0?l0ooooooooool0oooooomJoooo
o`005_ooool00ol0003oooooooooo`0Looooo`03o`3o0?oooooooooo0?ooooooFoooool001Kooooo
00?o0000ooooooooool07?ooool00ol0o`3oooooooooo`3ooooooe_ooooo000Fooooo`03o`000?oo
oooooooo01_ooooo00?o0?l0ooooooooool0oooooomLooooo`005_ooool00ol0003oooooooooo`0J
ooooo`03o`3o0?oooooooooo0?ooooooGOooool001Kooooo00?o0000ooooooooool06_ooool00ol0
o`3oooooooooo`3ooooooegooooo000Fooooo`?o00006Oooool00ol0o`3oooooooooo`3ooooooeko
oooo000Fooooo`03o`000?oooooooooo01Wooooo00?o0?l0ooooooooool0oooooomNooooo`005_oo
ool00ol0003oooooooooo`0Hooooo`03o`3o0?oooooooooo0?ooooooGoooool001Kooooo00?o0000
ooooooooool06?ooool00ol0o`3oooooooooo`3ooooooeoooooo000Fooooo`03o`000?oooooooooo
01Sooooo00?o0?l0ooooooooool0oooooomOooooo`005_ooool00ol0003oooooooooo`0Gooooo`03
o`3o0?oooooooooo0?ooooooH?ooool001Kooooo00?o0000ooooooooool05oooool00ol0o`3ooooo
ooooo`3oooooof3ooooo000Fooooo`03o`000?oooooooooo01Oooooo00?o0?l0ooooooooool0oooo
oomPooooo`005_ooool00ol0003oooooooooo`0Fooooo`03o`3o0?oooooooooo0?ooooooHOooool0
01Kooooo00?o0000ooooooooool04_ooool5o`3o0?ooooooHoooool001Kooooo00?o0000oooooooo
ool04Oooool00ol0o`3oooooooooo`3oooooofKooooo000Fooooo`?o00003?ooool5o`3o0?oooooo
JOooool001Kooooo00?o0000ooooooooool02oooool00ol0o`3oooooooooo`3oooooofcooooo000F
ooooo`03o`000?oooooooooo00[ooooo00?o0?l0ooooooooool0oooooom]ooooo`005_ooool00ol0
003oooooooooo`0:ooooo`03o`3o0?oooooooooo0?ooooooKOooool001Kooooo00?o0000oooooooo
ool02Oooool00ol0o`3oooooooooo`3oooooofkooooo000Fooooo`03o`000?oooooooooo00Wooooo
00?o0?l0ooooooooool0oooooom^ooooo`005_ooool00ol0003oooooooooo`08ooooo`03o`3o0?oo
oooooooo0?ooooooKoooool001Kooooo00?o0000ooooooooool02?ooool00ol0o`3oooooooooo`3o
ooooofoooooo000Fooooo`03o`000?oooooooooo00Sooooo00?o0?l0ooooooooool0oooooom_oooo
o`005_ooool00ol0003oooooooooo`07ooooo`03o`3o0?oooooooooo0?ooooooL?ooool001Kooooo
00?o0000ooooooooool01oooool00ol0o`3oooooooooo`3oooooog3ooooo000Fooooo`03o`000?oo
oooooooo00Kooooo00?o0?l0ooooooooool0oooooomaooooo`005_ooool3o`0000Kooooo00?o0?l0
ooooooooool0oooooomaooooo`005_ooool00ol0003oooooooooo`06ooooo`03o`3o0?oooooooooo
0?ooooooLOooool001Kooooo00?o0000ooooooooool01Oooool00ol0o`3oooooooooo`3oooooog;o
oooo000Fooooo`03o`000?oooooooooo00Gooooo00?o0?l0ooooooooool0oooooombooooo`005_oo
ool00ol0003oooooooooo`05ooooo`03o`3o0?oooooooooo0?ooooooL_ooool001Kooooo00?o0000
ooooooooool01?ooool00ol0o`3oooooooooo`3oooooog?ooooo000Fooooo`03o`000?oooooooooo
00Cooooo00?o0?l0ooooooooool0oooooomcooooo`005_ooool00ol0003oooooooooo`04ooooo`03
o`3o0?oooooooooo0?ooooooLoooool000Sooooo0ol00003ooooo`;o00001_ooool00ol0003ooooo
ooooo`04ooooo`03o`3o0?oooooooooo0?ooooooLoooool000Wooooo00Go0000oooooooooooooooo
o`000002ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0o`3ooooo
ooooo`3oooooogCooooo0006ooooo`Go00000_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00?ooooo00?o0?l0ooooooooool0oooooomdooooo`001_ooool01?l0003ooooooooo
ool00003ooooo`04o`000?ooooooooooo`0000Gooooo1?l00002ooooo`03o`3o0?oooooooooo0?oo
ooooM?ooool000Oooooo00?o0000ooooool000000oooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0?l0ooooooooool0oooooomdooooo`002?ooool2o`0000?ooooo
00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`3o0?oooooooooo0?oo
ooooM?ooool000Wooooo00?o0000ooooooooool00_ooool2o`0000Kooooo00?o0000ooooooooool0
0_ooool00ol0o`3oooooooooo`3oooooogGooooo000Fooooo`03o`000?oooooooooo00;ooooo00?o
0?l0ooooooooool0oooooomeooooo`005_ooool00ol0003oooooooooo`02ooooo`03o`3o0?oooooo
oooo0?ooooooMOooool001Kooooo00?o0000ooooooooool00_ooool00ol0o`3oooooooooo`3ooooo
ogGooooo000Fooooo`03o`000?oooooooooo00;ooooo00?o0?l0ooooooooool0oooooomeooooo`00
5_ooool01Ol0003ooooooooooooooooo0?l00?ooooooN?ooool001Kooooo00Go0000oooooooooooo
ooooo`3o003oooooogSooooo000Fooooo`05o`000?ooooooooooooooool0o`00oooooomhooooo`00
5_ooool3o`000003ooooool0o`3ooooo0?ooooooMoooool001Kooooo00Go0000oooooooooooooooo
o`3o003oooooogSooooo000Fooooo`04o`000?ooooooooooo`3o0?ooooooNOooool001Kooooo00Co
0000oooooooooooo0?l0oooooomiooooo`005_ooool00ol0003oooooooooo`3oooooog[ooooo000F
ooooo`03o`000?oooooooooo0?ooooooN_ooool001Kooooo00?o0000ooooooooool0oooooomjoooo
o`005_ooool00ol0003oooooooooo`3oooooog[ooooo000Fooooo`03o`000?oooooooooo0?oooooo
N_ooool001Kooooo00?o0000ooooooooool0oooooomjooooo`005_ooool00ol0003oooooooooo`3o
ooooog[ooooo000Fooooo`03o`000?oooooooooo0?ooooooN_ooool001Kooooo0ol0003oooooog[o
oooo000Fooooo`03o`000?oooooooooo0?ooooooN_ooool001Kooooo00?o0000ooooooooool0oooo
oomjooooo`005_ooool00ol0003oooooooooo`3oooooog[ooooo000Fooooo`03o`000?oooooooooo
0?ooooooN_ooool001Kooooo00?o0000ooooooooool0oooooomjooooo`005_ooool00ol0003ooooo
ooooo`3oooooog[ooooo000Fooooo`03o`000?oooooooooo0?ooooooN_ooool001Kooooo00?o0000
ooooooooool0oooooomjooooo`005_ooool01?l0003ooooooooooooo003oooooogWooooo000Foooo
o`04o`000?ooooooooooool00?ooooooNOooool001Kooooo0ol000000ooo003oooooooooo`3ooooo
ogOooooo000Fooooo`05o`000?oooooooooooooooooo0000oooooomhooooo`005_ooool01Ol0003o
ooooooooooooooooo`000?ooooooN?ooool001Kooooo00Go0000ooooooooooooooooool0003ooooo
ogSooooo000Fooooo`05o`000?oooooooooooooooooo0000oooooomhooooo`005_ooool01Ol0003o
ooooooooooooooooo`000?ooooooN?ooool001Kooooo00?o0000ooooooooool00_ooool00ooo003o
ooooooooo`3oooooogGooooo000Fooooo`03o`000?oooooooooo00;ooooo00?oo`00ooooooooool0
oooooomeooooo`005_ooool00ol0003oooooooooo`02ooooo`03ool00?oooooooooo0?ooooooMOoo
ool000Oooooo0ol00004ooooo`;o00001_ooool00ol0003oooooooooo`02ooooo`03ool00?oooooo
oooo0?ooooooMOooool000Kooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oooooo
ooooo`0000Gooooo00?o0000ooooooooool00oooool00ooo003oooooooooo`3oooooogCooooo0006
ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00?ooooo00?oo`00ooooooooool0oooooomdooooo`001_ooool4o`0000?ooooo00Co
0000oooooooooooo00001Oooool4o`0000;ooooo00?oo`00ooooooooool0oooooomdooooo`001_oo
ool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0
0oooool00ooo003oooooooooo`3oooooogCooooo0007ooooo`03o`000?oooooooooo00?ooooo00Co
0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03ool00?oooooooooo0?oooooo
M?ooool000Sooooo0_l00004ooooo`;o00001_ooool00ol0003oooooooooo`04ooooo`03ool00?oo
oooooooo0?ooooooLoooool001Kooooo00?o0000ooooooooool01?ooool00ooo003oooooooooo`3o
ooooog?ooooo000Fooooo`03o`000?oooooooooo00Cooooo00?oo`00ooooooooool0oooooomcoooo
o`005_ooool00ol0003oooooooooo`05ooooo`03ool00?oooooooooo0?ooooooL_ooool001Kooooo
00?o0000ooooooooool01Oooool00ooo003oooooooooo`3oooooog;ooooo000Fooooo`03o`000?oo
oooooooo00Gooooo00?oo`00ooooooooool0oooooombooooo`005_ooool00ol0003oooooooooo`06
ooooo`03ool00?oooooooooo0?ooooooLOooool001Kooooo00?o0000ooooooooool01_ooool00ooo
003oooooooooo`3oooooog7ooooo000Fooooo`?o00001_ooool00ooo003oooooooooo`3oooooog7o
oooo000Fooooo`03o`000?oooooooooo00Oooooo00?oo`00ooooooooool0oooooom`ooooo`005_oo
ool00ol0003oooooooooo`07ooooo`03ool00?oooooooooo0?ooooooL?ooool001Kooooo00?o0000
ooooooooool01oooool00ooo003oooooooooo`3oooooog3ooooo000Fooooo`03o`000?oooooooooo
00Sooooo00?oo`00ooooooooool0oooooom_ooooo`005_ooool00ol0003oooooooooo`08ooooo`03
ool00?oooooooooo0?ooooooKoooool001Kooooo00?o0000ooooooooool02Oooool00ooo003ooooo
ooooo`3oooooofkooooo000Fooooo`03o`000?oooooooooo00Wooooo00?oo`00ooooooooool0oooo
oom^ooooo`005_ooool00ol0003oooooooooo`09ooooo`03ool00?oooooooooo0?ooooooK_ooool0
01Kooooo00?o0000ooooooooool02_ooool2ool00?ooooooK_ooool001Kooooo00?o0000oooooooo
ool03?ooool00ooo003oooooooooo`3oooooof_ooooo000Fooooo`?o00003Oooool4ool00?oooooo
JOooool001Kooooo00?o0000ooooooooool04Oooool00ooo003oooooooooo`3oooooofKooooo000F
ooooo`03o`000?oooooooooo01;ooooo00?oo`00ooooooooool0oooooomUooooo`005_ooool00ol0
003oooooooooo`0Cooooo`Coo`00oooooomSooooo`005_ooool00ol0003oooooooooo`0Fooooo`03
ool00?oooooooooo0?ooooooHOooool001Kooooo00?o0000ooooooooool05oooool00ooo003ooooo
ooooo`3oooooof3ooooo000Fooooo`03o`000?oooooooooo01Oooooo00?oo`00ooooooooool0oooo
oomPooooo`005_ooool00ol0003oooooooooo`0Gooooo`03ool00?oooooooooo0?ooooooH?ooool0
01Kooooo00?o0000ooooooooool06?ooool00ooo003oooooooooo`3ooooooeoooooo000Fooooo`03
o`000?oooooooooo01Sooooo00?oo`00ooooooooool0oooooomOooooo`005_ooool00ol0003ooooo
ooooo`0Hooooo`03ool00?oooooooooo0?ooooooGoooool001Kooooo00?o0000ooooooooool06Ooo
ool00ooo003oooooooooo`3ooooooekooooo000Fooooo`?o00006Oooool00ooo003oooooooooo`3o
oooooekooooo000Fooooo`03o`000?oooooooooo01Wooooo00?oo`00ooooooooool0oooooomNoooo
o`005_ooool00ol0003oooooooooo`0Jooooo`03ool00?oooooooooo0?ooooooGOooool001Kooooo
00?o0000ooooooooool06oooool00ooo003oooooooooo`3ooooooecooooo000Fooooo`03o`000?oo
oooooooo01cooooo00?oo`00ooooooooool0oooooomKooooo`005_ooool00ol0003oooooooooo`0L
ooooo`03ool00?oooooooooo0?ooooooFoooool001Kooooo00?o0000ooooooooool07Oooool00ooo
003oooooooooo`3ooooooe[ooooo000Fooooo`03o`000?oooooooooo01kooooo00?oo`00oooooooo
ool0oooooomIooooo`005_ooool00ol0003oooooooooo`0Nooooo`03ool00?oooooooooo0?oooooo
FOooool000Oooooo0ol00004ooooo`;o00001_ooool00ol0003oooooooooo`0Oooooo`Goo`00oooo
oomFooooo`001_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo0000
1Oooool00ol0003oooooooooo`0Tooooo`03ool00?oooooooooo0?ooooooDoooool000Kooooo00Go
0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000Gooooo1?l0000Tooooo`Wo
o`00oooooom<ooooo`001oooool3o`0000?ooooo00Co0000oooooooooooo00001Oooool00ol0003o
ooooooooo`0^ooooo`03ool00?oooooooooo0?ooooooBOooool000Kooooo00Go0000oooooooooooo
ooooo`000002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0;oooool@ool0
0?oooooo>oooool000Kooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oooooooooo
o`0000Gooooo00?o0000ooooooooool0?oooool00ooo003oooooooooo`3oooooocSooooo0007oooo
o`?o00001?ooool2o`0000Kooooo00?o0000ooooooooool0@?ooool00ooo003oooooooooo`3ooooo
ocOooooo000Fooooo`03o`000?oooooooooo047ooooo1Ooo0003ooooo`?oo`00ooooool^ooooo`00
5_ooool00ol0003oooooooooo`16ooooo`03ool00?ooooooo`0000?ooooo00?oo`00ooooooooool0
ooooool[ooooo`005_ooool00ol0003oooooooooo`17ooooo`03ool00?oooooooooo00?ooooo3ooo
003ooooooakooooo000Fooooo`03o`000?oooooooooo05cooooo00?oo`00ooooooooool0oooooolK
ooooo`005_ooool00ol0003oooooooooo`1Mooooo`03ool00?oooooooooo0?oooooo6_ooool001Ko
oooo00?o0000ooooooooool0G_ooool00ooo003oooooooooo`3ooooooaWooooo000Fooooo`03o`00
0?oooooooooo05oooooo00?oo`00ooooooooool0oooooolHooooo`005_ooool3o`00063ooooo0_oo
0006ooooo`Goo`00ooooool=ooooo`005_ooool00ol0003oooooooooo`1Rooooo`03ool00?oooooo
oooo00;ooooo00?oo`00ooooooooool00_ooool00ooo003oooooooooo`3oooooo`_ooooo000Foooo
o`03o`000?oooooooooo06?ooooo1?oo0006ooooo`04ool00?ooooooooooooooo`Woo`00ooooool0
01Kooooo00?o0000ooooooooool0K_ooool00ooo003oooooool00009ooooo`03ool00?oooooooooo
0?cooooo000Fooooo`03o`000?oooooooooo06oooooo00?oo`00ooooooooool02Oooool00ooo003o
ooooooooo`3kooooo`005_ooool00ol0003oooooooooo`1kooooo`03ool00?oooooooooo0?_ooooo
000Fooooo`03o`000?oooooooooo07cooooo00?oo`00ooooooooool0n_ooool001Kooooo00?o0000
ooooooooool0OOooool00ooo003oooooooooo`3iooooo`005_ooool00ol0003oooooooooo`1noooo
o`03ool00?oooooooooo0?Sooooo000Fooooo`03o`000?oooooooooo07oooooo00?oo`00oooooooo
ool0moooool001Kooooo00?o0000ooooooooool0Ooooool00ooo003oooooooooo`3gooooo`005_oo
ool3o`00083ooooo00?oo`00ooooooooool0m_ooool001Kooooo00?o0000ooooooooool0POooool0
0ooo003oooooooooo`3eooooo`005_ooool00ol0003oooooooooo`22ooooo`03ool00?oooooooooo
0?Cooooo000Fooooo`03o`000?oooooooooo08?ooooo00?oo`00ooooooooool0loooool001Kooooo
00?o0000ooooooooool0Q?ooool00ooo003oooooooooo`3booooo`005_ooool00ol0003ooooooooo
o`25ooooo`Goo`00koooool001Kooooo00?o0000ooooooooool0R_ooool00ooo003oooooooooo`3/
ooooo`005_ooool00ol0003oooooooooo`2;ooooo`;oo`00k?ooool001Kooooo00?o0000oooooooo
ool0SOooool00ooo003oooooooooo`3Yooooo`005_ooool00ol0003oooooooooo`2>ooooo`;oo`00
jOooool001Kooooo00?o0000ooooooooool0T?ooool00ooo003oooooooooo`3Vooooo`005_ooool3
o`00097ooooo00?oo`00ooooooooool0iOooool001Kooooo00?o0000ooooooooool0TOooool00ooo
003oooooooooo`3Uooooo`005_ooool00ol0003oooooooooo`2Booooo`03ool00?oooooooooo0>Co
oooo000Fooooo`03o`000?oooooooooo09;ooooo00?oo`00ooooooooool0i?ooool001Kooooo00?o
0000ooooooooool0Toooool00ooo003oooooooooo`3Sooooo`005_ooool00ol0003oooooooooo`2C
ooooo`03ool00?oooooooooo0>?ooooo000Fooooo`03o`000?oooooooooo09?ooooo00?oo`00oooo
ooooool0hoooool001Kooooo00?o0000ooooooooool0U?ooool00ooo003oooooooooo`3Rooooo`00
5_ooool00ol0003oooooooooo`2Dooooo`03ool00?oooooooooo0>;ooooo00001?oooooo0000o`00
0?l00004ooooo`;o00001?ooool2o`0000Kooooo00?o0000ooooooooool0UOooool00ooo003ooooo
ooooo`3Qooooo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo
00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`2Fooooo`03ool00?oooooooooo0>3o
oooo0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00005ooooo`03o`000?oooooooooo09Kooooo00?oo`00ooooooooool0h?ooool000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?oooooooooo
o`0000Gooooo1?l0002FooooomSoo`002_ooool000;ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0oooo
oomjooooo`0000?oooooo`000?l000001?ooool01?l0003oooooooooool00002ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool0oooooomjooooo`000_ooool00ol0003ooooooooo
o`03ooooo`;o00001?ooool2o`0000Kooooo00?o0000ooooooooool0oooooomjooooo`005_ooool0
0ol0003oooooooooo`3oooooog[ooooo000Fooooo`03o`000?oooooooooo0?ooooooN_ooool00001
\
\>"],
  ImageRangeCache->{{{0, 401.125}, {247.562, 0}} -> {-7.448, -6.07037, \
0.325252, 0.438557}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
If you find this hard to read, we can adjust the background for \
better contrast.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(MultipleListPlot[strategy1, 
        strategy2, \[IndentingNewLine]SymbolShape \[Rule] 
          None, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]PlotStyle \[Rule] {Red, 
            Green}, \[IndentingNewLine]Background \[Rule] 
          RGBColor[ .9,  .9,  .9]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Background color
.9 .9 .9 r
MFill
% Scaling calculations
0.0238095 0.00793651 0.0147151 0.00588604 [
[.18254 .00222 -6 -9 ]
[.18254 .00222 6 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.65873 .00222 -6 -9 ]
[.65873 .00222 6 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .13244 -12 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -12 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -12 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -12 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -18 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 0 r
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(20)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(40)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(80)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(100)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(120)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.06349 .01472 m
.06349 .01847 L
s
.10317 .01472 m
.10317 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.22222 .01472 m
.22222 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.30159 .01472 m
.30159 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.42063 .01472 m
.42063 .01847 L
s
.46032 .01472 m
.46032 .01847 L
s
.53968 .01472 m
.53968 .01847 L
s
.57937 .01472 m
.57937 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.69841 .01472 m
.69841 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.77778 .01472 m
.77778 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.89683 .01472 m
.89683 .01847 L
s
.93651 .01472 m
.93651 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(20)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(40)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(60)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(80)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(100)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.001 w
.03175 .33256 m
.03968 .37965 L
.04762 .40319 L
.05556 .42085 L
.06349 .42674 L
.07143 .42674 L
.07937 .43262 L
.0873 .43262 L
.09524 .45617 L
.10317 .46794 L
.11111 .47971 L
.11905 .47971 L
.12698 .4856 L
.13492 .4856 L
.14286 .4856 L
.15079 .49148 L
.15873 .49148 L
.16667 .49148 L
.1746 .49148 L
.18254 .49148 L
.19048 .49148 L
.19841 .49737 L
.20635 .49737 L
.21429 .50326 L
.22222 .49737 L
.23016 .50326 L
.2381 .50326 L
.24603 .50326 L
.25397 .50326 L
.2619 .50326 L
.26984 .50914 L
.27778 .51503 L
.28571 .52091 L
.29365 .52091 L
.30159 .51503 L
.30952 .51503 L
.31746 .5268 L
.3254 .52091 L
.33333 .52091 L
.34127 .52091 L
.34921 .5268 L
.35714 .53857 L
.36508 .55034 L
.37302 .55623 L
.38095 .55623 L
.38889 .56212 L
.39683 .568 L
.40476 .57389 L
.4127 .59155 L
.42063 .60332 L
Mistroke
.42857 .60332 L
.43651 .60332 L
.44444 .60332 L
.45238 .60332 L
.46032 .60332 L
.46825 .60332 L
.47619 .60332 L
.48413 .60332 L
.49206 .60332 L
.5 .60332 L
.50794 .60332 L
.51587 .60332 L
.52381 .60332 L
.53175 .60332 L
.53968 .60332 L
.54762 .60332 L
.55556 .60332 L
.56349 .60332 L
.57143 .60332 L
.57937 .60332 L
.5873 .60332 L
.59524 .60332 L
.60317 .60332 L
.61111 .60332 L
.61905 .60332 L
.62698 .60332 L
.63492 .60332 L
.64286 .60332 L
.65079 .60332 L
.65873 .60332 L
.66667 .60332 L
.6746 .60332 L
.68254 .60332 L
.69048 .60332 L
.69841 .60332 L
.70635 .60332 L
.71429 .60332 L
.72222 .60332 L
.73016 .60332 L
.7381 .60332 L
.74603 .60332 L
.75397 .60332 L
.7619 .60332 L
.76984 .60332 L
.77778 .60332 L
.78571 .60332 L
.79365 .60332 L
.80159 .60332 L
.80952 .60332 L
.81746 .60332 L
Mistroke
.8254 .60332 L
.83333 .60332 L
.84127 .60332 L
.84921 .60332 L
.85714 .60332 L
.86508 .60332 L
.87302 .60332 L
.88095 .60332 L
.88889 .60332 L
.89683 .60332 L
.90476 .60332 L
.9127 .60332 L
.92063 .60332 L
.92857 .60332 L
.93651 .60332 L
.94444 .60332 L
.95238 .60332 L
.96032 .60332 L
.96825 .60332 L
.97619 .60332 L
Mfstroke
0 1 0 r
.03175 .28547 m
.03968 .23838 L
.04762 .21484 L
.05556 .19718 L
.06349 .1913 L
.07143 .1913 L
.07937 .18541 L
.0873 .18541 L
.09524 .16187 L
.10317 .15009 L
.11111 .13832 L
.11905 .13832 L
.12698 .13244 L
.13492 .13244 L
.14286 .13244 L
.15079 .12655 L
.15873 .12655 L
.16667 .12655 L
.1746 .12655 L
.18254 .12655 L
.19048 .12655 L
.19841 .12066 L
.20635 .12066 L
.21429 .11478 L
.22222 .12066 L
.23016 .11478 L
.2381 .11478 L
.24603 .11478 L
.25397 .11478 L
.2619 .11478 L
.26984 .10889 L
.27778 .10301 L
.28571 .09712 L
.29365 .09712 L
.30159 .10301 L
.30952 .10301 L
.31746 .09123 L
.3254 .09712 L
.33333 .09712 L
.34127 .09712 L
.34921 .09123 L
.35714 .07946 L
.36508 .06769 L
.37302 .0618 L
.38095 .0618 L
.38889 .05592 L
.39683 .05003 L
.40476 .04415 L
.4127 .02649 L
.42063 .01472 L
Mistroke
.42857 .01472 L
.43651 .01472 L
.44444 .01472 L
.45238 .01472 L
.46032 .01472 L
.46825 .01472 L
.47619 .01472 L
.48413 .01472 L
.49206 .01472 L
.5 .01472 L
.50794 .01472 L
.51587 .01472 L
.52381 .01472 L
.53175 .01472 L
.53968 .01472 L
.54762 .01472 L
.55556 .01472 L
.56349 .01472 L
.57143 .01472 L
.57937 .01472 L
.5873 .01472 L
.59524 .01472 L
.60317 .01472 L
.61111 .01472 L
.61905 .01472 L
.62698 .01472 L
.63492 .01472 L
.64286 .01472 L
.65079 .01472 L
.65873 .01472 L
.66667 .01472 L
.6746 .01472 L
.68254 .01472 L
.69048 .01472 L
.69841 .01472 L
.70635 .01472 L
.71429 .01472 L
.72222 .01472 L
.73016 .01472 L
.7381 .01472 L
.74603 .01472 L
.75397 .01472 L
.7619 .01472 L
.76984 .01472 L
.77778 .01472 L
.78571 .01472 L
.79365 .01472 L
.80159 .01472 L
.80952 .01472 L
.81746 .01472 L
Mistroke
.8254 .01472 L
.83333 .01472 L
.84127 .01472 L
.84921 .01472 L
.85714 .01472 L
.86508 .01472 L
.87302 .01472 L
.88095 .01472 L
.88889 .01472 L
.89683 .01472 L
.90476 .01472 L
.9127 .01472 L
.92063 .01472 L
.92857 .01472 L
.93651 .01472 L
.94444 .01472 L
.95238 .01472 L
.96032 .01472 L
.96825 .01472 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[18]:=",
  ImageSize->{402.562, 248.812},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006B0000n2000`400?l00000o`00003ooooVi^JC
onKViP00C_oVi^H5o`0000?oi^KV0_l0000fonKViP?o00000ooVi^H2o`0003Coi^KV0ol00004onKV
iP;o0000=OoVi^H3o`0000Coi^KV0_l0000aonKViP?o00001?oVi^H2o`0000Coi^KV0_l0000_onKV
iP?o00000_oVi^H5o`0000?oi^KV0_l00003onKViP00CooVi^H01?l0003oi^KVonKVi_l00002onKV
iP04o`000?oVi^Koi^KVo`0003Koi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKViP03o`000?oV
i^Koi^KV033oi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`0003?o
i^KV00Go0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`00037oi^KV00?o0000
onKVi_oVi^H00_oVi^H01?l0003oi^KVonKVi_l00002onKViP04o`000?oVi^Koi^KVo`0002ooi^KV
00?o0000onKVi_oVi^H00_oVi^H01?l0003oi^KVonKVi_l00002onKViP04o`000?oVi^Koi^KVo`00
00;oi^KV001@onKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKVi_oVi^Ko0000<ooVi^H5o`00
00;oi^KV00Co0000onKVi_oVi^Ko0000<_oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co
0000onKVi_oVi^Ko0000<ooVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oV
i^Ko0000<OoVi^H00ol0003oi^KVonKViP02onKViP04o`000?oVi^Koi^KVo`0000;oi^KV00Co0000
onKVi_oVi^Ko0000;ooVi^H00ol0003oi^KVonKViP03onKViP03o`000?oVi^Koi^KV00;oi^KV00Co
0000onKVi_oVi^Ko00000_oVi^H0057oi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKViP03o`00
0?oVi^Koi^KV037oi^KV00Co0000onKVi_oVi^Ko00000ooVi^H01?l0003oi^KVonKVi_l0000bonKV
iPCo00000ooVi^H01?l0003oi^KVonKVi_l0000donKViP?o00000ooVi^H01?l0003oi^KVonKVi_l0
000aonKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKVi_oVi^Ko00000_oVi^H01?l0003oi^KV
onKVi_l0000_onKViP03o`000?oVi^Koi^KV00Coi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKV
iP7o00000OoVi^H1onKViP00C_oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKV
i_oVi^Ko0000=?oVi^H00ol0003oi^KVo`000003onKViP04o`000?oVi^Koi^KVo`0003;oi^KV00?o
0000onKVi_oVi^H01?oVi^H01?l0003oi^KVonKVi_l0000conKViP05o`000?oVi^Koi^KVonKVi_l0
00000_oVi^H01?l0003oi^KVonKVi_l0000aonKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKV
i_oVi^Ko00000_oVi^H01?l0003oi^KVonKVi_l0000_onKViP05o`000?oVi^Koi^KVonKVi_l00000
0ooVi^H01?l0003oi^KVonKVi_l00002onKViP7o00000OoVi^H1onKViP00C_oVi^H01Ol0003oi^KV
onKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko0000=OoVi^H2o`0000?oi^KV00Co0000onKV
i_oVi^Ko0000<ooVi^H00ol0003oi^KVonKViP03onKViP04o`000?oVi^Koi^KVo`0003?oi^KV00Go
0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`00033oi^KV0_l00004onKViP04
o`000?oVi^Koi^KVo`0000;oi^KV00Co0000onKVi_oVi^Ko0000;_oVi^H2o`0000?oi^KV00Go0000
onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`0000;oi^KV001?onKViP?o00001?oV
i^H2o`0003Ooi^KV00?o0000onKVi_oVi^H00_oVi^H2o`0003Goi^KV0_l00004onKViP;o0000=OoV
i^H3o`0000Coi^KV0_l0000bonKViP03o`000?oVi^Koi^KV00?oi^KV0_l00004onKViP;o0000<?oV
i^H00ol0003oi^KVonKViP02onKViP?o00001?oVi^H2o`0000?oi^KV000FonKViP03o`000?oVi^Ko
i^KV0?ooi^KVN_oVi^H001Koi^KV00?o0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H00ol0003o
i^KVonKViP3oonKViW[oi^KV000FonKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Koi^KV00?o
0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H00ol0003oi^KVonKViP3oonKViW[oi^KV000=onKV
iZ?o0000fOl0o`09o`0000005_oVi^H00ol0003oi^KVonKViP0=onKViP03o`000?oVi^Koi^KV00co
i^KV00?o0000onKVi_oVi^H03OoVi^H00ol0003oi^KVonKViP0<onKViP03o`000?oVi^Koi^KV00co
i^KV00?o0000onKVi_oVi^H03OoVi^H00ol0003oi^KVonKViP0<onKViP03o`000?oVi^Koi^KV00go
i^KV00?o0000onKVi_oVi^H03?oVi^H00ol0003oi^KVonKViP0<onKViP03o`3o0?oVi^Koi^KV00go
i^KV00?o0000onKVi_oVi^H03?oVi^H00ol0003oi^KVonKViP0<onKViP03o`000?oVi^Koi^KV00go
i^KV00?o0000onKVi_oVi^H03?oVi^H00ol0003oi^KVonKViP0=onKViP03o`000?oVi^Koi^KV00co
i^KV00?o0000onKVi_oVi^H03?oVi^H00ol0003oi^KVonKViP0=onKViP03o`000?oVi^Koi^KV00co
i^KV00?o0000onKVi_oVi^H03OoVi^H00ol0003oi^KVonKViP0<onKViP03o`000?oVi^Koi^KV00co
i^KV00?o0000onKVi_oVi^H03OoVi^H00ol0003oi^KVonKViP07onKViP005_oVi^H00ol0003oi^KV
onKViP0konKViP03o`000?oVi^Koi^KV03_oi^KV00?o0000onKVi_oVi^H06_oVi^H00ol0o`3oi^KV
onKViP0MonKViP03o`000?oVi^Koi^KV03_oi^KV00?o0000onKVi_oVi^H0>_oVi^H00ol0003oi^KV
onKViP0konKViP03o`000?oVi^Koi^KV00Ooi^KV000FonKViP03o`000?oVi^Koi^KV09Goi^KV00?o
0?l0onKVi_oVi^H0hOoVi^H001Koi^KV00?o0000onKVi_oVi^H0U?oVi^H00ol0o`3oi^KVonKViP3R
onKViP005_oVi^H00ol0003oi^KVonKViP2DonKViP03o`3o0?oVi^Koi^KV0>;oi^KV000FonKViP03
o`000?oVi^Koi^KV09?oi^KV00?o0?l0onKVi_oVi^H0hooVi^H001Koi^KV00?o0000onKVi_oVi^H0
TooVi^H00ol0o`3oi^KVonKViP3SonKViP005_oVi^H00ol0003oi^KVonKViP2BonKViP03o`3o0?oV
i^Koi^KV0>Coi^KV000FonKViP03o`000?oVi^Koi^KV09;oi^KV00?o0?l0onKVi_oVi^H0i?oVi^H0
01Koi^KV00?o0000onKVi_oVi^H0T_oVi^H00ol0o`3oi^KVonKViP3TonKViP005_oVi^H3o`00093o
i^KV0_l0o`3WonKViP005_oVi^H00ol0003oi^KVonKViP2?onKViP03o`3o0?oVi^Koi^KV0>Ooi^KV
000FonKViP03o`000?oVi^Koi^KV08koi^KV00?o0?l0onKVi_oVi^H0j?oVi^H001Koi^KV00?o0000
onKVi_oVi^H0SOoVi^H00ol0o`3oi^KVonKViP3YonKViP005_oVi^H00ol0003oi^KVonKViP2<onKV
iP03o`3o0?oVi^Koi^KV0>[oi^KV000FonKViP03o`000?oVi^Koi^KV08[oi^KV0_l0o`3]onKViP00
5_oVi^H00ol0003oi^KVonKViP29onKViP03o`3o0?oVi^Koi^KV0>goi^KV000FonKViP03o`000?oV
i^Koi^KV08Coi^KV1Ol0o`3`onKViP005_oVi^H00ol0003oi^KVonKViP23onKViP03o`3o0?oVi^Ko
i^KV0??oi^KV000FonKViP03o`000?oVi^Koi^KV08;oi^KV00?o0?l0onKVi_oVi^H0m?oVi^H001Ko
i^KV00?o0000onKVi_oVi^H0P_oVi^H00ol0o`3oi^KVonKViP3donKViP005_oVi^H3o`00087oi^KV
00?o0?l0onKVi_oVi^H0mOoVi^H001Koi^KV00?o0000onKVi_oVi^H0P?oVi^H00ol0o`3oi^KVonKV
iP3fonKViP005_oVi^H00ol0003oi^KVonKViP20onKViP03o`3o0?oVi^Koi^KV0?Koi^KV000FonKV
iP03o`000?oVi^Koi^KV07ooi^KV00?o0?l0onKVi_oVi^H0mooVi^H001Koi^KV00?o0000onKVi_oV
i^H0OooVi^H00ol0o`3oi^KVonKViP3gonKViP005_oVi^H00ol0003oi^KVonKViP1nonKViP03o`3o
0?oVi^Koi^KV0?Soi^KV000FonKViP03o`000?oVi^Koi^KV07goi^KV00?o0?l0onKVi_oVi^H0nOoV
i^H001Koi^KV00?o0000onKVi_oVi^H0KooVi^H2o`3o00[oi^KV0_l0o`3lonKViP005_oVi^H00ol0
003oi^KVonKViP1_onKViP03o`3o0?oVi^Ko0?l000Soi^KV00?o0?l0onKVi_oVi^H0o?oVi^H001Ko
i^KV00?o0000onKVi_oVi^H0HooVi^H4o`3o00Ooi^KV00Co0?l0onKVi_oVi^Koi^KV2?l0o`3oonKV
iP005_oVi^H00ol0003oi^KVonKViP1RonKViP03o`3o0?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oV
i^H00ooVi^H00ol0o`3oi^KVonKViP3oonKViP[oi^KV000FonKViP03o`000?oVi^Koi^KV067oi^KV
00?o0?l0onKVi_oVi^H01?oVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV0?ooi^KV
2_oVi^H001Koi^KV0ol0001OonKViP;o0?l02?oVi^H4o`3o0?ooi^KV3OoVi^H001Koi^KV00?o0000
onKVi_oVi^H0G_oVi^H00ol0o`3oi^KVonKViP3oonKViQWoi^KV000FonKViP03o`000?oVi^Koi^KV
05coi^KV0_l0o`3oonKViQcoi^KV000FonKViP03o`000?oVi^Koi^KV05_oi^KV00?o0?l0onKVi_oV
i^H0oooVi^HLonKViP005_oVi^H00ol0003oi^KVonKViP17onKViP03o`3o0?oVi^Koi^KV00?oi^KV
3_l0o`3oonKViQooi^KV000FonKViP03o`000?oVi^Koi^KV04Koi^KV00?o0?l0onKVi_l0o`000ooV
i^H00ol0o`3oi^KVonKViP3oonKViR_oi^KV000FonKViP03o`000?oVi^Koi^KV04Goi^KV00Oo0?l0
onKVi_oVi^Koi^KVo`3o0?oVi^Ko0?l00?ooi^KV;_oVi^H001Koi^KV00?o0000onKVi_oVi^H0@?oV
i^H5o`3o00Goi^KV00?o0?l0onKVi_oVi^H0oooVi^H]onKViP005_oVi^H00ol0003oi^KVonKViP0o
onKViP03o`3o0?oVi^Koi^KV0?ooi^KV>?oVi^H000Koi^KV1Ol00003onKViP;o00001_oVi^H00ol0
003oi^KVonKViP0^onKViQ7o0?l0oooVi^HkonKViP001ooVi^H01?l0003oi^KVonKVi_l00002onKV
iP04o`000?oVi^Koi^KVo`0000Goi^KV00?o0000onKVi_oVi^H0;OoVi^H00ol0o`3oi^KVonKViP3o
onKViT[oi^KV0008onKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKVi_oVi^Ko00001OoVi^H4
o`0002?oi^KV2Ol0o`3oonKViTgoi^KV0009onKViP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H0
0ol0003oi^KVonKViP03onKViP03o`000?oVi^Koi^KV02?oi^KV00?o0?l0onKVi_oVi^H0oooVi^ID
onKViP001_oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00001OoV
i^H00ol0003oi^KVonKViP0OonKViPCo0?l0oooVi^IGonKViP001_oVi^H01Ol0003oi^KVonKVi_oV
i^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00001OoVi^H00ol0003oi^KVonKViP0OonKViP03o`3o
0?oVi^Koi^KV0?ooi^KVF?oVi^H000Ooi^KV0ol00004onKViP;o00001_oVi^H00ol0003oi^KVonKV
iP0NonKViP03o`3o0?oVi^Koi^KV0?ooi^KVFOoVi^H001Koi^KV00?o0000onKVi_oVi^H07OoVi^H0
0ol0o`3oi^KVonKViP3oonKViU[oi^KV000FonKViP03o`000?oVi^Koi^KV01goi^KV00?o0?l0onKV
i_oVi^H0oooVi^IJonKViP005_oVi^H00ol0003oi^KVonKViP0LonKViP03o`3o0?oVi^Koi^KV0?oo
i^KVFooVi^H001Koi^KV00?o0000onKVi_oVi^H07?oVi^H00ol0o`3oi^KVonKViP3oonKViU_oi^KV
000FonKViP03o`000?oVi^Koi^KV01_oi^KV00?o0?l0onKVi_oVi^H0oooVi^ILonKViP005_oVi^H0
0ol0003oi^KVonKViP0JonKViP03o`3o0?oVi^Koi^KV0?ooi^KVGOoVi^H001Koi^KV00?o0000onKV
i_oVi^H06_oVi^H00ol0o`3oi^KVonKViP3oonKViUgoi^KV000FonKViP?o00006OoVi^H00ol0o`3o
i^KVonKViP3oonKViUkoi^KV000FonKViP03o`000?oVi^Koi^KV01Woi^KV00?o0?l0onKVi_oVi^H0
oooVi^INonKViP005_oVi^H00ol0003oi^KVonKViP0HonKViP03o`3o0?oVi^Koi^KV0?ooi^KVGooV
i^H001Koi^KV00?o0000onKVi_oVi^H06?oVi^H00ol0o`3oi^KVonKViP3oonKViUooi^KV000FonKV
iP03o`000?oVi^Koi^KV01Soi^KV00?o0?l0onKVi_oVi^H0oooVi^IOonKViP005_oVi^H00ol0003o
i^KVonKViP0GonKViP03o`3o0?oVi^Koi^KV0?ooi^KVH?oVi^H001Koi^KV00?o0000onKVi_oVi^H0
5ooVi^H00ol0o`3oi^KVonKViP3oonKViV3oi^KV000FonKViP03o`000?oVi^Koi^KV01Ooi^KV00?o
0?l0onKVi_oVi^H0oooVi^IPonKViP005_oVi^H00ol0003oi^KVonKViP0FonKViP03o`3o0?oVi^Ko
i^KV0?ooi^KVHOoVi^H001Koi^KV00?o0000onKVi_oVi^H04_oVi^H5o`3o0?ooi^KVHooVi^H001Ko
i^KV00?o0000onKVi_oVi^H04OoVi^H00ol0o`3oi^KVonKViP3oonKViVKoi^KV000FonKViP?o0000
3?oVi^H5o`3o0?ooi^KVJOoVi^H001Koi^KV00?o0000onKVi_oVi^H02ooVi^H00ol0o`3oi^KVonKV
iP3oonKViVcoi^KV000FonKViP03o`000?oVi^Koi^KV00[oi^KV00?o0?l0onKVi_oVi^H0oooVi^I]
onKViP005_oVi^H00ol0003oi^KVonKViP0:onKViP03o`3o0?oVi^Koi^KV0?ooi^KVKOoVi^H001Ko
i^KV00?o0000onKVi_oVi^H02OoVi^H00ol0o`3oi^KVonKViP3oonKViVkoi^KV000FonKViP03o`00
0?oVi^Koi^KV00Woi^KV00?o0?l0onKVi_oVi^H0oooVi^I^onKViP005_oVi^H00ol0003oi^KVonKV
iP08onKViP03o`3o0?oVi^Koi^KV0?ooi^KVKooVi^H001Koi^KV00?o0000onKVi_oVi^H02?oVi^H0
0ol0o`3oi^KVonKViP3oonKViVooi^KV000FonKViP03o`000?oVi^Koi^KV00Soi^KV00?o0?l0onKV
i_oVi^H0oooVi^I_onKViP005_oVi^H00ol0003oi^KVonKViP07onKViP03o`3o0?oVi^Koi^KV0?oo
i^KVL?oVi^H001Koi^KV00?o0000onKVi_oVi^H01ooVi^H00ol0o`3oi^KVonKViP3oonKViW3oi^KV
000FonKViP?o00001_oVi^H00ol0o`3oi^KVonKViP3oonKViW7oi^KV000FonKViP03o`000?oVi^Ko
i^KV00Koi^KV00?o0?l0onKVi_oVi^H0oooVi^IaonKViP005_oVi^H00ol0003oi^KVonKViP06onKV
iP03o`3o0?oVi^Koi^KV0?ooi^KVLOoVi^H001Koi^KV00?o0000onKVi_oVi^H01OoVi^H00ol0o`3o
i^KVonKViP3oonKViW;oi^KV000FonKViP03o`000?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H0
oooVi^IbonKViP005_oVi^H00ol0003oi^KVonKViP05onKViP03o`3o0?oVi^Koi^KV0?ooi^KVL_oV
i^H001Koi^KV00?o0000onKVi_oVi^H01?oVi^H00ol0o`3oi^KVonKViP3oonKViW?oi^KV000FonKV
iP03o`000?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oVi^H0oooVi^IconKViP005_oVi^H00ol0003o
i^KVonKViP04onKViP03o`3o0?oVi^Koi^KV0?ooi^KVLooVi^H000Soi^KV0ol00003onKViP;o0000
1_oVi^H00ol0003oi^KVonKViP04onKViP03o`3o0?oVi^Koi^KV0?ooi^KVLooVi^H000Woi^KV00Go
0000onKVi_oVi^Koi^KVo`000002onKViP03o`000?oVi^Koi^KV00?oi^KV00?o0000onKVi_oVi^H0
1?oVi^H00ol0o`3oi^KVonKViP3oonKViW?oi^KV0006onKViPGo00000_oVi^H01?l0003oi^KVonKV
i_l00005onKViP03o`000?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_oVi^H0oooVi^IdonKViP001_oV
i^H01?l0003oi^KVonKVi_l00003onKViP04o`000?oVi^Koi^KVo`0000Goi^KV1?l00002onKViP03
o`3o0?oVi^Koi^KV0?ooi^KVM?oVi^H000Ooi^KV00?o0000onKVi_l000000ooVi^H01?l0003oi^KV
onKVi_l00005onKViP03o`000?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_oVi^H0oooVi^IdonKViP00
2?oVi^H2o`0000?oi^KV00Co0000onKVi_oVi^Ko00001OoVi^H00ol0003oi^KVonKViP03onKViP03
o`3o0?oVi^Koi^KV0?ooi^KVM?oVi^H000Woi^KV00?o0000onKVi_oVi^H00_oVi^H2o`0000Koi^KV
00?o0000onKVi_oVi^H00ooVi^H00ol0o`3oi^KVonKViP3oonKViWCoi^KV000FonKViP03o`000?oV
i^Koi^KV00?oi^KV00?o0?l0onKVi_oVi^H0oooVi^IdonKViP005_oVi^H00ol0003oi^KVonKViP02
onKViP03o`3o0?oVi^Koi^KV0?ooi^KVMOoVi^H001Koi^KV00?o0000onKVi_oVi^H00_oVi^H00ol0
o`3oi^KVonKViP3oonKViWGoi^KV000FonKViP03o`000?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oV
i^H0oooVi^IeonKViP005_oVi^H00ol0003oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV0?ooi^KV
MOoVi^H001Koi^KV00?o0000onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKViP3oonKViWGoi^KV000F
onKViP03o`000?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oVi^H0oooVi^IeonKViP005_oVi^H3o`00
00;oi^KV00?o0?l0onKVi_oVi^H0oooVi^IeonKViP005_oVi^H01Ol0003oi^KVonKVi_oVi^Ko0?l0
0?ooi^KVN?oVi^H001Koi^KV00Go0000onKVi_oVi^Koi^KVo`3o003oonKViWSoi^KV000FonKViP05
o`000?oVi^Koi^KVonKVi_l0o`00oooVi^IhonKViP005_oVi^H00ol0003oi^KVonKViP3oonKViW[o
i^KV000FonKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Koi^KV00?o0000onKVi_oVi^H0oooV
i^IjonKViP005_oVi^H00ol0003oi^KVonKViP3oonKViW[oi^KV000FonKViP03o`000?oVi^Koi^KV
0?ooi^KVN_oVi^H001Koi^KV00?o0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H00ol0003oi^KV
onKViP3oonKViW[oi^KV000FonKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Koi^KV0ol0003o
onKViW[oi^KV000FonKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Koi^KV00?o0000onKVi_oV
i^H0oooVi^IjonKViP005_oVi^H00ol0003oi^KVonKViP3oonKViW[oi^KV000FonKViP03o`000?oV
i^Koi^KV0?ooi^KVN_oVi^H001Koi^KV00?o0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H00ol0
003oi^KVonKViP3oonKViW[oi^KV000FonKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Koi^KV
00?o0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H01Ol0003oi^KVonKVi_oVi^Koo`000?ooi^KV
N?oVi^H001Koi^KV00Go0000onKVi_oVi^Koi^KVool0003oonKViWSoi^KV000FonKViP?o000000?o
i^KVool00?oVi^H0oooVi^IgonKViP005_oVi^H01Ol0003oi^KVonKVi_oVi^Koo`000?ooi^KVN?oV
i^H001Koi^KV00?o0000onKVi_oVi^H00_oVi^H00ooo003oi^KVonKViP3oonKViWGoi^KV000FonKV
iP03o`000?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H0oooVi^IeonKViP005_oVi^H00ol0003o
i^KVonKViP02onKViP03ool00?oVi^Koi^KV0?ooi^KVMOoVi^H001Koi^KV00?o0000onKVi_oVi^H0
0_oVi^H00ooo003oi^KVonKViP3oonKViWGoi^KV000FonKViP03o`000?oVi^Koi^KV00;oi^KV00?o
o`00onKVi_oVi^H0oooVi^IeonKViP005_oVi^H00ol0003oi^KVonKViP02onKViP03ool00?oVi^Ko
i^KV0?ooi^KVMOoVi^H001Koi^KV00?o0000onKVi_oVi^H00ooVi^H00ooo003oi^KVonKViP3oonKV
iWCoi^KV0007onKViP?o00001?oVi^H2o`0000Koi^KV00?o0000onKVi_oVi^H00ooVi^H00ooo003o
i^KVonKViP3oonKViWCoi^KV0006onKViP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H01?l0003o
i^KVonKVi_l00005onKViP03o`000?oVi^Koi^KV00?oi^KV00?oo`00onKVi_oVi^H0oooVi^IdonKV
iP001_oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00001OoVi^H4
o`0000;oi^KV00?oo`00onKVi_oVi^H0oooVi^IdonKViP001_oVi^H4o`0000?oi^KV00Co0000onKV
i_oVi^Ko00001OoVi^H00ol0003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV0?ooi^KVM?oVi^H0
00Koi^KV00?o0000onKVi_oVi^H01?oVi^H01?l0003oi^KVonKVi_l00005onKViP03o`000?oVi^Ko
i^KV00?oi^KV00?oo`00onKVi_oVi^H0oooVi^IdonKViP001ooVi^H00ol0003oi^KVonKViP03onKV
iP04o`000?oVi^Koi^KVo`0000Goi^KV00?o0000onKVi_oVi^H01?oVi^H00ooo003oi^KVonKViP3o
onKViW?oi^KV0008onKViP;o00001?oVi^H2o`0000Koi^KV00?o0000onKVi_oVi^H01?oVi^H00ooo
003oi^KVonKViP3oonKViW?oi^KV000FonKViP03o`000?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oV
i^H0oooVi^IconKViP005_oVi^H00ol0003oi^KVonKViP04onKViP03ool00?oVi^Koi^KV0?ooi^KV
LooVi^H001Koi^KV00?o0000onKVi_oVi^H01OoVi^H00ooo003oi^KVonKViP3oonKViW;oi^KV000F
onKViP03o`000?oVi^Koi^KV00Goi^KV00?oo`00onKVi_oVi^H0oooVi^IbonKViP005_oVi^H00ol0
003oi^KVonKViP05onKViP03ool00?oVi^Koi^KV0?ooi^KVL_oVi^H001Koi^KV00?o0000onKVi_oV
i^H01_oVi^H00ooo003oi^KVonKViP3oonKViW7oi^KV000FonKViP03o`000?oVi^Koi^KV00Koi^KV
00?oo`00onKVi_oVi^H0oooVi^IaonKViP005_oVi^H3o`0000Koi^KV00?oo`00onKVi_oVi^H0oooV
i^IaonKViP005_oVi^H00ol0003oi^KVonKViP07onKViP03ool00?oVi^Koi^KV0?ooi^KVL?oVi^H0
01Koi^KV00?o0000onKVi_oVi^H01ooVi^H00ooo003oi^KVonKViP3oonKViW3oi^KV000FonKViP03
o`000?oVi^Koi^KV00Ooi^KV00?oo`00onKVi_oVi^H0oooVi^I`onKViP005_oVi^H00ol0003oi^KV
onKViP08onKViP03ool00?oVi^Koi^KV0?ooi^KVKooVi^H001Koi^KV00?o0000onKVi_oVi^H02?oV
i^H00ooo003oi^KVonKViP3oonKViVooi^KV000FonKViP03o`000?oVi^Koi^KV00Woi^KV00?oo`00
onKVi_oVi^H0oooVi^I^onKViP005_oVi^H00ol0003oi^KVonKViP09onKViP03ool00?oVi^Koi^KV
0?ooi^KVK_oVi^H001Koi^KV00?o0000onKVi_oVi^H02OoVi^H00ooo003oi^KVonKViP3oonKViVko
i^KV000FonKViP03o`000?oVi^Koi^KV00[oi^KV0_oo003oonKViVkoi^KV000FonKViP03o`000?oV
i^Koi^KV00coi^KV00?oo`00onKVi_oVi^H0oooVi^I[onKViP005_oVi^H3o`0000goi^KV1?oo003o
onKViVWoi^KV000FonKViP03o`000?oVi^Koi^KV017oi^KV00?oo`00onKVi_oVi^H0oooVi^IVonKV
iP005_oVi^H00ol0003oi^KVonKViP0BonKViP03ool00?oVi^Koi^KV0?ooi^KVIOoVi^H001Koi^KV
00?o0000onKVi_oVi^H04ooVi^H4ool00?ooi^KVHooVi^H001Koi^KV00?o0000onKVi_oVi^H05_oV
i^H00ooo003oi^KVonKViP3oonKViV7oi^KV000FonKViP03o`000?oVi^Koi^KV01Ooi^KV00?oo`00
onKVi_oVi^H0oooVi^IPonKViP005_oVi^H00ol0003oi^KVonKViP0GonKViP03ool00?oVi^Koi^KV
0?ooi^KVH?oVi^H001Koi^KV00?o0000onKVi_oVi^H05ooVi^H00ooo003oi^KVonKViP3oonKViV3o
i^KV000FonKViP03o`000?oVi^Koi^KV01Soi^KV00?oo`00onKVi_oVi^H0oooVi^IOonKViP005_oV
i^H00ol0003oi^KVonKViP0HonKViP03ool00?oVi^Koi^KV0?ooi^KVGooVi^H001Koi^KV00?o0000
onKVi_oVi^H06?oVi^H00ooo003oi^KVonKViP3oonKViUooi^KV000FonKViP03o`000?oVi^Koi^KV
01Woi^KV00?oo`00onKVi_oVi^H0oooVi^INonKViP005_oVi^H3o`0001Woi^KV00?oo`00onKVi_oV
i^H0oooVi^INonKViP005_oVi^H00ol0003oi^KVonKViP0IonKViP03ool00?oVi^Koi^KV0?ooi^KV
G_oVi^H001Koi^KV00?o0000onKVi_oVi^H06_oVi^H00ooo003oi^KVonKViP3oonKViUgoi^KV000F
onKViP03o`000?oVi^Koi^KV01_oi^KV00?oo`00onKVi_oVi^H0oooVi^ILonKViP005_oVi^H00ol0
003oi^KVonKViP0LonKViP03ool00?oVi^Koi^KV0?ooi^KVFooVi^H001Koi^KV00?o0000onKVi_oV
i^H07?oVi^H00ooo003oi^KVonKViP3oonKViU_oi^KV000FonKViP03o`000?oVi^Koi^KV01goi^KV
00?oo`00onKVi_oVi^H0oooVi^IJonKViP005_oVi^H00ol0003oi^KVonKViP0NonKViP03ool00?oV
i^Koi^KV0?ooi^KVFOoVi^H001Koi^KV00?o0000onKVi_oVi^H07_oVi^H00ooo003oi^KVonKViP3o
onKViUWoi^KV0007onKViP?o00001?oVi^H2o`0000Koi^KV00?o0000onKVi_oVi^H07ooVi^H5ool0
0?ooi^KVE_oVi^H000Koi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KV
o`0000Goi^KV00?o0000onKVi_oVi^H09?oVi^H00ooo003oi^KVonKViP3oonKViU?oi^KV0006onKV
iP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H01?l0003oi^KVonKVi_l00005onKViPCo00009?oV
i^H9ool00?ooi^KVC?oVi^H000Ooi^KV0ol00003onKViP04o`000?oVi^Koi^KVo`0000Goi^KV00?o
0000onKVi_oVi^H0;_oVi^H00ooo003oi^KVonKViP3oonKViTWoi^KV0006onKViP05o`000?oVi^Ko
i^KVonKVi_l000000_oVi^H01?l0003oi^KVonKVi_l00005onKViP03o`000?oVi^Koi^KV02ooi^KV
4?oo003oonKViS_oi^KV0006onKViP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H01?l0003oi^KV
onKVi_l00005onKViP03o`000?oVi^Koi^KV03ooi^KV00?oo`00onKVi_oVi^H0oooVi^HhonKViP00
1ooVi^H3o`0000Coi^KV0_l00006onKViP03o`000?oVi^Koi^KV043oi^KV00?oo`00onKVi_oVi^H0
oooVi^HgonKViP005_oVi^H00ol0003oi^KVonKViP11onKViPGoo`000ooVi^H3ool00?ooi^KV;_oV
i^H001Koi^KV00?o0000onKVi_oVi^H0A_oVi^H00ooo003oi^KVool00003onKViP03ool00?oVi^Ko
i^KV0?ooi^KV:ooVi^H001Koi^KV00?o0000onKVi_oVi^H0AooVi^H00ooo003oi^KVonKViP03onKV
iPooo`00oooVi^HNonKViP005_oVi^H00ol0003oi^KVonKViP1LonKViP03ool00?oVi^Koi^KV0?oo
i^KV6ooVi^H001Koi^KV00?o0000onKVi_oVi^H0GOoVi^H00ooo003oi^KVonKViP3oonKViQ[oi^KV
000FonKViP03o`000?oVi^Koi^KV05koi^KV00?oo`00onKVi_oVi^H0oooVi^HIonKViP005_oVi^H0
0ol0003oi^KVonKViP1OonKViP03ool00?oVi^Koi^KV0?ooi^KV6?oVi^H001Koi^KV0ol0001PonKV
iP;oo`001_oVi^H5ool00?ooi^KV3OoVi^H001Koi^KV00?o0000onKVi_oVi^H0H_oVi^H00ooo003o
i^KVonKViP02onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H0oooVi^H;onKViP00
5_oVi^H00ol0003oi^KVonKViP1SonKViPCoo`001_oVi^H01?oo003oi^KVonKVi_oVi^H:ool00?ko
i^KV000FonKViP03o`000?oVi^Koi^KV06koi^KV00?oo`00onKVi_oo00002_oVi^H00ooo003oi^KV
onKViP3konKViP005_oVi^H00ol0003oi^KVonKViP1_onKViP03ool00?oVi^Koi^KV00[oi^KV00?o
o`00onKVi_oVi^H0n_oVi^H001Koi^KV00?o0000onKVi_oVi^H0O?oVi^H00ooo003oi^KVonKViP3j
onKViP005_oVi^H00ol0003oi^KVonKViP1monKViP03ool00?oVi^Koi^KV0?Woi^KV000FonKViP03
o`000?oVi^Koi^KV07koi^KV00?oo`00onKVi_oVi^H0n?oVi^H001Koi^KV00?o0000onKVi_oVi^H0
O_oVi^H00ooo003oi^KVonKViP3honKViP005_oVi^H00ol0003oi^KVonKViP1oonKViP03ool00?oV
i^Koi^KV0?Ooi^KV000FonKViP03o`000?oVi^Koi^KV07ooi^KV00?oo`00onKVi_oVi^H0mooVi^H0
01Koi^KV0ol00020onKViP03ool00?oVi^Koi^KV0?Koi^KV000FonKViP03o`000?oVi^Koi^KV087o
i^KV00?oo`00onKVi_oVi^H0mOoVi^H001Koi^KV00?o0000onKVi_oVi^H0P_oVi^H00ooo003oi^KV
onKViP3donKViP005_oVi^H00ol0003oi^KVonKViP23onKViP03ool00?oVi^Koi^KV0??oi^KV000F
onKViP03o`000?oVi^Koi^KV08Coi^KV00?oo`00onKVi_oVi^H0l_oVi^H001Koi^KV00?o0000onKV
i_oVi^H0QOoVi^H5ool00>ooi^KV000FonKViP03o`000?oVi^Koi^KV08[oi^KV00?oo`00onKVi_oV
i^H0k?oVi^H001Koi^KV00?o0000onKVi_oVi^H0RooVi^H2ool00>coi^KV000FonKViP03o`000?oV
i^Koi^KV08goi^KV00?oo`00onKVi_oVi^H0jOoVi^H001Koi^KV00?o0000onKVi_oVi^H0S_oVi^H2
ool00>Woi^KV000FonKViP03o`000?oVi^Koi^KV093oi^KV00?oo`00onKVi_oVi^H0i_oVi^H001Ko
i^KV0ol0002AonKViP03ool00?oVi^Koi^KV0>Goi^KV000FonKViP03o`000?oVi^Koi^KV097oi^KV
00?oo`00onKVi_oVi^H0iOoVi^H001Koi^KV00?o0000onKVi_oVi^H0T_oVi^H00ooo003oi^KVonKV
iP3TonKViP005_oVi^H00ol0003oi^KVonKViP2BonKViP03ool00?oVi^Koi^KV0>Coi^KV000FonKV
iP03o`000?oVi^Koi^KV09?oi^KV00?oo`00onKVi_oVi^H0hooVi^H001Koi^KV00?o0000onKVi_oV
i^H0TooVi^H00ooo003oi^KVonKViP3SonKViP005_oVi^H00ol0003oi^KVonKViP2ConKViP03ool0
0?oVi^Koi^KV0>?oi^KV000FonKViP03o`000?oVi^Koi^KV09Coi^KV00?oo`00onKVi_oVi^H0h_oV
i^H001Koi^KV00?o0000onKVi_oVi^H0U?oVi^H00ooo003oi^KVonKViP3RonKViP0000Coi^KVo`00
0?l0003o00001?oVi^H2o`0000Coi^KV0_l00006onKViP03o`000?oVi^Koi^KV09Goi^KV00?oo`00
onKVi_oVi^H0hOoVi^H000;oi^KV00?o0000onKVi_oVi^H00_oVi^H01?l0003oi^KVonKVi_l00002
onKViP04o`000?oVi^Koi^KVo`0000Goi^KV00?o0000onKVi_oVi^H0U_oVi^H00ooo003oi^KVonKV
iP3PonKViP000_oVi^H00ol0003oi^KVonKViP02onKViP04o`000?oVi^Koi^KVo`0000;oi^KV00Co
0000onKVi_oVi^Ko00001OoVi^H00ol0003oi^KVonKViP2FonKViP03ool00?oVi^Koi^KV0>3oi^KV
0002onKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKVi_oVi^Ko00000_oVi^H01?l0003oi^KV
onKVi_l00005onKViPCo0000U_oVi^KIool000Woi^KV0002onKViP03o`000?oVi^Koi^KV00;oi^KV
00Co0000onKVi_oVi^Ko00000_oVi^H01?l0003oi^KVonKVi_l00005onKViP03o`000?oVi^Koi^KV
0?ooi^KVN_oVi^H00003onKVi_l0003o000000Coi^KV00Co0000onKVi_oVi^Ko00000_oVi^H01?l0
003oi^KVonKVi_l00005onKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H000;oi^KV00?o0000onKV
i_oVi^H00ooVi^H2o`0000Coi^KV0_l00006onKViP03o`000?oVi^Koi^KV0?ooi^KVN_oVi^H001Ko
i^KV00?o0000onKVi_oVi^H0oooVi^IjonKViP005_oVi^H00ol0003oi^KVonKViP3oonKViW[oi^KV
0000\
\>"],
  ImageRangeCache->{{{0, 401.562}, {247.812, 0}} -> {-7.44773, -6.06624, \
0.324911, 0.438098}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "The somewhat \"random walk\" nature of this graph results from the random \
assignment of Row and Column roles to the players. Recall that whenever two \
players connected by an edge play a game, a fair coin is flipped to decide \
who is Row and who is Column. The outcome of these coin flips crucially \
determines what happens on the boundary between ",
  StyleBox["Boxing",
    FontSlant->"Italic"],
  " and ",
  StyleBox["Theatre",
    FontSlant->"Italic"],
  ", and the gradual accumulation of successes for ",
  StyleBox["Boxing, ",
    FontSlant->"Italic"],
  "in this case, cause it to dominate in the end."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[bos4]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using edge weights for nondeterministic interactions", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "All of the above examples have been using \"unweighted\" graphs. (The \
reason for the scare quote is that ",
  StyleBox["Combinatorica ",
    FontSlant->"Italic"],
  "really considers graphs for which a weight has not been deliberately \
assigned to have an edge weight of 1.) If we want to use edge weights we need \
to (",
  StyleBox["a",
    FontSlant->"Italic"],
  ") assign weights to the edges of a graph, and (",
  StyleBox["b",
    FontSlant->"Italic"],
  ") tell the model that we intend to use these edges weights."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Edge weights should lie strictly in the range ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, 1]\)\)]],
  ". You won't get an error if you assign edges weights outside this range, \
but they do not have a meaningful interpretation as a probability. The \
primary reason for assigning weights to edges is to transform the interaction \
and update model from one of deterministic interaction and updating into \
probabilistic interaction and updating."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "When edges in the interaction neighborhood are assigned weights, the \
weight determines the probability that the interaction specified by that edge \
will take place each generation. Weights assigned to edges in the update \
neighborhood are a little more complicated. Suppose that edge ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  " is assigned weight ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ". If ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  " is ",
  StyleBox["undirected",
    FontSlant->"Italic"],
  ", then the probability that ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " appears in ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "'s update neighborhood on any given generation is ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ", and the probability that ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " appears in ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  "'s update neighborhood on any given generation is also ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ". If ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  " is ",
  StyleBox["directed",
    FontSlant->"Italic"],
  ", then the edge is taken as pointing from ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", meaning that the probability of ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " appearing in ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "'s update neighborhood on any given generation is ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ".  "
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The interaction and update neighborhood do not both need to be \
assigned weights.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "The ",
  StyleBox["Combinatorica",
    FontSlant->"Italic"],
  " function ",
  StyleBox["SetEdgeWeights", "Input"],
  " can be used to set the weights of edges to a random real between 0 and \
1."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(g = SetEdgeWeights[GridGraph[10, 10]];\)\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Here's the first edge and the associated weight.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Edges[g, All]\)\[LeftDoubleBracket]1\[RightDoubleBracket]\)], "Input",\

  CellLabel->"In[21]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{1, 2}, EdgeWeight \[Rule] 0.7128305902504127`}\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "The options ",
  StyleBox["UseInteractionNeighborhoodWeights",
    FontWeight->"Bold"],
  " and ",
  StyleBox["UseUpdateNeighborhoodWeights",
    FontWeight->"Bold"],
  " control whether weights assigned to the specified graph are to be used in \
the dynamics. The default for both options is ",
  StyleBox["False",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"bos5", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] g\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({1, 2}\), \({0, 0}\)},
                    {\({0, 0}\), \({2, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {\ Red, Green}\), 
            ",", "\[IndentingNewLine]", \(UseInteractionNeighborhoodWeights \
\[Rule] True\)}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[22]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[bos5], 
        VertexStyle \[Rule] Box[Large]];\)\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 0 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
0 1 0 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
1 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
0 1 0 r
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
1 0 0 r
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
0 1 0 r
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
1 0 0 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
0 1 0 r
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
1 0 0 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
0 1 0 r
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
1 0 0 r
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
0 1 0 r
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
1 0 0 r
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
0 1 0 r
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
1 0 0 r
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
0 1 0 r
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
1 0 0 r
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
0 1 0 r
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
1 0 0 r
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
0 1 0 r
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
1 0 0 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
0 1 0 r
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
1 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
0 1 0 r
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
1 0 0 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
0 1 0 r
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
1 0 0 r
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
0 1 0 r
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
1 0 0 r
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
0 1 0 r
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
1 0 0 r
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
0 1 0 r
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
1 0 0 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 1 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
1 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
0 1 0 r
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
1 0 0 r
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
0 1 0 r
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
1 0 0 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
0 1 0 r
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
1 0 0 r
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
0 1 0 r
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
1 0 0 r
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
0 1 0 r
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
1 0 0 r
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
0 1 0 r
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
1 0 0 r
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[23]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`000ooooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo
0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooo
oolCool000[o00004ol0o`0:o`0001?o0?l02_l0000Cool000[o00004ol0o`0:o`0001?oo`002_l0
000Cool000[o00004ol0o`0:o`0001?oo`002_l0000Co`3o00Gooooo0003oooooa?oo`002_oooolC
o`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Go
oooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo
000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ol0o`0:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:
oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooo
oa?o0?l01Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`00
0ooooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`00
2_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l01Ooo
ool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00Gooooo0003
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_l0000Cool000[o00004ooo000:o`0001?o0?l0
2_l0000Co`3o00[o00004ooo000:o`0001?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?o0?l0
1Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[o
oooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00Gooooo
0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
0?l01Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oo
oolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool0
00?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0
o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:o`0001?o
0?l02_l0000Cool000[o00004ol0o`0:o`0001?oo`002_l0000Cool000[o00004ooo000:o`0001?o
o`002_l0000Cool000[o00004ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Ooo
ool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`000ooooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`00
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`00
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oo
oolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo0005
ooooo`000ooooolCool000[o00004ol0o`0:o`0001?oo`002_l0000Cool000[o00004ooo000:o`00
01?oo`002_l0000Cool000[o00004ol0o`0:o`0001?oo`002_l0000Cool000Gooooo0003oooooa?o
o`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l0
2_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`001Oooool0
00?ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo0005ooooo`000ooooolCool000[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo0003oooooa?oo`002_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`001Oooool000?ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo
0005ooooo`000ooooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000Gooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooo
oa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`00
0ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
o`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o
00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_l0000Cool000[o00004ooo000:
o`0001?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?o0?l02_l0000Cool000[o00004ol0o`0:
o`0001?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05oooo
o`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`00
2_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolC
o`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l01Oooool000?o
oooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo
4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?o0?l02_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`0:oooooa?o0?l01Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o
0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Go
oooo0003oooooa?oo`002_l0000Cool000[o00004ol0o`0:o`0001?o0?l02_l0000Co`3o00[o0000
4ol0o`0:o`0001?o0?l02_l0000Cool000[o00004ooo000:o`0001?o0?l01Oooool000?ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`00
0ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`05ooooo`000ooooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o
00Gooooo0003oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003
oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o
o`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l0
1Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oo
oolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolC
o`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o
00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:o`0001?o0?l02_l0000Co`3o00[o
00004ooo000:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:o`0001?oo`002_l0000Cool000[o
00004ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo
0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:
oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo000:oooooa?o
0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000[ooooo4ol0o`05ooooo`000ooo
oolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolC
o`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00Go
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo0003oooooa?o0?l02_oooolCool000[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo
4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo
0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000Gooooo0003oooo
oa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?o
o`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
o`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`00
2_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Ooo
ool000?ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolC
ool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o
00[o00004ooo000:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:o`0001?oo`002_l0000Co`3o
00[o00004ooo000:o`0001?oo`002_l0000Cool000Gooooo0003oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo
4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000Gooooo0003
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o
o`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`00
1Oooool000?ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolC
o`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
o`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool0
00[ooooo4ooo000:oooooa?oo`001Oooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000?o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05
ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oo
oolCo`3o00Gooooo0003oooooa?o0?l02_l0000Co`3o00[o00004ooo000:o`0001?oo`002_l0000C
o`3o00[o00004ooo000:o`0001?o0?l02_l0000Co`3o00[o00004ooo000:o`0001?o0?l01Oooool0
00?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0
o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00Gooooo0003oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Oooool000?ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ol0o`05ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCool000[ooooo4ol0o`0:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCo`3o00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oo
oolCo`3o00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?o0?l01Ooo
ool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolC
ool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ol0o`05ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool0
00[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool0
00Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[o
oooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo
4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo
4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ol0
o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo
000:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:
oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:
oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooo
oa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005oooo
o`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o
0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l0
2_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l0
2_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:o`0001?o0?l02_l0
000Co`3o00[o00004ooo000:o`0001?oo`002_l0000Co`3o00[o00004ol0o`0:o`0001?oo`002_l0
000Cool000[o00004ooo0005ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolC
ool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolC
ool000Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool0
00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?o
oooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[o
oooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo
4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo
4ooo000:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o0?l02_oooolCo`3o00[ooooo4ol0
o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo
000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:
oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ooo0005
ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oooolCool000[ooooo4ooo000:oooo
oa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCool000Gooooo0003oooooa?o
0?l02_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`002_oooolCool000[ooooo4ol0o`0:oooooa?o
0?l02_oooolCool000[ooooo4ooo000:oooooa?oo`001Oooool000?ooooo4ol0o`0:oooooa?o0?l0
2_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oooolCo`3o00[ooooo4ol0o`0:oooooa?oo`00
2_oooolCool000[ooooo4ooo0005ooooo`000ooooolCo`3o00[ooooo4ol0o`0:oooooa?o0?l02_oo
oolCool000[ooooo4ooo000:oooooa?o0?l02_oooolCo`3o00[ooooo4ooo000:oooooa?oo`002_oo
oolCool000Gooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Now run the model for 120 generations.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          StrategyHistogram[bos5]]; \[IndentingNewLine]Evolve[
          bos5], {120}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[24]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Organise the strategies into lists as before, then plot.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(s1data = Map[First, l];\)\), "\n", 
    \(\(\(s2data = Map[Last, l];\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(MultipleListPlot[s1data, 
        s2data, \[IndentingNewLine]SymbolShape \[Rule] 
          None, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]PlotStyle \[Rule] {Red, 
            Green}, \[IndentingNewLine]Background \[Rule] 
          RGBColor[ .9,  .9,  .9], \[IndentingNewLine]PlotRange \[Rule] 
          All\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[29]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Background color
.9 .9 .9 r
MFill
% Scaling calculations
0.0158063 0.0080032 -0.742061 0.0210216 [
[.17587 .0863 -6 -9 ]
[.17587 .0863 6 0 ]
[.33593 .0863 -6 -9 ]
[.33593 .0863 6 0 ]
[.496 .0863 -6 -9 ]
[.496 .0863 6 0 ]
[.65606 .0863 -6 -9 ]
[.65606 .0863 6 0 ]
[.81613 .0863 -9 -9 ]
[.81613 .0863 9 0 ]
[.97619 .0863 -9 -9 ]
[.97619 .0863 9 0 ]
[.00331 .20391 -12 -4.5 ]
[.00331 .20391 0 4.5 ]
[.00331 .30902 -12 -4.5 ]
[.00331 .30902 0 4.5 ]
[.00331 .41412 -12 -4.5 ]
[.00331 .41412 0 4.5 ]
[.00331 .51923 -12 -4.5 ]
[.00331 .51923 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 0 r
.25 Mabswid
[ ] 0 setdash
.17587 .0988 m
.17587 .10505 L
s
[(20)] .17587 .0863 0 1 Mshowa
.33593 .0988 m
.33593 .10505 L
s
[(40)] .33593 .0863 0 1 Mshowa
.496 .0988 m
.496 .10505 L
s
[(60)] .496 .0863 0 1 Mshowa
.65606 .0988 m
.65606 .10505 L
s
[(80)] .65606 .0863 0 1 Mshowa
.81613 .0988 m
.81613 .10505 L
s
[(100)] .81613 .0863 0 1 Mshowa
.97619 .0988 m
.97619 .10505 L
s
[(120)] .97619 .0863 0 1 Mshowa
.125 Mabswid
.05582 .0988 m
.05582 .10255 L
s
.09584 .0988 m
.09584 .10255 L
s
.13585 .0988 m
.13585 .10255 L
s
.21589 .0988 m
.21589 .10255 L
s
.2559 .0988 m
.2559 .10255 L
s
.29592 .0988 m
.29592 .10255 L
s
.37595 .0988 m
.37595 .10255 L
s
.41597 .0988 m
.41597 .10255 L
s
.45598 .0988 m
.45598 .10255 L
s
.53601 .0988 m
.53601 .10255 L
s
.57603 .0988 m
.57603 .10255 L
s
.61605 .0988 m
.61605 .10255 L
s
.69608 .0988 m
.69608 .10255 L
s
.73609 .0988 m
.73609 .10255 L
s
.77611 .0988 m
.77611 .10255 L
s
.85614 .0988 m
.85614 .10255 L
s
.89616 .0988 m
.89616 .10255 L
s
.93617 .0988 m
.93617 .10255 L
s
.25 Mabswid
0 .0988 m
1 .0988 L
s
.01581 .20391 m
.02206 .20391 L
s
[(45)] .00331 .20391 1 0 Mshowa
.01581 .30902 m
.02206 .30902 L
s
[(50)] .00331 .30902 1 0 Mshowa
.01581 .41412 m
.02206 .41412 L
s
[(55)] .00331 .41412 1 0 Mshowa
.01581 .51923 m
.02206 .51923 L
s
[(60)] .00331 .51923 1 0 Mshowa
.125 Mabswid
.01581 .11982 m
.01956 .11982 L
s
.01581 .14084 m
.01956 .14084 L
s
.01581 .16187 m
.01956 .16187 L
s
.01581 .18289 m
.01956 .18289 L
s
.01581 .22493 m
.01956 .22493 L
s
.01581 .24595 m
.01956 .24595 L
s
.01581 .26697 m
.01956 .26697 L
s
.01581 .288 m
.01956 .288 L
s
.01581 .33004 m
.01956 .33004 L
s
.01581 .35106 m
.01956 .35106 L
s
.01581 .37208 m
.01956 .37208 L
s
.01581 .3931 m
.01956 .3931 L
s
.01581 .43515 m
.01956 .43515 L
s
.01581 .45617 m
.01956 .45617 L
s
.01581 .47719 m
.01956 .47719 L
s
.01581 .49821 m
.01956 .49821 L
s
.01581 .07778 m
.01956 .07778 L
s
.01581 .05676 m
.01956 .05676 L
s
.01581 .03574 m
.01956 .03574 L
s
.01581 .01472 m
.01956 .01472 L
s
.01581 .54025 m
.01956 .54025 L
s
.01581 .56128 m
.01956 .56128 L
s
.01581 .5823 m
.01956 .5823 L
s
.01581 .60332 m
.01956 .60332 L
s
.25 Mabswid
.01581 0 m
.01581 .61803 L
s
1 0 0 r
.001 w
.02381 .35106 m
.03181 .3931 L
.03982 .35106 L
.04782 .35106 L
.05582 .37208 L
.06383 .37208 L
.07183 .3931 L
.07983 .37208 L
.08784 .45617 L
.09584 .54025 L
.10384 .56128 L
.11184 .60332 L
.11985 .60332 L
.12785 .54025 L
.13585 .56128 L
.14386 .54025 L
.15186 .49821 L
.15986 .37208 L
.16787 .35106 L
.17587 .37208 L
.18387 .3931 L
.19188 .26697 L
.19988 .24595 L
.20788 .24595 L
.21589 .26697 L
.22389 .24595 L
.23189 .24595 L
.2399 .26697 L
.2479 .26697 L
.2559 .33004 L
.26391 .33004 L
.27191 .33004 L
.27991 .33004 L
.28792 .30902 L
.29592 .3931 L
.30392 .35106 L
.31192 .33004 L
.31993 .35106 L
.32793 .41412 L
.33593 .37208 L
.34394 .37208 L
.35194 .33004 L
.35994 .33004 L
.36795 .26697 L
.37595 .30902 L
.38395 .35106 L
.39196 .33004 L
.39996 .33004 L
.40796 .43515 L
.41597 .47719 L
Mistroke
.42397 .3931 L
.43197 .3931 L
.43998 .45617 L
.44798 .41412 L
.45598 .43515 L
.46399 .45617 L
.47199 .43515 L
.47999 .37208 L
.488 .33004 L
.496 .37208 L
.504 .43515 L
.512 .47719 L
.52001 .47719 L
.52801 .45617 L
.53601 .47719 L
.54402 .45617 L
.55202 .3931 L
.56002 .41412 L
.56803 .37208 L
.57603 .37208 L
.58403 .37208 L
.59204 .37208 L
.60004 .37208 L
.60804 .33004 L
.61605 .35106 L
.62405 .33004 L
.63205 .30902 L
.64006 .35106 L
.64806 .35106 L
.65606 .33004 L
.66407 .37208 L
.67207 .35106 L
.68007 .35106 L
.68808 .288 L
.69608 .288 L
.70408 .30902 L
.71208 .30902 L
.72009 .288 L
.72809 .33004 L
.73609 .288 L
.7441 .24595 L
.7521 .24595 L
.7601 .22493 L
.76811 .26697 L
.77611 .22493 L
.78411 .26697 L
.79212 .24595 L
.80012 .26697 L
.80812 .26697 L
.81613 .24595 L
Mistroke
.82413 .26697 L
.83213 .30902 L
.84014 .30902 L
.84814 .288 L
.85614 .30902 L
.86415 .288 L
.87215 .288 L
.88015 .22493 L
.88816 .26697 L
.89616 .26697 L
.90416 .24595 L
.91216 .24595 L
.92017 .30902 L
.92817 .26697 L
.93617 .26697 L
.94418 .24595 L
.95218 .26697 L
.96018 .24595 L
.96819 .24595 L
.97619 .24595 L
Mfstroke
0 1 0 r
.02381 .26697 m
.03181 .22493 L
.03982 .26697 L
.04782 .26697 L
.05582 .24595 L
.06383 .24595 L
.07183 .22493 L
.07983 .24595 L
.08784 .16187 L
.09584 .07778 L
.10384 .05676 L
.11184 .01472 L
.11985 .01472 L
.12785 .07778 L
.13585 .05676 L
.14386 .07778 L
.15186 .11982 L
.15986 .24595 L
.16787 .26697 L
.17587 .24595 L
.18387 .22493 L
.19188 .35106 L
.19988 .37208 L
.20788 .37208 L
.21589 .35106 L
.22389 .37208 L
.23189 .37208 L
.2399 .35106 L
.2479 .35106 L
.2559 .288 L
.26391 .288 L
.27191 .288 L
.27991 .288 L
.28792 .30902 L
.29592 .22493 L
.30392 .26697 L
.31192 .288 L
.31993 .26697 L
.32793 .20391 L
.33593 .24595 L
.34394 .24595 L
.35194 .288 L
.35994 .288 L
.36795 .35106 L
.37595 .30902 L
.38395 .26697 L
.39196 .288 L
.39996 .288 L
.40796 .18289 L
.41597 .14084 L
Mistroke
.42397 .22493 L
.43197 .22493 L
.43998 .16187 L
.44798 .20391 L
.45598 .18289 L
.46399 .16187 L
.47199 .18289 L
.47999 .24595 L
.488 .288 L
.496 .24595 L
.504 .18289 L
.512 .14084 L
.52001 .14084 L
.52801 .16187 L
.53601 .14084 L
.54402 .16187 L
.55202 .22493 L
.56002 .20391 L
.56803 .24595 L
.57603 .24595 L
.58403 .24595 L
.59204 .24595 L
.60004 .24595 L
.60804 .288 L
.61605 .26697 L
.62405 .288 L
.63205 .30902 L
.64006 .26697 L
.64806 .26697 L
.65606 .288 L
.66407 .24595 L
.67207 .26697 L
.68007 .26697 L
.68808 .33004 L
.69608 .33004 L
.70408 .30902 L
.71208 .30902 L
.72009 .33004 L
.72809 .288 L
.73609 .33004 L
.7441 .37208 L
.7521 .37208 L
.7601 .3931 L
.76811 .35106 L
.77611 .3931 L
.78411 .35106 L
.79212 .37208 L
.80012 .35106 L
.80812 .35106 L
.81613 .37208 L
Mistroke
.82413 .35106 L
.83213 .30902 L
.84014 .30902 L
.84814 .33004 L
.85614 .30902 L
.86415 .33004 L
.87215 .33004 L
.88015 .3931 L
.88816 .35106 L
.89616 .35106 L
.90416 .37208 L
.91216 .37208 L
.92017 .30902 L
.92817 .35106 L
.93617 .35106 L
.94418 .37208 L
.95218 .35106 L
.96018 .37208 L
.96819 .37208 L
.97619 .37208 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[29]:=",
  ImageSize->{331.562, 204.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005;0000c2000`400?l00000o`00003ooooVi^I<
onKViP00oooVi^I<onKViP00oooVi^I<onKViP003ooVi^H00ol0003oi^KVonKViP3oonKViS[oi^KV
000?onKViP03o`000?oVi^Koi^KV0?ooi^KV>_oVi^H000ooi^KV00?o0000onKVi_oVi^H0oooVi^Hj
onKViP003ooVi^H00ol0003oi^KVonKViP3oonKViS[oi^KV000?onKViP03o`000?oVi^Koi^KV0?oo
i^KV>_oVi^H000ooi^KV0ol0000LonKViPCo0?l0oooVi^HJonKViP003ooVi^H00ol0003oi^KVonKV
iP0LonKViP04o`3o0?oVi^Koi^KVo`3o0?ooi^KV6_oVi^H000ooi^KV00?o0000onKVi_oVi^H07?oV
i^H01?l0o`3oi^KVonKVi_l0o`3oonKViQ[oi^KV000?onKViP03o`000?oVi^Koi^KV01_oi^KV00Go
0?l0onKVi_oVi^Koi^KVo`3o003oonKViQ[oi^KV000?onKViP03o`000?oVi^Koi^KV01_oi^KV00Go
0?l0onKVi_oVi^Koi^KVo`3o003oonKViQ[oi^KV000?onKViP03o`000?oVi^Koi^KV01_oi^KV00Go
0?l0onKVi_oVi^Koi^KVo`3o003oonKViQ[oi^KV000?onKViP03o`000?oVi^Koi^KV01_oi^KV00?o
0?l0onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKViP3oonKViQOoi^KV000?onKViP?o00006_oVi^H0
0ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV00?o0000onKV
i_oVi^H06_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV0?ooi^KV5ooVi^H000oo
i^KV00?o0000onKVi_oVi^H06_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV0?oo
i^KV5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o
0?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06_oVi^H00ol0o`3oi^KVonKV
iP03onKViP03o`3o0?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06OoVi^H0
0ol0o`3oi^KVonKViP04onKViP03o`3o0?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV0ol0000IonKV
iP03o`3o0?oVi^Koi^KV00Coi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o003oonKViQGoi^KV000?onKV
iP03o`000?oVi^Koi^KV01Woi^KV00?o0?l0onKVi_oVi^H01?oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko
0?l00?ooi^KV5OoVi^H000ooi^KV00?o0000onKVi_oVi^H06OoVi^H00ol0o`3oi^KVonKViP04onKV
iP03o`3o0?oVi^Koi^KV00;o0?l01OoVi^H5o`0000?oi^KV0_l0000[onKViP?o00000ooVi^H2o`00
02[oi^KV0ol00004onKViP;o0000:_oVi^H3o`0000Coi^KV0_l0000WonKViP?o00001?oVi^H2o`00
00Coi^KV0_l0000TonKViP?o00000_oVi^H5o`0000?oi^KV0_l00003onKViP003ooVi^H00ol0003o
i^KVonKViP0HonKViP03o`3o0?oVi^Koi^KV00Koi^KV00Go0?l0onKVi_l0o`3oi^KVo`3o0005onKV
iP04o`000?oVi^Koi^KVo`0000;oi^KV00Co0000onKVi_oVi^Ko0000:ooVi^H01Ol0003oi^KVonKV
i_oVi^Ko000000;oi^KV00?o0000onKVi_oVi^H09_oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;o
i^KV00Co0000onKVi_oVi^Ko0000:?oVi^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000
onKVi_oVi^Ko00009ooVi^H00ol0003oi^KVonKViP02onKViP04o`000?oVi^Koi^KVo`0000;oi^KV
00Co0000onKVi_oVi^Ko00009?oVi^H00ol0003oi^KVonKViP02onKViP04o`000?oVi^Koi^KVo`00
00;oi^KV00Co0000onKVi_oVi^Ko00000_oVi^H000ooi^KV00?o0000onKVi_oVi^H06?oVi^H00ol0
o`3oi^KVonKViP06onKViP;o0?l00_oVi^H00ol0o`3oi^KVonKViP04onKViP03o`000?oVi^Koi^KV
00;oi^KV00Co0000onKVi_oVi^Ko0000:?oVi^H5o`0000;oi^KV00Co0000onKVi_oVi^Ko0000:?oV
i^H01Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko0000:?oVi^H01Ol0003o
i^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00009ooVi^H00ol0003oi^KVonKViP02
onKViP04o`000?oVi^Koi^KVo`0000;oi^KV00Co0000onKVi_oVi^Ko00009?oVi^H00ol0003oi^KV
onKViP03onKViP03o`000?oVi^Koi^KV00;oi^KV00Co0000onKVi_oVi^Ko00000_oVi^H000ooi^KV
00?o0000onKVi_oVi^H06?oVi^H00ol0o`3oi^KVonKViP06onKViP;o0?l00_oVi^H00ol0o`3oi^KV
onKViP05onKViP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H00ol0003oi^KVonKViP0VonKViP04
o`000?oVi^Koi^KVo`0000?oi^KV00Co0000onKVi_oVi^Ko0000:?oVi^H4o`0000?oi^KV00Co0000
onKVi_oVi^Ko0000:OoVi^H3o`0000?oi^KV00Co0000onKVi_oVi^Ko00009ooVi^H00ol0003oi^KV
onKViP02onKViP04o`000?oVi^Koi^KVo`0000;oi^KV00Co0000onKVi_oVi^Ko00009?oVi^H00ol0
003oi^KVonKViP04onKViP05o`000?oVi^Koi^KVonKVi_l000000_oVi^H1o`00007oi^KV0OoVi^H0
00ooi^KV00?o0000onKVi_oVi^H05ooVi^H00ol0o`3oi^KVonKViP07onKViP;o0?l00ooVi^H01Ol0
o`3oi^KVonKVi_oVi^Ko000000?oi^KV00Co0000onKVi_oVi^Ko00000_oVi^H00ol0003oi^KVonKV
iP0WonKViP03o`000?oVi^Ko000000?oi^KV00Co0000onKVi_oVi^Ko0000:?oVi^H00ol0003oi^KV
onKViP04onKViP04o`000?oVi^Koi^KVo`0002Soi^KV00Go0000onKVi_oVi^Koi^KVo`000002onKV
iP04o`000?oVi^Koi^KVo`0002Ooi^KV00?o0000onKVi_oVi^H00_oVi^H01?l0003oi^KVonKVi_l0
0002onKViP04o`000?oVi^Koi^KVo`0002Coi^KV00Go0000onKVi_oVi^Koi^KVo`000003onKViP04
o`000?oVi^Koi^KVo`0000;oi^KV0Ol00001onKViP7oi^KV000?onKViP?o00005ooVi^H00ol0o`3o
i^KVonKViP07onKViP03o`3o0?oVi^Koi^KV00;oi^KV00Go0?l0onKVi_oVi^Koi^KVo`000003onKV
iP04o`000?oVi^Koi^KVo`0000;oi^KV00?o0000onKVi_oVi^H0:?oVi^H2o`0000?oi^KV00Co0000
onKVi_oVi^Ko0000:OoVi^H00ol0003oi^KVonKViP03onKViP04o`000?oVi^Koi^KVo`0002Soi^KV
00Go0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`0002Koi^KV0_l00004onKV
iP04o`000?oVi^Koi^KVo`0000;oi^KV00Co0000onKVi_oVi^Ko00008ooVi^H2o`0000?oi^KV00Go
0000onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`0000;oi^KV000?onKViP03o`00
0?oVi^Koi^KV01Ooi^KV00?o0?l0onKVi_oVi^H03?oVi^H00ol0o`3oi^KVonKViP02onKViP?o0000
1?oVi^H2o`0002coi^KV00?o0000onKVi_oVi^H00_oVi^H2o`0002_oi^KV0_l00004onKViP;o0000
:_oVi^H3o`0000Coi^KV0_l0000XonKViP03o`000?oVi^Koi^KV00?oi^KV0_l00004onKViP;o0000
9OoVi^H00ol0003oi^KVonKViP02onKViP?o00001?oVi^H2o`0000?oi^KV000?onKViP03o`000?oV
i^Koi^KV01Ooi^KV00?o0?l0onKVi_oVi^H03?oVi^H00ol0o`3oi^KVonKViP3oonKViQ7oi^KV000?
onKViP03o`000?oVi^Koi^KV01Ooi^KV00?o0?l0onKVi_oVi^H03OoVi^H00ol0o`3oi^KVonKViP3o
onKViQ3oi^KV000?onKViP03o`000?oVi^Koi^KV01Ooi^KV00?o0?l0onKVi_oVi^H03OoVi^H00ol0
o`3oi^KVonKViP3oonKViQ3oi^KV000?onKViP03o`000?oVi^Koi^KV01Koi^KV00?o0?l0onKVi_oV
i^H03_oVi^H00ol0o`3oi^KVonKViP3oonKViQ3oi^KV000?onKViP03o`000?oVi^Koi^KV01Koi^KV
00?o0?l0onKVi_oVi^H03_oVi^H00ol0o`3oi^KVonKViP3oonKViQ3oi^KV000:onKViQko000000?o
0?l0o`000?l000003ol000000ol0o`3o0000o`00003oo`0000ko00000OoVi^H000ooi^KV00?o0000
onKVi_oVi^H02_oVi^H00ol0003oi^KVonKViP09onKViP03o`3o0?l0003oi^KV00_oi^KV00Go0000
onKVi_oVi^Koi^KVo`3o0007onKViP03o`000?oVi^Koi^KV00[oi^KV00?o0000onKVi_oVi^H02_oV
i^H00ol0003oi^KVonKViP0:onKViP03o`000?oVi^Koi^KV00Woi^KV00?o0000onKVi_oVi^H02_oV
i^H00ol0003oi^KVonKViP0:onKViP03o`000?oVi^Koi^KV00[oi^KV00?o0000onKVi_oVi^H02OoV
i^H00ol0003oi^KVonKViP0:onKViP03o`000?oVi^Koi^KV00[oi^KV00?o0000onKVi_oVi^H02OoV
i^H00ol0003oi^KVonKViP0:onKViP03o`000?oVi^Koi^KV00[oi^KV00?o0000onKVi_oVi^H02_oV
i^H00ol0003oi^KVonKViP09onKViP03o`000?oVi^Koi^KV00[oi^KV00?o0000onKVi_oVi^H02_oV
i^H00ol0003oi^KVonKViP0:onKViP03o`000?oVi^Koi^KV00Woi^KV00?o0000onKVi_oVi^H02_oV
i^H00ol0003oi^KVonKViP07onKViP003ooVi^H00ol0003oi^KVonKViP0FonKViP03o`3o0?oVi^Ko
i^KV00ooi^KV00?o0?l0onKVi_oVi^H01OoVi^H00ol0003oi^KVonKViP0`onKViP03o`000?oVi^Ko
i^KV033oi^KV00?o0000onKVi_oVi^H0<?oVi^H00ol0003oi^KVonKViP0`onKViP03o`000?oVi^Ko
i^KV033oi^KV00?o0000onKVi_oVi^H01ooVi^H000ooi^KV00?o0000onKVi_oVi^H05_oVi^H00ol0
o`3oi^KVonKViP0?onKViP03o`3o0?oVi^Koi^KV0?ooi^KV3ooVi^H000ooi^KV00?o0000onKVi_oV
i^H05_oVi^H00ol0o`3oi^KVonKViP0?onKViP03o`3o0?oVi^Koi^KV0?ooi^KV3ooVi^H000ooi^KV
00?o0000onKVi_oVi^H05_oVi^H00ol0o`3oi^KVonKViP0@onKViP03o`3o0?oVi^Koi^KV0?ooi^KV
3_oVi^H000ooi^KV0ol0000FonKViP03o`3o0?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oVi^H0oooV
i^H>onKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0
onKVi_oVi^H0oooVi^H>onKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV
017oi^KV00?o0?l0onKVi_oVi^H0oooVi^H>onKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03
o`3o0?oVi^Koi^KV017oi^KV00?o0?l0onKVi_oVi^H0oooVi^H>onKViP003ooVi^H00ol0003oi^KV
onKViP0EonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0onKVi_oVi^H0oooVi^H>onKViP003ooV
i^H00ol0003oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0onKVi_oVi^H0oooV
i^H>onKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0
onKVi_oVi^H0oooVi^H>onKViP003ooVi^H3o`0001Goi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0
o`3oi^KVonKViP1AonKViP03o`3o0?oVi^Koi^KV01_oi^KV1?l0o`04onKViP03o`3o0?oVi^Koi^KV
09?oi^KV000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0o`3o
i^KVonKViP1AonKViP03o`3o0?oVi^Koi^KV01_oi^KV00Co0?l0onKVi_oVi^Ko0?l01?oVi^H00ol0
o`3oi^KVonKViP2ConKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV017o
i^KV00?o0?l0onKVi_oVi^H0DOoVi^H00ol0o`3oi^KVonKViP0KonKViP04o`3o0?oVi^Koi^KVo`3o
00?oi^KV00?o0?l0onKVi_l0o`00U?oVi^H000ooi^KV00?o0000onKVi_oVi^H05?oVi^H00ol0o`3o
i^KVonKViP0BonKViP03o`3o0?oVi^Koi^KV053oi^KV0_l0o`0MonKViP05o`3o0?oVi^Koi^KVonKV
i_l0o`000_oVi^H00ol0o`3oi^KVo`3o002DonKViP003ooVi^H00ol0003oi^KVonKViP0DonKViP03
o`3o0?oVi^Koi^KV01?oi^KV00?o0?l0onKVi_oVi^H0CooVi^H2o`3o01coi^KV00?o0?l0onKVi_oV
i^H00_oVi^H00ol0o`3oi^KVo`3o0003onKViP03o`3o0?oVi^Koi^KV097oi^KV000?onKViP03o`00
0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H04ooVi^H00ol0o`3oi^KVonKViP1?onKViP;o0?l0
7?oVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Ko0?l000?oi^KV00?o0?l0onKVi_oVi^H0
TOoVi^H000ooi^KV00?o0000onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP0ConKViP03o`3o0?oV
i^Koi^KV04ooi^KV0_l0o`0LonKViP03o`3o0?oVi^Koi^KV00?oi^KV0_l0o`03onKViP03o`3o0?oV
i^Koi^KV097oi^KV000?onKViP?o00005?oVi^H00ol0o`3oi^KVonKViP0ConKViP03o`3o0?oVi^Ko
i^KV04koi^KV00?o0?l0onKVi_l0o`001_oVi^H00ol0o`3oi^KVonKViP05onKViP03o`3o0?oVi^Ko
i^KV00_oi^KV00?o0?l0onKVi_oVi^H00ooVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Ko
i^KV093oi^KV000?onKViP03o`000?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H04ooVi^H00ol0
o`3oi^KVonKViP1>onKViP04o`3o0?oVi^Koi^KVo`3o00Goi^KV00?o0?l0onKVi_oVi^H01OoVi^H0
0ol0o`3oi^KVonKViP0;onKViP03o`3o0?oVi^Koi^KV00Woi^KV00?o0?l0onKVi_oVi^H0T?oVi^H0
00ooi^KV00?o0000onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV
04koi^KV00Co0?l0onKVi_oVi^Ko0?l01OoVi^H00ol0o`3oi^KVonKViP04onKViP03o`3o0?oVi^Ko
0?l000coi^KV00?o0?l0onKVi_oVi^H02OoVi^H00ol0o`3oi^KVonKViP2@onKViP003ooVi^H00ol0
003oi^KVonKViP0DonKViP03o`3o0?oVi^Koi^KV01?oi^KV00?o0?l0onKVi_oVi^H0C_oVi^H01?l0
o`3oi^KVonKVi_l0o`05onKViP;o0?l01OoVi^H00ol0o`3oi^KVo`3o000<onKViP03o`3o0?oVi^Ko
i^KV00Woi^KV00?o0?l0onKVi_oVi^H0T?oVi^H000ooi^KV00?o0000onKVi_oVi^H05?oVi^H00ol0
o`3oi^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV04koi^KV00Co0?l0onKVi_oVi^Ko0?l01OoVi^H2
o`3o00Goi^KV00Co0?l0onKVi_oVi^Ko0?l02_oVi^H00ol0o`3oi^KVonKViP0:onKViP03o`3o0?oV
i^Koi^KV093oi^KV000?onKViP03o`000?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H04ooVi^H0
0ol0o`3oi^KVonKViP1=onKViP05o`3o0?oVi^Koi^KVonKVi_l0o`001OoVi^H2o`3o00Coi^KV00Go
0?l0onKVi_oVi^Koi^KVo`3o000:onKViP03o`3o0?oVi^Koi^KV00[oi^KV00?o0?l0onKVi_oVi^H0
T?oVi^H000ooi^KV0ol0000DonKViP03o`3o0?oVi^Koi^KV01?oi^KV00?o0?l0onKVi_oVi^H0COoV
i^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Coi^KV00?o0?l0onKVi_l0o`001?oVi^H00ol0o`3oi^KV
onKViP02onKViP03o`3o0?oVi^Koi^KV00Ooi^KV00?o0?l0onKVi_oVi^H02ooVi^H00ol0o`3oi^KV
onKViP2?onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o
0?l0onKVi_oVi^H0COoVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Coi^KV00Co0?l0onKVi_oVi^Ko
0?l00ooVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV00Ooi^KV00?o0?l0onKVi_oV
i^H02ooVi^H00ol0o`3oi^KVonKViP2?onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03o`3o
0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H0COoVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Co
i^KV00Co0?l0onKVi_oVi^Ko0?l00_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV
00Ooi^KV00?o0?l0onKVi_oVi^H02ooVi^H00ol0o`3oi^KVonKViP2?onKViP003ooVi^H00ol0003o
i^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H0COoVi^H01Ol0o`3o
i^KVonKVi_oVi^Ko0?l000Coi^KV00Co0?l0onKVi_oVi^Ko0?l00_oVi^H00ol0o`3oi^KVonKViP03
onKViP03o`3o0?oVi^Koi^KV00Ooi^KV00?o0?l0onKVi_oVi^H02ooVi^H00ol0o`3oi^KVonKViP2?
onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o0?l0onKV
i_oVi^H0COoVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Coi^KV00Ko0?l0onKVi_oVi^Ko0?l0onKV
i_l0o`06onKViP03o`3o0?oVi^Koi^KV00Koi^KV00?o0?l0onKVi_oVi^H03?oVi^H00ol0o`3oi^KV
onKViP2?onKViP000_oVi^H3o`0000;oi^KV0ol00005onKViP03o`000?oVi^Koi^KV01?oi^KV00?o
0?l0onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP1=onKViP05o`3o0?oVi^Koi^KVonKVi_l0o`00
1?oVi^H01_l0o`3oi^KVonKVi_l0o`3oi^KVo`3o00Koi^KV00?o0?l0onKVi_oVi^H01_oVi^H00ol0
o`3oi^KVonKViP0<onKViP03o`3o0?oVi^Koi^KV08ooi^KV0003onKViP04o`000?oVi^Koi^KVo`00
00?oi^KV00?o0000onKVi_oVi^H00_oVi^H00ol0003oi^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV
01Coi^KV00?o0?l0onKVi_oVi^H0COoVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Coi^KV00Co0?l0
onKVi_oVi^Koi^KV0_l0o`07onKViP03o`3o0?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H03?oV
i^H00ol0o`3oi^KVonKViP2?onKViP001Ol00005onKViP03o`000?oVi^Koi^KV00;oi^KV0ol0000C
onKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H0=?oVi^H00ol0o`3oi^KVonKViP0F
onKViP05o`3o0?oVi^Koi^KVonKVi_l0o`001?oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Soi^KV
00?o0?l0onKVi_oVi^H01OoVi^H00ol0o`3oi^KVonKViP0<onKViP05o`3o0?oVi^Koi^KVonKVi_l0
o`00SOoVi^H00004o`000?oVi^Koi^KVo`0000Koi^KV00?o0000onKVi_oVi^H00_oVi^H00ol0003o
i^KVonKViP0ConKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H0=?oVi^H00ol0o`3o
i^KVonKViP0FonKViP03o`3o0?oVi^Koi^KV00;oi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o000<onKV
iP03o`3o0?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H03?oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko
0?l008goi^KV00001?oVi^Ko0000onKVi_l00002onKViPCo00001OoVi^H00ol0003oi^KVonKViP0C
onKViP03o`3o0?oVi^Koi^KV01Coi^KV00?o0?l0onKVi_oVi^H0=?oVi^H00ol0o`3oi^KVonKViP0E
onKViP03o`3o0?oVi^Koi^KV00?oi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o000<onKViP03o`3o0?oV
i^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H03?oVi^H00ol0o`3oi^KVonKViP02o`3o08goi^KV0002
onKViP;o00000_oVi^H00ol0003oi^KVonKViP06onKViP03o`000?oVi^Koi^KV01?oi^KV00?o0?l0
onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP0donKViP03o`3o0?oVi^Koi^KV01Goi^KV00?o0?l0
onKVi_oVi^H00ooVi^H01?l0o`3oi^KVonKVi_l0o`0=onKViP03o`3o0?oVi^Koi^KV00Goi^KV00?o
0?l0onKVi_oVi^H03OoVi^H01?l0o`3oi^KVo`3o0?l0o`2=onKViP000ooVi^H00ol0003oi^KVonKV
iP05o`0000Coi^KV00?o0000onKVi_oVi^H04ooVi^H00ol0o`3oi^KVonKViP0EonKViP03o`3o0?oV
i^Koi^KV03;oi^KV00?o0?l0onKVi_l0o`005_oVi^H00ol0o`3oi^KVonKViP03onKViP04o`3o0?oV
i^Koi^KVo`3o00goi^KV00?o0?l0onKVi_oVi^H01?oVi^H00ol0o`3oi^KVonKViP0>onKViP;o0?l0
0_oVi^H00ol0o`3oi^KVonKViP2:onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03o`3o0?oV
i^Koi^KV01Goi^KV00?o0?l0onKVi_oVi^H0<_oVi^H00ol0o`3oi^KVo`3o000FonKViP03o`3o0?oV
i^Koi^KV00?oi^KV00Co0?l0onKVi_oVi^Ko0?l03OoVi^H00ol0o`3oi^KVonKViP04onKViP03o`3o
0?oVi^Koi^KV00koi^KV0_l0o`02onKViP03o`3o0?oVi^Koi^KV08[oi^KV000?onKViP03o`000?oV
i^Koi^KV01?oi^KV00?o0?l0onKVi_oVi^H05OoVi^H00ol0o`3oi^KVonKViP0bonKViP03o`3o0?oV
i^Ko0?l001Koi^KV00?o0?l0onKVi_oVi^H00ooVi^H01?l0o`3oi^KVonKVi_l0o`0=onKViP03o`3o
0?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oVi^H03_oVi^H2o`3o00;oi^KV00?o0?l0onKVi_oVi^H0
R_oVi^H000ooi^KV0ol00003onKViP03o`3o0?oVi^Koi^KV00Woi^KV00Co0?l0onKVi_oVi^Ko0?l0
6?oVi^H00ol0o`3oi^KVonKViP05onKViP03o`3o0?oVi^Koi^KV027oi^KV00?o0?l0onKVi_oVi^H0
1_oVi^H00ol0o`3oi^KVo`3o000FonKViP03o`3o0?oVi^Koi^KV00?oi^KV1?l0o`0=onKViP03o`3o
0?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oVi^H03_oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l003go
i^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003oi^KVonKViP0OonKViP03ool00?oVi^Koi^KV02Go
i^KV000?onKViP03o`000?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_oVi^H02OoVi^H01?l0o`3oi^KV
onKVi_l0o`0HonKViP03o`3o0?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H08OoVi^H00ol0o`3o
i^KVonKViP06onKViP03o`3o0?oVi^Ko0?l001Koi^KV00?o0?l0onKVi_oVi^H05?oVi^H00ol0o`3o
i^KVonKViP04onKViP03o`3o0?oVi^Koi^KV01;oi^KV00?o0?l0onKVi_oVi^H0>ooVi^H00ooo003o
i^KVonKViP02onKViP03ool00?oVi^Koi^KV01ooi^KV00?oo`00onKVi_oVi^H09OoVi^H000ooi^KV
00?o0000onKVi_oVi^H00ooVi^H00ol0o`3oi^KVonKViP08onKViP05o`3o0?oVi^Ko0?l0onKVi_l0
o`006?oVi^H00ol0o`3oi^KVonKViP04onKViP;o0?l08ooVi^H00ol0o`3oi^KVonKViP06onKViP03
o`3o0?oVi^Ko0?l001Koi^KV00?o0?l0onKVi_oVi^H05OoVi^H00ol0o`3oi^KVonKViP03onKViP03
o`3o0?oVi^Koi^KV01;oi^KV00?o0?l0onKVi_oVi^H0>_oVi^H2ool000Coi^KV00?oo`00onKVi_oV
i^H07ooVi^H00ooo003oi^KVonKViP0UonKViP003ooVi^H00ol0003oi^KVonKViP02onKViP;o0?l0
2_oVi^H01Ol0o`3oi^KVo`3o0?oVi^Ko0?l001Soi^KV00?o0?l0onKVi_oVi^H01?oVi^H2o`3o02?o
i^KV00?o0?l0onKVi_oVi^H01_oVi^H00ol0o`3oi^KVo`3o000FonKViP03o`3o0?oVi^Koi^KV01Go
i^KV00?o0?l0onKVi_oVi^H00ooVi^H00ol0o`3oi^KVonKViP0BonKViP03o`3o0?oVi^Koi^KV03[o
i^KV00?oo`00onKVi_oo00000ooVi^H2ool0023oi^KV00?oo`00onKVi_oVi^H09OoVi^H000ooi^KV
00?o0000onKVi_oVi^H00_oVi^H00ol0o`3oi^KVo`3o0009onKViP03o`3o0?oVi^Koi^KV00;o0?l0
6?oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Ko0?l002?oi^KV0_l0o`06onKViP05o`3o
0?oVi^Koi^KVonKVi_l0o`005OoVi^H00ol0o`3oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV00;o
i^KV00?o0?l0onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP0ionKViP03ool00?oVi^Koo`0000;o
i^KV00?oo`00onKVi_oo00007ooVi^H00ooo003oi^KVool0000VonKViP003ooVi^H00ol0003oi^KV
onKViP02onKViP03o`3o0?oVi^Ko0?l000Soi^KV00Co0?l0onKVi_oVi^Koi^KV0_l0o`0HonKViP03
o`3o0?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_l0o`008_oVi^H00ol0o`3oi^KVo`3o0006onKViP05
o`3o0?oVi^Koi^KVonKVi_l0o`005OoVi^H00ol0o`3oi^KVonKViP0EonKViP03o`3o0?oVi^Koi^KV
00;oi^KV00?o0?l0onKVi_oVi^H05?oVi^H00ol0o`3oi^KVonKViP0honKViP04ool00?oVi^Koi^KV
ool000;oi^KV00?oo`00onKVi_oo00007ooVi^H00ooo003oi^KVool0000VonKViP003ooVi^H3o`00
00;oi^KV00?o0?l0onKVi_l0o`001OoVi^H4o`3o00Coi^KV00?o0?l0onKVi_oVi^H05_oVi^H00ol0
o`3oi^KVonKViP02onKViP04o`3o0?oVi^Koi^KVo`3o00Coi^KV1?oo0004onKViP?oo`004ooVi^H0
0ol0o`3oi^KVo`3o0006onKViP04o`3o0?oVi^Koi^KVonKViPCo0?l04_oVi^H00ol0o`3oi^KVonKV
iP0EonKViP03o`3o0?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oVi^H05?oVi^H;o`3o01Coi^KV00?o
0?l0onKVi_oVi^H05_oVi^H4ool000;oi^KV00Koo`00onKVi_oVi^Koo`00onKVi_oo0003onKViP03
ool00?oVi^Koi^KV00Goi^KV00?oo`00onKVi_oVi^H04OoVi^H00ooo003oi^KVool00005onKViPCo
o`002OoVi^H00ooo003oi^KVonKViP02onKViPKoo`002OoVi^H000ooi^KV00Go0000onKVi_oVi^Ko
i^KVo`3o0002onKViP03o`3o0?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_oVi^H07_oVi^H00ol0o`3o
i^KVonKViP02onKViP04o`3o0?oVi^Koi^KVo`3o00Coi^KV00Coo`00onKVi_oVi^Koo`001?oVi^H0
0ooo003oi^KVool0000ConKViP03o`3o0?oVi^Ko0?l000Koi^KV00?o0?l0onKVi_oVi^H01?oVi^H0
0ol0o`3oi^KVonKViP0@onKViP03o`3o0?oVi^Koi^KV01Goi^KV00?o0?l0onKVi_oVi^H00_oVi^H0
0ol0o`3oi^KVonKViP0NonKViP03o`3o0?oVi^Koi^KV01;oi^KV00?o0?l0onKVi_oVi^H05_oVi^H0
0ooo003oi^KVonKViP04onKViP03ool00?oVi^Koo`0000;oi^KV00Coo`00onKVi_oVi^Koo`001ooV
i^H00ooo003oi^KVonKViP0AonKViP03ool00?oVi^Koo`0000Goi^KV00Coo`00onKVi_oVi^Koo`00
2OoVi^H00ooo003oi^KVonKViP02onKViP03ool00?oVi^Koi^KV00coi^KV000?onKViP05o`000?oV
i^Koi^KVonKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV01ooi^KV
00Go0?l0onKVi_oVi^Koi^KVo`3o0003onKViP05o`3o0?oVi^Koi^KVonKVi_oo00000_oVi^H01Ooo
003oi^KVonKVi_oVi^Koo`0000?oi^KV00?oo`00onKVi_oVi^H04?oVi^H00ol0o`3oi^KVo`3o0006
onKViP03o`3o0?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ol0o`3oi^KVonKViP0E
onKViP03o`3o0?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oVi^H07_oVi^H00ol0o`3oi^KVonKViP0B
onKViP03o`3o0?oVi^Koi^KV01Koi^KV00?oo`00onKVi_oVi^H01?oVi^H00ooo003oi^KVool00002
onKViP03ool00?oVi^Koi^KV00;oo`001_oVi^H2ool001;oi^KV00?oo`00onKVi_oo00001OoVi^H0
1?oo003oi^KVonKVi_oo0008onKViP;oo`000ooVi^H00ooo003oi^KVonKViP0=onKViP003ooVi^H0
1Ol0003oi^KVonKVi_oVi^Ko0?l000;oi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKV
iP0PonKViP04o`3o0?oVi^Koi^KVo`3o00Coi^KV00Co0?l0onKVi_oVi^Koo`001?oVi^H01?oo003o
i^KVonKVi_oo0003onKViP03ool00?oVi^Koi^KV013oi^KV00?o0?l0onKVi_l0o`001_oVi^H00ol0
o`3oi^KVonKViP04onKViP03o`3o0?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oVi^H05_oVi^H01Ol0
o`3oi^KVonKVi_oVi^Ko0?l0027oi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ol0o`3oi^KVo`3o000G
onKViP03ool00?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oo00000_oVi^H01Ooo003oi^KVool00?oV
i^Koo`0000Goi^KV00?oo`00onKVi_oo00004_oVi^H00ooo003oi^KVool00004onKViP05ool00?oV
i^Koi^KVonKVi_oo00002?oVi^H00ooo003oi^KVool00002onKViP03ool00?oVi^Koi^KV00goi^KV
000?onKViP05o`000?oVi^Koi^KVonKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o
0?oVi^Koi^KV027oi^KV00?o0?l0onKVi_l0o`001?oVi^H01?l0o`3oi^KVonKVi_oo0004onKViP03
ool00?oVi^Koo`0000Goi^KV00?oo`00onKVi_oVi^H03ooVi^H00ol0o`3oi^KVo`3o0006onKViP03
o`3o0?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H03ooVi^H00ol0o`3oi^KVonKViP0FonKViP04
o`3o0?oVi^Koi^KVo`3o02;oi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ol0o`3oi^KVo`3o000FonKV
iP03ool00?oVi^Koi^KV00Goi^KV00?oo`00onKVi_oo00000_oVi^H00ooo003oi^KVool00002onKV
iP03ool00?oVi^Koi^KV00;oi^KV00Coo`00onKVi_oVi^Koo`004OoVi^H00ooo003oi^KVool00004
onKViP05ool00?oVi^Koi^KVonKVi_oo00001ooVi^H01_oo003oi^KVonKVi_oo003oi^KVool0013o
i^KV000?onKViP05o`000?oVi^Koi^KVonKVi_l0o`000ooVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l0
02?oi^KV00?o0?l0onKVi_l0o`001?oVi^H01?l0o`3oi^KVonKVi_oo0004onKViP03ool00?oVi^Ko
o`0000Goi^KV00?oo`00onKVi_oVi^H03ooVi^H01?l0o`3oi^KVonKVi_l0o`04onKViP03o`3o0?oV
i^Koi^KV00Koi^KV00?o0?l0onKVi_oVi^H03ooVi^H00ol0o`3oi^KVonKViP0FonKViP04o`3o0?oV
i^Koi^KVo`3o02;oi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ol0o`3oi^KVo`3o000FonKViP03ool0
0?oVi^Koi^KV00Goi^KV0_oo0003onKViP03ool00?oVi^Koo`0000;oi^KV00?oo`00onKVi_oVi^H0
0_oVi^H01?oo003oi^KVonKVi_oo000@onKViP05ool00?oVi^Koi^KVonKVi_oo00000ooVi^H01Ooo
003oi^KVonKVi_oVi^Koo`0000Ooi^KV00Koo`00onKVi_oVi^Koo`00onKVi_oo000@onKViP003ooV
i^H01?l0003oi^KVonKVi_l0o`04onKViP04o`3o0?oVi^Koi^KVo`3o02Coi^KV0_l0o`05onKViP03
o`3o0?oVi^Koo`0000Koi^KV0_oo0005onKViP03ool00?oVi^Koi^KV00ooi^KV00Co0?l0onKVi_oV
i^Ko0?l01?oVi^H00ol0o`3oi^KVonKViP06onKViP03o`3o0?oVi^Koi^KV00koi^KV00?o0?l0onKV
i_oVi^H05ooVi^H01?l0o`3oi^KVonKVi_l0o`0RonKViP03o`3o0?oVi^Koi^KV013oi^KV00Co0?l0
onKVi_oVi^Ko0?l05OoVi^H00ooo003oi^KVonKViP06onKViP03ool00?oVi^Koi^KV00;oi^KV00Go
o`00onKVi_oVi^Koi^KVool00003onKViP05ool00?oVi^Koi^KVonKVi_oo00004?oVi^H01Ooo003o
i^KVonKVi_oVi^Koo`0000;oi^KV00?oo`00onKVi_oVi^H00ooVi^H00ooo003oi^KVonKViP04onKV
iP04ool00?oVi^Koi^KVonKViP;oo`004?oVi^H000ooi^KV0ol000000ol0o`3oi^KVonKViP02onKV
iPCo0?l09OoVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koo`0000Koi^KV00?oo`00onKV
i_oVi^H01OoVi^H3ool000koi^KV00Co0?l0onKVi_oVi^Ko0?l01?oVi^H00ol0o`3oi^KVonKViP06
onKViP03o`3o0?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKViP02
onKViP03o`3o0?oVi^Koi^KV01Ooi^KV00Co0?l0onKVi_oVi^Ko0?l08_oVi^H01Ol0o`3oi^KVonKV
i_oVi^Ko0?l000Ooi^KV1?l0o`03onKViP03o`3o0?oVi^Koi^KV00Co0?l04_oVi^H00ooo003oi^KV
onKViP06onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H00_oVi^H4ool000Coi^KV
00?oo`00onKVi_oVi^H03OoVi^H01?oo003oi^KVonKVi_oVi^H4ool000Goi^KV00Coo`00onKVi_oV
i^Koi^KV0ooo0004onKViP03ool00?oVi^Koi^KV00ooi^KV000?onKViP03o`000?oVi^Koi^KV03Co
i^KV00?o0?l0onKVi_oo00004?oVi^H00ooo003oi^KVonKViP0;onKViP05o`3o0?oVi^Koi^KVonKV
i_l0o`001?oVi^H00ol0o`3oi^KVonKViP06onKViP03o`3o0?oVi^Koi^KV00Coi^KV00?oo`00onKV
i_oVi^H00_oVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV01Soi^KV00?o0?l0onKV
i_l0o`008ooVi^H01?l0o`3oi^KVonKVi_l0o`07onKViP04o`3o0?oVi^Koi^KVo`3o00;oi^KV00?o
0?l0onKVi_oVi^H01?oVi^H00ol0o`3oi^KVonKViP0@onKViP03ool00?oVi^Koi^KV01Soi^KV00?o
o`00onKVi_oVi^H03OoVi^H00ooo003oi^KVonKViP0:onKViP05ool00?oVi^Koi^KVonKVi_oo0000
6?oVi^H000ooi^KV00?o0000onKVi_oVi^H0=?oVi^H00ol0o`3oi^KVool0000@onKViP03ool00?oV
i^Koi^KV00_oi^KV00?o0?l0onKVi_oVi^H00_oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Soi^KV
00?o0?l0onKVi_oVi^H01?oVi^H00ooo003oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV00;oi^KV
00?o0?l0onKVi_oVi^H06?oVi^H00ol0o`3oi^KVo`3o000SonKViP03o`3o0?oVi^Koi^KV00;o0?l0
1_oVi^H01?l0o`3oi^KVonKVi_l0o`02onKViP03o`3o0?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oV
i^H04?oVi^H00ooo003oi^KVonKViP0HonKViP03ool00?oVi^Koi^KV00goi^KV00?oo`00onKVi_oV
i^H02_oVi^H01Ooo003oi^KVonKVi_oVi^Koo`0001Soi^KV000?onKViP03o`000?oVi^Koi^KV03Co
i^KV00?o0?l0onKVi_oo00004?oVi^H00ooo003oi^KVonKViP0;onKViP03o`3o0?oVi^Koi^KV00;o
i^KV00Co0?l0onKVi_oVi^Ko0?l02OoVi^H00ol0o`3oi^KVonKViP04onKViP07ool00?oVi^Koi^KV
onKVi_l0o`3oi^KVo`3o0003onKViP03o`3o0?oVi^Koi^KV01Soi^KV00?o0?l0onKVi_l0o`008ooV
i^H01Ol0o`3oi^KVo`3o0?oVi^Ko0?l000Koi^KV00Oo0?l0onKVi_oVi^Koi^KVo`3o0?oVi^Ko0?l0
00Koi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ooo003oi^KVonKViP0HonKViP03ool00?oVi^Koi^KV
00goi^KV00?oo`00onKVi_oVi^H02_oVi^H01?oo003oi^KVonKVi_oo000IonKViP003ooVi^H00ol0
003oi^KVonKViP0donKViP03o`3o0?oVi^Koo`00017oi^KV00?oo`00onKVi_oVi^H02_oVi^H00ol0
o`3oi^KVonKViP03onKViP03o`3o0?oVi^Ko0?l000[oi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ooo
003oi^KVool00002onKViP03o`3o0?oVi^Ko0?l000?oi^KV00?o0?l0onKVi_oVi^H06?oVi^H2o`3o
02Coi^KV00?o0?l0onKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV
00;oi^KV00?o0?l0onKVi_l0o`001_oVi^H00ol0o`3oi^KVonKViP0?onKViP03ool00?oVi^Koi^KV
01[oi^KV00?oo`00onKVi_oVi^H03?oVi^H00ooo003oi^KVonKViP0:onKViP04ool00?oVi^Koi^KV
ool001Woi^KV000?onKViP03o`000?oVi^Koi^KV03Coi^KV00?o0?l0onKVi_oo00004OoVi^H00ooo
003oi^KVonKViP0:onKViP03o`3o0?oVi^Koi^KV00?oi^KV00?o0?l0onKVi_l0o`002_oVi^H00ol0
o`3oi^KVonKViP02onKViP03ool00?oVi^Koo`0000;oi^KV00?o0?l0onKVi_l0o`000ooVi^H00ol0
o`3oi^KVonKViP0HonKViP;o0?l09?oVi^H00ol0o`3oi^KVo`3o0002onKViP03o`3o0?oVi^Koi^KV
00;oi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ol0o`3oi^KVo`3o0006onKViP03o`3o0?oVi^Koi^KV
00ooi^KV00?oo`00onKVi_oVi^H06_oVi^H00ooo003oi^KVonKViP0;onKViP03ool00?oVi^Koi^KV
00_oi^KV00Coo`00onKVi_oVi^Koo`006OoVi^H000ooi^KV00?o0000onKVi_oVi^H0=?oVi^H00ol0
o`3oi^KVool0000AonKViP03ool00?oVi^Koi^KV00[oi^KV00?o0?l0onKVi_oVi^H00ooVi^H2o`3o
00_oi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ooo003oi^KVool00002onKViP04o`3o0?oVi^Koi^KV
o`3o00;oi^KV00?o0?l0onKVi_oVi^H06OoVi^H00ol0o`3oi^KVonKViP0SonKViP05o`3o0?oVi^Ko
i^KVonKVi_l0o`001?oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oVi^Koi^KV00Koi^KV00?o
0?l0onKVi_oVi^H03_oVi^H00ooo003oi^KVonKViP0JonKViP03ool00?oVi^Koi^KV00_oi^KV00?o
o`00onKVi_oVi^H02ooVi^H01?oo003oi^KVonKVi_oo000IonKViP003ooVi^H3o`0003Coi^KV00?o
0?l0onKVi_oo00004OoVi^H00ooo003oi^KVo`3o0008o`3o00;oi^KV00?o0?l0onKVi_oVi^H01?oV
i^H00ol0o`3oi^KVonKViP09onKViPCo0?l000Coi^KVool00?oVi^Koo`000_oVi^H00ol0o`3oi^KV
onKViP04o`3o01_oi^KV00?o0?l0onKVi_oVi^H08ooVi^H00ol0o`3oi^KVonKViP02onKViP05o`3o
0?oVi^Koi^KVonKVi_l0o`001OoVi^H00ol0o`3oi^KVonKViP06onKViP03o`3o0?oo003oo`0000;o
o`001ooVi^H00ooo003oi^KVo`3o0002onKViP03ool00?oVi^Koi^KV01[oi^KV00?oo`00onKVi_oV
i^H00ooVi^H00ooo003oi^KVonKViP02onKViPCoo`003OoVi^H01?oo003oi^KVonKVi_oo000IonKV
iP003ooVi^H00ol0003oi^KVonKViP0eonKViP03o`3o0?oVi^Koi^KV013oi^KV00?oo`00onKVi_l0
o`001ooVi^H01?l0o`3oi^KVonKVi_l0o`0EonKViP07o`3o0?oVi^Koo`00onKVi_oo003oi^KVo`3o
001=onKViP05o`3o0?oVi^Koi^KVonKVi_l0o`003_oVi^H01Ol0o`3oo`00onKVi_oVi^Koo`0000Oo
i^KV00?oo`00onKVi_l0o`000_oVi^H00ooo003oi^KVonKViP0JonKViP03ool00?oVi^Koi^KV00?o
i^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003oi^KVonKViP0>onKViP03ool00?oVi^Koo`0001[o
i^KV000?onKViP03o`000?oVi^Koi^KV03Goi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ooo003oi^KV
o`3o0007onKViP04o`3o0?oVi^Koi^KVo`3o01Goi^KV00Oo0?l0onKVi_oo003oi^KVool00?oVi^Ko
0?l004koi^KV00Co0?l0onKVi_oVi^Ko0?l03_oVi^H01Ol0o`3oo`00onKVi_oVi^Koo`0000Koi^KV
0_oo00001OoVi^Ko0?l0onKVi_oVi^Koo`0001coi^KV00?oo`00onKVi_oVi^H00ooVi^H2ool000?o
i^KV00?oo`00onKVi_oVi^H03_oVi^H00ooo003oi^KVool0000JonKViP003ooVi^H00ol0003oi^KV
onKViP0eonKViP03o`3o0?oVi^Koi^KV013oi^KV00?oo`00onKVi_l0o`002?oVi^H2o`3o01Koi^KV
00Oo0?l0onKVi_oo003oi^KVool00?oVi^Ko0?l004koi^KV00Co0?l0onKVi_oVi^Ko0?l03_oVi^H0
0ol0o`3oo`00onKViP02onKViP03ool00?oVi^Koi^KV00?oi^KV0_oo00001?oVi^Ko0?l0o`3o0?oo
000MonKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oo00000_oVi^H00ooo003oi^KVonKV
iP0@onKViP;oo`006_oVi^H00004onKVi_l0003o0000o`0000Coi^KV0_l00005onKViP03o`000?oV
i^Koi^KV03Goi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0o`3oi^KVonKViP07onKViP;o0?l05ooV
i^H01_l0o`3oi^KVonKVi_oVi^Koo`00o`3o04ooi^KV0_l0o`0?onKViP03o`3o0?oo003oi^KV00;o
i^KV00?oo`00onKVi_oVi^H00_oVi^H01ooo003oi^KVonKVi_l0o`3oi^KVo`3o0?oo00007_oVi^H0
1Ooo003oi^KVonKVi_oVi^Koo`0000;oi^KV00?oo`00onKVi_oo00004_oVi^H2ool001[oi^KV0000
1Ol0003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00001?oVi^H00ol0003oi^KV
onKViP0eonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0onKVi_oVi^H01ooVi^H2o`3o01Ooi^KV
00Ko0?l0onKVi_oVi^Koi^KVool00?l0o`1?onKViP;o0?l03ooVi^H00ol0o`3oo`00onKViP02onKV
iP03ool00?oVi^Koi^KV00;oi^KV00Ooo`00onKVi_oVi^Ko0?l0onKVi_l0o`3oo`0001koi^KV00Go
o`00onKVi_oVi^Koi^KVool00002onKViP03ool00?oVi^Koo`0001;oi^KV0_oo000JonKViP001?oV
i^H01?l0003oi^KVonKVi_l00002onKViP03o`000?oVi^Koi^KV00;oi^KV00?o0000onKVi_oVi^H0
=OoVi^H00ol0o`3oi^KVonKViP0AonKViP03o`3o0?oVi^Koi^KV00Soi^KV00?o0?l0onKVi_oVi^H0
5OoVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l0053oi^KV0_l0o`0?onKViP03o`3o0?oVi^Koi^KV00?o
i^KV00Goo`00onKVi_oVi^Koi^KVool00002onKViP04o`3o0?oVi^Ko0?l0ool001koi^KV00Coo`00
onKVi_oVi^Koo`000ooVi^H2ool001?oi^KV00?oo`00onKVi_oVi^H06OoVi^H000Coi^KV00Co0000
onKVi_oVi^Ko00000_oVi^H00ol0003oi^KVonKViP02onKViP?o0000=OoVi^H00ol0o`3oi^KVonKV
iP0AonKViP03o`3o0?oVi^Koi^KV00Soi^KV00?o0?l0onKVi_oVi^H05OoVi^H01Ol0o`3oi^KVonKV
i_oVi^Ko0?l0057oi^KV00?o0?l0onKVi_oVi^H03OoVi^H00ol0o`3oi^KVonKViP03onKViPCo0?l0
0ooVi^H01?l0o`3oi^KVool00?l0o`0NonKViPCo0?l01?oVi^H00ol0o`3oi^KVonKViP0AonKViP03
o`3o0?oVi^Koi^KV01Woi^KV0004o`0000?oi^KV00Co0000onKVi_oVi^Ko00001?oVi^H00ol0003o
i^KVonKViP0eonKViP03o`3o0?oVi^Koi^KV017oi^KV00?o0?l0onKVi_oVi^H02?oVi^H00ooo003o
i^KVonKViP0EonKViP05o`3o0?oVi^Koi^KVonKVi_l0o`00DOoVi^H00ooo003oi^KVonKViP0=onKV
iP03o`3o0?oVi^Koi^KV00?oi^KV00Co0?l0onKVi_oVi^Ko0?l00ooVi^H01?l0o`3oi^KVool00?l0
o`0NonKViP04o`3o0?oVi^Koi^KVo`3o00Coi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0o`3oi^KV
onKViP0IonKViP0000?o0000onKVi_oVi^H01?oVi^H01?l0003oi^KVonKVi_l00004onKViP03o`00
0?oVi^Koi^KV03Goi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0o`3oi^KVonKViP07onKViP;oo`00
5ooVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l0053oi^KV0_oo000?onKViP03ool00?l0o`3oi^KV00;o
i^KV00?o0?l0onKVi_oVi^H00_oVi^H01ol0o`3oi^KVonKVi_l0o`3oi^KVool00?l0o`007_oVi^H0
1Ol0o`3oi^KVonKVi_oVi^Ko0?l000;oi^KV00?o0?l0onKVi_l0o`004_oVi^H00ol0o`3oi^KVonKV
iP0IonKViP001Ol00003onKViP;o00001OoVi^H00ol0003oi^KVonKViP0eonKViP03o`3o0?oVi^Ko
i^KV017oi^KV00?o0?l0onKVi_oVi^H01ooVi^H2ool001Ooi^KV00Ko0?l0onKVi_oVi^Koi^KVo`3o
0?oo001?onKViP;oo`003ooVi^H00ooo003o0?l0onKViP02onKViP03o`3o0?oVi^Koi^KV00;oi^KV
00Oo0?l0onKVi_oVi^Ko0?l0onKVi_oo003o0?l001koi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o0002
onKViP03o`3o0?oVi^Ko0?l001;oi^KV0_l0o`0JonKViP003ooVi^H00ol0003oi^KVonKViP0eonKV
iP03o`3o0?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oo00002?oVi^H2ool001Koi^KV00Ooo`00onKV
i_l0o`3oi^KVo`3o0?oVi^Koo`0004koi^KV00Coo`00onKVi_oVi^Koo`003_oVi^H00ooo003o0?l0
onKViP02onKViP03o`3o0?oVi^Koi^KV00?oi^KV0_l0o`001?oVi^Koo`00ool00?l0o`0MonKViP03
o`3o0?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP0@onKViP;o
0?l06_oVi^H000ooi^KV00?o0000onKVi_oVi^H0=OoVi^H00ol0o`3oi^KVonKViP0@onKViP03o`3o
0?oVi^Koo`0000Ooi^KV00Coo`00onKVi_oVi^Koo`005OoVi^H01ooo003oi^KVo`3o0?oVi^Ko0?l0
onKVi_oo0000C_oVi^H01?oo003oi^KVonKVi_oo000>onKViP05ool00?l0o`3oi^KVonKVi_l0o`00
1_oVi^H2o`3o0005onKVi_oo003oi^KVonKVi_l0o`007?oVi^H00ol0o`3oi^KVonKViP03onKViP;o
0?l00ooVi^H00ol0o`3oi^KVonKViP0?onKViP;o0?l06_oVi^H000ooi^KV0ol0000eonKViP03o`3o
0?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oo00002?oo0002onKViP03ool00?oVi^Koi^KV00Coi^KV
00?oo`00onKVi_oVi^H02OoVi^H4ool00006onKVi_l0o`3oi^KVo`3o0?oVi^Koo`000ooVi^H4ool0
01_oi^KV00?oo`00onKVi_oVi^H08ooVi^H00ooo003oi^KVonKViP02onKViP05ool00?oVi^Koi^KV
onKVi_oo00001OoVi^H00ooo003oi^KVonKViP06onKViP03ool00?l0o`3o0?l000;o0?l01ooVi^H0
0ol0o`3oi^KVool00002onKViP03o`3o0?oVi^Koi^KV01[oi^KV00?o0?l0onKVi_oVi^H00ooVi^H0
0ol0o`3oi^KVonKViP02onKViPCo0?l03OoVi^H00ol0o`3oi^KVo`3o000JonKViP003ooVi^H00ol0
003oi^KVonKViP0donKViP03ool00?oVi^Ko0?l0017oi^KV00?o0?l0onKVi_oVi^H02_oVi^H00ooo
003oi^KVonKViP04onKViP03ool00?oVi^Koi^KV00Woi^KV00?oo`00onKVi_oVi^H00_oVi^H00ol0
o`3oi^KVo`3o0002onKViP04ool00?oVi^Koi^KVool000;oi^KV00?oo`00onKVi_oVi^H06OoVi^H0
0ooo003oi^KVonKViP0SonKViP03ool00?oVi^Koi^KV00;oi^KV00Goo`00onKVi_oVi^Koi^KVool0
0005onKViP03ool00?oVi^Koi^KV00Koi^KV00?oo`00onKVi_oVi^H03_oVi^H00ol0o`3oi^KVonKV
iP0JonKViP03o`3o0?oVi^Koi^KV00_oi^KV00?o0?l0onKVi_oVi^H02ooVi^H01?l0o`3oi^KVonKV
i_l0o`0IonKViP003ooVi^H00ol0003oi^KVonKViP0donKViP03ool00?oVi^Ko0?l0017oi^KV00?o
0?l0onKVi_oVi^H02_oVi^H00ooo003oi^KVonKViP03onKViP;oo`002ooVi^H00ooo003oi^KVonKV
iP02onKViP03o`3o0?oVi^Ko0?l000;oi^KV00Coo`00onKVi_oVi^Koo`000_oVi^H00ooo003oi^KV
onKViP0IonKViP03ool00?oVi^Koi^KV02?oi^KV00Goo`00onKVi_oVi^Koi^KVool00004onKViP03
ool00?oVi^Koi^KV00?oi^KV00?oo`00onKVi_oVi^H01_oVi^H00ooo003oi^KVonKViP0>onKViP03
o`3o0?oVi^Koi^KV01[oi^KV00?o0?l0onKVi_oVi^H02ooVi^H00ol0o`3oi^KVonKViP0;onKViP04
o`3o0?oVi^Koi^KVo`3o01Woi^KV000?onKViP03o`000?oVi^Koi^KV03Coi^KV00?oo`00onKVi_l0
o`004OoVi^H00ol0o`3oi^KVonKViP0:onKViP03ool00?oVi^Koi^KV00?oi^KV00?oo`00onKVi_oo
00002_oVi^H00ooo003oi^KVonKViP02onKViP03o`3o0?oVi^Ko0?l000;oi^KV00?oo`00onKVi_oo
00000ooVi^H00ooo003oi^KVonKViP0HonKViP;oo`009?oVi^H00ooo003oi^KVool00002onKViP03
ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003oi^KVool00006onKViP03
ool00?oVi^Koi^KV00ooi^KV00?o0?l0onKVi_oVi^H06_oVi^H00ol0o`3oi^KVonKViP0;onKViP03
o`3o0?oVi^Koi^KV00_oi^KV00Co0?l0onKVi_oVi^Ko0?l06OoVi^H000ooi^KV00?o0000onKVi_oV
i^H0=?oVi^H00ooo003oi^KVo`3o000AonKViP03o`3o0?oVi^Koi^KV00[oi^KV00?oo`00onKVi_oV
i^H00_oVi^H01?oo003oi^KVonKVi_oo0009onKViP03ool00?oVi^Koi^KV00?oi^KV00?o0?l0onKV
i_l0o`000_oVi^H00ooo003oi^KVool00003onKViP03ool00?oVi^Koi^KV01Soi^KV00?oo`00onKV
i_oo00008ooVi^H01Ooo003oi^KVool00?oVi^Koo`0000Goi^KV00?oo`00onKVi_oVi^H00_oVi^H0
0ooo003oi^KVool00006onKViP03ool00?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oVi^H06OoVi^H0
0ol0o`3oi^KVonKViP0<onKViP03o`3o0?oVi^Koi^KV00[oi^KV00Co0?l0onKVi_oVi^Ko0?l06OoV
i^H000ooi^KV00?o0000onKVi_oVi^H0=?oVi^H00ooo003oi^KVo`3o000@onKViP03o`3o0?oVi^Ko
i^KV00_oi^KV00?oo`00onKVi_oVi^H00_oVi^H01?oo003oi^KVonKVi_oo0009onKViP03ool00?oV
i^Koi^KV00Coi^KV00Oo0?l0onKVi_oVi^Koi^KVool00?oVi^Koo`0000?oi^KV00?oo`00onKVi_oV
i^H06?oVi^H00ooo003oi^KVool0000SonKViP05ool00?oVi^Koo`00onKVi_oo00001_oVi^H01ooo
003oi^KVonKVi_oVi^Koo`00onKVi_oo00001_oVi^H00ooo003oi^KVonKViP0@onKViP03o`3o0?oV
i^Koi^KV01Soi^KV00?o0?l0onKVi_oVi^H03OoVi^H00ol0o`3oi^KVonKViP0:onKViP04o`3o0?oV
i^Koi^KVo`3o01Woi^KV000?onKViP03o`000?oVi^Koi^KV03Coi^KV00?oo`00onKVi_l0o`004?oV
i^H00ol0o`3oi^KVonKViP0;onKViP03ool00?oVi^Koi^KV00;oi^KV00Goo`00onKVi_oVi^Koi^KV
ool00008onKViP03ool00?oVi^Koi^KV00Coi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ooo003oi^KV
onKViP02onKViP03ool00?oVi^Koi^KV01Soi^KV00?oo`00onKVi_oo00008ooVi^H00ooo003oi^KV
onKViP02ool000Koi^KV00Coo`00onKVi_oVi^Koo`000_oVi^H00ooo003oi^KVonKViP04onKViP03
ool00?oVi^Koi^KV013oi^KV00?o0?l0onKVi_oVi^H06?oVi^H00ol0o`3oi^KVonKViP0=onKViP03
o`3o0?oVi^Koi^KV00[oi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o000HonKViP003ooVi^H3o`000003
ool00?oVi^Koi^KV00;oi^KV1?oo000UonKViP03ool00?oVi^Koi^KV00?oi^KV00?oo`00onKVi_l0
o`001_oVi^H00ol0o`3oi^KVonKViP05onKViP?o0?l03OoVi^H01Ooo003oi^KVonKVi_oVi^Koo`00
00Coi^KV00?oo`00onKVi_oVi^H01_oVi^H00ooo003oi^KVonKViP04onKViP03o`3o0?oVi^Koi^KV
00;oi^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003oi^KVonKViP0HonKViP03ool00?oVi^Koo`00
02?oi^KV00Coo`00onKVi_oVi^Koo`001ooVi^H4ool000;oi^KV00Coo`00onKVi_oVi^Koi^KV1?oo
000BonKViP03o`3o0?oVi^Koi^KV00Koi^KV00?o0?l0onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKV
iP02onKViPCo0?l01?oVi^H00ol0o`3oi^KVonKViP0=onKViP04o`3o0?oVi^Koi^KVonKViPCo0?l0
1OoVi^H01?l0o`3oi^KVonKVi_oVi^H3o`3o00Coi^KV00?o0?l0onKVi_oVi^H03ooVi^H000ooi^KV
00Co0000onKVi_oVi^Koo`001?oVi^H01?oo003oi^KVonKVi_oo000UonKViP03ool00?oVi^Koi^KV
00?oi^KV00?oo`00onKVi_l0o`001_oVi^H00ol0o`3oi^KVonKViP05onKViP03o`3o0?oVi^Koi^KV
00koi^KV00Coo`00onKVi_oVi^Koo`001?oVi^H00ooo003oi^KVonKViP06onKViP03ool00?oVi^Ko
i^KV00koi^KV00?oo`00onKVi_oVi^H05ooVi^H01?oo003oi^KVonKVi_oo000RonKViP03ool00?oV
i^Koi^KV013oi^KV00Coo`00onKVi_oVi^Koo`005OoVi^H00ol0o`3oi^KVonKViP06onKViP03o`3o
0?oVi^Koi^KV00;oi^KV00?o0?l0onKVi_oVi^H00_oVi^H01?l0o`3oi^KVonKVi_l0o`04onKViP03
o`3o0?oVi^Koi^KV00goi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o0002onKViP03o`3o0?oVi^Koi^KV
00?oi^KV00?o0?l0onKVi_oVi^H00ooVi^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV
00ooi^KV000?onKViP04o`000?oVi^Koi^KVool000Coi^KV00Coo`00onKVi_oVi^Koo`009?oVi^H2
ool000Goi^KV00?oo`00onKVi_l0o`001_oVi^H2o`3o00Goi^KV00?o0?l0onKVi_oVi^H03ooVi^H0
1?oo003oi^KVonKVi_oo0004onKViP03ool00?oVi^Koi^KV00Koi^KV00?oo`00onKVi_oVi^H03ooV
i^H00ooo003oi^KVonKViP0FonKViP04ool00?oVi^Koi^KVool002;oi^KV00?oo`00onKVi_oVi^H0
4?oVi^H01?oo003oi^KVonKVi_oo000EonKViP03o`3o0?oVi^Koi^KV00Koi^KV00?o0?l0onKVi_oV
i^H00_oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000?oi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o000@
onKViP05o`3o0?oVi^Koi^KVonKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP03onKViP03o`3o0?oV
i^Koi^KV00Coi^KV00Co0?l0onKVi_oVi^Koi^KV0_l0o`0@onKViP003ooVi^H01Ol0003oi^KVonKV
i_oVi^Koo`0000?oi^KV00Goo`00onKVi_oVi^Koi^KVool0000SonKViP03ool00?oVi^Koo`0000Co
i^KV00Coo`00onKVi_oVi^Ko0?l01?oVi^H00ol0o`3oi^KVo`3o0005onKViP03o`3o0?oVi^Koi^KV
00ooi^KV00Coo`00onKVi_oVi^Koo`001?oVi^H00ooo003oi^KVonKViP06onKViP03ool00?oVi^Ko
i^KV00ooi^KV00?oo`00onKVi_oVi^H05_oVi^H01?oo003oi^KVonKVi_oo000RonKViP03ool00?oV
i^Koi^KV013oi^KV00?oo`00onKVi_oo00005_oVi^H00ol0o`3oi^KVonKViP05onKViP;o0?l00ooV
i^H00ol0o`3oi^KVo`3o0002onKViP03o`3o0?oVi^Koi^KV00;oi^KV00Co0?l0onKVi_oVi^Ko0?l0
4?oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000?oi^KV00Go0?l0onKVi_oVi^Koi^KVo`3o0007onKV
iP06o`3o0?oVi^Koi^KVo`3o0?oVi^Ko0?l04?oVi^H000ooi^KV00Go0000onKVi_oVi^Koi^KVool0
0002onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H08?oVi^H01?oo003oi^KVonKV
i_oo0004onKViP04ool00?oVi^Koi^KVo`3o00Coi^KV00Co0?l0onKVi_oVi^Ko0?l00ooVi^H00ol0
o`3oi^KVonKViP0@onKViP03ool00?oVi^Koo`0000Koi^KV00?oo`00onKVi_oVi^H01?oVi^H00ooo
003oi^KVonKViP0@onKViP03ool00?oVi^Koi^KV01Koi^KV00Goo`00onKVi_oVi^Koi^KVool0000Q
onKViP03ool00?oVi^Koi^KV013oi^KV00?oo`00onKVi_oo00005ooVi^H00ol0o`3oi^KVonKViP04
onKViP03o`3o0?oVi^Ko0?l000;oi^KV00Go0?l0onKVi_l0o`3oi^KVo`3o0005onKViP03o`3o0?oV
i^Ko0?l001;oi^KV00?o0?l0onKVi_l0o`001?oVi^H01Ol0o`3oi^KVonKVi_oVi^Ko0?l000Soi^KV
00?o0?l0onKVi_l0o`000_oVi^H00ol0o`3oi^KVonKViP0=onKViP003ooVi^H01Ol0003oi^KVonKV
i_oVi^Koo`0000;oi^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003oi^KVonKViP0PonKViP04ool0
0?oVi^Koi^KVool000Coi^KV00Coo`00onKVi_oVi^Ko0?l01?oVi^H01?l0o`3oi^KVonKVi_l0o`03
onKViP03o`3o0?oVi^Koi^KV013oi^KV00?oo`00onKVi_oo00001_oVi^H00ooo003oi^KVonKViP04
onKViP03ool00?oVi^Koi^KV013oi^KV00?oo`00onKVi_oVi^H05_oVi^H01Ooo003oi^KVonKVi_oV
i^Koo`00027oi^KV00?oo`00onKVi_oVi^H04?oVi^H00ooo003oi^KVool0000GonKViP03o`3o0?oV
i^Koi^KV00Coi^KV00?o0?l0onKVi_l0o`000_oVi^H01Ol0o`3oi^KVo`3o0?oVi^Ko0?l000Goi^KV
00?o0?l0onKVi_l0o`004_oVi^H00ol0o`3oi^KVo`3o0004onKViP05o`3o0?oVi^Koi^KVonKVi_l0
o`002?oVi^H00ol0o`3oi^KVo`3o0002onKViP03o`3o0?oVi^Koi^KV00goi^KV000?onKViP05o`00
0?oVi^Koi^KVonKVi_oo00000_oVi^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV01oo
i^KV00Goo`00onKVi_oVi^Koi^KVool00003onKViP05ool00?oVi^Koi^KVonKVi_l0o`000_oVi^H0
1Ol0o`3oi^KVonKVi_oVi^Ko0?l000?oi^KV00?o0?l0onKVi_oVi^H04?oVi^H00ooo003oi^KVool0
0006onKViP03ool00?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oVi^H04?oVi^H00ooo003oi^KVonKV
iP0EonKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H07_oVi^H00ooo003oi^KVonKV
iP0BonKViP03ool00?oVi^Koi^KV01Koi^KV00?o0?l0onKVi_oVi^H01?oVi^H00ol0o`3oi^KVo`3o
0002onKViP03o`3o0?oVi^Koi^KV00;o0?l01_oVi^H2o`3o01;oi^KV00?o0?l0onKVi_l0o`001OoV
i^H01?l0o`3oi^KVonKVi_l0o`08onKViP;o0?l00ooVi^H00ol0o`3oi^KVonKViP0=onKViP003ooV
i^H3o`0000;oi^KV00?oo`00onKVi_oo00001OoVi^H4ool000Coi^KV00?oo`00onKVi_oVi^H05_oV
i^H00ooo003oi^KVonKViP02onKViP04ool00?oVi^Koi^KVool000Coi^KV1?l0o`04onKViP?o0?l0
4ooVi^H00ooo003oi^KVool00006onKViP04ool00?oVi^Koi^KVonKViPCoo`004_oVi^H00ooo003o
i^KVonKViP0EonKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H05?oVi^H;ool001Co
i^KV00?oo`00onKVi_oVi^H05_oVi^H4o`3o00;oi^KV00Ko0?l0onKVi_oVi^Ko0?l0onKVi_l0o`03
onKViP03o`3o0?oVi^Koi^KV00Goi^KV00?o0?l0onKVi_oVi^H04OoVi^H00ol0o`3oi^KVo`3o0005
onKViPCo0?l02OoVi^H00ol0o`3oi^KVonKViP02onKViPKo0?l02OoVi^H000ooi^KV00?o0000onKV
i_oVi^H00_oVi^H00ooo003oi^KVool00008onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKV
i_oVi^H05_oVi^H00ooo003oi^KVonKViP02onKViP04ool00?oVi^Koi^KVool002;oi^KV00?oo`00
onKVi_oo00001_oVi^H01Ooo003oi^KVonKVi_oVi^Koo`0001Goi^KV00?oo`00onKVi_oVi^H05OoV
i^H00ooo003oi^KVonKViP02onKViP03ool00?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H0>?oV
i^H01?l0o`3oi^KVonKVi_l0o`02onKViP03o`3o0?oVi^Ko0?l001ooi^KV00?o0?l0onKVi_l0o`00
9_oVi^H000ooi^KV00?o0000onKVi_oVi^H00_oVi^H00ooo003oi^KVool00009onKViP03ool00?oV
i^Koi^KV00;oo`006?oVi^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koo`0002;oi^KV00?o
o`00onKVi_oo00001_oVi^H01Ooo003oi^KVonKVi_oVi^Koo`0001Goi^KV00?oo`00onKVi_oVi^H0
5OoVi^H00ooo003oi^KVonKViP02onKViP03ool00?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H0
>OoVi^H00ol0o`3oi^KVo`3o0002onKViP03o`3o0?oVi^Ko0?l001ooi^KV00?o0?l0onKVi_l0o`00
9_oVi^H000ooi^KV00?o0000onKVi_oVi^H00_oVi^H00ooo003oi^KVool00009onKViP03ool00?oV
i^Koi^KV00;oo`006?oVi^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koo`0002?oi^KV0_oo
0006onKViP05ool00?oVi^Koi^KVonKVi_oo00005OoVi^H00ooo003oi^KVonKViP0EonKViP03ool0
0?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H05?oVi^H00ooo003oi^KVonKViP0ionKViP03o`3o
0?oVi^Ko0?l000;oi^KV00?o0?l0onKVi_l0o`007ooVi^H00ol0o`3oi^KVo`3o000VonKViP003ooV
i^H00ol0003oi^KVonKViP02onKViP;oo`002_oVi^H01Ooo003oi^KVool00?oVi^Koo`0001Soi^KV
00?oo`00onKVi_oVi^H01?oVi^H2ool002?oi^KV00?oo`00onKVi_oVi^H01_oVi^H00ooo003oi^KV
ool0000FonKViP03ool00?oVi^Koi^KV01Goi^KV00?oo`00onKVi_oVi^H00ooVi^H00ooo003oi^KV
onKViP0BonKViP03ool00?oVi^Koi^KV03[oi^KV00?o0?l0onKVi_l0o`000ooVi^H2o`3o023oi^KV
00?o0?l0onKVi_oVi^H09OoVi^H000ooi^KV00?o0000onKVi_oVi^H00ooVi^H00ooo003oi^KVonKV
iP09onKViP;oo`0000?oi^KVool00?oVi^H05ooVi^H00ooo003oi^KVonKViP04onKViP;oo`008ooV
i^H00ooo003oi^KVonKViP06onKViP03ool00?oVi^Koo`0001Koi^KV00?oo`00onKVi_oVi^H05OoV
i^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV01;oi^KV00?oo`00onKVi_oVi^H0>ooV
i^H00ol0o`3oi^KVonKViP02onKViP03o`3o0?oVi^Koi^KV01ooi^KV00?o0?l0onKVi_oVi^H09OoV
i^H000ooi^KV0ol00003onKViP03ool00?oVi^Koi^KV00Woi^KV00Coo`00onKVi_oVi^Koo`006?oV
i^H00ooo003oi^KVonKViP05onKViP03ool00?oVi^Koi^KV027oi^KV00?oo`00onKVi_oVi^H01_oV
i^H00ooo003oi^KVool0000FonKViP03ool00?oVi^Koi^KV00?oi^KV1?oo000=onKViP03ool00?oV
i^Koi^KV00Coi^KV00?oo`00onKVi_oVi^H03_oVi^H01Ooo003oi^KVonKVi_oVi^Koo`0003goi^KV
00?o0?l0onKVi_oVi^H00_oVi^H00ol0o`3oi^KVonKViP0OonKViP03o`3o0?oVi^Koi^KV02Goi^KV
000?onKViP03o`000?oVi^Koi^KV01?oi^KV00?oo`00onKVi_oVi^H05OoVi^H00ooo003oi^KVonKV
iP0bonKViP03ool00?oVi^Koo`0001Koi^KV00?oo`00onKVi_oVi^H00ooVi^H01?oo003oi^KVonKV
i_oo000=onKViP03ool00?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oVi^H03_oVi^H01Ooo003oi^KV
onKVi_oVi^Koo`0008coi^KV000?onKViP03o`000?oVi^Koi^KV01?oi^KV00?oo`00onKVi_oVi^H0
5OoVi^H00ooo003oi^KVonKViP0bonKViP03ool00?oVi^Koo`0001Koi^KV00?oo`00onKVi_oVi^H0
0ooVi^H01?oo003oi^KVonKVi_oo000=onKViP03ool00?oVi^Koi^KV00Coi^KV00?oo`00onKVi_oV
i^H03_oVi^H2ool000;oi^KV00?oo`00onKVi_oVi^H0R_oVi^H000ooi^KV00?o0000onKVi_oVi^H0
4ooVi^H00ooo003oi^KVonKViP0EonKViP03ool00?oVi^Koi^KV03;oi^KV00?oo`00onKVi_oo0000
5_oVi^H00ooo003oi^KVonKViP03onKViP04ool00?oVi^Koi^KVool000goi^KV00?oo`00onKVi_oV
i^H01?oVi^H00ooo003oi^KVonKViP0>onKViP;oo`000_oVi^H00ooo003oi^KVonKViP2:onKViP00
00Coi^KVo`000?l0003o00000ooVi^H3o`0000Goi^KV00?o0000onKVi_oVi^H04ooVi^H00ooo003o
i^KVonKViP0EonKViP03ool00?oVi^Koi^KV03;oi^KV00?oo`00onKVi_oo00005_oVi^H00ooo003o
i^KVonKViP03onKViP04ool00?oVi^Koi^KVool000goi^KV00?oo`00onKVi_oVi^H01?oVi^H00ooo
003oi^KVonKViP0>onKViP05ool00?oVi^Koo`00onKVi_oo0000S?oVi^H00007o`000?oVi^Koi^KV
onKVi_l0003oi^KVo`000003onKViP03o`000?oVi^Koi^KV00;oi^KV00?o0000onKVi_oVi^H04ooV
i^H00ooo003oi^KVonKViP0DonKViP03ool00?oVi^Koi^KV03Coi^KV00?oo`00onKVi_oVi^H05OoV
i^H00ooo003oi^KVonKViP03onKViP04ool00?oVi^Koi^KVool000goi^KV00?oo`00onKVi_oVi^H0
1OoVi^H00ooo003oi^KVonKViP0=onKViP04ool00?oVi^Koo`00ool008goi^KV0004onKViP03o`00
0?oVi^Koi^KV00?oi^KV00?o0000onKVi_oVi^H00_oVi^H00ol0003oi^KVonKViP0ConKViP03ool0
0?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H0=?oVi^H00ooo003oi^KVonKViP0FonKViP03ool0
0?oVi^Koi^KV00;oi^KV00Goo`00onKVi_oVi^Koi^KVool0000<onKViP03ool00?oVi^Koi^KV00Go
i^KV00?oo`00onKVi_oVi^H03?oVi^H00ooo003oi^KVonKViP02ool008goi^KV0004onKViP03o`00
0?oVi^Koi^KV00?oi^KV00?o0000onKVi_oVi^H00_oVi^H3o`0001?oi^KV00?oo`00onKVi_oVi^H0
5?oVi^H00ooo003oi^KVonKViP0donKViP03ool00?oVi^Koi^KV01Koi^KV00?oo`00onKVi_oVi^H0
0_oVi^H01Ooo003oi^KVonKVi_oVi^Koo`0000?oi^KV00?oo`00onKVi_oVi^H01_oVi^H00ooo003o
i^KVonKViP05onKViP03ool00?oVi^Koi^KV00coi^KV00Goo`00onKVi_oVi^Koi^KVool0002=onKV
iP001?l00002onKViPCo00001OoVi^H00ol0003oi^KVonKViP0ConKViP03ool00?oVi^Koi^KV01Co
i^KV00?oo`00onKVi_oVi^H0COoVi^H01Ooo003oi^KVonKVi_oVi^Koo`0000Coi^KV00Goo`00onKV
i_oVi^Koi^KVool00008onKViP03ool00?oVi^Koi^KV00Goi^KV00?oo`00onKVi_oVi^H03?oVi^H0
0ooo003oi^KVonKViP2?onKViP0000?o0000onKVi_oVi^H00ooVi^H00ol0003oi^KVonKViP06onKV
iP03o`000?oVi^Koi^KV01?oi^KV00?oo`00onKVi_oVi^H05?oVi^H00ooo003oi^KVonKViP1=onKV
iP05ool00?oVi^Koi^KVonKVi_oo00001?oVi^H01?oo003oi^KVonKVi_oVi^H2ool000Ooi^KV00?o
o`00onKVi_oVi^H01OoVi^H00ooo003oi^KVonKViP0<onKViP03ool00?oVi^Koi^KV08ooi^KV0005
o`000003onKVi_l0003o000000?o00001?oVi^H00ol0003oi^KVonKViP0ConKViP03ool00?oVi^Ko
i^KV01Coi^KV00?oo`00onKVi_oVi^H0COoVi^H01Ooo003oi^KVonKVi_oVi^Koo`0000Coi^KV00Ko
o`00onKVi_oVi^Koo`00onKVi_oo0006onKViP03ool00?oVi^Koi^KV00Koi^KV00?oo`00onKVi_oV
i^H03?oVi^H00ooo003oi^KVonKViP2?onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03ool0
0?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H0COoVi^H01Ooo003oi^KVonKVi_oVi^Koo`0000Co
i^KV00Coo`00onKVi_oVi^Koo`000_oVi^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV
00Koi^KV00?oo`00onKVi_oVi^H03?oVi^H00ooo003oi^KVonKViP2?onKViP003ooVi^H00ol0003o
i^KVonKViP0ConKViP03ool00?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H0COoVi^H01Ooo003o
i^KVonKVi_oVi^Koo`0000Coi^KV00Coo`00onKVi_oVi^Koo`000_oVi^H00ooo003oi^KVonKViP03
onKViP03ool00?oVi^Koi^KV00Ooi^KV00?oo`00onKVi_oVi^H02ooVi^H00ooo003oi^KVonKViP2?
onKViP003ooVi^H00ol0003oi^KVonKViP0ConKViP03ool00?oVi^Koi^KV01Coi^KV00?oo`00onKV
i_oVi^H0COoVi^H01Ooo003oi^KVonKVi_oVi^Koo`0000Coi^KV00Coo`00onKVi_oVi^Koo`000_oV
i^H00ooo003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV00Ooi^KV00?oo`00onKVi_oVi^H02ooV
i^H00ooo003oi^KVonKViP2?onKViP003ooVi^H3o`0001Coi^KV00?oo`00onKVi_oVi^H04ooVi^H0
0ooo003oi^KVonKViP1=onKViP05ool00?oVi^Koi^KVonKVi_oo00001?oVi^H00ooo003oi^KVool0
0004onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H01ooVi^H00ooo003oi^KVonKV
iP0;onKViP03ool00?oVi^Koi^KV08ooi^KV000?onKViP03o`000?oVi^Koi^KV01Coi^KV00?oo`00
onKVi_oVi^H04ooVi^H00ooo003oi^KVonKViP1=onKViP05ool00?oVi^Koi^KVonKVi_oo00001?oV
i^H00ooo003oi^KVool00004onKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H01ooV
i^H00ooo003oi^KVonKViP0;onKViP03ool00?oVi^Koi^KV08ooi^KV000?onKViP03o`000?oVi^Ko
i^KV01Coi^KV00?oo`00onKVi_oVi^H04ooVi^H00ooo003oi^KVonKViP1=onKViP05ool00?oVi^Ko
i^KVonKVi_oo00001OoVi^H2ool000Goi^KV00Coo`00onKVi_oVi^Koo`002_oVi^H00ooo003oi^KV
onKViP0:onKViP03ool00?oVi^Koi^KV093oi^KV000?onKViP03o`000?oVi^Koi^KV01Coi^KV00?o
o`00onKVi_oVi^H04ooVi^H00ooo003oi^KVonKViP1>onKViP04ool00?oVi^Koi^KVool000Goi^KV
0_oo0005onKViP04ool00?oVi^Koi^KVool000[oi^KV00?oo`00onKVi_oVi^H02_oVi^H00ooo003o
i^KVonKViP2@onKViP003ooVi^H00ol0003oi^KVonKViP0DonKViP03ool00?oVi^Koi^KV01?oi^KV
00?oo`00onKVi_oVi^H0C_oVi^H01?oo003oi^KVonKVi_oo0005onKViP;oo`001OoVi^H00ooo003o
i^KVool0000<onKViP03ool00?oVi^Koi^KV00Woi^KV00?oo`00onKVi_oVi^H0T?oVi^H000ooi^KV
00?o0000onKVi_oVi^H05?oVi^H00ooo003oi^KVonKViP0ConKViP03ool00?oVi^Koi^KV04koi^KV
00Coo`00onKVi_oVi^Koo`001OoVi^H00ooo003oi^KVonKViP05onKViP;oo`003?oVi^H00ooo003o
i^KVonKViP09onKViP03ool00?oVi^Koi^KV093oi^KV000?onKViP?o00005?oVi^H00ooo003oi^KV
onKViP0ConKViP03ool00?oVi^Koi^KV04koi^KV00Coo`00onKVi_oVi^Koo`001OoVi^H00ooo003o
i^KVonKViP05onKViP03ool00?oVi^Koi^KV00_oi^KV00?oo`00onKVi_oVi^H00ooVi^H00ooo003o
i^KVonKViP03onKViP03ool00?oVi^Koi^KV093oi^KV000?onKViP03o`000?oVi^Koi^KV01Coi^KV
00?oo`00onKVi_oVi^H04ooVi^H00ooo003oi^KVonKViP1?onKViP;oo`007?oVi^H00ooo003oi^KV
onKViP03onKViP03ool00?oVi^Koi^KV00?oi^KV00?oo`00onKVi_oVi^H0T?oVi^H000ooi^KV00?o
0000onKVi_oVi^H05?oVi^H00ooo003oi^KVonKViP0ConKViP03ool00?oVi^Koi^KV04ooi^KV0_oo
000LonKViP03ool00?oVi^Koi^KV00?oi^KV0_oo0003onKViP03ool00?oVi^Koi^KV097oi^KV000?
onKViP03o`000?oVi^Koi^KV01Coi^KV00?oo`00onKVi_oVi^H04ooVi^H00ooo003oi^KVonKViP1?
onKViP;oo`007?oVi^H00ooo003oi^KVonKViP02onKViP03ool00?oVi^Koo`0000?oi^KV00?oo`00
onKVi_oVi^H0TOoVi^H000ooi^KV00?o0000onKVi_oVi^H05?oVi^H00ooo003oi^KVonKViP0ConKV
iP03ool00?oVi^Koi^KV04ooi^KV0_oo000LonKViP03ool00?oVi^Koi^KV00;oi^KV00Koo`00onKV
i_oVi^Koo`00onKVi_oo002DonKViP003ooVi^H00ol0003oi^KVonKViP0EonKViP03ool00?oVi^Ko
i^KV017oi^KV00?oo`00onKVi_oVi^H0D?oVi^H2ool001goi^KV00Goo`00onKVi_oVi^Koi^KVool0
0002onKViP03ool00?oVi^Koo`0009Coi^KV000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?oo`00
onKVi_oVi^H04OoVi^H00ooo003oi^KVonKViP1AonKViP03ool00?oVi^Koi^KV01_oi^KV00Coo`00
onKVi_oVi^Koo`000ooVi^H00ooo003oi^KVool0002DonKViP003ooVi^H3o`0001Goi^KV00?oo`00
onKVi_oVi^H04OoVi^H00ooo003oi^KVonKViP1AonKViP03ool00?oVi^Koi^KV01_oi^KV1?oo0004
onKViP03ool00?oVi^Koi^KV09?oi^KV000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?oo`00onKV
i_oVi^H04OoVi^H00ooo003oi^KVonKViP3oonKViPkoi^KV000?onKViP03o`000?oVi^Koi^KV01Go
i^KV00?oo`00onKVi_oVi^H04OoVi^H00ooo003oi^KVonKViP3oonKViPkoi^KV000?onKViP03o`00
0?oVi^Koi^KV01Goi^KV00?oo`00onKVi_oVi^H04OoVi^H00ooo003oi^KVonKViP3oonKViPkoi^KV
000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?oo`00onKVi_oVi^H04OoVi^H00ooo003oi^KVonKV
iP3oonKViPkoi^KV000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?oo`00onKVi_oVi^H04OoVi^H0
0ooo003oi^KVonKViP3oonKViPkoi^KV000?onKViP03o`000?oVi^Koi^KV01Goi^KV00?oo`00onKV
i_oVi^H04OoVi^H00ooo003oi^KVonKViP3oonKViPkoi^KV000?onKViP?o00005_oVi^H00ooo003o
i^KVonKViP0@onKViP03ool00?oVi^Koi^KV0?ooi^KV3_oVi^H000ooi^KV00?o0000onKVi_oVi^H0
5_oVi^H00ooo003oi^KVonKViP0@onKViP03ool00?oVi^Koi^KV0?ooi^KV3_oVi^H000ooi^KV00?o
0000onKVi_oVi^H05_oVi^H00ooo003oi^KVonKViP0@onKViP03ool00?oVi^Koi^KV0?ooi^KV3_oV
i^H000ooi^KV00?o0000onKVi_oVi^H05_oVi^H00ooo003oi^KVonKViP0?onKViP03ool00?oVi^Ko
i^KV0?ooi^KV3ooVi^H00004onKVi_l0003o0000o`0000Coi^KV0_l00005onKViP03o`000?oVi^Ko
i^KV01Koi^KV00?oo`00onKVi_oVi^H03ooVi^H00ooo003oi^KVonKViP3oonKViPooi^KV00001Ol0
003oi^KVonKVi_oVi^Ko000000;oi^KV00Co0000onKVi_oVi^Ko00001?oVi^H00ol0003oi^KVonKV
iP0FonKViP03ool00?oVi^Koi^KV00ooi^KV00?oo`00onKVi_oVi^H0oooVi^H?onKViP0000Go0000
onKVi_oVi^Koi^KVo`000002onKViP04o`000?oVi^Koi^KVo`0000Coi^KV0ol0000FonKViP03ool0
0?oVi^Koi^KV00ooi^KV00?oo`00onKVi_oVi^H0oooVi^H?onKViP001?l00003onKViP04o`000?oV
i^Koi^KVo`0000Coi^KV00?o0000onKVi_oVi^H05_oVi^H00ooo003oi^KVonKViP0>onKViP03ool0
0?oVi^Koi^KV0?ooi^KV4?oVi^H00003o`000?oVi^Koi^KV00Coi^KV00Co0000onKVi_oVi^Ko0000
1?oVi^H00ol0003oi^KVonKViP0FonKViP03ool00?oVi^Koi^KV00koi^KV00?oo`00onKVi_oVi^H0
oooVi^H@onKViP0000?oi^KVo`000?oVi^H01?oVi^H01?l0003oi^KVonKVi_l00004onKViP03o`00
0?oVi^Koi^KV01Ooi^KV00?oo`00onKVi_oVi^H03OoVi^H00ooo003oi^KVonKViP3oonKViQ3oi^KV
0002onKViP;o00001?oVi^H2o`0000Goi^KV00?o0000onKVi_oVi^H05ooVi^H00ooo003oi^KVonKV
iP0=onKViP03ool00?oVi^Koi^KV0?ooi^KV4?oVi^H000ooi^KV00?o0000onKVi_oVi^H05ooVi^H0
0ooo003oi^KVonKViP0=onKViP03ool00?oVi^Koi^KV0?ooi^KV4?oVi^H000ooi^KV00?o0000onKV
i_oVi^H05ooVi^H00ooo003oi^KVonKViP0<onKViP03ool00?oVi^Koi^KV0?ooi^KV4OoVi^H000oo
i^KV0ol0000GonKViP03ool00?oVi^Koi^KV00Ooi^KV00?oo`00onKVi_oVi^H00_oVi^H00ooo003o
i^KVonKViP3oonKViQ7oi^KV000?onKViP03o`000?oVi^Koi^KV01Ooi^KV00?oo`00onKVi_oVi^H0
1ooVi^H00ooo003oi^KVonKViP02onKViP03ool00?oVi^Koi^KV0?ooi^KV4OoVi^H000ooi^KV00?o
0000onKVi_oVi^H05ooVi^H00ooo003oi^KVonKViP07onKViP;oo`000ooVi^H00ooo003oi^KVonKV
iP3oonKViQ7oi^KV000?onKViP03o`000?oVi^Koi^KV01Soi^KV00?oo`00onKVi_oVi^H01_oVi^H2
ool000;oi^KV00?oo`00onKVi_oVi^H0oooVi^HBonKViP003ooVi^H00ol0003oi^KVonKViP0HonKV
iP03ool00?oVi^Koi^KV00Koi^KV00Goo`00onKVi_oo003oi^KVool0003oonKViQCoi^KV000?onKV
iP03o`000?oVi^Koi^KV01Soi^KV00?oo`00onKVi_oVi^H01_oVi^H01Ooo003oi^KVool00?oVi^Ko
o`000?ooi^KV5?oVi^H000ooi^KV00?o0000onKVi_oVi^H06OoVi^H00ooo003oi^KVonKViP04onKV
iP03ool00?oVi^Koi^KV00;oo`00oooVi^HEonKViP003ooVi^H3o`0001Woi^KV00?oo`00onKVi_oV
i^H01?oVi^H01Ooo003oi^KVonKVi_oVi^Koo`000?ooi^KV5OoVi^H000ooi^KV00?o0000onKVi_oV
i^H06OoVi^H00ooo003oi^KVonKViP04onKViP03ool00?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV
00?o0000onKVi_oVi^H06OoVi^H00ooo003oi^KVonKViP04onKViP03ool00?oVi^Koi^KV0?ooi^KV
5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06_oVi^H00ooo003oi^KVonKViP03onKViP03ool00?oV
i^Koi^KV0?ooi^KV5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06_oVi^H00ooo003oi^KVonKViP03
onKViP03ool00?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV00?o0000onKVi_oVi^H06_oVi^H00ooo
003oi^KVonKViP03onKViP03ool00?oVi^Koi^KV0?ooi^KV5ooVi^H000ooi^KV0ol0000JonKViP03
ool00?oVi^Koi^KV00?oi^KV00?oo`00onKVi_oVi^H0oooVi^HGonKViP003ooVi^H00ol0003oi^KV
onKViP0KonKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H0oooVi^HGonKViP003ooV
i^H00ol0003oi^KVonKViP0KonKViP03ool00?oVi^Koi^KV00;oi^KV00?oo`00onKVi_oVi^H0oooV
i^HGonKViP003ooVi^H00ol0003oi^KVonKViP0KonKViP05ool00?oVi^Koi^KVonKVi_oo0000oooV
i^HJonKViP003ooVi^H00ol0003oi^KVonKViP0KonKViP05ool00?oVi^Koi^KVonKVi_oo0000oooV
i^HJonKViP003ooVi^H00ol0003oi^KVonKViP0KonKViP05ool00?oVi^Koi^KVonKVi_oo0000oooV
i^HJonKViP003ooVi^H00ol0003oi^KVonKViP0LonKViP04ool00?oVi^Koi^KVool00?ooi^KV6_oV
i^H000ooi^KV0ol0000LonKViPCoo`00oooVi^HJonKViP003ooVi^H00ol0003oi^KVonKViP3oonKV
iS[oi^KV000?onKViP03o`000?oVi^Koi^KV0?ooi^KV>_oVi^H000ooi^KV00?o0000onKVi_oVi^H0
oooVi^HjonKViP003ooVi^H00ol0003oi^KVonKViP3oonKViS[oi^KV000?onKViP03o`000?oVi^Ko
i^KV0?ooi^KV>_oVi^H00?ooi^KVC?oVi^H00?ooi^KVC?oVi^H00?ooi^KVC?oVi^H00001\
\>"],
  ImageRangeCache->{{{0, 330.562}, {203.938, 0}} -> {-6.27432, 34.7552, \
0.392692, 0.149503}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Adding a random element to the interaction can make it less likely \
that a strategy with an initial advantage in terms of overall numbers will be \
able to dominate. This model only had probabilistic interactions - the update \
neighborhood was always deterministic.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[bos5]\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Defining custom learning rules", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell["Simple learning rules", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "It is possible to define custom learning rules within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and tell some (or all) agents to use these new learning rules instead of \
",
  StyleBox["Imitate the best",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Custom learning rules are created by defining a function. The \
function must take one argument, a reference to the agent for whom the \
learning rule is being invoked. Any relevant information can be obtained from \
this reference by calling the appropriate function.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Let ",
  StyleBox["agent",
    FontSlant->"Italic"],
  " be a reference to an agent. Functions for retrieving information include"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[GridBox[{
        {
          StyleBox[
            RowBox[{"Score", "[", 
              StyleBox["agent",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Italic"], "]"}],
            GridBoxOptions->{ColumnAlignments->{Left}}], 
          StyleBox[
            RowBox[{
              StyleBox["The",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox["score",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox["of",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"], 
              StyleBox["agent",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Italic"], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain"]}],
            GridBoxOptions->{ColumnAlignments->{Left}}]},
        {
          StyleBox[
            RowBox[{"Strategy", "[", 
              StyleBox["agent",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Italic"], "]"}],
            GridBoxOptions->{ColumnAlignments->{Left}}], 
          RowBox[{
            StyleBox["The",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["current",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["strategy",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["of",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["agent",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Italic",
              GridBoxOptions->{ColumnAlignments->{Left}}]}]},
        {
          RowBox[{
            StyleBox["InteractionNeighbors",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["[",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["agent",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Italic",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["]",
              GridBoxOptions->{ColumnAlignments->{Left}}]}], 
          RowBox[{
            StyleBox["The",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["members",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["of",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            RowBox[{
              StyleBox["agent",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Italic",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["'",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}]}], 
            StyleBox["s",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["interaction",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["neighborhood",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}]}]},
        {
          RowBox[{
            StyleBox[
              RowBox[{"Update", 
                StyleBox["Neighbors",
                  GridBoxOptions->{ColumnAlignments->{Left}}]}]], 
            StyleBox["[",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["agent",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Italic",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["]",
              GridBoxOptions->{ColumnAlignments->{Left}}]}], 
          RowBox[{
            RowBox[{
              StyleBox["The",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["members",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["of",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              RowBox[{
                StyleBox["agent",
                  "Text",
                  FontFamily->"Times",
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["'",
                  "Text",
                  FontFamily->"Times",
                  FontWeight->"Plain",
                  FontSlant->"Plain",
                  GridBoxOptions->{ColumnAlignments->{Left}}]}], 
              StyleBox["s",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["update",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["neighborhood",
                "Text",
                FontFamily->"Times",
                FontWeight->"Plain",
                FontSlant->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}]}], 
            StyleBox[" ",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[IndentingNewLine]",
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[\((\(with\ some\ caveates ... \) see\ below)\),
              "Text",
              FontFamily->"Times",
              FontWeight->"Plain",
              FontSlant->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}]}]}
        }]], "Input",
  TextAlignment->Left,
  GridBoxOptions->{ColumnAlignments->{Left}},
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "By way of illustration, here is how one would define ",
  StyleBox["Imitate the Best",
    FontSlant->"Italic"],
  " within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "First, let's define a helper function that takes an agent ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  " and constructs a list of ordered pairs of the form ",
  Cell[BoxData[
      \(TraditionalForm\`{{s\_1, \[Sigma]\_1}, {s\_2, \[Sigma]\_2},  ... , {s\
\_n, \[Sigma]\_n}}\)]],
  ", for all ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " agents in ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  "'s update neighborhood. In this list, ",
  Cell[BoxData[
      \(TraditionalForm\`s\_i\)]],
  " denotes the strategy of agent ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_i\)]],
  " denotes the strategy of agent ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(CreatePairs[agent_] := 
        Module[{nbrs, pairs}, \[IndentingNewLine]nbrs = 
            UpdateNeighbors[agent]; \[IndentingNewLine]pairs = 
            Map[\[IndentingNewLine]Function[i, {Score[i], Strategy[i]}], 
              nbrs\[IndentingNewLine]]; \[IndentingNewLine]Return[
            pairs];\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "In order for the structure of this to be clear, you may need to read up on \
the use of ",
  StyleBox["Map", "Input"],
  " and anonymous functions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Given this, defining ",
  StyleBox["Imitate the best",
    FontSlant->"Italic"],
  " is straightforward. Comments are included."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{\(ImitateTheBest[agent_]\), ":=", "\[IndentingNewLine]", 
        RowBox[{"Module", "[", "\[IndentingNewLine]", 
          RowBox[{\({pairs, max, top, newStrategy}\), ",", 
            StyleBox[\( (*local\ variables*) \),
              FontColor->RGBColor[0, 0, 1]], 
            StyleBox["\[IndentingNewLine]",
              FontColor->RGBColor[0, 0, 1]], 
            StyleBox[\( (*Form\ pairs\ using\ the\ helper\ function*) \),
              FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
            
            RowBox[{\(pairs = CreatePairs[agent]\), ";", 
              "\[IndentingNewLine]", 
              
              StyleBox[\( (*Find\ the\ maximum\ score\ among\ update\ \
neighbors*) \),
                FontColor->RGBColor[0, 0, 1]], 
              "\[IndentingNewLine]", \(max = Max[Map[First, pairs]]\), ";", 
              "\[IndentingNewLine]", 
              
              StyleBox[\( (*\[IndentingNewLine]\(Check\ if\ max\ is\ greater\ \
than\ current\ score ... \)\ i . e . , 
                  only\ update\ if\ \(\(dissatisfied\)\(.\)\)\
\[IndentingNewLine]*) \),
                FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
                RowBox[{\(max > Score[agent]\), ",", "\[IndentingNewLine]", 
                  RowBox[{"Module", "[", 
                    RowBox[{\({}\), ",", "\[IndentingNewLine]", 
                      
                      StyleBox[\( (*Pick\ out\ the\ maximally\ scoring\ \
strategies*) \),
                        FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
                      
                      
                      RowBox[{\(top = 
                          Select[pairs, \((First[#] \[Equal] max)\) &]\), ";",
                         "\[IndentingNewLine]", 
                        
                        StyleBox[\( (*\[IndentingNewLine]Choose\ a\ strategy\ \
at\ random . \ 
                                The\ Flatten\ and\ Last\ are\ needed\ to\ \
remove\ unnecessary\ {}' 
                              s\ and\ to\ extract\ an\ element\ from\ a\ set\
\[IndentingNewLine]*) \),
                          FontColor->RGBColor[0, 0, 1]], 
                        "\[IndentingNewLine]", \(newStrategy = 
                          Flatten[RandomKSubset[top, 1]] // Last\), ";", 
                        "\[IndentingNewLine]", 
                        
                        StyleBox[\( (*\[IndentingNewLine]Now\ set\ the\ next\ \
strategy\ of\ the\ agent\ to\ \(\(newStrategy\)\(.\)\)\[IndentingNewLine]*) \
\),
                          FontColor->RGBColor[0, 0, 1]], 
                        "\[IndentingNewLine]", \(SetNextStrategy[agent, 
                          newStrategy]\), ";"}]}], "\[IndentingNewLine]", 
                    "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
          "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(max\\)\\\" is similar to existing symbol \
\\\"\\!\\(Max\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(top\\)\\\" is similar to existing symbol \
\\\"\\!\\(Top\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[6]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
A different learning rule adopts a strategy at random whenever the \
agent earns less than the maximally scoring strategy. Since we need to know \
the total number of strategies possible, let's define it for the Prisoner's \
Dilemma.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(RandomAdoptionWhenDissatisfied[agent_]\), ":=", 
        RowBox[{"Module", "[", 
          RowBox[{\({pairs, max, top, newS}\), ",", "\[IndentingNewLine]", 
            
            RowBox[{\(pairs = CreatePairs[agent]\), ";", 
              "\[IndentingNewLine]", \(max = Max[Map[First, pairs]]\), ";", 
              StyleBox[\( (*Find\ max\ score*) \),
                FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
                RowBox[{\(max > Score[agent]\), ",", 
                  StyleBox[\( (*If\ I' m\ dissatisfied ... *) \),
                    FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
                  RowBox[{"If", "[", 
                    
                    RowBox[{\(Random[] <  .5\), ",", 
                      "\[IndentingNewLine]", \(SetNextStrategy[agent, 1]\), 
                      ",", 
                      StyleBox[\( (*Cooperate*) \),
                        FontColor->RGBColor[0, 0, 1]], 
                      StyleBox["\[IndentingNewLine]",
                        FontColor->RGBColor[0, 0, 1]], \(SetNextStrategy[
                        agent, 2]\)}], " ", 
                    StyleBox[\( (*Defect*) \),
                      FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
                    "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
          "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[7]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Let's try this out for the Prisoner's Dilemma on a ring, with \
Interaction and Update neighborhoods wired up so that Cooperate normally \
wins.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(StrategyLegend[{LightGray, 
          Black}, {"\<Cooperate\>", "\<Defect\>"}];\)\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.666667],
                Background->GrayLevel[0.666667]],
              BoxMargins->False], "Cooperate"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0],
                Background->GrayLevel[0]],
              BoxMargins->False], "Defect"}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[8]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"pd5", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] Cycle[200]\), ",", 
            "\[IndentingNewLine]", \(UpdateNeighborhood \[Rule] 
              MakeSimple[GraphPower[Cycle[200], 2]]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({2.5, 2.5}\), \({0, 3}\)},
                    {\({3, 0}\), \({1, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {LightGray, 
                Black}\), ",", 
            "\[IndentingNewLine]", \(NeighborhoodsToDraw \[Rule] \
{Interaction}\)}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[9]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[pd5], 
        VertexStyle \[Rule] Disk[Small]];\)\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.663 .663 .663 r
.012 w
.95432 .51428 Mdot
0 0 0 r
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.663 .663 .663 r
.94895 .57111 Mdot
0 0 0 r
.94649 .58517 Mdot
.663 .663 .663 r
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
0 0 0 r
.9323 .64046 Mdot
.663 .663 .663 r
.92767 .65397 Mdot
0 0 0 r
.92263 .66733 Mdot
.663 .663 .663 r
.91716 .68052 Mdot
.91129 .69354 Mdot
0 0 0 r
.905 .70636 Mdot
.89832 .71898 Mdot
.663 .663 .663 r
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
0 0 0 r
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.663 .663 .663 r
.83135 .81116 Mdot
0 0 0 r
.82141 .82141 Mdot
.663 .663 .663 r
.81116 .83135 Mdot
.8006 .84096 Mdot
0 0 0 r
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.663 .663 .663 r
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
0 0 0 r
.69354 .91129 Mdot
.68052 .91716 Mdot
.663 .663 .663 r
.66733 .92263 Mdot
.65397 .92767 Mdot
0 0 0 r
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.663 .663 .663 r
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
0 0 0 r
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.663 .663 .663 r
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
0 0 0 r
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.663 .663 .663 r
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
0 0 0 r
.34603 .92767 Mdot
.663 .663 .663 r
.33267 .92263 Mdot
0 0 0 r
.31948 .91716 Mdot
.30646 .91129 Mdot
.663 .663 .663 r
.29364 .905 Mdot
0 0 0 r
.28102 .89832 Mdot
.663 .663 .663 r
.26862 .89125 Mdot
.25644 .88379 Mdot
0 0 0 r
.24451 .87595 Mdot
.663 .663 .663 r
.23282 .86773 Mdot
0 0 0 r
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.663 .663 .663 r
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
0 0 0 r
.14977 .78974 Mdot
.14084 .77859 Mdot
.663 .663 .663 r
.13227 .76718 Mdot
.12405 .75549 Mdot
0 0 0 r
.11621 .74356 Mdot
.663 .663 .663 r
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
0 0 0 r
.08284 .68052 Mdot
.663 .663 .663 r
.07737 .66733 Mdot
0 0 0 r
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.663 .663 .663 r
.05973 .61304 Mdot
0 0 0 r
.0564 .59916 Mdot
.663 .663 .663 r
.05351 .58517 Mdot
0 0 0 r
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.663 .663 .663 r
.04568 .51428 Mdot
0 0 0 r
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.663 .663 .663 r
.05105 .42889 Mdot
0 0 0 r
.05351 .41483 Mdot
.663 .663 .663 r
.0564 .40084 Mdot
0 0 0 r
.05973 .38696 Mdot
.663 .663 .663 r
.0635 .37319 Mdot
.0677 .35954 Mdot
0 0 0 r
.07233 .34603 Mdot
.07737 .33267 Mdot
.663 .663 .663 r
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
0 0 0 r
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.663 .663 .663 r
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
0 0 0 r
.14977 .21026 Mdot
.663 .663 .663 r
.15904 .1994 Mdot
0 0 0 r
.16865 .18884 Mdot
.17859 .17859 Mdot
.663 .663 .663 r
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
0 0 0 r
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.663 .663 .663 r
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
0 0 0 r
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.663 .663 .663 r
.35954 .0677 Mdot
0 0 0 r
.37319 .0635 Mdot
.38696 .05973 Mdot
.663 .663 .663 r
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
0 0 0 r
.48572 .04568 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.51428 .04568 Mdot
0 0 0 r
.52854 .04635 Mdot
.54278 .04747 Mdot
.663 .663 .663 r
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
0 0 0 r
.59916 .0564 Mdot
.663 .663 .663 r
.61304 .05973 Mdot
0 0 0 r
.62681 .0635 Mdot
.64046 .0677 Mdot
.663 .663 .663 r
.65397 .07233 Mdot
.66733 .07737 Mdot
0 0 0 r
.68052 .08284 Mdot
.663 .663 .663 r
.69354 .08871 Mdot
.70636 .095 Mdot
0 0 0 r
.71898 .10168 Mdot
.73138 .10875 Mdot
.663 .663 .663 r
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
0 0 0 r
.78974 .14977 Mdot
.663 .663 .663 r
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
0 0 0 r
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.663 .663 .663 r
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
0 0 0 r
.89125 .26862 Mdot
.89832 .28102 Mdot
.663 .663 .663 r
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
0 0 0 r
.92263 .33267 Mdot
.92767 .34603 Mdot
.663 .663 .663 r
.9323 .35954 Mdot
0 0 0 r
.9365 .37319 Mdot
.94027 .38696 Mdot
.663 .663 .663 r
.9436 .40084 Mdot
0 0 0 r
.94649 .41483 Mdot
.94895 .42889 Mdot
.663 .663 .663 r
.95096 .44303 Mdot
0 0 0 r
.95253 .45722 Mdot
.663 .663 .663 r
.95365 .47146 Mdot
0 0 0 r
.95432 .48572 Mdot
.663 .663 .663 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[10]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00P_ooool3ojVYZ@05oooooonYZJWoZJVYojVYZOoo
ool00ol000001OooooooZJVYojVYZOnYZJWooooo00?oZJVY00Goooooo`000?l0003o0000ooooo`03
o`0008?ooooo001iooooo`?oZJVY00KoooooojVYZOnYZJWoZJVYooooool00003ojVYZ@04o`000?nY
ZJWoZJVYojVYZ@Go00000onYZJT01?l0003oZJVYojVYZOnYZJT9o`0000?oZJVY00CoooooojVYZOnY
ZJWoZJVYNoooool007Gooooo0onYZJT01OooooooZJVYojVYZOnYZJWo000000?oZJVY00?o0000oooo
oonYZJT00_nYZJT01OooooooZJVYojVYZOnYZJWooooo00?o000000GoooooojVYZOnYZJWoZJVYoooo
o`03ojVYZ@05ooooool0003o0000o`000?ooool00ol000001OooooooZJVYojVYZOnYZJWo000000?o
ZJVY00Co0000ojVYZOnYZJWoZJVYMoooool0077ooooo0onYZJT01OooooooZJVYojVYZOnYZJWo0000
00?oZJVY00CoooooojVYZOnYZJWoZJVY7_ooool3ojVYZ@05oooooonYZJWoZJVYojVYZOooool00onY
ZJT4o`0007?ooooo001]ooooo`?o000000GoooooojVYZOnYZJWoZJVYo`000003ojVYZBkooooo0onY
ZJT00ooooooo0000o`000002o`0000?oZJVYKoooool006Wooooo0ol000000ooooooo0000o`000002
o`0000?oZJVY=_ooool3o`000004oooooonYZJWoZJVYojVYZ@Co0000Joooool006Kooooo0onYZJT7
o`0003kooooo0onYZJT00ooooooo0000o`000005o`0006Oooooo001Rooooo`Co00000onYZJT3o`00
04Kooooo0ol000000ooooooo0000o`000002o`0000?oZJVYHoooool006;ooooo0ol000001?oooooo
ZJVYojVYZOnYZJU=ooooo`?o000000CoooooojVYZOnYZJWoZJVYHoooool005kooooo1ol0001Eoooo
o`?oZJVY00Co0000ojVYZOnYZJWoZJVYGoooool005[ooooo1ol0001Mooooo`?oZJVY1?l0001Koooo
o`00F_ooool3o`000004ooooool0003o0000o`0005gooooo0onYZJT01?oooooo0000o`000?l0001K
ooooo`00E_ooool7o`0006Gooooo0ol00003ojVYZESooooo001Fooooo`?o0000K?ooool3ojVYZ@03
o`000?oooooooooo05Gooooo001Cooooo`?oZJVY0ol0001/ooooo`?oZJVY00CoooooojVYZOnYZJWo
ZJVYE?ooool005?ooooo0onYZJUcooooo`?oZJVYE?ooool004oooooo0onYZJT01?l0003oZJVYojVY
ZOnYZJUcooooo`?oZJVY1?l0001@ooooo`00Coooool3ojVYZG_ooooo0ol0001@ooooo`00Boooool3
ojVYZ@04o`000?nYZJWoZJVYojVYZG_ooooo1_l0001=ooooo`00Boooool3ojVYZH;ooooo0ol0001=
ooooo`00B?ooool3o`0000?oZJVYP_ooool4o`0000?oZJVYBOooool004Sooooo0ol00029ooooo`?o
ZJVYBOooool004Cooooo1ol00029ooooo`KoZJVYA_ooool004Cooooo0ol0002@ooooo`?oZJVYA_oo
ool004?ooooo1?l0002@ooooo`?oZJVY0_l00014ooooo`00@Oooool3o`0009Kooooo0onYZJU3oooo
o`00@Oooool3o`0009Kooooo0onYZJT00ol0003oooooooooo`10ooooo`00?_ooool3ojVYZ@?o0000
U_ooool3ojVYZ@04oooooonYZJWoZJVYojVYZCoooooo000nooooo`?oZJVYWOooool3ojVYZCoooooo
000mooooo`04o`000?nYZJWoZJVYojVYZIgooooo0onYZJT00ol0003oooooooooo`0looooo`00>ooo
ool3ojVYZJ?ooooo0ol0000looooo`00>oooool3ojVYZJ?ooooo0ol0000looooo`00>?ooool6ojVY
ZJ?ooooo0ol00003ojVYZCWooooo000hooooo`?oZJVYZOooool3ojVYZCWooooo000hooooo`?oZJVY
ZOooool3ojVYZCWooooo000eooooo`?oZJVY[oooool3ojVYZCKooooo000eooooo`?oZJVY[oooool3
ojVYZCKooooo000eooooo`?oZJVY[oooool3ojVYZCKooooo000booooo`?o0000]Oooool3ojVYZC?o
oooo000booooo`?o0000]Oooool3ojVYZC?ooooo000booooo`?o0000]Oooool3ojVYZC?ooooo000_
ooooo`?o0000^oooool3o`00033ooooo000_ooooo`?o0000^oooool3o`00033ooooo000_ooooo`?o
0000^oooool3o`00033ooooo000/ooooo`?oZJVY`Oooool3o`0002gooooo000/ooooo`?oZJVY`Ooo
ool3o`0002gooooo000/ooooo`?oZJVY`Oooool3o`0002gooooo000Yooooo`?o0000a_ooool3o`00
02_ooooo000Yooooo`?o0000a_ooool3o`0002_ooooo000Yooooo`?o0000a_ooool3o`0002_ooooo
000Wooooo`?oZJVYboooool3o`0002Sooooo000Wooooo`?oZJVYboooool3o`0002Sooooo000Woooo
o`?oZJVYboooool3o`0002Sooooo000Vooooo`03o`000?oooooooooo0<oooooo00?o0000oooooooo
ool09Oooool002Cooooo0onYZJW@ooooo`?oZJVY9_ooool002Cooooo0onYZJW@ooooo`?oZJVY9_oo
ool002Cooooo0onYZJW@ooooo`?oZJVY00?o0000ooooooooool08oooool002;ooooo0onYZJWEoooo
o`?oZJVY8oooool002;ooooo0onYZJWEooooo`?oZJVY8oooool002;ooooo0onYZJWEooooo`?oZJVY
8oooool0023ooooo0ol0003Iooooo`?oZJVY8Oooool0023ooooo0ol0003Iooooo`?oZJVY8Oooool0
023ooooo0ol0003Iooooo`?oZJVY8Oooool0023ooooo00?o0000ooooooooool0foooool00ol0003o
ooooooooo`0Oooooo`007_ooool3o`000=gooooo0ol0000Oooooo`007_ooool3o`000=gooooo0ol0
000Oooooo`007_ooool3o`000=gooooo0ol0000Oooooo`006oooool4o`000>7ooooo0ol0000Moooo
o`006oooool3o`000>;ooooo0ol0000Mooooo`006oooool3o`000>;ooooo0ol0000Mooooo`007?oo
ool00ol0003oooooooooo`3Sooooo`03o`000?oooooooooo01_ooooo000Jooooo`?oZJVYiOooool3
ojVYZA_ooooo000Jooooo`?oZJVYiOooool3ojVYZA_ooooo000Jooooo`?oZJVYiOooool3ojVYZA_o
oooo000Jooooo`03o`000?oooooooooo0>Oooooo00?o0000ooooooooool06Oooool001Sooooo0onY
ZJWYooooo`?oZJVY6Oooool001Sooooo0onYZJWYooooo`?oZJVY6Oooool001Sooooo0onYZJWYoooo
o`?oZJVY6Oooool001Kooooo0onYZJW]ooooo`?oZJVY5oooool001Kooooo0onYZJW]ooooo`?oZJVY
5oooool001Kooooo0onYZJW]ooooo`?oZJVY5oooool001Oooooo00?o0000ooooooooool0kOooool0
0ol0003oooooooooo`0Fooooo`005Oooool3o`000>oooooo0ol0000Fooooo`005Oooool3o`000>oo
oooo0ol0000Fooooo`005Oooool3o`000>oooooo0ol0000Fooooo`005Oooool00ol0003ooooooooo
o`3aooooo`03o`000?oooooooooo01Cooooo000Cooooo`?o0000loooool3o`0001Cooooo000Coooo
o`?o0000loooool3o`0001Cooooo000Cooooo`?o0000loooool3o`0001Cooooo000Dooooo`03o`00
0?oooooooooo0??ooooo00?o0000ooooooooool04oooool001;ooooo0onYZJWeooooo`?oZJVY4ooo
ool001;ooooo0onYZJWeooooo`?oZJVY4oooool001;ooooo0onYZJWeooooo`?oZJVY4oooool001?o
oooo00?o0000ooooooooool0mOooool00ol0003oooooooooo`0Booooo`004Oooool3ojVYZOOooooo
0ol0000Booooo`004Oooool3ojVYZOOooooo0ol0000Booooo`004Oooool3ojVYZOOooooo0ol0000B
ooooo`004_ooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo017ooooo000?ooooo`?o
0000n_ooool3o`00017ooooo000?ooooo`?o0000n_ooool3o`00017ooooo000?ooooo`?o0000n_oo
ool3o`00017ooooo000Aooooo`03o`000?oooooooooo0?Wooooo00?o0000ooooooooool04?ooool0
00oooooo0onYZJWkooooo`?oZJVY4?ooool000oooooo0onYZJWkooooo`?oZJVY4?ooool000oooooo
0onYZJWkooooo`?oZJVY4?ooool0013ooooo00?o0000ooooooooool0noooool00ol0003ooooooooo
o`0?ooooo`003_ooool3o`000?gooooo0ol0000?ooooo`003_ooool3o`000?gooooo0ol0000?oooo
o`003_ooool3o`000?gooooo0ol0000?ooooo`003oooool00ol0003oooooooooo`3mooooo`03o`00
0?oooooooooo00kooooo000=ooooo`?oZJVYooooool3o`0000kooooo000=ooooo`?oZJVYooooool3
o`0000kooooo000=ooooo`?oZJVYooooool3o`0000kooooo000>ooooo`03o`000?oooooooooo0?oo
oooo00?o0000ooooooooool03Oooool000cooooo0ol0003oooooo`7ooooo0onYZJT>ooooo`003?oo
ool3o`000?oooooo0Oooool3ojVYZ@kooooo000<ooooo`?o0000ooooool1ooooo`?oZJVY3_ooool0
00kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003?ooool3o`000?oo
oooo0_ooool3o`0000gooooo000<ooooo`?o0000ooooool2ooooo`?o00003Oooool000cooooo0ol0
003oooooo`;ooooo0ol0000=ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000
ooooooooool03?ooool000cooooo0ol0003oooooo`;ooooo0onYZJT=ooooo`003?ooool3o`000?oo
oooo0_ooool3ojVYZ@gooooo000<ooooo`?o0000ooooool2ooooo`?oZJVY3Oooool000gooooo00?o
0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo00cooooo000;ooooo`?o0000ooooool3
ooooo`?o00003Oooool000_ooooo0ol0003oooooo`?ooooo0ol0000=ooooo`002oooool3o`000?oo
oooo0oooool3o`0000gooooo000=ooooo`03o`000?oooooooooo0?oooooo0_ooool00ol0003ooooo
ooooo`0<ooooo`002oooool3o`000?oooooo0oooool3ojVYZ@gooooo000;ooooo`?o0000ooooool3
ooooo`?oZJVY3Oooool000_ooooo0ol0003oooooo`?ooooo0onYZJT=ooooo`003Oooool00ol0003o
ooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?ooool000_ooooo0onYZJWoooooo`?ooooo
0onYZJT=ooooo`002oooool3ojVYZOoooooo0oooool3ojVYZ@gooooo000;ooooo`?oZJVYooooool3
ooooo`?oZJVY3Oooool000gooooo00?o0000ooooooooool0ooooool2ooooo`03o`000?oooooooooo
00cooooo000<ooooo`?o0000ooooool2ooooo`?o00003Oooool000cooooo0ol0003oooooo`;ooooo
0ol0000=ooooo`003?ooool3o`000?oooooo0_ooool3o`0000gooooo000=ooooo`03o`000?oooooo
oooo0?oooooo0_ooool00ol0003oooooooooo`0<ooooo`003?ooool3o`000?oooooo0_ooool3o`00
00gooooo000<ooooo`?o0000ooooool2ooooo`?o00003Oooool000cooooo0ol0003oooooo`;ooooo
0ol0000=ooooo`003Oooool00ol0003oooooooooo`3oooooo`;ooooo00?o0000ooooooooool03?oo
ool000kooooo00?o0000ooooooooool0ooooool00ol0003oooooooooo`0=ooooo`003?ooool3o`00
0?oooooo0Oooool3o`0000kooooo000<ooooo`?o0000ooooool1ooooo`?o00003_ooool000cooooo
0ol0003oooooo`7ooooo0ol0000>ooooo`003_ooool00ol0003oooooooooo`3oooooo`03o`000?oo
oooooooo00gooooo000=ooooo`?o0000ooooool3ojVYZ@kooooo000=ooooo`?o0000ooooool3ojVY
Z@kooooo000=ooooo`?o0000ooooool3ojVYZ@kooooo000?ooooo`03o`000?oooooooooo0?gooooo
00?o0000ooooooooool03_ooool000kooooo0onYZJWmooooo`?o00003oooool000kooooo0onYZJWm
ooooo`?o00003oooool000kooooo0onYZJWmooooo`?o00003oooool0013ooooo00?o0000oooooooo
ool0noooool00ol0003oooooooooo`0?ooooo`003oooool3o`000?_ooooo0onYZJT@ooooo`003ooo
ool3o`000?_ooooo0onYZJT@ooooo`003oooool3o`000?_ooooo0onYZJT@ooooo`004Oooool00ol0
003oooooooooo`3iooooo`03o`000?oooooooooo013ooooo000?ooooo`?oZJVYn_ooool3ojVYZA7o
oooo000?ooooo`?oZJVYn_ooool3ojVYZA7ooooo000?ooooo`?oZJVYn_ooool3ojVYZA7ooooo000B
ooooo`03o`000?oooooooooo0?Oooooo00?o0000ooooooooool04Oooool0017ooooo0ol0003goooo
o`?oZJVY4_ooool0017ooooo0ol0003gooooo`?oZJVY4_ooool0017ooooo0ol0003gooooo`?oZJVY
4_ooool001;ooooo0ol0003eooooo`?o00004oooool001;ooooo0ol0003eooooo`?o00004oooool0
01;ooooo0ol0003eooooo`?o00004oooool001?ooooo00?o0000ooooooooool0mOooool00ol0003o
ooooooooo`0Booooo`004oooool3o`000??ooooo0onYZJTDooooo`004oooool3o`000??ooooo0onY
ZJTDooooo`004oooool3o`000??ooooo0onYZJTDooooo`005Oooool00ol0003oooooooooo`3aoooo
o`03o`000?oooooooooo01Cooooo000Eooooo`?oZJVYkoooool3o`0001Kooooo000Eooooo`?oZJVY
koooool3o`0001Kooooo000Eooooo`?oZJVYkoooool3o`0001Kooooo000Fooooo`03o`000?oooooo
oooo0>oooooo00?o0000ooooooooool05Oooool001Kooooo0ol0003]ooooo`?oZJVY5oooool001Ko
oooo0ol0003]ooooo`?oZJVY5oooool001Kooooo0ol0003]ooooo`?oZJVY5oooool001Sooooo00?o
0000ooooooooool0joooool00ol0003oooooooooo`0Gooooo`006?ooool3ojVYZNWooooo0onYZJTI
ooooo`006?ooool3ojVYZNWooooo0onYZJTIooooo`006?ooool3ojVYZNWooooo0onYZJTIooooo`00
6_ooool3ojVYZNGooooo0ol0000Kooooo`006_ooool3ojVYZNGooooo0ol0000Kooooo`006_ooool3
ojVYZNGooooo0ol0000Kooooo`007?ooool00ol0003oooooooooo`3Sooooo`03o`000?oooooooooo
01_ooooo000Kooooo`?oZJVYh_ooool3o`0001gooooo000Kooooo`?oZJVYh_ooool3o`0001gooooo
000Kooooo`?oZJVYh_ooool3o`0001gooooo000Nooooo`03o`000?oooooooooo0=oooooo00?o0000
ooooooooool07Oooool001kooooo0onYZJWMooooo`?oZJVY7oooool001kooooo0onYZJWMooooo`?o
ZJVY7oooool001kooooo0onYZJWMooooo`?oZJVY7oooool0023ooooo0ol0003Iooooo`?oZJVY8Ooo
ool0023ooooo0ol0003Iooooo`?oZJVY8Oooool0023ooooo0ol0003Iooooo`?oZJVY8Oooool002;o
oooo00?o0000ooooooooool0eoooool00ol0003oooooooooo`0Qooooo`008_ooool3ojVYZMGooooo
0onYZJTSooooo`008_ooool3ojVYZMGooooo0onYZJTSooooo`008_ooool3ojVYZMGooooo0onYZJTS
ooooo`009?ooool3ojVYZM3ooooo0onYZJT00ol0003oooooooooo`0Sooooo`009?ooool3ojVYZM3o
oooo0onYZJTVooooo`009?ooool3ojVYZM3ooooo0onYZJTVooooo`009oooool3o`000<_ooooo0ol0
000Xooooo`009oooool3o`000<_ooooo0ol0000Xooooo`009oooool3o`000<_ooooo0ol0000Xoooo
o`00:Oooool3o`000<Kooooo1?l0000Zooooo`00:Oooool3o`000<Kooooo0ol0000[ooooo`00:Ooo
ool3o`000<Kooooo0ol0000[ooooo`00;?ooool3ojVYZL7ooooo0ol0000]ooooo`00;?ooool3ojVY
ZL7ooooo0ol0000]ooooo`00;?ooool3ojVYZL7ooooo0ol0000]ooooo`00;oooool3ojVYZK_ooooo
0onYZJT`ooooo`00;oooool3ojVYZK_ooooo0onYZJT`ooooo`00;oooool3ojVYZK_ooooo0onYZJT`
ooooo`00<_ooool3ojVYZKGooooo0ol0000cooooo`00<_ooool3ojVYZKGooooo0ol0000cooooo`00
<_ooool3ojVYZKGooooo0ol0000cooooo`00=Oooool3ojVYZJoooooo0onYZJTfooooo`00=Oooool3
ojVYZJoooooo0onYZJTfooooo`00=Oooool3ojVYZJoooooo0onYZJTfooooo`00>?ooool3o`000:Wo
oooo0onYZJTiooooo`00>?ooool3o`000:Wooooo0onYZJTiooooo`00>?ooool3o`000:Wooooo0onY
ZJTiooooo`00>oooool3o`000:?ooooo0ol0000looooo`00>oooool3o`000:?ooooo0ol0000loooo
o`00>oooool6o`0009gooooo1_l0000looooo`00?_ooool3o`0009gooooo0ol0000oooooo`00?_oo
ool3o`0009gooooo0ol0000oooooo`00@Oooool3ojVYZIKooooo1?l00012ooooo`00@Oooool3ojVY
ZIKooooo0ol00013ooooo`00@Oooool3ojVYZ@?o0000T?ooool6o`0004?ooooo0014ooooo`Co0000
Soooool3o`0004Kooooo0014ooooo`?o000000CoooooojVYZOnYZJWoZJVYROooool3ojVYZ@?o0000
A_ooool004Sooooo0onYZJV9ooooo`?oZJVYBOooool004Sooooo1_nYZJV2ooooo`?oZJVY00Co0000
ojVYZOnYZJWoZJVYBOooool004_ooooo0onYZJT00ol0003oooooooooo`1oooooo`?oZJVYCOooool0
04_ooooo0onYZJT00ooooooo0000ooooo`1nooooo`04o`000?nYZJWoZJVYojVYZDgooooo001?oooo
o`?o0000Noooool3ojVYZE3ooooo001?ooooo`Co00000onYZJUcooooo`?oZJVY00Co0000ojVYZOnY
ZJWoZJVYD?ooool004oooooo0ol000001?ooooooZJVYojVYZOnYZJUcooooo`?oZJVY00CoooooojVY
ZOnYZJWoZJVYD?ooool005?ooooo0onYZJT3o`0006cooooo1?l00003ojVYZECooooo001Fooooo`?o
0000K?ooool3o`0005Sooooo001Fooooo`Oo0000IOooool6o`0005Sooooo001Jooooo`Co00000onY
ZJUMooooo`?oZJVY1?l0001Kooooo`00F_ooool3o`000004oooooonYZJWoZJVYojVYZEgooooo0onY
ZJT01?oooooo0000o`000?l0001Kooooo`00G_ooool3ojVYZ@Co0000EOooool3ojVYZ@04o`000?nY
ZJWoZJVYojVYZEoooooo001Rooooo`Co00000onYZJU=ooooo`Co00000onYZJUSooooo`00H_ooool3
o`000003oooooonYZJWoZJVY00CoZJVYA_ooool7o`000004oooooonYZJWoZJVYojVYZF?ooooo001V
ooooo`KoZJVY0_l00013ooooo`Co000000Coooooo`000?l0003o0000Ioooool006Wooooo0onYZJT0
1?ooooooZJVYojVYZOnYZJT4o`0003Kooooo1ol000001?oooooo0000o`000?l0001[ooooo`00KOoo
ool3ojVYZ@03ooooool0003o000000Go0000;_ooool3ojVYZ@Co000000Coooooo`000?l0003o0000
Koooool006gooooo0onYZJT01Ooooooo0000o`000?l0003ooooo00Oo00009_ooool3ojVYZ@05o`00
0?nYZJWoZJVYojVYZOooool00ol000001?oooooo0000o`000?l0001_ooooo`00MOooool3o`000003
ooooool0003o000000;o00000onYZJT2o`0000?oZJVY00CoooooojVYZOnYZJWoZJVY3Oooool3ojVY
Z@05oooooonYZJWoZJVYojVYZOl000000onYZJT01Ol0003oZJVYojVYZOnYZJWooooo00?oZJVYMooo
ool007Wooooo0ol000001?ooooooZJVYojVYZOnYZJT2ooooo`?oZJVY00Co0000ojVYZOnYZJWoZJVY
3Ol00003ojVYZ@05o`000?nYZJWoZJVYojVYZOooool00onYZJT01?ooooooZJVYojVYZOnYZJUkoooo
o`00OOooool3ojVYZ@;ooooo0onYZJT01OooooooZJVYojVYZOnYZJWooooo00?o000000Goooooo`00
0?l0003o0000ooooo`03o`000005oooooonYZJWoZJVYojVYZOooool00onYZJT01?ooooooZJVYojVY
ZOnYZJUoooooo`00R_ooool3o`000005ooooool0003o0000o`000?ooool00ol0002;ooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "To make everyone in the model use the custom learning rule, you need to \
assign it to agents. The function ",
  StyleBox["Agents", "Input"],
  " returns a list of every agent in the model."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(agentList = Agents[pd5];\)\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["We can check that there are 200 agents in the model.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[agentList]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(200\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "The function for setting the learning rule is ",
  StyleBox["SetLearningRule", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Map[\[IndentingNewLine]\((SetLearningRule[#, 
              RandomAdoptionWhenDissatisfied])\) &, \
\[IndentingNewLine]agentList\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "We can verify that all agents are using the custom learning rule we just \
set. ",
  StyleBox["LearningRule[", "Input"],
  StyleBox["agent",
    FontSlant->"Italic"],
  StyleBox["]", "Input"],
  " returns the custom learning rule for ",
  StyleBox["agent",
    FontSlant->"Italic"],
  ". If no custom learning rule is set, ",
  StyleBox["LearningRule", "Input"],
  " returns nothing."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  " ",
  StyleBox["GetAgent[", "Input"],
  StyleBox["model, i",
    FontSlant->"Italic"],
  StyleBox["]", "Input"],
  " returns the ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th agent of ",
  StyleBox["model",
    FontSlant->"Italic"],
  ". So, to verify that the 23rd agent of ",
  StyleBox["pd5", "Input"],
  " is using our custom learning rule, we evaluate"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(LearningRule[GetAgent[pd5, \ 23]]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(RandomAdoptionWhenDissatisfied\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Now let's step the model forward 50 generations and see what \
happens.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Showing 50 circular graphs side-by-side is not an efficient way of \
representing the information. In cases like this, a ",
  StyleBox["linear embedding",
    FontSlant->"Italic"],
  " is better. A linear embedding places vertices ",
  Cell[BoxData[
      \(TraditionalForm\`1, \[Ellipsis], n\)]],
  " at coordinates ",
  Cell[BoxData[
      \(TraditionalForm\`\((i, 0)\)\)]],
  " for ",
  Cell[BoxData[
      \(TraditionalForm\`i \[Element] {1, \[Ellipsis], n}\)]],
  ". Once we draw each of these 100 graphs as a linear embedding, we can \
stack them vertically so that the first generation starts at the top and then \
runs down."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "The fastest way to create a linear embedding is to call ",
  StyleBox["ToColorArray[", "Input"],
  StyleBox["model",
    FontSlant->"Italic"],
  StyleBox["]", "Input"],
  ". This returns a list of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " ",
  StyleBox["RGBColor", "Input"],
  " primitives, with the ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th element of the list",
  " corresponding to the color of the ",
  StyleBox["i",
    FontSlant->"Italic"],
  "th agent in the model."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Also, since using a custom learning rule for 200 agents runs slower than \
the default rule of ",
  StyleBox["Imitate the Best",
    FontSlant->"Italic"],
  ", we create and use a progress bar to indicate how many generations we've \
completed."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(bar = 
        CreateProgressBar[
          50, "\<Generations completed (model: pd5)\>"];\)\), "\n", 
    \(Do[\[IndentingNewLine]AppendTo[l, 
        ToColorArray[pd5]]; \[IndentingNewLine]Evolve[
        pd5]; \[IndentingNewLine]IncrementProgressBar[
        bar], {50}\[IndentingNewLine]]\)}], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Now show the time evolution.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]Graphics[\[IndentingNewLine]RasterArray[
            l]\[IndentingNewLine]], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .25 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.00595238 0.0047619 [
[ 0 0 0 0 ]
[ 1 .25 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .25 L
0 .25 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .00595 translate
.95238 .2381 scale
600 string
200 50 8 [200 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000
000000000000A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000000000000000000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000000000A9A9A9000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000000000
A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
A9A9A9000000000000000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000
A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000000000
A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000000000
A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000
000000000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9000000000000000000000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[18]:=",
  ImageSize->{404, 101},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006D0000IB000`400?l00000o`00003ooooooonE
ooooo`00oooooonEooooo`00oooooonEooooo`00oooooonEooooo`002oooool20:VYZ@H000000P2Y
ZJT00`000000ZJVY0:VYZ@040:VYZ@8000000P2YZJT2000000@0ZJVY0`0000080:VYZ@H000000P2Y
ZJT2000000<0ZJVY200000080:VYZ@<00000102YZJT8000000T0ZJVY1P0000060:VYZ@H000001@2Y
ZJT200000080ZJVY100000020:VYZ@800000102YZJT00`000000ZJVY0:VYZ@06000000P0ZJVY0`00
00040:VYZ@800000202YZJT200000080ZJVY1@0000020:VYZ@8000000P2YZJT800000080ZJVY2@00
00020:VYZ@8000000P2YZJT2000000@0ZJVY0`0000060:VYZ@H000001P2YZJT00`000000ZJVY0:VY
Z@04000000P0ZJVY1@0000060:VYZ@P000000P2YZJT4000000/0ZJVY0P0000040:VYZ@<000001P2Y
ZJT200000080ZJVY100000040:VYZ@8000000`2YZJT4000000P0ZJVY0P0000050:VYZ@P000001P2Y
ZJT4000000D0ZJVY100000020:VYZ@@000000P2YZJT300000080ZJVY0P0000020:VYZ@8000000P2Y
ZJT<ooooo`002oooool20:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@040:VYZ@8000000P2YZJT2
000000@0ZJVY0`0000080:VYZ@H000000P2YZJT2000000<0ZJVY200000080:VYZ@<00000102YZJT8
000000T0ZJVY1P0000060:VYZ@H000001@2YZJT200000080ZJVY100000020:VYZ@800000102YZJT0
0`000000ZJVY0:VYZ@06000000P0ZJVY0`0000040:VYZ@800000202YZJT200000080ZJVY1@000002
0:VYZ@8000000P2YZJT800000080ZJVY2@0000020:VYZ@8000000P2YZJT2000000@0ZJVY0`000006
0:VYZ@H000001P2YZJT00`000000ZJVY0:VYZ@04000000P0ZJVY1@0000060:VYZ@P000000P2YZJT4
000000/0ZJVY0P0000040:VYZ@<000001P2YZJT200000080ZJVY100000040:VYZ@8000000`2YZJT4
000000P0ZJVY0P0000050:VYZ@P000001P2YZJT4000000D0ZJVY100000020:VYZ@@000000P2YZJT3
00000080ZJVY0P0000020:VYZ@8000000P2YZJT<ooooo`002oooool40:VYZ@8000002@2YZJT20000
00H0ZJVY1000000=0:VYZ@8000000P2YZJT2000000L0ZJVY0P00000<0:VYZ@80000000<0ZJVY0000
000000000P0000020:VYZ@P000001`2YZJT400000080ZJVY0P0000020:VYZ@P000001`2YZJT20000
0080ZJVY0P0000040:VYZ@8000001`2YZJT6000000H0ZJVY0P0000000`2YZJT00000000000040:VY
Z@P00000102YZJT5000000H0ZJVY1P0000070:VYZ@P000000P2YZJT2000000T0ZJVY0P0000040:VY
Z@800000102YZJT6000000<0ZJVY100000060:VYZ@@000002@2YZJT2000001<0ZJVY100000020:VY
Z@@000000P2YZJT3000000P0ZJVY0P0000020:VYZ@@000000`2YZJT2000000@0ZJVY0P0000040:VY
Z@8000000P2YZJT2000000<0ZJVY0P0000060:VYZ@8000000P2YZJT200000080ZJVY0P0000090:VY
Z@800000202YZJT00`000000ZJVY0:VYZ@080:VYZ@cooooo000;ooooo`@0ZJVY0P0000090:VYZ@80
00001P2YZJT4000000d0ZJVY0P0000020:VYZ@8000001`2YZJT2000000`0ZJVY0P0000000`2YZJT0
00000000000200000080ZJVY200000070:VYZ@@000000P2YZJT200000080ZJVY200000070:VYZ@80
00000P2YZJT2000000@0ZJVY0P0000070:VYZ@H000001P2YZJT2000000030:VYZ@000000000000@0
ZJVY200000040:VYZ@D000001P2YZJT6000000L0ZJVY200000020:VYZ@8000002@2YZJT2000000@0
ZJVY0P0000040:VYZ@H000000`2YZJT4000000H0ZJVY100000090:VYZ@8000004`2YZJT400000080
ZJVY100000020:VYZ@<00000202YZJT200000080ZJVY100000030:VYZ@800000102YZJT2000000@0
ZJVY0P0000020:VYZ@8000000`2YZJT2000000H0ZJVY0P0000020:VYZ@8000000P2YZJT2000000T0
ZJVY0P0000080:VYZ@030000002YZJT0ZJVY00P0ZJVY3?ooool000_ooooo0P2YZJT400000080ZJVY
0P0000050:VYZ@8000000P2YZJT8000000/0ZJVY200000020:VYZ@80000000<0ZJVY000000000000
100000060:VYZ@8000000P2YZJT700000080ZJVY0P00000?0:VYZ@8000000P2YZJT200000080ZJVY
0P0000020:VYZ@D00000102YZJT8000000@0ZJVY100000090:VYZ@800000102YZJT7000000@0ZJVY
0P0000040:VYZ@800000102YZJT2000000030:VYZ@000000000000@0ZJVY100000040:VYZ@800000
1@2YZJT400000080ZJVY1P0000090:VYZ@8000000P2YZJT?00000080ZJVY1P0000020:VYZ@H00000
1`2YZJT6000000l0ZJVY4@0000060:VYZ@@000000P2YZJT;00000080ZJVY0P0000040:VYZ@H00000
00<0ZJVY0000000000003P0000020:VYZ@<00000202YZJT200000080ZJVY1`00000:0:VYZ@cooooo
000;ooooo`80ZJVY100000020:VYZ@8000001@2YZJT200000080ZJVY2000000;0:VYZ@P000000P2Y
ZJT2000000030:VYZ@000000000000@000001P2YZJT200000080ZJVY1`0000020:VYZ@8000003`2Y
ZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT5000000@0ZJVY200000040:VYZ@@000002@2Y
ZJT2000000@0ZJVY1`0000040:VYZ@800000102YZJT2000000@0ZJVY0P0000000`2YZJT000000000
00040:VYZ@@00000102YZJT2000000D0ZJVY100000020:VYZ@H000002@2YZJT200000080ZJVY3`00
00020:VYZ@H000000P2YZJT6000000L0ZJVY1P00000?0:VYZA4000001P2YZJT400000080ZJVY2`00
00020:VYZ@800000102YZJT6000000030:VYZ@000000000000h000000P2YZJT3000000P0ZJVY0P00
00020:VYZ@L000002P2YZJT<ooooo`002oooool20:VYZ@l00000102YZJT6000001<0ZJVY1@000004
0:VYZ@@000000P2YZJT400000080ZJVY0P0000030:VYZ@8000000P2YZJT4000000X0ZJVY00<00000
0:VYZ@2YZJT00P0000020:VYZ@P00000102YZJT700000080ZJVY100000020:VYZ@L00000102YZJT6
00000080ZJVY1P0000000`2YZJT00000000000040:VYZ@8000000P2YZJT600000080ZJVY0P000000
0`2YZJT000000000000200000080ZJVY2P0000070:VYZ@8000000P2YZJT2000000@0ZJVY0P000004
0:VYZ@030000002YZJT0ZJVY00800000202YZJT4000000@0ZJVY2@0000020:VYZ@H000001`2YZJT6
000000d0ZJVY0P0000020:VYZA4000001P2YZJT200000080ZJVY0P0000050:VYZ@8000000P2YZJT4
00000080ZJVY5@0000020:VYZ@8000000P2YZJT2000000030:VYZ@000000000000@0ZJVY1P000004
0:VYZ@80000000<0ZJVY000000000000202YZJT<ooooo`002oooool20:VYZ@l00000102YZJT60000
01<0ZJVY1@0000040:VYZ@@000000P2YZJT400000080ZJVY0P0000030:VYZ@8000000P2YZJT40000
00X0ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@P00000102YZJT700000080ZJVY100000020:VY
Z@L00000102YZJT600000080ZJVY1P0000000`2YZJT00000000000040:VYZ@8000000P2YZJT60000
0080ZJVY0P0000000`2YZJT000000000000200000080ZJVY2P0000070:VYZ@8000000P2YZJT20000
00@0ZJVY0P0000040:VYZ@030000002YZJT0ZJVY00800000202YZJT4000000@0ZJVY2@0000020:VY
Z@H000001`2YZJT6000000d0ZJVY0P0000020:VYZA4000001P2YZJT200000080ZJVY0P0000050:VY
Z@8000000P2YZJT400000080ZJVY5@0000020:VYZ@8000000P2YZJT2000000030:VYZ@0000000000
00@0ZJVY1P0000040:VYZ@80000000<0ZJVY000000000000202YZJT<ooooo`002oooool400000080
ZJVY1@0000040:VYZ@@000000P2YZJT8000000l0ZJVY100000020:VYZ@030000002YZJT0ZJVY00H0
00000P2YZJT400000080ZJVY0`0000020:VYZ@@000000P2YZJT2000000P0ZJVY2`0000020:VYZ@H0
00000P2YZJT300000080ZJVY0P0000020:VYZ@P00000102YZJT300000080ZJVY300000020:VYZ@T0
0000102YZJT400000080ZJVY3@0000020:VYZ@@000001`2YZJT800000080ZJVY1`0000020:VYZ@H0
00001P2YZJT2000000@0ZJVY00<000000:VYZ@2YZJT00P2YZJT400000080ZJVY1000000I0:VYZ@03
0000002YZJT0ZJVY0100000000<0ZJVY0000000000002P0000020:VYZ@800000102YZJT300000080
ZJVY100000020:VYZ@800000102YZJTC00000080ZJVY0P0000000`2YZJT00000000000020:VYZ@@0
00000P2YZJT8000000030:VYZ@000000000000@00000102YZJT<ooooo`002oooool400000080ZJVY
0P0000020:VYZ@030000002YZJT0ZJVY010000003@2YZJT200000080ZJVY0P0000020:VYZ@030000
002YZJT0ZJVY00H000000P2YZJT=000000@0ZJVY0P00000;0:VYZ@8000000P2YZJT400000080ZJVY
2@0000040:VYZ@P000000P2YZJT?00000080ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY
0:VYZ@0200000080ZJVY0P0000020:VYZ@/000000P2YZJT400000080ZJVY2P0000050:VYZ@800000
0P2YZJT400000080ZJVY0P0000020:VYZ@L000000P2YZJT4000000@0ZJVY0P0000040:VYZ@d00000
5`2YZJT200000080ZJVY0`0000060:VYZ@@00000102YZJT2000000030:VYZ@000000000000@0ZJVY
300000020:VYZ@<000000P2YZJT200000080ZJVY100000020:VYZ@@0000000<0ZJVY000000000000
2P0000040:VYZ@@0000000<0ZJVY000000000000102YZJT200000080ZJVY0P0000020:VYZ@L00000
0P2YZJT400000080ZJVY3?ooool000_ooooo100000020:VYZ@8000000P2YZJT00`000000ZJVY0:VY
Z@0@000000d0ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0600000080ZJVY3@00
00040:VYZ@8000002`2YZJT200000080ZJVY100000020:VYZ@T00000102YZJT800000080ZJVY3`00
00020:VYZ@8000000P2YZJT200000080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@8000000P2Y
ZJT;00000080ZJVY100000020:VYZ@X000001@2YZJT200000080ZJVY100000020:VYZ@8000000P2Y
ZJT700000080ZJVY100000040:VYZ@800000102YZJT=000001L0ZJVY0P0000020:VYZ@<000001P2Y
ZJT4000000@0ZJVY0P0000000`2YZJT00000000000040:VYZ@`000000P2YZJT300000080ZJVY0P00
00020:VYZ@@000000P2YZJT4000000030:VYZ@000000000000X00000102YZJT4000000030:VYZ@00
0000000000@0ZJVY0P0000020:VYZ@8000000P2YZJT700000080ZJVY100000020:VYZ@cooooo000;
ooooo`8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT04000000=0:VYZ@8000000P2YZJT5
000000H0ZJVY0P0000020:VYZ@800000102YZJT2000000030:VYZ@000000000000800000102YZJT2
00000080ZJVY0P0000060:VYZ@T000000P2YZJT200000080ZJVY1@0000020:VYZ@@00000102YZJT2
00000080ZJVY6P0000020:VYZ@8000000P2YZJT200000080ZJVY1P0000040:VYZ@L000000P2YZJT2
00000080ZJVY1P0000020:VYZ@D00000202YZJT2000000@0ZJVY00<000000:VYZ@2YZJT020000002
0:VYZ@@00000102YZJT?00000140ZJVY1P0000030:VYZ@@000000P2YZJT600000080ZJVY10000000
0`2YZJT00000000000040:VYZ@@000001P2YZJT500000080ZJVY5`0000040:VYZ@@000000P2YZJT4
000000030:VYZ@00000000000080ZJVY100000020:VYZ@800000102YZJT=000000cooooo000;oooo
o`8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT04000000=0:VYZ@8000000P2YZJT50000
00H0ZJVY0P0000020:VYZ@800000102YZJT2000000030:VYZ@000000000000800000102YZJT20000
0080ZJVY0P0000060:VYZ@T000000P2YZJT200000080ZJVY1@0000020:VYZ@@00000102YZJT20000
0080ZJVY6P0000020:VYZ@8000000P2YZJT200000080ZJVY1P0000040:VYZ@L000000P2YZJT20000
0080ZJVY1P0000020:VYZ@D00000202YZJT2000000@0ZJVY00<000000:VYZ@2YZJT0200000020:VY
Z@@00000102YZJT?00000140ZJVY1P0000030:VYZ@@000000P2YZJT600000080ZJVY100000000`2Y
ZJT00000000000040:VYZ@@000001P2YZJT500000080ZJVY5`0000040:VYZ@@000000P2YZJT40000
00030:VYZ@00000000000080ZJVY100000020:VYZ@800000102YZJT=000000cooooo000;ooooo`80
00000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY1P0000040:VYZ@03
0000002YZJT0ZJVY00H0ZJVY0P0000020:VYZ@@00000102YZJT3000000@0ZJVY300000000`2YZJT0
00000000000200000080ZJVY200000070:VYZ@800000102YZJT600000080ZJVY0P0000020:VYZ@L0
00000P2YZJT4000000H0ZJVY4@0000040:VYZ@030000002YZJT0ZJVY008000000P2YZJT2000000@0
ZJVY100000040:VYZ@030000002YZJT0ZJVY0080ZJVY1P0000060:VYZ@8000000P2YZJT00`000000
ZJVY0:VYZ@02000000@0ZJVY5@0000020:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@0600000080
ZJVY1000000?0:VYZ@800000102YZJT200000080ZJVY2@0000020:VYZ@8000000P2YZJT200000080
ZJVY2@0000060:VYZ@8000000P2YZJT300000080ZJVY100000020:VYZ@/00000102YZJT400000080
ZJVY100000020:VYZ@@0000000<0ZJVY0000000000000P2YZJT200000080ZJVY2P0000030:VYZ@@0
00000P2YZJT2000000cooooo000;ooooo`8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT0
0P2YZJT200000080ZJVY1P0000040:VYZ@030000002YZJT0ZJVY00H0ZJVY0P0000020:VYZ@@00000
102YZJT3000000@0ZJVY300000000`2YZJT000000000000200000080ZJVY200000070:VYZ@800000
102YZJT600000080ZJVY0P0000020:VYZ@L000000P2YZJT4000000H0ZJVY4@0000040:VYZ@030000
002YZJT0ZJVY008000000P2YZJT2000000@0ZJVY100000040:VYZ@030000002YZJT0ZJVY0080ZJVY
1P0000060:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@02000000@0ZJVY5@0000020:VYZ@H00000
0P2YZJT00`000000ZJVY0:VYZ@0600000080ZJVY1000000?0:VYZ@800000102YZJT200000080ZJVY
2@0000020:VYZ@8000000P2YZJT200000080ZJVY2@0000060:VYZ@8000000P2YZJT300000080ZJVY
100000020:VYZ@/00000102YZJT400000080ZJVY100000020:VYZ@@0000000<0ZJVY000000000000
0P2YZJT200000080ZJVY2P0000030:VYZ@@000000P2YZJT2000000cooooo000;ooooo`8000000P2Y
ZJT2000000@0ZJVY4@0000020:VYZ@<000000P2YZJT<00000080ZJVY2`0000020:VYZ@800000102Y
ZJT2000000030:VYZ@0000000000008000000P2YZJT2000000X0ZJVY1`0000020:VYZ@H000000P2Y
ZJT2000000D0ZJVY100000020:VYZ@800000202YZJT;00000080ZJVY1P0000020:VYZ@T000000P2Y
ZJT600000080ZJVY0`0000020:VYZ@H00000102YZJT200000080ZJVY8P0000020:VYZ@800000102Y
ZJT3000000@0ZJVY100000020:VYZ@H000002@2YZJT200000080ZJVY2P0000000`2YZJT000000000
00080:VYZ@8000000P2YZJT200000080ZJVY00<000000:VYZ@2YZJT01P2YZJT400000080ZJVY1`00
00020:VYZ@@000000P2YZJT2000000P0ZJVY00<000000:VYZ@2YZJT00P2YZJT400000080ZJVY1000
00020:VYZ@@0000000<0ZJVY0000000000000P2YZJT4000000@0ZJVY1`0000020:VYZ@P000003?oo
ool000_ooooo0P0000020:VYZ@800000102YZJTA00000080ZJVY0`0000020:VYZ@`000000P2YZJT;
00000080ZJVY0P0000040:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@8000002P2YZJT7
00000080ZJVY1P0000020:VYZ@8000001@2YZJT400000080ZJVY0P0000080:VYZ@/000000P2YZJT6
00000080ZJVY2@0000020:VYZ@H000000P2YZJT300000080ZJVY1P0000040:VYZ@8000000P2YZJTR
00000080ZJVY0P0000040:VYZ@<00000102YZJT400000080ZJVY1P0000090:VYZ@8000000P2YZJT:
000000030:VYZ@000000000000P0ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@06
0:VYZ@@000000P2YZJT700000080ZJVY100000020:VYZ@800000202YZJT00`000000ZJVY0:VYZ@02
0:VYZ@@000000P2YZJT400000080ZJVY100000000`2YZJT00000000000020:VYZ@@00000102YZJT7
00000080ZJVY2000000<ooooo`002oooool20:VYZ@T00000102YZJT6000000@0ZJVY1`0000020:VY
Z@800000102YZJT200000080ZJVY1P0000050:VYZ@8000000P2YZJT4000000@0ZJVY0P0000050:VY
Z@@00000202YZJT200000080ZJVY00<000000:VYZ@2YZJT0400000000`2YZJT00000000000040000
0080ZJVY0P0000080:VYZ@8000000`2YZJT200000080ZJVY2P0000020:VYZ@030000002YZJT0ZJVY
00H000000P2YZJT200000080ZJVY0P0000020:VYZ@/000000P2YZJT4000000@0ZJVY0P0000000`2Y
ZJT000000000000E00000080ZJVY100000020:VYZ@/000000P2YZJT400000080ZJVY1P0000070:VY
Z@@00000102YZJT4000000D0ZJVY100000060:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@040:VY
Z@X000000P2YZJT300000080ZJVY200000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@060000
0080ZJVY2P0000000`2YZJT00000000000020:VYZ@X00000102YZJT5000000H0ZJVY3?ooool000_o
oooo0P2YZJT9000000@0ZJVY1P0000040:VYZ@L000000P2YZJT2000000@0ZJVY0P0000020:VYZ@H0
00001@2YZJT200000080ZJVY100000040:VYZ@8000001@2YZJT4000000P0ZJVY0P0000020:VYZ@03
0000002YZJT0ZJVY0100000000<0ZJVY000000000000100000020:VYZ@800000202YZJT2000000<0
ZJVY0P0000020:VYZ@X000000P2YZJT00`000000ZJVY0:VYZ@0600000080ZJVY0P0000020:VYZ@80
00000P2YZJT;00000080ZJVY100000040:VYZ@80000000<0ZJVY0000000000005@0000020:VYZ@@0
00000P2YZJT;00000080ZJVY100000020:VYZ@H000001`2YZJT4000000@0ZJVY100000050:VYZ@@0
00001P2YZJT600000080ZJVY00<000000:VYZ@2YZJT0102YZJT:00000080ZJVY0`0000020:VYZ@P0
00000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT01P0000020:VYZ@X0000000<0ZJVY00000000
00000P2YZJT:000000@0ZJVY1@0000060:VYZ@cooooo000;ooooo`H000000P2YZJT300000080ZJVY
200000020:VYZ@T000000P2YZJT400000080ZJVY2`0000040:VYZ@P000000P2YZJT5000000@0ZJVY
100000040:VYZ@8000000P2YZJT200000080ZJVY0`0000020:VYZAl000000P2YZJT300000080ZJVY
200000040:VYZ@D000000P2YZJT2000000H0ZJVY200000020:VYZ@030000002YZJT0ZJVY0080ZJVY
2P0000040:VYZ@<000000P2YZJT200000080ZJVY300000030:VYZ@8000000P2YZJT400000080ZJVY
0P0000020:VYZ@@0000000<0ZJVY0000000000000P2YZJT400000080ZJVY100000020:VYZ@800000
0`2YZJT6000000@0ZJVY0P0000090:VYZ@@000001P2YZJT200000080ZJVY3`0000020:VYZ@800000
0P2YZJT300000080ZJVY300000020:VYZ@<00000102YZJT:00000080ZJVY0P0000030:VYZ@800000
0P2YZJT2000000@0ZJVY3`0000040:VYZ@cooooo000;ooooo`H000000P2YZJT300000080ZJVY2000
00020:VYZ@T000000P2YZJT400000080ZJVY2`0000040:VYZ@P000000P2YZJT5000000@0ZJVY1000
00040:VYZ@8000000P2YZJT200000080ZJVY0`0000020:VYZAl000000P2YZJT300000080ZJVY2000
00040:VYZ@D000000P2YZJT2000000H0ZJVY200000020:VYZ@030000002YZJT0ZJVY0080ZJVY2P00
00040:VYZ@<000000P2YZJT200000080ZJVY300000030:VYZ@8000000P2YZJT400000080ZJVY0P00
00020:VYZ@@0000000<0ZJVY0000000000000P2YZJT400000080ZJVY100000020:VYZ@8000000`2Y
ZJT6000000@0ZJVY0P0000090:VYZ@@000001P2YZJT200000080ZJVY3`0000020:VYZ@8000000P2Y
ZJT300000080ZJVY300000020:VYZ@<00000102YZJT:00000080ZJVY0P0000030:VYZ@8000000P2Y
ZJT2000000@0ZJVY3`0000040:VYZ@cooooo000;ooooo`8000000P2YZJT200000080ZJVY0`000002
0:VYZ@@000000P2YZJT:000000030:VYZ@000000000000X00000102YZJT200000080ZJVY1`000004
0:VYZ@T000000P2YZJT600000080ZJVY100000020:VYZ@L000000P2YZJTI00000080ZJVY4`000002
0:VYZ@@000000P2YZJT500000080ZJVY0P0000040:VYZ@8000000P2YZJT5000000@0ZJVY0P000006
0:VYZ@8000000P2YZJT9000000H0ZJVY0P0000040:VYZ@T000000P2YZJT200000080ZJVY0P000002
0:VYZ@@0000000<0ZJVY0000000000000P0000020:VYZA4000000P2YZJT2000000@0ZJVY0P000009
0:VYZ@@000000P2YZJT8000000<0ZJVY1P0000020:VYZ@800000102YZJT400000080ZJVY50000002
0:VYZ@X000000P2YZJT400000080ZJVY0`0000020:VYZ@P00000102YZJT3000000@0ZJVY10000002
0:VYZ@cooooo000;ooooo`8000000P2YZJT200000080ZJVY0`0000020:VYZ@@000000P2YZJT:0000
00030:VYZ@000000000000X00000102YZJT200000080ZJVY1`0000040:VYZ@T000000P2YZJT60000
0080ZJVY100000020:VYZ@L000000P2YZJTI00000080ZJVY4`0000020:VYZ@@000000P2YZJT50000
0080ZJVY0P0000040:VYZ@8000000P2YZJT5000000@0ZJVY0P0000060:VYZ@8000000P2YZJT90000
00H0ZJVY0P0000040:VYZ@T000000P2YZJT200000080ZJVY0P0000020:VYZ@@0000000<0ZJVY0000
000000000P0000020:VYZA4000000P2YZJT2000000@0ZJVY0P0000090:VYZ@@000000P2YZJT80000
00<0ZJVY1P0000020:VYZ@800000102YZJT400000080ZJVY500000020:VYZ@X000000P2YZJT40000
0080ZJVY0`0000020:VYZ@P00000102YZJT3000000@0ZJVY100000020:VYZ@cooooo000;ooooo`H0
00000P2YZJT300000080ZJVY200000020:VYZ@8000000P2YZJT2000000030:VYZ@00000000000080
ZJVY100000020:VYZ@800000102YZJT500000080ZJVY1P0000020:VYZ@800000102YZJT700000080
ZJVY0P0000020:VYZA400000102YZJT=00000080ZJVY100000020:VYZ@@000000P2YZJT900000080
ZJVY0P0000020:VYZ@l00000102YZJT2000000H0ZJVY1`0000020:VYZ@8000000P2YZJT200000080
ZJVY100000050:VYZ@X000000P2YZJT2000000030:VYZ@00000000000080ZJVY2P0000040:VYZ@80
000000<0ZJVY000000000000200000020:VYZAT000000`2YZJT200000080ZJVY300000020:VYZ@<0
00000P2YZJT600000080ZJVY900000020:VYZ@8000000P2YZJT900000080ZJVY0P0000040:VYZ@80
00000P2YZJT700000080ZJVY1P00000<ooooo`002oooool600000080ZJVY0`0000020:VYZ@P00000
0P2YZJT200000080ZJVY0P0000000`2YZJT00000000000020:VYZ@@000000P2YZJT2000000@0ZJVY
1@0000020:VYZ@H000000P2YZJT2000000@0ZJVY1`0000020:VYZ@8000000P2YZJTA000000@0ZJVY
3@0000020:VYZ@@000000P2YZJT400000080ZJVY2@0000020:VYZ@8000000P2YZJT?000000@0ZJVY
0P0000060:VYZ@L000000P2YZJT200000080ZJVY0P0000020:VYZ@@000001@2YZJT:00000080ZJVY
0P0000000`2YZJT00000000000020:VYZ@X00000102YZJT2000000030:VYZ@000000000000P00000
0P2YZJTI000000<0ZJVY0P0000020:VYZ@`000000P2YZJT300000080ZJVY1P0000020:VYZB@00000
0P2YZJT200000080ZJVY2@0000020:VYZ@800000102YZJT200000080ZJVY1`0000020:VYZ@H00000
3?ooool000_ooooo102YZJT200000080ZJVY3@0000020:VYZ@/000000P2YZJT800000080ZJVY0P00
00020:VYZ@030000002YZJT0ZJVY00@00000102YZJT2000000@0ZJVY0`0000020:VYZ@H000000P2Y
ZJT400000080ZJVY3@0000020:VYZ@800000102YZJT2000000<0ZJVY0P0000020:VYZ@@000000P2Y
ZJT900000080ZJVY200000020:VYZ@8000000P2YZJT5000000@0ZJVY0P0000040:VYZ@800000102Y
ZJT2000000030:VYZ@0000000000008000000P2YZJT400000080ZJVY100000020:VYZ@8000000`2Y
ZJT2000000H0ZJVY0P0000020:VYZ@@0000000<0ZJVY000000000000100000020:VYZ@8000000P2Y
ZJT2000000@0ZJVY1`0000020:VYZ@X000000P2YZJT=000000@0ZJVY0P0000030:VYZ@800000102Y
ZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY0P0000020:VYZ@P000000P2Y
ZJTH000000@0ZJVY2P0000070:VYZ@@00000102YZJT200000080ZJVY100000000`2YZJT000000000
0008000000cooooo000;ooooo`@0ZJVY0P0000020:VYZ@d000000P2YZJT;00000080ZJVY20000002
0:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@04000000@0ZJVY0P0000040:VYZ@<000000P2YZJT6
00000080ZJVY100000020:VYZ@d000000P2YZJT2000000@0ZJVY0P0000030:VYZ@8000000P2YZJT4
00000080ZJVY2@0000020:VYZ@P000000P2YZJT200000080ZJVY1@0000040:VYZ@800000102YZJT2
000000@0ZJVY0P0000000`2YZJT000000000000200000080ZJVY100000020:VYZ@@000000P2YZJT2
000000<0ZJVY0P0000060:VYZ@8000000P2YZJT4000000030:VYZ@000000000000@000000P2YZJT2
00000080ZJVY0P0000040:VYZ@L000000P2YZJT:00000080ZJVY3@0000040:VYZ@8000000`2YZJT2
000000@0ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY0`0000020:VYZ@8000000P2YZJT8
00000080ZJVY600000040:VYZ@X000001`2YZJT4000000@0ZJVY0P0000020:VYZ@@0000000<0ZJVY
0000000000002000000<ooooo`002oooool20:VYZ@@000001@2YZJT4000000@0ZJVY0P0000020:VY
Z@8000001`2YZJT600000080ZJVY0P0000040:VYZ@8000000P2YZJT700000080ZJVY100000020:VY
Z@D000000P2YZJT200000080ZJVY4@0000040:VYZ@8000000P2YZJT200000080ZJVY100000000`2Y
ZJT000000000000<00000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00H00000102YZJT90000
0080ZJVY100000020:VYZ@@00000102YZJT700000080ZJVY100000020:VYZ@P000000`2YZJT20000
0080ZJVY1P0000020:VYZ@@0000000<0ZJVY0000000000000P0000040:VYZA4000000P2YZJT20000
00H0ZJVY0P0000020:VYZ@<000000P2YZJT200000080ZJVY1P0000020:VYZ@8000000P2YZJTA0000
0080ZJVY00<000000:VYZ@2YZJT0200000020:VYZ@@000000P2YZJT2000000<0ZJVY3P0000020:VY
Z@<00000102YZJT:000000L0ZJVY100000040:VYZ@@000000P2YZJT5000000H0ZJVY0P00000<oooo
o`002oooool20:VYZ@8000000P2YZJT200000080ZJVY00<000000:VYZ@2YZJT0400000030:VYZ@80
00000P2YZJT200000080ZJVY100000020:VYZ@@000000`2YZJT400000080ZJVY2P0000000`2YZJT0
00000000000800000080ZJVY200000000`2YZJT00000000000020:VYZ@P000000P2YZJT400000003
0:VYZ@0000000000024000000P2YZJT500000080ZJVY100000020:VYZ@H000000P2YZJT2000000<0
ZJVY0P0000020:VYZ@@000000P2YZJT200000080ZJVY100000000`2YZJT000000000000400000080
ZJVY3`0000020:VYZ@@00000102YZJT7000000H0ZJVY1P0000020:VYZ@@0000000<0ZJVY00000000
0000100000080:VYZ@@000000P2YZJT;00000080ZJVY1`0000020:VYZ@8000000P2YZJT400000080
ZJVY200000030:VYZ@8000000P2YZJT:00000080ZJVY2@0000020:VYZ@H000001@2YZJT200000080
ZJVY0P0000040:VYZ@@000000P2YZJT300000080ZJVY0P0000040:VYZ@8000003?ooool000_ooooo
0P2YZJT200000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY010000000`2YZJT200000080ZJVY
0P0000020:VYZ@@000000P2YZJT4000000<0ZJVY100000020:VYZ@X0000000<0ZJVY000000000000
200000020:VYZ@P0000000<0ZJVY0000000000000P2YZJT800000080ZJVY100000000`2YZJT00000
0000000Q00000080ZJVY1@0000020:VYZ@@000000P2YZJT600000080ZJVY0P0000030:VYZ@800000
0P2YZJT400000080ZJVY0P0000020:VYZ@@0000000<0ZJVY000000000000100000020:VYZ@l00000
0P2YZJT4000000@0ZJVY1`0000060:VYZ@H000000P2YZJT4000000030:VYZ@000000000000@00000
202YZJT400000080ZJVY2`0000020:VYZ@L000000P2YZJT200000080ZJVY100000020:VYZ@P00000
0`2YZJT200000080ZJVY2P0000020:VYZ@T000000P2YZJT6000000D0ZJVY0P0000020:VYZ@800000
102YZJT400000080ZJVY0`0000020:VYZ@800000102YZJT2000000cooooo000;ooooo`80ZJVY1P00
00020:VYZ@030000002YZJT0ZJVY014000000P2YZJT800000080ZJVY0P0000020:VYZ@/000000P2Y
ZJT200000080ZJVY0P0000020:VYZ@T000000P2YZJT200000080ZJVY0P0000020:VYZ@8000000`2Y
ZJT200000080ZJVY200000020:VYZ@@0000000<0ZJVY0000000000000P0000020:VYZAd000000P2Y
ZJT500000080ZJVY2P0000020:VYZ@@0000000<0ZJVY0000000000000P0000040:VYZ@8000000P2Y
ZJT400000080ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY2`0000020:VYZ@@000000P2Y
ZJT8000000030:VYZ@000000000000H0ZJVY100000020:VYZ@@0000000<0ZJVY0000000000001000
00060:VYZ@/000000P2YZJT200000080ZJVY200000050:VYZ@@000000P2YZJT6000000@0ZJVY0P00
00030:VYZ@800000202YZJT9000000@0ZJVY0P0000020:VYZ@8000002@2YZJT200000080ZJVY1000
00020:VYZ@@000000P2YZJT2000000030:VYZ@000000000000@000000P2YZJT2000000cooooo000;
ooooo`80ZJVY1P0000020:VYZ@030000002YZJT0ZJVY014000000P2YZJT800000080ZJVY0P000002
0:VYZ@/000000P2YZJT200000080ZJVY0P0000020:VYZ@T000000P2YZJT200000080ZJVY0P000002
0:VYZ@8000000`2YZJT200000080ZJVY200000020:VYZ@@0000000<0ZJVY0000000000000P000002
0:VYZAd000000P2YZJT500000080ZJVY2P0000020:VYZ@@0000000<0ZJVY0000000000000P000004
0:VYZ@8000000P2YZJT400000080ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY2`000002
0:VYZ@@000000P2YZJT8000000030:VYZ@000000000000H0ZJVY100000020:VYZ@@0000000<0ZJVY
000000000000100000060:VYZ@/000000P2YZJT200000080ZJVY200000050:VYZ@@000000P2YZJT6
000000@0ZJVY0P0000030:VYZ@800000202YZJT9000000@0ZJVY0P0000020:VYZ@8000002@2YZJT2
00000080ZJVY100000020:VYZ@@000000P2YZJT2000000030:VYZ@000000000000@000000P2YZJT2
000000cooooo000;ooooo`@0ZJVY2@0000020:VYZ@8000000P2YZJT6000000@0ZJVY1`0000020:VY
Z@8000000P2YZJT400000080ZJVY0P0000030:VYZ@P000000P2YZJT200000080ZJVY0P0000000`2Y
ZJT0000000000002000000@0ZJVY1P0000020:VYZ@8000000P2YZJT5000000H0ZJVY4@0000040:VY
ZA400000102YZJT6000000030:VYZ@000000000000@000000P2YZJT200000080ZJVY0P0000020:VY
Z@T000000P2YZJT400000080ZJVY200000050:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VY
Z@030000002YZJT0ZJVY00P000000P2YZJT400000080ZJVY2`0000020:VYZ@/000000P2YZJT40000
00@0ZJVY0P0000060:VYZ@030000002YZJT0ZJVY00@000000P2YZJT8000000<0ZJVY0P0000020:VY
Z@8000000P2YZJT6000000@0ZJVY2`00000:0:VYZ@<00000102YZJT:00000080ZJVY1`0000020:VY
Z@d00000202YZJT2000000cooooo000;ooooo`@0ZJVY2@0000020:VYZ@8000000P2YZJT6000000@0
ZJVY1`0000020:VYZ@8000000P2YZJT400000080ZJVY0P0000030:VYZ@P000000P2YZJT200000080
ZJVY0P0000000`2YZJT0000000000002000000@0ZJVY1P0000020:VYZ@8000000P2YZJT5000000H0
ZJVY4@0000040:VYZA400000102YZJT6000000030:VYZ@000000000000@000000P2YZJT200000080
ZJVY0P0000020:VYZ@T000000P2YZJT400000080ZJVY200000050:VYZ@8000000P2YZJT2000000@0
ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00P000000P2YZJT400000080ZJVY2`0000020:VYZ@/0
00000P2YZJT4000000@0ZJVY0P0000060:VYZ@030000002YZJT0ZJVY00@000000P2YZJT8000000<0
ZJVY0P0000020:VYZ@8000000P2YZJT6000000@0ZJVY2`00000:0:VYZ@<00000102YZJT:00000080
ZJVY1`0000020:VYZ@d00000202YZJT2000000cooooo000;ooooo`H000001@2YZJT200000080ZJVY
2P0000020:VYZ@<000001P2YZJT>000000<0ZJVY0P0000020:VYZ@@000000P2YZJT200000080ZJVY
2@0000020:VYZ@8000000P2YZJT8000000<0ZJVY0P0000040:VYZ@@000000P2YZJT900000080ZJVY
0P0000040:VYZA<000000P2YZJT;00000080ZJVY200000060:VYZ@D000000P2YZJT8000000H0ZJVY
0`0000020:VYZ@H00000102YZJT200000080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@@00000
0P2YZJT8000000D0ZJVY200000020:VYZ@L000000P2YZJT>00000080ZJVY00<000000:VYZ@2YZJT0
100000020:VYZ@P000000`2YZJT200000080ZJVY1P0000020:VYZ@800000102YZJT2000000030:VY
Z@000000000000@000000P2YZJT2000000T0ZJVY0P0000020:VYZ@@000000P2YZJT200000080ZJVY
1P0000030:VYZ@8000000P2YZJT2000000@0ZJVY0P0000040:VYZ@<000000P2YZJT200000080ZJVY
0P00000<ooooo`002oooool6000000D0ZJVY0P0000020:VYZ@X000000P2YZJT3000000H0ZJVY3P00
00030:VYZ@8000000P2YZJT400000080ZJVY0P0000020:VYZ@T000000P2YZJT200000080ZJVY2000
00030:VYZ@800000102YZJT400000080ZJVY2@0000020:VYZ@800000102YZJTC00000080ZJVY2`00
00020:VYZ@P000001P2YZJT500000080ZJVY200000060:VYZ@<000000P2YZJT6000000@0ZJVY0P00
00020:VYZ@030000002YZJT0ZJVY008000000P2YZJT400000080ZJVY200000050:VYZ@P000000P2Y
ZJT700000080ZJVY3P0000020:VYZ@030000002YZJT0ZJVY00@000000P2YZJT8000000<0ZJVY0P00
00020:VYZ@H000000P2YZJT2000000@0ZJVY0P0000000`2YZJT000000000000400000080ZJVY0P00
00090:VYZ@8000000P2YZJT400000080ZJVY0P0000020:VYZ@H000000`2YZJT200000080ZJVY0P00
00040:VYZ@800000102YZJT300000080ZJVY0P0000020:VYZ@8000003?ooool000_ooooo0P2YZJT2
00000080ZJVY4`0000020:VYZ@8000001@2YZJT2000000H0ZJVY0P0000020:VYZ@@0000000<0ZJVY
0000000000000P0000020:VYZ@8000000P2YZJT4000000D0ZJVY0P0000040:VYZ@@00000202YZJT7
00000080ZJVY2P0000020:VYZ@D000000P2YZJT400000080ZJVY0P0000040:VYZA4000001@2YZJT:
00000080ZJVY0P0000090:VYZ@X000000P2YZJT2000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2
000000@0ZJVY0P0000020:VYZ@030000002YZJT0ZJVY008000000P2YZJT8000000@0ZJVY0P000009
0:VYZ@@000000P2YZJT200000080ZJVY2@0000020:VYZ@/00000102YZJT200000080ZJVY20000003
0:VYZ@8000000P2YZJT200000080ZJVY1P0000040:VYZ@80000000<0ZJVY0000000000000P2YZJT2
00000080ZJVY0P0000020:VYZ@8000000P2YZJT700000080ZJVY0P0000020:VYZ@8000000P2YZJT6
000000<0ZJVY0P0000020:VYZ@P000000P2YZJT500000080ZJVY0P0000040:VYZ@cooooo000;oooo
o`80ZJVY0P0000020:VYZA<000000P2YZJT2000000D0ZJVY0P0000060:VYZ@8000000P2YZJT40000
00030:VYZ@0000000000008000000P2YZJT200000080ZJVY100000050:VYZ@800000102YZJT40000
00P0ZJVY1`0000020:VYZ@X000000P2YZJT500000080ZJVY100000020:VYZ@800000102YZJTA0000
00D0ZJVY2P0000020:VYZ@8000002@2YZJT:00000080ZJVY0P0000050:VYZ@8000000P2YZJT20000
0080ZJVY0P0000040:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY200000040:VY
Z@8000002@2YZJT400000080ZJVY0P0000020:VYZ@T000000P2YZJT;000000@0ZJVY0P0000020:VY
Z@P000000`2YZJT200000080ZJVY0P0000020:VYZ@H00000102YZJT2000000030:VYZ@0000000000
0080ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY1`0000020:VYZ@8000000P2YZJT20000
0080ZJVY1P0000030:VYZ@8000000P2YZJT800000080ZJVY1@0000020:VYZ@800000102YZJT<oooo
o`002oooool20:VYZ@P0000000<0ZJVY0000000000001P0000080:VYZ@<00000102YZJT2000000H0
ZJVY0P0000020:VYZ@L00000102YZJT200000080ZJVY3@0000020:VYZ@H00000202YZJT00`000000
ZJVY0:VYZ@0200000080ZJVY0P0000040:VYZA/000000`2YZJT:000000L0ZJVY0P0000040:VYZ@@0
00000P2YZJT200000080ZJVY0P0000030:VYZ@800000102YZJT2000000H0ZJVY0P0000050:VYZ@80
00000P2YZJT800000080ZJVY1@0000020:VYZ@800000202YZJT2000000@0ZJVY0P0000070:VYZ@@0
00000P2YZJT6000000L0ZJVY1P0000020:VYZ@8000000P2YZJT3000000@0ZJVY0P0000080:VYZ@@0
000000<0ZJVY000000000000300000040:VYZ@80000000<0ZJVY0000000000000P2YZJT200000080
ZJVY1P0000020:VYZ@8000000`2YZJT200000080ZJVY3`0000020:VYZ@800000102YZJT600000080
ZJVY2`0000020:VYZ@cooooo000;ooooo`80ZJVY200000000`2YZJT0000000000006000000P0ZJVY
0`0000040:VYZ@8000001P2YZJT200000080ZJVY1`0000040:VYZ@8000000P2YZJT=00000080ZJVY
1P0000080:VYZ@030000002YZJT0ZJVY008000000P2YZJT2000000@0ZJVY6`0000030:VYZ@X00000
1`2YZJT2000000@0ZJVY100000020:VYZ@8000000P2YZJT2000000<0ZJVY0P0000040:VYZ@800000
1P2YZJT2000000D0ZJVY0P0000020:VYZ@P000000P2YZJT500000080ZJVY0P0000080:VYZ@800000
102YZJT2000000L0ZJVY100000020:VYZ@H000001`2YZJT600000080ZJVY0P0000020:VYZ@<00000
102YZJT2000000P0ZJVY100000000`2YZJT000000000000<000000@0ZJVY0P0000000`2YZJT00000
000000020:VYZ@8000000P2YZJT600000080ZJVY0P0000030:VYZ@8000000P2YZJT?00000080ZJVY
0P0000040:VYZ@H000000P2YZJT;00000080ZJVY3?ooool000_ooooo2P0000000`2YZJT000000000
00020:VYZ@@000001P2YZJT200000080ZJVY1`0000020:VYZ@H000001P2YZJT500000080ZJVY1P00
00020:VYZ@D000000P2YZJT8000000H0ZJVY2@0000020:VYZ@8000000P2YZJT4000000@0ZJVY3`00
00020:VYZ@/000003@2YZJT2000000@0ZJVY300000030:VYZ@P00000102YZJT4000000@0ZJVY0`00
00080:VYZ@H000000`2YZJT6000000H0ZJVY2`0000020:VYZ@l00000102YZJT6000000@0ZJVY1`00
00020:VYZ@H00000102YZJT9000000@0ZJVY200000040:VYZ@D000000P2YZJT200000080ZJVY1000
00020:VYZ@@0000000<0ZJVY0000000000001P0000020:VYZ@X0000000<0ZJVY0000000000000P00
00040:VYZ@8000000P2YZJT200000080ZJVY1`0000020:VYZ@8000000P2YZJT<ooooo`002oooool:
000000030:VYZ@00000000000080ZJVY100000060:VYZ@8000000P2YZJT700000080ZJVY1P000006
0:VYZ@D000000P2YZJT600000080ZJVY1@0000020:VYZ@P000001P2YZJT900000080ZJVY0P000002
0:VYZ@@00000102YZJT?00000080ZJVY2`00000=0:VYZ@800000102YZJT<000000<0ZJVY20000004
0:VYZ@@00000102YZJT3000000P0ZJVY1P0000030:VYZ@H000001P2YZJT;00000080ZJVY3`000004
0:VYZ@H00000102YZJT700000080ZJVY1P0000040:VYZ@T00000102YZJT8000000@0ZJVY1@000002
0:VYZ@8000000P2YZJT400000080ZJVY100000000`2YZJT000000000000600000080ZJVY2P000000
0`2YZJT0000000000002000000@0ZJVY0P0000020:VYZ@8000000P2YZJT700000080ZJVY0P000002
0:VYZ@cooooo000;ooooo`P0ZJVY0P0000000`2YZJT000000000000400000080ZJVY0P0000020:VY
Z@L00000102YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT300000080ZJVY0P0000020:VY
Z@8000000P2YZJT200000080ZJVY1@0000020:VYZ@@00000102YZJT2000000H0ZJVY2`0000020:VY
Z@@000000P2YZJT=00000080ZJVY0P0000020:VYZ@D000004P2YZJT500000080ZJVY0P0000020:VY
Z@@000000P2YZJT4000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@@000000P2YZJT30000
00P0ZJVY0P0000040:VYZ@8000000`2YZJT800000080ZJVY2P0000000`2YZJT00000000000020:VY
Z@`000000P2YZJT00`000000ZJVY0:VYZ@0:00000080ZJVY0P0000020:VYZ@L000000P2YZJT:0000
00<0ZJVY0P0000040:VYZ@800000102YZJT400000080ZJVY0`0000020:VYZ@H000000P2YZJT80000
00030:VYZ@00000000000080ZJVY2P0000020:VYZ@@0000000<0ZJVY000000000000200000020:VY
Z@8000000P2YZJT=000000cooooo000;ooooo`P0ZJVY0P0000000`2YZJT000000000000400000080
ZJVY0P0000020:VYZ@L00000102YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT300000080
ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY1@0000020:VYZ@@00000102YZJT2000000H0
ZJVY2`0000020:VYZ@@000000P2YZJT=00000080ZJVY0P0000020:VYZ@D000004P2YZJT500000080
ZJVY0P0000020:VYZ@@000000P2YZJT4000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@@0
00000P2YZJT3000000P0ZJVY0P0000040:VYZ@8000000`2YZJT800000080ZJVY2P0000000`2YZJT0
0000000000020:VYZ@`000000P2YZJT00`000000ZJVY0:VYZ@0:00000080ZJVY0P0000020:VYZ@L0
00000P2YZJT:000000<0ZJVY0P0000040:VYZ@800000102YZJT400000080ZJVY0`0000020:VYZ@H0
00000P2YZJT8000000030:VYZ@00000000000080ZJVY2P0000020:VYZ@@0000000<0ZJVY00000000
0000200000020:VYZ@8000000P2YZJT=000000cooooo000;ooooo`@0ZJVY1`0000020:VYZ@P00000
202YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT800000080ZJVY0`0000020:VYZ@H00000
0P2YZJT400000080ZJVY0`0000020:VYZ@X000000P2YZJT200000080ZJVY0P0000030:VYZ@800000
0P2YZJT800000080ZJVY0P0000000`2YZJT000000000000400000080ZJVY200000020:VYZ@800000
4@2YZJT700000080ZJVY4@0000060:VYZ@`000002@2YZJT8000000<0ZJVY0P0000040:VYZ@P00000
102YZJT5000000@0ZJVY0P0000040:VYZ@8000000P2YZJT7000000H0ZJVY1P0000040:VYZ@030000
002YZJT0ZJVY00X000000P2YZJT4000000030:VYZ@000000000000@0ZJVY0P0000040:VYZ@@00000
0P2YZJT;00000080ZJVY0P0000020:VYZ@8000000`2YZJT200000080ZJVY0P0000040:VYZ@X00000
00<0ZJVY0000000000000P2YZJT600000080ZJVY100000030:VYZ@@000001P2YZJT<ooooo`002ooo
ool40:VYZ@L000000P2YZJT8000000P0ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY2000
00020:VYZ@<000000P2YZJT600000080ZJVY100000020:VYZ@<000000P2YZJT:00000080ZJVY0P00
00020:VYZ@8000000`2YZJT200000080ZJVY200000020:VYZ@80000000<0ZJVY0000000000001000
00020:VYZ@P000000P2YZJT200000140ZJVY1`0000020:VYZA4000001P2YZJT<000000T0ZJVY2000
00030:VYZ@800000102YZJT8000000@0ZJVY1@0000040:VYZ@800000102YZJT200000080ZJVY1`00
00060:VYZ@H00000102YZJT00`000000ZJVY0:VYZ@0:00000080ZJVY100000000`2YZJT000000000
00040:VYZ@800000102YZJT400000080ZJVY2`0000020:VYZ@8000000P2YZJT2000000<0ZJVY0P00
00020:VYZ@800000102YZJT:000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@@000000`2Y
ZJT4000000H0ZJVY3?ooool000_ooooo102YZJT700000080ZJVY0P0000020:VYZ@H000000P2YZJT7
00000080ZJVY100000060:VYZ@L000000P2YZJTG000000H0ZJVY100000020:VYZ@<000000P2YZJT4
000000X0ZJVY1@0000020:VYZ@X00000102YZJT2000000l0ZJVY2@0000020:VYZ@@000000P2YZJT8
000000<0ZJVY0P0000080:VYZ@8000003`2YZJT200000080ZJVY100000030:VYZ@800000102YZJT2
000000@0ZJVY0P0000020:VYZ@T000000P2YZJT200000080ZJVY100000020:VYZ@8000001@2YZJT2
000000@0ZJVY0P0000020:VYZ@800000102YZJT3000000P0ZJVY0P0000020:VYZ@8000000`2YZJT8
000000@0ZJVY200000030:VYZ@X000000P2YZJT4000000030:VYZ@00000000000080ZJVY20000002
0:VYZ@8000000P2YZJT2000000030:VYZ@000000000000800000102YZJT=000000P0ZJVY3?ooool0
00_ooooo1P2YZJT500000080ZJVY0P0000040:VYZ@@000000P2YZJT200000080ZJVY0`0000020:VY
Z@H000000P2YZJT200000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY014000000P2YZJT20000
0080ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY0`0000020:VYZ@H000001P2YZJT20000
0080ZJVY0`0000020:VYZ@@000000P2YZJT4000000@0ZJVY0P00000F0:VYZ@8000000P2YZJT80000
00@0ZJVY0`0000020:VYZ@@000001P2YZJT2000000/0ZJVY1P0000040:VYZ@8000000P2YZJT50000
0080ZJVY0P0000020:VYZ@X000001@2YZJT4000000@0ZJVY1P0000030:VYZ@H000000P2YZJT60000
00@0ZJVY00<000000:VYZ@2YZJT00P0000060:VYZ@H000000P2YZJT500000080ZJVY100000020:VY
Z@@000000P2YZJT7000000@0ZJVY200000030:VYZ@`000000P2YZJT200000080ZJVY0P0000000`2Y
ZJT000000000000200000080ZJVY2P00000=0:VYZ@cooooo000;ooooo`H0ZJVY1@0000020:VYZ@80
0000102YZJT400000080ZJVY0P0000020:VYZ@<000000P2YZJT600000080ZJVY0P0000020:VYZ@80
00000P2YZJT00`000000ZJVY0:VYZ@0A00000080ZJVY0P0000020:VYZ@8000000P2YZJT200000080
ZJVY0P0000020:VYZ@<000000P2YZJT6000000H0ZJVY0P0000020:VYZ@<000000P2YZJT400000080
ZJVY100000040:VYZ@8000005P2YZJT200000080ZJVY200000040:VYZ@<000000P2YZJT4000000H0
ZJVY0P00000;0:VYZ@H00000102YZJT200000080ZJVY1@0000020:VYZ@8000000P2YZJT:000000D0
ZJVY100000040:VYZ@H000000`2YZJT600000080ZJVY1P0000040:VYZ@030000002YZJT0ZJVY0080
00001P2YZJT600000080ZJVY1@0000020:VYZ@@000000P2YZJT400000080ZJVY1`0000040:VYZ@P0
00000`2YZJT<00000080ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@X0
00003@2YZJT<ooooo`002oooool40:VYZ@@000000P2YZJT700000080ZJVY2P0000000`2YZJT00000
00000006000000@0ZJVY0P0000020:VYZ@8000000P2YZJT5000000@0ZJVY200000020:VYZ@T00000
0P2YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT2000000H0ZJVY0P0000020:VYZ@d00000
0P2YZJT400000080ZJVY100000000`2YZJT000000000000@0:VYZ@030000002YZJT0ZJVY00@00000
0P2YZJTG00000080ZJVY1P0000070:VYZ@P000000P2YZJT2000000<0ZJVY0P0000020:VYZ@P00000
0P2YZJT400000080ZJVY00<000000:VYZ@2YZJT0102YZJT800000080ZJVY1`0000020:VYZ@800000
0P2YZJT600000080ZJVY2@0000080:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY
100000020:VYZ@@000000P2YZJT300000080ZJVY2P0000020:VYZ@8000000P2YZJT300000080ZJVY
0P0000040:VYZ@X000000`2YZJT200000080ZJVY200000020:VYZ@8000002`2YZJT<ooooo`002ooo
ool40:VYZ@@000000P2YZJT700000080ZJVY2P0000000`2YZJT0000000000006000000@0ZJVY0P00
00020:VYZ@8000000P2YZJT5000000@0ZJVY200000020:VYZ@T000000P2YZJT200000080ZJVY0P00
00020:VYZ@<000000P2YZJT2000000H0ZJVY0P0000020:VYZ@d000000P2YZJT400000080ZJVY1000
00000`2YZJT000000000000@0:VYZ@030000002YZJT0ZJVY00@000000P2YZJTG00000080ZJVY1P00
00070:VYZ@P000000P2YZJT2000000<0ZJVY0P0000020:VYZ@P000000P2YZJT400000080ZJVY00<0
00000:VYZ@2YZJT0102YZJT800000080ZJVY1`0000020:VYZ@8000000P2YZJT600000080ZJVY2@00
00080:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY100000020:VYZ@@000000P2Y
ZJT300000080ZJVY2P0000020:VYZ@8000000P2YZJT300000080ZJVY0P0000040:VYZ@X000000`2Y
ZJT200000080ZJVY200000020:VYZ@8000002`2YZJT<ooooo`002oooool20:VYZ@8000000P2YZJT7
00000080ZJVY3P0000000`2YZJT00000000000040:VYZ@8000000P2YZJT400000080ZJVY10000000
0`2YZJT000000000000400000080ZJVY0P0000020:VYZ@T000001P2YZJT8000000L0ZJVY0P000004
0:VYZ@X0000000<0ZJVY000000000000300000020:VYZ@@0000000<0ZJVY0000000000001000000<
0:VYZ@d000000P2YZJT=00000080ZJVY0P0000020:VYZ@8000000P2YZJT4000000<0ZJVY0P000002
0:VYZ@H000000P2YZJT4000000030:VYZ@000000000000800000102YZJT400000080ZJVY10000002
0:VYZ@030000002YZJT0ZJVY008000000P2YZJT400000080ZJVY0P0000020:VYZ@d000000P2YZJT8
000000L0ZJVY0P0000040:VYZ@/000000P2YZJT400000080ZJVY100000020:VYZ@T000000P2YZJT8
00000080ZJVY0`0000020:VYZA4000000P2YZJT2000000H0ZJVY0P0000020:VYZ@@000002`2YZJT<
ooooo`002oooool20:VYZ@8000000P2YZJT700000080ZJVY3P0000000`2YZJT00000000000040:VY
Z@8000000P2YZJT400000080ZJVY100000000`2YZJT000000000000400000080ZJVY0P0000020:VY
Z@T000001P2YZJT8000000L0ZJVY0P0000040:VYZ@X0000000<0ZJVY000000000000300000020:VY
Z@@0000000<0ZJVY0000000000001000000<0:VYZ@d000000P2YZJT=00000080ZJVY0P0000020:VY
Z@8000000P2YZJT4000000<0ZJVY0P0000020:VYZ@H000000P2YZJT4000000030:VYZ@0000000000
00800000102YZJT400000080ZJVY100000020:VYZ@030000002YZJT0ZJVY008000000P2YZJT40000
0080ZJVY0P0000020:VYZ@d000000P2YZJT8000000L0ZJVY0P0000040:VYZ@/000000P2YZJT40000
0080ZJVY100000020:VYZ@T000000P2YZJT800000080ZJVY0`0000020:VYZA4000000P2YZJT20000
00H0ZJVY0P0000020:VYZ@@000002`2YZJT<ooooo`002oooool800000080ZJVY0`0000020:VYZ@X0
00000P2YZJT;00000080ZJVY100000020:VYZ@/000000P2YZJT8000000T0ZJVY200000060:VYZ@/0
00001P2YZJT2000000030:VYZ@0000000000008000000P2YZJT2000000@0ZJVY3@00000A0:VYZ@@0
0000102YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT400000080ZJVY0P0000020:VYZ@80
00000P2YZJT9000000H0ZJVY3@0000020:VYZ@`000000P2YZJT00`000000ZJVY0:VYZ@04000000@0
ZJVY100000020:VYZ@L00000102YZJT8000000/0ZJVY0P0000040:VYZ@8000000P2YZJT300000080
ZJVY3P0000020:VYZ@T000000P2YZJT400000080ZJVY2`0000020:VYZ@P000000P2YZJT300000080
ZJVY0P0000060:VYZ@8000000P2YZJT4000000/0ZJVY3?ooool000_ooooo200000020:VYZ@<00000
0P2YZJT:00000080ZJVY2`0000020:VYZ@@000000P2YZJT;00000080ZJVY200000090:VYZ@P00000
1P2YZJT;000000H0ZJVY0P0000000`2YZJT000000000000200000080ZJVY0P0000040:VYZ@d00000
4@2YZJT4000000@0ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY100000020:VYZ@800000
0P2YZJT200000080ZJVY2@0000060:VYZ@d000000P2YZJT<00000080ZJVY00<000000:VYZ@2YZJT0
100000040:VYZ@@000000P2YZJT7000000@0ZJVY2000000;0:VYZ@800000102YZJT200000080ZJVY
0`0000020:VYZ@h000000P2YZJT900000080ZJVY100000020:VYZ@/000000P2YZJT800000080ZJVY
0`0000020:VYZ@8000001P2YZJT200000080ZJVY1000000;0:VYZ@cooooo000;ooooo`800000102Y
ZJT200000080ZJVY3`0000020:VYZ@8000001`2YZJT200000080ZJVY2P0000050:VYZ@8000000P2Y
ZJT200000080ZJVY0P0000090:VYZ@8000000P2YZJT6000000/0ZJVY0P0000040:VYZ@800000102Y
ZJT2000000030:VYZ@0000000000008000000P2YZJT400000080ZJVY200000030:VYZ@@000003`2Y
ZJT>00000080ZJVY1`0000040:VYZ@H000000P2YZJT4000000<0ZJVY0P0000060:VYZ@@000000P2Y
ZJTA000000@0ZJVY2`0000020:VYZ@@000000P2YZJT300000080ZJVY0P0000040:VYZ@X000002@2Y
ZJT2000000@0ZJVY0P0000020:VYZ@<000000P2YZJT2000000@0ZJVY100000020:VYZ@8000000P2Y
ZJT2000000D0ZJVY0P0000020:VYZ@@000000P2YZJT4000000030:VYZ@00000000000080ZJVY0P00
00020:VYZ@@000000P2YZJT200000080ZJVY1`0000040:VYZ@@000000P2YZJT2000000d0ZJVY3?oo
ool000_ooooo0P0000040:VYZ@8000000P2YZJT?00000080ZJVY0P0000070:VYZ@8000000P2YZJT:
000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2000000T0ZJVY0P0000020:VYZ@H000002`2YZJT2
000000@0ZJVY0P0000040:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@@000000P2YZJT8
000000<0ZJVY1000000?0:VYZ@h000000P2YZJT7000000@0ZJVY1P0000020:VYZ@@000000`2YZJT2
000000H0ZJVY100000020:VYZA400000102YZJT;00000080ZJVY100000020:VYZ@<000000P2YZJT2
000000@0ZJVY2P0000090:VYZ@800000102YZJT200000080ZJVY0`0000020:VYZ@800000102YZJT4
00000080ZJVY0P0000020:VYZ@8000001@2YZJT200000080ZJVY100000020:VYZ@@0000000<0ZJVY
0000000000000P2YZJT200000080ZJVY100000020:VYZ@8000000P2YZJT7000000@0ZJVY10000002
0:VYZ@8000003@2YZJT<ooooo`002oooool200000080ZJVY100000020:VYZ@030000002YZJT0ZJVY
0080ZJVY100000020:VYZ@8000000P2YZJT4000000030:VYZ@000000000000@0ZJVY0P0000040:VY
Z@800000102YZJT2000000<0ZJVY300000020:VYZ@80000000<0ZJVY0000000000000P2YZJT20000
00H0ZJVY0P00000;0:VYZ@P00000102YZJT=00000080ZJVY3`00000?0:VYZ@@000000P2YZJT40000
0080ZJVY1P0000000`2YZJT0000000000002000000@0ZJVY0P0000020:VYZ@P000000`2YZJT20000
0080ZJVY200000020:VYZ@<000000P2YZJT800000080ZJVY200000000`2YZJT00000000000020000
0080ZJVY3@0000020:VYZ@@000000P2YZJT200000080ZJVY0P00000;0:VYZA<00000102YZJT40000
0080ZJVY1`0000020:VYZ@X000000P2YZJT200000080ZJVY4@0000070:VYZ@@000000P2YZJT20000
00@0ZJVY1@0000080:VYZ@8000003?ooool000_ooooo0P0000020:VYZ@@000000P2YZJT00`000000
ZJVY0:VYZ@020:VYZ@@000000P2YZJT200000080ZJVY100000000`2YZJT00000000000040:VYZ@80
0000102YZJT2000000@0ZJVY0P0000030:VYZ@`000000P2YZJT2000000030:VYZ@00000000000080
ZJVY0P0000060:VYZ@8000002`2YZJT8000000@0ZJVY3@0000020:VYZ@l000003`2YZJT400000080
ZJVY100000020:VYZ@H0000000<0ZJVY0000000000000P0000040:VYZ@8000000P2YZJT8000000<0
ZJVY0P0000020:VYZ@P000000P2YZJT300000080ZJVY200000020:VYZ@P0000000<0ZJVY00000000
00000P0000020:VYZ@d000000P2YZJT400000080ZJVY0P0000020:VYZ@8000002`2YZJTC000000@0
ZJVY100000020:VYZ@L000000P2YZJT:00000080ZJVY0P0000020:VYZA4000001`2YZJT400000080
ZJVY0P0000040:VYZ@D00000202YZJT2000000cooooo000;ooooo`8000000P2YZJT200000080ZJVY
1@0000020:VYZ@8000000P2YZJT4000000H0ZJVY0`0000040:VYZ@X000000P2YZJT300000080ZJVY
200000060:VYZ@80000000<0ZJVY000000000000100000060:VYZ@@000001`2YZJT@000000<0ZJVY
0P0000040:VYZ@H00000102YZJT7000000l0ZJVY0P0000020:VYZ@P000000P2YZJT6000000030:VY
Z@000000000000@000000P2YZJT400000080ZJVY1`0000020:VYZ@H00000102YZJT200000080ZJVY
0`0000020:VYZ@P000000P2YZJT200000080ZJVY2@0000020:VYZ@800000102YZJT2000000@0ZJVY
1@0000020:VYZ@`000001@2YZJT4000000@0ZJVY200000000`2YZJT000000000000G000000@0ZJVY
0P0000020:VYZ@L000000P2YZJT8000000@0ZJVY100000090:VYZ@H000000P2YZJT5000000H0ZJVY
0P0000020:VYZ@cooooo000;ooooo`8000000P2YZJT200000080ZJVY1@0000020:VYZ@8000000P2Y
ZJT4000000H0ZJVY0`0000040:VYZ@X000000P2YZJT300000080ZJVY200000060:VYZ@80000000<0
ZJVY000000000000100000060:VYZ@@000001`2YZJT@000000<0ZJVY0P0000040:VYZ@H00000102Y
ZJT7000000l0ZJVY0P0000020:VYZ@P000000P2YZJT6000000030:VYZ@000000000000@000000P2Y
ZJT400000080ZJVY1`0000020:VYZ@H00000102YZJT200000080ZJVY0`0000020:VYZ@P000000P2Y
ZJT200000080ZJVY2@0000020:VYZ@800000102YZJT2000000@0ZJVY1@0000020:VYZ@`000001@2Y
ZJT4000000@0ZJVY200000000`2YZJT000000000000G000000@0ZJVY0P0000020:VYZ@L000000P2Y
ZJT8000000@0ZJVY100000090:VYZ@H000000P2YZJT5000000H0ZJVY0P0000020:VYZ@cooooo000;
ooooo`P000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY0P0000020:VYZ@8000001@2YZJT2
000000@0ZJVY0P0000020:VYZA4000001P2YZJT200000080ZJVY0P00000?0:VYZ@@000000`2YZJT2
000000H0ZJVY2P0000000`2YZJT000000000000200000080ZJVY100000020:VYZ@8000000P2YZJT5
000000@0ZJVY0P00000:0:VYZ@80000000<0ZJVY0000000000001P0000020:VYZ@X0000000<0ZJVY
000000000000100000020:VYZ@X000000P2YZJT;00000080ZJVY0P0000020:VYZ@T00000102YZJT:
000000<0ZJVY0P0000020:VYZ@8000000P2YZJT4000000@0ZJVY1@0000020:VYZ@P00000102YZJT2
000000030:VYZ@00000000000080ZJVY0P0000020:VYZ@d000000P2YZJT@000000030:VYZ@000000
000000@000000P2YZJT?00000080ZJVY100000040:VYZ@@000003@2YZJT200000080ZJVY0P000002
0:VYZ@<000000P2YZJT6000000cooooo000;ooooo`P000000P2YZJT00`000000ZJVY0:VYZ@040000
0080ZJVY0P0000020:VYZ@8000001@2YZJT2000000@0ZJVY0P0000020:VYZA4000001P2YZJT20000
0080ZJVY0P00000?0:VYZ@@000000`2YZJT2000000H0ZJVY2P0000000`2YZJT00000000000020000
0080ZJVY100000020:VYZ@8000000P2YZJT5000000@0ZJVY0P00000:0:VYZ@80000000<0ZJVY0000
000000001P0000020:VYZ@X0000000<0ZJVY000000000000100000020:VYZ@X000000P2YZJT;0000
0080ZJVY0P0000020:VYZ@T00000102YZJT:000000<0ZJVY0P0000020:VYZ@8000000P2YZJT40000
00@0ZJVY1@0000020:VYZ@P00000102YZJT2000000030:VYZ@00000000000080ZJVY0P0000020:VY
Z@d000000P2YZJT@000000030:VYZ@000000000000@000000P2YZJT?00000080ZJVY100000040:VY
Z@@000003@2YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT6000000cooooo000;ooooo`@0
0000102YZJT300000080ZJVY100000020:VYZ@8000000P2YZJT900000080ZJVY100000020:VYZ@80
0000102YZJT5000000X0ZJVY2@00000<0:VYZ@H000000`2YZJT2000000H0ZJVY0P0000040:VYZ@@0
000000<0ZJVY000000000000200000020:VYZA<000001P2YZJT2000000<0ZJVY0P0000020:VYZ@H0
00000P2YZJT200000080ZJVY0P0000000`2YZJT00000000000020:VYZ@8000000P2YZJT200000080
ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT200000080
ZJVY100000070:VYZ@800000102YZJT2000000H0ZJVY1`0000020:VYZ@8000000P2YZJT400000080
ZJVY0`0000020:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@80000000<0ZJVY00000000
00001P0000040:VYZ@l000000P2YZJT400000080ZJVY600000020:VYZ@8000000P2YZJT400000080
ZJVY2000000;0:VYZ@P0000000<0ZJVY0000000000002000000<ooooo`002oooool4000000@0ZJVY
0`0000020:VYZ@@000000P2YZJT200000080ZJVY2@0000020:VYZ@@000000P2YZJT2000000@0ZJVY
1@00000:0:VYZ@T00000302YZJT6000000<0ZJVY0P0000060:VYZ@800000102YZJT4000000030:VY
Z@000000000000P000000P2YZJTC000000H0ZJVY0P0000030:VYZ@8000000P2YZJT600000080ZJVY
0P0000020:VYZ@80000000<0ZJVY0000000000000P2YZJT200000080ZJVY0P0000020:VYZ@800000
0P2YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY0P0000020:VYZ@@00000
1`2YZJT2000000@0ZJVY0P0000060:VYZ@L000000P2YZJT200000080ZJVY100000020:VYZ@<00000
0P2YZJT200000080ZJVY100000020:VYZ@@000000P2YZJT2000000030:VYZ@000000000000H00000
102YZJT?00000080ZJVY100000020:VYZAP000000P2YZJT200000080ZJVY100000020:VYZ@P00000
2`2YZJT8000000030:VYZ@000000000000P000003?ooool000_ooooo100000020:VYZ@T00000102Y
ZJT600000080ZJVY2@0000020:VYZ@l000003P2YZJT5000000X0ZJVY1P0000030:VYZ@8000001P2Y
ZJT400000080ZJVY100000000`2YZJT000000000000200000080ZJVY100000020:VYZ@800000102Y
ZJT700000080ZJVY0P0000040:VYZ@8000000P2YZJT2000000030:VYZ@000000000000@00000102Y
ZJT600000080ZJVY2@0000020:VYZ@8000000P2YZJT200000080ZJVY2`0000020:VYZ@8000002@2Y
ZJT600000080ZJVY100000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@040:VYZ@P000000P2Y
ZJT2000000<0ZJVY100000020:VYZ@H00000102YZJT2000000030:VYZ@00000000000080ZJVY1000
00040:VYZ@T000000P2YZJT400000080ZJVY2P0000000`2YZJT000000000000>00000080ZJVY00<0
00000:VYZ@2YZJT00P0000040:VYZ@8000000P2YZJT2000000@0ZJVY0P00000A0:VYZ@80000000<0
ZJVY000000000000102YZJT4000000cooooo000;ooooo`80ZJVY1P0000020:VYZ@030000002YZJT0
ZJVY00800000102YZJT200000080ZJVY1`0000020:VYZ@X000000P2YZJT6000000030:VYZ@000000
000000`0ZJVY1@0000020:VYZ@8000001P2YZJT200000080ZJVY2@0000040:VYZ@H000000P2YZJT4
000000030:VYZ@000000000000H00000102YZJT400000080ZJVY0`0000040:VYZ@X000000P2YZJT3
000000@0ZJVY0P0000020:VYZ@P000000P2YZJT3000000P0ZJVY0P0000020:VYZ@800000102YZJT2
000000<0ZJVY0P0000020:VYZ@@000002@2YZJT200000080ZJVY200000020:VYZ@@000000P2YZJT0
0`000000ZJVY0:VYZ@020:VYZ@8000002P2YZJT2000000<0ZJVY100000020:VYZ@H000000P2YZJT9
00000080ZJVY0P0000020:VYZ@@000000P2YZJT2000000030:VYZ@00000000000080ZJVY10000002
0:VYZ@8000000P2YZJTA00000080ZJVY100000020:VYZ@030000002YZJT0ZJVY00800000102YZJT<
000000d0ZJVY0P0000020:VYZ@d000003?ooool000_ooooo0P2YZJT600000080ZJVY00<000000:VY
Z@2YZJT00P0000040:VYZ@8000000P2YZJT700000080ZJVY2P0000020:VYZ@H0000000<0ZJVY0000
00000000302YZJT500000080ZJVY0P0000060:VYZ@8000000P2YZJT9000000@0ZJVY1P0000020:VY
Z@@0000000<0ZJVY0000000000001P0000040:VYZ@@000000P2YZJT3000000@0ZJVY2P0000020:VY
Z@<00000102YZJT200000080ZJVY200000020:VYZ@<00000202YZJT200000080ZJVY0P0000040:VY
Z@8000000`2YZJT200000080ZJVY100000090:VYZ@8000000P2YZJT800000080ZJVY100000020:VY
Z@030000002YZJT0ZJVY0080ZJVY0P00000:0:VYZ@8000000`2YZJT400000080ZJVY1P0000020:VY
Z@T000000P2YZJT200000080ZJVY100000020:VYZ@80000000<0ZJVY0000000000000P2YZJT40000
0080ZJVY0P0000020:VYZA4000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P0000040:VY
Z@`000003@2YZJT200000080ZJVY3@00000<ooooo`002oooool4000000@0ZJVY1`0000020:VYZ@@0
00000P2YZJT;000000H0ZJVY0P0000020:VYZ@8000001@2YZJT2000000`0ZJVY1@0000020:VYZ@80
00000P2YZJT200000080ZJVY0P0000040:VYZ@@0000000<0ZJVY0000000000001P0000020:VYZ@P0
000000<0ZJVY000000000000102YZJT400000080ZJVY2`0000020:VYZ@@00000102YZJT700000080
ZJVY3P0000020:VYZ@<00000202YZJT6000000@0ZJVY1`0000040:VYZ@@000001`2YZJT600000080
ZJVY2P0000020:VYZ@<000000P2YZJT200000080ZJVY0P0000060:VYZ@T000000P2YZJT?000000@0
ZJVY0P0000060:VYZ@T000000P2YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT900000080
ZJVY0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY0P0000040:VYZ@H0
00002@2YZJT600000080ZJVY0P0000000`2YZJT000000000000200000080ZJVY1000000<ooooo`00
2oooool4000000@0ZJVY1`0000020:VYZ@@000000P2YZJT;000000H0ZJVY0P0000020:VYZ@800000
1@2YZJT2000000`0ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@@00000
00<0ZJVY0000000000001P0000020:VYZ@P0000000<0ZJVY000000000000102YZJT400000080ZJVY
2`0000020:VYZ@@00000102YZJT700000080ZJVY3P0000020:VYZ@<00000202YZJT6000000@0ZJVY
1`0000040:VYZ@@000001`2YZJT600000080ZJVY2P0000020:VYZ@<000000P2YZJT200000080ZJVY
0P0000060:VYZ@T000000P2YZJT?000000@0ZJVY0P0000060:VYZ@T000000P2YZJT200000080ZJVY
0P0000020:VYZ@8000000P2YZJT900000080ZJVY0P0000020:VYZ@@000000P2YZJT00`000000ZJVY
0:VYZ@0400000080ZJVY0P0000040:VYZ@H000002@2YZJT600000080ZJVY0P0000000`2YZJT00000
0000000200000080ZJVY1000000<ooooo`002oooool20:VYZ@@000000P2YZJT=00000080ZJVY0P00
00020:VYZ@<000000P2YZJT4000000@0ZJVY1P0000040:VYZ@<000000P2YZJT2000000H0ZJVY0P00
00040:VYZ@T000000P2YZJT200000080ZJVY2@0000040:VYZ@8000000P2YZJT4000000@0ZJVY1@00
00020:VYZ@@000000P2YZJT200000080ZJVY0P0000030:VYZ@h000000P2YZJT900000080ZJVY3`00
00080:VYZ@X000000`2YZJT200000080ZJVY100000020:VYZ@800000102YZJT00`000000ZJVY0:VY
Z@020:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@030000002YZJT0ZJVY00@000000P2Y
ZJT2000000H0ZJVY2@0000020:VYZ@800000102YZJT4000000<0ZJVY200000020:VYZ@8000000P2Y
ZJT4000000030:VYZ@0000000000008000000P2YZJT200000080ZJVY1P0000020:VYZ@<000000P2Y
ZJT2000000@0ZJVY100000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2Y
ZJT800000080ZJVY0P0000070:VYZ@8000000P2YZJT200000080ZJVY100000030:VYZ@8000000P2Y
ZJT4000000cooooo000;ooooo`80ZJVY100000020:VYZ@d000000P2YZJT200000080ZJVY0`000002
0:VYZ@@00000102YZJT6000000@0ZJVY0`0000020:VYZ@8000001P2YZJT2000000@0ZJVY2@000002
0:VYZ@8000000P2YZJT9000000@0ZJVY0P0000020:VYZ@@00000102YZJT500000080ZJVY10000002
0:VYZ@8000000P2YZJT2000000<0ZJVY3P0000020:VYZ@T000000P2YZJT?000000P0ZJVY2P000003
0:VYZ@8000000P2YZJT400000080ZJVY0P0000040:VYZ@030000002YZJT0ZJVY0080ZJVY0P000002
0:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT0100000020:VYZ@8000001P2YZJT9
00000080ZJVY0P0000040:VYZ@@000000`2YZJT800000080ZJVY0P0000020:VYZ@@0000000<0ZJVY
0000000000000P0000020:VYZ@8000000P2YZJT600000080ZJVY0`0000020:VYZ@800000102YZJT4
00000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY0080ZJVY0P0000020:VYZ@P000000P2YZJT2
000000L0ZJVY0P0000020:VYZ@8000000P2YZJT4000000<0ZJVY0P0000020:VYZ@@000003?ooool0
00_ooooo0P2YZJT400000080ZJVY1`0000040:VYZ@8000000P2YZJT?00000080ZJVY0P0000020:VY
Z@L00000102YZJT2000000@0ZJVY100000040:VYZ@030000002YZJT0ZJVY00H0ZJVY100000040:VY
Z@8000000`2YZJT2000000@0ZJVY100000020:VYZ@800000102YZJTJ00000080ZJVY0P0000040:VY
Z@L00000102YZJT800000080ZJVY2@0000060:VYZ@8000001P2YZJT4000000<0ZJVY0P0000040:VY
Z@8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY100000020:VY
Z@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT2000000@0ZJVY0P0000030:VY
Z@8000000P2YZJT200000080ZJVY100000020:VYZ@H0000000<0ZJVY000000000000100000040:VY
Z@@000000P2YZJT5000000@0ZJVY2P0000020:VYZ@80000000<0ZJVY0000000000000P0000020:VY
Z@X000000P2YZJT7000000@0ZJVY2P0000030:VYZ@`000000P2YZJT2000000030:VYZ@0000000000
00P000003?ooool000_ooooo0P2YZJT400000080ZJVY1`0000040:VYZ@8000000P2YZJT?00000080
ZJVY0P0000020:VYZ@L00000102YZJT2000000@0ZJVY100000040:VYZ@030000002YZJT0ZJVY00H0
ZJVY100000040:VYZ@8000000`2YZJT2000000@0ZJVY100000020:VYZ@800000102YZJTJ00000080
ZJVY0P0000040:VYZ@L00000102YZJT800000080ZJVY2@0000060:VYZ@8000001P2YZJT4000000<0
ZJVY0P0000040:VYZ@8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080
ZJVY100000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT2000000@0
ZJVY0P0000030:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@H0000000<0ZJVY00000000
0000100000040:VYZ@@000000P2YZJT5000000@0ZJVY2P0000020:VYZ@80000000<0ZJVY00000000
00000P0000020:VYZ@X000000P2YZJT7000000@0ZJVY2P0000030:VYZ@`000000P2YZJT200000003
0:VYZ@000000000000P000003?ooool000_ooooo1P0000020:VYZ@d000000P2YZJT400000080ZJVY
0`0000020:VYZ@@000000P2YZJT400000080ZJVY0P0000020:VYZ@T000000P2YZJT4000000@0ZJVY
1@0000040:VYZ@H000000P2YZJT2000000<0ZJVY0P0000040:VYZ@@000000P2YZJT400000080ZJVY
00<000000:VYZ@2YZJT00P2YZJTC000000@0ZJVY0P0000020:VYZA4000000P2YZJT800000080ZJVY
00<000000:VYZ@2YZJT07@0000020:VYZ@@000000P2YZJT300000080ZJVY0P0000040:VYZ@800000
0P2YZJT200000080ZJVY0`0000040:VYZ@P000000P2YZJT700000080ZJVY0P0000020:VYZ@@00000
0P2YZJT900000080ZJVY0P0000040:VYZ@P0000000<0ZJVY0000000000000P2YZJT600000080ZJVY
0P0000020:VYZ@@000000`2YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT=00000080ZJVY
0P0000040:VYZ@@000000`2YZJT600000080ZJVY3`0000020:VYZ@8000003?ooool000_ooooo1P00
00020:VYZ@d000000P2YZJT400000080ZJVY0`0000020:VYZ@@000000P2YZJT400000080ZJVY0P00
00020:VYZ@T000000P2YZJT4000000@0ZJVY1@0000040:VYZ@H000000P2YZJT2000000<0ZJVY0P00
00040:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJTC000000@0ZJVY0P00
00020:VYZA4000000P2YZJT800000080ZJVY00<000000:VYZ@2YZJT07@0000020:VYZ@@000000P2Y
ZJT300000080ZJVY0P0000040:VYZ@8000000P2YZJT200000080ZJVY0`0000040:VYZ@P000000P2Y
ZJT700000080ZJVY0P0000020:VYZ@@000000P2YZJT900000080ZJVY0P0000040:VYZ@P0000000<0
ZJVY0000000000000P2YZJT600000080ZJVY0P0000020:VYZ@@000000`2YZJT200000080ZJVY0P00
00020:VYZ@8000000P2YZJT=00000080ZJVY0P0000040:VYZ@@000000`2YZJT600000080ZJVY3`00
00020:VYZ@8000003?ooool000_ooooo0P0000020:VYZ@H0000000<0ZJVY0000000000000P000002
0:VYZ@@000000P2YZJTC000000@0ZJVY0P0000050:VYZ@8000000P2YZJT200000080ZJVY10000002
0:VYZ@L000000P2YZJT400000080ZJVY1P0000030:VYZ@@000000P2YZJT400000080ZJVY1`000002
0:VYZ@@00000102YZJT800000080ZJVY0`0000040:VYZ@P000000P2YZJT500000080ZJVY20000002
0:VYZ@@000000P2YZJT3000000@0ZJVY4`0000040:VYZ@8000000P2YZJT400000080ZJVY1`000002
0:VYZ@P000000P2YZJT300000080ZJVY3P0000070:VYZ@8000000P2YZJT:00000080ZJVY1`000002
0:VYZ@@000000P2YZJT700000080ZJVY100000040:VYZ@8000000P2YZJT=00000080ZJVY0P000002
0:VYZ@@0000000<0ZJVY0000000000001P2YZJT4000000@0ZJVY1@0000020:VYZ@8000000P2YZJT2
00000080ZJVY0P0000040:VYZ@<000000P2YZJT8000000cooooo000;ooooo`8000000P2YZJT60000
00030:VYZ@0000000000008000000P2YZJT400000080ZJVY4`0000040:VYZ@8000001@2YZJT20000
0080ZJVY0P0000020:VYZ@@000000P2YZJT700000080ZJVY100000020:VYZ@H000000`2YZJT40000
0080ZJVY100000020:VYZ@L000000P2YZJT4000000@0ZJVY200000020:VYZ@<00000102YZJT80000
0080ZJVY1@0000020:VYZ@P000000P2YZJT400000080ZJVY0`0000040:VYZA<00000102YZJT20000
0080ZJVY100000020:VYZ@L000000P2YZJT800000080ZJVY0`0000020:VYZ@h000001`2YZJT20000
0080ZJVY2P0000020:VYZ@L000000P2YZJT400000080ZJVY1`0000020:VYZ@@00000102YZJT20000
0080ZJVY3@0000020:VYZ@8000000P2YZJT4000000030:VYZ@000000000000H0ZJVY100000040:VY
Z@D000000P2YZJT200000080ZJVY0P0000020:VYZ@800000102YZJT300000080ZJVY2000000<oooo
o`002oooool600000080ZJVY1`0000020:VYZ@@000000P2YZJT200000080ZJVY2@0000040:VYZ@@0
00000P2YZJT900000080ZJVY100000020:VYZ@8000000P2YZJT=00000080ZJVY0P0000020:VYZ@@0
000000<0ZJVY0000000000000P0000020:VYZ@@000000P2YZJTC00000080ZJVY100000020:VYZ@T0
0000102YZJT=000000@0ZJVY3`0000020:VYZ@h000000`2YZJTZ00000080ZJVY100000020:VYZ@80
00000P2YZJT2000000D0ZJVY100000020:VYZ@8000000P2YZJT400000080ZJVY00<000000:VYZ@2Y
ZJT0100000020:VYZ@@000000P2YZJT200000080ZJVY0`0000020:VYZ@X000001P2YZJT900000080
ZJVY0P0000020:VYZ@8000000`2YZJT2000000P0ZJVY100000020:VYZA<000000P2YZJT7000000H0
ZJVY3?ooool000_ooooo1P0000020:VYZ@L000000P2YZJT400000080ZJVY0P0000020:VYZ@T00000
102YZJT400000080ZJVY2@0000020:VYZ@@000000P2YZJT200000080ZJVY3@0000020:VYZ@800000
0P2YZJT4000000030:VYZ@0000000000008000000P2YZJT400000080ZJVY4`0000020:VYZ@@00000
0P2YZJT9000000@0ZJVY3@0000040:VYZ@l000000P2YZJT>000000<0ZJVY:P0000020:VYZ@@00000
0P2YZJT200000080ZJVY0P0000050:VYZ@@000000P2YZJT200000080ZJVY100000020:VYZ@030000
002YZJT0ZJVY00@000000P2YZJT400000080ZJVY0P0000020:VYZ@<000000P2YZJT:000000H0ZJVY
2@0000020:VYZ@8000000P2YZJT2000000<0ZJVY0P0000080:VYZ@@000000P2YZJTC00000080ZJVY
1`0000060:VYZ@cooooo000;ooooo`X000000`2YZJT200000080ZJVY100000020:VYZ@@000000P2Y
ZJT700000080ZJVY1P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0800000080ZJVY0P00
00040:VYZ@030000002YZJT0ZJVY008000001P2YZJT600000080ZJVY1`0000020:VYZ@@000000P2Y
ZJT200000080ZJVY2@0000040:VYZ@8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2Y
ZJT600000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY100000020:VYZ@/000000P2Y
ZJT200000080ZJVY0P0000020:VYZ@X000000`2YZJT2000000@0ZJVY800000020:VYZ@P000000P2Y
ZJT200000080ZJVY0P0000030:VYZ@@00000102YZJT2000000@0ZJVY0P0000020:VYZ@030000002Y
ZJT0ZJVY0080ZJVY0P0000020:VYZ@800000102YZJT700000080ZJVY1P0000020:VYZ@8000001P2Y
ZJT00`000000ZJVY0:VYZ@0200000080ZJVY1P0000020:VYZ@8000000`2YZJT4000000@0ZJVY1P00
00020:VYZ@@0000000<0ZJVY0000000000001P0000040:VYZ@8000000P2YZJT2000000030:VYZ@00
000000000080ZJVY0P0000040:VYZ@cooooo000;ooooo`X000000`2YZJT200000080ZJVY10000002
0:VYZ@@000000P2YZJT700000080ZJVY1P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@08
00000080ZJVY0P0000040:VYZ@030000002YZJT0ZJVY008000001P2YZJT600000080ZJVY1`000002
0:VYZ@@000000P2YZJT200000080ZJVY2@0000040:VYZ@8000000P2YZJT400000080ZJVY00<00000
0:VYZ@2YZJT00P2YZJT600000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY10000002
0:VYZ@/000000P2YZJT200000080ZJVY0P0000020:VYZ@X000000`2YZJT2000000@0ZJVY80000002
0:VYZ@P000000P2YZJT200000080ZJVY0P0000030:VYZ@@00000102YZJT2000000@0ZJVY0P000002
0:VYZ@030000002YZJT0ZJVY0080ZJVY0P0000020:VYZ@800000102YZJT700000080ZJVY1P000002
0:VYZ@8000001P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY1P0000020:VYZ@8000000`2YZJT4
000000@0ZJVY1P0000020:VYZ@@0000000<0ZJVY0000000000001P0000040:VYZ@8000000P2YZJT2
000000030:VYZ@00000000000080ZJVY0P0000040:VYZ@cooooo000;ooooo`8000001P2YZJT30000
0080ZJVY0P0000040:VYZ@P000000P2YZJT300000080ZJVY1P0000060:VYZ@L000000P2YZJT20000
00@0ZJVY100000020:VYZ@/000000P2YZJT200000080ZJVY1P0000000`2YZJT000000000000<0000
0080ZJVY0`0000040:VYZ@8000000P2YZJT6000000@0ZJVY5P0000020:VYZ@X00000102YZJT90000
0080ZJVY100000060:VYZ@D000000P2YZJT8000000@0ZJVY5@0000040:VYZ@D00000102YZJT20000
0080ZJVY3`0000020:VYZ@8000000P2YZJT400000080ZJVY1`0000020:VYZ@P000000`2YZJT20000
0080ZJVY0P0000020:VYZ@8000000P2YZJT4000000@0ZJVY1`0000020:VYZ@P000000P2YZJT70000
0080ZJVY4@0000020:VYZ@P000000P2YZJT9000000@0ZJVY3?ooool000_ooooo0P0000060:VYZ@<0
00000P2YZJT2000000@0ZJVY200000020:VYZ@<000000P2YZJT6000000H0ZJVY1`0000020:VYZ@80
0000102YZJT400000080ZJVY2`0000020:VYZ@8000000P2YZJT6000000030:VYZ@000000000000`0
00000P2YZJT3000000@0ZJVY0P0000020:VYZ@H00000102YZJTF00000080ZJVY2P0000040:VYZ@T0
00000P2YZJT4000000H0ZJVY1@0000020:VYZ@P00000102YZJTE000000@0ZJVY1@0000040:VYZ@80
00000P2YZJT?00000080ZJVY0P0000020:VYZ@@000000P2YZJT700000080ZJVY200000030:VYZ@80
00000P2YZJT200000080ZJVY0P0000020:VYZ@@00000102YZJT700000080ZJVY200000020:VYZ@L0
00000P2YZJTA00000080ZJVY200000020:VYZ@T00000102YZJT<ooooo`002oooool2000000@0ZJVY
100000000`2YZJT000000000000I000000P0ZJVY1`0000020:VYZ@@000000P2YZJT9000000@0ZJVY
300000020:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@8000000P2YZJT400000080ZJVY
0`0000040:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@80000000<0ZJVY000000000000
300000020:VYZ@<000000P2YZJT400000080ZJVY100000020:VYZ@@000000P2YZJT500000080ZJVY
0P0000020:VYZ@8000000P2YZJT2000000@0ZJVY2`0000020:VYZ@@000000`2YZJT:00000080ZJVY
100000020:VYZ@L000000P2YZJT4000000@0ZJVY0P0000020:VYZ@T000000P2YZJT200000080ZJVY
3@0000020:VYZ@@000000P2YZJT2000000<0ZJVY2P0000020:VYZ@T00000102YZJT200000080ZJVY
100000020:VYZ@8000000P2YZJT300000080ZJVY4@0000020:VYZ@8000000P2YZJT200000080ZJVY
100000020:VYZ@<000000P2YZJT4000000@0ZJVY3?ooool000_ooooo0P0000020:VYZ@/000000P2Y
ZJT?000000@0ZJVY100000040:VYZ@800000102YZJTA00000080ZJVY0`0000020:VYZ@@000000P2Y
ZJT2000000H0ZJVY0`0000020:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VYZ@T000000P2Y
ZJT800000080ZJVY0P0000070:VYZ@800000102YZJT700000080ZJVY100000020:VYZ@X000000P2Y
ZJT00`000000ZJVY0:VYZ@020:VYZ@h000000P2YZJT3000000H0ZJVY0P0000020:VYZ@H0000000<0
ZJVY0000000000004P0000030:VYZ@h000001`2YZJT2000000@0ZJVY0P0000040:VYZ@D000000P2Y
ZJT8000000@0ZJVY1`0000060:VYZ@8000000P2YZJT4000000@0ZJVY00<000000:VYZ@2YZJT00P2Y
ZJT2000000@0ZJVY1P0000020:VYZ@<000000P2YZJT<000000@0ZJVY00<000000:VYZ@2YZJT00P00
00020:VYZ@@000000P2YZJT=00000080ZJVY0P00000<ooooo`002oooool200000080ZJVY2`000002
0:VYZ@l00000102YZJT4000000@0ZJVY0P0000040:VYZA4000000P2YZJT300000080ZJVY10000002
0:VYZ@8000001P2YZJT300000080ZJVY0P0000020:VYZ@800000102YZJT200000080ZJVY2@000002
0:VYZ@P000000P2YZJT2000000L0ZJVY0P0000040:VYZ@L000000P2YZJT400000080ZJVY2P000002
0:VYZ@030000002YZJT0ZJVY0080ZJVY3P0000020:VYZ@<000001P2YZJT200000080ZJVY1P000000
0`2YZJT000000000000B000000<0ZJVY3P0000070:VYZ@800000102YZJT2000000@0ZJVY1@000002
0:VYZ@P00000102YZJT7000000H0ZJVY0P0000020:VYZ@@00000102YZJT00`000000ZJVY0:VYZ@02
0:VYZ@800000102YZJT600000080ZJVY0`0000020:VYZ@`00000102YZJT00`000000ZJVY0:VYZ@02
00000080ZJVY100000020:VYZ@d000000P2YZJT2000000cooooo000;ooooo`8000000P2YZJT40000
0080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@8000000P2YZJT600000080ZJVY0`0000020:VY
Z@H00000102YZJT900000080ZJVY1P0000020:VYZ@T000000P2YZJT800000080ZJVY0P0000070:VY
Z@8000000P2YZJT2000000@0ZJVY2@0000020:VYZ@P000001P2YZJT2000000<0ZJVY300000040:VY
Z@/000001P2YZJT5000000@0ZJVY0P0000040:VYZ@/000001P2YZJT400000080ZJVY5@0000020:VY
Z@T00000102YZJT400000080ZJVY0P0000030:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VY
Z@L000000P2YZJT2000000@0ZJVY2`0000020:VYZ@8000000P2YZJT400000080ZJVY100000020:VY
Z@<00000102YZJT400000080ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY0P0000020:VY
Z@P00000102YZJT00`000000ZJVY0:VYZ@0200000080ZJVY2P0000020:VYZ@<000000P2YZJT60000
00cooooo000;ooooo`8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@80
00000P2YZJT600000080ZJVY0`0000020:VYZ@H00000102YZJT900000080ZJVY1P0000020:VYZ@T0
00000P2YZJT800000080ZJVY0P0000070:VYZ@8000000P2YZJT2000000@0ZJVY2@0000020:VYZ@P0
00001P2YZJT2000000<0ZJVY300000040:VYZ@/000001P2YZJT5000000@0ZJVY0P0000040:VYZ@/0
00001P2YZJT400000080ZJVY5@0000020:VYZ@T00000102YZJT400000080ZJVY0P0000030:VYZ@80
00000P2YZJT2000000@0ZJVY0P0000020:VYZ@L000000P2YZJT2000000@0ZJVY2`0000020:VYZ@80
00000P2YZJT400000080ZJVY100000020:VYZ@<00000102YZJT400000080ZJVY0P0000020:VYZ@80
00000P2YZJT300000080ZJVY0P0000020:VYZ@P00000102YZJT00`000000ZJVY0:VYZ@0200000080
ZJVY2P0000020:VYZ@<000000P2YZJT6000000cooooo000;ooooo`H00000102YZJT500000080ZJVY
1P0000020:VYZ@8000000P2YZJT300000080ZJVY1P0000040:VYZ@H0000000<0ZJVY000000000000
0P2YZJT200000080ZJVY0P0000020:VYZ@8000001@2YZJT6000000H0ZJVY4`0000020:VYZ@800000
1@2YZJT200000080ZJVY0P0000040:VYZ@800000102YZJTC00000080ZJVY0`0000060:VYZ@H00000
202YZJT00`000000ZJVY0:VYZ@0<00000080ZJVY1@0000060:VYZ@@000000P2YZJT200000080ZJVY
3@0000020:VYZ@T000000P2YZJT8000000@0ZJVY2`0000020:VYZ@H000001`2YZJT600000080ZJVY
0P0000040:VYZ@@000000`2YZJT2000000@0ZJVY3@0000040:VYZ@8000000P2YZJT800000080ZJVY
00<000000:VYZ@2YZJT01P0000020:VYZ@P000000P2YZJT00`000000ZJVY0:VYZ@0800000080ZJVY
3`0000020:VYZ@cooooo000;ooooo`H00000102YZJT500000080ZJVY1P0000020:VYZ@8000000P2Y
ZJT300000080ZJVY1P0000040:VYZ@H0000000<0ZJVY0000000000000P2YZJT200000080ZJVY0P00
00020:VYZ@8000001@2YZJT6000000H0ZJVY4`0000020:VYZ@8000001@2YZJT200000080ZJVY0P00
00040:VYZ@800000102YZJTC00000080ZJVY0`0000060:VYZ@H00000202YZJT00`000000ZJVY0:VY
Z@0<00000080ZJVY1@0000060:VYZ@@000000P2YZJT200000080ZJVY3@0000020:VYZ@T000000P2Y
ZJT8000000@0ZJVY2`0000020:VYZ@H000001`2YZJT600000080ZJVY0P0000040:VYZ@@000000`2Y
ZJT2000000@0ZJVY3@0000040:VYZ@8000000P2YZJT800000080ZJVY00<000000:VYZ@2YZJT01P00
00020:VYZ@P000000P2YZJT00`000000ZJVY0:VYZ@0800000080ZJVY3`0000020:VYZ@cooooo000;
ooooo`/000000P2YZJT600000080ZJVY2`0000020:VYZ@H000000P2YZJT6000000<0ZJVY0P000002
0:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@030000002YZJT0ZJVY00H000000P2YZJT4
00000080ZJVY2`0000040:VYZ@H000000P2YZJT300000080ZJVY200000020:VYZ@@000000`2YZJT8
000000@0ZJVY0P0000020:VYZ@<00000102YZJT600000080ZJVY0P0000040:VYZ@L000000P2YZJT2
00000080ZJVY100000020:VYZ@L000000P2YZJT600000080ZJVY0P0000020:VYZ@80000000<0ZJVY
000000000000200000020:VYZ@800000102YZJT900000080ZJVY0P0000040:VYZ@8000001@2YZJT4
00000080ZJVY0P0000020:VYZ@@00000102YZJT3000000@0ZJVY100000040:VYZ@l00000102YZJT2
000000@0ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY1P0000040:VYZ@D000000P2YZJT2
00000080ZJVY1P0000020:VYZ@D000000P2YZJT200000080ZJVY0P0000020:VYZ@800000102YZJT5
00000080ZJVY0P0000020:VYZ@cooooo000;ooooo`/000000P2YZJT600000080ZJVY2`0000020:VY
Z@H000000P2YZJT6000000<0ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VY
Z@030000002YZJT0ZJVY00H000000P2YZJT400000080ZJVY2`0000040:VYZ@H000000P2YZJT30000
0080ZJVY200000020:VYZ@@000000`2YZJT8000000@0ZJVY0P0000020:VYZ@<00000102YZJT60000
0080ZJVY0P0000040:VYZ@L000000P2YZJT200000080ZJVY100000020:VYZ@L000000P2YZJT60000
0080ZJVY0P0000020:VYZ@80000000<0ZJVY000000000000200000020:VYZ@800000102YZJT90000
0080ZJVY0P0000040:VYZ@8000001@2YZJT400000080ZJVY0P0000020:VYZ@@00000102YZJT30000
00@0ZJVY100000040:VYZ@l00000102YZJT2000000@0ZJVY00<000000:VYZ@2YZJT00P2YZJT20000
0080ZJVY1P0000040:VYZ@D000000P2YZJT200000080ZJVY1P0000020:VYZ@D000000P2YZJT20000
0080ZJVY0P0000020:VYZ@800000102YZJT500000080ZJVY0P0000020:VYZ@cooooo000;oooooaL0
00000P2YZJT200000080ZJVY4`0000020:VYZ@<000000P2YZJT200000080ZJVY0P0000020:VYZ@@0
00000P2YZJT00`000000ZJVY0:VYZ@020:VYZA00000000<0ZJVY0000000000003P0000020:VYZ@d0
00000P2YZJT4000000<0ZJVY0P0000040:VYZ@800000102YZJT200000080ZJVY2`0000040:VYZ@80
00000P2YZJT4000000<0ZJVY0P0000020:VYZ@8000000P2YZJT800000080ZJVY0`0000020:VYZ@80
0000102YZJT;00000080ZJVY0P0000040:VYZ@H000000P2YZJT2000000D0ZJVY1P0000020:VYZ@H0
00000`2YZJT400000080ZJVY0P0000040:VYZ@/000000P2YZJT4000000@0ZJVY0P0000030:VYZ@@0
0000102YZJT400000080ZJVY100000020:VYZ@<000000P2YZJT:000000@0ZJVY00<000000:VYZ@2Y
ZJT00P0000020:VYZ@8000000P2YZJT600000080ZJVY0P0000030:VYZ@P000000P2YZJT200000080
ZJVY0`0000020:VYZ@H000000P2YZJT<ooooo`002ooooolG00000080ZJVY0P0000020:VYZA<00000
0P2YZJT300000080ZJVY0P0000020:VYZ@8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT0
0P2YZJT@000000030:VYZ@000000000000h000000P2YZJT=00000080ZJVY100000030:VYZ@800000
102YZJT2000000@0ZJVY0P0000020:VYZ@/00000102YZJT200000080ZJVY100000030:VYZ@800000
0P2YZJT200000080ZJVY200000020:VYZ@<000000P2YZJT2000000@0ZJVY2`0000020:VYZ@800000
102YZJT600000080ZJVY0P0000050:VYZ@H000000P2YZJT6000000<0ZJVY100000020:VYZ@800000
102YZJT;00000080ZJVY100000040:VYZ@8000000`2YZJT4000000@0ZJVY100000020:VYZ@@00000
0P2YZJT300000080ZJVY2P0000040:VYZ@030000002YZJT0ZJVY008000000P2YZJT200000080ZJVY
1P0000020:VYZ@8000000`2YZJT800000080ZJVY0P0000020:VYZ@<000000P2YZJT600000080ZJVY
3?ooool00?ooooooUOooool00?ooooooUOooool00001\
\>"],
  ImageRangeCache->{{{0, 403}, {100, 0}} -> {-5.78856, -1.25026, 0.525006, \
0.525006}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "If you look carefully, you'll see the plot is upside down. When ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " plots the ",
  StyleBox["RasterArray", "Input"],
  ", it draws row ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " with ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " coordinate ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  ". Since ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " shows plots using the standard Cartesian plane, the first row of the ",
  StyleBox["RasterArray", "Input"],
  " - i.e., the first time step in our run - appears at the bottom. To change \
this situation, simply ",
  StyleBox["Reverse", "Input"],
  " the list before putting it in the ",
  StyleBox["RasterArray", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]Graphics[\[IndentingNewLine]RasterArray[
            Reverse[l]]\[IndentingNewLine]], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .25 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.00595238 0.0047619 [
[ 0 0 0 0 ]
[ 1 .25 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .25 L
0 .25 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .00595 translate
.95238 .2381 scale
600 string
200 50 8 [200 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9000000000000000000000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000
000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000000000A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000000000
A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000000000
A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000
A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000
A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000000000
000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000
A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9000000000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000000000000000A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9000000
000000000000000000000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000
000000000000A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9
000000000000000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000000000000000000000000000000000000000A9A9A9000000000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000000000000000
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9000000000000000000A9A9A9000000000000000000000000000000A9A9A9
000000000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000000000000000000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000
000000A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000
000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000
A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9
000000000000A9A9A9000000000000A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000000000000000000000A9A9A9000000000000A9A9A9
000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000
000000000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000
000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000000000000000000000000000000000000000A9A9A9000000000000000000
000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000000000
000000A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000000000000000
000000000000A9A9A9000000000000000000A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000000000A9A9A9000000000000000000000000000000000000
000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000
A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[19]:=",
  ImageSize->{404, 101},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006D0000IB000`400?l00000o`00003ooooooonE
ooooo`00oooooonEooooo`00oooooonEooooo`00oooooonEooooo`002ooooolG00000080ZJVY0P00
00020:VYZA<000000P2YZJT300000080ZJVY0P0000020:VYZ@8000000P2YZJT400000080ZJVY00<0
00000:VYZ@2YZJT00P2YZJT@000000030:VYZ@000000000000h000000P2YZJT=00000080ZJVY1000
00030:VYZ@800000102YZJT2000000@0ZJVY0P0000020:VYZ@/00000102YZJT200000080ZJVY1000
00030:VYZ@8000000P2YZJT200000080ZJVY200000020:VYZ@<000000P2YZJT2000000@0ZJVY2`00
00020:VYZ@800000102YZJT600000080ZJVY0P0000050:VYZ@H000000P2YZJT6000000<0ZJVY1000
00020:VYZ@800000102YZJT;00000080ZJVY100000040:VYZ@8000000`2YZJT4000000@0ZJVY1000
00020:VYZ@@000000P2YZJT300000080ZJVY2P0000040:VYZ@030000002YZJT0ZJVY008000000P2Y
ZJT200000080ZJVY1P0000020:VYZ@8000000`2YZJT800000080ZJVY0P0000020:VYZ@<000000P2Y
ZJT600000080ZJVY3?ooool000_ooooo5`0000020:VYZ@8000000P2YZJTC00000080ZJVY0`000002
0:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY40000000
0`2YZJT000000000000>00000080ZJVY3@0000020:VYZ@@000000`2YZJT2000000@0ZJVY0P000004
0:VYZ@8000000P2YZJT;000000@0ZJVY0P0000020:VYZ@@000000`2YZJT200000080ZJVY0P000002
0:VYZ@P000000P2YZJT300000080ZJVY0P0000040:VYZ@/000000P2YZJT2000000@0ZJVY1P000002
0:VYZ@8000001@2YZJT600000080ZJVY1P0000030:VYZ@@000000P2YZJT2000000@0ZJVY2`000002
0:VYZ@@00000102YZJT2000000<0ZJVY100000040:VYZ@@000000P2YZJT400000080ZJVY0`000002
0:VYZ@X00000102YZJT00`000000ZJVY0:VYZ@0200000080ZJVY0P0000020:VYZ@H000000P2YZJT2
000000<0ZJVY200000020:VYZ@8000000P2YZJT300000080ZJVY1P0000020:VYZ@cooooo000;oooo
o`/000000P2YZJT600000080ZJVY2`0000020:VYZ@H000000P2YZJT6000000<0ZJVY0P0000020:VY
Z@8000000P2YZJT200000080ZJVY0P0000040:VYZ@030000002YZJT0ZJVY00H000000P2YZJT40000
0080ZJVY2`0000040:VYZ@H000000P2YZJT300000080ZJVY200000020:VYZ@@000000`2YZJT80000
00@0ZJVY0P0000020:VYZ@<00000102YZJT600000080ZJVY0P0000040:VYZ@L000000P2YZJT20000
0080ZJVY100000020:VYZ@L000000P2YZJT600000080ZJVY0P0000020:VYZ@80000000<0ZJVY0000
00000000200000020:VYZ@800000102YZJT900000080ZJVY0P0000040:VYZ@8000001@2YZJT40000
0080ZJVY0P0000020:VYZ@@00000102YZJT3000000@0ZJVY100000040:VYZ@l00000102YZJT20000
00@0ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY1P0000040:VYZ@D000000P2YZJT20000
0080ZJVY1P0000020:VYZ@D000000P2YZJT200000080ZJVY0P0000020:VYZ@800000102YZJT50000
0080ZJVY0P0000020:VYZ@cooooo000;ooooo`/000000P2YZJT600000080ZJVY2`0000020:VYZ@H0
00000P2YZJT6000000<0ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@03
0000002YZJT0ZJVY00H000000P2YZJT400000080ZJVY2`0000040:VYZ@H000000P2YZJT300000080
ZJVY200000020:VYZ@@000000`2YZJT8000000@0ZJVY0P0000020:VYZ@<00000102YZJT600000080
ZJVY0P0000040:VYZ@L000000P2YZJT200000080ZJVY100000020:VYZ@L000000P2YZJT600000080
ZJVY0P0000020:VYZ@80000000<0ZJVY000000000000200000020:VYZ@800000102YZJT900000080
ZJVY0P0000040:VYZ@8000001@2YZJT400000080ZJVY0P0000020:VYZ@@00000102YZJT3000000@0
ZJVY100000040:VYZ@l00000102YZJT2000000@0ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080
ZJVY1P0000040:VYZ@D000000P2YZJT200000080ZJVY1P0000020:VYZ@D000000P2YZJT200000080
ZJVY0P0000020:VYZ@800000102YZJT500000080ZJVY0P0000020:VYZ@cooooo000;ooooo`H00000
102YZJT500000080ZJVY1P0000020:VYZ@8000000P2YZJT300000080ZJVY1P0000040:VYZ@H00000
00<0ZJVY0000000000000P2YZJT200000080ZJVY0P0000020:VYZ@8000001@2YZJT6000000H0ZJVY
4`0000020:VYZ@8000001@2YZJT200000080ZJVY0P0000040:VYZ@800000102YZJTC00000080ZJVY
0`0000060:VYZ@H00000202YZJT00`000000ZJVY0:VYZ@0<00000080ZJVY1@0000060:VYZ@@00000
0P2YZJT200000080ZJVY3@0000020:VYZ@T000000P2YZJT8000000@0ZJVY2`0000020:VYZ@H00000
1`2YZJT600000080ZJVY0P0000040:VYZ@@000000`2YZJT2000000@0ZJVY3@0000040:VYZ@800000
0P2YZJT800000080ZJVY00<000000:VYZ@2YZJT01P0000020:VYZ@P000000P2YZJT00`000000ZJVY
0:VYZ@0800000080ZJVY3`0000020:VYZ@cooooo000;ooooo`H00000102YZJT500000080ZJVY1P00
00020:VYZ@8000000P2YZJT300000080ZJVY1P0000040:VYZ@H0000000<0ZJVY0000000000000P2Y
ZJT200000080ZJVY0P0000020:VYZ@8000001@2YZJT6000000H0ZJVY4`0000020:VYZ@8000001@2Y
ZJT200000080ZJVY0P0000040:VYZ@800000102YZJTC00000080ZJVY0`0000060:VYZ@H00000202Y
ZJT00`000000ZJVY0:VYZ@0<00000080ZJVY1@0000060:VYZ@@000000P2YZJT200000080ZJVY3@00
00020:VYZ@T000000P2YZJT8000000@0ZJVY2`0000020:VYZ@H000001`2YZJT600000080ZJVY0P00
00040:VYZ@@000000`2YZJT2000000@0ZJVY3@0000040:VYZ@8000000P2YZJT800000080ZJVY00<0
00000:VYZ@2YZJT01P0000020:VYZ@P000000P2YZJT00`000000ZJVY0:VYZ@0800000080ZJVY3`00
00020:VYZ@cooooo000;ooooo`8000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P000002
0:VYZ@8000000P2YZJT600000080ZJVY0`0000020:VYZ@H00000102YZJT900000080ZJVY1P000002
0:VYZ@T000000P2YZJT800000080ZJVY0P0000070:VYZ@8000000P2YZJT2000000@0ZJVY2@000002
0:VYZ@P000001P2YZJT2000000<0ZJVY300000040:VYZ@/000001P2YZJT5000000@0ZJVY0P000004
0:VYZ@/000001P2YZJT400000080ZJVY5@0000020:VYZ@T00000102YZJT400000080ZJVY0P000003
0:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VYZ@L000000P2YZJT2000000@0ZJVY2`000002
0:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@<00000102YZJT400000080ZJVY0P000002
0:VYZ@8000000P2YZJT300000080ZJVY0P0000020:VYZ@P00000102YZJT00`000000ZJVY0:VYZ@02
00000080ZJVY2P0000020:VYZ@<000000P2YZJT6000000cooooo000;ooooo`8000000P2YZJT40000
0080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@8000000P2YZJT600000080ZJVY0`0000020:VY
Z@H00000102YZJT900000080ZJVY1P0000020:VYZ@T000000P2YZJT800000080ZJVY0P0000070:VY
Z@8000000P2YZJT2000000@0ZJVY2@0000020:VYZ@P000001P2YZJT2000000<0ZJVY300000040:VY
Z@/000001P2YZJT5000000@0ZJVY0P0000040:VYZ@/000001P2YZJT400000080ZJVY5@0000020:VY
Z@T00000102YZJT400000080ZJVY0P0000030:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VY
Z@L000000P2YZJT2000000@0ZJVY2`0000020:VYZ@8000000P2YZJT400000080ZJVY100000020:VY
Z@<00000102YZJT400000080ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY0P0000020:VY
Z@P00000102YZJT00`000000ZJVY0:VYZ@0200000080ZJVY2P0000020:VYZ@<000000P2YZJT60000
00cooooo000;ooooo`8000000P2YZJT;00000080ZJVY3`0000040:VYZ@@00000102YZJT2000000@0
ZJVY4@0000020:VYZ@<000000P2YZJT400000080ZJVY0P0000060:VYZ@<000000P2YZJT200000080
ZJVY0P0000040:VYZ@8000000P2YZJT900000080ZJVY200000020:VYZ@8000001`2YZJT2000000@0
ZJVY1`0000020:VYZ@@000000P2YZJT:00000080ZJVY00<000000:VYZ@2YZJT00P2YZJT>00000080
ZJVY0`0000060:VYZ@8000000P2YZJT6000000030:VYZ@0000000000018000000`2YZJT>000000L0
ZJVY0P0000040:VYZ@800000102YZJT500000080ZJVY200000040:VYZ@L000001P2YZJT200000080
ZJVY100000040:VYZ@030000002YZJT0ZJVY0080ZJVY0P0000040:VYZ@H000000P2YZJT300000080
ZJVY300000040:VYZ@030000002YZJT0ZJVY008000000P2YZJT400000080ZJVY3@0000020:VYZ@80
00003?ooool000_ooooo0P0000040:VYZ@@0000000<0ZJVY0000000000006@0000080:VYZ@L00000
0P2YZJT400000080ZJVY2@0000040:VYZ@`000000P2YZJT2000000030:VYZ@000000000000800000
0P2YZJT200000080ZJVY100000020:VYZ@<00000102YZJT200000080ZJVY0P0000020:VYZ@@00000
0P2YZJT2000000030:VYZ@000000000000`000000P2YZJT300000080ZJVY100000020:VYZ@@00000
0P2YZJT400000080ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@/00000
0P2YZJT4000000<0ZJVY2P0000020:VYZ@@000000P2YZJT700000080ZJVY100000040:VYZ@800000
0P2YZJT900000080ZJVY0P0000020:VYZ@d000000P2YZJT400000080ZJVY0P0000030:VYZ@X00000
0P2YZJT9000000@0ZJVY0P0000020:VYZ@@000000P2YZJT200000080ZJVY0`0000020:VYZA400000
0P2YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT300000080ZJVY100000040:VYZ@cooooo
000;ooooo`800000102YZJT4000000030:VYZ@000000000001T00000202YZJT700000080ZJVY1000
00020:VYZ@T00000102YZJT<00000080ZJVY0P0000000`2YZJT000000000000200000080ZJVY0P00
00020:VYZ@@000000P2YZJT3000000@0ZJVY0P0000020:VYZ@8000000P2YZJT400000080ZJVY0P00
00000`2YZJT000000000000<00000080ZJVY0`0000020:VYZ@@000000P2YZJT400000080ZJVY1000
00020:VYZ@D000000P2YZJT200000080ZJVY0P0000020:VYZ@800000102YZJT;00000080ZJVY1000
00030:VYZ@X000000P2YZJT400000080ZJVY1`0000020:VYZ@@00000102YZJT200000080ZJVY2@00
00020:VYZ@8000000P2YZJT=00000080ZJVY100000020:VYZ@8000000`2YZJT:00000080ZJVY2@00
00040:VYZ@8000000P2YZJT400000080ZJVY0P0000020:VYZ@<000000P2YZJTA00000080ZJVY0P00
00020:VYZ@8000000P2YZJT400000080ZJVY0`0000020:VYZ@@00000102YZJT<ooooo`002oooool2
000000H0ZJVY0`0000020:VYZ@800000102YZJT800000080ZJVY0`0000020:VYZ@H000001P2YZJT7
00000080ZJVY0P0000040:VYZ@@000000P2YZJT;00000080ZJVY0P0000020:VYZ@H0000000<0ZJVY
000000000000300000020:VYZ@<00000102YZJT200000080ZJVY1P0000040:VYZAH000000P2YZJT:
000000@0ZJVY2@0000020:VYZ@@000001P2YZJT500000080ZJVY200000040:VYZAD00000102YZJT5
000000@0ZJVY0P0000020:VYZ@l000000P2YZJT200000080ZJVY100000020:VYZ@L000000P2YZJT8
000000<0ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY100000040:VYZ@L000000P2YZJT8
00000080ZJVY1`0000020:VYZA4000000P2YZJT800000080ZJVY2@0000040:VYZ@cooooo000;oooo
o`8000001P2YZJT300000080ZJVY0P0000040:VYZ@P000000P2YZJT300000080ZJVY1P0000060:VY
Z@L000000P2YZJT2000000@0ZJVY100000020:VYZ@/000000P2YZJT200000080ZJVY1P0000000`2Y
ZJT000000000000<00000080ZJVY0`0000040:VYZ@8000000P2YZJT6000000@0ZJVY5P0000020:VY
Z@X00000102YZJT900000080ZJVY100000060:VYZ@D000000P2YZJT8000000@0ZJVY5@0000040:VY
Z@D00000102YZJT200000080ZJVY3`0000020:VYZ@8000000P2YZJT400000080ZJVY1`0000020:VY
Z@P000000`2YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT4000000@0ZJVY1`0000020:VY
Z@P000000P2YZJT700000080ZJVY4@0000020:VYZ@P000000P2YZJT9000000@0ZJVY3?ooool000_o
oooo2P0000030:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@L000000P2YZJT600000080
ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00P000000P2YZJT2000000@0ZJVY00<000000:VYZ@2Y
ZJT00P0000060:VYZ@H000000P2YZJT700000080ZJVY100000020:VYZ@8000000P2YZJT9000000@0
ZJVY0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@H000000P2YZJT400000080
ZJVY00<000000:VYZ@2YZJT00P2YZJT400000080ZJVY2`0000020:VYZ@8000000P2YZJT200000080
ZJVY2P0000030:VYZ@800000102YZJTP00000080ZJVY200000020:VYZ@8000000P2YZJT2000000<0
ZJVY100000040:VYZ@800000102YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080
ZJVY0P0000040:VYZ@L000000P2YZJT600000080ZJVY0P0000060:VYZ@030000002YZJT0ZJVY0080
00000P2YZJT600000080ZJVY0P0000030:VYZ@@00000102YZJT600000080ZJVY100000000`2YZJT0
000000000006000000@0ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P2YZJT2000000@0
ZJVY3?ooool000_ooooo2P0000030:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@L00000
0P2YZJT600000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00P000000P2YZJT2000000@0ZJVY
00<000000:VYZ@2YZJT00P0000060:VYZ@H000000P2YZJT700000080ZJVY100000020:VYZ@800000
0P2YZJT9000000@0ZJVY0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@H00000
0P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT400000080ZJVY2`0000020:VYZ@800000
0P2YZJT200000080ZJVY2P0000030:VYZ@800000102YZJTP00000080ZJVY200000020:VYZ@800000
0P2YZJT2000000<0ZJVY100000040:VYZ@800000102YZJT200000080ZJVY00<000000:VYZ@2YZJT0
0P2YZJT200000080ZJVY0P0000040:VYZ@L000000P2YZJT600000080ZJVY0P0000060:VYZ@030000
002YZJT0ZJVY008000000P2YZJT600000080ZJVY0P0000030:VYZ@@00000102YZJT600000080ZJVY
100000000`2YZJT0000000000006000000@0ZJVY0P0000020:VYZ@80000000<0ZJVY000000000000
0P2YZJT2000000@0ZJVY3?ooool000_ooooo1P0000020:VYZ@L000000P2YZJT400000080ZJVY0P00
00020:VYZ@T00000102YZJT400000080ZJVY2@0000020:VYZ@@000000P2YZJT200000080ZJVY3@00
00020:VYZ@8000000P2YZJT4000000030:VYZ@0000000000008000000P2YZJT400000080ZJVY4`00
00020:VYZ@@000000P2YZJT9000000@0ZJVY3@0000040:VYZ@l000000P2YZJT>000000<0ZJVY:P00
00020:VYZ@@000000P2YZJT200000080ZJVY0P0000050:VYZ@@000000P2YZJT200000080ZJVY1000
00020:VYZ@030000002YZJT0ZJVY00@000000P2YZJT400000080ZJVY0P0000020:VYZ@<000000P2Y
ZJT:000000H0ZJVY2@0000020:VYZ@8000000P2YZJT2000000<0ZJVY0P0000080:VYZ@@000000P2Y
ZJTC00000080ZJVY1`0000060:VYZ@cooooo000;ooooo`H000000P2YZJT700000080ZJVY10000002
0:VYZ@8000000P2YZJT9000000@0ZJVY100000020:VYZ@T000000P2YZJT400000080ZJVY0P000002
0:VYZ@d000000P2YZJT200000080ZJVY100000000`2YZJT000000000000200000080ZJVY10000002
0:VYZA<000000P2YZJT400000080ZJVY2@0000040:VYZ@d00000102YZJT?00000080ZJVY3P000003
0:VYZBX000000P2YZJT400000080ZJVY0P0000020:VYZ@8000001@2YZJT400000080ZJVY0P000002
0:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY100000020:VYZ@8000000P2YZJT3
00000080ZJVY2P0000060:VYZ@T000000P2YZJT200000080ZJVY0P0000030:VYZ@800000202YZJT4
00000080ZJVY4`0000020:VYZ@L000001P2YZJT<ooooo`002oooool200000080ZJVY1P0000000`2Y
ZJT000000000000200000080ZJVY100000020:VYZA<00000102YZJT2000000D0ZJVY0P0000020:VY
Z@8000000P2YZJT400000080ZJVY1`0000020:VYZ@@000000P2YZJT6000000<0ZJVY100000020:VY
Z@@000000P2YZJT700000080ZJVY100000040:VYZ@P000000P2YZJT3000000@0ZJVY200000020:VY
Z@D000000P2YZJT800000080ZJVY100000020:VYZ@<00000102YZJTC000000@0ZJVY0P0000020:VY
Z@@000000P2YZJT700000080ZJVY200000020:VYZ@<000000P2YZJT>000000L0ZJVY0P0000020:VY
Z@X000000P2YZJT700000080ZJVY100000020:VYZ@L000000P2YZJT4000000@0ZJVY0P0000020:VY
Z@d000000P2YZJT200000080ZJVY100000000`2YZJT00000000000060:VYZ@@00000102YZJT50000
0080ZJVY0P0000020:VYZ@8000000P2YZJT2000000@0ZJVY0`0000020:VYZ@P000003?ooool000_o
oooo0P0000020:VYZ@H0000000<0ZJVY0000000000000P0000020:VYZ@@000000P2YZJTC000000@0
ZJVY0P0000050:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@L000000P2YZJT400000080
ZJVY1P0000030:VYZ@@000000P2YZJT400000080ZJVY1`0000020:VYZ@@00000102YZJT800000080
ZJVY0`0000040:VYZ@P000000P2YZJT500000080ZJVY200000020:VYZ@@000000P2YZJT3000000@0
ZJVY4`0000040:VYZ@8000000P2YZJT400000080ZJVY1`0000020:VYZ@P000000P2YZJT300000080
ZJVY3P0000070:VYZ@8000000P2YZJT:00000080ZJVY1`0000020:VYZ@@000000P2YZJT700000080
ZJVY100000040:VYZ@8000000P2YZJT=00000080ZJVY0P0000020:VYZ@@0000000<0ZJVY00000000
00001P2YZJT4000000@0ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@<0
00000P2YZJT8000000cooooo000;ooooo`H000000P2YZJT=00000080ZJVY100000020:VYZ@<00000
0P2YZJT400000080ZJVY100000020:VYZ@8000000P2YZJT900000080ZJVY100000040:VYZ@D00000
102YZJT600000080ZJVY0P0000030:VYZ@800000102YZJT400000080ZJVY100000020:VYZ@030000
002YZJT0ZJVY0080ZJVY4`0000040:VYZ@8000000P2YZJTA00000080ZJVY200000020:VYZ@030000
002YZJT0ZJVY01d000000P2YZJT400000080ZJVY0`0000020:VYZ@800000102YZJT200000080ZJVY
0P0000020:VYZ@<00000102YZJT800000080ZJVY1`0000020:VYZ@8000000P2YZJT400000080ZJVY
2@0000020:VYZ@800000102YZJT8000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@800000
0P2YZJT4000000<0ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY3@0000020:VYZ@800000
102YZJT4000000<0ZJVY1P0000020:VYZ@l000000P2YZJT2000000cooooo000;ooooo`H000000P2Y
ZJT=00000080ZJVY100000020:VYZ@<000000P2YZJT400000080ZJVY100000020:VYZ@8000000P2Y
ZJT900000080ZJVY100000040:VYZ@D00000102YZJT600000080ZJVY0P0000030:VYZ@800000102Y
ZJT400000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY4`0000040:VYZ@8000000P2Y
ZJTA00000080ZJVY200000020:VYZ@030000002YZJT0ZJVY01d000000P2YZJT400000080ZJVY0`00
00020:VYZ@800000102YZJT200000080ZJVY0P0000020:VYZ@<00000102YZJT800000080ZJVY1`00
00020:VYZ@8000000P2YZJT400000080ZJVY2@0000020:VYZ@800000102YZJT8000000030:VYZ@00
000000000080ZJVY1P0000020:VYZ@8000000P2YZJT4000000<0ZJVY0P0000020:VYZ@8000000P2Y
ZJT200000080ZJVY3@0000020:VYZ@800000102YZJT4000000<0ZJVY1P0000020:VYZ@l000000P2Y
ZJT2000000cooooo000;ooooo`80ZJVY100000020:VYZ@L00000102YZJT200000080ZJVY3`000002
0:VYZ@8000000P2YZJT7000000@0ZJVY0P0000040:VYZ@@00000102YZJT00`000000ZJVY0:VYZ@06
0:VYZ@@00000102YZJT2000000<0ZJVY0P0000040:VYZ@@000000P2YZJT2000000@0ZJVY6P000002
0:VYZ@800000102YZJT7000000@0ZJVY200000020:VYZ@T000001P2YZJT2000000H0ZJVY10000003
0:VYZ@800000102YZJT200000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY0P000002
0:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY0P000004
0:VYZ@8000000`2YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT6000000030:VYZ@000000
000000@00000102YZJT400000080ZJVY1@0000040:VYZ@X000000P2YZJT2000000030:VYZ@000000
0000008000000P2YZJT:00000080ZJVY1`0000040:VYZ@X000000`2YZJT<00000080ZJVY0P000000
0`2YZJT0000000000008000000cooooo000;ooooo`80ZJVY100000020:VYZ@L00000102YZJT20000
0080ZJVY3`0000020:VYZ@8000000P2YZJT7000000@0ZJVY0P0000040:VYZ@@00000102YZJT00`00
0000ZJVY0:VYZ@060:VYZ@@00000102YZJT2000000<0ZJVY0P0000040:VYZ@@000000P2YZJT20000
00@0ZJVY6P0000020:VYZ@800000102YZJT7000000@0ZJVY200000020:VYZ@T000001P2YZJT20000
00H0ZJVY100000030:VYZ@800000102YZJT200000080ZJVY100000020:VYZ@030000002YZJT0ZJVY
0080ZJVY0P0000020:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT20000
0080ZJVY0P0000040:VYZ@8000000`2YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT60000
00030:VYZ@000000000000@00000102YZJT400000080ZJVY1@0000040:VYZ@X000000P2YZJT20000
00030:VYZ@0000000000008000000P2YZJT:00000080ZJVY1`0000040:VYZ@X000000`2YZJT<0000
0080ZJVY0P0000000`2YZJT0000000000008000000cooooo000;ooooo`80ZJVY100000020:VYZ@d0
00000P2YZJT200000080ZJVY0`0000020:VYZ@@00000102YZJT6000000@0ZJVY0`0000020:VYZ@80
00001P2YZJT2000000@0ZJVY2@0000020:VYZ@8000000P2YZJT9000000@0ZJVY0P0000020:VYZ@@0
0000102YZJT500000080ZJVY100000020:VYZ@8000000P2YZJT2000000<0ZJVY3P0000020:VYZ@T0
00000P2YZJT?000000P0ZJVY2P0000030:VYZ@8000000P2YZJT400000080ZJVY0P0000040:VYZ@03
0000002YZJT0ZJVY0080ZJVY0P0000020:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2Y
ZJT0100000020:VYZ@8000001P2YZJT900000080ZJVY0P0000040:VYZ@@000000`2YZJT800000080
ZJVY0P0000020:VYZ@@0000000<0ZJVY0000000000000P0000020:VYZ@8000000P2YZJT600000080
ZJVY0`0000020:VYZ@800000102YZJT400000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY0080
ZJVY0P0000020:VYZ@P000000P2YZJT2000000L0ZJVY0P0000020:VYZ@8000000P2YZJT4000000<0
ZJVY0P0000020:VYZ@@000003?ooool000_ooooo0P2YZJT400000080ZJVY3@0000020:VYZ@800000
0P2YZJT300000080ZJVY100000040:VYZ@H00000102YZJT300000080ZJVY0P0000060:VYZ@800000
102YZJT900000080ZJVY0P0000020:VYZ@T00000102YZJT200000080ZJVY100000040:VYZ@D00000
0P2YZJT400000080ZJVY0P0000020:VYZ@8000000`2YZJT>00000080ZJVY2@0000020:VYZ@l00000
202YZJT:000000<0ZJVY0P0000020:VYZ@@000000P2YZJT2000000@0ZJVY00<000000:VYZ@2YZJT0
0P2YZJT200000080ZJVY100000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY
0P0000060:VYZ@T000000P2YZJT2000000@0ZJVY100000030:VYZ@P000000P2YZJT200000080ZJVY
100000000`2YZJT000000000000200000080ZJVY0P0000020:VYZ@H000000P2YZJT300000080ZJVY
0P0000040:VYZ@@000000P2YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY
200000020:VYZ@8000001`2YZJT200000080ZJVY0P0000020:VYZ@@000000`2YZJT200000080ZJVY
1000000<ooooo`002oooool4000000@0ZJVY1`0000020:VYZ@@000000P2YZJT;000000H0ZJVY0P00
00020:VYZ@8000001@2YZJT2000000`0ZJVY1@0000020:VYZ@8000000P2YZJT200000080ZJVY0P00
00040:VYZ@@0000000<0ZJVY0000000000001P0000020:VYZ@P0000000<0ZJVY000000000000102Y
ZJT400000080ZJVY2`0000020:VYZ@@00000102YZJT700000080ZJVY3P0000020:VYZ@<00000202Y
ZJT6000000@0ZJVY1`0000040:VYZ@@000001`2YZJT600000080ZJVY2P0000020:VYZ@<000000P2Y
ZJT200000080ZJVY0P0000060:VYZ@T000000P2YZJT?000000@0ZJVY0P0000060:VYZ@T000000P2Y
ZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT900000080ZJVY0P0000020:VYZ@@000000P2Y
ZJT00`000000ZJVY0:VYZ@0400000080ZJVY0P0000040:VYZ@H000002@2YZJT600000080ZJVY0P00
00000`2YZJT000000000000200000080ZJVY1000000<ooooo`002oooool4000000@0ZJVY1`000002
0:VYZ@@000000P2YZJT;000000H0ZJVY0P0000020:VYZ@8000001@2YZJT2000000`0ZJVY1@000002
0:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@@0000000<0ZJVY0000000000001P000002
0:VYZ@P0000000<0ZJVY000000000000102YZJT400000080ZJVY2`0000020:VYZ@@00000102YZJT7
00000080ZJVY3P0000020:VYZ@<00000202YZJT6000000@0ZJVY1`0000040:VYZ@@000001`2YZJT6
00000080ZJVY2P0000020:VYZ@<000000P2YZJT200000080ZJVY0P0000060:VYZ@T000000P2YZJT?
000000@0ZJVY0P0000060:VYZ@T000000P2YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT9
00000080ZJVY0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY0P000004
0:VYZ@H000002@2YZJT600000080ZJVY0P0000000`2YZJT000000000000200000080ZJVY1000000<
ooooo`002oooool20:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@02000000@0ZJVY0P0000020:VY
Z@L000000P2YZJT:00000080ZJVY1P0000000`2YZJT000000000000<0:VYZ@D000000P2YZJT20000
00H0ZJVY0P0000020:VYZ@T00000102YZJT600000080ZJVY100000000`2YZJT00000000000060000
00@0ZJVY100000020:VYZ@<00000102YZJT:00000080ZJVY0`0000040:VYZ@8000000P2YZJT80000
0080ZJVY0`0000080:VYZ@8000000P2YZJT2000000@0ZJVY0P0000030:VYZ@8000000P2YZJT40000
00T0ZJVY0P0000020:VYZ@P000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2YZJT20000
00X0ZJVY0P0000030:VYZ@@000000P2YZJT600000080ZJVY2@0000020:VYZ@8000000P2YZJT40000
0080ZJVY0P0000000`2YZJT00000000000020:VYZ@@000000P2YZJT200000080ZJVY4@0000020:VY
Z@@000000P2YZJT00`000000ZJVY0:VYZ@02000000@0ZJVY3000000=0:VYZ@8000000P2YZJT=0000
00cooooo000;ooooo`@000000P2YZJT9000000@0ZJVY1P0000020:VYZ@T000000P2YZJT?000000h0
ZJVY1@00000:0:VYZ@H000000`2YZJT2000000H0ZJVY100000020:VYZ@@0000000<0ZJVY00000000
00000P0000020:VYZ@@000000P2YZJT2000000@0ZJVY1`0000020:VYZ@800000102YZJT200000080
ZJVY0P0000000`2YZJT0000000000004000000@0ZJVY1P0000020:VYZ@T000000P2YZJT200000080
ZJVY0P0000020:VYZ@/000000P2YZJT2000000T0ZJVY1P0000020:VYZ@@000000P2YZJT400000080
ZJVY00<000000:VYZ@2YZJT0102YZJT800000080ZJVY0P0000030:VYZ@@000000P2YZJT6000000@0
ZJVY0P0000000`2YZJT00000000000020:VYZ@@00000102YZJT900000080ZJVY100000020:VYZ@X0
000000<0ZJVY0000000000003P0000020:VYZ@030000002YZJT0ZJVY00800000102YZJT200000080
ZJVY0P0000040:VYZ@8000004@2YZJT2000000030:VYZ@000000000000@0ZJVY1000000<ooooo`00
2oooool400000080ZJVY2@0000040:VYZ@H000000P2YZJT900000080ZJVY3`00000>0:VYZ@D00000
2P2YZJT6000000<0ZJVY0P0000060:VYZ@@000000P2YZJT4000000030:VYZ@000000000000800000
0P2YZJT400000080ZJVY0P0000040:VYZ@L000000P2YZJT2000000@0ZJVY0P0000020:VYZ@800000
00<0ZJVY000000000000100000040:VYZ@H000000P2YZJT900000080ZJVY0P0000020:VYZ@800000
0P2YZJT;00000080ZJVY0P0000090:VYZ@H000000P2YZJT400000080ZJVY100000020:VYZ@030000
002YZJT0ZJVY00@0ZJVY200000020:VYZ@8000000`2YZJT400000080ZJVY1P0000040:VYZ@800000
00<0ZJVY0000000000000P2YZJT4000000@0ZJVY2@0000020:VYZ@@000000P2YZJT:000000030:VY
Z@000000000000h000000P2YZJT00`000000ZJVY0:VYZ@02000000@0ZJVY0P0000020:VYZ@800000
102YZJT200000140ZJVY0P0000000`2YZJT00000000000040:VYZ@@000003?ooool000_ooooo1000
00040:VYZ@<000000P2YZJT400000080ZJVY0P0000020:VYZ@T000000P2YZJT400000080ZJVY0P00
00040:VYZ@D000002P2YZJT9000000`0ZJVY1P0000030:VYZ@8000001P2YZJT2000000@0ZJVY1000
00000`2YZJT000000000000800000080ZJVY4`0000060:VYZ@8000000`2YZJT200000080ZJVY1P00
00020:VYZ@8000000P2YZJT2000000030:VYZ@00000000000080ZJVY0P0000020:VYZ@8000000P2Y
ZJT200000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY0080ZJVY0P0000020:VYZ@8000000P2Y
ZJT4000000L0ZJVY0P0000040:VYZ@8000001P2YZJT700000080ZJVY0P0000020:VYZ@@000000P2Y
ZJT300000080ZJVY0P0000020:VYZ@@000000P2YZJT400000080ZJVY0P0000000`2YZJT000000000
0006000000@0ZJVY3`0000020:VYZ@@000000P2YZJTH00000080ZJVY0P0000020:VYZ@@000000P2Y
ZJT8000000/0ZJVY200000000`2YZJT0000000000008000000cooooo000;ooooo`@00000102YZJT3
00000080ZJVY100000020:VYZ@8000000P2YZJT900000080ZJVY100000020:VYZ@800000102YZJT5
000000X0ZJVY2@00000<0:VYZ@H000000`2YZJT2000000H0ZJVY0P0000040:VYZ@@0000000<0ZJVY
000000000000200000020:VYZA<000001P2YZJT2000000<0ZJVY0P0000020:VYZ@H000000P2YZJT2
00000080ZJVY0P0000000`2YZJT00000000000020:VYZ@8000000P2YZJT200000080ZJVY0P000002
0:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT200000080ZJVY10000007
0:VYZ@800000102YZJT2000000H0ZJVY1`0000020:VYZ@8000000P2YZJT400000080ZJVY0`000002
0:VYZ@8000000P2YZJT400000080ZJVY100000020:VYZ@80000000<0ZJVY0000000000001P000004
0:VYZ@l000000P2YZJT400000080ZJVY600000020:VYZ@8000000P2YZJT400000080ZJVY2000000;
0:VYZ@P0000000<0ZJVY0000000000002000000<ooooo`002oooool800000080ZJVY00<000000:VY
Z@2YZJT0100000020:VYZ@8000000P2YZJT2000000D0ZJVY0P0000040:VYZ@8000000P2YZJTA0000
00H0ZJVY0P0000020:VYZ@8000003`2YZJT4000000<0ZJVY0P0000060:VYZ@X0000000<0ZJVY0000
000000000P0000020:VYZ@@000000P2YZJT200000080ZJVY1@0000040:VYZ@8000002P2YZJT20000
00030:VYZ@000000000000H000000P2YZJT:000000030:VYZ@000000000000@000000P2YZJT:0000
0080ZJVY2`0000020:VYZ@8000000P2YZJT9000000@0ZJVY2P0000030:VYZ@8000000P2YZJT20000
0080ZJVY100000040:VYZ@D000000P2YZJT8000000@0ZJVY0P0000000`2YZJT00000000000020:VY
Z@8000000P2YZJT=00000080ZJVY400000000`2YZJT000000000000400000080ZJVY3`0000020:VY
Z@@00000102YZJT4000000d0ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY1P00000<oooo
o`002oooool800000080ZJVY00<000000:VYZ@2YZJT0100000020:VYZ@8000000P2YZJT2000000D0
ZJVY0P0000040:VYZ@8000000P2YZJTA000000H0ZJVY0P0000020:VYZ@8000003`2YZJT4000000<0
ZJVY0P0000060:VYZ@X0000000<0ZJVY0000000000000P0000020:VYZ@@000000P2YZJT200000080
ZJVY1@0000040:VYZ@8000002P2YZJT2000000030:VYZ@000000000000H000000P2YZJT:00000003
0:VYZ@000000000000@000000P2YZJT:00000080ZJVY2`0000020:VYZ@8000000P2YZJT9000000@0
ZJVY2P0000030:VYZ@8000000P2YZJT200000080ZJVY100000040:VYZ@D000000P2YZJT8000000@0
ZJVY0P0000000`2YZJT00000000000020:VYZ@8000000P2YZJT=00000080ZJVY400000000`2YZJT0
00000000000400000080ZJVY3`0000020:VYZ@@00000102YZJT4000000d0ZJVY0P0000020:VYZ@80
00000P2YZJT300000080ZJVY1P00000<ooooo`002oooool200000080ZJVY0P0000020:VYZ@D00000
0P2YZJT200000080ZJVY100000060:VYZ@<00000102YZJT:00000080ZJVY0`0000020:VYZ@P00000
1P2YZJT2000000030:VYZ@000000000000@000001P2YZJT4000000L0ZJVY400000030:VYZ@800000
102YZJT6000000@0ZJVY1`00000?0:VYZ@8000000P2YZJT800000080ZJVY1P0000000`2YZJT00000
0000000400000080ZJVY100000020:VYZ@L000000P2YZJT6000000@0ZJVY0P0000020:VYZ@<00000
0P2YZJT800000080ZJVY0P0000020:VYZ@T000000P2YZJT2000000@0ZJVY0P0000040:VYZ@D00000
0P2YZJT<000000D0ZJVY100000040:VYZ@P0000000<0ZJVY0000000000005`0000040:VYZ@800000
0P2YZJT700000080ZJVY200000040:VYZ@@000002@2YZJT600000080ZJVY1@0000060:VYZ@800000
0P2YZJT<ooooo`002oooool200000080ZJVY0P0000020:VYZ@D000000P2YZJT200000080ZJVY1000
00060:VYZ@<00000102YZJT:00000080ZJVY0`0000020:VYZ@P000001P2YZJT2000000030:VYZ@00
0000000000@000001P2YZJT4000000L0ZJVY400000030:VYZ@800000102YZJT6000000@0ZJVY1`00
000?0:VYZ@8000000P2YZJT800000080ZJVY1P0000000`2YZJT000000000000400000080ZJVY1000
00020:VYZ@L000000P2YZJT6000000@0ZJVY0P0000020:VYZ@<000000P2YZJT800000080ZJVY0P00
00020:VYZ@T000000P2YZJT2000000@0ZJVY0P0000040:VYZ@D000000P2YZJT<000000D0ZJVY1000
00040:VYZ@P0000000<0ZJVY0000000000005`0000040:VYZ@8000000P2YZJT700000080ZJVY2000
00040:VYZ@@000002@2YZJT600000080ZJVY1@0000060:VYZ@8000000P2YZJT<ooooo`002oooool2
00000080ZJVY100000020:VYZ@030000002YZJT0ZJVY0080ZJVY100000020:VYZ@8000000P2YZJT4
000000030:VYZ@000000000000@0ZJVY0P0000040:VYZ@800000102YZJT2000000<0ZJVY30000002
0:VYZ@80000000<0ZJVY0000000000000P2YZJT2000000H0ZJVY0P00000;0:VYZ@P00000102YZJT=
00000080ZJVY3`00000?0:VYZ@@000000P2YZJT400000080ZJVY1P0000000`2YZJT0000000000002
000000@0ZJVY0P0000020:VYZ@P000000`2YZJT200000080ZJVY200000020:VYZ@<000000P2YZJT8
00000080ZJVY200000000`2YZJT000000000000200000080ZJVY3@0000020:VYZ@@000000P2YZJT2
00000080ZJVY0P00000;0:VYZA<00000102YZJT400000080ZJVY1`0000020:VYZ@X000000P2YZJT2
00000080ZJVY4@0000070:VYZ@@000000P2YZJT2000000@0ZJVY1@0000080:VYZ@8000003?ooool0
00_ooooo0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@@000000P2YZJT20000
0080ZJVY100000000`2YZJT00000000000040:VYZ@800000102YZJT2000000@0ZJVY0P0000030:VY
Z@`000000P2YZJT2000000030:VYZ@00000000000080ZJVY0P0000060:VYZ@8000002`2YZJT80000
00@0ZJVY3@0000020:VYZ@l000003`2YZJT400000080ZJVY100000020:VYZ@H0000000<0ZJVY0000
000000000P0000040:VYZ@8000000P2YZJT8000000<0ZJVY0P0000020:VYZ@P000000P2YZJT30000
0080ZJVY200000020:VYZ@P0000000<0ZJVY0000000000000P0000020:VYZ@d000000P2YZJT40000
0080ZJVY0P0000020:VYZ@8000002`2YZJTC000000@0ZJVY100000020:VYZ@L000000P2YZJT:0000
0080ZJVY0P0000020:VYZA4000001`2YZJT400000080ZJVY0P0000040:VYZ@D00000202YZJT20000
00cooooo000;ooooo`800000102YZJT200000080ZJVY3`0000020:VYZ@8000001`2YZJT200000080
ZJVY2P0000050:VYZ@8000000P2YZJT200000080ZJVY0P0000090:VYZ@8000000P2YZJT6000000/0
ZJVY0P0000040:VYZ@800000102YZJT2000000030:VYZ@0000000000008000000P2YZJT400000080
ZJVY200000030:VYZ@@000003`2YZJT>00000080ZJVY1`0000040:VYZ@H000000P2YZJT4000000<0
ZJVY0P0000060:VYZ@@000000P2YZJTA000000@0ZJVY2`0000020:VYZ@@000000P2YZJT300000080
ZJVY0P0000040:VYZ@X000002@2YZJT2000000@0ZJVY0P0000020:VYZ@<000000P2YZJT2000000@0
ZJVY100000020:VYZ@8000000P2YZJT2000000D0ZJVY0P0000020:VYZ@@000000P2YZJT400000003
0:VYZ@00000000000080ZJVY0P0000020:VYZ@@000000P2YZJT200000080ZJVY1`0000040:VYZ@@0
00000P2YZJT2000000d0ZJVY3?ooool000_ooooo0P0000040:VYZ@8000000P2YZJT?00000080ZJVY
0P0000070:VYZ@8000000P2YZJT:000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2000000T0ZJVY
0P0000020:VYZ@H000002`2YZJT2000000@0ZJVY0P0000040:VYZ@80000000<0ZJVY000000000000
0P0000020:VYZ@@000000P2YZJT8000000<0ZJVY1000000?0:VYZ@h000000P2YZJT7000000@0ZJVY
1P0000020:VYZ@@000000`2YZJT2000000H0ZJVY100000020:VYZA400000102YZJT;00000080ZJVY
100000020:VYZ@<000000P2YZJT2000000@0ZJVY2P0000090:VYZ@800000102YZJT200000080ZJVY
0`0000020:VYZ@800000102YZJT400000080ZJVY0P0000020:VYZ@8000001@2YZJT200000080ZJVY
100000020:VYZ@@0000000<0ZJVY0000000000000P2YZJT200000080ZJVY100000020:VYZ@800000
0P2YZJT7000000@0ZJVY100000020:VYZ@8000003@2YZJT<ooooo`002oooool800000080ZJVY0`00
00020:VYZ@X000000P2YZJT;00000080ZJVY100000020:VYZ@/000000P2YZJT8000000T0ZJVY2000
00060:VYZ@/000001P2YZJT2000000030:VYZ@0000000000008000000P2YZJT2000000@0ZJVY3@00
000A0:VYZ@@00000102YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT400000080ZJVY0P00
00020:VYZ@8000000P2YZJT9000000H0ZJVY3@0000020:VYZ@`000000P2YZJT00`000000ZJVY0:VY
Z@04000000@0ZJVY100000020:VYZ@L00000102YZJT8000000/0ZJVY0P0000040:VYZ@8000000P2Y
ZJT300000080ZJVY3P0000020:VYZ@T000000P2YZJT400000080ZJVY2`0000020:VYZ@P000000P2Y
ZJT300000080ZJVY0P0000060:VYZ@8000000P2YZJT4000000/0ZJVY3?ooool000_ooooo20000002
0:VYZ@<000000P2YZJT:00000080ZJVY2`0000020:VYZ@@000000P2YZJT;00000080ZJVY20000009
0:VYZ@P000001P2YZJT;000000H0ZJVY0P0000000`2YZJT000000000000200000080ZJVY0P000004
0:VYZ@d000004@2YZJT4000000@0ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY10000002
0:VYZ@8000000P2YZJT200000080ZJVY2@0000060:VYZ@d000000P2YZJT<00000080ZJVY00<00000
0:VYZ@2YZJT0100000040:VYZ@@000000P2YZJT7000000@0ZJVY2000000;0:VYZ@800000102YZJT2
00000080ZJVY0`0000020:VYZ@h000000P2YZJT900000080ZJVY100000020:VYZ@/000000P2YZJT8
00000080ZJVY0`0000020:VYZ@8000001P2YZJT200000080ZJVY1000000;0:VYZ@cooooo000;oooo
o`80ZJVY0P0000020:VYZ@L000000P2YZJT>000000030:VYZ@000000000000@0ZJVY0P0000020:VY
Z@@000000P2YZJT4000000030:VYZ@000000000000@000000P2YZJT200000080ZJVY2@0000060:VY
Z@P000001`2YZJT2000000@0ZJVY2P0000000`2YZJT000000000000<00000080ZJVY100000000`2Y
ZJT0000000000004000000`0ZJVY3@0000020:VYZ@d000000P2YZJT200000080ZJVY0P0000020:VY
Z@@000000`2YZJT200000080ZJVY1P0000020:VYZ@@0000000<0ZJVY0000000000000P0000040:VY
Z@@000000P2YZJT400000080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@@000000P2YZJT20000
0080ZJVY3@0000020:VYZ@P000001`2YZJT2000000@0ZJVY2`0000020:VYZ@@000000P2YZJT40000
0080ZJVY2@0000020:VYZ@P000000P2YZJT300000080ZJVY4@0000020:VYZ@8000001P2YZJT20000
0080ZJVY1000000;0:VYZ@cooooo000;ooooo`80ZJVY0P0000020:VYZ@L000000P2YZJT>00000003
0:VYZ@000000000000@0ZJVY0P0000020:VYZ@@000000P2YZJT4000000030:VYZ@000000000000@0
00000P2YZJT200000080ZJVY2@0000060:VYZ@P000001`2YZJT2000000@0ZJVY2P0000000`2YZJT0
00000000000<00000080ZJVY100000000`2YZJT0000000000004000000`0ZJVY3@0000020:VYZ@d0
00000P2YZJT200000080ZJVY0P0000020:VYZ@@000000`2YZJT200000080ZJVY1P0000020:VYZ@@0
000000<0ZJVY0000000000000P0000040:VYZ@@000000P2YZJT400000080ZJVY00<000000:VYZ@2Y
ZJT00P0000020:VYZ@@000000P2YZJT200000080ZJVY3@0000020:VYZ@P000001`2YZJT2000000@0
ZJVY2`0000020:VYZ@@000000P2YZJT400000080ZJVY2@0000020:VYZ@P000000P2YZJT300000080
ZJVY4@0000020:VYZ@8000001P2YZJT200000080ZJVY1000000;0:VYZ@cooooo000;ooooo`@0ZJVY
100000020:VYZ@L000000P2YZJT:000000030:VYZ@000000000000H00000102YZJT200000080ZJVY
0P0000020:VYZ@D00000102YZJT800000080ZJVY2@0000020:VYZ@8000000P2YZJT200000080ZJVY
0`0000020:VYZ@8000001P2YZJT200000080ZJVY3@0000020:VYZ@@000000P2YZJT4000000030:VY
Z@00000000000100ZJVY00<000000:VYZ@2YZJT0100000020:VYZAL000000P2YZJT6000000L0ZJVY
200000020:VYZ@8000000`2YZJT200000080ZJVY200000020:VYZ@@000000P2YZJT00`000000ZJVY
0:VYZ@040:VYZ@P000000P2YZJT700000080ZJVY0P0000020:VYZ@H000000P2YZJT9000000P0ZJVY
0P0000020:VYZ@030000002YZJT0ZJVY008000000P2YZJT400000080ZJVY100000020:VYZ@<00000
0P2YZJT:00000080ZJVY0P0000020:VYZ@<000000P2YZJT2000000@0ZJVY2P0000030:VYZ@800000
0P2YZJT800000080ZJVY0P00000;0:VYZ@cooooo000;ooooo`@0ZJVY100000020:VYZ@L000000P2Y
ZJT:000000030:VYZ@000000000000H00000102YZJT200000080ZJVY0P0000020:VYZ@D00000102Y
ZJT800000080ZJVY2@0000020:VYZ@8000000P2YZJT200000080ZJVY0`0000020:VYZ@8000001P2Y
ZJT200000080ZJVY3@0000020:VYZ@@000000P2YZJT4000000030:VYZ@00000000000100ZJVY00<0
00000:VYZ@2YZJT0100000020:VYZAL000000P2YZJT6000000L0ZJVY200000020:VYZ@8000000`2Y
ZJT200000080ZJVY200000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@040:VYZ@P000000P2Y
ZJT700000080ZJVY0P0000020:VYZ@H000000P2YZJT9000000P0ZJVY0P0000020:VYZ@030000002Y
ZJT0ZJVY008000000P2YZJT400000080ZJVY100000020:VYZ@<000000P2YZJT:00000080ZJVY0P00
00020:VYZ@<000000P2YZJT2000000@0ZJVY2P0000030:VYZ@8000000P2YZJT800000080ZJVY0P00
000;0:VYZ@cooooo000;ooooo`H0ZJVY1@0000020:VYZ@800000102YZJT400000080ZJVY0P000002
0:VYZ@<000000P2YZJT600000080ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0A
00000080ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT6
000000H0ZJVY0P0000020:VYZ@<000000P2YZJT400000080ZJVY100000040:VYZ@8000005P2YZJT2
00000080ZJVY200000040:VYZ@<000000P2YZJT4000000H0ZJVY0P00000;0:VYZ@H00000102YZJT2
00000080ZJVY1@0000020:VYZ@8000000P2YZJT:000000D0ZJVY100000040:VYZ@H000000`2YZJT6
00000080ZJVY1P0000040:VYZ@030000002YZJT0ZJVY008000001P2YZJT600000080ZJVY1@000002
0:VYZ@@000000P2YZJT400000080ZJVY1`0000040:VYZ@P000000`2YZJT<00000080ZJVY0P000002
0:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@X000003@2YZJT<ooooo`002oooool40:VY
Z@L000000P2YZJT200000080ZJVY1P0000020:VYZ@L000000P2YZJT4000000H0ZJVY1`0000020:VY
ZAL000001P2YZJT400000080ZJVY0`0000020:VYZ@@000002P2YZJT500000080ZJVY2P0000040:VY
Z@8000003`2YZJT900000080ZJVY100000020:VYZ@P000000`2YZJT2000000P0ZJVY0P00000?0:VY
Z@8000000P2YZJT4000000<0ZJVY0P0000040:VYZ@800000102YZJT200000080ZJVY2@0000020:VY
Z@8000000P2YZJT400000080ZJVY0P0000050:VYZ@800000102YZJT200000080ZJVY0P0000040:VY
Z@<00000202YZJT200000080ZJVY0P0000030:VYZ@P00000102YZJT8000000<0ZJVY2P0000020:VY
Z@@0000000<0ZJVY0000000000000P2YZJT800000080ZJVY0P0000020:VYZ@80000000<0ZJVY0000
000000000P0000040:VYZ@d00000202YZJT<ooooo`002oooool40:VYZ@L000000P2YZJT200000080
ZJVY1P0000020:VYZ@L000000P2YZJT4000000H0ZJVY1`0000020:VYZAL000001P2YZJT400000080
ZJVY0`0000020:VYZ@@000002P2YZJT500000080ZJVY2P0000040:VYZ@8000003`2YZJT900000080
ZJVY100000020:VYZ@P000000`2YZJT2000000P0ZJVY0P00000?0:VYZ@8000000P2YZJT4000000<0
ZJVY0P0000040:VYZ@800000102YZJT200000080ZJVY2@0000020:VYZ@8000000P2YZJT400000080
ZJVY0P0000050:VYZ@800000102YZJT200000080ZJVY0P0000040:VYZ@<00000202YZJT200000080
ZJVY0P0000030:VYZ@P00000102YZJT8000000<0ZJVY2P0000020:VYZ@@0000000<0ZJVY00000000
00000P2YZJT800000080ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P0000040:VYZ@d0
0000202YZJT<ooooo`002oooool40:VYZ@L000000P2YZJT8000000P0ZJVY00<000000:VYZ@2YZJT0
0P2YZJT200000080ZJVY200000020:VYZ@<000000P2YZJT600000080ZJVY100000020:VYZ@<00000
0P2YZJT:00000080ZJVY0P0000020:VYZ@8000000`2YZJT200000080ZJVY200000020:VYZ@800000
00<0ZJVY000000000000100000020:VYZ@P000000P2YZJT200000140ZJVY1`0000020:VYZA400000
1P2YZJT<000000T0ZJVY200000030:VYZ@800000102YZJT8000000@0ZJVY1@0000040:VYZ@800000
102YZJT200000080ZJVY1`0000060:VYZ@H00000102YZJT00`000000ZJVY0:VYZ@0:00000080ZJVY
100000000`2YZJT00000000000040:VYZ@800000102YZJT400000080ZJVY2`0000020:VYZ@800000
0P2YZJT2000000<0ZJVY0P0000020:VYZ@800000102YZJT:000000030:VYZ@00000000000080ZJVY
1P0000020:VYZ@@000000`2YZJT4000000H0ZJVY3?ooool000_ooooo102YZJT700000080ZJVY2000
00080:VYZ@030000002YZJT0ZJVY0080ZJVY0P0000020:VYZ@P000000P2YZJT300000080ZJVY1P00
00020:VYZ@@000000P2YZJT300000080ZJVY2P0000020:VYZ@8000000P2YZJT2000000<0ZJVY0P00
00020:VYZ@P000000P2YZJT2000000030:VYZ@000000000000@000000P2YZJT800000080ZJVY0P00
000A0:VYZ@L000000P2YZJTA000000H0ZJVY300000090:VYZ@P000000`2YZJT2000000@0ZJVY2000
00040:VYZ@D00000102YZJT2000000@0ZJVY0P0000020:VYZ@L000001P2YZJT6000000@0ZJVY00<0
00000:VYZ@2YZJT02P0000020:VYZ@@0000000<0ZJVY000000000000102YZJT2000000@0ZJVY1000
00020:VYZ@/000000P2YZJT200000080ZJVY0P0000030:VYZ@8000000P2YZJT2000000@0ZJVY2P00
00000`2YZJT00000000000020:VYZ@H000000P2YZJT4000000<0ZJVY100000060:VYZ@cooooo000;
ooooo`P0ZJVY0P0000000`2YZJT000000000000400000080ZJVY0P0000020:VYZ@L00000102YZJT2
00000080ZJVY0P0000020:VYZ@@000000P2YZJT300000080ZJVY0P0000020:VYZ@8000000P2YZJT2
00000080ZJVY1@0000020:VYZ@@00000102YZJT2000000H0ZJVY2`0000020:VYZ@@000000P2YZJT=
00000080ZJVY0P0000020:VYZ@D000004P2YZJT500000080ZJVY0P0000020:VYZ@@000000P2YZJT4
000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@@000000P2YZJT3000000P0ZJVY0P000004
0:VYZ@8000000`2YZJT800000080ZJVY2P0000000`2YZJT00000000000020:VYZ@`000000P2YZJT0
0`000000ZJVY0:VYZ@0:00000080ZJVY0P0000020:VYZ@L000000P2YZJT:000000<0ZJVY0P000004
0:VYZ@800000102YZJT400000080ZJVY0`0000020:VYZ@H000000P2YZJT8000000030:VYZ@000000
00000080ZJVY2P0000020:VYZ@@0000000<0ZJVY000000000000200000020:VYZ@8000000P2YZJT=
000000cooooo000;ooooo`P0ZJVY0P0000000`2YZJT000000000000400000080ZJVY0P0000020:VY
Z@L00000102YZJT200000080ZJVY0P0000020:VYZ@@000000P2YZJT300000080ZJVY0P0000020:VY
Z@8000000P2YZJT200000080ZJVY1@0000020:VYZ@@00000102YZJT2000000H0ZJVY2`0000020:VY
Z@@000000P2YZJT=00000080ZJVY0P0000020:VYZ@D000004P2YZJT500000080ZJVY0P0000020:VY
Z@@000000P2YZJT4000000030:VYZ@00000000000080ZJVY1P0000020:VYZ@@000000P2YZJT30000
00P0ZJVY0P0000040:VYZ@8000000`2YZJT800000080ZJVY2P0000000`2YZJT00000000000020:VY
Z@`000000P2YZJT00`000000ZJVY0:VYZ@0:00000080ZJVY0P0000020:VYZ@L000000P2YZJT:0000
00<0ZJVY0P0000040:VYZ@800000102YZJT400000080ZJVY0`0000020:VYZ@H000000P2YZJT80000
00030:VYZ@00000000000080ZJVY2P0000020:VYZ@@0000000<0ZJVY000000000000200000020:VY
Z@8000000P2YZJT=000000cooooo000;ooooo`X0000000<0ZJVY0000000000000P2YZJT4000000H0
ZJVY0P0000020:VYZ@L000000P2YZJT6000000H0ZJVY1@0000020:VYZ@H000000P2YZJT500000080
ZJVY200000060:VYZ@T000000P2YZJT200000080ZJVY100000040:VYZ@l000000P2YZJT;000000d0
ZJVY0P0000040:VYZ@`000000`2YZJT8000000@0ZJVY100000040:VYZ@<00000202YZJT6000000<0
ZJVY1P0000060:VYZ@/000000P2YZJT?000000@0ZJVY1P0000040:VYZ@L000000P2YZJT6000000@0
ZJVY2@0000040:VYZ@P00000102YZJT500000080ZJVY0P0000020:VYZ@@000000P2YZJT400000003
0:VYZ@000000000000H000000P2YZJT:000000030:VYZ@000000000000800000102YZJT200000080
ZJVY0P0000020:VYZ@L000000P2YZJT200000080ZJVY3?ooool000_ooooo2P0000000`2YZJT00000
000000020:VYZ@@000001P2YZJT200000080ZJVY1`0000020:VYZ@H000001P2YZJT500000080ZJVY
1P0000020:VYZ@D000000P2YZJT8000000H0ZJVY2@0000020:VYZ@8000000P2YZJT4000000@0ZJVY
3`0000020:VYZ@/000003@2YZJT2000000@0ZJVY300000030:VYZ@P00000102YZJT4000000@0ZJVY
0`0000080:VYZ@H000000`2YZJT6000000H0ZJVY2`0000020:VYZ@l00000102YZJT6000000@0ZJVY
1`0000020:VYZ@H00000102YZJT9000000@0ZJVY200000040:VYZ@D000000P2YZJT200000080ZJVY
100000020:VYZ@@0000000<0ZJVY0000000000001P0000020:VYZ@X0000000<0ZJVY000000000000
0P0000040:VYZ@8000000P2YZJT200000080ZJVY1`0000020:VYZ@8000000P2YZJT<ooooo`002ooo
ool20:VYZ@P0000000<0ZJVY0000000000001P0000080:VYZ@<00000102YZJT2000000H0ZJVY0P00
00020:VYZ@L00000102YZJT200000080ZJVY3@0000020:VYZ@H00000202YZJT00`000000ZJVY0:VY
Z@0200000080ZJVY0P0000040:VYZA/000000`2YZJT:000000L0ZJVY0P0000040:VYZ@@000000P2Y
ZJT200000080ZJVY0P0000030:VYZ@800000102YZJT2000000H0ZJVY0P0000050:VYZ@8000000P2Y
ZJT800000080ZJVY1@0000020:VYZ@800000202YZJT2000000@0ZJVY0P0000070:VYZ@@000000P2Y
ZJT6000000L0ZJVY1P0000020:VYZ@8000000P2YZJT3000000@0ZJVY0P0000080:VYZ@@0000000<0
ZJVY000000000000300000040:VYZ@80000000<0ZJVY0000000000000P2YZJT200000080ZJVY1P00
00020:VYZ@8000000`2YZJT200000080ZJVY3`0000020:VYZ@800000102YZJT600000080ZJVY2`00
00020:VYZ@cooooo000;ooooo`80ZJVY200000000`2YZJT0000000000006000000P0ZJVY0`000004
0:VYZ@8000001P2YZJT200000080ZJVY1`0000040:VYZ@8000000P2YZJT=00000080ZJVY1P000008
0:VYZ@030000002YZJT0ZJVY008000000P2YZJT2000000@0ZJVY6`0000030:VYZ@X000001`2YZJT2
000000@0ZJVY100000020:VYZ@8000000P2YZJT2000000<0ZJVY0P0000040:VYZ@8000001P2YZJT2
000000D0ZJVY0P0000020:VYZ@P000000P2YZJT500000080ZJVY0P0000080:VYZ@800000102YZJT2
000000L0ZJVY100000020:VYZ@H000001`2YZJT600000080ZJVY0P0000020:VYZ@<00000102YZJT2
000000P0ZJVY100000000`2YZJT000000000000<000000@0ZJVY0P0000000`2YZJT0000000000002
0:VYZ@8000000P2YZJT600000080ZJVY0P0000030:VYZ@8000000P2YZJT?00000080ZJVY0P000004
0:VYZ@H000000P2YZJT;00000080ZJVY3?ooool000_ooooo0P2YZJT200000080ZJVY4`0000020:VY
Z@8000001@2YZJT2000000H0ZJVY0P0000020:VYZ@@0000000<0ZJVY0000000000000P0000020:VY
Z@8000000P2YZJT4000000D0ZJVY0P0000040:VYZ@@00000202YZJT700000080ZJVY2P0000020:VY
Z@D000000P2YZJT400000080ZJVY0P0000040:VYZA4000001@2YZJT:00000080ZJVY0P0000090:VY
Z@X000000P2YZJT2000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VY
Z@030000002YZJT0ZJVY008000000P2YZJT8000000@0ZJVY0P0000090:VYZ@@000000P2YZJT20000
0080ZJVY2@0000020:VYZ@/00000102YZJT200000080ZJVY200000030:VYZ@8000000P2YZJT20000
0080ZJVY1P0000040:VYZ@80000000<0ZJVY0000000000000P2YZJT200000080ZJVY0P0000020:VY
Z@8000000P2YZJT700000080ZJVY0P0000020:VYZ@8000000P2YZJT6000000<0ZJVY0P0000020:VY
Z@P000000P2YZJT500000080ZJVY0P0000040:VYZ@cooooo000;ooooo`80ZJVY0P0000020:VYZA<0
00000P2YZJT2000000D0ZJVY0P0000060:VYZ@8000000P2YZJT4000000030:VYZ@00000000000080
00000P2YZJT200000080ZJVY100000050:VYZ@800000102YZJT4000000P0ZJVY1`0000020:VYZ@X0
00000P2YZJT500000080ZJVY100000020:VYZ@800000102YZJTA000000D0ZJVY2P0000020:VYZ@80
00002@2YZJT:00000080ZJVY0P0000050:VYZ@8000000P2YZJT200000080ZJVY0P0000040:VYZ@80
00000P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY200000040:VYZ@8000002@2YZJT400000080
ZJVY0P0000020:VYZ@T000000P2YZJT;000000@0ZJVY0P0000020:VYZ@P000000`2YZJT200000080
ZJVY0P0000020:VYZ@H00000102YZJT2000000030:VYZ@00000000000080ZJVY0P0000020:VYZ@80
00000P2YZJT200000080ZJVY1`0000020:VYZ@8000000P2YZJT200000080ZJVY1P0000030:VYZ@80
00000P2YZJT800000080ZJVY1@0000020:VYZ@800000102YZJT<ooooo`002oooool6000000D0ZJVY
0P0000020:VYZ@X000000P2YZJT3000000H0ZJVY3P0000030:VYZ@8000000P2YZJT400000080ZJVY
0P0000020:VYZ@T000000P2YZJT200000080ZJVY200000030:VYZ@800000102YZJT400000080ZJVY
2@0000020:VYZ@800000102YZJTC00000080ZJVY2`0000020:VYZ@P000001P2YZJT500000080ZJVY
200000060:VYZ@<000000P2YZJT6000000@0ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00800000
0P2YZJT400000080ZJVY200000050:VYZ@P000000P2YZJT700000080ZJVY3P0000020:VYZ@030000
002YZJT0ZJVY00@000000P2YZJT8000000<0ZJVY0P0000020:VYZ@H000000P2YZJT2000000@0ZJVY
0P0000000`2YZJT000000000000400000080ZJVY0P0000090:VYZ@8000000P2YZJT400000080ZJVY
0P0000020:VYZ@H000000`2YZJT200000080ZJVY0P0000040:VYZ@800000102YZJT300000080ZJVY
0P0000020:VYZ@8000003?ooool000_ooooo1P0000050:VYZ@8000000P2YZJT:00000080ZJVY0`00
00060:VYZ@h000000`2YZJT200000080ZJVY100000020:VYZ@8000000P2YZJT900000080ZJVY0P00
00020:VYZ@P000000`2YZJT2000000@0ZJVY100000020:VYZ@T000000P2YZJT2000000@0ZJVY4`00
00020:VYZ@/000000P2YZJT8000000H0ZJVY1@0000020:VYZ@P000001P2YZJT300000080ZJVY1P00
00040:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY100000020:VYZ@P000001@2Y
ZJT800000080ZJVY1`0000020:VYZ@h000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY2000
00030:VYZ@8000000P2YZJT600000080ZJVY0P0000040:VYZ@80000000<0ZJVY0000000000001000
00020:VYZ@8000002@2YZJT200000080ZJVY100000020:VYZ@8000000P2YZJT6000000<0ZJVY0P00
00020:VYZ@800000102YZJT2000000@0ZJVY0`0000020:VYZ@8000000P2YZJT2000000cooooo000;
ooooo`@0ZJVY2@0000020:VYZ@8000000P2YZJT6000000@0ZJVY1`0000020:VYZ@8000000P2YZJT4
00000080ZJVY0P0000030:VYZ@P000000P2YZJT200000080ZJVY0P0000000`2YZJT0000000000002
000000@0ZJVY1P0000020:VYZ@8000000P2YZJT5000000H0ZJVY4@0000040:VYZA400000102YZJT6
000000030:VYZ@000000000000@000000P2YZJT200000080ZJVY0P0000020:VYZ@T000000P2YZJT4
00000080ZJVY200000050:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VYZ@030000002YZJT0
ZJVY00P000000P2YZJT400000080ZJVY2`0000020:VYZ@/000000P2YZJT4000000@0ZJVY0P000006
0:VYZ@030000002YZJT0ZJVY00@000000P2YZJT8000000<0ZJVY0P0000020:VYZ@8000000P2YZJT6
000000@0ZJVY2`00000:0:VYZ@<00000102YZJT:00000080ZJVY1`0000020:VYZ@d00000202YZJT2
000000cooooo000;ooooo`@0ZJVY2@0000020:VYZ@8000000P2YZJT6000000@0ZJVY1`0000020:VY
Z@8000000P2YZJT400000080ZJVY0P0000030:VYZ@P000000P2YZJT200000080ZJVY0P0000000`2Y
ZJT0000000000002000000@0ZJVY1P0000020:VYZ@8000000P2YZJT5000000H0ZJVY4@0000040:VY
ZA400000102YZJT6000000030:VYZ@000000000000@000000P2YZJT200000080ZJVY0P0000020:VY
Z@T000000P2YZJT400000080ZJVY200000050:VYZ@8000000P2YZJT2000000@0ZJVY0P0000020:VY
Z@030000002YZJT0ZJVY00P000000P2YZJT400000080ZJVY2`0000020:VYZ@/000000P2YZJT40000
00@0ZJVY0P0000060:VYZ@030000002YZJT0ZJVY00@000000P2YZJT8000000<0ZJVY0P0000020:VY
Z@8000000P2YZJT6000000@0ZJVY2`00000:0:VYZ@<00000102YZJT:00000080ZJVY1`0000020:VY
Z@d00000202YZJT2000000cooooo000;ooooo`80ZJVY1P0000020:VYZ@030000002YZJT0ZJVY0140
00000P2YZJT800000080ZJVY0P0000020:VYZ@/000000P2YZJT200000080ZJVY0P0000020:VYZ@T0
00000P2YZJT200000080ZJVY0P0000020:VYZ@8000000`2YZJT200000080ZJVY200000020:VYZ@@0
000000<0ZJVY0000000000000P0000020:VYZAd000000P2YZJT500000080ZJVY2P0000020:VYZ@@0
000000<0ZJVY0000000000000P0000040:VYZ@8000000P2YZJT400000080ZJVY1@0000020:VYZ@80
00000P2YZJT200000080ZJVY2`0000020:VYZ@@000000P2YZJT8000000030:VYZ@000000000000H0
ZJVY100000020:VYZ@@0000000<0ZJVY000000000000100000060:VYZ@/000000P2YZJT200000080
ZJVY200000050:VYZ@@000000P2YZJT6000000@0ZJVY0P0000030:VYZ@800000202YZJT9000000@0
ZJVY0P0000020:VYZ@8000002@2YZJT200000080ZJVY100000020:VYZ@@000000P2YZJT200000003
0:VYZ@000000000000@000000P2YZJT2000000cooooo000;ooooo`80ZJVY1P0000020:VYZ@030000
002YZJT0ZJVY014000000P2YZJT800000080ZJVY0P0000020:VYZ@/000000P2YZJT200000080ZJVY
0P0000020:VYZ@T000000P2YZJT200000080ZJVY0P0000020:VYZ@8000000`2YZJT200000080ZJVY
200000020:VYZ@@0000000<0ZJVY0000000000000P0000020:VYZAd000000P2YZJT500000080ZJVY
2P0000020:VYZ@@0000000<0ZJVY0000000000000P0000040:VYZ@8000000P2YZJT400000080ZJVY
1@0000020:VYZ@8000000P2YZJT200000080ZJVY2`0000020:VYZ@@000000P2YZJT8000000030:VY
Z@000000000000H0ZJVY100000020:VYZ@@0000000<0ZJVY000000000000100000060:VYZ@/00000
0P2YZJT200000080ZJVY200000050:VYZ@@000000P2YZJT6000000@0ZJVY0P0000030:VYZ@800000
202YZJT9000000@0ZJVY0P0000020:VYZ@8000002@2YZJT200000080ZJVY100000020:VYZ@@00000
0P2YZJT2000000030:VYZ@000000000000@000000P2YZJT2000000cooooo000;ooooo`80ZJVY0P00
00020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0@000000<0ZJVY0P0000020:VYZ@8000000P2Y
ZJT400000080ZJVY100000030:VYZ@@000000P2YZJT:000000030:VYZ@000000000000P000000P2Y
ZJT8000000030:VYZ@00000000000080ZJVY200000020:VYZ@@0000000<0ZJVY0000000000008@00
00020:VYZ@D000000P2YZJT400000080ZJVY1P0000020:VYZ@8000000`2YZJT200000080ZJVY1000
00020:VYZ@8000000P2YZJT4000000030:VYZ@000000000000@000000P2YZJT?00000080ZJVY1000
00040:VYZ@L000001P2YZJT600000080ZJVY100000000`2YZJT0000000000004000000P0ZJVY1000
00020:VYZ@/000000P2YZJT700000080ZJVY0P0000020:VYZ@@000000P2YZJT8000000<0ZJVY0P00
00020:VYZ@X000000P2YZJT900000080ZJVY1P0000050:VYZ@8000000P2YZJT2000000@0ZJVY1000
00020:VYZ@<000000P2YZJT2000000@0ZJVY0P00000<ooooo`002oooool20:VYZ@@000001@2YZJT4
000000@0ZJVY0P0000020:VYZ@8000001`2YZJT600000080ZJVY0P0000040:VYZ@8000000P2YZJT7
00000080ZJVY100000020:VYZ@D000000P2YZJT200000080ZJVY4@0000040:VYZ@8000000P2YZJT2
00000080ZJVY100000000`2YZJT000000000000<00000080ZJVY0P0000020:VYZ@030000002YZJT0
ZJVY00H00000102YZJT900000080ZJVY100000020:VYZ@@00000102YZJT700000080ZJVY10000002
0:VYZ@P000000`2YZJT200000080ZJVY1P0000020:VYZ@@0000000<0ZJVY0000000000000P000004
0:VYZA4000000P2YZJT2000000H0ZJVY0P0000020:VYZ@<000000P2YZJT200000080ZJVY1P000002
0:VYZ@8000000P2YZJTA00000080ZJVY00<000000:VYZ@2YZJT0200000020:VYZ@@000000P2YZJT2
000000<0ZJVY3P0000020:VYZ@<00000102YZJT:000000L0ZJVY100000040:VYZ@@000000P2YZJT5
000000H0ZJVY0P00000<ooooo`002oooool20:VYZ@@000001@2YZJT4000000@0ZJVY0P0000020:VY
Z@8000001`2YZJT600000080ZJVY0P0000040:VYZ@8000000P2YZJT700000080ZJVY100000020:VY
Z@D000000P2YZJT200000080ZJVY4@0000040:VYZ@8000000P2YZJT200000080ZJVY100000000`2Y
ZJT000000000000<00000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00H00000102YZJT90000
0080ZJVY100000020:VYZ@@00000102YZJT700000080ZJVY100000020:VYZ@P000000`2YZJT20000
0080ZJVY1P0000020:VYZ@@0000000<0ZJVY0000000000000P0000040:VYZA4000000P2YZJT20000
00H0ZJVY0P0000020:VYZ@<000000P2YZJT200000080ZJVY1P0000020:VYZ@8000000P2YZJTA0000
0080ZJVY00<000000:VYZ@2YZJT0200000020:VYZ@@000000P2YZJT2000000<0ZJVY3P0000020:VY
Z@<00000102YZJT:000000L0ZJVY100000040:VYZ@@000000P2YZJT5000000H0ZJVY0P00000<oooo
o`002oooool40:VYZ@8000000P2YZJT=00000080ZJVY2`0000020:VYZ@P000000P2YZJT200000080
ZJVY00<000000:VYZ@2YZJT0100000040:VYZ@800000102YZJT300000080ZJVY1P0000020:VYZ@@0
00000P2YZJT=00000080ZJVY0P0000040:VYZ@8000000`2YZJT200000080ZJVY100000020:VYZ@T0
00000P2YZJT800000080ZJVY0P0000020:VYZ@D00000102YZJT2000000@0ZJVY0P0000040:VYZ@80
000000<0ZJVY0000000000000P0000020:VYZ@@000000P2YZJT400000080ZJVY0P0000030:VYZ@80
00001P2YZJT200000080ZJVY100000000`2YZJT000000000000400000080ZJVY0P0000020:VYZ@80
0000102YZJT700000080ZJVY2P0000020:VYZ@d00000102YZJT2000000<0ZJVY0P0000040:VYZ@80
00000P2YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT200000080ZJVY200000020:VYZAP0
0000102YZJT:000000L0ZJVY100000040:VYZ@8000000P2YZJT4000000030:VYZ@000000000000P0
00003?ooool000_ooooo102YZJT200000080ZJVY3@0000020:VYZ@/000000P2YZJT800000080ZJVY
0P0000020:VYZ@030000002YZJT0ZJVY00@00000102YZJT2000000@0ZJVY0`0000020:VYZ@H00000
0P2YZJT400000080ZJVY3@0000020:VYZ@800000102YZJT2000000<0ZJVY0P0000020:VYZ@@00000
0P2YZJT900000080ZJVY200000020:VYZ@8000000P2YZJT5000000@0ZJVY0P0000040:VYZ@800000
102YZJT2000000030:VYZ@0000000000008000000P2YZJT400000080ZJVY100000020:VYZ@800000
0`2YZJT2000000H0ZJVY0P0000020:VYZ@@0000000<0ZJVY000000000000100000020:VYZ@800000
0P2YZJT2000000@0ZJVY1`0000020:VYZ@X000000P2YZJT=000000@0ZJVY0P0000030:VYZ@800000
102YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY0P0000020:VYZ@P00000
0P2YZJTH000000@0ZJVY2P0000070:VYZ@@00000102YZJT200000080ZJVY100000000`2YZJT00000
00000008000000cooooo000;ooooo`H000000P2YZJT300000080ZJVY200000020:VYZ@8000000P2Y
ZJT2000000030:VYZ@00000000000080ZJVY100000020:VYZ@800000102YZJT500000080ZJVY1P00
00020:VYZ@800000102YZJT700000080ZJVY0P0000020:VYZA400000102YZJT=00000080ZJVY1000
00020:VYZ@@000000P2YZJT900000080ZJVY0P0000020:VYZ@l00000102YZJT2000000H0ZJVY1`00
00020:VYZ@8000000P2YZJT200000080ZJVY100000050:VYZ@X000000P2YZJT2000000030:VYZ@00
000000000080ZJVY2P0000040:VYZ@80000000<0ZJVY000000000000200000020:VYZAT000000`2Y
ZJT200000080ZJVY300000020:VYZ@<000000P2YZJT600000080ZJVY900000020:VYZ@8000000P2Y
ZJT900000080ZJVY0P0000040:VYZ@8000000P2YZJT700000080ZJVY1P00000<ooooo`002oooool6
00000080ZJVY0`0000020:VYZ@P000000P2YZJT200000080ZJVY0P0000000`2YZJT0000000000002
0:VYZ@@000000P2YZJT2000000@0ZJVY1@0000020:VYZ@H000000P2YZJT2000000@0ZJVY1`000002
0:VYZ@8000000P2YZJTA000000@0ZJVY3@0000020:VYZ@@000000P2YZJT400000080ZJVY2@000002
0:VYZ@8000000P2YZJT?000000@0ZJVY0P0000060:VYZ@L000000P2YZJT200000080ZJVY0P000002
0:VYZ@@000001@2YZJT:00000080ZJVY0P0000000`2YZJT00000000000020:VYZ@X00000102YZJT2
000000030:VYZ@000000000000P000000P2YZJTI000000<0ZJVY0P0000020:VYZ@`000000P2YZJT3
00000080ZJVY1P0000020:VYZB@000000P2YZJT200000080ZJVY2@0000020:VYZ@800000102YZJT2
00000080ZJVY1`0000020:VYZ@H000003?ooool000_ooooo0P0000020:VYZ@8000000P2YZJT30000
0080ZJVY100000020:VYZ@X0000000<0ZJVY0000000000002P0000040:VYZ@8000000P2YZJT70000
00@0ZJVY2@0000020:VYZ@H000000P2YZJT400000080ZJVY1`0000020:VYZAT000000P2YZJTC0000
0080ZJVY100000020:VYZ@D000000P2YZJT2000000@0ZJVY0P0000020:VYZ@D00000102YZJT20000
00H0ZJVY0P0000020:VYZ@T000001P2YZJT2000000@0ZJVY2@0000020:VYZ@8000000P2YZJT20000
0080ZJVY100000000`2YZJT000000000000200000080ZJVY4@0000020:VYZ@800000102YZJT20000
00T0ZJVY100000020:VYZ@P000000`2YZJT600000080ZJVY0P0000040:VYZ@@000000P2YZJTD0000
0080ZJVY2P0000020:VYZ@@000000P2YZJT300000080ZJVY200000040:VYZ@<00000102YZJT40000
0080ZJVY3?ooool000_ooooo0P0000020:VYZ@8000000P2YZJT300000080ZJVY100000020:VYZ@X0
000000<0ZJVY0000000000002P0000040:VYZ@8000000P2YZJT7000000@0ZJVY2@0000020:VYZ@H0
00000P2YZJT400000080ZJVY1`0000020:VYZAT000000P2YZJTC00000080ZJVY100000020:VYZ@D0
00000P2YZJT2000000@0ZJVY0P0000020:VYZ@D00000102YZJT2000000H0ZJVY0P0000020:VYZ@T0
00001P2YZJT2000000@0ZJVY2@0000020:VYZ@8000000P2YZJT200000080ZJVY100000000`2YZJT0
00000000000200000080ZJVY4@0000020:VYZ@800000102YZJT2000000T0ZJVY100000020:VYZ@P0
00000`2YZJT600000080ZJVY0P0000040:VYZ@@000000P2YZJTD00000080ZJVY2P0000020:VYZ@@0
00000P2YZJT300000080ZJVY200000040:VYZ@<00000102YZJT400000080ZJVY3?ooool000_ooooo
1P0000020:VYZ@<000000P2YZJT800000080ZJVY2@0000020:VYZ@@000000P2YZJT;000000@0ZJVY
200000020:VYZ@D00000102YZJT4000000@0ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY
7`0000020:VYZ@<000000P2YZJT8000000@0ZJVY1@0000020:VYZ@8000001P2YZJT800000080ZJVY
00<000000:VYZ@2YZJT00P2YZJT:000000@0ZJVY0`0000020:VYZ@8000000P2YZJT<000000<0ZJVY
0P0000020:VYZ@@000000P2YZJT200000080ZJVY100000000`2YZJT00000000000020:VYZ@@00000
0P2YZJT400000080ZJVY0P0000030:VYZ@H00000102YZJT2000000T0ZJVY100000060:VYZ@800000
0P2YZJT?00000080ZJVY0P0000020:VYZ@<000000P2YZJT<00000080ZJVY0`0000040:VYZ@X00000
0P2YZJT2000000<0ZJVY0P0000020:VYZ@800000102YZJT?000000@0ZJVY3?ooool000_ooooo1P00
00020:VYZ@<000000P2YZJT800000080ZJVY2@0000020:VYZ@@000000P2YZJT;000000@0ZJVY2000
00020:VYZ@D00000102YZJT4000000@0ZJVY0P0000020:VYZ@8000000P2YZJT300000080ZJVY7`00
00020:VYZ@<000000P2YZJT8000000@0ZJVY1@0000020:VYZ@8000001P2YZJT800000080ZJVY00<0
00000:VYZ@2YZJT00P2YZJT:000000@0ZJVY0`0000020:VYZ@8000000P2YZJT<000000<0ZJVY0P00
00020:VYZ@@000000P2YZJT200000080ZJVY100000000`2YZJT00000000000020:VYZ@@000000P2Y
ZJT400000080ZJVY0P0000030:VYZ@H00000102YZJT2000000T0ZJVY100000060:VYZ@8000000P2Y
ZJT?00000080ZJVY0P0000020:VYZ@<000000P2YZJT<00000080ZJVY0`0000040:VYZ@X000000P2Y
ZJT2000000<0ZJVY0P0000020:VYZ@800000102YZJT?000000@0ZJVY3?ooool000_ooooo0P2YZJT9
000000@0ZJVY1P0000040:VYZ@L000000P2YZJT2000000@0ZJVY0P0000020:VYZ@H000001@2YZJT2
00000080ZJVY100000040:VYZ@8000001@2YZJT4000000P0ZJVY0P0000020:VYZ@030000002YZJT0
ZJVY0100000000<0ZJVY000000000000100000020:VYZ@800000202YZJT2000000<0ZJVY0P000002
0:VYZ@X000000P2YZJT00`000000ZJVY0:VYZ@0600000080ZJVY0P0000020:VYZ@8000000P2YZJT;
00000080ZJVY100000040:VYZ@80000000<0ZJVY0000000000005@0000020:VYZ@@000000P2YZJT;
00000080ZJVY100000020:VYZ@H000001`2YZJT4000000@0ZJVY100000050:VYZ@@000001P2YZJT6
00000080ZJVY00<000000:VYZ@2YZJT0102YZJT:00000080ZJVY0`0000020:VYZ@P000000P2YZJT4
00000080ZJVY00<000000:VYZ@2YZJT01P0000020:VYZ@X0000000<0ZJVY0000000000000P2YZJT:
000000@0ZJVY1@0000060:VYZ@cooooo000;ooooo`80ZJVY2@0000040:VYZ@H00000102YZJT70000
0080ZJVY0P0000040:VYZ@8000000P2YZJT6000000D0ZJVY0P0000020:VYZ@@00000102YZJT20000
00D0ZJVY100000080:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0@000000030:VYZ@0000000000
00@000000P2YZJT2000000P0ZJVY0P0000030:VYZ@8000000P2YZJT:00000080ZJVY00<000000:VY
Z@2YZJT01P0000020:VYZ@8000000P2YZJT200000080ZJVY2`0000020:VYZ@@00000102YZJT20000
00030:VYZ@000000000001D000000P2YZJT400000080ZJVY2`0000020:VYZ@@000000P2YZJT60000
00L0ZJVY100000040:VYZ@@000001@2YZJT4000000H0ZJVY1P0000020:VYZ@030000002YZJT0ZJVY
00@0ZJVY2P0000020:VYZ@<000000P2YZJT800000080ZJVY100000020:VYZ@030000002YZJT0ZJVY
00H000000P2YZJT:000000030:VYZ@00000000000080ZJVY2P0000040:VYZ@D000001P2YZJT<oooo
o`002oooool200000080ZJVY0P0000040:VYZA4000000P2YZJT300000080ZJVY300000020:VYZ@/0
00000P2YZJT2000000@0ZJVY0P0000000`2YZJT000000000000200000080ZJVY0P00000:0:VYZ@L0
00000P2YZJT600000080ZJVY0P0000050:VYZ@@000000P2YZJT2000000P0ZJVY2`0000020:VYZ@H0
00000P2YZJT900000080ZJVY1P0000020:VYZ@<000000P2YZJT6000000@0ZJVY0P0000020:VYZB80
00000P2YZJT2000000@0ZJVY0`0000040:VYZ@@000000P2YZJT6000000T0ZJVY0P0000020:VYZ@X0
000000<0ZJVY000000000000202YZJT200000080ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00H0
ZJVY100000020:VYZ@L000000P2YZJT400000080ZJVY0P0000080:VYZ@030000002YZJT0ZJVY0080
ZJVY100000020:VYZ@@000000P2YZJT4000000030:VYZ@00000000000080ZJVY100000040:VYZ@L0
00000P2YZJT8000000cooooo000;ooooo`8000000P2YZJT2000000@0ZJVY4@0000020:VYZ@<00000
0P2YZJT<00000080ZJVY2`0000020:VYZ@800000102YZJT2000000030:VYZ@000000000000800000
0P2YZJT2000000X0ZJVY1`0000020:VYZ@H000000P2YZJT2000000D0ZJVY100000020:VYZ@800000
202YZJT;00000080ZJVY1P0000020:VYZ@T000000P2YZJT600000080ZJVY0`0000020:VYZ@H00000
102YZJT200000080ZJVY8P0000020:VYZ@800000102YZJT3000000@0ZJVY100000020:VYZ@H00000
2@2YZJT200000080ZJVY2P0000000`2YZJT00000000000080:VYZ@8000000P2YZJT200000080ZJVY
00<000000:VYZ@2YZJT01P2YZJT400000080ZJVY1`0000020:VYZ@@000000P2YZJT2000000P0ZJVY
00<000000:VYZ@2YZJT00P2YZJT400000080ZJVY100000020:VYZ@@0000000<0ZJVY000000000000
0P2YZJT4000000@0ZJVY1`0000020:VYZ@P000003?ooool000_ooooo0P0000020:VYZ@@000000P2Y
ZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT6000000@0ZJVY00<000000:VYZ@2YZJT01P2Y
ZJT200000080ZJVY100000040:VYZ@<00000102YZJT<000000030:VYZ@0000000000008000000P2Y
ZJT8000000L0ZJVY0P0000040:VYZ@H000000P2YZJT200000080ZJVY1`0000020:VYZ@@000001P2Y
ZJTA000000@0ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@800000102YZJT4000000@0ZJVY00<0
00000:VYZ@2YZJT00P2YZJT6000000H0ZJVY0P0000020:VYZ@030000002YZJT0ZJVY00800000102Y
ZJTE00000080ZJVY1P0000020:VYZ@030000002YZJT0ZJVY00H000000P2YZJT4000000l0ZJVY0P00
00040:VYZ@8000000P2YZJT900000080ZJVY0P0000020:VYZ@8000000P2YZJT9000000H0ZJVY0P00
00020:VYZ@<000000P2YZJT400000080ZJVY2`0000040:VYZ@@000000P2YZJT400000080ZJVY1000
00000`2YZJT00000000000020:VYZ@8000000P2YZJT:000000<0ZJVY100000020:VYZ@8000003?oo
ool000_ooooo0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT6
000000@0ZJVY00<000000:VYZ@2YZJT01P2YZJT200000080ZJVY100000040:VYZ@<00000102YZJT<
000000030:VYZ@0000000000008000000P2YZJT8000000L0ZJVY0P0000040:VYZ@H000000P2YZJT2
00000080ZJVY1`0000020:VYZ@@000001P2YZJTA000000@0ZJVY00<000000:VYZ@2YZJT00P000002
0:VYZ@800000102YZJT4000000@0ZJVY00<000000:VYZ@2YZJT00P2YZJT6000000H0ZJVY0P000002
0:VYZ@030000002YZJT0ZJVY00800000102YZJTE00000080ZJVY1P0000020:VYZ@030000002YZJT0
ZJVY00H000000P2YZJT4000000l0ZJVY0P0000040:VYZ@8000000P2YZJT900000080ZJVY0P000002
0:VYZ@8000000P2YZJT9000000H0ZJVY0P0000020:VYZ@<000000P2YZJT400000080ZJVY2`000004
0:VYZ@@000000P2YZJT400000080ZJVY100000000`2YZJT00000000000020:VYZ@8000000P2YZJT:
000000<0ZJVY100000020:VYZ@8000003?ooool000_ooooo0P0000020:VYZ@@000000P2YZJT00`00
0000ZJVY0:VYZ@0@000000d0ZJVY0P0000020:VYZ@D000001P2YZJT200000080ZJVY0P0000040:VY
Z@80000000<0ZJVY0000000000000P0000040:VYZ@8000000P2YZJT2000000H0ZJVY2@0000020:VY
Z@8000000P2YZJT500000080ZJVY100000040:VYZ@8000000P2YZJTJ00000080ZJVY0P0000020:VY
Z@8000000P2YZJT6000000@0ZJVY1`0000020:VYZ@8000000P2YZJT600000080ZJVY1@0000080:VY
Z@800000102YZJT00`000000ZJVY0:VYZ@0800000080ZJVY100000040:VYZ@l000004@2YZJT60000
00<0ZJVY100000020:VYZ@H000000P2YZJT4000000030:VYZ@000000000000@0ZJVY100000060:VY
Z@D000000P2YZJTG000000@0ZJVY100000020:VYZ@@0000000<0ZJVY0000000000000P2YZJT40000
0080ZJVY0P0000040:VYZ@d000003?ooool000_ooooo0P0000020:VYZ@@000000P2YZJT00`000000
ZJVY0:VYZ@0@000000d0ZJVY0P0000020:VYZ@D000001P2YZJT200000080ZJVY0P0000040:VYZ@80
000000<0ZJVY0000000000000P0000040:VYZ@8000000P2YZJT2000000H0ZJVY2@0000020:VYZ@80
00000P2YZJT500000080ZJVY100000040:VYZ@8000000P2YZJTJ00000080ZJVY0P0000020:VYZ@80
00000P2YZJT6000000@0ZJVY1`0000020:VYZ@8000000P2YZJT600000080ZJVY1@0000080:VYZ@80
0000102YZJT00`000000ZJVY0:VYZ@0800000080ZJVY100000040:VYZ@l000004@2YZJT6000000<0
ZJVY100000020:VYZ@H000000P2YZJT4000000030:VYZ@000000000000@0ZJVY100000060:VYZ@D0
00000P2YZJTG000000@0ZJVY100000020:VYZ@@0000000<0ZJVY0000000000000P2YZJT400000080
ZJVY0P0000040:VYZ@d000003?ooool000_ooooo100000020:VYZ@8000000P2YZJT00`000000ZJVY
0:VYZ@0@000000d0ZJVY0P0000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0600000080ZJVY
3@0000040:VYZ@8000002`2YZJT200000080ZJVY100000020:VYZ@T00000102YZJT800000080ZJVY
3`0000020:VYZ@8000000P2YZJT200000080ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@800000
0P2YZJT;00000080ZJVY100000020:VYZ@X000001@2YZJT200000080ZJVY100000020:VYZ@800000
0P2YZJT700000080ZJVY100000040:VYZ@800000102YZJT=000001L0ZJVY0P0000020:VYZ@<00000
1P2YZJT4000000@0ZJVY0P0000000`2YZJT00000000000040:VYZ@`000000P2YZJT300000080ZJVY
0P0000020:VYZ@@000000P2YZJT4000000030:VYZ@000000000000X00000102YZJT4000000030:VY
Z@000000000000@0ZJVY0P0000020:VYZ@8000000P2YZJT700000080ZJVY100000020:VYZ@cooooo
000;ooooo`@000000P2YZJT5000000@0ZJVY100000020:VYZ@P000003`2YZJT400000080ZJVY00<0
00000:VYZ@2YZJT01P0000020:VYZ@@000000P2YZJT300000080ZJVY100000020:VYZ@800000202Y
ZJT;00000080ZJVY1P0000020:VYZ@<000000P2YZJT200000080ZJVY200000040:VYZ@<000000P2Y
ZJT<00000080ZJVY2@0000040:VYZ@@000000P2YZJT=00000080ZJVY100000070:VYZ@P000000P2Y
ZJT700000080ZJVY1P0000060:VYZ@800000102YZJT00`000000ZJVY0:VYZ@020:VYZ@@000000P2Y
ZJT4000001T0ZJVY00<000000:VYZ@2YZJT0400000000`2YZJT000000000000:00000080ZJVY0P00
00040:VYZ@<000000P2YZJT400000080ZJVY0P0000040:VYZA<000000P2YZJT2000000030:VYZ@00
000000000080ZJVY100000020:VYZ@P0000000<0ZJVY000000000000100000040:VYZ@cooooo000;
ooooo`@000000P2YZJT5000000@0ZJVY100000020:VYZ@P000003`2YZJT400000080ZJVY00<00000
0:VYZ@2YZJT01P0000020:VYZ@@000000P2YZJT300000080ZJVY100000020:VYZ@800000202YZJT;
00000080ZJVY1P0000020:VYZ@<000000P2YZJT200000080ZJVY200000040:VYZ@<000000P2YZJT<
00000080ZJVY2@0000040:VYZ@@000000P2YZJT=00000080ZJVY100000070:VYZ@P000000P2YZJT7
00000080ZJVY1P0000060:VYZ@800000102YZJT00`000000ZJVY0:VYZ@020:VYZ@@000000P2YZJT4
000001T0ZJVY00<000000:VYZ@2YZJT0400000000`2YZJT000000000000:00000080ZJVY0P000004
0:VYZ@<000000P2YZJT400000080ZJVY0P0000040:VYZA<000000P2YZJT2000000030:VYZ@000000
00000080ZJVY100000020:VYZ@P0000000<0ZJVY000000000000100000040:VYZ@cooooo000;oooo
o`80ZJVY3`0000040:VYZ@H000004`2YZJT5000000@0ZJVY100000020:VYZ@@000000P2YZJT20000
00<0ZJVY0P0000020:VYZ@@000002P2YZJT00`000000ZJVY0:VYZ@0200000080ZJVY200000040:VY
Z@L000000P2YZJT400000080ZJVY1`0000040:VYZ@H000000P2YZJT6000000030:VYZ@0000000000
00@0ZJVY0P0000020:VYZ@H000000P2YZJT2000000030:VYZ@0000000000008000000P2YZJT:0000
00L0ZJVY0P0000020:VYZ@800000102YZJT2000000@0ZJVY00<000000:VYZ@2YZJT00P0000080:VY
Z@@00000102YZJT900000080ZJVY1P0000070:VYZ@H000003@2YZJT200000080ZJVY4@0000060:VY
Z@8000000P2YZJT2000000D0ZJVY0P0000020:VYZ@@000000P2YZJTE00000080ZJVY0P0000020:VY
Z@80000000<0ZJVY000000000000102YZJT6000000@0ZJVY0P0000000`2YZJT00000000000080:VY
Z@cooooo000;ooooo`80ZJVY3`0000040:VYZ@H000004`2YZJT5000000@0ZJVY100000020:VYZ@@0
00000P2YZJT2000000<0ZJVY0P0000020:VYZ@@000002P2YZJT00`000000ZJVY0:VYZ@0200000080
ZJVY200000040:VYZ@L000000P2YZJT400000080ZJVY1`0000040:VYZ@H000000P2YZJT600000003
0:VYZ@000000000000@0ZJVY0P0000020:VYZ@H000000P2YZJT2000000030:VYZ@00000000000080
00000P2YZJT:000000L0ZJVY0P0000020:VYZ@800000102YZJT2000000@0ZJVY00<000000:VYZ@2Y
ZJT00P0000080:VYZ@@00000102YZJT900000080ZJVY1P0000070:VYZ@H000003@2YZJT200000080
ZJVY4@0000060:VYZ@8000000P2YZJT2000000D0ZJVY0P0000020:VYZ@@000000P2YZJTE00000080
ZJVY0P0000020:VYZ@80000000<0ZJVY000000000000102YZJT6000000@0ZJVY0P0000000`2YZJT0
0000000000080:VYZ@cooooo000;ooooo`80ZJVY100000020:VYZ@8000001@2YZJT200000080ZJVY
2000000;0:VYZ@P000000P2YZJT2000000030:VYZ@000000000000@000001P2YZJT200000080ZJVY
1`0000020:VYZ@8000003`2YZJT200000080ZJVY0P0000020:VYZ@8000000P2YZJT5000000@0ZJVY
200000040:VYZ@@000002@2YZJT2000000@0ZJVY1`0000040:VYZ@800000102YZJT2000000@0ZJVY
0P0000000`2YZJT00000000000040:VYZ@@00000102YZJT2000000D0ZJVY100000020:VYZ@H00000
2@2YZJT200000080ZJVY3`0000020:VYZ@H000000P2YZJT6000000L0ZJVY1P00000?0:VYZA400000
1P2YZJT400000080ZJVY2`0000020:VYZ@800000102YZJT6000000030:VYZ@000000000000h00000
0P2YZJT3000000P0ZJVY0P0000020:VYZ@L000002P2YZJT<ooooo`002oooool20:VYZ@@000000P2Y
ZJT2000000D0ZJVY0P0000020:VYZ@P000002`2YZJT800000080ZJVY0P0000000`2YZJT000000000
0004000000H0ZJVY0P0000020:VYZ@L000000P2YZJT2000000l0ZJVY0P0000020:VYZ@8000000P2Y
ZJT200000080ZJVY1@0000040:VYZ@P00000102YZJT4000000T0ZJVY0P0000040:VYZ@L00000102Y
ZJT2000000@0ZJVY0P0000040:VYZ@80000000<0ZJVY000000000000102YZJT4000000@0ZJVY0P00
00050:VYZ@@000000P2YZJT6000000T0ZJVY0P0000020:VYZ@l000000P2YZJT600000080ZJVY1P00
00070:VYZ@H000003`2YZJTA000000H0ZJVY100000020:VYZ@/000000P2YZJT2000000@0ZJVY1P00
00000`2YZJT000000000000>00000080ZJVY0`0000080:VYZ@8000000P2YZJT7000000X0ZJVY3?oo
ool000_ooooo102YZJT2000000T0ZJVY0P0000060:VYZ@@000003@2YZJT200000080ZJVY0P000007
0:VYZ@800000302YZJT2000000030:VYZ@0000000000008000000P2YZJT8000000L0ZJVY10000002
0:VYZ@8000000P2YZJT8000000L0ZJVY0P0000020:VYZ@800000102YZJT2000000L0ZJVY1P000006
0:VYZ@80000000<0ZJVY000000000000102YZJT8000000@0ZJVY1@0000060:VYZ@H000001`2YZJT8
00000080ZJVY0P0000090:VYZ@800000102YZJT2000000@0ZJVY1P0000030:VYZ@@000001P2YZJT4
000000T0ZJVY0P00000C0:VYZ@@000000P2YZJT400000080ZJVY0`0000080:VYZ@8000000P2YZJT4
000000<0ZJVY0P0000040:VYZ@800000102YZJT200000080ZJVY0P0000030:VYZ@8000001P2YZJT2
00000080ZJVY0P0000020:VYZ@8000002@2YZJT2000000P0ZJVY00<000000:VYZ@2YZJT0202YZJT<
ooooo`002oooool40:VYZ@8000002@2YZJT2000000H0ZJVY1000000=0:VYZ@8000000P2YZJT20000
00L0ZJVY0P00000<0:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@P000001`2YZJT40000
0080ZJVY0P0000020:VYZ@P000001`2YZJT200000080ZJVY0P0000040:VYZ@8000001`2YZJT60000
00H0ZJVY0P0000000`2YZJT00000000000040:VYZ@P00000102YZJT5000000H0ZJVY1P0000070:VY
Z@P000000P2YZJT2000000T0ZJVY0P0000040:VYZ@800000102YZJT6000000<0ZJVY100000060:VY
Z@@000002@2YZJT2000001<0ZJVY100000020:VYZ@@000000P2YZJT3000000P0ZJVY0P0000020:VY
Z@@000000`2YZJT2000000@0ZJVY0P0000040:VYZ@8000000P2YZJT2000000<0ZJVY0P0000060:VY
Z@8000000P2YZJT200000080ZJVY0P0000090:VYZ@800000202YZJT00`000000ZJVY0:VYZ@080:VY
Z@cooooo000;ooooo`80ZJVY1P0000020:VYZ@030000002YZJT0ZJVY00@0ZJVY0P0000020:VYZ@80
0000102YZJT3000000P0ZJVY1P0000020:VYZ@8000000`2YZJT8000000P0ZJVY0`0000040:VYZ@P0
00002@2YZJT6000000H0ZJVY1P0000050:VYZ@8000000P2YZJT400000080ZJVY0P0000040:VYZ@03
0000002YZJT0ZJVY00H00000202YZJT3000000@0ZJVY0P0000080:VYZ@8000000P2YZJT500000080
ZJVY0P0000020:VYZ@P000000P2YZJT900000080ZJVY0P0000020:VYZ@800000102YZJT3000000H0
ZJVY1P0000060:VYZ@030000002YZJT0ZJVY00@00000202YZJT5000000H0ZJVY200000020:VYZ@@0
00002`2YZJT2000000@0ZJVY0`0000060:VYZ@8000000P2YZJT4000000@0ZJVY0P0000030:VYZ@@0
0000202YZJT2000000D0ZJVY200000060:VYZ@@000001@2YZJT400000080ZJVY100000020:VYZ@<0
00000P2YZJT200000080ZJVY0P0000020:VYZ@cooooo000;ooooo`80ZJVY1P0000020:VYZ@030000
002YZJT0ZJVY00@0ZJVY0P0000020:VYZ@800000102YZJT3000000P0ZJVY1P0000020:VYZ@800000
0`2YZJT8000000P0ZJVY0`0000040:VYZ@P000002@2YZJT6000000H0ZJVY1P0000050:VYZ@800000
0P2YZJT400000080ZJVY0P0000040:VYZ@030000002YZJT0ZJVY00H00000202YZJT3000000@0ZJVY
0P0000080:VYZ@8000000P2YZJT500000080ZJVY0P0000020:VYZ@P000000P2YZJT900000080ZJVY
0P0000020:VYZ@800000102YZJT3000000H0ZJVY1P0000060:VYZ@030000002YZJT0ZJVY00@00000
202YZJT5000000H0ZJVY200000020:VYZ@@000002`2YZJT2000000@0ZJVY0`0000060:VYZ@800000
0P2YZJT4000000@0ZJVY0P0000030:VYZ@@00000202YZJT2000000D0ZJVY200000060:VYZ@@00000
1@2YZJT400000080ZJVY100000020:VYZ@<000000P2YZJT200000080ZJVY0P0000020:VYZ@cooooo
003ooooooiGooooo003ooooooiGooooo0000\
\>"],
  ImageRangeCache->{{{0, 403}, {100, 0}} -> {-5.78856, -1.25026, 0.525006, \
0.525006}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[pd5]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Compare this with what happens when we do ",
  StyleBox["not",
    FontSlant->"Italic"],
  " use the custom learning rule (note that I've modified the payoff matrix \
slightly):"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"pd6", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] Cycle[200]\), ",", 
            "\[IndentingNewLine]", \(UpdateNeighborhood \[Rule] 
              MakeSimple[GraphPower[Cycle[200], 2]]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({2, 2}\), \({0, 3}\)},
                    {\({3, 0}\), \({1, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {LightGray, 
                Black}\), ",", 
            "\[IndentingNewLine]", \(NeighborhoodsToDraw \[Rule] \
{Interaction}\)}], "\[IndentingNewLine]", "]"}]}], 
      ";"}], "\[IndentingNewLine]", \(SetStrategies[
        pd6, \[IndentingNewLine]Table[
          If[Random[] <  .8, 1, 2], {200}]\[IndentingNewLine]];\)}], "Input",
  CellLabel->"In[21]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l2 = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l2, 
          ToColorArray[pd6]]; \[IndentingNewLine]Evolve[
          pd6], {32}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[23]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]Graphics[\[IndentingNewLine]RasterArray[
            Reverse[l2]]\[IndentingNewLine]], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .16 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.00380952 0.0047619 [
[ 0 0 0 0 ]
[ 1 .16 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .16 L
0 .16 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .00381 translate
.95238 .15238 scale
600 string
200 32 8 [200 0 0 32 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9
A9A9A9000000000000000000000000000000000000000000
000000A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[25]:=",
  ImageSize->{403.5, 64.5},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006C0000@2000`400?l00000o`00003ooooooonD
ooooo`00oooooonDooooo`003?oooooo000007/000003Oooool000coooooo`00001k000000gooooo
000<ooooool00000N`00000=ooooo`003?oooooo000007/000003Oooool000coooooo`00001k0000
00gooooo000<ooooool00000N`00000=ooooo`003?oooooo000007/000003Oooool000coooooo`00
001k000000gooooo000<ooooool00000N`00000=ooooo`003?oooooo000007/000003Oooool000co
ooooo`00001k000000gooooo000<ooooool00000N`00000=ooooo`003?oooooo000007/000003Ooo
ool000coooooo`00001k000000gooooo000<ooooool00000N`00000=ooooo`003?oooooo000007/0
00003Oooool000coooooo`00001k000000gooooo000<ooooool00000N`00000=ooooo`003?oooooo
000007/000003Oooool000coooooo`00001k000000gooooo000<ooooool00000N`00000=ooooo`00
3?oooooo000007/000003Oooool000coooooo`00001k000000gooooo000<oooooiL00000102YZJWO
000000gooooo000<oooooiL00000102YZJWO000000gooooo000<oooooiD000001P2YZJWO000000go
oooo000<oooooiD000001P2YZJWO000000gooooo000<oooooiD000001P2YZJWO000000gooooo000<
oooooiD000001P2YZJWO000000gooooo000<oooooi<00000202YZJWO000000gooooo000<oooooi<0
0000202YZJWO000000gooooo000<oooooi<00000202YZJWO000000gooooo000<oooooi8000002@2Y
ZJWO000000gooooo000<oooooi8000002@2YZJWO000000gooooo000<oooooi8000002`2YZJWM0000
00gooooo000<oooooi8000002`2YZJWM000000gooooo000<oooooi8000002`2YZJWM000000gooooo
000<oooooi8000002`2YZJWM000000gooooo000<oooooi8000002`2YZJWM000000gooooo000<oooo
oi8000002`2YZJWM000000gooooo000<oooooi8000002`2YZJWM000000gooooo000<oooooi800000
2`2YZJWM000000gooooo000<oooooi8000002`2YZJWM000000gooooo000<oooooi8000002`2YZJWM
000000gooooo000<oooooi0000003@2YZJWM000000gooooo000<oooooi0000003@2YZJWM000000go
oooo000<oooooi0000003@2YZJWM000000gooooo000<oooooi0000003@2YZJWM000000gooooo000<
oooooi0000003@2YZJWM000000gooooo000<oooooi0000003@2YZJWM000000gooooo000<oooooa40
0000102YZJTg000000030:VYZ@0000000000030000000`2YZJT>000000d0ZJVYT00000000`2YZJT0
00000000000T000000@0ZJVY8P00000=ooooo`003?oooolA000000H0ZJVY<`0000050:VYZBh00000
1@2YZJT>000000l0ZJVY@P0000040:VYZDH000001@2YZJT>000000030:VYZ@000000000001400000
1P2YZJTR000000gooooo000<oooooa4000001P2YZJTc000000D0ZJVY;P0000050:VYZ@h000003`2Y
ZJU2000000@0ZJVYAP0000050:VYZ@h0000000<0ZJVY0000000000004@0000060:VYZB8000003Ooo
ool000cooooo4@0000080:VYZC4000001`2YZJTZ000000L0ZJVY3000000C0:VYZD0000001P2YZJU4
000000D0ZJVY300000050:VYZA400000202YZJT?000000@0ZJVY3@00000=ooooo`003?oooolA0000
00P0ZJVY<@0000070:VYZBX000001`2YZJT<000001<0ZJVY@00000060:VYZD@000001@2YZJT<0000
00D0ZJVY4@0000080:VYZ@l00000102YZJT=000000gooooo000<ooooo`8000000P2YZJT=000000P0
ZJVY<@0000070:VYZBP000002@2YZJT<000001<0ZJVY500000040:VYZAD00000102YZJT?000000H0
ZJVY:@0000020:VYZAL000001`2YZJT:000000L0ZJVY4@0000080:VYZ@d000001P2YZJT=000000go
oooo000<ooooo`8000000P2YZJT=000000P0ZJVY<@0000070:VYZBP000002@2YZJT<000001<0ZJVY
500000040:VYZAD00000102YZJT?000000H0ZJVY:@0000020:VYZAL000001`2YZJT:000000L0ZJVY
4@0000080:VYZ@d000001P2YZJT=000000gooooo000<ooooo`H0ZJVY2@00000:0:VYZA8000000P2Y
ZJTM000000T0ZJVY2P0000020:VYZAP000002`2YZJT:000001D0ZJVY1`0000020:VYZ@X000001@2Y
ZJTC000000H0ZJVY3`0000070:VYZBH000001P2YZJT900000080ZJVY2P0000070:VYZ@X000001`2Y
ZJTA000000X0ZJVY2`0000060:VYZ@d000003Oooool000cooooo1P2YZJT9000000X0ZJVY4P000002
0:VYZAd000002@2YZJT:00000080ZJVY6000000;0:VYZ@X000005@2YZJT700000080ZJVY2P000005
0:VYZA<000001P2YZJT?000000L0ZJVY9P0000060:VYZ@T000000P2YZJT:000000L0ZJVY2P000007
0:VYZA4000002P2YZJT;000000H0ZJVY3@00000=ooooo`003?ooool90:VYZ@8000004@2YZJT20000
00P0ZJVY0P0000070:VYZ@@00000102YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT20000
0080ZJVY0P00000A0:VYZ@8000002@2YZJT2000000H0ZJVY0P0000050:VYZ@8000004`2YZJT20000
01T0ZJVY1`0000060:VYZ@8000003@2YZJT2000000L0ZJVY0P00000>0:VYZ@030000002YZJT0ZJVY
0080ZJVY0P00000?0:VYZ@8000001P2YZJT2000000D0ZJVY0P0000040:VYZ@8000001P2YZJT00`00
0000ZJVY0:VYZ@0<0:VYZ@8000002@2YZJT2000000l0ZJVY0P00000?0:VYZ@800000102YZJT20000
0080ZJVY0`00000>0:VYZ@030000002YZJT0ZJVY00@00000202YZJT300000080ZJVY0P0000040:VY
Z@gooooo000<ooooo`T0ZJVY0P00000A0:VYZ@800000202YZJT2000000L0ZJVY100000040:VYZ@80
00000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@8000000P2YZJT200000140ZJVY0P0000090:VYZ@80
00001P2YZJT2000000D0ZJVY0P00000C0:VYZ@8000006@2YZJT7000000H0ZJVY0P00000=0:VYZ@80
00001`2YZJT2000000h0ZJVY00<000000:VYZ@2YZJT00P2YZJT2000000l0ZJVY0P0000060:VYZ@80
00001@2YZJT2000000@0ZJVY0P0000060:VYZ@030000002YZJT0ZJVY00`0ZJVY0P0000090:VYZ@80
00003`2YZJT2000000l0ZJVY0P0000040:VYZ@8000000P2YZJT3000000h0ZJVY00<000000:VYZ@2Y
ZJT0100000080:VYZ@<000000P2YZJT2000000@0ZJVY3Oooool00?ooooooU?ooool00001\
\>"],
  ImageRangeCache->{{{0, 402.5}, {63.5, 0}} -> {-6.48926, -0.800168, 0.52914, \
0.52914}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "The uneven march occurs because boundary Defectors earn payoffs of 4, \
tying with interior Cooperators who earn payoffs of 4. Boundary Defectors \
won't switch, but boundary Cooperators will. Since agents flip a coin to \
break ties, boundary Cooperators will flip a coin to decide whether to stay a \
Cooperator or become a Defector, hence the uneven rate of advance. If we \
return the payoff matrix to the one used for ",
  StyleBox["pd5", "Input"],
  ", this doesn't happen."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[pd6]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"pd7", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] Cycle[200]\), ",", 
            "\[IndentingNewLine]", \(UpdateNeighborhood \[Rule] 
              MakeSimple[GraphPower[Cycle[200], 2]]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({2.5, 2.5}\), \({0, 3}\)},
                    {\({3, 0}\), \({1, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {LightGray, 
                Black}\), ",", 
            "\[IndentingNewLine]", \(NeighborhoodsToDraw \[Rule] \
{Interaction}\)}], "\[IndentingNewLine]", "]"}]}], 
      ";"}], "\n", \(SetStrategies[
        pd7, \[IndentingNewLine]Table[
          If[Random[] <  .75, 2, 1], {200}]\[IndentingNewLine]];\)}], "Input",\

  CellLabel->"In[27]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l3 = {};\)\), "\n", 
    \(\(Do[\[IndentingNewLine]AppendTo[l3, 
          ToColorArray[pd7]]; \[IndentingNewLine]Evolve[
          pd7], {50}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[29]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]Graphics[RasterArray[Reverse[l3]]], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .25 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.00595238 0.0047619 [
[ 0 0 0 0 ]
[ 1 .25 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .25 L
0 .25 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .00595 translate
.95238 .2381 scale
600 string
200 50 8 [200 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9000000000000000000A9A9A9000000000000000000000000000000000000
000000A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9
000000000000000000000000A9A9A9000000000000000000000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000A9A9A9000000000000000000A9A9A9000000000000
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[31]:=",
  ImageSize->{404.625, 101.125},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006D0000IB000`400?l00000o`00003ooooooonE
ooooo`00oooooonEooooo`00oooooonEooooo`00oooooonEooooo`002oooooli0:VYZKd00000Q`2Y
ZJT<ooooo`002oooooli0:VYZKd00000Q`2YZJT<ooooo`002ooooolg0:VYZL400000Q@2YZJT<oooo
o`002ooooolg0:VYZL400000Q@2YZJT<ooooo`002ooooole0:VYZLD00000P`2YZJT<ooooo`002ooo
oole0:VYZLD00000P`2YZJT<ooooo`002ooooolc0:VYZLP00000PP2YZJT<ooooo`002ooooolc0:VY
ZLP00000PP2YZJT<ooooo`002ooooolb0:VYZL/00000P02YZJT<ooooo`002oooool`0:VYZLl00000
OP2YZJT<ooooo`002oooool`0:VYZLl00000OP2YZJT<ooooo`002oooool^0:VYZM<00000O02YZJT<
ooooo`002oooool^0:VYZM<00000O02YZJT<ooooo`002oooool/0:VYZML00000NP2YZJT<ooooo`00
2oooool/0:VYZML00000NP2YZJT<ooooo`002ooooolZ0:VYZM/00000N02YZJT<ooooo`002ooooolZ
0:VYZM/00000N02YZJT<ooooo`002ooooolX0:VYZMl00000MP2YZJT<ooooo`002ooooolX0:VYZMl0
0000MP2YZJT<ooooo`002ooooolV0:VYZN<00000M02YZJT<ooooo`002ooooolV0:VYZN<00000M02Y
ZJT<ooooo`002ooooolT0:VYZNL00000LP2YZJT<ooooo`002ooooolT0:VYZNL00000LP2YZJT<oooo
o`002ooooolR0:VYZN/00000L02YZJT<ooooo`002ooooolR0:VYZN/00000L02YZJT<ooooo`002ooo
oolP0:VYZNl00000KP2YZJT<ooooo`002ooooolP0:VYZNl00000KP2YZJT<ooooo`002ooooolN0:VY
ZO800000K@2YZJT<ooooo`002ooooolM0:VYZOD00000J`2YZJT<ooooo`002ooooolM0:VYZOD00000
J`2YZJT<ooooo`002ooooolK0:VYZOT00000J@2YZJT<ooooo`002ooooolK0:VYZOT00000J@2YZJT<
ooooo`002ooooolI0:VYZOd00000I`2YZJT<ooooo`002ooooolI0:VYZOd00000I`2YZJT<ooooo`00
2ooooolG0:VYZOl000000P00001U0:VYZ@cooooo000;oooooaL0ZJVYo`000002000006D0ZJVY3?oo
ool000_ooooo5@2YZJWo000000H00000H`2YZJT<ooooo`002ooooolE0:VYZOl000001P00001S0:VY
Z@cooooo000;oooooa<0ZJVYo`00000:00000640ZJVY3?ooool000_ooooo4`2YZJWo000000X00000
H@2YZJT<ooooo`002ooooolA0:VYZOl000003P00001O0:VYZ@cooooo000;oooooa40ZJVYo`00000>
000005l0ZJVY3?ooool000_ooooo3`2YZJWo000001800000G@2YZJT<ooooo`002oooool?0:VYZOl0
00004P00001M0:VYZ@cooooo000;ooooo`d0ZJVYo`00000F000005/0ZJVY3?ooool000_ooooo3@2Y
ZJWo000001H00000F`2YZJT<ooooo`002oooool;0:VYZOl000006@00001J0:VYZ@cooooo000;oooo
o`X0ZJVYo`00000L000005P0ZJVY3?ooool000_ooooo2P2YZJWo000001`00000F02YZJT<ooooo`00
2oooool80:VYZOl000008000001F0:VYZ@cooooo000;ooooo`P0ZJVYo`00000P000005H0ZJVY3?oo
ool000_ooooo1P2YZJWo000002@00000E02YZJT<ooooo`002oooool60:VYZOl000009000001D0:VY
Z@cooooo000;ooooo`@0ZJVYo`00000X00000580ZJVY3?ooool000_ooooo102YZJWo000002P00000
DP2YZJT<ooooo`002oooool20:VYZOl00000;000001@0:VYZ@cooooo000;ooooo`80ZJVYo`00000/
00000500ZJVY3?ooool000_oooooo`00000`000004h0ZJVY3?ooool000_oooooo`00000`000004h0
ZJVY3?ooool000_oooooo`00000b000004X0ZJVY0P00000<ooooo`002ooooooo000003800000BP2Y
ZJT2000000cooooo000;ooooool00000=00000160:VYZ@@000003?ooool000_oooooo`00000d0000
04H0ZJVY1000000<ooooo`002ooooooo000003H00000@P2YZJT6000000cooooo000;ooooool00000
=P0000120:VYZ@H000003?ooool000_oooooo`00000h000003h0ZJVY2000000<ooooo`002ooooooo
000003T00000>`2YZJT:000000cooooo000;ooooool00000>@00000k0:VYZ@X000003?ooool000_o
ooooo`00000k000003P0ZJVY2`00000<ooooo`002ooooooo000003/00000>02YZJT;000000cooooo
000;ooooool00000?@00000d0:VYZ@d000003?ooool000_oooooo`00000m000003@0ZJVY3@00000<
ooooo`002ooooooo000003l00000<02YZJT?000000cooooo000;ooooool00000?`00000`0:VYZ@l0
00003?ooool000_oooooo`000011000002`0ZJVY4@00000<ooooo`002ooooooo000004400000;02Y
ZJTA000000cooooo000;ooooool00000@`00000X0:VYZA<000003?ooool000_oooooo`0000130000
02P0ZJVY4`00000<ooooo`002ooooooo000004D00000902YZJTE000000cooooo000;ooooool00000
A@00000T0:VYZAD000003?ooool000_oooooo`00001700000200ZJVY5`00000<ooooo`002ooooooo
000004L00000802YZJTG000000cooooo000;ooooool00000B@00000L0:VYZAT000003?ooool000_o
ooooo`000019000001`0ZJVY6@00000<ooooo`002ooooooo000004/00000602YZJTK000000cooooo
000;ooooool00000C000000E0:VYZAd000003?ooool000_oooooo`00001<000001D0ZJVY7@00000<
ooooo`002ooooooo000004h000004P2YZJTN000000cooooo000;ooooool00000CP00000B0:VYZAh0
00003?ooool000_oooooo`00001@000000h0ZJVY8000000<ooooo`002ooooooo0000050000003P2Y
ZJTP000000cooooo000;oooooc8000000`2YZJWo000001d000002P2YZJTR000000cooooo000;oooo
oc8000000`2YZJWo000001d000002P2YZJTR000000cooooo000;ooooo`@0ZJVY2@0000020:VYZA/0
00000P2YZJT7000000H0ZJVY0P0000020:VYZA4000000P2YZJT:000000030:VYZ@00000000000080
ZJVY1P0000040:VYZ@8000000P2YZJTA000000@0ZJVY1@0000020:VYZBP00000102YZJT600000080
ZJVY0`0000040:VYZ@@000000P2YZJT;00000080ZJVY100000040:VYZ@/000000P2YZJT200000080
ZJVY0P0000020:VYZ@8000000P2YZJT500000080ZJVY1P0000020:VYZ@d000000P2YZJTA00000080
ZJVY200000020:VYZ@L00000102YZJT600000080ZJVY100000000`2YZJT00000000000020:VYZ@@0
00001P2YZJT7000000@0ZJVY100000020:VYZ@H000000P2YZJT500000080ZJVY1000000<ooooo`00
2oooool40:VYZ@T000000P2YZJTK00000080ZJVY1`0000060:VYZ@8000000P2YZJTA00000080ZJVY
2P0000000`2YZJT00000000000020:VYZ@H00000102YZJT200000080ZJVY4@0000040:VYZ@D00000
0P2YZJTX000000@0ZJVY1P0000020:VYZ@<00000102YZJT400000080ZJVY2`0000020:VYZ@@00000
102YZJT;00000080ZJVY0P0000020:VYZ@8000000P2YZJT200000080ZJVY1@0000020:VYZ@H00000
0P2YZJT=00000080ZJVY4@0000020:VYZ@P000000P2YZJT7000000@0ZJVY1P0000020:VYZ@@00000
00<0ZJVY0000000000000P2YZJT4000000H0ZJVY1`0000040:VYZ@@000000P2YZJT600000080ZJVY
1@0000020:VYZ@@000003?ooool00?ooooooUOooool00?ooooooUOooool00001\
\>"],
  ImageRangeCache->{{{0, 403.625}, {100.125, 0}} -> {-5.82035, -1.25026, \
0.52435, 0.52435}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
This shows that the successful advance of Cooperate in the \
Prisoner's Dilemma with unequal interaction and update neighborhoods \
crucially depends upon the learning rule.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(DeleteModel[pd7]\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Learning rules which require memory", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Some learning rules require memory or adding new variables to the \
agent. Two examples are traditional and exponentially weighted fictitious \
play.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell["Traditional fictitious play", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "For traditional fictitious play, each individual has a parameter ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\)]],
  " measuring how rapidly he learns. Given a particular ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\)]],
  " parameter, a list of observations ",
  Cell[BoxData[
      \(TraditionalForm\`{a\_1, \[Ellipsis], a\_n}\)]],
  " of the number of times each strategy ",
  Cell[BoxData[
      \(TraditionalForm\`i \[Element] {1, \[Ellipsis], n}\)]],
  " was played, the time ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  ", and a current probability distribution ",
  Cell[BoxData[
      \(TraditionalForm\`{x\_1, \[Ellipsis], x\_n}\)]],
  " over strategies, traditional fictitious play says that individuals ought \
to update their belief vector according to the formula:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(TraditionalForm\`x\_i\^\[Prime] = \(\(a\_i\/\(\[Sum]\+\(j = 1\)\%n a\_j\
\) + t + \[Lambda]\)\/\(1 + t + \[Lambda]\)\) x\_i\)], "DisplayFormula",
  TextAlignment->Center,
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
The following function takes these parameters and returns the \
updated belief vector.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(TraditionalFictitiousPlay[\[Lambda]_, t_, observations_List, 
        distribution_List] := \[IndentingNewLine]Module[{tot}, \
\[IndentingNewLine]tot = Apply[Plus, observations]; \[IndentingNewLine]If[
          tot === 0, \[IndentingNewLine]Return[d], \[IndentingNewLine]Return[
            N[\(1\/\(t + 
                      1 + \[Lambda]\)\) \((observations\/tot + \((t + \
\[Lambda])\)*
                      distribution)\)]]\[IndentingNewLine]]\[IndentingNewLine]\
]\)], "Input",
  CellLabel->"In[33]:=",
  Background->RGBColor[0.984192, 1, 0.723949],
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalFictitiousPlay[\[Lambda], 
      t, {a\_1, a\_2, a\_3}, {x\_1, x\_2, x\_3}]\)], "Input",
  CellLabel->"In[34]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({\(a\_1.`\/\(a\_1.` + a\_2.` + a\_3.`\) + \((t + \[Lambda])\)\ x\_1.`\)\
\/\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t + \[Lambda]\), \(a\_2.`\/\(a\_1.` + \
a\_2.` + a\_3.`\) + \((t + \[Lambda])\)\ x\_2.`\)\/\(\(\(1.`\)\(\
\[InvisibleSpace]\)\) + t + \[Lambda]\), \(a\_3.`\/\(a\_1.` + a\_2.` + a\_3.`\
\) + \((t + \[Lambda])\)\ x\_3.`\)\/\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t + \
\[Lambda]\)}\)], "Output",
  CellLabel->"Out[34]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exponential fictitious play", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Exponentially weighted fictitious play takes a similar set of inputs \
(although here the measure of the rate of learning is denoted by ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\)]],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(\(ExponentialFictitiousPlay[\[Theta]_, t_, observations_List, 
        distribution_List] := 
      Module[{tot}, \[IndentingNewLine]tot = 
          Apply[Plus, observations]; \[IndentingNewLine]If[
          tot === 0, \[IndentingNewLine]Return[
            distribution], \[IndentingNewLine]Return[
            N[\(1\/\(1 - \[Theta]\^\(t + 2\)\)\) \((\((\[Theta]\^\(t + 1\) - \
\[Theta]\^\(t + 2\))\) observations\/tot + \((1 - \[Theta]\^\(t + 1\))\) 
                      distribution)\)]]\[IndentingNewLine]]\[IndentingNewLine]\
]\)\(\[IndentingNewLine]\)
    \)\)], "Input",
  CellLabel->"In[35]:=",
  Background->RGBColor[0.984192, 1, 0.723949],
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ExponentialFictitiousPlay[\[Theta], 
      t, {a\_1, a\_2, a\_3}, {x\_1, x\_2, x\_3}]\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({\(\(\((\[Theta]\^\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t\) - 1.`\ \
\[Theta]\^\(\(\(2.`\)\(\[InvisibleSpace]\)\) + t\))\)\ a\_1.`\)\/\(a\_1.` + a\
\_2.` + a\_3.`\) + \((\(\(1.`\)\(\[InvisibleSpace]\)\) - 1.`\ \
\[Theta]\^\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t\))\)\ \
x\_1.`\)\/\(\(\(1.`\)\(\[InvisibleSpace]\)\) - 1.`\ \[Theta]\^\(\(\(2.`\)\(\
\[InvisibleSpace]\)\) + t\)\), \
\(\(\((\[Theta]\^\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t\) - 1.`\ \[Theta]\^\(\
\(\(2.`\)\(\[InvisibleSpace]\)\) + t\))\)\ a\_2.`\)\/\(a\_1.` + a\_2.` + \
a\_3.`\) + \((\(\(1.`\)\(\[InvisibleSpace]\)\) - 1.`\ \[Theta]\^\(\(\(1.`\)\(\
\[InvisibleSpace]\)\) + t\))\)\ x\_2.`\)\/\(\(\(1.`\)\(\[InvisibleSpace]\)\) \
- 1.`\ \[Theta]\^\(\(\(2.`\)\(\[InvisibleSpace]\)\) + t\)\), \
\(\(\((\[Theta]\^\(\(\(1.`\)\(\[InvisibleSpace]\)\) + t\) - 1.`\ \[Theta]\^\(\
\(\(2.`\)\(\[InvisibleSpace]\)\) + t\))\)\ a\_3.`\)\/\(a\_1.` + a\_2.` + \
a\_3.`\) + \((\(\(1.`\)\(\[InvisibleSpace]\)\) - 1.`\ \[Theta]\^\(\(\(1.`\)\(\
\[InvisibleSpace]\)\) + t\))\)\ x\_3.`\)\/\(\(\(1.`\)\(\[InvisibleSpace]\)\) \
- 1.`\ \[Theta]\^\(\(\(2.`\)\(\[InvisibleSpace]\)\) + t\)\)}\)], "Output",
  CellLabel->"Out[36]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples and definitions", "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "In order to use these rules, we need some way of being able to store data \
in the agent and retrieve it when we go to update. The functions ",
  StyleBox["Data[", "Input"],
  StyleBox["agent",
    FontSlant->"Italic"],
  StyleBox["]", "Input"],
  " and ",
  StyleBox["SetData[", "Input"],
  StyleBox["agent, data",
    FontSlant->"Italic"],
  StyleBox["]", "Input"],
  " provide us with a way to do this."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Essentially, ",
  StyleBox["SetData", "Input"],
  " allows us to store any ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expression inside an agent. Since everything in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is an expression, this means we can store any meaningful construct for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " inside the agent. In particular, we can store lists containing new values \
or variables that an agent's update rule may need."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Before doing that, here's a simple example of how it works. \
Consider a simple model\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"pd8", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] Cycle[5]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({2, 2}\), \({0, 3}\)},
                    {\({3, 0}\), \({1, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {LightGray, 
                Black}\)}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[37]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Here's what the model looks like:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[pd8]];\)\)], "Input",
  CellLabel->"In[38]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
.663 .663 .663 r
.025 w
.60667 .95401 Mdot
.04812 .7806 Mdot
0 0 0 r
.04812 .2194 Mdot
.60667 .04599 Mdot
.663 .663 .663 r
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[38]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3o`00073ooooo002/ooooo`Go0000Koooool00:_ooooo1ol0001^ooooo`00Zooo
ool7o`0006kooooo002Yooooo`Wo0000K_ooool00:Kooooo0ol00003ooooo`Go0000Koooool00:;o
oooo1?l00007ooooo`Co0000Koooool009oooooo0ol0000?ooooo`03o`000?oooooooooo06cooooo
002Looooo`?o00004_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`0001Kooooo00?o0000
ooooooooool0Joooool009Gooooo1?l0000Jooooo`03o`000?oooooooooo06[ooooo002Booooo`?o
00007oooool00ol0003oooooooooo`1Yooooo`00Soooool3o`0002;ooooo00?o0000ooooooooool0
JOooool008_ooooo1?l0000Vooooo`03o`000?oooooooooo06Sooooo0028ooooo`?o0000:oooool0
0ol0003oooooooooo`1Wooooo`00QOooool3o`0002oooooo00?o0000ooooooooool0I_ooool008;o
oooo0ol0000cooooo`03o`000?oooooooooo06Gooooo001nooooo`Co0000=_ooool00ol0003ooooo
ooooo`1Uooooo`00Noooool3o`0003_ooooo00?o0000ooooooooool0I?ooool007Sooooo0ol0000o
ooooo`03o`000?oooooooooo06?ooooo001dooooo`Co0000@oooool00ol0003oooooooooo`1Roooo
o`00LOooool3o`0004Oooooo00?o0000ooooooooool0H_ooool006kooooo0ol0001;ooooo`03o`00
0?oooooooooo067ooooo001[ooooo`?o0000Coooool00ol0003oooooooooo`1Pooooo`00Ioooool4
o`0005?ooooo00?o0000ooooooooool0Goooool006Cooooo0ol0001Gooooo`03o`000?oooooooooo
05oooooo001Qooooo`?o0000Foooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0005oooooo
00?o0000ooooooooool0GOooool005[ooooo0ol0001Tooooo`03o`000?oooooooooo05cooooo001G
ooooo`?o0000Ioooool00ol0003oooooooooo`1Looooo`00E?ooool3o`0006_ooooo00?o0000oooo
ooooool0Foooool0053ooooo1?l0001_ooooo`03o`000?oooooooooo05[ooooo001=ooooo`?o0000
M?ooool00ol0003oooooooooo`1Iooooo`00B_ooool3o`0007Oooooo00?o0000ooooooooool0FOoo
ool004Kooooo1?l0001kooooo`03o`000?oooooooooo05Sooooo0013ooooo`?o0000P?ooool00ol0
003oooooooooo`1Gooooo`00@?ooool3o`0008Cooooo00?o0000ooooooooool0E_ooool003gooooo
0ol00028ooooo`03o`000?oooooooooo05Gooooo000iooooo`Co0000Roooool00ol0003ooooooooo
o`1Eooooo`00=_ooool3o`00093ooooo00?o0000ooooooooool0E?ooool003?ooooo0ol0002Doooo
o`03o`000?oooooooooo05?ooooo000_ooooo`Co0000V?ooool00ol0003oooooooooo`1Booooo`00
;?ooool3o`0009cooooo00?o0000ooooooooool0D_ooool002Wooooo0ol0002Pooooo`03o`000?oo
oooooooo057ooooo000Vooooo`?o0000Y?ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`00
0:Sooooo00?o0000ooooooooool0Coooool001oooooo0ol0002/ooooo`03o`000?oooooooooo04oo
oooo000Looooo`?o0000/?ooool00ol0003oooooooooo`1>ooooo`006?ooool4o`000;Cooooo00?o
0000ooooooooool0COooool001Gooooo0ol0002iooooo`03o`000?oooooooooo04cooooo000<oooo
o`?o00000oooool3o`000;cooooo00?o0000ooooooooool0C?ooool000_ooooo1ol00030ooooo`03
o`000?oooooooooo04_ooooo000:ooooo`Oo0000`_ooool00ol0003oooooooooo`1:ooooo`002_oo
ool7o`000<?ooooo00?o0000ooooooooool0BOooool000[ooooo1ol00033ooooo`03o`000?oooooo
oooo04Wooooo000;ooooo`Go0000aOooool00ol0003oooooooooo`18ooooo`003?ooool3o`000<Oo
oooo00?o0000ooooooooool0Aoooool000gooooo00?o0000ooooooooool0aoooool00ol0003ooooo
ooooo`16ooooo`003Oooool00ol0003oooooooooo`38ooooo`03o`000?oooooooooo04Gooooo000=
ooooo`03o`000?oooooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000oooo
ooooool0bOooool00ol0003oooooooooo`14ooooo`003Oooool00ol0003oooooooooo`3:ooooo`03
o`000?oooooooooo04?ooooo000=ooooo`03o`000?oooooooooo0<_ooooo00?o0000ooooooooool0
@_ooool000gooooo00?o0000ooooooooool0boooool00ol0003oooooooooo`12ooooo`003Oooool0
0ol0003oooooooooo`3<ooooo`03o`000?oooooooooo047ooooo000=ooooo`03o`000?oooooooooo
0<gooooo00?o0000ooooooooool0@?ooool000gooooo00?o0000ooooooooool0c_ooool00ol0003o
ooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>ooooo`03o`000?oooooooooo03oooooo
000=ooooo`03o`000?oooooooooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000
ooooooooool0d?ooool00ol0003oooooooooo`0mooooo`003Oooool00ol0003oooooooooo`3Aoooo
o`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0d_ooool00ol0003oooooooooo`0kooooo`003Ooo
ool00ol0003oooooooooo`3Cooooo`03o`000?oooooooooo03[ooooo000=ooooo`03o`000?oooooo
oooo0=Cooooo00?o0000ooooooooool0>Oooool000gooooo00?o0000ooooooooool0e?ooool00ol0
003oooooooooo`0iooooo`003Oooool00ol0003oooooooooo`3Eooooo`03o`000?oooooooooo03So
oooo000=ooooo`03o`000?oooooooooo0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o
0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool00ol0003oooooooooo`3G
ooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooooooo0=Sooooo00?o0000oooo
ooooool0=Oooool000gooooo00?o0000ooooooooool0fOooool00ol0003oooooooooo`0dooooo`00
3Oooool00ol0003oooooooooo`3Jooooo`03o`000?oooooooooo03?ooooo000=ooooo`03o`000?oo
oooooooo0=_ooooo00?o0000ooooooooool0<_ooool000gooooo00?o0000ooooooooool0foooool0
0ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`3Looooo`03o`000?oooooooooo
037ooooo000=ooooo`03o`000?oooooooooo0=gooooo00?o0000ooooooooool0<?ooool000gooooo
00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0003ooooooooo
o`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oooooooooo0=oooooo00?o0000
ooooooooool0;_ooool000gooooo00?o0000ooooooooool0h?ooool00ol0003oooooooooo`0]oooo
o`003Oooool00ol0003oooooooooo`3Qooooo`03o`000?oooooooooo02cooooo000=ooooo`03o`00
0?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000ooooooooool0h_oo
ool00ol0003oooooooooo`0[ooooo`003Oooool00ol0003oooooooooo`3Sooooo`03o`000?oooooo
oooo02[ooooo000=ooooo`03o`000?oooooooooo0>Cooooo00?o0000ooooooooool0:Oooool000go
oooo00?o0000ooooooooool0i?ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003ooooo
ooooo`3Uooooo`03o`000?oooooooooo02Sooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o
0000ooooooooool09oooool000gooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0V
ooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oooooooooo02Kooooo000=ooooo`03
o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000gooooo00?o0000ooooooooool0
jOooool00ol0003oooooooooo`0Tooooo`003Oooool00ol0003oooooooooo`3Zooooo`03o`000?oo
oooooooo02?ooooo000=ooooo`03o`000?oooooooooo0>_ooooo00?o0000ooooooooool08_ooool0
00gooooo00?o0000ooooooooool0joooool00ol0003oooooooooo`0Rooooo`003Oooool00ol0003o
ooooooooo`3/ooooo`03o`000?oooooooooo027ooooo000=ooooo`03o`000?oooooooooo0>gooooo
00?o0000ooooooooool08?ooool000gooooo00?o0000ooooooooool0k_ooool00ol0003ooooooooo
o`0Oooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oooooooooo01oooooo000=oooo
o`03o`000?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo00?o0000oooooooo
ool0l?ooool00ol0003oooooooooo`0Mooooo`003Oooool00ol0003oooooooooo`3aooooo`03o`00
0?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000ooooooooool07?oo
ool000gooooo00?o0000ooooooooool0l_ooool00ol0003oooooooooo`0Kooooo`003Oooool00ol0
003oooooooooo`3cooooo`03o`000?oooooooooo01[ooooo000=ooooo`03o`000?oooooooooo0?Co
oooo00?o0000ooooooooool06Oooool000gooooo00?o0000ooooooooool0m?ooool00ol0003ooooo
ooooo`0Iooooo`003Oooool00ol0003oooooooooo`3eooooo`03o`000?oooooooooo01Sooooo000=
ooooo`03o`000?oooooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o0000oooo
ooooool0moooool00ol0003oooooooooo`0Fooooo`003Oooool00ol0003oooooooooo`3gooooo`03
o`000?oooooooooo01Kooooo000=ooooo`03o`000?oooooooooo0?Sooooo00?o0000ooooooooool0
5Oooool000gooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0Dooooo`003Oooool0
0ol0003oooooooooo`3jooooo`03o`000?oooooooooo01?ooooo000=ooooo`03o`000?oooooooooo
0?_ooooo00?o0000ooooooooool04_ooool000gooooo00?o0000ooooooooool0noooool00ol0003o
ooooooooo`0Booooo`003Oooool00ol0003oooooooooo`3looooo`03o`000?oooooooooo017ooooo
000=ooooo`03o`000?oooooooooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000
ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`3noooo
o`03o`000?ooooooZJVY00;oZJVY3Oooool000gooooo00?o0000ooooooooool0ooooool5ojVYZ@co
oooo000=ooooo`03o`000?oooooooooo0?kooooo1onYZJT;ooooo`003Oooool00ol0003ooooooooo
o`3nooooo`OoZJVY2oooool000gooooo00?o0000ooooooooool0o_ooool7ojVYZ@_ooooo000=oooo
o`03o`000?oooooooooo0?oooooo1OnYZJT<ooooo`003Oooool00ol0003oooooooooo`3oooooo`04
o`000?nYZJWoZJVYojVYZ@gooooo000=ooooo`03o`000?oooooooooo0?kooooo00?o0000oooooooo
ool03oooool000gooooo00?o0000ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Ooo
ool00ol0003oooooooooo`3mooooo`03o`000?oooooooooo013ooooo000=ooooo`03o`000?oooooo
oooo0?cooooo00?o0000ooooooooool04Oooool000gooooo00?o0000ooooooooool0noooool00ol0
003oooooooooo`0Booooo`003Oooool00ol0003oooooooooo`3kooooo`03o`000?oooooooooo01;o
oooo000=ooooo`03o`000?oooooooooo0?[ooooo00?o0000ooooooooool04oooool000gooooo00?o
0000ooooooooool0nOooool00ol0003oooooooooo`0Dooooo`003Oooool00ol0003oooooooooo`3h
ooooo`03o`000?oooooooooo01Gooooo000=ooooo`03o`000?oooooooooo0?Oooooo00?o0000oooo
ooooool05_ooool000gooooo00?o0000ooooooooool0moooool00ol0003oooooooooo`0Fooooo`00
3Oooool00ol0003oooooooooo`3fooooo`03o`000?oooooooooo01Oooooo000=ooooo`03o`000?oo
oooooooo0?Gooooo00?o0000ooooooooool06?ooool000gooooo00?o0000ooooooooool0m?ooool0
0ol0003oooooooooo`0Iooooo`003Oooool00ol0003oooooooooo`3dooooo`03o`000?oooooooooo
01Wooooo000=ooooo`03o`000?oooooooooo0??ooooo00?o0000ooooooooool06_ooool000gooooo
00?o0000ooooooooool0l_ooool00ol0003oooooooooo`0Kooooo`003Oooool00ol0003ooooooooo
o`3aooooo`03o`000?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000
ooooooooool07?ooool000gooooo00?o0000ooooooooool0l?ooool00ol0003oooooooooo`0Moooo
o`003Oooool00ol0003oooooooooo`3_ooooo`03o`000?oooooooooo01kooooo000=ooooo`03o`00
0?oooooooooo0>kooooo00?o0000ooooooooool07oooool000gooooo00?o0000ooooooooool0k_oo
ool00ol0003oooooooooo`0Oooooo`003Oooool00ol0003oooooooooo`3]ooooo`03o`000?oooooo
oooo023ooooo000=ooooo`03o`000?oooooooooo0>cooooo00?o0000ooooooooool08Oooool000go
oooo00?o0000ooooooooool0joooool00ol0003oooooooooo`0Rooooo`003Oooool00ol0003ooooo
ooooo`3[ooooo`03o`000?oooooooooo02;ooooo000=ooooo`03o`000?oooooooooo0>[ooooo00?o
0000ooooooooool08oooool000gooooo00?o0000ooooooooool0jOooool00ol0003oooooooooo`0T
ooooo`003Oooool00ol0003oooooooooo`3Xooooo`03o`000?oooooooooo02Gooooo000=ooooo`03
o`000?oooooooooo0>Oooooo00?o0000ooooooooool09_ooool000gooooo00?o0000ooooooooool0
ioooool00ol0003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`3Vooooo`03o`000?oo
oooooooo02Oooooo000=ooooo`03o`000?oooooooooo0>Gooooo00?o0000ooooooooool0:?ooool0
00gooooo00?o0000ooooooooool0i?ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003o
ooooooooo`3Tooooo`03o`000?oooooooooo02Wooooo000=ooooo`03o`000?oooooooooo0>?ooooo
00?o0000ooooooooool0:_ooool000gooooo00?o0000ooooooooool0h_ooool00ol0003ooooooooo
o`0[ooooo`003Oooool00ol0003oooooooooo`3Qooooo`03o`000?oooooooooo02cooooo000=oooo
o`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000oooooooo
ool0h?ooool00ol0003oooooooooo`0]ooooo`003Oooool00ol0003oooooooooo`3Oooooo`03o`00
0?oooooooooo02kooooo000=ooooo`03o`000?oooooooooo0=kooooo00?o0000ooooooooool0;ooo
ool000gooooo00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0
003oooooooooo`3Mooooo`03o`000?oooooooooo033ooooo000=ooooo`03o`000?oooooooooo0=co
oooo00?o0000ooooooooool0<Oooool000gooooo00?o0000ooooooooool0foooool00ol0003ooooo
ooooo`0booooo`003Oooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo03;ooooo000=
ooooo`03o`000?oooooooooo0=[ooooo00?o0000ooooooooool0<oooool000gooooo00?o0000oooo
ooooool0fOooool00ol0003oooooooooo`0dooooo`003Oooool00ol0003oooooooooo`3Hooooo`03
o`000?oooooooooo03Gooooo000=ooooo`03o`000?oooooooooo0=Oooooo00?o0000ooooooooool0
=_ooool000gooooo00?o0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool0
0ol0003oooooooooo`3Fooooo`03o`000?oooooooooo03Oooooo000=ooooo`03o`000?oooooooooo
0=Gooooo00?o0000ooooooooool0>?ooool000gooooo00?o0000ooooooooool0e?ooool00ol0003o
ooooooooo`0iooooo`003Oooool00ol0003oooooooooo`3Dooooo`03o`000?oooooooooo03Wooooo
000=ooooo`03o`000?oooooooooo0=?ooooo00?o0000ooooooooool0>_ooool000gooooo00?o0000
ooooooooool0d_ooool00ol0003oooooooooo`0kooooo`003Oooool00ol0003oooooooooo`3Aoooo
o`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0d?ooool00ol0003oooooooooo`0mooooo`003Ooo
ool00ol0003oooooooooo`3?ooooo`03o`000?oooooooooo03kooooo000=ooooo`03o`000?oooooo
oooo0<kooooo00?o0000ooooooooool0?oooool000gooooo00?o0000ooooooooool0c_ooool00ol0
003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3=ooooo`03o`000?oooooooooo043o
oooo000=ooooo`03o`000?oooooooooo0<cooooo00?o0000ooooooooool0@Oooool000gooooo00?o
0000ooooooooool0boooool00ol0003oooooooooo`12ooooo`003Oooool00ol0003oooooooooo`3;
ooooo`03o`000?oooooooooo04;ooooo000=ooooo`03o`000?oooooooooo0<[ooooo00?o0000oooo
ooooool0@oooool000gooooo00?o0000ooooooooool0bOooool00ol0003oooooooooo`14ooooo`00
3Oooool00ol0003oooooooooo`38ooooo`03o`000?oooooooooo04Gooooo000=ooooo`03o`000?oo
oooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000ooooooooool0aoooool0
0ol0003oooooooooo`16ooooo`003?ooool3ojVYZLOooooo00?o0000ooooooooool0Aoooool000_o
oooo1OnYZJW5ooooo`03o`000?oooooooooo04Sooooo000:ooooo`OoZJVY`oooool00ol0003ooooo
ooooo`19ooooo`002_ooool7ojVYZL?ooooo00?o0000ooooooooool0BOooool000[ooooo1onYZJT2
o`000<3ooooo00?o0000ooooooooool0B_ooool000_ooooo1OnYZJT3ooooo`?o0000_?ooool00ol0
003oooooooooo`1;ooooo`003?ooool3ojVYZ@Oooooo1?l0002gooooo`03o`000?oooooooooo04co
oooo000Jooooo`?o0000]?ooool00ol0003oooooooooo`1<ooooo`007Oooool3o`000;3ooooo00?o
0000ooooooooool0COooool0023ooooo0ol0002/ooooo`03o`000?oooooooooo04kooooo000Soooo
o`Co0000Yoooool00ol0003oooooooooo`1?ooooo`009oooool3o`000:Cooooo00?o0000oooooooo
ool0Coooool002[ooooo0ol0002Pooooo`03o`000?oooooooooo053ooooo000]ooooo`Co0000Vooo
ool00ol0003oooooooooo`1Aooooo`00<Oooool3o`0009Oooooo00?o0000ooooooooool0D_ooool0
03Cooooo0ol0002Dooooo`03o`000?oooooooooo05;ooooo000gooooo`?o0000T?ooool00ol0003o
ooooooooo`1Cooooo`00>_ooool4o`0008_ooooo00?o0000ooooooooool0E?ooool003kooooo0ol0
0027ooooo`03o`000?oooooooooo05Gooooo0011ooooo`?o0000Q?ooool00ol0003oooooooooo`1E
ooooo`00A?ooool4o`0007oooooo00?o0000ooooooooool0E_ooool004Sooooo0ol0001kooooo`03
o`000?oooooooooo05Oooooo001;ooooo`?o0000Moooool00ol0003oooooooooo`1Hooooo`00C_oo
ool3o`0007?ooooo00?o0000ooooooooool0FOooool0057ooooo1?l0001_ooooo`03o`000?oooooo
oooo05Wooooo001Eooooo`?o0000Joooool00ol0003oooooooooo`1Jooooo`00F?ooool3o`0006Oo
oooo00?o0000ooooooooool0Foooool005_ooooo1?l0001Rooooo`03o`000?oooooooooo05cooooo
001Oooooo`?o0000Goooool00ol0003oooooooooo`1Looooo`00H_ooool3o`0005_ooooo00?o0000
ooooooooool0GOooool006Gooooo0ol0001Gooooo`03o`000?oooooooooo05kooooo001Xooooo`Co
0000D_ooool00ol0003oooooooooo`1Oooooo`00K?ooool3o`0004oooooo00?o0000ooooooooool0
Goooool006oooooo0ol0001;ooooo`03o`000?oooooooooo063ooooo001booooo`Co0000A_ooool0
0ol0003oooooooooo`1Qooooo`00M_ooool3o`0004;ooooo00?o0000ooooooooool0H_ooool007Wo
oooo0ol0000oooooo`03o`000?oooooooooo06;ooooo001looooo`?o0000>oooool00ol0003ooooo
ooooo`1Sooooo`00Ooooool4o`0003Kooooo00?o0000ooooooooool0I?ooool008?ooooo0ol0000b
ooooo`03o`000?oooooooooo06Gooooo0026ooooo`?o0000;oooool00ol0003oooooooooo`1Uoooo
o`00ROooool4o`0002[ooooo00?o0000ooooooooool0I_ooool008gooooo0ol0000Vooooo`03o`00
0?oooooooooo06Oooooo002@ooooo`?o00008_ooool00ol0003oooooooooo`1Xooooo`00Toooool3
o`0001kooooo00?o0000ooooooooool0JOooool009Kooooo1?l0000Jooooo`03o`000?oooooooooo
06Wooooo002Jooooo`?o00005_ooool00ol0003oooooooooo`1Zooooo`00WOooool3o`0001;ooooo
00?o0000ooooooooool0Joooool00:3ooooo1?l0000=ooooo`03o`000?oooooooooo06cooooo002T
ooooo`?o00001_ooool3ojVYZ@03ooooool0003ooooo06gooooo002Wooooo`?o00000_ooool5ojVY
ZFoooooo002Zooooo`03o`000?nYZJWoZJVY00GoZJVYK_ooool00:_ooooo1onYZJU^ooooo`00Zooo
ool7ojVYZFkooooo002/ooooo`GoZJVYKoooool00:gooooo0onYZJU`ooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Pick out an agent.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(bob = GetAgent[pd8, 1];\)\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Since we haven't defined any data for Bob, calls to ",
  StyleBox["Data", "Input"],
  " return nothing."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(Data[bob]\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
However, we can store a list of 20 random numbers (for example) in \
Bob's data.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(SetData[
      bob, \[IndentingNewLine]Table[
        Random[], {20}]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[41]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Data[bob]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({0.3231614822049083`, 0.28850739489281174`, 0.3389667285937453`, 
      0.6656098655071319`, 0.37044655339646526`, 0.9621639340443804`, 
      0.22257925541643095`, 0.9986793194092541`, 0.2877272829889356`, 
      0.1902449692781398`, 0.31222722383582857`, 0.9351547307879742`, 
      0.5885162815292886`, 0.3305267921138989`, 0.2593691140405508`, 
      0.3454536789369821`, 0.9071536651680254`, 0.47224768466698097`, 
      0.96461741520595`, 0.9893961354710541`}\)], "Output",
  CellLabel->"Out[42]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "In order to use fictitious play, we need to set a value of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\)]],
  " for Bob, in addition to a variable ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " tracking what generation it is (it should start at 0). We also need to \
set an initial belief vector for Bob. If we want Bob to assign equal \
probability to all strategies we initially set Bob's belief vector to ",
  Cell[BoxData[
      \(TraditionalForm\`{ .5,  .5}\)]],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(SetData[bob, {1.23, 0, {0.5, 0.5}}]\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "The first entry in the list is Bob's ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\)]],
  " value, the second is the generation, and the third is Bob's belief \
vector. Setting data overwrites what was there previously:"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Data[bob]\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({1.23`, 0, {0.5`, 0.5`}}\)], "Output",
  CellLabel->"Out[44]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Step the model forward one generation:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Evolve[pd8];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[ExtractGraph[pd8]];\)\)}], "Input",
  CellLabel->"In[45]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
.663 .663 .663 r
.025 w
.60667 .95401 Mdot
0 0 0 r
.04812 .7806 Mdot
.04812 .2194 Mdot
.60667 .04599 Mdot
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[45]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3o`00073ooooo002/ooooo`Go0000Koooool00:_ooooo1ol0001^ooooo`00Zooo
ool7o`0006kooooo002Yooooo`Wo0000K_ooool00:Kooooo0ol00003ooooo`Go0000Koooool00:;o
oooo1?l00007ooooo`Co0000Koooool009oooooo0ol0000?ooooo`03o`000?oooooooooo06cooooo
002Looooo`?o00004_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`0001Kooooo00?o0000
ooooooooool0Joooool009Gooooo1?l0000Jooooo`03o`000?oooooooooo06[ooooo002Booooo`?o
00007oooool00ol0003oooooooooo`1Yooooo`00Soooool3o`0002;ooooo00?o0000ooooooooool0
JOooool008_ooooo1?l0000Vooooo`03o`000?oooooooooo06Sooooo0028ooooo`?o0000:oooool0
0ol0003oooooooooo`1Wooooo`00QOooool3o`0002oooooo00?o0000ooooooooool0I_ooool008;o
oooo0ol0000cooooo`03o`000?oooooooooo06Gooooo001nooooo`Co0000=_ooool00ol0003ooooo
ooooo`1Uooooo`00Noooool3o`0003_ooooo00?o0000ooooooooool0I?ooool007Sooooo0ol0000o
ooooo`03o`000?oooooooooo06?ooooo001dooooo`Co0000@oooool00ol0003oooooooooo`1Roooo
o`00LOooool3o`0004Oooooo00?o0000ooooooooool0H_ooool006kooooo0ol0001;ooooo`03o`00
0?oooooooooo067ooooo001[ooooo`?o0000Coooool00ol0003oooooooooo`1Pooooo`00Ioooool4
o`0005?ooooo00?o0000ooooooooool0Goooool006Cooooo0ol0001Gooooo`03o`000?oooooooooo
05oooooo001Qooooo`?o0000Foooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0005oooooo
00?o0000ooooooooool0GOooool005[ooooo0ol0001Tooooo`03o`000?oooooooooo05cooooo001G
ooooo`?o0000Ioooool00ol0003oooooooooo`1Looooo`00E?ooool3o`0006_ooooo00?o0000oooo
ooooool0Foooool0053ooooo1?l0001_ooooo`03o`000?oooooooooo05[ooooo001=ooooo`?o0000
M?ooool00ol0003oooooooooo`1Iooooo`00B_ooool3o`0007Oooooo00?o0000ooooooooool0FOoo
ool004Kooooo1?l0001kooooo`03o`000?oooooooooo05Sooooo0013ooooo`?o0000P?ooool00ol0
003oooooooooo`1Gooooo`00@?ooool3o`0008Cooooo00?o0000ooooooooool0E_ooool003gooooo
0ol00028ooooo`03o`000?oooooooooo05Gooooo000iooooo`Co0000Roooool00ol0003ooooooooo
o`1Eooooo`00=_ooool3o`00093ooooo00?o0000ooooooooool0E?ooool003?ooooo0ol0002Doooo
o`03o`000?oooooooooo05?ooooo000_ooooo`Co0000V?ooool00ol0003oooooooooo`1Booooo`00
;?ooool3o`0009cooooo00?o0000ooooooooool0D_ooool002Wooooo0ol0002Pooooo`03o`000?oo
oooooooo057ooooo000Vooooo`?o0000Y?ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`00
0:Sooooo00?o0000ooooooooool0Coooool001oooooo0ol0002/ooooo`03o`000?oooooooooo04oo
oooo000Looooo`?o0000/?ooool00ol0003oooooooooo`1>ooooo`006?ooool4o`000;Cooooo00?o
0000ooooooooool0COooool001Gooooo0ol0002iooooo`03o`000?oooooooooo04cooooo000<oooo
o`?o00000oooool3o`000;cooooo00?o0000ooooooooool0C?ooool000_ooooo1ol00030ooooo`03
o`000?oooooooooo04_ooooo000:ooooo`Oo0000`_ooool00ol0003oooooooooo`1:ooooo`002_oo
ool7o`000<?ooooo00?o0000ooooooooool0BOooool000[ooooo1ol00033ooooo`03o`000?oooooo
oooo04Wooooo000;ooooo`Go0000aOooool00ol0003oooooooooo`18ooooo`003?ooool3o`000<Oo
oooo00?o0000ooooooooool0Aoooool000gooooo00?o0000ooooooooool0aoooool00ol0003ooooo
ooooo`16ooooo`003Oooool00ol0003oooooooooo`38ooooo`03o`000?oooooooooo04Gooooo000=
ooooo`03o`000?oooooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000oooo
ooooool0bOooool00ol0003oooooooooo`14ooooo`003Oooool00ol0003oooooooooo`3:ooooo`03
o`000?oooooooooo04?ooooo000=ooooo`03o`000?oooooooooo0<_ooooo00?o0000ooooooooool0
@_ooool000gooooo00?o0000ooooooooool0boooool00ol0003oooooooooo`12ooooo`003Oooool0
0ol0003oooooooooo`3<ooooo`03o`000?oooooooooo047ooooo000=ooooo`03o`000?oooooooooo
0<gooooo00?o0000ooooooooool0@?ooool000gooooo00?o0000ooooooooool0c_ooool00ol0003o
ooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>ooooo`03o`000?oooooooooo03oooooo
000=ooooo`03o`000?oooooooooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000
ooooooooool0d?ooool00ol0003oooooooooo`0mooooo`003Oooool00ol0003oooooooooo`3Aoooo
o`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0d_ooool00ol0003oooooooooo`0kooooo`003Ooo
ool00ol0003oooooooooo`3Cooooo`03o`000?oooooooooo03[ooooo000=ooooo`03o`000?oooooo
oooo0=Cooooo00?o0000ooooooooool0>Oooool000gooooo00?o0000ooooooooool0e?ooool00ol0
003oooooooooo`0iooooo`003Oooool00ol0003oooooooooo`3Eooooo`03o`000?oooooooooo03So
oooo000=ooooo`03o`000?oooooooooo0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o
0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool00ol0003oooooooooo`3G
ooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooooooo0=Sooooo00?o0000oooo
ooooool0=Oooool000gooooo00?o0000ooooooooool0fOooool00ol0003oooooooooo`0dooooo`00
3Oooool00ol0003oooooooooo`3Jooooo`03o`000?oooooooooo03?ooooo000=ooooo`03o`000?oo
oooooooo0=_ooooo00?o0000ooooooooool0<_ooool000gooooo00?o0000ooooooooool0foooool0
0ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`3Looooo`03o`000?oooooooooo
037ooooo000=ooooo`03o`000?oooooooooo0=gooooo00?o0000ooooooooool0<?ooool000gooooo
00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0003ooooooooo
o`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oooooooooo0=oooooo00?o0000
ooooooooool0;_ooool000gooooo00?o0000ooooooooool0h?ooool00ol0003oooooooooo`0]oooo
o`003Oooool00ol0003oooooooooo`3Qooooo`03o`000?oooooooooo02cooooo000=ooooo`03o`00
0?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000ooooooooool0h_oo
ool00ol0003oooooooooo`0[ooooo`003Oooool00ol0003oooooooooo`3Sooooo`03o`000?oooooo
oooo02[ooooo000=ooooo`03o`000?oooooooooo0>Cooooo00?o0000ooooooooool0:Oooool000go
oooo00?o0000ooooooooool0i?ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003ooooo
ooooo`3Uooooo`03o`000?oooooooooo02Sooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o
0000ooooooooool09oooool000gooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0V
ooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oooooooooo02Kooooo000=ooooo`03
o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000gooooo00?o0000ooooooooool0
jOooool00ol0003oooooooooo`0Tooooo`003Oooool00ol0003oooooooooo`3Zooooo`03o`000?oo
oooooooo02?ooooo000=ooooo`03o`000?oooooooooo0>_ooooo00?o0000ooooooooool08_ooool0
00gooooo00?o0000ooooooooool0joooool00ol0003oooooooooo`0Rooooo`003Oooool00ol0003o
ooooooooo`3/ooooo`03o`000?oooooooooo027ooooo000=ooooo`03o`000?oooooooooo0>gooooo
00?o0000ooooooooool08?ooool000gooooo00?o0000ooooooooool0k_ooool00ol0003ooooooooo
o`0Oooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oooooooooo01oooooo000=oooo
o`03o`000?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo00?o0000oooooooo
ool0l?ooool00ol0003oooooooooo`0Mooooo`003Oooool00ol0003oooooooooo`3aooooo`03o`00
0?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000ooooooooool07?oo
ool000gooooo00?o0000ooooooooool0l_ooool00ol0003oooooooooo`0Kooooo`003Oooool00ol0
003oooooooooo`3cooooo`03o`000?oooooooooo01[ooooo000=ooooo`03o`000?oooooooooo0?Co
oooo00?o0000ooooooooool06Oooool000gooooo00?o0000ooooooooool0m?ooool00ol0003ooooo
ooooo`0Iooooo`003Oooool00ol0003oooooooooo`3eooooo`03o`000?oooooooooo01Sooooo000=
ooooo`03o`000?oooooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o0000oooo
ooooool0moooool00ol0003oooooooooo`0Fooooo`003Oooool00ol0003oooooooooo`3gooooo`03
o`000?oooooooooo01Kooooo000=ooooo`03o`000?oooooooooo0?Sooooo00?o0000ooooooooool0
5Oooool000gooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0Dooooo`003Oooool0
0ol0003oooooooooo`3jooooo`03o`000?oooooooooo01?ooooo000=ooooo`03o`000?oooooooooo
0?_ooooo00?o0000ooooooooool04_ooool000gooooo00?o0000ooooooooool0noooool00ol0003o
ooooooooo`0Booooo`003Oooool00ol0003oooooooooo`3looooo`03o`000?oooooooooo017ooooo
000=ooooo`03o`000?oooooooooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000
ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`3noooo
o`03o`000?oooooo000000;o00003Oooool000gooooo00?o0000ooooooooool0ooooool5o`0000co
oooo000=ooooo`03o`000?oooooooooo0?kooooo1ol0000;ooooo`003Oooool00ol0003ooooooooo
o`3nooooo`Oo00002oooool000gooooo00?o0000ooooooooool0o_ooool7o`0000_ooooo000=oooo
o`03o`000?oooooooooo0?oooooo1Ol0000<ooooo`003Oooool00ol0003oooooooooo`3oooooo`Co
00003Oooool000gooooo00?o0000ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Ooo
ool00ol0003oooooooooo`3nooooo`03o`000?oooooooooo00oooooo000=ooooo`03o`000?oooooo
oooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000ooooooooool0o?ooool00ol0
003oooooooooo`0Aooooo`003Oooool00ol0003oooooooooo`3kooooo`03o`000?oooooooooo01;o
oooo000=ooooo`03o`000?oooooooooo0?_ooooo00?o0000ooooooooool04_ooool000gooooo00?o
0000ooooooooool0n_ooool00ol0003oooooooooo`0Cooooo`003Oooool00ol0003oooooooooo`3i
ooooo`03o`000?oooooooooo01Cooooo000=ooooo`03o`000?oooooooooo0?Sooooo00?o0000oooo
ooooool05Oooool000gooooo00?o0000ooooooooool0moooool00ol0003oooooooooo`0Fooooo`00
3Oooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo01Kooooo000=ooooo`03o`000?oo
oooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o0000ooooooooool0mOooool0
0ol0003oooooooooo`0Hooooo`003Oooool00ol0003oooooooooo`3dooooo`03o`000?oooooooooo
01Wooooo000=ooooo`03o`000?oooooooooo0?Cooooo00?o0000ooooooooool06Oooool000gooooo
00?o0000ooooooooool0loooool00ol0003oooooooooo`0Jooooo`003Oooool00ol0003ooooooooo
o`3booooo`03o`000?oooooooooo01_ooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000
ooooooooool07?ooool000gooooo00?o0000ooooooooool0lOooool00ol0003oooooooooo`0Loooo
o`003Oooool00ol0003oooooooooo`3`ooooo`03o`000?oooooooooo01gooooo000=ooooo`03o`00
0?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo00?o0000ooooooooool0k_oo
ool00ol0003oooooooooo`0Oooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oooooo
oooo01oooooo000=ooooo`03o`000?oooooooooo0>gooooo00?o0000ooooooooool08?ooool000go
oooo00?o0000ooooooooool0k?ooool00ol0003oooooooooo`0Qooooo`003Oooool00ol0003ooooo
ooooo`3[ooooo`03o`000?oooooooooo02;ooooo000=ooooo`03o`000?oooooooooo0>_ooooo00?o
0000ooooooooool08_ooool000gooooo00?o0000ooooooooool0j_ooool00ol0003oooooooooo`0S
ooooo`003Oooool00ol0003oooooooooo`3Yooooo`03o`000?oooooooooo02Cooooo000=ooooo`03
o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000gooooo00?o0000ooooooooool0
ioooool00ol0003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oo
oooooooo02Kooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o0000ooooooooool09oooool0
00gooooo00?o0000ooooooooool0iOooool00ol0003oooooooooo`0Xooooo`003Oooool00ol0003o
ooooooooo`3Tooooo`03o`000?oooooooooo02Wooooo000=ooooo`03o`000?oooooooooo0>Cooooo
00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0hoooool00ol0003ooooooooo
o`0Zooooo`003Oooool00ol0003oooooooooo`3Rooooo`03o`000?oooooooooo02_ooooo000=oooo
o`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000oooooooo
ool0hOooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003oooooooooo`3Pooooo`03o`00
0?oooooooooo02gooooo000=ooooo`03o`000?oooooooooo0=oooooo00?o0000ooooooooool0;_oo
ool000gooooo00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0
003oooooooooo`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oooooooooo0=go
oooo00?o0000ooooooooool0<?ooool000gooooo00?o0000ooooooooool0g?ooool00ol0003ooooo
ooooo`0aooooo`003Oooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo03;ooooo000=
ooooo`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool0<_ooool000gooooo00?o0000oooo
ooooool0f_ooool00ol0003oooooooooo`0cooooo`003Oooool00ol0003oooooooooo`3Iooooo`03
o`000?oooooooooo03Cooooo000=ooooo`03o`000?oooooooooo0=Sooooo00?o0000ooooooooool0
=Oooool000gooooo00?o0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool0
0ol0003oooooooooo`3Gooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooooooo
0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o0000ooooooooool0eOooool00ol0003o
ooooooooo`0hooooo`003Oooool00ol0003oooooooooo`3Dooooo`03o`000?oooooooooo03Wooooo
000=ooooo`03o`000?oooooooooo0=Cooooo00?o0000ooooooooool0>Oooool000gooooo00?o0000
ooooooooool0doooool00ol0003oooooooooo`0jooooo`003Oooool00ol0003oooooooooo`3Boooo
o`03o`000?oooooooooo03_ooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0dOooool00ol0003oooooooooo`0looooo`003Ooo
ool00ol0003oooooooooo`3@ooooo`03o`000?oooooooooo03gooooo000=ooooo`03o`000?oooooo
oooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000ooooooooool0c_ooool00ol0
003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>ooooo`03o`000?oooooooooo03oo
oooo000=ooooo`03o`000?oooooooooo0<gooooo00?o0000ooooooooool0@?ooool000gooooo00?o
0000ooooooooool0c?ooool00ol0003oooooooooo`11ooooo`003Oooool00ol0003oooooooooo`3;
ooooo`03o`000?oooooooooo04;ooooo000=ooooo`03o`000?oooooooooo0<_ooooo00?o0000oooo
ooooool0@_ooool000gooooo00?o0000ooooooooool0b_ooool00ol0003oooooooooo`13ooooo`00
3Oooool00ol0003oooooooooo`39ooooo`03o`000?oooooooooo04Cooooo000=ooooo`03o`000?oo
oooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000ooooooooool0b?ooool0
0ol0003oooooooooo`15ooooo`003Oooool00ol0003oooooooooo`37ooooo`03o`000?oooooooooo
04Kooooo000<ooooo`?o0000aoooool00ol0003oooooooooo`17ooooo`002oooool5o`000<Gooooo
00?o0000ooooooooool0B?ooool000[ooooo1ol00033ooooo`03o`000?oooooooooo04Wooooo000:
ooooo`Oo0000`oooool00ol0003oooooooooo`19ooooo`002_ooool9o`000<3ooooo00?o0000oooo
ooooool0B_ooool000_ooooo1Ol00003ooooo`?o0000_?ooool00ol0003oooooooooo`1;ooooo`00
3?ooool3o`0000Oooooo1?l0002gooooo`03o`000?oooooooooo04cooooo000Jooooo`?o0000]?oo
ool00ol0003oooooooooo`1<ooooo`007Oooool3o`000;3ooooo00?o0000ooooooooool0COooool0
023ooooo0ol0002/ooooo`03o`000?oooooooooo04kooooo000Sooooo`Co0000Yoooool00ol0003o
ooooooooo`1?ooooo`009oooool3o`000:Cooooo00?o0000ooooooooool0Coooool002[ooooo0ol0
002Pooooo`03o`000?oooooooooo053ooooo000]ooooo`Co0000Voooool00ol0003oooooooooo`1A
ooooo`00<Oooool3o`0009Oooooo00?o0000ooooooooool0D_ooool003Cooooo0ol0002Dooooo`03
o`000?oooooooooo05;ooooo000gooooo`?o0000T?ooool00ol0003oooooooooo`1Cooooo`00>_oo
ool4o`0008_ooooo00?o0000ooooooooool0E?ooool003kooooo0ol00027ooooo`03o`000?oooooo
oooo05Gooooo0011ooooo`?o0000Q?ooool00ol0003oooooooooo`1Eooooo`00A?ooool4o`0007oo
oooo00?o0000ooooooooool0E_ooool004Sooooo0ol0001kooooo`03o`000?oooooooooo05Oooooo
001;ooooo`?o0000Moooool00ol0003oooooooooo`1Hooooo`00C_ooool3o`0007?ooooo00?o0000
ooooooooool0FOooool0057ooooo1?l0001_ooooo`03o`000?oooooooooo05Wooooo001Eooooo`?o
0000Joooool00ol0003oooooooooo`1Jooooo`00F?ooool3o`0006Oooooo00?o0000ooooooooool0
Foooool005_ooooo1?l0001Rooooo`03o`000?oooooooooo05cooooo001Oooooo`?o0000Goooool0
0ol0003oooooooooo`1Looooo`00H_ooool3o`0005_ooooo00?o0000ooooooooool0GOooool006Go
oooo0ol0001Gooooo`03o`000?oooooooooo05kooooo001Xooooo`Co0000D_ooool00ol0003ooooo
ooooo`1Oooooo`00K?ooool3o`0004oooooo00?o0000ooooooooool0Goooool006oooooo0ol0001;
ooooo`03o`000?oooooooooo063ooooo001booooo`Co0000A_ooool00ol0003oooooooooo`1Qoooo
o`00M_ooool3o`0004;ooooo00?o0000ooooooooool0H_ooool007Wooooo0ol0000oooooo`03o`00
0?oooooooooo06;ooooo001looooo`?o0000>oooool00ol0003oooooooooo`1Sooooo`00Ooooool4
o`0003Kooooo00?o0000ooooooooool0I?ooool008?ooooo0ol0000booooo`03o`000?oooooooooo
06Gooooo0026ooooo`?o0000;oooool00ol0003oooooooooo`1Uooooo`00ROooool4o`0002[ooooo
00?o0000ooooooooool0I_ooool008gooooo0ol0000Vooooo`03o`000?oooooooooo06Oooooo002@
ooooo`?o00008_ooool00ol0003oooooooooo`1Xooooo`00Toooool3o`0001kooooo00?o0000oooo
ooooool0JOooool009Kooooo1?l0000Jooooo`03o`000?oooooooooo06Wooooo002Jooooo`?o0000
5_ooool00ol0003oooooooooo`1Zooooo`00WOooool3o`0001;ooooo00?o0000ooooooooool0Jooo
ool00:3ooooo1?l0000=ooooo`03o`000?oooooooooo06cooooo002Tooooo`?o00001_ooool3ojVY
Z@03ooooool0003ooooo06gooooo002Wooooo`?o00000_ooool5ojVYZFoooooo002Zooooo`03o`00
0?nYZJWoZJVY00GoZJVYK_ooool00:_ooooo1onYZJU^ooooo`00Zoooool7ojVYZFkooooo002/oooo
o`GoZJVYKoooool00:gooooo0onYZJU`ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "We can get a list of the observed strategies for an agent by calling ",
  StyleBox["ObservedStrategies", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ObservedStrategies[bob]\)], "Input",
  CellLabel->"In[47]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({2, 0}\)], "Output",
  CellLabel->"Out[47]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Notice that, although we've stepped the model forward one \
generation, Bob still holds the data we set.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Data[bob]\)], "Input",
  CellLabel->"In[48]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({1.23`, 0, {0.5`, 0.5`}}\)], "Output",
  CellLabel->"Out[48]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
Here's an example of how we can extract the data stored in Bob and \
use it to update his beliefs according to traditional fictitious play:\
\>", \
"Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Module[{\[Lambda], t, obs, beliefs, data}, \[IndentingNewLine]data = 
        Data[bob]; \[IndentingNewLine]\[Lambda] = 
        data\[LeftDoubleBracket]1\[RightDoubleBracket]; \[IndentingNewLine]t \
= data\[LeftDoubleBracket]2\[RightDoubleBracket]; \[IndentingNewLine]obs = 
        ObservedStrategies[bob]; \[IndentingNewLine]beliefs = 
        data\[LeftDoubleBracket]3\[RightDoubleBracket]; \
\[IndentingNewLine]TraditionalFictitiousPlay[\[Lambda], t, obs, 
        beliefs]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[49]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(data\\)\\\" is similar to existing symbol \\\
\"\\!\\(Data\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[49]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({0.7242152466367713`, 0.2757847533632287`}\)], "Output",
  CellLabel->"Out[49]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[pd8]\)], "Input",
  CellLabel->"In[50]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "So far, so good. But this is only part of what fictitious play involves. \
Given a vector ",
  Cell[BoxData[
      \(TraditionalForm\`v\&\[RightVector]\)]],
  " representing one's beliefs as to the probability of certain strategies \
being played in the next round, we then have to calculate the expected \
payoffs for each strategy so as to choose the best response."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell["Suppose the payoff matrix looks like the following:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{"M", "=", 
        RowBox[{"(", GridBox[{
              {\({x\_11, y\_11}\), 
                "\[CenterEllipsis]", \({x\_\(1  n\), y\_\(1  n\)}\)},
              {"\[VerticalEllipsis]", "\[DescendingEllipsis]", 
                "\[VerticalEllipsis]"},
              {\({x\_n1, y\_n1}\), "\[CenterEllipsis]", \({x\_nn, y\_nn}\)}
              }], ")"}]}], TraditionalForm]]]], "Text",
  TextAlignment->Center,
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "If the belief vector ",
  Cell[BoxData[
      \(TraditionalForm\`v\&\[RightVector] = \((p\_1, \[Ellipsis], 
          p\_n)\)\)]],
  " represents a subjective probability distribution over other people's \
strategies, then the expected payoff for following strategy ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " is"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\(1\/2\) \(\(p\_1\)(
            x\_i1 + y\_\(1  i\))\) + \(1\/2\) \(\(p\_2\)(
            x\_i2 + y\_\(2  i\))\) + \[CenterEllipsis] + \(1\/2\) \(\(p\_n\)(
            x\_in + y\_ni)\)\)]],
  "."
}], "Text",
  TextAlignment->Center,
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  StyleBox["ExpectedPayoffs", "Input"],
  " takes a belief vector and payoff matrix as its arguments and returns a \
list of expected payoffs for each strategy. Notice that in the definition of \
the function we have ",
  StyleBox["m\[LeftDoubleBracket]i,j,1\[RightDoubleBracket]", "Input"],
  " and ",
  StyleBox["m\[LeftDoubleBracket]j,i,2\[RightDoubleBracket]", "Input"],
  ". This is because if"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"m", "=", 
      RowBox[{"(", GridBox[{
            {\({2, 2}\), \({0, 3}\)},
            {\({3, 0}\), \({1, 1}\)}
            }], ")"}]}]], "Input",
  CellLabel->"In[51]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{{2, 2}, {0, 3}}, {{3, 0}, {1, 1}}}\)], "Output",
  CellLabel->"Out[51]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "then ",
  StyleBox["m\[LeftDoubleBracket]i,j\[RightDoubleBracket]", "Input"],
  " refers to the entry in row ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " and column ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  ", namely an ordered tuple. The third number (either 1 or 2) is needed to \
specify the payoff for row or column."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(ExpectedPayoffs[b_?VectorQ, m_] := 
        Module[{size}, \[IndentingNewLine]size = 
            Length[m]; \
\[IndentingNewLine]Table[\[IndentingNewLine]\[Sum]\+\(j = 1\)\%size\( 1\/2\) 
                b\[LeftDoubleBracket]
                  j\[RightDoubleBracket] \((m\[LeftDoubleBracket]i, j, 
                      1\[RightDoubleBracket] + 
                    m\[LeftDoubleBracket]j, i, 
                      2\[RightDoubleBracket])\), \[IndentingNewLine]{i, 1, 
              size}\[IndentingNewLine]]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[52]:=",
  Background->RGBColor[0.984192, 1, 0.723949],
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Here's an example for a Prisoner's Dilemma payoff matrix and flat \
prior.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpectedPayoffs", "[", 
      RowBox[{\({ .5,  .5}\), ",", 
        RowBox[{"(", GridBox[{
              {\({2, 2}\), \({0, 3}\)},
              {\({3, 0}\), \({1, 1}\)}
              }], ")"}]}], "]"}]], "Input",
  CellLabel->"In[53]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({1.`, 2.`}\)], "Output",
  CellLabel->"Out[53]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Given all this, we may define ",
  StyleBox["TraditionalFictitiousPlayLR", "Input"],
  " (the \"LR\" stands for Learning Rule) as follows, assuming that the ",
  StyleBox["Data", "Input"],
  " for an agent is of the form ",
  Cell[BoxData[
      \(TraditionalForm\`{\[Lambda], t, v\&\[RightVector]}\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`v\&\[RightVector]\)]],
  " is the vector of beliefs."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{\(TraditionalFictitiousPlayLR[agent_]\), ":=", 
        "\[IndentingNewLine]", 
        RowBox[{"Module", "[", "\[IndentingNewLine]", 
          
          RowBox[{\({\[Lambda], t, obs, b, data, m, newVector, max, ep, next, 
              strategies}\), ",", "\[IndentingNewLine]", 
            
            RowBox[{\(data = Data[agent]\), ";", 
              "\[IndentingNewLine]", \(\[Lambda] = 
                data\[LeftDoubleBracket]1\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(t = 
                data\[LeftDoubleBracket]2\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(b = 
                data\[LeftDoubleBracket]3\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(obs = ObservedStrategies[agent]\), ";",
               "\[IndentingNewLine]", \(m = PayoffMatrix[agent]\), ";", 
              "\[IndentingNewLine]", 
              StyleBox[\( (*Now\ determine\ best\ response*) \),
                FontColor->RGBColor[0, 0, 1]], 
              "\[IndentingNewLine]", \(ep = ExpectedPayoffs[b, m]\), ";", 
              "\[IndentingNewLine]", \(max = Max[ep]\), ";", 
              "\[IndentingNewLine]", \(strategies = 
                Position[ep, max] // Flatten\), ";", 
              "\[IndentingNewLine]", \(next = 
                RandomKSubset[strategies, 1] // First\), ";", 
              "\[IndentingNewLine]", \(SetNextStrategy[agent, next]\), ";", 
              "\[IndentingNewLine]", 
              StyleBox[\( (*Now\ update\ the\ data\ for\ the\ agent*) \),
                FontColor->RGBColor[0, 0, 1]], 
              "\[IndentingNewLine]", \(newVector = 
                TraditionalFictitiousPlay[\[Lambda], t, obs, b]\), ";", 
              "\[IndentingNewLine]", \(SetData[
                agent, {\[Lambda], t + 1, newVector}]\), ";"}]}], 
          "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[54]:=",
  Background->RGBColor[0.984192, 1, 0.723949],
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(next\\)\\\" is similar to existing symbol \\\
\"\\!\\(Next\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[54]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(strategies\\)\\\" is similar to existing \
symbol \\\"\\!\\(Strategies\\)\\\". \
\\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\\\"RefGuideLinkText\
\\\", ButtonFrame->None, ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], \
"Message",
  CellLabel->"From In[54]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
The only difference between this and exponential fictitious play \
involves which function we use to update belief vectors.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(ExponentialFictitiousPlayLR[agent_]\), ":=", 
        "\[IndentingNewLine]", 
        RowBox[{"Module", "[", "\[IndentingNewLine]", 
          
          RowBox[{\({\[Theta], t, obs, b, data, m, newVector, max, ep, next, 
              strategies}\), ",", "\[IndentingNewLine]", 
            
            RowBox[{\(data = Data[agent]\), ";", 
              "\[IndentingNewLine]", \(\[Theta] = 
                data\[LeftDoubleBracket]1\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(t = 
                data\[LeftDoubleBracket]2\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(b = 
                data\[LeftDoubleBracket]3\[RightDoubleBracket]\), ";", 
              "\[IndentingNewLine]", \(obs = ObservedStrategies[agent]\), ";",
               "\[IndentingNewLine]", \(m = PayoffMatrix[agent]\), ";", 
              "\[IndentingNewLine]", 
              StyleBox[\( (*Now\ determine\ best\ response*) \),
                FontColor->RGBColor[0, 0, 1]], 
              "\[IndentingNewLine]", \(ep = ExpectedPayoffs[b, m]\), ";", 
              "\[IndentingNewLine]", \(max = Max[ep]\), ";", 
              "\[IndentingNewLine]", \(strategies = 
                Position[ep, max] // Flatten\), ";", 
              "\[IndentingNewLine]", \(next = 
                RandomKSubset[strategies, 1] // First\), ";", 
              "\[IndentingNewLine]", \(SetNextStrategy[agent, next]\), ";", 
              "\[IndentingNewLine]", 
              StyleBox[\( (*Now\ update\ the\ data\ for\ the\ agent*) \),
                FontColor->RGBColor[0, 0, 1]], 
              "\[IndentingNewLine]", \(newVector = 
                ExponentialFictitiousPlay[\[Theta], t, obs, b]\), ";", 
              "\[IndentingNewLine]", \(SetData[
                agent, {\[Theta], t + 1, newVector}]\), ";"}]}], 
          "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[55]:=",
  Background->RGBColor[0.984192, 1, 0.723949],
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Now let's put all this together into a model\[Ellipsis]\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"pd9", "=", 
        RowBox[{"LocalInteractionModel", "[", "\[IndentingNewLine]", 
          
          RowBox[{\(InteractionNeighborhood \[Rule] Cycle[5]\), ",", 
            "\[IndentingNewLine]", 
            RowBox[{"PayoffMatrix", "\[Rule]", 
              RowBox[{"(", GridBox[{
                    {\({2, 2}\), \({0, 3}\)},
                    {\({3, 0}\), \({1, 1}\)}
                    }], ")"}]}], ",", 
            "\[IndentingNewLine]", \(StrategyColors \[Rule] {LightGray, 
                Black}\)}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[56]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Here's a function which initializes the data set of each \
agent\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Initialize[agent_] := 
        Module[{\[Lambda]}, \[IndentingNewLine]\[Lambda] = 
            Random[Real, {0, 4}]; \[IndentingNewLine]SetData[
            agent, {\[Lambda], 
              0, {0.5, 0.5}}];\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[57]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Apply it to each agent to initialize them.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(agentList = Agents[pd9];\)\), "\n", 
    \(\(Map[Initialize, agentList];\)\)}], "Input",
  CellLabel->"In[58]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Let's inspect the data for each agent.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Data, Agents[pd9]]\)], "Input",
  CellLabel->"In[60]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{2.033592957101664`, 0, {0.5`, 0.5`}}, {1.9656109911108919`, 
        0, {0.5`, 0.5`}}, {3.2198363654018864`, 
        0, {0.5`, 0.5`}}, {1.5604080932460693`, 
        0, {0.5`, 0.5`}}, {0.7409470282820309`, 0, {0.5`, 0.5`}}}\)], "Output",\

  CellLabel->"Out[60]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Set the learning rule for each agent to ",
  StyleBox["TraditionalFictitiousPlayLR.", "Input"]
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Map[\[IndentingNewLine]SetLearningRule[#, 
            TraditionalFictitiousPlayLR] &, \[IndentingNewLine]agentList\
\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[61]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Verify that each agent has the right learning rule installed.\
\>", \
"Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[LearningRule, agentList]\)], "Input",
  CellLabel->"In[62]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({TraditionalFictitiousPlayLR, TraditionalFictitiousPlayLR, 
      TraditionalFictitiousPlayLR, TraditionalFictitiousPlayLR, 
      TraditionalFictitiousPlayLR}\)], "Output",
  CellLabel->"Out[62]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["The world should look something like this:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[pd9]];\)\)], "Input",
  CellLabel->"In[63]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
.025 w
.60667 .95401 Mdot
.663 .663 .663 r
.04812 .7806 Mdot
.04812 .2194 Mdot
0 0 0 r
.60667 .04599 Mdot
.663 .663 .663 r
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[63]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3o`00073ooooo002/ooooo`Go0000Koooool00:_ooooo1ol0001^ooooo`00Zooo
ool7o`0006kooooo002Yooooo`Wo0000K_ooool00:Kooooo0ol00003ooooo`Go0000Koooool00:;o
oooo1?l00007ooooo`Co0000Koooool009oooooo0ol0000?ooooo`03o`000?oooooooooo06cooooo
002Looooo`?o00004_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`0001Kooooo00?o0000
ooooooooool0Joooool009Gooooo1?l0000Jooooo`03o`000?oooooooooo06[ooooo002Booooo`?o
00007oooool00ol0003oooooooooo`1Yooooo`00Soooool3o`0002;ooooo00?o0000ooooooooool0
JOooool008_ooooo1?l0000Vooooo`03o`000?oooooooooo06Sooooo0028ooooo`?o0000:oooool0
0ol0003oooooooooo`1Wooooo`00QOooool3o`0002oooooo00?o0000ooooooooool0I_ooool008;o
oooo0ol0000cooooo`03o`000?oooooooooo06Gooooo001nooooo`Co0000=_ooool00ol0003ooooo
ooooo`1Uooooo`00Noooool3o`0003_ooooo00?o0000ooooooooool0I?ooool007Sooooo0ol0000o
ooooo`03o`000?oooooooooo06?ooooo001dooooo`Co0000@oooool00ol0003oooooooooo`1Roooo
o`00LOooool3o`0004Oooooo00?o0000ooooooooool0H_ooool006kooooo0ol0001;ooooo`03o`00
0?oooooooooo067ooooo001[ooooo`?o0000Coooool00ol0003oooooooooo`1Pooooo`00Ioooool4
o`0005?ooooo00?o0000ooooooooool0Goooool006Cooooo0ol0001Gooooo`03o`000?oooooooooo
05oooooo001Qooooo`?o0000Foooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0005oooooo
00?o0000ooooooooool0GOooool005[ooooo0ol0001Tooooo`03o`000?oooooooooo05cooooo001G
ooooo`?o0000Ioooool00ol0003oooooooooo`1Looooo`00E?ooool3o`0006_ooooo00?o0000oooo
ooooool0Foooool0053ooooo1?l0001_ooooo`03o`000?oooooooooo05[ooooo001=ooooo`?o0000
M?ooool00ol0003oooooooooo`1Iooooo`00B_ooool3o`0007Oooooo00?o0000ooooooooool0FOoo
ool004Kooooo1?l0001kooooo`03o`000?oooooooooo05Sooooo0013ooooo`?o0000P?ooool00ol0
003oooooooooo`1Gooooo`00@?ooool3o`0008Cooooo00?o0000ooooooooool0E_ooool003gooooo
0ol00028ooooo`03o`000?oooooooooo05Gooooo000iooooo`Co0000Roooool00ol0003ooooooooo
o`1Eooooo`00=_ooool3o`00093ooooo00?o0000ooooooooool0E?ooool003?ooooo0ol0002Doooo
o`03o`000?oooooooooo05?ooooo000_ooooo`Co0000V?ooool00ol0003oooooooooo`1Booooo`00
;?ooool3o`0009cooooo00?o0000ooooooooool0D_ooool002Wooooo0ol0002Pooooo`03o`000?oo
oooooooo057ooooo000Vooooo`?o0000Y?ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`00
0:Sooooo00?o0000ooooooooool0Coooool001oooooo0ol0002/ooooo`03o`000?oooooooooo04oo
oooo000Looooo`?o0000/?ooool00ol0003oooooooooo`1>ooooo`006?ooool4o`000;Cooooo00?o
0000ooooooooool0COooool001Gooooo0ol0002iooooo`03o`000?oooooooooo04cooooo000<oooo
o`?oZJVY0oooool3o`000;cooooo00?o0000ooooooooool0C?ooool000_ooooo1OnYZJT2o`000<3o
oooo00?o0000ooooooooool0Boooool000[ooooo1onYZJW2ooooo`03o`000?oooooooooo04[ooooo
000:ooooo`OoZJVY`oooool00ol0003oooooooooo`19ooooo`002_ooool7ojVYZL?ooooo00?o0000
ooooooooool0BOooool000_ooooo1OnYZJW5ooooo`03o`000?oooooooooo04Sooooo000<ooooo`?o
ZJVYaoooool00ol0003oooooooooo`17ooooo`003Oooool00ol0003oooooooooo`37ooooo`03o`00
0?oooooooooo04Kooooo000=ooooo`03o`000?oooooooooo0<Sooooo00?o0000ooooooooool0AOoo
ool000gooooo00?o0000ooooooooool0b?ooool00ol0003oooooooooo`15ooooo`003Oooool00ol0
003oooooooooo`39ooooo`03o`000?oooooooooo04Cooooo000=ooooo`03o`000?oooooooooo0<[o
oooo00?o0000ooooooooool0@oooool000gooooo00?o0000ooooooooool0boooool00ol0003ooooo
ooooo`12ooooo`003Oooool00ol0003oooooooooo`3;ooooo`03o`000?oooooooooo04;ooooo000=
ooooo`03o`000?oooooooooo0<cooooo00?o0000ooooooooool0@Oooool000gooooo00?o0000oooo
ooooool0cOooool00ol0003oooooooooo`10ooooo`003Oooool00ol0003oooooooooo`3>ooooo`03
o`000?oooooooooo03oooooo000=ooooo`03o`000?oooooooooo0<kooooo00?o0000ooooooooool0
?oooool000gooooo00?o0000ooooooooool0coooool00ol0003oooooooooo`0nooooo`003Oooool0
0ol0003oooooooooo`3@ooooo`03o`000?oooooooooo03gooooo000=ooooo`03o`000?oooooooooo
0=7ooooo00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool0dOooool00ol0003o
ooooooooo`0looooo`003Oooool00ol0003oooooooooo`3Booooo`03o`000?oooooooooo03_ooooo
000=ooooo`03o`000?oooooooooo0=?ooooo00?o0000ooooooooool0>_ooool000gooooo00?o0000
ooooooooool0e?ooool00ol0003oooooooooo`0iooooo`003Oooool00ol0003oooooooooo`3Doooo
o`03o`000?oooooooooo03Wooooo000=ooooo`03o`000?oooooooooo0=Gooooo00?o0000oooooooo
ool0>?ooool000gooooo00?o0000ooooooooool0e_ooool00ol0003oooooooooo`0gooooo`003Ooo
ool00ol0003oooooooooo`3Gooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooo
oooo0=Oooooo00?o0000ooooooooool0=_ooool000gooooo00?o0000ooooooooool0f?ooool00ol0
003oooooooooo`0eooooo`003Oooool00ol0003oooooooooo`3Iooooo`03o`000?oooooooooo03Co
oooo000=ooooo`03o`000?oooooooooo0=[ooooo00?o0000ooooooooool0<oooool000gooooo00?o
0000ooooooooool0foooool00ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`3K
ooooo`03o`000?oooooooooo03;ooooo000=ooooo`03o`000?oooooooooo0=cooooo00?o0000oooo
ooooool0<Oooool000gooooo00?o0000ooooooooool0gOooool00ol0003oooooooooo`0`ooooo`00
3Oooool00ol0003oooooooooo`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oo
oooooooo0=kooooo00?o0000ooooooooool0;oooool000gooooo00?o0000ooooooooool0goooool0
0ol0003oooooooooo`0^ooooo`003Oooool00ol0003oooooooooo`3Pooooo`03o`000?oooooooooo
02gooooo000=ooooo`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo
00?o0000ooooooooool0hOooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003ooooooooo
o`3Rooooo`03o`000?oooooooooo02_ooooo000=ooooo`03o`000?oooooooooo0>?ooooo00?o0000
ooooooooool0:_ooool000gooooo00?o0000ooooooooool0i?ooool00ol0003oooooooooo`0Yoooo
o`003Oooool00ol0003oooooooooo`3Tooooo`03o`000?oooooooooo02Wooooo000=ooooo`03o`00
0?oooooooooo0>Gooooo00?o0000ooooooooool0:?ooool000gooooo00?o0000ooooooooool0i_oo
ool00ol0003oooooooooo`0Wooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oooooo
oooo02Kooooo000=ooooo`03o`000?oooooooooo0>Oooooo00?o0000ooooooooool09_ooool000go
oooo00?o0000ooooooooool0j?ooool00ol0003oooooooooo`0Uooooo`003Oooool00ol0003ooooo
ooooo`3Yooooo`03o`000?oooooooooo02Cooooo000=ooooo`03o`000?oooooooooo0>[ooooo00?o
0000ooooooooool08oooool000gooooo00?o0000ooooooooool0joooool00ol0003oooooooooo`0R
ooooo`003Oooool00ol0003oooooooooo`3[ooooo`03o`000?oooooooooo02;ooooo000=ooooo`03
o`000?oooooooooo0>cooooo00?o0000ooooooooool08Oooool000gooooo00?o0000ooooooooool0
kOooool00ol0003oooooooooo`0Pooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oo
oooooooo01oooooo000=ooooo`03o`000?oooooooooo0>kooooo00?o0000ooooooooool07oooool0
00gooooo00?o0000ooooooooool0koooool00ol0003oooooooooo`0Nooooo`003Oooool00ol0003o
ooooooooo`3`ooooo`03o`000?oooooooooo01gooooo000=ooooo`03o`000?oooooooooo0?7ooooo
00?o0000ooooooooool07?ooool000gooooo00?o0000ooooooooool0lOooool00ol0003ooooooooo
o`0Looooo`003Oooool00ol0003oooooooooo`3booooo`03o`000?oooooooooo01_ooooo000=oooo
o`03o`000?oooooooooo0??ooooo00?o0000ooooooooool06_ooool000gooooo00?o0000oooooooo
ool0m?ooool00ol0003oooooooooo`0Iooooo`003Oooool00ol0003oooooooooo`3dooooo`03o`00
0?oooooooooo01Wooooo000=ooooo`03o`000?oooooooooo0?Gooooo00?o0000ooooooooool06?oo
ool000gooooo00?o0000ooooooooool0m_ooool00ol0003oooooooooo`0Gooooo`003Oooool00ol0
003oooooooooo`3gooooo`03o`000?oooooooooo01Kooooo000=ooooo`03o`000?oooooooooo0?Oo
oooo00?o0000ooooooooool05_ooool000gooooo00?o0000ooooooooool0n?ooool00ol0003ooooo
ooooo`0Eooooo`003Oooool00ol0003oooooooooo`3iooooo`03o`000?oooooooooo01Cooooo000=
ooooo`03o`000?oooooooooo0?[ooooo00?o0000ooooooooool04oooool000gooooo00?o0000oooo
ooooool0noooool00ol0003oooooooooo`0Booooo`003Oooool00ol0003oooooooooo`3kooooo`03
o`000?oooooooooo01;ooooo000=ooooo`03o`000?oooooooooo0?cooooo00?o0000ooooooooool0
4Oooool000gooooo00?o0000ooooooooool0oOooool00ol0003oooooooooo`0@ooooo`003Oooool0
0ol0003oooooooooo`3nooooo`03o`000?oooooooooo00oooooo000=ooooo`03o`000?oooooooooo
0?kooooo00?o0000oooooonYZJT00_nYZJT=ooooo`003Oooool00ol0003oooooooooo`3oooooo`Go
ZJVY3?ooool000gooooo00?o0000ooooooooool0o_ooool7ojVYZ@_ooooo000=ooooo`03o`000?oo
oooooooo0?kooooo1onYZJT;ooooo`003Oooool00ol0003oooooooooo`3nooooo`OoZJVY2oooool0
00gooooo00?o0000ooooooooool0ooooool5ojVYZ@cooooo000=ooooo`03o`000?oooooooooo0?oo
oooo00Co0000ojVYZOnYZJWoZJVY3Oooool000gooooo00?o0000ooooooooool0o_ooool00ol0003o
ooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`3nooooo`03o`000?oooooooooo00oooooo
000=ooooo`03o`000?oooooooooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000
ooooooooool0o?ooool00ol0003oooooooooo`0Aooooo`003Oooool00ol0003oooooooooo`3koooo
o`03o`000?oooooooooo01;ooooo000=ooooo`03o`000?oooooooooo0?_ooooo00?o0000oooooooo
ool04_ooool000gooooo00?o0000ooooooooool0n_ooool00ol0003oooooooooo`0Cooooo`003Ooo
ool00ol0003oooooooooo`3iooooo`03o`000?oooooooooo01Cooooo000=ooooo`03o`000?oooooo
oooo0?Sooooo00?o0000ooooooooool05Oooool000gooooo00?o0000ooooooooool0moooool00ol0
003oooooooooo`0Fooooo`003Oooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo01Ko
oooo000=ooooo`03o`000?oooooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o
0000ooooooooool0mOooool00ol0003oooooooooo`0Hooooo`003Oooool00ol0003oooooooooo`3d
ooooo`03o`000?oooooooooo01Wooooo000=ooooo`03o`000?oooooooooo0?Cooooo00?o0000oooo
ooooool06Oooool000gooooo00?o0000ooooooooool0loooool00ol0003oooooooooo`0Jooooo`00
3Oooool00ol0003oooooooooo`3booooo`03o`000?oooooooooo01_ooooo000=ooooo`03o`000?oo
oooooooo0?7ooooo00?o0000ooooooooool07?ooool000gooooo00?o0000ooooooooool0lOooool0
0ol0003oooooooooo`0Looooo`003Oooool00ol0003oooooooooo`3`ooooo`03o`000?oooooooooo
01gooooo000=ooooo`03o`000?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo
00?o0000ooooooooool0k_ooool00ol0003oooooooooo`0Oooooo`003Oooool00ol0003ooooooooo
o`3^ooooo`03o`000?oooooooooo01oooooo000=ooooo`03o`000?oooooooooo0>gooooo00?o0000
ooooooooool08?ooool000gooooo00?o0000ooooooooool0k?ooool00ol0003oooooooooo`0Qoooo
o`003Oooool00ol0003oooooooooo`3[ooooo`03o`000?oooooooooo02;ooooo000=ooooo`03o`00
0?oooooooooo0>_ooooo00?o0000ooooooooool08_ooool000gooooo00?o0000ooooooooool0j_oo
ool00ol0003oooooooooo`0Sooooo`003Oooool00ol0003oooooooooo`3Yooooo`03o`000?oooooo
oooo02Cooooo000=ooooo`03o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000go
oooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0Vooooo`003Oooool00ol0003ooooo
ooooo`3Wooooo`03o`000?oooooooooo02Kooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o
0000ooooooooool09oooool000gooooo00?o0000ooooooooool0iOooool00ol0003oooooooooo`0X
ooooo`003Oooool00ol0003oooooooooo`3Tooooo`03o`000?oooooooooo02Wooooo000=ooooo`03
o`000?oooooooooo0>Cooooo00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0
hoooool00ol0003oooooooooo`0Zooooo`003Oooool00ol0003oooooooooo`3Rooooo`03o`000?oo
oooooooo02_ooooo000=ooooo`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool0
00gooooo00?o0000ooooooooool0hOooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003o
ooooooooo`3Pooooo`03o`000?oooooooooo02gooooo000=ooooo`03o`000?oooooooooo0=oooooo
00?o0000ooooooooool0;_ooool000gooooo00?o0000ooooooooool0g_ooool00ol0003ooooooooo
o`0_ooooo`003Oooool00ol0003oooooooooo`3Nooooo`03o`000?oooooooooo02oooooo000=oooo
o`03o`000?oooooooooo0=gooooo00?o0000ooooooooool0<?ooool000gooooo00?o0000oooooooo
ool0g?ooool00ol0003oooooooooo`0aooooo`003Oooool00ol0003oooooooooo`3Kooooo`03o`00
0?oooooooooo03;ooooo000=ooooo`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool0<_oo
ool000gooooo00?o0000ooooooooool0f_ooool00ol0003oooooooooo`0cooooo`003Oooool00ol0
003oooooooooo`3Iooooo`03o`000?oooooooooo03Cooooo000=ooooo`03o`000?oooooooooo0=So
oooo00?o0000ooooooooool0=Oooool000gooooo00?o0000ooooooooool0eoooool00ol0003ooooo
ooooo`0fooooo`003Oooool00ol0003oooooooooo`3Gooooo`03o`000?oooooooooo03Kooooo000=
ooooo`03o`000?oooooooooo0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o0000oooo
ooooool0eOooool00ol0003oooooooooo`0hooooo`003Oooool00ol0003oooooooooo`3Dooooo`03
o`000?oooooooooo03Wooooo000=ooooo`03o`000?oooooooooo0=Cooooo00?o0000ooooooooool0
>Oooool000gooooo00?o0000ooooooooool0doooool00ol0003oooooooooo`0jooooo`003Oooool0
0ol0003oooooooooo`3Booooo`03o`000?oooooooooo03_ooooo000=ooooo`03o`000?oooooooooo
0=7ooooo00?o0000ooooooooool0??ooool000gooooo00?o0000ooooooooool0dOooool00ol0003o
ooooooooo`0looooo`003Oooool00ol0003oooooooooo`3@ooooo`03o`000?oooooooooo03gooooo
000=ooooo`03o`000?oooooooooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000
ooooooooool0c_ooool00ol0003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>oooo
o`03o`000?oooooooooo03oooooo000=ooooo`03o`000?oooooooooo0<gooooo00?o0000oooooooo
ool0@?ooool000gooooo00?o0000ooooooooool0c?ooool00ol0003oooooooooo`11ooooo`003Ooo
ool00ol0003oooooooooo`3;ooooo`03o`000?oooooooooo04;ooooo000=ooooo`03o`000?oooooo
oooo0<_ooooo00?o0000ooooooooool0@_ooool000gooooo00?o0000ooooooooool0b_ooool00ol0
003oooooooooo`13ooooo`003Oooool00ol0003oooooooooo`39ooooo`03o`000?oooooooooo04Co
oooo000=ooooo`03o`000?oooooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o
0000ooooooooool0b?ooool00ol0003oooooooooo`15ooooo`003Oooool00ol0003oooooooooo`37
ooooo`03o`000?oooooooooo04Kooooo000<ooooo`?oZJVYaoooool00ol0003oooooooooo`17oooo
o`002oooool5ojVYZLGooooo00?o0000ooooooooool0B?ooool000[ooooo1onYZJW3ooooo`03o`00
0?oooooooooo04Wooooo000:ooooo`OoZJVY`oooool00ol0003oooooooooo`19ooooo`002_ooool7
ojVYZ@;o0000`?ooool00ol0003oooooooooo`1:ooooo`002oooool5ojVYZ@?ooooo0ol0002loooo
o`03o`000?oooooooooo04_ooooo000<ooooo`?oZJVY1oooool4o`000;Oooooo00?o0000oooooooo
ool0C?ooool001[ooooo0ol0002dooooo`03o`000?oooooooooo04cooooo000Mooooo`?o0000/?oo
ool00ol0003oooooooooo`1=ooooo`008?ooool3o`000:cooooo00?o0000ooooooooool0C_ooool0
02?ooooo1?l0002Wooooo`03o`000?oooooooooo04oooooo000Wooooo`?o0000Y?ooool00ol0003o
ooooooooo`1?ooooo`00:_ooool3o`000:3ooooo00?o0000ooooooooool0D?ooool002gooooo1?l0
002Kooooo`03o`000?oooooooooo057ooooo000aooooo`?o0000Uoooool00ol0003oooooooooo`1B
ooooo`00=?ooool3o`0009Cooooo00?o0000ooooooooool0D_ooool003Oooooo0ol0002@ooooo`03
o`000?oooooooooo05?ooooo000jooooo`Co0000Roooool00ol0003oooooooooo`1Dooooo`00?_oo
ool3o`0008Oooooo00?o0000ooooooooool0EOooool0047ooooo0ol00024ooooo`03o`000?oooooo
oooo05Gooooo0014ooooo`Co0000Ooooool00ol0003oooooooooo`1Fooooo`00B?ooool3o`0007_o
oooo00?o0000ooooooooool0Eoooool004_ooooo0ol0001gooooo`03o`000?oooooooooo05Sooooo
001>ooooo`?o0000Loooool00ol0003oooooooooo`1Iooooo`00DOooool4o`0006oooooo00?o0000
ooooooooool0FOooool005Gooooo0ol0001[ooooo`03o`000?oooooooooo05[ooooo001Hooooo`?o
0000Ioooool00ol0003oooooooooo`1Kooooo`00Foooool4o`0006;ooooo00?o0000ooooooooool0
G?ooool005oooooo0ol0001Oooooo`03o`000?oooooooooo05cooooo001Rooooo`?o0000Foooool0
0ol0003oooooooooo`1Mooooo`00IOooool3o`0005Oooooo00?o0000ooooooooool0G_ooool006So
oooo1?l0001Booooo`03o`000?oooooooooo05oooooo001/ooooo`?o0000Coooool00ol0003ooooo
ooooo`1Oooooo`00Koooool3o`0004_ooooo00?o0000ooooooooool0H?ooool007;ooooo1?l00016
ooooo`03o`000?oooooooooo067ooooo001fooooo`?o0000@_ooool00ol0003oooooooooo`1Roooo
o`00NOooool3o`0003oooooo00?o0000ooooooooool0H_ooool007cooooo0ol0000kooooo`03o`00
0?oooooooooo06?ooooo001oooooo`Co0000=_ooool00ol0003oooooooooo`1Tooooo`00Poooool3
o`0003;ooooo00?o0000ooooooooool0IOooool008Kooooo0ol0000_ooooo`03o`000?oooooooooo
06Gooooo0029ooooo`Co0000:_ooool00ol0003oooooooooo`1Vooooo`00SOooool3o`0002Kooooo
00?o0000ooooooooool0Ioooool0093ooooo0ol0000Rooooo`03o`000?oooooooooo06Sooooo002C
ooooo`?o00007_ooool00ol0003oooooooooo`1Yooooo`00U_ooool4o`0001[ooooo00?o0000oooo
ooooool0JOooool009[ooooo0ol0000Fooooo`03o`000?oooooooooo06[ooooo002Mooooo`?o0000
4_ooool00ol0003oooooooooo`1[ooooo`00X?ooool4o`0000gooooo00?o0000ooooooooool0K?oo
ool00:Cooooo0ol00006ooooo`?o000000?oooooo`000?ooool0KOooool00:Oooooo0ol00002oooo
o`Go0000Koooool00:[ooooo2?l0001^ooooo`00Zoooool7o`0006kooooo002[ooooo`Oo0000K_oo
ool00:cooooo1Ol0001_ooooo`00[Oooool3o`00073ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Step the model forward 10 generations:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Evolve[pd9, Generations \[Rule] 10];\)\)], "Input",
  CellLabel->"In[64]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[pd9]];\)\)], "Input",
  CellLabel->"In[65]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0106104 0.962486 0.0459856 0.919711 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.60667 .95401 m
.04812 .7806 L
s
.60667 .95401 m
.95188 .5 L
s
.04812 .7806 m
.04812 .2194 L
s
.04812 .2194 m
.60667 .04599 L
s
.60667 .04599 m
.95188 .5 L
s
.025 w
.60667 .95401 Mdot
.04812 .7806 Mdot
.04812 .2194 Mdot
.60667 .04599 Mdot
.95188 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[65]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00[Oooool3o`00073ooooo002/ooooo`Go0000Koooool00:_ooooo1ol0001^ooooo`00Zooo
ool7o`0006kooooo002Yooooo`Wo0000K_ooool00:Kooooo0ol00003ooooo`Go0000Koooool00:;o
oooo1?l00007ooooo`Co0000Koooool009oooooo0ol0000?ooooo`03o`000?oooooooooo06cooooo
002Looooo`?o00004_ooool00ol0003oooooooooo`1/ooooo`00VOooool3o`0001Kooooo00?o0000
ooooooooool0Joooool009Gooooo1?l0000Jooooo`03o`000?oooooooooo06[ooooo002Booooo`?o
00007oooool00ol0003oooooooooo`1Yooooo`00Soooool3o`0002;ooooo00?o0000ooooooooool0
JOooool008_ooooo1?l0000Vooooo`03o`000?oooooooooo06Sooooo0028ooooo`?o0000:oooool0
0ol0003oooooooooo`1Wooooo`00QOooool3o`0002oooooo00?o0000ooooooooool0I_ooool008;o
oooo0ol0000cooooo`03o`000?oooooooooo06Gooooo001nooooo`Co0000=_ooool00ol0003ooooo
ooooo`1Uooooo`00Noooool3o`0003_ooooo00?o0000ooooooooool0I?ooool007Sooooo0ol0000o
ooooo`03o`000?oooooooooo06?ooooo001dooooo`Co0000@oooool00ol0003oooooooooo`1Roooo
o`00LOooool3o`0004Oooooo00?o0000ooooooooool0H_ooool006kooooo0ol0001;ooooo`03o`00
0?oooooooooo067ooooo001[ooooo`?o0000Coooool00ol0003oooooooooo`1Pooooo`00Ioooool4
o`0005?ooooo00?o0000ooooooooool0Goooool006Cooooo0ol0001Gooooo`03o`000?oooooooooo
05oooooo001Qooooo`?o0000Foooool00ol0003oooooooooo`1Nooooo`00GOooool4o`0005oooooo
00?o0000ooooooooool0GOooool005[ooooo0ol0001Tooooo`03o`000?oooooooooo05cooooo001G
ooooo`?o0000Ioooool00ol0003oooooooooo`1Looooo`00E?ooool3o`0006_ooooo00?o0000oooo
ooooool0Foooool0053ooooo1?l0001_ooooo`03o`000?oooooooooo05[ooooo001=ooooo`?o0000
M?ooool00ol0003oooooooooo`1Iooooo`00B_ooool3o`0007Oooooo00?o0000ooooooooool0FOoo
ool004Kooooo1?l0001kooooo`03o`000?oooooooooo05Sooooo0013ooooo`?o0000P?ooool00ol0
003oooooooooo`1Gooooo`00@?ooool3o`0008Cooooo00?o0000ooooooooool0E_ooool003gooooo
0ol00028ooooo`03o`000?oooooooooo05Gooooo000iooooo`Co0000Roooool00ol0003ooooooooo
o`1Eooooo`00=_ooool3o`00093ooooo00?o0000ooooooooool0E?ooool003?ooooo0ol0002Doooo
o`03o`000?oooooooooo05?ooooo000_ooooo`Co0000V?ooool00ol0003oooooooooo`1Booooo`00
;?ooool3o`0009cooooo00?o0000ooooooooool0D_ooool002Wooooo0ol0002Pooooo`03o`000?oo
oooooooo057ooooo000Vooooo`?o0000Y?ooool00ol0003oooooooooo`1@ooooo`008_ooool4o`00
0:Sooooo00?o0000ooooooooool0Coooool001oooooo0ol0002/ooooo`03o`000?oooooooooo04oo
oooo000Looooo`?o0000/?ooool00ol0003oooooooooo`1>ooooo`006?ooool4o`000;Cooooo00?o
0000ooooooooool0COooool001Gooooo0ol0002iooooo`03o`000?oooooooooo04cooooo000<oooo
o`?o00000oooool3o`000;cooooo00?o0000ooooooooool0C?ooool000_ooooo1ol00030ooooo`03
o`000?oooooooooo04_ooooo000:ooooo`Oo0000`_ooool00ol0003oooooooooo`1:ooooo`002_oo
ool7o`000<?ooooo00?o0000ooooooooool0BOooool000[ooooo1ol00033ooooo`03o`000?oooooo
oooo04Wooooo000;ooooo`Go0000aOooool00ol0003oooooooooo`18ooooo`003?ooool3o`000<Oo
oooo00?o0000ooooooooool0Aoooool000gooooo00?o0000ooooooooool0aoooool00ol0003ooooo
ooooo`16ooooo`003Oooool00ol0003oooooooooo`38ooooo`03o`000?oooooooooo04Gooooo000=
ooooo`03o`000?oooooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000oooo
ooooool0bOooool00ol0003oooooooooo`14ooooo`003Oooool00ol0003oooooooooo`3:ooooo`03
o`000?oooooooooo04?ooooo000=ooooo`03o`000?oooooooooo0<_ooooo00?o0000ooooooooool0
@_ooool000gooooo00?o0000ooooooooool0boooool00ol0003oooooooooo`12ooooo`003Oooool0
0ol0003oooooooooo`3<ooooo`03o`000?oooooooooo047ooooo000=ooooo`03o`000?oooooooooo
0<gooooo00?o0000ooooooooool0@?ooool000gooooo00?o0000ooooooooool0c_ooool00ol0003o
ooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>ooooo`03o`000?oooooooooo03oooooo
000=ooooo`03o`000?oooooooooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000
ooooooooool0d?ooool00ol0003oooooooooo`0mooooo`003Oooool00ol0003oooooooooo`3Aoooo
o`03o`000?oooooooooo03cooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0d_ooool00ol0003oooooooooo`0kooooo`003Ooo
ool00ol0003oooooooooo`3Cooooo`03o`000?oooooooooo03[ooooo000=ooooo`03o`000?oooooo
oooo0=Cooooo00?o0000ooooooooool0>Oooool000gooooo00?o0000ooooooooool0e?ooool00ol0
003oooooooooo`0iooooo`003Oooool00ol0003oooooooooo`3Eooooo`03o`000?oooooooooo03So
oooo000=ooooo`03o`000?oooooooooo0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o
0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool00ol0003oooooooooo`3G
ooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooooooo0=Sooooo00?o0000oooo
ooooool0=Oooool000gooooo00?o0000ooooooooool0fOooool00ol0003oooooooooo`0dooooo`00
3Oooool00ol0003oooooooooo`3Jooooo`03o`000?oooooooooo03?ooooo000=ooooo`03o`000?oo
oooooooo0=_ooooo00?o0000ooooooooool0<_ooool000gooooo00?o0000ooooooooool0foooool0
0ol0003oooooooooo`0booooo`003Oooool00ol0003oooooooooo`3Looooo`03o`000?oooooooooo
037ooooo000=ooooo`03o`000?oooooooooo0=gooooo00?o0000ooooooooool0<?ooool000gooooo
00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0003ooooooooo
o`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oooooooooo0=oooooo00?o0000
ooooooooool0;_ooool000gooooo00?o0000ooooooooool0h?ooool00ol0003oooooooooo`0]oooo
o`003Oooool00ol0003oooooooooo`3Qooooo`03o`000?oooooooooo02cooooo000=ooooo`03o`00
0?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000ooooooooool0h_oo
ool00ol0003oooooooooo`0[ooooo`003Oooool00ol0003oooooooooo`3Sooooo`03o`000?oooooo
oooo02[ooooo000=ooooo`03o`000?oooooooooo0>Cooooo00?o0000ooooooooool0:Oooool000go
oooo00?o0000ooooooooool0i?ooool00ol0003oooooooooo`0Yooooo`003Oooool00ol0003ooooo
ooooo`3Uooooo`03o`000?oooooooooo02Sooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o
0000ooooooooool09oooool000gooooo00?o0000ooooooooool0ioooool00ol0003oooooooooo`0V
ooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oooooooooo02Kooooo000=ooooo`03
o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000gooooo00?o0000ooooooooool0
jOooool00ol0003oooooooooo`0Tooooo`003Oooool00ol0003oooooooooo`3Zooooo`03o`000?oo
oooooooo02?ooooo000=ooooo`03o`000?oooooooooo0>_ooooo00?o0000ooooooooool08_ooool0
00gooooo00?o0000ooooooooool0joooool00ol0003oooooooooo`0Rooooo`003Oooool00ol0003o
ooooooooo`3/ooooo`03o`000?oooooooooo027ooooo000=ooooo`03o`000?oooooooooo0>gooooo
00?o0000ooooooooool08?ooool000gooooo00?o0000ooooooooool0k_ooool00ol0003ooooooooo
o`0Oooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oooooooooo01oooooo000=oooo
o`03o`000?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo00?o0000oooooooo
ool0l?ooool00ol0003oooooooooo`0Mooooo`003Oooool00ol0003oooooooooo`3aooooo`03o`00
0?oooooooooo01cooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000ooooooooool07?oo
ool000gooooo00?o0000ooooooooool0l_ooool00ol0003oooooooooo`0Kooooo`003Oooool00ol0
003oooooooooo`3cooooo`03o`000?oooooooooo01[ooooo000=ooooo`03o`000?oooooooooo0?Co
oooo00?o0000ooooooooool06Oooool000gooooo00?o0000ooooooooool0m?ooool00ol0003ooooo
ooooo`0Iooooo`003Oooool00ol0003oooooooooo`3eooooo`03o`000?oooooooooo01Sooooo000=
ooooo`03o`000?oooooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o0000oooo
ooooool0moooool00ol0003oooooooooo`0Fooooo`003Oooool00ol0003oooooooooo`3gooooo`03
o`000?oooooooooo01Kooooo000=ooooo`03o`000?oooooooooo0?Sooooo00?o0000ooooooooool0
5Oooool000gooooo00?o0000ooooooooool0nOooool00ol0003oooooooooo`0Dooooo`003Oooool0
0ol0003oooooooooo`3jooooo`03o`000?oooooooooo01?ooooo000=ooooo`03o`000?oooooooooo
0?_ooooo00?o0000ooooooooool04_ooool000gooooo00?o0000ooooooooool0noooool00ol0003o
ooooooooo`0Booooo`003Oooool00ol0003oooooooooo`3looooo`03o`000?oooooooooo017ooooo
000=ooooo`03o`000?oooooooooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000
ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Oooool00ol0003oooooooooo`3noooo
o`03o`000?oooooo000000;o00003Oooool000gooooo00?o0000ooooooooool0ooooool5o`0000co
oooo000=ooooo`03o`000?oooooooooo0?kooooo1ol0000;ooooo`003Oooool00ol0003ooooooooo
o`3nooooo`Oo00002oooool000gooooo00?o0000ooooooooool0o_ooool7o`0000_ooooo000=oooo
o`03o`000?oooooooooo0?oooooo1Ol0000<ooooo`003Oooool00ol0003oooooooooo`3oooooo`Co
00003Oooool000gooooo00?o0000ooooooooool0o_ooool00ol0003oooooooooo`0?ooooo`003Ooo
ool00ol0003oooooooooo`3nooooo`03o`000?oooooooooo00oooooo000=ooooo`03o`000?oooooo
oooo0?gooooo00?o0000ooooooooool04?ooool000gooooo00?o0000ooooooooool0o?ooool00ol0
003oooooooooo`0Aooooo`003Oooool00ol0003oooooooooo`3kooooo`03o`000?oooooooooo01;o
oooo000=ooooo`03o`000?oooooooooo0?_ooooo00?o0000ooooooooool04_ooool000gooooo00?o
0000ooooooooool0n_ooool00ol0003oooooooooo`0Cooooo`003Oooool00ol0003oooooooooo`3i
ooooo`03o`000?oooooooooo01Cooooo000=ooooo`03o`000?oooooooooo0?Sooooo00?o0000oooo
ooooool05Oooool000gooooo00?o0000ooooooooool0moooool00ol0003oooooooooo`0Fooooo`00
3Oooool00ol0003oooooooooo`3gooooo`03o`000?oooooooooo01Kooooo000=ooooo`03o`000?oo
oooooooo0?Kooooo00?o0000ooooooooool05oooool000gooooo00?o0000ooooooooool0mOooool0
0ol0003oooooooooo`0Hooooo`003Oooool00ol0003oooooooooo`3dooooo`03o`000?oooooooooo
01Wooooo000=ooooo`03o`000?oooooooooo0?Cooooo00?o0000ooooooooool06Oooool000gooooo
00?o0000ooooooooool0loooool00ol0003oooooooooo`0Jooooo`003Oooool00ol0003ooooooooo
o`3booooo`03o`000?oooooooooo01_ooooo000=ooooo`03o`000?oooooooooo0?7ooooo00?o0000
ooooooooool07?ooool000gooooo00?o0000ooooooooool0lOooool00ol0003oooooooooo`0Loooo
o`003Oooool00ol0003oooooooooo`3`ooooo`03o`000?oooooooooo01gooooo000=ooooo`03o`00
0?oooooooooo0>oooooo00?o0000ooooooooool07_ooool000gooooo00?o0000ooooooooool0k_oo
ool00ol0003oooooooooo`0Oooooo`003Oooool00ol0003oooooooooo`3^ooooo`03o`000?oooooo
oooo01oooooo000=ooooo`03o`000?oooooooooo0>gooooo00?o0000ooooooooool08?ooool000go
oooo00?o0000ooooooooool0k?ooool00ol0003oooooooooo`0Qooooo`003Oooool00ol0003ooooo
ooooo`3[ooooo`03o`000?oooooooooo02;ooooo000=ooooo`03o`000?oooooooooo0>_ooooo00?o
0000ooooooooool08_ooool000gooooo00?o0000ooooooooool0j_ooool00ol0003oooooooooo`0S
ooooo`003Oooool00ol0003oooooooooo`3Yooooo`03o`000?oooooooooo02Cooooo000=ooooo`03
o`000?oooooooooo0>Sooooo00?o0000ooooooooool09Oooool000gooooo00?o0000ooooooooool0
ioooool00ol0003oooooooooo`0Vooooo`003Oooool00ol0003oooooooooo`3Wooooo`03o`000?oo
oooooooo02Kooooo000=ooooo`03o`000?oooooooooo0>Kooooo00?o0000ooooooooool09oooool0
00gooooo00?o0000ooooooooool0iOooool00ol0003oooooooooo`0Xooooo`003Oooool00ol0003o
ooooooooo`3Tooooo`03o`000?oooooooooo02Wooooo000=ooooo`03o`000?oooooooooo0>Cooooo
00?o0000ooooooooool0:Oooool000gooooo00?o0000ooooooooool0hoooool00ol0003ooooooooo
o`0Zooooo`003Oooool00ol0003oooooooooo`3Rooooo`03o`000?oooooooooo02_ooooo000=oooo
o`03o`000?oooooooooo0>7ooooo00?o0000ooooooooool0;?ooool000gooooo00?o0000oooooooo
ool0hOooool00ol0003oooooooooo`0/ooooo`003Oooool00ol0003oooooooooo`3Pooooo`03o`00
0?oooooooooo02gooooo000=ooooo`03o`000?oooooooooo0=oooooo00?o0000ooooooooool0;_oo
ool000gooooo00?o0000ooooooooool0g_ooool00ol0003oooooooooo`0_ooooo`003Oooool00ol0
003oooooooooo`3Nooooo`03o`000?oooooooooo02oooooo000=ooooo`03o`000?oooooooooo0=go
oooo00?o0000ooooooooool0<?ooool000gooooo00?o0000ooooooooool0g?ooool00ol0003ooooo
ooooo`0aooooo`003Oooool00ol0003oooooooooo`3Kooooo`03o`000?oooooooooo03;ooooo000=
ooooo`03o`000?oooooooooo0=_ooooo00?o0000ooooooooool0<_ooool000gooooo00?o0000oooo
ooooool0f_ooool00ol0003oooooooooo`0cooooo`003Oooool00ol0003oooooooooo`3Iooooo`03
o`000?oooooooooo03Cooooo000=ooooo`03o`000?oooooooooo0=Sooooo00?o0000ooooooooool0
=Oooool000gooooo00?o0000ooooooooool0eoooool00ol0003oooooooooo`0fooooo`003Oooool0
0ol0003oooooooooo`3Gooooo`03o`000?oooooooooo03Kooooo000=ooooo`03o`000?oooooooooo
0=Kooooo00?o0000ooooooooool0=oooool000gooooo00?o0000ooooooooool0eOooool00ol0003o
ooooooooo`0hooooo`003Oooool00ol0003oooooooooo`3Dooooo`03o`000?oooooooooo03Wooooo
000=ooooo`03o`000?oooooooooo0=Cooooo00?o0000ooooooooool0>Oooool000gooooo00?o0000
ooooooooool0doooool00ol0003oooooooooo`0jooooo`003Oooool00ol0003oooooooooo`3Boooo
o`03o`000?oooooooooo03_ooooo000=ooooo`03o`000?oooooooooo0=7ooooo00?o0000oooooooo
ool0??ooool000gooooo00?o0000ooooooooool0dOooool00ol0003oooooooooo`0looooo`003Ooo
ool00ol0003oooooooooo`3@ooooo`03o`000?oooooooooo03gooooo000=ooooo`03o`000?oooooo
oooo0<oooooo00?o0000ooooooooool0?_ooool000gooooo00?o0000ooooooooool0c_ooool00ol0
003oooooooooo`0oooooo`003Oooool00ol0003oooooooooo`3>ooooo`03o`000?oooooooooo03oo
oooo000=ooooo`03o`000?oooooooooo0<gooooo00?o0000ooooooooool0@?ooool000gooooo00?o
0000ooooooooool0c?ooool00ol0003oooooooooo`11ooooo`003Oooool00ol0003oooooooooo`3;
ooooo`03o`000?oooooooooo04;ooooo000=ooooo`03o`000?oooooooooo0<_ooooo00?o0000oooo
ooooool0@_ooool000gooooo00?o0000ooooooooool0b_ooool00ol0003oooooooooo`13ooooo`00
3Oooool00ol0003oooooooooo`39ooooo`03o`000?oooooooooo04Cooooo000=ooooo`03o`000?oo
oooooooo0<Sooooo00?o0000ooooooooool0AOooool000gooooo00?o0000ooooooooool0b?ooool0
0ol0003oooooooooo`15ooooo`003Oooool00ol0003oooooooooo`37ooooo`03o`000?oooooooooo
04Kooooo000<ooooo`?o0000aoooool00ol0003oooooooooo`17ooooo`002oooool5o`000<Gooooo
00?o0000ooooooooool0B?ooool000[ooooo1ol00033ooooo`03o`000?oooooooooo04Wooooo000:
ooooo`Oo0000`oooool00ol0003oooooooooo`19ooooo`002_ooool9o`000<3ooooo00?o0000oooo
ooooool0B_ooool000_ooooo1Ol00003ooooo`?o0000_?ooool00ol0003oooooooooo`1;ooooo`00
3?ooool3o`0000Oooooo1?l0002gooooo`03o`000?oooooooooo04cooooo000Jooooo`?o0000]?oo
ool00ol0003oooooooooo`1<ooooo`007Oooool3o`000;3ooooo00?o0000ooooooooool0COooool0
023ooooo0ol0002/ooooo`03o`000?oooooooooo04kooooo000Sooooo`Co0000Yoooool00ol0003o
ooooooooo`1?ooooo`009oooool3o`000:Cooooo00?o0000ooooooooool0Coooool002[ooooo0ol0
002Pooooo`03o`000?oooooooooo053ooooo000]ooooo`Co0000Voooool00ol0003oooooooooo`1A
ooooo`00<Oooool3o`0009Oooooo00?o0000ooooooooool0D_ooool003Cooooo0ol0002Dooooo`03
o`000?oooooooooo05;ooooo000gooooo`?o0000T?ooool00ol0003oooooooooo`1Cooooo`00>_oo
ool4o`0008_ooooo00?o0000ooooooooool0E?ooool003kooooo0ol00027ooooo`03o`000?oooooo
oooo05Gooooo0011ooooo`?o0000Q?ooool00ol0003oooooooooo`1Eooooo`00A?ooool4o`0007oo
oooo00?o0000ooooooooool0E_ooool004Sooooo0ol0001kooooo`03o`000?oooooooooo05Oooooo
001;ooooo`?o0000Moooool00ol0003oooooooooo`1Hooooo`00C_ooool3o`0007?ooooo00?o0000
ooooooooool0FOooool0057ooooo1?l0001_ooooo`03o`000?oooooooooo05Wooooo001Eooooo`?o
0000Joooool00ol0003oooooooooo`1Jooooo`00F?ooool3o`0006Oooooo00?o0000ooooooooool0
Foooool005_ooooo1?l0001Rooooo`03o`000?oooooooooo05cooooo001Oooooo`?o0000Goooool0
0ol0003oooooooooo`1Looooo`00H_ooool3o`0005_ooooo00?o0000ooooooooool0GOooool006Go
oooo0ol0001Gooooo`03o`000?oooooooooo05kooooo001Xooooo`Co0000D_ooool00ol0003ooooo
ooooo`1Oooooo`00K?ooool3o`0004oooooo00?o0000ooooooooool0Goooool006oooooo0ol0001;
ooooo`03o`000?oooooooooo063ooooo001booooo`Co0000A_ooool00ol0003oooooooooo`1Qoooo
o`00M_ooool3o`0004;ooooo00?o0000ooooooooool0H_ooool007Wooooo0ol0000oooooo`03o`00
0?oooooooooo06;ooooo001looooo`?o0000>oooool00ol0003oooooooooo`1Sooooo`00Ooooool4
o`0003Kooooo00?o0000ooooooooool0I?ooool008?ooooo0ol0000booooo`03o`000?oooooooooo
06Gooooo0026ooooo`?o0000;oooool00ol0003oooooooooo`1Uooooo`00ROooool4o`0002[ooooo
00?o0000ooooooooool0I_ooool008gooooo0ol0000Vooooo`03o`000?oooooooooo06Oooooo002@
ooooo`?o00008_ooool00ol0003oooooooooo`1Xooooo`00Toooool3o`0001kooooo00?o0000oooo
ooooool0JOooool009Kooooo1?l0000Jooooo`03o`000?oooooooooo06Wooooo002Jooooo`?o0000
5_ooool00ol0003oooooooooo`1Zooooo`00WOooool3o`0001;ooooo00?o0000ooooooooool0Jooo
ool00:3ooooo1?l0000=ooooo`03o`000?oooooooooo06cooooo002Tooooo`?o00001_ooool3o`00
0003ooooool0003ooooo06gooooo002Wooooo`?o00000_ooool5o`0006oooooo002Zooooo`So0000
K_ooool00:_ooooo1ol0001^ooooo`00Zoooool7o`0006kooooo002/ooooo`Go0000Koooool00:go
oooo0ol0001`ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQoooo
o`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQoooo
o`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0110188, -0.0500055, \
0.00362016, 0.00378853}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["Here's everyone's data set:", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Data, agentList]\)], "Input",
  CellLabel->"In[66]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({{2.033592957101664`, 
        10, {0.16759719941836765`, 
          0.8324028005816325`}}, {1.9656109911108919`, 
        10, {0.123922254923464`, 0.876077745076536`}}, {3.2198363654018864`, 
        10, {0.1596024432059452`, 0.840397556794055`}}, {1.5604080932460693`, 
        10, {0.15399145361166638`, 
          0.8460085463883336`}}, {0.7409470282820309`, 
        10, {0.03449169921102118`, 0.9655083007889789`}}}\)], "Output",
  CellLabel->"Out[66]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[pd9]\)], "Input",
  CellLabel->"In[67]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
Obviously, in this case it's not so interesting because best \
response dictates people always defect in the Prisoner's Dilemma. Let's try \
another game, though.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Stag Hunt on a lattice with fictitious play", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"stagHunt", "=", 
        RowBox[{"(", GridBox[{
              {\({3, 3}\), \({0, 2}\)},
              {\({2, 0}\), \({2, 2}\)}
              }], ")"}]}], 
      ";"}], "\n", \(stagColors = {\ LightGray, 
          Black};\), "\n", \(StrategyLegend[
        stagColors, {"\<Hunt Stag\>", "\<Hunt Rabbit\>"}];\)}], "Input",
  CellLabel->"In[68]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.666667],
                Background->GrayLevel[0.666667]],
              BoxMargins->False], \(Hunt\ Stag\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0],
                Background->GrayLevel[0]],
              BoxMargins->False], \(Hunt\ Rabbit\)}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[68]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(stag1 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] GridGraph[5, 5], \[IndentingNewLine]PayoffMatrix \[Rule] 
            stagHunt, \[IndentingNewLine]StrategyColors \[Rule] 
            stagColors, \[IndentingNewLine]ShowVertexNumbers \[Rule] 
            True, \[IndentingNewLine]WindowLabel -> "\<Stag hunt on a lattice\
\>"\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Let's change the definition of ",
  StyleBox["Initialize", "Input"],
  " so that the agent's initial belief vector puts all the weight on whatever \
strategy they originally follow."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Initialize[agent_] := 
        Module[{\[Lambda], s}, \[IndentingNewLine]\[Lambda] = 
            Random[Real, {0, 4}]; \[IndentingNewLine]s = 
            Strategy[agent]; \[IndentingNewLine]SetData[
            agent, {\[Lambda], 0, 
              If[s \[Equal] 1, {1.0, 0.0}, {0.0, 
                  1.0}]}];\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(agentList = Agents[stag1];\)\), "\n", 
    \(\(Map[Initialize, agentList];\)\), "\[IndentingNewLine]", 
    \(\(Map[\[IndentingNewLine]SetLearningRule[#, 
            TraditionalFictitiousPlayLR] &, \[IndentingNewLine]agentList\
\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[73]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Verify that a randomly chosen agent is configured \"right\".", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(bob = 
        agentList\[LeftDoubleBracket]12\[RightDoubleBracket];\)\), "\
\[IndentingNewLine]", 
    \(\(Print[Data[bob]];\)\), "\[IndentingNewLine]", 
    \(\(Print[LearningRule[bob]];\)\)}], "Input",
  CellLabel->"In[76]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({3.754186556623273`, 0, {0.`, 1.`}}\)], "Print",
  CellLabel->"From In[76]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(TraditionalFictitiousPlayLR\)], "Print",
  CellLabel->"From In[76]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[TextData[{
  "Here's what the model looks like, both in \"standard\" graph form, and as \
a ",
  StyleBox["RasterArray", "Input"],
  "."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]GraphicsArray[\[IndentingNewLine]{\
\[IndentingNewLine]ShowGraph[
              ExtractGraph[stag1], \[IndentingNewLine]DisplayFunction \[Rule] 
                Identity, \[IndentingNewLine]VertexStyle \[Rule] 
                Disk[Large]], \
\[IndentingNewLine]Graphics[\[IndentingNewLine]RasterArray[
                Partition[ToColorArray[stag1], 5]], 
              AspectRatio \[Rule] 
                Automatic]\[IndentingNewLine]}\[IndentingNewLine]]\
\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[79]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .47619 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0113379 0.47619 [
[ 0 0 0 0 ]
[ 1 .47619 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .47619 L
0 .47619 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0113379 0.477324 0.464853 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
0 0 0 r
.72727 .27273 Mdot
.663 .663 .663 r
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.663 .663 .663 r
.27273 .72727 Mdot
0 0 0 r
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.663 .663 .663 r
.72727 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.0113379 0.97619 0.464853 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.0238095 0.190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
15 string
5 5 8 [5 0 0 5 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000
000000000000000000A9A9A9000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[79]:=",
  ImageSize->{400, 190.438},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006@0000_R000`400?l00000o`00003ooooooonA
ooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonA
ooooo`00oooooonAooooo`00oooooonAooooo`004?ooool4o`0002Gooooo1?l0000Uooooo`CoZJVY
9Oooool4ojVYZBGooooo1?nYZJWHooooo`003_ooool8o`00027ooooo2?l0000Qooooo`SoZJVY8Ooo
ool8ojVYZB7ooooo2?nYZJWFooooo`003Oooool:o`0001oooooo2_l0000Oooooo`[oZJVY7oooool:
ojVYZAoooooo2_nYZJTIooooodGo0000J?nYZJT?ooooo`003Oooool:o`0001oooooo2_l0000Ooooo
o`[oZJVY7oooool:ojVYZAoooooo2_nYZJTIooooodGo0000J?nYZJT?ooooo`003?ooool<o`0001go
oooo3?l0000Mooooo`coZJVY7Oooool<ojVYZAgooooo3?nYZJTHooooodGo0000J?nYZJT?ooooo`00
3?oooomBo`0000coZJVY7Ol0000<ojVYZAgo00003?nYZJTHooooodGo0000J?nYZJT?ooooo`003?oo
ool<o`0001gooooo3?l0000Mooooo`coZJVY7Oooool<ojVYZAgooooo3?nYZJTHooooodGo0000J?nY
ZJT?ooooo`003?ooool<o`0001gooooo3?l0000Mooooo`coZJVY7Oooool<ojVYZAgooooo3?nYZJTH
ooooodGo0000J?nYZJT?ooooo`003Oooool:o`0001oooooo2_l0000Oooooo`[oZJVY7oooool:ojVY
ZAoooooo2_nYZJTIooooodGo0000J?nYZJT?ooooo`003Oooool:o`0001oooooo2_l0000Oooooo`[o
ZJVY7oooool:ojVYZAoooooo2_nYZJTIooooodGo0000J?nYZJT?ooooo`003_ooool8o`00027ooooo
2?l0000Qooooo`SoZJVY8Oooool8ojVYZB7ooooo2?nYZJTJooooodGo0000J?nYZJT?ooooo`004?oo
ool4o`0002Gooooo1?l0000Uooooo`CoZJVY9Oooool4ojVYZBGooooo1?nYZJTLooooodGo0000J?nY
ZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooAOl0001X
ojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooom5o`00
06SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooodGo
0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo
AOl0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooo
oom5o`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0K
ooooodGo0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
01_oooooAOl0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool06ooooom5o`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0KooooodGo0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo01_oooooAOl0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool06ooooom5o`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0KooooodGo0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo01_oooooAOl0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool06ooooom5o`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0KooooodGo0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo01_oooooAOl0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool06ooooom5o`0006SoZJVY3oooool001;ooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0KooooodGo0000J?nYZJT?ooooo`004_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo01_oooooAOl0001XojVYZ@oooooo000Booooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool06ooooom5o`0006SoZJVY3oooool001;ooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0KooooodGo0000J?nYZJT?ooooo`004_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooAOl0001XojVYZ@oooooo000Booooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooom5o`0006SoZJVY3oooool001;ooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooodGo0000J?nYZJT?ooooo`004_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooJ?nYZJTSo`0002;oZJVY3ooo
ool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooofSoZJVY8ol0000R
ojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXojVY
ZB?o00008_nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_o
ooooJ?nYZJTSo`0002;oZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0KooooofSoZJVY8ol0000RojVYZ@oooooo000@ooooo`CoZJVY9Oooool4ojVYZBGooooo1?nY
ZJTUooooo`Co00009Oooool4ojVYZAcoooooJ?nYZJTSo`0002;oZJVY3oooool000kooooo2?nYZJTQ
ooooo`SoZJVY8Oooool8ojVYZB7ooooo2?l0000Qooooo`SoZJVY6_oooomXojVYZB?o00008_nYZJT?
ooooo`003Oooool:ojVYZAoooooo2_nYZJTOooooo`[oZJVY7oooool:o`0001oooooo2_nYZJTIoooo
ofSoZJVY8ol0000RojVYZ@oooooo000=ooooo`[oZJVY7oooool:ojVYZAoooooo2_nYZJTOooooo`[o
00007oooool:ojVYZAWoooooJ?nYZJTSo`0002;oZJVY3oooool000cooooo3?nYZJTMooooo`coZJVY
7Oooool<ojVYZAgooooo3?l0000Mooooo`coZJVY6?oooomXojVYZB?o00008_nYZJT?ooooo`003?oo
ool<ojVYZAgo00003?nYZJTMo`0000coZJVYA_l0000<ojVYZASoooooJ?nYZJTSo`0002;oZJVY3ooo
ool000cooooo3?nYZJTMooooo`coZJVY7Oooool<ojVYZAgooooo3?l0000Mooooo`coZJVY6?oooomX
ojVYZB?o00008_nYZJT?ooooo`003?ooool<ojVYZAgooooo3?nYZJTMooooo`coZJVY7Oooool<o`00
01gooooo3?nYZJTHooooofSoZJVY8ol0000RojVYZ@oooooo000=ooooo`[oZJVY7oooool:ojVYZAoo
oooo2_nYZJTOooooo`[o00007oooool:ojVYZAWoooooJ?nYZJTSo`0002;oZJVY3oooool000gooooo
2_nYZJTOooooo`[oZJVY7oooool:ojVYZAoooooo2_l0000Oooooo`[oZJVY6OoooomXojVYZB?o0000
8_nYZJT?ooooo`003_ooool8ojVYZB7ooooo2?nYZJTQooooo`SoZJVY8Oooool8o`00027ooooo2?nY
ZJTJooooofSoZJVY8ol0000RojVYZ@oooooo000@ooooo`CoZJVY9Oooool4ojVYZBGooooo1?nYZJTU
ooooo`Co00009Oooool4ojVYZAcoooooJ?nYZJTSo`0002;oZJVY3oooool001;ooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0KooooofSoZJVY8ol0000RojVYZ@oooooo000Booooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXojVYZB?o00008_nYZJT?ooooo`00
4_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooJ?nYZJTSo`0002;oZJVY
3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooofSoZJVY8ol0
000RojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomX
ojVYZB?o00008_nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
01_oooooJ?nYZJTSo`0002;oZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0KooooofSoZJVY8ol0000RojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool06ooooomXojVYZB?o00008_nYZJT?ooooo`004_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo01_oooooJ?nYZJTSo`0002;oZJVY3oooool001;ooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0KooooofSoZJVY8ol0000RojVYZ@oooooo000Boooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXojVYZB?o00008_nYZJT?oooo
o`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooJ?nYZJTSo`0002;o
ZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooofSoZJVY
8ol0000RojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooo
oomXojVYZB?o00008_nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo01_oooooJ?nYZJTSo`0002;oZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0KooooofSoZJVY8ol0000RojVYZ@oooooo000Booooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool06ooooomXojVYZB?o00008_nYZJT?ooooo`004_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo01_ooooo8onYZJTRo`0006SoZJVY3oooool001;ooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?oZJVY8_l0001XojVYZ@oooooo000B
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooolSojVYZB;o0000J?nYZJT?
ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8onYZJTRo`00
06SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?o
ZJVY8_l0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
6ooooolSojVYZB;o0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo01_ooooo8onYZJTRo`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Kooooob?oZJVY8_l0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool06ooooolSojVYZB;o0000J?nYZJT?ooooo`004_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8onYZJTRo`0006SoZJVY3oooool001;ooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?oZJVY8_l0001XojVYZ@oooooo
000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooolSojVYZB;o0000J?nY
ZJT?ooooo`004?ooool4ojVYZBGooooo1?l0000Uooooo`CoZJVY9Oooool4ojVYZBGooooo1?nYZJTL
ooooob?oZJVY8_l0001XojVYZ@oooooo000>ooooo`SoZJVY8Oooool8o`00027ooooo2?nYZJTQoooo
o`SoZJVY8Oooool8ojVYZA[ooooo8onYZJTRo`0006SoZJVY3oooool000gooooo2_nYZJTOooooo`[o
00007oooool:ojVYZAoooooo2_nYZJTOooooo`[oZJVY6OoooolSojVYZB;o0000J?nYZJT?ooooo`00
3Oooool:ojVYZAoooooo2_l0000Oooooo`[oZJVY7oooool:ojVYZAoooooo2_nYZJTIooooob?oZJVY
8_l0001XojVYZ@oooooo000<ooooo`coZJVY7Oooool<o`0001gooooo3?nYZJTMooooo`coZJVY7Ooo
ool<ojVYZASooooo8onYZJTRo`0006SoZJVY3oooool000cooooo3?nYZJU6o`0000coZJVY7Ol0000<
ojVYZAgo00003?nYZJTHooooob?oZJVY8_l0001XojVYZ@oooooo000<ooooo`coZJVY7Oooool<o`00
01gooooo3?nYZJTMooooo`coZJVY7Oooool<ojVYZASooooo8onYZJTRo`0006SoZJVY3oooool000co
oooo3?nYZJTMooooo`co00007Oooool<ojVYZAgooooo3?nYZJTMooooo`coZJVY6?oooolSojVYZB;o
0000J?nYZJT?ooooo`003Oooool:ojVYZAoooooo2_l0000Oooooo`[oZJVY7oooool:ojVYZAoooooo
2_nYZJTIooooob?oZJVY8_l0001XojVYZ@oooooo000=ooooo`[oZJVY7oooool:o`0001oooooo2_nY
ZJTOooooo`[oZJVY7oooool:ojVYZAWooooo8onYZJTRo`0006SoZJVY3oooool000kooooo2?nYZJTQ
ooooo`So00008Oooool8ojVYZB7ooooo2?nYZJTQooooo`SoZJVY6_oooolSojVYZB;o0000J?nYZJT?
ooooo`004?ooool4ojVYZBGooooo1?l0000Uooooo`CoZJVY9Oooool4ojVYZBGooooo1?nYZJTLoooo
ob?oZJVY8_l0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool06ooooolSojVYZB;o0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`00
0?oooooooooo01_ooooo8onYZJTRo`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Kooooob?oZJVY8_l0001XojVYZ@oooooo000Booooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool06ooooolSojVYZB;o0000J?nYZJT?ooooo`004_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8onYZJTRo`0006SoZJVY3oooool001;o
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?oZJVY8_l0001XojVYZ@oo
oooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooolSojVYZB;o0000
J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8onY
ZJTRo`0006SoZJVY3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0K
ooooob?oZJVY8_l0001XojVYZ@oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool06ooooolSojVYZB;o0000J?nYZJT?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo01_ooooo8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000Booooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool06ooooolSo`0002;oZJVY8ol0000SojVYZB;o0000
3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?o00008_nY
ZJTSo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo01_ooooo8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool06ooooolSo`0002;oZJVY8ol0000SojVYZB;o00003oooool001;o
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?o00008_nYZJTSo`0002?o
ZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo
8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool06ooooolSo`0002;oZJVY8ol0000SojVYZB;o00003oooool001;ooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Kooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?
ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8ol0000RojVY
ZB?o00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool06ooooolSo`0002;oZJVY8ol0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Kooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`004_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8ol0000RojVYZB?o00008onY
ZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooolS
o`0002;oZJVY8ol0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Kooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8ol0000RojVYZB?o00008onYZJTRo`0000oo
oooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooolSo`0002;oZJVY
8ol0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Kooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo01_ooooo8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000@oooo
o`Co00009Oooool4ojVYZBGooooo1?l0000Uooooo`CoZJVY9Oooool4o`0001cooooo8ol0000RojVY
ZB?o00008onYZJTRo`0000oooooo000>ooooo`So00008Oooool8ojVYZB7ooooo2?l0000Qooooo`So
ZJVY8Oooool8o`0001[ooooo8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000=ooooo`[o0000
7oooool:ojVYZAoooooo2_l0000Oooooo`[oZJVY7oooool:o`0001Wooooo8ol0000RojVYZB?o0000
8onYZJTRo`0000oooooo000=ooooo`[o00007oooool:ojVYZAoooooo2_l0000Oooooo`[oZJVY7ooo
ool:o`0001Wooooo8ol0000RojVYZB?o00008onYZJTRo`0000oooooo000<ooooo`co00007Oooool<
ojVYZAgooooo3?l0000Mooooo`coZJVY7Oooool<o`0001Sooooo8ol0000RojVYZB?o00008onYZJTR
o`0000oooooo000<ooooobWo00003?nYZJU6o`0000coZJVY:Ol0000Hooooob?o00008_nYZJTSo`00
02?oZJVY8_l0000?ooooo`003?ooool<o`0001gooooo3?nYZJTMooooo`co00007Oooool<ojVYZAgo
oooo3?l0000Hooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`003?ooool<o`0001gooooo
3?nYZJTMooooo`co00007Oooool<ojVYZAgooooo3?l0000Hooooob?o00008_nYZJTSo`0002?oZJVY
8_l0000?ooooo`003Oooool:o`0001oooooo2_nYZJTOooooo`[o00007oooool:ojVYZAoooooo2_l0
000Iooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`003Oooool:o`0001oooooo2_nYZJTO
ooooo`[o00007oooool:ojVYZAoooooo2_l0000Iooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?
ooooo`003_ooool8o`00027ooooo2?nYZJTQooooo`So00008Oooool8ojVYZB7ooooo2?l0000Joooo
ob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`004?ooool4o`0002Gooooo1?nYZJTUooooo`Co
00009Oooool4ojVYZBGooooo1?l0000Looooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`00
4_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8ol0000RojVYZB?o0000
8onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooo
oolSo`0002;oZJVY8ol0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Kooooob?o00008_nYZJTSo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo8ol0000RojVYZB?o00008onYZJTRo`00
00oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXo`0002?o
ZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_ooooo
J?l0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0KooooofSo00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool06ooooomXo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo01_oooooJ?l0000SojVYZB;o00003oooool001;ooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0KooooofSo00008onYZJTRo`0000oooooo000Booooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXo`0002?oZJVY8_l0000?ooooo`004_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooJ?l0000SojVYZB;o00003ooo
ool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooo
oooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooofSo00008onYZJTR
o`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0
003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXo`00
02?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_o
ooooJ?l0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`0KooooofSo00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o
0000ooooooooool06ooooomXo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0V
ooooo`03o`000?oooooooooo01_oooooJ?l0000SojVYZB;o00003oooool001;ooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`0KooooofSo00008onYZJTRo`0000oooooo000Booooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXo`0002?oZJVY8_l0000?ooooo`00
4_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0
9_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo01_oooooJ?l0000SojVYZB;o0000
3oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0KooooofSo00008onY
ZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomX
o`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo
01_oooooJ?l0000SojVYZB;o00003oooool001;ooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003o
ooooooooo`0KooooofSo00008onYZJTRo`0000oooooo000Booooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo
00?o0000ooooooooool06ooooomXo`0002?oZJVY8_l0000?ooooo`004_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo01_oooooJ?l0000SojVYZB;o00003oooool001;ooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0KooooofSo00008onYZJTRo`0000oooooo000Boooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool06ooooomXo`0002?oZJVY8_l0000?oooo
o`004?ooool4o`0002Gooooo1?l0000Uooooo`Co00009Oooool4ojVYZBGooooo1?l0000LooooofSo
00008onYZJTRo`0000oooooo000>ooooo`So00008Oooool8o`00027ooooo2?l0000Qooooo`SoZJVY
8Oooool8o`0001[oooooJ?l0000SojVYZB;o00003oooool000gooooo2_l0000Oooooo`[o00007ooo
ool:o`0001oooooo2_nYZJTOooooo`[o00006OoooomXo`0002?oZJVY8_l0000?ooooo`003Oooool:
o`0001oooooo2_l0000Oooooo`[o00007oooool:ojVYZAoooooo2_l0000IooooofSo00008onYZJTR
o`0000oooooo000<ooooo`co00007Oooool<o`0001gooooo3?l0000Mooooo`coZJVY7Oooool<o`00
01SoooooJ?l0000SojVYZB;o00003oooool000coooooNol0000<ojVYZBWo00006?oooomXo`0002?o
ZJVY8_l0000?ooooo`003?ooool<o`0001gooooo3?l0000Mooooo`co00007Oooool<ojVYZAgooooo
3?l0000HooooofSo00008onYZJTRo`0000oooooo000<ooooo`co00007Oooool<o`0001gooooo3?l0
000Mooooo`coZJVY7Oooool<o`0001SoooooJ?l0000SojVYZB;o00003oooool000gooooo2_l0000O
ooooo`[o00007oooool:o`0001oooooo2_nYZJTOooooo`[o00006OoooomXo`0002?oZJVY8_l0000?
ooooo`003Oooool:o`0001oooooo2_l0000Oooooo`[o00007oooool:ojVYZAoooooo2_l0003Eoooo
o`003_ooool8o`00027ooooo2?l0000Qooooo`So00008Oooool8ojVYZB7ooooo2?l0003Fooooo`00
4?ooool4o`0002Gooooo1?l0000Uooooo`Co00009Oooool4ojVYZBGooooo1?l0003Hooooo`00oooo
oonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooo
oonAooooo`00\
\>"],
  ImageRangeCache->{{{0, 399}, {189.438, 0}} -> {-0.0531273, -0.0238146, \
0.00527884, 0.00527884}, {{10.0625, 190.438}, {184.875, 4.5}} -> {-0.111371, \
-0.0778297, 0.00609847, 0.00609847}, {{208.5, 388.938}, {184.875, 4.5}} -> \
{-6.19461, -0.257824, 0.0291063, 0.0291063}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[stag1]]; \[IndentingNewLine]Evolve[
          stag1], {10}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[80]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[
        Partition[l, 3], \[IndentingNewLine]VertexStyle \[Rule] 
          Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[82]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0238095 0.31746 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.321429 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
0 0 0 r
.04545 .5 Mdot
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.0238095 0.64881 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .27273 Mdot
.27273 .27273 Mdot
.663 .663 .663 r
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
0 0 0 r
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.663 .663 .663 r
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.0238095 0.97619 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .27273 Mdot
.27273 .27273 Mdot
.663 .663 .663 r
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
0 0 0 r
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.663 .663 .663 r
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.35119 0.321429 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.35119 0.64881 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.35119 0.97619 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
0 0 0 r
.04545 .5 Mdot
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.678571 0.321429 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
0 0 0 r
.72727 .27273 Mdot
.663 .663 .663 r
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.663 .663 .663 r
.27273 .72727 Mdot
0 0 0 r
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.663 .663 .663 r
.72727 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.678571 0.64881 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
0 0 0 r
.72727 .27273 Mdot
.663 .663 .663 r
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.663 .663 .663 r
.27273 .72727 Mdot
0 0 0 r
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.663 .663 .663 r
.72727 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.678571 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.663 .663 .663 r
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.663 .663 .663 r
.72727 .72727 Mdot
0 0 0 r
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.663 .663 .663 r
.72727 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[82]:=",
  ImageSize->{403, 403},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006C0001Tb000`400?l00000o`00003ooooooonD
ooooo`00oooooonDooooo`00oooooonDooooo`00oooooonDooooo`00oooooonDooooo`00oooooonD
ooooo`00oooooonDooooo`00oooooonDooooo`00oooooonDooooo`00oooooonDooooo`00oooooonD
ooooo`00oooooonDooooo`003Oooool4o`0001Oooooo1?l0000Gooooo`CoZJVY5oooool4ojVYZASo
oooo1?nYZJTBooooo`Co00006?ooool4o`0001Oooooo1?nYZJTGooooo`CoZJVY5oooool4ojVYZA?o
oooo1?l0000Gooooo`Co00005oooool4ojVYZASooooo1?nYZJTGooooo`CoZJVY3_ooool000cooooo
1_l0000Eooooo`Ko00005Oooool6ojVYZAGooooo1_nYZJTFooooo`KoZJVY4?ooool6o`0001Kooooo
1_l0000Eooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTAooooo`Ko00005Oooool6o`0001Gooooo
1_nYZJTFooooo`KoZJVY5Oooool6ojVYZ@gooooo000;ooooo`So00004oooool8o`0001?ooooo2?nY
ZJTCooooo`SoZJVY5?ooool8ojVYZ@kooooo2?l0000Dooooo`So00004oooool8ojVYZA?ooooo2?nY
ZJTCooooo`SoZJVY3oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nY
ZJT<ooooo`002oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>
ooooo`So00005?ooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oooooo2?l0000C
ooooo`So00004oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool000_ooooo=_l00008ojVY
ZA?o00002?nYZJTDo`0000SoZJVY3_oooolgo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJT?oooo
ocKo00002?nYZJTDo`0000SoZJVY4ol00008ojVYZ@cooooo000;ooooo`So00004oooool8o`0001?o
oooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZ@kooooo2?l0000Dooooo`So00004oooool8ojVYZA?o
oooo2?nYZJTCooooo`SoZJVY3oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY5?ooool8ojVYZA?o
oooo2?nYZJT<ooooo`003?ooool6o`0001Gooooo1_l0000Eooooo`KoZJVY5Oooool6ojVYZAKooooo
1_nYZJT@ooooo`Ko00005_ooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5Oooool6ojVYZA7ooooo
1_l0000Eooooo`Ko00005Oooool6ojVYZAKooooo1_nYZJTEooooo`KoZJVY3Oooool000gooooo1?l0
000Gooooo`Co00005oooool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY4_ooool4o`0001Sooooo1?l0
000Gooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTCooooo`Co00005oooool4o`0001Oooooo1?nY
ZJTHooooo`CoZJVY5oooool4ojVYZ@kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003Oooool4ojVYZAOooooo1?nY
ZJTGooooo`CoZJVY5oooool4ojVYZASooooo1?nYZJTBooooo`Co00006?ooool4o`0001Oooooo1?nY
ZJTGooooo`CoZJVY5oooool4ojVYZA?ooooo1?l0000Gooooo`Co00005oooool4ojVYZASooooo1?nY
ZJTGooooo`CoZJVY3_ooool000cooooo1_nYZJTEooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTF
ooooo`KoZJVY4?ooool6o`0001Kooooo1_l0000Eooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTA
ooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTFooooo`KoZJVY5Oooool6ojVYZ@gooooo000;oooo
o`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZ@kooooo2?l0000Doooo
o`So00004oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY3oooool8o`0001?ooooo2?l0000Coooo
o`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<ooooo`002oooool8ojVYZA?ooooo2?nYZJTCooooo`So
ZJVY4oooool8ojVYZACooooo2?nYZJT>ooooo`So00005?ooool8o`0001?ooooo2?nYZJTCooooo`So
ZJVY4oooool8ojVYZ@oooooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?nYZJTCooooo`So
ZJVY3?ooool000_ooooo2?nYZJTCo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJTDo`0000SoZJVY
3_oooolgo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJT?ooooocKo00002?nYZJTDo`0000SoZJVY
4ol00008ojVYZ@cooooo000;ooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY5?oo
ool8ojVYZ@kooooo2?l0000Dooooo`So00004oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY3ooo
ool8o`0001?ooooo2?l0000Cooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<ooooo`003?ooool6
ojVYZAGooooo1_nYZJTEooooo`KoZJVY5Oooool6ojVYZAKooooo1_nYZJT@ooooo`Ko00005_ooool6
o`0001Gooooo1_nYZJTEooooo`KoZJVY5Oooool6ojVYZA7ooooo1_l0000Eooooo`Ko00005Oooool6
ojVYZAKooooo1_nYZJTEooooo`KoZJVY3Oooool000gooooo1?nYZJTGooooo`CoZJVY5oooool4ojVY
ZAOooooo1?nYZJTHooooo`CoZJVY4_ooool4o`0001Sooooo1?l0000Gooooo`CoZJVY5oooool4ojVY
ZAOooooo1?nYZJTCooooo`Co00005oooool4o`0001Oooooo1?nYZJTHooooo`CoZJVY5oooool4ojVY
Z@kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000gooooo1?l0000Gooooo`Co00005oooool4ojVYZAOo
oooo1?nYZJTHooooo`CoZJVY4_ooool4o`0001Sooooo1?l0000Gooooo`Co00005oooool4ojVYZAOo
oooo1?nYZJTCooooo`Co00005oooool4o`0001Oooooo1?l0000Hooooo`CoZJVY5oooool4ojVYZ@ko
oooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZA3ooooo
1_l0000Fooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY4Oooool6o`0001Gooooo
1_l0000Eooooo`Ko00005_ooool6ojVYZAGooooo1_nYZJT=ooooo`002oooool8o`0001?ooooo2?l0
000Cooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>ooooo`So00005?ooool8o`0001?ooooo2?l0
000Cooooo`SoZJVY4oooool8ojVYZ@oooooo2?l0000Cooooo`So00004oooool8o`0001Cooooo2?nY
ZJTCooooo`SoZJVY3?ooool000_ooooo=_l00008ojVYZA?o00002?nYZJTDo`0000SoZJVY3_oooomB
o`0000SoZJVY4ol00008ojVYZ@ooooooD_l00008ojVYZA?o00002?nYZJT<ooooo`002oooool8o`00
01?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>ooooo`So00005?ooool8o`00
01?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZ@oooooo2?l0000Cooooo`So00004oooool8o`00
01Cooooo2?nYZJTCooooo`SoZJVY3?ooool000_ooooo2?l0000Cooooo`So00004oooool8ojVYZA?o
oooo2?nYZJTDooooo`SoZJVY3_ooool8o`0001Cooooo2?l0000Cooooo`So00004oooool8ojVYZA?o
oooo2?nYZJT?ooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`SoZJVY4oooool8ojVYZ@co
oooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZA3ooooo
1_l0000Fooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY4Oooool6o`0001Gooooo
1_l0000Eooooo`Ko00005_ooool6ojVYZAGooooo1_nYZJT=ooooo`003Oooool4o`0001Oooooo1?l0
000Gooooo`CoZJVY5oooool4ojVYZASooooo1?nYZJTBooooo`Co00006?ooool4o`0001Oooooo1?l0
000Gooooo`CoZJVY5oooool4ojVYZA?ooooo1?l0000Gooooo`Co00005oooool4o`0001Sooooo1?nY
ZJTGooooo`CoZJVY3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool0
00oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000=ooooo`Co00005oooool4o`0001Oooooo1?l0
000Gooooo`Co00006?ooool4o`0001;ooooo1?l0000Hooooo`Co00005oooool4o`0001Oooooo1?l0
000Gooooo`Co00004oooool4o`0001Oooooo1?l0000Gooooo`Co00006?ooool4o`0001Oooooo1?l0
000>ooooo`003?ooool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`0001Kooooo1_l0000@
ooooo`Ko00005_ooool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`00017ooooo1_l0000E
ooooo`Ko00005Oooool6o`0001Kooooo1_l0000Eooooo`Ko00003Oooool000_ooooo2?l0000Coooo
o`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00003_ooool8o`0001Cooooo2?l0000Coooo
o`So00004oooool8o`0001?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Doooo
o`So00004oooool8o`0000cooooo000;ooooogGo00003_oooomeo`0000ooooooMOl0000<ooooo`00
2oooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8o`0001Cooooo2?l0000>ooooo`So0000
5?ooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8o`0000oooooo2?l0000Cooooo`So0000
4oooool8o`0001Cooooo2?l0000Cooooo`So00003?ooool000_ooooo2?l0000Cooooo`So00004ooo
ool8o`0001?ooooo2?l0000Dooooo`So00003_ooool8o`0001Cooooo2?l0000Cooooo`So00004ooo
ool8o`0001?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00004ooo
ool8o`0000cooooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6
o`00013ooooo1_l0000Fooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko00004Oooool6
o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6o`0001Gooooo1_l0000=ooooo`003Oooool4o`00
01Oooooo1?l0000Gooooo`Co00005oooool4o`0001Sooooo1?l0000Booooo`Co00006?ooool4o`00
01Oooooo1?l0000Gooooo`Co00005oooool4o`0001?ooooo1?l0000Gooooo`Co00005oooool4o`00
01Sooooo1?l0000Gooooo`Co00003_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000=ooooo`Co00005oooool4o`00
01Oooooo1?l0000Gooooo`Co00006?ooool4o`0001;ooooo1?l0000Hooooo`Co00005oooool4o`00
01Oooooo1?l0000Gooooo`Co00004oooool4o`0001Oooooo1?l0000Gooooo`Co00006?ooool4o`00
01Oooooo1?l0000>ooooo`003?ooool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`0001Ko
oooo1_l0000@ooooo`Ko00005_ooool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`00017o
oooo1_l0000Eooooo`Ko00005Oooool6o`0001Kooooo1_l0000Eooooo`Ko00003Oooool000_ooooo
2?l0000Cooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00003_ooool8o`0001Cooooo
2?l0000Cooooo`So00004oooool8o`0001?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo
2?l0000Dooooo`So00004oooool8o`0000cooooo000;ooooogGo00003_oooomeo`0000ooooooMOl0
000<ooooo`002oooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8o`0001Cooooo2?l0000>
ooooo`So00005?ooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8o`0000oooooo2?l0000C
ooooo`So00004oooool8o`0001Cooooo2?l0000Cooooo`So00003?ooool000_ooooo2?l0000Coooo
o`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00003_ooool8o`0001Cooooo2?l0000Coooo
o`So00004oooool8o`0001?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Doooo
o`So00004oooool8o`0000cooooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko
00005_ooool6o`00013ooooo1_l0000Fooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko
00004Oooool6o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6o`0001Gooooo1_l0000=ooooo`00
3Oooool4o`0001Oooooo1?l0000Gooooo`Co00005oooool4o`0001Sooooo1?l0000Booooo`Co0000
6?ooool4o`0001Oooooo1?l0000Gooooo`Co00005oooool4o`0001?ooooo1?l0000Gooooo`Co0000
5oooool4o`0001Sooooo1?l0000Gooooo`Co00003_ooool00?ooooooU?ooool00?ooooooU?ooool0
0?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool0
0?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool0
0?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool000gooooo1?l0000Gooooo`Co00005ooo
ool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY4_ooool4o`0001Sooooo1?l0000Gooooo`CoZJVY5ooo
ool4ojVYZAOooooo1?nYZJTCooooo`Co00005oooool4o`0001Oooooo1?nYZJTHooooo`CoZJVY5ooo
ool4ojVYZ@kooooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5_ooool6
ojVYZA3ooooo1_l0000Fooooo`Ko00005Oooool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY4Oooool6
o`0001Gooooo1_l0000Eooooo`KoZJVY5_ooool6ojVYZAGooooo1_nYZJT=ooooo`002oooool8o`00
01?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>ooooo`So00005?ooool8o`00
01?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oooooo2?l0000Cooooo`So00004oooool8ojVY
ZACooooo2?nYZJTCooooo`SoZJVY3?ooool000_ooooo=_l00008ojVYZA?o00002?nYZJTDo`0000So
ZJVY3_oooolgo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJT?ooooocKo00002?nYZJTDo`0000So
ZJVY4ol00008ojVYZ@cooooo000;ooooo`So00004oooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY
5?ooool8ojVYZ@kooooo2?l0000Dooooo`So00004oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY
3oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<ooooo`002ooo
ool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>ooooo`So00005?oo
ool8o`0001?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oooooo2?l0000Cooooo`So00004ooo
ool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool000cooooo1_l0000Eooooo`Ko00005Oooool6
ojVYZAGooooo1_nYZJTFooooo`KoZJVY4?ooool6o`0001Kooooo1_l0000Eooooo`KoZJVY5Oooool6
ojVYZAGooooo1_nYZJTAooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTFooooo`KoZJVY5Oooool6
ojVYZ@gooooo000=ooooo`Co00005oooool4o`0001Oooooo1?nYZJTGooooo`CoZJVY6?ooool4ojVY
ZA;ooooo1?l0000Hooooo`Co00005oooool4ojVYZAOooooo1?nYZJTGooooo`CoZJVY4oooool4o`00
01Oooooo1?l0000Gooooo`CoZJVY6?ooool4ojVYZAOooooo1?nYZJT>ooooo`003oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo
000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00ko
oooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool0
00gooooo1?nYZJTGooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY4_ooool4ojVY
ZASooooo1?nYZJTGooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTCooooo`CoZJVY5oooool4ojVY
ZAOooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZ@kooooo000<ooooo`KoZJVY5Oooool6ojVYZAGo
oooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZA3ooooo1_nYZJTFooooo`KoZJVY5Oooool6ojVYZAGo
oooo1_nYZJTEooooo`KoZJVY4Oooool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZAGo
oooo1_nYZJT=ooooo`002oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZACooooo
2?nYZJT>ooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oooooo
2?nYZJTCooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool000_ooooo2?nY
ZJTCo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJTDo`0000SoZJVY3_ooool8ojVYZACo00002?nY
ZJTCo`0000SoZJVY4ol00008ojVYZA?o00002?nYZJT?ooooo`SoZJVY4ol00008ojVYZA?o00002?nY
ZJTDo`0000SoZJVY4ol00008ojVYZ@cooooo000;ooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTC
ooooo`SoZJVY5?ooool8ojVYZ@kooooo2?nYZJTDooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTC
ooooo`SoZJVY3oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<
ooooo`002oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZACooooo2?nYZJT>oooo
o`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oooooo2?nYZJTCoooo
o`SoZJVY4oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool000cooooo1_nYZJTEooooo`Ko
ZJVY5Oooool6ojVYZAGooooo1_nYZJTFooooo`KoZJVY4?ooool6ojVYZAKooooo1_nYZJTEooooo`Ko
ZJVY5Oooool6ojVYZAGooooo1_nYZJTAooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTFooooo`Ko
ZJVY5Oooool6ojVYZ@gooooo000=ooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTGooooo`CoZJVY
6?ooool4ojVYZA;ooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTGooooo`CoZJVY
4oooool4ojVYZAOooooo1?nYZJTGooooo`CoZJVY6?ooool4ojVYZAOooooo1?nYZJT>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool03_ooool000gooooo1?nYZJTGooooo`Co00005oooool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY
4_ooool4ojVYZASooooo1?l0000Gooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTCooooo`Co0000
5oooool4o`0001Oooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZ@kooooo000<ooooo`KoZJVY5Ooo
ool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZA3ooooo1_nYZJTFooooo`Ko00005Ooo
ool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY4Oooool6o`0001Gooooo1_l0000Eooooo`KoZJVY5_oo
ool6ojVYZAGooooo1_nYZJT=ooooo`002oooool8ojVYZA?ooooo2?l0000Cooooo`SoZJVY4oooool8
ojVYZACooooo2?nYZJT>ooooo`SoZJVY5?ooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY4oooool8
ojVYZ@oooooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool0
00_ooooo2?nYZJTCooooo`So00004oooool8ojVYZA?ooooo2?nYZJTDooooo`SoZJVY3_ooool8ojVY
ZACooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJT?ooooo`So00004oooool8o`00
01?ooooo2?nYZJTDooooo`SoZJVY4oooool8ojVYZ@cooooo000;ooooo`SoZJVY;_l00008ojVYZA?o
00002?nYZJTDo`0000SoZJVY3_ooool8ojVYZBoo00002?nYZJTCo`0000SoZJVY4ol00008ojVYZ@oo
oooo=_l00008ojVYZACo00002?nYZJTCo`0000SoZJVY3?ooool000_ooooo2?nYZJTCooooo`So0000
4oooool8ojVYZA?ooooo2?nYZJTDooooo`SoZJVY3_ooool8ojVYZACooooo2?l0000Cooooo`SoZJVY
4oooool8ojVYZA?ooooo2?nYZJT?ooooo`So00004oooool8o`0001?ooooo2?nYZJTDooooo`SoZJVY
4oooool8ojVYZ@cooooo000<ooooo`KoZJVY5Oooool6o`0001Gooooo1_nYZJTEooooo`KoZJVY5_oo
ool6ojVYZA3ooooo1_nYZJTFooooo`Ko00005Oooool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY4Ooo
ool6o`0001Gooooo1_l0000Eooooo`KoZJVY5_ooool6ojVYZAGooooo1_nYZJT=ooooo`003Oooool4
ojVYZAOooooo1?l0000Gooooo`CoZJVY5oooool4ojVYZASooooo1?nYZJTBooooo`CoZJVY6?ooool4
o`0001Oooooo1?nYZJTGooooo`CoZJVY5oooool4ojVYZA?ooooo1?l0000Gooooo`Co00005oooool4
ojVYZASooooo1?nYZJTGooooo`CoZJVY3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo
000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000=ooooo`Co00005oooool4
o`0001Oooooo1?l0000Gooooo`CoZJVY6?ooool4o`0001;ooooo1?l0000Hooooo`Co00005oooool4
o`0001Oooooo1?nYZJTGooooo`Co00004oooool4o`0001Oooooo1?l0000Gooooo`Co00006?ooool4
o`0001Oooooo1?l0000>ooooo`003?ooool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6ojVY
ZAKooooo1_l0000@ooooo`Ko00005_ooool6o`0001Gooooo1_l0000Eooooo`KoZJVY5Oooool6o`00
017ooooo1_l0000Eooooo`Ko00005Oooool6o`0001Kooooo1_l0000Eooooo`Ko00003Oooool000_o
oooo2?l0000Cooooo`So00004oooool8o`0001?ooooo2?nYZJTDooooo`So00003_ooool8o`0001Co
oooo2?l0000Cooooo`So00004oooool8ojVYZA?ooooo2?l0000?ooooo`So00004oooool8o`0001?o
oooo2?l0000Dooooo`So00004oooool8o`0000cooooo000;ooooo`So00004oooool8o`0001?ooooo
2?l0000Cooooo`SoZJVY5?ooool8o`0000kooooo2?l0000Dooooo`So00004oooool8o`0001?ooooo
2?nYZJTCooooo`So00003oooool8o`0001?ooooo2?l0000Cooooo`So00005?ooool8o`0001?ooooo
2?l0000<ooooo`002ooooomAo`0000SoZJVY7?l0000>oooooe;o00002?nYZJTKo`0000ooooooMOl0
000<ooooo`002oooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?l0000>
ooooo`So00005?ooool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8o`0000oooooo2?l0000C
ooooo`So00004oooool8o`0001Cooooo2?l0000Cooooo`So00003?ooool000cooooo1_l0000Eoooo
o`Ko00005Oooool6o`0001Gooooo1_nYZJTFooooo`Ko00004?ooool6o`0001Kooooo1_l0000Eoooo
o`Ko00005Oooool6ojVYZAGooooo1_l0000Aooooo`Ko00005Oooool6o`0001Gooooo1_l0000Foooo
o`Ko00005Oooool6o`0000gooooo000=ooooo`Co00005oooool4o`0001Oooooo1?l0000Gooooo`Co
ZJVY6?ooool4o`0001;ooooo1?l0000Hooooo`Co00005oooool4o`0001Oooooo1?nYZJTGooooo`Co
00004oooool4o`0001Oooooo1?l0000Gooooo`Co00006?ooool4o`0001Oooooo1?l0000>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo00kooooo000=ooooo`Co00005oooool4o`0001Oooooo1?l0000Gooooo`Co0000
6?ooool4o`0001;ooooo1?l0000Hooooo`Co00005oooool4o`0001Oooooo1?l0000Gooooo`Co0000
4oooool4o`0001Oooooo1?l0000Gooooo`Co00006?ooool4o`0001Oooooo1?l0000>ooooo`003?oo
ool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`0001Kooooo1_l0000@ooooo`Ko00005_oo
ool6o`0001Gooooo1_l0000Eooooo`Ko00005Oooool6o`00017ooooo1_l0000Eooooo`Ko00005Ooo
ool6o`0001Kooooo1_l0000Eooooo`Ko00003Oooool000_ooooo2?l0000Cooooo`So00004oooool8
o`0001?ooooo2?l0000Dooooo`So00003_ooool8o`0001Cooooo2?l0000Cooooo`So00004oooool8
o`0001?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00004oooool8
o`0000cooooo000;ooooogGo00003_oooomeo`0000ooooooMOl0000<ooooo`002oooool8o`0001?o
oooo2?l0000Cooooo`So00004oooool8o`0001Cooooo2?l0000>ooooo`So00005?ooool8o`0001?o
oooo2?l0000Cooooo`So00004oooool8o`0000oooooo2?l0000Cooooo`So00004oooool8o`0001Co
oooo2?l0000Cooooo`So00003?ooool000_ooooo2?l0000Cooooo`So00004oooool8o`0001?ooooo
2?l0000Dooooo`So00003_ooool8o`0001Cooooo2?l0000Cooooo`So00004oooool8o`0001?ooooo
2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`So00004oooool8o`0000cooooo
000<ooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6o`00013ooooo1_l0
000Fooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`Ko00004Oooool6o`0001Gooooo1_l0
000Eooooo`Ko00005_ooool6o`0001Gooooo1_l0000=ooooo`003Oooool4o`0001Oooooo1?l0000G
ooooo`Co00005oooool4o`0001Sooooo1?l0000Booooo`Co00006?ooool4o`0001Oooooo1?l0000G
ooooo`Co00005oooool4o`0001?ooooo1?l0000Gooooo`Co00005oooool4o`0001Sooooo1?l0000G
ooooo`Co00003_ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?oo
ool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?oo
ool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?oo
ool000gooooo1?l0000Gooooo`Co00005oooool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY4_ooool4
o`0001Sooooo1?l0000Gooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTCooooo`Co00005oooool4
o`0001Oooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZ@kooooo000<ooooo`Ko00005Oooool6o`00
01Gooooo1_nYZJTEooooo`KoZJVY5_ooool6ojVYZA3ooooo1_l0000Fooooo`Ko00005Oooool6ojVY
ZAGooooo1_nYZJTEooooo`KoZJVY4Oooool6o`0001Gooooo1_l0000Eooooo`KoZJVY5_ooool6ojVY
ZAGooooo1_nYZJT=ooooo`002oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZACo
oooo2?nYZJT>ooooo`So00005?ooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY4oooool8ojVYZ@oo
oooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool000_ooooo
2?l0000Cooooo`So00004oooool8ojVYZA?ooooo2?nYZJTDooooo`SoZJVY3_ooool8o`0001Cooooo
2?l0000Cooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJT?ooooo`So00004oooool8o`0001?ooooo
2?nYZJTDooooo`SoZJVY4oooool8ojVYZ@cooooo000;ooooocKo00002?nYZJTCo`0000SoZJVY5?l0
0008ojVYZ@kooooo=ol00008ojVYZA?o00002?nYZJTCo`0000SoZJVY3ooooolfo`0000SoZJVY5?l0
0008ojVYZA?o00002?nYZJT<ooooo`002oooool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8
ojVYZACooooo2?nYZJT>ooooo`So00005?ooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY4oooool8
ojVYZ@oooooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?nYZJTCooooo`SoZJVY3?ooool0
00cooooo1_l0000Eooooo`Ko00005Oooool6ojVYZAGooooo1_nYZJTFooooo`KoZJVY4?ooool6o`00
01Kooooo1_l0000Eooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTAooooo`Ko00005Oooool6o`00
01Gooooo1_nYZJTFooooo`KoZJVY5Oooool6ojVYZ@gooooo000=ooooo`Co00005oooool4o`0001Oo
oooo1?nYZJTGooooo`CoZJVY6?ooool4ojVYZA;ooooo1?l0000Hooooo`Co00005oooool4ojVYZAOo
oooo1?nYZJTGooooo`CoZJVY4oooool4o`0001Oooooo1?l0000Gooooo`CoZJVY6?ooool4ojVYZAOo
oooo1?nYZJT>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000=ooooo`CoZJVY5oooool4ojVYZAOooooo
1?nYZJTGooooo`Co00006?ooool4ojVYZA;ooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZAOooooo
1?l0000Gooooo`CoZJVY4oooool4ojVYZAOooooo1?nYZJTGooooo`CoZJVY6?ooool4ojVYZAOooooo
1?nYZJT>ooooo`003?ooool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY5Oooool6o`0001Kooooo1_nY
ZJT@ooooo`KoZJVY5_ooool6ojVYZAGooooo1_nYZJTEooooo`Ko00005Oooool6ojVYZA7ooooo1_nY
ZJTEooooo`KoZJVY5Oooool6ojVYZAKooooo1_nYZJTEooooo`KoZJVY3Oooool000_ooooo2?nYZJTC
ooooo`SoZJVY4oooool8ojVYZA?ooooo2?l0000Dooooo`SoZJVY3_ooool8ojVYZACooooo2?nYZJTC
ooooo`SoZJVY4oooool8o`0001?ooooo2?nYZJT?ooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTD
ooooo`SoZJVY4oooool8ojVYZ@cooooo000;ooooo`SoZJVY4ol00008ojVYZA?o00002?nYZJT_o`00
00SoZJVY3_ooool8ojVYZACo00002?nYZJTCo`0000SoZJVY;_l00008ojVYZ@oooooo2?nYZJTCo`00
00SoZJVY4ol00008ojVYZACo00002?nYZJTCo`0000SoZJVY3?ooool000_ooooo2?nYZJTCooooo`So
ZJVY4oooool8ojVYZA?ooooo2?l0000Dooooo`SoZJVY3_ooool8ojVYZACooooo2?nYZJTCooooo`So
ZJVY4oooool8o`0001?ooooo2?nYZJT?ooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTDooooo`So
ZJVY4oooool8ojVYZ@cooooo000;ooooo`SoZJVY4oooool8ojVYZA?ooooo2?nYZJTCooooo`So0000
5?ooool8ojVYZ@kooooo2?nYZJTDooooo`SoZJVY4oooool8ojVYZA?ooooo2?l0000Cooooo`SoZJVY
3oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<ooooo`003?oo
ool6ojVYZAGooooo1_nYZJTEooooo`KoZJVY5Oooool6o`0001Kooooo1_nYZJT@ooooo`KoZJVY5_oo
ool6ojVYZAGooooo1_nYZJTEooooo`Ko00005Oooool6ojVYZA7ooooo1_nYZJTEooooo`KoZJVY5Ooo
ool6ojVYZAKooooo1_nYZJTEooooo`KoZJVY3Oooool000gooooo1?nYZJTGooooo`CoZJVY5oooool4
ojVYZAOooooo1?l0000Hooooo`CoZJVY4_ooool4ojVYZASooooo1?nYZJTGooooo`CoZJVY5oooool4
o`0001Oooooo1?nYZJTCooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTHooooo`CoZJVY5oooool4
ojVYZ@kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0>ooooo`003Oooool4ojVYZAOooooo1?l0000Gooooo`CoZJVY5oooool4
ojVYZASooooo1?nYZJTBooooo`CoZJVY6?ooool4o`0001Oooooo1?nYZJTGooooo`CoZJVY5oooool4
ojVYZA?ooooo1?nYZJTGooooo`Co00005oooool4ojVYZASooooo1?nYZJTGooooo`CoZJVY3_ooool0
00cooooo1_nYZJTEooooo`Ko00005Oooool6ojVYZAGooooo1_nYZJTFooooo`KoZJVY4?ooool6ojVY
ZAKooooo1_l0000Eooooo`KoZJVY5Oooool6ojVYZAGooooo1_nYZJTAooooo`KoZJVY5Oooool6o`00
01Gooooo1_nYZJTFooooo`KoZJVY5Oooool6ojVYZ@gooooo000;ooooo`SoZJVY4oooool8o`0001?o
oooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZ@kooooo2?nYZJTDooooo`So00004oooool8ojVYZA?o
oooo2?nYZJTCooooo`SoZJVY3oooool8ojVYZA?ooooo2?l0000Cooooo`SoZJVY5?ooool8ojVYZA?o
oooo2?nYZJT<ooooo`002oooool8ojVYZBko00002?nYZJTCo`0000SoZJVY5?l00008ojVYZ@kooooo
2?nYZJT_o`0000SoZJVY4ol00008ojVYZA?o00002?nYZJT?ooooo`SoZJVY;_l00008ojVYZACo0000
2?nYZJTCo`0000SoZJVY3?ooool000_ooooo2?nYZJTCooooo`So00004oooool8ojVYZA?ooooo2?nY
ZJTDooooo`SoZJVY3_ooool8ojVYZACooooo2?l0000Cooooo`SoZJVY4oooool8ojVYZA?ooooo2?nY
ZJT?ooooo`SoZJVY4oooool8o`0001?ooooo2?nYZJTDooooo`SoZJVY4oooool8ojVYZ@cooooo000;
ooooo`SoZJVY4oooool8o`0001?ooooo2?nYZJTCooooo`SoZJVY5?ooool8ojVYZ@kooooo2?nYZJTD
ooooo`So00004oooool8ojVYZA?ooooo2?nYZJTCooooo`SoZJVY3oooool8ojVYZA?ooooo2?l0000C
ooooo`SoZJVY5?ooool8ojVYZA?ooooo2?nYZJT<ooooo`003?ooool6ojVYZAGooooo1_l0000Eoooo
o`KoZJVY5Oooool6ojVYZAKooooo1_nYZJT@ooooo`KoZJVY5_ooool6o`0001Gooooo1_nYZJTEoooo
o`KoZJVY5Oooool6ojVYZA7ooooo1_nYZJTEooooo`Ko00005Oooool6ojVYZAKooooo1_nYZJTEoooo
o`KoZJVY3Oooool000gooooo1?nYZJTGooooo`Co00005oooool4ojVYZAOooooo1?nYZJTHooooo`Co
ZJVY4_ooool4ojVYZASooooo1?l0000Gooooo`CoZJVY5oooool4ojVYZAOooooo1?nYZJTCooooo`Co
ZJVY5oooool4o`0001Oooooo1?nYZJTHooooo`CoZJVY5oooool4ojVYZ@kooooo000?ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_oo
ool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`00
3oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
3_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003Oooool4o`0001Oooooo1?nYZJTGooooo`Co00005oooool4ojVYZASooooo1?l0000Booooo`Co
00006?ooool4ojVYZAOooooo1?l0000Gooooo`CoZJVY5oooool4o`0001?ooooo1?l0000Gooooo`Co
00005oooool4o`0001Sooooo1?nYZJTGooooo`Co00003_ooool000cooooo1_l0000Eooooo`KoZJVY
5Oooool6o`0001Gooooo1_nYZJTFooooo`Ko00004?ooool6o`0001Kooooo1_nYZJTEooooo`Ko0000
5Oooool6ojVYZAGooooo1_l0000Aooooo`Ko00005Oooool6o`0001Gooooo1_l0000Fooooo`KoZJVY
5Oooool6o`0000gooooo000;ooooo`So00004oooool8ojVYZA?ooooo2?l0000Cooooo`SoZJVY5?oo
ool8o`0000kooooo2?l0000Dooooo`SoZJVY4oooool8o`0001?ooooo2?nYZJTCooooo`So00003ooo
ool8o`0001?ooooo2?l0000Cooooo`So00005?ooool8ojVYZA?ooooo2?l0000<ooooo`002ooooolK
o`0000SoZJVY;_l00008ojVYZAco00003_oooolLo`0000SoZJVY;_l00008ojVYZA_o00003ooooomB
o`0000SoZJVY6ol0000<ooooo`002oooool8o`0001?ooooo2?nYZJTCooooo`So00004oooool8ojVY
ZACooooo2?l0000>ooooo`So00005?ooool8ojVYZA?ooooo2?l0000Cooooo`SoZJVY4oooool8o`00
00oooooo2?l0000Cooooo`So00004oooool8o`0001Cooooo2?nYZJTCooooo`So00003?ooool000_o
oooo2?l0000Cooooo`SoZJVY4oooool8o`0001?ooooo2?nYZJTDooooo`So00003_ooool8o`0001Co
oooo2?nYZJTCooooo`So00004oooool8ojVYZA?ooooo2?l0000?ooooo`So00004oooool8o`0001?o
oooo2?l0000Dooooo`SoZJVY4oooool8o`0000cooooo000<ooooo`Ko00005Oooool6ojVYZAGooooo
1_l0000Eooooo`KoZJVY5_ooool6o`00013ooooo1_l0000Fooooo`KoZJVY5Oooool6o`0001Gooooo
1_nYZJTEooooo`Ko00004Oooool6o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6ojVYZAGooooo
1_l0000=ooooo`003Oooool4o`0001Oooooo1?nYZJTGooooo`Co00005oooool4ojVYZASooooo1?l0
000Booooo`Co00006?ooool4ojVYZAOooooo1?l0000Gooooo`CoZJVY5oooool4o`0001?ooooo1?l0
000Gooooo`Co00005oooool4o`0001Sooooo1?nYZJTGooooo`Co00003_ooool000oooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?oooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hoooo
o`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>oooo
o`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooooooo
ool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?oo
ool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>
ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000oooo
ooooool03_ooool000oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03
o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0
6?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo000?ooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0>ooooo`003oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03_ooool000oooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00kooooo
000=ooooo`Co00005oooool4o`0001Oooooo1?l0000Gooooo`CoZJVY6?ooool4o`0001;ooooo1?l0
000Hooooo`Co00005oooool4o`0001Oooooo1?nYZJTGooooo`Co00004oooool4o`0001Oooooo1?l0
000Gooooo`Co00006?ooool4ojVYZAOooooo1?l0000>ooooo`003?ooool6o`0001Gooooo1_l0000E
ooooo`Ko00005Oooool6ojVYZAKooooo1_l0000@ooooo`Ko00005_ooool6o`0001Gooooo1_l0000E
ooooo`KoZJVY5Oooool6o`00017ooooo1_l0000Eooooo`Ko00005Oooool6o`0001Kooooo1_nYZJTE
ooooo`Ko00003Oooool000_ooooo2?l0000Cooooo`So00004oooool8o`0001?ooooo2?nYZJTDoooo
o`So00003_ooool8o`0001Cooooo2?l0000Cooooo`So00004oooool8ojVYZA?ooooo2?l0000?oooo
o`So00004oooool8o`0001?ooooo2?l0000Dooooo`SoZJVY4oooool8o`0000cooooo000;oooooe7o
00002?nYZJTLo`0000koooooD_l00008ojVYZA_o00003ooooomBo`0000SoZJVY6ol0000<ooooo`00
2oooool8o`0001?ooooo2?l0000Cooooo`So00004oooool8ojVYZACooooo2?l0000>ooooo`So0000
5?ooool8o`0001?ooooo2?l0000Cooooo`SoZJVY4oooool8o`0000oooooo2?l0000Cooooo`So0000
4oooool8o`0001Cooooo2?nYZJTCooooo`So00003?ooool000_ooooo2?l0000Cooooo`So00004ooo
ool8o`0001?ooooo2?nYZJTDooooo`So00003_ooool8o`0001Cooooo2?l0000Cooooo`So00004ooo
ool8ojVYZA?ooooo2?l0000?ooooo`So00004oooool8o`0001?ooooo2?l0000Dooooo`SoZJVY4ooo
ool8o`0000cooooo000<ooooo`Ko00005Oooool6o`0001Gooooo1_l0000Eooooo`KoZJVY5_ooool6
o`00013ooooo1_l0000Fooooo`Ko00005Oooool6o`0001Gooooo1_nYZJTEooooo`Ko00004Oooool6
o`0001Gooooo1_l0000Eooooo`Ko00005_ooool6ojVYZAGooooo1_l0000=ooooo`003Oooool4o`00
01Oooooo1?l0000Gooooo`Co00005oooool4ojVYZASooooo1?l0000Booooo`Co00006?ooool4o`00
01Oooooo1?l0000Gooooo`CoZJVY5oooool4o`0001?ooooo1?l0000Gooooo`Co00005oooool4o`00
01Sooooo1?nYZJTGooooo`Co00003_ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?oo
ool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?oo
ool00?ooooooU?ooool00?ooooooU?ooool00?ooooooU?ooool00001\
\>"],
  ImageRangeCache->{{{0, 402}, {402, 0}} -> {-0.0750158, -0.0750158, \
0.00783591, 0.00783591}, {{9.5625, 129.188}, {392.375, 272.75}} -> \
{-0.137937, -0.138512, 0.00919549, 0.00919549}, {{141.125, 260.812}, \
{392.375, 272.75}} -> {-1.34801, -0.138512, 0.00919549, 0.00919549}, \
{{272.75, 392.375}, {392.375, 272.75}} -> {-2.55808, -0.138512, 0.00919549, \
0.00919549}, {{9.5625, 129.188}, {260.812, 141.125}} -> {-0.137937, -1.34858, \
0.00919549, 0.00919549}, {{141.125, 260.812}, {260.812, 141.125}} -> \
{-1.34704, -1.34762, 0.00919069, 0.00919069}, {{272.75, 392.375}, {260.812, \
141.125}} -> {-2.55808, -1.34858, 0.00919549, 0.00919549}, {{9.5625, \
129.188}, {129.188, 9.5625}} -> {-0.137937, -2.55865, 0.00919549, \
0.00919549}, {{141.125, 260.812}, {129.188, 9.5625}} -> {-1.34801, -2.55865, \
0.00919549, 0.00919549}, {{272.75, 392.375}, {129.188, 9.5625}} -> {-2.55808, \
-2.55865, 0.00919549, 0.00919549}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["And here's the last generation...", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowLabeledGraph[l\[LeftDoubleBracket]10\[RightDoubleBracket], 
        VertexStyle \[Rule] Disk[Large]];\)\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
[(1)] .02545 .02545 1 0 Mshowa
.27273 .04545 Mdot
[(2)] .25273 .02545 1 0 Mshowa
.663 .663 .663 r
.5 .04545 Mdot
0 0 0 r
[(3)] .48 .02545 1 0 Mshowa
.663 .663 .663 r
.72727 .04545 Mdot
0 0 0 r
[(4)] .70727 .02545 1 0 Mshowa
.663 .663 .663 r
.95455 .04545 Mdot
0 0 0 r
[(5)] .93455 .02545 1 0 Mshowa
.04545 .27273 Mdot
[(6)] .02545 .25273 1 0 Mshowa
.27273 .27273 Mdot
[(7)] .25273 .25273 1 0 Mshowa
.663 .663 .663 r
.5 .27273 Mdot
0 0 0 r
[(8)] .48 .25273 1 0 Mshowa
.663 .663 .663 r
.72727 .27273 Mdot
0 0 0 r
[(9)] .70727 .25273 1 0 Mshowa
.663 .663 .663 r
.95455 .27273 Mdot
0 0 0 r
[(10)] .93455 .25273 1 0 Mshowa
.04545 .5 Mdot
[(11)] .02545 .48 1 0 Mshowa
.27273 .5 Mdot
[(12)] .25273 .48 1 0 Mshowa
.5 .5 Mdot
[(13)] .48 .48 1 0 Mshowa
.663 .663 .663 r
.72727 .5 Mdot
0 0 0 r
[(14)] .70727 .48 1 0 Mshowa
.663 .663 .663 r
.95455 .5 Mdot
0 0 0 r
[(15)] .93455 .48 1 0 Mshowa
.04545 .72727 Mdot
[(16)] .02545 .70727 1 0 Mshowa
.27273 .72727 Mdot
[(17)] .25273 .70727 1 0 Mshowa
.5 .72727 Mdot
[(18)] .48 .70727 1 0 Mshowa
.72727 .72727 Mdot
[(19)] .70727 .70727 1 0 Mshowa
.95455 .72727 Mdot
[(20)] .93455 .70727 1 0 Mshowa
.04545 .95455 Mdot
[(21)] .02545 .93455 1 0 Mshowa
.27273 .95455 Mdot
[(22)] .25273 .93455 1 0 Mshowa
.5 .95455 Mdot
[(23)] .48 .93455 1 0 Mshowa
.72727 .95455 Mdot
[(24)] .70727 .93455 1 0 Mshowa
.95455 .95455 Mdot
[(25)] .93455 .93455 1 0 Mshowa
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[83]:=",
  ImageSize->{235, 235},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003[0000jb000`400?l00000o`00003ojoooool0
0>_ooooo003[ooooo`00joooool000?o00001?ooool6o`0002Sooooo1Ol00003ooooo`Ko0000:?oo
ool3o`0000Cooooo1_nYZJTYooooo`?o00000oooool6ojVYZBSooooo0ol00004ooooo`KoZJVY2Ooo
ool00003ooooool0003ooooo00;ooooo2_l0000Wooooo`05o`000?ooooooooooo`000?ooool02_l0
000Uooooo`06o`000?ooooooooooooooool0003ooooo2_nYZJTXooooo`03o`000?oooooooooo00[o
ZJVY9Oooool01_l0003ooooooooooooooooo0000ooooo`[oZJVY1oooool00004ooooool0003ooooo
ooooo`co00009oooool00ol0003oooooooooo`0<o`0002Sooooo00?o0000ojVYZOnYZJT02_nYZJTT
ooooo`Go00003?nYZJTXooooo`03o`000?nYZJWoZJVY00[oZJVY1_ooool00003ooooool0003ooooo
00ko00009oooool?o`0002Gooooo0_l0000>ojVYZB?ooooo00Co0000oooooooooooo00003_nYZJTW
ooooo`03o`000?nYZJWoZJVY00_oZJVY1Oooool00003ooooool0003ooooo00ko00009?ooool01?l0
003oooooooooooooool>o`0002Oooooo00?o0000ojVYZOnYZJT02onYZJTTooooo`03o`000?oooooo
000000koZJVY8oooool4o`0000koZJVY1Oooool001;o00008oooool00ol0003oooooooooo`0@o`00
02;ooooo00Go0000ooooooooooooZJVYo`00000>ojVYZBCooooo0_l0000?ojVYZB;ooooo00?o0000
ooooooooool04?nYZJT4ooooo`0000?oooooo`000?l000003ol0000Toooooa;o00008oooool3o`00
00ooZJVY9Oooool00ol0003oZJVYojVYZ@0=ojVYZB;ooooo1Ol0000>ojVYZ@Cooooo0002ooooof_o
00004?nYZJTUo`00013oZJVY9Ol0000@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo
4?nYZJTUoooooa3oZJVY9Oooool@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo4?nY
ZJTUoooooa3oZJVY9Oooool@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo4?nYZJTU
oooooa3oZJVY9Oooool@ojVYZ@Cooooo0003ooooo`ko0000:?ooool>o`0002Oooooo3_nYZJTWoooo
o`koZJVY9oooool>ojVYZ@Gooooo0003ooooo`ko0000:?ooool>o`0002Oooooo3_nYZJTWooooo`ko
ZJVY9oooool>ojVYZ@Gooooo0004ooooo`co0000:_ooool<o`0002Wooooo3?nYZJTYooooo`coZJVY
:Oooool<ojVYZ@Kooooo0005ooooo`[o0000;?ooool:o`0002_ooooo2_nYZJT[ooooo`[oZJVY:ooo
ool:ojVYZ@Oooooo0007ooooo`Ko0000<?ooool6o`0002oooooo1_nYZJT_ooooo`KoZJVY;oooool6
ojVYZ@Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0c
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool0
00[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool02Oooool000?o00001?ooool6o`0002[ooooo00?o0000ooooooooool00oooool6o`0002Sooooo
0ol00004ooooo`KoZJVY:?ooool2o`0000Gooooo1_nYZJTRooooo`?o00001?ooool2o`0000Cooooo
1_nYZJT9ooooo`000oooool00ol0003oooooo`000009o`0002Sooooo00Co0000oooooooooooooooo
2_l0000Uooooo`06o`000?ooooooooooooooool0003ooooo2_nYZJTXooooo`03o`000?oooooooooo
00[oZJVY8Oooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooo`000?ooool02_nYZJT7
ooooo`000oooool=o`0002Sooooo00?o0000ooooool000002ol0000Tooooo`05o`000?oooooooooo
ooooool000003?nYZJTXooooo`03o`000?nYZJWoZJVY00[oZJVY8?ooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000coZJVY1_ooool0017o00009oooool?o`0002Cooooo0ol0000>
ojVYZBCooooo1?l0000=ojVYZAoooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
000=ojVYZ@Gooooo0003ooooo`ko0000:?ooool>o`0002?ooooo00Go0000ooooooooooooooooo`00
000=ojVYZB?ooooo00Go0000ooooooooooooooooo`00000=ojVYZAoooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool0000=ojVYZ@Gooooo00000ol0003oooooo`00000?o`0002?ooooo
00?o0000ooooooooool04?l0000Rooooo`05o`000?ooooooooooojVYZOl000003_nYZJTRooooo`05
o`000?ooooooooooojVYZOl000003_nYZJTMooooo`;o00001?ooool01?l0003oooooojVYZOl0000>
ojVYZ@Cooooo00000ooooooo0000o`00000?o`0002?ooooo4ol0000Sooooo`?o00003onYZJTSoooo
o`?o00003onYZJTNooooo`03o`000?oooooooooo00?ooooo0_l0000?ojVYZ@Cooooo0002ooooof_o
00004?nYZJTUo`00013oZJVY9Ol0000@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo
4?nYZJTUoooooa3oZJVY9Oooool@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo4?nY
ZJTUoooooa3oZJVY9Oooool@ojVYZ@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo4?nYZJTU
oooooa3oZJVY9Oooool@ojVYZ@Cooooo0003ooooo`ko0000:?ooool>o`0002Oooooo3_nYZJTWoooo
o`koZJVY9oooool>ojVYZ@Gooooo0003ooooo`ko0000:?ooool>o`0002Oooooo3_nYZJTWooooo`ko
ZJVY9oooool>ojVYZ@Gooooo0004ooooo`co0000:_ooool<o`0002Wooooo3?nYZJTYooooo`coZJVY
:Oooool<ojVYZ@Kooooo0005ooooo`[o0000;?ooool:o`0002_ooooo2_nYZJT[ooooo`[oZJVY:ooo
ool:ojVYZ@Oooooo0007ooooo`Ko0000<?ooool6o`0002oooooo1_nYZJT_ooooo`KoZJVY;oooool6
ojVYZ@Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0c
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool0
00[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool02Oooool000?o00001?ooool6o`0002?ooooo0ol00002ooooo`Go00000oooool6o`0002;ooooo
0ol00003ooooo`?o00001?ooool6o`0002;ooooo0ol00004ooooo`?o00000oooool6ojVYZB;ooooo
0ol00003ooooo`?o00001?ooool6ojVYZ@Wooooo00000ooooooo0000ooooo`02ooooo`[o00008_oo
ool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooo`000?ooool02_l0000Qooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooo`000009o`00027ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0:ojVYZB7ooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?ooooooZJVY00WoZJVY1oooool00004ooooool0003oooooooooo`co00008Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00co00008?ooool00ol0003oooooooooo`05oooo
o`go00008?ooool01?l0003oooooooooooooool5o`0000coZJVY8?ooool00ol0003oooooooooo`05
ooooo`03o`000?nYZJWoZJVY00[oZJVY1_ooool00003ooooool0003ooooo00ko00008?ooool00ol0
003oooooooooo`04ooooo`oo00007oooool00ol0003oooooooooo`03oooooa3o00007oooool01Ol0
003ooooooooooooooooo000000;ooooo00?o0000ojVYZOnYZJT03?nYZJTOooooo`03o`000?oooooo
oooo00Gooooo00?o0000ojVYZOnYZJT02onYZJT5ooooo`0000?oooooo`000?ooool03_l0000Poooo
o`05o`000?ooooooooooooooool000000oooool>o`0001oooooo00?o0000ooooooooool01Oooool>
o`0001oooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00000>ojVYZAoooooo00Co0000
oooooooooooooooo1?l0000>ojVYZ@Gooooo000Bo`0001kooooo0_l00003ooooo`03o`000?oooooo
oooo013o00007Oooool2o`0000?ooooo00?o0000ooooooooool04?l0000Mooooo`;o00001Oooool2
o`0000ooZJVY7Oooool2o`0000?ooooo00?o0000ooooooooool04?nYZJT4ooooo`0000?oooooo`00
0?l000003ol0000Oooooo`03o`000?oooooooooo00;ooooo4_l0000Nooooo`03o`000?oooooooooo
00;ooooo4_l0000Nooooo`03o`000?oooooooooo00Cooooo00?o0000ojVYZOnYZJT03OnYZJTNoooo
o`04o`000?ooooooooooooooo`Go00003_nYZJT4ooooo`000_oooonPo`00013oZJVY9Ol0000@ojVY
Z@Cooooo0002oooooa3o00009_ooool@o`0002Gooooo4?l0000Uoooooa3oZJVY9Oooool@ojVYZ@Co
oooo0002oooooa3o00009_ooool@o`0002Gooooo4?l0000Uoooooa3oZJVY9Oooool@ojVYZ@Cooooo
0002oooooa3o00009_ooool@o`0002Gooooo4?l0000Uoooooa3oZJVY9Oooool@ojVYZ@Cooooo0003
ooooo`ko0000:?ooool>o`0002Oooooo3_l0000Wooooo`koZJVY9oooool>ojVYZ@Gooooo0003oooo
o`ko0000:?ooool>o`0002Oooooo3_l0000Wooooo`koZJVY9oooool>ojVYZ@Gooooo0004ooooo`co
0000:_ooool<o`0002Wooooo3?l0000Yooooo`coZJVY:Oooool<ojVYZ@Kooooo0005ooooo`[o0000
;?ooool:o`0002_ooooo2_l0000[ooooo`[oZJVY:oooool:ojVYZ@Oooooo0007ooooo`Ko0000<?oo
ool6o`0002oooooo1_l0000_ooooo`KoZJVY;oooool6ojVYZ@Wooooo000:ooooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wo
oooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0coooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0c
ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000oooo
ooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool0
00[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000?o00001?ooool6o`0002?o
oooo0ol00004ooooo`03o`000?oooooooooo00?ooooo1_l0000Rooooo`?o00000oooool3o`0000Co
oooo1_l0000Rooooo`?o00000oooool2o`0000Gooooo1_l0000Qooooo`Go00000oooool2o`0000Co
oooo1_l00009ooooo`000oooool00ol0003oooooo`000009o`0002;ooooo00?o0000ooooooooool0
0oooool01?l0003oooooooooooooool:o`00027ooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000Wo00008Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00[o00008?ooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooo`000?ooool02_l0
0007ooooo`000oooool=o`00027ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00000;
o`00023ooooo00Go0000ooooooooooooooooo`000003ooooo`go00008?ooool00ol0003ooooooooo
o`05ooooo`go00008?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00go00001_oo
ool0017o00008?ooool00ol0003oooooooooo`04ooooo`oo00007oooool00ol0003oooooooooo`02
oooooa7o00007oooool00ol0003oooooooooo`02oooooa7o00008?ooool01Ol0003ooooooooooooo
oooo000000;ooooo3_l00005ooooo`000oooool>o`00023ooooo00?o0000ooooooooool01Oooool>
o`0001oooooo00Go0000ooooooooooooooooo`000003ooooo`ko00007oooool01Ol0003ooooooooo
oooooooo000000?ooooo3_l0000Mooooo`05o`000?ooooooooooooooool000000_ooool00ol0003o
ooooooooo`0>o`0000Gooooo00000ol0003oooooo`00000?o`0001kooooo0_l00003ooooo`03o`00
0?oooooooooo013o00007Oooool2o`0000?ooooo00?o0000ooooooooool04?l0000Mooooo`;o0000
0oooool00ol0003oooooooooo`0@o`0001cooooo00Go0000ooooooooooooooooo`000002ooooo`03
o`000?oooooo000000oo00001?ooool00003ooooool0003o000000oo00007oooool01?l0003ooooo
oooooooooolCo`0001kooooo00?o0000ooooooooool00_oooolBo`0001kooooo00?o0000oooooooo
ool00_oooolBo`0001gooooo0ol00004oooooa7o00001?ooool000;ooooo4?l0000Voooooa3o0000
9Oooool@o`0002Gooooo4?l0000Uoooooa3o00001?ooool000;oooooiOl00004ooooo`000_ooool@
o`0002Kooooo4?l0000Uoooooa3o00009Oooool@o`0002Gooooo4?l00004ooooo`000_ooool@o`00
02Kooooo4?l0000Uoooooa3o00009Oooool@o`0002Gooooo4?l00004ooooo`000oooool>o`0002So
oooo3_l0000Wooooo`ko00009oooool>o`0002Oooooo3_l00005ooooo`000oooool>o`0002Sooooo
3_l0000Wooooo`ko00009oooool>o`0002Oooooo3_l00005ooooo`001?ooool<o`0002[ooooo3?l0
000Yooooo`co0000:Oooool<o`0002Wooooo3?l00006ooooo`001Oooool:o`0002cooooo2_l0000[
ooooo`[o0000:oooool:o`0002_ooooo2_l00007ooooo`001oooool6o`00033ooooo1_l0000_oooo
o`Ko0000;oooool6o`0002oooooo1_l00009ooooo`002_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03
o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000oooooooo
ool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wo
oooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0
<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool0
0ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo
00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000
ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0coooo
o`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`00
0?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0booooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03;o
oooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`0booooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool0<oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo03;ooooo00?o
0000ooooooooool02Oooool000?o00001?ooool6o`0002;ooooo1Ol000000ooooooo0000o`000003
o`0000?ooooo1_l0000Qooooo`Go00000_ooool3o`0000Cooooo1_l0000Qooooo`Go00000oooool3
o`0000?ooooo1_l0000Qooooo`Go00000_ooool3o`0000Cooooo1_l00009ooooo`0000?oooooo`00
0?ooool00_ooool:o`00027ooooo00Co0000oooooooooooo00000_ooool01Ol0003oooooooooool0
003ooooo00[o00008?ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000ooooool0
00002Ol0000Pooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool02_l0000Poooo
o`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003oooooo`000009o`0000Oooooo0000
1?oooooo0000ooooooooool<o`00027ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`0<o`00023ooooo00?o0000ooooooooool01Oooool=o`00023ooooo00Co0000oooooooooooooooo
4Ol0000Pooooo`03o`000?oooooooooo00Gooooo3Ol00006ooooo`0000?oooooo`000?ooool03_l0
000Qooooo`03o`000?oooooooooo00?ooooo3ol0000Pooooo`03o`000?oooooooooo00;ooooo4?l0
000Pooooo`04o`000?ooooooooooo`0000;ooooo3ol0000Pooooo`03o`000?oooooooooo00Cooooo
3_l00005ooooo`0000?oooooo`000?ooool03_l0000Nooooo`07o`000?ooooooooooooooool0003o
ooooo`000003ooooo`ko00007Oooool01Ol0003ooooooooooooooooo000000Gooooo3_l0000Moooo
o`05o`000?ooooooooooooooool000000_ooool00ol0003oooooo`00000>o`0001gooooo00Ko0000
ooooooooooooooooo`000?oooolBo`0000Gooooo000Bo`0001gooooo00Oo0000oooooooooooooooo
o`000?oooooo000000;ooooo4?l0000Looooo`07o`000?ooooooooooooooool0003oooooo`000002
oooooa3o00007?ooool01Ol0003ooooooooooooooooo000000?ooooo4Ol0000Looooo`07o`000?oo
ooooooooooooool0003oooooo`000002oooooa3o00001?ooool00003ooooool0003o000000oo0000
7_ooool3o`0000?ooooo4_l0000Mooooo`?o00000ooooolBo`0001gooooo0ol00005oooooa3o0000
7Oooool3o`0000;ooooo4ol00004ooooo`000_oooooUo`0000Cooooo0002oooooa3o00009_ooool@
o`0002Gooooo4?l0000Uoooooa3o00009Oooool@o`0000Cooooo0002oooooa3o00009_ooool@o`00
02Gooooo4?l0000Uoooooa3o00009Oooool@o`0000Cooooo0002oooooa3o00009_ooool@o`0002Go
oooo4?l0000Uoooooa3o00009Oooool@o`0000Cooooo0003ooooo`ko0000:?ooool>o`0002Oooooo
3_l0000Wooooo`ko00009oooool>o`0000Gooooo0003ooooo`ko0000:?ooool>o`0002Oooooo3_l0
000Wooooo`ko00009oooool>o`0000Gooooo0004ooooo`co0000:_ooool<o`0002Wooooo3?l0000Y
ooooo`co0000:Oooool<o`0000Kooooo0005ooooo`[o0000;?ooool:o`0002_ooooo2_l0000[oooo
o`[o0000:oooool:o`0000Oooooo0007ooooo`Ko0000<?ooool6o`0002oooooo1_l0000_ooooo`Ko
0000;oooool6o`0000Wooooo003[ooooo`00joooool00001\
\>"],
  ImageRangeCache->{{{0, 234}, {234, 0}} -> {-0.0500054, -0.0500054, \
0.0047009, 0.0047009}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["\<\
A few agent may not have decided to hunt rabbit. What are their \
data sets like? (You might need to change the agent number as needed.)\
\>", \
"Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Data[GetAgent[stag1, 14]]\)], "Input",
  CellLabel->"In[84]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({2.687267077855664`, 
      10, {0.8029520475403643`, 0.19704795245963538`}}\)], "Output",
  CellLabel->"Out[84]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[stag1]\)], "Input",
  CellLabel->"In[85]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["Let's try this with exponential fictitious play.", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(stag2 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] GridGraph[5, 5], \[IndentingNewLine]PayoffMatrix \[Rule] 
            stagHunt, \[IndentingNewLine]StrategyColors \[Rule] 
            stagColors, \[IndentingNewLine]ShowVertexNumbers \[Rule] 
            True, \[IndentingNewLine]WindowLabel \[Rule] "\<Stag hunt, \
exponential fictitious play\>"\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[86]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Change the definition of ",
  StyleBox["Initialize", "Input"],
  " so that it sets \[Theta] to values in the right range."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Initialize[agent_] := 
        Module[{\[Theta], s}, \[IndentingNewLine]\[Theta] = 
            Random[Real, {1, 5}]; \[IndentingNewLine]s = 
            Strategy[agent]; \[IndentingNewLine]SetData[
            agent, {\[Theta], 0, 
              If[s \[Equal] 1, {1.0, 0.0}, {0.0, 
                  1.0}]}];\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[87]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(agentList = Agents[stag2];\)\), "\n", 
    \(\(Map[Initialize, agentList];\)\), "\[IndentingNewLine]", 
    \(\(Map[\[IndentingNewLine]SetLearningRule[#, 
            ExponentialFictitiousPlayLR] &, \[IndentingNewLine]agentList\
\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[88]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(bob = GetAgent[stag2, 8];\)\), "\[IndentingNewLine]", 
    \(\(Print[Data[bob]];\)\), "\[IndentingNewLine]", 
    \(\(Print[LearningRule[bob]];\)\)}], "Input",
  CellLabel->"In[91]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \({2.048784014863821`, 0, {1.`, 0.`}}\)], "Print",
  CellLabel->"From In[91]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(ExponentialFictitiousPlayLR\)], "Print",
  CellLabel->"From In[91]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowLabeledGraph[ExtractGraph[stag2]];\)\)], "Input",
  CellLabel->"In[94]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.025 w
.04545 .04545 Mdot
0 0 0 r
[(1)] .02545 .02545 1 0 Mshowa
.27273 .04545 Mdot
[(2)] .25273 .02545 1 0 Mshowa
.5 .04545 Mdot
[(3)] .48 .02545 1 0 Mshowa
.663 .663 .663 r
.72727 .04545 Mdot
0 0 0 r
[(4)] .70727 .02545 1 0 Mshowa
.663 .663 .663 r
.95455 .04545 Mdot
0 0 0 r
[(5)] .93455 .02545 1 0 Mshowa
.04545 .27273 Mdot
[(6)] .02545 .25273 1 0 Mshowa
.27273 .27273 Mdot
[(7)] .25273 .25273 1 0 Mshowa
.663 .663 .663 r
.5 .27273 Mdot
0 0 0 r
[(8)] .48 .25273 1 0 Mshowa
.72727 .27273 Mdot
[(9)] .70727 .25273 1 0 Mshowa
.95455 .27273 Mdot
[(10)] .93455 .25273 1 0 Mshowa
.663 .663 .663 r
.04545 .5 Mdot
0 0 0 r
[(11)] .02545 .48 1 0 Mshowa
.27273 .5 Mdot
[(12)] .25273 .48 1 0 Mshowa
.663 .663 .663 r
.5 .5 Mdot
0 0 0 r
[(13)] .48 .48 1 0 Mshowa
.663 .663 .663 r
.72727 .5 Mdot
0 0 0 r
[(14)] .70727 .48 1 0 Mshowa
.663 .663 .663 r
.95455 .5 Mdot
0 0 0 r
[(15)] .93455 .48 1 0 Mshowa
.04545 .72727 Mdot
[(16)] .02545 .70727 1 0 Mshowa
.27273 .72727 Mdot
[(17)] .25273 .70727 1 0 Mshowa
.5 .72727 Mdot
[(18)] .48 .70727 1 0 Mshowa
.72727 .72727 Mdot
[(19)] .70727 .70727 1 0 Mshowa
.95455 .72727 Mdot
[(20)] .93455 .70727 1 0 Mshowa
.04545 .95455 Mdot
[(21)] .02545 .93455 1 0 Mshowa
.27273 .95455 Mdot
[(22)] .25273 .93455 1 0 Mshowa
.5 .95455 Mdot
[(23)] .48 .93455 1 0 Mshowa
.72727 .95455 Mdot
[(24)] .70727 .93455 1 0 Mshowa
.663 .663 .663 r
.95455 .95455 Mdot
0 0 0 r
[(25)] .93455 .93455 1 0 Mshowa
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[94]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`000_ooool3
o`0003gooooo1Ol0000mooooo`?o0000?oooool3o`0003kooooo0ol0000Fooooo`000oooool00ol0
003oooooooooo`0mooooo`04o`000?ooooooooooo`0003cooooo00Go0000ooooooooooooooooo`00
000oooooo`03o`000?oooooooooo03cooooo00Go0000ooooooooooooooooo`00000Eooooo`000ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool0>_oo
ool5o`00047ooooo00?o0000ooooooooool04oooool000?ooooo00?o0000ooooooooool0?oooool0
0ol0003oooooooooo`0mooooo`;o0000?Oooool01?l0003oooooooooool00012ooooo`03o`000?oo
oooooooo01?ooooo0003ooooo`03o`000?oooooooooo03cooooo00Go0000ooooooooooooooooo`00
0010ooooo`03o`000?oooooooooo03_ooooo00?o0000ooooool00000?_ooool4o`0001Kooooo0002
ooooo`;o0000?_ooool01Ol0003ooooooooooooooooo000003cooooo00Go0000oooooooooooooooo
o`00000nooooo`;o0000?_ooool00ol0003oooooooooo`0Gooooo`000oooool00ol0003ooooooooo
o`05ooooo`?oZJVY=Oooool3o`0000Oooooo0ol0000dooooo`?o00001oooool3o`0003Kooooo00?o
0000ooooooooool01Oooool3ojVYZCCooooo1Ol00005ooooo`?oZJVY3Oooool000[ooooo1OnYZJTm
ooooo`Go0000??ooool5o`0003cooooo1OnYZJTlooooo`GoZJVY3?ooool000Wooooo1onYZJTkoooo
o`Oo0000>_ooool7o`0003[ooooo1onYZJTjooooo`OoZJVY2oooool000Wooooo1onYZJVmo`0000Oo
ZJVY>_l00007ojVYZ@_ooooo0009ooooo`OoZJVY>oooool7o`0003[ooooo1ol0000jooooo`OoZJVY
>_ooool7ojVYZ@_ooooo000:ooooo`GoZJVY?Oooool5o`0003cooooo1Ol0000looooo`GoZJVY??oo
ool5ojVYZ@cooooo000;ooooo`?oZJVY?oooool3o`0003kooooo0ol0000nooooo`?oZJVY?_ooool3
ojVYZ@gooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`000_ooool3o`0000Sooooo00?o
0000ooooooooool0=?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo03?ooooo0ol0
0008ooooo`03o`000?oooooooooo03?ooooo0_l00009ooooo`03o`000?oooooooooo02kooooo0ol0
0004ooooo`;o00001oooool00ol0003oooooooooo`0<ooooo`0000?oooooo`000?ooool00_ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo03Cooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`0booooo`05o`000?ooooooooooooooool000001oooool00ol0003ooooooooo
o`0eooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool03?ooool00003ooooool0
003ooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0eooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool0<_ooool01Ol0003ooooooooooooooooo000000Oooooo
00?o0000ooooooooool0=_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo02oooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00co
oooo00000ooooooo0000o`000002o`0000Sooooo00?o0000ooooooooool0=Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo03?ooooo0ol00008ooooo`03o`000?oooooooooo03?ooooo
1?l00007ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool00_ooool01?l0003ooooo
ooooool00006ooooo`03o`000?oooooooooo00cooooo00000ooooooo0000ooooo`0:ooooo`03o`00
0?oooooooooo03Kooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0booooo`05o`00
0?ooooooooooooooool000001oooool00ol0003oooooooooo`0booooo`05o`000?oooooooooooooo
ool000001oooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00001_ooool00ol0003oooooooooo`0<ooooo`000_ooool00ol0003oooooooooo`06oooo
o`?o0000=?ooool01Ol0003ooooooooooooooooo000000Kooooo0ol0000cooooo`05o`000?oooooo
ooooooooool000001_ooool3ojVYZC?ooooo00Go0000ooooooooooooooooo`000006ooooo`?o0000
;oooool2o`0000Cooooo00Co0000oooooooooooo00001Oooool3o`0000gooooo0003ooooo`;o0000
1Oooool5o`0003?ooooo1Ol00005ooooo`Go0000<oooool3o`0000Kooooo1OnYZJTcooooo`?o0000
1_ooool5o`0002oooooo00?o0000ooooooooool00oooool2o`0000Gooooo1Ol0000<ooooo`002Ooo
ool7o`0003_ooooo1ol0000jooooo`OoZJVY>_ooool7o`0003[ooooo1ol0000;ooooo`002Ooooon3
o`0000OoZJVYP_l0000;ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`OoZJVY>_ooool7o`00
03[ooooo1ol0000;ooooo`002_ooool5o`0003gooooo1Ol0000looooo`GoZJVY??ooool5o`0003co
oooo1Ol0000<ooooo`002oooool3o`0003oooooo0ol0000nooooo`?oZJVY?_ooool3o`0003kooooo
0ol0000=ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000;ooooo0ol00008ooooo`03
o`000?oooooooooo02gooooo0ol00002ooooo`Go00001oooool00ol0003oooooooooo`0]ooooo`?o
00000oooool3o`0000Sooooo00?o0000ooooooooool0;Oooool3o`0000Cooooo0ol00007ooooo`03
o`000?oooooooooo02kooooo0ol00003ooooo`?o00001oooool00ol0003oooooooooo`0<ooooo`00
0oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00007ooooo`03o`000?oooooooooo02kooooo00Go0000oooooooo
ooooooooo`000003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;oooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<
ooooo`000oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02kooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02kooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo02kooooo00Co0000oooo
oooooooooooo1Ol00007ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo0003ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`03ooooo`;o00002?ooool00ol0
003oooooooooo`0^ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00cooooo0003ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool0;_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000002?ooool0
0ol0003oooooooooo`0_ooooo`04o`000?ooooooooooooooo`Co00001oooool00ol0003ooooooooo
o`0<ooooo`000_ooool2o`0000Oooooo0onYZJT_ooooo`;o00000oooool01Ol0003ooooooooooooo
oooo000000Kooooo0ol0000^ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Kooooo
0onYZJT^ooooo`;o00001Oooool2o`0000Oooooo0onYZJT_ooooo`;o00000oooool00ol0003ooooo
ooooo`07ooooo`?oZJVY3Oooool000?ooooo00?o0000ooooooooool01?ooool5ojVYZBoooooo00?o
0000ooooooooool00_ooool3o`0000Kooooo1Ol0000^ooooo`03o`000?oooooooooo00;ooooo0ol0
0006ooooo`GoZJVY;_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo1OnY
ZJT_ooooo`04o`000?ooooooooooooooo`Go00001?ooool5ojVYZ@cooooo0009ooooo`OoZJVY>ooo
ool7o`0003[ooooo1onYZJTjooooo`OoZJVY>_ooool7ojVYZ@_ooooo0009ooooo`OoZJVYO?l00007
ojVYZC[o00001onYZJTjo`0000OoZJVY2oooool000Wooooo1onYZJTkooooo`Oo0000>_ooool7ojVY
ZC[ooooo1onYZJTjooooo`OoZJVY2oooool000[ooooo1OnYZJTmooooo`Go0000??ooool5ojVYZCco
oooo1OnYZJTlooooo`GoZJVY3?ooool000_ooooo0onYZJToooooo`?o0000?_ooool3ojVYZCkooooo
0onYZJTnooooo`?oZJVY3Oooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo0002ooooo`?o
00002?ooool00ol0003oooooooooo`0]ooooo`?o00001?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo02gooooo0ol00003ooooo`?o00002?ooool00ol0003oooooooooo`0]ooooo`?o
00000oooool2o`0000Wooooo00?o0000ooooooooool0;Oooool5o`0000?ooooo0_l00007ooooo`03
o`000?oooooooooo00cooooo00000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool0;_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0^ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooo
oooooooo00001_ooool00ol0003oooooooooo`0<ooooo`0000?oooooo`000?ooool00_ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo02kooooo00Go0000ooooooooooooooooo`000003
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool03?ooool00003ooooool0003o
000000;o00002?ooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00;ooooo0ol00008
ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool00_ooool4o`0000Oooooo00?o0000
ooooooooool0<?ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`0<ooooo`0000?oooooo`000?ooool02_ooool00ol0003oooooooooo`0^
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0^
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo02kooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;Oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooo
oooo00001_ooool00ol0003oooooooooo`0<ooooo`000_ooool00ol0003oooooooooo`06ooooo`?o
0000;oooool2o`0000?ooooo00Go0000ooooooooooooooooo`000006ooooo`?o0000;_ooool2o`00
00?ooooo00Go0000ooooooooooooooooo`000006ooooo`?o0000;_ooool2o`0000?ooooo00Go0000
ooooooooooooooooo`000006ooooo`?o0000;_ooool01Ol0003ooooooooooooooooo000000;ooooo
00Co0000oooooooooooo00001Oooool3o`0000gooooo0003ooooo`;o00001Oooool5o`0002oooooo
00Co0000oooooooooooooooo1Ol00005ooooo`Go0000;_ooool00ol0003oooooooooo`02ooooo`?o
00001_ooool5o`0002kooooo00?o0000ooooooooool00_ooool3o`0000Kooooo1Ol0000^ooooo`?o
00001?ooool2o`0000Gooooo1Ol0000<ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`Oo0000
>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooooooo`0000go00002oooool000Wooooo1ol0000k
ooooo`Oo0000>_ooool7o`0003[ooooo1ol0000jooooo`Oo00002oooool000[ooooo1Ol0000moooo
o`Go0000??ooool5o`0003cooooo1Ol0000looooo`Go00003?ooool000_ooooo0ol0000oooooo`?o
0000?_ooool3o`0003kooooo0ol0000nooooo`?o00003Oooool000gooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03ko
oooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooo
ooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o
0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0n
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03
o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0
?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003o
ooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo
00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003ooooooooo
o`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0noooo
o`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000oooooooo
ool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`00
0?oooooooooo03kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo03kooooo00?o0000ooooooooool0?_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo03kooooo00?o0000ooooooooool0?_ooool00ol0003oooooooooo`0nooooo`03o`000?oooooo
oooo00cooooo0002ooooo`?o00002?ooool00ol0003oooooooooo`0/ooooo`Go000000?oooooo`00
0?l000000ol00007ooooo`03o`000?oooooooooo02cooooo1Ol00002ooooo`?o00002?ooool00ol0
003oooooooooo`0/ooooo`Go00000oooool3o`0000Oooooo00?o0000ooooooooool0;Oooool5o`00
00;ooooo0ol00007ooooo`03o`000?oooooooooo00cooooo0003ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool0;Oooool01?l0003oooooooooool00002ooooo`04o`000?oooooooooo
o`0000Oooooo00?o0000ooooooooool0;Oooool01_l0003oooooooooool0003oooooo`0000?ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0]ooooo`04o`000?ooooooooooo`0000Co
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0^ooooo`06o`000?ooooooooooo`00
0?oooooo00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo0003oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool01?l0003oooooooooooooool5
o`0000Oooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool03?ooool000?ooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0_ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`0_ooooo`03o`000?oooooooooo00;ooooo0_l00008ooooo`03o`000?oooooooooo
02oooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo033ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00cooooo0003ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0;?ooool01ol0
003ooooooooooooooooo0000ooooool000000oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo02cooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;?ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool0
00002?ooool00ol0003oooooooooo`0]ooooo`06o`000?ooooooooooooooool0003ooooo1?l00007
ooooo`03o`000?oooooooooo00cooooo0002ooooo`;o00002Oooool00ol0003oooooooooo`0/oooo
o`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool0;?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo02cooooo00Go0000ooooooooooooooooo`000003oooo
o`;o00002?ooool00ol0003oooooooooo`0]ooooo`07o`000?ooooooooooooooool0003oooooo`00
000:ooooo`03o`000?oooooooooo00cooooo0003ooooo`03o`000?oooooooooo00Gooooo0ol0000_
ooooo`?o00000oooool3o`0000Oooooo0ol0000^ooooo`?o00000oooool3o`0000Oooooo0ol0000^
ooooo`?o00001Oooool00ol0003oooooooooo`05ooooo`?o0000;oooool3o`0000;ooooo1Ol00005
ooooo`?oZJVY3Oooool000[ooooo1Ol0000mooooo`Go0000??ooool5o`0003cooooo1Ol0000loooo
o`GoZJVY3?ooool000Woooooool00006o`0000OoZJVY2oooool000Wooooo1ol0000kooooo`Oo0000
>_ooool7o`0003[ooooo1ol0000jooooo`OoZJVY2oooool000Wooooo1ol0000kooooo`Oo0000>_oo
ool7o`0003[ooooo1ol0000jooooo`OoZJVY2oooool000[ooooo1Ol0000mooooo`Go0000??ooool5
o`0003cooooo1Ol0000looooo`GoZJVY3?ooool000_ooooo0ol0000oooooo`?o0000?_ooool3o`00
03kooooo0ol0000nooooo`?oZJVY3Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[stag2]]; \[IndentingNewLine]Evolve[
          stag2], {9}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[95]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[Partition[l, 3], 
        VertexStyle \[Rule] Disk[Large]];\)\)], "Input",
  CellLabel->"In[97]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0238095 0.31746 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.321429 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.0238095 0.64881 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.0238095 0.97619 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.35119 0.321429 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.35119 0.64881 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.35119 0.97619 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.678571 0.321429 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.27273 .04545 Mdot
.5 .04545 Mdot
.663 .663 .663 r
.72727 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .27273 Mdot
.27273 .27273 Mdot
.663 .663 .663 r
.5 .27273 Mdot
0 0 0 r
.72727 .27273 Mdot
.95455 .27273 Mdot
.663 .663 .663 r
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.663 .663 .663 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.678571 0.64881 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.27273 .04545 Mdot
.5 .04545 Mdot
.663 .663 .663 r
.72727 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .27273 Mdot
.27273 .27273 Mdot
.663 .663 .663 r
.5 .27273 Mdot
0 0 0 r
.72727 .27273 Mdot
.95455 .27273 Mdot
.663 .663 .663 r
.04545 .5 Mdot
0 0 0 r
.27273 .5 Mdot
.663 .663 .663 r
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
0 0 0 r
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.663 .663 .663 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.678571 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.27273 .04545 L
s
.04545 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.5 .04545 L
s
.27273 .04545 m
.27273 .27273 L
s
.5 .04545 m
.72727 .04545 L
s
.5 .04545 m
.5 .27273 L
s
.72727 .04545 m
.95455 .04545 L
s
.72727 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.27273 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.27273 .27273 m
.5 .27273 L
s
.27273 .27273 m
.27273 .5 L
s
.5 .27273 m
.72727 .27273 L
s
.5 .27273 m
.5 .5 L
s
.72727 .27273 m
.95455 .27273 L
s
.72727 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.27273 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.27273 .5 m
.5 .5 L
s
.27273 .5 m
.27273 .72727 L
s
.5 .5 m
.72727 .5 L
s
.5 .5 m
.5 .72727 L
s
.72727 .5 m
.95455 .5 L
s
.72727 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.27273 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.5 .72727 L
s
.27273 .72727 m
.27273 .95455 L
s
.5 .72727 m
.72727 .72727 L
s
.5 .72727 m
.5 .95455 L
s
.72727 .72727 m
.95455 .72727 L
s
.72727 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.04545 .95455 m
.27273 .95455 L
s
.27273 .95455 m
.5 .95455 L
s
.5 .95455 m
.72727 .95455 L
s
.72727 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.663 .663 .663 r
.95455 .04545 Mdot
0 0 0 r
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[97]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`002Oooool3
o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3
o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3
o`00013ooooo0ol0000@ooooo`?o00002oooool000SoooooD_l0000<oooooe;o00003?oooomBo`00
00[ooooo0008ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000co
oooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000ko
oooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:ooooo`002?ooool5o`0000kooooo
1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo
1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo
1Ol0000>ooooo`Go00002_ooool000Wooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0
000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0
000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000_ooooo000:
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool02Oooool000Wooooo0ol0000@ooooo`?o00004Ooo
ool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?oo
ool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?oo
ool3o`0000_ooooo0008ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5
o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5
o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:ooooo`002?oooomBo`00
00coooooD_l0000<oooooe;o00002_ooool000Sooooo1Ol0000>ooooo`Go00003oooool5o`0000ko
oooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000ko
oooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000[o
oooo0009ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo
0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo
0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000;ooooo`002_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00Wooooo0009ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o0000
4?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o0000
3_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000;ooooo`002?oo
ool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_oo
ool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003ooo
ool5o`0000kooooo1Ol0000>ooooo`Go00002_ooool000SoooooD_l0000<oooooe;o00003?oooomB
o`0000[ooooo0008ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`00
00cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`00
00kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:ooooo`002Oooool3o`00013o
oooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017o
oooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013o
oooo0ol0000@ooooo`?o00002oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool02Oooool000Wooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o0000
3_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o0000
4?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000_ooooo0008oooooe;o00003?oo
oomBo`0000coooooD_l0000:ooooo`002?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5
o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5
o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00002_ooool0
00Sooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`00
00kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`00
00oooooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0009ooooo`?o00004?ooool3o`00017o
oooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013o
oooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013o
oooo0ol0000;ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo0009ooooo`?o
00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o
00004Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o
00004?ooool3o`00013ooooo0ol0000;ooooo`002?ooool5o`0000kooooo1Ol0000?ooooo`Go0000
3_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go0000
3_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go0000
2_ooool000SoooooD_l0000<oooooe;o00003?oooomBo`0000[ooooo0008ooooo`Go00003_ooool5
o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5
o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5
o`0000kooooo1Ol0000:ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00
013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`00
00kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool00?oo
oooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oo
oooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oo
oooo8Oooool00?oooooo8Oooool000Wooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0
000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0
000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000_ooooo0008
oooooe;o00003?oooomBo`0000coooooD_l0000:ooooo`002?ooool5o`0000kooooo1Ol0000?oooo
o`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>oooo
o`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>oooo
o`Go00002_ooool000Sooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go
00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go
00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0009ooooo`?o0000
4?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o0000
4Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o0000
4?ooool3o`00013ooooo0ol0000;ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wo
oooo0009ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo
0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo
0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000;ooooo`002?ooool5o`0000kooooo1Ol0
000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0
000>ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0
000>ooooo`Go00002_ooool000SoooooD_l0000<oooooe;o00003?oooomBo`0000[ooooo0008oooo
o`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0000>oooo
o`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0000?oooo
o`Go00003_ooool5o`0000kooooo1Ol0000:ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?o
00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o
00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o
00002oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002Oooool3o`00013o
oooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017o
oooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013o
oooo0ol0000@ooooo`?o00002oooool000Sooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo
1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo
1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo
0008oooooe;o00003?oooomBo`0000coooooD_l0000:ooooo`002?ooool5o`0000kooooo1Ol0000?
ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>
ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>
ooooo`Go00002_ooool000Wooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@oooo
o`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>oooo
o`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000_ooooo000:ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo0009ooooo`?o00004?ooool3o`00017o
oooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013o
oooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013o
oooo0ol0000;ooooo`002?oooomBo`0000coooooD_l0000<oooooe;o00002_ooool000Sooooo1Ol0
000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0
000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0
000>ooooo`Go00003_ooool5o`0000[ooooo0008ooooo`Go00003_ooool5o`0000oooooo1Ol0000>
ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>
ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:
ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000>oooo
o`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`0000kooooo0ol0000@oooo
o`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool000[ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`09ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00
013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`00
00kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool000So
oooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000ko
oooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oo
oooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0008oooooe;o00003?oooomBo`0000cooooo
D_l0000:ooooo`002?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0
000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0
000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00002_ooool000Wooooo0ol0000@
ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00013ooooo0ol0000A
ooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@
ooooo`?o00004?ooool3o`0000_ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo0009ooooo`?oZJVY
4?ooool3o`00017ooooo0ol0000@ooooo`?oZJVY4?ooool3ojVYZ@kooooo0onYZJT@ooooo`?o0000
4Oooool3o`00013ooooo0onYZJT@ooooo`?oZJVY3_ooool3o`00013ooooo0ol0000Aooooo`?o0000
4?ooool3o`00013ooooo0onYZJT;ooooo`002?ooool5ojVYZCGo00001OnYZJT>o`0000GoZJVY3?oo
ool5ojVYZCGo00001OnYZJT>o`0000GoZJVY3?oooom=o`0000GoZJVY2_ooool000Sooooo1OnYZJT>
ooooo`Go00003oooool5o`0000kooooo1OnYZJT>ooooo`GoZJVY3?ooool5ojVYZ@kooooo1Ol0000?
ooooo`Go00003_ooool5ojVYZ@kooooo1OnYZJT<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>
ooooo`Go00003_ooool5ojVYZ@[ooooo0008ooooo`GoZJVY3_ooool5o`0000oooooo1Ol0000>oooo
o`GoZJVY3_ooool5ojVYZ@cooooo1OnYZJT>ooooo`Go00003oooool5o`0000kooooo1OnYZJT>oooo
o`GoZJVY3?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1OnYZJT:oooo
o`002Oooool3ojVYZA3ooooo0ol0000Aooooo`?o00004?ooool3ojVYZA3ooooo0onYZJT>ooooo`?o
ZJVY4?ooool3o`00017ooooo0ol0000@ooooo`?oZJVY4?ooool3ojVYZ@kooooo0ol0000@ooooo`?o
00004Oooool3o`00013ooooo0ol0000@ooooo`?oZJVY2oooool000[ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool0
00[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`09ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?oZJVY4?ooool3o`00013o
oooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0onYZJT@ooooo`?o00004?ooool3o`0000ko
oooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool000Sooooo
1Ol0000>ooooo`Go00003oooool5ojVYZ@kooooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo
1Ol0000?ooooo`GoZJVY3_ooool5o`0000kooooo1Ol0000<ooooo`Go00003_ooool5o`0000oooooo
1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0008ooooobOo00001OnYZJTVo`0000cooooo9ol0
0005ojVYZBKo00003?oooomBo`0000[ooooo0008ooooo`Go00003_ooool5o`0000oooooo1OnYZJT>
ooooo`Go00003_ooool5o`0000cooooo1Ol0000>ooooo`Go00003oooool5ojVYZ@kooooo1Ol0000>
ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:
ooooo`002Oooool3o`00013ooooo0ol0000Aooooo`?oZJVY4?ooool3o`00013ooooo0ol0000>oooo
o`?o00004?ooool3o`00017ooooo0onYZJT@ooooo`?o00004?ooool3o`0000kooooo0ol0000@oooo
o`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool000[ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`09ooooo`002Oooool3ojVYZA3ooooo0ol0000Aooooo`?oZJVY4?ooool3ojVY
ZA3ooooo0onYZJT>ooooo`?oZJVY4?ooool3o`00017ooooo0onYZJT@ooooo`?oZJVY4?ooool3ojVY
Z@kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00002oooool000So
oooo1OnYZJT>ooooo`Go00003oooool5ojVYZ@kooooo1OnYZJT>ooooo`GoZJVY3?ooool5ojVYZ@ko
oooo1Ol0000?ooooo`GoZJVY3_ooool5ojVYZ@kooooo1OnYZJT<ooooo`Go00003_ooool5o`0000oo
oooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0008ooooo`GoZJVY8_l00005ojVYZ@ko0000
1OnYZJT>o`0000GoZJVY3?ooool5ojVYZB;o00001OnYZJT>o`0000GoZJVY3_l00005ojVYZ@cooooo
D_l0000:ooooo`002?ooool5ojVYZ@kooooo1Ol0000?ooooo`GoZJVY3_ooool5ojVYZ@kooooo1OnY
ZJT<ooooo`GoZJVY3_ooool5o`0000oooooo1OnYZJT>ooooo`GoZJVY3_ooool5ojVYZ@cooooo1Ol0
000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00002_ooool000Wooooo0onYZJT@
ooooo`?o00004Oooool3ojVYZA3ooooo0onYZJT@ooooo`?oZJVY3_ooool3ojVYZA3ooooo0ol0000A
ooooo`?oZJVY4?ooool3ojVYZA3ooooo0onYZJT>ooooo`?o00004?ooool3o`00017ooooo0ol0000@
ooooo`?o00004?ooool3o`0000_ooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool0
00[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00Wooooo0009ooooo`?o00004?ooool3o`00017ooooo0ol0000@ooooo`?o00004?ooool3o`00
00kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@ooooo`?o00003_ooool3o`00
013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0000;ooooo`002?oooomBo`0000co
ooooD_l0000<oooooe;o00002_ooool000Sooooo1Ol0000>ooooo`Go00003oooool5o`0000kooooo
1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`0000kooooo
1Ol0000<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo
0008ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`0000cooooo1Ol0
000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`Go00003?ooool5o`0000kooooo1Ol0
000?ooooo`Go00003_ooool5o`0000kooooo1Ol0000:ooooo`002Oooool3o`00013ooooo0ol0000A
ooooo`?o00004?ooool3o`00013ooooo0ol0000>ooooo`?o00004?ooool3o`00017ooooo0ol0000@
ooooo`?o00004?ooool3o`0000kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0000@
ooooo`?o00002oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aoooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`00
0?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013o
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wo
oooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04Oooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo017ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`002Oooool3
o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0onYZJT>ooooo`?o00004?ooool3
o`00017ooooo0ol0000@ooooo`?o00004?ooool3ojVYZ@kooooo0ol0000@ooooo`?o00004Oooool3
o`00013ooooo0ol0000@ooooo`?o00002oooool000Sooooo1Ol0000>ooooo`Go00003oooool5o`00
00kooooo1Ol0000>ooooo`GoZJVY3?ooool5o`0000kooooo1Ol0000?ooooo`Go00003_ooool5o`00
00kooooo1OnYZJT<ooooo`Go00003_ooool5o`0000oooooo1Ol0000>ooooo`Go00003_ooool5o`00
00[ooooo0008ooooodgo00001OnYZJT<ooooodgo00001OnYZJT<oooooe;o00002_ooool000Sooooo
1Ol0000>ooooo`Go00003oooool5o`0000kooooo1Ol0000>ooooo`GoZJVY3?ooool5o`0000kooooo
1Ol0000?ooooo`Go00003_ooool5o`0000kooooo1OnYZJT<ooooo`Go00003_ooool5o`0000oooooo
1Ol0000>ooooo`Go00003_ooool5o`0000[ooooo0009ooooo`?o00004?ooool3o`00017ooooo0ol0
000@ooooo`?o00004?ooool3ojVYZ@kooooo0ol0000@ooooo`?o00004Oooool3o`00013ooooo0ol0
000@ooooo`?oZJVY3_ooool3o`00013ooooo0ol0000Aooooo`?o00004?ooool3o`00013ooooo0ol0
000;ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0750158, -0.0750158, \
0.0109757, 0.0109757}, {{6.8125, 92.25}, {280.125, 194.75}} -> {-0.138183, \
-0.138586, 0.0128845, 0.0128845}, {{100.75, 186.188}, {280.125, 194.75}} -> \
{-1.34852, -0.138586, 0.0128845, 0.0128845}, {{194.75, 280.125}, {280.125, \
194.75}} -> {-2.55925, -0.138586, 0.0128845, 0.0128845}, {{6.8125, 92.25}, \
{186.188, 100.75}} -> {-0.137717, -1.34797, 0.012875, 0.012875}, {{100.75, \
186.188}, {186.188, 100.75}} -> {-1.34717, -1.34797, 0.012875, 0.012875}, \
{{194.75, 280.125}, {186.188, 100.75}} -> {-2.55925, -1.34932, 0.0128845, \
0.0128845}, {{6.8125, 92.25}, {92.25, 6.8125}} -> {-0.137717, -2.55742, \
0.012875, 0.012875}, {{100.75, 186.188}, {92.25, 6.8125}} -> {-1.34717, \
-2.55742, 0.012875, 0.012875}, {{194.75, 280.125}, {92.25, 6.8125}} -> \
{-2.55925, -2.55966, 0.0128845, 0.0128845}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[stag2]\)], "Input",
  CellLabel->"In[98]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Divide the Cake", "Section",
  CellTags->"LocalInteractions:Introduction"],

Cell["Construct the payoff matrix for a cake size of 10.", "Text",
  FontSize->16,
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(dtdPM = 
        Table[\[IndentingNewLine]If[
            i + j \[LessEqual] 10, {i, j}, {0, 0}], {i, 1, 10}, {j, 1, 
            10}\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(dtdPM // TraditionalForm\)}], "Input",
  CellLabel->"In[4]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {\({1, 1}\), \({1, 2}\), \({1, 3}\), \({1, 4}\), \({1, 
                  5}\), \({1, 6}\), \({1, 7}\), \({1, 8}\), \({1, 9}\), \({0, 
                  0}\)},
              {\({2, 1}\), \({2, 2}\), \({2, 3}\), \({2, 4}\), \({2, 
                  5}\), \({2, 6}\), \({2, 7}\), \({2, 8}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({3, 1}\), \({3, 2}\), \({3, 3}\), \({3, 4}\), \({3, 
                  5}\), \({3, 6}\), \({3, 7}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({4, 1}\), \({4, 2}\), \({4, 3}\), \({4, 4}\), \({4, 
                  5}\), \({4, 6}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({5, 1}\), \({5, 2}\), \({5, 3}\), \({5, 4}\), \({5, 
                  5}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({6, 1}\), \({6, 2}\), \({6, 3}\), \({6, 4}\), \({0, 
                  0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({7, 1}\), \({7, 2}\), \({7, 3}\), \({0, 0}\), \({0, 
                  0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({8, 1}\), \({8, 2}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({9, 1}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)},
              {\({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 0}\), \({0, 
                  0}\)}
              }], "\[NoBreak]", ")"}],
        MatrixForm[ {{{1, 1}, {1, 2}, {1, 3}, {1, 4}, {1, 5}, {1, 6}, {1, 
          7}, {1, 8}, {1, 9}, {0, 0}}, {{2, 1}, {2, 2}, {2, 3}, {2, 4}, {2, 
          5}, {2, 6}, {2, 7}, {2, 8}, {0, 0}, {0, 0}}, {{3, 1}, {3, 2}, {3, 
          3}, {3, 4}, {3, 5}, {3, 6}, {3, 7}, {0, 0}, {0, 0}, {0, 0}}, {{4, 
          1}, {4, 2}, {4, 3}, {4, 4}, {4, 5}, {4, 6}, {0, 0}, {0, 0}, {0, 
          0}, {0, 0}}, {{5, 1}, {5, 2}, {5, 3}, {5, 4}, {5, 5}, {0, 0}, {0, 
          0}, {0, 0}, {0, 0}, {0, 0}}, {{6, 1}, {6, 2}, {6, 3}, {6, 4}, {0, 
          0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, {{7, 1}, {7, 2}, {7, 
          3}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, {{8, 
          1}, {8, 2}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 
          0}, {0, 0}}, {{9, 1}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 
          0}, {0, 0}, {0, 0}, {0, 0}}, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 
          0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}}, TableDepth -> 2]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[5]//TraditionalForm=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell["The following are the colors I've always used.", "Text",
  FontSize->16,
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(dtdColors = \[IndentingNewLine]{\[IndentingNewLine]RGBColor[0, 
            0,  .5], \[IndentingNewLine]RGBColor[0,  .5, 
            0], \[IndentingNewLine]RGBColor[ .5, 0, 
            0], \[IndentingNewLine]RGBColor[0, 1, 
            1], \ \[IndentingNewLine]RGBColor[1, 0, 
            1], \ \[IndentingNewLine]RGBColor[1, 1, 
            0], \[IndentingNewLine]RGBColor[ .9,  .9,  .9], \
\[IndentingNewLine]RGBColor[ .5,  .5,  .5], \[IndentingNewLine]RGBColor[0, 1, 
            0], \[IndentingNewLine]RGBColor[1, 0, 
            0]\ \[IndentingNewLine]};\)\), "\n", 
    \(\(StrategyLegend[\[IndentingNewLine]dtdColors, \
\[IndentingNewLine]Table["\<Demand \>" <> ToString[i], {i, 1, 
            10}]\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[6]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 0, 0.500008],
                Background->RGBColor[0, 0, 0.500008]],
              BoxMargins->False], \(Demand\ 1\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 0.500008, 0],
                Background->RGBColor[0, 0.500008, 0]],
              BoxMargins->False], \(Demand\ 2\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0.500008, 0, 0],
                Background->RGBColor[0.500008, 0, 0]],
              BoxMargins->False], \(Demand\ 3\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 1, 1],
                Background->RGBColor[0, 1, 1]],
              BoxMargins->False], \(Demand\ 4\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 0, 1],
                Background->RGBColor[1, 0, 1]],
              BoxMargins->False], \(Demand\ 5\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 1, 0],
                Background->RGBColor[1, 1, 0]],
              BoxMargins->False], \(Demand\ 6\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.900008],
                Background->GrayLevel[0.900008]],
              BoxMargins->False], \(Demand\ 7\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.500008],
                Background->GrayLevel[0.500008]],
              BoxMargins->False], \(Demand\ 8\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 1, 0],
                Background->RGBColor[0, 1, 0]],
              BoxMargins->False], \(Demand\ 9\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 0, 0],
                Background->RGBColor[1, 0, 0]],
              BoxMargins->False], \(Demand\ 10\)}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[6]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["On a ring", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(dtd1 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] Cycle[200], \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors, \[IndentingNewLine]WindowLabel \[Rule] "\<Divide the \
cake on a ring\>"\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[103]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ToColorArray[dtd1]]; \[IndentingNewLine]Evolve[
          dtd1], {30}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[104]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]Graphics[\[IndentingNewLine]RasterArray[\
\[IndentingNewLine]Reverse[l]\[IndentingNewLine]]\[IndentingNewLine]], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[106]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .15 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.00357143 0.0047619 [
[ 0 0 0 0 ]
[ 1 .15 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .15 L
0 .15 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .00357 translate
.95238 .14286 scale
600 string
200 30 8 [200 0 0 30 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFE5E5E5FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF7F0000FF00FFFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFE5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF7F0000FF00FF7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF7F0000FF00FF7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FF007F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000FF00FF7F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
E5E5E5E5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FF0000FF007F7F7F7F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
7F0000FF00FFFF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00E5E5E500FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5
E5E5E5E5E5E5FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
7F7F7F00FF0000FF007F7F7F7F7F7F7F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FFE5E5E5FF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00E5E5E5E5E5E5
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFE5E5E500FF00
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5
E5E5E5E5E5E57F7F7FFFFF007F000000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF7F7F7F00FFFF
7F7F7F00FF0000FF007F7F7F7F7F7F7F7F7F7F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F000000FF00
FF00FFE5E5E5FF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F00007F0000
7F0000FF00FFFF00FFFF00FFFF00FFFF00FF7F0000FFFF00FFFF00FFFF00E5E5E5E5E5E5
E5E5E500FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF007F0000FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F000000FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00007F00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFE5E5E5E5E5E500FF00
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
007F007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00E5E5E5E5E5E5
E5E5E5E5E5E57F7F7FFFFF007F7F7F7F000000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF7F7F7F7F7F7F00FFFF
7F7F7F00FF0000FF007F7F7F7F7F7F7F7F7F7F7F7F7F7F7F00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFFFF00E5E5E500FF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F000000FF0000FF00
FF00FFE5E5E5FF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF7F00007F00007F0000
7F00007F0000FF00FFFF00FFFF00FF7F0000FF0000FFFF00FFFF00FFFF00E5E5E5E5E5E5
E5E5E5E5E5E500FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF007F00007F0000FF00
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF7F000000FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F00007F0000007F0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00007F007F00007F7F7FFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFE5E5E5E5E5E5E5E5E500FF00
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5007F00
7F0000FF000000007FFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00E5E5E5E5E5E5
E5E5E5E5E5E57F7F7FFFFF007F7F7F7F0000FF000000FFFF00FFFF00FFFF00FFFFFFFF00
FFFF00FFFF00FFFF007F000000FFFF00FFFF00FFFF00FFFF7F7F7F7F7F7F7F7F7F00FFFF
7F7F7F00FF0000FF007F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F00FFFF00FFFF00FFFF
00FFFF00FFFFE5E5E5FFFF00E5E5E500FF0000FF0000FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFE5E5E500FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FF00FF007F000000FF0000FF00
FF00FFE5E5E5FF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FFFF00
FFFF00FFFF00FFFF00007F0000FF00FF00FFFF00FFFF00FF7F00007F00007F00007F0000
7F00007F00007F0000FF00FF7F000000FF00FF0000FFFF00FFFF00FFFF007F0000E5E5E5
E5E5E5E5E5E57F7F7F00FFFF00FFFFFF00FFFF00FFFF00FF007F00007F0000FF0000FF00
00FF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFFFF007F7F7F7F000000FFFF00FFFF
00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FF7F00007F000000FF00FF0000007F00
00FFFFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00007F007F00007F7F7F7F7F7F7F7F7F
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FF0000FFFF00FFFF
00FFFF00FFFF00FFFF7F0000E5E5E5E5E5E5E5E5E500FF00
00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5007F007F0000
00FF00FF000000007FFF0000FF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F0000E5E5E5
E5E5E5E5E5E57F7F7FFFFF007F7F7F7F0000FF0000E5E5E500FFFF00FFFF007F00007F00
FFFF00FFFF00FFFF007F00007F000000FFFF00FFFF7F7F7F7F7F7F7F7F7F7F7F7F00FFFF
7F7F7F00FF0000FF007F7F7F7F7F7F7F7F7F007F007F7F7F7F7F7F7F7F7F00FFFF00FFFF
00FFFFE5E5E5E5E5E5FFFF00E5E5E500FF0000FF0000FF0000FFFF00FFFF00FFFF00FFFF
7F000000FFFF00FFFF00FFFFE5E5E5E5E5E5E5E5E500FFFF00FFFF00FFFF7F000000FFFF
00FFFFFFFF00FFFF00FFFF00FF00FFFF00FF7F7F7FFF00FF00FF007F000000FF0000FF00
FF00FFE5E5E5FF00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF007F00007F7F7FFFFF00
FFFF00FFFF00007F0000FF0000FF0000FF00FF00FF007F0000FF007F00007F00007F0000
7F00007F0000FF00FF7F7F7F7F000000FF00FF0000FFFF00FFFF00FFFF007F00007F0000
E5E5E57F7F7F7F7F7F7F7F7F00FFFF00FFFFFF00FF007F00007F00007F0000007F00FF00
FF00FFFFFF0000FFFF00FFFF00FFFFFF00FF00FF00FFFF007F7F7FFF00007F000000FFFF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FF7F000000FF007F000000FF00FF0000007F00
00FF0000FFFFFF00FFFFFF00FFFF00FFFF00007F007F000000FF00007F007F7F7F7F7F7F
7F7F7FFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FF0000FF0000FF0000FFFF
00FFFF7F7F7F7F7F7F00FFFF7F0000E5E5E5E5E5E500FF00
00FF0000007FE5E5E5E5E5E500007FFFFF007F0000007F00E5E5E5007F00FF00007F0000
00FF00FF000000007FFF000000FF00FF00FFFF00FFFF000000FF00FFFF00007F007F0000
E5E5E500FFFF7F7F7FFFFF007F7F7F7F0000FF0000E5E5E57F7F7F00FFFF00007F007F00
007F00FFFF007F000000FF007F00007F000000FFFF007F007F7F7FFF00FF7F7F7F00FFFF
7F7F7FFF000000FF0000007F7F7F7F00007FFF0000007F007F7F7F007F0000007F00FFFF
007F00E5E5E5E5E5E5FFFF007F0000E5E5E500FF0000007F00FFFF007F0000FFFFFF0000
00FF007F000000FFFF00007F007F00E5E5E5FF00FFFF000000FFFF7F00007F7F7F7F0000
00FFFF7F7F7FFFFF0000007FFF00FF7F7F7F7F7F7FFF00FF00FF007F000000FF0000FF00
FF00FFE5E5E5FF0000E5E5E57F7F7F00007F7F0000FFFF007F000000FF007F7F7F7F7F7F
FFFF0000007F007F0000007F00FF00FF00FF007F0000FF0000FF00FF00007F000000007F
7F00007F7F7FFF00FF7F7F7F7F000000FF00FF000000FF00FFFF0000007F7F000000FF00
E5E5E57F7F7F7F7F7F00007F00FFFFFF00FF00FF00007F00007F0000007FFF000000FF00
FF00FFE5E5E5FFFF0000FFFFFF00FF7F7F7F00FF00FFFF007F7F7FFF00007F00007F7F7F
00FFFFFFFF007F00007F0000FF00FF7F00007F7F7F00FF007F000000FF00FF0000007F00
00FF00FF000000FFFFFF00FFFFFF00007F007F00007F7F7F00FF00FF0000007F007F7F7F
00FF00007F00FF00FF007F00007F00FFFF007F7F7F00FFFF00007F00FF0000007F00FFFF
E5E5E57F7F7F7F7F7FFF000000FFFF7F0000E5E5E500FFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[106]:=",
  ImageSize->{439.562, 65.9375},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006g0000@B000`400?l00000o`00003ooooooonh
ooooo`00oooooonhooooo`003OoooomM0?l0obD00?ooE03o0?l40?oo0080O`001@3oo`2k0?l0o`ko
oooo000=oooooed0o`3o9@00oomD0?l0o`@0ool00P1o00050?oo0;/0o`3o3_ooool000goooooF`3o
0?lY003ooe80o`3o103oo`0207l000D0ool0^`3o0?l>ooooo`003OoooomK0?l0obT00?ooDP3o0?l4
0?oo0080O`001@3oo`2k0?l0o`kooooo000=oooooeT0o`3o;@00oom@0?l0o`@0ool00P1o00050?oo
0;/0o`3o3_ooool000goooooF@3o0?l]003ooe00o`3o103oo`0207l000D0ool0^`3o0?l>ooooo`00
3OoooomG0?l0obd00?oo0P3oo`1@0?l0o`@0ool00P1o00050?oo0;/0o`3o3_ooool000goooooE`3o
0?l]003oo`80ool0D03o0?l40?oo0080O`001@3oo`2k0?l0o`kooooo000=oooooe@0o`3o;P00ool6
0?oo04h0o`3o103oo`0207l000D0ool0^`3o0?l>ooooo`003OoooomD0?l0obh00?oo1P3oo`1>0?l0
o`@0ool00P1o00050?oo0;/0o`3o3_ooool000goooooDP3o0?lb003oo`80ool00P00oom>0?l0o`@0
ool00P1o00050?oo0;/0o`3o3_ooool000goooooDP3o0?lb003oo`80ool00P00oom>0?l0o`@0ool0
0P1o00050?oo0;/0o`3o3_ooool000goooooD03o0?lb003oo`H0ool00P00oom<0?l0o`@0ool00P1o
00050?oo0;/0o`3o3_ooool000goooooD03o0?lb003oo`H0ool00P00oom<0?l0o`@0ool00P1o0005
0?oo0;/0o`3o3_ooool000goooooCP3o0?lf003oo`80ool01P00oom:0?l0o`@0ool00P1o00050?oo
0;/0o`3o3_ooool000goooooCP3o0?lf003oo`80ool01P00oom:0?l0o`@0ool00P1o00050?oo0;/0
o`3o3_ooool000goooooCP3o0?lf003oo`80ool01P00oom:0?l0o`@0ool00P1o00050?oo0;/0o`3o
3_ooool000goooooC03o0?lf003oo`H0ool01P00oom80?l0o`@0ool00P1o00050?oo0;/0o`3o3_oo
ool000goooooC03o0?lf003oo`H0ool01P00oom80?l0o`@0ool00P1o00050?oo0;/0o`3o3_ooool0
00goooooC03o0?l20?oo03H00?oo0P3oo`0:003oodH0o`3o103oo`0207l000D0ool0^`3o0?l>oooo
o`003Ooooom<0?l0o`80ool0=P00ool20?oo00X00?ooAP3o0?l40?oo0080O`001@3oo`2k0?l0o`ko
oooo000=ooooodX0o`3o1P3oo`0b003oo`H0ool02P00oom40?l0o`@0ool00P1o00050?oo0;/0o`3o
3_ooool000goooooBP3o0?l60?oo03800?oo1P3oo`0:003ood@0o`3o103oo`0207l000D0ool0^`3o
0?l>ooooo`003Oooool2003oodH0o`3o1000ool20?oo03H00?oo0P3oo`0>003ood80o`3o103oo`02
07l000D0ool0^`3o0?l>ooooo`003Oooool2003oodH0o`3o1000ool20?oo03H00?oo0P3oo`0>003o
od80o`3o103oo`0207l000D0ool0^`3o0?l>ooooo`003Oooool4003ood80o`3o1000ool60?oo0380
0?oo1P3oo`0>003ood00o`3o103oo`0207l000D0ool0^@3o0?l2003oo`kooooo000=ooooo`@00?oo
@P3o0?l4003oo`H0ool0<P00ool60?oo00h00?oo@03o0?l40?oo0080O`001@3oo`2i0?l0o`800?oo
3_ooool000gooooo1P00ooln0?l0o`P00?oo0P3oo`0f003oo`80ool04P00ooln0?l0o`@0ool00P1o
00050?oo0;L0o`3o1000ool>ooooo`003Oooool6003ooch0o`3o2000ool20?oo03H00?oo0P3oo`0B
003ooch0o`3o103oo`0207l000D0ool0]`3o0?l4003oo`kooooo000=ooooo`P00?oo>P3o0?l8003o
o`H0ool0<P00ool60?oo01<00?oo>`3o0?l40?oo0080O`001@3oo`2e0?l0o`H00?oo3_ooool000go
oooo2000oolj0?l0o`P00?oo1P3oo`0b003oo`H0ool04`00oolk0?l0o`@0ool00P1o00050?oo0;D0
o`3o1P00ool>ooooo`003Oooool:003oocH0o`3o3000ool20?oo03H00?oo0P3oo`0G003oocT0o`3o
103oo`0207l000D0ool0/`3o0?l8003oo`kooooo000=ooooo`X00?oo=P3o0?l<003oo`80ool0=P00
ool20?oo01L00?oo>@3o0?l40?oo0080O`001@3oo`2c0?l0o`P00?oo3_ooool000gooooo3000oolb
0?l0o``00?oo1P3oo`0b003oo`H0ool05`00oole0?l0o`80O`00103oo`0207l000D0ool0/@3o0?l:
003oo`kooooo000=ooooo``00?oo<P3o0?l<003oo`H0ool0<P00ool60?oo01L00?oo=@3o0?l207l0
00@0ool00P1o00050?oo0;40o`3o2P00ool>ooooo`003Oooool:003oo`80ool0<03o0?l@003oo`80
ool0=P00ool20?oo01/00?oo<@3o0?l207l00080o`3o103oo`0207l000D0ool0[`3o0?l<003oo`ko
oooo000=ooooo`X00?oo0P3oo`0`0?l0oa000?oo0P3oo`0f003oo`80ool06`00oola0?l0o`80O`00
0P3o0?l40?oo0080O`001@3oo`2_0?l0o``00?oo3_ooool000gooooo2000ool40?oo02h0o`3o4000
ool60?oo03800?oo1P3oo`0K003oobd0o`3o0P1o00020?l0o`H0ool00P1o00050?oo0:d0o`3o3P00
ool>ooooo`003Oooool8003oo`@0ool0;P3o0?l@003oo`H0ool0<P00ool60?oo01/00?oo;@3o0?l2
07l00080o`3o1P3oo`0207l000D0ool0[@3o0?l>003oo`kooooo000=ooooo`H00?oo0P3UiND60?oo
02X0o`3o5000ool20?oo03H00?oo0P3oo`0O003oobT0o`3o0P1o00020?l0o`P0ool00P1o00050?oo
0:/0o`3o4000ool>ooooo`003Oooool6003oo`80iNGU1P3oo`0Z0?l0oa@00?oo0P3oo`0f003oo`80
ool07`00oolY0?l0o`80O`000P3o0?l80?oo0080O`001@3oo`2[0?l0oa000?oo3_ooool000gooooo
1000ool40>GUi@H0ool00P3UiNDV0?l0oa@00?oo1P3oo`0b003oo`H0ool07`00oolU0?l0o`80O`00
0P3o0?l207moO`P0ool00P1o00050?oo0:P0o`3o4`00ool>ooooo`003Oooool4003oo`@0iNGU1P3o
o`020>GUiBH0o`3o5000ool60?oo03800?oo1P3oo`0O003oobD0o`3o0P1o00020?l0o`80Ogmo203o
o`0207l000D0ool0Z03o0?lC003oo`kooooo000=ooooo`800?oo0P00o`040>GUi@H0ool01@3UiNDQ
0?l0oaP00?oo0P3oo`0f003oo`80ool08`00oolQ0?l0o`80O`000P3o0?l407moO`P0ool00P1o0005
0?oo02T0o`3o1000oomi0?l0oaD00?oo3_ooool000gooooo0P00ool2003o00@0iNGU1P3oo`050>GU
iB40o`3o6000ool20?oo03H00?oo0P3oo`0S003oob40o`3o0P1o00020?l0o`@0Ogmo203oo`0207l0
00D0ool0:@3o0?l4003oogT0o`3o5@00ool>ooooo`003Oooool4003o00@0iNGU1P3oo`070>GUiAl0
o`3o0P3oo`0D003oo`H0ool05`00ool2003o0080Ogmo5`00ool60?oo02<00?oo7@3o0?l207l00080
o`3o1P1oOgl80?oo0080O`001@3oo`0Y0?l0o`80ool01000oolK0?l0o`@00?ooEP3o0?lG003oo`ko
oooo000=ooooo`@00?l0103UiND60?oo00L0iNGU7`3o0?l20?oo01@00?oo1P3oo`0G003oo`800?l0
0P1oOglG003oo`H0ool08`00oolM0?l0o`80O`000P3o0?l607moO`P0ool00P1o00050?oo02T0o`3o
0P3oo`04003ooa/0o`3o1000oomF0?l0oaL00?oo3_ooool000gooooo1000o`040>GUi@H0ool01`3U
iNDO0?l0o`80ool05000ool60?oo01L00?oo0P00o`0207moOaL00?oo1P3oo`0S003ooad0o`3o0P1o
00020?l0o`H0Ogmo203oo`0207l000D0ool0:@3o0?l20?oo00@00?oo6`3o0?l4003ooeH0o`3o5`00
ool>ooooo`003Oooool4003o00@0iNGU1P3oo`090>GUiAX0o`3o1@3UiND20?oo01@00?oo0P3oo`0G
003oo`@00?l0101oOglG003oo`80ool09`00oolI0?l0o`80O`000P3o0?l20?l000H0Ogmo203oo`02
07l000D0ool09`3o0?l40?oo0080iNGU1000oolG0?l0o`P00?ooDP3o0?lG003oo`800?l03_ooool0
00gooooo1000o`040>GUi@H0ool02@3UiNDJ0?l0o`D0iNGU0P3oo`0D003oo`80ool05`00ool4003o
00@0Ogmo5`00ool20?oo02L00?oo6@3o0?l207l00080o`3o0P3o000607moO`P0ool00P1o00050?oo
02L0o`3o103oo`020>GUi@@00?oo5`3o0?l8003ooe80o`3o5`00ool2003o00kooooo000=ooooo`@0
0?l0103UiND60?oo00/0iNGU5P3o0?l70>GUi@@0ool04000ool60?oo01<00?oo0P1oOgl4003o00H0
Ogmo4`00ool60?oo02L00?oo5@3o0?l207l00080o`3o0P3UiND20?l000H0Ogmo203oo`0207l000L0
ool04P3o0?l207l000l0o`3o1P3oo`040>GUi@@00?oo4`3o0?l<003oo`X0o`3o1000oolX0?l0o`@0
ool0503o0?lG003oo`80iNGU0P00o`0>ooooo`003Oooool4003o00@0iNGU1P3oo`0;0>GUiAH0o`3o
1`3UiND40?oo01000?oo1P3oo`0C003oo`80Ogmo1000o`0607moOa<00?oo1P3oo`0W003ooaD0o`3o
0P1o00020?l0o`80iNGU0P3o000607moO`P0ool00P1o00070?oo0180o`3o0P1o000?0?l0o`H0ool0
103UiND4003ooa<0o`3o3000ool:0?l0o`@00?oo:03o0?l40?oo01@0o`3o5`00ool20>GUi@800?l0
3_ooool000gooooo1000o`040>GUi@H0ool03@3UiNDB0?l0o`T0iNGU0P1oOgl20?oo0080O`004000
ool20?oo01400?oo0P1oOgl2003oo`80Ogmo1000o`0807moOa<00?oo0P3oo`0[003ooa40o`3o0P1o
0002003o0080o`3o0P3UiND20?l000H0Ogmo203oo`0207l000T0ool03P3o0?l607l000/0o`3o0P1o
00060?oo00H0iNGU1000ool?0?l0o`8007l03P00ool60?l0o`P00?oo4@3o0?l207l000800?oo3P3o
0?l70?oo008007l04P3o0?l20?oo01<00?oo103UiND2003o00kooooo000=ooooo`@00?l0103UiND6
0?oo00d0iNGU4P3o0?l90>GUi@80Ogmo0P3oo`0207l001000?oo0P3oo`0A003oo`80Ogmo0P00ool2
07moO`@00?l0201oOglC003oo`80ool0:`00oolA0?l0o`80O`000P00o`020?l0o`80iNGU0P3o0006
07moO`P0ool00P1o00090?oo00h0o`3o1P1o000;0?l0o`80O`001P3oo`060>GUi@@00?oo3`3o0?l2
001o00h00?oo1P3o0?l8003ooa40o`3o0P1o0002003oo`h0o`3o1`3oo`02001o0180o`3o0P3oo`0C
003oo`@0iNGU0P00o`0>ooooo`003Oooool4003o00@0iNGU1P3oo`0;0>GUi@8007l00P1o000>0?l0
o`80ool02@3UiND207moO`80ool00P1oOgl207l000`00?oo1P3oo`0=003oo`@0Ogmo0P00ool207mo
O`@00?l02P1oOgl?003oo`80ool00P3UiND2003o02/00?oo3@3o0?l207l000@00?l00P3o0?l20>GU
i@80o`001P1oOgl80?oo0080O`002`3oo`0:0?l0o`X0O`001`3o0?l207l00080o`001P3oo`080>GU
i@@00?oo2P3o0?l5001o00800?l03P00ool20?l0o`80O`002P00ool=0?l0o`@0O`000P00O`02003o
o`X0o`3o1`3oo`02001o0080O`000P1oOgl>0?l0o`H0ool03`00ool60>GUi@800?l03_ooool000go
oooo1000o`040>GUi@H0ool02`3UiND2001o0080O`003P3o0?l20?oo00T0iNGU0P1oOgl20?oo0080
Ogmo0P1o000<003oo`H0ool03@00ool407moO`800?oo0P1oOgl4003o00X0Ogmo3`00ool20?oo0080
iNGU0P00o`0[003oo`d0o`3o0P1o0004003o0080o`3o0P3UiND20?l000H0Ogmo203oo`0207l000/0
ool02P3o0?l:07l000L0o`3o0P1o00020?l000H0ool0203UiND4003oo`X0o`3o1@00O`02003o00h0
0?oo0P3o0?l207l000X00?oo3@3o0?l407l0008007l00P00ool:0?l0o`L0ool00P00O`0207l00080
Ogmo3P3o0?l60?oo00l00?oo1P3UiND2003o00kooooo000=ooooo`@00?l0103UiND60?oo00T0iNGU
0P00O`0207l00080o`000P0007l:0?l0o`@0ool02@3UiND207moO`80ool00P1oOgl207l00080o`00
2000ool80?oo0080O`002@00ool607moO`800?oo0P1oOgl4003o00`0Ogmo2`00ool20>GUi@80ool0
0P3UiND4003o01D00?oo0P3UiND@003oo`80ool02`3o0?l2003o0080O`001000o`020?l0o`80iNGU
0P3o000607moO`P0ool00P1o00090?oo008007l00P00o`060?l0o`h0O`000P3o0?l307l000800?l0
0P3o00060?oo0080O`001P3UiND207moO`@00?oo1P3o0?l5001o00H00?l02P00ool20?l0o`80ool0
0P1oOgl207l000H00?oo0P3oo`0;0?l0o`@0O`000P00o`020?l0008007l00P00ool60?l0o`L0ool0
0P00O`0207l000H0Ogmo2P3o0?l60?oo00800?oo0P00o`0;003oo`80O`001P3UiND2003o00kooooo
000=ooooo`@00?l0103UiND60?oo00T0iNGU0P00O`0207l00080o`000P0007l:0?l0o`@0ool02@3U
iND207moO`80ool00P1oOgl207l00080o`002000ool80?oo0080O`002@00ool607moO`800?oo0P1o
Ogl4003o00`0Ogmo2`00ool20>GUi@80ool00P3UiND4003o01D00?oo0P3UiND@003oo`80ool02`3o
0?l2003o0080O`001000o`020?l0o`80iNGU0P3o000607moO`P0ool00P1o00090?oo008007l00P00
o`060?l0o`h0O`000P3o0?l307l000800?l00P3o00060?oo0080O`001P3UiND207moO`@00?oo1P3o
0?l5001o00H00?l02P00ool20?l0o`80ool00P1oOgl207l000H00?oo0P3oo`0;0?l0o`@0O`000P00
o`020?l0008007l00P00ool60?l0o`L0ool00P00O`0207l000H0Ogmo2P3o0?l60?oo00800?oo0P00
o`0;003oo`80O`001P3UiND2003o00kooooo000=ooooo`@00?l0103UiND60?oo00L0iNGU0P00O`02
07l000800?l00P3o00020000O`80o`00203o0?l40?oo0080O`001`3UiND207moO`80ool00P1oOgl2
07l00080o`000P3UiND4003oo`@007l01P3oo`0407l000D00?oo201oOgl2003oo`80Ogmo1000o`06
07moO`8007l01P1oOgl7003oo`@0iNGU0P3oo`020>GUi@H00?l02000ool207l000L00?oo1P3UiND6
003oo`80O`001000ool60?oo00@0o`3o0P1oOgl30?l0o`800?l00P1o0004003o0080o`3o0P3UiND2
0?l000H0Ogmo1P3oo`0207l00080Ogmo1`3oo`02001o00H00?l00P3o0?l2001o00800?l02P1o0002
0?l0o`80Ogmo0`1o0002003o0080o`001P3oo`0407l00080iNGU1P1oOgl4003oo`80o`3o1`00O`02
0000O`800?l00P3o0?l20?oo00H00?oo0P3o0?l2003o0080ool00P1oOgl20?l00080O`000P00ool7
0?oo00H0o`3o0P1o0002003o0080O`000P00o`020?l0008007l00P00o`02003oo`80o`3o1P3oo`03
001o0080O`000P00o`02001o00H0Ogmo1P3o0?l60?oo00800?oo1P00o`05003oo`@0Ogmo0P00ool2
07l000@0iNGU0P00o`0>ooooo`003Oooool4003o00@0iNGU1P3oo`070>GUi@8007l00P1o0002003o
0080o`000P0007l20?l000P0o`3o103oo`0207l000L0iNGU0P1oOgl20?oo0080Ogmo0P1o00020?l0
0080iNGU1000ool4001o00H0ool0101o0005003oo`P0Ogmo0P00ool207moO`@00?l01P1oOgl2001o
00H0Ogmo1`00ool40>GUi@80ool00P3UiND6003o00P00?oo0P1o0007003oo`H0iNGU1P00ool207l0
00@00?oo1P3oo`040?l0o`80Ogmo0`3o0?l2003o0080O`001000o`020?l0o`80iNGU0P3o000607mo
O`H0ool00P1o000207moO`L0ool00P00O`06003o0080o`3o0P00O`02003o00X0O`000P3o0?l207mo
O`<0O`000P00o`020?l000H0ool0101o00020>GUi@H0Ogmo1000ool20?l0o`L007l00P0007l2003o
0080o`3o0P3oo`06003oo`80o`3o0P00o`020?oo0080Ogmo0P3o000207l000800?oo1`3oo`060?l0
o`80O`000P00o`0207l000800?l00P3o0002001o00800?l00P00ool20?l0o`H0ool00`00O`0207l0
00800?l00P00O`0607moO`H0o`3o1P3oo`02003oo`H00?l01@00ool407moO`800?oo0P1o00040>GU
i@800?l03_ooool000gooooo0P00o`020000O`@0iNGU0P0007l20?oo0080O`000P00O`030>GUi@80
07l00P3o000207l000800?l00P3o00020000O`80o`000P00o`040?l0o`80o`000P00o`020?oo0080
07l00P1o00030>GUi@800?oo0P1oOgl20?oo0080Ogmo0P1o00020?l00080iNGU0P1oOgl2003oo`80
001o1000O`020?oo0080O`000P00o`0507l000800?oo0P00O`0207moO`80o`3o0P1oOgl2003oo`80
Ogmo0P3o0002003o0080001o0P1oOgl20000O`80o`000P00O`0207moO`8007l00`0007l2003oo`80
07l0103UiND20?oo0080O`000P3UiND2003o0080001o0P00ool2001o00800?oo0P3o0002003o0080
O`000`00ool20000O`8007l00P3UiND20?l0o`80o`000P00ool207l00080Ogmo0P1o0002003oo`80
Ogmo0P3oo`020000O`80o`3o101oOgl30?l0o`800?l00P1o0004003o0080o`3o0P3UiND20?l00080
iNGU0P1oOgl20000O`80O`000P3oo`0207l000800?l0101oOgl30?oo0080001o0P00O`020000O`80
0?l00P3o0?l2001o00@00?l00P3o000207l00080001o0P1o000207moO`80o`3o0P1oOgl307l00080
0?l00P3o0002003o0080ool00P0007l207l000800?l00P3UiND407moO`80001o0P00ool20?l0o`80
0?l01000O`030000O`80o`000P00o`020?l0o`80iNGU0P3oo`02003oo`80o`3o0P1oOgl2003o0080
ool00P1oOgl20?l00080O`000P1oOgl2003oo`80ool01@1o00020?l0o`80O`000P1oOgl2003o0080
O`000P00o`020?l0008007l00P00o`020?l000800?oo0P3o0?l20?oo008007l00`1o000207moO`80
0?l00P3o0002001o0080Ogmo0P00o`02001o0080o`3o1000O`020?oo0080Ogmo0P00ool20000O`80
0?l00P0007l3003oo`80iNGU101oOgl20?l000800?oo0P1o00020>GUi@800?oo3_ooool000gooooo
0P00o`020000O`@0iNGU0P0007l20?oo0080O`000P00O`030>GUi@8007l00P3o000207l000800?l0
0P3o00020000O`80o`000P00o`040?l0o`80o`000P00o`020?oo008007l00P1o00030>GUi@800?oo
0P1oOgl20?oo0080Ogmo0P1o00020?l00080iNGU0P1oOgl2003oo`80001o1000O`020?oo0080O`00
0P00o`0507l000800?oo0P00O`0207moO`80o`3o0P1oOgl2003oo`80Ogmo0P3o0002003o0080001o
0P1oOgl20000O`80o`000P00O`0207moO`8007l00`0007l2003oo`8007l0103UiND20?oo0080O`00
0P3UiND2003o0080001o0P00ool2001o00800?oo0P3o0002003o0080O`000`00ool20000O`8007l0
0P3UiND20?l0o`80o`000P00ool207l00080Ogmo0P1o0002003oo`80Ogmo0P3oo`020000O`80o`3o
101oOgl30?l0o`800?l00P1o0004003o0080o`3o0P3UiND20?l00080iNGU0P1oOgl20000O`80O`00
0P3oo`0207l000800?l0101oOgl30?oo0080001o0P00O`020000O`800?l00P3o0?l2001o00@00?l0
0P3o000207l00080001o0P1o000207moO`80o`3o0P1oOgl307l000800?l00P3o0002003o0080ool0
0P0007l207l000800?l00P3UiND407moO`80001o0P00ool20?l0o`800?l01000O`030000O`80o`00
0P00o`020?l0o`80iNGU0P3oo`02003oo`80o`3o0P1oOgl2003o0080ool00P1oOgl20?l00080O`00
0P1oOgl2003oo`80ool01@1o00020?l0o`80O`000P1oOgl2003o0080O`000P00o`020?l0008007l0
0P00o`020?l000800?oo0P3o0?l20?oo008007l00`1o000207moO`800?l00P3o0002001o0080Ogmo
0P00o`02001o0080o`3o1000O`020?oo0080Ogmo0P00ool20000O`800?l00P0007l3003oo`80iNGU
101oOgl20?l000800?oo0P1o00020>GUi@800?oo3_ooool00?oooooo^?ooool00001\
\>"],
  ImageRangeCache->{{{0, 438.562}, {64.9375, 0}} -> {-6.37041, -0.750159, \
0.485087, 0.485087}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[dtd1]\)], "Input",
  CellLabel->"In[107]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["On a ring with dissatisfaction-driven random adoption", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(dtd2 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] Cycle[30], \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors, \[IndentingNewLine]WindowLabel \[Rule] "\<Divide the \
cake on a ring\>"\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[108]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Need to modify the definition of ",
  StyleBox["RandomAdoptionWhenDissatisfied", "Input"],
  " to take into account the different number of strategies for this game."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(RandomAdoptionWhenDissatisfied[agent_]\), ":=", 
        RowBox[{"Module", "[", 
          RowBox[{\({pairs, max, top, newS}\), ",", "\[IndentingNewLine]", 
            
            RowBox[{\(pairs = CreatePairs[agent]\), ";", 
              "\[IndentingNewLine]", \(max = Max[Map[First, pairs]]\), ";", 
              StyleBox[\( (*Find\ max\ score*) \),
                FontColor->RGBColor[0, 0, 1]], "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
                RowBox[{\(max > Score[agent]\), ",", 
                  StyleBox[\( (*If\ I' m\ dissatisfied ... *) \),
                    FontColor->RGBColor[0, 0, 1]], 
                  "\[IndentingNewLine]", \(SetNextStrategy[agent, 
                      Random[Integer, {1, 10}]];\)}], "\[IndentingNewLine]", 
                "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
  CellLabel->"In[109]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(Map[\[IndentingNewLine]SetLearningRule[#, 
            RandomAdoptionWhenDissatisfied] &, \[IndentingNewLine]Agents[
          dtd2]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[110]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(bar = 
        ConstructProgressBar[
          100, "\<Generations completed - dtd2\>"];\)\), \
"\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          StrategyHistogram[dtd2]]; \[IndentingNewLine]Evolve[
          dtd2]; \[IndentingNewLine]IncrementProgressBar[
          bar], {100}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[111]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell["\<\
In the last run of the above on my computer, we wound up at a final \
state where everyone earned an average payoff of zero.\
\>", "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(l2 = 
        Table[\[IndentingNewLine]Map[\((#[\([i]\)])\) &, 
            l], \[IndentingNewLine]{i, 1, 
            10}\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[114]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(MultipleListPlot[
        l2, \[IndentingNewLine]PlotJoined \[Rule] 
          True, \[IndentingNewLine]SymbolShape \[Rule] 
          None, \[IndentingNewLine]PlotStyle \[Rule] 
          dtdColors, \[IndentingNewLine]Background \[Rule] 
          RGBColor[0, 0, 0], \[IndentingNewLine]PlotRange \[Rule] 
          All\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[115]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Background color
0 0 0 r
MFill
% Scaling calculations
0.0141895 0.00962001 0.0147151 0.0452772 [
[.20659 .00222 -6 -9 ]
[.20659 .00222 6 0 ]
[.39899 .00222 -6 -9 ]
[.39899 .00222 6 0 ]
[.59139 .00222 -6 -9 ]
[.59139 .00222 6 0 ]
[.78379 .00222 -6 -9 ]
[.78379 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.00169 .10527 -6 -4.5 ]
[.00169 .10527 0 4.5 ]
[.00169 .19582 -6 -4.5 ]
[.00169 .19582 0 4.5 ]
[.00169 .28638 -6 -4.5 ]
[.00169 .28638 0 4.5 ]
[.00169 .37693 -6 -4.5 ]
[.00169 .37693 0 4.5 ]
[.00169 .46749 -12 -4.5 ]
[.00169 .46749 0 4.5 ]
[.00169 .55804 -12 -4.5 ]
[.00169 .55804 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
1 1 1 r
.25 Mabswid
[ ] 0 setdash
.20659 .01472 m
.20659 .02097 L
s
[(20)] .20659 .00222 0 1 Mshowa
.39899 .01472 m
.39899 .02097 L
s
[(40)] .39899 .00222 0 1 Mshowa
.59139 .01472 m
.59139 .02097 L
s
[(60)] .59139 .00222 0 1 Mshowa
.78379 .01472 m
.78379 .02097 L
s
[(80)] .78379 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.06229 .01472 m
.06229 .01847 L
s
.11039 .01472 m
.11039 .01847 L
s
.15849 .01472 m
.15849 .01847 L
s
.25469 .01472 m
.25469 .01847 L
s
.30279 .01472 m
.30279 .01847 L
s
.35089 .01472 m
.35089 .01847 L
s
.44709 .01472 m
.44709 .01847 L
s
.49519 .01472 m
.49519 .01847 L
s
.54329 .01472 m
.54329 .01847 L
s
.63949 .01472 m
.63949 .01847 L
s
.68759 .01472 m
.68759 .01847 L
s
.73569 .01472 m
.73569 .01847 L
s
.83189 .01472 m
.83189 .01847 L
s
.87999 .01472 m
.87999 .01847 L
s
.92809 .01472 m
.92809 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.01419 .10527 m
.02044 .10527 L
s
[(2)] .00169 .10527 1 0 Mshowa
.01419 .19582 m
.02044 .19582 L
s
[(4)] .00169 .19582 1 0 Mshowa
.01419 .28638 m
.02044 .28638 L
s
[(6)] .00169 .28638 1 0 Mshowa
.01419 .37693 m
.02044 .37693 L
s
[(8)] .00169 .37693 1 0 Mshowa
.01419 .46749 m
.02044 .46749 L
s
[(10)] .00169 .46749 1 0 Mshowa
.01419 .55804 m
.02044 .55804 L
s
[(12)] .00169 .55804 1 0 Mshowa
.125 Mabswid
.01419 .03735 m
.01794 .03735 L
s
.01419 .05999 m
.01794 .05999 L
s
.01419 .08263 m
.01794 .08263 L
s
.01419 .12791 m
.01794 .12791 L
s
.01419 .15055 m
.01794 .15055 L
s
.01419 .17319 m
.01794 .17319 L
s
.01419 .21846 m
.01794 .21846 L
s
.01419 .2411 m
.01794 .2411 L
s
.01419 .26374 m
.01794 .26374 L
s
.01419 .30902 m
.01794 .30902 L
s
.01419 .33166 m
.01794 .33166 L
s
.01419 .35429 m
.01794 .35429 L
s
.01419 .39957 m
.01794 .39957 L
s
.01419 .42221 m
.01794 .42221 L
s
.01419 .44485 m
.01794 .44485 L
s
.01419 .49013 m
.01794 .49013 L
s
.01419 .51276 m
.01794 .51276 L
s
.01419 .5354 m
.01794 .5354 L
s
.01419 .58068 m
.01794 .58068 L
s
.01419 .60332 m
.01794 .60332 L
s
.25 Mabswid
.01419 0 m
.01419 .61803 L
s
0 0 .5 r
.001 w
.02381 .05999 m
.03343 .60332 L
.04305 .19582 L
.05267 .15055 L
.06229 .05999 L
.07191 .05999 L
.08153 .01472 L
.09115 .05999 L
.10077 .01472 L
.11039 .01472 L
.12001 .01472 L
.12963 .01472 L
.13925 .01472 L
.14887 .01472 L
.15849 .01472 L
.16811 .01472 L
.17773 .01472 L
.18735 .01472 L
.19697 .01472 L
.20659 .01472 L
.21621 .01472 L
.22583 .01472 L
.23545 .01472 L
.24507 .01472 L
.25469 .01472 L
.26431 .01472 L
.27393 .01472 L
.28355 .01472 L
.29317 .01472 L
.30279 .01472 L
.31241 .01472 L
.32203 .01472 L
.33165 .01472 L
.34127 .01472 L
.35089 .01472 L
.36051 .01472 L
.37013 .01472 L
.37975 .01472 L
.38937 .01472 L
.39899 .01472 L
.40861 .01472 L
.41823 .01472 L
.42785 .01472 L
.43747 .01472 L
.44709 .01472 L
.45671 .01472 L
.46633 .01472 L
.47595 .01472 L
.48557 .01472 L
.49519 .01472 L
Mistroke
.50481 .01472 L
.51443 .01472 L
.52405 .01472 L
.53367 .01472 L
.54329 .01472 L
.55291 .01472 L
.56253 .01472 L
.57215 .01472 L
.58177 .01472 L
.59139 .01472 L
.60101 .01472 L
.61063 .01472 L
.62025 .01472 L
.62987 .01472 L
.63949 .01472 L
.64911 .01472 L
.65873 .01472 L
.66835 .01472 L
.67797 .01472 L
.68759 .01472 L
.69721 .01472 L
.70683 .01472 L
.71645 .01472 L
.72607 .01472 L
.73569 .01472 L
.74531 .01472 L
.75493 .01472 L
.76455 .01472 L
.77417 .01472 L
.78379 .01472 L
.79341 .01472 L
.80303 .01472 L
.81265 .01472 L
.82227 .01472 L
.83189 .01472 L
.84151 .01472 L
.85113 .01472 L
.86075 .01472 L
.87037 .01472 L
.87999 .01472 L
.88961 .01472 L
.89923 .01472 L
.90885 .01472 L
.91847 .01472 L
.92809 .01472 L
.93771 .01472 L
.94733 .01472 L
.95695 .01472 L
.96657 .01472 L
.97619 .01472 L
Mistroke
Mfstroke
0 .5 0 r
.02381 .28638 m
.03343 .05999 L
.04305 .10527 L
.05267 .05999 L
.06229 .05999 L
.07191 .01472 L
.08153 .01472 L
.09115 .01472 L
.10077 .01472 L
.11039 .01472 L
.12001 .01472 L
.12963 .01472 L
.13925 .01472 L
.14887 .01472 L
.15849 .01472 L
.16811 .01472 L
.17773 .01472 L
.18735 .01472 L
.19697 .01472 L
.20659 .01472 L
.21621 .01472 L
.22583 .01472 L
.23545 .01472 L
.24507 .01472 L
.25469 .01472 L
.26431 .01472 L
.27393 .01472 L
.28355 .01472 L
.29317 .01472 L
.30279 .01472 L
.31241 .01472 L
.32203 .01472 L
.33165 .01472 L
.34127 .01472 L
.35089 .01472 L
.36051 .01472 L
.37013 .01472 L
.37975 .01472 L
.38937 .01472 L
.39899 .01472 L
.40861 .01472 L
.41823 .01472 L
.42785 .01472 L
.43747 .01472 L
.44709 .01472 L
.45671 .01472 L
.46633 .01472 L
.47595 .01472 L
.48557 .01472 L
.49519 .01472 L
Mistroke
.50481 .01472 L
.51443 .01472 L
.52405 .01472 L
.53367 .01472 L
.54329 .01472 L
.55291 .01472 L
.56253 .01472 L
.57215 .01472 L
.58177 .01472 L
.59139 .01472 L
.60101 .01472 L
.61063 .01472 L
.62025 .01472 L
.62987 .01472 L
.63949 .01472 L
.64911 .01472 L
.65873 .01472 L
.66835 .01472 L
.67797 .01472 L
.68759 .01472 L
.69721 .01472 L
.70683 .01472 L
.71645 .01472 L
.72607 .01472 L
.73569 .01472 L
.74531 .01472 L
.75493 .01472 L
.76455 .01472 L
.77417 .01472 L
.78379 .01472 L
.79341 .01472 L
.80303 .01472 L
.81265 .01472 L
.82227 .01472 L
.83189 .01472 L
.84151 .01472 L
.85113 .01472 L
.86075 .01472 L
.87037 .01472 L
.87999 .01472 L
.88961 .01472 L
.89923 .01472 L
.90885 .01472 L
.91847 .01472 L
.92809 .01472 L
.93771 .01472 L
.94733 .01472 L
.95695 .01472 L
.96657 .01472 L
.97619 .01472 L
Mistroke
Mfstroke
.5 0 0 r
.02381 .37693 m
.03343 .15055 L
.04305 .15055 L
.05267 .15055 L
.06229 .10527 L
.07191 .05999 L
.08153 .10527 L
.09115 .05999 L
.10077 .05999 L
.11039 .05999 L
.12001 .05999 L
.12963 .05999 L
.13925 .05999 L
.14887 .05999 L
.15849 .05999 L
.16811 .05999 L
.17773 .05999 L
.18735 .05999 L
.19697 .05999 L
.20659 .05999 L
.21621 .05999 L
.22583 .05999 L
.23545 .05999 L
.24507 .05999 L
.25469 .05999 L
.26431 .05999 L
.27393 .05999 L
.28355 .05999 L
.29317 .05999 L
.30279 .05999 L
.31241 .05999 L
.32203 .05999 L
.33165 .05999 L
.34127 .05999 L
.35089 .05999 L
.36051 .05999 L
.37013 .05999 L
.37975 .05999 L
.38937 .05999 L
.39899 .05999 L
.40861 .05999 L
.41823 .05999 L
.42785 .05999 L
.43747 .05999 L
.44709 .05999 L
.45671 .05999 L
.46633 .05999 L
.47595 .05999 L
.48557 .05999 L
.49519 .05999 L
Mistroke
.50481 .05999 L
.51443 .05999 L
.52405 .05999 L
.53367 .05999 L
.54329 .05999 L
.55291 .05999 L
.56253 .05999 L
.57215 .05999 L
.58177 .05999 L
.59139 .05999 L
.60101 .05999 L
.61063 .05999 L
.62025 .05999 L
.62987 .05999 L
.63949 .05999 L
.64911 .05999 L
.65873 .05999 L
.66835 .05999 L
.67797 .05999 L
.68759 .05999 L
.69721 .05999 L
.70683 .05999 L
.71645 .05999 L
.72607 .05999 L
.73569 .05999 L
.74531 .05999 L
.75493 .05999 L
.76455 .05999 L
.77417 .05999 L
.78379 .05999 L
.79341 .05999 L
.80303 .05999 L
.81265 .05999 L
.82227 .05999 L
.83189 .05999 L
.84151 .05999 L
.85113 .05999 L
.86075 .05999 L
.87037 .05999 L
.87999 .05999 L
.88961 .05999 L
.89923 .05999 L
.90885 .05999 L
.91847 .05999 L
.92809 .05999 L
.93771 .05999 L
.94733 .05999 L
.95695 .05999 L
.96657 .05999 L
.97619 .05999 L
Mistroke
Mfstroke
0 1 1 r
.02381 .15055 m
.03343 .2411 L
.04305 .19582 L
.05267 .28638 L
.06229 .10527 L
.07191 .19582 L
.08153 .15055 L
.09115 .19582 L
.10077 .15055 L
.11039 .15055 L
.12001 .15055 L
.12963 .15055 L
.13925 .15055 L
.14887 .15055 L
.15849 .15055 L
.16811 .15055 L
.17773 .15055 L
.18735 .15055 L
.19697 .15055 L
.20659 .15055 L
.21621 .15055 L
.22583 .15055 L
.23545 .15055 L
.24507 .15055 L
.25469 .15055 L
.26431 .15055 L
.27393 .15055 L
.28355 .15055 L
.29317 .15055 L
.30279 .15055 L
.31241 .15055 L
.32203 .15055 L
.33165 .15055 L
.34127 .15055 L
.35089 .15055 L
.36051 .15055 L
.37013 .15055 L
.37975 .15055 L
.38937 .15055 L
.39899 .15055 L
.40861 .15055 L
.41823 .15055 L
.42785 .15055 L
.43747 .15055 L
.44709 .15055 L
.45671 .15055 L
.46633 .15055 L
.47595 .15055 L
.48557 .15055 L
.49519 .15055 L
Mistroke
.50481 .15055 L
.51443 .15055 L
.52405 .15055 L
.53367 .15055 L
.54329 .15055 L
.55291 .15055 L
.56253 .15055 L
.57215 .15055 L
.58177 .15055 L
.59139 .15055 L
.60101 .15055 L
.61063 .15055 L
.62025 .15055 L
.62987 .15055 L
.63949 .15055 L
.64911 .15055 L
.65873 .15055 L
.66835 .15055 L
.67797 .15055 L
.68759 .15055 L
.69721 .15055 L
.70683 .15055 L
.71645 .15055 L
.72607 .15055 L
.73569 .15055 L
.74531 .15055 L
.75493 .15055 L
.76455 .15055 L
.77417 .15055 L
.78379 .15055 L
.79341 .15055 L
.80303 .15055 L
.81265 .15055 L
.82227 .15055 L
.83189 .15055 L
.84151 .15055 L
.85113 .15055 L
.86075 .15055 L
.87037 .15055 L
.87999 .15055 L
.88961 .15055 L
.89923 .15055 L
.90885 .15055 L
.91847 .15055 L
.92809 .15055 L
.93771 .15055 L
.94733 .15055 L
.95695 .15055 L
.96657 .15055 L
.97619 .15055 L
Mistroke
Mfstroke
1 0 1 r
.02381 .15055 m
.03343 .10527 L
.04305 .19582 L
.05267 .19582 L
.06229 .15055 L
.07191 .10527 L
.08153 .10527 L
.09115 .10527 L
.10077 .15055 L
.11039 .15055 L
.12001 .15055 L
.12963 .15055 L
.13925 .15055 L
.14887 .15055 L
.15849 .15055 L
.16811 .15055 L
.17773 .15055 L
.18735 .15055 L
.19697 .15055 L
.20659 .15055 L
.21621 .15055 L
.22583 .15055 L
.23545 .15055 L
.24507 .15055 L
.25469 .15055 L
.26431 .15055 L
.27393 .15055 L
.28355 .15055 L
.29317 .15055 L
.30279 .15055 L
.31241 .15055 L
.32203 .15055 L
.33165 .15055 L
.34127 .15055 L
.35089 .15055 L
.36051 .15055 L
.37013 .15055 L
.37975 .15055 L
.38937 .15055 L
.39899 .15055 L
.40861 .15055 L
.41823 .15055 L
.42785 .15055 L
.43747 .15055 L
.44709 .15055 L
.45671 .15055 L
.46633 .15055 L
.47595 .15055 L
.48557 .15055 L
.49519 .15055 L
Mistroke
.50481 .15055 L
.51443 .15055 L
.52405 .15055 L
.53367 .15055 L
.54329 .15055 L
.55291 .15055 L
.56253 .15055 L
.57215 .15055 L
.58177 .15055 L
.59139 .15055 L
.60101 .15055 L
.61063 .15055 L
.62025 .15055 L
.62987 .15055 L
.63949 .15055 L
.64911 .15055 L
.65873 .15055 L
.66835 .15055 L
.67797 .15055 L
.68759 .15055 L
.69721 .15055 L
.70683 .15055 L
.71645 .15055 L
.72607 .15055 L
.73569 .15055 L
.74531 .15055 L
.75493 .15055 L
.76455 .15055 L
.77417 .15055 L
.78379 .15055 L
.79341 .15055 L
.80303 .15055 L
.81265 .15055 L
.82227 .15055 L
.83189 .15055 L
.84151 .15055 L
.85113 .15055 L
.86075 .15055 L
.87037 .15055 L
.87999 .15055 L
.88961 .15055 L
.89923 .15055 L
.90885 .15055 L
.91847 .15055 L
.92809 .15055 L
.93771 .15055 L
.94733 .15055 L
.95695 .15055 L
.96657 .15055 L
.97619 .15055 L
Mistroke
Mfstroke
1 1 0 r
.02381 .19582 m
.03343 .10527 L
.04305 .15055 L
.05267 .2411 L
.06229 .2411 L
.07191 .2411 L
.08153 .2411 L
.09115 .19582 L
.10077 .15055 L
.11039 .15055 L
.12001 .15055 L
.12963 .15055 L
.13925 .15055 L
.14887 .15055 L
.15849 .15055 L
.16811 .15055 L
.17773 .15055 L
.18735 .15055 L
.19697 .15055 L
.20659 .15055 L
.21621 .15055 L
.22583 .15055 L
.23545 .15055 L
.24507 .15055 L
.25469 .15055 L
.26431 .15055 L
.27393 .15055 L
.28355 .15055 L
.29317 .15055 L
.30279 .15055 L
.31241 .15055 L
.32203 .15055 L
.33165 .15055 L
.34127 .15055 L
.35089 .15055 L
.36051 .15055 L
.37013 .15055 L
.37975 .15055 L
.38937 .15055 L
.39899 .15055 L
.40861 .15055 L
.41823 .15055 L
.42785 .15055 L
.43747 .15055 L
.44709 .15055 L
.45671 .15055 L
.46633 .15055 L
.47595 .15055 L
.48557 .15055 L
.49519 .15055 L
Mistroke
.50481 .15055 L
.51443 .15055 L
.52405 .15055 L
.53367 .15055 L
.54329 .15055 L
.55291 .15055 L
.56253 .15055 L
.57215 .15055 L
.58177 .15055 L
.59139 .15055 L
.60101 .15055 L
.61063 .15055 L
.62025 .15055 L
.62987 .15055 L
.63949 .15055 L
.64911 .15055 L
.65873 .15055 L
.66835 .15055 L
.67797 .15055 L
.68759 .15055 L
.69721 .15055 L
.70683 .15055 L
.71645 .15055 L
.72607 .15055 L
.73569 .15055 L
.74531 .15055 L
.75493 .15055 L
.76455 .15055 L
.77417 .15055 L
.78379 .15055 L
.79341 .15055 L
.80303 .15055 L
.81265 .15055 L
.82227 .15055 L
.83189 .15055 L
.84151 .15055 L
.85113 .15055 L
.86075 .15055 L
.87037 .15055 L
.87999 .15055 L
.88961 .15055 L
.89923 .15055 L
.90885 .15055 L
.91847 .15055 L
.92809 .15055 L
.93771 .15055 L
.94733 .15055 L
.95695 .15055 L
.96657 .15055 L
.97619 .15055 L
Mistroke
Mfstroke
.9 .9 .9 r
.02381 .01472 m
.03343 .10527 L
.04305 .19582 L
.05267 .19582 L
.06229 .33166 L
.07191 .42221 L
.08153 .37693 L
.09115 .42221 L
.10077 .42221 L
.11039 .42221 L
.12001 .42221 L
.12963 .42221 L
.13925 .42221 L
.14887 .42221 L
.15849 .42221 L
.16811 .42221 L
.17773 .42221 L
.18735 .42221 L
.19697 .42221 L
.20659 .42221 L
.21621 .42221 L
.22583 .42221 L
.23545 .42221 L
.24507 .42221 L
.25469 .42221 L
.26431 .42221 L
.27393 .42221 L
.28355 .42221 L
.29317 .42221 L
.30279 .42221 L
.31241 .42221 L
.32203 .42221 L
.33165 .42221 L
.34127 .42221 L
.35089 .42221 L
.36051 .42221 L
.37013 .42221 L
.37975 .42221 L
.38937 .42221 L
.39899 .42221 L
.40861 .42221 L
.41823 .42221 L
.42785 .42221 L
.43747 .42221 L
.44709 .42221 L
.45671 .42221 L
.46633 .42221 L
.47595 .42221 L
.48557 .42221 L
.49519 .42221 L
Mistroke
.50481 .42221 L
.51443 .42221 L
.52405 .42221 L
.53367 .42221 L
.54329 .42221 L
.55291 .42221 L
.56253 .42221 L
.57215 .42221 L
.58177 .42221 L
.59139 .42221 L
.60101 .42221 L
.61063 .42221 L
.62025 .42221 L
.62987 .42221 L
.63949 .42221 L
.64911 .42221 L
.65873 .42221 L
.66835 .42221 L
.67797 .42221 L
.68759 .42221 L
.69721 .42221 L
.70683 .42221 L
.71645 .42221 L
.72607 .42221 L
.73569 .42221 L
.74531 .42221 L
.75493 .42221 L
.76455 .42221 L
.77417 .42221 L
.78379 .42221 L
.79341 .42221 L
.80303 .42221 L
.81265 .42221 L
.82227 .42221 L
.83189 .42221 L
.84151 .42221 L
.85113 .42221 L
.86075 .42221 L
.87037 .42221 L
.87999 .42221 L
.88961 .42221 L
.89923 .42221 L
.90885 .42221 L
.91847 .42221 L
.92809 .42221 L
.93771 .42221 L
.94733 .42221 L
.95695 .42221 L
.96657 .42221 L
.97619 .42221 L
Mistroke
Mfstroke
.5 .5 .5 r
.02381 .15055 m
.03343 .05999 L
.04305 .10527 L
.05267 .15055 L
.06229 .10527 L
.07191 .10527 L
.08153 .15055 L
.09115 .10527 L
.10077 .10527 L
.11039 .10527 L
.12001 .10527 L
.12963 .10527 L
.13925 .10527 L
.14887 .10527 L
.15849 .10527 L
.16811 .10527 L
.17773 .10527 L
.18735 .10527 L
.19697 .10527 L
.20659 .10527 L
.21621 .10527 L
.22583 .10527 L
.23545 .10527 L
.24507 .10527 L
.25469 .10527 L
.26431 .10527 L
.27393 .10527 L
.28355 .10527 L
.29317 .10527 L
.30279 .10527 L
.31241 .10527 L
.32203 .10527 L
.33165 .10527 L
.34127 .10527 L
.35089 .10527 L
.36051 .10527 L
.37013 .10527 L
.37975 .10527 L
.38937 .10527 L
.39899 .10527 L
.40861 .10527 L
.41823 .10527 L
.42785 .10527 L
.43747 .10527 L
.44709 .10527 L
.45671 .10527 L
.46633 .10527 L
.47595 .10527 L
.48557 .10527 L
.49519 .10527 L
Mistroke
.50481 .10527 L
.51443 .10527 L
.52405 .10527 L
.53367 .10527 L
.54329 .10527 L
.55291 .10527 L
.56253 .10527 L
.57215 .10527 L
.58177 .10527 L
.59139 .10527 L
.60101 .10527 L
.61063 .10527 L
.62025 .10527 L
.62987 .10527 L
.63949 .10527 L
.64911 .10527 L
.65873 .10527 L
.66835 .10527 L
.67797 .10527 L
.68759 .10527 L
.69721 .10527 L
.70683 .10527 L
.71645 .10527 L
.72607 .10527 L
.73569 .10527 L
.74531 .10527 L
.75493 .10527 L
.76455 .10527 L
.77417 .10527 L
.78379 .10527 L
.79341 .10527 L
.80303 .10527 L
.81265 .10527 L
.82227 .10527 L
.83189 .10527 L
.84151 .10527 L
.85113 .10527 L
.86075 .10527 L
.87037 .10527 L
.87999 .10527 L
.88961 .10527 L
.89923 .10527 L
.90885 .10527 L
.91847 .10527 L
.92809 .10527 L
.93771 .10527 L
.94733 .10527 L
.95695 .10527 L
.96657 .10527 L
.97619 .10527 L
Mistroke
Mfstroke
0 1 0 r
.02381 .05999 m
.03343 .05999 L
.04305 .05999 L
.05267 .05999 L
.06229 .2411 L
.07191 .2411 L
.08153 .19582 L
.09115 .2411 L
.10077 .28638 L
.11039 .28638 L
.12001 .28638 L
.12963 .28638 L
.13925 .28638 L
.14887 .28638 L
.15849 .28638 L
.16811 .28638 L
.17773 .28638 L
.18735 .28638 L
.19697 .28638 L
.20659 .28638 L
.21621 .28638 L
.22583 .28638 L
.23545 .28638 L
.24507 .28638 L
.25469 .28638 L
.26431 .28638 L
.27393 .28638 L
.28355 .28638 L
.29317 .28638 L
.30279 .28638 L
.31241 .28638 L
.32203 .28638 L
.33165 .28638 L
.34127 .28638 L
.35089 .28638 L
.36051 .28638 L
.37013 .28638 L
.37975 .28638 L
.38937 .28638 L
.39899 .28638 L
.40861 .28638 L
.41823 .28638 L
.42785 .28638 L
.43747 .28638 L
.44709 .28638 L
.45671 .28638 L
.46633 .28638 L
.47595 .28638 L
.48557 .28638 L
.49519 .28638 L
Mistroke
.50481 .28638 L
.51443 .28638 L
.52405 .28638 L
.53367 .28638 L
.54329 .28638 L
.55291 .28638 L
.56253 .28638 L
.57215 .28638 L
.58177 .28638 L
.59139 .28638 L
.60101 .28638 L
.61063 .28638 L
.62025 .28638 L
.62987 .28638 L
.63949 .28638 L
.64911 .28638 L
.65873 .28638 L
.66835 .28638 L
.67797 .28638 L
.68759 .28638 L
.69721 .28638 L
.70683 .28638 L
.71645 .28638 L
.72607 .28638 L
.73569 .28638 L
.74531 .28638 L
.75493 .28638 L
.76455 .28638 L
.77417 .28638 L
.78379 .28638 L
.79341 .28638 L
.80303 .28638 L
.81265 .28638 L
.82227 .28638 L
.83189 .28638 L
.84151 .28638 L
.85113 .28638 L
.86075 .28638 L
.87037 .28638 L
.87999 .28638 L
.88961 .28638 L
.89923 .28638 L
.90885 .28638 L
.91847 .28638 L
.92809 .28638 L
.93771 .28638 L
.94733 .28638 L
.95695 .28638 L
.96657 .28638 L
.97619 .28638 L
Mistroke
Mfstroke
1 0 0 r
.02381 .05999 m
.03343 .01472 L
.04305 .15055 L
.05267 .01472 L
.06229 .10527 L
.07191 .05999 L
.08153 .15055 L
.09115 .10527 L
.10077 .15055 L
.11039 .15055 L
.12001 .15055 L
.12963 .15055 L
.13925 .15055 L
.14887 .15055 L
.15849 .15055 L
.16811 .15055 L
.17773 .15055 L
.18735 .15055 L
.19697 .15055 L
.20659 .15055 L
.21621 .15055 L
.22583 .15055 L
.23545 .15055 L
.24507 .15055 L
.25469 .15055 L
.26431 .15055 L
.27393 .15055 L
.28355 .15055 L
.29317 .15055 L
.30279 .15055 L
.31241 .15055 L
.32203 .15055 L
.33165 .15055 L
.34127 .15055 L
.35089 .15055 L
.36051 .15055 L
.37013 .15055 L
.37975 .15055 L
.38937 .15055 L
.39899 .15055 L
.40861 .15055 L
.41823 .15055 L
.42785 .15055 L
.43747 .15055 L
.44709 .15055 L
.45671 .15055 L
.46633 .15055 L
.47595 .15055 L
.48557 .15055 L
.49519 .15055 L
Mistroke
.50481 .15055 L
.51443 .15055 L
.52405 .15055 L
.53367 .15055 L
.54329 .15055 L
.55291 .15055 L
.56253 .15055 L
.57215 .15055 L
.58177 .15055 L
.59139 .15055 L
.60101 .15055 L
.61063 .15055 L
.62025 .15055 L
.62987 .15055 L
.63949 .15055 L
.64911 .15055 L
.65873 .15055 L
.66835 .15055 L
.67797 .15055 L
.68759 .15055 L
.69721 .15055 L
.70683 .15055 L
.71645 .15055 L
.72607 .15055 L
.73569 .15055 L
.74531 .15055 L
.75493 .15055 L
.76455 .15055 L
.77417 .15055 L
.78379 .15055 L
.79341 .15055 L
.80303 .15055 L
.81265 .15055 L
.82227 .15055 L
.83189 .15055 L
.84151 .15055 L
.85113 .15055 L
.86075 .15055 L
.87037 .15055 L
.87999 .15055 L
.88961 .15055 L
.89923 .15055 L
.90885 .15055 L
.91847 .15055 L
.92809 .15055 L
.93771 .15055 L
.94733 .15055 L
.95695 .15055 L
.96657 .15055 L
.97619 .15055 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[115]:=",
  ImageSize->{586.188, 362.312},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40009:0001JR000`400?l00000o`00003ooooooooo
ooooodcooooo001lo`0000Gooooo0ol00002ooooofKo00000oooool3o`0000;oooooIOl00003oooo
o`Co00000_oooomUo`0000?ooooo1?l00002ooooof;o00000oooool4o`0000;ooooo1?l00002oooo
o`So00000Oooool007go000000Coooooo`000?l0003ooooo0_l000001?oooooo0000o`000?oooomV
o`000005ooooool0003o0000o`000?ooool00_l000000ooooooo0000o`00001Qo`000005ooooool0
003o0000o`000?ooool00_l000001?oooooo0000o`000?oooomSo`000005ooooool0003o0000o`00
0?ooool00_l000001?oooooo0000o`000?oooomRo`000003ooooool0003o000000;o000000Cooooo
o`000?l0003ooooo0_l000001?oooooo0000o`000?ooool7o`00007ooooo001no`000003ooooool0
003o000000;o000000Coooooo`000?l0003oooooHol00005ooooo`;o000000Coooooo`000?l0003o
ooooHol000001Ooooooo0000o`000?l0003ooooo00;o000000Coooooo`000?l0003oooooHol00000
1Ooooooo0000o`000?l0003ooooo00;o000000Coooooo`000?l0003oooooH_l000000ooooooo0000
o`000002o`000004ooooool0003o0000ooooo`;o000000Coooooo`000?l0003ooooo1ol00001oooo
o`00Ool000001Ooooooo0000o`000?l0003ooooo00;o000000?oooooo`000?l00000HOl000001?oo
oooo0000o`000?ooool3o`000004ooooool0003o0000ooooof?o00001?ooool3o`000004ooooool0
003o0000ooooofCo00000oooool3o`000004ooooool0003o0000ooooof;o000000?oooooo`000?l0
00000_l000001?oooooo0000o`000?ooool2o`000004ooooool0003o0000ooooo`Oo00000Oooool0
07co000000Goooooo`000?l0003o0000ooooo`02o`000004ooooool0003o0000ooooofCo000000?o
ooooo`000?ooool00ol000001?oooooo0000o`000?oooomSo`000003ooooool0003o000000Co0000
00Coooooo`000?l0003oooooHol000001Ooooooo0000o`000?l0003ooooo00;o000000Coooooo`00
0?l0003oooooH_l000000ooooooo0000o`000002o`000004ooooool0003o0000ooooo`;o000000Co
ooooo`000?l0003ooooo1ol00001ooooo`00O?l000001Ooooooo0000o`000?l0003ooooo00;o0000
00Coooooo`000?l0003oooooIOl00002ooooo`?o000000Coooooo`000?l0003oooooI?l000000ooo
oooo0000o`000003o`000004ooooool0003o0000ooooof?o000000Goooooo`000?l0003o0000oooo
o`02o`000004ooooool0003o0000ooooof7o00000_ooool4o`000004ooooool0003o0000ooooo`;o
000000Coooooo`000?l0003ooooo1ol00001ooooo`00OOl00003ooooo`Co00000_oooomWo`000003
ooooool0003o000000;o00000_oooomVo`0000;ooooo1?l00002ooooofGo00000oooool4o`0000;o
ooooHol000000ooooooo0000o`000003o`0000;ooooo1?l00002ooooo`So00000Oooool001Co0000
00?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0
003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool0
01Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l0
0000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co0000
0Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co000000?ooooo
o`000?l00000ool0003oo`0003Co00000Oooool000co00003Oooool00ooVi^Koooooooooo`03oooo
o`03ool00?oooooooooo00Sooooo00?oo`00ooooooooool02?ooooooo`1o0?oo07l02?l0O`0?oooo
o`005?l000000ooooooo0000o`000002o`000003onKVi_l0003o000000?o000000?oo`00o`000?l0
00002?l000000ooo003o0000o`000002o`000003ooooool0003o000000?o000000?o07l0o`000?l0
00000_l000000ol007oo0000o`000008o`000003o`00Ool0003o000000?o000000?oooooo`000?l0
00006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So000000?oooooo`000?l0
00006Ol000000ooooooo0000o`00000Ho`000003ooooool0003o000001Wo000000?oooooo`000?l0
00006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So000000?oooooo`000?l0
00006Ol000000ooooooo0000o`00000Ho`000003ooooool0003o000001Wo000000?oooooo`000?l0
00006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So000000?oooooo`000?l0
00006Ol000000ooooooo0000o`00000Io`000003ooooool0003o000001So000000?oooooo`000?l0
00003?l00001ooooo`005?l000000ooooooo0000o`000002o`000003onKVi_l0003o000000?o0000
00?oo`00o`000?l000002?l000000ooo003o0000o`000002o`000003ooooool0003o000000?o0000
00?o07l0o`000?l000000_l000000ol007oo0000o`000008o`000003o`00Ool0003o000000?o0000
00?oooooo`000?l000006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So0000
00?oooooo`000?l000006Ol000000ooooooo0000o`00000Ho`000003ooooool0003o000001Wo0000
00?oooooo`000?l000006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So0000
00?oooooo`000?l000006Ol000000ooooooo0000o`00000Ho`000003ooooool0003o000001Wo0000
00?oooooo`000?l000006?l000000ooooooo0000o`00000Io`000003ooooool0003o000001So0000
00?oooooo`000?l000006Ol000000ooooooo0000o`00000Io`000003ooooool0003o000001So0000
00?oooooo`000?l000003?l00001ooooo`005?l000000ooooooo0000o`000002o`000003onKVi_l0
003o000000;o00000_oo000:o`000003ool00?l0003o000000Oo000000?o07l0o`000?l000000ol0
0002o`00O`Wo000000?o001oo`000?l00000>_l000000ooooooo0000o`00001[o`000003ooooool0
003o000006_o000000?oooooo`000?l00000Jol000000ooooooo0000o`00001[o`000003ooooool0
003o000000co00000Oooool001Co000000?oooooo`000?l000000_l000000ooVi^Ko0000o`000002
o`0000;oo`002_l000000ooo003o0000o`000007o`000003o`1o0?l0003o000000;o000000?o001o
o`000?l007l02?l000000ol007oo0000o`00000ko`000003ooooool0003o000006_o000000?ooooo
o`000?l00000Jol000000ooooooo0000o`00001[o`000003ooooool0003o000006_o000000?ooooo
o`000?l000003?l00001ooooo`005?l000000ooooooo0000o`000003o`000004onKVi_l0003o0000
o`0000;oo`002_l000000ooo003o0000o`000007o`000003o`1o0?l0003o000000;o000000?o001o
o`000?l007l02?l000000ol007oo0000o`00003oo`000?oo00001?l00001ooooo`005?l000000ooo
oooo0000o`000003o`000004onKVi_l0003o0000o`0000;oo`002_l00002ool000So000000?o07l0
o`000?l000000_l000000ol007oo0000o`00O`08o`000003o`00Ool0003o00000?oo0000ool00004
o`00007ooooo000Do`000003ooooool0003o000000?o000000Koi^KVo`000?l0003oo`00o`000?oo
0009o`000003ool00?l0003oo`0000Oo000000?o07l0o`000?l000000ol000001?l007oo0000o`00
0?l007l7o`000003o`00Ool0003o00000?oo0000ool00004o`00007ooooo000Do`000003ooooool0
003o000000?o000000Koi^KVo`000?l0003oo`00o`000?oo0009o`000003ool00?l0003oo`0000Oo
000000?o07l0o`000?l000000ol000001?l007oo0000o`000?l007l7o`000003o`00Ool0003o0000
0?oo0000ool00004o`00007ooooo000Do`000003ooooool0003o000000?o000000Coi^KVo`000?l0
003oo`000_l000000ooo003o0000o`000006o`000003ool00?l0003oo`0000Oo000000?o07l0o`00
0?l000000_l000001Ol007oo0000o`000?l0003o001o00Ko000000?o001oo`000?l00000ool0003o
o`0000Go00000Oooool001Co000000?oooooo`000?l000000ol000001?oVi^Ko0000o`000?oo0002
o`000003ool00?l0003o000000Ko000000?oo`00o`000?oo00001ol000000ol0O`3o0000o`000002
o`000005o`00Ool0003o0000o`000?l007l01_l000000ol007oo0000o`00003oo`000?oo00001Ol0
0001ooooo`005?l000000ooooooo0000o`000003o`000004onKVi_l0003o0000ool000;o000000?o
o`00o`000?l000001_l000000ooo003o0000ool00007o`000003o`1o0?l0003o000000;o000000Go
001oo`000?l0003o0000o`00O`06o`000003o`00Ool0003o00000?oo0000ool00005o`00007ooooo
000Do`000003ooooool0003o000000?o000000?oi^KVo`000?oo00000ol000000ooo003o0000o`00
0006o`000003ool00?l0003oo`0000Ko000000?o07l0o`000?l000000ol000000ol007oo0000o`00
0002o`000003o`00Ool0003o000000?o000000?o001oo`000?l00000ool0003oo`0000Go00000Ooo
ool001Co00000oooool3o`000003onKVi_l0003oo`0000?o000000?oo`00o`000?l000001_l00000
0ooo003o0000ool00006o`000003o`1o0?l0003o000000?o000000?o001oo`000?l000000_l00000
0ol007oo0000o`000003o`000003o`00Ool0003o00000?oo0000ool00005o`00007ooooo000Do`00
0003ooooool0003o000000Co000000?oi^KVool00?l000000_l000000ooo003o0000o`000006o`00
0003ool00?l0003oo`0000Ko000000?o07l0o`000?l000000_l000000ol007oo0000o`000003o`00
0003o`00Ool0003o000000;o000000?o001oo`000?l00000ool0003oo`0000Ko00000Oooool001Co
000000?oooooo`000?l000001?l000000ooVi^Koo`00o`000002o`000003ool00?l0003o000000Ko
000000?oo`00o`000?oo00001_l000000ol0O`3o0000o`000002o`000003o`00Ool0003o000000?o
000000?o001oo`000?l000000_l000000ol007oo0000o`00003oo`000?oo00001_l00001ooooo`00
5?l000000ooooooo0000o`000004o`000003ool00?l0003o000000;o000000?oo`00o`000?l00000
1_l000000ooo003o0000ool00005o`000003o`1o0?l0003o000000?o000000?o001oo`000?l00000
1?l000001Ol007oo0000o`000?l0003o001o0?oo0000ool00008o`00007ooooo000Do`000003oooo
ool0003o000000Co000000?oo`00o`000?l000000_l000000ooo003o0000o`000006o`000004ool0
0?l0003o0000ool000Co000000?o07l0o`000?l000000ol000000ol007oo0000o`000004o`000005
o`00Ool0003o0000o`000?l007l0ool0003oo`0000So00000Oooool001Co000000?oooooo`000?l0
00001?l000000ooo003o0000o`000002o`000003ool00?l0003o000000Ko000000Coo`00o`000?l0
003oo`001?l000000ol0O`3o0000o`000003o`000003o`00Ool0003o000000Co000000Go001oo`00
0?l0003o0000o`00O`3oo`000?oo00002?l00001ooooo`005?l000000ooooooo0000o`000004o`00
0003ool00?l0003o000000;o000000?oo`00o`000?l000001_l000001?oo003o0000o`000?oo0004
o`000003o`1o0?l0003o000000;o000000?o001oo`000?l000001Ol000001?l007oo0000o`000?l0
07ooo`000?oo00002Ol00001ooooo`005?l000000ooooooo0000o`000003o`000003ool00?oVi^Ko
000000?o000000?oo`00o`000?l000001Ol000001Ooo003o0000o`000?l0003oo`0000?o000000?o
07l0o`000?l000000ol000000ol007oo0000o`000006o`000003o`00Ool0003o001o0?oo0000ool0
0009o`00007ooooo000Do`000003ooooool0003o000000?o000000?oo`00onKVi_l000000ol00000
0ooo003o0000o`000005o`000005ool00?l0003o0000o`000?oo00000ol000000ol0O`3o0000o`00
0003o`000003o`00Ool0003o000000Ko000000?o001oo`000?l007l0ool0003oo`0000Wo00000Ooo
ool001Co000000?oooooo`000?l000000ol000000ooo003oi^KVo`000003o`000003ool00?l0003o
000000Go000000Goo`00o`000?l0003o0000ool00003o`000003o`1o0?l0003o000000?o000000?o
001oo`000?l000001_l000000ol007oo0000o`00O`3oo`000?oo00002Ol00001ooooo`005?l00000
0ooooooo0000o`000003o`000003ool00?l0003oi^KV00?o000000?oo`00o`000?l000001Ol00000
1Ooo003o0000o`000?l0003oo`0000?o000000?o07l0o`000?l000000ol000000ol007oo0000o`00
0006o`000003o`00Ool0003o001o0?oo0000ool00009o`00007ooooo000Do`000003ooooool0003o
000000?o000000?oo`00o`000?oVi^H01?l000000ooo003o0000o`000004o`000005ool00?l0003o
0000o`000?oo00000ol000000ol0O`3o0000o`000002o`000003o`00Ool0003o000000Oo00000_l0
07ooo`000?oo00002_l00001ooooo`005?l000000ooooooo0000o`000002o`000004ool00?l0003o
0000onKViPCo000000?oo`00o`000?l000001?l000001Ooo003o0000o`000?l0003oo`0000;o0000
00?o07l0o`000?l000000ol000000ol007oo0000o`000008o`000003o`00Ool0003o00000?oo0000
ool00008o`00007ooooo000Do`0000?ooooo0_l000000ooo003o0?l0o`3o0005o`3o0003ool00?l0
o`3o0?l000Co0?l000Goo`00o`3o0?l0o`3o07l0ool00003o`1o00Go001o00?oo`00o`000?l00000
2?l0003oogl00?coO`003_l00001ooooo`005?l000000ooooooo0000o`000002o`000004o`00Ool0
003o0000onKViP;o000000?oOgmoo`000?oo00001_l000000ooo003o0000o`3o0002o`000003ool0
0?l0003o001o00Go000000?oo`00o`000?l000002?l000000omo003o0000o`00003oo`000?oo0000
2?l00001ooooo`005?l000000ooooooo0000o`000002o`000004o`00Ool0003o0000onKViP;o0000
00?oOgmoo`000?oo00001_l000000ooo003o0000o`3o0002o`000003ool00?l0003o001o00Go0000
00?oo`00o`000?l000002?l000000omo003o0000o`00003oo`000?oo00002?l00001ooooo`005?l0
00000ooooooo0000o`000002o`000004o`00Ool0003o0000onKViP;o000000?oOgmoo`000?oo0000
1_l000000ooo003o07l0o`3o0002o`000003ool00?l0003o001o00Co00000_oo0009o`000003ogl0
0?l0003o00000?oo0000ool00009o`00007ooooo000Do`000003ooooool0003o000000;o000000Co
001oo`000?l0003oi^KV0_l00002ogmoO`03ool00?l0003o000000Co000000?oo`00o`1o0?l0o`00
0_l000000ooo003o0000o`00O`04o`0000;oo`0000?oO`00o`000?l000001_l000000omo003o0000
o`00003oo`000?oo00002Ol00001ooooo`005?l000000ooooooo0000o`000002o`000009o`00Ool0
003o0000o`000?oVi^KoOgmoo`1o0?moOgooo`0000Ko000000?oo`00o`1o0?l0o`000_l000000ooo
003o0000o`00O`04o`0000;oo`0000?oO`00o`000?l000001_l000000omo003o0000o`00003oo`00
0?oo00002Ol00001ooooo`005?l000000ooooooo0000o`000002o`000009o`00Ool0003o0000o`00
0?oVi^KoOgmoo`1o0?moOgooo`0000Ko000000?oo`00o`1o0?l0o`000_l000000ooo003o001oo`00
0004o`000003ool00?l0003oo`0000So000000?oO`00o`000?l00000ool0003oo`0000Wo00000Ooo
ool001Co000000?oooooo`000?l000000_l000002Ol007oo0000o`000?l0003oi^KVogmoOol0O`3o
Ogmoool00005o`000004ool00?l0O`3o0000o`3o00;o000000?oo`00o`00Ool000000ol000001?oo
003o0000o`000?oo0007o`000003ogl00?l0003o00000?oo0000ool0000:o`00007ooooo000Do`00
0003ooooool0003o000000;o000000Wo001oo`000?l0003o0000onKVi_moOgoo07l0ogmoOooo0000
1Ol000001?oo003o07l0o`000?l0o`02o`000003ool00?l007oo000000?o000000Coo`00o`000?l0
003oo`001ol000000omo003o0000o`00003oo`000?oo00002_l00001ooooo`005?l000000ooooooo
0000o`000002o`000009o`00Ool0003o0000o`000?oVi^KoOgmoo`1o0?l0003oo`0000Go000000Co
o`00o`1o0?l0003o0?l00_l000000ooo003o001oo`000003o`000005ool00?l0003o0000ool00?mo
00001_l000000omo003o0000o`00003oo`000?oo00002_l00001ooooo`005?l000000ooooooo0000
o`000002o`000009o`00Ool0003o0000o`000?oVi^KoOgmoo`1o0?l0003oo`0000Go000000Coo`00
o`1o0?l0003o0?l00_l000000ooo003o001oo`000003o`000005ool00?l0003o0000ool00?mo0000
1_l000000omo003o0000o`00003oo`000?oo00002_l00001ooooo`005?l000000ooooooo0000o`00
0002o`000009o`00Ool0003o0000o`000?oVi^KoOgmoo`1o0?l0003oo`0000Go000000Goo`00o`1o
0?l0003o0000o`3o0002o`000003ool00?l0003o000000;o000000Goo`00o`000?l0003oo`00ogl0
0006o`000003ogl00?l0003o00000?oo0000ool0000:o`00007ooooo000Do`000003ooooool0003o
000000;o000000Ko001oo`000?l0003o0000onKVi_moOgl2o`000003ool00?l0003o000000?o0000
00Goo`00o`000?l0003o0000o`3o0002o`000005ool00?l0003o0000o`000?oo00000ol000000ooo
003oO`00o`000004o`000003ogl00?l0003o00000?oo0000ool0000;o`00007ooooo000Do`0000?o
oooo0_l000001_l007oo0000o`000?l0003oi^KVogmoO`;o000000?oo`00o`000?l000000ol00000
1Ooo003o0000o`000?l0003o0?l000;o000000Goo`00o`000?l0003o0000ool00003o`000003ool0
0?mo003o000000Co000000?oO`00o`000?l00000ool0003oo`0000_o00000Oooool001Co000000?o
ooooo`000?l000000_l000001_l007oo0000o`000?l0003oOgmoonKViP?o000000?oo`00o`000?l0
00000_l000001Ooo003o0000o`000?l0003o0?l000;o000000Goo`00o`000?l0003o0000ool00003
o`000003ool00?l0003oO`0000Co000000?oO`00o`000?l00000ool0003oo`0000_o00000Oooool0
01Co000000?oooooo`000?l000000_l000001_l007oo0000o`000?l0003oOgmoonKViP?o000000?o
o`00o`000?l000000_l000001Ooo003o0000o`000?l0003o0?l000;o000000Goo`00o`000?l0003o
0000ool00003o`000003ool00?l0003oO`0000Co000000?oO`00o`000?l00000ool0003oo`0000_o
00000Oooool001Co000000?oooooo`000?l000000_l000001_l007oo0000o`000?l0003oOgmoonKV
iP?o000000Koo`00o`000?l0003o0000o`1o0?oo0003o`000004o`3o0?l0003o0000ool000;o0000
00?oo`00o`000?l000000_l000000ooo003o0000ogl00003o`000003ogl00?l0003o00000?oo0000
ool0000<o`00007ooooo000Do`000003ooooool0003o000000;o000000Ko001oo`000?l0003o0000
ogmoOooVi^H3o`000006ool00?l0003o0000o`000?l0O`3oo`000ol000001?l0o`3o0000o`00Oooo
0002o`000003ool00?l0003o000000?o000000?oo`00ogl00?l000000_l000000omo003o0000o`00
003oo`000?oo00003?l00001ooooo`005?l000000ooooooo0000o`000002o`000006o`00Ool0003o
0000o`000?moOgooi^KV0ol000001_oo003o0000o`000?l0003o07l0ool000?o000000Co0?l0o`00
0?l007ooo`000_l000000ooo003o0000o`000003o`000003ool00?mo003o000000;o000000?oO`00
o`000?l00000ool0003oo`0000co00000Oooool001Co000000?oooooo`000?l000000_l000001_l0
07oo0000o`000?l0003oOgmoonKViP?o000000Koo`00ogmoOol0003o0000o`1o0?oo0003o`000004
o`3o0?l0003o001oool000;o000000?oo`00o`000?l000000ol000000ooo003o0000ogl00002o`00
0003ogl00?l0003o00000?oo0000ool0000<o`00007ooooo000Do`000003ooooool0003o000000;o
000000Ko001oo`000?l0003o0000ogmoOooVi^H3o`000005ool00?moOgoo0000o`1o0?oo00001?l0
00001_l0o`3o0000o`00Oooo003o0000ool000Ko000000Goo`00o`000?mo003o0000ogl0003oo`00
0?oo00003ol00001ooooo`005?l000000ooooooo0000o`000002o`000006o`00Ool0003o0000o`00
0?moOgooi^KV0ol000001Ooo003oOgmoo`000?l0O`3oo`0000Co000000Ko0?l0o`000?l007ooo`00
o`000?oo0006o`000005ool00?l0003oO`00o`000?mo0000ool0003oo`0000oo00000Oooool001Co
000000?oooooo`000?l000000_l000001_l007oo0000o`000?l0003oOgmoonKViP?o000000Goo`00
ogmoOol0003o07l0ool00004o`000004o`3o0?l0003o001oo`0000;oo`001_l000001Ooo003o0000
ogl00?l0003oO`000?oo0000ool0000?o`00007ooooo000Do`000003ooooool0003o000000;o0000
00Oo001oo`000?l0003oOgmoo`000?l0O`3oi^KV00;o000000Goo`00ogmoOol0003o07l0ool00004
o`000004o`3o0?l0003o001oo`0000;oo`001_l000001Ooo003o0000ogl00?l0003oO`000?oo0000
ool0000?o`00007ooooo0007o`0000Gooooo2?l000000ooooooo0000o`000002o`000007o`00Ool0
003o0000ogmoOol0003o07l0onKViP02o`000005ool00?l0003oOgmoo`1o0?oo00001?l000001?l0
o`3o0000o`00Ool00002ool000Ko000000?oo`00o`000?l000000_mo003oo`000?oo00003ol00001
ooooo`002?l000001?oooooo0000o`000?ooool8o`000003ooooool0003o000000;o000000Oo001o
o`000?l0003oOgmoo`000?l0O`3oi^KV00;o000000Goo`00o`000?moOgoo0000ool00004o`000005
o`3o0?l0003o001oo`000?oo00001ol000001?oo003o0000o`000?mo003oo`000?oo00004?l00001
ooooo`002Ol000000ooooooo0000o`000008o`0000Cooooo00So0000o`00Ool0003o0000ogmoOol0
003o07l0onKViP;o000000Goo`00o`000?moOgoo0000ool00004o`000005o`3o0?l0003o001oo`00
0?oo00001_moOgl00ooo0?ooo`00ool0o`07ool0o`03ool00?moOgooOgmo0?ooOgmonOmoOgl>o`00
007ooooo000:o`000003ooooool0003o000000Oo000000?oooooo`000?l000000ol000001_l007oo
0000ogmoOol0003o07l0onKViP;o000000Goo`00o`000?moOgoo0000ool00004o`000005o`3o0?l0
07oo0000o`000?moOgl01Ol000001?moOgoo0000o`000?oo0007o`000003ool00?l0003o00000?oo
0000ool00008o`00007ooooo0007o`000005ooooool0003o0000o`000?ooool02?l000000ooooooo
0000o`000003o`000006o`00Ool0003oOgmoo`000?l0O`3oi^KV0_l000001Ooo003o0000ogmoOol0
003oo`0000Co000000Go0?l0o`00Ool0003o0000ogmoO`05o`000004ogmoOol0003o0000ool000Oo
000000?oo`00o`000?l00000ool0003oo`0000So00000Oooool000Oo000000Goooooo`000?l0003o
0000ooooo`08o`000003ooooool0003o000000?o000000Ko001oo`000?moOgoo0000ool0oooVi^H2
o`000005ool00?l0003o0000ogmoOooo00001?l000001Ol0o`3o001oo`000?l0003oOgmo00Co0000
00Goo`3oogmoOol0003o0000ool00006o`0000;oo`00ool0003oo`0000[o00000Oooool000So0000
0oooool9o`000003ooooool0003o000000?o000000Oo001oo`000?moOgoo0000ool0oooVi^Koool0
00;o000000Coo`00o`000?moOgooo`001?l000001Ol0o`3o001oo`000?moOgoo0?oo00Co000000Go
o`3oo`000?moOgoo0000ool00006o`000003ool00?l0003oo`000?oo0000ool00009o`00007ooooo
000Do`000003ooooool0003o000000?o000000Oo001oogmoOol0003o0000oooo0?oVi^Koool000;o
000000Coo`00o`000?moOgooo`001?l000001?l0o`3o001oo`000?moOgl2o`3oo`?o000000Goo`3o
o`000?moOgoo0000ool00006o`000003ool00?l0003oo`000?oo0000ool00009o`00007ooooo000D
o`000003ooooool0003o000000?o000000Oo001oogmoOol0003o0000oooo0?l0003oi^KV00;o0000
00Coo`00o`000?moOgooo`001Ol000000ol0o`3o0000ogmoO`02o`3oo`?o000000Goo`3oo`000?mo
Ogoo0000ool00006o`000003ool00?l0003oo`000?oo0000ool00009o`00007ooooo000Do`000003
ooooool0003o000000?o000000Oo001oogmoOol0003oo`3ooooo0?l0003oi^KV00;o000000Coo`00
o`000?oo003oOgmo1Ol000000ol0o`3o0000ogmoO`02o`3oo`;o000000Koo`3oo`000?l0003oOgmo
o`000?oo0005o`000004ool00?l0003o0000ool00?oo0000ool00009o`00007ooooo000Do`000003
ooooool0003o000000?o000000Oo001oogmoOol0003oo`3ooooo0?l0003oi^KV00;o000000Coo`00
o`000?oo003oOgmo1Ol000000ol0o`3o0000ogmoO`02o`3oo`;o000000Koo`3oo`000?l0003oOgmo
o`000?oo0005o`000004ool00?l0003o0000ool00?oo0000ool00009o`00007ooooo000Do`000003
ooooool0003o000000?o000000go001oogmoOol0003oo`3ooooo0?l0003oi^KVoooo0?l0003oo`00
o`000?oo003oOgmo00Go000000?o0?l0ogmoOol000000_l0ool2o`000006ool0ool0003o0000o`00
0?moOgooo`001Ol000001Ooo003o0000o`000?l0003oo`000?oo0000ool00008o`00007ooooo000D
o`000003ooooool0003o000000?o000000go001oogmoOol0003oo`3ooooo0?l0003oi^KVoooo0?l0
003oo`00o`000?oo003oOgmo00Go000000?o0?l0ogmoOol000000_l0ool2o`000006ool0ool0003o
0000o`000?moOgooo`001Ol000001Ooo003o0000o`000?l0003oo`000?oo0000ool00008o`00007o
oooo000Do`000003ooooool0003o000000?o000000go001oogmoOol0003oo`3ooooo0?l0003oi^KV
oooo0?l0003oo`00o`000?oo003oOgmo00Co000000Ko001oo`3o0?moOgoo0?ooo`000?l0ool2o`00
0007ool0ool0003o0000o`000?moOgoo0000ool00004o`000005ool00?l0003o0000o`000?oo0000
ool0003oo`0000So00000Oooool001Co000000?oooooo`000?l000000ol000003_l007ooOgmoool0
ool0O`3oool0o`000?oVi^Koool0o`000?oo003o0000ool00?l0003oOgmo0ol000002?l007oo0?l0
ogmoOol0oooo0000o`3oool0003oo`3o1?l000000omoOgoo0000ool00003o`000003ool00?l0003o
000000;o000000?oo`00o`000?l00000ool0003oo`0000Ko00000Oooool001Co00000oooool3o`00
000>o`00OomoOgooo`3oo`1o0?ooo`3o0000onKVi_ooo`3o0000ool00?l0003oo`00o`000?moOgl3
o`000008o`00Ool0o`3oOgmoo`3oool0003o0?ooo`000?oo0?l4o`000003ogmoOol0003oo`0000?o
000000?oo`00o`000?l000000_l000000ooo003o0000o`00003oo`000?oo00001_l00001ooooo`00
5?l000000ooooooo0000o`000003o`000004ogmoOol0003oo`3ooooo00;o000000Soi^KVo`000?oo
o`3oo`00o`000?oo003o0000ogmoO`?o000000Co001oo`3o0?l0003o0?oo0_l000000ol0ooooo`3o
o`000004o`000003ogmoOooo003o000000;o000000?oo`00o`000?l000000ol000000ooo003o0000
o`00003oo`000?oo00001Ol00001ooooo`005?l000000ooooooo0000o`000003o`000004ogmoOol0
003oo`3ooooo00;o000000Soi^KVo`000?ooo`3oo`00o`000?oo003o0000ogmoO`?o000000Co001o
o`3o0?l0003o0?oo0_l000000ol0ooooo`3oo`000004o`000003ogmoOooo003o000000;o000000?o
o`00o`000?l000000ol000000ooo003o0000o`00003oo`000?oo00001Ol00001ooooo`005?l00000
0ooooooo0000o`000003o`000004ogmoOooo0?oo0000oooo00;o000000Koi^KVo`000?ooo`3oo`00
o`000?oo0002o`000007ogmoOol0003o0000o`00Ool0o`3o0000o`3oo`02o`000003ool0ool0003o
000000Co000000GoOgmoool00?l0003o0000ool00006o`000003ool00?l0003o00000?oo0000ool0
0005o`00007ooooo000Do`000003ooooool0003o000000?o000000CoOgmoool0ool0003oool00ol0
00001OoVi^Koool0ool00?l0003oo`0000;o000000OoOgmoo`000?l0003o001oo`3o0?l0003o0?oo
00;o000000?oo`3oo`000?l000001?l000001OmoOgooo`00o`000?l0003oo`0000Ko000000?oo`00
o`000?l00000ool0003oo`0000Go00000Oooool001Co000000?oooooo`000?l000000ol000001?mo
Ogooo`3oo`000?ooo`03o`000005onKVi_ooo`3oo`00o`000?oo00000_l000001omoOgoo0000o`00
0?l007oo0?l0o`000?l0ool00_l000000ooo0?oo0000o`000004o`000005ogmoOooo003o0000o`00
0?oo00001_l000000ooo003o0000o`00003oo`000?oo00001Ol00001ooooo`005?l000000ooooooo
0000o`000003o`000004ogmoOooo0?oo0000oooo00?o000000?oi^KVo`000?ooo`000_oo0002o`00
0007ogmoOol0003o0000ogmoOol0o`3o0000o`3oo`02o`000003ool0ool0003o000000Go000000Co
o`00o`000?l0003oo`001ol000000ooo003o0000o`00003oo`000?oo00001?l00001ooooo`005?l0
00000ooooooo0000o`000003o`000004ogmoOol0003o0000oooo00?o000000Coi^KVo`000?ooo`3o
o`001?l000002OmoOgoo0000ogmoOol0o`3o0000o`3oool0003oo`3oo`3oo`07o`000003ool00?l0
003oo`0000So000000?oo`00o`000?l00000ool0003oo`0000Co00000Oooool001Co000000?ooooo
o`000?l000000ol000001?moOgoo0000o`000?ooo`03o`000004onKVi_l0003oool0ool000Co0000
00WoOgmoo`000?moOgoo0?l0o`000?l0oooo0000ool0ool0ool01ol000000ooo003o0000ool00008
o`000003ool00?l0003o00000?oo0000ool00004o`00007ooooo000Do`000003ooooool0003o0000
00?o000000CoOgmoo`000?l0003oool00ol000001?oVi^Ko0000oooo0?oo0004o`000009ogmoOol0
07ooOgmoo`3o0?l0003o0?ooo`000?oo0?oo0?oo00Oo000000?oOgmoool00?oo00002?l000000ooo
003o0000o`00003oo`000?oo00001?l00001ooooo`005?l000000ooooooo0000o`000002o`000005
ogmoOooo0?oo0000oooo0?l0O`000ol000001?oVi^Ko0000oooo0?oo0004o`00000:ogmoOol007oo
Ogmoo`3o0?l0003o0?ooo`000?oo0?oo0000o`3oo`Ko000000?oOgmoool00?oo00002?l000000ooo
003o0000o`00003oo`000?oo00001?l00001ooooo`005?l000000ooooooo0000o`000002o`000005
ogmoOooo0?oo0000oooo0?l0O`000ol000001?oVi^Ko0000o`000?oo0004o`0000;oOgmo00So0000
o`3o0?l0003o0?ooo`000?oo0?oo0000o`3oo`Oo00000_oo0009o`000003ool00?l0003o00000?oo
0000ool00003o`00007ooooo000Do`000003ooooool0003o000000;o000000GoOgmoo`00Ool0003o
ool0o`1o0003o`000004onKVi_l0003o0000ool000Go000000KoOgmoo`000?l0o`3o0000o`3ooooo
0?l2o`000003o`3oool0003o000000Go000000?oo`00o`000?l000002?l000000ooo003o0000o`00
003oo`000?oo00000ol00001ooooo`005?l00003ooooo`;o000000KoOgmoo`00Ool0003oool0o`1o
0?l00002ogl00004onKVi_mo003oO`00ool000GoO`0000KoOgmoo`000?l0o`3o0000o`3ooooo0?l2
o`000003o`3oool0003o000000Go000000?oo`00o`000?l000002?l0003oool00?Ooo`003_l00001
ooooo`005?l000000ooooooo0000o`000002o`000007o`3oool007oo0000oooo0?l0O`3o0000ogl0
0002o`000003onKVi_l0003oool000Go000000Ko001oo`000?l0003o0?l0o`3ooooo0?l2o`000003
o`3oool0003o000000Go000000?o0?ooo`000?l000002?l000000oooo`3o0000o`00003oo`000?oo
00000ol00001ooooo`005?l000000ooooooo0000o`000002o`000007o`3oool007oo0000oooo0?l0
O`3o0000ogl00002o`000003onKVi_l0003oool000Go000000Ko001oo`000?l0003o0?l0o`3ooooo
0?l2o`000003o`3oool0003o000000Go000000?o0?ooo`000?l000002?l000000oooo`3o0000o`00
003oo`000?oo00000ol00001ooooo`005?l000000ooooooo0000o`000002o`000004o`3oool007oo
0000oooo00;o000000KoO`00o`000?l0003oi^KVo`000?ooo`04o`000007o`00Ool0003o0000o`00
0?l0o`3o0?ooool0o`02o`000003o`3oool0003o000000Go00000_l0ool9o`000003oooo0?l0003o
00000?oo0000ool00003o`00007ooooo000Do`000003ooooool0003o000000;o000000Co0?ooo`00
Ool0003oool00_l000001_mo003o0000o`000?oVi^Ko0000oooo00Co000000Ko001oo`000?l0003o
0000o`3o0?oo0?l3o`000003o`3oool0003o000000Co000000?o0?ooo`000?l0ool02?l000000ooo
o`3o0000o`00003oo`000?oo00001?l00001ooooo`005?l000000ooooooo0000o`000003o`000003
o`3oooooo`3o07l000;o000000CoO`00o`000?l0003oi^KV0_l000001Oooo`3o0000o`000?l0003o
001o00?o000000?o0?l0ool0ool000000ol000000ol0oooo0000o`000003o`000003o`3oool0003o
0?oo00So000000?oool0o`000?l00000ool0003oo`0000Co00000Oooool001Co000000?oooooo`00
0?l000000ol000000ol0ooooool0o`1o0002o`000004ogl00?l0003o0000onKViP;o000000Goool0
o`000?l0003o0000o`00O`03o`000003o`3o0?oo0?oo000000?o000000?o0?ooo`000?l000000ol0
00000ol0oooo0000o`3oo`08o`000003oooo0?l0003o00000?oo0000ool00004o`00007ooooo000D
o`000003ooooool0003o000000?o000000?o0?oooooo0?l0O`000_l000001?mo003o0000o`000?oV
i^H2o`000004oooo0?l0003o0000o`00O`Co000000?o0?l0ool0ool000000ol000000ol0oooo0000
o`000003o`000004o`3oool0003o0000o`3oo`Oo000000?oool0o`000?l00000ool0003oo`0000Co
00000Oooool001Co000000?oooooo`000?l000000ol000000ol0ooooool0o`1o0002o`000004ogl0
0?l0003o0000onKViP;o000000Coool0o`000?l0003o001o1?l000000ol0o`3oo`3oo`000003o`00
0003o`3oool0003o000000?o000000Co0?ooo`000?l0003o0?oo1ol000000oooo`3o0000o`00003o
o`000?oo00001?l00001ooooo`005?l000000ooooooo0000o`000003o`000003o`3oooooo`3o07l0
00;o000000CoO`00o`000?l0003oi^KV0_l000001?ooo`3o0000o`000?l007l4o`000003o`3o0?l0
003o000000?o000000?o0?ooo`000?l000000_l000001Ol0oooo0000o`000?l0003o0?oo00Ko0000
00?oool0o`000?l00000ool0003oo`0000Go00000Oooool001Co000000?oooooo`000?l000000ol0
00000ol0ooooool0o`1o0002o`000004ogl00?l0003o0000onKViP;o000000Coool0o`000?l0003o
001o1?l000000ol0o`3o0000o`000003o`000003o`3oool0003o000000;o000000Go0?ooo`000?l0
003o0000o`3oo`06o`000003oooo0?l0003o00000?oo0000ool00005o`00007ooooo000Do`000003
ooooool0003o000000?o000000?o0?oooooo0?l0O`000_l000001omo003o0000o`000?l0003oi^KV
o`000?ooo`000_l000000ol007oo0000o`000002o`000003o`3o0?l0003o000000?o000000?o0?oo
o`000?l000000_l000001Ol0oooo0000o`000?l0003o0?oo00Ko000000?oool0o`000?l00000ool0
003oo`0000Go00000Oooool001Co000000?oooooo`000?l000000ol000001Ol0ooooool0o`1o0?l0
003oO`0000Co000000Goi^KVo`000?ooo`3o0000o`00O`05o`000003o`3o0?l0003o000000?o0000
00?o0?ooo`000?l000000_l000000ol0oooo0000o`000002o`000003o`3oool0003o000000?o0000
00?oool0o`000?l00000ool0003oo`0000Go00000Oooool001Co00000oooool3o`000005o`3ooooo
o`3o07l0o`000?mo00001?l000001OoVi^Ko0000oooo0?l0003o001o00Go000000?o0?l0o`000?l0
00000ol000000ol0oooo0000o`000002o`000003o`3oool0003o000000;o000000?o0?ooo`000?l0
00000ol000000oooo`3o0000o`00003oo`000?oo00001Ol00001ooooo`005?l000000ooooooo0000
o`000003o`000005oooo0?l0oooo07l0o`000?mo00001?l000001OoVi^Ko0000o`000?ooo`3o001o
00Co000000?oo`3oo`3o0?l000001Ol000001?l0oooo0000o`000?l0ool5o`000003o`3oool0003o
000000;o000000?oool0o`000?l00000ool0003oo`0000Ko00000Oooool001Co000000?oooooo`00
0?l000000ol000001Oooo`3o0?ooo`1o0?l0003oO`0000Co000000Goi^KVo`000?l0003oool0o`00
O`04o`000003ool0ool0o`3o000000Go000000Co0?ooo`000?l0003o0?oo1Ol000000ol0oooo0000
o`000002o`000003oooo0?l0003o00000?oo0000ool00006o`00007ooooo000Do`000003ooooool0
003o000000?o000000Goool0o`3oool0O`3o0000ogl00004o`000004onKVi_l0003o0000oooo00Go
000000?oo`3oo`3o0?l000001Ol000001?l0oooo0000o`000?l0ool6o`000005o`3oool0003o0000
o`000?ooo`00ool0003oo`0000So00000Oooool001Co000000?oooooo`000?l000000ol000001Ooo
o`3o0?ooo`1o0?l0003oO`0000Co000000Coi^KVo`000?l0003oool01Ol000000ooo0?oo0?l0o`00
0005o`000004o`3oool0003o0000o`3oo`Ko000000Go0?ooo`000?l0003o0000oooo003oo`000?oo
00002?l00001ooooo`005?l000000ooooooo0000o`000003o`000005oooo0?l0oooo07l0o`000?mo
00001?l000001?oVi^Ko0000o`000?ooo`05o`000003ool0ool0o`3o000000Go000000Co0?ooo`00
0?l0003o0?oo1_l000001Ol0oooo0000o`000?l0003oool00?oo0000ool00008o`00007ooooo000D
o`000003ooooool0003o000000?o000000Goool0o`3oool0O`3o0000ogl00004o`000004onKVi_l0
003o0000oooo00Co000000?oo`3oo`000?l0o`001_l000000ol0oooo0000o`3oo`07o`000004o`3o
ool0003o0000oooo0?oo0000ool00009o`00007ooooo000Do`000003ooooool0003o000000?o0000
00Goool0o`3oool0O`3o0000ogl00004o`000004onKVi_l0003o001ooooo00Co000000?oo`3oo`00
0?l0o`001_l000000ol0oooo0000o`3oo`08o`000003o`3oool0003oool00?oo0000ool00009o`00
007ooooo000Do`000003ooooool0003o000000?o000000Goool0o`3oool0O`3o0000ogl00004o`00
0004onKVi_l0003o001ooooo00Co000000?oo`3oo`000?l0o`001_l000000ol0oooo0000o`3oo`08
o`000003o`3oool0003oool00?oo0000ool00009o`00007ooooo000Do`000003ooooool0003o0000
00?o000000Goool0o`3oool0O`3o0000ogl00005o`000003onKVi_l007ooool000Co000000Coo`3o
o`000?l0oooo0?l01Ol000000ol0oooo0000o`3oo`08o`000003o`3oool0003oool00?oo0000ool0
0009o`00007ooooo0009o`0000?ooooo2?l000000ooooooo0000o`000002o`000006oooo0?l007oo
0000o`3oool0003oO`001Ol000001?oVi^Ko001oo`000?ooo`03o`000004ool0ool0003o0?ooo`3o
00Ko00000_l0ool8o`000003o`3oool0003oool00?oo0000ool00009o`00007ooooo000:o`000003
ooooool0003o000000Oo000000?oooooo`000?l000000_l000001_ooo`3o001oo`000?l0oooo0000
ogl000Go000000Coi^KVo`00Ool0003oool00_l000001Ooo0?oo0000o`000?l0oooo0?l000Ko0000
00?o0?ooo`000?l000001ol000000ol0ooooool0o`00003oo`000?oo00002Ol00001ooooo`001ol0
0005ooooo`So000000?oooooo`000?l000000_l000001_ooo`3o001oo`1o0?l0oooo0000ogl000Go
000000Coi^KVo`000?l0003oool00_l000001Ooo0?oo0000o`000?l0oooo0?l000Ko000000?o0?oo
o`000?l000002?l000000oooo`3o0000o`00003oo`000?oo00002?l00001ooooo`001ol000001?oo
oooo0000o`000?ooool9o`0000Cooooo00Oo0000oooo0?l007oo07l0o`3oool0003oO`0000Go0000
1ooVi^H2o`000003o`3oool0o`3o000000Go000000?o0?ooo`000?l000000_l000000ol0o`3o0000
o`000003o`000003oooo0?l0003o00000?oo0000ool00008o`00007ooooo0008o`000003ooooool0
003ooooo00Wo000000?oooooo`000?l000001?l000001?l0O`3o0?ooo`000?mo0005o`000004o`3o
ool0003o0000oooo00;o000000Goi^KVo`000?l0oooo0000o`3o000;o`000003o`3o0?l0003o0000
00?o000000?oool0o`000?l00000ool0003oo`0000So00000Oooool000Wo00000_ooool9o`000003
ooooool0003o000000Co000000Co07l0o`3oool0003oO`001Ol000001?l0oooo0000o`000?ooo`02
o`000005onKVi_l0003o0?ooo`000?l0o`002ol000000ol0o`3o0000o`000003o`000003oooo0?l0
003o00000?oo0000ool00008o`00007ooooo000:o`000003ooooool0003o000000Oo000000?ooooo
o`000?l000001?l000001?l0O`3o0?ooo`000?mo0005o`000004o`3oool0003o0000oooo00;o0000
00Goi^KVo`000?l0oooo0000o`3o000;o`0000;o0?l00ol000000oooo`3o0000o`00003oo`000?oo
00002Ol00001ooooo`005?l000000ooooooo0000o`000004o`000004o`1o0?l0oooo0000ogl000Co
00000_l0ool2o`000008oooo0?l0003o0000onKVi_l0003o0?ooo`000?l0o`0:o`000003o`3o0?l0
003o0?l000?o000000?oool0o`000?l00000ool0003oo`0000Wo00000Oooool001Co000000?ooooo
o`000?l000001?l000001?l0O`3o0000o`3ooomo0004o`000003o`3oool007oo0?oo00;o000000Oo
ool0o`000?oVi^Ko0000o`3oool0003o0?l000[o000000?o0?l0o`000?l0o`000ol000000oooo`3o
0000o`00003oo`000?oo00002Ol00001ooooo`005?l000000ooooooo0000o`000004o`000004o`1o
0?l0003o0?ooogl000Co000000?o0?ooo`00Ool0ool00_l000001oooo`3o0000onKVi_l0003o0?oo
o`000?l0o`002_l000000ol0o`3o0000o`3o0003o`000003oooo0?l0003o00000?oo0000ool00009
o`00007ooooo000Do`000003ooooool0003o000000Co000000?o07l0o`000?l0ool01Ol000000ol0
oooo001oo`3oo`02o`000007oooo0?l0003oi^KVo`000?l0oooo0000o`3o000:o`000006o`3o0?l0
003o0000o`3o0?l0003oool0ool0003oo`0000co00000Oooool001Co000000?oooooo`000?l00000
1?l000000ol0O`3o0000o`3oo`05o`000003o`3oool007oo0?oo00;o000000Ooool0o`000?oVi^Ko
0000o`3oool0003o0?l000[o000000Ko0?l0o`000?l0003o0?l0o`000?ooo`3oo`000?oo00003?l0
0001ooooo`005?l000000ooooooo0000o`000004o`000003o`1o0?l0003o0?oo00Co000000Co0?oo
o`000?l007oo0?oo0_l000001oooo`3o0000o`000?oVi^Ko0?ooo`000?l0o`002Ol000001ol0o`3o
0000o`000?l0003o0?l0o`000?ooo`00ool0003oo`0000co00000Oooool001Co000000?oooooo`00
0?l000001?l000000ol0O`3o0000o`3oo`04o`000004o`3oool0003o001oo`3oo`;o000000Ooool0
o`000?l0003oi^KVo`3oool0003o0?l000Wo000000Oo0?l0o`000?l0003o0000o`3o0?l0003oool0
0?oo0000ool0000<o`00007ooooo000Do`000003ooooool0003o000000Co000000?o07l0o`000?l0
ool01?l000001?l0oooo0000o`00Ool0ool2o`000007oooo0?l0003o0000onKVi_l0oooo0000o`3o
0009o`000007o`3o0?l0003o0000o`000?l0o`3o0000oooo003oo`000?oo00003?l00001ooooo`00
5?l000000ooooooo0000o`000004o`000003o`1o0?l0003o0?oo00Co000000Co0?ooo`000?l007oo
0?oo0_l000001oooo`3o0000o`000?oVi^Ko0?ooo`000?l0o`002Ol000000ol0o`3o0000o`000002
o`000003o`3o0?l0003o00000?oo0000ool0000;o`00007ooooo000Do`0000?ooooo1?l000000ol0
O`3o0000o`3oo`04o`000004o`3oool0003o001oo`3oo`;o000000Ooool0o`000?l0003oi^KVo`3o
ool0003o0?l000Wo000000?o0?l0o`000?l000000_l000000ol0o`3o0000o`00003oo`000?oo0000
2ol00001ooooo`005?l000000ooooooo0000o`000004o`000004o`1o0?l0003oO`00o`3oo`;o0000
00Ko0?ooo`000?l0003o001oo`000?l0ool2o`000006oooo0?l0003oi^KVo`3oool0003o0?l02?l0
00000ol0o`3o0000o`000003o`000003o`3o0?l0003o00000?oo0000ool0000;o`00007ooooo000D
o`000003ooooool0003o000000Co000000Co07l0o`000?mo003o0?oo0_l000001_l0oooo0000o`00
0?l007oo0000o`3oo`;o000000Koool0o`000?oVi^Ko0?ooo`000?l0o`08o`000003o`3o0?l0003o
000000?o000000?o0?l0o`000?l00000ool0003oo`0000_o00000Oooool001Co000000?oooooo`00
0?l000001?l000001?l0O`3o0000ogl00?l0ool2o`000006o`3oool0003o0000o`00Ool0003o0?oo
0_l000001_ooo`3o0000onKVi_l0oooo0000o`3o00So000000?o0?l0o`000?l000000_l000000ooo
o`3o0000o`3o003oo`000?oo00003?l00001ooooo`005?l000000ooooooo0000o`000004o`000004
o`1o0?l0003oO`00o`3oo`;o000000Ko0?ooo`000?l0003o001oo`000?l0ool2o`000004oooo0?l0
003oi^KVo`3oo`;o000000?o0?l0o`000?l000001Ol000000ol0o`3o0000o`000002o`000003oooo
0?l0003o0?l00?oo0000ool0000<o`00007ooooo000Do`000003ooooool0003o000000Co000000Co
07l0o`000?mo003o0?oo0_l000001_l0oooo0000o`000?l007oo0000o`3oo`;o000000Coool0o`00
0?oVi^Ko0?oo0_l000000ol0o`3o0000o`000005o`000003o`3o0?l0003o000000;o000000?oool0
o`000?l0o`00ool0003oo`0000co00000Oooool001Co000000?oooooo`000?l000001?l000001_l0
O`3o0000ogl00?l0oooo0000o`3oo`?o000000?o001oo`000?l0ool00_l000001?ooo`3o0000onKV
i_l0ool2o`000003o`3o0?l0003o000000Co000000?o0?l0o`000?l000000ol000000oooo`3o0000
o`3o003oo`000?oo00003?l00001ooooo`005?l000000ooooooo0000o`000003o`000007o`1o0?l0
07oo0000ogl00?l0oooo0000o`3oo`03o`000003o`00Ool0003o0?oo00;o000000Coool0o`000?oV
i^Ko0?oo0_l000000ol0o`3o0000o`000004o`000003o`3o0?l0003o000000;o000000Goool0o`00
0?l0003o0000o`3o003oo`000?oo00002ol00001ooooo`005?l000000ooooooo0000o`000003o`00
0007o`1o0?l007oo0000ogl00?l0oooo0000o`3oo`03o`000003o`00Ool0003o0?oo00;o000000Co
ool0o`000?oVi^Ko0?oo0_l000000ol0o`3o0000o`000004o`000003o`3o0?l0003o000000;o0000
00Goool0o`000?l0003o0000o`3o003oo`000?oo00002ol00001ooooo`005?l000000ooooooo0000
o`000003o`000007o`1o0?l007oo0000ogl00?l0oooo0000o`3oo`03o`000003o`00Ool0003o0?oo
00;o000000?oool0o`000?oVi^H00ol000000ol0o`3o0000o`000004o`000003o`3o0?l0003o0000
00;o000000Goool0o`000?l0003o0000o`3o003oo`000?oo00002ol00001ooooo`005?l000000ooo
oooo0000o`000003o`000005o`1o0?l007oo0000ogl00?l000000_l0ool3o`000004o`00Ool0003o
0000o`3oo`;o000000?oool0onKVi_l000000_l000000ol0o`3o0000o`000004o`000003o`3o0?l0
003o000000;o000000Goool0o`000?l0003o0000o`3o003oo`000?oo00002ol00001ooooo`005?l0
00000ooooooo0000o`000003o`000006o`1o0?l007oo0000ogl00?l0003o0?oo0ol000001Ol007oo
0000o`000?l0003o0?oo00;o000000?oool0o`3ooooVi^H00_l000000ol0o`3o0000o`000003o`00
0003o`3o0?l0003o000000?o000000Goool0o`000?l0003o0000o`3o003oo`000?oo00002ol00001
ooooo`005?l000000ooooooo0000o`000003o`000006o`1o0?l007oo0000ogl00?l0003o0?oo0ol0
00001Ol007oo0000o`000?l0003o0?oo00;o000000?oool0o`3ooooVi^H00_l000000ol0o`3o0000
o`000003o`000003o`3o0?l0003o000000;o000000?oool0o`000?l000000ol000000ol0o`3o0000
o`00003oo`000?oo00002?l00001ooooo`005?l00003ooooo`?o000000Ko07l0o`00Ool0003oO`00
o`000?l0ool3o`000005o`00Ool0003o0000o`000?l0ool00_l00002oooo0003onKVi_ooo`3oool0
00Oo0?l01Oooo`05o`000003o`3o0?l0003o00000?oo0000ool00008o`00007ooooo000Do`000003
ooooool0003o000000?o000000Co07l0o`00Ool0003oO`001Ol000001Ol007oo0000o`000?l0003o
0?oo00?o000000?o0?ooonKVi_l000004_l000000ol0o`3o0000o`00003oo`000?oo00002?l00001
ooooo`005?l000000ooooooo0000o`000003o`000004o`1o0?l007oo0000ogl000Go000000Go001o
o`000?l0003o0000o`3oo`03o`000003o`3ooooVi^Ko000001;o000000?o0?l0o`000?l00000ool0
003oo`0000So00000Oooool001Co000000?oooooo`000?l000000ol000000ol0O`3o001oogl00006
o`000005o`00Ool0003o0000o`000?l0ool00ol000000ol0ooooi^KVo`00000Bo`000003o`3o0?l0
003o00000?oo0000ool00008o`00007ooooo000Do`000003ooooool0003o000000?o000000?o07l0
o`00Oomo00001_l000001Ol007oo0000o`000?l0003o0?oo00?o000000?o0?ooonKVi_l000004ol0
00000ol0o`3o0000o`00003oo`000?oo00001ol00001ooooo`005?l000000ooooooo0000o`000003
o`000003o`1o0?l007ooO`0000Ko000000?o001oo`000?l000000_l000001Ol0oooo0000o`000?l0
ooooi^KV01Co000000?o0?l0o`000?l00000ool0003oo`0000Oo00000Oooool001Co000000?ooooo
o`000?l000000ol000000ol0O`3o001oogl00006o`000003o`00Ool0003o000000;o000000Go0?oo
o`000?l0003o0?ooonKViP0Do`000003o`3o0?l0003o00000?oo0000ool00007o`00007ooooo000D
o`000003ooooool0003o000000?o000000?o07l0o`00Oomo00001_l000000ol007oo0000o`000002
o`000005o`3oool0003o0000o`3ooooVi^H05?l000000ol0o`3o0000o`00003oo`000?oo00001ol0
0001ooooo`005?l000000ooooooo0000o`000003o`000003o`1o0?l007ooO`0000Ko000000?o001o
o`000?l000000_l000001Ol0oooo0000o`000?l0ooooi^KV01Co000000?o0?l0o`000?l00000ool0
003oo`0000Oo00000Oooool001Co000000?oooooo`000?l000000ol000000ol0O`3o001oogl00006
o`000003o`00Ool0003o000000;o000000Go0?ooo`000?l0003o0?ooonKViP0Eo`000003o`3o0?l0
003o00000?oo0000ool00006o`00007ooooo000Do`000003ooooool0003o000000?o000000?o07l0
o`00Oomo00001_l000000ol007oo0000o`000002o`000005o`3oool0003o0000o`3ooooVi^H05Ol0
00000ol0o`3o0000o`00003oo`000?oo00001_l00001ooooo`005?l000000ooooooo0000o`000003
o`000003o`1o0?l007ooO`0000Ko000000?o001oo`000?l000000_l000001Ol0oooo0000o`000?l0
ooooi^KV01Go000000?o0?l0o`000?l00000ool0003oo`0000Ko00000Oooool001Co000000?ooooo
o`000?l000000ol000000ol0O`3o001oogl00006o`000003o`00Ool0003o000000;o000000Go0?oo
o`000?l0003o0?ooonKViP0Eo`000003o`3o0?l0003o00000?oo0000ool00006o`00007ooooo000D
o`0000?ooooo0ol000000ol0O`3o001oogl00006o`000003o`00Ool0003o000000;o000000Go0?oo
o`000?l0003o0?ooonKViP0Eo`000003o`3o0?l0003o00000?oo0000ool00006o`00007ooooo000D
o`000003ooooool0003o000000?o000000?o07l0o`00Oomo00001_l000000ol007oo0000o`000003
o`000005o`3oool0003o0?ooo`000?oVi^H05Ol000000ol0o`3o0000o`00003oo`000?oo00001Ol0
0001ooooo`005?l000000ooooooo0000o`000003o`000003o`1o0?l007ooO`0000Ko000000?o001o
o`000?l000000ol000001Ol0oooo0000o`3oool0003oi^KV01Go000000?o0?l0o`000?l00000ool0
003oo`0000Go00000Oooool001Co000000?oooooo`000?l000000_l000001?l0O`3o0000o`00Oomo
0006o`000003o`00Ool0003o000000?o000000Go0?ooo`000?l0oooo0000onKViP0Eo`000003o`3o
0?l0003o00000?oo0000ool00005o`00007ooooo000Do`000003ooooool0003o000000;o000000Co
07l0o`000?l007ooO`001_l000000ol007oo0000o`000003o`0000;o0?oo0_l000000ooVi^Ko0000
o`00000Co`000003o`3o0?l0003o00000?oo0000ool00005o`00007ooooo000Do`000003ooooool0
003o000000;o000000Co07l0o`000?l007ooO`001_l000000ol007oo0000o`000003o`0000;o0?oo
0_l000000ooVi^Ko0000o`00000Co`000003o`3o0?l0003o00000?oo0000ool00005o`00007ooooo
000Do`000003ooooool0003o000000;o000000Co07l0o`000?l007ooO`001_l000000ol007oo0000
o`000003o`0000;o0?oo0_l000000ooVi^Ko0000o`00000Do`000003o`3o0?l0003o00000?oo0000
ool00004o`00007ooooo000Do`000003ooooool0003o000000;o000000Co07l0o`000?l007ooO`00
1_l000000ol007oo0000o`000003o`0000;o0?oo0_l000000ooVi^Ko0000o`00000Do`000003o`3o
0?l0003o00000?oo0000ool00004o`00007ooooo000Do`000003ooooool0003o000000;o000000Co
07l0o`000?l007ooO`001_l000000ol007oo0000o`000003o`0000;o0?oo0_l000000ooVi^Ko0000
o`00000Do`000003o`3o0?l0003o00000?oo0000ool00004o`00007ooooo000Do`000003ooooool0
003o000000;o000000Co07l0o`000?l007ooO`001_l000000ol007oo0000o`000003o`0000;o0?oo
0_l000000ooVi^Ko0000o`00000Do`000003o`3o0?l0003o00000?oo0000ool00004o`00007ooooo
0008o`0000?ooooo2Ol000000ooooooo0000o`000002o`000004o`1o0?l0003o001oogl000Ko0000
00?o001oo`000?l000001?l000001?l0oooo0000o`000?oVi^HFo`000003o`3o0?l0003o00000?oo
0000ool00004o`00007ooooo0007o`000005ooooool0003o0000o`000?ooool02?l000000ooooooo
0000o`000002o`000004o`1o0?l0003o001oogl000Ko000000?o001oo`000?l000001?l000001?l0
oooo0000o`000?oVi^HGo`000003o`3o0?l0003o00000?oo0000ool00003o`00007ooooo0007o`00
0005ooooool0003o0000o`000?ooool02?l000000ooooooo0000o`000002o`000003o`1o0?l0003o
O`0000Oo000000?o001oo`000?l000001?l000001?l0oooo0000o`000?oVi^HGo`000003o`3o0?l0
003o00000?oo0000ool00003o`00007ooooo0007o`0000Cooooo2Ol00004ooooo`04o`000?l0O`3o
0000ogl000Oo000000?o001oo`000?l000001?l000001?l0oooo0000o`000?oVi^HGo`000?oo0?l0
mol0o`0>o`00007ooooo0007o`000003ooooool0003o000000[o000000?oooooo`000?l000001?l0
00000omo003o001oo`000005o`000003o`00Ool0003o000000Oo000000?oi^KVo`000?l00000ool0
003oo`0001_o00000Oooool000So000000?oooooo`000?l000002Ol000000ooooooo0000o`000004
o`000003ogl00?l007oo000000Go000000?o001oo`000?l000001ol000000ooVi^Ko0000o`00003o
o`000?oo00006ol00001ooooo`002Ol00002ooooo`Wo000000?oooooo`000?l000001?l000000omo
003o001oo`000005o`000003o`00Ool0003o000000Oo000000?oi^KVo`000?l00000ool0003oo`00
01_o00000Oooool001Co000000?oooooo`000?l000001?l000000omo003o001oo`000005o`000003
o`00Ool0003o000000So000000?oi^KVo`000?l00000ool0003oo`0001[o00000Oooool001Co0000
00?oooooo`000?l000001?l000000omo003o001oo`000005o`000003o`00Ool0003o000000So0000
00?oi^KVo`000?l00000ool0003oo`0001[o00000Oooool001Co000000?oooooo`000?l000001?l0
00000omo003o001oo`000005o`000003o`00Ool0003o000000So000000?oi^KVo`000?l00000ool0
003oo`0001[o00000Oooool001Co000000?oooooo`000?l000001?l000000omo003o001oo`000005
o`000003o`00Ool0003o000000So000000?oi^KVo`000?l00000ool0003oo`0001[o00000Oooool0
01Co000000?oooooo`000?l000001?l000000omo003o001oo`000005o`000003o`00Ool0003o0000
00So000000?oi^KVo`000?l00000ool0003oo`0001[o00000Oooool001Co000000?oooooo`000?l0
00001?l000000omo003o001oo`000005o`000003o`00Ool0003o000000So000000?oi^KVo`000?l0
0000ool0003oo`0001[o00000Oooool001Co000000?oooooo`000?l000001?l000000omo003o001o
o`000005o`000003o`00Ool0003o000000So000000?oi^KVo`000?l00000ool0003oo`0001[o0000
0Oooool001Co000000?oooooo`000?l000001?l000000omo003o001oo`000005o`000003o`00Ool0
003o000000So000000?oi^KVo`000?l00000ool0003oo`0001[o00000Oooool001Co000000?ooooo
o`000?l000001?l000000omo003o001oo`000005o`000003o`00Ool0003o000000So000000?oi^KV
o`000?l00000ool0003oo`0001[o00000Oooool001Co00000oooool4o`000003ogl00?l007oo0000
00Go000000?o001oo`000?l000002?l000000ooVi^Ko0000o`00003oo`000?oo00006_l00001oooo
o`005?l000000ooooooo0000o`000004o`000003ogl00?l007oo000000Go000000?o001oo`000?l0
00002?l000000ooVi^Ko0000o`00003oo`000?oo00006_l00001ooooo`005?l000000ooooooo0000
o`000004o`000003ogl00?l007oo000000Go000000?o001oo`000?l000002?l000000ooVi^Ko0000
o`00003oo`000?oo00006_l00001ooooo`005?l000000ooooooo0000o`000004o`000003ogl00?l0
07oo000000Go000000?o001oo`000?l000002?l000000ooVi^Ko0000o`00003oo`000?oo00006_l0
0001ooooo`005?l000000ooooooo0000o`000004o`000003ogl00?l007oo000000Go000000?o001o
o`000?l000002?l000000ooVi^Ko0000o`00003oo`000?oo00006_l00001ooooo`005?l000000ooo
oooo0000o`000004o`000003ogl00?l007oo000000Go000000?o001oo`000?l000002?l000000ooV
i^Ko0000o`00003oo`000?oo00006_l00001ooooo`005?l000000ooooooo0000o`000004o`000003
ogl00?l007oo000000Co000000?o001oo`000?l000002_l000000ooVi^Ko0000o`00003oo`000?oo
00006Ol00001ooooo`005?l000000ooooooo0000o`000003o`000003ogl00?l0003o001o00Go0000
00?o001oo`000?l000002_l000000ooVi^Ko0000o`00003oo`000?oo00006Ol00001ooooo`005?l0
00000ooooooo0000o`000003o`000003ogl00?l0003o001o00Go000000?o001oo`000?l000002_l0
00000ooVi^Ko0000o`00003oo`000?oo00006Ol00001ooooo`005?l000000ooooooo0000o`000003
o`000003ogl00?l0003o001o00Go000000?o001oo`000?l000002_l000000ooVi^Ko0000o`00003o
o`000?oo00006Ol00001ooooo`005?l000000ooooooo0000o`000003o`000003ogl00?l0003o001o
00Go000000?o001oo`000?l000002_l000000ooVi^Ko0000o`00003oo`000?oo00006Ol00001oooo
o`005?l000000ooooooo0000o`000003o`000003ogl00?l0003o001o00Go000000?o001oo`000?l0
00002_l000000ooVi^Ko0000o`00003oo`000?oo00006Ol00001ooooo`005?l000000ooooooo0000
o`000003o`000003ogl00?l0003o001o00Go000000?o001oo`000?l000002_l000000ooVi^Ko0000
o`00003oo`000?oo00006Ol00001ooooo`005?l00003ooooo`?o000000?oO`00o`000?l007l01Ol0
00000ol007oo0000o`00000:o`000003onKVi_l0003o00000?oo0000ool0000Io`00007ooooo000D
o`000003ooooool0003o000000?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000:
o`000003onKVi_l0003o00000?oo0000ool0000Io`00007ooooo000Do`000003ooooool0003o0000
00?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000:o`000003onKVi_l0003o0000
0?oo0000ool0000Io`00007ooooo000Do`000003ooooool0003o000000?o000000?oO`00o`000?l0
07l01Ol000000ol007oo0000o`00000:o`000003onKVi_l0003o00000?oo0000ool0000Io`00007o
oooo000Do`000003ooooool0003o000000?o000000?oO`00o`000?l007l01Ol000000ol007oo0000
o`00000:o`000003onKVi_l0003o00000?oo0000ool0000Io`00007ooooo000Do`000003ooooool0
003o000000?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000;o`000003onKVi_l0
003o00000?oo0000ool0000Ho`00007ooooo000Do`000003ooooool0003o000000?o000000?oO`00
o`000?l007l01Ol000000ol007oo0000o`00000;o`000003onKVi_l0003o00000?oo0000ool0000H
o`00007ooooo000Do`000003ooooool0003o000000?o000000?oO`00o`000?l007l01Ol000000ol0
07oo0000o`00000;o`000003onKVi_l0003o00000?oo0000ool0000Ho`00007ooooo000Do`000003
ooooool0003o000000?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000;o`000003
onKVi_l0003o00000?oo0000ool0000Ho`00007ooooo000Do`000003ooooool0003o000000?o0000
00?oO`00o`000?l007l01Ol000000ol007oo0000o`00000;o`000003onKVi_l0003o00000?oo0000
ool0000Ho`00007ooooo000Do`000003ooooool0003o000000?o000000?oO`00o`000?l007l01Ol0
00000ol007oo0000o`00000;o`000003onKVi_l0003o00000?oo0000ool0000Ho`00007ooooo000D
o`000003ooooool0003o000000?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000;
o`000003onKVi_l0003o00000?oo0000ool0000Ho`00007ooooo000Do`000003ooooool0003o0000
00?o000000?oO`00o`000?l007l01Ol000000ol007oo0000o`00000;o`000003onKVi_l0003o0000
0?oo0000ool0000Ho`00007ooooo000Do`0000?ooooo0ol000000omo003o0000o`00O`05o`000003
o`00Ool0003o000000_o000000?oi^KVo`000?l00000ool0003oo`0001So00000Oooool001Co0000
00?oooooo`000?l000000ol000000omo003o0000o`00O`05o`000003o`00Ool0003o000000co0000
00?oi^KVo`000?l00000ool0003oo`0001Oo00000Oooool001Co000000?oooooo`000?l000000ol0
00000omo003o0000o`00O`05o`000003o`00Ool0003o000000co000000?oi^KVo`000?l00000ool0
003oo`0001Oo00000Oooool001Co000000?oooooo`000?l000000_l000001?mo003o0000o`000?l0
07l5o`000003o`00Ool0003o000000co000000?oi^KVo`000?l00000ool0003oo`0001Oo00000Ooo
ool001Co000000?oooooo`000?l000000_l000001?mo003o0000o`000?l007l5o`000003o`00Ool0
003o000000co000000?oi^KVo`000?l00000ool0003oo`0001Oo00000Oooool001Co000000?ooooo
o`000?l000000_l000001?mo003o0000o`000?l007l5o`000003o`00Ool0003o000000co000000?o
i^KVo`000?l00000ool0003oo`0001Oo00000Oooool001Co000000?oooooo`000?l000000_l00000
1?mo003o0000o`000?l007l5o`000003o`00Ool0003o000000co000000?oi^KVo`000?l00000ool0
003oo`0001Oo00000Oooool001Co000000?oooooo`000?l000000_l000001?mo003o0000o`000?l0
07l5o`000003o`00Ool0003o000000co000000?oi^KVo`000?l00000ool0003oo`0001Oo00000Ooo
ool001Co000000?oooooo`000?l000000_l000001?mo003o0000o`000?l007l5o`000003o`00Ool0
003o000000co000000?oi^KVo`000?l00000ool0003oo`0001Oo00000Oooool001Co000000?ooooo
o`000?l000000_l000001?mo003o0000o`000?l007l5o`000003o`00Ool0003o000000go000000?o
i^KVo`000?l00000ool0003oo`0001Ko00000Oooool000So00000oooool9o`000003ooooool0003o
000000;o000000CoO`00o`000?l0003o001o1Ol000000ol007oo0000o`00000=o`000003onKVi_l0
003o00000?oo0000ool0000Fo`00007ooooo0007o`000005ooooool0003o0000o`000?ooool02?l0
00000ooooooo0000o`000002o`000004ogl00?l0003o0000o`00O`Go000000?o001oo`000?l00000
3Ol000000ooVi^Ko0000o`00003oo`000?oo00005_l00001ooooo`001ol000001Ooooooo0000o`00
0?l0003ooooo00So000000?oooooo`000?l000000_l000001?mo003o0000o`000?l007l5o`000003
o`00Ool0003o000000go000000?oi^KVo`000?l00000ool0003oo`0001Ko00000Oooool000So0000
0oooool9o`0000Cooooo00?o0000ogl00?l000000_l000000ol007oo0000o`000002o`000003o`00
Ool0003o000000go000000?oi^KVo`000?l000001Ol000000ooVi^Ko0000o`00003oo`000?oo0000
3_l00001ooooo`001ol000001Ooooooo0000o`000?l0003ooooo00So000000?oooooo`000?l00000
1_l000000ol007oo0000o`000002o`000003o`00Ool0003o000000go000000?oi^KVo`000?l00000
1Ol000000ooVi^Ko0000o`00003oo`000?oo00003_l00001ooooo`001ol000001Ooooooo0000o`00
0?l0003ooooo00So000000?oooooo`000?l000001_l000000ol007oo0000o`000002o`000003o`00
Ool0003o000000go000000?oi^KVo`000?l000001Ol000000ooVi^Ko0000o`00003oo`000?oo0000
3_l00001ooooo`002?l00003ooooo`Wo000000?oooooo`000?l000001_l000000ol007oo0000o`00
0002o`000003o`00Ool0003o000000go000000?oi^KVo`000?l000001?l000000ooVi^Ko0000onKV
iP3oo`000?oo00003ol00001ooooo`005?l000000ooooooo0000o`000006o`000003o`00Ool0003o
000000;o000000?o001oo`000?l000003Ol000000ooVi^Ko0000o`000004o`000003onKVi_l0003o
i^KV0?oo0000ool0000?o`00007ooooo000Do`000003ooooool0003o000000Ko000000?o001oo`00
0?l000000_l000000ol007oo0000o`00000>o`000003onKVi_l0003o000000?o000000?oi^KVo`00
0?oVi^H0ool0003oo`0000oo00000Oooool001Co000000?oooooo`000?l000001_l000000ol007oo
0000o`000002o`000003o`00Ool0003o000000ko000000?oi^KVo`000?l000000ol000000ooVi^Ko
0000onKViP3oo`000?oo00003ol00001ooooo`005?l000000ooooooo0000o`000006o`000003o`00
Ool0003o000000;o000000?o001oo`000?l000003_l000000ooVi^Ko0000o`000003o`000004onKV
i_l0003o0000onKVi_oo0000ool0000>o`00007ooooo000Do`000003ooooool0003o000000Ko0000
00?o001oo`000?l000000_l000000ol007oo0000o`00000>o`000003onKVi_l0003o000000;o0000
00Goi^KVo`000?l0003o0000onKViP3oo`000?oo00003_l00001ooooo`005?l000000ooooooo0000
o`000006o`000003o`00Ool0003o000000;o000000?o001oo`000?l000003_l000000ooVi^Ko0000
o`000002o`000005onKVi_l0003o0000o`000?oVi^H0ool0003oo`0000ko00000Oooool001Co0000
00?oooooo`000?l000001_l000000ol007oo0000o`000002o`000003o`00Ool0003o000000ko0000
00?oi^KVo`000?l000000_l000001OoVi^Ko0000o`000?l0003oi^KV0?oo0000ool0000>o`00007o
oooo000Do`000003ooooool0003o000000Ko000000?o001oo`000?l000000_l000000ol007oo0000
o`00000>o`000003onKVi_l0003o000000;o000000?oi^KVo`000?l000000_l000000ooVi^Ko0000
o`00003oo`000?oo00002ol00001ooooo`005?l000000ooooooo0000o`000006o`000003o`00Ool0
003o000000;o000000?o001oo`000?l000003_l000000ooVi^Ko0000o`000002o`000003onKVi_l0
003o000000;o000000?oi^KVo`000?l00000ool0003oo`0000_o00000Oooool001Co00000oooool6
o`000005o`00Ool0003o0000o`000?l007l04_l000001?oVi^Ko0000o`000?oVi^H5o`000003onKV
i_l0003o00000?oo0000ool0000;o`00007ooooo000Do`000003ooooool0003o000000Ko000000Go
001oo`000?l0003o0000o`00O`0Bo`000004onKVi_l0003o0000onKViPGo000000?oi^KVo`000?l0
0000ool0003oo`0000_o00000Oooool001Co000000?oooooo`000?l000001_l000001Ol007oo0000
o`000?l0003o001o01;o000000Coi^KVo`000?l0003oi^KV1Ol000000ooVi^Ko0000o`00003oo`00
0?oo00002ol00001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`00
0?l007l04_l000001?oVi^Ko0000o`000?oVi^H6o`000003onKVi_l0003o00000?oo0000ool0000:
o`00007ooooo000Do`000003ooooool0003o000000Ko000000Go001oo`000?l0003o0000o`00O`0B
o`000004onKVi_l0003o0000onKViPKo000000?oi^KVo`000?l00000ool0003oo`0000[o00000Ooo
ool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0003o001o01;o000000?o
i^KVo`000?oVi^H01ol000000ooVi^Ko0000o`00003oo`000?oo00002_l00001ooooo`005?l00000
0ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l04_l000000ooVi^Ko0000onKV
iP07o`000003onKVi_l0003o00000?oo0000ool0000:o`00007ooooo000Do`000003ooooool0003o
000000Ko000000Go001oo`000?l0003o0000o`00O`0Bo`000003onKVi_l0003oi^KV00So000000?o
i^KVo`000?l00000ool0003oo`0000Wo00000Oooool001Co000000?oooooo`000?l000001_l00000
1Ol007oo0000o`000?l0003o001o01;o000000?oi^KVo`000?oVi^H02?l000000ooVi^Ko0000o`00
003oo`000?oo00002Ol00001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o
0000o`000?l007l04ol00002onKViPSo000000?oi^KVo`000?l00000ool0003oo`0000Wo00000Ooo
ool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0003o001o01?o000000?o
i^KVo`000?l000001ol000000ooVi^Ko0000o`00003oo`000?oo00002Ol00001ooooo`005?l00000
0ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l04ol000000ooVi^Ko0000o`00
0008o`000003onKVi_l0003o00000?oo0000ool00008o`00007ooooo000Do`0000?ooooo1_l00000
1Ol007oo0000o`000?l0003o001o01?o000000?oi^KVo`000?l000002?l0003oonKVi_coi^KV3_l0
0001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l0ool0
003oo`0002Wo00000Oooool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0
003o001o0?oo0000ool0000Yo`00007ooooo000Do`000003ooooool0003o000000Ko000000Go001o
o`000?l0003o0000o`00O`3oo`000?oo0000:Ol00001ooooo`005?l000000ooooooo0000o`000006
o`000005o`00Ool0003o0000o`000?l007l0ool0003oo`0002Wo00000Oooool001Co000000?ooooo
o`000?l000001_l000001Ol007oo0000o`000?l0003o001o0?oo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000Ko000000Go001oo`000?l0003o0000o`00O`3oo`000?oo0000:Ol0
0001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l0ool0
003oo`0002Wo00000Oooool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0
003o001o0?oo0000ool0000Yo`00007ooooo000Do`000003ooooool0003o000000Ko000000Go001o
o`000?l0003o0000o`00O`3oo`000?oo0000:Ol00001ooooo`005?l000000ooooooo0000o`000006
o`000005o`00Ool0003o0000o`000?l007l0ool0003oo`0002Wo00000Oooool001Co000000?ooooo
o`000?l000001_l000001Ol007oo0000o`000?l0003o001o0?oo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000Ko000000Go001oo`000?l0003o0000o`00O`3oo`000?oo0000:Ol0
0001ooooo`005?l00003ooooo`Ko000000Go001oo`000?l0003o0000o`00O`3oo`000?oo0000:Ol0
0001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l0ool0
003oo`0002Wo00000Oooool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0
003o001o0?oo0000ool0000Yo`00007ooooo000Do`000003ooooool0003o000000Ko000000Go001o
o`000?l0003o0000o`00O`3oo`000?oo0000:Ol00001ooooo`005?l000000ooooooo0000o`000006
o`000005o`00Ool0003o0000o`000?l007l0ool0003oo`0002Wo00000Oooool001Co000000?ooooo
o`000?l000001_l000001Ol007oo0000o`000?l0003o001o0?oo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000Ko000000Go001oo`000?l0003o0000o`00O`3oo`000?oo0000:Ol0
0001ooooo`005?l000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l0ool0
003oo`0002Wo00000Oooool001Co000000?oooooo`000?l000001_l000001Ol007oo0000o`000?l0
003o001o0?oo0000ool0000Yo`00007ooooo000Do`000003ooooool0003o000000Ko000000Go001o
o`000?l0003o0000o`00O`3oo`000?oo0000:Ol00001ooooo`005?l000000ooooooo0000o`000006
o`000005o`00Ool0003o0000o`000?l007l0ool0003oo`0002Wo00000Oooool000;o00000oooool4
o`0000;ooooo2Ol000000ooooooo0000o`000006o`000005o`00Ool0003o0000o`000?l007l0ool0
003oo`0002Wo00000Oooool000?o000000?oooooo`000?l000000_l000001?oooooo0000o`000?oo
ool8o`000003ooooool0003o000000Ko000000Go001oo`000?l0003o0000o`00O`3oo`000?oo0000
:Ol00001ooooo`000ol000000ooooooo0000o`000002o`000004ooooool0003o0000ooooo`So0000
1?ooool5o`000005o`00Ool0003o0000o`000?l007l0ool0003oo`0002Wo00000Oooool000?o0000
00?oooooo`000?l000000_l000001?oooooo0000o`000?ooool8o`000003ooooool0003o000000Oo
000000Co001oo`000?l0003o001oool0003oo`0002Wo00000Oooool000?o000000?oooooo`000?l0
00000_l000001?oooooo0000o`000?ooool8o`000003ooooool0003o000000Oo000000Co001oo`00
0?l0003o001oool0003oo`0002Wo00000Oooool000;o00000_ooool4o`000004ooooool0003o0000
ooooo`So000000?oooooo`000?l000001ol000001?l007oo0000o`000?l007ooo`000?oo0000:Ol0
0001ooooo`000ol000000ooooooo0000o`000003o`0000;ooooo2Ol000000ooooooo0000o`000007
o`000004o`00Ool0003o0000o`00Oooo0000ool0000Yo`00007ooooo000Do`000003ooooool0003o
000000Oo000000Co001oo`000?l0003o001oool0003oo`0002Wo00000Oooool001Co000000?ooooo
o`000?l000001ol000001?l007oo0000o`000?l007ooo`000?oo0000:Ol00001ooooo`005?l00000
0ooooooo0000o`000007o`000004o`00Ool0003o0000o`00Oooo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000Oo000000Co001oo`000?l0003o001oool0003oo`0002Wo00000Ooo
ool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001
ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00
007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`000?l007l0ool0003oo`0002[o
00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000
:_l00001ooooo`005?l00003ooooo`Oo000000?o001oo`000?l007l0ool0003oo`0002[o00000Ooo
ool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001
ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00
007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`000?l007l0ool0003oo`0002[o
00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000
:_l00001ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o001o0?oo0000ool0
000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`000?l007l0ool0003o
o`0002[o00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`00
0?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o001o0?oo
0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`000?l007l0
ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00
O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o
001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`00
0?l007l0ool0003oo`0002[o00000Oooool001Co00000oooool7o`000003o`00Ool0003o001o0?oo
0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`000?l007l0
ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo0000o`00
O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`000003o`00Ool0003o
001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o001oo`00
0?l007l0ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l000001ol000000ol007oo
0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`000003o`00
Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo000000?o
001oo`000?l007l0ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l000001ol00000
0ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`00
0003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo
000000?o001oo`000?l007l0ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l00000
1ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`00
0007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`0000?ooooo1ol00000
0ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`000007o`00
0003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o000000Oo
000000?o001oo`000?l007l0ool0003oo`0002[o00000Oooool001Co000000?oooooo`000?l00000
1ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo0000o`00
0007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`000003ooooool0003o
000000Oo000000?o001oo`000?l007l0ool0003oo`0002[o00000Oooool001Co000000?oooooo`00
0?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l000000ooooooo
0000o`000007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo000Do`000003oooo
ool0003o000000Oo000000?o001oo`000?l007l0ool0003oo`0002[o00000Oooool001Co000000?o
ooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001ooooo`005?l00000
0ooooooo0000o`000007o`000003o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo0002o`00
00?ooooo0_l00005ooooo`So000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`00
0?oo0000:_l00001ooooo`000ol000000ooooooo0000o`000002o`000004ooooool0003o0000oooo
o`So000000?oooooo`000?l000001ol000000ol007oo0000o`00O`3oo`000?oo0000:_l00001oooo
o`000ol000000ooooooo0000o`000003o`000003ooooool0003o000000So00001?ooool6o`000003
o`00Ool0003o001o0?oo0000ool0000Zo`00007ooooo0003o`000003ooooool0003o000000Co0000
00?oooooo`000?l000001ol000000ooooooo0000o`000008o`0000;o001oool0003oo`0002[o0000
0Oooool000?o000000Goooooo`000?l0003o0000ooooo`03o`000003ooooool0003o000000Ko0000
00?oooooo`000?l000002?l00002o`00Oooo0000ool0000Zo`00007ooooo0002o`0000;ooooo0ol0
00001Ooooooo0000o`000?l0003ooooo00So000000?oooooo`000?l000002?l00002o`00Oooo0000
ool0000Zo`00007ooooo0003o`000003ooooool0003o000000;o00000oooool9o`000003ooooool0
003o000000So000000?o001oo`000?l00000ool0003oo`0002Wo00000Oooool001Co000000?ooooo
o`000?l000002?l000000ol007oo0000o`00003oo`000?oo0000:Ol00001ooooo`005?l000000ooo
oooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Yo`00007ooooo000Do`000003
ooooool0003o000000So000000?o001oo`000?l00000ool0003oo`0002Wo00000Oooool001Co0000
00?oooooo`000?l000002?l000000ol007oo0000o`00003oo`000?oo0000:Ol00001ooooo`005?l0
00000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000So000000?o001oo`000?l00000ool0003oo`0002Wo00000Oooool0
01Co000000?oooooo`000?l000002?l000000ol007oo0000o`00003oo`000?oo0000:Ol00001oooo
o`005?l000000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Yo`00007o
oooo000Do`0000?ooooo2?l000000ol007oo0000o`00003oo`000?oo0000:Ol00001ooooo`005?l0
00000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Yo`00007ooooo000D
o`000003ooooool0003o000000So000000?o001oo`000?l00000ool0003oo`0002Wo00000Oooool0
01Co000000?oooooo`000?l000002?l000000ol007oo0000o`00003oo`000?oo0000:Ol00001oooo
o`005?l000000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Yo`00007o
oooo000Do`000003ooooool0003o000000So000000?o001oo`000?l00000ool0003oo`0002Wo0000
0Oooool001Co000000?oooooo`000?l000002?l000000ol007oo0000o`00003oo`000?oo0000:Ol0
0001ooooo`005?l000000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000ool0000Y
o`00007ooooo000Do`000003ooooool0003o000000So000000?o001oo`000?l00000ool0003oo`00
02Wo00000Oooool001Co000000?oooooo`000?l000002?l000000ol007oo0000o`00003oo`000?oo
0000:Ol00001ooooo`005?l000000ooooooo0000o`000008o`000003o`00Ool0003o00000?oo0000
ool0000Yo`00007ooooo000Do`000003ooooool0003o000000So000000?o001oo`000?l00000ool0
003oo`0002Wo00000Oooool001Co000000?oooooo`000?l000002?l000000ol007oo0000o`00003o
o`000?oo0000:Ol00001ooooo`005?l00003ooooo`So000000?o001oo`000?l00000ool0003oo`00
02Wo00000Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co0000
00?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0
003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool0
01Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l0
0000ool0003oo`0003Co00000Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co0000
0Oooool001Co000000?oooooo`000?l00000ool0003oo`0003Co00000Oooool00001\
\>"],
  ImageRangeCache->{{{0, 585.188}, {361.312, 0}} -> {-3.66572, -0.623482, \
0.181697, 0.038605}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[dtd2]\)], "Input",
  CellLabel->"In[116]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["On a small-world network", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gr = 
        AddEdges[
          Cycle[200], \[IndentingNewLine]{{63, 132}, {37, 190}, {79, 
              163}, {53, 150}, {43, 
              148}}\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[gr];\)\)}], "Input",
  CellLabel->"In[117]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.68052 .91716 m
.9323 .35954 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.59916 .9436 m
.47146 .04635 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.45722 .95253 m
.5 .04545 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.31948 .91716 m
.25644 .11621 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.14084 .77859 m
.68052 .08284 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
.025 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[117]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00P_ooool3o`000005ooooool0003o0000o`000?ooool00ol000001Ooooooo0000o`000?l0
003ooooo00?o000000Goooooo`000?l0003o0000ooooo`03o`0008?ooooo001jooooo`?o000000?o
ooooo`000?l000008Ol000001?oooooo0000o`000?l0001kooooo`00MOooool3o`000003ooooool0
003o000002ko0000Moooool0077ooooo??l0001cooooo`00KOoooom4o`0006oooooo001Zoooood_o
0000Joooool006Kooooo7ol000001Ooooooo0000o`000?l0003ooooo00?o000000Goooooo`000?l0
003o0000ooooo`03o`000005ooooool0003o0000o`000?ooool07ol0001Wooooo`00H_oooolGo`00
0005ooooool0003o0000o`000?ooool00ol00006ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool3o`000003ooooool0003o000001Wo0000Hoooool0067ooooo5ol0000@oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool05?oooolHo`0006;ooooo001NoooooaKo
00005?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01Sooooo5ol0001Oooooo`00
F_oooolFo`0001Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0LoooooaOo0000
Foooool005Wooooo5?l0000Kooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool08?oo
oolDo`0005[ooooo001Foooooa?o00007oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo02Cooooo4_l0001Hooooo`00EOooool@o`0002?ooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0Xooooo`oo0000Eoooool005?ooooo3ol0000Vooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool0:oooool?o`0005Cooooo001Booooo`oo0000:?ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo02gooooo3ol0001Cooooo`00Coooool>o`0002cooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0`ooooo`oo0000D?ooool004kooooo3?l0000_
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0;oooool01?l0003ooooooooooooo
ool=o`0004oooooo001<ooooo`go0000<?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo02oooooo00?o0000ooooooooool00_ooool>o`0004gooooo001;ooooo`co0000<_ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool01_ooool<o`00
04cooooo0018ooooo`ko0000<oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo02go
oooo00?o0000ooooooooool02?ooool>o`0004Wooooo0017ooooo`co0000=_ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool03?ooool<o`0004Sooooo
0015ooooo`go0000>?ooool01Ol0003ooooooooooooooooo000002gooooo00?o0000ooooooooool0
3_ooool=o`0004Kooooo0014ooooo`co0000>_ooool01Ol0003ooooooooooooooooo000002gooooo
00?o0000ooooooooool04?ooool<o`0004Gooooo0013ooooo`co0000>oooool01Ol0003ooooooooo
oooooooo000002cooooo00?o0000ooooooooool04_ooool<o`0004Cooooo0011ooooo`_o0000?_oo
ool01Ol0003ooooooooooooooooo000002_ooooo00?o0000ooooooooool05_ooool;o`0004;ooooo
0010ooooo`_o0000?oooool01Ol0003ooooooooooooooooo000002[ooooo00?o0000ooooooooool0
6?ooool;o`00047ooooo000nooooo`co0000@?ooool01Ol0003ooooooooooooooooo000002[ooooo
00?o0000ooooooooool06_ooool;o`0003oooooo000mooooo`_o000000?oooooo`000?ooool0?ooo
ool01Ol0003ooooooooooooooooo000002Wooooo00?o0000ooooooooool07?ooool;o`0003kooooo
000looooo`[o00000oooool00ol0003oooooooooo`0oooooo`04o`000?ooooooooooo`0002Sooooo
00?o0000ooooooooool07oooool:o`0003gooooo000kooooo`[o00001Oooool00ol0003ooooooooo
o`0nooooo`04o`000?ooooooooooo`0002Oooooo00?o0000ooooooooool08Oooool:o`0003cooooo
000jooooo`[o00001_ooool00ol0003oooooooooo`0nooooo`04o`000?ooooooooooo`0002Kooooo
00?o0000ooooooooool08oooool:o`0003_ooooo000hooooo`[o00002?ooool00ol0003ooooooooo
o`0nooooo`04o`000?ooooooooooo`0002Kooooo00?o0000ooooooooool09Oooool:o`0003Wooooo
000gooooo`[o00002Oooool00ol0003oooooooooo`0nooooo`04o`000?ooooooooooo`0002Gooooo
00?o0000ooooooooool09oooool:o`0003Sooooo000fooooo`[o00002_ooool00ol0003ooooooooo
o`0nooooo`04o`000?ooooooooooo`0002Cooooo00?o0000ooooooooool0:Oooool:o`0003Oooooo
000eooooo`[o00002oooool00ol0003oooooooooo`0nooooo`04o`000?ooooooooooo`0002?ooooo
00?o0000ooooooooool0:oooool:o`0003Kooooo000dooooo`[o00003?ooool00ol0003ooooooooo
o`0oooooo`;o00009?ooool00ol0003oooooooooo`0/ooooo`[o0000=Oooool003?ooooo2Ol0000>
ooooo`03o`000?oooooooooo03oooooo0_l0000Sooooo`03o`000?oooooooooo02oooooo2Ol0000d
ooooo`00<_ooool9o`0000oooooo00?o0000ooooooooool0?oooool2o`0002;ooooo00?o0000oooo
ooooool0<Oooool9o`0003?ooooo000aooooo`Wo00004?ooool00ol0003oooooooooo`0oooooo`;o
00008Oooool00ol0003oooooooooo`0cooooo`Wo0000<_ooool0033ooooo2Ol0000Aooooo`03o`00
0?oooooooooo03oooooo0_l0000Pooooo`03o`000?oooooooooo03Gooooo2Ol0000aooooo`00;ooo
ool9o`0001;ooooo00?o0000ooooooooool0?oooool2o`00023ooooo00?o0000ooooooooool0=_oo
ool9o`00033ooooo000^ooooo`Wo00004oooool00ol0003oooooooooo`0oooooo`;o00007oooool0
0ol0003oooooooooo`0hooooo`Wo0000;oooool002gooooo2Ol0000Eooooo`03o`000?oooooooooo
03oooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0jooooo`Wo0000;_ooool002co
oooo2Ol0000Fooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`0looooo`Wo0000;Oooool002_ooooo2Ol0000Gooooo`03o`000?oooooooooo03oooooo
00?o0000ooooooooool06oooool00ol0003oooooooooo`0mooooo`Wo0000;?ooool002[ooooo2Ol0
000Hooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0oooooo`Wo0000:oooool002Wooooo2Ol0000Iooooo`03o`000?oooooooooo03oooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`11ooooo`So0000:oooool002Sooooo2Ol0000Joooo
o`03o`000?oooooooooo03oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`13oooo
o`So0000:_ooool002Oooooo2Ol0000Kooooo`03o`000?oooooooooo03oooooo00?o0000oooooooo
ool05oooool00ol0003oooooooooo`15ooooo`So0000:Oooool002Oooooo2?l0000Looooo`03o`00
0?oooooooooo03oooooo0_l0000Hooooo`03o`000?oooooooooo04Kooooo2?l0000Xooooo`009_oo
ool8o`0001gooooo00?o0000ooooooooool0?oooool2o`0001Oooooo00?o0000ooooooooool0Aooo
ool9o`0002Oooooo000Uooooo`So00007_ooool00ol0003oooooooooo`0oooooo`;o00005_ooool0
0ol0003oooooooooo`19ooooo`Wo00009_ooool002Cooooo2?l0000Oooooo`03o`000?oooooooooo
03oooooo0_l0000Eooooo`03o`000?oooooooooo04_ooooo2?l0000Vooooo`008oooool9o`0001oo
oooo00?o0000ooooooooool0?oooool2o`0001Gooooo00?o0000ooooooooool0C?ooool8o`0002Go
oooo000Rooooo`Wo00008?ooool00ol0003oooooooooo`0oooooo`;o00005?ooool00ol0003ooooo
ooooo`1>ooooo`So00009?ooool002;ooooo2?l0000Rooooo`03o`000?oooooooooo03kooooo0_l0
000Cooooo`03o`000?oooooooooo053ooooo1ol0000Tooooo`008_ooool7o`0002?ooooo00?o0000
ooooooooool0?_ooool2o`0001;ooooo00?o0000ooooooooool0DOooool8o`0002?ooooo000Qoooo
o`Oo00009?ooool00ol0003oooooooooo`0mooooo`04o`000?ooooooooooo`00017ooooo00?o0000
ooooooooool0D_ooool8o`0002;ooooo000Pooooo`Oo00009Oooool00ol0003oooooooooo`0moooo
o`04o`000?ooooooooooo`00013ooooo00?o0000ooooooooool0E?ooool8o`00027ooooo000Poooo
o`Oo00009Oooool00ol0003oooooooooo`0mooooo`04o`000?ooooooooooo`0000oooooo00?o0000
ooooooooool0E_ooool7o`00027ooooo000Oooooo`So00009Oooool00ol0003oooooooooo`0moooo
o`04o`000?ooooooooooo`0000kooooo00?o0000ooooooooool0Eoooool8o`00023ooooo000Noooo
o`So00009_ooool00ol0003oooooooooo`0mooooo`04o`000?ooooooooooo`0000gooooo00?o0000
ooooooooool0FOooool8o`0001oooooo000Nooooo`Oo00009oooool00ol0003oooooooooo`0moooo
o`04o`000?ooooooooooo`0000gooooo00?o0000ooooooooool0F_ooool7o`0001oooooo000Noooo
o`Oo00009oooool00ol0003oooooooooo`0mooooo`04o`000?ooooooooooo`0000cooooo00?o0000
ooooooooool0Foooool7o`0001oooooo000Mooooo`Oo0000:?ooool00ol0003oooooooooo`0moooo
o`05o`000?ooooooooooooooool000002_ooool00ol0003oooooooooo`1Mooooo`Oo00007_ooool0
01cooooo1ol0000Yooooo`03o`000?oooooooooo03gooooo00Go0000ooooooooooooooooo`000009
ooooo`03o`000?oooooooooo05oooooo1ol0000Mooooo`007?ooool7o`0002Wooooo00?o0000oooo
ooooool0?Oooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooooooool0Goooool7
o`0001gooooo000Kooooo`So0000:Oooool00ol0003oooooooooo`0mooooo`05o`000?oooooooooo
ooooool000002?ooool00ol0003oooooooooo`1Pooooo`So00007?ooool001[ooooo2?l0000[oooo
o`03o`000?oooooooooo03cooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooo
oooo06;ooooo2?l0000Kooooo`006_ooool7o`0002cooooo00?o0000ooooooooool0??ooool01Ol0
003ooooooooooooooooo000000Kooooo00?o0000ooooooooool0I?ooool7o`0001_ooooo000Joooo
o`Oo0000;?ooool00ol0003oooooooooo`0looooo`05o`000?ooooooooooooooool000001Oooool0
0ol0003oooooooooo`1Uooooo`Oo00006oooool001Wooooo1ol0000]ooooo`03o`000?oooooooooo
03cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
06Kooooo1ol0000Jooooo`006?ooool7o`0002kooooo00?o0000ooooooooool0??ooool00ol0003o
ooooooooo`02ooooo`05o`000?ooooooooooooooool00000J_ooool7o`0001Wooooo000Hooooo`Oo
0000;_ooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo0000Joooool7o`0001Wooooo000Hooooo`Oo0000;_ooool00ol0003oooooooooo`0looooo`03
o`000?oooooooooo00;ooooo00?o0000ooooool00000K?ooool7o`0001Wooooo000Gooooo`Oo0000
;oooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
KOooool7o`0001Sooooo000Fooooo`Oo0000<?ooool00ol0003oooooooooo`0looooo`03o`000?oo
oooooooo00;ooooo0_l0001_ooooo`Oo00005oooool001Kooooo1ol0000`ooooo`03o`000?oooooo
oooo03cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`1^ooooo`Oo00005oooool0
01Gooooo2?l0000`ooooo`03o`000?oooooooooo03cooooo00Oo0000ooooooooooooooooo`000?oo
oooo000006oooooo2?l0000Fooooo`005?ooool8o`00037ooooo00?o0000ooooooooool0>oooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool0K_ooool8o`0001Gooooo000D
ooooo`Oo0000<oooool00ol0003oooooooooo`0jooooo`05o`000?ooooooooooooooool000000_oo
ool00ol0003oooooooooo`1_ooooo`Oo00005Oooool001Cooooo1ol0000cooooo`03o`000?oooooo
oooo03[ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`1_ooooo`Oo00005Ooo
ool001Cooooo1_l0000dooooo`03o`000?oooooooooo03[ooooo00?o0000ooooool000001?ooool0
0ol0003oooooooooo`1`ooooo`Ko00005Oooool001?ooooo1ol0000dooooo`03o`000?oooooooooo
03[ooooo0_l00005ooooo`03o`000?oooooooooo073ooooo1ol0000Dooooo`004oooool7o`0003Co
oooo00?o0000ooooooooool0>_ooool2o`0000Gooooo00?o0000ooooooooool0L?ooool7o`0001Co
oooo000Cooooo`Oo0000=?ooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0Koooool7o`0001Cooooo000Booooo`Oo0000=Oooool00ol0003ooooooooo
o`0iooooo`;o00001oooool00ol0003oooooooooo`1`ooooo`Oo00004oooool0017ooooo1ol0000f
ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooool000001oooool00ol0003oooooooooo`1a
ooooo`Oo00004_ooool0017ooooo1ol0000fooooo`03o`000?oooooooooo03Oooooo00Co0000oooo
oooooooo00001oooool00ol0003oooooooooo`1aooooo`Oo00004_ooool0017ooooo1ol0000foooo
o`03o`000?oooooooooo03Oooooo00Co0000oooooooooooo00001oooool00ol0003oooooooooo`1a
ooooo`Oo00004_ooool0017ooooo1_l0000gooooo`03o`000?oooooooooo03Kooooo00Go0000oooo
ooooooooooooo`000007ooooo`03o`000?oooooooooo07;ooooo1_l0000Booooo`004?ooool7o`00
03Oooooo00?o0000ooooooooool0=Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool0L_ooool7o`00017ooooo000@ooooo`Oo0000=oooool00ol0003o
ooooooooo`0dooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`1aooooo`Oo00004Oooool0013ooooo1ol0000hooooo`03o`000?oooooooooo03?ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo077ooooo
1ol0000Aooooo`004?ooool6o`0003Wooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0L_ooool6o`00017ooooo000?
ooooo`Oo0000>Oooool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`1booooo`Oo00004?ooool000oooooo1ol0000ioooo
o`03o`000?oooooooooo033ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo07;ooooo1ol0000@ooooo`003oooool7o`0003Wooooo00?o0000oooooooo
ool0<?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0L_ooool7o`00013ooooo000?ooooo`Ko0000>_ooool00ol0003oooooooooo`0_ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`1booooo`Oo0000
4?ooool000kooooo1ol0000jooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo073ooooo00?o0000ooooool000001_l0
000?ooooo`003_ooool7o`0003[ooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0L?ooool00ol0003oooooo`000006
o`0000oooooo000>ooooo`Oo0000>_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`1_ooooo`03o`000?oooooooooo
00Oo00003oooool000kooooo1_l0000kooooo`03o`000?oooooooooo02cooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo06oooooo00Co0000oooooooo
oooooooo1_l0000?ooooo`003Oooool7o`0003_ooooo00?o0000ooooooooool0:oooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0K_ooool00ol0003o
ooooooooo`02ooooo`Oo00003_ooool000gooooo1ol0000kooooo`03o`000?oooooooooo02[ooooo
00?o0000ooooooooool02oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo06kooooo
00?o0000ooooooooool00_ooool7o`0000kooooo000=ooooo`Oo0000>oooool00ol0003ooooooooo
o`0Yooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo00?ooooo1ol0000>ooooo`003Oooool6o`0003gooooo00?o0000
ooooooooool0:?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`04ooooo`Ko00003_ooool000cooooo1ol0000moooo
o`03o`000?oooooooooo02Oooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo06_ooooo00?o0000ooooooooool01Oooool7o`0000gooooo000<ooooo`Oo
0000?Oooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00kooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`1[ooooo`03o`000?oooooooooo00Gooooo1ol0000=ooooo`00
3?ooool7o`0003gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool0Joooool00ol0003oooooooooo`05ooooo`Oo0000
3Oooool000cooooo1_l0000nooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo06[ooooo00?o0000ooooooooool01ooo
ool6o`0000gooooo000;ooooo`Oo0000?_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1Zooooo`03o`000?oooooo
oooo00Oooooo1ol0000<ooooo`002oooool7o`0003kooooo00?o0000ooooooooool08oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0JOooool00ol0
003oooooooooo`08ooooo`Oo00003?ooool000_ooooo1ol0000nooooo`03o`000?oooooooooo02;o
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo06Wo
oooo00?o0000ooooooooool02?ooool7o`0000cooooo000<ooooo`Go0000?oooool00ol0003ooooo
ooooo`0Qooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02_ooool00ol0003ooooo
ooooo`1Wooooo`03o`000?oooooooooo00[ooooo1Ol0000=ooooo`002oooool7o`0003kooooo00?o
0000ooooooooool08Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool0Ioooool00ol0003oooooooooo`09ooooo`Oo00003?ooool000_ooooo1ol0000n
ooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo06Kooooo00?o0000ooooooooool02_ooool7o`0000cooooo000;oooo
o`Oo0000?_ooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool02_ooool00ol0003oooooooooo`1Vooooo`03o`000?oooooooooo00[ooooo1ol0000<oooo
o`002oooool6o`0003oooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Fooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool0I_ooool00ol0003oooooooooo`0;ooooo`Ko
00003?ooool000[ooooo1ol00010ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0
5oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Gooooo00?o0000ooooooooool0
3?ooool7o`0000_ooooo000:ooooo`Oo0000@?ooool00ol0003oooooooooo`0Looooo`03o`000?oo
oooooooo01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1Uooooo`03o`000?oo
oooooooo00cooooo1ol0000;ooooo`002_ooool7o`00043ooooo00?o0000ooooooooool06oooool0
0ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0Hoooool0
0ol0003oooooooooo`0=ooooo`Oo00002oooool000_ooooo1Ol00011ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
06?ooooo00?o0000ooooooooool03_ooool5o`0000cooooo000:ooooo`Oo0000@?ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`1Rooooo`03o`000?oooooooooo00kooooo1ol0000;ooooo`002_ooool7o`00043ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool0H_ooool00ol0003oooooooooo`0>ooooo`Oo00002oooool000[ooooo1ol0
0010ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo067ooooo00?o0000ooooooooool03oooool7o`0000_ooooo000;
ooooo`Go0000@Oooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo013ooooo1Ol0000<
ooooo`002_ooool7o`00043ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Loooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool0H?ooool00ol0003oooooooooo`0@oooo
o`Oo00002oooool000[ooooo1ol00010ooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool07Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05oooooo00?o0000oooooooo
ool04?ooool7o`0000_ooooo000:ooooo`Oo0000@?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`1Oooooo`03o`00
0?oooooooooo013ooooo1ol0000;ooooo`002_ooool6o`00047ooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0G_oo
ool00ol0003oooooooooo`0Booooo`Go00003?ooool000Wooooo1ol00011ooooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo05kooooo00?o0000ooooooooool04Oooool7o`0000_ooooo0009ooooo`Oo0000@_ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool03Oooool00ol0
003oooooooooo`1Mooooo`03o`000?oooooooooo01;ooooo1ol0000;ooooo`002Oooool7o`0004;o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool0GOooool00ol0003oooooooooo`0Booooo`Oo00002oooool000[ooooo
1_l00012ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool08Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo05cooooo00?o0000ooooooooool05?ooool5o`0000cooooo
000:ooooo`Oo0000@Oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo02;ooooo00?o
0000ooooooooool03_ooool00ol0003oooooooooo`1Kooooo`03o`000?oooooooooo01?ooooo1ol0
000;ooooo`002_ooool7o`00047ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0S
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0F_ooool00ol0003oooooooooo`0D
ooooo`Oo00002oooool000[ooooo1ol00011ooooo`03o`000?oooooooooo00kooooo00?o0000oooo
ooooool08oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05[ooooo00?o0000oooo
ooooool05?ooool7o`0000_ooooo000;ooooo`Go0000@_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo02Cooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Jooooo`03
o`000?oooooooooo01Gooooo1Ol0000<ooooo`002_ooool7o`00047ooooo00?o0000ooooooooool0
3?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
FOooool00ol0003oooooooooo`0Eooooo`Oo00002oooool000[ooooo1ol00011ooooo`03o`000?oo
oooooooo00_ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0>ooooo`03o`000?oo
oooooooo05Wooooo00?o0000ooooooooool05Oooool7o`0000_ooooo000:ooooo`Oo0000@Oooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool03_ooool0
0ol0003oooooooooo`1Hooooo`03o`000?oooooooooo01Kooooo1ol0000;ooooo`002oooool5o`00
04;ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo
00oooooo00?o0000ooooooooool0Eoooool00ol0003oooooooooo`0Gooooo`Go00003?ooool000[o
oooo1ol00011ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0:?ooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool05oooool7o`0000_o
oooo000:ooooo`Oo0000@_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02Sooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Fooooo`03o`000?oooooooooo01Oooooo
1ol0000;ooooo`002_ooool7o`0004;ooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0Yooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0EOooool00ol0003ooooooooo
o`0Hooooo`Oo00002oooool000_ooooo1_l00012ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool0:Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo05Gooooo00?o0000
ooooooooool06?ooool6o`0000cooooo000;ooooo`Oo0000@Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo02[ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`1Doooo
o`03o`000?oooooooooo01Sooooo1ol0000<ooooo`002oooool7o`00047ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool0E?ooool00ol0003oooooooooo`0Hooooo`Oo00003?ooool000_ooooo1ol00011ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo05?ooooo00?o0000ooooooooool06?ooool7o`0000cooooo000<ooooo`Go0000@_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo01[ooooo1Ol0000=ooooo`002oooool7
o`00047ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0Iooooo`Oo00003?ooool0
00_ooooo1ol00011ooooo`05o`000?ooooooooooooooool00000<?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo057ooooo00?o0000ooooooooool06_ooool7o`0000cooooo000;oooo
o`Oo0000@Oooool01?l0003oooooooooool0000aooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool0DOooool00ol0003oooooooooo`0Jooooo`Oo00003?ooool000cooooo1Ol00012oooo
o`03o`000?oooooo000003;ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1@oooo
o`03o`000?oooooooooo01cooooo1Ol0000=ooooo`003Oooool5o`00047ooooo00?o0000ooooool0
0000<_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo053ooooo00?o0000oooooooo
ool06oooool5o`0000kooooo000<ooooo`Oo0000@?ooool2o`0003?ooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo01_ooooo1ol0000=ooooo`003?oo
ool7o`00043ooooo0_l0000cooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0C_oo
ool00ol0003oooooooooo`0Kooooo`Oo00003Oooool000cooooo1ol0000oooooo`03o`000?oooooo
000003?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`1>ooooo`03o`000?oooooo
oooo01_ooooo1ol0000=ooooo`003Oooool6o`0003kooooo00Co0000oooooooooooo0000<oooool0
0ol0003oooooooooo`0Booooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool07?ooool6
o`0000kooooo000=ooooo`Oo0000?Oooool01?l0003oooooooooool0000cooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`0Kooooo`Oo00003_ooool0
00gooooo1ol0000looooo`05o`000?ooooooooooooooool00000<oooool00ol0003oooooooooo`0B
ooooo`03o`000?oooooooooo04cooooo00?o0000ooooooooool07?ooool7o`0000kooooo000=oooo
o`Oo0000>oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo037ooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`1<ooooo`03o`000?oooooooooo01cooooo1ol0000>oooo
o`003_ooool6o`0003[ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0aooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool0Boooool00ol0003oooooooooo`0Mooooo`Ko
00003oooool000kooooo1ol0000iooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
<Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo04[ooooo00?o0000ooooooooool0
7?ooool7o`0000oooooo000>ooooo`Oo0000>?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo033ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`19ooooo`03o`000?oo
oooooooo01gooooo1ol0000?ooooo`003_ooool7o`0003Oooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0`ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0BOooool0
0ol0003oooooooooo`0Mooooo`Oo00003oooool000oooooo1ol0000eooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
04Sooooo00?o0000ooooooooool07Oooool7o`00013ooooo000?ooooo`Oo0000=?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`18ooooo`03o`000?oooooooooo01gooooo1ol0000@ooooo`003oooool7o`0003Cooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool0B?ooool00ol0003oooooooooo`0Mooooo`Oo00004?ooool0013ooooo1_l0
000cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0;oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo04Oooooo00?o0000ooooooooool07_ooool6o`00017ooooo000@
ooooo`Oo0000<Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo02oooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo01gooooo1ol0000A
ooooo`004?ooool7o`00033ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0_oooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0AOooool00ol0003oooooooooo`0Noooo
o`Oo00004Oooool0013ooooo1ol0000`ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool0;oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04Gooooo00?o0000oooooooo
ool07_ooool7o`00017ooooo000Aooooo`Ko0000;oooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo02oooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`14ooooo`03o`00
0?oooooooooo01oooooo1_l0000Booooo`004Oooool7o`0002gooooo00?o0000ooooooooool03Ooo
ool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0A?oo
ool00ol0003oooooooooo`0Nooooo`Oo00004_ooool0017ooooo1ol0000/ooooo`03o`000?oooooo
oooo00kooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo04?ooooo00?o0000ooooooooool07oooool7o`0001;ooooo000Aooooo`Oo0000;?ooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`13ooooo`03o`000?oooooooooo01oooooo1ol0000Booooo`004_ooool7o`0002[o
oooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01Ko
oooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`0Nooooo`Oo00004oooool001?ooooo
1ol0000Xooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0;oooool00ol0003ooooo
ooooo`0Fooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool07_ooool7o`0001Cooooo
000Cooooo`Oo00009oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo02oooooo00?o
0000ooooooooool05_ooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01kooooo1ol0
000Dooooo`004oooool7o`0002Kooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0_
ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`0O
ooooo`Oo00005?ooool001Cooooo1_l0000Vooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool0;oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo043ooooo00?o0000oooo
ooooool07oooool6o`0001Gooooo000Dooooo`Oo00009?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo02kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0oooooo`03
o`000?oooooooooo01oooooo1ol0000Eooooo`005?ooool7o`0002?ooooo00?o0000ooooooooool0
5Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0
?oooool00ol0003oooooooooo`0Oooooo`Oo00005Oooool001Cooooo1ol0000Rooooo`03o`000?oo
oooooooo01Kooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0Gooooo`03o`000?oo
oooooooo03gooooo00?o0000ooooooooool08?ooool7o`0001Gooooo000Eooooo`Oo00008Oooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool05oooool0
0ol0003oooooooooo`0mooooo`03o`000?oooooooooo01oooooo1ol0000Fooooo`005_ooool7o`00
01oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool0??ooool00ol0003oooooooooo`0Oooooo`Oo00005oooool001Ko
oooo1ol0000Nooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0;Oooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo03cooooo00?o0000ooooooooool07oooool7o`0001Oo
oooo000Fooooo`So00007?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo02gooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo01kooooo
2?l0000Gooooo`005oooool8o`0001[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0]ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0>oooool00ol0003ooooooooo
o`0Nooooo`So00006?ooool001Sooooo1ol0000Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool0;Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo03_ooooo00?o0000
ooooooooool07_ooool7o`0001Wooooo000Hooooo`Oo00006Oooool00ol0003oooooooooo`0Koooo
o`03o`000?oooooooooo02gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0ioooo
o`03o`000?oooooooooo01oooooo1ol0000Iooooo`006Oooool7o`0001Oooooo00?o0000oooooooo
ool07?ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool0>Oooool00ol0003oooooooooo`0Nooooo`Oo00006_ooool001[ooooo1ol0000Eooooo`03o`00
0?oooooooooo01gooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo03Sooooo00?o0000ooooooooool07_ooool7o`0001_ooooo000Jooooo`Oo00005Ooo
ool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0hooooo`03o`000?oooooooooo01kooooo1ol0000Kooooo`006_ooool8
o`0001?ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool0=oooool00ol0003oooooooooo`0Nooooo`So00006oooool0
01_ooooo2?l0000Aooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0;?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo03Oooooo00?o0000ooooooooool07Oooool8o`00
01cooooo000Looooo`Oo00004?ooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo02co
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo01go
oooo1ol0000Mooooo`007?ooool7o`0000oooooo00?o0000ooooooooool08_ooool00ol0003ooooo
ooooo`0/ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0=Oooool00ol0003ooooo
ooooo`0Nooooo`Oo00007Oooool001gooooo1ol0000>ooooo`03o`000?oooooooooo02;ooooo00?o
0000ooooooooool0;?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo03Gooooo00?o
0000ooooooooool07Oooool7o`0001kooooo000Nooooo`Oo00003?ooool00ol0003oooooooooo`0S
ooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0d
ooooo`03o`000?oooooooooo01gooooo1ol0000Oooooo`007_ooool7o`0000_ooooo00?o0000oooo
ooooool09?ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool0=?ooool00ol0003oooooooooo`0Mooooo`Oo00007oooool001kooooo2?l00009ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo03?ooooo00?o0000ooooooooool07Oooool8o`0001oooooo000Oooooo`So0000
2?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo01cooooo2?l0000Pooooo`008?oo
ool7o`0000Oooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0/ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`0Mooooo`Oo00008Ooo
ool0023ooooo1ol00006ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool0;?ooool0
0ol0003oooooooooo`0Kooooo`03o`000?oooooooooo037ooooo00?o0000ooooooooool07?ooool8
o`00027ooooo000Qooooo`Oo00001?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
02cooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo
01cooooo2?l0000Rooooo`008_ooool7o`0000;ooooo00?o0000ooooooooool0:_ooool00ol0003o
ooooooooo`0[ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`0Kooooo`So00008oooool002;ooooo2?l000000ooooooo0000ooooo`0[ooooo`03o`00
0?oooooooooo02[ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo01_ooooo1ol0000Tooooo`008_ooool9o`0002gooooo00?o0000ooooooooool0:_oo
ool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool06ooo
ool8o`0002Cooooo000Sooooo`Wo0000;?ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooo
oooo01cooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Jooooo`So00009Oooool0
02Cooooo2?l0000/ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool07?ooool00ol0
003oooooooooo`0^ooooo`03o`000?oooooooooo01[ooooo2?l0000Vooooo`009Oooool8o`0002_o
oooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo02ko
oooo00?o0000ooooooooool06Oooool9o`0002Kooooo000Vooooo`So0000:_ooool00ol0003ooooo
ooooo`0Zooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`0Iooooo`Wo00009oooool002Oooooo2?l0000Yooooo`03o`000?oooooooooo02[ooooo00?o
0000ooooooooool07Oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01Wooooo2?l0
000Xooooo`009oooool9o`0002Sooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0M
ooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool06Oooool8o`0002Wooooo000Xoooo
o`Wo00009oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo01gooooo00?o0000oooo
ooooool0:oooool00ol0003oooooooooo`0Hooooo`So0000:_ooool002Wooooo2Ol0000Vooooo`03
o`000?oooooooooo02[ooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0[ooooo`03
o`000?oooooooooo01Oooooo2?l0000[ooooo`00:_ooool9o`0002Gooooo00?o0000ooooooooool0
:_ooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0
5oooool9o`0002_ooooo000[ooooo`Wo00009?ooool00ol0003oooooooooo`0Zooooo`03o`000?oo
oooooooo01gooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Fooooo`Wo0000;?oo
ool002cooooo2Ol0000Tooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0Xooooo`03o`000?oooooooooo01Kooooo2Ol0000]ooooo`00;Oooool9o`00
02?ooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo
02Sooooo00?o0000ooooooooool05Oooool9o`0002kooooo000^ooooo`Wo00008_ooool00ol0003o
ooooooooo`0Yooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool09oooool00ol0003o
ooooooooo`0Eooooo`Wo0000;oooool002oooooo2Ol0000Qooooo`03o`000?oooooooooo02Wooooo
00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo01Cooooo
2Ol0000`ooooo`00<?ooool9o`00023ooooo00?o0000ooooooooool0:Oooool00ol0003ooooooooo
o`0Nooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool05?ooool9o`00037ooooo000a
ooooo`Wo00007oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo01kooooo00?o0000
ooooooooool09_ooool00ol0003oooooooooo`0Cooooo`Wo0000<_ooool003;ooooo2Ol0000Noooo
o`03o`000?oooooooooo02Wooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01?ooooo2Ol0000cooooo`00<oooool9o`0001gooooo00?o0000oooooooo
ool0:Oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo02Cooooo00?o0000oooooooo
ool04_ooool9o`0003Cooooo000dooooo`Wo00007?ooool00ol0003oooooooooo`0Yooooo`03o`00
0?oooooooooo01oooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0Aooooo`Wo0000
=Oooool003Gooooo2Ol0000Kooooo`03o`000?oooooooooo02Sooooo00?o0000ooooooooool08?oo
ool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo017ooooo2Ol0000fooooo`00=_ooool9
o`0001[ooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooo
oooo02?ooooo00?o0000ooooooooool04?ooool9o`0003Oooooo000gooooo`[o00006?ooool00ol0
003oooooooooo`0Xooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool08_ooool00ol0
003oooooooooo`0?ooooo`[o0000>?ooool003Sooooo2_l0000Gooooo`03o`000?oooooooooo02So
oooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00ko
oooo2_l0000iooooo`00>Oooool:o`0001Oooooo00?o0000ooooooooool09oooool00ol0003ooooo
ooooo`0Pooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool03_ooool:o`0003[ooooo
000jooooo`[o00005_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo027ooooo00?o
0000ooooooooool08?ooool00ol0003oooooooooo`0=ooooo`[o0000>oooool003_ooooo2_l0000E
ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`0O
ooooo`03o`000?oooooooooo00gooooo2_l0000looooo`00?Oooool;o`0001;ooooo00?o0000oooo
ooooool09oooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo01oooooo00?o0000oooo
ooooool02_ooool;o`0003kooooo000nooooo`_o00004Oooool00ol0003oooooooooo`0Wooooo`03
o`000?oooooooooo027ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`09ooooo`_o
0000?oooool003oooooo3?l0000?ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool0
8Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00Sooooo3?l00010ooooo`00@?oo
ool<o`0000kooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0Qooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool01oooool<o`00047ooooo0011ooooo`ko00002oooool0
0ol0003oooooooooo`0Wooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool07Oooool0
0ol0003oooooooooo`05ooooo`ko0000@_ooool004Cooooo3?l0000:ooooo`03o`000?oooooooooo
02Oooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
00Cooooo3?l00015ooooo`00AOooool=o`0000Sooooo00?o0000ooooooooool09oooool00ol0003o
ooooooooo`0Rooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool00oooool=o`0004Ko
oooo0017ooooo`co00001oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo02;ooooo
00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`co0000B?ooool004Sooooo3_l0
0004ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool08_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00ko0000BOooool004_ooooo3?l00003ooooo`03o`000?oooooo
oooo02Oooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
000000_o0000C?ooool004cooooo3Ol00002ooooo`03o`000?oooooooooo02Kooooo00?o0000oooo
ooooool08_ooool00ol0003oooooooooo`0Iooooo`ko0000COooool004kooooo3?l000000ooooooo
0000ooooo`0Wooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool06?ooool=o`0004oo
oooo001?ooooo`ko00009oooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo01Gooooo
3_l0001@ooooo`00D_ooool?o`0002?ooooo00?o0000ooooooooool08oooool00ol0003ooooooooo
o`0Aooooo`oo0000Doooool005?ooooo3ol0000Rooooo`03o`000?oooooooooo02?ooooo00?o0000
ooooooooool04?ooool?o`0005Cooooo001Eoooooa3o00007_ooool00ol0003oooooooooo`0Toooo
o`03o`000?oooooooooo00gooooo3ol0001Gooooo`00E_oooolCo`0001[ooooo00?o0000oooooooo
ool09?ooool00ol0003oooooooooo`09oooooa;o0000F?ooool005Wooooo5?l0000Fooooo`03o`00
0?oooooooooo02Cooooo00?o0000ooooooooool01OoooolDo`0005[ooooo001JoooooaKo00004ooo
ool00ol0003oooooooooo`0Tooooo`04o`000?oooooooooooooooaOo0000Foooool005kooooo5_l0
000?ooooo`03o`000?oooooooooo02Cooooo5ol0001Oooooo`00HOoooolGo`0000_ooooo00?o0000
ooooooooool08?oooolHo`0006;ooooo001RoooooaOo000000Goooooo`000?l0003o0000ooooo`03
o`0000;ooooo00?o0000ooooooooool06?ooool3o`000003ooooool0003o000001Wo0000Hoooool0
06Kooooo7ol000001Ooooooo0000o`000?l0003ooooo00?o00001Oooool3o`000005ooooool0003o
0000o`000?ooool07ol0001Wooooo`00J_oooom;o`0006_ooooo001]ooooodCo0000Koooool0077o
oooo??l0001cooooo`00MOooool3o`000003ooooool0003o000002ko0000Moooool007[ooooo0ol0
00000ooooooo0000o`00000Qo`000004ooooool0003o0000o`0007_ooooo0022ooooo`?o000000Go
ooooo`000?l0003o0000ooooo`0;o`000005ooooool0003o0000o`000?ooool00ol00023ooooo`00
S_ooool3o`0008oooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(dtd3 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] gr, \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[119]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[
        ExtractGraph[dtd3], \[IndentingNewLine]VertexStyle \[Rule] 
          Disk[Small]];\)\)], "Input",
  CellLabel->"In[120]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.68052 .91716 m
.9323 .35954 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.59916 .9436 m
.47146 .04635 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.45722 .95253 m
.5 .04545 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.31948 .91716 m
.25644 .11621 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.14084 .77859 m
.68052 .08284 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
1 1 0 r
.012 w
.95432 .51428 Mdot
1 0 1 r
.95365 .52854 Mdot
.498 .498 .498 r
.95253 .54278 Mdot
1 1 0 r
.95096 .55697 Mdot
1 0 1 r
.94895 .57111 Mdot
0 1 1 r
.94649 .58517 Mdot
1 0 0 r
.9436 .59916 Mdot
.498 0 0 r
.94027 .61304 Mdot
1 0 1 r
.9365 .62681 Mdot
.498 0 0 r
.9323 .64046 Mdot
.92767 .65397 Mdot
0 1 1 r
.92263 .66733 Mdot
1 1 0 r
.91716 .68052 Mdot
1 0 0 r
.91129 .69354 Mdot
.498 0 0 r
.905 .70636 Mdot
1 1 0 r
.89832 .71898 Mdot
1 0 0 r
.89125 .73138 Mdot
.498 0 0 r
.88379 .74356 Mdot
1 1 0 r
.87595 .75549 Mdot
0 .498 0 r
.86773 .76718 Mdot
0 0 .498 r
.85916 .77859 Mdot
1 0 1 r
.85023 .78974 Mdot
.498 0 0 r
.84096 .8006 Mdot
1 0 1 r
.83135 .81116 Mdot
.498 .498 .498 r
.82141 .82141 Mdot
1 0 1 r
.81116 .83135 Mdot
.8006 .84096 Mdot
0 1 0 r
.78974 .85023 Mdot
0 1 1 r
.77859 .85916 Mdot
0 .498 0 r
.76718 .86773 Mdot
.498 0 0 r
.75549 .87595 Mdot
.498 .498 .498 r
.74356 .88379 Mdot
.898 .898 .898 r
.73138 .89125 Mdot
1 1 0 r
.71898 .89832 Mdot
.498 .498 .498 r
.70636 .905 Mdot
0 1 1 r
.69354 .91129 Mdot
.498 .498 .498 r
.68052 .91716 Mdot
0 1 1 r
.66733 .92263 Mdot
0 .498 0 r
.65397 .92767 Mdot
.898 .898 .898 r
.64046 .9323 Mdot
0 1 1 r
.62681 .9365 Mdot
0 .498 0 r
.61304 .94027 Mdot
0 1 1 r
.59916 .9436 Mdot
1 0 0 r
.58517 .94649 Mdot
.498 .498 .498 r
.57111 .94895 Mdot
0 1 1 r
.55697 .95096 Mdot
1 0 1 r
.54278 .95253 Mdot
0 .498 0 r
.52854 .95365 Mdot
1 1 0 r
.51428 .95432 Mdot
.898 .898 .898 r
.5 .95455 Mdot
0 0 .498 r
.48572 .95432 Mdot
.498 .498 .498 r
.47146 .95365 Mdot
.898 .898 .898 r
.45722 .95253 Mdot
0 0 .498 r
.44303 .95096 Mdot
.498 .498 .498 r
.42889 .94895 Mdot
0 0 .498 r
.41483 .94649 Mdot
.898 .898 .898 r
.40084 .9436 Mdot
0 1 1 r
.38696 .94027 Mdot
1 0 0 r
.37319 .9365 Mdot
.898 .898 .898 r
.35954 .9323 Mdot
1 1 0 r
.34603 .92767 Mdot
.898 .898 .898 r
.33267 .92263 Mdot
0 .498 0 r
.31948 .91716 Mdot
.498 0 0 r
.30646 .91129 Mdot
1 0 1 r
.29364 .905 Mdot
0 .498 0 r
.28102 .89832 Mdot
0 1 0 r
.26862 .89125 Mdot
0 .498 0 r
.25644 .88379 Mdot
.498 0 0 r
.24451 .87595 Mdot
0 1 0 r
.23282 .86773 Mdot
0 1 1 r
.22141 .85916 Mdot
0 .498 0 r
.21026 .85023 Mdot
0 1 1 r
.1994 .84096 Mdot
1 0 1 r
.18884 .83135 Mdot
1 0 0 r
.17859 .82141 Mdot
.498 0 0 r
.16865 .81116 Mdot
0 0 .498 r
.15904 .8006 Mdot
0 1 0 r
.14977 .78974 Mdot
.898 .898 .898 r
.14084 .77859 Mdot
1 1 0 r
.13227 .76718 Mdot
0 1 0 r
.12405 .75549 Mdot
1 0 0 r
.11621 .74356 Mdot
1 0 1 r
.10875 .73138 Mdot
.498 .498 .498 r
.10168 .71898 Mdot
1 1 0 r
.095 .70636 Mdot
1 0 1 r
.08871 .69354 Mdot
0 1 1 r
.08284 .68052 Mdot
0 .498 0 r
.07737 .66733 Mdot
1 0 0 r
.07233 .65397 Mdot
0 1 0 r
.0677 .64046 Mdot
0 1 1 r
.0635 .62681 Mdot
0 .498 0 r
.05973 .61304 Mdot
0 0 .498 r
.0564 .59916 Mdot
1 0 0 r
.05351 .58517 Mdot
.498 0 0 r
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
0 0 .498 r
.04635 .52854 Mdot
.498 0 0 r
.04568 .51428 Mdot
1 0 0 r
.04545 .5 Mdot
.498 .498 .498 r
.04568 .48572 Mdot
1 0 0 r
.04635 .47146 Mdot
1 1 0 r
.04747 .45722 Mdot
.498 .498 .498 r
.04904 .44303 Mdot
0 1 1 r
.05105 .42889 Mdot
1 0 0 r
.05351 .41483 Mdot
1 0 1 r
.0564 .40084 Mdot
.898 .898 .898 r
.05973 .38696 Mdot
1 0 0 r
.0635 .37319 Mdot
.498 .498 .498 r
.0677 .35954 Mdot
1 0 0 r
.07233 .34603 Mdot
0 1 1 r
.07737 .33267 Mdot
1 1 0 r
.08284 .31948 Mdot
1 0 1 r
.08871 .30646 Mdot
1 0 0 r
.095 .29364 Mdot
.10168 .28102 Mdot
1 1 0 r
.10875 .26862 Mdot
1 0 0 r
.11621 .25644 Mdot
0 0 .498 r
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.898 .898 .898 r
.15904 .1994 Mdot
.498 .498 .498 r
.16865 .18884 Mdot
1 0 1 r
.17859 .17859 Mdot
1 1 0 r
.18884 .16865 Mdot
.498 .498 .498 r
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
1 0 1 r
.24451 .12405 Mdot
0 .498 0 r
.25644 .11621 Mdot
0 1 1 r
.26862 .10875 Mdot
.498 0 0 r
.28102 .10168 Mdot
1 1 0 r
.29364 .095 Mdot
0 .498 0 r
.30646 .08871 Mdot
.31948 .08284 Mdot
0 1 0 r
.33267 .07737 Mdot
1 0 1 r
.34603 .07233 Mdot
.35954 .0677 Mdot
1 0 0 r
.37319 .0635 Mdot
0 1 0 r
.38696 .05973 Mdot
.40084 .0564 Mdot
.498 0 0 r
.41483 .05351 Mdot
0 1 0 r
.42889 .05105 Mdot
1 0 0 r
.44303 .04904 Mdot
1 1 0 r
.45722 .04747 Mdot
.47146 .04635 Mdot
0 0 .498 r
.48572 .04568 Mdot
1 0 0 r
.5 .04545 Mdot
.498 0 0 r
.51428 .04568 Mdot
.52854 .04635 Mdot
.898 .898 .898 r
.54278 .04747 Mdot
.498 0 0 r
.55697 .04904 Mdot
1 1 0 r
.57111 .05105 Mdot
1 0 1 r
.58517 .05351 Mdot
.59916 .0564 Mdot
0 .498 0 r
.61304 .05973 Mdot
0 1 0 r
.62681 .0635 Mdot
0 1 1 r
.64046 .0677 Mdot
1 1 0 r
.65397 .07233 Mdot
.898 .898 .898 r
.66733 .07737 Mdot
1 0 1 r
.68052 .08284 Mdot
.69354 .08871 Mdot
.498 0 0 r
.70636 .095 Mdot
1 0 0 r
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
0 1 1 r
.75549 .12405 Mdot
1 1 0 r
.76718 .13227 Mdot
0 1 0 r
.77859 .14084 Mdot
1 0 0 r
.78974 .14977 Mdot
.498 0 0 r
.8006 .15904 Mdot
1 0 1 r
.81116 .16865 Mdot
0 1 0 r
.82141 .17859 Mdot
.498 0 0 r
.83135 .18884 Mdot
0 1 0 r
.84096 .1994 Mdot
1 0 0 r
.85023 .21026 Mdot
.898 .898 .898 r
.85916 .22141 Mdot
1 0 1 r
.86773 .23282 Mdot
0 .498 0 r
.87595 .24451 Mdot
.88379 .25644 Mdot
1 1 0 r
.89125 .26862 Mdot
1 0 0 r
.89832 .28102 Mdot
0 .498 0 r
.905 .29364 Mdot
1 0 0 r
.91129 .30646 Mdot
0 1 0 r
.91716 .31948 Mdot
1 0 0 r
.92263 .33267 Mdot
0 .498 0 r
.92767 .34603 Mdot
1 0 1 r
.9323 .35954 Mdot
.898 .898 .898 r
.9365 .37319 Mdot
0 0 .498 r
.94027 .38696 Mdot
0 1 1 r
.9436 .40084 Mdot
.898 .898 .898 r
.94649 .41483 Mdot
.94895 .42889 Mdot
.498 .498 .498 r
.95096 .44303 Mdot
1 0 1 r
.95253 .45722 Mdot
0 .498 0 r
.95365 .47146 Mdot
0 1 0 r
.95432 .48572 Mdot
1 0 0 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[120]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00P_ooool3oooo0005ooooooooo`3oool0oooo0?oo
ool00ol007l01Oooooooo`00ool00?oo003ooooo00?oO`0000Goooooogl00?mo003oO`00ooooo`03
onGUiH?ooooo001iooooo`?o0?l000Koooooool00?oo003oo`00ooooool00003oooo0005o`000?oo
o`3oool0oooo0?l000000ol007l01Ol0003oo`00ool00?oo003o000000?oO`0000Go0000ogl00?mo
003oO`00o`000003onGUi@05o`000?mo003oO`00ogl00?ooool00oooo`1kooooo`00MOooool3ogl0
0005ooooool0o`3o0?l0o`3o0?l000000ooo00000ol0003ooooooooo0002oooo0005ooooooooo`3o
ool0oooo0?ooool00ol007l01Oooooooo`00ool00?oo003ooooo00?oO`0000Goooooogl00?mo003o
O`00ooooo`03onGUi@05oooooomo003oO`00ogl00?l000000oooo`001?l0003oo`3oool0oooo0?mg
ooooo`00LOooool3o`3o0005oooooomo003oO`00ogl00?l000000ol0o`001?ooooooo`00ool00?oo
0007ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool3ogl00005oooooooo
o`3oool0oooo0?ooool00ooo0?l01?l0003oo`3oool0oooo0?mcooooo`00KOooool3o`3o0005oooo
ool0o`3o0?l0o`3o0?l000000omo000?ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool05?ooool3ool0o`05oooooooo0?ooo`3oool0ool000000ol0O`1_ooooo`00JOooool3ool00005
ooooool0o`3o0?l0o`3o0?l000000ol0o`0Cooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool06?ooool3ool0o`05ooooool0O`3o07l0o`1o0?l000000ol0o`1[ooooo`00I_ooool3ool0
o`?oo`0000Co0000o`3o0?l0o`3o0?l06?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo01cooooo0ol0O`001Ooooooo0?l0o`3o0?l0o`3o000000?o0?ooIoooool006;ooooo0ooo0?l0
1?l0003oo`3oool0oooo0?l3ool001cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Pooooo`?o0?l000Goooooo`3oool0oooo0?ooo`000003oooo06?ooooo001Rooooo`?oo`3o00Co
ooooool0oooo0?ooo`3o7oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo02Cooooo
0ol0ool01?ooooooool0oooo0?ooo`1Sooooo`00G_ooool3o`3o0004o`000?oo0?ooo`3oool0ob?o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Xooooo`?oool000Co0000onGUiOoU
iNGoiNGUGoooool005[ooooo0ol0O`001?l0003o0?l0o`3o0?l0o`0Wooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool0;?ooool3onGUi@04o`000?oo0?ooo`3oool0oe_ooooo001Joooo
o`?o07l000Coooooo`3o0?l0o`3o0?l09oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo02cooooo0ooUiND01?ooooooo`3oool0oooo0?mKooooo`00E_ooool3o`1o0004o`000?l0O`3o
07l0o`1o02_ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`Koo`3oF?oo
ool005Kooooo0ol0O`0`ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0<Oooool0
0ol0003oooooooooo`03ool0o`03o`000?oooooooooo05Gooooo001Cooooo`?oool00ol0O`0`oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0<?ooool01?l0003oooooooooooooool3
ool0o`04oooooomo003oO`00ogl005Cooooo001Cooooo`?oool0<oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool01_ooool3ogl005Cooooo001?oooo
o`?oO`0000Co0000oooo0?ooo`3oool0<oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo02oooooo00?o0000ooooooooool01_ooool3ogl00004o`000?oo003oo`00ool0053ooooo001?
ooooo`?oO`00=oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo02kooooo00?o0000
ooooooooool02oooool3ool0053ooooo001;ooooo`?o0?oo00Co0000ogl00?mo003oO`00=oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool03?ooool6
ool004gooooo001;ooooo`?o0?oo>oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
02cooooo00?o0000ooooooooool04?ooool3ool004gooooo0018ooooo`?o07l00ol0oollooooo`05
o`000?ooooooooooooooool00000;Oooool00ol0003oooooooooo`0Aooooo`?oo`0000Co0000ool0
0?oo003oo`00BOooool004Sooooo0ol0O`0oooooo`05o`000?ooooooooooooooool00000;Oooool0
0ol0003oooooooooo`0Eooooo`?oo`00BOooool004Cooooo0ooo0?l01?l0003o07l0o`1o0?l0O`0o
ooooo`05o`000?ooooooooooooooool00000;?ooool00ol0003oooooooooo`0Fooooo`?oo`000ol0
oom6ooooo`00A?ooool3ool0o`;ooooo00?o0000ooooooooool0?_ooool01Ol0003ooooooooooooo
oooo000002_ooooo00?o0000ooooooooool06_ooool3o`3oodKooooo0013ooooo`04o`000?oo0?oo
o`3oool0o`;ooooo00?o0000ooooooooool0?_ooool01Ol0003ooooooooooooooooo000002[ooooo
00?o0000ooooooooool06oooool3o`3oo`;o0000A?ooool0047ooooo0omoOgl5ooooo`03o`000?oo
oooooooo03kooooo00Go0000ooooooooooooooooo`00000Zooooo`03o`000?oooooooooo01kooooo
0oooo`13ooooo`00@Oooool3ogmoO`Gooooo00?o0000ooooooooool0?_ooool01Ol0003ooooooooo
oooooooo000002Wooooo00?o0000ooooooooool07oooool3oooo0003o`000?oooooooooo043ooooo
000nooooo`KoOgmo1Oooool00ol0003oooooooooo`0oooooo`04o`000?ooooooooooo`0002Sooooo
00?o0000ooooooooool08?ooool3oooo0004ooooool0o`3o0?l0o`3o03oooooo000nooooo`?oOgmo
2Oooool00ol0003oooooooooo`0nooooo`04o`000?ooooooooooo`0002Oooooo00?o0000oooooooo
ool09Oooool3o`3o03oooooo000mooooo`04o`000?moOgooOgmoogmoO`Wooooo00?o0000oooooooo
ool0?_ooool01?l0003oooooooooool0000Vooooo`03o`000?oooooooooo02Kooooo0ol0o`000ol0
003oooooooooo`0looooo`00>oooool3ogmoO`cooooo00?o0000ooooooooool0?_ooool01?l0003o
ooooooooool0000Vooooo`03o`000?oooooooooo02Wooooo0ooo000looooo`00>oooool3ogmoO`co
oooo00?o0000ooooooooool0?_ooool01?l0003oooooooooool0000Uooooo`03o`000?oooooooooo
02[ooooo0ooo000looooo`00>?ooool6ogmoO`cooooo00?o0000ooooooooool0?_ooool01?l0003o
ooooooooool0000Tooooo`03o`000?oooooooooo02_ooooo0ooo0003ogl003Wooooo000hooooo`?o
Ogmo3oooool00ol0003oooooooooo`0nooooo`04o`000?ooooooooooo`0002?ooooo00?o0000oooo
ooooool0;oooool3ogl003Wooooo000hooooo`?oOgmo3oooool00ol0003oooooooooo`0oooooo`;o
00009?ooool00ol0003oooooooooo`0_ooooo`?oO`00>Oooool003Gooooo0oooo`0Booooo`03o`00
0?oooooooooo03oooooo0_l0000Sooooo`03o`000?oooooooooo03?ooooo0ooo0?lfooooo`00=Ooo
ool3oooo01;ooooo00?o0000ooooooooool0?oooool2o`0002;ooooo00?o0000ooooooooool0=?oo
ool3ool0ocKooooo000eooooo`?oool04_ooool00ol0003oooooooooo`0oooooo`;o00008Oooool0
0ol0003oooooooooo`0eooooo`?oo`3o=_ooool003;ooooo0ooo0?lEooooo`03o`000?oooooooooo
03oooooo0_l0000Pooooo`03o`000?oooooooooo03Wooooo0ol0o`0cooooo`00<_ooool3ool0oaGo
oooo00?o0000ooooooooool0?oooool2o`00023ooooo00?o0000ooooooooool0>Oooool3o`3o03?o
oooo000booooo`?oo`3o5Oooool00ol0003oooooooooo`0oooooo`;o00007oooool00ol0003ooooo
ooooo`0jooooo`?o0?l0<oooool002oooooo0omoOglIooooo`03o`000?oooooooooo03oooooo00?o
0000ooooooooool07?ooool00ol0003oooooooooo`0nooooo`?oO`00<?ooool002oooooo0omoOglI
ooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`0o
ooooo`?oO`00<?ooool002oooooo0omoOglIooooo`03o`000?oooooooooo03oooooo00?o0000oooo
ooooool06oooool00ol0003oooooooooo`0oooooo`?oO`00<?ooool002cooooo0ooUiNDLooooo`03
o`000?oooooooooo03oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`13ooooo`?o
0?l0;Oooool002cooooo0ooUiNDLooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`14ooooo`?o0?l0;Oooool002cooooo0ooUiNDLooooo`03o`000?oo
oooooooo03oooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`15ooooo`?o0?l0;Ooo
ool002Wooooo0ol007lOooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool05oooool0
0ol0003oooooooooo`18ooooo`?oo`00:oooool002Wooooo0ol007lOooooo`03o`000?oooooooooo
03oooooo0_l0000Hooooo`03o`000?oooooooooo04Sooooo0ooo000[ooooo`00:Oooool3o`00Oaoo
oooo00?o0000ooooooooool0?oooool2o`0001Oooooo00?o0000ooooooooool0BOooool3ool002_o
oooo000Wooooo`?o001o8Oooool00ol0003oooooooooo`0oooooo`;o00005_ooool00ol0003ooooo
ooooo`1=ooooo`?oiNGU:?ooool002Oooooo0ol007lQooooo`03o`000?oooooooooo03oooooo0_l0
000Eooooo`03o`000?oooooooooo04kooooo0ooUiNDXooooo`009oooool3o`00Ob7ooooo00?o0000
ooooooooool0?oooool2o`0001Gooooo00?o0000ooooooooool0C_ooool3onGUiBSooooo000Voooo
o`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0?oooool2o`0001Cooooo00?o0000oooo
ooooool0D_ooool00ol0003oooooooooo`0Uooooo`009?ooool3o`00ObGooooo00?o0000oooooooo
ool0?_ooool2o`0001?ooooo00?o0000ooooooooool0D_ooool3ool0obKooooo000Tooooo`?o001o
9Oooool00ol0003oooooooooo`0nooooo`;o00004_ooool00ol0003oooooooooo`1Cooooo`?oo`3o
9_ooool002Cooooo0ol007lUooooo`03o`000?oooooooooo03gooooo00Co0000oooooooooooo0000
4Oooool00ol0003oooooooooo`1Cooooo`?oo`3o00?o0000ooooooooool08oooool002;ooooo0ol0
07lWooooo`03o`000?oooooooooo03gooooo00Co0000oooooooooooo00004?ooool00ol0003ooooo
ooooo`1Gooooo`?o07l08oooool002;ooooo0ol007lWooooo`03o`000?oooooooooo03gooooo00Co
0000oooooooooooo00003oooool00ol0003oooooooooo`1Hooooo`?o07l08oooool002;ooooo0ol0
07lWooooo`03o`000?oooooooooo03gooooo00Co0000oooooooooooo00003_ooool00ol0003ooooo
ooooo`1Iooooo`?o07l08oooool0023ooooo0ooo000Yooooo`03o`000?oooooooooo03gooooo00Co
0000oooooooooooo00003Oooool00ol0003oooooooooo`1Looooo`?o07l08Oooool0023ooooo0ooo
000Yooooo`03o`000?oooooooooo03gooooo00Co0000oooooooooooo00003Oooool00ol0003ooooo
ooooo`1Looooo`?o07l08Oooool0023ooooo0ooo000Yooooo`03o`000?oooooooooo03gooooo00Co
0000oooooooooooo00003?ooool00ol0003oooooooooo`1Mooooo`?o07l08Oooool0023ooooo00?o
0000ooooooooool0:Oooool00ol0003oooooooooo`0mooooo`05o`000?ooooooooooooooool00000
2_ooool00ol0003oooooooooo`1Pooooo`03o`000?oooooooooo01oooooo000Nooooo`?oool0:ooo
ool00ol0003oooooooooo`0mooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooo
ooooo`1Qooooo`?oool07oooool001kooooo0oooo`0[ooooo`03o`000?oooooooooo03gooooo00Go
0000ooooooooooooooooo`000009ooooo`03o`000?oooooooooo067ooooo0oooo`0Oooooo`007_oo
ool3oooo02_ooooo00?o0000ooooooooool0?Oooool01Ol0003ooooooooooooooooo000000Sooooo
00?o0000ooooooooool0H_ooool3oooo01oooooo000Kooooo`?oo`0000?o0000ooooooooool0;?oo
ool00ol0003oooooooooo`0looooo`05o`000?ooooooooooooooool000001oooool00ol0003ooooo
ooooo`1Uooooo`?oo`007Oooool001_ooooo0ooo000_ooooo`03o`000?oooooooooo03cooooo00Go
0000ooooooooooooooooo`000006ooooo`03o`000?oooooooooo06Kooooo0ooo000Mooooo`006ooo
ool3ool002oooooo00?o0000ooooooooool0??ooool01Ol0003ooooooooooooooooo000000Gooooo
00?o0000ooooooooool0Ioooool3ool001gooooo000Looooo`03o`000?oooooooooo02kooooo00?o
0000ooooooooool0??ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool0JOooool00ol0003oooooooooo`0Kooooo`006_ooool3ool0033ooooo00?o0000
ooooooooool0??ooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool00000K?oo
ool3o`1o01_ooooo000Jooooo`?oo`00<?ooool00ol0003oooooooooo`0looooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo0000KOooool3o`1o01_ooooo000Jooooo`?oo`00<?ooool0
0ol0003oooooooooo`0looooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000K_ooool3
o`1o01_ooooo000Jooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool0??ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo0000073ooooo00?o0000ooooooooool06Oooool001So
oooo0ooo0?lbooooo`03o`000?oooooooooo03cooooo00?o0000ooooooooool00_ooool2o`00077o
oooo0ooo000Iooooo`006?ooool3ool0oc;ooooo00?o0000ooooooooool0??ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo073ooooo0ooo000Iooooo`006?ooool3ool0oc;ooooo00?o
0000ooooooooool0??ooool01ol0003ooooooooooooooooo0000ooooool00000LOooool3ool001Wo
oooo000Fooooo`?oool0=?ooool00ol0003oooooooooo`0kooooo`05o`000?ooooooooooooooool0
00000_ooool00ol0003oooooooooo`1aooooo`?o0?l05oooool001Kooooo0oooo`0eooooo`03o`00
0?oooooooooo03[ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo077o
oooo0ol0o`0Gooooo`005_ooool3oooo03Gooooo00?o0000ooooooooool0>_ooool01?l0003ooooo
ooooool00003ooooo`03o`000?oooooooooo077ooooo0ol0o`0Gooooo`005oooool00ol0003ooooo
ooooo`0dooooo`03o`000?oooooooooo03[ooooo00?o0000ooooool000001?ooool00ol0003ooooo
ooooo`1booooo`03o`000?oooooooooo01Kooooo000Eooooo`?o0?oo=_ooool00ol0003ooooooooo
o`0jooooo`;o00001Oooool00ol0003oooooooooo`1booooo`?oo`005_ooool001Gooooo0ol0oolf
ooooo`03o`000?oooooooooo03[ooooo0_l00005ooooo`03o`000?oooooooooo07;ooooo0ooo000F
ooooo`005Oooool3o`3oocKooooo00?o0000ooooooooool0>_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo077ooooo0ooo000Fooooo`005Oooool00ol0003oooooooooo`0fooooo`03
o`000?oooooooooo03Wooooo0_l00007ooooo`03o`000?oooooooooo07?ooooo00?o0000oooooooo
ool05?ooool001?ooooo0ooo000hooooo`03o`000?oooooooooo03Sooooo00?o0000ooooool00000
1oooool00ol0003oooooooooo`1cooooo`?o07l05?ooool001?ooooo0ooo000hooooo`03o`000?oo
oooooooo03Oooooo00Co0000oooooooooooo00001oooool00ol0003oooooooooo`1cooooo`?o07l0
5?ooool001?ooooo0ooo000hooooo`03o`000?oooooooooo03Oooooo00Co0000oooooooooooo0000
1oooool00ol0003oooooooooo`1cooooo`?o07l05?ooool001Cooooo00?o0000ooooooooool0=ooo
ool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000001oooool00ol0003ooooo
ooooo`1dooooo`03o`000?oooooooooo01?ooooo000Booooo`?oOgmo>Oooool00ol0003ooooooooo
o`0eooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`1dooooo`?oo`3o4oooool001;ooooo0omoOgliooooo`03o`000?oooooooooo03Cooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo07?ooooo0ooo0?lC
ooooo`004_ooool3ogmoOc[ooooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0Loooool3ool0oa?ooooo000Cooooo`03
o`000?oooooooooo03Wooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool0Loooool2o`0001Cooooo000Aooooo`?oo`00
>oooool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`1cooooo`04o`000?oUiNGoiNGUonGUiA;ooooo000Aooooo`?oo`00
>oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`1booooo`03o`000?ooooooiNGU00;oiNGU4_ooool0017ooooo0ooo
000kooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo07;ooooo00?o0000oooooooUiND00_oUiNDBooooo`004_ooool0
0ol0003oooooooooo`0jooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo07;ooooo00Co0000oooooooooooo00004ooo
ool000oooooo0ooUiNDmooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo073ooooo00Co0000oooooooooooooooo0ol0
07lAooooo`003oooool3onGUiCgooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0L?ooool01?l0003ooooooooooooo
ool3o`00Oa7ooooo000?ooooo`?oiNGU?Oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`1_ooooo`03o`000?oooooo
oooo00;ooooo0ol007lAooooo`004Oooool00ol0003oooooooooo`0kooooo`03o`000?oooooooooo
02cooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
06oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0@ooooo`003oooool3ool0ocgo
oooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`04ooooo`?o0?oo4?ooool000oooooo
0ooo0?lmooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool02oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool01?ooool3o`3ooa3ooooo
000?ooooo`?oo`3o?Oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo00Gooooo0ol0
ool@ooooo`004?ooool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo02Sooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0?ooooo`003_ooool3ool003oooooo00?o0000oooo
ooooool09oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool0Joooool00ol0003oooooooooo`07ooooo`?oiNGU3oooool000kooooo0ooo000oooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo06_ooooo00?o0000ooooooooool01oooool3onGUi@oooooo000>ooooo`?oo`00
?oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`1[ooooo`03o`000?oooooooooo00Oooooo0ooUiND?ooooo`003ooo
ool00ol0003oooooooooo`0nooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo06[ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0>ooooo`003Oooool3o`3ood3ooooo00?o0000ooooooooool09?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0J_ooool0
0ol0003oooooooooo`09ooooo`?oiNGU3_ooool000gooooo0ol0oom0ooooo`03o`000?oooooooooo
02?ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
06Wooooo00?o0000ooooooooool02_ooool3onGUi@kooooo000=ooooo`?o0?oo@?ooool00ol0003o
ooooooooo`0Rooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`1Yooooo`03o`000?oooooooooo00[ooooo0ooUiND>ooooo`003_ooool00ol0003ooooo
ooooo`0oooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`0:ooooo`03o`000?oooooooooo06Oooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`0=ooooo`003?ooool3ogmoOd7ooooo00?o0000ooooooooool08Oooool00ol0003ooooooooo
o`0Cooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0Ioooool00ol0003ooooooooo
o`0;ooooo`?oOgmo3_ooool000cooooo0omoOgm1ooooo`03o`000?oooooooooo023ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Kooooo00?o0000
ooooooooool03?ooool3ogmoO`kooooo000<ooooo`?oOgmo@Oooool00ol0003oooooooooo`0Ooooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1Voooo
o`03o`000?oooooooooo00cooooo0omoOgl>ooooo`003_ooool00ol0003oooooooooo`0oooooo`03
o`000?oooooooooo01kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo06Kooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0=ooooo`00
3?ooool3oooo04;ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0Gooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool0IOooool00ol0003oooooooooo`0>ooooo`?oo`3o
3Oooool000cooooo0oooo`12ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Gooooo00?o0000ooooooooool03_oo
ool3ool0o`gooooo000<ooooo`?oool0@_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`1Sooooo`03o`000?oooooo
oooo00oooooo0ooo0?l=ooooo`003Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
06?ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0<ooooo`003?ooool3ool004;o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool0H_ooool00ol0003oooooooooo`0@ooooo`?o07l03Oooool000cooooo
0ooo0012ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo06;ooooo00?o0000ooooooooool04?ooool3o`1o00gooooo
000<ooooo`?oo`00@_ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`1Qooooo`03o`000?oooooooooo017ooooo0ol0
O`0=ooooo`003Oooool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01Oooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo067ooooo00?o0000
ooooooooool04_ooool00ol0003oooooooooo`0<ooooo`002oooool3ogmoOd?ooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool0H?ooool00ol0003oooooooooo`0Booooo`?o0?l03Oooool000_ooooo0omoOgm3ooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo05oooooo00?o0000ooooooooool04_ooool3o`3o00gooooo000;ooooo`?oOgmo
@oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0
3Oooool00ol0003oooooooooo`1Oooooo`03o`000?oooooooooo01;ooooo0ol0o`0=ooooo`003Ooo
ool00ol0003oooooooooo`11ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool07_oo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05kooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0<ooooo`002oooool3ool004?ooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`0Oooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0G_ooool0
0ol0003oooooooooo`0Cooooo`?oo`003Oooool000_ooooo0ooo0014ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
05gooooo00?o0000ooooooooool05?ooool3ool000gooooo000;ooooo`?oo`00A?ooool00ol0003o
ooooooooo`0Aooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`1Mooooo`03o`000?oooooooooo01Cooooo0ooo000=ooooo`003Oooool00ol0003ooooo
ooooo`12ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool08Oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo05cooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0<ooooo`002oooool3ogl004Cooooo00?o0000ooooooooool03oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0Foooool00ol0003ooooooooo
o`0Eooooo`?oool03Oooool000_ooooo0omo0014ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool08oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05[ooooo00?o0000
ooooooooool05_ooool3oooo00gooooo000;ooooo`?oO`00A?ooool00ol0003oooooooooo`0>oooo
o`03o`000?oooooooooo02?ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Joooo
o`03o`000?oooooooooo01Kooooo0oooo`0=ooooo`003Oooool00ol0003oooooooooo`12ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo05[ooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0<ooooo`00
3?ooool3o`00Od?ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo00kooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`0Gooooo`?oo`3o
3Oooool000cooooo0ol007m3ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05Wooooo00?o0000ooooooooool05ooo
ool3ool0o`gooooo000<ooooo`?o001o@oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo02Oooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Hooooo`03o`000?oooooo
oooo01Sooooo0ooo0?l=ooooo`003Oooool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo
05Oooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0<ooooo`003?ooool3ogl004?o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0Iooooo`?oOgmo3Oooool000cooooo
0omo0014ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0:?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo05Kooooo00?o0000ooooooooool06Oooool3ogmoO`gooooo
000<ooooo`?oO`00A?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo02Wooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo01[ooooo0omo
Ogl=ooooo`003Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool0:Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo05Gooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`12oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooooooo05Cooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`0=oooo
o`003?ooool3ogl004Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0[ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool0E?ooool00ol0003oooooooooo`0Jooooo`?o
ool03_ooool000cooooo0omo0014ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
;?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo05?ooooo00?o0000ooooooooool0
6_ooool3oooo00kooooo000<ooooo`?oO`00A?ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo02gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1Booooo`03o`000?oo
oooooooo01_ooooo0oooo`0>ooooo`003_ooool00ol0003oooooooooo`12ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo05;ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0=ooooo`003Oooool3ogl0
04?ooooo00Go0000ooooooooooooooooo`00000`ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool0DOooool00ol0003oooooooooo`0Looooo`?oo`3o3_ooool000gooooo0omo0013oooo
o`04o`000?ooooooooooo`00037ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1A
ooooo`03o`000?oooooooooo01cooooo0ooo0?l>ooooo`003Oooool3ogl004?ooooo00?o0000oooo
ool00000<_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo053ooooo00?o0000oooo
ooooool07Oooool3ool0o`kooooo000?ooooo`03o`000?oooooooooo047ooooo00?o0000ooooool0
0000<_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo053ooooo00?o0000oooooooo
ool07Oooool00ol0003oooooooooo`0>ooooo`003_ooool3ool004;ooooo0_l0000cooooo`03o`00
0?oooooooooo01;ooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0Mooooo`?o0?oo
3oooool000kooooo0ooo0012ooooo`;o0000<oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo04kooooo00?o0000ooooooooool07Oooool3o`3oo`oooooo000>ooooo`?oo`00@Oooool0
0ol0003oooooo`00000cooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0C_ooool0
0ol0003oooooooooo`0Mooooo`?o0?oo3oooool0013ooooo00?o0000ooooooooool0?_ooool01?l0
003oooooooooool0000cooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0COooool0
0ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00oooooo000?ooooo`?o001o?oooool01?l0
003oooooooooool0000cooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0COooool0
0ol0003oooooooooo`0Mooooo`?oo`004?ooool000oooooo0ol007lnooooo`05o`000?oooooooooo
ooooool00000<oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo04cooooo00?o0000
ooooooooool07_ooool3ool0013ooooo000?ooooo`?o001o?Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo037ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`1<oooo
o`03o`000?oooooooooo01kooooo0ooo000@ooooo`004Oooool00ol0003oooooooooo`0jooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo04_ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0@ooooo`00
3oooool3o`1o03cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0aooooo`03o`00
0?oooooooooo01?ooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`0Nooooo`?oO`00
4Oooool000oooooo0ol0O`0kooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool07ooo
ool3ogl0017ooooo000?ooooo`?o07l0>_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo033ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`19ooooo`03o`000?oooooo
oooo01oooooo0omo000Aooooo`004_ooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
04Sooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0Aooooo`004Oooool3o`3oocKo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool0B?ooool00ol0003oooooooooo`0Oooooo`?oo`3o4_ooool0017ooooo
0ol0oolfooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0<?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool07oooool3ool0oa;ooooo
000Aooooo`?o0?oo=Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo02oooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo023ooooo0ooo
0?lBooooo`004_ooool3o`3o03?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0_
ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0A_ooool00ol0003oooooooooo`0O
ooooo`?oO`004oooool001;ooooo0ol0o`0booooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool0;oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04Gooooo00?o0000oooo
ooooool08?ooool3ogl001?ooooo000Booooo`?o0?l0<_ooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo02oooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`15ooooo`03
o`000?oooooooooo023ooooo0omo000Cooooo`004oooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo04Cooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0Booooo`004ooo
ool3ool002oooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0_ooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`0Pooooo`?oO`005?oo
ool001?ooooo0ooo000^ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0;oooool0
0ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04?ooooo00?o0000ooooooooool08Oooool3
ogl001Cooooo000Cooooo`?oo`00;_ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
02oooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooooooo
027ooooo0omo000Dooooo`005Oooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo04;o
oooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`0Dooooo`005Oooool3o`1o02[ooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01Kooooo
00?o0000ooooooooool0@Oooool00ol0003oooooooooo`0Pooooo`?o0?oo5_ooool001Gooooo0ol0
O`0Yooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0;oooool00ol0003ooooooooo
o`0Fooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool08?ooool3o`3ooaKooooo000E
ooooo`?o07l0:?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo02oooooo00?o0000
ooooooooool05_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo027ooooo0ol0oolF
ooooo`005_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool0;oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo043ooooo00?o0000oooo
ooooool08_ooool00ol0003oooooooooo`0Eooooo`005_ooool3o`3oobKooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool0?oooool00ol0003oooooooooo`0Qooooo`?oool05oooool001Kooooo0ol0oolUooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0Fooooo`03o`00
0?oooooooooo03oooooo00?o0000ooooooooool08Oooool3oooo01Oooooo000Fooooo`?o0?oo9?oo
ool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0mooooo`03o`000?oooooooooo02;ooooo0oooo`0Gooooo`006?ooool0
0ol0003oooooooooo`0Rooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0;_ooool0
0ol0003oooooooooo`0Gooooo`03o`000?oooooooooo03gooooo00?o0000ooooooooool08_ooool0
0ol0003oooooooooo`0Gooooo`006?ooool3ool0ob7ooooo00?o0000ooooooooool05oooool00ol0
003oooooooooo`0]ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool0??ooool00ol0
003oooooooooo`0Qooooo`?oo`006Oooool001Sooooo0ooo0?lPooooo`03o`000?oooooooooo01So
oooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo03co
oooo00?o0000ooooooooool08Oooool3ool001Wooooo000Hooooo`?oo`3o7oooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool06?ooool00ol0003ooooo
ooooo`0looooo`03o`000?oooooooooo027ooooo0ooo000Iooooo`006_ooool3oooo01cooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool0>oooool00ol0003oooooooooo`0Pooooo`?oO`006oooool001[ooooo0oooo`0L
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0H
ooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool08?ooool3ogl001_ooooo000Joooo
o`?oool06oooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo02gooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0iooooo`03o`000?oooooooooo027ooooo0omo000Koooo
o`007?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01cooooo00?o0000oooooooo
ool0;Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo03Wooooo00?o0000oooooooo
ool08Oooool00ol0003oooooooooo`0Kooooo`006oooool3ogmoOaSooooo00?o0000ooooooooool0
7Oooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
>?ooool00ol0003oooooooooo`0Pooooo`?oool07Oooool001_ooooo0omoOglHooooo`03o`000?oo
oooooooo01kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo03Sooooo00?o0000ooooooooool08?ooool3oooo01gooooo000Kooooo`?oOgmo5oooool0
0ol0003oooooooooo`0Oooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0gooooo`03o`000?oooooooooo027ooooo0oooo`0Mooooo`007_ooool00ol0
003oooooooooo`0Cooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool0;?ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo03Oooooo00?o0000ooooooooool08Oooool00ol0
003oooooooooo`0Mooooo`007_ooool3ool0oa;ooooo00?o0000ooooooooool08Oooool00ol0003o
ooooooooo`0/ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0=oooool00ol0003o
ooooooooo`0Oooooo`?oo`007oooool001kooooo0ooo0?lAooooo`03o`000?oooooooooo02;ooooo
00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo03Gooooo
00?o0000ooooooooool08?ooool3ool001oooooo000Nooooo`?oo`3o4Oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0eooooo`03o`000?oooooooooo023ooooo0ooo000Oooooo`008?ooool3ool000kooooo00?o0000
ooooooooool08oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool0=?ooool00ol0003oooooooooo`0Oooooo`?oO`008Oooool0023ooooo0ooo000=oooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo03Cooooo00?o0000ooooooooool07oooool3ogl0027ooooo000Pooooo`?o
o`003?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo02cooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo023ooooo0omo000Qooooo`00
8_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
;?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0
8?ooool00ol0003oooooooooo`0Qooooo`008_ooool3o`3o00Wooooo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0<_oo
ool00ol0003oooooooooo`0Oooooo`?oool08oooool002;ooooo0ol0o`08ooooo`03o`000?oooooo
oooo02Oooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooo
oooo037ooooo00?o0000ooooooooool07oooool3oooo02?ooooo000Rooooo`?o0?l01oooool00ol0
003oooooooooo`0Xooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`0`ooooo`03o`000?oooooooooo023ooooo0oooo`0Sooooo`009?ooool3oooo00Co
oooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo01_o
oooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Mooooo`?o07l000?o0000oooooooo
ool08oooool002Cooooo0oooo`04ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0
:_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool0
7Oooool3o`1o02Kooooo000Tooooo`?oool00oooool00ol0003oooooooooo`0[ooooo`03o`000?oo
oooooooo02[ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0_ooooo`03o`000?oo
oooooooo01kooooo0ol0O`0Vooooo`009oooool3onGUiBkooooo00?o0000ooooooooool0:_ooool0
0ol0003oooooooooo`0Looooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool07?ooool3
o`00ObSooooo000Wooooo`?oiNGU;_ooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo
01cooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`0Mooooo`?o001o:?ooool002Oo
oooo0ooUiND^ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool07?ooool00ol0003o
ooooooooo`0^ooooo`03o`000?oooooooooo01gooooo0ol007lXooooo`00:Oooool3o`3o02cooooo
00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo02cooooo
00?o0000ooooooooool06oooool3ool0o`03o`000?oooooooooo02Sooooo000Yooooo`?o0?l0;?oo
ool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool0;?oo
ool00ol0003oooooooooo`0Kooooo`?oo`3o:oooool002Wooooo0ol0o`0/ooooo`03o`000?oooooo
oooo02[ooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0[ooooo`03o`000?oooooo
oooo01cooooo0ooo0?l[ooooo`00;?ooool3o`00ObWooooo00?o0000ooooooooool0:_ooool00ol0
003oooooooooo`0Mooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool06_ooool3ogl0
02gooooo000/ooooo`?o001o:Oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo01go
oooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0Jooooo`?oO`00;Oooool002cooooo
0ol007lYooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool07Oooool00ol0003ooooo
ooooo`0Zooooo`03o`000?oooooooooo01_ooooo0omo000]ooooo`00;oooool3ogl002Kooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo02[ooooo00?o
0000ooooooooool06?ooool3ool0oc3ooooo000_ooooo`?oO`009oooool00ol0003oooooooooo`0Y
ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0I
ooooo`?oo`3o<?ooool002oooooo0omo000Wooooo`03o`000?oooooooooo02Wooooo00?o0000oooo
ooooool07_ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo01Wooooo0ooo0?l`oooo
o`00<_ooool3ool002Cooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Nooooo`03
o`000?oooooooooo02Oooooo00?o0000ooooooooool05oooool3ogmoOc?ooooo000booooo`?oo`00
9?ooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0
9oooool00ol0003oooooooooo`0Gooooo`?oOgmo<oooool003;ooooo0ooo000Tooooo`03o`000?oo
oooooooo02Wooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Vooooo`03o`000?oo
oooooooo01Sooooo0omoOglcooooo`00=Oooool3ool0ob7ooooo00?o0000ooooooooool0:Oooool0
0ol0003oooooooooo`0Nooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool05Oooool3
ool0ocKooooo000eooooo`?oo`3o8Oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo
01kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Fooooo`?oo`3o=_ooool003Go
oooo0ooo0?lQooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool07oooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo01Kooooo0ooo0?lfooooo`00>?ooool3o`3ooakooooo
00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool04oooool3ool0ocWooooo000hooooo`?o0?oo7_ooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool08oooool00ol0003ooooooooo
o`0Dooooo`?oo`3o>Oooool003Sooooo0ol0oolNooooo`03o`000?oooooooooo02Sooooo00?o0000
ooooooooool08?ooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo01Cooooo0ooo0?li
ooooo`00>oooool3o`1o01_ooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0Poooo
o`03o`000?oooooooooo02;ooooo00?o0000ooooooooool04_ooool3o`3o03cooooo000kooooo`?o
07l06oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo023ooooo00?o0000oooooooo
ool08_ooool00ol0003oooooooooo`0Booooo`?o0?l0??ooool003_ooooo0ol0O`03o`3ooaWooooo
00?o0000ooooooooool09oooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo027ooooo
00?o0000ooooooooool04?ooool3o`3oo`?o0?l0??ooool003kooooo0ol0oolIooooo`03o`000?oo
oooooooo02Oooooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`0Pooooo`03o`000?oo
oooooooo013ooooo0ol0ooloooooo`00?_ooool3o`3ooaWooooo00?o0000ooooooooool09oooool0
0ol0003oooooooooo`0Qooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool04Oooool3
o`3oocoooooo0011ooooo`?o0?l05_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo
027ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0=ooooo`?o07l000?o0000oooo
ooooool0@?ooool0047ooooo0ol0o`0Fooooo`03o`000?oooooooooo02Oooooo00?o0000oooooooo
ool08Oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo00gooooo0ol0O`13ooooo`00
@Oooool3o`3o00?oO`004oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo027ooooo
00?o0000ooooooooool07_ooool00ol0003oooooooooo`0;ooooo`?oO`000ol0O`13ooooo`00A?oo
ool3ogl00003o`000?oooooooooo013ooooo00?o0000ooooooooool09oooool00ol0003ooooooooo
o`0Qooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool02oooool3ogl004Kooooo0014
ooooo`?oO`0000Coooooo`1o0?l0O`3o07l03oooool00ol0003oooooooooo`0Wooooo`03o`000?oo
oooooooo027ooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`09ooooo`?oOgmo0omo
0016ooooo`00B?ooool3o`1o00oooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0R
ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool02Oooool3ogmoOdWooooo0018oooo
o`?o07l00ol0o`0<ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool08_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo00Kooooo0ooUiND01?l0003oOgmoogmoOomoOgm9
ooooo`00Boooool3o`3o0003o`000?oooooooooo00Wooooo00?o0000ooooooooool09oooool00ol0
003oooooooooo`0Rooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool01_ooool3onGU
iDgooooo001;ooooo`?o0?l000?oooooo`000?ooool02Oooool00ol0003oooooooooo`0Wooooo`03
o`000?oooooooooo02;ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`06ooooo`04
o`000?oUiNGoiNGUonGUiDgooooo001?ooooo`?o07l02?ooool00ol0003oooooooooo`0Wooooo`03
o`000?oooooooooo02;ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`?o
ool0D?ooool004oooooo0ol0O`001?l0003oo`3oool0oooo0?l5ooooo`03o`000?oooooooooo02Ko
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0Iooooo`04o`000?oooooooooooooo
o`?oOgmo00Co0000oooo0?ooo`3oool0D?ooool004oooooo0ol0O`001?ooooooo`3oool0oooo0?l5
ooooo`03o`000?oooooooooo02Kooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0I
ooooo`04o`000?ooooooooooooooo`?oOgmo00Cooooooooo0?ooo`3oool0D?ooool005?ooooo0ooo
0?l3ogl000;ooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`0Sooooo`03o`000?oo
oooooooo01Sooooo0ol0ool01?l0003oOgmoogmoOomoOgmDooooo`00E_ooool3ogl000;ooooo00?o
0000ooooooooool09_ooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo01Oooooo00Co
0000o`3oool0oooo0?ooF?ooool005Kooooo0omo00001?l0003o07l0o`1o0?l0O`0Wooooo`03o`00
0?oooooooooo02?ooooo00?o0000ooooooooool05Oooool3ogmoO`?o0?ooF?ooool005[ooooo0ol0
O`001?l0003oiNGUonGUiOoUiNDRooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0
4Oooool3o`3oo`04o`000?moOgooOgmoogmoOe_ooooo001Jooooo`?o07l000CoooooonGUiOoUiNGo
iNGU8_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo017ooooo0ol0ool01?oooooo
OgmoogmoOomoOgmKooooo`00G_ooool3onGUi@04o`000?ooo`3oool0oooo01kooooo00?o0000oooo
ooooool09?ooool00ol0003oooooooooo`0=ooooo`?o07l000Co0000o`3oool0oooo0?ooGoooool0
06;ooooo0oooo`001?l0003oiNGUonGUiOoUiNDJooooo`03o`000?oooooooooo02Cooooo00?o0000
ooooooooool02Oooool3onGUi@04o`000?l0O`3o07l0o`1o06?ooooo001Rooooo`?oool000Cooooo
onGUiOoUiNGoiNGU0ooo000Gooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool01?oo
ool3o`3oo`05o`000?oUiNGoiNGUonGUiOooool00ol0O`1Sooooo`00I_ooool3onGUi@?oo`000_l0
000Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00oooool01Ol0003o0?ooo`3o
ool0oooooooo00?oiNGUIoooool006Wooooo0ooo00001Ooooooo0?ooo`3oool0oooo000000?oiNGU
3oooool00ol0003oooooooooo`0Tooooo`?o0?oo00Go0000o`1o0?l0O`3o07l0ooooo`03o`3oof_o
oooo001]ooooo`?o0?oo00GoooooonGUiOoUiNGoiNGUo`000003o`00O`_ooooo00?o0000oooooooo
ool08?ooool3ool00005o`000?l0oooo0?ooo`3oooooool00ol0O`1_ooooo`00KOooool3o`3oo`05
oooooooUiNGoiNGUonGUiOooool00ol007l01?l0003oOgmoogmoOomoOgl7ooooo`03o`000?oooooo
oooo01cooooo0omoOgl01Ol0003oo`00ool00?oo003ooooo00?o0?oo00Coooooo`1o0?l0O`3o07l0
Koooool007Gooooo0ol007l01OooooooOgmoogmoOomoOgoo000000?o001o0_l00003onGUi@04oooo
oomoOgooOgmoogmoO`gooooo0ol0O`001Oooooooo`3oool0oooo0?oo000000?o0?oo00Go0000ogmo
OomoOgooOgmoooooo`03ool007Oooooo001iooooo`?oOgmo00Coooooo`00Ool007oo001o0_ooool3
onGUi@05o`000?moOgooOgmoogmoOol000000ol007l01Ol0003oiNGUonGUiOoUiNGo000000?oool0
00Go0000o`1o0?l0O`3o07l0o`000003ool0o`05ooooool0oooo0?ooo`3oooooool00omoOgmkoooo
o`00OOooool3o`00O`;ooooo0ooUiND01OooooooOgmoogmoOomoOgoooooo00?o001o00GoooooonGU
iOoUiNGoiNGUooooo`03oooo0005ooooool0O`3o07l0o`1o0?ooool00ooo0?l01?oooooo0?ooo`3o
ool0oomoooooo`00R_ooool3o`00O`05oooooooUiNGoiNGUonGUiOooool00oooo`2;ooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(Evolve[
      dtd3, \[IndentingNewLine]Generations \[Rule] 
        30, \[IndentingNewLine]DisplayPause \[Rule] 
        0.3\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[121]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[dtd3]];\)\)], "Input",
  CellLabel->"In[122]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95432 .51428 m
.95365 .52854 L
s
.95432 .51428 m
.95455 .5 L
s
.95365 .52854 m
.95253 .54278 L
s
.95253 .54278 m
.95096 .55697 L
s
.95096 .55697 m
.94895 .57111 L
s
.94895 .57111 m
.94649 .58517 L
s
.94649 .58517 m
.9436 .59916 L
s
.9436 .59916 m
.94027 .61304 L
s
.94027 .61304 m
.9365 .62681 L
s
.9365 .62681 m
.9323 .64046 L
s
.9323 .64046 m
.92767 .65397 L
s
.92767 .65397 m
.92263 .66733 L
s
.92263 .66733 m
.91716 .68052 L
s
.91716 .68052 m
.91129 .69354 L
s
.91129 .69354 m
.905 .70636 L
s
.905 .70636 m
.89832 .71898 L
s
.89832 .71898 m
.89125 .73138 L
s
.89125 .73138 m
.88379 .74356 L
s
.88379 .74356 m
.87595 .75549 L
s
.87595 .75549 m
.86773 .76718 L
s
.86773 .76718 m
.85916 .77859 L
s
.85916 .77859 m
.85023 .78974 L
s
.85023 .78974 m
.84096 .8006 L
s
.84096 .8006 m
.83135 .81116 L
s
.83135 .81116 m
.82141 .82141 L
s
.82141 .82141 m
.81116 .83135 L
s
.81116 .83135 m
.8006 .84096 L
s
.8006 .84096 m
.78974 .85023 L
s
.78974 .85023 m
.77859 .85916 L
s
.77859 .85916 m
.76718 .86773 L
s
.76718 .86773 m
.75549 .87595 L
s
.75549 .87595 m
.74356 .88379 L
s
.74356 .88379 m
.73138 .89125 L
s
.73138 .89125 m
.71898 .89832 L
s
.71898 .89832 m
.70636 .905 L
s
.70636 .905 m
.69354 .91129 L
s
.69354 .91129 m
.68052 .91716 L
s
.68052 .91716 m
.66733 .92263 L
s
.68052 .91716 m
.9323 .35954 L
s
.66733 .92263 m
.65397 .92767 L
s
.65397 .92767 m
.64046 .9323 L
s
.64046 .9323 m
.62681 .9365 L
s
.62681 .9365 m
.61304 .94027 L
s
.61304 .94027 m
.59916 .9436 L
s
.59916 .9436 m
.58517 .94649 L
s
.59916 .9436 m
.47146 .04635 L
s
.58517 .94649 m
.57111 .94895 L
s
.57111 .94895 m
.55697 .95096 L
s
.55697 .95096 m
.54278 .95253 L
s
.54278 .95253 m
.52854 .95365 L
s
.52854 .95365 m
.51428 .95432 L
s
.51428 .95432 m
.5 .95455 L
s
.5 .95455 m
.48572 .95432 L
s
.48572 .95432 m
.47146 .95365 L
s
.47146 .95365 m
.45722 .95253 L
s
.45722 .95253 m
.44303 .95096 L
s
.45722 .95253 m
.5 .04545 L
s
.44303 .95096 m
.42889 .94895 L
s
.42889 .94895 m
.41483 .94649 L
s
.41483 .94649 m
.40084 .9436 L
s
.40084 .9436 m
.38696 .94027 L
s
.38696 .94027 m
.37319 .9365 L
s
.37319 .9365 m
.35954 .9323 L
s
.35954 .9323 m
.34603 .92767 L
s
.34603 .92767 m
.33267 .92263 L
s
.33267 .92263 m
.31948 .91716 L
s
.31948 .91716 m
.30646 .91129 L
s
.31948 .91716 m
.25644 .11621 L
s
.30646 .91129 m
.29364 .905 L
s
.29364 .905 m
.28102 .89832 L
s
.28102 .89832 m
.26862 .89125 L
s
.26862 .89125 m
.25644 .88379 L
s
.25644 .88379 m
.24451 .87595 L
s
.24451 .87595 m
.23282 .86773 L
s
.23282 .86773 m
.22141 .85916 L
s
.22141 .85916 m
.21026 .85023 L
s
.21026 .85023 m
.1994 .84096 L
s
.1994 .84096 m
.18884 .83135 L
s
.18884 .83135 m
.17859 .82141 L
s
.17859 .82141 m
.16865 .81116 L
s
.16865 .81116 m
.15904 .8006 L
s
.15904 .8006 m
.14977 .78974 L
s
.14977 .78974 m
.14084 .77859 L
s
.14084 .77859 m
.13227 .76718 L
s
.14084 .77859 m
.68052 .08284 L
s
.13227 .76718 m
.12405 .75549 L
s
.12405 .75549 m
.11621 .74356 L
s
.11621 .74356 m
.10875 .73138 L
s
.10875 .73138 m
.10168 .71898 L
s
.10168 .71898 m
.095 .70636 L
s
.095 .70636 m
.08871 .69354 L
s
.08871 .69354 m
.08284 .68052 L
s
.08284 .68052 m
.07737 .66733 L
s
.07737 .66733 m
.07233 .65397 L
s
.07233 .65397 m
.0677 .64046 L
s
.0677 .64046 m
.0635 .62681 L
s
.0635 .62681 m
.05973 .61304 L
s
.05973 .61304 m
.0564 .59916 L
s
.0564 .59916 m
.05351 .58517 L
s
.05351 .58517 m
.05105 .57111 L
s
.05105 .57111 m
.04904 .55697 L
s
.04904 .55697 m
.04747 .54278 L
s
.04747 .54278 m
.04635 .52854 L
s
.04635 .52854 m
.04568 .51428 L
s
.04568 .51428 m
.04545 .5 L
s
.04545 .5 m
.04568 .48572 L
s
.04568 .48572 m
.04635 .47146 L
s
.04635 .47146 m
.04747 .45722 L
s
.04747 .45722 m
.04904 .44303 L
s
.04904 .44303 m
.05105 .42889 L
s
.05105 .42889 m
.05351 .41483 L
s
.05351 .41483 m
.0564 .40084 L
s
.0564 .40084 m
.05973 .38696 L
s
.05973 .38696 m
.0635 .37319 L
s
.0635 .37319 m
.0677 .35954 L
s
.0677 .35954 m
.07233 .34603 L
s
.07233 .34603 m
.07737 .33267 L
s
.07737 .33267 m
.08284 .31948 L
s
.08284 .31948 m
.08871 .30646 L
s
.08871 .30646 m
.095 .29364 L
s
.095 .29364 m
.10168 .28102 L
s
.10168 .28102 m
.10875 .26862 L
s
.10875 .26862 m
.11621 .25644 L
s
.11621 .25644 m
.12405 .24451 L
s
.12405 .24451 m
.13227 .23282 L
s
.13227 .23282 m
.14084 .22141 L
s
.14084 .22141 m
.14977 .21026 L
s
.14977 .21026 m
.15904 .1994 L
s
.15904 .1994 m
.16865 .18884 L
s
.16865 .18884 m
.17859 .17859 L
s
.17859 .17859 m
.18884 .16865 L
s
.18884 .16865 m
.1994 .15904 L
s
.1994 .15904 m
.21026 .14977 L
s
.21026 .14977 m
.22141 .14084 L
s
.22141 .14084 m
.23282 .13227 L
s
.23282 .13227 m
.24451 .12405 L
s
.24451 .12405 m
.25644 .11621 L
s
.25644 .11621 m
.26862 .10875 L
s
.26862 .10875 m
.28102 .10168 L
s
.28102 .10168 m
.29364 .095 L
s
.29364 .095 m
.30646 .08871 L
s
.30646 .08871 m
.31948 .08284 L
s
.31948 .08284 m
.33267 .07737 L
s
.33267 .07737 m
.34603 .07233 L
s
.34603 .07233 m
.35954 .0677 L
s
.35954 .0677 m
.37319 .0635 L
s
.37319 .0635 m
.38696 .05973 L
s
.38696 .05973 m
.40084 .0564 L
s
.40084 .0564 m
.41483 .05351 L
s
.41483 .05351 m
.42889 .05105 L
s
.42889 .05105 m
.44303 .04904 L
s
.44303 .04904 m
.45722 .04747 L
s
.45722 .04747 m
.47146 .04635 L
s
.47146 .04635 m
.48572 .04568 L
s
.48572 .04568 m
.5 .04545 L
s
.5 .04545 m
.51428 .04568 L
s
.51428 .04568 m
.52854 .04635 L
s
.52854 .04635 m
.54278 .04747 L
s
.54278 .04747 m
.55697 .04904 L
s
.55697 .04904 m
.57111 .05105 L
s
.57111 .05105 m
.58517 .05351 L
s
.58517 .05351 m
.59916 .0564 L
s
.59916 .0564 m
.61304 .05973 L
s
.61304 .05973 m
.62681 .0635 L
s
.62681 .0635 m
.64046 .0677 L
s
.64046 .0677 m
.65397 .07233 L
s
.65397 .07233 m
.66733 .07737 L
s
.66733 .07737 m
.68052 .08284 L
s
.68052 .08284 m
.69354 .08871 L
s
.69354 .08871 m
.70636 .095 L
s
.70636 .095 m
.71898 .10168 L
s
.71898 .10168 m
.73138 .10875 L
s
.73138 .10875 m
.74356 .11621 L
s
.74356 .11621 m
.75549 .12405 L
s
.75549 .12405 m
.76718 .13227 L
s
.76718 .13227 m
.77859 .14084 L
s
.77859 .14084 m
.78974 .14977 L
s
.78974 .14977 m
.8006 .15904 L
s
.8006 .15904 m
.81116 .16865 L
s
.81116 .16865 m
.82141 .17859 L
s
.82141 .17859 m
.83135 .18884 L
s
.83135 .18884 m
.84096 .1994 L
s
.84096 .1994 m
.85023 .21026 L
s
.85023 .21026 m
.85916 .22141 L
s
.85916 .22141 m
.86773 .23282 L
s
.86773 .23282 m
.87595 .24451 L
s
.87595 .24451 m
.88379 .25644 L
s
.88379 .25644 m
.89125 .26862 L
s
.89125 .26862 m
.89832 .28102 L
s
.89832 .28102 m
.905 .29364 L
s
.905 .29364 m
.91129 .30646 L
s
.91129 .30646 m
.91716 .31948 L
s
.91716 .31948 m
.92263 .33267 L
s
.92263 .33267 m
.92767 .34603 L
s
.92767 .34603 m
.9323 .35954 L
s
.9323 .35954 m
.9365 .37319 L
s
.9365 .37319 m
.94027 .38696 L
s
.94027 .38696 m
.9436 .40084 L
s
.9436 .40084 m
.94649 .41483 L
s
.94649 .41483 m
.94895 .42889 L
s
.94895 .42889 m
.95096 .44303 L
s
.95096 .44303 m
.95253 .45722 L
s
.95253 .45722 m
.95365 .47146 L
s
.95365 .47146 m
.95432 .48572 L
s
.95432 .48572 m
.95455 .5 L
s
1 0 1 r
.025 w
.95432 .51428 Mdot
.95365 .52854 Mdot
.95253 .54278 Mdot
.95096 .55697 Mdot
.94895 .57111 Mdot
.94649 .58517 Mdot
.9436 .59916 Mdot
.94027 .61304 Mdot
.9365 .62681 Mdot
.9323 .64046 Mdot
.92767 .65397 Mdot
.92263 .66733 Mdot
.91716 .68052 Mdot
.91129 .69354 Mdot
.905 .70636 Mdot
.89832 .71898 Mdot
.89125 .73138 Mdot
.88379 .74356 Mdot
.87595 .75549 Mdot
.86773 .76718 Mdot
.85916 .77859 Mdot
.85023 .78974 Mdot
.84096 .8006 Mdot
.83135 .81116 Mdot
.82141 .82141 Mdot
.81116 .83135 Mdot
.8006 .84096 Mdot
.78974 .85023 Mdot
.77859 .85916 Mdot
.76718 .86773 Mdot
.75549 .87595 Mdot
.74356 .88379 Mdot
.73138 .89125 Mdot
.71898 .89832 Mdot
.70636 .905 Mdot
.69354 .91129 Mdot
.68052 .91716 Mdot
.66733 .92263 Mdot
.65397 .92767 Mdot
.64046 .9323 Mdot
.62681 .9365 Mdot
.61304 .94027 Mdot
.59916 .9436 Mdot
.58517 .94649 Mdot
.57111 .94895 Mdot
.55697 .95096 Mdot
.54278 .95253 Mdot
.52854 .95365 Mdot
.51428 .95432 Mdot
.5 .95455 Mdot
.48572 .95432 Mdot
.47146 .95365 Mdot
.45722 .95253 Mdot
.44303 .95096 Mdot
.42889 .94895 Mdot
.41483 .94649 Mdot
.40084 .9436 Mdot
.38696 .94027 Mdot
.37319 .9365 Mdot
.35954 .9323 Mdot
.34603 .92767 Mdot
0 1 1 r
.33267 .92263 Mdot
.31948 .91716 Mdot
.30646 .91129 Mdot
1 0 1 r
.29364 .905 Mdot
.28102 .89832 Mdot
.26862 .89125 Mdot
.25644 .88379 Mdot
.24451 .87595 Mdot
.23282 .86773 Mdot
.22141 .85916 Mdot
.21026 .85023 Mdot
.1994 .84096 Mdot
.18884 .83135 Mdot
.17859 .82141 Mdot
.16865 .81116 Mdot
.15904 .8006 Mdot
.14977 .78974 Mdot
.14084 .77859 Mdot
.13227 .76718 Mdot
.12405 .75549 Mdot
.11621 .74356 Mdot
.10875 .73138 Mdot
.10168 .71898 Mdot
.095 .70636 Mdot
.08871 .69354 Mdot
.08284 .68052 Mdot
.07737 .66733 Mdot
.07233 .65397 Mdot
.0677 .64046 Mdot
.0635 .62681 Mdot
.05973 .61304 Mdot
.0564 .59916 Mdot
.05351 .58517 Mdot
.05105 .57111 Mdot
.04904 .55697 Mdot
.04747 .54278 Mdot
.04635 .52854 Mdot
.04568 .51428 Mdot
.04545 .5 Mdot
.04568 .48572 Mdot
.04635 .47146 Mdot
.04747 .45722 Mdot
.04904 .44303 Mdot
.05105 .42889 Mdot
.05351 .41483 Mdot
.0564 .40084 Mdot
.05973 .38696 Mdot
.0635 .37319 Mdot
.0677 .35954 Mdot
.07233 .34603 Mdot
.07737 .33267 Mdot
.08284 .31948 Mdot
.08871 .30646 Mdot
.095 .29364 Mdot
.10168 .28102 Mdot
.10875 .26862 Mdot
.11621 .25644 Mdot
.12405 .24451 Mdot
.13227 .23282 Mdot
.14084 .22141 Mdot
.14977 .21026 Mdot
.15904 .1994 Mdot
.16865 .18884 Mdot
.17859 .17859 Mdot
.18884 .16865 Mdot
.1994 .15904 Mdot
.21026 .14977 Mdot
.22141 .14084 Mdot
.23282 .13227 Mdot
0 1 1 r
.24451 .12405 Mdot
.25644 .11621 Mdot
.26862 .10875 Mdot
1 0 1 r
.28102 .10168 Mdot
.29364 .095 Mdot
.30646 .08871 Mdot
.31948 .08284 Mdot
.33267 .07737 Mdot
.34603 .07233 Mdot
.35954 .0677 Mdot
.37319 .0635 Mdot
.38696 .05973 Mdot
.40084 .0564 Mdot
.41483 .05351 Mdot
.42889 .05105 Mdot
.44303 .04904 Mdot
.45722 .04747 Mdot
.47146 .04635 Mdot
.48572 .04568 Mdot
.5 .04545 Mdot
.51428 .04568 Mdot
.52854 .04635 Mdot
.54278 .04747 Mdot
.55697 .04904 Mdot
.57111 .05105 Mdot
.58517 .05351 Mdot
.59916 .0564 Mdot
.61304 .05973 Mdot
.62681 .0635 Mdot
.64046 .0677 Mdot
.65397 .07233 Mdot
.66733 .07737 Mdot
.68052 .08284 Mdot
.69354 .08871 Mdot
.70636 .095 Mdot
.71898 .10168 Mdot
.73138 .10875 Mdot
.74356 .11621 Mdot
.75549 .12405 Mdot
.76718 .13227 Mdot
.77859 .14084 Mdot
.78974 .14977 Mdot
.8006 .15904 Mdot
.81116 .16865 Mdot
.82141 .17859 Mdot
.83135 .18884 Mdot
.84096 .1994 Mdot
.85023 .21026 Mdot
.85916 .22141 Mdot
.86773 .23282 Mdot
.87595 .24451 Mdot
.88379 .25644 Mdot
.89125 .26862 Mdot
.89832 .28102 Mdot
.905 .29364 Mdot
.91129 .30646 Mdot
.91716 .31948 Mdot
.92263 .33267 Mdot
.92767 .34603 Mdot
.9323 .35954 Mdot
.9365 .37319 Mdot
.94027 .38696 Mdot
.9436 .40084 Mdot
.94649 .41483 Mdot
.94895 .42889 Mdot
.95096 .44303 Mdot
.95253 .45722 Mdot
.95365 .47146 Mdot
.95432 .48572 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[122]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00P_ooool3ool0o`05oooooooo0?ooo`3oool0ooooool00ooo0?l01Oooooooo`3oool0oooo
0?oooooo00?oo`3o00Goooooool0oooo0?ooo`3oooooo`03ool0oh?ooooo001jooooo`?oo`3o00?o
ooooool0oooo0?l08Ooo0?l01?ooooooo`3oool0oooo0?mkooooo`00MOooool3ool0o`03oooooooo
0?ooo`3o02koo`3oMoooool0077ooooo??oo0?mcooooo`00KOoooom4ool0ofoooooo001Zoooood_o
o`3oJoooool006Kooooo7ooo0?l01Oooooooo`3oool0oooo0?oooooo00?oo`3o00Goooooool0oooo
0?ooo`3oooooo`03ool0o`05oooooooo0?ooo`3oool0ooooool07ooo0?mWooooo`00H_oooolGool0
o`05oooooooo0?ooo`3oool0ooooool00ooo0?l6ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool3ool0o`03oooooooo0?ooo`3o01Woo`3oHoooool0067ooooo5ooo0?l@oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool05?oooolHool0of;ooooo001NoooooaKo
o`3o5?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01Sooooo5ooo0?mOooooo`00
F_oooolFool0oaSooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0LoooooaOoo`3o
Foooool005Wooooo5?oo0?lKooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool08?oo
oolDool0oe[ooooo001Foooooa?oo`3o7oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo02Cooooo4_oo0?mHooooo`00EOooool@ool0ob?ooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0Xooooo`ooo`3oEoooool005?ooooo3ooo0?lVooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool0:oooool?ool0oeCooooo001Booooo`ooo`3o:?ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo02gooooo3ooo0?mCooooo`00Coooool>ool0obcooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0`ooooo`03o`000?oo0?ooo`3o00coo`3oD?oo
ool004kooooo3?oo0?l_ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0;oooool0
1?l0003oooooooooooooool=ool0odoooooo001<ooooo`03o`3ooooo0?ooo`3o00[oo`3o<?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool00_ooool>
ool0odgooooo001;ooooo`;o0?oo2_oo0?lbooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool0;_ooool00ol0003oooooooooo`06ooooo`coo`3oC?ooool004Sooooo1Ol0ool9ool0oc?o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo00So
oooo3_oo0?m9ooooo`00Aoooool7o`3oo`Goo`3o=_ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo02cooooo00?o0000ooooooooool03?ooool<ool0odSooooo0015ooooo`[o0?oo0ooo
0?lhooooo`05o`000?ooooooooooooooool00000;Oooool00ol0003oooooooooo`0>ooooo`goo`3o
A_ooool004Cooooo3?l0ooljooooo`05o`000?ooooooooooooooool00000;Oooool00ol0003ooooo
ooooo`0@ooooo`coo`3oAOooool004?ooooo3?l0oolkooooo`05o`000?ooooooooooooooool00000
;?ooool00ol0003oooooooooo`0Booooo`coo`3oA?ooool0047ooooo0_oo0?l9o`3oockooooo00Go
0000ooooooooooooooooo`00000[ooooo`03o`000?oooooooooo01Kooooo2ooo0?m2ooooo`00@?oo
ool3ool0o`So0?oo?oooool01Ol0003ooooooooooooooooo000002[ooooo00?o0000ooooooooool0
6?ooool;ool0od7ooooo000nooooo`Koo`3o1Ol0ool00ol0003oooooooooo`0nooooo`05o`000?oo
ooooooooooooool00000:_ooool00ol0003oooooooooo`0Jooooo`_oo`3o?oooool003gooooo2?oo
0?l3o`3oo`03ooooool0003ooooo03oooooo00Go0000ooooooooooooooooo`00000Yooooo`03o`00
0?oooooooooo01cooooo2ooo0?lnooooo`00??ooool:ool0o`?ooooo00?o0000ooooooooool0?ooo
ool01?l0003oooooooooool0000Xooooo`03o`000?oooooooooo01oooooo2_oo0?lmooooo`00>ooo
ool:ool0o`Gooooo00?o0000ooooooooool0?_ooool01?l0003oooooooooool0000Wooooo`03o`00
0?oooooooooo027ooooo2_oo0?llooooo`00>_ooool:ool0o`Kooooo00?o0000ooooooooool0?_oo
ool01?l0003oooooooooool0000Vooooo`03o`000?oooooooooo02?ooooo2_oo0?lkooooo`00>?oo
ool:ool0o`Sooooo00?o0000ooooooooool0?_ooool01?l0003oooooooooool0000Vooooo`03o`00
0?oooooooooo02Gooooo2_oo0?liooooo`00=oooool:ool0o`Wooooo00?o0000ooooooooool0?_oo
ool01?l0003oooooooooool0000Uooooo`03o`000?oooooooooo02Oooooo2_oo0?lhooooo`00=_oo
ool:ool0o`[ooooo00?o0000ooooooooool0?_ooool01?l0003oooooooooool0000Tooooo`03o`00
0?oooooooooo02Wooooo2_oo0?lgooooo`00=Oooool:ool0o`_ooooo00?o0000ooooooooool0?_oo
ool01?l0003oooooooooool0000Sooooo`03o`000?oooooooooo02_ooooo2_oo0?lfooooo`00=?oo
ool:ool0o`cooooo00?o0000ooooooooool0?oooool2o`0002Cooooo00?o0000ooooooooool0;?oo
ool:ool0ocGooooo000cooooo`Woo`3o3_ooool00ol0003oooooooooo`0oooooo`;o00008oooool0
0ol0003oooooooooo`0_ooooo`Woo`3o=?ooool003;ooooo2Ooo0?l?ooooo`03o`000?oooooooooo
03oooooo0_l0000Rooooo`03o`000?oooooooooo037ooooo2Ooo0?lcooooo`00<Oooool9ool0oa3o
oooo00?o0000ooooooooool0?oooool2o`00027ooooo00?o0000ooooooooool0<oooool9ool0oc;o
oooo000`ooooo`Woo`3o4Oooool00ol0003oooooooooo`0oooooo`;o00008?ooool00ol0003ooooo
ooooo`0eooooo`Woo`3o<Oooool002oooooo2Ooo0?lBooooo`03o`000?oooooooooo03oooooo0_l0
000Pooooo`03o`000?oooooooooo03Kooooo2Ooo0?l`ooooo`00;_ooool9ool0oa?ooooo00?o0000
ooooooooool0?oooool2o`0001oooooo00?o0000ooooooooool0>?ooool9ool0oboooooo000]oooo
o`Woo`3o5Oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo01cooooo00?o0000oooo
ooooool0>_ooool9ool0obkooooo000/ooooo`Woo`3o5_ooool00ol0003oooooooooo`0oooooo`03
o`000?oooooooooo01_ooooo00?o0000ooooooooool0??ooool9ool0obgooooo000[ooooo`Woo`3o
5oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool0
?Oooool9ool0obcooooo000Zooooo`Woo`3o6?ooool00ol0003oooooooooo`0oooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool0?oooool9ool0ob_ooooo000Yooooo`Woo`3o6Oooool0
0ol0003oooooooooo`0oooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0@Oooool8
ool0ob_ooooo000Xooooo`Woo`3o6_ooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool0@oooool8ool0ob[ooooo000Wooooo`Woo`3o6oooool00ol0003o
ooooooooo`0oooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0AOooool8ool0obWo
oooo000Wooooo`Soo`3o7?ooool00ol0003oooooooooo`0oooooo`;o00006?ooool00ol0003ooooo
ooooo`16ooooo`Soo`3o:?ooool002Kooooo2?oo0?lMooooo`03o`000?oooooooooo03oooooo0_l0
000Gooooo`03o`000?oooooooooo04Oooooo2Ooo0?lWooooo`009Oooool8ool0oakooooo00?o0000
ooooooooool0?oooool2o`0001Kooooo00?o0000ooooooooool0BOooool9ool0obKooooo000Toooo
o`Soo`3o7oooool00ol0003oooooooooo`0oooooo`;o00005Oooool00ol0003oooooooooo`1;oooo
o`Soo`3o9_ooool002?ooooo2Ooo0?lOooooo`03o`000?oooooooooo03oooooo0_l0000Eooooo`03
o`000?oooooooooo04cooooo2?oo0?lUooooo`008_ooool9ool0ob3ooooo00?o0000ooooooooool0
?oooool2o`0001Cooooo00?o0000ooooooooool0C_ooool8ool0obCooooo000Rooooo`Soo`3o8_oo
ool00ol0003oooooooooo`0nooooo`;o00004oooool00ol0003oooooooooo`1@ooooo`Ooo`3o9?oo
ool002;ooooo1ooo0?lSooooo`03o`000?oooooooooo03kooooo0_l0000Booooo`03o`000?oooooo
oooo057ooooo2?oo0?lSooooo`008Oooool7ool0obCooooo00?o0000ooooooooool0?Oooool01?l0
003oooooooooool0000Aooooo`03o`000?oooooooooo05;ooooo2?oo0?lRooooo`008?ooool7ool0
obGooooo00?o0000ooooooooool0?Oooool01?l0003oooooooooool0000@ooooo`03o`000?oooooo
oooo05Cooooo2?oo0?lQooooo`008?ooool7ool0obGooooo00?o0000ooooooooool0?Oooool01?l0
003oooooooooool0000?ooooo`03o`000?oooooooooo05Kooooo1ooo0?lQooooo`007oooool8ool0
obGooooo00?o0000ooooooooool0?Oooool01?l0003oooooooooool0000>ooooo`03o`000?oooooo
oooo05Oooooo2?oo0?lPooooo`007_ooool8ool0obKooooo00?o0000ooooooooool0?Oooool01?l0
003oooooooooool0000=ooooo`03o`000?oooooooooo05Wooooo2?oo0?lOooooo`007_ooool7ool0
obOooooo00?o0000ooooooooool0?Oooool01?l0003oooooooooool0000=ooooo`03o`000?oooooo
oooo05[ooooo1ooo0?lOooooo`007_ooool7ool0obOooooo00?o0000ooooooooool0?Oooool01?l0
003oooooooooool0000<ooooo`03o`000?oooooooooo05_ooooo1ooo0?lOooooo`007Oooool7ool0
obSooooo00?o0000ooooooooool0?Oooool01Ol0003ooooooooooooooooo000000[ooooo00?o0000
ooooooooool0GOooool7ool0oakooooo000Looooo`Ooo`3o:Oooool00ol0003oooooooooo`0moooo
o`05o`000?ooooooooooooooool000002Oooool00ol0003oooooooooo`1Oooooo`Ooo`3o7Oooool0
01cooooo1ooo0?lYooooo`03o`000?oooooooooo03gooooo00Go0000ooooooooooooooooo`000009
ooooo`03o`000?oooooooooo05oooooo1ooo0?lMooooo`006oooool8ool0obWooooo00?o0000oooo
ooooool0?Oooool01Ol0003ooooooooooooooooo000000Sooooo00?o0000ooooooooool0H?ooool8
ool0oacooooo000Jooooo`Soo`3o:oooool00ol0003oooooooooo`0looooo`05o`000?oooooooooo
ooooool000001oooool00ol0003oooooooooo`1Rooooo`Soo`3o6oooool001[ooooo1ooo0?l/oooo
o`03o`000?oooooooooo03cooooo00Go0000ooooooooooooooooo`000006ooooo`03o`000?oooooo
oooo06Cooooo1ooo0?lKooooo`006_ooool7ool0obcooooo00?o0000ooooooooool0??ooool01Ol0
003ooooooooooooooooo000000Gooooo00?o0000ooooooooool0IOooool7ool0oa_ooooo000Ioooo
o`Ooo`3o;Oooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`1Vooooo`Ooo`3o6_ooool001Sooooo1ooo0?l^ooooo`03
o`000?oooooooooo03cooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo0000
06[ooooo1ooo0?lIooooo`006?ooool7ool0obkooooo00?o0000ooooooooool0??ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooooooooooo`0006_ooooo1ooo0?lIooooo`006?ooool7ool0obko
oooo00?o0000ooooooooool0??ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000006co
oooo1ooo0?lIooooo`005oooool7ool0oboooooo00?o0000ooooooooool0??ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooo000006gooooo1ooo0?lHooooo`005_ooool7ool0oc3ooooo00?o
0000ooooooooool0??ooool00ol0003oooooooooo`02ooooo`;o0000Koooool7ool0oaOooooo000F
ooooo`Ooo`3o<?ooool00ol0003oooooooooo`0looooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool0K_ooool7ool0oaOooooo000Eooooo`Soo`3o<?ooool00ol0003oooooooooo`0loooo
o`07o`000?ooooooooooooooool0003oooooo`00001_ooooo`Soo`3o5_ooool001Cooooo2?oo0?la
ooooo`03o`000?oooooooooo03_ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo06kooooo2?oo0?lEooooo`005?ooool7ool0oc?ooooo00?o0000ooooooooool0>_ooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool0Koooool7ool0oaGooooo000D
ooooo`Ooo`3o<oooool00ol0003oooooooooo`0jooooo`04o`000?ooooooooooo`0000?ooooo00?o
0000ooooooooool0Koooool7ool0oaGooooo000Dooooo`Koo`3o=?ooool00ol0003oooooooooo`0j
ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool0L?ooool6ool0oaGooooo000Coooo
o`Ooo`3o=?ooool00ol0003oooooooooo`0jooooo`;o00001Oooool00ol0003oooooooooo`1`oooo
o`Ooo`3o5?ooool001?ooooo1ooo0?ldooooo`03o`000?oooooooooo03[ooooo0_l00005ooooo`03
o`000?oooooooooo073ooooo1ooo0?lDooooo`004oooool7ool0ocCooooo00?o0000ooooooooool0
>_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo06oooooo1ooo0?lDooooo`004_oo
ool7ool0ocGooooo00?o0000ooooooooool0>Oooool2o`0000Oooooo00?o0000ooooooooool0L?oo
ool7ool0oa?ooooo000Aooooo`Ooo`3o=_ooool00ol0003oooooooooo`0hooooo`03o`000?oooooo
000000Oooooo00?o0000ooooooooool0LOooool7ool0oa;ooooo000Aooooo`Ooo`3o=_ooool00ol0
003oooooooooo`0gooooo`04o`000?ooooooooooo`0000Oooooo00?o0000ooooooooool0LOooool7
ool0oa;ooooo000Aooooo`Ooo`3o=_ooool00ol0003oooooooooo`0gooooo`04o`000?oooooooooo
o`0000Oooooo00?o0000ooooooooool0LOooool7ool0oa;ooooo000Aooooo`Koo`3o=oooool00ol0
003oooooooooo`0fooooo`05o`000?ooooooooooooooool000001oooool00ol0003oooooooooo`1b
ooooo`Koo`3o4_ooool0013ooooo1ooo0?lgooooo`03o`000?oooooooooo03Gooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo07;ooooo1ooo0?lAoooo
o`004?ooool7ool0ocOooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool0LOooool7ool0oa7ooooo000@ooooo`Ooo`3o
>?ooool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`1aooooo`Ooo`3o4Oooool0013ooooo1_oo0?liooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo07;ooooo1_oo0?lAooooo`003oooool7ool0ocWooooo00?o0000ooooooooool0<Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0L_ooool7
ool0oa3ooooo000?ooooo`Ooo`3o>Oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`1booooo`Ooo`3o4?ooool000oo
oooo1ooo0?liooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo07;ooooo1ooo0?l@ooooo`003oooool6ool0oc[ooooo
00?o0000ooooooooool0;oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool0L_ooool00ol0003oo`3oool0o`04ool0oa3ooooo000>ooooo`Ooo`3o>_oo
ool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`1`ooooo`03o`000?ooooooo`3o00Koo`3o3oooool000kooooo1ooo0?lj
ooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo073ooooo00?o0000oooooooo0?l01_oo0?l?ooooo`003_ooool7ool0
oc[ooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool0Koooool00ol0003oooooooooo`07ool0o`oooooo000>ooooo`Ko
o`3o>oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`1_ooooo`04o`000?ooooooooooooooo`Koo`3o3oooool000go
oooo1ooo0?lkooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool00_ooool7ool0o`ko
oooo000=ooooo`Ooo`3o>oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00_ooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo00;ooooo
1ooo0?l>ooooo`003Oooool7ool0oc_ooooo00?o0000ooooooooool0:Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`03ooooo`Ooo`3o3_ooool000gooooo1_oo0?lmooooo`03o`000?oooooooooo02Sooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool01?ooool6ool0o`kooooo000<ooooo`Ooo`3o?Oooool00ol0003oooooooooo`0Woooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1[oooo
o`03o`000?oooooooooo00Gooooo1ooo0?l=ooooo`003?ooool7ool0ocgooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool0Joooool00ol0003oooooooooo`05ooooo`Ooo`3o3Oooool000cooooo1ooo0?lmooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo06_ooooo00?o0000ooooooooool01Oooool7ool0o`gooooo000<ooooo`Koo`3o?_oo
ool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`1Zooooo`03o`000?oooooooooo00Oooooo1_oo0?l=ooooo`002oooool7
ool0ockooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool0J_ooool00ol0003oooooooooo`07ooooo`Ooo`3o3?ooool0
00_ooooo1ooo0?lnooooo`03o`000?oooooooooo02?ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo06Wooooo00?o0000ooooooooool02?ooool7ool0
o`cooooo000;ooooo`Ooo`3o?_ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo01;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`1Yooooo`03o`000?oooooooooo00So
oooo1ooo0?l<ooooo`003?ooool5ool0ocoooooo00?o0000ooooooooool08Oooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0Ioooool00ol0003ooooo
ooooo`0:ooooo`Goo`3o3Oooool000_ooooo1ooo0?lnooooo`03o`000?oooooooooo027ooooo00?o
0000ooooooooool04oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo06Oooooo00?o
0000ooooooooool02Oooool7ool0o`cooooo000;ooooo`Ooo`3o?_ooool00ol0003oooooooooo`0P
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`1V
ooooo`03o`000?oooooooooo00[ooooo1ooo0?l<ooooo`002oooool7ool0ockooooo00?o0000oooo
ooooool07oooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool0I_ooool00ol0003oooooooooo`0:ooooo`Ooo`3o3?ooool000_ooooo1_oo0?loooooo`03
o`000?oooooooooo01kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo06Kooooo00?o0000ooooooooool02oooool6ool0o`cooooo000:ooooo`Ooo`3o
@?ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0
2_ooool00ol0003oooooooooo`1Uooooo`03o`000?oooooooooo00cooooo1ooo0?l;ooooo`002_oo
ool7ool0od3ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0Gooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool0IOooool00ol0003oooooooooo`0<ooooo`Ooo`3o2ooo
ool000[ooooo1ooo0?m0ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0;ooooo`03o`000?oooooooooo06?ooooo00?o0000ooooooooool03Oooool7
ool0o`_ooooo000;ooooo`Goo`3o@Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Sooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`1Sooooo`03o`000?oooooooooo
00kooooo1Ooo0?l<ooooo`002_ooool7ool0od3ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0H_ooool00ol0003o
ooooooooo`0>ooooo`Ooo`3o2oooool000[ooooo1ooo0?m0ooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo06;ooooo
00?o0000ooooooooool03_ooool7ool0o`_ooooo000:ooooo`Ooo`3o@?ooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`1Qooooo`03o`000?oooooooooo00oooooo1ooo0?l;ooooo`002oooool5ool0od7ooooo00?o0000
ooooooooool05oooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool0HOooool00ol0003oooooooooo`0@ooooo`Goo`3o3?ooool000[ooooo1ooo0?m0oooo
o`03o`000?oooooooooo01Kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo063ooooo00?o0000ooooooooool04?ooool7ool0o`_ooooo000:ooooo`Oo
o`3o@?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01gooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`1Oooooo`03o`000?oooooooooo013ooooo1ooo0?l;ooooo`00
2_ooool7ool0od3ooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Mooooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool0Goooool00ol0003oooooooooo`0@ooooo`Ooo`3o
2oooool000[ooooo1_oo0?m1ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool07_oo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05kooooo00?o0000ooooooooool04_oo
ool5ool0o`cooooo0009ooooo`Ooo`3o@Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo01oooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`1Nooooo`03o`000?oooooo
oooo017ooooo1ooo0?l;ooooo`002Oooool7ool0od;ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0Pooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0GOooool00ol0
003oooooooooo`0Booooo`Ooo`3o2oooool000Wooooo1ooo0?m2ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo05go
oooo00?o0000ooooooooool04_ooool7ool0o`_ooooo000:ooooo`Koo`3o@_ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`1Looooo`03o`000?oooooooooo01Cooooo1Ooo0?l<ooooo`002_ooool7ool0od7ooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00kooooo00?o
0000ooooooooool0Foooool00ol0003oooooooooo`0Cooooo`Ooo`3o2oooool000[ooooo1ooo0?m1
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo05[ooooo00?o0000ooooooooool05?ooool7ool0o`_ooooo000:oooo
o`Ooo`3o@Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo02?ooooo00?o0000oooo
ooooool03_ooool00ol0003oooooooooo`1Jooooo`03o`000?oooooooooo01Cooooo1ooo0?l;oooo
o`002oooool5ool0od;ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Tooooo`03
o`000?oooooooooo00kooooo00?o0000ooooooooool0F_ooool00ol0003oooooooooo`0Eooooo`Go
o`3o3?ooool000[ooooo1ooo0?m1ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo05Wooooo00?o0000ooooooooool0
5Oooool7ool0o`_ooooo000:ooooo`Ooo`3o@Oooool00ol0003oooooooooo`0;ooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`1Iooooo`03o`000?oo
oooooooo01Gooooo1ooo0?l;ooooo`002_ooool7ool0od7ooooo00?o0000ooooooooool02_ooool0
0ol0003oooooooooo`0Wooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0F?ooool0
0ol0003oooooooooo`0Fooooo`Ooo`3o2oooool000_ooooo1Ooo0?m2ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo
05Oooooo00?o0000ooooooooool05oooool5ool0o`cooooo000:ooooo`Ooo`3o@Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo02Sooooo00?o0000ooooooooool03oooool00ol0003o
ooooooooo`1Fooooo`03o`000?oooooooooo01Oooooo1ooo0?l;ooooo`002_ooool7ool0od;ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool0E_ooool00ol0003oooooooooo`0Gooooo`Ooo`3o2oooool000[ooooo1ooo
0?m2ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0:Oooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool06?ooool7ool0o`_ooooo000;
ooooo`Koo`3o@_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo02Wooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`1Eooooo`03o`000?oooooooooo01Sooooo1_oo0?l<
ooooo`002oooool7ool0od7ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0Zoooo
o`03o`000?oooooooooo013ooooo00?o0000ooooooooool0E?ooool00ol0003oooooooooo`0Hoooo
o`Ooo`3o3?ooool000_ooooo1ooo0?m1ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool0:oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo05Cooooo00?o0000oooooooo
ool06?ooool7ool0o`cooooo000;ooooo`Ooo`3o@Oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo02cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1Cooooo`03o`00
0?oooooooooo01Sooooo1ooo0?l<ooooo`003?ooool5ool0od;ooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0D_oo
ool00ol0003oooooooooo`0Jooooo`Goo`3o3Oooool000_ooooo1ooo0?m1ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo05;ooooo00?o0000ooooooooool06Oooool7ool0o`cooooo000;ooooo`Ooo`3o@Oooool01Ol0
003ooooooooooooooooo0000033ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`1A
ooooo`03o`000?oooooooooo01[ooooo1ooo0?l<ooooo`002oooool7ool0od7ooooo00Co0000oooo
oooooooo0000<Oooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo057ooooo00?o0000
ooooooooool06_ooool7ool0o`cooooo000<ooooo`Goo`3o@_ooool00ol0003oooooo`00000boooo
o`03o`000?oooooooooo017ooooo00?o0000ooooooooool0D?ooool00ol0003oooooooooo`0Loooo
o`Goo`3o3Oooool000gooooo1Ooo0?m1ooooo`03o`000?oooooo000003;ooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo01_ooooo1Ooo0?l>ooooo`00
3?ooool7ool0od3ooooo0_l0000cooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
C_ooool00ol0003oooooooooo`0Kooooo`Ooo`3o3Oooool000cooooo1ooo0?m0ooooo`;o0000<ooo
ool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo04kooooo00?o0000ooooooooool06ooo
ool7ool0o`gooooo000<ooooo`Ooo`3o?oooool00ol0003oooooo`00000cooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool0C_ooool00ol0003oooooooooo`0Kooooo`Ooo`3o3Oooool0
00gooooo1_oo0?lnooooo`04o`000?ooooooooooo`0003?ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`1=ooooo`03o`000?oooooooooo01cooooo1_oo0?l>ooooo`003Oooool7ool0
ocgooooo00Co0000oooooooooooo0000<oooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo04gooooo00?o0000ooooooooool06oooool7ool0o`kooooo000=ooooo`Ooo`3o??ooool01Ol0
003ooooooooooooooooo000003?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`1<
ooooo`03o`000?oooooooooo01cooooo1ooo0?l>ooooo`003Oooool7ool0oc_ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0aooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool0C?ooool00ol0003oooooooooo`0Looooo`Ooo`3o3_ooool000kooooo1_oo0?ljooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool0<Oooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo04_ooooo00?o0000ooooooooool07Oooool6ool0o`oooooo000>ooooo`Ooo`3o
>Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo037ooooo00?o0000ooooooooool0
4oooool00ol0003oooooooooo`1:ooooo`03o`000?oooooooooo01cooooo1ooo0?l?ooooo`003_oo
ool7ool0ocSooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool0BOooool00ol0003oooooooooo`0Mooooo`Ooo`3o3ooo
ool000kooooo1ooo0?lgooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0<?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool07Oooool7
ool0o`oooooo000?ooooo`Ooo`3o=Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
033ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo
01gooooo1ooo0?l@ooooo`003oooool7ool0ocCooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0`ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0B?ooool00ol0003o
ooooooooo`0Mooooo`Ooo`3o4?ooool000oooooo1ooo0?ldooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo04Sooooo
00?o0000ooooooooool07Oooool7ool0oa3ooooo000@ooooo`Koo`3o<oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`17ooooo`03o`000?oooooooooo01kooooo1_oo0?lAooooo`004?ooool7ool0oc7ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool0A_ooool00ol0003oooooooooo`0Mooooo`Ooo`3o4Oooool0013ooooo1ooo0?l`oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo04Gooooo00?o0000ooooooooool07_ooool7ool0oa7ooooo000@ooooo`Oo
o`3o<?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo02oooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo01kooooo1ooo0?lAooooo`00
4Oooool6ool0oboooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0_ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`0Oooooo`Koo`3o
4_ooool0017ooooo1ooo0?l]ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0;ooo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo04Cooooo00?o0000ooooooooool07_oo
ool7ool0oa;ooooo000Aooooo`Ooo`3o;?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooo
oooo02oooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooo
oooo01oooooo1ooo0?lBooooo`004Oooool7ool0obcooooo00?o0000ooooooooool03_ooool00ol0
003oooooooooo`0_ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0@oooool00ol0
003oooooooooo`0Oooooo`Ooo`3o4_ooool001;ooooo1ooo0?lZooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo04;o
oooo00?o0000ooooooooool07_ooool7ool0oa?ooooo000Cooooo`Ooo`3o:?ooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`11ooooo`03o`000?oooooooooo01kooooo1ooo0?lDooooo`004oooool7ool0obOooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01Kooooo00?o
0000ooooooooool0@Oooool00ol0003oooooooooo`0Nooooo`Ooo`3o5?ooool001?ooooo1ooo0?lV
ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool07oooool7ool0oaCooooo000Doooo
o`Koo`3o9_ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo02oooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo01oooooo1_oo0?lEoooo
o`005?ooool7ool0obCooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0^ooooo`03
o`000?oooooooooo01Kooooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`0Oooooo`Oo
o`3o5Oooool001Cooooo1ooo0?lSooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0
;_ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0
7oooool7ool0oaGooooo000Dooooo`Ooo`3o8_ooool00ol0003oooooooooo`0Fooooo`03o`000?oo
oooooooo02kooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0mooooo`03o`000?oo
oooooooo023ooooo1ooo0?lEooooo`005Oooool7ool0ob7ooooo00?o0000ooooooooool05_ooool0
0ol0003oooooooooo`0^ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0?Oooool0
0ol0003oooooooooo`0Oooooo`Ooo`3o5_ooool001Kooooo1ooo0?lOooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo
03cooooo00?o0000ooooooooool07oooool7ool0oaOooooo000Fooooo`Ooo`3o7_ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0looooo`03o`000?oooooooooo01oooooo1ooo0?lGooooo`005_ooool8ool0oacooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo01Sooooo
00?o0000ooooooooool0??ooool00ol0003oooooooooo`0Nooooo`Soo`3o5oooool001Oooooo2?oo
0?lJooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0;Oooool00ol0003ooooooooo
o`0Hooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool07_ooool8ool0oaSooooo000H
ooooo`Ooo`3o6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo02gooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`0kooooo`03o`000?oooooooooo01kooooo1ooo0?lI
ooooo`006?ooool7ool0oaWooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`0]oooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`0Ooooo
o`Ooo`3o6Oooool001Wooooo1ooo0?lGooooo`03o`000?oooooooooo01cooooo00?o0000oooooooo
ool0;Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo03Wooooo00?o0000oooooooo
ool07_ooool7ool0oa[ooooo000Jooooo`Ooo`3o5Oooool00ol0003oooooooooo`0Mooooo`03o`00
0?oooooooooo02gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0hooooo`03o`00
0?oooooooooo01kooooo1ooo0?lKooooo`006_ooool7ool0oaGooooo00?o0000ooooooooool07_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0>?oo
ool00ol0003oooooooooo`0Nooooo`Ooo`3o6oooool001[ooooo2?oo0?lCooooo`03o`000?oooooo
oooo01oooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo03Oooooo00?o0000ooooooooool07_ooool8ool0oa_ooooo000Kooooo`Soo`3o4Oooool00ol0
003oooooooooo`0Pooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0gooooo`03o`000?oooooooooo01gooooo2?oo0?lLooooo`007?ooool7ool0oa3o
oooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool0=oooool00ol0003oooooooooo`0Mooooo`Ooo`3o7Oooool001cooooo
1ooo0?l?ooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo03Gooooo00?o0000ooooooooool07_ooool7ool0oagooooo
000Mooooo`Ooo`3o3_ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo02cooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo01gooooo1ooo
0?lNooooo`007_ooool7ool0o`cooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`0/
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`0M
ooooo`Ooo`3o7oooool001kooooo1ooo0?l;ooooo`03o`000?oooooooooo02Cooooo00?o0000oooo
ooooool0;?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo03Cooooo00?o0000oooo
ooooool07Oooool7ool0oaoooooo000Nooooo`Soo`3o2Oooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo02cooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0cooooo`03
o`000?oooooooooo01gooooo2?oo0?lOooooo`007oooool8ool0o`Sooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
<oooool00ol0003oooooooooo`0Looooo`Soo`3o8?ooool0023ooooo1ooo0?l7ooooo`03o`000?oo
oooooooo02Kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo03;ooooo00?o0000ooooooooool07Oooool7ool0ob7ooooo000Pooooo`Ooo`3o1_ooool0
0ol0003oooooooooo`0Wooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06oooool0
0ol0003oooooooooo`0aooooo`03o`000?oooooooooo01cooooo2?oo0?lQooooo`008Oooool7ool0
o`Cooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo
01_ooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Looooo`Soo`3o8_ooool002;o
oooo1ooo0?l2ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0:oooool00ol0003o
ooooooooo`0Kooooo`03o`000?oooooooooo033ooooo00?o0000ooooooooool06oooool8ool0ob?o
oooo000Rooooo`Soo`3o00?oooooo`000?ooool0:oooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo01cooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`0Kooooo`Ooo`3o
9?ooool002;ooooo2Ooo0?l]ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool07?oo
ool00ol0003oooooooooo`0_ooooo`03o`000?oooooooooo01_ooooo2?oo0?lTooooo`008oooool9
ool0obcooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooo
oooo02oooooo00?o0000ooooooooool06_ooool8ool0obGooooo000Tooooo`Soo`3o;?ooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0;_ooool00ol0
003oooooooooo`0Jooooo`Soo`3o9_ooool002Gooooo2?oo0?l[ooooo`03o`000?oooooooooo02[o
oooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo01Wo
oooo2Ooo0?lVooooo`009_ooool8ool0ob[ooooo00?o0000ooooooooool0:_ooool00ol0003ooooo
ooooo`0Mooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool06Oooool9ool0obOooooo
000Wooooo`Soo`3o:Oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo01gooooo00?o
0000ooooooooool0;?ooool00ol0003oooooooooo`0Iooooo`Soo`3o:?ooool002Oooooo2Ooo0?lX
ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool07Oooool00ol0003oooooooooo`0[
ooooo`03o`000?oooooooooo01Wooooo2?oo0?lYooooo`00:?ooool9ool0obOooooo00?o0000oooo
ooooool0:_ooool00ol0003oooooooooo`0Mooooo`03o`000?oooooooooo02_ooooo00?o0000oooo
ooooool06?ooool8ool0ob[ooooo000Yooooo`Woo`3o9_ooool00ol0003oooooooooo`0Zooooo`03
o`000?oooooooooo01gooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`0Gooooo`So
o`3o:oooool002[ooooo2Ooo0?lUooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0
7Oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo01Oooooo2Ooo0?l[ooooo`00:ooo
ool9ool0obCooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`0Mooooo`03o`000?oo
oooooooo02[ooooo00?o0000ooooooooool05_ooool9ool0obcooooo000/ooooo`Woo`3o9?ooool0
0ol0003oooooooooo`0Yooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0:?ooool0
0ol0003oooooooooo`0Fooooo`Woo`3o;Oooool002gooooo2Ooo0?lSooooo`03o`000?oooooooooo
02Wooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
01Gooooo2Ooo0?l^ooooo`00;_ooool9ool0ob;ooooo00?o0000ooooooooool0:Oooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool05Oooool9ool0oboo
oooo000_ooooo`Woo`3o8Oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo01kooooo
00?o0000ooooooooool09oooool00ol0003oooooooooo`0Dooooo`Woo`3o<?ooool0033ooooo2Ooo
0?lPooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool07_ooool00ol0003ooooooooo
o`0Vooooo`03o`000?oooooooooo01Cooooo2Ooo0?laooooo`00<Oooool9ool0oaoooooo00?o0000
ooooooooool0:Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool04oooool9ool0oc;ooooo000booooo`Woo`3o7_ooool00ol0003oooooooooo`0Yoooo
o`03o`000?oooooooooo01kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Coooo
o`Woo`3o<oooool003?ooooo2Ooo0?lMooooo`03o`000?oooooooooo02Wooooo00?o0000oooooooo
ool07oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo01;ooooo2Ooo0?ldooooo`00
=?ooool9ool0oacooooo00?o0000ooooooooool0:Oooool00ol0003oooooooooo`0Oooooo`03o`00
0?oooooooooo02Cooooo00?o0000ooooooooool04Oooool9ool0ocGooooo000eooooo`Woo`3o6ooo
ool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo023ooooo00?o0000ooooooooool08ooo
ool00ol0003oooooooooo`0Aooooo`Woo`3o=_ooool003Kooooo2Ooo0?lJooooo`03o`000?oooooo
oooo02Sooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0Sooooo`03o`000?oooooo
oooo013ooooo2Ooo0?lgooooo`00=oooool:ool0oaSooooo00?o0000ooooooooool0:?ooool00ol0
003oooooooooo`0Pooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool03oooool:ool0
ocSooooo000hooooo`[oo`3o5oooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo023o
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0>ooooo`[oo`3o>Oooool003Wooooo
2_oo0?lGooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool08?ooool00ol0003ooooo
ooooo`0Qooooo`03o`000?oooooooooo00kooooo2_oo0?ljooooo`00>_ooool:ool0oaKooooo00?o
0000ooooooooool09oooool00ol0003oooooooooo`0Qooooo`03o`000?oooooooooo023ooooo00?o
0000ooooooooool03Oooool:ool0oc_ooooo000kooooo`[oo`3o5Oooool00ol0003oooooooooo`0W
ooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0=
ooooo`[oo`3o??ooool003gooooo2ooo0?lBooooo`03o`000?oooooooooo02Oooooo00?o0000oooo
ooooool08Oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo00[ooooo2ooo0?lnoooo
o`00?_ooool;ool0oa7ooooo00?o0000ooooooooool09oooool00ol0003oooooooooo`0Qooooo`03
o`000?oooooooooo01oooooo00?o0000ooooooooool02Oooool;ool0ocoooooo000oooooo`coo`3o
3oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo027ooooo00?o0000ooooooooool0
7_ooool00ol0003oooooooooo`08ooooo`coo`3o@?ooool0043ooooo3?oo0?l>ooooo`03o`000?oo
oooooooo02Oooooo00?o0000ooooooooool08Oooool00ol0003oooooooooo`0Nooooo`03o`000?oo
oooooooo00Oooooo3?oo0?m1ooooo`00@Oooool>ool0o`_ooooo00?o0000ooooooooool09oooool0
0ol0003oooooooooo`0Qooooo`03o`000?oooooooooo01gooooo00?o0000ooooooooool01Oooool>
ool0od;ooooo0014ooooo`coo`3o2_ooool00ol0003oooooooooo`0Wooooo`03o`000?oooooooooo
02;ooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`04ooooo`coo`3oAOooool004Go
oooo3Ooo0?l8ooooo`03o`000?oooooooooo02Oooooo00?o0000ooooooooool08_ooool00ol0003o
ooooooooo`0Kooooo`03o`000?oooooooooo00?ooooo3Ooo0?m6ooooo`00Aoooool<ool0o`Oooooo
00?o0000ooooooooool09oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo01_ooooo
00?o0000ooooooooool00_ooool<ool0odSooooo0018ooooo`koo`3o1?ooool00ol0003ooooooooo
o`0Wooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0>ool0odWooooo001;ooooo`coo`3o0oooool00ol0003oooooooooo`0Wooooo`03o`000?oooooo
oooo02;ooooo00?o0000ooooooooool06_ooool00ol0003oooooool0o`0;ool0odcooooo001<oooo
o`coo`3o00Co0?oooooooooooooo0000:?ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooo
oooo01Wooooo3_oo0?m=ooooo`00C_ooool:ool0o`;o0?oo00?oooooo`000?ooool09oooool00ol0
003oooooooooo`0Rooooo`03o`000?oooooooooo01Sooooo3Ooo0?m?ooooo`00Coooool9ool0o`Go
0?oo9oooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo01Gooooo3_oo0?m@ooooo`00
D_ooool5ool0o`[o0?oo8oooool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo017ooooo
3ooo0?mCooooo`00Doooool3ool0o`co0?oo8_ooool00ol0003oooooooooo`0Sooooo`03o`000?oo
oooooooo013ooooo3ooo0?mDooooo`00EOooool>o`3oo`;oo`3o7_ooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo00gooooo3ooo0?mGooooo`00E_ooool=o`3oo`Koo`3o6_ooool00ol0
003oooooooooo`0Tooooo`03o`000?oooooooooo00Wooooo4_oo0?mHooooo`00FOooool:o`3oo`[o
o`3o5_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00Gooooo5?oo0?mJooooo`00
F_ooool8o`3oo`koo`3o4oooool00ol0003oooooooooo`0Tooooo`04o`000?oooooooooooooooaOo
o`3oFoooool005kooooo0ol0oolCool0o`oooooo00?o0000ooooooooool09?oooolGool0oeoooooo
001QoooooaOoo`3o2oooool00ol0003oooooooooo`0PoooooaSoo`3oH_ooool006;ooooo5ooo0?l0
1Oooooooo`3oool0oooo0?oooooo00?oo`3o0_ooool00ol0003oooooooooo`0Hooooo`?oo`3o00?o
ooooool0oooo0?l06Ooo0?mSooooo`00I_oooolOool0o`05oooooooo0?ooo`3oool0ooooool00ooo
0?l5ooooo`?oo`3o00Goooooool0oooo0?ooo`3oooooo`0Oool0ofOooooo001Zoooood_oo`3oJooo
ool006goooooA?oo0?m_ooooo`00LOoooollool0og?ooooo001eooooo`?oo`3o00?oooooool0oooo
0?l0;_oo0?mgooooo`00N_ooool3ool0o`03oooooooo0?ooo`3o027oo`3o00Coooooool0oooo0?oo
o`3oNoooool008;ooooo0ooo0?l01Oooooooo`3oool0oooo0?oooooo00_oo`3o00Goooooool0oooo
0?ooo`3oooooo`03ool0oh?ooooo002>ooooo`?oo`3oSoooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[dtd3]\)], "Input",
  CellLabel->"In[123]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["von Neumann", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(vnDTD = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] VonNeumann[10, 10], \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[124]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[
        ExtractGraph[vnDTD], \[IndentingNewLine]VertexStyle \[Rule] 
          Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[125]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.498 .498 .498 r
.14646 .04545 Mdot
0 0 .498 r
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 0 r
.65152 .04545 Mdot
1 1 0 r
.75253 .04545 Mdot
0 1 0 r
.85354 .04545 Mdot
0 1 1 r
.95455 .04545 Mdot
.898 .898 .898 r
.04545 .14646 Mdot
.498 .498 .498 r
.14646 .14646 Mdot
1 0 0 r
.24747 .14646 Mdot
.34848 .14646 Mdot
1 1 0 r
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
0 1 0 r
.65152 .14646 Mdot
1 0 1 r
.75253 .14646 Mdot
.498 0 0 r
.85354 .14646 Mdot
1 1 0 r
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
0 .498 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
.898 .898 .898 r
.75253 .24747 Mdot
1 1 0 r
.85354 .24747 Mdot
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
0 1 0 r
.14646 .34848 Mdot
0 0 .498 r
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.498 0 0 r
.55051 .34848 Mdot
0 0 .498 r
.65152 .34848 Mdot
.498 0 0 r
.75253 .34848 Mdot
.498 .498 .498 r
.85354 .34848 Mdot
.898 .898 .898 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
0 0 .498 r
.14646 .44949 Mdot
.24747 .44949 Mdot
0 .498 0 r
.34848 .44949 Mdot
1 1 0 r
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
0 0 .498 r
.65152 .44949 Mdot
1 0 0 r
.75253 .44949 Mdot
1 0 1 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
0 .498 0 r
.14646 .55051 Mdot
.898 .898 .898 r
.24747 .55051 Mdot
0 0 .498 r
.34848 .55051 Mdot
1 0 0 r
.44949 .55051 Mdot
0 .498 0 r
.55051 .55051 Mdot
0 1 0 r
.65152 .55051 Mdot
.898 .898 .898 r
.75253 .55051 Mdot
.85354 .55051 Mdot
1 0 0 r
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
0 1 0 r
.14646 .65152 Mdot
.498 .498 .498 r
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.498 .498 .498 r
.65152 .65152 Mdot
0 0 .498 r
.75253 .65152 Mdot
.898 .898 .898 r
.85354 .65152 Mdot
1 0 0 r
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.498 .498 .498 r
.14646 .75253 Mdot
0 0 .498 r
.24747 .75253 Mdot
0 1 0 r
.34848 .75253 Mdot
1 0 0 r
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
1 0 1 r
.65152 .75253 Mdot
0 .498 0 r
.75253 .75253 Mdot
1 1 0 r
.85354 .75253 Mdot
1 0 0 r
.95455 .75253 Mdot
1 1 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
0 1 0 r
.44949 .85354 Mdot
0 0 .498 r
.55051 .85354 Mdot
.498 .498 .498 r
.65152 .85354 Mdot
1 0 1 r
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
1 1 0 r
.95455 .85354 Mdot
.498 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.498 .498 .498 r
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
0 1 0 r
.44949 .95455 Mdot
1 0 0 r
.55051 .95455 Mdot
.498 0 0 r
.65152 .95455 Mdot
0 1 1 r
.75253 .95455 Mdot
1 0 0 r
.85354 .95455 Mdot
0 .498 0 r
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[125]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`002_ooool6o`3ooaOooooo1_mo
OglGooooo`Ko001o5oooool6o`00OaOooooo1_l0oolGooooo`Koool05oooool6o`3o01Oooooo1_oo
o`0Gooooo`Ko0?l05oooool6o`3oo`_ooooo0008ooooo`[o0?oo4oooool:ogmoOa?ooooo2_l007lC
ooooo`[o001o4oooool:o`3ooa?ooooo2_ooo`0Cooooo`[o0?l04oooool:oooo01?ooooo2_l0o`0C
ooooo`[o0?oo2Oooool000Kooooo3_l0ool?ooooo`koOgmo3oooool>o`00O`oooooo3_l007l?oooo
o`ko0?oo3oooool>oooo00oooooo3_l0o`0?ooooo`koool03oooool>o`3o00oooooo3_l0ool7oooo
o`001Oooool@o`3oo`gooooo4?moOgl=oooooa3o001o3Oooool@o`00O`gooooo4?l0ool=oooooa3o
ool03Oooool@o`3o00gooooo4?ooo`0=oooooa3o0?l03Oooool@o`3oo`Kooooo0005oooooa3o0?oo
3Oooool@ogmoO`gooooo4?l007l=oooooa3o001o3Oooool@o`3oo`gooooo4?ooo`0=oooooa3o0?l0
3Oooool@oooo00gooooo4?l0o`0=oooooa3o0?oo1_ooool000Cooooo4_l0ool;oooooa;oOgmo2ooo
oolBo`00O`_ooooo4_l007l;oooooa;o0?oo2ooooolBoooo00_ooooo4_l0o`0;oooooa;oool02ooo
oolBo`3o00_ooooo4_l0ool5ooooo`001?oooolBo`3oo`_ooooo4_moOgl;oooooa;o001o2ooooolB
o`00O`_ooooo4_l0ool;oooooa;oool02ooooolBo`3o00_ooooo4_ooo`0;oooooa;o0?l02ooooolB
o`3oo`Gooooo0003oooooaCo0?oo2OoooolDogmoO`Wooooo5?l007l9oooooaCo001o2OoooolDo`3o
o`Wooooo5?ooo`09oooooaCo0?l02OoooolDoooo00Wooooo5?l0o`09oooooaCo0?oo1?ooool000?o
oooo5?l0ool9oooooaCoOgmo2OoooolDo`00O`Wooooo5?l007l9oooooaCo0?oo2OoooolDoooo00Wo
oooo5?l0o`09oooooaCoool02OoooolDo`3o00Wooooo5?l0ool4ooooo`000ooooolDo`3oo`Wooooo
5?moOgl9oooooaCo001o2OoooolDo`00O`Wooooo5?l0ool9oooooaCoool02OoooolDo`3o00Wooooo
5?ooo`09oooooaCo0?l02OoooolDo`3oo`Cooooo0003oooooaCo0?oo2Ol0000DogmoO`Wo00005?l0
07l9o`0001Co001o2Ol0000Do`3oo`Wo00005?ooo`09o`0001Co0?l02Ol0000Doooo00Wo00005?l0
o`09o`0001Co0?oo1?ooool000?ooooo5?l0ool9oooooaCoOgmo2OoooolDo`00O`Wooooo5?l007l9
oooooaCo0?oo2OoooolDoooo00Wooooo5?l0o`09oooooaCoool02OoooolDo`3o00Wooooo5?l0ool4
ooooo`000ooooolDo`3oo`Wooooo5?moOgl9oooooaCo001o2OoooolDo`00O`Wooooo5?l0ool9oooo
oaCoool02OoooolDo`3o00Wooooo5?ooo`09oooooaCo0?l02OoooolDo`3oo`Cooooo0004oooooa;o
0?oo2ooooolBogmoO`_ooooo4_l007l;oooooa;o001o2ooooolBo`3oo`_ooooo4_ooo`0;oooooa;o
0?l02ooooolBoooo00_ooooo4_l0o`0;oooooa;o0?oo1Oooool000Cooooo4_l0ool;oooooa;oOgmo
2ooooolBo`00O`_ooooo4_l007l;oooooa;o0?oo2ooooolBoooo00_ooooo4_l0o`0;oooooa;oool0
2ooooolBo`3o00_ooooo4_l0ool5ooooo`001Oooool@o`3oo`gooooo4?moOgl=oooooa3o001o3Ooo
ool@o`00O`gooooo4?l0ool=oooooa3oool03Oooool@o`3o00gooooo4?ooo`0=oooooa3o0?l03Ooo
ool@o`3oo`Kooooo0005oooooa3o0?oo3Oooool@ogmoO`gooooo4?l007l=oooooa3o001o3Oooool@
o`3oo`gooooo4?ooo`0=oooooa3o0?l03Oooool@oooo00gooooo4?l0o`0=oooooa3o0?oo1_ooool0
00Kooooo3_l0ool?ooooo`koOgmo3oooool>o`00O`oooooo3_l007l?ooooo`ko0?oo3oooool>oooo
00oooooo3_l0o`0?ooooo`koool03oooool>o`3o00oooooo3_l0ool7ooooo`002?ooool:o`3ooa?o
oooo2_moOglCooooo`[o001o4oooool:o`00Oa?ooooo2_l0oolCooooo`[oool04oooool:o`3o01?o
oooo2_ooo`0Cooooo`[o0?l04oooool:o`3oo`Wooooo000:ooooo`Ko0?oo5oooool6ogmoOaOooooo
1_l007lGooooo`Ko001o5oooool6o`3ooaOooooo1_ooo`0Gooooo`Ko0?l05oooool6oooo01Oooooo
1_l0o`0Gooooo`Ko0?oo2oooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000[ooooo1_oUiNDGooooo`KoOgmo5oooool6ool001Oooooo
1_oo000Gooooo`Koool05oooool6o`3ooaOooooo1_l0o`0Gooooo`Koo`3o5oooool6ogl001Oooooo
1_ooo`0;ooooo`002?ooool:onGUiA?ooooo2_moOglCooooo`[oo`004oooool:ool001?ooooo2_oo
o`0Cooooo`[o0?oo4oooool:o`3o01?ooooo2_oo0?lCooooo`[oO`004oooool:oooo00Wooooo0006
ooooo`koiNGU3oooool>ogmoO`oooooo3_oo000?ooooo`koo`003oooool>oooo00oooooo3_l0ool?
ooooo`ko0?l03oooool>ool0o`oooooo3_mo000?ooooo`koool01oooool000Gooooo4?oUiND=oooo
oa3oOgmo3Oooool@ool000gooooo4?oo000=oooooa3oool03Oooool@o`3oo`gooooo4?l0o`0=oooo
oa3oo`3o3Oooool@ogl000gooooo4?ooo`06ooooo`001Oooool@onGUi@gooooo4?moOgl=oooooa3o
o`003Oooool@ool000gooooo4?ooo`0=oooooa3o0?oo3Oooool@o`3o00gooooo4?oo0?l=oooooa3o
O`003Oooool@oooo00Kooooo0004oooooa;oiNGU2ooooolBogmoO`_ooooo4_oo000;oooooa;oo`00
2ooooolBoooo00_ooooo4_l0ool;oooooa;o0?l02ooooolBool0o`_ooooo4_mo000;oooooa;oool0
1Oooool000Cooooo4_oUiND;oooooa;oOgmo2ooooolBool000_ooooo4_oo000;oooooa;oool02ooo
oolBo`3oo`_ooooo4_l0o`0;oooooa;oo`3o2ooooolBogl000_ooooo4_ooo`05ooooo`000ooooolD
onGUi@Wooooo5?moOgl9oooooaCoo`002OoooolDool000Wooooo5?ooo`09oooooaCo0?oo2OoooolD
o`3o00Wooooo5?oo0?l9oooooaCoO`002OoooolDoooo00Cooooo0003oooooaCoiNGU2OoooolDogmo
O`Wooooo5?oo0009oooooaCoo`002OoooolDoooo00Wooooo5?l0ool9oooooaCo0?l02OoooolDool0
o`Wooooo5?mo0009oooooaCoool01?ooool000?ooooo5?oUiND9oooooaCoOgmo2OoooolDool000Wo
oooo5?oo0009oooooaCoool02OoooolDo`3oo`Wooooo5?l0o`09oooooaCoo`3o2OoooolDogl000Wo
oooo5?ooo`04ooooo`000ooooolDonGUi@Wo00005?moOgl9o`0001Coo`002Ol0000Dool000Wo0000
5?ooo`09o`0001Co0?oo2Ol0000Do`3o00Wo00005?oo0?l9o`0001CoO`002Ol0000Doooo00Cooooo
0003oooooaCoiNGU2OoooolDogmoO`Wooooo5?oo0009oooooaCoo`002OoooolDoooo00Wooooo5?l0
ool9oooooaCo0?l02OoooolDool0o`Wooooo5?mo0009oooooaCoool01?ooool000?ooooo5?oUiND9
oooooaCoOgmo2OoooolDool000Wooooo5?oo0009oooooaCoool02OoooolDo`3oo`Wooooo5?l0o`09
oooooaCoo`3o2OoooolDogl000Wooooo5?ooo`04ooooo`001?oooolBonGUi@_ooooo4_moOgl;oooo
oa;oo`002ooooolBool000_ooooo4_ooo`0;oooooa;o0?oo2ooooolBo`3o00_ooooo4_oo0?l;oooo
oa;oO`002ooooolBoooo00Gooooo0004oooooa;oiNGU2ooooolBogmoO`_ooooo4_oo000;oooooa;o
o`002ooooolBoooo00_ooooo4_l0ool;oooooa;o0?l02ooooolBool0o`_ooooo4_mo000;oooooa;o
ool01Oooool000Gooooo4?oUiND=oooooa3oOgmo3Oooool@ool000gooooo4?oo000=oooooa3oool0
3Oooool@o`3oo`gooooo4?l0o`0=oooooa3oo`3o3Oooool@ogl000gooooo4?ooo`06ooooo`001Ooo
ool@onGUi@gooooo4?moOgl=oooooa3oo`003Oooool@ool000gooooo4?ooo`0=oooooa3o0?oo3Ooo
ool@o`3o00gooooo4?oo0?l=oooooa3oO`003Oooool@oooo00Kooooo0006ooooo`koiNGU3oooool>
ogmoO`oooooo3_oo000?ooooo`koo`003oooool>oooo00oooooo3_l0ool?ooooo`ko0?l03oooool>
ool0o`oooooo3_mo000?ooooo`koool01oooool000Sooooo2_oUiNDCooooo`[oOgmo4oooool:ool0
01?ooooo2_oo000Cooooo`[oool04oooool:o`3ooa?ooooo2_l0o`0Cooooo`[oo`3o4oooool:ogl0
01?ooooo2_ooo`09ooooo`002_ooool6onGUiAOooooo1_moOglGooooo`Koo`005oooool6ool001Oo
oooo1_ooo`0Gooooo`Ko0?oo5oooool6o`3o01Oooooo1_oo0?lGooooo`KoO`005oooool6oooo00_o
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000:ooooo`Koo`3o5oooool6o`3ooaOooooo1_oo000Gooooo`Ko07l05oooool6o`1o01Oo
oooo1_oo000Gooooo`Koo`3o5oooool6onGUiAOooooo1_ooo`0Gooooo`Koool02oooool000Sooooo
2_oo0?lCooooo`[o0?oo4oooool:ool001?ooooo2_l0O`0Cooooo`[o07l04oooool:ool001?ooooo
2_oo0?lCooooo`[oiNGU4oooool:oooo01?ooooo2_ooo`09ooooo`001_ooool>ool0o`oooooo3_l0
ool?ooooo`koo`003oooool>o`1o00oooooo3_l0O`0?ooooo`koo`003oooool>ool0o`oooooo3_oU
iND?ooooo`koool03oooool>oooo00Oooooo0005oooooa3oo`3o3Oooool@o`3oo`gooooo4?oo000=
oooooa3o07l03Oooool@o`1o00gooooo4?oo000=oooooa3oo`3o3Oooool@onGUi@gooooo4?ooo`0=
oooooa3oool01_ooool000Gooooo4?oo0?l=oooooa3o0?oo3Oooool@ool000gooooo4?l0O`0=oooo
oa3o07l03Oooool@ool000gooooo4?oo0?l=oooooa3oiNGU3Oooool@oooo00gooooo4?ooo`06oooo
o`001?oooolBool0o`_ooooo4_l0ool;oooooa;oo`002ooooolBo`1o00_ooooo4_l0O`0;oooooa;o
o`002ooooolBool0o`_ooooo4_oUiND;oooooa;oool02ooooolBoooo00Gooooo0004oooooa;oo`3o
2ooooolBo`3oo`_ooooo4_oo000;oooooa;o07l02ooooolBo`1o00_ooooo4_oo000;oooooa;oo`3o
2ooooolBonGUi@_ooooo4_ooo`0;oooooa;oool01Oooool000?ooooo5?oo0?l9oooooaCo0?oo2Ooo
oolDool000Wooooo5?l0O`09oooooaCo07l02OoooolDool000Wooooo5?oo0?l9oooooaCoiNGU2Ooo
oolDoooo00Wooooo5?ooo`04ooooo`000ooooolDool0o`Wooooo5?l0ool9oooooaCoo`002OoooolD
o`1o00Wooooo5?l0O`09oooooaCoo`002OoooolDool0o`Wooooo5?oUiND9oooooaCoool02OoooolD
oooo00Cooooo0003oooooaCoo`3o2OoooolDo`3oo`Wooooo5?oo0009oooooaCo07l02OoooolDo`1o
00Wooooo5?oo0009oooooaCoo`3o2OoooolDonGUi@Wooooo5?ooo`09oooooaCoool01?ooool000?o
oooo5?oo0?l9o`0001Co0?oo2Ol0000Dool000Wo00005?l0O`09o`0001Co07l02Ol0000Dool000Wo
00005?oo0?l9o`0001CoiNGU2Ol0000Doooo00Wo00005?ooo`04ooooo`000ooooolDool0o`Wooooo
5?l0ool9oooooaCoo`002OoooolDo`1o00Wooooo5?l0O`09oooooaCoo`002OoooolDool0o`Wooooo
5?oUiND9oooooaCoool02OoooolDoooo00Cooooo0003oooooaCoo`3o2OoooolDo`3oo`Wooooo5?oo
0009oooooaCo07l02OoooolDo`1o00Wooooo5?oo0009oooooaCoo`3o2OoooolDonGUi@Wooooo5?oo
o`09oooooaCoool01?ooool000Cooooo4_oo0?l;oooooa;o0?oo2ooooolBool000_ooooo4_l0O`0;
oooooa;o07l02ooooolBool000_ooooo4_oo0?l;oooooa;oiNGU2ooooolBoooo00_ooooo4_ooo`05
ooooo`001?oooolBool0o`_ooooo4_l0ool;oooooa;oo`002ooooolBo`1o00_ooooo4_l0O`0;oooo
oa;oo`002ooooolBool0o`_ooooo4_oUiND;oooooa;oool02ooooolBoooo00Gooooo0005oooooa3o
o`3o3Oooool@o`3oo`gooooo4?oo000=oooooa3o07l03Oooool@o`1o00gooooo4?oo000=oooooa3o
o`3o3Oooool@onGUi@gooooo4?ooo`0=oooooa3oool01_ooool000Gooooo4?oo0?l=oooooa3o0?oo
3Oooool@ool000gooooo4?l0O`0=oooooa3o07l03Oooool@ool000gooooo4?oo0?l=oooooa3oiNGU
3Oooool@oooo00gooooo4?ooo`06ooooo`001_ooool>ool0o`oooooo3_l0ool?ooooo`koo`003ooo
ool>o`1o00oooooo3_l0O`0?ooooo`koo`003oooool>ool0o`oooooo3_oUiND?ooooo`koool03ooo
ool>oooo00Oooooo0008ooooo`[oo`3o4oooool:o`3ooa?ooooo2_oo000Cooooo`[o07l04oooool:
o`1o01?ooooo2_oo000Cooooo`[oo`3o4oooool:onGUiA?ooooo2_ooo`0Cooooo`[oool02Oooool0
00[ooooo1_oo0?lGooooo`Ko0?oo5oooool6ool001Oooooo1_l0O`0Gooooo`Ko07l05oooool6ool0
01Oooooo1_oo0?lGooooo`KoiNGU5oooool6oooo01Oooooo1_ooo`0;ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`002_ooool6ool0
oaOooooo1_l0o`0Gooooo`Ko001o5oooool6o`3o01Oooooo1_l0o`0Gooooo`KoO`005oooool6o`00
OaOooooo1_mo000Gooooo`KoOgmo5oooool6onGUi@_ooooo0008ooooo`[oo`3o4oooool:o`3o01?o
oooo2_l007lCooooo`[o0?l04oooool:o`3o01?ooooo2_mo000Cooooo`[o001o4oooool:ogl001?o
oooo2_moOglCooooo`[oiNGU2Oooool000Kooooo3_oo0?l?ooooo`ko0?l03oooool>o`00O`oooooo
3_l0o`0?ooooo`ko0?l03oooool>ogl000oooooo3_l007l?ooooo`koO`003oooool>ogmoO`oooooo
3_oUiND7ooooo`001Oooool@ool0o`gooooo4?l0o`0=oooooa3o001o3Oooool@o`3o00gooooo4?l0
o`0=oooooa3oO`003Oooool@o`00O`gooooo4?mo000=oooooa3oOgmo3Oooool@onGUi@Kooooo0005
oooooa3oo`3o3Oooool@o`3o00gooooo4?l007l=oooooa3o0?l03Oooool@o`3o00gooooo4?mo000=
oooooa3o001o3Oooool@ogl000gooooo4?moOgl=oooooa3oiNGU1_ooool000Cooooo4_oo0?l;oooo
oa;o0?l02ooooolBo`00O`_ooooo4_l0o`0;oooooa;o0?l02ooooolBogl000_ooooo4_l007l;oooo
oa;oO`002ooooolBogmoO`_ooooo4_oUiND5ooooo`001?oooolBool0o`_ooooo4_l0o`0;oooooa;o
001o2ooooolBo`3o00_ooooo4_l0o`0;oooooa;oO`002ooooolBo`00O`_ooooo4_mo000;oooooa;o
Ogmo2ooooolBonGUi@Gooooo0003oooooaCoo`3o2OoooolDo`3o00Wooooo5?l007l9oooooaCo0?l0
2OoooolDo`3o00Wooooo5?mo0009oooooaCo001o2OoooolDogl000Wooooo5?moOgl9oooooaCoiNGU
1?ooool000?ooooo5?oo0?l9oooooaCo0?l02OoooolDo`00O`Wooooo5?l0o`09oooooaCo0?l02Ooo
oolDogl000Wooooo5?l007l9oooooaCoO`002OoooolDogmoO`Wooooo5?oUiND4ooooo`000ooooolD
ool0o`Wooooo5?l0o`09oooooaCo001o2OoooolDo`3o00Wooooo5?l0o`09oooooaCoO`002OoooolD
o`00O`Wooooo5?mo0009oooooaCoOgmo2OoooolDonGUi@Cooooo0003oooooaCoo`3o2Ol0000Do`3o
00Wo00005?l007l9o`0001Co0?l02Ol0000Do`3o00Wo00005?mo0009o`0001Co001o2Ol0000Dogl0
00Wo00005?moOgl9o`0001CoiNGU1?ooool000?ooooo5?oo0?l9oooooaCo0?l02OoooolDo`00O`Wo
oooo5?l0o`09oooooaCo0?l02OoooolDogl000Wooooo5?l007l9oooooaCoO`002OoooolDogmoO`Wo
oooo5?oUiND4ooooo`000ooooolDool0o`Wooooo5?l0o`09oooooaCo001o2OoooolDo`3o00Wooooo
5?l0o`09oooooaCoO`002OoooolDo`00O`Wooooo5?mo0009oooooaCoOgmo2OoooolDonGUi@Cooooo
0004oooooa;oo`3o2ooooolBo`3o00_ooooo4_l007l;oooooa;o0?l02ooooolBo`3o00_ooooo4_mo
000;oooooa;o001o2ooooolBogl000_ooooo4_moOgl;oooooa;oiNGU1Oooool000Cooooo4_oo0?l;
oooooa;o0?l02ooooolBo`00O`_ooooo4_l0o`0;oooooa;o0?l02ooooolBogl000_ooooo4_l007l;
oooooa;oO`002ooooolBogmoO`_ooooo4_oUiND5ooooo`001Oooool@ool0o`gooooo4?l0o`0=oooo
oa3o001o3Oooool@o`3o00gooooo4?l0o`0=oooooa3oO`003Oooool@o`00O`gooooo4?mo000=oooo
oa3oOgmo3Oooool@onGUi@Kooooo0005oooooa3oo`3o3Oooool@o`3o00gooooo4?l007l=oooooa3o
0?l03Oooool@o`3o00gooooo4?mo000=oooooa3o001o3Oooool@ogl000gooooo4?moOgl=oooooa3o
iNGU1_ooool000Kooooo3_oo0?l?ooooo`ko0?l03oooool>o`00O`oooooo3_l0o`0?ooooo`ko0?l0
3oooool>ogl000oooooo3_l007l?ooooo`koO`003oooool>ogmoO`oooooo3_oUiND7ooooo`002?oo
ool:ool0oa?ooooo2_l0o`0Cooooo`[o001o4oooool:o`3o01?ooooo2_l0o`0Cooooo`[oO`004ooo
ool:o`00Oa?ooooo2_mo000Cooooo`[oOgmo4oooool:onGUi@Wooooo000:ooooo`Koo`3o5oooool6
o`3o01Oooooo1_l007lGooooo`Ko0?l05oooool6o`3o01Oooooo1_mo000Gooooo`Ko001o5oooool6
ogl001Oooooo1_moOglGooooo`KoiNGU2oooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000[ooooo1_l0oolGooooo`Ko001o5oooool6
o`00OaOooooo1_l0O`0Gooooo`Koool05oooool6ool001Oooooo1_l007lGooooo`Koo`005oooool6
ool0oaOooooo1_ooo`0;ooooo`002?ooool:o`3ooa?ooooo2_l007lCooooo`[o001o4oooool:o`1o
01?ooooo2_ooo`0Cooooo`[oo`004oooool:o`00Oa?ooooo2_oo000Cooooo`[oo`3o4oooool:oooo
00Wooooo0006ooooo`ko0?oo3oooool>o`00O`oooooo3_l007l?ooooo`ko07l03oooool>oooo00oo
oooo3_oo000?ooooo`ko001o3oooool>ool000oooooo3_oo0?l?ooooo`koool01oooool000Gooooo
4?l0ool=oooooa3o001o3Oooool@o`00O`gooooo4?l0O`0=oooooa3oool03Oooool@ool000gooooo
4?l007l=oooooa3oo`003Oooool@ool0o`gooooo4?ooo`06ooooo`001Oooool@o`3oo`gooooo4?l0
07l=oooooa3o001o3Oooool@o`1o00gooooo4?ooo`0=oooooa3oo`003Oooool@o`00O`gooooo4?oo
000=oooooa3oo`3o3Oooool@oooo00Kooooo0004oooooa;o0?oo2ooooolBo`00O`_ooooo4_l007l;
oooooa;o07l02ooooolBoooo00_ooooo4_oo000;oooooa;o001o2ooooolBool000_ooooo4_oo0?l;
oooooa;oool01Oooool000Cooooo4_l0ool;oooooa;o001o2ooooolBo`00O`_ooooo4_l0O`0;oooo
oa;oool02ooooolBool000_ooooo4_l007l;oooooa;oo`002ooooolBool0o`_ooooo4_ooo`05oooo
o`000ooooolDo`3oo`Wooooo5?l007l9oooooaCo001o2OoooolDo`1o00Wooooo5?ooo`09oooooaCo
o`002OoooolDo`00O`Wooooo5?oo0009oooooaCoo`3o2OoooolDoooo00Cooooo0003oooooaCo0?oo
2OoooolDo`00O`Wooooo5?l007l9oooooaCo07l02OoooolDoooo00Wooooo5?oo0009oooooaCo001o
2OoooolDool000Wooooo5?oo0?l9oooooaCoool01?ooool000?ooooo5?l0ool9oooooaCo001o2Ooo
oolDo`00O`Wooooo5?l0O`09oooooaCoool02OoooolDool000Wooooo5?l007l9oooooaCoo`002Ooo
oolDool0o`Wooooo5?ooo`04ooooo`000ooooolDo`3oo`Wo00005?l007l9o`0001Co001o2Ol0000D
o`1o00Wo00005?ooo`09o`0001Coo`002Ol0000Do`00O`Wo00005?oo0009o`0001Coo`3o2Ol0000D
oooo00Cooooo0003oooooaCo0?oo2OoooolDo`00O`Wooooo5?l007l9oooooaCo07l02OoooolDoooo
00Wooooo5?oo0009oooooaCo001o2OoooolDool000Wooooo5?oo0?l9oooooaCoool01?ooool000?o
oooo5?l0ool9oooooaCo001o2OoooolDo`00O`Wooooo5?l0O`09oooooaCoool02OoooolDool000Wo
oooo5?l007l9oooooaCoo`002OoooolDool0o`Wooooo5?ooo`04ooooo`001?oooolBo`3oo`_ooooo
4_l007l;oooooa;o001o2ooooolBo`1o00_ooooo4_ooo`0;oooooa;oo`002ooooolBo`00O`_ooooo
4_oo000;oooooa;oo`3o2ooooolBoooo00Gooooo0004oooooa;o0?oo2ooooolBo`00O`_ooooo4_l0
07l;oooooa;o07l02ooooolBoooo00_ooooo4_oo000;oooooa;o001o2ooooolBool000_ooooo4_oo
0?l;oooooa;oool01Oooool000Gooooo4?l0ool=oooooa3o001o3Oooool@o`00O`gooooo4?l0O`0=
oooooa3oool03Oooool@ool000gooooo4?l007l=oooooa3oo`003Oooool@ool0o`gooooo4?ooo`06
ooooo`001Oooool@o`3oo`gooooo4?l007l=oooooa3o001o3Oooool@o`1o00gooooo4?ooo`0=oooo
oa3oo`003Oooool@o`00O`gooooo4?oo000=oooooa3oo`3o3Oooool@oooo00Kooooo0006ooooo`ko
0?oo3oooool>o`00O`oooooo3_l007l?ooooo`ko07l03oooool>oooo00oooooo3_oo000?ooooo`ko
001o3oooool>ool000oooooo3_oo0?l?ooooo`koool01oooool000Sooooo2_l0oolCooooo`[o001o
4oooool:o`00Oa?ooooo2_l0O`0Cooooo`[oool04oooool:ool001?ooooo2_l007lCooooo`[oo`00
4oooool:ool0oa?ooooo2_ooo`09ooooo`002_ooool6o`3ooaOooooo1_l007lGooooo`Ko001o5ooo
ool6o`1o01Oooooo1_ooo`0Gooooo`Koo`005oooool6o`00OaOooooo1_oo000Gooooo`Koo`3o5ooo
ool6oooo00_ooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000:ooooo`Koo`3o5oooool6o`1o01Oooooo1_oUiNDGooooo`Ko001o5ooo
ool6ool001Oooooo1_l0O`0Gooooo`Ko0?l05oooool6onGUiAOooooo1_oUiNDGooooo`Koo`002ooo
ool000Sooooo2_oo0?lCooooo`[o07l04oooool:onGUiA?ooooo2_l007lCooooo`[oo`004oooool:
o`1o01?ooooo2_l0o`0Cooooo`[oiNGU4oooool:onGUiA?ooooo2_oo0009ooooo`001_ooool>ool0
o`oooooo3_l0O`0?ooooo`koiNGU3oooool>o`00O`oooooo3_oo000?ooooo`ko07l03oooool>o`3o
00oooooo3_oUiND?ooooo`koiNGU3oooool>ool000Oooooo0005oooooa3oo`3o3Oooool@o`1o00go
oooo4?oUiND=oooooa3o001o3Oooool@ool000gooooo4?l0O`0=oooooa3o0?l03Oooool@onGUi@go
oooo4?oUiND=oooooa3oo`001_ooool000Gooooo4?oo0?l=oooooa3o07l03Oooool@onGUi@gooooo
4?l007l=oooooa3oo`003Oooool@o`1o00gooooo4?l0o`0=oooooa3oiNGU3Oooool@onGUi@gooooo
4?oo0006ooooo`001?oooolBool0o`_ooooo4_l0O`0;oooooa;oiNGU2ooooolBo`00O`_ooooo4_oo
000;oooooa;o07l02ooooolBo`3o00_ooooo4_oUiND;oooooa;oiNGU2ooooolBool000Gooooo0004
oooooa;oo`3o2ooooolBo`1o00_ooooo4_oUiND;oooooa;o001o2ooooolBool000_ooooo4_l0O`0;
oooooa;o0?l02ooooolBonGUi@_ooooo4_oUiND;oooooa;oo`001Oooool000?ooooo5?oo0?l9oooo
oaCo07l02OoooolDonGUi@Wooooo5?l007l9oooooaCoo`002OoooolDo`1o00Wooooo5?l0o`09oooo
oaCoiNGU2OoooolDonGUi@Wooooo5?oo0004ooooo`000ooooolDool0o`Wooooo5?l0O`09oooooaCo
iNGU2OoooolDo`00O`Wooooo5?oo0009oooooaCo07l02OoooolDo`3o00Wooooo5?oUiND9oooooaCo
iNGU2OoooolDool000Cooooo0003oooooaCoo`3o2Ol0000Do`1o00Wo00005?oUiND9o`0001Co001o
2Ol0000Dool000Wo00005?l0O`09o`0001Co0?l02Ol0000DonGUi@Wo00005?oUiND9o`0001Coo`00
1?ooool000?ooooo5?oo0?l9oooooaCo07l02OoooolDonGUi@Wooooo5?l007l9oooooaCoo`002Ooo
oolDo`1o00Wooooo5?l0o`09oooooaCoiNGU2OoooolDonGUi@Wooooo5?oo0004ooooo`000ooooolD
ool0o`Wooooo5?l0O`09oooooaCoiNGU2OoooolDo`00O`Wooooo5?oo0009oooooaCo07l02OoooolD
o`3o00Wooooo5?oUiND9oooooaCoiNGU2OoooolDool000Cooooo0003oooooaCoo`3o2OoooolDo`1o
00Wooooo5?oUiND9oooooaCo001o2OoooolDool000Wooooo5?l0O`09oooooaCo0?l02OoooolDonGU
i@Wooooo5?oUiND9oooooaCoo`001?ooool000Cooooo4_oo0?l;oooooa;o07l02ooooolBonGUi@_o
oooo4_l007l;oooooa;oo`002ooooolBo`1o00_ooooo4_l0o`0;oooooa;oiNGU2ooooolBonGUi@_o
oooo4_oo0005ooooo`001?oooolBool0o`_ooooo4_l0O`0;oooooa;oiNGU2ooooolBo`00O`_ooooo
4_oo000;oooooa;o07l02ooooolBo`3o00_ooooo4_oUiND;oooooa;oiNGU2ooooolBool000Gooooo
0005oooooa3oo`3o3Oooool@o`1o00gooooo4?oUiND=oooooa3o001o3Oooool@ool000gooooo4?l0
O`0=oooooa3o0?l03Oooool@onGUi@gooooo4?oUiND=oooooa3oo`001_ooool000Gooooo4?oo0?l=
oooooa3o07l03Oooool@onGUi@gooooo4?l007l=oooooa3oo`003Oooool@o`1o00gooooo4?l0o`0=
oooooa3oiNGU3Oooool@onGUi@gooooo4?oo0006ooooo`001_ooool>ool0o`oooooo3_l0O`0?oooo
o`koiNGU3oooool>o`00O`oooooo3_oo000?ooooo`ko07l03oooool>o`3o00oooooo3_oUiND?oooo
o`koiNGU3oooool>ool000Oooooo0008ooooo`[oo`3o4oooool:o`1o01?ooooo2_oUiNDCooooo`[o
001o4oooool:ool001?ooooo2_l0O`0Cooooo`[o0?l04oooool:onGUiA?ooooo2_oUiNDCooooo`[o
o`002Oooool000[ooooo1_oo0?lGooooo`Ko07l05oooool6onGUiAOooooo1_l007lGooooo`Koo`00
5oooool6o`1o01Oooooo1_l0o`0Gooooo`KoiNGU5oooool6onGUiAOooooo1_oo000;ooooo`003Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
2_ooool6o`3ooaOooooo1_l0o`0Gooooo`KoOgmo5oooool6o`3ooaOooooo1_l0oolGooooo`Ko0?oo
5oooool6ogmoOaOooooo1_l007lGooooo`KoiNGU5oooool6ool000_ooooo0008ooooo`[o0?oo4ooo
ool:o`3o01?ooooo2_moOglCooooo`[o0?oo4oooool:o`3ooa?ooooo2_l0oolCooooo`[oOgmo4ooo
ool:o`00Oa?ooooo2_oUiNDCooooo`[oo`002Oooool000Kooooo3_l0ool?ooooo`ko0?l03oooool>
ogmoO`oooooo3_l0ool?ooooo`ko0?oo3oooool>o`3oo`oooooo3_moOgl?ooooo`ko001o3oooool>
onGUi@oooooo3_oo0007ooooo`001Oooool@o`3oo`gooooo4?l0o`0=oooooa3oOgmo3Oooool@o`3o
o`gooooo4?l0ool=oooooa3o0?oo3Oooool@ogmoO`gooooo4?l007l=oooooa3oiNGU3Oooool@ool0
00Kooooo0005oooooa3o0?oo3Oooool@o`3o00gooooo4?moOgl=oooooa3o0?oo3Oooool@o`3oo`go
oooo4?l0ool=oooooa3oOgmo3Oooool@o`00O`gooooo4?oUiND=oooooa3oo`001_ooool000Cooooo
4_l0ool;oooooa;o0?l02ooooolBogmoO`_ooooo4_l0ool;oooooa;o0?oo2ooooolBo`3oo`_ooooo
4_moOgl;oooooa;o001o2ooooolBonGUi@_ooooo4_oo0005ooooo`001?oooolBo`3oo`_ooooo4_l0
o`0;oooooa;oOgmo2ooooolBo`3oo`_ooooo4_l0ool;oooooa;o0?oo2ooooolBogmoO`_ooooo4_l0
07l;oooooa;oiNGU2ooooolBool000Gooooo0003oooooaCo0?oo2OoooolDo`3o00Wooooo5?moOgl9
oooooaCo0?oo2OoooolDo`3oo`Wooooo5?l0ool9oooooaCoOgmo2OoooolDo`00O`Wooooo5?oUiND9
oooooaCoo`001?ooool000?ooooo5?l0ool9oooooaCo0?l02OoooolDogmoO`Wooooo5?l0ool9oooo
oaCo0?oo2OoooolDo`3oo`Wooooo5?moOgl9oooooaCo001o2OoooolDonGUi@Wooooo5?oo0004oooo
o`000ooooolDo`3oo`Wo00005?l0o`09o`0001CoOgmo2Ol0000Do`3oo`Wo00005?l0ool9o`0001Co
0?oo2Ol0000DogmoO`Wo00005?l007l9o`0001CoiNGU2Ol0000Dool000Cooooo0003oooooaCo0?oo
2OoooolDo`3o00Wooooo5?moOgl9oooooaCo0?oo2OoooolDo`3oo`Wooooo5?l0ool9oooooaCoOgmo
2OoooolDo`00O`Wooooo5?oUiND9oooooaCoo`001?ooool000?ooooo5?l0ool9oooooaCo0?l02Ooo
oolDogmoO`Wooooo5?l0ool9oooooaCo0?oo2OoooolDo`3oo`Wooooo5?moOgl9oooooaCo001o2Ooo
oolDonGUi@Wooooo5?oo0004ooooo`000ooooolDo`3oo`Wooooo5?l0o`09oooooaCoOgmo2OoooolD
o`3oo`Wooooo5?l0ool9oooooaCo0?oo2OoooolDogmoO`Wooooo5?l007l9oooooaCoiNGU2OoooolD
ool000Cooooo0004oooooa;o0?oo2ooooolBo`3o00_ooooo4_moOgl;oooooa;o0?oo2ooooolBo`3o
o`_ooooo4_l0ool;oooooa;oOgmo2ooooolBo`00O`_ooooo4_oUiND;oooooa;oo`001Oooool000Co
oooo4_l0ool;oooooa;o0?l02ooooolBogmoO`_ooooo4_l0ool;oooooa;o0?oo2ooooolBo`3oo`_o
oooo4_moOgl;oooooa;o001o2ooooolBonGUi@_ooooo4_oo0005ooooo`001Oooool@o`3oo`gooooo
4?l0o`0=oooooa3oOgmo3Oooool@o`3oo`gooooo4?l0ool=oooooa3o0?oo3Oooool@ogmoO`gooooo
4?l007l=oooooa3oiNGU3Oooool@ool000Kooooo0005oooooa3o0?oo3Oooool@o`3o00gooooo4?mo
Ogl=oooooa3o0?oo3Oooool@o`3oo`gooooo4?l0ool=oooooa3oOgmo3Oooool@o`00O`gooooo4?oU
iND=oooooa3oo`001_ooool000Kooooo3_l0ool?ooooo`ko0?l03oooool>ogmoO`oooooo3_l0ool?
ooooo`ko0?oo3oooool>o`3oo`oooooo3_moOgl?ooooo`ko001o3oooool>onGUi@oooooo3_oo0007
ooooo`002?ooool:o`3ooa?ooooo2_l0o`0Cooooo`[oOgmo4oooool:o`3ooa?ooooo2_l0oolCoooo
o`[o0?oo4oooool:ogmoOa?ooooo2_l007lCooooo`[oiNGU4oooool:ool000Wooooo000:ooooo`Ko
0?oo5oooool6o`3o01Oooooo1_moOglGooooo`Ko0?oo5oooool6o`3ooaOooooo1_l0oolGooooo`Ko
Ogmo5oooool6o`00OaOooooo1_oUiNDGooooo`Koo`002oooool000gooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000[ooooo1_l0oolGooooo`Ko
Ogmo5oooool6o`00OaOooooo1_l0o`0Gooooo`Koo`005oooool6o`3o01Oooooo1_oo0?lGooooo`Ko
07l05oooool6oooo01Oooooo1_oo000;ooooo`002?ooool:o`3ooa?ooooo2_moOglCooooo`[o001o
4oooool:o`3o01?ooooo2_oo000Cooooo`[o0?l04oooool:ool0oa?ooooo2_l0O`0Cooooo`[oool0
4oooool:ool000Wooooo0006ooooo`ko0?oo3oooool>ogmoO`oooooo3_l007l?ooooo`ko0?l03ooo
ool>ool000oooooo3_l0o`0?ooooo`koo`3o3oooool>o`1o00oooooo3_ooo`0?ooooo`koo`001ooo
ool000Gooooo4?l0ool=oooooa3oOgmo3Oooool@o`00O`gooooo4?l0o`0=oooooa3oo`003Oooool@
o`3o00gooooo4?oo0?l=oooooa3o07l03Oooool@oooo00gooooo4?oo0006ooooo`001Oooool@o`3o
o`gooooo4?moOgl=oooooa3o001o3Oooool@o`3o00gooooo4?oo000=oooooa3o0?l03Oooool@ool0
o`gooooo4?l0O`0=oooooa3oool03Oooool@ool000Kooooo0004oooooa;o0?oo2ooooolBogmoO`_o
oooo4_l007l;oooooa;o0?l02ooooolBool000_ooooo4_l0o`0;oooooa;oo`3o2ooooolBo`1o00_o
oooo4_ooo`0;oooooa;oo`001Oooool000Cooooo4_l0ool;oooooa;oOgmo2ooooolBo`00O`_ooooo
4_l0o`0;oooooa;oo`002ooooolBo`3o00_ooooo4_oo0?l;oooooa;o07l02ooooolBoooo00_ooooo
4_oo0005ooooo`000ooooolDo`3oo`Wooooo5?moOgl9oooooaCo001o2OoooolDo`3o00Wooooo5?oo
0009oooooaCo0?l02OoooolDool0o`Wooooo5?l0O`09oooooaCoool02OoooolDool000Cooooo0003
oooooaCo0?oo2OoooolDogmoO`Wooooo5?l007l9oooooaCo0?l02OoooolDool000Wooooo5?l0o`09
oooooaCoo`3o2OoooolDo`1o00Wooooo5?ooo`09oooooaCoo`001?ooool000?ooooo5?l0ool9o`00
01CoOgmo2Ol0000Do`00O`Wo00005?l0o`09o`0001Coo`002Ol0000Do`3o00Wo00005?oo0?l9o`00
01Co07l02Ol0000Doooo00Wo00005?oo0004ooooo`000ooooolDo`3oo`Wooooo5?moOgl9oooooaCo
001o2OoooolDo`3o00Wooooo5?oo0009oooooaCo0?l02OoooolDool0o`Wooooo5?l0O`09oooooaCo
ool02OoooolDool000Cooooo0003oooooaCo0?oo2OoooolDogmoO`Wooooo5?l007l9oooooaCo0?l0
2OoooolDool000Wooooo5?l0o`09oooooaCoo`3o2OoooolDo`1o00Wooooo5?ooo`09oooooaCoo`00
1?ooool000?ooooo5?l0ool9oooooaCoOgmo2OoooolDo`00O`Wooooo5?l0o`09oooooaCoo`002Ooo
oolDo`3o00Wooooo5?oo0?l9oooooaCo07l02OoooolDoooo00Wooooo5?oo0004ooooo`001?oooolB
o`3oo`_ooooo4_moOgl;oooooa;o001o2ooooolBo`3o00_ooooo4_oo000;oooooa;o0?l02ooooolB
ool0o`_ooooo4_l0O`0;oooooa;oool02ooooolBool000Gooooo0004oooooa;o0?oo2ooooolBogmo
O`_ooooo4_l007l;oooooa;o0?l02ooooolBool000_ooooo4_l0o`0;oooooa;oo`3o2ooooolBo`1o
00_ooooo4_ooo`0;oooooa;oo`001Oooool000Gooooo4?l0ool=oooooa3oOgmo3Oooool@o`00O`go
oooo4?l0o`0=oooooa3oo`003Oooool@o`3o00gooooo4?oo0?l=oooooa3o07l03Oooool@oooo00go
oooo4?oo0006ooooo`001Oooool@o`3oo`gooooo4?moOgl=oooooa3o001o3Oooool@o`3o00gooooo
4?oo000=oooooa3o0?l03Oooool@ool0o`gooooo4?l0O`0=oooooa3oool03Oooool@ool000Kooooo
0006ooooo`ko0?oo3oooool>ogmoO`oooooo3_l007l?ooooo`ko0?l03oooool>ool000oooooo3_l0
o`0?ooooo`koo`3o3oooool>o`1o00oooooo3_ooo`0?ooooo`koo`001oooool000Sooooo2_l0oolC
ooooo`[oOgmo4oooool:o`00Oa?ooooo2_l0o`0Cooooo`[oo`004oooool:o`3o01?ooooo2_oo0?lC
ooooo`[o07l04oooool:oooo01?ooooo2_oo0009ooooo`002_ooool6o`3ooaOooooo1_moOglGoooo
o`Ko001o5oooool6o`3o01Oooooo1_oo000Gooooo`Ko0?l05oooool6ool0oaOooooo1_l0O`0Goooo
o`Koool05oooool6ool000_ooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Ooo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000:ooooo`Koool05oooool6oooo01Oooooo1_ooo`0Goooo
o`Koo`005oooool6o`3o01Oooooo1_l007lGooooo`KoOgmo5oooool6ool0oaOooooo1_l0o`0Goooo
o`Koool02oooool000Sooooo2_ooo`0Cooooo`[oool04oooool:oooo01?ooooo2_oo000Cooooo`[o
0?l04oooool:o`00Oa?ooooo2_moOglCooooo`[oo`3o4oooool:o`3o01?ooooo2_ooo`09ooooo`00
1_ooool>oooo00oooooo3_ooo`0?ooooo`koool03oooool>ool000oooooo3_l0o`0?ooooo`ko001o
3oooool>ogmoO`oooooo3_oo0?l?ooooo`ko0?l03oooool>oooo00Oooooo0005oooooa3oool03Ooo
ool@oooo00gooooo4?ooo`0=oooooa3oo`003Oooool@o`3o00gooooo4?l007l=oooooa3oOgmo3Ooo
ool@ool0o`gooooo4?l0o`0=oooooa3oool01_ooool000Gooooo4?ooo`0=oooooa3oool03Oooool@
oooo00gooooo4?oo000=oooooa3o0?l03Oooool@o`00O`gooooo4?moOgl=oooooa3oo`3o3Oooool@
o`3o00gooooo4?ooo`06ooooo`001?oooolBoooo00_ooooo4_ooo`0;oooooa;oool02ooooolBool0
00_ooooo4_l0o`0;oooooa;o001o2ooooolBogmoO`_ooooo4_oo0?l;oooooa;o0?l02ooooolBoooo
00Gooooo0004oooooa;oool02ooooolBoooo00_ooooo4_ooo`0;oooooa;oo`002ooooolBo`3o00_o
oooo4_l007l;oooooa;oOgmo2ooooolBool0o`_ooooo4_l0o`0;oooooa;oool01Oooool000?ooooo
5?ooo`09oooooaCoool02OoooolDoooo00Wooooo5?oo0009oooooaCo0?l02OoooolDo`00O`Wooooo
5?moOgl9oooooaCoo`3o2OoooolDo`3o00Wooooo5?ooo`04ooooo`000ooooolDoooo00Wooooo5?oo
o`09oooooaCoool02OoooolDool000Wooooo5?l0o`09oooooaCo001o2OoooolDogmoO`Wooooo5?oo
0?l9oooooaCo0?l02OoooolDoooo00Cooooo0003oooooaCoool02Ol0000Doooo00Wo00005?ooo`09
o`0001Coo`002Ol0000Do`3o00Wo00005?l007l9o`0001CoOgmo2Ol0000Dool0o`Wo00005?l0o`09
o`0001Coool01?ooool000?ooooo5?ooo`09oooooaCoool02OoooolDoooo00Wooooo5?oo0009oooo
oaCo0?l02OoooolDo`00O`Wooooo5?moOgl9oooooaCoo`3o2OoooolDo`3o00Wooooo5?ooo`04oooo
o`000ooooolDoooo00Wooooo5?ooo`09oooooaCoool02OoooolDool000Wooooo5?l0o`09oooooaCo
001o2OoooolDogmoO`Wooooo5?oo0?l9oooooaCo0?l02OoooolDoooo00Cooooo0003oooooaCoool0
2OoooolDoooo00Wooooo5?ooo`09oooooaCoo`002OoooolDo`3o00Wooooo5?l007l9oooooaCoOgmo
2OoooolDool0o`Wooooo5?l0o`09oooooaCoool01?ooool000Cooooo4_ooo`0;oooooa;oool02ooo
oolBoooo00_ooooo4_oo000;oooooa;o0?l02ooooolBo`00O`_ooooo4_moOgl;oooooa;oo`3o2ooo
oolBo`3o00_ooooo4_ooo`05ooooo`001?oooolBoooo00_ooooo4_ooo`0;oooooa;oool02ooooolB
ool000_ooooo4_l0o`0;oooooa;o001o2ooooolBogmoO`_ooooo4_oo0?l;oooooa;o0?l02ooooolB
oooo00Gooooo0005oooooa3oool03Oooool@oooo00gooooo4?ooo`0=oooooa3oo`003Oooool@o`3o
00gooooo4?l007l=oooooa3oOgmo3Oooool@ool0o`gooooo4?l0o`0=oooooa3oool01_ooool000Go
oooo4?ooo`0=oooooa3oool03Oooool@oooo00gooooo4?oo000=oooooa3o0?l03Oooool@o`00O`go
oooo4?moOgl=oooooa3oo`3o3Oooool@o`3o00gooooo4?ooo`06ooooo`001_ooool>oooo00oooooo
3_ooo`0?ooooo`koool03oooool>ool000oooooo3_l0o`0?ooooo`ko001o3oooool>ogmoO`oooooo
3_oo0?l?ooooo`ko0?l03oooool>oooo00Oooooo0008ooooo`[oool04oooool:oooo01?ooooo2_oo
o`0Cooooo`[oo`004oooool:o`3o01?ooooo2_l007lCooooo`[oOgmo4oooool:ool0oa?ooooo2_l0
o`0Cooooo`[oool02Oooool000[ooooo1_ooo`0Gooooo`Koool05oooool6oooo01Oooooo1_oo000G
ooooo`Ko0?l05oooool6o`00OaOooooo1_moOglGooooo`Koo`3o5oooool6o`3o01Oooooo1_ooo`0;
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`002_ooool6ogl001Oooooo1_mo000Gooooo`KoOgmo5oooool6o`3ooaOooooo1_l0o`0G
ooooo`Koo`005oooool6ogl001Oooooo1_l0oolGooooo`Koo`005oooool6o`1o00_ooooo0008oooo
o`[oO`004oooool:ogl001?ooooo2_moOglCooooo`[o0?oo4oooool:o`3o01?ooooo2_oo000Coooo
o`[oO`004oooool:o`3ooa?ooooo2_oo000Cooooo`[o07l02Oooool000Kooooo3_mo000?ooooo`ko
O`003oooool>ogmoO`oooooo3_l0ool?ooooo`ko0?l03oooool>ool000oooooo3_mo000?ooooo`ko
0?oo3oooool>ool000oooooo3_l0O`07ooooo`001Oooool@ogl000gooooo4?mo000=oooooa3oOgmo
3Oooool@o`3oo`gooooo4?l0o`0=oooooa3oo`003Oooool@ogl000gooooo4?l0ool=oooooa3oo`00
3Oooool@o`1o00Kooooo0005oooooa3oO`003Oooool@ogl000gooooo4?moOgl=oooooa3o0?oo3Ooo
ool@o`3o00gooooo4?oo000=oooooa3oO`003Oooool@o`3oo`gooooo4?oo000=oooooa3o07l01_oo
ool000Cooooo4_mo000;oooooa;oO`002ooooolBogmoO`_ooooo4_l0ool;oooooa;o0?l02ooooolB
ool000_ooooo4_mo000;oooooa;o0?oo2ooooolBool000_ooooo4_l0O`05ooooo`001?oooolBogl0
00_ooooo4_mo000;oooooa;oOgmo2ooooolBo`3oo`_ooooo4_l0o`0;oooooa;oo`002ooooolBogl0
00_ooooo4_l0ool;oooooa;oo`002ooooolBo`1o00Gooooo0003oooooaCoO`002OoooolDogl000Wo
oooo5?moOgl9oooooaCo0?oo2OoooolDo`3o00Wooooo5?oo0009oooooaCoO`002OoooolDo`3oo`Wo
oooo5?oo0009oooooaCo07l01?ooool000?ooooo5?mo0009oooooaCoO`002OoooolDogmoO`Wooooo
5?l0ool9oooooaCo0?l02OoooolDool000Wooooo5?mo0009oooooaCo0?oo2OoooolDool000Wooooo
5?l0O`04ooooo`000ooooolDogl000Wo00005?mo0009o`0001CoOgmo2Ol0000Do`3oo`Wo00005?l0
o`09o`0001Coo`002Ol0000Dogl000Wo00005?l0ool9o`0001Coo`002Ol0000Do`1o00Cooooo0003
oooooaCoO`002OoooolDogl000Wooooo5?moOgl9oooooaCo0?oo2OoooolDo`3o00Wooooo5?oo0009
oooooaCoO`002OoooolDo`3oo`Wooooo5?oo0009oooooaCo07l01?ooool000?ooooo5?mo0009oooo
oaCoO`002OoooolDogmoO`Wooooo5?l0ool9oooooaCo0?l02OoooolDool000Wooooo5?mo0009oooo
oaCo0?oo2OoooolDool000Wooooo5?l0O`04ooooo`000ooooolDogl000Wooooo5?mo0009oooooaCo
Ogmo2OoooolDo`3oo`Wooooo5?l0o`09oooooaCoo`002OoooolDogl000Wooooo5?l0ool9oooooaCo
o`002OoooolDo`1o00Cooooo0004oooooa;oO`002ooooolBogl000_ooooo4_moOgl;oooooa;o0?oo
2ooooolBo`3o00_ooooo4_oo000;oooooa;oO`002ooooolBo`3oo`_ooooo4_oo000;oooooa;o07l0
1Oooool000Cooooo4_mo000;oooooa;oO`002ooooolBogmoO`_ooooo4_l0ool;oooooa;o0?l02ooo
oolBool000_ooooo4_mo000;oooooa;o0?oo2ooooolBool000_ooooo4_l0O`05ooooo`001Oooool@
ogl000gooooo4?mo000=oooooa3oOgmo3Oooool@o`3oo`gooooo4?l0o`0=oooooa3oo`003Oooool@
ogl000gooooo4?l0ool=oooooa3oo`003Oooool@o`1o00Kooooo0005oooooa3oO`003Oooool@ogl0
00gooooo4?moOgl=oooooa3o0?oo3Oooool@o`3o00gooooo4?oo000=oooooa3oO`003Oooool@o`3o
o`gooooo4?oo000=oooooa3o07l01_ooool000Kooooo3_mo000?ooooo`koO`003oooool>ogmoO`oo
oooo3_l0ool?ooooo`ko0?l03oooool>ool000oooooo3_mo000?ooooo`ko0?oo3oooool>ool000oo
oooo3_l0O`07ooooo`002?ooool:ogl001?ooooo2_mo000Cooooo`[oOgmo4oooool:o`3ooa?ooooo
2_l0o`0Cooooo`[oo`004oooool:ogl001?ooooo2_l0oolCooooo`[oo`004oooool:o`1o00Wooooo
000:ooooo`KoO`005oooool6ogl001Oooooo1_moOglGooooo`Ko0?oo5oooool6o`3o01Oooooo1_oo
000Gooooo`KoO`005oooool6o`3ooaOooooo1_oo000Gooooo`Ko07l02oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[vnDTD]]; \[IndentingNewLine]Evolve[
          vnDTD], {12}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[126]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[\[IndentingNewLine]Partition[l, 4], 
        VertexStyle \[Rule] Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[128]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .74419 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.0177187 0.238095 [
[ 0 0 0 0 ]
[ 1 .74419 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .74419 L
0 .74419 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0177187 0.245293 0.239203 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
1 1 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
0 1 1 r
.34848 .14646 Mdot
1 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
0 1 1 r
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
0 1 1 r
.85354 .24747 Mdot
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.0177187 0.488926 0.239203 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 1 r
.34848 .04545 Mdot
1 1 0 r
.44949 .04545 Mdot
0 1 1 r
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
1 1 0 r
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
0 1 1 r
.75253 .24747 Mdot
1 0 1 r
.85354 .24747 Mdot
0 1 1 r
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.0177187 0.732558 0.239203 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
1 1 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
1 1 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
0 1 1 r
.75253 .14646 Mdot
1 0 1 r
.85354 .14646 Mdot
0 1 1 r
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.0177187 0.97619 0.239203 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 1 r
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
1 0 1 r
.85354 .04545 Mdot
0 1 1 r
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
1 1 0 r
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
1 0 1 r
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.261351 0.245293 0.482835 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
1 1 0 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
.75253 .04545 Mdot
1 1 0 r
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
0 1 1 r
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
1 1 0 r
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 1 r
.04545 .24747 Mdot
1 0 1 r
.14646 .24747 Mdot
0 1 1 r
.24747 .24747 Mdot
1 1 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
0 1 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
1 1 0 r
.85354 .24747 Mdot
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
1 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.498 0 0 r
.75253 .34848 Mdot
.898 .898 .898 r
.85354 .34848 Mdot
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
1 1 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.898 .898 .898 r
.85354 .44949 Mdot
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
1 1 0 r
.44949 .55051 Mdot
.55051 .55051 Mdot
1 0 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.898 .898 .898 r
.95455 .55051 Mdot
1 0 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
0 1 1 r
.44949 .65152 Mdot
1 0 1 r
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
0 1 1 r
.34848 .75253 Mdot
1 0 1 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.498 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
0 1 1 r
.34848 .85354 Mdot
.44949 .85354 Mdot
1 0 1 r
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.498 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
1 1 0 r
.44949 .95455 Mdot
1 0 1 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.261351 0.488926 0.482835 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
0 1 1 r
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
1 1 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
0 1 1 r
.75253 .04545 Mdot
.85354 .04545 Mdot
1 1 0 r
.95455 .04545 Mdot
0 1 1 r
.04545 .14646 Mdot
1 0 1 r
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
1 1 0 r
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
1 1 0 r
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
1 1 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
0 1 1 r
.75253 .24747 Mdot
.85354 .24747 Mdot
1 1 0 r
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
1 1 0 r
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
1 1 0 r
.55051 .34848 Mdot
0 1 1 r
.65152 .34848 Mdot
.75253 .34848 Mdot
.498 0 0 r
.85354 .34848 Mdot
.898 .898 .898 r
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
1 1 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
1 0 1 r
.75253 .44949 Mdot
.85354 .44949 Mdot
.898 .898 .898 r
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 1 r
.44949 .55051 Mdot
1 0 1 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
0 1 1 r
.34848 .85354 Mdot
1 0 1 r
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.498 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
.44949 .95455 Mdot
1 0 1 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.261351 0.732558 0.482835 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
1 0 1 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
1 1 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
1 1 0 r
.65152 .14646 Mdot
0 1 1 r
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
1 1 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
0 1 1 r
.85354 .24747 Mdot
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
0 1 1 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
1 1 0 r
.65152 .34848 Mdot
0 1 1 r
.75253 .34848 Mdot
.85354 .34848 Mdot
.498 0 0 r
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
1 0 1 r
.55051 .44949 Mdot
0 1 1 r
.65152 .44949 Mdot
1 0 1 r
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
1 0 1 r
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.261351 0.97619 0.482835 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
1 1 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
0 1 1 r
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
0 1 1 r
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
0 1 1 r
.34848 .24747 Mdot
1 1 0 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
0 1 1 r
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
1 0 1 r
.55051 .34848 Mdot
1 1 0 r
.65152 .34848 Mdot
1 0 1 r
.75253 .34848 Mdot
.85354 .34848 Mdot
0 1 1 r
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.504983 0.245293 0.726467 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.498 .498 .498 r
.14646 .04545 Mdot
0 0 .498 r
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 0 r
.65152 .04545 Mdot
1 1 0 r
.75253 .04545 Mdot
0 1 0 r
.85354 .04545 Mdot
0 1 1 r
.95455 .04545 Mdot
.898 .898 .898 r
.04545 .14646 Mdot
.498 .498 .498 r
.14646 .14646 Mdot
1 0 0 r
.24747 .14646 Mdot
.34848 .14646 Mdot
1 1 0 r
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
0 1 0 r
.65152 .14646 Mdot
1 0 1 r
.75253 .14646 Mdot
.498 0 0 r
.85354 .14646 Mdot
1 1 0 r
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
0 .498 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
1 0 0 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
.898 .898 .898 r
.75253 .24747 Mdot
1 1 0 r
.85354 .24747 Mdot
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
0 1 0 r
.14646 .34848 Mdot
0 0 .498 r
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.498 0 0 r
.55051 .34848 Mdot
0 0 .498 r
.65152 .34848 Mdot
.498 0 0 r
.75253 .34848 Mdot
.498 .498 .498 r
.85354 .34848 Mdot
.898 .898 .898 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
0 0 .498 r
.14646 .44949 Mdot
.24747 .44949 Mdot
0 .498 0 r
.34848 .44949 Mdot
1 1 0 r
.44949 .44949 Mdot
1 0 0 r
.55051 .44949 Mdot
0 0 .498 r
.65152 .44949 Mdot
1 0 0 r
.75253 .44949 Mdot
1 0 1 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
0 .498 0 r
.14646 .55051 Mdot
.898 .898 .898 r
.24747 .55051 Mdot
0 0 .498 r
.34848 .55051 Mdot
1 0 0 r
.44949 .55051 Mdot
0 .498 0 r
.55051 .55051 Mdot
0 1 0 r
.65152 .55051 Mdot
.898 .898 .898 r
.75253 .55051 Mdot
.85354 .55051 Mdot
1 0 0 r
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
0 1 0 r
.14646 .65152 Mdot
.498 .498 .498 r
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.498 .498 .498 r
.65152 .65152 Mdot
0 0 .498 r
.75253 .65152 Mdot
.898 .898 .898 r
.85354 .65152 Mdot
1 0 0 r
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.498 .498 .498 r
.14646 .75253 Mdot
0 0 .498 r
.24747 .75253 Mdot
0 1 0 r
.34848 .75253 Mdot
1 0 0 r
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
1 0 1 r
.65152 .75253 Mdot
0 .498 0 r
.75253 .75253 Mdot
1 1 0 r
.85354 .75253 Mdot
1 0 0 r
.95455 .75253 Mdot
1 1 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
1 0 0 r
.34848 .85354 Mdot
0 1 0 r
.44949 .85354 Mdot
0 0 .498 r
.55051 .85354 Mdot
.498 .498 .498 r
.65152 .85354 Mdot
1 0 1 r
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
1 1 0 r
.95455 .85354 Mdot
.498 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.498 .498 .498 r
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
0 1 0 r
.44949 .95455 Mdot
1 0 0 r
.55051 .95455 Mdot
.498 0 0 r
.65152 .95455 Mdot
0 1 1 r
.75253 .95455 Mdot
1 0 0 r
.85354 .95455 Mdot
0 .498 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.504983 0.488926 0.726467 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.498 .498 .498 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 1 r
.34848 .04545 Mdot
1 1 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
1 0 1 r
.75253 .04545 Mdot
.498 0 0 r
.85354 .04545 Mdot
1 1 0 r
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
.498 .498 .498 r
.14646 .14646 Mdot
0 0 .498 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
.498 0 0 r
.75253 .14646 Mdot
1 1 0 r
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
0 1 0 r
.14646 .24747 Mdot
0 1 1 r
.24747 .24747 Mdot
0 1 0 r
.34848 .24747 Mdot
1 1 0 r
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
.898 .898 .898 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.498 0 0 r
.85354 .24747 Mdot
1 1 0 r
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
0 0 .498 r
.55051 .34848 Mdot
.498 0 0 r
.65152 .34848 Mdot
.898 .898 .898 r
.75253 .34848 Mdot
.498 0 0 r
.85354 .34848 Mdot
.898 .898 .898 r
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
0 1 0 r
.14646 .44949 Mdot
.898 .898 .898 r
.24747 .44949 Mdot
0 1 0 r
.34848 .44949 Mdot
1 1 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
0 1 0 r
.65152 .44949 Mdot
.898 .898 .898 r
.75253 .44949 Mdot
1 0 1 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
.898 .898 .898 r
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 1 r
.44949 .55051 Mdot
0 1 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.898 .898 .898 r
.85354 .55051 Mdot
1 0 0 r
.95455 .55051 Mdot
1 0 1 r
.04545 .65152 Mdot
.498 .498 .498 r
.14646 .65152 Mdot
.898 .898 .898 r
.24747 .65152 Mdot
0 1 0 r
.34848 .65152 Mdot
0 1 1 r
.44949 .65152 Mdot
0 1 0 r
.55051 .65152 Mdot
.65152 .65152 Mdot
.498 .498 .498 r
.75253 .65152 Mdot
.898 .898 .898 r
.85354 .65152 Mdot
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.498 .498 .498 r
.14646 .75253 Mdot
0 1 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 .498 0 r
.85354 .75253 Mdot
1 1 0 r
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
0 1 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
1 0 1 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
1 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.498 0 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
0 1 0 r
.44949 .95455 Mdot
0 0 .498 r
.55051 .95455 Mdot
.498 .498 .498 r
.65152 .95455 Mdot
1 0 1 r
.75253 .95455 Mdot
0 1 1 r
.85354 .95455 Mdot
1 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.504983 0.732558 0.726467 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.498 .498 .498 r
.14646 .04545 Mdot
.24747 .04545 Mdot
1 1 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
.498 0 0 r
.75253 .04545 Mdot
1 1 0 r
.85354 .04545 Mdot
.498 0 0 r
.95455 .04545 Mdot
.498 .498 .498 r
.04545 .14646 Mdot
.14646 .14646 Mdot
1 1 0 r
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
1 1 0 r
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
.498 .498 .498 r
.14646 .24747 Mdot
0 1 1 r
.24747 .24747 Mdot
1 1 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.898 .898 .898 r
.65152 .24747 Mdot
.75253 .24747 Mdot
1 1 0 r
.85354 .24747 Mdot
.498 0 0 r
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
0 1 0 r
.14646 .34848 Mdot
0 1 1 r
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 1 r
.55051 .34848 Mdot
.898 .898 .898 r
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.498 0 0 r
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.898 .898 .898 r
.24747 .44949 Mdot
1 1 0 r
.34848 .44949 Mdot
0 1 0 r
.44949 .44949 Mdot
1 1 0 r
.55051 .44949 Mdot
.498 0 0 r
.65152 .44949 Mdot
.898 .898 .898 r
.75253 .44949 Mdot
.498 0 0 r
.85354 .44949 Mdot
.898 .898 .898 r
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.898 .898 .898 r
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 0 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.898 .898 .898 r
.85354 .55051 Mdot
1 0 0 r
.95455 .55051 Mdot
1 0 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.898 .898 .898 r
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
0 1 0 r
.65152 .65152 Mdot
1 0 1 r
.75253 .65152 Mdot
0 .498 0 r
.85354 .65152 Mdot
.898 .898 .898 r
.95455 .65152 Mdot
1 0 1 r
.04545 .75253 Mdot
1 1 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
0 1 0 r
.34848 .75253 Mdot
0 1 1 r
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
1 0 1 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
0 .498 0 r
.95455 .75253 Mdot
1 1 0 r
.04545 .85354 Mdot
.498 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
1 1 0 r
.34848 .85354 Mdot
0 1 0 r
.44949 .85354 Mdot
.55051 .85354 Mdot
1 0 1 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.498 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
1 0 1 r
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 1 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.504983 0.97619 0.726467 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 0 1 r
.07 w
.04545 .04545 Mdot
.498 .498 .498 r
.14646 .04545 Mdot
1 1 0 r
.24747 .04545 Mdot
0 1 1 r
.34848 .04545 Mdot
.44949 .04545 Mdot
1 1 0 r
.55051 .04545 Mdot
0 1 1 r
.65152 .04545 Mdot
1 1 0 r
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 0 1 r
.04545 .14646 Mdot
1 1 0 r
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
1 1 0 r
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 0 1 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
.24747 .24747 Mdot
1 1 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
1 1 0 r
.75253 .24747 Mdot
.898 .898 .898 r
.85354 .24747 Mdot
1 1 0 r
.95455 .24747 Mdot
1 0 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
0 1 1 r
.24747 .34848 Mdot
1 1 0 r
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
1 1 0 r
.55051 .34848 Mdot
.498 0 0 r
.65152 .34848 Mdot
.898 .898 .898 r
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
1 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 1 1 r
.34848 .44949 Mdot
1 1 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.898 .898 .898 r
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
1 0 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.44949 .55051 Mdot
1 1 0 r
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.898 .898 .898 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
1 0 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
1 0 1 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.898 .898 .898 r
.95455 .65152 Mdot
1 0 1 r
.04545 .75253 Mdot
.498 0 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
0 1 1 r
.34848 .75253 Mdot
.44949 .75253 Mdot
1 0 1 r
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
1 1 0 r
.04545 .85354 Mdot
.498 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
1 1 0 r
.44949 .85354 Mdot
1 0 1 r
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
1 1 0 r
.04545 .95455 Mdot
.498 0 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
0 1 0 r
.44949 .95455 Mdot
1 0 1 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[128]:=",
  ImageSize->{439.875, 327.375},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006g0001Ab000`400?l00000o`00003ooooooonh
ooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonh
ooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`003Oooool4
ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`06ooooo`Coool01Oooool4
o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?l0ool6ooooo`Coool01_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co0?oo1_ooool4
o`3oo`Kooooo1?l0ool>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
oooo00Gooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo3_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Co0?oo1_ooool4
o`3oo`Kooooo1?l0ool6ooooo`Coo`3o1Oooool4o`3oo`kooooo000<ooooo`Koo`3o0oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_ooo`03ooooo`Ko0?oo1?ooool6o`3o
o`Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3o
o`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3o
o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo
00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3o
o`Cooooo1_oo0?l3ooooo`Ko0?oo3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koool01?l00006oooo00?o00001_l0ool4o`0000Ko0?oo1?l00006o`3oo`Co
00001_l0ool<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Ko0?oo1?l00006oooo00Co
00001_l0ool3o`0000Ko0?oo1?l00006o`3oo`Co00001_l0ool4o`0000Ko0?oo3?ooool6ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_ooo`03o`0000Koool01?l00006o`3oo`Co
00001_l0ool4o`0000Ko0?oo1?l00006o`3oo`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l3o`0000Koo`3o1?l00006o`3oo`Co00001_l0ool4o`0000Ko0?oo1?l00006ool0o`?o
00001_l0ool=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_ooo`04ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`?ooooo
1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6oooo00?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`Cooooo
1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo
1_oo0?l4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Koo`3o0oooool6o`3oo`gooooo
000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_oo
o`03ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_l0
ool4ooooo`Ko0?oo1?ooool6o`3oo`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koool00oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0
ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_l0
ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_oo0?l3ooooo`Ko0?oo3Oooool000gooooo1?oo0?l5
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coool01_ooool4oooo00Gooooo1?l0ool6
ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Co0?oo1_ooool4oooo00Kooooo1?l0ool5ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6
ooooo`Co0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`05
ooooo`Coool01_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?l0ool6
ooooo`Co0?oo1_ooool4ool0o`Gooooo1?l0ool>ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4
o`3oo`Kooooo1?ooo`06ooooo`Coool01Oooool4oooo00Kooooo1?l0ool6ooooo`Co0?oo1_ooool4
o`3oo`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4
oooo00Gooooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool>ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4oooo00Gooooo1?ooo`06ooooo`Coool01_ooool4
o`3oo`Kooooo1?oo0?l6ooooo`Co0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Gooooo1?oo0?l6ooooo`Coool01_ooool4o`3oo`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4
ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo
00Cooooo1_ooo`03ooooo`Koool01?ooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koool00oooool6o`3o
o`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6ool0
o`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0
o`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000co
oooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_l0ool4o`0000Koool01?l00006oooo00?o
00001_ooo`04o`0000Ko0?oo1?l00006o`3oo`Co00001_l0ool<ooooo`Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006o`3oo`Co00001_ooo`03o`0000Ko0?oo1?l00006o`3oo`Co
00001_l0ool4o`0000Ko0?oo3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_ooo`03o`0000Koool01?l00006oooo00Co00001_l0ool4o`0000Koo`3o1?l00006o`3oo`co
oooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006oooo00Co
00001_l0ool4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo
1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo
1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Ko0?oo1?ooool6oooo00?ooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo
1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00?ooooo
1_ooo`04ooooo`Koool01?ooool6o`3oo`Cooooo1_oo0?l4ooooo`Ko0?oo3?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koool01?ooool6o`3oo`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo
0?l4ooooo`Ko0?oo1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool01?ooool6o`3oo`Cooooo1_l0
ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0
ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo00Cooooo1_oo
o`04ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_oo
0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool6
ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool>
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?ooo`05
ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo3_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`05ooooo`Coool01_ooool4oooo00Kooooo1?l0ool6
ooooo`Coo`3o1_ooool4o`3oo`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5
ooooo`Coo`3o1_ooool4oooo00Kooooo1?l0ool6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4
ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool6ooooo`Coool01Oooool4
oooo00Kooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Gooooo1?oo0?l6ooooo`Co0?oo1_ooool4
ool0o`Kooooo1?l0ool>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Gooooo1?ooo`06ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_ooo`03ooooo`Koool01?ooool6oooo
00Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o1?ooool6o`3o
o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6oooo
00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Ko0?oo1?l00006oooo00?o00001_ooo`04o`0000Koool01?l00006o`3oo`Co
00001_l0ool<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_l0ool3o`0000Koo`3o1?l00006o`3oo`Co00001_oo0?l4o`0000Ko0?oo3?ooool6ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koool01?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o
00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_l0ool4ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`?ooooo
1_oo0?l4ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_ooo`04ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo
000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_oo
o`03ooooo`Koool01?ooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6ool0o`Cooooo1_l0
ool4ooooo`Koo`3o1?ooool6o`3oo`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o0oooool6oooo00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4oooo00Gooooo1?ooo`06
ooooo`Coool01_ooool4o`3oo`Kooooo1?l0ool>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool5ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?oo0?l6
ooooo`Co0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5
ooooo`Coool01_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4
ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000co
oooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`co
oooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo
0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool0
00cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l=ooooo`003?ooool6ool0o`?o
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o0ol00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co
00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co
00001_oo0?l4o`0000Koo`3o0ol00006ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Ooo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`go
oooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool0
00cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Ooo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`go
oooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool0
00cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Ooo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Ooo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_oo
ool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0
o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0
o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?o
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`go
oooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo
1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo
1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo
0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`00oooooonhooooo`00oooooonhoooo
o`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhoooo
o`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhoooo
o`003Oooool4ool0o`Gooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co
ool01Oooool4o`3oo`Kooooo1?l0ool6ooooo`Coool01_ooool4oooo00kooooo1?oo0?l6ooooo`Co
0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Co
0?oo1_ooool4o`3oo`Kooooo1?ooo`0>ooooo`Co0?oo1_ooool4ool0o`Kooooo1?l0ool6ooooo`Co
ool01_ooool4o`3oo`Gooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co
0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4oooo00Gooooo1?ooo`06ooooo`Co
ool01_ooool4oooo00Kooooo1?l0ool6ooooo`Co0?oo1Oooool4o`3oo`kooooo000<ooooo`Koo`3o
0oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`03ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool03?ooool6ool0o`Cooooo1_l0ool4ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo
1?ooool6oooo00cooooo1_l0ool4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo
0oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Koool0
1?ooool6o`3oo`Cooooo1_l0ool3ooooo`Ko0?oo3Oooool000cooooo1_oo0?l3o`0000Koool01?l0
0006o`3oo`Co00001_l0ool4o`0000Ko0?oo1?l00006oooo00?o00001_l0ool4o`0000Ko0?oo1?l0
0006oooo00Co00001_ooo`0<ooooo`Koo`3o1?l00006o`3oo`Co00001_l0ool4o`0000Ko0?oo1?l0
0006oooo00Co00001_ooo`03o`0000Koool01?l00006o`3oo`Co00001_l0ool4o`0000Koool03?oo
ool6o`3oo`Co00001_oo0?l4o`0000Ko0?oo1?l00006oooo00Co00001_l0ool3o`0000Koool01?l0
0006o`3oo`Co00001_l0ool4o`0000Ko0?oo1?l00006o`3oo`cooooo1_oo0?l4o`0000Koo`3o1?l0
0006ool0o`Co00001_ooo`03o`0000Koool01?l00006oooo00Co00001_ooo`04o`0000Ko0?oo1?l0
0006o`3oo`?o00001_l0ool=ooooo`003?ooool6ool0o`?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6
o`3oo`Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Koool01?ooool6
oooo00cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Koool01?ooool6
oooo00?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`0<ooooo`Ko0?oo1?ooool6
ool0o`Cooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6
o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
oooo00?ooooo1_ooo`04ooooo`Koool01?ooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo0oooool6
o`3oo`gooooo000<ooooo`Koo`3o0oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3o
o`Cooooo1_ooo`03ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool03?ooool6ool0
o`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo
00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6oooo00cooooo1_l0ool4ooooo`Koo`3o1?ooool6o`3o
o`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3o
o`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo
00Cooooo1_ooo`04ooooo`Koool01?ooool6o`3oo`Cooooo1_l0ool3ooooo`Ko0?oo3Oooool000go
oooo1?oo0?l5ooooo`Coool01_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Go
oooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4o`3oo`Ko
oooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Kooooo1?ooo`05ooooo`Coool01_ooool4o`3oo`Ko
oooo1?l0ool6ooooo`Coool03_ooool4o`3oo`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4oooo00Ko
oooo1?l0ool5ooooo`Coool01_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`05ooooo`Coool01_ooool4oooo00Ko
oooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`Gooooo1?l0ool>ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?l0ool6ooooo`Co
0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1Oooool4o`3oo`Kooooo1?l0ool6ooooo`Co
ool01_ooool4oooo00kooooo1?l0ool6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?ooo`06ooooo`Co
0?oo1_ooool4oooo00Gooooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?ooo`0>ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coool01_ooool4oooo00Gooooo1?l0ool6ooooo`Co
ool01_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co
o`3o1_ooool4oooo00Gooooo1?ooo`06ooooo`Coool01_ooool4oooo00Kooooo1?ooo`06ooooo`Co
0?oo1Oooool4o`3oo`kooooo000<ooooo`Koo`3o0oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_l0ool3ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool0
3?ooool6o`3oo`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo00Cooooo1_l0ool4ooooo`Koool0
0oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6oooo00cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_ooo`04ooooo`Koool00oooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool0
0oooool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6oooo00Cooooo1_l0ool3ooooo`Ko0?oo
3Oooool000cooooo1_oo0?l3o`0000Ko0?oo1?l00006o`3oo`Co00001_l0ool4o`0000Ko0?oo1?l0
0006o`3oo`?o00001_l0ool4o`0000Ko0?oo1?l00006oooo00Co00001_ooo`0<ooooo`Ko0?oo1?l0
0006ool0o`Co00001_l0ool4o`0000Koool01?l00006o`3oo`Co00001_ooo`03o`0000Ko0?oo1?l0
0006o`3oo`Co00001_l0ool4o`0000Koool03?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l0
0006oooo00Co00001_ooo`03o`0000Ko0?oo1?l00006oooo00Co00001_l0ool4o`0000Ko0?oo1?l0
0006o`3oo`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_ooo`03o`0000Koool01?l0
0006oooo00Co00001_ooo`04o`0000Koool01?l00006o`3oo`?o00001_l0ool=ooooo`003?ooool6
ool0o`?ooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo0oooool6
o`3oo`Cooooo1_l0ool4ooooo`Koool01?ooool6oooo00cooooo1_l0ool4ooooo`Koo`3o1?ooool6
o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00?ooooo1_l0ool4ooooo`Ko0?oo1?ooool6
o`3oo`Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool01?ooool6
oooo00?ooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00?ooooo1_ooo`04ooooo`Koool01?ooool6
oooo00Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`gooooo000<ooooo`Koo`3o0oooool6o`3o
o`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool3ooooo`Ko0?oo1?ooool6o`3o
o`Cooooo1_ooo`04ooooo`Koool03?ooool6o`3oo`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo
00Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6oooo
00cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_ooo`04ooooo`Koool00oooool6o`3o
o`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6oooo
00Cooooo1_l0ool3ooooo`Ko0?oo3Oooool000gooooo1?oo0?l5ooooo`Co0?oo1_ooool4o`3oo`Ko
oooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Ko
oooo1?ooo`0>ooooo`Co0?oo1_ooool4ool0o`Kooooo1?l0ool6ooooo`Coool01_ooool4o`3oo`Ko
oooo1?ooo`05ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Coool03_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4oooo00Kooooo1?ooo`05ooooo`Co0?oo1_ooool4oooo00Ko
oooo1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Ko
oooo1?ooo`05ooooo`Coool01_ooool4oooo00Kooooo1?ooo`06ooooo`Coool01_ooool4o`3oo`Go
oooo1?l0ool>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003Oooool4o`3oo`Gooooo1?oo0?l6ooooo`Co0?oo1_ooool4oooo00Kooooo1?l0ool6ooooo`Co
ool01Oooool4o`3oo`Kooooo1?l0ool6ooooo`Coool01_ooool4oooo00kooooo1?oo0?l6ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?ooo`06ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Co
0?oo1_ooool4o`3oo`Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co
ool01_ooool4o`3oo`Gooooo1?ooo`06ooooo`Coool01_ooool4oooo00Kooooo1?l0ool6ooooo`Co
0?oo3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Gooooo1?ooo`06ooooo`Co
ool01_ooool4oooo00Kooooo1?ooo`06ooooo`Coool01Oooool4o`3oo`kooooo000<ooooo`Ko0?oo
0oooool6ool0o`Cooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`Cooooo1_ooo`03ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool03?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo1_l0ool4ooooo`Ko0?oo
1?ooool6oooo00cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_ooo`04ooooo`Ko0?oo
0oooool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6o`3oo`Cooooo1_l0ool<ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6oooo00Cooooo1_ooo`04ooooo`Koool0
1?ooool6oooo00Cooooo1_ooo`03ooooo`Ko0?oo3Oooool000cooooo1_l0ool3o`0000Koo`3o1?l0
0006o`3oo`Co00001_ooo`04o`0000Ko0?oo1?l00006oooo00?o00001_l0ool4o`0000Ko0?oo1?l0
0006oooo00Co00001_ooo`0<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koool01?l0
0006oooo00Co00001_ooo`03o`0000Koool01?l00006o`3oo`Co00001_l0ool4o`0000Koool03?oo
ool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006oooo00Co00001_l0ool3o`0000Koool01?l0
0006oooo00Co00001_ooo`04o`0000Ko0?oo1?l00006o`3oo`cooooo1_oo0?l4o`0000Koo`3o1?l0
0006ool0o`Co00001_l0ool3o`0000Koool01?l00006oooo00Co00001_ooo`04o`0000Koool01?l0
0006oooo00?o00001_l0ool=ooooo`003?ooool6o`3oo`?ooooo1_oo0?l4ooooo`Ko0?oo1?ooool6
oooo00Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Koool01?ooool6
oooo00cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_ooo`04ooooo`Koool01?ooool6
oooo00?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koool01?ooool6o`3oo`?ooooo1_ooo`04ooooo`Koool01?ooool6
oooo00Cooooo1_l0ool4ooooo`Ko0?oo3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
o`3oo`?ooooo1_ooo`04ooooo`Koool01?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6
o`3oo`gooooo000<ooooo`Ko0?oo0oooool6ool0o`Cooooo1_l0ool4ooooo`Koool01?ooool6o`3o
o`Cooooo1_ooo`03ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool03?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6oooo
00Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6oooo00cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6o`3o
o`Cooooo1_l0ool<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6oooo
00Cooooo1_ooo`04ooooo`Koool01?ooool6oooo00Cooooo1_ooo`03ooooo`Ko0?oo3Oooool000go
oooo1?l0ool5ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?ooo`06ooooo`Co0?oo1_ooool4oooo00Go
oooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coool01_ooool4oooo00Kooooo1?ooo`05ooooo`Coool01_ooool4o`3oo`Ko
oooo1?l0ool6ooooo`Coool03_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4oooo00Ko
oooo1?l0ool5ooooo`Coool01_ooool4oooo00Kooooo1?ooo`06ooooo`Co0?oo1_ooool4o`3oo`ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool5ooooo`Coool01_ooool4oooo00Ko
oooo1?ooo`06ooooo`Coool01_ooool4oooo00Gooooo1?l0ool>ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Co
o`3o1_ooool4oooo00Kooooo1?ooo`06ooooo`Coool01Oooool4oooo00Kooooo1?mo0006ooooo`Co
iNGU1_ooool4onGUi@kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`06ooooo`Co
0?oo1_ooool4oooo00Gooooo1?l0ool6ooooo`Co0?oo1_ooool4ogl000Kooooo1?oUiND>ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co
ool01_ooool4o`3oo`Kooooo1?l0ool6ooooo`CoO`003_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co
o`3o1_ooool4ool0o`Gooooo1?l0ool6ooooo`Coo`3o1_ooool4oooo00Kooooo1?oo0?l6ooooo`Co
o`3o1Oooool4o`3oo`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koool0
1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool01?ooool6ogl000Cooooo1_oUiND4ooooo`KoiNGU
3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_l0ool4ooooo`Koool0
0oooool6o`3oo`Cooooo1_l0ool4ooooo`KoO`001?ooool6onGUi@cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_l0ool4ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo
1?ooool6o`3oo`Cooooo1_mo000<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o
0oooool6o`3oo`Cooooo1_oo0?l4ooooo`Koool01?ooool6ool0o`Cooooo1_oo0?l3ooooo`Ko0?oo
3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_ooo`04o`0000Koool01?l0
0006oooo00?o00001_ooo`04o`0000KoO`001?l00006onGUi@Co00001_oUiND<ooooo`Koo`3o1?l0
0006ool0o`Co00001_oo0?l4o`0000Koool01?l00006o`3oo`Co00001_ooo`03o`0000Ko0?oo1?l0
0006o`3oo`Co00001_mo0004o`0000KoiNGU3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l0
0006o`3oo`Co00001_l0ool3o`0000Ko0?oo1?l00006oooo00Co00001_l0ool4o`0000Ko0?oo1?l0
0006ogl000cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Ko0?oo1?l0
0006ool0o`Co00001_ooo`04o`0000Koo`3o1?l00006ool0o`?o00001_l0ool=ooooo`003?ooool6
ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6
oooo00Cooooo1_mo0004ooooo`KoiNGU1?ooool6onGUi@cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00?ooooo1_l0ool4ooooo`Ko0?oo1?ooool6
ogl000Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6
o`3oo`?ooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`Cooooo1_l0ool4ooooo`KoO`003?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_l0ool4ooooo`Koo`3o1?ooool6
oooo00Cooooo1_oo0?l4ooooo`Koo`3o0oooool6o`3oo`gooooo000<ooooo`Koo`3o0oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koool01?ooool6oooo00Cooooo1_ooo`03ooooo`Koool01?ooool6ogl0
00Cooooo1_oUiND4ooooo`KoiNGU3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo
00Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`KoO`001?ooool6onGU
i@cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Ko0?oo0oooool6o`3o
o`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_mo000<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6o`3oo`Cooooo1_oo0?l4ooooo`Koool01?ooool6ool0
o`Cooooo1_oo0?l3ooooo`Ko0?oo3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Ko
oooo1?ooo`06ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`CoO`001_ooool4onGUi@Ko
oooo1?oUiND>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coool01_ooool4o`3oo`Ko
oooo1?ooo`05ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?mo0006ooooo`CoiNGU3_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?l0ool5ooooo`Co0?oo1_ooool4oooo00Ko
oooo1?l0ool6ooooo`Co0?oo1_ooool4ogl000kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Ko
oooo1?oo0?l5ooooo`Co0?oo1_ooool4ool0o`Kooooo1?ooo`06ooooo`Coo`3o1_ooool4ool0o`Go
oooo1?l0ool>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?ooo`06ooooo`Coool01Oooool4oooo00Kooooo1?ooo`06oooo
o`CoiNGU1_ooool4onGUi@kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4oooo00Gooooo1?ooo`06ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oUiND>oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Gooooo1?oo0?l6oooo
o`Co0?oo1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool01?ooool6oooo00Cooooo1_oUiND4ooooo`Ko
iNGU3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
ool00oooool6oooo00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6onGUi@cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6ool0o`Cooooo1_l0ool4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Ko
o`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koool0
1?l00006oooo00?o00001_ooo`04o`0000Koool01?l00006onGUi@Co00001_oUiND<ooooo`Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_ooo`03o`0000Koool0
1?l00006ool0o`Co00001_oo0?l4o`0000KoiNGU3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_l0ool3o`0000Koo`3o1?l00006o`3oo`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_ooo`04ooooo`Koool00ooo
ool6oooo00Cooooo1_ooo`04ooooo`KoiNGU1?ooool6onGUi@cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00?ooooo1_ooo`04ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6o`3oo`?ooooo1_oo0?l4ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool01?ooool6
oooo00Cooooo1_oUiND4ooooo`KoiNGU3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koool00oooool6oooo00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
onGUi@cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6
ool0o`Cooooo1_l0ool4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Coool01_ooool4onGU
i@Kooooo1?oUiND>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?ooo`05ooooo`Coool01_ooool4ool0o`Kooooo1?oo0?l6ooooo`CoiNGU3_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool5ooooo`Coo`3o1_ooool4o`3o
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?ooo`06oooo
o`Coool01Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4onGUi@kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00Cooooo1_ooo`03ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`KoiNGU3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koool01?l00006oooo00?o00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oUiND<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006o`3oo`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_ooo`04ooooo`Koool00oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6onGUi@cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0ooo
ool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
oooo00Cooooo1_ooo`03ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`KoiNGU3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool0
00gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coool01_ooool4oooo
00Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oUiND>ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?l0ool6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6o`3oo`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Ko
o`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Ko0?oo
1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o0ooo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003Oooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?oo0?l6oooo
o`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_l0ool4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6o`3oo`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0ooo
ool6ool0o`gooooo000<ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool0
00gooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool6ooooo`Coo`3o1_ooool4ool0
o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ogl000Gooooo1?mo0006oooo
o`CoO`001_ooool4o`3oo`Kooooo1?l0ool6ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6oooo
o`Coo`3o1Oooool4ool0o`kooooo000<ooooo`KoO`000oooool6ogl000Cooooo1_mo0004ooooo`Ko
0?oo1?ooool6o`3oo`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko
o`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Ko
o`3o3Oooool000cooooo1_mo0003o`0000KoO`001?l00006ogl000Co00001_l0ool4o`0000Ko0?oo
1?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Ko0?oo1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o
1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Koo`3o
1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?oo
ool6ogl000?ooooo1_mo0004ooooo`KoO`001?ooool6o`3oo`Cooooo1_l0ool4ooooo`Koo`3o0ooo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`KoO`000oooool6
ogl000Cooooo1_mo0004ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
o`3oo`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?mo0005ooooo`CoO`001_ooool4ogl0
00Kooooo1?l0ool6ooooo`Co0?oo1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0
o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ogl000Gooooo1?mo0006
ooooo`CoO`001_ooool4ogl000Kooooo1?ooo`06ooooo`Coo`3o1Oooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`kooooo1?mo0006ooooo`CoO`001_ooool4ogl000Kooooo1?l0ool6
ooooo`Co0?oo1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l>
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4ool0o`Gooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6
ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`KoO`000oooool6ogl000Cooooo1_mo0004oooo
o`KoO`001?ooool6oooo00Cooooo1_oo0?l3ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4oooo
o`Koo`3o3?ooool6ogl000Cooooo1_mo0004ooooo`KoO`001?ooool6o`3oo`Cooooo1_l0ool4oooo
o`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_oo0?l4oooo
o`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4oooo
o`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4oooo
o`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3oooo
o`Koo`3o3Oooool000cooooo1_mo0003o`0000KoO`001?l00006ogl000Co00001_mo0004o`0000Ko
ool01?l00006ool0o`?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l<ooooo`Ko
O`001?l00006ogl000Co00001_mo0004o`0000Ko0?oo1?l00006o`3oo`Co00001_oo0?l3o`0000Ko
o`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6ool0o`Co00001_oo0?l4o`0000Ko
o`3o1?l00006o`3oo`Co00001_oo0?l3o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Ko
o`3o1?l00006ool0o`cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l3o`0000Ko
o`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`00
3?ooool6ogl000?ooooo1_mo0004ooooo`KoO`001?ooool6ogl000Cooooo1_ooo`04ooooo`Koo`3o
0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_mo0004ooooo`KoO`00
1?ooool6ogl000Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo
1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o
3?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`?ooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`KoO`000ooo
ool6ogl000Cooooo1_mo0004ooooo`KoO`001?ooool6oooo00Cooooo1_oo0?l3ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o3?ooool6ogl000Cooooo1_mo0004ooooo`KoO`001?oo
ool6o`3oo`Cooooo1_l0ool4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koo`3o0ooo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?mo0005ooooo`CoO`001_ooool4
ogl000Kooooo1?mo0006ooooo`Coool01_ooool4ool0o`Gooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l>ooooo`CoO`001_ooool4ogl000Kooooo1?mo0006ooooo`Co0?oo1_ooool4
o`3oo`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o3_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Kooooo1?oo0?l5ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4
ool0o`Gooooo1?oo0?l>ooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooo
oonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooo
oonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`003Ooo
ool4o`3oo`Gooooo1?moOgl6ooooo`Co001o1_ooool4o`00O`Kooooo1?l0ool6ooooo`Coool01Ooo
ool4o`3o00Kooooo1?ooo`06ooooo`Co0?l01_ooool4o`3oo`kooooo1?moOgl6ooooo`CoOgmo1_oo
ool4ogmoO`Kooooo1?l0ool6ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Coo`3o1_oo
ool4ogl000Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4ogmoO`Kooooo1?moOgl6ooooo`Coool01_oo
ool4o`3oo`Gooooo1?ooo`06ooooo`Co0?oo1_ooool4ogl000Kooooo1?ooo`06ooooo`CoO`003_oo
ool4ool0o`Kooooo1?moOgl6ooooo`Coool01_ooool4o`3oo`Gooooo1?l0ool6ooooo`Coool01_oo
ool4o`3oo`Kooooo1?ooo`06ooooo`Coool01Oooool4oooo00kooooo000<ooooo`Ko0?oo0oooool6
ogmoO`Cooooo1_l007l4ooooo`Ko001o1?ooool6o`3oo`Cooooo1_ooo`03ooooo`Ko0?l01?ooool6
oooo00Cooooo1_l0o`04ooooo`Ko0?oo3?ooool6ogmoO`Cooooo1_moOgl4ooooo`KoOgmo1?ooool6
o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo1_oo0?l4ooooo`KoO`001?ooool6
oooo00cooooo1_oo0?l4ooooo`KoOgmo1?ooool6ogmoO`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6
oooo00Cooooo1_l0ool4ooooo`KoO`001?ooool6oooo00Cooooo1_mo000<ooooo`Koo`3o1?ooool6
ogmoO`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6
oooo00Cooooo1_ooo`03ooooo`Koool03Oooool000cooooo1_l0ool3o`0000KoOgmo1?l00006o`00
O`Co00001_l007l4o`0000Ko0?oo1?l00006oooo00?o00001_l0o`04o`0000Koool01?l00006o`3o
00Co00001_l0ool<ooooo`KoOgmo1?l00006ogmoO`Co00001_moOgl4o`0000Ko0?oo1?l00006oooo
00Co00001_ooo`03o`0000Koool01?l00006ool0o`Co00001_mo0004o`0000Koool03?ooool6ool0
o`Co00001_moOgl4o`0000KoOgmo1?l00006oooo00Co00001_l0ool3o`0000Koool01?l00006o`3o
o`Co00001_mo0004o`0000Koool01?l00006ogl000cooooo1_oo0?l4o`0000KoOgmo1?l00006oooo
00Co00001_l0ool3o`0000Ko0?oo1?l00006oooo00Co00001_l0ool4o`0000Koool01?l00006oooo
00?o00001_ooo`0=ooooo`003?ooool6o`3oo`?ooooo1_moOgl4ooooo`Ko001o1?ooool6o`00O`Co
oooo1_l0ool4ooooo`Koool00oooool6o`3o00Cooooo1_ooo`04ooooo`Ko0?l01?ooool6o`3oo`co
oooo1_moOgl4ooooo`KoOgmo1?ooool6ogmoO`Cooooo1_l0ool4ooooo`Koool01?ooool6oooo00?o
oooo1_ooo`04ooooo`Koo`3o1?ooool6ogl000Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6ogmoO`Co
oooo1_moOgl4ooooo`Koool01?ooool6o`3oo`?ooooo1_ooo`04ooooo`Ko0?oo1?ooool6ogl000Co
oooo1_ooo`04ooooo`KoO`003?ooool6ool0o`Cooooo1_moOgl4ooooo`Koool01?ooool6o`3oo`?o
oooo1_l0ool4ooooo`Koool01?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo00go
oooo000<ooooo`Ko0?oo0oooool6ogmoO`Cooooo1_l007l4ooooo`Ko001o1?ooool6o`3oo`Cooooo
1_ooo`03ooooo`Ko0?l01?ooool6oooo00Cooooo1_l0o`04ooooo`Ko0?oo3?ooool6ogmoO`Cooooo
1_moOgl4ooooo`KoOgmo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo
1_oo0?l4ooooo`KoO`001?ooool6oooo00cooooo1_oo0?l4ooooo`KoOgmo1?ooool6ogmoO`Cooooo
1_ooo`04ooooo`Ko0?oo0oooool6oooo00Cooooo1_l0ool4ooooo`KoO`001?ooool6oooo00Cooooo
1_mo000<ooooo`Koo`3o1?ooool6ogmoO`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`Cooooo
1_ooo`04ooooo`Ko0?oo1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool03Oooool000gooooo1?l0
ool5ooooo`CoOgmo1_ooool4o`00O`Kooooo1?l007l6ooooo`Co0?oo1_ooool4oooo00Gooooo1?l0
o`06ooooo`Coool01_ooool4o`3o00Kooooo1?l0ool>ooooo`CoOgmo1_ooool4ogmoO`Kooooo1?mo
Ogl6ooooo`Co0?oo1_ooool4oooo00Kooooo1?ooo`05ooooo`Coool01_ooool4ool0o`Kooooo1?mo
0006ooooo`Coool03_ooool4ool0o`Kooooo1?moOgl6ooooo`CoOgmo1_ooool4oooo00Kooooo1?l0
ool5ooooo`Coool01_ooool4o`3oo`Kooooo1?mo0006ooooo`Coool01_ooool4ogl000kooooo1?oo
0?l6ooooo`CoOgmo1_ooool4oooo00Kooooo1?l0ool5ooooo`Co0?oo1_ooool4oooo00Kooooo1?l0
ool6ooooo`Coool01_ooool4oooo00Gooooo1?ooo`0>ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0=ooooo`003Oooool4onGUi@Gooooo1?moOgl6ooooo`Coo`001_oo
ool4ool000Kooooo1?ooo`06ooooo`Co0?oo1Oooool4o`3o00Kooooo1?oo0?l6ooooo`CoO`001_oo
ool4oooo00kooooo1?oo0?l6ooooo`CoOgmo1_ooool4o`00O`Kooooo1?ooo`06ooooo`Coool01_oo
ool4oooo00Gooooo1?l0ool6ooooo`CoO`001_ooool4oooo00Kooooo1?ooo`0>ooooo`CoOgmo1_oo
ool4ogmoO`Kooooo1?ooo`06ooooo`Coool01_ooool4oooo00Gooooo1?ooo`06ooooo`Co0?oo1_oo
ool4oooo00Kooooo1?ooo`06ooooo`Coool03_ooool4ool0o`Kooooo1?ooo`06ooooo`Co0?oo1_oo
ool4oooo00Gooooo1?ooo`06ooooo`Coool01_ooool4o`3oo`Kooooo1?ooo`06ooooo`Coool01Ooo
ool4oooo00kooooo000<ooooo`KoiNGU0oooool6ogmoO`Cooooo1_oo0004ooooo`Koo`001?ooool6
oooo00Cooooo1_l0ool3ooooo`Ko0?l01?ooool6ool0o`Cooooo1_mo0004ooooo`Koool03?ooool6
ool0o`Cooooo1_moOgl4ooooo`Ko001o1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6
o`3oo`Cooooo1_mo0004ooooo`Koool01?ooool6oooo00cooooo1_moOgl4ooooo`KoOgmo1?ooool6
oooo00Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo1_l0ool4ooooo`Koool01?ooool6
oooo00Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6oooo00Cooooo1_l0ool4ooooo`Koool00oooool6
oooo00Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00Cooooo1_ooo`03ooooo`Koool03Oooool0
00cooooo1_oUiND3o`0000KoOgmo1?l00006ool000Co00001_oo0004o`0000Koool01?l00006o`3o
o`?o00001_l0o`04o`0000Koo`3o1?l00006ogl000Co00001_ooo`0<ooooo`Koo`3o1?l00006ogmo
O`Co00001_l007l4o`0000Koool01?l00006oooo00Co00001_ooo`03o`0000Ko0?oo1?l00006ogl0
00Co00001_ooo`04o`0000Koool03?ooool6ogmoO`Co00001_moOgl4o`0000Koool01?l00006oooo
00Co00001_ooo`03o`0000Koool01?l00006o`3oo`Co00001_ooo`04o`0000Koool01?l00006oooo
00cooooo1_oo0?l4o`0000Koool01?l00006o`3oo`Co00001_ooo`03o`0000Koool01?l00006oooo
00Co00001_l0ool4o`0000Koool01?l00006oooo00?o00001_ooo`0=ooooo`003?ooool6onGUi@?o
oooo1_moOgl4ooooo`Koo`001?ooool6ool000Cooooo1_ooo`04ooooo`Ko0?oo0oooool6o`3o00Co
oooo1_oo0?l4ooooo`KoO`001?ooool6oooo00cooooo1_oo0?l4ooooo`KoOgmo1?ooool6o`00O`Co
oooo1_ooo`04ooooo`Koool01?ooool6oooo00?ooooo1_l0ool4ooooo`KoO`001?ooool6oooo00Co
oooo1_ooo`0<ooooo`KoOgmo1?ooool6ogmoO`Cooooo1_ooo`04ooooo`Koool01?ooool6oooo00?o
oooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool03?ooool6ool0o`Co
oooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00?ooooo1_ooo`04ooooo`Koool01?ooool6o`3oo`Co
oooo1_ooo`04ooooo`Koool00oooool6oooo00gooooo000<ooooo`KoiNGU0oooool6ogmoO`Cooooo
1_oo0004ooooo`Koo`001?ooool6oooo00Cooooo1_l0ool3ooooo`Ko0?l01?ooool6ool0o`Cooooo
1_mo0004ooooo`Koool03?ooool6ool0o`Cooooo1_moOgl4ooooo`Ko001o1?ooool6oooo00Cooooo
1_ooo`04ooooo`Koool00oooool6o`3oo`Cooooo1_mo0004ooooo`Koool01?ooool6oooo00cooooo
1_moOgl4ooooo`KoOgmo1?ooool6oooo00Cooooo1_ooo`04ooooo`Koool00oooool6oooo00Cooooo
1_l0ool4ooooo`Koool01?ooool6oooo00Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6oooo00Cooooo
1_l0ool4ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Ko0?oo1?ooool6oooo00Cooooo
1_ooo`03ooooo`Koool03Oooool000gooooo1?oUiND5ooooo`CoOgmo1_ooool4ool000Kooooo1?oo
0006ooooo`Coool01_ooool4o`3oo`Gooooo1?l0o`06ooooo`Coo`3o1_ooool4ogl000Kooooo1?oo
o`0>ooooo`Coo`3o1_ooool4ogmoO`Kooooo1?l007l6ooooo`Coool01_ooool4oooo00Kooooo1?oo
o`05ooooo`Co0?oo1_ooool4ogl000Kooooo1?ooo`06ooooo`Coool03_ooool4ogmoO`Kooooo1?mo
Ogl6ooooo`Coool01_ooool4oooo00Kooooo1?ooo`05ooooo`Coool01_ooool4o`3oo`Kooooo1?oo
o`06ooooo`Coool01_ooool4oooo00kooooo1?oo0?l6ooooo`Coool01_ooool4o`3oo`Kooooo1?oo
o`05ooooo`Coool01_ooool4oooo00Kooooo1?l0ool6ooooo`Coool01_ooool4oooo00Gooooo1?oo
o`0>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003Ooo
ool4ool0o`Gooooo1?l0ool6ooooo`Coo`001_ooool4o`1o00Kooooo1?l0O`06ooooo`Coo`001Ooo
ool4ool0o`Kooooo1?oUiND6ooooo`Coool01_ooool4oooo00kooooo1?oo0?l6ooooo`Co0?l01_oo
ool4o`3oo`Kooooo1?l0o`06ooooo`Coool01_ooool4o`3oo`Gooooo1?oUiND6ooooo`CoiNGU1_oo
ool4ogl000Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4ogmoO`Kooooo1?l0ool6ooooo`Coool01_oo
ool4o`3oo`Gooooo1?l0ool6ooooo`CoiNGU1_ooool4onGUi@Kooooo1?ooo`06ooooo`CoO`003_oo
ool4ool0o`Kooooo1?l0ool6ooooo`Co0?oo1_ooool4oooo00Gooooo1?l0ool6ooooo`Co0?oo1_oo
ool4o`3oo`Kooooo1?ooo`06ooooo`CoiNGU1Oooool4oooo00kooooo000<ooooo`Koo`3o0oooool6
o`3oo`Cooooo1_oo0004ooooo`Ko07l01?ooool6o`1o00Cooooo1_oo0003ooooo`Koo`3o1?ooool6
onGUi@Cooooo1_ooo`04ooooo`Koool03?ooool6ool0o`Cooooo1_l0o`04ooooo`Ko0?oo1?ooool6
o`3o00Cooooo1_ooo`04ooooo`Ko0?oo0oooool6onGUi@Cooooo1_oUiND4ooooo`KoO`001?ooool6
oooo00cooooo1_oo0?l4ooooo`KoOgmo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo0oooool6
o`3oo`Cooooo1_oUiND4ooooo`KoiNGU1?ooool6oooo00Cooooo1_mo000<ooooo`Koo`3o1?ooool6
o`3oo`Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6
oooo00Cooooo1_oUiND3ooooo`Koool03Oooool000cooooo1_oo0?l3o`0000Ko0?oo1?l00006ool0
00Co00001_l0O`04o`0000Ko07l01?l00006ool000?o00001_oo0?l4o`0000KoiNGU1?l00006oooo
00Co00001_ooo`0<ooooo`Koo`3o1?l00006o`3o00Co00001_l0ool4o`0000Ko0?l01?l00006oooo
00Co00001_l0ool3o`0000KoiNGU1?l00006onGUi@Co00001_mo0004o`0000Koool03?ooool6ool0
o`Co00001_moOgl4o`0000Ko0?oo1?l00006oooo00Co00001_l0ool3o`0000Ko0?oo1?l00006onGU
i@Co00001_oUiND4o`0000Koool01?l00006ogl000cooooo1_oo0?l4o`0000Ko0?oo1?l00006o`3o
o`Co00001_ooo`03o`0000Ko0?oo1?l00006o`3oo`Co00001_l0ool4o`0000Koool01?l00006onGU
i@?o00001_ooo`0=ooooo`003?ooool6ool0o`?ooooo1_l0ool4ooooo`Koo`001?ooool6o`1o00Co
oooo1_l0O`04ooooo`Koo`000oooool6ool0o`Cooooo1_oUiND4ooooo`Koool01?ooool6oooo00co
oooo1_oo0?l4ooooo`Ko0?l01?ooool6o`3oo`Cooooo1_l0o`04ooooo`Koool01?ooool6o`3oo`?o
oooo1_oUiND4ooooo`KoiNGU1?ooool6ogl000Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6ogmoO`Co
oooo1_l0ool4ooooo`Koool01?ooool6o`3oo`?ooooo1_l0ool4ooooo`KoiNGU1?ooool6onGUi@Co
oooo1_ooo`04ooooo`KoO`003?ooool6ool0o`Cooooo1_l0ool4ooooo`Ko0?oo1?ooool6oooo00?o
oooo1_l0ool4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_ooo`04ooooo`KoiNGU0oooool6oooo00go
oooo000<ooooo`Koo`3o0oooool6o`3oo`Cooooo1_oo0004ooooo`Ko07l01?ooool6o`1o00Cooooo
1_oo0003ooooo`Koo`3o1?ooool6onGUi@Cooooo1_ooo`04ooooo`Koool03?ooool6ool0o`Cooooo
1_l0o`04ooooo`Ko0?oo1?ooool6o`3o00Cooooo1_ooo`04ooooo`Ko0?oo0oooool6onGUi@Cooooo
1_oUiND4ooooo`KoO`001?ooool6oooo00cooooo1_oo0?l4ooooo`KoOgmo1?ooool6o`3oo`Cooooo
1_ooo`04ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_oUiND4ooooo`KoiNGU1?ooool6oooo00Cooooo
1_mo000<ooooo`Koo`3o1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo
1_l0ool4ooooo`Ko0?oo1?ooool6oooo00Cooooo1_oUiND3ooooo`Koool03Oooool000gooooo1?oo
0?l5ooooo`Co0?oo1_ooool4ool000Kooooo1?l0O`06ooooo`Co07l01_ooool4ool000Gooooo1?oo
0?l6ooooo`CoiNGU1_ooool4oooo00Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4o`3o00Kooooo1?l0
ool6ooooo`Co0?l01_ooool4oooo00Kooooo1?l0ool5ooooo`CoiNGU1_ooool4onGUi@Kooooo1?mo
0006ooooo`Coool03_ooool4ool0o`Kooooo1?moOgl6ooooo`Co0?oo1_ooool4oooo00Kooooo1?l0
ool5ooooo`Co0?oo1_ooool4onGUi@Kooooo1?oUiND6ooooo`Coool01_ooool4ogl000kooooo1?oo
0?l6ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?ooo`05ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?l0
ool6ooooo`Coool01_ooool4onGUi@Gooooo1?ooo`0>ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3Oooool4ool0o`Gooooo1?l0o`06ooooo`Co001o1_ooool4o`3o00Kooooo1?l0o`06ooooo`CoO`00
1Oooool4o`00O`Kooooo1?mo0006ooooo`CoOgmo1_ooool4onGUi@kooooo1?l0o`06ooooo`Co0?l0
1_ooool4o`3o00Kooooo1?l0o`06ooooo`Co0?l01_ooool4o`00O`Gooooo1?mo0006ooooo`CoiNGU
1_ooool4ogl000Kooooo1?oUiND>ooooo`Coo`3o1_ooool4o`3o00Kooooo1?l0ool6ooooo`Co0?l0
1_ooool4o`3o00Gooooo1?l0ool6ooooo`CoiNGU1_ooool4onGUi@Kooooo1?oUiND6ooooo`CoO`00
3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo1_ooool4oooo00Gooooo1?l0ool6ooooo`Coool0
1_ooool4ogl000Kooooo1?oUiND6ooooo`CoiNGU1Oooool4onGUi@kooooo000<ooooo`Koo`3o0ooo
ool6o`3o00Cooooo1_l007l4ooooo`Ko0?l01?ooool6o`3o00Cooooo1_mo0003ooooo`Ko001o1?oo
ool6ogl000Cooooo1_moOgl4ooooo`KoiNGU3?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l01?oo
ool6o`3o00Cooooo1_l0o`04ooooo`Ko001o0oooool6ogl000Cooooo1_oUiND4ooooo`KoO`001?oo
ool6onGUi@cooooo1_oo0?l4ooooo`Ko0?l01?ooool6o`3oo`Cooooo1_l0o`04ooooo`Ko0?l00ooo
ool6o`3oo`Cooooo1_oUiND4ooooo`KoiNGU1?ooool6onGUi@Cooooo1_mo000<ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Cooooo1_ooo`04ooooo`KoO`001?oo
ool6onGUi@Cooooo1_oUiND3ooooo`KoiNGU3Oooool000cooooo1_oo0?l3o`0000Ko0?l01?l00006
o`00O`Co00001_l0o`04o`0000Ko0?l01?l00006ogl000?o00001_l007l4o`0000KoO`001?l00006
ogmoO`Co00001_oUiND<ooooo`Ko0?l01?l00006o`3o00Co00001_l0o`04o`0000Ko0?l01?l00006
o`3o00Co00001_l007l3o`0000KoO`001?l00006onGUi@Co00001_mo0004o`0000KoiNGU3?ooool6
ool0o`Co00001_l0o`04o`0000Ko0?oo1?l00006o`3o00Co00001_l0o`03o`0000Ko0?oo1?l00006
onGUi@Co00001_oUiND4o`0000KoiNGU1?l00006ogl000cooooo1_oo0?l4o`0000Koo`3o1?l00006
o`3oo`Co00001_ooo`03o`0000Ko0?oo1?l00006oooo00Co00001_mo0004o`0000KoiNGU1?l00006
onGUi@?o00001_oUiND=ooooo`003?ooool6ool0o`?ooooo1_l0o`04ooooo`Ko001o1?ooool6o`3o
00Cooooo1_l0o`04ooooo`KoO`000oooool6o`00O`Cooooo1_mo0004ooooo`KoOgmo1?ooool6onGU
i@cooooo1_l0o`04ooooo`Ko0?l01?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l01?ooool6o`00
O`?ooooo1_mo0004ooooo`KoiNGU1?ooool6ogl000Cooooo1_oUiND<ooooo`Koo`3o1?ooool6o`3o
00Cooooo1_l0ool4ooooo`Ko0?l01?ooool6o`3o00?ooooo1_l0ool4ooooo`KoiNGU1?ooool6onGU
i@Cooooo1_oUiND4ooooo`KoO`003?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo
00?ooooo1_l0ool4ooooo`Koool01?ooool6ogl000Cooooo1_oUiND4ooooo`KoiNGU0oooool6onGU
i@gooooo000<ooooo`Koo`3o0oooool6o`3o00Cooooo1_l007l4ooooo`Ko0?l01?ooool6o`3o00Co
oooo1_mo0003ooooo`Ko001o1?ooool6ogl000Cooooo1_moOgl4ooooo`KoiNGU3?ooool6o`3o00Co
oooo1_l0o`04ooooo`Ko0?l01?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko001o0oooool6ogl000Co
oooo1_oUiND4ooooo`KoO`001?ooool6onGUi@cooooo1_oo0?l4ooooo`Ko0?l01?ooool6o`3oo`Co
oooo1_l0o`04ooooo`Ko0?l00oooool6o`3oo`Cooooo1_oUiND4ooooo`KoiNGU1?ooool6onGUi@Co
oooo1_mo000<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool4ooooo`Koool00oooool6o`3oo`Co
oooo1_ooo`04ooooo`KoO`001?ooool6onGUi@Cooooo1_oUiND3ooooo`KoiNGU3Oooool000gooooo
1?oo0?l5ooooo`Co0?l01_ooool4o`00O`Kooooo1?l0o`06ooooo`Co0?l01_ooool4ogl000Gooooo
1?l007l6ooooo`CoO`001_ooool4ogmoO`Kooooo1?oUiND>ooooo`Co0?l01_ooool4o`3o00Kooooo
1?l0o`06ooooo`Co0?l01_ooool4o`3o00Kooooo1?l007l5ooooo`CoO`001_ooool4onGUi@Kooooo
1?mo0006ooooo`CoiNGU3_ooool4ool0o`Kooooo1?l0o`06ooooo`Co0?oo1_ooool4o`3o00Kooooo
1?l0o`05ooooo`Co0?oo1_ooool4onGUi@Kooooo1?oUiND6ooooo`CoiNGU1_ooool4ogl000kooooo
1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?ooo`05ooooo`Co0?oo1_ooool4oooo00Kooooo
1?mo0006ooooo`CoiNGU1_ooool4onGUi@Gooooo1?oUiND>ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4o`3oo`Gooooo1?l007l6ooooo`Co001o
1_ooool4o`1o00Kooooo1?ooo`06ooooo`Coo`001Oooool4o`00O`Kooooo1?oo0006ooooo`Coo`3o
1_ooool4oooo00kooooo1?oo0?l6ooooo`Co0?l01_ooool4onGUi@Kooooo1?l0o`06ooooo`Coool0
1_ooool4oooo00Gooooo1?l0o`06ooooo`CoiNGU1_ooool4ool0o`Kooooo1?ooo`0>ooooo`Coo`3o
1_ooool4ool0o`Kooooo1?oUiND6ooooo`Coool01_ooool4o`3o00Gooooo1?ooo`06ooooo`CoO`00
1_ooool4onGUi@Kooooo1?mo0006ooooo`CoiNGU3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o
1_ooool4o`3oo`Gooooo1?ooo`06ooooo`Coool01_ooool4oooo00Kooooo1?oUiND6ooooo`CoiNGU
1Oooool4onGUi@kooooo000<ooooo`Ko0?oo0oooool6o`00O`Cooooo1_l007l4ooooo`Ko07l01?oo
ool6oooo00Cooooo1_oo0003ooooo`Ko001o1?ooool6ool000Cooooo1_oo0?l4ooooo`Koool03?oo
ool6ool0o`Cooooo1_l0o`04ooooo`KoiNGU1?ooool6o`3o00Cooooo1_ooo`04ooooo`Koool00ooo
ool6o`3o00Cooooo1_oUiND4ooooo`Koo`3o1?ooool6oooo00cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6onGUi@Cooooo1_ooo`04ooooo`Ko0?l00oooool6oooo00Cooooo1_mo0004ooooo`KoiNGU1?oo
ool6ogl000Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0ooo
ool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6onGUi@Cooooo1_oUiND3ooooo`KoiNGU3Ooo
ool000cooooo1_l0ool3o`0000Ko001o1?l00006o`00O`Co00001_l0O`04o`0000Koool01?l00006
ool000?o00001_l007l4o`0000Koo`001?l00006ool0o`Co00001_ooo`0<ooooo`Koo`3o1?l00006
o`3o00Co00001_oUiND4o`0000Ko0?l01?l00006oooo00Co00001_ooo`03o`0000Ko0?l01?l00006
onGUi@Co00001_oo0?l4o`0000Koool03?ooool6ool0o`Co00001_oo0?l4o`0000KoiNGU1?l00006
oooo00Co00001_l0o`03o`0000Koool01?l00006ogl000Co00001_oUiND4o`0000KoO`001?l00006
onGUi@cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_l0ool3o`0000Koool01?l00006
oooo00Co00001_ooo`04o`0000KoiNGU1?l00006onGUi@?o00001_oUiND=ooooo`003?ooool6o`3o
o`?ooooo1_l007l4ooooo`Ko001o1?ooool6o`1o00Cooooo1_ooo`04ooooo`Koo`000oooool6o`00
O`Cooooo1_oo0004ooooo`Koo`3o1?ooool6oooo00cooooo1_oo0?l4ooooo`Ko0?l01?ooool6onGU
i@Cooooo1_l0o`04ooooo`Koool01?ooool6oooo00?ooooo1_l0o`04ooooo`KoiNGU1?ooool6ool0
o`Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oUiND4ooooo`Koool01?ooool6o`3o
00?ooooo1_ooo`04ooooo`KoO`001?ooool6onGUi@Cooooo1_mo0004ooooo`KoiNGU3?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`?ooooo1_ooo`04ooooo`Koool01?ooool6oooo
00Cooooo1_oUiND4ooooo`KoiNGU0oooool6onGUi@gooooo000<ooooo`Ko0?oo0oooool6o`00O`Co
oooo1_l007l4ooooo`Ko07l01?ooool6oooo00Cooooo1_oo0003ooooo`Ko001o1?ooool6ool000Co
oooo1_oo0?l4ooooo`Koool03?ooool6ool0o`Cooooo1_l0o`04ooooo`KoiNGU1?ooool6o`3o00Co
oooo1_ooo`04ooooo`Koool00oooool6o`3o00Cooooo1_oUiND4ooooo`Koo`3o1?ooool6oooo00co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6onGUi@Cooooo1_ooo`04ooooo`Ko0?l00oooool6oooo00Co
oooo1_mo0004ooooo`KoiNGU1?ooool6ogl000Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Ko0?oo0oooool6oooo00Cooooo1_ooo`04ooooo`Koool01?ooool6onGUi@Co
oooo1_oUiND3ooooo`KoiNGU3Oooool000gooooo1?l0ool5ooooo`Co001o1_ooool4o`00O`Kooooo
1?l0O`06ooooo`Coool01_ooool4ool000Gooooo1?l007l6ooooo`Coo`001_ooool4ool0o`Kooooo
1?ooo`0>ooooo`Coo`3o1_ooool4o`3o00Kooooo1?oUiND6ooooo`Co0?l01_ooool4oooo00Kooooo
1?ooo`05ooooo`Co0?l01_ooool4onGUi@Kooooo1?oo0?l6ooooo`Coool03_ooool4ool0o`Kooooo
1?oo0?l6ooooo`CoiNGU1_ooool4oooo00Kooooo1?l0o`05ooooo`Coool01_ooool4ogl000Kooooo
1?oUiND6ooooo`CoO`001_ooool4onGUi@kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo
1?l0ool5ooooo`Coool01_ooool4oooo00Kooooo1?ooo`06ooooo`CoiNGU1_ooool4onGUi@Gooooo
1?oUiND>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3Oooool4ool0o`Gooooo1?l0O`06ooooo`CoiNGU1_ooool4o`00O`Kooooo1?oo0006ooooo`Co07l0
1Oooool4o`3o00Kooooo1?oUiND6ooooo`CoiNGU1_ooool4ool000kooooo1?oo0?l6ooooo`CoiNGU
1_ooool4onGUi@Kooooo1?oUiND6ooooo`Co0?oo1_ooool4o`3o00Gooooo1?l0o`06ooooo`Co0?l0
1_ooool4onGUi@Kooooo1?oo000>ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oUiND6ooooo`Co0?oo
1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co0?l01_ooool4o`3o00Kooooo1?oUiND6ooooo`Coo`00
3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Coool0
1_ooool4o`3oo`Kooooo1?oUiND6ooooo`CoiNGU1Oooool4onGUi@kooooo000<ooooo`Koo`3o0ooo
ool6o`1o00Cooooo1_oUiND4ooooo`Ko001o1?ooool6ool000Cooooo1_l0O`03ooooo`Ko0?l01?oo
ool6onGUi@Cooooo1_oUiND4ooooo`Koo`003?ooool6ool0o`Cooooo1_oUiND4ooooo`KoiNGU1?oo
ool6onGUi@Cooooo1_l0ool4ooooo`Ko0?l00oooool6o`3o00Cooooo1_l0o`04ooooo`KoiNGU1?oo
ool6ool000cooooo1_oo0?l4ooooo`Koo`3o1?ooool6onGUi@Cooooo1_l0ool4ooooo`Ko0?oo0ooo
ool6o`3oo`Cooooo1_l0o`04ooooo`Ko0?l01?ooool6onGUi@Cooooo1_oo000<ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_ooo`04ooooo`Ko0?oo1?oo
ool6onGUi@Cooooo1_oUiND3ooooo`KoiNGU3Oooool000cooooo1_oo0?l3o`0000Ko07l01?l00006
onGUi@Co00001_l007l4o`0000Koo`001?l00006o`1o00?o00001_l0o`04o`0000KoiNGU1?l00006
onGUi@Co00001_oo000<ooooo`Koo`3o1?l00006onGUi@Co00001_oUiND4o`0000KoiNGU1?l00006
o`3oo`Co00001_l0o`03o`0000Ko0?l01?l00006o`3o00Co00001_oUiND4o`0000Koo`003?ooool6
ool0o`Co00001_oo0?l4o`0000KoiNGU1?l00006o`3oo`Co00001_l0ool3o`0000Ko0?oo1?l00006
o`3o00Co00001_l0o`04o`0000KoiNGU1?l00006ool000cooooo1_oo0?l4o`0000Koo`3o1?l00006
ool0o`Co00001_l0ool3o`0000Ko0?oo1?l00006oooo00Co00001_l0ool4o`0000KoiNGU1?l00006
onGUi@?o00001_oUiND=ooooo`003?ooool6ool0o`?ooooo1_l0O`04ooooo`KoiNGU1?ooool6o`00
O`Cooooo1_oo0004ooooo`Ko07l00oooool6o`3o00Cooooo1_oUiND4ooooo`KoiNGU1?ooool6ool0
00cooooo1_oo0?l4ooooo`KoiNGU1?ooool6onGUi@Cooooo1_oUiND4ooooo`Ko0?oo1?ooool6o`3o
00?ooooo1_l0o`04ooooo`Ko0?l01?ooool6onGUi@Cooooo1_oo000<ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oUiND4ooooo`Ko0?oo1?ooool6o`3oo`?ooooo1_l0ool4ooooo`Ko0?l01?ooool6o`3o
00Cooooo1_oUiND4ooooo`Koo`003?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3o
o`?ooooo1_l0ool4ooooo`Koool01?ooool6o`3oo`Cooooo1_oUiND4ooooo`KoiNGU0oooool6onGU
i@gooooo000<ooooo`Koo`3o0oooool6o`1o00Cooooo1_oUiND4ooooo`Ko001o1?ooool6ool000Co
oooo1_l0O`03ooooo`Ko0?l01?ooool6onGUi@Cooooo1_oUiND4ooooo`Koo`003?ooool6ool0o`Co
oooo1_oUiND4ooooo`KoiNGU1?ooool6onGUi@Cooooo1_l0ool4ooooo`Ko0?l00oooool6o`3o00Co
oooo1_l0o`04ooooo`KoiNGU1?ooool6ool000cooooo1_oo0?l4ooooo`Koo`3o1?ooool6onGUi@Co
oooo1_l0ool4ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_l0o`04ooooo`Ko0?l01?ooool6onGUi@Co
oooo1_oo000<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0oooool6o`3oo`Co
oooo1_ooo`04ooooo`Ko0?oo1?ooool6onGUi@Cooooo1_oUiND3ooooo`KoiNGU3Oooool000gooooo
1?oo0?l5ooooo`Co07l01_ooool4onGUi@Kooooo1?l007l6ooooo`Coo`001_ooool4o`1o00Gooooo
1?l0o`06ooooo`CoiNGU1_ooool4onGUi@Kooooo1?oo000>ooooo`Coo`3o1_ooool4onGUi@Kooooo
1?oUiND6ooooo`CoiNGU1_ooool4o`3oo`Kooooo1?l0o`05ooooo`Co0?l01_ooool4o`3o00Kooooo
1?oUiND6ooooo`Coo`003_ooool4ool0o`Kooooo1?oo0?l6ooooo`CoiNGU1_ooool4o`3oo`Kooooo
1?l0ool5ooooo`Co0?oo1_ooool4o`3o00Kooooo1?l0o`06ooooo`CoiNGU1_ooool4ool000kooooo
1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo1?l0ool5ooooo`Co0?oo1_ooool4oooo00Kooooo
1?l0ool6ooooo`CoiNGU1_ooool4onGUi@Gooooo1?oUiND>ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4o`3oo`Gooooo1?l0o`06ooooo`CoOgmo
1_ooool4o`3oo`Kooooo1?l0ool6ooooo`Co0?oo1Oooool4ogmoO`Kooooo1?l007l6ooooo`CoiNGU
1_ooool4ool000kooooo1?oo0?l6ooooo`CoOgmo1_ooool4onGUi@Kooooo1?l0o`06ooooo`Co0?oo
1_ooool4o`3o00Gooooo1?l0o`06ooooo`CoOgmo1_ooool4onGUi@Kooooo1?oUiND>ooooo`Coo`3o
1_ooool4ool0o`Kooooo1?oUiND6ooooo`Co0?oo1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co0?l0
1_ooool4ool0o`Kooooo1?l0O`06ooooo`CoiNGU3_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o
1_ooool4o`3oo`Gooooo1?l0ool6ooooo`Co0?oo1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o
1Oooool4onGUi@kooooo000<ooooo`Ko0?oo0oooool6o`3o00Cooooo1_moOgl4ooooo`Ko0?oo1?oo
ool6o`3oo`Cooooo1_l0ool3ooooo`KoOgmo1?ooool6o`00O`Cooooo1_oUiND4ooooo`Koo`003?oo
ool6ool0o`Cooooo1_moOgl4ooooo`KoiNGU1?ooool6o`3o00Cooooo1_l0ool4ooooo`Ko0?l00ooo
ool6o`3o00Cooooo1_moOgl4ooooo`KoiNGU1?ooool6onGUi@cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6onGUi@Cooooo1_l0ool4ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_l0o`04ooooo`Koo`3o1?oo
ool6o`1o00Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo0ooo
ool6o`3oo`Cooooo1_l0ool4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`KoiNGU3Ooo
ool000cooooo1_l0ool3o`0000Ko0?l01?l00006ogmoO`Co00001_l0ool4o`0000Ko0?oo1?l00006
o`3oo`?o00001_moOgl4o`0000Ko001o1?l00006onGUi@Co00001_oo000<ooooo`Koo`3o1?l00006
ogmoO`Co00001_oUiND4o`0000Ko0?l01?l00006o`3oo`Co00001_l0o`03o`0000Ko0?l01?l00006
ogmoO`Co00001_oUiND4o`0000KoiNGU3?ooool6ool0o`Co00001_oo0?l4o`0000KoiNGU1?l00006
o`3oo`Co00001_l0ool3o`0000Ko0?oo1?l00006o`3o00Co00001_oo0?l4o`0000Ko07l01?l00006
onGUi@cooooo1_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_l0ool3o`0000Ko0?oo1?l00006
o`3oo`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oUiND=ooooo`003?ooool6o`3o
o`?ooooo1_l0o`04ooooo`KoOgmo1?ooool6o`3oo`Cooooo1_l0ool4ooooo`Ko0?oo0oooool6ogmo
O`Cooooo1_l007l4ooooo`KoiNGU1?ooool6ool000cooooo1_oo0?l4ooooo`KoOgmo1?ooool6onGU
i@Cooooo1_l0o`04ooooo`Ko0?oo1?ooool6o`3o00?ooooo1_l0o`04ooooo`KoOgmo1?ooool6onGU
i@Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oUiND4ooooo`Ko0?oo1?ooool6o`3o
o`?ooooo1_l0ool4ooooo`Ko0?l01?ooool6ool0o`Cooooo1_l0O`04ooooo`KoiNGU3?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6o`3oo`?ooooo1_l0ool4ooooo`Ko0?oo1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6onGUi@gooooo000<ooooo`Ko0?oo0oooool6o`3o00Co
oooo1_moOgl4ooooo`Ko0?oo1?ooool6o`3oo`Cooooo1_l0ool3ooooo`KoOgmo1?ooool6o`00O`Co
oooo1_oUiND4ooooo`Koo`003?ooool6ool0o`Cooooo1_moOgl4ooooo`KoiNGU1?ooool6o`3o00Co
oooo1_l0ool4ooooo`Ko0?l00oooool6o`3o00Cooooo1_moOgl4ooooo`KoiNGU1?ooool6onGUi@co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6onGUi@Cooooo1_l0ool4ooooo`Ko0?oo0oooool6o`3oo`Co
oooo1_l0o`04ooooo`Koo`3o1?ooool6o`1o00Cooooo1_oUiND<ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l4ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_l0ool4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_oo0?l3ooooo`KoiNGU3Oooool000gooooo1?l0ool5ooooo`Co0?l01_ooool4ogmoO`Kooooo
1?l0ool6ooooo`Co0?oo1_ooool4o`3oo`Gooooo1?moOgl6ooooo`Co001o1_ooool4onGUi@Kooooo
1?oo000>ooooo`Coo`3o1_ooool4ogmoO`Kooooo1?oUiND6ooooo`Co0?l01_ooool4o`3oo`Kooooo
1?l0o`05ooooo`Co0?l01_ooool4ogmoO`Kooooo1?oUiND6ooooo`CoiNGU3_ooool4ool0o`Kooooo
1?oo0?l6ooooo`CoiNGU1_ooool4o`3oo`Kooooo1?l0ool5ooooo`Co0?oo1_ooool4o`3o00Kooooo
1?oo0?l6ooooo`Co07l01_ooool4onGUi@kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Kooooo
1?l0ool5ooooo`Co0?oo1_ooool4o`3oo`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo
1?oUiND>ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`00
3Oooool4o`3oo`Gooooo1?moOgl6ooooo`Co001o1_ooool4o`3o00Kooooo1?oo0006ooooo`Co0?l0
1Oooool4ool0o`Kooooo1?l0O`06ooooo`Coool01_ooool4ool000kooooo1?ooo`06ooooo`CoOgmo
1_ooool4o`3o00Kooooo1?l0o`06ooooo`Co0?l01_ooool4o`3o00Gooooo1?l0o`06ooooo`Coo`3o
1_ooool4o`1o00Kooooo1?ooo`0>ooooo`Coo`3o1_ooool4oooo00Kooooo1?ooo`06ooooo`Co0?l0
1_ooool4o`3oo`Gooooo1?l0o`06ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co07l0
3_ooool4ool0o`Kooooo1?mo0006ooooo`CoO`001_ooool4o`3oo`Gooooo1?l0ool6ooooo`Coo`3o
1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Ko0?oo0ooo
ool6ogmoO`Cooooo1_l007l4ooooo`Ko0?l01?ooool6ool000Cooooo1_l0o`03ooooo`Koo`3o1?oo
ool6o`1o00Cooooo1_ooo`04ooooo`Koo`003?ooool6oooo00Cooooo1_moOgl4ooooo`Ko0?l01?oo
ool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l00oooool6o`3o00Cooooo1_oo0?l4ooooo`Ko07l01?oo
ool6oooo00cooooo1_oo0?l4ooooo`Koool01?ooool6oooo00Cooooo1_l0o`04ooooo`Ko0?oo0ooo
ool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0O`0<ooooo`Koo`3o1?oo
ool6ogl000Cooooo1_mo0004ooooo`Ko0?oo0oooool6o`3oo`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_l0ool3o`0000KoOgmo1?l00006
o`00O`Co00001_l0o`04o`0000Koo`001?l00006o`3o00?o00001_oo0?l4o`0000Ko07l01?l00006
oooo00Co00001_oo000<ooooo`Koool01?l00006ogmoO`Co00001_l0o`04o`0000Ko0?l01?l00006
o`3o00Co00001_l0o`03o`0000Ko0?l01?l00006ool0o`Co00001_l0O`04o`0000Koool03?ooool6
ool0o`Co00001_ooo`04o`0000Koool01?l00006o`3o00Co00001_l0ool3o`0000Ko0?l01?l00006
ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006o`1o00cooooo1_oo0?l4o`0000KoO`001?l00006
ogl000Co00001_l0ool3o`0000Ko0?oo1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006
ool0o`?o00001_oo0?l=ooooo`003?ooool6o`3oo`?ooooo1_moOgl4ooooo`Ko001o1?ooool6o`3o
00Cooooo1_oo0004ooooo`Ko0?l00oooool6ool0o`Cooooo1_l0O`04ooooo`Koool01?ooool6ool0
00cooooo1_ooo`04ooooo`KoOgmo1?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l01?ooool6o`3o
00?ooooo1_l0o`04ooooo`Koo`3o1?ooool6o`1o00Cooooo1_ooo`0<ooooo`Koo`3o1?ooool6oooo
00Cooooo1_ooo`04ooooo`Ko0?l01?ooool6o`3oo`?ooooo1_l0o`04ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l4ooooo`Ko07l03?ooool6ool0o`Cooooo1_mo0004ooooo`KoO`001?ooool6o`3o
o`?ooooo1_l0ool4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0
o`gooooo000<ooooo`Ko0?oo0oooool6ogmoO`Cooooo1_l007l4ooooo`Ko0?l01?ooool6ool000Co
oooo1_l0o`03ooooo`Koo`3o1?ooool6o`1o00Cooooo1_ooo`04ooooo`Koo`003?ooool6oooo00Co
oooo1_moOgl4ooooo`Ko0?l01?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l00oooool6o`3o00Co
oooo1_oo0?l4ooooo`Ko07l01?ooool6oooo00cooooo1_oo0?l4ooooo`Koool01?ooool6oooo00Co
oooo1_l0o`04ooooo`Ko0?oo0oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Co
oooo1_l0O`0<ooooo`Koo`3o1?ooool6ogl000Cooooo1_mo0004ooooo`Ko0?oo0oooool6o`3oo`Co
oooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo
1?l0ool5ooooo`CoOgmo1_ooool4o`00O`Kooooo1?l0o`06ooooo`Coo`001_ooool4o`3o00Gooooo
1?oo0?l6ooooo`Co07l01_ooool4oooo00Kooooo1?oo000>ooooo`Coool01_ooool4ogmoO`Kooooo
1?l0o`06ooooo`Co0?l01_ooool4o`3o00Kooooo1?l0o`05ooooo`Co0?l01_ooool4ool0o`Kooooo
1?l0O`06ooooo`Coool03_ooool4ool0o`Kooooo1?ooo`06ooooo`Coool01_ooool4o`3o00Kooooo
1?l0ool5ooooo`Co0?l01_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4o`1o00kooooo
1?oo0?l6ooooo`CoO`001_ooool4ogl000Kooooo1?l0ool5ooooo`Co0?oo1_ooool4ool0o`Kooooo
1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=oooo
o`003Oooool4oooo00Gooooo1?ooo`06ooooo`Coool01_ooool4ool000Kooooo1?l0o`06ooooo`Co
001o1Oooool4ogmoO`Kooooo1?oo0?l6ooooo`Co0?l01_ooool4oooo00kooooo1?ooo`06ooooo`Co
ool01_ooool4oooo00Kooooo1?l0o`06ooooo`Co0?l01_ooool4o`3o00Gooooo1?oo0?l6ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?ooo`0>ooooo`Coool01_ooool4ogl000Kooooo1?mo0006ooooo`Co
ool01_ooool4o`3o00Gooooo1?l0o`06ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co
o`3o3_ooool4oooo00Kooooo1?mo0006ooooo`CoO`001_ooool4ogl000Gooooo1?ooo`06ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1Oooool4ool0o`kooooo000<ooooo`Koool0
0oooool6oooo00Cooooo1_ooo`04ooooo`Koo`001?ooool6o`3o00Cooooo1_l007l3ooooo`KoOgmo
1?ooool6ool0o`Cooooo1_l0o`04ooooo`Koool03?ooool6oooo00Cooooo1_ooo`04ooooo`Koool0
1?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l00oooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6oooo00cooooo1_ooo`04ooooo`KoO`001?ooool6ogl000Cooooo1_ooo`04ooooo`Ko0?l0
0oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l<ooooo`Koool0
1?ooool6ogl000Cooooo1_mo0004ooooo`KoO`000oooool6oooo00Cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000cooooo1_ooo`03ooooo`Koool01?oo
ool6oooo00Cooooo1_oo0004ooooo`Ko0?l01?ooool6o`00O`?ooooo1_moOgl4ooooo`Koo`3o1?oo
ool6o`3o00Cooooo1_ooo`0<ooooo`Koool01?ooool6oooo00Cooooo1_ooo`04ooooo`Ko0?l01?oo
ool6o`3o00Cooooo1_l0o`03ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koool03?oo
ool6oooo00Cooooo1_mo0004ooooo`KoO`001?ooool6oooo00Cooooo1_l0o`03ooooo`Ko0?l01?oo
ool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`cooooo1_ooo`04ooooo`KoO`001?oo
ool6ogl000Cooooo1_mo0003ooooo`Koool01?ooool6ool0o`Cooooo1_oo0?l4ooooo`Koo`3o1?oo
ool6ool0o`?ooooo1_oo0?l=ooooo`003?ooool6oooo00?o00001_ooo`04o`0000Koool01?l00006
ool000Co00001_l0o`04o`0000Ko001o0ol00006ogmoO`Co00001_oo0?l4o`0000Ko0?l01?l00006
oooo00cooooo1_ooo`04o`0000Koool01?l00006oooo00Co00001_l0o`04o`0000Ko0?l01?l00006
o`3o00?o00001_oo0?l4o`0000Koo`3o1?l00006ool0o`Co00001_ooo`0<ooooo`Koool01?l00006
ogl000Co00001_mo0004o`0000Koool01?l00006o`3o00?o00001_l0o`04o`0000Koo`3o1?l00006
ool0o`Co00001_oo0?l4o`0000Koo`3o3?ooool6oooo00Co00001_mo0004o`0000KoO`001?l00006
ogl000?o00001_ooo`04o`0000Koo`3o1?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o0ol00006
ool0o`gooooo000<ooooo`Koool00oooool6oooo00Cooooo1_ooo`04ooooo`Koo`001?ooool6o`3o
00Cooooo1_l007l3ooooo`KoOgmo1?ooool6ool0o`Cooooo1_l0o`04ooooo`Koool03?ooool6oooo
00Cooooo1_ooo`04ooooo`Koool01?ooool6o`3o00Cooooo1_l0o`04ooooo`Ko0?l00oooool6ool0
o`Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6oooo00cooooo1_ooo`04ooooo`KoO`001?ooool6ogl0
00Cooooo1_ooo`04ooooo`Ko0?l00oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l<ooooo`Koool01?ooool6ogl000Cooooo1_mo0004ooooo`KoO`000oooool6oooo
00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000go
oooo1?ooo`05ooooo`Coool01_ooool4oooo00Kooooo1?oo0006ooooo`Co0?l01_ooool4o`00O`Go
oooo1?moOgl6ooooo`Coo`3o1_ooool4o`3o00Kooooo1?ooo`0>ooooo`Coool01_ooool4oooo00Ko
oooo1?ooo`06ooooo`Co0?l01_ooool4o`3o00Kooooo1?l0o`05ooooo`Coo`3o1_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coool03_ooool4oooo00Kooooo1?mo0006ooooo`CoO`001_ooool4oooo00Ko
oooo1?l0o`05ooooo`Co0?l01_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`ko
oooo1?ooo`06ooooo`CoO`001_ooool4ogl000Kooooo1?mo0005ooooo`Coool01_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Gooooo1?oo0?l>ooooo`003_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`0=ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0=ooooo`003_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0=
ooooo`003_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0=ooooo`003Oooool4ogl000Gooooo1?mo0006ooooo`Co
Ogmo1_ooool4o`3oo`Kooooo1?l0o`06ooooo`Coo`001Oooool4ogl000Kooooo1?l0ool6ooooo`Co
o`001_ooool4o`1o00kooooo1?ooo`06ooooo`CoO`001_ooool4ogl000Kooooo1?l0ool6ooooo`Co
0?l01_ooool4o`00O`Gooooo1?moOgl6ooooo`Coo`3o1_ooool4o`3oo`Kooooo1?ooo`0>ooooo`Co
O`001_ooool4ogl000Kooooo1?mo0006ooooo`Co0?l01_ooool4o`3o00Gooooo1?l0o`06ooooo`Co
o`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co0?oo3_ooool4oooo00Kooooo1?mo0006ooooo`Co
O`001_ooool4ogl000Gooooo1?l0o`06ooooo`Coo`3o1_ooool4ool0o`Kooooo1?oo0?l6ooooo`Co
o`3o1Oooool4ool0o`kooooo000<ooooo`KoO`000oooool6ogl000Cooooo1_moOgl4ooooo`Ko0?oo
1?ooool6o`3o00Cooooo1_oo0003ooooo`KoO`001?ooool6o`3oo`Cooooo1_oo0004ooooo`Ko07l0
3?ooool6oooo00Cooooo1_mo0004ooooo`KoO`001?ooool6o`3oo`Cooooo1_l0o`04ooooo`Ko001o
0oooool6ogmoO`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo00cooooo1_mo0004ooooo`KoO`00
1?ooool6ogl000Cooooo1_l0o`04ooooo`Ko0?l00oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o
1?ooool6ool0o`Cooooo1_l0ool<ooooo`Koool01?ooool6ogl000Cooooo1_mo0004ooooo`KoO`00
0oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l3ooooo`Koo`3o
3Oooool000cooooo1_mo0003o`0000KoO`001?l00006ogmoO`Co00001_l0ool4o`0000Ko0?l01?l0
0006ool000?o00001_mo0004o`0000Ko0?oo1?l00006ool000Co00001_l0O`0<ooooo`Koool01?l0
0006ogl000Co00001_mo0004o`0000Ko0?oo1?l00006o`3o00Co00001_l007l3o`0000KoOgmo1?l0
0006ool0o`Co00001_l0ool4o`0000Koool03?ooool6ogl000Co00001_mo0004o`0000KoO`001?l0
0006o`3o00Co00001_l0o`03o`0000Ko0?l01?l00006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l0
0006o`3oo`cooooo1_ooo`04o`0000KoO`001?l00006ogl000Co00001_mo0003o`0000Ko0?l01?l0
0006ool0o`Co00001_oo0?l4o`0000Koo`3o1?l00006ool0o`?o00001_oo0?l=ooooo`003?ooool6
ogl000?ooooo1_mo0004ooooo`KoOgmo1?ooool6o`3oo`Cooooo1_l0o`04ooooo`Koo`000oooool6
ogl000Cooooo1_l0ool4ooooo`Koo`001?ooool6o`1o00cooooo1_ooo`04ooooo`KoO`001?ooool6
ogl000Cooooo1_l0ool4ooooo`Ko0?l01?ooool6o`00O`?ooooo1_moOgl4ooooo`Koo`3o1?ooool6
o`3oo`Cooooo1_ooo`0<ooooo`KoO`001?ooool6ogl000Cooooo1_mo0004ooooo`Ko0?l01?ooool6
o`3o00?ooooo1_l0o`04ooooo`Koo`3o1?ooool6ool0o`Cooooo1_oo0?l4ooooo`Ko0?oo3?ooool6
oooo00Cooooo1_mo0004ooooo`KoO`001?ooool6ogl000?ooooo1_l0o`04ooooo`Koo`3o1?ooool6
ool0o`Cooooo1_oo0?l4ooooo`Koo`3o0oooool6ool0o`gooooo000<ooooo`KoO`000oooool6ogl0
00Cooooo1_moOgl4ooooo`Ko0?oo1?ooool6o`3o00Cooooo1_oo0003ooooo`KoO`001?ooool6o`3o
o`Cooooo1_oo0004ooooo`Ko07l03?ooool6oooo00Cooooo1_mo0004ooooo`KoO`001?ooool6o`3o
o`Cooooo1_l0o`04ooooo`Ko001o0oooool6ogmoO`Cooooo1_oo0?l4ooooo`Ko0?oo1?ooool6oooo
00cooooo1_mo0004ooooo`KoO`001?ooool6ogl000Cooooo1_l0o`04ooooo`Ko0?l00oooool6o`3o
00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0o`Cooooo1_l0ool<ooooo`Koool01?ooool6ogl0
00Cooooo1_mo0004ooooo`KoO`000oooool6o`3o00Cooooo1_oo0?l4ooooo`Koo`3o1?ooool6ool0
o`Cooooo1_oo0?l3ooooo`Koo`3o3Oooool000gooooo1?mo0005ooooo`CoO`001_ooool4ogmoO`Ko
oooo1?l0ool6ooooo`Co0?l01_ooool4ool000Gooooo1?mo0006ooooo`Co0?oo1_ooool4ool000Ko
oooo1?l0O`0>ooooo`Coool01_ooool4ogl000Kooooo1?mo0006ooooo`Co0?oo1_ooool4o`3o00Ko
oooo1?l007l5ooooo`CoOgmo1_ooool4ool0o`Kooooo1?l0ool6ooooo`Coool03_ooool4ogl000Ko
oooo1?mo0006ooooo`CoO`001_ooool4o`3o00Kooooo1?l0o`05ooooo`Co0?l01_ooool4ool0o`Ko
oooo1?oo0?l6ooooo`Coo`3o1_ooool4o`3oo`kooooo1?ooo`06ooooo`CoO`001_ooool4ogl000Ko
oooo1?mo0005ooooo`Co0?l01_ooool4ool0o`Kooooo1?oo0?l6ooooo`Coo`3o1_ooool4ool0o`Go
oooo1?oo0?l>ooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhoooo
o`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhooooo`00oooooonhoooo
o`00\
\>"],
  ImageRangeCache->{{{0, 438.875}, {326.375, 0}} -> {-0.10151, -0.0744342, \
0.00957681, 0.00957681}, {{10.5625, 107.688}, {318.562, 221.438}} -> \
{-0.169633, -0.138488, 0.0113257, 0.0113257}, {{117.438, 214.562}, {318.562, \
221.438}} -> {-1.38007, -0.138488, 0.0113257, 0.0113257}, {{224.25, 321.375}, \
{318.562, 221.438}} -> {-2.5898, -0.138488, 0.0113257, 0.0113257}, {{331.125, \
428.25}, {318.562, 221.438}} -> {-3.80024, -0.138488, 0.0113257, 0.0113257}, \
{{10.5625, 107.688}, {211.75, 114.562}} -> {-0.169633, -1.34857, 0.0113257, \
0.0113257}, {{117.438, 214.562}, {211.75, 114.562}} -> {-1.38007, -1.34857, \
0.0113257, 0.0113257}, {{224.25, 321.375}, {211.75, 114.562}} -> {-2.5898, \
-1.34857, 0.0113257, 0.0113257}, {{331.125, 428.25}, {211.75, 114.562}} -> \
{-3.80024, -1.34857, 0.0113257, 0.0113257}, {{10.5625, 107.688}, {104.875, \
7.75}} -> {-0.169633, -2.55865, 0.0113257, 0.0113257}, {{117.438, 214.562}, \
{104.875, 7.75}} -> {-1.38007, -2.55865, 0.0113257, 0.0113257}, {{224.25, \
321.375}, {104.875, 7.75}} -> {-2.5898, -2.55865, 0.0113257, 0.0113257}, \
{{331.125, 428.25}, {104.875, 7.75}} -> {-3.80024, -2.55865, 0.0113257, \
0.0113257}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[vnDTD]\)], "Input",
  CellLabel->"In[129]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moore 8", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "This next model will use a significantly larger lattice than previous \
models. Hence, we set the ",
  StyleBox["ShowWindow",
    FontWeight->"Bold"],
  " option to ",
  StyleBox["False",
    FontWeight->"Bold"],
  " so that the model can be run without the additional overhead of visual \
updating after each generation. Depending on the speed of your computer, it \
may take some time to initialize. However, once"
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[\[IndentingNewLine]moore8DTD = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] Moore8[50, 50], \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors, \[IndentingNewLine]ShowWindow \[Rule] 
            False\[IndentingNewLine]]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\(9.14`\ Second\), ",", 
        InterpretationBox[\(\[LeftGuillemet]JavaObject[
              "Model"] \[RightGuillemet]\),
          JLink`Objects`JavaObject136101490589697]}], "}"}]], "Output",
  CellLabel->"Out[8]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(RasterLatticeRepresentation[\[IndentingNewLine]moore8DTD, 50, 
          AspectRatio \[Rule] Automatic\[IndentingNewLine]] // 
        Show;\)\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FFFFFF0000E5E5E500FFFF7F0000E5E5E500FFFF00FF00FF00FF00FF00FF0000FFFF00
00FF00FF00FF7F7F7F007F007F7F7FFF00FFFF00007F00007F7F7F00FFFF00FF007F7F7F
FF000000FFFF7F7F7FFFFF00FF00FF7F7F7F00FF0000FF00007F00FF000000007F007F00
E5E5E5E5E5E500FF00FF00FFFF00FF007F00FF00FFE5E5E5FF00FFFF000000FF00E5E5E5
00FF00FF0000
FFFF00E5E5E5FFFF007F7F7F00FFFF7F0000FF0000E5E5E57F0000E5E5E57F7F7FFF00FF
00FF00E5E5E500007F00007F7F000000007FE5E5E5007F00FF0000FF000000FF0000FF00
00007FE5E5E5FF0000E5E5E57F0000FF00FF00FF00FFFF0000FF007F0000007F00FFFF00
FF00FF00FF007F7F7FFF0000FF0000E5E5E5E5E5E500007F7F000000FF00FFFF0000FF00
7F0000007F00
FFFF00FF00FF7F0000007F007F7F7FE5E5E5FF000000FF00E5E5E5FF00FF7F7F7FFF00FF
FFFF00FF00FF00FFFF00FF00FFFF00FF00FF00007F7F000000FF00FFFF00FF0000FF00FF
FF00007F000000FFFFFF0000007F00FFFF00E5E5E500007FFFFF0000FFFFFF0000FFFF00
00007F007F0000007F007F007F7F7FE5E5E5E5E5E500FFFF00FF007F0000FFFF00FF00FF
FF00FF00FF00
00FF00FF00FFFFFF00E5E5E5FF00FFFF00FF7F7F7FFF000000007F00007FFF00FF7F7F7F
E5E5E57F7F7F007F007F7F7F00FF007F0000007F00E5E5E57F0000FF00FF00FF007F0000
7F000000FF00007F0000FFFF007F007F0000E5E5E500FF007F000000007FFF0000E5E5E5
00007F7F000000007F7F000000FFFF00007FE5E5E500FF007F000000FFFFFF0000007F00
FFFF00FF00FF
00FFFFFF00FF00FFFF00FF00FFFF00007F00E5E5E5E5E5E5FFFF00FF00007F7F7F00FFFF
FF000000007F00007FFF00FF00FFFFFF0000FFFF00E5E5E5FFFF00FFFF0000FF00007F00
FF00FF007F007F000000FFFF00FFFFE5E5E57F000000007F7F00007F000000007F7F0000
FF000000007F7F7F7FFF00FF7F0000007F00007F00FF000000007F7F7F7FFF00FFFF0000
FFFF00FFFF00
FF0000FFFF007F00007F7F7F7F0000E5E5E5FF00FF7F7F7FFF0000E5E5E500007F7F0000
7F7F7FFF000000FFFF00FF0000FFFF00FF0000FF00007F00007F00007F0000FF00E5E5E5
007F00FFFF00FF00FF00007FFF00007F7F7FE5E5E5FF00FFFF00FF00007FFF00007F7F7F
00FF00FF000000FFFF007F00007F00FF00007F0000FF00007F0000FF00007F000000007F
FFFF00FFFF00
007F00FF00FF007F00FF00FFFF00FFE5E5E57F7F7FE5E5E57F0000FF00FF7F7F7FFFFF00
FF00FF00FFFF00FF0000FFFFE5E5E5007F0000007F00FF007F0000FFFF0000FF007F7F7F
E5E5E500FF00007F00FF0000FF0000E5E5E500FFFF00FF0000FFFFFF00FF7F0000FF00FF
7F7F7FE5E5E500FFFF00FF00007F0000007F007F00007F00007F0000FFFF7F0000FFFF00
00FF007F7F7F
FF00FF7F7F7FFF0000E5E5E5FF00FF007F00FF0000FF00FFE5E5E57F000000FF0000FFFF
007F00FF0000FF00FF00FF0000007FFF000000FF00FFFF007F0000FF00FFE5E5E5E5E5E5
00FFFFE5E5E5E5E5E5FF00FFE5E5E500FFFF7F7F7F007F00E5E5E500FF0000FFFF00FF00
7F7F7F00FFFF00FF007F7F7F00FFFFFF00FFFF00FF7F7F7FFF000000FF00007F00E5E5E5
FF000000FF00
FF0000FF00FF00FF00FF000000FFFF7F000000FF00FF00FF7F7F7FE5E5E57F0000007F00
7F7F7F7F7F7F7F7F7F7F0000FF00FF7F0000FF0000FF0000FF00FF00007FFF000000FF00
7F7F7F00FFFF007F0000FF0000007FE5E5E500FFFF00FF0000FFFF00007F00007F00FFFF
00007FFF00007F000000007F00FF007F7F7F00FF007F7F7F00FF0000007FFF0000FF0000
7F000000007F
00FF00FF000000007FFFFF007F7F7FFF00FFFFFF007F000000007F00007F00007F00FFFF
00FF0000FFFF7F7F7F00FF00FF00FF00FFFFFF00FFFFFF00E5E5E5FF00FF00FFFF007F00
FFFF00FF0000FF000000FF00E5E5E5FF00FF7F7F7F007F00FF00FFFF00FF7F0000E5E5E5
00FFFF007F00FF00FFFF0000007F007F00007F000000007FFF00FFFF0000E5E5E5E5E5E5
FF00FF00FF00
00FF00FF0000E5E5E57F7F7F007F00FF0000FF00FFFF00007F7F7FE5E5E5E5E5E5FF00FF
00007F00007F00FFFF00FFFF7F00007F7F7F00007FFF000000FFFFFF00FF00FF00FF0000
FF0000FF00FF00FF007F0000FF00007F00007F7F7F7F0000E5E5E5FF00FF00FF0000007F
E5E5E5FF0000FF0000007F00E5E5E57F0000E5E5E5FFFF00FF00FFFFFF0000007F00FF00
E5E5E5FF00FF
00FFFFFF0000FFFF007F00007F7F7FFF00FFFFFF007F000000FFFFE5E5E5FF00FF007F00
00FFFFFF00FF007F00007F00FF00FF00007F007F0000FF00FF00FF007F00E5E5E5FF00FF
FF00FF00007F7F7F7F00007F7F7F7F007F00FF0000E5E5E5FFFF00E5E5E500FFFF7F7F7F
FF0000FF000000007F7F0000007F0000FF007F0000E5E5E5007F00FF00FFE5E5E57F7F7F
E5E5E500FF00
00007F007F00007F00FF00FF00007FFFFF007F00007F7F7F00007F7F7F7F007F007F7F7F
FF00FF7F000000FF007F7F7F7F0000FF00FF7F7F7F00FFFF7F7F7F7F7F7F00FF007F7F7F
FFFF0000FF00FF00FF007F007F7F7F00007FFF000000FF007F00007F7F7FFF000000FF00
FFFF0000FF0000FF0000007F00FF00FF00FF00FF00FF00FF00007F7F7F7FFF0000E5E5E5
E5E5E500007F
00FF00FF000000FF0000FF00FF000000FFFFFF0000FFFF007F0000FFFF00FFFF00E5E5E5
7F7F7FFFFF007F00007F0000FFFF00FF0000007F00FF00FF7F7F7F7F00007F0000007F00
00FFFF7F7F7FFF00FFFF00FFFFFF0000FF00FF00FFFF0000E5E5E500007F007F007F7F7F
007F0000007F7F7F7FFF00FF00FFFFFF00FF007F00FF00FF00FFFFE5E5E57F0000FF00FF
00007FFFFF00
E5E5E500007FFF00FFFFFF0000FFFF00FFFF7F7F7F00FFFF7F0000FFFF00FFFF00007F00
E5E5E5007F0000007F7F00007F7F7F7F00007F0000E5E5E57F7F7F7F7F7F007F00FFFF00
00FFFF00FFFFE5E5E57F7F7FFF00007F000000FFFF00007F007F007F7F7FFF000000007F
00007FFF00FF007F00FF0000FF00007F7F7F7F0000E5E5E5FF000000FFFF00007F00FFFF
7F0000FF0000
FFFF007F7F7FE5E5E5FF00FF007F00007F0000007F00007F00FF007F7F7F7F7F7FFF0000
00FF00FFFF0000007F007F00FF00007F0000FFFF0000FF0000007F00007FFFFF00FF00FF
FF00FF00FF00E5E5E57F00007F00007F0000FFFF0000FFFF00FFFF007F00FFFF0000FF00
00FF00FF0000FF00007F00007F0000FF0000FF000000FF007F0000FF0000FF0000FF0000
FF000000FFFF
7F000000FF00007F0000FF00FF00FFFFFF00007F007F0000007F00FFFF0000FF00E5E5E5
00007FFF000000FF00007F007F0000FF00FF7F00007F0000FF00007F000000007FFF00FF
FFFF00FF000000FFFF7F7F7F00FFFF00007F00007F007F0000FFFF7F000000FF00007F00
00007F00007F00007F007F0000007F7F000000FFFFE5E5E5FF0000FF00FFFF0000E5E5E5
00007F7F0000
007F0000007F7F7F7F00007F00FF00FFFF007F000000FFFFFFFF00FF00FF7F7F7FFF0000
7F0000FF00FFE5E5E500FFFFFFFF00FF000000FF00FF00FFFF000000007F00007F7F7F7F
FFFF007F7F7FFF0000FF000000FFFF00FF007F7F7F00FF00FFFF00007F00FF000000FFFF
007F00FF0000FF000000FF00FFFF007F00007F000000FFFFFF0000FFFF0000FFFF7F0000
00FFFF007F00
00FFFF7F7F7F00007F00FF00FF000000007F007F00FFFF0000FF00E5E5E500007FFF0000
FFFF007F0000FFFF00E5E5E5FF0000FF00FFFFFF0000FFFF007F0000007F00FF00FFFF00
00FFFF00FF00FF00FFFF0000007F007F7F7F00FF00FF00007F7F7F7F00007F0000FFFF00
FF000000FF00E5E5E500FFFF00FF00FF00007F000000FFFF00FFFF00FF00007F0000FFFF
00FF0000007F
FF00007F00007F7F7F007F007F000000FF00FFFF007F7F7F7F7F7F007F0000FFFF7F7F7F
00FF00FF0000FFFF00E5E5E57F00007F0000FFFF0000FFFF00FFFFE5E5E57F000000007F
00FFFFFF0000FF000000007FFF00007F000000007F7F7F7F00FFFF00007F00007F007F00
FF0000007F00FF00FF00007F7F7F7F00FFFFFF00FF007F007F7F7F00FFFFFF0000FFFF00
E5E5E57F0000
7F00007F7F7FFF0000FFFF0000007F7F7F7F007F007F7F7F7F7F7FFF000000FFFF7F0000
FF00FFE5E5E5FF000000FFFF00FFFF00FF0000FF0000007F00007F007F0000007F00FFFF
00FF0000FFFF00007FE5E5E5E5E5E500007F7F000000FFFF007F007F0000FFFF00007F00
7F7F7FFFFF00FFFF0000007F00FFFFFF00007F000000FF0000FFFF00FFFFE5E5E5FFFF00
007F00FFFF00
FFFF0000FF00007F0000FF007F0000FF00FF00FF0000FF007F7F7F00FF0000FF007F7F7F
E5E5E5FF00007F000000FF0000FF0000FF00FF00007F7F7F00007FFF0000FF00007F0000
FF000000FF00FF0000FFFF00FF00FFFF0000FFFF00FFFF00E5E5E500FF00FF00FFFF0000
FF00FFFFFF00FFFF0000FFFF00007F007F00FFFF007F7F7F7F7F7F00FFFF007F0000FFFF
00FF007F0000
00007F00FF00FF000000007FE5E5E57F0000FF00FF00FF0000FF0000007FFF0000FF0000
00007FFF000000FF00FFFF007F0000FFFF00FF00FF7F7F7FFF0000FFFF00FFFF007F7F7F
00007FFFFF0000FF00FF00FF7F7F7F7F7F7F00007F00007FFFFF0000007F7F7F7F00FFFF
FF000000FF007F00007F7F7FFF0000E5E5E500007FE5E5E5FFFF007F7F7F00FFFF00007F
00007F7F0000
00007F7F0000FFFF00007F00007F007F7F7FFF000000FF00FF00007F7F7F7F0000FFFF00
FFFF00007F00007F00E5E5E5007F00FF000000007F7F7F7F7F00007F7F7F00007F00007F
FFFF00FF00FF007F00FFFF007F7F7F7F0000FF000000FFFFFF00FFE5E5E57F0000E5E5E5
FFFF0000FF00FF00FF007F007F00007F0000FF00FF00FF007F0000FFFF0000FFFFE5E5E5
FFFF00FFFF00
FF0000FFFF00FF000000FF00E5E5E5E5E5E57F000000FF007F7F7FFFFF00E5E5E57F0000
FF000000FFFF00FF0000FF007F00007F7F7FFF00FF00007FE5E5E57F0000FF00FFFFFF00
00FF00007F0000007F7F7F7F7F000000007FFFFF00007F00E5E5E5FF00FF00007FE5E5E5
00007FFFFF0000FFFF7F0000FF00FF00007FFFFF0000FFFFFF00FF007F00FF00FFFF0000
7F7F7F00FF00
FF00FF007F0000FFFF00FF00FF00FFE5E5E5FFFF00FF0000E5E5E5FF0000007F00FF0000
7F7F7F00FF00E5E5E5FFFF00FF00FF00007FFFFF00FF00FFE5E5E5FF00FF007F007F0000
00007F00FF0000007F00FF0000FFFFFF000000007F7F7F7F00FFFFE5E5E5FF00007F7F7F
E5E5E500FF0000007F00FF00E5E5E5E5E5E5E5E5E5E5E5E57F7F7F00007F7F7F7FFF0000
FF00FF7F7F7F
00007F007F00FF000000FFFFFFFF00FF00FF00FFFFFFFF0000FF0000007F00FFFF7F0000
7F0000007F00007F0000007FFF0000E5E5E5E5E5E5FF00FF00007FFFFF0000FFFF7F0000
00FF007F000000007F007F00007F0000007FFF00FFFFFF00FF00FFFF00FF00007FFF00FF
007F0000FF00FF00007F0000FF00FF7F0000007F00FFFF007F00007F7F7FFF000000FFFF
00FFFFE5E5E5
00FFFF00FFFF00007FFFFF00E5E5E5E5E5E57F00007F000000FF00FF00FFFF00FF00FFFF
7F00007F000000FF007F7F7F00007F00FFFF7F000000FFFF00FF00FFFF0000007F007F00
E5E5E5FF000000FF0000FF0000007FFFFF00007F00007F0000FF0000FF0000007FE5E5E5
00FFFFFF000000FFFF7F000000007F007F00FF00FF00007FFF00FFFF0000FF00FF7F0000
FF00FFE5E5E5
E5E5E5E5E5E5007F007F7F7FFFFF0000FF007F7F7FFF0000E5E5E5FF000000FF0000FF00
FF00007F00007F000000FFFF7F0000FF00FFFFFF00FFFF00E5E5E5FF0000E5E5E500FFFF
E5E5E500FFFFE5E5E57F0000007F00FF00FF00FF00007F007F000000007FFF00FF00FF00
FFFF0000FFFFFFFF00007F00007F00FF000000FF00FF00FFFFFF007F7F7FFF0000007F00
007F00FF0000
00007FFFFF007F0000FF0000FF0000E5E5E500007FFF00FF007F00E5E5E500007F00FF00
00007F00FF00E5E5E500FF00E5E5E500FF00007F00007F00FF000000007FFFFF007F7F7F
FF000000FFFFE5E5E57F7F7F7F7F7F00007FFF00FF00007F00FFFF00FF00FF0000E5E5E5
FFFF00FF00007F0000007F00FF0000007F0000007F00007F007F00FFFF00FF00FFFF00FF
00007F7F7F7F
00FF00FFFF0000007FE5E5E57F000000007F007F0000FFFFE5E5E500FFFFFFFF0000FF00
7F7F7F7F0000E5E5E5FF00FF00FFFFFFFF007F0000FF00FFFF00FF007F00FF00FFFFFF00
FFFF007F7F7FFF00FF7F7F7F00007F00FF00E5E5E5E5E5E500FF0000FF0000007F00FFFF
00FF0000FFFF00FF00FF00FFFFFF00E5E5E57F7F7F00FF00FFFF00FF00007F7F7F00FFFF
7F7F7FFFFF00
00007FFF00FF7F7F7F00007FE5E5E5FFFF0000FF007F7F7FFFFF00007F00FF00FFE5E5E5
00007FE5E5E5FF00007F7F7F00FFFFFF00FFE5E5E5007F00E5E5E5007F0000007F7F0000
FFFF007F7F7FFF0000FF000000FF0000FFFFFF0000FF00FF00FFFF00FF00FF000000007F
7F7F7FFF000000007F007F00007F00FFFF00E5E5E500FF00007F00007F00E5E5E500FF00
00FF00FF0000
00FFFFFF00007F00007F0000FFFF0000FF00007F00FF00FFFF00FF00FFFFFF000000FF00
FF00FFE5E5E5E5E5E57F0000E5E5E5007F00FF00FF007F0000FF00007F007F0000FF00FF
00007F00007F00FF0000007F7F0000E5E5E5E5E5E500FF0000FF00FF00FFFF0000E5E5E5
FF0000FF00FF007F007F7F7F007F00FF000000FFFF7F0000FFFF00FF00FFFF0000FF00FF
FF0000FF00FF
E5E5E500FF007F0000FF0000FF00FFFF0000FF0000007F0000FFFF00FFFF00007FE5E5E5
E5E5E57F7F7F00007F00FFFFFF00FF007F007F7F7F00FFFFFF00FF00FFFFFF00FF00FF00
00FF00007F0000FFFFFF00FF00007F7F7F7F7F7F7FFFFF00007F007F0000FFFF00FF0000
007F00FF0000FF00FFE5E5E5FFFF0000007FFFFF00007F0000FFFF007F00FFFF00E5E5E5
FF0000FF0000
E5E5E5FF00FFFF0000FF00007F7F7FE5E5E57F00007F0000FFFF007F000000FFFF007F00
00007FFFFF0000FF0000FF0000FFFF00007FFF00FFE5E5E5FF00FFE5E5E5E5E5E5007F00
FFFF0000007F007F007F0000FF00FFFF00007F7F7F007F00FF00FFE5E5E5007F00FFFF00
00FFFF7F0000FFFF0000007F00007F007F00E5E5E57F7F7F00007F00007F007F007F7F7F
007F00FF00FF
007F007F7F7F7F7F7FE5E5E500FFFF00FF00FFFF0000FFFF7F00007F0000E5E5E5007F00
7F7F7F7F00007F7F7F7F7F7FFF00FFFFFF007F7F7F00FFFF7F000000007FE5E5E57F0000
00007F00FFFF007F00FF0000FF000000007F7F0000FFFF00FFFF0000FF0000FF0000007F
007F00FF00FFFFFF0000FFFFFF00FF007F00FF00FF007F007F000000FF00FF0000FF00FF
7F7F7FFF0000
007F00007F007F7F7FFF000000FF00FF00FF7F0000FF00FF00FF00FF000000007F00007F
007F007F00007F7F7FFF00FF7F7F7FE5E5E500007FE5E5E5FFFF0000007F00FF007F0000
E5E5E500FFFF7F7F7FFF00FF7F7F7FE5E5E5E5E5E500FFFFFFFF0000FF00FF00007F7F7F
FFFF007F7F7F00FFFF00FFFF00FF00E5E5E500007F7F000000FF007F0000007F007F7F7F
00FF00FFFF00
7F000000FF007F0000E5E5E5E5E5E500FFFF00007F00FFFF00007FFFFF0000007F00FF00
FF000000FF00FFFF00007F00FF00FFFF00FF00FF00007F00FFFF0000FFFF7F7F7FE5E5E5
007F00FF000000FFFF007F00FF0000FF00FFE5E5E5E5E5E500FFFF00007F7F00007F7F7F
E5E5E57F7F7F7F7F7F007F007F000000FFFF7F000000FFFFFF0000FF00FFE5E5E5FF0000
7F0000007F00
FF00FFFF00FFFF00FF00007F007F0000FF00FF00FF00FFFF007F00E5E5E500007FFFFF00
7F7F7FFF00FF007F007F7F7F00007FE5E5E5FF00FF00FFFF00007F7F0000FF0000007F00
00FFFFFF00FF00007F007F00007F00E5E5E5FF00FFFF0000007F0000FFFF00FF00007F00
FFFF0000FFFFFF00FF00FFFF007F00FFFF007F7F7F00FFFFE5E5E500FFFF007F0000FFFF
7F7F7FFF00FF
00FFFF007F0000007F7F7F7F007F0000007F7F00007F00007F7F7F00FFFF007F00FFFF00
FF00FF7F7F7F00007F7F0000FFFF007F0000FF0000E5E5E500FF007F0000007F00E5E5E5
00007FFF000000FFFFFFFF00E5E5E500FFFF00FFFFE5E5E57F0000007F00FFFF00FFFF00
FFFF00E5E5E5FF0000FF00FF007F007F7F7F00FFFFFF00FFFF00FFE5E5E57F00007F0000
7F0000007F00
00007FFFFF00007F007F000000FF0000FF00FF00FFFF00FF00FFFFFF00FFFF00FFFF0000
E5E5E5FFFF00007F007F000000FF007F7F7FE5E5E5FF0000E5E5E5FFFF00FF000000FF00
FF00FF00007FE5E5E57F000000FF007F00007F7F7F00007FFF000000FFFF7F000000FFFF
7F7F7F7F7F7F00007FFF00007F7F7F7F00007F0000007F00FF00FF7F7F7F007F0000FF00
00FF007F7F7F
007F0000007FE5E5E5E5E5E5007F0000FF00E5E5E500FFFF00FFFF7F7F7FE5E5E500007F
FF0000007F00FFFF00FF00FFE5E5E500FF0000FF00FF00FF00007FFF000000FFFFFFFF00
FF00FF00FF00FFFF0000FF00E5E5E500FF0000FF0000FF00FF00FF007F00FFFF00007F00
FF0000FF00FF00007FFFFF00E5E5E500FF007F7F7FFF0000FFFF00FF00FF00007F00FFFF
007F00FFFF00
00007FFF000000FFFF00FFFFFFFF0000FFFF00FFFFFFFF00FFFF0000007F00007F7F7F7F
E5E5E5FF00FF00007FFFFF00FFFF007F7F7FFF00FF00FFFFFF0000E5E5E57F000000FF00
00007F7F7F7F00FF0000007FFF00FF00FFFF007F007F7F7FE5E5E5FFFF0000007FE5E5E5
7F7F7F007F007F0000FF00FFFF00FFFFFF00FF00FFFF0000007F0000007F7F7F7F00FF00
7F7F7F7F0000
00007FFF00007F00007F0000007F00FF0000FF00FF00007F7F00007F7F7F00007F00007F
00007FFF000000007FFF00FFE5E5E500007F00FFFF7F7F7F00FFFFE5E5E57F7F7FFF00FF
FF00FF00FF0000007FFFFF0000FFFF00FF0000007F7F0000007F00FF000000007F00FFFF
FF00FF00007FFF000000FFFF7F7F7FE5E5E5FF00FFE5E5E5FFFF0000FF00007F0000FFFF
00007FFF00FF
7F00007F0000007F007F7F7F00007F7F7F7F7F7F7FE5E5E5FFFF007F0000FF0000007F00
FF00FFFF0000FFFF0000FFFF7F000000007F7F7F7F00007FFF00FFFF00007F7F7F007F00
FFFF007F0000FFFF00FF000000FF0000FF0000FFFF7F0000FF00FFFFFF0000FF007F7F7F
FFFF00FF00007F7F7F7F00007F0000FF00FF7F0000007F00FF00FFFFFF00FF00007F7F7F
7F0000E5E5E5
7F7F7FFF00FFE5E5E5FFFF00007F00FFFF007F0000FF00007F7F7FFF0000FFFF00FF00FF
E5E5E5007F00E5E5E500FF00E5E5E5FF00FF7F7F7FFFFF007F7F7F00FFFF00FFFF7F0000
7F00007F00007F7F7F00FFFF7F0000FFFF0000FF00E5E5E5007F00E5E5E57F7F7F7F7F7F
FF00007F0000FF00FF007F00FFFF00E5E5E500FF007F000000FFFF007F00FF00007F0000
00FF007F7F7F
00FFFF00007FFFFF00FF0000FF000000FF00FF0000FFFF0000FFFFE5E5E500FFFFFF0000
FF0000FF0000FF00FF00007F00FF00FF0000E5E5E5FF00FF7F7F7F7F7F7F00007FE5E5E5
7F7F7F00007FE5E5E5FFFF0000007FFFFF00FFFF007F7F7FFFFF0000FF00E5E5E5FFFF00
00FFFF00007FFF00FF00007FFFFF00E5E5E500FF00FFFF0000FF00FF00FFFF00FF7F7F7F
007F00FF00FF
00FF007F7F7F00FFFF00FFFF007F00FF0000E5E5E57F7F7F00FFFF00FFFFFFFF007F0000
FFFF007F000000FFFF00FF007F7F7F00007F00FF007F000000007FE5E5E5FF00FF7F7F7F
FF000000007FFF0000FFFF007F7F7FE5E5E500FF00007F0000007FFFFF00FFFF007F7F7F
E5E5E5FFFF00FF00FF007F0000FFFF007F00FF000000FF0000FFFF00FFFF00FFFF00FF00
7F7F7F007F00
00007FFFFF00E5E5E57F000000FF00FF00FF00007F7F7F7FFF00FFFF00FF00007F00FFFF
FFFF007F0000FFFF007F0000FF00FF7F0000E5E5E500FFFF7F7F7FFF000000FFFF00FF00
FF00FFFFFF00E5E5E5007F0000007F007F0000007FFFFF0000FFFF00007FE5E5E5007F00
00007FFFFF00FF00007F7F7FFFFF00FF00FFFFFF007F00007F7F7FFF0000007F00007F00
7F00007F7F7F
007F007F7F7F7F7F7F00007F7F7F7F7F0000FFFF0000FFFF00FFFFFF00FF007F0000007F
FF00FF7F7F7F007F00007F00FFFF00FF00FF00007F7F7F7FFF0000E5E5E5FF00FFFF00FF
00007FFFFF00007F00E5E5E57F0000FFFF00FF0000FF00FF007F0000007FFF00FF00FFFF
00FFFF007F007F00007F0000FFFF0000007F007F00FFFF0000007F00FFFFE5E5E5FF00FF
E5E5E500FFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[9]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`001_ooool6o`3oo`Goo`001_oUiND5o`3oo`KoO`00
1OoUiND6o`3oo`Go0?l01_oo0?l5o`3o00Koo`001Oooo`06o`3o00Goo`3o1_moOgl5o`1o00KoOgmo
1Ooo0?l6ool000GoO`001_moOgl5o`3oo`Go0?l01_moOgl5ool000Ko0?oo1OmoOgl6oooo00Goo`3o
1_moOgl;o`3o00Go07l01_oo0005o`00O`Ko07l02ooUiND5o`3o00_oo`3o1_l0O`05ool0o`KoiNGU
1Ooo0?l6ool000Go0?l01_oUiND5o`3o00Koo`001oooool000Kooooo1_l0ool5ool000KoiNGU1Ol0
ool6ogl000GoiNGU1_l0ool5o`3o00Koo`3o1Ol0o`06ool000Goool01_l0o`05ool0o`KoOgmo1Ol0
O`06ogmoO`Goo`3o1_oo0005ogl000KoOgmo1Ol0ool5o`3o00KoOgmo1Ooo0006o`3oo`GoOgmo1_oo
o`05ool0o`KoOgmo2ol0o`05o`1o00Koo`001Ol007l6o`1o00_oiNGU1Ol0o`0;ool0o`Ko07l01Ooo
0?l6onGUi@Goo`3o1_oo0005o`3o00KoiNGU1Ol0o`06ool000Oooooo0006ooooo`Ko0?oo1Ooo0006
onGUi@Go0?oo1_mo0005onGUi@Ko0?oo1Ol0o`06ool0o`Go0?l01_oo0005oooo00Ko0?l01Ooo0?l6
ogmoO`Go07l01_moOgl5ool0o`Koo`001Omo0006ogmoO`Go0?oo1Ol0o`06ogmoO`Goo`001_l0ool5
ogmoO`Koool01Ooo0?l6ogmoO`_o0?l01Ol0O`06ool000Go001o1_l0O`0;onGUi@Go0?l02ooo0?l6
o`1o00Goo`3o1_oUiND5ool0o`Koo`001Ol0o`06onGUi@Go0?l01_oo0007ooooo`001_ooool6o`3o
o`Goo`001_oUiND5o`3oo`KoO`001OoUiND6o`3oo`Go0?l01_oo0?l5o`3o00Koo`001Oooo`06o`3o
00Goo`3o1_moOgl5o`1o00KoOgmo1Ooo0?l6ool000GoO`001_moOgl5o`3oo`Go0?l01_moOgl5ool0
00Ko0?oo1OmoOgl6oooo00Goo`3o1_moOgl;o`3o00Go07l01_oo0005o`00O`Ko07l02ooUiND5o`3o
00_oo`3o1_l0O`05ool0o`KoiNGU1Ooo0?l6ool000Go0?l01_oUiND5o`3o00Koo`001oooool000Ko
oooo1_l0ool5ool000KoiNGU1Ol0ool6ogl000GoiNGU1_l0ool5o`3o00Koo`3o1Ol0o`06ool000Go
ool01_l0o`05ool0o`KoOgmo1Ol0O`06ogmoO`Goo`3o1_oo0005ogl000KoOgmo1Ol0ool5o`3o00Ko
Ogmo1Ooo0006o`3oo`GoOgmo1_ooo`05ool0o`KoOgmo2ol0o`05o`1o00Koo`001Ol007l6o`1o00_o
iNGU1Ol0o`0;ool0o`Ko07l01Ooo0?l6onGUi@Goo`3o1_oo0005o`3o00KoiNGU1Ol0o`06ool000Oo
oooo0006ooooo`Ko0?oo1Ooo0006onGUi@Go0?oo1_mo0005onGUi@Ko0?oo1Ol0o`06ool0o`Go0?l0
1_oo0005oooo00Ko0?l01Ooo0?l6ogmoO`Go07l01_moOgl5ool0o`Koo`001Omo0006ogmoO`Go0?oo
1Ol0o`06ogmoO`Goo`001_l0ool5ogmoO`Koool01Ooo0?l6ogmoO`_o0?l01Ol0O`06ool000Go001o
1_l0O`0;onGUi@Go0?l02ooo0?l6o`1o00Goo`3o1_oUiND5ool0o`Koo`001Ol0o`06onGUi@Go0?l0
1_oo0007ooooo`001_ooool6oooo00GoiNGU1_ooo`05ogmoO`Ko0?oo1Omo0006ool000GoiNGU1_mo
0005onGUi@KoOgmo1Ooo0?l6o`3o00GoiNGU2ol007l6ogl000Go001o1_oUiND5o`1o00_oo`002ol0
o`05o`00O`KoiNGU1Ooo0006onGUi@GoO`001_oo0?l5o`3o00Koool01Ol0o`06ogl000Go07l01_oo
o`05ool0o`Ko0?l01OmoOgl;ool000_oiNGU1_l007l5ogl000Ko0?l01Oooo`06o`3o00GoO`001_l0
O`07ooooo`001_ooool6oooo00GoiNGU1_ooo`05ogmoO`Ko0?oo1Omo0006ool000GoiNGU1_mo0005
onGUi@KoOgmo1Ooo0?l6o`3o00GoiNGU2ol007l6ogl000Go001o1_oUiND5o`1o00_oo`002ol0o`05
o`00O`KoiNGU1Ooo0006onGUi@GoO`001_oo0?l5o`3o00Koool01Ol0o`06ogl000Go07l01_ooo`05
ool0o`Ko0?l01OmoOgl;ool000_oiNGU1_l007l5ogl000Ko0?l01Oooo`06o`3o00GoO`001_l0O`07
ooooo`001_ooool6oooo00GoiNGU1_ooo`05ogmoO`Ko0?oo1Omo0006ool000GoiNGU1_mo0005onGU
i@KoOgmo1Ooo0?l6o`3o00GoiNGU2ol007l6ogl000Go001o1_oUiND5o`1o00_oo`002ol0o`05o`00
O`KoiNGU1Ooo0006onGUi@GoO`001_oo0?l5o`3o00Koool01Ol0o`06ogl000Go07l01_ooo`05ool0
o`Ko0?l01OmoOgl;ool000_oiNGU1_l007l5ogl000Ko0?l01Oooo`06o`3o00GoO`001_l0O`07oooo
o`001_ooool6oooo00GoiNGU1_ooo`05ogmoO`Ko0?oo1Omo0006ool000GoiNGU1_mo0005onGUi@Ko
Ogmo1Ooo0?l6o`3o00GoiNGU2ol007l6ogl000Go001o1_oUiND5o`1o00_oo`002ol0o`05o`00O`Ko
iNGU1Ooo0006onGUi@GoO`001_oo0?l5o`3o00Koool01Ol0o`06ogl000Go07l01_ooo`05ool0o`Ko
0?l01OmoOgl;ool000_oiNGU1_l007l5ogl000Ko0?l01Oooo`06o`3o00GoO`001_l0O`07ooooo`00
1_ooool6oooo00GoiNGU1_ooo`05ogmoO`Ko0?oo1Omo0006ool000GoiNGU1_mo0005onGUi@KoOgmo
1Ooo0?l6o`3o00GoiNGU2ol007l6ogl000Go001o1_oUiND5o`1o00_oo`002ol0o`05o`00O`KoiNGU
1Ooo0006onGUi@GoO`001_oo0?l5o`3o00Koool01Ol0o`06ogl000Go07l01_ooo`05ool0o`Ko0?l0
1OmoOgl;ool000_oiNGU1_l007l5ogl000Ko0?l01Oooo`06o`3o00GoO`001_l0O`07ooooo`001_oo
ool6oooo00Goo`3o1_mo0005o`1o00KoOgmo1OoUiND6ool000Go0?l01_oUiND5ool0o`KoOgmo1Ooo
0?l6oooo00Goo`3o1_l0ool5o`3o00Koool01Ooo0?l6o`00O`GoO`001_l0o`05oooo00Goo`001_oo
0?l5ool000KoO`001Ol0ool6ool000Go07l01_ooo`05onGUi@Ko001o1Oooo`06o`3oo`Goo`001_oo
o`05o`00O`Ko07l01Ol007l6o`1o00GoOgmo2ooUiND6o`3oo`Go0?l01_mo0005oooo00_oo`3o1_l0
o`07ooooo`001_ooool6oooo00Goo`3o1_mo0005o`1o00KoOgmo1OoUiND6ool000Go0?l01_oUiND5
ool0o`KoOgmo1Ooo0?l6oooo00Goo`3o1_l0ool5o`3o00Koool01Ooo0?l6o`00O`GoO`001_l0o`05
oooo00Goo`001_oo0?l5ool000KoO`001Ol0ool6ool000Go07l01_ooo`05onGUi@Ko001o1Oooo`06
o`3oo`Goo`001_ooo`05o`00O`Ko07l01Ol007l6o`1o00GoOgmo2ooUiND6o`3oo`Go0?l01_mo0005
oooo00_oo`3o1_l0o`07ooooo`001_ooool6oooo00Goo`3o1_mo0005o`1o00KoOgmo1OoUiND6ool0
00Go0?l01_oUiND5ool0o`KoOgmo1Ooo0?l6oooo00Goo`3o1_l0ool5o`3o00Koool01Ooo0?l6o`00
O`GoO`001_l0o`05oooo00Goo`001_oo0?l5ool000KoO`001Ol0ool6ool000Go07l01_ooo`05onGU
i@Ko001o1Oooo`06o`3oo`Goo`001_ooo`05o`00O`Ko07l01Ol007l6o`1o00GoOgmo2ooUiND6o`3o
o`Go0?l01_mo0005oooo00_oo`3o1_l0o`07ooooo`001_ooool6oooo00Goo`3o1_mo0005o`1o00Ko
Ogmo1OoUiND6ool000Go0?l01_oUiND5ool0o`KoOgmo1Ooo0?l6oooo00Goo`3o1_l0ool5o`3o00Ko
ool01Ooo0?l6o`00O`GoO`001_l0o`05oooo00Goo`001_oo0?l5ool000KoO`001Ol0ool6ool000Go
07l01_ooo`05onGUi@Ko001o1Oooo`06o`3oo`Goo`001_ooo`05o`00O`Ko07l01Ol007l6o`1o00Go
Ogmo2ooUiND6o`3oo`Go0?l01_mo0005oooo00_oo`3o1_l0o`07ooooo`001_ooool6oooo00Goo`3o
1_mo0005o`1o00KoOgmo1OoUiND6ool000Go0?l01_oUiND5ool0o`KoOgmo1Ooo0?l6oooo00Goo`3o
1_l0ool5o`3o00Koool01Ooo0?l6o`00O`GoO`001_l0o`05oooo00Goo`001_oo0?l5ool000KoO`00
1Ol0ool6ool000Go07l01_ooo`05onGUi@Ko001o1Oooo`06o`3oo`Goo`001_ooo`05o`00O`Ko07l0
1Ol007l6o`1o00GoOgmo2ooUiND6o`3oo`Go0?l01_mo0005oooo00_oo`3o1_l0o`07ooooo`001_oo
ool6oooo00Goo`3o1_mo0005o`1o00KoOgmo1OoUiND6ool000Go0?l01_oUiND5ool0o`KoOgmo1Ooo
0?l6oooo00Goo`3o1_l0ool5o`3o00Koool01Ooo0?l6o`00O`GoO`001_l0o`05oooo00Goo`001_oo
0?l5ool000KoO`001Ol0ool6ool000Go07l01_ooo`05onGUi@Ko001o1Oooo`06o`3oo`Goo`001_oo
o`05o`00O`Ko07l01Ol007l6o`1o00GoOgmo2ooUiND6o`3oo`Go0?l01_mo0005oooo00_oo`3o1_l0
o`07ooooo`001_ooool6o`3o00Goo`3o1_ooo`05onGUi@_oo`3o1_moOgl5ool000_o001o1_oo0?l5
ogmoO`KoiNGU1OmoOgl6o`1o00GoOgmo1_l0o`05ogl000Ko07l01OoUiND6ogl000Goo`3o1Ol0o`0;
ogl000Ko0?l01Ol0O`06o`3oo`Go07l01_mo0005onGUi@Ko0?l01Omo0006o`00O`Goo`001_oUiND5
o`00O`KoO`001Ol007l6ogl000Go0?oo1_l007l5onGUi@Ko0?l01Omo0006o`3oo`Goo`001_l0O`05
oooo00Koo`3o1oooool000Kooooo1_l0o`05ool0o`Koool01OoUiND;ool0o`KoOgmo1Ooo000;o`00
O`Koo`3o1OmoOgl6onGUi@GoOgmo1_l0O`05ogmoO`Ko0?l01Omo0006o`1o00GoiNGU1_mo0005ool0
o`Go0?l02omo0006o`3o00Go07l01_l0ool5o`1o00KoO`001OoUiND6o`3o00GoO`001_l007l5ool0
00KoiNGU1Ol007l6ogl000Go001o1_mo0005o`3oo`Ko001o1OoUiND6o`3o00GoO`001_l0ool5ool0
00Ko07l01Oooo`06ool0o`Oooooo0006ooooo`Ko0?l01Ooo0?l6oooo00GoiNGU2ooo0?l6ogmoO`Go
o`002ol007l6ool0o`GoOgmo1_oUiND5ogmoO`Ko07l01OmoOgl6o`3o00GoO`001_l0O`05onGUi@Ko
O`001Ooo0?l5o`3o00_oO`001_l0o`05o`1o00Ko0?oo1Ol0O`06ogl000GoiNGU1_l0o`05ogl000Ko
001o1Ooo0006onGUi@Go001o1_mo0005o`00O`KoO`001Ol0ool6o`00O`GoiNGU1_l0o`05ogl000Ko
0?oo1Ooo0006o`1o00Goool01_oo0?l7ooooo`001_ooool6o`3o00Goo`3o1_ooo`05onGUi@_oo`3o
1_moOgl5ool000_o001o1_oo0?l5ogmoO`KoiNGU1OmoOgl6o`1o00GoOgmo1_l0o`05ogl000Ko07l0
1OoUiND6ogl000Goo`3o1Ol0o`0;ogl000Ko0?l01Ol0O`06o`3oo`Go07l01_mo0005onGUi@Ko0?l0
1Omo0006o`00O`Goo`001_oUiND5o`00O`KoO`001Ol007l6ogl000Go0?oo1_l007l5onGUi@Ko0?l0
1Omo0006o`3oo`Goo`001_l0O`05oooo00Koo`3o1oooool000Kooooo1_l0o`05ool0o`Koool01OoU
iND;ool0o`KoOgmo1Ooo000;o`00O`Koo`3o1OmoOgl6onGUi@GoOgmo1_l0O`05ogmoO`Ko0?l01Omo
0006o`1o00GoiNGU1_mo0005ool0o`Go0?l02omo0006o`3o00Go07l01_l0ool5o`1o00KoO`001OoU
iND6o`3o00GoO`001_l007l5ool000KoiNGU1Ol007l6ogl000Go001o1_mo0005o`3oo`Ko001o1OoU
iND6o`3o00GoO`001_l0ool5ool000Ko07l01Oooo`06ool0o`Oooooo0006ooooo`Ko0?oo1Ooo0?l6
o`3oo`Go0?l01_ooo`05o`1o00_oiNGU1_ooo`05ool000KoOgmo1Ol0ool6ool000_o001o1Ooo0?l6
o`3oo`Goo`001_ooo`05onGUi@_oool01Ol0o`06o`1o00Goo`3o1_l0O`05ogl000_o0?oo1_oUiND5
ogl000Ko001o2omo0005o`00O`KoO`001Ooo0006o`00O`GoOgmo1_oo0?l5ogl000_o07l01_oo0005
o`00O`KoOgmo1Ooo0?l6ool000_oool01oooool000Kooooo1_l0ool5ool0o`Ko0?oo1Ol0o`06oooo
00Go07l02ooUiND6oooo00Goo`001_moOgl5o`3oo`Koo`002ol007l5ool0o`Ko0?oo1Ooo0006oooo
00GoiNGU2oooo`05o`3o00Ko07l01Ooo0?l6o`1o00GoO`002ol0ool6onGUi@GoO`001_l007l;ogl0
00Go001o1_mo0005ool000Ko001o1OmoOgl6ool0o`GoO`002ol0O`06ool000Go001o1_moOgl5ool0
o`Koo`002oooo`07ooooo`001_ooool6o`3oo`Goo`3o1_l0ool5o`3o00Koool01Ol0O`0;onGUi@Ko
ool01Ooo0006ogmoO`Go0?oo1_oo000;o`00O`Goo`3o1_l0ool5ool000Koool01OoUiND;oooo00Go
0?l01_l0O`05ool0o`Ko07l01Omo000;o`3oo`KoiNGU1Omo0006o`00O`_oO`001Ol007l6ogl000Go
o`001_l007l5ogmoO`Koo`3o1Omo000;o`1o00Koo`001Ol007l6ogmoO`Goo`3o1_oo000;oooo00Oo
oooo0006ooooo`Ko0?oo1Ooo0?l6o`3oo`Go0?l01_ooo`05o`1o00_oiNGU1_ooo`05ool000KoOgmo
1Ol0ool6ool000_o001o1Ooo0?l6o`3oo`Goo`001_ooo`05onGUi@_oool01Ol0o`06o`1o00Goo`3o
1_l0O`05ogl000_o0?oo1_oUiND5ogl000Ko001o2omo0005o`00O`KoO`001Ooo0006o`00O`GoOgmo
1_oo0?l5ogl000_o07l01_oo0005o`00O`KoOgmo1Ooo0?l6ool000_oool01oooool000Kooooo1_l0
ool5ool0o`Ko0?oo1Ol0o`06oooo00Go07l02ooUiND6oooo00Goo`001_moOgl5o`3oo`Koo`002ol0
07l5ool0o`Ko0?oo1Ooo0006oooo00GoiNGU2oooo`05o`3o00Ko07l01Ooo0?l6o`1o00GoO`002ol0
ool6onGUi@GoO`001_l007l;ogl000Go001o1_mo0005ool000Ko001o1OmoOgl6ool0o`GoO`002ol0
O`06ool000Go001o1_moOgl5ool0o`Koo`002oooo`07ooooo`001_ooool6o`3oo`Goo`3o1_l0ool5
o`3o00Koool01Ol0O`0;onGUi@Koool01Ooo0006ogmoO`Go0?oo1_oo000;o`00O`Goo`3o1_l0ool5
ool000Koool01OoUiND;oooo00Go0?l01_l0O`05ool0o`Ko07l01Omo000;o`3oo`KoiNGU1Omo0006
o`00O`_oO`001Ol007l6ogl000Goo`001_l007l5ogmoO`Koo`3o1Omo000;o`1o00Koo`001Ol007l6
ogmoO`Goo`3o1_oo000;oooo00Oooooo0006ooooo`Koo`001Oooo`06ogl000GoOgmo1_mo0005onGU
i@Koo`3o1OmoOgl6ool000GoiNGU1_l007l5ogl000KoOgmo1Ooo0006o`3oo`Go0?l01_l0ool;o`3o
013o07l01Ol0o`06onGUi@Go07l01_ooo`05ool0o`Ko001o1Ooo0006ogmoO`GoiNGU2ooo0?l6o`00
O`Goo`001_moOgl5o`3o00Koo`001Ol0ool;o`1o00Koo`001Omo0006ool000GoO`001_oo0005ogl0
00Ko001o2oooo`07ooooo`001_ooool6ool000Goool01_mo0005ogmoO`KoO`001OoUiND6ool0o`Go
Ogmo1_oo0005onGUi@Ko001o1Omo0006ogmoO`Goo`001_l0ool5o`3o00Ko0?oo2ol0o`0@o`1o00Go
0?l01_oUiND5o`1o00Koool01Ooo0?l6o`00O`Goo`001_moOgl5onGUi@_oo`3o1_l007l5ool000Ko
Ogmo1Ol0o`06ool000Go0?oo2ol0O`06ool000GoO`001_oo0005ogl000Koo`001Omo0006o`00O`_o
ool01oooool000Kooooo1_oo0005oooo00KoO`001OmoOgl6ogl000GoiNGU1_oo0?l5ogmoO`Koo`00
1OoUiND6o`00O`GoO`001_moOgl5ool000Ko0?oo1Ol0o`06o`3oo`_o0?l04?l0O`05o`3o00KoiNGU
1Ol0O`06oooo00Goo`3o1_l007l5ool000KoOgmo1OoUiND;ool0o`Ko001o1Ooo0006ogmoO`Go0?l0
1_oo0005o`3oo`_o07l01_oo0005ogl000Koo`001Omo0006ool000GoO`001_l007l;oooo00Oooooo
0006ooooo`Koo`001Oooo`06ogl000GoOgmo1_mo0005onGUi@Koo`3o1OmoOgl6ool000GoiNGU1_l0
07l5ogl000KoOgmo1Ooo0006o`3oo`Go0?l01_l0ool;o`3o013o07l01Ol0o`06onGUi@Go07l01_oo
o`05ool0o`Ko001o1Ooo0006ogmoO`GoiNGU2ooo0?l6o`00O`Goo`001_moOgl5o`3o00Koo`001Ol0
ool;o`1o00Koo`001Omo0006ool000GoO`001_oo0005ogl000Ko001o2oooo`07ooooo`001_ooool6
ool000Goool01_mo0005ogmoO`KoO`001OoUiND6ool0o`GoOgmo1_oo0005onGUi@Ko001o1Omo0006
ogmoO`Goo`001_l0ool5o`3o00Ko0?oo2ol0o`0@o`1o00Go0?l01_oUiND5o`1o00Koool01Ooo0?l6
o`00O`Goo`001_moOgl5onGUi@_oo`3o1_l007l5ool000KoOgmo1Ol0o`06ool000Go0?oo2ol0O`06
ool000GoO`001_oo0005ogl000Koo`001Omo0006o`00O`_oool01oooool000Kooooo1_l0O`05ool0
o`Ko07l02ooo0?l5onGUi@KoOgmo1OoUiND6ogl000Goo`3o1_moOgl5oooo00Koo`3o1Ol0ool6o`3o
00Go0?oo1_oUiND5o`1o00Ko001o1Ol0o`06ogl000Goool01Ol0o`06ogmoO`GoiNGU1_l0o`05o`1o
00_oo`001_oUiND5o`3oo`Ko0?l01Ol0ool6ool0o`GoO`001_oo0?l5ogmoO`KoiNGU1Ol0ool6o`3o
00Go07l01_l007l@o`1o00Ko0?oo1Omo0006oooo00Go0?l01_moOgl7ooooo`001_ooool6o`1o00Go
o`3o1_l0O`0;ool0o`GoiNGU1_moOgl5onGUi@KoO`001Ooo0?l6ogmoO`Goool01_oo0?l5o`3oo`Ko
0?l01Ol0ool6onGUi@Go07l01_l007l5o`3o00KoO`001Oooo`05o`3o00KoOgmo1OoUiND6o`3o00Go
07l02ooo0006onGUi@Go0?oo1_l0o`05o`3oo`Koo`3o1Omo0006ool0o`GoOgmo1_oUiND5o`3oo`Ko
0?l01Ol0O`06o`00Oa3o07l01_l0ool5ogl000Koool01Ol0o`06ogmoO`Oooooo0006ooooo`Ko07l0
1Ooo0?l6o`1o00_oo`3o1OoUiND6ogmoO`GoiNGU1_mo0005ool0o`KoOgmo1Oooo`06ool0o`Go0?oo
1_l0o`05o`3oo`KoiNGU1Ol0O`06o`00O`Go0?l01_mo0005oooo00Go0?l01_moOgl5onGUi@Ko0?l0
1Ol0O`0;ool000KoiNGU1Ol0ool6o`3o00Go0?oo1_oo0?l5ogl000Koo`3o1OmoOgl6onGUi@Go0?oo
1_l0o`05o`1o00Ko001o4?l0O`06o`3oo`GoO`001_ooo`05o`3o00KoOgmo1oooool000Kooooo1_l0
O`05ool0o`Ko07l02ooo0?l5onGUi@KoOgmo1OoUiND6ogl000Goo`3o1_moOgl5oooo00Koo`3o1Ol0
ool6o`3o00Go0?oo1_oUiND5o`1o00Ko001o1Ol0o`06ogl000Goool01Ol0o`06ogmoO`GoiNGU1_l0
o`05o`1o00_oo`001_oUiND5o`3oo`Ko0?l01Ol0ool6ool0o`GoO`001_oo0?l5ogmoO`KoiNGU1Ol0
ool6o`3o00Go07l01_l007l@o`1o00Ko0?oo1Omo0006oooo00Go0?l01_moOgl7ooooo`001_ooool6
o`1o00Goo`3o1_l0O`0;ool0o`GoiNGU1_moOgl5onGUi@KoO`001Ooo0?l6ogmoO`Goool01_oo0?l5
o`3oo`Ko0?l01Ol0ool6onGUi@Go07l01_l007l5o`3o00KoO`001Oooo`05o`3o00KoOgmo1OoUiND6
o`3o00Go07l02ooo0006onGUi@Go0?oo1_l0o`05o`3oo`Koo`3o1Omo0006ool0o`GoOgmo1_oUiND5
o`3oo`Ko0?l01Ol0O`06o`00Oa3o07l01_l0ool5ogl000Koool01Ol0o`06ogmoO`Oooooo0006oooo
o`Koo`3o1OmoOgl6ool000GoiNGU1_oo0?l5o`1o00Koo`001Ooo0?l6onGUi@GoO`001_l0o`05o`3o
o`Ko07l01Ooo0006ool0o`Go0?l01_l007l5ool000Ko0?l01Oooo`06ogl000Goo`3o2ooUiND5o`3o
o`_oiNGU1_oo0?l5onGUi@Ko0?oo1OmoOgl6o`1o00GoiNGU1_l0o`05o`3oo`Ko0?l01OmoOgl6o`3o
o`Go0?l01_moOgl5o`3oo`_oo`3o1_moOgl5ool000Ko0?l01Ol0O`06onGUi@Goo`001_l0o`07oooo
o`001_ooool6ool0o`GoOgmo1_oo0005onGUi@Koo`3o1Ol0O`06ool000Goo`3o1_oUiND5ogl000Ko
0?l01Ol0ool6o`1o00Goo`001_oo0?l5o`3o00Ko001o1Ooo0006o`3o00Goool01_mo0005ool0o`_o
iNGU1Ol0ool;onGUi@Koo`3o1OoUiND6o`3oo`GoOgmo1_l0O`05onGUi@Ko0?l01Ol0ool6o`3o00Go
Ogmo1_l0ool5o`3o00KoOgmo1Ol0ool;ool0o`KoOgmo1Ooo0006o`3o00Go07l01_oUiND5ool000Ko
0?l01oooool000Kooooo1_oo0?l5ogmoO`Koo`001OoUiND6ool0o`Go07l01_oo0005ool0o`KoiNGU
1Omo0006o`3o00Go0?oo1_l0O`05ool000Koo`3o1Ol0o`06o`00O`Goo`001_l0o`05oooo00KoO`00
1Ooo0?l;onGUi@Go0?oo2ooUiND6ool0o`GoiNGU1_l0ool5ogmoO`Ko07l01OoUiND6o`3o00Go0?oo
1_l0o`05ogmoO`Ko0?oo1Ol0o`06ogmoO`Go0?oo2ooo0?l6ogmoO`Goo`001_l0o`05o`1o00KoiNGU
1Ooo0006o`3o00Oooooo0006ooooo`Koo`3o1OmoOgl6ool000GoiNGU1_oo0?l5o`1o00Koo`001Ooo
0?l6onGUi@GoO`001_l0o`05o`3oo`Ko07l01Ooo0006ool0o`Go0?l01_l007l5ool000Ko0?l01Ooo
o`06ogl000Goo`3o2ooUiND5o`3oo`_oiNGU1_oo0?l5onGUi@Ko0?oo1OmoOgl6o`1o00GoiNGU1_l0
o`05o`3oo`Ko0?l01OmoOgl6o`3oo`Go0?l01_moOgl5o`3oo`_oo`3o1_moOgl5ool000Ko0?l01Ol0
O`06onGUi@Goo`001_l0o`07ooooo`001_ooool6ool0o`GoOgmo1_oo0005onGUi@Koo`3o1Ol0O`06
ool000Goo`3o1_oUiND5ogl000Ko0?l01Ol0ool6o`1o00Goo`001_oo0?l5o`3o00Ko001o1Ooo0006
o`3o00Goool01_mo0005ool0o`_oiNGU1Ol0ool;onGUi@Koo`3o1OoUiND6o`3oo`GoOgmo1_l0O`05
onGUi@Ko0?l01Ol0ool6o`3o00GoOgmo1_l0ool5o`3o00KoOgmo1Ol0ool;ool0o`KoOgmo1Ooo0006
o`3o00Go07l01_oUiND5ool000Ko0?l01oooool000Kooooo1_oo0?l5ogmoO`Koo`001OoUiND6ool0
o`Go07l01_oo0005ool0o`KoiNGU1Omo0006o`3o00Go0?oo1_l0O`05ool000Koo`3o1Ol0o`06o`00
O`Goo`001_l0o`05oooo00KoO`001Ooo0?l;onGUi@Go0?oo2ooUiND6ool0o`GoiNGU1_l0ool5ogmo
O`Ko07l01OoUiND6o`3o00Go0?oo1_l0o`05ogmoO`Ko0?oo1Ol0o`06ogmoO`Go0?oo2ooo0?l6ogmo
O`Goo`001_l0o`05o`1o00KoiNGU1Ooo0006o`3o00Oooooo0006ooooo`Koo`001Ooo0?l6o`3o00Go
o`001_l0ool5ogl000Ko0?l01Ooo0?l6ogmoO`GoiNGU1_mo0005o`1o017oOgmo1Omo0006ool0o`Go
O`002ooo0006ool0o`Go001o1Ooo0006o`3o00GoOgmo1_l0ool5o`1o00Ko0?l01Ol007l6onGUi@Go
0?oo1_l0o`05o`3oo`_o001o1_l0ool5o`00O`Koo`001Omo0006o`00O`Go0?l01_moOgl5o`3o00Ko
Ogmo1Ol0o`06o`00O`_oo`001Omo0006o`00O`Oooooo0006ooooo`Koo`001Ooo0?l6o`3o00Goo`00
1_l0ool5ogl000Ko0?l01Ooo0?l6ogmoO`GoiNGU1_mo0005o`1o017oOgmo1Omo0006ool0o`GoO`00
2ooo0006ool0o`Go001o1Ooo0006o`3o00GoOgmo1_l0ool5o`1o00Ko0?l01Ol007l6onGUi@Go0?oo
1_l0o`05o`3oo`_o001o1_l0ool5o`00O`Koo`001Omo0006o`00O`Go0?l01_moOgl5o`3o00KoOgmo
1Ol0o`06o`00O`_oo`001Omo0006o`00O`Oooooo0006ooooo`Koo`001Ooo0?l6o`3o00Goo`001_l0
ool5ogl000Ko0?l01Ooo0?l6ogmoO`GoiNGU1_mo0005o`1o017oOgmo1Omo0006ool0o`GoO`002ooo
0006ool0o`Go001o1Ooo0006o`3o00GoOgmo1_l0ool5o`1o00Ko0?l01Ol007l6onGUi@Go0?oo1_l0
o`05o`3oo`_o001o1_l0ool5o`00O`Koo`001Omo0006o`00O`Go0?l01_moOgl5o`3o00KoOgmo1Ol0
o`06o`00O`_oo`001Omo0006o`00O`Oooooo0006ooooo`Koo`001Ooo0?l6o`3o00Goo`001_l0ool5
ogl000Ko0?l01Ooo0?l6ogmoO`GoiNGU1_mo0005o`1o017oOgmo1Omo0006ool0o`GoO`002ooo0006
ool0o`Go001o1Ooo0006o`3o00GoOgmo1_l0ool5o`1o00Ko0?l01Ol007l6onGUi@Go0?oo1_l0o`05
o`3oo`_o001o1_l0ool5o`00O`Koo`001Omo0006o`00O`Go0?l01_moOgl5o`3o00KoOgmo1Ol0o`06
o`00O`_oo`001Omo0006o`00O`Oooooo0006ooooo`Koo`001Ooo0?l6o`3o00Goo`001_l0ool5ogl0
00Ko0?l01Ooo0?l6ogmoO`GoiNGU1_mo0005o`1o017oOgmo1Omo0006ool0o`GoO`002ooo0006ool0
o`Go001o1Ooo0006o`3o00GoOgmo1_l0ool5o`1o00Ko0?l01Ol007l6onGUi@Go0?oo1_l0o`05o`3o
o`_o001o1_l0ool5o`00O`Koo`001Omo0006o`00O`Go0?l01_moOgl5o`3o00KoOgmo1Ol0o`06o`00
O`_oo`001Omo0006o`00O`Oooooo0006ooooo`Ko0?l01Ooo0006o`00O`Goool01_moOgl5ool0o`Ko
ool01Omo000Ao`00O`Go0?oo1_l0o`05o`3oo`KoOgmo1Ol0o`06ool0o`Go0?oo1_oo0?l5oooo00Ko
iNGU1Ooo0?l5o`3oo`Ko07l01Oooo`0;ool000Ko0?l01OoUiND6ool0o`GoOgmo1_l0O`0;ool0o`Go
O`001_oUiND5o`3oo`Ko07l01Ooo0?l6ool000Go07l02omo0006o`00O`Goo`3o1_oo000;onGUi@Go
o`3o1_l0o`07ooooo`001_ooool6o`3o00Goo`001_l007l5oooo00KoOgmo1Ooo0?l6oooo00GoO`00
4Ol007l5o`3oo`Ko0?l01Ol0ool6ogmoO`Go0?l01_oo0?l5o`3oo`Koo`3o1Oooo`06onGUi@Goo`3o
1Ol0ool6o`1o00Goool02ooo0006o`3o00GoiNGU1_oo0?l5ogmoO`Ko07l02ooo0?l5ogl000KoiNGU
1Ol0ool6o`1o00Goo`3o1_oo0005o`1o00_oO`001_l007l5ool0o`Koo`002ooUiND5ool0o`Ko0?l0
1oooool000Kooooo1_l0o`05ool000Ko001o1Oooo`06ogmoO`Goo`3o1_ooo`05ogl0017o001o1Ol0
ool6o`3o00Go0?oo1_moOgl5o`3o00Koo`3o1Ol0ool6ool0o`Goool01_oUiND5ool0o`Go0?oo1_l0
O`05oooo00_oo`001_l0o`05onGUi@Koo`3o1OmoOgl6o`1o00_oo`3o1Omo0006onGUi@Go0?oo1_l0
O`05ool0o`Koo`001Ol0O`0;ogl000Ko001o1Ooo0?l6ool000_oiNGU1Ooo0?l6o`3o00Oooooo0006
ooooo`Ko0?l01Ooo0006o`00O`Goool01_moOgl5ool0o`Koool01Omo000Ao`00O`Go0?oo1_l0o`05
o`3oo`KoOgmo1Ol0o`06ool0o`Go0?oo1_oo0?l5oooo00KoiNGU1Ooo0?l5o`3oo`Ko07l01Oooo`0;
ool000Ko0?l01OoUiND6ool0o`GoOgmo1_l0O`0;ool0o`GoO`001_oUiND5o`3oo`Ko07l01Ooo0?l6
ool000Go07l02omo0006o`00O`Goo`3o1_oo000;onGUi@Goo`3o1_l0o`07ooooo`001_ooool6o`3o
00Goo`001_l007l5oooo00KoOgmo1Ooo0?l6oooo00GoO`004Ol007l5o`3oo`Ko0?l01Ol0ool6ogmo
O`Go0?l01_oo0?l5o`3oo`Koo`3o1Oooo`06onGUi@Goo`3o1Ol0ool6o`1o00Goool02ooo0006o`3o
00GoiNGU1_oo0?l5ogmoO`Ko07l02ooo0?l5ogl000KoiNGU1Ol0ool6o`1o00Goo`3o1_oo0005o`1o
00_oO`001_l007l5ool0o`Koo`002ooUiND5ool0o`Ko0?l01oooool000Kooooo1_l0o`05ool000Ko
001o1Oooo`06ogmoO`Goo`3o1_ooo`05ogl0017o001o1Ol0ool6o`3o00Go0?oo1_moOgl5o`3o00Ko
o`3o1Ol0ool6ool0o`Goool01_oUiND5ool0o`Go0?oo1_l0O`05oooo00_oo`001_l0o`05onGUi@Ko
o`3o1OmoOgl6o`1o00_oo`3o1Omo0006onGUi@Go0?oo1_l0O`05ool0o`Koo`001Ol0O`0;ogl000Ko
001o1Ooo0?l6ool000_oiNGU1Ooo0?l6o`3o00Oooooo0006ooooo`Ko0?l01Ooo0006onGUi@GoOgmo
1_l0O`05ool000Koo`3o1Ooo0006ogmoO`_oiNGU1Ooo0?l;o`00O`_o0?oo1_mo0005ogmoO`Ko001o
1Ooo0006o`3oo`Goo`3o1Ol0o`0;ool000Koo`3o1Ol0o`06ogl000Goo`001_mo0005ogmoO`KoO`00
1OoUiND6ool0o`Go0?l01_l007l5onGUi@_oo`001_l0O`05onGUi@KoO`001OoUiND6oooo00Goo`3o
1_ooo`05o`00O`Ko0?l01OoUiND6ool0o`Oooooo0006ooooo`Ko0?l01Ooo0006onGUi@GoOgmo1_l0
O`05ool000Koo`3o1Ooo0006ogmoO`_oiNGU1Ooo0?l;o`00O`_o0?oo1_mo0005ogmoO`Ko001o1Ooo
0006o`3oo`Goo`3o1Ol0o`0;ool000Koo`3o1Ol0o`06ogl000Goo`001_mo0005ogmoO`KoO`001OoU
iND6ool0o`Go0?l01_l007l5onGUi@_oo`001_l0O`05onGUi@KoO`001OoUiND6oooo00Goo`3o1_oo
o`05o`00O`Ko0?l01OoUiND6ool0o`Oooooo0006ooooo`Ko0?l01Ooo0006onGUi@GoOgmo1_l0O`05
ool000Koo`3o1Ooo0006ogmoO`_oiNGU1Ooo0?l;o`00O`_o0?oo1_mo0005ogmoO`Ko001o1Ooo0006
o`3oo`Goo`3o1Ol0o`0;ool000Koo`3o1Ol0o`06ogl000Goo`001_mo0005ogmoO`KoO`001OoUiND6
ool0o`Go0?l01_l007l5onGUi@_oo`001_l0O`05onGUi@KoO`001OoUiND6oooo00Goo`3o1_ooo`05
o`00O`Ko0?l01OoUiND6ool0o`Oooooo0006ooooo`Ko0?l01Ooo0006onGUi@GoOgmo1_l0O`05ool0
00Koo`3o1Ooo0006ogmoO`_oiNGU1Ooo0?l;o`00O`_o0?oo1_mo0005ogmoO`Ko001o1Ooo0006o`3o
o`Goo`3o1Ol0o`0;ool000Koo`3o1Ol0o`06ogl000Goo`001_mo0005ogmoO`KoO`001OoUiND6ool0
o`Go0?l01_l007l5onGUi@_oo`001_l0O`05onGUi@KoO`001OoUiND6oooo00Goo`3o1_ooo`05o`00
O`Ko0?l01OoUiND6ool0o`Oooooo0006ooooo`Ko0?l01Ooo0006onGUi@GoOgmo1_l0O`05ool000Ko
o`3o1Ooo0006ogmoO`_oiNGU1Ooo0?l;o`00O`_o0?oo1_mo0005ogmoO`Ko001o1Ooo0006o`3oo`Go
o`3o1Ol0o`0;ool000Koo`3o1Ol0o`06ogl000Goo`001_mo0005ogmoO`KoO`001OoUiND6ool0o`Go
0?l01_l007l5onGUi@_oo`001_l0O`05onGUi@KoO`001OoUiND6oooo00Goo`3o1_ooo`05o`00O`Ko
0?l01OoUiND6ool0o`Oooooo0006ooooo`Ko0?oo1Ooo0006oooo00GoO`001_moOgl5ool0o`Koool0
1Omo0006o`3oo`GoiNGU1_oo0?l5o`1o00Ko0?oo1Ooo0?l;o`1o00Koo`3o1Ol007l6o`1o00Go0?l0
1_oo0?l5o`1o00GoiNGU2ooo0?l6o`00O`GoOgmo1_l007l5ogmoO`Ko07l01Ooo0006onGUi@Goool0
1_oUiND5o`3oo`KoOgmo2ooo0005o`00O`KoO`001Ol0O`06o`3o00GoO`001_oUiND5o`1o00Koo`3o
1OoUiND6ogmoO`GoiNGU1_l0o`07ooooo`001_ooool6o`3oo`Goo`001_ooo`05ogl000KoOgmo1Ooo
0?l6oooo00GoO`001_l0ool5onGUi@Koo`3o1Ol0O`06o`3oo`Goo`3o2ol0O`06ool0o`Go001o1_l0
O`05o`3o00Koo`3o1Ol0O`05onGUi@_oo`3o1_l007l5ogmoO`Ko001o1OmoOgl6o`1o00Goo`001_oU
iND5oooo00KoiNGU1Ol0ool6ogmoO`_oo`001Ol007l6ogl000Go07l01_l0o`05ogl000KoiNGU1Ol0
O`06ool0o`GoiNGU1_moOgl5onGUi@Ko0?l01oooool000Kooooo1_l0ool5ool000Koool01Omo0006
ogmoO`Goo`3o1_ooo`05ogl000Ko0?oo1OoUiND6ool0o`Go07l01_l0ool5ool0o`_o07l01_oo0?l5
o`00O`Ko07l01Ol0o`06ool0o`Go07l01OoUiND;ool0o`Ko001o1OmoOgl6o`00O`GoOgmo1_l0O`05
ool000KoiNGU1Oooo`06onGUi@Go0?oo1_moOgl;ool000Go001o1_mo0005o`1o00Ko0?l01Omo0006
onGUi@Go07l01_oo0?l5onGUi@KoOgmo1OoUiND6o`3o00Oooooo0006ooooo`Ko0?oo1Ooo0006oooo
00GoO`001_moOgl5ool0o`Koool01Omo0006o`3oo`GoiNGU1_oo0?l5o`1o00Ko0?oo1Ooo0?l;o`1o
00Koo`3o1Ol007l6o`1o00Go0?l01_oo0?l5o`1o00GoiNGU2ooo0?l6o`00O`GoOgmo1_l007l5ogmo
O`Ko07l01Ooo0006onGUi@Goool01_oUiND5o`3oo`KoOgmo2ooo0005o`00O`KoO`001Ol0O`06o`3o
00GoO`001_oUiND5o`1o00Koo`3o1OoUiND6ogmoO`GoiNGU1_l0o`07ooooo`001_ooool6o`3oo`Go
o`001_ooo`05ogl000KoOgmo1Ooo0?l6oooo00GoO`001_l0ool5onGUi@Koo`3o1Ol0O`06o`3oo`Go
o`3o2ol0O`06ool0o`Go001o1_l0O`05o`3o00Koo`3o1Ol0O`05onGUi@_oo`3o1_l007l5ogmoO`Ko
001o1OmoOgl6o`1o00Goo`001_oUiND5oooo00KoiNGU1Ol0ool6ogmoO`_oo`001Ol007l6ogl000Go
07l01_l0o`05ogl000KoiNGU1Ol0O`06ool0o`GoiNGU1_moOgl5onGUi@Ko0?l01oooool000Kooooo
1_l0ool5ool000Koool01Omo0006ogmoO`Goo`3o1_ooo`05ogl000Ko0?oo1OoUiND6ool0o`Go07l0
1_l0ool5ool0o`_o07l01_oo0?l5o`00O`Ko07l01Ol0o`06ool0o`Go07l01OoUiND;ool0o`Ko001o
1OmoOgl6o`00O`GoOgmo1_l0O`05ool000KoiNGU1Oooo`06onGUi@Go0?oo1_moOgl;ool000Go001o
1_mo0005o`1o00Ko0?l01Omo0006onGUi@Go07l01_oo0?l5onGUi@KoOgmo1OoUiND6o`3o00Oooooo
0006ooooo`Ko001o2ol0O`05ool0o`Ko001o1Oooo`06ogl000GoOgmo1_l007l5ogmoO`Ko07l01Omo
Ogl6ool0o`GoO`001_l0o`05ogmoO`KoO`001Ooo0?l6ogmoO`Go0?oo2omoOgl5o`3o00KoOgmo1Ooo
o`06o`3o00Goo`3o1_l0O`05ogmoO`Ko001o1Ooo0006o`3o00GoO`001_moOgl5ool000Ko0?l01Ooo
o`0;o`3o00Ko001o1Ol0o`06ool0o`Go0?l01_oo0?l5o`00O`KoOgmo1Ooo000;onGUi@Ko001o1ooo
ool000Kooooo1_l007l;o`1o00Goo`3o1_l007l5oooo00KoO`001OmoOgl6o`00O`GoOgmo1_l0O`05
ogmoO`Koo`3o1Omo0006o`3o00GoOgmo1_mo0005ool0o`KoOgmo1Ol0ool;ogmoO`Go0?l01_moOgl5
oooo00Ko0?l01Ooo0?l6o`1o00GoOgmo1_l007l5ool000Ko0?l01Omo0006ogmoO`Goo`001_l0o`05
oooo00_o0?l01_l007l5o`3o00Koo`3o1Ol0o`06ool0o`Go001o1_moOgl5ool000_oiNGU1_l007l7
ooooo`001_ooool6o`00O`_o07l01Ooo0?l6o`00O`Goool01_mo0005ogmoO`Ko001o1OmoOgl6o`1o
00GoOgmo1_oo0?l5ogl000Ko0?l01OmoOgl6ogl000Goo`3o1_moOgl5o`3oo`_oOgmo1Ol0o`06ogmo
O`Goool01_l0o`05ool0o`Ko07l01OmoOgl6o`00O`Goo`001_l0o`05ogl000KoOgmo1Ooo0006o`3o
00Goool02ol0o`06o`00O`Go0?l01_oo0?l5o`3o00Koo`3o1Ol007l6ogmoO`Goo`002ooUiND6o`00
O`Oooooo0006ooooo`Ko001o2ol0O`05ool0o`Ko001o1Oooo`06ogl000GoOgmo1_l007l5ogmoO`Ko
07l01OmoOgl6ool0o`GoO`001_l0o`05ogmoO`KoO`001Ooo0?l6ogmoO`Go0?oo2omoOgl5o`3o00Ko
Ogmo1Oooo`06o`3o00Goo`3o1_l0O`05ogmoO`Ko001o1Ooo0006o`3o00GoO`001_moOgl5ool000Ko
0?l01Oooo`0;o`3o00Ko001o1Ol0o`06ool0o`Go0?l01_oo0?l5o`00O`KoOgmo1Ooo000;onGUi@Ko
001o1oooool000Kooooo1_l007l;o`1o00Goo`3o1_l007l5oooo00KoO`001OmoOgl6o`00O`GoOgmo
1_l0O`05ogmoO`Koo`3o1Omo0006o`3o00GoOgmo1_mo0005ool0o`KoOgmo1Ol0ool;ogmoO`Go0?l0
1_moOgl5oooo00Ko0?l01Ooo0?l6o`1o00GoOgmo1_l007l5ool000Ko0?l01Omo0006ogmoO`Goo`00
1_l0o`05oooo00_o0?l01_l007l5o`3o00Koo`3o1Ol0o`06ool0o`Go001o1_moOgl5ool000_oiNGU
1_l007l7ooooo`001_ooool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002ooo
o`05onGUi@KoOgmo1Oooo`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0
ool6ogmoO`_oo`3o1Oooo`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l0
07l5ogmoO`Koo`3o1Ol0ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_oo
o`07ooooo`001_ooool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002oooo`05
onGUi@KoOgmo1Oooo`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0ool6
ogmoO`_oo`3o1Oooo`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l007l5
ogmoO`Koo`3o1Ol0ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_ooo`07
ooooo`001_ooool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002oooo`05onGU
i@KoOgmo1Oooo`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0ool6ogmo
O`_oo`3o1Oooo`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l007l5ogmo
O`Koo`3o1Ol0ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_ooo`07oooo
o`001_ooool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002oooo`05onGUi@Ko
Ogmo1Oooo`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0ool6ogmoO`_o
o`3o1Oooo`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l007l5ogmoO`Ko
o`3o1Ol0ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_ooo`07ooooo`00
1_ooool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002oooo`05onGUi@KoOgmo
1Oooo`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0ool6ogmoO`_oo`3o
1Oooo`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l007l5ogmoO`Koo`3o
1Ol0ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_ooo`07ooooo`001_oo
ool6o`3o00Goo`002ol0o`06ool000Go0?oo1_oo0005oooo00KoO`002oooo`05onGUi@KoOgmo1Ooo
o`0;ogl000Koool01Ooo0006o`1o00Goo`3o1_moOgl:ogl000Ko07l01Ol0ool6ogmoO`_oo`3o1Ooo
o`06o`3o00Goo`3o1_oo0005onGUi@Ko001o1Ol0O`06ogmoO`Go07l01_l007l5ogmoO`Koo`3o1Ol0
ool6ool0o`Go07l01_oo0?l5o`3oo`KoiNGU1Omo0006ool0o`Go001o1_ooo`07ooooo`001_ooool6
onGUi@Go001o1_oo0?l5oooo00_o0?oo1_moOgl5o`3oo`KoO`002oooo`05o`1o00KoiNGU1Ol0O`06
o`00O`GoO`001_moOgl;ogl000GoiNGU2omoOgl5o`1o00Koool02ol0ool5onGUi@KoOgmo1Ooo0006
ogl000Go0?oo1_l007l5o`1o00KoOgmo1Ooo000;o`00O`Koo`3o1Ol0O`0;ool000KoOgmo1Omo0006
onGUi@Goo`001_l0ool5o`00O`Ko0?oo1Omo0006ool000Oooooo0006ooooo`KoiNGU1Ol007l6ool0
o`Goool02ol0ool6ogmoO`Go0?oo1_mo000;oooo00Go07l01_oUiND5o`1o00Ko001o1Omo0006ogmo
O`_oO`001OoUiND;ogmoO`Go07l01_ooo`0;o`3oo`GoiNGU1_moOgl5ool000KoO`001Ol0ool6o`00
O`Go07l01_moOgl5ool000_o001o1_oo0?l5o`1o00_oo`001_moOgl5ogl000KoiNGU1Ooo0006o`3o
o`Go001o1_l0ool5ogl000Koo`001oooool000Kooooo1_oUiND5o`00O`Koo`3o1Oooo`0;o`3oo`Ko
Ogmo1Ol0ool6ogl000_oool01Ol0O`06onGUi@Go07l01_l007l5ogl000KoOgmo2omo0005onGUi@_o
Ogmo1Ol0O`06oooo00_o0?oo1OoUiND6ogmoO`Goo`001_mo0005o`3oo`Ko001o1Ol0O`06ogmoO`Go
o`002ol007l6ool0o`Go07l02ooo0006ogmoO`GoO`001_oUiND5ool000Ko0?oo1Ol007l6o`3oo`Go
O`001_oo0007ooooo`001_ooool6onGUi@Go001o1_oo0?l5oooo00_o0?oo1_moOgl5o`3oo`KoO`00
2oooo`05o`1o00KoiNGU1Ol0O`06o`00O`GoO`001_moOgl;ogl000GoiNGU2omoOgl5o`1o00Koool0
2ol0ool5onGUi@KoOgmo1Ooo0006ogl000Go0?oo1_l007l5o`1o00KoOgmo1Ooo000;o`00O`Koo`3o
1Ol0O`0;ool000KoOgmo1Omo0006onGUi@Goo`001_l0ool5o`00O`Ko0?oo1Omo0006ool000Oooooo
0006ooooo`KoiNGU1Ol007l6ool0o`Goool02ol0ool6ogmoO`Go0?oo1_mo000;oooo00Go07l01_oU
iND5o`1o00Ko001o1Omo0006ogmoO`_oO`001OoUiND;ogmoO`Go07l01_ooo`0;o`3oo`GoiNGU1_mo
Ogl5ool000KoO`001Ol0ool6o`00O`Go07l01_moOgl5ool000_o001o1_oo0?l5o`1o00_oo`001_mo
Ogl5ogl000KoiNGU1Ooo0006o`3oo`Go001o1_l0ool5ogl000Koo`001oooool000Kooooo1_ooo`05
ogmoO`KoiNGU1Ooo0?l;o`1o00_o001o1_l0o`0;ogmoO`Goo`001_l0o`05oooo00Ko001o1Ol0O`06
ool000GoO`001_ooo`05o`3o00_o001o1Oooo`0;ool0o`Ko0?l01OoUiNDAogl000Goool02ol0ool6
o`1o00Goool02ol0o`0;ool000_oO`002ooo0006o`3o00GoO`005_oo0006o`3oo`Oooooo0006oooo
o`Koool01OmoOgl6onGUi@Goo`3o2ol0O`0;o`00O`Ko0?l02omoOgl5ool000Ko0?l01Oooo`06o`00
O`Go07l01_oo0005ogl000Koool01Ol0o`0;o`00O`Goool02ooo0?l6o`3o00GoiNGU4Omo0005oooo
00_o0?oo1_l0O`05oooo00_o0?l02ooo000;ogl000_oo`001_l0o`05ogl001Koo`001_l0ool7oooo
o`001_ooool6oooo00GoOgmo1_oUiND5ool0o`_o07l02ol007l6o`3o00_oOgmo1Ooo0006o`3o00Go
ool01_l007l5o`1o00Koo`001Omo0006oooo00Go0?l02ol007l5oooo00_oo`3o1_l0o`05onGUiA7o
O`001Oooo`0;o`3oo`Ko07l01Oooo`0;o`3o00_oo`002omo000;ool000Ko0?l01Omo000Fool000Ko
0?oo1oooool000Kooooo1_ooo`05ogmoO`KoiNGU1Ooo0?l;o`1o00_o001o1_l0o`0;ogmoO`Goo`00
1_l0o`05oooo00Ko001o1Ol0O`06ool000GoO`001_ooo`05o`3o00_o001o1Oooo`0;ool0o`Ko0?l0
1OoUiNDAogl000Goool02ol0ool6o`1o00Goool02ol0o`0;ool000_oO`002ooo0006o`3o00GoO`00
5_oo0006o`3oo`Oooooo0006ooooo`Koool01OmoOgl6onGUi@Goo`3o2ol0O`0;o`00O`Ko0?l02omo
Ogl5ool000Ko0?l01Oooo`06o`00O`Go07l01_oo0005ogl000Koool01Ol0o`0;o`00O`Goool02ooo
0?l6o`3o00GoiNGU4Omo0005oooo00_o0?oo1_l0O`05oooo00_o0?l02ooo000;ogl000_oo`001_l0
o`05ogl001Koo`001_l0ool7ooooo`001_ooool6oooo00GoOgmo1_oUiND5ool0o`_o07l02ol007l6
o`3o00_oOgmo1Ooo0006o`3o00Goool01_l007l5o`1o00Koo`001Omo0006oooo00Go0?l02ol007l5
oooo00_oo`3o1_l0o`05onGUiA7oO`001Oooo`0;o`3oo`Ko07l01Oooo`0;o`3o00_oo`002omo000;
ool000Ko0?l01Omo000Fool000Ko0?oo1oooool000Kooooo1_mo0005o`3o00Ko07l01Ol0o`06ool0
o`Goool01_l0O`05ogl000Ko07l01Oooo`06o`3o00GoiNGU1_l007l5ool000Ko0?l01Ol0O`06ogl0
00Goo`3o2omo0006ool000GoO`001Ol007l6ool0o`Goool01_oo0005o`3oo`KoOgmo1Ol0ool;o`00
O`Ko07l01Ol0ool6ogl000Go0?l01_l0O`0@o`00O`Ko07l01Ol007l6ogl000Go0?oo1_oUiND5ool0
00Koo`3o1Ooo0006onGUi@Go001o1_mo0007ooooo`001_ooool6ogl000Go0?l01_l0O`05o`3o00Ko
o`3o1Oooo`06o`1o00GoO`001_l0O`05oooo00Ko0?l01OoUiND6o`00O`Goo`001_l0o`05o`1o00Ko
O`001Ooo0?l;ogl000Koo`001Omo0005o`00O`Koo`3o1Oooo`06ool000Go0?oo1_moOgl5o`3oo`_o
001o1_l0O`05o`3oo`KoO`001Ol0o`06o`1o013o001o1_l0O`05o`00O`KoO`001Ol0ool6onGUi@Go
o`001_oo0?l5ool000KoiNGU1Ol007l6ogl000Oooooo0006ooooo`KoO`001Ol0o`06o`1o00Go0?l0
1_oo0?l5oooo00Ko07l01Omo0006o`1o00Goool01_l0o`05onGUi@Ko001o1Ooo0006o`3o00Go07l0
1_mo0005ool0o`_oO`001_oo0005ogl000Go001o1_oo0?l5oooo00Koo`001Ol0ool6ogmoO`Go0?oo
2ol007l6o`1o00Go0?oo1_mo0005o`3o00Ko07l04?l007l6o`1o00Go001o1_mo0005o`3oo`KoiNGU
1Ooo0006ool0o`Goo`001_oUiND5o`00O`KoO`001oooool000Kooooo1_mo0005o`3o00Ko07l01Ol0
o`06ool0o`Goool01_l0O`05ogl000Ko07l01Oooo`06o`3o00GoiNGU1_l007l5ool000Ko0?l01Ol0
O`06ogl000Goo`3o2omo0006ool000GoO`001Ol007l6ool0o`Goool01_oo0005o`3oo`KoOgmo1Ol0
ool;o`00O`Ko07l01Ol0ool6ogl000Go0?l01_l0O`0@o`00O`Ko07l01Ol007l6ogl000Go0?oo1_oU
iND5ool000Koo`3o1Ooo0006onGUi@Go001o1_mo0007ooooo`001_ooool6ogl000Go0?l01_l0O`05
o`3o00Koo`3o1Oooo`06o`1o00GoO`001_l0O`05oooo00Ko0?l01OoUiND6o`00O`Goo`001_l0o`05
o`1o00KoO`001Ooo0?l;ogl000Koo`001Omo0005o`00O`Koo`3o1Oooo`06ool000Go0?oo1_moOgl5
o`3oo`_o001o1_l0O`05o`3oo`KoO`001Ol0o`06o`1o013o001o1_l0O`05o`00O`KoO`001Ol0ool6
onGUi@Goo`001_oo0?l5ool000KoiNGU1Ol007l6ogl000Oooooo0006ooooo`Ko07l01Ol007l6ogmo
O`Go001o1_l0o`05oooo00KoO`001Ol0ool6oooo00Goo`3o1_moOgl5ool000KoO`001Ooo0?l6onGU
i@Go0?oo1_ooo`05ool000Ko0?l01Ooo0?l6ool000[o001o1_moOgl5oooo00KoOgmo2ooo0005o`3o
o`Ko0?l01OmoOgl6o`3o00Goool01_l0O`05ool000Ko0?oo1Ol0O`0;ool000Ko0?l01Oooo`0;ogl0
00Ko0?oo1Ooo0006oooo00Go0?oo1_mo0005o`3oo`Ko07l01oooool000Kooooo1_l0O`05o`00O`Ko
Ogmo1Ol007l6o`3o00Goool01_mo0005o`3oo`Koool01Ooo0?l6ogmoO`Goo`001_mo0005ool0o`Ko
iNGU1Ol0ool6oooo00Goo`001_l0o`05ool0o`Koo`002_l007l6ogmoO`Goool01_moOgl;ool000Go
0?oo1_l0o`05ogmoO`Ko0?l01Oooo`06o`1o00Goo`001_l0ool5o`1o00_oo`001_l0o`05oooo00_o
O`001_l0ool5ool000Koool01Ol0ool6ogl000Go0?oo1_l0O`07ooooo`001_ooool6o`1o00Go001o
1_moOgl5o`00O`Ko0?l01Oooo`06ogl000Go0?oo1_ooo`05ool0o`KoOgmo1Ooo0006ogl000Goo`3o
1_oUiND5o`3oo`Koool01Ooo0006o`3o00Goo`3o1_oo000:o`00O`KoOgmo1Oooo`06ogmoO`_oo`00
1Ol0ool6o`3o00GoOgmo1_l0o`05oooo00Ko07l01Ooo0006o`3oo`Go07l02ooo0006o`3o00Goool0
2omo0006o`3oo`Goo`001_ooo`05o`3oo`KoO`001Ol0ool6o`1o00Oooooo0006ooooo`Ko07l01Ol0
07l6ogmoO`Go001o1_l0o`05oooo00KoO`001Ol0ool6oooo00Goo`3o1_moOgl5ool000KoO`001Ooo
0?l6onGUi@Go0?oo1_ooo`05ool000Ko0?l01Ooo0?l6ool000[o001o1_moOgl5oooo00KoOgmo2ooo
0005o`3oo`Ko0?l01OmoOgl6o`3o00Goool01_l0O`05ool000Ko0?oo1Ol0O`0;ool000Ko0?l01Ooo
o`0;ogl000Ko0?oo1Ooo0006oooo00Go0?oo1_mo0005o`3oo`Ko07l01oooool000Kooooo1_l0O`05
o`00O`KoOgmo1Ol007l6o`3o00Goool01_mo0005o`3oo`Koool01Ooo0?l6ogmoO`Goo`001_mo0005
ool0o`KoiNGU1Ol0ool6oooo00Goo`001_l0o`05ool0o`Koo`002_l007l6ogmoO`Goool01_moOgl;
ool000Go0?oo1_l0o`05ogmoO`Ko0?l01Oooo`06o`1o00Goo`001_l0ool5o`1o00_oo`001_l0o`05
oooo00_oO`001_l0ool5ool000Koool01Ol0ool6ogl000Go0?oo1_l0O`07ooooo`001_ooool6o`3o
o`GoOgmo1_l007l5o`3o00Koo`001Ol007l6o`1o00Goool01_l0o`05onGUi@Ko001o1Ooo0006oooo
00GoO`001_ooo`05onGUi@Koo`001Ooo0?l6oooo00Go0?oo1_l0O`05o`00O`Go0?l01_ooo`05o`3o
o`Ko0?l01Ooo0?l6ool000Go07l01_moOgl5o`3o00Koo`001OmoOgl;ogl000Koool01Ooo0006o`3o
00GoiNGU1_l0ool5o`3o00Koo`001Omo000;o`3oo`Ko0?l01Ol0O`06o`3oo`Go0?l01_l007l7oooo
o`001_ooool6o`3oo`GoOgmo1_l007l5o`3o00Koo`001Ol007l6o`1o00Goool01_l0o`05onGUi@Ko
001o1Ooo0006oooo00GoO`001_ooo`05onGUi@Koo`001Ooo0?l6oooo00Go0?oo1_l0O`05o`00O`Go
0?l01_ooo`05o`3oo`Ko0?l01Ooo0?l6ool000Go07l01_moOgl5o`3o00Koo`001OmoOgl;ogl000Ko
ool01Ooo0006o`3o00GoiNGU1_l0ool5o`3o00Koo`001Omo000;o`3oo`Ko0?l01Ol0O`06o`3oo`Go
0?l01_l007l7ooooo`001_ooool6o`3oo`GoOgmo1_l007l5o`3o00Koo`001Ol007l6o`1o00Goool0
1_l0o`05onGUi@Ko001o1Ooo0006oooo00GoO`001_ooo`05onGUi@Koo`001Ooo0?l6oooo00Go0?oo
1_l0O`05o`00O`Go0?l01_ooo`05o`3oo`Ko0?l01Ooo0?l6ool000Go07l01_moOgl5o`3o00Koo`00
1OmoOgl;ogl000Koool01Ooo0006o`3o00GoiNGU1_l0ool5o`3o00Koo`001Omo000;o`3oo`Ko0?l0
1Ol0O`06o`3oo`Go0?l01_l007l7ooooo`001_ooool6o`3oo`GoOgmo1_l007l5o`3o00Koo`001Ol0
07l6o`1o00Goool01_l0o`05onGUi@Ko001o1Ooo0006oooo00GoO`001_ooo`05onGUi@Koo`001Ooo
0?l6oooo00Go0?oo1_l0O`05o`00O`Go0?l01_ooo`05o`3oo`Ko0?l01Ooo0?l6ool000Go07l01_mo
Ogl5o`3o00Koo`001OmoOgl;ogl000Koool01Ooo0006o`3o00GoiNGU1_l0ool5o`3o00Koo`001Omo
000;o`3oo`Ko0?l01Ol0O`06o`3oo`Go0?l01_l007l7ooooo`001_ooool6o`3oo`GoOgmo1_l007l5
o`3o00Koo`001Ol007l6o`1o00Goool01_l0o`05onGUi@Ko001o1Ooo0006oooo00GoO`001_ooo`05
onGUi@Koo`001Ooo0?l6oooo00Go0?oo1_l0O`05o`00O`Go0?l01_ooo`05o`3oo`Ko0?l01Ooo0?l6
ool000Go07l01_moOgl5o`3o00Koo`001OmoOgl;ogl000Koool01Ooo0006o`3o00GoiNGU1_l0ool5
o`3o00Koo`001Omo000;o`3oo`Ko0?l01Ol0O`06o`3oo`Go0?l01_l007l7ooooo`001_ooool6o`3o
o`GoOgmo1_l007l5o`3o00Koo`001Ol007l6o`1o00Goool01_l0o`05onGUi@Ko001o1Ooo0006oooo
00GoO`001_ooo`05onGUi@Koo`001Ooo0?l6oooo00Go0?oo1_l0O`05o`00O`Go0?l01_ooo`05o`3o
o`Ko0?l01Ooo0?l6ool000Go07l01_moOgl5o`3o00Koo`001OmoOgl;ogl000Koool01Ooo0006o`3o
00GoiNGU1_l0ool5o`3o00Koo`001Omo000;o`3oo`Ko0?l01Ol0O`06o`3oo`Go0?l01_l007l7oooo
o`001_ooool6ool000GoO`001_moOgl5o`1o00KoO`001Ol0o`06oooo00_oOgmo1Ol0O`06o`3oo`Go
Ogmo1_l0o`05ool000Koool01OoUiND;ogl000Koool02ol0ool5onGUi@GoO`001_l007l5o`3oo`_o
o`001_l007l5ool000KoO`001Ol007l6ogmoO`Go0?oo2ol007l6o`1o00Goo`001_l0O`05ool0o`Ko
001o1OmoOgl6o`3oo`Goo`3o1_l0O`05ogmoO`Ko0?oo1Ooo0006oooo00GoiNGU1_mo0007ooooo`00
1_ooool6ool000GoO`001_moOgl5o`1o00KoO`001Ol0o`06oooo00_oOgmo1Ol0O`06o`3oo`GoOgmo
1_l0o`05ool000Koool01OoUiND;ogl000Koool02ol0ool5onGUi@GoO`001_l007l5o`3oo`_oo`00
1_l007l5ool000KoO`001Ol007l6ogmoO`Go0?oo2ol007l6o`1o00Goo`001_l0O`05ool0o`Ko001o
1OmoOgl6o`3oo`Goo`3o1_l0O`05ogmoO`Ko0?oo1Ooo0006oooo00GoiNGU1_mo0007ooooo`001_oo
ool6ool000GoO`001_moOgl5o`1o00KoO`001Ol0o`06oooo00_oOgmo1Ol0O`06o`3oo`GoOgmo1_l0
o`05ool000Koool01OoUiND;ogl000Koool02ol0ool5onGUi@GoO`001_l007l5o`3oo`_oo`001_l0
07l5ool000KoO`001Ol007l6ogmoO`Go0?oo2ol007l6o`1o00Goo`001_l0O`05ool0o`Ko001o1Omo
Ogl6o`3oo`Goo`3o1_l0O`05ogmoO`Ko0?oo1Ooo0006oooo00GoiNGU1_mo0007ooooo`001_ooool6
ool000GoO`001_moOgl5o`1o00KoO`001Ol0o`06oooo00_oOgmo1Ol0O`06o`3oo`GoOgmo1_l0o`05
ool000Koool01OoUiND;ogl000Koool02ol0ool5onGUi@GoO`001_l007l5o`3oo`_oo`001_l007l5
ool000KoO`001Ol007l6ogmoO`Go0?oo2ol007l6o`1o00Goo`001_l0O`05ool0o`Ko001o1OmoOgl6
o`3oo`Goo`3o1_l0O`05ogmoO`Ko0?oo1Ooo0006oooo00GoiNGU1_mo0007ooooo`001_ooool6ool0
00GoO`001_moOgl5o`1o00KoO`001Ol0o`06oooo00_oOgmo1Ol0O`06o`3oo`GoOgmo1_l0o`05ool0
00Koool01OoUiND;ogl000Koool02ol0ool5onGUi@GoO`001_l007l5o`3oo`_oo`001_l007l5ool0
00KoO`001Ol007l6ogmoO`Go0?oo2ol007l6o`1o00Goo`001_l0O`05ool0o`Ko001o1OmoOgl6o`3o
o`Goo`3o1_l0O`05ogmoO`Ko0?oo1Ooo0006oooo00GoiNGU1_mo0007ooooo`001_ooool6ogl000Go
Ogmo1_oo0005oooo00Ko001o1OmoOgl6o`1o00_oOgmo1Ooo0006o`3oo`GoO`001_oo0?l5onGUi@Ko
o`002ol0ool;o`3o00_o001o1Ol0O`05o`00O`Ko0?oo1Ol0o`06o`3oo`Go001o2ooUiND6o`00O`Go
O`001_l0ool5o`1o00KoO`001Oooo`06o`1o00GoOgmo2oooo`06o`00O`Go0?oo1_oo0005ogl000Ko
0?l02ol0ool5onGUi@Koool01Ol0O`06oooo00Oooooo0006ooooo`KoO`001OmoOgl6ool000Goool0
1_l007l5ogmoO`Ko07l02omoOgl5ool000Ko0?oo1Omo0006ool0o`GoiNGU1_oo000;o`3oo`_o0?l0
2ol007l5o`1o00Go001o1_l0ool5o`3o00Ko0?oo1Ol007l;onGUi@Ko001o1Omo0006o`3oo`Go07l0
1_mo0005oooo00Ko07l01OmoOgl;oooo00Ko001o1Ol0ool6ool000GoO`001_l0o`0;o`3oo`GoiNGU
1_ooo`05o`1o00Koool01oooool000Kooooo1_mo0005ogmoO`Koo`001Oooo`06o`00O`GoOgmo1_l0
O`0;ogmoO`Goo`001_l0ool5ogl000Koo`3o1OoUiND6ool000_o0?oo2ol0o`0;o`00O`Go07l01Ol0
07l6o`3oo`Go0?l01_l0ool5o`00O`_oiNGU1_l007l5ogl000Ko0?oo1Ol0O`06ogl000Goool01_l0
O`05ogmoO`_oool01_l007l5o`3oo`Koo`001Omo0006o`3o00_o0?oo1OoUiND6oooo00Go07l01_oo
o`07ooooo`001_ooool6ogl000GoOgmo1_oo0005oooo00Ko001o1OmoOgl6o`1o00_oOgmo1Ooo0006
o`3oo`GoO`001_oo0?l5onGUi@Koo`002ol0ool;o`3o00_o001o1Ol0O`05o`00O`Ko0?oo1Ol0o`06
o`3oo`Go001o2ooUiND6o`00O`GoO`001_l0ool5o`1o00KoO`001Oooo`06o`1o00GoOgmo2oooo`06
o`00O`Go0?oo1_oo0005ogl000Ko0?l02ol0ool5onGUi@Koool01Ol0O`06oooo00Oooooo0006oooo
o`KoO`001OmoOgl6ool000Goool01_l007l5ogmoO`Ko07l02omoOgl5ool000Ko0?oo1Omo0006ool0
o`GoiNGU1_oo000;o`3oo`_o0?l02ol007l5o`1o00Go001o1_l0ool5o`3o00Ko0?oo1Ol007l;onGU
i@Ko001o1Omo0006o`3oo`Go07l01_mo0005oooo00Ko07l01OmoOgl;oooo00Ko001o1Ol0ool6ool0
00GoO`001_l0o`0;o`3oo`GoiNGU1_ooo`05o`1o00Koool01oooool000Kooooo1_mo0005ogmoO`Ko
o`001Oooo`06o`00O`GoOgmo1_l0O`0;ogmoO`Goo`001_l0ool5ogl000Koo`3o1OoUiND6ool000_o
0?oo2ol0o`0;o`00O`Go07l01Ol007l6o`3oo`Go0?l01_l0ool5o`00O`_oiNGU1_l007l5ogl000Ko
0?oo1Ol0O`06ogl000Goool01_l0O`05ogmoO`_oool01_l007l5o`3oo`Koo`001Omo0006o`3o00_o
0?oo1OoUiND6oooo00Go07l01_ooo`07ooooo`001_ooool6oooo00Go0?l01_l0O`05o`3o00KoO`00
1Ooo0?l;o`3o00KoOgmo2ol0o`05ogmoO`KoiNGU1Ooo0006ogl0013o0?l01_oo0005ogmoO`Ko001o
2_oo0006ogl000Goo`001_l0o`05ool000Koool01Ooo0?l6ool000_oool01OoUiND6o`3o00Goo`3o
1_oo0005ool0o`_oool01_l0ool5o`00O`Ko07l01Oooo`0;ogmoO`Ko0?oo1Ol0O`06o`3oo`Go0?l0
1_mo0007ooooo`001_ooool6oooo00Go0?l01_l0O`05o`3o00KoO`001Ooo0?l;o`3o00KoOgmo2ol0
o`05ogmoO`KoiNGU1Ooo0006ogl0013o0?l01_oo0005ogmoO`Ko001o2_oo0006ogl000Goo`001_l0
o`05ool000Koool01Ooo0?l6ool000_oool01OoUiND6o`3o00Goo`3o1_oo0005ool0o`_oool01_l0
ool5o`00O`Ko07l01Oooo`0;ogmoO`Ko0?oo1Ol0O`06o`3oo`Go0?l01_mo0007ooooo`001_ooool6
oooo00Go0?l01_l0O`05o`3o00KoO`001Ooo0?l;o`3o00KoOgmo2ol0o`05ogmoO`KoiNGU1Ooo0006
ogl0013o0?l01_oo0005ogmoO`Ko001o2_oo0006ogl000Goo`001_l0o`05ool000Koool01Ooo0?l6
ool000_oool01OoUiND6o`3o00Goo`3o1_oo0005ool0o`_oool01_l0ool5o`00O`Ko07l01Oooo`0;
ogmoO`Ko0?oo1Ol0O`06o`3oo`Go0?l01_mo0007ooooo`001_ooool6oooo00Go0?l01_l0O`05o`3o
00KoO`001Ooo0?l;o`3o00KoOgmo2ol0o`05ogmoO`KoiNGU1Ooo0006ogl0013o0?l01_oo0005ogmo
O`Ko001o2_oo0006ogl000Goo`001_l0o`05ool000Koool01Ooo0?l6ool000_oool01OoUiND6o`3o
00Goo`3o1_oo0005ool0o`_oool01_l0ool5o`00O`Ko07l01Oooo`0;ogmoO`Ko0?oo1Ol0O`06o`3o
o`Go0?l01_mo0007ooooo`001_ooool6oooo00Go0?l01_l0O`05o`3o00KoO`001Ooo0?l;o`3o00Ko
Ogmo2ol0o`05ogmoO`KoiNGU1Ooo0006ogl0013o0?l01_oo0005ogmoO`Ko001o2_oo0006ogl000Go
o`001_l0o`05ool000Koool01Ooo0?l6ool000_oool01OoUiND6o`3o00Goo`3o1_oo0005ool0o`_o
ool01_l0ool5o`00O`Ko07l01Oooo`0;ogmoO`Ko0?oo1Ol0O`06o`3oo`Go0?l01_mo0007ooooo`00
1_ooool6o`00O`Go0?l01_oo0005o`00O`KoiNGU1Omo0006ool0o`_o0?l01Ol007l;ool000Ko001o
1Ooo0006o`3o00Goool01_mo0005oooo00Koo`3o1OmoOgl6ool000[oool01_moOgl5o`00O`Koool0
1Ol0o`06ool0o`_oOgmo2ol007l5oooo00Ko001o1OmoOgl6o`3oo`Goo`001_l0o`05ogl000KoOgmo
1Ooo0006onGUi@Go001o1_oUiND5oooo00KoOgmo1Ol0ool;o`00O`KoO`001oooool000Kooooo1_l0
07l5o`3o00Koo`001Ol007l6onGUi@GoO`001_oo0?l;o`3o00Go001o2ooo0006o`00O`Goo`001_l0
o`05oooo00KoO`001Oooo`06ool0o`GoOgmo1_oo000:oooo00KoOgmo1Ol007l6oooo00Go0?l01_oo
0?l;ogmoO`_o001o1Oooo`06o`00O`GoOgmo1_l0ool5ool000Ko0?l01Omo0006ogmoO`Goo`001_oU
iND5o`00O`KoiNGU1Oooo`06ogmoO`Go0?oo2ol007l6ogl000Oooooo0006ooooo`Ko001o1Ol0o`06
ool000Go001o1_oUiND5ogl000Koo`3o2ol0o`05o`00O`_oo`001_l007l5ool000Ko0?l01Oooo`06
ogl000Goool01_oo0?l5ogmoO`Koo`002_ooo`06ogmoO`Go001o1_ooo`05o`3o00Koo`3o2omoOgl;
o`00O`Goool01_l007l5ogmoO`Ko0?oo1Ooo0006o`3o00GoO`001_moOgl5ool000KoiNGU1Ol007l6
onGUi@Goool01_moOgl5o`3oo`_o001o1_mo0007ooooo`001_ooool6o`00O`Go0?l01_oo0005o`00
O`KoiNGU1Omo0006ool0o`_o0?l01Ol007l;ool000Ko001o1Ooo0006o`3o00Goool01_mo0005oooo
00Koo`3o1OmoOgl6ool000[oool01_moOgl5o`00O`Koool01Ol0o`06ool0o`_oOgmo2ol007l5oooo
00Ko001o1OmoOgl6o`3oo`Goo`001_l0o`05ogl000KoOgmo1Ooo0006onGUi@Go001o1_oUiND5oooo
00KoOgmo1Ol0ool;o`00O`KoO`001oooool000Kooooo1_l007l5o`3o00Koo`001Ol007l6onGUi@Go
O`001_oo0?l;o`3o00Go001o2ooo0006o`00O`Goo`001_l0o`05oooo00KoO`001Oooo`06ool0o`Go
Ogmo1_oo000:oooo00KoOgmo1Ol007l6oooo00Go0?l01_oo0?l;ogmoO`_o001o1Oooo`06o`00O`Go
Ogmo1_l0ool5ool000Ko0?l01Omo0006ogmoO`Goo`001_oUiND5o`00O`KoiNGU1Oooo`06ogmoO`Go
0?oo2ol007l6ogl000Oooooo0006ooooo`Ko001o1Ol0o`06ool000Go001o1_oUiND5ogl000Koo`3o
2ol0o`05o`00O`_oo`001_l007l5ool000Ko0?l01Oooo`06ogl000Goool01_oo0?l5ogmoO`Koo`00
2_ooo`06ogmoO`Go001o1_ooo`05o`3o00Koo`3o2omoOgl;o`00O`Goool01_l007l5ogmoO`Ko0?oo
1Ooo0006o`3o00GoO`001_moOgl5ool000KoiNGU1Ol007l6onGUi@Goool01_moOgl5o`3oo`_o001o
1_mo0007ooooo`001_ooool6o`00O`GoO`001_ooo`0;o`1o00GoOgmo1_oo0005o`3o00Koo`001Omo
Ogl6ogl000_oool02ol0O`05onGUi@Ko07l01Ooo0006o`00O`GoOgmo1_mo0005ogmoO`_o001o1Ooo
o`06ool0o`Go07l01_ooo`05ogmoO`KoO`001Ooo0006o`3oo`Goo`3o1_oUiND5ogl000KoiNGU1Ooo
o`06o`3o00Goo`3o1_l0O`0;ogl000Goo`3o1_l0o`05ogl000Koool01Ol0ool6onGUi@_oool01ooo
ool000Kooooo1_l007l5ogl000Koool02ol0O`05ogmoO`Koo`001Ol0o`06ool000GoOgmo1_mo000;
oooo00_o07l01OoUiND6o`1o00Goo`001_l007l5ogmoO`KoO`001OmoOgl;o`00O`Goool01_oo0?l5
o`1o00Koool01OmoOgl6ogl000Goo`001_l0ool5ool0o`KoiNGU1Omo0006onGUi@Goool01_l0o`05
ool0o`Ko07l02omo0005ool0o`Ko0?l01Omo0006oooo00Go0?oo1_oUiND;oooo00Oooooo0006oooo
o`Ko001o1Omo0006oooo00_o07l01OmoOgl6ool000Go0?l01_oo0005ogmoO`KoO`002oooo`0;o`1o
00GoiNGU1_l0O`05ool000Ko001o1OmoOgl6ogl000GoOgmo2ol007l5oooo00Koo`3o1Ol0O`06oooo
00GoOgmo1_mo0005ool000Ko0?oo1Ooo0?l6onGUi@GoO`001_oUiND5oooo00Ko0?l01Ooo0?l6o`1o
00_oO`001Ooo0?l6o`3o00GoO`001_ooo`05o`3oo`KoiNGU2oooo`07ooooo`001_ooool6o`00O`Go
O`001_ooo`0;o`1o00GoOgmo1_oo0005o`3o00Koo`001OmoOgl6ogl000_oool02ol0O`05onGUi@Ko
07l01Ooo0006o`00O`GoOgmo1_mo0005ogmoO`_o001o1Oooo`06ool0o`Go07l01_ooo`05ogmoO`Ko
O`001Ooo0006o`3oo`Goo`3o1_oUiND5ogl000KoiNGU1Oooo`06o`3o00Goo`3o1_l0O`0;ogl000Go
o`3o1_l0o`05ogl000Koool01Ol0ool6onGUi@_oool01oooool000Kooooo1_l007l5ogl000Koool0
2ol0O`05ogmoO`Koo`001Ol0o`06ool000GoOgmo1_mo000;oooo00_o07l01OoUiND6o`1o00Goo`00
1_l007l5ogmoO`KoO`001OmoOgl;o`00O`Goool01_oo0?l5o`1o00Koool01OmoOgl6ogl000Goo`00
1_l0ool5ool0o`KoiNGU1Omo0006onGUi@Goool01_l0o`05ool0o`Ko07l02omo0005ool0o`Ko0?l0
1Omo0006oooo00Go0?oo1_oUiND;oooo00Oooooo0006ooooo`Koo`001Oooo`06ool000Go0?l02ooU
iND6ogl000Go0?l01_moOgl5oooo00KoiNGU1Omo0006ool000Go0?oo2ol0o`06ogl000GoOgmo1_oo
0?l5o`00O`KoiNGU1Omo0005ool0o`Koool01Ol0o`06o`1o00Go001o1_moOgl5ogl000Ko001o1Ooo
o`06o`1o00GoiNGU1_oo0?l5o`00O`KoiNGU1Ol007l6oooo00Go0?oo1_mo0005ool0o`Ko001o1Ooo
o`06o`3oo`Goo`3o1_l0O`05ool0o`Koo`001OmoOgl6o`3o00Oooooo0006ooooo`Koo`001Oooo`06
ool000Go0?l02ooUiND6ogl000Go0?l01_moOgl5oooo00KoiNGU1Omo0006ool000Go0?oo2ol0o`06
ogl000GoOgmo1_oo0?l5o`00O`KoiNGU1Omo0005ool0o`Koool01Ol0o`06o`1o00Go001o1_moOgl5
ogl000Ko001o1Oooo`06o`1o00GoiNGU1_oo0?l5o`00O`KoiNGU1Ol007l6oooo00Go0?oo1_mo0005
ool0o`Ko001o1Oooo`06o`3oo`Goo`3o1_l0O`05ool0o`Koo`001OmoOgl6o`3o00Oooooo0006oooo
o`Koo`001Oooo`06ool000Go0?l02ooUiND6ogl000Go0?l01_moOgl5oooo00KoiNGU1Omo0006ool0
00Go0?oo2ol0o`06ogl000GoOgmo1_oo0?l5o`00O`KoiNGU1Omo0005ool0o`Koool01Ol0o`06o`1o
00Go001o1_moOgl5ogl000Ko001o1Oooo`06o`1o00GoiNGU1_oo0?l5o`00O`KoiNGU1Ol007l6oooo
00Go0?oo1_mo0005ool0o`Ko001o1Oooo`06o`3oo`Goo`3o1_l0O`05ool0o`Koo`001OmoOgl6o`3o
00Oooooo0006ooooo`Koo`001Oooo`06ool000Go0?l02ooUiND6ogl000Go0?l01_moOgl5oooo00Ko
iNGU1Omo0006ool000Go0?oo2ol0o`06ogl000GoOgmo1_oo0?l5o`00O`KoiNGU1Omo0005ool0o`Ko
ool01Ol0o`06o`1o00Go001o1_moOgl5ogl000Ko001o1Oooo`06o`1o00GoiNGU1_oo0?l5o`00O`Ko
iNGU1Ol007l6oooo00Go0?oo1_mo0005ool0o`Ko001o1Oooo`06o`3oo`Goo`3o1_l0O`05ool0o`Ko
o`001OmoOgl6o`3o00Oooooo0006ooooo`Koo`001Oooo`06ool000Go0?l02ooUiND6ogl000Go0?l0
1_moOgl5oooo00KoiNGU1Omo0006ool000Go0?oo2ol0o`06ogl000GoOgmo1_oo0?l5o`00O`KoiNGU
1Omo0005ool0o`Koool01Ol0o`06o`1o00Go001o1_moOgl5ogl000Ko001o1Oooo`06o`1o00GoiNGU
1_oo0?l5o`00O`KoiNGU1Ol007l6oooo00Go0?oo1_mo0005ool0o`Ko001o1Oooo`06o`3oo`Goo`3o
1_l0O`05ool0o`Koo`001OmoOgl6o`3o00Oooooo0006ooooo`Koo`001Oooo`06ool000Go0?l02ooU
iND6ogl000Go0?l01_moOgl5oooo00KoiNGU1Omo0006ool000Go0?oo2ol0o`06ogl000GoOgmo1_oo
0?l5o`00O`KoiNGU1Omo0005ool0o`Koool01Ol0o`06o`1o00Go001o1_moOgl5ogl000Ko001o1Ooo
o`06o`1o00GoiNGU1_oo0?l5o`00O`KoiNGU1Ol007l6oooo00Go0?oo1_mo0005ool0o`Ko001o1Ooo
o`06o`3oo`Goo`3o1_l0O`05ool0o`Koo`001OmoOgl6o`3o00Oooooo0006ooooo`Koo`3o1Ol0O`06
o`3oo`Go0?l01_oo0?l5onGUi@Koool01Ooo0006onGUi@Goo`001_l0O`05ool000KoOgmo1Ol0o`06
onGUi@Goool01_oo0?l5o`00O`Koool01Ooo0?l6onGUi@Goo`3o1Ol0O`06ogl000Go001o1_l0o`05
o`00O`Ko0?l01Ol0ool6ool000Go001o1_moOgl5o`3oo`KoiNGU1Ooo0006ogmoO`GoiNGU1_l0o`05
o`00O`Ko0?l05_oUiND5ogmoO`Ko001o1OmoOgl6ool000Goo`3o1_moOgl7ooooo`001_ooool6ool0
o`Go07l01_l0ool5o`3o00Koo`3o1OoUiND6oooo00Goo`001_oUiND5ool000Ko07l01Ooo0006ogmo
O`Go0?l01_oUiND5oooo00Koo`3o1Ol007l6oooo00Goo`3o1_oUiND5ool0o`Go07l01_mo0005o`00
O`Ko0?l01Ol007l6o`3o00Go0?oo1_oo0005o`00O`KoOgmo1Ol0ool6onGUi@Goo`001_moOgl5onGU
i@Ko0?l01Ol007l6o`3o01KoiNGU1OmoOgl6o`00O`GoOgmo1_oo0005ool0o`KoOgmo1oooool000Ko
oooo1_oo0?l5o`1o00Ko0?oo1Ol0o`06ool0o`GoiNGU1_ooo`05ool000KoiNGU1Ooo0006o`1o00Go
o`001_moOgl5o`3o00KoiNGU1Oooo`06ool0o`Go001o1_ooo`05ool0o`KoiNGU1Ooo0?l5o`1o00Ko
O`001Ol007l6o`3o00Go001o1_l0o`05o`3oo`Koo`001Ol007l6ogmoO`Go0?oo1_oUiND5ool000Ko
Ogmo1OoUiND6o`3o00Go001o1_l0o`0FonGUi@GoOgmo1_l007l5ogmoO`Koo`001Ooo0?l6ogmoO`Oo
oooo0006ooooo`Koo`3o1Ol0O`06o`3oo`Go0?l01_oo0?l5onGUi@Koool01Ooo0006onGUi@Goo`00
1_l0O`05ool000KoOgmo1Ol0o`06onGUi@Goool01_oo0?l5o`00O`Koool01Ooo0?l6onGUi@Goo`3o
1Ol0O`06ogl000Go001o1_l0o`05o`00O`Ko0?l01Ol0ool6ool000Go001o1_moOgl5o`3oo`KoiNGU
1Ooo0006ogmoO`GoiNGU1_l0o`05o`00O`Ko0?l05_oUiND5ogmoO`Ko001o1OmoOgl6ool000Goo`3o
1_moOgl7ooooo`001_ooool6ool0o`Go07l01_l0ool5o`3o00Koo`3o1OoUiND6oooo00Goo`001_oU
iND5ool000Ko07l01Ooo0006ogmoO`Go0?l01_oUiND5oooo00Koo`3o1Ol007l6oooo00Goo`3o1_oU
iND5ool0o`Go07l01_mo0005o`00O`Ko0?l01Ol007l6o`3o00Go0?oo1_oo0005o`00O`KoOgmo1Ol0
ool6onGUi@Goo`001_moOgl5onGUi@Ko0?l01Ol007l6o`3o01KoiNGU1OmoOgl6o`00O`GoOgmo1_oo
0005ool0o`KoOgmo1oooool000Kooooo1_l007l5o`1o00Koo`001Ol0ool6oooo00Goo`3o1_l0ool5
oooo00Ko0?l01Ol007l6o`3oo`_oO`002ol0O`05o`00O`Koo`002ooUiND5ool0o`Ko001o1Oooo`05
o`3oo`KoO`001Ol0o`06ogl000Go001o2ol0O`06o`00O`Goo`3o1_ooo`0;ool0o`Go001o1_oo0?l5
o`1o00Ko0?l01Ooo0006ogl000Goo`3o1_mo0005o`1o00Koool01Omo0006ogmoO`Goo`002ol0ool6
onGUi@Oooooo0006ooooo`Ko001o1Ol0O`06ool000Go0?oo1_ooo`05ool0o`Ko0?oo1Oooo`06o`3o
00Go001o1_l0ool;ogl000_o07l01Ol007l6ool000_oiNGU1Ooo0?l6o`00O`Goool01Ol0ool6ogl0
00Go0?l01_mo0005o`00O`_o07l01_l007l5ool0o`Koool02ooo0?l5o`00O`Koo`3o1Ol0O`06o`3o
00Goo`001_mo0005ool0o`KoO`001Ol0O`06oooo00GoO`001_moOgl5ool000_o0?oo1_oUiND7oooo
o`001_ooool6o`00O`Go07l01_oo0005o`3oo`Koool01Ooo0?l6o`3oo`Goool01_l0o`05o`00O`Ko
0?oo2omo000;o`1o00Go001o1_oo000;onGUi@Goo`3o1_l007l5oooo00Go0?oo1_mo0005o`3o00Ko
O`001Ol007l;o`1o00Ko001o1Ooo0?l6oooo00_oo`3o1Ol007l6ool0o`Go07l01_l0o`05ool000Ko
O`001Ooo0?l6ogl000Go07l01_ooo`05ogl000KoOgmo1Ooo000;o`3oo`KoiNGU1oooool000Kooooo
1_l007l5o`1o00Koo`001Ol0ool6oooo00Goo`3o1_l0ool5oooo00Ko0?l01Ol007l6o`3oo`_oO`00
2ol0O`05o`00O`Koo`002ooUiND5ool0o`Ko001o1Oooo`05o`3oo`KoO`001Ol0o`06ogl000Go001o
2ol0O`06o`00O`Goo`3o1_ooo`0;ool0o`Go001o1_oo0?l5o`1o00Ko0?l01Ooo0006ogl000Goo`3o
1_mo0005o`1o00Koool01Omo0006ogmoO`Goo`002ol0ool6onGUi@Oooooo0006ooooo`Ko001o1Ol0
O`06ool000Go0?oo1_ooo`05ool0o`Ko0?oo1Oooo`06o`3o00Go001o1_l0ool;ogl000_o07l01Ol0
07l6ool000_oiNGU1Ooo0?l6o`00O`Goool01Ol0ool6ogl000Go0?l01_mo0005o`00O`_o07l01_l0
07l5ool0o`Koool02ooo0?l5o`00O`Koo`3o1Ol0O`06o`3o00Goo`001_mo0005ool0o`KoO`001Ol0
O`06oooo00GoO`001_moOgl5ool000_o0?oo1_oUiND7ooooo`001_ooool;o`3oo`Ko001o1Oooo`0;
onGUi@_oO`001_l0o`0;ool0o`Go0?oo2omo0006o`3o00GoOgmo1_l007l5o`3oo`KoO`001Ol0ool6
o`3o00Goool01Ol007l6o`1o00GoiNGU1_oo000;o`3o00Go001o1_ooo`0;o`1o00_o0?l01Ol007l6
onGUi@Go0?oo1_oo0005o`3oo`KoO`001Ol007l6o`1o00Goo`3o1_l007l5ool0o`Koo`001Ooo0?l6
ogl000Goo`3o1_oUiND7ooooo`001_ooool;o`3oo`Ko001o1Oooo`0;onGUi@_oO`001_l0o`0;ool0
o`Go0?oo2omo0006o`3o00GoOgmo1_l007l5o`3oo`KoO`001Ol0ool6o`3o00Goool01Ol007l6o`1o
00GoiNGU1_oo000;o`3o00Go001o1_ooo`0;o`1o00_o0?l01Ol007l6onGUi@Go0?oo1_oo0005o`3o
o`KoO`001Ol007l6o`1o00Goo`3o1_l007l5ool0o`Koo`001Ooo0?l6ogl000Goo`3o1_oUiND7oooo
o`001_ooool;o`3oo`Ko001o1Oooo`0;onGUi@_oO`001_l0o`0;ool0o`Go0?oo2omo0006o`3o00Go
Ogmo1_l007l5o`3oo`KoO`001Ol0ool6o`3o00Goool01Ol007l6o`1o00GoiNGU1_oo000;o`3o00Go
001o1_ooo`0;o`1o00_o0?l01Ol007l6onGUi@Go0?oo1_oo0005o`3oo`KoO`001Ol007l6o`1o00Go
o`3o1_l007l5ool0o`Koo`001Ooo0?l6ogl000Goo`3o1_oUiND7ooooo`001_ooool;o`3oo`Ko001o
1Oooo`0;onGUi@_oO`001_l0o`0;ool0o`Go0?oo2omo0006o`3o00GoOgmo1_l007l5o`3oo`KoO`00
1Ol0ool6o`3o00Goool01Ol007l6o`1o00GoiNGU1_oo000;o`3o00Go001o1_ooo`0;o`1o00_o0?l0
1Ol007l6onGUi@Go0?oo1_oo0005o`3oo`KoO`001Ol007l6o`1o00Goo`3o1_l007l5ool0o`Koo`00
1Ooo0?l6ogl000Goo`3o1_oUiND7ooooo`001_ooool;o`3oo`Ko001o1Oooo`0;onGUi@_oO`001_l0
o`0;ool0o`Go0?oo2omo0006o`3o00GoOgmo1_l007l5o`3oo`KoO`001Ol0ool6o`3o00Goool01Ol0
07l6o`1o00GoiNGU1_oo000;o`3o00Go001o1_ooo`0;o`1o00_o0?l01Ol007l6onGUi@Go0?oo1_oo
0005o`3oo`KoO`001Ol007l6o`1o00Goo`3o1_l007l5ool0o`Koo`001Ooo0?l6ogl000Goo`3o1_oU
iND7ooooo`001_ooool;o`3oo`Ko001o1Oooo`0;onGUi@_oO`001_l0o`0;ool0o`Go0?oo2omo0006
o`3o00GoOgmo1_l007l5o`3oo`KoO`001Ol0ool6o`3o00Goool01Ol007l6o`1o00GoiNGU1_oo000;
o`3o00Go001o1_ooo`0;o`1o00_o0?l01Ol007l6onGUi@Go0?oo1_oo0005o`3oo`KoO`001Ol007l6
o`1o00Goo`3o1_l007l5ool0o`Koo`001Ooo0?l6ogl000Goo`3o1_oUiND7ooooo`001_ooool;onGU
i@Ko07l01OmoOgl6oooo00Go0?l01_moOgl5ool000KoiNGU1Ooo000;o`3o00Koo`002omo0005o`3o
o`KoO`001Ooo0?l;oooo00KoiNGU1Ooo0005onGUi@Ko0?oo1OoUiND6o`3oo`GoiNGU1_mo0005o`1o
00Koo`3o1Ol0o`06o`1o00GoO`001_l007l5ool0o`Ko0?l01Oooo`06o`3oo`Goool02ol0O`06ool0
00Go0?l01_oo0?l5oooo00KoOgmo1Ooo000;o`1o00Koo`001oooool000Kooooo2ooUiND6o`1o00Go
Ogmo1_ooo`05o`3o00KoOgmo1Ooo0006onGUi@Goo`002ol0o`06ool000_oO`001Ol0ool6ogl000Go
o`3o2oooo`06onGUi@Goo`001OoUiND6o`3oo`GoiNGU1_l0ool5onGUi@KoO`001Ol0O`06ool0o`Go
0?l01_l0O`05ogl000Ko001o1Ooo0?l6o`3o00Goool01_l0ool5oooo00_o07l01_oo0005o`3o00Ko
o`3o1Oooo`06ogmoO`Goo`002ol0O`06ool000Oooooo0006ooooo`_oiNGU1_l0O`05ogmoO`Koool0
1Ol0o`06ogmoO`Goo`001_oUiND5ool000_o0?l01_oo000;ogl000Go0?oo1_mo0005ool0o`_oool0
1_oUiND5ool000GoiNGU1_l0ool5onGUi@Ko0?oo1OoUiND6ogl000Go07l01_oo0?l5o`3o00Ko07l0
1Omo0006o`00O`Goo`3o1_l0o`05oooo00Ko0?oo1Oooo`0;o`1o00Koo`001Ol0o`06ool0o`Goool0
1_moOgl5ool000_o07l01_oo0007ooooo`001_ooool;onGUi@Ko07l01OmoOgl6oooo00Go0?l01_mo
Ogl5ool000KoiNGU1Ooo000;o`3o00Koo`002omo0005o`3oo`KoO`001Ooo0?l;oooo00KoiNGU1Ooo
0005onGUi@Ko0?oo1OoUiND6o`3oo`GoiNGU1_mo0005o`1o00Koo`3o1Ol0o`06o`1o00GoO`001_l0
07l5ool0o`Ko0?l01Oooo`06o`3oo`Goool02ol0O`06ool000Go0?l01_oo0?l5oooo00KoOgmo1Ooo
000;o`1o00Koo`001oooool000Kooooo2ooUiND6o`1o00GoOgmo1_ooo`05o`3o00KoOgmo1Ooo0006
onGUi@Goo`002ol0o`06ool000_oO`001Ol0ool6ogl000Goo`3o2oooo`06onGUi@Goo`001OoUiND6
o`3oo`GoiNGU1_l0ool5onGUi@KoO`001Ol0O`06ool0o`Go0?l01_l0O`05ogl000Ko001o1Ooo0?l6
o`3o00Goool01_l0ool5oooo00_o07l01_oo0005o`3o00Koo`3o1Oooo`06ogmoO`Goo`002ol0O`06
ool000Oooooo0006ooooo`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGU
i@Ko001o1Ol0o`06o`00O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo
00KoOgmo1Ooo0006o`3oo`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGU
i@Goool01_oo0005ogl000Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmo
O`Oooooo0006ooooo`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGUi@Ko
001o1Ol0o`06o`00O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo00Ko
Ogmo1Ooo0006o`3oo`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGUi@Go
ool01_oo0005ogl000Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmoO`Oo
oooo0006ooooo`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGUi@Ko001o
1Ol0o`06o`00O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo00KoOgmo
1Ooo0006o`3oo`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGUi@Goool0
1_oo0005ogl000Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmoO`Oooooo
0006ooooo`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGUi@Ko001o1Ol0
o`06o`00O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo00KoOgmo1Ooo
0006o`3oo`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGUi@Goool01_oo
0005ogl000Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmoO`Oooooo0006
ooooo`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGUi@Ko001o1Ol0o`06
o`00O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo00KoOgmo1Ooo0006
o`3oo`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGUi@Goool01_oo0005
ogl000Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmoO`Oooooo0006oooo
o`Ko001o1Oooo`06ogl000_oo`001OoUiND6o`00O`Goo`3o1_l0O`05onGUi@Ko001o1Ol0o`06o`00
O`Go0?l01_oUiND5o`3o00KoiNGU1Ol0o`0;o`1o00Koo`001Ol007l5oooo00KoOgmo1Ooo0006o`3o
o`GoiNGU2omoOgl6o`00O`Goo`3o1_l007l5o`3oo`Ko0?l01Ooo0006onGUi@Goool01_oo0005ogl0
00Ko07l01Ooo0006o`1o00_o001o1Ol0O`06oooo00_oo`3o1Ol007l6ogmoO`Oooooo0006ooooo`Ko
0?l01Oooo`06o`00O`GoiNGU1_mo0005o`00O`Ko07l01Ol0ool6onGUi@Go0?oo1_ooo`05o`3o00Ko
Ogmo1Omo0006onGUi@Goo`3o1_l0ool5oooo00KoO`002ooo0?l5o`1o00Goo`3o2oooo`06ogmoO`Go
o`3o1_moOgl5o`00O`Ko0?l02ooUiND;o`3o00Go001o1_l0ool5o`3o00Ko0?oo1Ol0o`06ool0o`Go
ool01_oUiND5ogmoO`Ko0?l01Oooo`06ool000GoOgmo1_l0ool5ogmoO`Koool01oooool000Kooooo
1_l0o`05oooo00Ko001o1OoUiND6ogl000Go001o1_l0O`05o`3oo`KoiNGU1Ol0ool6oooo00Go0?l0
1_moOgl5ogl000KoiNGU1Ooo0?l6o`3oo`Goool01_mo000;ool0o`Go07l01Ooo0?l;oooo00KoOgmo
1Ooo0?l6ogmoO`Go001o1_l0o`0;onGUi@_o0?l01Ol007l6o`3oo`Go0?l01_l0ool5o`3o00Koo`3o
1Oooo`06onGUi@GoOgmo1_l0o`05oooo00Koo`001OmoOgl6o`3oo`GoOgmo1_ooo`07ooooo`001_oo
ool6o`3o00Goool01_l007l5onGUi@KoO`001Ol007l6o`1o00Go0?oo1_oUiND5o`3oo`Koool01Ol0
o`06ogmoO`GoO`001_oUiND5ool0o`Ko0?oo1Oooo`06ogl000_oo`3o1Ol0O`05ool0o`_oool01_mo
Ogl5ool0o`KoOgmo1Ol007l6o`3o00_oiNGU2ol0o`05o`00O`Ko0?oo1Ol0o`06o`3oo`Go0?l01_oo
0?l5oooo00KoiNGU1OmoOgl6o`3o00Goool01_oo0005ogmoO`Ko0?oo1OmoOgl6oooo00Oooooo0006
ooooo`Ko0?l01Oooo`06o`00O`GoiNGU1_mo0005o`00O`Ko07l01Ol0ool6onGUi@Go0?oo1_ooo`05
o`3o00KoOgmo1Omo0006onGUi@Goo`3o1_l0ool5oooo00KoO`002ooo0?l5o`1o00Goo`3o2oooo`06
ogmoO`Goo`3o1_moOgl5o`00O`Ko0?l02ooUiND;o`3o00Go001o1_l0ool5o`3o00Ko0?oo1Ol0o`06
ool0o`Goool01_oUiND5ogmoO`Ko0?l01Oooo`06ool000GoOgmo1_l0ool5ogmoO`Koool01oooool0
00Kooooo1_l0o`05oooo00Ko001o1OoUiND6ogl000Go001o1_l0O`05o`3oo`KoiNGU1Ol0ool6oooo
00Go0?l01_moOgl5ogl000KoiNGU1Ooo0?l6o`3oo`Goool01_mo000;ool0o`Go07l01Ooo0?l;oooo
00KoOgmo1Ooo0?l6ogmoO`Go001o1_l0o`0;onGUi@_o0?l01Ol007l6o`3oo`Go0?l01_l0ool5o`3o
00Koo`3o1Oooo`06onGUi@GoOgmo1_l0o`05oooo00Koo`001OmoOgl6o`3oo`GoOgmo1_ooo`07oooo
o`001_ooool6o`00O`Goo`3o1_moOgl5o`00O`KoiNGU1Oooo`06o`3o00GoOgmo1_ooo`05o`1o00Ko
o`3o1OoUiND6o`00O`GoiNGU1_oo0005ogmoO`Ko0?oo1Ooo0?l6onGUi@Go07l01_oUiND5o`1o00Go
001o1_mo0005oooo00KoOgmo2ooo0005o`3o00Ko0?oo1Ooo0006ool0o`Go0?oo1_l0o`05ool000Ko
001o1OmoOgl6ool000Go001o2ol0O`06oooo00GoiNGU1_l0o`0;o`1o00GoiNGU2ol0o`06ool000Oo
oooo0006ooooo`Ko001o1Ooo0?l6ogmoO`Go001o1_oUiND5oooo00Ko0?l01OmoOgl6oooo00Go07l0
1_oo0?l5onGUi@Ko001o1OoUiND6ool000GoOgmo1_l0ool5ool0o`KoiNGU1Ol0O`06onGUi@Go07l0
1Ol007l6ogl000Goool01_moOgl;ool000Go0?l01_l0ool5ool000Koo`3o1Ol0ool6o`3o00Goo`00
1_l007l5ogmoO`Koo`001Ol007l;o`1o00Koool01OoUiND6o`3o00_o07l01OoUiND;o`3o00Koo`00
1oooool000Kooooo1_l007l5ool0o`KoOgmo1Ol007l6onGUi@Goool01_l0o`05ogmoO`Koool01Ol0
O`06ool0o`GoiNGU1_l007l5onGUi@Koo`001OmoOgl6o`3oo`Goo`3o1_oUiND5o`1o00KoiNGU1Ol0
O`05o`00O`KoO`001Oooo`06ogmoO`_oo`001Ol0o`06o`3oo`Goo`001_oo0?l5o`3oo`Ko0?l01Ooo
0006o`00O`GoOgmo1_oo0005o`00O`_o07l01_ooo`05onGUi@Ko0?l02ol0O`05onGUi@_o0?l01_oo
0007ooooo`001_ooool6o`00O`Goo`3o1_moOgl5o`00O`KoiNGU1Oooo`06o`3o00GoOgmo1_ooo`05
o`1o00Koo`3o1OoUiND6o`00O`GoiNGU1_oo0005ogmoO`Ko0?oo1Ooo0?l6onGUi@Go07l01_oUiND5
o`1o00Go001o1_mo0005oooo00KoOgmo2ooo0005o`3o00Ko0?oo1Ooo0006ool0o`Go0?oo1_l0o`05
ool000Ko001o1OmoOgl6ool000Go001o2ol0O`06oooo00GoiNGU1_l0o`0;o`1o00GoiNGU2ol0o`06
ool000Oooooo0006ooooo`Ko001o1Ooo0?l6ogmoO`Go001o1_oUiND5oooo00Ko0?l01OmoOgl6oooo
00Go07l01_oo0?l5onGUi@Ko001o1OoUiND6ool000GoOgmo1_l0ool5ool0o`KoiNGU1Ol0O`06onGU
i@Go07l01Ol007l6ogl000Goool01_moOgl;ool000Go0?l01_l0ool5ool000Koo`3o1Ol0ool6o`3o
00Goo`001_l007l5ogmoO`Koo`001Ol007l;o`1o00Koool01OoUiND6o`3o00_o07l01OoUiND;o`3o
00Koo`001oooool000Kooooo1_l007l5ool0o`KoOgmo1Ol007l6onGUi@Goool01_l0o`05ogmoO`Ko
ool01Ol0O`06ool0o`GoiNGU1_l007l5onGUi@Koo`001OmoOgl6o`3oo`Goo`3o1_oUiND5o`1o00Ko
iNGU1Ol0O`05o`00O`KoO`001Oooo`06ogmoO`_oo`001Ol0o`06o`3oo`Goo`001_oo0?l5o`3oo`Ko
0?l01Ooo0006o`00O`GoOgmo1_oo0005o`00O`_o07l01_ooo`05onGUi@Ko0?l02ol0O`05onGUi@_o
0?l01_oo0007ooooo`001_ooool6o`3oo`Goo`002omo0006oooo00Go0?l01_l0O`0;ool0o`Go0?oo
1_oo0005o`3o00Koo`3o2ooUiND5ogl000KoiNGU1Ol0O`06ool0o`Go07l01_l0o`05o`1o00GoO`00
1_oo0?l;o`00O`Go0?l01_l007l5ogl000_oiNGU2ol0o`06ool0o`Goo`001_oUiND5ool000Koo`3o
1Ol0O`06ogmoO`Go07l01_oo0005o`3oo`KoO`001Oooo`06ool0o`Goo`001_oo0?l5ool000Koo`3o
1oooool000Kooooo1_l0ool5ool000_oO`001_ooo`05o`3o00Ko07l02ooo0?l5o`3oo`Koo`001Ol0
o`06ool0o`_oiNGU1Omo0006onGUi@Go07l01_oo0?l5o`1o00Ko0?l01Ol0O`05ogl000Koo`3o2ol0
07l5o`3o00Ko001o1Omo000;onGUi@_o0?l01_oo0?l5ool000KoiNGU1Ooo0006ool0o`Go07l01_mo
Ogl5o`1o00Koo`001Ol0ool6ogl000Goool01_oo0?l5ool000Koo`3o1Ooo0006ool0o`Oooooo0006
ooooo`Ko0?oo1Ooo000;ogl000Koool01Ol0o`06o`1o00_oo`3o1Ol0ool6ool000Go0?l01_oo0?l;
onGUi@GoO`001_oUiND5o`1o00Koo`3o1Ol0O`06o`3o00Go07l01Omo0006ool0o`_o001o1Ol0o`06
o`00O`GoO`002ooUiND;o`3o00Koo`3o1Ooo0006onGUi@Goo`001_oo0?l5o`1o00KoOgmo1Ol0O`06
ool000Go0?oo1_mo0005oooo00Koo`3o1Ooo0006ool0o`Goo`001_oo0?l7ooooo`001_ooool6o`3o
o`Goo`002omo0006oooo00Go0?l01_l0O`0;ool0o`Go0?oo1_oo0005o`3o00Koo`3o2ooUiND5ogl0
00KoiNGU1Ol0O`06ool0o`Go07l01_l0o`05o`1o00GoO`001_oo0?l;o`00O`Go0?l01_l007l5ogl0
00_oiNGU2ol0o`06ool0o`Goo`001_oUiND5ool000Koo`3o1Ol0O`06ogmoO`Go07l01_oo0005o`3o
o`KoO`001Oooo`06ool0o`Goo`001_oo0?l5ool000Koo`3o1oooool000Kooooo1_l0ool5ool000_o
O`001_ooo`05o`3o00Ko07l02ooo0?l5o`3oo`Koo`001Ol0o`06ool0o`_oiNGU1Omo0006onGUi@Go
07l01_oo0?l5o`1o00Ko0?l01Ol0O`05ogl000Koo`3o2ol007l5o`3o00Ko001o1Omo000;onGUi@_o
0?l01_oo0?l5ool000KoiNGU1Ooo0006ool0o`Go07l01_moOgl5o`1o00Koo`001Ol0ool6ogl000Go
ool01_oo0?l5ool000Koo`3o1Ooo0006ool0o`Oooooo0006ooooo`KoiNGU1Ol0o`06ogl000Goo`00
1_oo0?l;ool000Go07l02ol0ool6o`00O`_oiNGU1OmoOgl6o`00O`Go0?oo1_oo0?l5o`1o00KoOgmo
1Ol0ool6ool0o`Go0?oo1Ooo0?l;o`3o00Ko07l01Ol0ool6ool0o`Go001o2omoOgl6oooo00Go07l0
1_mo0005oooo00Koo`001Ol0O`06ool000Goo`3o1_oUiND5oooo00Ko001o1Oooo`06o`1o00Go0?oo
1_l0O`05oooo00KoiNGU2ooo0007ooooo`001_ooool6onGUi@Go0?l01_mo0005ool000Koo`3o2ooo
0005o`1o00_o0?oo1_l007l;onGUi@GoOgmo1_l007l5o`3oo`Koo`3o1Ol0O`06ogmoO`Go0?oo1_oo
0?l5o`3oo`Goo`3o2ol0o`06o`1o00Go0?oo1_oo0?l5o`00O`_oOgmo1_ooo`05o`1o00KoO`001Ooo
o`06ool000Go07l01_oo0005ool0o`KoiNGU1Oooo`06o`00O`Goool01_l0O`05o`3oo`Ko07l01Ooo
o`06onGUi@_oo`001oooool000Kooooo1_oUiND5o`3o00KoO`001Ooo0006ool0o`_oo`001Ol0O`0;
o`3oo`Ko001o2ooUiND5ogmoO`Ko001o1Ol0ool6ool0o`Go07l01_moOgl5o`3oo`Koo`3o1Ol0ool5
ool0o`_o0?l01_l0O`05o`3oo`Koo`3o1Ol007l;ogmoO`Koool01Ol0O`06ogl000Goool01_oo0005
o`1o00Koo`001Ooo0?l6onGUi@Goool01_l007l5oooo00Ko07l01Ol0ool6o`1o00Goool01_oUiND;
ool000Oooooo0006ooooo`KoiNGU1Ol0o`06ogl000Goo`001_oo0?l;ool000Go07l02ol0ool6o`00
O`_oiNGU1OmoOgl6o`00O`Go0?oo1_oo0?l5o`1o00KoOgmo1Ol0ool6ool0o`Go0?oo1Ooo0?l;o`3o
00Ko07l01Ol0ool6ool0o`Go001o2omoOgl6oooo00Go07l01_mo0005oooo00Koo`001Ol0O`06ool0
00Goo`3o1_oUiND5oooo00Ko001o1Oooo`06o`1o00Go0?oo1_l0O`05oooo00KoiNGU2ooo0007oooo
o`001_ooool6onGUi@Go0?l01_mo0005ool000Koo`3o2ooo0005o`1o00_o0?oo1_l007l;onGUi@Go
Ogmo1_l007l5o`3oo`Koo`3o1Ol0O`06ogmoO`Go0?oo1_oo0?l5o`3oo`Goo`3o2ol0o`06o`1o00Go
0?oo1_oo0?l5o`00O`_oOgmo1_ooo`05o`1o00KoO`001Oooo`06ool000Go07l01_oo0005ool0o`Ko
iNGU1Oooo`06o`00O`Goool01_l0O`05o`3oo`Ko07l01Oooo`06onGUi@_oo`001oooool000Kooooo
1_oUiND5o`3o00KoO`001Ooo0006ool0o`_oo`001Ol0O`0;o`3oo`Ko001o2ooUiND5ogmoO`Ko001o
1Ol0ool6ool0o`Go07l01_moOgl5o`3oo`Koo`3o1Ol0ool5ool0o`_o0?l01_l0O`05o`3oo`Koo`3o
1Ol007l;ogmoO`Koool01Ol0O`06ogl000Goool01_oo0005o`1o00Koo`001Ooo0?l6onGUi@Goool0
1_l007l5oooo00Ko07l01Ol0ool6o`1o00Goool01_oUiND;ool000Oooooo0006ooooo`KoiNGU1Ooo
0?l;ool000KoOgmo1OoUiND;ogl000Koool01Omo0006o`3oo`Go07l01_l007l5oooo00_o0?l01_l0
ool5o`00O`Koo`3o1OoUiND6ool0o`[oiNGU1_l0O`05oooo00Ko001o1Ol0O`06ogl000Goo`3o1_oo
0005ogmoO`Ko07l01Ooo0?l6onGUi@Go07l01_ooo`05o`3oo`KoO`001Oooo`0;o`00O`Ko07l01OoU
iND6ogmoO`_o001o1Ol0O`06ogmoO`Go07l01_oo0?l7ooooo`001_ooool6onGUi@Goo`3o2ooo0006
ogmoO`GoiNGU2omo0006oooo00GoO`001_l0ool5o`1o00Ko001o1Oooo`0;o`3o00Ko0?oo1Ol007l6
ool0o`GoiNGU1_oo0?l:onGUi@Ko07l01Oooo`06o`00O`Go07l01_mo0005ool0o`Koo`001OmoOgl6
o`1o00Goo`3o1_oUiND5o`1o00Koool01Ol0ool6ogl000Goool02ol007l6o`1o00GoiNGU1_moOgl;
o`00O`Go07l01_moOgl5o`1o00Koo`3o1oooool000Kooooo1_oUiND5ool0o`_oo`001_moOgl5onGU
i@_oO`001_ooo`05ogl000Ko0?oo1Ol0O`06o`00O`Goool02ol0o`06o`3oo`Go001o1_oo0?l5onGU
i@Koo`3o2_oUiND6o`1o00Goool01_l007l5o`1o00KoO`001Ooo0?l6ool000GoOgmo1_l0O`05ool0
o`KoiNGU1Ol0O`06oooo00Go0?oo1_mo0005oooo00_o001o1_l0O`05onGUi@KoOgmo2ol007l5o`1o
00KoOgmo1Ol0O`06ool0o`Oooooo0006ooooo`KoiNGU1Ooo0?l;ool000KoOgmo1OoUiND;ogl000Ko
ool01Omo0006o`3oo`Go07l01_l007l5oooo00_o0?l01_l0ool5o`00O`Koo`3o1OoUiND6ool0o`[o
iNGU1_l0O`05oooo00Ko001o1Ol0O`06ogl000Goo`3o1_oo0005ogmoO`Ko07l01Ooo0?l6onGUi@Go
07l01_ooo`05o`3oo`KoO`001Oooo`0;o`00O`Ko07l01OoUiND6ogmoO`_o001o1Ol0O`06ogmoO`Go
07l01_oo0?l7ooooo`001_ooool6onGUi@Goo`3o2ooo0006ogmoO`GoiNGU2omo0006oooo00GoO`00
1_l0ool5o`1o00Ko001o1Oooo`0;o`3o00Ko0?oo1Ol007l6ool0o`GoiNGU1_oo0?l:onGUi@Ko07l0
1Oooo`06o`00O`Go07l01_mo0005ool0o`Koo`001OmoOgl6o`1o00Goo`3o1_oUiND5o`1o00Koool0
1Ol0ool6ogl000Goool02ol007l6o`1o00GoiNGU1_moOgl;o`00O`Go07l01_moOgl5o`1o00Koo`3o
1oooool000Kooooo1_l0O`0;ogmoO`GoiNGU1_l0ool5o`3o00Koool01Ol0ool;ogl000KoiNGU1Ol0
O`06ogmoO`GoO`002omoOgl6ool0o`Goool01_moOgl5o`3oo`KoO`001Ol007l5onGUi@KoO`001Ol0
07l6o`3oo`Go07l02ooo0006o`00O`GoO`002oooo`0;o`3o00Ko001o1Ol0O`06ool0o`Goool01_l0
ool5ool0o`Ko07l01Ooo0?l6o`1o00GoO`001_l0o`05ool000Koo`3o1OmoOgl6ool000Oooooo0006
ooooo`Ko07l02omoOgl5onGUi@Ko0?oo1Ol0o`06oooo00Go0?oo2omo0006onGUi@Go07l01_moOgl5
ogl000_oOgmo1_oo0?l5oooo00KoOgmo1Ol0ool6ogl000Go001o1OoUiND6ogl000Go001o1_l0ool5
o`1o00_oo`001_l007l5ogl000_oool02ol0o`06o`00O`Go07l01_oo0?l5oooo00Ko0?oo1Ooo0?l6
o`1o00Goo`3o1_l0O`05ogl000Ko0?l01Ooo0006ool0o`GoOgmo1_oo0007ooooo`001_ooool6o`1o
00_oOgmo1OoUiND6o`3oo`Go0?l01_ooo`05o`3oo`_oO`001_oUiND5o`1o00KoOgmo1Omo000;ogmo
O`Koo`3o1Oooo`06ogmoO`Go0?oo1_mo0005o`00O`GoiNGU1_mo0005o`00O`Ko0?oo1Ol0O`0;ool0
00Ko001o1Omo000;oooo00_o0?l01_l007l5o`1o00Koo`3o1Oooo`06o`3oo`Goo`3o1_l0O`05ool0
o`Ko07l01Omo0006o`3o00Goo`001_oo0?l5ogmoO`Koo`001oooool000Kooooo1_l0O`0;ogmoO`Go
iNGU1_l0ool5o`3o00Koool01Ol0ool;ogl000KoiNGU1Ol0O`06ogmoO`GoO`002omoOgl6ool0o`Go
ool01_moOgl5o`3oo`KoO`001Ol007l5onGUi@KoO`001Ol007l6o`3oo`Go07l02ooo0006o`00O`Go
O`002oooo`0;o`3o00Ko001o1Ol0O`06ool0o`Goool01_l0ool5ool0o`Ko07l01Ooo0?l6o`1o00Go
O`001_l0o`05ool000Koo`3o1OmoOgl6ool000Oooooo0006ooooo`Ko07l02omoOgl5onGUi@Ko0?oo
1Ol0o`06oooo00Go0?oo2omo0006onGUi@Go07l01_moOgl5ogl000_oOgmo1_oo0?l5oooo00KoOgmo
1Ol0ool6ogl000Go001o1OoUiND6ogl000Go001o1_l0ool5o`1o00_oo`001_l007l5ogl000_oool0
2ol0o`06o`00O`Go07l01_oo0?l5oooo00Ko0?oo1Ooo0?l6o`1o00Goo`3o1_l0O`05ogl000Ko0?l0
1Ooo0006ool0o`GoOgmo1_oo0007ooooo`001_ooool;o`1o00KoOgmo1Ooo0006o`3o00Goo`3o1_mo
0005ool0o`Ko0?l01Ooo000;o`00O`Ko07l01Omo0006ogmoO`Goo`3o1_moOgl5onGUi@Ko001o1OoU
iND6oooo00Go001o1Ol0o`06ogl000GoiNGU1_l0ool5ogmoO`Koo`3o1OmoOgl;onGUi@Ko0?oo1Ooo
o`06o`3o00Goo`001_moOgl5oooo00KoOgmo2ol0ool5o`3o00KoiNGU1Ol007l6ogl000Go0?l01_mo
0005o`1o00KoOgmo1Ol0o`06oooo00Oooooo0006ooooo`_o07l01_moOgl5ool000Ko0?l01Ooo0?l6
ogl000Goo`3o1_l0o`05ool000_o001o1_l0O`05ogl000KoOgmo1Ooo0?l6ogmoO`GoiNGU1_l007l5
onGUi@Koool01Ol007l5o`3o00KoO`001OoUiND6o`3oo`GoOgmo1_oo0?l5ogmoO`_oiNGU1_l0ool5
oooo00Ko0?l01Ooo0006ogmoO`Goool01_moOgl;o`3oo`Go0?l01_oUiND5o`00O`KoO`001Ol0o`06
ogl000Go07l01_moOgl5o`3o00Koool01oooool000Kooooo2ol0O`06ogmoO`Goo`001_l0o`05ool0
o`KoO`001Ooo0?l6o`3o00Goo`002ol007l6o`1o00GoO`001_moOgl5ool0o`KoOgmo1OoUiND6o`00
O`GoiNGU1_ooo`05o`00O`Go0?l01_mo0005onGUi@Ko0?oo1OmoOgl6ool0o`GoOgmo2ooUiND6o`3o
o`Goool01_l0o`05ool000KoOgmo1Oooo`06ogmoO`_o0?oo1Ol0o`06onGUi@Go001o1_mo0005o`3o
00KoO`001Ol0O`06ogmoO`Go0?l01_ooo`07ooooo`001_ooool;o`1o00KoOgmo1Ooo0006o`3o00Go
o`3o1_mo0005ool0o`Ko0?l01Ooo000;o`00O`Ko07l01Omo0006ogmoO`Goo`3o1_moOgl5onGUi@Ko
001o1OoUiND6oooo00Go001o1Ol0o`06ogl000GoiNGU1_l0ool5ogmoO`Koo`3o1OmoOgl;onGUi@Ko
0?oo1Oooo`06o`3o00Goo`001_moOgl5oooo00KoOgmo2ol0ool5o`3o00KoiNGU1Ol007l6ogl000Go
0?l01_mo0005o`1o00KoOgmo1Ol0o`06oooo00Oooooo0006ooooo`_o07l01_moOgl5ool000Ko0?l0
1Ooo0?l6ogl000Goo`3o1_l0o`05ool000_o001o1_l0O`05ogl000KoOgmo1Ooo0?l6ogmoO`GoiNGU
1_l007l5onGUi@Koool01Ol007l5o`3o00KoO`001OoUiND6o`3oo`GoOgmo1_oo0?l5ogmoO`_oiNGU
1_l0ool5oooo00Ko0?l01Ooo0006ogmoO`Goool01_moOgl;o`3oo`Go0?l01_oUiND5o`00O`KoO`00
1Ol0o`06ogl000Go07l01_moOgl5o`3o00Koool01oooool000Kooooo2ol0O`06ogmoO`Goo`001_l0
o`05ool0o`KoO`001Ooo0?l6o`3o00Goo`002ol007l6o`1o00GoO`001_moOgl5ool0o`KoOgmo1OoU
iND6o`00O`GoiNGU1_ooo`05o`00O`Go0?l01_mo0005onGUi@Ko0?oo1OmoOgl6ool0o`GoOgmo2ooU
iND6o`3oo`Goool01_l0o`05ool000KoOgmo1Oooo`06ogmoO`_o0?oo1Ol0o`06onGUi@Go001o1_mo
0005o`3o00KoO`001Ol0O`06ogmoO`Go0?l01_ooo`07ooooo`001_ooool6ogl000Go0?l01_mo000;
onGUi@Go0?oo1_l007l5o`3oo`Ko001o1Oooo`06o`00O`Go0?l01_oo0005o`3o00Koool01Ol0O`0;
ool0o`Ko0?l01Ol0O`06oooo00Go0?oo1OmoOgl6onGUi@Go07l01_oo0005o`3oo`Ko07l01Ooo0006
ool0o`_oiNGU1Ol0ool6o`00O`GoO`001_moOgl5onGUi@_oOgmo1_l0O`05ogl000Ko0?oo1Omo0006
o`3oo`Goo`001_oo0?l5onGUi@Koo`001Omo0006o`1o00Oooooo0006ooooo`KoO`001Ol0o`06ogl0
00_oiNGU1Ol0ool6o`00O`Go0?oo1_l007l5oooo00Ko001o1Ol0o`06ool000Go0?l01_ooo`05o`1o
00_oo`3o1_l0o`05o`1o00Koool01Ol0ool5ogmoO`KoiNGU1Ol0O`06ool000Go0?oo1_l0O`05ool0
00Koo`3o2ooUiND5o`3oo`Ko001o1Omo0006ogmoO`GoiNGU2omoOgl6o`1o00GoO`001_l0ool5ogl0
00Ko0?oo1Ooo0006ool0o`GoiNGU1_oo0005ogl000Ko07l01oooool000Kooooo1_mo0005o`3o00Ko
O`002ooUiND5o`3oo`Ko001o1Ol0ool6o`00O`Goool01_l007l5o`3o00Koo`001Ol0o`06oooo00Go
07l02ooo0?l6o`3o00Go07l01_ooo`05o`3oo`GoOgmo1_oUiND5o`1o00Koo`001Ol0ool6o`1o00Go
o`001_oo0?l;onGUi@Go0?oo1_l007l5ogl000KoOgmo1OoUiND;ogmoO`Ko07l01Omo0006o`3oo`Go
O`001_l0ool5ool000Koo`3o1OoUiND6ool000GoO`001_l0O`07ooooo`001_ooool6ogl000Go0?l0
1_mo000;onGUi@Go0?oo1_l007l5o`3oo`Ko001o1Oooo`06o`00O`Go0?l01_oo0005o`3o00Koool0
1Ol0O`0;ool0o`Ko0?l01Ol0O`06oooo00Go0?oo1OmoOgl6onGUi@Go07l01_oo0005o`3oo`Ko07l0
1Ooo0006ool0o`_oiNGU1Ol0ool6o`00O`GoO`001_moOgl5onGUi@_oOgmo1_l0O`05ogl000Ko0?oo
1Omo0006o`3oo`Goo`001_oo0?l5onGUi@Koo`001Omo0006o`1o00Oooooo0006ooooo`KoO`001Ol0
o`06ogl000_oiNGU1Ol0ool6o`00O`Go0?oo1_l007l5oooo00Ko001o1Ol0o`06ool000Go0?l01_oo
o`05o`1o00_oo`3o1_l0o`05o`1o00Koool01Ol0ool5ogmoO`KoiNGU1Ol0O`06ool000Go0?oo1_l0
O`05ool000Koo`3o2ooUiND5o`3oo`Ko001o1Omo0006ogmoO`GoiNGU2omoOgl6o`1o00GoO`001_l0
ool5ogl000Ko0?oo1Ooo0006ool0o`GoiNGU1_oo0005ogl000Ko07l01oooool000Kooooo4Ooo0?l5
o`00O`Ko07l01Ol0o`06ool0o`Go0?oo1_l0O`05onGUi@Ko001o1Oooo`06ogmoO`Goo`3o1_l0O`05
ogmoO`Ko001o1OoUiND6ool0o`Go0?oo1_l007l5ogl000Goo`001_l0O`05o`3oo`Koo`3o1Ol007l;
o`1o00KoiNGU1Ooo0?l6ool000Go07l01_l0ool5o`3o00Ko07l01Oooo`06o`3oo`Goo`3o1_l0ool5
o`1o00Koool01OmoOgl6o`3oo`GoiNGU1_l0ool5o`1o00Ko0?oo1OmoOgl6ool0o`Oooooo0006oooo
oa7oo`3o1Ol007l6o`1o00Go0?l01_oo0?l5o`3oo`Ko07l01OoUiND6o`00O`Goool01_moOgl5ool0
o`Ko07l01OmoOgl6o`00O`GoiNGU1_oo0?l5o`3oo`Ko001o1Omo0005ool000Ko07l01Ol0ool6ool0
o`Go001o2ol0O`06onGUi@Goo`3o1_oo0005o`1o00Ko0?oo1Ol0o`06o`1o00Goool01_l0ool5ool0
o`Ko0?oo1Ol0O`06oooo00GoOgmo1_l0ool5onGUi@Ko0?oo1Ol0O`06o`3oo`GoOgmo1_oo0?l7oooo
o`001_oooolAool0o`Go001o1_l0O`05o`3o00Koo`3o1Ol0ool6o`1o00GoiNGU1_l007l5oooo00Ko
Ogmo1Ooo0?l6o`1o00GoOgmo1_l007l5onGUi@Koo`3o1Ol0ool6o`00O`GoO`001Ooo0006o`1o00Go
0?oo1_oo0?l5o`00O`_o07l01_oUiND5ool0o`Koo`001Ol0O`06o`3oo`Go0?l01_l0O`05oooo00Ko
0?oo1Ooo0?l6o`3oo`Go07l01_ooo`05ogmoO`Ko0?oo1OoUiND6o`3oo`Go07l01_l0ool5ogmoO`Ko
o`3o1oooool000Kooooo4Ooo0?l5o`00O`Ko07l01Ol0o`06ool0o`Go0?oo1_l0O`05onGUi@Ko001o
1Oooo`06ogmoO`Goo`3o1_l0O`05ogmoO`Ko001o1OoUiND6ool0o`Go0?oo1_l007l5ogl000Goo`00
1_l0O`05o`3oo`Koo`3o1Ol007l;o`1o00KoiNGU1Ooo0?l6ool000Go07l01_l0ool5o`3o00Ko07l0
1Oooo`06o`3oo`Goo`3o1_l0ool5o`1o00Koool01OmoOgl6o`3oo`GoiNGU1_l0ool5o`1o00Ko0?oo
1OmoOgl6ool0o`Oooooo0006oooooa7oo`3o1Ol007l6o`1o00Go0?l01_oo0?l5o`3oo`Ko07l01OoU
iND6o`00O`Goool01_moOgl5ool0o`Ko07l01OmoOgl6o`00O`GoiNGU1_oo0?l5o`3oo`Ko001o1Omo
0005ool000Ko07l01Ol0ool6ool0o`Go001o2ol0O`06onGUi@Goo`3o1_oo0005o`1o00Ko0?oo1Ol0
o`06o`1o00Goool01_l0ool5ool0o`Ko0?oo1Ol0O`06oooo00GoOgmo1_l0ool5onGUi@Ko0?oo1Ol0
O`06o`3oo`GoOgmo1_oo0?l7ooooo`001_oooolAool0o`Go001o1_l0O`05o`3o00Koo`3o1Ol0ool6
o`1o00GoiNGU1_l007l5oooo00KoOgmo1Ooo0?l6o`1o00GoOgmo1_l007l5onGUi@Koo`3o1Ol0ool6
o`00O`GoO`001Ooo0006o`1o00Go0?oo1_oo0?l5o`00O`_o07l01_oUiND5ool0o`Koo`001Ol0O`06
o`3oo`Go0?l01_l0O`05oooo00Ko0?oo1Ooo0?l6o`3oo`Go07l01_ooo`05ogmoO`Ko0?oo1OoUiND6
o`3oo`Go07l01_l0ool5ogmoO`Koo`3o1oooool000Kooooo1_l0ool5o`1o00Ko001o1OmoOgl6o`1o
00Go001o2omo0006ogmoO`Go0?oo1_l0O`05oooo00Koo`3o1OmoOgl6o`00O`GoO`001_ooo`05ogl0
00Koo`001OoUiND6o`3o00GoO`001Ol0O`06onGUi@Go001o1_oo0005o`3oo`Koool01OoUiND;o`3o
o`KoiNGU1Omo0006o`1o013oool01_oUiND5ool000Koo`3o1Ol0O`06ogmoO`Go0?oo2ooo0?l6onGU
iA3oO`001_l0O`07ooooo`001_ooool6o`3oo`Go07l01_l007l5ogmoO`Ko07l01Ol007l;ogl000Ko
Ogmo1Ol0ool6o`1o00Goool01_oo0?l5ogmoO`Ko001o1Omo0006oooo00GoO`001_oo0005onGUi@Ko
0?l01Omo0005o`1o00KoiNGU1Ol007l6ool000Go0?oo1_ooo`05onGUi@_o0?oo1_oUiND5ogl000Ko
07l04?ooo`06onGUi@Goo`001_oo0?l5o`1o00KoOgmo1Ol0ool;ool0o`KoiNGU4?mo0006o`1o00Oo
oooo0006ooooo`Ko0?oo1Ol0O`06o`00O`GoOgmo1_l0O`05o`00O`_oO`001_moOgl5o`3oo`Ko07l0
1Oooo`06ool0o`GoOgmo1_l007l5ogl000Koool01Omo0006ool000GoiNGU1_l0o`05ogl000Go07l0
1_oUiND5o`00O`Koo`001Ol0ool6oooo00GoiNGU2ol0ool6onGUi@GoO`001_l0O`0@oooo00KoiNGU
1Ooo0006ool0o`Go07l01_moOgl5o`3oo`_oo`3o1_oUiND@ogl000Ko07l01oooool000Kooooo1_l0
ool5o`1o00Ko001o1OmoOgl6o`1o00Go001o2omo0006ogmoO`Go0?oo1_l0O`05oooo00Koo`3o1Omo
Ogl6o`00O`GoO`001_ooo`05ogl000Koo`001OoUiND6o`3o00GoO`001Ol0O`06onGUi@Go001o1_oo
0005o`3oo`Koool01OoUiND;o`3oo`KoiNGU1Omo0006o`1o013oool01_oUiND5ool000Koo`3o1Ol0
O`06ogmoO`Go0?oo2ooo0?l6onGUiA3oO`001_l0O`07ooooo`001_ooool6o`3oo`Go07l01_l007l5
ogmoO`Ko07l01Ol007l;ogl000KoOgmo1Ol0ool6o`1o00Goool01_oo0?l5ogmoO`Ko001o1Omo0006
oooo00GoO`001_oo0005onGUi@Ko0?l01Omo0005o`1o00KoiNGU1Ol007l6ool000Go0?oo1_ooo`05
onGUi@_o0?oo1_oUiND5ogl000Ko07l04?ooo`06onGUi@Goo`001_oo0?l5o`1o00KoOgmo1Ol0ool;
ool0o`KoiNGU4?mo0006o`1o00Oooooo0006ooooo`Ko001o1Oooo`06o`1o00GoO`002ol0o`0;ool0
o`Ko0?oo2ooo0?l5ool000KoiNGU1Oooo`06o`1o00GoO`001_l0o`05ogmoO`KoiNGU1Ooo0006onGU
i@Goool01Ooo0006o`3o00Goo`3o1_l007l5onGUi@KoO`001Ol0o`06ogl000GoOgmo1_l007l5ool0
00Ko0?oo1Omo0006o`3oo`_oOgmo1Ol007l6ool000GoOgmo2omo0006o`1o00Goo`3o1_moOgl5o`1o
00_o0?l01_moOgl7ooooo`001_ooool6o`00O`Goool01_l0O`05ogl000_o0?l02ooo0?l6o`3oo`_o
o`3o1Ooo0006onGUi@Goool01_l0O`05ogl000Ko0?l01OmoOgl6onGUi@Goo`001_oUiND5oooo00Go
o`001_l0o`05ool0o`Ko001o1OoUiND6ogl000Go0?l01_mo0005ogmoO`Ko001o1Ooo0006o`3oo`Go
O`001_l0ool;ogmoO`Go001o1_oo0005ogmoO`_oO`001_l0O`05ool0o`KoOgmo1Ol0O`0;o`3o00Ko
Ogmo1oooool000Kooooo1_l007l5oooo00Ko07l01Omo000;o`3o00_oo`3o1_l0ool;ool0o`Goo`00
1_oUiND5oooo00Ko07l01Omo0006o`3o00GoOgmo1_oUiND5ool000KoiNGU1Oooo`05ool000Ko0?l0
1Ooo0?l6o`00O`GoiNGU1_mo0005o`3o00KoO`001OmoOgl6o`00O`Goo`001_l0ool5ogl000Ko0?oo
2omoOgl5o`00O`Koo`001OmoOgl;ogl000Ko07l01Ooo0?l6ogmoO`Go07l02ol0o`06ogmoO`Oooooo
0006ooooo`Ko001o1Oooo`06o`1o00GoO`002ol0o`0;ool0o`Ko0?oo2ooo0?l5ool000KoiNGU1Ooo
o`06o`1o00GoO`001_l0o`05ogmoO`KoiNGU1Ooo0006onGUi@Goool01Ooo0006o`3o00Goo`3o1_l0
07l5onGUi@KoO`001Ol0o`06ogl000GoOgmo1_l007l5ool000Ko0?oo1Omo0006o`3oo`_oOgmo1Ol0
07l6ool000GoOgmo2omo0006o`1o00Goo`3o1_moOgl5o`1o00_o0?l01_moOgl7ooooo`001_ooool6
o`00O`Goool01_l0O`05ogl000_o0?l02ooo0?l6o`3oo`_oo`3o1Ooo0006onGUi@Goool01_l0O`05
ogl000Ko0?l01OmoOgl6onGUi@Goo`001_oUiND5oooo00Goo`001_l0o`05ool0o`Ko001o1OoUiND6
ogl000Go0?l01_mo0005ogmoO`Ko001o1Ooo0006o`3oo`GoO`001_l0ool;ogmoO`Go001o1_oo0005
ogmoO`_oO`001_l0O`05ool0o`KoOgmo1Ol0O`0;o`3o00KoOgmo1oooool000Kooooo1_l007l5oooo
00Ko07l01Omo000;o`3o00_oo`3o1_l0ool;ool0o`Goo`001_oUiND5oooo00Ko07l01Omo0006o`3o
00GoOgmo1_oUiND5ool000KoiNGU1Oooo`05ool000Ko0?l01Ooo0?l6o`00O`GoiNGU1_mo0005o`3o
00KoO`001OmoOgl6o`00O`Goo`001_l0ool5ogl000Ko0?oo2omoOgl5o`00O`Koo`001OmoOgl;ogl0
00Ko07l01Ooo0?l6ogmoO`Go07l02ol0o`06ogmoO`Oooooo0006ooooo`Ko07l01Ol007l;onGUi@Ko
07l01Ol0o`06onGUi@_o0?oo1OmoOgl6onGUi@Go001o1_oo0005o`1o00Koool01Ooo0?l6onGUi@_o
0?l01Ooo0?l6o`00O`Goo`001Ol0ool6oooo00Goo`3o1_l0o`05oooo00Ko0?l01OoUiNDAo`3o00Go
o`3o1_l0O`05oooo00Ko07l01Ooo0006ool0o`Go001o1_ooo`05onGUi@Ko0?l01OmoOgl6ool000Go
ool01_oo0?l5o`00O`Ko0?oo1Ol0O`06oooo00Oooooo0006ooooo`Ko07l01Ol007l;onGUi@Ko07l0
1Ol0o`06onGUi@_o0?oo1OmoOgl6onGUi@Go001o1_oo0005o`1o00Koool01Ooo0?l6onGUi@_o0?l0
1Ooo0?l6o`00O`Goo`001Ol0ool6oooo00Goo`3o1_l0o`05oooo00Ko0?l01OoUiNDAo`3o00Goo`3o
1_l0O`05oooo00Ko07l01Ooo0006ool0o`Go001o1_ooo`05onGUi@Ko0?l01OmoOgl6ool000Goool0
1_oo0?l5o`00O`Ko0?oo1Ol0O`06oooo00Oooooo0006ooooo`Ko07l01Ol007l;onGUi@Ko07l01Ol0
o`06onGUi@_o0?oo1OmoOgl6onGUi@Go001o1_oo0005o`1o00Koool01Ooo0?l6onGUi@_o0?l01Ooo
0?l6o`00O`Goo`001Ol0ool6oooo00Goo`3o1_l0o`05oooo00Ko0?l01OoUiNDAo`3o00Goo`3o1_l0
O`05oooo00Ko07l01Ooo0006ool0o`Go001o1_ooo`05onGUi@Ko0?l01OmoOgl6ool000Goool01_oo
0?l5o`00O`Ko0?oo1Ol0O`06oooo00Oooooo0006ooooo`Ko07l01Ol007l;onGUi@Ko07l01Ol0o`06
onGUi@_o0?oo1OmoOgl6onGUi@Go001o1_oo0005o`1o00Koool01Ooo0?l6onGUi@_o0?l01Ooo0?l6
o`00O`Goo`001Ol0ool6oooo00Goo`3o1_l0o`05oooo00Ko0?l01OoUiNDAo`3o00Goo`3o1_l0O`05
oooo00Ko07l01Ooo0006ool0o`Go001o1_ooo`05onGUi@Ko0?l01OmoOgl6ool000Goool01_oo0?l5
o`00O`Ko0?oo1Ol0O`06oooo00Oooooo0006ooooo`Ko07l01Ol007l;onGUi@Ko07l01Ol0o`06onGU
i@_o0?oo1OmoOgl6onGUi@Go001o1_oo0005o`1o00Koool01Ooo0?l6onGUi@_o0?l01Ooo0?l6o`00
O`Goo`001Ol0ool6oooo00Goo`3o1_l0o`05oooo00Ko0?l01OoUiNDAo`3o00Goo`3o1_l0O`05oooo
00Ko07l01Ooo0006ool0o`Go001o1_ooo`05onGUi@Ko0?l01OmoOgl6ool000Goool01_oo0?l5o`00
O`Ko0?oo1Ol0O`06oooo00Oooooo0006ooooo`Ko001o1Ooo000;o`3oo`Koool02ol0ool;oooo00_o
001o1OmoOgl6onGUi@Goo`3o1_l007l;oooo00GoOgmo1_oo0?l5o`3oo`Koo`001OoUiND5ogl000Ko
0?l01Ol007l6ogmoO`Go0?l01_l007l5ool0o`Ko0?oo1Ol0O`06ogmoO`GoiNGU1_ooo`05o`00O`Ko
iNGU1OmoOgl6o`1o00GoO`002ooo0?l6oooo00Goo`3o1_oo0005o`1o00Ko001o1OmoOgl6o`3o00Go
Ogmo1_mo0007ooooo`001_ooool6o`00O`Goo`002ol0ool6oooo00_o0?oo2oooo`0;o`00O`GoOgmo
1_oUiND5ool0o`Ko001o2oooo`05ogmoO`Koo`3o1Ol0ool6ool000GoiNGU1Omo0006o`3o00Go001o
1_moOgl5o`3o00Ko001o1Ooo0?l6o`3oo`Go07l01_moOgl5onGUi@Koool01Ol007l6onGUi@GoOgmo
1_l0O`05ogl000_oo`3o1_ooo`05ool0o`Koo`001Ol0O`06o`00O`GoOgmo1_l0o`05ogmoO`KoO`00
1oooool000Kooooo1_l007l5ool000_o0?oo1_ooo`0;o`3oo`_oool02ol007l5ogmoO`KoiNGU1Ooo
0?l6o`00O`_oool01OmoOgl6ool0o`Go0?oo1_oo0005onGUi@GoO`001_l0o`05o`00O`KoOgmo1Ol0
o`06o`00O`Goo`3o1_l0ool5o`1o00KoOgmo1OoUiND6oooo00Go001o1_oUiND5ogmoO`Ko07l01Omo
000;ool0o`Koool01Ooo0?l6ool000Go07l01_l007l5ogmoO`Ko0?l01OmoOgl6ogl000Oooooo0006
ooooo`Ko001o1Ooo000;o`3oo`Koool02ol0ool;oooo00_o001o1OmoOgl6onGUi@Goo`3o1_l007l;
oooo00GoOgmo1_oo0?l5o`3oo`Koo`001OoUiND5ogl000Ko0?l01Ol007l6ogmoO`Go0?l01_l007l5
ool0o`Ko0?oo1Ol0O`06ogmoO`GoiNGU1_ooo`05o`00O`KoiNGU1OmoOgl6o`1o00GoO`002ooo0?l6
oooo00Goo`3o1_oo0005o`1o00Ko001o1OmoOgl6o`3o00GoOgmo1_mo0007ooooo`001_ooool6o`00
O`Goo`002ol0ool6oooo00_o0?oo2oooo`0;o`00O`GoOgmo1_oUiND5ool0o`Ko001o2oooo`05ogmo
O`Koo`3o1Ol0ool6ool000GoiNGU1Omo0006o`3o00Go001o1_moOgl5o`3o00Ko001o1Ooo0?l6o`3o
o`Go07l01_moOgl5onGUi@Koool01Ol007l6onGUi@GoOgmo1_l0O`05ogl000_oo`3o1_ooo`05ool0
o`Koo`001Ol0O`06o`00O`GoOgmo1_l0o`05ogmoO`KoO`001oooool000Kooooo1_l007l5ool000_o
0?oo1_ooo`0;o`3oo`_oool02ol007l5ogmoO`KoiNGU1Ooo0?l6o`00O`_oool01OmoOgl6ool0o`Go
0?oo1_oo0005onGUi@GoO`001_l0o`05o`00O`KoOgmo1Ol0o`06o`00O`Goo`3o1_l0ool5o`1o00Ko
Ogmo1OoUiND6oooo00Go001o1_oUiND5ogmoO`Ko07l01Omo000;ool0o`Koool01Ooo0?l6ool000Go
07l01_l007l5ogmoO`Ko0?l01OmoOgl6ogl000Oooooo0006ooooo`Ko001o1Ooo000;ogl000Ko07l0
1Ooo0006ool0o`Go001o1_mo0005ogmoOa7o001o1Ooo0006o`00O`Goo`3o1_oUiND5o`00O`Ko0?oo
1OmoOgl6o`3oo`GoiNGU1OmoOgl;ool0o`Ko0?l01Ol007l6oooo00Go0?oo1_l0o`05o`00O`KoO`00
1Ol0O`06ool000Go001o1_l0ool5ool0o`Ko001o1Ooo0006o`3oo`GoOgmo1_oUiND5ool0o`KoiNGU
1Oooo`06o`3o00Go07l01_l0ool5o`00O`Koo`3o1oooool000Kooooo1_l007l5ool000_oO`001_l0
O`05ool000Koo`3o1Ol007l6ogl000GoOgmo4Ol007l5ool000Ko001o1Ooo0?l6onGUi@Go001o1_l0
ool5ogmoO`Ko0?oo1OoUiND5ogmoO`_oo`3o1_l0o`05o`00O`Koool01Ol0ool6o`3o00Go001o1_mo
0005o`1o00Koo`001Ol007l6o`3oo`Goo`3o1_l007l5ool000Ko0?oo1OmoOgl6onGUi@Goo`3o1_oU
iND5oooo00Ko0?l01Ol0O`06o`3oo`Go001o1_oo0?l7ooooo`001_ooool6o`00O`Goo`002omo0006
o`1o00Goo`001_oo0?l5o`00O`KoO`001OmoOglAo`00O`Goo`001_l007l5ool0o`KoiNGU1Ol007l6
o`3oo`GoOgmo1_l0ool5onGUi@GoOgmo2ooo0?l6o`3o00Go001o1_ooo`05o`3oo`Ko0?l01Ol007l6
ogl000Go07l01_oo0005o`00O`Ko0?oo1Ooo0?l6o`00O`Goo`001_l0ool5ogmoO`KoiNGU1Ooo0?l6
onGUi@Goool01_l0o`05o`1o00Ko0?oo1Ol007l6ool0o`Oooooo0006ooooo`Ko001o1Ooo000;ogl0
00Ko07l01Ooo0006ool0o`Go001o1_mo0005ogmoOa7o001o1Ooo0006o`00O`Goo`3o1_oUiND5o`00
O`Ko0?oo1OmoOgl6o`3oo`GoiNGU1OmoOgl;ool0o`Ko0?l01Ol007l6oooo00Go0?oo1_l0o`05o`00
O`KoO`001Ol0O`06ool000Go001o1_l0ool5ool0o`Ko001o1Ooo0006o`3oo`GoOgmo1_oUiND5ool0
o`KoiNGU1Oooo`06o`3o00Go07l01_l0ool5o`00O`Koo`3o1oooool000Kooooo1_l007l5ool000_o
O`001_l0O`05ool000Koo`3o1Ol007l6ogl000GoOgmo4Ol007l5ool000Ko001o1Ooo0?l6onGUi@Go
001o1_l0ool5ogmoO`Ko0?oo1OoUiND5ogmoO`_oo`3o1_l0o`05o`00O`Koool01Ol0ool6o`3o00Go
001o1_mo0005o`1o00Koo`001Ol007l6o`3oo`Goo`3o1_l007l5ool000Ko0?oo1OmoOgl6onGUi@Go
o`3o1_oUiND5oooo00Ko0?l01Ol0O`06o`3oo`Go001o1_oo0?l7ooooo`001_ooool;ogl000Ko07l0
1OmoOgl6o`00O`_oOgmo1OoUiND6oooo00GoO`001_oo0005o`1o00Koo`3o1Ooo0006oooo00Go0?oo
1_mo0005o`00O`KoOgmo1Ol007l6ool0o`Goo`001OmoOgl6o`1o00Goool01_mo0005oooo00Koo`00
2ol0o`05o`3oo`KoO`001Ooo0?l6oooo00Go0?l01_moOgl5oooo00Koo`001OmoOgl;ogl000Koo`3o
1Omo0006o`1o00Goo`3o1_ooo`05ool000KoOgmo1Omo0006onGUi@Oooooo0006ooooo`_oO`001_l0
O`05ogmoO`Ko001o2omoOgl5onGUi@Koool01Omo0006ool000Go07l01_oo0?l5ool000Koool01Ol0
ool6ogl000Go001o1_moOgl5o`00O`Koo`3o1Ooo0005ogmoO`Ko07l01Oooo`06ogl000Goool01_oo
000;o`3o00Go0?oo1_mo0005ool0o`Koool01Ol0o`06ogmoO`Goool01_oo0005ogmoO`_oO`001_oo
0?l5ogl000Ko07l01Ooo0?l6oooo00Goo`001_moOgl5ogl000KoiNGU1oooool000Kooooo2omo0006
o`1o00GoOgmo1_l007l;ogmoO`GoiNGU1_ooo`05ogl000Koo`001Ol0O`06ool0o`Goo`001_ooo`05
o`3oo`KoO`001Ol007l6ogmoO`Go001o1_oo0?l5ool000GoOgmo1_l0O`05oooo00KoO`001Oooo`06
ool000_o0?l01Ol0ool6ogl000Goo`3o1_ooo`05o`3o00KoOgmo1Oooo`06ool000GoOgmo2omo0006
ool0o`GoO`001_l0O`05ool0o`Koool01Ooo0006ogmoO`GoO`001_oUiND7ooooo`001_ooool;ogl0
00Ko07l01OmoOgl6o`00O`_oOgmo1OoUiND6oooo00GoO`001_oo0005o`1o00Koo`3o1Ooo0006oooo
00Go0?oo1_mo0005o`00O`KoOgmo1Ol007l6ool0o`Goo`001OmoOgl6o`1o00Goool01_mo0005oooo
00Koo`002ol0o`05o`3oo`KoO`001Ooo0?l6oooo00Go0?l01_moOgl5oooo00Koo`001OmoOgl;ogl0
00Koo`3o1Omo0006o`1o00Goo`3o1_ooo`05ool000KoOgmo1Omo0006onGUi@Oooooo0006ooooo`_o
O`001_l0O`05ogmoO`Ko001o2omoOgl5onGUi@Koool01Omo0006ool000Go07l01_oo0?l5ool000Ko
ool01Ol0ool6ogl000Go001o1_moOgl5o`00O`Koo`3o1Ooo0005ogmoO`Ko07l01Oooo`06ogl000Go
ool01_oo000;o`3o00Go0?oo1_mo0005ool0o`Koool01Ol0o`06ogmoO`Goool01_oo0005ogmoO`_o
O`001_oo0?l5ogl000Ko07l01Ooo0?l6oooo00Goo`001_moOgl5ogl000KoiNGU1oooool000Kooooo
1_moOgl5ool0o`KoiNGU1Oooo`06o`1o00Goool01_mo0005ool000KoOgmo1Ooo0006oooo00Goo`3o
1_oUiND5o`1o00KoiNGU1Ol0o`06onGUi@Goo`3o1_moOgl5oooo00KoOgmo2_l0oolAogl000GoOgmo
1_l0ool5ogl000Koool01Ol0o`06onGUi@Go07l01_oUiND;ogmoO`Goo`001_mo0005ool0o`Ko07l0
1Oooo`06onGUi@Go0?l01_mo0005o`3oo`Ko07l01Ooo0006ogl000Go0?l01_moOgl7ooooo`001_oo
ool6ogmoO`Goo`3o1_oUiND5oooo00Ko07l01Oooo`06ogl000Goo`001_moOgl5ool000Koool01Ooo
0?l6onGUi@Go07l01_oUiND5o`3o00KoiNGU1Ooo0?l6ogmoO`Goool01_moOgl:o`3ooa7oO`001Omo
Ogl6o`3oo`GoO`001_ooo`05o`3o00KoiNGU1Ol0O`06onGUi@_oOgmo1Ooo0006ogl000Goo`3o1_l0
O`05oooo00KoiNGU1Ol0o`06ogl000Go0?oo1_l0O`05ool000KoO`001Ol0o`06ogmoO`Oooooo0006
ooooo`KoOgmo1Ooo0?l6onGUi@Goool01_l0O`05oooo00KoO`001Ooo0006ogmoO`Goo`001_ooo`05
ool0o`KoiNGU1Ol0O`06onGUi@Go0?l01_oUiND5ool0o`KoOgmo1Oooo`06ogmoO`[o0?oo4Omo0005
ogmoO`Ko0?oo1Omo0006oooo00Go0?l01_oUiND5o`1o00KoiNGU2omoOgl5ool000KoO`001Ooo0?l6
o`1o00Goool01_oUiND5o`3o00KoO`001Ol0ool6o`1o00Goo`001_mo0005o`3o00KoOgmo1oooool0
00Kooooo1_moOgl5ool0o`KoiNGU1Oooo`06o`1o00Goool01_mo0005ool000KoOgmo1Ooo0006oooo
00Goo`3o1_oUiND5o`1o00KoiNGU1Ol0o`06onGUi@Goo`3o1_moOgl5oooo00KoOgmo2_l0oolAogl0
00GoOgmo1_l0ool5ogl000Koool01Ol0o`06onGUi@Go07l01_oUiND;ogmoO`Goo`001_mo0005ool0
o`Ko07l01Oooo`06onGUi@Go0?l01_mo0005o`3oo`Ko07l01Ooo0006ogl000Go0?l01_moOgl7oooo
o`001_ooool6ogmoO`Goo`3o1_oUiND5oooo00Ko07l01Oooo`06ogl000Goo`001_moOgl5ool000Ko
ool01Ooo0?l6onGUi@Go07l01_oUiND5o`3o00KoiNGU1Ooo0?l6ogmoO`Goool01_moOgl:o`3ooa7o
O`001OmoOgl6o`3oo`GoO`001_ooo`05o`3o00KoiNGU1Ol0O`06onGUi@_oOgmo1Ooo0006ogl000Go
o`3o1_l0O`05oooo00KoiNGU1Ol0o`06ogl000Go0?oo1_l0O`05ool000KoO`001Ol0o`06ogmoO`Oo
oooo0006ooooo`KoOgmo1Ooo0?l6onGUi@Goool01_l0O`05oooo00KoO`001Ooo0006ogmoO`Goo`00
1_ooo`05ool0o`KoiNGU1Ol0O`06onGUi@Go0?l01_oUiND5ool0o`KoOgmo1Oooo`06ogmoO`[o0?oo
4Omo0005ogmoO`Ko0?oo1Omo0006oooo00Go0?l01_oUiND5o`1o00KoiNGU2omoOgl5ool000KoO`00
1Ooo0?l6o`1o00Goool01_oUiND5o`3o00KoO`001Ol0ool6o`1o00Goo`001_mo0005o`3o00KoOgmo
1oooool000Kooooo1_l0ool5o`00O`Koool02ooo0005o`3o00Koo`001Oooo`06o`3oo`GoiNGU1_l0
ool@ool000Koo`3o1Ol007l6o`3o00Goo`001_oUiND5ool0o`_oOgmo1Ol007l6onGUi@GoOgmo1_l0
07l5onGUi@Koool01Ol007l;oooo00KoOgmo1Oooo`06o`3o00GoiNGU1_ooo`05o`3oo`Ko001o1Ooo
0?l6o`00O`Goool01_oUiND5o`3o00Koool01Ol0o`0;ool0o`KoOgmo1Ol0O`06ool0o`Oooooo0006
ooooo`Ko0?oo1Ol007l6oooo00_oo`001Ol0o`06ool000Goool01_l0ool5onGUi@Ko0?oo4?oo0006
ool0o`Go001o1_l0o`05ool000KoiNGU1Ooo0?l;ogmoO`Go001o1_oUiND5ogmoO`Ko001o1OoUiND6
oooo00Go001o2oooo`06ogmoO`Goool01_l0o`05onGUi@Koool01Ol0ool6o`00O`Goo`3o1_l007l5
oooo00KoiNGU1Ol0o`06oooo00Go0?l02ooo0?l6ogmoO`Go07l01_oo0?l7ooooo`001_ooool6o`3o
o`Go001o1_ooo`0;ool000Go0?l01_oo0005oooo00Ko0?oo1OoUiND6o`3ooa3oo`001_oo0?l5o`00
O`Ko0?l01Ooo0006onGUi@Goo`3o2omoOgl5o`00O`KoiNGU1OmoOgl6o`00O`GoiNGU1_ooo`05o`00
O`_oool01_moOgl5oooo00Ko0?l01OoUiND6oooo00Go0?oo1_l007l5ool0o`Ko001o1Oooo`06onGU
i@Go0?l01_ooo`05o`3o00_oo`3o1_moOgl5o`1o00Koo`3o1oooool000Kooooo1_l0ool5o`00O`Ko
ool02ooo0005o`3o00Koo`001Oooo`06o`3oo`GoiNGU1_l0ool@ool000Koo`3o1Ol007l6o`3o00Go
o`001_oUiND5ool0o`_oOgmo1Ol007l6onGUi@GoOgmo1_l007l5onGUi@Koool01Ol007l;oooo00Ko
Ogmo1Oooo`06o`3o00GoiNGU1_ooo`05o`3oo`Ko001o1Ooo0?l6o`00O`Goool01_oUiND5o`3o00Ko
ool01Ol0o`0;ool0o`KoOgmo1Ol0O`06ool0o`Oooooo0006ooooo`Ko0?oo1Ol007l6oooo00_oo`00
1Ol0o`06ool000Goool01_l0ool5onGUi@Ko0?oo4?oo0006ool0o`Go001o1_l0o`05ool000KoiNGU
1Ooo0?l;ogmoO`Go001o1_oUiND5ogmoO`Ko001o1OoUiND6oooo00Go001o2oooo`06ogmoO`Goool0
1_l0o`05onGUi@Koool01Ol0ool6o`00O`Goo`3o1_l007l5oooo00KoiNGU1Ol0o`06oooo00Go0?l0
2ooo0?l6ogmoO`Go07l01_oo0?l7ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oU
iND5ogmoO`_o0?oo1_ooo`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo
0006o`00O`GoiNGU1Ooo0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0
O`05o`00O`_oool01_moOgl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0
ool6o`3o00GoOgmo1_l0O`07ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oUiND5
ogmoO`_o0?oo1_ooo`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo0006
o`00O`GoiNGU1Ooo0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0O`05
o`00O`_oool01_moOgl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0ool6
o`3o00GoOgmo1_l0O`07ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oUiND5ogmo
O`_o0?oo1_ooo`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo0006o`00
O`GoiNGU1Ooo0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0O`05o`00
O`_oool01_moOgl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0ool6o`3o
00GoOgmo1_l0O`07ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oUiND5ogmoO`_o
0?oo1_ooo`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo0006o`00O`Go
iNGU1Ooo0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0O`05o`00O`_o
ool01_moOgl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0ool6o`3o00Go
Ogmo1_l0O`07ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oUiND5ogmoO`_o0?oo
1_ooo`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo0006o`00O`GoiNGU
1Ooo0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0O`05o`00O`_oool0
1_moOgl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0ool6o`3o00GoOgmo
1_l0O`07ooooo`001_ooool6o`3o00GoOgmo2ol0ool6o`1o00Goo`001_oUiND5ogmoO`_o0?oo1_oo
o`05ogl000Koool01Omo0006o`3oo`Go0?l01_moOgl5o`00O`Ko0?l01Omo0006o`00O`GoiNGU1Ooo
0?l6ogmoO`Goo`001_l007l5ool000Koool01OmoOgl6onGUi@Go0?l01_l0O`05o`00O`_oool01_mo
Ogl5onGUi@Koool01Ooo0?l6o`1o00Go0?oo1_l0O`05ool000Ko0?l04?l0ool6o`3o00GoOgmo1_l0
O`07ooooo`001_ooool6o`00O`Goool01_oUiND5ogl000Ko0?l01Ooo0?l6o`00O`GoOgmo2ooo0?l6
o`00O`Go0?oo1_ooo`05ogl000Koool01Omo0006ool0o`GoO`001_oUiND5o`3oo`KoOgmo1Ooo0005
o`3oo`Ko0?l01Ooo0?l6oooo00GoiNGU1_l0O`05o`00O`Ko07l01Ol007l6oooo00Go0?oo1_l007l5
onGUi@Ko07l01Ol007l6oooo00Goo`001_moOgl5oooo00Koo`3o1Oooo`06ogl000GoOgmo1_oo000;
o`1o00GoO`001_moOgl7ooooo`001_ooool6o`00O`Goool01_oUiND5ogl000Ko0?l01Ooo0?l6o`00
O`GoOgmo2ooo0?l6o`00O`Go0?oo1_ooo`05ogl000Koool01Omo0006ool0o`GoO`001_oUiND5o`3o
o`KoOgmo1Ooo0005o`3oo`Ko0?l01Ooo0?l6oooo00GoiNGU1_l0O`05o`00O`Ko07l01Ol007l6oooo
00Go0?oo1_l007l5onGUi@Ko07l01Ol007l6oooo00Goo`001_moOgl5oooo00Koo`3o1Oooo`06ogl0
00GoOgmo1_oo000;o`1o00GoO`001_moOgl7ooooo`001_ooool6o`00O`Goool01_oUiND5ogl000Ko
0?l01Ooo0?l6o`00O`GoOgmo2ooo0?l6o`00O`Go0?oo1_ooo`05ogl000Koool01Omo0006ool0o`Go
O`001_oUiND5o`3oo`KoOgmo1Ooo0005o`3oo`Ko0?l01Ooo0?l6oooo00GoiNGU1_l0O`05o`00O`Ko
07l01Ol007l6oooo00Go0?oo1_l007l5onGUi@Ko07l01Ol007l6oooo00Goo`001_moOgl5oooo00Ko
o`3o1Oooo`06ogl000GoOgmo1_oo000;o`1o00GoO`001_moOgl7ooooo`001_ooool6o`00O`Goool0
1_oUiND5ogl000Ko0?l01Ooo0?l6o`00O`GoOgmo2ooo0?l6o`00O`Go0?oo1_ooo`05ogl000Koool0
1Omo0006ool0o`GoO`001_oUiND5o`3oo`KoOgmo1Ooo0005o`3oo`Ko0?l01Ooo0?l6oooo00GoiNGU
1_l0O`05o`00O`Ko07l01Ol007l6oooo00Go0?oo1_l007l5onGUi@Ko07l01Ol007l6oooo00Goo`00
1_moOgl5oooo00Koo`3o1Oooo`06ogl000GoOgmo1_oo000;o`1o00GoO`001_moOgl7ooooo`001_oo
ool6o`00O`Goool01_oUiND5ogl000Ko0?l01Ooo0?l6o`00O`GoOgmo2ooo0?l6o`00O`Go0?oo1_oo
o`05ogl000Koool01Omo0006ool0o`GoO`001_oUiND5o`3oo`KoOgmo1Ooo0005o`3oo`Ko0?l01Ooo
0?l6oooo00GoiNGU1_l0O`05o`00O`Ko07l01Ol007l6oooo00Go0?oo1_l007l5onGUi@Ko07l01Ol0
07l6oooo00Goo`001_moOgl5oooo00Koo`3o1Oooo`06ogl000GoOgmo1_oo000;o`1o00GoO`001_mo
Ogl7ooooo`001_ooool6o`1o00_oOgmo1Ol007l6ogmoO`GoO`001_ooo`0;o`3oo`Goo`3o1_l0O`05
o`00O`Koo`3o1OmoOgl;o`1o00Koool01Ooo0?l6o`00O`GoOgmo1_oo0005onGUi@_oo`3o1Ol007l6
oooo00Go07l01_oUiND5ogl000Koool01Ooo0006ool0o`Go07l01_l007l5ool0o`_o0?oo1_l0O`0;
ogl000Goool01_l007l5o`1o00Koool01Ol007l6o`3oo`GoiNGU1_oo0?l5onGUi@Ko0?oo1oooool0
00Kooooo1_l0O`0;ogmoO`Go001o1_moOgl5ogl000Koool02ol0ool5ool0o`Ko07l01Ol007l6ool0
o`GoOgmo2ol0O`06oooo00Goo`3o1_l007l5ogmoO`Koo`001OoUiND;ool0o`Go001o1_ooo`05o`1o
00KoiNGU1Omo0006oooo00Goo`001_oo0?l5o`1o00Ko001o1Ooo0?l;o`3oo`Ko07l02omo0005oooo
00Ko001o1Ol0O`06oooo00Go001o1_l0ool5onGUi@Koo`3o1OoUiND6o`3oo`Oooooo0006ooooo`Ko
07l02omoOgl5o`00O`KoOgmo1Omo0006oooo00_o0?oo1Ooo0?l6o`1o00Go001o1_oo0?l5ogmoO`_o
07l01_ooo`05ool0o`Ko001o1OmoOgl6ool000GoiNGU2ooo0?l5o`00O`Koool01Ol0O`06onGUi@Go
O`001_ooo`05ool000Koo`3o1Ol0O`06o`00O`Goo`3o2ol0ool6o`1o00_oO`001Oooo`06o`00O`Go
07l01_ooo`05o`00O`Ko0?oo1OoUiND6ool0o`GoiNGU1_l0ool7ooooo`001_ooool6o`1o00_oOgmo
1Ol007l6ogmoO`GoO`001_ooo`0;o`3oo`Goo`3o1_l0O`05o`00O`Koo`3o1OmoOgl;o`1o00Koool0
1Ooo0?l6o`00O`GoOgmo1_oo0005onGUi@_oo`3o1Ol007l6oooo00Go07l01_oUiND5ogl000Koool0
1Ooo0006ool0o`Go07l01_l007l5ool0o`_o0?oo1_l0O`0;ogl000Goool01_l007l5o`1o00Koool0
1Ol007l6o`3oo`GoiNGU1_oo0?l5onGUi@Ko0?oo1oooool000Kooooo1_l0O`0;ogmoO`Go001o1_mo
Ogl5ogl000Koool02ol0ool5ool0o`Ko07l01Ol007l6ool0o`GoOgmo2ol0O`06oooo00Goo`3o1_l0
07l5ogmoO`Koo`001OoUiND;ool0o`Go001o1_ooo`05o`1o00KoiNGU1Omo0006oooo00Goo`001_oo
0?l5o`1o00Ko001o1Ooo0?l;o`3oo`Ko07l02omo0005oooo00Ko001o1Ol0O`06oooo00Go001o1_l0
ool5onGUi@Koo`3o1OoUiND6o`3oo`Oooooo0006ooooo`Ko07l02omoOgl5o`00O`KoOgmo1Omo0006
oooo00_o0?oo1Ooo0?l6o`1o00Go001o1_oo0?l5ogmoO`_o07l01_ooo`05ool0o`Ko001o1OmoOgl6
ool000GoiNGU2ooo0?l5o`00O`Koool01Ol0O`06onGUi@GoO`001_ooo`05ool000Koo`3o1Ol0O`06
o`00O`Goo`3o2ol0ool6o`1o00_oO`001Oooo`06o`00O`Go07l01_ooo`05o`00O`Ko0?oo1OoUiND6
ool0o`GoiNGU1_l0ool7ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-1.25026, -1.25026, 0.182929, \
0.182929}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[
          l, \[IndentingNewLine]RasterLatticeRepresentation[moore8DTD, 50, 
            AspectRatio \[Rule] 
              Automatic]\[IndentingNewLine]]; \[IndentingNewLine]Evolve[
          moore8DTD], {16}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[10]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]GraphicsArray[\[IndentingNewLine]Partition[l, 
            4]\[IndentingNewLine]]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.0238095 0.238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.245293 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.0238095 0.488926 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.0238095 0.732558 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.0238095 0.97619 0.245293 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.267442 0.245293 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.267442 0.488926 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.267442 0.732558 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.267442 0.97619 0.488926 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.511074 0.245293 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F00007F00007F00007F0000FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFFFF00FFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFFFF00FFFF00FFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF0000FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF0000FFFF
00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF
7F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF
FFFF007F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFFFF00FFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.511074 0.488926 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFFFF00FFFF00FFFF00
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.511074 0.732558 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
FF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.511074 0.97619 0.732558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.754707 0.245293 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FFFFFF0000E5E5E500FFFF7F0000E5E5E500FFFF00FF00FF00FF00FF00FF0000FFFF00
00FF00FF00FF7F7F7F007F007F7F7FFF00FFFF00007F00007F7F7F00FFFF00FF007F7F7F
FF000000FFFF7F7F7FFFFF00FF00FF7F7F7F00FF0000FF00007F00FF000000007F007F00
E5E5E5E5E5E500FF00FF00FFFF00FF007F00FF00FFE5E5E5FF00FFFF000000FF00E5E5E5
00FF00FF0000
FFFF00E5E5E5FFFF007F7F7F00FFFF7F0000FF0000E5E5E57F0000E5E5E57F7F7FFF00FF
00FF00E5E5E500007F00007F7F000000007FE5E5E5007F00FF0000FF000000FF0000FF00
00007FE5E5E5FF0000E5E5E57F0000FF00FF00FF00FFFF0000FF007F0000007F00FFFF00
FF00FF00FF007F7F7FFF0000FF0000E5E5E5E5E5E500007F7F000000FF00FFFF0000FF00
7F0000007F00
FFFF00FF00FF7F0000007F007F7F7FE5E5E5FF000000FF00E5E5E5FF00FF7F7F7FFF00FF
FFFF00FF00FF00FFFF00FF00FFFF00FF00FF00007F7F000000FF00FFFF00FF0000FF00FF
FF00007F000000FFFFFF0000007F00FFFF00E5E5E500007FFFFF0000FFFFFF0000FFFF00
00007F007F0000007F007F007F7F7FE5E5E5E5E5E500FFFF00FF007F0000FFFF00FF00FF
FF00FF00FF00
00FF00FF00FFFFFF00E5E5E5FF00FFFF00FF7F7F7FFF000000007F00007FFF00FF7F7F7F
E5E5E57F7F7F007F007F7F7F00FF007F0000007F00E5E5E57F0000FF00FF00FF007F0000
7F000000FF00007F0000FFFF007F007F0000E5E5E500FF007F000000007FFF0000E5E5E5
00007F7F000000007F7F000000FFFF00007FE5E5E500FF007F000000FFFFFF0000007F00
FFFF00FF00FF
00FFFFFF00FF00FFFF00FF00FFFF00007F00E5E5E5E5E5E5FFFF00FF00007F7F7F00FFFF
FF000000007F00007FFF00FF00FFFFFF0000FFFF00E5E5E5FFFF00FFFF0000FF00007F00
FF00FF007F007F000000FFFF00FFFFE5E5E57F000000007F7F00007F000000007F7F0000
FF000000007F7F7F7FFF00FF7F0000007F00007F00FF000000007F7F7F7FFF00FFFF0000
FFFF00FFFF00
FF0000FFFF007F00007F7F7F7F0000E5E5E5FF00FF7F7F7FFF0000E5E5E500007F7F0000
7F7F7FFF000000FFFF00FF0000FFFF00FF0000FF00007F00007F00007F0000FF00E5E5E5
007F00FFFF00FF00FF00007FFF00007F7F7FE5E5E5FF00FFFF00FF00007FFF00007F7F7F
00FF00FF000000FFFF007F00007F00FF00007F0000FF00007F0000FF00007F000000007F
FFFF00FFFF00
007F00FF00FF007F00FF00FFFF00FFE5E5E57F7F7FE5E5E57F0000FF00FF7F7F7FFFFF00
FF00FF00FFFF00FF0000FFFFE5E5E5007F0000007F00FF007F0000FFFF0000FF007F7F7F
E5E5E500FF00007F00FF0000FF0000E5E5E500FFFF00FF0000FFFFFF00FF7F0000FF00FF
7F7F7FE5E5E500FFFF00FF00007F0000007F007F00007F00007F0000FFFF7F0000FFFF00
00FF007F7F7F
FF00FF7F7F7FFF0000E5E5E5FF00FF007F00FF0000FF00FFE5E5E57F000000FF0000FFFF
007F00FF0000FF00FF00FF0000007FFF000000FF00FFFF007F0000FF00FFE5E5E5E5E5E5
00FFFFE5E5E5E5E5E5FF00FFE5E5E500FFFF7F7F7F007F00E5E5E500FF0000FFFF00FF00
7F7F7F00FFFF00FF007F7F7F00FFFFFF00FFFF00FF7F7F7FFF000000FF00007F00E5E5E5
FF000000FF00
FF0000FF00FF00FF00FF000000FFFF7F000000FF00FF00FF7F7F7FE5E5E57F0000007F00
7F7F7F7F7F7F7F7F7F7F0000FF00FF7F0000FF0000FF0000FF00FF00007FFF000000FF00
7F7F7F00FFFF007F0000FF0000007FE5E5E500FFFF00FF0000FFFF00007F00007F00FFFF
00007FFF00007F000000007F00FF007F7F7F00FF007F7F7F00FF0000007FFF0000FF0000
7F000000007F
00FF00FF000000007FFFFF007F7F7FFF00FFFFFF007F000000007F00007F00007F00FFFF
00FF0000FFFF7F7F7F00FF00FF00FF00FFFFFF00FFFFFF00E5E5E5FF00FF00FFFF007F00
FFFF00FF0000FF000000FF00E5E5E5FF00FF7F7F7F007F00FF00FFFF00FF7F0000E5E5E5
00FFFF007F00FF00FFFF0000007F007F00007F000000007FFF00FFFF0000E5E5E5E5E5E5
FF00FF00FF00
00FF00FF0000E5E5E57F7F7F007F00FF0000FF00FFFF00007F7F7FE5E5E5E5E5E5FF00FF
00007F00007F00FFFF00FFFF7F00007F7F7F00007FFF000000FFFFFF00FF00FF00FF0000
FF0000FF00FF00FF007F0000FF00007F00007F7F7F7F0000E5E5E5FF00FF00FF0000007F
E5E5E5FF0000FF0000007F00E5E5E57F0000E5E5E5FFFF00FF00FFFFFF0000007F00FF00
E5E5E5FF00FF
00FFFFFF0000FFFF007F00007F7F7FFF00FFFFFF007F000000FFFFE5E5E5FF00FF007F00
00FFFFFF00FF007F00007F00FF00FF00007F007F0000FF00FF00FF007F00E5E5E5FF00FF
FF00FF00007F7F7F7F00007F7F7F7F007F00FF0000E5E5E5FFFF00E5E5E500FFFF7F7F7F
FF0000FF000000007F7F0000007F0000FF007F0000E5E5E5007F00FF00FFE5E5E57F7F7F
E5E5E500FF00
00007F007F00007F00FF00FF00007FFFFF007F00007F7F7F00007F7F7F7F007F007F7F7F
FF00FF7F000000FF007F7F7F7F0000FF00FF7F7F7F00FFFF7F7F7F7F7F7F00FF007F7F7F
FFFF0000FF00FF00FF007F007F7F7F00007FFF000000FF007F00007F7F7FFF000000FF00
FFFF0000FF0000FF0000007F00FF00FF00FF00FF00FF00FF00007F7F7F7FFF0000E5E5E5
E5E5E500007F
00FF00FF000000FF0000FF00FF000000FFFFFF0000FFFF007F0000FFFF00FFFF00E5E5E5
7F7F7FFFFF007F00007F0000FFFF00FF0000007F00FF00FF7F7F7F7F00007F0000007F00
00FFFF7F7F7FFF00FFFF00FFFFFF0000FF00FF00FFFF0000E5E5E500007F007F007F7F7F
007F0000007F7F7F7FFF00FF00FFFFFF00FF007F00FF00FF00FFFFE5E5E57F0000FF00FF
00007FFFFF00
E5E5E500007FFF00FFFFFF0000FFFF00FFFF7F7F7F00FFFF7F0000FFFF00FFFF00007F00
E5E5E5007F0000007F7F00007F7F7F7F00007F0000E5E5E57F7F7F7F7F7F007F00FFFF00
00FFFF00FFFFE5E5E57F7F7FFF00007F000000FFFF00007F007F007F7F7FFF000000007F
00007FFF00FF007F00FF0000FF00007F7F7F7F0000E5E5E5FF000000FFFF00007F00FFFF
7F0000FF0000
FFFF007F7F7FE5E5E5FF00FF007F00007F0000007F00007F00FF007F7F7F7F7F7FFF0000
00FF00FFFF0000007F007F00FF00007F0000FFFF0000FF0000007F00007FFFFF00FF00FF
FF00FF00FF00E5E5E57F00007F00007F0000FFFF0000FFFF00FFFF007F00FFFF0000FF00
00FF00FF0000FF00007F00007F0000FF0000FF000000FF007F0000FF0000FF0000FF0000
FF000000FFFF
7F000000FF00007F0000FF00FF00FFFFFF00007F007F0000007F00FFFF0000FF00E5E5E5
00007FFF000000FF00007F007F0000FF00FF7F00007F0000FF00007F000000007FFF00FF
FFFF00FF000000FFFF7F7F7F00FFFF00007F00007F007F0000FFFF7F000000FF00007F00
00007F00007F00007F007F0000007F7F000000FFFFE5E5E5FF0000FF00FFFF0000E5E5E5
00007F7F0000
007F0000007F7F7F7F00007F00FF00FFFF007F000000FFFFFFFF00FF00FF7F7F7FFF0000
7F0000FF00FFE5E5E500FFFFFFFF00FF000000FF00FF00FFFF000000007F00007F7F7F7F
FFFF007F7F7FFF0000FF000000FFFF00FF007F7F7F00FF00FFFF00007F00FF000000FFFF
007F00FF0000FF000000FF00FFFF007F00007F000000FFFFFF0000FFFF0000FFFF7F0000
00FFFF007F00
00FFFF7F7F7F00007F00FF00FF000000007F007F00FFFF0000FF00E5E5E500007FFF0000
FFFF007F0000FFFF00E5E5E5FF0000FF00FFFFFF0000FFFF007F0000007F00FF00FFFF00
00FFFF00FF00FF00FFFF0000007F007F7F7F00FF00FF00007F7F7F7F00007F0000FFFF00
FF000000FF00E5E5E500FFFF00FF00FF00007F000000FFFF00FFFF00FF00007F0000FFFF
00FF0000007F
FF00007F00007F7F7F007F007F000000FF00FFFF007F7F7F7F7F7F007F0000FFFF7F7F7F
00FF00FF0000FFFF00E5E5E57F00007F0000FFFF0000FFFF00FFFFE5E5E57F000000007F
00FFFFFF0000FF000000007FFF00007F000000007F7F7F7F00FFFF00007F00007F007F00
FF0000007F00FF00FF00007F7F7F7F00FFFFFF00FF007F007F7F7F00FFFFFF0000FFFF00
E5E5E57F0000
7F00007F7F7FFF0000FFFF0000007F7F7F7F007F007F7F7F7F7F7FFF000000FFFF7F0000
FF00FFE5E5E5FF000000FFFF00FFFF00FF0000FF0000007F00007F007F0000007F00FFFF
00FF0000FFFF00007FE5E5E5E5E5E500007F7F000000FFFF007F007F0000FFFF00007F00
7F7F7FFFFF00FFFF0000007F00FFFFFF00007F000000FF0000FFFF00FFFFE5E5E5FFFF00
007F00FFFF00
FFFF0000FF00007F0000FF007F0000FF00FF00FF0000FF007F7F7F00FF0000FF007F7F7F
E5E5E5FF00007F000000FF0000FF0000FF00FF00007F7F7F00007FFF0000FF00007F0000
FF000000FF00FF0000FFFF00FF00FFFF0000FFFF00FFFF00E5E5E500FF00FF00FFFF0000
FF00FFFFFF00FFFF0000FFFF00007F007F00FFFF007F7F7F7F7F7F00FFFF007F0000FFFF
00FF007F0000
00007F00FF00FF000000007FE5E5E57F0000FF00FF00FF0000FF0000007FFF0000FF0000
00007FFF000000FF00FFFF007F0000FFFF00FF00FF7F7F7FFF0000FFFF00FFFF007F7F7F
00007FFFFF0000FF00FF00FF7F7F7F7F7F7F00007F00007FFFFF0000007F7F7F7F00FFFF
FF000000FF007F00007F7F7FFF0000E5E5E500007FE5E5E5FFFF007F7F7F00FFFF00007F
00007F7F0000
00007F7F0000FFFF00007F00007F007F7F7FFF000000FF00FF00007F7F7F7F0000FFFF00
FFFF00007F00007F00E5E5E5007F00FF000000007F7F7F7F7F00007F7F7F00007F00007F
FFFF00FF00FF007F00FFFF007F7F7F7F0000FF000000FFFFFF00FFE5E5E57F0000E5E5E5
FFFF0000FF00FF00FF007F007F00007F0000FF00FF00FF007F0000FFFF0000FFFFE5E5E5
FFFF00FFFF00
FF0000FFFF00FF000000FF00E5E5E5E5E5E57F000000FF007F7F7FFFFF00E5E5E57F0000
FF000000FFFF00FF0000FF007F00007F7F7FFF00FF00007FE5E5E57F0000FF00FFFFFF00
00FF00007F0000007F7F7F7F7F000000007FFFFF00007F00E5E5E5FF00FF00007FE5E5E5
00007FFFFF0000FFFF7F0000FF00FF00007FFFFF0000FFFFFF00FF007F00FF00FFFF0000
7F7F7F00FF00
FF00FF007F0000FFFF00FF00FF00FFE5E5E5FFFF00FF0000E5E5E5FF0000007F00FF0000
7F7F7F00FF00E5E5E5FFFF00FF00FF00007FFFFF00FF00FFE5E5E5FF00FF007F007F0000
00007F00FF0000007F00FF0000FFFFFF000000007F7F7F7F00FFFFE5E5E5FF00007F7F7F
E5E5E500FF0000007F00FF00E5E5E5E5E5E5E5E5E5E5E5E57F7F7F00007F7F7F7FFF0000
FF00FF7F7F7F
00007F007F00FF000000FFFFFFFF00FF00FF00FFFFFFFF0000FF0000007F00FFFF7F0000
7F0000007F00007F0000007FFF0000E5E5E5E5E5E5FF00FF00007FFFFF0000FFFF7F0000
00FF007F000000007F007F00007F0000007FFF00FFFFFF00FF00FFFF00FF00007FFF00FF
007F0000FF00FF00007F0000FF00FF7F0000007F00FFFF007F00007F7F7FFF000000FFFF
00FFFFE5E5E5
00FFFF00FFFF00007FFFFF00E5E5E5E5E5E57F00007F000000FF00FF00FFFF00FF00FFFF
7F00007F000000FF007F7F7F00007F00FFFF7F000000FFFF00FF00FFFF0000007F007F00
E5E5E5FF000000FF0000FF0000007FFFFF00007F00007F0000FF0000FF0000007FE5E5E5
00FFFFFF000000FFFF7F000000007F007F00FF00FF00007FFF00FFFF0000FF00FF7F0000
FF00FFE5E5E5
E5E5E5E5E5E5007F007F7F7FFFFF0000FF007F7F7FFF0000E5E5E5FF000000FF0000FF00
FF00007F00007F000000FFFF7F0000FF00FFFFFF00FFFF00E5E5E5FF0000E5E5E500FFFF
E5E5E500FFFFE5E5E57F0000007F00FF00FF00FF00007F007F000000007FFF00FF00FF00
FFFF0000FFFFFFFF00007F00007F00FF000000FF00FF00FFFFFF007F7F7FFF0000007F00
007F00FF0000
00007FFFFF007F0000FF0000FF0000E5E5E500007FFF00FF007F00E5E5E500007F00FF00
00007F00FF00E5E5E500FF00E5E5E500FF00007F00007F00FF000000007FFFFF007F7F7F
FF000000FFFFE5E5E57F7F7F7F7F7F00007FFF00FF00007F00FFFF00FF00FF0000E5E5E5
FFFF00FF00007F0000007F00FF0000007F0000007F00007F007F00FFFF00FF00FFFF00FF
00007F7F7F7F
00FF00FFFF0000007FE5E5E57F000000007F007F0000FFFFE5E5E500FFFFFFFF0000FF00
7F7F7F7F0000E5E5E5FF00FF00FFFFFFFF007F0000FF00FFFF00FF007F00FF00FFFFFF00
FFFF007F7F7FFF00FF7F7F7F00007F00FF00E5E5E5E5E5E500FF0000FF0000007F00FFFF
00FF0000FFFF00FF00FF00FFFFFF00E5E5E57F7F7F00FF00FFFF00FF00007F7F7F00FFFF
7F7F7FFFFF00
00007FFF00FF7F7F7F00007FE5E5E5FFFF0000FF007F7F7FFFFF00007F00FF00FFE5E5E5
00007FE5E5E5FF00007F7F7F00FFFFFF00FFE5E5E5007F00E5E5E5007F0000007F7F0000
FFFF007F7F7FFF0000FF000000FF0000FFFFFF0000FF00FF00FFFF00FF00FF000000007F
7F7F7FFF000000007F007F00007F00FFFF00E5E5E500FF00007F00007F00E5E5E500FF00
00FF00FF0000
00FFFFFF00007F00007F0000FFFF0000FF00007F00FF00FFFF00FF00FFFFFF000000FF00
FF00FFE5E5E5E5E5E57F0000E5E5E5007F00FF00FF007F0000FF00007F007F0000FF00FF
00007F00007F00FF0000007F7F0000E5E5E5E5E5E500FF0000FF00FF00FFFF0000E5E5E5
FF0000FF00FF007F007F7F7F007F00FF000000FFFF7F0000FFFF00FF00FFFF0000FF00FF
FF0000FF00FF
E5E5E500FF007F0000FF0000FF00FFFF0000FF0000007F0000FFFF00FFFF00007FE5E5E5
E5E5E57F7F7F00007F00FFFFFF00FF007F007F7F7F00FFFFFF00FF00FFFFFF00FF00FF00
00FF00007F0000FFFFFF00FF00007F7F7F7F7F7F7FFFFF00007F007F0000FFFF00FF0000
007F00FF0000FF00FFE5E5E5FFFF0000007FFFFF00007F0000FFFF007F00FFFF00E5E5E5
FF0000FF0000
E5E5E5FF00FFFF0000FF00007F7F7FE5E5E57F00007F0000FFFF007F000000FFFF007F00
00007FFFFF0000FF0000FF0000FFFF00007FFF00FFE5E5E5FF00FFE5E5E5E5E5E5007F00
FFFF0000007F007F007F0000FF00FFFF00007F7F7F007F00FF00FFE5E5E5007F00FFFF00
00FFFF7F0000FFFF0000007F00007F007F00E5E5E57F7F7F00007F00007F007F007F7F7F
007F00FF00FF
007F007F7F7F7F7F7FE5E5E500FFFF00FF00FFFF0000FFFF7F00007F0000E5E5E5007F00
7F7F7F7F00007F7F7F7F7F7FFF00FFFFFF007F7F7F00FFFF7F000000007FE5E5E57F0000
00007F00FFFF007F00FF0000FF000000007F7F0000FFFF00FFFF0000FF0000FF0000007F
007F00FF00FFFFFF0000FFFFFF00FF007F00FF00FF007F007F000000FF00FF0000FF00FF
7F7F7FFF0000
007F00007F007F7F7FFF000000FF00FF00FF7F0000FF00FF00FF00FF000000007F00007F
007F007F00007F7F7FFF00FF7F7F7FE5E5E500007FE5E5E5FFFF0000007F00FF007F0000
E5E5E500FFFF7F7F7FFF00FF7F7F7FE5E5E5E5E5E500FFFFFFFF0000FF00FF00007F7F7F
FFFF007F7F7F00FFFF00FFFF00FF00E5E5E500007F7F000000FF007F0000007F007F7F7F
00FF00FFFF00
7F000000FF007F0000E5E5E5E5E5E500FFFF00007F00FFFF00007FFFFF0000007F00FF00
FF000000FF00FFFF00007F00FF00FFFF00FF00FF00007F00FFFF0000FFFF7F7F7FE5E5E5
007F00FF000000FFFF007F00FF0000FF00FFE5E5E5E5E5E500FFFF00007F7F00007F7F7F
E5E5E57F7F7F7F7F7F007F007F000000FFFF7F000000FFFFFF0000FF00FFE5E5E5FF0000
7F0000007F00
FF00FFFF00FFFF00FF00007F007F0000FF00FF00FF00FFFF007F00E5E5E500007FFFFF00
7F7F7FFF00FF007F007F7F7F00007FE5E5E5FF00FF00FFFF00007F7F0000FF0000007F00
00FFFFFF00FF00007F007F00007F00E5E5E5FF00FFFF0000007F0000FFFF00FF00007F00
FFFF0000FFFFFF00FF00FFFF007F00FFFF007F7F7F00FFFFE5E5E500FFFF007F0000FFFF
7F7F7FFF00FF
00FFFF007F0000007F7F7F7F007F0000007F7F00007F00007F7F7F00FFFF007F00FFFF00
FF00FF7F7F7F00007F7F0000FFFF007F0000FF0000E5E5E500FF007F0000007F00E5E5E5
00007FFF000000FFFFFFFF00E5E5E500FFFF00FFFFE5E5E57F0000007F00FFFF00FFFF00
FFFF00E5E5E5FF0000FF00FF007F007F7F7F00FFFFFF00FFFF00FFE5E5E57F00007F0000
7F0000007F00
00007FFFFF00007F007F000000FF0000FF00FF00FFFF00FF00FFFFFF00FFFF00FFFF0000
E5E5E5FFFF00007F007F000000FF007F7F7FE5E5E5FF0000E5E5E5FFFF00FF000000FF00
FF00FF00007FE5E5E57F000000FF007F00007F7F7F00007FFF000000FFFF7F000000FFFF
7F7F7F7F7F7F00007FFF00007F7F7F7F00007F0000007F00FF00FF7F7F7F007F0000FF00
00FF007F7F7F
007F0000007FE5E5E5E5E5E5007F0000FF00E5E5E500FFFF00FFFF7F7F7FE5E5E500007F
FF0000007F00FFFF00FF00FFE5E5E500FF0000FF00FF00FF00007FFF000000FFFFFFFF00
FF00FF00FF00FFFF0000FF00E5E5E500FF0000FF0000FF00FF00FF007F00FFFF00007F00
FF0000FF00FF00007FFFFF00E5E5E500FF007F7F7FFF0000FFFF00FF00FF00007F00FFFF
007F00FFFF00
00007FFF000000FFFF00FFFFFFFF0000FFFF00FFFFFFFF00FFFF0000007F00007F7F7F7F
E5E5E5FF00FF00007FFFFF00FFFF007F7F7FFF00FF00FFFFFF0000E5E5E57F000000FF00
00007F7F7F7F00FF0000007FFF00FF00FFFF007F007F7F7FE5E5E5FFFF0000007FE5E5E5
7F7F7F007F007F0000FF00FFFF00FFFFFF00FF00FFFF0000007F0000007F7F7F7F00FF00
7F7F7F7F0000
00007FFF00007F00007F0000007F00FF0000FF00FF00007F7F00007F7F7F00007F00007F
00007FFF000000007FFF00FFE5E5E500007F00FFFF7F7F7F00FFFFE5E5E57F7F7FFF00FF
FF00FF00FF0000007FFFFF0000FFFF00FF0000007F7F0000007F00FF000000007F00FFFF
FF00FF00007FFF000000FFFF7F7F7FE5E5E5FF00FFE5E5E5FFFF0000FF00007F0000FFFF
00007FFF00FF
7F00007F0000007F007F7F7F00007F7F7F7F7F7F7FE5E5E5FFFF007F0000FF0000007F00
FF00FFFF0000FFFF0000FFFF7F000000007F7F7F7F00007FFF00FFFF00007F7F7F007F00
FFFF007F0000FFFF00FF000000FF0000FF0000FFFF7F0000FF00FFFFFF0000FF007F7F7F
FFFF00FF00007F7F7F7F00007F0000FF00FF7F0000007F00FF00FFFFFF00FF00007F7F7F
7F0000E5E5E5
7F7F7FFF00FFE5E5E5FFFF00007F00FFFF007F0000FF00007F7F7FFF0000FFFF00FF00FF
E5E5E5007F00E5E5E500FF00E5E5E5FF00FF7F7F7FFFFF007F7F7F00FFFF00FFFF7F0000
7F00007F00007F7F7F00FFFF7F0000FFFF0000FF00E5E5E5007F00E5E5E57F7F7F7F7F7F
FF00007F0000FF00FF007F00FFFF00E5E5E500FF007F000000FFFF007F00FF00007F0000
00FF007F7F7F
00FFFF00007FFFFF00FF0000FF000000FF00FF0000FFFF0000FFFFE5E5E500FFFFFF0000
FF0000FF0000FF00FF00007F00FF00FF0000E5E5E5FF00FF7F7F7F7F7F7F00007FE5E5E5
7F7F7F00007FE5E5E5FFFF0000007FFFFF00FFFF007F7F7FFFFF0000FF00E5E5E5FFFF00
00FFFF00007FFF00FF00007FFFFF00E5E5E500FF00FFFF0000FF00FF00FFFF00FF7F7F7F
007F00FF00FF
00FF007F7F7F00FFFF00FFFF007F00FF0000E5E5E57F7F7F00FFFF00FFFFFFFF007F0000
FFFF007F000000FFFF00FF007F7F7F00007F00FF007F000000007FE5E5E5FF00FF7F7F7F
FF000000007FFF0000FFFF007F7F7FE5E5E500FF00007F0000007FFFFF00FFFF007F7F7F
E5E5E5FFFF00FF00FF007F0000FFFF007F00FF000000FF0000FFFF00FFFF00FFFF00FF00
7F7F7F007F00
00007FFFFF00E5E5E57F000000FF00FF00FF00007F7F7F7FFF00FFFF00FF00007F00FFFF
FFFF007F0000FFFF007F0000FF00FF7F0000E5E5E500FFFF7F7F7FFF000000FFFF00FF00
FF00FFFFFF00E5E5E5007F0000007F007F0000007FFFFF0000FFFF00007FE5E5E5007F00
00007FFFFF00FF00007F7F7FFFFF00FF00FFFFFF007F00007F7F7FFF0000007F00007F00
7F00007F7F7F
007F007F7F7F7F7F7F00007F7F7F7F7F0000FFFF0000FFFF00FFFFFF00FF007F0000007F
FF00FF7F7F7F007F00007F00FFFF00FF00FF00007F7F7F7FFF0000E5E5E5FF00FFFF00FF
00007FFFFF00007F00E5E5E57F0000FFFF00FF0000FF00FF007F0000007FFF00FF00FFFF
00FFFF007F007F00007F0000FFFF0000007F007F00FFFF0000007F00FFFFE5E5E5FF00FF
E5E5E500FFFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.754707 0.488926 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
E5E5E5FFFF00FFFF00FFFF0000FFFF00FFFF7F00007F00007F00007F0000E5E5E5FF00FF
E5E5E57F7F7F7F7F7F7F7F7F7F7F7FE5E5E5E5E5E5E5E5E5007F007F7F7F00FF0000FF00
E5E5E5E5E5E5E5E5E57F00007F00007F0000FF00FFFFFF007F00007F0000FFFF00FFFF00
FFFF007F7F7F7F7F7F7F7F7FFF00FFE5E5E5E5E5E57F00007F00007F0000E5E5E57F0000
7F00007F0000
FF00FF7F00007F00007F000000FFFF00FFFF7F0000E5E5E5E5E5E5E5E5E5FF00FFFF00FF
FF00FF7F7F7F7F7F7FFFFF00FF00FFE5E5E5E5E5E5E5E5E57F00007F7F7FFF00FFFF00FF
7F00007F00007F00007F0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00
FFFF007F7F7F7F7F7F7F7F7F7F7F7F7F7F7FE5E5E57F00007F0000FFFF00FFFF00FF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FF7F0000007F0000FFFF7F0000E5E5E5E5E5E5E5E5E5FF00FFFF00FF
7F7F7F7F7F7F7F7F7FFFFF00FF00FFE5E5E5E5E5E5E5E5E5E5E5E57F0000FF00FF7F0000
7F00007F000000FFFF00FFFF00FFFFE5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5
E5E5E57F7F7F7F7F7F7F7F7F00FFFF00FFFFE5E5E5E5E5E500FFFF00FFFF00FFFFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FF7F0000007F00007F00007F00E5E5E5E5E5E5E5E5E5FF00FFFF00FF
7F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FFFF00FFFF00FF00FF
FF00FFFF00FF00FFFF00FFFF00FFFFE5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5
E5E5E57F7F7FFF00FFFF00FFFF00FF00FFFFE5E5E5E5E5E500FFFF00FFFFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF0000FFFF7F00007F0000007F00FFFF00E5E5E5FF00FFFF00FFFF00FF
7F7F7F7F7F7FFF00FFFF00FFFF00FFFFFF00E5E5E5E5E5E5E5E5E5FFFF00FFFF00FF00FF
FF00FFFF00FF00FFFF00FFFF00FFFFE5E5E5FF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5
E5E5E57F7F7FFF00FFFF00FFFF00FF00FFFFE5E5E5E5E5E500FFFF00FFFFFF00FFFF00FF
FFFF00FFFF00
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FF007F007F00007F00007F0000FFFF00FFFF00
FFFF0000FFFFFF00FFFF00FFFF00FFFFFF00E5E5E5E5E5E5E5E5E5FFFF00FFFF00FF00FF
FF00FFFF00FF00FFFF00FFFF00FFFFE5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FF7F7F7FFF00FFFF00FFFF00FF7F00007F00007F000000FFFFFF00FFFFFF00FFFF00
FFFF00FFFF00
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFE5E5E57F00007F00007F0000FFFF00FFFF00
FFFF00FF00FF00FFFF00FFFF00FFFF00FFFFFFFF007F0000FFFF00FFFF00FFFF00007F00
FFFF00FF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FF0000FFFF7F7F7F7F7F7FFF00FFFF00FFFF00FF7F7F7F7F7F7F00FFFFFFFF00FFFF00
FFFF0000FF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000E5E5E5E5E5E5E5E5E5FFFF00
FFFF00FF00FF00FFFFFF00FFFF00FFFF00FFFFFF007F0000FFFF00FFFF00FFFF007F7F7F
E5E5E5E5E5E5FF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF7F7F7F7F7F7FFF00FFFF00FFFF00FF7F7F7F7F7F7F00FFFFFFFF00FFFF00
FFFF007F0000
7F7F7F7F7F7FFFFF00FF00FFFF00FFFF00FFFF00FF7F7F7FE5E5E5E5E5E5E5E5E57F0000
00FFFF00FF00FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF
E5E5E5E5E5E5FF00FFE5E5E5E5E5E5E5E5E500FFFFFF00FFFF00FFFF00FFFF00FFE5E5E5
E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F7F7F7F7F7F00FF00E5E5E5E5E5E5
E5E5E5FF00FF
FF00FFE5E5E5FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FF00FF
FF00FF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF
FFFF0000FF0000FF00E5E5E5E5E5E5E5E5E500FFFFFF00FFFF00FFFF00FFFF00FFE5E5E5
E5E5E5FF00FFFF00FFE5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FFFF00FF00FFE5E5E5FF00FF
FF00FFFF00FF
00FFFFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF
FF00FF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFE5E5E5
E5E5E5FF00FFFF00FFE5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFFFFFF00FFFF007F0000FF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFFFFF00
FFFF00FF00FFFF00FF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFF00FFFF00FFFF00FFE5E5E5
00FF0000FF0000FF00E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FFE5E5E5
E5E5E5E5E5E5
00FF00FFFF007F00007F000000FFFF00FFFFFFFF00FFFF00FFFF00FFFF00FFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F7F7FFF00FFFF00FF007F0000FFFF
00FFFFFF00FFFF00FF7F7F7F7F7F7F7F7F7FFF00FFE5E5E5E5E5E5E5E5E57F7F7F7F7F7F
7F7F7F00FF0000FF0000FF00FF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FF
FF00FFFFFF00
00FF0000FF00FF00FF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00FFFF00FFFF00FF00FF
FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FF7F7F7F7F7F7F7F7F7FFFFF00FFFF00
FFFF00FF00FFFF00FF7F7F7F7F7F7F7F7F7F00FFFFE5E5E57F7F7F7F7F7F7F7F7F7F7F7F
7F7F7F00FF0000FF0000FF00FF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FF
FF00FFFFFF00
00FF0000FF00FF00FFFF00FFFF00FF00FFFFFFFF00FFFF00FFFF00FFFF00FFFF007F7F7F
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF007F7F7F7F7F7FFFFF00FFFF00
FFFF0000FFFFFF00FFFF00FF7F0000FFFF00FFFF00FFFF007F7F7F7F7F7F7F7F7F00FF00
00FF0000FF007F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FF
FF00FFFFFF00
7F7F7F7F7F7FFF00FFFF00FFFFFF00FFFF00FFFF007F7F7F7F00007F0000E5E5E5E5E5E5
FFFF00FFFF00FFFF007F00007F0000FFFF00FFFF00FFFF007F7F7F7F7F7FFFFF00FFFF00
FFFF0000FFFF00FFFF00FFFF7F0000FFFF00FFFF00FFFF007F7F7F7F7F7F7F7F7F00FF00
00FF0000FF00FF00FF7F00007F00007F0000E5E5E5E5E5E5E5E5E500FFFF00FFFF00FFFF
00FFFF7F0000
7F7F7F7F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF0000FFFF00FFFFFFFF00E5E5E57F0000
FFFF00FFFF00FFFF00FFFF007F0000FFFF00FFFF00FFFF00FF00FFFFFF00FF00FFFF00FF
FF00FFFF00FF7F000000FFFF7F0000FFFF00FFFF00FFFF0000FFFFFFFF0000FF0000FF00
00FF0000FF0000FF00FFFF00FFFF00FFFF007F00007F000000FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF0000FFFF00FFFFFFFF00E5E5E57F0000
7F00007F00007F0000FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FF0000FF0000FF00
FFFF0000FFFFFF00FF00FFFF00FFFF7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF00FFFF00
FFFF00E5E5E500FF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7F7F7F7F7F7F7FFFFF00FFFF0000FFFF00FFFF00FFFFFFFF0000FFFF00FFFF
7F00007F00007F0000FFFF007F0000FFFF0000FFFF00FFFFE5E5E5E5E5E5E5E5E500FF00
FFFF0000FFFFFF00FF00FFFF00FF007F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF00FFFF00
FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7FFFFF00FFFF00FFFF0000FF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF
7F00007F00007F00007F00007F0000FFFF0000FFFF00FFFFE5E5E5E5E5E5E5E5E500FF00
FFFF0000FFFFE5E5E5E5E5E5E5E5E5E5E5E500FFFF00FFFF00FFFFFFFF00FFFF00FFFF00
FFFF00FFFF00FFFF0000FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFE5E5E5
E5E5E5E5E5E5
FFFF007F7F7FFFFF00FFFF00FFFF00FF00FFFF00FFFFFF007F7F7F00FFFF00FFFF00FFFF
E5E5E5E5E5E500FFFF7F00007F0000FFFF0000FFFF00FFFFE5E5E5E5E5E5E5E5E57F0000
00FFFF00FFFF00FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00
7F7F7FFFFF00FFFF0000FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFFE5E5E5
00FF0000FF00FFFF00E5E5E5E5E5E5E5E5E5FF00FFFF00FF00FF0000FFFF00FFFFE5E5E5
E5E5E5E5E5E5FFFF00FFFF00FFFF00FFFF007F7F7F7F7F7F7F7F7FFFFF007F7F7F7F7F7F
7F7F7F00FF0000FF00E5E5E5E5E5E5FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00
7F7F7FFFFF00FFFF0000FFFFE5E5E5E5E5E5E5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF
00FFFF00FF00
00FF0000FF0000FF00E5E5E5E5E5E5E5E5E5FF00FFFF00FF00FF007F0000FFFF00FFFF00
FFFF00FFFF00E5E5E5E5E5E5E5E5E5FFFF007F7F7F7F7F7F7F7F7FFFFF007F7F7F7F7F7F
7F7F7FFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5
E5E5E5FF00FF00FFFF00FFFFE5E5E5E5E5E5E5E5E5FF00FFFFFF00FFFF0000FFFF00FFFF
00FFFFFFFF00
FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E57F0000FF00FFFFFF00E5E5E5E5E5E5FFFF00
FFFF00FFFF00E5E5E5E5E5E5E5E5E57F7F7F7F7F7FE5E5E5E5E5E5FF00FFFFFF00FFFF00
FFFF00FF00FF7F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FF00FFFF00FF7F0000E5E5E5
E5E5E500FFFF7F0000FF00FFE5E5E5E5E5E5E5E5E5FF00FFFFFF00FFFF00FFFF0000FFFF
FFFF00FFFF00
FFFF00FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5FFFF00FFFF00E5E5E5E5E5E5FFFF00
FFFF00FFFF00E5E5E5E5E5E5E5E5E57F7F7F7F7F7FE5E5E5FF00FFFF00FFFF00FFFFFF00
00FF0000FF0000FF0000FF0000FF00FFFF00FFFF00FFFF00FF00FFFF00FF7F7F7F7F7F7F
7F7F7FFF00FF7F0000E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FFFF00FFFF00FFFF0000FFFF
FFFF00FFFF00
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF00FFFF00FFFF
7F0000E5E5E5E5E5E5E5E5E57F7F7F7F7F7F7F7F7FE5E5E5FF00FF00FFFF00FFFF00FFFF
00FF0000FF0000FF0000FF0000FF00FFFF00FFFF00FFFF00FFFF00FF00FF7F7F7F7F7F7F
7F7F7F00FFFF7F0000E5E5E5E5E5E5E5E5E57F0000E5E5E5FF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FF00FFFF00FFFF00FF
00FFFFE5E5E57F7F7F7F7F7F7F7F7F7F00007F00007F0000FF00FF00FFFF00FFFF00FFFF
00FF0000FF0000FF0000FF00FFFF00FFFF00FFFF00FFFF0000FF0000FF007F7F7F7F7F7F
7F7F7F00FFFF7F0000E5E5E5E5E5E5E5E5E5FF00FFFF00FFFFFF007F7F7F00FFFFFF00FF
FF00FFFF00FF
E5E5E5E5E5E5E5E5E500FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FF00FFFF00FFFF00FF
00FFFF7F00007F7F7F7F7F7FFF00FFFFFF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF
E5E5E5E5E5E500FF0000FF00FFFF00FFFF00FFFF00FFFF0000FF0000FF0000FF00E5E5E5
00FFFFFFFF00FFFF00FFFF00FF00FF00FF00FF00FFFF00FFFF00FFFFFF0000FFFFFF00FF
FF00FFFF00FF
E5E5E5E5E5E5E5E5E5FFFF00E5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFE5E5E57F7F7F7F7F7FFF00FFFFFF00FFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E500FF007F7F7FFFFF00FFFF00FFFF00FF00FF00FF0000FF0000FF00E5E5E5
00FFFFFFFF00FFFF00FFFF007F000000FF00FF00FFFF00FFFF00FFFFFF00FF00FFFF00FF
FF00FFFF00FF
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FF00FF00FFFF00FFFF00FF0000FF00
00FF00E5E5E5E5E5E5E5E5E5FF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF7F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FF00FF0000FF0000FF00FFFF00
00FFFFFFFF00FFFF00FFFF00FF00FF00FF00FF00FFFF00FFFF00FFFFFF00FF00FFFF00FF
FF00FF7F7F7F
FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FFFF0000FFFF00FFFF00FF0000FF00
00FF00E5E5E5E5E5E500FFFFFF00FFE5E5E5E5E5E5E5E5E5E5E5E5E5E5E5FF00FFFF00FF
7F0000FFFF007F7F7F7F7F7F7F7F7F7F7F7FFF00FFFF00FF00FF0000FF0000FF00FFFF00
FFFF007F0000FF00FFFF00FFFF00FFFFFF007F7F7F7F7F7FFFFF00FFFF00FF00FFFF00FF
FF00FF7F7F7F
FFFF00FFFF00E5E5E5E5E5E5E5E5E5E5E5E5FFFF00FF00FFFF00FFFF00FF00FFFF00FF00
00FF007F7F7F7F000000FFFFFF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FF00FFFF00FF
FF00FFFFFF0000FF0000FF0000FF0000FF0000FF00E5E5E5FF00FFFF00FFFF00FFE5E5E5
FF00FFFF00FF7F7F7F7F7F7F7F7F7FFFFF007F7F7FFFFF00FFFF00FFFF00FF00FF00FFFF
00FFFF7F7F7F
FF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FFFF0000FFFF00FFFF00FFFF00FFFFE5E5E5
E5E5E57F7F7F7F0000FF00FFFF00FF7F7F7F7F7F7FE5E5E5E5E5E5E5E5E5FF00FF00FF00
00FF0000FFFFFF00FFFF00FFFF00FF00FF00E5E5E5007F007F00007F00007F0000E5E5E5
FF00FFFF00FF7F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00
E5E5E5FF00FF
FF00FF7F00007F00007F00007F00007F00007F0000FFFF00FFFF00FFFF0000FFFF00FFFF
E5E5E5FFFF00FFFF00FF00FFFF00FF7F7F7F7F7F7F7F7F7FFF00FFFF00FFFF00FFFFFF00
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF007F007F7F7FE5E5E5E5E5E5FFFF00FFFF00
FFFF0000FFFF7F7F7F7F7F7F7F7F7FE5E5E5E5E5E5FFFF00FFFF00FFFF00FF00FFFFFF00
7F7F7F007F00
7F7F7F7F7F7F7F7F7F7F0000E5E5E5FFFF0000FFFFFFFF00FFFF00FFFF00E5E5E5E5E5E5
7F7F7F7F7F7FFFFF00FF00FFFF00FF7F7F7F7F7F7F7F7F7FFF00FFE5E5E5E5E5E5FFFF00
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF7F0000FFFF00E5E5E5E5E5E5FFFF00FFFF00
FFFF00FFFF00FFFF00FFFF00FF00FFE5E5E5E5E5E5E5E5E500FFFF00FFFFFFFF00FFFF00
7F7F7F007F00
7F7F7F7F7F7F7F7F7F7F7F7FFF00FFFFFF00FF00FFFFFF00FFFF00FFFF00E5E5E5E5E5E5
7F7F7F7F7F7FFFFF0000FFFF00FFFF00FFFFE5E5E5FFFF00FFFF00FFFF00E5E5E5FFFF00
FFFF00FFFF0000FFFFFF00FFFF00FFFF00FF00FFFF00FFFFE5E5E5E5E5E5FFFF00FFFF00
FFFF00FFFF00FFFF00FFFF00FF00FFE5E5E5E5E5E5E5E5E57F7F7F00FF0000FF007F7F7F
7F7F7FFFFF00
7F7F7F7F7F7F7F7F7FE5E5E5E5E5E5FFFF00FF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5
7F7F7F7F7F7FFFFF00FF00FFFF00FFFF00FFE5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5
E5E5E5E5E5E500FFFF00FFFF7F7F7F7F000000FFFF00FFFF00FFFF00FFFF7F7F7F7F7F7F
FF00FFFFFF00FFFF00FF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFE5E5E5E5E5E5E5E5E5
FF00FFFFFF00
FF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FFE5E5E5E5E5E500FF0000FF00
00FF00FFFF007F7F7F7F7F7F7F7F7FFF00FFE5E5E5FFFF00FFFF00FFFF00E5E5E5E5E5E5
FF00FFFF00FFFF00FF00FFFF7F7F7F7F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF7F7F7F
7F7F7F00FFFF00FFFF00FFFFFFFF00FFFF00FFFF007F0000FF00FFE5E5E5E5E5E5E5E5E5
FF00FFFF00FF
FF00FFFF00FF7F7F7F7F7F7F7F7F7FFF00FFFF00FFFF00FFE5E5E5E5E5E500FF0000FF00
00FF007F7F7F7F7F7F7F7F7F7F7F7FFFFF00FF00FFFFFF00FFFF00FFFF00E5E5E5E5E5E5
FF00FFFF00FFFF00FFFFFF00FFFF00E5E5E5E5E5E5E5E5E500FFFFFFFF00FFFF00FFFF00
FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFE5E5E5E5E5E5
FF00FFFF00FF
FFFF00FFFF00FFFF007F7F7F7F7F7FFF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FF00FF
7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFF00FF00FFFF00FFFFFF00FFFF00FFFF00E5E5E5
FF00FFFF00FFFF00FFFFFF00FFFF00E5E5E5E5E5E5E5E5E500FFFFFFFF00FFFF00FFFF00
00FFFFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFE5E5E500FFFF
FF00FFFF00FF
FFFF00FFFF00FFFF007F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFE5E5E5
7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF007F0000FF00FFFFFF00FFFF00FFFF00E5E5E5
E5E5E500FFFFFFFF00FFFF00FFFF00E5E5E5E5E5E5E5E5E500FFFFFFFF00FFFF00FFFF00
00FFFFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000
7F7F7F7F7F7F
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF007F7F7F7F7F7F
7F7F7FE5E5E5FFFF00FFFF007F0000FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00
FFFF0000FF00FFFF00E5E5E5E5E5E5E5E5E57F7F7F7F7F7F00FFFFFFFF00FFFF00FFFF00
00FFFF7F7F7F7F00007F0000FF00FF7F0000007F00FF00FFFF00FFFF00FFFF00FF7F7F7F
00FF007F7F7F
007F00E5E5E500FFFFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF007F7F7F7F7F7F
7F7F7FE5E5E5FFFF00FFFF00E5E5E5E5E5E5FF00FF00FFFF00FFFF7F0000FF00FFFF00FF
FF00FFFF00FF00FF00E5E5E5E5E5E5E5E5E57F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00
7F7F7F7F7F7F7F00007F0000FF00FFFF00FFE5E5E5E5E5E5FF00FFFF00FFFF00FF7F7F7F
00FF007F7F7F
7F00007F00007F7F7F7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF00FFFF007F7F7F7F7F7F
7F7F7FE5E5E5FF00FF7F0000E5E5E57F7F7F7F7F7F7F7F7F00FFFF7F0000FF00FFFFFF00
FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF7F7F7F7F7F7FFFFF00FFFF00FFFF007F7F7F
7F7F7F7F7F7F7F00007F00007F00007F0000E5E5E5E5E5E5E5E5E57F7F7F7F7F7F7F7F7F
00FF007F7F7F
FF00FFFF00FF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FE5E5E57F7F7F7F7F7F7F7F7FFF00FF
FF00FFFF00FFFF00FFE5E5E57F00007F7F7F7F7F7F7F7F7FFF00FF00FFFFFF00FFFFFF00
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF007F00007F00007F0000FF00FFFFFF00FFFF00
FFFF00FF00FFFF00FFFF00FFFFFF00FFFF007F000000FFFF00FFFF00FFFFFFFF007F7F7F
7F7F7F7F0000
FF00FFFF00FF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FE5E5E5E5E5E500FFFF00FFFFFF00FF
FF00FFFF00FFFF00FF7F0000E5E5E57F7F7F7F7F7F7F7F7F7F7F7F7F7F7FE5E5E5FFFF00
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF007F7F7F7F7F7F7F7F7FFF00FFFFFF00FFFF00
FFFF00FF00FFFF00FFFF00FFFFFF00FFFF007F000000FFFFFF00FFFF00FFFF00FFFF00FF
7F7F7F7F0000
FF00FFFF00FFFF00FFE5E5E5FFFF00FFFF00FFFF0000FFFF00FFFFFFFF00FFFF00FFFF00
FFFF00FFFF0000FFFF00FFFFE5E5E5E5E5E57F7F7F7F7F7F7F7F7F7F7F7FE5E5E5E5E5E5
7F00007F0000E5E5E57F7F7F7F7F7F7F7F7FE5E5E57F7F7F7F7F7FFFFF00FFFF00E5E5E5
E5E5E5FF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FF
7F7F7F7F7F7F
00FF0000FFFF00FFFF00FFFF00FFFFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FF00FFFF
00FFFFFFFF00FFFF00FFFF00FF00FFE5E5E5E5E5E5E5E5E57F7F7F7F7F7FE5E5E5E5E5E5
E5E5E5E5E5E5E5E5E57F7F7F7F7F7F7F7F7FFFFF0000FFFF00FFFF00FFFFE5E5E5E5E5E5
FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF
7F7F7F7F7F7F
FFFF0000FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FF00FFFF
00FFFFFFFF00FFFF00FFFF00FF00FFE5E5E5E5E5E5E5E5E57F0000E5E5E5FF00FFFF00FF
FF00FFE5E5E5E5E5E57F7F7F7F7F7F7F7F7FFFFF0000FFFF00FFFF00FFFFE5E5E5E5E5E5
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF7F7F7F
7F7F7F7F7F7F
FFFF00FFFF00FFFF007F0000FF00FFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FF00FFFF
00FFFFFFFF00FFFF00FFFF00FF00FFE5E5E5E5E5E5E5E5E500FFFF00FFFFFF00FFFF00FF
FF00FFE5E5E5E5E5E5E5E5E5E5E5E5FFFF00FFFF0000FFFF00FFFF00FFFFE5E5E5E5E5E5
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00E5E5E5FF00FFFF00FF
7F00007F0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.754707 0.732558 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
7F00007F0000FFFF0000FFFF00FFFF00FFFFE5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FF
FF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FF
FF00FFE5E5E5E5E5E5FFFF00FFFF00FFFF00FFFF007F00007F00007F00007F0000FFFF00
FFFF007F7F7F7F7F7F7F7F7FFF00FFE5E5E57F00007F0000FFFF00FFFF00FFFF00FF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFE5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FF
FF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFFFF00FFFF00FFFF00FFFF007F00007F00007F00007F0000FFFF00
FFFF007F7F7F7F7F7F00FFFF00FFFF00FFFF7F00007F0000FFFF00FFFF00FFFF00FF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFE5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F00007F0000FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00
E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF7F00007F0000FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF00FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E57F0000FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5
E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5FFFF00FF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
7F0000FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF7F00007F0000FFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FF
FF00FFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E57F0000E5E5E5
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F00007F0000FFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00
7F00007F0000
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E57F000000FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF00
FF00FFFF00FF
FF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFFFF00FF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFFFF00FF00FFFF00FFE5E5E500FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FF7F7F7F00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF007F00007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFF00FFFF00FF7F7F7F00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
7F7F7F00FF0000FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF0000FFFFFFFF007F7F7F7F7F7F7F7F7F
7F7F7F00FF0000FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FF00FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF7F00007F00007F00007F0000E5E5E5
FF00FFFFFF007F00007F0000FF00FFFF00FFFF00FFFF00FF7F7F7FFFFF00FFFF00FFFF00
00FFFFFF00FFFF00FFFF00FF00FFFF7F0000FFFF0000FFFFFFFF007F7F7F7F7F7F00FF00
00FF00FF00FF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF
7F7F7FFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF7F0000FFFF00
7F0000FFFF00FFFF007F00007F00007F0000FFFF00FFFF00FFFF00FF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FF00FFFF7F0000FFFF0000FFFF00FFFF00FFFFFFFF0000FF00
00FF00FF00FF7F0000FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FF
FF00FF00FFFF
7F7F7FFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFFFFFF007F0000
FFFF00FFFF007F0000FFFF007F000000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FF00FFFF7F0000FFFF0000FFFF00FFFF00FFFFFFFF0000FF00
00FF00FF00FF7F00007F00007F000000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7F7F7F7F7F7F7FFFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF
00FFFF7F0000FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FF00FFFF7F0000FFFF0000FFFF00FFFF00FFFFFFFF00FFFF00
FF00FFFF00FFFFFF00FFFF00FFFF0000FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7F7F7F7F7F7F7FFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF7F0000FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFFE5E5E5FFFF00
00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00
FF00FFFFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7FFFFF00FFFF00FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF7F0000FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF00FFFFE5E5E57F0000FFFF00
FFFF00FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00
FF00FFFFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
FFFF007F7F7FFFFF00FFFF00FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF7F000000FFFF7F0000FFFF00FFFF00FFFF0000FFFF00FFFFE5E5E57F0000FFFF00
FFFF00FFFF0000FFFFE5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00
FFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
00FF0000FF00FFFF00E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
E5E5E500FFFF00FFFF00FFFF7F000000FFFFFFFF0000FFFF00FFFFE5E5E57F000000FFFF
00FFFF00FFFF00FFFFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00E5E5E5E5E5E5
FF00FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
00FF0000FF0000FF00E5E5E5E5E5E5FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF007F7F7FFF00FFFF00FFFF00FF7F7F7F
FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF7F0000
00FFFF00FFFF00FFFF00FFFF00FFFFE5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFF007F0000FFFF00
FFFF00FFFF00FFFF00E5E5E5E5E5E57F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FF
FFFF00FF00FFFF00FFFF00FF7F7F7FFFFF00FFFF00FF00FFFF00FFFF00FFFF00FF7F0000
FF00FF00FFFF00FFFF00FFFFE5E5E5E5E5E5FF00FFFFFF00FFFF0000FFFF00FFFF00FFFF
00FFFFFFFF00
FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF7F0000E5E5E5E5E5E5E5E5E57F7F7FE5E5E5FF00FF00FFFF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FF00FF0000FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF7F0000
FF00FFFF00FFFF00FF7F0000E5E5E57F00007F00007F0000FF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FF7F0000E5E5E5E5E5E57F00007F00007F0000FF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FF0000FF0000FF0000FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF7F7F7F
FF00FFFF00FFFF00FF7F0000E5E5E57F0000FF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FF
00FFFFFFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FF7F00007F0000FF00FF7F00007F00007F0000FF00FF00FFFF00FFFF00FFFF00FFFF
00FFFF00FF0000FF0000FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FF00FFFF
00FFFF00FFFF7F00007F0000E5E5E5FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF7F0000FF00FF7F0000FFFF007F0000FF00FF00FFFF00FFFF00FFFF00FFFF
00FFFFE5E5E500FF0000FF00FFFF00FFFF00FF00FFFF00FFFF00FF00FF0000FF0000FFFF
00FFFFFFFF00FFFF007F00007F0000FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FF
E5E5E5E5E5E500FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF7F0000FF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFFFF00FFFF00FF
FF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FF00FFFF0000FFFF
00FFFF00FFFFFFFF00FF00FF7F0000FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF
FF00FFFF00FF
E5E5E5E5E5E5FFFF00FFFF00FFFF00E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FF
FF00FF7F0000FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FF00FFFF0000FFFF
00FFFF00FFFF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00E5E5E5E5E5E5E5E5E5FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
E5E5E57F000000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFE5E5E5E5E5E5E5E5E500FFFFFF00FFFF00FFFF00FFFF00FF00FFFF
E5E5E57F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FF
FF00FF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F7F7FFFFF00FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FF7F00007F00007F000000FFFFFF00FFFF00FFFF00FFFF00FF00FFFF
00FFFF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FFFF00FFFF00FF7F7F7F7F7F7FFFFF00FFFF00FFFF00FFFF00FF00FF00FFFF00FFFF
00FFFF00FFFF
FF00FFFF00FFFF00FFE5E5E5E5E5E500FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF7F0000FF00FFFF00FFFF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF007F00007F00007F00007F0000
00FFFF00FFFF00FFFFFFFF00FF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF7F7F7F
7F7F7F7F7F7F
7F00007F00007F0000E5E5E5E5E5E500FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF
00FFFFFFFF0000FFFF00FFFF00FFFFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFFFF00E5E5E5E5E5E5FFFF00
00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF00FFFF7F7F7F
7F7F7F7F7F7F
7F7F7F7F7F7F7F0000E5E5E5E5E5E500FFFF00FFFF00FFFFFF00FFFF00FFFFFF00E5E5E5
7F7F7FFFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FFFF00
FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFFFF00
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF7F7F7F
7F7F7F7F7F7F
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5E5E5E5
7F7F7FFFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFFFF00FFFF00FFFF00E5E5E5FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF7F0000FF00FFFF00FFFF00FFE5E5E5FF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FF0000FF00
00FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00E5E5E5FF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF
FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FF
FF00FF7F7F7F7F7F7F7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00E5E5E5FF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF0000FFFF
FF00FFFF00FFFF00FFFF00FF00FFFFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FF7F7F7FFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FF
FF00FF7F7F7FFFFF00FFFF00FFFF007F0000FF00FFFF00FFFF00FFFFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00E5E5E500FFFFFFFF00FFFF00FFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FF
FF00FF7F7F7FFFFF007F00007F0000FF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFFFF00E5E5E5E5E5E500FFFF00FFFF00FFFFFFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
E5E5E5E5E5E5E5E5E500FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FF
7F7F7FFFFF00FFFF007F00007F0000FF00FFFF00FFFF00FFFF00FF00FFFF7F0000FF00FF
00FFFF00FFFF00FFFFFFFF00E5E5E5E5E5E57F7F7F00FFFF00FFFF00FFFFFFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
7F00007F7F7F
E5E5E5E5E5E5E5E5E500FFFFFFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF007F7F7F
7F7F7FFF00FF7F00007F00007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF0000FFFF
00FFFF00FFFF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FF007F7F7F
FF00FFFF00FFFF00FF00FFFFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF7F0000FF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFF7F00007F00007F0000FF00FFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFF00FFFF00FFFF00FF
7F00007F0000
FF00FFFF00FFFF00FF7F7F7F7F7F7F7F7F7F7F7F7FE5E5E500FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FF7F00007F00007F7F7F00FFFF00FFFFFF00FFFF00FFFF00FF
FFFF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF7F00007F00007F0000FF00FFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000
FF00FFFF00FFFF00FFFF00FFFFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF7F00007F7F7FFF00FF00FFFF00FFFF00FFFF7F0000
7F00007F00007F0000E5E5E5FFFF00FFFF007F00007F00007F00007F0000FFFF00FFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFFE5E5E5E5E5E5E5E5E5E5E5E57F7F7FE5E5E57F0000
7F00007F00007F0000E5E5E5FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FF7F000000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FF
FF00FF7F00007F0000E5E5E57F7F7FFFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F7F7F
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFFFF00FFFF00FFFF00FF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FF
FF00FFFF00FFE5E5E5E5E5E57F7F7FFFFF0000FFFF00FFFF00FFFF00FFFF00FFFFE5E5E5
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFF00FFFFFFFF00FF00FFFF00FFFF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FF
FF00FFFF00FFE5E5E5E5E5E5E5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFE5E5E5
FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF
FF00FF7F0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.754707 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0.0238095 0.0190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
150 string
50 50 8 [50 0 0 50 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFE5E5E5FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFFFFFF00FFFF007F00007F00007F00007F00007F00007F0000
FFFF007F7F7F00FFFF00FFFF00FFFF00FFFF00FFFF7F00007F0000FFFF00FF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFFE5E5E5FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFFFF00FFFF007F00007F00007F00007F00007F00007F0000
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FFFF00FF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF7F0000
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFFFF00FF00FFFF00FF
FF00FF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFFFFFF0000FFFFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00
00FFFF00FFFF7F0000FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFFFF00
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00
00FFFF00FFFF7F0000FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFFFF00
FF00FF00FFFFFFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7F7F7F7FFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FFFF00
00FFFF00FFFFFFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFFFF00
FF00FF00FFFFFFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
7F7F7F7F7F7FFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFFFF00FFFF0000FFFF00FFFF00FFFF00FFFFFFFF0000FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00
FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
FFFF007F7F7FFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF7F0000FFFF00FFFF00
00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
00FF0000FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFF00FFFF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00
FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFFFF00FF00FFFFFFFF00FFFF00FFFF0000FFFF00FFFF
00FFFF00FFFF
FFFF0000FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00
FFFF0000FFFF7F0000FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFFE5E5E500FFFFFFFF00FFFF00FFFF0000FFFF00FFFF
00FFFF00FFFF
FFFF0000FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFE5E5E5E5E5E57F00007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF7F0000FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFFFF00FFFF00FF00FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF
00FFFF00FFFF00FF0000FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFE5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FFFF00FFFF00FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
FFFF00FFFF00FFFF00E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
E5E5E500FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFE5E5E5E5E5E5FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF7F000000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF7F000000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF0000FFFF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF
7F7F7F7F7F7F
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF
FFFF0000FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF0000FFFF00FFFF
FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00
FFFF0000FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFFFFFF0000FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF7F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF7F00007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFFFF0000FFFF00FFFF00FFFFFFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
E5E5E5FFFF0000FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFFFF00FFFF007F0000FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFFFFFF00FFFF00FF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF7F0000
FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF7F000000FFFF00FFFF00FFFF00FFFF00FFFF7F0000FF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FF
FF00FF7F00007F000000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF00FFFF00FFFF00FFFF00FFFF7F00007F0000FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF7F00007F0000FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
FF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF7F00007F0000FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF7F0000E5E5E500FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFF00FFFF00FFFF00FFFF7F00007F0000FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FF7F0000FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
00FFFF00FFFF00FFFF00FFFF00FFFFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FF00FFFF00FFFFFFFF00FF00FFE5E5E57F00007F0000FF00FFFF00FFFF00FFFF00FF
FF00FFFF00FFFF00FFE5E5E5FFFF0000FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF
FF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FFFF00FF
FF00FFFF00FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[12]:=",
  ImageSize->{440, 440},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006h0001^2000`400?l00000o`00003oooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`003?oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
o`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool0
00coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
ool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oo
oomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
oooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
oed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`00
3?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
o`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool0
00coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
ool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`00ooooooniooooo`00ooooooniooooo`00oooo
ooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00oooo
ooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00oooo
ooniooooo`00ooooooniooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
o`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool0
00coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
ool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oo
oomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
oooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
oed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`00
3?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
o`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM
0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>
ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool0
00coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
ool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00
ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00
ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00
3?ooool;0?l0o`800?oo:@3o0?l40?oo00L0O`00703o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?ooool;0?l0o`800?oo:@3o0?l40?oo00L0O`00703o0?l>oooooed0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?ooool;0?l0o`800?oo:@3o0?l2003oobD0o`3o3_oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo2`3o0?l2003oobT0o`3o0P00oolU0?l0
o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o
3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>
oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooo
oed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0
o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oo
oomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM
0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0
o`koooooG@3o0?l>ooooo`003?oooolm0?l0o`80ool07P3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>ooooo`003?oooolm0?l0o`80ool07P3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooolh0?l0o`@00?oo1@3oo`0L0?l0o`kooooo?@3o0?l20?oo01h0o`3o3_oo
oomM0?l0o`koooooG@3o0?l>ooooo`003?oooolh0?l0o`@00?oo1@3oo`0L0?l0o`kooooo?@3o0?l2
0?oo01h0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?ooool=0?l0o`d00?oo7P3o0?l20?oo
00800?oo1@3oo`02003ooaX0o`3o3_oooolA0?l0o`D0ool0903o0?l7003ooa`0o`3o3_oooomM0?l0
o`koooooG@3o0?l>ooooo`003?ooool=0?l0o`d00?oo7P3o0?l20?oo00800?oo1@3oo`02003ooaX0
o`3o3_oooolA0?l0o`D0ool0903o0?l7003ooa`0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`00
3?ooool=0?l0o`H00?oo00<0ool0003oo`00ool01P00oolL0?l0o`80ool02@00oolJ0?l0o`kooooo
3`3o0?l2003oo`D0ool00P00oolP0?l0o`T00?oo703o0?l>oooooa40o`3o0P00ool00`3oo`000?oo
003oo`0V0?l0o`<00?oo7P3o0?l>oooooed0o`3o3_ooool000cooooo3@3o0?l6003oo`030?oo0000
ool00?oo00H00?oo703o0?l20?oo00T00?oo6P3o0?l>ooooo`l0o`3o0P00ool50?oo00800?oo803o
0?l9003ooa`0o`3o3_oooolA0?l0o`800?oo00<0ool0003oo`00ool09P3o0?l3003ooah0o`3o3_oo
oomM0?l0o`kooooo000<ooooo`d0o`3o1P00ool00`3oo`000?oo003oo`02003oo`H0ool01`00oolC
0?l0o`80ool02@00oolJ0?l0o`kooooo4@3o0?l50?oo00H00?oo7P3o0?l7003ooa`0o`3o3_oooolD
0?l0o`800?oo0P3o0?l60?oo01h0o`3o0`00oolN0?l0o`koooooG@3o0?l>ooooo`003?oooolC0?l0
o`D00?oo2`3oo`02003ooa<0o`3o4@00ool>0?l0o`H00?oo3_oooolF0?l0o`@00?oo0P3oo`07003o
oaL0o`3o1`00oolL0?l0o`kooooo603o0?l90?oo01/0o`3o0`00oolN0?l0o`kooooo6P3o0?l20?oo
0440o`3o3_ooool000cooooo4`3o0?l5003oo`/0ool00P00oolC0?l0oa400?oo3P3o0?l6003oo`ko
oooo5P3o0?l4003oo`80ool01`00oolG0?l0o`L00?oo703o0?l>oooooaP0o`3o2@3oo`0K0?l0o`<0
0?oo7P3o0?l>oooooaX0o`3o0P3oo`110?l0o`kooooo000<ooooo`@0Ogmo403o0?l4003oo`/0ool0
1000oolA0?l0oa400?oo3P3o0?l6003oo`kooooo5P3o0?l8003oo`<0ool00P00oolG0?l0o`d00?oo
4P3o0?l4003oo`kooooo603o0?l90?oo03`0o`3o3_oooolJ0?l0o`@00?oo0P3oo`0m0?l0o`kooooo
000<ooooo`@0Ogmo403o0?l4003oo`/0ool01000oolA0?l0oa400?oo3P3o0?l6003oo`kooooo5P3o
0?l8003oo`<0ool00P00oolG0?l0o`d00?oo4P3o0?l4003oo`kooooo603o0?l90?oo03`0o`3o3_oo
oolJ0?l0o`@00?oo0P3oo`0m0?l0o`kooooo000<ooooo`80ool00P1oOgl@0?l0o`P00?oo1`3oo`0E
0?l0oa400?oo3P3o0?l6003oo`kooooo0P3oo`0D0?l0o`/00?oo>03o0?l4003oo`kooooo603o0?l4
003oo`D0ool0>P3o0?l2003oo`kooooo703o0?l2003oocl0o`3o3_ooool000cooooo0P3oo`0207mo
Oa00o`3o2000ool70?oo01D0o`3o4@00ool>0?l0o`H00?oo3_ooool20?oo01@0o`3o2`00oolh0?l0
o`@00?oo3_oooolH0?l0o`@00?oo1@3oo`0j0?l0o`800?oo3_oooolL0?l0o`800?oo?`3o0?l>oooo
o`003?ooool40?oo0100o`3o3@00oolE0?l0o`800?oo7`3o0?l6003oo`kooooo1000oolB0?l0o`P0
0?oo>`3o0?l4003oo`kooooo6P3o0?l2003ood40o`3o3_oooomM0?l0o`kooooo000<ooooo`@0ool0
403o0?l=003ooaD0o`3o0P00oolO0?l0o`H00?oo3_ooool4003ooa80o`3o2000oolk0?l0o`@00?oo
3_oooolJ0?l0o`800?oo@@3o0?l>oooooed0o`3o3_ooool000cooooo1P00ool>0?l0o``00?oo5P3o
0?l2003ooa`0o`3o2@00ool>ooooo`@00?oo503o0?l6003oocl0o`3o3_ooool2003ooe/0o`3o3_oo
oomM0?l0o`kooooo000<ooooo`H00?oo3P3o0?l<003ooaH0o`3o0P00oolL0?l0o`T00?oo3_ooool4
003ooa@0o`3o1P00oolo0?l0o`kooooo0P00oomK0?l0o`koooooG@3o0?l>ooooo`003?ooool6003o
oa00o`3o2P00oolm0?l0o`kooooo1000oomI0?l0o`kooooo0P00oomK0?l0o`koooooG@3o0?l>oooo
o`003?ooool6003ooa00o`3o2P00oolm0?l0o`kooooo1000oomI0?l0o`kooooo0P00oomK0?l0o`ko
ooooG@3o0?l>ooooo`003?ooool7003ooaL0o`3o0P3oo`0m0?l0o`kooooo1000oomI0?l0o`kooooo
0P00oomK0?l0o`koooooG@3o0?l>ooooo`003?ooool7003ooeH0o`3o3_ooool60?oo05L0o`3o3_oo
ool4003ooeT0o`3o3_oooomM0?l0o`kooooo000<ooooo`L00?ooEP3o0?l>ooooo`H0ool0E`3o0?l>
ooooo`@00?ooF@3o0?l>oooooed0o`3o3_ooool000cooooo103oo`05003ooe@0o`3o3_ooool60?oo
05L0o`3o3_ooool40?oo05T0o`3o3_ooool2003ooe/0o`3o3_ooool000cooooo103oo`05003ooe@0
o`3o3_ooool60?oo05L0o`3o3_ooool40?oo05T0o`3o3_ooool2003ooe/0o`3o3_ooool000cooooo
2@00oomD0?l0o`kooooo1P3oo`1G0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo2@00
oomD0?l0o`kooooo1P3oo`1G0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo1`00ooll
0?l0o`@00?oo5P3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?ooool7003o
oc`0o`3o1000oolF0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0
o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oo
oomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool0
00coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooomM0?l0o`ko
ooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooed0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>oooooed0o`3o3_ooool000coooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o
0?l>ooooo`003?oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooo
ocd0o`3o2000oolH0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<ooooocd0
o`3o2000oolH0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<ooooocd0o`3o
0P00oolN0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<ooooocd0o`3o0P00
oolN0?l0o`koooooG@3o0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooaH0o`3o1000oolS
0?l0o`800?oo7P3o0?l>oooooed0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolF0?l0
o`@00?oo8`3o0?l2003ooah0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
oooo?03o0?l50?oo01`0o`3o3_oooolm0?l0o`800?oo7P3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
oooo000<oooooc`0o`3o1@3oo`0L0?l0o`kooooo?@3o0?l2003ooah0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?ooooll0?l0o`D0ool0703o0?l>oooooc`0o`3o00<00?oo0?oo003oo`007P3o
0?l>oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooc`0o`3o1@3oo`0L0?l0o`kooooo?03o0?l0
0`00ool0ool00?oo000N0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000cooooo>03o0?l20?oo
00800?oo1@3oo`0L0?l0o`kooooo?03o0?l3003ooah0o`3o3_oooomM0?l0o`koooooG@3o0?l>oooo
o`003?oooolf0?l0o`@0ool01@00oolN0?l0o`kooooo>03o0?l5003oob00o`3o3_oooomM0?l0o`ko
ooooG@3o0?l>ooooo`003?oooolf0?l0o`@0ool01@00oolN0?l0o`kooooo>03o0?l5003oob00o`3o
3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolf0?l0o`@0ool01@00oolN0?l0o`kooooo>03o
0?l5003oob00o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolf0?l0o`@0ool01@00oolN
0?l0o`kooooo>03o0?l5003oob00o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolf0?l0
o`/00?oo703o0?l>ooooocP0o`3o0P00oolS0?l0o`koooooG@3o0?l>oooooed0o`3o3_ooool000co
oooo=P3o0?l;003ooa`0o`3o3_oooolh0?l0o`800?oo8`3o0?l>oooooed0o`3o3_oooomM0?l0o`ko
oooo000<oooooc@0o`3o2000oolQ0?l0o`kooooo>03o0?l2003oob<0o`3o3_oooomM0?l0o`kooooo
G@3o0?l>ooooo`003?oooold0?l0o`P00?oo8@3o0?l>ooooocP0o`3o0P00oolS0?l0o`koooooG@3o
0?l>oooooed0o`3o3_ooool000cooooo=03o0?l8003oob40o`3o3_oooolf0?l0o`@00?oo8`3o0?l>
oooooed0o`3o3_oooomM0?l0o`kooooo000<oooooc@0o`3o2000oolQ0?l0o`kooooo=P3o0?l4003o
ob<0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolb0?l0o`H0ool01000oolQ0?l0o`ko
oooo=03o0?l6003oob<0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oooolb0?l0o`H0ool0
1000oolQ0?l0o`kooooo=03o0?l6003oob<0o`3o3_oooomM0?l0o`koooooG@3o0?l>ooooo`003?oo
oolQ0?l0o`800?oo3`3o0?l60?oo00T00?oo703o0?l>oooooc@0o`3o0P3oo`04003oob<0o`3o3_oo
oolf0?l0o`800?oo9@3o0?l>oooooed0o`3o3_ooool000cooooo703o0?l7003oo`l0o`3o1P3oo`09
003ooa`0o`3o3_oooold0?l0o`/00?oo7P3o0?l>ooooocH0o`3o0P00oolU0?l0o`koooooG@3o0?l>
ooooo`003?oooolL0?l0o`L00?oo3`3o0?l60?oo00T00?oo703o0?l>oooooc@0o`3o2`00oolN0?l0
o`kooooo=P3o0?l2003oobD0o`3o3_oooomM0?l0o`kooooo000<oooooaX0o`3o2@00oolA0?l0o`80
ool02`00oolL0?l0o`kooooo7P3o0?l3003ooa<0o`3o2`00oolN0?l0o`kooooo=P3o0?l7003oob00
o`3o3_oooomM0?l0o`kooooo000<oooooaX0o`3o2@00oolA0?l0o`80ool02`00oolL0?l0o`kooooo
7P3o0?l3003ooa<0o`3o2`00oolN0?l0o`kooooo=P3o0?l7003oob00o`3o3_oooomM0?l0o`kooooo
000<ooooo`L00?oo4`3o0?l9003ooa40o`3o0P3oo`0;003ooa`0o`3o3_oooolL0?l0o`D00?oo4`3o
0?l;003ooah0o`3o3_oooolf0?l0o`L00?oo803o0?l>ooooocP0o`3o1000oolQ0?l0o`kooooo000<
ooooo`L00?oo4`3o0?l9003ooa40o`3o0P3oo`0;003ooa`0o`3o3_oooolL0?l0o`D00?oo4`3o0?l;
003ooah0o`3o3_oooolf0?l0o`L00?oo803o0?l>ooooocP0o`3o1000oolQ0?l0o`kooooo003ooooo
okWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooo
okWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooo
okWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo000<ooooo`800?oo0P3o0002
0>GUi@03003oo`1o0000O`000080iNGU0P00ool2003o0080o`3o0P00o`000`3o0000ool00?oo0002
003o0080o`3o0P1oOgl2001o0080Ogmo00<0o`3o0?l0003o00000P1o000207moO`800?oo0P00o`02
07moO`80o`0000<00?oo07moO`1oOgl00P3oo`020?l0o`80Ogmo1000o`000`00O`00o`000?l00002
0000O`8007l0103UiND2003o00<0o`3o0P00O`020?l0o`80iNGU0P3o0?l20?l00003003o003UiND0
iNGU00800?l00P3o000>ooooo`80iNGU1@3oo`04003oo`P0O`0000<0iNGU0?l0o`3o0?l00P3UiND8
07moO`D0iNGU0P00O`0207moO`@00?l01@3UiND607l00080o`3o0P3oo`0307l000H0ool01@1oOgl2
0?l0o`@0iNGU1P1o00000`3UiND0O`0007l0000407l000kooooo101o00020?oo00D00?oo1P3UiND9
0?l0o`80Ogmo1@3o0?l20>GUi@@0O`00203o0?l30>GUi@P0ool01`1o00040?oo00D0Ogmo0P3o0?l2
0>GUi@@0O`001@3oo`060?l0o`kooooo2@3o0?l4003oo`80iNGU503o0?l207l000/0o`3o1000ool4
0?oo00/0O`000P3oo`0207moO`T00?oo101o00020?oo00L0o`3o3_ooool000cooooo0P00ool20?l0
0080iNGU00<00?oo07l0001o00000P3UiND2003oo`800?l00P3o0?l2003o00030?l0003oo`00ool0
00800?l00P3o0?l207moO`8007l00P1oOgl00`3o0?l0o`000?l0000207l00080Ogmo0P00ool2003o
0080Ogmo0P3o00000`00ool0Ogmo07moO`020?oo0080o`3o0P1oOgl4003o0003001o003o0000o`00
0080001o0P00O`040>GUi@800?l00`3o0?l2001o0080o`3o0P3UiND20?l0o`80o`0000<00?l00>GU
i@3UiND00P00o`020?l000kooooo0P3UiND50?oo00@00?oo201o00000`3UiND0o`3o0?l0o`020>GU
i@P0Ogmo1@3UiND2001o0080Ogmo1000o`050>GUi@H0O`000P3o0?l20?oo00<0O`001P3oo`0507mo
O`80o`3o103UiND607l000030>GUi@1o0000O`0000@0O`003_ooool407l00080ool01@00ool60>GU
i@T0o`3o0P1oOgl50?l0o`80iNGU101o00080?l0o`<0iNGU203oo`0707l000@0ool01@1oOgl20?l0
o`80iNGU101o00050?oo00H0o`3o3_ooool90?l0o`@00?oo0P3UiNDD0?l0o`80O`002`3o0?l4003o
o`@0ool02`1o00020?oo0080Ogmo2@00ool407l00080ool01`3o0?l>ooooo`003?ooool20?oo0080
iNGU0P3oo`000`1oOgl00?oo003oo`0207l00080o`000P3UiND207l00080iNGU00<0Ogmo0?l0o`3o
0?l00P00o`020>GUi@@0001o0P1o00000`0007l0iNGU0>GUi@02001o00@0o`001000o`020000O`03
0>GUi@3o0000o`000080iNGU0P1o00020?l0o`800?l00P3oo`000`00o`00O`0007l00002001o0080
ool00P3o0?l2003o0080Ogmo0`3o00040>GUi@80001o0P1o0002003o00030?oo0000o`000?l00080
O`000P00O`0>ooooo`80o`3o1@1o0004003oo`80O`001P3UiND50?l0o`@0Ogmo0P3oo`020?l0o`D0
iNGU0P1o000207moO`@0o`3o1`1o000A0?oo00T0Ogmo0P3UiND407l000<0ool01P3o0?l>ooooo`L0
o`3o1000ool60>GUi@T0o`3o0P1oOgl50?l0o`80iNGU101o00080?l0o`<00?oo203oo`0707l000@0
ool0101oOgl5003oo`@0O`001@3oo`060?l0o`kooooo2@3o0?l4003oo`80iNGU9@3o0?l40?oo00/0
O`003@3o0?l207l00080ool02@3o0?l>ooooo`003?ooool20?oo0080iNGU0P3oo`000`1oOgl00?oo
003oo`0207l00080o`000P3UiND207l00080iNGU00<0Ogmo0?l0o`3o0?l00P00o`020>GUi@@0001o
0P1o00000`0007l0iNGU0>GUi@02001o00@0o`001000o`020000O`030>GUi@3o0000o`000080iNGU
0P1o00020?l0o`800?l00P3oo`000`00o`00O`0007l00002001o0080ool00P3o0?l2003o0080Ogmo
0`3o00040>GUi@80001o0P1o0002003o00030?oo0000o`000?l00080O`000P00O`0>ooooo`80o`3o
1@1o0004003oo`80O`001P3UiND50?l0o`@0Ogmo0P3oo`020?l0o`D0iNGU0P1o000207moO`@0o`3o
1`1o000A0?oo00T0Ogmo0P3UiND407l000<0ool01P3o0?l>ooooo`L0o`3o1000ool60>GUi@T0o`3o
0P1oOgl50?l0o`80iNGU101o00080?l0o`<00?oo203oo`0707l000@0ool0101oOgl5003oo`@0O`00
1@3oo`060?l0o`kooooo2@3o0?l4003oo`80iNGU9@3o0?l40?oo00/0O`003@3o0?l207l00080ool0
2@3o0?l>ooooo`003?ooool20?oo0080o`3o0P1o00000`00O`00Ogmo07moO`020>GUi@80o`000P00
o`020>GUi@80o`3o00<0Ogmo0?l0o`3o0?l00P3oo`020?l0o`800?oo0P00o`020?oo00030?l0o`00
07l0001o0080O`000P00o`020?oo0080o`000P3o0?l20?l0000307l00000ool00?oo0080o`000P00
O`020?oo0080iNGU0P0007l00`3oo`000?oo003oo`020?l00080ool00P0007l2001o0080001o00<0
07l007moO`1oOgl0103UiND2003oo`800?l00P1o00000`3oo`00o`3o0?l0o`020?l0o`800?l03_oo
ool60?l0o`0307l00000O`0007l000800?oo0P1o00060>GUi@<0o`3o1P1oOgl20?oo0080o`3o1`3U
iND207l00080o`3o1@1o0006003oo`@0iNGU1@3oo`060>GUi@D0Ogmo1000ool40>GUi@D00?oo1P3o
0?l>ooooo`L0o`3o1000ool60>GUiA80o`3o101o000;0?l0oa40ool00P3UiND90?l0o`@0O`000P3o
o`090?l0o`kooooo2@3o0?l6003oobD0o`3o1P00ool70?l0o`80O`003@3o0?l2003oo`/0o`3o3_oo
ool000cooooo0P3oo`020?l0o`80O`0000<007l007moO`1oOgl00P3UiND20?l000800?l00P3UiND2
0?l0o`0307moO`3o0?l0o`3o0080ool00P3o0?l2003oo`800?l00P3oo`000`3o0?l0001o0000O`02
07l000800?l00P3oo`020?l00080o`3o0P3o00000`1o00000?oo003oo`020?l0008007l00P3oo`02
0>GUi@80001o00<0ool0003oo`00ool00P3o00020?oo0080001o0P00O`020000O`03001o001oOgl0
Ogmo00@0iNGU0P00ool2003o0080O`0000<0ool00?l0o`3o0?l00P3o0?l2003o00kooooo1P3o0?l0
0`1o000007l0001o0002003oo`80O`001P3UiND30?l0o`H0Ogmo0P3oo`020?l0o`L0iNGU0P1o0002
0?l0o`D0O`001P00ool40>GUi@D0ool01P3UiND507moO`@00?oo103UiND5003oo`H0o`3o3_ooool7
0?l0o`@00?oo1P3UiNDB0?l0o`@0O`002`3o0?lA0?oo0080iNGU2@3o0?l407l00080ool02@3o0?l>
ooooo`T0o`3o1P00oolU0?l0o`H00?oo1`3o0?l207l000d0o`3o0P00ool;0?l0o`kooooo000<oooo
o`800?l00P3o0?l20?oo00030>GUi@3o0?l0o`3o0080o`3o0P1oOgl20?l000@0001o00<0o`3o07mo
O`1oOgl00P3UiND207moO`8007l00P1oOgl2003o000307l00000O`0007l00080iNGU0P1o00020?l0
o`800?l0101o00000`00o`0007l0001o0002003oo`8007l00P1o00020>GUi@800?l000<0O`000000
O`0007l00P3o00020>GUi@80001o0P1o00020000O`0307l00000ool00?oo0080001o0P3UiND2003o
0080O`000P00ool00`3o000007l0001o00020?oo0080o`3o3_ooool60?l0o`0307l00000O`0007l0
00@007l01P3UiND30?l0o`@0Ogmo1`3o0?l60>GUi@@0ool01@3o0?l6003oo`@0iNGU1@3oo`060>GU
i@80Ogmo1@3o0?l2003oo`@0iNGU1000ool70?l0o`kooooo1`3o0?l6003oo`@0ool04P3o0?l20>GU
i@80O`002`3o0?l6003oo`T0o`3o103UiND90?l0o`H00?oo2@3o0?l>oooooe00o`3o0P00ool;0?l0
o`kooooo000<ooooo`800?oo0P3o0?l2003oo`03003o003oo`00ool0008007l0103UiND20?oo0080
o`0000<0Ogmo003oo`00ool00P3o00040000O`80o`3o0P00ool00`3o0000ool00?oo00020>GUi@@0
ool00P00o`02001o0080o`3o00<007l007l0001o00001000ool20>GUi@80O`000P0007l307l00080
001o0P1o00020?l00080001o0P1oOgl00`3o0?l0O`0007l00004001o0080o`000P0007l207moO`03
0?l0o`3o0000o`0000@0ool03_ooool60?oo0003003oo`1o0000O`000080O`000P00O`020?oo0080
iNGU1@3o0?l407moO`H0o`3o00<0ool00>GUi@3UiND0103UiND40?oo00D0o`3o1P00ool20>GUi@L0
o`3o1P3UiND207moO`D0o`3o0P00ool40>GUi@@00?oo0`3o0?l40?oo00kooooo103o0?l2003oo`L0
o`3o103oo`0B0?l0o`@0iNGU0P3oo`090?l0o`@00?oo3@3o0?l207l000T0o`3o1P00ool90?l0o`ko
ooooG@3o0?l>ooooo`003?ooool2003oo`80o`3o0P00ool00`00o`00ool00?oo0002001o00@0iNGU
0P3oo`020?l0000307moO`00ool00?oo0080o`00100007l20?l0o`800?oo00<0o`000?oo003oo`00
0P3UiND40?oo00800?l00P00O`020?l0o`03001o001o0000O`0000@00?oo0P3UiND207l00080001o
0`1o00020000O`80O`000P3o00020000O`80Ogmo00<0o`3o07l0001o00001000O`020?l00080001o
0P1oOgl00`3o0?l0o`000?l000040?oo00kooooo1P3oo`000`00ool0O`0007l0000207l0008007l0
0P3oo`020>GUi@D0o`3o101oOgl60?l0o`030?oo003UiND0iNGU00@0iNGU103oo`050?l0o`H00?oo
0P3UiND70?l0o`H0iNGU0P1oOgl50?l0o`800?oo103UiND4003oo`<0o`3o103oo`0>ooooo`@0o`3o
0P00ool70?l0o`@0ool04P3o0?l40>GUi@80ool02@3o0?l4003oo`d0o`3o0P1o00090?l0o`H00?oo
2@3o0?l>oooooed0o`3o3_ooool000cooooo0P3o00020?oo0080O`0000<0Ogmo07l0001o00000P3U
iND20?l0o`80Ogmo0P3o00020>GUi@030000O`1o0000O`000080Ogmo0P3o0002003oo`800?l00P00
ool3003o00H007l00P00o`020>GUi@8007l000<0ool00?l0o`3o0?l00P0007l20?l00080Ogmo0P3U
iND30?l0o`80001o0P3o000207moO`800?l00P3o0002003oo`<007l00P3o000207l00080o`000P1o
00020?l0000307l0000007l0001o00@0ool03_ooool60?oo00D0o`3o0P00O`0607l000D0ool00P00
ool60?l0o`030?oo003UiND0iNGU00@0iNGU103oo`050?l0o`H00?oo0P3UiND90?l0o`80O`000P3o
0?l207moO`D0o`3o1P1o0002003oo`80o`3o1`3oo`0>ooooo`@0ool00P00ool70?l0o`@0ool03`3o
0?l3003oo`@0O`000P3oo`0J0?l0o`800?oo3@3o0?l5003oo`@0o`3o0P3oo`0>ooooobD0o`3o0P1o
000f0?l0o`kooooo000<ooooo`80o`000P3oo`0207l0000307moO`1o0000O`000080iNGU0P3o0?l2
07moO`80o`000P3UiND00`0007l0O`0007l0000207moO`80o`000P00ool2003o00800?oo0`00o`06
001o00800?l00P3UiND2001o00030?oo003o0?l0o`3o0080001o0P3o000207moO`80iNGU0`3o0?l2
0000O`80o`000P1oOgl2003o0080o`000P00ool3001o0080o`000P1o00020?l00080O`000P3o0000
0`1o0000001o0000O`040?oo00kooooo1P3oo`050?l0o`8007l01P1o00050?oo00800?oo1P3o0?l0
0`3oo`00iNGU0>GUi@040>GUi@@0ool01@3o0?l6003oo`80iNGU2@3o0?l207l00080o`3o0P1oOgl5
0?l0o`H0O`000P00ool20?l0o`L0ool03_ooool40?oo00800?oo1`3o0?l40?oo00l0o`3o0`00ool4
07l00080ool06P3o0?l2003oo`d0o`3o1@00ool40?l0o`80ool03_oooolU0?l0o`80O`00=P3o0?l>
ooooo`003?ooool2001o0080o`3o0P00O`030?l0o`80iNGU0P1oOgl20>GUi@80O`000P3o0?l00`1o
Ogl0ool00?oo00020?l0o`800?oo0P00o`02003oo`80iNGU00<007l00000O`0007l00P00o`0207l0
0080ool00P00o`0207moO`80iNGU00<00?l0001o0000O`00103o00020>GUi@800?oo0P00o`000`00
ool0o`3o0?l0o`0207l00080o`3o0P1oOgl20>GUi@800?oo00<00?l0001o0000O`000P0007l6001o
00800?oo00<0O`000?oo003oo`000P00o`0207moO`kooooo1P3oo`050?l0o`80iNGU1P1o00050?oo
0080o`3o1`00ool20?oo0080O`001P3oo`02001o0080ool01`3o0?l20>GUi@T0o`3o0P00ool2003o
00800?oo0`1oOgl60?l0o`@0Ogmo0P00ool50?oo00800?l03_ooool?0?l0o`@0iNGU00<0O`000>GU
i@3UiND03@3o0?l407l00080ool06P3o0?l4003ooa00o`3o0P3oo`0407l000koooooG@3o0?l>oooo
o`003?ooool2001o0080o`3o0P00O`030?l0o`80iNGU0P1oOgl20>GUi@80O`000P3o0?l00`1oOgl0
ool00?oo00020?l0o`800?oo0P00o`02003oo`80iNGU00<007l00000O`0007l00P00o`0207l00080
ool00P00o`0207moO`80iNGU00<00?l0001o0000O`00103o00020>GUi@800?oo0P00o`000`00ool0
o`3o0?l0o`0207l00080o`3o0P1oOgl20>GUi@800?oo00<00?l0001o0000O`000P0007l6001o0080
0?oo00<0O`000?oo003oo`000P00o`0207moO`kooooo1P3oo`050?l0o`80iNGU1P1o00050?oo0080
o`3o1`00ool20?oo0080O`001P3oo`02001o0080ool01`3o0?l20>GUi@T0o`3o0P00ool2003o0080
0?oo0`1oOgl60?l0o`@0Ogmo0P00ool50?oo00800?l03_ooool?0?l0o`@0iNGU00<0O`000>GUi@3U
iND03@3o0?l407l00080ool06P3o0?l4003ooa00o`3o0P3oo`0407l000koooooG@3o0?l>ooooo`00
3?ooool20?l0o`80Ogmo0P3o00000`3UiND0o`3o0?l0o`02001o0080o`000P3o0?l20>GUi@80O`00
00<00?l0003oo`00ool00P00O`020?l00080o`3o0P00o`020000O`030?l00000o`000?l00080ool0
0P1o00020?l0o`@0iNGU0P00ool30>GUi@80o`3o0P3UiND2003oo`80Ogmo0P00O`000`3UiND00?l0
003o0002003oo`800?l00P1oOgl2003oo`800?l000<0Ogmo003oo`00ool0103o0?l207moO`80o`00
0P00o`000`00O`00iNGU0>GUi@020?l000800?l03_ooool=0?l0o`80O`001@3UiND40?oo0080o`3o
0P00ool50?l0o`80ool00P1o00060?oo0080Ogmo0`3UiND60?l0o`H00?oo1@3o0?l6003oo`<0Ogmo
1P3o0?l407moO`800?oo1@3oo`0207l000kooooo3`3o0?l40>GUi@0307l00000ool00?oo00800?oo
2`3o0?l2003ooah0o`3o0P00ool=0?l0o`D00?oo0P3oo`040?l0o`koooooG@3o0?l>ooooo`003?oo
ool20?l0o`80Ogmo0P3o00000`3UiND0o`3o0?l0o`02001o0080o`000P3o0?l20>GUi@80O`0000<0
0?l0003oo`00ool00P00O`020?l00080o`3o0P00o`020000O`030?l00000o`000?l00080ool00P1o
00020?l0o`@0iNGU0P00ool30>GUi@80o`3o0P3UiND2003oo`80Ogmo0P00O`000`3UiND00?l0003o
0002003oo`800?l00P1oOgl2003oo`800?l000<0Ogmo003oo`00ool0103o0?l207moO`80o`000P00
o`000`00O`00iNGU0>GUi@020?l000800?l03_ooool=0?l0o`80O`001@3UiND40?oo0080o`3o0P00
ool50?l0o`80ool00P1o00060?oo0080Ogmo0`3UiND60?l0o`H00?oo1@3o0?l6003oo`<0Ogmo1P3o
0?l407moO`800?oo1@3oo`0207l000kooooo3`3o0?l40>GUi@0307l00000ool00?oo00800?oo2`3o
0?l2003ooah0o`3o0P00ool=0?l0o`D00?oo0P3oo`040?l0o`koooooG@3o0?l>ooooo`003?ooool2
0?l00080o`3o0P00o`000`3o00000?oo003oo`0207l000800?l00P3o0?l207moO`80iNGU00<0O`00
001o0000O`001P1oOgl207l00080o`3o00<0O`000?l0003o00000P3o00020?l0o`80001o0P3o0002
003o0080Ogmo00<00?oo001o0000O`000P00o`020000O`80iNGU0P00ool2003o0003003oo`0007l0
001o0080001o0P00ool20000O`80o`000P1o00000`0007l00?l0003o000207moO`800?l00P1oOgl2
003o0080001o0`3o000207l00080001o3_ooool407moO`80ool01`3o0?l207moO`D0iNGU0P1o0002
003oo`800?l01`3o0?l2003oo`P0o`3o0P00ool30>GUi@80o`3o1P3UiND2003oo`L0o`3o103UiND;
0?l0o`@0Ogmo0P00o`050>GUi@80o`3o3_ooool40?l0o`800?oo2@3o0?l40>GUiAX0o`3o0P00ool7
0?l0o`800?oo6P3o0?l2003oo`80ool01`3o0?l>oooooed0o`3o3_ooool000cooooo0P3o00020?l0
o`800?l000<0o`00003oo`00ool00P1o0002003o0080o`3o0P1oOgl20>GUi@0307l00000O`0007l0
00H0Ogmo0P1o00020?l0o`0307l0003o0000o`000080o`000P3o0?l20000O`80o`000P00o`0207mo
O`03003oo`00O`0007l000800?l00P0007l20>GUi@800?oo0P00o`000`00ool0001o0000O`020000
O`800?oo0P0007l20?l00080O`0000<0001o003o0000o`000P1oOgl2003o0080Ogmo0P00o`020000
O`<0o`000P1o00020000O`kooooo101oOgl20?oo00L0o`3o0P1oOgl50>GUi@80O`000P00ool2003o
00L0o`3o0P00ool80?l0o`800?oo0`3UiND20?l0o`H0iNGU0P00ool70?l0o`@0iNGU2`3o0?l407mo
O`800?l01@3UiND20?l0o`kooooo103o0?l2003oo`T0o`3o103UiNDJ0?l0o`800?oo1`3o0?l2003o
oaX0o`3o0P00ool20?oo00L0o`3o3_oooomM0?l0o`kooooo000<ooooo`800?l00P3o00020000O`03
0?oo001oOgl0Ogmo0080o`3o0P3oo`0207l000D0001o0P00ool2003o00800?oo0P1oOgl2003o0080
o`3o00<00?oo0?l0o`3o0?l00P3oo`020>GUi@80o`3o0P00ool2001o0080ool00`3o0002003o0080
iNGU0P3o0?l207moO`8007l00`3o0?l207l00080iNGU0P00ool2001o0080o`3o00<0o`00001o0000
O`00101o00020000O`80o`3o0P3o00030>GUi@80o`3o0P00o`0>ooooo`80o`3o0P3UiND20?oo0003
003oo`3o0?l0o`3o00H0o`3o1@3UiND40?l0o`@00?oo1@3o0?l4003oo`H0o`3o0P00ool20?oo00<0
0?l01P3UiND2003oo`L0o`3o103UiND40?l0o`T0iNGU0P3oo`020?l0o`030>GUi@3o0?l0o`3o00@0
o`3o3_ooool6003oobL0o`3o0P00ool00`3oo`00o`3o0?l0o`020?l0o`80iNGU0P00oolU0?l0o`ko
ooooG@3o0?l>ooooo`003?ooool2003o0080o`000P0007l00`3oo`00Ogmo07moO`020?l0o`80ool0
0P1o00050000O`800?oo0P00o`02003oo`80Ogmo0P00o`020?l0o`03003oo`3o0?l0o`3o0080ool0
0P3UiND20?l0o`800?oo0P00O`020?oo00<0o`000P00o`020>GUi@80o`3o0P1oOgl2001o00<0o`3o
0P1o00020>GUi@800?oo0P00O`020?l0o`030?l00000O`0007l000@0O`000P0007l20?l0o`80o`00
0`3UiND20?l0o`800?l03_ooool20?l0o`80iNGU0P3oo`000`00ool0o`3o0?l0o`060?l0o`D0iNGU
103o0?l4003oo`D0o`3o1000ool60?l0o`800?oo0P3oo`03003o00H0iNGU0P00ool70?l0o`@0iNGU
103o0?l90>GUi@80ool00P3o0?l00`3UiND0o`3o0?l0o`040?l0o`kooooo1P00oolW0?l0o`800?oo
00<0ool00?l0o`3o0?l00P3o0?l20>GUi@800?oo9@3o0?l>oooooed0o`3o3_ooool000cooooo0P00
o`020?l00080iNGU00<0Ogmo001o0000O`000P3o00020?l0o`80o`000P1oOgl30>GUi@80o`3o1000
07l4003oo`80O`0000<0Ogmo0000O`0007l00P3o0002003oo`80o`3o0P00o`040?l000030?l0o`00
o`000?l00080O`000P3o000207l00080Ogmo0P1o00000`3UiND0o`3o0?l0o`02003o0080001o0P3U
iND40?l00003001o003UiND0iNGU0080O`000P3UiND20?oo0080o`3o0P3oo`000`0007l00?l0003o
00020>GUi@80o`3o3_ooool2003oo`D0ool0203o0?l40>GUi@h0o`3o0P00ool80?l0o`800?oo0P3o
0?l;07moO`L0o`3o103UiND40?l0o`T0iNGU0P3oo`090?l0o`kooooo103oo`02003oobL0o`3o0P00
ool50?l0o`80Ogmo0P00oolB0?l0o`H0iNGU3@3o0?l>ooooo`@0ool0F@3o0?l>ooooo`003?ooool2
003oo`80o`000P3oo`000`1o0000Ogmo07moO`020?l0o`80ool00P1o0002003oo`80iNGU00<0o`3o
001o0000O`000P00ool20?l0o`@007l00P3o0?l00`0007l007l0001o0002003o0080o`3o0P00O`02
0>GUi@@0o`3o00<0001o07moO`1oOgl00P0007l207moO`8007l00P3o00020>GUi@030?oo003UiND0
iNGU00800?oo0P1oOgl40?l00080001o00<0O`00001o0000O`000P00o`0207l00080iNGU0P00O`02
0?l0o`030>GUi@1oOgl0Ogmo0080iNGU0P00o`0>ooooo`800?oo103oo`000`1o0000o`3o0?l0o`06
0?l0o`@0iNGU403o0?l207moO`H0o`3o103oo`030?l0o`X0Ogmo1@3o0?l20>GUi@H00?l02@3UiND5
0?l0o`H0iNGU3_ooool20?oo00@0O`009`3o0?l3003oo`@0o`3o0P1oOglW0?l0o`kooooo103oo`1I
0?l0o`kooooo000<ooooo`800?oo0P3o00020?oo000307l0001oOgl0Ogmo0080o`3o0P3oo`0207l0
00800?oo0P3UiND00`3o0?l007l0001o0002003oo`80o`3o1000O`020?l0o`030000O`00O`0007l0
00800?l00P3o0?l2001o0080iNGU103o0?l00`0007l0Ogmo07moO`020000O`80Ogmo0P00O`020?l0
0080iNGU00<0ool00>GUi@3UiND00P00ool207moO`@0o`000P0007l00`1o000007l0001o0002003o
0080O`000P3UiND2001o0080o`3o00<0iNGU07moO`1oOgl00P3UiND2003o00kooooo0P00ool40?oo
000307l0003o0?l0o`3o00H0o`3o103UiND@0?l0o`80Ogmo1P3o0?l40?oo00<0o`3o2P1oOgl50?l0
o`80iNGU1P00o`090>GUi@D0o`3o1P3UiND>ooooo`80ool0101o000W0?l0o`<00?oo103o0?l207mo
ObL0o`3o3_ooool40?oo05T0o`3o3_ooool000cooooo0P0007l4001o00030?l0o`0007l0001o0080
ool00P1o000207moO`80001o0P1oOgl00`00O`00Ogmo07moO`020?l0o`80O`000P00o`0207moO`80
O`0000<0o`3o07moO`1oOgl00P00ool407moO`800?l00P1oOgl20?oo0003003o003o0?l0o`3o0080
07l00P1oOgl20000O`80o`000P00o`000`1o0000Ogmo07moO`020?l000800?l00P3oo`04003o0003
0000O`00o`000?l00080o`3o0P00o`020?l0o`80001o0P1oOgl00`3o0000iNGU0>GUi@020>GUi@80
001o3_ooool2003o0080ool00`1o0004003oo`T0ool00P00ool=0?l0o`80Ogmo103o0?l2001o00@0
0?oo0`3o0?l607moO`80o`3o1@3UiND607moO`D00?l02P3o0?l20>GUi@D0o`3o0P3oo`0>ooooo`80
ool00P3o0?l2003oobL0o`3o0`00ool40?l0o`80Ogmo1000ool90?l0o`80Ogmo1000o`0D0?l0o`ko
ooooG@3o0?l>ooooo`003?ooool20000O`@007l000<0o`3o0000O`0007l00P3oo`0207l00080Ogmo
0P0007l207moO`03001o001oOgl0Ogmo0080o`3o0P1o0002003o0080Ogmo0P1o00000`3o0?l0Ogmo
07moO`02003oo`@0Ogmo0P00o`0207moO`80ool000<00?l00?l0o`3o0?l00P00O`0207moO`80001o
0P3o0002003o000307l0001oOgl0Ogmo0080o`000P00o`020?oo00@00?l000<0001o003o0000o`00
0P3o0?l2003o0080o`3o0P0007l207moO`030?l0003UiND0iNGU0080iNGU0P0007l>ooooo`800?l0
0P3oo`0307l000@00?oo2@3oo`02003oo`d0o`3o0P1oOgl40?l0o`8007l01000ool30?l0o`H0Ogmo
0P3o0?l50>GUi@H0Ogmo1@00o`0:0?l0o`80iNGU1@3o0?l20?oo00kooooo0P3oo`020?l0o`800?oo
9`3o0?l3003oo`@0o`3o0P1oOgl4003oo`T0o`3o0P1oOgl4003o01@0o`3o3_oooomM0?l0o`kooooo
000<ooooo`800?l00P3o0003003o0080o`000P00ool20?l00080ool00P1o00030?oo0080iNGU0P1o
Ogl20?oo00@0O`000P3oo`000`3o000007l0001o00020?l0o`80Ogmo101o0002001o00800?oo00<0
Ogmo0?l0o`3o0?l00P3o0?l20?oo00800?l00P3o0?l20?l000030>GUi@0007l0001o008007l00P1o
Ogl2001o0080001o0P1oOgl00`3o0?l00?oo003oo`020?l0o`8007l00P3o0?l2003oo`80iNGU00<0
O`000?l0o`3o0?l00P0007l20?oo00kooooo1000o`020?l0o`D00?oo2@3oo`020?l0o`P0ool01@3o
0?l607moO`H0ool00`3o0?l607moO`800?oo0P3UiND907moO`D00?l02P3o0?l20>GUi@D0o`3o0P3o
o`0>ooooo`80ool01`3o0?l4003oo`H0ool01@00oolA0?l0o`H00?oo1`3o0?l40?oo00800?oo00<0
ool007moO`1oOgl01P1oOgl4003o01@0o`3o3_oooolf0?l0o`L00?oo0P3oo`060?l0o`80O`005P3o
0?l>ooooo`003?ooool2003o0080o`000`00o`020?l000800?oo0P3o00020?oo0080O`000`3oo`02
0>GUi@80Ogmo0P3oo`0407l00080ool000<0o`00001o0000O`000P3o0?l207moO`@0O`000P00O`02
003oo`0307moO`3o0?l0o`3o0080o`3o0P3oo`02003o0080o`3o0P3o00000`3UiND0001o0000O`02
001o0080Ogmo0P00O`020000O`80Ogmo00<0o`3o003oo`00ool00P3o0?l2001o0080o`3o0P00ool2
0>GUi@0307l0003o0?l0o`3o0080001o0P3oo`0>ooooo`@00?l00P3o0?l5003oo`T0ool00P3o0?l8
0?oo00D0o`3o1P1oOgl60?oo00<0o`3o1P1oOgl2003oo`80iNGU2@1oOgl5003o00X0o`3o0P3UiND5
0?l0o`80ool03_ooool20?oo00L0o`3o1000ool60?oo00D00?oo4@3o0?l6003oo`L0o`3o103oo`02
003oo`030?oo001oOgl0Ogmo00H0Ogmo1000o`0D0?l0o`kooooo=P3o0?l7003oo`80ool01P3o0?l2
07l001H0o`3o3_ooool000cooooo0P3UiND20000O`80o`3o00<0ool0003oo`00ool00P00ool207mo
O`800?oo0P1o00030?oo008007l00P3UiND2001o0080001o0P1o000207moO`<0O`000P3UiND407mo
O`8007l00P3oo`03003oo`80iNGU0P1oOgl20?l00080O`000P00ool20000O`03001o001oOgl0Ogmo
0080o`00100007l20?l0o`8007l00`3o000207moO`80O`000P3UiND20?l000800?oo00<0001o003o
o`00ool00P1o00020?l000kooooo1000o`050?l0o`800?oo2@3oo`0207moO`l0ool0101oOgl60?oo
0003003oo`3o0?l0o`3o0080o`3o0P1o00060?oo00D0Ogmo1P00o`0307moO`X0o`3o0P3UiND50?l0
o`80ool03_ooool2003o00L0o`3o1000ool707l00080iNGU0P3o0?l20?oo00@0O`001`3o0?l207mo
O`H0ool00P00ool50?l0o`800?oo0P1o00020?oo00800?oo00<0ool007moO`1oOgl00P1oOgl4003o
0080o`3o0P1o000B0?l0o`800?oo3_ooool;0?l0o`/00?oo4@3o0?l6003oo`T0o`3o1P3oo`000`00
ool0ool00?oo0002003oo`80ool06P3o0?l>ooooo`003?ooool20>GUi@80001o0P3o0?l00`3oo`00
0?oo003oo`02003oo`80Ogmo0P00ool207l000<0ool00P00O`020>GUi@8007l00P0007l207l00080
Ogmo0`1o00020>GUi@@0Ogmo0P00O`020?oo00<00?oo0P3UiND207moO`80o`000P1o0002003oo`80
001o00<007l007moO`1oOgl00P3o00040000O`80o`3o0P00O`030?l00080Ogmo0P1o00020>GUi@80
o`000P00ool00`0007l00?oo003oo`0207l00080o`003_ooool4003o00D0o`3o0P00ool90?oo0080
Ogmo3`3oo`0407moO`H0ool000<00?oo0?l0o`3o0?l00P3o0?l207l000H0ool01@1oOgl6003o00<0
Ogmo2P3o0?l20>GUi@D0o`3o0P3oo`0>ooooo`800?l01`3o0?l4003oo`L0O`000P3UiND20?l0o`80
ool0101o00070?l0o`80Ogmo1P3oo`02003oo`D0o`3o0P00ool207l00080ool00P00ool00`3oo`00
Ogmo07moO`0207moO`@00?l00P3o0?l207l00180o`3o0P00ool>ooooo`/0o`3o2`00oolA0?l0o`H0
0?oo2@3o0?l60?oo0003003oo`3oo`00ool000800?oo0P3oo`0J0?l0o`kooooo000<ooooo`80ool0
0P1oOgl20>GUi@030?l0o`00O`0007l0008007l0100007l2003o00<0Ogmo0P3o0002003o0080ool0
0P0007l2001o0080o`0000<0O`000?oo003oo`000P00o`040000O`80ool0103o0?l00`00o`00iNGU
0>GUi@0607l00080ool00`00ool2001o0080ool01000o`040?l000<0O`00103o0002003o0080O`00
1`3o0002003oo`kooooo101oOgl30?l0o`H0ool00P1oOgl407l000<0iNGU1P3oo`0407l000D0ool0
101oOgl60?oo00D00?oo0P1o00060?oo00D0Ogmo1P00o`020?l0o`D0O`001P3UiND7003oo`80O`00
3_ooool207moO`L0o`3o2P00ool00`1o0000ool00?oo000207l000@0ool01@1o00060?oo00H0o`3o
0P00ool50?l0o`800?oo0P1o00020?oo00D00?oo0P3oo`04003o0080o`3o0P1o00090?l0o`@00?oo
1@3o0?l2003oo`kooooo2`3o0?l6003oo`/0ool01`3o0?l4003oo`l0o`3o1P3oo`05003oo`80ool0
6P3o0?l>ooooo`003?ooool20?oo0080Ogmo0P3UiND00`3o0?l007l0001o0002001o00@0001o0P00
o`0307moO`80o`000P00o`020?oo0080001o0P00O`020?l0000307l0003oo`00ool000800?l01000
07l20?oo00@0o`3o00<00?l00>GUi@3UiND01P1o00020?oo00<00?oo0P00O`020?oo00@00?l0103o
000307l000@0o`000P00o`0207l000L0o`000P00ool>ooooo`@0Ogmo0`3o0?l60?oo0080Ogmo101o
00030>GUi@H0ool0101o00050?oo00@0Ogmo1P3oo`05003oo`80O`001P3oo`0507moO`H00?l00P3o
0?l507l000H0iNGU1`00ool207l000kooooo0P1oOgl70?l0o`X00?oo00<0O`000?oo003oo`000P1o
00040?oo00D0O`001P3oo`060?l0o`800?oo1@3o0?l2003oo`80O`000P3oo`05003oo`80ool01000
o`020?l0o`80O`002@3o0?l4003oo`D0o`3o0P00ool>ooooo`/0o`3o1P00ool;0?oo00L0o`3o1000
ool?0?l0o`H0ool01@00ool20?oo01X0o`3o3_ooool000cooooo0P1o0002003o008007l000<00?l0
0?l0o`3o0?l00P3oo`02001o0080O`000P00O`020?oo0003003o003UiND0iNGU0080001o0P3o0002
003o008007l00P1o00000`3o0?l0O`0007l0000207l00080o`000P1o00020000O`80o`3o0P3oo`00
0`3o00000?oo003oo`0207moO`800?oo100007l2001o0003003oo`1o0000O`0000800?l00P00O`06
0000O`03001o000007l0001o0080O`000P00ool20>GUi@80o`000P3o0?l00`3o0000iNGU0>GUi@02
0000O`80O`003_ooool707moO`H0ool01000ool20?oo00030>GUi@1o0000O`0000P0ool00P1o0005
0?oo0080o`3o0P3oo`070?l0o`80O`000P00ool207l000H0ool000<00?oo0?oo003oo`002P00o`05
0?oo00@0O`002`00ool>ooooo`80Ogmo1`3o0?l20?oo00P00?oo00<0ool007l0001o0000103oo`02
07l00080ool00P1o0009003oo`@0o`3o0P00ool50?l0o`800?oo0P1o00020?oo00D00?oo0P3oo`04
003o0080o`3o1@1o000A003oo`kooooo2`3o0?l6003oo`D0ool01000ool207l00080ool02@00ool?
0?l0o`H0ool01@00ool20?oo01X0o`3o3_ooool000cooooo0P1o0002003o008007l000<00?l00?l0
o`3o0?l00P3oo`02001o0080O`000P00O`020?oo0003003o003UiND0iNGU0080001o0P3o0002003o
008007l00P1o00000`3o0?l0O`0007l0000207l00080o`000P1o00020000O`80o`3o0P3oo`000`3o
00000?oo003oo`0207moO`800?oo100007l2001o0003003oo`1o0000O`0000800?l00P00O`060000
O`03001o000007l0001o0080O`000P00ool20>GUi@80o`000P3o0?l00`3o0000iNGU0>GUi@020000
O`80O`003_ooool707moO`H0ool01000ool20?oo00030>GUi@1o0000O`0000P0ool00P1o00050?oo
0080o`3o0P3oo`070?l0o`80O`000P00ool207l000H0ool000<00?oo0?oo003oo`002P00o`050?oo
00@0O`002`00ool>ooooo`80Ogmo1`3o0?l20?oo00P00?oo00<0ool007l0001o0000103oo`0207l0
0080ool00P1o0009003oo`@0o`3o0P00ool50?l0o`800?oo0P1o00020?oo00D00?oo0P3oo`04003o
0080o`3o1@1o000A003oo`kooooo2`3o0?l6003oo`D0ool01000ool207l00080ool02@00ool?0?l0
o`H0ool01@00ool20?oo01X0o`3o3_ooool000cooooo0P00O`020000O`80Ogmo00<0001o003o0000
o`000P3oo`0207l000800?oo0P3oo`020?l0o`0307moO`3o0000o`000080O`000P3o0?l20>GUi@80
0?oo0P3oo`000`3o00000?l0003o00020?l0o`80o`00100007l207moO`80ool000<0Ogmo0?l0003o
00000P3o0002003oo`800?l00P1oOgl2003o00030?oo0000O`0007l00080o`000P00ool2001o00@0
o`0000<00?l00?oo003oo`00101o0002003oo`80o`000P3oo`000`00ool0O`0007l00002003oo`80
07l03_ooool707moO`H0ool01000ool20?oo00030>GUi@1o0000O`0000H0O`001@3oo`06003oo`H0
0?l00P3oo`000`00ool0o`3o0?l0o`04003oo`@0Ogmo2`3oo`020>GUi@800?l01@3oo`0?003oo`ko
oooo1`1oOgl20?oo00X00?oo00<0ool0003oo`00ool00P00ool207l000H0ool02@00ool40?l0o`80
0?oo1@3o0?l2003oo`80O`000P3oo`05003oo`@0ool0103o0?l50?oo00800?oo1P3o0?l9003oo`ko
oooo1`3o0?l:003oo`D0ool01000ool207l00080ool02@00ool?0?l0o`H0ool01@00ool20?oo0080
o`3o0P00ool50?oo00X0o`3o1`00ool>ooooo`003?ooool2003oo`80Ogmo0P0007l00`00o`00o`00
0?l000020000O`8007l00P3oo`02003o0080iNGU00<0001o0?l0003o00000P3oo`0207l00080ool0
0P3UiND20?l000030?l0o`3oo`00ool000800?oo0P00O`020000O`800?l00P3oo`02003oo`03003o
003o0?l0o`3o0080o`000P00O`0207moO`800?l00P3o00000`1oOgl0O`0007l0000207l00080ool0
0P3o0002003o0080iNGU00<00?oo003o0000o`000P3o000207l000@00?oo0P00o`000`00O`000?oo
003oo`02003o0080001o3_ooool707moO`@0ool01P00ool20?oo00<00?oo1P1o00020?oo0080O`00
00<0ool0003oo`00ool00P00ool60>GUi@800?l00P3oo`000`00ool0o`3o0?l0o`02003oo`800?l0
101oOgl;0?oo00@0o`3o1@3oo`040?l0o`/00?oo3_ooool707moO`80ool03`00ool207l000T0ool0
1P00ool20>GUi@80ool00P00ool50?l0o`/00?oo103oo`020?l0o`80ool01@00ool80?l0o`T00?oo
3_ooool607moO`030?l0o`00ool00?oo00P00?oo1@3oo`04003oo`@0ool02`00ool20?l0o`800?oo
2@3o0?l60?oo00D00?oo0P3oo`020?l0o`800?oo0`3oo`0<0?l0o`L00?oo3_ooool000cooooo0P00
ool207moO`80001o00<00?l00?l0003o00000P0007l2001o0080ool00P00o`020>GUi@030000O`3o
0000o`000080ool00P1o00020?oo0080iNGU0P3o00000`3o0?l0ool00?oo0002003oo`8007l00P00
07l2003o0080ool00P00ool00`00o`00o`3o0?l0o`020?l0008007l00P1oOgl2003o0080o`0000<0
Ogmo07l0001o00000P1o00020?oo0080o`000P00o`020>GUi@03003oo`00o`000?l00080o`000P1o
0004003oo`800?l000<007l0003oo`00ool00P00o`020000O`kooooo1`1oOgl40?oo00H00?oo0P3o
o`03003oo`H0O`000P3oo`0207l000030?oo0000ool00?oo00800?oo1P3UiND2003o0080ool000<0
0?oo0?l0o`3o0?l00P00ool2003o00@0Ogmo2`3oo`040?l0o`D0ool0103o0?l;003oo`kooooo1`1o
Ogl20?oo00l00?oo0P1o00090?oo00H00?oo0P3UiND20?oo00800?oo1@3o0?l;003oo`@0ool00P3o
0?l20?oo00D00?oo203o0?l9003oo`kooooo1P1oOgl00`3o0?l00?oo003oo`08003oo`D0ool01000
ool40?oo00/00?oo0P3o0?l2003oo`T0o`3o1P3oo`05003oo`80ool00P3o0?l2003oo`<0ool0303o
0?l7003oo`kooooo000<ooooo`80o`000P1o000207moO`03001o001o0000O`0000800?l00P3oo`04
07moO`8007l000<00?oo07moO`1oOgl00P00o`020?l00080ool00P3UiND307l00080ool01000ool2
0>GUi@80O`000P0007l2003oo`<0o`000P0007l20?l00080O`000P0007l207moO`03003oo`0007l0
001o0080001o0P00O`020?l0008007l00P3o0?l00`0007l0Ogmo07moO`02003oo`80o`3o0P00O`02
07moO`800?oo00<0o`000?oo003oo`000P3UiND207l000kooooo101oOgl50?oo00800?l01P3oo`05
003oo`X0O`0000<0ool0003oo`00ool00P00ool60>GUi@800?l00P3oo`000`00ool0iNGU0>GUi@06
0>GUi@D00?oo303oo`03003oo`H0o`3o1@00ool60>GUi@kooooo101oOgl30?oo0080o`3o3`00ool2
07l000T0ool01000ool20>GUi@80O`00103oo`050?l0o`/00?oo103oo`020?l0o`80ool01@00ool8
0?l0o`T00?oo3_ooool407moO`80ool02`3o0?l=003oo`<0ool02000ool20?oo00800?oo2@3o0?l;
003oo`@0ool01@00ool<0?l0o`L00?oo3_ooool000cooooo0P3o000207l00080Ogmo00<007l007l0
001o00000P00o`020?oo00@0Ogmo0P00O`000`00ool0Ogmo07moO`02003o0080o`000P3oo`020>GU
i@<0O`000P3oo`04003oo`80iNGU0P1o00020000O`800?oo0`3o00020000O`80o`000P1o00020000
O`80Ogmo00<00?oo0000O`0007l00P0007l2001o0080o`000P00O`020?l0o`030000O`1oOgl0Ogmo
00800?oo0P3o0?l2001o0080Ogmo0P00ool00`3o0000ool00?oo00020>GUi@80O`003_ooool407mo
O`D0ool00P00o`060?oo00D00?oo2P1o00000`3oo`000?oo003oo`02003oo`H0iNGU0P00o`020?oo
0003003oo`3UiND0iNGU00H0iNGU1@00ool<0?oo00<00?oo1P3o0?l5003oo`H0iNGU3_ooool407mo
O`<0ool00P3o0?l?003oo`80O`002@3oo`04003oo`80iNGU0P1o00040?oo00D0o`3o2`00ool40?oo
0080o`3o0P3oo`05003oo`P0o`3o2@00ool>ooooo`@0Ogmo0P3oo`0;0?l0o`d00?oo0`3oo`08003o
o`80ool00P00ool90?l0o`/00?oo103oo`05003oo``0o`3o1`00ool>ooooo`003?ooool207l00080
Ogmo0P3o00000`3oo`00001o0000O`0207moO`8007l0101oOgl20?l00003003oo`1o0000O`000080
o`3o0P3UiND20?l000@00?oo0`00o`040000O`8007l00P0007l2003oo`800?l000<00?oo0000O`00
07l0103UiND20000O`80O`000P00ool00`00O`00O`0007l000020?oo008007l00P1oOgl40?oo0003
0000O`00ool00?oo0080o`000P1o0002003o00@00?oo00<0iNGU0?oo003oo`000P00O`020?oo00ko
oooo0P3oo`0207moO`D0ool0103o0?l20?oo0080Ogmo1@00ool40>GUi@800?oo101o00000`3oo`00
0?oo003oo`02003oo`H0iNGU0P1o0003003oo`800?l0103UiND=0?oo0080Ogmo103oo`03003oo`H0
o`3o2@00ool20>GUi@kooooo0P3oo`0207moO`<0ool0203o0?l9003oo`80O`000P00ool207l000D0
ool01000ool20>GUi@80O`001@3oo`02003oo`80iNGU0P3oo`09003oo`P0ool01@00ool80?l0o`T0
0?oo3_ooool20?oo0080Ogmo0P3oo`0=0?l0oa@00?oo0P1o00040?oo00D00?oo0P3o0?l=003oo`80
o`3o1`00ool<0?l0o`L00?oo3_ooool000cooooo0P1o000207moO`80o`0000<0ool00000O`0007l0
0P1oOgl2001o00@0Ogmo0P3o00000`00ool0O`0007l000020?l0o`80iNGU0P3o0004003oo`<00?l0
100007l2001o0080001o0P00ool2003o0003003oo`0007l0001o00@0iNGU0P0007l207l000800?oo
00<007l007l0001o00000P3oo`02001o0080Ogmo103oo`000`0007l00?oo003oo`020?l00080O`00
0P00o`04003oo`030>GUi@3oo`00ool0008007l00P3oo`0>ooooo`80ool00P1oOgl50?oo00@0o`3o
0P3oo`0207moO`D00?oo103UiND2003oo`@0O`0000<0ool0003oo`00ool00P00ool60>GUi@80O`00
0`00ool2003o00@0iNGU3@3oo`0207moO`@0ool00`00ool60?l0o`T00?oo0P3UiND>ooooo`80ool0
0P1oOgl30?oo00P0o`3o2@00ool207l000800?oo0P1o00050?oo00@00?oo0P3UiND207l000D0ool0
0P00ool20>GUi@80ool02@00ool80?oo00D00?oo203o0?l9003oo`kooooo0P3oo`0207moO`80ool0
3@3o0?lD003oo`80O`00103oo`05003oo`80o`3o3@00ool20?l0o`L00?oo303o0?l7003oo`kooooo
000<ooooo`80ool00P00o`02001o0003003o001o0000O`000080o`3o1000o`0207moO`<00?l00P1o
Ogl20>GUi@80o`000P1o0005003o0080o`000P1oOgl20000O`@0o`000P1o00020?l00003003o003o
0000o`000080ool00P3o0?l20?l000@0ool000<0iNGU003o0000o`000P3o0?l20?l00080o`3o103o
o`000`00ool0001o0000O`02001o0080ool0101oOgl2003oo`03001o0000ool00?oo00800?l00P1o
000>ooooo`@00?l00P3oo`050>GUi@@0o`3o0P00o`03003oo`H0iNGU1`3oo`0607moO`80ool01P1o
Ogl3003o00@0iNGU3@3oo`0207moO`@0ool000<00?oo0>GUi@3UiND0103UiND20?oo00T00?oo0P00
o`0>ooooo`@00?l00P3oo`000`3UiND0o`3o0?l0o`080?l0o`D00?oo0P3UiND6003oo`80O`0000<0
0?oo0?oo003oo`001000ool20>GUi@80O`001`00ool20?l0o`/0ool0103UiND20?l0o`L00?oo103o
0?l=003oo`kooooo1000o`020?oo00d0o`3o4P00ool?0?l0oaH00?oo303o0?l7003oo`kooooo000<
ooooo`80ool00P00o`02001o0003003o001o0000O`000080o`3o1000o`0207moO`<00?l00P1oOgl2
0>GUi@80o`000P1o0005003o0080o`000P1oOgl20000O`@0o`000P1o00020?l00003003o003o0000
o`000080ool00P3o0?l20?l000@0ool000<0iNGU003o0000o`000P3o0?l20?l00080o`3o103oo`00
0`00ool0001o0000O`02001o0080ool0101oOgl2003oo`03001o0000ool00?oo00800?l00P1o000>
ooooo`@00?l00P3oo`050>GUi@@0o`3o0P00o`03003oo`H0iNGU1`3oo`0607moO`80ool01P1oOgl3
003o00@0iNGU3@3oo`0207moO`@0ool000<00?oo0>GUi@3UiND0103UiND20?oo00T00?oo0P00o`0>
ooooo`@00?l00P3oo`000`3UiND0o`3o0?l0o`080?l0o`D00?oo0P3UiND6003oo`80O`0000<00?oo
0?oo003oo`001000ool20>GUi@80O`001`00ool20?l0o`/0ool0103UiND20?l0o`L00?oo103o0?l=
003oo`kooooo1000o`020?oo00d0o`3o4P00ool?0?l0oaH00?oo303o0?l7003oo`kooooo000<oooo
o`80001o0P00o`020?l000030000O`3UiND0iNGU0080O`000P3o0?l4003o0080001o0`3o00020000
O`80o`000P00o`020?oo0080O`0000<0ool00?l0o`3o0?l00P1oOgl20?l000@0ool00P1oOgl20000
O`030?oo0000o`000?l00080o`3o101oOgl40000O`030?oo000007l0001o0080Ogmo0P00ool20?l0
00800?l00P1o00000`1oOgl0o`000?l000020>GUi@80001o0P3UiND20?oo0080Ogmo00<00?oo0000
O`0007l00P0007l207l000kooooo1P00o`050>GUi@@0o`3o0P00o`0207l000L0ool01P3UiND00`3o
o`00Ogmo07moO`0407moO`80ool01P1oOgl30?l0o`/0ool0203UiND20?l0o`<00?oo1P3UiND20?l0
o`@0ool01@00ool20?oo00kooooo1P00o`030>GUi@P0o`3o1@00ool=0?oo0080Ogmo1P3o0?l207mo
O`L0o`3o1P3oo`070?l0o`80O`002@00ool20>GUi@80ool03@00ool>ooooo`H0ool03@3o0?l50?oo
00/00?oo4`3o0?l20?oo00l0o`3o1@00ool20?l0o`800?oo1P3oo`07003oo`kooooo000<ooooo`80
001o0P00o`020?l000030000O`3UiND0iNGU0080O`000P3o0?l4003o0080001o0`3o00020000O`80
o`000P00o`020?oo0080O`0000<0ool00?l0o`3o0?l00P1oOgl20?l000@0ool00P1oOgl20000O`03
0?oo0000o`000?l00080o`3o101oOgl40000O`030?oo000007l0001o0080Ogmo0P00ool20?l00080
0?l00P1o00000`1oOgl0o`000?l000020>GUi@80001o0P3UiND20?oo0080Ogmo00<00?oo0000O`00
07l00P0007l207l000kooooo1P00o`050>GUi@@0o`3o0P00o`0207l000L0ool01P3UiND00`3oo`00
Ogmo07moO`0407moO`80ool01P1oOgl30?l0o`/0ool0203UiND20?l0o`<00?oo1P3UiND20?l0o`@0
ool01@00ool20?oo00kooooo1P00o`030>GUi@P0o`3o1@00ool=0?oo0080Ogmo1P3o0?l207moO`L0
o`3o1P3oo`070?l0o`80O`002@00ool20>GUi@80ool03@00ool>ooooo`H0ool03@3o0?l50?oo00/0
0?oo4`3o0?l20?oo00l0o`3o1@00ool20?l0o`800?oo1P3oo`07003oo`kooooo000<ooooo`80001o
0P1o00020?oo00<007l00P1oOgl20?l000800?l00P3o000207moO`0307l0003oo`00ool00080ool0
1000O`020>GUi@8007l000<0o`000000O`0007l00P1oOgl207l00080Ogmo100007l20?oo00030?l0
o`00O`0007l00080ool00P1oOgl207l00080o`000P00ool00`3o0?l0iNGU0>GUi@0207l00080iNGU
0P3oo`02003o0080o`3o00<007l007l0001o00000P1o00020?l0o`800?l00P1o00020?oo0003003o
o`3UiND0iNGU00@0ool03_ooool60?oo00D0iNGU0P1o00020?l0o`80ool00`3UiND60?oo00H0iNGU
0`1oOgl40>GUi@80o`3o1P3oo`000`3o0?l0Ogmo07moO`0407moO`H0ool00`3o0?l207l000@0iNGU
0P00ool207l000030?l0o`3UiND0iNGU00@0iNGU0P3o0?l50?oo00800?oo103oo`0>ooooo`T0ool0
203o0?l20?oo000307l0003oo`00ool000H0ool0103UiND307moO`X0o`3o0P3oo`050?l0o`80Ogmo
103oo`070?l0o`80O`000P3o0?l5003oo`@0iNGU0P3o0?l40?oo00L00?oo0P3oo`0>ooooo`80ool0
1`00ool80?l0o`L0ool00P00ool207l000D0ool05@3o0?l20?oo0180o`3o0P00ool20>GUi@800?oo
1P3oo`07003oo`kooooo000<ooooo`80001o0P1o00020?oo00<007l00P1oOgl20?l000800?l00P3o
000207moO`0307l0003oo`00ool00080ool01000O`020>GUi@8007l000<0o`000000O`0007l00P1o
Ogl207l00080Ogmo100007l20?oo00030?l0o`00O`0007l00080ool00P1oOgl207l00080o`000P00
ool00`3o0?l0iNGU0>GUi@0207l00080iNGU0P3oo`02003o0080o`3o00<007l007l0001o00000P1o
00020?l0o`800?l00P1o00020?oo0003003oo`3UiND0iNGU00@0ool03_ooool60?oo00D0iNGU0P1o
00020?l0o`80ool00`3UiND60?oo00H0iNGU0`1oOgl40>GUi@80o`3o1P3oo`000`3o0?l0Ogmo07mo
O`0407moO`H0ool00`3o0?l207l000@0iNGU0P00ool207l000030?l0o`3UiND0iNGU00@0iNGU0P3o
0?l50?oo00800?oo103oo`0>ooooo`T0ool0203o0?l20?oo000307l0003oo`00ool000H0ool0103U
iND307moO`X0o`3o0P3oo`050?l0o`80Ogmo103oo`070?l0o`80O`000P3o0?l5003oo`@0iNGU0P3o
0?l40?oo00L00?oo0P3oo`0>ooooo`80ool01`00ool80?l0o`L0ool00P00ool207l000D0ool05@3o
0?l20?oo0180o`3o0P00ool20>GUi@800?oo1P3oo`07003oo`kooooo000<ooooo`80o`000P3oo`02
0?l00003003o003UiND0iNGU0080iNGU0P1o0002003o0080Ogmo0P3oo`000`3UiND0O`0007l00002
0?l000800?oo1000o`0207l0000307moO`3o0?l0o`3o0080001o0P3UiND207l00080o`3o0P3oo`02
003o0003001o000007l0001o0080Ogmo0P1o00020000O`80ool00P00O`000`3UiND0o`3o0?l0o`02
0000O`80iNGU0P0007l20?oo00800?oo00<0O`000?l0o`3o0?l00P0007l20?oo00800?oo0P3o0?l2
001o00030?l0o`3o0000o`000080Ogmo0P00o`0>ooooo`L0ool01P3UiND40?oo00<0iNGU1P3oo`06
0>GUi@<0Ogmo0P3UiND60?l0o`80ool02@00o`060?oo00<0o`3o1P1oOgl20?l0o`80O`002@3UiND5
0?oo00800?oo103oo`0>ooooo`@0ool05000ool207l000H0iNGU00<0Ogmo0>GUi@3UiND00P3o0?l:
003oo`<0o`3o1000o`040?oo00L0o`3o0P1o00060?l0o`0307l0003UiND0iNGU00H0O`000P3o0?l9
003oo`kooooo0P3oo`07003ooa40o`3o103UiND307l001D0o`3o0P3oo`0B0?l0o`80iNGU4@3o0?l>
ooooo`003?ooool20?l0o`8007l00P00ool00`00o`00o`3o0?l0o`020>GUi@80ool00P3o00020>GU
i@80o`0000<007l00?l0003o00000P1oOgl2003o0080iNGU0P3oo`020?l0o`030000O`3oo`00ool0
0080o`3o0P3UiND20?l0o`8007l00P1o00020000O`03003o000007l0001o00800?l00P00ool20?l0
0080001o0P1oOgl00`00ool0iNGU0>GUi@020?l00080Ogmo0P3UiND2003o0080001o00<00?l00>GU
i@3UiND01P3UiND207moO`80001o00<0Ogmo0?l0003o00000P3o0?l207moO`kooooo1P3oo`09003o
o`80ool01@00ool207l000H0iNGU1@1oOgl20>GUi@80o`3o1P00ool9003o00L0ool00P3o0?l607mo
O`800?oo0P1o00050>GUi@80O`000P3UiND40?l0o`L00?oo3_ooool2003oo`80ool03@00ool70?l0
o`80O`00103UiND507l00080o`3o2P00ool7003o00@0ool01`3o0?l207moO`H0o`3o00<0O`000>GU
i@3UiND00P1o00060?l0o`800?oo1`3o0?l>ooooo`H0ool02@00oolA0?l0o`0307l0003o0?l0o`3o
00@0o`3o2@00ool40?l0o`800?l00P3oo`0B0?l0o`80iNGU4@3o0?l>ooooo`003?ooool20?l0o`80
07l00P00ool00`00o`00o`3o0?l0o`020>GUi@80ool00P3o00020>GUi@80o`0000<007l00?l0003o
00000P1oOgl2003o0080iNGU0P3oo`020?l0o`030000O`3oo`00ool00080o`3o0P3UiND20?l0o`80
07l00P1o00020000O`03003o000007l0001o00800?l00P00ool20?l00080001o0P1oOgl00`00ool0
iNGU0>GUi@020?l00080Ogmo0P3UiND2003o0080001o00<00?l00>GUi@3UiND01P3UiND207moO`80
001o00<0Ogmo0?l0003o00000P3o0?l207moO`kooooo1P3oo`09003oo`80ool01@00ool207l000H0
iNGU1@1oOgl20>GUi@80o`3o1P00ool9003o00L0ool00P3o0?l607moO`800?oo0P1o00050>GUi@80
O`000P3UiND40?l0o`L00?oo3_ooool2003oo`80ool03@00ool70?l0o`80O`00103UiND507l00080
o`3o2P00ool7003o00@0ool01`3o0?l207moO`H0o`3o00<0O`000>GUi@3UiND00P1o00060?l0o`80
0?oo1`3o0?l>ooooo`H0ool02@00oolA0?l0o`0307l0003o0?l0o`3o00@0o`3o2@00ool40?l0o`80
0?l00P3oo`0B0?l0o`80iNGU4@3o0?l>ooooo`003?ooool20000O`8007l00P3o00000`00ool0ool0
0?oo00020?l0o`800?oo0P3oo`02003o0080001o00<00?oo07l0001o00000P1o0004001o0080001o
0P3o00030>GUi@80o`3o0P0007l20?oo00800?oo0P1o0002003o000307l0000007l0001o00@007l0
0P0007l20?l0o`80ool00`3o0?l20000O`80o`3o0P00O`02003o0080o`0000<0O`000?l0o`3o0?l0
0P1o0002001o0080ool00P1o000207moO`030?l00000ool00?oo00800?oo0P3UiND>ooooo`l00?oo
0P3oo`050?l0o`800?oo0P3UiND607moO`D0O`000P3o0?l6003oo`L00?l0203oo`03003o00H0Ogmo
0P00ool207l000D0iNGU103o0?l20?oo0080Ogmo00<00?oo0?l0o`3o0?l0103o0?l>ooooo`800?oo
0P3oo`0=003oo`L0o`3o101o00020?l0o`D0O`000P3o0?l:003oo`D00?l0203oo`050?l0o`H00?oo
0`1o00020>GUi@P0o`3o0P00ool70?l0o`kooooo1P3oo`05003ooaD0o`3o00<0ool00?l0o`3o0?l0
103o0?l9003oo`@00?l00P3oo`0D0?l0o`80iNGU4@3o0?l>ooooo`003?ooool20000O`8007l00P3o
00000`00ool0ool00?oo00020?l0o`800?oo0P3oo`02003o0080001o00<00?oo07l0001o00000P1o
0004001o0080001o0P3o00030>GUi@80o`3o0P0007l20?oo00800?oo0P1o0002003o000307l00000
07l0001o00@007l00P0007l20?l0o`80ool00`3o0?l20000O`80o`3o0P00O`02003o0080o`0000<0
O`000?l0o`3o0?l00P1o0002001o0080ool00P1o000207moO`030?l00000ool00?oo00800?oo0P3U
iND>ooooo`l00?oo0P3oo`050?l0o`800?oo0P3UiND607moO`D0O`000P3o0?l6003oo`L00?l0203o
o`03003o00H0Ogmo0P00ool207l000D0iNGU103o0?l20?oo0080Ogmo00<00?oo0?l0o`3o0?l0103o
0?l>ooooo`800?oo0P3oo`0=003oo`L0o`3o101o00020?l0o`D0O`000P3o0?l:003oo`D00?l0203o
o`050?l0o`H00?oo0`1o00020>GUi@P0o`3o0P00ool70?l0o`kooooo1P3oo`05003ooaD0o`3o00<0
ool00?l0o`3o0?l0103o0?l9003oo`@00?l00P3oo`0D0?l0o`80iNGU4@3o0?l>ooooo`003?ooool4
003oo`80001o00<0ool00>GUi@3UiND00P3UiND407l000800?l00`3o0?l2003oo`@0O`000P00o`02
07moO`80001o00<00?oo07l0001o00000P00ool2003o0080ool00P0007l2001o0080iNGU00<0o`00
003o0000o`000P00o`020000O`80ool01000O`03003o0080001o0P3UiND2003oo`80o`000P00ool0
0`1o0000001o0000O`02001o0080o`3o0P0007l20?l0o`80o`0000<0o`3o07l0001o00000P3o0?l2
0>GUi@kooooo1P3UiND9003oo`80ool01@3o0?l2003oo`80O`00101oOgl20?l0o`L0ool01P00ool3
0>GUi@@00?l0203oo`05003o0080iNGU0P00ool50?oo0080o`3o0P00o`060?l0o`80ool000<00?oo
0?l0o`3o0?l0103o0?l>ooooo`d00?oo2`3o0?l2003oo`80O`000P3o0?l207l000030?oo001o0000
O`000080o`3o2P00ool00`3UiND00?l0003o0002003o00@0ool01@3o0?l4003o00@00?oo103oo`03
07l000P0o`3o0P00ool70?l0o`kooooo1P3oo`05003ooaD0o`3o00<0ool00?l0o`3o0?l03@3o0?l2
003oo`l0o`3o0P00ool60?oo00030>GUi@3o0?l0o`3o0140o`3o3_ooool000cooooo1000ool20000
O`030?oo003UiND0iNGU0080iNGU101o0002003o00<0o`3o0P00ool407l000800?l00P1oOgl20000
O`03003oo`1o0000O`0000800?oo0P00o`020?oo0080001o0P00O`020>GUi@030?l00000o`000?l0
00800?l00P0007l20?oo00@007l00`00o`020000O`80iNGU0P00ool20?l000800?oo00<0O`000000
O`0007l00P00O`020?l0o`80001o0P3o0?l20?l000030?l0o`1o0000O`000080o`3o0P3UiND>oooo
o`H0iNGU2@00ool20?oo00D0o`3o0P00ool207l000@0Ogmo0P3o0?l70?oo00H00?oo0`3UiND4003o
00P0ool01@00o`020>GUi@800?oo1@3oo`020?l0o`800?l01P3o0?l20?oo0003003oo`3o0?l0o`3o
00@0o`3o3_ooool=003oo`/0o`3o0P00ool207l00080o`3o0P1o00000`3oo`00O`0007l000020?l0
o`X00?oo00<0iNGU003o0000o`000P00o`040?oo00D0o`3o1000o`04003oo`@0ool00`1o00080?l0
o`800?oo1`3o0?l>ooooo`H0ool01@00oolE0?l0o`030?oo003o0?l0o`3o00d0o`3o0P00ool?0?l0
o`800?oo1P3oo`000`3UiND0o`3o0?l0o`0A0?l0o`kooooo000<ooooo`@0iNGU0P00O`000`1oOgl0
ool00?oo0002003o0080Ogmo0P3o00020>GUi@80o`000`00o`020?l000@0O`000P00ool207l00003
0?l0o`3oo`00ool00080ool00P3UiND20?l00080iNGU0P00ool20>GUi@03003oo`3UiND0iNGU0080
O`000P00O`020?l0o`800?l00P00O`000`1o0000001o0000O`020?l0o`800?l00P3oo`02003oo`80
ool00`00O`020?l000800?l00P3o0?l20?oo0080Ogmo00<0o`00001o0000O`000P00O`020?l000ko
oooo1P3UiND00`3oo`00iNGU0>GUi@040>GUi@T0o`3o0P00ool20>GUi@@0Ogmo0P3o0?l70?oo00T0
iNGU0P00o`0207moO`H0ool00P3o0?l5003o0080iNGU0P00ool50?oo0080O`000P00o`060?l0o`80
ool01`3o0?l>ooooo`@0iNGU2@00ool;0?l0o`800?oo0P1o00040?l0o`D0ool01000ool60?l0o`<0
0?oo2`3o0?l2003o0080ool01P00ool20?oo00030?l0o`1o0000O`0000P0o`3o0P00ool70?l0o`ko
oooo2`00oolF0?l0o`80ool00P00oolJ0?l0o`@00?oo1P3oo`0D0?l0o`kooooo000<ooooo`@0iNGU
0P00O`000`1oOgl0ool00?oo0002003o0080Ogmo0P3o00020>GUi@80o`000`00o`020?l000@0O`00
0P00ool207l000030?l0o`3oo`00ool00080ool00P3UiND20?l00080iNGU0P00ool20>GUi@03003o
o`3UiND0iNGU0080O`000P00O`020?l0o`800?l00P00O`000`1o0000001o0000O`020?l0o`800?l0
0P3oo`02003oo`80ool00`00O`020?l000800?l00P3o0?l20?oo0080Ogmo00<0o`00001o0000O`00
0P00O`020?l000kooooo1P3UiND00`3oo`00iNGU0>GUi@040>GUi@T0o`3o0P00ool20>GUi@@0Ogmo
0P3o0?l70?oo00T0iNGU0P00o`0207moO`H0ool00P3o0?l5003o0080iNGU0P00ool50?oo0080O`00
0P00o`060?l0o`80ool01`3o0?l>ooooo`@0iNGU2@00ool;0?l0o`800?oo0P1o00040?l0o`D0ool0
1000ool60?l0o`<00?oo2`3o0?l2003o0080ool01P00ool20?oo00030?l0o`1o0000O`0000P0o`3o
0P00ool70?l0o`kooooo2`00oolF0?l0o`80ool00P00oolJ0?l0o`@00?oo1P3oo`0D0?l0o`kooooo
000<ooooo`80001o0P3oo`0207l000<0o`000P3UiND20000O`80o`3o0P00O`020>GUi@030000O`00
o`000?l00080001o0P00o`020>GUi@800?l00P3UiND00`00o`0007l0001o0002001o0080o`000P00
07l20?oo0080Ogmo0P3o00000`00ool0iNGU0>GUi@0407moO`80001o0P3o0?l20000O`03003oo`00
o`000?l00080o`000P3UiND20?oo0080o`000P1o00000`00O`00o`000?l00002001o00@0001o0P00
O`020?oo00<0o`3o0P0007l207moO`kooooo3@3UiND20?l0o`@00?oo1@00o`060>GUi@80o`3o1@3o
o`080?l0o`<00?oo101oOgl80?l0o`D00?l00P3oo`02003oo`D0ool00P3o0?l2003o00H0o`3o0P3o
o`050?l0o`80Ogmo3_ooool40>GUi@D0ool00P3UiND=0?l0o`@00?oo1@3o0?l40?oo00X0o`3o00<0
O`000?oo003oo`002`3o0?l2003o0080ool01P00ool207l001@0o`3o3_ooool20>GUi@L00?oo6P3o
0?l2003ooaX0o`3o1P00oolH0?l0o`kooooo000<ooooo`80001o0P3oo`0207l000<0o`000P3UiND2
0000O`80o`3o0P00O`020>GUi@030000O`00o`000?l00080001o0P00o`020>GUi@800?l00P3UiND0
0`00o`0007l0001o0002001o0080o`000P0007l20?oo0080Ogmo0P3o00000`00ool0iNGU0>GUi@04
07moO`80001o0P3o0?l20000O`03003oo`00o`000?l00080o`000P3UiND20?oo0080o`000P1o0000
0`00O`00o`000?l00002001o00@0001o0P00O`020?oo00<0o`3o0P0007l207moO`kooooo3@3UiND2
0?l0o`@00?oo1@00o`060>GUi@80o`3o1@3oo`080?l0o`<00?oo101oOgl80?l0o`D00?l00P3oo`02
003oo`D0ool00P3o0?l2003o00H0o`3o0P3oo`050?l0o`80Ogmo3_ooool40>GUi@D0ool00P3UiND=
0?l0o`@00?oo1@3o0?l40?oo00X0o`3o00<0O`000?oo003oo`002`3o0?l2003o0080ool01P00ool2
07l001@0o`3o3_ooool20>GUi@L00?oo6P3o0?l2003ooaX0o`3o1P00oolH0?l0o`kooooo000<oooo
o`800?l00P3oo`020000O`030>GUi@1o0000O`000080001o0P00O`02003oo`80iNGU0P00ool00`3o
o`000?l0003o000207moO`80O`000P3UiND20?l0o`800?oo00<0ool007l0001o0000103o0?l2001o
0080o`3o103oo`000`1oOgl0o`3o0?l0o`0207moO`80001o0P00o`040>GUi@<00?l00P0007l2003o
o`800?l00P00ool2003o00030?l0o`3oo`00ool00080iNGU0P1oOgl2003o0080ool00P3o00000`1o
Ogl00?oo003oo`0207moO`80ool03_ooool40?oo00T0iNGU0P3oo`04003oo`D00?l0103UiND2003o
o`80o`3o2@3UiND40?l0o`80O`0000<0ool007moO`1oOgl01P1oOgl40?l0o`D00?l0103oo`0207l0
00D0o`3o0P3oo`0407moO`@0ool01@3o0?l207moO`kooooo103oo`050>GUi@80ool02@3o0?l2003o
o`80iNGU0P1o0002003ooa<0o`3o00<0O`000?oo003oo`003`3o0?l6003ooaH0o`3o3_ooool60?l0
o`<0iNGUE03o0?l>ooooo`003?ooool2003o0080ool00P0007l00`3UiND0O`0007l000020000O`80
07l00P00ool20>GUi@800?oo00<0ool0003o0000o`000P1oOgl207l00080iNGU0P3o0?l2003oo`03
0?oo001o0000O`0000@0o`3o0P00O`020?l0o`@0ool000<0Ogmo0?l0o`3o0?l00P1oOgl20000O`80
0?l0103UiND3003o0080001o0P00ool2003o00800?oo0P00o`000`3o0?l0ool00?oo00020>GUi@80
Ogmo0P00o`020?oo0080o`0000<0Ogmo003oo`00ool00P1oOgl20?oo00kooooo103oo`090>GUi@80
ool01000ool5003o00@0iNGU0P00ool20?l0o`T0iNGU103o0?l207l000030?oo001oOgl0Ogmo00H0
Ogmo103o0?l5003o00@0ool00P1o00050?l0o`80ool0101oOgl40?oo00D0o`3o0P1oOgl>ooooo`@0
ool01@3UiND20?oo00T0o`3o0P00ool20>GUi@80O`000P00oolC0?l0o`0307l0003oo`00ool000l0
o`3o1P00oolF0?l0o`kooooo1P3o0?l30>GUiE@0o`3o3_ooool000cooooo0P0007l20?l0o`80Ogmo
00<0001o0>GUi@3UiND00P3oo`02003o0080Ogmo0P3oo`02001o00030?l0o`3UiND0iNGU0080001o
0P3UiND20?l00080Ogmo0P00ool00`3o0?l0iNGU0>GUi@02001o0080iNGU0P00O`020000O`80O`00
0P3oo`000`1oOgl0o`000?l000020?l000800?l00P00ool20?l00080o`3o00<00?oo003o0000o`00
0P3o00020000O`80Ogmo0P3o00020000O`<007l00P3oo`020>GUi@800?l01000O`000`3UiND00?l0
003o0002003o0080o`003_ooool40?oo00L0iNGU0P3oo`060?l0o`03003oo`00o`000?l000800?l0
0P1oOgl207l000800?oo1@3o0?l60>GUi@H0o`3o00<0ool0003o0000o`002000o`020>GUi@D0o`3o
0P3UiND40?l0o`D0Ogmo0P3oo`0207moO`H0ool000<0o`3o003oo`00ool00P00ool207moO`kooooo
1P3o0?l50>GUi@800?oo1`3o0?l2003oo`80iNGU0P1o000;0?l0o`80iNGU203o0?l00`1o0000o`3o
0?l0o`0?0?l0o`80O`002`3o0?l207moO`@0ool02@3o0?l>ooooo`L0o`3o0P1o0002003oo`/0o`3o
0P00oom50?l0o`kooooo000<ooooo`80001o0P3o0?l207moO`030000O`3UiND0iNGU0080ool00P00
o`0207moO`80ool00P00O`000`3o0?l0iNGU0>GUi@020000O`80iNGU0P3o000207moO`800?oo00<0
o`3o0>GUi@3UiND00P00O`020>GUi@8007l00P0007l207l00080ool000<0Ogmo0?l0003o00000P3o
0002003o00800?oo0P3o00020?l0o`03003oo`00o`000?l00080o`000P0007l207moO`80o`000P00
07l3001o0080ool00P3UiND2003o00@007l000<0iNGU003o0000o`000P00o`020?l000kooooo103o
o`070>GUi@80ool01P3o0?l00`00ool00?l0003o0002003o0080Ogmo0P1o0002003oo`D0o`3o1P3U
iND60?l0o`030?oo0000o`000?l000P00?l00P3UiND50?l0o`80iNGU103o0?l507moO`80ool00P1o
Ogl60?oo00030?l0o`00ool00?oo00800?oo0P1oOgl>ooooo`H0o`3o1@3UiND2003oo`L0o`3o0P00
ool20>GUi@80O`002`3o0?l20>GUi@P0o`3o00<0O`000?l0o`3o0?l03`3o0?l207l000/0o`3o0P1o
Ogl40?oo00T0o`3o3_ooool70?l0o`80O`000P00ool;0?l0o`800?ooA@3o0?l>ooooo`003?ooool2
003oo`80o`000`1o00020?oo00800?l00P00O`040?l0o`800?oo00<0o`00003o0000o`000P3o0?l4
0>GUi@80O`000P3UiND00`00O`00o`3o0?l0o`02001o00800?l00P00O`0207l00080o`3o0`0007l2
003o0080001o0P1o00040>GUi@<00?l00P3o0?l20?l00080iNGU0P3o00020?l0o`8007l000<0Ogmo
001o0000O`000P3o0002003oo`80O`000P3oo`020?l0o`030?l0003o0?l0o`3o0080o`000P3o0?l>
ooooo`H0o`3o1@3UiND20?oo00L00?oo103UiND207moO`80O`00103o0?l307moO`H0iNGU0P3o0?l4
003o0003003oo`3o0?l0o`3o00@0o`3o0P00o`020>GUi@8007l01@1o00020>GUi@@0o`3o1@1oOgl:
0?oo00030?l0o`3oo`00ool00080iNGU0P3o0?l>ooooo`H0o`3o1@1o0002003oo`L0o`3o1000ool2
07l002L0o`3o0P1o00060?l0o`<0Ogmo203oo`020?l0o`L00?oo3_ooool70?l0o`80O`000P00ool;
0?l0o`800?oo=P3o0?l6003oo`T0o`3o3_ooool000cooooo0P3UiND2003o0080O`0000<0o`000?l0
o`3o0?l0103o0002001o00@00?oo00<0001o0>GUi@3UiND00P3UiND207moO`80001o0P00ool20?l0
o`03001o001oOgl0Ogmo00800?oo0P3o0?l2003oo`80o`3o1000o`000`00O`000?oo003oo`020?l0
o`80001o101oOgl20?oo0003001o001o0000O`000080ool00P3o0002001o0080o`000P3o0?l00`3U
iND0ool00?oo00020000O`80ool00P00O`02003oo`8007l000<0ool00>GUi@3UiND0103o000>oooo
o`80o`3o2`1o00060?oo00<00?oo0P3UiND40?oo00@0o`3o1@1oOgl60?l0o`D0ool0203o0?l2001o
0080Ogmo0`3UiND60?oo00800?oo1@1oOgl40>GUi@H0ool000<0o`3o0?oo003oo`000P1oOgl2001o
00kooooo1P3o0?l30>GUi@l00?oo0P1o00070?l0o`80Ogmo2P3o0?l2003oo`/0o`3o0P3oo`0707l0
00H00?oo00<0ool00?l0o`3o0?l00P3o0?l20?oo00L00?oo1P1oOgl>ooooo`L0o`3o1000ool;0?l0
o`800?oo8P3o0?l20?oo0180o`3o3`00ool>ooooo`003?ooool20>GUi@800?l00P1o00000`3o0000
o`3o0?l0o`040?l0008007l01000ool00`0007l0iNGU0>GUi@020>GUi@80Ogmo0P0007l2003oo`80
o`3o00<007l007moO`1oOgl00P00ool20?l0o`800?oo0P3o0?l4003o0003001o0000ool00?oo0080
o`3o0P0007l407moO`80ool000<007l007l0001o00000P3oo`020?l0008007l00P3o00020?l0o`03
0>GUi@3oo`00ool00080001o0P3oo`02001o00800?oo0P00O`000`3oo`00iNGU0>GUi@040?l000ko
oooo0P3o0?l;07l000H0ool00`00ool20>GUi@@0ool0103o0?l507moO`H0o`3o1@3oo`080?l0o`80
07l00P1oOgl30>GUi@H0ool00P00ool507moO`@0iNGU1P3oo`000`3o0?l0ool00?oo000207moO`80
07l03_ooool60?l0o`<0iNGU3`00ool207l000L0o`3o0P1oOgl:0?l0o`800?oo2`3o0?l20?oo00L0
O`001P00ool00`3oo`00o`3o0?l0o`020?l0o`80ool01`00ool607moO`kooooo1`3o0?l4003oo`/0
o`3o0P00oolR0?l0o`80ool04P3o0?l?003oo`kooooo000<ooooo`80iNGU0P3o0?l30?l00080Ogmo
0P3UiND407l00080ool00P1o00000`00ool007l0001o00020000O`80ool01000o`02003oo`030000
O`3o0?l0o`3o0080iNGU0P3o0?l40>GUi@8007l00P3oo`000`0007l007l0001o000207l00080o`3o
0P3o000207moO`8007l000<0o`3o0>GUi@3UiND00P00O`020?oo00800?oo0P1o00020?oo00<0001o
0P00O`020>GUi@80Ogmo100007l00`00O`00Ogmo07moO`02001o0080o`3o3_ooool607moO`0307l0
003UiND0iNGU0080ool00P00ool60?oo00<0iNGU101oOgl20?oo00@0o`3o1@1oOgl20?l0o`@0iNGU
1@3oo`080?l0o`80O`000P3oo`030>GUi@/0ool00P3o0?l60>GUi@@00?oo0`3oo`0207moO`8007l0
3_ooool607l000<0iNGU1P00ool50?oo00@00?oo0P3oo`06003oo`030?l0o`00ool00?oo00X0o`3o
0P3oo`0;0?l0o`800?oo00<0ool00>GUi@3UiND00P3UiND20?oo00H00?oo1@3o0?l20?oo00L00?oo
1P1oOgl>ooooo`L0o`3o4@00oolT0?l0o`D0ool00P00ool40?oo00/0o`3o1`00ool407moO`kooooo
000<ooooo`80iNGU0P3o0?l30?l00080Ogmo0P3UiND407l00080ool00P1o00000`00ool007l0001o
00020000O`80ool01000o`02003oo`030000O`3o0?l0o`3o0080iNGU0P3o0?l40>GUi@8007l00P3o
o`000`0007l007l0001o000207l00080o`3o0P3o000207moO`8007l000<0o`3o0>GUi@3UiND00P00
O`020?oo00800?oo0P1o00020?oo00<0001o0P00O`020>GUi@80Ogmo100007l00`00O`00Ogmo07mo
O`02001o0080o`3o3_ooool607moO`0307l0003UiND0iNGU0080ool00P00ool60?oo00<0iNGU101o
Ogl20?oo00@0o`3o1@1oOgl20?l0o`@0iNGU1@3oo`080?l0o`80O`000P3oo`030>GUi@/0ool00P3o
0?l60>GUi@@00?oo0`3oo`0207moO`8007l03_ooool607l000<0iNGU1P00ool50?oo00@00?oo0P3o
o`06003oo`030?l0o`00ool00?oo00X0o`3o0P3oo`0;0?l0o`800?oo00<0ool00>GUi@3UiND00P3U
iND20?oo00H00?oo1@3o0?l20?oo00L00?oo1P1oOgl>ooooo`L0o`3o4@00oolT0?l0o`D0ool00P00
ool40?oo00/0o`3o1`00ool407moO`kooooo000<ooooo`8007l0101oOgl00`3UiND00?oo003oo`02
003o0080ool00P00ool407l000030>GUi@00O`0007l00080Ogmo0P1o000407moO`80o`3o00<0ool0
07moO`1oOgl00P00ool207l00080001o0P3UiND207l00080001o00<00?oo001o0000O`00103o0002
0000O`80O`000`3oo`04003o0080001o0P00O`020?l0o`80ool000<00?oo0?l0o`3o0?l00P00O`02
0?l0o`8007l00P1o0002003o00030?l0003o0?l0o`3o0080Ogmo0P3o000>ooooo`L0Ogmo0P3o0?l2
0?oo0080o`3o1P3oo`030>GUi@@0Ogmo0P3oo`05003oo`80iNGU1P3oo`020>GUi@D0ool00P00ool6
0?l0o`@00?oo0`3UiND;0?oo0080o`3o1P3UiND207moO`<00?l0101oOgl20?oo00kooooo101oOgl2
07l000<0iNGU1P00ool40?l0o`030?oo003UiND0iNGU0080Ogmo0P3oo`06003oo`T0o`3o0P3UiND4
0?oo0003003oo`3o0?l0o`3o00P0o`3o1`00ool60?oo00T0o`3o1`00ool607moO`kooooo503o0?l2
003oo`80ool01P00oolN0?l0o`<0ool01000ool40?oo01H0o`3o3_ooool000cooooo0P00O`0407mo
O`030>GUi@00ool00?oo00800?l00P3oo`02003oo`@0O`0000<0iNGU001o0000O`000P1oOgl207l0
00@0Ogmo0P3o0?l00`3oo`00Ogmo07moO`02003oo`80O`000P0007l20>GUi@80O`000P0007l00`00
ool007l0001o00040?l00080001o0P1o00030?oo00@00?l00P0007l2001o0080o`3o0P3oo`000`00
ool0o`3o0?l0o`02001o0080o`3o0P00O`0207l000800?l000<0o`000?l0o`3o0?l00P1oOgl20?l0
00kooooo1`1oOgl20?l0o`80ool00P3o0?l60?oo00<0iNGU101oOgl20?oo00D00?oo0P3UiND60?oo
0080iNGU1@3oo`02003oo`H0o`3o1000ool30>GUi@/0ool00P3o0?l60>GUi@80Ogmo0`00o`0407mo
O`80ool03_ooool407moO`80O`000`3UiND6003oo`@0o`3o00<0ool00>GUi@3UiND00P1oOgl20?oo
00H00?oo2@3o0?l20>GUi@@0ool000<00?oo0?l0o`3o0?l0203o0?l7003oo`H0ool02@3o0?l7003o
o`H0Ogmo3_oooolD0?l0o`800?oo0P3oo`06003ooah0o`3o0`3oo`04003oo`@0ool05P3o0?l>oooo
o`003?ooool4001o0080Ogmo00<0o`00003o0000o`000P3o0?l207l00080o`3o0P00o`020?l000<0
001o0P00O`0207l00080Ogmo0P3o0?l207moO`030>GUi@0007l0001o0080iNGU0P3oo`020000O`80
0?l00P1o00020>GUi@03003oo`1oOgl0Ogmo0080o`3o0P1oOgl40>GUi@800?oo00<0ool0003o0000
o`000P3o000207moO`80ool00P1oOgl3003oo`800?l00P3UiND20000O`80O`000P00o`0207l00003
001o001oOgl0Ogmo00800?l00P3oo`0>ooooo`H0Ogmo0`3UiND20?oo00H0o`3o1@3UiND407moO`80
ool01@3o0?l20>GUi@H0ool01`3UiND4003oo`80Ogmo0P1o0007003oo`@0Ogmo0P3o0?l40?oo00D0
o`3o0P3UiND40?l0o`D0iNGU0P3o0?l20?oo00kooooo4`3o0?l30>GUi@80Ogmo0P3oo`06003oo`<0
o`3o1P3oo`020>GUi@l0o`3o4P00ool20?l0o`80O`001P3o0?l00`3UiND0o`3o0?l0o`040?l0o`ko
oooo503o0?l40?oo00@00?oo803o0?l5003ooa`0o`3o3_ooool000cooooo1000O`0207moO`030?l0
0000o`000?l00080o`3o0P1o00020?l0o`800?l00P3o00030000O`8007l00P1o000207moO`80o`3o
0P1oOgl00`3UiND0001o0000O`020>GUi@80ool00P0007l2003o0080O`000P3UiND00`00ool0Ogmo
07moO`020?l0o`80Ogmo103UiND2003oo`030?oo0000o`000?l00080o`000P1oOgl20?oo0080Ogmo
0`00ool2003o0080iNGU0P0007l207l000800?l00P1o00000`00O`00Ogmo07moO`02003o0080ool0
3_ooool607moO`<0iNGU0P3oo`060?l0o`D0iNGU101oOgl20?oo00D0o`3o0P3UiND60?oo00L0iNGU
1000ool207moO`80O`001`00ool407moO`80o`3o103oo`050?l0o`80iNGU103o0?l50>GUi@80o`3o
0P3oo`0>oooooa<0o`3o0`3UiND207moO`80ool01P00ool30?l0o`H0ool00P3UiND?0?l0oa800?oo
0P3o0?l207l000H0o`3o00<0iNGU0?l0o`3o0?l0103o0?l>oooooa@0o`3o103oo`04003oob00o`3o
1@00oolL0?l0o`kooooo000<ooooo`80O`000P00o`0207l000<0iNGU0P00ool20000O`800?oo0P00
07l20?oo00030000O`00o`000?l00080o`000P00o`020?oo008007l00`3o0?l2003o008007l00P3o
o`02003oo`80Ogmo0P3UiND2001o00030?l00000ool00?oo008007l00P3o00020?l0o`@0iNGU00<0
0?oo0000O`0007l00P1o000207moO`80iNGU101oOgl00`00O`00O`0007l00002003oo`80O`000P00
ool20?l00080o`3o00<0iNGU0?l0003o00000P1o0002001o00kooooo1`3o0?l20>GUi@H0o`3o103U
iND5003o0080ool01P1oOgl00`3o0?l0iNGU0>GUi@060?oo00@0iNGU1@3o0?l2003oo`@0Ogmo2@00
ool407moO`D00?oo1P3oo`0207l00080o`3o1@3UiND40?l0o`kooooo4`3o0?l5003o0080ool02`3o
0?l40?oo0080iNGU2@3o0?l8003oo`D0ool00P00ool70?l0o`800?oo4@3o0?l>oooooaP0o`3o1000
oolP0?l0o`D00?oo703o0?l>ooooo`003?ooool207l000800?l00P1o00030>GUi@800?oo0P0007l2
003oo`80001o0P3oo`000`0007l00?l0003o00020?l000800?l00P3oo`02001o00<0o`3o0P00o`02
001o0080ool00P00ool207moO`80iNGU0P00O`000`3o00000?oo003oo`02001o0080o`000P3o0?l4
0>GUi@03003oo`0007l0001o0080O`000P1oOgl20>GUi@@0Ogmo00<007l007l0001o00000P00ool2
07l000800?oo0P3o00020?l0o`030>GUi@3o0000o`000080O`000P00O`0>ooooo`L0o`3o0P3UiND6
0?l0o`@0iNGU1@00o`020?oo00H0Ogmo00<0o`3o0>GUi@3UiND01P3oo`040>GUi@D0o`3o0P00ool4
07moO`T00?oo101oOgl5003oo`H0ool00P1o00020?l0o`D0iNGU103o0?l>oooooa<0o`3o1@00o`02
0?oo00/0o`3o103oo`020>GUi@T0o`3o2000ool50?oo00800?oo1`3o0?l2003ooa40o`3o3_oooolH
0?l0o`@00?oo803o0?l5003ooa`0o`3o3_ooool000cooooo1P3o0?l00`0007l007l0001o0002003o
0080o`3o0P00ool2001o0080iNGU00<0001o0?oo003oo`000P1oOgl20?l0o`8007l00P1oOgl20000
O`030>GUi@3o0?l0o`3o00800?oo0P0007l207l00080o`000P00O`02003oo`030?l0o`0007l0001o
00@007l00P3UiND20?l0o`80o`0000<007l0003oo`00ool00P00o`02001o0080ool00P00ool20?l0
o`03003oo`00O`0007l00080ool00P1oOgl2003oo`80iNGU0P00ool00`00O`000?oo003oo`0207mo
O`80o`3o3_ooool40?l0o`D0Ogmo1P3o0?l40>GUi@D00?l0201oOgl00`3oo`00o`3o0?l0o`060?oo
00@0iNGU1@3o0?l40?oo00H0iNGU00<00?oo0?oo003oo`001P3oo`050?l0o`H0ool01P3o0?l30>GU
i@@0o`3o3_oooolA0?l0o`80iNGU1@3o0?l607moO`T0o`3o0P3oo`020>GUi@T0o`3o2000ool50?oo
00800?oo1`3o0?l2003oo`80ool03`3o0?l>ooooocH0o`3o1`00ool20?oo00800?oo703o0?l>oooo
o`003?ooool60?l0o`030000O`00O`0007l000800?l00P3o0?l2003oo`8007l00P3UiND00`0007l0
ool00?oo000207moO`80o`3o0P00O`0207moO`80001o00<0iNGU0?l0o`3o0?l00P00ool20000O`80
O`000P3o0002001o00800?oo00<0o`3o0000O`0007l01000O`020>GUi@80o`3o0P3o00000`00O`00
0?oo003oo`02003o008007l00P3oo`02003oo`80o`3o00<00?oo001o0000O`000P3oo`0207moO`80
0?oo0P3UiND2003oo`03001o0000ool00?oo0080Ogmo0P3o0?l>ooooo`@0o`3o1@1oOgl60?l0o`@0
iNGU1@00o`0807moO`030?oo003o0?l0o`3o00H0ool0103UiND50?l0o`@0ool01P3UiND00`00ool0
ool00?oo00060?oo00D0o`3o1P3oo`060?l0o`<0iNGU103o0?l>oooooa40o`3o0P3UiND50?l0o`H0
Ogmo2@3o0?l20?oo0080iNGU2@3o0?l8003oo`D0ool00P00ool70?l0o`800?oo0P3oo`0?0?l0o`ko
oooo=P3o0?l7003oo`80ool00P00oolL0?l0o`kooooo000<ooooo`800?oo0P00O`020000O`0307mo
O`00O`0007l00080001o101o000207moO`800?oo00<007l00?oo003oo`000P3o0?l207moO`80001o
0P1o00020?oo000307l0003o0000o`000080iNGU0P00o`0207l0008007l00P3UiND20000O`030?l0
0000ool00?oo0080ool00P3UiND4003oo`80iNGU00<0O`00001o0000O`001P3oo`020>GUi@80o`00
00<0o`3o001o0000O`000P1oOgl2003oo`@0o`3o0P3UiND507l0008007l03_ooool60?oo00<0Ogmo
1P3o0?l50>GUi@80o`3o2P1oOgl00`3oo`00o`3o0?l0o`020?l0o`H0ool00P3UiND50?l0o`@0ool0
1P3UiND00`00ool0ool00?oo00040?oo00800?oo1@3o0?l60?oo00H0o`3o00<0iNGU003oo`00ool0
103o0?l>ooooo`H0o`3o00<0Ogmo0?l0o`3o0?l02P3o0?l00`00ool0o`3o0?l0o`020?l0o`80Ogmo
1P3oo`000`1o0000o`3o0?l0o`040?l0o`@0ool02@3o0?l40?oo0080iNGU0P00ool50?oo00800?oo
2@3o0?l2003oo`l0o`3o3_oooolJ0?l0o`80O`006P3o0?l;003ooa`0o`3o3_ooool000cooooo0P00
07l20?oo008007l000<0O`00003o0000o`000P00o`040?l0o`800?oo0`3o0?l20?l00080iNGU0P3o
o`02001o0080O`000P00o`000`1oOgl0iNGU0>GUi@020?l00080iNGU0P3oo`020?l000800?l00P3o
0?l00`0007l0iNGU0>GUi@0207l000800?l00P1o000207moO`80001o00<0o`00003oo`00ool00P1o
0002003oo`@0Ogmo0P0007l00`3o0000Ogmo07moO`0407l0008007l00P3o0?l207moO`03001o0000
o`000?l000800?l00P1oOgl>ooooo`H0ool00`1oOgl80?l0o`800?oo00<0o`3o0>GUi@3UiND0101o
Ogl70?oo0080O`000P3o0?l60?oo00@0iNGU00<00?oo0?oo003oo`00103oo`060>GUi@03003oo`3o
o`00ool000@0ool00P00ool70?l0o`800?oo2@3o0?l207l000@0Ogmo3_ooool70?oo00`0o`3o00<0
0?oo0?l0o`3o0?l00P3o0?l207moO`80ool0101o00070?l0o`@0ool02@3o0?l20?oo00@0iNGU1@00
ool20?oo00800?oo6P3o0?l>oooooaX0o`3o101o000H0?l0o`80ool01@00ool40?oo01`0o`3o3_oo
ool000cooooo0P0007l20?oo008007l000<0O`00003o0000o`000P00o`040?l0o`800?oo0`3o0?l2
0?l00080iNGU0P3oo`02001o0080O`000P00o`000`1oOgl0iNGU0>GUi@020?l00080iNGU0P3oo`02
0?l000800?l00P3o0?l00`0007l0iNGU0>GUi@0207l000800?l00P1o000207moO`80001o00<0o`00
003oo`00ool00P1o0002003oo`@0Ogmo0P0007l00`3o0000Ogmo07moO`0407l0008007l00P3o0?l2
07moO`03001o0000o`000?l000800?l00P1oOgl>ooooo`H0ool00`1oOgl80?l0o`800?oo00<0o`3o
0>GUi@3UiND0101oOgl70?oo0080O`000P3o0?l60?oo00@0iNGU00<00?oo0?oo003oo`00103oo`06
0>GUi@03003oo`3oo`00ool000@0ool00P00ool70?l0o`800?oo2@3o0?l207l000@0Ogmo3_ooool7
0?oo00`0o`3o00<00?oo0?l0o`3o0?l00P3o0?l207moO`80ool0101o00070?l0o`@0ool02@3o0?l2
0?oo00@0iNGU1@00ool20?oo00800?oo6P3o0?l>oooooaX0o`3o101o000H0?l0o`80ool01@00ool4
0?oo01`0o`3o3_ooool000cooooo0P00O`020000O`<0iNGU0P00O`02003o0080iNGU1000ool207mo
O`030>GUi@0007l0001o0080o`000P00O`020?oo0080o`3o0P3UiND3003o0080o`3o0P0007l20?l0
00800?oo0P3oo`020?l0o`03003o003oo`00ool000800?l00P3UiND6003o00030?l0o`00O`0007l0
0080ool00P00O`020?l00080o`3o0P0007l00`3oo`00iNGU0>GUi@02003o0080Ogmo0P3o00020?oo
0080o`3o00<0001o003oo`00ool00P00O`020?oo00kooooo3@3oo`020?l0o`@0ool01@1oOgl20>GU
i@@0ool00P1o00050?l0o`X0ool000<00?l00?oo003oo`001P3UiND407moO`03003oo`3oo`00ool0
00@0ool00P00ool207moO`<0O`000P3o0?l207l0008007l01`3o0?l207moO`800?l00P1oOgl>oooo
o`H0iNGU00<00?oo0?l0o`3o0?l02P3o0?l00`00ool0o`3o0?l0o`0207moO`@0ool0101o00070?l0
o`800?oo0P1o00020?l0o`D00?oo0P3oo`040>GUi@80Ogmo1@00ool20?oo00800?oo5P3o0?l207l0
0080Ogmo3_ooool20>GUi@80ool00P00oolB0?l0o`@0ool00P1o000F0?l0o`T00?oo103oo`0L0?l0
o`kooooo000<ooooo`8007l00P0007l30>GUi@8007l00P00o`020>GUi@@00?oo0P1oOgl00`3UiND0
001o0000O`020?l0008007l00P3oo`020?l0o`80iNGU0`00o`020?l0o`80001o0P3o0002003oo`80
ool00P3o0?l00`00o`00ool00?oo0002003o0080iNGU1P00o`000`3o0?l007l0001o00020?oo0080
07l00P3o00020?l0o`80001o00<0ool00>GUi@3UiND00P00o`0207moO`80o`000P3oo`020?l0o`03
0000O`00ool00?oo008007l00P3oo`0>ooooo`d0ool00P3o0?l40?oo00D0Ogmo0P3UiND40?oo0080
O`001@3o0?l:0?oo0003003o003oo`00ool000H0iNGU101oOgl00`00ool0ool00?oo00040?oo0080
0?oo0P1oOgl307l00080o`3o0P1o0002001o00L0o`3o0P1oOgl2003o0080Ogmo3_ooool60>GUi@03
003oo`3o0?l0o`3o00X0o`3o00<00?oo0?l0o`3o0?l00P1oOgl40?oo00@0O`001`3o0?l2003oo`80
O`000P3o0?l5003oo`80ool0103UiND207moO`D00?oo0P3oo`02003ooaH0o`3o0P1o000207moO`ko
oooo0P3UiND20?oo00800?oo4P3o0?l40?oo0080O`005P3o0?l9003oo`@0ool0703o0?l>ooooo`00
3?ooool20000O`80o`000`00ool20?oo00@00?oo103oo`030000O`80Ogmo0P3UiND20?l0o`80001o
103oo`000`1oOgl0o`3o0?l0o`02003oo`80o`000P3UiND207l000800?l00P0007l00`1oOgl00?l0
003o00020000O`80o`3o0P00ool2001o0080Ogmo00<0iNGU0?oo003oo`000P0007l20>GUi@80Ogmo
0P00O`0207l000<0o`3o0P3oo`020?l0o`80o`000P00O`020000O`0307moO`00o`000?l00080Ogmo
0P1o000>ooooo`8007l00P3UiND2003oo`d0ool01@1oOgl20>GUi@@0ool00`3UiND20?l0o`@00?oo
0P1o00070?l0o`800?l01P3UiND507moO`H0ool0101oOgl307l000@0o`3o103UiND50?l0o`80Ogmo
0P00o`0207moO`kooooo1P3UiND00`00ool0ool00?oo00020?oo00H0o`3o0`3oo`0407moO`80o`3o
1P1o000@0?l0o`@0ool02`00ool20?oo00H00?oo0P1o000@0?l0o`800?l00P1oOgl>oooooc80o`3o
2`00ool40?oo01X0o`3o0P1o000>ooooo`003?ooool20000O`80o`000`00ool20?oo00@00?oo103o
o`030000O`80Ogmo0P3UiND20?l0o`80001o103oo`000`1oOgl0o`3o0?l0o`02003oo`80o`000P3U
iND207l000800?l00P0007l00`1oOgl00?l0003o00020000O`80o`3o0P00ool2001o0080Ogmo00<0
iNGU0?oo003oo`000P0007l20>GUi@80Ogmo0P00O`0207l000<0o`3o0P3oo`020?l0o`80o`000P00
O`020000O`0307moO`00o`000?l00080Ogmo0P1o000>ooooo`8007l00P3UiND2003oo`d0ool01@1o
Ogl20>GUi@@0ool00`3UiND20?l0o`@00?oo0P1o00070?l0o`800?l01P3UiND507moO`H0ool0101o
Ogl307l000@0o`3o103UiND50?l0o`80Ogmo0P00o`0207moO`kooooo1P3UiND00`00ool0ool00?oo
00020?oo00H0o`3o0`3oo`0407moO`80o`3o1P1o000@0?l0o`@0ool02`00ool20?oo00H00?oo0P1o
000@0?l0o`800?l00P1oOgl>oooooc80o`3o2`00ool40?oo01X0o`3o0P1o000>ooooo`003?ooool2
0000O`80o`000`1o0002001o0080o`000P3o0?l20000O`80O`000P1oOgl50000O`80o`000P0007l2
0?l0o`80iNGU00<0001o003oo`00ool00P1oOgl2003oo`80iNGU0P1oOgl40?l0o`03003o000007l0
001o0080ool00P00ool2003o0080001o0P1o00000`00O`00o`000?l000020000O`800?oo0P3o0?l2
0000O`80o`0000<00?oo07moO`1oOgl00P3UiND20?l0o`80iNGU0P3oo`02003o0003001o0000ool0
0?oo0080001o0P3o0?l>ooooo`@0O`001@1oOgl:0?oo00D0Ogmo0P3UiND20?l0o`80O`000P3UiND5
07moO`800?oo0P1o00020?l0o`/0ool00P00ool407moO`D0ool01P1oOgl707l000H0iNGU1@1oOgl2
003o0080Ogmo3_ooool60?l0o`03003oo`3oo`00ool000X0ool02`3o0?l207l000030?l0o`00ool0
0?oo00@00?oo2`3o0?l20?oo00H00?oo1@1o00020?l0o`80ool03@3o0?l207l000L0o`3o101o000>
ooooo`L0o`3o0P00ool40?l0o`H00?oo7`3o0?lA003ooaX0o`3o3_ooool000cooooo0P0007l20?l0
00<0O`000P00O`020?l00080o`3o0P0007l207l00080Ogmo1@0007l20?l00080001o0P3o0?l20>GU
i@030000O`00ool00?oo0080Ogmo0P00ool20>GUi@80Ogmo103o0?l00`00o`00001o0000O`020?oo
00800?oo0P00o`020000O`80O`0000<007l00?l0003o00000P0007l2003oo`80o`3o0P0007l20?l0
0003003oo`1oOgl0Ogmo0080iNGU0P3o0?l20>GUi@80ool00P00o`000`00O`000?oo003oo`020000
O`80o`3o3_ooool407l000D0Ogmo2P3oo`0507moO`80iNGU0P3o0?l207l00080iNGU1@1oOgl2003o
o`80O`000P3o0?l;0?oo00800?oo101oOgl50?oo00H0Ogmo1`1o00060>GUi@D0Ogmo0P00o`0207mo
O`kooooo1P3o0?l00`00ool0ool00?oo000:0?oo00/0o`3o0P1o00000`3o0?l00?oo003oo`04003o
o`/0o`3o0P3oo`06003oo`D0O`000P3o0?l20?oo00d0o`3o0P1o00070?l0o`@0O`003_ooool70?l0
o`800?oo103o0?l6003ooal0o`3o4@00oolJ0?l0o`kooooo000<ooooo`@0O`000P00O`000`1oOgl0
001o0000O`0407moO`80iNGU0P3oo`0207l000030?l00000O`0007l00080o`3o0P3o00020?oo0080
0?oo0P1o00000`0007l0Ogmo07moO`020000O`80o`3o0P3o000207moO`8007l00P3oo`000`1o0000
ool00?oo00020?l000@00?l00P00ool207l000030?l0o`3oo`00ool000800?l00P1oOgl20?oo0080
o`000P1oOgl307l00080o`3o0P1o0002001o0080o`3o0P3oo`000`3o0000Ogmo07moO`0207l00080
iNGU3_ooool40?l0o`T0Ogmo0P3UiND507moO`P0o`3o0P3UiND207l000D0Ogmo0P3o0?l2003oo`80
o`3o3@3oo`0507l00080o`3o1P3oo`050?l0o`@0ool00P1o0006003oo`030?oo001oOgl0Ogmo0080
Ogmo0P1o000>ooooo`H0o`3o1`1oOgl20>GUi@D00?oo2P3o0?l307l00080Ogmo1000ool60?l0o`L0
ool01P00ool507l00080o`3o0P3oo`090?l0o`80ool00P00ool;0?l0o`80O`003_ooool90?l0o`X0
0?oo403o0?l2003oo`/0o`3o0P1o000:003oo`0307l0003o0?l0o`3o01h0o`3o3_ooool000cooooo
101o0002001o000307moO`0007l0001o00@0Ogmo0P3UiND20?oo0080O`0000<0o`00001o0000O`00
0P3o0?l20?l00080ool00P00ool207l000030000O`1oOgl0Ogmo0080001o0P3o0?l20?l00080Ogmo
0P00O`020?oo000307l0003oo`00ool00080o`001000o`02003oo`80O`0000<0o`3o0?oo003oo`00
0P00o`0207moO`80ool00P3o000207moO`<0O`000P3o0?l207l0008007l00P3o0?l20?oo00030?l0
001oOgl0Ogmo0080O`000P3UiND>ooooo`@0o`3o2@1oOgl20>GUi@D0Ogmo203o0?l20>GUi@80O`00
1@1oOgl20?l0o`800?oo0P3o0?l=0?oo00D0O`000P3o0?l60?oo00D0o`3o103oo`0207l000H00?oo
00<0ool007moO`1oOgl00P1oOgl207l000kooooo1P3o0?l707moO`80iNGU1@00ool:0?l0o`<0O`00
0P1oOgl4003oo`H0o`3o1`3oo`06003oo`D0O`000P3o0?l20?oo00T0o`3o0P3oo`02003oo`/0o`3o
0P1o000>ooooo`T0o`3o2P00ool@0?l0o`800?oo2`3o0?l207l000X00?oo00<0O`000?l0o`3o0?l0
7P3o0?l>ooooo`003?ooool207moO`80o`3o0P3UiND00`3oo`0007l0001o00020?oo0080O`000P3o
000207moO`80o`0000<0ool00?l0o`3o0?l00P3UiND2001o0080iNGU0P00o`020>GUi@030?l0o`1o
Ogl0Ogmo0080ool00P1oOgl4003oo`D0O`000P1oOgl2003oo`80O`000P3oo`02003o0080iNGU00<0
07l00>GUi@3UiND0101oOgl20?l00080O`000P3o0?l00`00O`00ool00?oo00020>GUi@800?l00P1o
0002003oo`8007l000<0o`0007l0001o00000P00o`0207moO`kooooo103o0?l907moO`@0iNGU0`00
ool80?l0o`80O`000P3UiND907moO`80iNGU3@3oo`0507moO`80o`3o1P3oo`050?l0o`@0ool00P1o
0002003oo`L0o`3o0P1oOgl207l000kooooo1`3o0?l40?oo00T00?oo103o0?l8003oo`0307l0001o
Ogl0Ogmo0080o`3o1P00ool707l00080iNGU103oo`0707l000@0ool02@3o0?l20?oo00800?oo2`3o
0?l207l000kooooo1`3o0?l2003ooaL0o`3o1@00ool:0?l0o`<0O`004@00ool90?l0o`800?oo3`3o
0?l>ooooo`003?ooool207moO`80o`3o0P3UiND00`3oo`0007l0001o00020?oo0080O`000P3o0002
07moO`80o`0000<0ool00?l0o`3o0?l00P3UiND2001o0080iNGU0P00o`020>GUi@030?l0o`1oOgl0
Ogmo0080ool00P1oOgl4003oo`D0O`000P1oOgl2003oo`80O`000P3oo`02003o0080iNGU00<007l0
0>GUi@3UiND0101oOgl20?l00080O`000P3o0?l00`00O`00ool00?oo00020>GUi@800?l00P1o0002
003oo`8007l000<0o`0007l0001o00000P00o`0207moO`kooooo103o0?l907moO`@0iNGU0`00ool8
0?l0o`80O`000P3UiND907moO`80iNGU3@3oo`0507moO`80o`3o1P3oo`050?l0o`@0ool00P1o0002
003oo`L0o`3o0P1oOgl207l000kooooo1`3o0?l40?oo00T00?oo103o0?l8003oo`0307l0001oOgl0
Ogmo0080o`3o1P00ool707l00080iNGU103oo`0707l000@0ool02@3o0?l20?oo00800?oo2`3o0?l2
07l000kooooo1`3o0?l2003ooaL0o`3o1@00ool:0?l0o`<0O`004@00ool90?l0o`800?oo3`3o0?l>
ooooo`003?ooool2003oo`80001o0P3oo`030?l000800?l00P3o00020?oo00800?oo0P3UiND00`00
ool0o`000?l000040?l00080o`3o0P0007l2003o00030?l0003UiND0iNGU0080o`3o101oOgl20000
O`80iNGU0P1oOgl00`0007l0iNGU0>GUi@020?oo0080001o103oo`0207moO`030?oo0000o`000?l0
0080iNGU0P3oo`02003oo`80001o0P3o0?l00`0007l0ool00?oo00020>GUi@800?l00P3oo`02003o
00<0o`3o0P1oOgl2001o0080o`3o3_ooool60?l0o`030>GUi@3oo`00ool000@0ool01000ool90?oo
00@00?oo0`3UiND807moO`@0iNGU0`1o00020>GUi@H0Ogmo0P3UiND307moO`@0ool0103UiND50?l0
o`H0ool00P00ool70?l0o`@0Ogmo3_ooool60?l0o`D00?oo203o0?l=003oo`L0iNGU0P1oOgl20>GU
i@L0O`000P3UiND40?oo00T00?oo0P3oo`090?l0o`80O`000P00ool;0?l0o`80O`003_ooool70?l0
o`800?oo4@3o0?l7003oo`@0O`002`3o0?l407l00080ool03@00ool90?l0o`800?oo3`3o0?l>oooo
o`003?ooool2003o0080Ogmo0`00ool2001o0080o`000P3UiND207moO`@00?oo00<0ool007l0001o
00000P3oo`0207l000800?oo0P00o`0207moO`030000O`00o`000?l00080O`000P0007l20>GUi@80
o`3o0P1oOgl20?l000030000O`3o0000o`000080ool00P1oOgl20>GUi@800?l00P00O`000`0007l0
ool00?oo00020?oo0080Ogmo0P3UiND20?oo0080o`3o00<007l0003oo`00ool00P00O`020?l00080
0?l01@00ool2003o0080Ogmo0P00O`0>ooooo`800?l01`00ool20?l0o`80ool01`3o0?l4003oo`H0
ool00P3o0?l50>GUi@@0Ogmo2@3UiND607moO`80ool01@00ool40>GUi@80ool01@3o0?l80?oo00L0
o`3o101oOgl>ooooo`80o`3o2@00ool;0?l0o`X00?oo00<0o`3o0>GUi@3UiND0101o00080?l0o`<0
O`000P3UiND207moO`80ool02@00ool20?oo00T0o`3o0P3oo`02003oo`/0o`3o0P1oOgl>ooooo`L0
o`3o0P00ool?0?l0o`T00?oo101o000=0?l0o`80O`000P3UiND=003oo`T0o`3o0P00ool?0?l0o`ko
oooo000<ooooo`800?l00P1oOgl3003oo`8007l00P3o00020>GUi@80Ogmo1000ool00`3oo`00O`00
07l000020?oo0080O`000P00ool2003o0080Ogmo00<0001o003o0000o`000P1o00020000O`80iNGU
0P3o0?l207moO`80o`0000<0001o0?l0003o00000P3oo`0207moO`80iNGU0P00o`02001o00030000
O`3oo`00ool00080ool00P1oOgl20>GUi@80ool00P3o0?l00`00O`000?oo003oo`02001o0080o`00
0P00o`05003oo`800?l00P1oOgl2001o00kooooo0P00o`07003oo`80o`3o0P3oo`070?l0o`@00?oo
1P3oo`020?l0o`D0iNGU101oOgl90>GUi@H0Ogmo0P3oo`05003oo`@0iNGU0P3oo`050?l0o`P0ool0
1`3o0?l407moO`kooooo0P3o0?l9003oo`/0o`3o2P00ool00`3o0?l0iNGU0>GUi@0407l000P0o`3o
0`1o00020>GUi@80Ogmo0P3oo`09003oo`80ool02@3o0?l20?oo00800?oo2`3o0?l207moO`kooooo
1`3o0?l2003oo`l0o`3o2@00ool407l000d0o`3o0P1o00020>GUi@d00?oo2@3o0?l2003oo`l0o`3o
3_ooool000cooooo0P0007l20?oo0080iNGU00<0O`00003o0000o`000P3o0?l20000O`80Ogmo103o
0?l00`0007l00?oo003oo`020?oo0080O`000P3oo`0207l00080o`3o00<0O`000>GUi@3UiND00P00
ool207moO`80o`000P00ool2003o0080o`3o00<0ool00>GUi@3UiND00P00O`020000O`8007l00P00
07l20?oo0003003oo`0007l0001o0080iNGU0P00O`020000O`80ool00P3o00000`1oOgl0ool00?oo
00020?l0o`80ool00P1o000207moO`80o`000`00O`0207l00080Ogmo3_ooool20?oo00L00?oo103o
o`070?l0o`@00?oo1P3oo`020?l0o`D0iNGU0P1o00020>GUi@H0o`3o0`3UiND607moO`80ool01@00
ool40>GUi@l0ool01@00ool607moO`kooooo2`00ool;0?l0o`@00?oo1P3oo`000`3o0?l0iNGU0>GU
i@0407l000T0o`3o103UiND207moO`80ool02@00ool20>GUi@T0o`3o103oo`0;0?l0o`80O`003_oo
ool9003oo`l0o`3o2@00ool407l000d0o`3o0P1o00020?oo00d00?oo2@3o0?l2003oo`l0o`3o3_oo
ool000cooooo0P0007l20?oo0080iNGU00<0O`00003o0000o`000P3o0?l20000O`80Ogmo103o0?l0
0`0007l00?oo003oo`020?oo0080O`000P3oo`0207l00080o`3o00<0O`000>GUi@3UiND00P00ool2
07moO`80o`000P00ool2003o0080o`3o00<0ool00>GUi@3UiND00P00O`020000O`8007l00P0007l2
0?oo0003003oo`0007l0001o0080iNGU0P00O`020000O`80ool00P3o00000`1oOgl0ool00?oo0002
0?l0o`80ool00P1o000207moO`80o`000`00O`0207l00080Ogmo3_ooool20?oo00L00?oo103oo`07
0?l0o`@00?oo1P3oo`020?l0o`D0iNGU0P1o00020>GUi@H0o`3o0`3UiND607moO`80ool01@00ool4
0>GUi@l0ool01@00ool607moO`kooooo2`00ool;0?l0o`@00?oo1P3oo`000`3o0?l0iNGU0>GUi@04
07l000T0o`3o103UiND207moO`80ool02@00ool20>GUi@T0o`3o103oo`0;0?l0o`80O`003_ooool9
003oo`l0o`3o2@00ool407l000d0o`3o0P1o00020?oo00d00?oo2@3o0?l2003oo`l0o`3o3_ooool0
00cooooo0P00O`0407moO`030000O`1oOgl0Ogmo0080O`000P3oo`04003oo`80o`3o00<007l00000
O`0007l00P3o0?l207moO`@007l00P3oo`000`3o0?l0001o0000O`0207moO`80o`000P3UiND40?l0
o`80001o00<0ool0001o0000O`000P3UiND207l00080ool00P3o00020?l0o`03001o000007l0001o
0080o`3o1000ool2001o00<0O`000P3oo`020000O`8007l00P3oo`020000O`800?oo00<0iNGU0?l0
o`3o0?l00P3UiND2003oo`kooooo1P3oo`000`1o0000o`3o0?l0o`040?oo00L0o`3o1000ool60?oo
0080o`3o1@3UiND4003oo`H0o`3o1`3UiND40?oo00D00?oo103UiNDA0?oo0080iNGU0`3o0?l407l0
00kooooo2`00ool;0?l0o`@00?oo0P3oo`050?l0o`80iNGU101o00090?l0o`H0iNGU0P3oo`09003o
o`80iNGU3@3oo`09003oo`80o`3o0P1o000>ooooo`T00?oo3`3o0?l4003oo`80ool00P3o0?l00`3U
iND0O`0007l0000207l000d0o`3o0P3UiND20?oo00d00?oo6P3o0?l>ooooo`003?ooool2001o00@0
Ogmo00<0001o07moO`1oOgl00P1o00020?oo00@00?oo0P3o0?l00`00O`00001o0000O`020?l0o`80
Ogmo1000O`020?oo00030?l0o`0007l0001o0080Ogmo0P3o00020>GUi@@0o`3o0P0007l00`3oo`00
07l0001o00020>GUi@80O`000P3oo`020?l00080o`3o00<007l00000O`0007l00P3o0?l4003oo`80
07l00`1o00020?oo0080001o0P00O`020?oo0080001o0P00ool00`3UiND0o`3o0?l0o`020>GUi@80
0?oo3_ooool60?oo000307l0003o0?l0o`3o00@0ool01`3o0?l4003oo`H0ool00P3o0?l50>GUi@@0
0?oo1P3o0?l70>GUi@@0ool01@00ool40>GUiA40ool00P3UiND30?l0o`@0O`003_ooool;003oo`/0
o`3o1000ool20?oo00D0o`3o0P3UiND407l000T0o`3o1P3UiND20?oo00T00?oo0P3UiND=0?oo00T0
0?oo0P3o0?l207l000kooooo2@00ool?0?l0o`@00?oo0P3oo`020?l0o`030>GUi@1o0000O`000080
O`003@3o0?l20>GUi@80ool03@00oolJ0?l0o`kooooo003ooooookWooooo003ooooookWooooo003o
oooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003o
oooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo003ooooookWooooo0000
\
\>"],
  ImageRangeCache->{{{0, 439}, {439, 0}} -> {-0.100021, -0.100021, 0.0095673, \
0.0095673}, {{10.4375, 107.625}, {428.5, 331.312}} -> {-6.88859, -6.92235, \
0.540199, 0.540199}, {{117.375, 214.625}, {428.5, 331.312}} -> {-64.673, \
-6.92235, 0.540199, 0.540199}, {{224.312, 321.562}, {428.5, 331.312}} -> \
{-122.441, -6.92235, 0.540199, 0.540199}, {{331.312, 428.5}, {428.5, \
331.312}} -> {-180.225, -6.92235, 0.540199, 0.540199}, {{10.4375, 107.625}, \
{321.562, 224.312}} -> {-6.88859, -64.7067, 0.540199, 0.540199}, {{117.375, \
214.625}, {321.562, 224.312}} -> {-64.6154, -64.6491, 0.539852, 0.539852}, \
{{224.312, 321.562}, {321.562, 224.312}} -> {-122.346, -64.6491, 0.539852, \
0.539852}, {{331.312, 428.5}, {321.562, 224.312}} -> {-180.225, -64.7067, \
0.540199, 0.540199}, {{10.4375, 107.625}, {214.625, 117.375}} -> {-6.88859, \
-122.474, 0.540199, 0.540199}, {{117.375, 214.625}, {214.625, 117.375}} -> \
{-64.6154, -122.379, 0.539852, 0.539852}, {{224.312, 321.562}, {214.625, \
117.375}} -> {-122.346, -122.379, 0.539852, 0.539852}, {{331.312, 428.5}, \
{214.625, 117.375}} -> {-180.225, -122.474, 0.540199, 0.540199}, {{10.4375, \
107.625}, {107.625, 10.4375}} -> {-6.88859, -180.259, 0.540199, 0.540199}, \
{{117.375, 214.625}, {107.625, 10.4375}} -> {-64.673, -180.259, 0.540199, \
0.540199}, {{224.312, 321.562}, {107.625, 10.4375}} -> {-122.441, -180.259, \
0.540199, 0.540199}, {{331.312, 428.5}, {107.625, 10.4375}} -> {-180.225, \
-180.259, 0.540199, 0.540199}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[moore8DTD]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A random graph", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(ranGraph1 = RandomGraph[100,  .02];\)\), "\[IndentingNewLine]", 
    \(\(ShowGraph[ranGraph1];\)\)}], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95365 .52854 m
.18884 .83135 L
s
.95365 .52854 m
.05351 .41483 L
s
.95096 .55697 m
.64046 .9323 L
s
.94027 .61304 m
.85023 .78974 L
s
.94027 .61304 m
.08871 .69354 L
s
.94027 .61304 m
.07737 .66733 L
s
.94027 .61304 m
.04635 .52854 L
s
.9323 .64046 m
.04635 .52854 L
s
.9323 .64046 m
.83135 .18884 L
s
.9323 .64046 m
.94649 .41483 L
s
.92263 .66733 m
.61304 .94027 L
s
.92263 .66733 m
.52854 .95365 L
s
.92263 .66733 m
.18884 .83135 L
s
.92263 .66733 m
.04904 .44303 L
s
.92263 .66733 m
.5 .04545 L
s
.88379 .74356 m
.66733 .92263 L
s
.88379 .74356 m
.58517 .94649 L
s
.86773 .76718 m
.76718 .86773 L
s
.86773 .76718 m
.74356 .88379 L
s
.86773 .76718 m
.04635 .52854 L
s
.86773 .76718 m
.38696 .05973 L
s
.86773 .76718 m
.95365 .47146 L
s
.85023 .78974 m
.04904 .44303 L
s
.85023 .78974 m
.95455 .5 L
s
.83135 .81116 m
.52854 .04635 L
s
.81116 .83135 m
.23282 .13227 L
s
.81116 .83135 m
.95455 .5 L
s
.78974 .85023 m
.94027 .38696 L
s
.76718 .86773 m
.66733 .07737 L
s
.74356 .88379 m
.61304 .94027 L
s
.71898 .89832 m
.11621 .74356 L
s
.71898 .89832 m
.08871 .69354 L
s
.71898 .89832 m
.23282 .13227 L
s
.71898 .89832 m
.25644 .11621 L
s
.71898 .89832 m
.55697 .04904 L
s
.71898 .89832 m
.69354 .08871 L
s
.69354 .91129 m
.47146 .04635 L
s
.69354 .91129 m
.61304 .05973 L
s
.66733 .92263 m
.23282 .86773 L
s
.66733 .92263 m
.55697 .04904 L
s
.66733 .92263 m
.58517 .05351 L
s
.64046 .9323 m
.11621 .74356 L
s
.64046 .9323 m
.05973 .61304 L
s
.64046 .9323 m
.33267 .07737 L
s
.64046 .9323 m
.94649 .41483 L
s
.55697 .95096 m
.05973 .61304 L
s
.55697 .95096 m
.10168 .28102 L
s
.55697 .95096 m
.33267 .07737 L
s
.52854 .95365 m
.16865 .81116 L
s
.52854 .95365 m
.95096 .44303 L
s
.5 .95455 m
.47146 .95365 L
s
.47146 .95365 m
.10168 .28102 L
s
.47146 .95365 m
.13227 .23282 L
s
.47146 .95365 m
.74356 .11621 L
s
.44303 .95096 m
.44303 .04904 L
s
.41483 .94649 m
.38696 .94027 L
s
.38696 .94027 m
.10168 .71898 L
s
.35954 .9323 m
.05973 .61304 L
s
.35954 .9323 m
.92263 .33267 L
s
.33267 .92263 m
.05973 .38696 L
s
.33267 .92263 m
.5 .04545 L
s
.30646 .91129 m
.10168 .71898 L
s
.30646 .91129 m
.88379 .25644 L
s
.30646 .91129 m
.91129 .30646 L
s
.25644 .88379 m
.66733 .07737 L
s
.23282 .86773 m
.04635 .47146 L
s
.23282 .86773 m
.44303 .04904 L
s
.23282 .86773 m
.86773 .23282 L
s
.21026 .85023 m
.47146 .04635 L
s
.18884 .83135 m
.11621 .25644 L
s
.18884 .83135 m
.30646 .08871 L
s
.18884 .83135 m
.66733 .07737 L
s
.14977 .78974 m
.58517 .05351 L
s
.14977 .78974 m
.81116 .16865 L
s
.13227 .76718 m
.78974 .14977 L
s
.11621 .74356 m
.05351 .41483 L
s
.11621 .74356 m
.71898 .10168 L
s
.08871 .69354 m
.30646 .08871 L
s
.0677 .64046 m
.0677 .35954 L
s
.0677 .64046 m
.91129 .30646 L
s
.05973 .61304 m
.11621 .25644 L
s
.04635 .52854 m
.41483 .05351 L
s
.04635 .52854 m
.91129 .30646 L
s
.04635 .47146 m
.94649 .41483 L
s
.04904 .44303 m
.10168 .28102 L
s
.05973 .38696 m
.0677 .35954 L
s
.05973 .38696 m
.95365 .47146 L
s
.07737 .33267 m
.21026 .14977 L
s
.08871 .30646 m
.69354 .08871 L
s
.11621 .25644 m
.14977 .21026 L
s
.14977 .21026 m
.94027 .38696 L
s
.16865 .18884 m
.89832 .28102 L
s
.23282 .13227 m
.58517 .05351 L
s
.23282 .13227 m
.83135 .18884 L
s
.25644 .11621 m
.52854 .04635 L
s
.25644 .11621 m
.66733 .07737 L
s
.33267 .07737 m
.35954 .0677 L
s
.35954 .0677 m
.61304 .05973 L
s
.38696 .05973 m
.66733 .07737 L
s
.41483 .05351 m
.94027 .38696 L
s
.5 .04545 m
.61304 .05973 L
s
.5 .04545 m
.89832 .28102 L
s
.64046 .0677 m
.94027 .38696 L
s
.69354 .08871 m
.95365 .47146 L
s
.76718 .13227 m
.78974 .14977 L
s
.81116 .16865 m
.91129 .30646 L
s
.83135 .18884 m
.95096 .44303 L
s
.025 w
.95365 .52854 Mdot
.95096 .55697 Mdot
.94649 .58517 Mdot
.94027 .61304 Mdot
.9323 .64046 Mdot
.92263 .66733 Mdot
.91129 .69354 Mdot
.89832 .71898 Mdot
.88379 .74356 Mdot
.86773 .76718 Mdot
.85023 .78974 Mdot
.83135 .81116 Mdot
.81116 .83135 Mdot
.78974 .85023 Mdot
.76718 .86773 Mdot
.74356 .88379 Mdot
.71898 .89832 Mdot
.69354 .91129 Mdot
.66733 .92263 Mdot
.64046 .9323 Mdot
.61304 .94027 Mdot
.58517 .94649 Mdot
.55697 .95096 Mdot
.52854 .95365 Mdot
.5 .95455 Mdot
.47146 .95365 Mdot
.44303 .95096 Mdot
.41483 .94649 Mdot
.38696 .94027 Mdot
.35954 .9323 Mdot
.33267 .92263 Mdot
.30646 .91129 Mdot
.28102 .89832 Mdot
.25644 .88379 Mdot
.23282 .86773 Mdot
.21026 .85023 Mdot
.18884 .83135 Mdot
.16865 .81116 Mdot
.14977 .78974 Mdot
.13227 .76718 Mdot
.11621 .74356 Mdot
.10168 .71898 Mdot
.08871 .69354 Mdot
.07737 .66733 Mdot
.0677 .64046 Mdot
.05973 .61304 Mdot
.05351 .58517 Mdot
.04904 .55697 Mdot
.04635 .52854 Mdot
.04545 .5 Mdot
.04635 .47146 Mdot
.04904 .44303 Mdot
.05351 .41483 Mdot
.05973 .38696 Mdot
.0677 .35954 Mdot
.07737 .33267 Mdot
.08871 .30646 Mdot
.10168 .28102 Mdot
.11621 .25644 Mdot
.13227 .23282 Mdot
.14977 .21026 Mdot
.16865 .18884 Mdot
.18884 .16865 Mdot
.21026 .14977 Mdot
.23282 .13227 Mdot
.25644 .11621 Mdot
.28102 .10168 Mdot
.30646 .08871 Mdot
.33267 .07737 Mdot
.35954 .0677 Mdot
.38696 .05973 Mdot
.41483 .05351 Mdot
.44303 .04904 Mdot
.47146 .04635 Mdot
.5 .04545 Mdot
.52854 .04635 Mdot
.55697 .04904 Mdot
.58517 .05351 Mdot
.61304 .05973 Mdot
.64046 .0677 Mdot
.66733 .07737 Mdot
.69354 .08871 Mdot
.71898 .10168 Mdot
.74356 .11621 Mdot
.76718 .13227 Mdot
.78974 .14977 Mdot
.81116 .16865 Mdot
.83135 .18884 Mdot
.85023 .21026 Mdot
.86773 .23282 Mdot
.88379 .25644 Mdot
.89832 .28102 Mdot
.91129 .30646 Mdot
.92263 .33267 Mdot
.9323 .35954 Mdot
.94027 .38696 Mdot
.94649 .41483 Mdot
.95096 .44303 Mdot
.95365 .47146 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[14]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00Q_ooool3o`0000Gooooo0ol00005ooooo`?o0000Qoooool007kooooo0ol00004ooooo`Go
00000oooool5o`0000?ooooo1Ol00004ooooo`?o0000Ooooool007Gooooo0ol00005ooooo`Go0000
0_ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00002ooooo`Go0000
1?ooool3o`0007Oooooo001dooooo`Go00000oooool7o`000003ooooool0003o000000Go000000?o
ooooo`000?l000003Ol000000ooooooo0000o`000005o`0000;ooooo1Ol0001fooooo`00KOooool3
o`0000?ooooo1ol00002ooooo`Oo000000?oooooo`000?l000001Ol000000ooooooo0000o`00000E
o`000003ooooool0003o000000Go00000oooool3o`0006oooooo001/ooooo`Go00000_ooool7o`00
00;ooooo1ol00002ooooo`Go00000oooool5o`000003ooooool0003ooooo00Go00000_ooool?o`00
00;ooooo1Ol0001^ooooo`00I_ooool3o`0000;ooooo1ol000000ooooooo0000o`000005o`0000?o
oooo1Ol00004ooooo`Oo000000?oooooo`000?l000000_l00002ooooo`Go00001?oooolFo`0000?o
oooo0ol0001Wooooo`00IOooool5o`000003ooooool0003o000000co000000?oooooo`000?l00000
0_ooool3o`0000Cooooo1?l00005ooooo`04o`000?ooooooooooo`0000Cooooo0ol00003oooooaOo
00000_ooool5o`0006Kooooo001Tooooo`ko00000ooooolXo`0000;ooooo0_l00005ooooo`?o0000
0oooool7o`000003ooooool0003o000000Go0000IOooool005kooooo0ol00003oooooaGo00001?oo
ool5o`0000?ooooo00?o0000ooooooooool04?l000000ooooooo0000o`000005ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooool000001Oooool5o`0000;ooooo1ol00003ooooo`?o0000Gooo
ool005gooooo1Ol000000ooooooo0000o`000006o`0000;ooooo0ol000001?oooooo0000ooooool0
0006ooooo`Oo00001?ooool01Ol0003ooooooooooooooooo000000Cooooo1?l000000ooooooo0000
ooooo`03oooooa3o00001_ooool3o`0000?ooooo1ol00002ooooo`Go0000G_ooool005cooooo1ol0
0002ooooo`Go00001oooool00ol0003oooooo`000002ooooo`Co00000oooool01Ol0003oooooo`00
0?oooooo000000?ooooo00Co0000oooooooooooooooo1_l00005ooooo`03o`000?oooooooooo00?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo4_l00002
ooooo`Oo0000GOooool005Kooooo0ol00003ooooo`Oo00000oooool3o`0000Wooooo1?l00007oooo
o`03o`000?oooooo000000;ooooo2?l00003ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooooooool3o`0000;ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo3Ol00002ooooo`?o0000F?ooool0
05Gooooo1Ol00002ooooo`Oo00002oooool4o`000003ooooool0003ooooo00Sooooo00?o0000oooo
ool000001Ol00004ooooo`05o`000?ooooooooooooooool000001oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00;ooooo0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo
oooo00Cooooo3?l000000ooooooo0000o`000005o`000003ooooool0003o000000?o0000Eoooool0
05Cooooo1ol00002ooooo`Go00002?ooool4o`0000;ooooo00Co0000oooooooooooo00001oooool5
o`0000Cooooo0ol00003ooooo`04o`000?ooooooooooo`0000Sooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`02ooooo`oo000000?oooooo`000?ooool02Oooool00ol0003oooooo`00
0004o`000003ooooool0003o000000Go0000E_ooool005Cooooo1ol00003ooooo`?o00001Oooool4
o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`Co000000Goooooo`000?oooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00009oooo
o`03o`000?oooooo000000_o00000oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Wooooo1Ol00002ooooo`Oo0000E_ooool004oooooo0ol00002ooooo`Oo00001Oooool00ol0003o
ooooo`000003o`0000Wooooo00?o0000ooooooooool01Ol00005ooooo`04o`000?ooooooooooo`00
00?ooooo00?o0000ooooooooool00_ooool3o`0000;ooooo00?o0000ooooool000002Ol00003oooo
o`04o`000?ooooooooooooooo`;o00000oooool2o`000003ooooool0003ooooo00Kooooo00?o0000
ooooooooool02_ooool00ol0003oooooo`000003ooooo`Oo00000oooool3o`00053ooooo001>oooo
o`Go00000_ooool5o`0000Cooooo1?l0000<ooooo`Co00000oooool00ol0003oooooooooo`04oooo
o`04o`000?ooooooooooo`0000?ooooo3?l00009ooooo`05o`000?ooooooooooooooool000000ooo
ool2o`0000Gooooo0_l00007ooooo`03o`000?oooooooooo00Wooooo0ol000001Ooooooo0000o`00
0?l0003ooooo00Go00000oooool5o`0004oooooo001=ooooo`Oo00000_ooool3o`0000;ooooo0ol0
0002ooooo`;o00001oooool5o`0000Sooooo00?o0000ooooooooool02ol00007ooooo`;o00003?oo
ool01Ol0003ooooooooooooooooo000000;ooooo0ol00005ooooo`;o00001_ooool00ol0003ooooo
ooooo`09ooooo`Go00001?ooool3o`000003ooooool0003ooooo00Oo0000C_ooool004gooooo1ol0
0002ooooo`Go00001_ooool00ol0003oooooooooo`05o`0000?ooooo00?o0000ooooooooool02_l0
0002ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`06ooooo`Co0000
2oooool01ol0003oooooooooool0003oooooo`000?ooool00_l00005ooooo`04o`000?oooooo0000
o`0000Cooooo00?o0000ooooooooool01oooool3o`0000?ooooo00?o0000ooooooooool01?ooool0
1?l0003oooooo`000?ooool7o`0004kooooo0018ooooo`?o00000_ooool:o`0000Wooooo3ol00008
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00008ooooo`;o0000
0_ooool00ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool000000_ooool2o`0000Go
oooo00Co0000oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo00Gooooo1?l00004oooo
o`;o00001Oooool00ol0003oooooooooo`08o`0000;ooooo0ol00019ooooo`00Aoooool5o`0000;o
oooo1Ol00002ooooo`;o00000_ooool;o`0000Oooooo00?o0000ooooooooool02_ooool01?l0003o
ooooooooool00003ooooo`03o`000?oooooo000000[ooooo00Co0000oooooooooooooooo0_l00009
ooooo`04o`000?ooooooooooo`0000;ooooo0_l00005ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooool000001?ooool3o`0000;ooooo0_l00004ooooo`03o`000?oooooo000000Cooooo00Co
0000oooooooooooooooo1_l00002ooooo`Go0000B?ooool004Kooooo5Ol00006ooooo`;o00001ooo
ool00ol0003oooooooooo`0<ooooo`;o00001?ooool00ol0003oooooo`000009ooooo`;o00001Ooo
ool00ol0003oooooooooo`07ooooo`?o00000_ooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool2o`000004ooooool0003o0000o`0000Cooooo0_l00005ooooo`04o`000?ooooooooooo`0000?o
oooo00Co0000oooooooooooooooo1Ol00002ooooo`Oo0000Aoooool004Kooooo2?l00004ooooo`Go
00002oooool2o`0000Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`02ooooo`03
o`000?oooooo000000Wooooo0_l00006ooooo`;o00001oooool2o`000006ooooool0003oooooo`00
0?oooooo00001Oooool00ol0003oooooooooo`03ooooo`Co00001Oooool00ol0003oooooo`000005
ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooooooool2o`0000Gooooo
1ol00017ooooo`00A_ooool<o`0000;ooooo00?o0000ooooool000002oooool2o`0000Gooooo00?o
0000ooooooooool03_ooool00ol0003oooooooooo`02ooooo`;o00002_ooool2o`0000Sooooo00?o
0000ooooooooool01?ooool2o`000006ooooool0003oooooo`000?oooooo00001Oooool00ol0003o
ooooooooo`03o`0000;ooooo00Co0000ooooool0003o00000oooool2o`0000Kooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`000002ooooo`04o`000?ooooooooooo`0000Cooooo1ol00017
ooooo`00@Oooool3o`0000?ooooo1_l00007ooooo`03o`000?oooooo000000_ooooo0_l00004oooo
o`03o`000?oooooooooo00oooooo0_l00003ooooo`;o00002_ooool00ol0003oooooo`000008oooo
o`;o00000oooool01ol0003oooooooooool0003oooooo`000?ooool00_l00005ooooo`?o00001Ooo
ool02Ol0003ooooooooooooooooo0000ooooool0003oooooo`000007ooooo`09o`000?oooooooooo
ooooool0003oooooo`000?oooooo000000?ooooo00?o0000ooooooooool00oooool5o`0000?ooooo
0ol00012ooooo`00@?ooool;o`0000Sooooo00?o0000ooooool000003?ooool00ol0003oooooo`00
0002ooooo`03o`000?oooooooooo00oooooo00Co0000oooooooooooo00000_ooool2o`0000[ooooo
00?o0000ooooool000002_ooool2o`000008ooooool0003oooooooooool0003oooooo`000?ooool2
o`0000?ooooo0ol00007ooooo`03o`000?oooooooooo00;ooooo0ol00008ooooo`03o`000?oooooo
oooo00;ooooo0ol00005ooooo`03o`000?oooooooooo00?ooooo0ol00003ooooo`Go0000@Oooool0
03oooooo2Ol00003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000003Oooool2o`00
0003ooooool0003ooooo017ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool00_ooool2o`000003ooooool0003o
000000?o00000_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00Co0000
ooooool0003o00001oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`Oo0000@?ooool003oooooo1ol00002
ooooo`[o000000Coooooo`000?oooooo00003Oooool00ol0003oooooo`00000Cooooo`07o`000?oo
ooooooooooooool0003oooooo`00000:ooooo`04o`000?ooooooooooo`0000cooooo0ol00003oooo
o`Go00001?ooool00ol0003oooooooooo`05ooooo`07o`000?ooooooooooooooool0003oooooo`00
0002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`Oo0000@?ooool003oooooo1ol00006
ooooo`03o`000?oooooooooo00?ooooo2_l00007ooooo`03o`000?oooooo000001?ooooo00?o0000
ooooooooool00_ooool2o`0000[ooooo00Go0000ooooooooooooooooo`00000:ooooo`04o`000?oo
ooooooooooooo`Ko000000?oooooo`000?ooool00oooool00ol0003oooooooooo`05ooooo`07o`00
0?ooooooooooooooool0003oooooo`000003ooooo`;o00001?ooool02Ol0003ooooooooooooooooo
0000ooooool0003oooooo`000006ooooo`04o`000?ooooooooooo`0000Gooooo1ol00010ooooo`00
>oooool3o`0000;ooooo1Ol00008ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00001_ooool:o`0001;ooooo00?o0000ooooooooool00oooool2o`0000[ooooo00Go0000oooooooo
ooooooooo`000009ooooo`03o`000?oooooooooo00Go000000Goooooo`000?l0003oooooo`000004
ooooo`03o`000?oooooooooo00Gooooo00Ko0000oooooooooooo0000ooooool00006ooooo`;o0000
0_ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooo
ool000001_ooool5o`000003ooooool0003o000000;o0000??ooool003[ooooo1Ol00002ooooo`Go
00001oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool02oooool2o`00
0003ooooool0003o000000Wo00001oooool00ol0003oooooooooo`03ooooo`;o00002_ooool01Ol0
003ooooooooooooooooo000000Wooooo0ol00005ooooo`05o`000?oooooo0000ooooool000001?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01_oo
ool01Ol0003oooooo`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000005ooooo`03
o`000?oooooo000000Oooooo0ol00002ooooo`Go0000>oooool003Wooooo1ol00005ooooo`;o0000
1oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00_ooooo00Go0000ooooool0
003oooooo`00000:ooooo`[o00000oooool3o`0000Wooooo00Go0000ooooooooooooooooo`000007
ooooo`?o00002?ooool4o`0000Gooooo00?o0000ooooooooool01?ooool01Ol0003oooooo`000?oo
oooo000000[ooooo0ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0:ooooo`Oo0000>_ooool003Wooooo1ol00006ooooo`;o00001oooool0
0ol0003oooooo`000002ooooo`03o`000?oooooooooo00_ooooo00Go0000ooooool0003oooooo`00
000Aooooo`03o`000?oooooooooo00[o00001?ooool00ol0003oooooooooo`03ooooo`04o`000?oo
ooooooooooooo`?o000000?oooooo`000?ooool02_ooool3o`0000Cooooo00?o0000ooooooooool0
1Oooool01?l0003oooooooooool0000;ooooo`?o00001?ooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo00;ooooo0_l0000;ooooo`Oo0000>_ooool003Wooooo1ol00006ooooo`;o0000
1oooool2o`0000?ooooo00?o0000ooooooooool02_ooool01ol0003ooooooooooooooooo0000oooo
ool000003oooool00ol0003oooooooooo`03ooooo`05o`000?oooooo0000ooooool000000oooool>
o`0000?ooooo00?o0000ooooooooool02_ooool3o`0000Cooooo00?o0000ooooooooool01?ooool2
o`000003ooooool0003ooooo00[ooooo00Co0000ooooool0003ooooo0_l00002ooooo`03o`000?oo
oooooooo00?ooooo00Co0000oooooooooooooooo0_l0000;ooooo`Oo0000>_ooool003Gooooo0ol0
0002ooooo`Go00002?ooool2o`0000Oooooo00Go0000ooooooooooooooooo`00000;ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooool000003oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooo000000;ooooo00?o0000ooooooooool01?ooool01?l0003oooooooooooooool<o`0000[o
oooo1Ol00002ooooo`03o`000?oooooooooo00Cooooo0_l000000ooooooo0000ooooo`09ooooo`04
o`000?ooooooooooo`0000?ooooo00?o0000ooooool000001_ooool00ol0003oooooo`000002oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooool000001?l00002ooooo`?o0000=_ooool003Co
oooo1_l000001?oooooo0000o`000?l00009ooooo`03o`000?oooooo000000Kooooo00Go0000oooo
ooooooooooooo`00000:ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000003oooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01?oo
ool01?l0003oooooo`000?l00002ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo
oooo3Ol00002ooooo`03o`000?oooooo000000Gooooo00Co0000ooooool0003o00002_ooool01Ol0
003ooooooooooooooooo000000Cooooo0_l00007ooooo`;o00000oooool00ol0003oooooooooo`05
ooooo`04o`000?ooooooooooooooo`?o00000_ooool5o`0003Gooooo000cooooo`Oo00003_ooool0
0ol0003oooooo`000004ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000kooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00;ooooo0ol00005ooooo`05o`00
0?ooooooooooooooool000003_ooool:o`0000;ooooo00Co0000oooooooooooo00002oooool01Ol0
003ooooooooooooooooo000000Gooooo0ol00006ooooo`05o`000?ooooooooooooooool000001_oo
ool00ol0003oooooooooo`06ooooo`Oo0000=?ooool003?ooooo1ol0000?ooooo`03o`000?oooooo
000000?ooooo00Ko0000oooooooooooo0000ooooool00009ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooool000003Oooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000Co
oooo1Ol00007ooooo`05o`000?ooooooooooooooool000003_ooool01?l0003oooooo`000?l00003
ooooo`;o000000?oooooo`000?l000002?l00006ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool00oooool01?l0003oooooooooool00004ooooo`03o`000?oooooo000000?ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`07ooooo`Oo0000=?ooool003?ooooo1ol0000?
ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool00_l00009ooooo`03o`000?oooooo
oooo00Kooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`02ooooo`05o`000?oo
ooooooooooooool000000_ooool3o`0000?ooooo00?o0000ooooooooool01Oooool01?l0003ooooo
ooooool0000?ooooo`03o`000?oooooo000000;o00000_ooool00ol0003oooooo`000002o`0000;o
oooo0_l00004ooooo`[o000000?oooooo`000?ooool01?ooool01?l0003oooooooooooooool2o`00
00;ooooo00Co0000oooooooooooo00000oooool01?l0003oooooooooool0000:ooooo`Oo0000=?oo
ool003Cooooo1Ol0000Aooooo`05o`000?oooooo0000ooooool000001?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Oooooo00Co0000oooooooooooo00003?ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooooooo`?o00000oooool01?l0003oooooooooool00007oooo
o`04o`000?ooooooooooo`00013ooooo00Oo0000ooooool0003oooooo`000?oooooo000000?ooooo
00?o0000ooooooooool00_l00008ooooo`03o`000?oooooooooo00?ooooo2ol00002ooooo`?o0000
0oooool01_l0003oooooooooool0003oooooo`0000_ooooo1_l0000eooooo`00;oooool3o`0000?o
oooo0ol0000Cooooo`04o`000?oooooo0000o`0000Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`08ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000002o`0000Kooooo00?o0000ooooool000002?ooool2o`00017ooooo00Oo0000
ooooool0003oooooo`000?oooooo000000;ooooo00Ko0000ooooool0003o0000ooooool00007oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool01?l0003oooooooooooooool:
o`0000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`04o`0000;ooooo0ol0000`
ooooo`00;_ooool5o`0000;ooooo00?o0000ooooooooool04oooool01?l0003oooooooooool00004
ooooo`;o00001_ooool00ol0003oooooooooo`09ooooo`04o`000?ooooooooooo`0000[ooooo00?o
0000ooooooooool00_ooool3o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00
0008ooooo`03o`000?oooooooooo013ooooo00?o0000ooooool000000_ooool01?l0003oooooo`00
0?l00003ooooo`?o00001_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool01?ooool01?l0003oooooooooool00002ooooo`_o00000oooool00ol0003o
ooooooooo`05ooooo`Ko0000;oooool002gooooo2?l0000Gooooo`;o000000?oooooo`000?ooool0
0_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo
00002_ooool00ol0003oooooo`000003o`0000Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo013ooooo00Co0000ooooool0003o00000_ooool3o`00
00;ooooo00?o0000ooooooooool00_l00004ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`;o000000?ooooo
o`000?ooool00_ooool00ol0003oooooooooo`03ooooo`[o000000?oooooo`000?l000001Ol0000^
ooooo`00;Oooool8o`0001Sooooo00Co0000oooooooooooo00000_ooool00ol0003oooooo`000003
ooooo`03o`000?oooooooooo00_ooooo00Co0000oooooooooooo00002Oooool3o`0000?ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`;o00001oooool2o`0001;ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000?ooooo0ol000000ooooooo
0000ooooo`0:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`04ooooo`?o00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Ko
oooo2?l0000^ooooo`00;Oooool7o`000003ooooool0003o000000Ko00003oooool2o`0000;ooooo
00?o0000ooooool000000oooool00ol0003oooooo`00000>ooooo`05o`000?ooooooooooooooool0
00001Oooool4o`0000Gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`;o
00001oooool00ol0003oooooo`00000Aooooo`04o`000?ooooooooooo`0000;ooooo0_l00002oooo
o`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02
o`0000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`08ooooo`Oo0000;_ooool0
02kooooo1Ol0000:ooooo`So00001oooool00ol0003oooooo`000002ooooo`;o00000oooool2o`00
013ooooo00Co0000oooooooooooo00000oooool2o`0000?ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000001Oooool0
1?l0003oooooooooool0000Aooooo`03o`000?oooooooooo00;o000000Koooooo`000?l0003o0000
ooooool00004ooooo`04o`000?oooooo0000ooooo`;o00002_ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02oooo
o`;o00000_ooool01?l0003oooooooooool0000<ooooo`Go0000;oooool002Wooooo0ol00003oooo
o`Co00004_ooool8o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo00Co00001Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000004Oooool00ol0003oooooooooo`03o`000004ooooool0
003oooooo`0000Gooooo0_l00004ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
1Oooool3o`000003ooooool0003ooooo00gooooo0ol00002ooooo`?o0000:oooool002Sooooo1Ol0
0004ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooool000001ol000001?oooooo0000oooo
ool0000?ooooo`Go00001oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000?o
oooo00?o0000ooooooooool04?ooool00ol0003oooooo`000002o`000004ooooool0003oooooo`00
00Gooooo00?o0000ooooooooool01?ooool2o`0000Sooooo00Go0000ooooooooooooooooo`000004
ooooo`03o`000?oooooooooo00;o00002Oooool3o`00017ooooo1ol0000Zooooo`009oooool7o`00
00;ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo0000
ooooo`Wo00002Oooool3o`0000?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`02
o`000005ooooool0003o0000ooooool000000oooool2o`0000Sooooo0_l00006ooooo`05o`000?oo
ooooooooooooool000001?ooool00ol0003oooooo`00000<ooooo`?o00004?ooool8o`0002Wooooo
000Wooooo`Oo000000?oooooo`000?ooool06_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
oooo0000ooooo`;o00000oooool01?l0003oooooooooooooool9o`0000?ooooo00Co0000oooooooo
oooo00001_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`03ooooo`;o00001Oooool00ol0003oooooooooo`0?oooo
o`03o`000?oooooo000000;o000000?oooooo`000?l000000_ooool01Ol0003oooooo`000?oooooo
000000[ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o
0000ooooool000003?ooool00ol0003oooooo`000003o`0000kooooo2?l0000Yooooo`009oooool;
o`0001Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02o`0000?ooooo00?o0000
ooooooooool01Oooool2o`0000;ooooo2Ol00005ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo0_l00005
ooooo`03o`000?oooooooooo00oooooo00Go0000ooooool0003o0000ooooo`02o`0000?ooooo0_l0
0002ooooo`03o`000?oooooooooo00Wooooo0_l00003ooooo`05o`000?ooooooooooooooool00000
0oooool2o`0000cooooo00Co0000oooooooooooooooo0_l000000ooooooo0000o`00000<ooooo`So
0000:Oooool002Sooooo1Ol000000ooooooo0000ooooo`02ooooo`Co00005?ooool00ol0003ooooo
ooooo`04ooooo`?o00000_ooool01Ol0003ooooooooooooooooo000000?ooooo0ol00008ooooo`04
o`000?ooooooooooooooo`So00000_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000006ooooo`03o`000?oooooooooo
00kooooo0_l000000ooooooo0000ooooo`02o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo
00_ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000cooooo00?o
0000ooooooooool00oooool2o`0000;ooooo00?o0000ooooooooool02_ooool6o`0002[ooooo000T
ooooo`?o000000?oooooo`000?l000000_l000000ooooooo0000ooooo`07ooooo`Go00003oooool0
0ol0003oooooooooo`04ooooo`;o00001?ooool01?l0003oooooooooooooool3o`0000_ooooo00?o
0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000;ooooo2?l00002ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00kooooo00Go0000oooooooooooo0000ooooo`03o`0000;ooooo00Go0000oooo
ool0003oooooo`00000>ooooo`?o00000oooool01?l0003oooooooooool0000;ooooo`;o00001ooo
ool01?l0003oooooooooooooool2o`0000[ooooo00?o0000ooooool000000_l00002ooooo`?o0000
9_ooool002?ooooo1_l00004ooooo`03o`000?oooooooooo00_ooooo1?l0000:ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooool000001?ooool4o`0000kooooo00Go0000oooooooooooooooo
o`000003ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool9o`0000Cooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00kooooo
00Co0000oooooooooooooooo1?l000001Ooooooo0000oooooooooooo000000;ooooo00?o0000oooo
ooooool03Oooool01_l0003ooooooooooooooooo0000ooooo`;o00002_ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00;ooooo0_l00008ooooo`06o`000?oooooo0000ooooool0003o
oooo1Ol0000Uooooo`008_ooool7o`0000?ooooo00?o0000ooooooooool04?ooool4o`0000Kooooo
00?o0000ooooooooool01?ooool2o`0000;ooooo1?l000001Ooooooo0000oooooooooooo000000go
oooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000002?ooool0
0ol0003oooooooooo`03ooooo`So00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00gooooo0_l00003ooooo`03o`000?oooooo000000;o00001?ooool00ol0003oooooo`00000?
ooooo`?o000000Goooooo`000?l0003oooooo`000009ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Wo00009?oo
ool002;ooooo1ol00002ooooo`03o`000?oooooooooo01Gooooo1Ol000000ooooooo0000ooooo`05
ooooo`;o000000?oooooo`000?l000001Oooool00ol0003oooooo`00000=ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`Wo00003Oooool01?l0003ooooo
ooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo0_l0000@ooooo`06o`000?oooooo0000
o`000?oooooo00002Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Cooooo0_l0
0006ooooo`04o`000?oooooo0000ooooo`Oo00009?ooool002;ooooo1ol000000ooooooo0000oooo
o`0Looooo`Co00000oooool4o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool0000<ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00002?ooool01?l0003o
ooooooooooooool9o`0000Gooooo00Go0000oooooooooooo0000ooooo`02o`000003ooooool0003o
000000Kooooo0_l0000@ooooo`03o`000?oooooo000000?o00002?ooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo00Gooooo0_l00004ooooo`03o`000?oooooooooo00So00009?ooool0
02?ooooo1Ol00002ooooo`03o`000?oooooooooo01_ooooo00Co0000oooooooooooooooo1Ol00004
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000003Oooool01ol0003ooooooooooooo
oooo0000ooooool000000oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00?ooooo
00Co0000oooooooooooo00002?ooool01Ol0003ooooooooooooooooo000000Sooooo2Ol000001Ooo
oooo0000o`000?l0003ooooo00;o00001_ooool00ol0003oooooooooo`0>ooooo`;o000000?ooooo
o`000?ooool00_l00006ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01_ooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool000001?l0000Uooooo`009?ooool3
o`0000;ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`04o`0000;ooooo1Ol00000
0ooooooo0000ooooo`04ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool00_ooool2
o`0000?ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000[ooooo00Co0000oooooooooooo00003?ooool01?l0003oooooooooooooool8o`0000Ko
oooo0_l0000?ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo013ooooo0_l00008ooooo`;o000000?oooooo`000?ooool00_ooool5o`0002Ko
oooo000Pooooo`Co00000oooool2o`0001kooooo0ol00003ooooo`04o`000?ooooooooooo`0000Co
oooo1Ol00003ooooo`03o`000?oooooo000000_ooooo00?o0000ooooooooool00_ooool2o`0000?o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00_ooooo00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`02ooooo`?o000000?ooooo
o`000?ooool02_l0000>ooooo`;o00000_ooool2o`0000?ooooo0_l000000ooooooo0000ooooo`0C
ooooo`;o00002Oooool2o`0000?ooooo00Co0000oooooooooooo00000oooool3o`00027ooooo000O
ooooo`Go00000oooool2o`0001_ooooo1?l00004ooooo`05o`000?ooooooooooooooool000001_oo
ool00ol0003oooooooooo`06o`0000_ooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooo
oooooooo000000[ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00000<ooooo`04o`00
0?ooooooooooo`0000Wooooo00?o0000ooooooooool00oooool2o`0000;ooooo00Go0000ooooool0
003oooooo`000005ooooo`03o`000?oooooo000000Oo00001Oooool00ol0003oooooo`000002oooo
o`;o00001Oooool00ol0003oooooooooo`0Dooooo`;o00002_ooool3o`000006ooooool0003ooooo
ooooool0003ooooo1Ol0000Pooooo`007_ooool7o`0000;ooooo0_l0000Iooooo`;o00000_ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`Go00002Oooool01?l0003oooooooooooooool2o`0000;ooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000gooooo00?o
0000ooooool000002Oooool00ol0003oooooooooo`03ooooo`09o`000?ooooooooooooooool0003o
ooooo`000?oooooo000000Gooooo00Co0000oooooooooooo00001_ooool8o`0000;ooooo00?o0000
ooooooooool00oooool01?l0003oooooo`000?l0000Dooooo`03o`000?oooooo000000Wooooo0ol0
00001?oooooo0000ooooooooool8o`0001oooooo000Nooooo`Oo000000Coooooo`000?oooooo0000
5_ooool3o`0000Cooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool00oooool2o`0000;ooooo1?l00005ooooo`04o`000?oooooooooooooo
o`;o00000_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo0_l0000?oooo
o`;o00002?ooool00ol0003oooooooooo`03ooooo`;o00000oooool01_l0003oooooooooool0003o
ooooo`0000Cooooo00Co0000oooooooooooo00002Oooool01?l0003oooooooooooooool9o`0000Co
oooo00?o0000ooooooooool04_ooool00ol0003oooooo`000008ooooo`03o`000?oooooooooo00?o
00000_ooool7o`0001oooooo000Nooooo`So00000oooool00ol0003oooooooooo`0@ooooo`?o0000
1oooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01?ooool2o`0000Gooooo1_l00003ooooo`;o000000?oooooo`000?ooool02_ooool01?l0003o
ooooooooooooool2o`00013ooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`03oooo
o`;o000000?oooooo`000?ooool00_ooool01Ol0003oooooo`000?oooooo000000Gooooo00Co0000
oooooooooooo00001_ooool2o`0000Gooooo00?o0000ooooool000000oooool00ol0003oooooo`00
0007o`0001;ooooo00?o0000ooooool000001_ooool2o`0000Cooooo0_l000000ooooooo0000o`00
0005o`0001oooooo000Oooooo`Go000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`0=
ooooo`?o00002Oooool01Ol0003ooooooooooooooooo000000Gooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`;o00002Oooool4o`0000;ooooo0_l0000;ooooo`04o`000?oo
ooooooooooooo`;o00004Oooool2o`0000Kooooo00?o0000ooooooooool00_ooool2o`0000;ooooo
00Wo0000ooooooooooooooooo`000?oooooo0000ooooool000001Oooool01Ol0003ooooooooooooo
oooo000000Cooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000002ooooo`03o`000?oo
oooooooo00Oooooo2Ol0000:ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooooooooooo
00001?ooool2o`000003ooooool0003o000000?o00008?ooool0023ooooo0ol000000ooooooo0000
ooooo`03ooooo`03o`000?oooooooooo00[ooooo0ol0000<ooooo`04o`000?ooooooooooo`0000Ko
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooool000001?l0000;ooooo`05o`000?ooooooooooooooool000004oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000_ooool01ol0
003ooooooooooooooooo0000ooooool000000_ooool00ol0003oooooooooo`02ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`06ooooo`;o000000?oooooo`000?l00000
3?ooool00ol0003oooooooooo`04ooooo`So00000oooool01?l0003oooooooooool00002ooooo`05
o`000?ooooooooooooooool000001?ooool00ol0003oooooo`000004o`00027ooooo000Looooo`?o
000000Goooooo`000?l0003oooooo`000006ooooo`03o`000?oooooooooo00Oooooo0_l0000?oooo
o`03o`000?oooooo000000Oooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooool000000_ooool7o`0000Kooooo00Go0000oooo
ooooooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000003ooooo`07o`000?ooooooooooooooool0003oooooo`000002ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000009ooooo`;o0000
00?oooooo`000?ooool03Oooool2o`0000cooooo2Ol00004ooooo`03o`000?oooooooooo00;ooooo
00Co0000ooooool0003ooooo0_l00002ooooo`?o00007Oooool001_ooooo1_l000000ooooooo0000
o`000006ooooo`03o`000?oooooooooo00Cooooo0ol0000@ooooo`03o`000?oooooo000000Sooooo
00?o0000ooooooooool02_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooool000000_ooool2o`0000Gooooo1?l00002ooooo`03o`000?oooooooooo00;o0000
5?ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01ol0
003ooooooooooooooooo0000ooooool000000oooool01Ol0003ooooooooooooooooo000000Gooooo
00?o0000ooooooooool02?ooool4o`00013ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
ooooo`09o`0000?ooooo00?o0000ooooooooool00_ooool6o`0001cooooo000Jooooo`Oo000000?o
ooooo`000?ooool01oooool00ol0003oooooooooo`03o`0001?ooooo00?o0000ooooool000002?oo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_oo
ool2o`0000?ooooo0_l00009ooooo`Ko00005Oooool2o`000003ooooool0003ooooo00;ooooo00Co
0000oooooooooooo00001?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool01Ol0
003ooooooooooooooooo000000Cooooo00?o0000ooooool000002Oooool01?l0003oooooooooool0
000Aooooo`;o00003_ooool2o`0000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
000>o`0001_ooooo000Jooooo`Wo00002?ooool3o`0001Kooooo0_l00009ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`06ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool02?ooool00ol0003oooooo`000004o`0001;ooooo0_l0
00000ooooooo0000ooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000?ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00Sooooo0_l00002ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02oooool2o`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000?ooooo1ol0
000Kooooo`006_ooool7o`000003ooooool0003ooooo00Cooooo0ol000000ooooooo0000ooooo`0E
ooooo`;o00002Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00Cooooo0_l00007
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool02?ooool2o`000003ooooool0
003ooooo00;ooooo1Ol0000=ooooo`03o`000?oooooo000000?ooooo00Co0000oooooooooooo0000
1_ooool01?l0003oooooooooooooool2o`0000Cooooo00Co0000oooooooooooo00000_ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000000_ooool00ol0
003oooooooooo`0Booooo`;o00002_ooool00ol0003oooooo`000003ooooo`07o`000?oooooooooo
ooooool0003oooooo`000003ooooo`04o`000?ooooooooooooooo`Oo00006oooool001_ooooo1_l0
00000ooooooo0000ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`05ooooo`;o00001Ooo
ool2o`0000;ooooo00Co0000oooooooooooo00002_ooool2o`000003ooooool0003ooooo00Oooooo
1?l0000:ooooo`05o`000?oooooo0000ooooool000000oooool00ol0003oooooooooo`04ooooo`04
o`000?ooooooooooooooo`;o00001?ooool01_l0003oooooooooool0003oooooo`0000Kooooo00?o
0000ooooooooool00_ooool01ol0003oooooo`000?oooooo0000ooooool000005_ooool00ol0003o
ooooooooo`06ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`02o`00
00Gooooo00Co0000oooooooooooooooo1Ol0000Looooo`007?ooool5o`000003ooooool0003o0000
00;o00001oooool00ol0003oooooooooo`0Booooo`;o00002_ooool00ol0003oooooooooo`0>oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooool000001?ooool2o`000003ooooool0003ooooo
00?ooooo00?o0000ooooooooool01oooool01?l0003oooooooooool0000<ooooo`Go00001Oooool0
1?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Gooooo00Oo0000oooooooooooooooo
o`000?oooooo000000Cooooo00Co0000ooooool0003o00001oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooool0003oooooo`00000Gooooo`;o00001Oooool00ol0003ooooooooo
o`03ooooo`06o`000?ooooooooooooooool0003ooooo0_l00005ooooo`03o`000?oooooooooo00;o
oooo0ol0000Mooooo`007?ooool00ol0003oooooo`000004o`0000[ooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0
1?ooool2o`0000Cooooo00Co0000ooooool0003o00001?ooool00ol0003oooooooooo`06ooooo`;o
00000_ooool00ol0003oooooooooo`0?ooooo`Ko000000?oooooo`000?l000000oooool00ol0003o
ooooooooo`05ooooo`07o`000?ooooooooooooooool0003oooooo`000004ooooo`04o`000?oooooo
0000o`0000Sooooo00?o0000ooooool000000oooool01Ol0003oooooo`000?oooooo000001Wooooo
0_l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`0Kooooo`006?ooool3o`000003ooooool0
003o000000?o000000?oooooo`000?ooool02_ooool00ol0003oooooooooo`0@ooooo`;o00002ooo
ool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Gooooo0_l00003ooooo`04o`000?oo
oooo0000o`0000Cooooo00?o0000ooooooooool01_ooool2o`000003ooooool0003ooooo01Cooooo
1ol000000ooooooo0000ooooo`07ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`;o0000
00?oooooo`000?ooool02?ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool0
00006oooool2o`0000Sooooo00Co0000oooooooooooooooo0ol00006ooooo`03o`000?oooooooooo
00Cooooo1?l0000Iooooo`005oooool7o`000006ooooool0003o0000o`000?oooooo00002_ooool0
0ol0003oooooooooo`0?ooooo`;o00003?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo00Cooooo00Go0000ooooool0003oooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003oooooo`000?oooooo000001Gooooo00Co0000ooooool0003o
00000_ooool4o`0000Oooooo00?o0000ooooooooool00_ooool2o`0000Cooooo0_l000000ooooooo
0000ooooo`07ooooo`;o00000oooool01_l0003oooooooooool0003oooooo`0001_ooooo0ol00008
ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03
ooooo`Go00006?ooool001Kooooo1ol00002ooooo`05o`000?oooooo0000ooooool000002oooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooo000000_ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`04ooooo`04o`000?oooooo0000o`0000;ooooo0_l00005ooooo`03o`000?oo
oooooooo00Cooooo00Go0000ooooool0003oooooo`00000Dooooo`05o`000?oooooo0000ooooool0
00000oooool00ol0003oooooooooo`04o`0000?ooooo00?o0000ooooooooool00_ooool2o`0000?o
oooo00Go0000ooooool0003oooooo`000007ooooo`04o`000?ooooooooooo`0000;ooooo00Ko0000
oooooooooooo0000ooooool0000Jooooo`05o`000?ooooooooooooooool000001oooool2o`0000?o
oooo0_l00006ooooo`03o`000?oooooooooo00;ooooo1ol0000Gooooo`005_ooool7o`0000;ooooo
00?o0000ooooool000000_l0000;ooooo`03o`000?oooooooooo00cooooo00Co0000oooooooooooo
00002oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Cooooo00Co0000oooooooo
oooo00000_ooool2o`0000Kooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0001?ooooo00?o0000ooooool000000_ooool01?l0003oooooooooooooool2o`0000Gooooo
1Ol00003ooooo`;o00000_ooool01?l0003oooooooooooooool2o`0000Kooooo00?o0000oooooooo
ool00_ooool2o`0000?ooooo00?o0000ooooool000006Oooool00ol0003oooooooooo`03ooooo`;o
00001Oooool2o`0000Cooooo0_l00006ooooo`_o00005oooool001Kooooo1ol00002ooooo`03o`00
0?oooooooooo00;o00003?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo000000cooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool01Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool0
00004oooool2o`0000?ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`06oooo
o`Ko000000?oooooo`000?ooool00oooool2o`0000Gooooo00?o0000ooooooooool00oooool2o`00
00?ooooo00?o0000ooooool000006?ooool00ol0003oooooooooo`06ooooo`05o`000?oooooooooo
ooooool000000_ooool01ol0003ooooooooooooooooo0000ooooool000000_ooool6o`000003oooo
ool0003o000000Go00005oooool001Oooooo1Ol00002ooooo`04o`000?ooooooooooooooo`;o0000
3?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000oooooooo
ool04oooool00ol0003oooooooooo`05ooooo`06o`000?oooooo0000o`000?oooooo00001_ooool0
1Ol0003ooooooooooooooooo000000?ooooo0_l0000Cooooo`;o00001Oooool00ol0003oooooo`00
0002ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool5o`0000;ooooo0_l0
0004ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool2o`0001Oooooo00?o
0000ooooooooool02?ooool3o`0000?ooooo00Co0000oooooooooooooooo1Ol000000ooooooo0000
o`000002o`0000;ooooo1_l0000Hooooo`006?ooool3o`0000?ooooo00?o0000ooooooooool00_oo
ool2o`0000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000cooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool01?ooool00ol0003oooooo`000004ooooo`;o00004oooool00ol0003o
ooooooooo`04ooooo`;o00001?ooool00ol0003oooooooooo`05ooooo`04o`000?oooooooooooooo
o`;o00001?ooool4o`0000?ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000003oooo
o`;o00005_ooool00ol0003oooooooooo`0;ooooo`;o00000oooool4o`0000?ooooo0_l00003oooo
o`03o`000?oooooo000000;ooooo0ol0000Iooooo`005Oooool3o`000003ooooool0003ooooo00?o
oooo00?o0000ooooooooool00_ooool00ol0003oooooo`00000;ooooo`03o`000?oooooooooo00Oo
oooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool00oooool2o`000005ooooool0003o0000ooooool000001_ooool00ol0003oooooo`000004
ooooo`;o00004_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01?ooool01_l0003oooooooooool0003oooooo`0000Oooooo1_l00006ooooo`03o`00
0?oooooo000000?ooooo0_l0000Eooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Ol000000ooooooo0000o`000002o`000003ooooool0003ooooo00;ooooo0_l00006ooooo`?o0000
5_ooool001Cooooo1_l00004ooooo`03o`000?oooooooooo00;ooooo0ol0000<ooooo`03o`000?oo
oooooooo00Kooooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo01Gooooo
00?o0000ooooooooool00oooool3o`0000;ooooo0_l00006ooooo`;o00001Oooool2o`00017ooooo
00?o0000ooooooooool01_ooool2o`0000Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
o`000002ooooo`03o`000?oooooooooo00Gooooo0_l00004ooooo`Co00000_ooool01?l0003ooooo
ooooool00002ooooo`;o00005?ooool00ol0003oooooooooo`0;ooooo`Co000000Coooooo`000?l0
003ooooo1?l00004ooooo`03o`000?oooooooooo00;o00001Oooool5o`0001Gooooo000Cooooo`Oo
00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000_ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo00?o0000oooooooo
ool05oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000006
ooooo`;o00001Oooool2o`00013ooooo0_l00007ooooo`03o`000?oooooo000000Kooooo00?o0000
ooooooooool00oooool2o`0000?ooooo0_l00006ooooo`;o00002?ooool5o`000003ooooool0003o
oooo00;o00004oooool00ol0003oooooooooo`08ooooo`Co00001_ooool4o`000003ooooool0003o
oooo00Cooooo0_l000000ooooooo0000ooooo`03ooooo`Oo00005?ooool001?ooooo1ol00004oooo
o`03o`000?oooooooooo00?ooooo0_l000000ooooooo0000ooooo`09ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
01Sooooo00Co0000oooooooooooo00000_ooool2o`0000;ooooo0_l00005ooooo`;o00001Oooool0
0ol0003oooooooooo`0>ooooo`;o00002?ooool00ol0003oooooo`000007ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
000000Wooooo00Co0000oooooooooooooooo1Ol0000Booooo`03o`000?oooooooooo00Gooooo1?l0
0002ooooo`03o`000?oooooooooo00?ooooo0_l00002ooooo`Co00001Oooool2o`0000;ooooo00Co
0000oooooooooooooooo1ol0000Dooooo`004oooool7o`0000?ooooo00?o0000ooooooooool01?oo
ool01?l0003oooooo`000?l0000;ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01Sooooo00So0000oooooooooooo
0000ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000005
ooooo`03o`000?oooooooooo00gooooo0_l00008ooooo`04o`000?ooooooooooo`0000Oooooo00Co
0000oooooooooooooooo0_l00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
0_l00008ooooo`03o`000?oooooooooo00?ooooo1_l0000>ooooo`03o`000?oooooooooo00;ooooo
1?l00005ooooo`04o`000?ooooooooooooooo`?o00001_ooool2o`0000Cooooo00Ko0000oooooooo
oooo0000ooooool00003ooooo`Oo00005?ooool001Cooooo1Ol00004ooooo`03o`000?oooooooooo
00Cooooo00Go0000ooooool0003oooooo`00000:ooooo`05o`000?ooooooooooooooool000001_oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01Wooooo00Go0000ooooool0003ooooo
o`000002ooooo`03o`000?oooooooooo00;o00000oooool01?l0003oooooooooool00004ooooo`03
o`000?oooooooooo00gooooo0_l00008ooooo`05o`000?ooooooooooooooool000001oooool00ol0
003oooooo`000002ooooo`03o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`000003
ooooo`;o00001oooool00ol0003oooooooooo`05ooooo`;o00000oooool5o`0000Sooooo00?o0000
ooooool000000ol00008ooooo`03o`000?oooooooooo00;o00002_ooool2o`0000;ooooo00Go0000
ooooooooooooooooo`000002ooooo`04o`000?ooooooooooooooo`Go00005Oooool001Gooooo0ol0
0005ooooo`03o`000?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool0000:ooooo`04
o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Joooo
o`04o`000?ooooooooooo`0000;ooooo00So0000oooooooooooo0000ooooool0003oooooo`0000?o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<ooooo`;o00002?ooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000004oooo
o`04o`000?ooooooooooo`0000Cooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`05
ooooo`;o00002?ooool4o`0000;ooooo1?l0000;ooooo`Co00003Oooool3o`0000Gooooo00?o0000
ooooool000000oooool4o`0001Kooooo000Gooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooool000001_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooool000001?ooool01Ol0003oooooooooool0003ooooo00;o00000oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo00_ooooo0_l00009ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01Oooool2o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0005ooooo`03o`000?oooooo000000Kooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`09ooooo`Ko00002_ooool3o`00013ooooo1?l00005ooooo`05o`000?oooooo0000ooooool00000
6_ooool001;ooooo0ol00002ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?oo
ool01ol0003ooooooooooooooooo0000ooooool000002_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool06oooool3o`0000Cooooo00Go0000oooooooo
oooo0000ooooo`02o`0000Cooooo00?o0000ooooooooool00_l0000<ooooo`03o`000?oooooo0000
00Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`04o`000?oooooooooo
o`0000;ooooo00?o0000ooooooooool00_ooool2o`0000Kooooo00?o0000ooooool000001Oooool0
0ol0003oooooooooo`07ooooo`;o00001_ooool4o`0000;ooooo00Co0000oooooooooooooooo1?l0
0004ooooo`;o000000?oooooo`000?ooool04?ooool2o`0000;ooooo0_l00004ooooo`?o00001Ooo
ool3o`0001?ooooo000Aooooo`Ko00001_ooool00ol0003oooooooooo`05ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool01_ooool2o`0000Kooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`0Kooooo`?o00001Oooool01_l0003oooooo`000?l0003oooooo`0000Co
oooo00?o0000ooooooooool00_l0000;ooooo`03o`000?oooooo000000Wooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool0
00001?ooool2o`0000Kooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`07oooo
o`;o00000_ooool4o`0000Gooooo00?o0000ooooooooool01_ooool5o`000003ooooool0003ooooo
013ooooo0_l000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`Go00004_ooool0013ooooo1ol00006ooooo`03o`000?oooooooooo00Gooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000001Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo01[ooooo00Co0000ooooool0003o00001_ooool3
o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooo`000?l0000:ooooo`04o`000?oo
ooooooooo`0000Sooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000Go
oooo00?o0000ooooool000000oooool3o`0000Kooooo00Co0000oooooooooooo00001?ooool00ol0
003oooooooooo`07ooooo`Co00002?ooool00ol0003oooooooooo`06ooooo`;o00001?ooool4o`00
00kooooo00?o0000ooooooooool00_l00004ooooo`;o000000Coooooo`000?oooooo00000oooool7
o`00017ooooo000@ooooo`Oo00001Oooool00ol0003oooooooooo`06ooooo`05o`000?oooooooooo
ooooool000000_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000Cooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Jooooo`05o`000?oooooo0000ooooool00000
1Oooool3o`0000?ooooo00Ko0000ooooooooooooooooo`000?ooool2o`0000Wooooo00Co0000oooo
oooooooo00002Oooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`0000Oooooo0_l0
0002ooooo`04o`000?oooooo0000o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03
o`000?oooooooooo00Cooooo1Ol00002ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool3o`0000Gooooo00?o0000ooooooooool00_ooool5o`0000Sooooo00Oo0000oooooooo
ooooooooo`000?oooooo000000Gooooo00Go0000ooooooooooooooooo`000002ooooo`Oo00004Ooo
ool0013ooooo1ol00005ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool0
1?l0003oooooooooool00004ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooooooo
o`00000?ooooo`03o`000?oooooooooo01[ooooo00Go0000ooooool0003oooooo`000006ooooo`;o
00001?ooool01Ol0003oooooooooool0003ooooo00;o00002?ooool01?l0003oooooooooool00009
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000002Oooool00ol0003oooooo`000003
ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00000_ooool00ol0003ooooooooo
o`04o`0000Cooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00?ooooo
0_l00007ooooo`03o`000?oooooooooo00Sooooo1?l00003ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000oooool01Ol0003oooooo`000?l0003ooooo00;o000000?oooooo`00
0?l000001Ol0000Aooooo`004Oooool5o`0000Kooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`02ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`03oooo
o`05o`000?ooooooooooooooool000003oooool00ol0003oooooooooo`0Jooooo`;o00000oooool0
0ol0003oooooooooo`02ooooo`?o00001Oooool00ol0003oooooooooo`03o`0000Oooooo00Go0000
ooooooooooooooooo`000009ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?oo
ool2o`0000Cooooo0_l00006ooooo`05o`000?ooooooooooo`000?ooool01?l00008ooooo`;o0000
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooooooo`?o00002?ooool00ol0003o
ooooooooo`0=ooooo`Go00000oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Go0000ooooool0003o0000ooooo`05o`0001;ooooo000Booooo`Co00001_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00Ko0000oooooooooooo0000ooooool0
0007ooooo`03o`000?oooooo0000013ooooo00?o0000ooooooooool06Oooool00ol0003oooooo`00
0004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01?ooool01Ol0003ooooo
o`000?oooooo000000Kooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooooooo
00Oooooo0_l0000:ooooo`;o00001?ooool2o`0000Kooooo1_l0000;ooooo`;o00000oooool01?l0
003oooooooooool00002ooooo`;o00002_ooool00ol0003oooooooooo`0?ooooo`04o`000?oooooo
ooooooooo`Go00000_ooool2o`0000Kooooo0_l000000ooooooo0000ooooo`03o`0001?ooooo000B
ooooo`;o000000?oooooo`000?ooool01Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool00_l00007ooooo`03o`000?oooooo000000oooooo00?o0000
ooooooooool06_ooool00ol0003oooooo`000004ooooo`04o`000?ooooooooooo`0000?ooooo00?o
0000ooooooooool00oooool01Ol0003oooooo`000?oooooo000000Gooooo00Go0000oooooooooooo
ooooo`00000:ooooo`03o`000?oooooooooo00Gooooo0_l000000ooooooo0000ooooo`07ooooo`;o
000000?oooooo`000?l000000oooool2o`0000Cooooo0ol00003ooooo`;o00002oooool2o`0000Co
oooo1Ol0000;ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool01_ooool5o`0000Oo
oooo1?l0000Fooooo`004?ooool5o`0000Oooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00;o00002?ooool2o`0000oooooo00?o0000ooooooooool0
6_ooool2o`0000Kooooo0_l00004ooooo`03o`000?oooooooooo00Cooooo1?l00004ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`05ooooo`05o`00
0?ooooooooooooooool000001oooool01ol0003ooooooooooooooooo0000ooooool000000_ooool6
o`0000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`;o00001?ooool2
o`0000kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`07ooooo`04o`000?oooooo
0000ooooo`Go00001?ooool01Ol0003oooooo`000?l0003ooooo00?o00004Oooool000oooooo1_l0
0006ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool2o`000003ooooool0
003ooooo00Oooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Iooooo`03o`000?oo
oooo000000Oooooo00?o0000ooooooooool00_ooool2o`0000Kooooo0ol00003ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooo000000Co00001_ooool01?l0003oooooooooooooool2o`0000cooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`0<ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
2?ooool2o`0000?ooooo00Co0000oooooooooooooooo1Ol00002ooooo`Ko00004?ooool000kooooo
1ol00006ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool01Ol0003ooooo
oooooooooooo000000Oooooo0_l0000?ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooool0
00001_ooool2o`0000Cooooo0_l00006ooooo`?o00000_ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool5o`000003ooooool0003o000000Ko
oooo00Co0000oooooooooooooooo0_l0000<ooooo`;o000000?oooooo`000?ooool00_ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo2_l0000?ooooo`003_ooool9o`0000Co
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`;o00000oooool00ol0003o
ooooooooo`04ooooo`;o00003oooool00ol0003oooooooooo`0Iooooo`;o00001_ooool01?l0003o
ooooooooool00004ooooo`;o00001?ooool01_l0003oooooooooool0003oooooo`0000Gooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Ko
0000ooooooooooooooooo`000?ooool4o`000003ooooool0003ooooo00;ooooo0_l000000ooooooo
0000ooooo`04ooooo`07o`000?ooooooooooooooool0003oooooo`000008ooooo`Co000000?ooooo
o`000?ooool00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool02Oooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooool000001_l0000?ooooo`003_ooool7o`0000;ooooo2ol00004ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooool000000oooool00ol0003oooooooooo`03ooooo`04o`000?oooooooooo
o`0000kooooo00?o0000ooooooooool06Oooool2o`0000Kooooo00Co0000oooooooooooo00001?oo
ool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00;o00001Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_ooool00ol0003oooooo`00
0003o`0000Gooooo00Co0000oooooooooooooooo0_l000000ooooooo0000ooooo`04ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool01?ooool2o`0000?ooooo0_l00006ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooo000000Gooooo00?o0000ooooooooool01Oooool8o`0000oooooo000?ooooo`Go0000
1oooool00ol0003oooooooooo`04ooooo`_o00000_ooool01Ol0003ooooooooooooooooo000000Co
oooo00Co0000oooooooooooo00003Oooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo0000
00Gooooo00?o0000ooooooooool00_ooool01ol0003ooooooooooooooooo0000ooooool000000ooo
ool00ol0003oooooooooo`02o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo00Gooooo1Ol00009ooooo`07o`000?ooooooooooooooool0003o
ooooo`000005ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool00ol00005oooo
o`;o00001oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00kooooo00?o0000oooo
ooooool02_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooool000001?l0000@ooooo`004?ooool4o`0000Oooooo00?o0000ooooooooool02?ooool01Ol0
003ooooooooooooooooo000000;ooooo2ol00003ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool06Oooool2o`0000Kooooo00?o0000ooooooooool00oooool01?l0003oooooooooool0
0002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00003ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`Co00000oooool00ol0003ooooo
ooooo`08ooooo`04o`000?ooooooooooooooo`?o00001Oooool01?l0003oooooooooool00003oooo
o`?o00001oooool3o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0>oooo
o`03o`000?oooooooooo00[ooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooool000000_l0000Aooooo`004_ooool2o`0000Kooooo00?o0000oooo
ooooool02Oooool01?l0003oooooooooool00006ooooo`05o`000?ooooooooooooooool000000_oo
ool;o`0000Oooooo00?o0000ooooooooool06Oooool2o`0000Gooooo00?o0000ooooooooool01?oo
ool01ol0003ooooooooooooooooo0000ooooool000000_ooool00ol0003oooooo`000002o`000004
ooooool0003oooooo`0000_ooooo00Go0000ooooooooooooooooo`000002ooooo`?o00001_ooool2
o`0000[ooooo00?o0000ooooooooool00_ooool2o`0000Kooooo00Co0000ooooool0003ooooo0ol0
0008ooooo`04o`000?oooooo0000o`0000Sooooo00Go0000ooooooooooooooooo`00000@ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooool000004oooool000kooooo0ol000000ooooooo0000
o`000006ooooo`03o`000?oooooooooo00Wooooo00Co0000oooooooooooo00001_ooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00oooool;o`0001So
oooo0_l00004ooooo`03o`000?oooooooooo00Kooooo00Co0000oooooooooooo00000_ooool2o`00
0007ooooool0003oooooo`000?oooooo0000ooooo`02o`0000[ooooo00Co0000oooooooooooooooo
1?l00008ooooo`03o`000?oooooo000000[ooooo00?o0000ooooooooool00_ooool3o`0000Gooooo
1?l00003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_l00009ooooo`03o`00
0?oooooo0000013ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool2o`000004ooooool0003o0000o`0000oo
oooo000=ooooo`Go000000?oooooo`000?l000001Oooool00ol0003oooooooooo`0:ooooo`;o0000
2?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
2Oooool01?l0003oooooooooooooool:o`0000gooooo0_l00005ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooool000000oooool2o`0000;ooooo00Co0000ooooool0003ooooo0_l0000:oooo
o`Co00002oooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000004ooooo`?o00001Oooool00ol0003oooooooooo`03ooooo`04o`000?oooooo
ooooooooo`;o00002Oooool00ol0003oooooo`00000?ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooool000001?l0000>ooooo`003?ooool9o`0000Gooooo00?o0000ooooooooool02_ooool0
0ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool000001_ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00_ooooo2ol00002ooooo`;o00001?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooo000000?ooooo00Ko0000ooooooooooooooooo`000?ooool2o`000003
ooooool0003ooooo00Gooooo1?l00002ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool0
00000_ooool2o`000003ooooool0003o000000Kooooo00Go0000ooooooooooooooooo`000004oooo
o`;o00002_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo2?l0000=oooo
o`003?ooool8o`000003ooooool0003ooooo00?ooooo00?o0000ooooooooool02Oooool2o`0000[o
oooo00Co0000oooooooooooooooo0_l00006ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool05_ooool;o`0000Wooooo00Co0000ooooool0003ooooo0ol00002ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooool000000ol00003ooooo`03o`000?oooooo000000_ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000000_l00003ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo0000
1Oooool2o`0000Wooooo00?o0000ooooool000003Oooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01_ooool=o`0000gooooo000<oooooa3o00002_oo
ool2o`0000_ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooooooo`go000000?oooooo`000?l000000_ooool00ol0003oooooo`000005o`0000Kooooo
00?o0000ooooool000002oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo
00Co0000oooooooooooooooo0ol000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooool000001_ooool2o`0000Sooooo00Co0000oooooooooooo00003?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo4Ol00002ooooo`03o`000?oooooo0000
00Ko00003Oooool000gooooo1Ol000001Ooooooo0000oooooooooooo000000?ooooo00?o0000oooo
ool000001ol000001?oooooo0000ooooool0000;ooooo`03o`000?oooooo000000;ooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Sooooo0_l00003
ooooo`03o`000?oooooooooo00cooooo3?l00004ooooo`03o`000?oooooooooo00?ooooo0_l0000;
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01oooool5o`0000;ooooo0_l00003
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000006
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02?ooool01?l0003ooooooooooooo
ool@o`0000Kooooo00?o0000ooooooooool01oooool01?l0003oooooooooool00002ooooo`Go0000
3_ooool000kooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
2?ooool8o`0000Sooooo0_l00003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`0Hooooo`;o00000oooool00ol0003oooooooooo`0<ooooo`;o0000
00?oooooo`000?l000001Ol000000ooooooo0000o`00000:o`0000[ooooo0_l00009ooooo`03o`00
0?oooooooooo00Kooooo0_l00003ooooo`07o`000?ooooooooooo`000?oooooo0000ooooo`02o`00
00Oooooo0_l00007ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool00_ooool@o`00
00Sooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo0000
00;ooooo1?l0000?ooooo`003oooool2o`0000;ooooo00Oo0000ooooooooooooooooo`000?oooooo
000000Wooooo00Go0000ooooooooooooooooo`000004ooooo`Wo00001Oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool06?ooool01Ol0003ooooooooo
oooooooo000000kooooo00?o0000ooooool000000_l00003ooooo`;o00000oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooo000000co00002oooool00ol0003oooooooooo`03ooooo`?o0000
00?oooooo`000?ooool00_ooool01?l0003oooooooooool00002ooooo`?o00001_ooool01?l0003o
ooooooooool00002oooooa7o00000_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`09ooooo`04o`000?oooooooooo
ooooo`?o00004?ooool000gooooo1?l00002ooooo`07o`000?ooooooooooooooool0003oooooo`00
0008ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02_ooool8o`0000Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0Hooooo`05o`000?ooooooooooooooool00000
3?ooool4o`000003ooooool0003o000000Cooooo00Go0000ooooooooooooooooo`000005ooooo`?o
00002_ooool<o`0000;ooooo00?o0000ooooool000000_l00004ooooo`05o`000?oooooooooooooo
ool000000_ooool00ol0003oooooo`00000?o`0000Cooooo00?o0000ooooool000000oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo00Sooooo00Go0000oooooooooooo0000ooooo`04
o`0000kooooo000<ooooo`Go00000_ooool00ol0003oooooooooo`02ooooo`;o00002?ooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Wooooo0_l00005ooooo`Wo00001?ooool00ol0
003oooooooooo`0Gooooo`;o00000_ooool00ol0003oooooooooo`07ooooo`Go00001?ooool2o`00
00?ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0ol00009ooooo`03o`000?oooooooooo
00[ooooo5?l000001Ooooooo0000o`000?oooooo000000?ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool02?ooool0
1Ol0003ooooooooooooooooo000000cooooo00?o0000ooooooooool03oooool00ol0003ooooooooo
o`08ooooo`;o000000?oooooo`000?ooool01Ol0000=ooooo`002oooool7o`000003ooooool0003o
oooo00?ooooo0_l00007ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool0
1?l0003oooooooooool00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02?l0
000Fooooo`05o`000?ooooooooooooooool000001Oooool4o`0000?ooooo00?o0000ooooooooool0
0oooool2o`0000;ooooo00Co0000ooooool0003o00000oooool01_l0003ooooooooooooooooo0000
ooooo`;o00002?ooool00ol0003oooooo`00000?o`0000Oooooo2ol000000ooooooo0000o`000002
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool00ol0003oooooo`000002
ooooo`03o`000?oooooooooo00Wooooo00Co0000oooooooooooo00003?ooool00ol0003ooooooooo
o`0@ooooo`03o`000?oooooooooo00Oooooo00Go0000oooooooooooo0000ooooo`07o`0000cooooo
000;ooooo`Oo000000?oooooo`000?ooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000Cooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo
2Ol0000=ooooo`04o`000?ooooooooooo`0000;ooooo1?l00006ooooo`03o`000?oooooooooo00Co
oooo0ol000001_oooooo0000ooooool0003oooooo`0000?ooooo4Ol00009ooooo`;o00000oooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo0000oooo
o`0;o`0000;ooooo00?o0000ooooooooool00oooool01Ol0003oooooo`000?oooooo000000gooooo
0_l0000=ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01oooool01Ol0003ooooo
ooooool0003ooooo00Oo00003?ooool000_ooooo2_l00003ooooo`;o00001Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0;ooooo`So00001Oooool00ol0003oooooo`000003o`0000Wooooo00?o0000ooooooooool0
0_oooolAo`0000;ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Oo
oooo0ol00006ooooo`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000002oooo
o`;o00000oooool2o`0000Kooooo3?l0000>ooooo`;o00003?ooool00ol0003oooooooooo`0Boooo
o`03o`000?oooooooooo00Kooooo00Co0000ooooool0003ooooo2?l0000<ooooo`003?ooool=o`00
0003ooooool0003ooooo00?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02oooo
o`05o`000?ooooooooooooooool000005Oooool8o`000003ooooool0003o000000ko00000oooool4
o`0000;ooooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000Cooooo
00Co0000oooooooooooo00002?ooool2o`0000Wooooo00?o0000ooooooooool01_ooool01Ol0003o
oooooooooooooooo000000;ooooo0_l00003ooooo`;o00002Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooo000000[o00001?ooool01?l0003oooooooooool0000:ooooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000003ooooo`Ko00003Oooool0
00gooooo0ol000001Ooooooo0000ooooool0003ooooo00;o000000?oooooo`000?l000000ol00000
0ooooooo0000ooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooooooo
o`0<oooooaOo00000oooool00ol0003oooooooooo`06ooooo`07o`000?oooooo0000ooooool0003o
ooooo`000002ooooo`07o`000?oooooo0000ooooool0003oooooo`000005ooooo`03o`000?oooooo
000000Kooooo0ol0000;ooooo`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`00
0002ooooo`;o00000_ooool3o`0000Wooooo00?o0000ooooooooool00_ooool2o`000003ooooool0
003ooooo00Sooooo2ol00006ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooo`000003ooooo`Go00003_ooool0017ooooo00?o0000ooooool000000oooool2
o`0000;ooooo00?o0000ooooool000000ol00007ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo4_l00003
ooooo`?o00001Oooool00ol0003oooooo`000009ooooo`So00001?ooool3o`0000;ooooo0_l00003
ooooo`06o`000?ooooooooooo`000?oooooo00001_ooool00ol0003oooooooooo`03ooooo`;o0000
3_ooool00ol0003oooooooooo`06ooooo`05o`000?ooooooooooooooool000000_ooool2o`000006
ooooool0003oooooo`000?oooooo00002Oooool01Ol0003ooooooooooooooooo000000;ooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00_o00004_oo
ool00ol0003oooooooooo`05ooooo`;o00001?ooool2o`00017ooooo000<ooooo`?o00000_ooool0
0ol0003oooooo`000004ooooo`Go00001?ooool3o`0000Cooooo00?o0000ooooooooool01_ooool0
1?l0003oooooooooooooool@o`000003ooooool0003ooooo00gooooo1?l00007ooooo`?o00002ooo
ool00ol0003oooooooooo`04ooooo`So000000?oooooo`000?l000000_ooool2o`0000;ooooo0_l0
0006ooooo`04o`000?oooooo0000ooooo`?o00004Oooool00ol0003oooooooooo`05ooooo`05o`00
0?ooooooooooooooool000000_ooool3o`0000;ooooo00Co0000oooooooooooo00002?ooool00ol0
003oooooooooo`02o`0000;ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool:o`0000Sooooo00?o0000oooo
ooooool01?ooool00ol0003oooooo`000003ooooo`03o`000?oooooo000000Co00003Oooool000_o
oooo1Ol00002ooooo`;o00001?ooool00ol0003oooooooooo`03o`0000Kooooo4Ol00005ooooo`03
o`000?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool0000:ooooo`Co00002oooool2
o`0000_ooooo00?o0000ooooooooool02?ooool2o`000003ooooool0003ooooo00So00000_ooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o00005?ooool00ol0003oooooooooo`05
ooooo`05o`000?ooooooooooooooool000000_ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000007ooooo`05o`000?oooooo0000ooooool000000_ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0?ooooo`_o
00001?ooool2o`0000?ooooo00?o0000ooooooooool01_l0000<ooooo`002_ooool7o`000004oooo
ool0003o0000oooooa3o00000oooool4o`0000Cooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`04ooooo`04o`000?ooooooooooooooo`;o00001_ooool4o`0000oooooo0_l0000:oooo
o`03o`000?oooooooooo00Wooooo00Go0000ooooooooooooooooo`000002ooooo`;o00001?ooool;
o`0000;ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`05ooooo`06o`000?oooooo
ooooooooool0003ooooo0ol00003ooooo`05o`000?ooooooooooooooool000002?ooool2o`000005
ooooool0003oooooooooool000002?ooool00ol0003oooooooooo`08ooooo`04o`000?oooooooooo
o`0001_ooooo00?o0000ooooool000002_l000000ooooooo0000o`000005o`0000_ooooo000:oooo
o`_o00000_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00?o00001_ooool0
0ol0003oooooooooo`05o`0000[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02
o`0000;ooooo1?l0000Cooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02_ooool0
1Ol0003ooooooooooooooooo000000;ooooo0_l00004ooooo`?o00000oooool:o`00017ooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o000000?oooooo`000?ooool00ooo
ool01Ol0003ooooooooooooooooo000000Oooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00[ooooo0_l0000Looooo`03o`000?oooooooooo00;o
oooo0_l00003ooooo`03o`000?oooooooooo00So00002oooool000[ooooo1ol000000ooooooo0000
o`000003ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool00oooool2o`0000Cooooo
00?o0000ooooooooool01?ooool4o`0000Oooooo00?o0000ooooooooool01Oooool00ol0003ooooo
o`000003o`0001Kooooo0_l00009ooooo`03o`000?oooooooooo00[ooooo0_l00003ooooo`06o`00
0?oooooo0000o`000?oooooo00000oooool2o`000004ooooool0003o0000o`0000Sooooo00?o0000
ooooool000001ol0000:ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0_l0
0002ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003oooooooooo`03ooooo`05o`00
0?oooooo0000ooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`05o`000?oooooooooooooo
ool000001?ooool7o`0000_ooooo000;ooooo`Go00000_ooool2o`0000?ooooo0_l00005ooooo`03
o`000?oooooooooo00Cooooo0_l00002ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00_ooool4o`0000Cooooo00?o0000ooooooooool00_ooool4o`000003ooooool0003ooooo01Oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0:ooooo`;o00001?ooool2o`000004
ooooool0003oooooo`0000?ooooo1Ol0000:ooooo`03o`000?oooooooooo00Kooooo2?l00002oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_l00003ooooo`05o`000?oooooooooo
ooooool000001?ooool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooo`000?oooooo0000
0oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooool00000
7Oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool00oooool6o`0000co
oooo000<ooooo`?o00001?ooool01?l0003oooooooooooooool2o`0000Kooooo00?o0000oooooooo
ool01Oooool3o`0000Gooooo00?o0000ooooooooool01_ooool4o`000003ooooool0003o000000;o
00000oooool01?l0003oooooo`000?l0000Fooooo`;o00002?ooool00ol0003oooooooooo`0;oooo
o`;o00001?ooool2o`000004ooooool0003oooooooooo`Co000000?oooooo`000?l000003?ooool0
0ol0003oooooooooo`0=ooooo`So00000_ooool01Ol0003oooooo`000?oooooo000000?ooooo00Go
0000ooooooooooooooooo`000005ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooool0
00000oooool01Ol0003ooooooooooooooooo000000gooooo00Go0000ooooooooooooooooo`00000L
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00
0002o`0000gooooo000?ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Oooooo0ol00003ooooo`03o`000?oooooooooo00Sooooo1_l0
0005ooooo`03o`000?oooooo000001Kooooo0_l00007ooooo`03o`000?oooooooooo00_ooooo00?o
0000ooooool000000_ooool2o`000005ooooool0003oooooo`000?ooool00ol00002ooooo`04o`00
0?oooooo0000o`0000cooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`03ooooo`So
00000_ooool01Ol0003ooooooooooooooooo000000Kooooo0_l00005ooooo`;o00000oooool01?l0
003oooooooooool0000=ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool06_ooool0
1_l0003oooooo`000?l0003oooooo`0000Oooooo00?o0000ooooool000003oooool000_ooooo0ol0
00000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00;o00002?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00?o00001oooool4o`0000Cooooo00?o0000ooooool000000ol0
00001Ooooooo0000oooooooooooo000001Cooooo0_l00007ooooo`03o`000?oooooooooo00_ooooo
00Ko0000oooooooooooo0000ooooool00003ooooo`Co00000_ooool00ol0003oooooo`000002oooo
o`;o00003Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool00_l000000ooooooo0000o`000006o`0000Gooooo00?o0000ooooooooool01?ooool2o`00
00?ooooo00?o0000ooooool000003Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
01Wooooo00Ko0000ooooool0003o0000ooooool00007ooooo`03o`000?oooooo000000;o00003Ooo
ool000[ooooo1Ol000001_oooooo0000oooooooooooo0000ooooo`;o00002Oooool00ol0003ooooo
ooooo`07ooooo`04o`000?ooooooooooooooo`?o000000?oooooo`000?l000000_l00008ooooo`03
o`000?oooooooooo00?ooooo1Ol0000Dooooo`;o00001_ooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00;o00000oooool2o`000007ooooool0003oooooooooool0003oooooo`000004
ooooo`;o00003?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00?ooooo0_l00002
ooooo`;o00000oooool01_l0003ooooooooooooooooo0000ooooo`So00001Oooool01?l0003ooooo
ooooooooool2o`0000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Hooooo`?o
000000?oooooo`000?ooool01oooool6o`0000cooooo0009ooooo`So00000_ooool2o`000003oooo
ool0003ooooo00Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`06o`0000cooooo
00?o0000ooooooooool00oooool00ol0003oooooo`000004o`00013ooooo0ol00005ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool00_l000001?oooooo0000o`000?l00002ooooo`;o
00000_ooool2o`000003ooooool0003ooooo00;ooooo0_l0000=ooooo`03o`000?oooooooooo00go
oooo00?o0000ooooooooool00_ooool00ol0003oooooo`000002ooooo`;o00001?ooool01?l0003o
ooooooooool00005ooooo`03o`000?oooooooooo00Wo000000?oooooo`000?ooool03?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo01Wooooo0ol00009ooooo`Oo00002oooool000Wo
oooo2?l00002ooooo`;o000000?oooooo`000?ooool02Oooool00ol0003oooooooooo`06ooooo`Co
00001Oooool2o`0000[ooooo00?o0000ooooooooool00oooool01Ol0003oooooo`000?oooooo0000
00;ooooo0ol0000<ooooo`04o`000?oooooo0000o`0000Cooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`03o`0000Gooooo0_l000000ooooooo0000o`000002ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0<ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003oooooo`000?oooooo000000Cooooo00Co0000oooooooooooo
00001?ooool01Ol0003ooooooooooooooooo000000Gooooo0_l000000ooooooo0000o`000007o`00
00Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Gooooo`?o00002_ooool7o`00
00_ooooo0009ooooo`Oo000000?oooooo`000?ooool00_l000000ooooooo0000ooooo`0:ooooo`04
o`000?ooooooooooooooo`Go00000_ooool00ol0003oooooooooo`05ooooo`?o00002?ooool00ol0
003oooooooooo`02ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool4o`0000Sooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo000000;o00001ooo
ool2o`000003ooooool0003ooooo00?ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooool000001?ooool2o`000003ooooool0
003ooooo00?ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00?ooooo0_l00002ooooo`03o`000?oooooooooo00Cooooo2?l0000:ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooool000002_ooool8o`0000_ooooo000:ooooo`Go00000_oo
ool4o`000003ooooool0003ooooo00[ooooo1?l00004ooooo`03o`000?oooooo000000_ooooo0_l0
0006ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000Sooooo1?l00003ooooo`04o`00
0?oooooo0000o`0000?ooooo00?o0000ooooooooool03Oooool4o`000003ooooool0003ooooo00Ko
oooo0ol00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0;ooooo`;o00001Oooool2o`000003ooooool0003ooooo00?ooooo00Oo0000oooooooo
ooooooooo`000?oooooo000000Oooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooo
oooo000000[ooooo00Co0000oooooooooooooooo2?l00002ooooo`03o`000?oooooooooo01Gooooo
0ol0000:ooooo`Oo00003?ooool000_ooooo0ol00003ooooo`Co000000?oooooo`000?ooool01_oo
ool4o`000003ooooool0003ooooo00Gooooo00?o0000ooooool000003Oooool2o`0000Gooooo00Co
0000oooooooooooo00001?ooool00ol0003oooooo`00000;ooooo`Co000000?oooooo`000?l00000
0_ooool00ol0003oooooooooo`0<ooooo`?o000000Goooooo`000?ooooooooooo`000008ooooo`;o
00001_ooool01?l0003oooooooooooooool2o`0000oooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooooooo
ool2o`0000Sooooo00Co0000oooooooooooooooo0ol00002ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool02_ooool8o`00017ooooo00?o0000ooooool000000_l00009ooooo`;o0000
00Coooooo`000?l0003o00003Oooool0017ooooo1Ol00005ooooo`?o00001Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooo000000oooooo0ol00003ooooo`03o`000?oooooo000000Gooooo
0_l0000=ooooo`03o`000?oooooo000000?o000000?oooooo`000?ooool02_ooool3o`000004oooo
ool0003oooooo`0000;ooooo00?o0000ooooooooool01_ooool2o`0000Sooooo00Go0000oooooooo
ooooooooo`00000?ooooo`03o`000?oooooooooo00Wooooo0_l00006ooooo`05o`000?oooooooooo
ooooool000001?ooool01?l0003oooooooooool0000:ooooo`04o`000?oooooo0000ooooo`;o0000
0_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00oooooo00Co0000oooooooooooo
oooo2?l00009ooooo`;o000000?oooooo`000?l000002Oooool4o`0000oooooo000<ooooo`?o0000
00Coooooo`000?oooooooooo0ol000000ooooooo0000o`000002o`0000Wooooo00?o0000oooooooo
ool00oooool2o`0001?ooooo0_l000001?oooooo0000ooooool00006ooooo`;o00003?ooool01?l0
003oooooo`000?l00002ooooo`Co00001oooool2o`0000?ooooo00?o0000ooooool000000oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooo000000Sooooo00Co0000oooooooooooooooo0_l0
000?ooooo`03o`000?oooooooooo00Oooooo00Co0000oooooooooooo00001Oooool01Ol0003ooooo
oooooooooooo000000Cooooo00Co0000ooooool0003o00002oooool00ol0003oooooooooo`02o`00
00;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo
00Wooooo2_l000001?oooooo0000ooooool00009ooooo`03o`000?oooooo000000?o00003Oooool0
00_ooooo1Ol00003ooooo`Co00003Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
01?ooooo1?l00006ooooo`;o00002oooool00ol0003oooooooooo`02o`000003ooooool0003ooooo
00?ooooo1ol00005ooooo`;o00001?ooool00ol0003oooooooooo`04ooooo`04o`000?oooooo0000
o`0000Wooooo00Ko0000oooooooooooo0000ooooool0000?ooooo`03o`000?oooooooooo00Gooooo
00Go0000ooooooooooooooooo`000004ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`;o
000000?oooooo`000?ooool02Oooool2o`0000;ooooo00Go0000ooooool0003oooooo`000006oooo
o`03o`000?oooooooooo01;ooooo00?o0000ooooooooool03oooool9o`0000Oooooo00?o0000oooo
ool000001?l0000<ooooo`002_ooool<o`0000oooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`0Eooooo`?o00001_ooool2o`0000Wooooo00Co0000oooooooooooooooo0ol00007oooo
o`Ko00000oooool2o`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03o`00
00Wooooo00?o0000ooooooooool00_l0000@ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`02ooooo`;o00000oooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000Wooooo00[o0000oooooooooooo0000ooooool0003oooooo`000?oo
oooo00001Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00kooooo00Oo0000oooo
ooooooooooooo`000?oooooo000000;ooooo3ol0000;ooooo`002_ooool<o`0000oooooo00?o0000
ooooooooool00_ooool2o`0001Sooooo00Co0000ooooool0003o00001Oooool00ol0003ooooooooo
o`07ooooo`06o`000?ooooooooooo`000?oooooo00001Oooool3o`0000Kooooo1?l00006ooooo`03
o`000?oooooooooo00;ooooo00Go0000oooooooooooo0000ooooo`02o`0000Wooooo00Ko0000oooo
oooooooo0000ooooool0000@ooooo`05o`000?ooooooooooooooool000001Oooool00ol0003ooooo
ooooo`02ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000So
oooo00?o0000ooooooooool00_ooool2o`0000;ooooo0_l00004ooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool03Oooool2o`0000;ooooo00Go0000ooooooooooooooooo`000008oooo
o`So00002oooool000[ooooo5?l00008ooooo`04o`000?ooooooooooooooo`;o00005oooool2o`00
00?ooooo0_l00003ooooo`03o`000?oooooooooo00Kooooo00Co0000oooooooooooooooo0_l00004
ooooo`;o00002oooool00ol0003oooooo`000003o`0000;ooooo00?o0000ooooooooool00_l00003
ooooo`04o`000?oooooo0000o`0000[ooooo00?o0000ooooooooool00_l0000Aooooo`03o`000?oo
oooo000000Oooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00003ooooo`05
o`000?oooooo0000ooooool000001oooool00ol0003oooooooooo`03ooooo`;o00000_ooool00ol0
003oooooo`000002ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool03?ooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01Oooool9o`0000_ooooo000;oooo
o`Go00000oooool4o`0000?ooooo3ol000000ooooooo0000o`00000Gooooo`03o`000?oooooo0000
00Cooooo0ol000000ooooooo0000ooooo`06ooooo`04o`000?ooooooooooooooo`;o000000Cooooo
o`000?l0003o00003?ooool00ol0003oooooooooo`04ooooo`Go00001?ooool00ol0003ooooooooo
o`02o`0000_ooooo00Go0000ooooool0003oooooo`00000@ooooo`;o00002?ooool01?l0003ooooo
ooooool00002ooooo`04o`000?ooooooooooo`0000;ooooo00Ko0000oooooooooooo0000ooooool0
0006ooooo`03o`000?oooooooooo00Gooooo00So0000oooooooooooo0000ooooool0003oooooo`00
01_ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`02ooooo`05o`000?oooooooooo
ooooool000001_ooool9o`0000cooooo000<ooooo`?o00001?ooool2o`000003ooooool0003o0000
00;o00002?oooolAo`00013ooooo00?o0000ooooool000001oooool3o`0000Gooooo00Ko0000oooo
ooooooooooooo`000?ooool2o`0000kooooo0_l00007ooooo`03o`000?oooooo000000?o00000_oo
ool01_l0003oooooooooool0003oooooo`0000[ooooo00?o0000ooooool000000_ooool00ol0003o
ooooooooo`0=ooooo`;o00002?ooool01?l0003oooooooooool00002ooooo`06o`000?oooooooooo
o`000?oooooo00000oooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00Kooooo0_l0
0002ooooo`?o00007Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooooooool3o`000003ooooool0003o000000;ooooo
0ol0000=ooooo`004oooool2o`000003ooooool0003ooooo00;ooooo0ol0000:ooooo`03o`000?oo
oooo000000So000000?oooooo`000?l000002Ol00005ooooo`04o`000?ooooooooooo`0000Sooooo
0_l00003ooooo`04o`000?ooooooooooooooo`Co00003Oooool3o`0000Sooooo0_l00004ooooo`Co
000000Coooooo`000?oooooo00002oooool01Ol0003oooooo`000?oooooo000000kooooo00Co0000
oooooooooooo00002?ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00;o00001?oo
ool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Oooooo0_l00002ooooo`;o0000
7_ooool00ol0003oooooooooo`07ooooo`;o00001?ooool00ol0003oooooooooo`03ooooo`?o0000
0oooool3o`0001;ooooo000=ooooo`?o00000oooool2o`000003ooooool0003o000000Gooooo1?l0
0007ooooo`05o`000?ooooooooooooooool000001Oooool8o`0000Cooooo2ol00006ooooo`Ko0000
00Coooooo`000?l0003ooooo0_l0000=ooooo`03o`000?oooooo000000Wooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`04o`0000_ooooo00?o0000ooooool000000_ooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool2o`0000?o
oooo00?o0000ooooooooool00_l00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooo
ooooool02?ooool01?l0003oooooo`000?ooool2o`0001oooooo00?o0000ooooooooool01Oooool2
o`0000Gooooo00Co0000oooooooooooooooo0ol00005ooooo`?o000000Coooooo`000?l0003o0000
3_ooool000cooooo1Ol00002ooooo`;o000000?oooooo`000?l000002Oooool3o`0000Cooooo00Go
0000ooooooooooooooooo`00000=ooooo`So000000?oooooo`000?ooool00_ooool00ol0003ooooo
o`00000=o`000004ooooool0003oooooo`0000cooooo00Co0000oooooooooooo00002?ooool2o`00
00Gooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004o`0000Sooooo0_l00002oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04oooo
o`;o00000oooool01Ol0003oooooo`000?oooooo000000Gooooo00Go0000ooooool0003oooooo`00
000;ooooo`06o`000?ooooooooooo`000?oooooo00008?ooool00ol0003oooooooooo`04ooooo`;o
00001?ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool00oooool8o`0000go
oooo000;ooooo`Oo000000Coooooo`000?l0003ooooo0_l0000<ooooo`Ko00000_ooool00ol0003o
ooooooooo`0Cooooo`So00001_ooool>o`0000Sooooo00Co0000oooooooooooo00002?ooool2o`00
00Kooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`Co00001Oooool2
o`0000;ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00?ooooo0_l00003ooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`04o`000?oooooo
0000o`0000cooooo00Co0000ooooool0003ooooo0_l0000Pooooo`03o`000?oooooooooo00?ooooo
0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`03ooooo`Wo00003?ooool000_ooooo1ol0
00000ooooooo0000o`000002ooooo`;o00003_ooool4o`000003ooooool0003ooooo01Cooooo00?o
0000ooooooooool00oooool00ol0003oooooo`000008o`000003ooooool0003ooooo00;ooooo0ol0
0003ooooo`_o00002?ooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00?ooooo00Co
0000oooooooooooo00001oooool4o`000003ooooool0003o000000?ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00?ooooo0_l00003ooooo`;o0000
0_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000gooooo00Co0000ooooool0003o
oooo0ol0000Pooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`?o00001_ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooo000000Oo00003?ooool000_ooooo1ol000000ooo
oooo0000ooooo`02ooooo`;o00003_ooool00ol0003oooooooooo`04o`0001Cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`02o`0000?ooooo2Ol000000ooooooo0000o`000007oooo
o`04o`000?ooooooooooooooo`co00001Oooool00ol0003oooooooooo`03ooooo`04o`000?oooooo
ooooo`0000_ooooo1?l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`02ooooo`;o00000oooool2o`0000;ooooo00?o0000ooooooooool00ooo
ool3o`0000gooooo0_l00003ooooo`;o00008Oooool01_l0003oooooo`000?oooooo0000ooooo`?o
000000?oooooo`000?ooool01oooool01?l0003oooooooooooooool:o`0000cooooo000<ooooo`Go
00000_ooool01ol0003ooooooooooooooooo0000ooooool000003Oooool01ol0003oooooooooool0
003oooooo`000?ooool01?l0000@ooooo`03o`000?oooooooooo00Cooooo0ol00005ooooo`;o0000
00Koooooo`000?ooooooooooo`000?ooool8o`0000;ooooo00Go0000ooooooooooooooooo`000007
ooooo`04o`000?ooooooooooooooo`_o000000Goooooo`000?ooooooooooo`00000>ooooo`Go0000
1?ooool00ol0003oooooooooo`0;ooooo`04o`000?ooooooooooooooo`;o00000_ooool00ol0003o
ooooo`000002ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000003Oooool00ol0003o
ooooooooo`03ooooo`;o00008?ooool2o`000004ooooool0003o0000o`0000Cooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`04o`000003ooooool0003o000000?o00003Oooool000go
oooo0ol00003ooooo`07o`000?ooooooooooooooool0003oooooo`00000<ooooo`03o`000?oooooo
oooo00;ooooo0_l00005ooooo`?o00003Oooool00ol0003oooooooooo`02ooooo`;o000000?ooooo
o`000?ooool01?ooool3o`000007ooooool0003oooooooooool0003oooooo`000005ooooo`[o0000
1Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
0oooool;o`0000Sooooo00Co0000oooooooooooooooo1Ol0000?ooooo`07o`000?oooooo0000oooo
ool0003oooooo`000002ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000ooooool0003o0000
3?ooool2o`0000Gooooo0_l0000Pooooo`Co00001_ooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Co000000?oooooo`000?l000000_l0000>ooooo`004oooool01Ol0003ooooooooo
oooooooo000000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`;o0000
2?ooool4o`0000Wooooo00?o0000ooooool000000_l00003ooooo`03o`000?oooooooooo00?ooooo
00Go0000oooooooooooo0000ooooo`02o`000003ooooool0003ooooo00Oooooo0ol00002ooooo`03
o`000?oooooooooo00So000000?oooooo`000?ooool00oooool00ol0003oooooooooo`05ooooo`05
o`000?ooooooooooooooool000001Oooool:o`0000?ooooo00Co0000ooooool0003ooooo1?l0000<
ooooo`;o00000_ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000002ooooo`;o00002oooool00ol0003oooooo`000005ooooo`;o00007oooool2o`000003oooo
ool0003ooooo00Gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02o`000003oooo
ool0003o000001;ooooo000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000002ooo
ool00ol0003oooooooooo`03ooooo`;o00002oooool3o`0000Kooooo0_l00007ooooo`04o`000?oo
ooooooooooooo`;o00000_ooool01_l0003oooooooooool0003oooooo`0000Oooooo00Co0000oooo
oooooooooooo0_l00006ooooo`03o`000?oooooooooo00;ooooo2?l00006ooooo`05o`000?oooooo
ooooooooool000003_ooool?o`0000Wooooo0_l000000ooooooo0000o`000003ooooo`04o`000?oo
ooooooooo`0000;ooooo00Co0000oooooooooooooooo0_l0000:ooooo`04o`000?ooooooooooo`00
00Gooooo00?o0000ooooool000006oooool4o`0000;ooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`08ooooo`Co000000?oooooo`000?ooool04_ooool000kooooo0ol00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`04oooo
o`;o00003_ooool6o`0000Wooooo00?o0000ooooooooool00_l00004ooooo`05o`000?oooooo0000
ooooool000001_ooool00ol0003oooooooooo`03ooooo`?o00000oooool00ol0003oooooooooo`04
ooooo`05o`000?ooooooooooooooool000000_ooool8o`0000;ooooo00?o0000ooooooooool03?oo
ool2o`0000;ooooo0_l00006ooooo`_o000000Koooooo`000?l0003o0000ooooool00003ooooo`06
o`000?ooooooooooo`000?oooooo00001?ooool00ol0003oooooo`000008ooooo`05o`000?oooooo
ooooooooool000001Oooool00ol0003oooooo`00000Hooooo`?o00000_ooool01_l0003ooooooooo
ool0003oooooo`0000Gooooo00?o0000ooooooooool02?ooool3o`000004ooooool0003o0000oooo
o`?o00003oooool000gooooo1ol00004ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0=ooooo`;o000000Coooooo`000?l0003o00002Oooool0
0ol0003oooooo`000005ooooo`;o000000?oooooo`000?l000001Oooool00ol0003oooooooooo`03
ooooo`04o`000?ooooooooooooooo`?o00001oooool01Ol0003ooooooooooooooooo000000Sooooo
00?o0000ooooool000001ol0000:ooooo`?o00000_ooool00ol0003oooooooooo`0:ooooo`ko0000
0oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0001Gooooo0_l00004ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00000;ooooo`?o000000?ooooo
o`000?l000001Ol0000>ooooo`003?ooool8o`0000Cooooo00Go0000ooooooooooooooooo`000008
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02_ooool3o`0000Cooooo00?o0000
ooooool000000_l00007ooooo`03o`000?oooooooooo00Gooooo00Co0000ooooool0003o00001?oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l000000ooooooo0000o`00
0005ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`02ooooo`So00000_ooool2o`0000?ooooo00?o0000ooooooooool03_ooool5o`000003oooo
ool0003ooooo00;ooooo3?l00006ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`02o`00017ooooo0ol00005ooooo`05o`000?ooooooooooooooool0
00000oooool01Ol0003ooooooooooooooooo000000_ooooo1?l000000ooooooo0000o`000005o`00
00gooooo000<ooooo`So00001?ooool01Ol0003ooooooooooooooooo000000Oooooo00?o0000oooo
ooooool01_ooool2o`0000Wooooo0_l00007ooooo`03o`000?oooooooooo00;ooooo1?l00002oooo
o`;o00001oooool01?l0003oooooo`000?l00003ooooo`03o`000?oooooooooo00Cooooo00Go0000
ooooooooooooooooo`000005ooooo`;o00000_ooool01Ol0003ooooooooooooooooo000000Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`0:ooooo`Wo00003oooool2o`0000;ooooo
1?l00002ooooo`;o000000?oooooo`000?ooool01Oooool<o`0000;ooooo00?o0000ooooooooool0
1?ooool01?l0003oooooo`000?l0000?ooooo`;o00001oooool01Ol0003ooooooooooooooooo0000
00Gooooo00Co0000oooooooooooo00002_ooool01?l0003oooooo`000?l00002ooooo`Oo00003Ooo
ool000cooooo2?l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`06ooooo`;o00001_ooool3o`0000Wooooo00?o0000ooooooooool01_ooool3
o`000003ooooool0003ooooo00Gooooo0ol000001?oooooo0000ooooool00006ooooo`05o`000?oo
ooooooooooooool000002?ooool3o`0000;ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00[ooooo00Go0000ooooool0003oooooo`000002ooooo`So
00002?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000?o000000?oooooo`000?oo
ool01Oooool2o`0000Cooooo00?o0000ooooooooool00oooool:o`0000;ooooo0_l0000;ooooo`?o
00002Oooool01Ol0003ooooooooooooooooo000000Kooooo0_l0000;ooooo`03o`000?oooooo0000
00?ooooo1ol0000=ooooo`003Oooool7o`0000Gooooo00Go0000ooooooooooooooooo`000006oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooool000000oooool2o`0000cooooo00?o0000oooo
ooooool01_ooool2o`000003ooooool0003o000000;o00000oooool01?l0003oooooo`000?l00002
ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`00000;ooooo`Co00002_oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Wooooo00Ko0000oooooooooooo0000
ooooool00003ooooo`03o`000?oooooooooo00Cooooo2Ol00005ooooo`03o`000?oooooooooo00;o
000000?oooooo`000?l000000_l00004ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Cooooo3?l00002ooooo`;o00002oooool01Ol0003ooooooooooooooooo0000
00Oooooo0_l0000;ooooo`?o00000oooool6o`0000kooooo000>ooooo`?o000000?oooooo`000?l0
00001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool01oooool00ol0003oooooo`000003o`0000gooooo00?o0000ooooooooool01_ooool00ol0003o
ooooo`000003ooooo`03o`000?oooooo000000?o000000?oooooo`000?ooool00ol00006ooooo`04
o`000?ooooooooooo`0000cooooo00?o0000ooooool000000_l00009ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`00000;ooooo`_o000000Goooooo`000?ooooooooooo`000002ooooo`Ko0000
0_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo
oooo0_l00005ooooo`co00000oooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000oooo
ooooool02?ooool01?l0003oooooo`000?l00003ooooo`03o`000?oooooo000000;o00003oooool0
01;ooooo0_l00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`07ooooo`Co00003oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo0000
00Cooooo00?o0000ooooooooool00_ooool4o`0000?ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool0000=ooooo`03o`000?oooooo000000;ooooo0ol00006ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`03ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool02?ooool00ol0003oooooo`000007ooooo`Wo00001?oo
ool4o`0000[ooooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool0
00000_ooool2o`0000cooooo2_l00006ooooo`03o`000?oooooo000000Wooooo1?l00003ooooo`03
o`000?oooooooooo017ooooo000@ooooo`Co00001Oooool00ol0003oooooooooo`02ooooo`05o`00
0?ooooooooooooooool000002?ooool2o`000004ooooool0003oooooo`0000oooooo00?o0000oooo
ooooool01?ooool01?l0003oooooooooool00005ooooo`04o`000?ooooooooooo`0000;ooooo1Ol0
00000ooooooo0000ooooo`02ooooo`03o`000?oooooo000000gooooo00?o0000ooooool000001_oo
ool2o`0000Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`06ooooo`04o`00
0?ooooooooooo`0000Kooooo0_l000000ooooooo0000o`000002ooooo`03o`000?oooooooooo00co
00001_ooool00ol0003oooooooooo`04ooooo`06o`000?ooooooooooooooool0003ooooo0ol0000?
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooooooo2ol00006ooooo`Co000000Co
ooooo`000?oooooooooo0ol0000Aooooo`003oooool5o`0000Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooool00005ooooo`?o00000oooool01?l0003oooooooooool0000>ooooo`03
o`000?oooooooooo00?ooooo00Co0000oooooooooooo00001_ooool01Ol0003oooooooooool0003o
oooo00;o000000Coooooo`000?oooooooooo0ol000001?oooooo0000ooooool0000>ooooo`03o`00
0?oooooo000000Sooooo0_l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003o
ooooooooo`05ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooool000000_ooool01?l0
003oooooooooool00007ooooo`;o000000?oooooo`000?l000001_l00002ooooo`03o`000?oooooo
oooo00Cooooo00Co0000oooooooooooooooo0_l000000ooooooo0000ooooo`0>ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01oooool01Ol0003oooooooooool0003ooooo00co0000
00?oooooo`000?l000000ol0000@ooooo`003_ooool7o`0000Gooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00003ooooo`;o00001_ooool01?l0003oooooooooool0000>ooooo`03
o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooo0000
00;o00000_ooool00ol0003oooooooooo`02ooooo`Go00003_ooool2o`0000_ooooo0ol00006oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01oooool01Ol0003ooooooooooooooooo
000000;ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Gooooo00Go
0000ooooooooooooooooo`000002ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool0
2?ooool7o`0000;ooooo1?l000001Ooooooo0000oooooooooooo000000kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00oooool00ol0003oooooo`00000;o`0000oooooo000>ooooo`Oo00001Oooool00ol0003ooooo
ooooo`03ooooo`;o000000Coooooo`000?l0003o00002?ooool01Ol0003ooooooooooooooooo0000
00gooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00Kooooo0ol00000
0ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`?o00002oooool2
o`0000gooooo0ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02?ooool0
1Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003oooooo`00
00Kooooo0ol0000<ooooo`go00000_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool01_oooolBo`000003ooooool0003o000000Ko00003oooool000ko
oooo2Ol00003ooooo`03o`000?oooooooooo00?ooooo0ol0000;ooooo`05o`000?oooooooooooooo
ool000003Oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Sooooo00Co0000
oooooooooooo00000oooool01?l0003oooooooooooooool2o`0000Kooooo1?l00007ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool00_l00003ooooo`04o`000?ooooooooooo`0000_o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o000000?ooooo
o`000?ooool00oooool01?l0003oooooo`000?l0000<ooooo`03o`000?oooooo000000Go00001?oo
ool8o`0000Kooooo6Ol00003ooooo`;o000000?oooooo`000?ooool00oooool7o`0000oooooo000?
ooooo`Go00000_ooool2o`0000;ooooo00Co0000oooooooooooooooo1?l0000<ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02Oooool01?l0003oooooooooool00004ooooo`03o`00
0?oooooooooo00Oooooo0ol000000ooooooo0000ooooo`03ooooo`06o`000?oooooo0000o`000?oo
oooo00002Oooool3o`0000Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`02oooo
o`Co000000?oooooo`000?ooool03?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooool000001?ooool01Ol0003ooooooooooooooooo000000Kooooo00Go0000oooooooooooo0000
ooooo`02o`0000?ooooo00Go0000ooooooooooooooooo`00000<ooooo`;o00001OoooolEo`000003
ooooool0003o000000Wo00000oooool00ol0003oooooooooo`05ooooo`04o`000?oooooo0000o`00
00;ooooo00?o0000ooooooooool00_ooool6o`00013ooooo000@ooooo`Go00000_ooool4o`000003
ooooool0003o000000;ooooo0_l0000<ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00Oooooo0_l000000ooooooo
0000o`000005ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool02Oooool4o`0000oo
oooo00?o0000ooooooooool01Oooool2o`0000kooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`02o`0000Gooooo00Co0000oooooooooooo00002?ooool00ol0003oooooo`000002oooo
o`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool01_ooool?o`000003ooooool0003o
000000;o000000?oooooo`000?ooool04Ol00007ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooool000000_ooool00ol0003oooooooooo`02ooooo`Go00004Oooool001;ooooo1?l00003oooo
o`?o00001?ooool00ol0003oooooo`00000<ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool02?ooool2o`0000Gooooo00?o0000ooooooooool01_ooool00ol0003oooooo`000002oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<oooo
o`Co00003?ooool00ol0003oooooooooo`05ooooo`04o`000?oooooo0000o`0000gooooo00?o0000
ooooooooool00_ooool01?l0003oooooo`000?l00004ooooo`04o`000?ooooooooooo`0000Wooooo
00?o0000ooooool000000_oooolBo`0000?ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
o`00000:o`0000Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`So0000
1oooool3o`0000;ooooo00?o0000ooooooooool00oooool3o`0001?ooooo000Aooooo`So000000?o
ooooo`000?l000000_l00002ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`08ooooo`;o00001?ooool00ol0003oooooooooo`06ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000ooooooooool0
2_ooool2o`0000?ooooo0ol00009ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool00000
0_ooool3o`0000[ooooo00?o0000ooooooooool00_ooool2o`0000;ooooo00So0000oooooooooooo
ooooo`000?oooooo0000oooooa7o00001Oooool00ol0003oooooooooo`02ooooo`?o00001?ooool:
o`0000Gooooo00Ko0000ooooooooooooooooo`000?ooool5o`0000Gooooo00?o0000ooooooooool0
2_ooool9o`000005ooooool0003oooooooooool000001?ooool5o`0001;ooooo000@ooooo`So0000
0oooool2o`000003ooooool0003o000000?ooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`06ooooo`04o`000?ooooooooooo`0000?ooooo0_l00002ooooo`05o`000?oooooo0000oooo
ool000001Oooool00ol0003oooooooooo`09ooooo`;o00001_ooool4o`0000Gooooo00?o0000oooo
ooooool00oooool01?l0003oooooooooool00005ooooo`;o00002?oooolAo`0000Wooooo00?o0000
ooooool000000_ooool00ol0003oooooo`000005ooooo`03o`000?oooooo000000Wo000000?ooooo
o`000?ooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooooooo1Ol00002ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`0?o`00017ooooo000@ooooo`Oo000000Soooooo`000?ooooooooooo`00
0?oooooo0000ooooo`;o00000_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool01_ooool2o`0000Cooooo00?o0000ooooooooool01Oooool01Ol0003o
oooooooooooooooo000000;ooooo00Oo0000ooooool0003oooooo`000?oooooo000000;ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo000000[ooooo0ol0
0002ooooo`03o`000?oooooooooo00;ooooo00Ko0000ooooooooooooooooo`000?ooool@o`0000;o
oooo00Co0000oooooooooooo00000_ooool01?l0003oooooo`000?l0000=ooooo`03o`000?oooooo
000000co00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool00oooool00ol0003oooooo`000005ooooo`Co00002_ooool00ol0003oooooooooo`04
ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`Oo00004Oooool0013ooooo1ol00002
ooooo`03o`000?oooooo000000;ooooo0_l000001Ooooooo0000o`000?oooooo000000[ooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`;o00001?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003ooooo
ooooool00003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01oooool2o`0000_o
oooo4?l00008ooooo`?o00001?ooool00ol0003oooooo`000003ooooo`06o`000?ooooooooooo`00
0?oooooo00001oooool<o`0000?ooooo00?o0000ooooool000000ol0000;ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`;o00001oooool0
0ol0003oooooooooo`04o`0000Kooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooo
oooo000000;ooooo00?o0000ooooooooool00_ooool7o`00017ooooo000Aooooo`Go00001?ooool0
1ol0003oooooo`000?oooooo0000ooooool000000_ooool2o`0000[ooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooooooool0003ooooo0_l000000ooo
oooo0000ooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool?o`00
00Gooooo00?o0000ooooool000000_l00005ooooo`03o`000?oooooooooo00Wooooo0_l00002oooo
o`03o`000?oooooo000000Cooooo00?o0000ooooool000002Ol00007ooooo`?o000000Coooooo`00
0?oooooooooo0ol00002ooooo`;o00002oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool00_ooool2o`0000Kooooo00?o0000ooooooooool01Oooool4
o`000003ooooool0003ooooo00Cooooo00Go0000ooooooooooooooooo`000002ooooo`?o00000ooo
ool6o`0001;ooooo000Booooo`?o00001_ooool2o`000004ooooool0003oooooooooo`;o000000Co
ooooo`000?l0003o00002?ooool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000Kooooo00?o0000ooooooooool00oooool01_l0
003oooooooooool0003oooooo`0000;ooooo00Co0000oooooooooooooooo4Ol00002ooooo`03o`00
0?oooooooooo00kooooo00?o0000ooooool000000_ooool3o`0000;ooooo00?o0000ooooooooool0
2oooool>o`0000kooooo0ol000001?oooooo0000o`000?l00003ooooo`04o`000?ooooooooooo`00
00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`07o`000?oooooooooo
ooooool0003oooooo`000004ooooo`03o`000?oooooooooo00[ooooo0ol00003ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool00ol00003ooooo`03o`000?oooooo000000;o00004ooo
ool001cooooo0_l00005ooooo`04o`000?oooooo0000ooooo`;o00001_ooool00ol0003ooooooooo
o`05ooooo`04o`000?ooooooooooo`0000?ooooo00Co0000oooooooooooo00001_ooool01?l0003o
ooooooooooooool?o`0000?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`00013ooooo0_l00006ooooo`Co00000oooool:o`000004ooooool0003o
0000o`0000Gooooo00Co0000ooooool0003o00003Oooool00ol0003oooooo`000004o`0000Cooooo
00Co0000oooooooooooo00002oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00_ooooo00?o0000ooooool000000ol00003ooooo`05o`000?ooooooooooooooool000000_ooool0
0ol0003oooooo`00000Hooooo`005Oooool3o`0000Cooooo0_l00006ooooo`;o00000oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo0_l00003ooooo`oo00000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`04
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0>ooooo`03o`000?oo
oooooooo00;ooooo2_l0000:ooooo`04o`000?oooooo0000ooooo`;o00000_ooool01_l0003ooooo
o`000?l0003oooooo`0000_ooooo00?o0000ooooool000000_l00002ooooo`03o`000?oooooooooo
00?ooooo00Co0000ooooool0003o00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooool000001?ooool01?l0003oooooooooool0000=ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooool000000ol00002ooooo`;o00000oooool4o`0001Kooooo000D
ooooo`Go00000oooool3o`0000Oooooo00Co0000oooooooooooooooo0_l000000ooooooo0000o`00
000>o`000004ooooool0003oooooo`0000Cooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`02o`0000;ooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00Co
oooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00[ooooo2_l00006oooo
o`04o`000?ooooooooooooooo`Co00001Oooool01Ol0003ooooooooooooooooo000000;ooooo0_l0
0002ooooo`04o`000?ooooooooooo`0000[ooooo0ol000000ooooooo0000o`000002ooooo`03o`00
0?oooooooooo00Cooooo0ol0000:ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000gooooo00Go0000oooooooooooo
ooooo`000004ooooo`03o`000?oooooo000000?o00000oooool5o`0001Gooooo000Cooooo`Oo0000
0_oooolBo`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`000004ooooo`03o`00
0?oooooo000000?ooooo00?o0000ooooooooool01oooool01?l0003oooooo`000?l00003ooooo`05
o`000?ooooooooooooooool000001Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo0000
00?ooooo00?o0000ooooool000002_l00003ooooo`03o`000?oooooo000000[ooooo00?o0000oooo
ooooool01Oooool3o`000003ooooool0003ooooo00?ooooo00Go0000oooooooooooo0000ooooo`04
o`0000;ooooo00?o0000ooooooooool01Oooool3o`0000Cooooo00Co0000oooooooooooo00001ooo
ool2o`0000[ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`;o00001_oo
ool2o`0000kooooo00Co0000oooooooooooo00001Oooool01?l0003oooooooooooooool;o`0001Co
oooo000Cooooo`_o000000?oooooo`000?ooool01_ooool00ol0003oooooo`000004ooooo`?o0000
1Oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool000000oooool00ol0003o
ooooooooo`07ooooo`05o`000?oooooo0000ooooool000000_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool:o`0000gooooo00Co0000oooo
oooooooo00002_ooool00ol0003oooooooooo`08ooooo`Co00000oooool2o`0000Cooooo0ol00002
ooooo`04o`000?ooooooooooooooo`?o000000?oooooo`000?ooool01?ooool2o`000003ooooool0
003ooooo00Kooooo0_l0000:ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooool00005ooooo`;o00003Oooool01?l0003oooooooooool00005ooooo`04
o`000?ooooooooooooooo`;o00000oooool7o`0001Cooooo000Cooooo`Oo00000_ooool2o`0000;o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02o`0000Cooooo0_l00003ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool00_l00003ooooo`03o`000?oooooooooo00So
oooo0_l00002ooooo`03o`000?oooooo000000Kooooo2ol00003ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02oooool01Ol0003ooooooooooooooooo000000[ooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`02ooooo`Co00001?ooool00ol0003oooooooooo`04o`00
0003ooooool0003o000000?ooooo00?o0000ooooooooool01?ooool2o`000003ooooool0003ooooo
00Kooooo0ol00009ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool01?l0
003oooooooooool00005ooooo`03o`000?oooooo000000cooooo00?o0000ooooool000001_ooool0
1?l0003oooooooooooooool<o`0001Cooooo000Dooooo`Go00000_ooool00ol0003oooooo`000003
ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oo
oooo0000ooooool000001_ooool01?l0003oooooo`000?l00002ooooo`03o`000?oooooooooo00Wo
oooo00Co0000oooooooooooooooo2ol00005ooooo`03o`000?oooooooooo00Cooooo0_l00003oooo
o`03o`000?oooooooooo00[ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool4o`0000;ooooo00Co0000oooo
oooooooooooo1?l00004ooooo`03o`000?oooooooooo00Gooooo0_l000000ooooooo0000ooooo`06
ooooo`03o`000?oooooo000000Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02
ooooo`05o`000?ooooooooooooooool000001?ooool01?l0003oooooooooool0000<ooooo`;o0000
1oooool6o`0000;ooooo1ol0000Eooooo`005Oooool3o`0000?ooooo00?o0000ooooool000001?oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l000001?oooooo0000oooo
ooooool2o`0000Kooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`04ooooo`[o0000
00?oooooo`000?ooool01oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo
00Ko0000oooooooooooo0000ooooool0000<ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
oooooooo0000ooooo`08o`000003ooooool0003o000000?o00002Oooool00ol0003oooooo`000007
ooooo`04o`000?oooooo0000o`0000Sooooo00?o0000ooooooooool01_ooool01Ol0003ooooooooo
oooooooo000000Gooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooool000000oooool5o`0000;ooooo0_l00002ooooo`;o00000_ooool3o`0001Ko
oooo000Kooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`03ooooo`;o000000Coooooo`000?oooooooooo0_l00005ooooo`_o00001?ooool2o`0000?o
oooo00?o0000ooooool000002Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooooooo0_l0000;ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`Ko00000_ooool00ol0003oooooo`000002o`0000Sooooo00?o0000
ooooool000001oooool01Ol0003oooooo`000?oooooo000000Oooooo00?o0000ooooooooool01_oo
ool01?l0003oooooooooool00006ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000oooooooo
ool01oooool6o`0000?ooooo00Co0000oooooooooooooooo0_l000000ooooooo0000o`00000Loooo
o`006?ooool3o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo00Cooooo3ol0000:ooooo`;o000000?oooooo`000?ooool00_ooool2o`0000[o
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Cooooo0_l0000:oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool4o`0000Gooooo1Ol00003oooo
o`;o00001_ooool00ol0003oooooo`000008ooooo`;o00000_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooool000002?ooool00ol0003oooooo`000005oooo
o`03o`000?oooooooooo00Cooooo1Ol00006ooooo`03o`000?oooooooooo00;ooooo2?l0000Ioooo
o`005oooool5o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0;o`0000Ko
oooo1Ol0000:ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool02_ooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00[ooooo00Go0000ooooooooooooooooo`00
0006ooooo`03o`000?oooooooooo00;ooooo0ol000000ooooooo0000ooooo`06ooooo`Go00000ooo
ool3o`0000?ooooo00?o0000ooooool000002?ooool2o`0000?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`06ooooo`;o00002Oooool2o`0000Oooooo00?o0000ooooool000000ol0
0002ooooo`03o`000?oooooooooo00Kooooo00Co0000oooooooooooooooo2_l0000Hooooo`005_oo
ool7o`000003ooooool0003o000000So00000_ooool00ol0003oooooooooo`03ooooo`;o000000?o
ooooo`000?ooool01_ooool3o`0000Wooooo00Go0000ooooooooooooooooo`000003ooooo`;o0000
3Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo0_l00007ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool00006ooooo`03
o`000?oooooooooo00?o00000_ooool00ol0003oooooo`000007ooooo`05o`000?ooooooooooo`00
0?ooool00ol00003ooooo`;o000000Goooooo`000?l0003oooooo`000007ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Kooooo0_l00009ooooo`03o`000?oooooo
oooo00?ooooo1Ol00005ooooo`;o00001oooool00ol0003oooooooooo`02ooooo`?o000000?ooooo
o`000?l000001Ol0000Gooooo`005_ooool9o`0000Sooooo00Go0000ooooooooooooooooo`000003
ooooo`03o`000?oooooo000000;ooooo0_l00004ooooo`Go00001oooool00ol0003oooooooooo`02
ooooo`05o`000?ooooooooooooooool000003_ooool00ol0003oooooooooo`03ooooo`;o00001_oo
ool2o`0000Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0:ooooo`03o`000?oo
oooo000000Kooooo00Co0000ooooool0003o00001Oooool2o`0000Oooooo00Go0000ooooool0003o
ooooo`000004ooooo`Co000000Koooooo`000?l0003o0000ooooool00007ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`;o00002Oooool6
o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000007ooooo`03o`000?oooooo
oooo00Co000000?oooooo`000?l000001_l0000Gooooo`005_ooool9o`0000Wooooo00Co0000oooo
oooooooo00000_ooool01?l0003oooooooooool00004ooooo`08o`000?ooooooooooooooool0003o
ooooo`000?ooool2o`0000Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooooooool2
o`0000oooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`04o`000?oooooo
ooooo`0000Cooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0:ooooo`;o00001_oo
ool3o`0000Sooooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00Gooooo1_l00007ooooo`03o`000?oooooo000000Cooooo00Go0000oooooooooooo
ooooo`000007ooooo`03o`000?oooooo000000Oooooo1?l00009ooooo`04o`000?oooooooooooooo
o`;o00002?ooool02?l0003oooooo`000?oooooo0000ooooool0003ooooo2?l0000Gooooo`005ooo
ool5o`0000?ooooo0ol00007ooooo`05o`000?oooooo0000ooooool000000oooool00ol0003ooooo
ooooo`03ooooo`?o00000_ooool00ol0003oooooooooo`03o`0000;ooooo00?o0000ooooooooool0
0oooool01?l0003oooooooooooooool2o`0000oooooo00?o0000ooooooooool00_ooool2o`0000Go
oooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool02_ooool2o`0000Cooooo0_l0000;ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`00000:ooooo`Go00001Oooool00ol0003oooooo`000005oooo
o`04o`000?ooooooooooo`0000Kooooo00Co0000oooooooooooo00000_ooool5o`000004ooooool0
003oooooo`0000[ooooo00Go0000ooooool0003oooooo`000007ooooo`04o`000?oooooo0000oooo
o`;o000000?oooooo`000?l000000_ooool5o`0001Sooooo000Hooooo`?o00001?ooool00ol0003o
ooooooooo`03o`0000Gooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`04ooooo`;o
00000_ooool01?l0003oooooooooool00002ooooo`;o00001_ooool00ol0003oooooooooo`02o`00
017ooooo00Ko0000oooooooooooo0000ooooool00005ooooo`07o`000?ooooooooooooooool0003o
ooooo`00000=ooooo`03o`000?oooooooooo00[ooooo0_l000000ooooooo0000o`000002o`0000_o
oooo0_l00004ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000002Oooool3o`000004
ooooool0003o0000o`0000?ooooo00?o0000ooooool000001_ooool00ol0003oooooo`000005oooo
o`03o`000?oooooooooo00Co00001Oooool01Ol0003ooooooooooooooooo000000Wooooo0_l00002
ooooo`03o`000?oooooooooo00Gooooo0_l000000ooooooo0000ooooo`02o`000003ooooool0003o
oooo00;ooooo0ol0000Iooooo`007?ooool4o`0000Gooooo0ol00003ooooo`;o00001?ooool00ol0
003oooooooooo`03ooooo`;o000000?oooooo`000?l000001?ooool01Ol0003oooooo`000?oooooo
000000Cooooo00Co0000oooooooooooooooo0_l0000Aooooo`04o`000?ooooooooooo`0000;ooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool02Oooool01?l0003oooooo`000?l00002ooooo`03o`000?oooooooooo00[o
oooo00?o0000ooooool000000_ooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`00
00Sooooo0ol00004ooooo`Co000000?oooooo`000?ooool01_ooool2o`0000?ooooo1Ol000000ooo
oooo0000ooooo`05ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01oooool2o`00
0003ooooool0003ooooo00Kooooo00Wo0000oooooooooooo0000ooooool0003oooooo`000?ooool0
0ol0000Mooooo`006oooool5o`0000Sooooo0ol000000ooooooo0000o`000004ooooo`03o`000?oo
oooooooo00?o00000oooool2o`0000?ooooo0_l00003ooooo`;o00000_ooool01_l0003ooooooooo
ool0003oooooo`0001;ooooo0_l00003ooooo`03o`000?oooooooooo00;ooooo00Oo0000oooooooo
ooooooooo`000?oooooo000000gooooo00?o0000ooooooooool02?ooool4o`0000;ooooo00?o0000
ooooooooool02oooool01Ol0003oooooo`000?oooooo000000Wooooo00?o0000ooooool000002?oo
ool3o`0000Oooooo1?l00007ooooo`Co000000?oooooo`000?ooool00oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`02o`0000Gooooo0ol000001_oooooo0000oooooooooooo0000ooooo`Ko00007?ooool001[ooooo
1ol00008ooooo`03o`000?oooooo000000;o00001?ooool00ol0003oooooo`000002ooooo`06o`00
0?ooooooooooooooool0003ooooo0_l000000ooooooo0000o`000005ooooo`?o000000Goooooo`00
0?ooooooooooo`00000Booooo`;o00001?ooool01?l0003oooooooooool00003ooooo`04o`000?oo
ooooooooo`0000gooooo00?o0000ooooooooool01_ooool2o`0000Cooooo0_l0000=ooooo`04o`00
0?ooooooooooooooo`;o00002_ooool00ol0003oooooo`000007ooooo`?o00001oooool00ol0003o
ooooo`000009o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02o`0000Co
oooo00?o0000ooooool000000_l00003ooooo`Wo00006oooool001[ooooo1ol00007ooooo`03o`00
0?oooooooooo00?ooooo0ol000000ooooooo0000o`000002ooooo`03o`000?oooooooooo00;ooooo
00Oo0000ooooooooooooooooo`000?oooooo000000Kooooo1?l000000ooooooo0000ooooo`0Boooo
o`;o00001?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`03ooooo`Co00001Oooool00ol0003oooooooooo`0<
ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooo`000008ooooo`;o00002?oo
ool<o`000003ooooool0003ooooo00Gooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooooooo0_l00003ooooo`04o`00
0?ooooooooooo`0000Cooooo00?o0000ooooool000001ol0000Kooooo`006_ooool7o`0000Kooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`04o`000003ooooool0003ooooo00?ooooo
00Go0000oooooooooooo0000ooooo`03o`0000Gooooo00?o0000ooooool000000ol0000Cooooo`;o
00001Oooool01Ol0003oooooo`000?oooooo000000Kooooo00?o0000ooooooooool02Oooool01?l0
003oooooooooooooool2o`0000;ooooo00?o0000ooooooooool00oooool2o`0000kooooo00?o0000
ooooooooool00_l0000<ooooo`03o`000?oooooo000000Oooooo0_l00004ooooo`Go000000?ooooo
o`000?ooool00_ooool2o`0000;ooooo0ol00008ooooo`05o`000?ooooooooooooooool000002?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000_l0
0002ooooo`;o00001?ooool00ol0003oooooo`000007o`0001_ooooo000Kooooo`Ko00001Oooool0
0ol0003oooooooooo`06ooooo`04o`000?oooooo0000o`0000;ooooo0ol00003ooooo`03o`000?oo
oooo000000Cooooo00?o0000ooooooooool00oooool2o`0000?ooooo00?o0000ooooooooool04?oo
ool01?l0003oooooooooool00004ooooo`04o`000?oooooo0000o`0000Oooooo00?o0000oooooooo
ool02Oooool4o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooo`00000=ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool02_ooool00ol0003oooooo`000006oooo
o`Ko00001?ooool00ol0003oooooo`000005ooooo`;o000000?oooooo`000?ooool01?l00004oooo
o`04o`000?ooooooooooo`0000Wooooo00Go0000ooooooooooooooooo`000005ooooo`04o`000?oo
oooo0000o`0000;ooooo0_l00004ooooo`05o`000?ooooooooooo`000?ooool01Ol0000Looooo`00
7?ooool3o`000004ooooool0003o0000o`0000;ooooo00?o0000ooooooooool01oooool2o`000004
ooooool0003oooooo`0000?ooooo1Ol00006ooooo`;o00000oooool2o`0000?ooooo0ol0000@oooo
o`05o`000?ooooooooooooooool000000oooool00ol0003oooooo`000009ooooo`03o`000?oooooo
oooo00Oooooo0_l00005ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`00
000<ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00
0003ooooo`Ko00001oooool01?l0003oooooooooool00006ooooo`?o000000Coooooo`000?oooooo
oooo0ol000001Ooooooo0000oooooooooooo000000[ooooo00?o0000ooooool000001Oooool02Ol0
003oooooo`000?oooooo0000ooooool0003oooooo`000004ooooo`04o`000?oooooo0000o`0000;o
oooo0ol0000Mooooo`008?ooool5o`0000[ooooo1?l000000ooooooo0000ooooo`04ooooo`Co0000
1Oooool01?l0003oooooo`000?ooool2o`0000?ooooo00Co0000oooooooooooooooo0_l0000=oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00?ooooo0ol000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0:ooooo`;o00001?ooool00ol0
003oooooooooo`0;ooooo`Go00000oooool00ol0003oooooo`000007ooooo`04o`000?oooooooooo
o`0000Kooooo0_l000000ooooooo0000o`000005ooooo`Co00002oooool2o`0000Kooooo0_l00003
ooooo`03o`000?oooooo000000Gooooo00?o0000ooooool000000_l0000Qooooo`007oooool6o`00
00Wooooo00Co0000ooooool0003ooooo0_l00005ooooo`;o00000oooool3o`0000?ooooo00Co0000
ooooool0003o00001?ooool00ol0003oooooooooo`03ooooo`;o00002oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;o00002oooool01?l0003oooooooooooooool2o`0000Cooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool02Oooool2o`0000Kooooo00?o0000ooooooooool01_ooool5o`000003oooo
ool0003ooooo00Cooooo00?o0000ooooool000001oooool01?l0003oooooooooool00005ooooo`03
o`000?oooooo000000?ooooo0_l00004ooooo`03o`000?oooooo000000?o00002?ooool2o`0000Go
oooo0_l00003ooooo`04o`000?oooooo0000o`0000Cooooo00?o0000ooooool000001?l0000Poooo
o`007_ooool8o`0000Oooooo00Oo0000ooooooooooooooooo`000?oooooo000000Cooooo00?o0000
ooooool000001_ooool4o`0000;ooooo0_l00003ooooo`03o`000?oooooooooo00Gooooo0_l00008
ooooo`03o`000?oooooooooo00Cooooo00Co0000ooooool0003ooooo0_l0000;ooooo`?o00001_oo
ool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool4o`0000Gooooo
00?o0000ooooool000001?ooool00ol0003oooooo`000007ooooo`04o`000?ooooooooooo`0000Co
oooo00Co0000oooooooooooo00001?ooool4o`000003ooooool0003o000000Cooooo0ol00005oooo
o`;o00000oooool2o`000003ooooool0003ooooo00;o00000_ooool00ol0003oooooooooo`03oooo
o`So00007oooool001kooooo2_l00004ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`04
o`000?ooooooooooo`0000Wooooo0ol00002ooooo`03o`000?oooooo000000[ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;o00002Oooool2o`00
0003ooooool0003ooooo00Kooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo00Oooooo0_l00007ooooo`Go00002_ooool00ol0003oooooo`000003
ooooo`03o`000?oooooo000000Oooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000Gooooo00?o0000ooooooooool00_l00008ooooo`Go000000Goooooo`000?oooooo
ooooo`000002ooooo`03o`000?oooooo000000Cooooo0_l00004ooooo`So00007oooool001kooooo
2ol00002ooooo`03o`000?oooooooooo00?ooooo00Co0000ooooool0003ooooo0_l00003ooooo`03
o`000?oooooooooo00Sooooo1ol0000;ooooo`;o00001?ooool00ol0003oooooooooo`05ooooo`;o
00000_ooool00ol0003oooooo`000005ooooo`?o00000oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooo000000Cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`;o
00001?ooool4o`000003ooooool0003ooooo00cooooo00?o0000ooooool000001?ooool3o`0000Ko
oooo00Oo0000ooooooooooooooooo`000?oooooo000000Cooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`03o`0000Wooooo1_l00003ooooo`;o00001Oooool00ol0003oooooo`000002
ooooo`03o`000?oooooo000000Ko00007oooool001oooooo1Ol00002ooooo`Ko00001Oooool3o`00
0003ooooool0003o000000?ooooo00?o0000ooooooooool02?ooool2o`0000?ooooo0ol0000<oooo
o`;o00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o00000_ooool01?l0
003oooooooooooooool2o`0000Oooooo00?o0000ooooooooool01?ooool2o`0000Cooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000Co00001Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooo000000?ooooo0ol00006ooooo`04o`000?oooooo
ooooooooo`;o00001Oooool00ol0003oooooooooo`05ooooo`;o000000?oooooo`000?ooool00ol0
0005ooooo`04o`000?ooooooooooooooo`Go000000?oooooo`000?ooool01?ooool01?l0003ooooo
ooooool00002ooooo`So00008?ooool0023ooooo0ol00005ooooo`Wo000000Coooooo`000?l0003o
00000_ooool00ol0003oooooo`00000:ooooo`?o00001?ooool00ol0003oooooo`000003o`0000[o
oooo0_l00006ooooo`04o`000?ooooooooooooooo`;o00000oooool3o`0000[ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool00_ooool5o`0000[ooooo00?o0000ooooooooool03?ooool2o`0000?ooooo0ol00007oooo
o`03o`000?oooooooooo00;o00001Oooool00ol0003oooooooooo`05ooooo`;o000000?oooooo`00
0?ooool00oooool2o`0000;ooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo0ol0
0004ooooo`06o`000?ooooooooooooooool0003ooooo0ol000001?oooooo0000o`000?l0000Qoooo
o`009?ooool3o`000003ooooool0003o000000;ooooo0ol000000ooooooo0000o`000002o`0000Co
oooo0_l0000:ooooo`;o000000Goooooo`000?ooooooooooo`000004ooooo`03o`000?oooooo0000
00;o00002?ooool3o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooo`000002o`00
00cooooo00?o0000ooooooooool00_ooool2o`0000?ooooo00?o0000ooooooooool02oooool6o`00
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo0000
00;ooooo0_l000000ooooooo0000ooooo`05ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000
ooooooooool01Oooool2o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000o`000003
ooooo`04o`000?oooooo0000o`0000;ooooo1?l00004ooooo`Go00009Oooool002?ooooo1Ol00002
ooooo`05o`000?ooooooooooo`000?ooool00_l000000ooooooo0000o`000004o`000003ooooool0
003ooooo00Sooooo00Go0000ooooool0003oooooo`000002ooooo`03o`000?oooooooooo00?ooooo
0_l00002ooooo`?o00001?ooool01ol0003ooooooooooooooooo0000ooooool000001Oooool3o`00
00?ooooo00?o0000ooooooooool02_ooool01_l0003oooooooooool0003oooooo`0000;ooooo00?o
0000ooooooooool02?ooool5o`0000?ooooo00Go0000ooooooooooooooooo`00000;ooooo`03o`00
0?oooooooooo00gooooo00Ko0000ooooool0003oooooo`000?ooool2o`0000Kooooo00Co0000oooo
ool0003o00001_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o00001oooool4
o`0000Cooooo0_l000000ooooooo0000ooooo`04ooooo`[o00009Oooool002;ooooo1ol00002oooo
o`03o`000?oooooooooo00;o000000?oooooo`000?l000000_l00003ooooo`Co00001_ooool00ol0
003oooooo`000003ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool00_ooool3o`00
0003ooooool0003ooooo00?ooooo0_l00003ooooo`;o000000?oooooo`000?l000000oooool00ol0
003oooooooooo`0;ooooo`07o`000?oooooo0000ooooool0003oooooo`000007ooooo`Co00001?oo
ool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0
3Oooool2o`000003ooooool0003ooooo00;o00001_ooool2o`0000;ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000001_ooool00ol0003o
ooooo`000002ooooo`;o000000?oooooo`000?l000000_l00005ooooo`03o`000?oooooooooo00Wo
00009?ooool002;ooooo1ol00003ooooo`05o`000?oooooo0000ooooool000000oooool3o`0000;o
oooo00?o0000ooooool000000ol00003ooooo`03o`000?oooooo000000?ooooo00?o0000ooooool0
00002?ooool2o`0000Gooooo0ol00002ooooo`03o`000?oooooooooo00?o00000oooool00ol0003o
ooooo`000003ooooo`03o`000?oooooooooo00[ooooo0_l00002ooooo`;o00000oooool5o`0000Wo
oooo0_l00004ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool03Oooool01Ol0003o
ooooo`000?l0003ooooo00;o00001_ooool01Ol0003ooooooooooooooooo000000Gooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool3o`0000Gooooo0_l00000
0ooooooo0000o`000005ooooo`04o`000?oooooo0000ooooo`So00009?ooool002;ooooo1ol00004
ooooo`05o`000?oooooo0000ooooool000000_ooool2o`000004ooooool0003o0000o`0000;ooooo
00?o0000ooooooooool01?l000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`05o`0000;ooooo0_l000001Ooooooo0000
oooooooooooo000000?ooooo00?o0000ooooooooool02_ooool2o`0000;ooooo1Ol0000>ooooo`;o
00001?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00kooooo0ol000001?oooooo
0000ooooool00005ooooo`05o`000?ooooooooooooooool000001Oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000_ooool2o`0000Kooooo00?o0000
ooooooooool00ol00005ooooo`;o000000?oooooo`000?ooool01ol0000Tooooo`008oooool5o`00
00Gooooo0_l00003ooooo`04o`000?oooooo0000o`0000?ooooo1?l00003ooooo`03o`000?oooooo
oooo00Co00000_ooool00ol0003oooooooooo`0:ooooo`;o000000?oooooo`000?ooool00_ooool6
o`0000?ooooo0_l00003ooooo`05o`000?ooooooooooooooool000002_ooool6o`0001;ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`;o00000oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool01?l0003oooooo`00
0?ooool2o`0000Kooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00Co0000ooooool0003o
00000oooool5o`0002Gooooo000Tooooo`?o00000_ooool5o`000003ooooool0003ooooo00;ooooo
0_l000001?oooooo0000ooooool00003ooooo`?o000000Goooooo`000?ooooooooooo`000003oooo
o`Co00003Oooool5o`0000Kooooo0ol000001_oooooo0000o`000?l0003oooooo`0000?ooooo00?o
0000ooooooooool01Oooool4o`0000;ooooo00?o0000ooooool000004Oooool2o`0000Cooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0>ooooo`06o`000?oooooo0000o`000?oooooo
00000oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooo
ooooooooo`000007ooooo`04o`000?oooooo0000o`0000Kooooo00?o0000ooooooooool00oooool2
o`0000;ooooo1?l000000ooooooo0000o`000006o`0002Kooooo000Xooooo`Go00000oooool2o`00
00;ooooo0ol000000ooooooo0000ooooo`04ooooo`Co000000?oooooo`000?ooool01?ooool6o`00
00Wooooo0_l000000ooooooo0000ooooo`09ooooo`Go000000?oooooo`000?ooool00_ooool00ol0
003oooooo`000004o`000003ooooool0003ooooo00;ooooo00Co0000ooooool0003o00004?ooool0
1?l0003oooooooooool00003ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03ooo
ool3o`000005ooooool0003oooooooooool000000_ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00;ooooo00Go0000ooooooooooooooooo`000007ooooo`Co00001Oooool00ol0003o
ooooooooo`04ooooo`05o`000?oooooo0000o`000?ooool02?l0000Zooooo`009oooool7o`0000Co
oooo00Co0000ooooool0003ooooo0ol00006ooooo`03o`000?oooooooooo00?o00000oooool01?l0
003oooooooooool00003ooooo`Co00000_ooool4o`000003ooooool0003o000000_ooooo00?o0000
ooooooooool00ol000000ooooooo0000o`000002o`0000Gooooo00Go0000ooooooooooooooooo`00
0003ooooo`03o`000?oooooooooo00gooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00kooooo00?o0000ooooooooool03_ooool3o`0000;ooooo0_l00003ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00007ooooo`?o
00000_ooool01Ol0003ooooooooooooooooo000000Kooooo00Co0000ooooool0003o00000_ooool0
0ol0003oooooo`000006o`0002Wooooo000Wooooo`Oo00001Oooool2o`0000;ooooo0_l00005oooo
o`05o`000?oooooo0000ooooool000000_ooool4o`0000;ooooo00?o0000ooooooooool01Oooool5
o`000005ooooool0003oooooooooool000002Oooool00ol0003oooooooooo`07o`000003ooooool0
003ooooo00?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o00003Oooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`00017ooooo00?o0000ooooooooool03oooool2o`0000;o
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Cooooo00Co0000oooo
oooooooo00001_ooool00ol0003oooooo`000004ooooo`?o00001oooool2o`000004ooooool0003o
ooooo`0000;ooooo1ol0000Yooooo`009oooool7o`0000Kooooo0_l00002ooooo`;o00000oooool0
1?l0003oooooooooooooool2o`0000Gooooo0_l00002ooooo`04o`000?ooooooooooooooo`?o0000
1?ooool4o`0000;ooooo0_l00006ooooo`Co00001?ooool00ol0003oooooo`000003o`0000;ooooo
00Go0000ooooooooooooooooo`000005ooooo`;o00003?ooool00ol0003oooooooooo`03ooooo`04
o`000?ooooooooooo`00017ooooo00?o0000ooooooooool03oooool3o`000003ooooool0003ooooo
00Cooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooo000000Kooooo00Co0000
ooooool0003o00001?ooool2o`0000Oooooo0_l00002ooooo`?o00000_ooool7o`0002Wooooo000X
ooooo`Go00002?ooool01_l0003oooooooooool0003oooooo`0000;ooooo00Co0000oooooooooooo
oooo0_l00005ooooo`03o`000?oooooo000000;o000000?oooooo`000?l000001_ooool2o`0000?o
oooo2_l000000ooooooo0000ooooo`06ooooo`03o`000?oooooooooo00Go00000_ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooo000000[ooooo00?o0000ooooooooool01Oooool00ol0003o
ooooo`00000Booooo`03o`000?oooooooooo00kooooo1Ol00005ooooo`05o`000?oooooooooooooo
ool000001?ooool2o`0000Gooooo0_l00002ooooo`;o00000oooool00ol0003oooooo`000006oooo
o`;o00000_ooool01?l0003oooooo`000?l00002ooooo`Ko0000:_ooool002Wooooo0ol00003oooo
o`?o00000oooool4o`0000;ooooo0_l00004ooooo`03o`000?oooooo000000?ooooo00Co0000oooo
oooooooooooo1Ol00006ooooo`03o`000?oooooooooo00?ooooo1ol00003ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool00ol000001?oooooo0000o`000?l00006ooooo`04o`000?oo
ooooooooo`0000Sooooo00?o0000ooooooooool01_ooool2o`0001?ooooo00?o0000ooooooooool0
3_ooool01_l0003oooooo`000?l0003oooooo`0000Cooooo00Go0000ooooooooooooooooo`000004
ooooo`;o00001?ooool01ol0003ooooooooooooooooo0000ooooool000000_ooool01Ol0003ooooo
oooooooooooo000000Cooooo00Go0000ooooool0003oooooo`000002ooooo`Go000000Coooooo`00
0?l0003o0000:oooool002kooooo1Ol00002ooooo`09o`000?oooooo0000ooooool0003oooooo`00
0?oooooo000000?ooooo00Co0000oooooooooooo00000_ooool01?l0003oooooo`000?l00002oooo
o`03o`000?oooooooooo00?o00000_ooool00ol0003oooooooooo`04o`0000Kooooo1Ol00008oooo
o`03o`000?oooooooooo00Go00000_ooool3o`0000?ooooo00Co0000oooooooooooo00001oooool0
0ol0003oooooooooo`07ooooo`;o00005?ooool00ol0003oooooooooo`0>ooooo`Go00001?ooool0
0ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000001?ooool01Ol0003ooooooooo
oooooooo000000?ooooo0_l00005ooooo`04o`000?oooooo0000o`0000;ooooo0_l00003ooooo`Go
0000;oooool002gooooo1ol000000ooooooo0000ooooo`03o`000007ooooool0003oooooooooool0
003oooooo`000004ooooo`Co00001Oooool00ol0003oooooooooo`02ooooo`Go00002oooool3o`00
0003ooooool0003o000000;o00001Oooool01Ol0003oooooo`000?l0003ooooo00?o00001?ooool4
o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`;o00005?ooool0
0ol0003oooooooooo`0=ooooo`Ko00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool2o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`05ooooo`04o`000?ooooooooooooooo`;o00000oooool7o`0002kooooo000]ooooo`Oo000000Go
ooooo`000?oooooo0000ooooo`02o`0000Cooooo0_l00003ooooo`Co00001_ooool:o`0000gooooo
00?o0000ooooooooool00oooool4o`000004ooooool0003o0000ooooo`;o00000oooool2o`0000Go
oooo1?l000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00Wooooo0_l0000Dooooo`03
o`000?oooooooooo00cooooo00Go0000ooooool0003o0000ooooo`02o`0000Cooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`02o`000003ooooool0003ooooo00?ooooo00?o0000oooo
ooooool00_ooool2o`0000Gooooo0_l000000ooooooo0000ooooo`02o`0000Cooooo1ol0000^oooo
o`00;Oooool9o`000003ooooool0003ooooo00;o00001Oooool4o`0000;ooooo0_l00002ooooo`Ko
00000_ooool2o`0000Cooooo0ol0000:ooooo`?o00001oooool6o`0000Gooooo0_l00003ooooo`04
o`000?ooooooooooooooo`?o000000?oooooo`000?ooool02oooool2o`0001Gooooo00?o0000oooo
ooooool02oooool01Ol0003oooooo`000?l0003ooooo00?o00000oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00?o00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
000000Cooooo00Co0000oooooooooooooooo0_l000000ooooooo0000ooooo`02ooooo`So0000;_oo
ool002kooooo1Ol00002ooooo`?o000000Coooooo`000?oooooo00000oooool2o`000003ooooool0
003ooooo00;ooooo1?l00005ooooo`05o`000?oooooo0000ooooool000001oooool3o`0000Oooooo
00Co0000oooooooooooo00001_ooool00ol0003oooooo`000005o`0000Gooooo00Co0000oooooooo
oooo00001Oooool4o`0000_ooooo0_l0000Eooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool00_l00002ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo0000
00;o00001Oooool00ol0003oooooooooo`02o`0000;ooooo00?o0000ooooooooool00_l00004oooo
o`?o00000oooool8o`0002oooooo000_ooooo`?o00000oooool5o`000003ooooool0003o000000?o
000000?oooooo`000?l000001?l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool0
00002oooool3o`0000Cooooo00Co0000oooooooooooooooo0_l00003ooooo`03o`000?oooooooooo
00;o00000_ooool00ol0003oooooo`000002o`0000?ooooo0ol00005ooooo`;o00000_ooool3o`00
00Sooooo0_l0000Eooooo`03o`000?oooooooooo00Wooooo00Co0000oooooooooooooooo0_l00003
ooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`;o000000?oooooo`000?ooool00oooool0
1?l0003oooooooooool00004ooooo`03o`000?oooooo000000Gooooo00Go0000oooooooooooo0000
ooooo`04o`0000;ooooo0ol0000`ooooo`00=?ooool5o`000003ooooool0003o000000Go00000ooo
ool3o`0000?ooooo00?o0000ooooooooool00_ooool2o`000003ooooool0003ooooo00gooooo0_l0
00000ooooooo0000ooooo`05ooooo`08o`000?ooooooooooo`000?oooooo0000ooooool00002oooo
o`03o`000?oooooooooo00;ooooo1?l000000ooooooo0000o`000003ooooo`03o`000?oooooo0000
00Gooooo0ol00004ooooo`04o`000?ooooooooooo`0001Gooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`02ooooo`;o00000oooool2o`0000;ooooo00?o0000ooooooooool00_ooool2
o`0000;ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00?ooooo0_l0
0004ooooo`;o00000oooool6o`0003Gooooo000cooooo`co00001_ooool2o`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00oooooo0ol00006ooooo`;o000000?o
ooooo`000?ooool00_ooool01?l0003oooooooooool00007ooooo`Go000000Goooooo`000?oooooo
ooooo`000007ooooo`?o000000Goooooo`000?ooooooooooo`00000Eooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool00oooool2o`0000Cooooo0_l00002ooooo`03o`000?oooooooooo
00?o00000_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00?ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo2?l0000dooooo`00<oooool7o`0000?ooooo1?l00004oooo
o`03o`000?oooooo000000?ooooo00?o0000ooooool000000oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooo000000;o00000oooool3o`0000Gooooo00?o0000ooooool000002?ooool00ol0
003oooooooooo`04o`0000;ooooo00?o0000ooooooooool01oooool3o`000003ooooool0003ooooo
01Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00;ooooo0_l00002ooooo`04o`000?oooooo0000ooooo`;o00000_ooool00ol0003oooooo`000002
ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`04o`000?ooooooooooo`0000Gooooo2?l0
000dooooo`00<oooool7o`0000?ooooo0_l000000ooooooo0000o`000002o`000004ooooool0003o
ooooooooo`;o00000_ooool01ol0003ooooooooooooooooo0000ooooool000004Oooool00ol0003o
ooooooooo`02ooooo`?o000000Coooooo`000?oooooo00001?ooool01?l0003oooooooooool00007
ooooo`04o`000?ooooooooooo`0000;ooooo1Ol00008ooooo`03o`000?oooooooooo00?o00005Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo0_l00003ooooo`?o000000Oo
ooooo`000?l0003oooooo`000?oooooo000000;ooooo0_l00002ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool00_ooool00ol0003oooooo`000006ooooo`So0000=?ooool003Cooooo
1Ol00004ooooo`;o00000_ooool00ol0003oooooooooo`02o`0000Cooooo0_l000001Ooooooo0000
oooooooooooo000000;ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`03ooooo`04
o`000?oooooo0000o`0000;ooooo0_l00003ooooo`03o`000?oooooo000000Oooooo00?o0000oooo
ool000001?ooool2o`000003ooooool0003o000000?o00001?ooool00ol0003oooooooooo`02oooo
o`Go00004Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo0_l00003oooo
o`03o`000?oooooo000000;o000000?oooooo`000?ooool00_ooool01_l0003oooooo`000?l0003o
ooooo`0000?ooooo0_l00005ooooo`;o00001_ooool00ol0003oooooo`000005o`0003Gooooo000e
ooooo`?o00000oooool4o`0000?ooooo00Co0000ooooool0003ooooo0ol000000ooooooo0000o`00
0002ooooo`;o00000oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooool000000_ooool3o`000005ooooool0003oooooooooool000000_ooool00ol0003ooooo
ooooo`04ooooo`;o00001oooool3o`0000?ooooo1Ol00004ooooo`03o`000?oooooooooo00;ooooo
0ol0000>ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01oooool2o`0000Cooooo
1?l000000ooooooo0000ooooo`02ooooo`03o`000?oooooo000000;o00000oooool00ol0003ooooo
ooooo`04ooooo`;o00001Oooool4o`0000;ooooo0ol0000fooooo`00>_ooool6o`0000?ooooo0_l0
0004ooooo`;o000000Goooooo`000?l0003oooooo`000004ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool00_ooool00ol0003oooooo`000006ooooo`Co000000Coooooo`000?oooooo
00001_ooool00ol0003oooooooooo`08ooooo`;o00001Oooool5o`000003ooooool0003ooooo00Ko
oooo0ol0000<ooooo`04o`000?ooooooooooo`0000[ooooo0_l00003ooooo`;o000000Coooooo`00
0?l0003o00001?ooool4o`0000;ooooo0_l00006ooooo`03o`000?oooooo000000Cooooo1Ol0000k
ooooo`00>Oooool7o`0000Cooooo0_l00002ooooo`03o`000?oooooooooo00Co000000?oooooo`00
0?ooool00_ooool00ol0003oooooooooo`0<ooooo`07o`000?ooooooooooooooool0003oooooo`00
000:ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`;o00002oooool3o`0000?ooooo00?o
0000ooooooooool00_ooool4o`0000Sooooo0ol00009ooooo`03o`000?oooooo000000_ooooo0_l0
0002ooooo`03o`000?oooooo000000;ooooo0_l00004ooooo`;o000000Coooooo`000?oooooo0000
1oooool01?l0003oooooooooool00003ooooo`Oo0000>_ooool003Wooooo1ol00003ooooo`03o`00
0?oooooooooo00;o00001_ooool2o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool0
2oooool01_l0003oooooooooool0003oooooo`0000gooooo1?l00003ooooo`03o`000?oooooo0000
00cooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0000
00?o00001oooool3o`0000Oooooo00?o0000ooooooooool02_ooool4o`0000;ooooo00?o0000oooo
ooooool00_l00003ooooo`;o00000_ooool2o`0000Oooooo00Go0000ooooooooooooooooo`000002
ooooo`So0000>_ooool003Wooooo2?l00002ooooo`03o`000?oooooooooo00;o00001Oooool00ol0
003oooooooooo`03o`0000;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`02o`00
00oooooo0_l000001?oooooo0000o`000?l00002ooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool00_l00007ooooo`03o`000?oooooooooo00?ooooo1?l00006ooooo`?o00001?ooool0
0ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00?ooooo1Ol00002ooooo`03o`000?oooooo
000000;o00001_ooool2o`0000Cooooo00Co0000ooooool0003ooooo1ol0000jooooo`00>_ooool5
o`000003ooooool0003o000000;o000000Goooooo`000?ooooooooooo`000004ooooo`04o`000?oo
ooooooooooooo`Co000000?oooooo`000?ooool02oooool01?l0003oooooo`000?l0000Aooooo`So
00003Oooool00ol0003oooooooooo`03o`0000Gooooo00?o0000ooooooooool01oooool4o`0000Go
oooo1Ol0000<ooooo`;o00001Oooool3o`000005ooooool0003oooooo`000?ooool00_l000000ooo
oooo0000ooooo`04ooooo`03o`000?oooooooooo00Cooooo0ol000000ooooooo0000o`000004o`00
03_ooooo000kooooo`?o00000_ooool5o`0000Cooooo00Co0000oooooooooooo00001_ooool5o`00
00[ooooo00Co0000ooooool0003o00004_ooool01?l0003oooooo`000?l00003ooooo`?o00003?oo
ool00ol0003oooooo`000002ooooo`;o00000oooool00ol0003oooooooooo`0;ooooo`Co00000ooo
ool4o`0000Wooooo0_l000000ooooooo0000ooooo`04ooooo`03o`000?oooooooooo00?o000000Ko
ooooo`000?oooooo0000ooooool00004ooooo`03o`000?oooooooooo00Cooooo1Ol00002ooooo`?o
0000??ooool003oooooo1ol00004ooooo`03o`000?oooooo000000Oooooo0_l000001?oooooo0000
o`000?l00008ooooo`?o00005?ooool00ol0003oooooo`000002o`000004ooooool0003ooooooooo
o`?o00002Oooool2o`0000Gooooo0_l00002ooooo`03o`000?oooooooooo00kooooo1?l00002oooo
o`Co00001Oooool01Ol0003ooooooooooooooooo000000Gooooo0_l000000ooooooo0000o`000002
o`0000;ooooo00?o0000ooooool000000oooool00ol0003oooooooooo`04ooooo`Oo0000@?ooool0
03oooooo2?l00004ooooo`03o`000?oooooooooo00Oooooo0ol00002ooooo`?o00001Oooool2o`00
01Cooooo00Co0000ooooool0003o00000_ooool2o`0000Gooooo0_l00008ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooool000004Oooool00ol0003oooooooooo`04o`0000;ooooo1Ol00004
ooooo`;o00000oooool00ol0003oooooo`000003o`000007ooooool0003oooooooooool0003ooooo
o`000002ooooo`03o`000?oooooooooo00Gooooo1ol00010ooooo`00?oooool7o`000003ooooool0
003o000000Oo00001oooool2o`0000Cooooo0_l00002ooooo`;o00005?ooool01_l0003ooooooooo
ool0003oooooo`0000;ooooo0_l00006ooooo`?o00001Oooool2o`0000Oooooo0_l0000@ooooo`03
o`000?oooooooooo00Cooooo1?l000000ooooooo0000ooooo`03o`0000;ooooo0_l00002ooooo`03
o`000?oooooooooo00?o00000_ooool2o`0000;ooooo1?l00003ooooo`?o000000?oooooo`000?l0
00001_l00010ooooo`00@?ooool5o`0000;ooooo1Ol000001?oooooo0000ooooooooool9o`0000Ko
oooo1?l0000Cooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`03o`000?oooooo000000So
oooo0ol000001?oooooo0000ooooool00008ooooo`?o00003_ooool00ol0003oooooooooo`04oooo
o`04o`000?ooooooooooooooo`Co00000_ooool4o`0000;ooooo00?o0000ooooool000000ol00003
ooooo`03o`000?oooooooooo00;o00001?ooool5o`0000;ooooo1Ol00011ooooo`00@Oooool3o`00
00;ooooo1ol000000ooooooo0000ooooo`08ooooo`go00004Oooool00ol0003oooooooooo`02oooo
o`05o`000?oooooo0000ooooool000000_ooool00ol0003oooooooooo`08ooooo`?o000000?ooooo
o`000?ooool01oooool01?l0003oooooo`000?l0000;ooooo`03o`000?oooooooooo00Gooooo00Co
0000ooooool0003o00001?ooool8o`000003ooooool0003o000000?o00001?ooool01?l0003ooooo
o`000?l00003ooooo`Oo00000_ooool3o`0004;ooooo0016ooooo`Oo00000_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooo000000?ooooo2_l0000<ooooo`03o`000?oooooooooo00Cooooo
0_l000000ooooooo0000ooooo`02ooooo`;o00002Oooool00ol0003oooooo`000002o`0000Oooooo
00Co0000oooooooooooooooo0_l00008ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool02?ooool7o`000003ooooool0003o000000Cooooo1?l00003ooooo`Oo0000Aoooool004Kooooo
1ol00002ooooo`?o00002?ooool01Ol0003oooooo`000?oooooo000000;ooooo00?o0000oooooooo
ool00oooool:o`0000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooo`000?l00005oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo0ol0000;oooo
o`Wo00001?ooool5o`000003ooooool0003o000000Ko0000Aoooool004Oooooo1Ol00002ooooo`Go
00001oooool2o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool01_ooool2o`0000Go
oooo2?l00004ooooo`?o00001_ooool2o`0000Gooooo00?o0000ooooooooool00oooool00ol0003o
ooooo`000002o`0000?ooooo00?o0000ooooooooool00oooool2o`0000?ooooo00?o0000oooooooo
ool01_ooool01Ol0003ooooooooooooooooo000000[ooooo0_l000000ooooooo0000ooooo`02oooo
o`Ko000000?oooooo`000?l000000ol00002ooooo`Go0000B?ooool004Sooooo0ol00002ooooo`Oo
00001oooool2o`000005ooooool0003o0000ooooool000002_ooool3o`0000?ooooo00?o0000oooo
ooooool01?ooool9o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04oooo
o`04o`000?ooooooooooooooo`Co00001oooool2o`000003ooooool0003ooooo00Kooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`06ooooo`?o000000?oooooo`000?ooool01?ooool0
0ol0003oooooo`000009o`0000;ooooo0ol00019ooooo`00COooool7o`0000Oooooo00?o0000oooo
ool000000_ooool00ol0003oooooooooo`0<ooooo`?o00003?ooool2o`0000;ooooo2Ol000000ooo
oooo0000ooooo`07ooooo`03o`000?oooooooooo00?ooooo0ol00007ooooo`;o00001Oooool2o`00
00Gooooo00?o0000ooooooooool01Oooool4o`000003ooooool0003ooooo00Cooooo0_l00002oooo
o`So0000C_ooool004gooooo2?l000002ooooooo0000o`000?l0003oooooo`000?oooooo0000oooo
ool0003ooooo00;o00003_ooool4o`0000[ooooo0ol00009ooooo`So00000oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;o00001?ooool00ol0003oooooooooo`02o`0000;ooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`;o000000Coooooo`000?l0003o
00001Oooool5o`000003ooooool0003o000000Go0000C_ooool004kooooo1Ol00002ooooo`Go0000
0_ooool00ol0003oooooooooo`02o`000003ooooool0003o000000_ooooo00?o0000ooooooooool0
0_ooool2o`0000Wooooo0_l0000;ooooo`;o00001?ooool9o`000003ooooool0003ooooo00;ooooo
1?l00005ooooo`;o00002Oooool01?l0003oooooooooooooool3o`000004ooooool0003oooooo`00
00Gooooo1Ol00003ooooo`Go0000Coooool004oooooo0ol00002ooooo`Oo00000_ooool01?l0003o
ooooooooool00003ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01Oooool3o`00
00Kooooo0ol00009ooooo`04o`000?ooooooooooo`0000Sooooo00Co0000oooooooooooooooo2ol0
0002ooooo`Co00002?ooool02Ol0003oooooooooool0003oooooo`000?oooooo0000ooooo`03o`00
00?ooooo1ol00003ooooo`?o0000D?ooool005Cooooo1ol00002ooooo`Go00000oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Wooooo0_l00004ooooo`?o00002Oooool01?l0003o
ooooooooooooool2o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooo000000Oo000000?oooooo`000?l000001oooool2o`000004ooooool0003ooooo
ooooo`;o000000?oooooo`000?l000002_l0001Fooooo`00E?ooool7o`0000;ooooo1_l00003oooo
o`;o00001?ooool00ol0003oooooooooo`0<ooooo`?o000000?oooooo`000?ooool00_l00008oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo
000000Cooooo00Co0000oooooooooooooooo0_l00002ooooo`oo00000_ooool00ol0003oooooo`00
0005o`000003ooooool0003o000000Go0000E_ooool005Gooooo1Ol00002ooooo`So00001?ooool0
1?l0003oooooooooool0000Booooo`;o000000?oooooo`000?l000001oooool00ol0003ooooooooo
o`05ooooo`;o00001Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`000007ooooo`;o00000_oooolCo`000003ooooool0003o000000?o0000Eoooool005Kooooo
0ol00003ooooo`Oo000000?oooooo`000?ooool01Ol0000Dooooo`Go00001_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00;ooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00?o0000ooooooooool02?ooool5o`000003ooooool0003ooooo00go00000_ooool3o`00
05Sooooo001Looooo`Oo00000_ooool6o`0001Cooooo00?o0000ooooooooool00ol00005ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02o`0000;o
oooo0_l0000<ooooo`_o00000_ooool7o`0005gooooo001Mooooo`Go00000_ooool<o`0000oooooo
00Ko0000ooooooooooooooooo`000?ooool3o`000003ooooool0003ooooo00_ooooo0_l00003oooo
o`03o`000?oooooo000000;ooooo00?o0000ooooooooool02oooool3o`000003ooooool0003ooooo
00Oo00000_ooool5o`0005kooooo001Nooooo`?o00000oooool7o`000003ooooool0003o000000?o
00003_ooool01?l0003oooooooooooooool2o`0000?ooooo0_l0000>ooooo`03o`000?oooooooooo
00;o000000?oooooo`000?l000003?ooool6o`000003ooooool0003o000000Ko00000oooool3o`00
05oooooo001Tooooo`ko00000oooool3o`0000Oooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`02ooooo`Co00003?ooool2o`000004ooooool0003o0000o`0000Oooooo0ol00002oooo
o`So000000?oooooo`000?l000001Ol0001Uooooo`00IOooool5o`000003ooooool0003o000000Go
00000_ooool5o`0000Gooooo0ol00004ooooo`04o`000?ooooooooooo`0000Cooooo0_l0000<oooo
o`Ko00001?ooool6o`000003ooooool0003o000000Go00000_ooool5o`0006Kooooo001Vooooo`?o
00000_ooool?o`0000?ooooo1Ol00003ooooo`Co00001_ooool3o`0000Cooooo0ol000000ooooooo
0000o`000006o`0000;ooooo1ol000000ooooooo0000o`000005o`0000?ooooo0ol0001Wooooo`00
K?ooool5o`0000;ooooo1ol00002ooooo`Oo00000_ooool5o`0000Cooooo0ol000001?oooooo0000
o`000?ooool5o`0000;ooooo1ol000000ooooooo0000o`000005o`0000;ooooo1Ol0001^ooooo`00
KOooool3o`0000?ooooo1ol00002ooooo`Oo000000?oooooo`000?l000001Ol00002ooooo`Go0000
0_ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00003ooooo`?o0000
Koooool007Cooooo1Ol00003ooooo`Oo000000?oooooo`000?l000003Ol000000ooooooo0000o`00
0005o`000003ooooool0003o000000Go00000_ooool5o`0007Kooooo001eooooo`?o00001Oooool5
o`0000;ooooo1ol000000ooooooo0000o`000005o`000003ooooool0003o000000Go00000_ooool5
o`0000Cooooo0ol0001gooooo`00O_ooool3o`0000Cooooo1Ol00002ooooo`Oo00000_ooool5o`00
00Cooooo0ol0001oooooo`00Q_ooool3o`0000Cooooo1Ol00004ooooo`?o0000Qoooool008kooooo
0ol0002?ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(ran1DTD = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] ranGraph1, \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[ran1DTD]];\)\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95365 .52854 m
.18884 .83135 L
s
.95365 .52854 m
.05351 .41483 L
s
.95096 .55697 m
.64046 .9323 L
s
.94027 .61304 m
.85023 .78974 L
s
.94027 .61304 m
.08871 .69354 L
s
.94027 .61304 m
.07737 .66733 L
s
.94027 .61304 m
.04635 .52854 L
s
.9323 .64046 m
.04635 .52854 L
s
.9323 .64046 m
.83135 .18884 L
s
.9323 .64046 m
.94649 .41483 L
s
.92263 .66733 m
.61304 .94027 L
s
.92263 .66733 m
.52854 .95365 L
s
.92263 .66733 m
.18884 .83135 L
s
.92263 .66733 m
.04904 .44303 L
s
.92263 .66733 m
.5 .04545 L
s
.88379 .74356 m
.66733 .92263 L
s
.88379 .74356 m
.58517 .94649 L
s
.86773 .76718 m
.76718 .86773 L
s
.86773 .76718 m
.74356 .88379 L
s
.86773 .76718 m
.04635 .52854 L
s
.86773 .76718 m
.38696 .05973 L
s
.86773 .76718 m
.95365 .47146 L
s
.85023 .78974 m
.04904 .44303 L
s
.85023 .78974 m
.95455 .5 L
s
.83135 .81116 m
.52854 .04635 L
s
.81116 .83135 m
.23282 .13227 L
s
.81116 .83135 m
.95455 .5 L
s
.78974 .85023 m
.94027 .38696 L
s
.76718 .86773 m
.66733 .07737 L
s
.74356 .88379 m
.61304 .94027 L
s
.71898 .89832 m
.11621 .74356 L
s
.71898 .89832 m
.08871 .69354 L
s
.71898 .89832 m
.23282 .13227 L
s
.71898 .89832 m
.25644 .11621 L
s
.71898 .89832 m
.55697 .04904 L
s
.71898 .89832 m
.69354 .08871 L
s
.69354 .91129 m
.47146 .04635 L
s
.69354 .91129 m
.61304 .05973 L
s
.66733 .92263 m
.23282 .86773 L
s
.66733 .92263 m
.55697 .04904 L
s
.66733 .92263 m
.58517 .05351 L
s
.64046 .9323 m
.11621 .74356 L
s
.64046 .9323 m
.05973 .61304 L
s
.64046 .9323 m
.33267 .07737 L
s
.64046 .9323 m
.94649 .41483 L
s
.55697 .95096 m
.05973 .61304 L
s
.55697 .95096 m
.10168 .28102 L
s
.55697 .95096 m
.33267 .07737 L
s
.52854 .95365 m
.16865 .81116 L
s
.52854 .95365 m
.95096 .44303 L
s
.5 .95455 m
.47146 .95365 L
s
.47146 .95365 m
.10168 .28102 L
s
.47146 .95365 m
.13227 .23282 L
s
.47146 .95365 m
.74356 .11621 L
s
.44303 .95096 m
.44303 .04904 L
s
.41483 .94649 m
.38696 .94027 L
s
.38696 .94027 m
.10168 .71898 L
s
.35954 .9323 m
.05973 .61304 L
s
.35954 .9323 m
.92263 .33267 L
s
.33267 .92263 m
.05973 .38696 L
s
.33267 .92263 m
.5 .04545 L
s
.30646 .91129 m
.10168 .71898 L
s
.30646 .91129 m
.88379 .25644 L
s
.30646 .91129 m
.91129 .30646 L
s
.25644 .88379 m
.66733 .07737 L
s
.23282 .86773 m
.04635 .47146 L
s
.23282 .86773 m
.44303 .04904 L
s
.23282 .86773 m
.86773 .23282 L
s
.21026 .85023 m
.47146 .04635 L
s
.18884 .83135 m
.11621 .25644 L
s
.18884 .83135 m
.30646 .08871 L
s
.18884 .83135 m
.66733 .07737 L
s
.14977 .78974 m
.58517 .05351 L
s
.14977 .78974 m
.81116 .16865 L
s
.13227 .76718 m
.78974 .14977 L
s
.11621 .74356 m
.05351 .41483 L
s
.11621 .74356 m
.71898 .10168 L
s
.08871 .69354 m
.30646 .08871 L
s
.0677 .64046 m
.0677 .35954 L
s
.0677 .64046 m
.91129 .30646 L
s
.05973 .61304 m
.11621 .25644 L
s
.04635 .52854 m
.41483 .05351 L
s
.04635 .52854 m
.91129 .30646 L
s
.04635 .47146 m
.94649 .41483 L
s
.04904 .44303 m
.10168 .28102 L
s
.05973 .38696 m
.0677 .35954 L
s
.05973 .38696 m
.95365 .47146 L
s
.07737 .33267 m
.21026 .14977 L
s
.08871 .30646 m
.69354 .08871 L
s
.11621 .25644 m
.14977 .21026 L
s
.14977 .21026 m
.94027 .38696 L
s
.16865 .18884 m
.89832 .28102 L
s
.23282 .13227 m
.58517 .05351 L
s
.23282 .13227 m
.83135 .18884 L
s
.25644 .11621 m
.52854 .04635 L
s
.25644 .11621 m
.66733 .07737 L
s
.33267 .07737 m
.35954 .0677 L
s
.35954 .0677 m
.61304 .05973 L
s
.38696 .05973 m
.66733 .07737 L
s
.41483 .05351 m
.94027 .38696 L
s
.5 .04545 m
.61304 .05973 L
s
.5 .04545 m
.89832 .28102 L
s
.64046 .0677 m
.94027 .38696 L
s
.69354 .08871 m
.95365 .47146 L
s
.76718 .13227 m
.78974 .14977 L
s
.81116 .16865 m
.91129 .30646 L
s
.83135 .18884 m
.95096 .44303 L
s
0 1 0 r
.025 w
.95365 .52854 Mdot
1 1 0 r
.95096 .55697 Mdot
.898 .898 .898 r
.94649 .58517 Mdot
1 0 0 r
.94027 .61304 Mdot
0 1 1 r
.9323 .64046 Mdot
1 1 0 r
.92263 .66733 Mdot
.498 .498 .498 r
.91129 .69354 Mdot
0 1 1 r
.89832 .71898 Mdot
0 0 .498 r
.88379 .74356 Mdot
.898 .898 .898 r
.86773 .76718 Mdot
1 1 0 r
.85023 .78974 Mdot
.898 .898 .898 r
.83135 .81116 Mdot
1 0 0 r
.81116 .83135 Mdot
1 0 1 r
.78974 .85023 Mdot
1 1 0 r
.76718 .86773 Mdot
1 0 0 r
.74356 .88379 Mdot
1 1 0 r
.71898 .89832 Mdot
0 0 .498 r
.69354 .91129 Mdot
.898 .898 .898 r
.66733 .92263 Mdot
0 0 .498 r
.64046 .9323 Mdot
0 1 0 r
.61304 .94027 Mdot
.498 0 0 r
.58517 .94649 Mdot
0 1 1 r
.55697 .95096 Mdot
1 0 1 r
.52854 .95365 Mdot
0 1 0 r
.5 .95455 Mdot
0 1 1 r
.47146 .95365 Mdot
.498 0 0 r
.44303 .95096 Mdot
.898 .898 .898 r
.41483 .94649 Mdot
1 0 0 r
.38696 .94027 Mdot
0 .498 0 r
.35954 .9323 Mdot
0 1 0 r
.33267 .92263 Mdot
.498 0 0 r
.30646 .91129 Mdot
0 .498 0 r
.28102 .89832 Mdot
0 0 .498 r
.25644 .88379 Mdot
1 1 0 r
.23282 .86773 Mdot
.21026 .85023 Mdot
.498 .498 .498 r
.18884 .83135 Mdot
.16865 .81116 Mdot
0 0 .498 r
.14977 .78974 Mdot
.13227 .76718 Mdot
0 1 1 r
.11621 .74356 Mdot
0 .498 0 r
.10168 .71898 Mdot
0 1 1 r
.08871 .69354 Mdot
1 0 0 r
.07737 .66733 Mdot
0 0 .498 r
.0677 .64046 Mdot
1 0 1 r
.05973 .61304 Mdot
.898 .898 .898 r
.05351 .58517 Mdot
.498 0 0 r
.04904 .55697 Mdot
0 1 0 r
.04635 .52854 Mdot
0 .498 0 r
.04545 .5 Mdot
.498 .498 .498 r
.04635 .47146 Mdot
.898 .898 .898 r
.04904 .44303 Mdot
0 1 1 r
.05351 .41483 Mdot
1 0 1 r
.05973 .38696 Mdot
.498 .498 .498 r
.0677 .35954 Mdot
0 1 0 r
.07737 .33267 Mdot
.08871 .30646 Mdot
1 0 1 r
.10168 .28102 Mdot
.898 .898 .898 r
.11621 .25644 Mdot
1 1 0 r
.13227 .23282 Mdot
0 .498 0 r
.14977 .21026 Mdot
0 1 1 r
.16865 .18884 Mdot
1 1 0 r
.18884 .16865 Mdot
.498 .498 .498 r
.21026 .14977 Mdot
0 0 .498 r
.23282 .13227 Mdot
.498 .498 .498 r
.25644 .11621 Mdot
1 1 0 r
.28102 .10168 Mdot
.498 0 0 r
.30646 .08871 Mdot
1 0 0 r
.33267 .07737 Mdot
.898 .898 .898 r
.35954 .0677 Mdot
1 0 0 r
.38696 .05973 Mdot
0 1 1 r
.41483 .05351 Mdot
.898 .898 .898 r
.44303 .04904 Mdot
.498 0 0 r
.47146 .04635 Mdot
1 0 1 r
.5 .04545 Mdot
0 .498 0 r
.52854 .04635 Mdot
1 0 0 r
.55697 .04904 Mdot
.58517 .05351 Mdot
0 1 0 r
.61304 .05973 Mdot
1 0 0 r
.64046 .0677 Mdot
0 0 .498 r
.66733 .07737 Mdot
1 1 0 r
.69354 .08871 Mdot
.498 0 0 r
.71898 .10168 Mdot
.898 .898 .898 r
.74356 .11621 Mdot
0 .498 0 r
.76718 .13227 Mdot
0 1 1 r
.78974 .14977 Mdot
.498 .498 .498 r
.81116 .16865 Mdot
1 0 1 r
.83135 .18884 Mdot
.85023 .21026 Mdot
.898 .898 .898 r
.86773 .23282 Mdot
1 0 0 r
.88379 .25644 Mdot
.898 .898 .898 r
.89832 .28102 Mdot
.498 0 0 r
.91129 .30646 Mdot
0 1 1 r
.92263 .33267 Mdot
0 0 .498 r
.9323 .35954 Mdot
1 0 0 r
.94027 .38696 Mdot
0 .498 0 r
.94649 .41483 Mdot
.95096 .44303 Mdot
.498 .498 .498 r
.95365 .47146 Mdot
.898 .898 .898 r
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[17]:=",
  ImageSize->{356, 356},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005T0001I2000`400?l00000o`00003ooooooomU
ooooo`00oooooomUooooo`00oooooomUooooo`00oooooomUooooo`00oooooomUooooo`00oooooomU
ooooo`00oooooomUooooo`00oooooomUooooo`00oooooomUooooo`00oooooomUooooo`00oooooomU
ooooo`00oooooomUooooo`00oooooomUooooo`00oooooomUooooo`00YOooool4ogl000Kooooo1?oo
0?l6ooooo`Co07l0Yoooool009_ooooo1?oUiND5ooooo`KoO`001?ooool6ool0o`Cooooo1_l0O`05
ooooo`Coo`00WOooool0097ooooo1?l0ool5ooooo`KoiNGU0oooool8ogl000;ooooo2?oo0?l2oooo
o`So07l00oooool6ool000Gooooo1?oo002Cooooo`00T?ooool6o`3oo`?ooooo2?oUiND2ooooo`So
O`000_ooool8ool0o`03o`000?oooooo07l000Oo07l00_ooool8ool000?ooooo1_oo002Booooo`00
Soooool8o`3oo`;ooooo2?oUiND2ooooo`SoO`000_ooool8ool0o`;o00002?l0O`02ooooo`Soo`00
0_ooool8ool0097ooooo0027ooooo`Coo`001?ooool8o`3oo`;ooooo2?oUiND2ooooo`SoO`000_oo
ool8ool0o`03o`000?oooooo07l000Oo07l00_l00008ool000;ooooo2?oo0004ooooo`Co0?l0ROoo
ool008Kooooo1_oo0003ooooo`So0?oo0_ooool8onGUi@?ooooo1_mo0004ooooo`Koo`3o0_ooool2
o`0000Ko07l00oooool8ool000;o00002?oo0003ooooo`Ko0?l0R?ooool007gooooo1?oUiND4oooo
o`Soo`000_ooool8o`3oo`03o`000?oooooooooo00KoiNGU1Oooool4ogl00003ooooool0003o0000
00;o000000?oooooool0oooo0?l00_oo0?l00ol0003oooooooooo`02ooooo`03o`000?l0O`3o07l0
00;o07l01Oooool6ool000?ooooo2?oo0002o`0000So0?l01?ooool4ool007oooooo001looooo`Ko
iNGU0oooool8ool000?o00001_l0ool2ooooo`03o`000?oooooooooo00CoiNGU1oooool4o`0000Ko
oooo00Go0000ooooooooooooooooo`000005ooooo`05o`000?ooooooooooo`000?ooool01?l00004
ool000?ooooo0_l00006ool000?o00002?l0o`03ooooo`Koo`00O_ooool007_ooooo2?oUiND2oooo
o`Soo`000oooool00ol0003o0?ooo`3oo`02o`3ooa7o00000_ooool01?l0003ooooooooooooooolN
o`0000?ooooo1?oo0004ooooo`So0?l00_ooool8ool007gooooo001kooooo`SoiNGU0_ooool8ool0
0003ooooool0003o000002?o00000_ooool6o`0000;ooooo0_l00007ooooo`03o`000?oooooooooo
00?ooooo00Co0000oooooooooooo00001Oooool8o`3o00;ooooo2?oo001mooooo`00M?ooool4ool0
00?ooooo2?oUiND3o`0000Koo`000_l000000ooooooo0000ooooo`09ooooo`Co00001oooool01Ol0
003ooooooooooooooooo000000Kooooo00?o0000ooooool000004Ol00002ooooo`03o`000?oooooo
oooo00?ooooo00Go0000ooooooooooooooooo`000005ooooo`Ko0?l00oooool8ool000?ooooo1?l0
07mfooooo`00Loooool6ool000;ooooo2?oUiND4ooooo`Coo`0000Co0000oooooooooooo00002?oo
ool3o`000003ooooool0003ooooo00?o00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00;ooooo1?l00005ooooo`03o`000?oooooooooo00Cooooo00Co0000ooooool0003o0000
0_ooool@o`0000?ooooo1?l0o`04ooooo`Soo`000_ooool6o`00OgGooooo001booooo`Soo`0000?o
0000oooooooUiND01OoUiND:ooooo`;o00001Oooool4o`0000Cooooo00?o0000ooooool000000_oo
ool01Ol0003ooooooooooooooooo000000Cooooo1Ol000000ooooooo0000ooooo`06ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool00_l00002ooooo`;o00000oooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooooooo`co00001_oo0002ooooo`So001oM?ooool007;ooooo
2?oo0003ooooo`CoiNGU2oooool2o`000003ooooool0003o000000;o00002?ooool00ol0003ooooo
o`000003ooooo`;o000000?oooooo`000?l000000ol000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`Co0000
0_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1?ooool4ool000?o00002?l007mdooooo`00J_ooool4ogl000Cooooo2?oo000Aooooo`Co00002ooo
ool01?l0003oooooooooool00002ooooo`Go00001Oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`;o0000
00?oooooo`000?ooool01Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo000000[o0000
00?oooooo`00Ool007l01_l007l4ooooo`Coool0K?ooool006Wooooo1_mo0003ooooo`Soo`003Ooo
ool5o`0000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooo`000003o`0000?ooooo0ol0
0004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00?ooooo1?l00006ooooo`03o`000?oooooo000000Wo00002ooo
ool00ol0003o001oo`00O`06o`00O`?ooooo1_ooo`1[ooooo`00J?ooool8ogl000?ooooo1_oo000:
ooooo`Co00000oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Gooooo1Ol00000
0ooooooo0000ooooo`04ooooo`03o`000?oooooooooo00;o00000_ooool01Ol0003ooooooooooooo
oooo000000_ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`_o00002?oo
ool00ol0003oooooooooo`0;ooooo`03o`000?l007oo001o00Co001o0oooool8oooo06[ooooo001X
ooooo`SoO`001?ooool4ool000Oooooo1?l00006ooooo`03o`000?oooooooooo00?ooooo00Co0000
oooooooooooooooo1?l00003ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool0
0oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooool0
00002ol00002ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0<ooooo`03o`000?l007oo001o00;o001o1?ooool8oooo06[ooooo001Xooooo`SoO`00
1oooool2o`0000?ooooo0ol0000:ooooo`03o`000?oooooooooo00;ooooo1Ol00006ooooo`05o`00
0?ooooooooooooooool000001?ooool00ol0003oooooooooo`04ooooo`?o00000_ooool00ol0003o
ooooooooo`02ooooo`[o00001?ooool00ol0003oooooooooo`03ooooo`;o00000oooool2o`0000;o
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0<ooooo`;o00000_ooool00ol0003o
ooooooooo`02ooooo`Soool0J_ooool0067ooooo1?ooo`03ooooo`SoO`001oooool5o`0000cooooo
00?o0000ooooool000000ol00003ooooo`03o`000?oooooooooo00Gooooo00Go0000oooooooooooo
ooooo`000004ooooo`03o`000?oooooooooo00Cooooo2_l00009ooooo`03o`000?oooooooooo00?o
oooo00Co0000oooooooooooooooo0ol00005ooooo`?o00002Oooool00ol0003oooooooooo`0<oooo
o`06o`000?oooooo0000ooooool0003ooooo0ol00008oooo00?ooooo1?mo001Sooooo`00H?ooool6
oooo00?ooooo1_mo0005ooooo`Go00003?ooool5o`0000Sooooo00?o0000ooooooooool01?ooool0
1?l0003oooooooooooooool<o`0000?ooooo0ol0000>ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooooooo0ol00007ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
2oooool2o`0000;ooooo0ol00004ooooo`Koool00oooool6ogl006;ooooo001Oooooo`Soool00ooo
ool4ogl000;ooooo1?l00004ooooo`;o00001oooool4o`000003ooooool0003ooooo00[ooooo00?o
0000ooooool000002Ol00003ooooo`03o`000?oooooooooo00Sooooo0ol0000?ooooo`05o`000?oo
ooooooooooooool000000_ooool01?l0003oooooo`000?l00008ooooo`;o00001oooool00ol0003o
ooooooooo`0:ooooo`Go000000?oooooo`000?ooool01Oooool4oooo0003o`000?oooooooooo00So
O`00HOooool005oooooo2?ooo`03ooooo`Ko00002?ooool2o`0000;ooooo1Ol00004ooooo`03o`00
0?oooooooooo00;ooooo2_l00004ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo
ooooo`00000:ooooo`;o000000?oooooo`000?l000003Oooool00ol0003oooooooooo`02ooooo`07
o`000?oooooo0000ooooool0003oooooo`000007ooooo`04o`000?oooooo0000o`0000Gooooo00?o
0000ooooooooool02_ooool2o`0000Cooooo0_l00007ooooo`05o`000?ooooooooooo`000?ooool0
2?mo001Qooooo`00Goooool8oooo0003ooooool0003o000000;o00003?ooool4o`0000?ooooo2ol0
000:ooooo`05o`000?ooooooooooooooool000001?ooool01?l0003oooooooooool0000;ooooo`;o
00000oooool00ol0003oooooooooo`0;ooooo`04o`000?ooooooooooooooo`;o00000_ooool00ol0
003oooooo`000007ooooo`04o`000?ooooooooooooooo`;o00000oooool00ol0003oooooooooo`07
ooooo`Co00001Oooool00ol0003oooooo`000006ooooo`05o`000?ooooooooooooooool000002?mo
001Qooooo`00FOooool4ogmoO`;ooooo2?ooo`000ol0003oooooooooo`02o`0000Oooooo3?l00005
ooooo`03o`000?oooooooooo00gooooo00Go0000ooooooooooooooooo`000004ooooo`04o`000?oo
ooooooooo`0000cooooo00?o0000ooooooooool00_ooool2o`0000cooooo00Go0000oooooooooooo
ooooo`000003ooooo`;o00001oooool00ol0003oooooooooo`03ooooo`05o`000?oooooooooooooo
ool000001Oooool3o`000004ooooool0003oooooo`0000Gooooo00Co0000oooooooooooo00001Ooo
ool01Ol0003ooooooooooooooooo000000SoO`000oooool4onGUiE[ooooo001Hooooo`KoOgmo0_oo
ool6oooo00?ooooo00?o0000ooooool000002Ol00004ooooo`;o00002?ooool00ol0003ooooooooo
o`0?ooooo`03o`000?oooooo000000Gooooo00Co0000oooooooooooo00003?ooool00ol0003ooooo
ooooo`04ooooo`;o00002_ooool00ol0003oooooooooo`02o`0000?ooooo0_l00007ooooo`03o`00
0?oooooooooo00Cooooo0_l000001?oooooo0000ooooooooool3o`0000Cooooo0_l00006ooooo`05
o`000?ooooooooooooooool000001?ooool00ol0003oooooooooo`02ooooo`03o`000?mo003oO`00
00CoO`000oooool6onGUiEWooooo001Gooooo`SoOgmo0_l00004oooo00[o00002_ooool2o`0000Oo
oooo00?o0000ooooooooool04Oooool2o`0000Gooooo00?o0000ooooool000003Oooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00Sooooo0_l000000ooooooo0000ooooo`02ooooo`;o
00001oooool00ol0003oooooooooo`06ooooo`Co00001_ooool00ol0003oooooo`000007ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?mo003oO`00
00;oO`000oooool8onGUiESooooo001Gooooo`SoOgmo00?o0000ooooooooool01?ooool6o`0000go
oooo00?o0000ooooool000001_ooool00ol0003oooooooooo`0Aooooo`;o00001Oooool00ol0003o
ooooo`00000<ooooo`;o00002Oooool2o`0000Wooooo00Co0000oooooooooooo00000_ooool00ol0
003oooooo`000006ooooo`03o`000?oooooooooo00Gooooo1?l00006ooooo`03o`000?oooooo0000
00Sooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00;ooooo0_l00007
ooooo`SoiNGUF?ooool005Oooooo2?moOgl3ooooo`Co00000_ooool00ol0003oooooo`00000?oooo
o`;o00001Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ool000003?ooool00ol0003oooooo`00000:ooooo`03o`000?oooooooooo00Gooooo0_l00002oooo
o`06o`000?ooooooooooo`000?oooooo00001_ooool00ol0003oooooooooo`02ooooo`?o00000_oo
ool01?l0003oooooo`000?l00004ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00003ooooo`04o`000?ooooooooooo`0000Kooooo2?oUiNEHoooo
o`00D?ooool4o`00O`?ooooo2?moOgl3o`0000Kooooo00?o0000ooooool000003oooool00ol0003o
ooooo`000003ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooool000001?ooool2o`0000go
oooo00?o0000ooooool000002oooool2o`0000Gooooo00?o0000ooooool000000_ooool01Ol0003o
ooooo`000?oooooo000000Kooooo00?o0000ooooool000000_l00005ooooo`04o`000?oooooooooo
ooooo`;o000000Coooooo`000?oooooo00002Oooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooo000000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`03ooooo`SoiNGU0_oo
ool4o`1o05;ooooo001?ooooo`Ko001o0oooool6ogmoO`[ooooo00?o0000ooooool000003oooool0
0ol0003oooooo`000002ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooool000001?ooool2
o`0000gooooo00?o0000ooooool000003Oooool01Ol0003ooooooooooooooooo000000?ooooo00Oo
0000ooooool0003oooooo`000?oooooo000000Gooooo0ol00008ooooo`03o`000?oooooooooo00?o
oooo0_l000000ooooooo0000ooooo`08ooooo`03o`000?oooooooooo00Cooooo0_l000000ooooooo
0000ooooo`04ooooo`03o`000?oooooooooo00Cooooo1_oUiND2ooooo`Ko07l0DOooool004kooooo
2?l007l3o`0000CoOgmo00?o0000ooooooooool01oooool01?l0003oooooooooool0000?ooooo`05
o`000?oooooo0000ooooool000005oooool01?l0003oooooooooool00003ooooo`;o00003Oooool0
0ol0003oooooo`00000>ooooo`?o00001?ooool01ol0003oooooo`000?oooooo0000ooooool00000
0oooool2o`000003ooooool0003ooooo00Sooooo00?o0000ooooooooool00oooool3o`0000[ooooo
00?o0000ooooooooool01Oooool2o`0000Oooooo00?o0000ooooooooool01?ooool4onGUi@;ooooo
2?l0O`1@ooooo`00C_ooool8o`00O`;o00001_ooool00ol0003oooooooooo`06ooooo`04o`000?oo
ooooooooo`00013ooooo00Co0000ooooool0003o00005oooool01Ol0003ooooooooooooooooo0000
00;ooooo00?o0000ooooooooool02oooool01Ol0003ooooooooooooooooo000000oooooo0_l00004
ooooo`04o`000?oooooo0000ooooo`Go00001?ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool00ol00008ooooo`03o`000?oooooooooo00Gooooo0_l0
0008ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool8o`1o053ooooo001>
ooooo`So001o0_ooool:o`0000Cooooo00Go0000ooooooooooooooooo`00000@ooooo`03o`000?oo
oooo000001Oooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0000=ooooo`05o`00
0?ooooooooooooooool000003_ooool01?l0003oooooooooool00004ooooo`Ko00001_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000oooool2o`00
00Kooooo00?o0000ooooooooool01?ooool2o`000003ooooool0003ooooo00Kooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`04ooooo`So07l0D?ooool004kooooo2?l007l9ooooo`03
o`000?oooooooooo00[o00003oooool3o`0001Oooooo00?o0000ooooooooool00oooool00ol0003o
ooooo`00000=ooooo`05o`000?ooooooooooooooool000003_ooool01?l0003oooooooooooooool2
o`000003ooooool0003o000000?o000000?oooooo`000?ooool01Oooool00ol0003oooooooooo`07
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00;ooooo00Ko0000oooooooooooo0000ooooool00007ooooo`05
o`000?ooooooooooooooool000001_ooool8o`1o053ooooo001?ooooo`Ko001o2_ooool00ol0003o
ooooooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo2ol00004ooooo`04o`000?oooooooooo
o`0001Kooooo00?o0000ooooooooool01?ooool2o`0000gooooo00Go0000ooooooooooooooooo`00
000=ooooo`03o`000?oooooooooo00;ooooo0ol00002ooooo`?o000000?oooooo`000?ooool01Ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01Ooo
ool2o`0000Cooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000Oo
oooo00Co0000oooooooooooo00001oooool6o`1o0003ooooool0003ooooo04kooooo0018ooooo`Co
Ogmo1?ooool4o`00O`;o00002_ooool00ol0003oooooooooo`02ooooo`05o`000?oooooooooooooo
ool000003Oooool:o`0001Cooooo00?o0000ooooooooool01?ooool2o`0000gooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`09ooooo`04o`000?ooooooooooooooo`;o00000oooool2
o`000005ooooool0003o0000ooooool000001_ooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooo000000;ooooo00?o0000ooooooooool01oooool2o`0000;ooooo00Co0000oooooooooooo0000
1?ooool01Ol0003ooooooooooooooooo000000Oooooo0_l00009ooooo`Co07l00oooool00ol0003o
0?ooo`3oo`02o`3ood[ooooo0017ooooo`KoOgmo2?ooool2o`0000[ooooo00Go0000oooooooooooo
ooooo`000003ooooo`03o`000?oooooooooo00kooooo00Go0000ooooool0003oooooo`000002oooo
o`[o00002Oooool00ol0003oooooooooo`05ooooo`;o00003?ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Wooooo1?l00007ooooo`05o`000?oooooo0000ooooool000001_ooool0
0ol0003oooooooooo`07ooooo`;o00000_ooool00ol0003oooooooooo`0:ooooo`05o`000?oooooo
0000ooooool000001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo0_l0
000@ooooo`Ko0?ooBOooool004Kooooo2?moOgl8ooooo`;o00002Oooool01Ol0003ooooooooooooo
oooo000000?ooooo00?o0000ooooooooool03Oooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00Wooooo2_l00006ooooo`04o`000?oooooo0000o`0000_ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`07ooooo`?o00002oooool2o`0000;ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`07ooooo`;o00000_ooool00ol0003oooooooooo`0;ooooo`?o
00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03Oooool8o`3oodSooooo0016ooooo`SoOgmo2?ooool2o`0000[ooooo00?o0000ooooool00000
1?ooool00ol0003oooooooooo`0<ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003o
ooooooooo`0Booooo`_o00002oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Co
oooo0ol000000ooooooo0000ooooo`0<ooooo`;o000000?oooooo`000?ooool01?ooool00ol0003o
ooooooooo`07ooooo`04o`000?ooooooooooo`0000oooooo0ol00005ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00000>ooooo`So0?ooB?ooool004Ko
oooo2?moOgl9ooooo`;o00002Oooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool00oooool01?l0003oooooooooool0000Cooooo`03o`000?oooooooooo
00Cooooo00Co0000ooooool0003ooooo2_l000000ooooooo0000ooooo`05ooooo`03o`000?oooooo
oooo00?o00000oooool00ol0003oooooooooo`0=ooooo`?o00001Oooool00ol0003oooooooooo`06
ooooo`;o00000_ooool00ol0003oooooooooo`0<ooooo`;o00000_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Gooooo00Oo0000ooooooooooooooooo`000?oooooo000000kooooo
2?l0oom8ooooo`00A_ooool8ogmoO`Wooooo00?o0000ooooool000002Oooool2o`0000?ooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0001;ooooo
00?o0000ooooooooool01?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Ko
oooo2ol00006ooooo`03o`000?oooooooooo00gooooo1Ol00003ooooo`03o`000?oooooooooo00Ko
oooo0_l000000ooooooo0000ooooo`0>ooooo`03o`000?oooooo000000;ooooo0_l00002ooooo`03
o`000?oooooooooo00Kooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0;oooo
o`So0?ooB?ooool0047ooooo1?ooo`02ooooo`KoOgmo2oooool00ol0003oooooo`000008ooooo`;o
00000oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooo
oooo00004_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`02ooooo`?o000000?oooooo`000?l000002Ol0000<
ooooo`?o00000_ooool01Ol0003ooooooooooooooooo000000Kooooo00Go0000ooooool0003ooooo
o`00000>ooooo`04o`000?ooooooooooo`0000Cooooo0ol00009ooooo`;o00001?ooool00ol0003o
ooooooooo`0:ooooo`03o`000?l0oooo0?oo00Co0?oo0oooool4ogmoOd;ooooo0010ooooo`Koool0
00?oooooo`000?moOgl00omoOgl=ooooo`03o`000?oooooo000000Oooooo0_l00003ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool01_ooool01?l0003oooooooooool0000Aooooo`03
o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00?o00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00?ooooo2_l00002ooooo`;o000000Coooooo`000?oooooooooo0_l000000ooooooo0000ooooo`04
ooooo`03o`000?oooooooooo00;o00003_ooool01Ol0003ooooooooooooooooo000000Kooooo0_l0
0008ooooo`;o00001?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Co0?oo0ooo
ool6ogmoOd7ooooo000oooooo`Soool000?o0000ooooooooool03oooool00ol0003oooooo`000007
ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`07
ooooo`04o`000?ooooooooooo`00017ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooo
oooooooo000000Cooooo00?o0000ooooooooool01?ooool3o`0000Kooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`0=ooooo`co00001Oooool01Ol0003ooooooooooooooooo000000go
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000So
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oo
oooo2?moOgm0ooooo`00?oooool8oooo01?ooooo00?o0000ooooool000001Oooool01?l0003ooooo
ooooool00002ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01oooool01?l0003o
ooooooooool0000@ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000005
ooooo`03o`000?oooooooooo00Co00002?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo013ooooo00?o0000ooooool000000_l00003ooooo`co00003?ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_l00005ooooo`03o`000?oooooo
000000Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`SoOgmo@?ooool0
03oooooo2?ooo`0Dooooo`03o`000?oooooo000000Cooooo00Oo0000ooooooooooooooooo`000?oo
oooo000000[ooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000oooooo
00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000Gooooo0ol00003ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0@ooooo`06
o`000?oooooo0000o`000?oooooo00000oooool00ol0003oooooo`000002ooooo`04o`000?oooooo
ooooooooo`[o00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool00_ooool2o`0000?ooooo00Co0000oooooooooooo00001?ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Wooooo2?moOgm0ooooo`00?oooool8oooo01Cooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02_ooool01?l0003oooooooooool0000?ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooooooool4o`0000Cooooo00?o0000oooo
ooooool01_ooool01Ol0003ooooooooooooooooo000001?ooooo00Ko0000oooooooooooo0000oooo
ool00003ooooo`03o`000?oooooooooo00;o00000oooool00ol0003oooooo`000008ooooo`_o0000
1_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo2?moOgm0ooooo`00
@?ooool6oooo01Kooooo00Ko0000oooooooooooo0000ooooool00005ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool02oooool01?l0003oooooooooool0000?ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool00_ooool4o`0000Cooooo00Co0000oooooooooooo00002_oo
ool00ol0003oooooo`00000Eooooo`06o`000?oooooo0000o`000?oooooo00000_ooool01Ol0003o
ooooooooool0003ooooo00;o000000Coooooo`000?oooooo00002Oooool00ol0003oooooooooo`06
ooooo`[o00001Oooool2o`0000Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
000<ooooo`03o`000?ooooooOgmo00GoOgmo@Oooool003Wooooo1?l0ool4ooooo`Coool05oooool0
1?l0003oooooooooooooool2o`0000Gooooo0_l00008ooooo`03o`000?oooooooooo00_ooooo00Go
0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
0ol00007ooooo`04o`000?ooooooooooo`0000[ooooo00?o0000ooooool000005Oooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooo000000;ooooo00Go0000ooooooooooooooooo`000002
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00_o00000_ooool01ol0003ooooooooooooooooo0000ooooool00000
3?ooool01?l0003oooooooooooooool4ogmoO`03o`000?oooooooooo00Coo`3o>oooool003Sooooo
1_l0ool4ooooo`03o`000?oooooooooo01Sooooo00Co0000oooooooooooo00001_ooool2o`0000Oo
oooo00?o0000ooooooooool03?ooool01Ol0003ooooooooooooooooo000000gooooo00?o0000oooo
ooooool00oooool4o`0000;ooooo00?o0000ooooooooool01_ooool00ol0003oooooo`00000:oooo
o`;o00005_ooool01?l0003oooooooooool00002ooooo`05o`000?oooooo0000ooooool000000ooo
ool00ol0003oooooo`000002o`0000Oooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00Kooooo00Go0000oooooooooooo0000ooooo`0:o`0000[ooooo
00?o0000ooooooooool01_ooool00ol0003oooooool0o`05ool0oc[ooooo000gooooo`So0?oo0ooo
ool00ol0003oooooooooo`0Iooooo`04o`000?oooooo0000o`0000Gooooo00?o0000ooooool00000
1Oooool00ol0003oooooooooo`0>ooooo`04o`000?ooooooooooo`0000gooooo00?o0000oooooooo
ool00_ooool2o`0000Gooooo00?o0000ooooooooool01oooool2o`0000[ooooo0_l0000Fooooo`04
o`000?ooooooooooo`0000?ooooo0ol00004ooooo`05o`000?ooooooooooooooool000001_ooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`02ooooo`;o00000oooool01_l0003oooooo`000?oooooo0000ooooo`[o0000
2_ooool8ool0ocWooooo000gooooo`So0?oo0_ooool00ol0003oooooooooo`0Jooooo`05o`000?oo
oooo0000ooooool000001?ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00oo
oooo00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00?o000000?oooooo`00
0?ooool01?ooool00ol0003oooooooooo`07ooooo`;o00002_ooool2o`0001Kooooo00?o0000oooo
ooooool00_l00003ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`?o00000oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`;o00000_ooool01?l0003oooooooooool00009ooooo`03o`000?oooooo
000000Wo00002?oo0?liooooo`00=oooool8o`3oo`;o00007_ooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo013ooooo00Go
0000ooooooooooooooooo`00000;ooooo`?o00001?ooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00Oooooo0_l00009ooooo`03o`000?oooooo000001Kooooo00Go0000oooooooooooo
ooooo`000003ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooo`00
0?oooooo00003_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01_ooool2o`000005ooooool0003oooooooooool000001oooool00ol0003ooooooooo
o`0:ooooo`Soo`3o>Oooool003Oooooo2?l0ool00ooooooo0000o`000006o`0001Oooooo0_l00003
ooooo`03o`000?oooooo000000Cooooo00?o0000ooooool000004_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Gooooo1?l00005ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01oooool00ol0003oooooo`000008ooooo`03o`000?oooooo000001Kooooo00Co0000
oooooooooooooooo0_l000001?oooooo0000ooooool00002ooooo`03o`000?oooooooooo00?ooooo
00Co0000oooooooooooooooo0_l0000>ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`06ooooo`Co000000Goooooo`000?ooooooooooo`000005oooo
o`03o`000?oooooooooo00_ooooo2?oo0?liooooo`00>?ooool6o`3oo`03ooooool0003ooooo00Oo
oooo2?l0000?ooooo`;o00001?ooool2o`0000Cooooo0_l0000Dooooo`05o`000?oooooooooooooo
ool000001?ooool3o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooool000001oooool01Ol0003ooooooooooooooooo
000001Gooooo00Co0000oooooooooooooooo0_l000000ooooooo0000ooooo`02o`000003ooooool0
003ooooo00Gooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`0;ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00;ooooo0ol000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00gooooo1_oo
0?ljooooo`00>Oooool4o`3oo`03ooooool0003ooooo013ooooo2?l00006ooooo`04o`000?oooooo
ooooo`0000?ooooo0_l00004ooooo`;o00005?ooool01Ol0003ooooooooooooooooo000000;ooooo
0_l00005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`07ooooo`04o`000?ooooooooooo`0000Kooooo00Go0000ooooooooooooooooo`00000Foooo
o`03o`000?oooooooooo00?o00000_ooool3o`0000Oooooo0_l00004ooooo`;o00002oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`;o000000Goooooo`000?ooooooooooo`00000Aooooo`Coo`3o>oooool0
03?ooooo1?l0O`06ooooo`03o`000?oooooooooo01Sooooo2?l00002ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooo`00000Dooooo`04o`000?ooooooooooooooo`?o00001ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01ooo
ool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0Dooooo`06o`000?ooooooooooo`000?oooooo00000_ooool00ol0003oooooo`000007oooo
o`03o`000?oooooooooo00Gooooo0_l0000:ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00Co0000oooo
ool0003o00005?ooool3o`000003oooooooo0?ooo`3o00;oo`3o=Oooool003;ooooo1_l0O`05oooo
o`03o`000?oooooooooo01gooooo00?o0000ooooooooool02Ol000001Ooooooo0000oooooooooooo
000001?ooooo1?l000000ooooooo0000ooooo`07ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01Cooooo00Co0000
oooooooooooooooo0_l00002ooooo`;o000000?oooooo`000?ooool01Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_ooool2o`0001Kooooo0_l0
0006ool0ocCooooo000aooooo`So07l00oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooo
oooo00?ooooo00Go0000oooooooooooo0000ooooo`09o`0000kooooo1?l00003ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00006ooooo`03o`00
0?oooooooooo01?ooooo00?o0000ooooooooool00_l000000ooooooo0000ooooo`02o`0000;ooooo
00?o0000ooooooooool00_ooool2o`0000[ooooo0_l00007ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00_ooooo
00Co0000ooooool0003o00005?ooool00ol0003oo`3oool0o`06ool0oc?ooooo000aooooo`So07l0
0_ooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool00000
0_ooool2o`0000Cooooo00?o0000ooooooooool02?l00004ooooo`;o00000oooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool2o`0000Oooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`02o`000004ooooool0003oooooo`0000?o
oooo00Oo0000ooooooooooooooooo`000?oooooo000000cooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`0;ooooo`04o`000?oooooo0000ooooo`;o00004oooool00ol0003oo`3oool0
o`06ool0oc?ooooo000aooooo`So07l01?l0000Oooooo`03o`000?oooooooooo00Gooooo0_l00000
1?oooooo0000ooooool00005ooooo`03o`000?oooooooooo00Oooooo2?l000000ooooooo0000oooo
o`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool2o`0000Oooooo00?o
0000ooooooooool04oooool02Ol0003oooooo`000?oooooo0000ooooool0003oooooo`000004oooo
o`03o`000?oooooo000000;ooooo00?o0000ooooooooool02oooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000003Oooool0
1?l0003oooooooooooooool2o`000003ooooool0003o000001;ooooo2?oo0?lcooooo`00<Oooool8
o`1o0004ooooool0003oooooooooo`Co00006oooool00ol0003oooooooooo`06ooooo`;o00000ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Gooooo0ol00007ooooo`So00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`04ooooo`;o00001oooool00ol0003oooooooooo`0Cooooo`;o00000_oo
ool00ol0003oooooooooo`02o`0000Gooooo00Go0000ooooooooooooooooo`00000?ooooo`05o`00
0?ooooooooooooooool000001?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00cooooo0_l00005ooooo`;o00000_ooool2o`00013ooooo2?oo0?lcooooo`00<_ooool6o`1o0003
ooooool0003ooooo00Kooooo1Ol0000Fooooo`03o`000?oooooooooo00Kooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooooooo0ol0000<ooooo`03o`000?oooooooooo
00;ooooo2Ol00002ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool04_ooool2o`00
00;ooooo00?o0000ooooooooool00_l00004ooooo`;o00000oooool00ol0003oooooooooo`0>oooo
o`;o000000?oooooo`000?ooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo0000
00cooooo00?o0000ooooooooool01_ooool2o`0000?ooooo00?o0000ooooooooool03Oooool00ol0
003oo`3oool0o`04ool0ocCooooo000cooooo`Co07l000?oooooo`000?ooool03?ooool4o`00017o
oooo00?o0000ooooooooool01oooool2o`0000Gooooo00?o0000ooooooooool00_ooool3o`0000oo
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooooooo
o`So00001?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00Co0000oooo
oooooooo00001oooool00ol0003oooooooooo`0Booooo`;o00000oooool2o`000003ooooool0003o
000000;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0@ooooo`;o00001?oo
ool01ol0003ooooooooooooooooo0000ooooool000003?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00;ooooo0_l0000=ooooo`03o`000?ooooooo`3o00?oo`3o=Oooool002gooooo
1?ooo`000ooooooo0000ooooo`04ooooo`03o`000?oooooooooo00oooooo1?l0000=ooooo`03o`00
0?oooooooooo00Kooooo0_l000000ooooooo0000ooooo`04ooooo`03o`000?oooooo000000;o0000
4Oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000001Oooool00ol0003o
ooooooooo`03ooooo`So00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Booooo`;o00000oooool2o`000005
ooooool0003o0000ooooool000001?ooool01?l0003oooooooooool0000Cooooo`05o`000?oooooo
ooooooooool000000oooool2o`0000cooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`03ooooo`;o00003?ooool01?l0003oooooo`000?l00004ooooo`CoiNGU;_ooool002cooooo1_oo
o`05ooooo`03o`000?oooooooooo01Cooooo1Ol00008ooooo`03o`000?oooooooooo00Gooooo0_l0
0002ooooo`03o`000?oooooooooo00;ooooo0ol00003ooooo`03o`000?oooooooooo00kooooo00?o
0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool0
2?ooool01?l0003oooooooooooooool8o`0000;ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo017ooooo00?o0000ooooool000000oooool00ol0003o
ooooooooo`03o`0000Gooooo00Co0000oooooooooooo00004oooool3o`0000;ooooo00Co0000oooo
oooooooo00003?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo0_l0000:
ooooo`03o`000?oooooooooo00;o00000_ooool6onGUiBgooooo000[ooooo`Soool00oooool00ol0
003oooooooooo`0Jooooo`Co00000oooool00ol0003oooooooooo`06ooooo`;o00000oooool4o`00
00;ooooo00Go0000ooooooooooooooooo`00000Aooooo`03o`000?oooooooooo00;ooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01Ooo
ool01?l0003oooooooooooooool8o`000003ooooool0003ooooo00Cooooo00?o0000ooooooooool0
3oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000001oooool00ol0
003oooooo`00000Cooooo`08o`000?ooooooooooo`000?oooooo0000ooooool0000<ooooo`03o`00
0?oooooooooo00gooooo0_l00007ooooo`03o`000?oooooooooo00Oooooo00Ko0000oooooooooooo
0000ooooool00008onGUiBcooooo000[ooooo`Soool00_ooool00ol0003oooooooooo`0Oooooo`Go
00001_ooool01?l0003oooooo`000?ooool4o`0000Gooooo00Go0000ooooooooooooooooo`00000@
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04
ooooo`So00004Oooool01?l0003oooooooooool00002ooooo`;o00000_ooool2o`0000Sooooo0_l0
000Cooooo`04o`000?ooooooooooooooo`Co00002oooool00ol0003oooooooooo`0?ooooo`;o0000
1oooool2o`0000Oooooo00Ko0000ooooooooooooooooo`000?ooool8onGUiBcooooo000[ooooo`So
ool00_ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooo000000?o000000?oooooo`000?oo
ool00ol00003ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`00000?oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00;ooooo2_l00007ooooo`05o`000?ooooooooooooooool000000_ooool0
1Ol0003oooooo`000?l0003ooooo00;o00001oooool2o`0001?ooooo00Co0000oooooooooooooooo
1?l0000:ooooo`03o`000?oooooooooo017ooooo0_l00008ooooo`;o00001Oooool01_l0003ooooo
oooooooooooo0000ooooo`SoiNGU;?ooool002_ooooo2?ooo`000ooooooo0000ooooo`0Sooooo`03
o`000?oooooooooo00Cooooo1Ol00005ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooo
oooo00003oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000Cooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000002oooool0
1Ol0003ooooooooooooooooo000000Wooooo2?l00003ooooo`03o`000?oooooo000000;ooooo00?o
0000ooooooooool00_l00008ooooo`03o`000?oooooooooo017ooooo00Go0000oooooooooooo0000
ooooo`04o`0000Sooooo00?o0000ooooooooool04oooool2o`0000Wooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000SoiNGU;?ooool002cooooo1_ooo`000ooooooo0000
ooooo`0Tooooo`03o`000?oooooooooo00;ooooo0ol00003ooooo`Go000000?oooooo`000?ooool0
1_ooool01?l0003oooooooooool0000?ooooo`03o`000?oooooooooo00?ooooo0_l00004ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo0000
00So00000_ooool2o`0000Sooooo0_l0000Booooo`03o`000?oooooo000000;ooooo0_l00002oooo
o`;o00001Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo000000Wooooo0_l00003oooo
o`06o`000?ooooooooooooooool0003ooooo1_oUiND]ooooo`00;Oooool4oooo0003ooooool0003o
oooo02Cooooo00?o0000ooooooooool00ol000000ooooooo0000ooooo`03ooooo`03o`000?oooooo
oooo00;ooooo1Ol00005ooooo`03o`000?oooooo000000oooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool01?ooool0
1?l0003oooooooooool0000=ooooo`05o`000?ooooooooooooooool000003Oooool00ol0003ooooo
ooooo`04ooooo`04o`000?oooooo0000ooooo`So00001Oooool2o`0001;ooooo0_l00003ooooo`;o
00001?ooool01Ol0003ooooooooooooooooo000001Sooooo00?o0000ooooool000002_ooool2o`00
0003ooooool0003ooooo00?ooooo00?o0000oooooooUiND00ooUiND^ooooo`009oooool4onGUi@03
ooooool0003ooooo00;ooooo00?o0000ooooool000009Oooool3o`0000?ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000?o000000Goooooo`000?oo
ooooooooo`00000>ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00003_ooool0
1?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00Gooooo0_l00002ooooo`06o`000?oo
ooooooooo`000?oooooo00000_ooool8o`00017ooooo0_l00003ooooo`03o`000?oooooooooo00Co
oooo0ol0000Jooooo`03o`000?oooooo000000_ooooo0_l00003ooooo`;o00000_ooool01?l0003o
ooooooooooooool4ool002Wooooo000Vooooo`KoiNGU1Oooool00ol0003oooooooooo`0Qooooo`Co
00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool00oooool4o`000003ooooool0003ooooo00cooooo00?o0000ooooooooool00oooool2o`0000?o
oooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`00
00oooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`04ooooo`;o00001?ooool0
0ol0003oooooooooo`02o`0000Oooooo00?o0000ooooooooool02?l00008ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool2o`0001[ooooo00Co0000oooooooooooo00002ooo
ool2o`000005ooooool0003oooooooooool000000_ooool00ol0003oooooool00005ool002Sooooo
000Uooooo`SoiNGU0oooool2o`00027ooooo0_l00002ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Gooooo1_l0
000;ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`03o`000?oooooooooo00_ooooo00?o
0000ooooooooool00oooool01?l0003oooooooooool0000@ooooo`04o`000?ooooooooooo`0000[o
oooo00?o0000ooooooooool01Oooool2o`0000Cooooo00Ko0000oooooooooooo0000ooooool00006
ooooo`04o`000?ooooooooooo`0000Oooooo2?l00002ooooo`05o`000?ooooooooooooooool00000
1_ooool00ol0003oooooooooo`02o`0001Wooooo00Co0000oooooooooooo00002oooool3o`0000;o
oooo00Co0000oooooooooooooooo2?oo000Wooooo`009Oooool8onGUi@?ooooo00?o0000ooooool0
00007Oooool3o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000002ooooo`Co
00001oooool01ol0003ooooooooooooooooo0000ooooool000000_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000004_ooool00ol0003oooooo`00000:
ooooo`03o`000?oooooooooo00Cooooo0ol00004ooooo`06o`000?ooooooooooo`000?oooooo0000
1oooool01?l0003oooooooooool0000<ooooo`03o`000?oooooo000000Oo00001?ooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooool000002oooool00ol0003o
ooooooooo`02o`000004ooooool0003oooooooooo`Soo`009oooool002Gooooo2?oUiND2ooooo`04
o`000?ooooooooooo`0001[ooooo0ol00007ooooo`05o`000?ooooooooooooooool000001oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Kooooo0_l00006ooooo`Go00000_ooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool000003Oooool00ol0003ooooooooo
o`03ooooo`;o00004oooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooool000001?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Cooooo
00Go0000ooooooooooooooooo`00000:ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00_l000000ooooooo0000o`000006o`0000?ooooo0_l0000Gooooo`04o`000?ooooooooooo`00
00Wooooo0_l00004ooooo`;o00000_ooool8ool002Oooooo000Uooooo`SoiNGU00?oooooo`000?oo
ool00_ooool00ol0003oooooooooo`0Eooooo`?o00002Oooool01Ol0003ooooooooooooooooo0000
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo0000
00[ooooo1?l00002ooooo`03o`000?oooooooooo00;o00003_ooool00ol0003oooooooooo`03oooo
o`;o00005?ooool2o`0000Sooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00004
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooo
oooooooo000000Wooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02o`0000?ooooo
00?o0000ooooooooool00oooool8o`0001Gooooo00Co0000oooooooooooo00001oooool00ol0003o
ooooo`000005ooooo`;o000000?oooooool00?oo00001_oo000Wooooo`009_ooool6onGUi@03oooo
ool0003ooooo00Cooooo00?o0000ooooooooool04Oooool3o`0000cooooo00Co0000oooooooooooo
00002?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Gooooo0_l0000=ooooo`Ko
000000?oooooo`000?ooool03Oooool00ol0003oooooooooo`02ooooo`;o00005_ooool2o`0000Oo
oooo00?o0000ooooooooool01?ooool2o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003o
ooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00Ko0000ooooool0003oooooo`000?ooool2
o`0000cooooo2_l0000>ooooo`04o`000?ooooooooooo`0000Gooooo00Go0000oooooooooooooooo
o`000005ooooo`?o00001_oo000Xooooo`009oooool4onGUi@;ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0?ooooo`;o00003oooool00ol0003oooooo`000009ooooo`03o`000?oo
oooooooo00_ooooo00?o0000ooooooooool01_ooool2o`0000cooooo00?o0000ooooool000000ooo
ool4o`0000cooooo00?o0000ooooooooool00_ooool2o`0001Kooooo0_l00006ooooo`03o`000?oo
oooooooo00Cooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`06ooooo`;o000000?oooooo`000?l000004?ooool2o`0000Kooooo2Ol0
0005ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooo00001?oo000Yooooo`00:?ooool2o`0000;ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0;ooooo`?o00004?ooool01?l0003ooooo
ooooool00009ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01_ooool2o`0000_o
oooo00?o0000ooooool000000oooool2o`0000;ooooo1Ol00007ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`05o`000?oooooooooooooo
ool000002Oooool2o`000003ooooool0003ooooo01;ooooo00?o0000ooooooooool03?ooool8o`00
00;ooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0000
00?o0000:_ooool002;ooooo1?oo0?l2ooooo`04o`000?oooooo0000o`0000Sooooo00?o0000oooo
ooooool02?ooool3o`0001?ooooo00?o0000ooooool000002_ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02Oooool00ol0003oooooo`000003oooo
o`03o`000?oooooo000000Kooooo1?l00003ooooo`04o`000?ooooooooooooooo`;o00006?ooool2
o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooo`000005ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Cooooo00Co0000oooooooooooo00002_ooool01?l0003oooooo`000?l0000Dooooo`;o0000
4oooool9o`0000;ooooo00?o0000ooooooooool01?ooool2o`0000?ooooo0_l000000oooooooiNGU
onGUi@02onGUiBCooooo000Qooooo`Koo`3o00?oooooo`000?ooool00_l00009ooooo`03o`000?oo
oooooooo00Cooooo0ol0000Fooooo`;o00002oooool00ol0003oooooooooo`0=ooooo`03o`000?oo
oooooooo00Oooooo00?o0000ooooooooool02Oooool2o`0000?ooooo00?o0000ooooool000002_oo
ool4o`0000?ooooo0_l0000Iooooo`05o`000?ooooooooooooooool000001Oooool01?l0003ooooo
ooooool00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Gooooo0_l0000;ooooo`;o000000?oooooo`000?oo
ool05Oooool00ol0003oooooooooo`0@ooooo`06o`000?ooooooooooo`000?oooooo00000oooool8
o`000003ooooool0003o000000Gooooo1_oUiNDSooooo`008?ooool8ool0o`03o`000?oooooo0000
00[ooooo00?o0000ooooooooool00_ooool2o`0001Sooooo0_l0000<ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`09ooooo`05o`000?oooooooooo
ooooool000000_ooool00ol0003oooooooooo`0;ooooo`Ko00006Oooool2o`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00?ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00
000;ooooo`04o`000?ooooooooooo`0001Oooooo0_l0000Aooooo`;o00000oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;ooooo1ol00008onGUiB;ooooo000Pooooo`Soo`3o0ol0
000;ooooo`Co00006_ooool2o`0000cooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool00_l000000ooooooo0000o`000002o`0001Gooooo
00Co0000ooooool0003o00001?ooool01Ol0003ooooooooooooooooo000000Kooooo00?o0000oooo
ooooool00oooool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool000001Oooool0
1?l0003oooooooooool0000:ooooo`;o00000_ooool00ol0003oooooooooo`0Gooooo`03o`000?oo
oooooooo00kooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooooooo`SoiNGU8_ooool0023ooooo2?oo0?l00ol0003ooooo
o`000009ooooo`?o00007?ooool2o`0000cooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000003ooooo`03
o`000?oooooooooo00[ooooo00Go0000ooooool0003oooooo`000005ooooo`Go00004?ooool00ol0
003oooooo`000004ooooo`05o`000?ooooooooooooooool000001oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooo
oooo000000Cooooo00?o0000ooooooooool01_ooool3o`0000;ooooo00?o0000ooooooooool06?oo
ool2o`0000gooooo00?o0000ooooool000001Oooool01Ol0003ooooooooooooooooo000000;ooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooooooool8onGUiB;ooooo000Pooooo`Soo`3o
00?o0000ooooool000001_ooool3o`0000?ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0;ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01_ooool2o`0000Oooooo
00Go0000ooooool0003oooooo`000004ooooo`03o`000?oooooooooo00[ooooo00Go0000ooooool0
003oooooo`00000:ooooo`Co00003Oooool3o`0000;ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001Oooool01Ol0
003ooooooooooooooooo000000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04
ooooo`04o`000?oooooo0000o`0000;ooooo00?o0000ooooooooool06_ooool2o`0000[ooooo00Go
0000ooooooooooooooooo`000005ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`08onGUiB;ooooo000Qooooo`Koo`3o00Co0000oooooooooooo
00000oooool3o`0000Kooooo00?o0000ooooooooool06?ooool2o`0000gooooo00?o0000oooooooo
ool04Oooool00ol0003oooooooooo`07ooooo`;o00001_ooool01Ol0003oooooo`000?oooooo0000
00Gooooo00?o0000ooooooooool02Oooool2o`0000;ooooo00?o0000ooooooooool03?ooool5o`00
00Sooooo00?o0000ooooooooool00_l00004ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`000005ooooo`07o`000?ooooooooooooooool0003oooooo`00
0007ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000000_ooool00ol0003oooooo`00
000Nooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;o00001_ooool01?l0003oooooooooooooool6onGUiB?ooooo
000Rooooo`Coo`3o0_l00002ooooo`04o`000?oooooo0000o`0000[ooooo00?o0000ooooooooool0
5_ooool2o`0000kooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooo000000Gooooo00Go0000ooooool0003oooooo`000005ooooo`03o`000?oooooooooo00Wo
oooo00Go0000ooooooooooooooooo`00000Cooooo`Co00000oooool2o`0000;ooooo0_l00004oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000005oooo
o`04o`000?ooooooooooooooo`;o00002Oooool01Ol0003ooooooooooooooooo000000;ooooo00Ko
0000oooooooooooo0000ooooool0000Oooooo`;o00001Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooool000002?ooool01?l0003oooooooooooooool4onGU
iBCooooo000Sooooo`03o`000?oooooo000000;o000000Coooooo`000?l0003o00003?ooool00ol0
003oooooooooo`0Fooooo`;o00003_ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Kooooo0_l00005ooooo`04o`000?oooooo0000o`0000Oooooo00?o0000ooooooooool02?ooool0
1?l0003oooooooooool0000Hooooo`Oo000000?oooooo`000?ooool00_ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00?ooooo0_l00006ooooo`04o`000?ooooooooooo`0000_ooooo
00?o0000ooooool000000oooool01_l0003oooooooooool0003oooooo`00027ooooo00Go0000oooo
ooooooooooooo`000008ooooo`03o`000?oooooooooo00Cooooo0_l00008ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool08Oooool001gooooo1?l0o`001?oooooo0000ooooooooool4
o`0000;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
000000kooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`07ooooo`;o00000oooool0
1Ol0003ooooooooooooooooo000000Oooooo00?o0000ooooooooool01oooool2o`0000;ooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`05o`0000;ooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`03ooooo`;o00001_ooool01?l0003oooooo`000?l0000;ooooo`;o0000
0oooool01ol0003ooooooooooooooooo0000ooooool000008_ooool3o`0000[ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo00Cooooo00?o0000ogl0
0?mo00000_mo000Oooooo`007?ooool6o`3o00Co000000?oooooo`000?l000000_ooool00ol0003o
ooooooooo`0;ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooool000003_ooool00ol0003o
ooooooooo`0Eooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000000_ooool01Ol0003o
oooooooooooooooo000000Oooooo00?o0000ooooooooool01oooool2o`0000;ooooo00?o0000oooo
ooooool05oooool00ol0003oooooooooo`02o`0000?ooooo1?l0000:ooooo`03o`000?oooooooooo
00?ooooo0_l00007ooooo`03o`000?oooooo000000_ooooo0_l00003ooooo`07o`000?oooooooooo
ooooool0003oooooo`00000Sooooo`;o00002_ooool01Ol0003ooooooooooooooooo000000;ooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`KoO`007_ooool001_ooooo2?l0
o`001?l0003oooooooooooooool3o`000003ooooool0003ooooo00gooooo00?o0000ooooooooool0
4Oooool01?l0003oooooooooool0000>ooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool01oooool01Ol0003oooooo`000?oooooo000000;ooooo0_l00008ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo
00;o00001?ooool2o`000003ooooool0003o000000?o00001Oooool00ol0003oooooooooo`03oooo
o`;o00001_ooool2o`000003ooooool0003ooooo00Wooooo00Co0000oooooooooooo00000_ooool0
1ol0003ooooooooooooooooo0000ooooool000008_ooool00ol0003oooooooooo`02o`0000Wooooo
00?o0000ooooool000001?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo
2?mo000Mooooo`006oooool8o`3o0004o`000?ooooooooooo`0000;ooooo0ol0000>ooooo`03o`00
0?oooooooooo017ooooo00Co0000oooooooooooo00003_ooool00ol0003oooooooooo`0Fooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooo000000Oo
oooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
01Kooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooooooool2o`0000Kooooo1?l0
00000ooooooo0000ooooo`04ooooo`;o00001Oooool01Ol0003oooooo`000?oooooo000000Wooooo
00Go0000ooooooooooooooooo`000002ooooo`07o`000?ooooooooooooooool0003oooooo`00000Q
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`03
ooooo`;o00002?ooool00ol0003oooooooooo`02ooooo`SoO`007Oooool001_ooooo2?l0o`03oooo
o`03o`000?oooooooooo00?o00003oooool00ol0003oooooooooo`0?ooooo`04o`000?oooooooooo
o`0000oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooool000001oooool00ol0003oooooooooo`04ooooo`06o`000?oooooo
ooooo`000?oooooo00006Oooool00ol0003oooooo`000003ooooo`07o`000?ooooooooooooooool0
003oooooo`000009ooooo`Go00000oooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
ooooo`;o00002?ooool00ol0003oooooooooo`03ooooo`;o00001?ooool00ol0003oooooo`00000P
ooooo`03o`000?oooooooooo00Gooooo0_l00007ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooool000001oooool00ol0003oooooooooo`02o`0000SoO`007Oooool001_ooooo2?l0o`03oooo
o`04o`000?ooooooooooooooo`;o00003oooool00ol0003oooooooooo`0>ooooo`05o`000?oooooo
ooooooooool000003oooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo00Oooooo0ol0
00001Ooooooo0000oooooooooooo000000Sooooo00?o0000ooooooooool00_ooool01ol0003ooooo
oooooooooooo0000ooooool000006?ooool00ol0003oooooo`000004ooooo`04o`000?oooooooooo
o`0000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`05o`0000;ooooo00?o0000
ooooooooool00_ooool2o`0000Kooooo0_l00007ooooo`03o`000?oooooooooo00?ooooo0_l0000O
ooooo`03o`000?oooooooooo00Sooooo0_l00004ooooo`03o`000?oooooo000000Cooooo00Co0000
oooooooooooo00001Oooool6o`0000SoO`007Oooool001cooooo1_l0o`04ooooo`04o`000?oooooo
ooooooooo`?o00003_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00Kooooo0_l0
00000ooooooo0000o`000002ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_oo
ool01ol0003ooooooooooooooooo0000ooooool000005oooool00ol0003oooooo`000005ooooo`04
o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03oooo
o`Ko00001?ooool2o`0000Cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`03oooo
o`;o00007_ooool00ol0003oooooooooo`0;ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`05o`000004ooooool0003o0000o`0000?ooooo1_mo
000Nooooo`007Oooool4o`3o00Gooooo00?o0000ooooooooool00_ooool2o`0000oooooo00?o0000
ooooooooool03?ooool01Ol0003ooooooooooooooooo000000oooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`06ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool0
1Oooool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooool000005oooool2o`0000Oo
oooo0_l00005ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00oooool2o`0000Co
oooo1_l00003ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooool000001?ooool00ol0003o
ooooo`00000Looooo`03o`000?oooooooooo00gooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo1Ol00002ooooo`;o00000_ooool01Ol0003oooooooooool0003ooooo00CoO`007oooool001oo
oooo0_l00005ooooo`03o`000?oooooooooo00;ooooo0ol0000>ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool01Oooool2o`0000;ooooo0_l00002ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooool000001Oooool00ol0003oooooo`00000Fooooo`;o00002?ooool2o`0000Go
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`03ooooo`;o00002?ooool5o`0000[o
oooo00Co0000oooooooooooo00000oooool00ol0003oooooo`00000Kooooo`03o`000?oooooooooo
00kooooo00?o0000ooooool000001?ooool5o`0000?ooooo0ol00004ooooo`03o`000?oooooo0000
02Gooooo000Nooooo`;o00001Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000ko
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool06oooool00ol0003oooooooooo`04ooooo`04o`000?oooooo0000o`00
00;ooooo00?o0000ooooool000002?ooool00ol0003oooooo`000005ooooo`03o`000?oooooo0000
01Gooooo0_l00009ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo000000Wooooo00Co0000ooooool0003ooooo1?l00006
ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooool000006_ooool00ol0003ooooooooo
o`0>ooooo`04o`000?ooooooooooooooo`Ko00000_ooool01?l0003oooooo`000?l00002ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool09?ooool001Wooooo1?l0o`000ooooooo0000
o`000005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000003_ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03_ooool00ol0003ooooo
ooooo`0Looooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooo`000008ooooo`;o00001_ooool2o`0001Kooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000002oooo
o`03o`000?oooooooooo00Oooooo0_l00006ooooo`Co000000?oooooo`000?ooool00oooool01_l0
003oooooooooool0003oooooo`0001Wooooo00?o0000ooooooooool03_ooool00ol0003oooooo`00
0003o`000006ooooool0003oooooooooool0003ooooo0ol00005ooooo`04o`000?oooooooooooooo
o`;o00001oooool4o`3ooa_ooooo000Hooooo`Ko0?l000?oooooo`000?ooool01?ooool00ol0003o
ooooooooo`04ooooo`;o000000?oooooo`000?ooool03Oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Looooo`03
o`000?oooooooooo00?ooooo00Co0000oooooooooooo00000oooool00ol0003oooooo`000008oooo
o`03o`000?oooooooooo00Cooooo0_l0000Eooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ool000001oooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000
ooooooooool01oooool2o`0000[ooooo1Ol00002ooooo`05o`000?oooooo0000ooooool000006?oo
ool00ol0003oooooooooo`0=ooooo`Co00001_ooool4o`0000;ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00?ooooo1_l0oolJooooo`005oooool8
o`3o00Kooooo00?o0000ooooooooool01?ooool01Ol0003oooooo`000?oooooo000000gooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool07Oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooool00000
0oooool2o`0000Oooooo0_l00006ooooo`;o00005?ooool2o`0000[ooooo00?o0000ooooool00000
2?ooool00ol0003oooooooooo`05ooooo`;o00001?ooool00ol0003oooooooooo`06ooooo`?o0000
2oooool01?l0003oooooooooooooool5o`000003ooooool0003ooooo01Kooooo00?o0000oooooooo
ool02_ooool5o`0000Sooooo1_l000000ooooooo0000ooooo`06ooooo`05o`000?oooooooooooooo
ool000001?ooool8o`3ooaWooooo000Gooooo`So0?l01_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooo000000;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Mooooo`05
o`000?ooooooooooooooool000000_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooool000001Oooool2o`0001?ooooo0_l0000:ooooo`04o`000?oooooooooo
o`0000Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00Gooooo00Co0000ooooool0003o00002_ooool00ol0003oooooooooo`05ooooo`Ko00005?oo
ool00ol0003oooooooooo`07ooooo`Co00001?ooool00ol0003oooooooooo`04ooooo`;o00001Ooo
ool3o`0000Kooooo00?o0000ooooool000000oooool01?l0003oooooooooooooool8o`3ooaWooooo
000Gooooo`So0?l01_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo0000ooooool00000
3Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0Oooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooo
oooo00000oooool2o`0000Gooooo00Co0000oooooooooooo00001Oooool2o`0001;ooooo0_l0000;
ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000ooooooooool00oooool2o`0000Gooooo0_l0
0005ooooo`06o`000?ooooooooooo`000?oooooo00002Oooool00ol0003oooooooooo`06ooooo`?o
00000_ooool4o`0000oooooo00?o0000ooooooooool01?ooool4o`0000Oooooo00?o0000oooooooo
ool00_ooool3o`0000Sooooo0_l00005ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooooooool8o`3ooaWooooo000Gooooo`So0?l01Oooool00ol0003oooooooooo`06oooo
o`03o`000?oooooo000000;ooooo00?o0000ooooooooool02oooool01Ol0003ooooooooooooooooo
000000Oooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Pooooo`05o`000?oooooo
0000ooooool000000oooool00ol0003oooooooooo`02ooooo`;o00000oooool01Ol0003ooooooooo
oooooooo000000Gooooo00?o0000ooooooooool04?ooool00ol0003oooooo`00000:ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool01oooool01ol0003ooooooooooooooooo0000oooo
ool000001_ooool00ol0003oooooooooo`02ooooo`07o`000?ooooooooooooooool0003oooooo`00
0009ooooo`03o`000?oooooooooo00Oooooo0_l00006ooooo`Go00002Oooool01?l0003ooooooooo
ooooool4o`0000[ooooo00Co0000oooooooooooooooo0_l0000<ooooo`;o00000oooool00ol0003o
ooooooooo`03ooooo`04o`000?ooooooooooo`0000;ooooo2?l0oolIooooo`006?ooool6o`3o00Ko
oooo00?o0000ooooooooool01_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00[ooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooooooo00oooooo00?o0000
ooooooooool08?ooool2o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooooooo
ool2o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooool000002oooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`04ooooo`05o`000?oo
ooooooooooooool000001Oooool00ol0003oooooo`000007ooooo`03o`000?oooooooooo00Sooooo
0_l0000;ooooo`Co00001?ooool00ol0003oooooo`000002o`0000gooooo00?o0000ooooool00000
0_l0000?ooooo`Co00001oooool00ol0003oooooo`000003ooooo`Ko0?oo6_ooool001Wooooo1?l0
o`000ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool0
3oooool00ol0003oooooooooo`0Qooooo`04o`000?ooooooooooo`0000Gooooo00So0000oooooooo
oooo0000ooooool0003oooooo`0000Gooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`0>ooooo`03o`000?oooooo000000_ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`08ooooo`;o00000oooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000Kooooo
00?o0000ooooool000001oooool00ol0003oooooooooo`08ooooo`;o00003oooool6o`0000oooooo
0ol0000Booooo`04o`000?oooooo0000o`0000Oooooo00?o0000ooooool000000oooool00ol0003o
0?ooo`3oo`02o`3ooa_ooooo000Looooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1_ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool02_ooool2o`0000So
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Qooooo`?o00001_ooool01?l0003o
ooooooooool00002ooooo`;o00001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00gooooo00Co0000oooooooooooo00002oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00Sooooo0_l00003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000001oooool0
0ol0003oooooo`000007ooooo`03o`000?oooooooooo00Sooooo0ol0000<ooooo`Go00000_ooool4
o`0000[ooooo0ol0000Dooooo`;o00000_ooool2o`0000Kooooo00Go0000ooooool0003oooooo`00
000Pooooo`005_ooool4ogmoO`;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`06
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`09ooooo`;o00002?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo027ooooo0ol00007ooooo`05o`000?oo
ooooooooo`000?ooool00_l00006ooooo`05o`000?ooooooooooooooool000003oooool00ol0003o
ooooo`00000;ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00Cooooo0_l00008ooooo`04o`000?oooooooooo
o`0000Gooooo00?o0000ooooooooool02Oooool3o`0000Sooooo1?l00003ooooo`03o`000?oooooo
oooo00Gooooo1?l00004ooooo`?o00005_ooool00ol0003oooooo`000003ooooo`03o`000?oooooo
oooo00Cooooo0ol00006ooooo`Co001o5oooool001Gooooo1_moOgl00ooooooo0000ooooo`06oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0000:
ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Q
ooooo`04o`000?ooooooooooo`0000Oooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00_l0000>ooooo`04o`000?ooooooooooo`0000_ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00001Oooool2o`0000Sooooo00Co0000oooooooooooo00001Oooool00ol0
003oooooooooo`09ooooo`;o000000?oooooo`000?ooool00_ooool4o`0000Kooooo00?o0000oooo
ooooool02_ooool6o`0001Kooooo00Co0000ooooool0003o00001?ooool2o`0000Gooooo0_l00005
ooooo`Ko001o5_ooool001Cooooo2?moOgl8ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool00oooool01?l0003oooooooooool00009ooooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`0Qooooo`05o`000?oooooo0000ooooool00000
1oooool01?l0003oooooo`000?l00002ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00_l0000=ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`03ooooo`;o00001Oooool01?l0003oooooooooool00004ooooo`?o00002?ooool0
1Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00
0003o`0000Wooooo00?o0000ooooooooool02_ooool2o`0000Cooooo1?l0000Booooo`06o`000?oo
ooooooooo`000?oooooo00001Oooool2o`0000;ooooo00?o0000ooooool000001?ooool8o`00OaGo
oooo000Dooooo`SoOgmo1oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo
00Go0000ooooooooooooooooo`000007ooooo`05o`000?ooooooooooooooool000001_ooool00ol0
003oooooooooo`0@ooooo`03o`000?oooooooooo027ooooo00Go0000ooooool0003oooooo`000008
ooooo`;o00001?ooool00ol0003oooooooooo`03ooooo`04o`000?oooooo0000o`0000cooooo00Co
0000oooooooooooo00003?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool01_ooool00ol0003oooooo`000002ooooo`;o000000?oooooo`000?l00000
2?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool02Oooool4o`000003
ooooool0003ooooo00Wooooo00?o0000ooooooooool02?ooool3o`0000Gooooo00?o0000oooooooo
ool00_ooool5o`0000cooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo
00Cooooo0_l00003ooooo`04o`000?ooooooooooooooo`So001o5Oooool001Cooooo2?moOgl7oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00006
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo027ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`05
ooooo`?o00001?ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo0000o`0000_ooooo00Go
0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooo
00002_ooool2o`000003ooooool0003ooooo00;ooooo0_l00008ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`05ooooo`Co00000_ooool01Ol0003ooooooooooooooooo
000000Sooooo00?o0000ooooooooool01_ooool3o`0000Oooooo00?o0000ooooooooool02?ooool4
o`0000Oooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00006ooooo`?o00000_oo
ool2o`0000;ooooo2?l007lEooooo`005?ooool8ogmoO`Oooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`0Pooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool01Oooool3o`0000Gooooo00?o
0000ooooooooool00_ooool3o`0000[ooooo00Go0000ooooooooooooooooo`00000<ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooool000002oooool3o`0000Cooooo0_l00008ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool4o`0000Kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo0_l00009ooooo`03
o`000?oooooooooo00gooooo1Ol000000ooooooo0000ooooo`04ooooo`05o`000?oooooooooooooo
ool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo0_l000000ooooooo001oo`00O`06
o`00OaGooooo000Eooooo`KoOgmo2?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Cooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo023ooooo00?o0000
ooooool000001?ooool00ol0003oooooooooo`03ooooo`04o`000?oooooo0000o`0000Kooooo00Oo
0000ooooooooooooooooo`000?oooooo000000Wooooo00Go0000ooooooooooooooooo`00000<oooo
o`03o`000?oooooooooo00Wooooo0_l0000=ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ool000001oooool00ol0003oooooooooo`02ooooo`04o`000?oooooo0000ooooo`?o00002_ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool3
o`0000[ooooo00?o0000ooooooooool04oooool4o`0000Cooooo00Go0000ooooooooooooooooo`00
0002ooooo`03o`000?oooooooooo00?ooooo0ol000000ooooooo0000ooooo`06o`00OaKooooo000F
ooooo`CoOgmo00?o0000ooooooooool01_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Cooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Kooooo00Co
0000oooooooooooo00005?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooo000000Gooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00007ooooo`06o`000?ooooooooooo`00
0?oooooo00002?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0;ooooo`?o00001Oooool2o`0000Sooooo00Co0000
oooooooooooooooo1?l0000=ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooooooo
o`000004ooooo`;o00003?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00;ooooo
1Ol00003ooooo`03o`000?oooooo000000Sooooo0_l000000ooooooo0000ooooo`04o`00OaOooooo
000Gooooo`?o00002Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Gooooo00Co
0000oooooooooooooooo0_l00009ooooo`04o`000?ooooooooooo`0001?ooooo00?o0000oooooooo
ool08?ooool00ol0003oooooo`000006ooooo`05o`000?ooooooooooooooool000001?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00?o00001oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02Oooool00ol0003oooooo`00000;oooo
o`04o`000?oooooo0000o`0000Gooooo0_l00008ooooo`Co000000?oooooo`000?l000003_ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooo000000;ooooo0ol0000=ooooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool01oooool6o`0000[ooooo0_l000000ooooooo0000ooooo`0J
ooooo`005_ooool01?l0003oooooo`000?l00008ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool01Oooool01?l0003oooooooooooooool2o`0000[ooooo00?o0000ooooool000004ooo
ool00ol0003oooooooooo`0Pooooo`03o`000?oooooo000000Oooooo00Co0000oooooooooooo0000
1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000;o00001_ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02?ooool01?l0003ooooo
ooooool0000:ooooo`04o`000?ooooooooooooooo`;o00001?ooool2o`0000Cooooo1Ol00004oooo
o`03o`000?oooooooooo00gooooo0_l00005ooooo`;o000000?oooooo`000?l000003oooool00ol0
003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo1Ol00008
ooooo`Co00006_ooool001?ooooo1?oo0?l00ooooooo0000o`000008ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool01_ooool01Ol0003oooooo`000?oooooo000000[ooooo0_l0000D
ooooo`03o`000?oooooooooo023ooooo00?o0000ooooool000002?ooool2o`0000Gooooo0_l00007
ooooo`Co00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool02?ooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooooooool00_oo
ool00ol0003oooooo`000003ooooo`Ko00001?ooool00ol0003oooooooooo`02ooooo`;o00003_oo
ool2o`0000Gooooo0ol0000@ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02_oo
ool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Go00000oooool00ol0003ooooooooo
o`02o`0000Coo`005Oooool001;ooooo1_oo0?l00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000002ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Pooooo`;o00002Oooool2
o`0000Gooooo0_l00008ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00Co0000ooooool0003ooooo1Ol0
0007ooooo`03o`000?oooooooooo00;ooooo0_l0000>ooooo`;o00000oooool2o`0000;ooooo00?o
0000ooooooooool03_ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00_ooooo0_l0
0004ooooo`03o`000?oooooooooo00Cooooo1Ol00002ooooo`Koo`005?ooool0017ooooo2?oo0?l9
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0O
ooooo`03o`000?oooooo000000Wooooo0_l00006ooooo`;o00001_ooool01?l0003oooooo`000?l0
0003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`04ooooo`Co00000_ooool00ol0003oooooo`000007ooooo`05o`000?ooooooooooooooool00000
0_ooool00ol0003oooooooooo`0;ooooo`Ko00000oooool00ol0003oooooooooo`0=ooooo`03o`00
0?oooooooooo01;ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Oooooo0_l00008ool001?ooooo000Aooooo`Soo`3o0_l00007ooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000003ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooool000004oooool00ol0003oooooooooo`0Oooooo`03o`000?oo
oooo000000Sooooo00Co0000oooooooooooo00001Oooool00ol0003oooooo`000005ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04
o`000?ooooooooooooooo`Go00000oooool01Ol0003oooooo`000?oooooo000000Oooooo00Go0000
ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00_ooooo0_l00002ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`07ooooo`03o`000?oo
ooooo`0000Ooo`004oooool0017ooooo2?oo0?l2ooooo`_o00002Oooool00ol0003oooooooooo`04
ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool01Oooool00ol0003oooooo`00
000Booooo`03o`000?oooooooooo023ooooo0_l00009ooooo`04o`000?ooooooooooo`0000Gooooo
00?o0000ooooool000001Oooool00ol0003oooooooooo`03o`0000Kooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
0_ooool5o`0000Cooooo00Wo0000ooooooooooooooooo`000?oooooo0000ooooool000001oooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool02?ooool5o`000003ooooool0
003ooooo00Kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00gooooo0_l00007ooooo`03o`000?oooooooooo00Oooooo00?o0000ool00?oo00001_oo
000Cooooo`004Oooool8ool0o`Sooooo00?o0000ooooooooool00_ooool:o`0000Kooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooooooo
o`00000Aooooo`03o`000?oooooooooo023ooooo0_l00008ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00;o
00001oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool02?ooool00ol0003oooooo`000002o`0000Sooooo00?o0000ooooooooool00_ooool2o`000003
ooooool0003ooooo00Kooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`05oooo
o`;o00000oooool3o`0000Wooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Boooo
o`03o`000?oooooooooo00gooooo00Co0000oooooooooooo00001oooool00ol0003oooooooooo`06
ooooo`03o`000?oo003oo`0000Koo`004oooool001;ooooo1_oo0?l00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo00_ooooo3?l00004ooooo`03o`000?oooooooooo00?ooooo00Go0000oooo
ooooooooooooo`00000Aooooo`03o`000?oooooooooo01oooooo00?o0000ooooool000002?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000001?ooool0
0ol0003oooooooooo`03o`0000Gooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Oooooo1Ol0000:ooooo`03o`000?oooooooooo00?ooooo00Co0000
oooooooooooo00001_ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00;ooooo
0ol00005ooooo`;o00002_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01;ooooo
00?o0000ooooooooool03Oooool01Ol0003ooooooooooooooooo000000Oooooo00?o0000oooooooo
ool01oooool00ol0003oo`00ool00004ool001Cooooo000Cooooo`Coo`3o0_l00008ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`_o0000
1?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo01oooooo0_l00008ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;o000000?oooooo`000?l000000oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00Cooooo0_l00006ooooo`Co00000_ooool2o`0000cooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooool00006ooooo`04o`000?ooooooooooo`0000Gooooo00Co0000
ooooool0003o00002?ooool00ol0003oooooo`00000:ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`06ooooo`03o`000?ooooooo`0000?oo`005Ooo
ool001Kooooo0ol00008ooooo`03o`000?oooooooooo00_ooooo00Go0000ooooooooooooooooo`00
0007ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooo0000ooooo`0:o`0000_ooooo
00?o0000ooooooooool07oooool2o`0000Oooooo00?o0000ooooooooool01_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00?o0000ooooool000000_ooool2o`000004ooooool0003oooooo`00
00;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00;ooooo1?l00006ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool00oooool00ol0
003oooooo`000007ooooo`03o`000?oooooo000000Cooooo0ol00009ooooo`;o000000?oooooo`00
0?ooool02_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooo`00000Gooooo`005_ooool00ol0003oooooo`000008ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooool000002Oooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool00_ooool<o`00027ooooo0_l00007oooo
o`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000003ooooo`07o`000?oooooo
0000ooooool0003oooooo`000002ooooo`;o000000?oooooo`000?ooool03?ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooo000000?o00002Oooool00ol0003oooooo`00000<ooooo`03o`00
0?oooooooooo00?ooooo0ol00007ooooo`;o00000oooool2o`000003ooooool0003ooooo00Sooooo
00Go0000ooooool0003oooooo`00000;ooooo`05o`000?ooooooooooooooool000005?ooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`06ooooo`;o00006?ooool0017ooooo1?l0ool01?l0003oooooooooool00008oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooool000002Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooo000000Kooooo00?o0000ooooooooool03Oooool:o`0001Oooooo0_l00007oooo
o`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`000003ooooo`?o00000_ooool0
1Ol0003oooooooooool0003ooooo00;o00003Oooool01?l0003oooooooooooooool4o`0000_ooooo
0_l00002ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool00oooool3o`0000Oooooo
1Ol00004ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0_l0000<ooooo`04
o`000?ooooooooooo`0001?ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04o`1o
01;ooooo000@ooooo`Ko0?oo0_ooool2o`0000Oooooo00?o0000ooooooooool03?ooool2o`0000_o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02?ooool;o`0000cooooo0_l00007ooooo`03o`000?oooooooooo00Wo
oooo00Co0000oooooooooooo00000oooool2o`0000?ooooo00Go0000oooooooooooo0000ooooo`02
o`0000cooooo1_l0000=ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02_ooool0
0ol0003oooooooooo`03ooooo`04o`000?oooooo0000o`0000Kooooo0_l00007ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool2o`0000gooooo0_l0000Cooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooool0O`001Ol0O`0Aooooo`003oooool8o`3oo`03ooooool0003o
000000Kooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo01?ooooo2_l00002ooooo`;o00001_ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooo
ooooo`0000?ooooo0_l00003ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003ooooo
ooooo`06ooooo`Co00000oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool02_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000?o
oooo1?l00009ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool2o`0000go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo07l000Oo
07l04?ooool000oooooo2?l0ool01?oooooo0000ooooool00005ooooo`03o`000?oooooooooo00co
oooo0_l0000=ooooo`03o`000?oooooooooo00;ooooo0_l00007ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool07Oooool;o`0000gooooo00?o0000ooooool000000_ooool3o`0000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`Co00000_ooool0
1Ol0003ooooooooooooooooo000000gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?l00003ooooo`03o`000?oo
oooooooo00Oooooo00Go0000ooooooooooooooooo`000006ooooo`;o00003?ooool00ol0003ooooo
o`00000Aooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`04o`000?ooooooooooooooo`Co00002?l0O`0@ooooo`003oooool8o`3oo`So0000
00?oooooo`000?ooool03Oooool2o`0000kooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool07_ooool2o`0000Ko
oooo00?o0000ooooool000002_l00003ooooo`08o`000?oooooo0000o`000?oooooo0000ooooool0
0002ooooo`03o`000?oooooooooo00;o00001?ooool3o`0000Kooooo00Co0000oooooooooooo0000
3Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
0oooool00ol0003oooooo`000002o`0000Gooooo00?o0000ooooooooool01oooool01?l0003ooooo
ooooool00007ooooo`;o00002oooool01Ol0003ooooooooooooooooo000000oooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0:oooooa3o00000_ooool00ol0003oooooo`1o0007o`1o
013ooooo000?ooooo`So0?oo00Goooooo`000?ooooooooooo`000003ooooo`So00001oooool00ol0
003oooooo`00000>ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01kooooo0_l00005ooooo`03o`000?oooooo
oooo00_ooooo2_l000001_oooooo0000ooooool0003oooooo`0000;ooooo1Ol00007ooooo`04o`00
0?ooooooooooo`0000gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00;ooooo0_l000001?oooooo0000ooooool00003ooooo`;o00002_ooool2o`00
00Sooooo00?o0000ooooool000002Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool02oooool@o`000003ooooool0003ooooo00[ooooo00Go0000oooo
ooooooooooooo`000002ooooo`So07l04?ooool0013ooooo1_l0ool2ooooo`04o`000?oooooooooo
o`0000Cooooo00?o0000ooooooooool01?ooool8o`000003ooooool0003ooooo00kooooo00?o0000
ooooool000000oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool07_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00oooooo0_l00002
ooooo`03o`000?oooooo000000[o00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
000000gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
000000?o00000oooool00ol0003oooooo`000002ooooo`;o00002_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`09
oooooa3o000000?oooooo`000?ooool03_ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooo
ooooo`0000?ooooo1_l0O`0Aooooo`004Oooool4o`3oo`?ooooo00Go0000ooooooooooooooooo`00
0003ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?l00009ooooo`;o00001Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool07ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00oooooo0ol000001?oooooo0000oooo
ooooool5o`0000?ooooo00?o0000ooooooooool02ol0000=ooooo`03o`000?oooooooooo00[ooooo
00?o0000ooooooooool02Oooool2o`0000?ooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00;ooooo0_l00009ooooo`03o`000?oooooo000000Sooooo00?o0000ooooool000001oooool01?l0
003ooooooooooooooolAo`0000oooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooo000000Cooooo1?l0O`0Booooo`004oooool00ol0003oooooooooo`02oooo
o`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool00oooool8o`0000;ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool07_ooool2
o`0000Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`05o`0000;ooooo0_l00005
ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooooooo3Ol00002ooooo`03o`000?oo
oooooooo00_ooooo00?o0000ooooooooool01_ooool4o`0000Gooooo00Co0000oooooooooooo0000
0_ooool2o`0000;ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00006oooooa3o
00001Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00gooooo00?o0000oooooooo
ool03oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00;ooooo0_l000000ooooooo
0000ooooo`0Booooo`004_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`00000;ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02oooool8o`000003oooo
ool0003ooooo00Gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Nooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool03_ooool7o`0000Gooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`04ooooo`04o`000?oooooo0000o`0000cooooo2_l00006ooooo`03
o`000?oooooooooo00Cooooo0_l00003ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003oooooooooool0003ooooo013o00000_ooool00ol0003oooooo`000006
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`02
ooooo`04o`000?oooooo0000o`0001?ooooo000?ooooo`CoiNGU00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooo000000[ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooo000000Cooooo2?l000000ooooooo0000ooooo`07
ooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<
ooooo`Co00001?ooool2o`0000Cooooo0ol00005ooooo`03o`000?oooooooooo00Cooooo0_l0000=
ooooo`03o`000?oooooooooo00Oooooo3Ol00005ooooo`03o`000?oooooo000000oo00001?ooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000001Oooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03_ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00_ooooo0_l00002ooooo`03o`000?oooooo
oooo00Co07l04Oooool000kooooo1_oUiND4ooooo`03o`000?oooooooooo00?ooooo0_l00009oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooo`000007oooo
o`03o`000?oooooooooo00;ooooo2?l00002ooooo`03o`000?oooooooooo01kooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`08ooooo`Co000000?oooooo`000?ooool01Oooool2o`00
00Cooooo0ol00005ooooo`03o`000?oooooooooo00?ooooo0_l0000=ooooo`03o`000?oooooooooo
00cooooo4Ol00003ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000ooooool0003oooooo`00
0004ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooo`00
0004ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`0>ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02_ooool01ol0003ooooooooo
ool0003oooooo`000?ooool01_l0O`0@ooooo`003Oooool8onGUi@?ooooo00?o0000ooooooooool0
0oooool2o`0000Sooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0;ooooo`05o`00
0?ooooooooooooooool000001_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?o
oooo2?l0000Jooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01_ooool3o`0000Co
oooo00?o0000ooooooooool01_ooool2o`0000;ooooo00Go0000ooooool0003oooooo`000004oooo
o`03o`000?oooooooooo00;ooooo00Co0000ooooool0003o00002oooool@o`000004ooooool0003o
0000o`0000[ooooo00?o0000ooooooooool02ol000000ooooooo0000o`000003ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool01?ooool00ol0003oooooo`000003ooooo`03o`000?oo
oooooooo00gooooo00Co0000oooooooooooo00004?ooool00ol0003oooooooooo`0Cooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool00_l000000ooooooo07l0o`1o0006o`1o00oooooo
000=ooooo`SoiNGU0oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00oooool8o`0001;ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`02ooooo`Co00001_ooool00ol0003oooooooooo`06ooooo`?o000000Ooooooo`000?oooooooooo
o`000?oooooo000000Gooooo00?o0000ooooooooool04?l0000?ooooo`;o00000_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00So0000ooooooooooooooooo`000?oooooo
0000ooooo`_o00001?ooool00ol0003oooooooooo`04ooooo`06o`000?ooooooooooo`000?oooooo
00004Oooool2o`00013ooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00;o000000?oooooo`1o0?l0O`001_l0O`0?ooooo`003Oooool8onGUi@Co0000
1Oooool2o`0000Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool03?ooool8o`0000[ooooo00Ko0000oooooooooooo
ooooo`000?ooool4o`0000Wooooo00?o0000ooooooooool01ooooolBo`0000;ooooo00Go0000oooo
ooooooooooooo`000007ooooo`03o`000?oooooooooo00_ooooo0ol00004ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool00_ooool01?l0003oooooooooooooool2o`0000Cooooo00?o
0000ooooool000001_ooool:o`000004ooooool0003oooooooooo`;o00004_ooool00ol0003ooooo
ooooo`0>ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool02?ooool01ol0003ooooo
oooooooooooo0000ooooool000002?l0O`0?ooooo`003Oooool8onGUi@Oo00000_ooool00ol0003o
ooooo`000005ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo01Cooooo2?l000001?oooooo0000o`000?ooool4o`00
00Wooooo4ol00002ooooo`05o`000?ooooooooooooooool000000oooool2o`0000;ooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo0_l00008
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool01?l0003ooooooooooooo
ool2o`0000Cooooo0_l0000<ooooo`03o`000?oooooooooo00;ooooo2ol0000<ooooo`03o`000?oo
oooo000000kooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`08ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000o`1o0?l0O`001_l0O`0?ooooo`003_ooool6onGUi@04o`000?oo
ooooooooooooo`;o00000_ooool5o`0000Cooooo00?o0000ooooooooool02_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool07?oooolAo`0000?ooooo
00?o0000ooooooooool02?ooool01Ol0003oooooo`000?oooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02?ooool3o`0000[ooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`02ooooo`04o`000?ooooooooooooooo`;o00001?ooool2o`0000cooooo00?o0000
ooooooooool00oooool01?l0003oooooo`000?l00006ooooo`[o000000?oooooo`000?ooool00_oo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?oo
ool01?l0003oooooooooool00003ooooo`03o`000?l0O`3o07l000Co07l04?ooool000oooooo1?oU
iND2ooooo`05o`000?ooooooooooo`000?ooool00_l00002ooooo`03o`000?oooooo000000?o0000
00?oooooo`000?ooool02oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool03OoooolAo`0000;ooooo00Co0000oooooooooooooooo2?l00003ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooool000000_ooool00ol0003oooooo`000004ooooo`07
o`000?oooooo0000ooooool0003oooooo`000007ooooo`04o`000?oooooo0000o`0000Wooooo0_l0
000=ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ooooool2o`0000?ooooo0_l000000ooooooo0000ooooo`0;ooooo`03o`000?oooooooooo00;ooooo
00Co0000ooooool0003o00004?ooool;o`0000Kooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`07ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000o`1o0?l0O`000_l0O`0Aoooo
o`005Oooool01?l0003oooooooooool00003ooooo`?o00000oooool00ol0003oooooo`000003o`00
00[ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00oooool01?l0003oooooooooooooool@o`0000Oooooo1?l00005
ooooo`04o`000?ooooooooooo`0000_ooooo2?l00006ooooo`04o`000?oooooo0000o`0000;ooooo
00?o0000ooooool000001?ooool2o`0000;ooooo00?o0000ooooool000002?ooool00ol0003ooooo
ooooo`06ooooo`?o00003oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooooooo0_l00002ooooo`03o`000?oooooo000000;ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`02ooooo`;o000000?oooooo`000?ooool03_ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00;ooooo2_l0000Iooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooool000001Oooool3o`0001Cooooo000Eooooo`04o`000?ooooooooooo`0000Go
oooo0ol000000ooooooo0000o`000004ooooo`Co00001_ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Oooooo4_l0000Booooo`Co00002?ooool01?l0003oooooooooool0000=oooo
o`03o`000?oooooooooo00Cooooo2Ol000001?oooooo0000ooooooooool2o`0000Cooooo0_l00003
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000001Oooool2o`0001?ooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o000000?ooooo
o`000?ooool00_ooool00ol0003oooooo`00000;ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooo00003_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00oooool;o`0000kooooo00?o0000ooooooooool01_ooool00ol0003oooooo`00
0004ooooo`04o`000?ooooooooooo`0001Cooooo000>ooooo`CoOgmo0oooool01?l0003ooooooooo
ool00004ooooo`04o`000?ooooooooooooooo`?o00002?ooool3o`0000?ooooo00?o0000oooooooo
ool00oooool@o`0000Sooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00000?ooooo`Co
00003?ooool00ol0003oooooo`00000=ooooo`03o`000?oooooooooo00_ooooo2_l00002ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool01_l0003ooooooooooooooooo
0000ooooo`?o00005Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00?ooooo00Co
0000ooooool0003ooooo0_l00004ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000oooooooo
ool00_l00002ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00oooooo2ol00003ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooool000001Oooool01?l0003oooooooooool00004ogmoOa3ooooo000=ooooo`Ko
Ogmo0_ooool01?l0003oooooooooool00004ooooo`06o`000?ooooooooooooooool0003ooooo0ol0
0002oooooa3o00001?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`00000<ooooo`?o00004?ooool00ol0003oooooo`00
000<ooooo`03o`000?oooooooooo00cooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooo000000So00001_ooool00ol0003oooooooooo`02ooooo`;o00006?ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00;ooooo00Go0000ooooool0003oooooo`000005ooooo`05
o`000?ooooooooooooooool000002_ooool01Ol0003oooooo`000?oooooo000000?ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool06oooool:o`000004ooooool0003oooooo`0000Cooooo00Co0000oooooooooooooooo
1_moOgl?ooooo`003?ooool8ogmoO`;ooooo00?o0000ooooool000003ol0000:ooooo`Go00001Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Kooooo00Oo0000oooooooooooooooo
o`000?oooooo000000Sooooo1?l0000Booooo`?o00003?ooool00ol0003oooooooooo`0=ooooo`05
o`000?ooooooooooooooool000000oooool3o`0000Gooooo2_l000001_oooooo0000o`000?l0003o
ooooo`0001Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`06o`000?oo
oooo0000o`000?oooooo00001?ooool01Ol0003ooooooooooooooooo000000_ooooo00Co0000oooo
oooooooo00000oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00_ooooo00Co0000
oooooooooooo00008_ooool00ol0003oooooooooo`03ooooo`_o00002?moOgl>ooooo`003?ooool8
ogmoO`Go00001?ooool01?l0003oooooooooool00003ooooo`04o`000?ooooooooooooooo`;o0000
2Oooool01?l0003oooooooooooooool5o`0000gooooo00?o0000ooooooooool01_ooool01?l0003o
ooooooooooooool2o`0000Cooooo1?l0000Fooooo`03o`000?oooooo000000_ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?o00001Oooool01?l0
003oooooooooool00003ooooo`_o00005oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00;ooooo0ol00002ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`07ooooo`;o00000_ooool01Ol0003ooooooooooooooooo000000Wooooo00?o
0000ooooooooool03Oooool2o`0002?ooooo00?o0000ooooooooool00oooool2o`0000Cooooo00?o
0000ooooooooool00_ooool8ogmoO`kooooo000<ooooo`SoOgmo0_ooool00ol0003oooooo`000004
ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool01Oooool4o`0000_ooooo00?o0000ooooooooool01Oooool01?l0003ooooooooooooo
ool6o`0001[ooooo0_l0000;ooooo`03o`000?oooooooooo00gooooo0_l00005ooooo`06o`000?oo
ooooooooo`000?oooooo00001Oooool2o`0000;ooooo00Co0000ooooool0003o00002Oooool9o`00
00oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02ooooo`;o00000oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0
003oooooo`000002ooooo`05o`000?ooooooooooooooool000002?ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool00oooool2o`0000?ooooo00?o0000
ooooooooool00oooool8ogmoO`kooooo000<ooooo`SoOgmo0_ooool00ol0003oooooo`000004oooo
o`;o00001_ooool00ol0003oooooooooo`05ooooo`;o00001?ooool00ol0003oooooooooo`05oooo
o`04o`000?ooooooooooooooo`Co00001oooool00ol0003oooooooooo`06ooooo`Go00007Oooool3
o`0000[ooooo00?o0000ooooooooool03_ooool2o`0000Gooooo00Co0000ooooool0003o00000_oo
ool02?l0003ooooooooooooooooo0000ooooool0003ooooo0ol0000<ooooo`03o`000?oooooooooo
00Gooooo2?l00007ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool2o`00
00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03_ooool00ol0003oooooo`00000Sooooo`03o`000?oooooooooo00?ooooo00Go
0000ooooooooooooooooo`000006ooooo`SoOgmo3_ooool000gooooo1_moOgl3ooooo`03o`000?oo
oooo000000Cooooo0_l00007ooooo`03o`000?oooooooooo00Kooooo0_l00002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01_ooool4o`0000Cooooo00?o0000ooooooooool00_oo
ool5o`000003ooooool0003o000001gooooo0_l0000:ooooo`03o`000?oooooooooo00kooooo00?o
0000ooooool000001Oooool2o`000003ooooool0003ooooo00;ooooo00Go0000oooooooooooo0000
ooooo`02o`000003ooooool0003ooooo00gooooo00?o0000ooooooooool03?ooool9o`0000Sooooo
00Co0000oooooooooooooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`03ooooo`07o`000?ooooooooooooooool0003oooooo`000003
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03_ooool01Ol0003ooooooooooooo
oooo000002;ooooo00?o0000ooooooooool00_ooool2o`0000?ooooo00?o0000ooooooooool01?oo
ool00ol0003oOgmoogmoO`04ogmoO`oooooo000>ooooo`CoOgmo00?o0000ooooooooool00_ooool2
o`0000?ooooo0_l00008ooooo`03o`000?oooooooooo00Sooooo0ol00006ooooo`03o`000?oooooo
oooo00[ooooo2Ol00004ooooo`05o`000?oooooo0000ooooool000007?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00oooooo00?o0000ooooool000001?ooool01Ol0003oooooo`00
0?oooooo000000?ooooo00?o0000ooooool000000_l00002ooooo`03o`000?oooooo000000kooooo
00?o0000ooooooooool04oooool8o`000007ooooool0003oooooooooool0003oooooo`000005oooo
o`05o`000?ooooooooooooooool000001oooool01Ol0003ooooooooooooooooo000000Gooooo0_l0
0003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo027ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00Gooooo00?o0000oooooomoOgl00omoOgl@ooooo`004_ooool00ol0003ooooooooo
o`02ooooo`;o00000oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo0ol0
0004ooooo`03o`000?oooooooooo00_ooooo1ol00006ooooo`;o000000?oooooo`000?ooool06_oo
ool2o`0000Wooooo00?o0000ooooooooool03oooool00ol0003oooooo`000004ooooo`06o`000?oo
ooooooooo`000?oooooo00000oooool3o`0000?ooooo0_l000000ooooooo0000ooooo`0=ooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool01Oooool8o`0000Cooooo00Go0000oooooooo
ooooooooo`000008ooooo`03o`000?oooooo000000Kooooo0_l00004ooooo`05o`000?oooooooooo
ooooool000004Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo023ooooo00?o0000
ooooooooool00_l00002ooooo`03o`000?oooooooooo00Kooooo00Co0000oooooooooooo00004_oo
ool001?ooooo00Co0000oooooooooooooooo0_l00002ooooo`;o00002_ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00;o000000?oooooo`000?ooool02Oooool4o`0000Cooooo00?o
0000ooooooooool01?l00003ooooo`04o`000?ooooooooooo`0001[ooooo0_l00008ooooo`03o`00
0?oooooooooo00oooooo00Co0000oooooooooooo00000oooool01_l0003ooooooooooooooooo0000
ooooo`Co000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`02o`0000oooooo00?o0000
ooooooooool04_ooool00ol0003oooooooooo`07ooooo`05o`000?ooooooooooo`000?ooool02Ol0
00000ooooooo0000ooooo`06ooooo`;o00001oooool2o`0000Cooooo00Co0000oooooooooooo0000
4Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0
0_l000000ooooooo0000ooooo`08ooooo`03o`000?oooooo000001?ooooo000>ooooo`Co07l000?o
ooooo`000?ooool00_ooool2o`000004ooooool0003oooooo`0000_ooooo00?o0000ooooooooool0
2Oooool01?l0003oooooooooooooool2o`0000Oooooo0ol00009ooooo`03o`000?oooooooooo00?o
oooo1?l00002ooooo`03o`000?oooooooooo01Sooooo0_l00007ooooo`03o`000?oooooooooo013o
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`?o00001?ooool2o`00
0003ooooool0003ooooo00;ooooo00?o0000ooooool000003oooool00ol0003oooooooooo`0Boooo
o`03o`000?oooooooooo00Gooooo0_l00002ooooo`03o`000?oooooo000000Cooooo00Co0000oooo
oooooooooooo2?l000000ooooooo0000o`000008ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooool000004Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01oooooo00Ko0000
ooooool0003o0000ooooool0000:ooooo`03o`000?ooooooiNGU00?oiNGU4?ooool000gooooo1_l0
O`001?oooooo0000ooooooooool2o`000004ooooool0003oooooo`0000_ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`02ooooo`So00003?ooool00ol0003oooooooooo`06ooooo`Go
00006?ooool2o`0000Oooooo00?o0000ooooooooool04?ooool01ol0003ooooooooooooooooo0000
ooooool000000oooool5o`0000?ooooo00Co0000oooooooooooo00001Oooool2o`00013ooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000;ooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo2?l0
0003ooooo`;o00000oooool2o`00017ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Oooooo`Co00002oooool00ol0003oiNGUonGUi@04onGUi@oooooo000<ooooo`So07l000Co0000
oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02?oo
ool01?l0003oooooooooooooool7o`0000oooooo00?o0000ooooooooool01Oooool01?l0003ooooo
o`000?ooool5o`0001?ooooo0ol00006ooooo`03o`000?oooooooooo013ooooo00?o0000oooooooo
ool00_ooool2o`0000;ooooo0_l00003ooooo`;o00000oooool01?l0003oooooo`000?l00006oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooool000001Oooool01Ol0003ooooooooooooo
oooo000000Gooooo00Go0000ooooooooooooooooo`000004ooooo`Wo00004Oooool00ol0003ooooo
ooooo`0:ooooo`03o`000?oooooooooo01kooooo00Co0000ooooool0003o00002oooool8onGUi@ko
oooo000<ooooo`So07l000Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02?ooool5o`0000Kooooo0_l0000=ooooo`03o`000?oooooooooo00Co
oooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooooooool3o`00013ooooo0ol00006
ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool00_ooool4o`0000Gooooo0_l00002
ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool00oooool2o`00013ooooo00?o0000
ooooooooool04?ooool00ol0003oooooooooo`02ooooo`07o`000?ooooooooooooooool0003ooooo
o`000002ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000004ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`09o`0000Wo
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Mooooo`Co00003?ooool8onGUi@ko
oooo000<ooooo`So07l000?oooooo`000?ooool00_l00002ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01?ooool4o`0000?ooooo00?o0000ooooooooool01_ooool3o`0000_ooooo
00?o0000ooooooooool00oooool01ol0003ooooooooooooooooo0000ooooool000001_ooool4o`00
00_ooooo0_l000000ooooooo0000ooooo`04ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool00_ooool2o`0000Sooooo0_l00002ooooo`;o00001?ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00oooooo00?o0000ooooooooool04?ooool01?l0003oooooooooool00004
ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooo
oooo000000Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`05o`000?oo
oooo0000ooooool000001oooool9o`0000oooooo00?o0000ooooooooool07?ooool3o`0000gooooo
2?oUiND>ooooo`003?ooool8o`1o0003ooooool0003ooooo00;o00000_ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Co00000oooool01?l0003oooooooooool0000<ooooo`;o0000
2_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
2Oooool4o`0000Oooooo0ol00005ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool0
1?l0000:ooooo`03o`000?oooooo000000Kooooo00?o0000ooooooooool00oooool2o`00017ooooo
00?o0000ooooooooool03oooool00ol0003oooooo`000005ooooo`03o`000?oooooo000000;ooooo
00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`;o00000_ooool00ol0003oooooooooo`0<ooooo`Wo
00002Oooool00ol0003oooooooooo`0Jooooo`04o`000?oooooo0000o`0000gooooo2?oUiND>oooo
o`003Oooool6o`1o00?ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`0;ooooo`Co
00001oooool01?l0003oooooooooool0000>ooooo`;o00002?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooool000003Oooool4o`0000;ooooo00Co0000oooo
ool0003o00001?ooool00ol0003oooooooooo`0Aooooo`?o000000Coooooo`000?oooooo00002_oo
ool3o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooooooo00kooooo0_l00006ooooo`;o00000oooool00ol0003oooooooooo`03ooooo`05o`000?oo
ooooooooooooool000000_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00?ooooo
0_l00003ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool01oooool8o`000003oooo
ool0003ooooo01_ooooo1?l0000=ooooo`03o`000?oUiNGoiNGU00CoiNGU3oooool000kooooo1?l0
O`03ooooo`;o000000Goooooo`000?l0003oooooo`000009ooooo`Co00000_ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooo0000017ooooo0ol00006ooooo`05o`000?ooooooooooooooool0
00001Oooool00ol0003oooooo`00000@ooooo`Co000000?oooooo`000?ooool00_ooool00ol0003o
ooooooooo`0@ooooo`?o00000_ooool01?l0003oooooooooool0000:ooooo`;o00002Oooool00ol0
003oooooooooo`02ooooo`;o00004_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
00Kooooo00Go0000ooooooooooooooooo`000006ooooo`06o`000?ooooooooooo`000?oooooo0000
2oooool01?l0003oooooooooooooool2o`000005ooooool0003oooooooooool000002oooool00ol0
003oooooooooo`0@ooooo`So00005Oooool00ol0003oooooo`000002o`0000cooooo0_l000000ooo
ooooiNGUonGUi@02onGUiA3ooooo000Dooooo`03o`000?oooooooooo00?o000000?oooooo`000?oo
ool01?ooool4o`0000Oooooo00?o0000ooooooooool01_ooool2o`0001Cooooo0_l00004ooooo`05
o`000?ooooooooooooooool000001_ooool2o`00017ooooo00?o0000ooooool000001Ol0000Aoooo
o`;o000000Goooooo`000?ooooooooooo`000003ooooo`03o`000?oooooooooo00Oooooo0ol00009
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooo000000Oooooo00Go0000ooooooooooooooooo`000006ooooo`03o`000?oo
oooooooo00;o00003?ooool02?l0003oooooooooool0003oooooo`000?oooooo00000_ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool00_ooool8o`00
00cooooo00Co0000ooooool0003ooooo0_l0000<ooooo`;o000000?oooooo`000?ooool04_ooool0
01?ooooo00Co0000oooooooooooooooo1?l00003ooooo`?o00002oooool00ol0003oooooooooo`06
ooooo`;o00005_ooool2o`0000?ooooo00?o0000ooooool000001oooool00ol0003oooooo`00000?
ooooo`03o`000?oooooooooo00;o00000_ooool5o`0000_ooooo0ol00002ooooo`04o`000?oooooo
ooooo`0000?ooooo00?o0000ooooooooool01oooool01?l0003oooooo`000?l0000:ooooo`03o`00
0?oooooooooo00;ooooo0_l0000Booooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo
00001_ooool2o`0000?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02o`0000go
oooo0_l00002ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Fooooo`03o`000?oooooooooo00Wooooo2?l00004ooooo`;o000000Coooooo`000?oo
oooo00003?ooool2o`000003ooooool0003ooooo01;ooooo000>ooooo`Co0?l000?oooooo`000?oo
ool00oooool6o`0000oooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0Gooooo`;o
000000Coooooo`000?oooooo00002?ooool2o`0000oooooo00Oo0000ooooool0003oooooo`000?oo
oooo000000Gooooo1?l00005ooooo`;o00001?ooool01?l0003oooooo`000?l00004ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool00_l0000:ooooo`03o`000?oooooooooo00;ooooo
0_l0000Cooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`000006ooooo`;o
00001?ooool00ol0003oooooooooo`03ooooo`05o`000?oooooo0000ooooool000003Oooool2o`00
00;ooooo00Ko0000oooooooooooo0000ooooool00008ooooo`03o`000?oooooooooo01Oooooo00?o
0000ooooooooool04Oooool8o`000003ooooool0003ooooo00_ooooo00?o0000ooooool000001?l0
o`0@ooooo`003Oooool6o`3o00?ooooo1Ol0000Booooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool06Oooool4o`0000Wooooo00?o0000ooooooooool03?ooool01Ol0003oooooooooool0
003ooooo00;o00002_ooool5o`0000Kooooo0_l00006ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00ol0000;ooooo`03o`000?oooooooooo00;ooooo0_l0000Cooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`;o00001?oo
ool00ol0003oooooooooo`03ooooo`;o00000_ooool00ol0003oooooooooo`0:ooooo`03o`000?oo
oooo000000;ooooo00Ko0000oooooooooooo0000ooooool00007ooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool04oooool01?l0003oooooooooooooool9o`0000Kooooo00?o0000oooo
ool0o`001Ol0o`0?ooooo`003?ooool8o`3o00;o00000_ooool3o`0001?ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0Kooooo`?o00002Oooool00ol0003oooooooooo`0;ooooo`06
o`000?ooooooooooo`000?oooooo00002oooool2o`0000;ooooo1?l00002ooooo`;o00001oooool0
0ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`?o00003?ooool01ol0003ooooooooo
oooooooo0000ooooool000004oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00?o
oooo00Go0000ooooooooooooooooo`00000;ooooo`07o`000?ooooooooooooooool0003oooooo`00
0003ooooo`;o00001_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo017ooooo00Go
0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo1ol00008o`3o00kooooo
000<ooooo`So0?l00_l00002ooooo`?o00004oooool00ol0003oooooooooo`03ooooo`;o00007_oo
ool01?l0003oooooo`000?l00007ooooo`03o`000?oooooooooo00[ooooo00Ko0000oooooooooooo
0000ooooool00009ooooo`?o00002?ooool4o`0000Oooooo00?o0000ooooooooool00oooool02Ol0
003ooooooooooooooooo0000ooooool0003oooooo`00000;ooooo`07o`000?ooooooooooooooool0
003oooooo`00000Dooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00oooool2o`00
00;ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`;o00000oooool2o`00
00Gooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0@ooooo`;o00000oooool01Ol0
003ooooooooooooooooo000000[ooooo0_l00008o`3o00kooooo000<ooooo`So0?l03Ol0000>oooo
o`03o`000?oooooooooo00;ooooo0_l0000Nooooo`;o00000_ooool3o`0000Gooooo00?o0000oooo
ooooool02?ooool01?l0003oooooooooooooool3o`0000Oooooo0_l0000<ooooo`03o`000?oooooo
oooo00Co00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Go0000
ooooool0003oooooo`00000<ooooo`07o`000?ooooooooooooooool0003oooooo`00000Cooooo`05
o`000?ooooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo0000
00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool01?ooool2o`0000?ooooo00?o0000ooooool000000oooool00ol0
003oooooooooo`0Nooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool00_ooool01Ol0
003ooooooooooooooooo000000[ooooo0_l00008o`3o00kooooo000<ooooo`So0?l00_ooool5o`00
00;ooooo3_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0000Nooooo`;o00001Oooool2o`00
00?ooooo0_l00009ooooo`07o`000?ooooooooooooooool0003oooooo`000004ooooo`?o00003Ooo
ool00ol0003oooooooooo`05ooooo`Co00000oooool00ol0003oooooooooo`02ooooo`06o`000?oo
ooooooooo`000?oooooo00003Oooool01_l0003oooooooooool0003oooooo`0001Cooooo00?o0000
ooooool000002Oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`03o`000?oooooooooo023o
oooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool01oooool2o`0000So0?l03_ooool000gooooo1_l0o`05ooooo`Go0000
2?oooolBo`0001cooooo00Co0000oooooooooooo00001_ooool2o`0000;ooooo00?o0000oooooooo
ool01_ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool00_l0000?oooo
o`;o00002Oooool00ol0003oooooo`000002o`0000Gooooo00Ko0000oooooooooooo0000ooooool0
000=ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool04_ooool00ol0003ooooo
ooooo`09ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Oooooo0_l00003ooooo`;o00000_ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooo
oooo00cooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool6
o`0000Ko0?l03oooool000kooooo1?l0o`06ooooo`;o000000?oooooo`000?ooool00ol0000<oooo
o`So00000_ooool;o`0001;ooooo00Co0000oooooooooooo00002?ooool4o`0000Oooooo00?o0000
ooooooooool00_l000000ooooooo0000o`000002o`00013ooooo0_l0000:ooooo`05o`000?oooooo
ooooo`000?ooool01Ol00003ooooo`04o`000?ooooooooooo`0000gooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo000000[ooooo00Co0000oooooooo
oooo00000oooool01ol0003ooooooooooooooooo0000ooooool000001?ooool01?l0003ooooooooo
ool00006ooooo`03o`000?oooooooooo00Sooooo0_l00004ooooo`03o`000?oooooo000002Cooooo
00?o0000ooooooooool02_ooool2o`0000Gooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`02ooooo`;o00000_ooool3o`0000;ooooo1?l0o`0@ooooo`006?ooool2o`000003ooooool0
003ooooo00?ooooo1?l0000=ooooo`03o`000?oooooo000000So00001Oooool:o`0000Sooooo00Go
0000ooooooooooooooooo`00000:ooooo`;o00001Oooool01Ol0003oooooooooool0003ooooo00?o
00004_ooool2o`0000_ooooo00?o0000ooooool000001_ooool5o`0000;ooooo00?o0000oooooooo
ool03?ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00oooooo00Go0000oooooooo
ooooooooo`000009ooooo`03o`000?oooooo000000Cooooo00Co0000oooooooooooooooo0_l00006
ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool02Oooool00ol0003oooooo`000003
ooooo`;o00009_ooool00ol0003oooooooooo`08ooooo`;o00001Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooo000000;o00001?ooool3o`0001Kooooo000Hooooo`;o000000?oooooo`00
0?l000001oooool3o`0000[ooooo0_l00003ooooo`03o`000?oooooooooo00?ooooo2?l00007oooo
o`_o000000?oooooo`000?ooool02Oooool4o`0000;ooooo00?o0000ooooooooool01?l000000ooo
oooo0000ooooo`0@ooooo`03o`000?oooooo000000_ooooo0_l00006ooooo`03o`000?oooooooooo
00;ooooo1?l0000>ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000001?ooool0
1Ol0003ooooooooooooooooo000000Oooooo00?o0000ooooool000001?ooool00ol0003ooooooooo
o`0:ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`08ooooo`;o00001Oooool00ol0003oooooooooo`04ooooo`;o00001oooool3o`0001Kooooo
000?ooooo`CoO`001Oooool2o`000003ooooool0003o000000[ooooo1?l00007ooooo`05o`000?oo
ooooooooooooool000003Oooool8o`0000Oooooo00?o0000ooooooooool02_l00003ooooo`;o0000
00Goooooo`000?l0003o0000ooooo`02o`000005ooooool0003oooooooooool000004?ooool00ol0
003oooooo`00000;ooooo`;o00001oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00Go00002oooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`07ooooo`;o00001?ooool00ol0003oooooooooo`02o`00
00Oooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0;ooooo`04o`000?oooooo
ooooooooo`Co00009oooool00ol0003oooooooooo`06ooooo`;o00001_ooool01?l0003ooooooooo
ooooool3o`000003ooooool0003ooooo00Gooooo1?l000000oooooooool0oooo0002oooo017ooooo
000>ooooo`KoO`001?ooool2o`000004ooooool0003oooooo`0000gooooo0ol00003ooooo`;o0000
0oooool00ol0003oooooooooo`0Cooooo`So00001?ooool00ol0003oooooooooo`05ooooo`_o0000
0_ooool00ol0003oooooo`00000@ooooo`03o`000?oooooo000000_ooooo0_l00007ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`Co00001oooool00ol0003o
ooooo`000003ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00Ko0000oooooooooooo0000ooooool00006
ooooo`06o`000?ooooooooooo`000?oooooo00003_ooool01_l0003ooooooooooooooooo0000oooo
o`;o0000:?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000Gooooo00?o0000oooo
ooooool00_l00004ooooo`03o`000?oooooooooo00Cooooo00Co0000ooooool0003ooooo1_ooo`0@
ooooo`003Oooool8ogl000?ooooo00Co0000oooooooooooooooo0_l0000@ooooo`Co000000Gooooo
o`000?ooooooooooo`00000Kooooo`03o`000?oooooo000000Oo00002?ooool4o`000003ooooool0
003ooooo00[o00002_ooool01?l0003oooooooooool0000:ooooo`03o`000?oooooo000000Oooooo
00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000Oooooo1?l00004ooooo`03
o`000?oooooo000000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool00ol0003oooooo`000002ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00_l0000?ooooo`07o`000?ooooooooooo`00
0?oooooo0000ooooo`02o`0002Oooooo00?o0000ooooooooool00oooool00ol0003oooooo`000006
ooooo`?o00001_ooool00ol0003oooooooooo`04ooooo`?o00002?ooo`0?ooooo`003Oooool8ogl0
00?ooooo00Oo0000ooooooooooooooooo`000?oooooo000001;ooooo1?l000000ooooooo0000oooo
o`0Jooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02Ol000001?oooooo0000oooo
ool00003ooooo`Co00001Oooool;o`000003ooooool0003ooooo00Wooooo00Co0000oooooooooooo
00001_ooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000002oooool4o`00
0003ooooool0003o000000?ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo
000000Oooooo00?o0000ooooool000004?ooool00ol0003oooooo`000003ooooo`?o0000:?ooool0
1Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool00_ooool3o`0000Sooooo00?o
0000ooooooooool01?ooool3o`0000Soool03oooool000gooooo2?mo0003ooooo`07o`000?oooooo
ooooooooool0003oooooo`00000Aooooo`04o`000?ooooooooooooooo`Go00006Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00?ooooo0_l000000ooooooo0000o`000006o`000005
ooooool0003oooooo`000?ooool00_l0000;ooooo`03o`000?oooooooooo00co00000oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`00000?
ooooo`Co00000oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`000004ooooo`;o00000oooool00ol0003oooooooooo`05
ooooo`?o00004?ooool2o`0000Cooooo0ol0000Yooooo`03o`000?oooooo000000;ooooo00?o0000
ooooooooool00ol000000ooooooo0000ooooo`09ooooo`03o`000?oooooooooo00;ooooo00Co0000
ooooool0003o00002?ooo`0?ooooo`003Oooool8ogl000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`03o`000?oooooooooo00kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`00
0?oooooo000000;o00005_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000;o0000
1Oooool3o`0000?ooooo2Ol00007ooooo`;o00000oooool00ol0003oooooooooo`07ooooo`co0000
1oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00kooooo1_l00006ooooo`03o`00
0?oooooooooo00kooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o0000oooo
ool000000oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo0000013ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooo`00000Xooooo`06o`000?oooooo0000ooooool0003ooooo0_l0
0004ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool2o`000003ooooool0
003oool000Ooool03oooool000kooooo1_mo0004ooooo`05o`000?ooooooooooooooool000000_oo
ool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?oo
ool3o`0001?ooooo00?o0000ooooooooool01_ooool2o`0000Oooooo0ol000001Ooooooo0000oooo
oooooooo000000;ooooo00?o0000ooooooooool00_ooool8o`0000Cooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo2ol00002ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooooooool00_ooool4o`000003ooooool0003ooooo013ooooo
00Oo0000ooooooooooooooooo`000?oooooo000000;ooooo00Co0000oooooooooooo00000oooool0
0ol0003oooooooooo`04ooooo`;o000000?oooooo`000?ooool03_ooool2o`0000Kooooo0ol0000Y
ooooo`03o`000?oooooo000000;o00001_ooool00ol0003oooooooooo`08ooooo`04o`000?oooooo
ooooooooo`?o000000?oooooo`000?ooool01_ooo`0@ooooo`003oooool4ogl000Gooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool0000>ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooool000001oooool4o`0000oooooo00?o0000ooooooooool00oooool3o`000003ooooool0
003ooooo00Kooooo00?o0000ooooool000000_ooool00ol0003oooooo`000002ooooo`03o`000?oo
oooooooo00Gooooo1?l000000ooooooo0000o`000006o`0000Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02_l0
000:ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooooooo1?l0000@ooooo`03o`00
0?oooooo000000;ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000Go
oooo00Go0000ooooool0003oooooo`00000>ooooo`03o`000?oooooo000000Kooooo0_l000000ooo
oooo0000ooooo`0Vooooo`Co00001oooool00ol0003oooooooooo`09ooooo`0;o`000?oooooooooo
ooooool0003oooooo`000?oooooo0000ooooool000001?ooo`0Aooooo`005oooool2o`0000Cooooo
00Co0000oooooooooooo00003_ooool00ol0003oooooooooo`04ooooo`;o00002oooool3o`0000co
oooo00Co0000oooooooooooooooo0_l00004ooooo`03o`000?oooooooooo00Cooooo00Co0000oooo
oooooooo00000_ooool2o`0000;ooooo00?o0000ooooooooool02?ooool3o`0000?ooooo00?o0000
ooooooooool00_ooool8o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool<o`0000Cooooo00?o0000
ooooool000001?ooool4o`0000gooooo0_l00003ooooo`03o`000?oooooooooo00;ooooo00Go0000
ooooooooooooooooo`000004ooooo`04o`000?ooooooooooooooo`;o00003Oooool01?l0003ooooo
ooooool00006ooooo`04o`000?oooooo0000o`0002Gooooo0ol000000ooooooo0000o`000007oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00ol00002ooooo`;o00005_ooool001Oo
oooo0_l00004ooooo`05o`000?ooooooooooooooool000003Oooool00ol0003oooooooooo`04oooo
o`;o00003_ooool4o`0000Oooooo00?o0000ooooool000000_l00007ooooo`03o`000?oooooooooo
00;ooooo0_l00003ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01oooool0
1?l0003oooooooooooooool3o`0000Sooooo00?o0000ooooooooool00_ooool9o`0000Wooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`0?ooooo`[o00001_ooool3o`0000_ooooo00?o
0000ooooooooool00_l00004ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooo
ooooo`02ooooo`;o00003?ooool01Ol0003ooooooooooooooooo000000Kooooo00Co0000oooooooo
oooo00008oooool3o`0000?ooooo0_l00007ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ool000000_l00002ooooo`03o`000?oooooooooo01Gooooo000Aooooo`CoiNGU0_ooool2o`0000Co
oooo00Go0000ooooooooooooooooo`00000<ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool04?ooool3o`0000Cooooo0_l0000:ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
0ol0003oooooooooo`02o`000003ooooool0003ooooo00Oooooo00?o0000ooooooooool01?ooool3
o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`Oo00000oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00oooooo0_l00003ooooo`;o00000oooool=
o`0000Kooooo00Co0000ooooool0003ooooo0_l00004ooooo`05o`000?ooooooooooooooool00000
0_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000[ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`05ooooo`04o`000?oooooo0000o`0001oooooo0ol000000ooooooo
0000ooooo`02ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02_ooool01?l0
003oooooo`000?l00003ooooo`03o`000?ooooooiNGU00?oiNGU4_ooool0013ooooo1_oUiND00ooo
oooo0000o`000004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo01?ooooo1Ol0000<ooooo`04o`000?oooooo0000
o`0000Kooooo0_l000001?oooooo0000ooooool00007ooooo`03o`000?oooooooooo00Cooooo00Co
0000oooooooooooooooo0_l00002ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`05ooooo`So000000?oooooo`000?ooool04Oooool2o`000005oooo
ool0003oooooooooool000003Oooool<o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooooooool2o`0000Kooooo00?o0000ooooool000002Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00007Oooool2o`0000Cooooo00Go0000
ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02ooo
ool4o`0000;ooooo0_l00006onGUiA7ooooo000?ooooo`SoiNGU0_l00005ooooo`05o`000?oooooo
ooooooooool000002oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo017ooooo0_l0
0003ooooo`?o00002_ooool2o`000003ooooool0003ooooo00Gooooo0_l00002ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`;o00002Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_oo
ool8o`0000cooooo0ol00003ooooo`03o`000?oooooooooo017ooooo3ol00002ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooool000002?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`00000Iooooo`?o00001Oooool01Ol0003ooooo
oooooooooooo000000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0;ooooo`Co
000000?oooooo`000?ooool02?oUiND@ooooo`003oooool8onGUi@;o00001Oooool01Ol0003ooooo
oooooooooooo000000[ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0>ooooo`?o
00001Oooool00ol0003oooooooooo`03o`0000Oooooo0ol00008ooooo`03o`000?oooooooooo00;o
00001Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool0
00000_l00006ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00?ooooo2?l00005ooooo`;o00001?ooool00ol0003oooooooooo`0Boooo
o`Oo00001?ooool00ol0003oooooo`00000<o`0000Sooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`05o`000?ooooooooooooooool000005oooool2o`0000Oooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000003Ooo
ool3o`000004ooooool0003oooooooooo`SoiNGU4?ooool000oooooo2?oUiND2o`0000Gooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo0_l0
000<ooooo`?o00002?ooool00ol0003oooooooooo`03ooooo`Co00000_ooool01?l0003oooooo`00
0?l00008ooooo`03o`000?oooooooooo00;o00001?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Cooooo00Co0000oooooooooooo
00003?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00_ooooo2?l00004ooooo`03
o`000?oooooooooo013ooooo0_l00004ooooo`Go000000Koooooo`000?oooooo0000ooooool00008
ooooo`[o00002?ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`;o00004ooo
ool3o`0000Sooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooo000000kooooo1?l00002ooooo`03o`000?oUiNGoiNGU00KoiNGU4?ooool000oooooo2?oUiND0
0ooooooo0000ooooo`04ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooo000000Wooooo0_l0000:ooooo`03o`000?oooooo
oooo00Sooooo0ol000000ooooooo0000o`000007ooooo`07o`000?oooooo0000ooooool0003ooooo
o`000002ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`06ooooo`;o000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool03_ooool01?l0003oooooo`000?ooool9o`0000oo
oooo00?o0000ooooooooool01?ooool2o`0000;ooooo1?l000000ooooooo0000ooooo`07ooooo`;o
00001_ooool00ol0003oooooo`00000:o`0000Cooooo00?o0000ooooooooool00_ooool2o`00013o
oooo0_l0000:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01_ooool00ol0003o
ooooo`00000=ooooo`04o`000?oooooo0000o`0000?ooooo00?o0000onGUiOoUiND01_oUiND@oooo
o`004?ooool6onGUi@03o`000?oooooo000000Gooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooool000001_ooool3o`0000co
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04o`0000Kooooo00Co0000ooooool0
003o00000oooool2o`0000Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`09oooo
o`?o000000?oooooo`000?ooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00Oo0000ooooooooooooooooo`000?oooooo000000Cooooo00?o0000ooooooooool01ol0
0007ooooo`;o00001oooool01Ol0003ooooooooooooooooo000000;ooooo1?l00006ooooo`;o0000
1Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00_o00003Oooool3o`0000_ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`;o00003Oooool01?l0003ooooo
o`000?l00003ooooo`03o`000?ooooooiNGU00GoiNGU4Oooool0017ooooo1?oUiND01?oooooo0000
ooooool00006ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`09ooooo`04o`000?ooooooooooo`0000?ooooo0_l0000?ooooo`03o`000?oooooooooo
00Oooooo00Go0000ooooooooooooooooo`000002ooooo`Co00000oooool01?l0003oooooooooool0
0003ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`0;ooooo`04o`000?oooooo0000o`0000gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0;ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool00ol0003o
ooooooooo`07ooooo`So00002?ooool00ol0003oooooooooo`02o`0000?ooooo00?o0000oooooooo
ool01?l00002ooooo`;o00001?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00_ooool;o`000003ooooool0003o000000kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`08ooooo`;o00003Oooool00ol0003oooooo`000004ooooo`03
o`000?oooooooooo00CoiNGU4_ooool001Kooooo00?o0000ooooool000001_ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_ooool00ol0003ooooo
ooooo`04o`00017ooooo00?o0000ooooooooool01oooool01?l0003oooooooooool00004ooooo`03
o`000?oooooooooo00Co00000_ooool01_l0003oooooooooool0003oooooo`0000Oooooo00Co0000
oooooooooooooooo0_l0000>ooooo`04o`000?oooooo0000ooooo`;o00002oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool00oooool01?l0003ooooo
ooooool00004ooooo`03o`000?oooooooooo00cooooo3Ol000000ooooooo0000ooooo`02ooooo`03
o`000?oooooooooo00Cooooo1?l00003ooooo`03o`000?oooooooooo00[ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo3?l00006ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool02?ooool2o`0000cooooo00?o0000ooooool0
00000_l00003ooooo`03o`000?oooooooooo01Kooooo000Fooooo`03o`000?oooooo000000Kooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo
00?o0000ooooool000000_l0000Cooooo`03o`000?oooooooooo00Kooooo00Go0000oooooooooooo
ooooo`000004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?l00003ooooo`03
o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`00000@ooooo`03o`000?oooooo0000
00?ooooo0_l00009ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`05ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool02_ooool0
0ol0003oooooo`000008ooooo`Wo00002Oooool5o`0000cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo0_l0000<ooooo`co00002oooool0
0ol0003oooooo`00000;ooooo`?o000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`0F
ooooo`004oooool4ool0o`03ooooool0003ooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo0ol00002ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000Kooooo00Go0000
ooooooooooooooooo`000002ooooo`Go00000_ooool00ol0003oooooooooo`02ooooo`05o`000?oo
ooooooooooooool000004Oooool2o`0000Kooooo0ol00006ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo000000Wooooo0_l000001?oooooo
0000ooooool00002ooooo`So00000_ooool00ol0003oooooo`000002ooooo`Co00002?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000000ol0000Boooo
o`03o`000?oooooooooo00?ooooo3Ol00003ooooo`03o`000?oooooooooo00Oooooo0_l000001?oo
oooo0000ooooool00003ooooo`Coo`005Oooool001;ooooo1_oo0?l00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`00000:ooooo`;o00000_ooool0
1Ol0003ooooooooooooooooo000001;ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooo
oooooooo000000Oooooo00Co0000oooooooooooo00000oooool01Ol0003oooooooooool0003ooooo
00Co00000oooool01?l0003oooooooooool0000Aooooo`03o`000?oooooo000000Wooooo0_l00004
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07
ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003oooooooooo`07ooooo`04o`000?oo
ooooooooo`0000Sooooo00?o0000ooooool000000_ooool2o`0000;ooooo00?o0000ooooooooool0
1Oooool8o`0000?ooooo1?l00004ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
0oooool4o`0001?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`09ooooo`[o0000
1?ooool4o`000004ooooool0003oooooooooo`Koo`005?ooool0017ooooo2?oo0?l7ooooo`03o`00
0?oooooooooo00Cooooo00Co0000oooooooooooo00001oooool3o`0000Cooooo00Go0000oooooooo
ooooooooo`00000Aooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooo000000;ooooo0_l00002ooooo`03o`000?oooooooooo
00?ooooo0ol00002ooooo`03o`000?oooooooooo013ooooo0_l0000<ooooo`;o00000_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02Oooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool01_ooool01Ol0003ooooooooooooo
oooo000000Oooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000So
oooo0_l00004ooooo`[o000000?oooooo`000?ooool01_ooool01?l0003oooooooooooooool2o`00
0005ooooool0003oooooooooool000004_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`_o00002?oo000C
ooooo`004Oooool8ool0o`Oooooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00005
ooooo`;o00001oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Oooooo00Oo0000
ooooool0003oooooo`000?oooooo000000?ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`000004o`00017ooooo0_l0000>ooooo`?o00002?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00[ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Co
oooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000oooool00ol0003oooooooooo`05ooooo`?o00003?ooool7o`0000Cooooo1?l00003
ooooo`04o`000?ooooooooooo`00017ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05o`0000Soo`004oooool0017ooooo2?oo0?l7ooooo`03o`000?oooooooooo00Gooooo0_l00003
ooooo`?o00002Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo00Go0000
ooooool0003oooooo`000004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000000ooo
ool3o`0000kooooo00?o0000ooooooooool03oooool3o`0000Oooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`0<ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`04ooooo`04o`00
0?ooooooooooo`0000oooooo00?o0000ooooooooool02Ol00003ooooo`05o`000?oooooooooooooo
ool000003oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ool000004ol00008ool001?ooooo000Aooooo`Soo`3o0_l00005ooooo`03o`000?oooooooooo00Go
oooo0_l000000ooooooo0000o`00000<ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool03Oooool01Ol0003ooooooooooooooooo000000Gooooo00?o0000ooooooooool02Oooool2o`00
00;ooooo00?o0000ooooooooool00oooool01ol0003ooooooooooooooooo0000ooooool000001ooo
ool4o`0000[ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`02o`0000Gooooo00Go
0000ooooooooooooooooo`00000?ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo
ooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`04ooooo`08o`000?ooooooooooo`000?oooooo0000ooooool00005ooooo`05o`000?oo
ooooooooooooool000003oooool01?l0003oooooooooooooool7o`000003ooooool0003o000000Oo
00003Oooool00ol0003oooooooooo`04oooooaSo00000_ooool2o`0000;ooooo00?o0000oooooooo
ool00_ooool00ol0003oo`00ool00006ool001?ooooo000Booooo`Koo`3o00Co0000ooooool0003o
00001?ooool00ol0003oooooooooo`05ooooo`?o00003_ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00gooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`09oooo
o`;o00000_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooooooo`;o000000?ooooo
o`000?ooool02Oooool3o`0000Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`02
ooooo`?o00000_ooool01?l0003oooooooooool0000@ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00gooooo00Co0000ooooool0003o00001Oooool00ol0003o
ooooo`000003o`000003ooooool0003ooooo00;ooooo5_l00003ooooo`[o00002Oooool01Ol0003o
ooooo`000?oooooo000000;ooooo00?o0000ooooooooool00_ooool00ol0003oooooool00005ool0
01Cooooo000Cooooo`Coo`3o00Coooooo`000?oooooooooo0ol00003ooooo`03o`000?oooooooooo
00;ooooo1?l0000@ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?ooool00ol0
003oooooo`000006ooooo`03o`000?oooooooooo00Wooooo00Ko0000oooooooooooo0000ooooool0
0005ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0;ooooo`Co00000_oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Gooooo0ol000000ooooooo0000oooo
o`0Aooooo`03o`000?oooooooooo00;ooooo00Oo0000ooooooooooooooooo`000?oooooo000000Go
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`07ooooo`04o`000?ooooooooooo`00
00;ooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00cooooo0ol00007
oooooa3o00000ooooolAo`0000Wooooo00?o0000ooooooooool01oooool01?l0003oooooo`000?l0
0002ooooo`;o00001Oooool2o`0000Coo`005Oooool001Sooooo0_l00003ooooo`?o000000?ooooo
o`000?ooool00ol00002ooooo`03o`000?oooooo000000oooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`0;ooooo`03o`000?oooooo000000Kooooo00?o0000ooooooooool02Oooool2
o`0000;ooooo0_l00007ooooo`?o00001?ooool00ol0003oooooooooo`0>ooooo`?o00004_ooool0
0ol0003oooooooooo`07ooooo`;o00004oooool00ol0003oooooooooo`03ooooo`05o`000?oooooo
ooooooooool000001_ooool01Ol0003ooooooooooooooooo000000[ooooo00Co0000oooooooooooo
00000_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00Gooooo4_l00007oooo
o`03o`000?oooooo000000?o000000?oooooo`000?l000002?l00002ooooo`So00002_ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01?ooool3o`00
01Sooooo000Hooooo`03o`000?oooooo000000?ooooo1Ol00004ooooo`04o`000?ooooooooooo`00
00oooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0:ooooo`;o00001oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool01Oooool2o`00
00Kooooo00?o0000ooooooooool03_ooool5o`0000oooooo00?o0000ooooooooool01oooool5o`00
013ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02o`0000Gooooo00Go0000oooo
ooooooooooooo`00000;ooooo`04o`000?ooooooooooo`0000;ooooo4_l00002ooooo`?o00000ooo
ool00ol0003oooooooooo`06ooooo`04o`000?ooooooooooooooo`ko00001?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00;ooooo2?l000000ooooooo0000ooooo`09ooooo`03o`00
0?oooooo000000?ooooo00?o0000ooooooooool01?ooool2o`0001Wooooo000Fooooo`Co001o00Co
ooooo`000?oooooooooo1_l00003ooooo`04o`000?ooooooooooo`0000oooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool02?ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00Gooooo0_l00006ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooool000000oooool4o`0000_ooooo00?o0000oooooooo
ool01_ooool00ol0003oooooo`000003ooooo`;o00003oooool00ol0003oooooooooo`03ooooo`;o
00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo4Ol00007oooo
o`;o00001_ooool2o`0000Kooooo00?o0000ooooooooool02_l00005ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooooooo1?l00007ooooo`03o`000?oooooooooo00_ooooo1ol00004
ooooo`04o`000?ooooooooooo`0000;ooooo0_l00006ooooo`Co0?oo5oooool001Gooooo1_l007l0
0ooooooo0000o`000003ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000oooooooooooooooo
o`00000>ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02?ooool2o`0000Oooooo
00?o0000ooooooooool01oooool01?l0003oooooooooool00004ooooo`;o00001Oooool00ol0003o
ooooo`000007ooooo`03o`000?oooooooooo00cooooo0_l00008ooooo`?o00002?ooool00ol0003o
ooooooooo`05ooooo`04o`000?ooooooooooo`0000Gooooo0_l0000=ooooo`03o`000?oooooooooo
00?ooooo4Ol00009ooooo`03o`000?oooooo000000;ooooo0_l000000ooooooo0000ooooo`06oooo
o`04o`000?ooooooooooooooo`ko00001_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000003o`0000?ooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`03ooooo`Wo000000?oooooo`000?l000001Oooool6
o`3ooaKooooo000Dooooo`So001o00?oooooo`000?ooool00oooool2o`000003ooooool0003o0000
00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`07ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooo`000004ooooo`;o00000_ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooool000002oooool4o`0000Cooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo0_l00002
oooooa7o00000oooool01_l0003oooooooooool0003oooooo`0000oooooo00?o0000ooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo2_l00008ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`05o`000?oooooooooooooo
ool000001_ooool4o`0000gooooo00?o0000ooooooooool01_ooool01_l0003ooooooooooooooooo
0000ooooo`So00002?l0oolEooooo`005?ooool8o`00O`;ooooo00So0000ooooooooooooooooo`00
0?oooooo0000ooooo`?o00000oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo
00Oooooo00Go0000ooooooooooooooooo`000004ooooo`04o`000?ooooooooooo`0000;ooooo0_l0
0003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00
000?ooooo`?o000000?oooooo`000?ooool01?oooolBo`0000Wooooo00Co0000oooooooooooo0000
0oooool01_l0003ooooooooooooooooo0000ooooo`;o00004?ooool00ol0003oooooo`00000:o`00
00;ooooo0ol0000=ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`03ooooo`03o`000?oooooo000000Wooooo00?o0000ooooool000000ol00009oooo
o`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000002ooooo`;o00001Oooool8
o`3ooaGooooo000Dooooo`So001o0oooool01?l0003oooooooooool00002ooooo`;o00000_ooool2
o`000003ooooool0003ooooo00cooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool01Oooool2o`0000Cooooo00?o
0000ooooooooool00_ooool02?l0003oooooooooool0003oooooo`000?oooooo00001?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooo00002_ooool@
o`0000Gooooo00?o0000ooooooooool02Oooool2o`0000Oooooo00?o0000ooooool000001?ooool0
1ol0003ooooooooooooooooo0000ooooool000002Oooool<o`0000;ooooo00Go0000oooooooooooo
ooooo`000002ooooo`;o000000?oooooo`000?l000003Oooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool2o`0000Sooooo00?o0000oooooooo
ool01?ooool4o`0000Gooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo0000
00?ooooo0_l00005ooooo`So0?oo5Oooool001Cooooo2?l007l4ooooo`03o`000?oooooo000000;o
oooo00?o0000ooooool000000oooool3o`0000gooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`03ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool2o`000003ooooool0003o
oooo00Cooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05oooooa3o00002Oooool0
0ol0003oooooooooo`03o`0000Kooooo00?o0000ooooooooool02oooool2o`0000Gooooo00?o0000
ooooool000001Oooool01?l0003oooooooooooooool;o`0000Wooooo0ol00002ooooo`04o`000?oo
ooooooooooooo`?o00000oooool2o`0000gooooo00?o0000ooooooooool01oooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_ooool00ol0003ooooo
ooooo`09ooooo`?o000000?oooooo`000?ooool01_ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00;o000000?oooooo`000?ooool00oooool8o`3ooaGooooo000Eooooo`Ko001o1_oo
ool2o`0000;ooooo00?o0000ooooooooool00_l00003ooooo`;o00003?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`000005ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool@o`000005ooooool0003ooooooooo
ool000005?ooool01?l0003oooooooooool00002ooooo`?o00001?ooool00ol0003oooooooooo`0<
ooooo`?o00000oooool=o`0001;ooooo00?o0000ooooool000000_ooool01?l0003oooooo`000?l0
0002ooooo`04o`000?ooooooooooooooo`;o00003Oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000007ooooo`03o`00
0?oooooooooo00cooooo1?l00004ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
oooo0_l00002ooooo`05o`000?ooooooooooooooool000001_l0oolFooooo`005_ooool4o`00O`So
oooo0_l000000ooooooo0000ooooo`03ooooo`04o`000?ooooooooooooooo`;o00002oooool00ol0
003oooooooooo`07ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000004Ol00008
ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00005?ooool00ol0003oooooo`00
0006ooooo`Go00002oooool:o`0000Oooooo0_l000000ooooooo0000o`00000Aooooo`;o000000?o
ooooo`000?ooool00ol00005ooooo`03o`000?oooooooooo00;o00003Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`04o`0000Gooooo
00Co0000oooooooooooooooo0_l00002ooooo`04o`000?ooooooooooo`0000;ooooo1?l0oolGoooo
o`008oooool2o`0000Kooooo0_l000000ooooooo0000ooooo`02o`0000Wooooo00?o0000oooooooo
ool01oooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00;ooooo4Ol00003ooooo`;o
00000oooool00ol0003oooooo`000007ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool00oooool01Ol0003ooooooooooooooooo000001Cooooo0_l0000;ooooo`co00001oooool4o`00
00Kooooo0ol00002ooooo`03o`000?oooooooooo00gooooo00Co0000ooooool0003ooooo0_l00000
0ooooooo0000ooooo`06ooooo`04o`000?oooooo0000o`0000gooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000001?l0
0003ooooo`03o`000?oooooo000000?ooooo00?o0000ooooool000007Oooool001Wooooo1?oo0006
ooooo`;o00002?ooool00ol0003oooooo`000002ooooo`;o00001oooool00ol0003oooooooooo`05
oooooa3o00001?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo0_l00003
ooooo`04o`000?ooooooooooo`0000Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`03ooooo`05o`000?ooooooooooooooool000005?ooool01?l0003oooooooooooooool;o`000003
ooooool0003o000000;o00002oooool01?l0003oooooooooooooool3o`0000?ooooo00Co0000oooo
oooooooo00000_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo000000?o00000_ooool0
0ol0003oooooooooo`06ooooo`;o000000?oooooo`000?ooool02oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00Sooooo0_l00006ooooo`05o`000?ooooooooooooooool000004?oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo1_l00005ooooo`Coool06ooo
ool001Sooooo1_oo0005ooooo`;o00002Oooool2o`0000?ooooo4?l00002ooooo`;o00001_ooool0
1?l0003oooooooooool00005ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000001oooool00ol0003ooooo
ooooo`06ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003oooooooooo`0<ooooo`[o
00002_ooool00ol0003oooooooooo`03ooooo`?o00001oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo00?o00000oooool01Ol0003ooooooooooooooooo000000cooooo0ol00002oooo
o`;o00000_ooool00ol0003oooooooooo`06ooooo`;o000000?oooooo`000?ooool02oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01Oooool00ol0
003oooooo`00000Aooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool01?l0
003oooooooooooooool4o`0000;ooooo1_ooo`0Jooooo`005oooool8ool000Cooooo4?l00003oooo
o`;o00000oooool00ol0003oooooooooo`06ooooo`04o`000?ooooooooooo`0000Cooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo
0_l00003ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`07ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooooooooooooooo2ol00004oooo
o`03o`000?oooooo000000gooooo00?o0000ooooooooool01_ooool4o`0000;ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`03o`000004ooooool0003oooooo`0000?ooooo00?o0000
ooooooooool01oooool3o`0000Cooooo0_l00002ooooo`03o`000?oooooooooo00Oooooo0ol0000<
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool2o`0000Oooooo00?o0000
ooooool000004?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00Oo0000
ooooooooooooooooo`000?oooooo000000;ooooo00?o0000oooo0?ooo`001_ooo`0Iooooo`005ooo
ool8ool000Ko000000?oooooo`000?ooool02?ooool3o`0000Gooooo0_l000000ooooooo0000oooo
o`06ooooo`05o`000?ooooooooooooooool000001?ooool01?l0003oooooooooool00004ooooo`03
o`000?oooooooooo00Sooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooool000002Ol0000>ooooo`04o`000?ooooooooooo`0000gooooo00?o0000ooooooooool02_oo
ool3o`0000Kooooo00?o0000ooooool000000oooool3o`000005ooooool0003oooooooooool00000
1oooool3o`0000Kooooo0_l00002ooooo`03o`000?oooooooooo00Sooooo0_l0000<ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01_ooool00ol0003oooooo`000008ooooo`03o`00
0?oooooooooo00oooooo00Go0000ooooooooooooooooo`000007ooooo`04o`000?oooooooooooooo
o`;o00001?ooool8oooo01Wooooo000Gooooo`Soo`000oooool00ol0003oooooo`000002ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooool000001_ooool00ol0003oooooo`000005ooooo`03
o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000002oooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`co00001?ooool00ol0003oooooooooo`0?ooooo`05o`00
0?ooooooooooooooool000003Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00?o
00000oooool2o`0000Gooooo00Co0000ooooool0003o00000oooool01?l0003oooooooooooooool3
o`000003ooooool0003ooooo00Oooooo0_l00002ooooo`03o`000?oooooooooo00Oooooo0_l0000<
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooo
oooo000000Oooooo0_l0000@ooooo`04o`000?ooooooooooo`0000Oooooo00?o0000ooooooooool0
0_ooool2o`000004ooooool0003o0000o`0000Soool06Oooool001Oooooo2?oo0003ooooo`03o`00
0?oooooo000000?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02o`0000Gooooo
0_l00004ooooo`03o`000?oooooooooo00Cooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00?o0000ooooooooool02_ooool00ol0003oooooo`000002ooooo`04o`000?oooooooooo
o`0000Gooooo2ol00009ooooo`;o00001?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00?ooooo1Ol00004ooooo`03o`000?oooooooooo00;ooooo0_l000000ooooooo0000ooooo`02
o`0000?ooooo00?o0000ooooooooool01oooool2o`0000;ooooo00?o0000ooooooooool01oooool3
o`0000_ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00000@ooooo`04o`000?oooooo
ooooo`0000Oooooo00Co0000oooooooooooooooo1?l00003ooooo`Soool06Oooool001Sooooo1_oo
0004ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo00?ooooo0ol000000ooooooo0000ooooo`07ooooo`03o`000?oo
oooooooo00;o00000oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo000000;ooooo2_l0
0003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_ooool2o`0000Cooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000;o00000_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000;o00001?ooool00ol0003ooooooooo
o`08ooooo`;o00000_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooo000000cooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000kooooo00Co0000oooooooooooo
00001oooool6o`000003ooooool0003ooooo00;ooooo0_l00006oooo01[ooooo000Iooooo`Coo`00
1?ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;o00002?ooool00ol0003o
ooooooooo`02o`0000;ooooo00?o0000ooooooooool01Oooool;o`000004ooooool0003oooooo`00
00[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`04o`000?oooooooooo
o`0000?ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`04o`000?oooooo
ooooo`0000;ooooo1?l00003ooooo`;o00000_ooool3o`000003ooooool0003ooooo00_ooooo00Co
0000oooooooooooo00002Oooool01?l0003oooooo`000?l0000;ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo000000Co
oooo1Ol00004ooooo`;o00000oooool2o`0000;ooooo1?ooo`0Kooooo`008Oooool01Ol0003ooooo
oooooooooooo000000Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`;o
00000_ooool00ol0003oooooo`000002ooooo`;o00001oooool3o`000003ooooool0003o000000So
00001_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000cooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`04ooooo`07o`000?ooooooooooooooool0003oooooo`00000>
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool01_l0003ooooooooooooo
oooo0000ooooo`?o00000_ooool01Ol0003oooooooooool0003ooooo00;o00003?ooool01?l0003o
ooooooooool00009ooooo`05o`000?oooooo0000ooooool000002_ooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooo
oooo000000Kooooo00?o0000ooooooooool02_ooool2o`000003ooooool0003o000000;o00001?oo
ool02?l0003ooooooooooooooooo0000ooooool0003ooooo0ol0000Rooooo`008Oooool01Ol0003o
oooooooooooooooo000000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04oooo
o`04o`000?oooooo0000o`0000Gooooo3?l0000>ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooool000003?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`00000=ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`03o`0000?ooooo1?l00002ooooo`03o`000?oooooo
oooo00;o00002_ooool01?l0003oooooooooool00009ooooo`03o`000?oooooo000000;ooooo00?o
0000ooooooooool01oooool00ol0003oooooooooo`07ooooo`05o`000?ooooooooooooooool00000
2Oooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00Sooooo1Ol00007ooooo`03
o`000?oooooooooo00;ooooo00Go0000ooooool0003o0000ooooo`02o`0002;ooooo000Mooooo`Co
0?oo1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00oooool:o`000003ooooool0003o000000Gooooo0ol0000>ooooo`;o00001Oooool2o`0000ko
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?o
oooo0_l0000<ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool3o`0000So
oooo1?l00004ooooo`?o00001oooool01?l0003oooooooooool00009ooooo`03o`000?oooooo0000
00?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo0000
00[ooooo00?o0000ooooool000002?ooool00ol0003oooooooooo`03ooooo`Co000000?oooooo`00
0?ooool02Oooool01_l0003ooooooooooooooooo0000ooooo`Go00001?moOglOooooo`007?ooool6
o`3oo`?ooooo00?o0000ooooooooool01_ooool;o`0000;ooooo0ol00008ooooo`;o00000_ooool2
o`0000oooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo00Cooooo00Go0000ooooooooooooooooo`000007ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`?o000000?oooooo`000?ooool02?oo
ool2o`000003ooooool0003o000000;o00000oooool2o`0000Gooooo0_l000000ooooooo0000oooo
o`08ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`08ooooo`;o00003?ooool2o`0000Wooooo1Ol00005ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool00_ooool4o`0000;ooooo1_moOglNooooo`006oooool8o`3oo`03ooooool0
003o000000[o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?o00002?oo
ool5o`0000gooooo00?o0000ooooool000001Oooool2o`00013ooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooool00007ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0=ooooo`05o`000?ooooooooooooooool000002?ooool00ol0003o
ooooooooo`02ooooo`?o00000_ooool2o`0000[ooooo00Co0000oooooooooooo00000oooool3o`00
00;ooooo0_l00004ooooo`03o`000?oooooo000000Wooooo00Co0000oooooooooooo00000oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02oooool0
0ol0003oooooooooo`04ooooo`Go00002?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooool000000_ooool8ogmoOagooooo000Kooooo`So0?oo00Co0000oooo
oooooooo00002Oooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00008ooooo`Co00002oooool2o`0000;ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool00000
1oooool00ol0003oooooo`000008ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0
3_ooool00ol0003oooooo`000008ooooo`04o`000?ooooooooooooooo`;o00001_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool00oooool3o`000008
ooooool0003o0000o`000?oooooo0000ooooool0000:ooooo`03o`000?oooooo000000Cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00[ooooo0_l0
0002ooooo`Co00001Oooool00ol0003oooooooooo`04ooooo`;o00002Oooool00ol0003ooooooooo
o`02ooooo`04o`000?oooooo0000o`0000;ooooo2?moOglMooooo`006oooool8o`3oo`;o000000?o
ooooo`000?ooool02Oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooo
ooooo`000003ooooo`;o00001Oooool2o`000004ooooool0003o0000o`0000Wooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool01?ooool2o`0000Oooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooool000001oooool00ol0003oooooo`000002
o`0000Sooooo00?o0000ooooooooool01_ooool01Ol0003ooooooooooooooooo000000;ooooo00?o
0000ooooooooool01Oooool6o`000003ooooool0003ooooo00Wooooo00?o0000ooooool000001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo0_l0000;ooooo`Go00002_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Wooooo00?o0000ooooooooool01Ol0
0002ooooo`03o`000?moOgooOgmo00KoOgmo7Oooool001_ooooo2?l0ool2ooooo`?o00002_ooool0
1ol0003ooooooooooooooooo0000ooooool000001?ooool00ol0003oooooooooo`03ooooo`04o`00
0?ooooooooooooooo`;o000000Coooooo`000?oooooooooo0_l00007ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool00oooool2o`0001;ooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`06ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0>ooooo`;o00001oooool3o`0000[ooooo0_l00007ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00002oooool5o`0000Wooooo00?o0000ooooool000001_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000001ooo
ool5o`0000;ooooo00?o0000ooooooooool02Oooool01ol0003ooooooooooooooooo0000ooooool0
00002_ooool00ol0003oooooo`000002ooooo`03o`000?oooooo000000;ooooo00?o0000ogmoOomo
Ogl01_moOglMooooo`007?ooool6o`3oo`Cooooo00?o0000ooooool000000_l00008ooooo`03o`00
0?oooooooooo00;o00001Oooool00ol0003oooooooooo`04ooooo`?o000000Coooooo`000?oooooo
00000oooool3o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o
00005?ooool00ol0003oooooooooo`03ooooo`;o00001oooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00Cooooo0ol0000<ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool01?ooool01?l0
003oooooooooool0000;ooooo`Ko00001oooool01?l0003oooooooooool00006ooooo`05o`000?oo
ooooooooooooool000001oooool01?l0003oooooooooool00003ooooo`Co00001?ooool01Ol0003o
oooooooooooooooo000000_ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`07oooo
o`?o00000_ooool01_l0003oooooooooool0003oooooo`0000;ooooo1_moOglNooooo`007Oooool4
o`3oo`Gooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000ooooooooool00_l00005
ooooo`03o`000?oooooooooo00Gooooo0ol00002ooooo`05o`000?ooooooooooooooool000000_oo
ool2o`000003ooooool0003ooooo00Gooooo00?o0000ooooooooool00_ooool2o`0001Cooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000007ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000004?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo000000?ooooo0ol0
000>ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool01Oooool01?l0003ooooooooo
ool0000;ooooo`;o000000Coooooo`000?oooooooooo0ol00004ooooo`04o`000?ooooooooooo`00
00Oooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`05o`0000Oooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0:ooooo`;o00000_ooool00ol0003oooooooooo`07
ooooo`04o`000?ooooooooooooooo`;o000000Coooooo`000?oooooo00001?ooool4ogmoOaoooooo
000Wooooo`03o`000?oooooooooo00Cooooo0ol00003ooooo`;o000000?oooooo`000?ooool01?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00ooo
ool01?l0003oooooooooooooool2o`0000Gooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`00000Eooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00;ooooo0_l0000Aooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01?l000000ooooooo0000ooooo`0=ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooo
ooooool01oooool01?l0003oooooooooool0000:ooooo`;o000000?oooooo`000?ooool00oooool6
o`0000;ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00005ooooo`Co00000_oo
ool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02_oo
ool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00;o000000Koooooo`000?oooooo
0000ooooool0000Wooooo`008_ooool4o`1o0003ooooool0003ooooo00Sooooo1Ol00002ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00002oooo
o`;o00001?ooool01?l0003oooooooooool00002ooooo`;o00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00?o0000ooooooooool05?ooool00ol0003oooooo`000003ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00oooool2o`00017ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool03Oooool01ol0003ooooooooo
oooooooo0000ooooool000002oooool01?l0003oooooooooool0000:ooooo`?o00001_ooool7o`00
00Sooooo00?o0000ooooool000001_l00005ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`08ooooo`05o`000?oooooo0000ooooool000001oooool3
o`000005ooooool0003oooooooooool000000_ooool2o`0000Co0?oo9?ooool0027ooooo1_l0O`00
0ol0003oooooooooo`09ooooo`Co000000?oooooo`000?ooool01?ooool01Ol0003ooooooooooooo
oooo000000;ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo0_l00005ooooo`;o000000?o
ooooo`000?ooool00_ooool00ol0003oooooo`00000Gooooo`03o`000?oooooo000000Cooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00013ooooo
00?o0000ooooooooool02_ooool4o`000005ooooool0003oooooooooool000004?ooool01?l0003o
ooooooooooooool2o`0000cooooo00Co0000oooooooooooo00002_ooool3o`0000Wooooo0_l00000
1?oooooo0000o`000?l00005ooooo`Co00001?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`07ooooo`04o`000?oo
ooooooooooooo`;o00001_ooool01Ol0003oooooo`000?oooooo000000?ooooo00Co0000oooooooo
oooo00001_l0oolSooooo`008?ooool8o`1o00[ooooo00?o0000ooooooooool00_ooool3o`0000Co
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000008ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003ooooo
ooooo`0Fooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool01Ol0003ooooo
oooooooooooo000000Cooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`09ooooo`;o
00001Oooool2o`00013ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<ooooo`04
o`000?ooooooooooo`0000Wooooo0ol00009ooooo`03o`000?oooooo000000;o000000?oooooo`00
0?l000000ol00003ooooo`05o`000?ooooooooooooooool000001oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_l00004ooooo`03o`000?oooooo
000000So0?oo8_ooool0023ooooo2?l0O`09ooooo`03o`000?oooooooooo00Cooooo1ol000000ooo
oooo0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo00Wo0000ooooooooooooooooo`000?oo
oooo0000ooooool000001oooool3o`000005ooooool0003oooooooooool000006?ooool00ol0003o
ooooooooo`04ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool01_ooool4o`0000Kooooo00?o0000oooo
ooooool03oooool01ol0003ooooooooooooooooo0000ooooool000003_ooool00ol0003oooooo`00
0009ooooo`?o00002Oooool00ol0003oooooo`000003o`0000Cooooo0ol000001Ooooooo0000o`00
0?oooooo000000Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000?ooooo0_l00003ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00002?l0oolRooooo`008?ooool8o`1o0003o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`?o0000
0_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l000000ooooooo0000
ooooo`06ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo000000Oooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`04ooooo`;o00000_ooool00ol0003oooooooooo`04oooo
o`;o00004Oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00cooooo00Co0000
oooooooooooo00002?ooool00ol0003oooooo`000007ooooo`Co00000_ooool00ol0003oooooo`00
0002o`0000Cooooo1?l00009ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooool0003ooooo
o`000004ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00002?l0oolRooooo`00
8?ooool8o`1o0003o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`02o`0000;ooooo0ol00005ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003o
ooooo`000006ooooo`03o`000?oooooo000000;ooooo0_l0000Hooooo`03o`000?oooooo000000Go
oooo00Ko0000oooooooooooo0000ooooool00009ooooo`03o`000?oooooooooo00_ooooo00Co0000
oooooooooooooooo0ol00003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000004?oo
ool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00cooooo00Co0000oooooooooooo
00001oooool00ol0003oooooo`000002ooooo`Go00000oooool01?l0003oooooooooool00004oooo
o`;o00000oooool00ol0003oooooo`000003o`0000Kooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`02o`0000Cooooo0_l00004ooooo`03o`000?oooooooooo00;o00002?l0oolRoooo
o`008Oooool6o`1o0003ooooool0003o000000Cooooo00?o0000ooooooooool02Oooool3o`000003
ooooool0003ooooo00;ooooo00?o0000ooooool000000_l00002ooooo`04o`000?ooooooooooo`00
00Cooooo0ol00004ooooo`04o`000?ooooooooooo`0000?ooooo0ol0000Eooooo`05o`000?oooooo
ooooooooool000001Oooool01?l0003oooooo`000?l0000:ooooo`03o`000?oooooooooo00_ooooo
00Co0000ooooool0003o00001Oooool00ol0003oooooooooo`04ooooo`05o`000?oooooooooooooo
ool000003oooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ool000001oooool5o`0000Sooooo00Co0000oooooooooooo00001_ooool2o`000005ooooool0003o
0000ooooool000000_ooool4o`0000?ooooo00Go0000ooooooooooooooooo`00000<ooooo`05o`00
0?ooooooooooooooool000001_ooool01Ol0003oooooo`000?oooooo000000;ooooo0ol00005oooo
o`04o`000?ooooooooooo`0000;ooooo1_l0oolSooooo`008_ooool4o`1o00;ooooo0ol00002oooo
o`03o`000?oooooooooo00_ooooo0_l000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo
00;ooooo0ol000000ooooooo0000ooooo`06ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ool000001?ooool01?l0003oooooooooool0000Dooooo`05o`000?ooooooooooooooool000001Ooo
ool00ol0003oooooo`00000<ooooo`03o`000?oooooooooo00Wooooo0ol00006ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0=ooooo`;o00001_oo
ool00ol0003oooooooooo`0=ooooo`04o`000?ooooooooooo`0000;ooooo1Ol000000ooooooo0000
o`000009ooooo`05o`000?ooooooooooooooool000001oooool3o`0000;ooooo00?o0000oooooooo
ool00oooool4o`0000?ooooo00?o0000ooooooooool02oooool00ol0003oooooo`000007ooooo`;o
00000oooool2o`0000;ooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00003oooo
o`Co0?oo9?ooool002Oooooo1?l0ool2o`0000gooooo0_l000000ooooooo0000o`000002ooooo`03
o`000?oooooooooo00Gooooo1?l00007ooooo`;o00000_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`00017ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`;o
00003Oooool00ol0003oooooooooo`06ooooo`;o000000?oooooo`000?ooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?ooool00ol0003o
ooooo`000007ooooo`03o`000?oooooooooo00gooooo1Ol00005ooooo`03o`000?oooooo000000Wo
oooo00Go0000ooooooooooooooooo`000007ooooo`;o000000Goooooo`000?l0003oooooo`000007
ooooo`Co00003_ooool2o`0000Oooooo0_l00004ooooo`04o`000?ooooooooooo`0000Gooooo00?o
0000ooooooooool01?l007lYooooo`009_ooool6o`3oo`03o`000?oooooooooo00[ooooo00Go0000
oooooooooooo0000ooooo`03o`0000Oooooo0_l00003ooooo`?o00001?ooool01_l0003oooooo`00
0?l0003oooooo`0000Cooooo00?o0000ooooooooool01?ooool2o`0000oooooo00?o0000oooooooo
ool01?ooool01?l0003oooooooooooooool3o`0000gooooo00?o0000ooooooooool00oooool3o`00
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool02oooool2o`0000Sooooo00?o0000ooooooooool02_ooool4o`00
00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`000009ooooo`05o`000?oooooooooo
ooooool000001_ooool00ol0003oooooo`000003ooooo`?o00001_ooool00ol0003oooooo`000004
o`0000_ooooo00?o0000ooooooooool01Oooool2o`0000Cooooo00Go0000ooooool0003oooooo`00
0005ooooo`03o`000?oooooo001o00Go001o:?ooool002Gooooo2?l0ool2o`0000Wooooo00Go0000
ooooooooooooooooo`000002ooooo`;o00001oooool00ol0003oooooooooo`05ooooo`?o00000_oo
ool01?l0003oooooo`000?l00005ooooo`03o`000?oooooooooo00Kooooo0_l0000<ooooo`03o`00
0?oooooooooo00Gooooo00Go0000oooooooooooo0000ooooo`02o`0000kooooo00?o0000oooooooo
ool00_l00006ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`05ooooo`Go00001?ooool01?l0003oooooooooool00005ooooo`03o`000?oooooo0000
00Wooooo00Go0000ooooooooooooooooo`000005ooooo`04o`000?ooooooooooo`0000Gooooo0ol0
0004ooooo`03o`000?oooooo000000Cooooo0ol00008ooooo`;o00001Oooool00ol0003oooooo`00
0003ooooo`05o`000?ooooooooooooooool000001_ooool00ol0003o001oo`00O`06o`00ObOooooo
000Uooooo`So0?oo00?o0000ooooool000001oooool00ol0003oooooooooo`03ooooo`04o`000?oo
oooo0000o`0000Gooooo0ol0000:ooooo`?o000000Coooooo`000?oooooo00001?ooool00ol0003o
ooooooooo`08ooooo`;o00002_ooool00ol0003oooooooooo`06ooooo`;o00000_ooool2o`0000ko
oooo0ol00008ooooo`05o`000?ooooooooooooooool000001_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`04o`00
00[ooooo00?o0000ooooool000001Oooool00ol0003oooooo`000009ooooo`05o`000?oooooooooo
ooooool000001Oooool01?l0003oooooooooool00006ooooo`04o`000?oooooo0000o`0000;ooooo
0_l00008ooooo`Co00000oooool00ol0003oooooo`000003ooooo`;o000000?oooooo`000?ooool0
0_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooo`00O`07o`00ObOooooo
000Uooooo`So0?oo1?l00005ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooool0003ooooo
o`000003ooooo`04o`000?ooooooooooo`0000gooooo1_l000000ooooooo0000ooooo`0<ooooo`;o
00001oooool00ol0003oooooooooo`07ooooo`07o`000?ooooooooooooooool0003oooooo`00000;
ooooo`;o000000?oooooo`000?ooool02?ooool01Ol0003ooooooooooooooooo000000Gooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`;o00002Oooool5o`0000kooooo00Co
0000oooooooooooo00001?ooool01?l0003oooooo`000?l00008ooooo`05o`000?oooooooooooooo
ool000001?ooool01Ol0003ooooooooooooooooo000000Oooooo00?o0000ooooooooool01?l0000<
ooooo`?o00000oooool00ol0003oooooo`000003ooooo`04o`000?ooooooooooo`0000Gooooo0_l0
0004ooooo`03o`000?oooooo001o00Oo001o9oooool002Gooooo2?l0ool00ooooooo0000o`000004
o`000003ooooool0003ooooo00Oooooo0_l00002ooooo`03o`000?oooooo000000?ooooo00?o0000
ooooooooool02oooool01?l0003oooooooooooooool4o`0000oooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000;ooooo00?o0000ooooool000002?oo
ool3o`0000Cooooo00?o0000ooooooooool01_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool01oooool00ol0003oooooo`000005ooooo`Co0000
0_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooo000000Cooooo00Co0000ooooool0003o
00002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000oooo
ooooool01_ooool01?l0003oooooooooooooool3o`0000cooooo00?o0000ooooool000001?l00004
ooooo`03o`000?oooooo000000Gooooo00Co0000oooooooooooo00000oooool00ol0003oooooo`00
O`07o`00ObOooooo000Vooooo`Ko0?oo1?ooool8o`0000Kooooo0_l00002ooooo`;o00001?ooool0
0ol0003oooooooooo`0;ooooo`;o00001Oooool3o`0000kooooo0_l00003ooooo`03o`000?oooooo
oooo00Kooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00005ooooo`;o0000
2?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000;ooooo1?l00007ooooo`03o`00
0?oooooooooo00oooooo00?o0000ooooool000000oooool01?l0003oooooo`000?l00008ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`07ooooo`06
o`000?oooooo0000ooooool0003ooooo0_l00009ooooo`03o`000?oooooooooo00;ooooo1Ol00000
1?oooooo0000o`000?l00006ooooo`08o`000?ooooooooooooooool0003oooooo`000?ooool2o`00
00Ko001o:?ooool002Oooooo1?l0ool6ooooo`03o`000?oooooo000000;o00000_ooool4o`000004
ooooool0003oooooo`0000;ooooo0_l00003ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ool000001Oooool00ol0003oooooo`000003o`0000gooooo0_l000000ooooooo0000ooooo`06oooo
o`03o`000?oooooooooo00;ooooo0_l00003ooooo`03o`000?oooooooooo00?o00002_ooool00ol0
003oooooooooo`05ooooo`;o00001Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo1_l0000;ooooo`03o`000?oooooooooo00oooooo00Co0000oooooooooooo00000_ooool0
1?l0003oooooo`000?l00009ooooo`04o`000?ooooooooooooooo`;o00001_ooool00ol0003ooooo
ooooo`08ooooo`;o000000?oooooo`000?ooool00_ooool2o`0000Oooooo00Go0000oooooooooooo
ooooo`000002ooooo`03o`000?oooooooooo00Co00001Oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooo000000;o000000?oooooo`000?l007l00ol007lYooooo`00<_ooool00ol0003ooooo
ooooo`04o`0000?ooooo1?l000001Ooooooo0000oooooooooooo000000;ooooo00?o0000oooooooo
ool02_ooool2o`0000;ooooo00Go0000ooooooooooooooooo`000003ooooo`;o000000Coooooo`00
0?l0003o00003?ooool2o`0000Kooooo00?o0000ooooooooool00oooool2o`0000?ooooo0ol0000>
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0=
ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool03oooool00ol0003oooooo`000003ooooo`;o000000?oooooo`000?ooool01oooool0
1Ol0003ooooooooooooooooo000000Oooooo00?o0000ooooooooool02?ooool01?l0003ooooooooo
ool00005ooooo`;o00001?ooool00ol0003oooooooooo`02o`0000?ooooo00?o0000ooooooooool0
0_l00002ooooo`?o00000_ooool00ol0003oooooooooo`03ooooo`?o000000?oooooo`000?ooool0
;Oooool002gooooo1?l007l00ol0003oooooo`000002ooooo`;o000000?oooooo`000?l000000_l0
0002ooooo`Co00000_ooool00ol0003oooooo`00000<ooooo`;o00000_ooool01Ol0003ooooooooo
oooooooo000000Gooooo00Co0000oooooooooooooooo0ol00009ooooo`04o`000?oooooo0000o`00
00?ooooo00?o0000ooooooooool01Oooool01_l0003oooooo`000?l0003oooooo`00013ooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03_ooool5o`000003ooooool0
003ooooo00;ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0@ooooo`03o`000?oo
oooo000000;ooooo00Co0000ooooool0003o00002?ooool01Ol0003ooooooooooooooooo000000Oo
oooo00?o0000ooooooooool02?ooool2o`000003ooooool0003ooooo00Kooooo1?l00002ooooo`03
o`000?oooooooooo00Cooooo0_l000000ooooooo0000ooooo`04ooooo`?o00001?ooool2o`0000Co
iNGU;_ooool002cooooo1_l007l2ooooo`05o`000?ooooooooooooooool000000_ooool6o`0000;o
oooo1Ol0000;ooooo`;o00001?ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo
00?ooooo0ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00007ooooo`?o00001?ooool00ol0
003oooooooooo`0=ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oo
oooooooo00_ooooo1?l000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool03Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo000000;ooooo00Co
0000ooooool0003o00002?ooool01_l0003oooooooooool0003oooooo`0000Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`02o`0000Wooooo1?l00007ooooo`03o`000?oooooo0000
00Oooooo00?o0000ooooool000001?l00006onGUiBgooooo000[ooooo`So001o0_ooool01ol0003o
oooooooooooooooo0000ooooool000000_ooool4o`0000Gooooo1?l00007ooooo`03o`000?oooooo
000000Cooooo00Co0000oooooooooooo00001oooool2o`0000Kooooo0ol00002ooooo`03o`000?oo
oooooooo00?ooooo0_l00005ooooo`;o000000?oooooo`000?l000001?ooool00ol0003ooooooooo
o`0>ooooo`03o`000?oooooo000000;ooooo00Co0000oooooooooooo00002_ooool4o`0000Gooooo
00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo013o
oooo00Ko0000ooooool0003oooooo`000?ooool2o`0000Sooooo00?o0000ooooool000000_ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooo000000?o00001Oooool2o`000003ooooool0003o
oooo00Kooooo00Ko0000oooooooooooo0000ooooool00008onGUiBcooooo000[ooooo`So001o0ooo
ool2o`0000;ooooo0_l000001?oooooo0000ooooooooool4o`0000?ooooo0_l00002ooooo`Co0000
0oooool00ol0003oooooo`000005ooooo`;o00002_ooool00ol0003oooooooooo`06ooooo`?o0000
1?ooool00ol0003oooooooooo`05o`0000?ooooo00?o0000ooooool000001?ooool00ol0003ooooo
ooooo`0>ooooo`;o00000_ooool00ol0003oooooo`000006ooooo`Go00002Oooool00ol0003ooooo
o`000005ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool04?ooool02Ol0003ooooo
o`000?oooooo0000ooooool0003oooooo`000007ooooo`;o00000oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool2o`0000Kooooo00?o0000
ooooool000001?ooool3o`000004ooooool0003oooooooooo`;o00001_ooool01ol0003oooooo`00
0?l0003oooooo`000?ooool02?oUiND/ooooo`00:oooool8o`00O`Gooooo0_l00002ooooo`04o`00
0?ooooooooooooooo`;o00000_ooool3o`0000;ooooo00?o0000ooooooooool00oooool5o`0000Ko
oooo0_l0000;ooooo`;o00001oooool00ol0003oooooo`000003o`0000?ooooo1?l00003ooooo`04
o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool03_ooool01?l0003oooooooooooooool2
o`0000?ooooo1?l0000?ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`0Aooooo`08o`000?oooooo0000o`000?oooooo0000ooooool00007ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00?ooooo0_l00006ooooo`;o00001oooool3o`0000;ooooo00?o0000oooooooo
ool01Oooool2o`000003ooooool0003o000000;ooooo2?oUiND/ooooo`00:oooool8o`00O`Kooooo
00Go0000ooooool0003oooooo`000002ooooo`03o`000?oooooo000000?ooooo1Ol00006ooooo`03
o`000?oooooo000000?o00001?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool00_ooool00ol0003oooooo`000002o`0000Cooooo0_l000000ooooooo
0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool03?ooool00ol0003o
ooooo`000002ooooo`Go00004_ooool2o`0000Kooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Aooooo`07o`000?oooooo0000ooooool0003oooooo`000006ooooo`;o00001?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool2
o`0000Cooooo0_l00007ooooo`04o`000?ooooooooooooooo`Co00001Oooool2o`000003ooooool0
003ooooo00;ooooo00?o0000onGUiOoUiND01_oUiND/ooooo`00;?ooool6o`00O`Kooooo00?o0000
ooooool000000oooool3o`000003ooooool0003ooooo00;ooooo00Co0000oooooooooooooooo0ol0
0003ooooo`04o`000?ooooooooooo`0000?ooooo1Ol0000>ooooo`;o00000oooool01?l0003ooooo
ooooooooool3o`0000?ooooo0ol00003ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooool000001?l0000Eooooo`03o`000?oooooo000000Ko
oooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Aooooo`05o`000?oooooo0000oooo
ool000000_ooool00ol0003oooooooooo`03ooooo`;o00001Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000002oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0_l000000ooo
oooo0000ooooo`03ooooo`;o00001_oUiND]ooooo`00;Oooool4o`00O`Oooooo0_l000000ooooooo
0000ooooo`03ooooo`;o000000Goooooo`000?ooooooooooo`000005ooooo`Co00000_ooool00ol0
003oooooooooo`05ooooo`Go00003Oooool00ol0003oooooo`000002ooooo`?o00002?ooool3o`00
0003ooooool0003o000000?ooooo00Go0000ooooooooooooooooo`00000;ooooo`Co00000_ooool0
0ol0003oooooo`00000Dooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`0Booooo`04o`000?oooooo0000o`0000;ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;o00002?ooool00ol0003ooooo
ooooo`04ooooo`04o`000?oooooo0000ooooo`Co000000?oooooo`000?ooool01?ooool2o`000003
oooooooUiNGoiNGU00;oiNGU;_ooool003?ooooo1?l007l2o`0000?ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool00000
0ol00007ooooo`03o`000?oooooo000000;ooooo0ol0000;ooooo`Go00003Oooool5o`000003oooo
ool0003ooooo00?ooooo00?o0000ooooooooool00oooool6o`0000Cooooo00Co0000ooooool0003o
00004oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool04_ooool00ol0003oooooooooo`0Booooo`?o00000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool01_ooool00ol0003oooooo`000002o`0000Oooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`02o`000004ooooool0003o0000o`0000;ooooo1?ooo`00
0ol0003oooooooooo`0booooo`00<_ooool6o`00O`Gooooo00Co0000oooooooooooooooo0_l00000
1?oooooo0000ooooool00007ooooo`04o`000?ooooooooooooooo`?o00001Oooool01?l0003ooooo
ooooool00004ooooo`Co00001_ooool5o`0000kooooo00?o0000ooooooooool01Ol00003ooooo`03
o`000?oooooo000000?o00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooooooooooo
ool000004oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`0Booooo`?o00000_ooool01?l0003oooooooooool00004
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooo
oooo000000Wooooo0ol00002ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_oo
ool01?l0003oooooo`000?l00002ooooo`Co00001_ooo`0dooooo`00<Oooool8o`00O`Gooooo00Go
0000oooooooooooo0000ooooo`02o`000003ooooool0003ooooo00Gooooo00Go0000ooooool0003o
ooooo`000003ooooo`?o00000_ooool01?l0003oooooooooool00008ooooo`Oo00000_ooool3o`00
00gooooo00?o0000ooooooooool00oooool7o`0000Oooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool00oooool01Ol0
003ooooooooooooooooo000001Gooooo00?o0000ooooooooool04oooool3o`0000;ooooo0_l00005
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooo
oooo000000Sooooo0_l000001Ooooooo0000oooooooooooo000000Cooooo00?o0000ooooooooool0
1_ooool00ol0003oooooo`000002ooooo`?o00000_ooool8oooo03?ooooo000aooooo`So001o1_oo
ool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_l0
0006ooooo`?o00000oooool00ol0003oooooooooo`06ooooo`So00000oooool00ol0003ooooooooo
o`0:ooooo`04o`000?ooooooooooooooo`Wo00001_ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00?ooooo0_l0000@ooooo`03o`000?oooooooooo00Cooooo00Go0000oooooooooooo
ooooo`00000Fooooo`03o`000?oooooooooo01;ooooo0ol00002ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`00
00Sooooo00Co0000ooooool0003o00001?ooool01?l0003oooooooooool00009ooooo`;o00000ooo
ool00ol0003oooooo`000002ooooo`Soool0<oooool0037ooooo2?l007l7ooooo`;o00000oooool2
o`0000Gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`06o`000?oooooo
0000o`000?oooooo00001_ooool2o`0000Cooooo00?o0000ooooool000001?l000000ooooooo0000
o`000009ooooo`Go00001?ooool00ol0003oooooooooo`04o`0000Cooooo00Go0000oooooooooooo
ooooo`000006ooooo`03o`000?oooooo000000kooooo00?o0000ooooooooool01_ooool01?l0003o
ooooooooool0000Fooooo`03o`000?oooooooooo01;ooooo1?l000000ooooooo0000ooooo`05oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooo`000008oooo
o`04o`000?ooooooooooo`0000Kooooo0_l00009ooooo`;o00000oooool3o`0000?ooooo2?ooo`0c
ooooo`00<Oooool8o`00O`Sooooo00Co0000oooooooooooooooo1?l00003ooooo`07o`000?oooooo
ooooooooool0003oooooo`000006ooooo`04o`000?ooooooooooooooo`?o00000_ooool3o`0000Ko
oooo0_l00005ooooo`Co00001?ooool4o`000003ooooool0003ooooo00Oooooo00Ko0000oooooooo
ooooooooo`000?ooool4o`0000?ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`00000=
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooool000006?ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooo000000?o00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooool000001_ooool2o`0000;ooooo00?o0000ooooool000001?ooool0
0ol0003oooooo`000008ooooo`;o00000oooool01?l0003oooooooooool00002ooooo`03o`000?oo
o`3oool000Koool0<oooool003;ooooo1_l007l9ooooo`;o00000_ooool2o`0000;ooooo00?o0000
ooooool000001?ooool00ol0003oooooo`000006ooooo`03o`000?oooooooooo00?ooooo1?l00007
ooooo`;o00002_ooool4o`0000Cooooo00?o0000ooooooooool02?ooool01?l0003ooooooooooooo
ool2o`000003ooooool0003ooooo00?o00002?ooool01?l0003oooooooooool0000;ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooool000006?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool01?ooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00?ooooo0_l00006ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool2o`000003ooooool0003ooooo
00;o00000_ooool2o`000004ooooool0003oooooo`0000Koool0=?ooool003?ooooo1?l007l2oooo
o`CoOgmo1Oooool2o`000004ooooool0003oooooo`0000;ooooo00?o0000ooooooooool00oooool0
1?l0003oooooooooool00004ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`?o00001?oo
ool00ol0003oooooooooo`04ooooo`Go00000_ooool00ol0003oooooo`000004o`0000[ooooo00Ko
0000ooooooooooooooooo`000?ooool2o`0000;ooooo00?o0000ooooool000000_l00005ooooo`05
o`000?ooooooooooooooool000002Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo0000
01Sooooo00?o0000ooooooooool04_ooool3o`000003ooooool0003o000000Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`03ooooo`;o00001?ooool2o`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`00
0003ooooo`;o00001?ooool00ol0003oiNGUonGUi@02onGUi@;ooooo1?ooo`0eooooo`00>?ooool6
ogmoO`Cooooo00Ko0000ooooool0003o0000ooooool00002ooooo`;o00000oooool00ol0003ooooo
ooooo`02ooooo`05o`000?ooooooooooooooool000000_ooool2o`0000?ooooo00?o0000oooooooo
ool00oooool2o`000003ooooool0003ooooo00;ooooo1?l00008ooooo`03o`000?oooooooooo00;o
oooo1?l00008ooooo`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo0ol00002oooo
o`05o`000?ooooooooooooooool000002?ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool04Oooool2o`000004ooooool0003o0000o`0000Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool00oooool01ol0003ooooooooooooooooo0000ooooool000001_ooool01?l0003o
ooooo`000?l00003ooooo`;o00001Oooool6onGUiC[ooooo000gooooo`SoOgmo0oooool02?l0003o
ooooo`000?l0003oooooo`000?oooooo00000_ooool2o`000003ooooool0003ooooo00Cooooo00?o
0000ooooooooool00ol00006ooooo`03o`000?oooooooooo00Cooooo1Ol0000=ooooo`;o00000_oo
ool01?l0003oooooooooooooool3o`0000Gooooo00?o0000ooooooooool00_l00002ooooo`04o`00
0?oooooo0000o`0000Kooooo0ol00004ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo017ooooo0_l000001?oooooo
0000ooooool00005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool01Ol0
003ooooooooooooooooo000000Gooooo00?o0000ooooooooool00_ooool2o`0000Sooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`03ooooo`SoiNGU>Oooool003Oooooo2?moOgl3oooo
o`07o`000?oooooo0000ooooool0003oooooo`000005ooooo`;o00001_ooool3o`0000Sooooo00?o
0000ooooooooool01Ol00002ooooo`?o00003_ooool00ol0003oooooo`000006ooooo`Co00000_oo
ool2o`0000;ooooo0_l00004ooooo`;o00001_ooool4o`0000;ooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool04?oo
ool3o`0000;ooooo0ol00004ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooooooo
0_l000000ooooooo0000o`000005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
1Oooool2o`000003ooooool0003ooooo00;ooooo0_l00005ooooo`SoiNGU>Oooool003Oooooo2?mo
Ogl01_l0003oooooooooool0003oooooo`0000;ooooo0_l00005ooooo`;o00000oooool3o`000003
ooooool0003ooooo00Oooooo1?l00002ooooo`03o`000?oooooo000000Gooooo0ol0000<ooooo`;o
00002_ooool4o`0000;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04
o`000?ooooooooooooooo`Co00000_ooool00ol0003oooooooooo`0=ooooo`;o00006oooool00ol0
003oooooooooo`0>ooooo`04o`000?oooooo0000o`0000?ooooo0_l00005ooooo`03o`000?oooooo
oooo00?ooooo00Go0000oooooooooooo0000ooooo`02o`0000Kooooo00?o0000ooooooooool00_oo
ool3o`0000Kooooo00?o0000ooooooooool00_ooool4o`0000Gooooo00?o0000onGUiOoUiND01_oU
iNDiooooo`00=oooool8ogmoO`06ooooool0003o0000o`000?oooooo00000_ooool3o`0000Cooooo
00?o0000ooooool000000_l00004ooooo`;o00000_ooool5o`0000;ooooo00Ko0000oooooooooooo
0000ooooool00009ooooo`?o00002Oooool00ol0003oooooo`00000:ooooo`03o`000?oooooo0000
00Co00001oooool2o`0000?ooooo00?o0000ooooooooool01?ooool3o`00013ooooo00?o0000oooo
ool000006_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00;o00000oooool3o`00
00Cooooo00?o0000ooooooooool00oooool01_l0003oooooooooool0003oooooo`0000Kooooo00?o
0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool0
0oooool2o`000003ooooool0003ooooo00?ooooo0_l00008onGUiCWooooo000hooooo`KoOgmo1?oo
ool3o`000004ooooool0003oooooooooo`;o00000oooool3o`000003ooooool0003ooooo00?ooooo
1?l00008ooooo`;o00000_ooool00ol0003oooooooooo`0:ooooo`;o00001oooool00ol0003ooooo
ooooo`02o`0000Sooooo00Co0000ooooool0003o00000_ooool5o`0000Gooooo0_l000000ooooooo
0000ooooo`06ooooo`03o`000?oooooo000000;o00003Oooool00ol0003oooooo`00000Jooooo`03
o`000?oooooooooo00cooooo00Co0000oooooooooooooooo0_l00004ooooo`;o00001?ooool00ol0
003oooooooooo`04ooooo`Go00001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool00_l00005ooooo`05o`000?oooooo0000ooooool000000oooool3
o`000003oooooooUiNGoiNGU00CoiNGU>_ooool003Wooooo1?moOgl5ooooo`03o`000?oooooo0000
00;o00000oooool4o`0000Cooooo1_l00002ooooo`03o`000?oooooooooo00Kooooo00Co0000oooo
oooooooo00003oooool3o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00;o00000oooool01?l0003oooooooooooooool4o`0000?ooooo0_l0
0006ooooo`03o`000?oooooo000000?ooooo1?l00008ooooo`04o`000?ooooooooooo`0001_ooooo
00?o0000ooooooooool02_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo
0_l00003ooooo`03o`000?oooooooooo00Cooooo0_l00002ooooo`03o`000?oooooooooo00?ooooo
00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooo000000Kooooo00Go0000oooooooo
ooooooooo`000003ooooo`03o`000?oooooo000000?ooooo1?oUiNDkooooo`00@Oooool4ogmoO`03
o`000?oooooo000000[o00000_ooool3o`0000Cooooo00?o0000ooooooooool01?ooool01Ol0003o
ooooo`000?oooooo000001;ooooo0ol000000ooooooo0000ooooo`04ooooo`;o00000oooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Oooooo1ol00003ooooo`04o`000?oo
ooooooooo`0000Oooooo0ol00005ooooo`04o`000?ooooooooooo`0001_ooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`03ooooo`;o00001?ooool2o`0000?ooooo00?o0000oooooooo
ool00oooool2o`0000?ooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo0000
00Kooooo0_l00006ooooo`03o`000?oooooooooo00;ooooo0ol00004ool004;ooooo0010ooooo`Ko
Ogmo00Coooooo`000?l0003ooooo1?l00008ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`02o`0001Kooooo0ol00006ooooo`03o`000?oooooo0000
00;ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Wooooo1?l00000
0ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00Oooooo0ol00002ooooo`05o`000?oooooo
ooooooooool000006oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo0_l0
0004ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`?o00000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooooooool01?ooool2o`0000Gooooo0_l00005ooooo`;o0000
1_oo0011ooooo`00?oooool8ogmoO`?o000000?oooooo`000?l000000_l00007ooooo`;o00001Ooo
ool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
0_l00005ooooo`;o000000?oooooo`000?ooool01?ooool01?l0003oooooooooool0000:ooooo`04
o`000?ooooooooooooooo`Co00000oooool00ol0003oooooooooo`09ooooo`?o00000oooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01Oooool2o`00
00Cooooo0ol00002ooooo`06o`000?ooooooooooooooool0003ooooo0_l00003ooooo`04o`000?oo
ooooooooo`0000?ooooo00?o0000ooooooooool00oooool2o`000003ooooool0003ooooo00?ooooo
00?o0000ooooooooool01?ooool2o`0000Soo`00@?ooool003oooooo2?moOgl5ooooo`03o`000?oo
oooooooo00;o00001?ooool01?l0003oooooooooool00005ooooo`;o00001?ooool2o`0001Kooooo
00?o0000ooooooooool00_ooool3o`000005ooooool0003oooooooooool000001_ooool01?l0003o
ooooooooool0000:ooooo`04o`000?ooooooooooo`0000;ooooo1_l0000<ooooo`03o`000?oooooo
000000?o00006oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo0_l00005
ooooo`?o000000?oooooo`000?ooool00_l000001?oooooo0000ooooool00003ooooo`03o`000?oo
oooo000000?ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo
00?o0000ooooooooool01Oooool00ol0003oooooool00007ool0043ooooo000oooooo`SoOgmo1Ooo
ool01?l0003oooooooooooooool4o`000003ooooool0003ooooo00;ooooo0_l00003ooooo`;o0000
1?ooool00ol0003oooooo`00000Eooooo`03o`000?oooooooooo00Gooooo1?l000000ooooooo0000
o`000005ooooo`04o`000?ooooooooooo`0000Wooooo00Co0000oooooooooooo00001?ooool2o`00
0003ooooool0003o000000;o00002Oooool00ol0003oooooooooo`02ooooo`?o00006_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Oooooo0_l00005ooooo`03o`000?oooooo0000
00?o00000oooool01?l0003oooooooooool00002ooooo`04o`000?oooooo0000ooooo`;o00001?oo
ool2o`0000Cooooo00Co0000oooooooooooo00001oooool2o`000003oooooooo003oo`0000Koo`00
@?ooool003oooooo2?moOgl4ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool00_l0
0005ooooo`03o`000?oooooo000000;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?l000000ooooooo0000
ooooo`03ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000ooooool000001oooool2o`000003
ooooool0003ooooo00Co00001?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000;o
00005oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo0_l00005ooooo`03
o`000?oooooo000000;o00000oooool01Ol0003ooooooooooooooooo000000;ooooo0_l000000ooo
oooo0000ooooo`04ooooo`03o`000?oooooooooo00Gooooo0_l00007ooooo`;o000000?oooooo`00
0?oo00001ooo0010ooooo`00@?ooool6ogmoO`;ooooo1?ooo`000ooooooo0000ooooo`03ooooo`03
o`000?oooooooooo00Co00000oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000?o
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000Co
oooo1?l00003ooooo`04o`000?ooooooooooo`0000Sooooo0_l0000:ooooo`;o00001Oooool5o`00
00Kooooo00?o0000ooooooooool00_ooool3o`0001Cooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`08ooooo`;o00001Oooool00ol0003oooooo`000003o`000003ooooool0003ooooo
00?ooooo00Ko0000oooooooooooo0000ooooool00004ooooo`;o00001oooool2o`0000Kooooo1?oo
0?l00ol0003oooooooooo`06ool0047ooooo0011ooooo`CoOgmo0_ooool6oooo0003ooooool0003o
oooo00?ooooo0_l00004ooooo`?o000000?oooooo`000?l000000_ooool2o`0000Cooooo00?o0000
ooooooooool04Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000Sooooo0_l00002
ooooo`04o`000?ooooooooooo`0000Sooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`06ooooo`Co00000oooool00ol0003oooooooooo`05ooooo`?o00004_ooool01Ol0003ooooooooo
oooooooo000000_ooooo0_l00006ooooo`03o`000?oooooooooo00;o000000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`0000Cooooo00?o0000ooooooooool01_ooool00ol0003ooooo
o`000005ooooo`Koo`3o0oooool4ool004;ooooo0016ooooo`Soool000?o0000ooooooooool00ooo
ool00ol0003oooooooooo`03ooooo`Co000000Goooooo`000?ooooooooooo`000005ooooo`03o`00
0?oooooooooo013ooooo00?o0000ooooooooool00oooool00ol0003oooooo`00000;ooooo`Co0000
0_ooool00ol0003oooooooooo`04ooooo`;o00003_ooool2o`0000Gooooo00?o0000ooooooooool0
0_ooool4o`0000[ooooo1?l0000>ooooo`04o`000?ooooooooooo`0000gooooo00?o0000oooooooo
ool00_ooool3o`0000;ooooo0ol00005ooooo`;o000000?oooooo`000?ooool00oooool00ol0003o
ooooooooo`06ooooo`04o`000?ooooooooooo`0000Cooooo2?oo0?m8ooooo`00A_ooool8oooo0003
o`000?oooooooooo00?ooooo0_l00003ooooo`03o`000?oooooooooo00;ooooo1?l000000ooooooo
0000ooooo`03ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool00_ooool00ol0003o
ooooo`00000=ooooo`Ko00001Oooool00ol0003oooooo`00000>ooooo`04o`000?oooooo0000o`00
00?ooooo00?o0000ooooooooool01_ooool4o`0000[ooooo0ol0000;ooooo`03o`000?oooooo0000
00kooooo0_l00002ooooo`04o`000?ooooooooooo`0000?ooooo0_l00005ooooo`Co00000_ooool2
o`0000Sooooo00Go0000ooooooooooooooooo`000004ooooo`Soo`3oB?ooool004Kooooo2?ooo`00
0ooooooo0000ooooo`03ooooo`05o`000?oooooo0000ooooool000001oooool3o`000003ooooool0
003ooooo00?ooooo00?o0000ooooooooool03oooool01ol0003ooooooooooooooooo0000ooooool0
00004?ooool2o`000004ooooool0003o0000o`0000;ooooo00Co0000oooooooooooo00003oooool0
0ol0003oooooooooo`02o`000003ooooool0003ooooo00Oooooo00Co0000oooooooooooooooo1?l0
0009ooooo`?o00002Oooool00ol0003oooooooooo`0=ooooo`;o000000?oooooo`000?ooool00_oo
ool01?l0003oooooooooooooool2o`0000Gooooo0_l000001?oooooo0000ooooool00008ooooo`;o
00001?ooool01Ol0003ooooooooooooooooo000000Soo`3oB?ooool004Kooooo2?ooo`000ooooooo
0000ooooo`02ooooo`05o`000?ooooooooooooooool000002?ooool00ol0003oooooooooo`03o`00
00?ooooo00?o0000ooooooooool03oooool01_l0003oooooooooool0003oooooo`0001;ooooo0_l0
00000ooooooo0000ooooo`02o`0000?ooooo00?o0000ooooooooool03_ooool01?l0003ooooooooo
ooooool2o`0000Sooooo00?o0000ooooooooool01Oooool4o`0000Sooooo0ol00005ooooo`;o0000
3oooool3o`0000Gooooo00?o0000ooooool000000ol00003ooooo`?o000000?oooooo`000?l00000
2?ooool00ol0003oooooooooo`04ooooo`05o`000?oooooo0000o`000?ooool02?oo0?m8ooooo`00
Aoooool6oooo00?ooooo1?ooo`02ooooo`03o`000?oooooo000000Oooooo00Co0000oooooooooooo
oooo1?l00002ooooo`03o`000?oooooooooo00gooooo00Ko0000oooooooooooo0000ooooool0000D
ooooo`;o000000Coooooo`000?l0003ooooo1?l0000@ooooo`06o`000?ooooooooooooooool0003o
oooo0_l00006ooooo`03o`000?oooooooooo00Wooooo0ol00008ooooo`?o00000_ooool2o`0000oo
oooo0_l00006ooooo`06o`000?oooooo0000o`000?oooooo00000_ooool00ol0003oooooooooo`02
o`000003ooooool0003ooooo00Kooooo00?o0000ooooooooool01?ooool4oooo00;ooooo00?o0000
ool0oooo0?l01?oo0?m9ooooo`00B?ooool4oooo00?ooooo1_ooo`001Ol0003ooooooooooooooooo
000000Gooooo00?o0000ooooooooool00oooool00ol0003oooooo`000002o`000003ooooool0003o
oooo00gooooo00Go0000ooooool0003oooooo`00000Fooooo`?o00001Oooool3o`0000oooooo00Co
0000oooooooooooo00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00_ooooo
1?l00007ooooo`?o000000?oooooo`000?ooool03?ooool3o`0000Kooooo0_l000000ooooooo0000
ooooo`03o`0000;ooooo0_l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1?ooool6oooo0003o`000?oooooooooo00Coo`3oB_ooool004kooooo2?ooo`04ooooo`;o00000ooo
ool00ol0003oooooooooo`04ooooo`?o000000Coooooo`000?l0003o00003Oooool2o`000003oooo
ool0003ooooo01Kooooo0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00?o0000
3Oooool00ol0003oooooo`000004ooooo`;o00001?ooool00ol0003oooooooooo`0?ooooo`Co0000
0oooool00ol0003oooooooooo`03o`0000[ooooo0_l000000ooooooo0000o`000006ooooo`;o0000
00Coooooo`000?oooooooooo0_l000001ooooooo0000oooooooooooo0000ooooool000001Oooool0
0ol0003oooooooooo`04ooooo`Soool0D?ooool004kooooo2?ooo`06ooooo`03o`000?oooooo0000
00Wooooo0_l00002ooooo`?o00002oooool3o`0001Sooooo0_l000001?oooooo0000o`000?l00002
ooooo`03o`000?oooooooooo00?ooooo0ol0000:ooooo`;o00001oooool2o`0000;ooooo00?o0000
ooooooooool04oooool4o`0000Cooooo1?l00006ooooo`04o`000?ooooooooooooooo`;o00001_oo
ool2o`000005ooooool0003oooooo`000?ooool00_l00003ooooo`03o`000?oooooo000000?ooooo
0_l00007ooooo`Soool0D?ooool004kooooo2?ooo`000ol0003oooooooooo`04ooooo`;o00002_oo
ool2o`000004ooooool0003oooooooooo`?o00002?ooool2o`0001Sooooo00?o0000ooooool00000
0_ooool01Ol0003oooooo`000?oooooo000000Sooooo0_l00009ooooo`03o`000?oooooooooo00Oo
oooo0ol0000Fooooo`03o`000?oooooooooo00Go00000oooool3o`000003ooooool0003o000000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooo`000002ooooo`Go00000oooool00ol0003o
ooooo`000002ooooo`03o`000?oooooooooo00Oooooo2?ooo`1@ooooo`00C_ooool8oooo0003oooo
ool0003o000000Ko00002oooool2o`000003ooooool0003ooooo00?ooooo0_l00005ooooo`;o0000
6?ooool01ol0003ooooooooooooooooo0000ooooool000000_ooool2o`0000[ooooo0ol00006oooo
o`;o00002_ooool2o`0001Cooooo00?o0000ooooooooool01Oooool4o`0000?ooooo1?l00004oooo
o`03o`000?oooooooooo00?ooooo00Co0000ooooool0003ooooo0ol00002ooooo`09o`000?oooooo
ooooooooool0003oooooo`000?oooooo000000Wooooo00?o0000oooo0?ooo`001_ooo`1@ooooo`00
Coooool6oooo00Cooooo1?l007l00ol0003oooooo`000007o`0000Gooooo0_l00006ooooo`?o0000
00?oooooo`000?l000006?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;o
oooo0_l0000<ooooo`?o00000_ooool00ol0003oooooo`00000;ooooo`;o00004oooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00Co00000oooool3o`000003ooooool0003ooooo00?o
oooo00Co0000oooooooooooooooo0ol00004ooooo`05o`000?ooooooooooo`000?ooool00_l00005
ooooo`Coo`0000?o0000ooooooooool01_ooo`1Aooooo`00D?ooool4oooo00Cooooo1_l007l2oooo
o`03o`000?oooooooooo00Cooooo2?l00008ooooo`?o00005oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooo000000;ooooo00Co0000ooooool0003o00003Oooool3o`0000;ooooo00?o0000
ooooooooool02?ooool01?l0003oooooo`000?l0000@ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool00_l00002ooooo`Co00000_ooool3o`0000?ooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`03ooooo`;o00000_ooool2o`0000Gooooo1_oo0003ooooo`Coool0D_oo
ool005Oooooo2?l007l2ooooo`03o`000?oooooooooo00[ooooo2_l000001?oooooo0000o`000?l0
000Dooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000000_ooool01Ol0003ooooooooo
oooooooo000000kooooo0ol000000ooooooo0000ooooo`09ooooo`03o`000?oooooooooo00;o0000
3Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo000000Sooooo2?l000000ooooooo0000
o`000002o`000003ooooool0003ooooo00Cooooo00?o0000ooooool000000_l00004ooooo`Soo`00
F?ooool005Oooooo2?l007l3ooooo`03o`000?oooooooooo00[ooooo0_l00004ooooo`_o00004?oo
ool00ol0003oooooooooo`06ooooo`05o`000?oooooo0000ooooool000001?ooool2o`0000_ooooo
00Co0000oooooooooooooooo0ol00009ooooo`03o`000?oooooooooo00;ooooo0_l0000:ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooool4o`000008ooooool0003o0000o`00
0?oooooo0000ooooool00006ooooo`04o`000?oooooo0000o`0000Cooooo2?oo001Hooooo`00Eooo
ool8o`00O`Cooooo00?o0000ooooooooool02Oooool3o`0000;ooooo0_l00003ooooo`03o`000?oo
oooooooo00?ooooo2Ol00007ooooo`03o`000?oooooooooo00Sooooo0_l000000ooooooo0000oooo
o`05ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool4o`0000Kooooo00?o
0000ooooooooool01?ooool2o`0000Oooooo00?o0000ooooooooool02Oooool3o`0000cooooo0_l0
00000ooooooo0000o`000007o`0000Gooooo0_l000000ooooooo0000o`000004ooooo`Soo`00F?oo
ool005Oooooo2?l007l2ooooo`Co07l02oooool01Ol0003oooooo`000?oooooo000000?ooooo00?o
0000ooooooooool01_ooool3o`0000Cooooo2?l0000:ooooo`04o`000?oooooo0000o`0000Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?o0000
0oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0003oooooooooo`09ooooo`04o`00
0?ooooooooooo`0000cooooo1?l00002ooooo`03o`000?oooooo000000Go00000_ooool00ol0003o
ooooooooo`04oooo00?o00002?oo001Hooooo`00F?ooool6o`00O`;ooooo1_l0O`0:ooooo`;o0000
00?oooooo`000?l000000oooool00ol0003oooooooooo`09ooooo`;o00001oooool00ol0003ooooo
ooooo`08o`0000?ooooo0ol00008ooooo`;o00001_ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo0ol000000ooooooo0000ooooo`08ooooo`05o`000?ooooooooooooooool0
00002_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo0_l00002ooooo`03
o`000?oooooooooo00;ooooo1ol000000oooooooool0oooo0004oooo00?ooooo1_oo001Iooooo`00
FOooool4o`00O`;ooooo2?l0O`0:ooooo`;o00000_ooool01?l0003oooooooooool0000=ooooo`?o
00000oooool00ol0003oooooooooo`09ooooo`So00002?ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool2o`0000[ooooo0ol00009ooooo`;o
00001Oooool00ol0003oooooooooo`09ooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`;o
000000Goooooo`000?l0003oooooo`000008oooo00?ooooo1?oo001Jooooo`00Goooool8o`1o0003
o`000?oooooooooo00Oooooo00?o0000ooooool000000_ooool00ol0003oooooo`00000@ooooo`?o
00004?ooool2o`0000;ooooo2?l000000ooooooo0000o`000003ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool01Oooool4o`0000Sooooo0ol00007ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01_ooool4o`000003ooooool0003ooooo00Kooooo0_l00003ooooo`03
o`000?ooo`3oool000Koool0HOooool005oooooo2?l0O`000ooooooo0000ooooo`06ooooo`05o`00
0?oooooo0000ooooool000000_ooool2o`00017ooooo1?l0000>ooooo`?o00002Oooool8o`0000So
oooo00?o0000ooooooooool01Oooool01?l0003oooooooooooooool3o`0000Cooooo00Co0000oooo
oooooooooooo0_l00003ooooo`;o00002Oooool00ol0003oooooooooo`05ooooo`;o000000Gooooo
o`000?l0003oooooo`000007ooooo`;o000000Coooooo`000?l0003ooooo2?ooo`1Qooooo`00Gooo
ool8o`1o00;ooooo00?o0000ogl00?mo00000_mo00001Ooooooo0000oooooooooooo000000;ooooo
00Go0000ooooool0003oooooo`00000?ooooo`03o`000?oooooooooo00;ooooo0_l0000=ooooo`;o
00003Oooool2o`0000;ooooo2?l000000ooooooo0000ooooo`06ooooo`03o`000?oooooooooo00?o
oooo1?l00006ooooo`?o00002oooool00ol0003oooooooooo`04ooooo`;o000000?oooooo`000?oo
ool00_l00007ooooo`Co001o0oooool8oooo067ooooo001Pooooo`Ko07l00oooool6ogl000Cooooo
00?o0000ooooooooool00_l00002ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool3o`0000[ooooo00?o0000ooooool000003Oooool2o`0000Wooooo2?l000000ooooooo0000
ooooo`07ooooo`;o00001_ooool3o`0000_ooooo00Co0000oooooooooooooooo0ol00002ooooo`03
o`000?oooooo000000Oooooo1_l007l3ooooo`Koool0H_ooool0067ooooo1?l0O`03ooooo`SoO`00
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`;o00002?ooool3o`0000cooooo00?o0000ooooooooool00_l0000:ooooo`03
o`000?oooooooooo00;ooooo2_l00002ooooo`?o000000?oooooo`000?l000000oooool00ol0003o
ooooooooo`08ooooo`06o`000?ooooooooooo`000?oooooo00000_ooool01?l0003oooooo`000?l0
0005ooooo`03o`000?l007oo001o00Ko001o0oooool4oooo06?ooooo001Xooooo`SoO`001?ooool0
0ol0003oooooooooo`02o`0000?ooooo0_l00008ooooo`03o`000?oooooooooo00cooooo0ol00005
ooooo`04o`000?oooooo0000o`0000_ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool:o`0000Cooooo0_l0
0008ooooo`?o000000Soooooo`000?ooooooooooo`000?oooooo0000ooooo`;o00000_ooool2o`00
0003ooooool007oo001o00Ko001oJ_ooool006Sooooo2?mo0004ooooo`Co0?l000?oooooo`000?oo
ool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo013ooooo0_l00003ooooo`04
o`000?oooooo0000o`0000_ooooo00?o0000ooooooooool00oooool2o`0000Sooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool02Ol00006ooooo`;o000000?oooooo`000?ooool00_ooool2o`000003oooooooUiNGoiNGU
00;oiNGU0_l00002ooooo`So001oJ_ooool006Sooooo2?mo0003ooooo`Ko0?l000?oooooo`000?oo
ool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01?ooooo1?l00002ooooo`;o
00002Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool00_ooool01Ol0003ooooooooooooooooo000000Gooooo0_l00007ooooo`?o000000?oooooo`00
0?l000002?l00003ooooo`03o`000?oooooo000000KoiNGU0oooool8o`00Of[ooooo001Yooooo`Ko
O`000oooool8o`3o0003ooooool0003ooooo00?ooooo0_l000000ooooooo0000ooooo`0Hooooo`;o
000000?oooooo`000?l000002Oooool00ol0003oooooooooo`07ooooo`;o00001_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool02_ooool2o`00
00;ooooo0_l00002ooooo`So00002?oUiND3ooooo`Ko001oJoooool006[ooooo1?mo0004ooooo`So
0?l00_ooool00ol0003o07l0o`1o0002o`1o0003ooooool0003ooooo01Wooooo00?o0000ooooool0
00000ol00008ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01?ooool01Ol0003o
oooooooooooooooo000000;ooooo00Co0000oooooooooooooooo0_l0000?ooooo`?o000000Cooooo
o`000?l0003ooooo1?l007l3o`0000SoiNGU1?ooool4o`00Ofcooooo001booooo`So0?l00_ooool6
o`1o0003ooooool0003ooooo01Sooooo00Co0000oooooooooooooooo0ol00006ooooo`03o`000?oo
oooooooo00_ooooo0_l00005ooooo`04o`000?ooooooooooooooo`;o00000oooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00?o00001_l007l2ooooo`SoiNGUM?ooool007;ooooo2?l0
o`000ooooooo07l0o`1o0006o`1o0003ooooool0003ooooo01Oooooo00?o0000ooooooooool00_oo
ool5o`0000?ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`02ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`0>ooooo`;o00000oooool00ol0003ooooo
o`00O`07o`00O`03oooooooUiNGoiNGU00KoiNGUM?ooool007?ooooo1_l0o`02ooooo`So07l00_oo
ool2o`0000Coo`004_ooool00ol0003oooooooooo`02ooooo`;o00000oooool2o`000003ooooool0
003ooooo00oooooo00?o0000ooooooooool00_ooool01Ol0003oooooo`000?l0003ooooo00;o0000
4?ooool4o`3o0004ooooool0003oooooooooo`So001o0_ooool6onGUiGGooooo001dooooo`Co0?l0
0oooool8o`1o00?ooooo1_oo000Aooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
0oooool3o`00013ooooo0_l00003ooooo`04o`000?oooooo0000o`00017ooooo1_l0o`03o`0000So
001o0oooool4onGUiGKooooo001kooooo`So07l00_ooool8ool000Cooooo1?oUiND8ooooo`03o`00
0?oooooooooo00?ooooo0_l00004ooooo`03o`000?oooooooooo00;o00004?ooool01Ol0003ooooo
o`000?l0003ooooo00;o00002?ooool4ogl000?ooooo00?o0000o`3o0?l0o`001_l0o`02ooooo`So
001oOOooool007cooooo1_l0O`03ooooo`Soo`000oooool6onGUi@Oooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo0ol0000>ooooo`Co0000
00?oooooo`000?ooool01_ooool6ogl00003ooooool0003ooooo00So0?l00oooool6o`00Ogkooooo
001mooooo`Co07l01?ooool8ool000;o00002?oUiND4ooooo`CoO`001_ooool01?l0003ooooooooo
ool00008ooooo`;o00003?ooool3o`000003ooooool0oooo0?oo00;o0?oo1?ooool8ogl000;ooooo
2?l0o`04ooooo`Co001oOoooool008Gooooo2?oo0002ooooo`SoiNGU0oooool6ogl000Gooooo1?l0
ool6ooooo`Co0?l00ol00003ooooo`Coo`3o00?oooooo`000?l000000_ooool6o`3oo`?ooooo2?mo
0002ooooo`So0?l0Qoooool008Kooooo1_oo0003ooooo`SoiNGU0_ooool8ogl000?ooooo1_l0ool4
ooooo`Ko0?l00_ooool2o`0000Koo`3o0oooool8o`3oo`;ooooo2?mo0003ooooo`Ko0?l0R?ooool0
08Oooooo1?oo0004ooooo`SoiNGU0_ooool8ogl000;ooooo2?l0ool2ooooo`So0?l00_ooool8ool0
o`;ooooo2?l0ool2ooooo`SoO`001?ooool4o`3o08Wooooo002@ooooo`KoiNGU0oooool8ogl000;o
oooo2?l0ool2o`0000So0?l00_ooool8ool0o`;ooooo2?l0ool3ooooo`KoO`00T_ooool0097ooooo
1?oUiND4ooooo`SoO`000_ooool8o`3oo`;ooooo2?l0o`02ooooo`Soo`3o0_ooool8o`3oo`Cooooo
1?mo002Cooooo`00V_ooool6ogl000?ooooo2?l0ool2ooooo`So0?l00_ooool8ool0o`?ooooo1_l0
oonLooooo`00Voooool4ogl000Gooooo1_l0ool4ooooo`Ko0?l01?ooool6ool0o`Gooooo1?l0oonM
ooooo`00YOooool4o`3oo`Kooooo1?l0o`06ooooo`Coo`3oYoooool00?ooooooIOooool00?oooooo
IOooool00?ooooooIOooool00?ooooooIOooool00?ooooooIOooool00?ooooooIOooool00?oooooo
IOooool00?ooooooIOooool00?ooooooIOooool00?ooooooIOooool00?ooooooIOooool00?oooooo
IOooool00001\
\>"],
  ImageRangeCache->{{{0, 355}, {355, 0}} -> {-0.0500054, -0.0500054, \
0.00309862, 0.00309862}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(Do[\[IndentingNewLine]Evolve[
        ran1DTD], {40}\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[ran1DTD]];\)\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.95365 .52854 m
.18884 .83135 L
s
.95365 .52854 m
.05351 .41483 L
s
.95096 .55697 m
.64046 .9323 L
s
.94027 .61304 m
.85023 .78974 L
s
.94027 .61304 m
.08871 .69354 L
s
.94027 .61304 m
.07737 .66733 L
s
.94027 .61304 m
.04635 .52854 L
s
.9323 .64046 m
.04635 .52854 L
s
.9323 .64046 m
.83135 .18884 L
s
.9323 .64046 m
.94649 .41483 L
s
.92263 .66733 m
.61304 .94027 L
s
.92263 .66733 m
.52854 .95365 L
s
.92263 .66733 m
.18884 .83135 L
s
.92263 .66733 m
.04904 .44303 L
s
.92263 .66733 m
.5 .04545 L
s
.88379 .74356 m
.66733 .92263 L
s
.88379 .74356 m
.58517 .94649 L
s
.86773 .76718 m
.76718 .86773 L
s
.86773 .76718 m
.74356 .88379 L
s
.86773 .76718 m
.04635 .52854 L
s
.86773 .76718 m
.38696 .05973 L
s
.86773 .76718 m
.95365 .47146 L
s
.85023 .78974 m
.04904 .44303 L
s
.85023 .78974 m
.95455 .5 L
s
.83135 .81116 m
.52854 .04635 L
s
.81116 .83135 m
.23282 .13227 L
s
.81116 .83135 m
.95455 .5 L
s
.78974 .85023 m
.94027 .38696 L
s
.76718 .86773 m
.66733 .07737 L
s
.74356 .88379 m
.61304 .94027 L
s
.71898 .89832 m
.11621 .74356 L
s
.71898 .89832 m
.08871 .69354 L
s
.71898 .89832 m
.23282 .13227 L
s
.71898 .89832 m
.25644 .11621 L
s
.71898 .89832 m
.55697 .04904 L
s
.71898 .89832 m
.69354 .08871 L
s
.69354 .91129 m
.47146 .04635 L
s
.69354 .91129 m
.61304 .05973 L
s
.66733 .92263 m
.23282 .86773 L
s
.66733 .92263 m
.55697 .04904 L
s
.66733 .92263 m
.58517 .05351 L
s
.64046 .9323 m
.11621 .74356 L
s
.64046 .9323 m
.05973 .61304 L
s
.64046 .9323 m
.33267 .07737 L
s
.64046 .9323 m
.94649 .41483 L
s
.55697 .95096 m
.05973 .61304 L
s
.55697 .95096 m
.10168 .28102 L
s
.55697 .95096 m
.33267 .07737 L
s
.52854 .95365 m
.16865 .81116 L
s
.52854 .95365 m
.95096 .44303 L
s
.5 .95455 m
.47146 .95365 L
s
.47146 .95365 m
.10168 .28102 L
s
.47146 .95365 m
.13227 .23282 L
s
.47146 .95365 m
.74356 .11621 L
s
.44303 .95096 m
.44303 .04904 L
s
.41483 .94649 m
.38696 .94027 L
s
.38696 .94027 m
.10168 .71898 L
s
.35954 .9323 m
.05973 .61304 L
s
.35954 .9323 m
.92263 .33267 L
s
.33267 .92263 m
.05973 .38696 L
s
.33267 .92263 m
.5 .04545 L
s
.30646 .91129 m
.10168 .71898 L
s
.30646 .91129 m
.88379 .25644 L
s
.30646 .91129 m
.91129 .30646 L
s
.25644 .88379 m
.66733 .07737 L
s
.23282 .86773 m
.04635 .47146 L
s
.23282 .86773 m
.44303 .04904 L
s
.23282 .86773 m
.86773 .23282 L
s
.21026 .85023 m
.47146 .04635 L
s
.18884 .83135 m
.11621 .25644 L
s
.18884 .83135 m
.30646 .08871 L
s
.18884 .83135 m
.66733 .07737 L
s
.14977 .78974 m
.58517 .05351 L
s
.14977 .78974 m
.81116 .16865 L
s
.13227 .76718 m
.78974 .14977 L
s
.11621 .74356 m
.05351 .41483 L
s
.11621 .74356 m
.71898 .10168 L
s
.08871 .69354 m
.30646 .08871 L
s
.0677 .64046 m
.0677 .35954 L
s
.0677 .64046 m
.91129 .30646 L
s
.05973 .61304 m
.11621 .25644 L
s
.04635 .52854 m
.41483 .05351 L
s
.04635 .52854 m
.91129 .30646 L
s
.04635 .47146 m
.94649 .41483 L
s
.04904 .44303 m
.10168 .28102 L
s
.05973 .38696 m
.0677 .35954 L
s
.05973 .38696 m
.95365 .47146 L
s
.07737 .33267 m
.21026 .14977 L
s
.08871 .30646 m
.69354 .08871 L
s
.11621 .25644 m
.14977 .21026 L
s
.14977 .21026 m
.94027 .38696 L
s
.16865 .18884 m
.89832 .28102 L
s
.23282 .13227 m
.58517 .05351 L
s
.23282 .13227 m
.83135 .18884 L
s
.25644 .11621 m
.52854 .04635 L
s
.25644 .11621 m
.66733 .07737 L
s
.33267 .07737 m
.35954 .0677 L
s
.35954 .0677 m
.61304 .05973 L
s
.38696 .05973 m
.66733 .07737 L
s
.41483 .05351 m
.94027 .38696 L
s
.5 .04545 m
.61304 .05973 L
s
.5 .04545 m
.89832 .28102 L
s
.64046 .0677 m
.94027 .38696 L
s
.69354 .08871 m
.95365 .47146 L
s
.76718 .13227 m
.78974 .14977 L
s
.81116 .16865 m
.91129 .30646 L
s
.83135 .18884 m
.95096 .44303 L
s
1 0 1 r
.025 w
.95365 .52854 Mdot
0 1 1 r
.95096 .55697 Mdot
.898 .898 .898 r
.94649 .58517 Mdot
0 1 1 r
.94027 .61304 Mdot
.9323 .64046 Mdot
1 0 1 r
.92263 .66733 Mdot
.498 .498 .498 r
.91129 .69354 Mdot
0 1 1 r
.89832 .71898 Mdot
.88379 .74356 Mdot
.86773 .76718 Mdot
.85023 .78974 Mdot
1 0 1 r
.83135 .81116 Mdot
0 1 1 r
.81116 .83135 Mdot
.78974 .85023 Mdot
1 0 1 r
.76718 .86773 Mdot
0 1 1 r
.74356 .88379 Mdot
.71898 .89832 Mdot
1 0 1 r
.69354 .91129 Mdot
0 1 1 r
.66733 .92263 Mdot
.64046 .9323 Mdot
1 0 1 r
.61304 .94027 Mdot
0 1 1 r
.58517 .94649 Mdot
1 0 1 r
.55697 .95096 Mdot
.52854 .95365 Mdot
.5 .95455 Mdot
.47146 .95365 Mdot
0 1 1 r
.44303 .95096 Mdot
.41483 .94649 Mdot
.38696 .94027 Mdot
1 0 1 r
.35954 .9323 Mdot
.33267 .92263 Mdot
0 1 1 r
.30646 .91129 Mdot
0 .498 0 r
.28102 .89832 Mdot
1 0 1 r
.25644 .88379 Mdot
0 1 1 r
.23282 .86773 Mdot
1 0 1 r
.21026 .85023 Mdot
.18884 .83135 Mdot
.16865 .81116 Mdot
0 1 1 r
.14977 .78974 Mdot
.13227 .76718 Mdot
.11621 .74356 Mdot
.10168 .71898 Mdot
.08871 .69354 Mdot
.07737 .66733 Mdot
.0677 .64046 Mdot
1 0 1 r
.05973 .61304 Mdot
.898 .898 .898 r
.05351 .58517 Mdot
.498 0 0 r
.04904 .55697 Mdot
0 1 1 r
.04635 .52854 Mdot
0 .498 0 r
.04545 .5 Mdot
0 1 1 r
.04635 .47146 Mdot
1 0 1 r
.04904 .44303 Mdot
0 1 1 r
.05351 .41483 Mdot
.05973 .38696 Mdot
1 1 0 r
.0677 .35954 Mdot
0 1 0 r
.07737 .33267 Mdot
0 1 1 r
.08871 .30646 Mdot
1 0 1 r
.10168 .28102 Mdot
.11621 .25644 Mdot
.13227 .23282 Mdot
0 1 1 r
.14977 .21026 Mdot
1 0 1 r
.16865 .18884 Mdot
1 1 0 r
.18884 .16865 Mdot
.498 .498 .498 r
.21026 .14977 Mdot
0 1 1 r
.23282 .13227 Mdot
1 0 1 r
.25644 .11621 Mdot
1 1 0 r
.28102 .10168 Mdot
1 0 1 r
.30646 .08871 Mdot
0 1 1 r
.33267 .07737 Mdot
1 0 1 r
.35954 .0677 Mdot
.38696 .05973 Mdot
0 1 1 r
.41483 .05351 Mdot
.44303 .04904 Mdot
1 0 1 r
.47146 .04635 Mdot
.5 .04545 Mdot
.52854 .04635 Mdot
0 1 1 r
.55697 .04904 Mdot
.58517 .05351 Mdot
1 0 1 r
.61304 .05973 Mdot
0 1 1 r
.64046 .0677 Mdot
1 0 1 r
.66733 .07737 Mdot
0 1 1 r
.69354 .08871 Mdot
.71898 .10168 Mdot
1 0 1 r
.74356 .11621 Mdot
0 1 1 r
.76718 .13227 Mdot
.78974 .14977 Mdot
.81116 .16865 Mdot
.83135 .18884 Mdot
1 0 1 r
.85023 .21026 Mdot
0 1 1 r
.86773 .23282 Mdot
.88379 .25644 Mdot
1 0 1 r
.89832 .28102 Mdot
0 1 1 r
.91129 .30646 Mdot
1 0 1 r
.92263 .33267 Mdot
0 0 .498 r
.9323 .35954 Mdot
0 1 1 r
.94027 .38696 Mdot
.94649 .41483 Mdot
1 0 1 r
.95096 .44303 Mdot
0 1 1 r
.95365 .47146 Mdot
.95455 .5 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[19]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00Q_ooool3ool0o`Gooooo0ooo0?l5ooooo`?oo`3oQoooool007kooooo0ol0ool4ooooo`Go
o`3o0oooool5ool0o`?ooooo1Ooo0?l4ooooo`?o0?ooOoooool007Gooooo0ol0ool5ooooo`Go0?oo
0_ooool7ool0o`03oooooooo0?ooo`3o00Goo`3o00?oooooool0oooo0?l01Ooo0?l2ooooo`Go0?oo
1?ooool3o`3oogOooooo001dooooo`Go0?oo0oooool7o`3oo`03oooooooo0?ooo`3o00Goo`3o00?o
ooooool0oooo0?l01Ooo0?l00ol0003oo`3oool0o`05ool0o`03ooooool0oooo0?oo00Go0?oo0_oo
ool5o`3oogKooooo001]ooooo`?oo`3o0oooool7o`3oo`;ooooo1ol0ool00oooooooo`3oool0o`05
ool0o`03oooooooo0?ooo`3o00Goo`3o00?o0000ool0oooo0?l01Ooo0?l00ol0003o0?ooo`3oo`05
o`3oo`03ooooool0oooo0?oo00Go0?oo0oooool3ool0ofoooooo001/ooooo`Goo`3o0_ooool7o`3o
o`;ooooo1ol0ool2ooooo`Goo`3o0oooool5ool0o`03ooooool0003ooooo00Goo`3o0_ooool7o`3o
o`03o`000?l0oooo0?oo00Go0?oo0_ooool5ool0ofkooooo001Vooooo`?oo`3o0_ooool7ool0o`03
ooooool0oooo0?oo00Go0?oo0oooool5o`3oo`Cooooo0ooo0?l4o`000005oooooooo0?ooo`3oool0
ool000000_ooool2o`0000?oo`3o1?ooool5o`3oo`;o00001ol0ool00ol0003oo`3oool0o`05ool0
o`?ooooo0ol0oomWooooo`00IOooool5ool0o`03oooooooo0?ooo`3o00Goo`3o0_l00005o`3oo`03
ooooool0003o000000;ooooo0ol0ool4ooooo`Co00001Oooool01?l0003oooooooooool00004oooo
o`?o00000oooool2o`0000?o0?oo1?l00005o`3oo`;o00001ooo0?l2ooooo`Go0?ooI_ooool006Co
oooo3_oo0?l3ooooo`?o0?oo9Ol00002ooooo`;o00001Oooool3o`3oo`?ooooo1ooo0?l00ooooooo
0?ooo`3oo`05o`3oofGooooo001Nooooo`?o0?oo0oooool7ool0o`03o`000?oo0?ooo`3o00?oo`3o
2?l00004ooooo`Go00000oooool00ol0003oooooooooo`0@o`000003ooooool0003o000000Gooooo
00?o0000ooooooooool00_ooool00ol0003oooooo`000005ooooo`Goo`3o0_ooool7o`3oo`?ooooo
0ooo0?mOooooo`00GOooool5o`3oo`03ooooool0003oo`3o00Koo`3o0_ooool3ool0o`04ooooool0
003oooooo`0000Kooooo1ol00004ooooo`05o`000?ooooooooooooooool000001?ooool4o`000003
ooooool0003ooooo00?ooooo4?l00006ooooo`?oo`3o0oooool7o`3oo`;ooooo1Ooo0?mNooooo`00
G?ooool7o`3oo`;ooooo1Ooo0?l7ooooo`03o`000?oooooo000000;ooooo1?l00003ooooo`05o`00
0?oooooo0000ooooool000000oooool01?l0003oooooooooooooool6o`0000Gooooo00?o0000oooo
ooooool00oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool=o`0000Go0?oo0_ooool7ool0oegooooo001Fooooo`?oo`3o0oooool7o`3oo`?ooooo0ooo0?l9
ooooo`Co00001oooool00ol0003oooooo`000002ooooo`So00000oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooooooo0ol00002ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`?o0?oo0ol0
0007ool0o`;ooooo0ol0oomHooooo`00EOooool5ool0o`;ooooo1ol0ool;ooooo`Co000000?ooooo
o`000?ooool02?ooool00ol0003oooooo`000005o`0000Cooooo00Go0000ooooooooooooooooo`00
0007ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`000003ooooool0
003ooooo00?ooooo00?o0000ooooooooool01?ooool<o`000003oooooooo0?ooo`3o00Goo`3o00?o
ooooo`3oool0ool00ol0oomGooooo`00E?ooool7ool0o`;ooooo1Ol0ool8ooooo`Co00000_ooool0
1?l0003oooooooooool00007ooooo`Go00001?ooool3o`0000?ooooo00Co0000oooooooooooo0000
2?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo3ol000000ooooooo0000
ooooo`09ooooo`03o`000?ooooooo`3o00Coo`3o00?oooooo`3oool0ool01Ol0oomFooooo`00E?oo
ool7ool0o`?ooooo0ol0ool5ooooo`Co00001_ooool01Ol0003ooooooooooooooooo000000;ooooo
1?l000001Ooooooo0000oooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`04o`000?ooooooooooo`0000Wooooo00?o0000ooooool000002ol00003ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool02Oooool2o`0000?oo`3o0_ooool7o`3ooeKooooo001?oooo
o`?oool00_ooool7ool0o`Gooooo00?o0000ooooool000000ol00009ooooo`03o`000?oooooooooo
00Go00001Oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00;ooooo0ol00002
ooooo`03o`000?oooooo000000Wo00000oooool01?l0003oooooooooooooool2o`0000?ooooo0_l0
00000ooooooo0000ooooo`06ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooool000000ooo
ool7o`3oo`?ooooo0ol0oom@ooooo`00C_ooool5oooo00;ooooo1Ooo0?l4ooooo`Co00003?ooool4
o`0000?ooooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00003ooooo`co00002Ooo
ool01Ol0003ooooooooooooooooo000000?ooooo0_l00005ooooo`;o00001oooool00ol0003ooooo
ooooo`09ooooo`?o000000Goooooo`000?l0003o0000ooooo`05o`3oo`?ooooo1Ol0oom?ooooo`00
COooool7oooo00;ooooo0ooo0?l2ooooo`?o00000_ooool2o`0000Oooooo1Ol00008ooooo`03o`00
0?oooooooooo00_o00001oooool2o`0000cooooo00Go0000ooooooooooooooooo`000002ooooo`?o
00001Oooool2o`0000Kooooo00?o0000ooooooooool02Oooool5o`0000Cooooo0ol0ool00ooooooo
0000ooooo`07o`3oodkooooo001=ooooo`Ooool00_ooool5o`0000Kooooo00?o0000ooooooooool0
1Ol00003ooooo`03o`000?oooooooooo00[o00000_ooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01_ooool4o`0000_ooooo00Oo0000oooooooooooo0000ooooool0003o
oooo00;o00001Oooool01?l0003oooooo`000?l00004ooooo`03o`000?oooooooooo00Oooooo0ol0
0003ooooo`03o`000?oooooooooo00Cooooo00Co0000ooooool0003ooooo1ol0oom>ooooo`00B?oo
ool3ool0o`;ooooo1oooo`03o`0000Wooooo3ol00008ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00008ooooo`;o00000_ooool00ol0003oooooooooo`08ooooo`05
o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Co0000oooooooooooooooo0_l00002
ooooo`03o`000?oooooooooo00Gooooo1?l00004ooooo`;o00001Oooool01?l0003oooooooooool0
0007o`3oo`;ooooo0ooo0?m9ooooo`00Aoooool5ool0o`;ooooo1Oooo`02ooooo`;o00000_ooool;
o`0000Oooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool00003ooooo`03o`000?oo
oooo000000[ooooo00Co0000oooooooooooooooo0_l00009ooooo`04o`000?ooooooooooo`0000;o
oooo0_l00005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000001?ooool3o`0000;o
oooo0_l00004ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000005o`3o
o`;ooooo1Ooo0?m8ooooo`00A_ooool7ool0o`;o00000oooo`09o`0000Kooooo0_l00007ooooo`03
o`000?oooooooooo00cooooo0_l00004ooooo`03o`000?oooooo000000Wooooo0_l00005ooooo`03
o`000?oooooooooo00Oooooo0ol00002ooooo`;o00001_ooool00ol0003oooooooooo`03ooooo`;o
000000Coooooo`000?l0003o00001?ooool2o`0000Gooooo00Co0000oooooooooooo00000oooool0
1?l0003oooooooooooooool2o`0000?o0?oo0_ooool7ool0odOooooo0016ooooo`Ooo`3o00?o0000
ooooooooool00_ooool5o`0000_ooooo0_l00005ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`000009ooooo`;o00001_ooool2o`0000Oooooo0_l00000
1_oooooo0000ooooool0003oooooo`0000Gooooo00?o0000ooooooooool00oooool4o`0000Gooooo
00?o0000ooooool000001Oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooo
oooooooo0_l00005ooooo`Ooo`3oAoooool004Kooooo1ooo0?l5o`0000;ooooo00?o0000ooooool0
00002oooool2o`0000Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`02ooooo`;o
00002_ooool2o`0000Sooooo00?o0000ooooooooool01?ooool2o`000006ooooool0003oooooo`00
0?oooooo00001Oooool00ol0003oooooooooo`03o`0000;ooooo00Co0000ooooool0003o00000ooo
ool2o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000002ooooo`04o`000?oo
ooooooooo`0000Cooooo1ooo0?m7ooooo`00@Oooool3o`3oo`?ooooo1Ooo0?l00ol0003ooooooooo
o`05ooooo`03o`000?oooooo000000_ooooo0_l00004ooooo`03o`000?oooooooooo00oooooo0_l0
0003ooooo`;o00002_ooool00ol0003oooooo`000008ooooo`;o00000oooool01ol0003ooooooooo
ool0003oooooo`000?ooool00_l00005ooooo`?o00001Oooool02Ol0003ooooooooooooooooo0000
ooooool0003oooooo`000007ooooo`09o`000?ooooooooooooooool0003oooooo`000?oooooo0000
00?ooooo00?o0000ooooooooool00oooool5ool0o`?ooooo0ol0oom2ooooo`00@?ooool5o`3oo`?o
00000ooo0?l8ooooo`03o`000?oooooo000000cooooo00?o0000ooooool000000_ooool00ol0003o
ooooooooo`0?ooooo`04o`000?ooooooooooo`0000;ooooo0_l0000:ooooo`03o`000?oooooo0000
00[ooooo0_l000002?oooooo0000oooooooooooo0000ooooool0003ooooo0_l00003ooooo`?o0000
1oooool00ol0003oooooooooo`02ooooo`?o00002?ooool00ol0003oooooooooo`02ooooo`?o0000
1Oooool00ol0003oooooooooo`03ooooo`?oo`3o0oooool5o`3ood7ooooo000oooooo`Oo0?oo0_l0
0003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000003Oooool2o`000003ooooool0
003ooooo017ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooo000000cooooo00?o0000ooooooooool00_ooool2o`000003ooooool0003o000000?o0000
0_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00Co0000ooooool0003o
00001oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`02ooooo`Oo0?oo@?ooool003oooooo1ol0ool2ooooo`[o0000
00Coooooo`000?oooooo00003Oooool00ol0003oooooo`00000Cooooo`07o`000?oooooooooooooo
ool0003oooooo`00000:ooooo`04o`000?ooooooooooo`0000cooooo0ol00003ooooo`Go00001?oo
ool00ol0003oooooooooo`05ooooo`07o`000?ooooooooooooooool0003oooooo`000002ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`02ooooo`Oo0?oo@?ooool003oooooo1ol0ool6ooooo`03o`00
0?oooooooooo00?ooooo2_l00007ooooo`03o`000?oooooo000001?ooooo00?o0000ooooooooool0
0_ooool2o`0000[ooooo00Go0000ooooooooooooooooo`00000:ooooo`04o`000?oooooooooooooo
o`Ko000000?oooooo`000?ooool00oooool00ol0003oooooooooo`05ooooo`07o`000?oooooooooo
ooooool0003oooooo`000003ooooo`;o00001?ooool02Ol0003ooooooooooooooooo0000ooooool0
003oooooo`000006ooooo`04o`000?ooooooooooo`0000Gooooo1ol0oom0ooooo`00>oooool3ogmo
O`;ooooo1Ol0ool8ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001_ooool:
o`0001;ooooo00?o0000ooooooooool00oooool2o`0000[ooooo00Go0000ooooooooooooooooo`00
0009ooooo`03o`000?oooooooooo00Go000000Goooooo`000?l0003oooooo`000004ooooo`03o`00
0?oooooooooo00Gooooo00Ko0000oooooooooooo0000ooooool00006ooooo`;o00000_ooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooool000001_oo
ool5o`3oo`03ooooool0003o0?oo00;o0?oo??ooool003[ooooo1OmoOgl2ooooo`?o0?oo0_l00007
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0;ooooo`;o000000?o
ooooo`000?l000002Ol00007ooooo`03o`000?oooooooooo00?ooooo0_l0000:ooooo`05o`000?oo
ooooooooooooool000002Oooool3o`0000Gooooo00Go0000ooooool0003oooooo`000004ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`06ooooo`05
o`000?oooooo0000ooooool000000oooool01Ol0003ooooooooooooooooo000000Gooooo00?o0000
ooooool000001oooool3o`3oo`;ooooo1Ol0oolkooooo`00>Oooool7ogmoO`Gooooo0_l00007oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool02oooool01Ol0003oooooo`000?oo
oooo000000[ooooo2_l00003ooooo`?o00002Oooool01Ol0003ooooooooooooooooo000000Oooooo
0ol00008ooooo`Co00001Oooool00ol0003oooooooooo`04ooooo`05o`000?oooooo0000ooooool0
00002_ooool3o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00[ooooo1ol0ooljooooo`00>Oooool7ogmoO`Kooooo0_l00007ooooo`03o`00
0?oooooo000000;ooooo00?o0000ooooooooool02oooool01Ol0003oooooo`000?oooooo0000017o
oooo00?o0000ooooooooool02_l00004ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooo
oooooooo0ol000000ooooooo0000ooooo`0:ooooo`?o00001?ooool00ol0003oooooooooo`05oooo
o`04o`000?ooooooooooo`0000_ooooo0ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool00_ooool2o`0000_ooooo1ol0ooljooooo`00>Oooool7ogmoO`Kooooo0_l00007oooo
o`;o00000oooool00ol0003oooooooooo`0:ooooo`07o`000?ooooooooooooooool0003oooooo`00
000?ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooool0003oooooo`000003ooooo`ko0000
0oooool00ol0003oooooooooo`0:ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`;o0000
00?oooooo`000?ooool02_ooool01?l0003oooooo`000?ooool2o`0000;ooooo00?o0000oooooooo
ool00oooool01?l0003oooooooooooooool2o`0000_ooooo1ol0ooljooooo`00=Oooool3oooo00;o
oooo1OmoOgl8ooooo`;o00001oooool01Ol0003ooooooooooooooooo000000_ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`00000?ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ool000000_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`co00002_ooool5
o`0000;ooooo00?o0000ooooooooool01?ooool2o`000003ooooool0003ooooo00Wooooo00Co0000
oooooooooooo00000oooool00ol0003oooooo`000006ooooo`03o`000?oooooo000000;ooooo00?o
0000ooooooooool01oooool00ol0003oooooo`3oo`04o`3oo`;ooooo0ol0oolfooooo`00=?ooool5
oooo0003o`000?ooooooOgmo00;oOgmo2Oooool00ol0003oooooo`000006ooooo`05o`000?oooooo
ooooooooool000002_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000oooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Cooooo
00Co0000ooooool0003o00000_ooool00ol0003oooooooooo`03ooooo`04o`000?oooooooooooooo
o`go00000_ooool00ol0003oooooo`000005ooooo`04o`000?oooooo0000o`0000[ooooo00Go0000
ooooooooooooooooo`000004ooooo`;o00001oooool2o`0000?ooooo00?o0000ooooooooool01Ooo
ool01?l0003oooooooooooooool3o`3oo`;ooooo1Ol0ooleooooo`00<oooool7oooo00kooooo00?o
0000ooooool000001?ooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01?ooool01?l0003oooooooooool0000>ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000oooool00ol0003oooooooooo`02ooooo`?o00001Oooool01Ol0003o
oooooooooooooooo000000kooooo2_l00002ooooo`04o`000?ooooooooooo`0000_ooooo00Go0000
ooooooooooooooooo`000005ooooo`?o00001_ooool01Ol0003ooooooooooooooooo000000Kooooo
00?o0000ooooooooool01_ooool7o`3oocCooooo000cooooo`Ooool03oooool00ol0003oooooo`00
0003ooooo`06o`000?ooooooooooo`000?oooooo00002Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooo000000gooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00004oooo
o`Go00001oooool01Ol0003ooooooooooooooooo000000kooooo00Co0000ooooool0003o00000ooo
ool2o`000003ooooool0003o000000So00001_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00?ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooo`000003ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool01oooool7o`3oocCooooo000cooooo`Ooool03ooo
ool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00;o00002Oooool00ol0003ooooooooo
o`06ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool00_ooool01Ol0003ooooo
oooooooooooo000000;ooooo0ol00003ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooo
oooo00003oooool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooool000000_l00002oooo
o`;o00001?ooool:o`000003ooooool0003ooooo00Cooooo00Co0000oooooooooooooooo0_l00002
ooooo`04o`000?ooooooooooo`0000?ooooo00Co0000oooooooooooo00002_ooool7o`3oocCooooo
000dooooo`Goool04Oooool01Ol0003oooooo`000?oooooo000000Cooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`07ooooo`04o`000?ooooooooooo`0000cooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooooooool3o`0000?ooooo00Co0000oooooooooooo00001oooool0
1?l0003oooooooooool0000@ooooo`07o`000?oooooo0000ooooool0003oooooo`000003ooooo`03
o`000?oooooooooo00;o00002?ooool00ol0003oooooooooo`03ooooo`_o00000_ooool3o`0000?o
oooo00Ko0000oooooooooooo0000ooooool0000;ooooo`03o`000?l0oooo0?oo00?o0?oo=Oooool0
02oooooo0ooo0?l3ooooo`?oool04oooool01?l0003oooooo`000?l00004ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool02?ooool01?l0003oooooooooool0000<ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooool000000_l00006ooooo`03o`000?oooooo000000Sooooo0_l0
000Aooooo`07o`000?oooooo0000ooooool0003oooooo`000002ooooo`06o`000?oooooo0000o`00
0?oooooo00001oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooooooo2_l00002ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00ol0
ool00ol0003oooooooooo`03o`3ooc3ooooo000^ooooo`Goo`3o0_ooool00ol0003oooooooooo`0C
ooooo`04o`000?ooooooooooo`0000Cooooo0_l00006ooooo`03o`000?oooooooooo00Wooooo00Co
0000oooooooooooo00002_ooool00ol0003oooooooooo`02ooooo`?o00000_ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooo000000Sooooo00?o0000ooooooooool04?ooool00ol0003ooooo
o`000002ooooo`04o`000?oooooo0000o`0000?ooooo0ol00006ooooo`03o`000?oooooooooo00Oo
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`00
00;ooooo2ol00003ooooo`03o`000?oooooooooo00Gooooo00?o0000o`3oool0ool00ol0ool_oooo
o`00;Oooool7ool0o`03o`000?oooooooooo01Gooooo0_l000000ooooooo0000ooooo`02ooooo`03
o`000?oooooo000000Cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000:oooo
o`03o`000?oooooo000000?o00001?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Oooooo00?o0000ooooooooool04?ooool01?l0003oooooo`000?l00002ooooo`?o00000_ooool0
0ol0003oooooooooo`02o`0000Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooooooo0_l000000ooooooo0000oooo
o`02ooooo`03o`000?oooooooooo00?ooooo2_l000000ooooooo0?ooo`3oo`05o`3oobkooooo000]
ooooo`Ooo`3o00?o0000ooooooooool05_ooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooo000000?ooooo00?o0000ooooooooool02oooool01?l0003oooooooooool00009ooooo`?o0000
0oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Gooooo0_l00007ooooo`;o0000
4_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000oooool3o`00
0003ooooool0003ooooo00[ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Cooooo0ol00002ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01_ooool00ol0003o0?ooo`3oo`05o`3oobkooooo000]ooooo`Ooo`3o00?oooooo`000?l0
00001_l0000?ooooo`;o00000_ooool00ol0003oooooo`000003ooooo`03o`000?oooooo000000ko
oooo00Go0000ooooooooooooooooo`000005ooooo`Co00001Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Gooooo0_l00007ooooo`03o`000?oooooo0000017ooooo00Co0000oooo
oooooooo00000_ooool2o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
000<ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00Sooooo1ol0ool^ooooo`00;_ooool5ool0o`[ooooo2?l00007ooooo`03o`000?oooooo
000000;ooooo0_l00003ooooo`;o00004?ooool01?l0003oooooooooool00003ooooo`;o00000ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooo`000005ooooo`04o`000?ooooooooooo`00017ooooo00?o0000ooooooooool0
0_l000001_oooooo0000o`000?l0003oooooo`0000Cooooo00Co0000ooooool0003ooooo0_l0000:
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`04o`000?ooooooooooo`0000cooooo1Ol0
ool_ooooo`00:Oooool3o`3oo`?ooooo0ooo0?l00ol0003oooooooooo`0@ooooo`So000000?ooooo
o`000?ooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool01?l00005ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`00000Aooooo`03o`000?oooooooooo00?o000000Coooooo`000?oooooo00001Oooool2o`0000Co
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05ooooo`?o000000?oooooo`000?oo
ool03Oooool3o`3oo`;ooooo0ooo0?l[ooooo`00:?ooool5o`3oo`Cooooo00?o0000ooooooooool0
5oooool00ol0003oooooo`000007o`000004ooooool0003oooooo`0000oooooo1Ol00007ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`0@oooo
o`03o`000?oooooo000000;o000000Coooooo`000?oooooo00001Oooool00ol0003oooooooooo`04
ooooo`;o00002?ooool01Ol0003ooooooooooooooooo000000Cooooo00?o0000ooooooooool00_l0
0009ooooo`?o00004Oooool2o`0000Goo`3o:_ooool002Oooooo1ol0ool2ooooo`03o`000?oooooo
oooo01Sooooo00?o0000ooooooooool00_ooool01?l0003oooooo`000?ooool9o`0000Wooooo0ol0
0003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0005ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool00_l000001Ooooooo0000o`00
0?oooooo000000?ooooo0_l00008ooooo`;o00001_ooool01Ol0003ooooooooooooooooo000000Co
oooo00?o0000ooooool000003?ooool3o`00013ooooo00?o0000ool0oooo0?l01Ooo0?lYooooo`00
9oooool7o`3oo`03ooooool0003ooooo01[ooooo00?o0000ooooooooool00_ooool01?l0003ooooo
o`000?ooool2o`0000?ooooo00Co0000oooooooooooooooo2Ol00003ooooo`04o`000?oooooooooo
o`0000Kooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool00oooool2o`0000Gooooo00?o0000ooooooooool03oooool0
0ol0003oooooo`000002o`000003ooooool0003o000000;ooooo00Go0000ooooool0003oooooo`00
000:ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooo000000cooooo00?o0000ooooool000000ol0000>ooooo`03o`000?oo0?ooo`3o00Goo`3o
:Oooool002Oooooo1ol0ool4o`0001Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`02o`0000?ooooo00?o0000ooooooooool01Oooool2o`0000;ooooo2Ol00005ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00?ooooo0_l00005ooooo`03o`000?oooooooooo00oooooo00Go0000ooooool0003o0000
ooooo`02o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo00Wooooo0_l00003ooooo`05o`00
0?ooooooooooooooool000000oooool2o`0000cooooo00Co0000oooooooooooooooo0_l000000ooo
oooo0000o`00000<ooooo`03o`000?oo0?ooo`3o00Goo`3o:Oooool002Sooooo1Ol0ool00ooooooo
0000ooooo`02ooooo`Co00005?ooool00ol0003oooooooooo`04ooooo`?o00000_ooool01Ol0003o
oooooooooooooooo000000?ooooo0ol00008ooooo`04o`000?ooooooooooooooo`So00000_ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000006ooooo`03o`000?oooooooooo00kooooo0_l000000ooooooo0000ooooo`02
o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo00_ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000cooooo00?o0000ooooooooool00oooool2o`0000;ooooo
00?o0000ooooooooool02_ooool00ol0003oo`3oool0o`03ool0ob[ooooo000Tooooo`?oo`3o00?o
ooooo`000?l0ool00_l0ool00ooooooo0000ooooo`07ooooo`Go00003oooool00ol0003ooooooooo
o`04ooooo`;o00001?ooool01?l0003oooooooooooooool3o`0000_ooooo00?o0000ooooooooool0
0_ooool01Ol0003ooooooooooooooooo000000;ooooo2?l00002ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo
00kooooo00Go0000oooooooooooo0000ooooo`03o`0000;ooooo00Go0000ooooool0003oooooo`00
000>ooooo`?o00000oooool01?l0003oooooooooool0000;ooooo`;o00001oooool01?l0003ooooo
ooooooooool2o`0000[ooooo00?o0000oooooooo0?l00_oo0?l2ooooo`?o0?oo9_ooool002?ooooo
1Ooo0?l00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00_ooooo1?l0000:ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooool000001?ooool4o`0000kooooo00Go0000oooooooooooo
ooooo`000003ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool9o`0000Co
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00ko
oooo00Co0000oooooooooooooooo1?l000001Ooooooo0000oooooooooooo000000;ooooo00?o0000
ooooooooool03Oooool01_l0003ooooooooooooooooo0000ooooo`;o00002_ooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00;ooooo0_l00008ooooo`06o`000?oooooo0000ooooool0
003ooooo1Ol0oolUooooo`008_ooool7ool0o`?ooooo00?o0000ooooooooool04?ooool4o`0000Ko
oooo00?o0000ooooooooool01?ooool2o`0000;ooooo1?l000001Ooooooo0000oooooooooooo0000
00gooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000002?oo
ool00ol0003oooooooooo`03ooooo`So00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooooooo00gooooo0_l00003ooooo`03o`000?oooooo000000;o00001?ooool00ol0003oooooo`00
000?ooooo`?o000000Goooooo`000?l0003oooooo`000009ooooo`03o`000?oooooooooo00[ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;o0000
1ol0oolTooooo`008_ooool7ool0o`;ooooo00?o0000ooooooooool05Oooool5o`000003ooooool0
003ooooo00Gooooo0_l000000ooooooo0000o`000005ooooo`03o`000?oooooo000000gooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo2Ol0000=ooooo`04
o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001_ooool2o`00013ooooo00Ko0000
ooooool0003o0000ooooool00009ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1?ooool2o`0000Kooooo00Co0000ooooool0003ooooo1ol0oolTooooo`008_ooool7ool0o`03oooo
ool0003ooooo01cooooo1?l00003ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00008oooo
o`04o`000?ooooooooooooooo`Wo00001Oooool01Ol0003oooooooooool0003ooooo00;o000000?o
ooooo`000?l000001_ooool2o`00013ooooo00?o0000ooooool000000ol00008ooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool01Oooool2o`0000Cooooo00Co0000oooooooooooo0000
1ol0oolTooooo`008oooool5ool0o`;ooooo00?o0000ooooooooool06oooool01?l0003ooooooooo
ooooool5o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooo`00000=ooooo`07o`00
0?ooooooooooooooool0003oooooo`000003ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool00oooool01?l0003oooooooooool00008ooooo`05o`000?ooooooooooooooool000002?oo
ool9o`000005ooooool0003o0000o`000?ooool00_l00006ooooo`03o`000?oooooooooo00kooooo
0_l000000ooooooo0000ooooo`02o`0000Kooooo00?o0000ooooooooool03oooool00ol0003ooooo
ooooo`06ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooo`3oo`04o`3oobGo
oooo000Tooooo`?oo`3o0_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo00Co0000
0_ooool5o`000003ooooool0003ooooo00Cooooo00?o0000ooooool000003?ooool00ol0003ooooo
ooooo`02ooooo`;o00000oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00002_ooool01?l0003oooooooooool0000<ooooo`04o`000?oooooooooo
ooooo`So00001_ooool2o`0000oooooo00?o0000ooooooooool00_l00002ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool04?ooool2o`0000Sooooo0_l000000ooooooo0000ooooo`02
ooooo`;o00000ol0oolVooooo`008?ooool3ool0o`04o`000?ooooooooooooooo`;o00007_ooool3
o`0000?ooooo00Co0000oooooooooooo00001?ooool5o`0000?ooooo00?o0000ooooool000002ooo
ool00ol0003oooooooooo`02ooooo`;o00000oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00;ooooo00Co0000oooooooooooo00002oooool01?l0003oooooooooool0000:ooooo`03
o`000?oooooooooo00;ooooo0ol000000ooooooo0000ooooo`0:o`0000kooooo0_l00002ooooo`;o
00000oooool2o`000003ooooool0003ooooo01?ooooo0_l00009ooooo`;o00000oooool01?l0003o
ooooooooool00003ooooo`?o0?oo8Oooool001oooooo1Ooo0?l3ooooo`;o00006oooool4o`0000Co
oooo00Go0000ooooooooooooooooo`000006ooooo`03o`000?oooooooooo00Ko00002oooool00ol0
003oooooooooo`02ooooo`05o`000?ooooooooooooooool000002_ooool00ol0003oooooooooo`02
ooooo`03o`000?oooooo000000cooooo00Co0000oooooooooooo00002Oooool00ol0003ooooooooo
o`03ooooo`;o00000_ooool01Ol0003oooooo`000?oooooo000000Gooooo00?o0000ooooool00000
1ol00005ooooo`03o`000?oooooo000000;ooooo0_l00005ooooo`03o`000?oooooooooo01Cooooo
0_l0000:ooooo`?o000000Koooooo`000?ooooooooooo`000?ooool5o`3oob3ooooo000Nooooo`Oo
o`3o0_ooool2o`0001Wooooo0_l00002ooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo1Ol00009ooooo`04
o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooool000003Oooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo
00?ooooo00Wo0000ooooooooooooooooo`000?oooooo0000ooooool000001Oooool01?l0003ooooo
ooooool00006ooooo`So00000_ooool00ol0003oooooooooo`03ooooo`04o`000?oooooo0000o`00
01Cooooo00?o0000ooooool000002Oooool3o`000005ooooool0003oooooooooool000001ol0oolO
ooooo`007_ooool7ool0o`04ooooool0003oooooo`0001Kooooo0ol00004ooooo`05o`000?oooooo
ooooooooool000001Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo0_l0
0002ooooo`Co00001Oooool01?l0003oooooooooooooool2o`0000;ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`02ooooo`;o00003oooool2o`0000Sooooo00?o0000ooooooooool0
0oooool2o`0000?ooooo00Ko0000oooooooooooo0000ooooool00004ooooo`04o`000?oooooooooo
o`0000Wooooo00Co0000oooooooooooooooo2Ol00004ooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooool000002?ooool00ol0003oooooooooo`03o`0000;ooooo1ol0oolOooooo`007_ooool7
ool0o`05o`000?ooooooooooooooool000004_ooool3o`0000Oooooo00Co0000oooooooooooo0000
1Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Cooooo0_l00005ooooo`Ko0000
0oooool2o`000003ooooool0003ooooo00[ooooo00Co0000oooooooooooooooo0_l0000@ooooo`03
o`000?oooooo000000Kooooo00?o0000ooooooooool00oooool2o`000003ooooool0003ooooo00;o
oooo00Go0000ooooool0003oooooo`000005ooooo`04o`000?ooooooooooo`0000Kooooo0_l00005
ooooo`03o`000?oooooo000000?ooooo00?o0000ooooool000001ol0000Booooo`03o`000?oooooo
000000Kooooo0_l00004ooooo`;o000000?oooooo`3oool0ool01Ol0oolOooooo`007oooool5ool0
o`03ooooool0003ooooo00;ooooo00?o0000ooooooooool03Oooool3o`0000Wooooo00Go0000oooo
ooooooooooooo`000005ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool2
o`0000Wooooo1?l00002ooooo`;o00002oooool01?l0003oooooooooooooool2o`00017ooooo0_l0
0006ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`09o`000?ooooooooooooooool0003o
ooooo`000?oooooo000000Gooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`07ooooo`Wo00002_ooool0
1?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000Cooooo0_l000000ooooooo0?oo
o`3oo`03o`3oob3ooooo000Pooooo`?oo`3o00?oooooo`000?ooool00oooool00ol0003ooooooooo
o`0:ooooo`?o00003?ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo000000Co0000
2oooool01Ol0003ooooooooooooooooo000001?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooo000000;ooooo00Oo0000ooooooooooooooooo`000?oooooo
000000;ooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o
0000ooooooooool01_ooool2o`000003ooooool0003o000000cooooo00?o0000ooooooooool01?oo
ool8o`0000?ooooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000Co
oooo00Co0000ooooool0003o00000ol0oolQooooo`007?ooool3ool0o`05ooooool0003o0000oooo
ool000001_ooool00ol0003oooooooooo`07ooooo`;o00003oooool00ol0003oooooo`000007oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooo000000;ooooo1ol00006ooooo`05o`000?ooooooooooooooool000004oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000000oooool0
1ol0003ooooooooooooooooo0000ooooool000000_ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooool000002Oooool2o`000003ooooool0003ooooo00gooooo
0_l0000<ooooo`Wo00001?ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo0000ooooo`;o
00000_ooool3ool0oagooooo000Kooooo`Goo`3o00Co0000ooooool0003o00001_ooool00ol0003o
ooooooooo`04ooooo`?o00004?ooool00ol0003oooooo`000008ooooo`03o`000?oooooooooo00[o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooo000000;o
oooo0_l00005ooooo`Co00000_ooool00ol0003oooooooooo`02o`0001Cooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooooooo00So
oooo1?l0000@ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool02Ol00003ooooo`03
o`000?oooooooooo00;ooooo00?o0000ool0oooo0?l00ooo0?lLooooo`006_ooool7ool0o`03oooo
ool0003ooooo00Oooooo00?o0000ooooooooool00ol0000Cooooo`03o`000?oooooo000000Sooooo
00?o0000ooooooooool02_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Kooooo
0_l00003ooooo`;o00002Oooool6o`0001Gooooo0_l000000ooooooo0000ooooo`02ooooo`04o`00
0?ooooooooooo`0000Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooo000000Wooooo00Co0000oooooooooooo0000
4Oooool2o`0000kooooo0_l00002ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
1ol00007ool0oa_ooooo000Jooooo`Ooo`3o0_l00008ooooo`?o00005_ooool2o`0000Wooooo00?o
0000ooooooooool02oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Kooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo000000Co0000
4_ooool2o`000003ooooool0003ooooo00;ooooo00Co0000oooooooooooo00001?ooool00ol0003o
ooooooooo`02ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00002?ooool2o`0000;ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0;
ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo0000
0oooool7ool0oa_ooooo000Jooooo`Ooo`3o00?oooooo`000?ooool01?ooool3o`000003ooooool0
003ooooo01Gooooo0_l00009ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool01?oo
ool2o`0000Oooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`08ooooo`;o0000
00?oooooo`000?ooool00_ooool5o`0000gooooo00?o0000ooooool000000oooool01?l0003ooooo
ooooool00006ooooo`04o`000?ooooooooooooooo`;o00001?ooool01?l0003oooooooooool00002
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`000002
ooooo`03o`000?oooooooooo01;ooooo0_l0000:ooooo`03o`000?oooooo000000?ooooo00Oo0000
ooooooooooooooooo`000?oooooo000000?ooooo00Co0000oooooooooooooooo1ooo0?lKooooo`00
6oooool5ool0o`03o`000?oooooo000000?ooooo0_l00004ooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00Gooooo
0_l00005ooooo`;o00000_ooool01?l0003oooooooooool0000:ooooo`;o000000?oooooo`000?oo
ool01oooool4o`0000[ooooo00Go0000ooooool0003oooooo`000003ooooo`03o`000?oooooooooo
00Cooooo00Co0000oooooooooooooooo0_l00004ooooo`06o`000?ooooooooooo`000?oooooo0000
1_ooool00ol0003oooooooooo`02ooooo`07o`000?oooooo0000ooooool0003oooooo`00000Foooo
o`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
oooo00;o00001Oooool01?l0003oooooooooooooool5ool0oacooooo000Looooo`?oo`3o0_l00000
0ooooooo0000o`000002o`0000Oooooo00?o0000ooooooooool04_ooool2o`0000[ooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo000000Cooooo0_l00000
0ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00Oooooo00Co0000oooooooooooo00003?oo
ool5o`0000Gooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`05ooooo`07o`00
0?ooooooooooooooool0003oooooo`000004ooooo`04o`000?oooooo0000o`0000Oooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?oooooo0000ooooool000005oooool2o`0000Gooooo
00?o0000ooooooooool00oooool01_l0003ooooooooooooooooo0000ooooo`;o00001Oooool00ol0
003oooooooooo`02ooooo`?oo`3o7Oooool001cooooo00?o0000ooooool000001?l0000:ooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0?ooooo`03
o`000?oooooooooo00Cooooo0_l00004ooooo`04o`000?oooooo0000o`0000Cooooo00?o0000oooo
ooooool01_ooool2o`0000;ooooo00?o0000ooooooooool03oooool6o`000003ooooool0003o0000
00?ooooo00?o0000ooooooooool01Oooool01ol0003ooooooooooooooooo0000ooooool000001?oo
ool01?l0003oooooo`000?l00008ooooo`03o`000?oooooo000000?ooooo00Go0000ooooool0003o
ooooo`00000Iooooo`;o00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o
oooo0ol00006ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool06oooool001Sooooo
0ol0ool00ooooooo0000o`000003o`000003ooooool0003ooooo00[ooooo00?o0000ooooooooool0
4?ooool2o`0000_ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`05ooooo`;o0000
0oooool01?l0003oooooo`000?l00004ooooo`03o`000?oooooooooo00Kooooo0_l000000ooooooo
0000ooooo`0Dooooo`Oo000000?oooooo`000?ooool01oooool00ol0003oooooooooo`02ooooo`;o
00001?ooool2o`000003ooooool0003ooooo00Sooooo00Go0000ooooooooooooooooo`000002oooo
o`03o`000?oooooo000001_ooooo0_l00008ooooo`04o`000?ooooooooooooooo`?o00001_ooool0
0ol0003oooooooooo`04ooooo`04o`000?l0oooo0?ooo`3ooaWooooo000Gooooo`Go0?oo0_l00000
1_oooooo0000o`000?l0003oooooo`0000[ooooo00?o0000ooooooooool03oooool2o`0000cooooo
00?o0000ooooooooool04?ooool00ol0003oooooooooo`04ooooo`05o`000?oooooo0000ooooool0
00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooool0
003oooooo`00000Eooooo`04o`000?oooooo0000o`0000;ooooo1?l00007ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`;o000000?oooooo`000?ooool01oooool2o`0000?ooooo00Ko0000
oooooooooooo0000ooooool0000Kooooo`?o00002?ooool00ol0003oooooo`000002ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool00oooool5o`3ooaSooooo000Fooooo`Oo0?oo0_oo
ool01Ol0003oooooo`000?oooooo000000_ooooo00?o0000ooooooooool03Oooool00ol0003ooooo
o`00000;ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool01?ooool01?l0003ooooo
o`000?l00002ooooo`;o00001Oooool00ol0003oooooooooo`04ooooo`05o`000?oooooo0000oooo
ool000005?ooool01Ol0003oooooo`000?oooooo000000?ooooo00?o0000ooooooooool01?l00003
ooooo`03o`000?oooooooooo00;ooooo0_l00003ooooo`05o`000?oooooo0000ooooool000001ooo
ool01?l0003oooooooooool00002ooooo`06o`000?ooooooooooo`000?oooooo00006_ooool01Ol0
003ooooooooooooooooo000000Oooooo0_l00003ooooo`;o00001_ooool00ol0003oooooooooo`02
ooooo`Oo0?oo5oooool001Kooooo1ol0ool2ooooo`03o`000?oooooo000000;o00002oooool00ol0
003oooooooooo`0<ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo0_l00006ooooo`03o`000?oo
oooooooo00;ooooo00Ko0000oooooooooooo0000ooooool0000Cooooo`03o`000?oooooo000000;o
oooo00Co0000oooooooooooooooo0_l00005ooooo`Go00000oooool2o`0000;ooooo00Co0000oooo
oooooooooooo0_l00006ooooo`03o`000?oooooooooo00;ooooo0_l00003ooooo`03o`000?oooooo
000001Wooooo00?o0000ooooooooool00oooool2o`0000Gooooo0_l00004ooooo`;o00001_ooool4
o`0000Oo0?oo5oooool001Kooooo1ol0ool2ooooo`03o`000?oooooooooo00;o00003?ooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool01Oooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooool000004oooool2o`0000?ooooo00Co0000
oooooooooooo00000_ooool00ol0003oooooooooo`06ooooo`Ko000000?oooooo`000?ooool00ooo
ool2o`0000Gooooo00?o0000ooooooooool00oooool2o`0000?ooooo00?o0000ooooool000006?oo
ool00ol0003oooooooooo`06ooooo`05o`000?ooooooooooooooool000000_ooool01ol0003ooooo
oooooooooooo0000ooooool000000_ooool6o`000003ooooool0oooo0?oo00Go0?oo5oooool001Oo
oooo1Ol0ool2ooooo`04o`000?ooooooooooooooo`;o00003?ooool00ol0003oooooooooo`0:oooo
o`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`05
ooooo`06o`000?oooooo0000o`000?oooooo00001_ooool01Ol0003ooooooooooooooooo000000?o
oooo0_l0000Cooooo`;o00001Oooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool00_ooool5o`0000;ooooo0_l00004ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00_ooool2o`0001Oooooo00?o0000ooooooooool02?ooool3o`0000?o
oooo00Co0000oooooooooooooooo1Ol000000ooooooo0000o`000002o`0000;ooooo00?o0000o`3o
ool0ool00ol0oolHooooo`006?ooool3o`3oo`?ooooo00?o0000ooooooooool00_ooool2o`0000co
oooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000cooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool01?ooool00ol0003oooooo`000004ooooo`;o00004oooool00ol0003oooooooooo`04
ooooo`;o00001?ooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooooooo`;o00001?oo
ool4o`0000?ooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000003ooooo`;o00005_oo
ool00ol0003oooooooooo`0;ooooo`;o00000oooool4o`0000?ooooo0_l00003ooooo`03o`000?oo
oooo000000;ooooo0ol0oolIooooo`005Oooool3o`3o0003ooooool0003ooooo00?ooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`00000;ooooo`03o`000?oooooooooo00Oooooo00Go0000
ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool00ooo
ool2o`000005ooooool0003o0000ooooool000001_ooool00ol0003oooooo`000004ooooo`;o0000
4_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1?ooool01_l0003oooooooooool0003oooooo`0000Oooooo1_l00006ooooo`03o`000?oooooo0000
00?ooooo0_l0000Eooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01Ol000000ooo
oooo0000o`000002o`000003ooooool0003ooooo00;ooooo0_l00006ooooo`?oo`3o5_ooool001Co
oooo1Ol0o`000ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo0ol0000<ooooo`03
o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo
01Gooooo00?o0000ooooooooool00oooool3o`0000;ooooo0_l00006ooooo`;o00001Oooool2o`00
017ooooo00?o0000ooooooooool01_ooool2o`0000Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooo`000002ooooo`03o`000?oooooooooo00Gooooo0_l00004ooooo`Co00000_ooool01?l0
003oooooooooool00002ooooo`;o00005?ooool00ol0003oooooooooo`0;ooooo`Co000000Cooooo
o`000?l0003ooooo1?l00004ooooo`03o`000?oooooooooo00;o00001Oooool5ool0oaGooooo000C
ooooo`Oo0?l01?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000_ooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool05oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000006ooooo`;o00001Oooool2o`00013ooooo0_l00007ooooo`03o`000?oooooo000000Kooooo
00?o0000ooooooooool00oooool2o`0000?ooooo0_l00006ooooo`;o00002?ooool5o`000003oooo
ool0003ooooo00;o00004oooool00ol0003oooooooooo`08ooooo`Co00001_ooool4o`000003oooo
ool0003ooooo00Cooooo0_l000000ooooooo0000ooooo`03ooooo`Ooo`3o5?ooool001?ooooo1ol0
o`04ooooo`03o`000?oooooooooo00?ooooo0_l000000ooooooo0000ooooo`09ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0;ooooo`03o`000?oo
oooooooo01Sooooo00Co0000oooooooooooo00000_ooool2o`0000;ooooo0_l00005ooooo`;o0000
1Oooool00ol0003oooooooooo`0>ooooo`;o00002?ooool00ol0003oooooo`000007ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooo000000Wooooo00Co0000oooooooooooooooo1Ol0000Booooo`03o`000?oooooooooo00Go
oooo1?l00002ooooo`03o`000?oooooooooo00?ooooo0_l00002ooooo`Co00001Oooool2o`0000;o
oooo00Co0000oooooooooooooooo1ooo0?lDooooo`004oooool7o`3o00?ooooo00?o0000oooooooo
ool01?ooool01?l0003oooooo`000?l0000;ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01Sooooo00So0000oooo
oooooooo0000ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`000005ooooo`03o`000?oooooooooo00gooooo0_l00008ooooo`04o`000?ooooooooooo`0000Oo
oooo00Co0000oooooooooooooooo0_l00005ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00_l00008ooooo`03o`000?oooooooooo00?ooooo1_l0000>ooooo`03o`000?oooooooooo
00;ooooo1?l00005ooooo`04o`000?ooooooooooooooo`?o00001_ooool2o`0000Cooooo00Ko0000
oooooooooooo0000ooooool00003ooooo`Ooo`3o5?ooool001Cooooo1Ol0o`04ooooo`03o`000?oo
oooooooo00Cooooo00Go0000ooooool0003oooooo`00000:ooooo`05o`000?ooooooooooooooool0
00001_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo01Wooooo00Go0000ooooool0
003oooooo`000002ooooo`03o`000?oooooooooo00;o00000oooool01?l0003oooooooooool00004
ooooo`03o`000?oooooooooo00gooooo0_l00008ooooo`05o`000?ooooooooooooooool000001ooo
ool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooooooo
o`000003ooooo`;o00001oooool00ol0003oooooooooo`05ooooo`;o00000oooool5o`0000Sooooo
00?o0000ooooool000000ol00008ooooo`03o`000?oooooooooo00;o00002_ooool2o`0000;ooooo
00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooooooo`Goo`3o5Oooool001Go
oooo0ol0o`05ooooo`03o`000?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool0000:
ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`0Jooooo`04o`000?ooooooooooo`0000;ooooo00So0000oooooooooooo0000ooooool0003ooooo
o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<ooooo`;o00002?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`00
0004ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooool000001_ooool00ol0003ooooo
ooooo`05ooooo`;o00002?ooool4o`0000;ooooo1?l0000;ooooo`Co00003Oooool3o`0000Gooooo
00?o0000ooooool000000oooool01?l0003oo`3oool0oooo0?lFooooo`005oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooo000000Kooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooo000000Cooooo00Go0000oooooooooooo0000ooooo`02o`00
00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0;ooooo`;o00002Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Gooooo0_l00003ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooool000001Oooool00ol0003oooooo`000006ooooo`03o`000?oooooooooo
00Kooooo00?o0000ooooooooool02Oooool6o`0000[ooooo0ol0000@ooooo`Co00001Oooool01Ol0
003oooooo`000?oooooo000001[ooooo000Booooo`?oool00_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000[ooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01_ooooo0ol0
0004ooooo`05o`000?ooooooooooo`000?ooool00_l00004ooooo`03o`000?oooooooooo00;o0000
3?ooool00ol0003oooooo`000008ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
1?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo0_l00006ooooo`03
o`000?oooooo000000Gooooo00?o0000ooooooooool01oooool2o`0000Kooooo1?l00002ooooo`04
o`000?ooooooooooooooo`Co00001?ooool2o`000003ooooool0003ooooo013ooooo0_l00002oooo
o`;o00001?ooool3o`0000Gooooo0ol007lCooooo`004Oooool5oooo0003o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00Kooooo0_l00006ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool06oooool3o`00
00Gooooo00Ko0000ooooool0003o0000ooooool00004ooooo`03o`000?oooooooooo00;o00002ooo
ool00ol0003oooooo`000009ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo0_l00006ooooo`04o`000?oo
ooooooooo`0000Cooooo00?o0000ooooooooool01oooool2o`0000;ooooo1?l00005ooooo`03o`00
0?oooooooooo00Kooooo1Ol000000ooooooo0000ooooo`0@ooooo`;o000000?oooooo`000?ooool0
0_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool5o`00Oa;ooooo000@ooooo`Ooool01_oo
ool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`0Jooooo`04o`000?oooooo0000o`0000Kooooo0ol00002ooooo`03o`000?oooooooooo00;o
oooo00Co0000ooooool0003o00002_ooool01?l0003oooooooooool00008ooooo`03o`000?oooooo
oooo00Gooooo00Go0000ooooooooooooooooo`000005ooooo`03o`000?oooooo000000?ooooo0ol0
0006ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool01oooool4o`0000Sooooo
00?o0000ooooooooool01_ooool2o`0000Cooooo1?l0000>ooooo`03o`000?oooooooooo00;o0000
1?ooool2o`000004ooooool0003oooooo`0000?ooooo1ol007lAooooo`004?ooool7oooo00Gooooo
00?o0000ooooooooool01_ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000oooooooo
ool01?ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool06_ooool01Ol0003oooooo`000?oooooo000000Gooooo0ol00003ooooo`06o`000?oooooo
ooooooooool0003ooooo0_l00009ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000oooooooo
ool01Oooool01?l0003oooooooooool00007ooooo`;o00000_ooool01?l0003oooooo`000?l00006
ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`04ooooo`Go00000_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo0ol00005ooooo`03o`000?oo
oooooooo00;ooooo1Ol00008ooooo`07o`000?ooooooooooooooool0003oooooo`000005ooooo`05
o`000?ooooooooooooooool000000_ooool7o`00Oa7ooooo000@ooooo`Ooool01Oooool00ol0003o
ooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001?ooool00ol0
003oooooooooo`02ooooo`05o`000?ooooooooooooooool000003oooool00ol0003oooooooooo`0J
ooooo`05o`000?oooooo0000ooooool000001_ooool2o`0000Cooooo00Go0000oooooooooooo0000
ooooo`02o`0000Sooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooo000000Wooooo00?o0000ooooool000000oooool00ol0003oooooooooo`05ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01?l00004ooooo`05o`000?oooooooooo
ooooool000001Oooool00ol0003oooooooooo`03ooooo`;o00001oooool00ol0003oooooooooo`08
ooooo`Co00000oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooool0003o0000ooooo`02o`000003ooooool007oo001o00Go001o4Oooool0017ooooo1Ooo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooo
oooooooo000000;ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000oo
oooo00?o0000ooooooooool06_ooool2o`0000?ooooo00?o0000ooooooooool00_ooool3o`0000Go
oooo00?o0000ooooooooool00ol00007ooooo`05o`000?ooooooooooooooool000002Oooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo0_l00004ooooo`;o00001_ooool01Ol0
003oooooooooool0003ooooo00Co00002?ooool2o`0000;ooooo00?o0000ooooooooool00_ooool0
1?l0003oooooooooooooool3o`0000Sooooo00?o0000ooooooooool03Oooool5o`0000?ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo0000o`000?oo
ool01Ol007lBooooo`004_ooool3oooo0003o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`03ooooo`06o`000?ooooooooooo`000?oooooo00001oooool00ol0
003oooooo`00000@ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooool000001?ooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooool0003oooooo`00
0006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003oooooooooo`07ooooo`;o0000
2_ooool2o`0000Cooooo0_l00006ooooo`Ko00002oooool2o`0000?ooooo00Co0000oooooooooooo
00000_ooool2o`0000[ooooo00?o0000ooooooooool03oooool01?l0003oooooooooooooool5o`00
00;ooooo0_l00006ooooo`;o000000?oooooo`000?ooool00ol007lCooooo`004_ooool2o`000003
ooooool0003ooooo00Gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00;o00001oooool00ol0003oooooo`00000?ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooool000001?ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo
00?ooooo00Go0000ooooool0003oooooo`000005ooooo`05o`000?ooooooooooooooool000002_oo
ool00ol0003oooooooooo`05ooooo`;o000000?oooooo`000?ooool01oooool2o`000003ooooool0
003o000000?ooooo0_l00004ooooo`?o00000oooool2o`0000_ooooo0_l00004ooooo`Go00002ooo
ool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Kooooo1Ol00007ooooo`Co00005_oo
ool0013ooooo0ol0ool2o`0000Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00;o00002?ooool2o`0000oooooo00?o0000ooooooooool06_ooool2
o`0000Kooooo0_l00004ooooo`03o`000?oooooooooo00Cooooo1?l00004ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000001oooool01ol0003ooooooooooooooooo0000ooooool000000_ooool6o`0000;o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`;o00001?ooool2o`0000ko
oooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`07ooooo`04o`000?oooooo0000oooo
o`Go00001?ooool01Ol0003oooooo`000?l0003ooooo00?o0?oo4Oooool000oooooo1Ol0ool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool2o`00
0003ooooool0003ooooo00Oooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooo000000Oooooo00?o0000ooooooooool00_ooool2o`0000Kooooo0ol00003oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooo000000Co00001_ooool01?l0003oooooooooooooool2o`0000cooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`0<ooooo`03o`000?oooooooooo00kooooo00?o0000
ooooooooool02?ooool2o`0000?ooooo00Co0000oooooooooooooooo1Ol00002ooooo`03o`000?l0
oooo0?oo00?o0?oo4?ooool000kooooo1ol0ool6ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool01Ol0003ooooooooooooooooo000000Oooooo0_l0000?ooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooool000001_ooool2o`0000Cooooo0_l00006ooooo`?o00000_oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_oo
ool5o`000003ooooool0003o000000Kooooo00Co0000oooooooooooooooo0_l0000<ooooo`;o0000
00?oooooo`000?ooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00kooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo
0ol00007o`3oo`oooooo000>ooooo`Oo0?oo0_l00004ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool01?ooool2o`0000?ooooo00?o0000ooooooooool01?ooool2o`0000oooooo00?o
0000ooooooooool06Oooool2o`0000Kooooo00Co0000oooooooooooo00001?ooool2o`0000Cooooo
00Ko0000oooooooooooo0000ooooool00005ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`05ooooo`06o`000?ooooooooooooooool0003ooooo1?l0
00000ooooooo0000ooooo`02ooooo`;o000000?oooooo`000?ooool01?ooool01ol0003ooooooooo
oooooooo0000ooooool000002?ooool4o`000003ooooool0003ooooo00?ooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooool0
00001Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0?oo00Ko0?oo3oooool000kooooo
1ol0ool2ooooo`_o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool00oooool01?l0003oooooooooool0000>ooooo`03o`000?oooooooooo01Wo
oooo0_l00006ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooool000000oooool00ol0
003oooooooooo`02o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooool000000ol00005ooooo`04o`000?oooooooooo
ooooo`;o000000?oooooo`000?ooool01?ooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Cooooo0_l00003ooooo`;o00001_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00kooooo00?o0000ooooooooool02_ooool00ol0003oooooo`000005ooooo`03o`000?oo
oooooooo00Gooooo00?o0000o`3oool0ool01Ol0ool?ooooo`003oooool5o`3oo`Oooooo00?o0000
ooooooooool01?ooool;o`0000;ooooo00Go0000ooooooooooooooooo`000004ooooo`04o`000?oo
ooooooooo`0000gooooo00?o0000ooooooooool06Oooool00ol0003oooooo`000005ooooo`03o`00
0?oooooooooo00;ooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o0000oooo
ooooool00_l00005ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`05ooooo`Go00002Oooool01ol0003ooooooooooooooooo0000ooooool000001Ooo
ool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00?o00001Oooool2o`0000Oooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00[ooooo
00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0?oo00Co
0?oo4?ooool0013ooooo0ol0ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Sooooo
00Go0000ooooooooooooooooo`000002ooooo`_o00000oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo01Wooooo0_l00006ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooo
oooo00000_ooool01?l0003oooooooooool00002ooooo`?o00000oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool4o`0000?ooooo00?o0000
ooooooooool02?ooool01?l0003oooooooooooooool3o`0000Gooooo00Co0000oooooooooooo0000
0oooool3o`0000Oooooo0ol00007ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0:ooooo`05o`000?ooooooooooooooool000001oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooo0?oo00;o0?oo4Oooool001;ooooo0_l00006ooooo`03o`00
0?oooooooooo00Wooooo00Co0000oooooooooooo00001_ooool01Ol0003ooooooooooooooooo0000
00;ooooo2ol00007ooooo`03o`000?oooooooooo01Wooooo0_l00005ooooo`03o`000?oooooooooo
00Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000;ooooo00?o0000ooooool000000_l0
00001?oooooo0000ooooool0000;ooooo`05o`000?ooooooooooooooool000000_ooool3o`0000Ko
oooo0_l0000:ooooo`03o`000?oooooooooo00;ooooo0_l00006ooooo`04o`000?oooooo0000oooo
o`?o00002?ooool01?l0003oooooo`000?l00008ooooo`05o`000?ooooooooooooooool000004?oo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooo000001?ooooo000>ooooo`?o0?oo00?ooooo
o`000?l000001_ooool00ol0003oooooooooo`09ooooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00?ooooo2ol0
000Hooooo`;o00001?ooool00ol0003oooooooooo`06ooooo`04o`000?ooooooooooo`0000;ooooo
0_l000001ooooooo0000ooooool0003oooooo`000?ooool00_l0000:ooooo`04o`000?oooooooooo
ooooo`Co00002?ooool00ol0003oooooo`00000:ooooo`03o`000?oooooooooo00;ooooo0ol00005
ooooo`Co00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o00002Oooool0
0ol0003oooooo`00000@ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo0_l000001?oooooo0?ooo`3oool0
ool?ooooo`003Oooool5o`3oo`03ooooool0003o000000Gooooo00?o0000ooooooooool02_ooool2
o`0000Sooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Wooooo00Co0000oooooooooooooooo2_l0000=ooooo`;o00001Oooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooo000000?ooooo0_l00002ooooo`04o`000?oooooo0000ooooo`;o0000
2_ooool4o`0000_ooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooool000001?ooool3o`0000Gooooo00?o0000ooooooooool00oooool01?l0003o
ooooooooooooool2o`0000Wooooo00?o0000ooooool000003oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooo0?oo00Co0?oo3_ooool000cooooo1ol0ool2o`0000Gooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool000001_ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00_ooooo2ol00002ooooo`;o00001?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooo000000?ooooo00Ko0000ooooooooooooooooo`000?oo
ool2o`000003ooooool0003ooooo00Gooooo1?l00002ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000000_ooool2o`000003ooooool0003o000000Kooooo00Go0000oooooooooooooooo
o`000004ooooo`;o00002_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00[ooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo
00?o0000o`3oool0ool01Ol0ool=ooooo`003?ooool7o`3oo`03o`000?oooooo000000Cooooo00?o
0000ooooooooool02Oooool2o`0000[ooooo00Co0000oooooooooooooooo0_l00006ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool05_ooool;o`0000Wooooo00Co0000ooooool0003o
oooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooool000000ol00003oooo
o`03o`000?oooooo000000_ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooool000000_l00003ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00001Oooool2o`0000Wooooo00?o0000ooooool000003Oooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01_ooool6
o`0000Oo0?oo3Oooool000cooooo1ol0ool9o`0000[ooooo0_l0000;ooooo`03o`000?oooooo0000
00;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool00_ooool01?l0003oooooooooooooool=o`000003ooooool0003o
000000;ooooo00?o0000ooooool000001Ol00006ooooo`03o`000?oooooo000000_ooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`04o`000?ooooooooooooooo`?o000000?o
ooooo`000?ooool00_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000Kooooo0_l0
0008ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool00ol0003ooooo
ooooo`03oooooa7o00000_ooool00ol0003oooooo`3oo`06o`3oo`gooooo000=ooooo`Go0?oo00Go
ooooo`000?ooooooooooo`000003ooooo`03o`000?oooooo000000Oo000000Coooooo`000?oooooo
00002oooool00ol0003oooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`0Hooooo`;o00000oooool00ol0003oooooooooo`0<ooooo`co
00001?ooool00ol0003oooooooooo`03ooooo`;o00002oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Oooooo1Ol00002ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`02ooooo`03
o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo4?l00006ooooo`03o`000?oooooooooo
00Oooooo00Co0000oooooooooooo00000_ooool5o`3oo`kooooo000>ooooo`?o0?oo0_ooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00Sooooo2?l00008ooooo`;o00000oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool06?ooool2
o`0000?ooooo00?o0000ooooooooool03?ooool2o`000003ooooool0003o000000Go000000?ooooo
o`000?l000002_l0000:ooooo`;o00002Oooool00ol0003oooooooooo`06ooooo`;o00000oooool0
1ol0003oooooooooool0003oooooo`000?ooool00_l00007ooooo`;o00001oooool00ol0003ooooo
o`000005ooooo`03o`000?oooooooooo00;ooooo4?l00008ooooo`03o`000?oooooooooo00_ooooo
00?o0000ooooooooool02?ooool00ol0003oooooo`000002ooooo`04o`000?l0oooo0?ooo`3oo`oo
oooo000?ooooo`;o00000_ooool01ol0003ooooooooooooooooo0000ooooool000002Oooool01Ol0
003ooooooooooooooooo000000Cooooo2Ol00005ooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0Hooooo`05o`000?ooooooooooooooool000003_oo
ool00ol0003oooooo`000002o`0000?ooooo0_l00003ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooool000003?l0000;ooooo`03o`000?oooooooooo00?ooooo0ol000000ooooooo0000oooo
o`02ooooo`04o`000?ooooooooooo`0000;ooooo0ol00006ooooo`04o`000?ooooooooooo`0000;o
oooo4Ol00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Wooooo00Co0000oooooooooooooooo0ol0000@oooo
o`003Oooool3ool0o`04o`000?ooooooooooo`0000?ooooo00?o0000ooooool000002?ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00[ooooo2?l00004ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool06?ooool01Ol0003ooooooooooooooooo000000cooooo1?l00000
0ooooooo0000o`000004ooooo`05o`000?ooooooooooooooool000001Oooool3o`0000[ooooo3?l0
0002ooooo`03o`000?oooooo000000;o00001?ooool01Ol0003ooooooooooooooooo000000;ooooo
00?o0000ooooool000003ol00004ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool0
1oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`08ooooo`06o`000?ooooooooooo`000?oooooo00000ooo0?l>oooo
o`003?ooool5ool0o`;ooooo00?o0000ooooooooool00_ooool2o`0000Sooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`09ooooo`;o00001Oooool9o`0000Cooooo00?o0000oooooooo
ool05oooool2o`0000;ooooo00?o0000ooooooooool01oooool5o`0000Cooooo0_l00003ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`?o00002Oooool00ol0003oooooooooo`0:oooooaCo
000000Goooooo`000?l0003oooooo`000003ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00Sooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool02?ooool2
o`000003ooooool0003ooooo00Goo`3o3Oooool000_ooooo1ooo0?l00ooooooo0000ooooo`03oooo
o`;o00001oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00Co0000oooo
oooooooo00001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00So00005_ooool0
1Ol0003ooooooooooooooooo000000Gooooo1?l00003ooooo`03o`000?oooooooooo00?ooooo0_l0
0002ooooo`04o`000?oooooo0000o`0000?ooooo00Ko0000ooooooooooooooooo`000?ooool2o`00
00Sooooo00?o0000ooooool000003ol00007ooooo`_o000000?oooooo`000?l000000_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000000_ooool00ol0
003oooooooooo`09ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool04?ooool0
0ol0003oooooooooo`07ooooo`05o`000?ooooooooooo`000?ooool01ooo0?l<ooooo`002oooool7
ool0o`03ooooool0003ooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`Wo00003Ooo
ool01?l0003oooooooooool00002ooooo`Co00001_ooool00ol0003oooooooooo`04ooooo`?o0000
00Koooooo`000?oooooo0000ooooool00003oooooa7o00002Oooool2o`0000?ooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooo`000?ooool02ol00002
ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooool0003oooooo`00000=ooooo`;o00003Ooo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Oooooo00Go0000oooooooooooo0000
ooooo`07ool0o`cooooo000;ooooo`Ooo`3o0ol00003ooooo`;o00001Oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003ooooooooo
o`0;ooooo`So00001Oooool00ol0003oooooo`000003o`0000Wooooo00?o0000ooooooooool00_oo
oolAo`0000;ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Oooooo
0ol00006ooooo`03o`000?oooooooooo00Kooooo00Go0000ooooooooooooooooo`000002ooooo`;o
00000oooool2o`0000Kooooo3?l0000>ooooo`;o00003?ooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo00Kooooo00Go0000ooooool0003oooooo`000007ool0o`cooooo000<ooooo`Go
o`3o2?l000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00_ooool01Ol0003ooooooooooooooooo000001Gooooo2?l000000ooooooo0000o`00000>o`00
00?ooooo1?l00002ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000ooooooooooooooooo`00
0004ooooo`04o`000?ooooooooooo`0000Sooooo0_l00009ooooo`03o`000?oooooooooo00Kooooo
00Go0000ooooooooooooooooo`000002ooooo`;o00000oooool2o`0000Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooo`00000:o`0000Cooooo00Co0000oooooooooooo00002_ooool00ol0
003oooooooooo`0Cooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000000oooool00ol0
003oo`3oool0o`03ool0o`gooooo000=ooooo`?oo`3o00Goooooo`000?oooooo0000ooooo`02o`00
0003ooooool0003o000000?o000000?oooooo`000?ooool02?ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo
000000?ooooo00?o0000ooooooooool03?oooolGo`0000?ooooo00?o0000ooooooooool01_ooool0
1ol0003oooooo`000?oooooo0000ooooool000000_ooool01ol0003oooooo`000?oooooo0000oooo
ool000001Oooool00ol0003oooooo`000006ooooo`?o00002oooool00ol0003oooooooooo`06oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000;ooooo0ol00009ooooo`03o`000?oooooo
oooo00;ooooo0_l000000ooooooo0000ooooo`08ooooo`_o00001_ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000000oooool2o`0000?oo`3o3_ooool0
017ooooo00?o0000ooooool000000oooool2o`0000;ooooo00?o0000ooooool000000ol00007oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00;ooooo4_l00003ooooo`?o00001Oooool00ol0003oooooo`000009oooo
o`So00001?ooool3o`0000;ooooo0_l00003ooooo`06o`000?ooooooooooo`000?oooooo00001_oo
ool00ol0003oooooooooo`03ooooo`;o00003_ooool00ol0003oooooooooo`06ooooo`05o`000?oo
ooooooooooooool000000_ooool2o`000006ooooool0003oooooo`000?oooooo00002Oooool01Ol0
003ooooooooooooooooo000000;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00_o00004_ooool00ol0003oooooooooo`05ooooo`;o00001?ooool2
o`00017ooooo000<ooooo`?o0?oo0_ooool00ol0003oooooo`000004ooooo`Go00001?ooool3o`00
00Cooooo00?o0000ooooooooool01_ooool01?l0003oooooooooooooool@o`000003ooooool0003o
oooo00gooooo1?l00007ooooo`?o00002oooool00ol0003oooooooooo`04ooooo`So000000?ooooo
o`000?l000000_ooool2o`0000;ooooo0_l00006ooooo`04o`000?oooooo0000ooooo`?o00004Ooo
ool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000000_ooool3o`0000;ooooo
00Co0000oooooooooooo00002?ooool00ol0003oooooooooo`02o`0000;ooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool01?ooool:o`0000Sooooo00?o0000ooooooooool01?ooool00ol0003oooooo`000003ooooo`04
o`000?oooooo0000o`0000?o0?oo3Oooool000_ooooo1Ol0ool2ooooo`;o00001?ooool00ol0003o
ooooooooo`03o`0000Kooooo4Ol00005ooooo`03o`000?oooooooooo00Cooooo00Ko0000oooooooo
oooo0000ooooool0000:ooooo`Co00002oooool2o`0000_ooooo00?o0000ooooooooool02?ooool2
o`000003ooooool0003ooooo00So00000_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00;o00005?ooool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000000_oo
ool2o`0000?ooooo00Go0000ooooooooooooooooo`000007ooooo`05o`000?oooooo0000ooooool0
00000_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0?ooooo`_o00001?ooool2o`0000?ooooo00Co0000oooooooo
oooo00001Ol0ool<ooooo`002_ooool7o`3oo`04ooooool0003o0000oooooa3o00000oooool4o`00
00Cooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`04o`000?oooooooooo
ooooo`;o00001_ooool4o`0000oooooo0_l0000:ooooo`03o`000?oooooooooo00Wooooo00Go0000
ooooooooooooooooo`000002ooooo`;o00001?ooool;o`0000;ooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`05ooooo`06o`000?ooooooooooooooool0003ooooo0ol00003ooooo`05
o`000?ooooooooooooooool000002?ooool2o`000005ooooool0003oooooooooool000002?ooool0
0ol0003oooooooooo`08ooooo`04o`000?ooooooooooo`0001_ooooo00?o0000ooooool000002_l0
00000ooooooo0?ooo`3oo`05o`3oo`_ooooo000:ooooo`Oo0?oo1?l00002ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00ol00006ooooo`03o`000?oooooooooo00Go00002_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o00000_ooool4o`0001?ooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0:ooooo`05o`000?ooooooooooooooool00000
0_ooool2o`0000Cooooo0ol00003ooooo`[o00004Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00;ooooo0_l000000ooooooo0000ooooo`03ooooo`05o`000?ooooooooooooooool0
00001oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool02_ooool2o`0001cooooo00?o0000ooooooooool00_ooool2o`0000?ooooo00Co0000oooo
oooooooo00001ol0ool;ooooo`002_ooool7o`3oo`03ooooool0003o000000?ooooo00?o0000oooo
ool000000oooool00ol0003oooooooooo`03ooooo`;o00001?ooool00ol0003oooooooooo`04oooo
o`Co00001oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo000000?o00005_ooool2o`00
00Wooooo00?o0000ooooooooool02_ooool2o`0000?ooooo00Ko0000ooooool0003o0000ooooool0
0003ooooo`;o000000Coooooo`000?l0003o00002?ooool00ol0003oooooo`000007o`0000[ooooo
00?o0000ooooooooool01Oooool01?l0003oooooooooooooool2o`0000;ooooo00Go0000oooooooo
ooooooooo`000004ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooool0003oooooo`000003
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0K
ooooo`03o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`000004ooooo`Oo0?oo2ooo
ool000_ooooo1Ol0ool2ooooo`;o00000oooool2o`0000Gooooo00?o0000ooooooooool01?ooool2
o`0000;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`Co00001?ooool0
0ol0003oooooooooo`02ooooo`Co000000?oooooo`000?ooool05oooool00ol0003oooooooooo`07
ooooo`03o`000?oooooooooo00[ooooo0_l00004ooooo`;o000000Coooooo`000?oooooo00000ooo
ool5o`0000[ooooo00?o0000ooooooooool01_ooool8o`0000;ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`02o`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`00
0?oooooooooo00;ooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02oooool00ol0003oooooo`00000Mooooo`05o`000?oooooooooo
ooooool000000_ooool00ol0003oooooooooo`03ooooo`03o`000?l0oooo0?oo00?o0?oo3?ooool0
00cooooo0ol0ool4ooooo`04o`000?ooooooooooooooo`;o00001_ooool00ol0003oooooooooo`05
ooooo`?o00001Oooool00ol0003oooooooooo`06ooooo`Co000000?oooooo`000?l000000_l00003
ooooo`04o`000?oooooo0000o`0001Kooooo0_l00008ooooo`03o`000?oooooooooo00_ooooo0_l0
0004ooooo`;o000000Coooooo`000?oooooooooo1?l000000ooooooo0000o`00000<ooooo`03o`00
0?oooooooooo00gooooo2?l00002ooooo`05o`000?oooooo0000ooooool000000oooool01Ol0003o
oooooooooooooooo000000Gooooo00Co0000oooooooooooo00000oooool00ol0003oooooo`000003
ooooo`05o`000?ooooooooooooooool000003Oooool01Ol0003ooooooooooooooooo000001cooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0?oo00;o
0?oo3Oooool000oooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01oooool3o`0000?ooooo00?o0000ooooooooool02?ooool6o`0000Go
oooo00?o0000ooooool000005_ooool2o`0000Oooooo00?o0000ooooooooool02oooool00ol0003o
ooooo`000002ooooo`;o000000Goooooo`000?oooooo0000ooooo`03o`0000;ooooo00Co0000oooo
ool0003o00003?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00?ooooo2?l00002
ooooo`05o`000?ooooooooooooooool000001_ooool2o`0000Gooooo0_l00003ooooo`04o`000?oo
ooooooooo`0000gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Jooooo`06o`00
0?oooooo0000o`000?oooooo00001oooool00ol0003oooooo`00000?ooooo`002oooool3o`1o0003
ooooool0003ooooo00;ooooo00?o0000ooooooooool00_l00008ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool00ol00007ooooo`Co00001?ooool00ol0003oooooo`000003o`000005
ooooool0003oooooooooool000005?ooool2o`0000Oooooo00?o0000ooooooooool02oooool01_l0
003oooooooooool0003oooooo`0000?ooooo1?l00002ooooo`03o`000?oooooo000000;ooooo0_l0
000=ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`02o`000003ooooool0003o000000Ko00001Oooool00ol0003oooooooooo`04ooooo`;o00000ooo
ool00ol0003oooooo`00000=ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool06Ooo
ool01_l0003oooooo`000?l0003oooooo`0000Oooooo00?o0000ooooool0ool00_l0ool=ooooo`00
2_ooool5o`1o0006ooooool0003oooooooooool0003ooooo0_l00009ooooo`03o`000?oooooooooo
00Oooooo00Co0000oooooooooooooooo0ol000000ooooooo0000o`000002o`0000Sooooo00?o0000
ooooooooool00oooool5o`0001Cooooo0_l00006ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool00_l00003ooooo`;o000000Ooooooo`000?ooooooooooo`000?oooooo000000Cooooo
0_l0000<ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool00oooool2o`0000;ooooo
0_l00003ooooo`06o`000?ooooooooooooooool0003ooooo2?l00005ooooo`04o`000?oooooooooo
ooooo`;o00003Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Sooooo0ol00000
0ooooooo0000ooooo`07ooooo`03o`000?l0oooo0?oo00?o0?oo3?ooool000Wooooo1ol0O`000ol0
003oooooooooo`02o`000003ooooool0003ooooo00Wooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`06o`0000cooooo00?o0000ooooooooool00oooool00ol0003oooooo`000004o`00
013ooooo0ol00005ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool00_l000001?oo
oooo0000o`000?l00002ooooo`;o00000_ooool2o`000003ooooool0003ooooo00;ooooo0_l0000=
ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000002
ooooo`;o00001?ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00Wo000000?o
ooooo`000?ooool03?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo01Wooooo0ol0
0009ooooo`Oo0?oo2oooool000Wooooo1ol0O`000ol0003oooooooooo`02o`000003ooooool0003o
oooo00Wooooo00?o0000ooooooooool01_ooool4o`0000Gooooo0_l0000:ooooo`03o`000?oooooo
oooo00?ooooo00Go0000ooooool0003oooooo`000002ooooo`?o00003?ooool01?l0003oooooo`00
0?l00004ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool00ol00005ooooo`;o0000
00?oooooo`000?l000000_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool03?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooool0
003oooooo`000004ooooo`04o`000?ooooooooooo`0000Cooooo00Go0000ooooooooooooooooo`00
0005ooooo`;o000000?oooooo`000?l000001ol00006ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool05oooool3o`0000[ooooo1ol0ool;ooooo`002Oooool7o`1o0003ooooool0003o
oooo00;o000000?oooooo`000?ooool02_ooool01?l0003oooooooooooooool5o`0000;ooooo00?o
0000ooooooooool01Oooool3o`0000Sooooo00?o0000ooooooooool00_ooool01_l0003ooooooooo
ool0003oooooo`0000Cooooo1?l00008ooooo`03o`000?oooooo000000Cooooo00?o0000oooooooo
ool03Oooool00ol0003oooooo`000002o`0000Oooooo0_l000000ooooooo0000ooooo`03ooooo`03
o`000?oooooooooo00;ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooo000000Cooooo0_l000000ooooooo0000ooooo`03ooooo`04o`000?ooooooooooo`00
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`;o00000_ooool00ol0
003oooooooooo`04ooooo`So00002_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo0000
00[ooooo00?o0000o`3oool0ool01Ol0ool;ooooo`002_ooool5o`1o00;ooooo1?l000000ooooooo
0000ooooo`0:ooooo`Co00001?ooool00ol0003oooooo`00000;ooooo`;o00001_ooool01Ol0003o
oooooooooooooooo000000Cooooo0_l00008ooooo`Co00000oooool01?l0003oooooo`000?l00003
ooooo`03o`000?oooooooooo00gooooo1?l000000ooooooo0000ooooo`06ooooo`?o00001Oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool02oooool2
o`0000Gooooo0_l000000ooooooo0000ooooo`03ooooo`07o`000?ooooooooooooooool0003ooooo
o`000007ooooo`03o`000?oooooooooo00;ooooo00Go0000ooooooooooooooooo`00000:ooooo`04
o`000?ooooooooooooooo`So00000_ooool00ol0003oooooooooo`0Eooooo`?o00002_ooool2o`00
00Go0?oo3?ooool000_ooooo0ol0O`03ooooo`Co000000?oooooo`000?ooool01_ooool4o`000003
ooooool0003ooooo00Gooooo00?o0000ooooool000003Oooool2o`0000Gooooo00Co0000oooooooo
oooo00001?ooool00ol0003oooooo`00000;ooooo`Co000000?oooooo`000?l000000_ooool00ol0
003oooooooooo`0<ooooo`?o000000Goooooo`000?ooooooooooo`000008ooooo`;o00001_ooool0
1?l0003oooooooooooooool2o`0000oooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`04ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooooooool2o`0000So
oooo00Co0000oooooooooooooooo0ol00002ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool02_ooool8o`00017ooooo00?o0000ooooool000000_l00009ooooo`;o000000Coooooo`3o
ool0oooo0?oo3Oooool0017ooooo1Ol00005ooooo`?o00001Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooo000000oooooo0ol00003ooooo`03o`000?oooooo000000Gooooo0_l0000=oooo
o`03o`000?oooooo000000?o000000?oooooo`000?ooool02_ooool3o`000004ooooool0003ooooo
o`0000;ooooo00?o0000ooooooooool01_ooool2o`0000Sooooo00Go0000ooooooooooooooooo`00
000?ooooo`03o`000?oooooooooo00Wooooo0_l00006ooooo`05o`000?ooooooooooooooool00000
1?ooool01?l0003oooooooooool0000:ooooo`04o`000?oooooo0000ooooo`;o00000_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00oooooo00Co0000oooooooooooooooo2?l00009
ooooo`;o000000?oooooo`000?l000002Oooool4o`0000oooooo000<ooooo`?o0?oo00Coooooo`00
0?oooooooooo0ol000000ooooooo0000o`000002o`0000Wooooo00?o0000ooooooooool00oooool2
o`0001?ooooo0_l000001?oooooo0000ooooool00006ooooo`;o00003?ooool01?l0003oooooo`00
0?l00002ooooo`Co00001oooool2o`0000?ooooo00?o0000ooooool000000oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooo000000Sooooo00Co0000oooooooooooooooo0_l0000?ooooo`03
o`000?oooooooooo00Oooooo00Co0000oooooooooooo00001Oooool01Ol0003ooooooooooooooooo
000000Cooooo00Co0000ooooool0003o00002oooool00ol0003oooooooooo`02o`0000;ooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00Wooooo2_l0
00001?oooooo0000ooooool00009ooooo`03o`000?oooooo000000?oo`3o3Oooool000_ooooo1Ol0
ool3ooooo`Co00003Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01?ooooo1?l0
0006ooooo`;o00002oooool00ol0003oooooooooo`02o`000003ooooool0003ooooo00?ooooo1ol0
0005ooooo`;o00001?ooool00ol0003oooooooooo`04ooooo`04o`000?oooooo0000o`0000Wooooo
00Ko0000oooooooooooo0000ooooool0000?ooooo`03o`000?oooooooooo00Gooooo00Go0000oooo
ooooooooooooo`000004ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`;o000000?ooooo
o`000?ooool02Oooool2o`0000;ooooo00Go0000ooooool0003oooooo`000006ooooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool03oooool9o`0000Oooooo00?o0000oooooooo0?l01?oo
0?l<ooooo`002_ooool7o`3oo`Go00003oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo01Gooooo0ol00006ooooo`;o00002Oooool01?l0003oooooooooooooool3o`0000Oooooo1_l0
0003ooooo`;o00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?o00002Ooo
ool00ol0003oooooooooo`02o`00013ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooooooo00;ooooo0_l00003ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00002Oooool02_l0003oooooooooool0003oooooo`000?oooooo0000ooooool0
0005ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool03_ooool01ol0003ooooooooo
oooooooo0000ooooool000000_ooool8o`0000Ooo`3o2oooool000[ooooo1ol0ool5o`0000oooooo
00?o0000ooooooooool00_ooool2o`0001Sooooo00Co0000ooooool0003o00001Oooool00ol0003o
ooooooooo`07ooooo`06o`000?ooooooooooo`000?oooooo00001Oooool3o`0000Kooooo1?l00006
ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooo0000ooooo`02o`0000Wooooo00Ko
0000oooooooooooo0000ooooool0000@ooooo`05o`000?ooooooooooooooool000001Oooool00ol0
003oooooooooo`02ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
o`0000Sooooo00?o0000ooooooooool00_ooool2o`0000;ooooo0_l00004ooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool03Oooool2o`0000;ooooo00Go0000ooooooooooooooooo`00
0008ooooo`03o`000?oo0?ooo`3o00Goo`3o2oooool000[ooooo1ol0ool=o`0000Sooooo00Co0000
oooooooooooooooo0_l0000Gooooo`;o00000oooool2o`0000?ooooo00?o0000ooooooooool01_oo
ool01?l0003oooooooooooooool2o`0000Cooooo0_l0000;ooooo`03o`000?oooooo000000?o0000
0_ooool00ol0003oooooooooo`02o`0000?ooooo00Co0000ooooool0003o00002_ooool00ol0003o
ooooooooo`02o`00017ooooo00?o0000ooooool000001oooool01?l0003oooooooooool00002oooo
o`04o`000?ooooooooooo`0000?ooooo00Go0000ooooool0003oooooo`000007ooooo`03o`000?oo
oooooooo00?ooooo0_l00002ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0<ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooo
ooooo`05ooooo`;o00001ooo0?l;ooooo`002oooool5o`3oo`?ooooo1?l00003ooooo`oo000000?o
ooooo`000?l000005oooool00ol0003oooooo`000004ooooo`?o000000?oooooo`000?ooool01_oo
ool01?l0003oooooooooooooool2o`000004ooooool0003o0000o`0000cooooo00?o0000oooooooo
ool01?ooool5o`0000Cooooo00?o0000ooooooooool00_l0000;ooooo`05o`000?oooooo0000oooo
ool000004?ooool2o`0000Sooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool0
0002ooooo`06o`000?ooooooooooo`000?oooooo00001_ooool00ol0003oooooooooo`05ooooo`08
o`000?ooooooooooo`000?oooooo0000ooooool0000Kooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000Kooooo1?l00005ool0o`cooooo
000<ooooo`?o0?oo1?ooool2o`000003ooooool0003o000000;o00002?oooolAo`00013ooooo00?o
0000ooooool000001oooool3o`0000Gooooo00Ko0000ooooooooooooooooo`000?ooool2o`0000ko
oooo0_l00007ooooo`03o`000?oooooo000000?o00000_ooool01_l0003oooooooooool0003ooooo
o`0000[ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`0=ooooo`;o00002?ooool0
1?l0003oooooooooool00002ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003o
ooooo`000005ooooo`03o`000?oooooooooo00Kooooo0_l00002ooooo`?o00007Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooooooool3o`000003ooooool0003o000000;ooooo0ooo0?l=ooooo`004oooool2o`000003
ooooool0003ooooo00;ooooo0ol0000:ooooo`03o`000?oooooo000000So000000?oooooo`000?l0
00002Ol00005ooooo`04o`000?ooooooooooo`0000Sooooo0_l00003ooooo`04o`000?oooooooooo
ooooo`Co00003Oooool3o`0000Sooooo0_l00004ooooo`Co000000Coooooo`000?oooooo00002ooo
ool01Ol0003oooooo`000?oooooo000000kooooo00Co0000oooooooooooo00002?ooool00ol0003o
ooooo`000002ooooo`03o`000?oooooooooo00;o00001?ooool01?l0003oooooooooool00003oooo
o`03o`000?oooooooooo00Oooooo0_l00002ooooo`;o00007_ooool00ol0003oooooooooo`07oooo
o`;o00001?ooool00ol0003oooooooooo`03ooooo`?o00000oooool3o`0001;ooooo000=ooooo`?o
O`000oooool2o`000003ooooool0003o000000Gooooo1?l00007ooooo`05o`000?oooooooooooooo
ool000001Oooool8o`0000Cooooo2ol00006ooooo`Ko000000Coooooo`000?l0003ooooo0_l0000=
ooooo`03o`000?oooooo000000Wooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04
o`0000_ooooo00?o0000ooooool000000_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool01Oooool2o`0000?ooooo00?o0000ooooooooool00_l00004
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool02?ooool01?l0003oooooo`00
0?ooool2o`0001oooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00Co0000oooooooooooo
oooo0ol00005ooooo`?o000000Coooooo`3oool0oooo0?oo3_ooool000cooooo1Omo0002ooooo`;o
000000?oooooo`000?l000002Oooool3o`0000Cooooo00Go0000ooooooooooooooooo`00000=oooo
o`So000000?oooooo`000?ooool00_ooool00ol0003oooooo`00000=o`000004ooooool0003ooooo
o`0000cooooo00Co0000oooooooooooo00002?ooool2o`0000Gooooo00?o0000ooooooooool00_oo
ool00ol0003oooooo`000004o`0000Sooooo0_l00002ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`;o00000oooool01Ol0003oooooo`00
0?oooooo000000Gooooo00Go0000ooooool0003oooooo`00000;ooooo`06o`000?ooooooooooo`00
0?oooooo00008?ooool00ol0003oooooooooo`04ooooo`;o00001?ooool00ol0003oooooooooo`02
o`0000;ooooo00?o0000ooooooooool00oooool3o`0000Go0?oo3Oooool000_ooooo1omo00001?oo
oooo0000o`000?ooool2o`0000cooooo1_l00002ooooo`03o`000?oooooooooo01?ooooo2?l00006
ooooo`ko00002?ooool01?l0003oooooooooool00008ooooo`;o00001_ooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000?ooooo1?l00005ooooo`;o00000_ooool00ol0003ooooo
ooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool2o`0000?ooooo
0_l00002ooooo`03o`000?oooooooooo00?ooooo00Co0000ooooool0003o00003?ooool01?l0003o
ooooo`000?ooool2o`00023ooooo00?o0000ooooooooool00oooool2o`0000Gooooo0ol00004oooo
o`03o`000?oooooooooo00?ooooo0_l00007o`3oo`cooooo000;ooooo`OoO`0000?oooooo`000?l0
00000_ooool2o`0000kooooo1?l000000ooooooo0000ooooo`0Dooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooool000002?l000000ooooooo0000ooooo`02ooooo`?o00000oooool;o`0000So
oooo00?o0000ooooool000001Oooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`00
00Oooooo1?l000000ooooooo0000o`000003ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`03ooooo`;o00000oooool2o`0000;ooooo00?o0000oooo
ooooool00oooool00ol0003oooooo`00000=ooooo`04o`000?oooooo0000ooooo`?o00008?ooool0
1ol0003ooooooooooooooooo0000ooooool000000oooool3o`0000Kooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000007o`3oo`cooooo000;ooooo`OoO`0000?oooooo`000?ooool00_oo
ool2o`0000kooooo00?o0000ooooooooool01?l0000Dooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool00_l00003ooooo`Wo000000?oooooo`000?l000001oooool01?l0003ooooooooo
ooooool<o`0000Gooooo00?o0000ooooooooool00oooool01?l0003oooooooooool0000;ooooo`Co
00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool2o`0000?ooooo0_l00002ooooo`03o`000?oooooooooo00?ooooo0ol0000=ooooo`;o
00000oooool2o`00027ooooo00Ko0000ooooool0003oooooo`000?ooool3o`000003ooooool0003o
oooo00Oooooo00Co0000oooooooooooooooo0ol00007o`3oo`cooooo000<ooooo`GoO`000_ooool0
1ol0003ooooooooooooooooo0000ooooool000003Oooool01ol0003oooooooooool0003oooooo`00
0?ooool01?l0000@ooooo`03o`000?oooooooooo00Cooooo0ol00005ooooo`;o000000Koooooo`00
0?ooooooooooo`000?ooool8o`0000;ooooo00Go0000ooooooooooooooooo`000007ooooo`04o`00
0?ooooooooooooooo`_o000000Goooooo`000?ooooooooooo`00000>ooooo`Go00001?ooool00ol0
003oooooooooo`0;ooooo`04o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooo`000002
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000003Oooool00ol0003oooooooooo`03
ooooo`;o00008?ooool2o`000004ooooool0003o0000o`0000Cooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`04o`000003ooooool0oooo0?oo00?o0?oo3Oooool000gooooo0omo0003
ooooo`07o`000?ooooooooooooooool0003oooooo`00000<ooooo`03o`000?oooooooooo00;ooooo
0_l00005ooooo`?o00003Oooool00ol0003oooooooooo`02ooooo`;o000000?oooooo`000?ooool0
1?ooool3o`000007ooooool0003oooooooooool0003oooooo`000005ooooo`[o00001Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00oooool;o`00
00Sooooo00Co0000oooooooooooooooo1Ol0000?ooooo`07o`000?oooooo0000ooooool0003ooooo
o`000002ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000ooooool0003o00003?ooool2o`00
00Gooooo0_l0000Pooooo`Co00001_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
00Co000000?oooooo`000?l0ool00_l0ool>ooooo`004oooool01Ol0003ooooooooooooooooo0000
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`;o00002?ooool4o`00
00Wooooo00?o0000ooooool000000_l00003ooooo`03o`000?oooooooooo00?ooooo00Go0000oooo
oooooooo0000ooooo`02o`000003ooooool0003ooooo00Oooooo0ol00002ooooo`03o`000?oooooo
oooo00So000000?oooooo`000?ooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000001Oooool:o`0000?ooooo00Co0000ooooool0003ooooo1?l0000<ooooo`;o0000
0_ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000002oooo
o`;o00002oooool00ol0003oooooo`000005ooooo`;o00007oooool2o`000003ooooool0003ooooo
00Gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02o`000003ooooool0003o0000
01;ooooo000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000002oooool00ol0003o
ooooooooo`03ooooo`;o00002oooool3o`0000Kooooo0_l00007ooooo`04o`000?oooooooooooooo
o`;o00000_ooool01_l0003oooooooooool0003oooooo`0000Oooooo00Co0000oooooooooooooooo
0_l00006ooooo`03o`000?oooooooooo00;ooooo2?l00006ooooo`05o`000?ooooooooooooooool0
00003_ooool?o`0000Wooooo0_l000000ooooooo0000o`000003ooooo`04o`000?ooooooooooo`00
00;ooooo00Co0000oooooooooooooooo0_l0000:ooooo`04o`000?ooooooooooo`0000Gooooo00?o
0000ooooool000006oooool4o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`Co000000?oooooo`000?ooool04_ooool000kooooo0ooUiND2ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`04ooooo`;o00003_oo
ool6o`0000Wooooo00?o0000ooooooooool00_l00004ooooo`05o`000?oooooo0000ooooool00000
1_ooool00ol0003oooooooooo`03ooooo`?o00000oooool00ol0003oooooooooo`04ooooo`05o`00
0?ooooooooooooooool000000_ooool8o`0000;ooooo00?o0000ooooooooool03?ooool2o`0000;o
oooo0_l00006ooooo`_o000000Koooooo`000?l0003o0000ooooool00003ooooo`06o`000?oooooo
ooooo`000?oooooo00001?ooool00ol0003oooooo`000008ooooo`05o`000?ooooooooooooooool0
00001Oooool00ol0003oooooo`00000Hooooo`?o00000_ooool01_l0003oooooooooool0003ooooo
o`0000Gooooo00?o0000ooooooooool02?ooool3o`000004ooooool0003o0000ooooo`?oiNGU3ooo
ool000gooooo1OoUiND2o`0000Cooooo00Co0000oooooooooooo00002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00gooooo0_l000001?oooooo0000o`000?l00009ooooo`03o`00
0?oooooo000000Gooooo0_l000000ooooooo0000o`000005ooooo`03o`000?oooooooooo00?ooooo
00Co0000oooooooooooooooo0ol00007ooooo`05o`000?ooooooooooooooool000002?ooool00ol0
003oooooo`000007o`0000[ooooo0ol00002ooooo`03o`000?oooooooooo00[ooooo3_l00003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooool000001oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00005Oooool2o`0000Cooooo00Co0000
oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000_ooooo0ol000000ooooooo0000
o`000005onGUi@kooooo000<ooooo`OoiNGU00?o0000ooooooooool00_ooool01Ol0003ooooooooo
oooooooo000000Sooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0:ooooo`?o0000
1?ooool00ol0003oooooo`000002o`0000Oooooo00?o0000ooooooooool01Oooool01?l0003ooooo
o`000?l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`000003
ooooool0003o000000Gooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00;ooooo2?l00002ooooo`;o00000oooool00ol0003oooooooooo`0>oooo
o`Go000000?oooooo`000?ooool00_ooool<o`0000Kooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00;o00004Oooool3o`0000Gooooo00Go0000oooo
ooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000002oooool4o`000003oooooooU
iNGoiNGU00GoiNGU3Oooool000cooooo1ooUiND00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000001oooool00ol0003oooooooooo`06ooooo`;o00002Oooool2o`0000Oooooo00?o
0000ooooooooool00_ooool4o`0000;ooooo0_l00007ooooo`04o`000?oooooo0000o`0000?ooooo
00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000Gooooo0_l00002ooooo`05
o`000?ooooooooooooooool000002Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00[ooooo2Ol0000?ooooo`;o00000_ooool4o`0000;ooooo0_l000000ooooooo0000ooooo`05oooo
o`co00000_ooool00ol0003oooooooooo`04ooooo`04o`000?oooooo0000o`0000oooooo0_l00007
ooooo`05o`000?ooooooooooooooool000001Oooool01?l0003oooooooooool0000:ooooo`04o`00
0?oooooo0000o`0000;ooooo1ooUiND=ooooo`003?ooool7onGUi@03o`000?oooooooooo00;ooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo
0_l00006ooooo`?o00002Oooool00ol0003oooooooooo`06ooooo`?o000000?oooooo`000?ooool0
1Oooool3o`000004ooooool0003oooooo`0000Kooooo00Go0000ooooooooooooooooo`000008oooo
o`?o00000_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool02_ooool01Ol0003oooooo`000?oooooo000000;ooooo2?l00008ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooool000000ol000000ooooooo0000ooooo`05ooooo`;o00001?ooool0
0ol0003oooooooooo`03ooooo`[o00000_ooool2o`0000_ooooo0ol00009ooooo`05o`000?oooooo
ooooooooool000001_ooool2o`0000_ooooo00?o0000ooooool000000oooool7onGUi@gooooo000=
ooooo`GoiNGU0_l00005ooooo`05o`000?ooooooooooooooool000001_ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooo000000?ooooo0_l0000<ooooo`03o`000?oooooooooo00Kooooo0_l0
00000ooooooo0000o`000002o`0000?ooooo00Co0000ooooool0003o00000_ooool00ol0003ooooo
ooooo`04ooooo`05o`000?ooooooooooooooool000002oooool4o`0000[ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`09ooooo`06o`000?ooooooooooo`000?oooooo00000oooool0
0ol0003oooooooooo`04ooooo`Wo00001Oooool00ol0003oooooooooo`02o`000003ooooool0003o
000000;o00001?ooool2o`0000?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04
ooooo`co00000_ooool2o`0000_ooooo00Go0000ooooooooooooooooo`000007ooooo`;o00002ooo
ool3o`0000?ooooo00?o0000onGUiOoUiND00ooUiND>ooooo`003_ooool3onGUi@03ooooool0003o
000000Gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooool000000ol0000=ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooool000000oooool00ol0003oooooo`000003o`000003ooooool0003ooooo00?o00001_ooool0
1?l0003oooooooooool0000<ooooo`03o`000?oooooo000000;o00002Oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000002oooool;o`000005ooooool0003oooooooooool000000_ooool6o`00
00;ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`04o`000?oooooooooo
ooooo`;o00001Oooool<o`0000?ooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oo
oooooooo00Sooooo00Co0000ooooool0003o00000oooool00ol0003oooooonGUi@02onGUi@oooooo
000Booooo`;o00001Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01oooool4o`0000oooooo00?o0000ooooooooool01Oooool00ol0003oooooo`00
0004ooooo`03o`000?oooooooooo00;ooooo1?l00003ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00003Oooool00ol0003oooooo`000002ooooo`?o00001_ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00oooool01?l0003ooooo
ooooool00002ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooool000001oooool9o`0000Co
oooo1?l0000:ooooo`03o`000?oooooooooo00Cooooo00Oo0000ooooooooooooooooo`000?oooooo
000000;ooooo0_l0000<ooooo`[o00001_ooool00ol0003oooooo`000009ooooo`Co00000oooool0
0ol0003oooooooooo`0Aooooo`004?ooool3ool0o`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool01Ol0003ooooooooooooooooo000000Sooooo0_l000001?oooooo0000ooooool0
000?ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00001Oooool01?l0003ooooo
ooooool00002ooooo`Go000000?oooooo`000?ooool00_ooool00ol0003oooooo`00000=ooooo`03
o`000?oooooo000000Kooooo0_l00004ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooo
ooooool01_ooool01?l0003oooooooooool00006ooooo`;o000000?oooooo`000?l000000_ooool0
0ol0003oooooooooo`0<o`0000Kooooo00?o0000ooooooooool01?ooool01_l0003ooooooooooooo
oooo0000ooooo`?o00003oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooooooo`_o
00001_ooool4o`000004ooooool0003oooooooooo`?o0?oo4Oooool000oooooo1Ooo0?l5ooooo`03
o`000?oooooooooo00?ooooo00Co0000oooooooooooo00001Oooool3o`0000?ooooo00Co0000oooo
oooooooo00003_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000Kooooo00Go
0000oooooooooooo0000ooooo`02o`000004ooooool0003oooooooooo`?o000000Coooooo`000?oo
oooo00003_ooool00ol0003oooooo`000008ooooo`;o00000_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo
000000;ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00006ooooo`03o`000?oo
oooo000000;ooooo00Co0000oooooooooooo00001oooool2o`000003ooooool0003o000000Ko0000
0_ooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooooooo`;o000000?oooooo`000?oo
ool03_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo00Go0000oooooooo
oooo0000ooooo`0<o`000003ooooool0oooo0?oo00?o0?oo4?ooool000kooooo1ooo0?l5ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool2o`0000Kooooo00Co0000oooo
oooooooo00003_ooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000001ooo
ool00ol0003oooooo`000002o`0000;ooooo00?o0000ooooooooool00_ooool5o`0000kooooo0_l0
000;ooooo`?o00001_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Oooooo00Go
0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooo
ooooo`000005ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooo`000007oooo
o`03o`000?oooooooooo00Sooooo1ol00002ooooo`Co000000Goooooo`000?ooooooooooo`00000>
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooool000001?l00007o`3oo`oooooo000>oooo
o`Ooo`3o1Oooool00ol0003oooooooooo`03ooooo`;o000000Coooooo`000?l0003o00002?ooool0
1Ol0003ooooooooooooooooo000000gooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00Kooooo0ol000000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00;ooooo
0_l00002ooooo`?o00002oooool2o`0000gooooo0ol00004ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooo
ooooooooo`000004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool01_l0
003oooooooooool0003oooooo`0000Kooooo0ol0000<ooooo`go00000_ooool00ol0003ooooooooo
o`0:ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01_oooolBo`000003ooooool0
003o0?oo00Ko0?oo3oooool000kooooo1ooo0?l2o`0000?ooooo00?o0000ooooooooool00oooool3
o`0000_ooooo00Go0000ooooooooooooooooo`00000=ooooo`04o`000?ooooooooooo`0000?ooooo
00?o0000ooooooooool02?ooool01?l0003oooooooooool00003ooooo`04o`000?oooooooooooooo
o`;o00001_ooool4o`0000Oooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`02o`00
00?ooooo00Co0000oooooooooooo00002oooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00_l000000ooooooo0000ooooo`03ooooo`04o`000?oooooo0000o`0000co
oooo00?o0000ooooool000001Ol00004ooooo`So00001_oooolIo`0000?ooooo0_l000000ooooooo
0000ooooo`03ooooo`Oo0?oo3oooool000oooooo1Ooo0?l2ooooo`;o00000_ooool01?l0003ooooo
ooooooooool4o`0000cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`04
o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool01oooool3o`000003ooooool0003ooooo
00?ooooo00Ko0000ooooool0003o0000ooooool00009ooooo`?o00001?ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00;ooooo1?l000000ooooooo0000ooooo`0<ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool0
00001_ooool01Ol0003oooooooooool0003ooooo00;o00000oooool01Ol0003ooooooooooooooooo
000000cooooo0_l00005oooooaGo000000?oooooo`000?l000002Ol00003ooooo`03o`000?oooooo
oooo00Gooooo00Co0000ooooool0003o00000_ooool00ol0003oooooooooo`02ooooo`03o`000?l0
oooo0?oo00?o0?oo4?ooool0013ooooo0ooo0?l2o`0000;ooooo1?l000000ooooooo0000o`000002
ooooo`;o00003?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooool000001?ooool00ol0003oooooooooo`07ooooo`;o000000?oooooo`000?l000001Oooool0
0ol0003oooooo`000003ooooo`03o`000?oooooooooo00Wooooo1?l0000?ooooo`03o`000?oooooo
oooo00Gooooo0_l0000>ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_l00005
ooooo`04o`000?ooooooooooo`0000Sooooo00?o0000ooooool000000_ooool01?l0003ooooooooo
ool00004ooooo`03o`000?oooooooooo00Kooooo3ol000000ooooooo0000o`000002o`000003oooo
ool0003ooooo017o00001oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo000000;ooooo
00?o0000ooooooooool00_ooool2o`0000?o0?oo4Oooool001;ooooo0ol0ool01?l0003ooooooooo
ooooool3o`0000Cooooo00?o0000ooooool000003?ooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo00Sooooo0_l00005ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool00000
0_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
3?ooool4o`0000cooooo00?o0000ooooooooool01Oooool01?l0003oooooo`000?l0000=ooooo`03
o`000?oooooooooo00;ooooo00Co0000ooooool0003o00001?ooool01?l0003oooooooooool00009
ooooo`03o`000?oooooo000000;ooooo4_l00003ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooool000002_l00004ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool8
o`0000Oooooo0ol00002ooooo`03o`000?oooooooooo00?ooooo0ol0oolCooooo`004Oooool5o`3o
o`?o000000?oooooo`000?l000000_l00002ooooo`03o`000?oooooo000000cooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`08ooooo`;o00001?ooool00ol0003oooooooooo`06oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000
ooooooooool02_ooool2o`0000?ooooo0ol00009ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooool000000_ooool3o`0000[ooooo00?o0000ooooooooool00_ooool2o`0000;ooooo00So0000
ooooooooooooooooo`000?oooooo0000oooooa7o00001Oooool00ol0003oooooooooo`02ooooo`?o
00001?ooool:o`0000Gooooo00Ko0000ooooooooooooooooo`000?ooool5o`0000Gooooo00?o0000
ooooooooool02_ooool9o`000005ooooool0003oooooooooool000001?ooool5o`3ooa;ooooo000@
ooooo`Oo0?oo00Co0000oooooooooooooooo0_l000000ooooooo0000o`000003ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`07ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool01_ooool01?l0003oooooooooool00003ooooo`;o0000
0_ooool01Ol0003oooooo`000?oooooo000000Gooooo00?o0000ooooooooool02Oooool2o`0000Ko
oooo1?l00005ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00001Oooool2o`00
00Sooooo4Ol00009ooooo`03o`000?oooooo000000;ooooo00?o0000ooooool000001Oooool00ol0
003oooooo`000009o`000003ooooool0003ooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`02ooooo`04o`000?ooooooooooooooo`Go00000_ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?l00007o`3ooa7ooooo000@oooo
o`Oo0?oo00Soooooo`000?ooooooooooo`000?oooooo0000ooooo`;o00000_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01_ooool2o`0000Cooooo
00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000;ooooo00Oo0000ooooool0
003oooooo`000?oooooo000000;ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooo000000[ooooo0ol00002ooooo`03o`000?oooooooooo00;ooooo00Ko0000
ooooooooooooooooo`000?ooool@o`0000;ooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooo`000?l0000=ooooo`03o`000?oooooo000000co00002_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooo`000005oooo
o`Co00002_ooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooo`000?ooool00ol00003
ooooo`Oo0?oo4Oooool0013ooooo1ol0ool2ooooo`03o`000?oooooo000000;ooooo0_l000001Ooo
oooo0000o`000?oooooo000000[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04
ooooo`;o00001?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooo00000_ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool01oooool2o`0000_ooooo4?l00008ooooo`?o00001?ooool00ol0003ooooo
o`000003ooooo`06o`000?ooooooooooo`000?oooooo00001oooool<o`0000?ooooo00?o0000oooo
ool000000ol0000;ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`02ooooo`;o00001oooool00ol0003oooooooooo`04o`0000Kooooo00?o0000oooo
ooooool00oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool00_ooool7
o`3ooa7ooooo000Aooooo`Go0?oo1?ooool01ol0003oooooo`000?oooooo0000ooooool000000_oo
ool2o`0000[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooo000000Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`06o`000?oo
ooooooooooooool0003ooooo0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01?ooool?o`0000Gooooo00?o0000ooooool000000_l00005ooooo`03
o`000?oooooooooo00Wooooo0_l00002ooooo`03o`000?oooooo000000Cooooo00?o0000ooooool0
00002Ol00007ooooo`?o000000Coooooo`000?oooooooooo0ol00002ooooo`;o00002oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool2o`00
00Kooooo00?o0000ooooooooool01Oooool4o`000003ooooool0003ooooo00Cooooo00Go0000oooo
ooooooooooooo`000002ooooo`?o00000oooool00ol0003o0?ooo`3oo`03o`3ooa;ooooo000Boooo
o`?o0?oo1_ooool2o`000004ooooool0003oooooooooo`;o000000Coooooo`000?l0003o00002?oo
ool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000Kooooo00?o0000ooooooooool00oooool01_l0003oooooooooool0003ooooo
o`0000;ooooo00Co0000oooooooooooooooo4Ol00002ooooo`03o`000?oooooooooo00kooooo00?o
0000ooooool000000_ooool3o`0000;ooooo00?o0000ooooooooool02oooool>o`0000kooooo0ol0
00001?oooooo0000o`000?l00003ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`07o`000?ooooooooooooooool0003oooooo`000004
ooooo`03o`000?oooooooooo00[ooooo0ol00003ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool00ol00003ooooo`03o`000?oooooo0?oo00;o0?oo4oooool001cooooo0_l00005oooo
o`04o`000?oooooo0000ooooo`;o00001_ooool00ol0003oooooooooo`05ooooo`04o`000?oooooo
ooooo`0000?ooooo00Co0000oooooooooooo00001_ooool01?l0003oooooooooooooool?o`0000?o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`00
013ooooo0_l00006ooooo`Co00000oooool:o`000004ooooool0003o0000o`0000Gooooo00Co0000
ooooool0003o00003Oooool00ol0003oooooo`000004o`0000Cooooo00Co0000oooooooooooo0000
2oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooool0
00000ol00003ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooo`00000Hoooo
o`005Oooool3o`3oo`Cooooo0_l00006ooooo`;o00000oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo0_l00003ooooo`oo00000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooool000001_ooool00ol0003oooooooooo`04ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00;ooooo2_l0000:
ooooo`04o`000?oooooo0000ooooo`;o00000_ooool01_l0003oooooo`000?l0003oooooo`0000_o
oooo00?o0000ooooool000000_l00002ooooo`03o`000?oooooooooo00?ooooo00Co0000ooooool0
003o00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ool000001?ooool01?l0003oooooooooool0000=ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooool000000ol00002ooooo`;o00000oooool01?l0003oo`3oool0oooo0?lFooooo`005?ooool5
o`3oo`?ooooo0ol00007ooooo`04o`000?ooooooooooooooo`;o000000?oooooo`000?l000003_l0
00001?oooooo0000ooooool00004ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
0_l00002ooooo`05o`000?ooooooooooooooool000001Oooool00ol0003oooooooooo`04ooooo`05
o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`0:ooooo`[o00001_ooool01?l0
003oooooooooooooool4o`0000Gooooo00Go0000ooooooooooooooooo`000002ooooo`;o00000_oo
ool01?l0003oooooooooool0000:ooooo`?o000000?oooooo`000?l000000_ooool00ol0003ooooo
ooooo`04ooooo`?o00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01?ooool00ol0003oooooo`00000=ooooo`05o`000?ooooooooooooooool0
00001?ooool00ol0003oooooo`000003o`0000?ooooo1Ooo0?lEooooo`004oooool7o`3oo`;ooooo
4_l00002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooool000001?ooool00ol0003ooooo
o`000003ooooo`03o`000?oooooooooo00Oooooo00Co0000ooooool0003o00000oooool01Ol0003o
oooooooooooooooo000000Gooooo00?o0000ooooooooool01Oooool00ol0003oooooo`000003oooo
o`03o`000?oooooo000000[o00000oooool00ol0003oooooo`00000:ooooo`03o`000?oooooooooo
00Gooooo0ol000000ooooooo0000ooooo`03ooooo`05o`000?ooooooooooo`000?ooool01?l00002
ooooo`03o`000?oooooooooo00Gooooo0ol00004ooooo`04o`000?ooooooooooo`0000Oooooo0_l0
000:ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool2o`0000Kooooo0_l0
000>ooooo`04o`000?ooooooooooo`0000Gooooo00Co0000oooooooooooooooo1?l00007ool0oaCo
oooo000Cooooo`Oo0?oo1?l000000ooooooo0000ooooo`06ooooo`03o`000?oooooo000000Cooooo
0ol00005ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooo`000003ooooo`03
o`000?oooooooooo00Oooooo00Go0000ooooool0003oooooo`000002ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`[o00003Oooool01?l0
003oooooooooool0000:ooooo`03o`000?oooooooooo00Sooooo1?l00003ooooo`;o00001?ooool3
o`0000;ooooo00Co0000oooooooooooooooo0ol000000ooooooo0000ooooo`04ooooo`;o000000?o
ooooo`000?ooool01_ooool2o`0000[ooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`03ooooo`04o`000?ooooooooooo`0000Gooooo0_l0000=ooooo`04o`000?ooooooooooo`0000Go
oooo00Co0000oooooooooooooooo0_l00003ooooo`Ooo`3o5?ooool001?ooooo1ol0ool2ooooo`;o
00000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;o00001?ooool2o`0000?o
oooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02o`0000?ooooo00?o0000oooooooo
ool02?ooool2o`0000;ooooo00?o0000ooooool000001_ooool;o`0000?ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0;ooooo`05o`000?ooooooooooooooool000002_ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00;ooooo1?l00004ooooo`03o`000?oooooooooo
00Co000000?oooooo`000?l000000oooool00ol0003oooooooooo`04ooooo`;o000000?oooooo`00
0?ooool01_ooool3o`0000Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000Gooooo00?o0000ooooool000003?ooool00ol0003oooooo`000006
ooooo`04o`000?ooooooooooooooo`Go00001ooo0?lDooooo`005?ooool5o`3oo`;ooooo00?o0000
ooooool000000oooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003oooooo`000?oooooo000000Kooooo00Co0000ooooool0003o00000_ooool00ol0003o
ooooooooo`09ooooo`04o`000?ooooooooooooooo`_o00001Oooool00ol0003oooooooooo`04oooo
o`;o00000oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo1?l00002oooo
o`04o`000?ooooooooooooooo`Co00001?ooool00ol0003oooooooooo`05ooooo`;o000000?ooooo
o`000?ooool01_ooool00ol0003oooooo`000009ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool00_ooool01Ol0003ooooooooooooooooo000000Cooooo00Co0000oooooooooooo0000
3?ooool2o`0000Oooooo1_l00002ooooo`;o00001Ooo0?lEooooo`005Oooool3o`3oo`?ooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l0
00001?oooooo0000ooooooooool2o`0000Kooooo00?o0000ooooool000000_ooool00ol0003ooooo
ooooo`04ooooo`[o000000?oooooo`000?ooool01oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00?ooooo00Ko0000oooooooooooo0000ooooool0000<ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00Go0000oooooooooooo0000ooooo`08o`000003ooooool0003o000000?o00002Oooool0
0ol0003oooooo`000007ooooo`04o`000?oooooo0000o`0000Sooooo00?o0000ooooooooool01_oo
ool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooool000000oooool5o`0000;ooooo0_l00002ooooo`;o
00000_ooool3ool0oaKooooo000Kooooo`03o`000?oooooo000000Gooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`03ooooo`;o000000Coooooo`000?oooooooooo0_l00005ooooo`_o
00001?ooool2o`0000?ooooo00?o0000ooooool000002Oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooooooo0_l0000;ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Cooooo00Go0000ooooooooooooooooo`000003ooooo`Ko00000_ooool00ol0003oooooo`000002
o`0000Sooooo00?o0000ooooool000001oooool01Ol0003oooooo`000?oooooo000000Oooooo00?o
0000ooooooooool01_ooool01?l0003oooooooooool00006ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool01oooool6o`0000?ooooo00Co0000oooooooooooooooo0_l000000ooo
oooo0000o`00000Looooo`006?ooool3o`3oo`?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Cooooo3ol0000:ooooo`;o000000?oooooo`000?oo
ool00_ooool2o`0000[ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo0000
00Cooooo0_l0000:ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool4o`00
00Gooooo1Ol00003ooooo`;o00001_ooool00ol0003oooooo`000008ooooo`;o00000_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooool000002?ooool00ol0
003oooooo`000005ooooo`03o`000?oooooooooo00Cooooo1Ol00006ooooo`03o`000?oooooooooo
00;ooooo1Ol00003ogmoOaWooooo000Gooooo`Go0?oo0_ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00_o00001_ooool5o`0000[ooooo00Co0000oooooooooooo00001?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00001Oooool0
0ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02_ooool0
1Ol0003ooooooooooooooooo000000Kooooo00?o0000ooooooooool00_ooool3o`000003ooooool0
003ooooo00Kooooo1Ol00003ooooo`?o00000oooool00ol0003oooooo`000008ooooo`;o00000ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo0_l00009ooooo`;o00001ooo
ool00ol0003oooooo`000003o`0000;ooooo00?o0000ooooooooool01_ooool01?l0003ooooooooo
ooooool5o`0000GoOgmo6?ooool001Kooooo1ol0ool00ooooooo0000o`000008o`0000;ooooo00?o
0000ooooooooool00oooool2o`000003ooooool0003ooooo00Kooooo0ol00009ooooo`05o`000?oo
ooooooooooooool000000oooool2o`0000gooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`000006ooooo`;o00001oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00[ooooo
00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`03o`0000;ooooo00?o0000ooooool0
00001oooool01Ol0003oooooooooool0003ooooo00?o00000oooool2o`000005ooooool0003o0000
ooooool000001oooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06
ooooo`;o00002Oooool00ol0003oooooooooo`03ooooo`Go00001Oooool2o`0000Oooooo00?o0000
ooooooooool00_ooool3o`000003oooooomoOgooOgmo00GoOgmo5oooool001Kooooo1ol0ool2o`00
00Sooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000;ooooo0_l00004
ooooo`Go00001oooool00ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000003_oo
ool00ol0003oooooooooo`03ooooo`;o00001_ooool2o`0000Kooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooo000000Kooooo00Co0000ooooool0003o0000
1Oooool2o`0000Oooooo00Go0000ooooool0003oooooo`000004ooooo`Co000000Koooooo`000?l0
003o0000ooooool00007ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`06ooooo`;o00002Oooool6o`0000Cooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000007ooooo`03o`000?oooooooooo00Co000000?oooooo`000?moOgl01_moOglG
ooooo`005_ooool7o`3oo`;o00002Oooool01?l0003oooooooooool00002ooooo`04o`000?oooooo
ooooo`0000Cooooo00So0000ooooooooooooooooo`000?oooooo0000ooooo`;o00001Oooool00ol0
003oooooooooo`02ooooo`04o`000?ooooooooooooooo`;o00003oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00001?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00[ooooo0_l00006ooooo`?o00002?ooool00ol0003ooooooooo
o`04ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool6o`0000Oooooo
00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000Oooooo00?o0000ooooool0
00001oooool4o`0000Wooooo00Co0000oooooooooooooooo0_l00008ooooo`09o`000?oooooo0000
ooooool0003oooooo`000?oooooo000000OoOgmo5oooool001Oooooo1Ol0ool3ooooo`?o00001ooo
ool01Ol0003oooooo`000?oooooo000000?ooooo00?o0000ooooooooool00oooool3o`0000;ooooo
00?o0000ooooooooool00ol00002ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo
oooo0_l0000?ooooo`03o`000?oooooooooo00;ooooo0_l00005ooooo`05o`000?oooooooooooooo
ool000000_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo0_l00004oooo
o`;o00002oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ool000002_ooool5o`0000Gooooo00?o0000ooooool000001Oooool01?l0003oooooooooool00006
ooooo`04o`000?ooooooooooo`0000;ooooo1Ol000001?oooooo0000ooooool0000:ooooo`05o`00
0?oooooo0000ooooool000001oooool01?l0003oooooo`000?ooool2o`000003ooooool0003o0000
00;ooooo1OmoOglHooooo`006?ooool3o`3oo`Cooooo00?o0000ooooooooool00ol00005ooooo`03
o`000?oooooo000000Cooooo00?o0000ooooooooool01?ooool2o`0000;ooooo00Co0000oooooooo
oooo00000_ooool2o`0000Kooooo00?o0000ooooooooool00_l0000Aooooo`06o`000?oooooooooo
o`000?oooooo00001Oooool01ol0003ooooooooooooooooo0000ooooool000003Oooool00ol0003o
ooooooooo`0:ooooo`;o000000?oooooo`000?l000000_l0000;ooooo`;o00001?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooo000000Wooooo0ol000001?oooooo0000o`000?l00003oooo
o`03o`000?oooooo000000Kooooo00?o0000ooooool000001Oooool00ol0003oooooooooo`04o`00
00Gooooo00Go0000ooooooooooooooooo`000009ooooo`;o00000_ooool00ol0003oooooooooo`05
ooooo`;o000000?oooooo`000?ooool00_l000000ooooooo0000ooooo`02ooooo`?oOgmo6Oooool0
01cooooo0ol0ool00ol0003oooooooooo`03ooooo`?o00000oooool2o`0000Cooooo00?o0000oooo
ooooool00oooool2o`000003ooooool0003o000000Cooooo00Go0000ooooool0003oooooo`000004
ooooo`04o`000?ooooooooooooooo`;o00004Oooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo00Wooooo00Co0000ooooool0003o00000_ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooo000000;ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00008oooo
o`?o00001?ooool4o`000003ooooool0003ooooo00Kooooo0_l00003ooooo`Go000000?oooooo`00
0?ooool01Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Oooooo0_l000000ooo
oooo0000ooooo`06ooooo`09o`000?ooooooooooo`000?oooooo0000ooooool0003ooooo00?o0?oo
7Oooool001_ooooo1Ol0ool8ooooo`?o000000?oooooo`000?l000001?ooool00ol0003ooooooooo
o`03o`0000?ooooo0_l00003ooooo`;o00000oooool2o`0000;ooooo00Ko0000oooooooooooo0000
ooooool0000Booooo`;o00000oooool00ol0003oooooooooo`02ooooo`07o`000?oooooooooooooo
ool0003oooooo`00000=ooooo`03o`000?oooooooooo00Sooooo1?l00002ooooo`03o`000?oooooo
oooo00_ooooo00Go0000ooooool0003oooooo`000009ooooo`03o`000?oooooo000000Sooooo0ol0
0007ooooo`Co00001oooool4o`000003ooooool0003ooooo00?ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_l0
0005ooooo`?o000000Ooooooo`000?ooooooooooo`000?oooooo000000Go0?oo7?ooool001[ooooo
1ol0ool8ooooo`03o`000?oooooo000000;o00001?ooool00ol0003oooooo`000002ooooo`06o`00
0?ooooooooooooooool0003ooooo0_l000000ooooooo0000o`000005ooooo`?o000000Goooooo`00
0?ooooooooooo`00000Booooo`;o00001?ooool01?l0003oooooooooool00003ooooo`04o`000?oo
ooooooooo`0000gooooo00?o0000ooooooooool01_ooool2o`0000Cooooo0_l0000=ooooo`04o`00
0?ooooooooooooooo`;o00002_ooool00ol0003oooooo`000007ooooo`?o00001oooool00ol0003o
ooooo`000009o`0000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02o`0000Co
oooo00?o0000ooooool000000_l00003ooooo`;o00001ol0oolKooooo`006_ooool7o`3oo`Oooooo
00?o0000ooooooooool00oooool3o`000003ooooool0003o000000;ooooo00?o0000ooooooooool0
0_ooool01ol0003ooooooooooooooooo0000ooooool000001_ooool4o`000003ooooool0003ooooo
01;ooooo0_l00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00?ooooo1?l00005ooooo`03o`000?oooooo
oooo00cooooo00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooo000000Sooooo0_l0
0008ooooo`co000000?oooooo`000?ooool01Oooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01?ooool01?l0003oooooooooooooool2o`0000?ooooo
00Co0000oooooooooooo00001?ooool00ol0003oooooo`000007o`3ooa_ooooo000Jooooo`Oo0?oo
1_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co000000?oooooo`000?ooool0
0oooool01Ol0003oooooooooool0003ooooo00?o00001Oooool00ol0003oooooo`000003o`0001?o
oooo0_l00005ooooo`05o`000?oooooo0000ooooool000001_ooool00ol0003oooooooooo`09oooo
o`04o`000?ooooooooooooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`;o00003_ooool0
0ol0003oooooooooo`02o`0000cooooo00?o0000ooooool000001oooool2o`0000Cooooo1Ol00000
0ooooooo0000ooooo`02ooooo`;o00000_ooool3o`0000Sooooo00Go0000ooooooooooooooooo`00
0008ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0002o`0000;ooooo0_l00004ooooo`03o`000?oooooo000000Oo0?oo6oooool001_ooooo1Ol0ool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Kooooo00Co0000ooooool0003o00000_oo
ool3o`0000?ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`03ooooo`;o00000ooo
ool00ol0003oooooooooo`0@ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000ooooool0003o
00001oooool00ol0003oooooooooo`09ooooo`Co00000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooo000000gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooo000000Kooooo1_l00004ooooo`03o`000?oooooo000000Gooooo0_l000000ooo
oooo0000ooooo`04o`0000Cooooo00Co0000oooooooooooo00002Oooool01Ol0003ooooooooooooo
oooo000000Gooooo00Co0000ooooool0003o00000_ooool2o`0000Cooooo00Go0000oooooooooooo
0000ooooo`05o`3ooacooooo000Looooo`?o0?oo00Coooooo`000?l0003o00000_ooool00ol0003o
ooooooooo`07ooooo`;o000000Coooooo`000?oooooo00000oooool5o`0000Kooooo0_l00003oooo
o`;o00000oooool3o`00013ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000Wooooo00?o0000ooooooooool01oooool2o`0000Gooooo00?o0000ooooooooool00oooool0
1Ol0003ooooooooooooooooo000000cooooo00?o0000ooooool000000oooool00ol0003ooooooooo
o`0:ooooo`03o`000?oooooo000000?ooooo1_l00007ooooo`04o`000?ooooooooooo`0000Kooooo
0ol000001?oooooo0000ooooooooool3o`000005ooooool0003oooooooooool000002_ooool00ol0
003oooooo`000005ooooo`09o`000?oooooo0000ooooool0003oooooo`000?oooooo000000Cooooo
00Co0000ooooool0003o00000_ooool3o`3ooagooooo000Pooooo`?o0?oo0_l0000:ooooo`Co0000
00?oooooo`000?ooool01?ooool4o`0000Gooooo00Co0000ooooool0003ooooo0_l00003ooooo`04
o`000?ooooooooooooooo`;o00003Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`03ooooo`?o000000?oooooo`00
0?ooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool02_ooool2o`0000Cooooo00?o0000ooooooooool02oooool5o`0000?ooooo00?o0000oooo
ool000001oooool01?l0003oooooooooool00006ooooo`;o000000?oooooo`000?l000001Oooool4
o`0000_ooooo0_l00006ooooo`;o00000oooool00ol0003oooooo`000005ooooo`03o`000?oooooo
0?oo00;o0?oo8Oooool001oooooo1Ol0ool00ol0003oooooooooo`07ooooo`04o`000?oooooo0000
ooooo`;o00001Oooool2o`0000?ooooo0ol00003ooooo`04o`000?oooooo0000o`0000Cooooo00?o
0000ooooooooool00oooool2o`0000_ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`02o`0000_ooooo00Co0000oooooooooooooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Wooooo
0_l00006ooooo`03o`000?oooooooooo00Kooooo1Ol000000ooooooo0000ooooo`04ooooo`03o`00
0?oooooo000000Oooooo00Co0000oooooooooooo00001Oooool00ol0003oooooo`000003ooooo`;o
00001?ooool00ol0003oooooo`000003o`0000Sooooo0_l00005ooooo`;o00000oooool01?l0003o
ooooo`000?l00004ooooo`03o`000?oooooo0?oo00Co0?oo8?ooool001kooooo1ol0ool00ol0003o
ooooooooo`05ooooo`07o`000?ooooooooooooooool0003oooooo`000004ooooo`03o`000?oooooo
000000Kooooo1?l00002ooooo`;o00000oooool00ol0003oooooooooo`05ooooo`;o00002?ooool0
0ol0003oooooooooo`04ooooo`04o`000?oooooo0000ooooo`;o00002oooool3o`0000Kooooo00Go
0000ooooooooooooooooo`000004ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo1?l00005ooooo`03o`00
0?oooooo000000Cooooo00?o0000ooooool000001oooool01?l0003oooooooooool00004ooooo`04
o`000?ooooooooooo`0000Cooooo1?l000000ooooooo0000o`000004ooooo`?o00001Oooool2o`00
00?ooooo0_l000000ooooooo0000ooooo`02o`0000;ooooo00?o0000ooooooooool00oooool00ol0
003o0?ooo`3oo`05o`3ooaoooooo000Nooooo`Oo0?oo0ol00004ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`04o`000?ooooooooooo`0000Wooooo0ol00002ooooo`03o`000?oooooo0000
00[ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00;o00002Oooool2o`000003ooooool0003ooooo00Kooooo00Co0000oooooooooooo00001?ooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo0_l00007ooooo`Go00002_ooool0
0ol0003oooooo`000003ooooo`03o`000?oooooo000000Oooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00?o0000ooooooooool00_l00008ooooo`Go0000
00Goooooo`000?ooooooooooo`000002ooooo`03o`000?oooooo000000Cooooo0_l00004ooooo`03
o`000?l0oooo0?oo00Go0?oo7oooool001kooooo1ol0ool4o`0000;ooooo00?o0000ooooooooool0
0oooool01?l0003oooooo`000?ooool2o`0000?ooooo00?o0000ooooooooool02?ooool7o`0000_o
oooo0_l00004ooooo`03o`000?oooooooooo00Gooooo0_l00002ooooo`03o`000?oooooo000000Go
oooo0ol00003ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000001?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00Kooooo0_l00004ooooo`Co000000?oooooo`000?oo
ool03?ooool00ol0003oooooo`000004ooooo`?o00001_ooool01ol0003ooooooooooooooooo0000
ooooool000001?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?o00002Oooool6
o`0000?ooooo0_l00005ooooo`03o`000?oooooo000000;ooooo00?o0000ooooool0ool01_l0oolO
ooooo`007oooool5o`3oo`;ooooo1_l00005ooooo`?o000000?oooooo`000?l000000oooool00ol0
003oooooooooo`08ooooo`;o00000oooool3o`0000cooooo0_l00002ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00_l00002ooooo`04o`000?ooooooooooooooo`;o00001oooool0
0ol0003oooooooooo`04ooooo`;o00001?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooool000001?l00005ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooool000000oooool3o`0000Kooooo00Co0000oooooooooooooooo0_l00005ooooo`03o`000?oo
oooooooo00Gooooo0_l000000ooooooo0000ooooo`03o`0000Gooooo00Co0000oooooooooooooooo
1Ol000000ooooooo0000ooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo0ol00005o`3oob3o
oooo000Pooooo`?o0?oo1Oooool9o`000004ooooool0003o0000o`0000;ooooo00?o0000ooooool0
00002_ooool3o`0000Cooooo00?o0000ooooool000000ol0000:ooooo`;o00001_ooool01?l0003o
ooooooooooooool2o`0000?ooooo0ol0000:ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00;ooooo1Ol0000:oooo
o`03o`000?oooooooooo00cooooo0_l00003ooooo`?o00001oooool00ol0003oooooooooo`02o`00
00Gooooo00?o0000ooooooooool01Oooool2o`000003ooooool0003ooooo00?ooooo0_l00002oooo
o`07o`000?ooooooooooooooool0003oooooo`000003ooooo`?o00001?ooool01_l0003ooooooooo
oooooooo0000ooooo`?o000000Coooooo`3oool0oooo0?oo8Oooool002Cooooo0ol0ool00ooooooo
0000o`000002ooooo`?o000000?oooooo`000?l000000_l00004ooooo`;o00002_ooool2o`000005
ooooool0003oooooooooool000001?ooool00ol0003oooooo`000002o`0000Sooooo0ol00003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooool000000_l0000<ooooo`03o`000?oooooooooo
00;ooooo0_l00003ooooo`03o`000?oooooooooo00_ooooo1_l00002ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool03?ooool00ol0003oooooo`000002ooooo`;o000000?oooooo`00
0?ooool01Oooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00Gooooo0_l00000
0ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?l000000oooool01?l0003oooooo`000?l0
0002ooooo`Co00001?ooool01Ol0003o0?ooo`3oool0oooo000002Gooooo000Sooooo`Go0?oo0_oo
ool01Ol0003oooooooooool0003ooooo00;o000000?oooooo`000?l000001?l000000ooooooo0000
ooooo`08ooooo`05o`000?oooooo0000ooooool000000_ooool00ol0003oooooooooo`03ooooo`;o
00000_ooool3o`0000Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000Gooooo0ol00003
ooooo`03o`000?oooooooooo00[ooooo00Ko0000oooooooooooo0000ooooool00002ooooo`03o`00
0?oooooooooo00Sooooo1Ol00003ooooo`05o`000?ooooooooooooooool000002oooool00ol0003o
ooooooooo`0=ooooo`06o`000?oooooo0000ooooool0003ooooo0_l00006ooooo`04o`000?oooooo
0000o`0000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`02o`0000Oooooo1?l0
0004ooooo`;o000000?oooooo`000?ooool01?ooool5o`0000Go0?oo9Oooool002;ooooo1ol0ool2
ooooo`03o`000?oooooooooo00;o000000?oooooo`000?l000000_l00003ooooo`Co00001_ooool0
0ol0003oooooo`000003ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool00_ooool3
o`000003ooooool0003ooooo00?ooooo0_l00003ooooo`;o000000?oooooo`000?l000000oooool0
0ol0003oooooooooo`0;ooooo`07o`000?oooooo0000ooooool0003oooooo`000007ooooo`Co0000
1?ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00[ooooo00?o0000oooooooo
ool03Oooool2o`000003ooooool0003ooooo00;o00001_ooool2o`0000;ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000001_ooool00ol0
003oooooo`000002ooooo`;o000000?oooooo`000?l000000_l00005ooooo`03o`000?oooooooooo
00;o00001ol0oolTooooo`008_ooool7o`3oo`?ooooo00Go0000ooooool0003oooooo`000003oooo
o`?o00000_ooool00ol0003oooooo`000003o`0000?ooooo00?o0000ooooool000000oooool00ol0
003oooooo`000008ooooo`;o00001Oooool3o`0000;ooooo00?o0000ooooooooool00ol00003oooo
o`03o`000?oooooo000000?ooooo00?o0000ooooooooool02_ooool2o`0000;ooooo0_l00003oooo
o`Go00002Oooool2o`0000Cooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0=oooo
o`05o`000?oooooo0000o`000?ooool00_l00006ooooo`05o`000?ooooooooooooooool000001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`?o00001Ooo
ool2o`000003ooooool0003o000000Gooooo00Go0000ooooool0003oooooo`000007o`3oobCooooo
000Rooooo`Oo0?oo1?ooool01Ol0003oooooo`000?oooooo000000;ooooo0_l000001?oooooo0000
o`000?l00002ooooo`03o`000?oooooooooo00Co000000?oooooo`000?ooool00oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01Ol00002ooooo`;o
000000Goooooo`000?ooooooooooo`000003ooooo`03o`000?oooooooooo00[ooooo0_l00002oooo
o`Go00003_ooool2o`0000Cooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0>oooo
o`?o000000Coooooo`000?oooooo00001Oooool01Ol0003ooooooooooooooooo000000Gooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000;ooooo0_l0
0006ooooo`03o`000?oooooooooo00?o00001Oooool2o`000003ooooool0003ooooo00Oo0?oo9?oo
ool002?ooooo1Ol0ool5ooooo`;o00000oooool01?l0003oooooo`000?l00003ooooo`Co00000ooo
ool00ol0003oooooooooo`04o`0000;ooooo00?o0000ooooooooool02_ooool2o`000003ooooool0
003ooooo00;ooooo1_l00003ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000[ooooo
1_l0000Booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool2o`0000?ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00?ooooo
00Co0000ooooool0003ooooo0_l00006ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`04
o`000?oooooo0000o`0000?ooooo1Ol0oolUooooo`009?ooool3o`3oo`;ooooo0ol0ool2o`000003
ooooool0003ooooo00;ooooo0_l000001?oooooo0000ooooool00003ooooo`?o000000Goooooo`00
0?ooooooooooo`000003ooooo`Co00003Oooool5o`0000Kooooo0ol000001_oooooo0000o`000?l0
003oooooo`0000?ooooo00?o0000ooooooooool01Oooool4o`0000;ooooo00?o0000ooooool00000
4Oooool2o`0000Cooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0>ooooo`06o`00
0?oooooo0000o`000?oooooo00000oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo
00?ooooo00Go0000ooooooooooooooooo`000007ooooo`04o`000?oooooo0000o`0000Kooooo00?o
0000ooooooooool00oooool2o`0000;ooooo1?l000001?oooooo0?ooo`3oool0ool2o`0000?o0?oo
9_ooool002Sooooo1Ol0ool3ooooo`;o00000_ooool3o`000003ooooool0003ooooo00Cooooo1?l0
00000ooooooo0000ooooo`04ooooo`Ko00002Oooool2o`000003ooooool0003ooooo00Wooooo1Ol0
00000ooooooo0000ooooo`02ooooo`03o`000?oooooo000000Co000000?oooooo`000?ooool00_oo
ool01?l0003oooooo`000?l0000@ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0?ooooo`?o000000Goooooo`000?ooooooooooo`000002oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo
000000Oooooo1?l00005ooooo`03o`000?oooooooooo00Cooooo00Go0000ooooool0003o0000oooo
o`03o`0000Go0?oo:_ooool002Oooooo1ol0ool4ooooo`04o`000?oooooo0000ooooo`?o00001_oo
ool00ol0003oooooooooo`03o`0000?ooooo00Co0000oooooooooooo00000oooool4o`0000;ooooo
1?l000000ooooooo0000o`00000;ooooo`03o`000?oooooooooo00?o000000?oooooo`000?l00000
0_l00005ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0=ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00kooooo0ol00002ooooo`;o00000oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001oooool3o`0000;ooooo00Go0000ooooooooooooooooo`00
0006ooooo`04o`000?oooooo0000o`0000;ooooo00?o0000ooooool0ool01_l0oolYooooo`009ooo
ool7o`3oo`Gooooo0_l00002ooooo`;o00001Oooool01Ol0003oooooo`000?oooooo000000;ooooo
1?l00002ooooo`03o`000?oooooooooo00Gooooo1Ol000001Ooooooo0000oooooooooooo000000Wo
oooo00?o0000ooooooooool01ol000000ooooooo0000ooooo`03ooooo`05o`000?oooooooooooooo
ool000001?ooool2o`0000gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0000A
ooooo`03o`000?oooooooooo00oooooo0_l00002ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000004ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooool00000
1?ooool3o`0000Oooooo0_l000001?oooooo0000ooooool00002ooooo`Oo0?oo:Oooool002Oooooo
1ol0ool6ooooo`;o00000_ooool2o`0000?ooooo00Co0000oooooooooooooooo0_l00005ooooo`;o
00000_ooool01?l0003oooooooooooooool3o`0000Cooooo1?l00002ooooo`;o00001_ooool4o`00
00Cooooo00?o0000ooooool000000ol00002ooooo`05o`000?ooooooooooooooool000001Oooool2
o`0000cooooo00?o0000ooooooooool00oooool01?l0003oooooooooool0000Aooooo`03o`000?oo
oooooooo00oooooo0ol000000ooooooo0000ooooo`04ooooo`05o`000?ooooooooooooooool00000
1?ooool00ol0003oooooo`000006ooooo`04o`000?oooooo0000o`0000Cooooo0_l00007ooooo`;o
00000_ooool3o`0000;ooooo1ol0oolYooooo`00:?ooool5o`3oo`Sooooo00Ko0000oooooooooooo
0000ooooool00002ooooo`04o`000?ooooooooooooooo`;o00001Oooool00ol0003oooooo`000002
o`000003ooooool0003o000000Kooooo0_l00003ooooo`[o000000?oooooo`000?ooool01_ooool0
0ol0003oooooooooo`05o`0000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooo`00000:
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooool000004_ooool00ol0003oooooooooo`0>
ooooo`Go00001Oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00005ooooo`;o00000_oo
ool2o`0000?ooooo00?o0000ooooool000001_ooool2o`0000;ooooo00Co0000ooooool0003o0000
0_ooool00ol0003o0?ooo`3oo`03o`3oob[ooooo000Yooooo`?o0?oo0oooool3ool0o`?ooooo1?l0
0002ooooo`;o00001?ooool00ol0003oooooo`000003ooooo`04o`000?ooooooooooooooo`Go0000
1_ooool00ol0003oooooooooo`03ooooo`Oo00000oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00?o000000Coooooo`000?l0003o00001_ooool01?l0003oooooooooool00008oooo
o`03o`000?oooooooooo00Kooooo0_l0000Cooooo`03o`000?oooooooooo00kooooo00Ko0000oooo
ool0003o0000ooooool00004ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000Cooooo
00Oo0000ooooooooooooooooo`000?oooooo000000;ooooo00Go0000ooooooooooooooooo`000004
ooooo`05o`000?oooooo0000ooooool000000_ooool01_l0003oo`3oool0oooo0?oo0000ooooo`?o
0?oo:oooool002kooooo1Ooo0?l2ooooo`09o`000?oooooo0000ooooool0003oooooo`000?oooooo
000000?ooooo00Co0000oooooooooooo00000_ooool01?l0003oooooo`000?l00002ooooo`03o`00
0?oooooooooo00?o00000_ooool00ol0003oooooooooo`04o`0000Kooooo1Ol00008ooooo`03o`00
0?oooooooooo00Go00000_ooool3o`0000?ooooo00Co0000oooooooooooo00001oooool00ol0003o
ooooooooo`07ooooo`;o00005?ooool00ol0003oooooooooo`0>ooooo`Go00001?ooool00ol0003o
ooooooooo`02ooooo`05o`000?ooooooooooooooool000001?ooool01Ol0003ooooooooooooooooo
000000?ooooo0_l00005ooooo`04o`000?oooooo0000o`0000;ooooo0_l00003ooooo`Goo`3o;ooo
ool002gooooo1ooo0?l00ooooooo0000ooooo`03o`000007ooooool0003oooooooooool0003ooooo
o`000004ooooo`Co00001Oooool00ol0003oooooooooo`02ooooo`Go00002oooool3o`000003oooo
ool0003o000000;o00001Oooool01Ol0003oooooo`000?l0003ooooo00?o00001?ooool4o`0000?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`;o00005?ooool00ol0003o
ooooooooo`0=ooooo`Ko00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooooooooooo
ool000000_ooool2o`0000?ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`05oooo
o`04o`000?ooooooooooooooo`;o00000oooool7ool0obkooooo000]ooooo`Ooo`3o00Goooooo`00
0?oooooo0000ooooo`02o`0000Cooooo0_l00003ooooo`Co00001_ooool:o`0000gooooo00?o0000
ooooooooool00oooool4o`000004ooooool0003o0000ooooo`;o00000oooool2o`0000Gooooo1?l0
00000ooooooo0000ooooo`02ooooo`03o`000?oooooooooo00Wooooo0_l0000Dooooo`03o`000?oo
oooooooo00cooooo00Go0000ooooool0003o0000ooooo`02o`0000Cooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`02o`000003ooooool0003ooooo00?ooooo00?o0000ooooooooool0
0_ooool2o`0000Gooooo0_l000000ooooooo0000ooooo`02o`0000Cooooo1ooo0?l^ooooo`00;Ooo
ool7ool0o`;o000000?oooooo`000?ooool00_l00005ooooo`Co00000_ooool2o`0000;ooooo1_l0
0002ooooo`;o00001?ooool3o`0000[ooooo0ol00007ooooo`Ko00001Oooool2o`0000?ooooo00Co
0000oooooooooooooooo0ol000000ooooooo0000ooooo`0;ooooo`;o00005Oooool00ol0003ooooo
ooooo`0;ooooo`05o`000?oooooo0000o`000?ooool00ol00003ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool00ol00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool0
00001?ooool01?l0003oooooooooooooool2o`000003ooooool0003ooooo00;ooooo00?o0000ool0
oooo0?l01Ooo0?l^ooooo`00;_ooool5ool0o`;ooooo0ol000001?oooooo0000ooooool00003oooo
o`;o000000?oooooo`000?ooool00_ooool4o`0000Gooooo00Go0000ooooool0003oooooo`000007
ooooo`?o00001oooool01?l0003oooooooooool00006ooooo`03o`000?oooooo000000Go00001Ooo
ool01?l0003oooooooooool00005ooooo`Co00002oooool2o`0001Gooooo00?o0000ooooooooool0
2_ooool00ol0003oooooooooo`02o`0000;ooooo0_l00003ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooool000000_l00005ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003ooooo
ooooo`02o`0000Cooooo0ol00003ooooo`?o00001Ooo0?l_ooooo`00;oooool3ool0o`?ooooo0ooo
0?l2o`000003ooooool0003o000000?o000000?oooooo`000?l000001?l00002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooool000002oooool3o`0000Cooooo00Co0000oooooooooooooooo
0_l00003ooooo`03o`000?oooooooooo00;o00000_ooool00ol0003oooooo`000002o`0000?ooooo
0ol00005ooooo`;o00000_ooool3o`0000Sooooo0_l0000Eooooo`03o`000?oooooooooo00Wooooo
00Co0000oooooooooooooooo0_l00003ooooo`;o00000_ooool00ol0003oooooooooo`03ooooo`;o
000000?oooooo`000?ooool00oooool01?l0003oooooooooool00004ooooo`03o`000?oooooo0000
00Gooooo00Go0000oooooooooooo0000ooooo`03o`3oo`03o`000?oooooooooo00?oo`3o<?ooool0
03Cooooo1Ooo0?l00ooooooo0000o`000005o`0000?ooooo0ol00003ooooo`03o`000?oooooooooo
00;ooooo0_l000000ooooooo0000ooooo`0=ooooo`;o000000?oooooo`000?ooool01Oooool02?l0
003oooooooooool0003oooooo`000?oooooo00000_ooool00ol0003oooooooooo`02ooooo`Co0000
00?oooooo`000?l000000oooool00ol0003oooooo`000005ooooo`?o00001?ooool01?l0003ooooo
ooooool0000Eooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool2o`0000?o
oooo0_l00002ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`05o`000?oooooooooooooo
ool000000_ooool00ol0003oooooooooo`03ooooo`;o00001?ooool2o`0000?ooooo00?o0000o`3o
ool0ool00ol0ooleooooo`00<oooool7ool0o`Go00001_ooool2o`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00oooooo0ol00006ooooo`;o000000?oooooo`00
0?ooool00_ooool01?l0003oooooooooool00007ooooo`Go000000Goooooo`000?ooooooooooo`00
0007ooooo`?o000000Goooooo`000?ooooooooooo`00000Eooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool00oooool2o`0000Cooooo0_l00002ooooo`03o`000?oooooooooo00?o0000
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00?ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo00?o0000o`3oool0ool01Ol0ooldooooo`00<oooool7ool0o`?ooooo
1?l00004ooooo`03o`000?oooooo000000?ooooo00?o0000ooooool000000oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooo000000;o00000oooool3o`0000Gooooo00?o0000ooooool00000
2?ooool00ol0003oooooooooo`04o`0000;ooooo00?o0000ooooooooool01oooool3o`000003oooo
ool0003ooooo01Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00;ooooo0_l00002ooooo`04o`000?oooooo0000ooooo`;o00000_ooool00ol0003o
ooooo`000002ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`04o`000?ooooooooooo`00
00Gooooo00?o0000o`3oool0ool01Ol0ooldooooo`00<oooool7ool0o`?ooooo0_l000000ooooooo
0000o`000002o`000004ooooool0003oooooooooo`;o00000_ooool01ol0003ooooooooooooooooo
0000ooooool000004Oooool00ol0003oooooooooo`02ooooo`?o000000Coooooo`000?oooooo0000
1?ooool01?l0003oooooooooool00007ooooo`04o`000?ooooooooooo`0000;ooooo1Ol00008oooo
o`03o`000?oooooooooo00?o00005Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo0_l00003ooooo`?o000000Ooooooo`000?l0003oooooo`000?oooooo000000;ooooo0_l0
0002ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0006ooooo`03o`000?l0oooo0?oo00Go0?oo=?ooool003Cooooo1Ooo0?l4ooooo`;o00000_ooool0
0ol0003oooooooooo`02o`0000Cooooo0_l000001Ooooooo0000oooooooooooo000000;ooooo00?o
0000ooooooooool03_ooool00ol0003oooooooooo`03ooooo`04o`000?oooooo0000o`0000;ooooo
0_l00003ooooo`03o`000?oooooo000000Oooooo00?o0000ooooool000001?ooool2o`000003oooo
ool0003o000000?o00001?ooool00ol0003oooooooooo`02ooooo`Go00004Oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Kooooo0_l00003ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool00_ooool01_l0003oooooo`000?l0003oooooo`0000?ooooo0_l00005oooo
o`;o00001_ooool00ol0003oooooo`000005o`3oocGooooo000eooooo`?oo`3o0oooool3ool0o`08
o`000?ooooooooooooooool0003oooooo`000?ooool3o`000003ooooool0003o000000;ooooo0_l0
0003ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool00oooool00ol0003oooooo`00
0002ooooo`?o000000Goooooo`000?ooooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo
0_l00007ooooo`?o00000oooool5o`0000Cooooo00?o0000ooooooooool00_ooool3o`0000kooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`07ooooo`;o00001?ooool4o`000003oooo
ool0003ooooo00;ooooo00?o0000ooooool000000_l00003ooooo`03o`000?oooooooooo00Cooooo
0_l00005ooooo`?o0?oo00?o0000ooooooooool00ol0oolfooooo`00>_ooool5ool0o`04o`000?oo
ooooooooooooo`;o00001?ooool2o`000005ooooool0003o0000ooooool000001?ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001_ooool4o`000004
ooooool0003oooooo`0000Kooooo00?o0000ooooooooool02?ooool2o`0000Gooooo1Ol000000ooo
oooo0000ooooo`06ooooo`?o00003?ooool01?l0003oooooooooool0000:ooooo`;o00000oooool2
o`000004ooooool0003o0000o`0000Cooooo1?l00002ooooo`;o00001_ooool00ol0003oooooo`00
0004ooooo`Go0?oo>oooool003Wooooo1ooo0?l4ooooo`;o00000_ooool00ol0003oooooooooo`04
o`000003ooooool0003ooooo00;ooooo00?o0000ooooooooool03?ooool01ol0003ooooooooooooo
oooo0000ooooool000002_ooool3o`0000;ooooo00?o0000ooooooooool00_ooool2o`0000_ooooo
0ol00003ooooo`03o`000?oooooooooo00;ooooo1?l00008ooooo`?o00002Oooool00ol0003ooooo
o`00000;ooooo`;o00000_ooool00ol0003oooooo`000002ooooo`;o00001?ooool2o`000004oooo
ool0003oooooo`0000Oooooo00Co0000oooooooooooo00000oooool7o`3ooc[ooooo000iooooo`Oo
o`3o0oooool00ol0003oooooooooo`02o`0000Kooooo0_l000000ooooooo0000ooooo`02ooooo`03
o`000?oooooooooo00_ooooo00Ko0000oooooooooooo0000ooooool0000=ooooo`Co00000oooool0
0ol0003oooooo`00000<ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool01?ooool0
0ol0003oooooo`000003o`0000Oooooo0ol00007ooooo`03o`000?oooooooooo00[ooooo1?l00002
ooooo`03o`000?oooooooooo00;o00000oooool2o`0000;ooooo0_l00007ooooo`05o`000?oooooo
ooooooooool000000_ooool00ol0003o0?ooo`3oo`05o`3ooc[ooooo000iooooo`Ooo`3o00Co0000
oooooooooooo00000_ooool2o`0000Gooooo00?o0000ooooooooool00ol00002ooooo`03o`000?oo
oooooooo00_ooooo00?o0000ooooooooool00_l0000?ooooo`;o000000Coooooo`000?l0003o0000
0_ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00;o00001oooool00ol0003ooooo
ooooo`03ooooo`Co00001_ooool3o`0000Cooooo00?o0000ooooooooool02oooool00ol0003ooooo
ooooo`03ooooo`Go00000_ooool00ol0003oooooo`000002o`0000Kooooo0_l00004ooooo`04o`00
0?oooooo0000ooooo`Oo0?oo>_ooool003[ooooo1Ooo0?l00ooooooo0000o`3oo`02o`3oo`05oooo
ool0003oooooooooool000001?ooool01?l0003oooooooooooooool4o`000003ooooool0003ooooo
00_ooooo00Co0000ooooool0003o00004Oooool8o`0000gooooo00?o0000ooooooooool00ol00005
ooooo`03o`000?oooooooooo00Oooooo1?l00005ooooo`Go00003?ooool2o`0000Gooooo0ol00000
1Ooooooo0000ooooool0003ooooo00;o000000?oooooo`000?ooool01?ooool00ol0003ooooooooo
o`04ooooo`?oo`3o00?oooooo`000?l0ool01?l0oolkooooo`00>oooool3ool0o`;ooooo1Ol0ool4
ooooo`04o`000?ooooooooooo`0000Kooooo1Ol0000:ooooo`04o`000?oooooo0000o`0001;ooooo
00Co0000ooooool0003o00000oooool3o`0000cooooo00?o0000ooooool000000_ooool2o`0000?o
oooo00?o0000ooooooooool02oooool4o`0000?ooooo1?l00009ooooo`;o000000?oooooo`000?oo
ool01?ooool00ol0003oooooooooo`03o`000006ooooool0003oooooo`000?oooooo00001?ooool0
0ol0003oooooooooo`04ooooo`Goo`3o0_ooool3o`3ooccooooo000oooooo`Oo0?oo1?ooool00ol0
003oooooo`000007ooooo`;o000000Coooooo`000?l0003o00002?ooool3o`0001Cooooo00?o0000
ooooool000000_l000001?oooooo0000ooooooooool3o`0000Wooooo0_l00005ooooo`;o00000_oo
ool00ol0003oooooooooo`0>ooooo`Co00000_ooool4o`0000Gooooo00Go0000oooooooooooooooo
o`000005ooooo`;o000000?oooooo`000?l000000_l00002ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool01?ooool7ool0od3ooooo000oooooo`Oo0?oo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`07ooooo`?o00000_ooool3o`0000Gooooo0_l0000Dooooo`04o`000?oo
oooo0000o`0000;ooooo0_l00005ooooo`;o00002?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooo0000017ooooo00?o0000ooooooooool01?l00002ooooo`Go00001?ooool2o`0000?ooooo
00?o0000ooooool000000ol000001ooooooo0000oooooooooooo0000ooooool000000_ooool00ol0
003oooooooooo`05ooooo`Ooo`3o@?ooool003oooooo1ol0ool00ooooooo0000ool0o`02ool0o`Go
00001oooool2o`0000Cooooo0_l00002ooooo`;o00005?ooool01_l0003oooooooooool0003ooooo
o`0000;ooooo0_l00006ooooo`?o00001Oooool2o`0000Oooooo0_l0000@ooooo`03o`000?oooooo
oooo00Cooooo1?l000000ooooooo0000ooooo`03o`0000;ooooo0_l00002ooooo`03o`000?oooooo
oooo00?o00000_ooool2o`0000;ooooo1?l00003ooooo`?o0?oo00?oooooo`000?oo0?l01_oo0?m0
ooooo`00@?ooool5o`3oo`;ooooo1Ooo0?l01?oooooo0000ooooooooool9o`0000Kooooo1?l0000C
ooooo`03o`000?oooooooooo00;ooooo0_l00002ooooo`03o`000?oooooo000000Sooooo0ol00000
1?oooooo0000ooooool00008ooooo`?o00003_ooool00ol0003oooooooooo`04ooooo`04o`000?oo
ooooooooooooo`Co00000_ooool4o`0000;ooooo00?o0000ooooool000000ol00003ooooo`03o`00
0?oooooooooo00;o00001?ooool5o`3oo`;ooooo1Ooo0?m1ooooo`00@Oooool3o`3oo`;ooooo1ooo
0?l00ooooooo0000ooooo`08ooooo`go00004Oooool00ol0003oooooooooo`02ooooo`05o`000?oo
oooo0000ooooool000000_ooool00ol0003oooooooooo`08ooooo`?o000000?oooooo`000?ooool0
1oooool01?l0003oooooo`000?l0000;ooooo`03o`000?oooooooooo00Gooooo00Co0000ooooool0
003o00001?ooool8o`000003ooooool0003o000000?o00001?ooool01?l0003oooooo`000?l00003
ooooo`Oo0?oo0_ooool3ool0od;ooooo0016ooooo`Ooo`3o0_ooool00ol0003oooooooooo`07oooo
o`03o`000?oooooo000000?ooooo2_l0000<ooooo`03o`000?oooooooooo00Cooooo0_l000000ooo
oooo0000ooooo`02ooooo`;o00002Oooool00ol0003oooooo`000002o`0000Oooooo00Co0000oooo
oooooooooooo0_l00008ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool7
o`000003ooooool0003o000000Cooooo1?l00003ooooo`Oo0?ooAoooool004Kooooo1ooo0?l2oooo
o`?o07l02?ooool01Ol0003oooooo`000?oooooo000000;ooooo00?o0000ooooooooool00oooool:
o`0000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooo`000?l00005ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool00_ooool3o`0000Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Kooooo0ol0000;ooooo`Wo00001?oo
ool2o`0000?o0?oo00?oooooo`000?l0ool01_l0oom7ooooo`00Aoooool5ool0o`;ooooo1Ol0O`07
ooooo`;o000000?oooooo`000?ooool00_ooool00ol0003oooooooooo`06ooooo`;o00001Oooool8
o`0000Cooooo0ol00006ooooo`;o00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
000000;o00000oooool00ol0003oooooooooo`03ooooo`;o00000oooool00ol0003oooooooooo`06
ooooo`05o`000?ooooooooooooooool000002_ooool2o`000003ooooool0003ooooo00;ooooo1_l0
00000ooooooo0?ooo`3oo`03o`3oo`;ooooo1Ol0oom8ooooo`00B?ooool3ool0o`;ooooo1ol0O`07
ooooo`;o000000Goooooo`000?l0003oooooo`00000:ooooo`?o00000oooool00ol0003ooooooooo
o`04ooooo`Wo00001_ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Cooooo00Co
0000oooooooooooooooo1?l00007ooooo`;o000000?oooooo`000?ooool01_ooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooooooo00Kooooo0ol000000ooooooo0000ooooo`04ooooo`03o`00
0?oooooo000000;o00001ol0ool2ooooo`?o0?ooBOooool004gooooo1ol0O`07ooooo`03o`000?oo
oooo000000;ooooo00?o0000ooooooooool03?ooool3o`0000cooooo0_l00002ooooo`Wo000000?o
ooooo`000?ooool01oooool00ol0003oooooooooo`03ooooo`?o00001oooool2o`0000Gooooo0_l0
0005ooooo`03o`000?oooooooooo00Gooooo1?l000000ooooooo0000ooooo`04ooooo`;o00000_oo
ool00ol0003o0?ooo`3oo`05o`3oodkooooo001=ooooo`Oo07l000?o0000ooooool0ool00_l0ool0
1ooooooo0000ooooool0003oooooo`000?ooool00_l0000>ooooo`Co00002_ooool3o`0000Wooooo
2?l00003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_l00004ooooo`03o`00
0?oooooooooo00;o00000_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo
0_l000001?oooooo0000o`000?l00005ooooo`?oo`3o0_l000000ooooooo0?ooo`3oo`05o`3oodko
oooo001>ooooo`Go07l00_ooool5o`3oo`;ooooo00?o0000ooooooooool00_l000000ooooooo0000
o`00000;ooooo`03o`000?oooooooooo00;ooooo0_l00009ooooo`;o00002oooool2o`0000Cooooo
2Ol000000ooooooo0000ooooo`02ooooo`Co00001Oooool2o`0000Wooooo00Co0000oooooooooooo
oooo0ol000001?oooooo0000ooooool00005ooooo`Goo`3o0oooool5o`3oodoooooo001?ooooo`?o
07l00_ooool7o`3oo`;ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`07oooo
o`03o`000?oooooooooo00Gooooo0ol00006ooooo`?o00002Oooool01?l0003oooooooooool00008
ooooo`04o`000?ooooooooooooooo`_o00000_ooool4o`0000Sooooo00Wo0000oooooooooooo0000
ooooool0003oooooo`000?ooool00ol00003ooooo`Ooo`3o0oooool3o`3ooe3ooooo001Dooooo`Oo
0?oo0_ooool01Ol0003oo`3oool0oooo0?oo000000?ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`09ooooo`;o00001?ooool3o`0000Wooooo00Co0000oooooooooooooooo0_l00006
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000007
o`000003ooooool0003o000000Oooooo0_l000001?oooooo0000ooooooooool2o`000004ooooool0
oooo0?ooo`3oo`;o00001ooo0?mFooooo`00E?ooool7o`3oo`;ooooo1Ooo0?l01?l0003ooooooooo
ooooool2o`0000Cooooo00?o0000ooooooooool03?ooool3o`000003ooooool0003ooooo00;o0000
2?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00Go0000oooooooooooo
ooooo`000004ooooo`04o`000?ooooooooooooooo`;o00000_ooool?o`0000;ooooo00?o0000oooo
ool000001Ol0ool00oooooooo`3oool0o`05ool0oeKooooo001Eooooo`Go0?oo0_ooool7ool0o`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00004_ooool2o`000003ooooool0003o0000
00Oooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000001oooool2o`0000;ooooo3?l00007o`3oo`03oooooooo
0?ooo`3o00?oo`3oEoooool005Kooooo0ol0ool3ooooo`Ooo`3o00?oooooo`000?ooool00ooo0?l2
o`0001Cooooo1Ol00006ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool0
1ol0003ooooooooooooooooo0000ooooool000000oooool00ol0003oooooooooo`08ooooo`Go0000
00?oooooo`000?ooool00ol0ool3o`0000Oo0?oo0_ooool3ool0oeSooooo001Looooo`Ooo`3o0_oo
ool5ool0o`03o`000?oooooooooo01;ooooo00?o0000ooooooooool00ol00005ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`02o`0000;ooooo0_l0
000<ooooo`Ko00001Ol0ool2ooooo`Oo0?ooGOooool005gooooo1Ooo0?l2ooooo`Ooo`3o0_l00003
o`3oo`oooooo00Ko0000ooooooooooooooooo`000?ooool3o`000003ooooool0003ooooo00_ooooo
0_l00003ooooo`03o`000?oooooo000000;ooooo00?o0000ooooooooool02oooool3ool0o`03oooo
ool0003ooooo00Oo0?oo0_ooool5o`3ooekooooo001Nooooo`?oo`3o0oooool7ool0o`03ooooool0
oooo0?oo00?o0?oo3_ooool01?l0003oooooooooooooool2o`0000?ooooo0_l0000>ooooo`03o`00
0?oooooooooo00;o000000?oooooo`000?l000003?ooool00ol0003oo`3oool0o`03ool0o`03oooo
ool0003o0?oo00Ko0?oo0oooool3o`3ooeoooooo001Tooooo`Ooo`3o1ol0ool3ooooo`?o0?oo1ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo1?l0000<ooooo`;o000000Co
ooooo`000?l0003o00001oooool3o`3oo`;ooooo00?o0000ool0oooo0?l01Ooo0?l00ooooooo0?oo
o`3oo`05o`3oofGooooo001Uooooo`Goo`3o00?oooooo`3oool0ool01Ol0ool2ooooo`Go0?oo1Ooo
ool3o`3oo`Cooooo00Co0000oooooooooooo00001?ooool2o`0000cooooo0ol00003ool0o`Cooooo
1Ol0ool00ol0003oooooool0o`06ool0o`;ooooo1Ol0oomVooooo`00I_ooool3ool0o`;ooooo1ol0
ool00ol0003o0?ooo`3oo`05o`3oo`?ooooo1Ol0ool3ooooo`04o`000?oo0?ooo`3oool0o`Kooooo
0ol00004ooooo`?oo`3o00Coooooo`000?l0003o00001Ooo0?l2ooooo`Oo0?oo00?oooooool0oooo
0?l01Ooo0?l3ooooo`?o0?ooIoooool006cooooo1Ol0ool2ooooo`Oo0?oo0_ooool7o`3oo`;ooooo
1Ooo0?l4ooooo`?oo`3o00Coooooo`000?l0003ooooo1Ooo0?l2ooooo`Ooo`3o00?oooooo`3oool0
ool01Ol0ool2ooooo`Goo`3oK_ooool006gooooo0ol0ool3ooooo`Oo0?oo0_ooool7o`3oo`03oooo
oooo0?ooo`3o00Goo`3o0_ooool5ool0o`;ooooo1ooo0?l00oooooooo`3oool0o`05ool0o`03oooo
ool0oooo0?oo00Go0?oo0oooool3ool0ofoooooo001dooooo`Go0?oo0oooool7o`3oo`03oooooooo
0?ooo`3o00Goo`3o00?o0000ool0oooo0?l01Ooo0?l00oooooooo`3oool0o`05ool0o`03oooooooo
0?ooo`3o00Goo`3o0_ooool5o`3oogKooooo001eooooo`?o0?oo1Oooool5o`3oo`;ooooo1ooo0?l0
0oooooooo`3oool0o`05ool0o`03oooooooo0?ooo`3o00Goo`3o0_ooool5ool0o`Cooooo0ol0oomg
ooooo`00O_ooool3o`3oo`Cooooo1Ooo0?l2ooooo`Ooo`3o0_ooool5ool0o`Cooooo0ooo0?mooooo
o`00Q_ooool3ool0o`Cooooo1Ooo0?l4ooooo`?oo`3oQoooool008kooooo0ooo0?n?ooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[ran1DTD]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A random tree with some other links", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(g1 = RandomTree[150];\)\), "\n", 
    \(\(ShowGraph[g1];\)\)}], "Input",
  CellLabel->"In[21]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0628823 1.01227 0.132029 2.64058 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.12046 .6103 m
.09153 .6103 L
s
.80473 .22783 m
.7758 .22846 L
s
.34158 .22668 m
.31278 .23408 L
s
.34158 .22668 m
.37037 .21928 L
s
.37076 .23571 m
.39967 .23345 L
s
.37076 .23571 m
.39968 .23386 L
s
.37076 .23571 m
.34183 .23777 L
s
.3997 .23667 m
.42863 .23592 L
s
.3997 .23667 m
.42863 .23605 L
s
.3997 .23667 m
.37077 .23736 L
s
.48649 .23473 m
.45756 .2354 L
s
.1529 .37472 m
.17336 .32136 L
s
.1529 .37472 m
.13245 .42809 L
s
.42806 .20777 m
.45689 .20146 L
s
.42806 .20777 m
.45691 .20201 L
s
.42806 .20777 m
.39922 .21352 L
s
.42806 .20777 m
.45693 .20256 L
s
.4278 .2008 m
.39909 .21025 L
s
.17336 .32136 m
.18443 .25163 L
s
.17336 .32136 m
.20009 .29248 L
s
.48649 .23571 m
.45756 .23612 L
s
.45689 .20146 m
.48573 .19516 L
s
.09153 .76124 m
.09153 .68577 L
s
.09153 .76124 m
.11199 .8146 L
s
.09153 .76124 m
.07107 .8146 L
s
.31254 .22918 m
.34108 .21689 L
s
.31254 .22918 m
.28399 .24147 L
s
.39968 .23386 m
.4286 .23201 L
s
.07107 .48146 m
.09153 .53482 L
s
.42637 .17484 m
.39797 .18926 L
s
.42637 .17484 m
.45475 .16026 L
s
.42637 .17484 m
.45478 .16057 L
s
.80473 .22783 m
.83366 .2272 L
s
.80473 .22783 m
.7758 .22846 L
s
.80473 .22783 m
.7758 .22846 L
s
.80473 .22783 m
.83366 .2272 L
s
.80473 .22783 m
.83366 .2272 L
s
.92046 .22531 m
.89152 .22594 L
s
.39909 .21025 m
.42783 .20162 L
s
.39909 .21025 m
.37037 .21928 L
s
.57328 .23287 m
.54435 .2335 L
s
.57328 .23287 m
.60221 .23223 L
s
.39916 .21188 m
.37037 .21928 L
s
.28288 .22696 m
.2552 .24887 L
s
.68901 .23034 m
.66008 .23097 L
s
.60221 .23218 m
.57328 .23283 L
s
.7758 .22846 m
.74687 .22909 L
s
.63115 .2316 m
.66008 .23097 L
s
.63115 .2316 m
.60221 .23223 L
s
.27493 .17974 m
.25088 .22167 L
s
.54435 .2335 m
.51542 .23413 L
s
.18443 .25163 m
.19551 .18191 L
s
.37076 .23653 m
.39969 .2353 L
s
.37076 .23653 m
.34183 .23777 L
s
.3997 .23695 m
.42863 .23653 L
s
.3997 .23695 m
.37077 .23736 L
s
.3418 .2353 m
.3707 .2316 L
s
.3418 .2353 m
.31291 .239 L
s
.09153 .68577 m
.09153 .6103 L
s
.11199 .8146 m
.13245 .86797 L
s
.31278 .23408 m
.28399 .24147 L
s
.71794 .22972 m
.68901 .23035 L
s
.71794 .22972 m
.74687 .22909 L
s
.71794 .22972 m
.74687 .22909 L
s
.48573 .19516 m
.51456 .18885 L
s
.48649 .23477 m
.45756 .2354 L
s
.48649 .23477 m
.51542 .23412 L
s
.48649 .23477 m
.51542 .23413 L
s
.48649 .23477 m
.51542 .23415 L
s
.42862 .23407 m
.39969 .2353 L
s
.71794 .22973 m
.68901 .23036 L
s
.80473 .22783 m
.7758 .22846 L
s
.60221 .23224 m
.57328 .23287 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.63115 .23161 L
s
.45756 .2354 m
.42863 .23605 L
s
.68901 .23034 m
.66008 .23097 L
s
.54435 .23352 m
.51542 .23415 L
s
.66008 .23097 m
.68901 .23033 L
s
.09153 .6103 m
.09153 .53482 L
s
.51456 .18885 m
.54339 .18255 L
s
.71794 .2297 m
.74687 .22907 L
s
.71794 .2297 m
.68901 .23033 L
s
.80473 .22783 m
.7758 .22846 L
s
.68901 .23035 m
.71794 .22972 L
s
.68901 .23035 m
.66008 .23098 L
s
.68901 .23035 m
.71794 .22972 L
s
.22682 .2636 m
.25088 .22167 L
s
.22682 .2636 m
.2552 .24887 L
s
.22682 .2636 m
.20009 .29248 L
s
.63115 .23161 m
.60221 .23224 L
s
.63115 .23161 m
.66008 .23097 L
s
.54435 .2335 m
.57328 .23287 L
s
.54435 .2335 m
.57328 .23287 L
s
.54435 .2335 m
.51542 .23413 L
s
.07107 .8146 m
.05061 .86797 L
s
.39922 .21352 m
.37037 .21928 L
s
.74687 .22909 m
.7758 .22846 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.57328 .23287 L
s
.63115 .23161 m
.66008 .23097 L
s
.57328 .23287 m
.60221 .23224 L
s
.83366 .2272 m
.86259 .22657 L
s
.83366 .2272 m
.86259 .22657 L
s
.83366 .2272 m
.86259 .22657 L
s
.45756 .23547 m
.42863 .23605 L
s
.42641 .17544 m
.39797 .18926 L
s
.34108 .21689 m
.36955 .20338 L
s
.34108 .21689 m
.3697 .20581 L
s
.71794 .22972 m
.74687 .22909 L
s
.28399 .24147 m
.2552 .24887 L
s
.28399 .24147 m
.31291 .239 L
s
.66008 .23098 m
.63115 .23161 L
s
.89152 .22594 m
.86259 .22657 L
s
.89152 .22594 m
.92046 .22531 L
s
.94939 .22468 m
.92046 .22531 L
s
.39797 .18926 m
.36955 .20338 L
s
.3707 .2316 m
.3996 .22789 L
s
.68901 .23035 m
.66008 .23098 L
s
.68901 .23035 m
.71794 .22973 L
s
.48319 .1463 m
.45478 .16057 L
s
.48319 .1463 m
.5116 .13203 L
s
.68901 .23035 m
.66008 .23098 L
s
.36955 .20338 m
.39805 .19047 L
s
.42656 .17757 m
.39805 .19047 L
s
.80473 .22783 m
.7758 .22846 L
s
.09153 .53482 m
.11199 .48146 L
s
.66008 .23097 m
.68901 .23034 L
s
.68901 .23035 m
.66008 .23098 L
s
.7758 .22846 m
.74687 .22909 L
s
.71794 .22973 m
.68901 .23036 L
s
.66008 .23098 m
.63115 .23161 L
s
.63115 .23161 m
.66008 .23098 L
s
.7758 .22846 m
.80473 .22783 L
s
.7758 .22846 m
.74687 .22909 L
s
.54435 .23348 m
.51542 .23412 L
s
.54435 .23348 m
.57328 .23283 L
s
.11199 .48146 m
.13245 .42809 L
s
.42863 .23653 m
.45756 .23612 L
s
.3997 .23722 m
.37077 .23736 L
s
.37077 .23736 m
.34183 .23777 L
s
.74687 .22909 m
.71794 .22972 L
s
.74687 .22909 m
.7758 .22846 L
s
.66008 .23098 m
.68901 .23036 L
s
.74687 .22909 m
.7758 .22846 L
s
.74687 .22909 m
.71794 .22972 L
s
.74687 .22909 m
.71794 .22972 L
s
.34183 .23777 m
.31291 .239 L
s
.025 w
.12046 .6103 Mdot
.80473 .22783 Mdot
.34158 .22668 Mdot
.37076 .23571 Mdot
.3997 .23667 Mdot
.48649 .23473 Mdot
.1529 .37472 Mdot
.42806 .20777 Mdot
.4278 .2008 Mdot
.17336 .32136 Mdot
.48649 .23571 Mdot
.39967 .23345 Mdot
.45689 .20146 Mdot
.09153 .76124 Mdot
.31254 .22918 Mdot
.39968 .23386 Mdot
.45691 .20201 Mdot
.07107 .48146 Mdot
.42637 .17484 Mdot
.80473 .22783 Mdot
.80473 .22783 Mdot
.92046 .22531 Mdot
.39909 .21025 Mdot
.57328 .23287 Mdot
.39916 .21188 Mdot
.28288 .22696 Mdot
.68901 .23034 Mdot
.42783 .20162 Mdot
.60221 .23218 Mdot
.7758 .22846 Mdot
.63115 .2316 Mdot
.27493 .17974 Mdot
.54435 .2335 Mdot
.18443 .25163 Mdot
.37076 .23653 Mdot
.3997 .23695 Mdot
.4286 .23201 Mdot
.3418 .2353 Mdot
.09153 .68577 Mdot
.11199 .8146 Mdot
.31278 .23408 Mdot
.71794 .22972 Mdot
.42863 .23592 Mdot
.48573 .19516 Mdot
.48649 .23477 Mdot
.42862 .23407 Mdot
.71794 .22973 Mdot
.80473 .22783 Mdot
.60221 .23224 Mdot
.45756 .2354 Mdot
.68901 .23034 Mdot
.54435 .23352 Mdot
.66008 .23097 Mdot
.09153 .6103 Mdot
.51456 .18885 Mdot
.71794 .2297 Mdot
.80473 .22783 Mdot
.68901 .23035 Mdot
.19551 .18191 Mdot
.22682 .2636 Mdot
.63115 .23161 Mdot
.54435 .2335 Mdot
.07107 .8146 Mdot
.54339 .18255 Mdot
.39922 .21352 Mdot
.13245 .86797 Mdot
.74687 .22909 Mdot
.60221 .23224 Mdot
.63115 .23161 Mdot
.74687 .22907 Mdot
.57328 .23287 Mdot
.7758 .22846 Mdot
.83366 .2272 Mdot
.37037 .21928 Mdot
.83366 .2272 Mdot
.45756 .23547 Mdot
.42641 .17544 Mdot
.34108 .21689 Mdot
.57328 .23287 Mdot
.71794 .22972 Mdot
.28399 .24147 Mdot
.66008 .23098 Mdot
.89152 .22594 Mdot
.94939 .22468 Mdot
.39969 .2353 Mdot
.25088 .22167 Mdot
.63115 .23161 Mdot
.39797 .18926 Mdot
.3707 .2316 Mdot
.68901 .23035 Mdot
.48319 .1463 Mdot
.68901 .23035 Mdot
.36955 .20338 Mdot
.42656 .17757 Mdot
.3996 .22789 Mdot
.80473 .22783 Mdot
.09153 .53482 Mdot
.66008 .23097 Mdot
.68901 .23035 Mdot
.86259 .22657 Mdot
.63115 .23161 Mdot
.39805 .19047 Mdot
.7758 .22846 Mdot
.71794 .22973 Mdot
.2552 .24887 Mdot
.05061 .86797 Mdot
.74687 .22909 Mdot
.20009 .29248 Mdot
.66008 .23098 Mdot
.60221 .23224 Mdot
.42863 .23605 Mdot
.86259 .22657 Mdot
.63115 .23161 Mdot
.74687 .22909 Mdot
.7758 .22846 Mdot
.54435 .23348 Mdot
.11199 .48146 Mdot
.66008 .23097 Mdot
.42863 .23653 Mdot
.3997 .23722 Mdot
.80473 .22783 Mdot
.45475 .16026 Mdot
.37077 .23736 Mdot
.45756 .23612 Mdot
.45478 .16057 Mdot
.92046 .22531 Mdot
.51542 .23412 Mdot
.45693 .20256 Mdot
.74687 .22909 Mdot
.83366 .2272 Mdot
.66008 .23098 Mdot
.57328 .23283 Mdot
.74687 .22909 Mdot
.5116 .13203 Mdot
.60221 .23223 Mdot
.7758 .22846 Mdot
.13245 .42809 Mdot
.74687 .22909 Mdot
.71794 .22973 Mdot
.86259 .22657 Mdot
.68901 .23036 Mdot
.51542 .23413 Mdot
.71794 .22972 Mdot
.68901 .23034 Mdot
.51542 .23415 Mdot
.34183 .23777 Mdot
.68901 .23033 Mdot
.31291 .239 Mdot
.3697 .20581 Mdot
.7758 .22846 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[21]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00TOooool3
o`0008cooooo002@ooooo`Go0000Roooool008oooooo1ol0002:ooooo`00Soooool7o`0008[ooooo
002?ooooo`Oo0000R_ooool008Wooooo0ol000001?oooooo0000o`000?ooool5o`0008_ooooo0028
ooooo`Go00001?ooool3o`0008cooooo0027ooooo`Oo0000T_ooool008Oooooo1ol0002Booooo`00
POooool3o`0000;ooooo2?l0002Booooo`00P?ooool6o`0000;ooooo1Ol0002Cooooo`00Ooooool7
o`0000?ooooo0ol0002Dooooo`00Ooooool7o`0009[ooooo001iooooo`?o00000_ooool8o`0009[o
oooo001=ooooo`?o0000:?ooool7o`000003ooooool0003o000000?o0000Voooool003Oooooo0ol0
000Booooo`Go00009_ooool7o`0000?ooooo0ol0000Fooooo`?o0000Poooool003Kooooo1Ol0000@
ooooo`Oo00009Oooool7o`0001_ooooo1Ol00022ooooo`00=Oooool7o`0000oooooo1ol0000Ooooo
o`?o00000oooool7o`0001Cooooo0ol00003ooooo`Oo0000POooool003Gooooo1ol0000?ooooo`Oo
00007_ooool>o`0001?ooooo1Ol00002ooooo`Oo0000POooool003Gooooo1ol0000@ooooo`Go0000
7_ooool7o`0000;ooooo1Ol0000=ooooo`?o00000oooool?o`00087ooooo000fooooo`Go00004_oo
ool3o`0001oooooo1ol00003ooooo`?o00001_ooool3o`0000Cooooo1Ol00002ooooo`Oo00000_oo
ool5o`0008;ooooo000gooooo`?o00004_ooool00ol0003oooooooooo`0Jooooo`?o00000oooool7
o`0000;ooooo1Ol00004ooooo`Go00000_ooool?o`0000?ooooo0ol00023ooooo`00=oooool00ol0
003oooooooooo`0Booooo`03o`000?oooooooooo01Wooooo1ol000000ooooooo0000o`000003o`00
00;ooooo1ol00002ooooo`oo00000_ooool5o`0008[ooooo000gooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool06Oooool7o`0000?ooooo0ol00003ooooo`Oo00000_ooool7o`000003
ooooool0003o000000Go00000oooool3o`0008_ooooo000gooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool06Oooool7o`0000;ooooo1Ol000000ooooooo0000o`00000?o`0000;ooooo
1Ol0002Booooo`00=oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01?ooooo0ol0
0004ooooo`Oo000000?oooooo`000?l000001Ol000000ooooooo0000o`000006o`000003ooooool0
003o000000Go00000oooool3o`0009?ooooo000fooooo`03o`000?oooooooooo00gooooo0ol00000
0ooooooo0000ooooo`0Cooooo`oo000000?oooooo`000?l000003Ol00003ooooo`Go0000V_ooool0
03Kooooo00?o0000ooooooooool03?ooool5o`0001Cooooo2?l000000ooooooo0000o`00000=o`00
00;ooooo1Ol00005ooooo`?o0000NOooool3o`0000Kooooo0ol00005ooooo`?o00003_ooool003Ko
oooo00?o0000ooooooooool02oooool7o`0000Gooooo0ol00005ooooo`?o00000oooool7o`0000;o
oooo1ol000000ooooooo0000o`000005o`0000?ooooo0ol0001@ooooo`?o00001_ooool3o`0000Go
oooo0ol00005ooooo`?o00001_ooool3o`0000Gooooo0ol00004ooooo`Go00001?ooool5o`0000?o
oooo1Ol0000=ooooo`00=_ooool00ol0003oooooooooo`0;ooooo`Oo00001?ooool5o`0000?ooooo
1Ol000000ooooooo0000o`000006o`000003ooooool0003o000000Ko00000_ooool5o`0000Cooooo
0ol0000Wooooo`?o00001Oooool3o`0000Kooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo
1Ol00004ooooo`Go00000oooool5o`0000?ooooo1Ol00004ooooo`Go00000oooool5o`0000;ooooo
1ol00002ooooo`Oo000000?oooooo`000?l000001Ol0000<ooooo`00=_ooool00ol0003ooooooooo
o`0;ooooo`Oo00000oooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001_l0
0002ooooo`Go00000_ooool7o`0000;ooooo1Ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0006ooooo`?o00001?ooool5o`0000?ooooo1Ol00004ooooo`Go00000oooool5o`0000?ooooo1Ol0
0002ooooo`Oo00000_ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol0
0002ooooo`Oo000000?oooooo`000?l000007_l0000<ooooo`00=Oooool00ol0003oooooooooo`0=
ooooo`Go00001?ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00002
ooooo`oo000000?oooooo`000?l000001Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00004
ooooo`Go00000_ooool7o`000003ooooool0003o000000Go00000_ooool7o`000003ooooool0003o
000000Go000000?oooooo`000?l000001Ol000000ooooooo0000o`00000_o`000003ooooool0003o
000000Go00000_ooool7o`000003ooooool0003o000000Go00003?ooool003Gooooo00?o0000oooo
ooooool03_ooool3o`0000Gooooo1ol000000ooooooo0000o`00000No`000003ooooool0003o0000
00Go00000_ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00002oooo
ocSo00000_ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00002oooo
o`Oo000000?oooooo`000?l000001Ol00002ooooo`Go00001?ooool5o`0000?ooooo1Ol0000=oooo
o`00=Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00Gooooo3ol000000ooooooo
0000o`000010o`000003ooooool0003o000000Go000000?oooooo`000?l000001Ol00002ooooo`Oo
000000?oooooo`000?l000001Ol000000ooooooo0000o`000005o`0000;ooooo1Ol00004ooooo`Go
00000oooool5o`0000?ooooo1Ol00004ooooo`Go00000oooool5o`0000Cooooo0ol00006ooooo`?o
00001Oooool3o`0000kooooo000eooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool0
0ol000000ooooooo0000o`00000`o`0000;ooooo1ol000000ooooooo0000o`000005o`000003oooo
ool0003o000000Go00000_ooool7o`0000;ooooo1Ol00003ooooo`Go00001?ooool5o`0000?ooooo
1Ol00003ooooo`Go00001?ooool3o`0000Kooooo0ol00005ooooo`?o00001Oooool3o`0000Kooooo
0ol00005ooooo`?o00009oooool003?ooooo0ol0000?ooooo`03o`000?oooooo000000Co00000_oo
ool7o`000003ooooool0003o000000Go00000_ooool7o`000003ooooool0003o000000Go000000?o
ooooo`000?l000001Ol00002ooooo`Go00001?ooool5o`0000?ooooo1Ol00003ooooo`Go00001?oo
ool5o`0000Cooooo0ol00005ooooo`?o00001_ooool3o`0000Gooooo0ol00005ooooo`?o0000FOoo
ool003;ooooo1Ol0000=ooooo`03o`000?oooooo000000ko00000_ooool5o`0000Cooooo1Ol00003
ooooo`Go00000oooool5o`0000Cooooo0ol00006ooooo`?o00001Oooool3o`0000Gooooo0ol00006
ooooo`?o0000P_ooool0037ooooo1ol0000<ooooo`03o`000?oooooo000000Ko00000_ooool5o`00
00Cooooo0ol00006ooooo`?o00001Oooool3o`0000Gooooo0ol0002/ooooo`00<Oooool7o`0000So
oooo1?l00002ooooo`Oo00000oooool3o`000<gooooo000aooooo`Oo00001oooool7o`000003oooo
ool0003o000000?o0000e?ooool003;ooooo1Ol00007ooooo`Oo00000oooool3o`000=Gooooo000c
ooooo`?o00002?ooool7o`000=_ooooo000cooooo`03o`000?oooooooooo00Sooooo1ol0003Koooo
o`00<oooool00ol0003oooooooooo`09ooooo`Go0000g?ooool003?ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooo`000002o`000=gooooo000cooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0h?ooool003?ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`3Q
ooooo`00<oooool00ol0003oooooooooo`02ooooo`Co0000i?ooool003?ooooo00Co0000oooooooo
oooooooo1Ol0003Tooooo`00<_ooool01?l0003oooooooooooooool7o`000>?ooooo000booooo`04
o`000?ooooooooooooooo`Oo0000hoooool003;ooooo00Co0000oooooooooooooooo1ol0003Soooo
o`00<_ooool00ol0003oooooooooo`02ooooo`Go0000i?ooool003;ooooo00Ko0000oooooooooooo
ooooo`000?ooool3o`000>Gooooo000booooo`04o`000?ooooooooooo`000>[ooooo000`ooooo`?o
000000?oooooo`000?ooool0j_ooool002oooooo1Ol0003/ooooo`00;_ooool7o`000>_ooooo000^
ooooo`Oo0000joooool002kooooo1ol0003[ooooo`00;oooool5o`000>cooooo000`ooooo`?o0000
kOooool0033ooooo00?o0000ooooooooool0kOooool002oooooo00?o0000ooooooooool0k_ooool0
02oooooo00?o0000ooooooooool0k_ooool002kooooo00?o0000ooooooooool0koooool002kooooo
00?o0000ooooooooool0koooool002kooooo00?o0000ooooooooool0koooool002gooooo00?o0000
ooooooooool0l?ooool002gooooo00?o0000ooooooooool0l?ooool002[ooooo0ol0003cooooo`00
:Oooool5o`000?;ooooo000Xooooo`Oo0000lOooool002Sooooo1ol0003aooooo`00:?ooool7o`00
0?7ooooo000Yooooo`Go0000l_ooool002[ooooo0ol0003cooooo`00:_ooool00ol0003ooooooooo
o`3cooooo`00:Oooool00ol0003oooooooooo`3dooooo`00:Oooool00ol0003oooooooooo`3doooo
o`00:Oooool00ol0003oooooooooo`3dooooo`00:?ooool00ol0003oooooooooo`3eooooo`00:?oo
ool00ol0003oooooooooo`3eooooo`00:?ooool00ol0003oooooooooo`3eooooo`009oooool00ol0
003oooooooooo`3fooooo`009?ooool4o`000?Sooooo000Sooooo`Go0000n?ooool002;ooooo1ol0
003gooooo`008_ooool7o`000?Oooooo000Rooooo`Oo0000moooool002?ooooo1Ol0003hooooo`00
9?ooool3o`000?Wooooo000Uooooo`03o`000?oooooooooo0?Sooooo000Tooooo`03o`000?oooooo
oooo0?Wooooo000Tooooo`03o`000?oooooooooo0?Wooooo000Tooooo`03o`000?oooooooooo0?Wo
oooo000Sooooo`03o`000?oooooooooo0?[ooooo000Sooooo`03o`000?oooooooooo0?[ooooo000R
ooooo`03o`000?oooooooooo0?_ooooo000Rooooo`03o`000?oooooooooo0?_ooooo000Rooooo`03
o`000?oooooooooo0?_ooooo000Cooooo`?o00002Oooool3o`000?kooooo000Booooo`Go00001ooo
ool5o`000?gooooo000Aooooo`Oo00001Oooool7o`000?cooooo000Aooooo`Oo00001Oooool7o`00
0?cooooo000Aooooo`Oo00001Oooool7o`000?cooooo000Booooo`Go00001oooool5o`000?gooooo
000Cooooo`?o00002Oooool3o`000?kooooo000Eooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool0o_ooool001Kooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`3ooooo
o`005_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo0?oooooo000Gooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool0ooooool1ooooo`005oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo0?oooooo0Oooool001Oooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`3oooooo`7ooooo000Hooooo`05o`000?ooooooooooooooool00000ooooool4
ooooo`006?ooool01Ol0003ooooooooooooooooo00000?oooooo1?ooool001Wooooo0ol0003ooooo
o`Gooooo000Hooooo`Go0000ooooool4ooooo`005oooool7o`000?oooooo0oooool001Oooooo1ol0
003oooooo`?ooooo000Gooooo`Oo0000ooooool3ooooo`006?ooool5o`000?oooooo1?ooool001Wo
oooo0ol0003oooooo`Gooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o
0000ooooooooool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Joooo
o`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4ooooo`00
6_ooool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?oo
ool001[ooooo00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3ooooo
o`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0
ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooo
oooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0003o
ooooooooo`3oooooo`Cooooo000Iooooo`?o00001Oooool3o`000?cooooo000Hooooo`Go00000ooo
ool5o`000?_ooooo000Gooooo`Oo000000?oooooo`000?l000001Ol0003jooooo`005oooool?o`00
0?[ooooo000Gooooo`Oo000000?oooooo`000?l000001Ol0003jooooo`006?ooool5o`0000?ooooo
1Ol0003kooooo`006Oooool3o`0000Gooooo0ol0003looooo`006_ooool00ol0003oooooooooo`3o
ooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000oooooooo
ool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oo
oooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0
003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo
00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000J
ooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4oooo
o`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo
1?ooool001Wooooo0ol0003oooooo`Gooooo000Hooooo`Go0000ooooool4ooooo`005oooool7o`00
0?oooooo0oooool001Oooooo1ol0003oooooo`?ooooo000Gooooo`Oo0000ooooool3ooooo`006?oo
ool5o`000?oooooo1?ooool001Wooooo0ol0003oooooo`Gooooo000Jooooo`03o`000?oooooooooo
0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0003ooooo
ooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000
ooooooooool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03
o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0ooooool4ooooo`006_oo
ool00ol0003oooooooooo`3oooooo`Cooooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool0
01[ooooo00?o0000ooooooooool0ooooool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Co
oooo000Jooooo`03o`000?oooooooooo0?oooooo1?ooool001[ooooo00?o0000ooooooooool0oooo
ool4ooooo`006_ooool00ol0003oooooooooo`3oooooo`Cooooo000Iooooo`?o0000ooooool5oooo
o`006?ooool5o`000?oooooo1?ooool001Oooooo1ol0003oooooo`?ooooo000Gooooo`Oo0000oooo
ool3ooooo`005oooool7o`000?oooooo0oooool001Sooooo1Ol0003oooooo`Cooooo000Iooooo`?o
0000ooooool5ooooo`006Oooool00ol0003oooooo`00003oooooo`Gooooo000Hooooo`05o`000?oo
ooooooooooooool00000ooooool4ooooo`006?ooool01Ol0003ooooooooooooooooo00000?oooooo
1?ooool001Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3oooooo`7ooooo000G
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0ooooool1ooooo`005oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo0?oooooo0Oooool001Kooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`3oooooo`005_ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo0?oooooo000Cooooo`?o00002Oooool3o`000?kooooo000Booooo`Go00001oooool5
o`000?gooooo000Aooooo`Oo00001Oooool7o`000?cooooo000Aooooo`Oo00001Oooool7o`000?co
oooo000Aooooo`Oo00001Oooool7o`000?cooooo000Booooo`Go00001oooool5o`000?gooooo000C
ooooo`?o00002Oooool3o`000?kooooo000Cooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool0o?ooool001;ooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`3kooooo`00
4_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo0?_ooooo000Booooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool0noooool0017ooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`3jooooo`004Oooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo
0?[ooooo000@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0nOooool0013ooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`3iooooo`004?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo0?Wooooo000=ooooo`?o00005?ooool3o`000?Wooooo000<oooo
o`Go00004_ooool5o`000?Sooooo000;ooooo`Oo00004?ooool7o`000?Oooooo000;ooooo`Oo0000
4?ooool7o`000?Oooooo000;ooooo`Oo00004?ooool7o`000?Oooooo000<ooooo`Go00004_ooool5
o`000?Sooooo000=ooooo`?o00005?ooool3o`000?Wooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo
003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0621151, -0.0500019, \
0.00344212, 0.00131954}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(newEdges = 
        Table[{Random[Integer, {1, 150}], 
            Random[Integer, {1, 150}]}, {15}];\)\), "\n", 
    \(\(ranTree1 = MakeSimple[AddEdges[g1, newEdges]];\)\), "\n", 
    \(\(ShowGraph[ranTree1];\)\)}], "Input",
  CellLabel->"In[23]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0628823 1.01227 0.132029 2.64058 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.12046 .6103 m
.09153 .6103 L
s
.80473 .22783 m
.7758 .22846 L
s
.34158 .22668 m
.31278 .23408 L
s
.34158 .22668 m
.37037 .21928 L
s
.37076 .23571 m
.39967 .23345 L
s
.37076 .23571 m
.39968 .23386 L
s
.37076 .23571 m
.34183 .23777 L
s
.3997 .23667 m
.42863 .23592 L
s
.3997 .23667 m
.42863 .23605 L
s
.3997 .23667 m
.37077 .23736 L
s
.48649 .23473 m
.45756 .2354 L
s
.1529 .37472 m
.17336 .32136 L
s
.1529 .37472 m
.13245 .42809 L
s
.42806 .20777 m
.45689 .20146 L
s
.42806 .20777 m
.45691 .20201 L
s
.42806 .20777 m
.39922 .21352 L
s
.42806 .20777 m
.45693 .20256 L
s
.4278 .2008 m
.39909 .21025 L
s
.17336 .32136 m
.18443 .25163 L
s
.17336 .32136 m
.20009 .29248 L
s
.48649 .23571 m
.48573 .19516 L
s
.48649 .23571 m
.45756 .23612 L
s
.45689 .20146 m
.48573 .19516 L
s
.09153 .76124 m
.09153 .68577 L
s
.09153 .76124 m
.11199 .8146 L
s
.09153 .76124 m
.07107 .8146 L
s
.31254 .22918 m
.34108 .21689 L
s
.31254 .22918 m
.28399 .24147 L
s
.39968 .23386 m
.4286 .23201 L
s
.07107 .48146 m
.09153 .53482 L
s
.42637 .17484 m
.39797 .18926 L
s
.42637 .17484 m
.45475 .16026 L
s
.42637 .17484 m
.45478 .16057 L
s
.80473 .22783 m
.83366 .2272 L
s
.80473 .22783 m
.7758 .22846 L
s
.80473 .22783 m
.7758 .22846 L
s
.80473 .22783 m
.83366 .2272 L
s
.80473 .22783 m
.83366 .2272 L
s
.92046 .22531 m
.89152 .22594 L
s
.92046 .22531 m
.54435 .23348 L
s
.39909 .21025 m
.42783 .20162 L
s
.39909 .21025 m
.37037 .21928 L
s
.57328 .23287 m
.54435 .2335 L
s
.57328 .23287 m
.60221 .23223 L
s
.39916 .21188 m
.37037 .21928 L
s
.39916 .21188 m
.66008 .23098 L
s
.28288 .22696 m
.2552 .24887 L
s
.68901 .23034 m
.66008 .23097 L
s
.60221 .23218 m
.57328 .23283 L
s
.7758 .22846 m
.74687 .22909 L
s
.63115 .2316 m
.66008 .23097 L
s
.63115 .2316 m
.60221 .23223 L
s
.27493 .17974 m
.25088 .22167 L
s
.54435 .2335 m
.51542 .23413 L
s
.18443 .25163 m
.19551 .18191 L
s
.37076 .23653 m
.39969 .2353 L
s
.37076 .23653 m
.34183 .23777 L
s
.3997 .23695 m
.42863 .23653 L
s
.3997 .23695 m
.37077 .23736 L
s
.4286 .23201 m
.07107 .8146 L
s
.3418 .2353 m
.3707 .2316 L
s
.3418 .2353 m
.31291 .239 L
s
.09153 .68577 m
.09153 .6103 L
s
.11199 .8146 m
.13245 .86797 L
s
.31278 .23408 m
.28399 .24147 L
s
.71794 .22972 m
.68901 .23035 L
s
.71794 .22972 m
.74687 .22909 L
s
.71794 .22972 m
.74687 .22909 L
s
.48573 .19516 m
.51456 .18885 L
s
.48649 .23477 m
.45756 .2354 L
s
.48649 .23477 m
.74687 .22909 L
s
.48649 .23477 m
.51542 .23412 L
s
.48649 .23477 m
.51542 .23413 L
s
.48649 .23477 m
.51542 .23415 L
s
.42862 .23407 m
.39969 .2353 L
s
.71794 .22973 m
.68901 .23036 L
s
.80473 .22783 m
.7758 .22846 L
s
.60221 .23224 m
.57328 .23287 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.63115 .23161 L
s
.45756 .2354 m
.42863 .23605 L
s
.68901 .23034 m
.66008 .23097 L
s
.54435 .23352 m
.51542 .23415 L
s
.66008 .23097 m
.68901 .23033 L
s
.09153 .6103 m
.09153 .53482 L
s
.51456 .18885 m
.54339 .18255 L
s
.71794 .2297 m
.74687 .22907 L
s
.71794 .2297 m
.11199 .48146 L
s
.71794 .2297 m
.68901 .23033 L
s
.80473 .22783 m
.7758 .22846 L
s
.68901 .23035 m
.57328 .23287 L
s
.68901 .23035 m
.71794 .22972 L
s
.68901 .23035 m
.66008 .23098 L
s
.68901 .23035 m
.71794 .22972 L
s
.22682 .2636 m
.25088 .22167 L
s
.22682 .2636 m
.2552 .24887 L
s
.22682 .2636 m
.20009 .29248 L
s
.63115 .23161 m
.60221 .23224 L
s
.63115 .23161 m
.66008 .23097 L
s
.54435 .2335 m
.57328 .23287 L
s
.54435 .2335 m
.57328 .23287 L
s
.54435 .2335 m
.51542 .23413 L
s
.07107 .8146 m
.05061 .86797 L
s
.39922 .21352 m
.37037 .21928 L
s
.13245 .86797 m
.83366 .2272 L
s
.74687 .22909 m
.7758 .22846 L
s
.60221 .23224 m
.63115 .23161 L
s
.60221 .23224 m
.57328 .23287 L
s
.63115 .23161 m
.66008 .23097 L
s
.57328 .23287 m
.60221 .23224 L
s
.83366 .2272 m
.86259 .22657 L
s
.83366 .2272 m
.86259 .22657 L
s
.83366 .2272 m
.86259 .22657 L
s
.45756 .23547 m
.42863 .23605 L
s
.42641 .17544 m
.39797 .18926 L
s
.34108 .21689 m
.36955 .20338 L
s
.34108 .21689 m
.3697 .20581 L
s
.71794 .22972 m
.74687 .22909 L
s
.28399 .24147 m
.2552 .24887 L
s
.28399 .24147 m
.31291 .239 L
s
.66008 .23098 m
.63115 .23161 L
s
.89152 .22594 m
.86259 .22657 L
s
.89152 .22594 m
.92046 .22531 L
s
.94939 .22468 m
.92046 .22531 L
s
.39797 .18926 m
.36955 .20338 L
s
.3707 .2316 m
.3996 .22789 L
s
.68901 .23035 m
.66008 .23098 L
s
.68901 .23035 m
.71794 .22973 L
s
.48319 .1463 m
.45478 .16057 L
s
.48319 .1463 m
.5116 .13203 L
s
.68901 .23035 m
.66008 .23098 L
s
.36955 .20338 m
.39805 .19047 L
s
.42656 .17757 m
.39805 .19047 L
s
.80473 .22783 m
.7758 .22846 L
s
.09153 .53482 m
.11199 .48146 L
s
.66008 .23097 m
.68901 .23034 L
s
.68901 .23035 m
.66008 .23098 L
s
.63115 .23161 m
.7758 .22846 L
s
.39805 .19047 m
.74687 .22909 L
s
.7758 .22846 m
.74687 .22909 L
s
.71794 .22973 m
.68901 .23036 L
s
.74687 .22909 m
.13245 .42809 L
s
.66008 .23098 m
.63115 .23161 L
s
.63115 .23161 m
.66008 .23098 L
s
.7758 .22846 m
.80473 .22783 L
s
.7758 .22846 m
.74687 .22909 L
s
.54435 .23348 m
.45475 .16026 L
s
.54435 .23348 m
.51542 .23412 L
s
.54435 .23348 m
.57328 .23283 L
s
.11199 .48146 m
.13245 .42809 L
s
.42863 .23653 m
.45756 .23612 L
s
.3997 .23722 m
.37077 .23736 L
s
.37077 .23736 m
.34183 .23777 L
s
.51542 .23412 m
.71794 .22973 L
s
.74687 .22909 m
.71794 .22972 L
s
.74687 .22909 m
.7758 .22846 L
s
.66008 .23098 m
.68901 .23036 L
s
.74687 .22909 m
.7758 .22846 L
s
.74687 .22909 m
.71794 .22972 L
s
.74687 .22909 m
.71794 .22972 L
s
.71794 .22973 m
.51542 .23415 L
s
.34183 .23777 m
.31291 .239 L
s
.025 w
.12046 .6103 Mdot
.80473 .22783 Mdot
.34158 .22668 Mdot
.37076 .23571 Mdot
.3997 .23667 Mdot
.48649 .23473 Mdot
.1529 .37472 Mdot
.42806 .20777 Mdot
.4278 .2008 Mdot
.17336 .32136 Mdot
.48649 .23571 Mdot
.39967 .23345 Mdot
.45689 .20146 Mdot
.09153 .76124 Mdot
.31254 .22918 Mdot
.39968 .23386 Mdot
.45691 .20201 Mdot
.07107 .48146 Mdot
.42637 .17484 Mdot
.80473 .22783 Mdot
.80473 .22783 Mdot
.92046 .22531 Mdot
.39909 .21025 Mdot
.57328 .23287 Mdot
.39916 .21188 Mdot
.28288 .22696 Mdot
.68901 .23034 Mdot
.42783 .20162 Mdot
.60221 .23218 Mdot
.7758 .22846 Mdot
.63115 .2316 Mdot
.27493 .17974 Mdot
.54435 .2335 Mdot
.18443 .25163 Mdot
.37076 .23653 Mdot
.3997 .23695 Mdot
.4286 .23201 Mdot
.3418 .2353 Mdot
.09153 .68577 Mdot
.11199 .8146 Mdot
.31278 .23408 Mdot
.71794 .22972 Mdot
.42863 .23592 Mdot
.48573 .19516 Mdot
.48649 .23477 Mdot
.42862 .23407 Mdot
.71794 .22973 Mdot
.80473 .22783 Mdot
.60221 .23224 Mdot
.45756 .2354 Mdot
.68901 .23034 Mdot
.54435 .23352 Mdot
.66008 .23097 Mdot
.09153 .6103 Mdot
.51456 .18885 Mdot
.71794 .2297 Mdot
.80473 .22783 Mdot
.68901 .23035 Mdot
.19551 .18191 Mdot
.22682 .2636 Mdot
.63115 .23161 Mdot
.54435 .2335 Mdot
.07107 .8146 Mdot
.54339 .18255 Mdot
.39922 .21352 Mdot
.13245 .86797 Mdot
.74687 .22909 Mdot
.60221 .23224 Mdot
.63115 .23161 Mdot
.74687 .22907 Mdot
.57328 .23287 Mdot
.7758 .22846 Mdot
.83366 .2272 Mdot
.37037 .21928 Mdot
.83366 .2272 Mdot
.45756 .23547 Mdot
.42641 .17544 Mdot
.34108 .21689 Mdot
.57328 .23287 Mdot
.71794 .22972 Mdot
.28399 .24147 Mdot
.66008 .23098 Mdot
.89152 .22594 Mdot
.94939 .22468 Mdot
.39969 .2353 Mdot
.25088 .22167 Mdot
.63115 .23161 Mdot
.39797 .18926 Mdot
.3707 .2316 Mdot
.68901 .23035 Mdot
.48319 .1463 Mdot
.68901 .23035 Mdot
.36955 .20338 Mdot
.42656 .17757 Mdot
.3996 .22789 Mdot
.80473 .22783 Mdot
.09153 .53482 Mdot
.66008 .23097 Mdot
.68901 .23035 Mdot
.86259 .22657 Mdot
.63115 .23161 Mdot
.39805 .19047 Mdot
.7758 .22846 Mdot
.71794 .22973 Mdot
.2552 .24887 Mdot
.05061 .86797 Mdot
.74687 .22909 Mdot
.20009 .29248 Mdot
.66008 .23098 Mdot
.60221 .23224 Mdot
.42863 .23605 Mdot
.86259 .22657 Mdot
.63115 .23161 Mdot
.74687 .22909 Mdot
.7758 .22846 Mdot
.54435 .23348 Mdot
.11199 .48146 Mdot
.66008 .23097 Mdot
.42863 .23653 Mdot
.3997 .23722 Mdot
.80473 .22783 Mdot
.45475 .16026 Mdot
.37077 .23736 Mdot
.45756 .23612 Mdot
.45478 .16057 Mdot
.92046 .22531 Mdot
.51542 .23412 Mdot
.45693 .20256 Mdot
.74687 .22909 Mdot
.83366 .2272 Mdot
.66008 .23098 Mdot
.57328 .23283 Mdot
.74687 .22909 Mdot
.5116 .13203 Mdot
.60221 .23223 Mdot
.7758 .22846 Mdot
.13245 .42809 Mdot
.74687 .22909 Mdot
.71794 .22973 Mdot
.86259 .22657 Mdot
.68901 .23036 Mdot
.51542 .23413 Mdot
.71794 .22972 Mdot
.68901 .23034 Mdot
.51542 .23415 Mdot
.34183 .23777 Mdot
.68901 .23033 Mdot
.31291 .239 Mdot
.3697 .20581 Mdot
.7758 .22846 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[23]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00TOooool3
o`0008cooooo002@ooooo`Go0000Roooool008oooooo1ol0002:ooooo`00Soooool7o`0008[ooooo
002?ooooo`Oo0000R_ooool008Wooooo0ol000001?oooooo0000o`000?ooool5o`0008_ooooo0028
ooooo`Go00001?ooool3o`0008cooooo0027ooooo`Oo0000T_ooool008Oooooo1ol0002Booooo`00
POooool3o`0000;ooooo2?l0002Booooo`00P?ooool6o`0000;ooooo1Ol0002Cooooo`00Ooooool7
o`0000?ooooo0ol0002Dooooo`00Ooooool7o`0009[ooooo001iooooo`?o00000_ooool8o`0009[o
oooo001=ooooo`?o0000:?ooool7o`000003ooooool0003o000000?o000000?oooooo`000?ooool0
V?ooool003Oooooo0ol0000Booooo`Go00009_ooool7o`0000?ooooo0ol00003ooooo`03o`000?oo
oooooooo013ooooo0ol00023ooooo`00=_ooool5o`00013ooooo1ol0000Uooooo`Oo00002_ooool2
o`0000oooooo1Ol00022ooooo`00=Oooool7o`0000oooooo1ol0000Oooooo`?o00000oooool7o`00
00cooooo00?o0000ooooooooool01Oooool3o`0000?ooooo1ol00021ooooo`00=Oooool7o`0000oo
oooo1ol0000Nooooo`ko00003Oooool00ol0003oooooooooo`03ooooo`Go00000_ooool7o`00087o
oooo000eooooo`Oo00004?ooool5o`0001kooooo1ol00002ooooo`Go00003Oooool3o`0000?ooooo
3ol00021ooooo`00=_ooool5o`0001;ooooo0ol0000Oooooo`Wo000000Coooooo`000?l0003o0000
1_ooool3o`0000Cooooo1Ol00002ooooo`Oo00000_ooool5o`0008;ooooo000gooooo`?o00004_oo
ool00ol0003oooooooooo`0Jooooo`?o00000oooool7o`0000;ooooo3_l00002ooooo`oo00000ooo
ool3o`0008?ooooo000gooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool06Oooool7
o`000003ooooool0003o000000?o00000_ooool7o`0000;ooooo5_l0002:ooooo`00=oooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo01Wooooo1ol00003ooooo`?o00000oooool7o`00
00;ooooo1ol000000ooooooo0000o`00000;o`0008_ooooo000gooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool06Oooool7o`0000;ooooo1Ol000000ooooooo0000o`00000?o`0000;o
oooo1Ol00004ooooo`[o0000Q?ooool003Oooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Cooooo`?o00001?ooool7o`000003ooooool0003o000000Go000000?oooooo`000?l00000
1_l000000ooooooo0000o`000005o`0000?ooooo0ol00006ooooo`03o`000?oooooooooo00Kooooo
2Ol0001kooooo`00=_ooool00ol0003oooooooooo`0=ooooo`?o000000?oooooo`000?ooool04ooo
ool?o`000003ooooool0003o000000go00000oooool5o`0000Gooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0>ooooo`Wo0000L_ooool003Kooooo00?o0000ooooooooool03?ooool5
o`0001Cooooo2?l000000ooooooo0000o`00000=o`0000;ooooo3_l00005ooooo`03o`000?oooooo
oooo00Oooooo0_l0000Gooooo`Wo0000Aoooool3o`0000Kooooo0ol00005ooooo`?o00003_ooool0
03Kooooo00?o0000ooooooooool02oooool7o`0000Gooooo0ol00005ooooo`?o00000oooool7o`00
00;ooooo1ol000000ooooooo0000o`000005o`0000?ooooo0ol0000:ooooo`co00001Oooool00ol0
003oooooooooo`0Mooooo`Wo00003?ooool3o`0000Kooooo0ol00005ooooo`?o00001Oooool3o`00
00Kooooo0ol00005ooooo`?o00001?ooool5o`0000Cooooo1Ol00003ooooo`Go00003Oooool003Ko
oooo00?o0000ooooooooool02oooool7o`0000Cooooo1Ol00003ooooo`Go000000?oooooo`000?l0
00001_l000000ooooooo0000o`000006o`0000;ooooo1Ol00004ooooo`?o00003oooool00ol0003o
ooooooooo`04ooooo`go00001?ooool3o`0000Gooooo0ol00006ooooo`?o00001Oooool3o`000003
ooooool0003o000000Oo00000_ooool5o`0000Cooooo1Ol00003ooooo`Go00000oooool5o`0000Co
oooo1Ol00003ooooo`Go00000_ooool7o`0000;ooooo1ol000000ooooooo0000o`000005o`0000co
oooo000fooooo`03o`000?oooooooooo00_ooooo1ol00003ooooo`Oo000000?oooooo`000?l00000
1Ol000000ooooooo0000o`000006o`0000;ooooo1Ol00002ooooo`Oo00000_ooool5o`0000Gooooo
0ol00005ooooo`?o00001Oooool3o`0000Cooooo00?o0000ooooool000000_l000000ooooooo0000
o`00000>o`0000Cooooo1Ol00003ooooo`Go00000oooool5o`000003ooooool0003o000000Oo0000
00?oooooo`000?l000001Ol000000ooooooo0000o`000005o`000003ooooool0003o000000Go0000
0_ooool7o`000003ooooool0003o000001ko00003?ooool003Gooooo00?o0000ooooooooool03Ooo
ool5o`0000Cooooo1ol000000ooooooo0000o`000005o`000003ooooool0003o000000Go00000_oo
ool?o`000003ooooool0003o000000Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00001?oo
ool5o`0000;ooooo1ol000000ooooooo0000o`00000>o`000003ooooool0003o000003oo000000?o
ooooo`000?l000001Ol00002ooooo`Oo000000?oooooo`000?l000001Ol0000<ooooo`00=Oooool0
0ol0003oooooooooo`0>ooooo`?o00001Oooool7o`000003ooooool0003o000001ko000000?ooooo
o`000?l000001Ol00002ooooo`Oo000000?oooooo`000?l000001Ol000000ooooooo0000o`000005
o`0000;ooooo@Ol000000ooooooo0000o`000005o`000003ooooool0003o000000Go00000_ooool7
o`000003ooooool0003o000000Go00000_ooool5o`0000Cooooo1Ol00003ooooo`Go00003Oooool0
03Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`05ooooo`oo000000?oooooo`00
0?l00000D?l00002ooooo`Oo000000?oooooo`000?l000001Ol000000ooooooo0000o`000006o`00
0003ooooool0003o000000Co00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool6o`00
00?ooooo1Ol00004ooooo`?o00001_ooool3o`0000Gooooo0ol0000>ooooo`00=Oooool00ol0003o
ooooooooo`0=ooooo`03o`000?oooooooooo00?o000000?oooooo`000?l00000<?l00002ooooo`Oo
000000?oooooo`000?l000001Ol000000ooooooo0000o`000005o`0000;ooooo1ol00002ooooo`Go
00000oooool5o`0000Cooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1?l00006ooooo`?o
00001Oooool3o`0000Gooooo0ol00003ooooo`03o`000?oooooooooo00?o00001Oooool3o`0002Oo
oooo000cooooo`?o00003oooool00ol0003oooooo`000004o`0000;ooooo1ol000000ooooooo0000
o`000005o`0000;ooooo1ol000000ooooooo0000o`000005o`000003ooooool0003o000000Go0000
0_ooool5o`0000Cooooo1Ol00003ooooo`Go00000oooool5o`0000Cooooo1Ol00004ooooo`?o0000
1Oooool3o`0000Kooooo0ol00005ooooo`?o00001?ooool4o`000004ooooool0003o0000o`0001oo
oooo00?o0000ooooooooool0<oooool003;ooooo1Ol0000=ooooo`03o`000?oooooo000000ko0000
0_ooool5o`0000Cooooo1Ol00003ooooo`Go00000oooool5o`0000Cooooo0ol00006ooooo`?o0000
1Oooool3o`0000Gooooo0ol00006ooooo`?o00008oooool2o`0000;ooooo0ol0000Qooooo`03o`00
0?oooooooooo03Cooooo000aooooo`Oo00003?ooool00ol0003oooooo`000006o`0000;ooooo1Ol0
0004ooooo`?o00001_ooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool00ol0003ooooooooo
o`14ooooo`;o000000Coooooo`000?l0003o00008oooool00ol0003oooooooooo`0eooooo`00<Ooo
ool7o`0000Sooooo1?l00002ooooo`Oo00000oooool3o`0002Gooooo00?o0000ooooooooool0@Ooo
ool6o`0002Gooooo00?o0000ooooooooool0=_ooool0037ooooo1ol00007ooooo`Oo000000?ooooo
o`000?l000000ol0000[ooooo`03o`000?oooooooooo043ooooo1Ol0000Wooooo`03o`000?oooooo
oooo03Oooooo000booooo`Go00001oooool7o`0000?ooooo0ol0000[ooooo`03o`000?oooooooooo
03kooooo1Ol0000Yooooo`03o`000?oooooooooo03Sooooo000cooooo`?o00002?ooool7o`00037o
oooo00?o0000ooooooooool0??ooool4o`0002_ooooo00?o0000ooooooooool0>Oooool003?ooooo
00?o0000ooooooooool02?ooool7o`00033ooooo00?o0000ooooooooool0>_ooool4o`0002cooooo
0_l0000looooo`00<oooool00ol0003oooooooooo`09ooooo`Go0000<Oooool00ol0003ooooooooo
o`0gooooo`Co0000;_ooool00ol0003oooooooooo`0looooo`00<oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooo000000;o0000<Oooool00ol0003oooooooooo`0eooooo`Co0000<?ooool0
0ol0003oooooooooo`0mooooo`00<oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo
03?ooooo00?o0000ooooooooool0<oooool5o`00037ooooo00?o0000ooooooooool0?_ooool003?o
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo033o
oooo1Ol0000cooooo`03o`000?oooooooooo03oooooo000cooooo`03o`000?oooooooooo00;ooooo
1?l0000fooooo`03o`000?oooooooooo02kooooo0ol000000ooooooo0000o`00000dooooo`03o`00
0?oooooooooo043ooooo000cooooo`04o`000?ooooooooooooooo`Go0000=Oooool00ol0003ooooo
ooooo`0/ooooo`?o000000Coooooo`000?l0003o0000=Oooool00ol0003oooooooooo`11ooooo`00
<_ooool01?l0003oooooooooooooool7o`0003Cooooo00?o0000ooooooooool0:Oooool3o`0000;o
oooo0_l0000gooooo`03o`000?oooooooooo04;ooooo000booooo`04o`000?ooooooooooooooo`Oo
0000<oooool00ol0003oooooooooo`0Wooooo`?o00000oooool2o`0003Sooooo00?o0000oooooooo
ool0@oooool003;ooooo00Co0000oooooooooooooooo1ol0000cooooo`03o`000?oooooooooo02Co
oooo0ol00003ooooo`?o0000>Oooool00ol0003oooooooooo`14ooooo`00<_ooool00ol0003ooooo
ooooo`02ooooo`Go0000<oooool00ol0003oooooooooo`0Rooooo`?o00001?ooool2o`0003_ooooo
00?o0000ooooooooool0AOooool003;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`0003?o
oooo00?o0000ooooooooool08?ooool3o`0000Cooooo0ol0000kooooo`;o0000B?ooool003;ooooo
00Co0000oooooooooooo0000>?ooool00ol0003oooooooooo`0Looooo`Co00001Oooool2o`0003go
oooo00?o0000ooooooooool0B?ooool0033ooooo0ol000000ooooooo0000ooooo`0gooooo`03o`00
0?oooooooooo01[ooooo0ol00007ooooo`;o0000?_ooool00ol0003oooooooooo`19ooooo`00;ooo
ool5o`0003Wooooo00?o0000ooooooooool05oooool3o`0000Oooooo0ol0000oooooo`03o`000?oo
oooooooo04[ooooo000^ooooo`Oo0000=oooool00ol0003oooooooooo`0Eooooo`?o00002?ooool2
o`00047ooooo00?o0000ooooooooool0Boooool002kooooo1ol0000fooooo`03o`000?oooooooooo
01?ooooo0ol00008ooooo`?o0000@_ooool00ol0003oooooooooo`1<ooooo`00;_ooool7o`0003Ko
oooo00?o0000ooooooooool04?ooool3o`0000Wooooo0_l00014ooooo`03o`000?oooooooooo04go
oooo000_ooooo`Go0000=_ooool00ol0003oooooooooo`0>ooooo`?o00002_ooool2o`0004Gooooo
00?o0000ooooooooool0C_ooool0033ooooo0ol0000fooooo`03o`000?oooooooooo00cooooo0ol0
000:ooooo`?o0000A_ooool00ol0003oooooooooo`1?ooooo`00<?ooool00ol0003oooooooooo`0f
ooooo`03o`000?oooooooooo00Wooooo0ol0000;ooooo`;o0000B?ooool00ol0003oooooooooo`1@
ooooo`00;oooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo00Oooooo0ol0000<oooo
o`;o0000B?ooool2o`0005?ooooo000_ooooo`03o`000?oooooooooo03Kooooo00?o0000oooooooo
ool01?ooool3o`0000cooooo0ol00019ooooo`03o`000?oooooooooo05?ooooo000^ooooo`03o`00
0?oooooooooo03Kooooo00Co0000oooooooooooooooo1?l0000=ooooo`;o0000Boooool00ol0003o
ooooooooo`1Dooooo`00;_ooool00ol0003oooooooooo`0eooooo`03o`000?oooooo000000;o0000
3_ooool3o`0004cooooo00?o0000ooooooooool0EOooool002kooooo00?o0000ooooooooool0=?oo
ool3o`0000oooooo0_l0001>ooooo`03o`000?oooooooooo05Kooooo000]ooooo`03o`000?oooooo
oooo03;ooooo1?l0000?ooooo`;o0000Coooool00ol0003oooooooooo`1Gooooo`00;Oooool00ol0
003oooooooooo`0_ooooo`?o00000_ooool00ol0003oooooooooo`0;ooooo`?o0000D?ooool00ol0
003oooooooooo`1Hooooo`00:_ooool3o`0002oooooo0ol00005ooooo`03o`000?oooooooooo00Wo
oooo0_l0001Booooo`03o`000?oooooooooo05Wooooo000Yooooo`Go0000:oooool3o`0000Oooooo
00?o0000ooooooooool01oooool3o`0005?ooooo00?o0000ooooooooool0F_ooool002Sooooo1ol0
000Wooooo`?o00002_ooool00ol0003oooooooooo`05ooooo`;o0000EOooool00ol0003ooooooooo
o`1Kooooo`00:?ooool7o`0002Cooooo0ol0000<ooooo`03o`000?oooooooooo00Cooooo0_l0001F
ooooo`03o`000?oooooooooo05cooooo000Xooooo`Oo00008Oooool3o`0000kooooo00?o0000oooo
ooooool00_ooool3o`0005Kooooo0_l0001Oooooo`00:Oooool5o`0001oooooo0ol0000Aooooo`03
o`000?oooooooooo00;o0000F?ooool00ol0003oooooooooo`1Oooooo`00:_ooool3o`0001gooooo
0ol0000Cooooo`04o`000?oooooo0000o`0005Wooooo00?o0000ooooooooool0H?ooool002[ooooo
00?o0000ooooooooool06Oooool4o`0001Gooooo0ol0001Jooooo`03o`000?oooooooooo067ooooo
000Yooooo`03o`000?oooooooooo01Oooooo0ol0000Gooooo`?o0000Foooool00ol0003ooooooooo
o`1Rooooo`00:Oooool00ol0003oooooooooo`0Dooooo`?o00005oooool3o`000003ooooool0003o
oooo05[ooooo00?o0000ooooooooool0Hoooool002Wooooo00?o0000ooooooooool04Oooool3o`00
01Sooooo0_l00004ooooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool0I?ooool002So
oooo00?o0000ooooooooool03oooool3o`0001Wooooo0_l00005ooooo`03o`000?oooooooooo05So
oooo00?o0000ooooooooool0IOooool002Sooooo00?o0000ooooooooool03?ooool3o`0001Wooooo
0ol00006ooooo`03o`000?oooooooooo05Sooooo00?o0000ooooooooool0I_ooool002Sooooo00?o
0000ooooooooool02Oooool3o`0001[ooooo0_l00009ooooo`03o`000?oooooooooo05Oooooo00?o
0000ooooooooool0Ioooool002Oooooo00?o0000ooooooooool01oooool3o`0001[ooooo0ol0000:
ooooo`03o`000?oooooooooo05Oooooo00?o0000ooooooooool0J?ooool002Cooooo1?l00006oooo
o`?o00006oooool2o`0000gooooo00?o0000ooooooooool0EOooool2o`0006_ooooo000Sooooo`Go
00000oooool3o`0001cooooo0_l0000>ooooo`03o`000?oooooooooo05Gooooo00?o0000oooooooo
ool0Joooool002;ooooo2Ol0000Looooo`?o00003oooool00ol0003oooooooooo`1Eooooo`03o`00
0?oooooooooo06cooooo000Rooooo`Oo00007?ooool2o`0001;ooooo00?o0000ooooooooool0E?oo
ool00ol0003oooooooooo`1]ooooo`008_ooool7o`0001[ooooo0_l0000Cooooo`03o`000?oooooo
oooo05Cooooo00?o0000ooooooooool0K_ooool002?ooooo1Ol0000Hooooo`?o00005Oooool00ol0
003oooooooooo`1Cooooo`03o`000?oooooooooo06oooooo000Tooooo`?o00005oooool2o`0001Oo
oooo00?o0000ooooooooool0Doooool00ol0003oooooooooo`1`ooooo`009Oooool00ol0003ooooo
ooooo`0Cooooo`?o00006?ooool00ol0003oooooooooo`1Cooooo`03o`000?oooooooooo077ooooo
000Tooooo`03o`000?oooooooooo01;ooooo0_l0000Kooooo`03o`000?oooooooooo05;ooooo00?o
0000ooooooooool0L_ooool002Cooooo00?o0000ooooooooool04?ooool2o`0001cooooo00?o0000
ooooooooool0D_ooool00ol0003oooooooooo`1cooooo`009?ooool00ol0003oooooooooo`0=oooo
o`?o00007Oooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo07Cooooo000Sooooo`03
o`000?oooooooooo00cooooo0_l0000Pooooo`03o`000?oooooooooo053ooooo0_l0001gooooo`00
8oooool00ol0003oooooooooo`09ooooo`?o00008Oooool00ol0003oooooooooo`1@ooooo`03o`00
0?oooooooooo07Oooooo000Rooooo`03o`000?oooooooooo00Sooooo0_l0000Tooooo`03o`000?oo
oooooooo04oooooo00?o0000ooooooooool0N?ooool002;ooooo00?o0000ooooooooool01_ooool2
o`0002Gooooo00?o0000ooooooooool0Coooool00ol0003oooooooooo`1iooooo`008_ooool00ol0
003oooooooooo`03ooooo`?o00009_ooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo
07[ooooo000Cooooo`?o00002Oooool3o`0000Cooooo0_l0000Yooooo`03o`000?oooooooooo04ko
oooo00?o0000ooooooooool0Noooool001;ooooo1Ol00007ooooo`Go000000?oooooo`000?l00000
:_ooool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo07cooooo000Aooooo`Oo00001Ooo
ool7o`0002cooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`1mooooo`004Oooool7
o`0000Gooooo1ol0000[ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0O_ooool0
017ooooo1ol00005ooooo`Oo0000:_ooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo
07oooooo000Booooo`Go00001oooool5o`0002_ooooo00?o0000ooooooooool0C?ooool00ol0003o
ooooooooo`20ooooo`004oooool3o`0000Wooooo0ol0000[ooooo`03o`000?oooooooooo04_ooooo
0_l00023ooooo`005Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02[ooooo00?o
0000ooooooooool0Boooool00ol0003oooooooooo`23ooooo`005_ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool0B_ooool00ol0003oooooooooo`24
ooooo`005_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo02[ooooo00?o0000oooo
ooooool0B_ooool00ol0003oooooooooo`25ooooo`005oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo02_ooooo00?o0000ooooooooool0BOooool00ol0003oooooooooo`26ooooo`00
5oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0
BOooool00ol0003oooooooooo`27ooooo`005oooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo02Wooooo00?o0000ooooooooool0BOooool00ol0003oooooooooo`28ooooo`006?ooool0
1Ol0003ooooooooooooooooo000002cooooo00?o0000ooooooooool0B?ooool00ol0003ooooooooo
o`29ooooo`006?ooool01Ol0003ooooooooooooooooo000002_ooooo00?o0000ooooooooool0B?oo
ool00ol0003oooooooooo`2:ooooo`006Oooool3o`0002_ooooo00?o0000ooooooooool0B?ooool0
0ol0003oooooooooo`2;ooooo`006?ooool5o`0002[ooooo00?o0000ooooooooool0Aoooool00ol0
003oooooooooo`2<ooooo`005oooool7o`0002Sooooo00?o0000ooooooooool0A_ooool2o`0008oo
oooo000Gooooo`Oo0000:?ooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo08oooooo
000Gooooo`Oo00009oooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo093ooooo000H
ooooo`Go00009oooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo097ooooo000Ioooo
o`?o0000:?ooool00ol0003oooooooooo`14ooooo`03o`000?oooooooooo09;ooooo000Jooooo`03
o`000?oooooooooo02Kooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`2Cooooo`00
6_ooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo04?ooooo00?o0000ooooooooool0
U?ooool001[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`13ooooo`03o`000?oo
oooooooo09Gooooo000Jooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0@oooool0
0ol0003oooooooooo`2Fooooo`006_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo
04;ooooo00?o0000ooooooooool0Uoooool001[ooooo00?o0000ooooooooool08oooool00ol0003o
ooooooooo`11ooooo`;o0000V_ooool001[ooooo00?o0000ooooooooool08_ooool00ol0003ooooo
ooooo`11ooooo`03o`000?oooooooooo09[ooooo000Jooooo`03o`000?oooooooooo02;ooooo00?o
0000ooooooooool0@?ooool00ol0003oooooooooo`2Kooooo`006_ooool00ol0003oooooooooo`0Q
ooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool0W?ooool001[ooooo00?o0000oooo
ooooool08Oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo09gooooo000Jooooo`03
o`000?oooooooooo023ooooo00?o0000ooooooooool0?oooool00ol0003oooooooooo`2Nooooo`00
6_ooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0
Woooool001[ooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0nooooo`03o`000?oo
oooooooo0:3ooooo000Jooooo`03o`000?oooooooooo01kooooo00?o0000ooooooooool0?_ooool0
0ol0003oooooooooo`2Qooooo`006_ooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo
03gooooo00?o0000ooooooooool0X_ooool001Wooooo0ol00005ooooo`?o00005_ooool00ol0003o
ooooooooo`0mooooo`03o`000?oooooooooo0:?ooooo000Hooooo`Go00000oooool5o`0001Cooooo
00?o0000ooooooooool0??ooool2o`000:Kooooo000Gooooo`Oo000000?oooooo`000?l000001Ol0
000Cooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0Y_ooool001Oooooo3ol0000B
ooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0Yoooool001Oooooo1ol000000ooo
oooo0000o`000005o`00017ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`2Xoooo
o`006?ooool5o`0000?ooooo1Ol0000Booooo`03o`000?oooooooooo03[ooooo00?o0000oooooooo
ool0ZOooool001Wooooo0ol00005ooooo`?o00004_ooool00ol0003oooooooooo`0jooooo`03o`00
0?oooooooooo0:[ooooo000Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0>Ooo
ool00ol0003oooooooooo`2[ooooo`006_ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooo
oooo03Wooooo00?o0000ooooooooool0[?ooool001[ooooo00?o0000ooooooooool05oooool00ol0
003oooooooooo`0iooooo`03o`000?oooooooooo0:gooooo000Jooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool0>?ooool00ol0003oooooooooo`2^ooooo`006_ooool00ol0003ooooo
ooooo`0Fooooo`03o`000?oooooooooo03Sooooo00?o0000ooooooooool0[oooool001[ooooo00?o
0000ooooooooool05_ooool00ol0003oooooooooo`0fooooo`;o0000/_ooool001[ooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo0;;ooooo000Joooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`2coooo
o`006_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo03Gooooo00?o0000oooooooo
ool0]?ooool001[ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0eooooo`03o`00
0?oooooooooo0;Gooooo000Jooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0=Ooo
ool00ol0003oooooooooo`2fooooo`006_ooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo03Cooooo00?o0000ooooooooool0]oooool001[ooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`0dooooo`03o`000?oooooooooo0;Sooooo000Jooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool0<oooool00ol0003oooooooooo`2iooooo`006Oooool3o`00017ooooo
00?o0000ooooooooool0<oooool00ol0003oooooooooo`2jooooo`006?ooool5o`0000oooooo00?o
0000ooooooooool0<oooool00ol0003oooooooooo`2kooooo`005oooool7o`0000kooooo00?o0000
ooooooooool0<Oooool2o`000;kooooo000Gooooo`Oo00003Oooool00ol0003oooooooooo`0aoooo
o`03o`000?oooooooooo0;kooooo000Gooooo`Oo00003Oooool00ol0003oooooooooo`0`ooooo`03
o`000?oooooooooo0;oooooo000Hooooo`Go00003Oooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo0<3ooooo000Iooooo`?o00003Oooool00ol0003oooooooooo`0`ooooo`03o`000?oo
oooooooo0<7ooooo000Jooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0;oooool0
0ol0003oooooooooo`32ooooo`006_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
02oooooo00?o0000ooooooooool0`oooool001[ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0_ooooo`03o`000?oooooooooo0<Cooooo000Jooooo`03o`000?oooooooooo00[ooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`35ooooo`006_ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool0a_ooool001[ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0]ooooo`03o`000?oooooooooo0<Oooooo000Joooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0;?ooool2o`000<[ooooo000Jooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`3:ooooo`00
6_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02_ooooo00?o0000ooooooooool0
boooool001[ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0[ooooo`03o`000?oo
oooooooo0<cooooo000Jooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0:oooool0
0ol0003oooooooooo`3=ooooo`006_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
02[ooooo00?o0000ooooooooool0c_ooool001[ooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0Zooooo`03o`000?oooooooooo0<oooooo000Jooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool0:Oooool00ol0003oooooooooo`3@ooooo`006_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool0dOooool001Wooooo0ol00003
ooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool0d_ooool001Sooooo1Ol00002oooo
o`03o`000?oooooooooo02Sooooo00?o0000ooooooooool0doooool001Oooooo2?l0000Yooooo`;o
0000e_ooool001Oooooo2?l0000Xooooo`03o`000?oooooooooo0=Kooooo000Gooooo`Oo0000:?oo
ool00ol0003oooooooooo`3Gooooo`006?ooool5o`0002Sooooo00?o0000ooooooooool0f?ooool0
01Wooooo1?l0000Wooooo`03o`000?oooooooooo0=Wooooo000Iooooo`03o`000?oooooo000002Oo
oooo00?o0000ooooooooool0f_ooool001Sooooo00Go0000ooooool0003oooooo`00000Uooooo`03
o`000?oooooooooo0=_ooooo000Hooooo`05o`000?oooooo0000ooooool000009?ooool00ol0003o
ooooooooo`3Looooo`005oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo023ooooo
00?o0000ooooooooool0gOooool001Oooooo00?o0000ooooool000000oooool00ol0003ooooooooo
o`0Oooooo`03o`000?oooooooooo0=kooooo000Gooooo`;o00001?ooool00ol0003oooooooooo`0M
ooooo`;o0000hOooool001Kooooo0_l00006ooooo`03o`000?oooooooooo01_ooooo00?o0000oooo
ooooool0hOooool001Kooooo0_l00006ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool0h_ooool001?ooooo1?l00008ooooo`?o00006?ooool00ol0003oooooooooo`3Sooooo`004_oo
ool5o`0000Oooooo1Ol0000Fooooo`03o`000?oooooooooo0>Cooooo000Aooooo`Oo00001Oooool7
o`0001Cooooo00?o0000ooooooooool0iOooool0017ooooo1ol00005ooooo`Oo00004oooool00ol0
003oooooooooo`3Vooooo`004Oooool7o`0000Gooooo1ol0000Booooo`03o`000?oooooooooo0>Oo
oooo000Booooo`Go00001oooool5o`0001;ooooo00?o0000ooooooooool0j?ooool001?ooooo0ol0
0009ooooo`?o00004_ooool00ol0003oooooooooo`3Yooooo`004oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool0j_ooool001;ooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0<ooooo`;o0000kOooool001;ooooo00?o0000oooooooo
ool03Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo0>gooooo000Booooo`03o`00
0?oooooooooo00gooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`3^ooooo`004Ooo
ool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0kooo
ool0017ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo0?3ooooo000@ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`3aooooo`004?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool0l_ooool0013ooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo0??ooooo000=ooooo`?o00005?ooool3o`0000;ooooo00?o
0000ooooooooool0m?ooool000cooooo1Ol0000Booooo`Ko0000moooool000_ooooo1ol0000@oooo
o`Oo0000moooool000_ooooo1ol0000@ooooo`Oo0000moooool000_ooooo1ol0000@ooooo`Oo0000
moooool000cooooo1Ol0000Booooo`Go0000n?ooool000gooooo0ol0000Dooooo`?o0000nOooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool0
0?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {0.0621151, -0.0500019, \
0.00344212, 0.00131954}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(\(ran2DTD = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] ranTree1, \[IndentingNewLine]PayoffMatrix \[Rule] 
            dtdPM, \[IndentingNewLine]StrategyColors \[Rule] 
            dtdColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[ExtractGraph[ran2DTD]];\)\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 -0.555477 2.41605 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.1161 .61366 m
.08684 .61366 L
s
.80823 .21953 m
.77897 .22018 L
s
.33976 .21834 m
.31063 .22596 L
s
.33976 .21834 m
.36888 .21072 L
s
.36927 .22765 m
.39852 .22532 L
s
.36927 .22765 m
.39853 .22574 L
s
.36927 .22765 m
.34002 .22977 L
s
.39855 .22864 m
.42781 .22786 L
s
.39855 .22864 m
.42781 .228 L
s
.39855 .22864 m
.36928 .22935 L
s
.48633 .22664 m
.45707 .22733 L
s
.14892 .3709 m
.16961 .31591 L
s
.14892 .3709 m
.12823 .4259 L
s
.42724 .19885 m
.4564 .19235 L
s
.42724 .19885 m
.45642 .19292 L
s
.42724 .19885 m
.39806 .20478 L
s
.42724 .19885 m
.45643 .19348 L
s
.42697 .19168 m
.39793 .20141 L
s
.16961 .31591 m
.18081 .24405 L
s
.16961 .31591 m
.19665 .28615 L
s
.48634 .22765 m
.48556 .18586 L
s
.48634 .22765 m
.45707 .22807 L
s
.4564 .19235 m
.48556 .18586 L
s
.08684 .76921 m
.08684 .69143 L
s
.08684 .76921 m
.10753 .8242 L
s
.08684 .76921 m
.06615 .8242 L
s
.31038 .22092 m
.33926 .20825 L
s
.31038 .22092 m
.28151 .23359 L
s
.39853 .22574 m
.42778 .22383 L
s
.06615 .48089 m
.08684 .53589 L
s
.42552 .16492 m
.3968 .17978 L
s
.42552 .16492 m
.45423 .1499 L
s
.42552 .16492 m
.45426 .15021 L
s
.80823 .21953 m
.83749 .21888 L
s
.80823 .21953 m
.77897 .22018 L
s
.80823 .21952 m
.77897 .22017 L
s
.80823 .21952 m
.83749 .21887 L
s
.80823 .21952 m
.83749 .21887 L
s
.92528 .21693 m
.89602 .21758 L
s
.92528 .21693 m
.54486 .22534 L
s
.39793 .20141 m
.42701 .19252 L
s
.39793 .20141 m
.36888 .21072 L
s
.57412 .22472 m
.54486 .22537 L
s
.57412 .22472 m
.60339 .22407 L
s
.398 .20309 m
.36888 .21072 L
s
.398 .20309 m
.66191 .22278 L
s
.28039 .21863 m
.25239 .24121 L
s
.69118 .22211 m
.66191 .22277 L
s
.60339 .22401 m
.57412 .22468 L
s
.77897 .22017 m
.7497 .22082 L
s
.63265 .22341 m
.66191 .22276 L
s
.63265 .22341 m
.60339 .22407 L
s
.27235 .16996 m
.24802 .21317 L
s
.54486 .22537 m
.5156 .22602 L
s
.18081 .24405 m
.19201 .1722 L
s
.36928 .2285 m
.39854 .22722 L
s
.36928 .2285 m
.34002 .22977 L
s
.39855 .22892 m
.42781 .2285 L
s
.39855 .22892 m
.36928 .22935 L
s
.42778 .22383 m
.06615 .8242 L
s
.33999 .22723 m
.36922 .22341 L
s
.33999 .22723 m
.31076 .23104 L
s
.08684 .69143 m
.08684 .61366 L
s
.10753 .8242 m
.12823 .8792 L
s
.31063 .22596 m
.28151 .23359 L
s
.72044 .22147 m
.69118 .22212 L
s
.72044 .22147 m
.7497 .22082 L
s
.72044 .22147 m
.7497 .22082 L
s
.48556 .18586 m
.51472 .17936 L
s
.48634 .22668 m
.45707 .22733 L
s
.48634 .22668 m
.7497 .22082 L
s
.48634 .22668 m
.5156 .22601 L
s
.48634 .22668 m
.5156 .22602 L
s
.48634 .22668 m
.5156 .22603 L
s
.4278 .22595 m
.39854 .22722 L
s
.72044 .22148 m
.69118 .22213 L
s
.80823 .21953 m
.77897 .22018 L
s
.60339 .22407 m
.57412 .22472 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.63265 .22342 L
s
.45707 .22733 m
.42781 .228 L
s
.69118 .22211 m
.66191 .22277 L
s
.54486 .22539 m
.5156 .22603 L
s
.66191 .22276 m
.69118 .22211 L
s
.08684 .61366 m
.08684 .53589 L
s
.51472 .17936 m
.54389 .17286 L
s
.72044 .22145 m
.7497 .2208 L
s
.72044 .22145 m
.10753 .48089 L
s
.72044 .22145 m
.69118 .22211 L
s
.80823 .21952 m
.77897 .22017 L
s
.69118 .22212 m
.57412 .22472 L
s
.69118 .22212 m
.72044 .22147 L
s
.69118 .22212 m
.66191 .22277 L
s
.69118 .22212 m
.72044 .22147 L
s
.22369 .25638 m
.24802 .21317 L
s
.22369 .25638 m
.25239 .24121 L
s
.22369 .25638 m
.19665 .28615 L
s
.63265 .22342 m
.60339 .22407 L
s
.63265 .22342 m
.66191 .22277 L
s
.54486 .22537 m
.57412 .22472 L
s
.54486 .22537 m
.57412 .22472 L
s
.54486 .22537 m
.5156 .22602 L
s
.06615 .8242 m
.04545 .8792 L
s
.39806 .20478 m
.36888 .21072 L
s
.12823 .8792 m
.83749 .21888 L
s
.7497 .22082 m
.77897 .22017 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.57412 .22472 L
s
.63265 .22342 m
.66191 .22277 L
s
.57412 .22472 m
.60339 .22407 L
s
.83749 .21888 m
.86676 .21823 L
s
.83749 .21887 m
.86676 .21822 L
s
.83749 .21887 m
.86676 .21822 L
s
.45707 .2274 m
.42781 .228 L
s
.42557 .16554 m
.3968 .17978 L
s
.33926 .20825 m
.36805 .19433 L
s
.33926 .20825 m
.3682 .19684 L
s
.72044 .22147 m
.7497 .22082 L
s
.28151 .23359 m
.25239 .24121 L
s
.28151 .23359 m
.31076 .23104 L
s
.66191 .22277 m
.63265 .22342 L
s
.89602 .21758 m
.86676 .21823 L
s
.89602 .21758 m
.92528 .21693 L
s
.95455 .21628 m
.92528 .21693 L
s
.3968 .17978 m
.36805 .19433 L
s
.36922 .22341 m
.39845 .21959 L
s
.69118 .22213 m
.66191 .22278 L
s
.69118 .22213 m
.72044 .22148 L
s
.48299 .13551 m
.45426 .15021 L
s
.48299 .13551 m
.51173 .1208 L
s
.69118 .22213 m
.66191 .22278 L
s
.36805 .19433 m
.39688 .18103 L
s
.42571 .16773 m
.39688 .18103 L
s
.80823 .21953 m
.77897 .22018 L
s
.08684 .53589 m
.10753 .48089 L
s
.66191 .22277 m
.69118 .22212 L
s
.69118 .22213 m
.66191 .22278 L
s
.63265 .22342 m
.77897 .22018 L
s
.39688 .18103 m
.7497 .22083 L
s
.77897 .22018 m
.7497 .22083 L
s
.72044 .22148 m
.69118 .22213 L
s
.7497 .22082 m
.12823 .4259 L
s
.66191 .22278 m
.63265 .22342 L
s
.63265 .22342 m
.66191 .22278 L
s
.77897 .22018 m
.80823 .21953 L
s
.77897 .22018 m
.7497 .22083 L
s
.54486 .22534 m
.45423 .1499 L
s
.54486 .22534 m
.5156 .22601 L
s
.54486 .22534 m
.57412 .22468 L
s
.10753 .48089 m
.12823 .4259 L
s
.42781 .2285 m
.45707 .22807 L
s
.39855 .2292 m
.36928 .22935 L
s
.36928 .22935 m
.34002 .22977 L
s
.5156 .22601 m
.72044 .22148 L
s
.7497 .22083 m
.72044 .22147 L
s
.7497 .22083 m
.77897 .22018 L
s
.66191 .22278 m
.69118 .22213 L
s
.7497 .22083 m
.77897 .22018 L
s
.7497 .22083 m
.72044 .22147 L
s
.7497 .22083 m
.72044 .22147 L
s
.72044 .22148 m
.5156 .22603 L
s
.34002 .22977 m
.31076 .23104 L
s
1 1 0 r
.025 w
.1161 .61366 Mdot
.80823 .21953 Mdot
0 1 1 r
.33976 .21834 Mdot
1 1 0 r
.36927 .22765 Mdot
0 .498 0 r
.39855 .22864 Mdot
.898 .898 .898 r
.48633 .22664 Mdot
0 1 0 r
.14892 .3709 Mdot
.42724 .19885 Mdot
.498 .498 .498 r
.42697 .19168 Mdot
.16961 .31591 Mdot
1 1 0 r
.48634 .22765 Mdot
0 1 1 r
.39852 .22532 Mdot
0 1 0 r
.4564 .19235 Mdot
0 1 1 r
.08684 .76921 Mdot
0 0 .498 r
.31038 .22092 Mdot
.498 .498 .498 r
.39853 .22574 Mdot
.498 0 0 r
.45642 .19292 Mdot
.498 .498 .498 r
.06615 .48089 Mdot
0 .498 0 r
.42552 .16492 Mdot
1 1 0 r
.80823 .21953 Mdot
1 0 0 r
.80823 .21952 Mdot
1 1 0 r
.92528 .21693 Mdot
0 .498 0 r
.39793 .20141 Mdot
.57412 .22472 Mdot
.898 .898 .898 r
.398 .20309 Mdot
.498 0 0 r
.28039 .21863 Mdot
.498 .498 .498 r
.69118 .22211 Mdot
0 1 1 r
.42701 .19252 Mdot
.498 .498 .498 r
.60339 .22401 Mdot
.77897 .22017 Mdot
.63265 .22341 Mdot
0 1 1 r
.27235 .16996 Mdot
.498 0 0 r
.54486 .22537 Mdot
.18081 .24405 Mdot
.898 .898 .898 r
.36928 .2285 Mdot
0 1 0 r
.39855 .22892 Mdot
.42778 .22383 Mdot
.898 .898 .898 r
.33999 .22723 Mdot
1 0 1 r
.08684 .69143 Mdot
0 1 0 r
.10753 .8242 Mdot
.31063 .22596 Mdot
1 1 0 r
.72044 .22147 Mdot
1 0 0 r
.42781 .22786 Mdot
1 1 0 r
.48556 .18586 Mdot
.498 0 0 r
.48634 .22668 Mdot
0 0 .498 r
.4278 .22595 Mdot
.72044 .22148 Mdot
0 1 0 r
.80823 .21953 Mdot
.60339 .22407 Mdot
0 .498 0 r
.45707 .22733 Mdot
1 1 0 r
.69118 .22211 Mdot
0 1 0 r
.54486 .22539 Mdot
1 0 0 r
.66191 .22276 Mdot
0 0 .498 r
.08684 .61366 Mdot
.898 .898 .898 r
.51472 .17936 Mdot
1 1 0 r
.72044 .22145 Mdot
1 0 0 r
.80823 .21952 Mdot
.898 .898 .898 r
.69118 .22212 Mdot
0 0 .498 r
.19201 .1722 Mdot
1 1 0 r
.22369 .25638 Mdot
0 0 .498 r
.63265 .22342 Mdot
0 1 0 r
.54486 .22537 Mdot
0 0 .498 r
.06615 .8242 Mdot
.498 .498 .498 r
.54389 .17286 Mdot
.39806 .20478 Mdot
0 .498 0 r
.12823 .8792 Mdot
1 0 0 r
.7497 .22082 Mdot
1 0 1 r
.60339 .22407 Mdot
0 1 1 r
.63265 .22342 Mdot
0 .498 0 r
.7497 .2208 Mdot
.498 .498 .498 r
.57412 .22472 Mdot
1 0 0 r
.77897 .22017 Mdot
.83749 .21888 Mdot
.498 0 0 r
.36888 .21072 Mdot
0 1 1 r
.83749 .21887 Mdot
1 0 1 r
.45707 .2274 Mdot
0 1 1 r
.42557 .16554 Mdot
.498 0 0 r
.33926 .20825 Mdot
1 1 0 r
.57412 .22472 Mdot
1 0 1 r
.72044 .22147 Mdot
0 1 0 r
.28151 .23359 Mdot
0 .498 0 r
.66191 .22277 Mdot
.898 .898 .898 r
.89602 .21758 Mdot
1 0 0 r
.95455 .21628 Mdot
0 1 0 r
.39854 .22722 Mdot
1 0 0 r
.24802 .21317 Mdot
0 0 .498 r
.63265 .22342 Mdot
1 1 0 r
.3968 .17978 Mdot
0 0 .498 r
.36922 .22341 Mdot
.498 .498 .498 r
.69118 .22213 Mdot
.898 .898 .898 r
.48299 .13551 Mdot
0 1 0 r
.69118 .22213 Mdot
0 .498 0 r
.36805 .19433 Mdot
0 1 1 r
.42571 .16773 Mdot
1 0 0 r
.39845 .21959 Mdot
.498 .498 .498 r
.80823 .21953 Mdot
0 .498 0 r
.08684 .53589 Mdot
.498 0 0 r
.66191 .22277 Mdot
1 1 0 r
.69118 .22213 Mdot
0 0 .498 r
.86676 .21822 Mdot
0 .498 0 r
.63265 .22342 Mdot
.498 .498 .498 r
.39688 .18103 Mdot
.498 0 0 r
.77897 .22018 Mdot
0 0 .498 r
.72044 .22148 Mdot
.898 .898 .898 r
.25239 .24121 Mdot
0 .498 0 r
.04545 .8792 Mdot
0 0 .498 r
.7497 .22082 Mdot
.898 .898 .898 r
.19665 .28615 Mdot
0 1 1 r
.66191 .22278 Mdot
.898 .898 .898 r
.60339 .22407 Mdot
1 0 0 r
.42781 .228 Mdot
0 .498 0 r
.86676 .21823 Mdot
.498 .498 .498 r
.63265 .22342 Mdot
.898 .898 .898 r
.7497 .22082 Mdot
0 .498 0 r
.77897 .22018 Mdot
0 1 1 r
.54486 .22534 Mdot
.498 0 0 r
.10753 .48089 Mdot
0 .498 0 r
.66191 .22277 Mdot
0 1 1 r
.42781 .2285 Mdot
0 .498 0 r
.39855 .2292 Mdot
.498 .498 .498 r
.80823 .21953 Mdot
1 0 1 r
.45423 .1499 Mdot
.498 0 0 r
.36928 .22935 Mdot
0 .498 0 r
.45707 .22807 Mdot
.898 .898 .898 r
.45426 .15021 Mdot
0 1 1 r
.92528 .21693 Mdot
.898 .898 .898 r
.5156 .22601 Mdot
1 0 1 r
.45643 .19348 Mdot
0 1 0 r
.7497 .22083 Mdot
.898 .898 .898 r
.83749 .21887 Mdot
.498 0 0 r
.66191 .22278 Mdot
0 0 .498 r
.57412 .22468 Mdot
0 1 0 r
.7497 .22083 Mdot
1 0 0 r
.51173 .1208 Mdot
.498 0 0 r
.60339 .22407 Mdot
1 0 0 r
.77897 .22018 Mdot
1 1 0 r
.12823 .4259 Mdot
0 1 0 r
.7497 .22083 Mdot
0 0 .498 r
.72044 .22148 Mdot
1 1 0 r
.86676 .21822 Mdot
0 .498 0 r
.69118 .22213 Mdot
.498 0 0 r
.5156 .22602 Mdot
0 .498 0 r
.72044 .22147 Mdot
0 1 0 r
.69118 .22212 Mdot
0 1 1 r
.5156 .22603 Mdot
0 1 0 r
.34002 .22977 Mdot
1 0 0 r
.69118 .22211 Mdot
1 0 1 r
.31076 .23104 Mdot
.3682 .19684 Mdot
.498 0 0 r
.77897 .22018 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[27]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00TOooool3ool008cooooo002@ooooo`Goo`00Roooool008oooooo1ooo
002:ooooo`00Soooool7ool008[ooooo0029ooooo`?oiNGU0oooool7ool008[ooooo0028ooooo`Go
iNGU0_l000000oooooooo`00ool00003ool008_ooooo0027ooooo`OoiNGU0oooool3ool008cooooo
0027ooooo`OoiNGUT_ooool008Oooooo1ooUiNFBooooo`00POooool3onGUi@;ooooo0_l00005onGU
iI?ooooo0020ooooo`GoiNGU00Co0000oooooooooooooooo0ooUiNFDooooo`00Ooooool7onGUiI[o
oooo001oooooo`OoiNGUV_ooool007Wooooo0ol0ool3ooooo`OoiNGUV_ooool004gooooo0ol0oolX
ooooo`Go0?oo0_l000000oooooooiNGUonGUi@03onGUi@03o`000?oooooooooo09Sooooo000foooo
o`?o001o4oooool5o`3oobKooooo1ol0ool3ooooo`?oiNGU0_ooool00ol0003oooooooooo`0Boooo
o`?oOgmoP_ooool003Gooooo1Ol007lAooooo`Oo0?oo9Oooool7o`3oo`Wooooo0_l0000Aooooo`Go
OgmoPOooool003Cooooo1ol007l@ooooo`Oo0?oo7_ooool3oooo00Cooooo1ol0ool;ooooo`03o`00
0?oooooooooo00Kooooo0ooUiND4ooooo`OoOgmoP?ooool003Cooooo1ol007l@ooooo`Oo0?oo7Ooo
ool01Oooo`3oOgmoogmoOomoOgooool000?o00001ol0ool<ooooo`03o`000?oooooooooo00Cooooo
1OoUiND3ooooo`OoOgmoP?ooool003Cooooo1ol007lAooooo`Go0?oo7Oooool00oooo`3oOgmoogmo
O`03ogmoO`04oooo0?l0003oooooooooo`Go0?oo3Oooool3oooo00?ooooo1ooUiND2o`0000OoOgmo
P?ooool003Gooooo1Ol007lCooooo`?o0?oo7_ooool7ogmoO`04o`000?ooooooooooooooo`?o0?oo
3Oooool5oooo00;ooooo1ooUiND3ooooo`GoOgmoPOooool003Kooooo0ol007lCooooo`03o`000?oo
oooooooo01Wooooo0ol0O`03ooooo`OoOgmo00Coooooo`000?l0003o00000ol0ool3o`0000;ooooo
0ooo0?l4ooooo`Ooool000?o0000onGUiOoUiND01OoUiND4ooooo`?oOgmoP_ooool003Kooooo00?o
0000ooooooooool04oooool00ol0003oooooooooo`0Hooooo`05o`1o0?oo0?ooo`3oool0ool0O`00
0_l00007ogmoO`?ooooo1Ol0ool2ooooo`03o`000?oo0?ooo`3o00?oo`3o0_l000000oooooooool0
oooo0005oooo0003o`000?ooooooiNGU00CoiNGUR_ooool003Kooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`0Hooooo`03o`1o0?oo0?ooo`3o00?oo`3o00?o07l0ooooooooool01Omo
Ogl3ooooo`Oo0?oo00?oooooool0oooo0?l01Ooo0?l2o`0000Ooool00_l000001?ooooooiNGUonGU
iOoUiNF;ooooo`00=_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo01Wooooo1ooo
0?l3ooooo`CoOgmo00?o07l0ooooooooool01ol0ool00oooooooo`3oool0o`05ool0o`?ooooo1Ooo
o`02ooooo`So0000R?ooool003Kooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0C
ooooo`?oO`000oooool7ool0o`;ooooo00?o07l0ogmoOomoOgl00omoOgl00ol0O`3o0000o`3oo`06
o`3oo`03o`000?oo0?ooo`3o00Goo`3o1?ooool3oooo00Gooooo0_l00004ooooo`Wo0000Ooooool0
03Kooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Cooooo`GoO`000_l00007ool0
o`;ooooo1omoOgl00ol0003o0?l0o`3oo`04o`3oo`03o`3o0?oooooooooo00Goo`3o1_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00[ooooo2?l0001gooooo`00=Oooool00ol0003o
ooooooooo`0>ooooo`?oo`0000?o0000ooooooooool04_ooool7ogl000;ooooo1Ooo0?l2ooooo`03
o`000?moOgooOgmo00GoOgmo00?o0000o`3o0?l0o`000ol0ool2o`3o00?ooooo0ooo0?l7ooooo`03
o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool9o`0006kooooo000eooooo`03o`00
0?oooooooooo00gooooo1Ooo0005ooooo`?oO`003?ooool7ogl00003oooooomo003oO`0000?oo`3o
0_mo0002o`0000OoOgmo0_ooool5o`3o00go000000?oooooo`000?ooool02Oooool00ol0003ooooo
ooooo`0Iooooo`So0000=Oooool3onGUi@Gooooo0oooo`06ooooo`?oiNGU1Oooool3o`3oo`Gooooo
0ooo000=ooooo`00=Oooool00ol0003oooooooooo`0<ooooo`Ooo`000oooool5ogl000Gooooo0ol0
07l3ooooo`OoO`0000?oooooogl00?mo00001Omo0003ooooo`05ogmoOooo003oo`00ool00?moOgl0
1?ooool3o`3o00kooooo3ol0000Qooooo`;o00000omo0004o`0000;ooooo0ooo0005ooooo`?o07l0
1_ooool3o`3o00Gooooo0omo0005ooooo`?oOgmo1Oooool5onGUi@?ooooo1Oooo`04ooooo`GoiNGU
0oooool5o`3oo`?ooooo1Ooo000<ooooo`00=Oooool00ol0003oooooooooo`0<ooooo`Ooo`000_oo
ool7ogl000?ooooo00Go001oo`3o0?l0o`3o0?l0o`00O`02o`000003o`3ooomo003oO`0000?oO`00
00Co0?ooo`000?mo003oO`000ol007l2ogl000?ooooo1Ooo0004ooooo`?o001o3oooool00ol0003o
ooooooooo`04ooooo`?o0?oo1?ooool2o`0000?o0?oo1Ol00003o`00O`;o00001?ooool3ogl000Go
oooo0omoOgl4ooooo`GoO`000oooool00ol0003oo`00ool00003ool000;o000000?oooooo`1o0?l0
O`000ol0O`04ooooo`Go0?l00oooool5ogl000?ooooo1OmoOgl3ooooo`OoiNGU00?ooooooooo0?oo
o`001Oooo`02ooooo`OoiNGU00?oooooo`3oool0ool01Ol0ool00oooooooo`00ool00005ool000_o
oooo000eooooo`03o`000?oooooooooo00cooooo1ooo0002ooooo`OoO`000_ooool00ol007oo0?l0
o`3o0003o`3o0004o`00Oooooooo0?ooo`3oo`?oO`000_l0ool2ooooo`05o`00Oomo003oO`00ogl0
0?l007l00oooool2ool000?o07l00_oo0002ooooo`05o`00Ool0oooo0?ooo`3oool007l01Oooool3
o`1o00Gooooo0omo0004ooooo`Go0?oo1?ooool5o`3oo`?ooooo1Ol007l01?oooooo0000o`000?l0
0005ogl000?o00001OmoOgl2ooooo`OoO`000_ooool7ool00003ooooool0O`3o07l000Go07l00_l0
0007o`3o0003oooooomo003oO`0000GoO`0000?oooooogmoOomoOgl01OmoOgl00ooooooo0000onGU
i@06onGUi@03o`000?ooo`3oool000Goool00_l00007onGUi@03o`000?l0oooo0?oo00Go0?oo00?o
0000ool00?oo00001Ooo000;ooooo`00=?ooool00ol0003oooooooooo`0>ooooo`Goo`000oooool7
ogl000;ooooo0_l0o`03ool0o`;o0?l000?o0000o`3ooooUiND00ol0o`001?oUiNGo0?ooooooool0
07l5ogl00004o`00Oooooooo0000ool000Go07l000?oo`00ooooool007l01Ol0ool01?l007oooooo
ooooooooool5o`1o00?ooooo1Omo0002ooooo`Oo0?oo0_ooool7o`3oo`03ooooool007oo001o00Go
001o0_ooool7ogl00003o`000?moOgooOgmo00GoOgmo00?o0000ogl00?mo00001Omo0002o`0000Oo
o`0000?o0000o`1o0?l0O`001Ol0O`02o`0000Oo0?l000?o0000ogl00?mo00001Omo00000ol0003o
OgmoogmoO`05ogmoO`;o00001ooUiND00oooooooool0oooo0005oooo00;ooooo1ooUiND00ooooooo
0?ooo`3oo`05o`3oo`03oooooooo003oo`0000Goo`002oooool003Cooooo00?o0000ooooooooool0
3oooool3ool000Cooooo00?o0000ogl00?l0o`000_l0o`001Omo003ooooooooooooooooo0?l000Go
o`3o00?o0?l0oooooooUiND01Ol0o`000ooUiNGo0000ogl00006ogl000;o00001ol0O`000ol0003o
0?ooo`3oo`05o`3oo`;ooooo1ol0O`000oooooooO`00ogl00005ogl00003o`000?l0oooo0?oo00Go
0?oo0_l00007o`3oo`03o`000?l007oo001o00Go001o0_l00007ogl00003o`000?moOgooOgmo00Go
Ogmo00?o0000ogl00?mo00001Omo0002o`0000Ooo`0000?o0000o`1o0?l0O`001Ol0O`02o`0000Oo
0?l000?oooooogl00?mo00001Omo00000oooooooOgmoogmoO`05ogmoO`;ooooo00?o0000onGUiOoU
iND00ooUiND3ooooo`Goool01?ooool5onGUi@?ooooo1Ol0ool3ooooo`Goo`003?ooool003Cooooo
00?o0000ooooooooool03_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo0?l000Co0?l0
0ol00007ool0o`03o`000?l0o`3o0?l000Go0?l000?o0000ogl00?mo00001Omo0002o`0000Oo07l0
00?o0000o`3oool0ool01Ol0ool2o`0000Oo07l000?o0000ogl00?mo00001Omo00000ol0003o0?oo
o`3oo`05o`3oo`;o00001ol0ool00ooooooo001oo`00O`05o`00O`;ooooo1omo00000oooooooOgmo
ogmoO`05ogmoO`;ooooo1Omo0004ooooo`Goo`0000?o0000ooooooooool01Ol0O`001?l0003ooooo
ooooooooool5o`3o00?ooooo1Omo0003ooooo`GoOgmo0_ooool00ol0003oooooooooo`03onGUi@Go
oooo0oooo`06ooooo`?oiNGU1Oooool3o`3oo`Gooooo0ooo000=ooooo`00=?ooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo00?oiNGU00?oooooo`000?ooool01ol0o`02o`0000Ooo`3o
00?oooooo`3o0?l0o`001Ol0o`000oooooooO`00ogl00005ogl000;ooooo1ol0O`000ooooooo0?oo
o`3oo`05o`3oo`;ooooo1ol0O`000oooooooO`00ogl00005ogl000;ooooo1Ol0ool4ooooo`Go0?oo
0oooool5o`00O`Cooooo1Omo0003ooooo`GoOgmo1?ooool3ogl000Kooooo0ooo0004ooooo`04o`00
0?l0O`3o07l0o`1o00Kooooo0ol0o`05ooooo`?oO`001Oooool3ogmoO`;ooooo00?o0000oooooooo
ool0<_ooool003;ooooo0omo000?ooooo`03o`000?ooooooiNGU00CoiNGU0_ooool7o`3o0003o`00
0?ooooooo`3o00Koo`3o00?oooooo`3o0?l0o`001Ol0o`02ooooo`GoO`001?ooool5o`1o00?ooooo
1Ol0ool4ooooo`Go07l00oooool5ogl000Cooooo0ol0ool6ooooo`?o0?oo1Oooool3o`00O`Kooooo
0omo0005ooooo`?oOgmo3?ooool3o`0000?ooooo0ol0000Nooooo`03o`000?oooooooooo03?ooooo
000aooooo`GoO`003Oooool00ol0003oooooonGUi@06onGUi@03o`000?l0o`3o0?l000Go0?l00ooo
ool5ool0o`?ooooo1Ol0o`04ooooo`?oO`001_ooool3o`1o00Cooooo00Co0000o`3oool0oooo0?oo
1_ooool3o`1o00Gooooo0omo000dooooo`;o00000oooool3o`00023ooooo00?o0000ooooooooool0
=?ooool0033ooooo1omo000<ooooo`03o`000?ooooooiNGU00KoiNGU0_ooool5o`3o00Gooooo0ooo
0?l5ooooo`?o0?l05?ooool00ol0003oooooooooo`15ooooo`;o00000_ooool3o`0002;ooooo00?o
0000ooooooooool0=Oooool0033ooooo1omo0008ooooo`?oool000?o0000ooooooooool01ooUiND3
ooooo`?o0?l09Oooool00ol0003oooooooooo`12ooooo`?o000000Coooooo`000?l0003o00009?oo
ool00ol0003oooooooooo`0fooooo`00<?ooool7ogl000Oooooo1Oooo`02o`000003oooooooUiNGo
iNGU00?oiNGU:oooool00ol0003oooooooooo`11ooooo`;o000000Coooooo`000?l0003o00009_oo
ool00ol0003oooooooooo`0gooooo`00<Oooool5ogl000Oooooo1oooo`03ooooo`?oiNGU:oooool0
0ol0003oooooooooo`10ooooo`Go0000:?ooool00ol0003oooooooooo`0hooooo`00<_ooool3ogl0
00Sooooo1oooo`0aooooo`03o`000?oooooooooo03gooooo1Ol0000Zooooo`03o`000?oooooooooo
03Wooooo000booooo`03o`000?oooooooooo00Sooooo1oooo`0`ooooo`03o`000?oooooooooo03co
oooo1?l0000/ooooo`03o`000?oooooooooo03[ooooo000booooo`03o`000?oooooooooo00Wooooo
1Oooo`0aooooo`03o`000?oooooooooo03[ooooo0ol0000^ooooo`03o`000?oooooooooo03_ooooo
000booooo`03o`000?oooooooooo00Wooooo00Co0000oooo0?ooo`3oool0<Oooool00ol0003ooooo
ooooo`0hooooo`?o0000;oooool2o`0003kooooo000booooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool0<_ooool00ol0003oooooooooo`0fooooo`?o0000<Oooool00ol0003ooooooooo
o`0nooooo`00<_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo03?ooooo00?o0000
ooooooooool0<oooool4o`0003;ooooo00?o0000ooooooooool0?oooool003;ooooo00?o0000oooo
ooooool00_ooool3onGUi@03ooooool0003ooooo03Cooooo00?o0000ooooooooool0<Oooool5o`00
03?ooooo00?o0000ooooooooool0@?ooool003;ooooo00Co0000oooooooooooooooo1OoUiNDfoooo
o`03o`000?oooooooooo02kooooo1Ol0000eooooo`03o`000?oooooooooo047ooooo000aooooo`04
o`000?ooooooooooooooo`OoiNGU=?ooool00ol0003oooooooooo`0/ooooo`?o000000?oooooo`00
0?l00000=_ooool00ol0003oooooooooo`12ooooo`00<Oooool01?l0003oooooooooooooool7onGU
iC?ooooo00?o0000ooooooooool0:_ooool3o`000004ooooool0003o0000o`0003Oooooo00?o0000
ooooooooool0@oooool0037ooooo00Co0000oooooooooooooooo1ooUiNDcooooo`03o`000?oooooo
oooo02Oooooo0ol00002ooooo`;o0000>Oooool00ol0003oooooooooo`14ooooo`00<Oooool00ol0
003oooooooooo`02ooooo`GoiNGU<oooool00ol0003oooooooooo`0Uooooo`?o00000oooool2o`00
03[ooooo00?o0000ooooooooool0AOooool0037ooooo00Ko0000ooooooooooooooooo`000?ooool3
onGUiCCooooo00?o0000ooooooooool08_ooool3o`0000?ooooo0ol0000kooooo`03o`000?oooooo
oooo04Kooooo000aooooo`04o`000?ooooooooooo`0003Sooooo00?o0000ooooooooool08?ooool3
o`0000Cooooo0_l0000mooooo`03o`000?oooooooooo04Oooooo000_ooooo`?oOgmo00?oooooo`00
0?ooool0=oooool00ol0003oooooooooo`0Nooooo`?o00001Oooool2o`0003kooooo00?o0000oooo
ooooool0B?ooool002kooooo1OmoOgliooooo`03o`000?oooooooooo01_ooooo0ol00005ooooo`?o
0000?oooool00ol0003oooooooooo`19ooooo`00;Oooool7ogmoOcOooooo00?o0000ooooooooool0
6Oooool3o`0000Kooooo0_l00010ooooo`;o0000C?ooool002gooooo1omoOglgooooo`03o`000?oo
oooooooo01Kooooo0ol00006ooooo`?o0000@Oooool00ol0003oooooooooo`1<ooooo`00;Oooool7
ogmoOcKooooo00?o0000ooooooooool05?ooool3o`0000Oooooo0_l00013ooooo`03o`000?oooooo
oooo04gooooo000^ooooo`GoOgmo=_ooool00ol0003oooooooooo`0Aooooo`Co00002?ooool2o`00
04Cooooo00?o0000ooooooooool0C_ooool002oooooo0omoOglgooooo`03o`000?oooooooooo00ko
oooo0ol00009ooooo`?o0000AOooool00ol0003oooooooooo`1?ooooo`00;oooool00ol0003ooooo
ooooo`0fooooo`03o`000?oooooooooo00cooooo0ol0000:ooooo`;o0000Aoooool00ol0003ooooo
ooooo`1@ooooo`00;_ooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo00Wooooo0ol0
000;ooooo`;o0000B?ooool00ol0003oooooooooo`1Aooooo`00;_ooool00ol0003oooooooooo`0f
ooooo`03o`000?oooooooooo00Oooooo0ol0000;ooooo`?o0000BOooool00ol0003oooooooooo`1B
ooooo`00;_ooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo00Gooooo0ol0000<oooo
o`;o0000Boooool00ol0003oooooooooo`1Cooooo`00;Oooool00ol0003oooooooooo`0fooooo`03
o`000?oooooooooo00;ooooo0ol0000=ooooo`;o0000C?ooool00ol0003oooooooooo`1Dooooo`00
;Oooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo00?o00003Oooool3o`0004gooooo
00?o0000ooooooooool0EOooool002cooooo00?o0000ooooooooool0=_ooool3o`0000kooooo0_l0
001?ooooo`03o`000?oooooooooo05Kooooo000/ooooo`03o`000?oooooooooo03?ooooo0ol0000?
ooooo`;o0000D?ooool00ol0003oooooooooo`1Gooooo`00;?ooool00ol0003oooooooooo`0`oooo
o`?o000000?oooooo`000?ooool03?ooool3o`00057ooooo00?o0000ooooooooool0F?ooool002Wo
oooo0ol0o`0`ooooo`?o00001?ooool00ol0003oooooooooo`09ooooo`;o0000D_ooool2o`0005_o
oooo000Xooooo`Go0?l0;?ooool3o`0000Kooooo00?o0000ooooooooool02?ooool2o`0005?ooooo
00?o0000ooooooooool0Foooool002Oooooo1ol0o`0Xooooo`?o00002Oooool00ol0003ooooooooo
o`05ooooo`?o0000E?ooool00ol0003oooooooooo`1Looooo`009oooool7o`3o02Gooooo0ol0000;
ooooo`03o`000?oooooooooo00Cooooo0_l0001Fooooo`03o`000?oooooooooo05gooooo000Woooo
o`Oo0?l08_ooool3o`0000gooooo00?o0000ooooooooool00oooool2o`0005Oooooo00?o0000oooo
ooooool0G_ooool002Sooooo1Ol0o`0Pooooo`?o00004?ooool00ol0003oooooooooo`03o`0005So
oooo00?o0000ooooooooool0Goooool002Wooooo0ol0o`0Nooooo`?o00004_ooool01?l0003ooooo
o`000?l0001Jooooo`03o`000?oooooooooo063ooooo000Yooooo`03o`000?oooooooooo01_ooooo
0ol0000Dooooo`?o0000Foooool00ol0003oooooooooo`1Qooooo`00:?ooool00ol0003ooooooooo
o`0Iooooo`?o00005Oooool3o`0005cooooo00?o0000ooooooooool0H_ooool002Sooooo00?o0000
ooooooooool05_ooool3o`0001Kooooo0_l000000ooooooo0000ooooo`1Kooooo`03o`000?oooooo
oooo06?ooooo000Xooooo`03o`000?oooooooooo01?ooooo0ol0000Fooooo`?o00000oooool00ol0
003oooooooooo`1Iooooo`03o`000?oooooooooo06Cooooo000Wooooo`03o`000?oooooooooo017o
oooo0ol0000Gooooo`;o00001Oooool00ol0003oooooooooo`1Iooooo`03o`000?oooooooooo06Go
oooo000Wooooo`03o`000?oooooooooo00kooooo0ol0000Hooooo`;o00001_ooool00ol0003ooooo
ooooo`1Iooooo`03o`000?oooooooooo06Kooooo000Vooooo`03o`000?oooooooooo00cooooo0ol0
000Hooooo`?o00002?ooool00ol0003oooooooooo`1Gooooo`;o0000JOooool002Kooooo00?o0000
ooooooooool02Oooool3o`0001Wooooo0_l0000:ooooo`03o`000?oooooooooo05Oooooo00?o0000
ooooooooool0JOooool002Kooooo00?o0000ooooooooool01_ooool3o`0001[ooooo0_l0000<oooo
o`03o`000?oooooooooo05Kooooo00?o0000ooooooooool0J_ooool002?ooooo0oooo`06ooooo`?o
00006_ooool3o`0000gooooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`1[ooooo`00
8_ooool5oooo00;ooooo0ol0000Kooooo`;o00003oooool00ol0003oooooooooo`1Fooooo`03o`00
0?oooooooooo06cooooo000Qooooo`Ooool000?o0000ooooooooool06_ooool2o`00017ooooo00?o
0000ooooooooool0EOooool00ol0003oooooooooo`1]ooooo`008Oooool7oooo01[ooooo0ol0000B
ooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool0K_ooool0027ooooo1oooo`0Hoooo
o`;o00005Oooool00ol0003oooooooooo`1Dooooo`03o`000?oooooooooo06oooooo000Rooooo`Go
ool05oooool2o`0001Kooooo00?o0000ooooooooool0E?ooool00ol0003oooooooooo`1`ooooo`00
8oooool3oooo01Gooooo0ol0000Gooooo`03o`000?oooooooooo05Cooooo00?o0000ooooooooool0
LOooool002?ooooo00?o0000ooooooooool04oooool2o`0001[ooooo00?o0000ooooooooool0Dooo
ool00ol0003oooooooooo`1booooo`008_ooool00ol0003oooooooooo`0Booooo`;o00006oooool0
0ol0003oooooooooo`1Cooooo`03o`000?oooooooooo07?ooooo000Rooooo`03o`000?oooooooooo
00oooooo0ol0000Mooooo`03o`000?oooooooooo05;ooooo00?o0000ooooooooool0M?ooool002;o
oooo00?o0000ooooooooool03Oooool2o`0001oooooo00?o0000ooooooooool0D_ooool00ol0003o
ooooooooo`1eooooo`008Oooool00ol0003oooooooooo`0<ooooo`;o00008?ooool00ol0003ooooo
ooooo`1Aooooo`;o0000N?ooool0027ooooo00?o0000ooooooooool02Oooool3o`0002;ooooo00?o
0000ooooooooool0D?ooool00ol0003oooooooooo`1hooooo`008?ooool00ol0003oooooooooo`08
ooooo`;o00009?ooool00ol0003oooooooooo`1@ooooo`03o`000?oooooooooo07Wooooo000Poooo
o`03o`000?oooooooooo00Kooooo0_l0000Vooooo`03o`000?oooooooooo04oooooo00?o0000oooo
ooooool0N_ooool0023ooooo00?o0000ooooooooool00oooool3o`0002Oooooo00?o0000oooooooo
ool0Coooool00ol0003oooooooooo`1kooooo`004Oooool3ogmoO`Wooooo0omo0004ooooo`;o0000
:Oooool00ol0003oooooooooo`1?ooooo`03o`000?oooooooooo07cooooo000@ooooo`GoOgmo1ooo
ool5ogl00003ooooool0003o000002_ooooo00?o0000ooooooooool0C_ooool00ol0003ooooooooo
o`1mooooo`003oooool7ogmoO`Gooooo1omo000/ooooo`03o`000?oooooooooo04kooooo00?o0000
ooooooooool0O_ooool000oooooo1omoOgl5ooooo`OoO`00;?ooool00ol0003oooooooooo`1=oooo
o`03o`000?oooooooooo07oooooo000?ooooo`OoOgmo1Oooool7ogl002_ooooo00?o0000oooooooo
ool0COooool00ol0003oooooooooo`20ooooo`004?ooool5ogmoO`Oooooo1Omo000[ooooo`03o`00
0?oooooooooo04gooooo00?o0000ooooooooool0POooool0017ooooo0omoOgl9ooooo`?oO`00;?oo
ool00ol0003oooooooooo`1<ooooo`03o`000?oooooooooo08;ooooo000Cooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1<ooooo`03o`000?oooooo
oooo08?ooooo000Dooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;?ooool00ol0
003oooooooooo`1:ooooo`;o0000Q_ooool001Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0[ooooo`03o`000?oooooooooo04[ooooo00?o0000ooooooooool0Q_ooool001Gooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`0[ooooo`03o`000?oooooooooo04[ooooo
00?o0000ooooooooool0Qoooool001Gooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`0[ooooo`03o`000?oooooooooo04Wooooo00?o0000ooooooooool0R?ooool001Gooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo04Wooooo00?o0000
ooooooooool0ROooool001Kooooo00Go0000ooooooooooooooooo`00000]ooooo`03o`000?oooooo
oooo04Sooooo00?o0000ooooooooool0R_ooool001Kooooo00Go0000ooooooooooooooooo`00000/
ooooo`03o`000?oooooooooo04Sooooo00?o0000ooooooooool0Roooool001Oooooo0ol0O`0/oooo
o`03o`000?oooooooooo04Sooooo00?o0000ooooooooool0S?ooool001Kooooo1Ol0O`0[ooooo`03
o`000?oooooooooo04Oooooo00?o0000ooooooooool0SOooool001Gooooo1ol0O`0Yooooo`03o`00
0?oooooooooo04Oooooo00?o0000ooooooooool0S_ooool001Gooooo1ol0O`0Xooooo`03o`000?oo
oooooooo04Oooooo00?o0000ooooooooool0Soooool001Gooooo1ol0O`0Xooooo`03o`000?oooooo
oooo04Kooooo00?o0000ooooooooool0T?ooool001Kooooo1Ol0O`0Xooooo`03o`000?oooooooooo
04Kooooo00?o0000ooooooooool0TOooool001Oooooo0ol0O`0Yooooo`03o`000?oooooooooo04Go
oooo00?o0000ooooooooool0T_ooool001Sooooo00?o0000ooooooooool09oooool00ol0003ooooo
ooooo`14ooooo`;o0000UOooool001Sooooo00?o0000ooooooooool09_ooool00ol0003ooooooooo
o`14ooooo`03o`000?oooooooooo09Gooooo000Hooooo`03o`000?oooooooooo02Kooooo00?o0000
ooooooooool0@oooool00ol0003oooooooooo`2Fooooo`006?ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo04?ooooo00?o0000ooooooooool0Uoooool001Sooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`12ooooo`03o`000?oooooooooo09Sooooo000Hooooo`03o`00
0?oooooooooo02Cooooo00?o0000ooooooooool0@_ooool00ol0003oooooooooo`2Iooooo`006?oo
ool00ol0003oooooooooo`0Sooooo`03o`000?oooooooooo04;ooooo00?o0000ooooooooool0V_oo
ool001Sooooo00?o0000ooooooooool08oooool00ol0003oooooooooo`11ooooo`03o`000?oooooo
oooo09_ooooo000Hooooo`03o`000?oooooooooo02;ooooo00?o0000ooooooooool0@Oooool00ol0
003oooooooooo`2Looooo`006?ooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo043o
oooo00?o0000ooooooooool0WOooool001Sooooo00?o0000ooooooooool08Oooool00ol0003ooooo
ooooo`10ooooo`03o`000?oooooooooo09kooooo000Hooooo`03o`000?oooooooooo023ooooo00?o
0000ooooooooool0@?ooool00ol0003oooooooooo`2Oooooo`006?ooool00ol0003oooooooooo`0P
ooooo`03o`000?oooooooooo03oooooo00?o0000ooooooooool0X?ooool001Sooooo00?o0000oooo
ooooool07oooool00ol0003oooooooooo`0oooooo`03o`000?oooooooooo0:7ooooo000Hooooo`03
o`000?oooooooooo01oooooo00?o0000ooooooooool0?Oooool2o`000:Cooooo000Hooooo`03o`00
0?oooooooooo01kooooo00?o0000ooooooooool0?Oooool00ol0003oooooooooo`2Tooooo`005ooo
ool3o`00O`Kooooo0oooo`0Eooooo`03o`000?oooooooooo03gooooo00?o0000ooooooooool0YOoo
ool001Kooooo1Ol007l4ooooo`Goool05?ooool00ol0003oooooooooo`0looooo`03o`000?oooooo
oooo0:Kooooo000Eooooo`Oo001o0_ooool7oooo01;ooooo00?o0000ooooooooool0??ooool00ol0
003oooooooooo`2Wooooo`005Oooool7o`00O`;o00001oooo`0Booooo`03o`000?oooooooooo03_o
oooo00?o0000ooooooooool0Z?ooool001Gooooo1ol007l2ooooo`Ooool04Oooool00ol0003ooooo
ooooo`0kooooo`03o`000?oooooooooo0:Wooooo000Fooooo`Go001o1?ooool5oooo017ooooo00?o
0000ooooooooool0>oooool00ol0003oooooooooo`2Zooooo`005oooool3o`00O`Kooooo0oooo`0B
ooooo`03o`000?oooooooooo03[ooooo00?o0000ooooooooool0Zoooool001Sooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0jooooo`03o`000?oooooooooo0:cooooo000Hooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`2]ooooo`00
6?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo03Wooooo00?o0000ooooooooool0
[_ooool001Sooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0iooooo`03o`000?oo
oooooooo0:oooooo000Hooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool0=oooool2
o`000;;ooooo000Hooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool0=oooool00ol0
003oooooooooo`2booooo`006?ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo03Ko
oooo00?o0000ooooooooool0/oooool001Sooooo00?o0000ooooooooool05Oooool00ol0003ooooo
ooooo`0fooooo`03o`000?oooooooooo0;Cooooo000Hooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool0=_ooool00ol0003oooooooooo`2eooooo`006?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo03Gooooo00?o0000ooooooooool0]_ooool001Sooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`0eooooo`03o`000?oooooooooo0;Oooooo000Hooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`2hooooo`00
6?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo03Cooooo00?o0000ooooooooool0
^Oooool001Sooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0dooooo`03o`000?oo
oooooooo0;[ooooo000Hooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0<oooool0
0ol0003oooooooooo`2kooooo`005oooool3ool0oa7ooooo00?o0000ooooooooool0<oooool00ol0
003oooooooooo`2looooo`005_ooool5ool0o`oooooo00?o0000ooooooooool0<oooool00ol0003o
ooooooooo`2mooooo`005Oooool7ool0o`kooooo00?o0000ooooooooool0<_ooool00ol0003ooooo
ooooo`2nooooo`005Oooool7ool0o`gooooo00?o0000ooooooooool0<Oooool2o`000<7ooooo000E
ooooo`Ooo`3o3Oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo0<7ooooo000Foooo
o`Goo`3o3Oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo0<;ooooo000Gooooo`?o
o`3o3Oooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo0<?ooooo000Hooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`34ooooo`006?oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo02oooooo00?o0000ooooooooool0aOoo
ool001Sooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo0<Kooooo000Hooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool0;_ooool00ol0
003oooooooooo`37ooooo`006?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo02ko
oooo00?o0000ooooooooool0b?ooool001Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`0]ooooo`03o`000?oooooooooo0<Wooooo000Hooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool0;Oooool00ol0003oooooooooo`3:ooooo`006?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool0boooool001Sooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo0<cooooo000Hooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool0:oooool2o`000<oooooo000Hooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`3?ooooo`006?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo02[ooooo00?o0000ooooooooool0d?oo
ool001Sooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooo
oooo0=7ooooo000Hooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0:Oooool00ol0
003oooooooooo`3Booooo`006?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo02Wo
oooo00?o0000ooooooooool0doooool001Oooooo0ol0ool4ooooo`03o`000?oooooooooo02Sooooo
00?o0000ooooooooool0e?ooool001Kooooo1Ol0ool2ooooo`03o`000?oooooooooo02Sooooo00?o
0000ooooooooool0eOooool001Gooooo1ol0ool00ooooooo0000ooooo`0Xooooo`03o`000?oooooo
oooo0=Kooooo000Eooooo`Oo0?oo00?o0000ooooooooool09oooool00ol0003oooooooooo`3Goooo
o`005Oooool7o`3oobWooooo00?o0000ooooooooool0f?ooool001Kooooo1Ol0ool00ol0003ooooo
ooooo`0Vooooo`03o`000?oooooooooo0=Wooooo000Gooooo`?o0?oo00?o0000ooooooooool09_oo
ool00ol0003oooooooooo`3Jooooo`005oooool01?l0003oooooo`000?l0000Wooooo`03o`000?oo
oooooooo0=_ooooo000Fooooo`03o`000?oooooooooo00;o00009Oooool2o`000=kooooo000Foooo
o`05o`000?oooooo0000ooooool000009?ooool00ol0003oooooooooo`3Nooooo`005_ooool01Ol0
003oooooo`000?oooooo000002?ooooo00?o0000ooooooooool0goooool001Gooooo00?o0000oooo
ool000000oooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo0>3ooooo000Eooooo`03
o`000?oooooo000000?ooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`3Qooooo`00
5?ooool00ol0003oooooo`000005ooooo`03o`000?oooooooooo01cooooo00?o0000ooooooooool0
h_ooool001Cooooo0_l00006ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool0hooo
ool001Cooooo0_l00006ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0i?ooool0
017ooooo0ol007l00ol0003oooooooooo`06ooooo`?o0?l06?ooool00ol0003oooooooooo`3Uoooo
o`004?ooool5o`00O`Oooooo1Ol0o`0Fooooo`03o`000?oooooooooo0>Kooooo000?ooooo`Oo001o
1Oooool7o`3o01Cooooo00?o0000ooooooooool0ioooool000oooooo1ol007l5ooooo`Oo0?l04ooo
ool00ol0003oooooooooo`3Xooooo`003oooool7o`00O`Gooooo1ol0o`0Booooo`03o`000?oooooo
oooo0>Wooooo000@ooooo`Go001o1oooool5o`3o017ooooo0_l0003/ooooo`004Oooool3o`00O`Wo
oooo0ol0o`0Aooooo`03o`000?oooooooooo0>cooooo000Aooooo`03o`000?oooooooooo00_ooooo
00?o0000ooooooooool03_ooool00ol0003oooooooooo`3]ooooo`004Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0k_ooool0013ooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo0>oooooo000@oooo
o`03o`000?oooooooooo00gooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`3`oooo
o`004?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool0lOooool000oooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo0?;ooooo000?ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`3cooooo`003oooool00ol0003oooooooooo`0@ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool0m?ooool000kooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo0?Gooooo000;ooooo`?o07l000?o0000oooooooo
ool04_ooool3o`1o0003ooooool0003ooooo0?Oooooo000:ooooo`Go07l04oooool5o`1o0?Wooooo
0009ooooo`Oo07l04Oooool7o`1o0?Sooooo0009ooooo`Oo07l04Oooool7o`1o0?Sooooo0009oooo
o`Oo07l04Oooool7o`1o0?Sooooo000:ooooo`Go07l04oooool5o`1o0?Wooooo000;ooooo`?o07l0
5Oooool3o`1o0?[ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7ooooo003oooooob7o
oooo003oooooob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, 0.229909, \
0.00383279, 0.00144217}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[ran2DTD]]; \[IndentingNewLine]Evolve[
          ran2DTD], {16}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[28]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[Last[l]];\)\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 -0.555477 2.41605 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.1161 .61366 m
.08684 .61366 L
s
.80823 .21953 m
.77897 .22018 L
s
.33976 .21834 m
.31063 .22596 L
s
.33976 .21834 m
.36888 .21072 L
s
.36927 .22765 m
.39852 .22532 L
s
.36927 .22765 m
.39853 .22574 L
s
.36927 .22765 m
.34002 .22977 L
s
.39855 .22864 m
.42781 .22786 L
s
.39855 .22864 m
.42781 .228 L
s
.39855 .22864 m
.36928 .22935 L
s
.48633 .22664 m
.45707 .22733 L
s
.14892 .3709 m
.16961 .31591 L
s
.14892 .3709 m
.12823 .4259 L
s
.42724 .19885 m
.4564 .19235 L
s
.42724 .19885 m
.45642 .19292 L
s
.42724 .19885 m
.39806 .20478 L
s
.42724 .19885 m
.45643 .19348 L
s
.42697 .19168 m
.39793 .20141 L
s
.16961 .31591 m
.18081 .24405 L
s
.16961 .31591 m
.19665 .28615 L
s
.48634 .22765 m
.48556 .18586 L
s
.48634 .22765 m
.45707 .22807 L
s
.4564 .19235 m
.48556 .18586 L
s
.08684 .76921 m
.08684 .69143 L
s
.08684 .76921 m
.10753 .8242 L
s
.08684 .76921 m
.06615 .8242 L
s
.31038 .22092 m
.33926 .20825 L
s
.31038 .22092 m
.28151 .23359 L
s
.39853 .22574 m
.42778 .22383 L
s
.06615 .48089 m
.08684 .53589 L
s
.42552 .16492 m
.3968 .17978 L
s
.42552 .16492 m
.45423 .1499 L
s
.42552 .16492 m
.45426 .15021 L
s
.80823 .21953 m
.83749 .21888 L
s
.80823 .21953 m
.77897 .22018 L
s
.80823 .21952 m
.77897 .22017 L
s
.80823 .21952 m
.83749 .21887 L
s
.80823 .21952 m
.83749 .21887 L
s
.92528 .21693 m
.89602 .21758 L
s
.92528 .21693 m
.54486 .22534 L
s
.39793 .20141 m
.42701 .19252 L
s
.39793 .20141 m
.36888 .21072 L
s
.57412 .22472 m
.54486 .22537 L
s
.57412 .22472 m
.60339 .22407 L
s
.398 .20309 m
.36888 .21072 L
s
.398 .20309 m
.66191 .22278 L
s
.28039 .21863 m
.25239 .24121 L
s
.69118 .22211 m
.66191 .22277 L
s
.60339 .22401 m
.57412 .22468 L
s
.77897 .22017 m
.7497 .22082 L
s
.63265 .22341 m
.66191 .22276 L
s
.63265 .22341 m
.60339 .22407 L
s
.27235 .16996 m
.24802 .21317 L
s
.54486 .22537 m
.5156 .22602 L
s
.18081 .24405 m
.19201 .1722 L
s
.36928 .2285 m
.39854 .22722 L
s
.36928 .2285 m
.34002 .22977 L
s
.39855 .22892 m
.42781 .2285 L
s
.39855 .22892 m
.36928 .22935 L
s
.42778 .22383 m
.06615 .8242 L
s
.33999 .22723 m
.36922 .22341 L
s
.33999 .22723 m
.31076 .23104 L
s
.08684 .69143 m
.08684 .61366 L
s
.10753 .8242 m
.12823 .8792 L
s
.31063 .22596 m
.28151 .23359 L
s
.72044 .22147 m
.69118 .22212 L
s
.72044 .22147 m
.7497 .22082 L
s
.72044 .22147 m
.7497 .22082 L
s
.48556 .18586 m
.51472 .17936 L
s
.48634 .22668 m
.45707 .22733 L
s
.48634 .22668 m
.7497 .22082 L
s
.48634 .22668 m
.5156 .22601 L
s
.48634 .22668 m
.5156 .22602 L
s
.48634 .22668 m
.5156 .22603 L
s
.4278 .22595 m
.39854 .22722 L
s
.72044 .22148 m
.69118 .22213 L
s
.80823 .21953 m
.77897 .22018 L
s
.60339 .22407 m
.57412 .22472 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.63265 .22342 L
s
.45707 .22733 m
.42781 .228 L
s
.69118 .22211 m
.66191 .22277 L
s
.54486 .22539 m
.5156 .22603 L
s
.66191 .22276 m
.69118 .22211 L
s
.08684 .61366 m
.08684 .53589 L
s
.51472 .17936 m
.54389 .17286 L
s
.72044 .22145 m
.7497 .2208 L
s
.72044 .22145 m
.10753 .48089 L
s
.72044 .22145 m
.69118 .22211 L
s
.80823 .21952 m
.77897 .22017 L
s
.69118 .22212 m
.57412 .22472 L
s
.69118 .22212 m
.72044 .22147 L
s
.69118 .22212 m
.66191 .22277 L
s
.69118 .22212 m
.72044 .22147 L
s
.22369 .25638 m
.24802 .21317 L
s
.22369 .25638 m
.25239 .24121 L
s
.22369 .25638 m
.19665 .28615 L
s
.63265 .22342 m
.60339 .22407 L
s
.63265 .22342 m
.66191 .22277 L
s
.54486 .22537 m
.57412 .22472 L
s
.54486 .22537 m
.57412 .22472 L
s
.54486 .22537 m
.5156 .22602 L
s
.06615 .8242 m
.04545 .8792 L
s
.39806 .20478 m
.36888 .21072 L
s
.12823 .8792 m
.83749 .21888 L
s
.7497 .22082 m
.77897 .22017 L
s
.60339 .22407 m
.63265 .22342 L
s
.60339 .22407 m
.57412 .22472 L
s
.63265 .22342 m
.66191 .22277 L
s
.57412 .22472 m
.60339 .22407 L
s
.83749 .21888 m
.86676 .21823 L
s
.83749 .21887 m
.86676 .21822 L
s
.83749 .21887 m
.86676 .21822 L
s
.45707 .2274 m
.42781 .228 L
s
.42557 .16554 m
.3968 .17978 L
s
.33926 .20825 m
.36805 .19433 L
s
.33926 .20825 m
.3682 .19684 L
s
.72044 .22147 m
.7497 .22082 L
s
.28151 .23359 m
.25239 .24121 L
s
.28151 .23359 m
.31076 .23104 L
s
.66191 .22277 m
.63265 .22342 L
s
.89602 .21758 m
.86676 .21823 L
s
.89602 .21758 m
.92528 .21693 L
s
.95455 .21628 m
.92528 .21693 L
s
.3968 .17978 m
.36805 .19433 L
s
.36922 .22341 m
.39845 .21959 L
s
.69118 .22213 m
.66191 .22278 L
s
.69118 .22213 m
.72044 .22148 L
s
.48299 .13551 m
.45426 .15021 L
s
.48299 .13551 m
.51173 .1208 L
s
.69118 .22213 m
.66191 .22278 L
s
.36805 .19433 m
.39688 .18103 L
s
.42571 .16773 m
.39688 .18103 L
s
.80823 .21953 m
.77897 .22018 L
s
.08684 .53589 m
.10753 .48089 L
s
.66191 .22277 m
.69118 .22212 L
s
.69118 .22213 m
.66191 .22278 L
s
.63265 .22342 m
.77897 .22018 L
s
.39688 .18103 m
.7497 .22083 L
s
.77897 .22018 m
.7497 .22083 L
s
.72044 .22148 m
.69118 .22213 L
s
.7497 .22082 m
.12823 .4259 L
s
.66191 .22278 m
.63265 .22342 L
s
.63265 .22342 m
.66191 .22278 L
s
.77897 .22018 m
.80823 .21953 L
s
.77897 .22018 m
.7497 .22083 L
s
.54486 .22534 m
.45423 .1499 L
s
.54486 .22534 m
.5156 .22601 L
s
.54486 .22534 m
.57412 .22468 L
s
.10753 .48089 m
.12823 .4259 L
s
.42781 .2285 m
.45707 .22807 L
s
.39855 .2292 m
.36928 .22935 L
s
.36928 .22935 m
.34002 .22977 L
s
.5156 .22601 m
.72044 .22148 L
s
.7497 .22083 m
.72044 .22147 L
s
.7497 .22083 m
.77897 .22018 L
s
.66191 .22278 m
.69118 .22213 L
s
.7497 .22083 m
.77897 .22018 L
s
.7497 .22083 m
.72044 .22147 L
s
.7497 .22083 m
.72044 .22147 L
s
.72044 .22148 m
.5156 .22603 L
s
.34002 .22977 m
.31076 .23104 L
s
1 0 1 r
.025 w
.1161 .61366 Mdot
.80823 .21953 Mdot
.498 0 0 r
.33976 .21834 Mdot
1 0 1 r
.36927 .22765 Mdot
.39855 .22864 Mdot
.498 0 0 r
.48633 .22664 Mdot
1 0 1 r
.14892 .3709 Mdot
.498 0 0 r
.42724 .19885 Mdot
.42697 .19168 Mdot
1 0 1 r
.16961 .31591 Mdot
0 1 1 r
.48634 .22765 Mdot
1 0 1 r
.39852 .22532 Mdot
.498 0 0 r
.4564 .19235 Mdot
1 0 1 r
.08684 .76921 Mdot
.31038 .22092 Mdot
.39853 .22574 Mdot
.498 0 0 r
.45642 .19292 Mdot
1 0 1 r
.06615 .48089 Mdot
.898 .898 .898 r
.42552 .16492 Mdot
1 0 1 r
.80823 .21953 Mdot
0 1 1 r
.80823 .21952 Mdot
.498 0 0 r
.92528 .21693 Mdot
.39793 .20141 Mdot
1 0 1 r
.57412 .22472 Mdot
.498 0 0 r
.398 .20309 Mdot
1 0 1 r
.28039 .21863 Mdot
0 1 1 r
.69118 .22211 Mdot
.498 0 0 r
.42701 .19252 Mdot
.898 .898 .898 r
.60339 .22401 Mdot
1 0 1 r
.77897 .22017 Mdot
.63265 .22341 Mdot
.27235 .16996 Mdot
.54486 .22537 Mdot
.18081 .24405 Mdot
.36928 .2285 Mdot
.39855 .22892 Mdot
.42778 .22383 Mdot
.33999 .22723 Mdot
.08684 .69143 Mdot
.10753 .8242 Mdot
.31063 .22596 Mdot
.72044 .22147 Mdot
.498 0 0 r
.42781 .22786 Mdot
1 1 0 r
.48556 .18586 Mdot
.498 0 0 r
.48634 .22668 Mdot
1 1 0 r
.4278 .22595 Mdot
.498 0 0 r
.72044 .22148 Mdot
1 0 1 r
.80823 .21953 Mdot
.898 .898 .898 r
.60339 .22407 Mdot
.498 0 0 r
.45707 .22733 Mdot
0 1 1 r
.69118 .22211 Mdot
.498 0 0 r
.54486 .22539 Mdot
1 0 1 r
.66191 .22276 Mdot
.08684 .61366 Mdot
1 1 0 r
.51472 .17936 Mdot
1 0 1 r
.72044 .22145 Mdot
0 1 1 r
.80823 .21952 Mdot
1 0 1 r
.69118 .22212 Mdot
.19201 .1722 Mdot
.22369 .25638 Mdot
.63265 .22342 Mdot
.54486 .22537 Mdot
.06615 .8242 Mdot
.498 .498 .498 r
.54389 .17286 Mdot
.498 0 0 r
.39806 .20478 Mdot
1 0 1 r
.12823 .8792 Mdot
.7497 .22082 Mdot
.60339 .22407 Mdot
0 1 1 r
.63265 .22342 Mdot
1 0 1 r
.7497 .2208 Mdot
.57412 .22472 Mdot
0 1 1 r
.77897 .22017 Mdot
1 0 1 r
.83749 .21888 Mdot
.498 0 0 r
.36888 .21072 Mdot
0 1 1 r
.83749 .21887 Mdot
.498 0 0 r
.45707 .2274 Mdot
1 0 1 r
.42557 .16554 Mdot
.33926 .20825 Mdot
.57412 .22472 Mdot
.72044 .22147 Mdot
.28151 .23359 Mdot
.66191 .22277 Mdot
.898 .898 .898 r
.89602 .21758 Mdot
1 0 0 r
.95455 .21628 Mdot
1 0 1 r
.39854 .22722 Mdot
.24802 .21317 Mdot
.63265 .22342 Mdot
.3968 .17978 Mdot
.36922 .22341 Mdot
.898 .898 .898 r
.69118 .22213 Mdot
.48299 .13551 Mdot
.69118 .22213 Mdot
1 0 1 r
.36805 .19433 Mdot
.42571 .16773 Mdot
.39845 .21959 Mdot
.80823 .21953 Mdot
.08684 .53589 Mdot
0 1 1 r
.66191 .22277 Mdot
.898 .898 .898 r
.69118 .22213 Mdot
0 1 1 r
.86676 .21822 Mdot
1 0 1 r
.63265 .22342 Mdot
.39688 .18103 Mdot
.77897 .22018 Mdot
.498 0 0 r
.72044 .22148 Mdot
1 0 1 r
.25239 .24121 Mdot
.04545 .8792 Mdot
.7497 .22082 Mdot
.19665 .28615 Mdot
.898 .898 .898 r
.66191 .22278 Mdot
1 0 1 r
.60339 .22407 Mdot
.498 0 0 r
.42781 .228 Mdot
1 0 1 r
.86676 .21823 Mdot
.898 .898 .898 r
.63265 .22342 Mdot
1 0 1 r
.7497 .22082 Mdot
.77897 .22018 Mdot
.498 0 0 r
.54486 .22534 Mdot
1 0 1 r
.10753 .48089 Mdot
.66191 .22277 Mdot
0 1 1 r
.42781 .2285 Mdot
1 0 1 r
.39855 .2292 Mdot
.80823 .21953 Mdot
.498 0 0 r
.45423 .1499 Mdot
1 0 1 r
.36928 .22935 Mdot
0 1 1 r
.45707 .22807 Mdot
.898 .898 .898 r
.45426 .15021 Mdot
.92528 .21693 Mdot
.498 0 0 r
.5156 .22601 Mdot
.45643 .19348 Mdot
1 0 1 r
.7497 .22083 Mdot
0 1 1 r
.83749 .21887 Mdot
.498 0 0 r
.66191 .22278 Mdot
.57412 .22468 Mdot
1 0 1 r
.7497 .22083 Mdot
1 0 0 r
.51173 .1208 Mdot
1 0 1 r
.60339 .22407 Mdot
.77897 .22018 Mdot
.12823 .4259 Mdot
.7497 .22083 Mdot
.898 .898 .898 r
.72044 .22148 Mdot
0 1 1 r
.86676 .21822 Mdot
.498 0 0 r
.69118 .22213 Mdot
1 0 1 r
.5156 .22602 Mdot
.72044 .22147 Mdot
0 1 1 r
.69118 .22212 Mdot
.498 0 0 r
.5156 .22603 Mdot
1 0 1 r
.34002 .22977 Mdot
.69118 .22211 Mdot
.31076 .23104 Mdot
.3682 .19684 Mdot
.77897 .22018 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[30]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00TOooool3ool008cooooo002@ooooo`Goo`00Roooool008oooooo1ooo
002:ooooo`00Soooool7ool008[ooooo0029ooooo`?oiNGU0oooool7ool008[ooooo0028ooooo`Go
iNGU0_l000000oooooooo`00ool00003ool008_ooooo0027ooooo`OoiNGU0oooool3ool008cooooo
0027ooooo`OoiNGUT_ooool008Oooooo1ooUiNFBooooo`00POooool3onGUi@;ooooo0_l00005onGU
iI?ooooo0020ooooo`GoiNGU00Co0000oooooooooooooooo0ooUiNFDooooo`00Ooooool7onGUiI[o
oooo001oooooo`OoiNGUV_ooool007Wooooo0ooo0?l3ooooo`OoiNGUV_ooool004gooooo0ooo0?lX
ooooo`Goo`3o0_l000000oooooooiNGUonGUi@03onGUi@03o`000?oooooooooo09Sooooo000foooo
o`?oo`3o4oooool5ool0obKooooo1ooo0?l3ooooo`?oiNGU0_ooool00ol0003oooooooooo`0Boooo
o`?oOgmoP_ooool003Gooooo1Ooo0?lAooooo`Ooo`3o9Oooool7ool0o`Wooooo0_l0000Aooooo`Go
OgmoPOooool003Cooooo1ooo0?l@ooooo`Ooo`3o7_ooool3ool0o`Cooooo1ooo0?l;ooooo`03o`00
0?oooooooooo00Kooooo0oooo`04ooooo`OoOgmoP?ooool003Cooooo1ooo0?l@ooooo`Ooo`3o7Ooo
ool5ool0o`?o00001ooo0?l<ooooo`03o`000?oooooooooo00Cooooo1Oooo`03ooooo`OoOgmoP?oo
ool003Cooooo1ooo0?lAooooo`Goo`3o7Oooool7ool0o`03o`000?oooooooooo00Goo`3o3Oooool3
oooo00?ooooo1oooo`02o`0000OoOgmoP?ooool003Gooooo1Ooo0?lCooooo`?oo`3o7_ooool7ool0
o`04o`000?ooooooooooooooo`?oo`3o3Oooool5oooo00;ooooo1oooo`03ooooo`GoOgmoPOooool0
03Kooooo0ooo0?lCooooo`03o`000?oooooooooo01Wooooo0ooo0?l3ooooo`Ooo`3o00Coooooo`00
0?l0003o00000omo0003o`0000;ooooo0omo0004ooooo`Ooool000?o0000oooo0?ooo`001Oooo`04
ooooo`?oOgmoP_ooool003Kooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Hoooo
o`Goo`3o0_l00007ool0o`?ooooo1Omo0002ooooo`03o`000?mo003oO`0000?oO`000_l000000ooo
ooooool0oooo0005oooo0003o`000?ooooooool000Coool0R_ooool003Kooooo00?o0000oooooooo
ool04_ooool00ol0003oooooooooo`0Hooooo`Ooo`3o0_ooool5ool0o`?ooooo1omo00000ooooooo
O`00ogl00005ogl000;o00001oooo`02o`000004ooooooooo`3oool0oooo08_ooooo000fooooo`03
o`000?oooooooooo017ooooo00?o0000ooooooooool06Oooool7ool0o`?ooooo0ooo0?l2ogl000;o
oooo1omo00000oooooooO`00ogl00005ogl000?ooooo1Oooo`02ooooo`So0000R?ooool003Kooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Cooooo`?oo`3o0oooool7ool0o`;ooooo
1omo00000ol0003oO`00ogl00005ogl00003o`000?mo003oO`0000GoO`001?ooool3oooo00Gooooo
0_l00004ooooo`Wo0000Ooooool003Kooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Cooooo`Goo`3o0_l00007ool0o`;ooooo1omo00000ol0003oO`00ogl00005ogl000;ooooo1Omo
0006ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02_ooool8o`0007Oooooo000e
ooooo`03o`000?oooooooooo00kooooo0ooo0?l00ol0003oooooooooo`0Booooo`Ooo`3o0_ooool5
ool0o`;ooooo00?o0000ogl00?mo00001Omo00000ol0003oO`00ogl00005ogl000?ooooo0omo0007
ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool04Oooool9o`0006kooooo000eoooo
o`03o`000?oooooooooo00gooooo1Ooo0?l5ooooo`?oo`3o3?ooool7ool0o`03oooooomo003oO`00
00?oo`3o0_mo0002o`0000OoO`000_ooool5ogl000go000000?oooooo`000?ooool02Oooool00ol0
003oooooooooo`0Iooooo`So0000=Oooool3o`3oo`Gooooo0ol0ool6ooooo`?oiNGU1Oooool3onGU
i@Gooooo0ooo000=ooooo`00=Oooool00ol0003oooooooooo`0<ooooo`Ooo`3o0oooool5ool0o`Go
oooo0ooo0?l3ooooo`Ooo`3o00?oooooogl00?mo00001Omo0003ooooo`05ogl00?oo0?ooo`3oool0
oomo00001?ooool3ogl000kooooo3ol0000Qooooo`;o00000omo0004o`0000;ooooo0ooo0?l5oooo
o`?oo`3o1_ooool3ool0o`Gooooo0ooo0?l5ooooo`?oo`3o1Oooool5o`3oo`?ooooo1Ol0ool4oooo
o`GoiNGU0oooool5onGUi@?ooooo1Ooo000<ooooo`00=Oooool00ol0003oooooooooo`0<ooooo`Oo
o`3o0_ooool7ool0o`?ooooo1Ooo0?l2o`000003ogl00?oo0?ooo`3o00?oo`3o00CoO`00o`000?mo
003oO`000ooo0?l2ogl000?ooooo1Ooo0?l4ooooo`?oool03oooool00ol0003oooooooooo`04oooo
o`?oO`001?ooool2o`0000?oO`001Ol00003ogl000;o00001?ooool3ool0o`Gooooo0ooUiND4oooo
o`GoO`000oooool00ol0003oo`3oool0o`03ool0o`;o000000?oooooool0oooo0?l00ooo0?l4oooo
o`Goo`3o0oooool5ool0o`?ooooo1Ooo0?l3ooooo`Oo0?oo00?oooooo`3oool0ool01Ol0ool2oooo
o`OoiNGU00?oooooonGUiOoUiND01OoUiND00oooooooo`00ool00005ool000_ooooo000eooooo`03
o`000?oooooooooo00cooooo1ooo0?l2ooooo`Ooo`3o0_ooool7ool0o`03oooooomo003oO`0000?o
o`3o0_mo0002ooooo`Goo`3o0oooool7ool0o`;ooooo00Goool0o`3oool0oooo0?oooooo0005oooo
o`?o0?oo1Oooool3ogl000Cooooo1Omo0004ooooo`GoO`000oooool5ogl00004ooooool0003o0000
o`0000Goo`3o0ol00005onGUi@;ooooo1omo0002ooooo`Ooo`3o00?oooooool0oooo0?l01Ooo0?l2
o`0000Ooo`3o00?oooooool0oooo0?l01Ooo0?l00oooooooo`3oool0o`05ool0o`03ooooool0003o
0?oo00Ko0?oo00?o0000o`3oool0ool01Ol0ool2o`0000OoiNGU00?o0000onGUiOoUiND01OoUiND0
0ol0003oo`00ool00005ool000_ooooo000dooooo`03o`000?oooooooooo00kooooo1Ooo0?l3oooo
o`Ooo`3o0_ooool7ool0o`03o`000?mo003oo`3o00Coo`3o00?oO`00oooooooo0?l01_oo0?l00ooo
oooo0000ool0o`06ool0o`03ooooooooo`3o0?oo00Co0?oo00Coool0oooooooooooooooo1Ol0ool3
ooooo`GoO`000_ooool7ogl000;ooooo1omo00000oooooooO`00ogl00005ogl000;ooooo1ooo0?l0
0ol0003oiNGUonGUi@05onGUi@03o`000?mo003oO`0000GoO`000_l00007ool0o`03o`000?oo0?oo
o`3o00Goo`3o0_l00007ool0o`03o`000?oo0?ooo`3o00Goo`3o00?o0000ool0oooo0?l01Ooo0?l2
o`0000Oo0?oo00?oooooo`3oool0ool01Ol0ool2ooooo`OoiNGU00?oooooonGUiOoUiND01OoUiND0
0oooooooo`00ool00005ool000_ooooo000dooooo`03o`000?oooooooooo00oooooo0ooo0?l4oooo
o`03o`000?oo0?ooo`3o00?oo`3o0oooool7ool0o`03oooooooo0?ooo`3o00Goo`3o00?o0000ool0
oooo0?l01Ooo0?l2o`0000Ooo`3o00?o0000o`3oool0ool01Ol0ool2ooooo`Oo0?oo00?oooooogl0
0?mo00001Omo00000ol0003oO`00ogl00005ogl000;o00001omo00000ol0003oO`00ogl00005ogl0
00;o00001ooo0?l00ol0003oiNGUonGUi@05onGUi@03o`000?mo003oO`0000GoO`000_l00007ool0
o`03o`000?oo0?ooo`3o00Goo`3o0_l00007ool0o`03oooooooo0?ooo`3o00Goo`3o00?oooooool0
oooo0?l01Ooo0?l2ooooo`03o`000?l0oooo0?oo00?o0?oo0oooool5o`3oo`Cooooo1OoUiND3oooo
o`GoiNGU0oooool5ool000cooooo000dooooo`03o`000?oooooooooo00kooooo00?o0000oooooooo
ool01?ooool00ol0003oooooool0o`04ool0o`?o00001ooo0?l00ol0003oo`3oool0o`05ool0o`03
o`000?oo0?ooo`3o00Goo`3o0_l00007ool0o`03o`000?l0oooo0?oo00Go0?oo0_l00007o`3oo`03
o`000?mo003oO`0000GoO`0000?o0000ogl00?mo00001Omo0002o`0000OoO`0000?oooooogl00?mo
00001Omo0002ooooo`Ooo`3o00?oooooonGUiOoUiND01OoUiND2ooooo`GoO`001?ooool5ool0o`03
o`000?oooooooooo00Goo`3o00Co0000oooooooooooooooo1Ooo0?l3ooooo`Goo`3o0oooool5ool0
o`;ooooo00?o0000ooooooooool00ol0ool5ooooo`?o0?oo1_ooool3onGUi@Gooooo0ooUiND5oooo
o`?oo`003Oooool003Cooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`03ool0o`03
ooooool0003ooooo00Ooo`3o0_l00007ool0o`03oooooooo0?ooo`3o00Goo`3o00?oooooool0oooo
0?l01Ooo0?l2ooooo`Ooo`3o00?oooooo`3oool0ool01Ol0ool2ooooo`Oo0?oo00?oooooogl00?mo
00001Omo0002ooooo`GoO`001?ooool5ogl000?ooooo1Omo0004ooooo`Goo`3o0oooool5onGUi@Co
oooo0omo0006ooooo`?oo`3o1?ooool01?l0003oo`3oool0oooo0?l6ooooo`?oo`3o1Oooool3ool0
o`Gooooo0ooo0?l2ooooo`03o`000?oooooooooo03;ooooo000booooo`?oo`3o3oooool00ol0003o
ooooool0o`04ool0o`;ooooo1ooo0?l00ol0003oooooool0o`06ool0o`03oooooooo0?ooo`3o00Go
o`3o0_ooool5ool0o`Cooooo1Ooo0?l3ooooo`Go0?oo1?ooool5o`3oo`?ooooo1Omo0004ooooo`?o
O`001_ooool3ogl000Gooooo0omo0006ooooo`?oo`3o1Oooool3onGUi@cooooo0ol00003ooooo`?o
00007_ooool00ol0003oooooooooo`0cooooo`00<Oooool5ool0o`gooooo00?o0000oooooooo0?l0
1_oo0?l00ol0003oo`3oool0o`05ool0o`?ooooo1Ooo0?l3ooooo`Goo`3o1?ooool3ool0o`Kooooo
0ooo0?l4ooooo`04o`000?l0oooo0?ooo`3oo`Kooooo0ol0ool5ooooo`?oO`00=?ooool2o`0000?o
oooo0ol0000Pooooo`03o`000?oooooooooo03Cooooo000`ooooo`Ooo`3o3?ooool00ol0003ooooo
ool0o`06ool0o`;ooooo1Ooo0?l5ooooo`?oo`3o1Oooool3ool0oaCooooo00?o0000ooooooooool0
AOooool2o`0000;ooooo0ol0000Rooooo`03o`000?oooooooooo03Gooooo000`ooooo`Ooo`3o2?oo
ool3ool0o`03o`000?oooooooooo00Ooo`3o0oooool3ool0obGooooo00?o0000ooooooooool0@_oo
ool3o`000004ooooool0003o0000o`0002Cooooo00?o0000ooooooooool0=_ooool0033ooooo1ooo
0?l7ooooo`Goo`3o0_l000000oooooooo`3oool0o`03ool0ob_ooooo00?o0000ooooooooool0@Ooo
ool2o`000004ooooool0003o0000o`0002Kooooo00?o0000ooooooooool0=oooool0037ooooo1Ooo
0?l7ooooo`Ooo`3o0oooool3ool0ob_ooooo00?o0000ooooooooool0@?ooool5o`0002Sooooo00?o
0000ooooooooool0>?ooool003;ooooo0ooo0?l8ooooo`Ooo`3o<Oooool00ol0003oooooooooo`0m
ooooo`Go0000:_ooool00ol0003oooooooooo`0iooooo`00<_ooool00ol0003oooooooooo`08oooo
o`Ooo`3o<?ooool00ol0003oooooooooo`0looooo`Co0000;?ooool00ol0003oooooooooo`0joooo
o`00<_ooool00ol0003oooooooooo`09ooooo`Goo`3o<Oooool00ol0003oooooooooo`0jooooo`?o
0000;_ooool00ol0003oooooooooo`0kooooo`00<_ooool00ol0003oooooooooo`09ooooo`04o`00
0?oo0?ooo`3oool0oc7ooooo00?o0000ooooooooool0>?ooool3o`0002oooooo0_l0000nooooo`00
<_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo03;ooooo00?o0000ooooooooool0
=_ooool3o`00037ooooo00?o0000ooooooooool0?_ooool003;ooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`0cooooo`03o`000?oooooooooo03?ooooo1?l0000booooo`03o`000?oo
oooooooo03oooooo000booooo`03o`000?oooooooooo00;ooooo0ooo0?l00ooooooo0000ooooo`0d
ooooo`03o`000?oooooooooo037ooooo1Ol0000cooooo`03o`000?oooooooooo043ooooo000boooo
o`04o`000?ooooooooooooooo`Goo`3o=_ooool00ol0003oooooooooo`0^ooooo`Go0000=Oooool0
0ol0003oooooooooo`11ooooo`00<Oooool01?l0003oooooooooooooool7ool0ocCooooo00?o0000
ooooooooool0;?ooool3o`000003ooooool0003o000003Kooooo00?o0000ooooooooool0@_ooool0
037ooooo00Co0000oooooooooooooooo1ooo0?lcooooo`03o`000?oooooooooo02[ooooo0ol00000
1?oooooo0000o`000?l0000gooooo`03o`000?oooooooooo04?ooooo000aooooo`04o`000?oooooo
ooooooooo`Ooo`3o<oooool00ol0003oooooooooo`0Wooooo`?o00000_ooool2o`0003Wooooo00?o
0000ooooooooool0A?ooool0037ooooo00?o0000ooooooooool00_ooool5ool0oc?ooooo00?o0000
ooooooooool09Oooool3o`0000?ooooo0_l0000jooooo`03o`000?oooooooooo04Gooooo000aoooo
o`06o`000?ooooooooooooooool0003ooooo0ooo0?ldooooo`03o`000?oooooooooo02;ooooo0ol0
0003ooooo`?o0000>oooool00ol0003oooooooooo`16ooooo`00<Oooool01?l0003oooooooooool0
000hooooo`03o`000?oooooooooo023ooooo0ol00004ooooo`;o0000?Oooool00ol0003ooooooooo
o`17ooooo`00;oooool3ool0o`03ooooool0003ooooo03Oooooo00?o0000ooooooooool07_ooool3
o`0000Gooooo0_l0000nooooo`03o`000?oooooooooo04Sooooo000^ooooo`Goo`3o>Oooool00ol0
003oooooooooo`0Kooooo`?o00001Oooool3o`0003oooooo00?o0000ooooooooool0BOooool002go
oooo1ooo0?lgooooo`03o`000?oooooooooo01Wooooo0ol00006ooooo`;o0000@?ooool2o`0004co
oooo000]ooooo`Ooo`3o=oooool00ol0003oooooooooo`0Fooooo`?o00001_ooool3o`00047ooooo
00?o0000ooooooooool0C?ooool002gooooo1ooo0?lfooooo`03o`000?oooooooooo01Cooooo0ol0
0007ooooo`;o0000@oooool00ol0003oooooooooo`1=ooooo`00;_ooool5ool0ocKooooo00?o0000
ooooooooool04Oooool4o`0000Sooooo0_l00014ooooo`03o`000?oooooooooo04kooooo000_oooo
o`?oo`3o=oooool00ol0003oooooooooo`0>ooooo`?o00002Oooool3o`0004Gooooo00?o0000oooo
ooooool0Coooool002oooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`0<ooooo`?o
00002_ooool2o`0004Oooooo00?o0000ooooooooool0D?ooool002kooooo00?o0000ooooooooool0
=oooool00ol0003oooooooooo`09ooooo`?o00002oooool2o`0004Sooooo00?o0000ooooooooool0
DOooool002kooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`07ooooo`?o00002ooo
ool3o`0004Wooooo00?o0000ooooooooool0D_ooool002kooooo00?o0000ooooooooool0=Oooool0
0ol0003oooooooooo`05ooooo`?o00003?ooool2o`0004_ooooo00?o0000ooooooooool0Doooool0
02gooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`02ooooo`?o00003Oooool2o`00
04cooooo00?o0000ooooooooool0E?ooool002gooooo00?o0000ooooooooool0=Oooool00ol0003o
ooooooooo`03o`0000gooooo0ol0001=ooooo`03o`000?oooooooooo05Gooooo000/ooooo`03o`00
0?oooooooooo03Kooooo0ol0000>ooooo`;o0000Coooool00ol0003oooooooooo`1Fooooo`00;?oo
ool00ol0003oooooooooo`0cooooo`?o00003oooool2o`00053ooooo00?o0000ooooooooool0Eooo
ool002cooooo00?o0000ooooooooool0<?ooool3o`000003ooooool0003ooooo00cooooo0ol0001A
ooooo`03o`000?oooooooooo05Sooooo000Yooooo`?oo`3o<?ooool3o`0000Cooooo00?o0000oooo
ooooool02Oooool2o`0005;ooooo0_l0001Kooooo`00:?ooool5ool0obcooooo0ol00006ooooo`03
o`000?oooooooooo00Sooooo0_l0001Cooooo`03o`000?oooooooooo05_ooooo000Wooooo`Ooo`3o
:?ooool3o`0000Wooooo00?o0000ooooooooool01Oooool3o`0005Cooooo00?o0000ooooooooool0
G?ooool002Oooooo1ooo0?lUooooo`?o00002oooool00ol0003oooooooooo`04ooooo`;o0000E_oo
ool00ol0003oooooooooo`1Mooooo`009oooool7ool0ob;ooooo0ol0000=ooooo`03o`000?oooooo
oooo00?ooooo0_l0001Gooooo`03o`000?oooooooooo05kooooo000Xooooo`Goo`3o8?ooool3o`00
013ooooo00?o0000ooooooooool00ol0001Hooooo`03o`000?oooooooooo05oooooo000Yooooo`?o
o`3o7_ooool3o`0001;ooooo00Co0000ooooool0003o0000F_ooool00ol0003oooooooooo`1Poooo
o`00:Oooool00ol0003oooooooooo`0Kooooo`?o00005?ooool3o`0005_ooooo00?o0000oooooooo
ool0HOooool002Sooooo00?o0000ooooooooool06Oooool3o`0001Gooooo0ol0001Looooo`03o`00
0?oooooooooo06;ooooo000Xooooo`03o`000?oooooooooo01Kooooo0ol0000Fooooo`;o000000?o
ooooo`000?ooool0Foooool00ol0003oooooooooo`1Sooooo`00:?ooool00ol0003oooooooooo`0C
ooooo`?o00005_ooool3o`0000?ooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`1T
ooooo`009oooool00ol0003oooooooooo`0Aooooo`?o00005oooool2o`0000Gooooo00?o0000oooo
ooooool0FOooool00ol0003oooooooooo`1Uooooo`009oooool00ol0003oooooooooo`0>ooooo`?o
00006?ooool2o`0000Kooooo00?o0000ooooooooool0FOooool00ol0003oooooooooo`1Vooooo`00
9_ooool00ol0003oooooooooo`0<ooooo`?o00006?ooool3o`0000Sooooo00?o0000ooooooooool0
Eoooool2o`0006Wooooo000Vooooo`03o`000?oooooooooo00Wooooo0ol0000Iooooo`;o00002_oo
ool00ol0003oooooooooo`1Gooooo`03o`000?oooooooooo06Wooooo000Vooooo`03o`000?oooooo
oooo00Kooooo0ol0000Jooooo`;o00003?ooool00ol0003oooooooooo`1Fooooo`03o`000?oooooo
oooo06[ooooo000Sooooo`?oo`3o1_ooool3o`0001[ooooo0ol0000=ooooo`03o`000?oooooooooo
05Kooooo00?o0000ooooooooool0Joooool002;ooooo1Ooo0?l2ooooo`?o00006oooool2o`0000oo
oooo00?o0000ooooooooool0E_ooool00ol0003oooooooooo`1/ooooo`008Oooool7ool0o`03o`00
0?oooooooooo01[ooooo0_l0000Aooooo`03o`000?oooooooooo05Gooooo00?o0000ooooooooool0
KOooool0027ooooo1ooo0?lJooooo`?o00004_ooool00ol0003oooooooooo`1Eooooo`03o`000?oo
oooooooo06kooooo000Qooooo`Ooo`3o6?ooool2o`0001Gooooo00?o0000ooooooooool0E?ooool0
0ol0003oooooooooo`1_ooooo`008_ooool5ool0oaOooooo0_l0000Fooooo`03o`000?oooooooooo
05Cooooo00?o0000ooooooooool0L?ooool002?ooooo0ooo0?lEooooo`?o00005oooool00ol0003o
ooooooooo`1Dooooo`03o`000?oooooooooo077ooooo000Sooooo`03o`000?oooooooooo01?ooooo
0_l0000Jooooo`03o`000?oooooooooo05?ooooo00?o0000ooooooooool0L_ooool002;ooooo00?o
0000ooooooooool04_ooool2o`0001_ooooo00?o0000ooooooooool0Doooool00ol0003ooooooooo
o`1cooooo`008_ooool00ol0003oooooooooo`0?ooooo`?o00007Oooool00ol0003oooooooooo`1B
ooooo`03o`000?oooooooooo07Cooooo000Rooooo`03o`000?oooooooooo00gooooo0_l0000Ooooo
o`03o`000?oooooooooo05;ooooo00?o0000ooooooooool0MOooool0027ooooo00?o0000oooooooo
ool03?ooool2o`00023ooooo00?o0000ooooooooool0DOooool2o`0007Sooooo000Qooooo`03o`00
0?oooooooooo00Wooooo0ol0000Rooooo`03o`000?oooooooooo053ooooo00?o0000ooooooooool0
N?ooool0023ooooo00?o0000ooooooooool02?ooool2o`0002Cooooo00?o0000ooooooooool0D?oo
ool00ol0003oooooooooo`1iooooo`008?ooool00ol0003oooooooooo`06ooooo`;o00009_ooool0
0ol0003oooooooooo`1?ooooo`03o`000?oooooooooo07[ooooo000Pooooo`03o`000?oooooooooo
00?ooooo0ol0000Wooooo`03o`000?oooooooooo04oooooo00?o0000ooooooooool0Noooool0017o
oooo0ooo0?l9ooooo`?oo`3o1?ooool2o`0002Wooooo00?o0000ooooooooool0Coooool00ol0003o
ooooooooo`1looooo`004?ooool5ool0o`Oooooo1Ooo0?l00ooooooo0000o`00000[ooooo`03o`00
0?oooooooooo04kooooo00?o0000ooooooooool0OOooool000oooooo1ooo0?l5ooooo`Ooo`3o;?oo
ool00ol0003oooooooooo`1>ooooo`03o`000?oooooooooo07kooooo000?ooooo`Ooo`3o1Oooool7
ool0obcooooo00?o0000ooooooooool0COooool00ol0003oooooooooo`1oooooo`003oooool7ool0
o`Gooooo1ooo0?l[ooooo`03o`000?oooooooooo04gooooo00?o0000ooooooooool0P?ooool0013o
oooo1Ooo0?l7ooooo`Goo`3o:oooool00ol0003oooooooooo`1=ooooo`03o`000?oooooooooo087o
oooo000Aooooo`?oo`3o2Oooool3ool0obcooooo00?o0000ooooooooool0C?ooool00ol0003ooooo
ooooo`22ooooo`004oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo02_ooooo00?o
0000ooooooooool0C?ooool00ol0003oooooooooo`23ooooo`005?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo02cooooo00?o0000ooooooooool0B_ooool2o`0008Kooooo000Doooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`1:oooo
o`03o`000?oooooooooo08Kooooo000Eooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0:oooool00ol0003oooooooooo`1:ooooo`03o`000?oooooooooo08Oooooo000Eooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool0:oooool00ol0003oooooooooo`19ooooo`03o`00
0?oooooooooo08Sooooo000Eooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0:_oo
ool00ol0003oooooooooo`19ooooo`03o`000?oooooooooo08Wooooo000Fooooo`05o`000?oooooo
ooooooooool00000;Oooool00ol0003oooooooooo`18ooooo`03o`000?oooooooooo08[ooooo000F
ooooo`05o`000?ooooooooooooooool00000;?ooool00ol0003oooooooooo`18ooooo`03o`000?oo
oooooooo08_ooooo000Gooooo`?oo`3o;?ooool00ol0003oooooooooo`18ooooo`03o`000?oooooo
oooo08cooooo000Fooooo`Goo`3o:oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo
08gooooo000Eooooo`Ooo`3o:Oooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo08ko
oooo000Eooooo`Ooo`3o:?ooool00ol0003oooooooooo`17ooooo`03o`000?oooooooooo08oooooo
000Eooooo`Ooo`3o:?ooool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo093ooooo000F
ooooo`Goo`3o:?ooool00ol0003oooooooooo`16ooooo`03o`000?oooooooooo097ooooo000Goooo
o`?oo`3o:Oooool00ol0003oooooooooo`15ooooo`03o`000?oooooooooo09;ooooo000Hooooo`03
o`000?oooooooooo02Oooooo00?o0000ooooooooool0A?ooool2o`0009Gooooo000Hooooo`03o`00
0?oooooooooo02Kooooo00?o0000ooooooooool0A?ooool00ol0003oooooooooo`2Eooooo`006?oo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo04?ooooo00?o0000ooooooooool0U_oo
ool001Sooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`13ooooo`03o`000?oooooo
oooo09Oooooo000Hooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0@_ooool00ol0
003oooooooooo`2Hooooo`006?ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo04;o
oooo00?o0000ooooooooool0VOooool001Sooooo00?o0000ooooooooool08oooool00ol0003ooooo
ooooo`12ooooo`03o`000?oooooooooo09[ooooo000Hooooo`03o`000?oooooooooo02?ooooo00?o
0000ooooooooool0@Oooool00ol0003oooooooooo`2Kooooo`006?ooool00ol0003oooooooooo`0R
ooooo`03o`000?oooooooooo047ooooo00?o0000ooooooooool0W?ooool001Sooooo00?o0000oooo
ooooool08_ooool00ol0003oooooooooo`10ooooo`03o`000?oooooooooo09gooooo000Hooooo`03
o`000?oooooooooo027ooooo00?o0000ooooooooool0@?ooool00ol0003oooooooooo`2Nooooo`00
6?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo043ooooo00?o0000ooooooooool0
Woooool001Sooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0oooooo`03o`000?oo
oooooooo0:3ooooo000Hooooo`03o`000?oooooooooo01oooooo00?o0000ooooooooool0?oooool0
0ol0003oooooooooo`2Qooooo`006?ooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo
03gooooo0_l0002Tooooo`006?ooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo03go
oooo00?o0000ooooooooool0Y?ooool001Oooooo0ooo0?l6ooooo`?oo`3o5Oooool00ol0003ooooo
ooooo`0mooooo`03o`000?oooooooooo0:Gooooo000Fooooo`Goo`3o1?ooool5ool0oaCooooo00?o
0000ooooooooool0??ooool00ol0003oooooooooo`2Vooooo`005Oooool7ool0o`;ooooo1ooo0?lB
ooooo`03o`000?oooooooooo03cooooo00?o0000ooooooooool0Yoooool001Gooooo1ooo0?l2o`00
00Ooo`3o4_ooool00ol0003oooooooooo`0kooooo`03o`000?oooooooooo0:Sooooo000Eooooo`Oo
o`3o0_ooool7ool0oa7ooooo00?o0000ooooooooool0>oooool00ol0003oooooooooo`2Yooooo`00
5_ooool5ool0o`Cooooo1Ooo0?lAooooo`03o`000?oooooooooo03_ooooo00?o0000ooooooooool0
Z_ooool001Oooooo0ooo0?l6ooooo`?oo`3o4_ooool00ol0003oooooooooo`0jooooo`03o`000?oo
oooooooo0:_ooooo000Hooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0>_ooool0
0ol0003oooooooooo`2/ooooo`006?ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo
03Wooooo00?o0000ooooooooool0[Oooool001Sooooo00?o0000ooooooooool06?ooool00ol0003o
ooooooooo`0iooooo`03o`000?oooooooooo0:kooooo000Hooooo`03o`000?oooooooooo01Oooooo
00?o0000ooooooooool0>Oooool00ol0003oooooooooo`2_ooooo`006?ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo03Oooooo0_l0002booooo`006?ooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo03Oooooo00?o0000ooooooooool0/_ooool001Sooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo0;?ooooo000Hooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`2dooooo`00
6?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0
]Oooool001Sooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0eooooo`03o`000?oo
oooooooo0;Kooooo000Hooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0=Oooool0
0ol0003oooooooooo`2gooooo`006?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
03Cooooo00?o0000ooooooooool0^?ooool001Sooooo00?o0000ooooooooool04_ooool00ol0003o
ooooooooo`0dooooo`03o`000?oooooooooo0;Wooooo000Hooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`2jooooo`006?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0^oooool001Oooooo0ooo0?lA
ooooo`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0_?ooool001Kooooo1Ooo0?l?oooo
o`03o`000?oooooooooo03?ooooo00?o0000ooooooooool0_Oooool001Gooooo1ooo0?l>ooooo`03
o`000?oooooooooo03;ooooo00?o0000ooooooooool0__ooool001Gooooo1ooo0?l=ooooo`03o`00
0?oooooooooo037ooooo0_l00031ooooo`005Oooool7ool0o`gooooo00?o0000ooooooooool0<?oo
ool00ol0003oooooooooo`31ooooo`005_ooool5ool0o`gooooo00?o0000ooooooooool0<?ooool0
0ol0003oooooooooo`32ooooo`005oooool3ool0o`gooooo00?o0000ooooooooool0<?ooool00ol0
003oooooooooo`33ooooo`006?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo02oo
oooo00?o0000ooooooooool0a?ooool001Sooooo00?o0000ooooooooool02oooool00ol0003ooooo
ooooo`0_ooooo`03o`000?oooooooooo0<Gooooo000Hooooo`03o`000?oooooooooo00_ooooo00?o
0000ooooooooool0;_ooool00ol0003oooooooooo`36ooooo`006?ooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo02kooooo00?o0000ooooooooool0aoooool001Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo0<Sooooo000Hooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool0;Oooool00ol0003oooooooooo`39ooooo`00
6?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo02gooooo00?o0000ooooooooool0
b_ooool001Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0/ooooo`03o`000?oo
oooooooo0<_ooooo000Hooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0;?ooool0
0ol0003oooooooooo`3<ooooo`006?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
02_ooooo0_l0003?ooooo`006?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo02[o
oooo00?o0000ooooooooool0coooool001Sooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0Zooooo`03o`000?oooooooooo0=3ooooo000Hooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool0:Oooool00ol0003oooooooooo`3Aooooo`006?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo02Wooooo00?o0000ooooooooool0d_ooool001Sooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`0Yooooo`03o`000?oooooooooo0=?ooooo000Gooooo`?o
o`3o1?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo0=Cooooo000Fooooo`Goo`3o
0_ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo0=Gooooo000Eooooo`Ooo`3o00?o
ooooo`000?ooool0:?ooool00ol0003oooooooooo`3Fooooo`005Oooool7ool0o`03o`000?oooooo
oooo02Oooooo00?o0000ooooooooool0eoooool001Gooooo1ooo0?lYooooo`03o`000?oooooooooo
0=Sooooo000Fooooo`Goo`3o00?o0000ooooooooool09_ooool00ol0003oooooooooo`3Iooooo`00
5oooool3ool0o`03o`000?oooooooooo02Kooooo00?o0000ooooooooool0f_ooool001Oooooo00Co
0000ooooool0003o00009oooool00ol0003oooooooooo`3Kooooo`005_ooool00ol0003ooooooooo
o`02o`0002Gooooo0_l0003Nooooo`005_ooool01Ol0003oooooo`000?oooooo000002Cooooo00?o
0000ooooooooool0g_ooool001Kooooo00Go0000ooooool0003oooooo`00000Sooooo`03o`000?oo
oooooooo0=oooooo000Eooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool07oooool0
0ol0003oooooooooo`3Pooooo`005Oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo
01kooooo00?o0000ooooooooool0hOooool001Cooooo00?o0000ooooool000001Oooool00ol0003o
ooooooooo`0Looooo`03o`000?oooooooooo0>;ooooo000Dooooo`;o00001_ooool00ol0003ooooo
ooooo`0Kooooo`03o`000?oooooooooo0>?ooooo000Dooooo`;o00001_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo0>Cooooo000Aooooo`?oo`3o00?o0000ooooooooool01_ooool3
ool0oaSooooo00?o0000ooooooooool0iOooool0013ooooo1Ooo0?l7ooooo`Goo`3o5_ooool00ol0
003oooooooooo`3Vooooo`003oooool7ool0o`Gooooo1ooo0?lDooooo`03o`000?oooooooooo0>Oo
oooo000?ooooo`Ooo`3o1Oooool7ool0oa?ooooo00?o0000ooooooooool0j?ooool000oooooo1ooo
0?l5ooooo`Ooo`3o4_ooool00ol0003oooooooooo`3Yooooo`004?ooool5ool0o`Oooooo1Ooo0?lA
ooooo`;o0000k?ooool0017ooooo0ooo0?l9ooooo`?oo`3o4Oooool00ol0003oooooooooo`3/oooo
o`004Oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00kooooo00?o0000oooooooo
ool0kOooool0017ooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo0>kooooo000@ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`3_ooooo`004?ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooooooool0l?ooool0013ooooo00?o0000ooooooooool03_ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo0?7ooooo000?ooooo`03o`000?oooooooooo00oo
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`3booooo`003oooool00ol0003ooooo
ooooo`0@ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0loooool000oooooo00?o
0000ooooooooool04?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo0?Cooooo000>
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`3e
ooooo`002oooool3ool0o`03o`000?oooooooooo01;ooooo0ooo0?l00ooooooo0000ooooo`3goooo
o`002_ooool5ool0oa?ooooo1Ooo0?oiooooo`002Oooool7ool0oa7ooooo1ooo0?ohooooo`002Ooo
ool7ool0oa7ooooo1ooo0?ohooooo`002Oooool7ool0oa7ooooo1ooo0?ohooooo`002_ooool5ool0
oa?ooooo1Ooo0?oiooooo`002oooool3ool0oaGooooo0ooo0?ojooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, 0.229909, \
0.00383279, 0.00144217}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[ran2DTD]\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Stag Hunt on a lattice", "Section",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  StyleBox["Construct the payoff matrix as used in Brian Skryrm's definition \
of the Stag Hunt, from chapter 3 of ",
    FontSize->14],
  StyleBox["The Stag Hunt and the Evolution of Social Structure.",
    FontSize->14,
    FontSlant->"Italic"]
}], "Text",
  FontSize->16,
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"stagHunt", "=", 
        RowBox[{"(", GridBox[{
              {\({3, 3}\), \({0, 2}\)},
              {\({2, 0}\), \({2, 2}\)}
              }], ")"}]}], 
      ";"}], "\n", \(stagColors = {\ LightGray, 
          Black};\), "\n", \(StrategyLegend[
        stagColors, {"\<Hunt Stag\>", "\<Hunt Rabbit\>"}];\)}], "Input",
  CellLabel->"In[32]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.666667],
                Background->GrayLevel[0.666667]],
              BoxMargins->False], \(Hunt\ Stag\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0],
                Background->GrayLevel[0]],
              BoxMargins->False], \(Hunt\ Rabbit\)}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[32]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["von Neumann", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(vnStagModel = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] VonNeumann[10, 10], \[IndentingNewLine]PayoffMatrix \[Rule] 
            stagHunt, \[IndentingNewLine]StrategyColors \[Rule] 
            stagColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[35]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[
        ExtractGraph[vnStagModel], \[IndentingNewLine]VertexStyle \[Rule] 
          Box[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.663 .663 .663 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
0 0 0 r
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
0 0 0 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.663 .663 .663 r
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
0 0 0 r
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.663 .663 .663 r
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
0 0 0 r
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
0 0 0 r
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
.663 .663 .663 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
0 0 0 r
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
0 0 0 r
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
.663 .663 .663 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
0 0 0 r
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
0 0 0 r
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.663 .663 .663 r
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
0 0 0 r
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
.663 .663 .663 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
0 0 0 r
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.663 .663 .663 r
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
0 0 0 r
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.663 .663 .663 r
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.663 .663 .663 r
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
0 0 0 r
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.663 .663 .663 r
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
0 0 0 r
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 0 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.663 .663 .663 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
0 0 0 r
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.663 .663 .663 r
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
0 0 0 r
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.663 .663 .663 r
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
0 0 0 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.663 .663 .663 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
0 0 0 r
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.663 .663 .663 r
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
0 0 0 r
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.663 .663 .663 r
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
0 0 0 r
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[36]:=",
  ImageSize->{232, 232},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003X0000j2000`400?l00000o`00003oj?ooool0
0>Sooooo003Xooooo`00j?ooool00>Sooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009
ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3onYZJT9
ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009oooo
o`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?ojVYZ@Wooooo3_nYZJT5oooo
o`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo
00002Oooool>o`0000Wooooo3ol00008ooooo`ooZJVY2Oooool>ojVYZ@Gooooo0003ooooo`ko0000
2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko0000
2Oooool?o`0000Sooooo3onYZJT9ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`ooZJVY2?oo
ool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo00002?oo
ool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>
o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`ooZJVY2Oooool>
ojVYZ@Gooooo0003oooooaOo00003onYZJVDo`0000ooZJVY2Ol0000>ojVYZ@Gooooo0003ooooo`ko
00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko
00002Oooool?o`0000Sooooo3onYZJT9ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`ooZJVY
2?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo0000
2?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Ooo
ool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`ooZJVY2Ooo
ool>ojVYZ@Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?
o`0000Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3onYZJT9ooooo`koZJVY1Oooool0
00?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`00
00Wooooo3_l00009ooooo`oo00002?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>o`0000Wo
oooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Wo
oooo3ol00008ooooo`ooZJVY2Oooool>ojVYZ@Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo
3ol00009ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo
3onYZJT9ooooo`koZJVY1Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool02Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo
3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?ojVYZ@Wooooo
3_l00005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3onY
ZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Gooooo0003
ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009
ooooo`ko00002Oooool?o`0000Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_l00009oooo
o`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009oooo
o`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo
00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo
ZJVY2Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY
2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3onYZJT9ooooo`ko0000
1Oooool000?ooooo5ol0000?ojVYZB3o00003_nYZJT9o`0000ooZJVYC_l0000?ojVYZAOo00001Ooo
ool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?
o`0000Wooooo3_l00009ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`000oooool>o`00
00Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>o`00
00Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?ojVYZ@So
oooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000So
oooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo
3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?ojVYZ@Wooooo
3_l00005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3onY
ZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Gooooo0003
ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009
ooooo`ko00002Oooool?o`0000Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_l00009oooo
o`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009oooo
o`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo
00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo
ZJVY2Oooool>o`0000Gooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo0003oooo
o`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9oooo
o`ko00002Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`oo
ZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00009ooooo`oo
ZJVY2?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY
2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Wooooo3onYZJT8ooooo`ooZJVY
2Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Ooo
ool?o`0000Sooooo3onYZJT9ooooo`ko00002Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY1Ooo
ool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?
ojVYZ@Wooooo3_l00009ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>ojVY
Z@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY2Oooool>o`00
00Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@So
oooo3onYZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9ooooo`ko00002Oooool?ojVYZ@So
oooo3onYZJT9ooooo`koZJVY1Oooool000?ooooo3_nYZJT9o`0000ooZJVY2?l0000?ojVYZ@Wo0000
3_nYZJTPo`0000ooZJVY8?l0000?ojVYZ@So00003onYZJT9o`0000koZJVY1Oooool000?ooooo3_nY
ZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wooooo3_l0
0009ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3onYZJT8
ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Wooooo3onYZJT8
ooooo`ooZJVY2Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9oooo
o`koZJVY2Oooool?o`0000Sooooo3onYZJT9ooooo`ko00002Oooool?ojVYZ@Sooooo3onYZJT9oooo
o`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo
00002?ooool?ojVYZ@Wooooo3_l00009ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT5ooooo`00
0oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY
2Oooool>o`0000Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Ooo
ool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9ooooo`ko00002Ooo
ool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?
ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00009ooooo`ooZJVY2?ooool?
ojVYZ@Wooooo3_nYZJT5ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo
00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`09ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>
ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>
o`0000Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?ojVY
Z@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`ko00001Oooool000?o
oooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wo
oooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`000oooool>ojVYZ@Wooooo
3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo
3ol00008ooooo`oo00002Oooool>o`0000Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onY
ZJT9ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol0
0009ooooo`ko00001Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9
ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00005
ooooo`000oooool>ojVYZ@Wo00003onYZJT8o`0000ooZJVY2Ol0000>ojVYZ@Wo00003onYZJTPo`00
00koZJVYA_l00005ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wo
oooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>o`0000Go
oooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?ojVYZ@Sooooo
3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`ko00001Oooool000?ooooo3_nY
ZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_nY
ZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`000oooool>ojVYZ@Wooooo3onYZJT8
ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008
ooooo`oo00002Oooool>o`0000Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9oooo
o`koZJVY2Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009oooo
o`ko00001Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo
ZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`00
0oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo0000
2Oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>o`0000Gooooo000:ooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003o
ooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo
00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo
000:ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000
ooooooooool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo000:oooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo00Wooooo000:ooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05Oooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00Wooooo0003ooooo`koZJVY
2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko0000
2Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?oo
ool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?oo
ool?o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>
o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>
ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVY
Z@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY1Oooool000?o
oooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wo
oooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo
3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo
3onYZJT8ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Ol0000?ojVYZCOo00003onY
ZJTgo`0000ooZJVY8?l0000>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009
ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009
ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009oooo
o`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT5oooo
o`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo
00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`koZJVY
2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko0000
2Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?oo
ool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?oo
ool?o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo00002Oooool>
o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>
ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVY
Z@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY1Oooool000?o
oooo3_nYZJT9ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wo
oooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT5ooooo`002_ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0D
ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09
ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`00
2_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`09ooooo`000oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>ojVY
Z@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVY
Z@Gooooo0003ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@So
oooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo
3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo
3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3ol0
0008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol0
0008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009
ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009
ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9oooo
o`ooZJVY2?ooool?ojVYZ@Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT5oooo
o`000oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`oo
ZJVY2Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`koZJVY
>?l0000>ojVYZ@Wo00003onYZJT8o`0000ooZJVYCol0000>ojVYZ@Gooooo0003ooooo`koZJVY2Ooo
ool?o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Ooo
ool?o`0000Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo3_nYZJT9ooooo`oo00002?ooool?
o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00009ooooo`oo00002?ooool?
o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>ojVY
Z@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVY
Z@Gooooo0003ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@So
oooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo
3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo
3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT5ooooo`000oooool>ojVYZ@Wooooo3ol0
0008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol0
0008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009
ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009
ooooo`koZJVY1Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eoooo
o`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Ooo
ool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooo
oooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[o
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool02Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00009
ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00005
ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol00008oooo
o`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Gooooo0003ooooo`ko
ZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko
ZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_nYZJT9ooooo`ooZJVY
2?ooool?ojVYZ@Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY
2?ooool?ojVYZ@Wooooo3_l00005ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Ooo
ool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Ooo
ool>o`0000Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?
o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00001Oooool0
00?ooooo3_nYZJT9o`0000ooZJVY2?l0000?ojVYZDoo00003_nYZJT9o`0000ooZJVY2?l0000?ojVY
ZAOo00001Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00009ooooo`oo
00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00005ooooo`00
0oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Wooooo3ol00008ooooo`oo0000
2Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`0000Gooooo0003ooooo`koZJVY2Ooo
ool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY2Ooo
ool?ojVYZ@Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_nYZJT9ooooo`ooZJVY2?ooool?
ojVYZ@Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?
ojVYZ@Wooooo3_l00005ooooo`000oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`00
00Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>o`00
00Gooooo0003ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00002Oooool?o`0000So
oooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo
3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo
3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_l00005ooooo`002_ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooooooo
o`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`000oooool>o`0000Wooooo
3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo
3ol00008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol0
0009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3ol0
0009ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009
ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT5
ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8oooo
o`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`ko
00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko
00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`ooZJVY
2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo0000
2?ooool?o`0000Wooooo3_nYZJT5ooooo`000ooooolGo`0000ooZJVY=ol0000?ojVYZFKo00003_nY
ZJT5ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8
ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Gooooo0003oooo
o`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009oooo
o`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY1Oooool000?ooooo3_l00009ooooo`oo
ZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`oo
00002?ooool?o`0000Wooooo3_nYZJT5ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo0000
2Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo0000
2Oooool>ojVYZ@Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Ooo
ool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY1Ooo
ool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_l00009ooooo`ooZJVY2?ooool?
o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT5ooooo`000oooool>o`00
00Wooooo3onYZJT8ooooo`oo00002Oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>o`00
00Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Gooooo0003ooooo`ko00002Oooool?ojVYZ@So
oooo3ol00009ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`ko00002Oooool?o`0000So
oooo3ol00009ooooo`koZJVY1Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o
0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0E
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0E
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0E
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000oooo
ooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03
o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
2Oooool000[ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Cooooo00?o0000ooooooooool02Oooool000[ooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool0
0ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool02Oooool0
00?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?ooool?o`00
00Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`000oooool>o`0000Wo
oooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>o`0000Wo
oooo3ol00008ooooo`oo00002Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?ojVYZ@Sooooo
3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`ko00002Oooool?o`0000Sooooo
3ol00009ooooo`ko00001Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nY
ZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_l0
0005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008
ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Gooooo0003oooo
o`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009oooo
o`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko00001Oooool000?ooooo3_l00009ooooo`oo
ZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo
00002?ooool?o`0000Wooooo3_l00005ooooo`000ooooolGo`0000ooZJVY8?l0000>ojVYZHco0000
1Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?oo
ool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`000oooool>
o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo3ol00008ooooo`oo00002Oooool>
o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?ojVY
Z@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol00009ooooo`ko00002Oooool?o`00
00Sooooo3ol00009ooooo`ko00001Oooool000?ooooo3_l00009ooooo`ooZJVY2?ooool?o`0000Wo
oooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00009ooooo`oo00002?ooool?o`0000Wo
oooo3_l00005ooooo`000oooool>o`0000Wooooo3onYZJT8ooooo`oo00002Oooool>ojVYZ@Wooooo
3ol00008ooooo`oo00002Oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>o`0000Gooooo
0003ooooo`ko00002Oooool?ojVYZ@Sooooo3ol00009ooooo`koZJVY2Oooool?o`0000Sooooo3ol0
0009ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko00001Oooool000?ooooo3_l00009
ooooo`ooZJVY2?ooool?o`0000Wooooo3_nYZJT9ooooo`oo00002?ooool?o`0000Wooooo3_l00009
ooooo`oo00002?ooool?o`0000Wooooo3_l00005ooooo`002_ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Doooo
o`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09oooo
o`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000oooooooo
ool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooo
oooo01Gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0
003oooooooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`002_ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05?ooool00ol0003ooooo
ooooo`09ooooo`002_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05?ooool00ol0003oooooooooo`09ooooo`000oooool>o`0000Wooooo3ol00008
ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008
ooooo`ooZJVY2Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?o`0000Sooooo3ol00009oooo
o`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9oooo
o`ko00001Oooool000?ooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`oo
ZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`00
0oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY
2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Gooooo0003ooooo`ko00002Ooo
ool?o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Ooo
ool?o`0000Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_l00009ooooo`oo00002?ooool?
o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?
ojVYZ@Wooooo3_l00005ooooo`000ooooom6o`0000koZJVY2Ol0000?ojVYZ@So00003onYZJT9o`00
00koZJVY8?l0000?ojVYZAOo00001Oooool000?ooooo3_l00009ooooo`oo00002?ooool?o`0000Wo
oooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wo
oooo3_l00005ooooo`000oooool>o`0000Wooooo3ol00008ooooo`oo00002Oooool>ojVYZ@Wooooo
3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008ooooo`ooZJVY2Oooool>o`0000Gooooo
0003ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`koZJVY2Oooool?ojVYZ@Sooooo3onY
ZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9ooooo`ko00001Oooool000?ooooo3_l00009
ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY2?ooool?ojVYZ@Wooooo3_nYZJT9
ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`000oooool>o`0000Wooooo3ol00008oooo
o`oo00002Oooool>ojVYZ@Wooooo3onYZJT8ooooo`ooZJVY2Oooool>ojVYZ@Wooooo3ol00008oooo
o`ooZJVY2Oooool>o`0000Gooooo0003ooooo`ko00002Oooool?o`0000Sooooo3ol00009ooooo`ko
ZJVY2Oooool?ojVYZ@Sooooo3onYZJT9ooooo`koZJVY2Oooool?o`0000Sooooo3onYZJT9ooooo`ko
00001Oooool000?ooooo3_l00009ooooo`oo00002?ooool?o`0000Wooooo3_nYZJT9ooooo`ooZJVY
2?ooool?ojVYZ@Wooooo3_nYZJT9ooooo`oo00002?ooool?ojVYZ@Wooooo3_l00005ooooo`00j?oo
ool00>Sooooo003Xooooo`00\
\>"],
  ImageRangeCache->{{{0, 231}, {231, 0}} -> {-0.0500054, -0.0500054, \
0.00476195, 0.00476195}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\n", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[vnStagModel]]; \[IndentingNewLine]Evolve[
          vnStagModel], {9}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[37]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraphArray[\[IndentingNewLine]Partition[l, 3], 
        VertexStyle \[Rule] Box[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0238095 0.31746 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.321429 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
0 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.663 .663 .663 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
0 0 0 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.0238095 0.64881 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
0 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.663 .663 .663 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
0 0 0 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.0238095 0.97619 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
0 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.663 .663 .663 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
0 0 0 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.35119 0.321429 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
0 0 0 r
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 0 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.35119 0.64881 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
0 0 0 r
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
0 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.35119 0.97619 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.663 .663 .663 r
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
0 0 0 r
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.663 .663 .663 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
0 0 0 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
0 0 0 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.663 .663 .663 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
0 0 0 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.678571 0.321429 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.663 .663 .663 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
0 0 0 r
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
0 0 0 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.663 .663 .663 r
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
0 0 0 r
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.663 .663 .663 r
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
0 0 0 r
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
0 0 0 r
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
.663 .663 .663 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
0 0 0 r
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
0 0 0 r
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
.663 .663 .663 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
0 0 0 r
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
0 0 0 r
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.663 .663 .663 r
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
0 0 0 r
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
.663 .663 .663 r
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
0 0 0 r
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.663 .663 .663 r
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
0 0 0 r
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.663 .663 .663 r
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.663 .663 .663 r
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
0 0 0 r
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.663 .663 .663 r
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
0 0 0 r
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 0 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.663 .663 .663 r
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
0 0 0 r
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.663 .663 .663 r
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
0 0 0 r
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.663 .663 .663 r
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
0 0 0 r
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.663 .663 .663 r
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
0 0 0 r
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.663 .663 .663 r
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
0 0 0 r
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.663 .663 .663 r
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
0 0 0 r
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.678571 0.64881 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.663 .663 .663 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
0 0 0 r
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
0 0 0 r
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.663 .663 .663 r
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
0 0 0 r
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
0 0 0 r
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
0 0 0 r
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.663 .663 .663 r
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
0 0 0 r
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.663 .663 .663 r
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 0 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.663 .663 .663 r
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
0 0 0 r
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.678571 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.01364 .01364 m
.01364 .07727 L
.07727 .07727 L
.07727 .01364 L
F
.663 .663 .663 r
.11465 .01364 m
.11465 .07727 L
.17828 .07727 L
.17828 .01364 L
F
.21566 .01364 m
.21566 .07727 L
.27929 .07727 L
.27929 .01364 L
F
0 0 0 r
.31667 .01364 m
.31667 .07727 L
.3803 .07727 L
.3803 .01364 L
F
.41768 .01364 m
.41768 .07727 L
.48131 .07727 L
.48131 .01364 L
F
.51869 .01364 m
.51869 .07727 L
.58232 .07727 L
.58232 .01364 L
F
.6197 .01364 m
.6197 .07727 L
.68333 .07727 L
.68333 .01364 L
F
.72071 .01364 m
.72071 .07727 L
.78434 .07727 L
.78434 .01364 L
F
.663 .663 .663 r
.82172 .01364 m
.82172 .07727 L
.88535 .07727 L
.88535 .01364 L
F
.92273 .01364 m
.92273 .07727 L
.98636 .07727 L
.98636 .01364 L
F
.01364 .11465 m
.01364 .17828 L
.07727 .17828 L
.07727 .11465 L
F
.11465 .11465 m
.11465 .17828 L
.17828 .17828 L
.17828 .11465 L
F
.21566 .11465 m
.21566 .17828 L
.27929 .17828 L
.27929 .11465 L
F
.31667 .11465 m
.31667 .17828 L
.3803 .17828 L
.3803 .11465 L
F
0 0 0 r
.41768 .11465 m
.41768 .17828 L
.48131 .17828 L
.48131 .11465 L
F
.51869 .11465 m
.51869 .17828 L
.58232 .17828 L
.58232 .11465 L
F
.6197 .11465 m
.6197 .17828 L
.68333 .17828 L
.68333 .11465 L
F
.72071 .11465 m
.72071 .17828 L
.78434 .17828 L
.78434 .11465 L
F
.663 .663 .663 r
.82172 .11465 m
.82172 .17828 L
.88535 .17828 L
.88535 .11465 L
F
0 0 0 r
.92273 .11465 m
.92273 .17828 L
.98636 .17828 L
.98636 .11465 L
F
.663 .663 .663 r
.01364 .21566 m
.01364 .27929 L
.07727 .27929 L
.07727 .21566 L
F
.11465 .21566 m
.11465 .27929 L
.17828 .27929 L
.17828 .21566 L
F
.21566 .21566 m
.21566 .27929 L
.27929 .27929 L
.27929 .21566 L
F
.31667 .21566 m
.31667 .27929 L
.3803 .27929 L
.3803 .21566 L
F
.41768 .21566 m
.41768 .27929 L
.48131 .27929 L
.48131 .21566 L
F
0 0 0 r
.51869 .21566 m
.51869 .27929 L
.58232 .27929 L
.58232 .21566 L
F
.6197 .21566 m
.6197 .27929 L
.68333 .27929 L
.68333 .21566 L
F
.663 .663 .663 r
.72071 .21566 m
.72071 .27929 L
.78434 .27929 L
.78434 .21566 L
F
.82172 .21566 m
.82172 .27929 L
.88535 .27929 L
.88535 .21566 L
F
.92273 .21566 m
.92273 .27929 L
.98636 .27929 L
.98636 .21566 L
F
.01364 .31667 m
.01364 .3803 L
.07727 .3803 L
.07727 .31667 L
F
.11465 .31667 m
.11465 .3803 L
.17828 .3803 L
.17828 .31667 L
F
.21566 .31667 m
.21566 .3803 L
.27929 .3803 L
.27929 .31667 L
F
.31667 .31667 m
.31667 .3803 L
.3803 .3803 L
.3803 .31667 L
F
.41768 .31667 m
.41768 .3803 L
.48131 .3803 L
.48131 .31667 L
F
.51869 .31667 m
.51869 .3803 L
.58232 .3803 L
.58232 .31667 L
F
0 0 0 r
.6197 .31667 m
.6197 .3803 L
.68333 .3803 L
.68333 .31667 L
F
.72071 .31667 m
.72071 .3803 L
.78434 .3803 L
.78434 .31667 L
F
.663 .663 .663 r
.82172 .31667 m
.82172 .3803 L
.88535 .3803 L
.88535 .31667 L
F
0 0 0 r
.92273 .31667 m
.92273 .3803 L
.98636 .3803 L
.98636 .31667 L
F
.663 .663 .663 r
.01364 .41768 m
.01364 .48131 L
.07727 .48131 L
.07727 .41768 L
F
.11465 .41768 m
.11465 .48131 L
.17828 .48131 L
.17828 .41768 L
F
.21566 .41768 m
.21566 .48131 L
.27929 .48131 L
.27929 .41768 L
F
.31667 .41768 m
.31667 .48131 L
.3803 .48131 L
.3803 .41768 L
F
.41768 .41768 m
.41768 .48131 L
.48131 .48131 L
.48131 .41768 L
F
.51869 .41768 m
.51869 .48131 L
.58232 .48131 L
.58232 .41768 L
F
0 0 0 r
.6197 .41768 m
.6197 .48131 L
.68333 .48131 L
.68333 .41768 L
F
.72071 .41768 m
.72071 .48131 L
.78434 .48131 L
.78434 .41768 L
F
.82172 .41768 m
.82172 .48131 L
.88535 .48131 L
.88535 .41768 L
F
.92273 .41768 m
.92273 .48131 L
.98636 .48131 L
.98636 .41768 L
F
.663 .663 .663 r
.01364 .51869 m
.01364 .58232 L
.07727 .58232 L
.07727 .51869 L
F
.11465 .51869 m
.11465 .58232 L
.17828 .58232 L
.17828 .51869 L
F
.21566 .51869 m
.21566 .58232 L
.27929 .58232 L
.27929 .51869 L
F
.31667 .51869 m
.31667 .58232 L
.3803 .58232 L
.3803 .51869 L
F
.41768 .51869 m
.41768 .58232 L
.48131 .58232 L
.48131 .51869 L
F
.51869 .51869 m
.51869 .58232 L
.58232 .58232 L
.58232 .51869 L
F
0 0 0 r
.6197 .51869 m
.6197 .58232 L
.68333 .58232 L
.68333 .51869 L
F
.72071 .51869 m
.72071 .58232 L
.78434 .58232 L
.78434 .51869 L
F
.82172 .51869 m
.82172 .58232 L
.88535 .58232 L
.88535 .51869 L
F
.92273 .51869 m
.92273 .58232 L
.98636 .58232 L
.98636 .51869 L
F
.663 .663 .663 r
.01364 .6197 m
.01364 .68333 L
.07727 .68333 L
.07727 .6197 L
F
.11465 .6197 m
.11465 .68333 L
.17828 .68333 L
.17828 .6197 L
F
.21566 .6197 m
.21566 .68333 L
.27929 .68333 L
.27929 .6197 L
F
0 0 0 r
.31667 .6197 m
.31667 .68333 L
.3803 .68333 L
.3803 .6197 L
F
.663 .663 .663 r
.41768 .6197 m
.41768 .68333 L
.48131 .68333 L
.48131 .6197 L
F
0 0 0 r
.51869 .6197 m
.51869 .68333 L
.58232 .68333 L
.58232 .6197 L
F
.6197 .6197 m
.6197 .68333 L
.68333 .68333 L
.68333 .6197 L
F
.72071 .6197 m
.72071 .68333 L
.78434 .68333 L
.78434 .6197 L
F
.82172 .6197 m
.82172 .68333 L
.88535 .68333 L
.88535 .6197 L
F
.92273 .6197 m
.92273 .68333 L
.98636 .68333 L
.98636 .6197 L
F
.01364 .72071 m
.01364 .78434 L
.07727 .78434 L
.07727 .72071 L
F
.663 .663 .663 r
.11465 .72071 m
.11465 .78434 L
.17828 .78434 L
.17828 .72071 L
F
0 0 0 r
.21566 .72071 m
.21566 .78434 L
.27929 .78434 L
.27929 .72071 L
F
.31667 .72071 m
.31667 .78434 L
.3803 .78434 L
.3803 .72071 L
F
.41768 .72071 m
.41768 .78434 L
.48131 .78434 L
.48131 .72071 L
F
.51869 .72071 m
.51869 .78434 L
.58232 .78434 L
.58232 .72071 L
F
.6197 .72071 m
.6197 .78434 L
.68333 .78434 L
.68333 .72071 L
F
.72071 .72071 m
.72071 .78434 L
.78434 .78434 L
.78434 .72071 L
F
.82172 .72071 m
.82172 .78434 L
.88535 .78434 L
.88535 .72071 L
F
.92273 .72071 m
.92273 .78434 L
.98636 .78434 L
.98636 .72071 L
F
.01364 .82172 m
.01364 .88535 L
.07727 .88535 L
.07727 .82172 L
F
.11465 .82172 m
.11465 .88535 L
.17828 .88535 L
.17828 .82172 L
F
.21566 .82172 m
.21566 .88535 L
.27929 .88535 L
.27929 .82172 L
F
.31667 .82172 m
.31667 .88535 L
.3803 .88535 L
.3803 .82172 L
F
.41768 .82172 m
.41768 .88535 L
.48131 .88535 L
.48131 .82172 L
F
.51869 .82172 m
.51869 .88535 L
.58232 .88535 L
.58232 .82172 L
F
.6197 .82172 m
.6197 .88535 L
.68333 .88535 L
.68333 .82172 L
F
.72071 .82172 m
.72071 .88535 L
.78434 .88535 L
.78434 .82172 L
F
.82172 .82172 m
.82172 .88535 L
.88535 .88535 L
.88535 .82172 L
F
.92273 .82172 m
.92273 .88535 L
.98636 .88535 L
.98636 .82172 L
F
.01364 .92273 m
.01364 .98636 L
.07727 .98636 L
.07727 .92273 L
F
.11465 .92273 m
.11465 .98636 L
.17828 .98636 L
.17828 .92273 L
F
.21566 .92273 m
.21566 .98636 L
.27929 .98636 L
.27929 .92273 L
F
.31667 .92273 m
.31667 .98636 L
.3803 .98636 L
.3803 .92273 L
F
.41768 .92273 m
.41768 .98636 L
.48131 .98636 L
.48131 .92273 L
F
.51869 .92273 m
.51869 .98636 L
.58232 .98636 L
.58232 .92273 L
F
.6197 .92273 m
.6197 .98636 L
.68333 .98636 L
.68333 .92273 L
F
.72071 .92273 m
.72071 .98636 L
.78434 .98636 L
.78434 .92273 L
F
.82172 .92273 m
.82172 .98636 L
.88535 .98636 L
.88535 .92273 L
F
.92273 .92273 m
.92273 .98636 L
.98636 .98636 L
.98636 .92273 L
F
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[39]:=",
  ImageSize->{386.812, 386.812},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000620001PR000`400?l00000o`00003ooooooon3
ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3
ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3
ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004oooo
o`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4oooo
o`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005oooo
o`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5oooo
o`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004oooo
o`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo
00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo
ZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo
00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
ZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo
00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo0000
1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo0000
3oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo0000
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY
1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo0000
1Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Co00001onYZJT4o`0000SoZJVYDOl0
000?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJUAo`0000oooooo1onYZJT4o`0000SoZJVY1?l0
0007ojVYZE;o00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005
ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?
ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004
ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005
ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4oooo
o`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005oooo
o`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005oooo
o`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`So
ZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo
00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So0000
1?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So0000
1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo0000
3oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo0000
1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000>ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool0
00kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0<ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo
1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo
1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo
1onYZJT4ooooo`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0
000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5
ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005
ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004
ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Co00001onYZJT4o`00
00SoZJVY1?l00007ojVYZC?o00001onYZJT<o`0000oooooo1onYZJT4o`0000OoZJVY1Ol00007ojVY
Z@Co00002?nYZJTbo`0000SoZJVY2ol0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`00
00OoZJVY<ol00007ojVYZ@co00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Go
oooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Go
oooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0
000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000kooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<oooo
o`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0<ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7
ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7
ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7
o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7
ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`SoZJVY1?ooool7ojVY
Z@Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVY
Z@Gooooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY3oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`00
00Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:ooooo`SoZJVY1?l00007ojVYZ@Co
00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZA_o00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@oo
oooo1onYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY7?l00007ojVYZ@Co
00002?nYZJT4o`0000OoZJVY3oooool7ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co
00002?nYZJTKo`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT;ooooo`002_ooool8ojVYZ@Cooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol0
0004ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY3oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:
ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004
ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT;
ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`So00001?ooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@oooooo1onYZJT4oooo
o`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY2oooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`SoZJVY1?ooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`00
00Cooooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`00
00oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVY
Z@Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Co
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Co
oooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Co
oooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[ooooo
2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo
2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo
000:ooooo`SoZJVY1?l00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nY
ZJTKo`0000OoZJVY3?l0000?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l0
0007ojVYZ@Go00001onYZJTKo`0000SoZJVY2ol0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onY
ZJT5o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJTLo`0000OoZJVY3?l0000;ooooo`002_ooool8
ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7
o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7
ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8
ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool0
00[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVY
Z@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`00
00Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`00
00_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Go
oooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Co
oooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo
1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo
1onYZJT4ooooo`So00002oooool000kooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`SoZJVY
1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY
1?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So0000
1?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_oo
ool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?oo
ool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Ooo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?oo
ool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00002ooo
ool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7
ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`OoZJVY1Oooool7
o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7
ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8
o`0000_ooooo000:ooooo`SoZJVY1?l00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVY
Z@Co00002?nYZJT4o`0000OoZJVY8ol0000?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJT4o`00
00SoZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000OoZJVY8ol0000?ooooo`OoZJVY1?l00008ojVY
Z@Co00001onYZJT5o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT4o`0000SoZJVY8ol0000;oooo
o`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`Oo
ZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo
00001?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So
00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY
1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`OoZJVY
1Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY
1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo0000
1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Ooo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00003ooo
ool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?oo
ool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Ooo
ool7o`0000Cooooo2?l0000;ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:
ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004
ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;
ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4oooo
o`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005oooo
o`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004oooo
o`So00002oooool000[ooooo2?nYZJT4o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000Oo
ZJVY1?l00008ojVYZBko00003oooool7ojVYZ@Co00001onYZJT5o`0000OoZJVY1?l00008ojVYZ@Co
00001onYZJT5o`0000OoZJVY;_l0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000Oo
ZJVY1?l00008ojVYZ@Co00001onYZJT_o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004
ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:oooo
o`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004oooo
o`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;oooo
o`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVY
Z@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oo
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nY
ZJT4o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000OoZJVY>_l0000?ooooo`OoZJVY1?l0
0007ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l00007ojVYZC[o00003oooool7ojVYZ@Co00002?nY
ZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJTjo`0000_ooooo000:ooooo`SoZJVY1?ooool7
ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7
o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7
o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVY
Z@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`00
00Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[o
oooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Co
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Co
oooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_o
oooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0B
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?l00004ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So0000
1?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo0000
3oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo0000
1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?oo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?oo
ool7o`0000oooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?oo
ool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool<
o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJU6o`0000oooooo2ol00007ojVYZ@Go00001onYZJT4
o`0000SoZJVYAOl0000?ooooo`_o00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZDKo00002oooool0
00[ooooo2?l00004ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`00
00Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7ojVY
Z@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`00
00Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`00
00_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Co
oooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Go
oooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Co
oooo2?l0000;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo
1ol00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo
1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo
1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0
000?ooooo`Oo00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00001?ooool8o`0000_ooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`002_oo
ool8o`0000Cooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?oo
ool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Ooo
ool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?oo
ool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002ooo
ool000[ooooo2?l00004ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8
o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7
o`0000Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7
o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7
o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8
o`0000_ooooo000:ooooo`So00001?ooool7o`0000Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`00
00Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`00
00Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`00
00Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`So00001?ooool7ojVY
Z@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`00
00Cooooo2?l0000;ooooo`002_oooolGo`0000SoZJVYDOl0000?oooooaOo00001onYZJUAo`0000oo
oooo5ol00007ojVYZE;o00002oooool000[ooooo2?l00004ooooo`Oo00001?ooool8ojVYZ@Cooooo
1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo
1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo
1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo
2?l00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7o`0000Cooooo2?nY
ZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol0
0004ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004
ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?
ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004oooo
o`Oo00001?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4oooo
o`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00003oooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000>ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`00
3_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
3?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0<ooooo`002_ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`00
00oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Go
oooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Go
oooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooog3o00003oooool7o`0000Cooooo
1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo
1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo
1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo
2?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0
0004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000ooooooKol0
000?ooooog3o00002oooool000[ooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005
ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?
ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004
ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005
ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005oooo
o`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00005oooo
o`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo
00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool00?ooooooPoooool00?ooooooPooo
ool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPooo
ool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPooo
ool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool000[ooooo2?nYZJT4ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So0000
1?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So0000
1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo0000
3oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo0000
1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?oo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?oo
ool8o`0000Cooooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?oo
ool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?oo
ool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8
ojVYZ@Co00001onYZJT4o`0000SoZJVY?_l00007ojVYZ@co00003oooool7ojVYZ@Co00001onYZJT5
o`0000OoZJVY?_l00008ojVYZ@_o00003oooool7ojVYZ@Co00002?nYZJT4o`0000OoZJVY?ol00007
ojVYZ@co00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005oooo
o`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?oooo
o`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004oooo
o`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005oooo
o`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo
00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo
00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY
1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo0000
1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So0000
1?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?oo
ool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?oo
ool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003ooo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000>ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000ko
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00co
oooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`0<ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol0
0004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo1ol0
0005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1onY
ZJT4ooooo`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5
ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?
ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004
ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005
ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5oooo
o`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005oooo
o`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Co00001onYZJT4o`0000So
ZJVY1?l00007ojVYZC?o00001onYZJT<o`0000oooooo1onYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co
00002?nYZJTbo`0000SoZJVY2ol0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000Oo
ZJVY<ol00007ojVYZ@co00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo
1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo
1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onY
ZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol0
0005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004
ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[ooooo2?nYZJT4oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004oooo
o`So00001?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4oooo
o`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000>ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`00
3_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
3?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0<ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVY
Z@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`00
00Cooooo1ol00005ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVY
Z@Gooooo1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY3oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@oooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo
2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT;ooooo`002_ooool8ojVYZ@Co00001onY
ZJT4o`0000SoZJVY1?l00007ojVYZ@Go00001onYZJTKo`0000SoZJVY1?l00007ojVYZ@Go00001onY
ZJT?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l00007ojVYZAco00001onY
ZJT4o`0000SoZJVY1?l00007ojVYZ@oooooo1onYZJT4o`0000SoZJVY1?l00007ojVYZ@Go00001onY
ZJT4o`0000SoZJVY6ol00007ojVYZ@Go00001onYZJT4o`0000SoZJVY2oooool000[ooooo2?nYZJT4
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY3oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:oooo
o`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004oooo
o`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT;oooo
o`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
00001?ooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@oooooo1onYZJT4ooooo`Oo
ZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY2oooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`002_ooool8ojVYZ@Cooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo
2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo
1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[ooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol0
0004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?nY
ZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:
ooooo`SoZJVY1?l00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJTK
o`0000OoZJVY3?l0000?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l00007
ojVYZ@Go00001onYZJTKo`0000SoZJVY2ol0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5
o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJTLo`0000OoZJVY3?l0000;ooooo`002_ooool8ojVY
Z@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`00
00Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8ojVY
Z@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[o
oooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Co
oooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_o
oooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo
1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo
2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1onY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol0
0005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onY
ZJT4ooooo`So00002oooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0B
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
4oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`SoZJVY1?oo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?oo
ool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?oo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8
ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7
ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7
ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8
o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool0
00[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVY
Z@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`OoZJVY1Oooool7o`00
00Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`00
00_ooooo000:ooooo`SoZJVY1?l00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co
00002?nYZJT4o`0000OoZJVY8ol0000?ooooo`OoZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000So
ZJVY1?l00007ojVYZ@Go00001onYZJT4o`0000OoZJVY8ol0000?ooooo`OoZJVY1?l00008ojVYZ@Co
00001onYZJT5o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT4o`0000SoZJVY8ol0000;ooooo`00
2_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY
1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo0000
1?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So0000
2oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?oo
ool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`OoZJVY1Ooo
ool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Ooo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?oo
ool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7
ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7
ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7
ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7
ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7
o`0000Cooooo2?l0000;ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:oooo
o`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004oooo
o`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;oooo
o`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`Oo
ZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo
00001?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So
00002oooool000[ooooo2?nYZJT4o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000OoZJVY
1?l00008ojVYZBko00003oooool7ojVYZ@Co00001onYZJT5o`0000OoZJVY1?l00008ojVYZ@Co0000
1onYZJT5o`0000OoZJVY;_l0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000OoZJVY
1?l00008ojVYZ@Co00001onYZJT_o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4
ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So
00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`00
3_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
3?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Go
oooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4
o`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000OoZJVY>_l0000?ooooo`OoZJVY1?l00007
ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l00007ojVYZC[o00003oooool7ojVYZ@Co00002?nYZJT4
o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJTjo`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVY
Z@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Co
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo
2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo
2?l00004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo
000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7ojVYZ@Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol0
0004ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
000;ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Boooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool04_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?l00004ooooo`OoZJVY1?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?oo
ool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?oo
ool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003ooo
ool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7
ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8
ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7
o`0000oooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7
o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`00
00Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`OoZJVY1Oooool7ojVY
Z@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`00
00Cooooo1ol0000?ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`00
00Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[o
oooo3?l00007ojVYZEgo00003oooool;o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJU5o`0000oo
oooo2ol00008ojVYZ@Co00001onYZJT5o`0000OoZJVYA_l0000;ooooo`002_ooool8o`0000Cooooo
1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol0000?ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l0
0004ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00003oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:
ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1onYZJT5
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004
ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;
ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
4_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`So00001?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00oooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Co
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol0000?ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Co
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo
L?l0000?ooooofoo00003ooooolGo`0000OoZJVYD_l0000;ooooo`002_ooool8o`0000Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
000?ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004
ooooo`Oo00003oooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:oooo
o`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1ol00005oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004oooo
o`So00001?ooool7o`0000oooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004oooo
o`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;oooo
o`002_ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So
00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo
00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo
00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo
00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So
00002oooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`0Booooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oo
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol0000?ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l0
0004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00003oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:
ooooog3o00003ooooom_o`0000ooooooL?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?oooo
o`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`Oo
00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo
00003oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo
00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So0000
1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo0000
1?ooool8o`0000Cooooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo0000
1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So0000
1?ooool7o`0000oooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So0000
1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`00oooo
oon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooo
oon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooo
oon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`00oooooon3ooooo`002_oo
ool8o`0000Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?oo
ool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@oooooo1ol00004ooooo`OoZJVY1Ooo
ool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?oo
ool8ojVYZ@Cooooo1onYZJT?ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`SoZJVY2ooo
ool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8
o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT?ooooo`Oo00001?ooool7
ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo00001Oooool7
o`0000Cooooo2?nYZJT4ooooo`OoZJVY3oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7
o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8
ojVYZ@_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`00
00Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY3oooool7o`00
00Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool7o`00
00Gooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@oooooo1ol00004ooooo`SoZJVY1?ooool7ojVY
Z@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7ojVY
Z@Cooooo2?nYZJT;ooooo`002_ooool<o`0000OoZJVYB_l00007ojVYZ@Go00001onYZJT?ooooo`_o
00001onYZJU:o`0000SoZJVY1?l00007ojVYZ@oooooo2ol00008ojVYZ@Co00001onYZJToo`0000Oo
ZJVY1?l00008ojVYZ@_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo0000
1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY
3oooool7o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo0000
1?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@oooooo1ol00004ooooo`SoZJVY
1?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo0000
1Oooool7ojVYZ@Cooooo2?nYZJT;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`So00001?oo
ool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Ooo
ool7ojVYZ@oooooo1ol00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001Ooo
ool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT?ooooo`Oo00001?oo
ool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?oo
ool7o`0000Gooooo1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7
ojVYZ@Gooooo1onYZJT?ooooo`Oo00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7
o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY3oooool7
o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000>ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo
000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooooooo
o`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool03?ooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT5oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?oooo
o`Oo00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004oooo
o`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005oooo
o`OoZJVY1?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo
ZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`Oo
00003oooool7o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo
00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool<o`0000OoZJVY4?l00007ojVYZ@Go0000
1onYZJTWo`0000OoZJVY3?l0000?ooooo`_o00001onYZJT5o`0000OoZJVY1?l00008ojVYZC;o0000
2?nYZJT;o`0000oooooo1onYZJT4o`0000SoZJVY1?l00007ojVYZ@Go00001onYZJTco`0000OoZJVY
3?l0000;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`So00001?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oooooo1ol0
0004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo1ol0
0005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1onY
ZJT4ooooo`So00002oooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT5
ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1ol0000?
ooooo`Oo00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004
ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005
ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004oooo
o`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT5oooo
o`Oo00003oooool7o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005oooo
o`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004oooo
o`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`So
00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo
ZJVY1Oooool7o`0000oooooo1ol00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So
00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000kooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
4oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003_ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool0
00kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00cooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Co
oooo2?nYZJT4ooooo`Oo00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY3oooool7ojVYZ@Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Co
oooo2?nYZJT;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1ol00004ooooo`SoZJVY1?ooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@oooooo
1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo
1ol00005ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo
1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol0
0004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY3oooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo1onY
ZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:ooooo`SoZJVY1?l00007ojVYZ@Co00002?nYZJT4
o`0000OoZJVY4?l00008ojVYZ@oo00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@oooooo1onYZJT4
o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY7?l00007ojVYZ@Co00002?nYZJT4
o`0000OoZJVY3oooool7ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJTK
o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Cooooo2?nYZJT4oooo
o`OoZJVY1Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT?oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004oooo
o`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY2oooool000[ooooo2?nYZJT4ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00004ooooo`So
ZJVY1?ooool7ojVYZ@Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
ZJVY3oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@_ooooo000:ooooo`SoZJVY
1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo0000
1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7ojVYZ@oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT;ooooo`002_oo
ool8ojVYZ@Cooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`SoZJVY1?oo
ool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Ooo
ool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1onYZJT?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?oo
ool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY2ooo
ool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?o
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00cooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4
ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000[ooooo2?nYZJT4oooo
o`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT4oooo
o`So00001?ooool7o`0000Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4oooo
o`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`So
ZJVY1?l00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZA3o00001onYZJTSo`0000oo
oooo1onYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT4o`0000OoZJVY9ol00008ojVYZ@_o
00003oooool7ojVYZ@Co00002?nYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJT4o`0000Oo
ZJVY7?l00007ojVYZ@co00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo
1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo
1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00003oooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo
1ol00005ooooo`OoZJVY1?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1ol0
0005ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000oooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004
ooooo`Oo00001Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0000?
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4
ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00002oooool000kooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`00
3_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
3?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7
ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00005ooooo`OoZJVY3oooool7
ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7
o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7
ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7
o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`00
00Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?nYZJT4ooooo`Oo00001Oooool7ojVY
Z@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVY
Z@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`00
00Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Co
oooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Go
oooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?l00007ojVYZAco0000
1onYZJTKo`0000SoZJVY4?l00007ojVYZ@oooooo1onYZJT4o`0000OoZJVY1Ol00007ojVYZ@Co0000
2?nYZJT4o`0000OoZJVY>_l0000?ooooo`OoZJVY1?l00008ojVYZ@Co00001onYZJT5o`0000OoZJVY
1?l00008ojVYZ@Co00001onYZJT_o`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0
0005ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1onYZJT4
ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?nYZJT4
ooooo`Oo00001Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4
ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4
ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004oooo
o`SoZJVY1?ooool7o`0000Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo
00001?ooool8ojVYZ@Cooooo1ol00005ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo
ZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So
00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`00
3_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
3?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
01?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Co
oooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7o`0000Go
oooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`SoZJVY1?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7ojVYZ@Co
oooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`SoZJVY1?ooool7o`0000Cooooo
2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8ojVYZ@Cooooo
1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo
1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo1ol0
0004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJTK
o`0000OoZJVY1Ol00007ojVYZ@Co00002?nYZJTWo`0000OoZJVY3oooool7ojVYZ@Co00001onYZJT@
o`0000SoZJVY1?l00007ojVYZ@Go00001onYZJT^o`0000oooooo1onYZJT4o`0000SoZJVY1?l00007
ojVYZ@Go00001onYZJT4o`0000SoZJVY1?l00007ojVYZBoo00002oooool000[ooooo2?nYZJT4oooo
o`Oo00001?ooool8o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00004oooo
o`So00001?ooool7o`0000Gooooo1onYZJT?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1ol00004oooo
o`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004oooo
o`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4oooo
o`OoZJVY1?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So
ZJVY1?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo
00001?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY3oooool7ojVYZ@Cooooo1onYZJT5ooooo`Oo
00001?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So
00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4ooooo`So
ZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`00
2_ooool8ojVYZ@Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY
1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@oooooo1onYZJT4ooooo`OoZJVY
1Oooool7o`0000Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo1ol00005ooooo`Oo0000
1?ooool8o`0000Cooooo1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY
1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So0000
2oooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
01?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo
00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool04oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`002_ooool8ojVYZ@Cooooo1onY
ZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7ojVYZ@Cooooo2?nY
ZJT4ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?l0
0004ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0
000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nYZJT4
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1onYZJT4
ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onYZJT4
ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004
ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?nYZJT4
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:oooo
o`SoZJVY1?l00007ojVYZ@Co00002?nYZJTWo`0000OoZJVY1?l00008ojVYZ@Co00001onYZJT<o`00
00oooooo1onYZJT4o`0000OoZJVY1Ol00007ojVYZA3o00001onYZJTjo`0000oooooo1onYZJT4o`00
00SoZJVY1?l00007ojVYZA3o00002?nYZJTjo`0000_ooooo000:ooooo`SoZJVY1?ooool7ojVYZ@Co
oooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`OoZJVY1?ooool8ojVYZ@Co
oooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5ooooo`OoZJVY1?ooool8o`0000Co
oooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oo
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8ojVYZ@Cooooo
1onYZJT4ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7ojVYZ@Cooooo
2?nYZJT4ooooo`OoZJVY1Oooool7o`0000oooooo1onYZJT4ooooo`OoZJVY1Oooool7ojVYZ@Cooooo
2?l00004ooooo`OoZJVY1Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol0000?ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00001?ooool8ojVYZ@Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?nY
ZJT4ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1onY
ZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol0000?ooooo`OoZJVY1?ooool7ojVYZ@Gooooo1onY
ZJT4ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0
0004ooooo`Oo00003oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1Oooool7o`0000Cooooo2?nY
ZJT4ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:
ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004
ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT5ooooo`Oo00003oooool7ojVYZ@Cooooo1onYZJT5
ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool7o`0000Gooooo1ol00004
ooooo`So00001?ooool7o`0000oooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Gooooo1ol00004
ooooo`SoZJVY1?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;
ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00cooooo000>ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
4oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0<ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool03?ooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8
o`0000Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7
o`0000Gooooo1onYZJT?ooooo`Oo00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7
o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7
o`0000Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVY
Z@Cooooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`OoZJVY3oooool7o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00oooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Co
oooo1onYZJT4ooooo`So00001?ooool7o`0000Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7ojVYZ@oooooo1ol00004ooooo`OoZJVY1Oooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol0000?ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo
3?l00007ojVYZAco00001onYZJTco`0000OoZJVY3oooool;o`0000OoZJVYGOl0000?ooooo`_o0000
2?nYZJUMo`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001Ooo
ool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`OoZJVY3ooo
ool7o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?oo
ool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`SoZJVY1?oo
ool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Ooo
ool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`So00001?ooool7
o`0000Gooooo1onYZJT4ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7
ojVYZ@oooooo1ol00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8
ojVYZ@Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7
o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`00
00Cooooo1ol00005ooooo`OoZJVY1?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1onYZJT?ooooo`Oo00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`00
00Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Co
oooo2?l00004ooooo`Oo00001Oooool7ojVYZ@Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`OoZJVY3oooool7o`0000Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oo
oooo1ol00004ooooo`SoZJVY1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`003_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003ooooo
ooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000kooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<
ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool03?ooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT5ooooo`Oo
00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol0000?ooooo`Oo
00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`Oo
00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo2?l00004ooooo`Oo
00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo
00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Cooooo2?l00004ooooo`OoZJVY
1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo0000
3oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo0000
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`So0000
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo0000
1Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool<o`0000OoZJVY4?l00007ojVYZDKo00003ooo
oom_o`0000ooooooL?l0000;ooooo`002_ooool8o`0000Cooooo1onYZJT4ooooo`So00001?ooool7
ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7
o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001Oooool7
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7
o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`OoZJVY1?ooool8o`00
00Cooooo1onYZJT5ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`00
00Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`00
00Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`00
00Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7ojVYZ@Co
oooo2?l00004ooooo`OoZJVY1Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oo
oooo1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Co
oooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo
1onYZJT4ooooo`So00001?ooool7ojVYZ@Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol0000?ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo
1ol00004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool000kooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Sooooo00?o
0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000>
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<
ooooo`003_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool03?ooool000kooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03
o`000?oooooooooo00cooooo000:ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`OoZJVY
1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1onYZJT5ooooo`Oo0000
3oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol00005ooooo`Oo0000
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol00004ooooo`So0000
1?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l00004ooooo`Oo0000
1Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004ooooo`So00001?oo
ool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`OoZJVY1Ooo
ool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00001Ooo
ool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?oo
ool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?oo
ool7o`0000Gooooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7
ojVYZ@Gooooo1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7
o`0000Gooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7
o`0000Cooooo2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8
o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooob?o00001onYZJT5o`00
00OoZJVY1?l00008ojVYZ@Co00001onYZJT@o`0000OoZJVY3?l0000?ooooob;o00002?nYZJU5o`00
00ooooooL?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7ojVYZ@Go
oooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004ooooo`OoZJVY1Oooool7o`0000oo
oooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4ooooo`Oo00001Oooool7o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?ooooo`Oo00001?ooool8o`0000Co
oooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004ooooo`So00001?ooool7o`0000Go
oooo1ol00004ooooo`So00002oooool000[ooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1onYZJT5ooooo`OoZJVY1?ooool8ojVYZ@Cooooo1onYZJT4ooooo`So00001?ooool7ojVYZ@Gooooo
1ol0000?ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`SoZJVY1?ooool7o`0000Gooooo
1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00003oooool7o`0000Cooooo
2?l00004ooooo`Oo00001Oooool7o`0000Cooooo2?l00004ooooo`Oo00001?ooool8o`0000Cooooo
1ol00005ooooo`Oo00001?ooool8o`0000_ooooo000:ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`OoZJVY1Oooool7ojVYZ@Cooooo2?nYZJT4ooooo`OoZJVY1?ooool8o`0000Cooooo1onY
ZJT5ooooo`Oo00003oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8ojVYZ@Cooooo1ol0
0005ooooo`Oo00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000oooooo1ol0
0004ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00001?ooool7o`0000Cooooo2?l0
0004ooooo`Oo00001Oooool7o`0000Cooooo2?l0000;ooooo`002_ooool8o`0000Cooooo1ol00004
ooooo`So00001?ooool7ojVYZ@Gooooo1onYZJT4ooooo`SoZJVY1?ooool7ojVYZ@Cooooo2?l00004
ooooo`OoZJVY1Oooool7o`0000oooooo1ol00004ooooo`Oo00001Oooool7o`0000Cooooo2?nYZJT4
ooooo`Oo00001Oooool7o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol0000?
ooooo`Oo00001?ooool8o`0000Cooooo1ol00005ooooo`Oo00001?ooool8o`0000Cooooo1ol00004
ooooo`So00001?ooool7o`0000Gooooo1ol00004ooooo`So00002oooool00?ooooooPoooool00?oo
ooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00?oo
ooooPoooool00?ooooooPoooool00?ooooooPoooool00?ooooooPoooool00001\
\>"],
  ImageRangeCache->{{{0, 385.812}, {385.812, 0}} -> {-0.0750158, -0.0750158, \
0.00816468, 0.00816468}, {{9.1875, 124}, {376.625, 261.75}} -> {-0.13803, \
-0.13833, 0.00958093, 0.00958093}, {{135.438, 250.312}, {376.625, 261.75}} -> \
{-1.34692, -0.137982, 0.00957572, 0.00957572}, {{261.75, 376.625}, {376.625, \
261.75}} -> {-2.55645, -0.137982, 0.00957572, 0.00957572}, {{9.1875, 124}, \
{250.312, 135.438}} -> {-0.13803, -1.34852, 0.00958093, 0.00958093}, \
{{135.438, 250.312}, {250.312, 135.438}} -> {-1.34692, -1.34752, 0.00957572, \
0.00957572}, {{261.75, 376.625}, {250.312, 135.438}} -> {-2.55645, -1.34752, \
0.00957572, 0.00957572}, {{9.1875, 124}, {124, 9.1875}} -> {-0.13803, \
-2.55841, 0.00958093, 0.00958093}, {{135.438, 250.312}, {124, 9.1875}} -> \
{-1.34792, -2.55841, 0.00958093, 0.00958093}, {{261.75, 376.625}, {124, \
9.1875}} -> {-2.55811, -2.55841, 0.00958093, 0.00958093}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[vnStagModel]\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moore 8", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(m8StagHunt = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] Moore8[40, 40, 
              Wrap \[Rule] True], \[IndentingNewLine]PayoffMatrix \[Rule] 
            stagHunt, \[IndentingNewLine]StrategyColors \[Rule] 
            stagColors, \[IndentingNewLine]ShowWindow \[Rule] 
            False\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[41]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]RasterLatticeRepresentation[m8StagHunt, 40], 
        AspectRatio \[Rule] Automatic\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[43]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko0000
1onYZJTYo`0002;oZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo0000
3OnYZJT>o`0001GoZJVY1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onY
ZJTYo`0002;oZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnY
ZJT>o`0001GoZJVY1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onYZJTY
o`0002;oZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnYZJT>
o`0001GoZJVY1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onYZJTYo`00
02;oZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnYZJT>o`00
01GoZJVY1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onYZJTYo`0002;o
ZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnYZJT>o`0001Go
ZJVY1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onYZJTYo`0002;oZJVY
1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnYZJT>o`0001GoZJVY
1ol00007ooooo`001_ooool7o`0000OoZJVY3_l0000=ojVYZ@ko00001onYZJTYo`0002;oZJVY1ol0
0007ojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00003OnYZJT>o`0001GoZJVY1ol0
0007ooooo`001_ooool>ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6
o`0002?oZJVY8_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7
ooooo`001_ooool>ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`00
02?oZJVY8_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7oooo
o`001_ooool>ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`0002?o
ZJVY8_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`00
1_ooool>ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`0002?oZJVY
8_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_oo
ool>ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`0002?oZJVY8_l0
0007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool>
ojVYZ@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`0002?oZJVY8_l00007
ojVYZ@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVY
Z@ko00003OnYZJT>o`0000OoZJVY3_l0000DojVYZ@Oo00007?nYZJT6o`0002?oZJVY8_l00007ojVY
Z@Oo00003OnYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool7o`0000ko
ZJVY1ol0000RojVYZ@Oo00003_nYZJT6o`0000OoZJVY8ol0000DojVYZAGo00005?nYZJT7o`0000ko
ZJVY3_l00006ojVYZAco00001onYZJT7o`0000Oooooo0006ooooo`Oo00003_nYZJT7o`0002;oZJVY
1ol0000>ojVYZ@Ko00001onYZJTSo`0001CoZJVY5Ol0000DojVYZ@Oo00003_nYZJT>o`0000KoZJVY
7?l00007ojVYZ@Oo00001oooool000Kooooo1ol0000>ojVYZ@Oo00008_nYZJT7o`0000koZJVY1_l0
0007ojVYZB?o00005?nYZJTEo`0001CoZJVY1ol0000>ojVYZ@ko00001_nYZJTLo`0000OoZJVY1ol0
0007ooooo`001_ooool7o`0000koZJVY1ol0000RojVYZ@Oo00003_nYZJT6o`0000OoZJVY8ol0000D
ojVYZAGo00005?nYZJT7o`0000koZJVY3_l00006ojVYZAco00001onYZJT7o`0000Oooooo0006oooo
o`Oo00003_nYZJT7o`0002;oZJVY1ol0000>ojVYZ@Ko00001onYZJTSo`0001CoZJVY5Ol0000DojVY
Z@Oo00003_nYZJT>o`0000KoZJVY7?l00007ojVYZ@Oo00001oooool000Kooooo1ol0000>ojVYZ@Oo
00008_nYZJT7o`0000koZJVY1_l00007ojVYZB?o00005?nYZJTEo`0001CoZJVY1ol0000>ojVYZ@ko
00001_nYZJTLo`0000OoZJVY1ol00007ooooo`001_ooool7o`0000koZJVY1ol0000RojVYZ@Oo0000
3_nYZJT6o`0000OoZJVY8ol0000DojVYZAGo00005?nYZJT7o`0000koZJVY3_l00006ojVYZAco0000
1onYZJT7o`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZ@ko00003OnYZJT7o`0000OoZJVY1ol0
0007ojVYZ@Oo00001_nYZJT7o`0000OoZJVY8_l00007ojVYZ@ko00005OnYZJT6o`0000koZJVY1ol0
0007ojVYZACo00003_nYZJT7o`0000OoZJVY3_l00007ooooo`001_ooool7ojVYZ@Oo00001onYZJT>
o`0000goZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000KoZJVY1ol00007ojVYZB;o00001onYZJT>
o`0001GoZJVY1_l0000>ojVYZ@Oo00001onYZJTDo`0000koZJVY1ol00007ojVYZ@ko00001oooool0
00Kooooo1onYZJT7o`0000OoZJVY3_l0000=ojVYZ@Oo00001onYZJT7o`0000OoZJVY1ol00006ojVY
Z@Oo00001onYZJTRo`0000OoZJVY3_l0000EojVYZ@Ko00003_nYZJT7o`0000OoZJVY5?l0000>ojVY
Z@Oo00001onYZJT>o`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZ@ko00003OnYZJT7o`0000Oo
ZJVY1ol00007ojVYZ@Oo00001_nYZJT7o`0000OoZJVY8_l00007ojVYZ@ko00005OnYZJT6o`0000ko
ZJVY1ol00007ojVYZACo00003_nYZJT7o`0000OoZJVY3_l00007ooooo`001_ooool7ojVYZ@Oo0000
1onYZJT>o`0000goZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000KoZJVY1ol00007ojVYZB;o0000
1onYZJT>o`0001GoZJVY1_l0000>ojVYZ@Oo00001onYZJTDo`0000koZJVY1ol00007ojVYZ@ko0000
1oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0000=ojVYZ@Oo00001onYZJT7o`0000OoZJVY1ol0
0006ojVYZ@Oo00001onYZJTRo`0000OoZJVY3_l0000EojVYZ@Ko00003_nYZJT7o`0000OoZJVY5?l0
000>ojVYZ@Oo00001onYZJT>o`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZ@ko00003OnYZJT7
o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJT7o`0000OoZJVY8_l00007ojVYZ@ko00005OnYZJT6
o`0000koZJVY1ol00007ojVYZACo00003_nYZJT7o`0000OoZJVY3_l00007ooooo`001_ooool7ojVY
Z@Oo00001onYZJT>o`0000goZJVY3_l0000RojVYZ@Oo00001onYZJT7o`0000koZJVY1_l0001<ojVY
Z@Oo00001onYZJT6o`0000koZJVY1ol0000>ojVYZ@Oo00001oooool000Kooooo1onYZJT7o`0000Oo
ZJVY3_l0000=ojVYZ@ko00008_nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Ko0000C?nYZJT7o`0000Oo
ZJVY1_l0000>ojVYZ@Oo00003_nYZJT7o`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZ@ko0000
3OnYZJT>o`0002;oZJVY1ol00007ojVYZ@Oo00003_nYZJT6o`0004coZJVY1ol00007ojVYZ@Ko0000
3_nYZJT7o`0000koZJVY1ol00007ooooo`001_ooool7ojVYZ@Oo00001onYZJT>o`0000goZJVY3_l0
000RojVYZ@Oo00001onYZJT7o`0000koZJVY1_l0001<ojVYZ@Oo00001onYZJT6o`0000koZJVY1ol0
000>ojVYZ@Oo00001oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0000=ojVYZ@ko00008_nYZJT7
o`0000OoZJVY1ol0000>ojVYZ@Ko0000C?nYZJT7o`0000OoZJVY1_l0000>ojVYZ@Oo00003_nYZJT7
o`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZ@ko00003OnYZJT>o`0002;oZJVY1ol00007ojVY
Z@Oo00003_nYZJT6o`0004coZJVY1ol00007ojVYZ@Ko00003_nYZJT7o`0000koZJVY1ol00007oooo
o`001_ooool7ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Ko00005OnYZJT7o`0000Oo
ZJVY1ol00006ojVYZ@Oo00003_nYZJT7o`0000OoZJVY3Ol00007ojVYZBWo00007?nYZJT>o`0000Oo
oooo0006ooooo`OoZJVY3_l0000>ojVYZ@go00005OnYZJT7o`0000OoZJVY1_l0000EojVYZ@Oo0000
1onYZJT7o`0000KoZJVY1ol0000>ojVYZ@Oo00001onYZJT=o`0000OoZJVY:Ol0000LojVYZ@ko0000
1oooool000Kooooo1onYZJT>o`0000koZJVY3Ol0000EojVYZ@Oo00001onYZJT6o`0001GoZJVY1ol0
0007ojVYZ@Oo00001_nYZJT7o`0000koZJVY1ol00007ojVYZ@go00001onYZJTYo`0001coZJVY3_l0
0007ooooo`001_ooool7ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Ko00005OnYZJT7
o`0000OoZJVY1ol00006ojVYZ@Oo00003_nYZJT7o`0000OoZJVY3Ol00007ojVYZBWo00007?nYZJT>
o`0000Oooooo0006ooooo`OoZJVY3_l0000>ojVYZ@go00005OnYZJT7o`0000OoZJVY1_l0000EojVY
Z@Oo00001onYZJT7o`0000KoZJVY1ol0000>ojVYZ@Oo00001onYZJT=o`0000OoZJVY:Ol0000LojVY
Z@ko00001oooool000Kooooo1onYZJT>o`0000koZJVY3Ol0000EojVYZ@Oo00001onYZJT6o`0001Go
ZJVY1ol00007ojVYZ@Oo00001_nYZJT7o`0000koZJVY1ol00007ojVYZ@go00001onYZJTYo`0001co
ZJVY3_l00007ooooo`001_ooool7ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Ko0000
5OnYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00003_nYZJT7o`0000OoZJVY3Ol00007ojVYZBWo0000
7?nYZJT>o`0000Oooooo0006ooooo`koZJVY6ol00007ojVYZ@Oo00003_nYZJTKo`0001GoZJVY1ol0
000=ojVYZAGo00003_nYZJT7o`0000KoZJVY1ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY1ol0
000EojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVYZA_o00001onYZJT7o`0000koZJVY6ol0000E
ojVYZ@Oo00003OnYZJTEo`0000koZJVY1ol00006ojVYZ@Oo00001onYZJT7o`0000OoZJVY3_l00006
ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1oooool000Kooooo3_nYZJTKo`0000OoZJVY1ol0000>ojVY
ZA_o00005OnYZJT7o`0000goZJVY5Ol0000>ojVYZ@Oo00001_nYZJT7o`0000OoZJVY1ol00007ojVY
Z@ko00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Oooooo0006ooooo`koZJVY6ol00007ojVYZ@Oo
00003_nYZJTKo`0001GoZJVY1ol0000=ojVYZAGo00003_nYZJT7o`0000KoZJVY1ol00007ojVYZ@Oo
00001onYZJT>o`0000KoZJVY1ol0000EojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVYZA_o0000
1onYZJT7o`0000koZJVY6ol0000EojVYZ@Oo00003OnYZJTEo`0000koZJVY1ol00006ojVYZ@Oo0000
1onYZJT7o`0000OoZJVY3_l00006ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1oooool000Kooooo3_nY
ZJTKo`0000OoZJVY1ol0000>ojVYZA_o00005OnYZJT7o`0000goZJVY5Ol0000>ojVYZ@Oo00001_nY
ZJT7o`0000OoZJVY1ol00007ojVYZ@ko00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Oooooo0006
ooooo`koZJVY6ol00007ojVYZ@Oo00003_nYZJTKo`0001GoZJVY1ol0000=ojVYZAGo00003_nYZJT7
o`0000KoZJVY1ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY1ol0000EojVYZ@Oo00001onYZJT7
ooooo`001_ooool>o`0000OoZJVY1ol0000YojVYZA_o00001onYZJTEo`0000goZJVY:_l00006ojVY
Z@Oo00001onYZJTEo`0000OoZJVY1_l00007ojVYZAGo00003_nYZJT7ooooo`001_ooool>o`0000Oo
ZJVY1ol0000YojVYZA_o00001onYZJTEo`0000goZJVY:_l00006ojVYZ@Oo00001onYZJTEo`0000Oo
ZJVY1_l00007ojVYZAGo00003_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol0000YojVYZA_o0000
1onYZJTEo`0000goZJVY:_l00006ojVYZ@Oo00001onYZJTEo`0000OoZJVY1_l00007ojVYZAGo0000
3_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol0000YojVYZA_o00001onYZJTEo`0000goZJVY:_l0
0006ojVYZ@Oo00001onYZJTEo`0000OoZJVY1_l00007ojVYZAGo00003_nYZJT7ooooo`001_ooool>
o`0000OoZJVY1ol0000YojVYZA_o00001onYZJTEo`0000goZJVY:_l00006ojVYZ@Oo00001onYZJTE
o`0000OoZJVY1_l00007ojVYZAGo00003_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol0000YojVY
ZA_o00001onYZJTEo`0000goZJVY:_l00006ojVYZ@Oo00001onYZJTEo`0000OoZJVY1_l00007ojVY
ZAGo00003_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol0000YojVYZA_o00001onYZJTEo`0000go
ZJVY:_l00006ojVYZ@Oo00001onYZJTEo`0000OoZJVY1_l00007ojVYZAGo00003_nYZJT7ooooo`00
1_oooolYojVYZCOo00003_nYZJT>o`0001CoZJVY3_l00007ojVYZACo00007?nYZJTYo`0000koZJVY
1ol00007ooooo`001_oooolYojVYZCOo00003_nYZJT>o`0001CoZJVY3_l00007ojVYZACo00007?nY
ZJTYo`0000koZJVY1ol00007ooooo`001_oooolYojVYZCOo00003_nYZJT>o`0001CoZJVY3_l00007
ojVYZACo00007?nYZJTYo`0000koZJVY1ol00007ooooo`001_oooolYojVYZCOo00003_nYZJT>o`00
01CoZJVY3_l00007ojVYZACo00007?nYZJTYo`0000koZJVY1ol00007ooooo`001_oooolYojVYZCOo
00003_nYZJT>o`0001CoZJVY3_l00007ojVYZACo00007?nYZJTYo`0000koZJVY1ol00007ooooo`00
1_oooolYojVYZCOo00003_nYZJT>o`0001CoZJVY3_l00007ojVYZACo00007?nYZJTYo`0000koZJVY
1ol00007ooooo`001_oooolYojVYZCOo00003_nYZJT>o`0001CoZJVY3_l00007ojVYZACo00007?nY
ZJTYo`0000koZJVY1ol00007ooooo`001_ooool>o`0000OoZJVY5?l0000EojVYZ@ko00003OnYZJT7
o`0000OoZJVY3_l0000>ojVYZACo00001onYZJT7o`0000koZJVY8_l00007ojVYZ@go00007?nYZJT7
o`0000OoZJVY1oooool000Kooooo3_l00007ojVYZACo00005OnYZJT>o`0000goZJVY1ol00007ojVY
Z@ko00003_nYZJTDo`0000OoZJVY1ol0000>ojVYZB;o00001onYZJT=o`0001coZJVY1ol00007ojVY
Z@Oooooo0006ooooo`ko00001onYZJTDo`0001GoZJVY3_l0000=ojVYZ@Oo00001onYZJT>o`0000ko
ZJVY5?l00007ojVYZ@Oo00003_nYZJTRo`0000OoZJVY3Ol0000LojVYZ@Oo00001onYZJT7ooooo`00
1_ooool>o`0000OoZJVY5?l0000EojVYZ@ko00003OnYZJT7o`0000OoZJVY3_l0000>ojVYZACo0000
1onYZJT7o`0000koZJVY8_l00007ojVYZ@go00007?nYZJT7o`0000OoZJVY1oooool000Kooooo3_l0
0007ojVYZACo00005OnYZJT>o`0000goZJVY1ol00007ojVYZ@ko00003_nYZJTDo`0000OoZJVY1ol0
000>ojVYZB;o00001onYZJT=o`0001coZJVY1ol00007ojVYZ@Oooooo0006ooooo`ko00001onYZJTD
o`0001GoZJVY3_l0000=ojVYZ@Oo00001onYZJT>o`0000koZJVY5?l00007ojVYZ@Oo00003_nYZJTR
o`0000OoZJVY3Ol0000LojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVYZ@Oo00001onYZJT7o`00
00KoZJVY1ol00007ojVYZ@Oo00001onYZJTDo`0001coZJVY1ol00007ojVYZ@Ko00008onYZJT7o`00
00KoZJVY5Ol0000KojVYZ@Oo00007?nYZJT7o`0000Oooooo0006ooooo`koZJVY1ol00007ojVYZ@Oo
00001_nYZJT7o`0000OoZJVY1ol00007ojVYZACo00007?nYZJT7o`0000OoZJVY1_l0000SojVYZ@Oo
00001_nYZJTEo`0001_oZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_nYZJT7o`0000OoZJVY
1ol00006ojVYZ@Oo00001onYZJT7o`0000OoZJVY5?l0000LojVYZ@Oo00001onYZJT6o`0002?oZJVY
1ol00006ojVYZAGo00006onYZJT7o`0001coZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo00001onY
ZJT7o`0000KoZJVY1ol00007ojVYZ@Oo00001onYZJTDo`0001coZJVY1ol00007ojVYZ@Ko00008onY
ZJT7o`0000KoZJVY5Ol0000KojVYZ@Oo00007?nYZJT7o`0000Oooooo0006ooooo`koZJVY1ol00007
ojVYZ@Oo00001_nYZJT7o`0000OoZJVY1ol00007ojVYZACo00007?nYZJT7o`0000OoZJVY1_l0000S
ojVYZ@Oo00001_nYZJTEo`0001_oZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_nYZJT7o`00
00OoZJVY1ol00006ojVYZ@Oo00001onYZJT7o`0000OoZJVY5?l0000LojVYZ@Oo00001onYZJT6o`00
02?oZJVY1ol00006ojVYZAGo00006onYZJT7o`0001coZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo
00001onYZJT7o`0000KoZJVY1ol00007ojVYZ@Oo00001onYZJTDo`0001coZJVY1ol00007ojVYZ@Ko
00008onYZJT7o`0000KoZJVY5Ol0000KojVYZ@Oo00007?nYZJT7o`0000Oooooo0006ooooo`ko0000
5OnYZJT=o`0000OoZJVY1ol00007ojVYZ@ko00005?nYZJTEo`00033oZJVY6ol00007ojVYZB;o0000
3_nYZJT7o`0000OoZJVY1ol00007ooooo`001_ooool>o`0001GoZJVY3Ol00007ojVYZ@Oo00001onY
ZJT>o`0001CoZJVY5Ol0000`ojVYZA_o00001onYZJTRo`0000koZJVY1ol00007ojVYZ@Oo00001ooo
ool000Kooooo3_l0000EojVYZ@go00001onYZJT7o`0000OoZJVY3_l0000DojVYZAGo0000<?nYZJTK
o`0000OoZJVY8_l0000>ojVYZ@Oo00001onYZJT7o`0000Oooooo0006ooooo`ko00005OnYZJT=o`00
00OoZJVY1ol00007ojVYZ@ko00005?nYZJTEo`00033oZJVY6ol00007ojVYZB;o00003_nYZJT7o`00
00OoZJVY1ol00007ooooo`001_ooool>o`0001GoZJVY3Ol00007ojVYZ@Oo00001onYZJT>o`0001Co
ZJVY5Ol0000`ojVYZA_o00001onYZJTRo`0000koZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo
3_l0000EojVYZ@go00001onYZJT7o`0000OoZJVY3_l0000DojVYZAGo0000<?nYZJTKo`0000OoZJVY
8_l0000>ojVYZ@Oo00001onYZJT7o`0000Oooooo0006ooooo`ko00005OnYZJT=o`0000OoZJVY1ol0
0007ojVYZ@ko00005?nYZJTEo`00033oZJVY6ol00007ojVYZB;o00003_nYZJT7o`0000OoZJVY1ol0
0007ooooo`001_oooolEo`0001CoZJVY3_l00007ojVYZ@Oo00001onYZJT7o`0001CoZJVY5Ol0000K
ojVYZ@ko00001onYZJT7o`0000goZJVY3_l0000>ojVYZ@Oo00005?nYZJTEo`0000OoZJVY1oooool0
00Kooooo5Ol0000DojVYZ@ko00001onYZJT7o`0000OoZJVY1ol0000DojVYZAGo00006onYZJT>o`00
00OoZJVY1ol0000=ojVYZ@ko00003_nYZJT7o`0001CoZJVY5Ol00007ojVYZ@Oooooo0006oooooaGo
00005?nYZJT>o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJTEo`0001_oZJVY3_l00007ojVYZ@Oo
00003OnYZJT>o`0000koZJVY1ol0000DojVYZAGo00001onYZJT7ooooo`001_oooolEo`0001CoZJVY
3_l00007ojVYZ@Oo00001onYZJT7o`0001CoZJVY5Ol0000KojVYZ@ko00001onYZJT7o`0000goZJVY
3_l0000>ojVYZ@Oo00005?nYZJTEo`0000OoZJVY1oooool000Kooooo5Ol0000DojVYZ@ko00001onY
ZJT7o`0000OoZJVY1ol0000DojVYZAGo00006onYZJT>o`0000OoZJVY1ol0000=ojVYZ@ko00003_nY
ZJT7o`0001CoZJVY5Ol00007ojVYZ@Oooooo0006oooooaGo00005?nYZJT>o`0000OoZJVY1ol00007
ojVYZ@Oo00005?nYZJTEo`0001_oZJVY3_l00007ojVYZ@Oo00003OnYZJT>o`0000koZJVY1ol0000D
ojVYZAGo00001onYZJT7ooooo`001_oooolEo`0001CoZJVY3_l00007ojVYZ@Oo00001onYZJT7o`00
01CoZJVY5Ol0000KojVYZ@ko00001onYZJT7o`0000goZJVY3_l0000>ojVYZ@Oo00005?nYZJTEo`00
00OoZJVY1oooool000Kooooo1onYZJT>o`0002WoZJVY3_l00007ojVYZ@Ko00001onYZJTYo`0000Oo
ZJVY3_l0000RojVYZ@Oo00001onYZJTKo`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo
1onYZJT>o`0002WoZJVY3_l00007ojVYZ@Ko00001onYZJTYo`0000OoZJVY3_l0000RojVYZ@Oo0000
1onYZJTKo`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo1onYZJT>o`0002WoZJVY3_l0
0007ojVYZ@Ko00001onYZJTYo`0000OoZJVY3_l0000RojVYZ@Oo00001onYZJTKo`0001GoZJVY1ol0
0007ojVYZ@Oo00001oooool000Kooooo1onYZJT>o`0002WoZJVY3_l00007ojVYZ@Ko00001onYZJTY
o`0000OoZJVY3_l0000RojVYZ@Oo00001onYZJTKo`0001GoZJVY1ol00007ojVYZ@Oo00001oooool0
00Kooooo1onYZJT>o`0002WoZJVY3_l00007ojVYZ@Ko00001onYZJTYo`0000OoZJVY3_l0000RojVY
Z@Oo00001onYZJTKo`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo1onYZJT>o`0002Wo
ZJVY3_l00007ojVYZ@Ko00001onYZJTYo`0000OoZJVY3_l0000RojVYZ@Oo00001onYZJTKo`0001Go
ZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo1onYZJT>o`0002WoZJVY3_l00007ojVYZ@Ko0000
1onYZJTYo`0000OoZJVY3_l0000RojVYZ@Oo00001onYZJTKo`0001GoZJVY1ol00007ojVYZ@Oo0000
1oooool000Kooooo3_l0000>ojVYZ@go00003_nYZJTEo`0000goZJVY3_l0000LojVYZ@Ko00001onY
ZJT>o`0000OoZJVY:Ol0000EojVYZ@Ko00007?nYZJT7o`0000OoZJVY1oooool000Kooooo3_l0000>
ojVYZ@go00003_nYZJTEo`0000goZJVY3_l0000LojVYZ@Ko00001onYZJT>o`0000OoZJVY:Ol0000E
ojVYZ@Ko00007?nYZJT7o`0000OoZJVY1oooool000Kooooo3_l0000>ojVYZ@go00003_nYZJTEo`00
00goZJVY3_l0000LojVYZ@Ko00001onYZJT>o`0000OoZJVY:Ol0000EojVYZ@Ko00007?nYZJT7o`00
00OoZJVY1oooool000Kooooo3_l0000>ojVYZ@go00003_nYZJTEo`0000goZJVY3_l0000LojVYZ@Ko
00001onYZJT>o`0000OoZJVY:Ol0000EojVYZ@Ko00007?nYZJT7o`0000OoZJVY1oooool000Kooooo
3_l0000>ojVYZ@go00003_nYZJTEo`0000goZJVY3_l0000LojVYZ@Ko00001onYZJT>o`0000OoZJVY
:Ol0000EojVYZ@Ko00007?nYZJT7o`0000OoZJVY1oooool000Kooooo3_l0000>ojVYZ@go00003_nY
ZJTEo`0000goZJVY3_l0000LojVYZ@Ko00001onYZJT>o`0000OoZJVY:Ol0000EojVYZ@Ko00007?nY
ZJT7o`0000OoZJVY1oooool000Kooooo5Ol00007ojVYZ@Oo00001_nYZJT7o`0002?oZJVY1_l00007
ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo00001onYZJT=o`0001coZJVY3Ol00007
ojVYZ@ko00001onYZJT7o`0000OoZJVY1oooool000Kooooo5Ol00007ojVYZ@Oo00001_nYZJT7o`00
02?oZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo00001onYZJT=o`00
01coZJVY3Ol00007ojVYZ@ko00001onYZJT7o`0000OoZJVY1oooool000Kooooo5Ol00007ojVYZ@Oo
00001_nYZJT7o`0002?oZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo
00001onYZJT=o`0001coZJVY3Ol00007ojVYZ@ko00001onYZJT7o`0000OoZJVY1oooool000Kooooo
5Ol00007ojVYZ@Oo00001_nYZJT7o`0002?oZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY
3Ol0000EojVYZ@Oo00001onYZJT=o`0001coZJVY3Ol00007ojVYZ@ko00001onYZJT7o`0000OoZJVY
1oooool000Kooooo5Ol00007ojVYZ@Oo00001_nYZJT7o`0002?oZJVY1_l00007ojVYZ@ko00001onY
ZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo00001onYZJT=o`0001coZJVY3Ol00007ojVYZ@ko00001onY
ZJT7o`0000OoZJVY1oooool000Kooooo5Ol00007ojVYZ@Oo00001_nYZJT7o`0002?oZJVY1_l00007
ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo00001onYZJT=o`0001coZJVY3Ol00007
ojVYZ@ko00001onYZJT7o`0000OoZJVY1oooool000Kooooo5Ol00007ojVYZ@Oo00001_nYZJT7o`00
02?oZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000EojVYZ@Oo00001onYZJT=o`00
01coZJVY3Ol00007ojVYZ@ko00001onYZJT7o`0000OoZJVY1oooool000Kooooo1ol00007ojVYZ@ko
00001onYZJT6o`0000OoZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo
00001_nYZJTSo`0002WoZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko0000
1onYZJT6o`0000OoZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo0000
1_nYZJTSo`0002WoZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko00001onY
ZJT6o`0000OoZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001_nY
ZJTSo`0002WoZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko00001onYZJT6
o`0000OoZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJTS
o`0002WoZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko00001onYZJT6o`00
00OoZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJTSo`00
02WoZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko00001onYZJT6o`0000Oo
ZJVY5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJTSo`0002Wo
ZJVY3_l0000RojVYZ@ko00001oooool000Kooooo1ol00007ojVYZ@ko00001onYZJT6o`0000OoZJVY
5Ol0000>ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJTSo`0002WoZJVY
3_l0000RojVYZ@ko00001oooool000Kooooo1ol0000>ojVYZ@Oo00001onYZJT=o`0003OoZJVY1ol0
0007ojVYZACo00003_nYZJT7o`0000OoZJVY1ol0000=ojVYZ@Oo00003_nYZJT7o`0000koZJVY1_l0
0007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1oooool000Kooooo1ol0000>ojVYZ@Oo00001onYZJT=
o`0003OoZJVY1ol00007ojVYZACo00003_nYZJT7o`0000OoZJVY1ol0000=ojVYZ@Oo00003_nYZJT7
o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1oooool000Kooooo1ol0000>ojVY
Z@Oo00001onYZJT=o`0003OoZJVY1ol00007ojVYZACo00003_nYZJT7o`0000OoZJVY1ol0000=ojVY
Z@Oo00003_nYZJT7o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1oooool000Ko
oooo1ol0000>ojVYZ@Oo00001onYZJT=o`0003OoZJVY1ol00007ojVYZACo00003_nYZJT7o`0000Oo
ZJVY1ol0000=ojVYZ@Oo00003_nYZJT7o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000Oo
ZJVY1oooool000Kooooo1ol0000>ojVYZ@Oo00001onYZJT=o`0003OoZJVY1ol00007ojVYZACo0000
3_nYZJT7o`0000OoZJVY1ol0000=ojVYZ@Oo00003_nYZJT7o`0000koZJVY1_l00007ojVYZ@Oo0000
1onYZJT>o`0000OoZJVY1oooool000Kooooo1ol0000>ojVYZ@Oo00001onYZJT=o`0003OoZJVY1ol0
0007ojVYZACo00003_nYZJT7o`0000OoZJVY1ol0000=ojVYZ@Oo00003_nYZJT7o`0000koZJVY1_l0
0007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1oooool000Kooooo1ol0000>ojVYZ@Oo00001onYZJT=
o`0003OoZJVY1ol00007ojVYZACo00003_nYZJT7o`0000OoZJVY1ol0000=ojVYZ@Oo00003_nYZJT7
o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1oooool000Kooooo1onYZJT7o`00
00OoZJVY3_l0000=ojVYZ@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`00
00KoZJVY1ol0000EojVYZ@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000Oo
ZJVY3_l0000=ojVYZ@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000Ko
ZJVY1ol0000EojVYZ@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000OoZJVY
3_l0000=ojVYZ@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000KoZJVY
1ol0000EojVYZ@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0
000=ojVYZ@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000KoZJVY1ol0
000EojVYZ@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0000=
ojVYZ@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000KoZJVY1ol0000E
ojVYZ@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0000=ojVY
Z@Oo00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000KoZJVY1ol0000EojVY
Z@ko00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0000OoZJVY3_l0000=ojVYZ@Oo
00003_nYZJT7o`0002;oZJVY6ol00007ojVYZ@Oo00003_nYZJT>o`0000KoZJVY1ol0000EojVYZ@ko
00003OnYZJTLo`0000OoZJVY1oooool000Kooooo:Ol00007ojVYZ@ko00001onYZJTYo`0000OoZJVY
1ol0000=ojVYZ@ko00001onYZJT7o`0001CoZJVY8ol00007ojVYZA_o00001onYZJT7o`0000OoZJVY
1oooool000Kooooo:Ol00007ojVYZ@ko00001onYZJTYo`0000OoZJVY1ol0000=ojVYZ@ko00001onY
ZJT7o`0001CoZJVY8ol00007ojVYZA_o00001onYZJT7o`0000OoZJVY1oooool000Kooooo:Ol00007
ojVYZ@ko00001onYZJTYo`0000OoZJVY1ol0000=ojVYZ@ko00001onYZJT7o`0001CoZJVY8ol00007
ojVYZA_o00001onYZJT7o`0000OoZJVY1oooool000Kooooo:Ol00007ojVYZ@ko00001onYZJTYo`00
00OoZJVY1ol0000=ojVYZ@ko00001onYZJT7o`0001CoZJVY8ol00007ojVYZA_o00001onYZJT7o`00
00OoZJVY1oooool000Kooooo:Ol00007ojVYZ@ko00001onYZJTYo`0000OoZJVY1ol0000=ojVYZ@ko
00001onYZJT7o`0001CoZJVY8ol00007ojVYZA_o00001onYZJT7o`0000OoZJVY1oooool000Kooooo
:Ol00007ojVYZ@ko00001onYZJTYo`0000OoZJVY1ol0000=ojVYZ@ko00001onYZJT7o`0001CoZJVY
8ol00007ojVYZA_o00001onYZJT7o`0000OoZJVY1oooool000Kooooo:Ol00007ojVYZ@ko00001onY
ZJTYo`0000OoZJVY1ol0000=ojVYZ@ko00001onYZJT7o`0001CoZJVY8ol00007ojVYZA_o00001onY
ZJT7o`0000OoZJVY1oooool000Kooooo1ol0000EojVYZ@Oo00001_nYZJT>o`0000koZJVY1ol0000D
ojVYZ@Oo00001onYZJT7o`0001CoZJVY1ol00007ojVYZ@ko00006onYZJT7o`0000koZJVY1ol0000=
ojVYZ@ko00003_nYZJT7o`0000OoZJVY1oooool000Kooooo1ol0000EojVYZ@Oo00001_nYZJT>o`00
00koZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0001CoZJVY1ol00007ojVYZ@ko00006onYZJT7o`00
00koZJVY1ol0000=ojVYZ@ko00003_nYZJT7o`0000OoZJVY1oooool000Kooooo1ol0000EojVYZ@Oo
00001_nYZJT>o`0000koZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0001CoZJVY1ol00007ojVYZ@ko
00006onYZJT7o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT7o`0000OoZJVY1oooool000Kooooo
1ol0000EojVYZ@Oo00001_nYZJT>o`0000koZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0001CoZJVY
1ol00007ojVYZ@ko00006onYZJT7o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT7o`0000OoZJVY
1oooool000Kooooo1ol0000EojVYZ@Oo00001_nYZJT>o`0000koZJVY1ol0000DojVYZ@Oo00001onY
ZJT7o`0001CoZJVY1ol00007ojVYZ@ko00006onYZJT7o`0000koZJVY1ol0000=ojVYZ@ko00003_nY
ZJT7o`0000OoZJVY1oooool000Kooooo1ol0000EojVYZ@Oo00001_nYZJT>o`0000koZJVY1ol0000D
ojVYZ@Oo00001onYZJT7o`0001CoZJVY1ol00007ojVYZ@ko00006onYZJT7o`0000koZJVY1ol0000=
ojVYZ@ko00003_nYZJT7o`0000OoZJVY1oooool000Kooooo1onYZJT>o`0000koZJVY1_l00007ojVY
Z@Oo00001onYZJT7o`0000koZJVY3Ol0000EojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol0000DojVY
Z@Oo00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZAco00001oooool000Kooooo1onYZJT>o`0000ko
ZJVY1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY3Ol0000EojVYZ@Oo00006onYZJT7o`0000Oo
ZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZAco00001oooool000Kooooo
1onYZJT>o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY3Ol0000EojVYZ@Oo0000
6onYZJT7o`0000OoZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZAco0000
1oooool000Kooooo1onYZJT>o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY3Ol0
000EojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol0
000>ojVYZAco00001oooool000Kooooo1onYZJT>o`0000koZJVY1_l00007ojVYZ@Oo00001onYZJT7
o`0000koZJVY3Ol0000EojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol0000DojVYZ@Oo00001onYZJT7
o`0000OoZJVY3Ol0000>ojVYZAco00001oooool000Kooooo1onYZJT>o`0000koZJVY1_l00007ojVY
Z@Oo00001onYZJT7o`0000koZJVY3Ol0000EojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol0000DojVY
Z@Oo00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZAco00001oooool000Kooooo1onYZJT>o`0000ko
ZJVY1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY3Ol0000EojVYZ@Oo00006onYZJT7o`0000Oo
ZJVY1ol0000DojVYZ@Oo00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZAco00001oooool000Kooooo
5OnYZJT>o`0001CoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Ko0000
1onYZJT>o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Oo0000
1oooool000Kooooo5OnYZJT>o`0001CoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0
000>ojVYZ@Ko00001onYZJT>o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol0
0007ojVYZ@Oo00001oooool000Kooooo5OnYZJT>o`0001CoZJVY1ol00007ojVYZ@Oo00005?nYZJT7
o`0000OoZJVY1ol0000>ojVYZ@Ko00001onYZJT>o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT=
o`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo5OnYZJT>o`0001CoZJVY1ol00007ojVY
Z@Oo00005?nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Ko00001onYZJT>o`0000koZJVY1ol0000=ojVY
Z@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo5OnYZJT>o`0001Co
ZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Ko00001onYZJT>o`0000ko
ZJVY1ol0000=ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo
5OnYZJT>o`0001CoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Ko0000
1onYZJT>o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol00007ojVYZ@Oo0000
1oooool000Kooooo5OnYZJT>o`0001CoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0
000>ojVYZ@Ko00001onYZJT>o`0000koZJVY1ol0000=ojVYZ@ko00003_nYZJT=o`0001GoZJVY1ol0
0007ojVYZ@Oo00001oooool000Kooooo3_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY3_l0000D
ojVYZ@ko00001onYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00001onYZJT>o`0000OoZJVY3Ol00007
ojVYZAGo00001onYZJT7o`0000KoZJVY5Ol0000EojVYZ@Oooooo0006ooooo`ko00001onYZJT7o`00
00goZJVY1ol00007ojVYZ@ko00005?nYZJT>o`0000OoZJVY1ol00007ojVYZ@Oo00001_nYZJT7o`00
00OoZJVY3_l00007ojVYZ@go00001onYZJTEo`0000OoZJVY1ol00006ojVYZAGo00005OnYZJT7oooo
o`001_ooool>o`0000OoZJVY1ol0000=ojVYZ@Oo00001onYZJT>o`0001CoZJVY3_l00007ojVYZ@Oo
00001onYZJT7o`0000KoZJVY1ol00007ojVYZ@ko00001onYZJT=o`0000OoZJVY5Ol00007ojVYZ@Oo
00001_nYZJTEo`0001GoZJVY1oooool000Kooooo3_l00007ojVYZ@Oo00003OnYZJT7o`0000OoZJVY
3_l0000DojVYZ@ko00001onYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00001onYZJT>o`0000OoZJVY
3Ol00007ojVYZAGo00001onYZJT7o`0000KoZJVY5Ol0000EojVYZ@Oooooo0006ooooo`ko00001onY
ZJT7o`0000goZJVY1ol00007ojVYZ@ko00005?nYZJT>o`0000OoZJVY1ol00007ojVYZ@Oo00001_nY
ZJT7o`0000OoZJVY3_l00007ojVYZ@go00001onYZJTEo`0000OoZJVY1ol00006ojVYZAGo00005OnY
ZJT7ooooo`001_ooool>o`0000OoZJVY1ol0000=ojVYZ@Oo00001onYZJT>o`0001CoZJVY3_l00007
ojVYZ@Oo00001onYZJT7o`0000KoZJVY1ol00007ojVYZ@ko00001onYZJT=o`0000OoZJVY5Ol00007
ojVYZ@Oo00001_nYZJTEo`0001GoZJVY1oooool000Kooooo3_l00007ojVYZ@Oo00003OnYZJT7o`00
00OoZJVY3_l0000DojVYZ@ko00001onYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00001onYZJT>o`00
00OoZJVY3Ol00007ojVYZAGo00001onYZJT7o`0000KoZJVY5Ol0000EojVYZ@Oooooo0006ooooo`Oo
0000:OnYZJTEo`0002WoZJVY5Ol00006ojVYZ@ko00003_nYZJT>o`0000KoZJVY1ol0000>ojVYZAGo
00001_nYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool7o`0002WoZJVY
5Ol0000YojVYZAGo00001_nYZJT>o`0000koZJVY3_l00006ojVYZ@Oo00003_nYZJTEo`0000KoZJVY
1ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY1oooool000Kooooo1ol0000YojVYZAGo0000:OnY
ZJTEo`0000KoZJVY3_l0000>ojVYZ@ko00001_nYZJT7o`0000koZJVY5Ol00006ojVYZ@Oo00001onY
ZJT7o`0000OoZJVY1ol00007ojVYZ@Oooooo0006ooooo`Oo0000:OnYZJTEo`0002WoZJVY5Ol00006
ojVYZ@ko00003_nYZJT>o`0000KoZJVY1ol0000>ojVYZAGo00001_nYZJT7o`0000OoZJVY1ol00007
ojVYZ@Oo00001onYZJT7ooooo`001_ooool7o`0002WoZJVY5Ol0000YojVYZAGo00001_nYZJT>o`00
00koZJVY3_l00006ojVYZ@Oo00003_nYZJTEo`0000KoZJVY1ol00007ojVYZ@Oo00001onYZJT7o`00
00OoZJVY1oooool000Kooooo1ol0000YojVYZAGo0000:OnYZJTEo`0000KoZJVY3_l0000>ojVYZ@ko
00001_nYZJT7o`0000koZJVY5Ol00006ojVYZ@Oo00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo
oooo0006ooooo`Oo0000:OnYZJTEo`0002WoZJVY5Ol00006ojVYZ@ko00003_nYZJT>o`0000KoZJVY
1ol0000>ojVYZAGo00001_nYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_oo
ool>ojVYZ@Oo00005?nYZJT7o`0000OoZJVY5Ol0000=ojVYZAGo00005OnYZJTRo`0000koZJVY3Ol0
000>ojVYZ@Oo00006onYZJTEo`0000OoZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo00005?nYZJT7
o`0000OoZJVY5Ol0000=ojVYZAGo00005OnYZJTRo`0000koZJVY3Ol0000>ojVYZ@Oo00006onYZJTE
o`0000OoZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo00005?nYZJT7o`0000OoZJVY5Ol0000=ojVY
ZAGo00005OnYZJTRo`0000koZJVY3Ol0000>ojVYZ@Oo00006onYZJTEo`0000OoZJVY1ol00007oooo
o`001_ooool>ojVYZ@Oo00005?nYZJT7o`0000OoZJVY5Ol0000=ojVYZAGo00005OnYZJTRo`0000ko
ZJVY3Ol0000>ojVYZ@Oo00006onYZJTEo`0000OoZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo0000
5?nYZJT7o`0000OoZJVY5Ol0000=ojVYZAGo00005OnYZJTRo`0000koZJVY3Ol0000>ojVYZ@Oo0000
6onYZJTEo`0000OoZJVY1ol00007ooooo`001_ooool>ojVYZ@Oo00005?nYZJT7o`0000OoZJVY5Ol0
000=ojVYZAGo00005OnYZJTRo`0000koZJVY3Ol0000>ojVYZ@Oo00006onYZJTEo`0000OoZJVY1ol0
0007ooooo`001_ooool>o`0000OoZJVY1ol00007ojVYZ@go00001onYZJTEo`0000OoZJVY1_l0000E
ojVYZAGo00005?nYZJT7o`0001GoZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000OoZJVY1_l00007
ojVYZ@Oo00001onYZJT7o`0000koZJVY1oooool000Kooooo3_l00007ojVYZ@Oo00001onYZJT=o`00
00OoZJVY5Ol00007ojVYZ@Ko00005OnYZJTEo`0001CoZJVY1ol0000EojVYZ@Ko00001onYZJT7o`00
00OoZJVY3_l00007ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol0000>ojVYZ@Oooooo0006ooooo`ko
00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00001onYZJT6o`0001GoZJVY5Ol0000DojVYZ@Oo
00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@ko00001onYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo
00003_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol00007ojVYZ@go00001onYZJTEo`0000OoZJVY
1_l0000EojVYZAGo00005?nYZJT7o`0001GoZJVY1_l00007ojVYZ@Oo00001onYZJT>o`0000OoZJVY
1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY1oooool000Kooooo3_l00007ojVYZ@Oo00001onY
ZJT=o`0000OoZJVY5Ol00007ojVYZ@Ko00005OnYZJTEo`0001CoZJVY1ol0000EojVYZ@Ko00001onY
ZJT7o`0000OoZJVY3_l00007ojVYZ@Ko00001onYZJT7o`0000OoZJVY1ol0000>ojVYZ@Oooooo0006
ooooo`ko00001onYZJT7o`0000OoZJVY3Ol00007ojVYZAGo00001onYZJT6o`0001GoZJVY5Ol0000D
ojVYZ@Oo00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@ko00001onYZJT6o`0000OoZJVY1ol00007
ojVYZ@Oo00003_nYZJT7ooooo`001_ooool>o`0000OoZJVY1ol00007ojVYZ@go00001onYZJTEo`00
00OoZJVY1_l0000EojVYZAGo00005?nYZJT7o`0001GoZJVY1_l00007ojVYZ@Oo00001onYZJT>o`00
00OoZJVY1_l00007ojVYZ@Oo00001onYZJT7o`0000koZJVY1oooool000Kooooo3_l0000EojVYZ@go
00001onYZJT7o`0001_oZJVY3_l00007ojVYZ@Oo00001onYZJTKo`0000koZJVY3_l00006ojVYZ@Oo
00001onYZJT7o`0000koZJVY1ol00006ojVYZ@Oo00007?nYZJT7o`0000Oooooo0006ooooo`ko0000
5OnYZJT=o`0000OoZJVY1ol0000KojVYZ@ko00001onYZJT7o`0000OoZJVY6ol0000>ojVYZ@ko0000
1_nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Oo00001_nYZJT7o`0001coZJVY1ol00007ooooo`001_oo
ool>o`0001GoZJVY3Ol00007ojVYZ@Oo00006onYZJT>o`0000OoZJVY1ol00007ojVYZA_o00003_nY
ZJT>o`0000KoZJVY1ol00007ojVYZ@Oo00003_nYZJT7o`0000KoZJVY1ol0000LojVYZ@Oo00001ooo
ool000Kooooo3_l0000EojVYZ@go00001onYZJT7o`0001_oZJVY3_l00007ojVYZ@Oo00001onYZJTK
o`0000koZJVY3_l00006ojVYZ@Oo00001onYZJT7o`0000koZJVY1ol00006ojVYZ@Oo00007?nYZJT7
o`0000Oooooo0006ooooo`ko00005OnYZJT=o`0000OoZJVY1ol0000KojVYZ@ko00001onYZJT7o`00
00OoZJVY6ol0000>ojVYZ@ko00001_nYZJT7o`0000OoZJVY1ol0000>ojVYZ@Oo00001_nYZJT7o`00
01coZJVY1ol00007ooooo`001_ooool>o`0001GoZJVY3Ol00007ojVYZ@Oo00006onYZJT>o`0000Oo
ZJVY1ol00007ojVYZA_o00003_nYZJT>o`0000KoZJVY1ol00007ojVYZ@Oo00003_nYZJT7o`0000Ko
ZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_l0000EojVYZ@go00001onYZJT7o`0001_oZJVY
3_l00007ojVYZ@Oo00001onYZJTKo`0000koZJVY3_l00006ojVYZ@Oo00001onYZJT7o`0000koZJVY
1ol00006ojVYZ@Oo00007?nYZJT7o`0000Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nY
ZJT6o`0001coZJVY1ol00007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nY
ZJT7o`0000OoZJVY1ol0000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6
o`0001coZJVY1ol00007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7
o`0000OoZJVY1ol0000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6o`00
01coZJVY1ol00007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`00
00OoZJVY1ol0000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6o`0001co
ZJVY1ol00007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000Oo
ZJVY1ol0000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6o`0001coZJVY
1ol00007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY
1ol0000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6o`0001coZJVY1ol0
0007ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0
000EojVYZ@Oooooo0006ooooo`OoZJVY8_l0000EojVYZ@Oo00003_nYZJT6o`0001coZJVY1ol00007
ojVYZA_o00005OnYZJT6o`0000OoZJVY1ol00007ojVYZ@Oo00005?nYZJT7o`0000OoZJVY1ol0000E
ojVYZ@Oooooo0006ooooo`OoZJVY1ol00007ojVYZA_o00001onYZJT>o`0000OoZJVY1ol0000DojVY
Z@ko00001onYZJT=o`0001GoZJVY8_l0000EojVYZ@Oo00003OnYZJTZo`0000Oooooo0006ooooo`Oo
ZJVY1ol00007ojVYZA_o00001onYZJT>o`0000OoZJVY1ol0000DojVYZ@ko00001onYZJT=o`0001Go
ZJVY8_l0000EojVYZ@Oo00003OnYZJTZo`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZA_o0000
1onYZJT>o`0000OoZJVY1ol0000DojVYZ@ko00001onYZJT=o`0001GoZJVY8_l0000EojVYZ@Oo0000
3OnYZJTZo`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZA_o00001onYZJT>o`0000OoZJVY1ol0
000DojVYZ@ko00001onYZJT=o`0001GoZJVY8_l0000EojVYZ@Oo00003OnYZJTZo`0000Oooooo0006
ooooo`OoZJVY1ol00007ojVYZA_o00001onYZJT>o`0000OoZJVY1ol0000DojVYZ@ko00001onYZJT=
o`0001GoZJVY8_l0000EojVYZ@Oo00003OnYZJTZo`0000Oooooo0006ooooo`OoZJVY1ol00007ojVY
ZA_o00001onYZJT>o`0000OoZJVY1ol0000DojVYZ@ko00001onYZJT=o`0001GoZJVY8_l0000EojVY
Z@Oo00003OnYZJTZo`0000Oooooo0006ooooo`OoZJVY1ol00007ojVYZA_o00001onYZJT>o`0000Oo
ZJVY1ol0000DojVYZ@ko00001onYZJT=o`0001GoZJVY8_l0000EojVYZ@Oo00003OnYZJTZo`0000Oo
oooo0006ooooo`Oo00005OnYZJT7o`0002;oZJVY3_l0000RojVYZ@ko00001_nYZJT7o`0000koZJVY
3_l00007ojVYZA_o0000:OnYZJT>o`0000OoZJVY1ol00007ooooo`001_ooool7o`0001GoZJVY1ol0
000RojVYZ@ko00008_nYZJT>o`0000KoZJVY1ol0000>ojVYZ@ko00001onYZJTKo`0002WoZJVY3_l0
0007ojVYZ@Oo00001oooool000Kooooo1ol0000EojVYZ@Oo00008_nYZJT>o`0002;oZJVY3_l00006
ojVYZ@Oo00003_nYZJT>o`0000OoZJVY6ol0000YojVYZ@ko00001onYZJT7o`0000Oooooo0006oooo
o`Oo00005OnYZJT7o`0002;oZJVY3_l0000RojVYZ@ko00001_nYZJT7o`0000koZJVY3_l00007ojVY
ZA_o0000:OnYZJT>o`0000OoZJVY1ol00007ooooo`001_ooool7o`0001GoZJVY1ol0000RojVYZ@ko
00008_nYZJT>o`0000KoZJVY1ol0000>ojVYZ@ko00001onYZJTKo`0002WoZJVY3_l00007ojVYZ@Oo
00001oooool000Kooooo1ol0000EojVYZ@Oo00008_nYZJT>o`0002;oZJVY3_l00006ojVYZ@Oo0000
3_nYZJT>o`0000OoZJVY6ol0000YojVYZ@ko00001onYZJT7o`0000Oooooo0006ooooo`koZJVY1ol0
0007ojVYZ@Oo00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJTEo`0000goZJVY5Ol0
000EojVYZ@go00001onYZJT>o`0000koZJVY1_l00007ojVYZ@Oo00003_nYZJT7o`0000OoZJVY1ooo
ool000Kooooo3_nYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1_l0000>
ojVYZAGo00003OnYZJTEo`0001GoZJVY3Ol00007ojVYZ@ko00003_nYZJT6o`0000OoZJVY1ol0000>
ojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVYZ@Oo00001onYZJT7o`0000KoZJVY1ol0000EojVY
Z@Oo00001onYZJT6o`0000koZJVY5Ol0000=ojVYZAGo00005OnYZJT=o`0000OoZJVY3_l0000>ojVY
Z@Ko00001onYZJT7o`0000koZJVY1ol00007ojVYZ@Oooooo0006ooooo`koZJVY1ol00007ojVYZ@Oo
00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJTEo`0000goZJVY5Ol0000EojVYZ@go
00001onYZJT>o`0000koZJVY1_l00007ojVYZ@Oo00003_nYZJT7o`0000OoZJVY1oooool000Kooooo
3_nYZJT7o`0000OoZJVY1ol00006ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1_l0000>ojVYZAGo0000
3OnYZJTEo`0001GoZJVY3Ol00007ojVYZ@ko00003_nYZJT6o`0000OoZJVY1ol0000>ojVYZ@Oo0000
1onYZJT7ooooo`001_ooool>ojVYZ@Oo00001onYZJT7o`0000KoZJVY1ol0000EojVYZ@Oo00001onY
ZJT6o`0000koZJVY5Ol0000=ojVYZAGo00005OnYZJT=o`0000OoZJVY3_l0000>ojVYZ@Ko00001onY
ZJT7o`0000koZJVY1ol00007ojVYZ@Oooooo0006ooooo`koZJVY1ol00007ojVYZ@Oo00001_nYZJT7
o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJTEo`0000goZJVY5Ol0000EojVYZ@go00001onYZJT>
o`0000koZJVY1_l00007ojVYZ@Oo00003_nYZJT7o`0000OoZJVY1oooool000Kooooo3_nYZJT>o`00
01_oZJVY1ol00007ojVYZ@Oo00005?nYZJT>o`0002;oZJVY1ol0000RojVYZ@ko00001onYZJTEo`00
00KoZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_nYZJT>o`0001_oZJVY1ol00007ojVYZ@Oo
00005?nYZJT>o`0002;oZJVY1ol0000RojVYZ@ko00001onYZJTEo`0000KoZJVY1ol0000LojVYZ@Oo
00001oooool000Kooooo3_nYZJT>o`0001_oZJVY1ol00007ojVYZ@Oo00005?nYZJT>o`0002;oZJVY
1ol0000RojVYZ@ko00001onYZJTEo`0000KoZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_nY
ZJT>o`0001_oZJVY1ol00007ojVYZ@Oo00005?nYZJT>o`0002;oZJVY1ol0000RojVYZ@ko00001onY
ZJTEo`0000KoZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo3_nYZJT>o`0001_oZJVY1ol00007
ojVYZ@Oo00005?nYZJT>o`0002;oZJVY1ol0000RojVYZ@ko00001onYZJTEo`0000KoZJVY1ol0000L
ojVYZ@Oo00001oooool000Kooooo3_nYZJT>o`0001_oZJVY1ol00007ojVYZ@Oo00005?nYZJT>o`00
02;oZJVY1ol0000RojVYZ@ko00001onYZJTEo`0000KoZJVY1ol0000LojVYZ@Oo00001oooool000Ko
oooo3_nYZJT>o`0001_oZJVY1ol00007ojVYZ@Oo00005?nYZJT>o`0002;oZJVY1ol0000RojVYZ@ko
00001onYZJTEo`0000KoZJVY1ol0000LojVYZ@Oo00001oooool000Kooooo5OnYZJT7o`0000OoZJVY
3Ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY3_l0000>ojVYZ@Oo00006onYZJT7o`0000OoZJVY
3_l00006ojVYZ@Oo00007?nYZJT7o`0000KoZJVY7?l00007ojVYZ@Oo00001oooool000Kooooo5OnY
ZJT7o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY3_l0000>ojVYZ@Oo00006onY
ZJT7o`0000OoZJVY3_l00006ojVYZ@Oo00007?nYZJT7o`0000KoZJVY7?l00007ojVYZ@Oo00001ooo
ool000Kooooo5OnYZJT7o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY3_l0000>
ojVYZ@Oo00006onYZJT7o`0000OoZJVY3_l00006ojVYZ@Oo00007?nYZJT7o`0000KoZJVY7?l00007
ojVYZ@Oo00001oooool000Kooooo5OnYZJT7o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT>o`00
00KoZJVY3_l0000>ojVYZ@Oo00006onYZJT7o`0000OoZJVY3_l00006ojVYZ@Oo00007?nYZJT7o`00
00KoZJVY7?l00007ojVYZ@Oo00001oooool000Kooooo5OnYZJT7o`0000OoZJVY3Ol00007ojVYZ@Oo
00001onYZJT>o`0000KoZJVY3_l0000>ojVYZ@Oo00006onYZJT7o`0000OoZJVY3_l00006ojVYZ@Oo
00007?nYZJT7o`0000KoZJVY7?l00007ojVYZ@Oo00001oooool000Kooooo5OnYZJT7o`0000OoZJVY
3Ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY3_l0000>ojVYZ@Oo00006onYZJT7o`0000OoZJVY
3_l00006ojVYZ@Oo00007?nYZJT7o`0000KoZJVY7?l00007ojVYZ@Oo00001oooool000Kooooo5OnY
ZJT7o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT>o`0000KoZJVY3_l0000>ojVYZ@Oo00006onY
ZJT7o`0000OoZJVY3_l00006ojVYZ@Oo00007?nYZJT7o`0000KoZJVY7?l00007ojVYZ@Oo00001ooo
ool000Kooooo3_l00007ojVYZ@ko00001_nYZJTEo`0000koZJVY6ol00007ojVYZ@Oo00001onYZJT7
o`0002;oZJVY:Ol00007ojVYZ@Oo00001onYZJTDo`0000koZJVY3_l00007ooooo`001_ooool>o`00
00OoZJVY3_l00006ojVYZAGo00003_nYZJTKo`0000OoZJVY1ol00007ojVYZ@Oo00008_nYZJTYo`00
00OoZJVY1ol00007ojVYZACo00003_nYZJT>o`0000Oooooo0006ooooo`ko00001onYZJT>o`0000Ko
ZJVY5Ol0000>ojVYZA_o00001onYZJT7o`0000OoZJVY1ol0000RojVYZBWo00001onYZJT7o`0000Oo
ZJVY5?l0000>ojVYZ@ko00001oooool000Kooooo3_l00007ojVYZ@ko00001_nYZJTEo`0000koZJVY
6ol00007ojVYZ@Oo00001onYZJT7o`0002;oZJVY:Ol00007ojVYZ@Oo00001onYZJTDo`0000koZJVY
3_l00007ooooo`001_ooool>o`0000OoZJVY3_l00006ojVYZAGo00003_nYZJTKo`0000OoZJVY1ol0
0007ojVYZ@Oo00008_nYZJTYo`0000OoZJVY1ol00007ojVYZACo00003_nYZJT>o`0000Oooooo0006
ooooo`ko00001onYZJT>o`0000KoZJVY5Ol0000>ojVYZA_o00001onYZJT7o`0000OoZJVY1ol0000R
ojVYZBWo00001onYZJT7o`0000OoZJVY5?l0000>ojVYZ@ko00001oooool000Kooooo3_l00007ojVY
Z@ko00001_nYZJTEo`0000koZJVY6ol00007ojVYZ@Oo00001onYZJT7o`0002;oZJVY:Ol00007ojVY
Z@Oo00001onYZJTDo`0000koZJVY3_l00007ooooo`001_ooool7o`0000koZJVY1ol0000DojVYZ@ko
00005OnYZJT=o`0000koZJVY3_l00007ojVYZ@Ko00001onYZJT>o`0000OoZJVY1ol00007ojVYZ@go
00003_nYZJT7o`0000koZJVY1_l0000>ojVYZ@Oo00001onYZJT7o`0000OoZJVY1oooool000Kooooo
1ol0000>ojVYZ@Oo00005?nYZJT>o`0001GoZJVY3Ol0000>ojVYZ@ko00001onYZJT6o`0000OoZJVY
3_l00007ojVYZ@Oo00001onYZJT=o`0000koZJVY1ol0000>ojVYZ@Ko00003_nYZJT7o`0000OoZJVY
1ol00007ojVYZ@Oooooo0006ooooo`Oo00003_nYZJT7o`0001CoZJVY3_l0000EojVYZ@go00003_nY
ZJT>o`0000OoZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZ@Oo00003_nY
ZJT6o`0000koZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool7o`0000koZJVY1ol0000D
ojVYZ@ko00005OnYZJT=o`0000koZJVY3_l00007ojVYZ@Ko00001onYZJT>o`0000OoZJVY1ol00007
ojVYZ@go00003_nYZJT7o`0000koZJVY1_l0000>ojVYZ@Oo00001onYZJT7o`0000OoZJVY1oooool0
00Kooooo1ol0000>ojVYZ@Oo00005?nYZJT>o`0001GoZJVY3Ol0000>ojVYZ@ko00001onYZJT6o`00
00OoZJVY3_l00007ojVYZ@Oo00001onYZJT=o`0000koZJVY1ol0000>ojVYZ@Ko00003_nYZJT7o`00
00OoZJVY1ol00007ojVYZ@Oooooo0006ooooo`Oo00003_nYZJT7o`0001CoZJVY3_l0000EojVYZ@go
00003_nYZJT>o`0000OoZJVY1_l00007ojVYZ@ko00001onYZJT7o`0000OoZJVY3Ol0000>ojVYZ@Oo
00003_nYZJT6o`0000koZJVY1ol00007ojVYZ@Oo00001onYZJT7ooooo`001_ooool>ojVYZ@Oo0000
6onYZJT7o`0000OoZJVY1ol00007ojVYZACo00001onYZJT7o`0001GoZJVY?Ol00007ojVYZ@ko0000
1onYZJT7o`0000KoZJVY3_l00007ojVYZAGo00001oooool000Kooooo3_nYZJT7o`0001_oZJVY1ol0
0007ojVYZ@Oo00001onYZJTDo`0000OoZJVY1ol0000EojVYZCgo00001onYZJT>o`0000OoZJVY1ol0
0006ojVYZ@ko00001onYZJTEo`0000Oooooo0006ooooo`koZJVY1ol0000KojVYZ@Oo00001onYZJT7
o`0000OoZJVY5?l00007ojVYZ@Oo00005OnYZJTmo`0000OoZJVY3_l00007ojVYZ@Oo00001_nYZJT>
o`0000OoZJVY5Ol00007ooooo`001_ooool>ojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol00007ojVY
ZACo00001onYZJT7o`0001GoZJVY?Ol00007ojVYZ@ko00001onYZJT7o`0000KoZJVY3_l00007ojVY
ZAGo00001oooool000Kooooo3_nYZJT7o`0001_oZJVY1ol00007ojVYZ@Oo00001onYZJTDo`0000Oo
ZJVY1ol0000EojVYZCgo00001onYZJT>o`0000OoZJVY1ol00006ojVYZ@ko00001onYZJTEo`0000Oo
oooo0006ooooo`koZJVY1ol0000KojVYZ@Oo00001onYZJT7o`0000OoZJVY5?l00007ojVYZ@Oo0000
5OnYZJTmo`0000OoZJVY3_l00007ojVYZ@Oo00001_nYZJT>o`0000OoZJVY5Ol00007ooooo`001_oo
ool>ojVYZ@Oo00006onYZJT7o`0000OoZJVY1ol00007ojVYZACo00001onYZJT7o`0001GoZJVY?Ol0
0007ojVYZ@ko00001onYZJT7o`0000KoZJVY3_l00007ojVYZAGo00001oooool000Kooooo1ol0000E
ojVYZACo00007?nYZJT=o`0001coZJVY5?l0000EojVYZ@Oo00008_nYZJT7o`0001GoZJVY3Ol0000>
ojVYZ@Oo00001onYZJT7o`0000Oooooo0006ooooo`Oo00005OnYZJTDo`0001coZJVY3Ol0000LojVY
ZACo00005OnYZJT7o`0002;oZJVY1ol0000EojVYZ@go00003_nYZJT7o`0000OoZJVY1ol00007oooo
o`001_ooool7o`0001GoZJVY5?l0000LojVYZ@go00007?nYZJTDo`0001GoZJVY1ol0000RojVYZ@Oo
00005OnYZJT=o`0000koZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo1ol0000EojVYZACo0000
7?nYZJT=o`0001coZJVY5?l0000EojVYZ@Oo00008_nYZJT7o`0001GoZJVY3Ol0000>ojVYZ@Oo0000
1onYZJT7o`0000Oooooo0006ooooo`Oo00005OnYZJTDo`0001coZJVY3Ol0000LojVYZACo00005OnY
ZJT7o`0002;oZJVY1ol0000EojVYZ@go00003_nYZJT7o`0000OoZJVY1ol00007ooooo`001_ooool7
o`0001GoZJVY5?l0000LojVYZ@go00007?nYZJTDo`0001GoZJVY1ol0000RojVYZ@Oo00005OnYZJT=
o`0000koZJVY1ol00007ojVYZ@Oo00001oooool000Kooooo1ol0000EojVYZACo00007?nYZJT=o`00
01coZJVY5?l0000EojVYZ@Oo00008_nYZJT7o`0001GoZJVY3Ol0000>ojVYZ@Oo00001onYZJT7o`00
00Oooooo0006ooooo`OoZJVY1ol0000EojVYZ@go00008onYZJT=o`0000OoZJVY5Ol00007ojVYZ@go
00001onYZJT>o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY1ol0000=ojVYZAco
00001onYZJT7ooooo`001_ooool7ojVYZ@Oo00005OnYZJT=o`0002?oZJVY3Ol00007ojVYZAGo0000
1onYZJT=o`0000OoZJVY3_l00007ojVYZ@go00001onYZJT7o`0000OoZJVY1ol00007ojVYZ@Oo0000
3OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0001GoZJVY3Ol0000SojVYZ@go00001onY
ZJTEo`0000OoZJVY3Ol00007ojVYZ@ko00001onYZJT=o`0000OoZJVY1ol00007ojVYZ@Oo00001onY
ZJT7o`0000goZJVY7?l00007ojVYZ@Oooooo0006ooooo`OoZJVY1ol0000EojVYZ@go00008onYZJT=
o`0000OoZJVY5Ol00007ojVYZ@go00001onYZJT>o`0000OoZJVY3Ol00007ojVYZ@Oo00001onYZJT7
o`0000OoZJVY1ol0000=ojVYZAco00001onYZJT7ooooo`001_ooool7ojVYZ@Oo00005OnYZJT=o`00
02?oZJVY3Ol00007ojVYZAGo00001onYZJT=o`0000OoZJVY3_l00007ojVYZ@go00001onYZJT7o`00
00OoZJVY1ol00007ojVYZ@Oo00003OnYZJTLo`0000OoZJVY1oooool000Kooooo1onYZJT7o`0001Go
ZJVY3Ol0000SojVYZ@go00001onYZJTEo`0000OoZJVY3Ol00007ojVYZ@ko00001onYZJT=o`0000Oo
ZJVY1ol00007ojVYZ@Oo00001onYZJT7o`0000goZJVY7?l00007ojVYZ@Oooooo0006ooooo`OoZJVY
1ol0000EojVYZ@go00008onYZJT=o`0000OoZJVY5Ol00007ojVYZ@go00001onYZJT>o`0000OoZJVY
3Ol00007ojVYZ@Oo00001onYZJT7o`0000OoZJVY1ol0000=ojVYZAco00001onYZJT7ooooo`001_oo
ool7ojVYZAGo00001onYZJT6o`0000OoZJVY1ol0000>ojVYZ@Oo00001onYZJT6o`0000OoZJVY3_l0
0007ojVYZ@ko00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJT7o`0000koZJVY1ol0
0006ojVYZ@ko00001onYZJT7o`0000koZJVY1oooool000Kooooo1onYZJTEo`0000OoZJVY1_l00007
ojVYZ@Oo00003_nYZJT7o`0000OoZJVY1_l00007ojVYZ@ko00001onYZJT>o`0000KoZJVY1ol0000E
ojVYZ@Oo00001onYZJT6o`0000koZJVY1ol0000>ojVYZ@Oo00001_nYZJT>o`0000OoZJVY1ol0000>
ojVYZ@Oooooo0006ooooo`OoZJVY5Ol00007ojVYZ@Ko00001onYZJT7o`0000koZJVY1ol00007ojVY
Z@Ko00001onYZJT>o`0000OoZJVY3_l00006ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1_l0000>ojVY
Z@Oo00003_nYZJT7o`0000KoZJVY3_l00007ojVYZ@Oo00003_nYZJT7ooooo`001_ooool7ojVYZAGo
00001onYZJT6o`0000OoZJVY1ol0000>ojVYZ@Oo00001onYZJT6o`0000OoZJVY3_l00007ojVYZ@ko
00001_nYZJT7o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJT7o`0000koZJVY1ol00006ojVYZ@ko
00001onYZJT7o`0000koZJVY1oooool000Kooooo1onYZJTEo`0000OoZJVY1_l00007ojVYZ@Oo0000
3_nYZJT7o`0000OoZJVY1_l00007ojVYZ@ko00001onYZJT>o`0000KoZJVY1ol0000EojVYZ@Oo0000
1onYZJT6o`0000koZJVY1ol0000>ojVYZ@Oo00001_nYZJT>o`0000OoZJVY1ol0000>ojVYZ@Oooooo
0006ooooo`OoZJVY5Ol00007ojVYZ@Ko00001onYZJT7o`0000koZJVY1ol00007ojVYZ@Ko00001onY
ZJT>o`0000OoZJVY3_l00006ojVYZ@Oo00005OnYZJT7o`0000OoZJVY1_l0000>ojVYZ@Oo00003_nY
ZJT7o`0000KoZJVY3_l00007ojVYZ@Oo00003_nYZJT7ooooo`001_ooool7ojVYZAGo00001onYZJT6
o`0000OoZJVY1ol0000>ojVYZ@Oo00001onYZJT6o`0000OoZJVY3_l00007ojVYZ@ko00001_nYZJT7
o`0001GoZJVY1ol00007ojVYZ@Ko00003_nYZJT7o`0000koZJVY1ol00006ojVYZ@ko00001onYZJT7
o`0000koZJVY1oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Ooo
ool00?oooooo8Oooool00?oooooo8Oooool00?oooooo8Oooool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-1.00021, -1.00021, 0.146343, \
0.146343}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[
          l, \[IndentingNewLine]RasterLatticeRepresentation[m8StagHunt, 40, 
            AspectRatio \[Rule] 
              Automatic]\[IndentingNewLine]]; \[IndentingNewLine]Evolve[
          m8StagHunt, 
          UpdateDisplay \[Rule] 
            False], {30}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[44]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[\[IndentingNewLine]GraphicsArray[
          Partition[l, 5]]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.2037 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.0286596 0.190476 [
[ 0 0 0 0 ]
[ 1 1.2037 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.2037 L
0 1.2037 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0286596 0.200176 0.205026 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.0286596 0.39418 0.205026 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.0286596 0.588183 0.205026 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.0286596 0.782187 0.205026 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.0286596 0.97619 0.205026 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.222663 0.200176 0.39903 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.222663 0.39418 0.39903 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.222663 0.588183 0.39903 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.222663 0.782187 0.39903 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.222663 0.97619 0.39903 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.416667 0.200176 0.593034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.416667 0.39418 0.593034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.416667 0.588183 0.593034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.416667 0.782187 0.593034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.416667 0.97619 0.593034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.61067 0.200176 0.787037 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.61067 0.39418 0.787037 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.61067 0.588183 0.787037 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.61067 0.782187 0.787037 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.61067 0.97619 0.787037 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.804674 0.200176 0.981041 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.804674 0.39418 0.981041 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.804674 0.588183 0.981041 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.804674 0.782187 0.981041 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.804674 0.97619 0.981041 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.998677 0.200176 1.17504 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000000000
000000000000A9A9A9000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000000000
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
A9A9A9000000000000A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000000000
000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000A9A9A9A9A9A9000000
000000000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9000000
A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9000000000000000000000000000000A9A9A9
A9A9A9000000A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
000000A9A9A9000000000000000000000000000000000000A9A9A9000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000
000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9000000A9A9A9
000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000
000000A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9
000000000000000000000000000000000000A9A9A9000000000000A9A9A9000000000000
000000000000000000000000A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9000000000000000000
000000A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
A9A9A9A9A9A9000000A9A9A9
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9000000000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9000000000000000000A9A9A9000000A9A9A9
000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9000000
000000000000A9A9A9000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9
000000000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000A9A9A9000000A9A9A9000000000000000000000000A9A9A9A9A9A9
000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000000000000000A9A9A9
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9A9A9A9
A9A9A9000000A9A9A9000000000000000000000000A9A9A9000000000000A9A9A9000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9000000
000000A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000A9A9A9
000000A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9000000000000000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9000000
A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000A9A9A9000000000000000000A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000000000
000000000000A9A9A9000000
000000000000A9A9A9000000000000A9A9A9000000000000000000A9A9A9A9A9A9000000
000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9000000A9A9A9000000000000000000
A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
000000000000A9A9A9A9A9A9000000000000A9A9A9000000A9A9A9000000000000A9A9A9
000000A9A9A9000000000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9
000000A9A9A9000000A9A9A9
A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9000000A9A9A9000000
000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000A9A9A9000000000000A9A9A9000000A9A9A9000000000000
A9A9A9000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9000000A9A9A9000000
A9A9A9000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000A9A9A9000000000000000000A9A9A9000000000000A9A9A9000000000000
A9A9A9000000000000A9A9A9000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9000000000000000000A9A9A9000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9
000000A9A9A9000000000000A9A9A9000000000000A9A9A9000000A9A9A9A9A9A9A9A9A9
000000A9A9A9000000A9A9A9A9A9A9000000A9A9A9A9A9A9000000A9A9A9000000000000
A9A9A9000000A9A9A9A9A9A9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.998677 0.39418 1.17504 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.998677 0.588183 1.17504 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.998677 0.782187 1.17504 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.998677 0.97619 1.17504 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 0.0238095 0.0238095 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .02381 translate
.95238 .95238 scale
120 string
40 40 8 [40 0 0 40 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9000000000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000
000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000
000000000000000000000000
000000000000000000000000000000000000000000000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
000000000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000000000000000000000000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000A9A9A9
A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000000000
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
A9A9A9A9A9A9000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9
000000000000000000A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9A9000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[46]:=",
  ImageSize->{452.938, 545.188},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007400028B000`400?l00000o`00003oooooooo5
ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5
ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5
ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5
ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT20000
00d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0
ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0
ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY
3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY
3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00
000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00
000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm
0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool0
00cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VY
Z@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@80
00003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@80
00003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo
?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo
?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2Y
ZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2Y
ZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2
000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<oooo
ocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<oooo
ocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0
ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0
ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY
0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY
0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooo
oolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Ooo
ool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=
0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VY
Z@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VY
Z@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@co
oooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@co
oooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@800000
3@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@800000
3@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2Y
ZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<
ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<
ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT20000
00d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT20000
00d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0
ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0
ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY
2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY
3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00
000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm
0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm
0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VY
Z@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VY
Z@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@80
00003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@80
00003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo
?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo
000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2Y
ZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2
000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2
000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@@000002`2YZJT<oooo
ocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<oooo
ocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT2000000d0
ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0
ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY
0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oo
oolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1000000;0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oo
oolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY2`000004
0:VYZ@cooooo?@2YZJT7000000P0ZJVY2ooooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY0P00000=
0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT;000000@0ZJVY3?oooolm0:VY
Z@L00000202YZJT;ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VY
Z@8000003@2YZJT=ooooo`003?oooolm0:VYZ@d000000P2YZJT<ooooocd0ZJVY2@0000060:VYZ@_o
oooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY0P00000=0:VYZ@go
oooo000<ooooocd0ZJVY3@0000020:VYZ@cooooo?@2YZJT9000000H0ZJVY2ooooolm0:VYZ@L00000
202YZJT<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2Y
ZJT9000000H0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0ZJVY1`0000080:VYZ@cooooo?@2Y
ZJT6000000T0ZJVY3?oooolm0:VYZ@@000002`2YZJT=ooooo`003?oooolm0:VYZ@T000001P2YZJT<
ooooocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@H000002@2YZJT<
ooooocd0ZJVY1000000;0:VYZ@gooooo000<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT70000
00P0ZJVY2ooooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT60000
00T0ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0
ZJVY1`0000080:VYZ@cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@H000002@2YZJT=ooooo`00
3?oooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT7000000P0ZJVY
3?oooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1P0000090:VYZ@gooooo000<ooooocd0ZJVY1`00
00080:VYZ@cooooo?@2YZJT7000000P0ZJVY2ooooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`00
00080:VYZ@cooooo?@2YZJT6000000T0ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm
0:VYZ@L00000202YZJT;ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT7000000P0ZJVY3?oooolo
0:VYZ@D00000202YZJT=ooooo`003?oooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`0000080:VY
Z@_ooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT<ooooocl0ZJVY1@0000080:VY
Z@gooooo000<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT7000000P0ZJVY2ooooolm0:VYZ@L0
0000202YZJT<ooooocd0ZJVY1`0000080:VYZ@cooooo?`2YZJT4000000T0ZJVY3Oooool000cooooo
>@2YZJT;000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0ZJVY1`0000080:VYZ@cooooo
?`2YZJT5000000P0ZJVY3?oooom10:VYZ@8000002@2YZJT=ooooo`003?ooooli0:VYZ@/00000202Y
ZJT<ooooocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT7000000P0ZJVY3?oooolo0:VYZ@D00000202Y
ZJT<oooood40ZJVY0P0000090:VYZ@gooooo000<ooooocD0ZJVY3`0000080:VYZ@cooooo>@2YZJT;
000000P0ZJVY2ooooolm0:VYZ@L00000202YZJT<oooood40ZJVY0P0000090:VYZ@coooooC02YZJT=
ooooo`003?oooole0:VYZ@l00000202YZJT<ooooocT0ZJVY2`0000080:VYZ@_ooooo?@2YZJT70000
00P0ZJVY3?oooom10:VYZ@8000002@2YZJT<oooood`0ZJVY3Oooool000cooooo<@2YZJT8000000X0
ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooocD0ZJVY1000000:0:VYZ@030000002YZJT0ZJVY00H0
ZJVY2ooooom30:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooom<0:VYZ@coooooC02YZJT=ooooo`00
3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo=@2YZJT4000000X0ZJVY
00<000000:VYZ@2YZJT01P2YZJT;oooood<0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooood`0ZJVY
3?oooom<0:VYZ@gooooo000<oooooc<0ZJVY1P00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oo
oole0:VYZ@@000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo=`2YZJT2000001<0ZJVY3?oo
oom<0:VYZ@coooooC02YZJT=ooooo`003?oooolc0:VYZ@H000002P2YZJT00`000000ZJVY0:VYZ@06
0:VYZ@cooooo=@2YZJT4000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT;ooooocL0ZJVY0P00000C
0:VYZ@coooooC02YZJT<oooood`0ZJVY3Oooool000cooooo=@2YZJT4000000X0ZJVY00<000000:VY
Z@2YZJT01P2YZJT<ooooocL0ZJVY0P00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2ooooom<0:VY
Z@coooooC02YZJT<oooood`0ZJVY3Oooool000cooooo=@2YZJT4000000X0ZJVY00<000000:VYZ@2Y
ZJT01P2YZJT<ooooocL0ZJVY0P00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2ooooom<0:VYZ@co
ooooC02YZJT<oooood`0ZJVY3Oooool000cooooo=@2YZJT4000000X0ZJVY00<000000:VYZ@2YZJT0
1P2YZJT<ooooocL0ZJVY0P00000C0:VYZ@_oooooC02YZJT<oooood`0ZJVY3?oooom<0:VYZ@gooooo
000<ooooocD0ZJVY1000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolg0:VYZ@8000004`2Y
ZJT;oooood`0ZJVY3?oooom<0:VYZ@coooooC02YZJT=ooooo`003?oooolg0:VYZ@8000002P2YZJT0
0`000000ZJVY0:VYZ@060:VYZ@coooooC02YZJT;oooood`0ZJVY3?oooom<0:VYZ@coooooC02YZJT=
ooooo`003?oooolg0:VYZ@8000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@coooooC02YZJT;oooo
od`0ZJVY3?oooom<0:VYZ@coooooC02YZJT=ooooo`003?oooolg0:VYZ@`000002@2YZJT<ooooocT0
ZJVY2000000;0:VYZ@_oooooC02YZJT<oooood`0ZJVY3?oooom<0:VYZ@gooooo000<ooooocL0ZJVY
300000090:VYZ@cooooo>@2YZJT8000000/0ZJVY2ooooom<0:VYZ@coooooC02YZJT<oooood`0ZJVY
3Oooool000cooooo>@2YZJT:000000T0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY0P00
000=0:VYZ@coooooC02YZJT<oooood`0ZJVY3Oooool000cooooo>@2YZJT:000000T0ZJVY3?oooolm
0:VYZ@@000002`2YZJT;ooooocd0ZJVY0P00000=0:VYZ@coooooC02YZJT<oooood`0ZJVY3Oooool0
00cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY1000000;0:VY
Z@coooooC02YZJT<oooood`0ZJVY3Oooool000cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@@0
00002`2YZJT;ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooom<0:VYZ@go
oooo000<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY2ooooolm0:VYZ@@00000
2`2YZJT<ooooocd0ZJVY0P00000=0:VYZ@coooooC02YZJT=ooooo`003?oooolm0:VYZ@@000002`2Y
ZJT<ooooocd0ZJVY1000000;0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2Y
ZJT<oooood`0ZJVY3Oooool000cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT;
ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooom<0:VYZ@gooooo000<oooo
ocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY2ooooolm0:VYZ@8000003@2YZJT<oooo
ocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT4000000/0
ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0
ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY
0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY
0P00000=0:VYZ@gooooo000<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooo
oolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Ooo
ool000cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=
0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VY
Z@@000002`2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VY
Z@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo003oooooolGooooo003oooooolGooooo
003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo
003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo
000<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT6000000T0ZJVY2ooooolm0:VYZ@@000002`2Y
ZJT<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY3Oooool000cooooo?@2YZJT7
000000P0ZJVY3?oooolm0:VYZ@H000002@2YZJT;ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4
000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT=ooooo`003?oooolm0:VYZ@L00000202YZJT<oooo
ocd0ZJVY1P0000090:VYZ@_ooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT<oooo
ocd0ZJVY1000000;0:VYZ@gooooo000<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT6000000T0
ZJVY2ooooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0
ZJVY3Oooool000cooooo?@2YZJT6000000T0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY
1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oo
oolm0:VYZ@H000002@2YZJT<ooooocd0ZJVY1000000;0:VYZ@_ooooo?@2YZJT4000000/0ZJVY3?oo
oolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY1000000;
0:VYZ@cooooo?@2YZJT4000000/0ZJVY2ooooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY1000000;
0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VY
Z@@000002`2YZJT;ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VY
Z@8000003@2YZJT=ooooo`003?oooolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY1000000;0:VYZ@_o
oooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@go
oooo000<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0ZJVY2ooooolm0:VYZ@@00000
2`2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2Y
ZJT4000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0ZJVY1000000;0:VYZ@cooooo?@2Y
ZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@@000002`2YZJT<
ooooocd0ZJVY1000000;0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<
ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT40000
00/0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT20000
00d0ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@@000002`2YZJT;ooooocd0
ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`00
3?oooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1000000;0:VYZ@_ooooo?@2YZJT2000000d0ZJVY
3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00
000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00
000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm
0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm
0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VY
Z@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VY
Z@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@80
00003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo
?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;ooooocd0ZJVY0P00000=0:VYZ@cooooo
?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?oooolm0:VYZ@8000003@2Y
ZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2Y
ZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2
000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2
000000d0ZJVY3Oooool000cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT;oooo
ocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=oooo
o`003?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@_ooooo?@2YZJT2000000d0
ZJVY3?oooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooocd0ZJVY
0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY
0P00000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo0P00000k0:VYZ@8000002`2Y
ZJT2000000cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00
000=0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo0P00000k0:VYZ@8000002`2YZJT2
000000cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=
0:VYZ@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo0P00000k0:VYZ@8000002`2YZJT20000
00cooooo?@2YZJT2000000d0ZJVY2ooooolm0:VYZ@8000003@2YZJT<ooooocd0ZJVY0P00000=0:VY
Z@cooooo?@2YZJT2000000d0ZJVY3Oooool000cooooo1000000i0:VYZ@8000002`2YZJT2000000co
oooo0P00000k0:VYZ@8000002`2YZJT2000000_ooooo?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@80
00003@2YZJT<ooooocd0ZJVY0P00000=0:VYZ@gooooo000<ooooo`@00000>@2YZJT2000000/0ZJVY
0P00000<ooooo`800000>`2YZJT2000000/0ZJVY0P00000;ooooocd0ZJVY0P00000=0:VYZ@cooooo
?@2YZJT2000000d0ZJVY3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?ooool4000003T0ZJVY3`00
000<ooooo`800000>`2YZJT?000000_ooooo?@2YZJT?000000cooooo?@2YZJT;000000@0ZJVY3?oo
oolm0:VYZ@L00000202YZJT=ooooo`003?ooool4000003T0ZJVY3`00000<ooooo`800000>`2YZJT?
000000_ooooo?@2YZJT?000000cooooo?@2YZJT;000000@0ZJVY3?oooolm0:VYZ@L00000202YZJT=
ooooo`003?ooool4000003T0ZJVY3`00000<ooooo`@00000>@2YZJT?000000_ooooo0P00000k0:VY
Z@l000003?oooolm0:VYZ@l000003?oooolm0:VYZ@/00000102YZJT=ooooo`003?ooool4000003T0
ZJVY3`00000<ooooo`@00000>@2YZJT?000000_ooooo0P00000k0:VYZ@l000003?oooolm0:VYZ@l0
00003?oooolm0:VYZ@/00000102YZJT=ooooo`003?ooool4000003T0ZJVY3`00000<ooooo`@00000
>@2YZJT?000000_ooooo0P00000k0:VYZ@l000003?oooolm0:VYZ@l000003?oooolm0:VYZ@d00000
0P2YZJT=ooooo`003?ooool4000003T0ZJVY3`00000<ooooo`@00000>@2YZJT?000000_ooooo0P00
000k0:VYZ@l000003?oooolm0:VYZ@l000003?oooolm0:VYZ@d000000P2YZJT=ooooo`003?ooool:
000003<0ZJVY3`00000<ooooo`H00000=`2YZJT?000000_ooooo1000000i0:VYZ@l000003?ooool2
000003/0ZJVY3`00000<ooooocd0ZJVY3`00000=ooooo`003?ooool:000003<0ZJVY3`00000<oooo
o`H00000=`2YZJT?000000_ooooo1000000i0:VYZ@l000003?ooool2000003/0ZJVY3`00000<oooo
ocd0ZJVY3`00000=ooooo`003?ooool=00000300ZJVY3`00000<ooooo`X00000<`2YZJT?000000_o
oooo1P00000g0:VYZ@l000003?ooool2000003/0ZJVY3`00000<ooooocd0ZJVY3@0000020:VYZ@go
oooo000<ooooo`d00000<02YZJT?000000cooooo2P00000c0:VYZ@l000002oooool6000003L0ZJVY
3`00000<ooooo`800000>`2YZJT?000000cooooo?@2YZJT=00000080ZJVY3Oooool000cooooo?@2Y
ZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0ZJVY1`0000080:VYZ@cooooo?@2Y
ZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT=ooooo`003?oooolm0:VYZ@L00000202YZJT<
ooooocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT<
ooooocd0ZJVY1`0000080:VYZ@gooooo000<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT70000
00P0ZJVY2ooooolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT70000
00P0ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0
ZJVY1`0000080:VYZ@cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT=ooooo`00
3?ooool^0:VYZAH00000202YZJT<oooooc40ZJVY4`0000080:VYZ@_ooooo=@2YZJT?000000P0ZJVY
3?ooooli0:VYZ@/00000202YZJT<ooooocd0ZJVY1`0000080:VYZ@gooooo000<ooooobh0ZJVY5P00
00080:VYZ@cooooo<@2YZJTC000000P0ZJVY2ooooole0:VYZ@l00000202YZJT<ooooocT0ZJVY2`00
00080:VYZ@cooooo?@2YZJT7000000P0ZJVY3Oooool000cooooo:P2YZJTJ000000P0ZJVY3?ooool^
0:VYZAH00000202YZJT;oooooc40ZJVY4`0000080:VYZ@cooooo=@2YZJT?000000P0ZJVY3?ooooli
0:VYZ@/00000202YZJT=ooooo`003?oooolV0:VYZAh00000202YZJT<ooooobX0ZJVY6P0000080:VY
Z@_ooooo;P2YZJTF000000P0ZJVY3?oooola0:VYZA<00000202YZJT<ooooocD0ZJVY3`0000080:VY
Z@gooooo000<ooooobH0ZJVY7P0000080:VYZ@cooooo:P2YZJTJ000000P0ZJVY2oooool^0:VYZAH0
0000202YZJT<oooooc40ZJVY4`0000080:VYZ@cooooo=@2YZJT?000000P0ZJVY3Oooool000cooooo
902YZJTP000000P0ZJVY3?oooolV0:VYZAh00000202YZJT;ooooobX0ZJVY6P0000080:VYZ@cooooo
;P2YZJTF000000P0ZJVY3?oooola0:VYZA<00000202YZJT=ooooo`003?oooolT0:VYZB000000202Y
ZJT<ooooobH0ZJVY7P0000080:VYZ@_ooooo:P2YZJTJ000000P0ZJVY3?ooool^0:VYZAH00000202Y
ZJT<oooooc40ZJVY4`0000080:VYZ@gooooo000<ooooob80ZJVY5`00000:0:VYZ@030000002YZJT0
ZJVY00H0ZJVY3?oooolT0:VYZAD000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo:02YZJTA
000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooob`0ZJVY3@00000:0:VYZ@030000002YZJT0
ZJVY00H0ZJVY3?ooool`0:VYZ@T000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<oooo
ob80ZJVY5`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolT0:VYZAD000002P2YZJT00`00
0000ZJVY0:VYZ@060:VYZ@_ooooo:02YZJTA000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooo
ob`0ZJVY3@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool`0:VYZ@T000002P2YZJT00`00
0000ZJVY0:VYZ@060:VYZ@gooooo000<ooooob@0ZJVY5@00000:0:VYZ@030000002YZJT0ZJVY00H0
ZJVY3?oooolX0:VYZA4000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo;02YZJT=000000X0
ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc00ZJVY2@00000:0:VYZ@030000002YZJT0ZJVY00H0
ZJVY3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<ooooob@0ZJVY
5@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolX0:VYZA4000002P2YZJT00`000000ZJVY
0:VYZ@060:VYZ@_ooooo;02YZJT=000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc00ZJVY
2@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY
0:VYZ@060:VYZ@gooooo000<ooooobP0ZJVY4@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oo
ool/0:VYZ@d000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo<02YZJT9000000X0ZJVY00<0
00000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oo
oolc0:VYZ@H000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<ooooobP0ZJVY4@00000:
0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool/0:VYZ@d000002P2YZJT00`000000ZJVY0:VYZ@06
0:VYZ@_ooooo<02YZJT9000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000000:
0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolc0:VYZ@H000002P2YZJT00`000000ZJVY0:VYZ@06
0:VYZ@gooooo000<oooooc40ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooola0:VY
Z@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo<@2YZJT8000000X0ZJVY00<000000:VY
Z@2YZJT01P2YZJT<oooooc40ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolc0:VY
Z@H000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<oooooc40ZJVY2000000:0:VYZ@03
0000002YZJT0ZJVY00H0ZJVY3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_o
oooo<@2YZJT8000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000000:0:VYZ@03
0000002YZJT0ZJVY00H0ZJVY3?oooolc0:VYZ@H000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@go
oooo000<oooooc40ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooola0:VYZ@P00000
2P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo<@2YZJT8000000X0ZJVY00<000000:VYZ@2YZJT0
1P2YZJT<oooooc<0ZJVY1P00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooole0:VYZ@@00000
2P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<oooooc40ZJVY2000000:0:VYZ@030000002Y
ZJT0ZJVY00H0ZJVY3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo<@2Y
ZJT8000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc<0ZJVY1P00000:0:VYZ@030000002Y
ZJT0ZJVY00H0ZJVY3?oooole0:VYZ@@000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<
oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC000000P0ZJVY2ooooola0:VYZA<00000202YZJT<
oooooc<0ZJVY4@0000080:VYZ@cooooo=@2YZJT?000000P0ZJVY3Oooool000cooooo<@2YZJTC0000
00P0ZJVY3?oooola0:VYZA<00000202YZJT;oooooc40ZJVY4`0000080:VYZ@cooooo<`2YZJTA0000
00P0ZJVY3?oooole0:VYZ@l00000202YZJT=ooooo`003?oooola0:VYZA<00000202YZJT<oooooc40
ZJVY4`0000080:VYZ@_ooooo<`2YZJTA000000P0ZJVY3?oooole0:VYZ@l00000202YZJT<ooooocL0
ZJVY3@0000080:VYZ@gooooo000<oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC000000P0ZJVY
2ooooolc0:VYZA400000202YZJT<ooooocD0ZJVY3`0000080:VYZ@cooooo=`2YZJT=000000P0ZJVY
3Oooool000cooooo<@2YZJTC000000P0ZJVY3?oooola0:VYZA<00000202YZJT;oooooc<0ZJVY4@00
00080:VYZ@cooooo=@2YZJT?000000P0ZJVY3?ooooli0:VYZ@X000002@2YZJT=ooooo`003?oooola
0:VYZA<00000202YZJT<oooooc<0ZJVY4@0000080:VYZ@_ooooo=@2YZJT?000000P0ZJVY3?ooooli
0:VYZ@/00000202YZJT<ooooocd0ZJVY1P0000090:VYZ@gooooo000<oooooc40ZJVY4`0000080:VY
Z@cooooo<`2YZJTA000000P0ZJVY2ooooole0:VYZ@l00000202YZJT<ooooocT0ZJVY2`0000080:VY
Z@cooooo?@2YZJT6000000T0ZJVY3Oooool000cooooo<@2YZJTC000000P0ZJVY3?oooole0:VYZ@l0
0000202YZJT;ooooocT0ZJVY2`0000080:VYZ@cooooo?@2YZJT6000000T0ZJVY3?oooolm0:VYZ@@0
00002`2YZJT=ooooo`003?oooola0:VYZA<00000202YZJT<ooooocD0ZJVY3`0000080:VYZ@_ooooo
>@2YZJT;000000P0ZJVY3?oooolm0:VYZ@H000002@2YZJT<ooooocd0ZJVY1000000;0:VYZ@gooooo
000<ooooocD0ZJVY3`0000080:VYZ@cooooo>@2YZJT;000000P0ZJVY2ooooolm0:VYZ@L00000202Y
ZJT<ooooocd0ZJVY1P0000090:VYZ@cooooo?@2YZJT4000000/0ZJVY3Oooool000cooooo=@2YZJT?
000000P0ZJVY3?ooooli0:VYZ@/00000202YZJT;ooooocd0ZJVY1`0000080:VYZ@cooooo?@2YZJT6
000000T0ZJVY3?oooolm0:VYZ@@000002`2YZJT=ooooo`003?ooooli0:VYZ@/00000202YZJT<oooo
ocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT6000000T0ZJVY3?oooolm0:VYZ@@000002`2YZJT<oooo
ocd0ZJVY1000000;0:VYZ@gooooo000<ooooocT0ZJVY2`0000080:VYZ@cooooo?@2YZJT7000000P0
ZJVY2ooooolm0:VYZ@H000002@2YZJT<ooooocd0ZJVY1000000;0:VYZ@cooooo?@2YZJT4000000/0
ZJVY3Oooool000cooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT;ooooocd0ZJVY
1P0000090:VYZ@cooooo?@2YZJT4000000/0ZJVY3?oooolm0:VYZ@@000002`2YZJT=ooooo`003?oo
oolm0:VYZ@L00000202YZJT<ooooocd0ZJVY1`0000080:VYZ@_ooooo?@2YZJT6000000T0ZJVY3?oo
oolm0:VYZ@@000002`2YZJT<ooooocd0ZJVY1000000;0:VYZ@gooooo003oooooolGooooo003ooooo
olGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003ooooo
olGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003ooooo
olGooooo000<ooooocT0ZJVY4@0000020:VYZ@cooooo>`2YZJT?00000080ZJVY2ooooolm0:VYZ@/0
0000102YZJT<ooooocd0ZJVY2@0000060:VYZ@cooooo?@2YZJT7000000P0ZJVY3Oooool000cooooo
>@2YZJTA00000080ZJVY3?oooolk0:VYZ@l000000P2YZJT;ooooocd0ZJVY2`0000040:VYZ@cooooo
?@2YZJT9000000H0ZJVY3?oooolm0:VYZ@L00000202YZJT=ooooo`003?ooooli0:VYZA4000000P2Y
ZJT<oooooc/0ZJVY3`0000020:VYZ@_ooooo?@2YZJT=00000080ZJVY3?oooolm0:VYZ@/00000102Y
ZJT<ooooocd0ZJVY2@0000060:VYZ@gooooo000<ooooocT0ZJVY4@0000020:VYZ@cooooo>`2YZJT?
00000080ZJVY2ooooolm0:VYZ@d000000P2YZJT<ooooocd0ZJVY2`0000040:VYZ@cooooo?@2YZJT9
000000H0ZJVY3Oooool000cooooo>`2YZJT?00000080ZJVY3?oooolm0:VYZ@d000000P2YZJT;oooo
ocd0ZJVY3@0000020:VYZ@cooooo?@2YZJT;000000@0ZJVY3?oooolm0:VYZ@T000001P2YZJT=oooo
o`003?oooolk0:VYZ@l000000P2YZJT<ooooocd0ZJVY3@0000020:VYZ@_ooooo?@2YZJT=00000080
ZJVY3?oooolm0:VYZ@/00000102YZJT<ooooocd0ZJVY2@0000060:VYZ@gooooo000<oooooc/0ZJVY
1P0000070:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT;ooooocd0ZJVY
100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY
1000000;0:VYZ@gooooo000<oooooc/0ZJVY1P0000070:VYZ@8000000P2YZJT<ooooocd0ZJVY1000
00070:VYZ@8000000P2YZJT;ooooocd0ZJVY100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY1000
00070:VYZ@8000000P2YZJT<ooooocd0ZJVY1000000;0:VYZ@gooooo000<ooooocd0ZJVY10000007
0:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT;ooooocd0ZJVY10000007
0:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY1000000;
0:VYZ@gooooo000<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VY
Z@8000000P2YZJT;ooooocd0ZJVY100000070:VYZ@8000000P2YZJT<ooooocd0ZJVY100000070:VY
Z@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT=ooooo`003?oooolm0:VYZ@@0
00001`2YZJT200000080ZJVY3?oooolm0:VYZ@@000001`2YZJT200000080ZJVY2ooooolm0:VYZ@@0
00001`2YZJT200000080ZJVY3?oooolm0:VYZ@@000001`2YZJT200000080ZJVY3?oooolm0:VYZ@@0
00001`2YZJT200000080ZJVY3Oooool000cooooo?@2YZJT4000000L0ZJVY0P0000020:VYZ@cooooo
?@2YZJT4000000L0ZJVY0P0000020:VYZ@_ooooo?@2YZJT4000000L0ZJVY0P0000020:VYZ@cooooo
?@2YZJT4000000L0ZJVY0P0000020:VYZ@cooooo?@2YZJT4000000/0ZJVY3Oooool000cooooo?@2Y
ZJT4000000L0ZJVY0P0000020:VYZ@cooooo?@2YZJT4000000L0ZJVY0P0000020:VYZ@_ooooo?@2Y
ZJT4000000L0ZJVY0P0000020:VYZ@cooooo?@2YZJT4000000L0ZJVY0P0000020:VYZ@cooooo?@2Y
ZJT4000000/0ZJVY3Oooool000cooooo?@2YZJT=00000080ZJVY3?oooolm0:VYZ@d000000P2YZJT;
ooooocd0ZJVY3@0000020:VYZ@cooooo?@2YZJT=00000080ZJVY3?oooolm0:VYZ@/00000102YZJT=
ooooo`003?oooolm0:VYZ@d000000P2YZJT<ooooocd0ZJVY3@0000020:VYZ@_ooooo?@2YZJT=0000
0080ZJVY3?oooolm0:VYZ@d000000P2YZJT<ooooocd0ZJVY2`0000040:VYZ@gooooo000<ooooo`X0
0000<`2YZJT2000000/0ZJVY0P00000<ooooo`H00000=`2YZJT2000000/0ZJVY0P00000;ooooo`80
0000>`2YZJT2000000/0ZJVY0P00000<ooooocd0ZJVY0P00000=0:VYZ@cooooo?@2YZJT2000000d0
ZJVY3Oooool000cooooo2P00000c0:VYZ@8000002`2YZJT2000000cooooo1P00000g0:VYZ@800000
2`2YZJT2000000_ooooo0P00000k0:VYZ@8000002`2YZJT2000000cooooo?@2YZJT2000000d0ZJVY
3?oooolm0:VYZ@8000003@2YZJT=ooooo`003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?oo
ool4000003T0ZJVY0P00000;0:VYZ@8000002oooool4000003T0ZJVY0P00000;0:VYZ@8000003?oo
ool2000003/0ZJVY0P00000;0:VYZ@8000003?oooolm0:VYZ@8000003@2YZJT=ooooo`003?ooool4
000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000002oooool4
000003T0ZJVY0P00000;0:VYZ@8000003?ooool2000003/0ZJVY0P00000;0:VYZ@8000003?oooolm
0:VYZ@8000003@2YZJT=ooooo`003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool40000
03T0ZJVY0P00000;0:VYZ@8000002oooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool20000
03/0ZJVY0P00000;0:VYZ@8000003?oooolm0:VYZ@8000003@2YZJT=ooooo`003?ooool4000003T0
ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000002oooool4000003T0
ZJVY0P00000;0:VYZ@8000003?ooool2000003/0ZJVY0P00000;0:VYZ@8000003?oooolm0:VYZ@80
00003@2YZJT=ooooo`003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY
0P00000;0:VYZ@8000002oooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY
0P00000;0:VYZ@8000003?ooool2000003/0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1000
000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000_ooooo1000
000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000cooooo0P00
000k0:VYZ@8000002`2YZJT2000000gooooo000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<
ooooo`@00000>@2YZJT2000000/0ZJVY0P00000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<
ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`800000>`2YZJT2000000/0ZJVY0P00000=
ooooo`003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VY
Z@8000002oooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VY
Z@8000003?ooool2000003/0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1000000i0:VYZ@80
00002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000_ooooo1000000i0:VYZ@80
00002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@80
00002`2YZJT2000000gooooo000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000
>@2YZJT2000000/0ZJVY0P00000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000
>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?oo
ool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000002ooo
ool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?oo
ool4000003T0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1000000i0:VYZ@8000002`2YZJT2
000000cooooo1000000i0:VYZ@8000002`2YZJT2000000_ooooo1000000i0:VYZ@8000002`2YZJT2
000000cooooo1000000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2
000000gooooo000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT20000
00/0ZJVY0P00000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT20000
00/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000003T0
ZJVY3`00000<ooooo`@00000>@2YZJT?000000_ooooo1000000i0:VYZ@l000003?ooool4000003T0
ZJVY3`00000<ooooo`@00000>@2YZJT?000000gooooo000<ooooo`@00000>@2YZJT?000000cooooo
1000000i0:VYZ@l000002oooool4000003T0ZJVY3`00000<ooooo`@00000>@2YZJT?000000cooooo
1000000i0:VYZ@l000003Oooool000cooooo1000000i0:VYZ@l000003?ooool4000003T0ZJVY3`00
000;ooooo`@00000>@2YZJT?000000cooooo1000000i0:VYZ@l000003?ooool4000003T0ZJVY3`00
000=ooooo`003?ooool4000003T0ZJVY3`00000<ooooo`@00000>@2YZJT?000000_ooooo1000000i
0:VYZ@l000003?ooool4000003T0ZJVY3`00000<ooooo`@00000>@2YZJT?000000gooooo000<oooo
o`@00000>@2YZJT?000000cooooo1000000i0:VYZ@l000002oooool4000003T0ZJVY3`00000<oooo
o`@00000>@2YZJT?000000cooooo1000000i0:VYZ@l000003Oooool000cooooo1000000i0:VYZ@l0
00003?ooool4000003T0ZJVY3`00000;ooooo`@00000>@2YZJT?000000cooooo1000000i0:VYZ@l0
00003?ooool4000003T0ZJVY3`00000=ooooo`003?oooolM00000200ZJVY3`00000<oooooaT00000
902YZJT?000000_ooooo5@00000X0:VYZ@l000003?oooolA000002`0ZJVY3`00000<ooooo`d00000
<02YZJT?000000gooooo000<oooooad00000802YZJT?000000cooooo6@00000T0:VYZ@l000002ooo
oolE000002P0ZJVY3`00000<oooooa400000;02YZJT?000000cooooo3@00000`0:VYZ@l000003Ooo
ool000cooooo7P00000O0:VYZ@l000003?oooolM00000200ZJVY3`00000;oooooaT00000902YZJT?
000000cooooo5@00000X0:VYZ@l000003?oooolA000002`0ZJVY3`00000=ooooo`003?oooolN0000
01l0ZJVY3`00000<oooooad00000802YZJT?000000_ooooo6@00000T0:VYZ@l000003?oooolE0000
02P0ZJVY3`00000<oooooa400000;02YZJT?000000gooooo000<oooooah0ZJVY0P00000M0:VYZ@d0
00000P2YZJT<ooooocd0ZJVY2`0000040:VYZ@_ooooo?@2YZJT9000000H0ZJVY3?oooolm0:VYZ@L0
0000202YZJT<ooooocd0ZJVY1`0000080:VYZ@gooooo000<oooooah0ZJVY0P00000M0:VYZ@d00000
0P2YZJT<ooooocd0ZJVY2`0000040:VYZ@_ooooo?@2YZJT9000000H0ZJVY3?oooolm0:VYZ@L00000
202YZJT<ooooocd0ZJVY1`0000080:VYZ@gooooo000<oooooah0ZJVY0P00000M0:VYZ@d000000P2Y
ZJT<ooooocd0ZJVY2`0000040:VYZ@_ooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202Y
ZJT<ooooocd0ZJVY1`0000080:VYZ@gooooo000<oooooah0ZJVY0P00000M0:VYZ@d000000P2YZJT<
ooooocd0ZJVY2`0000040:VYZ@_ooooo?@2YZJT7000000P0ZJVY3?oooolm0:VYZ@L00000202YZJT<
ooooocd0ZJVY1`0000080:VYZ@gooooo000<oooooah0ZJVY:P0000040:VYZ@cooooo7P2YZJTV0000
00P0ZJVY2ooooolR0:VYZB800000202YZJT<ooooobH0ZJVY7P0000080:VYZ@cooooo:P2YZJTJ0000
00P0ZJVY3Oooool000cooooo7P2YZJTZ000000@0ZJVY3?oooolN0:VYZBH00000202YZJT;ooooob80
ZJVY8P0000080:VYZ@cooooo9P2YZJTN000000P0ZJVY3?oooolZ0:VYZAX00000202YZJT=ooooo`00
3?oooolN0:VYZBH00000202YZJT<oooooah0ZJVY9P0000080:VYZ@_ooooo802YZJTT000000P0ZJVY
3?oooolR0:VYZB800000202YZJT<ooooobH0ZJVY7P0000080:VYZ@gooooo000<oooooah0ZJVY9P00
00080:VYZ@cooooo7P2YZJTV000000P0ZJVY2ooooolN0:VYZBH00000202YZJT<ooooob00ZJVY9000
00080:VYZ@cooooo8P2YZJTR000000P0ZJVY3Oooool000cooooo7P2YZJTV000000P0ZJVY3?oooolN
0:VYZBH00000202YZJT;oooooah0ZJVY9P0000080:VYZ@cooooo802YZJTT000000P0ZJVY3?oooolR
0:VYZB800000202YZJT=ooooo`003?oooolN0:VYZBH00000202YZJT<oooooah0ZJVY9P0000080:VY
Z@_ooooo7P2YZJTV000000P0ZJVY3?oooolP0:VYZB@00000202YZJT<ooooob80ZJVY8P0000080:VY
Z@gooooo000<oooooah0ZJVY9P0000080:VYZ@cooooo7P2YZJTV000000P0ZJVY2ooooolN0:VYZBH0
0000202YZJT<ooooob00ZJVY900000080:VYZ@cooooo8P2YZJTR000000P0ZJVY3Oooool000cooooo
7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@030000
002YZJT0ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo
7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooob00ZJVY6@00000:0:VYZ@030000
002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2Y
ZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2Y
ZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2Y
ZJT<ooooob00ZJVY6@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJTK
000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0
ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo7P2YZJTK
000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooob00ZJVY6@00000:0:VYZ@030000002YZJT0
ZJVY00H0ZJVY3Oooool000cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooo
oah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2YZJT00`00
0000ZJVY0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooo
ob00ZJVY6@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJTK000000X0
ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0
ZJVY2ooooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo802YZJTI000000X0
ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooob@0ZJVY5@00000:0:VYZ@030000002YZJT0ZJVY00H0
ZJVY3Oooool000cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY
6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2YZJT00`000000ZJVY
0:VYZ@060:VYZ@cooooo802YZJTI000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooob@0ZJVY
5@00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY2000
000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY00<0
00000:VYZ@2YZJT01P2YZJT;oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT00`000000ZJVY0:VY
Z@060:VYZ@cooooo<@2YZJT8000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000
000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY2000000:
0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY00<00000
0:VYZ@2YZJT01P2YZJT;oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@06
0:VYZ@cooooo<@2YZJT8000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000000:
0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY2000000:0:VY
Z@030000002YZJT0ZJVY00H0ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY00<000000:VY
Z@2YZJT01P2YZJT;oooooc40ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooola0:VY
Z@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo<@2YZJT8000000X0ZJVY00<000000:VY
Z@2YZJT01P2YZJT=ooooo`003?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY00<000000:VYZ@2Y
ZJT01P2YZJT<oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_o
oooo<@2YZJT8000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooc40ZJVY2000000:0:VYZ@03
0000002YZJT0ZJVY00H0ZJVY3?oooola0:VYZ@P000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@go
oooo000<oooooah0ZJVY0P00000A0:VYZA<00000202YZJT<oooooah0ZJVY0P00000A0:VYZA<00000
202YZJT;oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC000000P0ZJVY3?oooola0:VYZA<00000
202YZJT=ooooo`003?oooolN0:VYZ@8000004@2YZJTC000000P0ZJVY3?oooolN0:VYZ@8000004@2Y
ZJTC000000P0ZJVY2ooooola0:VYZA<00000202YZJT<oooooc40ZJVY4`0000080:VYZ@cooooo<@2Y
ZJTC000000P0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY5@0000060:VYZ@cooooo<@2YZJTC
000000P0ZJVY2ooooola0:VYZA<00000202YZJT<oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC
000000P0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY5@0000060:VYZ@cooooo<@2YZJTC0000
00P0ZJVY2ooooola0:VYZA<00000202YZJT<oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC0000
00P0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY5@0000060:VYZ@cooooo<@2YZJTC000000P0
ZJVY2ooooola0:VYZA<00000202YZJT<oooooc40ZJVY4`0000080:VYZ@cooooo<@2YZJTC000000P0
ZJVY3Oooool000cooooo<@2YZJTG000000@0ZJVY3?oooola0:VYZAD000001P2YZJT;oooooc40ZJVY
4`0000080:VYZ@cooooo<@2YZJTC000000P0ZJVY3?oooola0:VYZA<00000202YZJT=ooooo`003?oo
oola0:VYZAL00000102YZJT<oooooc40ZJVY5@0000060:VYZ@_ooooo<@2YZJTC000000P0ZJVY3?oo
oola0:VYZA<00000202YZJT<oooooc40ZJVY4`0000080:VYZ@gooooo000<ooooob00ZJVY:0000004
0:VYZ@cooooo8P2YZJTT000000H0ZJVY2ooooolV0:VYZAh00000202YZJT<ooooobX0ZJVY6P000008
0:VYZ@cooooo;P2YZJTF000000P0ZJVY3Oooool000cooooo802YZJTX000000@0ZJVY3?oooolR0:VY
ZB@000001P2YZJT;ooooobH0ZJVY7P0000080:VYZ@cooooo:P2YZJTJ000000P0ZJVY3?ooool^0:VY
ZAH00000202YZJT=ooooo`003?oooolR0:VYZBP000000P2YZJT<ooooobH0ZJVY8P0000040:VYZ@_o
oooo:P2YZJTL000000H0ZJVY3?ooool^0:VYZAH00000202YZJT<oooooc40ZJVY4`0000080:VYZ@go
oooo000<ooooob80ZJVY:00000020:VYZ@cooooo9P2YZJTR000000@0ZJVY2ooooolZ0:VYZA`00000
1P2YZJT<ooooobh0ZJVY5P0000080:VYZ@cooooo<@2YZJTC000000P0ZJVY3Oooool000cooooo9P2Y
ZJTT00000080ZJVY3?oooolZ0:VYZAh00000102YZJT;ooooobh0ZJVY600000060:VYZ@cooooo<@2Y
ZJTC000000P0ZJVY3?oooole0:VYZ@l00000202YZJT=ooooo`003?oooolV0:VYZB@000000P2YZJT<
ooooobX0ZJVY7P0000040:VYZ@_ooooo;P2YZJTH000000H0ZJVY3?oooola0:VYZA<00000202YZJT<
ooooocD0ZJVY3`0000080:VYZ@gooooo000<ooooocT0ZJVY4@0000020:VYZ@cooooo>@2YZJTA0000
0080ZJVY2ooooolk0:VYZ@d00000102YZJT<ooooocd0ZJVY2@0000060:VYZ@cooooo?@2YZJT70000
00P0ZJVY3Oooool000cooooo>@2YZJTA00000080ZJVY3?ooooli0:VYZA4000000P2YZJT;oooooc/0
ZJVY3@0000040:VYZ@cooooo?@2YZJT9000000H0ZJVY3?oooolm0:VYZ@L00000202YZJT=ooooo`00
ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00
ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00ooooooo5ooooo`00
ooooooo5ooooo`003?oooolG0:VYZ@800000802YZJTA00000080ZJVY3?oooolG0:VYZ@800000802Y
ZJTA00000080ZJVY2oooooli0:VYZA4000000P2YZJT<ooooocT0ZJVY4@0000020:VYZ@cooooo>@2Y
ZJTA00000080ZJVY3Oooool000cooooo5`2YZJT200000200ZJVY4@0000020:VYZ@cooooo5`2YZJT2
00000200ZJVY4@0000020:VYZ@_ooooo>@2YZJTA00000080ZJVY3?ooooli0:VYZA4000000P2YZJT<
ooooocT0ZJVY4@0000020:VYZ@gooooo000<oooooaL0ZJVY0P00000P0:VYZA4000000P2YZJT<oooo
oaL0ZJVY0P00000P0:VYZA4000000P2YZJT;ooooocT0ZJVY4@0000020:VYZ@cooooo>@2YZJTA0000
0080ZJVY3?ooooli0:VYZA4000000P2YZJT=ooooo`003?oooolG0:VYZ@800000802YZJTA00000080
ZJVY3?oooolG0:VYZ@800000802YZJTA00000080ZJVY2oooooli0:VYZA4000000P2YZJT<ooooocT0
ZJVY4@0000020:VYZ@cooooo>@2YZJTA00000080ZJVY3Oooool000cooooo5`2YZJT200000200ZJVY
4@0000020:VYZ@cooooo5`2YZJT200000200ZJVY4@0000020:VYZ@_ooooo5`2YZJT200000200ZJVY
4@0000020:VYZ@cooooo>@2YZJTA00000080ZJVY3?ooooli0:VYZA4000000P2YZJT=ooooo`003?oo
oolG0:VYZ@800000802YZJTA00000080ZJVY3?oooolG0:VYZ@800000802YZJTA00000080ZJVY2ooo
oolG0:VYZ@800000802YZJTA00000080ZJVY3?ooooli0:VYZA4000000P2YZJT<ooooocT0ZJVY4@00
00020:VYZ@gooooo000<oooooaL0ZJVY0P00000P0:VYZ@P000001`2YZJT200000080ZJVY3?oooolG
0:VYZ@800000802YZJT8000000L0ZJVY0P0000020:VYZ@_ooooo5`2YZJT200000200ZJVY20000007
0:VYZ@8000000P2YZJT<ooooocT0ZJVY200000070:VYZ@8000000P2YZJT<ooooocT0ZJVY20000007
0:VYZ@8000000P2YZJT=ooooo`003?oooolG0:VYZ@800000802YZJT8000000L0ZJVY0P0000020:VY
Z@cooooo5`2YZJT200000200ZJVY200000070:VYZ@8000000P2YZJT;oooooaL0ZJVY0P00000P0:VY
Z@P000001`2YZJT200000080ZJVY3?ooooli0:VYZ@P000001`2YZJT200000080ZJVY3?ooooli0:VY
Z@P000001`2YZJT200000080ZJVY3Oooool000cooooo5`2YZJT200000200ZJVY200000070:VYZ@@0
00003?oooolG0:VYZ@800000802YZJT8000000L0ZJVY0P0000020:VYZ@_ooooo5`2YZJT200000200
ZJVY200000070:VYZ@8000000P2YZJT<oooooaL0ZJVY0P00000P0:VYZ@P000001`2YZJT200000080
ZJVY3?oooolk0:VYZ@H000001`2YZJT200000080ZJVY3Oooool000cooooo5`2YZJT200000200ZJVY
200000070:VYZ@@000003?oooolG0:VYZ@800000802YZJT8000000L0ZJVY0P0000020:VYZ@_ooooo
5`2YZJT200000200ZJVY200000070:VYZ@8000000P2YZJT<ooooocT0ZJVY200000070:VYZ@800000
0P2YZJT<oooooc/0ZJVY1P0000070:VYZ@8000000P2YZJT=ooooo`003?oooolG0:VYZ@800000802Y
ZJT8000000L0ZJVY1000000<oooooaL0ZJVY0P00000P0:VYZ@P000001`2YZJT200000080ZJVY2ooo
oolG0:VYZ@800000802YZJT8000000L0ZJVY0P0000020:VYZ@cooooo>@2YZJT8000000L0ZJVY0P00
00020:VYZ@cooooo>`2YZJT6000000L0ZJVY0P0000020:VYZ@gooooo000<ooooo`8000005@2YZJT2
00000200ZJVY200000070:VYZ@@000003?oooolG0:VYZ@800000802YZJT8000000L0ZJVY1000000;
oooooaL0ZJVY0P00000P0:VYZ@P000001`2YZJT200000080ZJVY3?oooolk0:VYZ@H000001`2YZJT2
00000080ZJVY3?oooolm0:VYZ@@000001`2YZJT200000080ZJVY3Oooool000cooooo0P00000E0:VY
Z@800000802YZJT8000000L0ZJVY1000000<oooooaL0ZJVY0P00000P0:VYZ@P000001`2YZJT40000
00_ooooo5`2YZJT200000200ZJVY200000070:VYZ@8000000P2YZJT<oooooc/0ZJVY1P0000070:VY
Z@8000000P2YZJT<ooooocd0ZJVY100000070:VYZ@8000000P2YZJT=ooooo`003?ooool2000001D0
ZJVY0P00000P0:VYZA<000003?oooolG0:VYZ@800000802YZJTC000000_ooooo>@2YZJTA00000080
ZJVY3?oooolk0:VYZ@l000000P2YZJT<ooooocd0ZJVY3@0000020:VYZ@gooooo000<ooooo`800000
5@2YZJT200000200ZJVY4`00000<oooooaL0ZJVY0P00000P0:VYZA<000002oooooli0:VYZA400000
0P2YZJT<oooooc/0ZJVY3`0000020:VYZ@cooooo?@2YZJT=00000080ZJVY3Oooool000cooooo7@00
000L0:VYZ@H000002`2YZJT2000000cooooo6@00000P0:VYZ@H000002`2YZJT2000000_ooooo5@00
000V0:VYZ@@000002`2YZJT2000000cooooo4@00000/0:VYZ@8000002`2YZJT2000000cooooo3@00
000`0:VYZ@8000002`2YZJT2000000gooooo000<oooooad00000702YZJT6000000/0ZJVY0P00000<
oooooaT00000802YZJT6000000/0ZJVY0P00000;oooooaD000009P2YZJT4000000/0ZJVY0P00000<
oooooa400000;02YZJT2000000/0ZJVY0P00000<ooooo`d00000<02YZJT2000000/0ZJVY0P00000=
ooooo`003?ooool600000340ZJVY2000000;0:VYZ@8000003?ooool4000003D0ZJVY1P00000;0:VY
Z@8000002oooool4000003L0ZJVY1000000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VY
Z@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1P00000a0:VYZ@P0
00002`2YZJT2000000cooooo1000000e0:VYZ@H000002`2YZJT2000000_ooooo1000000g0:VYZ@@0
00002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@80
00002`2YZJT2000000gooooo000<ooooo`@00000<`2YZJT8000000/0ZJVY0P00000<ooooo`@00000
=`2YZJT4000000/0ZJVY0P00000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000
>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?oo
ool4000003<0ZJVY2000000;0:VYZ@8000003?ooool4000003L0ZJVY1000000;0:VYZ@8000002ooo
ool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003?oo
ool4000003T0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1000000J0:VYZ@8000006`2YZJT4
000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000;ooooo`@00000>@2YZJT2
000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2
000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY0P00000K0:VYZ@@000002`2YZJT20000
00cooooo1000000i0:VYZ@8000002`2YZJT2000000_ooooo1000000i0:VYZ@8000002`2YZJT20000
00cooooo1000000i0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT20000
00gooooo000<ooooo`@000006P2YZJT2000001d0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0
ZJVY0P00000;0:VYZ@8000002oooool4000003T0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0
ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo
1000000J0:VYZ@8000007@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY
0P00000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY
0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000
000K0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZ@8000007@2YZJT2000000/0ZJVY0P00
000;ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00
000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000K
0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZ@8000007@2YZJT2000000/0ZJVY0P00000;
ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<
ooooo`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000K0:VY
Z@8000002`2YZJT2000000cooooo1000000J0:VYZ@8000007@2YZJT2000000/0ZJVY0P00000;oooo
o`@00000>@2YZJT2000000/0ZJVY0P00000<ooooo`@00000>@2YZJT2000000/0ZJVY0P00000<oooo
o`@00000>@2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000K0:VYZ@80
00002`2YZJT2000000cooooo1000000J0:VYZ@@000006`2YZJT2000000/0ZJVY0P00000;ooooo`@0
00006P2YZJT2000001d0ZJVY0P00000;0:VYZ@8000003?ooool4000003T0ZJVY0P00000;0:VYZ@80
00003?ooool4000003T0ZJVY0P00000;0:VYZ@8000003Oooool000cooooo1000000J0:VYZ@@00000
6`2YZJT2000000/0ZJVY0P00000<ooooo`@000006P2YZJT4000001/0ZJVY0P00000;0:VYZ@800000
2oooool4000001X0ZJVY0P00000M0:VYZ@8000002`2YZJT2000000cooooo1000000i0:VYZ@800000
2`2YZJT2000000cooooo1000000i0:VYZ@8000002`2YZJT2000000gooooo000<ooooo`@000006P2Y
ZJT4000001/0ZJVY3`00000<ooooo`@000006P2YZJT4000001/0ZJVY3`00000;ooooo`@000006P2Y
ZJT2000001d0ZJVY3`00000<ooooo`@00000>@2YZJT?000000cooooo1000000i0:VYZ@l000003Ooo
ool000cooooo1000000J0:VYZ@@000006`2YZJT?000000cooooo1000000J0:VYZ@@000006`2YZJT?
000000_ooooo1000000J0:VYZ@8000007@2YZJT?000000cooooo1000000i0:VYZ@l000003?ooool4
000003T0ZJVY3`00000=ooooo`003?ooool4000001X0ZJVY2000000G0:VYZ@l000003?ooool40000
01X0ZJVY1000000K0:VYZ@l000002oooool4000001X0ZJVY1000000K0:VYZ@l000003?ooool40000
01X0ZJVY0P00000M0:VYZ@l000003?ooool4000003T0ZJVY3`00000=ooooo`003?ooool4000001X0
ZJVY2000000G0:VYZ@l000003?ooool4000001X0ZJVY1000000K0:VYZ@l000002oooool4000001X0
ZJVY1000000K0:VYZ@l000003?ooool4000001X0ZJVY0P00000M0:VYZ@l000003?ooool4000003T0
ZJVY3`00000=ooooo`003?ooool4000001X0ZJVY2P00000E0:VYZ@l000003?ooool4000001X0ZJVY
2000000G0:VYZ@l000002oooool4000001X0ZJVY1000000K0:VYZ@l000003?ooool4000001X0ZJVY
0P00000M0:VYZ@l000003?ooool4000003T0ZJVY3`00000=ooooo`003?ooool4000001X0ZJVY2P00
000E0:VYZ@l000003?ooool4000001X0ZJVY2000000G0:VYZ@l000002oooool4000001X0ZJVY1000
000K0:VYZ@l000003?ooool4000001X0ZJVY0P00000M0:VYZ@l000003?ooool4000003T0ZJVY3`00
000=ooooo`003?oooolP000000P0ZJVY0P00000C0:VYZ@l000003?oooolP000001d0ZJVY3`00000;
ooooob0000007@2YZJT?000000cooooo8000000M0:VYZ@l000003?oooolP000001d0ZJVY3`00000=
ooooo`003?oooolP000000P0ZJVY0P00000C0:VYZ@l000003?oooolP000001d0ZJVY3`00000;oooo
ob0000007@2YZJT?000000cooooo8000000M0:VYZ@l000003?oooolP000001d0ZJVY3`00000=oooo
o`003?oooolP000000P0ZJVY0P00000C0:VYZ@l000003?oooolP000001d0ZJVY3`00000;ooooob00
00007@2YZJT?000000cooooo8000000M0:VYZ@l000003?oooolP000001d0ZJVY3`00000=ooooo`00
3?oooolP000000P0ZJVY0P00000C0:VYZ@l000003?oooolP000001d0ZJVY3`00000;ooooob000000
7@2YZJT?000000cooooo8000000M0:VYZ@l000003?oooolP000001d0ZJVY3`00000=ooooo`003?oo
ool4000001X0ZJVY0P0000080:VYZ@@000004@2YZJT?000000cooooo1000000J0:VYZ@800000202Y
ZJT2000001<0ZJVY3`00000;ooooo`@000006P2YZJT2000001d0ZJVY3`00000<ooooo`800000702Y
ZJT2000001d0ZJVY3`00000<oooooah0ZJVY0P00000M0:VYZ@l000003Oooool000cooooo1000000J
0:VYZ@800000202YZJT400000140ZJVY3`00000<ooooo`@000006P2YZJT2000000P0ZJVY0P00000C
0:VYZ@l000002oooool4000001X0ZJVY0P00000M0:VYZ@l000003?ooool2000001`0ZJVY0P00000M
0:VYZ@l000003?oooolN0:VYZ@8000007@2YZJT?000000gooooo000<ooooo`@000006P2YZJT20000
00P0ZJVY1000000A0:VYZ@l000003?ooool4000001X0ZJVY0P0000080:VYZ@8000004`2YZJT?0000
00_ooooo1000000J0:VYZ@8000007@2YZJT?000000cooooo0P00000L0:VYZ@8000007@2YZJT?0000
00cooooo7P2YZJT2000001d0ZJVY3`00000=ooooo`003?ooool4000001X0ZJVY0P0000080:VYZ@@0
00004@2YZJT?000000cooooo1000000J0:VYZ@800000202YZJT2000001<0ZJVY3`00000;ooooo`@0
00006P2YZJT2000001d0ZJVY3`00000<ooooo`800000702YZJT2000001d0ZJVY3`00000<oooooah0
ZJVY0P00000M0:VYZ@l000003Oooool000cooooo1000000J0:VYZBh000003?ooool4000001X0ZJVY
;P00000;ooooo`800000702YZJT^000000cooooo7P2YZJT^000000cooooo7P2YZJT/00000080ZJVY
3Oooool000cooooo1000000J0:VYZBh000003?ooool4000001X0ZJVY;P00000;ooooo`800000702Y
ZJT^000000cooooo7P2YZJT^000000cooooo7P2YZJT/00000080ZJVY3Oooool000cooooo1000000J
0:VYZBh000003?ooool4000001X0ZJVY;P00000;ooooo`800000702YZJT^000000cooooo7P2YZJT^
000000cooooo7P2YZJTZ000000@0ZJVY3Oooool000cooooo1000000J0:VYZBh000003?ooool20000
01`0ZJVY;P00000;oooooah0ZJVY;P00000<oooooah0ZJVY:P0000040:VYZ@cooooo7P2YZJTV0000
00P0ZJVY3Oooool000cooooo1000000J0:VYZBh000003?ooool2000001`0ZJVY;P00000;oooooah0
ZJVY;P00000<oooooah0ZJVY:P0000040:VYZ@cooooo7P2YZJTV000000P0ZJVY3Oooool000cooooo
1000000J0:VYZBH00000202YZJT<ooooo`800000702YZJTV000000P0ZJVY2ooooolN0:VYZBH00000
202YZJT<oooooah0ZJVY9P0000080:VYZ@cooooo7P2YZJTV000000P0ZJVY3Oooool000cooooo1000
000J0:VYZBH00000202YZJT<ooooo`800000702YZJTV000000P0ZJVY2ooooolN0:VYZBH00000202Y
ZJT<oooooah0ZJVY9P0000080:VYZ@cooooo7P2YZJTV000000P0ZJVY3Oooool000cooooo0P00000L
0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<00000
0:VYZ@2YZJT01P2YZJT;oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolN
0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<00000
0:VYZ@2YZJT01P2YZJT=ooooo`003?ooool2000001`0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY
00H0ZJVY3?oooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo7P2YZJTK0000
00X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY
00H0ZJVY3?oooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<ooooo`80
0000702YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@03
0000002YZJT0ZJVY00H0ZJVY2ooooolN0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@co
oooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<oooooah0ZJVY6`00000:0:VYZ@03
0000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo0P00000L0:VYZA/000002P2YZJT00`000000ZJVY
0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT;oooooah0ZJVY
6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolN0:VYZA/000002P2YZJT00`000000ZJVY
0:VYZ@060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT=ooooo`003?oo
oolN0:VYZA/000002P2YZJT9000000cooooo7P2YZJTK000000X0ZJVY1`0000020:VYZ@_ooooo7P2Y
ZJTK000000X0ZJVY0`0000060:VYZ@cooooo7P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2Y
ZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo7P2YZJTK
000000X0ZJVY2@00000<oooooah0ZJVY6`00000:0:VYZ@L000000P2YZJT;oooooah0ZJVY6`00000:
0:VYZ@<000001P2YZJT<oooooah0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?oooolN
0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<oooooah0ZJVY0P00000A0:VY
Z@P000002P2YZJT700000080ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY1`0000020:VY
Z@_ooooo7P2YZJT200000140ZJVY2000000:0:VYZ@D00000102YZJT<oooooah0ZJVY0P00000A0:VY
Z@P000002P2YZJT3000000H0ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY00<000000:VY
Z@2YZJT01P2YZJT=ooooo`003?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@co
oooo7P2YZJT200000140ZJVY2000000:0:VYZ@L000000P2YZJT;oooooah0ZJVY0P00000A0:VYZ@P0
00002P2YZJT5000000@0ZJVY3?oooolN0:VYZ@8000004@2YZJT8000000X0ZJVY0`0000060:VYZ@co
oooo7P2YZJT200000140ZJVY2000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo
7P2YZJT200000140ZJVY2000000:0:VYZ@L000000P2YZJT<oooooah0ZJVY0P00000A0:VYZ@P00000
2P2YZJT700000080ZJVY2ooooolN0:VYZ@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@cooooo
7P2YZJT200000140ZJVY2000000:0:VYZ@D00000102YZJT<oooooah0ZJVY0P00000A0:VYZ@P00000
2P2YZJT3000000H0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY2000000:0:VYZ@L000000P2Y
ZJT<oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT700000080ZJVY2ooooolN0:VYZ@8000004@2Y
ZJT8000000X0ZJVY1`0000020:VYZ@cooooo7P2YZJT200000140ZJVY2000000:0:VYZ@D00000102Y
ZJT<oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT3000000H0ZJVY3Oooool000cooooo7P2YZJT2
00000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@_ooooo7P2YZJT2
00000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY5`0000040:VYZ@cooooo7P2YZJT2
00000140ZJVY5@0000060:VYZ@gooooo000<oooooah0ZJVY0P00000A0:VYZAT000000P2YZJT<oooo
oah0ZJVY0P00000A0:VYZAT000000P2YZJT;oooooah0ZJVY0P00000A0:VYZAT000000P2YZJT<oooo
oah0ZJVY0P00000A0:VYZAL00000102YZJT<oooooah0ZJVY0P00000A0:VYZAD000001P2YZJT=oooo
o`003?oooolN0:VYZ@8000004@2YZJTI00000080ZJVY3?oooolN0:VYZ@8000004@2YZJTI00000080
ZJVY2ooooolN0:VYZ@8000004@2YZJTI00000080ZJVY3?oooolN0:VYZ@8000004@2YZJTI00000080
ZJVY3?oooolN0:VYZ@8000004@2YZJTG000000@0ZJVY3Oooool000cooooo7P2YZJT200000140ZJVY
6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@_ooooo7P2YZJT200000140ZJVY
6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY
5`0000040:VYZ@gooooo000<oooooad0ZJVY0`00000A0:VYZAT000000P2YZJT<oooooah0ZJVY0P00
000A0:VYZAT000000P2YZJT;oooooah0ZJVY0P00000A0:VYZAT000000P2YZJT<oooooah0ZJVY0P00
000A0:VYZAT000000P2YZJT<oooooah0ZJVY0P00000A0:VYZAL00000102YZJT=ooooo`003?oooolK
0:VYZ@D000004@2YZJTI00000080ZJVY3?oooolM0:VYZ@<000004@2YZJTI00000080ZJVY2ooooolN
0:VYZ@8000004@2YZJTI00000080ZJVY3?oooolN0:VYZ@8000004@2YZJTI00000080ZJVY3?oooolN
0:VYZ@8000004@2YZJTI00000080ZJVY3Oooool000cooooo6`2YZJT500000140ZJVY6@0000020:VY
Z@cooooo7@2YZJT300000140ZJVY6@0000020:VYZ@_ooooo7P2YZJT200000140ZJVY6@0000020:VY
Z@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VY
Z@gooooo000<oooooaT0ZJVY<@0000020:VYZ@cooooo6`2YZJT_00000080ZJVY2ooooolM0:VYZBd0
00000P2YZJT<oooooah0ZJVY;00000020:VYZ@cooooo7P2YZJT/00000080ZJVY3Oooool000cooooo
6@2YZJTa00000080ZJVY3?oooolK0:VYZBl000000P2YZJT;oooooad0ZJVY;@0000020:VYZ@cooooo
7P2YZJT/00000080ZJVY3?oooolN0:VYZB`000000P2YZJT=ooooo`003?oooolI0:VYZC4000000P2Y
ZJT<oooooa/0ZJVY;`0000020:VYZ@_ooooo7@2YZJT]00000080ZJVY3?oooolN0:VYZB`000000P2Y
ZJT<ooooob00ZJVY:P0000020:VYZ@gooooo000<oooooaT0ZJVY<@0000020:VYZ@cooooo6`2YZJT_
00000080ZJVY2ooooolM0:VYZBd000000P2YZJT<oooooah0ZJVY;00000020:VYZ@cooooo802YZJTZ
00000080ZJVY3Oooool000cooooo5`2YZJTc00000080ZJVY3?oooolI0:VYZC4000000P2YZJT;oooo
oa/0ZJVY;`0000020:VYZ@cooooo7P2YZJT/00000080ZJVY3?oooolR0:VYZBP000000P2YZJT=oooo
o`003?oooolG0:VYZC<000000P2YZJT<oooooaT0ZJVY<@0000020:VYZ@_ooooo6`2YZJT_00000080
ZJVY3?oooolN0:VYZB`000000P2YZJT<ooooob80ZJVY:00000020:VYZ@gooooo000<oooooaL0ZJVY
0P00000P0:VYZA4000000P2YZJT<ooooocT0ZJVY4@0000020:VYZ@_ooooo>@2YZJTA00000080ZJVY
3?ooooli0:VYZA4000000P2YZJT<ooooocT0ZJVY4@0000020:VYZ@gooooo000<oooooaL0ZJVY0P00
000P0:VYZA4000000P2YZJT<ooooocT0ZJVY4@0000020:VYZ@_ooooo>@2YZJTA00000080ZJVY3?oo
ooli0:VYZA4000000P2YZJT<ooooocT0ZJVY4@0000020:VYZ@gooooo003oooooolGooooo003ooooo
olGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003ooooo
olGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003oooooolGooooo003ooooo
olGooooo000<ooooo`H000004@2YZJT6000001<0ZJVY7000000<ooooo`8000005@2YZJT6000001H0
ZJVY6@00000;oooooaL0ZJVY1P00000J0:VYZAD000003?oooolG0:VYZ@H00000702YZJTA00000080
ZJVY3?oooolG0:VYZ@@000007P2YZJTA00000080ZJVY3Oooool000cooooo1P00000A0:VYZ@H00000
4`2YZJTL000000cooooo0P00000E0:VYZ@H000005P2YZJTI000000_ooooo5`2YZJT6000001X0ZJVY
5@00000<oooooaL0ZJVY1P00000L0:VYZA4000000P2YZJT<oooooaL0ZJVY1000000N0:VYZA400000
0P2YZJT=ooooo`003?ooool600000140ZJVY1P00000F0:VYZAT000003?ooool4000001<0ZJVY1P00
000J0:VYZAD000002oooool2000001D0ZJVY1P00000L0:VYZA<000003?oooolG0:VYZ@@000007P2Y
ZJTC000000cooooo5`2YZJT200000200ZJVY4@0000020:VYZ@gooooo000<ooooo`H000004@2YZJT6
000001H0ZJVY6@00000<ooooo`@000004`2YZJT6000001X0ZJVY5@00000;ooooo`8000005@2YZJT6
000001`0ZJVY4`00000<oooooaL0ZJVY1000000N0:VYZA<000003?oooolG0:VYZ@800000802YZJTA
00000080ZJVY3Oooool000cooooo2000000=0:VYZ@P00000702YZJTC000000cooooo1P00000A0:VY
Z@H00000702YZJTC000000_ooooo1000000C0:VYZ@H00000702YZJTC000000cooooo0P00000E0:VY
Z@800000802YZJTC000000cooooo5`2YZJT200000200ZJVY4`00000=ooooo`003?ooool8000000d0
ZJVY2000000L0:VYZA<000003?ooool600000140ZJVY1P00000L0:VYZA<000002oooool4000001<0
ZJVY1P00000L0:VYZA<000003?ooool2000001D0ZJVY0P00000P0:VYZA<000003?oooolG0:VYZ@80
0000802YZJTC000000gooooo000<ooooo`P000002`2YZJT=000001T0ZJVY200000070:VYZ@@00000
3?ooool6000000l0ZJVY2000000L0:VYZ@P000001`2YZJT4000000_ooooo1000000C0:VYZ@800000
802YZJT8000000L0ZJVY1000000<ooooo`8000005@2YZJT200000200ZJVY200000070:VYZ@@00000
3?oooolG0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool8000000/0ZJVY3@00
000I0:VYZ@P000001`2YZJT4000000cooooo1P00000?0:VYZ@P00000702YZJT8000000L0ZJVY1000
000;ooooo`@000004`2YZJT200000200ZJVY200000070:VYZ@@000003?ooool2000001D0ZJVY0P00
000P0:VYZ@P000001`2YZJT4000000cooooo5`2YZJT200000200ZJVY200000070:VYZ@@000003Ooo
ool000cooooo2000000;0:VYZ@H000001@2YZJT6000001D0ZJVY200000070:VYZ@@000003?ooool8
000000d0ZJVY100000050:VYZ@8000006@2YZJT8000000L0ZJVY1000000;ooooo`H000004@2YZJT2
00000200ZJVY200000070:VYZ@@000003?ooool4000001<0ZJVY0P00000P0:VYZ@P000001`2YZJT4
000000cooooo0P00000E0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool80000
00/0ZJVY1P0000050:VYZ@H000005@2YZJT8000000L0ZJVY1000000<ooooo`P000002`2YZJT60000
00D0ZJVY1000000G0:VYZ@P000001`2YZJT4000000_ooooo1P00000?0:VYZ@@000001@2YZJT20000
01T0ZJVY200000070:VYZ@@000003?ooool4000001<0ZJVY0P00000P0:VYZ@P000001`2YZJT40000
00cooooo0P00000E0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool8000000/0
ZJVY1P0000050:VYZ@H000005@2YZJT8000000L0ZJVY1000000<ooooo`P000002`2YZJT6000000D0
ZJVY1000000G0:VYZ@P000001`2YZJT4000000_ooooo1P00000?0:VYZ@@000001@2YZJT2000001T0
ZJVY200000070:VYZ@@000003?ooool4000001<0ZJVY0P00000P0:VYZ@P000001`2YZJT4000000co
oooo0P00000E0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool8000000/0ZJVY
1P0000050:VYZ@@000003P2YZJTA000000L0ZJVY1000000<ooooo`P000002`2YZJT6000000D0ZJVY
1000000A0:VYZ@h000001`2YZJT4000000_ooooo2000000=0:VYZ@@000001@2YZJT2000001L0ZJVY
2P0000070:VYZ@@000003?ooool600000140ZJVY0P00000P0:VYZ@P000001`2YZJT4000000cooooo
1000000C0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool8000000/0ZJVY1P00
00050:VYZ@@000003P2YZJTA000000L0ZJVY1000000<ooooo`P000002`2YZJT6000000D0ZJVY1000
000A0:VYZ@h000001`2YZJT4000000_ooooo2000000=0:VYZ@@000001@2YZJT2000001L0ZJVY2P00
00070:VYZ@@000003?ooool600000140ZJVY0P00000P0:VYZ@P000001`2YZJT4000000cooooo1000
000C0:VYZ@800000802YZJT8000000L0ZJVY1000000=ooooo`003?ooool8000000/0ZJVY3`00000>
0:VYZA`000003?ooool8000000/0ZJVY3`00000?0:VYZA/000002oooool8000000/0ZJVY3`00000A
0:VYZAT000003?ooool6000000l0ZJVY2`00000G0:VYZAD000003?ooool4000001<0ZJVY1P00000L
0:VYZA<000003Oooool000cooooo2000000;0:VYZ@l000003P2YZJTL000000cooooo2000000;0:VY
Z@l000003`2YZJTK000000_ooooo2000000;0:VYZ@l000004@2YZJTI000000cooooo1P00000?0:VY
Z@/000005`2YZJTE000000cooooo1000000C0:VYZ@H00000702YZJTC000000gooooo000<ooooob80
00003P2YZJT?000000/0ZJVY0P00000<ooooob8000003P2YZJT?000000/0ZJVY0P00000;ooooob80
00003`2YZJT>000000/0ZJVY0P00000<ooooob0000004`2YZJT<000000/0ZJVY0P00000<oooooah0
00006@2YZJT8000000/0ZJVY0P00000=ooooo`003?oooolR000000h0ZJVY3`00000;0:VYZ@800000
3?oooolR000000h0ZJVY3`00000;0:VYZ@8000002ooooolR000000l0ZJVY3P00000;0:VYZ@800000
3?oooolP000001<0ZJVY3000000;0:VYZ@8000003?oooolN000001T0ZJVY2000000;0:VYZ@800000
3Oooool000cooooo5`0000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00000<oooooaD000002@2Y
ZJT4000000h0ZJVY3`00000;0:VYZ@8000002ooooolA000000d0ZJVY1000000?0:VYZ@h000002`2Y
ZJT2000000cooooo3@00000A0:VYZ@@000004@2YZJT<000000/0ZJVY0P00000<ooooo`X00000502Y
ZJT2000001D0ZJVY2P00000;0:VYZ@8000003Oooool000cooooo5`0000070:VYZ@@000003P2YZJT?
000000/0ZJVY0P00000<oooooaD000002@2YZJT4000000h0ZJVY3`00000;0:VYZ@8000002ooooolA
000000d0ZJVY1000000?0:VYZ@h000002`2YZJT2000000cooooo3@00000A0:VYZ@@000004@2YZJT<
000000/0ZJVY0P00000<ooooo`X00000502YZJT2000001D0ZJVY2P00000;0:VYZ@8000003Oooool0
00cooooo1000000?0:VYZ@@000001`2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003?ooool40000
01X0ZJVY1000000>0:VYZ@l000002`2YZJT2000000_ooooo1000000J0:VYZ@@000003P2YZJT?0000
00/0ZJVY0P00000<ooooo`@000006P2YZJT4000000l0ZJVY3P00000;0:VYZ@8000003?ooool40000
01X0ZJVY0P00000C0:VYZ@`000002`2YZJT2000000gooooo000<ooooo`@000003`2YZJT4000000L0
ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000J0:VYZ@@000003P2YZJT?000000/0
ZJVY0P00000;ooooo`@000006P2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003?ooool4000001X0
ZJVY1000000?0:VYZ@h000002`2YZJT2000000cooooo1000000J0:VYZ@8000004`2YZJT<000000/0
ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo
1000000J0:VYZ@@000003P2YZJT?000000/0ZJVY0P00000;ooooo`@000006P2YZJT4000000l0ZJVY
3P00000;0:VYZ@8000003?ooool4000001X0ZJVY1000000A0:VYZ@`000002`2YZJT2000000cooooo
1000000J0:VYZ@@000005@2YZJT8000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000
000>0:VYZ@l000002`2YZJT2000000cooooo1000000J0:VYZ@@000003P2YZJT?000000/0ZJVY0P00
000;ooooo`@000006P2YZJT4000000l0ZJVY3P00000;0:VYZ@8000003?ooool4000001X0ZJVY1000
000A0:VYZ@`000002`2YZJT2000000cooooo1000000J0:VYZ@@000005@2YZJT8000000/0ZJVY0P00
000=ooooo`003?ooool4000001X0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000J
0:VYZ@@000003P2YZJT?000000/0ZJVY0P00000;ooooo`@000006P2YZJT400000140ZJVY3000000;
0:VYZ@8000003?ooool4000001X0ZJVY1000000E0:VYZ@P000002`2YZJT2000000cooooo1000000J
0:VYZ@@000006@2YZJT4000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000>0:VY
Z@l000002`2YZJT2000000cooooo1000000J0:VYZ@@000003P2YZJT?000000/0ZJVY0P00000;oooo
o`@000006P2YZJT400000140ZJVY3000000;0:VYZ@8000003?ooool4000001X0ZJVY1000000E0:VY
Z@P000002`2YZJT2000000cooooo1000000J0:VYZ@@000006@2YZJT4000000/0ZJVY0P00000=oooo
o`003?ooool4000001X0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000J0:VYZ@@0
00004@2YZJT<000000/0ZJVY0P00000;ooooo`@000006P2YZJT4000001D0ZJVY2000000;0:VYZ@80
00003?ooool4000001X0ZJVY1000000I0:VYZ@@000002`2YZJT2000000cooooo1000000J0:VYZ@@0
00006`2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY1000000>0:VYZ@l00000
2`2YZJT2000000cooooo1000000J0:VYZ@@000004@2YZJT<000000/0ZJVY0P00000;ooooo`@00000
6P2YZJT4000001D0ZJVY2000000;0:VYZ@8000003?ooool4000001X0ZJVY1000000I0:VYZ@@00000
2`2YZJT2000000cooooo1000000J0:VYZ@@000006`2YZJT2000000/0ZJVY0P00000=ooooo`003?oo
ool4000001X0ZJVY4000000?0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZ@`000004`2Y
ZJT2000000/0ZJVY0P00000;ooooo`@000006P2YZJT8000001L0ZJVY0P00000;0:VYZ@8000003?oo
ool4000001X0ZJVY1000000K0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZ@@000006`2Y
ZJT2000000/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY4000000?0:VYZ@8000002`2YZJT2
000000cooooo1000000J0:VYZA0000003`2YZJT2000000/0ZJVY0P00000;ooooo`@000006P2YZJT<
000001<0ZJVY0P00000;0:VYZ@8000003?ooool4000001X0ZJVY2000000G0:VYZ@8000002`2YZJT2
000000cooooo1000000J0:VYZ@@000006`2YZJT2000000/0ZJVY0P00000=ooooo`003?ooool40000
01X0ZJVY4000000?0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZA0000003`2YZJT20000
00/0ZJVY0P00000;ooooo`@000006P2YZJT<000001<0ZJVY0P00000;0:VYZ@8000003?ooool40000
01X0ZJVY2000000G0:VYZ@8000002`2YZJT2000000cooooo1000000J0:VYZ@@000006`2YZJT20000
00/0ZJVY0P00000=ooooo`003?ooool4000001X0ZJVY4000000?0:VYZ@l000003?ooool4000001X0
ZJVY4000000?0:VYZ@l000002oooool4000001X0ZJVY3P00000A0:VYZ@l000003?ooool4000001X0
ZJVY3000000C0:VYZ@l000003?ooool4000001X0ZJVY2000000G0:VYZ@l000003Oooool000cooooo
1000000J0:VYZA0000003`2YZJT?000000cooooo1000000J0:VYZA0000003`2YZJT?000000_ooooo
1000000J0:VYZ@h000004@2YZJT?000000cooooo1000000J0:VYZ@`000004`2YZJT?000000cooooo
1000000J0:VYZ@P000005`2YZJT?000000gooooo000<ooooo`P000005P2YZJT@000000l0ZJVY3`00
000<ooooo`@000006P2YZJT@000000l0ZJVY3`00000;ooooo`@000006P2YZJT@000000l0ZJVY3`00
000<ooooo`@000006P2YZJT>00000140ZJVY3`00000<ooooo`@000006P2YZJT<000001<0ZJVY3`00
000=ooooo`003?ooool8000001H0ZJVY4000000?0:VYZ@l000003?ooool4000001X0ZJVY4000000?
0:VYZ@l000002oooool4000001X0ZJVY4000000?0:VYZ@l000003?ooool4000001X0ZJVY3P00000A
0:VYZ@l000003?ooool4000001X0ZJVY3000000C0:VYZ@l000003Oooool000cooooo2`00000C0:VY
ZA0000003`2YZJT?000000cooooo2000000F0:VYZA0000003`2YZJT?000000_ooooo1000000J0:VY
ZA0000003`2YZJT?000000cooooo1000000J0:VYZ@h000004@2YZJT?000000cooooo1000000J0:VY
Z@`000004`2YZJT?000000gooooo000<ooooo`/000004`2YZJT@000000l0ZJVY3`00000<ooooo`P0
00005P2YZJT@000000l0ZJVY3`00000;ooooo`@000006P2YZJT@000000l0ZJVY3`00000<ooooo`@0
00006P2YZJT>00000140ZJVY3`00000<ooooo`@000006P2YZJT<000001<0ZJVY3`00000=ooooo`00
3?oooolP000000P0ZJVY1P00000?0:VYZ@l000003?oooolP000000P0ZJVY1P00000?0:VYZ@l00000
2ooooolP000000P0ZJVY1P00000?0:VYZ@l000003?oooolP000000P0ZJVY1P00000?0:VYZ@l00000
3?oooolP000000P0ZJVY1000000A0:VYZ@l000003Oooool000cooooo800000080:VYZ@H000003`2Y
ZJT?000000cooooo800000080:VYZ@H000003`2YZJT?000000_ooooo800000080:VYZ@H000003`2Y
ZJT?000000cooooo800000080:VYZ@H000003`2YZJT?000000cooooo800000080:VYZ@@000004@2Y
ZJT?000000gooooo000<ooooob000000202YZJT6000000l0ZJVY3`00000<ooooob000000202YZJT6
000000l0ZJVY3`00000;ooooob000000202YZJT6000000l0ZJVY3`00000<ooooob000000202YZJT6
000000l0ZJVY3`00000<ooooob000000202YZJT400000140ZJVY3`00000=ooooo`003?oooolP0000
00P0ZJVY1P00000?0:VYZ@l000003?oooolP000000P0ZJVY1P00000?0:VYZ@l000002ooooolP0000
00P0ZJVY1P00000?0:VYZ@l000003?oooolP000000P0ZJVY1P00000?0:VYZ@l000003?oooolP0000
00P0ZJVY1000000A0:VYZ@l000003Oooool000cooooo4@00000<0:VYZ@<00000202YZJT6000000l0
ZJVY3`00000<ooooo`l000003`2YZJT2000000P0ZJVY1P00000?0:VYZ@l000002oooool;000001<0
ZJVY0P0000080:VYZ@H000003`2YZJT?000000cooooo2000000F0:VYZ@800000202YZJT6000000l0
ZJVY3`00000<ooooo`@000006P2YZJT2000000P0ZJVY1P00000?0:VYZ@l000003Oooool000cooooo
4@00000<0:VYZ@<00000202YZJT6000000l0ZJVY3`00000<ooooo`l000003`2YZJT2000000P0ZJVY
1P00000?0:VYZ@l000002oooool;000001<0ZJVY0P0000080:VYZ@H000003`2YZJT?000000cooooo
2000000F0:VYZ@800000202YZJT6000000l0ZJVY3`00000<ooooo`@000006P2YZJT2000000P0ZJVY
1P00000?0:VYZ@l000003Oooool000cooooo100000090:VYZ@@00000302YZJT3000000P0ZJVY1P00
000=0:VYZA4000003?ooool4000001X0ZJVY0P0000080:VYZ@H000003`2YZJT?000000_ooooo1000
000J0:VYZ@800000202YZJT6000000l0ZJVY3`00000<ooooo`@000006P2YZJT2000000P0ZJVY1P00
000?0:VYZ@l000003?ooool4000001X0ZJVY0P0000080:VYZ@H000003`2YZJT?000000gooooo000<
ooooo`@000002@2YZJT4000000`0ZJVY0`0000080:VYZ@H000003@2YZJTA000000cooooo1000000J
0:VYZ@800000202YZJT6000000l0ZJVY3`00000;ooooo`@000006P2YZJT2000000P0ZJVY1P00000?
0:VYZ@l000003?ooool4000001X0ZJVY0P0000080:VYZ@H000003`2YZJT?000000cooooo1000000J
0:VYZ@800000202YZJT6000000l0ZJVY3`00000=ooooo`003?ooool4000000T0ZJVY0P00000?0:VY
ZBh000003?ooool4000001X0ZJVY;P00000;ooooo`@000006P2YZJT^000000cooooo1000000J0:VY
ZBh000003?ooool4000001X0ZJVY;P00000=ooooo`003?ooool4000000T0ZJVY0P00000?0:VYZBh0
00003?ooool4000001X0ZJVY;P00000;ooooo`@000006P2YZJT^000000cooooo1000000J0:VYZBh0
00003?ooool4000001X0ZJVY;P00000=ooooo`003?ooool4000001X0ZJVY;P00000<ooooo`@00000
6P2YZJT^000000_ooooo1000000J0:VYZBh000003?ooool4000001X0ZJVY;P00000<ooooo`@00000
6P2YZJT^000000gooooo000<ooooo`@000006P2YZJT^000000cooooo1000000J0:VYZBh000002ooo
ool4000001X0ZJVY;P00000<ooooo`@000006P2YZJT^000000cooooo1000000J0:VYZBh000003Ooo
ool000cooooo1000000J0:VYZBh000003?ooool4000001X0ZJVY;P00000;ooooo`@000006P2YZJT^
000000cooooo1000000J0:VYZBh000003?ooool4000001X0ZJVY;P00000=ooooo`003?ooool40000
01X0ZJVY9P0000080:VYZ@cooooo1000000J0:VYZBH00000202YZJT;ooooo`@000006P2YZJTV0000
00P0ZJVY3?ooool4000001X0ZJVY9P0000080:VYZ@cooooo1000000J0:VYZBH00000202YZJT=oooo
o`003?ooool4000001X0ZJVY9P0000080:VYZ@cooooo1000000J0:VYZBH00000202YZJT;ooooo`@0
00006P2YZJTV000000P0ZJVY3?ooool4000001X0ZJVY9P0000080:VYZ@cooooo1000000J0:VYZBH0
0000202YZJT=ooooo`003?ooool8000001H0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY
3?ooool4000001X0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY2oooool4000001X0ZJVY
6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool4000001X0ZJVY6`00000:0:VYZ@030000
002YZJT0ZJVY00H0ZJVY3?ooool4000001X0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY
3Oooool000cooooo2000000F0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo1000
000J0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@_ooooo1000000J0:VYZA/000002P2Y
ZJT00`000000ZJVY0:VYZ@060:VYZ@cooooo1000000J0:VYZA/000002P2YZJT00`000000ZJVY0:VY
Z@060:VYZ@cooooo1000000J0:VYZA/000002P2YZJT00`000000ZJVY0:VYZ@060:VYZ@gooooo000<
ooooo`X00000502YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooo`P000005P2YZJTK
000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT;ooooo`@000006P2YZJTK000000X0ZJVY00<00000
0:VYZ@2YZJT01P2YZJT<ooooo`@000006P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<
ooooo`@000006P2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT=ooooo`003?ooool:0000
01@0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool8000001H0ZJVY6`00000:0:VY
Z@030000002YZJT0ZJVY00H0ZJVY2oooool4000001X0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY
00H0ZJVY3?ooool4000001X0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool40000
01X0ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo2`00000C0:VYZA/0
00002P2YZJT9000000cooooo2P00000D0:VYZA/000002P2YZJT9000000_ooooo2000000F0:VYZA/0
00002P2YZJT9000000cooooo1000000J0:VYZA/000002P2YZJT9000000cooooo0P00000L0:VYZA/0
00002P2YZJT9000000gooooo000<ooooo`/000004`2YZJTK000000X0ZJVY2@00000<ooooo`X00000
502YZJTK000000X0ZJVY2@00000;ooooo`P000005P2YZJTK000000X0ZJVY2@00000<ooooo`@00000
6P2YZJTK000000X0ZJVY2@00000<ooooo`800000702YZJTK000000X0ZJVY2@00000=ooooo`003?oo
ool;000001<0ZJVY0P00000A0:VYZ@P000002P2YZJT9000000cooooo2P00000D0:VYZ@8000004@2Y
ZJT8000000X0ZJVY2@00000;ooooo`P000005P2YZJT200000140ZJVY2000000:0:VYZ@T000003?oo
ool4000001X0ZJVY0P00000A0:VYZ@P000002P2YZJT9000000cooooo7P2YZJT200000140ZJVY2000
000:0:VYZ@T000003Oooool000cooooo2`00000C0:VYZ@8000004@2YZJT8000000X0ZJVY2@00000<
ooooo`X00000502YZJT200000140ZJVY2000000:0:VYZ@T000002oooool8000001H0ZJVY0P00000A
0:VYZ@P000002P2YZJT9000000cooooo1000000J0:VYZ@8000004@2YZJT8000000X0ZJVY2@00000<
oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT9000000gooooo000<ooooo`P0ZJVY0`00000C0:VY
Z@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@cooooo2P2YZJT00`000000ZJVY0:VYZ@0A0:VY
Z@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@_ooooo7P2YZJT200000140ZJVY2000000:0:VY
Z@L000000P2YZJT<oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT700000080ZJVY3?oooolN0:VY
Z@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@gooooo000<ooooo`P0ZJVY0`00000C0:VYZ@80
00004@2YZJT8000000X0ZJVY1`0000020:VYZ@cooooo2P2YZJT00`000000ZJVY0:VYZ@0A0:VYZ@80
00004@2YZJT8000000X0ZJVY1`0000020:VYZ@_ooooo7P2YZJT200000140ZJVY2000000:0:VYZ@L0
00000P2YZJT<oooooah0ZJVY0P00000A0:VYZ@P000002P2YZJT700000080ZJVY3?oooolN0:VYZ@80
00004@2YZJT8000000X0ZJVY1`0000020:VYZ@gooooo000<ooooo`X0ZJVY00<000000:VYZ@2YZJT0
4@2YZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@_ooooo
7P2YZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@cooooo
7P2YZJT200000140ZJVY6@0000020:VYZ@gooooo000<ooooo`X0ZJVY00<000000:VYZ@2YZJT04@2Y
ZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@_ooooo7P2Y
ZJT200000140ZJVY6@0000020:VYZ@cooooo7P2YZJT200000140ZJVY6@0000020:VYZ@cooooo7P2Y
ZJT200000140ZJVY6@0000020:VYZ@gooooo000<ooooo`d0ZJVY4`00000A0:VYZAT000000P2YZJT<
oooooa40ZJVY3`00000A0:VYZAT000000P2YZJT;oooooaD0ZJVY2`00000A0:VYZAT000000P2YZJT<
oooooaT0ZJVY1`00000A0:VYZAT000000P2YZJT<oooooad0ZJVY0`00000A0:VYZAT000000P2YZJT=
ooooo`003?ooool=0:VYZA<000004@2YZJTI00000080ZJVY3?oooolA0:VYZ@l000004@2YZJTI0000
0080ZJVY2ooooolE0:VYZ@/000004@2YZJTI00000080ZJVY3?oooolI0:VYZ@L000004@2YZJTI0000
0080ZJVY3?oooolM0:VYZ@<000004@2YZJTI00000080ZJVY3Oooool000cooooo5`2YZJT900000140
ZJVY6@0000020:VYZ@cooooo5`2YZJT900000140ZJVY6@0000020:VYZ@_ooooo5`2YZJT900000140
ZJVY6@0000020:VYZ@cooooo5`2YZJT900000140ZJVY6@0000020:VYZ@cooooo6@2YZJT700000140
ZJVY6@0000020:VYZ@gooooo000<oooooaL0ZJVY2@00000A0:VYZA/000003?oooolG0:VYZ@T00000
4@2YZJTI00000080ZJVY2ooooolG0:VYZ@T000004@2YZJTI00000080ZJVY3?oooolG0:VYZ@T00000
4@2YZJTI00000080ZJVY3?oooolI0:VYZ@L000004@2YZJTI00000080ZJVY3Oooool000cooooo5`2Y
ZJT900000140ZJVY6`00000<oooooaL0ZJVY2@00000A0:VYZAT000000P2YZJT;oooooaL0ZJVY2@00
000A0:VYZAT000000P2YZJT<oooooaL0ZJVY2@00000A0:VYZAT000000P2YZJT<oooooaT0ZJVY1`00
000A0:VYZAT000000P2YZJT=ooooo`003?oooolG0:VYZCD000003?oooolG0:VYZC<000000P2YZJT;
oooooaL0ZJVY<`0000020:VYZ@cooooo5`2YZJTc00000080ZJVY3?oooolG0:VYZC<000000P2YZJT=
ooooo`003?oooolG0:VYZCD000003?oooolG0:VYZC<000000P2YZJT;oooooaL0ZJVY<`0000020:VY
Z@cooooo5`2YZJTc00000080ZJVY3?oooolG0:VYZC<000000P2YZJT=ooooo`003?ooool2000001D0
ZJVY=@00000<oooooaL0ZJVY=@00000;oooooaL0ZJVY<`0000020:VYZ@cooooo5`2YZJTc00000080
ZJVY3?oooolG0:VYZC<000000P2YZJT=ooooo`003?ooool2000001D0ZJVY=@00000<oooooaL0ZJVY
=@00000;oooooaL0ZJVY<`0000020:VYZ@cooooo5`2YZJTc00000080ZJVY3?oooolG0:VYZC<00000
0P2YZJT=ooooo`003?ooool2000001D0ZJVY=@00000<oooooaL0ZJVY=@00000;oooooaL0ZJVY<`00
00020:VYZ@cooooo5`2YZJTc00000080ZJVY3?oooolG0:VYZC<000000P2YZJT=ooooo`003?ooool2
000001D0ZJVY=@00000<oooooaL0ZJVY=@00000;oooooaL0ZJVY<`0000020:VYZ@cooooo5`2YZJTc
00000080ZJVY3?oooolG0:VYZC<000000P2YZJT=ooooo`003?ooool2000001D0ZJVY1P00000A0:VY
ZAh000003?ooool2000001D0ZJVY1P00000C0:VYZA`000002ooooolG0:VYZ@H000005P2YZJTI0000
00cooooo5`2YZJT6000001X0ZJVY4`0000020:VYZ@cooooo5`2YZJT4000001h0ZJVY4@0000020:VY
Z@gooooo000<ooooo`8000005@2YZJT600000140ZJVY7P00000<ooooo`8000005@2YZJT6000001<0
ZJVY7000000;oooooaL0ZJVY1P00000F0:VYZAT000003?oooolG0:VYZ@H000006P2YZJTC00000080
ZJVY3?oooolG0:VYZ@@000007P2YZJTA00000080ZJVY3Oooool00?ooooooaOooool00?ooooooaOoo
ool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOoo
ool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOoo
ool000cooooo0P0000020:VYZ@@000000`2YZJT400000080ZJVY300000090:VYZ@8000000P2YZJT2
00000080ZJVY0P0000000`2YZJT000000000000200000080ZJVY1P0000040:VYZ@<000001P2YZJT2
000000cooooo200000050:VYZA0000003@2YZJTR000000_ooooo200000070:VYZ@h000003@2YZJTR
000000cooooo2000000;0:VYZ@X000003`2YZJTP000000cooooo2000000?0:VYZ@H000004@2YZJTN
000000gooooo000<ooooo`8000000P2YZJT4000000<0ZJVY100000020:VYZ@`000002@2YZJT20000
0080ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P0000020:VYZ@H00000102YZJT30000
00H0ZJVY0P00000<ooooo`P000001@2YZJT@000000d0ZJVY8P00000;ooooo`P000001`2YZJT>0000
00d0ZJVY8P00000<ooooo`P000002`2YZJT:000000l0ZJVY8000000<ooooo`P000003`2YZJT60000
0140ZJVY7P00000=ooooo`003?ooool40:VYZ@@000000`2YZJT400000080ZJVY100000060:VYZ@80
00001`2YZJT2000000X0ZJVY2@0000020:VYZ@800000102YZJT2000000030:VYZ@00000000000080
ZJVY0P0000020:VYZ@cooooo200000050:VYZA0000003@2YZJTR000000_ooooo200000070:VYZ@h0
00003@2YZJTR000000cooooo200000070:VYZ@h000003`2YZJTP000000cooooo2000000;0:VYZ@X0
00004`2YZJTL000000gooooo000<ooooo`@0ZJVY100000030:VYZ@@000000P2YZJT4000000H0ZJVY
0P0000070:VYZ@8000002P2YZJT900000080ZJVY0P0000040:VYZ@80000000<0ZJVY000000000000
0P2YZJT200000080ZJVY3?ooool8000000D0ZJVY4000000=0:VYZB8000002oooool8000000L0ZJVY
3P00000=0:VYZB8000003?ooool8000000L0ZJVY3P00000?0:VYZB0000003?ooool8000000/0ZJVY
2P00000C0:VYZA`000003Oooool000cooooo0P0000040:VYZ@8000002@2YZJT2000000@0ZJVY0P00
00020:VYZ@T000001P2YZJT6000000D0ZJVY0P0000040:VYZ@@000000P2YZJT700000080ZJVY0P00
000<ooooo`P000001`2YZJTE000000H0ZJVY0P00000=0:VYZA<000002oooool8000000L0ZJVY4@00
000I0:VYZA<000003?ooool8000000T0ZJVY3000000L0:VYZA<000003?ooool8000000/0ZJVY2P00
000L0:VYZA<000003Oooool000cooooo0P0000040:VYZ@8000002@2YZJT2000000@0ZJVY0P000002
0:VYZ@T000001P2YZJT6000000D0ZJVY0P0000040:VYZ@@000000P2YZJT700000080ZJVY0P00000<
ooooo`P000001`2YZJTE000000H0ZJVY0P00000=0:VYZA<000002oooool8000000L0ZJVY4@00000I
0:VYZA<000003?ooool8000000T0ZJVY3000000L0:VYZA<000003?ooool8000000/0ZJVY2P00000L
0:VYZA<000003Oooool000cooooo0P2YZJT200000080ZJVY100000030:VYZ@8000000P2YZJT20000
0080ZJVY0P0000020:VYZ@8000000P2YZJT900000080ZJVY100000050:VYZ@800000102YZJT20000
0080ZJVY1P0000030:VYZ@8000000P2YZJT4000000cooooo200000070:VYZA/000003`2YZJT80000
00D0ZJVY1P00000;ooooo`P000001`2YZJTK000000l0ZJVY200000070:VYZ@@000003?ooool80000
00/0ZJVY5@00000A0:VYZ@P000001`2YZJT4000000cooooo2000000;0:VYZA4000005@2YZJT80000
00L0ZJVY1000000=ooooo`003?ooool20:VYZ@8000000P2YZJT4000000<0ZJVY0P0000020:VYZ@80
00000P2YZJT200000080ZJVY0P0000020:VYZ@T000000P2YZJT4000000D0ZJVY0P0000040:VYZ@80
00000P2YZJT6000000<0ZJVY0P0000020:VYZ@@000003?ooool8000000L0ZJVY6`00000?0:VYZ@P0
00001@2YZJT6000000_ooooo200000070:VYZA/000003`2YZJT8000000L0ZJVY1000000<ooooo`P0
00002`2YZJTE00000140ZJVY200000070:VYZ@@000003?ooool8000000/0ZJVY4@00000E0:VYZ@P0
00001`2YZJT4000000gooooo000<ooooo`80ZJVY0P0000020:VYZ@@000000`2YZJT4000000X0ZJVY
0P0000000`2YZJT00000000000040:VYZ@8000005@2YZJT200000080ZJVY0P0000030:VYZ@800000
102YZJT2000000cooooo200000070:VYZ@X000001@2YZJT<000000l0ZJVY200000070:VYZ@@00000
2oooool8000000/0ZJVY1P0000050:VYZ@`000003`2YZJT8000000L0ZJVY1000000<ooooo`P00000
2`2YZJT6000000D0ZJVY2P00000A0:VYZ@P000001`2YZJT4000000cooooo2000000;0:VYZ@H00000
1@2YZJT8000001<0ZJVY200000070:VYZ@@000003Oooool000cooooo0P2YZJT4000000@0ZJVY0`00
00060:VYZ@8000000P2YZJT2000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2000000@0ZJVY0P00
00020:VYZ@<000000P2YZJT<000000L0ZJVY1000000<ooooo`d000001P2YZJT6000000D0ZJVY3000
00070:VYZA0000001`2YZJT4000000_ooooo2P0000090:VYZ@H000001@2YZJT<000000/0ZJVY3000
00070:VYZ@@000003?ooool8000000/0ZJVY1P0000050:VYZ@`000003`2YZJT8000000L0ZJVY1000
000<ooooo`P000002`2YZJT6000000D0ZJVY2P00000A0:VYZ@P000001`2YZJT4000000gooooo000<
ooooo`80ZJVY100000040:VYZ@<000001P2YZJT200000080ZJVY0P0000050:VYZ@8000000P2YZJT2
00000080ZJVY0P0000040:VYZ@8000000P2YZJT300000080ZJVY300000070:VYZ@@000003?ooool=
000000H0ZJVY1P0000050:VYZ@`000001`2YZJT@000000L0ZJVY1000000;ooooo`X000002@2YZJT6
000000D0ZJVY3000000;0:VYZ@`000001`2YZJT4000000cooooo2000000;0:VYZ@H000001@2YZJT<
000000l0ZJVY200000070:VYZ@@000003?ooool8000000/0ZJVY1P0000050:VYZ@X000004@2YZJT8
000000L0ZJVY1000000=ooooo`003?ooool40:VYZ@L000000P2YZJT2000000@0ZJVY200000050:VY
Z@800000102YZJT6000000@0ZJVY00<000000:VYZ@2YZJT00P0000020:VYZ@8000000P2YZJT40000
0080ZJVY0P0000050:VYZ@8000000P2YZJT<ooooo`d000001P2YZJT6000000D0ZJVY100000060:VY
ZAT000001`2YZJT4000000_ooooo3@0000060:VYZ@H000001@2YZJT4000000H0ZJVY6@0000070:VY
Z@@000003?ooool:000000T0ZJVY1P0000050:VYZ@@00000202YZJTG000000L0ZJVY1000000<oooo
o`P000002`2YZJT6000000D0ZJVY1000000:0:VYZAD000001`2YZJT4000000gooooo000<ooooo`@0
ZJVY1`0000020:VYZ@800000102YZJT8000000D0ZJVY0P0000040:VYZ@H00000102YZJT00`000000
ZJVY0:VYZ@0200000080ZJVY0P0000020:VYZ@@000000P2YZJT2000000D0ZJVY0P0000020:VYZ@co
oooo3@0000060:VYZ@H000001@2YZJT4000000H0ZJVY6@0000070:VYZ@@000002oooool=000000H0
ZJVY1P0000050:VYZ@@000001P2YZJTI000000L0ZJVY1000000<ooooo`X000002@2YZJT6000000D0
ZJVY100000080:VYZAL000001`2YZJT4000000cooooo2000000;0:VYZ@H000001@2YZJT4000000X0
ZJVY5@0000070:VYZ@@000003Oooool000cooooo100000020:VYZ@8000002`2YZJT800000080ZJVY
1@0000040:VYZ@/000000P2YZJT200000080ZJVY1P0000020:VYZ@8000000P2YZJT5000000@0ZJVY
3?ooool=000000H0ZJVY3`0000060:VYZB@000002oooool=000000H0ZJVY3`0000060:VYZB@00000
3?ooool;000000P0ZJVY3`00000:0:VYZB0000003?ooool:000000T0ZJVY3`00000>0:VYZA`00000
3Oooool000cooooo100000020:VYZ@8000002`2YZJT800000080ZJVY1@0000040:VYZ@/000000P2Y
ZJT200000080ZJVY1P0000020:VYZ@8000000P2YZJT5000000@0ZJVY3?ooool=000000H0ZJVY3`00
00060:VYZB@000002oooool=000000H0ZJVY3`0000060:VYZB@000003?ooool;000000P0ZJVY3`00
000:0:VYZB0000003?ooool:000000T0ZJVY3`00000>0:VYZA`000003Oooool000cooooo2`2YZJT@
000000<0ZJVY100000060:VYZ@@000000P2YZJT5000000P0ZJVY2`0000040:VYZ@8000003?oooolR
000000H0ZJVY6`0000070:VYZ@8000002ooooolR000000X0ZJVY5@0000090:VYZ@8000003?oooolR
000000h0ZJVY3`00000;0:VYZ@8000003?oooolR000000h0ZJVY3`00000;0:VYZ@8000003Oooool0
00cooooo2`2YZJT@000000<0ZJVY100000060:VYZ@@000000P2YZJT5000000P0ZJVY2`0000040:VY
Z@8000003?oooolR000000H0ZJVY6`0000070:VYZ@8000002ooooolR000000X0ZJVY5@0000090:VY
Z@8000003?oooolR000000h0ZJVY3`00000;0:VYZ@8000003?oooolR000000h0ZJVY3`00000;0:VY
Z@8000003Oooool000cooooo100000020:VYZ@D000001P2YZJT4000000@0ZJVY0P0000020:VYZ@<0
0000102YZJT600000080ZJVY0P0000030:VYZ@X000000P2YZJT4000000L0ZJVY0P0000020:VYZ@co
oooo5`0000060:VYZ@T000002P2YZJTA000000T0ZJVY0P00000;oooooaL000001`2YZJT4000000h0
ZJVY4@0000090:VYZ@8000003?oooolG000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000co
oooo5`0000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00000=ooooo`003?ooool400000080ZJVY
1@0000060:VYZ@@00000102YZJT200000080ZJVY0`0000040:VYZ@H000000P2YZJT2000000<0ZJVY
2P0000020:VYZ@@000001`2YZJT200000080ZJVY3?oooolG000000H0ZJVY2@00000:0:VYZA400000
2@2YZJT2000000_ooooo5`0000070:VYZ@@000003P2YZJTA000000T0ZJVY0P00000<oooooaL00000
1`2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003?oooolG000000L0ZJVY1000000>0:VYZ@l00000
2`2YZJT2000000gooooo000<ooooo`@0ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P2Y
ZJT200000080ZJVY1P0000070:VYZ@8000000P2YZJT2000000X0ZJVY00<000000:VYZ@2YZJT01P00
00080:VYZ@8000001`2YZJT2000000cooooo1P0000050:VYZ@`000001`2YZJT4000000h0ZJVY4@00
00090:VYZ@8000002oooool4000000L0ZJVY300000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00
000<ooooo`@000002@2YZJT:000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000
000;0:VYZ@P000001`2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003Oooool000cooooo102YZJT2
00000080ZJVY0P0000000`2YZJT00000000000020:VYZ@8000000P2YZJT6000000L0ZJVY0P000002
0:VYZ@8000002P2YZJT00`000000ZJVY0:VYZ@06000000P0ZJVY0P0000070:VYZ@8000003?ooool6
000000D0ZJVY300000070:VYZ@@000003P2YZJTA000000T0ZJVY0P00000;ooooo`@000001`2YZJT<
000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo100000090:VYZ@X000001`2YZJT4
000000h0ZJVY3`00000;0:VYZ@8000003?ooool4000000/0ZJVY200000070:VYZ@@000003P2YZJT?
000000/0ZJVY0P00000=ooooo`003?ooool4000000H0ZJVY0`0000020:VYZ@8000000P2YZJT40000
00H0ZJVY1@00000>0:VYZ@L000000P2YZJT:000000<0ZJVY0P0000020:VYZ@8000003?ooool40000
00L0ZJVY300000070:VYZ@@000003P2YZJTA000000T0ZJVY0P00000;ooooo`@000002@2YZJT:0000
00L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000;0:VYZ@P000001`2YZJT40000
00h0ZJVY3`00000;0:VYZ@8000003?ooool4000000l0ZJVY100000070:VYZ@@000003P2YZJT?0000
00/0ZJVY0P00000=ooooo`003?ooool4000000H0ZJVY0`0000020:VYZ@8000000P2YZJT4000000H0
ZJVY1@00000>0:VYZ@L000000P2YZJT:000000<0ZJVY0P0000020:VYZ@8000003?ooool4000000L0
ZJVY300000070:VYZ@@000003P2YZJTA000000T0ZJVY0P00000;ooooo`@000002@2YZJT:000000L0
ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000;0:VYZ@P000001`2YZJT4000000h0
ZJVY3`00000;0:VYZ@8000003?ooool4000000l0ZJVY100000070:VYZ@@000003P2YZJT?000000/0
ZJVY0P00000=ooooo`003?ooool6000000D0ZJVY100000020:VYZ@8000000P2YZJT2000000H0ZJVY
1@0000080:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@020:VYZ@@00000102YZJT2000000D0ZJVY
1P0000020:VYZ@cooooo100000070:VYZ@`000001`2YZJT4000000h0ZJVY3`0000070:VYZ@H00000
2oooool4000000/0ZJVY200000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00000<ooooo`@00000
3`2YZJT4000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000cooooo1000000A0:VYZ@800000
1`2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003Oooool000cooooo1P0000050:VYZ@@000000P2Y
ZJT200000080ZJVY0P0000060:VYZ@D00000202YZJT400000080ZJVY00<000000:VYZ@2YZJT00P2Y
ZJT4000000@0ZJVY0P0000050:VYZ@H000000P2YZJT<ooooo`@000001`2YZJT<000000L0ZJVY1000
000>0:VYZ@l000001`2YZJT6000000_ooooo1000000;0:VYZ@P000001`2YZJT4000000h0ZJVY3`00
000;0:VYZ@8000003?ooool4000000l0ZJVY100000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00
000<ooooo`@000004@2YZJT2000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2000000gooooo000<
ooooo`80ZJVY1000000;0:VYZ@@000000P2YZJT200000080ZJVY2`0000020:VYZ@@000002@2YZJT2
00000080ZJVY200000050:VYZ@8000000P2YZJT2000000cooooo1000000=0:VYZ@H000001P2YZJT5
000000P0ZJVY5@0000070:VYZ@H000002oooool4000000l0ZJVY100000070:VYZ@@00000302YZJTA
000000T0ZJVY1000000<ooooo`@000004@2YZJT2000000L0ZJVY1000000>0:VYZ@l000002`2YZJT2
000000cooooo1000000J0:VYZ@@000003P2YZJT?000000/0ZJVY0P00000=ooooo`003?ooool20:VY
Z@@000002`2YZJT400000080ZJVY0P0000020:VYZ@/000000P2YZJT4000000T0ZJVY0P0000020:VY
Z@P000001@2YZJT200000080ZJVY0P00000<ooooo`@000003@2YZJT6000000H0ZJVY1@0000080:VY
ZAD000001`2YZJT6000000_ooooo1000000?0:VYZ@@000001`2YZJT4000000`0ZJVY4@0000090:VY
Z@@000003?ooool400000140ZJVY0P0000070:VYZ@@000003P2YZJT?000000/0ZJVY0P00000<oooo
o`@000006P2YZJT4000000h0ZJVY3`00000;0:VYZ@8000003Oooool000cooooo100000040:VYZ@<0
0000102YZJT6000000@0ZJVY100000070:VYZ@8000000P2YZJT400000080ZJVY2`0000060:VYZ@80
00001`2YZJT200000080ZJVY3?ooool6000000/0ZJVY900000080:VYZ@8000001`2YZJT6000000_o
oooo1000000?0:VYZB0000002P2YZJT2000000L0ZJVY1P00000<ooooo`@000004`2YZJTJ000000`0
ZJVY0P0000090:VYZ@@000003?ooool4000001L0ZJVY5@00000=0:VYZ@8000002`2YZJT2000000go
oooo000<ooooo`H000000P2YZJT2000000030:VYZ@000000000000X0ZJVY0P0000020:VYZ@<00000
0P2YZJT200000080ZJVY100000060:VYZ@80000000<0ZJVY0000000000000P0000080:VYZ@@00000
0P2YZJT300000080ZJVY0P0000020:VYZ@cooooo2P00000?0:VYZAX000002P2YZJT4000000D0ZJVY
1P00000;ooooo`H000005`2YZJTF000000X0ZJVY0P0000070:VYZ@H000003?ooool4000001X0ZJVY
4`00000<0:VYZ@8000002@2YZJT4000000cooooo1000000J0:VYZA8000003@2YZJT2000000/0ZJVY
0P00000=ooooo`003?ooool600000080ZJVY0P0000000`2YZJT000000000000:0:VYZ@8000000P2Y
ZJT300000080ZJVY0P0000020:VYZ@@000001P2YZJT2000000030:VYZ@000000000000800000202Y
ZJT400000080ZJVY0`0000020:VYZ@8000000P2YZJT<ooooo`X000003`2YZJTJ000000X0ZJVY1000
00050:VYZ@H000002oooool6000001L0ZJVY5P00000:0:VYZ@8000001`2YZJT6000000cooooo1000
000J0:VYZA<00000302YZJT2000000T0ZJVY1000000<ooooo`@000006P2YZJTB000000d0ZJVY0P00
000;0:VYZ@8000003Oooool000cooooo0P0000020:VYZ@@000000P2YZJT00`000000ZJVY0:VYZ@06
000000@0ZJVY0P0000020:VYZ@80000000<0ZJVY0000000000000P2YZJT200000080ZJVY2P00000;
0:VYZ@@000002@2YZJT4000000cooooo4@00000=0:VYZAD000002P2YZJT?000000_ooooo3`00000?
0:VYZA<00000302YZJT?000000cooooo2`00000C0:VYZA8000003@2YZJT?000000cooooo2000000F
0:VYZA0000003`2YZJT?000000gooooo000<ooooo`8000000P2YZJT400000080ZJVY00<000000:VY
Z@2YZJT01P0000040:VYZ@8000000P2YZJT2000000030:VYZ@00000000000080ZJVY0P0000020:VY
Z@X000002`2YZJT4000000T0ZJVY1000000<oooooa4000003@2YZJTE000000X0ZJVY3`00000;oooo
o`l000003`2YZJTC000000`0ZJVY3`00000<ooooo`/000004`2YZJTB000000d0ZJVY3`00000<oooo
o`P000005P2YZJT@000000l0ZJVY3`00000=ooooo`003?ooool2000000@0ZJVY0P0000020:VYZ@<0
0000402YZJT00`000000ZJVY0:VYZ@06000000@0ZJVY0P0000020:VYZ@8000000`2YZJT2000000@0
ZJVY0P0000040:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@0400000080ZJVY3?oooolA000000d0
ZJVY5@00000:0:VYZ@l000002oooool?000000l0ZJVY4P00000=0:VYZ@l000003?ooool=00000140
ZJVY4000000?0:VYZ@l000003?ooool;000001<0ZJVY4000000?0:VYZ@l000003Oooool000cooooo
0P0000040:VYZ@8000000P2YZJT300000100ZJVY00<000000:VYZ@2YZJT01P0000040:VYZ@800000
0P2YZJT2000000<0ZJVY0P0000040:VYZ@800000102YZJT200000080ZJVY00<000000:VYZ@2YZJT0
100000020:VYZ@cooooo4@00000=0:VYZAD000002P2YZJT?000000_ooooo3`00000?0:VYZA800000
3@2YZJT?000000cooooo3@00000A0:VYZA0000003`2YZJT?000000cooooo2`00000C0:VYZA000000
3`2YZJT?000000gooooo000<ooooo`80ZJVY0P0000020:VYZ@@000000`2YZJT2000000@0ZJVY0P00
00090:VYZ@P000000P2YZJT2000000@0ZJVY0`0000020:VYZ@8000001P2YZJT4000000@0ZJVY1`00
00020:VYZ@cooooo4@00000=0:VYZA0000003@2YZJTA000000_ooooo4@00000=0:VYZA0000003`2Y
ZJT?000000cooooo3`00000?0:VYZA0000003`2YZJT?000000cooooo3@00000A0:VYZA0000003`2Y
ZJT?000000gooooo000<ooooo`80ZJVY0P0000020:VYZ@@000000`2YZJT2000000@0ZJVY0P000009
0:VYZ@P000000P2YZJT2000000@0ZJVY0`0000020:VYZ@8000001P2YZJT4000000@0ZJVY1`000002
0:VYZ@cooooo4@00000=0:VYZA0000003@2YZJTA000000_ooooo4@00000=0:VYZA0000003`2YZJT?
000000cooooo3`00000?0:VYZA0000003`2YZJT?000000cooooo3@00000A0:VYZA0000003`2YZJT?
000000gooooo000<ooooo`/000000P2YZJT400000080ZJVY2`0000020:VYZ@800000102YZJT40000
0080ZJVY0P0000050:VYZ@X000000P2YZJT700000080ZJVY0P0000020:VYZ@cooooo800000080:VY
Z@H000001`2YZJTG000000_ooooo800000080:VYZ@H000002`2YZJTC000000cooooo800000080:VY
Z@H000003`2YZJT?000000cooooo800000080:VYZ@H000003`2YZJT?000000gooooo000<ooooo`/0
00000P2YZJT400000080ZJVY2`0000020:VYZ@800000102YZJT400000080ZJVY0P0000050:VYZ@X0
00000P2YZJT700000080ZJVY0P0000020:VYZ@cooooo800000080:VYZ@H000001`2YZJTG000000_o
oooo800000080:VYZ@H000002`2YZJTC000000cooooo800000080:VYZ@H000003`2YZJT?000000co
oooo800000080:VYZ@H000003`2YZJT?000000gooooo000<ooooo`8000001P2YZJT2000000030:VY
Z@000000000000800000102YZJT2000000H0ZJVY0P0000000`2YZJT00000000000060:VYZ@800000
0P2YZJT4000000L0ZJVY0P0000040:VYZ@800000102YZJT3000000@0ZJVY0P0000020:VYZ@cooooo
800000080:VYZ@H000001`2YZJTG000000_ooooo800000080:VYZ@H000002@2YZJTE000000cooooo
800000080:VYZ@H000002`2YZJTC000000cooooo800000080:VYZ@H000003`2YZJT?000000gooooo
000<ooooo`8000001P2YZJT2000000030:VYZ@000000000000800000102YZJT2000000H0ZJVY0P00
00000`2YZJT00000000000060:VYZ@8000000P2YZJT4000000L0ZJVY0P0000040:VYZ@800000102Y
ZJT3000000@0ZJVY0P0000020:VYZ@cooooo800000080:VYZ@H000001`2YZJTG000000_ooooo8000
00080:VYZ@H000002@2YZJTE000000cooooo800000080:VYZ@H000002`2YZJTC000000cooooo8000
00080:VYZ@H000003`2YZJT?000000gooooo000<ooooo`80ZJVY100000040:VYZ@030000002YZJT0
ZJVY008000000P2YZJT2000000@0ZJVY100000050:VYZ@800000202YZJT200000080ZJVY0P000005
0:VYZ@8000000P2YZJT200000080ZJVY100000030:VYZ@P000003?oooolC000000H0ZJVY1`000008
0:VYZ@H000001`2YZJTG000000_ooooo4`0000080:VYZ@D00000202YZJT6000000L0ZJVY5`00000<
oooooa<00000202YZJT5000000P0ZJVY1P0000090:VYZAD000003?oooolC000000P0ZJVY1@000008
0:VYZ@H000002`2YZJTC000000gooooo000<ooooo`80ZJVY100000040:VYZ@030000002YZJT0ZJVY
008000000P2YZJT2000000@0ZJVY100000050:VYZ@800000202YZJT200000080ZJVY0P0000050:VY
Z@8000000P2YZJT200000080ZJVY100000030:VYZ@P000003?oooolC000000H0ZJVY1`0000080:VY
Z@H000001`2YZJTG000000_ooooo4`0000080:VYZ@D00000202YZJT6000000L0ZJVY5`00000<oooo
oa<00000202YZJT5000000P0ZJVY1P0000090:VYZAD000003?oooolC000000P0ZJVY1@0000080:VY
Z@H000002`2YZJTC000000gooooo000<ooooo`H0ZJVY100000050:VYZ@8000000P2YZJT2000000H0
ZJVY0P0000000`2YZJT00000000000040:VYZ@8000000P2YZJT4000000@0ZJVY00<000000:VYZ@2Y
ZJT00P2YZJT4000000@0ZJVY100000050:VYZ@8000000P2YZJT2000000cooooo200000050:VYZ@H0
0000202YZJT5000000P0ZJVY1P0000070:VYZAL000002oooool4000000T0ZJVY1P0000080:VYZ@D0
0000202YZJT6000000L0ZJVY5`00000<ooooo`@000002@2YZJT6000000P0ZJVY1@0000080:VYZ@H0
00002@2YZJTE000000cooooo100000090:VYZ@H00000202YZJT5000000P0ZJVY1P00000;0:VYZA<0
00003Oooool000cooooo1P2YZJT4000000D0ZJVY0P0000020:VYZ@8000001P2YZJT2000000030:VY
Z@000000000000@0ZJVY0P0000020:VYZ@@00000102YZJT00`000000ZJVY0:VYZ@020:VYZ@@00000
102YZJT4000000D0ZJVY0P0000020:VYZ@8000003?ooool8000000D0ZJVY1P0000080:VYZ@D00000
202YZJT6000000L0ZJVY5`00000;ooooo`@000002@2YZJT6000000P0ZJVY1@0000080:VYZ@H00000
1`2YZJTG000000cooooo100000090:VYZ@H00000202YZJT5000000P0ZJVY1P0000090:VYZAD00000
3?ooool4000000T0ZJVY1P0000080:VYZ@D00000202YZJT6000000/0ZJVY4`00000=ooooo`003?oo
ool400000080ZJVY0P0000030:VYZ@8000000P2YZJT4000000H0ZJVY100000000`2YZJT000000000
00020:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@<000000P2YZJT600000080ZJVY0P00
00020:VYZ@D000001P2YZJT<ooooo`@000002@2YZJT6000000P0ZJVY<@00000;ooooo`@000002@2Y
ZJT6000000P0ZJVY<@00000<ooooo`@000002@2YZJT6000000P0ZJVY<@00000<ooooo`@000002@2Y
ZJT4000000`0ZJVY;`00000=ooooo`003?ooool400000080ZJVY0P0000030:VYZ@8000000P2YZJT4
000000H0ZJVY100000000`2YZJT00000000000020:VYZ@8000000P2YZJT200000080ZJVY10000002
0:VYZ@<000000P2YZJT600000080ZJVY0P0000020:VYZ@D000001P2YZJT<ooooo`@000002@2YZJT6
000000P0ZJVY<@00000;ooooo`@000002@2YZJT6000000P0ZJVY<@00000<ooooo`@000002@2YZJT6
000000P0ZJVY<@00000<ooooo`@000002@2YZJT4000000`0ZJVY;`00000=ooooo`003?ooool20000
00/0ZJVY1P00000;0:VYZ@H000000P2YZJT4000000@0ZJVY0`0000020:VYZ@800000102YZJT60000
0080ZJVY0P0000000`2YZJT00000000000020:VYZ@8000000P2YZJT<ooooo`@000002@2YZJT60000
00P0ZJVY<@00000;ooooo`@000002@2YZJT6000000P0ZJVY<@00000<ooooo`@000002@2YZJT60000
00P0ZJVY<@00000<ooooo`@000002@2YZJT4000000`0ZJVY;`00000=ooooo`003?ooool40:VYZ@80
00001@2YZJT200000080ZJVY1P0000040:VYZ@D000001P2YZJT:000000<0ZJVY100000040:VYZ@80
0000202YZJT500000080ZJVY0P00000<ooooo`@000002@2YZJT6000000P0ZJVY<@00000;ooooo`@0
00002@2YZJT6000000P0ZJVY<@00000<ooooo`@000002@2YZJT4000000`0ZJVY;`00000<ooooo`@0
00002@2YZJT2000000l0ZJVY;P00000=ooooo`003?ooool40:VYZ@8000001@2YZJT200000080ZJVY
1P0000040:VYZ@D000001P2YZJT:000000<0ZJVY100000040:VYZ@800000202YZJT500000080ZJVY
0P00000<ooooo`@000002@2YZJT6000000P0ZJVY<@00000;ooooo`@000002@2YZJT6000000P0ZJVY
<@00000<ooooo`@000002@2YZJT4000000`0ZJVY;`00000<ooooo`@000002@2YZJT2000000l0ZJVY
;P00000=ooooo`003?ooool400000080ZJVY0P0000020:VYZ@<000000P2YZJT600000080ZJVY0P00
00050:VYZ@H000001P2YZJT2000000D0ZJVY0P0000020:VYZ@8000000P2YZJT400000080ZJVY0P00
00020:VYZ@030000002YZJT0ZJVY00800000102YZJT<ooooo`@000001`2YZJT:000000H0ZJVY:@00
00080:VYZ@_ooooo100000090:VYZ@H00000202YZJTY000000P0ZJVY3?ooool4000000T0ZJVY1000
000<0:VYZBL00000202YZJT<ooooo`@000006P2YZJTV000000P0ZJVY3Oooool000cooooo10000002
0:VYZ@8000000P2YZJT300000080ZJVY1P0000020:VYZ@8000001@2YZJT6000000H0ZJVY0P000005
0:VYZ@8000000P2YZJT200000080ZJVY100000020:VYZ@8000000P2YZJT00`000000ZJVY0:VYZ@02
000000@0ZJVY3?ooool4000000L0ZJVY2P0000060:VYZBT00000202YZJT;ooooo`@000002@2YZJT6
000000P0ZJVY:@0000080:VYZ@cooooo100000090:VYZ@@00000302YZJTW000000P0ZJVY3?ooool4
000001X0ZJVY9P0000080:VYZ@gooooo000<ooooo`@000001P2YZJT300000080ZJVY0P0000080:VY
Z@@0000000<0ZJVY0000000000000P2YZJT8000000@0ZJVY0`0000020:VYZ@8000000P2YZJT20000
00@0ZJVY0P0000020:VYZ@8000001`2YZJT2000000cooooo4`0000060:VYZB8000001P2YZJT30000
00P0ZJVY2ooooolA000000`0ZJVY7000000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool=0000
0140ZJVY6`00000:0:VYZ@030000002YZJT0ZJVY00H0ZJVY3?ooool;000001<0ZJVY6`00000:0:VY
Z@030000002YZJT0ZJVY00H0ZJVY3Oooool000cooooo100000060:VYZ@<000000P2YZJT2000000P0
ZJVY100000000`2YZJT00000000000020:VYZ@P00000102YZJT300000080ZJVY0P0000020:VYZ@80
0000102YZJT200000080ZJVY0P0000070:VYZ@8000003?oooolC000000H0ZJVY8P0000060:VYZ@<0
0000202YZJT;oooooa400000302YZJTL000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooo`d0
00004@2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooo`/000004`2YZJTK000000X0
ZJVY00<000000:VYZ@2YZJT01P2YZJT=ooooo`003?ooool20:VYZ@T000001P2YZJT2000000@0ZJVY
0P0000070:VYZ@8000000P2YZJT8000000D0ZJVY0P0000020:VYZ@8000000P2YZJT2000000H0ZJVY
0P0000000`2YZJT00000000000060:VYZ@cooooo2`0000060:VYZ@8000002`2YZJTM000000P0ZJVY
00<000000:VYZ@2YZJT01P2YZJT;ooooo`/000004`2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT0
1P2YZJT<ooooo`/000004`2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT<ooooo`/00000
4`2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT=ooooo`003?ooool20:VYZ@T000001P2Y
ZJT2000000@0ZJVY0P0000070:VYZ@8000000P2YZJT8000000D0ZJVY0P0000020:VYZ@8000000P2Y
ZJT2000000H0ZJVY0P0000000`2YZJT00000000000060:VYZ@cooooo2`0000060:VYZ@8000002`2Y
ZJTM000000P0ZJVY00<000000:VYZ@2YZJT01P2YZJT;ooooo`/000004`2YZJTK000000X0ZJVY00<0
00000:VYZ@2YZJT01P2YZJT<ooooo`/000004`2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2Y
ZJT<ooooo`/000004`2YZJTK000000X0ZJVY00<000000:VYZ@2YZJT01P2YZJT=ooooo`003?ooool2
0:VYZ@8000000P2YZJT700000080ZJVY100000020:VYZ@8000001P2YZJT300000080ZJVY10000006
0:VYZ@T000001P2YZJT2000000@0ZJVY2`00000<ooooo`/000004`2YZJTK000000X0ZJVY2@00000;
ooooo`/000004`2YZJTK000000X0ZJVY2@00000<ooooo`/000004`2YZJTK000000X0ZJVY2@00000<
ooooo`/000004`2YZJTK000000X0ZJVY2@00000=ooooo`003?ooool20:VYZ@8000000P2YZJT70000
0080ZJVY100000020:VYZ@8000001P2YZJT300000080ZJVY100000060:VYZ@T000001P2YZJT20000
00@0ZJVY2`00000<ooooo`/000004`2YZJTK000000X0ZJVY2@00000;ooooo`/000004`2YZJTK0000
00X0ZJVY2@00000<ooooo`/000004`2YZJTK000000X0ZJVY2@00000<ooooo`/000004`2YZJTK0000
00X0ZJVY2@00000=ooooo`003?ooool2000000H0ZJVY0P0000090:VYZ@@000002@2YZJT400000080
ZJVY0P0000040:VYZ@@0000000<0ZJVY0000000000001P00000;0:VYZ@@000000P2YZJT2000000co
oooo2`0000080:VYZ@@000001`2YZJT>000000D0ZJVY2000000:0:VYZ@T000002oooool;000001<0
ZJVY300000070:VYZ@P000002P2YZJT9000000cooooo2`00000C0:VYZ@P000002`2YZJT8000000X0
ZJVY2@00000<ooooo`/000004`2YZJT4000000l0ZJVY2000000:0:VYZ@T000003Oooool000cooooo
0P0000060:VYZ@8000002@2YZJT4000000T0ZJVY100000020:VYZ@800000102YZJT4000000030:VY
Z@000000000000H000002`2YZJT400000080ZJVY0P00000<ooooo`/00000202YZJT4000000L0ZJVY
3P0000050:VYZ@P000002P2YZJT9000000_ooooo2`00000C0:VYZ@`000001`2YZJT8000000X0ZJVY
2@00000<ooooo`/000004`2YZJT8000000/0ZJVY2000000:0:VYZ@T000003?ooool;000001<0ZJVY
1000000?0:VYZ@P000002P2YZJT9000000gooooo000<ooooo`@0ZJVY0P0000020:VYZ@80000000<0
ZJVY0000000000001P2YZJT200000080ZJVY0P0000040:VYZ@D00000102YZJT6000000D0ZJVY1000
00020:VYZ@@00000102YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT200000080ZJVY3?oo
ool60:VYZ@D00000202YZJT4000000L0ZJVY0P0000080:VYZ@@000001@2YZJT8000000X0ZJVY1`00
00020:VYZ@_ooooo1P2YZJT5000001<0ZJVY0P00000A0:VYZ@P000002P2YZJT700000080ZJVY3?oo
ool60:VYZ@D000004`2YZJT200000140ZJVY2000000:0:VYZ@L000000P2YZJT<ooooo`H0ZJVY1@00
000C0:VYZ@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@gooooo000<ooooo`@0ZJVY0P000002
0:VYZ@80000000<0ZJVY0000000000001P2YZJT200000080ZJVY0P0000040:VYZ@D00000102YZJT6
000000D0ZJVY100000020:VYZ@@00000102YZJT200000080ZJVY00<000000:VYZ@2YZJT00P2YZJT2
00000080ZJVY3?ooool60:VYZ@D00000202YZJT4000000L0ZJVY0P0000080:VYZ@@000001@2YZJT8
000000X0ZJVY1`0000020:VYZ@_ooooo1P2YZJT5000001<0ZJVY0P00000A0:VYZ@P000002P2YZJT7
00000080ZJVY3?ooool60:VYZ@D000004`2YZJT200000140ZJVY2000000:0:VYZ@L000000P2YZJT<
ooooo`H0ZJVY1@00000C0:VYZ@8000004@2YZJT8000000X0ZJVY1`0000020:VYZ@gooooo000<oooo
o`@0ZJVY100000070:VYZ@8000000P2YZJT2000000H0ZJVY0`00000:0:VYZ@8000002@2YZJT40000
0080ZJVY1P0000020:VYZ@8000001`2YZJT2000000cooooo1P2YZJT5000000P0ZJVY100000060:VY
Z@<000004@2YZJTI00000080ZJVY2oooool60:VYZ@D00000202YZJT4000000L0ZJVY0P00000A0:VY
ZAT000000P2YZJT<ooooo`H0ZJVY1@00000C0:VYZ@8000004@2YZJTI00000080ZJVY3?ooool60:VY
Z@D000004`2YZJT200000140ZJVY6@0000020:VYZ@gooooo000<ooooo`@0ZJVY100000070:VYZ@80
00000P2YZJT2000000H0ZJVY0`00000:0:VYZ@8000002@2YZJT400000080ZJVY1P0000020:VYZ@80
00001`2YZJT2000000cooooo1P2YZJT5000000P0ZJVY100000060:VYZ@<000004@2YZJTI00000080
ZJVY2oooool60:VYZ@D00000202YZJT4000000L0ZJVY0P00000A0:VYZAT000000P2YZJT<ooooo`H0
ZJVY1@00000C0:VYZ@8000004@2YZJTI00000080ZJVY3?ooool60:VYZ@D000004`2YZJT200000140
ZJVY6@0000020:VYZ@gooooo000<ooooo`H0ZJVY0P0000020:VYZ@<000000P2YZJT200000080ZJVY
100000020:VYZ@@000000`2YZJT2000000P0ZJVY0P0000020:VYZ@<000000P2YZJT2000000P0ZJVY
0P0000020:VYZ@L000000P2YZJT2000000cooooo1P2YZJTJ00000140ZJVY6@0000020:VYZ@_ooooo
1P2YZJTJ00000140ZJVY6@0000020:VYZ@cooooo1P2YZJTJ00000140ZJVY6@0000020:VYZ@cooooo
2P2YZJTF00000140ZJVY6@0000020:VYZ@gooooo000<ooooo`H0ZJVY0P0000020:VYZ@<000000P2Y
ZJT200000080ZJVY100000020:VYZ@@000000`2YZJT2000000P0ZJVY0P0000020:VYZ@<000000P2Y
ZJT2000000P0ZJVY0P0000020:VYZ@L000000P2YZJT2000000cooooo1P2YZJTJ00000140ZJVY6@00
00020:VYZ@_ooooo1P2YZJTJ00000140ZJVY6@0000020:VYZ@cooooo1P2YZJTJ00000140ZJVY6@00
00020:VYZ@cooooo2P2YZJTF00000140ZJVY6@0000020:VYZ@gooooo000<ooooo`@000000P2YZJT4
000000030:VYZ@000000000000@00000102YZJT8000000030:VYZ@00000000000080ZJVY0P00000:
0:VYZ@/000000P2YZJT200000080ZJVY1@0000040:VYZ@@000003?ooool8000000D0ZJVY4`00000<
0:VYZB0000002oooool6000000T0ZJVY4@00000@0:VYZA`000003?ooool4000000d0ZJVY3`00000A
0:VYZA/000003?oooolC0:VYZ@d000004@2YZJTK000000gooooo000<ooooo`800000102YZJT20000
00D0ZJVY100000060:VYZ@@000000`2YZJT400000080ZJVY0P0000020:VYZ@@000000P2YZJT20000
00030:VYZ@000000000000800000102YZJT2000000@0ZJVY0P0000030:VYZ@8000000P2YZJT20000
0080ZJVY3?ooool8000000D0ZJVY5@0000080:VYZB8000002oooool4000000/0ZJVY4@00000<0:VY
ZB0000003?ooool200000140ZJVY3@00000@0:VYZA`000003?ooool2000001D0ZJVY2@00000A0:VY
ZA/000003Oooool000cooooo0P0000040:VYZ@8000001@2YZJT4000000H0ZJVY100000030:VYZ@@0
00000P2YZJT200000080ZJVY100000020:VYZ@80000000<0ZJVY0000000000000P0000040:VYZ@80
0000102YZJT2000000<0ZJVY0P0000020:VYZ@8000000P2YZJT<ooooo`P000001@2YZJTE000000P0
ZJVY8P00000;ooooo`@000002`2YZJTA000000`0ZJVY8000000<ooooo`8000004@2YZJT=00000100
ZJVY7000000<ooooo`8000005@2YZJT900000140ZJVY6`00000=ooooo`003?ooool40:VYZ@800000
1`2YZJT200000080ZJVY0P0000020:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@040:VYZA400000
0P2YZJT400000080ZJVY0P0000020:VYZ@<000000P2YZJT6000000cooooo0P00000C0:VYZCL00000
2oooool2000001D0ZJVY=@00000<ooooo`8000005@2YZJTe000000cooooo0P00000E0:VYZCD00000
3Oooool000cooooo102YZJT2000000L0ZJVY0P0000020:VYZ@8000000P2YZJT600000080ZJVY00<0
00000:VYZ@2YZJT0102YZJTA00000080ZJVY100000020:VYZ@8000000P2YZJT300000080ZJVY1P00
000<ooooo`8000004`2YZJTg000000_ooooo0P00000E0:VYZCD000003?ooool2000001D0ZJVY=@00
000<ooooo`8000005@2YZJTe000000gooooo000<ooooo`8000001P2YZJT5000000P0ZJVY10000007
0:VYZ@H000001P2YZJT2000000T0ZJVY0P0000060:VYZ@@000000`2YZJT200000080ZJVY0P00000<
ooooo`800000202YZJT3000000X0ZJVY=@00000;ooooo`8000005@2YZJTe000000cooooo0P00000E
0:VYZCD000003?ooool2000001D0ZJVY=@00000=ooooo`003?ooool2000000H0ZJVY1@0000080:VY
Z@@000001`2YZJT6000000H0ZJVY0P0000090:VYZ@8000001P2YZJT4000000<0ZJVY0P0000020:VY
Z@8000003?ooool2000000P0ZJVY0`00000:0:VYZCD000002oooool2000001D0ZJVY=@00000<oooo
o`8000005@2YZJTe000000cooooo0P00000E0:VYZCD000003Oooool000cooooo0P2YZJT2000000H0
ZJVY0`00000:0:VYZ@@000000P2YZJT500000080ZJVY100000020:VYZ@@000000P2YZJT300000080
ZJVY0P0000020:VYZ@8000000P2YZJT2000000@0ZJVY1`0000020:VYZ@cooooo0P0000080:VYZ@<0
00002P2YZJTe000000_ooooo0P00000E0:VYZCD000003?ooool2000001D0ZJVY=@00000<ooooo`80
00005@2YZJTe000000gooooo000<ooooo`80ZJVY0P0000060:VYZ@<000002P2YZJT400000080ZJVY
1@0000020:VYZ@@000000P2YZJT400000080ZJVY0`0000020:VYZ@8000000P2YZJT200000080ZJVY
0P0000040:VYZ@L000000P2YZJT<ooooo`800000202YZJT3000000X0ZJVY=@00000;ooooo`800000
5@2YZJTe000000cooooo0P00000E0:VYZCD000003?ooool2000001D0ZJVY=@00000=ooooo`003?oo
ool20:VYZ@H000000P2YZJT00`000000ZJVY0:VYZ@02000000@0ZJVY0P0000020:VYZ@8000000P2Y
ZJT300000080ZJVY100000020:VYZ@8000001P2YZJT2000000030:VYZ@000000000000@0ZJVY0P00
00040:VYZ@8000000P2YZJT300000080ZJVY0P0000040:VYZ@cooooo3@00000:0:VYZ@H000001`2Y
ZJTX000000_ooooo3@00000:0:VYZ@H000002`2YZJTT000000cooooo2P00000=0:VYZ@H000003@2Y
ZJTR000000cooooo1P00000A0:VYZ@H000003`2YZJTP000000gooooo000<ooooo`80ZJVY1P000002
0:VYZ@030000002YZJT0ZJVY00800000102YZJT200000080ZJVY0P0000020:VYZ@<000000P2YZJT4
00000080ZJVY0P0000060:VYZ@80000000<0ZJVY000000000000102YZJT2000000@0ZJVY0P000002
0:VYZ@<000000P2YZJT2000000@0ZJVY3?ooool=000000X0ZJVY1P0000070:VYZBP000002oooool=
000000X0ZJVY1P00000;0:VYZB@000003?ooool:000000d0ZJVY1P00000=0:VYZB8000003?ooool6
00000140ZJVY1P00000?0:VYZB0000003Oooool00?ooooooaOooool00?ooooooaOooool00?oooooo
aOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?oooooo
aOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?ooooooaOooool00?oooooo
aOooool00?ooooooaOooool00001\
\>"],
  ImageRangeCache->{{{0, 451.938}, {544.188, 0}} -> {-0.125026, -0.1516, \
0.0116168, 0.0116168}, {{10.75, 90.4375}, {531.125, 451.375}} -> {-6.66615, \
-7.90146, 0.527065, 0.527065}, {{98.4375, 178.125}, {531.125, 451.375}} -> \
{-52.8831, -7.90146, 0.527065, 0.527065}, {{186.062, 265.812}, {531.125, \
451.375}} -> {-98.9903, -7.8796, 0.526652, 0.526652}, {{273.75, 353.5}, \
{531.125, 451.375}} -> {-145.171, -7.8796, 0.526652, 0.526652}, {{361.438, \
441.125}, {531.125, 451.375}} -> {-191.501, -7.90146, 0.527065, 0.527065}, \
{{10.75, 90.4375}, {443.438, 363.75}} -> {-6.66615, -54.102, 0.527065, \
0.527065}, {{98.4375, 178.125}, {443.438, 363.75}} -> {-52.8831, -54.102, \
0.527065, 0.527065}, {{186.062, 265.812}, {443.438, 363.75}} -> {-99.0836, \
-54.102, 0.527065, 0.527065}, {{273.75, 353.5}, {443.438, 363.75}} -> \
{-145.301, -54.102, 0.527065, 0.527065}, {{361.438, 441.125}, {443.438, \
363.75}} -> {-191.501, -54.102, 0.527065, 0.527065}, {{10.75, 90.4375}, \
{355.75, 276.062}} -> {-6.66615, -100.319, 0.527065, 0.527065}, {{98.4375, \
178.125}, {355.75, 276.062}} -> {-52.8831, -100.319, 0.527065, 0.527065}, \
{{186.062, 265.812}, {355.75, 276.062}} -> {-99.0836, -100.319, 0.527065, \
0.527065}, {{273.75, 353.5}, {355.75, 276.062}} -> {-145.301, -100.319, \
0.527065, 0.527065}, {{361.438, 441.125}, {355.75, 276.062}} -> {-191.501, \
-100.319, 0.527065, 0.527065}, {{10.75, 90.4375}, {268.062, 188.375}} -> \
{-6.66615, -146.536, 0.527065, 0.527065}, {{98.4375, 178.125}, {268.062, \
188.375}} -> {-52.8831, -146.536, 0.527065, 0.527065}, {{186.062, 265.812}, \
{268.062, 188.375}} -> {-99.0836, -146.536, 0.527065, 0.527065}, {{273.75, \
353.5}, {268.062, 188.375}} -> {-145.301, -146.536, 0.527065, 0.527065}, \
{{361.438, 441.125}, {268.062, 188.375}} -> {-191.501, -146.536, 0.527065, \
0.527065}, {{10.75, 90.4375}, {180.375, 100.688}} -> {-6.66615, -192.753, \
0.527065, 0.527065}, {{98.4375, 178.125}, {180.375, 100.688}} -> {-52.8831, \
-192.753, 0.527065, 0.527065}, {{186.062, 265.812}, {180.375, 100.688}} -> \
{-99.0836, -192.753, 0.527065, 0.527065}, {{273.75, 353.5}, {180.375, \
100.688}} -> {-145.301, -192.753, 0.527065, 0.527065}, {{361.438, 441.125}, \
{180.375, 100.688}} -> {-191.501, -192.753, 0.527065, 0.527065}, {{10.75, \
90.4375}, {92.75, 13}} -> {-6.66615, -238.953, 0.527065, 0.527065}, \
{{98.4375, 178.125}, {92.75, 13}} -> {-52.8831, -238.953, 0.527065, \
0.527065}, {{186.062, 265.812}, {92.75, 13}} -> {-98.9903, -238.75, 0.526652, \
0.526652}, {{273.75, 353.5}, {92.75, 13}} -> {-145.171, -238.75, 0.526652, \
0.526652}, {{361.438, 441.125}, {92.75, 13}} -> {-191.501, -238.953, \
0.527065, 0.527065}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[m8StagHunt]\)], "Input",
  CellLabel->"In[47]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Ultimatum Game on a lattice", "Section",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  "Here's the payoff matrix for the Ultimatum game. \"Row\" indicates \
Proposer and \"Column\" means Receiver. The indexes ",
  Cell[BoxData[
      \(TraditionalForm\`1, \[Ellipsis], 8\)]],
  " correspond to strategies ",
  Cell[BoxData[
      \(TraditionalForm\`S\_1\)]],
  " through ",
  Cell[BoxData[
      \(TraditionalForm\`S\_8\)]],
  " as below."
}], "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[TextData[{
  " ",
  Cell[BoxData[GridBox[{
          {"  ", 
            StyleBox["Offer",
              FontFamily->"Times"], 
            StyleBox["Accept",
              FontFamily->"Times"]},
          {
            StyleBox[
              RowBox[{
                FormBox[\(\(S\_1\)\(\ \)\),
                  "TraditionalForm"], 
                StyleBox[\((Gamesman)\),
                  FontFamily->"Times"]}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["All",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            StyleBox[Cell[TextData[Cell[BoxData[
                  \(TraditionalForm\`S\_2\)]]]],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["None",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            FormBox[\(\(S\_3\)\(\ \)\),
              "TraditionalForm"], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            RowBox[{
              StyleBox[
                FormBox[\(S\_4\),
                  "TraditionalForm"],
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              RowBox[{"(", 
                StyleBox[\(Mad\ Dog\),
                  FontFamily->"Times"], 
                StyleBox[")",
                  FontFamily->"Times"]}]}], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            FormBox[
              RowBox[{\(S\_5\), " ", 
                RowBox[{"(", 
                  StyleBox[\(Easy\ Rider\),
                    FontFamily->"Times"], 
                  StyleBox[")",
                    FontFamily->"Times"]}]}],
              "TraditionalForm"], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["All",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            StyleBox[
              FormBox[\(S\_6\),
                "TraditionalForm"],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["None",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            FormBox[
              RowBox[{\(S\_7\), " ", 
                RowBox[{"(", 
                  StyleBox["Fairman",
                    FontFamily->"Times"], 
                  StyleBox[")",
                    FontFamily->"Times"]}]}],
              "TraditionalForm"], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]},
          {
            StyleBox[
              FormBox[\(S\_8\),
                "TraditionalForm"],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["5",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}], 
            StyleBox["1",
              FontFamily->"Times",
              GridBoxOptions->{RowLines->True}]}
          },
        ColumnAlignments->{Left}]]]
}], "Text",
  TextAlignment->Center,
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"ultGame", "=", 
        RowBox[{"(", GridBox[{
              {\({9, 1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\), \({9, 
                  1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\)},
              {\({9, 1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\), \({9, 
                  1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\)},
              {\({9, 1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\), \({9, 
                  1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\)},
              {\({9, 1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\), \({9, 
                  1}\), \({0, 0}\), \({0, 0}\), \({9, 1}\)},
              {\({5, 5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\), \({5, 
                  5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\)},
              {\({5, 5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\), \({5, 
                  5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\)},
              {\({5, 5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\), \({5, 
                  5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\)},
              {\({5, 5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\), \({5, 
                  5}\), \({0, 0}\), \({5, 5}\), \({0, 0}\)}
              }], ")"}]}], 
      ";"}], "\n", \(ugColors = {\ \[IndentingNewLine]RGBColor[0, 0, 
            0], \[IndentingNewLine]RGBColor[0, 0, 
            1]\ , \[IndentingNewLine]RGBColor[0, 1, 
            0]\ , \[IndentingNewLine]RGBColor[0, 1, 
            1]\ , \[IndentingNewLine]RGBColor[1, 0, 
            0]\ , \[IndentingNewLine]RGBColor[1, 0, 
            1]\ , \[IndentingNewLine]RGBColor[1, 1, 
            0]\ , \[IndentingNewLine]RGBColor[ .8,  .8,  .8]\ \
\[IndentingNewLine]};\), "\n", \(StrategyLegend[\[IndentingNewLine]ugColors, \
\[IndentingNewLine]{"\<Gamesman\>", "\<S2\>", "\<S3\>", "\<Mad Dog\>", \
"\<Easy Rider\>", "\<S6\>", "\<Fairman\>", \
"\<S8\>"}\[IndentingNewLine]];\)}], "Input",
  CellLabel->"In[52]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[GridBox[{
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0],
                Background->GrayLevel[0]],
              BoxMargins->False], "Gamesman"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 0, 1],
                Background->RGBColor[0, 0, 1]],
              BoxMargins->False], "S2"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 1, 0],
                Background->RGBColor[0, 1, 0]],
              BoxMargins->False], "S3"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[0, 1, 1],
                Background->RGBColor[0, 1, 1]],
              BoxMargins->False], \(Mad\ Dog\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 0, 0],
                Background->RGBColor[1, 0, 0]],
              BoxMargins->False], \(Easy\ Rider\)},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 0, 1],
                Background->RGBColor[1, 0, 1]],
              BoxMargins->False], "S6"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->RGBColor[1, 1, 0],
                Background->RGBColor[1, 1, 0]],
              BoxMargins->False], "Fairman"},
          {
            FrameBox[
              StyleBox[
                AdjustmentBox["Stra",
                  BoxMargins->{{1, 1}, {1, 1}}],
                FontColor->GrayLevel[0.8],
                Background->GrayLevel[0.8]],
              BoxMargins->False], "S8"}
          },
        ColumnAlignments->{Left}],
      DisplayForm]], "Print",
  CellLabel->"From In[52]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["von Neumann", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(ugVN = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] VonNeumann[10, 10], \[IndentingNewLine]PayoffMatrix \[Rule] 
            ultGame, \[IndentingNewLine]StrategyColors \[Rule] 
            ugColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[56]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[
        ExtractGraph[ugVN], \[IndentingNewLine]VertexStyle \[Rule] 
          Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[57]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 0 r
.24747 .04545 Mdot
1 0 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 0 1 r
.55051 .04545 Mdot
.8 .8 .8 r
.65152 .04545 Mdot
0 1 1 r
.75253 .04545 Mdot
1 1 0 r
.85354 .04545 Mdot
0 1 0 r
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
0 1 0 r
.14646 .14646 Mdot
0 0 1 r
.24747 .14646 Mdot
0 1 0 r
.34848 .14646 Mdot
1 0 1 r
.44949 .14646 Mdot
.8 .8 .8 r
.55051 .14646 Mdot
1 0 1 r
.65152 .14646 Mdot
.75253 .14646 Mdot
0 1 0 r
.85354 .14646 Mdot
1 0 0 r
.95455 .14646 Mdot
.8 .8 .8 r
.04545 .24747 Mdot
.14646 .24747 Mdot
0 0 1 r
.24747 .24747 Mdot
.8 .8 .8 r
.34848 .24747 Mdot
1 1 0 r
.44949 .24747 Mdot
.8 .8 .8 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
0 1 1 r
.75253 .24747 Mdot
.85354 .24747 Mdot
0 0 0 r
.95455 .24747 Mdot
0 1 1 r
.04545 .34848 Mdot
1 1 0 r
.14646 .34848 Mdot
0 0 0 r
.24747 .34848 Mdot
.8 .8 .8 r
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.8 .8 .8 r
.65152 .34848 Mdot
.75253 .34848 Mdot
1 0 1 r
.85354 .34848 Mdot
1 1 0 r
.95455 .34848 Mdot
.04545 .44949 Mdot
1 0 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 1 r
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
0 0 1 r
.65152 .44949 Mdot
0 1 0 r
.75253 .44949 Mdot
0 1 1 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.8 .8 .8 r
.14646 .55051 Mdot
0 0 1 r
.24747 .55051 Mdot
1 1 0 r
.34848 .55051 Mdot
1 0 1 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.8 .8 .8 r
.65152 .55051 Mdot
0 0 1 r
.75253 .55051 Mdot
0 0 0 r
.85354 .55051 Mdot
1 0 1 r
.95455 .55051 Mdot
.8 .8 .8 r
.04545 .65152 Mdot
0 1 1 r
.14646 .65152 Mdot
0 0 1 r
.24747 .65152 Mdot
.34848 .65152 Mdot
0 1 0 r
.44949 .65152 Mdot
0 1 1 r
.55051 .65152 Mdot
1 1 0 r
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
1 0 1 r
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
0 1 0 r
.14646 .75253 Mdot
1 1 0 r
.24747 .75253 Mdot
0 0 1 r
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
0 0 0 r
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
1 1 0 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
0 0 0 r
.24747 .85354 Mdot
1 1 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
0 1 1 r
.55051 .85354 Mdot
0 0 1 r
.65152 .85354 Mdot
0 1 0 r
.75253 .85354 Mdot
1 0 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
1 0 1 r
.44949 .95455 Mdot
0 1 0 r
.55051 .95455 Mdot
1 0 1 r
.65152 .95455 Mdot
.8 .8 .8 r
.75253 .95455 Mdot
0 0 1 r
.85354 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[57]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`002_ooool6oooo01Oooooo1_oo
o`0Gooooo`Ko0?l05oooool6ool001Oooooo1_l0oolGooooo`Koo`3o5oooool6olc<c1Oooooo1_l0
oolGooooo`Koool05oooool6o`3o00_ooooo0008ooooo`[oool04oooool:oooo01?ooooo2_l0o`0C
ooooo`[oo`004oooool:o`3ooa?ooooo2_oo0?lCooooo`[oc<c<4oooool:o`3ooa?ooooo2_ooo`0C
ooooo`[o0?l02Oooool000Kooooo3_ooo`0?ooooo`koool03oooool>o`3o00oooooo3_oo000?oooo
o`ko0?oo3oooool>ool0o`oooooo3_o<c<`?ooooo`ko0?oo3oooool>oooo00oooooo3_l0o`07oooo
o`001Oooool@oooo00gooooo4?ooo`0=oooooa3o0?l03Oooool@ool000gooooo4?l0ool=oooooa3o
o`3o3Oooool@olc<c0gooooo4?l0ool=oooooa3oool03Oooool@o`3o00Kooooo0005oooooa3oool0
3Oooool@oooo00gooooo4?l0o`0=oooooa3oo`003Oooool@o`3oo`gooooo4?oo0?l=oooooa3oc<c<
3Oooool@o`3oo`gooooo4?ooo`0=oooooa3o0?l01_ooool000Cooooo4_ooo`0;oooooa;oool02ooo
oolBo`3o00_ooooo4_oo000;oooooa;o0?oo2ooooolBool0o`_ooooo4_o<c<`;oooooa;o0?oo2ooo
oolBoooo00_ooooo4_l0o`05ooooo`001?oooolBoooo00_ooooo4_ooo`0;oooooa;o0?l02ooooolB
ool000_ooooo4_l0ool;oooooa;oo`3o2ooooolBolc<c0_ooooo4_l0ool;oooooa;oool02ooooolB
o`3o00Gooooo0003oooooaCoool02OoooolDoooo00Wooooo5?l0o`09oooooaCoo`002OoooolDo`3o
o`Wooooo5?oo0?l9oooooaCoc<c<2OoooolDo`3oo`Wooooo5?ooo`09oooooaCo0?l01?ooool000?o
oooo5?ooo`09oooooaCoool02OoooolDo`3o00Wooooo5?oo0009oooooaCo0?oo2OoooolDool0o`Wo
oooo5?o<c<`9oooooaCo0?oo2OoooolDoooo00Wooooo5?l0o`04ooooo`000ooooolDoooo00Wooooo
5?ooo`09oooooaCo0?l02OoooolDool000Wooooo5?l0ool9oooooaCoo`3o2OoooolDolc<c0Wooooo
5?l0ool9oooooaCoool02OoooolDo`3o00Cooooo0003oooooaCoool02Ol0000Doooo00Wo00005?l0
o`09o`0001Coo`002Ol0000Do`3oo`Wo00005?oo0?l9o`0001Coc<c<2Ol0000Do`3oo`Wo00005?oo
o`09o`0001Co0?l01?ooool000?ooooo5?ooo`09oooooaCoool02OoooolDo`3o00Wooooo5?oo0009
oooooaCo0?oo2OoooolDool0o`Wooooo5?o<c<`9oooooaCo0?oo2OoooolDoooo00Wooooo5?l0o`04
ooooo`000ooooolDoooo00Wooooo5?ooo`09oooooaCo0?l02OoooolDool000Wooooo5?l0ool9oooo
oaCoo`3o2OoooolDolc<c0Wooooo5?l0ool9oooooaCoool02OoooolDo`3o00Cooooo0004oooooa;o
ool02ooooolBoooo00_ooooo4_l0o`0;oooooa;oo`002ooooolBo`3oo`_ooooo4_oo0?l;oooooa;o
c<c<2ooooolBo`3oo`_ooooo4_ooo`0;oooooa;o0?l01Oooool000Cooooo4_ooo`0;oooooa;oool0
2ooooolBo`3o00_ooooo4_oo000;oooooa;o0?oo2ooooolBool0o`_ooooo4_o<c<`;oooooa;o0?oo
2ooooolBoooo00_ooooo4_l0o`05ooooo`001Oooool@oooo00gooooo4?ooo`0=oooooa3o0?l03Ooo
ool@ool000gooooo4?l0ool=oooooa3oo`3o3Oooool@olc<c0gooooo4?l0ool=oooooa3oool03Ooo
ool@o`3o00Kooooo0005oooooa3oool03Oooool@oooo00gooooo4?l0o`0=oooooa3oo`003Oooool@
o`3oo`gooooo4?oo0?l=oooooa3oc<c<3Oooool@o`3oo`gooooo4?ooo`0=oooooa3o0?l01_ooool0
00Kooooo3_ooo`0?ooooo`koool03oooool>o`3o00oooooo3_oo000?ooooo`ko0?oo3oooool>ool0
o`oooooo3_o<c<`?ooooo`ko0?oo3oooool>oooo00oooooo3_l0o`07ooooo`002?ooool:oooo01?o
oooo2_ooo`0Cooooo`[o0?l04oooool:ool001?ooooo2_l0oolCooooo`[oo`3o4oooool:olc<c1?o
oooo2_l0oolCooooo`[oool04oooool:o`3o00Wooooo000:ooooo`Koool05oooool6oooo01Oooooo
1_l0o`0Gooooo`Koo`005oooool6o`3ooaOooooo1_oo0?lGooooo`Koc<c<5oooool6o`3ooaOooooo
1_ooo`0Gooooo`Ko0?l02oooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000[ooooo1_ooo`0Gooooo`Ko0?l05oooool6o`00oaOooooo
1_l0o`0Gooooo`Koo`3o5oooool6olc<c1Oooooo1_oo0?lGooooo`Koo`3o5oooool6o`3o01Oooooo
1_oo000;ooooo`002?ooool:oooo01?ooooo2_l0o`0Cooooo`[o003o4oooool:o`3o01?ooooo2_oo
0?lCooooo`[oc<c<4oooool:ool0oa?ooooo2_oo0?lCooooo`[o0?l04oooool:ool000Wooooo0006
ooooo`koool03oooool>o`3o00oooooo3_l00?l?ooooo`ko0?l03oooool>ool0o`oooooo3_o<c<`?
ooooo`koo`3o3oooool>ool0o`oooooo3_l0o`0?ooooo`koo`001oooool000Gooooo4?ooo`0=oooo
oa3o0?l03Oooool@o`00o`gooooo4?l0o`0=oooooa3oo`3o3Oooool@olc<c0gooooo4?oo0?l=oooo
oa3oo`3o3Oooool@o`3o00gooooo4?oo0006ooooo`001Oooool@oooo00gooooo4?l0o`0=oooooa3o
003o3Oooool@o`3o00gooooo4?oo0?l=oooooa3oc<c<3Oooool@ool0o`gooooo4?oo0?l=oooooa3o
0?l03Oooool@ool000Kooooo0004oooooa;oool02ooooolBo`3o00_ooooo4_l00?l;oooooa;o0?l0
2ooooolBool0o`_ooooo4_o<c<`;oooooa;oo`3o2ooooolBool0o`_ooooo4_l0o`0;oooooa;oo`00
1Oooool000Cooooo4_ooo`0;oooooa;o0?l02ooooolBo`00o`_ooooo4_l0o`0;oooooa;oo`3o2ooo
oolBolc<c0_ooooo4_oo0?l;oooooa;oo`3o2ooooolBo`3o00_ooooo4_oo0005ooooo`000ooooolD
oooo00Wooooo5?l0o`09oooooaCo003o2OoooolDo`3o00Wooooo5?oo0?l9oooooaCoc<c<2OoooolD
ool0o`Wooooo5?oo0?l9oooooaCo0?l02OoooolDool000Cooooo0003oooooaCoool02OoooolDo`3o
00Wooooo5?l00?l9oooooaCo0?l02OoooolDool0o`Wooooo5?o<c<`9oooooaCoo`3o2OoooolDool0
o`Wooooo5?l0o`09oooooaCoo`001?ooool000?ooooo5?ooo`09oooooaCo0?l02OoooolDo`00o`Wo
oooo5?l0o`09oooooaCoo`3o2OoooolDolc<c0Wooooo5?oo0?l9oooooaCoo`3o2OoooolDo`3o00Wo
oooo5?oo0004ooooo`000ooooolDoooo00Wo00005?l0o`09o`0001Co003o2Ol0000Do`3o00Wo0000
5?oo0?l9o`0001Coc<c<2Ol0000Dool0o`Wo00005?oo0?l9o`0001Co0?l02Ol0000Dool000Cooooo
0003oooooaCoool02OoooolDo`3o00Wooooo5?l00?l9oooooaCo0?l02OoooolDool0o`Wooooo5?o<
c<`9oooooaCoo`3o2OoooolDool0o`Wooooo5?l0o`09oooooaCoo`001?ooool000?ooooo5?ooo`09
oooooaCo0?l02OoooolDo`00o`Wooooo5?l0o`09oooooaCoo`3o2OoooolDolc<c0Wooooo5?oo0?l9
oooooaCoo`3o2OoooolDo`3o00Wooooo5?oo0004ooooo`001?oooolBoooo00_ooooo4_l0o`0;oooo
oa;o003o2ooooolBo`3o00_ooooo4_oo0?l;oooooa;oc<c<2ooooolBool0o`_ooooo4_oo0?l;oooo
oa;o0?l02ooooolBool000Gooooo0004oooooa;oool02ooooolBo`3o00_ooooo4_l00?l;oooooa;o
0?l02ooooolBool0o`_ooooo4_o<c<`;oooooa;oo`3o2ooooolBool0o`_ooooo4_l0o`0;oooooa;o
o`001Oooool000Gooooo4?ooo`0=oooooa3o0?l03Oooool@o`00o`gooooo4?l0o`0=oooooa3oo`3o
3Oooool@olc<c0gooooo4?oo0?l=oooooa3oo`3o3Oooool@o`3o00gooooo4?oo0006ooooo`001Ooo
ool@oooo00gooooo4?l0o`0=oooooa3o003o3Oooool@o`3o00gooooo4?oo0?l=oooooa3oc<c<3Ooo
ool@ool0o`gooooo4?oo0?l=oooooa3o0?l03Oooool@ool000Kooooo0006ooooo`koool03oooool>
o`3o00oooooo3_l00?l?ooooo`ko0?l03oooool>ool0o`oooooo3_o<c<`?ooooo`koo`3o3oooool>
ool0o`oooooo3_l0o`0?ooooo`koo`001oooool000Sooooo2_ooo`0Cooooo`[o0?l04oooool:o`00
oa?ooooo2_l0o`0Cooooo`[oo`3o4oooool:olc<c1?ooooo2_oo0?lCooooo`[oo`3o4oooool:o`3o
01?ooooo2_oo0009ooooo`002_ooool6oooo01Oooooo1_l0o`0Gooooo`Ko003o5oooool6o`3o01Oo
oooo1_oo0?lGooooo`Koc<c<5oooool6ool0oaOooooo1_oo0?lGooooo`Ko0?l05oooool6ool000_o
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000:ooooo`Koc<c<5oooool6olc<c1Oooooo1_l00?lGooooo`Koc<c<5oooool6oooo01Oo
oooo1_o<c<`Gooooo`Koo`3o5oooool6o`3ooaOooooo1_l0oolGooooo`Ko00002oooool000Sooooo
2_o<c<`Cooooo`[oc<c<4oooool:o`00oa?ooooo2_o<c<`Cooooo`[oool04oooool:olc<c1?ooooo
2_oo0?lCooooo`[o0?oo4oooool:o`3ooa?ooooo2_l00009ooooo`001_ooool>olc<c0oooooo3_o<
c<`?ooooo`ko003o3oooool>olc<c0oooooo3_ooo`0?ooooo`koc<c<3oooool>ool0o`oooooo3_l0
ool?ooooo`ko0?oo3oooool>o`0000Oooooo0005oooooa3oc<c<3Oooool@olc<c0gooooo4?l00?l=
oooooa3oc<c<3Oooool@oooo00gooooo4?o<c<`=oooooa3oo`3o3Oooool@o`3oo`gooooo4?l0ool=
oooooa3o00001_ooool000Gooooo4?o<c<`=oooooa3oc<c<3Oooool@o`00o`gooooo4?o<c<`=oooo
oa3oool03Oooool@olc<c0gooooo4?oo0?l=oooooa3o0?oo3Oooool@o`3oo`gooooo4?l00006oooo
o`001?oooolBolc<c0_ooooo4_o<c<`;oooooa;o003o2ooooolBolc<c0_ooooo4_ooo`0;oooooa;o
c<c<2ooooolBool0o`_ooooo4_l0ool;oooooa;o0?oo2ooooolBo`0000Gooooo0004oooooa;oc<c<
2ooooolBolc<c0_ooooo4_l00?l;oooooa;oc<c<2ooooolBoooo00_ooooo4_o<c<`;oooooa;oo`3o
2ooooolBo`3oo`_ooooo4_l0ool;oooooa;o00001Oooool000?ooooo5?o<c<`9oooooaCoc<c<2Ooo
oolDo`00o`Wooooo5?o<c<`9oooooaCoool02OoooolDolc<c0Wooooo5?oo0?l9oooooaCo0?oo2Ooo
oolDo`3oo`Wooooo5?l00004ooooo`000ooooolDolc<c0Wooooo5?o<c<`9oooooaCo003o2OoooolD
olc<c0Wooooo5?ooo`09oooooaCoc<c<2OoooolDool0o`Wooooo5?l0ool9oooooaCo0?oo2OoooolD
o`0000Cooooo0003oooooaCoc<c<2OoooolDolc<c0Wooooo5?l00?l9oooooaCoc<c<2OoooolDoooo
00Wooooo5?o<c<`9oooooaCoo`3o2OoooolDo`3oo`Wooooo5?l0ool9oooooaCo00001?ooool000?o
oooo5?o<c<`9o`0001Coc<c<2Ol0000Do`00o`Wo00005?o<c<`9o`0001Coool02Ol0000Dolc<c0Wo
00005?oo0?l9o`0001Co0?oo2Ol0000Do`3ooago00001?ooool000?ooooo5?o<c<`9oooooaCoc<c<
2OoooolDo`00o`Wooooo5?o<c<`9oooooaCoool02OoooolDolc<c0Wooooo5?oo0?l9oooooaCo0?oo
2OoooolDo`3oo`Wooooo5?l00004ooooo`000ooooolDolc<c0Wooooo5?o<c<`9oooooaCo003o2Ooo
oolDolc<c0Wooooo5?ooo`09oooooaCoc<c<2OoooolDool0o`Wooooo5?l0ool9oooooaCo0?oo2Ooo
oolDo`0000Cooooo0004oooooa;oc<c<2ooooolBolc<c0_ooooo4_l00?l;oooooa;oc<c<2ooooolB
oooo00_ooooo4_o<c<`;oooooa;oo`3o2ooooolBo`3oo`_ooooo4_l0ool;oooooa;o00001Oooool0
00Cooooo4_o<c<`;oooooa;oc<c<2ooooolBo`00o`_ooooo4_o<c<`;oooooa;oool02ooooolBolc<
c0_ooooo4_oo0?l;oooooa;o0?oo2ooooolBo`3oo`_ooooo4_l00005ooooo`001Oooool@olc<c0go
oooo4?o<c<`=oooooa3o003o3Oooool@olc<c0gooooo4?ooo`0=oooooa3oc<c<3Oooool@ool0o`go
oooo4?l0ool=oooooa3o0?oo3Oooool@o`0000Kooooo0005oooooa3oc<c<3Oooool@olc<c0gooooo
4?l00?l=oooooa3oc<c<3Oooool@oooo00gooooo4?o<c<`=oooooa3oo`3o3Oooool@o`3oo`gooooo
4?l0ool=oooooa3o00001_ooool000Kooooo3_o<c<`?ooooo`koc<c<3oooool>o`00o`oooooo3_o<
c<`?ooooo`koool03oooool>olc<c0oooooo3_oo0?l?ooooo`ko0?oo3oooool>o`3oo`oooooo3_l0
0007ooooo`002?ooool:olc<c1?ooooo2_o<c<`Cooooo`[o003o4oooool:olc<c1?ooooo2_ooo`0C
ooooo`[oc<c<4oooool:ool0oa?ooooo2_l0oolCooooo`[o0?oo4oooool:o`0000Wooooo000:oooo
o`Koc<c<5oooool6olc<c1Oooooo1_l00?lGooooo`Koc<c<5oooool6oooo01Oooooo1_o<c<`Goooo
o`Koo`3o5oooool6o`3ooaOooooo1_l0oolGooooo`Ko00002oooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0I
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000[ooooo1_l0oolGoooo
o`Koool05oooool6o`0001Oooooo1_o<c<`Gooooo`Ko0?l05oooool6o`3o01Oooooo1_o<c<`Goooo
o`Koc<c<5oooool6ool0oaOooooo1_ooo`0;ooooo`002?ooool:o`3ooa?ooooo2_ooo`0Cooooo`[o
00004oooool:olc<c1?ooooo2_l0o`0Cooooo`[o0?l04oooool:olc<c1?ooooo2_o<c<`Cooooo`[o
o`3o4oooool:oooo00Wooooo0006ooooo`ko0?oo3oooool>oooo00oooooo3_l0000?ooooo`koc<c<
3oooool>o`3o00oooooo3_l0o`0?ooooo`koc<c<3oooool>olc<c0oooooo3_oo0?l?ooooo`koool0
1oooool000Gooooo4?l0ool=oooooa3oool03Oooool@o`0000gooooo4?o<c<`=oooooa3o0?l03Ooo
ool@o`3o00gooooo4?o<c<`=oooooa3oc<c<3Oooool@ool0o`gooooo4?ooo`06ooooo`001Oooool@
o`3oo`gooooo4?ooo`0=oooooa3o00003Oooool@olc<c0gooooo4?l0o`0=oooooa3o0?l03Oooool@
olc<c0gooooo4?o<c<`=oooooa3oo`3o3Oooool@oooo00Kooooo0004oooooa;o0?oo2ooooolBoooo
00_ooooo4_l0000;oooooa;oc<c<2ooooolBo`3o00_ooooo4_l0o`0;oooooa;oc<c<2ooooolBolc<
c0_ooooo4_oo0?l;oooooa;oool01Oooool000Cooooo4_l0ool;oooooa;oool02ooooolBo`0000_o
oooo4_o<c<`;oooooa;o0?l02ooooolBo`3o00_ooooo4_o<c<`;oooooa;oc<c<2ooooolBool0o`_o
oooo4_ooo`05ooooo`000ooooolDo`3oo`Wooooo5?ooo`09oooooaCo00002OoooolDolc<c0Wooooo
5?l0o`09oooooaCo0?l02OoooolDolc<c0Wooooo5?o<c<`9oooooaCoo`3o2OoooolDoooo00Cooooo
0003oooooaCo0?oo2OoooolDoooo00Wooooo5?l00009oooooaCoc<c<2OoooolDo`3o00Wooooo5?l0
o`09oooooaCoc<c<2OoooolDolc<c0Wooooo5?oo0?l9oooooaCoool01?ooool000?ooooo5?l0ool9
oooooaCoool02OoooolDo`0000Wooooo5?o<c<`9oooooaCo0?l02OoooolDo`3o00Wooooo5?o<c<`9
oooooaCoc<c<2OoooolDool0o`Wooooo5?ooo`04ooooo`000ooooolDo`3oo`Wo00005?ooo`0Vo`00
01Coc<c<2Ol0000Do`3o00Wo00005?l0o`09o`0001Coc<c<2Ol0000Dolc<c0Wo00005?oo0?l9o`00
01Coool01?ooool000?ooooo5?l0ool9oooooaCoool02OoooolDo`0000Wooooo5?o<c<`9oooooaCo
0?l02OoooolDo`3o00Wooooo5?o<c<`9oooooaCoc<c<2OoooolDool0o`Wooooo5?ooo`04ooooo`00
0ooooolDo`3oo`Wooooo5?ooo`09oooooaCo00002OoooolDolc<c0Wooooo5?l0o`09oooooaCo0?l0
2OoooolDolc<c0Wooooo5?o<c<`9oooooaCoo`3o2OoooolDoooo00Cooooo0004oooooa;o0?oo2ooo
oolBoooo00_ooooo4_l0000;oooooa;oc<c<2ooooolBo`3o00_ooooo4_l0o`0;oooooa;oc<c<2ooo
oolBolc<c0_ooooo4_oo0?l;oooooa;oool01Oooool000Cooooo4_l0ool;oooooa;oool02ooooolB
o`0000_ooooo4_o<c<`;oooooa;o0?l02ooooolBo`3o00_ooooo4_o<c<`;oooooa;oc<c<2ooooolB
ool0o`_ooooo4_ooo`05ooooo`001Oooool@o`3oo`gooooo4?ooo`0=oooooa3o00003Oooool@olc<
c0gooooo4?l0o`0=oooooa3o0?l03Oooool@olc<c0gooooo4?o<c<`=oooooa3oo`3o3Oooool@oooo
00Kooooo0005oooooa3o0?oo3Oooool@oooo00gooooo4?l0000=oooooa3oc<c<3Oooool@o`3o00go
oooo4?l0o`0=oooooa3oc<c<3Oooool@olc<c0gooooo4?oo0?l=oooooa3oool01_ooool000Kooooo
3_l0ool?ooooo`koool03oooool>o`0000oooooo3_o<c<`?ooooo`ko0?l03oooool>o`3o00oooooo
3_o<c<`?ooooo`koc<c<3oooool>ool0o`oooooo3_ooo`07ooooo`002?ooool:o`3ooa?ooooo2_oo
o`0Cooooo`[o00004oooool:olc<c1?ooooo2_l0o`0Cooooo`[o0?l04oooool:olc<c1?ooooo2_o<
c<`Cooooo`[oo`3o4oooool:oooo00Wooooo000:ooooo`Ko0?oo5oooool6oooo01Oooooo1_l0000G
ooooo`Koc<c<5oooool6o`3o01Oooooo1_l0o`0Gooooo`Koc<c<5oooool6olc<c1Oooooo1_oo0?lG
ooooo`Koool02oooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool03?ooool000[ooooo1_ooo`0Gooooo`Koo`005oooool6ool001Oooooo1_l00?lG
ooooo`Koo`005oooool6o`3ooaOooooo1_l00?lGooooo`Ko0?l05oooool6o`3ooaOooooo1_ooo`0;
ooooo`002?ooool:oooo01?ooooo2_oo000Cooooo`[oo`004oooool:o`00oa?ooooo2_oo000Coooo
o`[o0?oo4oooool:o`00oa?ooooo2_l0o`0Cooooo`[o0?oo4oooool:oooo00Wooooo0006ooooo`ko
ool03oooool>ool000oooooo3_oo000?ooooo`ko003o3oooool>ool000oooooo3_l0ool?ooooo`ko
003o3oooool>o`3o00oooooo3_l0ool?ooooo`koool01oooool000Gooooo4?ooo`0=oooooa3oo`00
3Oooool@ool000gooooo4?l00?l=oooooa3oo`003Oooool@o`3oo`gooooo4?l00?l=oooooa3o0?l0
3Oooool@o`3oo`gooooo4?ooo`06ooooo`001Oooool@oooo00gooooo4?oo000=oooooa3oo`003Ooo
ool@o`00o`gooooo4?oo000=oooooa3o0?oo3Oooool@o`00o`gooooo4?l0o`0=oooooa3o0?oo3Ooo
ool@oooo00Kooooo0004oooooa;oool02ooooolBool000_ooooo4_oo000;oooooa;o003o2ooooolB
ool000_ooooo4_l0ool;oooooa;o003o2ooooolBo`3o00_ooooo4_l0ool;oooooa;oool01Oooool0
00Cooooo4_ooo`0;oooooa;oo`002ooooolBool000_ooooo4_l00?l;oooooa;oo`002ooooolBo`3o
o`_ooooo4_l00?l;oooooa;o0?l02ooooolBo`3oo`_ooooo4_ooo`05ooooo`000ooooolDoooo00Wo
oooo5?oo0009oooooaCoo`002OoooolDo`00o`Wooooo5?oo0009oooooaCo0?oo2OoooolDo`00o`Wo
oooo5?l0o`09oooooaCo0?oo2OoooolDoooo00Cooooo0003oooooaCoool02OoooolDool000Wooooo
5?oo0009oooooaCo003o2OoooolDool000Wooooo5?l0ool9oooooaCo003o2OoooolDo`3o00Wooooo
5?l0ool9oooooaCoool01?ooool000?ooooo5?ooo`09oooooaCoo`002OoooolDool000Wooooo5?l0
0?l9oooooaCoo`002OoooolDo`3oo`Wooooo5?l00?l9oooooaCo0?l02OoooolDo`3oo`Wooooo5?oo
o`04ooooo`000ooooolDoooo00Wo00005?oo0009o`0001Coo`002Ol0000Do`00o`Wo00005?oo0009
o`0001Co0?oo2Ol0000Do`00o`Wo00005?l0o`09o`0001Co0?oo2Ol0000Doooo00Cooooo0003oooo
oaCoool02OoooolDool000Wooooo5?oo0009oooooaCo003o2OoooolDool000Wooooo5?l0ool9oooo
oaCo003o2OoooolDo`3o00Wooooo5?l0ool9oooooaCoool01?ooool000?ooooo5?ooo`09oooooaCo
o`002OoooolDool000Wooooo5?l00?l9oooooaCoo`002OoooolDo`3oo`Wooooo5?l00?l9oooooaCo
0?l02OoooolDo`3oo`Wooooo5?ooo`04ooooo`001?oooolBoooo00_ooooo4_oo000;oooooa;oo`00
2ooooolBo`00o`_ooooo4_oo000;oooooa;o0?oo2ooooolBo`00o`_ooooo4_l0o`0;oooooa;o0?oo
2ooooolBoooo00Gooooo0004oooooa;oool02ooooolBool000_ooooo4_oo000;oooooa;o003o2ooo
oolBool000_ooooo4_l0ool;oooooa;o003o2ooooolBo`3o00_ooooo4_l0ool;oooooa;oool01Ooo
ool000Gooooo4?ooo`0=oooooa3oo`003Oooool@ool000gooooo4?l00?l=oooooa3oo`003Oooool@
o`3oo`gooooo4?l00?l=oooooa3o0?l03Oooool@o`3oo`gooooo4?ooo`06ooooo`001Oooool@oooo
00gooooo4?oo000=oooooa3oo`003Oooool@o`00o`gooooo4?oo000=oooooa3o0?oo3Oooool@o`00
o`gooooo4?l0o`0=oooooa3o0?oo3Oooool@oooo00Kooooo0006ooooo`koool03oooool>ool000oo
oooo3_oo000?ooooo`ko003o3oooool>ool000oooooo3_l0ool?ooooo`ko003o3oooool>o`3o00oo
oooo3_l0ool?ooooo`koool01oooool000Sooooo2_ooo`0Cooooo`[oo`004oooool:ool001?ooooo
2_l00?lCooooo`[oo`004oooool:o`3ooa?ooooo2_l00?lCooooo`[o0?l04oooool:o`3ooa?ooooo
2_ooo`09ooooo`002_ooool6oooo01Oooooo1_oo000Gooooo`Koo`005oooool6o`00oaOooooo1_oo
000Gooooo`Ko0?oo5oooool6o`00oaOooooo1_l0o`0Gooooo`Ko0?oo5oooool6oooo00_ooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo
000:ooooo`Ko0?oo5oooool6olc<c1Oooooo1_l00?lGooooo`Koool05oooool6ool0oaOooooo1_l0
000Gooooo`Koc<c<5oooool6o`00oaOooooo1_l0000Gooooo`Koo`3o2oooool000Sooooo2_l0oolC
ooooo`[oc<c<4oooool:o`00oa?ooooo2_ooo`0Cooooo`[oo`3o4oooool:o`0001?ooooo2_o<c<`C
ooooo`[o003o4oooool:o`0001?ooooo2_oo0?l9ooooo`001_ooool>o`3oo`oooooo3_o<c<`?oooo
o`ko003o3oooool>oooo00oooooo3_oo0?l?ooooo`ko00003oooool>olc<c0oooooo3_l00?l?oooo
o`ko00003oooool>ool0o`Oooooo0005oooooa3o0?oo3Oooool@olc<c0gooooo4?l00?l=oooooa3o
ool03Oooool@ool0o`gooooo4?l0000=oooooa3oc<c<3Oooool@o`00o`gooooo4?l0000=oooooa3o
o`3o1_ooool000Gooooo4?l0ool=oooooa3oc<c<3Oooool@o`00o`gooooo4?ooo`0=oooooa3oo`3o
3Oooool@o`0000gooooo4?o<c<`=oooooa3o003o3Oooool@o`0000gooooo4?oo0?l6ooooo`001?oo
oolBo`3oo`_ooooo4_o<c<`;oooooa;o003o2ooooolBoooo00_ooooo4_oo0?l;oooooa;o00002ooo
oolBolc<c0_ooooo4_l00?l;oooooa;o00002ooooolBool0o`Gooooo0004oooooa;o0?oo2ooooolB
olc<c0_ooooo4_l00?l;oooooa;oool02ooooolBool0o`_ooooo4_l0000;oooooa;oc<c<2ooooolB
o`00o`_ooooo4_l0000;oooooa;oo`3o1Oooool000?ooooo5?l0ool9oooooaCoc<c<2OoooolDo`00
o`Wooooo5?ooo`09oooooaCoo`3o2OoooolDo`0000Wooooo5?o<c<`9oooooaCo003o2OoooolDo`00
00Wooooo5?oo0?l4ooooo`000ooooolDo`3oo`Wooooo5?o<c<`9oooooaCo003o2OoooolDoooo00Wo
oooo5?oo0?l9oooooaCo00002OoooolDolc<c0Wooooo5?l00?l9oooooaCo00002OoooolDool0o`Co
oooo0003oooooaCo0?oo2Ol0000Dolc<c0Wo00005?l00?l9o`0001Coool02Ol0000Dool0obKo0000
5?o<c<`9o`0001Co003o9_l0000Dool0o`Cooooo0003oooooaCo0?oo2OoooolDolc<c0Wooooo5?l0
0?l9oooooaCoool02OoooolDool0o`Wooooo5?l00009oooooaCoc<c<2OoooolDo`00o`Wooooo5?l0
0009oooooaCoo`3o1?ooool000?ooooo5?l0ool9oooooaCoc<c<2OoooolDo`00o`Wooooo5?ooo`09
oooooaCoo`3o2OoooolDo`0000Wooooo5?o<c<`9oooooaCo003o2OoooolDo`0000Wooooo5?oo0?l4
ooooo`000ooooolDo`3oo`Wooooo5?o<c<`9oooooaCo003o2OoooolDoooo00Wooooo5?oo0?l9oooo
oaCo00002OoooolDolc<c0Wooooo5?l00?l9oooooaCo00002OoooolDool0o`Cooooo0004oooooa;o
0?oo2ooooolBolc<c0_ooooo4_l00?l;oooooa;oool02ooooolBool0o`_ooooo4_l0000;oooooa;o
c<c<2ooooolBo`00o`_ooooo4_l0000;oooooa;oo`3o1Oooool000Cooooo4_l0ool;oooooa;oc<c<
2ooooolBo`00o`_ooooo4_ooo`0;oooooa;oo`3o2ooooolBo`0000_ooooo4_o<c<`;oooooa;o003o
2ooooolBo`0000_ooooo4_oo0?l5ooooo`001Oooool@o`3oo`gooooo4?o<c<`=oooooa3o003o3Ooo
ool@oooo00gooooo4?oo0?l=oooooa3o00003Oooool@olc<c0gooooo4?l00?l=oooooa3o00003Ooo
ool@ool0o`Kooooo0005oooooa3o0?oo3Oooool@olc<c0gooooo4?l00?l=oooooa3oool03Oooool@
ool0o`gooooo4?l0000=oooooa3oc<c<3Oooool@o`00o`gooooo4?l0000=oooooa3oo`3o1_ooool0
00Kooooo3_l0ool?ooooo`koc<c<3oooool>o`00o`oooooo3_ooo`0?ooooo`koo`3o3oooool>o`00
00oooooo3_o<c<`?ooooo`ko003o3oooool>o`0000oooooo3_oo0?l7ooooo`002?ooool:o`3ooa?o
oooo2_o<c<`Cooooo`[o003o4oooool:oooo01?ooooo2_oo0?lCooooo`[o00004oooool:olc<c1?o
oooo2_l00?lCooooo`[o00004oooool:ool0o`Wooooo000:ooooo`Ko0?oo5oooool6olc<c1Oooooo
1_l00?lGooooo`Koool05oooool6ool0oaOooooo1_l0000Gooooo`Koc<c<5oooool6o`00oaOooooo
1_l0000Gooooo`Koo`3o2oooool000gooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Joooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`00
0?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Iooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0
003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000[ooooo1_o<c<`Gooooo`Ko0?oo5oooool6o`00oaOooooo
1_l00?lGooooo`Ko0?l05oooool6o`3ooaOooooo1_ooo`0Gooooo`Koool05oooool6o`3o01Oooooo
1_oo0?l;ooooo`002?ooool:olc<c1?ooooo2_l0oolCooooo`[o003o4oooool:o`00oa?ooooo2_l0
o`0Cooooo`[o0?oo4oooool:oooo01?ooooo2_ooo`0Cooooo`[o0?l04oooool:ool0o`Wooooo0006
ooooo`koc<c<3oooool>o`3oo`oooooo3_l00?l?ooooo`ko003o3oooool>o`3o00oooooo3_l0ool?
ooooo`koool03oooool>oooo00oooooo3_l0o`0?ooooo`koo`3o1oooool000Gooooo4?o<c<`=oooo
oa3o0?oo3Oooool@o`00o`gooooo4?l00?l=oooooa3o0?l03Oooool@o`3oo`gooooo4?ooo`0=oooo
oa3oool03Oooool@o`3o00gooooo4?oo0?l6ooooo`001Oooool@olc<c0gooooo4?l0ool=oooooa3o
003o3Oooool@o`00o`gooooo4?l0o`0=oooooa3o0?oo3Oooool@oooo00gooooo4?ooo`0=oooooa3o
0?l03Oooool@ool0o`Kooooo0004oooooa;oc<c<2ooooolBo`3oo`_ooooo4_l00?l;oooooa;o003o
2ooooolBo`3o00_ooooo4_l0ool;oooooa;oool02ooooolBoooo00_ooooo4_l0o`0;oooooa;oo`3o
1Oooool000Cooooo4_o<c<`;oooooa;o0?oo2ooooolBo`00o`_ooooo4_l00?l;oooooa;o0?l02ooo
oolBo`3oo`_ooooo4_ooo`0;oooooa;oool02ooooolBo`3o00_ooooo4_oo0?l5ooooo`000ooooolD
olc<c0Wooooo5?l0ool9oooooaCo003o2OoooolDo`00o`Wooooo5?l0o`09oooooaCo0?oo2OoooolD
oooo00Wooooo5?ooo`09oooooaCo0?l02OoooolDool0o`Cooooo0003oooooaCoc<c<2OoooolDo`3o
o`Wooooo5?l00?l9oooooaCo003o2OoooolDo`3o00Wooooo5?l0ool9oooooaCoool02OoooolDoooo
00Wooooo5?l0o`09oooooaCoo`3o1?ooool000?ooooo5?o<c<`9o`0001Co0?oo2Ol0000Do`00o`Wo
00005?l00?l9o`0001Co0?l02Ol0000Do`3oo`Wo00005?ooo`09o`0001Coool02Ol0000Do`3o00Wo
00005?oo0?l4ooooo`000ooooolDolc<c0Wooooo5?l0ool9oooooaCo003o2OoooolDo`00o`Wooooo
5?l0o`09oooooaCo0?oo2OoooolDoooo00Wooooo5?ooo`09oooooaCo0?l02OoooolDool0o`Cooooo
0003oooooaCoc<c<2OoooolDo`3oo`Wooooo5?l00?l9oooooaCo003o2OoooolDo`3o00Wooooo5?l0
ool9oooooaCoool02OoooolDoooo00Wooooo5?l0o`09oooooaCoo`3o1?ooool000?ooooo5?o<c<`9
oooooaCo0?oo2OoooolDo`00o`Wooooo5?l00?l9oooooaCo0?l02OoooolDo`3oo`Wooooo5?ooo`09
oooooaCoool02OoooolDo`3o00Wooooo5?oo0?l4ooooo`001?oooolBolc<c0_ooooo4_l0ool;oooo
oa;o003o2ooooolBo`00o`_ooooo4_l0o`0;oooooa;o0?oo2ooooolBoooo00_ooooo4_ooo`0;oooo
oa;o0?l02ooooolBool0o`Gooooo0004oooooa;oc<c<2ooooolBo`3oo`_ooooo4_l00?l;oooooa;o
003o2ooooolBo`3o00_ooooo4_l0ool;oooooa;oool02ooooolBoooo00_ooooo4_l0o`0;oooooa;o
o`3o1Oooool000Gooooo4?o<c<`=oooooa3o0?oo3Oooool@o`00o`gooooo4?l00?l=oooooa3o0?l0
3Oooool@o`3oo`gooooo4?ooo`0=oooooa3oool03Oooool@o`3o00gooooo4?oo0?l6ooooo`001Ooo
ool@olc<c0gooooo4?l0ool=oooooa3o003o3Oooool@o`00o`gooooo4?l0o`0=oooooa3o0?oo3Ooo
ool@oooo00gooooo4?ooo`0=oooooa3o0?l03Oooool@ool0o`Kooooo0006ooooo`koc<c<3oooool>
o`3oo`oooooo3_l00?l?ooooo`ko003o3oooool>o`3o00oooooo3_l0ool?ooooo`koool03oooool>
oooo00oooooo3_l0o`0?ooooo`koo`3o1oooool000Sooooo2_o<c<`Cooooo`[o0?oo4oooool:o`00
oa?ooooo2_l00?lCooooo`[o0?l04oooool:o`3ooa?ooooo2_ooo`0Cooooo`[oool04oooool:o`3o
01?ooooo2_oo0?l9ooooo`002_ooool6olc<c1Oooooo1_l0oolGooooo`Ko003o5oooool6o`00oaOo
oooo1_l0o`0Gooooo`Ko0?oo5oooool6oooo01Oooooo1_ooo`0Gooooo`Ko0?l05oooool6ool0o`_o
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
00cooooo000:ooooo`Ko0?oo5oooool6o`3o01Oooooo1_ooo`0Gooooo`Ko003o5oooool6o`3o01Oo
oooo1_l0o`0Gooooo`Ko00005oooool6ool0oaOooooo1_l0o`0Gooooo`Koool02oooool000Sooooo
2_l0oolCooooo`[o0?l04oooool:oooo01?ooooo2_l00?lCooooo`[o0?l04oooool:o`3o01?ooooo
2_l0000Cooooo`[oo`3o4oooool:o`3o01?ooooo2_ooo`09ooooo`001_ooool>o`3oo`oooooo3_l0
o`0?ooooo`koool03oooool>o`00o`oooooo3_l0o`0?ooooo`ko0?l03oooool>o`0000oooooo3_oo
0?l?ooooo`ko0?l03oooool>oooo00Oooooo0005oooooa3o0?oo3Oooool@o`3o00gooooo4?ooo`0=
oooooa3o003o3Oooool@o`3o00gooooo4?l0o`0=oooooa3o00003Oooool@ool0o`gooooo4?l0o`0=
oooooa3oool01_ooool000Gooooo4?l0ool=oooooa3o0?l03Oooool@oooo00gooooo4?l00?l=oooo
oa3o0?l03Oooool@o`3o00gooooo4?l0000=oooooa3oo`3o3Oooool@o`3o00gooooo4?ooo`06oooo
o`001?oooolBo`3oo`_ooooo4_l0o`0;oooooa;oool02ooooolBo`00o`_ooooo4_l0o`0;oooooa;o
0?l02ooooolBo`0000_ooooo4_oo0?l;oooooa;o0?l02ooooolBoooo00Gooooo0004oooooa;o0?oo
2ooooolBo`3o00_ooooo4_ooo`0;oooooa;o003o2ooooolBo`3o00_ooooo4_l0o`0;oooooa;o0000
2ooooolBool0o`_ooooo4_l0o`0;oooooa;oool01Oooool000?ooooo5?l0ool9oooooaCo0?l02Ooo
oolDoooo00Wooooo5?l00?l9oooooaCo0?l02OoooolDo`3o00Wooooo5?l00009oooooaCoo`3o2Ooo
oolDo`3o00Wooooo5?ooo`04ooooo`000ooooolDo`3oo`Wooooo5?l0o`09oooooaCoool02OoooolD
o`00o`Wooooo5?l0o`09oooooaCo0?l02OoooolDo`0000Wooooo5?oo0?l9oooooaCo0?l02OoooolD
oooo00Cooooo0003oooooaCo0?oo2Ol0000Do`3o00Wo00005?ooo`09o`0001Co003o2Ol0000Do`3o
00Wo00005?l0o`0Vo`0001Coo`3o2Ol0000Do`3o00Wo00005?ooo`04ooooo`000ooooolDo`3oo`Wo
oooo5?l0o`09oooooaCoool02OoooolDo`00o`Wooooo5?l0o`09oooooaCo0?l02OoooolDo`0000Wo
oooo5?oo0?l9oooooaCo0?l02OoooolDoooo00Cooooo0003oooooaCo0?oo2OoooolDo`3o00Wooooo
5?ooo`09oooooaCo003o2OoooolDo`3o00Wooooo5?l0o`09oooooaCo00002OoooolDool0o`Wooooo
5?l0o`09oooooaCoool01?ooool000?ooooo5?l0ool9oooooaCo0?l02OoooolDoooo00Wooooo5?l0
0?l9oooooaCo0?l02OoooolDo`3o00Wooooo5?l00009oooooaCoo`3o2OoooolDo`3o00Wooooo5?oo
o`04ooooo`001?oooolBo`3oo`_ooooo4_l0o`0;oooooa;oool02ooooolBo`00o`_ooooo4_l0o`0;
oooooa;o0?l02ooooolBo`0000_ooooo4_oo0?l;oooooa;o0?l02ooooolBoooo00Gooooo0004oooo
oa;o0?oo2ooooolBo`3o00_ooooo4_ooo`0;oooooa;o003o2ooooolBo`3o00_ooooo4_l0o`0;oooo
oa;o00002ooooolBool0o`_ooooo4_l0o`0;oooooa;oool01Oooool000Gooooo4?l0ool=oooooa3o
0?l03Oooool@oooo00gooooo4?l00?l=oooooa3o0?l03Oooool@o`3o00gooooo4?l0000=oooooa3o
o`3o3Oooool@o`3o00gooooo4?ooo`06ooooo`001Oooool@o`3oo`gooooo4?l0o`0=oooooa3oool0
3Oooool@o`00o`gooooo4?l0o`0=oooooa3o0?l03Oooool@o`0000gooooo4?oo0?l=oooooa3o0?l0
3Oooool@oooo00Kooooo0006ooooo`ko0?oo3oooool>o`3o00oooooo3_ooo`0?ooooo`ko003o3ooo
ool>o`3o00oooooo3_l0o`0?ooooo`ko00003oooool>ool0o`oooooo3_l0o`0?ooooo`koool01ooo
ool000Sooooo2_l0oolCooooo`[o0?l04oooool:oooo01?ooooo2_l00?lCooooo`[o0?l04oooool:
o`3o01?ooooo2_l0000Cooooo`[oo`3o4oooool:o`3o01?ooooo2_ooo`09ooooo`002_ooool6o`3o
oaOooooo1_l0o`0Gooooo`Koool05oooool6o`00oaOooooo1_l0o`0Gooooo`Ko0?l05oooool6o`00
01Oooooo1_oo0?lGooooo`Ko0?l05oooool6oooo00_ooooo000=ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o
0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0J
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000:ooooo`Ko00005oooool6oooo
01Oooooo1_l0000Gooooo`Koool05oooool6oooo01Oooooo1_l0oolGooooo`Ko003o5oooool6o`3o
01Oooooo1_oo000Gooooo`Koo`002oooool000Sooooo2_l0000Cooooo`[oool04oooool:o`0001?o
oooo2_ooo`0Cooooo`[oool04oooool:o`3ooa?ooooo2_l00?lCooooo`[o0?l04oooool:ool001?o
oooo2_oo0009ooooo`001_ooool>o`0000oooooo3_ooo`0?ooooo`ko00003oooool>oooo00oooooo
3_ooo`0?ooooo`ko0?oo3oooool>o`00o`oooooo3_l0o`0?ooooo`koo`003oooool>ool000Oooooo
0005oooooa3o00003Oooool@oooo00gooooo4?l0000=oooooa3oool03Oooool@oooo00gooooo4?l0
ool=oooooa3o003o3Oooool@o`3o00gooooo4?oo000=oooooa3oo`001_ooool000Gooooo4?l0000=
oooooa3oool03Oooool@o`0000gooooo4?ooo`0=oooooa3oool03Oooool@o`3oo`gooooo4?l00?l=
oooooa3o0?l03Oooool@ool000gooooo4?oo0006ooooo`001?oooolBo`0000_ooooo4_ooo`0;oooo
oa;o00002ooooolBoooo00_ooooo4_ooo`0;oooooa;o0?oo2ooooolBo`00o`_ooooo4_l0o`0;oooo
oa;oo`002ooooolBool000Gooooo0004oooooa;o00002ooooolBoooo00_ooooo4_l0000;oooooa;o
ool02ooooolBoooo00_ooooo4_l0ool;oooooa;o003o2ooooolBo`3o00_ooooo4_oo000;oooooa;o
o`001Oooool000?ooooo5?l00009oooooaCoool02OoooolDo`0000Wooooo5?ooo`09oooooaCoool0
2OoooolDo`3oo`Wooooo5?l00?l9oooooaCo0?l02OoooolDool000Wooooo5?oo0004ooooo`000ooo
oolDo`0000Wooooo5?ooo`09oooooaCo00002OoooolDoooo00Wooooo5?ooo`09oooooaCo0?oo2Ooo
oolDo`00o`Wooooo5?l0o`09oooooaCoo`002OoooolDool000Cooooo0003oooooago00005?ooo`0V
o`0001Coool02Ol0000Doooo00Wo00005?l0ool9o`0001Co003o2Ol0000Do`3o00Wo00005?oo0009
o`0001Coo`001?ooool000?ooooo5?l00009oooooaCoool02OoooolDo`0000Wooooo5?ooo`09oooo
oaCoool02OoooolDo`3oo`Wooooo5?l00?l9oooooaCo0?l02OoooolDool000Wooooo5?oo0004oooo
o`000ooooolDo`0000Wooooo5?ooo`09oooooaCo00002OoooolDoooo00Wooooo5?ooo`09oooooaCo
0?oo2OoooolDo`00o`Wooooo5?l0o`09oooooaCoo`002OoooolDool000Cooooo0003oooooaCo0000
2OoooolDoooo00Wooooo5?l00009oooooaCoool02OoooolDoooo00Wooooo5?l0ool9oooooaCo003o
2OoooolDo`3o00Wooooo5?oo0009oooooaCoo`001?ooool000Cooooo4_l0000;oooooa;oool02ooo
oolBo`0000_ooooo4_ooo`0;oooooa;oool02ooooolBo`3oo`_ooooo4_l00?l;oooooa;o0?l02ooo
oolBool000_ooooo4_oo0005ooooo`001?oooolBo`0000_ooooo4_ooo`0;oooooa;o00002ooooolB
oooo00_ooooo4_ooo`0;oooooa;o0?oo2ooooolBo`00o`_ooooo4_l0o`0;oooooa;oo`002ooooolB
ool000Gooooo0005oooooa3o00003Oooool@oooo00gooooo4?l0000=oooooa3oool03Oooool@oooo
00gooooo4?l0ool=oooooa3o003o3Oooool@o`3o00gooooo4?oo000=oooooa3oo`001_ooool000Go
oooo4?l0000=oooooa3oool03Oooool@o`0000gooooo4?ooo`0=oooooa3oool03Oooool@o`3oo`go
oooo4?l00?l=oooooa3o0?l03Oooool@ool000gooooo4?oo0006ooooo`001_ooool>o`0000oooooo
3_ooo`0?ooooo`ko00003oooool>oooo00oooooo3_ooo`0?ooooo`ko0?oo3oooool>o`00o`oooooo
3_l0o`0?ooooo`koo`003oooool>ool000Oooooo0008ooooo`[o00004oooool:oooo01?ooooo2_l0
000Cooooo`[oool04oooool:oooo01?ooooo2_l0oolCooooo`[o003o4oooool:o`3o01?ooooo2_oo
000Cooooo`[oo`002Oooool000[ooooo1_l0000Gooooo`Koool05oooool6o`0001Oooooo1_ooo`0G
ooooo`Koool05oooool6o`3ooaOooooo1_l00?lGooooo`Ko0?l05oooool6ool001Oooooo1_oo000;
ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool0
6_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oo
oooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01Wooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo
01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003ooooooooo
o`0<ooooo`002_ooool6o`3o01Oooooo1_ooo`0Gooooo`Koool05oooool6o`3ooaOooooo1_oo0?lG
ooooo`Ko0?l05oooool6ool0oaOooooo1_o<c<`Gooooo`Ko003o5oooool6o`0000_ooooo0008oooo
o`[o0?l04oooool:oooo01?ooooo2_ooo`0Cooooo`[o0?oo4oooool:ool0oa?ooooo2_l0o`0Coooo
o`[oo`3o4oooool:olc<c1?ooooo2_l00?lCooooo`[o00002Oooool000Kooooo3_l0o`0?ooooo`ko
ool03oooool>oooo00oooooo3_l0ool?ooooo`koo`3o3oooool>o`3o00oooooo3_oo0?l?ooooo`ko
c<c<3oooool>o`00o`oooooo3_l00007ooooo`001Oooool@o`3o00gooooo4?ooo`0=oooooa3oool0
3Oooool@o`3oo`gooooo4?oo0?l=oooooa3o0?l03Oooool@ool0o`gooooo4?o<c<`=oooooa3o003o
3Oooool@o`0000Kooooo0005oooooa3o0?l03Oooool@oooo00gooooo4?ooo`0=oooooa3o0?oo3Ooo
ool@ool0o`gooooo4?l0o`0=oooooa3oo`3o3Oooool@olc<c0gooooo4?l00?l=oooooa3o00001_oo
ool000Cooooo4_l0o`0;oooooa;oool02ooooolBoooo00_ooooo4_l0ool;oooooa;oo`3o2ooooolB
o`3o00_ooooo4_oo0?l;oooooa;oc<c<2ooooolBo`00o`_ooooo4_l00005ooooo`001?oooolBo`3o
00_ooooo4_ooo`0;oooooa;oool02ooooolBo`3oo`_ooooo4_oo0?l;oooooa;o0?l02ooooolBool0
o`_ooooo4_o<c<`;oooooa;o003o2ooooolBo`0000Gooooo0003oooooaCo0?l02OoooolDoooo00Wo
oooo5?ooo`09oooooaCo0?oo2OoooolDool0o`Wooooo5?l0o`09oooooaCoo`3o2OoooolDolc<c0Wo
oooo5?l00?l9oooooaCo00001?ooool000?ooooo5?l0o`09oooooaCoool02OoooolDoooo00Wooooo
5?l0ool9oooooaCoo`3o2OoooolDo`3o00Wooooo5?oo0?l9oooooaCoc<c<2OoooolDo`00o`Wooooo
5?l00004ooooo`000ooooolDo`3o00Wo00005?ooo`09o`0001Coool02Ol0000Do`3oo`Wo00005?oo
0?l9o`0001Co0?l02Ol0000Dool0o`Wo00005?o<c<`9o`0001Co003o7Ol00004ooooo`000ooooolD
o`3o00Wooooo5?ooo`09oooooaCoool02OoooolDo`3oo`Wooooo5?oo0?l9oooooaCo0?l02OoooolD
ool0o`Wooooo5?o<c<`9oooooaCo003o2OoooolDo`0000Cooooo0003oooooaCo0?l02OoooolDoooo
00Wooooo5?ooo`09oooooaCo0?oo2OoooolDool0o`Wooooo5?l0o`09oooooaCoo`3o2OoooolDolc<
c0Wooooo5?l00?l9oooooaCo00001?ooool000?ooooo5?l0o`09oooooaCoool02OoooolDoooo00Wo
oooo5?l0ool9oooooaCoo`3o2OoooolDo`3o00Wooooo5?oo0?l9oooooaCoc<c<2OoooolDo`00o`Wo
oooo5?l00004ooooo`001?oooolBo`3o00_ooooo4_ooo`0;oooooa;oool02ooooolBo`3oo`_ooooo
4_oo0?l;oooooa;o0?l02ooooolBool0o`_ooooo4_o<c<`;oooooa;o003o2ooooolBo`0000Gooooo
0004oooooa;o0?l02ooooolBoooo00_ooooo4_ooo`0;oooooa;o0?oo2ooooolBool0o`_ooooo4_l0
o`0;oooooa;oo`3o2ooooolBolc<c0_ooooo4_l00?l;oooooa;o00001Oooool000Gooooo4?l0o`0=
oooooa3oool03Oooool@oooo00gooooo4?l0ool=oooooa3oo`3o3Oooool@o`3o00gooooo4?oo0?l=
oooooa3oc<c<3Oooool@o`00o`gooooo4?l00006ooooo`001Oooool@o`3o00gooooo4?ooo`0=oooo
oa3oool03Oooool@o`3oo`gooooo4?oo0?l=oooooa3o0?l03Oooool@ool0o`gooooo4?o<c<`=oooo
oa3o003o3Oooool@o`0000Kooooo0006ooooo`ko0?l03oooool>oooo00oooooo3_ooo`0?ooooo`ko
0?oo3oooool>ool0o`oooooo3_l0o`0?ooooo`koo`3o3oooool>olc<c0oooooo3_l00?l?ooooo`ko
00001oooool000Sooooo2_l0o`0Cooooo`[oool04oooool:oooo01?ooooo2_l0oolCooooo`[oo`3o
4oooool:o`3o01?ooooo2_oo0?lCooooo`[oc<c<4oooool:o`00oa?ooooo2_l00009ooooo`002_oo
ool6o`3o01Oooooo1_ooo`0Gooooo`Koool05oooool6o`3ooaOooooo1_oo0?lGooooo`Ko0?l05ooo
ool6ool0oaOooooo1_o<c<`Gooooo`Ko003o5oooool6o`0000_ooooo003oooooob7ooooo003ooooo
ob7ooooo003oooooob7ooooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[ugVN]]; \[IndentingNewLine]Evolve[
          ugVN], {25}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[58]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowGraphArray[\[IndentingNewLine]Partition[l, 5], 
      VertexStyle \[Rule] Disk[Large]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[60]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.0238095 0.190476 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.200176 0.200176 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
0 0 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 1 1 r
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 1 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
0 1 1 r
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
0 1 1 r
.85354 .75253 Mdot
0 0 0 r
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.0238095 0.39418 0.200176 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
0 1 1 r
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
0 0 0 r
.14646 .44949 Mdot
0 1 1 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
0 1 1 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
0 1 1 r
.75253 .75253 Mdot
0 0 0 r
.85354 .75253 Mdot
0 1 1 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.0238095 0.588183 0.200176 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 1 1 r
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
0 1 1 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
0 1 1 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 1 1 r
.75253 .85354 Mdot
0 0 0 r
.85354 .85354 Mdot
0 1 1 r
.95455 .85354 Mdot
0 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.0238095 0.782187 0.200176 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
0 1 1 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
0 1 1 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
0 1 1 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
0 1 1 r
.65152 .85354 Mdot
.75253 .85354 Mdot
0 0 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
0 1 1 r
.75253 .95455 Mdot
0 0 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.0238095 0.97619 0.200176 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
0 1 1 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
0 1 1 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
0 1 1 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
0 1 1 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
0 1 1 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
0 0 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
0 1 1 r
.75253 .95455 Mdot
.85354 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.217813 0.200176 0.39418 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
0 0 0 r
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
0 1 1 r
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 1 r
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
0 1 1 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.217813 0.39418 0.39418 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 0 0 r
.04545 .14646 Mdot
0 1 1 r
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
0 1 1 r
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 1 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 1 r
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
0 1 1 r
.95455 .85354 Mdot
0 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.217813 0.588183 0.39418 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
0 1 1 r
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
0 0 0 r
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
0 0 0 r
.24747 .24747 Mdot
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
0 1 1 r
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 1 r
.44949 .55051 Mdot
.55051 .55051 Mdot
0 0 0 r
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
0 1 1 r
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
0 1 1 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
0 1 1 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.217813 0.782187 0.39418 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 0 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
0 0 0 r
.24747 .14646 Mdot
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
0 1 1 r
.14646 .55051 Mdot
0 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
0 1 1 r
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.217813 0.97619 0.39418 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 0 0 r
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
0 0 0 r
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
0 0 0 r
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
0 1 1 r
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 1 r
.85354 .65152 Mdot
0 0 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.411817 0.200176 0.588183 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
0 1 1 r
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
0 0 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 0 0 r
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
0 0 0 r
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
0 0 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
0 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 1 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
0 0 0 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.411817 0.39418 0.588183 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
0 1 1 r
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 0 0 r
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
0 1 1 r
.75253 .65152 Mdot
0 0 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
0 0 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
0 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
0 0 0 r
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.411817 0.588183 0.588183 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 1 r
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
0 1 1 r
.75253 .65152 Mdot
.85354 .65152 Mdot
0 0 0 r
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
0 0 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
0 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.411817 0.782187 0.588183 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
0 0 0 r
.14646 .24747 Mdot
0 1 1 r
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 1 r
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
0 1 1 r
.85354 .75253 Mdot
0 0 0 r
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
0 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.411817 0.97619 0.588183 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
0 0 0 r
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 0 0 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 0 r
.04545 .34848 Mdot
.14646 .34848 Mdot
0 1 1 r
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
0 0 0 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
0 1 1 r
.55051 .55051 Mdot
0 0 0 r
.65152 .55051 Mdot
0 1 1 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 0 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 1 r
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
0 1 1 r
.85354 .75253 Mdot
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
0 0 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.60582 0.200176 0.782187 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 1 r
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
0 1 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
0 1 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
0 1 1 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 0 0 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
0 1 1 r
.65152 .44949 Mdot
0 0 0 r
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
0 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
0 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
1 1 0 r
.55051 .75253 Mdot
0 0 0 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 0 0 r
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
0 0 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.60582 0.39418 0.782187 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
0 1 1 r
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
1 1 0 r
.14646 .14646 Mdot
.24747 .14646 Mdot
0 1 0 r
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
0 1 0 r
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
0 1 0 r
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
0 0 0 r
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 1 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
0 0 0 r
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
0 0 0 r
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
0 0 0 r
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
0 0 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
1 1 0 r
.55051 .75253 Mdot
0 0 0 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
0 0 0 r
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 1 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
0 0 0 r
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.60582 0.588183 0.782187 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
1 1 0 r
.14646 .14646 Mdot
0 1 0 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
0 1 1 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
0 1 0 r
.14646 .24747 Mdot
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
0 1 1 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
0 0 0 r
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 1 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
0 1 1 r
.65152 .44949 Mdot
.75253 .44949 Mdot
0 0 0 r
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
0 0 0 r
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
0 0 0 r
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
0 0 0 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 1 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
0 0 0 r
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.60582 0.782187 0.782187 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
0 1 0 r
.14646 .14646 Mdot
1 1 0 r
.24747 .14646 Mdot
0 1 1 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
0 1 1 r
.14646 .24747 Mdot
0 1 0 r
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 1 r
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 0 0 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
0 0 0 r
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
0 0 0 r
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
0 0 0 r
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
0 0 0 r
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
0 0 0 r
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 1 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
0 0 0 r
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.60582 0.97619 0.782187 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 1 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
1 1 0 r
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
0 1 1 r
.24747 .24747 Mdot
0 0 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
0 1 1 r
.14646 .34848 Mdot
0 0 0 r
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 1 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 0 0 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
0 0 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
0 1 1 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
0 0 0 r
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
0 0 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
0 0 0 r
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 1 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 0 0 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.799824 0.200176 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 0 r
.24747 .04545 Mdot
1 0 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
1 0 1 r
.55051 .04545 Mdot
.8 .8 .8 r
.65152 .04545 Mdot
0 1 1 r
.75253 .04545 Mdot
1 1 0 r
.85354 .04545 Mdot
0 1 0 r
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
0 1 0 r
.14646 .14646 Mdot
0 0 1 r
.24747 .14646 Mdot
0 1 0 r
.34848 .14646 Mdot
1 0 1 r
.44949 .14646 Mdot
.8 .8 .8 r
.55051 .14646 Mdot
1 0 1 r
.65152 .14646 Mdot
.75253 .14646 Mdot
0 1 0 r
.85354 .14646 Mdot
1 0 0 r
.95455 .14646 Mdot
.8 .8 .8 r
.04545 .24747 Mdot
.14646 .24747 Mdot
0 0 1 r
.24747 .24747 Mdot
.8 .8 .8 r
.34848 .24747 Mdot
1 1 0 r
.44949 .24747 Mdot
.8 .8 .8 r
.55051 .24747 Mdot
1 0 1 r
.65152 .24747 Mdot
0 1 1 r
.75253 .24747 Mdot
.85354 .24747 Mdot
0 0 0 r
.95455 .24747 Mdot
0 1 1 r
.04545 .34848 Mdot
1 1 0 r
.14646 .34848 Mdot
0 0 0 r
.24747 .34848 Mdot
.8 .8 .8 r
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.8 .8 .8 r
.65152 .34848 Mdot
.75253 .34848 Mdot
1 0 1 r
.85354 .34848 Mdot
1 1 0 r
.95455 .34848 Mdot
.04545 .44949 Mdot
1 0 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 1 r
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
0 1 1 r
.55051 .44949 Mdot
0 0 1 r
.65152 .44949 Mdot
0 1 0 r
.75253 .44949 Mdot
0 1 1 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.8 .8 .8 r
.14646 .55051 Mdot
0 0 1 r
.24747 .55051 Mdot
1 1 0 r
.34848 .55051 Mdot
1 0 1 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.8 .8 .8 r
.65152 .55051 Mdot
0 0 1 r
.75253 .55051 Mdot
0 0 0 r
.85354 .55051 Mdot
1 0 1 r
.95455 .55051 Mdot
.8 .8 .8 r
.04545 .65152 Mdot
0 1 1 r
.14646 .65152 Mdot
0 0 1 r
.24747 .65152 Mdot
.34848 .65152 Mdot
0 1 0 r
.44949 .65152 Mdot
0 1 1 r
.55051 .65152 Mdot
1 1 0 r
.65152 .65152 Mdot
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
1 0 1 r
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
0 1 0 r
.14646 .75253 Mdot
1 1 0 r
.24747 .75253 Mdot
0 0 1 r
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
0 0 0 r
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
1 1 0 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
0 0 0 r
.24747 .85354 Mdot
1 1 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
0 1 1 r
.55051 .85354 Mdot
0 0 1 r
.65152 .85354 Mdot
0 1 0 r
.75253 .85354 Mdot
1 0 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 0 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
0 1 1 r
.34848 .95455 Mdot
1 0 1 r
.44949 .95455 Mdot
0 1 0 r
.55051 .95455 Mdot
1 0 1 r
.65152 .95455 Mdot
.8 .8 .8 r
.75253 .95455 Mdot
0 0 1 r
.85354 .95455 Mdot
0 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.217813 0.799824 0.39418 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
1 0 0 r
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
0 1 0 r
.14646 .14646 Mdot
0 0 1 r
.24747 .14646 Mdot
0 1 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.8 .8 .8 r
.55051 .14646 Mdot
.65152 .14646 Mdot
0 1 1 r
.75253 .14646 Mdot
.85354 .14646 Mdot
1 0 0 r
.95455 .14646 Mdot
1 1 0 r
.04545 .24747 Mdot
0 0 1 r
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
0 1 0 r
.44949 .24747 Mdot
.55051 .24747 Mdot
0 1 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
1 1 0 r
.04545 .34848 Mdot
1 0 0 r
.14646 .34848 Mdot
0 0 1 r
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.8 .8 .8 r
.75253 .34848 Mdot
0 1 1 r
.85354 .34848 Mdot
1 1 0 r
.95455 .34848 Mdot
0 1 1 r
.04545 .44949 Mdot
1 0 0 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 1 1 r
.44949 .44949 Mdot
0 1 0 r
.55051 .44949 Mdot
0 1 1 r
.65152 .44949 Mdot
0 0 1 r
.75253 .44949 Mdot
0 0 0 r
.85354 .44949 Mdot
1 1 0 r
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
1 0 0 r
.24747 .55051 Mdot
0 0 1 r
.34848 .55051 Mdot
0 0 0 r
.44949 .55051 Mdot
0 1 1 r
.55051 .55051 Mdot
0 0 0 r
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
0 0 1 r
.24747 .65152 Mdot
.34848 .65152 Mdot
1 0 1 r
.44949 .65152 Mdot
0 0 0 r
.55051 .65152 Mdot
1 1 0 r
.65152 .65152 Mdot
1 0 1 r
.75253 .65152 Mdot
0 0 0 r
.85354 .65152 Mdot
0 1 0 r
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
0 1 0 r
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
1 0 1 r
.85354 .75253 Mdot
0 1 0 r
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
0 1 0 r
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
1 0 0 r
.95455 .85354 Mdot
1 1 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
1 0 1 r
.34848 .95455 Mdot
0 1 0 r
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
1 0 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.411817 0.799824 0.588183 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
1 0 0 r
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
0 1 0 r
.34848 .14646 Mdot
.8 .8 .8 r
.44949 .14646 Mdot
0 1 0 r
.55051 .14646 Mdot
0 1 1 r
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
1 1 0 r
.04545 .24747 Mdot
1 0 0 r
.14646 .24747 Mdot
0 0 1 r
.24747 .24747 Mdot
0 1 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
0 1 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
1 0 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
0 1 0 r
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
0 1 1 r
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
1 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
0 0 0 r
.44949 .44949 Mdot
0 1 0 r
.55051 .44949 Mdot
0 0 1 r
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
0 0 0 r
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
1 0 0 r
.24747 .55051 Mdot
0 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
0 0 1 r
.34848 .65152 Mdot
0 0 0 r
.44949 .65152 Mdot
0 1 1 r
.55051 .65152 Mdot
0 0 0 r
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
1 0 0 r
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
0 1 0 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
1 1 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 1 0 r
.55051 .95455 Mdot
.65152 .95455 Mdot
1 0 0 r
.75253 .95455 Mdot
0 1 0 r
.85354 .95455 Mdot
1 0 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.60582 0.799824 0.782187 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
0 1 0 r
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
1 1 0 r
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
0 1 0 r
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
0 1 1 r
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
1 0 0 r
.24747 .34848 Mdot
.34848 .34848 Mdot
0 1 0 r
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
0 1 1 r
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
1 0 0 r
.44949 .44949 Mdot
0 0 0 r
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
0 1 1 r
.85354 .44949 Mdot
0 0 0 r
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
0 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
0 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
1 1 0 r
.55051 .75253 Mdot
1 0 1 r
.65152 .75253 Mdot
.75253 .75253 Mdot
0 0 0 r
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
1 0 1 r
.75253 .85354 Mdot
0 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
0 1 0 r
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.799824 0.799824 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
1 1 0 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
0 1 1 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
1 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
0 1 1 r
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
1 0 0 r
.24747 .24747 Mdot
.34848 .24747 Mdot
0 1 0 r
.44949 .24747 Mdot
0 1 1 r
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
0 0 0 r
.34848 .34848 Mdot
1 0 0 r
.44949 .34848 Mdot
0 0 0 r
.55051 .34848 Mdot
0 1 1 r
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
0 0 0 r
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
0 1 1 r
.75253 .44949 Mdot
0 0 0 r
.85354 .44949 Mdot
.95455 .44949 Mdot
0 1 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
0 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
0 1 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
0 0 0 r
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
0 1 1 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
1 1 0 r
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
0 0 0 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 1 1 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
1 0 1 r
.75253 .85354 Mdot
0 0 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
0 1 1 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
1 0 1 r
.75253 .95455 Mdot
0 1 0 r
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[60]:=",
  ImageSize->{440, 440},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006h0001^2000`400?l00000o`00003oooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`003?ooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo
1Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`_ooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo
1?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol00004ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002ooo
ool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol00003ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5o`3oo`?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5o`3oo`_o00001Ol0ool;o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool9ooooo`So00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool:o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool:
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2
o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool=
ooooo`002oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool2oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol00003oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo2_ooool5o`3oo`?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`003?ooool3o`3oo`Gooooo0ol00005ooooo`?o
0?oo1Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`_ooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o
0?oo1?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol00004ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o
0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`00
3Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
3?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0=ooooo`003?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`_ooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3o
o`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5o`3oo`?o
00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`Wooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool:ooooo`Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`?o
00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o
00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o
00001Ol0ool=ooooo`002oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0_ooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool:ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0
ool=ooooo`003?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_ooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`003Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Ooo
ool000_ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool9ooooo`Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo0ol00005
o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`co
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5oooo
o`?o0?oo3_ooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo1Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool9ooooo`Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_ooool5o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo
3Oooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool0
00gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool;ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool<ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol00003
ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo2_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00003
ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3
ooooo`Go0?oo3Oooool000_ooooo6?l00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3o
o`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`Wooooo1Ol0ool;o`0000Go0?oo0ol00005o`3o
o`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`[ooooo5ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`[ooooo3ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:ooooo`So00001Ol0ool3o`00
00Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`00
00Go0?oo0ol00005o`3oo`gooooo000;ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo2Oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00002ooooo`Go
00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo2_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000<ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0000
1Oooool3o`0000Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Ooo
ool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`3oo`Cooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo2oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`co
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Go
oooo0ol00004ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`ko
oooo000;ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo
1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo
1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000;ooooob3o00001Ol0ool:o`0000Go0?oo0ol0
0005o`3oo`?o00001Ol0ool3o`0000Go0?oo2Ooooolgo`0000Go0?oo0ol00005o`3oo`?o00001Ol0
ool:oooooaOo00001Ol0ool3o`0000Go0?oo4_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_oo
oolGo`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0
ool3o`0000Go0?oo2_ooool@o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol0
0005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`gooooo000;ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo1Ol00003ooooo`Go0?oo0oooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`[ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5
o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5
o`3oo`gooooo000<ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3o
o`Cooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00004ooooo`?o00001Oooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00004ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`00
00Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<
ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4
ooooo`?o0?oo1Oooool3o`3oo`kooooo000;ooooocOo00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`Wooooo;ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo;ol00005o`3o
o`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo;ol00005o`3oo`?o00001Ol0ool2o`00
00Go0?oo0ol00005o`3oo`[ooooo;ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3o
o`gooooo000;ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;o
oooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?o
oooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?o
oooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00002ooooo`Go00000oooool5o`0000?o
oooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?o
oooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?o
oooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?o
oooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000;ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo
1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo
000<ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool<ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=oooo
o`003?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`0000_ooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o
0?oo1Oooool3o`0000Gooooo0ol0ool<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo3?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005ooooo`?o
00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o
0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo
0oooool5o`0000Wooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool:ooooo`Go0000
0_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo
0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00002ooooo`Go0000
0oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo
0oooool5o`3oo`[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002ooo
ooloo`0000Go0?oo2?l00009ooooocOo00001Ol0ool;o`0000Go0?oo2_ooool_o`0000Go0?oo0_l0
0005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool_o`0000Go0?oo0ol00005o`3oo`;o00001Ol0
ool3o`0000Go0?oo2_ooool_o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Ooo
ool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol00009ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5
o`3oo`?ooooo1Ol00003ooooo`Go0?oo2_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5
o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3o
o`Gooooo0ol0000;ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo3?ooool3o`00
00Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005ooooo`?o00001?ooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0ool5
ooooo`?o00001Oooool3o`3oo`cooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`0000Gooooo0ol0000<
ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o00003_ooool000_ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00009ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00002_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go00000oooool5o`3oo`[ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3oooo
o`Go0?oo0_ooool5o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go00003Oooool000_oooooC?l00009ooooodco00002_oooolfo`0000Go0?oo2ol00005o`3oo`[o
oooo;ol00005o`3oo`?o00001Ol0ool?o`0000[ooooo;ol00005o`3oo`?o00001Ol0ool2o`0000Go
0?oo2?l0000=ooooo`002oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000Wooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go00000_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo
1Ol00003ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000[ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0000=ooooo`003?ooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000_ooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
000<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo3?ooool3o`0000Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o00001Oooool3o`0000cooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
000>ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000_ooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o0000
1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Gooooo0ol00004ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o0000
1Oooool3o`0000cooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0000>ooooo`002ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000Wooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol0000:ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00002_oo
ool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0000=ooooo`002ooooom<o`0000WoooooC?l0000:oooood_o00002_oooolg
o`0000Go0?oo3ol0000:ooooocOo00001Ol0ool2o`0000Go0?oo2?l0000=ooooo`002oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000Wooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol0000:ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`00
00?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000oooool5o`00
00;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0000=ooooo`003?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000_o
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o00001?ooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol00005ooooo`?o00003?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o00001Oooool3o`0000co
oooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0000>ooooo`00ooooooniooooo`00oooo
ooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00oooo
ooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`003?ooool3o`3oo`Gooooo
0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_ooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool<ooooo`?o00001?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo
0ol0ool4ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Cooooo
0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo
0ol0ool>ooooo`002oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol0
0003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5o`3oo`_o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo2Oooool@o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo2_ooool5o`3oo`?o00001Ol0ool2o`0000Go0?oo2ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`?o00001Ol0oolB
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool=
ooooo`002oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol00003oooo
o`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3oooo
o`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`003?ooool3o`3oo`Gooooo0ol00005ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`_ooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o
00001?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o
0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o
0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Cooooo0ol00005ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`00
3Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
3?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0=ooooo`003?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`_ooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool5ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o00001Oooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3o
o`cooooo0ol0ool5ooooo`?o00001Oooool3o`3oo`Cooooo0ol00005ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool@o`0000Go
0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go
0?oo0ol00005o`3oo`Wooooo2?l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`;o
00001Ol0ool3o`0000Go0?oo2ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o
00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`?o00001Ol0oolBo`0000Go0?oo0ol00005o`3oo`?o
00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo2ol00005o`3oo`[o
00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`go
oooo000;ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo
1Ol00003ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000;ooooo`Go00000oooool5o`0000?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo2Oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0
0003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0
ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000<
ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`0000Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<
ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol00005
ooooo`?o0?oo1?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4
ooooo`?o0?oo1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`00
3?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_ooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?o0000
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`Wooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`Wooooo1Ol0ool3o`0000Go0?oo0ol00005
o`3oo`_o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`[ooooo1Ol0ool2o`0000Go0?oo4ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`?o00001Ol0ool2o`0000Go0?oo2ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo2_ooool5
o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`gooooo000;ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`00
00?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`[ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3o
o`gooooo000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`0000Go
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o00001Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<oooo
o`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3o`0000Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4oooo
o`?o0?oo1Oooool3o`3oo`kooooo000;ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
00000oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo2Oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo2_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000;ooooo`So0000
1Ol0ool3o`0000Go0?oo2ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o0000
1Ol0ool3o`0000Go0?oo2Oooool8o`0000Go0?oo0ol00005o`3oo`_o00001Ol0ool3o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool2o`0000Go0?oo
0ol00005o`3oo`_o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo
0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol00003ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool9ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Ooo
ool000cooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o00001Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00go
oooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo3?ooool3o`0000Cooooo0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0
ool5ooooo`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`3oo`Gooooo0ol0
0004ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0
ool5ooooo`?o0?oo3_ooool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003
ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9
ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000;ooooo1Ol0ool3
ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol00003
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:
ooooo`Go00000oooool5o`3oo`?ooooo1Ol00002ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo8?l00005o`3o
o`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2OoooolPo`00
00Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool:oooo
o`Oo00001Ol0oolKo`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`[ooooo1Ol0ool3o`0000Go0?oo4_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`;o00001Ol0ool3o`0000Go0?oo2_ooool8o`0000Go0?oo4_l00005o`3oo`?o00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol00003ooooo`Go
00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go
00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo2_ooool5o`0000;ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`3oo`?ooooo1Ol00002ooooo`Go
00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo3Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Cooooo0ol0ool5ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0000
1Oooool3o`3oo`Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000co
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`co
oooo0ol00005ooooo`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Go
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool9ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo
1Ol00002ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000;ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00003ooooo`Go0?oo0_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo
1Ol0ool:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo=ol0
0005o`3oo`?o00001Ol0ool3o`0000Go0?oo2OoooolPo`0000Go0?oo4_l00005o`3oo`?o00001Ol0
ool3o`0000Go0?oo2_oooolOo`0000Go0?oo0ol00005o`3oo`[o00001Ol0ool3o`0000Go0?oo0ol0
0005o`3oo`[ooooo2?l00005o`3ooa[o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0
ool3o`0000Go0?oo2_ooool_o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Ooo
ool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol00002ooooo`Go00000oooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000;ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`[ooooo1Ol00003ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5
o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool;ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3o
o`Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`00
00Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`00
00Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005ooooo`?o0?oo1?ooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool;
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo3?ooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`3oo`cooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000<
ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Cooooo0ol0ool5ooooo`?o00003_ooool000_ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go0?oo2_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`[ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3oooo
o`Go00003Oooool000_ooooo?ol00005o`3oo`?o00001Ol0ool9ooooodOo00001Ol0ool:ooooodKo
00001Ol0ool:oooood_o00002_oooolno`0000Go0?oo2?l0000=ooooo`002oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`3oo`Wooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol0ool:ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo2_ooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo
1Ol0000=ooooo`003?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`_ooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool<ooooo`?o00001?ooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0
0005ooooo`?o0?oo3?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0000>ooooo`003Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`00
3?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`_ooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol0000<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo
3?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol0000>ooooo`002ooooom7o`0000Go0?oo2Ooooom<o`0000[oooooA_l0
0005o`3oo`[oooooBol0000:oooood_o00003Oooool000_ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol0ool9ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5
o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`[ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool0
00_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool9ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00002_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3o
o`[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`00
00?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00;ooooo1Ol00003ooooo`Go00003Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol0ool;ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Co
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o00001Oooool3o`3oo`cooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol0000<ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003_ooool000gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00gooooo000<ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00002oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`cooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00003?ooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000kooooo000;oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002Oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`3oo`[ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0000:oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00002_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000gooooo000;ooooodco00002Ooooom7o`0000Go0?oo2_oooom;o`0000[o
ooooBol0000:oooood_o00003Oooool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00009ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo2_ooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o0?oo3?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0
0005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00003_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=
ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol0000;ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
3?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`3oo`cooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0000
3_ooool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_oo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00009ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`0000;ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000ooo
ool5o`3oo`[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000_oooooC?l00009ooooodco00002_oooom6
o`0000Go0?oo2_oooom;o`0000[oooooBol0000=ooooo`002oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000Wooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0000:oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000=oooo
o`003?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000_ooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol0000<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o
0?oo3?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo0ol00005ooooo`?o
00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001?ooool3o`0000Gooooo0ol0000>ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`003?ooool3oooo00Gooooo0oooo`05ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`_ooooo0oooo`05ooooo`?oool01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?oool0
1?ooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002ooo
ool5oooo00?ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Oooo`03ooooo`Goool00ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Goool00_ooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5oooo00?o00001Oooo`03o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`Wooooo1Oooo`03o`0000Goool00ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool:ooooo`Goool00_l00005
oooo00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool=ooooo`002oooool5oooo
00?ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`Wooooo1Oooo`03ooooo`Goool00oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool:ooooo`Goool00_ooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool=ooooo`003?ooool3oooo00Gooooo0oooo`05ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_o
oooo0oooo`05ooooo`?oool01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?oool01?ooool3oooo00Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=
ooooo`003?ooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_ooooo0oooo`05oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5oooo
o`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5oooo00?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`Wooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go00000oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`00
2oooool5oooo00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`Wooooo1Oooo`03o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_ooool5o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool;o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo
0ol00005o`3oo`gooooo000;ooooo`Goool00oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Ooo
ool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000<ooooo`?oool01Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo2oooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3
o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000<ooooo`?o0?l01Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo2oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`0000Cooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0
ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000;
ooooo`Go0?l00ol00005o`3oo`?o00001Ol0ool;o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool9ooooo`Go0?oo0ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo2_ooool5o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3
o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool:
o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2
o`0000Go0?oo0ol00005o`3oo`[ooooo2?l00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool0
00_ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`[ooooo1Ol0ool3ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol0ool9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;o
oooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;o
oooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go00000_ooool5o`3oo`?o
oooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?o
oooo1Ol0ool:ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo
0ol0o`05ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo
0ol0ool5ooooo`?o00001?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`3oo`Gooooo
0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo
0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<oooo
o`?o0?oo1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o00002oooool3o`0000Gooooo0ol00005oooo
o`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`cooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<oooo
o`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Gooooo0ol00005oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4oooo
o`?o0?oo1Oooool3o`3oo`kooooo000;ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go
00000oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
00002Oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00002ooooo`Go
00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go
00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo2_ooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000;ooooo`Go0?oo
6ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool8o`0000Wooooo
8?l00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_oooolOo`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o0000
1Ol0ool:oooooaoo00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo
0ol00005o`3oo`[ooooo4?l00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o0000
1Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool=ooooo`002oooool5o`3oo`?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`0000Wooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0
0003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool:ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol0
0002ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0
ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo1Ol0
ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0
ool=ooooo`003?ooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000_ooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo3?ooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005
ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5
ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`003Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?oo
ool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o0?oo1Ooo
ool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`_ooooo0ol00005ooooo`?o00001Ooo
ool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Ooo
ool3o`3oo`Gooooo0ol0ool<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Ooo
ool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?oo
ool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Ooo
ool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00005ooooo`?o00001Ooo
ool3o`0000Cooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?oo
ool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`Wooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000_ooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5
o`3oo`[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002ooooolWo`00
00Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`Wooooo:?l00005o`3o
o`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool:oooooaoo00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo9ol00005o`3o
o`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:oooooaoo00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`gooooo000;ooooo`Go
00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2Oooool5o`0000?ooooo1Ol00003ooooo`Go
00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`[ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go
00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00003ooooo`Go
00000_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`gooooo000<ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
2oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol00004ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
3?ooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000=ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00gooooo000<ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o00002oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000cooooo0ol00004ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool<ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3?ooool3o`0000Go
oooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`0000Go
oooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`kooooo000;ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol00003ooooo`Go00002Oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000[ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo
1Ol0ool3ooooo`Go0?oo2_ooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo
000;oooooboo00001Ol0ool3o`0000Go0?oo4?l00009oooooboo00001Ol0ool3o`0000Go0?oo0ol0
0005o`3oo`So00002_ooool_o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_oo
ool_o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo2_oooolWo`0000Go0?oo2ol0
0005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`0000?ooooo1Ol00009ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00002_ooool5
o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5
o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5
o`0000?ooooo1Ol0ool3ooooo`Go00000oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool0
00cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0000;ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o00003?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`cooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`00
00Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o00001Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo
000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol0000;ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol00005
ooooo`?o00003?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000cooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Oooool3o`0000Gooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0ool5
ooooo`?o0?oo3_ooool000_ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00009oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`3oo`?ooooo1Ol00003ooooo`Go00002_ooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`0000[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool:oooo
o`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_oooooC?l00009ooooocOo
00001Ol0ool@o`0000[ooooo=_l00005o`3oo`?o00001Ol0ool8o`0000[ooooo?_l00005o`3oo`?o
00001Ol0ool:ooooocko00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00009ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`3oo`?ooooo1Ol00003ooooo`Go0000
2_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000[ooooo1Ol00003ooooo`Go0000
0_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0000
0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go0000
0oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go0?oo
3Oooool000cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?oo
ool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;ooooo`?o00001Ooo
ool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Ooo
ool3o`3oo`Gooooo0ol00005ooooo`?o00003?ooool3o`0000Cooooo0ol00005ooooo`?o00001Ooo
ool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0ool5ooooo`?o0?oo1Ooo
ool3o`0000cooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Ooo
ool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o00001Ooo
ool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Ooo
ool3o`0000Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool0
0ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo
0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo
0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;ooooo`?o0?oo1Oooool3o`0000Gooooo
0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo
0ol00005ooooo`?o00003?ooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo
0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo
0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo
0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Gooooo
0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo
0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol0
0003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0
0009ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0
0002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`0000;ooooo1Ol0
0003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0
0003ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0
000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo1Ol0oom7
o`0000Wooooo1Ol0oom7o`0000[oooooBol0000:ooooocko00001Ol0ool8o`0000[ooooo?_l00005
o`3oo`?o00001Ol0ool=ooooo`002oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00Wooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go00000_ooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`00
00?ooooo1Ol00003ooooo`Go00002_ooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`00
00[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0ool=ooooo`003?ooool3o`3oo`Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000_ooooo0ol0ool5ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol0000<ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Go
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00004ooooo`?o0?oo1Oooool3o`0000cooooo0ol00005ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Go
oooo0ol0ool>ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo
000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005oooo
o`?o00001Oooool3o`0000_ooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00005oooo
o`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000<oooo
o`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005oooo
o`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Gooooo0ol00004oooo
o`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004oooo
o`?o00001Oooool3o`0000cooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005oooo
o`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000>oooo
o`002oooool5o`3oo`?o00001Ol0ooloo`0000Wooooo1Ol0ool3o`0000Go0?oo?ol0000:ooooo`Go
0?ooA_l0000:oooood_o00002_oooom;o`0000gooooo000;ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00002Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo
1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00002_ooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000gooooo
000;ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol0
0003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002Oooool5o`3oo`?ooooo1Ol0
ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000[ooooo1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0
000:ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0
0003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00002_ooool5o`0000?ooooo1Ol0
0003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0
0002ooooo`Go00000oooool5o`0000gooooo000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00002oooool3o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool4
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004
ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00003?ooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000kooooo000=oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3oooo00Gooooo0oooo`04ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000_ooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o0000
1Oooool3o`0000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000>ooooo`002ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00?ooooo1Oooo`02ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000Wooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00002_oo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_oo
ool5o`0000?ooooo1Ol0000=ooooo`002oooool5o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
oooo00?o00001Oooo`0Wo`0000Wooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oocOo00002_ooool5
o`3oo`;o00001Ol0ooloo`0000[ooooo1Ol0ool3o`0000Go0?oo?_l0000:ooooo`Go0?oo0ol00005
o`3oocko00003Oooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Oooo`03oooo
o`Goool00_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00009oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00002oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000cooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0oooo`05ooooo`?oool01?ooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol0000;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00003?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o
0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001?ooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00004ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o
00003_ooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oo
oooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oo
oooo^Oooool000cooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool0
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?oool0
1Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3oooo00Cooooo0oooo`05ooooo`?oool0
1Oooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`cooooo0oooo`05ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?oool0
1Oooool3oooo00Gooooo0oooo`04ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00?ooooo1Oooo`03ooooo`Goool00_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool9ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_oo
ool5oooo00;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Oooo`03ooooo`Goool00_oo
ool5oooo00?ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool:ooooo`Goool00oooool5oooo00?ooooo1Oooo`02ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Ooo
ool000_ooooo1Oooo`03o`0000Goool00ol00005oooo00?o00001Oooo`03o`0000Goool00_l00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool9ooooo`Goool00ol00005
oooo00?o00001Oooo`03o`0000Goool00ol00005oooo00?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5oooo00;o00001Oooo`03o`0000Goool00ol00005
oooo00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`[ooooo1Oooo`03o`0000Goool00_l00005oooo00?o00001Oooo`03o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool:ooooo`Goool00ol00005
oooo00?o00001Oooo`02o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005
o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Oooo`03ooooo`Goool00oooool5oooo
00?ooooo1Oooo`03ooooo`Goool00_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool9ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5oooo
00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5oooo
00;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Oooo`03ooooo`Goool00_ooool5oooo
00?ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool:ooooo`Goool00oooool5oooo00?ooooo1Oooo`02ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000co
oooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?oool01Oooool3oooo00Go
oooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo3?ooool3oooo00Cooooo0oooo`05ooooo`?oool01Oooool3oooo00Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`co
oooo0oooo`05ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?oool01Oooool3oooo00Go
oooo0oooo`04ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0oooo`05oooo
o`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?o0?l01?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3oooo00Gooooo0oooo`05oooo
o`?o0?l01Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo3?ooool3o`3oo`Cooooo0oooo`05ooooo`?o0?l01Oooool3oooo00Gooooo0ol0ool5oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5oooo
o`?o0?l01?ooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?l01Oooool3oooo00Gooooo0ol0ool4oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5oooo
o`?o0?oo3_ooool000_ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go
0?l00_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go
0?oo0oooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Oooo`03ooooo`Go
0?l00oooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?l00_ooool5oooo00?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go
0?l00oooool5oooo00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000_ooooo1Oooo`03o`0000Goool0
0ol00005oooo00?o00001Oooo`03o`0000Go0?l00_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool9ooooo`Go0?oo0ol00005oooo00?o00001Oooo`03o`0000Go0?l0
0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_ooool5o`3oo`;o00001Oooo`03o`0000Go0?l00ol00005oooo00?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?l0
0_l00005oooo00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool:ooooo`Go0?l00ol00005oooo00?o00001Ol0ool2o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo
3Oooool000_ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0ooo
ool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Oooo`03ooooo`Go0?l00ooo
ool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?l00_ooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?l00ooo
ool5oooo00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0oooo`05ooooo`?oool01Oooool3
oooo00Gooooo0oooo`05ooooo`?o0?l01?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool;ooooo`?o0?oo1Oooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3
o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3
o`3oo`Cooooo0oooo`05ooooo`?o0?l01Oooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3
o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?l01?ooool3
oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool<ooooo`?o0?l01Oooool3oooo00Gooooo0ol0ool4ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool0
00gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
o`05ooooo`?o0?l01?ooool3o`3o00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0o`05ooooo`?o00001Oooool3o`3o00Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0
o`05ooooo`?o0?l01Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3o00Gooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool<ooooo`?o0?l01Oooool3o`3o00Gooooo0ol0ool4ooooo`?o00001Oooool3o`0000Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0o`03ooooo`Go0?l00_ooool5o`3o00?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0o`03
ooooo`Go00000oooool5o`3o00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo2_ooool5o`3oo`;ooooo1Ol0o`03ooooo`Go0?l00oooool5o`0000?ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0o`03
ooooo`Go0?oo0_ooool5o`3o00?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?l00oooool5o`3o00?ooooo1Ol0ool2
ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3
ooooo`Go0?oo3Oooool000_ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0o`03o`00
00Go0?l00_l00005o`3o00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool9oooo
o`Go0?oo0ol00005o`3oo`?o00001Ol0o`0;o`0000Go0?l00ol00005o`3oo`;o00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`?o00001Ol0ool:ooooo`Go0?oo0_l00005o`3o00?o00001Ol0o`0;o`00
00Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool:oooo
o`Go0?l00ol00005o`3oo`;o00001Ol0o`0Co`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`00
00Go0?oo0ol00005o`3oo`[ooooo1Ol0o`03o`0000Go0?l00ol00005o`3ooa;o00001Ol0ool3o`00
00Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0o`03ooooo`Go0?l00_ooool5o`3o00?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0o`03ooooo`Go
00000oooool5o`3o00?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo2_ooool5o`3oo`;ooooo1Ol0o`03ooooo`Go0?l00oooool5o`0000?ooooo1Ol0ool3ooooo`Go
0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0o`03ooooo`Go
0?oo0_ooool5o`3o00?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?l00oooool5o`3o00?ooooo1Ol0ool2ooooo`Go
00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
0?oo3Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0o`05ooooo`?o0?l0
1?ooool3o`3o00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool;ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0o`05ooooo`?o00001Oooool3o`3o00Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0o`05ooooo`?o0?l0
1Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`cooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3o00Gooooo0ol00005ooooo`?o0000
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?l0
1Oooool3o`3o00Gooooo0ol0ool4ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0?l01?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0000;ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`co
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0000<ooooo`?o0?l01Oooool3o`3oo`Go
oooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o00003_ooool000_ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oo`_o0000
1Ol0o`02o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2?l00009ooooo`Go0?oo
0ol00005o`3ooa_o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_ooool5o`3oo`;o00001Ol0oolKo`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0_l00005o`3ooa?o00001Ol0ool3o`0000Go0?oo
0ol00005o`3oo`;o00001Ol0ool8o`0000[ooooo1Ol0o`03o`0000Go0?oo4_l00005o`3oo`?o0000
1Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool8o`0000gooooo000;ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`3o00;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go00002Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`[ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go00002_ooool5o`3o00?ooooo1Ol0ool3ooooo`Go00000_oo
ool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`0000gooooo000;ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5
o`3o00;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00002Oooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5
o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool2ooooo`Go0?oo0oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5
o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go00000oooool5
o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go00002_ooool5
o`3o00?ooooo1Ol0ool3ooooo`Go00000_ooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000gooooo000<ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o00001Oooool3o`3o00Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol0ool5ooooo`?o00002oooool3o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3o
o`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o
o`Cooooo0ol0ool5ooooo`?o00003?ooool3o`3o00Gooooo0ol0ool5ooooo`?o00001?ooool3o`00
00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`00
00kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`3oo`Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000_ooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0
000<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?o00003?ooool3o`3oo`Gooooo0ol0
ool4ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`0000cooooo0ol0ool5ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool>ooooo`002oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00002
ooooo`Go00000oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000Wooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol00003
ooooo`Go00002_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`0000[ooooo1Ol0ool3
ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5o`3oo`?o00001Ol0ool3o`00
00Go0?oo6_l00005o`3ooaSo00002Oooool5o`3oo`?o00001Ol0oolKo`0000Go0?oo0_l00005o`3o
o`?o00001Ol0ool@o`0000[ooooo1Ol0ool2o`0000Go0?oo0ol00005o`3ooa_o00001Ol0ool2o`00
00Go0?oo4?l0000:ooooo`Go0?oo0ol00005o`3ooa[o00001Ol0ool3o`0000Go0?oo0ol00005o`3o
o`;o00001Ol0ool8o`0000[ooooo1Ol0oolRo`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`00
00Go0?oo0ol00005o`3oo`gooooo000;ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go
00000oooool5o`0000;ooooo1Ol00003ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go
00002Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000oooool5o`0000[ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol00003ooooo`Go
00000oooool5o`0000?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go
00002_ooool5o`3oo`?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go
0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`gooooo000<ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o0?oo
1Oooool3o`0000Gooooo0ol00005ooooo`?o00002oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0000
1Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0000
1Oooool3o`0000cooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0000<ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o00003?ooool3o`3oo`Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`kooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Ooo
ool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00002oooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000co
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Go
oooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3o`0000Co
oooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol00005ooooo`?o00003?ooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol00005ooooo`?o00001Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`0000ko
oooo000;ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002Oooool5o`3oo`?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo
1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol0000:ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol00003ooooo`Go00002_ooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`?ooooo
1Ol0ool2ooooo`Go0?oo0oooool5o`0000gooooo000;ooooo`Go0?oo0ol00005o`3oocoo00002Ooo
ool5o`3oob[o00001Ol0oolHo`0000[oooooBol0000:oooooboo00001Ol0ool3o`0000Go0?oo3ol0
000:oooooboo00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`So00003Oooool000_ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003
ooooo`Go00000oooool5o`0000?ooooo1Ol00009ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003
ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00003
ooooo`Go00002_ooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003
ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol00003
ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0ool3
ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002
ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3
ooooo`Go00003Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00005oooo
o`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;oooo
o`?o0?oo1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004oooo
o`?o0?oo1Oooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`0000Cooooo0ol00005oooo
o`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005oooo
o`?o00001Oooool3o`0000cooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005oooo
o`?o00001Oooool3o`0000Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`0000Gooooo0ol0000<oooo
o`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005oooo
o`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o00003_ooool000gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Ooo
ool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00005ooooo`?o00001?ooool3
o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0000;ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3
o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`3oo`Cooooo0ol00005ooooo`?o00001Oooool3
o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3
o`0000cooooo0ol0ool5ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3
o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000<ooooo`?o00001Oooool3
o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3
o`0000Cooooo0ol00005ooooo`?o00003_ooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00009ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`00
00?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3o
o`;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`00
00;ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go00000_ooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`00
00?ooooo1Ol0000:ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`00
00?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000_o
oooo1Ol0ool3o`0000Go0?oo0ol00005o`3oocOo00002Oooool5o`3oo`?o00001Ol0ooloo`0000[o
oooo1Ol0oom6o`0000[ooooo1Ol0oom6o`0000[oooooBol0000=ooooo`002oooool5o`3oo`?ooooo
1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000Wooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol0000:ooooo`Go0?oo0_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`?ooooo
1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00002ooooo`Go00000oooool5o`0000[ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo
1Ol0000=ooooo`003?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000_ooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1?ooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0
0005ooooo`?o00003?ooool3o`3oo`Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo0ol0
0005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`0000Gooooo0ol0000>ooooo`003Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`00
3?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1Oooool3oooo00Gooooo0ol00004ooooo`?oool0
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000_ooooo0ol0ool5ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0oooo`05ooooo`?o00001Oooool3oooo00Cooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?oool0
1Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol00005ooooo`?o0000
3?ooool3o`3oo`Gooooo0ol0ool4ooooo`?o00001Oooool3oooo00Gooooo0ol00005ooooo`?o0000
1Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o0000
1?ooool3o`0000Gooooo0ol0000>ooooo`002oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5oooo00?ooooo1Ol00002ooooo`Goool00oooool5o`0000?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000Wooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Oooo`03ooooo`Go00000ooo
ool5oooo00;ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol00003ooooo`Goool00oooool5o`0000?ooooo1Ol00003ooooo`Go00000_oo
ool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go00000ooo
ool5oooo00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000ooo
ool5o`0000[ooooo1Ol0ool3ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000=ooooo`002oooool5
o`3oo`?o00001Ol0ool3o`0000Go0?oo0ol00005oooo00[o00001Oooo`0Po`0000Wooooo1Ol0ool3
o`0000Go0?oo0ol00005o`3oo`?o00001Oooo`0;o`0000Goool07ol0000:ooooo`Go0?oo0_l00005
o`3oo`_o00001Oooo`0_o`0000[ooooo1Ol0ool3o`0000Go0?oo2_l00005oooo02oo00002_ooool5
o`3oodKo00003Oooool000_ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Oooo`03oooo
o`Go00000_ooool5oooo00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00009oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Goool00oooool5o`0000?ooooo1Oooo`02oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go00000oooool5oooo00?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`0000?ooooo1Oooo`03oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:oooo
o`Go0?oo0oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000cooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0oooo`05ooooo`?o00001?ooool3oooo00Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol0000;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
ool01Oooool3o`0000Gooooo0oooo`04ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00003?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o00001Oooool3oooo00Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o
0?oo1?ooool3o`0000Gooooo0oooo`05ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001?ooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`0000Gooooo0ol00004ooooo`?o
00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o
00003_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0ol0ool5ooooo`?o0?oo1Oooool3oooo
00Gooooo0oooo`05ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?o00001Oooool3o`00
00Gooooo0ol0000;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3oooo
00Gooooo0oooo`04ooooo`?oool01Oooool3o`0000Gooooo0ol00005ooooo`?o00003?ooool3o`3o
o`Cooooo0ol0ool5ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3o`00
00Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o0?oo1?ooool3oooo
00Gooooo0oooo`05ooooo`?oool01Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`00
00Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0oooo`04ooooo`?oool01Oooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003_ooool000_o
oooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00?ooooo1Oooo`03ooooo`Goool00_ooool5oooo00?o
oooo1Oooo`03ooooo`Go00000oooool5o`0000?ooooo1Ol00009ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`02ooooo`Goool00oooool5o`0000?o
oooo1Ol00003ooooo`Go00002_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Goool00oooool5oooo00?o
oooo1Oooo`03ooooo`Goool00oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000[o
oooo1Ol0ool3ooooo`Go0?oo0_ooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5o`0000?o
oooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Oooo`02ooooo`Goool00oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;o
oooo1Ol00003ooooo`Go00003Oooool000_ooooo1Ol0ool3o`0000Go0?oo0ol00005oooo00?o0000
1Oooo`03o`0000Goool00_l00005oooo00?o00001Oooo`0Ho`0000Wooooo1Ol0ool3o`0000Go0?oo
0ol00005oooo00?o00001Oooo`03o`0000Goool00ol00005oooo00;o00001Oooo`0Ho`0000[ooooo
1Ol0ool2o`0000Go0?oo0ol00005oooo00?o00001Oooo`03o`0000Goool00ol00005oooo01oo0000
2_ooool5o`3oo`?o00001Ol0ool2o`0000Goool00ol00005oooo00?o00001Oooo`0Wo`0000[ooooo
1Ol0ool3o`0000Go0?oo0ol00005oooo00;o00001Oooo`0_o`0000gooooo000;ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Oooo`03ooooo`Goool00oooool5oooo00;ooooo1Oooo`03ooooo`Goool00ooo
ool5o`0000?ooooo1Ol00003ooooo`Go00002Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Goool00ooo
ool5oooo00?ooooo1Oooo`03ooooo`Goool00_ooool5oooo00?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000[ooooo1Ol0ool2ooooo`Go0?oo0oooool5oooo00?ooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0ooo
ool5o`3oo`;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Ol00003ooooo`Go00000ooo
ool5o`0000;ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Goool00_oo
ool5oooo00?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go00000ooo
ool5o`0000gooooo000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3
oooo00Cooooo0oooo`05ooooo`?oool01Oooool3o`0000Gooooo0ol00005ooooo`?o00002oooool3
o`3oo`Gooooo0ol0ool5ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01?ooool3
oooo00Gooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool4ooooo`?o0?oo1Oooool3
oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol00004ooooo`?o00001Oooool3
o`0000Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3oo`Cooooo0oooo`05ooooo`?oool01Oooool3
oooo00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003?ooool3
o`3oo`Gooooo0ol0ool5ooooo`?oool01?ooool3oooo00Gooooo0ol00005ooooo`?o00001Oooool3
o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000kooooo000=ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo
000<ooooo`?o0?oo1Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Cooooo0ooo
o`05ooooo`?oool01Oooool3oooo00Gooooo0ol00005ooooo`?o00002oooool3o`3oo`Gooooo0ol0
ool5ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01?ooool3oooo00Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000cooooo0ol0ool4ooooo`?o0?oo1Oooool3oooo00Gooooo0ooo
o`05ooooo`?oool01Oooool3oooo00Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0
000<ooooo`?o0?oo1Oooool3o`3oo`Cooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ooo
o`05ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003?ooool3o`3oo`Gooooo0ol0
ool5ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?o00001Oooool3o`0000Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000kooooo000;ooooo`Go0?oo0ol00005oooo00?o00001Oooo`03
o`0000Goool00ol00005oooo00;o00001Oooo`03o`0000Goool00ol00005oooo013o00002Oooool5
o`3oo`?o00001Ol0ool3o`0000Goool00ol00005oooo00?o00001Oooo`03o`0000Goool00_l00005
oooo01So00002_ooool5o`3oo`;o00001Ol0ool3o`0000Goool00ol00005oooo00?o00001Oooo`03
o`0000Goool07ol0000:ooooo`Go0?oo0ol00005o`3oo`;o00001Oooo`03o`0000Goool00ol00005
oooo00?o00001Oooo`0Oo`0000[ooooo1Ol0ool3o`0000Go0?oo0ol00005oooo00;o00001Oooo`03
o`0000Goool09ol0000=ooooo`002oooool5o`3oo`?ooooo1Oooo`03ooooo`Goool00oooool5oooo
00?ooooo1Oooo`02ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go00000oooool5o`00
00Wooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5oooo
00;ooooo1Oooo`03ooooo`Go00000oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go00000_ooool5o`00
00?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Goool00oooool5oooo
00?ooooo1Oooo`03ooooo`Goool00oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`00
00[ooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00;ooooo1Oooo`03ooooo`Goool00oooool5o`00
00?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000=ooooo`002oooool5o`3oo`?o
oooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`02ooooo`Goool00oooool5oooo00?o
oooo1Oooo`03ooooo`Go00000oooool5o`0000Wooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00?o
oooo1Oooo`03ooooo`Goool00oooool5oooo00;ooooo1Oooo`03ooooo`Go00000oooool5o`0000?o
oooo1Ol0000:ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Oooo`03ooooo`Goool00oooool5oooo00?o
oooo1Oooo`03ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5o`3oo`?o
oooo1Ol0ool2ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5o`0000?o
oooo1Ol00002ooooo`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00;o
oooo1Oooo`03ooooo`Goool00oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?o
oooo1Ol0000=ooooo`003?ooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo
0oooo`04ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?o00001Oooool3o`0000_ooooo
0ol0ool5ooooo`?o0?oo1Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Cooooo
0oooo`05ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1?ooool3o`3oo`Gooooo
0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?o00001?ooool3o`0000Gooooo
0ol00005ooooo`?o00003?ooool3o`3oo`Gooooo0ol0ool4ooooo`?oool01Oooool3oooo00Gooooo
0oooo`05ooooo`?oool01Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000cooooo
0ol0ool5ooooo`?o0?oo1Oooool3oooo00Cooooo0oooo`05ooooo`?oool01Oooool3o`0000Gooooo
0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000>ooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooniooooo`00ooooooni
ooooo`00ooooooniooooo`00ooooooniooooo`003?ooool3oooo00Gooooo0oooo`05ooooo`?o0?l0
1Oooool3ool000Gooooo0ol0ool4ooooo`?oo`3o1Oooool3olc<c0Gooooo0ol0ool5ooooo`?oool0
1Oooool3o`3o00_ooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0o`05ooooo`?o0?oo
1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ooo000<ooooo`?oool0
1?ooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo
1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?oo`003?ooool3oooo00Gooooo0oooo`04ooooo`?oool0
1Oooool3oooo00Gooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo
1Oooool3o`3oo`cooooo0oooo`05ooooo`?oool01Oooool3oooo00Cooooo0oooo`05ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002ooo
ool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5ool000?ooooo1Ol0ool2ooooo`Goo`3o0ooo
ool5olc<c0?ooooo1Ol0ool3ooooo`Goool00oooool5o`3o00Wooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00?ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ooo000:ooooo`Goool00_ooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Goo`002_oo
ool5oooo00?ooooo1Oooo`02ooooo`Goool00oooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0ooo
ool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00;ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5oooo00?o00001Oooo`03o`0000Go0?l00ol00005
ool000?o00001Ol0ool2o`0000Goo`3o0ol00005olc<c0?o00001Ol0ool3o`0000Goool00ol00005
o`3o00Wooooo1Oooo`03o`0000Goool00ol00005oooo00?o00001Ol0o`03o`0000Go0?oo0ol00005
o`3oo`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ooo000:ooooo`Goool00_l00005
oooo00?o00001Oooo`03o`0000Go0?l00ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005
o`3oo`?o00001Ol0ool3o`0000Goo`002_ooool5oooo00?o00001Oooo`02o`0000Goool00ol00005
oooo00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005
o`3oo`[ooooo1Oooo`03o`0000Goool00ol00005oooo00;o00001Oooo`03o`0000Go0?oo0ol00005
o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool=ooooo`002oooool5oooo
00?ooooo1Oooo`03ooooo`Go0?l00oooool5ool000?ooooo1Ol0ool2ooooo`Goo`3o0oooool5olc<
c0?ooooo1Ol0ool3ooooo`Goool00oooool5o`3o00Wooooo1Oooo`03ooooo`Goool00oooool5oooo
00?ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ooo000:ooooo`Goool00_ooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5o`3o
o`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Goo`002_ooool5oooo
00?ooooo1Oooo`02ooooo`Goool00oooool5oooo00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o
o`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Oooo`03ooooo`Goool00oooool5oooo
00;ooooo1Oooo`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool=ooooo`003?ooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3ool000Go
oooo0ol0ool4ooooo`?oo`3o1Oooool3olc<c0Gooooo0ol0ool5ooooo`?oool01Oooool3o`3o00_o
oooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0o`05ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ooo000<ooooo`?oool01?ooool3oooo00Go
oooo0oooo`05ooooo`?o0?l01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Go
oooo0ol0ool5ooooo`?oo`003?ooool3oooo00Gooooo0oooo`04ooooo`?oool01Oooool3oooo00Go
oooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`co
oooo0oooo`05ooooo`?oool01Oooool3oooo00Cooooo0oooo`05ooooo`?o0?oo1Oooool3o`3oo`Go
oooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03Oooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=
ooooo`003?ooool3oooo00Gooooo0ol0o`05ooooo`?o003o1Oooool3o`3o00Gooooo0ooo0?l4oooo
o`?oc<c<1Oooool3ool0o`Gooooo0ooo0?l5ooooo`?o0?l01Oooool3ool000_ooooo0oooo`05oooo
o`?o0?l01Oooool3o`00o`Gooooo0ol0o`05ooooo`?o0?l01Oooool3olc<c0Cooooo0oo<c<`5oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ooo000<ooooo`?oool01?ooool3oooo00Gooooo0oooo`05oooo
o`?o0?l01Oooool3olc<c0Gooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo3?ooool3oooo00Gooooo0oooo`04ooooo`?oool01Oooool3oooo00Gooooo0ol0o`05oooo
o`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0oooo`05oooo
o`?oool01Oooool3oooo00Cooooo0oooo`05ooooo`?oool01Oooool3o`3oo`Gooooo0ol0ool5oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool>ooooo`002oooool5oooo00?ooooo1Ol0o`03ooooo`Go
003o0oooool5o`3o00?ooooo1Ooo0?l2ooooo`Goc<c<0oooool5ool0o`?ooooo1Ooo0?l3ooooo`Go
0?l00oooool5ool000Wooooo1Oooo`03ooooo`Go0?l00oooool5o`00o`?ooooo1Ol0o`03ooooo`Go
0?l00oooool5olc<c0;ooooo1Oo<c<`3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ooo000:ooooo`Go
ool00_ooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5olc<c0?ooooo1Ol0o`03ooooo`Go
0?oo0_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5oooo00?ooooo1Oooo`02ooooo`Go
ool00oooool5oooo00?ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go
0?oo0oooool5o`3oo`[ooooo1Oooo`03ooooo`Goool00oooool5oooo00;ooooo1Oooo`03ooooo`Go
ool00oooool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`00
2oooool5oooo00?o00001Ol0o`03o`0000Go003o0ol00005o`3o00?o00001Ooo0?l2o`0000Goc<c<
0ol00005ool0o`?o00001Ooo0?l3o`0000Go0?l00ol00005ool000Wooooo1Oooo`03o`0000Go0?l0
0ol00005o`00o`?o00001Ol0o`03o`0000Go0?l00ol00005olc<c0;o00001Oo<c<`3o`0000Go0?oo
0ol00005o`3oo`?o00001Ooo000:ooooo`Goool00_l00005oooo00?o00001Oooo`03o`0000Go0?l0
0ol00005olc<c0?o00001Ol0o`03o`0000Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
2_ooool5oooo00?o00001Oooo`02o`0000Goool00ol00005oooo00?o00001Ol0o`03o`0000Go0?oo
0ol00005o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`[ooooo1Oooo`03o`0000Goool0
0ol00005oooo00;o00001Oooo`03o`0000Goool00ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo
0_l00005o`3oo`?o00001Ol0ool=ooooo`002oooool5oooo00?ooooo1Ol0o`03ooooo`Go003o0ooo
ool5o`3o00?ooooo1Ooo0?l2ooooo`Goc<c<0oooool5ool0o`?ooooo1Ooo0?l3ooooo`Go0?l00ooo
ool5ool000Wooooo1Oooo`03ooooo`Go0?l00oooool5o`00o`?ooooo1Ol0o`03ooooo`Go0?l00ooo
ool5olc<c0;ooooo1Oo<c<`3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ooo000:ooooo`Goool00_oo
ool5oooo00?ooooo1Oooo`03ooooo`Go0?l00oooool5olc<c0?ooooo1Ol0o`03ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo2_ooool5oooo00?ooooo1Oooo`02ooooo`Goool00ooo
ool5oooo00?ooooo1Ol0o`03ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0ooo
ool5o`3oo`[ooooo1Oooo`03ooooo`Goool00oooool5oooo00;ooooo1Oooo`03ooooo`Goool00ooo
ool5o`3oo`?ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`003?ooool3
oooo00Gooooo0ol0o`05ooooo`?o003o1Oooool3o`3o00Gooooo0ooo0?l4ooooo`?oc<c<1Oooool3
ool0o`Gooooo0ooo0?l5ooooo`?o0?l01Oooool3ool000_ooooo0oooo`05ooooo`?o0?l01Oooool3
o`00o`Gooooo0ol0o`05ooooo`?o0?l01Oooool3olc<c0Cooooo0oo<c<`5ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ooo000<ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3
olc<c0Gooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3
oooo00Gooooo0oooo`04ooooo`?oool01Oooool3oooo00Gooooo0ol0o`05ooooo`?o0?oo1Oooool3
o`3oo`Gooooo0ol0ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0oooo`05ooooo`?oool01Oooool3
oooo00Cooooo0oooo`05ooooo`?oool01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3
o`3oo`Gooooo0ol0ool>ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3olc<c0Gooooo0oo<
c<`5ooooo`?o003o1Oooool3olc<c0Gooooo0oooo`04ooooo`?oc<c<1Oooool3ool0o`Gooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`0000_ooooo0oooo`05ooooo`?o003o1Oooool3o`00o`Gooooo0ol0
0?l5ooooo`?o0?l01Oooool3o`3o00Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
ool<ooooo`?oool01?ooool3ool000Gooooo0ol00?l5ooooo`?o0?l01Oooool3o`3o00Gooooo0ol0
o`05ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3o`3oo`Gooooo0ooo
o`04ooooo`?oo`001Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l01Oooool3o`3oo`Gooooo0ol0
ool4ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1Oooool3ool000Cooooo0ooo
0005ooooo`?o0?l01Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
ool>ooooo`002oooool5olc<c0?ooooo1Oo<c<`3ooooo`Go003o0oooool5olc<c0?ooooo1Oooo`02
ooooo`Goc<c<0oooool5ool0o`?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`0000Wooooo1Oooo`03
ooooo`Go003o0oooool5o`00o`?ooooo1Ol00?l3ooooo`Go0?l00oooool5o`3o00;ooooo1Ol0ool3
ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool:ooooo`Goool00_ooool5ool000?ooooo1Ol00?l3
ooooo`Go0?l00oooool5o`3o00?ooooo1Ol0o`03ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo2_ooool5o`3oo`?ooooo1Oooo`02ooooo`Goo`000oooool5o`3o00?ooooo1Ol0o`03
ooooo`Go0?l00oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3
ooooo`Go0?oo0oooool5ool000;ooooo1Ooo0003ooooo`Go0?l00oooool5o`3oo`?ooooo1Ol0ool3
ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool=ooooo`002oooool5olc<c0?o00001Oo<c<`3o`00
00Go003o0ol00005olc<c0?o00001Oooo`02o`0000Goc<c<0ol00005ool0o`?o00001Ol0ool3o`00
00Go0?oo2?l00009ooooo`Goool00ol00005o`00o`?o00001Ol00?l3o`0000Go003o0ol00005o`3o
00?o00001Ol0o`02o`0000Go0?oo0ol00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_ooool5oooo
00;o00001Ooo0003o`0000Go003o0ol00005o`3o00?o00001Ol0o`03o`0000Go0?l00ol00005o`3o
o`;o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Goool00_l00005ool0
00?o00001Ol0o`03o`0000Go0?l00ol00005o`3o00?o00001Ol0ool3o`0000Go0?oo0_l00005o`3o
o`?o00001Ol0ool:ooooo`Go0?oo0ol00005o`3oo`?o00001Ooo0002o`0000Goo`000ol00005o`3o
00?o00001Ol0ool3o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Oooool000_o
oooo1Oo<c<`3ooooo`Goc<c<0oooool5o`00o`?ooooo1Oo<c<`3ooooo`Goool00_ooool5olc<c0?o
oooo1Ooo0?l3ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00009ooooo`Goool00oooool5o`00o`?o
oooo1Ol00?l3ooooo`Go003o0oooool5o`3o00?ooooo1Ol0o`02ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol0ool3ooooo`Go0?oo2_ooool5oooo00;ooooo1Ooo0003ooooo`Go003o0oooool5o`3o00?o
oooo1Ol0o`03ooooo`Go0?l00oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[o
oooo1Ol0ool3ooooo`Goool00_ooool5ool000?ooooo1Ol0o`03ooooo`Go0?l00oooool5o`3o00?o
oooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ooo0002ooooo`Goo`000oooool5o`3o00?ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;o
oooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0oo<c<`5ooooo`?oc<c<1Oooool3o`00o`Gooooo
0oo<c<`5ooooo`?oool01?ooool3olc<c0Gooooo0ooo0?l5ooooo`?o0?oo1Oooool3o`3oo`Gooooo
0ol0000;ooooo`?oool01Oooool3o`00o`Gooooo0ol00?l5ooooo`?o003o1Oooool3o`3o00Gooooo
0ol0o`04ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool5ooooo`?o0?oo3?ooool3oooo00Cooooo
0ooo0005ooooo`?o003o1Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l01Oooool3o`3oo`Cooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?oool01?ooool3ool000Gooooo
0ol0o`05ooooo`?o0?l01Oooool3o`3o00Gooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo
0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ooo0004ooooo`?oo`001Oooool3o`3o00Gooooo
0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool03Oooool000cooooo0ol0ool5ooooo`?oool01Oooool3o`0000Gooooo0oo<c<`5ooooo`?o
0?l01?ooool3o`3o00Gooooo0oo<c<`5ooooo`?oc<c<1Oooool3ool0o`Gooooo0oooo`0;ooooo`?o
ool01Oooool3ool000Gooooo0ol00?l5ooooo`?o0?l01Oooool3o`3o00Gooooo0ol0o`04ooooo`?o
0?l01Oooool3olc<c0Gooooo0ol0ool5ooooo`?oool03?ooool3o`3oo`Cooooo0ooo0005ooooo`?o
o`001Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l01Oooool3o`3o00Cooooo0ol0ool5ooooo`?o
0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3ool000Gooooo0ooo0005ooooo`?o
0?l01Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool<ooooo`?o
0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o00001Oooool3ool000Gooooo0ol00005ooooo`?o
0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000_ooooo1Ol0ool3ooooo`Goool0
0oooool5o`0000?ooooo1Oo<c<`3ooooo`Go0?l00_ooool5o`3o00?ooooo1Oo<c<`3ooooo`Goc<c<
0oooool5ool0o`?ooooo1Oooo`09ooooo`Goool00oooool5ool000?ooooo1Ol00?l3ooooo`Go0?l0
0oooool5o`3o00?ooooo1Ol0o`02ooooo`Go0?l00oooool5olc<c0?ooooo1Ol0ool3ooooo`Goool0
2_ooool5o`3oo`;ooooo1Ooo0003ooooo`Goo`000oooool5o`3o00?ooooo1Ol0o`03ooooo`Go0?l0
0oooool5o`3o00;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo
0_ooool5ool000?ooooo1Ooo0003ooooo`Go0?l00oooool5o`3o00?ooooo1Ol0o`03ooooo`Go0?oo
0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0000
0oooool5ool000?ooooo1Ol00003ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo
3Oooool000_ooooo1Ol0ool3o`0000Goool02ol00005olc<c0?o00001Ol0o`02o`0000Go0?l00ol0
0005olc<c0?o00001Oo<c<`3o`0000Goo`3o0ol00005oooo00Wooooo1Oooo`03o`0000Goo`000ol0
0005o`00o`?o00001Ol0o`03o`0000Go0?l00ol00005o`3o00;o00001Ol0o`03o`0000Goc<c<0ol0
0005o`3oo`?o00001Oooo`0:ooooo`Go0?oo0_l00005ool000?o00001Ooo0003o`0000Go0?l00ol0
0005o`3o00?o00001Ol0o`03o`0000Go0?l00_l00005o`3oo`?o00001Ol0ool3o`0000Go0?oo2_oo
ool5o`3oo`?o00001Ol0ool2o`0000Goo`000ol00005ool000?o00001Ol0o`03o`0000Go0?l00ol0
0005o`3o00?o00001Ol0ool2o`0000Go0?oo0ol00005o`3oo`[ooooo1Ol0ool3o`0000Go0?oo0ol0
0005o`3oo`[o00001Ooo000;o`0000Go0?oo0ol00005o`3oo`;o00001Ol0ool3o`0000Go0?oo3Ooo
ool000_ooooo1Ol0ool3ooooo`Goool00oooool5o`0000?ooooo1Oo<c<`3ooooo`Go0?l00_ooool5
o`3o00?ooooo1Oo<c<`3ooooo`Goc<c<0oooool5ool0o`?ooooo1Oooo`09ooooo`Goool00oooool5
ool000?ooooo1Ol00?l3ooooo`Go0?l00oooool5o`3o00?ooooo1Ol0o`02ooooo`Go0?l00oooool5
olc<c0?ooooo1Ol0ool3ooooo`Goool02_ooool5o`3oo`;ooooo1Ooo0003ooooo`Goo`000oooool5
o`3o00?ooooo1Ol0o`03ooooo`Go0?l00oooool5o`3o00;ooooo1Ol0ool3ooooo`Go0?oo0oooool5
o`3oo`[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5ool000?ooooo1Ooo0003ooooo`Go0?l00oooool5
o`3o00?ooooo1Ol0o`03ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol0ool:ooooo`Go0?oo0oooool5
o`3oo`?ooooo1Ol0ool2ooooo`Go00000oooool5ool000?ooooo1Ol00003ooooo`Go0?oo0oooool5
o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo3Oooool000cooooo0ol0ool5ooooo`?oool01Oooool3o`00
00Gooooo0oo<c<`5ooooo`?o0?l01?ooool3o`3o00Gooooo0oo<c<`5ooooo`?oc<c<1Oooool3ool0
o`Gooooo0oooo`0;ooooo`?oool01Oooool3ool000Gooooo0ol00?l5ooooo`?o0?l01Oooool3o`3o
00Gooooo0ol0o`04ooooo`?o0?l01Oooool3olc<c0Gooooo0ol0ool5ooooo`?oool03?ooool3o`3o
o`Cooooo0ooo0005ooooo`?oo`001Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l01Oooool3o`3o
00Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`cooooo0ol0ool5ooooo`?o0?oo1?ooool3ool0
00Gooooo0ooo0005ooooo`?o0?l01Oooool3o`3o00Gooooo0ol0o`05ooooo`?o0?oo1?ooool3o`3o
o`Gooooo0ol0ool<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o00001Oooool3ool0
00Gooooo0ol00005ooooo`?o0?oo1Oooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo3_ooool000go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Co
oooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`04ooooo`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03Oooool000cooooo0oooo`05ooooo`?oo`001Oooool3ool000Gooooo0ol00?l5
ooooo`?oo`001?ooool3o`3oo`Gooooo0ol00?l5ooooo`?o0?l01Oooool3o`3oo`Gooooo0oooo`0;
ooooo`?o0?oo1Oooool3ool000Gooooo0ooo0005ooooo`?oo`001Oooool3o`3oo`Gooooo0ol0o`04
ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol00005ooooo`?oool03?ooool3o`3oo`Cooooo0ol0ool5
ooooo`?oo`001Oooool3ool000Gooooo0ol00005ooooo`?o0?l01Oooool3o`00o`Cooooo0ol00?l5
ooooo`?o003o1Oooool3o`0000cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005
ooooo`?oo`001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0000<
ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001Oooool3o`3oo`Cooooo0ol00005ooooo`?o00003_ooool000_ooooo1Oooo`03o`00
00Goo`000ol00005ool000?o00001Ol00?l3o`0000Goo`000_l00005o`3oo`?o00001Ol00?l3o`00
00Go0?l00ol00005o`3oo`?o00001Oooo`09ooooo`Go0?oo0ol00005ool000?o00001Ooo0003o`00
00Goo`000ol00005o`3oo`?o00001Ol0o`02o`0000Go0?oo0ol00005o`00o`_o00001Oooo`0:oooo
o`Go0?oo0_l00005o`3oo`?o00001Ooo0003o`0000Goo`002ol00005o`3o00?o00001Ol00?l2o`00
00Go003o0ol00005o`00o`So00002_ooool5o`3oo`?o00001Ol0ool2o`0000Go0?oo2ol00005ool0
01[o00001Ol0ool8o`0000[ooooo1Ol0ool3o`0000Go0?oo:_l00005o`3oo`oo00003Oooool000_o
oooo1Oooo`03ooooo`Goo`000oooool5ool000?ooooo1Ol00?l3ooooo`Goo`000_ooool5o`3oo`?o
oooo1Ol00?l3ooooo`Go0?l00oooool5o`3oo`?ooooo1Oooo`09ooooo`Go0?oo0oooool5ool000?o
oooo1Ooo0003ooooo`Goo`000oooool5o`3oo`?ooooo1Ol0o`02ooooo`Go0?oo0oooool5o`00o`?o
oooo1Ol00003ooooo`Goool02_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Goo`000oooool5ool000?o
oooo1Ol00003ooooo`Go0?l00oooool5o`00o`;ooooo1Ol00?l3ooooo`Go003o0oooool5o`0000[o
oooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol00003ooooo`Goo`000oooool5o`0000?o
oooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?o
oooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`3oo`;o
oooo1Ol00003ooooo`Go00003Oooool000_ooooo1Oooo`03ooooo`Goo`000oooool5ool000?ooooo
1Ol00?l3ooooo`Goo`000_ooool5o`3oo`?ooooo1Ol00?l3ooooo`Go0?l00oooool5o`3oo`?ooooo
1Oooo`09ooooo`Go0?oo0oooool5ool000?ooooo1Ooo0003ooooo`Goo`000oooool5o`3oo`?ooooo
1Ol0o`02ooooo`Go0?oo0oooool5o`00o`?ooooo1Ol00003ooooo`Goool02_ooool5o`3oo`;ooooo
1Ol0ool3ooooo`Goo`000oooool5ool000?ooooo1Ol00003ooooo`Go0?l00oooool5o`00o`;ooooo
1Ol00?l3ooooo`Go003o0oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo
1Ol00003ooooo`Goo`000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`3oo`?ooooo
1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00002ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000oooool5o`3oo`;ooooo1Ol00003ooooo`Go00003Oooool000cooooo0ooo
o`05ooooo`?oo`001Oooool3ool000Gooooo0ol00?l5ooooo`?oo`001?ooool3o`3oo`Gooooo0ol0
0?l5ooooo`?o0?l01Oooool3o`3oo`Gooooo0oooo`0;ooooo`?o0?oo1Oooool3ool000Gooooo0ooo
0005ooooo`?oo`001Oooool3o`3oo`Gooooo0ol0o`04ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol0
0005ooooo`?oool03?ooool3o`3oo`Cooooo0ol0ool5ooooo`?oo`001Oooool3ool000Gooooo0ol0
0005ooooo`?o0?l01Oooool3o`00o`Cooooo0ol00?l5ooooo`?o003o1Oooool3o`0000cooooo0ol0
ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?oo`001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001?ooool3o`3oo`Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0
0004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`3oo`Cooooo0ol0
0005ooooo`?o00003_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=
ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo000<ooooo`?o
0?oo1Oooool3olc<c0Gooooo0ol00?l5ooooo`?oool01Oooool3ool0o`Cooooo0ol00005ooooo`?o
c<c<1Oooool3o`00o`Gooooo0ol00005ooooo`?oo`3o2oooool3o`3oo`Gooooo0ol0ool5ooooo`?o
o`001Oooool3o`00o`Gooooo0ol00005ooooo`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000cooooo0ol0ool4ooooo`?o0?oo1Oooool3ool000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol0000<ooooo`?o
0?oo1Oooool3o`3oo`Cooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o
00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003?ooool3o`3oo`Gooooo0ol0ool5ooooo`?o
00001?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00004ooooo`?o
00001Oooool3o`0000kooooo000;ooooo`Go0?oo0oooool5olc<c0?ooooo1Ol00?l3ooooo`Goool0
0oooool5ool0o`;ooooo1Ol00003ooooo`Goc<c<0oooool5o`00o`?ooooo1Ol00003ooooo`Goo`3o
2Oooool5o`3oo`?ooooo1Ol0ool3ooooo`Goo`000oooool5o`00o`?ooooo1Ol00003ooooo`Go0?oo
0_ooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol0ool2ooooo`Go0?oo
0oooool5ool000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00002ooooo`Go0000
0oooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`;ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go0000
2_ooool5o`3oo`?ooooo1Ol0ool3ooooo`Go00000_ooool5o`0000?ooooo1Ol00003ooooo`Go0000
0oooool5o`0000?ooooo1Ol00002ooooo`Go00000oooool5o`0000gooooo000;ooooo`Go0?oo0ol0
0005olc<c0?o00001Ol00?l3o`0000Goool00ol00005ool0o`[o00001Oo<c<`3o`0000Go003o2ol0
0005ool0o`Wooooo1Ol0ool3o`0000Go0?oo0ol00005ool000?o00001Ol00?l;o`0000Go0?oo7ol0
000:ooooo`Go0?oo0_l00005o`3oo`?o00001Ooo000go`0000[ooooo1Ol0ool3o`0000Go0?oo?_l0
000:ooooo`Go0?oo0ol00005o`3oocko00003Oooool000_ooooo1Ol0ool3ooooo`Goc<c<0oooool5
o`00o`?ooooo1Oooo`03ooooo`Goo`3o0_ooool5o`0000?ooooo1Oo<c<`3ooooo`Go003o0oooool5
o`0000?ooooo1Ooo0?l9ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ooo0003ooooo`Go003o0oooool5
o`0000?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00002_ooool5
o`3oo`;ooooo1Ol0ool3ooooo`Goo`000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5
o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5
o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol00002ooooo`Go00000oooool5
o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool0
00cooooo0ol0ool5ooooo`?oc<c<1Oooool3o`00o`Gooooo0oooo`05ooooo`?oo`3o1?ooool3o`00
00Gooooo0oo<c<`5ooooo`?o003o1Oooool3o`0000Gooooo0ooo0?l;ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ooo0005ooooo`?o003o1Oooool3o`0000Gooooo0ol0ool4ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00003?ooool3o`3oo`Cooooo0ol0ool5ooooo`?oo`001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`00
00cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`00
00Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3o
o`Gooooo0ol00004ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`00
00Cooooo0ol00005ooooo`?o00003_ooool000gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo
000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000cooooo0oo<c<`5
ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol00?l5ooooo`?o0?l01?ooool3o`3oo`Gooooo0oooo`05
ooooo`?oool01Oooool3o`3o00Gooooo0ooo0?l;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00?l5
ooooo`?o003o1Oooool3ool0o`Gooooo0ol00004ooooo`?oool01Oooool3ool0o`Gooooo0ol00005
ooooo`?o0?l03?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol00005
ooooo`?o0?oo1Oooool3o`0000Cooooo0ol00005ooooo`?o00001Oooool3o`0000cooooo0ol0ool5
ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol00005ooooo`?o00001Oooool3o`0000Gooooo0ol00005
ooooo`?o00001?ooool3o`0000Gooooo0ol0000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4
ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001Oooool3o`0000Cooooo0ol00005
ooooo`?o00003_ooool000_ooooo1Oo<c<`3ooooo`Go0?oo0oooool5o`00o`?ooooo1Ol00?l3oooo
o`Go0?l00_ooool5o`3oo`?ooooo1Oooo`03ooooo`Goool00oooool5o`3o00?ooooo1Ooo0?l9oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol00?l3ooooo`Go003o0oooool5ool0o`?ooooo1Ol00002oooo
o`Goool00oooool5ool0o`?ooooo1Ol00003ooooo`Go0?l02_ooool5o`3oo`;ooooo1Ol0ool3oooo
o`Go0?oo0oooool5o`00o`?ooooo1Ol00003ooooo`Go0?oo0oooool5o`0000;ooooo1Ol00003oooo
o`Go00000oooool5o`0000[ooooo1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol00003oooo
o`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:oooo
o`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003oooo
o`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Oooool000_ooooo1Oo<c<`3o`0000Go
0?oo0ol00005o`00o`?o00001Ol00?l3o`0000Go0?l00_l00005o`3oo`?o00001Oooo`03o`0000Go
ool00ol00005o`3o00?o00001Ooo0?l9ooooo`Go0?oo0ol00005o`3oo`?o00001Ol00?l3o`0000Go
003o0ol00005ool0o`[o00001Oooo`03o`0000Goo`3o2ol00005o`3o00[ooooo1Ol0ool2o`0000Go
0?oo0ol00005o`3oo`?o00001Ol00?l;o`0000Go0?oo7ol0000:ooooo`Go0?oo0ol00005o`3oo`;o
00001Ol0oolgo`0000[ooooo1Ol0ool3o`0000Go0?oo0ol00005o`3oocKo00003Oooool000_ooooo
1Oo<c<`3ooooo`Go0?oo0oooool5o`00o`?ooooo1Ol00?l3ooooo`Go0?l00_ooool5o`3oo`?ooooo
1Oooo`03ooooo`Goool00oooool5o`3o00?ooooo1Ooo0?l9ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol00?l3ooooo`Go003o0oooool5ool0o`?ooooo1Ol00002ooooo`Goool00oooool5ool0o`?ooooo
1Ol00003ooooo`Go0?l02_ooool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`00o`?ooooo
1Ol00003ooooo`Go0?oo0oooool5o`0000;ooooo1Ol00003ooooo`Go00000oooool5o`0000[ooooo
1Ol0ool3ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Ol00003ooooo`Go00000oooool5o`0000?ooooo
1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?ooooo
1Ol0ool2ooooo`Go00000oooool5o`0000?ooooo1Ol00003ooooo`Go00000oooool5o`0000;ooooo
1Ol00003ooooo`Go00003Oooool000cooooo0oo<c<`5ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol0
0?l5ooooo`?o0?l01?ooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3o`3o00Gooooo0ooo
0?l;ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol00?l5ooooo`?o003o1Oooool3ool0o`Gooooo0ol0
0004ooooo`?oool01Oooool3ool0o`Gooooo0ol00005ooooo`?o0?l03?ooool3o`3oo`Cooooo0ol0
ool5ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol00005ooooo`?o0?oo1Oooool3o`0000Cooooo0ol0
0005ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Gooooo0ol00005ooooo`?o00001?ooool3o`0000Gooooo0ol0
000<ooooo`?o0?oo1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?o00001Oooool3o`0000Gooooo0ol0
0005ooooo`?o00001Oooool3o`0000Cooooo0ol00005ooooo`?o00003_ooool000gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool03Oooool000cooooo0ol0ool5ooooo`?o0?l01Oooool3oooo00Gooooo0ol00?l5ooooo`?o0?l0
1?ooool3o`3o00Gooooo0ol00005ooooo`?oo`3o1Oooool3o`3o00Gooooo0oooo`0;ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0o`05ooooo`?oool01Oooool3o`3o00Gooooo0ol0o`04ooooo`?o0?l0
1Oooool3o`3o00Gooooo0ooo0?l5ooooo`?o0?l03?ooool3o`3oo`Cooooo0ol0ool5ooooo`?o0?oo
1Oooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3o`3o00Cooooo0ooo0?l5ooooo`?o0?l0
1Oooool3ool000cooooo0ol0ool5ooooo`?o0?oo1?ooool3o`3oo`Gooooo0oooo`05ooooo`?o0000
1Oooool3oooo00Gooooo0ooo0?l5ooooo`?oo`3o1?ooool3o`0000Gooooo0ol0000<ooooo`?o0?oo
1Oooool3o`3oo`Gooooo0ol0ool4ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?o0000
1Oooool3o`0000Cooooo0ol00005ooooo`?o00003_ooool000_ooooo1Ol0ool3ooooo`Go0?l00ooo
ool5oooo00?ooooo1Ol00?l3ooooo`Go0?l00_ooool5o`3o00?ooooo1Ol00003ooooo`Goo`3o0ooo
ool5o`3o00?ooooo1Oooo`09ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0o`03ooooo`Goool00ooo
ool5o`3o00?ooooo1Ol0o`02ooooo`Go0?l00oooool5o`3o00?ooooo1Ooo0?l3ooooo`Go0?l02_oo
ool5o`3oo`;ooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00ooo
ool5o`3o00;ooooo1Ooo0?l3ooooo`Go0?l00oooool5ool000[ooooo1Ol0ool3ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Oooo`03ooooo`Go00000oooool5oooo00?ooooo1Ooo0?l3ooooo`Goo`3o0_oo
ool5o`0000?ooooo1Ol0000:ooooo`Go0?oo0oooool5o`3oo`?ooooo1Ol0ool2ooooo`Goool00ooo
ool5oooo00?ooooo1Oooo`03ooooo`Go00000oooool5o`0000;ooooo1Ol00003ooooo`Go00003Ooo
ool000_ooooo1Ol0ool3o`0000Go0?l00ol00005oooo00?o00001Ol00?l3o`0000Go0?l00_l00005
o`3o00_o00001Ooo0?l3o`0000Go0?l00ol00005oooo00Wooooo1Ol0ool3o`0000Go0?oo0ol00005
o`3o00?o00001Oooo`03o`0000Go0?l00ol00005o`3o00;o00001Ol0o`03o`0000Go0?l00ol00005
ool0o`?o00001Ol0o`0:ooooo`Go0?oo0_l00005o`3oo`?o00001Ol0ool3o`0000Goool00ol00005
oooo00?o00001Ol0o`03o`0000Go0?l00_l00005ool0o`?o00001Ol0o`03o`0000Goo`002_ooool5
o`3oo`?o00001Ol0ool2o`0000Go0?oo0ol00005oooo00_o00001Oooo`03o`0000Goo`3o0ol00005
ool0o`oo00002_ooool5o`3oo`?o00001Ol0ool3o`0000Go0?oo0_l00005oooo00?o00001Oooo`03
o`0000Goool07ol0000=ooooo`002oooool5o`3oo`?ooooo1Ol0o`03ooooo`Goool00oooool5o`00
o`?ooooo1Ol0o`02ooooo`Go0?l00oooool5o`0000?ooooo1Ooo0?l3ooooo`Go0?l00oooool5oooo
00Wooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3o00?ooooo1Oooo`03ooooo`Go0?l00oooool5o`3o
00;ooooo1Ol0o`03ooooo`Go0?l00oooool5ool0o`?ooooo1Ol0o`0:ooooo`Go0?oo0_ooool5o`3o
o`?ooooo1Ol0ool3ooooo`Goool00oooool5oooo00?ooooo1Ol0o`03ooooo`Go0?l00_ooool5ool0
o`?ooooo1Ol0o`03ooooo`Goo`002_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Go0?oo0oooool5oooo
00?ooooo1Ol00003ooooo`Goool00oooool5ool0o`?ooooo1Ooo0?l2ooooo`Go00000oooool5o`00
00[ooooo1Ol0ool3ooooo`Go0?oo0oooool5o`3oo`;ooooo1Oooo`03ooooo`Goool00oooool5oooo
00?ooooo1Ol00003ooooo`Go00000_ooool5o`0000?ooooo1Ol0000=ooooo`003?ooool3o`3oo`Go
oooo0ol0o`05ooooo`?oool01Oooool3o`00o`Gooooo0ol0o`04ooooo`?o0?l01Oooool3o`0000Go
oooo0ooo0?l5ooooo`?o0?l01Oooool3oooo00_ooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3o00Go
oooo0oooo`05ooooo`?o0?l01Oooool3o`3o00Cooooo0ol0o`05ooooo`?o0?l01Oooool3ool0o`Go
oooo0ol0o`0<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0ol0ool5ooooo`?oool01Oooool3oooo00Go
oooo0ol0o`05ooooo`?o0?l01?ooool3ool0o`Gooooo0ol0o`05ooooo`?oo`003?ooool3o`3oo`Go
oooo0ol0ool4ooooo`?o0?oo1Oooool3oooo00Gooooo0ol00005ooooo`?oool01Oooool3ool0o`Go
oooo0ooo0?l4ooooo`?o00001Oooool3o`0000cooooo0ol0ool5ooooo`?o0?oo1Oooool3o`3oo`Co
oooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol00005ooooo`?o00001?ooool3o`0000Go
oooo0ol0000>ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00gooooo
000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`0000Gooooo0oooo`05oooo
o`?o00001Oooool3oooo00Gooooo0oooo`04ooooo`?o0?oo1Oooool3o`00o`Gooooo0ol0o`05oooo
o`?oo`001Oooool3ool000_ooooo0ol0ool5ooooo`?oool01Oooool3oooo00Gooooo0oooo`05oooo
o`?oool01Oooool3o`3o00Cooooo0ol0o`05ooooo`?o0?l01Oooool3o`3o00Gooooo0ooo000<oooo
o`?o0?oo1?ooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05oooo
o`?o0?l01?ooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l03?ooool3o`3oo`Gooooo0ol0ool4oooo
o`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ooo0?l4oooo
o`?o0?l01Oooool3o`3o00cooooo0ol0ool5ooooo`?o0?oo1Oooool3oooo00Cooooo0oooo`05oooo
o`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oo`3o1?ooool3o`0000Gooooo0ol0000>oooo
o`002oooool5o`0000?ooooo1Oooo`03ooooo`Go00000oooool5oooo00?ooooo1Oooo`02ooooo`Go
0?oo0oooool5o`00o`?ooooo1Ol0o`03ooooo`Goo`000oooool5ool000Wooooo1Ol0ool3ooooo`Go
ool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5o`3o00;ooooo1Ol0o`03ooooo`Go
0?l00oooool5o`3o00?ooooo1Ooo000:ooooo`Go0?oo0_ooool5o`3oo`?ooooo1Oooo`03ooooo`Go
ool00oooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00_ooool5o`3o00?ooooo1Ol0o`03ooooo`Go
0?l02_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go
ool00oooool5oooo00?ooooo1Ooo0?l2ooooo`Go0?l00oooool5o`3o00[ooooo1Ol0ool3ooooo`Go
0?oo0oooool5oooo00;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go
o`3o0_ooool5o`0000?ooooo1Ol0000=ooooo`002oooool8o`0000Goool02ol00005oooo00?o0000
1Oooo`02o`0000Go0?oo0ol00005o`00o`?o00001Ol0o`03o`0000Goo`000ol00005ool000Wooooo
1Ol0ool3o`0000Goool00ol00005oooo00?o00001Oooo`03o`0000Goool00ol00005o`3o00;o0000
1Ol0o`03o`0000Go0?l00ol00005o`3o00?o00001Ooo000:ooooo`Go0?oo0_l00005o`3oo`?o0000
1Oooo`03o`0000Goool00ol00005oooo00?o00001Oooo`03o`0000Go0?l00_l00005o`3o00?o0000
1Ol0o`03o`0000Go0?l02_ooool5o`3oo`?o00001Ol0ool2o`0000Goool00ol00005oooo00?o0000
1Oooo`03o`0000Goool00ol00005oooo00?o00001Ooo0?l2o`0000Go0?l00ol00005o`3o00[ooooo
1Ol0ool3o`0000Go0?oo0ol00005oooo00;o00001Oooo`03o`0000Goool00ol00005oooo00?o0000
1Oooo`03o`0000Goo`3o3ol0000=ooooo`002oooool5o`0000?ooooo1Oooo`03ooooo`Go00000ooo
ool5oooo00?ooooo1Oooo`02ooooo`Go0?oo0oooool5o`00o`?ooooo1Ol0o`03ooooo`Goo`000ooo
ool5ool000Wooooo1Ol0ool3ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00ooo
ool5o`3o00;ooooo1Ol0o`03ooooo`Go0?l00oooool5o`3o00?ooooo1Ooo000:ooooo`Go0?oo0_oo
ool5o`3oo`?ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Go0?l00_oo
ool5o`3o00?ooooo1Ol0o`03ooooo`Go0?l02_ooool5o`3oo`?ooooo1Ol0ool2ooooo`Goool00ooo
ool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Ooo0?l2ooooo`Go0?l00ooo
ool5o`3o00[ooooo1Ol0ool3ooooo`Go0?oo0oooool5oooo00;ooooo1Oooo`03ooooo`Goool00ooo
ool5oooo00?ooooo1Oooo`03ooooo`Goo`3o0_ooool5o`0000?ooooo1Ol0000=ooooo`003?ooool3
o`0000Gooooo0oooo`05ooooo`?o00001Oooool3oooo00Gooooo0oooo`04ooooo`?o0?oo1Oooool3
o`00o`Gooooo0ol0o`05ooooo`?oo`001Oooool3ool000_ooooo0ol0ool5ooooo`?oool01Oooool3
oooo00Gooooo0oooo`05ooooo`?oool01Oooool3o`3o00Cooooo0ol0o`05ooooo`?o0?l01Oooool3
o`3o00Gooooo0ooo000<ooooo`?o0?oo1?ooool3o`3oo`Gooooo0oooo`05ooooo`?oool01Oooool3
oooo00Gooooo0oooo`05ooooo`?o0?l01?ooool3o`3o00Gooooo0ol0o`05ooooo`?o0?l03?ooool3
o`3oo`Gooooo0ol0ool4ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3
oooo00Gooooo0ooo0?l4ooooo`?o0?l01Oooool3o`3o00cooooo0ol0ool5ooooo`?o0?oo1Oooool3
oooo00Cooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oo`3o1?ooool3
o`0000Gooooo0ol0000>ooooo`003Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00gooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003o
ooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03Oooool000gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`0=ooooo`003?ooool3o`3o00Gooooo0ooo
o`05ooooo`?oool01Oooool3o`3oo`Gooooo0ooo0?l4ooooo`?o0?l01Oooool3ool0o`Gooooo0oo<
c<`5ooooo`?o003o1Oooool3o`0000_ooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ooo
0?l5ooooo`?o0?l01Oooool3o`3o00Cooooo0ol0o`05ooooo`?o0?l01Oooool3ool000Gooooo0ooo
000<ooooo`?oool01?ooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0
o`05ooooo`?o0?l01?ooool3ool000Gooooo0ol0o`05ooooo`?oo`003?ooool3o`3oo`Gooooo0ooo
o`04ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oool01Oooool3o`3o00Gooooo0ol0
o`04ooooo`?o0?l01Oooool3o`3o00cooooo0ol0ool5ooooo`?oool01Oooool3oooo00Cooooo0ooo
o`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oo`3o1?ooool3o`3o00Gooooo0ol0
o`0>ooooo`002oooool5o`3o00?ooooo1Oooo`03ooooo`Goool00oooool5o`3oo`?ooooo1Ooo0?l2
ooooo`Go0?l00oooool5ool0o`?ooooo1Oo<c<`3ooooo`Go003o0oooool5o`0000Wooooo1Oooo`03
ooooo`Goool00oooool5oooo00?ooooo1Ooo0?l3ooooo`Go0?l00oooool5o`3o00;ooooo1Ol0o`03
ooooo`Go0?l00oooool5ool000?ooooo1Ooo000:ooooo`Goool00_ooool5oooo00?ooooo1Oooo`03
ooooo`Goool00oooool5oooo00?ooooo1Ol0o`03ooooo`Go0?l00_ooool5ool000?ooooo1Ol0o`03
ooooo`Goo`002_ooool5o`3oo`?ooooo1Oooo`02ooooo`Goool00oooool5oooo00?ooooo1Oooo`03
ooooo`Goool00oooool5o`3o00?ooooo1Ol0o`02ooooo`Go0?l00oooool5o`3o00[ooooo1Ol0ool3
ooooo`Goool00oooool5oooo00;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?ooooo1Oooo`03
ooooo`Goo`3o0_ooool5o`3o00?ooooo1Ol0o`0=ooooo`002oooool5o`3o00?o00001Oooo`03o`00
00Goool00ol00005o`3oo`?o00001Ooo0?l2o`0000Go0?l00ol00005ool0o`?o00001Oo<c<`3o`00
00Go003o2?l00009ooooo`Goool00ol00005oooo00?o00001Oooo`03o`0000Goo`3o0ol00005o`3o
00?o00001Ol0o`02o`0000Go0?l00ol00005o`3o00?o00001Ooo0003o`0000Goo`002_ooool5oooo
00;o00001Oooo`03o`0000Goool00ol00005oooo00?o00001Oooo`03o`0000Go0?l00ol00005o`3o
00;o00001Ooo0003o`0000Go0?l00ol00005ool000[ooooo1Ol0ool3o`0000Goool00_l00005oooo
00?o00001Oooo`03o`0000Goool00ol00005oooo00?o00001Ol0o`03o`0000Go0?l00_l00005o`3o
00?o00001Ol0o`0:ooooo`Go0?oo0ol00005oooo00?o00001Oooo`02o`0000Goool00ol00005oooo
00?o00001Oooo`03o`0000Goool00ol00005ool0o`;o00001Ol0o`03o`0000Go0?l03Oooool000_o
oooo1Ol0o`03ooooo`Goool00oooool5oooo00?ooooo1Ol0ool3ooooo`Goo`3o0_ooool5o`3o00?o
oooo1Ooo0?l3ooooo`Goc<c<0oooool5o`00o`?ooooo1Ol00009ooooo`Goool00oooool5oooo00?o
oooo1Oooo`03ooooo`Goo`3o0oooool5o`3o00?ooooo1Ol0o`02ooooo`Go0?l00oooool5o`3o00?o
oooo1Ooo0003ooooo`Goo`002_ooool5oooo00;ooooo1Oooo`03ooooo`Goool00oooool5oooo00?o
oooo1Oooo`03ooooo`Go0?l00oooool5o`3o00;ooooo1Ooo0003ooooo`Go0?l00oooool5ool000[o
oooo1Ol0ool3ooooo`Goool00_ooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5oooo00?o
oooo1Ol0o`03ooooo`Go0?l00_ooool5o`3o00?ooooo1Ol0o`0:ooooo`Go0?oo0oooool5oooo00?o
oooo1Oooo`02ooooo`Goool00oooool5oooo00?ooooo1Oooo`03ooooo`Goool00oooool5ool0o`;o
oooo1Ol0o`03ooooo`Go0?l03Oooool000cooooo0ol0o`05ooooo`?oool01Oooool3oooo00Gooooo
0ol0ool5ooooo`?oo`3o1?ooool3o`3o00Gooooo0ooo0?l5ooooo`?oc<c<1Oooool3o`00o`Gooooo
0ol0000;ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?oo`3o1Oooool3o`3o00Gooooo
0ol0o`04ooooo`?o0?l01Oooool3o`3o00Gooooo0ooo0005ooooo`?oo`003?ooool3oooo00Cooooo
0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0oooo`05ooooo`?o0?l01Oooool3o`3o00Cooooo
0ooo0005ooooo`?o0?l01Oooool3ool000cooooo0ol0ool5ooooo`?oool01?ooool3oooo00Gooooo
0oooo`05ooooo`?oool01Oooool3oooo00Gooooo0ol0o`05ooooo`?o0?l01?ooool3o`3o00Gooooo
0ol0o`0<ooooo`?o0?oo1Oooool3oooo00Gooooo0oooo`04ooooo`?oool01Oooool3oooo00Gooooo
0oooo`05ooooo`?oool01Oooool3ool0o`Cooooo0ol0o`05ooooo`?o0?l03_ooool00?oooooo^Ooo
ool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Ooo
ool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Oooool00?oooooo^Ooo
ool00001\
\>"],
  ImageRangeCache->{{{0, 439}, {439, 0}} -> {-0.125026, -0.125026, 0.0119591, \
0.0119591}, {{10.4375, 87.875}, {428.5, 351.062}} -> {-0.198272, -0.199159, \
0.0142051, 0.0142051}, {{95.5625, 173}, {428.5, 351.062}} -> {-1.40748, \
-0.199159, 0.0142051, 0.0142051}, {{180.75, 258.188}, {428.5, 351.062}} -> \
{-2.61759, -0.199159, 0.0142051, 0.0142051}, {{265.938, 343.375}, {428.5, \
351.062}} -> {-3.82769, -0.199159, 0.0142051, 0.0142051}, {{351.062, 428.5}, \
{428.5, 351.062}} -> {-5.0369, -0.199159, 0.0142051, 0.0142051}, {{10.4375, \
87.875}, {343.375, 265.938}} -> {-0.198272, -1.40837, 0.0142051, 0.0142051}, \
{{95.5625, 173}, {343.375, 265.938}} -> {-1.40748, -1.40837, 0.0142051, \
0.0142051}, {{180.75, 258.188}, {343.375, 265.938}} -> {-2.61759, -1.40837, \
0.0142051, 0.0142051}, {{265.938, 343.375}, {343.375, 265.938}} -> {-3.82769, \
-1.40837, 0.0142051, 0.0142051}, {{351.062, 428.5}, {343.375, 265.938}} -> \
{-5.0369, -1.40837, 0.0142051, 0.0142051}, {{10.4375, 87.875}, {258.188, \
180.75}} -> {-0.198272, -2.61847, 0.0142051, 0.0142051}, {{95.5625, 173}, \
{258.188, 180.75}} -> {-1.40748, -2.61847, 0.0142051, 0.0142051}, {{180.75, \
258.188}, {258.188, 180.75}} -> {-2.61759, -2.61847, 0.0142051, 0.0142051}, \
{{265.938, 343.375}, {258.188, 180.75}} -> {-3.82769, -2.61847, 0.0142051, \
0.0142051}, {{351.062, 428.5}, {258.188, 180.75}} -> {-5.0369, -2.61847, \
0.0142051, 0.0142051}, {{10.4375, 87.875}, {173, 95.5625}} -> {-0.198272, \
-3.82857, 0.0142051, 0.0142051}, {{95.5625, 173}, {173, 95.5625}} -> \
{-1.40748, -3.82857, 0.0142051, 0.0142051}, {{180.75, 258.188}, {173, \
95.5625}} -> {-2.61759, -3.82857, 0.0142051, 0.0142051}, {{265.938, 343.375}, \
{173, 95.5625}} -> {-3.82769, -3.82857, 0.0142051, 0.0142051}, {{351.062, \
428.5}, {173, 95.5625}} -> {-5.0369, -3.82857, 0.0142051, 0.0142051}, \
{{10.4375, 87.875}, {87.875, 10.4375}} -> {-0.198272, -5.03779, 0.0142051, \
0.0142051}, {{95.5625, 173}, {87.875, 10.4375}} -> {-1.40748, -5.03779, \
0.0142051, 0.0142051}, {{180.75, 258.188}, {87.875, 10.4375}} -> {-2.61759, \
-5.03779, 0.0142051, 0.0142051}, {{265.938, 343.375}, {87.875, 10.4375}} -> \
{-3.82769, -5.03779, 0.0142051, 0.0142051}, {{351.062, 428.5}, {87.875, \
10.4375}} -> {-5.0369, -5.03779, 0.0142051, 0.0142051}},
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[60]=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[ugVN]\)], "Input",
  CellLabel->"In[61]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moore 8", "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[BoxData[
    \(\(ugM8 = 
        LocalInteractionModel[\[IndentingNewLine]InteractionNeighborhood \
\[Rule] Moore8[10, 10], \[IndentingNewLine]PayoffMatrix \[Rule] 
            ultGame, \[IndentingNewLine]StrategyColors \[Rule] 
            ugColors\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[63]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ShowGraph[
        ExtractGraph[ugM8], \[IndentingNewLine]VertexStyle \[Rule] 
          Disk[Large]\[IndentingNewLine]];\)\)], "Input",
  CellLabel->"In[64]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
0 1 1 r
.14646 .04545 Mdot
0 0 0 r
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
0 0 1 r
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
1 0 1 r
.75253 .04545 Mdot
.85354 .04545 Mdot
1 1 0 r
.95455 .04545 Mdot
1 0 0 r
.04545 .14646 Mdot
0 0 0 r
.14646 .14646 Mdot
0 1 0 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
1 0 1 r
.44949 .14646 Mdot
0 0 0 r
.55051 .14646 Mdot
0 0 1 r
.65152 .14646 Mdot
0 0 0 r
.75253 .14646 Mdot
0 0 1 r
.85354 .14646 Mdot
1 0 1 r
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.8 .8 .8 r
.14646 .24747 Mdot
1 0 1 r
.24747 .24747 Mdot
1 0 0 r
.34848 .24747 Mdot
0 0 0 r
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
0 0 1 r
.65152 .24747 Mdot
1 0 1 r
.75253 .24747 Mdot
.8 .8 .8 r
.85354 .24747 Mdot
1 1 0 r
.95455 .24747 Mdot
0 1 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.8 .8 .8 r
.24747 .34848 Mdot
0 1 1 r
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 0 r
.55051 .34848 Mdot
0 0 0 r
.65152 .34848 Mdot
1 0 0 r
.75253 .34848 Mdot
0 0 1 r
.85354 .34848 Mdot
0 1 0 r
.95455 .34848 Mdot
0 0 1 r
.04545 .44949 Mdot
1 0 1 r
.14646 .44949 Mdot
0 0 1 r
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
0 0 1 r
.44949 .44949 Mdot
0 1 0 r
.55051 .44949 Mdot
1 0 1 r
.65152 .44949 Mdot
1 0 0 r
.75253 .44949 Mdot
0 1 0 r
.85354 .44949 Mdot
1 0 0 r
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
1 0 0 r
.14646 .55051 Mdot
1 0 1 r
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.8 .8 .8 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
1 0 0 r
.75253 .55051 Mdot
0 1 0 r
.85354 .55051 Mdot
1 0 1 r
.95455 .55051 Mdot
0 0 1 r
.04545 .65152 Mdot
.8 .8 .8 r
.14646 .65152 Mdot
0 0 1 r
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
1 0 0 r
.44949 .65152 Mdot
0 1 0 r
.55051 .65152 Mdot
1 0 0 r
.65152 .65152 Mdot
1 0 1 r
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
1 0 1 r
.04545 .75253 Mdot
0 0 1 r
.14646 .75253 Mdot
0 1 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
1 0 0 r
.44949 .75253 Mdot
0 0 1 r
.55051 .75253 Mdot
1 0 0 r
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
0 1 1 r
.95455 .75253 Mdot
.8 .8 .8 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
0 0 1 r
.24747 .85354 Mdot
1 1 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
0 0 0 r
.55051 .85354 Mdot
0 0 1 r
.65152 .85354 Mdot
1 1 0 r
.75253 .85354 Mdot
1 0 1 r
.85354 .85354 Mdot
.8 .8 .8 r
.95455 .85354 Mdot
.04545 .95455 Mdot
0 0 0 r
.14646 .95455 Mdot
1 0 1 r
.24747 .95455 Mdot
0 0 1 r
.34848 .95455 Mdot
0 0 0 r
.44949 .95455 Mdot
1 1 0 r
.55051 .95455 Mdot
1 0 1 r
.65152 .95455 Mdot
0 0 0 r
.75253 .95455 Mdot
1 0 1 r
.85354 .95455 Mdot
0 0 1 r
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[64]:=",
  ImageSize->{288, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`002_ooool6o`0001Oooooo1_l0
oolGooooo`Ko00005oooool6o`3o01Oooooo1_l0o`0Gooooo`Ko003o5oooool6ool001Oooooo1_oo
0?lGooooo`Koo`3o5oooool6oooo00_ooooo0008ooooo`[o00004oooool:o`3ooa?ooooo2_l0000C
ooooo`[o0?l04oooool:o`3o01?ooooo2_l00?lCooooo`[oo`004oooool:ool0oa?ooooo2_oo0?lC
ooooo`[oool02Oooool000Kooooo3_l0000?ooooo`ko0?oo3oooool>o`0000oooooo3_l0o`0?oooo
o`ko0?l03oooool>o`00o`oooooo3_oo000?ooooo`koo`3o3oooool>ool0o`oooooo3_ooo`07oooo
o`001Oooool@o`0000gooooo4?l0ool=oooooa3o00003Oooool@o`3o00gooooo4?l0o`0=oooooa3o
003o3Oooool@ool000gooooo4?oo0?l=oooooa3oo`3o3Oooool@oooo00Kooooo0005oooooa3o0000
3Oooool@o`3oo`gooooo4?l0000=oooooa3o0?l03Oooool@o`3o00gooooo4?l00?l=oooooa3oo`00
3Oooool@ool0o`gooooo4?oo0?l=oooooa3oool01_ooool000Cooooo4_l0000;oooooa;o0?oo2ooo
oolBo`0000_ooooo4_l0o`0;oooooa;o0?l02ooooolBo`00o`_ooooo4_oo000;oooooa;oo`3o2ooo
oolBool0o`_ooooo4_ooo`05ooooo`001?oooolBo`0000_ooooo4_l0ool;oooooa;o00002ooooolB
o`3o00_ooooo4_l0o`0;oooooa;o003o2ooooolBool000_ooooo4_oo0?l;oooooa;oo`3o2ooooolB
oooo00Gooooo0003oooooaCo00002OoooolDo`3oo`Wooooo5?l00009oooooaCo0?l02OoooolDo`3o
00Wooooo5?l00?l9oooooaCoo`002OoooolDool0o`Wooooo5?oo0?l9oooooaCoool01?ooool000?o
oooo5?l00009oooooaCo0?oo2OoooolDo`0000Wooooo5?l0o`09oooooaCo0?l02OoooolDo`00o`Wo
oooo5?oo0009oooooaCoo`3o2OoooolDool0o`Wooooo5?ooo`04ooooo`000ooooolDo`0000Wooooo
5?l0ool9oooooaCo00002OoooolDo`3o00Wooooo5?l0o`09oooooaCo003o2OoooolDool000Wooooo
5?oo0?l9oooooaCoo`3o2OoooolDoooo00Cooooo0003oooooago00005?l0oolVo`0001Co0?l02Ol0
000Do`3o00Wo00005?l00?l9o`0001Coo`002Ol0000Dool0o`Wo00005?oo0?l9o`0001Coool01?oo
ool000?ooooo5?l00009oooooaCo0?oo2OoooolDo`0000Wooooo5?l0o`09oooooaCo0?l02OoooolD
o`00o`Wooooo5?oo0009oooooaCoo`3o2OoooolDool0o`Wooooo5?ooo`04ooooo`000ooooolDo`00
00Wooooo5?l0ool9oooooaCo00002OoooolDo`3o00Wooooo5?l0o`09oooooaCo003o2OoooolDool0
00Wooooo5?oo0?l9oooooaCoo`3o2OoooolDoooo00Cooooo0004oooooa;o00002ooooolBo`3oo`_o
oooo4_l0000;oooooa;o0?l02ooooolBo`3o00_ooooo4_l00?l;oooooa;oo`002ooooolBool0o`_o
oooo4_oo0?l;oooooa;oool01Oooool000Cooooo4_l0000;oooooa;o0?oo2ooooolBo`0000_ooooo
4_l0o`0;oooooa;o0?l02ooooolBo`00o`_ooooo4_oo000;oooooa;oo`3o2ooooolBool0o`_ooooo
4_ooo`05ooooo`001Oooool@o`0000gooooo4?l0ool=oooooa3o00003Oooool@o`3o00gooooo4?l0
o`0=oooooa3o003o3Oooool@ool000gooooo4?oo0?l=oooooa3oo`3o3Oooool@oooo00Kooooo0005
oooooa3o00003Oooool@o`3oo`gooooo4?l0000=oooooa3o0?l03Oooool@o`3o00gooooo4?l00?l=
oooooa3oo`003Oooool@ool0o`gooooo4?oo0?l=oooooa3oool01_ooool000Kooooo3ol0000>oooo
o`ko0?oo00?o0000ooooooooool03?ooool?o`0000kooooo3_l0o`000ol0003oooooooooo`0<oooo
o`ko0?l03oooool>o`00o`kooooo00?o0000ool00?oo00003?oo000>ooooo`03o`000?oo0?ooo`3o
00coo`3o3_ooool00ol0003oo`3oool0o`0<ool0o`kooooo00?o0000oooo0?ooo`003?ooo`07oooo
o`002?ooool:o`0000?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:o`3oo`?o
oooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:o`0000?ooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0:o`3o00?ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0:o`3o00;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0:o`00o`;o
oooo00?o0000ooooooooool02_ooool01?l0003oooooooooooooool:ool000;ooooo00?o0000oooo
ooooool02_ooool01?l0003oooooooooooooool:ool0o`;ooooo00?o0000ooooooooool02_ooool0
1?l0003oooooooooooooool:ool0o`;ooooo00?o0000ooooooooool02_ooool01?l0003ooooooooo
ooooool:oooo00Wooooo000:ooooo`Ko00001_ooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00?ooooo1_l0ool6ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00ooo
ool6o`0000Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`Ko0?l01_oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_l0o`05ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00oooool6o`00o`Gooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`Koo`001Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Cooooo1_oo0?l5ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?oo
ool6ool0o`Gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`Koool02ooo
ool000gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Oooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02_ooool01?l0003ooooo
ooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0
003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool0
1?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00Wooooo00Go0000oooooooooooooooo
o`00000;ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003o
ooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0
003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?oo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool0
0ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool0
0ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo
000000cooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2
o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2
o`0000gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02oooool2o`00
00gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`00
00gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo
00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo
00_ooooo0_l0000=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00[o
oooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooooooo
o`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oo
oooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooo000000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03
o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:oooo
o`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0
000<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00Go0000
ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00cooooo000:ooooo`Koo`001oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
00Cooooo1_l00007ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6o`3o
00Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Koool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_oo0?l7ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool01Oooool6o`0000Kooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`05ooooo`Ko003o1_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo
00Gooooo1_l00006ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6o`00
o`Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Koo`3o2oooool000So
oooo2_oo0004ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_l00004oooo
o`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_l0o`04ooooo`03o`000?oooooo
oooo00Sooooo00Co0000oooooooooooooooo2_ooo`04ooooo`03o`000?oooooooooo00Sooooo00Co
0000oooooooooooooooo2_oo0?l4ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
0_ooool:o`0000?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[o003o
0oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_l00003ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:o`00o`?ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`02ooooo`[oo`3o2Oooool000Kooooo3_oo00000ooooooo0000oooo
o`0;ooooo`oo000000?oooooo`000?ooool02oooool00ol0003o0?l0o`3o000<o`3o0003ooooool0
003ooooo00_ooooo00?o0000oooo0?ooo`003?ooo`000ooooooo0000ooooo`0;ooooo`03o`000?oo
0?ooo`3o00coo`3o00?oooooo`000?ooool02_ooool00ol0003oooooo`00000>o`0000cooooo00?o
0000ooooool00?l03Ol00?l00ol0003oooooooooo`0:ooooo`03o`000?oooooo000000ko00003?oo
ool00ol0003oooooo`00o`0=o`00o`03o`000?oooooooooo00[ooooo00?o0000oooooooo0?l03Ooo
0?l7ooooo`001Oooool@ool000gooooo4?l0000=oooooa3o0?l03Oooool@oooo00gooooo4?oo0?l=
oooooa3o00003Oooool@o`00o`gooooo4?l0000=oooooa3o003o3Oooool@ool0o`Kooooo0005oooo
oa3oo`003Oooool@o`0000gooooo4?l0o`0=oooooa3oool03Oooool@ool0o`gooooo4?l0000=oooo
oa3o003o3Oooool@o`0000gooooo4?l00?l=oooooa3oo`3o1_ooool000Cooooo4_oo000;oooooa;o
00002ooooolBo`3o00_ooooo4_ooo`0;oooooa;oo`3o2ooooolBo`0000_ooooo4_l00?l;oooooa;o
00002ooooolBo`00o`_ooooo4_oo0?l5ooooo`001?oooolBool000_ooooo4_l0000;oooooa;o0?l0
2ooooolBoooo00_ooooo4_oo0?l;oooooa;o00002ooooolBo`00o`_ooooo4_l0000;oooooa;o003o
2ooooolBool0o`Gooooo0003oooooaCoo`002OoooolDo`0000Wooooo5?l0o`09oooooaCoool02Ooo
oolDool0o`Wooooo5?l00009oooooaCo003o2OoooolDo`0000Wooooo5?l00?l9oooooaCoo`3o1?oo
ool000?ooooo5?oo0009oooooaCo00002OoooolDo`3o00Wooooo5?ooo`09oooooaCoo`3o2OoooolD
o`0000Wooooo5?l00?l9oooooaCo00002OoooolDo`00o`Wooooo5?oo0?l4ooooo`000ooooolDool0
00Wooooo5?l00009oooooaCo0?l02OoooolDoooo00Wooooo5?oo0?l9oooooaCo00002OoooolDo`00
o`Wooooo5?l00009oooooaCo003o2OoooolDool0o`Cooooo0003oooooaCoo`009_l0000Do`3o00Wo
00005?ooo`09o`0001Coo`3o9_l0000Do`00obKo00005?l00?l9o`0001Coo`3o1?ooool000?ooooo
5?oo0009oooooaCo00002OoooolDo`3o00Wooooo5?ooo`09oooooaCoo`3o2OoooolDo`0000Wooooo
5?l00?l9oooooaCo00002OoooolDo`00o`Wooooo5?oo0?l4ooooo`000ooooolDool000Wooooo5?l0
0009oooooaCo0?l02OoooolDoooo00Wooooo5?oo0?l9oooooaCo00002OoooolDo`00o`Wooooo5?l0
0009oooooaCo003o2OoooolDool0o`Cooooo0004oooooa;oo`002ooooolBo`0000_ooooo4_l0o`0;
oooooa;oool02ooooolBool0o`_ooooo4_l0000;oooooa;o003o2ooooolBo`0000_ooooo4_l00?l;
oooooa;oo`3o1Oooool000Cooooo4_oo000;oooooa;o00002ooooolBo`3o00_ooooo4_ooo`0;oooo
oa;oo`3o2ooooolBo`0000_ooooo4_l00?l;oooooa;o00002ooooolBo`00o`_ooooo4_oo0?l5oooo
o`001Oooool@ool000gooooo4?l0000=oooooa3o0?l03Oooool@oooo00gooooo4?oo0?l=oooooa3o
00003Oooool@o`00o`gooooo4?l0000=oooooa3o003o3Oooool@ool0o`Kooooo0005oooooa3oo`00
3Oooool@o`0000gooooo4?l0o`0=oooooa3oool03Oooool@ool0o`gooooo4?l0000=oooooa3o003o
3Oooool@o`0000gooooo4?l00?l=oooooa3oo`3o1_ooool000Kooooo3_oo00000ol0003ooooooooo
o`0<ooooo`oo00003_ooool>o`3o0003o`000?oooooooooo00cooooo3_ooo`000ol0003ooooooooo
o`0<ooooo`koo`3o3oooool>o`0000kooooo00?o0000o`00ool00?l03?l00?l>ooooo`oo00003_oo
ool00ol0003o003oo`00o`0<o`00o`kooooo00?o0000ool0oooo0?l03?oo0?l7ooooo`002?ooool:
ool000?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:o`0000?ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0:o`3o00?ooooo00?o0000ooooooooool02_ooool0
0ol0003oooooooooo`0:oooo00?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:
ool0o`;ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0:o`0000;ooooo00?o0000
ooooooooool02_ooool01?l0003oooooooooooooool:o`00o`;ooooo00?o0000ooooooooool02_oo
ool01?l0003oooooooooooooool:o`0000;ooooo00?o0000ooooooooool02_ooool01?l0003ooooo
ooooooooool:o`00o`;ooooo00?o0000ooooooooool02_ooool01?l0003oooooooooooooool:ool0
o`Wooooo000:ooooo`Koo`001_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?o
oooo1_l00006ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool6o`3o00Ko
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`Koool01_ooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_oo0?l5ooooo`03o`000?oooooooooo00Wo
oooo00?o0000ooooooooool00oooool6o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`04ooooo`Ko003o1Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Co
oooo1_l00005ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool6o`00o`Go
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`Koo`3o2oooool000gooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<
ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003ooooooooo
o`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003ooooo
ooooool0000<ooooo`03o`000?oooooooooo00Wooooo00Go0000ooooooooooooooooo`00000;oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:
ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003ooooooooo
ool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003ooooo
ooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003ooooo
ooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo000000cooooo
00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo
00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0
000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0
000=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00[ooooo00Co0000
oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo
00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[o
oooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooo0000
00cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooo
oooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?oo
ooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool0
0_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00Go0000oooooooooooo
ooooo`00000;ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003o
ooooooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00cooooo
000:ooooo`Ko0?l01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_o<
c<`7ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6ool0o`Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Koo`001oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Cooooo1_l00007ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01Oooool6oooo00Kooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`05ooooo`Ko003o1_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_oo
0?l6ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6olc<c0Kooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Koool02oooool000Sooooo2_l0o`04
ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_o<c<`4ooooo`03o`000?oo
oooooooo00Sooooo00Co0000oooooooooooooooo2_oo0?l4ooooo`03o`000?oooooooooo00Sooooo
00Co0000oooooooooooooooo2_oo0004ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooo
oooooooo2_l00004ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool:oooo
00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[o003o0oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_oo0?l3ooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool00_ooool:olc<c0?ooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`02ooooo`[oool02Oooool000Kooooo3_l0o`000ooooooo0000ooooo`0;ooooo`03
o`000?o<c<coc<c<00coc<c<00?oooooo`000?ooool02oooool00ol0003oo`3oool0o`0<ool0o`03
ooooool0003ooooo00_ooooo00?o0000ool00?oo00003?oo00000ooooooo0000ooooo`0;ooooo`oo
000000?oooooo`000?ooool02_ooool00ol0003ooooooooo000=oooo0003o`000?oooooooooo00[o
oooo00?o0000ooooool00?l03Ol00?l00ol0003oooooooooo`0:ooooo`03o`000?ooooooo`3o00go
o`3o00?o0000ooooooooool02_ooool00ol0003oooooolc<c00=olc<c003o`000?oooooooooo00[o
oooo00?o0000ooooooooo`003Oooo`07ooooo`001Oooool@o`3o00gooooo4?o<c<`=oooooa3oo`3o
3Oooool@ool000gooooo4?l0000=oooooa3oool03Oooool@o`00o`gooooo4?oo0?l=oooooa3oc<c<
3Oooool@oooo00Kooooo0005oooooa3o0?l03Oooool@olc<c0gooooo4?oo0?l=oooooa3oo`003Ooo
ool@o`0000gooooo4?ooo`0=oooooa3o003o3Oooool@ool0o`gooooo4?o<c<`=oooooa3oool01_oo
ool000Cooooo4_l0o`0;oooooa;oc<c<2ooooolBool0o`_ooooo4_oo000;oooooa;o00002ooooolB
oooo00_ooooo4_l00?l;oooooa;oo`3o2ooooolBolc<c0_ooooo4_ooo`05ooooo`001?oooolBo`3o
00_ooooo4_o<c<`;oooooa;oo`3o2ooooolBool000_ooooo4_l0000;oooooa;oool02ooooolBo`00
o`_ooooo4_oo0?l;oooooa;oc<c<2ooooolBoooo00Gooooo0003oooooaCo0?l02OoooolDolc<c0Wo
oooo5?oo0?l9oooooaCoo`002OoooolDo`0000Wooooo5?ooo`09oooooaCo003o2OoooolDool0o`Wo
oooo5?o<c<`9oooooaCoool01?ooool000?ooooo5?l0o`09oooooaCoc<c<2OoooolDool0o`Wooooo
5?oo0009oooooaCo00002OoooolDoooo00Wooooo5?l00?l9oooooaCoo`3o2OoooolDolc<c0Wooooo
5?ooo`04ooooo`000ooooolDo`3o00Wooooo5?o<c<`9oooooaCoo`3o2OoooolDool000Wooooo5?l0
0009oooooaCoool02OoooolDo`00o`Wooooo5?oo0?l9oooooaCoc<c<2OoooolDoooo00Cooooo0003
oooooaCo0?l02Ol0000Dolc<c0Wo00005?oo0?l9o`0001Coo`009_l0000Doooo00Wo00005?l00?l9
o`0001Coo`3o2Ol0000Dolc<c0Wo00005?ooo`04ooooo`000ooooolDo`3o00Wooooo5?o<c<`9oooo
oaCoo`3o2OoooolDool000Wooooo5?l00009oooooaCoool02OoooolDo`00o`Wooooo5?oo0?l9oooo
oaCoc<c<2OoooolDoooo00Cooooo0003oooooaCo0?l02OoooolDolc<c0Wooooo5?oo0?l9oooooaCo
o`002OoooolDo`0000Wooooo5?ooo`09oooooaCo003o2OoooolDool0o`Wooooo5?o<c<`9oooooaCo
ool01?ooool000Cooooo4_l0o`0;oooooa;oc<c<2ooooolBool0o`_ooooo4_oo000;oooooa;o0000
2ooooolBoooo00_ooooo4_l00?l;oooooa;oo`3o2ooooolBolc<c0_ooooo4_ooo`05ooooo`001?oo
oolBo`3o00_ooooo4_o<c<`;oooooa;oo`3o2ooooolBool000_ooooo4_l0000;oooooa;oool02ooo
oolBo`00o`_ooooo4_oo0?l;oooooa;oc<c<2ooooolBoooo00Gooooo0005oooooa3o0?l03Oooool@
olc<c0gooooo4?oo0?l=oooooa3oo`003Oooool@o`0000gooooo4?ooo`0=oooooa3o003o3Oooool@
ool0o`gooooo4?o<c<`=oooooa3oool01_ooool000Gooooo4?l0o`0=oooooa3oc<c<3Oooool@ool0
o`gooooo4?oo000=oooooa3o00003Oooool@oooo00gooooo4?l00?l=oooooa3oo`3o3Oooool@olc<
c0gooooo4?ooo`06ooooo`001_ooool>o`3o0003o`000?oooooooooo00cooooo3_o<c<`00ol0003o
ooooooooo`0<ooooo`koo`3o00?o0000ooooooooool03?ooool>ool00003o`000?oooooooooo00co
oooo3_l0000?ooooo`koool03_ooool00ol0003o003oo`00o`0<o`00o`kooooo00?o0000ool0oooo
0?l03?oo0?l>ooooo`03o`000?o<c<coc<c<00coc<c<3_ooool00ol0003oool0oooo000<oooo00Oo
oooo0008ooooo`[o0?l00oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[oc<c<
0oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[oo`3o0oooool00ol0003ooooo
ooooo`0:ooooo`03o`000?oooooooooo00[oo`000oooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00[o00000_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00[oool0
0_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooooooo`[o003o0_ooool00ol0003o
ooooooooo`0:ooooo`04o`000?ooooooooooooooo`[oo`3o0_ooool00ol0003oooooooooo`0:oooo
o`04o`000?ooooooooooooooo`[oc<c<0_ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooo
ooooooooo`[oool02Oooool000[ooooo1_l0o`06ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool00oooool6olc<c0Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03
ooooo`Koo`3o1_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_oo0006
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool6o`0000Gooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`03ooooo`Koool01Oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Cooooo1_l00?l5ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool6ool0o`Gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04
ooooo`Koc<c<1Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo1_ooo`0;
ooooo`003Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0
2Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0:ooooo`04o`000?oo
ooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03
o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:oooo
o`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooo
oooo000000_ooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`00
0?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04
o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=oooo
o`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=oooo
o`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ool000003?ooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;oooo
o`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;oooo
o`;o00003Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000oooooooo
ool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000oooooooo
ool02oooool2o`0000gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
2_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooo
oooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000
ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooool000003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo
00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[o
oooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooooooo
o`0000cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool01Ol0
003ooooooooooooooooo000000_ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooo
ooooool03?ooool000[ooooo1_l0ool7ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01?ooool6o`3oo`Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Ko
c<c<1oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_l0ool7ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6o`3oo`Oooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`05ooooo`Ko0?l01_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Gooooo1_l00006ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01Oooool6ool000Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Ko
003o1_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_l0o`0;ooooo`00
2?ooool:o`3oo`Cooooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:o`3oo`Co
oooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:olc<c0Cooooo00?o0000oooo
ooooool02?ooool01?l0003oooooooooooooool:o`3oo`Cooooo00?o0000ooooooooool02?ooool0
1?l0003oooooooooooooool:o`3oo`Cooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`02ooooo`[o0?l00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_l0
0003ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:ool000?ooooo00?o
0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[o003o0oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00;ooooo2_l0o`09ooooo`001_ooool>o`3oo`03ooooool0003o
oooo00_ooooo00?o0000o`3oool0ool03?l0ool00ooooooo0000ooooo`0;ooooo`03o`000?o<c<co
c<c<00coc<c<00?oooooo`000?ooool02oooool00ol0003o0?ooo`3oo`0<o`3oo`03ooooool0003o
oooo00_ooooo00?o0000o`3oool0ool03?l0ool00ooooooo0000ooooo`0:ooooo`03o`000?oooooo
0?l000go0?l000?o0000ooooooooool02_ooool00ol0003oooooo`00000>o`0000cooooo00?o0000
oooooooo00003Ooo00000ol0003oooooooooo`0:ooooo`03o`000?oooooo003o00go003o00?o0000
ooooooooool02_ooool00ol0003oooooo`3o000=o`3o00Oooooo0005oooooa3o0?oo3Oooool@o`3o
o`gooooo4?o<c<`=oooooa3o0?oo3Oooool@o`3oo`gooooo4?l0o`0=oooooa3o00003Oooool@ool0
00gooooo4?l00?l=oooooa3o0?l01_ooool000Gooooo4?l0ool=oooooa3o0?oo3Oooool@olc<c0go
oooo4?l0ool=oooooa3o0?oo3Oooool@o`3o00gooooo4?l0000=oooooa3oo`003Oooool@o`00o`go
oooo4?l0o`06ooooo`001?oooolBo`3oo`_ooooo4_l0ool;oooooa;oc<c<2ooooolBo`3oo`_ooooo
4_l0ool;oooooa;o0?l02ooooolBo`0000_ooooo4_oo000;oooooa;o003o2ooooolBo`3o00Gooooo
0004oooooa;o0?oo2ooooolBo`3oo`_ooooo4_o<c<`;oooooa;o0?oo2ooooolBo`3oo`_ooooo4_l0
o`0;oooooa;o00002ooooolBool000_ooooo4_l00?l;oooooa;o0?l01Oooool000?ooooo5?l0ool9
oooooaCo0?oo2OoooolDolc<c0Wooooo5?l0ool9oooooaCo0?oo2OoooolDo`3o00Wooooo5?l00009
oooooaCoo`002OoooolDo`00o`Wooooo5?l0o`04ooooo`000ooooolDo`3oo`Wooooo5?l0ool9oooo
oaCoc<c<2OoooolDo`3oo`Wooooo5?l0ool9oooooaCo0?l02OoooolDo`0000Wooooo5?oo0009oooo
oaCo003o2OoooolDo`3o00Cooooo0003oooooaCo0?oo2OoooolDo`3oo`Wooooo5?o<c<`9oooooaCo
0?oo2OoooolDo`3oo`Wooooo5?l0o`09oooooaCo00002OoooolDool000Wooooo5?l00?l9oooooaCo
0?l01?ooool000?ooooo5?l0ool9o`0001Co0?oo2Ol0000Dolc<c0Wo00005?l0ool9o`0001Co0?oo
2Ol0000Do`3o02Ko00005?oo0009o`0001Co003o2Ol0000Do`3o00Cooooo0003oooooaCo0?oo2Ooo
oolDo`3oo`Wooooo5?o<c<`9oooooaCo0?oo2OoooolDo`3oo`Wooooo5?l0o`09oooooaCo00002Ooo
oolDool000Wooooo5?l00?l9oooooaCo0?l01?ooool000?ooooo5?l0ool9oooooaCo0?oo2OoooolD
olc<c0Wooooo5?l0ool9oooooaCo0?oo2OoooolDo`3o00Wooooo5?l00009oooooaCoo`002OoooolD
o`00o`Wooooo5?l0o`04ooooo`001?oooolBo`3oo`_ooooo4_l0ool;oooooa;oc<c<2ooooolBo`3o
o`_ooooo4_l0ool;oooooa;o0?l02ooooolBo`0000_ooooo4_oo000;oooooa;o003o2ooooolBo`3o
00Gooooo0004oooooa;o0?oo2ooooolBo`3oo`_ooooo4_o<c<`;oooooa;o0?oo2ooooolBo`3oo`_o
oooo4_l0o`0;oooooa;o00002ooooolBool000_ooooo4_l00?l;oooooa;o0?l01Oooool000Gooooo
4?l0ool=oooooa3o0?oo3Oooool@olc<c0gooooo4?l0ool=oooooa3o0?oo3Oooool@o`3o00gooooo
4?l0000=oooooa3oo`003Oooool@o`00o`gooooo4?l0o`06ooooo`001Oooool@o`3oo`gooooo4?l0
ool=oooooa3oc<c<3Oooool@o`3oo`gooooo4?l0ool=oooooa3o0?l03Oooool@o`0000gooooo4?oo
000=oooooa3o003o3Oooool@o`3o00Kooooo0006ooooo`ko0?oo00?o0000ooooooooool03?ooool>
o`3oo`03o`000?oooooooooo00cooooo3_o<c<`00ol0003oooooooooo`0<ooooo`ko0?oo00?o0000
ooooooooool03?ooool>o`3oo`oooooo3_l0o`0>ooooo`oo00003_ooool00ol0003oo`00ool0000<
ool000kooooo00?o0000o`00ool00?l03?l00?l>ooooo`03o`000?l0o`3o0?l000co0?l01oooool0
00Sooooo2_l0ool3ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_l0ool3oooo
o`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_o<c<`3ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool02_l0ool3ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool02_l0ool2ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02_l0o`02oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo2_l00002ooooo`03o`000?oooooo
oooo00[ooooo00Co0000oooooooooooooooo2_oo0002ooooo`03o`000?oooooooooo00[ooooo00Co
0000oooooooooooooooo2_l00?l2ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo
oooo2_l0o`09ooooo`002_ooool6o`3oo`Kooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`03ooooo`Ko0?oo1_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo
1_o<c<`6ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool6o`3oo`Kooooo
00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`Ko0?oo1Oooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00?ooooo1_l0o`05ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool01?ooool6o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`04ooooo`Koo`001Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo
1_l00?l5ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool6o`3o00_ooooo
000=ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`07ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooo
oooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000
ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co
0000oooooooooooo00003?ooool00ol0003oooooooooo`09ooooo`05o`000?ooooooooooooooool0
00002oooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000oooo
ooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000
oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00
000<ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0
000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0
000=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=
ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=
ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;
ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;
ooooo`;o00003Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0:oooo
o`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0
000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003ooooo
ooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool00ol0003o
ooooo`00000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0
003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool0
1?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo0000
3?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`05o`000?oo
ooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0<ooooo`002_ooool6o`00o`Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04
ooooo`Koo`3o1oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_l00?l7
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6o`0000Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Ko003o1oooool00ol0003oooooooooo`05
ooooo`03o`000?oooooooooo00Gooooo1_l0o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000
ooooooooool01Oooool6ool0o`Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05
ooooo`Koo`001_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_l0o`06
ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6ool000_ooooo0008oooo
o`[o003o1?ooool00ol0003oooooooooo`08ooooo`04o`000?ooooooooooooooo`[oo`3o1?ooool0
0ol0003oooooooooo`08ooooo`04o`000?ooooooooooooooo`[o003o1?ooool00ol0003ooooooooo
o`08ooooo`04o`000?ooooooooooooooo`[o00001?ooool00ol0003oooooooooo`08ooooo`04o`00
0?ooooooooooooooo`[o003o1?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo2_l0o`03ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:ool0o`?o
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[oo`000oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_l0o`03ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool00_ooool:ool000Wooooo0006ooooo`ko003o00?oooooo`000?ooool0
2oooool00ol0003oo`3oool0o`0<ool0o`03ooooool0003ooooo00_ooooo00?o0000o`00ool00?l0
3?l00?l00ooooooo0000ooooo`0;ooooo`oo000000?oooooo`000?ooool02oooool00ol0003o003o
o`00o`0<o`00o`03ooooool0003ooooo00[ooooo00?o0000ooooool0o`003Ol0o`000ol0003ooooo
ooooo`0:ooooo`03o`000?ooooooo`3o00goo`3o00?o0000ooooooooool02_ooool00ol0003ooooo
ool0000=ool00003o`000?oooooooooo00[ooooo00?o0000ooooool0o`003Ol0o`000ol0003ooooo
ooooo`0:ooooo`03o`000?ooooooo`0000goo`001oooool000Gooooo4?l00?l=oooooa3oo`3o3Ooo
ool@o`00o`gooooo4?l0000=oooooa3o003o3Oooool@o`3o00gooooo4?oo0?l=oooooa3oo`003Ooo
ool@o`3o00gooooo4?oo0006ooooo`001Oooool@o`00o`gooooo4?oo0?l=oooooa3o003o3Oooool@
o`0000gooooo4?l00?l=oooooa3o0?l03Oooool@ool0o`gooooo4?oo000=oooooa3o0?l03Oooool@
ool000Kooooo0004oooooa;o003o2ooooolBool0o`_ooooo4_l00?l;oooooa;o00002ooooolBo`00
o`_ooooo4_l0o`0;oooooa;oo`3o2ooooolBool000_ooooo4_l0o`0;oooooa;oo`001Oooool000Co
oooo4_l00?l;oooooa;oo`3o2ooooolBo`00o`_ooooo4_l0000;oooooa;o003o2ooooolBo`3o00_o
oooo4_oo0?l;oooooa;oo`002ooooolBo`3o00_ooooo4_oo0005ooooo`000ooooolDo`00o`Wooooo
5?oo0?l9oooooaCo003o2OoooolDo`0000Wooooo5?l00?l9oooooaCo0?l02OoooolDool0o`Wooooo
5?oo0009oooooaCo0?l02OoooolDool000Cooooo0003oooooaCo003o2OoooolDool0o`Wooooo5?l0
0?l9oooooaCo00002OoooolDo`00o`Wooooo5?l0o`09oooooaCoo`3o2OoooolDool000Wooooo5?l0
o`09oooooaCoo`001?ooool000?ooooo5?l00?l9oooooaCoo`3o2OoooolDo`00o`Wooooo5?l00009
oooooaCo003o2OoooolDo`3o00Wooooo5?oo0?l9oooooaCoo`002OoooolDo`3o00Wooooo5?oo0004
ooooo`000ooooolDo`00o`Wo00005?oo0?l9o`0001Co003o9_l0000Do`00o`Wo00005?l0o`09o`00
01Coo`3o2Ol0000Dool000Wo00005?l0o`09o`0001Coo`001?ooool000?ooooo5?l00?l9oooooaCo
o`3o2OoooolDo`00o`Wooooo5?l00009oooooaCo003o2OoooolDo`3o00Wooooo5?oo0?l9oooooaCo
o`002OoooolDo`3o00Wooooo5?oo0004ooooo`000ooooolDo`00o`Wooooo5?oo0?l9oooooaCo003o
2OoooolDo`0000Wooooo5?l00?l9oooooaCo0?l02OoooolDool0o`Wooooo5?oo0009oooooaCo0?l0
2OoooolDool000Cooooo0004oooooa;o003o2ooooolBool0o`_ooooo4_l00?l;oooooa;o00002ooo
oolBo`00o`_ooooo4_l0o`0;oooooa;oo`3o2ooooolBool000_ooooo4_l0o`0;oooooa;oo`001Ooo
ool000Cooooo4_l00?l;oooooa;oo`3o2ooooolBo`00o`_ooooo4_l0000;oooooa;o003o2ooooolB
o`3o00_ooooo4_oo0?l;oooooa;oo`002ooooolBo`3o00_ooooo4_oo0005ooooo`001Oooool@o`00
o`gooooo4?oo0?l=oooooa3o003o3Oooool@o`0000gooooo4?l00?l=oooooa3o0?l03Oooool@ool0
o`gooooo4?oo000=oooooa3o0?l03Oooool@ool000Kooooo0005oooooa3o003o3Oooool@ool0o`go
oooo4?l00?l=oooooa3o00003Oooool@o`00o`gooooo4?l0o`0=oooooa3oo`3o3Oooool@ool000go
oooo4?l0o`0=oooooa3oo`001_ooool000Kooooo3_l00?l00ooooooo0000ooooo`0;ooooo`03o`00
0?oo0?ooo`3o00coo`3o00?oooooo`000?ooool02oooool00ol0003o003oo`00o`0<o`00o`03oooo
ool0003ooooo00_ooooo3ol000000ooooooo0000ooooo`0;ooooo`03o`000?l00?oo003o00co003o
00?o0000ooooooooool02oooool00ol0003o0?l0o`3o000<o`3o0003o`000?oooooooooo00[ooooo
00?o0000oooooooo0?l03Ooo0?l00ol0003oooooooooo`0:ooooo`03o`000?ooooooo`0000goo`00
00?o0000ooooooooool02_ooool00ol0003oooooo`3o000=o`3o0003o`000?oooooooooo00[ooooo
00?o0000oooooooo00003Ooo0007ooooo`002?ooool:o`00o`Cooooo00?o0000ooooooooool02?oo
ool01?l0003oooooooooooooool:ool0o`Cooooo00?o0000ooooooooool02?ooool01?l0003ooooo
ooooooooool:o`00o`Cooooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:o`00
00Cooooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:o`00o`?ooooo00?o0000
ooooooooool02Oooool01?l0003oooooooooooooool:o`3o00?ooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`02ooooo`[oo`3o0oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00;ooooo2_oo0003ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_oo
ool:o`3o00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[oo`002Ooo
ool000[ooooo1_l00?l7ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6
ool0o`Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Ko003o1oooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_l00007ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool01?ooool6o`00o`Kooooo00?o0000ooooooooool01oooool0
0ol0003oooooooooo`04ooooo`Ko0?l01_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Gooooo1_oo0?l6ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6
ool000Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Ko0?l01_ooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_oo000;ooooo`003Oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003o
ooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool0
0ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02Ooo
ool01Ol0003ooooooooooooooooo000000_ooooo00?o0000ooooooooool02_ooool01?l0003ooooo
ooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0
003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool0
1?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooo
oooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooo
oooo00[ooooo00?o0000ooooool000003?ooool00ol0003oooooooooo`0;ooooo`;o00003Oooool0
0ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool0
0ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0
003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool2o`00
00gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`00
00gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo
00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?oooooo
ooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooool000003?ooool00ol0003oooooooooo`0:ooooo`04o`00
0?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:
ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000
ooooooooool02Oooool01Ol0003ooooooooooooooooo000000_ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03?ooool000[o
oooo1_l0o`06ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool6ool000Ko
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`Koo`3o1_ooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_l0ool6ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool00oooool6olc<c0Kooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`04ooooo`Ko00001Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Co
oooo1_l00005ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool6ool000Go
oooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`Ko0?l01Oooool00ol0003o
ooooooooo`08ooooo`03o`000?oooooooooo00Cooooo1_oo0?l;ooooo`002?ooool:o`3o00?ooooo
00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:ool000?ooooo00?o0000ooooooooool0
2_ooool00ol0003oooooooooo`0:ool0o`?ooooo00?o0000ooooooooool02_ooool00ol0003ooooo
ooooo`0:o`3oo`?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:olc<c0?ooooo
00?o0000ooooooooool02Oooool01?l0003oooooooooooooool:o`0000;ooooo00?o0000oooooooo
ool02_ooool01?l0003oooooooooooooool:o`0000;ooooo00?o0000ooooooooool02_ooool01?l0
003oooooooooooooool:ool000;ooooo00?o0000ooooooooool02_ooool01?l0003ooooooooooooo
ool:o`3o00;ooooo00?o0000ooooooooool02_ooool01?l0003oooooooooooooool:ool0o`Wooooo
0006ooooo`ko0?l000?o0000ooooooooool03?ooool>ool00003o`000?oooooooooo00cooooo3_oo
0?l00ol0003oooooooooo`0<ooooo`ko0?oo00?o0000ooooooooool03?ooool>olc<c003o`000?oo
oooooooo00_ooooo3ol0000>ooooo`oo00003_ooool00ol0003oo`00ool0000<ool000kooooo00?o
0000o`3o0?l0o`003?l0o`0>ooooo`03o`000?oo0?ooo`3o00coo`3o1oooool000Gooooo4?l0o`0=
oooooa3oo`003Oooool@ool0o`gooooo4?l0ool=oooooa3oc<c<3Oooool@o`0000gooooo4?l0000=
oooooa3oo`003Oooool@o`3o00gooooo4?oo0?l6ooooo`001Oooool@o`3o00gooooo4?oo000=oooo
oa3oo`3o3Oooool@o`3oo`gooooo4?o<c<`=oooooa3o00003Oooool@o`0000gooooo4?oo000=oooo
oa3o0?l03Oooool@ool0o`Kooooo0004oooooa;o0?l02ooooolBool000_ooooo4_oo0?l;oooooa;o
0?oo2ooooolBolc<c0_ooooo4_l0000;oooooa;o00002ooooolBool000_ooooo4_l0o`0;oooooa;o
o`3o1Oooool000Cooooo4_l0o`0;oooooa;oo`002ooooolBool0o`_ooooo4_l0ool;oooooa;oc<c<
2ooooolBo`0000_ooooo4_l0000;oooooa;oo`002ooooolBo`3o00_ooooo4_oo0?l5ooooo`000ooo
oolDo`3o00Wooooo5?oo0009oooooaCoo`3o2OoooolDo`3oo`Wooooo5?o<c<`9oooooaCo00002Ooo
oolDo`0000Wooooo5?oo0009oooooaCo0?l02OoooolDool0o`Cooooo0003oooooaCo0?l02OoooolD
ool000Wooooo5?oo0?l9oooooaCo0?oo2OoooolDolc<c0Wooooo5?l00009oooooaCo00002OoooolD
ool000Wooooo5?l0o`09oooooaCoo`3o1?ooool000?ooooo5?l0o`09o`0001Coo`002Ol0000Dool0
o`Wo00005?l0ool9o`0001Coc<c<@ol0000Dool000Wo00005?l0o`09o`0001Coo`3o1?ooool000?o
oooo5?l0o`09oooooaCoo`002OoooolDool0o`Wooooo5?l0ool9oooooaCoc<c<2OoooolDo`0000Wo
oooo5?l00009oooooaCoo`002OoooolDo`3o00Wooooo5?oo0?l4ooooo`000ooooolDo`3o00Wooooo
5?oo0009oooooaCoo`3o2OoooolDo`3oo`Wooooo5?o<c<`9oooooaCo00002OoooolDo`0000Wooooo
5?oo0009oooooaCo0?l02OoooolDool0o`Cooooo0003oooooaCo0?l02OoooolDool000Wooooo5?oo
0?l9oooooaCo0?oo2OoooolDolc<c0Wooooo5?l00009oooooaCo00002OoooolDool000Wooooo5?l0
o`09oooooaCoo`3o1?ooool000Cooooo4_l0o`0;oooooa;oo`002ooooolBool0o`_ooooo4_l0ool;
oooooa;oc<c<2ooooolBo`0000_ooooo4_l0000;oooooa;oo`002ooooolBo`3o00_ooooo4_oo0?l5
ooooo`001?oooolBo`3o00_ooooo4_oo000;oooooa;oo`3o2ooooolBo`3oo`_ooooo4_o<c<`;oooo
oa;o00002ooooolBo`0000_ooooo4_oo000;oooooa;o0?l02ooooolBool0o`Gooooo0005oooooa3o
0?l03Oooool@ool000gooooo4?oo0?l=oooooa3o0?oo3Oooool@olc<c0gooooo4?l0000=oooooa3o
00003Oooool@ool000gooooo4?l0o`0=oooooa3oo`3o1_ooool000Gooooo4?l0o`0=oooooa3oo`00
3Oooool@ool0o`gooooo4?l0ool=oooooa3oc<c<3Oooool@o`0000gooooo4?l0000=oooooa3oo`00
3Oooool@o`3o00gooooo4?oo0?l6ooooo`001_ooool>o`3o0003ooooool0003ooooo00_ooooo00?o
0000ool00?oo00003?oo00000ooooooo0000ooooo`0;ooooo`03o`000?oo0?ooo`3o00coo`3o00?o
ooooo`000?ooool02oooool00ol0003o0?ooo`3oo`0<o`3oo`03ooooool0003ooooo00_ooooo00?o
0000olc<c?o<c<`03?o<c<`00ol0003oooooooooo`0;oooooa3o00003?ooool00ol0003oooooo`00
000>o`0000cooooo00?o0000oooooooo00003Ooo00000ol0003oooooooooo`0:ooooo`03o`000?oo
oooo0?l000go0?l000?o0000ooooooooool02_ooool00ol0003oooooool0o`0=ool0o`Oooooo0008
ooooo`[o0?l01?ooool00ol0003oooooooooo`08ooooo`04o`000?ooooooooooooooo`[oo`001?oo
ool00ol0003oooooooooo`08ooooo`04o`000?ooooooooooooooo`[oo`3o1?ooool00ol0003ooooo
ooooo`08ooooo`04o`000?ooooooooooooooo`[o0?oo1?ooool00ol0003oooooooooo`08ooooo`04
o`000?ooooooooooooooo`[oc<c<0oooool00ol0003oooooooooo`09ooooo`04o`000?oooooooooo
ooooo`[o00000oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_l00003
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:ool000?ooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[o0?l00oooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00;ooooo2_oo0?l9ooooo`002_ooool6o`3o00Oooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`04ooooo`Koo`001oooool00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo00Cooooo1_oo0?l7ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01?ooool6o`3oo`Oooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`04oooo
o`Koc<c<1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Cooooo1_l00006oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6o`0000Kooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`05ooooo`Koo`001_ooool00ol0003oooooooooo`06oooo
o`03o`000?oooooooooo00Gooooo1_l0o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool01Oooool6ool0o`_ooooo000=ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Oooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo0000
3?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000oooooooo
ool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooo
oooooooo00003?ooool00ol0003oooooooooo`09ooooo`05o`000?ooooooooooooooool000002ooo
ool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool0
2_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooo
oooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000oooo
ooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000oooo
ooooool02oooool2o`0000gooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00000<oooo
o`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=oooo
o`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03
o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03
o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0:ooooo`04o`00
0?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:
ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool00ol0003oooooo`00
000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003ooooo
ooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003o
ooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`09ooooo`05o`000?oooooooooo
ooooool000002oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooooooool02Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`0<ooooo`002_ooool6o`00o`Kooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`03ooooo`Koc<c<1_ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00?ooooo1_l00?l6ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
0oooool6o`3oo`Kooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`Koo`00
1_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Cooooo1_l0o`05ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool6ool000Gooooo00?o0000ooooooooool0
2?ooool00ol0003oooooooooo`04ooooo`Koo`3o1Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo1_l0o`05ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0
1?ooool6o`3o00_ooooo0008ooooo`[o003o0oooool00ol0003oooooooooo`0:ooooo`03o`000?oo
oooooooo00[oc<c<0oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[o003o0ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[o0?oo0oooool00ol0003ooooooooo
o`0:ooooo`03o`000?oooooooooo00[oo`000oooool00ol0003oooooooooo`09ooooo`04o`000?oo
ooooooooooooo`[o0?l00_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooooooo`[o
o`000_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooooooo`[oo`3o0_ooool00ol0
003oooooooooo`0:ooooo`04o`000?ooooooooooooooo`[o0?l00_ooool00ol0003oooooooooo`0:
ooooo`04o`000?ooooooooooooooo`[o0?l02Oooool000Kooooo3_l00?l00ol0003oooooooooo`0<
ooooo`koc<c<00?o0000ooooooooool03?ooool>o`00o`03o`000?oooooooooo00cooooo3_l0ool0
0ol0003oooooooooo`0<ooooo`koo`0000?o0000ooooooooool02oooool00ol0003o0?l0o`3o000<
o`3o00kooooo00?o0000ool00?oo00003?oo000>ooooo`03o`000?oo0?ooo`3o00coo`3o3_ooool0
0ol0003o0?l0o`3o000<o`3o00kooooo00?o0000o`3o0?l0o`003?l0o`07ooooo`001Oooool@o`00
o`gooooo4?o<c<`=oooooa3o003o3Oooool@o`3oo`gooooo4?oo000=oooooa3o0?l03Oooool@ool0
00gooooo4?oo0?l=oooooa3o0?l03Oooool@o`3o00Kooooo0005oooooa3o003o3Oooool@olc<c0go
oooo4?l00?l=oooooa3o0?oo3Oooool@ool000gooooo4?l0o`0=oooooa3oo`003Oooool@ool0o`go
oooo4?l0o`0=oooooa3o0?l01_ooool000Cooooo4_l00?l;oooooa;oc<c<2ooooolBo`00o`_ooooo
4_l0ool;oooooa;oo`002ooooolBo`3o00_ooooo4_oo000;oooooa;oo`3o2ooooolBo`3o00_ooooo
4_l0o`05ooooo`001?oooolBo`00o`_ooooo4_o<c<`;oooooa;o003o2ooooolBo`3oo`_ooooo4_oo
000;oooooa;o0?l02ooooolBool000_ooooo4_oo0?l;oooooa;o0?l02ooooolBo`3o00Gooooo0003
oooooaCo003o2OoooolDolc<c0Wooooo5?l00?l9oooooaCo0?oo2OoooolDool000Wooooo5?l0o`09
oooooaCoo`002OoooolDool0o`Wooooo5?l0o`09oooooaCo0?l01?ooool000?ooooo5?l00?l9oooo
oaCoc<c<2OoooolDo`00o`Wooooo5?l0ool9oooooaCoo`002OoooolDo`3o00Wooooo5?oo0009oooo
oaCoo`3o2OoooolDo`3o00Wooooo5?l0o`04ooooo`000ooooolDo`00o`Wo00005?o<c<`9o`0001Co
003o2Ol0000Do`3oo`Wo00005?oo0009o`0001Co0?l02Ol0000Dool000Wo00005?oo0?l9o`0001Co
0?l02Ol0000Do`3o00Cooooo0003oooooaCo003o2OoooolDolc<c0Wooooo5?l00?l9oooooaCo0?oo
2OoooolDool000Wooooo5?l0o`09oooooaCoo`002OoooolDool0o`Wooooo5?l0o`09oooooaCo0?l0
1?ooool000?ooooo5?l00?l9oooooaCoc<c<2OoooolDo`00o`Wooooo5?l0ool9oooooaCoo`002Ooo
oolDo`3o00Wooooo5?oo0009oooooaCoo`3o2OoooolDo`3o00Wooooo5?l0o`04ooooo`000ooooolD
o`00o`Wooooo5?o<c<`9oooooaCo003o2OoooolDo`3oo`Wooooo5?oo0009oooooaCo0?l02OoooolD
ool000Wooooo5?oo0?l9oooooaCo0?l02OoooolDo`3o00Cooooo0004oooooa;o003o2ooooolBolc<
c0_ooooo4_l00?l;oooooa;o0?oo2ooooolBool000_ooooo4_l0o`0;oooooa;oo`002ooooolBool0
o`_ooooo4_l0o`0;oooooa;o0?l01Oooool000Cooooo4_l00?l;oooooa;oc<c<2ooooolBo`00o`_o
oooo4_l0ool;oooooa;oo`002ooooolBo`3o00_ooooo4_oo000;oooooa;oo`3o2ooooolBo`3o00_o
oooo4_l0o`05ooooo`001Oooool@o`00o`gooooo4?o<c<`=oooooa3o003o3Oooool@o`3oo`gooooo
4?oo000=oooooa3o0?l03Oooool@ool000gooooo4?oo0?l=oooooa3o0?l03Oooool@o`3o00Kooooo
0005oooooa3o003o3Oooool@olc<c0gooooo4?l00?l=oooooa3o0?oo3Oooool@ool000gooooo4?l0
o`0=oooooa3oo`003Oooool@ool0o`gooooo4?l0o`0=oooooa3o0?l01_ooool000Kooooo3_l00?l0
0ooooooo0000ooooo`0;ooooo`03o`000?o<c<coc<c<00coc<c<00?oooooo`000?ooool02oooool0
0ol0003o003oo`00o`0<o`00o`03ooooool0003ooooo00_ooooo00?o0000o`3oool0ool03?l0ool0
0ooooooo0000ooooo`0;ooooo`03o`000?oo003oo`0000coo`0000?o0000ooooooooool02oooool0
0ol0003o0?l0o`3o000<o`3o0003o`000?oooooooooo00[ooooo00?o0000oooooooo00003Ooo0000
0ol0003oooooooooo`0:ooooo`03o`000?ooooooo`3o00goo`3o00?o0000ooooooooool02_ooool0
0ol0003oooooo`3o000=o`3o0003o`000?oooooooooo00[ooooo00?o0000ooooool0o`003Ol0o`07
ooooo`002?ooool:o`00o`Cooooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:
olc<c0Cooooo00?o0000ooooooooool02?ooool01?l0003oooooooooooooool:o`00o`Cooooo00?o
0000ooooooooool02?ooool01?l0003oooooooooooooool:o`3oo`Cooooo00?o0000ooooooooool0
2?ooool01?l0003oooooooooooooool:ool000?ooooo00?o0000ooooooooool02Oooool01?l0003o
ooooooooooooool:o`3o00?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02oooo
o`[oo`000oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_oo0?l3oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:o`3o00?ooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`02ooooo`[o0?l02Oooool000[ooooo1_l00?l7ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6olc<c0Oooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`04ooooo`Ko003o1oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Cooooo1_l0ool7ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool01?ooool6ool000Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`Ko
0?l01_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_oo0006ooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6ool0o`Kooooo00?o0000oooooooo
ool01_ooool00ol0003oooooooooo`05ooooo`Ko0?l01_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00Gooooo1_l0o`0;ooooo`003Oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0:ooooo`04o`000?oooooo
ooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`00
0?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04
o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo
000000_ooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oo
oooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`00
0?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03
o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03
o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooool0
00003?ooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o
00003Oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0;ooooo`;o0000
3Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o0000
3Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool0
2oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool0
2oooool2o`0000gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02_oo
ool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo
00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000oooo
ooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooool000003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o
0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo
00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`00
00cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool01Ol0003o
oooooooooooooooo000000_ooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool03?ooool000[ooooo1_oo0?l6ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool00oooool6o`00o`Kooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`03ooooo`Ko0?l01_ooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00?ooooo1_l0o`06ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00ooo
ool6ool000Kooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`04ooooo`Ko003o1Ooo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo1_oo0005ooooo`03o`000?oo
oooooooo00Sooooo00?o0000ooooooooool01?ooool6ool0o`Gooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`Ko0?l01Oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo00Cooooo1_l0ool;ooooo`002?ooool:ool0o`?ooooo00?o0000ooooooooool02_ooool0
0ol0003oooooooooo`0:o`00o`?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:
o`3o00?ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0:o`3o00?ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0:ool000?ooooo00?o0000ooooooooool02Oooool0
1?l0003oooooooooooooool:o`00o`;ooooo00?o0000ooooooooool02_ooool01?l0003ooooooooo
ooooool:ool000;ooooo00?o0000ooooooooool02_ooool01?l0003oooooooooooooool:ool0o`;o
oooo00?o0000ooooooooool02_ooool01?l0003oooooooooooooool:o`3o00;ooooo00?o0000oooo
ooooool02_ooool01?l0003oooooooooooooool:o`3oo`Wooooo0006ooooo`koo`3o00?o0000oooo
ooooool03?ooool>o`00o`03o`000?oooooooooo00cooooo3_l0o`000ol0003oooooooooo`0<oooo
o`ko0?l000?o0000ooooooooool03?ooool>ool00003o`000?oooooooooo00_ooooo00?o0000o`00
ool00?l03?l00?l>ooooo`03o`000?oo003oo`0000coo`003_ooool00ol0003oo`3oool0o`0<ool0
o`kooooo00?o0000o`3o0?l0o`003?l0o`0>ooooo`03o`000?l0oooo0?oo00co0?oo1oooool000Go
oooo4?oo0?l=oooooa3o003o3Oooool@o`3o00gooooo4?l0o`0=oooooa3oo`003Oooool@o`00o`go
oooo4?oo000=oooooa3oo`3o3Oooool@o`3o00gooooo4?l0ool6ooooo`001Oooool@ool0o`gooooo
4?l00?l=oooooa3o0?l03Oooool@o`3o00gooooo4?oo000=oooooa3o003o3Oooool@ool000gooooo
4?oo0?l=oooooa3o0?l03Oooool@o`3oo`Kooooo0004oooooa;oo`3o2ooooolBo`00o`_ooooo4_l0
o`0;oooooa;o0?l02ooooolBool000_ooooo4_l00?l;oooooa;oo`002ooooolBool0o`_ooooo4_l0
o`0;oooooa;o0?oo1Oooool000Cooooo4_oo0?l;oooooa;o003o2ooooolBo`3o00_ooooo4_l0o`0;
oooooa;oo`002ooooolBo`00o`_ooooo4_oo000;oooooa;oo`3o2ooooolBo`3o00_ooooo4_l0ool5
ooooo`000ooooolDool0o`Wooooo5?l00?l9oooooaCo0?l02OoooolDo`3o00Wooooo5?oo0009oooo
oaCo003o2OoooolDool000Wooooo5?oo0?l9oooooaCo0?l02OoooolDo`3oo`Cooooo0003oooooaCo
o`3o2OoooolDo`00o`Wooooo5?l0o`09oooooaCo0?l02OoooolDool000Wooooo5?l00?l9oooooaCo
o`002OoooolDool0o`Wooooo5?l0o`09oooooaCo0?oo1?ooool000?ooooo5?oo0?l9o`0001Co003o
2Ol0000Do`3o00Wo00005?l0o`09o`0001Coo`002Ol0000Do`00o`Wo00005?oo0009o`0001Coo`3o
2Ol0000Do`3o00Wo00005?l0ool4ooooo`000ooooolDool0o`Wooooo5?l00?l9oooooaCo0?l02Ooo
oolDo`3o00Wooooo5?oo0009oooooaCo003o2OoooolDool000Wooooo5?oo0?l9oooooaCo0?l02Ooo
oolDo`3oo`Cooooo0003oooooaCoo`3o2OoooolDo`00o`Wooooo5?l0o`09oooooaCo0?l02OoooolD
ool000Wooooo5?l00?l9oooooaCoo`002OoooolDool0o`Wooooo5?l0o`09oooooaCo0?oo1?ooool0
00?ooooo5?oo0?l9oooooaCo003o2OoooolDo`3o00Wooooo5?l0o`09oooooaCoo`002OoooolDo`00
o`Wooooo5?oo0009oooooaCoo`3o2OoooolDo`3o00Wooooo5?l0ool4ooooo`001?oooolBool0o`_o
oooo4_l00?l;oooooa;o0?l02ooooolBo`3o00_ooooo4_oo000;oooooa;o003o2ooooolBool000_o
oooo4_oo0?l;oooooa;o0?l02ooooolBo`3oo`Gooooo0004oooooa;oo`3o2ooooolBo`00o`_ooooo
4_l0o`0;oooooa;o0?l02ooooolBool000_ooooo4_l00?l;oooooa;oo`002ooooolBool0o`_ooooo
4_l0o`0;oooooa;o0?oo1Oooool000Gooooo4?oo0?l=oooooa3o003o3Oooool@o`3o00gooooo4?l0
o`0=oooooa3oo`003Oooool@o`00o`gooooo4?oo000=oooooa3oo`3o3Oooool@o`3o00gooooo4?l0
ool6ooooo`001Oooool@ool0o`gooooo4?l00?l=oooooa3o0?l03Oooool@o`3o00gooooo4?oo000=
oooooa3o003o3Oooool@ool000gooooo4?oo0?l=oooooa3o0?l03Oooool@o`3oo`Kooooo0006oooo
o`koo`3o00?oooooo`000?ooool02oooool00ol0003o003oo`00o`0<o`00o`03ooooool0003ooooo
00_ooooo00?o0000o`3o0?l0o`003?l0o`000ooooooo0000ooooo`0;ooooo`03o`000?l0o`3o0?l0
00co0?l000?oooooo`000?ooool02oooool00ol0003oo`00ool0000<ool00003o`000?oooooooooo
00_ooooo00?o0000o`00ool00?l03?l00?l00ol0003oooooooooo`0:ooooo`03o`000?ooooooo`00
00goo`0000?o0000ooooooooool02_ooool00ol0003oooooool0o`0=ool0o`03o`000?oooooooooo
00[ooooo00?o0000ooooool0o`003Ol0o`000ol0003oooooooooo`0:ooooo`03o`000?oooooo0?oo
00go0?oo1oooool000Sooooo2_oo0?l4ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooo
oooooooo2_l00?l4ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_l0o`04
ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_l0o`04ooooo`03o`000?oo
oooooooo00Sooooo00Co0000oooooooooooooooo2_oo0003ooooo`03o`000?oooooooooo00Wooooo
00Co0000oooooooooooooooo2_l00?l3ooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool00_ooool:ool000?ooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`02ooooo`[o
o`3o0oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo2_l0o`03ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:o`3oo`Wooooo000:ooooo`Koo`3o
1oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_l00?l7ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6o`3o00Oooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`04ooooo`Ko0?l01oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Cooooo1_oo0006ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0
1?ooool6o`00o`Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Koo`00
1_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_oo0?l6ooooo`03o`00
0?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6o`3o00Kooooo00?o0000ooooooooool0
1_ooool00ol0003oooooooooo`05ooooo`Ko0?oo2oooool000gooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02_ooool01?l0
003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?oo
ool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool0
2_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00Wooooo00Go0000oooooooo
ooooooooo`00000;ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool0
0ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_oo
ool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo
00003?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0;ooooo`;o0000
3Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o0000
3Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooo000000cooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool0
2oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool0
2oooool2o`0000gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02ooo
ool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02ooo
ool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oo
oooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oo
oooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?oo
ooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03
o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooo000000cooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool0000<
ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003ooooooooo
o`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003ooooo
ooooool0000<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00cooooo000:ooooo`Koc<c<1_ooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_ooo`06ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool00oooool6o`00o`Kooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`03ooooo`Koool01_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooo
oooo00?ooooo1_ooo`06ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01?ooool6
o`0000Gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04ooooo`Ko003o1Oooool0
0ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo1_ooo`05ooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool01?ooool6ool0o`Gooooo00?o0000ooooooooool02?ooool0
0ol0003oooooooooo`04ooooo`Koc<c<2oooool000Sooooo2_o<c<`3ooooo`03o`000?oooooooooo
00[ooooo00?o0000ooooooooool02_ooo`03ooooo`03o`000?oooooooooo00[ooooo00?o0000oooo
ooooool02_l00?l3ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooo`03oooo
o`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooo`03ooooo`03o`000?oooooooooo
00Wooooo00Co0000oooooooooooooooo2_l00002ooooo`03o`000?oooooooooo00[ooooo00Co0000
oooooooooooooooo2_l00?l2ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo
2_ooo`02ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo2_oo0?l2ooooo`03
o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo2_o<c<`9ooooo`001_ooool>olc<c003
o`000?oooooooooo00cooooo3_ooo`000ol0003oooooooooo`0<ooooo`ko003o00?o0000oooooooo
ool03?ooool>oooo0003o`000?oooooooooo00cooooo3_ooo`000ol0003oooooooooo`0;ooooo`oo
00003_ooool00ol0003o003oo`00o`0<o`00o`kooooo00?o0000oooo0?ooo`003?ooo`0>ooooo`03
o`000?oo0?ooo`3o00coo`3o3_ooool00ol0003oc<c<olc<c00<olc<c0Oooooo0005oooooa3oc<c<
3Oooool@oooo00gooooo4?l00?l=oooooa3oool03Oooool@oooo00gooooo4?l0000=oooooa3o003o
3Oooool@oooo00gooooo4?oo0?l=oooooa3oc<c<1_ooool000Gooooo4?o<c<`=oooooa3oool03Ooo
ool@o`00o`gooooo4?ooo`0=oooooa3oool03Oooool@o`0000gooooo4?l00?l=oooooa3oool03Ooo
ool@ool0o`gooooo4?o<c<`6ooooo`001?oooolBolc<c0_ooooo4_ooo`0;oooooa;o003o2ooooolB
oooo00_ooooo4_ooo`0;oooooa;o00002ooooolBo`00o`_ooooo4_ooo`0;oooooa;oo`3o2ooooolB
olc<c0Gooooo0004oooooa;oc<c<2ooooolBoooo00_ooooo4_l00?l;oooooa;oool02ooooolBoooo
00_ooooo4_l0000;oooooa;o003o2ooooolBoooo00_ooooo4_oo0?l;oooooa;oc<c<1Oooool000?o
oooo5?o<c<`9oooooaCoool02OoooolDo`00o`Wooooo5?ooo`09oooooaCoool02OoooolDo`0000Wo
oooo5?l00?l9oooooaCoool02OoooolDool0o`Wooooo5?o<c<`4ooooo`000ooooolDolc<c0Wooooo
5?ooo`09oooooaCo003o2OoooolDoooo00Wooooo5?ooo`09oooooaCo00002OoooolDo`00o`Wooooo
5?ooo`09oooooaCoo`3o2OoooolDolc<c0Cooooo0003oooooaCoc<c<2Ol0000Doooo00Wo00005?l0
0?l9o`0001Coool02Ol0000Doooo02Ko00005?l00?l9o`0001Coool02Ol0000Dool0o`Wo00005?o<
c<`4ooooo`000ooooolDolc<c0Wooooo5?ooo`09oooooaCo003o2OoooolDoooo00Wooooo5?ooo`09
oooooaCo00002OoooolDo`00o`Wooooo5?ooo`09oooooaCoo`3o2OoooolDolc<c0Cooooo0003oooo
oaCoc<c<2OoooolDoooo00Wooooo5?l00?l9oooooaCoool02OoooolDoooo00Wooooo5?l00009oooo
oaCo003o2OoooolDoooo00Wooooo5?oo0?l9oooooaCoc<c<1?ooool000?ooooo5?o<c<`9oooooaCo
ool02OoooolDo`00o`Wooooo5?ooo`09oooooaCoool02OoooolDo`0000Wooooo5?l00?l9oooooaCo
ool02OoooolDool0o`Wooooo5?o<c<`4ooooo`001?oooolBolc<c0_ooooo4_ooo`0;oooooa;o003o
2ooooolBoooo00_ooooo4_ooo`0;oooooa;o00002ooooolBo`00o`_ooooo4_ooo`0;oooooa;oo`3o
2ooooolBolc<c0Gooooo0004oooooa;oc<c<2ooooolBoooo00_ooooo4_l00?l;oooooa;oool02ooo
oolBoooo00_ooooo4_l0000;oooooa;o003o2ooooolBoooo00_ooooo4_oo0?l;oooooa;oc<c<1Ooo
ool000Gooooo4?o<c<`=oooooa3oool03Oooool@o`00o`gooooo4?ooo`0=oooooa3oool03Oooool@
o`0000gooooo4?l00?l=oooooa3oool03Oooool@ool0o`gooooo4?o<c<`6ooooo`001Oooool@olc<
c0gooooo4?ooo`0=oooooa3o003o3Oooool@oooo00gooooo4?ooo`0=oooooa3o00003Oooool@o`00
o`gooooo4?ooo`0=oooooa3oo`3o3Oooool@olc<c0Kooooo0006ooooo`koc<c<00?oooooo`000?oo
ool02oooool00ol0003oool0oooo000<oooo0003ooooool0003ooooo00_ooooo00?o0000o`00ool0
0?l03?l00?l00ooooooo0000ooooo`0;ooooo`03o`000?ooo`3oool000coool000?oooooo`000?oo
ool02oooool00ol0003oool0oooo000<oooo0003o`000?oooooooooo00_ooooo4?l0000<ooooo`03
o`000?oooooo003o00go003o00?o0000ooooooooool02_ooool00ol0003ooooooooo000=oooo0003
o`000?oooooooooo00[ooooo00?o0000oooooooo0?l03Ooo0?l00ol0003oooooooooo`0:ooooo`03
o`000?ooooooc<c<00goc<c<1oooool000Sooooo2_o<c<`4ooooo`03o`000?oooooooooo00Sooooo
00Co0000oooooooooooooooo2_ooo`04ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooo
oooooooo2_l00?l4ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_ooo`04
ooooo`03o`000?oooooooooo00Sooooo00Co0000oooooooooooooooo2_ooo`03ooooo`03o`000?oo
oooooooo00Wooooo00Co0000oooooooooooooooo2_l00003ooooo`03o`000?oooooooooo00Sooooo
00?o0000ooooooooool00_ooool:o`00o`?ooooo00?o0000ooooooooool02?ooool00ol0003ooooo
ooooo`02ooooo`[oool00oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00;ooooo
2_oo0?l3ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00_ooool:olc<c0Wooooo
000:ooooo`Koc<c<1oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Cooooo1_oo
o`07ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01?ooool6o`00o`Oooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`04ooooo`Koool01oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Cooooo1_ooo`06ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool01?ooool6o`0000Kooooo00?o0000ooooooooool01_ooool00ol0003ooooooooo
o`05ooooo`Ko003o1_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Gooooo1_oo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01Oooool6ool0o`Kooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`05ooooo`Koc<c<2oooool000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooo
oooooooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o
0000ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00Wooooo
00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooo
oooo00003?ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000
ooooooooool02_ooool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co
0000oooooooooooo00003?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003ooooooooo
o`0;ooooo`;o00003Oooool00ol0003oooooooooo`0;ooooo`;o00003Oooool00ol0003ooooooooo
o`0:ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o
0000ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000
ooooooooool02oooool2o`0000gooooo00?o0000ooooooooool02oooool2o`0000gooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo0_l0000=
ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00_ooooo0_l0000=
ooooo`03o`000?oooooooooo00_ooooo0_l0000=ooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003oooooooooo`0:
ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_ooool01?l0003ooooooooo
ool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool00ol0003o
ooooooooo`0:ooooo`03o`000?oooooo000000cooooo00?o0000ooooooooool02_ooool01?l0003o
ooooooooool0000<ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00003?ooool0
0ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000cooooo00?o0000ooooooooool02_oo
ool01?l0003oooooooooool0000<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oo
oooooooo00Wooooo00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool00_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01Oooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00cooooo000:oooo
o`Koc<c<1_ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00?ooooo1_l00006oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool6ool0o`Kooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`03ooooo`Ko003o1_ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00?ooooo1_l00006ooooo`03o`000?oooooooooo00Oooooo00?o0000oooo
ooooool01?ooool6oooo00Gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`04oooo
o`Koo`3o1Oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Cooooo1_l00005oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool01?ooool6ool0o`Gooooo00?o0000oooo
ooooool02?ooool00ol0003oooooooooo`04ooooo`Ko003o2oooool000Sooooo2_o<c<`3ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool02_l00003ooooo`03o`000?oooooooooo00[o
oooo00?o0000ooooooooool02_oo0?l3ooooo`03o`000?oooooooooo00[ooooo00?o0000oooooooo
ool02_l00?l3ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_l00003ooooo`03
o`000?oooooooooo00Wooooo00Co0000oooooooooooooooo2_ooo`02ooooo`03o`000?oooooooooo
00[ooooo00Co0000oooooooooooooooo2_oo0?l2ooooo`03o`000?oooooooooo00[ooooo00Co0000
oooooooooooooooo2_l00002ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo
2_oo0?l2ooooo`03o`000?oooooooooo00[ooooo00Co0000oooooooooooooooo2_l00?l9ooooo`00
1_ooool>olc<c003o`000?oooooooooo00cooooo3ol0000>ooooo`koo`3o00?o0000ooooooooool0
3?ooool>o`00o`03o`000?oooooooooo00cooooo3ol0000=ooooo`03o`000?ooo`3oool000coool0
3_ooool00ol0003oo`3oool0o`0<ool0o`kooooo3ol0000>ooooo`03o`000?oo0?ooo`3o00coo`3o
3_ooool00ol0003o003oo`00o`0<o`00o`Oooooo0005oooooa3oc<c<3Oooool@o`0000gooooo4?oo
0?l=oooooa3o003o3Oooool@o`0000gooooo4?ooo`0=oooooa3oo`3o3Oooool@o`0000gooooo4?oo
0?l=oooooa3o003o1_ooool000Gooooo4?o<c<`=oooooa3o00003Oooool@ool0o`gooooo4?l00?l=
oooooa3o00003Oooool@oooo00gooooo4?oo0?l=oooooa3o00003Oooool@ool0o`gooooo4?l00?l6
ooooo`001?oooolBolc<c0_ooooo4_l0000;oooooa;oo`3o2ooooolBo`00o`_ooooo4_l0000;oooo
oa;oool02ooooolBool0o`_ooooo4_l0000;oooooa;oo`3o2ooooolBo`00o`Gooooo0004oooooa;o
c<c<2ooooolBo`0000_ooooo4_oo0?l;oooooa;o003o2ooooolBo`0000_ooooo4_ooo`0;oooooa;o
o`3o2ooooolBo`0000_ooooo4_oo0?l;oooooa;o003o1Oooool000?ooooo5?o<c<`9oooooaCo0000
2OoooolDool0o`Wooooo5?l00?l9oooooaCo00002OoooolDoooo00Wooooo5?oo0?l9oooooaCo0000
2OoooolDool0o`Wooooo5?l00?l4ooooo`000ooooolDolc<c0Wooooo5?l00009oooooaCoo`3o2Ooo
oolDo`00o`Wooooo5?l00009oooooaCoool02OoooolDool0o`Wooooo5?l00009oooooaCoo`3o2Ooo
oolDo`00o`Cooooo0003oooooaCoc<c<9_l0000Dool0o`Wo00005?l00?lVo`0001Coool02Ol0000D
ool0obKo00005?oo0?l9o`0001Co003o1?ooool000?ooooo5?o<c<`9oooooaCo00002OoooolDool0
o`Wooooo5?l00?l9oooooaCo00002OoooolDoooo00Wooooo5?oo0?l9oooooaCo00002OoooolDool0
o`Wooooo5?l00?l4ooooo`000ooooolDolc<c0Wooooo5?l00009oooooaCoo`3o2OoooolDo`00o`Wo
oooo5?l00009oooooaCoool02OoooolDool0o`Wooooo5?l00009oooooaCoo`3o2OoooolDo`00o`Co
oooo0003oooooaCoc<c<2OoooolDo`0000Wooooo5?oo0?l9oooooaCo003o2OoooolDo`0000Wooooo
5?ooo`09oooooaCoo`3o2OoooolDo`0000Wooooo5?oo0?l9oooooaCo003o1?ooool000Cooooo4_o<
c<`;oooooa;o00002ooooolBool0o`_ooooo4_l00?l;oooooa;o00002ooooolBoooo00_ooooo4_oo
0?l;oooooa;o00002ooooolBool0o`_ooooo4_l00?l5ooooo`001?oooolBolc<c0_ooooo4_l0000;
oooooa;oo`3o2ooooolBo`00o`_ooooo4_l0000;oooooa;oool02ooooolBool0o`_ooooo4_l0000;
oooooa;oo`3o2ooooolBo`00o`Gooooo0005oooooa3oc<c<3Oooool@o`0000gooooo4?oo0?l=oooo
oa3o003o3Oooool@o`0000gooooo4?ooo`0=oooooa3oo`3o3Oooool@o`0000gooooo4?oo0?l=oooo
oa3o003o1_ooool000Gooooo4?o<c<`=oooooa3o00003Oooool@ool0o`gooooo4?l00?l=oooooa3o
00003Oooool@oooo00gooooo4?oo0?l=oooooa3o00003Oooool@ool0o`gooooo4?l00?l6ooooo`00
1_ooool>olc<c0oooooo3_l0000?ooooo`koo`3o3oooool>o`00o`oooooo3_l0000?ooooo`koool0
3oooool>ool0o`oooooo3_l0000?ooooo`koo`3o3oooool>o`00o`Oooooo0008ooooo`[oc<c<4ooo
ool:o`0001?ooooo2_oo0?lCooooo`[o003o4oooool:o`0001?ooooo2_ooo`0Cooooo`[oo`3o4ooo
ool:o`0001?ooooo2_oo0?lCooooo`[o003o2Oooool000[ooooo1_o<c<`Gooooo`Ko00005oooool6
ool0oaOooooo1_l00?lGooooo`Ko00005oooool6oooo01Oooooo1_oo0?lGooooo`Ko00005oooool6
ool0oaOooooo1_l00?l;ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[{
    \(\(l = {};\)\), "\[IndentingNewLine]", 
    \(\(Do[\[IndentingNewLine]AppendTo[l, 
          ExtractGraph[ugM8]]; \[IndentingNewLine]Evolve[
          ugM8], {25}\[IndentingNewLine]];\)\)}], "Input",
  CellLabel->"In[65]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(ShowGraphArray[\[IndentingNewLine]Partition[Take[l, 9], 3], 
        VertexStyle \[Rule] 
          Disk[Large]\[IndentingNewLine]];\)\(\[IndentingNewLine]\)
    \)\)], "Input",
  CellLabel->"In[67]:=",
  CellTags->"LocalInteractions:Introduction"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0238095 0.31746 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0238095 0.321429 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
0 0 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
1 1 0 r
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
0 0 0 r
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
1 1 0 r
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 0 0 r
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
1 1 0 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.0238095 0.64881 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
1 1 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
0 0 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
1 1 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
0 0 0 r
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.0238095 0.97619 0.321429 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
1 1 0 r
.04545 .95455 Mdot
0 0 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.35119 0.321429 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
0 1 0 r
.14646 .04545 Mdot
.24747 .04545 Mdot
.34848 .04545 Mdot
0 0 0 r
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 1 r
.04545 .14646 Mdot
0 1 0 r
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
0 0 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 1 r
.04545 .24747 Mdot
0 1 0 r
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
0 0 0 r
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
1 0 0 r
.95455 .44949 Mdot
1 1 0 r
.04545 .55051 Mdot
.14646 .55051 Mdot
0 0 0 r
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
1 0 0 r
.95455 .55051 Mdot
1 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
0 0 0 r
.55051 .65152 Mdot
.65152 .65152 Mdot
1 0 0 r
.75253 .65152 Mdot
1 1 0 r
.85354 .65152 Mdot
1 0 0 r
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.35119 0.64881 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 0 r
.24747 .04545 Mdot
0 0 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 1 r
.04545 .14646 Mdot
.14646 .14646 Mdot
0 1 0 r
.24747 .14646 Mdot
.34848 .14646 Mdot
0 0 0 r
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
1 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
0 0 0 r
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
0 0 0 r
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
1 1 0 r
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.35119 0.97619 0.64881 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 0 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
.04545 .14646 Mdot
.14646 .14646 Mdot
.24747 .14646 Mdot
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
.04545 .24747 Mdot
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
.04545 .34848 Mdot
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
.95455 .34848 Mdot
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
.85354 .44949 Mdot
.95455 .44949 Mdot
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
.85354 .55051 Mdot
.95455 .55051 Mdot
1 1 0 r
.04545 .65152 Mdot
0 0 0 r
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
.85354 .65152 Mdot
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
0 0 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
.75253 .75253 Mdot
1 1 0 r
.85354 .75253 Mdot
.95455 .75253 Mdot
.04545 .85354 Mdot
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
0 0 0 r
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
.75253 .85354 Mdot
1 1 0 r
.85354 .85354 Mdot
.95455 .85354 Mdot
.04545 .95455 Mdot
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.678571 0.321429 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
.07 w
.04545 .04545 Mdot
0 1 1 r
.14646 .04545 Mdot
0 0 0 r
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
0 0 1 r
.55051 .04545 Mdot
1 0 0 r
.65152 .04545 Mdot
1 0 1 r
.75253 .04545 Mdot
.85354 .04545 Mdot
1 1 0 r
.95455 .04545 Mdot
1 0 0 r
.04545 .14646 Mdot
0 0 0 r
.14646 .14646 Mdot
0 1 0 r
.24747 .14646 Mdot
1 1 0 r
.34848 .14646 Mdot
1 0 1 r
.44949 .14646 Mdot
0 0 0 r
.55051 .14646 Mdot
0 0 1 r
.65152 .14646 Mdot
0 0 0 r
.75253 .14646 Mdot
0 0 1 r
.85354 .14646 Mdot
1 0 1 r
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.8 .8 .8 r
.14646 .24747 Mdot
1 0 1 r
.24747 .24747 Mdot
1 0 0 r
.34848 .24747 Mdot
0 0 0 r
.44949 .24747 Mdot
1 1 0 r
.55051 .24747 Mdot
0 0 1 r
.65152 .24747 Mdot
1 0 1 r
.75253 .24747 Mdot
.8 .8 .8 r
.85354 .24747 Mdot
1 1 0 r
.95455 .24747 Mdot
0 1 1 r
.04545 .34848 Mdot
.14646 .34848 Mdot
.8 .8 .8 r
.24747 .34848 Mdot
0 1 1 r
.34848 .34848 Mdot
.44949 .34848 Mdot
0 1 0 r
.55051 .34848 Mdot
0 0 0 r
.65152 .34848 Mdot
1 0 0 r
.75253 .34848 Mdot
0 0 1 r
.85354 .34848 Mdot
0 1 0 r
.95455 .34848 Mdot
0 0 1 r
.04545 .44949 Mdot
1 0 1 r
.14646 .44949 Mdot
0 0 1 r
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
0 0 1 r
.44949 .44949 Mdot
0 1 0 r
.55051 .44949 Mdot
1 0 1 r
.65152 .44949 Mdot
1 0 0 r
.75253 .44949 Mdot
0 1 0 r
.85354 .44949 Mdot
1 0 0 r
.95455 .44949 Mdot
0 1 0 r
.04545 .55051 Mdot
1 0 0 r
.14646 .55051 Mdot
1 0 1 r
.24747 .55051 Mdot
0 1 1 r
.34848 .55051 Mdot
.8 .8 .8 r
.44949 .55051 Mdot
0 0 0 r
.55051 .55051 Mdot
.65152 .55051 Mdot
1 0 0 r
.75253 .55051 Mdot
0 1 0 r
.85354 .55051 Mdot
1 0 1 r
.95455 .55051 Mdot
0 0 1 r
.04545 .65152 Mdot
.8 .8 .8 r
.14646 .65152 Mdot
0 0 1 r
.24747 .65152 Mdot
0 1 1 r
.34848 .65152 Mdot
1 0 0 r
.44949 .65152 Mdot
0 1 0 r
.55051 .65152 Mdot
1 0 0 r
.65152 .65152 Mdot
1 0 1 r
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
1 0 1 r
.04545 .75253 Mdot
0 0 1 r
.14646 .75253 Mdot
0 1 0 r
.24747 .75253 Mdot
.34848 .75253 Mdot
1 0 0 r
.44949 .75253 Mdot
0 0 1 r
.55051 .75253 Mdot
1 0 0 r
.65152 .75253 Mdot
1 0 1 r
.75253 .75253 Mdot
0 1 0 r
.85354 .75253 Mdot
0 1 1 r
.95455 .75253 Mdot
.8 .8 .8 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
0 0 1 r
.24747 .85354 Mdot
1 1 0 r
.34848 .85354 Mdot
.44949 .85354 Mdot
0 0 0 r
.55051 .85354 Mdot
0 0 1 r
.65152 .85354 Mdot
1 1 0 r
.75253 .85354 Mdot
1 0 1 r
.85354 .85354 Mdot
.8 .8 .8 r
.95455 .85354 Mdot
.04545 .95455 Mdot
0 0 0 r
.14646 .95455 Mdot
1 0 1 r
.24747 .95455 Mdot
0 0 1 r
.34848 .95455 Mdot
0 0 0 r
.44949 .95455 Mdot
1 1 0 r
.55051 .95455 Mdot
1 0 1 r
.65152 .95455 Mdot
0 0 0 r
.75253 .95455 Mdot
1 0 1 r
.85354 .95455 Mdot
0 0 1 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.35119 0.678571 0.64881 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
.24747 .04545 Mdot
0 1 0 r
.34848 .04545 Mdot
0 0 0 r
.44949 .04545 Mdot
0 0 1 r
.55051 .04545 Mdot
0 0 0 r
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
0 0 1 r
.95455 .04545 Mdot
0 1 0 r
.04545 .14646 Mdot
.14646 .14646 Mdot
0 1 1 r
.24747 .14646 Mdot
0 0 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.8 .8 .8 r
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
.14646 .24747 Mdot
0 0 0 r
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.8 .8 .8 r
.95455 .24747 Mdot
0 1 0 r
.04545 .34848 Mdot
0 0 1 r
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
0 0 0 r
.44949 .34848 Mdot
.55051 .34848 Mdot
1 0 0 r
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
0 1 0 r
.95455 .34848 Mdot
1 0 0 r
.04545 .44949 Mdot
0 0 1 r
.14646 .44949 Mdot
.24747 .44949 Mdot
.34848 .44949 Mdot
0 0 0 r
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
1 0 0 r
.75253 .44949 Mdot
.85354 .44949 Mdot
0 1 0 r
.95455 .44949 Mdot
.8 .8 .8 r
.04545 .55051 Mdot
0 0 1 r
.14646 .55051 Mdot
.24747 .55051 Mdot
.34848 .55051 Mdot
0 1 0 r
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
1 0 0 r
.75253 .55051 Mdot
.85354 .55051 Mdot
0 1 0 r
.95455 .55051 Mdot
0 0 1 r
.04545 .65152 Mdot
.14646 .65152 Mdot
0 1 0 r
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
.55051 .65152 Mdot
.65152 .65152 Mdot
0 0 0 r
.75253 .65152 Mdot
0 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
1 1 0 r
.04545 .75253 Mdot
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
0 1 0 r
.44949 .75253 Mdot
.55051 .75253 Mdot
.65152 .75253 Mdot
1 1 0 r
.75253 .75253 Mdot
.85354 .75253 Mdot
0 1 0 r
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
.14646 .85354 Mdot
1 1 0 r
.24747 .85354 Mdot
.34848 .85354 Mdot
0 0 1 r
.44949 .85354 Mdot
.55051 .85354 Mdot
.65152 .85354 Mdot
1 1 0 r
.75253 .85354 Mdot
.85354 .85354 Mdot
0 1 0 r
.95455 .85354 Mdot
0 0 0 r
.04545 .95455 Mdot
.14646 .95455 Mdot
1 1 0 r
.24747 .95455 Mdot
.34848 .95455 Mdot
0 0 0 r
.44949 .95455 Mdot
.55051 .95455 Mdot
.65152 .95455 Mdot
1 1 0 r
.75253 .95455 Mdot
.85354 .95455 Mdot
1 0 1 r
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.678571 0.678571 0.97619 0.97619 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.14646 .04545 L
s
.04545 .04545 m
.04545 .14646 L
s
.04545 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .04545 L
s
.14646 .04545 m
.04545 .14646 L
s
.14646 .04545 m
.14646 .14646 L
s
.14646 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .04545 L
s
.24747 .04545 m
.14646 .14646 L
s
.24747 .04545 m
.24747 .14646 L
s
.24747 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .04545 L
s
.34848 .04545 m
.24747 .14646 L
s
.34848 .04545 m
.34848 .14646 L
s
.34848 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .04545 L
s
.44949 .04545 m
.34848 .14646 L
s
.44949 .04545 m
.44949 .14646 L
s
.44949 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .04545 L
s
.55051 .04545 m
.44949 .14646 L
s
.55051 .04545 m
.55051 .14646 L
s
.55051 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .04545 L
s
.65152 .04545 m
.55051 .14646 L
s
.65152 .04545 m
.65152 .14646 L
s
.65152 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .04545 L
s
.75253 .04545 m
.65152 .14646 L
s
.75253 .04545 m
.75253 .14646 L
s
.75253 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .04545 L
s
.85354 .04545 m
.75253 .14646 L
s
.85354 .04545 m
.85354 .14646 L
s
.85354 .04545 m
.95455 .14646 L
s
.95455 .04545 m
.85354 .14646 L
s
.95455 .04545 m
.95455 .14646 L
s
.04545 .14646 m
.14646 .14646 L
s
.04545 .14646 m
.04545 .24747 L
s
.04545 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .14646 L
s
.14646 .14646 m
.04545 .24747 L
s
.14646 .14646 m
.14646 .24747 L
s
.14646 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .14646 L
s
.24747 .14646 m
.14646 .24747 L
s
.24747 .14646 m
.24747 .24747 L
s
.24747 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .14646 L
s
.34848 .14646 m
.24747 .24747 L
s
.34848 .14646 m
.34848 .24747 L
s
.34848 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .14646 L
s
.44949 .14646 m
.34848 .24747 L
s
.44949 .14646 m
.44949 .24747 L
s
.44949 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .14646 L
s
.55051 .14646 m
.44949 .24747 L
s
.55051 .14646 m
.55051 .24747 L
s
.55051 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .14646 L
s
.65152 .14646 m
.55051 .24747 L
s
.65152 .14646 m
.65152 .24747 L
s
.65152 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .14646 L
s
.75253 .14646 m
.65152 .24747 L
s
.75253 .14646 m
.75253 .24747 L
s
.75253 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .14646 L
s
.85354 .14646 m
.75253 .24747 L
s
.85354 .14646 m
.85354 .24747 L
s
.85354 .14646 m
.95455 .24747 L
s
.95455 .14646 m
.85354 .24747 L
s
.95455 .14646 m
.95455 .24747 L
s
.04545 .24747 m
.14646 .24747 L
s
.04545 .24747 m
.04545 .34848 L
s
.04545 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .24747 L
s
.14646 .24747 m
.04545 .34848 L
s
.14646 .24747 m
.14646 .34848 L
s
.14646 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .24747 L
s
.24747 .24747 m
.14646 .34848 L
s
.24747 .24747 m
.24747 .34848 L
s
.24747 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .24747 L
s
.34848 .24747 m
.24747 .34848 L
s
.34848 .24747 m
.34848 .34848 L
s
.34848 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .24747 L
s
.44949 .24747 m
.34848 .34848 L
s
.44949 .24747 m
.44949 .34848 L
s
.44949 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .24747 L
s
.55051 .24747 m
.44949 .34848 L
s
.55051 .24747 m
.55051 .34848 L
s
.55051 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .24747 L
s
.65152 .24747 m
.55051 .34848 L
s
.65152 .24747 m
.65152 .34848 L
s
.65152 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .24747 L
s
.75253 .24747 m
.65152 .34848 L
s
.75253 .24747 m
.75253 .34848 L
s
.75253 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .24747 L
s
.85354 .24747 m
.75253 .34848 L
s
.85354 .24747 m
.85354 .34848 L
s
.85354 .24747 m
.95455 .34848 L
s
.95455 .24747 m
.85354 .34848 L
s
.95455 .24747 m
.95455 .34848 L
s
.04545 .34848 m
.14646 .34848 L
s
.04545 .34848 m
.04545 .44949 L
s
.04545 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .34848 L
s
.14646 .34848 m
.04545 .44949 L
s
.14646 .34848 m
.14646 .44949 L
s
.14646 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .34848 L
s
.24747 .34848 m
.14646 .44949 L
s
.24747 .34848 m
.24747 .44949 L
s
.24747 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .34848 L
s
.34848 .34848 m
.24747 .44949 L
s
.34848 .34848 m
.34848 .44949 L
s
.34848 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .34848 L
s
.44949 .34848 m
.34848 .44949 L
s
.44949 .34848 m
.44949 .44949 L
s
.44949 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .34848 L
s
.55051 .34848 m
.44949 .44949 L
s
.55051 .34848 m
.55051 .44949 L
s
.55051 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .34848 L
s
.65152 .34848 m
.55051 .44949 L
s
.65152 .34848 m
.65152 .44949 L
s
.65152 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .34848 L
s
.75253 .34848 m
.65152 .44949 L
s
.75253 .34848 m
.75253 .44949 L
s
.75253 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .34848 L
s
.85354 .34848 m
.75253 .44949 L
s
.85354 .34848 m
.85354 .44949 L
s
.85354 .34848 m
.95455 .44949 L
s
.95455 .34848 m
.85354 .44949 L
s
.95455 .34848 m
.95455 .44949 L
s
.04545 .44949 m
.14646 .44949 L
s
.04545 .44949 m
.04545 .55051 L
s
.04545 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .44949 L
s
.14646 .44949 m
.04545 .55051 L
s
.14646 .44949 m
.14646 .55051 L
s
.14646 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .44949 L
s
.24747 .44949 m
.14646 .55051 L
s
.24747 .44949 m
.24747 .55051 L
s
.24747 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .44949 L
s
.34848 .44949 m
.24747 .55051 L
s
.34848 .44949 m
.34848 .55051 L
s
.34848 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .44949 L
s
.44949 .44949 m
.34848 .55051 L
s
.44949 .44949 m
.44949 .55051 L
s
.44949 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .44949 L
s
.55051 .44949 m
.44949 .55051 L
s
.55051 .44949 m
.55051 .55051 L
s
.55051 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .44949 L
s
.65152 .44949 m
.55051 .55051 L
s
.65152 .44949 m
.65152 .55051 L
s
.65152 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .44949 L
s
.75253 .44949 m
.65152 .55051 L
s
.75253 .44949 m
.75253 .55051 L
s
.75253 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .44949 L
s
.85354 .44949 m
.75253 .55051 L
s
.85354 .44949 m
.85354 .55051 L
s
.85354 .44949 m
.95455 .55051 L
s
.95455 .44949 m
.85354 .55051 L
s
.95455 .44949 m
.95455 .55051 L
s
.04545 .55051 m
.14646 .55051 L
s
.04545 .55051 m
.04545 .65152 L
s
.04545 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .55051 L
s
.14646 .55051 m
.04545 .65152 L
s
.14646 .55051 m
.14646 .65152 L
s
.14646 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .55051 L
s
.24747 .55051 m
.14646 .65152 L
s
.24747 .55051 m
.24747 .65152 L
s
.24747 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .55051 L
s
.34848 .55051 m
.24747 .65152 L
s
.34848 .55051 m
.34848 .65152 L
s
.34848 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .55051 L
s
.44949 .55051 m
.34848 .65152 L
s
.44949 .55051 m
.44949 .65152 L
s
.44949 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .55051 L
s
.55051 .55051 m
.44949 .65152 L
s
.55051 .55051 m
.55051 .65152 L
s
.55051 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .55051 L
s
.65152 .55051 m
.55051 .65152 L
s
.65152 .55051 m
.65152 .65152 L
s
.65152 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .55051 L
s
.75253 .55051 m
.65152 .65152 L
s
.75253 .55051 m
.75253 .65152 L
s
.75253 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .55051 L
s
.85354 .55051 m
.75253 .65152 L
s
.85354 .55051 m
.85354 .65152 L
s
.85354 .55051 m
.95455 .65152 L
s
.95455 .55051 m
.85354 .65152 L
s
.95455 .55051 m
.95455 .65152 L
s
.04545 .65152 m
.14646 .65152 L
s
.04545 .65152 m
.04545 .75253 L
s
.04545 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .65152 L
s
.14646 .65152 m
.04545 .75253 L
s
.14646 .65152 m
.14646 .75253 L
s
.14646 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .65152 L
s
.24747 .65152 m
.14646 .75253 L
s
.24747 .65152 m
.24747 .75253 L
s
.24747 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .65152 L
s
.34848 .65152 m
.24747 .75253 L
s
.34848 .65152 m
.34848 .75253 L
s
.34848 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .65152 L
s
.44949 .65152 m
.34848 .75253 L
s
.44949 .65152 m
.44949 .75253 L
s
.44949 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .65152 L
s
.55051 .65152 m
.44949 .75253 L
s
.55051 .65152 m
.55051 .75253 L
s
.55051 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .65152 L
s
.65152 .65152 m
.55051 .75253 L
s
.65152 .65152 m
.65152 .75253 L
s
.65152 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .65152 L
s
.75253 .65152 m
.65152 .75253 L
s
.75253 .65152 m
.75253 .75253 L
s
.75253 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .65152 L
s
.85354 .65152 m
.75253 .75253 L
s
.85354 .65152 m
.85354 .75253 L
s
.85354 .65152 m
.95455 .75253 L
s
.95455 .65152 m
.85354 .75253 L
s
.95455 .65152 m
.95455 .75253 L
s
.04545 .75253 m
.14646 .75253 L
s
.04545 .75253 m
.04545 .85354 L
s
.04545 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .75253 L
s
.14646 .75253 m
.04545 .85354 L
s
.14646 .75253 m
.14646 .85354 L
s
.14646 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .75253 L
s
.24747 .75253 m
.14646 .85354 L
s
.24747 .75253 m
.24747 .85354 L
s
.24747 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .75253 L
s
.34848 .75253 m
.24747 .85354 L
s
.34848 .75253 m
.34848 .85354 L
s
.34848 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .75253 L
s
.44949 .75253 m
.34848 .85354 L
s
.44949 .75253 m
.44949 .85354 L
s
.44949 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .75253 L
s
.55051 .75253 m
.44949 .85354 L
s
.55051 .75253 m
.55051 .85354 L
s
.55051 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .75253 L
s
.65152 .75253 m
.55051 .85354 L
s
.65152 .75253 m
.65152 .85354 L
s
.65152 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .75253 L
s
.75253 .75253 m
.65152 .85354 L
s
.75253 .75253 m
.75253 .85354 L
s
.75253 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .75253 L
s
.85354 .75253 m
.75253 .85354 L
s
.85354 .75253 m
.85354 .85354 L
s
.85354 .75253 m
.95455 .85354 L
s
.95455 .75253 m
.85354 .85354 L
s
.95455 .75253 m
.95455 .85354 L
s
.04545 .85354 m
.14646 .85354 L
s
.04545 .85354 m
.04545 .95455 L
s
.04545 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .85354 L
s
.14646 .85354 m
.04545 .95455 L
s
.14646 .85354 m
.14646 .95455 L
s
.14646 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .85354 L
s
.24747 .85354 m
.14646 .95455 L
s
.24747 .85354 m
.24747 .95455 L
s
.24747 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .85354 L
s
.34848 .85354 m
.24747 .95455 L
s
.34848 .85354 m
.34848 .95455 L
s
.34848 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .85354 L
s
.44949 .85354 m
.34848 .95455 L
s
.44949 .85354 m
.44949 .95455 L
s
.44949 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .85354 L
s
.55051 .85354 m
.44949 .95455 L
s
.55051 .85354 m
.55051 .95455 L
s
.55051 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .85354 L
s
.65152 .85354 m
.55051 .95455 L
s
.65152 .85354 m
.65152 .95455 L
s
.65152 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .85354 L
s
.75253 .85354 m
.65152 .95455 L
s
.75253 .85354 m
.75253 .95455 L
s
.75253 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .85354 L
s
.85354 .85354 m
.75253 .95455 L
s
.85354 .85354 m
.85354 .95455 L
s
.85354 .85354 m
.95455 .95455 L
s
.95455 .85354 m
.85354 .95455 L
s
.95455 .85354 m
.95455 .95455 L
s
.04545 .95455 m
.14646 .95455 L
s
.14646 .95455 m
.24747 .95455 L
s
.24747 .95455 m
.34848 .95455 L
s
.34848 .95455 m
.44949 .95455 L
s
.44949 .95455 m
.55051 .95455 L
s
.55051 .95455 m
.65152 .95455 L
s
.65152 .95455 m
.75253 .95455 L
s
.75253 .95455 m
.85354 .95455 L
s
.85354 .95455 m
.95455 .95455 L
s
0 1 1 r
.07 w
.04545 .04545 Mdot
.14646 .04545 Mdot
0 1 0 r
.24747 .04545 Mdot
0 0 0 r
.34848 .04545 Mdot
.44949 .04545 Mdot
.55051 .04545 Mdot
.65152 .04545 Mdot
.75253 .04545 Mdot
.85354 .04545 Mdot
.95455 .04545 Mdot
0 1 1 r
.04545 .14646 Mdot
0 0 0 r
.14646 .14646 Mdot
0 1 0 r
.24747 .14646 Mdot
0 0 0 r
.34848 .14646 Mdot
.44949 .14646 Mdot
.55051 .14646 Mdot
.65152 .14646 Mdot
.75253 .14646 Mdot
.85354 .14646 Mdot
.95455 .14646 Mdot
0 1 0 r
.04545 .24747 Mdot
0 0 0 r
.14646 .24747 Mdot
.24747 .24747 Mdot
.34848 .24747 Mdot
.44949 .24747 Mdot
.55051 .24747 Mdot
.65152 .24747 Mdot
.75253 .24747 Mdot
.85354 .24747 Mdot
.95455 .24747 Mdot
0 0 1 r
.04545 .34848 Mdot
0 0 0 r
.14646 .34848 Mdot
.24747 .34848 Mdot
.34848 .34848 Mdot
.44949 .34848 Mdot
.55051 .34848 Mdot
.65152 .34848 Mdot
.75253 .34848 Mdot
.85354 .34848 Mdot
1 0 0 r
.95455 .34848 Mdot
0 0 1 r
.04545 .44949 Mdot
.14646 .44949 Mdot
.24747 .44949 Mdot
0 0 0 r
.34848 .44949 Mdot
.44949 .44949 Mdot
.55051 .44949 Mdot
.65152 .44949 Mdot
.75253 .44949 Mdot
1 0 0 r
.85354 .44949 Mdot
.95455 .44949 Mdot
0 0 1 r
.04545 .55051 Mdot
.14646 .55051 Mdot
.24747 .55051 Mdot
0 0 0 r
.34848 .55051 Mdot
.44949 .55051 Mdot
.55051 .55051 Mdot
.65152 .55051 Mdot
.75253 .55051 Mdot
1 0 0 r
.85354 .55051 Mdot
.95455 .55051 Mdot
1 1 0 r
.04545 .65152 Mdot
.14646 .65152 Mdot
.24747 .65152 Mdot
.34848 .65152 Mdot
.44949 .65152 Mdot
0 1 0 r
.55051 .65152 Mdot
.65152 .65152 Mdot
.75253 .65152 Mdot
1 1 0 r
.85354 .65152 Mdot
.95455 .65152 Mdot
0 0 0 r
.04545 .75253 Mdot
1 1 0 r
.14646 .75253 Mdot
.24747 .75253 Mdot
.34848 .75253 Mdot
.44949 .75253 Mdot
0 1 0 r
.55051 .75253 Mdot
1 1 0 r
.65152 .75253 Mdot
.75253 .75253 Mdot
.85354 .75253 Mdot
.95455 .75253 Mdot
0 0 0 r
.04545 .85354 Mdot
1 1 0 r
.14646 .85354 Mdot
.24747 .85354 Mdot
.34848 .85354 Mdot
.44949 .85354 Mdot
0 0 0 r
.55051 .85354 Mdot
1 1 0 r
.65152 .85354 Mdot
.75253 .85354 Mdot
.85354 .85354 Mdot
.95455 .85354 Mdot
0 0 0 r
.04545 .95455 Mdot
1 1 0 r
.14646 .95455 Mdot
.24747 .95455 Mdot
.34848 .95455 Mdot
.44949 .95455 Mdot
0 0 0 r
.55051 .95455 Mdot
1 1 0 r
.65152 .95455 Mdot
.75253 .95455 Mdot
.85354 .95455 Mdot
.95455 .95455 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[67]:=",
  ImageSize->{368, 368},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005`0001L2000`400?l00000o`00003oooooooma
ooooo`00oooooomaooooo`00oooooomaooooo`00oooooomaooooo`00oooooomaooooo`00ooooooma
ooooo`00oooooomaooooo`00oooooomaooooo`00oooooomaooooo`00oooooomaooooo`00ooooooma
ooooo`00oooooomaooooo`003?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000So
oooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0001;o
oooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000So
oooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol0000Booooo`?o00002?ooool3o`0000So
oooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000So
oooo0ol00008ooooo`?o00003_ooool000_ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol0000@ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000gooooo000:ooooof[o00003_oooomZo`0000koooooJ_l0
000<ooooo`002_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00003?ooool000[ooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000cooooo000;ooooo`Ko00001?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko
00004?ooool6o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko
000000?oooooo`000?ooool00_ooool6o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo
0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`00013ooooo1Ol000000ooooooo
0000ooooo`02ooooo`Ko000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?o
oooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003
ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?oo
ool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol0000=ooooo`003?ooool3o`0000;ooooo
00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`00
00;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_oo
ool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo
00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooo
oooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0001;ooooo0ol0
0002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;o
oooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?oooooooooo
o`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00003ooooo`03o`000?oo
oooo000000;ooooo0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`00
0?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0000Boooo
o`?o00000oooool00ol0003oooooo`000002ooooo`?o00000oooool01Ol0003oooooooooool0003o
oooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003ooooooooo
ool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003o
ooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool0
1Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o0000
3_ooool000gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo
000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool0
00gooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000001Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oo
ooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000004oooool01Ol0003ooooooooooooooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000003_ooool000gooooo00Co0000
oooooooooooo00001?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0001Cooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_oo
ool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0001?ooooo00Co0000
oooooooooooo00001Oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000kooooo000<ooooo`Co00001_ooool5o`0000Kooooo1Ol00006oooo
o`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006oooo
o`Co00004_ooool4o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006oooo
o`Go00001oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05oooo
o`?o000000?oooooo`000?ooool01Oooool3o`0001;ooooo1?l00007ooooo`?o000000?oooooo`00
0?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05oooo
o`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooo
oooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo
00Gooooo0ol0000>ooooo`002oooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013o
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00003Oooool000[oooooJ_l0000>ooooof[o00003_oooomZo`0000cooooo
000:ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
000<ooooo`002_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00003?ooool000_ooooo1_l00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1_l0000@oooo
o`Ko00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1_l000000ooo
oooo0000ooooo`02ooooo`Ko000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo
00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go00004?ooool5o`000003ooooool0003ooooo
00;ooooo1_l000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`00
0003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`00
0?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03oooo
o`Go000000?oooooo`000?ooool00oooool5o`0000gooooo000<ooooo`?o00000_ooool01?l0003o
ooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool0
1?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o0000
0_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002oooo
o`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool0
0002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00004_ooool3o`0000;ooooo
00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`00
00;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_oo
ool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000?ooooo00?o0000ooooool00000
0_ooool3o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooo
oooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0001;ooooo0ol00003
ooooo`03o`000?oooooo000000;ooooo0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0
0003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?oo
ool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oooooooooo
o`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oo
ooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0000>ooooo`00
3Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooo
ooooo`000003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`00000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`00000>ooooo`003Oooool01?l0003ooooooooo
ool00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooo00005?ooool01?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00004oooool01?l0003ooooooooo
ool00005ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooooooooooo00003_ooool000cooooo1?l00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006
ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1?l0000B
ooooo`Co00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00007
ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol00000
0ooooooo0000ooooo`05ooooo`?o00004_ooool4o`0000Oooooo0ol000000ooooooo0000ooooo`05
ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol00000
0ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003o
oooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3
o`0000kooooo000;ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo1Ol00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol0000=ooooo`002_oooomZo`0000koooooJ_l0000>ooooof[o00003?ooool000[ooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo
000:ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
000<ooooo`002oooool6o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool6o`00013ooooo1_l00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool6o`000003ooooool0003o
oooo00;ooooo1_l000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5
o`000003ooooool0003ooooo00?ooooo1Ol0000@ooooo`Go000000?oooooo`000?ooool00_ooool6
o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?ooooo
o`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03
ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol00000
0ooooooo0000ooooo`03ooooo`Go00003Oooool000cooooo0ol00002ooooo`04o`000?oooooooooo
o`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?oo
ooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04
o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002
ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo
0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol0000Booooo`?o00000_ooool01?l0003o
ooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool0
1?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o0000
0_ooool01?l0003oooooooooool00002ooooo`?o00000oooool00ol0003oooooo`000002ooooo`?o
00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003o
oooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00004_ooool3o`0000?ooooo00?o
0000ooooool000000_ooool3o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo
00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`00
00?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000oooo
o`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo
0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000kooooo000=ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000D
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo0000
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000kooooo000=ooooo`04o`000?ooooooooooo`0000Co
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002oooo
o`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool0000Dooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooooooooooo00000_ooool00ol0
003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool0
1?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooooooo
ool00oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool0000Cooooo`04o`000?ooooooooooo`0000Go
oooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002oooo
o`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool0000>ooooo`003?ooool4o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool4o`0001;ooooo1?l0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Oooooo0ol0
00000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0
003ooooo00Gooooo0ol0000Booooo`Co00001oooool3o`000003ooooool0003ooooo00Gooooo0ol0
00000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0
003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Ooo
ool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o00003_oo
ool000_ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5
o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go00001_ooool5
o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5
o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5
o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5
o`0000gooooo000:ooooof[o00003_oooomZo`0000koooooJ_l0000<ooooo`002_ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000[ooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo
000;ooooo`Ko00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko00004?ooool6o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko000000?oooooo`000?ooool00_oo
ool6o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?o
ooooo`000?ooool00oooool5o`00013ooooo1Ol000000ooooooo0000ooooo`02ooooo`Ko000000?o
ooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000oooo
o`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol0
00000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0
003ooooo00?ooooo1Ol0000=ooooo`003?ooool3o`0000;ooooo00Co0000oooooooooooo00000_oo
ool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo
00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooo
oooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co
0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;o
oooo00Co0000oooooooooooo00000_ooool3o`0001;ooooo0ol00002ooooo`04o`000?oooooooooo
o`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?oo
ooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04
o`000?ooooooooooo`0000;ooooo0ol00003ooooo`03o`000?oooooo000000;ooooo0ol00003oooo
o`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0
0003ooooo`05o`000?ooooooooooo`000?ooool00ol0000Booooo`?o00000oooool00ol0003ooooo
o`000002ooooo`?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003o
ooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool0
1Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o0000
0oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo
00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00003_ooool000gooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000004oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000003_ooool000gooooo00Co0000oooooooooooo00001?ooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0001Co
oooo00Co0000oooooooooooo00001?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0001?ooooo00Co0000oooooooooooo00001Oooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000ko
oooo000<ooooo`Co00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Co00004_ooool4o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001oooool3o`000003oooo
ool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool0
1Oooool3o`0001;ooooo1?l00007ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003oooo
ool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool0
1Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o0000
00?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol0000>ooooo`002ooo
ool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_oo
ool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo1Ol00006ooooo`Go00001_oo
ool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_oo
ool5o`0000Kooooo1Ol0000@ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_oo
ool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00003Ooo
ool000[oooooJ_l0000>ooooof[o00003_oooomZo`0000cooooo000:ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<ooooo`002_ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo
1_l00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1_l0000@ooooo`Ko00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1_l000000ooooooo0000ooooo`02ooooo`Ko0000
00?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000
ooooo`03ooooo`Go00004?ooool5o`000003ooooool0003ooooo00;ooooo1_l000000ooooooo0000
ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo
1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003oooo
ool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool0
0oooool5o`0000gooooo000<ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o0000
0_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002oooo
o`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool0
0002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003ooooo
ooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0
003oooooooooool00002ooooo`?o00004_ooool3o`0000;ooooo00Co0000oooooooooooo00000_oo
ool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo
00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooo
oooooooo00000_ooool3o`0000?ooooo00?o0000ooooool000000_ooool3o`0000?ooooo00Go0000
oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo
00Go0000oooooooooooo0000ooooo`03o`0001;ooooo0ol00003ooooo`03o`000?oooooo000000;o
oooo0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oooooooooo
o`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oo
ooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05
o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003
ooooo`05o`000?ooooooooooo`000?ooool00ol0000>ooooo`003Oooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00?ooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooo
oooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooooooo
o`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`00000>ooooo`003Oooool01?l0003oooooooooool00004ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooo
ooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00005?ooool01?l0
003oooooooooool00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00?ooooo00Co0000
oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00004oooool01?l0003oooooooooool00005ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooo
ooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00003_ooool000co
oooo1?l00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1?l0000Booooo`Co00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00007ooooo`?o000000?oooooo`000?oo
ool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o
00004_ooool4o`0000Oooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?oo
ool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o
000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo
0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`0000kooooo000;ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`00013ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000=ooooo`002_oo
oomZo`0000koooooJ_l0000>ooooof[o00003?ooool000[ooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo000:ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<ooooo`002oooool6o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool6o`00013ooooo1_l00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool6o`000003ooooool0003ooooo00;ooooo1_l000000ooooooo
0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?o
oooo1Ol0000@ooooo`Go000000?oooooo`000?ooool00_ooool6o`000003ooooool0003ooooo00?o
oooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003
ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?oo
ool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go
00003Oooool000cooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04
o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002
ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo
0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`00
00;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?oooooo
ooooo`0000;ooooo0ol0000Booooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o0000
0_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002oooo
o`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool0
0002ooooo`?o00000oooool00ol0003oooooo`000002ooooo`?o00000oooool01Ol0003ooooooooo
ool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003o
ooooooooool0003ooooo00?o00004_ooool3o`0000?ooooo00?o0000ooooool000000_ooool3o`00
00?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000oooo
o`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo
0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooo
oooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go
0000oooooooooooo0000ooooo`03o`0000kooooo000=ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00cooooo000=ooooo`05o`000?ooooooooooooooool000000_ooool0
1Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooo
oooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooooooo
o`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`00000Dooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo
00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000kooooo000=ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0000Dooooo`04o`000?oooooo
ooooo`0000Cooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002oooo
o`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00oooool01?l0003ooooooooo
ool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool0000Cooooo`04o`000?ooooooooooo`0000Gooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000oooooooo
ool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0000>ooooo`003?ooool4o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool4o`0001;ooooo1?l00006ooooo`Go00001_ooool5o`00
00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Oooooo0ol000000ooooooo0000ooooo`05oooo
o`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol0000Boooo
o`Co00001oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05oooo
o`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooo
oooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo
00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o00003_ooool000_ooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00004?ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000gooooo000:ooooof[o0000
3_oooomZo`0000koooooJ_l0000<ooooo`002_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000[ooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo000;ooooo`Ko00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Ko00004?ooool6o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Ko000000?oooooo`000?ooool00_ooool6o`000003ooooool0003ooooo
00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`00
013ooooo1Ol000000ooooooo0000ooooo`02ooooo`Ko000000?oooooo`000?ooool00oooool5o`00
0003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`00
0?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03oooo
o`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol0000=oooo
o`003?ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooo
oooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co
0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;o
oooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3
o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo0000
0_ooool3o`0001;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04
o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002
ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo
0ol00003ooooo`03o`000?oooooo000000;ooooo0ol00003ooooo`05o`000?ooooooooooo`000?oo
ool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oooooooooo
o`000?ooool00ol0000Booooo`?o00000oooool00ol0003oooooo`000002ooooo`?o00000oooool0
1Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o0000
0oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo
00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0
003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003ooooo
ooooool0003ooooo00?o00003_ooool000gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool04Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool03?ooool000gooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000004oooool01Ol0003o
oooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool0
1Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooo
oooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooooooo
o`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
3_ooool000gooooo00Co0000oooooooooooo00001?ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0001Cooooo00Co0000oooooooooooo0000
1?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0001?ooooo00Co0000oooooooooooo00001Oooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000kooooo000<ooooo`?oool000?o0000
ooooooooool01?ooool01Ol0003oool0oooo0?ooo`3o000000Kooooo1Ol00006ooooo`Go00001_oo
ool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`04o`000?oo
o`3oool0oooo01;ooooo1?l00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Oooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool0
1Oooool3o`000003ooooool0003ooooo00Gooooo0ol0000Booooo`Co00001oooool3o`000003oooo
ool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool0
1Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o0000
00?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000
ooooo`05ooooo`?o00003_ooool000_ooooo1Oooo`06ooooo`Goool01_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ooo
o`0@ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000gooooo000:ooooo`Ooool01?l00007oooo057o00001oooo`0>
ooooof[o00003_oooomZo`0000cooooo000:ooooo`Ooool01?ooool7oooo00Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Ooool03_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol0000<ooooo`002_ooool7oooo00Cooooo1oooo`04ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7oooo00kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo1Oooo`000ol0003ooooo
ooooo`02ooooo`03o`000?ooo`3oool000?oool000?o0000ooooooooool00_ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
00?o0000oooo0?ooo`000oooo`0@ooooo`Ko00001?ooool7o`0000Cooooo1ol00004ooooo`Oo0000
1?ooool7o`0000Cooooo1_l000000ooooooo0000ooooo`02ooooo`Ko000000?oooooo`000?ooool0
0oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go0000
4?ooool5o`000003ooooool0003ooooo00;ooooo1_l000000ooooooo0000ooooo`03ooooo`Go0000
00?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000
ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo
1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`0000gooooo
000<ooooo`?oool00_ooool01?l0003oooooooooool00002ooooo`?oool00_ooool01?l0003ooooo
ooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0
003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_oo
ool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o
00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002
ooooo`?oool04_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co
0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;o
oooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3
o`0000?ooooo00?o0000ooooool000000_ooool3o`0000?ooooo00Go0000oooooooooooo0000oooo
o`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo
0000ooooo`03o`0001;ooooo0ol00003ooooo`03o`000?oooooo000000;ooooo0ol00003ooooo`05
o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003
ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool0
0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`00
0?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oooooo
ooooo`000?ooool00ol0000>ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`0Aooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`0<ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000Cooooo`05o`000?oo
ooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000>
ooooo`003Oooool01?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00005?ooool01?l0003oooooooooool00004
ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00002ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo00000_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00004oooool01?l0003oooooooooool00005ooooo`04o`000?ooooooooooo`0000;ooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00003_ooool000cooooo0oooo`000ol0003o
ooooooooo`04ooooo`05o`000?ooo`3oool0oooo0?l000001_ooool01Ol0003oool0oooo0?ooo`3o
000000Kooooo00Go0000oooo0?ooo`3oool0o`000006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo00Co0000oooo0?ooo`3oool04_ooool3oooo0003o`00
0?oooooooooo00Cooooo00Go0000oooo0?ooo`3oool0o`000006ooooo`Go00001_ooool5o`0000Ko
oooo1Ol00006ooooo`Go00001oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo
0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`0001;ooooo1?l00007ooooo`?o
000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo
0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Go
oooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003
ooooool0003ooooo00Gooooo0ol0000>ooooo`002oooool5oooo00Kooooo1Oooo`06ooooo`Goool0
1_ooool5oooo00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5oooo013ooooo1Oooo`06ooooo`Goool01_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00003Oooool000[ooooo1oooo`04o`0000Ooool01?l0
0007oooo00Co00001oooo`0ko`0000Ooool03_ooool7oooo00Co00001oooo`1Ho`0000koooooJ_l0
000<ooooo`002_ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7oooo00kooooo1oooo`04
ooooo`Ooool01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00003?ooool000[ooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1oooo`0>oooo
o`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000cooooo000;ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Go
ool01_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
ool04?ooool5oooo00Kooooo1Oooo`06ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5o`00013ooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1Ol0000=ooooo`003?ooool3oooo0003ooooool0003ooooo00?ooooo
00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool0
00?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo01;ooooo0oooo`000ooooooo0000
ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000
ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`0000;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`0000;ooooo00?o0000oooo
ooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000oooo
ooooool00_ooool4o`0001;ooooo0ol00002ooooo`06o`000?ooooooooooooooool0003ooooo0ol0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
000>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000Cooooo
00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000004oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00000oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000D
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo0000
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000kooooo000<ooooo`?oool000?oooooo`000?ooool0
0oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`00
0_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool0
0oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool04_ooool3oooo0003oooo
ool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`00
0?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003oooo
ool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool01?ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0001;ooooo0oooo`000ooooooo0000ooooo`04ooooo`Co00000_ooool00ol0003ooooo
ooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooo
ooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooo
ooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooo
ooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00003_ooool000_ooooo1Ooo
o`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Oooo`0@ooooo`Goool01_ooool5oooo00Kooooo1Ooo
o`06ooooo`Goool01_ooool5o`0000Kooooo1Ol00006ooooo`Ko00001Oooool6o`0000Gooooo1_l0
0005ooooo`Go00004?ooool5oooo00Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5o`0000gooooo000:
ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ooool03_ooool7oooo00Cooooo1oooo`04
ooooo`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000kooooo1oooo`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<
ooooo`002_ooool7oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0`o`00
00Ooool03_ooool7oooo00Co00001oooo`04o`0000Ooool01?l00007oooo04;o00003_ooool7oooo
06?o00003?ooool000[ooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
ool01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1oooo`0>ooooo`Oo
ool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7oooo00Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000cooooo000;ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool0
1_ooool5oooo00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Goool0
4?ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo1Oooo`06ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol0000=ooooo`003?ooool3oooo00Sooooo0oooo`08ooooo`?oool02?oo
ool3oooo00Sooooo0oooo`08ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?oo
ool3oooo01;ooooo0oooo`08ooooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?o00002?oo
ool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol0000Booooo`?oool02?oo
ool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?oo
ool3o`0000Sooooo0ol00008ooooo`?o00003_ooool00?ooooooLOooool00?ooooooLOooool00?oo
ooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?oo
ooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?oo
ooooLOooool00?ooooooLOooool000cooooo0ol0ool8ooooo`?o0?l02?ooool3o`3o00Sooooo0ol0
o`08ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol0
000Booooo`?o0?oo2?ooool3o`3oo`Sooooo0ol0o`08ooooo`?o00002?ooool3o`0000Sooooo0ol0
0008ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o00004_ooool3o`3oo`Sooooo0ol0
ool8ooooo`?o0?oo2?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000Sooooo0ol0
0008ooooo`?o00002?ooool3o`0000kooooo000;ooooo`Go0?oo1_ooool5o`3o00Kooooo1Ol0o`06
ooooo`Go0?l01_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006
ooooo`Go00004?ooool5o`3oo`Kooooo1Ol0ool6ooooo`Go0?l01_ooool5o`0000Kooooo1Ol00006
ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo1Ol0ool6
ooooo`Go0?oo1_ooool5o`3oo`Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006
ooooo`Go00001_ooool5o`0000Kooooo1Ol0000=ooooo`002_ooool7o`3oo`Co00001ol0o`04o`00
00Oo0?l01?l00007o`3o04;o00003_ooool7o`3oo`Co00001ol0ool4o`0000Oo0?l0COl0000>oooo
o`Oo0?oo1?l00007o`3oo`Co00001ol0oom=o`0000cooooo000:ooooo`Oo0?oo1?ooool7o`3o00Co
oooo1ol0o`04ooooo`Oo0?l01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00003_ooool7o`3oo`Cooooo1ol0ool4ooooo`Oo0?l01?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000ko
oooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3oo`Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<ooooo`002_ooool7o`3oo`Cooooo
1ol0o`04ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3o00Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol0000>ooooo`Oo0?oo1?ooool7o`3oo`Cooooo1ol0ool4ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo1Ol0
ool00ol0003oooooooooo`02ooooo`03o`000?l0o`3o0?l000?o0?l000?o0000ooooooooool00_oo
ool00ol0003o0?l0o`3o0003o`3o0003o`000?oooooooooo00;ooooo00?o0000o`3o0?l0o`000ol0
o`000ol0003oooooooooo`02ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1_l0000@ooooo`Go0?oo00?o0000ooooooooool00_ooool00ol0003o0?ooo`3o
o`03o`3oo`03o`000?oooooooooo00;ooooo00?o0000o`3o0?l0o`000ol0o`000ol0003ooooooooo
o`02ooooo`Oo00001?ooool7o`0000Cooooo1_l000000ooooooo0000ooooo`02ooooo`Ko000000?o
ooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000oooo
o`03ooooo`Go00004?ooool5o`3oo`03ooooool0003ooooo00;ooooo00?o0000o`3oool0ool00ol0
ool00ooooooo0000ooooo`03ooooo`Go0?oo00?oooooo`000?ooool00oooool5o`000003ooooool0
003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00ooo
ool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?o
ooooo`000?ooool00oooool5o`0000gooooo000<ooooo`?o0?oo0_ooool01?l0003oooooooooool0
0002ooooo`?o0?l00_ooool01?l0003oooooooooool00002ooooo`?o0?l00_ooool01?l0003ooooo
ooooool00002ooooo`?o0?l00_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0
003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_oo
ool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o
00000_ooool01?l0003oooooooooool00002ooooo`?o00004_ooool3o`3oo`;ooooo00Co0000oooo
oooooooo00000_ooool3o`3oo`;ooooo00Co0000oooooooooooo00000_ooool3o`3o00;ooooo00Co
0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;o
oooo00Co0000oooooooooooo00000_ooool3o`0000?ooooo00?o0000ooooool000000_ooool3o`00
00?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000oooo
o`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0001;ooooo0ol0ool3ooooo`03o`00
0?oooooo000000;ooooo0ol0ool3ooooo`05o`000?ooooooooooo`000?ooool00ol0ool3ooooo`05
o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003
ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool0
0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`00
0?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0000>ooooo`003Oooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00;o
oooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003o
ooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool01Ol0003ooooo
oooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooooooo
o`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000005?oo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`00000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`00000>ooooo`003Oooool01?l0003oooooooooool00004oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00005?ooool01?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00?ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00004oooool01?l0003oooooooooool00005oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00003_ooool000cooooo0ol0ool00ol0003oooooooooo`04ooooo`05o`000?l0o`3o0?l0o`3o0?l0
00001_ooool01Ol0003o0?l0o`3o0?l0o`3o000000Kooooo00Go0000o`3o0?l0o`3o0?l0o`000006
ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1?l0000B
ooooo`?o0?oo00?o0000ooooooooool01?ooool01Ol0003o0?ooo`3oool0oooo000000Kooooo00Go
0000o`3o0?l0o`3o0?l0o`000006ooooo`05o`000?l0o`3o0?l0o`3o0?l000001_ooool5o`0000Ko
oooo1Ol00007ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Go
oooo0ol000000ooooooo0000ooooo`05ooooo`?o00004_ooool4o`0000Oooooo0ol000000ooooooo
0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Go
oooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003
ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?oo
ool01Oooool3o`0000kooooo000;ooooo`Go0?oo1_ooool5o`3o00Kooooo1Ol0o`06ooooo`Go0?l0
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
4?ooool5o`3oo`Kooooo1Ol0ool6ooooo`Go0?l01_ooool5o`3o00Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol0000=ooooo`002_ooool7o`3oo`Co00001ol0o`04o`0000Oo0?l01?l0
0007o`3o04;o00003_ooool7o`3oo`Co00001ol0ool4o`0000Oo0?l01?l00007o`3o04;o00003_oo
oomZo`0000cooooo000:ooooo`Oo0?oo1?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo0?l01?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7
o`3oo`Cooooo1ol0ool4ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7
o`0000Cooooo1ol0000<ooooo`002_ooool7o`3oo`Cooooo1ol0o`04ooooo`Oo0?l01?ooool7o`3o
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00kooooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo1Ol0ool00ol0003oooooooooo`02ooooo`03
o`000?l0o`3o0?l000?o0?l000?o0000ooooooooool00_ooool00ol0003o0?l0o`3o0003o`3o0003
o`000?oooooooooo00;ooooo00?o0000o`3o0?l0o`000ol0o`000ol0003oooooooooo`02ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1_l0000@ooooo`Go
0?oo00?o0000ooooooooool00_ooool00ol0003o0?ooo`3oo`03o`3oo`03o`000?oooooooooo00;o
oooo00?o0000o`3o0?l0o`000ol0o`000ol0003oooooooooo`02ooooo`03o`000?l0o`3o0?l000?o
0?l000?o0000ooooooooool00_ooool7o`0000Cooooo1_l000000ooooooo0000ooooo`02ooooo`Ko
000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo
0000ooooo`03ooooo`Go00004?ooool5o`000003ooooool0003ooooo00;ooooo1_l000000ooooooo
0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?o
oooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003
ooooool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?oo
ool00oooool5o`0000gooooo000<ooooo`?o0?oo0_ooool01?l0003oooooooooool00002ooooo`?o
0?l00_ooool01?l0003oooooooooool00002ooooo`?o0?l00_ooool01?l0003oooooooooool00002
ooooo`?o0?l00_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003ooooooooo
ool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool01?l0003o
ooooooooool00002ooooo`?o00000_ooool01?l0003oooooooooool00002ooooo`?o00000_ooool0
1?l0003oooooooooool00002ooooo`?o00004_ooool3o`3oo`;ooooo00Co0000oooooooooooo0000
0_ooool3o`3oo`;ooooo00Co0000oooooooooooo00000_ooool3o`3o00;ooooo00Co0000oooooooo
oooo00000_ooool3o`3o00;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000
oooooooooooo00000_ooool3o`0000?ooooo00?o0000ooooool000000_ooool3o`0000?ooooo00Go
0000oooooooooooo0000ooooo`03o`0000?ooooo00Go0000oooooooooooo0000ooooo`03o`0000?o
oooo00Go0000oooooooooooo0000ooooo`03o`0001;ooooo0ol00003ooooo`03o`000?oooooo0000
00;ooooo0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oooooo
ooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`00
0?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003oooo
o`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0
0003ooooo`05o`000?ooooooooooo`000?ooool00ol0000>ooooo`003Oooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00;ooooo00?o0000
ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo
00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool0
1Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`00000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
0003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`00000>ooooo`003Oooool01?l0003oooooooooool00004ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00005?ooool0
1?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00?ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00004oooool01?l0003oooooooooool00005ooooo`04o`000?oo
ooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00003_ooool0
00cooooo0ol0ool00ol0003oooooooooo`04ooooo`05o`000?l0o`3o0?l0o`3o0?l000001_ooool0
1Ol0003o0?l0o`3o0?l0o`3o000000Kooooo00Go0000o`3o0?l0o`3o0?l0o`000006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1?l0000Booooo`Co0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00007ooooo`?o0000
00?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000
ooooo`05ooooo`?o00004_ooool4o`0000Oooooo0ol000000ooooooo0000ooooo`05ooooo`?o0000
00?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000
ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo
0ol000000ooooooo0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`0000kooooo
000;ooooo`Go0?oo1_ooool5o`3o00Kooooo1Ol0o`06ooooo`Go0?l01_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00004?ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`00013ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
000=ooooo`002_ooool7o`3oo`Co00001ol0o`04o`0000Oo0?l01?l00007o`3o04;o00003_oooomZ
o`0000koooooJ_l0000<ooooo`002_ooool7o`3oo`Cooooo1ol0o`04ooooo`Oo0?l01?ooool7o`3o
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00003?ooool000[ooooo1ol0ool4ooooo`Oo0?l01?ooool7o`3o00Co
oooo1ol0o`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo000;ooooo`Go0?oo00?o0000ooooooooool0
0_ooool00ol0003o0?l0o`3o0003o`3o0003o`000?oooooooooo00;ooooo00?o0000o`3o0?l0o`00
0ol0o`000ol0003oooooooooo`02ooooo`03o`000?l0o`3o0?l000?o0?l000?o0000ooooooooool0
0_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko0000
4?ooool6o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ko0000
00?oooooo`000?ooool00_ooool6o`000003ooooool0003ooooo00?ooooo1Ol000000ooooooo0000
ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`00013ooooo1Ol000000ooooooo0000
ooooo`02ooooo`Ko000000?oooooo`000?ooool00oooool5o`000003ooooool0003ooooo00?ooooo
1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool00oooool5o`000003oooo
ool0003ooooo00?ooooo1Ol000000ooooooo0000ooooo`03ooooo`Go000000?oooooo`000?ooool0
0oooool5o`000003ooooool0003ooooo00?ooooo1Ol0000=ooooo`003?ooool3o`3oo`;ooooo00Co
0000oooooooooooo00000_ooool3o`3o00;ooooo00Co0000oooooooooooo00000_ooool3o`3o00;o
oooo00Co0000oooooooooooo00000_ooool3o`3o00;ooooo00Co0000oooooooooooo00000_ooool3
o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo0000
0_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0000;ooooo00Co0000oooooooo
oooo00000_ooool3o`0000;ooooo00Co0000oooooooooooo00000_ooool3o`0001;ooooo0ol00002
ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo
0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00002ooooo`04o`000?ooooooooooo`00
00;ooooo0ol00002ooooo`04o`000?ooooooooooo`0000;ooooo0ol00003ooooo`03o`000?oooooo
000000;ooooo0ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol00003ooooo`05o`000?oo
ooooooooo`000?ooool00ol00003ooooo`05o`000?ooooooooooo`000?ooool00ol0000Booooo`?o
00000oooool00ol0003oooooo`000002ooooo`?o00000oooool01Ol0003oooooooooool0003ooooo
00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0
003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0003ooooo
ooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00000oooool01Ol0
003oooooooooool0003ooooo00?o00000oooool01Ol0003oooooooooool0003ooooo00?o00003_oo
ool000gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo0000
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000go
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo
00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000001Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooo
ooooooooool000000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000004oooool01Ol0003ooooooooooooooooo000000?ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000003_ooool000gooooo00Co0000oooo
oooooooo00001?ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0001Cooooo00Co0000oooooooooooo00001?ooool01?l0003oooooooooool0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o
ooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0001?ooooo00Co0000oooo
oooooooo00001Oooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;o
oooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oooooo
ooooo`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooooooooooo`0000kooooo000<ooooo`Co00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Co
00004_ooool4o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o
000000?oooooo`000?ooool01Oooool3o`0001;ooooo1?l00007ooooo`?o000000?oooooo`000?oo
ool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo0000ooooo`05ooooo`?o
000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Gooooo0ol000000ooooooo
0000ooooo`05ooooo`?o000000?oooooo`000?ooool01Oooool3o`000003ooooool0003ooooo00Go
oooo0ol0000>ooooo`002oooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`00013ooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00003Oooool000[oooooJ_l0000>ooooof[o00003_oooomZo`0000cooooo000:
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<
ooooo`002_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00003?ooool000_ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000@ooooo`Go
00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Go00004?ooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool5o`0000gooooo000<ooooo`?o000000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000
ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000
ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000
ooooo`03ooooo`03o`000?oooooo000000;o00004_ooool3o`000003ooooool0003ooooo00?ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
0_ooool01_l0003ooooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02oooo
o`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02oooo
o`Co00004_ooool3o`0000;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`0000;ooooo00?o
0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o
0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o
0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o
0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000kooooo000=
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003ooooo
o`000004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
1?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003oooo
o`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000004oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000003_ooool000cooooo0ol000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003oooo
ool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003oooo
ool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003oooo
ool0003ooooo00?ooooo00?o0000oooooooo00000_oo000Booooo`?o000000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`04ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co0000
0_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co0000
4_ooool3o`000003ooooool0003ooooo00Cooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0000>ooooo`002oooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5ool0013ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1Ol0
000@ooooo`Go00001_ooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Go00003Oooool000[ooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ooo000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004
ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000cooooo000:oooo
of?o00001ooo000>ooooof[o00003_oooomZo`0000cooooo000:ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Ooo`003_ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000ko
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000<ooooo`002oooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool5ool0013ooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1Ol00006ooooo`Go00001_ooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo
1Ol0000@ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo
1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Go00003Oooool000cooooo0ol0
00000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00ooo
ool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00ooo
ool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00ooo
ool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000oooooooo00000_oo
000Booooo`?o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0
003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oo
oooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0
003ooooo00?ooooo00?o0000ooooool000000_l00002ooooo`06o`000?ooooooooooooooool0003o
oooo0ol00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0000Booooo`?o00000_ooool01_l0003o
oooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00003_ooool000gooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
0003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooo000000Cooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0<ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`00000Cooooo`05o`000?oooooooooooooo
ool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000>ooooo`003?oo
ool3oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000oooo
o`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00
0002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00
0002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooool0
0002ool001;ooooo0ol000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?o
ooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o
0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?o
ooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00Cooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l0000Booooo`?o000000?oooooo`000?ooool01?oo
ool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_oo
ool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_oo
ool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_oo
ool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_oo
ool4o`0000kooooo000;ooooo`Goool01_ooool5oooo00Kooooo1Ol00006ooooo`Go00001_ooool5
o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Goo`004?ooool5
o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool6
o`0000Gooooo1_l00005ooooo`Ko00001Oooool5o`00013ooooo1Ol00006ooooo`Ko00001Oooool6
o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6
o`0000Gooooo1Ol0000=ooooo`002_ooool7oooo00Cooooo1oooo`04ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7ool0
00kooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0000>ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00003?ooool000[ooooo1oooo`04o`0000Ooool0DOl00007ool000ko
ooooJ_l0000>ooooof[o00003?ooool000[ooooo1oooo`04ooooo`Ooool01?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ooo000>ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003_ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo
1ol00004ooooo`Oo00001?ooool7o`0000cooooo000;ooooo`Goool01_ooool5oooo00Kooooo1Ol0
0006ooooo`Go00001_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Goo`004?ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0
0006ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5o`00013ooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1Ol0000=ooooo`003?ooool3oooo0003ooooool0003o
oooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo
000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003o
oooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo
000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003o
oooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo
000000;o000000?oooooo`000?ooool00oooool00ol0003oooooool00002ool001;ooooo0ol00000
0ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool0
0ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l00000
0ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool0
0ol0003oooooo`000002o`0000;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`0000;ooooo
00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo
00?o0000ooooooooool00_ooool4o`0001;ooooo0ol00002ooooo`06o`000?ooooooooooooooool0
003ooooo0ol00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l0000>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000Cooooo00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000004oooool00ol0003oooooooooo`02
ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000
ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004
ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000
oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000000kooooo000<ooooo`?oool000?ooooo
o`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000
ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?ooooo
o`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?ooooo
o`000?ooool00oooool00ol0003oooooool00002ool00003ooooool0003ooooo00?ooooo00?o0000
ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooo`0000;oo`004_ooool3
oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03
ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002
oooo0003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03
ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool01?ooool4o`0000;ooooo00?o0000
ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000
ooooooooool00_ooool4o`0001;ooooo0oooo`000ooooooo0000ooooo`04ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool0
0ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00003_ooool0
00_ooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5o`00
00Kooooo1Ol00006ooooo`Goo`001_ooool5oooo00Kooooo1Ooo000@ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5o`0000Kooooo1Ol00006ooooo`Ko00001Oooool6o`00
00Gooooo1_l00005ooooo`Go00004?ooool5oooo00Kooooo1_l00005ooooo`Ko00001Oooool6o`00
00Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5o`00
00gooooo000:ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Co
oooo1ol00004ooooo`Oo00001?ooool7ool000Cooooo1oooo`04ooooo`Ooo`003_ooool7oooo00Co
oooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000kooooo1oooo`04ooooo`Oo00001?ooool7o`0000Co
oooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Co
oooo1ol0000<ooooo`002_ooool7oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co0000
1oooo`0Jo`0000Ooo`001?l00007oooo00Co00001ooo000>ooooo`Ooool01?l00007oooo00Co0000
1oooo`04o`0000Ooool0@_l0000>ooooo`Ooool0Hol0000<ooooo`002_ooool7oooo00Cooooo1ooo
o`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo00001?ooool7o`0000Cooooo1ooo
0004ooooo`Ooool01?ooool7ool000kooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ooo
o`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
000>ooooo`Ooool01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo1Oooo`06
ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5o`0000Kooooo1Ol00006
ooooo`Goo`001_ooool5oooo00Kooooo1Ooo000@ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06
ooooo`Goool01_ooool5o`0000Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005
ooooo`Go00004?ooool5oooo0003o`000?oooooooooo00?ooooo1_l00005ooooo`Ko00001Oooool6
o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5
o`0000gooooo000<ooooo`?oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo
0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03oooo
o`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo
0003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03oooo
o`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooool00002ool0
0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03oooo
o`03o`000?ooooooo`0000;oo`004_ooool3oooo0003ooooool0003ooooo00?ooooo00?o0000oooo
ooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`00
0?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000oooo
ool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o00000_ooool01_l0
003ooooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`Co00000_oo
ool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00004_oo
ool3oooo00;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`0000;ooooo00?o0000oooooooo
ool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000oooooooo
ool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000oooooooo
ool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000oooooooo
ool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000kooooo000=ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003oooooo`000004oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo0000
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000004ooo
ool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000003_ooool000cooooo0oooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool0
00;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo
00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool0
00;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo
00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool0
00;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo
00?ooooo00?o0000ooooooooo`000_ooo`0Booooo`?oool000?oooooo`000?ooool00oooool00ol0
003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooo
oooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0
003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooo
oooo0000ooooo`04ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0
003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool0
1?l0003oool0oooo0?ooo`0Booooo`?oool000?oooooo`000?ooool01?ooool01?l0003oool0oooo
0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool4o`00
00;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`00
00;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool01?l0
003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool03_oo
ool000_ooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5
oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`0@ooooo`Goool01_ooool5
oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Ol00006ooooo`Ko00001Oooool6
o`0000Gooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool04?ooool5oooo00Kooooo1Oooo`00
0ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooo
oooo00?ooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Goool000?o0000
ooooooooool00oooool5oooo00gooooo000:ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04oooo
o`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04oooo
o`Ooool03_ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ooool01?ooool7oooo00kooooo1oooo`04oooo
o`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7oooo00Cooooo1oooo`0<ooooo`002_ooool7oooo00Co00001oooo`04o`0000Oo
ool01?l00007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Oo
ool01?l00007oooo00kooooo1oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Oo
ool09Ol00007oooo00Co00001oooo`0>ooooo`Ooool01?l00007oooo00Co00001oooo`04o`0000Oo
ool0<?l00007oooo00Co00001oooo`0<ooooo`002_ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00kooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7oooo00Cooooo1oooo`0>ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7o`0000Cooooo1ol00004ooooo`Oo0000
1?ooool7o`0000Cooooo1oooo`04ooooo`Ooool03?ooool000_ooooo1Oooo`06ooooo`Goool01_oo
ool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_oo
ool5oooo00Kooooo1Oooo`0@ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_oo
ool5oooo00Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool04?ooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Goool000?o0000ooooooooool00ooo
ool5oooo00gooooo000<ooooo`?oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002
oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03
ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002
oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03
ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002
oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03
ooooo`03o`000?ooooooool000;oool04_ooool3oooo0003ooooool0003ooooo00?ooooo00?o0000
ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?ooooo
o`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000
ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o00000_ooool0
1_l0003ooooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`Co0000
0_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000oooooooo
ool00_ooool01?l0003oool0oooo0?ooo`0Booooo`?oool00_ooool01_l0003ooooooooooooooooo
0000ooooo`?oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo
00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0
oooo00kooooo000=ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000001?oo
ool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
01Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool0
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000004oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000003_ooool000cooooo0oooo`000ooooooo0000oooo
o`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo
0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000oooo
o`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo
0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000oooo
o`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo
0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`0Booooo`?oool000?o
ooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o
0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?o
ooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o
0000ooooooooo`000_ooo`000ooooooo0000ooooo`04ooooo`04o`000?ooo`3oool0oooo00;ooooo
00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0
oooo01;ooooo0oooo`000ooooooo0000ooooo`04ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o
0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003o
ooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003o
ooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo
00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`0>ooooo`002oooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo013ooooo1Oooo`06ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`000ol0003oooooooooo`03oooo
o`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`0@oooo
o`Goool01_ooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03oooo
o`Goool000?o0000ooooooooool00oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`00
00Gooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool03Oooool000[ooooo1oooo`04ooooo`Oo
ool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
ool01?ooool7oooo00Cooooo1oooo`0>ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
ool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
ool03_ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Oo
00001?ooool7o`0000Cooooo1ol00004ooooo`Ooool01?ooool7oooo00cooooo000:ooooo`Ooool0
1?l00007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool0
1?l00007oooo00Co00001oooo`04o`0000Ooool03_ooool7oooo00Co00001oooo`04o`0000Ooool0
1?l00007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool0
1?l00007oooo00kooooo1oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0`o`0000Ooool0
1?l00007oooo00cooooo000:ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool03_oo
ool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00kooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1oooo`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7oooo00Cooooo1oooo`0<ooooo`002oooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5
oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5
oooo013ooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5
oooo0003o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000
ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`0@ooooo`Goool000?o0000
ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03
ooooo`Goool000?o0000ooooooooool00oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6
o`0000Gooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool03Oooool000cooooo0oooo`000ooo
oooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0
003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooo
oooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0
003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooo
oooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0
003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`0Boooo
o`?oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo
00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool0
00;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo
00?ooooo00?o0000ooooooooo`000_ooo`02ooooo`06o`000?ooooooooooooooool0003ooooo0ooo
o`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003o
ool0oooo0?ooo`0Booooo`?oool00_ooool01_l0003ooooooooooooooooo0000ooooo`?oool00_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03
o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03
o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?oo
o`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00kooooo000=oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000
oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003oooooo`00
0004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?oo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool0
0_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03o`000?oo
oooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_oo
ool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000
oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00004oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo0000
0oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003oooo
o`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000003_ooool000cooooo0oooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
ooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
ooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
ooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`0Booooo`?oool000?oooooo`000?ooool00ooo
ool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_oo
o`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00ooo
ool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_oo
o`000ooooooo0000ooooo`04ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?oo
o`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo01;ooooo0oooo`00
0ooooooo0000ooooo`04ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_oo
ool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3o
ool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000oooo
ooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000
oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo
00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooo0?ooo`3oool03_ooool000_ooooo1Oooo`06ooooo`Goool01_ooool5oooo00Ko
oooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Ko
oooo1Oooo`0@ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Ko
oooo1Oooo`06ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?o
oooo1Oooo`000ol0003oooooooooo`03ooooo`Goool04?ooool5oooo00Kooooo1Oooo`000ol0003o
ooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?o
oooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003
o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000oooooooo
ool00oooool5oooo00gooooo000:ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
3_ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00kooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool0
1?ooool7oooo00Cooooo1oooo`0<ooooo`002_ooool7oooo00Co00001oooo`04o`0000Ooool01?l0
0007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool01?l0
0007oooo00kooooo1oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool01?l0
0007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0>ooooo`Ooool01?l0
0007oooo00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool01?l0
0007oooo00Co00001oooo`04o`0000Ooool03?ooool000[ooooo1oooo`04ooooo`Ooool01?ooool7
oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7
oooo00Cooooo1oooo`0>ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7
oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool03_ooool7
oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7
oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00cooooo000;ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool04?ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo
013ooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo
00Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`0=ooooo`003?ooool3oooo00So
oooo0oooo`08ooooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?oool02?ooool3oooo00So
oooo0oooo`08ooooo`?oool02?ooool3oooo01;ooooo0oooo`08ooooo`?oool02?ooool3oooo00So
oooo0oooo`08ooooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?oool02?ooool3oooo00So
oooo0oooo`0Booooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?oool02?ooool3oooo00So
oooo0oooo`08ooooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?oool03_ooool00?oooooo
LOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?oooooo
LOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?oooooo
LOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool000cooooo0ol00008ooooo`?o
0?oo2?ooool3o`0000Sooooo0ol0o`08ooooo`?o0?l02?ooool3o`00o`Sooooo0ooo0008ooooo`?o
o`3o2?ooool3ool0o`Sooooo0oooo`0Booooo`?o0?oo2?ooool3o`3oo`Sooooo0ol0ool8ooooo`?o
0?l02?ooool3o`0000Sooooo0ol00?l8ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?o
003o4_ooool3o`3oo`Sooooo0ol0ool8ooooo`?o0?l02?ooool3o`0000Sooooo0ol00008ooooo`?o
00002?ooool3o`0000Sooooo0ol00008ooooo`?o00002?ooool3o`0000kooooo000;ooooo`Go0000
1_ooool5o`3oo`Kooooo1Ol00006ooooo`Go0?l01_ooool5o`3o00Kooooo1Ol00?l6ooooo`Goo`00
1_ooool5ool0o`Kooooo1Ooo0?l6ooooo`Goool04?ooool5o`3oo`Kooooo1Ol0ool6ooooo`Go0?oo
1_ooool5o`3o00Kooooo1Ol00006ooooo`Go003o1_ooool5o`0000Kooooo1Ol00006ooooo`Go0000
1_ooool5o`00oa3ooooo1Ol0ool6ooooo`Go0?oo1_ooool5o`3o00Kooooo1Ol00006ooooo`Go0000
1_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Ol0000=ooooo`002_oo
ool7o`0000Cooooo1ol0ool4ooooo`Oo00001?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo003o1?oo
ool7ool000Cooooo1ooo0?l4ooooo`Ooo`3o1?ooool7oooo00kooooo1ol0ool4ooooo`Oo0?oo1?oo
ool7o`3oo`Cooooo1ol0o`04ooooo`Oo00001?ooool7o`00o`Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00?l>ooooo`Oo0?oo1?ooool7o`3oo`Cooooo1ol0o`04ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?oo
ool000[ooooo2ol00007o`3oo`oo00001ol0o`04o`0000Oo0?l01?l00007o`00o`Co00001ooo0004
o`0000Ooo`3o1?l00007ool0o`Co00001oooo`0>ooooo`Oo0?oo1?l00007o`3oo`Co00001ol0ool4
o`0000Oo0?l03ol00007o`00obGo00001ol00?l>ooooo`Oo0?oo1?l00007o`3oo`Co00001ol0o`1=
o`0000cooooo000:ooooo`Oo00001?ooool7o`3oo`Cooooo1ol00004ooooo`Oo0?l01?ooool7o`3o
00Cooooo1ol00?l4ooooo`Ooo`001?ooool7ool0o`Cooooo1ooo0?l4ooooo`Ooool03_ooool7o`3o
o`Cooooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3o00Cooooo1ol00004ooooo`Oo003o1?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`00o`kooooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3o
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol0000<ooooo`002oooool5o`0000Kooooo1Ol0ool6ooooo`Go00001_ooool5o`3o00Ko
oooo1Ol0o`06ooooo`Go003o1_ooool5ool000Kooooo1Ooo0?l6ooooo`Goo`3o1_ooool5oooo013o
oooo1Ol0ool6ooooo`Go0?oo1_ooool5o`3oo`Kooooo1Ol0o`06ooooo`Go00001_ooool5o`00o`03
o`000?oooooooooo00?ooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1Ol00?l@ooooo`Go
0?oo00?o0000ooooooooool00oooool5o`3oo`03o`000?oooooooooo00?ooooo1Ol0o`000ol0003o
ooooooooo`03ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Go
oooo1_l00005ooooo`Go00003Oooool000cooooo0ol000000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo0?oo00;o0?oo00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003oooo
ool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo0?l000;o0?l000?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03oooo
ool0003ooooo00?ooooo00?o0000oooooooo00000_oo00000ooooooo0000ooooo`03ooooo`03o`00
0?ooooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03oooo
ool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`0Booooo`?o0?oo00?oooooo`000?ooool0
0oooool00ol0003oooooo`3oo`02o`3oo`03ooooool0003ooooo00?ooooo00?o0000ooooool0ool0
0_l0ool00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00?l0
0_l00?l2ooooo`06o`000?ooooooooooooooool0003ooooo0ol00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo00Co0000o`00ool00?oo003o4_ooool3o`3oo`;ooooo00Ko0000ooooooooooooooooo`00
0?ooool3o`3oo`;ooooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o0?l0o`02ooooo`03
o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03
o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03
o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03
o`000?oooooooooo00;ooooo1?l0000>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooo
ooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000Cooooo00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo
00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000004oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo01;ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2
o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo017ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
01;ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`00
00Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0
003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo
00cooooo000=ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000kooooo000<oooo
o`?oo`0000?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo
00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool0
00;oool000?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03ooooool0003ooooo
00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo003o
00;o003o00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo
00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?ooooooo`3o
00;oo`3o4_ooool3o`3o0003ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooo
oooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo00?oooooo`000?ooool00oooool00ol0
003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooo
oooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool01?ooool4o`00
00;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`00
00;ooooo00?o0000ooooooooool00_ooool01?l0003oc<c<olc<c?o<c<`Booooo`?o0?oo00?ooooo
o`000?ooool01?ooool4o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o0?l0
o`02ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo1?l0000>ooooo`002oooool5ool000Kooooo1Ol00006
ooooo`Go0?l01_ooool5oooo00Kooooo1Ooo0?l6ooooo`Go00001_ooool5o`00o`Kooooo1Ol00006
ooooo`Go003o1_ooool5ool0oa3ooooo1Ol0o`06ooooo`Go0?l01_ooool5o`3oo`Kooooo1Ol00006
ooooo`Go00001_ooool5o`0000Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1Oo<c<`@
ooooo`Go0?oo1_ooool6o`0000Gooooo1Ol0o`000ol0003oooooooooo`03ooooo`Ko00001Oooool6
o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Go00003Oooool0
00[ooooo1ooo0004ooooo`Oo00001?ooool7o`3o00Cooooo1oooo`04ooooo`Ooo`3o1?ooool7o`00
00Cooooo1ol00?l4ooooo`Oo00001?ooool7o`00o`Cooooo1ooo0?l>ooooo`Oo0?l01?ooool7o`3o
00Cooooo1ol0ool4ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Ooc<c<3_ooool7o`3oo`Cooooo1ol00004ooooo`Oo0?l01?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00cooooo000:ooooo`Ooo`003ol00007o`3o00Co00001oooo`04o`0000Ooo`3o3ol00007o`00o`oo
00001ol00?l4o`0000Ooo`3o3_ooool7o`3o00Co00001ol0o`04o`0000Oo0?ooA_l00007olc<c0ko
oooo1ol0ool?o`0000Oo0?l0COl0000<ooooo`002_ooool7ool000Cooooo1ol00004ooooo`Oo0?l0
1?ooool7oooo00Cooooo1ooo0?l4ooooo`Oo00001?ooool7o`00o`Cooooo1ol00004ooooo`Oo003o
1?ooool7ool0o`kooooo1ol0o`04ooooo`Oo0?l01?ooool7o`3oo`Cooooo1ol00004ooooo`Oo0000
1?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1oo<c<`>ooooo`Oo0?oo
1?ooool7o`0000Cooooo1ol0o`04ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo0000
1?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_ooooo1Ooo0006ooooo`Go00001_oo
ool5o`3o00Kooooo1Oooo`06ooooo`Goo`3o1_ooool5o`0000Kooooo1Ol00?l6ooooo`Go00001_oo
ool5o`00o`Kooooo1Ooo0?l@ooooo`Go0?l01_ooool5o`3o00Kooooo1Ol0ool6ooooo`Go00001_oo
ool5o`0000Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Goc<c<4?oo
ool5o`3oo`03o`000?oooooooooo00?ooooo1_l00005ooooo`Go0?l000?o0000ooooooooool00ooo
ool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Ooo
ool5o`0000gooooo000<ooooo`?oo`0000?oooooo`000?ooool00oooool00ol0003oooooo`000002
o`000003ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03
ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003oooooool0o`02
ool0o`03ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03
ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003oooooo`000002
o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03
ooooo`03o`000?ooooooo`3o00;oo`3o4_ooool3o`3o0003ooooool0003ooooo00?ooooo00?o0000
ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo00?ooooo
o`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o00000_ooool0
1_l0003ooooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`Co0000
0_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`04o`00
0?o<c<coc<c<olc<c1;ooooo0ol0ool2ooooo`06o`000?ooooooooooooooool0003ooooo0ol00002
ooooo`03o`000?oooooooooo00;ooooo00Co0000o`3o0?l0o`3o0?l00_ooool00ol0003ooooooooo
o`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`Co00003_ooool000gooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000004ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo00?o0000ooooooooool0
0_ooool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`00
3Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`00
0?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool00000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`00
0?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o0000
1?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooo
ooooooooo`000003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`00000Cooooo`05o`000?ooooooooooooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooo
oooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooooooo
o`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`00000>ooooo`003?ooool3o`3o0003oooo
ool0003ooooo00?ooooo00?o0000ooooooo<c<`00_o<c<`00ooooooo0000ooooo`03ooooo`03o`00
0?ooooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooool00002ool00003oooo
ool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`00
0?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03oooo
ool0003ooooo00?ooooo00?o0000oooooooo0?l00_oo0?l00ooooooo0000ooooo`03ooooo`03o`00
0?ooooooc<c<00;oc<c<00?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo01;ooooo
0ol0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0
0oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00Cooooo1?l00002ooooo`03o`00
0?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`00
0?oooooooooo00;ooooo00Co0000olc<c?o<c<coc<c<4_ooool3o`3o0003ooooool0003ooooo00Co
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo1?l0000>ooooo`002oooool5o`3o00Kooooo1Oo<c<`6ooooo`Goo`3o1_ooool5ool000Kooooo
1Ol00006ooooo`Goool01_ooool5o`00o`Kooooo1Ooo0?l6ooooo`Goc<c<1_ooool5oooo013ooooo
1Ol0o`06ooooo`Go0?l01_ooool5o`0000Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo
1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1Oo<c<`@ooooo`Go0?l01_ooool6o`0000Gooooo
1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo
1_l00005ooooo`Go00003Oooool000[ooooo1ol0o`04ooooo`Ooc<c<1?ooool7ool0o`Cooooo1ooo
0004ooooo`Oo00001?ooool7oooo00Cooooo1ol00?l4ooooo`Ooo`3o1?ooool7olc<c0Cooooo1ooo
o`0>ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ooc<c<3_ooool7o`3o00Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000cooooo000:ooooo`Oo0?l01?l00007olc<c0Co00001ooo0?l4
o`0000Ooo`003ol00007oooo00Co00001ol00?l4o`0000Ooo`3o1?l00007olc<c0Co00001oooo`0>
ooooo`Oo0?l01?l00007o`3o057o00001oo<c<`>ooooo`Oo0?l0Hol0000<ooooo`002_ooool7o`3o
00Cooooo1oo<c<`4ooooo`Ooo`3o1?ooool7ool000Cooooo1ol00004ooooo`Ooool01?ooool7o`00
o`Cooooo1ooo0?l4ooooo`Ooc<c<1?ooool7oooo00kooooo1ol0o`04ooooo`Oo0?l01?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1oo<c<`>ooooo`Oo0?l01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00003?ooool000_o
oooo1Ol0o`06ooooo`Goc<c<1_ooool5ool0o`Kooooo1Ooo0006ooooo`Go00001_ooool5oooo00Ko
oooo1Ol00?l6ooooo`Goo`3o1_ooool5olc<c0Kooooo1Oooo`0@ooooo`Go0?l01_ooool5o`3o00Ko
oooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1_l00005ooooo`Ko00001Oooool6o`0000Go
oooo1_l00005ooooo`Goc<c<4?ooool5o`3o0003o`000?oooooooooo00?ooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool5o`0000gooooo000<ooooo`?o0?l000?oooooo`000?ooool00oooool00ol0003ooooo
olc<c002olc<c003ooooool0003ooooo00?ooooo00?o0000oooooooo0?l00_oo0?l00ooooooo0000
ooooo`03ooooo`03o`000?ooooooo`0000;oo`0000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`000003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000
ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003ooooo
ool0o`02ool0o`03ooooool0003ooooo00?ooooo00?o0000ooooooo<c<`00_o<c<`00ooooooo0000
ooooo`03ooooo`03o`000?ooooooool000;oool04_ooool3o`3o0003ooooool0003ooooo00?ooooo
00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o0000
0_ooool01_l0003ooooooooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02oooo
o`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02oooo
o`04o`000?o<c<coc<c<olc<c1;ooooo0ol0o`02ooooo`06o`000?ooooooooooooooool0003ooooo
0ol00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
1?l0000>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
0003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000Co
oooo00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000004oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004oooo
o`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03
o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo
000000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool00000
0oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000kooooo000<ooooo`?o0?oo00?oooooo`000?oo
ool00oooool00ol0003oooooo`3oo`02o`3oo`03ooooool0003ooooo00?ooooo00?o0000ooooooo<
c<`00_o<c<`00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo00?oooooo`000?oo
ool00oooool00ol0003oooooo`3oo`02o`3oo`03ooooool0003ooooo00?ooooo00?o0000ooooool0
o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?oo
ool00oooool00ol0003oooooool00002ool00003ooooool0003ooooo00?ooooo00?o0000ooooool0
0?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l04_ooool3o`3o0003
ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03
o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03
ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03
o`000?oooooo000000;o000000?oooooo`000?ooool01?ooool01?l0003oo`00ool00?oo0002oooo
o`03o`000?oooooooooo00;ooooo00Co0000ool00?oo003oo`000_ooool00ol0003oooooooooo`02
ooooo`04o`000?oo003oo`00ool000;ooooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o
0?l0o`0Booooo`?o003o00?oooooo`000?ooool01?ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool4o`0000;ooooo00?o0000ooooooooool0
0_ooool4o`0000;ooooo00?o0000ooooooooool00_ooool01?l0003oo`00ool00?oo000>ooooo`00
2oooool5o`3oo`Kooooo1Ol0ool6ooooo`Goc<c<1_ooool5o`3oo`Kooooo1Ol0ool6ooooo`Go0?l0
1_ooool5o`0000Kooooo1Ooo0006ooooo`Go003o1_ooool5o`3o013ooooo1Ol0o`06ooooo`Go003o
1_ooool5o`00o`Kooooo1Ol00?l6ooooo`Go00001_ooool5o`0000Kooooo1Ooo00000ol0003ooooo
ooooo`03ooooo`Goo`0000?o0000ooooooooool00oooool5ool00003o`000?oooooooooo00?ooooo
1Ol0o`0@ooooo`Go003o1_ooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo
1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Goo`003Oooool000[ooooo1ol0
ool4ooooo`Oo0?oo1?ooool7olc<c0Cooooo1ol0ool4ooooo`Oo0?oo1?ooool7o`3o00Cooooo1ol0
0004ooooo`Ooo`001?ooool7o`00o`Cooooo1ol0o`0>ooooo`Oo0?l01?ooool7o`00o`Cooooo1ol0
0?l4ooooo`Oo003o1?ooool7o`0000Cooooo1ol00004ooooo`Ooo`001?ooool7ool000Cooooo1ooo
0004ooooo`Oo0?l03_ooool7o`00o`Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol0
0004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7ool000cooooo000:
ooooo`Oo0?oo1?l00007o`3oo`Co00001oo<c<`4o`0000Oo0?oo1?l00007o`3oo`Co00001ol0o`0?
o`0000Ooo`001?l00007o`00o`Co00001ol0o`0>ooooo`Oo0?l01?l00007o`00o`Co00001ol00?l4
o`0000Oo003o6_l00007ool000Co00001ooo0004o`0000Ooo`001?l00007o`3o00kooooo1ol00?mL
o`0000Ooo`003?ooool000[ooooo1ol0ool4ooooo`Oo0?oo1?ooool7olc<c0Cooooo1ol0ool4oooo
o`Oo0?oo1?ooool7o`3o00Cooooo1ol00004ooooo`Ooo`001?ooool7o`00o`Cooooo1ol0o`0>oooo
o`Oo0?l01?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo003o1?ooool7o`0000Cooooo1ol00004oooo
o`Ooo`001?ooool7ool000Cooooo1ooo0004ooooo`Oo0?l03_ooool7o`00o`Cooooo1ol00004oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004oooo
o`Oo00001?ooool7ool000cooooo000;ooooo`Go0?oo1_ooool5o`3oo`Kooooo1Oo<c<`6ooooo`Go
0?oo1_ooool5o`3oo`Kooooo1Ol0o`06ooooo`Go00001_ooool5ool000Kooooo1Ol00?l6ooooo`Go
0?l04?ooool5o`3o00Kooooo1Ol00?l6ooooo`Go003o1_ooool5o`00o`Kooooo1Ol00006ooooo`Ko
00001Oooool5ool00003o`000?oooooooooo00?ooooo1Ooo00000ol0003oooooooooo`03ooooo`Go
o`0000?o0000ooooooooool00oooool5o`3o013ooooo1Ol00?l00ol0003oooooooooo`03ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1Ooo000=ooooo`003?ooool3o`3oo`03ooooool0003ooooo00?ooooo
00?o0000ooooool0ool00_l0ool00ooooooo0000ooooo`03ooooo`03o`000?ooooooc<c<00;oc<c<
00?oooooo`000?ooool00oooool00ol0003oooooo`3oo`02o`3oo`03ooooool0003ooooo00?ooooo
00?o0000ooooool0ool00_l0ool00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l0
00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo
00?o0000oooooooo00000_oo00000ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o
00?oooooo`000?ooool00oooool00ol0003oooooo`3o0002o`3o01;ooooo0ol0o`000ooooooo0000
ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003ooooo
o`00o`02o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000
ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003ooooo
o`000002o`0000;ooooo00Ko0000ooooooooooooooooo`000?ooool3ool000;ooooo00?o0000oooo
ooooool00_ooool01?l0003oo`00ool00?oo0002ooooo`03o`000?oooooooooo00;ooooo00Co0000
ool00?oo003oo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o0?l0o`3o01;ooooo
0ol00?l2ooooo`06o`000?ooooooooooooooool0003ooooo0ol00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo
00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo00Co0000ool00?oo003oo`003_ooool0
00gooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000ooo
ool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000001Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000004ooooo`03o`000?oo
oooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool0
1Ol0003ooooooooooooooooo000001?ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo
00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`05
o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000
oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooo
ooooo`00000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`00000>ooooo`003?ooool3o`00o`03ooooool0003ooooo00?ooooo00?o
0000oooooooo0?l00_oo0?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o00?o
ooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o
0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?o
ooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03ooooool0003ooooo00?ooooo00?o
0000oooooooo00000_oo00000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?o
ooooo`000?ooool00oooool00ol0003oooooool00002ool001;ooooo0ooo00000ooooooo0000oooo
o`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003oooooo`00
o`02o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000oooo
o`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00
0002o`000003ooooool0003ooooo00Cooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000ool00?oo003oo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo003oo`00ool000;o
oooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o0?l0o`0Booooo`?o003o00?oooooo`00
0?ooool01?ooool01?l0003o003oo`00ool00?l2ooooo`03o`000?oooooooooo00;ooooo00Co0000
o`00ool00?oo003o0_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003ooooooooo
o`02ooooo`04o`000?oo003oo`00ool000;ooooo00?o0000ooooooooool00_ooool01?l0003oo`00
ool00?oo000>ooooo`002oooool5o`00o`Kooooo1Ooo0?l6ooooo`Go003o1_ooool5o`0000Kooooo
1Ol00?l6ooooo`Go0?l01_ooool5ool0o`Kooooo1Ooo0006ooooo`Go0?l01_ooool5ool0013ooooo
1Ooo0006ooooo`Go003o1_ooool5o`00o`Kooooo1Ol00?l6ooooo`Go00001_ooool5o`0000Kooooo
1_l00005ooooo`Goo`0000?o0000ooooooooool00oooool5ool00003o`000?oooooooooo00?ooooo
1Ol0o`0@ooooo`Go003o1_ooool5o`00o`03o`000?oooooooooo00?ooooo1Ol00?l00ol0003ooooo
ooooo`03ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool6o`0000Gooooo
1Ooo00000ol0003oooooooooo`03ooooo`Goo`003Oooool000[ooooo1ol00?l4ooooo`Ooo`3o1?oo
ool7o`00o`Cooooo1ol00004ooooo`Oo003o1?ooool7o`3o00Cooooo1ooo0?l4ooooo`Ooo`001?oo
ool7o`3o00Cooooo1ooo000>ooooo`Ooo`001?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo003o1?oo
ool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7ool000Cooooo1ooo0004ooooo`Oo0?l03_oo
ool7o`00o`Cooooo1ol00?l4ooooo`Oo003o1?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?oo
ool7o`0000Cooooo1ol00004ooooo`Ooo`001?ooool7ool000cooooo000:ooooo`Oo003o1?l00007
ool0o`Co00001ol00?l?o`0000Oo003o1?l00007o`3o00Co00001ooo0?l4o`0000Ooo`001?l00007
o`3o00Co00001ooo000>ooooo`Ooo`001?l00007o`00o`Co00001ol00?l4o`0000Oo003o9Ol00007
ool000Co00001ooo0004o`0000Oo0?l03_ooool7o`00o`Co00001ol00?l4o`0000Oo003o>ol00007
ool000Co00001ooo000<ooooo`002_ooool7o`00o`Cooooo1ooo0?l4ooooo`Oo003o1?ooool7o`00
00Cooooo1ol00?l4ooooo`Oo0?l01?ooool7ool0o`Cooooo1ooo0004ooooo`Oo0?l01?ooool7ool0
00kooooo1ooo0004ooooo`Oo003o1?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo00001?ooool7o`00
00Cooooo1ol00004ooooo`Ooo`001?ooool7ool000Cooooo1ol0o`0>ooooo`Oo003o1?ooool7o`00
o`Cooooo1ol00?l4ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`00
00Cooooo1ooo0004ooooo`Ooo`003?ooool000_ooooo1Ol00?l6ooooo`Goo`3o1_ooool5o`00o`Ko
oooo1Ol00006ooooo`Go003o1_ooool5o`3o00Kooooo1Ooo0?l6ooooo`Goo`001_ooool5o`3o00Ko
oooo1Ooo000@ooooo`Goo`001_ooool5o`00o`Kooooo1Ol00?l6ooooo`Go003o1_ooool5o`0000Ko
oooo1_l00005ooooo`Ko00001Oooool5ool00003o`000?oooooooooo00?ooooo1Ooo00000ol0003o
ooooooooo`03ooooo`Go0?l04?ooool5o`00o`03o`000?oooooooooo00?ooooo1Ol00?l00ol0003o
ooooooooo`03ooooo`Go003o00?o0000ooooooooool00oooool6o`0000Gooooo1_l00005ooooo`Ko
00001Oooool6o`0000Gooooo1_l00005ooooo`Goo`0000?o0000ooooooooool00oooool5ool000go
oooo000<ooooo`?o003o00?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03oooo
ool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03oooo
ool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`00
0?ooooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooool00002ool00003oooo
ool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`00
0?ooooooo`0000;oo`004_ooool3ool00003ooooool0003ooooo00?ooooo00?o0000ooooool00?l0
0_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool0
0oooool00ol0003oooooo`00o`02o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool00000
0_l000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o00000_ooool01_l0003ooooo
oooooooooooo0000ooooo`?o00000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo003oo`00
ool000;ooooo00?o0000ooooooooool00_ooool01?l0003oo`00ool00?oo0002ooooo`03o`000?oo
oooooooo00;ooooo00Co0000o`3o0?l0o`3o0?l04_ooool3o`00o`;ooooo00Ko0000oooooooooooo
ooooo`000?ooool3o`00o`;ooooo00?o0000ooooooooool00_ooool01?l0003o003oo`00ool00?l2
ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002
ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002
ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo00Co0000
ool00?oo003oo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?oo003oo`00ool000kooooo
000=ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000001?ooool00ol0003o
ooooo`000004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo
000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`00000Cooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool0
00001?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo0000
0oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003oooo
o`04o`000?ooooooooooo`0000?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`;o00001?ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooo
ooooool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o
00001?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`02ooooo`03
o`000?oooooo000000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool2o`0000Cooooo00?o0000ooooooooool03?ooool000gooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`00
0?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000004oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000003_ooool000cooooo0ol0o`000ooooooo0000ooooo`03ooooo`03
o`000?ooooooo`0000;oo`0000?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03
ooooool0003ooooo00?ooooo00?o0000ooooool0ool00_l0ool00ooooooo0000ooooo`03ooooo`03
o`000?ooooooc<c<00;oc<c<00?oooooo`000?ooool00oooool00ol0003oooooo`000002o`000003
ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`03ooooo`03
o`000?ooooooo`0000;oo`0000?oooooo`000?ooool00oooool00ol0003oooooo`3o0002o`3o0003
ooooool0003ooooo00?ooooo00?o0000oooooooo0?l00_oo0?lBooooo`?oc<c<00?oooooo`000?oo
ool00oooool00ol0003oooooo`00o`02o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool0
0?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?oo
ool00oooool00ol0003oooooo`3o0002o`3o0003ooooool0003ooooo00?ooooo00?o0000ooooool0
o`000_l0o`000ooooooo0000ooooo`04ooooo`04o`000?l0o`3o0?l0o`3o00;ooooo00?o0000oooo
ooooool00_ooool01?l0003oo`00ool00?oo0002ooooo`03o`000?oooooooooo00;ooooo00Co0000
ool00?oo003oo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o0?l0o`3o01;ooooo
0ol00?l00ooooooo0000ooooo`04ooooo`04o`000?l00?oo003oo`00o`;ooooo00?o0000oooooooo
ool00_ooool01?l0003o003oo`00ool00?l2ooooo`03o`000?oooooooooo00;ooooo1?l00002oooo
o`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002oooo
o`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo1?l00002oooo
o`03o`000?oooooooooo00;ooooo00Co0000ool00?oo003oo`000_ooool00ol0003oooooooooo`02
ooooo`04o`000?oo003oo`00ool000kooooo000;ooooo`Go0?l01_ooool5ool000Kooooo1Ooo0?l6
ooooo`Go0?oo1_ooool5olc<c0Kooooo1Ol00006ooooo`Go00001_ooool5ool000Kooooo1Ol0o`06
ooooo`Goo`3o4?ooool5olc<c0Kooooo1Ol00?l6ooooo`Go003o1_ooool5o`00o`Kooooo1Ol0o`06
ooooo`Go0?l01_ooool5o`3o0003o`000?oooooooooo00?ooooo1Ooo00000ol0003oooooooooo`03
ooooo`Goo`0000?o0000ooooooooool00oooool5o`3o013ooooo1Ol00?l6ooooo`Go003o00?o0000
ooooooooool00oooool5o`00o`03o`000?oooooooooo00?ooooo1_l00005ooooo`Ko00001Oooool6
o`0000Gooooo1_l00005ooooo`Ko00001Oooool5ool00003o`000?oooooooooo00?ooooo1Ooo000=
ooooo`002_ooool7o`3o00Cooooo1ooo0004ooooo`Ooo`3o1?ooool7o`3oo`Cooooo1oo<c<`4oooo
o`Oo00001?ooool7o`0000Cooooo1ooo0004ooooo`Oo0?l01?ooool7ool0o`kooooo1oo<c<`4oooo
o`Oo003o1?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol0o`04oooo
o`Ooo`001?ooool7ool000Cooooo1ol0o`0>ooooo`Oo003o1?ooool7o`00o`Cooooo1ol00?l4oooo
o`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ooo0004oooo
o`Ooo`003?ooool000[ooooo1ol0o`04o`0000Ooo`001?l00007ool0o`Co00001ol0ool4o`0000Oo
c<c<6_l00007ool000Co00001ol0o`04o`0000Ooo`3o3_ooool7olc<c0Co00001ol00?l4o`0000Oo
003o1?l00007o`00o`Co00001ol0o`04o`0000Oo0?l01?l00007o`3o00Co00001ooo0004o`0000Oo
o`001?l00007o`3o00kooooo1ol00?l4o`0000Oo003o1?l00007o`00oc_o00001ooo0004o`0000Oo
o`003?ooool000[ooooo1ol0o`04ooooo`Ooo`001?ooool7ool0o`Cooooo1ol0ool4ooooo`Ooc<c<
1?ooool7o`0000Cooooo1ol00004ooooo`Ooo`001?ooool7o`3o00Cooooo1ooo0?l>ooooo`Ooc<c<
1?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo003o1?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo0?l0
1?ooool7ool000Cooooo1ooo0004ooooo`Oo0?l03_ooool7o`00o`Cooooo1ol00?l4ooooo`Oo003o
1?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1ol00004ooooo`Ooo`00
1?ooool7ool000cooooo000;ooooo`Go0?l01_ooool5ool000Kooooo1Ooo0?l6ooooo`Go0?oo1_oo
ool5olc<c0Kooooo1Ol00006ooooo`Go00001_ooool5ool000Kooooo1Ol0o`06ooooo`Goo`3o4?oo
ool5olc<c0Kooooo1Ol00?l6ooooo`Go003o1_ooool5o`00o`Kooooo1Ol0o`06ooooo`Go0?l000?o
0000ooooooooool00oooool5o`3o0003o`000?oooooooooo00?ooooo1Ooo00000ol0003ooooooooo
o`03ooooo`Goo`0000?o0000ooooooooool00oooool5o`3o013ooooo1Ol00?l00ol0003ooooooooo
o`03ooooo`Go003o00?o0000ooooooooool00oooool5o`00o`03o`000?oooooooooo00?ooooo1_l0
0005ooooo`Ko00001Oooool6o`0000Gooooo1_l00005ooooo`Ko00001Oooool5ool00003o`000?oo
oooooooo00?ooooo1Ooo000=ooooo`003?ooool3o`3o0003ooooool0003ooooo00?ooooo00?o0000
oooooooo00000_oo00000ooooooo0000ooooo`03ooooo`03o`000?ooooooo`3o00;oo`3o00?ooooo
o`000?ooool00oooool00ol0003oooooo`3oo`02o`3oo`03ooooool0003ooooo00?ooooo00?o0000
ooooooo<c<`00_o<c<`00ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?ooooo
o`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00?ooooo00?o0000
oooooooo00000_oo00000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?ooooo
o`000?ooool00oooool00ol0003oooooool0o`02ool0oa;ooooo0oo<c<`00ooooooo0000ooooo`03
ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003oooooo`00o`02
o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03
ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool00oooool00ol0003oooooo`3o0002
o`3o00;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`3o00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oo`00ool00?oo0002ooooo`03o`000?oooooooooo00;ooooo00Co0000ool00?oo
003oo`000_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o0?l0o`3o01;ooooo0ol00?l2
ooooo`06o`000?ooooooooooooooool0003ooooo0ol00?l2ooooo`03o`000?oooooooooo00;ooooo
00Co0000o`00ool00?oo003o0_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003o
ooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003o
ooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003o
ooooooooo`02ooooo`04o`000?oo003oo`00ool000;ooooo00?o0000ooooooooool00_ooool01?l0
003oo`00ool00?oo000>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000Cooooo00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooo
ooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000004oooool00ol0003ooooooooo
o`02ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo0000
0oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003oooo
o`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o
0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003oooooo`00
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo000=
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooo
ooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooo
oooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000kooooo000<ooooo`?o003o00?o
ooooo`000?ooool00oooool00ol0003oooooolc<c002olc<c003ooooool0003ooooo00?ooooo00?o
0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo00?o
ooooo`000?ooool00oooool00ol0003oooooool00002ool00003ooooool0003ooooo00?ooooo00?o
0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?ooooooo`0000;oo`0000?o
ooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03ooooool0003ooooo00?ooooo00?o
0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l04_oo
ool3o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000oooo
o`03ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool00oooool00ol0003oooooo`3o
0002o`3o0003ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000oooo
o`03ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool01?ooool01?l0003o0?l0o`3o
0?l0o`02ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo
00Co0000o`3o0?l0o`3o0?l00_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o0?l0o`3o
01;ooooo0oooo`000ooooooo0000ooooo`04ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000
ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;o
oooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o0?l0o`02ooooo`03o`000?oooooooooo
00;ooooo00Co0000o`3o0?l0o`3o0?l00_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o
0?l0o`3o00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool03_ooool000_ooooo1Ol00?l6ooooo`Goc<c<
1_ooool5o`00o`Kooooo1Ol0ool6ooooo`Goo`001_ooool5o`3o00Kooooo1Ooo0006ooooo`Goo`3o
1_ooool5o`3o00Kooooo1Ol0o`0@ooooo`Go003o1_ooool5o`00o`Kooooo1Ol0o`06ooooo`Go0?l0
1_ooool5o`3o00Kooooo1Ol0o`06ooooo`Go0?l000?o0000ooooooooool00oooool6o`0000Gooooo
1Ol0o`000ol0003oooooooooo`03ooooo`Go0?l04?ooool5oooo00Kooooo1Oooo`000ol0003ooooo
ooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo
1Oooo`000ol0003oooooooooo`03ooooo`Go0?l000?o0000ooooooooool00oooool5o`3o0003o`00
0?oooooooooo00?ooooo1Ol0o`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool0
0oooool5oooo00gooooo000:ooooo`Oo003o1?ooool7olc<c0Cooooo1ol00?l4ooooo`Oo0?oo1?oo
ool7ool000Cooooo1ol0o`04ooooo`Ooo`001?ooool7ool0o`Cooooo1ol0o`04ooooo`Oo0?l03_oo
ool7o`00o`Cooooo1ol00?l4ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo0?l01?oo
ool7o`3o00Cooooo1ol00004ooooo`Oo0?l01?ooool7o`3o00kooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo0?l01?oo
ool7oooo00Cooooo1oooo`0<ooooo`002_ooool7o`00o`Co00001oo<c<`4o`0000Oo003o1?l00007
o`3oo`Co00001ooo0004o`0000Oo0?l01?l00007ool000Co00001ooo0?l4o`0000Oo0?l01?l00007
o`3o00kooooo1ol00?l4o`0000Oo003o1?l00007o`3o00Co00001ol0o`04o`0000Oo0?l01?l00007
o`3o00Co00001ol0o`0?o`0000Oo0?l01?l00007o`3o00kooooo1oooo`04o`0000Ooool01?l00007
oooo00Co00001oooo`04o`0000Ooool01?l00007o`3o00Co00001ol0o`04o`0000Oo0?l01?l00007
oooo00Co00001oooo`0<ooooo`002_ooool7o`00o`Cooooo1oo<c<`4ooooo`Oo003o1?ooool7o`3o
o`Cooooo1ooo0004ooooo`Oo0?l01?ooool7ool000Cooooo1ooo0?l4ooooo`Oo0?l01?ooool7o`3o
00kooooo1ol00?l4ooooo`Oo003o1?ooool7o`3o00Cooooo1ol0o`04ooooo`Oo0?l01?ooool7o`3o
00Cooooo1ol0o`04ooooo`Oo00001?ooool7o`3o00Cooooo1ol0o`0>ooooo`Ooool01?ooool7oooo
00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1ol0o`04ooooo`Oo0?l01?ooool7o`3o
00Cooooo1oooo`04ooooo`Ooool03?ooool000_ooooo1Ol00?l6ooooo`Goc<c<1_ooool5o`00o`Ko
oooo1Ol0ool6ooooo`Goo`001_ooool5o`3o00Kooooo1Ooo0006ooooo`Goo`3o1_ooool5o`3o00Ko
oooo1Ol0o`0@ooooo`Go003o1_ooool5o`00o`Kooooo1Ol0o`06ooooo`Go0?l01_ooool5o`3o00Ko
oooo1Ol0o`000ol0003oooooooooo`03ooooo`Go0?l000?o0000ooooooooool00oooool6o`0000Go
oooo1Ol0o`000ol0003oooooooooo`03ooooo`Go0?l04?ooool5oooo0003o`000?oooooooooo00?o
oooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003
o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Go0?l000?o0000oooooooo
ool00oooool5o`3o0003o`000?oooooooooo00?ooooo1Ol0o`000ol0003oooooooooo`03ooooo`Go
ool000?o0000ooooooooool00oooool5oooo00gooooo000<ooooo`?o003o00?oooooo`000?ooool0
0oooool00ol0003oooooolc<c002olc<c003ooooool0003ooooo00?ooooo00?o0000ooooool00?l0
0_l00?l00ooooooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo00?oooooo`000?ooool0
0oooool00ol0003oooooool00002ool00003ooooool0003ooooo00?ooooo00?o0000ooooool0o`00
0_l0o`000ooooooo0000ooooo`03ooooo`03o`000?ooooooo`0000;oo`0000?oooooo`000?ooool0
0oooool00ol0003oooooool0o`02ool0o`03ooooool0003ooooo00?ooooo00?o0000ooooool0o`00
0_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l04_ooool3o`00o`03oooo
ool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`00
0?oooooo0?l000;o0?l000?oooooo`000?ooool00oooool00ol0003oooooo`3o0002o`3o0003oooo
ool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`00
0?oooooo0?l000;o0?l00_ooool01_l0003ooooooooooooooooo0000ooooo`?o0?l00_ooool00ol0
003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o0?l0
o`3o00;ooooo00?o0000ooooooooool00_ooool01?l0003o0?l0o`3o0?l0o`0Booooo`?oool00_oo
ool01_l0003ooooooooooooooooo0000ooooo`?oool00_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003ooooooooo
o`02ooooo`04o`000?l0o`3o0?l0o`3o00;ooooo00?o0000ooooooooool00_ooool01?l0003o0?l0
o`3o0?l0o`02ooooo`03o`000?oooooooooo00;ooooo00Co0000o`3o0?l0o`3o0?l00_ooool00ol0
003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool0
1?l0003oool0oooo0?ooo`0>ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
0003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo
00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000Cooooo00?o0000ooooool000001?ooool01Ol0003ooooooooooooooooo000000;o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000004oooool00ol0003ooooo
ooooo`02ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;o
oooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Co
oooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Co
oooo00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;o
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo01;ooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool2o`0000Cooooo
00?o0000ooooooooool00_ooool2o`0000Cooooo00?o0000ooooooooool00_ooool00ol0003ooooo
o`000004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo017ooooo
00?o0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00cooooo
000=ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooo
ooooooooooooo`00000Dooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003o
oooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000001?ooooo00Go0000ooooooooooooooooo`000003
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000kooooo000<ooooo`?oo`3o
00?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03ooooool0003ooooo00?ooooo
00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l0
00?oooooo`000?ooool00oooool00ol0003oooooool00002ool00003ooooool0003ooooo00?ooooo
00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03ooooo`03o`000?ooooooo`0000;oo`00
00?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0o`03ooooool0003ooooo00?ooooo
00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?oo00;o0?oo
4_ooool3oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000
ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003ooooo
oooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000
ooooo`03ooooo`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool01?ooool01?l0003o0?l0
o`3o0?l0o`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0
003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool0
1?l0003o0?l0o`3o0?l0o`0Booooo`?o000000?oooooo`000?ooool01?ooool01?l0003oool0oooo
0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0
003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000o`3o0?l0o`3o0?l00_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?oo
o`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00kooooo000;oooo
o`Goo`3o1_ooool5o`00o`Kooooo1Ol0o`06ooooo`Go0?l01_ooool5ool000Kooooo1Ol00?l6oooo
o`Goo`001_ooool5ool0o`Kooooo1Ol0o`06ooooo`Go0?oo4?ooool5oooo00Kooooo1Oooo`06oooo
o`Goool01_ooool5oooo00Kooooo1Ol0o`06ooooo`Go0?l01_ooool5o`3o0003o`000?oooooooooo
00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5o`3o
013ooooo1Ol00006ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo
00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5o`3o
0003o`000?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000oooo
ooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`0=ooooo`002_ooool7ool0o`Co
oooo1ol00?l4ooooo`Oo0?l01?ooool7o`3o00Cooooo1ooo0004ooooo`Oo003o1?ooool7ool000Co
oooo1ooo0?l4ooooo`Oo0?l01?ooool7o`3oo`kooooo1oooo`04ooooo`Ooool01?ooool7oooo00Co
oooo1oooo`04ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol0o`04ooooo`Ooool01?ooool7oooo00Co
oooo1ol0o`0>ooooo`Oo00001?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Co
oooo1ol0o`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool03?ooool000[ooooo
1ooo0?l4o`0000Oo003o1?l00007o`3o00Co00001ol0o`04o`0000Ooo`001?l00007o`00o`Co0000
1ooo0004o`0000Ooo`3o1?l00007o`3o00Co00001ol0ool>ooooo`Ooool01?l00007oooo00Co0000
1oooo`04o`0000Ooool01?l00007o`3o00Co00001ol0o`04o`0000Oo0?l01?l00007oooo00Co0000
1oooo`04o`0000Oo0?l03_ooool;o`0000Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool0
1?l00007o`3o00Co00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0<ooooo`002_oo
ool7ool0o`Cooooo1ol00?l4ooooo`Oo0?l01?ooool7o`3o00Cooooo1ooo0004ooooo`Oo003o1?oo
ool7ool000Cooooo1ooo0?l4ooooo`Oo0?l01?ooool7o`3oo`kooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1oooo`04ooooo`Oo0?l01?ooool7o`3o00Cooooo1ol0o`04ooooo`Ooool01?oo
ool7oooo00Cooooo1ol0o`0>ooooo`Oo00001?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?oo
ool7oooo00Cooooo1ol0o`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Ooool03?oo
ool000_ooooo1Ooo0?l6ooooo`Go003o1_ooool5o`3o00Kooooo1Ol0o`06ooooo`Goo`001_ooool5
o`00o`Kooooo1Ooo0006ooooo`Goo`3o1_ooool5o`3o00Kooooo1Ol0ool@ooooo`Goool01_ooool5
oooo00Kooooo1Oooo`06ooooo`Goool01_ooool5o`3o00Kooooo1Ol0o`000ol0003oooooooooo`03
ooooo`Go0?l000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`00
0ol0003oooooooooo`03ooooo`Go0?l04?ooool6o`0000Gooooo1Oooo`000ol0003oooooooooo`03
ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`00
0ol0003oooooooooo`03ooooo`Go0?l000?o0000ooooooooool00oooool5oooo0003o`000?oooooo
oooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5
oooo00gooooo000<ooooo`?oo`3o00?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00
o`03ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03oooo
o`03o`000?oooooo0?l000;o0?l000?oooooo`000?ooool00oooool00ol0003oooooool00002ool0
0003ooooool0003ooooo00?ooooo00?o0000ooooool00?l00_l00?l00ooooooo0000ooooo`03oooo
o`03o`000?ooooooo`0000;oo`0000?oooooo`000?ooool00oooool00ol0003oooooool0o`02ool0
o`03ooooool0003ooooo00?ooooo00?o0000ooooool0o`000_l0o`000ooooooo0000ooooo`03oooo
o`03o`000?oooooo0?oo00;o0?oo4_ooool3oooo0003ooooool0003ooooo00?ooooo00?o0000oooo
ooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?ooooooool000;oool000?oooooo`00
0?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000oooo
ool0o`000_l0o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo0?l000;o0?l00_ooool01_l0
003ooooooooooooooooo0000ooooo`?o0?l00_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
o`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03
o`000?oooooooooo00;ooooo00Co0000o`3o0?l0o`3o0?l04_ooool3o`0000;ooooo00Ko0000oooo
ooooooooooooo`000?ooool3oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo
0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003o
ooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0
003o0?l0o`3o0?l0o`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?oo
o`3oool03_ooool000gooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooo
ooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000001Cooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooo
ooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo
00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000004
ooooo`03o`000?oooooo000000Cooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oo
ooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooooooo
ool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo00?o0000ooooooooool00_ooool0
0ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo0000
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go
0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003o
ooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooo
oooo000000?ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oo
oooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?oo
ool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooo
oooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`0<ooooo`003Oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003o
oooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oooooo
ooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool0
00005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`00
0?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oooooooooo
ooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go
0000ooooooooooooooooo`00000Cooooo`05o`000?ooooooooooooooool000000oooool01Ol0003o
oooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool0
1?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo
000000;ooooo00Go0000ooooooooooooooooo`00000>ooooo`003?ooool3olc<c003ooooool0003o
oooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo
003o00;o003o00?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003o
oooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo
000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03ooooool0003o
oooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo
o`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooolc<c002olc<c1;ooooo0ol00000
0ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00oooool0
0ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_ooo`00
0ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00oooool0
0ol0003oooooo`00o`02o`00o`03ooooool0003ooooo00Cooooo00Co0000o`00ool00?oo003o0_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool0
0_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000o`3o0?l0
o`3o0?l04_ooool3o`000003ooooool0003ooooo00Cooooo00Co0000oooo0?ooo`3oool00_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_oo
ool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3o
ool00_ooool00ol0003oooooooooo`02ooooo`Co00000_ooool00ol0003oooooooooo`02ooooo`04
o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003ooooooooo
o`02ooooo`04o`000?ooo`3oool0oooo00kooooo000;ooooo`Goc<c<1_ooool5oooo00Kooooo1Ol0
0?l6ooooo`Goool01_ooool5oooo00Kooooo1Ol00006ooooo`Go003o1_ooool5oooo00Kooooo1Ooo
0?l6ooooo`Goc<c<4?ooool5o`0000Kooooo1Ol00006ooooo`Goool01_ooool5oooo00Kooooo1Ol0
0?l6ooooo`Go003o1_ooool5o`00o`03o`000?oooooooooo00?ooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool000?o0000ooooooooool00oooool5o`3o013ooooo1Ol00006ooooo`Goool000?o
0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool000?o0000ooooooooool00oooool6o`0000Gooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Ooo
o`0=ooooo`002_ooool7olc<c0Cooooo1oooo`04ooooo`Oo003o1?ooool7oooo00Cooooo1oooo`04
ooooo`Oo00001?ooool7o`00o`Cooooo1oooo`04ooooo`Ooo`3o1?ooool7olc<c0kooooo1ol00004
ooooo`Oo00001?ooool7oooo00Cooooo1oooo`04ooooo`Oo003o1?ooool7o`00o`Cooooo1ol00?l4
ooooo`Ooool01?ooool7oooo00Cooooo1ol0o`0>ooooo`Oo00001?ooool7oooo00Cooooo1oooo`04
ooooo`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04
ooooo`Ooool03?ooool000[ooooo1oo<c<`4o`0000Ooool01?l00007o`00o`Co00001oooo`04o`00
00Ooool03ol00007o`00o`Co00001oooo`04o`0000Ooo`3o1?l00007olc<c0kooooo5_l00007oooo
00Co00001oooo`04o`0000Oo003o1?l00007o`00o`Co00001ol00?l4o`0000Ooool01?l00007oooo
00Co00001ol0o`0>ooooo`_o00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0?o`00
00Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool03?ooool000[ooooo1oo<c<`4ooooo`Oo
ool01?ooool7o`00o`Cooooo1oooo`04ooooo`Ooool01?ooool7o`0000Cooooo1ol00?l4ooooo`Oo
ool01?ooool7ool0o`Cooooo1oo<c<`>ooooo`Oo00001?ooool7o`0000Cooooo1oooo`04ooooo`Oo
ool01?ooool7o`00o`Cooooo1ol00?l4ooooo`Oo003o1?ooool7oooo00Cooooo1oooo`04ooooo`Oo
0?l03_ooool7o`0000Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
00001?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00cooooo000;ooooo`Goc<c<
1_ooool5oooo00Kooooo1Ol00?l6ooooo`Goool01_ooool5oooo00Kooooo1Ol00006ooooo`Go003o
1_ooool5oooo00Kooooo1Ooo0?l6ooooo`Goc<c<4?ooool5o`0000Kooooo1Ol00006ooooo`Goool0
1_ooool5oooo00Kooooo1Ol00?l6ooooo`Go003o00?o0000ooooooooool00oooool5o`00o`03o`00
0?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool0
0oooool5o`3o013ooooo1_l00005ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`00
0?oooooooooo00?ooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool0
0oooool6o`0000Gooooo1Oooo`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool0
0oooool5oooo0003o`000?oooooooooo00?ooooo1Oooo`0=ooooo`003?ooool3olc<c003ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
oooo003o00;o003o00?oooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
oooo000000;o000000?oooooo`000?ooool00oooool00ol0003oooooo`00o`02o`00o`03ooooool0
003ooooo00?ooooo00?o0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oo
ooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooolc<c002olc<c1;ooooo0ol0
00000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool00ooo
ool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o0000ooooooooo`000_oo
o`000ooooooo0000ooooo`03ooooo`03o`000?oooooo003o00;o003o00?oooooo`000?ooool00ooo
ool00ol0003oooooo`00o`02o`00o`;ooooo00Ko0000ooooooooooooooooo`000?ooool3o`00o`;o
oooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?l0o`3o
0?l0o`3o01;ooooo0ol00002ooooo`06o`000?ooooooooooooooool0003ooooo0oooo`02ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02oooo
o`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?oo
o`02ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0oooo00;o
oooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooo0?ooo`3oool03_ooool000gooooo00Go0000ooooooooooooooooo`000002
ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`00
0?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001Cooooo00Go0000oooooooo
ooooooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool0
0003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000004ooooo`03o`000?oooooo000000Cooooo00Go0000oooooooooooooooo
o`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000001?ooooo00?o
0000ooooooooool00_ooool00ol0003oooooo`000004ooooo`05o`000?ooooooooooooooool00000
0_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0
003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?oo
ooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_oo
ool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo
0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0Aooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0Booooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0_l0
0004ooooo`03o`000?oooooooooo00;ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooool000001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`02ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`;o00001?ooool00ol0003ooooooooo
o`0<ooooo`003Oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000oooooooooooooooo
o`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002oooo
o`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?oo
ooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooo
oooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05
o`000?ooooooooooooooool000005?ooool01Ol0003ooooooooooooooooo000000;ooooo00Go0000
ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000oooooooooooo
ooooo`000002ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00003
ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0003ooooooooooooooooo000000?ooooo
00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool000000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooo
oooooooo000000;ooooo00Go0000ooooooooooooooooo`00000Cooooo`05o`000?oooooooooooooo
ool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000ooo
ool01Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`000003ooooo`04
o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000002ooooo`05o`000?oooooo
ooooooooool000000oooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000oooool0
1Ol0003ooooooooooooooooo000000;ooooo00Go0000ooooooooooooooooo`00000>ooooo`003?oo
ool3olc<c003ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`03ooooo`03o`000?ooooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooo`00
o`02o`00o`03ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`03ooooo`03o`000?ooooooool000;oool000?oooooo`000?ooool00oooool00ol0003oooooool0
o`02ool0o`03ooooool0003ooooo00?ooooo00?o0000ooooool000000_l000000ooooooo0000oooo
o`03ooooo`03o`000?ooooooo`3o00;oo`3o00?oooooo`000?ooool00oooool00ol0003oooooo`00
o`02o`00oa;ooooo0ol000000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?o
ooooo`000?ooool00oooool00ol0003ooooooooo0002oooo0003ooooool0003ooooo00?ooooo00?o
0000ooooooooo`000_ooo`000ooooooo0000ooooo`03ooooo`03o`000?oooooo000000;o000000?o
ooooo`000?ooool00oooool00ol0003oooooo`000002o`000003ooooool0003ooooo00Cooooo1?l0
0002ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003o
o`3oool0oooo0?lBooooo`?o000000?oooooo`000?ooool01?ooool01?l0003oool0oooo0?ooo`02
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo0?ooo`3oool00_ooool00ol0003ooooooooo
o`02ooooo`04o`000?ooo`3oool0oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0
oooo0?ooo`02ooooo`03o`000?oooooooooo00;ooooo1?l00002ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooo0?ooo`3oool00_ooool00ol0003oooooooooo`02ooooo`04o`000?ooo`3oool0
oooo00;ooooo00?o0000ooooooooool00_ooool01?l0003oool0oooo0?ooo`02ooooo`03o`000?oo
oooooooo00;ooooo00Co0000oooo0?ooo`3oool03_ooool000_ooooo1Oo<c<`6ooooo`Go00001_oo
ool5ool0o`Kooooo1Ol00?l6ooooo`Go00001_ooool5oooo00Kooooo1Ooo0?l6ooooo`Go00001_oo
ool5ool0o`Kooooo1Ol00?l@ooooo`Go00001_ooool5o`0000Kooooo1Oooo`06ooooo`Goool01_oo
ool5o`0000Kooooo1Ol00006ooooo`Ko00001Oooool5oooo0003o`000?oooooooooo00?ooooo1Ooo
o`000ol0003oooooooooo`03ooooo`Goo`3o4?ooool5o`0000Kooooo1Oooo`000ol0003ooooooooo
o`03ooooo`Goool000?o0000ooooooooool00oooool5oooo0003o`000?oooooooooo00?ooooo1Ooo
o`000ol0003oooooooooo`03ooooo`Ko00001Oooool5oooo0003o`000?oooooooooo00?ooooo1Ooo
o`000ol0003oooooooooo`03ooooo`Goool000?o0000ooooooooool00oooool5oooo00gooooo000:
ooooo`Ooc<c<1?ooool7o`0000Cooooo1ooo0?l4ooooo`Oo003o1?ooool7o`0000Cooooo1oooo`04
ooooo`Ooo`3o1?ooool7o`0000Cooooo1ooo0?l4ooooo`Oo003o3_ooool7o`0000Cooooo1ol00004
ooooo`Ooool01?ooool7oooo00Cooooo1ol00004ooooo`Oo00001?ooool7o`0000Cooooo1oooo`04
ooooo`Ooool01?ooool7ool0o`kooooo1ol00004ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04
ooooo`Ooool01?ooool7o`0000Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`0<
ooooo`002_ooool7olc<c0oo00001ooo0?l4o`0000Oo003o3ol00007oooo00Co00001ooo0?l?o`00
00Ooo`3o1?l00007o`00o`kooooo5_l00007oooo00Co00001oooo`0Uo`0000Ooool01?l00007oooo
00Co00001ooo0?l>ooooo`_o00001oooo`04o`0000Ooool01?l00007oooo00Co00001oooo`0?o`00
00Ooool01?l00007oooo00Co00001oooo`04o`0000Ooool03?ooool000[ooooo1oo<c<`4ooooo`Oo
00001?ooool7ool0o`Cooooo1ol00?l4ooooo`Oo00001?ooool7oooo00Cooooo1ooo0?l4ooooo`Oo
00001?ooool7ool0o`Cooooo1ol00?l>ooooo`Oo00001?ooool7o`0000Cooooo1oooo`04ooooo`Oo
ool01?ooool7o`0000Cooooo1ol00004ooooo`Oo00001?ooool7oooo00Cooooo1oooo`04ooooo`Oo
o`3o3_ooool7o`0000Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00Cooooo1oooo`04ooooo`Oo
00001?ooool7oooo00Cooooo1oooo`04ooooo`Ooool01?ooool7oooo00cooooo000;ooooo`Goc<c<
1_ooool5o`0000Kooooo1Ooo0?l6ooooo`Go003o1_ooool5o`0000Kooooo1Oooo`06ooooo`Goo`3o
1_ooool5o`0000Kooooo1Ooo0?l6ooooo`Go003o4?ooool5o`0000Kooooo1Ol00006ooooo`Goool0
1_ooool5oooo00Kooooo1Ol00006ooooo`Go00001_ooool5o`0000Kooooo1Oooo`06ooooo`Goool0
1_ooool5ool0oa3ooooo1Ol00006ooooo`Goool01_ooool5oooo00Kooooo1Oooo`06ooooo`Goool0
1_ooool5o`0000Kooooo1Oooo`06ooooo`Goool01_ooool5oooo00Kooooo1Oooo`0=ooooo`003?oo
ool3olc<c0Sooooo0ol00008ooooo`?oo`3o2?ooool3o`00o`Sooooo0ol00008ooooo`?oool02?oo
ool3ool0o`Sooooo0ol00008ooooo`?oo`3o2?ooool3o`00oa;ooooo0ol00008ooooo`?o00002?oo
ool3oooo00Sooooo0oooo`08ooooo`?o00002?ooool3o`0000Sooooo0ol00008ooooo`?oool02?oo
ool3oooo00Sooooo0ooo0?lBooooo`?o00002?ooool3oooo00Sooooo0oooo`08ooooo`?oool02?oo
ool3oooo00Sooooo0ol00008ooooo`?oool02?ooool3oooo00Sooooo0oooo`08ooooo`?oool03_oo
ool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOoo
ool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOooool00?ooooooLOoo
ool00001\
\>"],
  ImageRangeCache->{{{0, 367}, {367, 0}} -> {-0.0750158, -0.0750158, \
0.0085832, 0.0085832}, {{8.6875, 117.938}, {358.25, 249}} -> {-0.137478, \
-0.138107, 0.0100688, 0.0100688}, {{128.875, 238.062}, {358.25, 249}} -> \
{-1.34836, -0.138472, 0.0100745, 0.0100745}, {{249, 358.25}, {358.25, 249}} -> \
{-2.55712, -0.138107, 0.0100688, 0.0100688}, {{8.6875, 117.938}, {238.062, \
128.875}} -> {-0.137843, -1.34899, 0.0100745, 0.0100745}, {{128.875, \
238.062}, {238.062, 128.875}} -> {-1.34836, -1.34899, 0.0100745, 0.0100745}, \
{{249, 358.25}, {238.062, 128.875}} -> {-2.55887, -1.34899, 0.0100745, \
0.0100745}, {{8.6875, 117.938}, {117.938, 8.6875}} -> {-0.137478, -2.55775, \
0.0100688, 0.0100688}, {{128.875, 238.062}, {117.938, 8.6875}} -> {-1.34836, \
-2.5595, 0.0100745, 0.0100745}, {{249, 358.25}, {117.938, 8.6875}} -> \
{-2.55712, -2.55775, 0.0100688, 0.0100688}},
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[BoxData[
    \(DeleteModel[ugM8]\)], "Input",
  CellLabel->"In[68]:=",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1152}, {0, 700}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
WindowToolbars->"RulerBar",
WindowSize->{734, 625},
WindowMargins->{{Automatic, 47}, {Automatic, 20}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
ShowCellLabel->False,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "LocalInteractions:Introduction"->{
    Cell[1776, 53, 104, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1883, 56, 682, 15, 68, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2568, 73, 265, 6, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2870, 84, 121, 1, 34, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2994, 87, 237, 5, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3256, 96, 86, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3345, 99, 101, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3449, 102, 211, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3663, 110, 216, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3882, 116, 159, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4044, 122, 231, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4278, 130, 287, 8, 40, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4568, 140, 366, 11, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4937, 153, 300, 7, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5240, 162, 134, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5377, 167, 92, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5472, 170, 158, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5633, 175, 179, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5815, 181, 656, 13, 69, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6496, 198, 185, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6684, 204, 774, 22, 59, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[7473, 229, 505, 13, 55, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[8003, 246, 157, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[8163, 252, 17564, 356, 294, 2180, 160, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[25742, 611, 345, 7, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[26090, 620, 354, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[26447, 629, 257, 5, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[26707, 636, 17999, 376, 294, 2316, 177, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[44709, 1014, 201, 5, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[44913, 1021, 226, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[45142, 1028, 116, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[45261, 1033, 149, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[45413, 1039, 17896, 365, 294, 2236, 167, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[63312, 1406, 240, 6, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[63555, 1414, 140, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[63698, 1419, 174, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[63875, 1425, 313, 9, 41, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[64191, 1436, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[64349, 1444, 90, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[64442, 1447, 883, 17, 82, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[65328, 1466, 1195, 26, 114, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[66572, 1498, 90, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[66687, 1503, 99, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[66789, 1506, 241, 7, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[67055, 1517, 183, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[67241, 1523, 73644, 1488, 294, 8634, 680, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[140900, 3014, 355, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[141258, 3023, 488, 14, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[141771, 3041, 140, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[141914, 3046, 75190, 1552, 294, 9034, 730, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[217119, 4601, 394, 8, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[217516, 4611, 283, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[217802, 4619, 604, 15, 56, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[218431, 4638, 220, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[218654, 4644, 425525, 14683, 468, 139218, 11142, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[644194, 19330, 113, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[644310, 19333, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[644434, 19338, 811, 21, 72, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[645248, 19361, 187, 7, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[645438, 19370, 367, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[645808, 19379, 335, 9, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[646168, 19392, 138, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[646309, 19397, 31681, 480, 294, 1299, 99, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[678005, 19880, 139, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[678169, 19887, 138, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[678310, 19892, 31657, 477, 294, 1275, 96, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[709982, 20372, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[710106, 20377, 185, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[710316, 20387, 189, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[710508, 20393, 35465, 671, 294, 4789, 287, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[745988, 21067, 356, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[746369, 21078, 138, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[746510, 21083, 23212, 512, 294, 3414, 262, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[769737, 21598, 140, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[769880, 21603, 100, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[770005, 21608, 138, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[770146, 21613, 23183, 517, 294, 3462, 268, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[793344, 22133, 126, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[793507, 22141, 121, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[793631, 22144, 370, 11, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[794004, 22157, 1082, 38, 53, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[795089, 22197, 202, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[795294, 22203, 174, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[795493, 22212, 149, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[795645, 22217, 60955, 1468, 294, 14632, 890, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[856615, 23688, 387, 9, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[857027, 23701, 196, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[857226, 23707, 17547, 552, 294, 4904, 390, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[874788, 24262, 139, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[874930, 24268, 202, 7, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[875157, 24279, 424, 10, 73, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[875584, 24291, 751, 22, 59, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[876350, 24316, 517, 9, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[876870, 24327, 421, 7, 50, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[877316, 24338, 139, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[877458, 24343, 28898, 1389, 294, 14886, 1210, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[906371, 25735, 298, 6, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[906694, 25745, 249, 5, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[906946, 25752, 40818, 823, 403, 5388, 380, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[947779, 26578, 98, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[947880, 26581, 287, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[948170, 26589, 85, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[948280, 26594, 220, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[948503, 26600, 120798, 7572, 294, 88316, 7165, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1069316, 34175, 82, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1069401, 34178, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1069558, 34186, 109, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1069670, 34189, 403, 7, 50, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1070098, 34200, 375, 7, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1070476, 34209, 40661, 1225, 284, 11186, 855, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1111152, 35437, 104, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1111281, 35442, 230, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1111514, 35448, 63297, 2073, 284, 18522, 1515, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1174826, 37524, 539, 12, 56, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1175368, 37538, 529, 10, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1175900, 37550, 231, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1176156, 37558, 219, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1176378, 37564, 42574, 1327, 284, 12353, 948, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1218967, 38894, 144, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1219114, 38899, 87, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1219226, 38904, 219, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1219448, 38910, 40833, 1230, 284, 11236, 859, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1260296, 40143, 106, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1260405, 40146, 131, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1260539, 40151, 559, 11, 70, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1261101, 40164, 914, 26, 72, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262018, 40192, 176, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262197, 40198, 111, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262333, 40203, 148, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262484, 40208, 156, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262677, 40216, 217, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1262897, 40222, 97435, 3728, 294, 35183, 2954, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1360347, 43953, 320, 9, 40, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1360692, 43966, 717, 14, 186, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1361412, 43982, 61651, 2086, 216, 25635, 1636, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1423078, 46071, 745, 18, 70, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1423826, 46091, 879, 18, 78, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1424708, 46111, 574, 12, 54, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1425285, 46125, 452, 9, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1425762, 46138, 217, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1425982, 46144, 43911, 1326, 294, 12086, 927, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1469908, 47473, 139, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1470072, 47480, 179, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1470254, 47486, 42235, 1249, 294, 11261, 861, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1512504, 48738, 603, 11, 64, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1513132, 48753, 493, 11, 125, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1513628, 48766, 14458, 381, 137, 3107, 235, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1528101, 49150, 671, 11, 78, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1528775, 49163, 125, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1528937, 49171, 104, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1529044, 49174, 432, 9, 52, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1529479, 49185, 438, 11, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1529920, 49198, 681, 15, 112, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1530626, 49217, 178, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1530807, 49223, 66713, 1784, 294, 13599, 1123, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1597557, 51012, 126, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1597686, 51017, 112, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1597813, 51023, 154, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1597970, 51029, 295, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1598290, 51039, 112, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1598405, 51044, 107, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1598527, 51050, 224, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1598776, 51060, 172, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1598951, 51066, 569, 9, 47, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1599523, 51077, 746, 14, 101, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1600284, 51094, 631, 17, 57, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1600918, 51113, 199, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1601120, 51119, 209, 7, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1601354, 51130, 241, 5, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1601598, 51137, 214, 4, 41, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1601827, 51144, 304, 7, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602156, 51155, 123, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602282, 51160, 145, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602464, 51168, 123, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602590, 51173, 145, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602750, 51179, 80, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1602855, 51184, 286, 6, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1603144, 51192, 27183, 717, 257, 6942, 462, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1630342, 51912, 195, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1630562, 51920, 350, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1630915, 51929, 27915, 728, 255, 6938, 463, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1658845, 52660, 152, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1659022, 52668, 424, 8, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1659449, 52678, 28123, 733, 255, 6978, 466, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1687587, 53414, 692, 16, 68, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1688282, 53432, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1688440, 53440, 123, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1688566, 53443, 603, 15, 56, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1689172, 53460, 529, 10, 50, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1689704, 53472, 1672, 57, 82, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1691379, 53531, 152, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1691534, 53537, 273, 9, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1691810, 53548, 146, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1691959, 53553, 110, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1692094, 53558, 165, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1692262, 53564, 153, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1692430, 53570, 423, 13, 40, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1692856, 53585, 744, 16, 128, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1693625, 53605, 179, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1693807, 53611, 66738, 1787, 294, 13623, 1126, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1760560, 55401, 100, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1760663, 55404, 296, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1760962, 55412, 118, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1761105, 55417, 592, 12, 170, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1761700, 55431, 58926, 1132, 211, 7096, 487, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1820641, 56566, 337, 6, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1820981, 56574, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1821151, 56583, 98, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1821274, 56588, 92, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1821369, 56591, 330, 10, 40, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1821702, 56603, 338, 6, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1822043, 56611, 207, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1822253, 56619, 10883, 302, 94, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1833139, 56923, 261, 9, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1833403, 56934, 831, 28, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1834237, 56964, 433, 8, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1834673, 56974, 287, 9, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1834963, 56985, 205, 6, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1835193, 56995, 2992, 60, 442, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1838188, 57057, 429, 7, 33, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1838620, 57066, 429, 7, 33, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1839064, 57076, 303, 6, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1839370, 57084, 1514, 29, 170, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1840887, 57115, 214, 5, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1841126, 57124, 186, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1841315, 57130, 751, 22, 59, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1842081, 57155, 897, 19, 144, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1843003, 57178, 179, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1843185, 57184, 25988, 1111, 294, 12214, 935, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869188, 58298, 267, 6, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869458, 58306, 132, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869593, 58311, 114, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869732, 58316, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869856, 58321, 108, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1869979, 58327, 171, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1870153, 58334, 257, 5, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1870413, 58341, 461, 14, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1870877, 58357, 439, 16, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1871341, 58377, 137, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1871481, 58382, 135, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1871631, 58388, 141, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1871775, 58394, 710, 18, 52, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1872488, 58414, 576, 20, 41, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1873067, 58436, 322, 8, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1873392, 58446, 445, 10, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1873840, 58458, 90, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1873955, 58463, 268, 5, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[1874226, 58470, 126874, 1711, 107, 61641, 900, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2001115, 60184, 842, 27, 59, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2001982, 60215, 277, 5, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2002262, 60222, 126785, 1710, 107, 61641, 900, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2129062, 61935, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2129185, 61940, 255, 7, 24, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2129443, 61949, 1049, 22, 193, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2130495, 61973, 290, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2130810, 61983, 278, 5, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2131091, 61990, 44495, 658, 71, 39736, 594, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2175601, 62651, 557, 10, 67, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2176161, 62663, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2176284, 62668, 1039, 23, 193, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2177326, 62693, 273, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2177624, 62703, 227, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2177854, 62709, 66968, 971, 108, 61641, 900, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2244837, 63683, 243, 5, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2245083, 63690, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2245252, 63699, 103, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2245358, 63702, 219, 5, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2245602, 63711, 98, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2245703, 63714, 882, 22, 50, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2246588, 63738, 236, 4, 60, "DisplayFormula",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2246827, 63744, 156, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2246986, 63750, 624, 12, 186, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2247635, 63766, 190, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2247828, 63772, 493, 8, 50, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2248370, 63786, 98, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2248471, 63789, 276, 7, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2248750, 63798, 721, 14, 203, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2249496, 63816, 189, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2249688, 63822, 1230, 18, 110, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2250967, 63846, 95, 1, 34, "Subsubsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2251065, 63849, 483, 14, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2251551, 63865, 575, 15, 55, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2252129, 63882, 155, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2252287, 63888, 676, 15, 112, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2252966, 63905, 95, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2253086, 63910, 137, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2253226, 63915, 16754, 264, 294, 904, 63, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2269995, 64182, 80, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270078, 64185, 131, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270212, 64190, 181, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270396, 64197, 113, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270512, 64202, 150, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270665, 64208, 193, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270883, 64217, 113, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2270999, 64222, 570, 9, 57, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2271584, 64234, 568, 14, 50, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2272155, 64250, 139, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2272297, 64255, 305, 7, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2272627, 64266, 113, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2272743, 64271, 129, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2272887, 64277, 100, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2273012, 64282, 189, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2273204, 64288, 16713, 263, 294, 887, 62, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2289932, 64554, 197, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290154, 64563, 127, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290284, 64568, 111, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290410, 64574, 174, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290609, 64582, 113, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290725, 64587, 129, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2290869, 64593, 211, 5, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2291105, 64602, 602, 10, 138, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2291710, 64614, 432, 7, 33, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2292145, 64623, 147, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2292307, 64629, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2292430, 64634, 445, 9, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2292878, 64645, 113, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2292994, 64648, 509, 11, 55, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2293506, 64661, 401, 11, 38, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2293910, 64674, 336, 9, 28, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2294249, 64685, 473, 10, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2294747, 64699, 246, 7, 40, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2294996, 64708, 141, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2295152, 64714, 419, 12, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2295574, 64728, 686, 13, 182, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2296263, 64743, 144, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2296432, 64751, 311, 8, 40, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2296746, 64761, 115, 3, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2296876, 64767, 489, 13, 42, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2297390, 64784, 2053, 38, 330, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2299446, 64824, 432, 7, 33, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2299881, 64833, 446, 8, 33, "Message",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2300342, 64844, 193, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2300538, 64850, 2049, 38, 330, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2302590, 64890, 125, 3, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2302718, 64895, 676, 15, 112, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2303397, 64912, 134, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2303534, 64918, 336, 7, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2303873, 64927, 104, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2303980, 64930, 182, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2304165, 64936, 100, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2304290, 64941, 126, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2304419, 64946, 333, 7, 41, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2304767, 64956, 171, 4, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2304941, 64962, 246, 5, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2305190, 64969, 133, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2305348, 64977, 132, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2305483, 64982, 264, 5, 41, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2305762, 64990, 104, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2305891, 64995, 137, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2306031, 65000, 16742, 264, 294, 904, 63, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2322788, 65267, 100, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2322891, 65270, 144, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2323060, 65277, 137, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2323200, 65282, 16668, 261, 294, 862, 60, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2339883, 65546, 89, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2339997, 65551, 124, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2340124, 65556, 526, 10, 57, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2340665, 65569, 120, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2340788, 65574, 232, 5, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2341069, 65585, 115, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2341209, 65590, 435, 11, 73, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2341647, 65603, 763, 22, 59, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2342425, 65628, 496, 9, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2342924, 65639, 259, 6, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2343186, 65647, 441, 9, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2343630, 65658, 360, 7, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2343993, 65667, 122, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2344140, 65672, 298, 7, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2344441, 65681, 144, 3, 21, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2344588, 65686, 136, 3, 21, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2344739, 65692, 199, 6, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2344963, 65702, 612, 12, 186, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2345578, 65716, 30537, 606, 197, 3619, 268, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2376130, 66325, 292, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2376447, 66335, 231, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2376681, 66342, 142260, 3187, 409, 20291, 1675, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2518956, 69532, 95, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2519076, 69537, 213, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2519292, 69543, 35744, 633, 241, 3225, 226, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555051, 70179, 208, 5, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555284, 70188, 129, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555416, 70193, 181, 4, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555612, 70200, 122, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555737, 70205, 110, 1, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2555850, 70208, 517, 9, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2556370, 70219, 199, 5, 25, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2556572, 70226, 438, 9, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2557013, 70237, 360, 7, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2557398, 70248, 253, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2557654, 70255, 144, 3, 21, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2557801, 70260, 136, 3, 21, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2557974, 70268, 146, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2558123, 70273, 46683, 770, 294, 3251, 228, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2604821, 71046, 291, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2605137, 71056, 182, 4, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2605322, 71062, 106166, 2707, 294, 19800, 1635, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2711503, 73772, 122, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2711662, 73780, 80, 1, 61, "Section",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2711745, 73783, 128, 2, 26, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2711898, 73789, 321, 7, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2712222, 73798, 2990, 50, 177, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2715227, 73851, 124, 2, 26, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2715376, 73857, 838, 15, 282, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2716217, 73874, 3127, 86, 283, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2719381, 73965, 77, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2719461, 73968, 431, 8, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2719895, 73978, 291, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2720211, 73988, 305, 5, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2720519, 73995, 50737, 731, 72, 37302, 560, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2771271, 74729, 122, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2771430, 74737, 121, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2771554, 74740, 430, 8, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2771987, 74750, 243, 5, 39, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2772233, 74757, 968, 18, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2773204, 74777, 262, 5, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2773469, 74784, 482, 11, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2773954, 74797, 194, 4, 22, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2774151, 74803, 252, 6, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2774428, 74813, 449, 9, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2774880, 74824, 80692, 2024, 369, 19548, 1264, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2855587, 76851, 122, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2855746, 76859, 92, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2855863, 76864, 334, 8, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2856200, 76874, 48771, 1325, 294, 11186, 855, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2904986, 78202, 349, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2905360, 78213, 211, 5, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2905574, 78220, 54495, 1551, 294, 13018, 1033, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2960084, 79774, 249, 6, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2960358, 79784, 139, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[2960500, 79789, 49005, 1332, 294, 11226, 860, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3009520, 81124, 122, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3009679, 81132, 79, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3009761, 81135, 366, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3010152, 81146, 231, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3010386, 81153, 61554, 1416, 294, 9523, 768, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3071955, 82572, 293, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3072273, 82582, 221, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3072497, 82588, 319709, 11157, 334, 106412, 8517, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3392221, 93748, 123, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3392381, 93756, 75, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3392459, 93759, 493, 12, 56, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3392977, 93775, 463, 8, 138, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3393443, 93785, 312, 7, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3393792, 93797, 247, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3394042, 93804, 103878, 1390, 294, 16031, 300, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3497935, 95197, 416, 8, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3498376, 95209, 234, 4, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3498613, 95215, 345304, 5804, 446, 254498, 4676, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3843932, 101022, 126, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3844095, 101030, 82, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3844202, 101035, 204, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3844409, 101041, 150174, 2246, 294, 6083, 461, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3994598, 103290, 358, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3994981, 103301, 141, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[3995125, 103306, 208956, 3052, 362, 7100, 554, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4204096, 106361, 176, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4204297, 106369, 141, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4204441, 106374, 153143, 2339, 294, 6643, 525, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4357599, 108716, 124, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4357760, 108724, 103, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4357888, 108729, 166, 4, 42, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4358057, 108735, 22453, 816, 294, 8413, 637, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4380547, 109556, 319, 7, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4380869, 109565, 32595, 977, 294, 8884, 679, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4413479, 110545, 357, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4413861, 110556, 141, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4414005, 110561, 36483, 1143, 294, 10371, 815, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4450503, 111707, 296, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4450824, 111717, 127, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4450954, 111722, 35612, 1092, 294, 9788, 768, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4486581, 112817, 124, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4486754, 112826, 87, 1, 61, "Section",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4486844, 112829, 336, 9, 43, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4487205, 112842, 435, 11, 73, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4487643, 112855, 763, 22, 59, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4488443, 112882, 79, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4488525, 112885, 375, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4488925, 112896, 235, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4489163, 112903, 56525, 1658, 238, 13822, 1125, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4545703, 114564, 286, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4546014, 114574, 219, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4546236, 114580, 302549, 12131, 393, 121213, 9887, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4848800, 126714, 128, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4848965, 126722, 75, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4849043, 126725, 462, 9, 106, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4849530, 126738, 236, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4849769, 126744, 54645, 760, 294, 10541, 210, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4904429, 127507, 471, 10, 122, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4904925, 127521, 194, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[4905122, 127527, 414767, 7313, 552, 311976, 6036, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5319904, 134843, 127, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5320080, 134852, 96, 1, 61, "Section",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5320179, 134855, 428, 13, 36, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5320610, 134870, 3951, 114, 153, "Text",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5324586, 134988, 1927, 34, 373, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5326516, 135024, 2441, 70, 227, "Print",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5328994, 135099, 79, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5329076, 135102, 365, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5329466, 135113, 229, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5329698, 135120, 61295, 1413, 294, 9354, 766, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5391008, 136536, 290, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5391323, 136546, 213, 4, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5391539, 136552, 552962, 21587, 446, 217632, 17440, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5944504, 158141, 206, 5, 25, "Output",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5944725, 158149, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5944883, 158157, 75, 1, 36, "Subsection",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5944961, 158160, 361, 7, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5945347, 158171, 229, 5, 58, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[5945579, 158178, 117415, 2527, 294, 14882, 1256, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6063009, 160708, 290, 6, 90, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6063324, 160718, 272, 6, 74, "Input",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6063599, 160726, 409788, 14144, 374, 128182, 10661, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:Introduction"],
    Cell[6473402, 174873, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:Introduction"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"LocalInteractions:Introduction", 6474385, 174903}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 104, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[1883, 56, 682, 15, 68, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2568, 73, 265, 6, 36, "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2870, 84, 121, 1, 34, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2994, 87, 237, 5, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3256, 96, 86, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[3345, 99, 101, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[3449, 102, 211, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[3663, 110, 216, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3882, 116, 159, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[4044, 122, 231, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[4278, 130, 287, 8, 40, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4568, 140, 366, 11, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[4937, 153, 300, 7, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[5240, 162, 134, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5377, 167, 92, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[5472, 170, 158, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5633, 175, 179, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[5815, 181, 656, 13, 69, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[6496, 198, 185, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[6684, 204, 774, 22, 59, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[7473, 229, 505, 13, 55, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[8003, 246, 157, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[8163, 252, 17564, 356, 294, 2180, 160, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[25742, 611, 345, 7, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[26090, 620, 354, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[26447, 629, 257, 5, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[26707, 636, 17999, 376, 294, 2316, 177, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"],
Cell[44709, 1014, 201, 5, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[44913, 1021, 226, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[45142, 1028, 116, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[45261, 1033, 149, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[45413, 1039, 17896, 365, 294, 2236, 167, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"],
Cell[63312, 1406, 240, 6, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[63555, 1414, 140, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[63698, 1419, 174, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[63875, 1425, 313, 9, 41, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[64191, 1436, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64349, 1444, 90, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[64442, 1447, 883, 17, 82, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[65328, 1466, 1195, 26, 114, "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[66572, 1498, 90, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[66687, 1503, 99, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[66789, 1506, 241, 7, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[67055, 1517, 183, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[67241, 1523, 73644, 1488, 294, 8634, 680, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[140900, 3014, 355, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[141258, 3023, 488, 14, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[141771, 3041, 140, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[141914, 3046, 75190, 1552, 294, 9034, 730, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[217119, 4601, 394, 8, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[217516, 4611, 283, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[217802, 4619, 604, 15, 56, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[218431, 4638, 220, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[218654, 4644, 425525, 14683, 468, 139218, 11142, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[644194, 19330, 113, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[644310, 19333, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[644434, 19338, 811, 21, 72, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[645248, 19361, 187, 7, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[645438, 19370, 367, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[645808, 19379, 335, 9, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[646168, 19392, 138, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[646309, 19397, 31681, 480, 294, 1299, 99, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[678005, 19880, 139, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[678169, 19887, 138, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[678310, 19892, 31657, 477, 294, 1275, 96, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[709982, 20372, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[710106, 20377, 185, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[710316, 20387, 189, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[710508, 20393, 35465, 671, 294, 4789, 287, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[745988, 21067, 356, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[746369, 21078, 138, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[746510, 21083, 23212, 512, 294, 3414, 262, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[769737, 21598, 140, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[769880, 21603, 100, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[770005, 21608, 138, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[770146, 21613, 23183, 517, 294, 3462, 268, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[793344, 22133, 126, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[793507, 22141, 121, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[793631, 22144, 370, 11, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[794004, 22157, 1082, 38, 53, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[795089, 22197, 202, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[795294, 22203, 174, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[795493, 22212, 149, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[795645, 22217, 60955, 1468, 294, 14632, 890, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[856615, 23688, 387, 9, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[857027, 23701, 196, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[857226, 23707, 17547, 552, 294, 4904, 390, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[874788, 24262, 139, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[874930, 24268, 202, 7, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[875157, 24279, 424, 10, 73, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[875584, 24291, 751, 22, 59, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[876350, 24316, 517, 9, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[876870, 24327, 421, 7, 50, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[877316, 24338, 139, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[877458, 24343, 28898, 1389, 294, 14886, 1210, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[906371, 25735, 298, 6, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[906694, 25745, 249, 5, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[906946, 25752, 40818, 823, 403, 5388, 380, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[947779, 26578, 98, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[947880, 26581, 287, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[948170, 26589, 85, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[948280, 26594, 220, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[948503, 26600, 120798, 7572, 294, 88316, 7165, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1069316, 34175, 82, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1069401, 34178, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1069558, 34186, 109, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[1069670, 34189, 403, 7, 50, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1070098, 34200, 375, 7, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1070476, 34209, 40661, 1225, 284, 11186, 855, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1111152, 35437, 104, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1111281, 35442, 230, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1111514, 35448, 63297, 2073, 284, 18522, 1515, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1174826, 37524, 539, 12, 56, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1175368, 37538, 529, 10, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1175900, 37550, 231, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1176156, 37558, 219, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1176378, 37564, 42574, 1327, 284, 12353, 948, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1218967, 38894, 144, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1219114, 38899, 87, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1219226, 38904, 219, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1219448, 38910, 40833, 1230, 284, 11236, 859, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1260296, 40143, 106, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1260405, 40146, 131, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1260539, 40151, 559, 11, 70, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1261101, 40164, 914, 26, 72, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1262018, 40192, 176, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1262197, 40198, 111, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1262333, 40203, 148, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1262484, 40208, 156, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1262677, 40216, 217, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1262897, 40222, 97435, 3728, 294, 35183, 2954, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1360347, 43953, 320, 9, 40, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1360692, 43966, 717, 14, 186, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1361412, 43982, 61651, 2086, 216, 25635, 1636, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1423078, 46071, 745, 18, 70, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1423826, 46091, 879, 18, 78, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1424708, 46111, 574, 12, 54, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1425285, 46125, 452, 9, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1425762, 46138, 217, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1425982, 46144, 43911, 1326, 294, 12086, 927, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1469908, 47473, 139, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1470072, 47480, 179, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1470254, 47486, 42235, 1249, 294, 11261, 861, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1512504, 48738, 603, 11, 64, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1513132, 48753, 493, 11, 125, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1513628, 48766, 14458, 381, 137, 3107, 235, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1528101, 49150, 671, 11, 78, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1528775, 49163, 125, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1528937, 49171, 104, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[1529044, 49174, 432, 9, 52, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1529479, 49185, 438, 11, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1529920, 49198, 681, 15, 112, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1530626, 49217, 178, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1530807, 49223, 66713, 1784, 294, 13599, 1123, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1597557, 51012, 126, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1597686, 51017, 112, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1597813, 51023, 154, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1597970, 51029, 295, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1598290, 51039, 112, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1598405, 51044, 107, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1598527, 51050, 224, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1598776, 51060, 172, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1598951, 51066, 569, 9, 47, "Message",
  CellTags->"LocalInteractions:Introduction"],
Cell[1599523, 51077, 746, 14, 101, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1600284, 51094, 631, 17, 57, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1600918, 51113, 199, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1601120, 51119, 209, 7, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1601354, 51130, 241, 5, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1601598, 51137, 214, 4, 41, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1601827, 51144, 304, 7, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1602156, 51155, 123, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1602282, 51160, 145, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1602464, 51168, 123, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1602590, 51173, 145, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1602750, 51179, 80, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1602855, 51184, 286, 6, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1603144, 51192, 27183, 717, 257, 6942, 462, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1630342, 51912, 195, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1630562, 51920, 350, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1630915, 51929, 27915, 728, 255, 6938, 463, "GraphicsData", "PostScript",
   "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1658845, 52660, 152, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1659022, 52668, 424, 8, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1659449, 52678, 28123, 733, 255, 6978, 466, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1687587, 53414, 692, 16, 68, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1688282, 53432, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1688440, 53440, 123, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[1688566, 53443, 603, 15, 56, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1689172, 53460, 529, 10, 50, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1689704, 53472, 1672, 57, 82, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1691379, 53531, 152, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1691534, 53537, 273, 9, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1691810, 53548, 146, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1691959, 53553, 110, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1692094, 53558, 165, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1692262, 53564, 153, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1692430, 53570, 423, 13, 40, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1692856, 53585, 744, 16, 128, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1693625, 53605, 179, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1693807, 53611, 66738, 1787, 294, 13623, 1126, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1760560, 55401, 100, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1760663, 55404, 296, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1760962, 55412, 118, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1761105, 55417, 592, 12, 170, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1761700, 55431, 58926, 1132, 211, 7096, 487, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1820641, 56566, 337, 6, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1820981, 56574, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1821151, 56583, 98, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1821274, 56588, 92, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[1821369, 56591, 330, 10, 40, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1821702, 56603, 338, 6, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1822043, 56611, 207, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1822253, 56619, 10883, 302, 94, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1833139, 56923, 261, 9, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1833403, 56934, 831, 28, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1834237, 56964, 433, 8, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1834673, 56974, 287, 9, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1834963, 56985, 205, 6, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1835193, 56995, 2992, 60, 442, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1838188, 57057, 429, 7, 33, "Message",
  CellTags->"LocalInteractions:Introduction"],
Cell[1838620, 57066, 429, 7, 33, "Message",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1839064, 57076, 303, 6, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1839370, 57084, 1514, 29, 170, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1840887, 57115, 214, 5, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1841126, 57124, 186, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1841315, 57130, 751, 22, 59, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1842081, 57155, 897, 19, 144, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1843003, 57178, 179, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1843185, 57184, 25988, 1111, 294, 12214, 935, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1869188, 58298, 267, 6, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1869458, 58306, 132, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1869593, 58311, 114, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1869732, 58316, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1869856, 58321, 108, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1869979, 58327, 171, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1870153, 58334, 257, 5, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1870413, 58341, 461, 14, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1870877, 58357, 439, 16, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1871341, 58377, 137, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1871481, 58382, 135, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[1871631, 58388, 141, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1871775, 58394, 710, 18, 52, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1872488, 58414, 576, 20, 41, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1873067, 58436, 322, 8, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[1873392, 58446, 445, 10, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1873840, 58458, 90, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[1873955, 58463, 268, 5, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[1874226, 58470, 126874, 1711, 107, 61641, 900, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2001115, 60184, 842, 27, 59, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2001982, 60215, 277, 5, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2002262, 60222, 126785, 1710, 107, 61641, 900, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2129062, 61935, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2129185, 61940, 255, 7, 24, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2129443, 61949, 1049, 22, 193, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2130495, 61973, 290, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2130810, 61983, 278, 5, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2131091, 61990, 44495, 658, 71, 39736, 594, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2175601, 62651, 557, 10, 67, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2176161, 62663, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2176284, 62668, 1039, 23, 193, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2177326, 62693, 273, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2177624, 62703, 227, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2177854, 62709, 66968, 971, 108, 61641, 900, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2244837, 63683, 243, 5, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2245083, 63690, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2245252, 63699, 103, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2245358, 63702, 219, 5, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2245602, 63711, 98, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2245703, 63714, 882, 22, 50, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2246588, 63738, 236, 4, 60, "DisplayFormula",
  CellTags->"LocalInteractions:Introduction"],
Cell[2246827, 63744, 156, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2246986, 63750, 624, 12, 186, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2247635, 63766, 190, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2247828, 63772, 493, 8, 50, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2248370, 63786, 98, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2248471, 63789, 276, 7, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2248750, 63798, 721, 14, 203, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2249496, 63816, 189, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2249688, 63822, 1230, 18, 110, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2250967, 63846, 95, 1, 34, "Subsubsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2251065, 63849, 483, 14, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2251551, 63865, 575, 15, 55, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2252129, 63882, 155, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2252287, 63888, 676, 15, 112, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2252966, 63905, 95, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2253086, 63910, 137, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2253226, 63915, 16754, 264, 294, 904, 63, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2269995, 64182, 80, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270078, 64185, 131, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270212, 64190, 181, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270396, 64197, 113, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270512, 64202, 150, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270665, 64208, 193, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2270883, 64217, 113, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2270999, 64222, 570, 9, 57, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2271584, 64234, 568, 14, 50, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2272155, 64250, 139, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2272297, 64255, 305, 7, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2272627, 64266, 113, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2272743, 64271, 129, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2272887, 64277, 100, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2273012, 64282, 189, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2273204, 64288, 16713, 263, 294, 887, 62, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2289932, 64554, 197, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2290154, 64563, 127, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2290284, 64568, 111, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2290410, 64574, 174, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2290609, 64582, 113, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2290725, 64587, 129, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2290869, 64593, 211, 5, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2291105, 64602, 602, 10, 138, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2291710, 64614, 432, 7, 33, "Message",
  CellTags->"LocalInteractions:Introduction"],
Cell[2292145, 64623, 147, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2292307, 64629, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2292430, 64634, 445, 9, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2292878, 64645, 113, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2292994, 64648, 509, 11, 55, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2293506, 64661, 401, 11, 38, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2293910, 64674, 336, 9, 28, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2294249, 64685, 473, 10, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2294747, 64699, 246, 7, 40, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2294996, 64708, 141, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2295152, 64714, 419, 12, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2295574, 64728, 686, 13, 182, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2296263, 64743, 144, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2296432, 64751, 311, 8, 40, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2296746, 64761, 115, 3, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2296876, 64767, 489, 13, 42, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2297390, 64784, 2053, 38, 330, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2299446, 64824, 432, 7, 33, "Message",
  CellTags->"LocalInteractions:Introduction"],
Cell[2299881, 64833, 446, 8, 33, "Message",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2300342, 64844, 193, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2300538, 64850, 2049, 38, 330, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2302590, 64890, 125, 3, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2302718, 64895, 676, 15, 112, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2303397, 64912, 134, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2303534, 64918, 336, 7, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2303873, 64927, 104, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2303980, 64930, 182, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2304165, 64936, 100, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2304290, 64941, 126, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2304419, 64946, 333, 7, 41, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2304767, 64956, 171, 4, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2304941, 64962, 246, 5, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2305190, 64969, 133, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2305348, 64977, 132, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2305483, 64982, 264, 5, 41, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2305762, 64990, 104, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2305891, 64995, 137, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2306031, 65000, 16742, 264, 294, 904, 63, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2322788, 65267, 100, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2322891, 65270, 144, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2323060, 65277, 137, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2323200, 65282, 16668, 261, 294, 862, 60, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2339883, 65546, 89, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2339997, 65551, 124, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2340124, 65556, 526, 10, 57, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2340665, 65569, 120, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2340788, 65574, 232, 5, 36, "Text",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2341069, 65585, 115, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2341209, 65590, 435, 11, 73, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2341647, 65603, 763, 22, 59, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2342425, 65628, 496, 9, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2342924, 65639, 259, 6, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2343186, 65647, 441, 9, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2343630, 65658, 360, 7, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2343993, 65667, 122, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2344140, 65672, 298, 7, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2344441, 65681, 144, 3, 21, "Print",
  CellTags->"LocalInteractions:Introduction"],
Cell[2344588, 65686, 136, 3, 21, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2344739, 65692, 199, 6, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2344963, 65702, 612, 12, 186, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2345578, 65716, 30537, 606, 197, 3619, 268, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2376130, 66325, 292, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2376447, 66335, 231, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2376681, 66342, 142260, 3187, 409, 20291, 1675, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2518956, 69532, 95, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2519076, 69537, 213, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2519292, 69543, 35744, 633, 241, 3225, 226, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2555051, 70179, 208, 5, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2555284, 70188, 129, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2555416, 70193, 181, 4, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2555612, 70200, 122, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2555737, 70205, 110, 1, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2555850, 70208, 517, 9, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2556370, 70219, 199, 5, 25, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2556572, 70226, 438, 9, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2557013, 70237, 360, 7, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2557398, 70248, 253, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2557654, 70255, 144, 3, 21, "Print",
  CellTags->"LocalInteractions:Introduction"],
Cell[2557801, 70260, 136, 3, 21, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2557974, 70268, 146, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2558123, 70273, 46683, 770, 294, 3251, 228, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2604821, 71046, 291, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2605137, 71056, 182, 4, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2605322, 71062, 106166, 2707, 294, 19800, 1635, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2711503, 73772, 122, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2711662, 73780, 80, 1, 61, "Section",
  CellTags->"LocalInteractions:Introduction"],
Cell[2711745, 73783, 128, 2, 26, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2711898, 73789, 321, 7, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2712222, 73798, 2990, 50, 177, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2715227, 73851, 124, 2, 26, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2715376, 73857, 838, 15, 282, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2716217, 73874, 3127, 86, 283, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2719381, 73965, 77, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2719461, 73968, 431, 8, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2719895, 73978, 291, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2720211, 73988, 305, 5, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2720519, 73995, 50737, 731, 72, 37302, 560, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2771271, 74729, 122, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2771430, 74737, 121, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[2771554, 74740, 430, 8, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2771987, 74750, 243, 5, 39, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2772233, 74757, 968, 18, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2773204, 74777, 262, 5, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2773469, 74784, 482, 11, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2773954, 74797, 194, 4, 22, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[2774151, 74803, 252, 6, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2774428, 74813, 449, 9, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2774880, 74824, 80692, 2024, 369, 19548, 1264, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2855587, 76851, 122, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2855746, 76859, 92, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2855863, 76864, 334, 8, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2856200, 76874, 48771, 1325, 294, 11186, 855, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2904986, 78202, 349, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2905360, 78213, 211, 5, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2905574, 78220, 54495, 1551, 294, 13018, 1033, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[2960084, 79774, 249, 6, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[2960358, 79784, 139, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[2960500, 79789, 49005, 1332, 294, 11226, 860, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3009520, 81124, 122, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3009679, 81132, 79, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[3009761, 81135, 366, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3010152, 81146, 231, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3010386, 81153, 61554, 1416, 294, 9523, 768, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3071955, 82572, 293, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3072273, 82582, 221, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3072497, 82588, 319709, 11157, 334, 106412, 8517, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3392221, 93748, 123, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3392381, 93756, 75, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[3392459, 93759, 493, 12, 56, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3392977, 93775, 463, 8, 138, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3393443, 93785, 312, 7, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3393792, 93797, 247, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3394042, 93804, 103878, 1390, 294, 16031, 300, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3497935, 95197, 416, 8, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3498376, 95209, 234, 4, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3498613, 95215, 345304, 5804, 446, 254498, 4676, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3843932, 101022, 126, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3844095, 101030, 82, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3844202, 101035, 204, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3844409, 101041, 150174, 2246, 294, 6083, 461, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[3994598, 103290, 358, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[3994981, 103301, 141, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[3995125, 103306, 208956, 3052, 362, 7100, 554, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4204096, 106361, 176, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4204297, 106369, 141, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4204441, 106374, 153143, 2339, 294, 6643, 525, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4357599, 108716, 124, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4357760, 108724, 103, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4357888, 108729, 166, 4, 42, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4358057, 108735, 22453, 816, 294, 8413, 637, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4380547, 109556, 319, 7, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4380869, 109565, 32595, 977, 294, 8884, 679, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4413479, 110545, 357, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4413861, 110556, 141, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4414005, 110561, 36483, 1143, 294, 10371, 815, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4450503, 111707, 296, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4450824, 111717, 127, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4450954, 111722, 35612, 1092, 294, 9788, 768, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4486581, 112817, 124, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4486754, 112826, 87, 1, 61, "Section",
  CellTags->"LocalInteractions:Introduction"],
Cell[4486844, 112829, 336, 9, 43, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4487205, 112842, 435, 11, 73, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4487643, 112855, 763, 22, 59, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4488443, 112882, 79, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[4488525, 112885, 375, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4488925, 112896, 235, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4489163, 112903, 56525, 1658, 238, 13822, 1125, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4545703, 114564, 286, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4546014, 114574, 219, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4546236, 114580, 302549, 12131, 393, 121213, 9887, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4848800, 126714, 128, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4848965, 126722, 75, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[4849043, 126725, 462, 9, 106, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4849530, 126738, 236, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4849769, 126744, 54645, 760, 294, 10541, 210, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[4904429, 127507, 471, 10, 122, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[4904925, 127521, 194, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[4905122, 127527, 414767, 7313, 552, 311976, 6036, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[5319904, 134843, 127, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5320080, 134852, 96, 1, 61, "Section",
  CellTags->"LocalInteractions:Introduction"],
Cell[5320179, 134855, 428, 13, 36, "Text",
  CellTags->"LocalInteractions:Introduction"],
Cell[5320610, 134870, 3951, 114, 153, "Text",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[5324586, 134988, 1927, 34, 373, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5326516, 135024, 2441, 70, 227, "Print",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5328994, 135099, 79, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[5329076, 135102, 365, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[5329466, 135113, 229, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5329698, 135120, 61295, 1413, 294, 9354, 766, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[5391008, 136536, 290, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[5391323, 136546, 213, 4, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5391539, 136552, 552962, 21587, 446, 217632, 17440, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"],
Cell[5944504, 158141, 206, 5, 25, "Output",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[5944725, 158149, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5944883, 158157, 75, 1, 36, "Subsection",
  CellTags->"LocalInteractions:Introduction"],
Cell[5944961, 158160, 361, 7, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[5945347, 158171, 229, 5, 58, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[5945579, 158178, 117415, 2527, 294, 14882, 1256, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[6063009, 160708, 290, 6, 90, "Input",
  CellTags->"LocalInteractions:Introduction"],

Cell[CellGroupData[{
Cell[6063324, 160718, 272, 6, 74, "Input",
  CellTags->"LocalInteractions:Introduction"],
Cell[6063599, 160726, 409788, 14144, 374, 128182, 10661, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]],
Cell[6473402, 174873, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:Introduction"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

