(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     91683,       1402]*)
(*NotebookOutlinePosition[     93150,       1446]*)
(*  CellTagsIndexPosition[     93063,       1440]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Lattice", "ObjectName",
  CellTags->"LocalInteractions:54"],

Cell[TextData[{
  "\[FilledSmallSquare] ",
  StyleBox["Lattice",
    FontFamily->"Courier"],
  "[",
  StyleBox["cols",
    FontSlant->"Italic"],
  StyleBox[", rows, offsets",
    FontSlant->"Italic"],
  "] creates a lattice graph with ",
  StyleBox["cols",
    FontSlant->"Italic"],
  " columns and ",
  StyleBox["rows",
    FontSlant->"Italic"],
  " rows, where each vertex is connected via an edge to the vertices at the \
specified offsets"
}], "Usage",
  CellTags->"LocalInteractions:54"],

Cell[TextData[{
  "\[FilledSmallSquare] By default, the lattice does not wrap at the edges. A \
lattice graph which wraps can be obtained by setting the option ",
  StyleBox["Wrap",
    FontFamily->"Courier"],
  " to ",
  StyleBox["True.",
    FontFamily->"Courier"]
}], "Notes",
  CellTags->"LocalInteractions:54"],

Cell[CellGroupData[{

Cell["Examples", "ExampleSection",
  CellTags->"LocalInteractions:54"],

Cell[BoxData[
    \(<< LocalInteractions`Graphs`\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"LocalInteractions:54"],

Cell["\<\
Here is a lattice graph where each vertex is connected to vertices \
at the {N,SW,SE} compass points.\
\>", "Text",
  CellTags->"LocalInteractions:54"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Lattice[5, 5, {{0, 1}, {\(-1\), \(-1\)}, {1, \(-1\)}}] // 
        ShowGraph;\)\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"LocalInteractions:54"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.909091 0.0454545 0.909091 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 0 0 r
.005 w
[ ] 0 setdash
.04545 .04545 m
.04545 .27273 L
s
.04545 .04545 m
.27273 .27273 L
s
.27273 .04545 m
.04545 .27273 L
s
.27273 .04545 m
.27273 .27273 L
s
.27273 .04545 m
.5 .27273 L
s
.5 .04545 m
.27273 .27273 L
s
.5 .04545 m
.5 .27273 L
s
.5 .04545 m
.72727 .27273 L
s
.72727 .04545 m
.5 .27273 L
s
.72727 .04545 m
.72727 .27273 L
s
.72727 .04545 m
.95455 .27273 L
s
.95455 .04545 m
.72727 .27273 L
s
.95455 .04545 m
.95455 .27273 L
s
.04545 .27273 m
.04545 .5 L
s
.04545 .27273 m
.27273 .5 L
s
.27273 .27273 m
.04545 .5 L
s
.27273 .27273 m
.27273 .5 L
s
.27273 .27273 m
.5 .5 L
s
.5 .27273 m
.27273 .5 L
s
.5 .27273 m
.5 .5 L
s
.5 .27273 m
.72727 .5 L
s
.72727 .27273 m
.5 .5 L
s
.72727 .27273 m
.72727 .5 L
s
.72727 .27273 m
.95455 .5 L
s
.95455 .27273 m
.72727 .5 L
s
.95455 .27273 m
.95455 .5 L
s
.04545 .5 m
.04545 .72727 L
s
.04545 .5 m
.27273 .72727 L
s
.27273 .5 m
.04545 .72727 L
s
.27273 .5 m
.27273 .72727 L
s
.27273 .5 m
.5 .72727 L
s
.5 .5 m
.27273 .72727 L
s
.5 .5 m
.5 .72727 L
s
.5 .5 m
.72727 .72727 L
s
.72727 .5 m
.5 .72727 L
s
.72727 .5 m
.72727 .72727 L
s
.72727 .5 m
.95455 .72727 L
s
.95455 .5 m
.72727 .72727 L
s
.95455 .5 m
.95455 .72727 L
s
.04545 .72727 m
.04545 .95455 L
s
.04545 .72727 m
.27273 .95455 L
s
.27273 .72727 m
.04545 .95455 L
s
.27273 .72727 m
.27273 .95455 L
s
.27273 .72727 m
.5 .95455 L
s
.5 .72727 m
.27273 .95455 L
s
.5 .72727 m
.5 .95455 L
s
.5 .72727 m
.72727 .95455 L
s
.72727 .72727 m
.5 .95455 L
s
.72727 .72727 m
.72727 .95455 L
s
.72727 .72727 m
.95455 .95455 L
s
.95455 .72727 m
.72727 .95455 L
s
.95455 .72727 m
.95455 .95455 L
s
.025 w
.04545 .04545 Mdot
.27273 .04545 Mdot
.5 .04545 Mdot
.72727 .04545 Mdot
.95455 .04545 Mdot
.04545 .27273 Mdot
.27273 .27273 Mdot
.5 .27273 Mdot
.72727 .27273 Mdot
.95455 .27273 Mdot
.04545 .5 Mdot
.27273 .5 Mdot
.5 .5 Mdot
.72727 .5 Mdot
.95455 .5 Mdot
.04545 .72727 Mdot
.27273 .72727 Mdot
.5 .72727 Mdot
.72727 .72727 Mdot
.95455 .72727 Mdot
.04545 .95455 Mdot
.27273 .95455 Mdot
.5 .95455 Mdot
.72727 .95455 Mdot
.95455 .95455 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[2]:=",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000182000`400?l00000o`00003ooooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQ
ooooo`002oooool3o`0003oooooo0ol0000nooooo`?o0000?_ooool3o`0003kooooo0ol0000=oooo
o`002_ooool5o`0003gooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`00
2Oooool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Ooo
ool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7
o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002_ooool6o`00
03cooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002oooool3o`0000;o
oooo00?o0000ooooooooool0>?ooool00ol0003oooooo`000002o`000003ooooool0003ooooo03Wo
oooo00?o0000ooooool000000_l000000ooooooo0000ooooo`0iooooo`03o`000?oooooo000000;o
000000?oooooo`000?ooool0>Oooool00ol0003oooooo`000002o`0000gooooo000=ooooo`05o`00
0?ooooooooooooooool00000>?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool0=_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooooooo
oooooooo000000kooooo000=ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?oo
ool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o
0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooo
ooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
8_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
8_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
8_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
8_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool0
0ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003o
ooooooooo`0?ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo
00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003ooooooooo
o`0Aooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000
ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Coooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000oooooooo
ool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?oo
ool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooo
oooo01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooo
oooo01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooo
oooo01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0
003oooooooooo`0Hooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wo
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01_ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o
0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0003ooooooooo
o`0Looooo`04o`000?ooooooooooo`0001kooooo00?o0000ooooooooool07?ooool01?l0003ooooo
ooooool0000Nooooo`03o`000?oooooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0
003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003o
ooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003o
ooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooo
ooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooo
ooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooo
ooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Looooo`04o`000?ooooooooooo`0001kooooo
00?o0000ooooooooool07?ooool01?l0003oooooooooool0000Nooooo`03o`000?oooooooooo01co
oooo00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0Looooo`04o`000?oooooooooo
o`0001kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0
003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:
ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooo
ooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03
o`000?oooooooooo01Gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
5?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003ooooooooo
o`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Poooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Poooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Poooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Poooo
o`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Ooo
ool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooo
oooo00Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0
003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Ko
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0b
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooo
ooooool03?ooool000gooooo00Go0000ooooooooooooooooo`00000hooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000000ooo
ool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooo
ooooo`0fooooo`05o`000?ooooooooooooooool000003_ooool000_ooooo0ol00002ooooo`03o`00
0?oooooooooo03Sooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`0iooooo`03o`00
0?oooooo000000;o000000?oooooo`000?ooool0>Oooool00ol0003oooooo`000002o`000003oooo
ool0003ooooo03Wooooo00?o0000ooooool000000_l0000=ooooo`002_ooool6o`0003cooooo1Ol0
000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002Oooool7o`0003_ooooo1ol0000j
ooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7o`0003_ooooo1ol0000joooo
o`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`Oo
0000>_ooool7o`0003[ooooo1ol0000;ooooo`002_ooool6o`0003cooooo1Ol0000looooo`Go0000
??ooool5o`0003cooooo1Ol0000<ooooo`002oooool3o`0000;ooooo00?o0000ooooooooool0>?oo
ool00ol0003oooooo`000002o`000003ooooool0003ooooo03Wooooo00?o0000ooooool000000_l0
00000ooooooo0000ooooo`0iooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0>Ooo
ool00ol0003oooooo`000002o`0000gooooo000=ooooo`05o`000?ooooooooooooooool00000>?oo
ool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooo
oooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooooooooooooooo000000kooooo000=oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_oo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0
003oooooooooo`07ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00So
oooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool09_ooool00ol0003ooooo
ooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o
0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000oooo
ooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
7?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
7?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
7?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
7?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool0
0ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003o
ooooooooo`0Booooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo
00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool04?ooool00ol0003ooooooooo
o`0Dooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000
ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Foooo
o`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Foooo
o`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Foooo
o`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Foooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Oooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000oooooooo
ool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo01[ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0
003oooooooooo`0Kooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01co
oooo00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0Looooo`04o`000?oooooooooo
o`0001kooooo00?o0000ooooooooool07?ooool01?l0003oooooooooool0000Nooooo`03o`000?oo
oooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0<ooooo`003Ooo
ool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007ooo
ool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007ooo
ool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Mooooo`;o00007oooool0
0ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool0
0ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0
003oooooooooo`0Looooo`04o`000?ooooooooooo`0001kooooo00?o0000ooooooooool07?ooool0
1?l0003oooooooooool0000Nooooo`03o`000?oooooooooo01cooooo00Co0000oooooooooooo0000
7_ooool00ol0003oooooooooo`0Looooo`04o`000?ooooooooooo`0001kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo01Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0
003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oo
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool05_ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@
ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03
o`000?oooooooooo01;ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
4Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oo
oooooooo013ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool0
0ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo
00kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003o
ooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Voooo
o`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooo
oooo00Gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool00_ooool00ol0003ooooo
ooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool03?ooool000gooooo00Go
0000ooooooooooooooooo`00000hooooo`05o`000?ooooooooooooooool000000oooool00ol0003o
ooooooooo`0fooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0foooo
o`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0fooooo`05o`000?oooooo
ooooooooool000003_ooool000_ooooo0ol00002ooooo`03o`000?oooooooooo03Sooooo00?o0000
ooooool000000_l000000ooooooo0000ooooo`0iooooo`03o`000?oooooo000000;o000000?ooooo
o`000?ooool0>Oooool00ol0003oooooo`000002o`000003ooooool0003ooooo03Wooooo00?o0000
ooooool000000_l0000=ooooo`002_ooool6o`0003cooooo1Ol0000looooo`Go0000??ooool5o`00
03cooooo1Ol0000<ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[o
oooo1ol0000;ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo
1ol0000;ooooo`002Oooool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0
000;ooooo`002_ooool6o`0003cooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<
ooooo`002oooool3o`0000;ooooo00?o0000ooooooooool0>?ooool00ol0003oooooo`000002o`00
0003ooooool0003ooooo03Wooooo00?o0000ooooool000000_l000000ooooooo0000ooooo`0ioooo
o`03o`000?oooooo000000;o000000?oooooo`000?ooool0>Oooool00ol0003oooooo`000002o`00
00gooooo000=ooooo`05o`000?ooooooooooooooool00000>?ooool01Ol0003ooooooooooooooooo
000000?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o
0000ooooooooool0=_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
=_ooool01Ol0003ooooooooooooooooo000000kooooo000=ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo
00?o0000ooooooooool0=?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0<_ooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool0<?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool0;?ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00Oooooo00?o0000ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?oo
ool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00Wooooo00?o0000ooooooooool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0
003oooooooooo`0:ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool08_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool08?ooool00ol0003ooooo
ooooo`0<ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o
0000ooooooooool07_ooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000oooo
ooooool06_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo013ooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
5_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
5_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
5_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool0
5_ooool00ol0003oooooooooo`0Aooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo01;ooooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool0
0ol0003oooooooooo`0Cooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
01Cooooo00?o0000ooooooooool04?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Eooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003o
ooooooooo`0Eooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Kooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo
00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool02_ooool00ol0003ooooooooo
o`0Gooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool02?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo01Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Ioooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`00
0?oooooooooo01_ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01cooooo00Co0000oooooooooooo0000
7_ooool00ol0003oooooooooo`0Looooo`04o`000?ooooooooooo`0001kooooo00?o0000oooooooo
ool07?ooool01?l0003oooooooooool0000Nooooo`03o`000?oooooooooo01cooooo00Co0000oooo
oooooooo00007_ooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Moooo
o`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Moooo
o`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0<oooo
o`003Oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o
00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o
00007oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Looooo`04o`00
0?ooooooooooo`0001kooooo00?o0000ooooooooool07?ooool01?l0003oooooooooool0000Noooo
o`03o`000?oooooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0L
ooooo`04o`000?ooooooooooo`0001kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool06oooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool06_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo01[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool06Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Wooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`00
0?oooooooooo01Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05ooo
ool00ol0003oooooooooo`0:ooooo`03o`000?oooooooooo01Oooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo01Kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo01Kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo01Kooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo01Kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0
003oooooooooo`0>ooooo`03o`000?oooooooooo01Gooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool05?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Co
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool04oooool00ol0003ooooo
ooooo`0Booooo`03o`000?oooooooooo01?ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool04_ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0F
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool04?ooool00ol0003oooooooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03
o`000?oooooooooo00oooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
3_ooool00ol0003oooooooooo`0Looooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oo
oooooooo00gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo
00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo
00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo
00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo
00_ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo00[ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool02Oooool00ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool02?ooool00ol0003ooooooooo
o`0Xooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool01oooool00ol0003oooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`00
0?oooooooooo00Cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`0booooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooo
oooo00;ooooo00?o0000ooooooooool03?ooool000gooooo00Go0000ooooooooooooooooo`00000h
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`0fooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000003_ooool000_o
oooo0ol00002ooooo`03o`000?oooooooooo03Sooooo00?o0000ooooool000000_l000000ooooooo
0000ooooo`0iooooo`03o`000?oooooo000000;o000000?oooooo`000?ooool0>Oooool00ol0003o
ooooo`000002o`000003ooooool0003ooooo03Wooooo00?o0000ooooool000000_l0000=ooooo`00
2_ooool6o`0003cooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002Ooo
ool7o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7
o`0003_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7o`00
03_ooooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002_ooool6o`0003co
oooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002oooool3o`0000;ooooo
00?o0000ooooooooool0>?ooool00ol0003oooooo`000002o`000003ooooool0003ooooo03Wooooo
00?o0000ooooool000000_l000000ooooooo0000ooooo`0iooooo`03o`000?oooooo000000;o0000
00?oooooo`000?ooool0>Oooool00ol0003oooooo`000002o`0000gooooo000=ooooo`05o`000?oo
ooooooooooooool00000>?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooooooo
ool0=_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0=_ooool01Ol0003ooooooooooooo
oooo000000kooooo000=ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool0=?ooool0
0ol0003oooooooooo`02ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool0<_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0<?ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool0;_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0;?ooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00Oooooo00?o0000
ooooooooool0:_ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0:?ooool00ol0003oooooooooo`08oooo
o`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00Wooooo00?o0000oooooooo
ool09_ooool00ol0003oooooooooo`09ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00[ooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0:ooooo`03o`00
0?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool08_oo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00cooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooo
oooo00cooooo000=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool07_ooool00ol0
003oooooooooo`0=ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00ko
oooo00?o0000ooooooooool07?ooool00ol0003oooooooooo`0>ooooo`03o`000?oooooooooo00co
oooo000=ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00oooooo00?o0000ooooooooool06_ooool00ol0003ooooo
ooooo`0?ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo013ooooo00?o
0000ooooooooool06?ooool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo00cooooo000=
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo017ooooo00?o0000ooooooooool05_ooool00ol0003oooooooooo`0A
ooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo01;ooooo00?o0000oooo
ooooool05?ooool00ol0003oooooooooo`0Booooo`03o`000?oooooooooo00cooooo000=ooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo01?ooooo00?o0000ooooooooool04_ooool00ol0003oooooooooo`0Cooooo`03
o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool0
4?ooool00ol0003oooooooooo`0Dooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo01Gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0Eooooo`03o`000?oo
oooooooo00cooooo000=ooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo01Kooooo00?o0000ooooooooool03?ooool0
0ol0003oooooooooo`0Fooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo
01Oooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Gooooo`03o`000?oooooooooo
00cooooo000=ooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo01Sooooo00?o0000ooooooooool02?ooool00ol0003o
ooooooooo`0Hooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo01Wooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0Iooooo`03o`000?oooooooooo00cooooo
000=ooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo01[ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`0Jooooo`03o`000?oooooooooo00cooooo000=ooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo01_ooooo00?o0000
ooooooooool00_ooool00ol0003oooooooooo`0Kooooo`03o`000?oooooooooo00cooooo000=oooo
o`03o`000?oooooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0L
ooooo`04o`000?ooooooooooo`0001kooooo00?o0000ooooooooool07?ooool01?l0003ooooooooo
ool0000Nooooo`03o`000?oooooooooo01cooooo00Co0000oooooooooooo00007_ooool00ol0003o
ooooooooo`0<ooooo`003Oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooo
ooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooo
ooooo`0Mooooo`;o00007oooool00ol0003oooooooooo`0<ooooo`003Oooool00ol0003ooooooooo
o`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooooooo
o`0Mooooo`;o00007oooool00ol0003oooooooooo`0Mooooo`;o00007oooool00ol0003ooooooooo
o`0<ooooo`003Oooool00ol0003oooooooooo`0Looooo`04o`000?ooooooooooo`0001kooooo00?o
0000ooooooooool07?ooool01?l0003oooooooooool0000Nooooo`03o`000?oooooooooo01cooooo
00Co0000oooooooooooo00007_ooool00ol0003oooooooooo`0Looooo`04o`000?ooooooooooo`00
01kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool06oooool00ol0003o
ooooooooo`02ooooo`03o`000?oooooooooo01_ooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool06_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo01[ooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool06Oooool00ol0003ooooooooo
o`06ooooo`03o`000?oooooooooo01Wooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool06?ooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo01Sooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo01Oooooo00?o0000ooooooooool05oooool00ol0003oooooooooo`0:oooo
o`03o`000?oooooooooo01Oooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool05_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo01Kooooo00?o0000oooooooo
ool03?ooool000gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool05Oooool00ol0003oooooooooo`0>ooooo`03o`00
0?oooooooooo01Gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool05?oo
ool00ol0003oooooooooo`0@ooooo`03o`000?oooooooooo01Cooooo00?o0000ooooooooool03?oo
ool000gooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool04oooool00ol0003oooooooooo`0Booooo`03o`000?oooooo
oooo01?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool04_ooool00ol0
003oooooooooo`0Dooooo`03o`000?oooooooooo01;ooooo00?o0000ooooooooool03?ooool000go
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool04Oooool00ol0003oooooooooo`0Fooooo`03o`000?oooooooooo017o
oooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool04?ooool00ol0003ooooo
ooooo`0Hooooo`03o`000?oooooooooo013ooooo00?o0000ooooooooool03?ooool000gooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool03oooool00ol0003oooooooooo`0Jooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool03?ooool000gooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0L
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0L
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0L
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03_ooool00ol0003oooooooooo`0L
ooooo`03o`000?oooooooooo00kooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool03Oooool00ol0003oooooooooo`0Nooooo`03o`000?oooooooooo00gooooo00?o0000oooo
ooooool03?ooool000gooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool00ol0003oooooooooo`0Pooooo`03
o`000?oooooooooo00cooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0Rooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
3?ooool000gooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool02_ooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo00[ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool02Oooool0
0ol0003oooooooooo`0Vooooo`03o`000?oooooooooo00Wooooo00?o0000ooooooooool03?ooool0
00gooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool02?ooool00ol0003oooooooooo`0Xooooo`03o`000?oooooooooo
00Sooooo00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool01oooool00ol0003o
ooooooooo`0Zooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool03?ooool000gooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool01_ooool00ol0003oooooooooo`0/ooooo`03o`000?oooooooooo00Kooooo
00?o0000ooooooooool03?ooool000gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`0^ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool03?ooool000gooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool01?ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool03?ooool000gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`0boooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool03?ooool000gooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool00_ooool00ol0003oooooooooo`0dooooo`03o`000?oooooooooo00;ooooo00?o0000oooooooo
ool03?ooool000gooooo00Go0000ooooooooooooooooo`00000hooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000000oooool0
0ol0003oooooooooo`0fooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooooooo
o`0fooooo`05o`000?ooooooooooooooool000003_ooool000gooooo00Co0000oooooooooooo0000
>_ooool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo03Sooooo00Co0000oooooooo
oooo00000_ooool00ol0003oooooooooo`0hooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool0>?ooool01?l0003oooooooooool0000>ooooo`002oooool3o`000003ooooool0003o
oooo03_ooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1?l0000=ooooo`002_ooool5o`00
03gooooo1Ol0000looooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002Oooool7o`0003_o
oooo1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7o`0003_ooooo
1ol0000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002Oooool7o`0003_ooooo1ol0
000jooooo`Oo0000>_ooool7o`0003[ooooo1ol0000;ooooo`002_ooool5o`0003gooooo1Ol0000l
ooooo`Go0000??ooool5o`0003cooooo1Ol0000<ooooo`002oooool3o`0003oooooo0ol0000noooo
o`?o0000?_ooool3o`0003kooooo0ol0000=ooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooooolQooooo`00oooo
oolQooooo`00oooooolQooooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {287, 0}} -> {-0.0500054, -0.0500054, \
0.00383279, 0.00383279}},
  CellTags->"LocalInteractions:54"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1152}, {0, 700}},
WindowSize->{520, 578},
WindowMargins->{{250, Automatic}, {32, Automatic}},
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "LocalInteractions:54"->{
    Cell[1776, 53, 65, 1, 38, "ObjectName",
      CellTags->"LocalInteractions:54"],
    Cell[1844, 56, 492, 18, 48, "Usage",
      CellTags->"LocalInteractions:54"],
    Cell[2339, 76, 315, 9, 37, "Notes",
      CellTags->"LocalInteractions:54"],
    Cell[2679, 89, 70, 1, 38, "ExampleSection",
      CellTags->"LocalInteractions:54"],
    Cell[2752, 92, 121, 3, 26, "Input",
      CellTags->"LocalInteractions:54"],
    Cell[2876, 97, 161, 4, 36, "Text",
      CellTags->"LocalInteractions:54"],
    Cell[3062, 105, 174, 4, 42, "Input",
      CellTags->"LocalInteractions:54"],
    Cell[3239, 111, 88404, 1286, 294, 2617, 221, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"LocalInteractions:54"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"LocalInteractions:54", 92320, 1418}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 65, 1, 38, "ObjectName",
  CellTags->"LocalInteractions:54"],
Cell[1844, 56, 492, 18, 48, "Usage",
  CellTags->"LocalInteractions:54"],
Cell[2339, 76, 315, 9, 37, "Notes",
  CellTags->"LocalInteractions:54"],

Cell[CellGroupData[{
Cell[2679, 89, 70, 1, 38, "ExampleSection",
  CellTags->"LocalInteractions:54"],
Cell[2752, 92, 121, 3, 26, "Input",
  CellTags->"LocalInteractions:54"],
Cell[2876, 97, 161, 4, 36, "Text",
  CellTags->"LocalInteractions:54"],

Cell[CellGroupData[{
Cell[3062, 105, 174, 4, 42, "Input",
  CellTags->"LocalInteractions:54"],
Cell[3239, 111, 88404, 1286, 294, 2617, 221, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->"LocalInteractions:54"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

